#include "PeriodicTimer.h"
#include "common.h"
#include "OutputStream.h"
#include "InputStream.h"
#include "new.h"
#include "exception.h"
#include "Semaphore.h"
#include "Thread.h"
#include "FileSystemInterface.h"
#include "PersistentObject.h"
#include "PersistentMemoryObject.h"
#include "FreeRunningTimer.h"
#include "NameServer.h"

#define NUM_THREADS 8

class CounterTimer : public PeriodicTimer{
  public:
    CounterTimer(){}
    ~CounterTimer(){}
    unsigned int start( unsigned int times );
};

unsigned int CounterTimer::start(unsigned int times)
{
  PeriodicTimer::start(1000000);
  for(times;times>0;times--){
      await();
      *StandardOutput << times-1 << " seconds\n";
  }
  stop();
  return 1000000;
}

class MyThread : public Thread{
Semaphore* _s;
public:
MyThread(Semaphore *s): _s(s) {}
void Run(){
	*StandardOutput << "Hey! It works!\n";	
	_s->V();
}
};

int main() {

	*StandardOutput << "\nHello World Timer\n\n";
	Semaphore* s=new Semaphore(0,1);
	
	MyThread m(s);
	
	m.Start();

	s->P(); 
	
	int status=0;
	
	int returnType=0;
	
	PersistentObject* file = StandardFileSystemInterface->open("countdown", status, returnType, 0,0);
	
	PersistentMemoryObject* file1 = file->source();
	
	char buffer[4]="   ";
	
	file1->read(1, 3, buffer, 3);
	
	delete(file);
	
	*StandardOutput << "Here we go, what we read is: \n"<<buffer<<'\n';
	
	ProxiableObject* p =StandardNameServer->lookup( "SystemTimer", "FreeRunningTimer" );
	
	if(p==0){ 
		*StandardOutput << "Can't retrieve timer\n";
		return 0;
	}
	else *StandardOutput << "All right! We have a timer!\n";
	
	FreeRunningTimer timer(p);
	
	*StandardOutput <<"The time now is:"<<timer.time()<<'\n';
	
	try{

		PeriodicTimer *timer = new CounterTimer;	//on the heap to show new/delete functionality

		while(1){
			*StandardOutput << "Enter  number of seconds, 0 to exit\n";
			int c=0;
			*StandardInput >> c;
			if(c==0) break;
			timer->start(c);
			*StandardOutput << "Hello world!\n\n";
		}

		//timer->~PeriodicTimer();

		delete timer;

	}catch(bad_alloc)
	{
		*StandardOutput << "Error: Failure to allocate memory\n";
	}catch(invalid_object)
	{
		*StandardOutput << "Error: Object does not exist\n";
	}
	catch(exception& e)
	{
		*StandardOutput << "Error: ";
		e.what();
	}
	
	*StandardOutput <<"The time now is:"<<timer.time()<<'\n';

	*StandardOutput << "Exiting program\n";
	
	return 0;
}
