#include "common.h"
#include "NameServer.h"
#include "OutputStream.h"
#include "InputStream.h"
#include "FreeRunningTimer.h"
#include "TimingInterface.h"
#include "Thread.h"

#define FASTTRIES 1000
#define SLOWTRIES 1000000

class dummy {
public:
	virtual void noop();
};

void
dummy::noop()
{
}

void
null()
{
}

class MyThread : public Thread{
public:
MyThread() {}
void Run(){
	*StandardOutput << "Hey! It works!\n";	
}
};


FreeRunningTimer* SystemTimer;

int
main( int argc, char ** argv )
{
	
	TimingInterface * intf = new TimingInterface();

	dummy * d = new dummy;

	SystemTimer = new FreeRunningTimer(StandardNameServer->lookup( "SystemTimer", "FreeRunningTimer" ));

	int ntries=SLOWTRIES;
	
	*StandardOutput<< "Insert number of iterations for all tests (except thread creation). Suggested value: "
	<<SLOWTRIES<<'\n';
	
	*StandardInput>>ntries;
	
	*StandardOutput<< "Begin tests, with "<<ntries<<" iterations\n" ;
	
	unsigned int start = SystemTimer->time();
	for( unsigned int i = 0; i < ntries; i++ ) {
	}
	unsigned int end = SystemTimer->time();
	double emptyloop = double(end - start)/double(ntries);
	*StandardOutput<<"empty loop ="<<(int)(emptyloop*1000)<<" ns\n";

	start = SystemTimer->time();
	for(unsigned int i = 0; i < ntries; i++ ) {
		d->noop();
	}
	end = SystemTimer->time();
	double localmethod = (double(end - start) - emptyloop)/double(ntries);
	*StandardOutput<<"local method = "<< (int)(localmethod*1000) <<" ns\n";

	start = SystemTimer->time();
	for(unsigned int i = 0; i < ntries; i++ ) {
		intf->nullCall();
	}
	end = SystemTimer->time();
	double nullcall = (double(end - start) - emptyloop)/double(ntries);
	*StandardOutput<<"null call = "<< (int)(nullcall*1000)<<" ns\n";
	
	start = SystemTimer->time();
	for(unsigned int i = 0; i < ntries; i++ ) {
		intf->intParam( 0 );
	}
	end = SystemTimer->time();
	double onearg = (double(end - start) - emptyloop)/double(ntries);
	*StandardOutput<<"first integer arg = "<< (int)(onearg*1000)<<" ns\n";
	
	start = SystemTimer->time();
	for(unsigned int i = 0; i < ntries; i++ ) {
		intf->twoIntParams( 0, 0 );
	}
	end = SystemTimer->time();
	double twoargs = (double(end - start) - emptyloop)/double(ntries);
	*StandardOutput<<"second integer arg = "<< (int)(twoargs*1000)<<" ns\n";
	
	int nthreads = 30;
	
	*StandardOutput<< "Insert number of iterations for thread creation test. Suggested value: "
	<<nthreads<<'\n';
	
	*StandardInput>>nthreads;
	
	*StandardOutput<< "Begin tests, with "<<nthreads<<" iterations\n" ;
	
	start = SystemTimer->time();
	for(unsigned int i = 0; i < nthreads; i++ ) {
		MyThread thread;
	}
	end = SystemTimer->time();
	double threadtime = (double(end - start) - emptyloop)/double(nthreads);
	*StandardOutput<<"Thread create = "<<  (int)(threadtime*1000)<<" ns\n";
	return 0 ;
}



