#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "NameServer.h"
#include "VMInterface.h"
#include "VMInterfaceStar.h"
#include "OutputStream.h"
#include "RecordStream.h"
#include "RecordStreamStar.h"
#include "FileSystemInterface.h"
#include "FileSystemInterfaceStar.h"
#include "Ctype.h"
#include "Convert.h"

class Class;
class MemoryObjectCache;

char * zf = "ZeroFile";
char * cf = "ZeroFileCopy";
char * cacheType = "SimpleVMCache";

int
main( int argc, char ** argv )
{
    int size = 16000;
    
    switch(argc)
    {
      case 1:
	break;
      case 2:
	zf = argv[1];
	StandardOutput->formattedWrite( "%s %s\n", argv[0], argv[1]);
	break;
      case 3:
	zf = argv[1];
	size = atoi(argv[2]);
	StandardOutput->formattedWrite( "%s %s %d\n", argv[0],
				       argv[1], size);
	break;
      case 4:
	zf = argv[1];
	size = atoi(argv[2]);
	cacheType = argv[3];
	StandardOutput->formattedWrite( "%s %s %d \n", argv[0],
				       zf, size, cacheType);
	break;
      default:
	StandardOutput->formattedWrite( "Usage: %s FileToMap FileSize\n", argv[0]);
	return(1);
	break;
    }
    
    StandardOutput->formattedWrite( "Getting VMInterface\n" );
    VMInterfaceStar VMI = StandardNameServer->lookup("vmInterface", "VMInterface");
    Assert( VMI != 0 );
    VMI->test("Hello this is VMI");
    
    StandardOutput->formattedWrite( "Getting fileSystemInterface\n" );
    FileSystemInterfaceStar FSI =
	StandardNameServer->lookup("fileSystemInterface",
				   "FileSystemInterface");
    Assert (FSI != 0);
    ClassStar WFSC = StandardNameServer->lookup( "RecordStream", "Class" );
//     ClassStar PAC = StandardNameServer->lookup( "PersistentArray", "Class" );
    Assert (WFSC != 0);
    unsigned err;
//     RecordStreamStar zfile = FSI->open(zf, WFSC, err, O_RDRW, 0, PAC);
    RecordStreamStar zfile = FSI->open(zf, WFSC, err, O_RDRW);
    if(err)  
    {
      StandardOutput->formattedWrite("*** error %d while opening %s \n",
				     err, zf);
      return 1;
    }
    
    RecordStreamStar cfile = FSI->open(cf, WFSC, err, O_RDRW);
    if(err)  
    {
      StandardOutput->formattedWrite("*** error %d while opening %s \n",
				     err, cf);
      return 1;
    }

    StandardOutput->formattedWrite( "Doing DVM Setup\n" );
    VMI->setupDVM(zfile,cfile);
    
    StandardOutput->formattedWrite( "Adding to Domain\n" );
    char * zaddress = VMI->addToDomain(zfile, ReadWrite);
    char * caddress = VMI->addToDomain(cfile, ReadWrite);
    
    StandardOutput->formattedWrite( "Changing source file\n" );
    for ( int i=0; i < size; i += 2*1024 )
    {
	zaddress[i] = 'z';
    }
    StandardOutput->formattedWrite( "Changing Copy file\n" );
    for ( i=1024; i < size; i += 2*1024 )
    {
	caddress[i] = 'c';
    }

    StandardOutput->formattedWrite( "Reading source file\n" );
    for ( i=0; i < size; i += 1024 )
    {
	StandardOutput->formattedWrite("i = %d,val = %d;",i,zaddress[i]);
    }
    StandardOutput->formattedWrite("\n");
    
    StandardOutput->formattedWrite( "Done\n" );
    return 0;
}
