#ifdef __GNUC__
#pragma implementation
#endif


#include "Class.h"
#include "ClassStar.h"
#include "String.h"
#include "NameServer.h"
#include "FreeRunningTimer.h"
#include "FreeRunningTimerStar.h"
#include "TimeoutTimer.h"
#include "OutputStream.h"
#include "Convert.h"
#include "Semaphore.h"
#include "Process.h"
#include "ProcessStar.h"
#include "ThisProcess.h"
#include "FileSystemInterface.h"
#include "RecordStream.h"
#include "RecordStreamStar.h"
#include "PersistentMemoryObject.h"

#include <stdio.h>

//Global variables used by the tests
int divisor;  // No of thousand measurements
int iterations = 1; // No of measurements
int go; // signal for ending iterations when multiple processes
int loopcost; // time cost for for loop 
int LOOPCOST; // time for iterations of loop
int timecost; // time cost to read timer
int bcost; // time cost for semaphore P,V that blocks
int BCOST; // total iteration time cost for P,V that blocks
FreeRunningTimerStar SystemTimer;
TimeoutTimer *timer;
Semaphore *sem1;
Semaphore *sem2;
Semaphore *semb1;
Semaphore *semb2;
int tstart, tfinish; // times
char perf[200]; // buffer for output to file
RecordStream *outf = 0; // output file

//#define SPARC
// #ifdef SPARC
FileSystemInterface *StandardFileSystemInterface;

void initializeGlobals ()
{
    StandardFileSystemInterface = StandardNameServer -> lookup
    	("fileSystemInterface", "FileSystemInterface");
    Assert (StandardFileSystemInterface != 0);
    RecordStreamClass = StandardNameServer -> lookup
    	("RecordStream", "Class");
    Assert (RecordStreamClass != 0);
    PersistentMemoryObjectClass = StandardNameServer -> lookup
    	("PersistentMemoryObject", "Class");
}
// #endif

void nnull ()
     {}

int mainnull (int, char **)
     {}

void line (char * a, int nano, int micro, int its, int total )
     {
     sprintf(perf, "                                                                                             ");
     sprintf(perf, "%10s %10d %10d %10d %10d\n",a, nano,
        micro, its, total);
     Assert (outf != 0);
     outf -> write(perf,55);
     *StandardOutput << perf << eor;
     }


int main2 (int, char **)
{
  while (go == 1)
  {
  sem2 -> P ();
  sem1 -> V ();
  };
  return 0;
}

int mainb2 (int, char **)
{
  while (go == 1)
  {
  semb2 -> P ();
  semb1 -> V ();
  };
#ifdef FIXTHIS
  thisProcess () -> die ();
#endif
  return 0;
}

int main3 (int, char **)
{
  while (go == 1)
  {
  thisProcess() -> relinquishProcessor();
  };
#ifdef FIXTHIS
  thisProcess () -> die ();
#endif
  return 0;
}

//-----------------------------------------------------------------------------
void looptime()
{
    tstart = SystemTimer -> time();
    for (int i = 1; i< iterations; i++)
    	{};
    tfinish = SystemTimer -> time();
    loopcost = (tfinish - tstart)/iterations;
    LOOPCOST = (tfinish - tstart);
    line("raw loop",LOOPCOST/divisor,loopcost,iterations,LOOPCOST);
}
//-----------------------------------------------------------------------------
void systemtimecost()
{
    tstart = SystemTimer -> time();
    unsigned int x; // dummy
    for (int i = 1; i< iterations; i++)
    	{x = SystemTimer -> time(); };
    tfinish = SystemTimer -> time();
    int TIMECOST = tfinish-tstart - LOOPCOST;
    int timecost = TIMECOST/iterations;
    int ADJLOOPCOST = LOOPCOST - timecost;
    int adjloopcost = (LOOPCOST - timecost)/iterations;
    line("loop cost",ADJLOOPCOST/divisor,adjloopcost,iterations,ADJLOOPCOST);
    line("timer",TIMECOST/divisor,timecost,iterations,TIMECOST);
}
//-----------------------------------------------------------------------------
void pluspluscost()
{
    tstart = SystemTimer -> time();
    int p = 0;
    for (int i = 1; i< iterations; i++)
    	{ p++; }
    tfinish = SystemTimer -> time();
    int PLUSCOST = tfinish - tstart - LOOPCOST;
    int pluscost = PLUSCOST/iterations;
    line("int++",PLUSCOST/divisor,pluscost,iterations,PLUSCOST);
}
//-----------------------------------------------------------------------------
void lookupcost()
{
    tstart = SystemTimer -> time();
    for (int i = 1; i< iterations; i++)
    {
       //  Lookup the class in the name server.
       static ClassStar cls = (ClassStar) StandardNameServer -> lookup ("Process",
       "Class");
    };
    tfinish = SystemTimer -> time();
    int LOOKCOST = tfinish - tstart - LOOPCOST;
    int lookcost = (LOOKCOST)/iterations;
    line("lookup",LOOKCOST/divisor,lookcost,iterations,LOOKCOST);
}
//-----------------------------------------------------------------------------
void funcost()   // Measures cost of function call with no parameters
{
    tstart = SystemTimer -> time();
    for (int i = 1; i< iterations; i++)
    	{nnull(); };
    tfinish = SystemTimer -> time();
    int NULLCOST = tfinish - tstart - LOOPCOST;
    int nullcost = (NULLCOST)/iterations;
    line("nul func",NULLCOST/divisor,nullcost,iterations,NULLCOST);
}
//-----------------------------------------------------------------------------
void semPVcost()
{
    tstart = SystemTimer -> time();
    Semaphore s(0);
    for (int i = 1; i< iterations; i++)
    	{ s.V(); }
    tfinish = SystemTimer -> time();
    int VCOST = tfinish - tstart - LOOPCOST;
    int vcost = (VCOST)/iterations;
    line("sem.V",VCOST/divisor,vcost,iterations,VCOST);
//-----------------------------------------------------------------------------
    tstart = SystemTimer -> time();
    for (i = 1; i< iterations; i++)
    	{ s.P(); }
    tfinish = SystemTimer -> time();
    int PCOST = tfinish - tstart - LOOPCOST;
    int pcost = (PCOST)/iterations;
    line("sem.P",PCOST/divisor,pcost,iterations,PCOST);
}

//-----------------------------------------------------------------------------
void semPVblockingcost() // PV pairs that must block.  Note overhead includes
                     // any other processes running that are around.
{
  sem1 = new Semaphore (0);
  sem2 = new Semaphore (0);
  int status;
  ProcessStar proc = new ApplicationProcess (main2);
  Semaphore * done1 = new BinarySemaphore (0);
  proc -> notifyUponCompletion (done1, &status);
  if (proc == 0) 
  {
    *StandardOutput << "*** error while creating a process\n" << eor;
    return;
  }; 
  go = 1;
  proc -> ready ();
    tstart = SystemTimer -> time();
    for (int i = 1; i< iterations; i++)
    	{
        sem2 -> V ();
        sem1 -> P ();
        }
    tfinish = SystemTimer -> time();
        go = 0;
        sem2 -> V ();
        sem2 -> V ();
    done1->P();
    BCOST = tfinish - tstart - LOOPCOST;
    BCOST = BCOST/2;
    bcost = (BCOST)/iterations;
    line("PVBlocking",BCOST/divisor,bcost,iterations,BCOST);
}
//-----------------------------------------------------------------------------
void createsemcost()
{
    Semaphore *semA [divisor];
    tstart = SystemTimer -> time();
    for (int i = 1; i< divisor; i++)
    	{
  	semA[i] = new Semaphore (0);
        }
    tfinish = SystemTimer -> time();
    int SNCOST = tfinish - tstart - LOOPCOST/iterations*divisor;
    int sncost = (SNCOST)/divisor;
    line("SemCreate",SNCOST*1000/divisor,sncost,divisor,SNCOST);
//-----------------------------------------------------------------------------
    tstart = SystemTimer -> time();
    for (i = 1; i< divisor; i++)
    	{
  	semA[i] = 0;
        }
    tfinish = SystemTimer -> time();
    int SDCOST = tfinish - tstart - LOOPCOST/iterations*divisor;
    int sdcost = (SDCOST)/divisor;
    line("SemDelete",SDCOST*1000/divisor,sdcost,divisor,SDCOST);
}

//-----------------------------------------------------------------------------
void createintcost()
{
    //int *A [iterations];
    int **A = new int * [iterations];
    tstart = SystemTimer -> time();
    for (int i = 1; i< iterations; i++)
    	{
  	A[i] = new int(0);
        }
    tfinish = SystemTimer -> time();
    int IICOST = tfinish - tstart - LOOPCOST;
    int iicost = (IICOST)/iterations;
    line("IntCreate",IICOST/divisor,iicost,iterations,IICOST);
//-----------------------------------------------------------------------------
    tstart = SystemTimer -> time();
    for (i = 1; i< iterations; i++)
    	{
  	delete(A[i]);
        }
    tfinish = SystemTimer -> time();
    int IDCOST = tfinish - tstart - LOOPCOST;
    int idcost = (IDCOST)/iterations;
    line("IntDelete",IDCOST/divisor,idcost,iterations,IDCOST);
    delete A;
}
//-----------------------------------------------------------------------------
void binarysemPVblockingcost()
{
  semb1 = new BinarySemaphore (0);
  semb2 = new BinarySemaphore (0);
  ProcessStar proc = new ApplicationProcess (mainb2);
  Semaphore * done1 = new BinarySemaphore (0);
  int status;
  proc -> notifyUponCompletion (done1, &status);
  if (proc == 0) 
  {
    *StandardOutput << "*** error while creating a process\n" << eor;
    return;
  } 
  go = 1;
  proc -> ready ();
    tstart = SystemTimer -> time();
    for (int i = 1; i< iterations; i++)
    	{
        semb2 -> V ();
        semb1 -> P ();
        }
    tfinish = SystemTimer -> time();
        go = 0;
        semb2 -> V ();
        semb2 -> V ();
    done1->P();
    int BbCOST = tfinish - tstart - LOOPCOST;
        BbCOST = BbCOST/2;
    int bbcost = (BbCOST)/iterations;
    line("PVbinBlkg",BbCOST/divisor,bbcost,iterations,BbCOST);
}
//-----------------------------------------------------------------------------
void contextswitchcost()
{
  ProcessStar proc = new ApplicationProcess (main3);
  SemaphoreStar done = new BinarySemaphore (0);
  int status;
  proc -> notifyUponCompletion (done, &status);
  if (proc == 0) 
  {
    *StandardOutput << "*** error while creating a process\n" << eor;
    return;
  }; 
    go = 1;
    Process * me = thisProcess();
    proc -> ready ();
    tstart = SystemTimer -> time();
    for (int i = 1; i< iterations; i++)
    	{
	me->relinquishProcessor();
        }
    tfinish = SystemTimer -> time();
    go = 0;
    me->relinquishProcessor();
    done -> P ();
    int CSCOST = tfinish - tstart - LOOPCOST;
    CSCOST = CSCOST/2;
    int cscost = (CSCOST)/iterations ;
    line("Cntx Sw",CSCOST/divisor,cscost,iterations,CSCOST);
    line("PV diff",CSCOST/divisor-BCOST/divisor,cscost-bcost,iterations,CSCOST-BCOST);
}
//-----------------------------------------------------------------------------
void processcreatecost()
{
    ProcessStar proc[50];
    SemaphoreStar complete[50];
    tstart = SystemTimer -> time();
    for (int i = 1; i< divisor; i++)
    	{
        proc[i] = new ApplicationProcess (mainnull);
        complete[i] = new BinarySemaphore (0);
	int status;
    	proc[i] -> notifyUponCompletion (complete[i], &status);
        if (proc[i] == 0) 
        {
          *StandardOutput << "*** error while creating a process\n" << eor;
          return ;
        }; 
        }
    tfinish = SystemTimer -> time();
    int PLCOST = tfinish - tstart - (LOOPCOST/iterations*divisor);
    int plcost = (PLCOST)/divisor;
    line("Prcss Cr",PLCOST/divisor*1000,plcost,divisor,PLCOST);
//-----------------------------------------------------------------------------
    tstart = SystemTimer -> time();
    for (i = 1; i< divisor; i++)
    	{
        proc[i] -> ready ();
        }
    tfinish = SystemTimer -> time();
    int PRCOST = tfinish - tstart - (LOOPCOST/iterations*divisor);
    int prcost = (PRCOST)/divisor;
    line("Ready",PRCOST/divisor*1000,prcost,divisor,PRCOST);
//---------------------------------------------------------------------------
    tstart = SystemTimer -> time();
    for (i = 1; i< divisor; i++)
    	{
    	complete[i] -> P ();
        }
    tfinish = SystemTimer -> time();
    int PFCOST = tfinish - tstart - (LOOPCOST/iterations*divisor);
    int pfcost = (PFCOST)/divisor;
    line("Complete",PFCOST/divisor*1000,pfcost,divisor,PFCOST);
//---------------------------------------------------------------------------
    tstart = SystemTimer -> time();
    for (i = 1; i< divisor; i++)
    	{
        proc[i] = 0;
        }
    tfinish = SystemTimer -> time();
    int PDCOST = tfinish - tstart - (LOOPCOST/iterations*divisor);
    int pdcost = (PDCOST)/divisor;
    line("Prcss Dl",PDCOST/divisor*1000,pdcost,divisor,PDCOST);
//---------------------------------------------------------------------------
}
int main (int argc, char **argv)
{
#ifdef SPARC
    initializeGlobals ();
#endif
    int error;
    //  Iterate for argv[1] .
    if (argc > 2)
    {
    	*StandardOutput << "Usage: " <<  argv[0] << " <times>\n" << eor;
	return 1;
    }
    timer = new TimeoutTimer;
    if (timer == 0) return 2;
    if (argc > 1) iterations = atoi (argv [1]);
    if (iterations <= 0) return 3;
    divisor = iterations;
    iterations = iterations * 1000;
    SystemTimer = (FreeRunningTimerStar)
      StandardNameServer -> lookup("SystemTimer","FreeRunningTimer");
    //  Open the file "Perf.log" for writing.
    outf = StandardFileSystemInterface ->
       open ("Perf.log", RecordStreamClass, error, O_WRONLY | O_CREAT,
          0666, PersistentMemoryObjectClass);
    if (error || outf == 0)
    {
        //  Can't create the file.
        *StandardOutput << "*** error " << error << " while openning outfile\n"
          << eor;
        return 0;
    }
    else
    {
      //  Seek to the end of the file: 2 means use end-of-file as the origin,
      //  0 is the offset from the origin.
      if (outf -> setRecordNumber (0, 2) < 0)
      {
          //  Can't seek.
          *StandardOutput << "*** error while setting offset\n" << eor;
          return 0;
      }
    }

    sprintf(perf, "                                                                                             ");
    sprintf(perf, "*****************************\n");
    outf -> write(perf,30);

    sprintf(perf, "Iteration for %15d\n", iterations);
    outf -> write(perf,30);

    sprintf(perf, "%10s %10s %10s %10s %10s\n","Operations", 
        "Nanosecs", "Microsecs", "Iterations", "Tot Mic/s");
    outf -> write(perf,55);
    *StandardOutput << "Iteration for " << iterations << " \n" << eor;
    *StandardOutput << perf << eor;
    sprintf(perf, "                                                                                             ");

    looptime();
    systemtimecost();
    pluspluscost();
    lookupcost();
    funcost();
    semPVcost();
    semPVblockingcost();
    createsemcost();
    createintcost();
    binarysemPVblockingcost();
    contextswitchcost();
    processcreatecost();
    return 0;
}
