#ifdef __GNUC__
#pragma implementation
#endif

/*
 *     program analysis evaluates execution rates of c/fortran do-loops.
 *     through-put is measured in units of millions of floating-point
 *     operations per second, called mflops.
 */
#define MAXERR  1.0e-6

#include "NameServer.h"
#include "OutputStream.h"
#include "FreeRunningTimer.h"


double  x[1002], y[1002], z[1002], u[501], px[16][101], cx[16][101];
double  u1[6][23][3], u2[6][23][3], u3[6][23][3];
double  b[65][9], bnk1[6], c[65][9], bnk2[6], p[5][513], bnk3[6];
double  h[65][9], bnk4[6], bnk5[6], ex[68];
double  rh[68], dex[68], vx[151], xx[151], grd[151];
int     e[193], f[193];
 
long    nrops[] = {  0,  5, 10,  2,  2,
                         2,  2, 16, 36,
                        17,  9,  1,  1,
                         7, 11 };
 
long    loops[] = {   0, 400, 200,1000, 510,
                        1000,1000, 120,  40,
                         100, 100,1000,1000,
                         128, 150 };
 
double  checks[] = {  0,  0.811986948148e+07,  0.356310000000e+03,
                          0.356310000000e+03, -0.402412007078e+05,
                          0.136579037764e+06,  0.419716278716e+06,
                          0.429449847526e+07,  0.314064400000e+06,
                          0.182709000000e+07, -0.140415250000e+09,
                          0.374895020500e+09,  0.000000000000e+00,
                          0.171449024000e+06, -0.510829560800e+07 };


FreeRunningTimer * timer;

#define printf StandardOutput->formattedWrite

long
clock()
{
	return( timer->time() );
}

double
fabs( double x)
{
	return( x < 0 ? -x : x );
}

int
abs( int x)
{
	return( x < 0 ? -x : x );
}
 
int
main( int, char ** )
{
        extern void     init();
        extern long     clock();

        register int    nt, lw, nl1, nl2;
        register int    i, i2, ip, ir, ix, j, j2, k, kx, ky, l, m;
	register int i1, j1;
        double  ts[21], rt[21], rpm[21], cksum[21];
        double  r, t, a11, a12, a13, sig, a21, a22, a23, a31, a32, a33;
        double  bm28, bm27, bm26, bm25, bm24, bm23, bm22, c0, flx, rx1;
        register double q, s, scale, uu, du1, du2, du3, ar, br, cr, xi, ri;
        long    mops[20];


	printf( "Running\n" );

	timer = (FreeRunningTimer *)(ProxiableObject *)
		StandardNameServer->lookup( "SystemTimer", "FreeRunningTimer" );
	Assert (timer != 0);

        for (i=1; i<=20; i++)
                cksum[i] = 0.0;
        r = 4.86;        t = 276.0;        a11 = 0.5;
        a12 = 0.33;      a13 = 0.25;       sig = 0.8;
        a21 = 0.20;      a22 = 0.167;      a23 = 0.141;
        a31 = 0.125;     a32 = 0.111;      a33 = 0.10;
        bm28 = 0.1;      bm27 = 0.2;       bm26 = 0.3;
        bm25 = 0.4;      bm24 = 0.5;       bm23 = 0.6;
        bm22 = 0.7;      c0 = 0.8;         flx = 4.689;
        rx1 = 64.0;
/*
 *     end of initialization -- begin timing
 */
 
/* loop 1      hydro excerpt */
 
        init();
        ts[1] = (double) clock();
        q = 0.0;
        for (k=1; k<=400; k++)
                x[k] = q+y[k]*(r*z[k+10]+t*z[k+11]);
        ts[1] = (double) clock() - ts[1];
        for (k=1; k<=400; k++)
                cksum[1] += (double)k * x[k];
 
/* loop 2      mlr, inner product */
 
        init();
        ts[2] = (double) clock();
        q = 0.0;
        for (k=1; k<=996; k+=5)
                q += z[k]*x[k]+z[k+1]*x[k+1]+z[k+2]*x[k+2]+
                        z[k+3]*x[k+3]+z[k+4]*x[k+4];
        ts[2] = (double) clock() - ts[2];
        cksum[2] = q;
 
/* loop 3      inner prod */
 
        init();
        ts[3] = (double) clock();
        q = 0.0;
        for (k=1; k<=1000; k++)
                q += z[k]*x[k];
        ts[3] = (double) clock() - ts[3];
        cksum[3] = q;
 
/* loop 4      banded linear equarions */
 
        init();
        ts[4] = (double) clock();
        for (l=7; l<=107; l+=50) {
                lw=l;
                for (j=30; j<=870; j+=5)
                        x[l-1] -= x[lw++]*y[j];
                x[l-1] = y[5]*x[l-1];
                }
        ts[4] = (double) clock() - ts[4];
        for (l=7; l<=107; l+=50)
                cksum[4] += (double) l * x[l-1];
 
/* loop 5      tri-diagonal elimination, below diagonal */
 
        init();
        ts[5] = (double) clock();
        for (i=2; i<=998; i+=3)  {
                x[i] = z[i]*(y[i]-x[i-1]);
                x[i+1] = z[i+1]*(y[i+1]-x[i]);
                x[i+2] = z[i+2]*(y[i+2]-x[i+1]);
                }
        ts[5] = (double) clock() - ts[5];
        for (i=2; i<=1000; i++)
                cksum[5] += (double) i * x[i];
 
/* loop 6      tri-diagonal elimination, above diagonal */
 
        init();
        ts[6] = (double) clock();
        for (j=3; j<=999; j+=3) {
                i = 1003-j;
                x[i] = x[i]-z[i]*x[i+1];
                x[i-1] = x[i-1]-z[i-1]*x[i];
                x[i-2] = x[i-2]-z[i-2]*x[i-1];
                }
        ts[6] = (double) clock() - ts[6];
        for (j=1; j<=999; j++)  {
                l = 1001-j;
                cksum[6] += (double) j * x[l];
                }
 
/* loop 7      equation of state excerpt */
 
        init();
        ts[7] = (double) clock();
        for (m=1; m<=120; m++)
                x[m] = u[m]+r*(z[m]+r*y[m])+
                       t*(u[m+3]+r*(u[m+2]+r*u[m+1])+
                       t*(u[m+6]+r*(u[m+5]+r*u[m+4])));
        ts[7] = (double) clock() - ts[7];
        for (m=1; m<=120; m++)
                cksum[7] += (double) m * x[m];
 
/* loop 8      p.d.e. integration */
 
        init();
        ts[8] = (double) clock();
        nl1 = 1;        nl2 = 2;
        for (kx=2; kx<=3; kx++)  {
                for (ky=2; ky<=21; ky++) {
                        du1 = u1[kx][ky+1][nl1]-u1[kx][ky-1][nl1];
                        du2 = u2[kx][ky+1][nl1]-u2[kx][ky-1][nl1];
                        du3 = u3[kx][ky+1][nl1]-u3[kx][ky-1][nl1];
                        u1[kx][ky][nl2] = u1[kx][ky][nl1]+a11*du1+
                                a12*du2+a13*du3+sig*(u1[kx+1][ky][nl1]
                                -2.0*u1[kx][ky][nl1]+u1[kx-1][ky][nl1]);
                        u2[kx][ky][nl2] = u2[kx][ky][nl1]+a21*du1+
                                a22*du2+a23*du3+sig*(u2[kx+1][ky][nl1]
                                -2.0*u2[kx][ky][nl1]+u2[kx-1][ky][nl1]);
                        u3[kx][ky][nl2] = u3[kx][ky][nl1]+a31*du1+
                                a32*du2+a33*du3+sig*(u3[kx+1][ky][nl1]
                                -2.0*u3[kx][ky][nl1]+u3[kx-1][ky][nl1]);
                        }
                }
        ts[8] = (double) clock() - ts[8];
        for (i=1; i<=2; i++)
                for (kx=2; kx<=3; kx++)
                        for (ky=2; ky<=21; ky++)
                                cksum[8] += (double) kx * (double) ky *
                                        (double) i * (u1[kx][ky][i]+
                                        u2[kx][ky][i]+u3[kx][ky][i]);
 
/* loop 9      integrate predictors */
 
        init();
        ts[9] = (double) clock();
        for (i=1; i<=100; i++)
                px[1][i] = bm28*px[13][i] + bm27*px[12][i] +
                        bm26*px[11][i] + bm25*px[10][i] + bm24*px[9][i] +
                        bm23*px[8][i] + bm22*px[7][i] +
                        c0*(px[5][i] + px[6][i]) + px[3][i];
        ts[9] = (double) clock() - ts[9];
        for (i=1; i<=100; i++)
                cksum[9] += (double) i * px[1][i];
 
/* loop 10     difference predictors */
 
        init();
        ts[10] = (double) clock();
        for (i=1; i<=100; i++)   {
                ar = cx[5][i];
                br = ar-px[5][i];
                px[5][i] = ar;
                cr = br-px[6][i];
                px[6][i] = br;
                ar = cr-px[7][i];
                px[7][i] = cr;
                br = ar-px[8][i];
                px[8][i] = ar;
                cr = br-px[9][i];
                px[9][i] = br;
                ar = cr-px[10][i];
                px[10][i] = cr;
                br = ar-px[11][i];
                px[11][i] = ar;
                cr = br-px[12][i];
                px[12][i] = br;
                px[14][i] = cr-px[13][i];
                px[13][i] = cr;
                }
        ts[10] = (double) clock() - ts[10];
        for (i=1; i<=100; i++)
                for (k=5; k<=14; k++)
                        cksum[10] += (double) k * (double) i * px[k][i];
 
/* loop 11     first sum. */
 
        init();
        ts[11] = (double) clock();
        x[1] = y[1];
        for (k=2; k<=1000; k++)
                x[k] = x[k-1]+y[k];
        ts[11] = (double) clock() - ts[11];
        for (k=1; k<=1000; k++)
                cksum[11] += (double) k * x[k];
 
/* loop 12     first diff. */
 
        init();
        ts[12] = (double) clock();
        for (k=1; k<=999; k++)
                x[k] = y[k+1]-y[k];
        ts[12] = (double) clock() - ts[12];
        for (k=1; k<=999; k++)
                cksum[12] += (double) k * x[k];
 
 
/* loop 13      2-d particle pusher */
 
        init();
        ts[13] = (double) clock();
        for (ip=1; ip<=128; ip++)        {
                i1 = p[1][ip];  j1 = p[2][ip];
                p[3][ip] += b[i1][j1];
                p[4][ip] += c[i1][j1];
                p[1][ip] += p[3][ip];
                p[2][ip] += p[4][ip];
                i2 = (int) p[1][ip];
                j2 = (int) p[2][ip];
                p[1][ip] += y[i2+32];
                p[2][ip] += z[j2+32];
                i2 += e[i2+32];         j2 += f[j2+32];
                h[i2][j2] += 1.0;
                }
        ts[13] = (double) clock() - ts[13];
        for (ip=1; ip<=128; ip++)
                cksum[13] += (double) ip * (p[3][ip]+p[4][ip]+p[1][ip]+
                        p[2][ip]);
        for (k=1; k<=64; k++)
                for (ix=1; ix<=8; ix++)
                        cksum[13] += (double) k * (double) ix * h[k][ix];
 
/* loop 14      1-d particle pusher */
 
        init();
        ts[14] = (double) clock();
        for (k=1; k<=150; k++)   {
                ix = (int) grd[k];
                xi = (double) ix;
                vx[k] += ex[ix]+(xx[k]-xi)*dex[ix];
                xx[k] += vx[k]+flx;
                ir = (int) xx[k];
                ri = (double) ir;
                rx1 = xx[k]-ri;
                ir = abs(ir % 64);
                xx[k] = ri+rx1;
                rh[ir] += 1.0-rx1;
                rh[ir+1] += rx1;
                }
        ts[14] = (double) clock() - ts[14];
        for (k=1; k<=150; k++)
                cksum[14] += (double) k * (vx[k]+xx[k]);
        for (k=1; k<=67; k++)
                cksum[14] += (double) k * rh[k];
 
/* time the clock call */
 
        ts[15] = (double) clock();
        ts[15] = (double) clock() - ts[15];
 
/* scale= set to convert time to micro-seconds */
 
        scale=1.0;
        rt[15] = ts[15]*scale;
        printf("clock overhead = %9.2f usec\n",rt[15]);
 
        nt = 14.0;
        t = s = uu = 0.0;
        for (k=1; k<=nt; k++)    {
                rt[k] = (ts[k]-ts[15])*scale;
                t += rt[k];
                mops[k] = nrops[k]*loops[k];
                s += (double) mops[k];
                rpm[k] = 0.0;
                if (rt[k] != 0.0)
                        rpm[k] = (double) mops[k]/rt[k];
                uu += rpm[k];
                }
        uu /= (double) nt;
        s /= t;
        printf("\nloop         checksum       flops    time    mflops \n");
        for (k=1; k<=nt; k++)   {
                printf("%4d", k );
// Francis TODO
//		printf("  %20.12e%7d%9.1f%9.3f\n",cksum[k],mops[k],
//                        rt[k],rpm[k]);
                if (fabs(cksum[k]-checks[k]) > fabs(checks[k]*MAXERR))
                        printf("      %20.12e *** expected checksum\n",
                                checks[k]);
                }
        printf("\n   average mflops=%9.3f\n",uu);
	
	return(0);
}
void
init()
{
        register int    j, k, l;

        for (k=1; k<=1000; k++)  {
                x[k] = 1.11;
                y[k] = 1.123;
                z[k] = 0.321;
                }

	for (k=1; k<=500; k++)
		u[k] = 0.00025;
 
        for (k=1; k<=15; k++)    {
                for (l=1; l<=100; l++)   {
                        px[k][l] = l;
                        cx[k][l] = l;
                        }
                }
 
        for (j=1; j<6; j++)     {
                for (k=1; k<23; k++)    {
                        for (l=1; l<3; l++)     {
                                u1[j][k][l]=k;
                                u2[j][k][l]=k+k;
                                u3[j][k][l]=k+k+k;
                                }
                        }
                }
 
        for (j=1; j<65; j++)    {
                for (k=1; k<9; k++)     {
                        b[j][k] = 1.00025;
                        c[j][k] = 1.00025;
                        h[j][k] = 1.00025;
                        }
                }
 
        for (j=1; j<6; j++)     {
                bnk1[j] = j*100;
                bnk2[j] = j*110;
                bnk3[j] = j*120;
                bnk4[j] = j*130;
                bnk5[j] = j*140;
                }
 
        for (j=1; j<5; j++)     {
                for (k=1; k<513; k++)   {
                        p[j][k] = 1.00025;
                        }
                }
 
        for (j=1; j<193; j++)
                e[j] = f[j] = 1;
 
        for (j=1; j<68; j++)    {
                ex[j] = rh[j] = dex[j] = (double) j;
                }
 
        for (j=1; j<151; j++)   {
                vx[j] = 0.001;
                xx[j] = 0.001;
                grd[j] = (double) (j/8+3);
                }
}

