#ifdef __GNUC__
#pragma implementation
#endif

#include "NameServer.h"
#include "OutputStream.h"
#include "FreeRunningTimer.h"
#include "FreeRunningTimerStar.h"
#include "TimingInterface.h"
#include "TimingInterfaceStar.h"
#include "Assert.h"
#include "String.h"

#define FASTTRIES 1000
#define SLOWTRIES 10000

class Class;

class dummy {
public:
	virtual void noop();
};

void
dummy::noop()
{
}


FreeRunningTimerStar SystemTimer;

int
main( int argc, char ** argv )
{
	int ntries;

	if ( argc != 0 ) {
		ntries = 0;
		if ( argc == 2) {
			if ( strcmp(argv[1], "slow" ) == 0 )
				ntries = SLOWTRIES;
			else if ( strcmp(argv[1], "fast" ) == 0 )
				ntries = FASTTRIES;
		}
		if ( ntries == 0 ) {
			StandardOutput->formattedWrite(
				    "Usage: timings {slow|fast}\n" );
			return( 1 );
		}
	}
	else {
		StandardOutput->formattedWrite(
		    "Not called from shell, choosing \"fast\"\n" );
		ntries = FASTTRIES;
	}

	TimingInterfaceStar intfStar = new TimingInterface();
	Assert( intfStar != 0 );
	TimingInterface * intf = intfStar;

	StandardOutput->formattedWrite( "Begin tests\n" );

	dummy * d = new dummy;

	SystemTimer = (FreeRunningTimer *)(ProxiableObject *)
		StandardNameServer->lookup( "SystemTimer", "FreeRunningTimer" );

	Assert (SystemTimer != 0);

	unsigned int start = SystemTimer->time();
    unsigned int i;
	for( i = 0; i < ntries; i++ ) {
	}
	unsigned int end = SystemTimer->time();
	int emptyloop = end - start;
	StandardOutput->formattedWrite( "empty loop = %d us\n",
		emptyloop / ntries );

	start = SystemTimer->time();
	for( i = 0; i < ntries; i++ ) {
		extern void null();
		null();
	}
	end = SystemTimer->time();
	StandardOutput->formattedWrite( "empty function = %d us\n",
		((end - start) - emptyloop) / ntries );

	start = SystemTimer->time();
	for( i = 0; i < ntries; i++ ) {
		d->noop();
	}
	end = SystemTimer->time();
	StandardOutput->formattedWrite( "local method = %d us\n",
		((end - start) - emptyloop) / ntries );

	start = SystemTimer->time();
	for( i = 0; i < ntries; i++ ) {
		intf->nullCall();
	}
	end = SystemTimer->time();
	StandardOutput->formattedWrite( "null call = %d us\n",
		((end - start) - emptyloop) / ntries );

	unsigned int nulltime = end - start;
	
	start = SystemTimer->time();
	for( i = 0; i < ntries; i++ ) {
		intf->intParam( 0 );
	}
	end = SystemTimer->time();
	StandardOutput->formattedWrite( "first integer arg = %d us\n",
		((end - start) - nulltime) / ntries );
	unsigned int inttime = end - start;
	
	start = SystemTimer->time();
	for( i = 0; i < ntries; i++ ) {
		intf->twoIntParams( 0, 0 );
	}
	end = SystemTimer->time();
	StandardOutput->formattedWrite( "second integer arg = %d us\n",
		((end - start) - inttime) / ntries );
	
	
	char c;
	start = SystemTimer->time();
	for( i = 0; i < ntries; i++ ) {
		intf->ptrParam( &c );
	}
	end = SystemTimer->time();
	StandardOutput->formattedWrite(
		"pointer arg minus integer = %d us\n",
		((end - start) - inttime) / ntries );
	unsigned int ptrtime = end - start;
	
	start = SystemTimer->time();
	for( i = 0; i < ntries; i++ ) {
		intf->proxyParam( intf );
	}
	end = SystemTimer->time();
	StandardOutput->formattedWrite(
	    "called-ring ProxiableObject * arg minus pointer = %d us\n",
		((end - start) - ptrtime) / ntries );


	intf->setOutCallInterface( 0 );
	intf->setOutProxyInterface( 0 );
	return( 0 );
}

void
null()
{
}

#ifdef NEVER
OutCallInterface::OutCallInterface()
{
    anotherStar = new OutCallInterface( 0 );
    another = anotherStar;
}

OutCallInterface::OutCallInterface( int )
{
}

void
OutCallInterface::nullCall()
{
}

void
OutCallInterface::intParam( int )
{
}

void
OutCallInterface::twoIntParams( int, int )
{
}

void
OutCallInterface::ptrParam( char * )
{
}

void
OutCallInterface::proxyParam( TimingInterface * )
{
}

TimingInterface *
OutCallInterface::returnOldProxyAsterisk()
{
	return( this );
}

TimingInterface *
OutCallInterface::returnNewProxyAsterisk()
{
	return( another );
}

void
OutCallInterface::setOutProxyInterface( TimingInterface * intf )
{
	yetanotherStar = intf;
	yetanother = intf;
}

SubOutCallInterface::SubOutCallInterface()
{
    anotherStar = new SubOutCallInterface( 0 );
    another = anotherStar;
}

SubOutCallInterface::SubOutCallInterface( int )
{
}

void
SubOutCallInterface::nullCall()
{
}

void
SubOutCallInterface::intParam( int )
{
}

void
SubOutCallInterface::twoIntParams( int, int )
{
}

void
SubOutCallInterface::ptrParam( char * )
{
}

void
SubOutCallInterface::proxyParam( TimingInterface * )
{
}

void
SubOutCallInterface::subProxyParam( SubTimingInterface * )
{
}

TimingInterface *
SubOutCallInterface::returnOldProxyAsterisk()
{
	return( this );
}

TimingInterface *
SubOutCallInterface::returnNewProxyAsterisk()
{
	return( another );
}

SubTimingInterface *
SubOutCallInterface::subReturnOldProxyAsterisk()
{
	return( this );
}

void
SubOutCallInterface::setOutProxyInterface( TimingInterface * intf )
{
	yetanotherStar = intf;
	yetanother = intf;
}

#include "ClassInstaller.h"
DEFINECLASS("OutCallInterface",OutCallInterface,TimingInterface,0,0);
DEFINECLASS("SubOutCallInterface",SubOutCallInterface,SubTimingInterface,0,0);

#endif /*NEVER*/
