#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
#include "OutputStream.h"
#include "OutputStreamStar.h"
#include "NameServer.h"
#include "Class.h"
#include "RecordStream.h"
#include "Raid.h"
#include "FileSystemInterface.h"
#include "PersistentObject.h"
#include "PersistentMemoryObject.h"
#include "PersistentMemoryObjectStar.h"
#include "Process.h"
#include "ThisProcess.h"
#include "SystemInterface.h"
#include "InputStream.h"
#include "MemoryObjectDictionary.h"
#include "FileSystem.h"
#include "Tools.h"
#include "Convert.h"
#include "String.h"

#define isSource(c) ( (((c)>=' ')&&((c)<='~')) || ((c)=='\n') || ((c)=='\t') )

extern int exec( char * );
extern int GetLine( RecordStreamStar & Script, char * line );

extern void StartUp( int argc, char * *argv );

extern FileSystemInterfaceStar FSI;

extern PersistentClassStar MemoryObjectDictionaryClass;
extern const char * StatusCodes[];

int		res;
char		HOME[128];
char		PS1[128];
int		VERBOSE;
int		done;
int	UMASK;
int	UID;
int	GID;

main( int argc, char * *argv )
{
	char	cmdline[1024];

	res = 0;
	GID = 0;
	UMASK = 022;
	UID = 0;
	VERBOSE = 0;
	done = 0;
	strcpy( HOME, "/" );
	strcpy( PS1, "FiSh> " );

	StartUp( argc, argv );


	if( FSI == 0 ) {
		stdout->formattedWrite( "FileSystemInterface == 0!\n" );
		return( 1 );
		}

	stdout->formattedWrite( "%s", PS1 );
	stdout->write( "", 0 );
	stdout->flush();
	while( !done && gets( cmdline ) ) {
		if( !strcmp( cmdline, "exit" ) ) break;
		else exec( cmdline );
		stdout->formattedWrite( "%s", PS1 );
		stdout->write( "", 0 );
		stdout->flush();
	}

	return( 0 );
}

extern int match( char * string, char * pattern );

int
expand( char * fullPattern, char * matched )
{
	char	keyBuffer[32767];
	char	path[1024];
	char *	pattern;
	int	count;
	int	matches = 0;

	for( int i = strlen( fullPattern )-1; i >= 0; i-- ) {
		if( fullPattern[i] == '/' ) break;
	}
	if( i < 0 ) {
		strcpy( path, "." );
		pattern = fullPattern;
	}
	else {
		for( int j = 0; j < i; j++ ) path[j] = fullPattern[j];
		pattern = &fullPattern[i+1];
	}

	int matchedSize = 0;
	int status = 0;

	MemoryObjectDictionaryStar od = FSI->open( path,
					MemoryObjectDictionaryClass, status );
	if( od != 0 ) {
	    status = od->keys( keyBuffer, 32767, count );
	    if( status >= 0 ) {
		char * keyStart = keyBuffer;
		char * keyEnd = keyBuffer;

		while( *keyStart ) {
			while( *keyEnd != '\n' ) keyEnd++;
			*keyEnd = '\0';

			if( match(keyStart, pattern) &&
			    ( (*pattern == '.') || (*keyStart != '.') ) ) {
				if( strcmp( path, "." ) != 0 ) {
					strcpy( &matched[matchedSize], path );
					matchedSize += strlen( path );
					matched[matchedSize++] = '/';
				}
				strcpy( &matched[matchedSize], keyStart );
				matchedSize += strlen( keyStart );
				matched[matchedSize++] = ' ';
				matches++;
			}

			keyEnd++;
			keyStart = keyEnd;
		}
	    }
	}
	if( matches > 0) matched[--matchedSize] = '\0';
	return( matches );
}

int
hasMetaChar( char * string )
{
	for( char * ptr = string; *ptr != '\0'; ptr++ ) {
		switch( *ptr ) {
			case '*':
			case '?':
			case '[': return( 1 );
		}
	} 
	return( 0 );
}

int
exec( char * command )
{
	char *	argv[128];
	char *	saveMatched = new char[32767];
	char *	matched = saveMatched;
	char *	cmdline = new char[strlen( command )+1];
	strcpy( cmdline, command );

	int len = strlen( cmdline );
	int argc = 0;
	for( int i = 0; i < len; i++ ) {
    	    if( cmdline[i] > ' ' ) {
	        if( cmdline[i] == '"' ) {
		    argv[argc++] = &cmdline[++i];
		    while( ( ( cmdline[i] >= ' ' ) ||
			     ( cmdline[i] >= '\t' ) ) &&
			   ( cmdline[i] != '"'     ) ) i++;
    		    if( cmdline[i] != '"' ) {
			delete saveMatched;
			delete cmdline;
    		        return( UnbalancedParentheses );
		    }
		    cmdline[i] = '\0';
	        }
	        else {
		    argv[argc++] = &cmdline[i];
		    while( cmdline[i] > ' ' ) i++;
		    cmdline[i] = '\0';
		    if( hasMetaChar( argv[argc-1] ) ) {
			if( expand( argv[argc-1], matched ) > 0 ) {
			    argc--;
			    int matchedLen = strlen( matched );
			    for( int j = 0; j < matchedLen; j++ ) {
				if( matched[j] > ' ' ) {
			    	    argv[argc++] = &matched[j];
				    while( matched[j] > ' ' ) j++;
				    matched[j] = '\0';
				}
			    }
			    matched = &matched[++j];
			}
		    }
		}
	    }
	}

#define strNE( a, b ) strcmp( (a), (b) )
#define strEQ( a, b ) !strcmp( (a), (b) )
	if( VERBOSE && ( argc > 0 )
		&& strNE( argv[0], "echo" )
		&& strNE( argv[0], "pause" )
		&& strNE( argv[0], "verbose" ) ) stdout->formattedWrite( "+ %s\n", command );

	if( argc == 0 ) res = 0;
	else if( strEQ( argv[0], "append" ) )	res = append( argc, argv );
	else if( strEQ( argv[0], "cat" ) )	res = cat( argc, argv );
        else if( strEQ( argv[0], "create" ) )   res = create( argc, argv );
	else if( strEQ( argv[0], "cd" ) )	res = cd( argc, argv );
	else if( strEQ( argv[0], "chroot" ) )	res = cdroot( argc, argv );
	else if( strEQ( argv[0], "chgrp" ) )	res = chgroup( argc, argv );
	else if( strEQ( argv[0], "chmod" ) )	res = chmode( argc, argv );
	else if( strEQ( argv[0], "chown" ) )	res = chowner( argc, argv );
	else if( strEQ( argv[0], "cp" ) )	res = cp( argc, argv );
        else if( strEQ( argv[0], "dump" ) )     res = dump( argc, argv );
	else if( strEQ( argv[0], "cdb" ) )	res = cdb( argc, argv );
	else if( strEQ( argv[0], "ancestors" ) ) res = hierarchy( argc, argv );
	else if( strEQ( argv[0], "descendants" ) )  res = hierarchy( argc, argv );
	else if( strEQ( argv[0], "hierarchy" ) )res = hierarchy( argc, argv );
	else if( strEQ( argv[0], "instance" ) )	res = instance( argc, argv );
	else if( strEQ( argv[0], "inst") ) 	res = inst( argc, argv );
	else if( strEQ( argv[0], "members" ) )	res = instances( argc, argv );
	else if( strEQ( argv[0], "kindred" ) )	res = instances( argc, argv );
	else if( strEQ( argv[0], "odb" ) )	res = odb( argc, argv );
	else if( strEQ( argv[0], "echo" ) )	res = echo( argc, argv );
	else if( strEQ( argv[0], "help" ) )	res = help( argc, argv );
	else if( strEQ( argv[0], "ln" ) )	res = ln( argc, argv );
	else if( strEQ( argv[0], "ls" ) )	res = ls( argc, argv );
	else if( strEQ( argv[0], "mkdir" ) )	res = makedir( argc, argv );
	else if( strEQ( argv[0], "mkpo" ) )	res = makepo( argc, argv );
	else if( strEQ( argv[0], "mount" ) )	res = mnt( argc, argv );
	else if( strEQ( argv[0], "mv" ) )	res = mv( argc, argv );
	else if( strEQ( argv[0], "rm" ) )	res = rm( argc, argv );
	else if( strEQ( argv[0], "pause" ) ) {
		if( argc > 1 ) {
			stdout->formattedWrite( "\nplease type %s to continue...", argv[1] );
		}
		else {
			stdout->formattedWrite( "\nplease press <CR> to continue..." );
		}
		res = 0;
		gets( cmdline );
		stdout->formattedWrite( "\n" );
	}
	else if( strEQ( argv[0], "rmdir" ) )	res = remdir( argc, argv );
	//else if( strEQ( argv[0], "server" ) )	res = server( argc, argv );
	else if( strEQ( argv[0], "set" ) )	res = set( argc, argv );
	else if( strEQ( argv[0], "sync" ) )	res = synch( argc, argv );
	else if( strEQ( argv[0], "umask" ) )	res = umasks( argc, argv );
	else if( strEQ( argv[0], "umount" ) )	res = unmnt( argc, argv );
	else if( strEQ( argv[0], "verbose" ) ) {
		res = 0;
		if( strcmp( argv[1], "on" ) == 0 ) VERBOSE = 1;
		else if( strcmp( argv[1], "off" ) == 0 ) VERBOSE = 0;
		else stdout->formattedWrite( "VERBOSE is %s\n", VERBOSE ? "on" : "off" );
	}
	else {
		int status = 0;
		int wait = 1;
		if (strcmp( argv[argc-1], "&" ) == 0 ){
			wait = 0;
			argc--;
			}
/*FIX*/
//             extern ClassStar POSHExpressionClass;
//             PersistentObjectStar po = FSI->open( argv[0],
//                                                  POSHExpressionClass, status );
//             if( po != 0 ) {
//                 po->execute();
//             }
//             else {
// 		if( Script == 0 ) {
// 			res = FSI->chdir( argv[0] );
// 			if( res != 0 ) {
// 				stdout->formattedWrite( "%s: command not found.\n", cmdline );
// 			}
// 		}
// 		else {
// 			char lines[1024];
// 			Script->read( lines, 2, status );
// 			if( isSource( lines[0] ) && isSource( lines[1] ) ) {
// 				Script->setRecordNumber( 0, 0 );
// 				while( GetLine( Script, lines ) ) {
// 					if( !strcmp( lines, "exit" ) ) break;
// 					exec( lines );
// 				}
// 			}
// 			else
 			{
			    	ClassStar PersistentMemoryObjectClass =
				    StandardNameServer -> lookup
				    	("PersistentMemoryObject", "Class");
				PersistentMemoryObjectStar pmo =
				     FSI->open(argv[0],PersistentMemoryObjectClass,
				    	status);
				if (pmo == 0) {
					stdout->formattedWrite( "%s: command not found.\n", cmdline );
					return 0;
					}
				Assert (pmo != 0);
				Assert (status == 0);
				ProcessStar proc = StandardSystemInterface ->
				    buildProcessFromObjectFile (pmo, argc,
				    	argv);
				Assert (proc != 0);
				NameServerStar procNameServer =
					proc -> domainNameServer();
				procNameServer -> bind 
				    ("StandardInput", StandardInput);
				procNameServer -> bind
				    ("StandardOutput", StandardOutput);
				procNameServer = 0;
				proc -> ready ();
				if (wait) thisProcess () -> waitForProcess (proc);
 			}
// 		}
// 	    }
	}
	if( res < 0 ) res = -res;
	if( (Success < res) && (res < Zz) ) stdout->formattedWrite( "%s\n", StatusCodes[res] );
	delete cmdline;
	delete saveMatched;
	return( res );
}

int
echo( int argc, char *argv[] )
{
	for( int i = 1; i < argc; i++ ) {
		if( !strcmp( argv[i], "$?" ) )		stdout->formattedWrite( "%d ", res );
		else if( !strcmp( argv[i], "$GID" ) )	stdout->formattedWrite( "%d ", GID );
		else if( !strcmp( argv[i], "$HOME" ) )	stdout->formattedWrite( "%s ", HOME );
		else if( !strcmp( argv[i], "$PS1" ) )	stdout->formattedWrite( "%s ", PS1 );
		else if( !strcmp( argv[i], "$UID" ) )	stdout->formattedWrite( "%d ", UID );
		else if( argv[i][0] == '$' )		stdout->formattedWrite( " " );
		else 					stdout->formattedWrite( "%s ", argv[i] );
	}
	stdout->formattedWrite( "\n" );
	return( 0 );
}

int
umasks( int argc, char *argv[] )
{
	if( argc < 2 ) stdout->formattedWrite( "%o\n", UMASK );
	else UMASK = atoi( argv[1], 8 );
	return( 0 );
}

int
set( int argc, char *argv[] )
{
	if( argc != 3 ) return( NumberOfArguments );
	else if( !strcmp( argv[1], "GID" ) )	GID = atoi( argv[2] );
	else if( !strcmp( argv[1], "HOME" ) )	strcpy( HOME, argv[2] );
	else if( !strcmp( argv[1], "PS1" ) )	strcpy( PS1, argv[2] );
	else if( !strcmp( argv[1], "UID" ) )	UID = atoi( argv[2] );
	return( 0 );
}

int
GetLine( RecordStreamStar & Script, char * line )
{
	char	buf[1];
	int	i = 0;
	int	eof = 1;
        int     recSize = Script->recordSize();
        if ( recSize == 1 ) {
                int status = 0;
		while( Script->read( buf, 1, status ) ) {
			eof = 0;
			if( buf[0] == '\n' ) break;
			line[i++] = buf[0];
		}
	}
	line[i] = '\0';
	return( !eof );
}

int
help( int argc, char *argv[] )
{
	if ( argc < 2 ) {
stdout->formattedWrite( "%s", "\n\
append filename [...] filename\n\
cat filename [...]\n\
cd [dirname]\n\
cdb\n\
chgrp gid filename [...]\n\
chmod omode filename [...]\n\
chown uid filename [...]\n\
chroot [dirname]\n\
cp infile outfile\n\
create classname [record-size] filename [...]\n\
echo [arguments]\n\
inst\n\
instance hex-address\n\
kindred clasname [...]\n\
ln [-s] infile outfile\n\
ls filename [...]\n\
members clasname [...]\n\
mkdir dirname [...]\n\
mkpo persistentObjectName [...]\n\
mount special pathname\n\
mv infile outfile\n\
odb\n\
rm filename [...]\n\
rmdir dirname [...]\n\
set arguments\n\
sync\n\
umask [newmask]\n\
umount pathname\n\
" );
	}
	else {
		stdout->formattedWrite( "%s\n", argv[1] );
	}
	return( 0 );
}
