#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Taed Nelson (nelson@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * iconvert.cc: Converter for use with instrumentation.
 *
 */


#include <std.h>
#include <stdio.h>

#include "iconvert.h"

// I know globals are icky, but they're soooo much easier...
	
Boolean writeOwnerField = FALSE;
Boolean writeInformationField = FALSE;

#ifdef DEBUG
Boolean normalDebug = FALSE;
Boolean superDebug = FALSE;
unsigned int inputFileIndex = 0;
FILE *debugFile = stdout;
#define fread(a, b, c, d) inputFileIndex += (c) * (b), fread (a, b, c, d)
#endif


// ----------------------------------------------------------------------------
void ErrorEOF ()
{
	fprintf (stderr, "Error:\tPremature end-of-file in input; remaining data ignored.\n");
}


// ----------------------------------------------------------------------------
CpuSyncClass :: CpuSyncClass ()
{
	cpuSyncList = NULL;
}


CpuSyncClass :: ~CpuSyncClass ()
{
	delete cpuSyncList;
}


Boolean CpuSyncClass :: sync (unsigned int cpuid)
{
	register CpuSyncNode *searchNode;

	for (searchNode = cpuSyncList;
	     (searchNode != NULL) && (searchNode->cpuid != cpuid);
	     searchNode = searchNode->next);
	     
	if (searchNode == NULL) {
		searchNode = new CpuSyncNode (cpuid);
		searchNode->next = cpuSyncList;
		cpuSyncList = searchNode;
	}
	else {
		searchNode->sync++;

		if (searchNode->sync > bufferOverlap) {
			return (TRUE);
		}
		else {
			return (FALSE);
		}
	}

	return (FALSE);
}


Boolean CpuSyncClass :: resync (unsigned int cpuid)
{
	register CpuSyncNode *searchNode;

	for (searchNode = cpuSyncList;
	     (searchNode != NULL) && (searchNode->cpuid != cpuid);
	     searchNode = searchNode->next);
	     
	if (searchNode == NULL) {
		fprintf (stderr, "Internal Error:  Resynced CPU missing.\n");
		exit (1);
	}
	else {
		searchNode->sync--;

		if (searchNode->sync < bufferOverlap) {
			searchNode->sync++;
			return (FALSE);
		}
		else {
			return (TRUE);
		}
	}
}
	

// ----------------------------------------------------------------------------
TitleNodeClass :: TitleNodeClass (timeType Start, unsigned int Ownerid,
				  unsigned int Selfid, char *Title)
{
	stopActivityKnown = FALSE;
	stopActivity = 0;
	startActivity = Start;
	ownerid = Ownerid;
	selfid = Selfid;
	title = Title;
	next = NULL;
}


// ----------------------------------------------------------------------------

TitleListClass :: TitleListClass ()
{
	head = NULL;
}


char * TitleListClass :: lookup (timeType lookupTime,
				 unsigned int ownerid, unsigned int selfid)
{
	TitleNodeClass *search, *bestFit;
	timeType bestFitError = (timeType) -1;
	timeType searchError;
						

	// Find the most appropriate title.
	// I realize that it's confusing, but I wanted the functionality
	
	for (search = head, bestFit = NULL;
	     search != NULL;
	     search = search->next) {
		     
		if ((search->selfid != selfid) || (search->ownerid != ownerid))
			continue;

		if ((lookupTime >= search->startActivity)) {
			    
			if (search->stopActivityKnown) {
				if (lookupTime <= search->stopActivity) {
					
					// We found a perfect fit!!!
					break;
				}
				
				else {
					// It doesn't fit at all
					//   but maybe the scope is a bit off
					
					if (bestFit == NULL) {
						bestFit = search;
					}

					else {
						searchError = lookupTime - search->stopActivity;
						if (searchError < bestFitError) {
							bestFitError = searchError;
							bestFit = search;
						}
					}
				}
			}
			
			else {
				// If the correct one isn't destructed yet
				bestFit = search;
			}
		}

		else {
			// It doesn't fit at all
			//   but maybe the scope is a bit off
			
			searchError = search->startActivity - lookupTime;
			if (searchError < bestFitError) {
				bestFitError = searchError;
				bestFit = search;
			}
		}
	}

	if (search == NULL) {
		if (bestFit != NULL) {
			return (bestFit->title);
		}
		else {
			return (NULL);
		}
	}

	return (search->title);
}


Boolean TitleListClass :: update (timeType updateTime, unsigned int ownerid,
				  unsigned int selfid, char *title)
{
	TitleNodeClass *search;

	for (search = head; search != NULL; search = search->next) {
		if ((search->selfid == selfid) &&
		    (search->ownerid == ownerid) &&
		    (updateTime >= search->startActivity) &&
		    ( ! search->stopActivityKnown))
			break;
	}

	// Deactivate the scope of the title
	// (assume that there is no extra information in the second title)
	if (search != NULL) {

		// I'm not sure what to do about manager events
		if (search->selfid != 0 ) {
			search->stopActivity = updateTime;
			search->stopActivityKnown = TRUE;
		}
		
		return (FALSE);
	}
	
	// Actually add the title
	TitleNodeClass *newNode =
		new TitleNodeClass (updateTime, ownerid, selfid, title);
	newNode->next = head;
	head = newNode;

	return (TRUE);
}


// ----------------------------------------------------------------------------
GeneralEventClass :: GeneralEventClass ()
{
}


void GeneralEventClass :: setup (unsigned int cpu, TitleListClass *list,
				 Boolean sync)
{
	cpuid = cpu;

	/* The title may not be in the list yet, but do an early
	 * lookup to make dubugging easier...
	 */
	title = list->lookup (timestamp, header.ownerid, header.selfid);

#ifdef DEBUG
if (normalDebug)
	if (title == NULL) {
		fprintf (debugFile, "Setup with NULL title lookup (%07lo) for (%2d,%2d) at time %d.%06ld\n",
			 fileIndex, header.ownerid, header.selfid,
			 timestamp / 1000000, timestamp % 1000000);
	}
#endif

	if (syncevent = sync) {
		messagelength = 0;
		delete message;
	}
}


Boolean GeneralEventClass :: read (FILE *fp)
{
#ifdef DEBUG
	fileIndex = inputFileIndex;
#endif

	if (fread ((char *)&timestamp, sizeof (timestamp), 1, fp) != 1) {
		ErrorEOF();
		return (FALSE);
	}

	if (fread ((char *)&header, sizeof (header), 1, fp) != 1) {
		ErrorEOF();
		return (FALSE);
	}

	if (header.count) {
		if (fread ((char *)&count, sizeof (count), 1, fp) != 1) {
			ErrorEOF();
			return (FALSE);
		}
	}

	if (header.time) {
		if (fread ((char *)&deltatime, sizeof (deltatime), 1, fp) != 1) {
			ErrorEOF();
			return (FALSE);
		}
	}

	if (header.message) {
		if (fread ((char *)&messagelength, sizeof (messagelength), 1, fp) != 1) {
			ErrorEOF();
			return (FALSE);
		}
		if ((message = new char[messagelength]) == NULL) {
			fprintf (stderr, "Memory Error:  No free store remaining for message of size %u.\n", messagelength);
			fprintf (stderr, "               Assuming read error; remaining file ignored.\n");
			return (FALSE);
		}
		if (fread (message, sizeof (char), messagelength, fp) != messagelength) {
			ErrorEOF();
			return (FALSE);
		}
	}

	return (TRUE);
}


void GeneralEventClass :: write (FILE *fp, TitleListClass *list)
{
#ifdef DEBUG
	if (normalDebug) fprintf (fp, "(%07lo) ", fileIndex);
#endif

	if (title == NULL) {
		title = list->lookup (timestamp,
				      header.ownerid, header.selfid);
	}

	if (header.percpu) {
		fprintf (fp, "%4d.%06ld :*%2d*",
			 timestamp / 1000000, timestamp % 1000000, cpuid);
	}
	else {
		fprintf (fp, "%4d.%06d : %2d ",
			 timestamp / 1000000, timestamp % 1000000, cpuid);
	}

	if (writeOwnerField) {
		fprintf (fp, ": %-25s ", list->lookup (timestamp,
						       header.ownerid, 0));
	}

	fprintf (fp, ": %-40s", title);

	if (header.count) {
		fprintf (fp, " : #%10lu", count);
	}

	if (header.time) {
		fprintf (fp, " : @%4d.%06ld",
			 deltatime / 1000000, deltatime % 1000000);
	}

	// Where should I put this in the format???
	if (writeInformationField && header.message && !syncevent) {
		fprintf (fp, " : \"");
		for (int i = 0; i < messagelength; i++)
			fputc (message[i], fp);
		fprintf (fp, "\"");
	}

	fprintf (fp, "\n");
}


// ----------------------------------------------------------------------------
EventListClass :: EventListClass (TitleListClass *_titleList)
{
	titleList = _titleList;

	head = NULL;
	current = NULL;

	cpuSync = new CpuSyncClass ();
}


void EventListClass :: add (GeneralEventClass * event)
{
	if (current == NULL) {
		event->prev = NULL;
		event->next = NULL;
		current = event;
		head = event;
		return;
	}

	while ((current != NULL) && (current->timestamp != event->timestamp)) {
		register unsigned int currentTime, eventTime, nextTime;

		/* Buffer sync events can go before the current pointer
		 * if it's the first one from that cpu.  This assumes that
		 * all of the first sync times are nearly the same.
		 */
		if (event->syncevent && current->syncevent &&
		    (current->prev == NULL)) {
			if (((event->timestamp > current->timestamp) &&
			     ((event->timestamp - current->timestamp) >
			      (((timeType) -1) - event->timestamp +
			       current->timestamp))) ||
			    ((event->timestamp < current->timestamp) &&
			     ((current->timestamp - event->timestamp) <
			      (((timeType) -1) - current->timestamp +
			       event->timestamp)))) {
				event->prev = NULL;
				event->next = current;
				current->prev = event;
				current = NULL;
				break;
			}
		}
		
		if (current->next == NULL) {
			event->next = NULL;
			event->prev = current;
			current->next = event;
			current = NULL;
			break;
		}

		currentTime = current->timestamp;
		eventTime = event->timestamp;
		nextTime = current->next->timestamp;

		// Checks the proper ordering of events (messy)
		if (((eventTime < currentTime) &&
		     (eventTime < nextTime) &&
		     (currentTime > nextTime)) ||
		    ((eventTime > currentTime) &&
		     (eventTime < nextTime) &&
		     (currentTime < nextTime)) ||
		    ((eventTime > currentTime)
		     && (eventTime > nextTime) &&
		     (currentTime > nextTime))) {
			break;
		}
		else {
			current = current->next;
		}
	}

	if (current != NULL) {
		event->next = current->next;
		event->prev = current;
		if (current->next != NULL) current->next->prev = event;
		current->next = event;
	}

	current = event;

	if (event->syncevent) {
		if (cpuSync->sync (event->cpuid)) {
			write ();
		}
	}
}


void EventListClass :: reset (unsigned int cpuid)
{
	// Go to the end of the list
	while ((current != NULL) && (current->next != NULL))
		current = current->next;

	// Go backwards until we find the most recent event from that cpu
	while ((current != NULL) && (current->prev != NULL) && (current->cpuid != cpuid))
		current = current->prev;
}


void EventListClass :: write ()
{
	GeneralEventClass *kill;

        while (head->syncevent ? cpuSync->resync (head->cpuid) : TRUE) {
		head->write (fp, titleList);
		kill = head;
		head = head->next;
		delete kill;
	}
}


void EventListClass :: forcedWrite ()
{
	GeneralEventClass *kill;
		
        while (head != NULL) {
		head->write (fp, titleList);
		kill = head;
		head = head->next;
		delete kill;
	}
}


// ----------------------------------------------------------------------------

PrintHelpMessage (FILE *fp)
{
	fprintf (fp, "\n");
	fprintf (fp, "iConvert is a utility for decoding the output files produced by an\n");
	fprintf (fp, "\tinstrumented Choices operating system.\n");
	fprintf (fp, "Usage:\n");
#ifdef DEBUG
	fprintf (fp, "\ticonvert [-hfoi] [-DS] {FileExtension}\n");
#else
	fprintf (fp, "\ticonvert [-hfoi] {FileExtension}\n");
#endif
	fprintf (fp, "Input:\n");
	fprintf (fp, "\tIf {FileExtension} is not specified, stdin is used.\n");
	fprintf (fp, "\tOtherwise, the file \"iDump.{FileExtension}\" is used.\n");
	fprintf (fp, "Output:\n");
	fprintf (fp, "\tIf the [-f] option is not specified, stdout is used.\n");
	fprintf (fp, "\tOtherwise, the file \"oDump.{FileExtension}\" is used.\n");
	fprintf (fp, "\tIn either case, the output is _very_ wide.\n");
	fprintf (fp, "Options:\n");
	fprintf (fp, "\t-h (help)        prints this message.\n");
	fprintf (fp, "\t-f (file)        produces an output file.\n");
	fprintf (fp, "\t-o (owner)       outputs instrument manager names.\n");
	fprintf (fp, "\t-i (info)        suppresses the information field.\n");
#ifdef DEBUG
	fprintf (fp, "\t-D (debug)       produces some debugging messages.\n");
	fprintf (fp, "\t-S (super-debug) produces all debugging information.\n");
#endif
	fprintf (fp, "\n");
}


main (int argc, char **argv)
{
	int i;
	TitleListClass titleList;
	EventListClass eventList (&titleList);
	unsigned int currentCPUid = 9999;
	Boolean outputToFile = FALSE;
	FILE *inputFile, *outputFile;

	// Default file names (yes, I know I _should_ have done it w/ pointers)
	char inputFileName[256] = "iDump.";
	char outputFileName[256] = "oDump.";
	char *extension = "";

	if (argc < 2) {
#ifdef DEBUG
		fprintf (stderr, "\ticonvert [-hfoi] [-DS] {FileExtension}\n");
#else
		fprintf (stderr, "\ticonvert [-hfoi] {FileExtension}\n");
#endif
		exit (1);
	}

	for (i = 1; i < argc; i++) {
		if (argv[i][0] == '-') {
			for (int j = 1; argv[i][j] != '\0'; j++) {
				switch (argv[i][j]) {
				case 'f':
					outputToFile = TRUE;
					break;
				case 'o':
					writeOwnerField = TRUE;
					break;
				case 'i':
					writeInformationField = TRUE;
					break;
				case 'h':
					PrintHelpMessage(stderr);
					exit (0);
#ifdef DEBUG
				case 'S':
					superDebug = TRUE;
				case 'D':
					normalDebug = TRUE;
					break;
#endif
				default:
					fprintf (stderr, "Error:  Unknown command line option \'%c\'.\n", argv[i][j]);
					break;
				}
			}
		}
		else {
			extension = argv[i];
		}
	}

	strcat (inputFileName, extension);
	
	if ((inputFile = fopen (inputFileName, "r")) == NULL) {
		fprintf (stderr, "Error:  Can't open input file %s.\n", inputFileName);
		exit (1);
	}

	if (outputToFile) {
		strcat (outputFileName, extension);

		if ((outputFile = fopen (outputFileName, "w")) == NULL) {
			fprintf (stderr, "Error:  Can't open output file %s.\n", outputFileName);
			exit (1);
		}
	}
	else {
		outputFile = stdout;
	}

	eventList.setOutputFile (outputFile);

	// Just so we have a title for those buffer headers
	// titleList.update (0, 0, 0, "SystemBufferSync");
	
	while (ungetc (getc (inputFile), inputFile) != EOF) {
		GeneralEventClass *event = new GeneralEventClass ();

		// If there was a read error
		if ( ! event->read (inputFile)) {
			delete event;
			break;
		}

		// instrument create/destroy
		if (event->header.special) {
			char *title = NULL;

			if (event->header.message) {
				title = new char[event->messagelength + 1];
				memcpy (title, event->message, event->messagelength);
				title[event->messagelength] = '\0';
			}

			Boolean newEvent = titleList.update (event->timestamp,
							     event->header.ownerid,
							     event->header.selfid,
							     title);

			if (newEvent) {
#ifdef DEBUG
if (normalDebug) {
	fprintf (debugFile,
		 "New instrument event %d, %d at time %d.%06ld (%07lo) Title \"%s\"\n",
		 event->header.ownerid,
		 event->header.selfid,
		 event->timestamp / 1000000,
		 event->timestamp % 1000000,
		 event->fileIndex,
		 title);
}
#endif DEBUG
			}

			// A destructor often has some good information
			else {
#ifdef DEBUG
if (normalDebug) {
	fprintf (debugFile,
		 "Destroyed instrument event %d, %d at time %d.%06ld (%07lo) Title \"%s\"\n",
		 event->header.ownerid,
		 event->header.selfid,
		 event->timestamp / 1000000,
		 event->timestamp % 1000000,
		 event->fileIndex,
		 titleList.lookup (event->timestamp,
				   event->header.ownerid, event->header.selfid));
}
#endif DEBUG
				// This may not be true for future instruments
				if (event->header.count && event->count > 0) {
					event->setup (currentCPUid, &titleList);
#ifdef DEBUG
if (superDebug) event->write (debugFile, &titleList);
#endif
					eventList.add (event);

					// We need a new store
					if ((event = new GeneralEventClass()) == NULL) {
						fprintf (stderr, "Memory Error:  No free store remaining for a new event store.\n");
						exit (1);
					}
				}
			}
		}

		// Is it a manager-specific event?
		else if (event->header.selfid == 0) {

			// It's a buffer sync
			if (event->header.ownerid == 0) {
				currentCPUid = *((unsigned int *) event->message);
#ifdef DEBUG
if (normalDebug)
	fprintf (debugFile,
		 "New buffer event:  (%07lo) CPU %lu\n",
		 event->fileIndex,
		 currentCPUid);
#endif DEBUG
				event->setup (currentCPUid, &titleList, TRUE);
				eventList.reset (currentCPUid);
				eventList.add (event);
				
				// We need a new store
				if ((event = new GeneralEventClass()) == NULL) {
					fprintf (stderr, "Memory Error:  No free store remaining for a new event store.\n");
					exit (1);
				}
			}

			// Some special thing
			else {
#ifdef DEBUG
				unsigned int id = *((unsigned int *) event->message);
if (normalDebug)
	fprintf (debugFile, "Unknown User Manager format:  (%07lo) ??? %lu\n",
			event->fileIndex, id);
#endif
				fprintf (stderr, "Unsupported format...  It's debugging time...\n");
				exit (1);
			}
		}

		// just a normal event
		else {
			event->setup (currentCPUid, &titleList);
#ifdef DEBUG
if (superDebug) event->write (debugFile, &titleList);
#endif
			eventList.add (event);

			// We need a new store
			if ((event = new GeneralEventClass()) == NULL) {
				fprintf (stderr, "Memory Error:  No free store remaining for a new event store.\n");
				exit (1);
			}
		}
	}

	eventList.forcedWrite ();
}
