#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * MemoryObject.c - memory object classes.
 */
#include "Assert.h"
#include "Domain.h"
#include "FileSystem.h"
#include "MemoryObject.h"
#include "Raid.h"
#include "ThisProcess.h"

MemoryObject::MemoryObject( unsigned int numberOfUnits, int log2UnitSize )
: MemoryRange( numberOfUnits, log2UnitSize )
{
	initDebug();
	Assert( this != 0 );
	Raid( RaidConstructor )( "%N::MemoryObject(units %d,log2UnitSize %d)\n",
				 this, numberOfUnits, log2UnitSize );
}

MemoryObject::~MemoryObject()
{
	Raid( RaidConstructor )( "%N::~MemoryObject()\n", this );

	Assert( _referenceCount.value() == 0 );

}

int
MemoryObject::transient()
{
	return( 1 );
}

void
MemoryObject::synchronize()
{
	Raid( RaidMethod )( "%N::synchronize()\n", this );
	Raid( RaidMethod )( "%N::synchronize: done\n", this );
}

void
MemoryObject::noRemainingReferences()
{
	if( ! transient() ) {
		synchronize();
	}
	delete this;
}

int
MemoryObject::read( unsigned int, int, char *, int )
{
	Assert( NOTREACHED );
	return( 0 );
}

int
MemoryObject::write( unsigned int, int, char *, int )
{
	Assert( NOTREACHED );
	return( 0 );
}

int
MemoryObject::copy( MemoryObject * dest )
{
	Raid( RaidMethod )( "%N::copy( %N )\n", this, dest );

	int returnValue = 0;
	dest->setNumberOfUnits( 0 );
	int log2UnitDifference = _log2UnitSize - dest->log2UnitSize();
	int readUnits = 1;
	int writeUnits = 1;
	int bufferSize = unitSize();

	if( log2UnitDifference > 0 ) {
		writeUnits <<=  log2UnitDifference;
	}
	else if( log2UnitDifference < 0 ) {
		log2UnitDifference = -log2UnitDifference;
		readUnits <<= log2UnitDifference;
		bufferSize <<= log2UnitDifference;
	}

	Raid( RaidDetail )( "%N::copy() ru %d wu %d bs %d\n",
			    this, readUnits, writeUnits, bufferSize );

	char * buffer = new char[bufferSize];

	int w = 0;
	for( int r = 0; r < _numberOfUnits; r += readUnits, w += writeUnits ) {
		int status = read( r, readUnits, buffer, bufferSize );
		if( ( status < readUnits ) && ( r+status < _numberOfUnits ) ) {
			returnValue = ReadError;
			break;
		}
		if( dest->write( w, writeUnits, buffer, bufferSize) !=
								writeUnits ) {
			returnValue = WriteError;
			break;
		}
	}
	delete buffer;

	Raid( RaidMethod )( "%N::copy( %N ) returns %d\n",
			    this, dest, returnValue );
	return( returnValue );
}

int
MemoryObject::setNumberOfUnits( unsigned int units )
{
	return( _numberOfUnits = units );
}
 
#include "ClassInstaller.h"
DEFINECLASS("MemoryObject",MemoryObject,MemoryRange,0,0);
