/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * MemoryObject.h: MemoryObject class descriptions.
 */

#ifndef MemoryObject_h
#define MemoryObject_h
#ifdef __GNUG__
#pragma once
#pragma interface
#endif

#include "MemoryRange.h"	/* parent class */
#include "MemoryObjectStar.h"

extern ClassStar MemoryObjectClass;
class MemoryObject : public MemoryRange {
protected:
	~MemoryObject();

	void noRemainingReferences();

public:
	Class * classOf();

	proxiable MemoryObject( unsigned int numberOfUnits, int log2UnitSize );

	proxiable virtual int read( unsigned int startUnit, int count,
					char * addr, int nbytes );
	proxiable virtual int write( unsigned int startUnit, int count,
					char * addr, int nbytes );
	proxiable virtual int copy( MemoryObject * dest );

	proxiable virtual int setNumberOfUnits( unsigned int numberOfUnits );


	proxiable virtual void synchronize();

	/*
	 * This method is used to determine whether to flush the data in
	 * the MemoryObjects cache back to the MemoryObject when it
	 * has no remaining references.
	 */
	proxiable virtual int transient();
};

#endif MemoryObject_h
