#ifdef __GNUC__
#pragma implementation
#endif

#ifdef __GNUG__    
#pragma implementation "OutputStream.h"
#endif
#include "NameServer.h"
#include "NameServerStar.h"
#include "MemoryObjectDictionary.h"
#include "InputStream.h"
#include "OutputStream.h"
#include "OutputStreamStar.h"
#include "String.h"
#include "SystemInterface.h"
#include "Domain.h"
#include "Byte.h"
#include "FileSystem.h"
#include "FileSystemInterface.h"
#include "FileSystemInterfaceStar.h"
#include "ReadFileStream.h"
#include "ReadFileStreamStar.h"
#include "WriteFileStream.h"
#include "WriteFileStreamStar.h"
#include "Class.h"
#include "ClassInstaller.h"
#include "Convert.h"
#include "UNIXFile.h"
#include "ThisProcess.h"
#include "Ctype.h"
#include "stdio.h"

InputStream * stdin = 0;
OutputStream * stdout = 0;

extern int VERBOSE;
extern int UMASK;
extern int UID;
extern int GID;
extern int done;

FileSystemInterfaceStar FSI = 0;
extern FileSystemInterfaceStar CreateFSI();
extern exec( char * );

/*
 * Must define a subclass of OutputStream that writes to UNIX stdout
 * since the OutputStream class is abstract.
 * This class contributed to by Stan.
 */
class UNIXOutputStream : public OutputStream {
public:
    void acquireExclusiveUse() {};
    void releaseExclusiveUse() {};

    int write( const char *, int );
};

int
UNIXOutputStream::write( const char * buffer, int count )
{
    return ::write( 1, buffer, count );
}

/*
 * The global standard output stream
 */
UNIXOutputStream * TheConsoleOutputStream = 0;

void
_Assert( const char * expr, const char * file, const char * basefile, int line )
{
	if( strcmp( file, basefile ) == 0 ) {
		printf( "Assertion ( %s ) failed in %s, line %d\n",
			expr, file, line );
	}
	else {
		printf( "Assertion ( %s ) failed in %s (#included in %s), line %d\n",
			expr, file, basefile, line );
	}
	exit( 1 );
}

Process * 
BuildProcessFromCOFFFile( PersistentMemoryObject *, int, char * [] )
{
	stdout->formattedWrite( "Running binaries only works under Choices\n" );
	return( 0 );
}

Process * 
BuildBSDProcessFromCOFFFile( PersistentMemoryObject *, int, char ** )
{
	stdout->formattedWrite( "Running binaries only works under Choices\n" );
	return( 0 );
}

Process * ThisProcess = (Process *)32;
Domain * ThisDomain = (Domain *)48;
SystemInterface * StandardSystemInterface = (SystemInterface *)64;
NameServerStar TheNameServer = 0;

Domain *
Process::domain()
{
	return( ThisDomain );
}

void
Process::ready()
{
}

void
Process::waitForProcess( Process * )
{
}

NameServer *
Process::nameServer()
{
	return( StandardNameServer );
}

unsigned char
Domain::ring()
{
	return( 0 );
}

int
cpoff( int argc, char *argv[] )
{
	int	fd;

	if( argc < 2 ) {
		stdout->formattedWrite( "usage: cpoff filename unixfile\n" );
		return( NumberOfArguments );
	}
	char * Path1 = argv[1];
	char * Path2 = argv[2];
	if( argc < 3 ) Path2 = Path1;
	int status = 0;
	ReadFileStreamStar IFile = FSI->open( Path1, ReadFileStreamClass,
					      status );
	if( status != 0 ) {
		stdout->formattedWrite( "cannot open %s\n", Path1 );
		return( status );
	}
	if( ( fd = creat( Path2, UNIXAccessMask ) ) < 0 ) {
		stdout->formattedWrite( "cannot create %s\n", Path2 );
		return( BadUnixFilename );
	}
	char * buf = new char [1024];
	int amt = IFile->read( buf, 1024 );
	while( amt > 0 ) {
		int oamt = write( fd, buf, amt );
		if( oamt != amt ) {
			close( fd );
			delete buf;
			return( WriteError );
		}
		amt = IFile->read( buf, 1024 );
	}
	close( fd );
	delete buf;
	return( 0 );
}

int
cpon( int argc, char *argv[] )
{
	int fd;

	if( argc < 2 ) {
		stdout->formattedWrite( "usage: cpon unixfile filename\n" );
		return( NumberOfArguments );
	}
	char * Path1 = argv[1];
	char * Path2 = argv[2];
	if( argc < 3 ) Path2 = Path1;
	if( ( fd = open( Path1, 0 ) ) < 0 ) {
		stdout->formattedWrite( "cannot open %s\n", Path1 );
		return( BadUnixFilename );
	}
	int status = 0;
	char * buf = new char [16384];
	WriteFileStreamStar OFile = FSI->openOrCreate( Path2,
					WriteFileStreamClass, 0666, status );

	if( ( OFile == 0 ) || ( status != 0 ) ) {
		stdout->formattedWrite( "cannot create %s\n", Path2 );
		close( fd );
		return( status );
	}

	int amt = read( fd, buf, 16384 );
	while( amt > 0 ) {
		int oamt = OFile->write( buf, amt );
		if( oamt != amt ) {
			close( fd );
			delete buf;
			return( WriteError );
		}
		amt = read( fd, buf, 16384 );
	}

	close( fd );
	delete buf;

	return( 0 );
}

int
things( int , char * [] )
{
	return( 0 );
}

void
StartUp( int argc, char * argv[] )
{
	ProxiableObject::initialize();
	PersistentObjectStar::initialize();
	ClassInstaller::install();
	TheConsoleOutputStream = new UNIXOutputStream;
	stdout = TheConsoleOutputStream;
	stdout->setName( "#StandardOutput" );
	TheNameServer = new NameServer( 0 );
	StandardNameServer = TheNameServer;
	ObjectClass->bind( StandardNameServer );

	for( int a = 1; a < argc; a++ ) {
		if( strcmp( argv[a], "FSI" ) == 0 ) break;
		exec( argv[a] );
	}

	FSI = CreateFSI();

	for( a++; a < argc; a++ ) {
		if( !strcmp( argv[a], "exit" ) ) {
			done = 1;
			break;
		}
		exec( argv[a] );
	}
}

void
ShutDown()
{
	FSI = 0;
	TheNameServer = 0;

	PersistentObjectStar::cleanup();
	ObjectClass->cleanup();

	ObjectClass->displayKindred( 1 );
}

Process *
SystemInterface::buildProcessFromObjectFile( PersistentMemoryObject * ,
					int , char ** )
{
    return( 0 );
}

Process *
thisProcess()
{
	return( ThisProcess );
}

void *
__user_new( int, void * where )
{
	return( where );
}

#ifndef CFRONT
extern "C" int
__main()
{
	/*
	 * call static constructors
	 */
	typedef void (*PFV)();
	extern PFV __CTOR_LIST__[];

	for( PFV * pf = &__CTOR_LIST__[1]; *pf != 0; pf++ ) {
		(**pf)();
	}

	return( 0 );
}
#endif

/* This function under Choices should turn on and off the MMU because
 * from and to refer to physical addresses but here we fake it!
 */
extern "C"{
void PhysCopyBlock( unsigned char * from, unsigned char * to, int count){

	while( count-- > 0 )
		*(to++) = *(from++);

};
}


void
OutputStream::acquireExclusiveUse()
{
}

void
OutputStream::releaseExclusiveUse()
{
}

void
OutputStream::emitChar( char c, int width, OutputStreamPadSide padSide )
{
	int padLength = ( width > 0 ) ? ( width - 1 ) : 0;

	if ( padSide == padLeft )	emitSpaces( padLength );
	write( &c, 1 );
	if ( padSide == padRight )	emitSpaces( padLength );
}

void
OutputStream::emitString( const char * string, int width,
	OutputStreamPadSide padSide )
{
	/*
	 * Determine pad length.
	 */
	int padLength = width;
	for ( const char * p = string; *p != 0; p++ ) {
		if ( padLength == 0 )
			break;
		padLength--;
	}

	if ( padSide == padLeft )	emitSpaces( padLength );
	write( string, strlen( string ) );
	if ( padSide == padRight )	emitSpaces( padLength );
}

void
OutputStream::emitSpaces( int count )
{
	if( count == 0 ) return;

	static char spaces[] = { ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ',
	    ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '};

	for( int i = count; i > ( sizeof( spaces ) );
	     				i -= ( sizeof( spaces ) ) ) {
		write( spaces, sizeof( spaces ) );
        }
	Assert( i > 0 );
	Assert( i <= sizeof( spaces ) - 1 );
	write( spaces, i );
}

void
OutputStream::emitZeros( int count )
{
	if( count == 0 ) return;

	static char zeros[] = { '0', '0', '0', '0', '0', '0', '0', '0',
	    '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0'};

	for( int i = count; i > ( sizeof( zeros ) );
					   i -= ( sizeof( zeros ) ) ) {
		write( zeros, sizeof( zeros ) );
	}
	Assert( i > 0 );
	Assert( i <= sizeof( zeros ) - 1 );
	write( zeros, i );
}


/*
 * emitNumber - converts and prints an int-sized argument in a given base
 *
 * NOTE: Currently only bases 2, 8, 10, and 16 work.  Calls write().
 *	Assumes *caller* has acquired the stream's exclusive use if
 *	necessary.
 */
void
OutputStream::emitNumber( unsigned int arg, int base, int width,
	OutputStreamPadSide padSide, OutputStreamPadChar padChar )
{
	/*
	 * Determine prefix.
	 * Update/maintain pad length.
	 * Negate value if signed decimal.
	 */
	int padLength = width;
	char * prefix = 0;
	int prefixLength = 0;
	switch ( base ) {
		case 2:		/* binary */
			prefix = "0b";
			prefixLength = 2;
			break;
		case 8:		/* octal */
			if ( arg != 0 ) {
				prefix = "0";
				prefixLength = 1;
			}
			break;
		case -10:	/* signed decimal */
			base = 10;
			int sarg = (int) arg;
			if ( sarg < 0 ) {
				prefix = "-";
				prefixLength = 1;
				sarg = -sarg;
				arg = (unsigned int) sarg;
			}
			break;
		case 10:	/* unsigned decimal */
			break;
		default:	/* unknown - treat as hexadecimal */
			base = 16;
			/* Falls through... */
		case 16:	/* hexadecmial */
			prefix = "0x";
			prefixLength = 2;
			break;
	}
	padLength -= prefixLength;

	/*
	 * Convert value to digits into buffer.
	 * Update/maintain pad length.
	 * Note that this is in reverse order (i.e., LSD first).
	 * We test for completion at the bottom of the
	 * loop in order to assure that we always get at least
	 * one digit in case the value is initially zero.
	 */
	unsigned int val = arg;
	static const int bufferSize = 32;
	char buffer[bufferSize];
	char * p = buffer;
	do {
		*p++ = "0123456789abcdef"[val % base];
		padLength--;
	} while ( ( val /= base ) != 0 );
	p--;

	/*
	 * Emit left space padding, if appropriate.
	 */
	if ( ( padLength > 0 )   &&
	     ( padSide == padLeft ) &&
	     ( padChar == aSpace ) ) {
		emitSpaces( padLength );
	}

	/*
	 * Emit the prefix.
	 */
	if( prefix != 0 ) {
		Assert( prefixLength != 0 );
		write( prefix, prefixLength );
	}

	/*
	 * Emit left zero padding, if appropriate.
	 */
	if ( ( padLength > 0 )   &&
	     ( padSide == padLeft ) &&
	     ( padChar == aZero ) ) {
		emitZeros( padLength );
	}

	/*
	 * Emit all the digits in the buffer.
	 * Copy them into an inverted buffer since this is probably
	 * faster than writing them one at a time.
	 */
	char invertBuffer[bufferSize];
	char * q = invertBuffer;
	int len = p - buffer + 1;
	while( p >= buffer ) {
		*q++ = *p--;
	}
	write( invertBuffer, len );

	/*
	 * Emit right padding, if appropriate.
	 * This will always be space padding,
	 * since zero padding here makes no sense.
	 */
	if ( ( padLength > 0 ) && ( padSide == padRight ) ) {
		emitSpaces( padLength );
	}
}

/*
 * Formatted write to an OutputStream (using old C "printf" style formats).
 */
void
OutputStream::formattedWrite( const char * format, const formatArgs & a1,
	const formatArgs & a2, const formatArgs & a3, const formatArgs & a4 )
{
	formatter( format,
		( &a1 != 0 ) ? a1.l : 0,
		( &a2 != 0 ) ? a2.l : 0,
		( &a3 != 0 ) ? a3.l : 0,
		( &a4 != 0 ) ? a4.l : 0 );
}

/*
 * OutputStream::formatter - primitive for formatted output to a stream.
 *
 * NOTE: Uses write() to do writes.
 * Assumes *caller* has acquired the streams exclusive use if necessary.
 */
void
OutputStream::formatter( const char * format, long a1, long a2,
	long a3, long a4 )
{
	long	args[4];
	args[0] = a1;
	args[1] = a2;
	args[2] = a3;
	args[3] = a4;

	long * argp = &args[0];		// argument pointer

	/* format and print arguments to console */
	enum OutputStreamPadChar padChar = aSpace;
	enum OutputStreamPadSide padSide = padLeft;
	int width = 0;

	for( ;; ) {
		char c;		// current format string character

		const char * start = format;
		c = *format++;
		while( ( c != '\n' ) && ( c != '%' ) && ( c != '\0' ) ) {
			c = *format++;
		}

		int count = format - start - 1;
		if( count != 0 ) {
			write( start, count );
		}

		switch( c ) {
		   case '\n':
			write( "\n", 1 );
			break;

		   case '\0':
			return;

	   	   case '%':
			/*
			 * Conversion.  Convert/print argument.
			 * Conversion depends upon next format character.
			 * In each case where there is a real conversion
			 * we must increment the argument pointer.
			 */
			padChar = aSpace;
			padSide = padLeft;
			width = 0;
again:
			switch (c = *format++) {

				case '\0':
					/*
					 * Unexpected end of format.
					 * Just return.
					 */
					return;

				case '-':
					/*
					 * Pad on right, instead of left.
					 */
					padSide = padRight;
					goto again;

				case '0':
					/*
					 * A leading zero means pad with zeros,
					 * instead of spaces.
					 */
					if ( width == 0 ) {
						padChar = aZero;
						goto again;
					}
					/* else, Falls through... */
				case '1':
				case '2':
				case '3':
				case '4':
				case '5':
				case '6':
				case '7':
				case '8':
				case '9':
					/*
					 * Width digit.
					 */
					width = ( 10 * width ) + ( c - '0' );
					goto again;

				case 'd':
					/*
					 * Convert to signed decimal.
					 */
					emitNumber( (unsigned int) *argp, -10,
						    width, padSide, padChar );
					argp++;
					break;

				case 'o':
					/*
					 * Convert to octal.
					 */
					emitNumber( (unsigned int) *argp, 8,
						    width, padSide, padChar );
					argp++;
					break;

				case 'x':
					/*
					 * Convert to hexadecimal.
					 */
					emitNumber( (unsigned int) *argp, 16,
						    width, padSide, padChar );
					argp++;
					break;

				case 'c':
					/*
					 * Character.
					 */
					emitChar( *((char *) argp),
							width, padSide );
					argp++;
					break;

				case 's':
					/*
					 * String.
					 */
					emitString( *((char **) argp),
						width, padSide );
					argp++;
					break;

				case 'u':
					/*
					 * Convert to unsigned decimal.
					 */
					emitNumber( (unsigned int) *argp, 10,
					    width, padSide, padChar );
					argp++;
					break;

				case '%':
					/*
					 * Emit a '%'.
					 */
					write( &c, 1 );
					break;

				/*
			 	 * NOTE: Below are non-standard extensions.
			 	 */

				case 'b':
					/*
					 * Convert to binary.
					 */
					emitNumber( (unsigned int) *argp, 2,
						    width, padSide, padChar );
					argp++;
					break;

				case 'A':
					/*
					 * An address.
					 * Print as "@%010x".
					 */
					write( "@", 1 );
					emitNumber( (unsigned int) *argp,
					   16, 10, padLeft, aZero );
					argp++;
					break;

				case 'N':
					/*
					 * Send the writeName() message
					 * to the argument.
					 */
					Object * o = (Object *) *argp;
					argp++;
					if( o == 0 ) {
						write( "NULL", 4 );
					}
					else if( !o->alive() ) {
						write( "DEAD[", 5 );
						emitNumber( (unsigned int) o,
						   16, 0, padLeft, aSpace );
						write( "]", 1 );
						//o->writeName( this );
					}
					else {
						o->writeName( this );
					}
					break;

				case 'I':
					/*
					 * Send the inspect() message
					 * to the argument.
					 */
					o = (Object *) *argp;
					argp++;
					if( o == 0 ) {
						write( "*(NULL)", 7 );
					}
					else if( !o->alive() ) {
						write( "*(DEAD)", 7 );
					}
					else {
						o->inspect( this );
					}
					break;

				case 'T':
					/*
					 * A 'this'.
					 * Print as "[@%010x]".
					 */
					write( "[@", 2 );
					emitNumber( (unsigned int) *argp, 16, 10,
						padLeft, aZero );
					write( "]", 1 );
					argp++;
					break;

				default:
					/*
					 * Unknown conversion.
					 * Print conversion character and
					 * ignore argument.
					 */
					write( "%", 1 );
					write( &c, 1 );
					argp++;
					break;
			}
			break;

		   default:
			Assert( 0 );
			break;
		}
	}
}

OutputStream &
OutputStream::flush()
{
	return( *this );
}

OutputStream &
OutputStream::operator << ( char c )
{
	write( &c, 1 );
	return( *this );
}

OutputStream &
OutputStream::operator << ( const char * s )
{
	write( s, strlen( s ) );
	return( *this );
}

OutputStream &
OutputStream::operator << ( int i )
{
	emitNumber( i, -10, 0, padLeft, aZero );
	return( *this );
}

OutputStream &
OutputStream::operator << ( unsigned int i )
{
	emitNumber( i, 10, 0, padLeft, aZero );
	return( *this );
}

OutputStream &
OutputStream::operator << ( Object * o )
{
	if( o == 0 ) *this << "<NULL>";
	else if( !o->alive() ) {
		*this << "DEAD[" << asHex( (unsigned int)o ) << "]";
	}
	else o->writeName( this );
	return( *this );
}

OutputStream &
OutputStream::operator << ( Object & o )
{
	if( &o == 0 ) *this << "<NULL>";
	else if( !o.alive() ) {
		*this << "DEAD[" << asHex( (unsigned int)&o ) << "]";
	}
	else o.writeName( this );
	return( *this );
}

OutputStream &
OutputStream::operator << ( ProxiableObjectRef & o )
{
    if( o == 0 ) *this << "<NULL>";
    else if( !o->alive() ) {
	*this << "DEAD[" << asHex( (unsigned int)(ProxiableObject *)o ) << "]";
    }
    else o->writeName( this );
    return( *this );
}

OutputStream &
OutputStream::operator << ( ProxiableObjectStar & o )
{
    if( o == 0 ) *this << "<NULL>";
    else if( !o->alive() ) {
	*this << "DEAD[" << asHex( (unsigned int)(ProxiableObject *)o ) << "]";
    }
    else o->writeName( this );
    return( *this );
}

/*
 * The following methods are temporary for compatibility with the old
 * OutputStream class.
 */

/*
 * formatted print.
 */
void
OutputStream::printf( const char * format, const formatArgs & a1,
	const formatArgs & a2, const formatArgs & a3, const formatArgs & a4 )
{
	formatter( format,
		( &a1 != 0) ? a1.l : 0,
		( &a2 != 0) ? a2.l : 0,
		( &a3 != 0) ? a3.l : 0,
		( &a4 != 0) ? a4.l : 0 );
}

/*
 * CPUPrintf - print a message prefaced by the processor id.
 */
void
OutputStream::cpuPrintf( const char * format, const formatArgs & a1,
	const formatArgs & a2, const formatArgs & a3, const formatArgs & a4 )
{
	formatter( format,
		( &a1 != 0) ? a1.l : 0,
		( &a2 != 0) ? a2.l : 0,
		( &a3 != 0) ? a3.l : 0,
		( &a4 != 0) ? a4.l : 0 );
}

/*
 * CriticalPrintf - desperate print to console.
 */
void
OutputStream::criticalPrintf( const char * format, const formatArgs & a1,
	const formatArgs & a2, const formatArgs & a3, const formatArgs & a4 )
{
	write( "\n", 1 );
	formatter( "CRITICAL: ", 0, 0, 0, 0 );
	formatter( format,
		( &a1 != 0) ? a1.l : 0,
		( &a2 != 0) ? a2.l : 0,
		( &a3 != 0) ? a3.l : 0,
		( &a4 != 0) ? a4.l : 0 );
}

/*
 * UitimatePrintf - print with extreme prejudice... only use by something
 *		    called by Panic.
 */
void
OutputStream::ultimatePrintf( const char * format, const formatArgs & a1,
	const formatArgs & a2, const formatArgs & a3, const formatArgs & a4 )
{
	write( "\n", 1 );
	formatter( "ULTIMATE: ", 0, 0, 0, 0 );
	formatter( format,
		( &a1 != 0) ? a1.l : 0,
		( &a2 != 0) ? a2.l : 0,
		( &a3 != 0) ? a3.l : 0,
		( &a4 != 0) ? a4.l : 0 );
}

void
OutputStream::panic( const char * format, const formatArgs & a1,
	const formatArgs & a2, const formatArgs & a3, const formatArgs & a4 )
{
	write( "\n", 1 );
	formatter( "PANIC: ", 0, 0, 0, 0 );
	formatter( format,
		( &a1 != 0) ? a1.l : 0,
		( &a2 != 0) ? a2.l : 0,
		( &a3 != 0) ? a3.l : 0,
		( &a4 != 0) ? a4.l : 0 );

	extern void Halt();

	Halt();
	/* Assert( NOTREACHED ); */ // this would result in a
				    // recursive call to panic.
}



/*
 * These functions needed for compatibility with Choices.
 * added by Stan.
 */

void
Halt()
{
    exit( -1 );
}

OutputStream &
Console()
{
    return *TheConsoleOutputStream;
}

#include "ClassInstaller.h"
DEFINECLASS("OutputStream",OutputStream,ProxiableObject,0,0);
DEFINECLASS("SystemInterface",SystemInterface,ProxiableObject,0,0);
