/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ReferenceCount.h - A reference count.
 */

#ifndef ReferenceCount_h
#define ReferenceCount_h

class REFERENCE_COUNT {
protected:
	short	_count;
	char	_dead;
	char	_lock;
	
public:
	REFERENCE_COUNT();
	void increment();
	void decrement();
	int value();
};

inline
REFERENCE_COUNT::REFERENCE_COUNT()
{
	_count = 0;
	_lock = 0;
	_dead = 0;
}

inline void
REFERENCE_COUNT::increment()
{
	Assert( _lock == 0 );
	_lock = 1;
	if( _dead ) Assert( _count == 0 );
	else {
		Assert( _count >= 0 );
		++_count;
		Assert( _count > 0 );
	}
	Assert( _lock == 1 );
	_lock = 0;
}

inline void
REFERENCE_COUNT::decrement()
{
	Assert( _lock == 0 );
	_lock = 1;
	if( _dead ) Assert( _count == 0 );
	else {
		Assert( _count > 0 );
		if( --_count == 0 ) _dead = 1;
		Assert( _count >= 0 );
	}
	Assert( _lock == 1 );
	_lock = 0;
}

inline int
REFERENCE_COUNT::value()
{
	Assert( _count >= 0 );
	return( _count );
}

#endif	/* ReferenceCount_h */
