#ifdef __GNUC__
#pragma implementation
#endif

#include "UNIXFile.h"
#include "String.h"
#include "FileSystemInterface.h"
#include "StoredObject.h"
#include "StoredObjectStar.h"
#include "PersistentMemoryObjectStar.h"
#include "MemoryObjectContainer.h"
#include "MemoryObjectContainerStar.h"
#include "MemoryObjectDictionary.h"
#include "MemoryObjectDictionaryStar.h"
#include "OutputStream.h"
#include "FileSystem.h"
#include "NFSDisk.h"
#include "NFSDictionary.h"
#include "Domain.h"
#include "stdio.h"
#include "ThisProcess.h"

struct sockaddr {
	short	sa_family;
	char	sa_data[14];
};

static const int AF_UNIX = 1;
static const int SOCK_STREAM = 1;

extern int errno;
extern FileSystemInterfaceStar FSI;

MemoryObjectContainerStar MOC = 0;

int *
NFSopen( char * buf )
{
	int * handle = (int *)buf;

	int * result = new int[5];
	result[2] = 0;
	int status = 0;

	PersistentMemoryObjectStar p = MOC->open( handle[2], status );

	if( p != 0 ) {
		MemoryObjectDictionaryStar d = p->asA(
					MemoryObjectDictionaryClass, status );
		if( d != 0 ) {
			p = d->open( &buf[12], status);
			if( p != 0 ) {
				MemoryObjectContainerStar c = p->container();
				Assert( c != 0 );
				Assert( MOC == (MemoryObjectContainer *)c );

				result[0] = p->numberOfUnits();
				result[1] = p->log2UnitSize();
				result[2] = 2;
				result[3] = c->idNumber();
				result[4] = p->idNumber();
			}
		}
	}

	return( result );
}

char *
NFSkeys( char * buf, int & size, int & count )
{
	int * handle = (int *)buf;

	char * result = 0;
	int status = 0;

	PersistentMemoryObjectStar p = MOC->open( handle[2], status );

	if( p != 0 ) {
		MemoryObjectDictionaryStar d = p->asA(
					MemoryObjectDictionaryClass, status );
		if( d != 0 ) {
			size = d->keySize();
			result = new char[size];
			d->keys( result, size, count );
		}
	}

	return( result );
}

int
NFSsetNumberOfBytes( char * buf )
{
	int status = 0;
	int * handle = (int *)buf;

	int result = 0;

	PersistentMemoryObjectStar p = MOC->open( handle[2], status );

	if( p != 0 ) result = p->setNumberOfBytes( handle[3] );

	return( result );
}

int
NFSsetNumberOfUnits( char * buf )
{
	int status = 0;
	int * handle = (int *)buf;

	int result = 0;

	PersistentMemoryObjectStar p = MOC->open( handle[2], status );

	if( p != 0 ) result = p->setNumberOfUnits( handle[3] );

	return( result );
}

int
NFSsetUserId( char * buf )
{
	int status = 0;
	int * handle = (int *)buf;

	int result = 0;

	PersistentMemoryObjectStar p = MOC->open( handle[2], status );

	if( p != 0 ) result = p->setUserId( handle[3] );

	return( result );
}

int
NFSsetGroupId( char * buf )
{
	int status = 0;
	int * handle = (int *)buf;

	int result = 0;

	PersistentMemoryObjectStar p = MOC->open( handle[2], status );

	if( p != 0 ) result = p->setGroupId( handle[3] );

	return( result );
}

int
NFSsetMode( char * buf )
{
	int status = 0;
	int * handle = (int *)buf;

	int result = 0;

	PersistentMemoryObjectStar p = MOC->open( handle[2], status );

	if( p != 0 ) result = p->setMode( handle[3] );

	return( result );
}

char *
NFSread( char * buf, int & size )
{
	int status = 0;
	int * handle = (int *)buf;
	size = 0;

	PersistentMemoryObjectStar p = MOC->open( handle[2], status );
	if( p == 0 ) return( 0 );

	size = p->unitSize();
	char * result = new char[size];

	int unitsRead = p->read( handle[3], 1, result, size );
	Assert( unitsRead <= 1 );
	Assert( unitsRead >= 0 );

	if( unitsRead == 0 ) size = 0;
	
	return( result );
}

int
NFSwrite( char * buf )
{
	int status = 0;
	int * handle = (int *)buf;

	PersistentMemoryObjectStar p = MOC->open( handle[2], status );
	if( p == 0 ) return( 0 );

	int unitsWritten = p->write( handle[3], 1, &buf[16], p->unitSize() );
	Assert( unitsWritten <= 1 );
	Assert( unitsWritten >= 0 );

	return( unitsWritten );
}

UNIXFileInfo *
NFSinfo( char * buf )
{
	int status = 0;
	int * handle = (int *)buf;

	UNIXFileInfo * result = new UNIXFileInfo;

	PersistentMemoryObjectStar p = MOC->open( handle[2], status );

	if( p != 0 ) p->info( result );
	else result->device = -1;

	return( result );
}

int *
NFSmount( char * buf )
{
	int status = 0;
	PersistentMemoryObjectStar p = 0;

	StoredObjectStar s = FSI->open( buf, StoredObjectClass, status );

	if( s != 0 ) p = s->asA(  PersistentMemoryObjectClass, status );

	int * result = new int[5];
	result[2] = 0;
	if( p!= 0 ) {
		MemoryObjectContainerStar c = p->container();
		Assert( c != 0 );
		MOC = c;

		result[0] = p->numberOfUnits();
		result[1] = p->log2UnitSize();
		result[2] = 2;
		result[3] = c->idNumber();
		result[4] = p->idNumber();
	}

	return( result );
}

extern "C" {
	extern int socket( int, int, int );
	extern int bind( int, void *, int );
	extern int accept( int, void *, void * );
	extern int connect( int, void *, int );
	extern int listen( int, int );
	extern int unlink( char * );
	extern int close( int );
}

int
server( int argc, char *argv[] )
{
	if( argc != 2 ) {
		stdout->formattedWrite( "usage: serve host-name\n" );
		return( NumberOfArguments );
	}

	int sd = socket( AF_UNIX, SOCK_STREAM, 0 );
	if( sd < 0 ) {
		stdout->formattedWrite( "socket %d %d\n", sd, errno );
		return( 0 );
	}

	struct sockaddr sockaddr;
	sockaddr.sa_family = AF_UNIX;
	strcpy( sockaddr.sa_data, argv[1] );

	int sts = bind( sd, &sockaddr, sizeof( sockaddr ) );
	if( sts != 0 ) {
		stdout->formattedWrite( "bind %d %d\n", sts, errno );
		return( 0 );
	}

	sts = listen( sd, 1 );
	if( sts != 0 ) {
		stdout->formattedWrite( "listen %d %d\n", sts, errno );
		return( 0 );
	}

	int fromlen = 0;
	int ns = accept( sd, &sockaddr, &fromlen );
	if( ns < 0 ) {
		stdout->formattedWrite( "accept %d %d\n", sts, errno );
		return( 0 );
	}

	close( sd );

	char * buf = new char[8448];
	int count = 0;

	while( ( count = ::read( ns, buf, 8448 ) ) > 0 ) {
		int opcode = *(int *)buf;
		//stdout->formattedWrite( "%c %d\n", opcode, count );
		buf[count] = '\0';

		switch( opcode ) {
		case NFSmountOp: {
			int * result = NFSmount( &buf[4] );
			write( ns, (char *)result, ( result[2] + 3 ) * 4 );
			delete result;
			break;
		}
		case NFSunmountOp:
			break;

		case NFSreadOp: {
			int size = 0;
			char * result = NFSread( &buf[4], size );
			write( ns, result, size );
			delete result;
			break;
		}
		case NFSwriteOp: {
			int result = NFSwrite( &buf[4] );
			write( ns, (char *)&result, sizeof(int) );
			break;
		}
		case NFSinfoOp: {
			UNIXFileInfo * result = NFSinfo( &buf[4] );
			write( ns, (char *)result, sizeof( UNIXFileInfo ) );
			delete result;
			break;
		}
		case NFSsetNumberOfBytesOp: {
			int result = NFSsetNumberOfBytes( &buf[4] );
			write( ns, (char *)&result, sizeof( int ) );
			break;
		}
		case NFSsetNumberOfUnitsOp: {
			int result = NFSsetNumberOfUnits( &buf[4] );
			write( ns, (char *)&result, sizeof( int ) );
			break;
		}
		case NFSsetUserIdOp: {
			int result = NFSsetUserId( &buf[4] );
			write( ns, (char *)&result, sizeof( int ) );
			break;
		}
		case NFSsetGroupIdOp: {
			int result = NFSsetGroupId( &buf[4] );
			write( ns, (char *)&result, sizeof( int ) );
			break;
		}
		case NFSsetModeOp: {
			int result = NFSsetMode( &buf[4] );
			write( ns, (char *)&result, sizeof( int ) );
			break;
		}

		case NFSopenOp: {
			int * result = NFSopen( &buf[4] );
			write( ns, (char *)result, ( result[2] + 3 ) * 4 );
			delete result;
			break;
		}
		case NFScreateOp:
			break;
		case NFSaddOp:
			break;
		case NFSremoveOp:
			break;
		case NFSkeysOp: {
			int size = 0;
			int count = 0;
			char * result = NFSkeys( &buf[4], size, count );
			write( ns, (char *)&count, sizeof(int) );
			write( ns, result, size );
			delete result;
			break;
		}

		default:
			Assert( NOTREACHED );
		}
	}

	MOC = 0;
	close( ns );

	unlink( argv[1] );

	return( 0 );
}

PersistentMemoryObjectStar
clientMount( char * machine, char * path )
{
	int sd = socket( AF_UNIX, SOCK_STREAM, 0 );
	if( sd < 0 ) {
		stdout->formattedWrite( "socket %d %d\n", sd, errno );
		return( 0 );
	}

	struct sockaddr sockaddr;
	sockaddr.sa_family = AF_UNIX;
	strcpy( sockaddr.sa_data , machine );

	int sts = connect( sd, &sockaddr, sizeof( sockaddr ) );
	if( sts != 0 ) {
		stdout->formattedWrite( "connect %d %d\n", sts, errno );
		return( 0 );
	}

	char * buf = new char[8448];
	int * ibuf = (int *)buf;

	ibuf[0] = NFSmountOp;
	strcpy( (char *)&ibuf[1], path );

	write( sd, buf, sizeof(int) + strlen( path ) );
	int count = read( sd, buf, 8448 );
	Assert( count >= 12 );

	PersistentMemoryObjectStar p = 0;
	if( ibuf[2] > 0 ) {
		p = new NFSDisk( ibuf[0], ibuf[1], sd, &ibuf[2] );
	}

	delete buf;

	return( p );
}
