#ifdef __GNUC__
#pragma implementation
#endif


/**********************************************************************
$Id: interf.cc 2 2004-06-03 01:58:28Z jcarlyle $

$Source: /projects/choices/old/cvsroot/choices/Applications/Water/interf.cc,v $

$Log: interf.cc,v $
Revision 1.1  2004/06/03 01:58:30  jcarlyle
Initial revision

Revision 2.1  1992/06/03  19:52:23  zhang
Ansified version, initial checkin

Revision 1.4  1991/12/06  17:39:28  zhang
Added trace macros calls to procedure entry and exit

Revision 1.3  91/11/22  14:21:14  zhang
*** empty log message ***

Revision 1.2  91/10/30  11:09:25  zhang
Added log message

This routine calculates inter-molecular interaction forces.
**********************************************************************/




#ifndef BUFSIZ
#include "stdio.h"
#include "stdlib.h"
#endif
#include "Math.h"



				/* Main thread start and end times. */
extern unsigned start_time, end_time;
extern unsigned thread_num;
 


extern char *spinlock_addr;	/* For spinlock macro. */
 




#include "Assert.h"
#include "FreeRunningTimer.h"
#include "NameServer.h"
#include "Process.h"
#include "ThisProcess.h"
#include "UnixApplication.h"

extern FreeRunningTimer* choices_clock;


extern void *shnew(long n);
extern void initializeNewSharedAllocator(void);

#define MAX_NUM_PROCS 128
extern Process* child_procs[MAX_NUM_PROCS];
extern unsigned num_child_procs;
 
 



#include "String.h"


#include "ParmacsTrace.h"

extern char trace_fname[];
extern FILE *trace_file;
#define TRACE_BUFF_SIZE 262144
extern unsigned trace_buff[TRACE_BUFF_SIZE * 4];
extern unsigned *trace_loc;
extern unsigned *final_trace_loc;
unsigned *trace_buff_end;
extern unsigned thread_pid;
extern unsigned num_children;
extern char *ParmacsTrace_version;

extern void trace_io_error(char *func);
extern void trace_overflow_error(void);
 
 

#include "mdvar.h"
#include "water.h"
#include "wwpot.h"
#include "cnst.h"
#include "parameters.h"
#include "mddata.h"
#include "split.h"
#include "global.h"

static void
UPDATE_FORCES(int DEST, int mol, int comp, double XL[], double YL[],
	      double ZL[], double FF[]);

void
INTERF(int DEST, double *VIR)
{
     /* This routine gets called both from main() and from mdmain().
        When called from main(), it is used to estimate the initial
        accelerations by computing intermolecular forces.  When called
        from mdmain(), it is used to compute intermolecular forces.
        The parameter DEST specifies whether results go into the 
        accelerations or the forces. Uses routine UPDATE_FORCES in this
        file, and routine CSHIFT in file cshift.U */
/*
.....this routine calculates inter-molecular interaction forces
     the distances are arranged in the order  M-M, M-H1, M-H3, H1-M,
     H3-M, H1-H3, H1-H1, H3-H1, H3-H3, O-O, O-H1, O-H3, H1-O, H3-O, 
     where the M are "centers" of the molecules.
*/

    int mol, comp, dir, icomp;
    int    KC, K;
    double YL[15], XL[15], ZL[15], RS[15], FF[15], RL[15]; /* per-
            interaction arrays that hold some computed distances */
    double  FTEMP;
    double LVIR = 0.0;

    for (mol = StartMol[ProcID]; mol < StartMol[ProcID+1]; mol++) {
	int comp_last = mol + NMOL/2;
	if (NMOL%2==0 && mol%2) comp_last -= 1;
        for (icomp = mol+1; icomp <= comp_last; icomp++) {
	    comp = icomp;
	    if (comp > NMOL1) comp = comp%NMOL;

            /*  compute some intermolecular distances */

	    {
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcEnter;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 24;	/* data_id */


	} else
		trace_overflow_error();
}  
      	    CSHIFT(VAR[mol].F[DISP][XDIR],VAR[comp].F[DISP][XDIR],
	           VAR[mol].VM[XDIR],VAR[comp].VM[XDIR],XL,BOXH,BOXL);
	    {
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcExit;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 24;	/* data_id */


	} else
		trace_overflow_error();
}  

	    {
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcEnter;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 24;	/* data_id */


	} else
		trace_overflow_error();
}  
            CSHIFT(VAR[mol].F[DISP][YDIR],VAR[comp].F[DISP][YDIR],
	           VAR[mol].VM[YDIR],VAR[comp].VM[YDIR],YL,BOXH,BOXL);
	    {
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcExit;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 24;	/* data_id */


	} else
		trace_overflow_error();
}  

	    {
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcEnter;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 24;	/* data_id */


	} else
		trace_overflow_error();
}  
            CSHIFT(VAR[mol].F[DISP][ZDIR],VAR[comp].F[DISP][ZDIR],
	           VAR[mol].VM[ZDIR],VAR[comp].VM[ZDIR],ZL,BOXH,BOXL);
	    {
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcExit;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 24;	/* data_id */


	} else
		trace_overflow_error();
}  

            KC=0;
            for (K = 0; K < 9; K++) {
           	RS[K]=XL[K]*XL[K]+YL[K]*YL[K]+ZL[K]*ZL[K];
		if (RS[K] > CUT2) 
	  	    KC++;
 	    } /* for K */
      	    if (KC != 9) {
      		for (K = 0; K < 14; K++) 
       		    FF[K]=0.0;
      		if (RS[0] < CUT2) {
        	    FF[0]=QQ4/(RS[0]*sqrt(RS[0]))+REF4;
	    	    LVIR = LVIR + FF[0]*RS[0];
                } /* if */
      		for (K = 1; K < 5; K++) {
        	    if (RS[K] < CUT2) { 
			FF[K]= -QQ2/(RS[K]*sqrt(RS[K]))-REF2;
	  		LVIR = LVIR + FF[K]*RS[K];
		    } /* if */
		    if (RS[K+4] <= CUT2) { 
			RL[K+4]=sqrt(RS[K+4]);
	  		FF[K+4]=QQ/(RS[K+4]*RL[K+4])+REF1;
	  		LVIR = LVIR + FF[K+4]*RS[K+4];
		    } /* if */
      		} /* for K */
      		if (KC == 0) {
        	    RS[9]=XL[9]*XL[9]+YL[9]*YL[9]+ZL[9]*ZL[9];
	   	    RL[9]=sqrt(RS[9]);
	  	    FF[9]=AB1*exp(-B1*RL[9])/RL[9];
		    LVIR = LVIR + FF[9]*RS[9];
		    for (K = 10; K < 14; K++) { 
			FTEMP=AB2*exp(-B2*RL[K-5])/RL[K-5];
	  		FF[K-5]=FF[K-5]+FTEMP;
	  		LVIR= LVIR+FTEMP*RS[K-5];
	  		RS[K]=XL[K]*XL[K]+YL[K]*YL[K]+ZL[K]*ZL[K];
	  		RL[K]=sqrt(RS[K]);
	  		FF[K]=(AB3*exp(-B3*RL[K])-AB4*exp(-B4*RL[K]))/RL[K];
	  		LVIR = LVIR + FF[K]*RS[K];
		    } /* for K */
      		} /* if KC == 0 */

	        {
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcEnter;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 25;	/* data_id */


	} else
		trace_overflow_error();
}  
		UPDATE_FORCES(DEST, mol, comp, XL, YL, ZL, FF);
	        {
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcExit;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 25;	/* data_id */


	} else
		trace_overflow_error();
}  

    	    }  /* if KC != 9 */  
        } /* for comp */
    } /* for mol */

            /*  accumulate the running sum from private 
                 per-interaction partial sums   */
    {
	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_Lock;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->InterfVirLock;	/* data_id */


	} else
		trace_overflow_error();
}  
	{
	{
#ifdef SPARC
	unsigned status;
	do {
		register unsigned regvar;
		asm volatile ("ldstub %1,%0" : "=r" (regvar) : "m" ((gl->InterfVirLock[0].lock)) ); 
		status = regvar;
		if ( status ) thisProcess()->relinquishProcessor();
	} while( status );
/*
#define LOAD_REG_FROM_MEM(reg, var) \
({	register unsigned fake_register_variable asm(#reg); \
	asm volatile ("ld %1,%0" : "=r" (fake_register_variable) : "m" (var) ); })
*/
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->InterfVirLock[0].lock)));
	asm("L2222:");
	asm("cmpqb	$0, 0(_spinlock_addr)");
	asm("bne 	L2222");
	asm("sbitib	$0, 0(_spinlock_addr)");
	asm("bfs 	L2222"); 
	
#endif
} ;
} 
	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_LockDone;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->InterfVirLock;	/* data_id */


	} else
		trace_overflow_error();
}  
} 
    *VIR = *VIR + LVIR;
    {
	{
	gl->InterfVirLock[0].lock = 0;
} 
	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_Unlock;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->InterfVirLock;	/* data_id */


	} else
		trace_overflow_error();
}  
}

            /* wait till all forces are updated */

    {
	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_BarEnter;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->InterfBar;	/* data_id */


	} else
		trace_overflow_error();
}  

	

	{
#ifdef SPARC
	unsigned status;
	do {
		register unsigned regvar;
		asm volatile ("ldstub %1,%0" : "=r" (regvar) : "m" ((gl->InterfBar[0].lock)) ); 
		status = regvar;
		if ( status ) thisProcess()->relinquishProcessor();
	} while( status );
/*
#define LOAD_REG_FROM_MEM(reg, var) \
({	register unsigned fake_register_variable asm(#reg); \
	asm volatile ("ld %1,%0" : "=r" (fake_register_variable) : "m" (var) ); })
*/
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->InterfBar[0].lock)));
	asm("L2223:");
	asm("cmpqb	$0, 0(_spinlock_addr)");
	asm("bne 	L2223");
	asm("sbitib	$0, 0(_spinlock_addr)");
	asm("bfs 	L2223"); 
	
#endif
} ;

	if (gl->InterfBar[0].count[0] < (NumProcs - 1)) {
		gl->InterfBar[0].count[0]++;
		{
#ifdef SPARC
	(gl->InterfBar[0].lock) = 0;
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->InterfBar[0].lock)));
	asm("movqb	$0, 0(_spinlock_addr)");
#endif
} ;
		{
#ifdef SPARC
	unsigned status;
	do {
		register unsigned regvar;
		asm volatile ("ldstub %1,%0" : "=r" (regvar) : "m" ((gl->InterfBar[0].queue[0])) ); 
		status = regvar;
		if ( status ) thisProcess()->relinquishProcessor();
	} while( status );
/*
#define LOAD_REG_FROM_MEM(reg, var) \
({	register unsigned fake_register_variable asm(#reg); \
	asm volatile ("ld %1,%0" : "=r" (fake_register_variable) : "m" (var) ); })
*/
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->InterfBar[0].queue[0])));
	asm("L2224:");
	asm("cmpqb	$0, 0(_spinlock_addr)");
	asm("bne 	L2224");
	asm("sbitib	$0, 0(_spinlock_addr)");
	asm("bfs 	L2224"); 
	
#endif
} ;
	}

	if (gl->InterfBar[0].count[0] == 0) {
		{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_BarExitLast;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->InterfBar;	/* data_id */


	} else
		trace_overflow_error();
}  
		{
#ifdef SPARC
	(gl->InterfBar[0].lock) = 0;
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->InterfBar[0].lock)));
	asm("movqb	$0, 0(_spinlock_addr)");
#endif
} ;
	} else {
		(gl->InterfBar[0].count[0])--;
		{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_BarExit;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->InterfBar;	/* data_id */


	} else
		trace_overflow_error();
}  
		{
#ifdef SPARC
	(gl->InterfBar[0].queue[0]) = 0;
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->InterfBar[0].queue[0])));
	asm("movqb	$0, 0(_spinlock_addr)");
#endif
} ;
	}
 
} 

    /* divide final forces by masses */

    for (mol = StartMol[ProcID]; mol < StartMol[ProcID+1]; mol++) {
	for ( dir = XDIR; dir  <= ZDIR; dir++) {
	    VAR[mol].F[DEST][dir][H1] = VAR[mol].F[DEST][dir][H1] * FHM;
	    VAR[mol].F[DEST][dir][H2] = VAR[mol].F[DEST][dir][H2] * FHM;
	    VAR[mol].F[DEST][dir][O]  = VAR[mol].F[DEST][dir][O] * FOM;
	} /* for dir */
    } /* for mol */
}/* end of subroutine INTERF */

static void
UPDATE_FORCES(int DEST, int mol, int comp, double XL[], double YL[],
	      double ZL[], double FF[])
        /* from the computed distances etc., compute the 
            intermolecular forces and update the force (or 
            acceleration) locations */
{
    int K;
    double G110[3], G23[3], G45[3], TT1[3], TT[3], TT2[3];
    double GG[15][3];

	/*   CALCULATE X-COMPONENT FORCES */
	for (K = 0; K < 14; K++)  {
	    GG[K+1][XDIR] = FF[K]*XL[K];
	    GG[K+1][YDIR] = FF[K]*YL[K];
	    GG[K+1][ZDIR] = FF[K]*ZL[K];
	}

	G110[XDIR] = GG[10][XDIR]+GG[1][XDIR]*C1;
	G110[YDIR] = GG[10][YDIR]+GG[1][YDIR]*C1;
	G110[ZDIR] = GG[10][ZDIR]+GG[1][ZDIR]*C1;
	G23[XDIR] = GG[2][XDIR]+GG[3][XDIR];
	G23[YDIR] = GG[2][YDIR]+GG[3][YDIR];
	G23[ZDIR] = GG[2][ZDIR]+GG[3][ZDIR];
	G45[XDIR]=GG[4][XDIR]+GG[5][XDIR];
	G45[YDIR]=GG[4][YDIR]+GG[5][YDIR];
	G45[ZDIR]=GG[4][ZDIR]+GG[5][ZDIR];
	TT1[XDIR] =GG[1][XDIR]*C2;
	TT1[YDIR] =GG[1][YDIR]*C2;
	TT1[ZDIR] =GG[1][ZDIR]*C2;
	TT[XDIR] =G23[XDIR]*C2+TT1[XDIR];
	TT[YDIR] =G23[YDIR]*C2+TT1[YDIR];
	TT[ZDIR] =G23[ZDIR]*C2+TT1[ZDIR];
	TT2[XDIR]=G45[XDIR]*C2+TT1[XDIR];
	TT2[YDIR]=G45[YDIR]*C2+TT1[YDIR];
	TT2[ZDIR]=G45[ZDIR]*C2+TT1[ZDIR];
            /* lock locations for the molecule to be updated */
	{
	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ALock;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->MolLock[mol];	/* data_id */


	} else
		trace_overflow_error();
}  
	{
	{
#ifdef SPARC
	unsigned status;
	do {
		register unsigned regvar;
		asm volatile ("ldstub %1,%0" : "=r" (regvar) : "m" ((gl->MolLock[mol].lock)) ); 
		status = regvar;
		if ( status ) thisProcess()->relinquishProcessor();
	} while( status );
/*
#define LOAD_REG_FROM_MEM(reg, var) \
({	register unsigned fake_register_variable asm(#reg); \
	asm volatile ("ld %1,%0" : "=r" (fake_register_variable) : "m" (var) ); })
*/
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->MolLock[mol].lock)));
	asm("L2225:");
	asm("cmpqb	$0, 0(_spinlock_addr)");
	asm("bne 	L2225");
	asm("sbitib	$0, 0(_spinlock_addr)");
	asm("bfs 	L2225"); 
	
#endif
} ;
} 
	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ALockDone;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->MolLock[mol];	/* data_id */


	} else
		trace_overflow_error();
}  
}  
	VAR[mol].F[DEST][XDIR][O] +=
		G110[XDIR] + GG[11][XDIR] +GG[12][XDIR]+C1*G23[XDIR];
	VAR[mol].F[DEST][XDIR][H1] += 
		GG[6][XDIR]+GG[7][XDIR]+GG[13][XDIR]+TT[XDIR]+GG[4][XDIR];
	VAR[mol].F[DEST][XDIR][H2] +=
	 	GG[8][XDIR]+GG[9][XDIR]+GG[14][XDIR]+TT[XDIR]+GG[5][XDIR];
	VAR[mol].F[DEST][YDIR][O]  += 
		G110[YDIR]+GG[11][YDIR]+GG[12][YDIR]+C1*G23[YDIR];
	VAR[mol].F[DEST][YDIR][H1] += 
		GG[6][YDIR]+GG[7][YDIR]+GG[13][YDIR]+TT[YDIR]+GG[4][YDIR];
	VAR[mol].F[DEST][YDIR][H2] += 
		GG[8][YDIR]+GG[9][YDIR]+GG[14][YDIR]+TT[YDIR]+GG[5][YDIR];
	VAR[mol].F[DEST][ZDIR][O]  +=
		G110[ZDIR]+GG[11][ZDIR]+GG[12][ZDIR]+C1*G23[ZDIR];
	VAR[mol].F[DEST][ZDIR][H1] +=
		GG[6][ZDIR]+GG[7][ZDIR]+GG[13][ZDIR]+TT[ZDIR]+GG[4][ZDIR];
	VAR[mol].F[DEST][ZDIR][H2] +=
		GG[8][ZDIR]+GG[9][ZDIR]+GG[14][ZDIR]+TT[ZDIR]+GG[5][ZDIR];
	{
	{
	gl->MolLock[mol].lock = 0;
} 
	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_AUnlock;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->MolLock[mol];	/* data_id */


	} else
		trace_overflow_error();
}  
} 

	{
	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ALock;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->MolLock[comp];	/* data_id */


	} else
		trace_overflow_error();
}  
	{
	{
#ifdef SPARC
	unsigned status;
	do {
		register unsigned regvar;
		asm volatile ("ldstub %1,%0" : "=r" (regvar) : "m" ((gl->MolLock[comp].lock)) ); 
		status = regvar;
		if ( status ) thisProcess()->relinquishProcessor();
	} while( status );
/*
#define LOAD_REG_FROM_MEM(reg, var) \
({	register unsigned fake_register_variable asm(#reg); \
	asm volatile ("ld %1,%0" : "=r" (fake_register_variable) : "m" (var) ); })
*/
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->MolLock[comp].lock)));
	asm("L2226:");
	asm("cmpqb	$0, 0(_spinlock_addr)");
	asm("bne 	L2226");
	asm("sbitib	$0, 0(_spinlock_addr)");
	asm("bfs 	L2226"); 
	
#endif
} ;
} 
	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ALockDone;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->MolLock[comp];	/* data_id */


	} else
		trace_overflow_error();
}  
} 
        VAR[comp].F[DEST][XDIR][O] +=
		   -G110[XDIR]-GG[13][XDIR]-GG[14][XDIR]-C1*G45[XDIR];
	VAR[comp].F[DEST][XDIR][H1] +=
		   -GG[6][XDIR]-GG[8][XDIR]-GG[11][XDIR]-TT2[XDIR]-GG[2][XDIR];
	VAR[comp].F[DEST][XDIR][H2] +=
		   -GG[7][XDIR]-GG[9][XDIR]-GG[12][XDIR]-TT2[XDIR]-GG[3][XDIR];
	VAR[comp].F[DEST][YDIR][O] += 
		-G110[YDIR]-GG[13][YDIR]-GG[14][YDIR]-C1*G45[YDIR];
	VAR[comp].F[DEST][YDIR][H1] += 
		-GG[6][YDIR]-GG[8][YDIR]-GG[11][YDIR]-TT2[YDIR]-GG[2][YDIR];
	VAR[comp].F[DEST][YDIR][H2] += 
		-GG[7][YDIR]-GG[9][YDIR]-GG[12][YDIR]-TT2[YDIR]-GG[3][YDIR];
	VAR[comp].F[DEST][ZDIR][O] +=
		-G110[ZDIR]-GG[13][ZDIR]-GG[14][ZDIR]-C1*G45[ZDIR];
	VAR[comp].F[DEST][ZDIR][H1] +=
		-GG[6][ZDIR]-GG[8][ZDIR]-GG[11][ZDIR]-TT2[ZDIR]-GG[2][ZDIR];
	VAR[comp].F[DEST][ZDIR][H2] +=
		-GG[7][ZDIR]-GG[9][ZDIR]-GG[12][ZDIR]-TT2[ZDIR]-GG[3][ZDIR];
	{
	{
	gl->MolLock[comp].lock = 0;
} 
	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_AUnlock;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->MolLock[comp];	/* data_id */


	} else
		trace_overflow_error();
}  
}  
}           /* end of subroutine UPDATE_FORCES */
