#ifdef __GNUC__
#pragma implementation
#endif


/**********************************************************************
$Id: poteng.cc 2 2004-06-03 01:58:28Z jcarlyle $

$Source: /projects/choices/old/cvsroot/choices/Applications/Water/poteng.cc,v $

$Log: poteng.cc,v $
Revision 1.1  2004/06/03 01:58:30  jcarlyle
Initial revision

Revision 2.1  1992/06/03  19:52:23  zhang
Ansified version, initial checkin

Revision 1.3  1991/12/06  17:40:14  zhang
Added trace macros calls to procedure entry and exit

Revision 1.2  91/10/30  11:10:06  zhang
Added log message


This routine calculates the potential energy of the system.
**********************************************************************/





#ifndef BUFSIZ
#include "stdio.h"
#include "stdlib.h"
#endif
#include "Math.h"



				/* Main thread start and end times. */
extern unsigned start_time, end_time;
extern unsigned thread_num;
 


extern char *spinlock_addr;	/* For spinlock macro. */
 




#include "Assert.h"
#include "FreeRunningTimer.h"
#include "NameServer.h"
#include "Process.h"
#include "ThisProcess.h"
#include "UnixApplication.h"

extern FreeRunningTimer* choices_clock;


extern void *shnew(long n);
extern void initializeNewSharedAllocator(void);

#define MAX_NUM_PROCS 128
extern Process* child_procs[MAX_NUM_PROCS];
extern unsigned num_child_procs;
 
 



#include "String.h"


#include "ParmacsTrace.h"

extern char trace_fname[];
extern FILE *trace_file;
#define TRACE_BUFF_SIZE 262144
extern unsigned trace_buff[TRACE_BUFF_SIZE * 4];
extern unsigned *trace_loc;
extern unsigned *final_trace_loc;
unsigned *trace_buff_end;
extern unsigned thread_pid;
extern unsigned num_children;
extern char *ParmacsTrace_version;

extern void trace_io_error(char *func);
extern void trace_overflow_error(void);
 
 

#include "mdvar.h"
#include "frcnst.h"
#include "water.h"
#include "wwpot.h"
#include "parameters.h"
#include "mddata.h"
#include "split.h"
#include "global.h"

void			/* some shared sums computed by POTENG */
POTENG(double *POTA, double *POTR, double *PTRF)
{

/*
    this routine calculates the potential energy of the system.
     FC11 ,FC12, FC13, and FC33 are the quardratic force constants
*/

    int mol,comp;
    int    KC, K;
    double R1, R2, RX, COS, DT, DR1, DR2, DR1S, DR2S, DRP;
    /* , DRS; Not used. */
    double XL[15], YL[15], ZL[15], RS[15], RL[15];
    double DTS;
    double LPOTA, LPOTR, LPTRF;

    /*  compute intra-molecular potential energy */
    LPOTA=0.0;
    for (mol = StartMol[ProcID]; mol < StartMol[ProcID+1]; mol++) {
	VAR[mol].VM[XDIR] = C1 * VAR[mol].F[DISP][XDIR][ O] +
			    C2 * (VAR[mol].F[DISP][XDIR][H1] + 
				  VAR[mol].F[DISP][XDIR][H2] );
	VAR[mol].VM[YDIR] = C1*VAR[mol].F[DISP][YDIR][O] +
			    C2*(VAR[mol].F[DISP][YDIR][H1] + 
				VAR[mol].F[DISP][YDIR][H2] );
	VAR[mol].VM[ZDIR] = C1*VAR[mol].F[DISP][ZDIR][O] +
			    C2*(VAR[mol].F[DISP][ZDIR][H1] + 
				VAR[mol].F[DISP][ZDIR][H2] );
	R1 = pow( (VAR[mol].F[DISP][XDIR][O]-VAR[mol].F[DISP][XDIR][H1]),2.0) +
	     pow( (VAR[mol].F[DISP][YDIR][O]-VAR[mol].F[DISP][YDIR][H1]),2.0) +
	     pow( (VAR[mol].F[DISP][ZDIR][O]-VAR[mol].F[DISP][ZDIR][H1]),2.0);

	R2 = pow( (VAR[mol].F[DISP][XDIR][O]-VAR[mol].F[DISP][XDIR][H2]),2.0) +
	     pow( (VAR[mol].F[DISP][YDIR][O]-VAR[mol].F[DISP][YDIR][H2]),2.0) +
	     pow( (VAR[mol].F[DISP][ZDIR][O]-VAR[mol].F[DISP][ZDIR][H2]),2.0);

	RX = ( (VAR[mol].F[DISP][XDIR][O]-VAR[mol].F[DISP][XDIR][H1]) *
	       (VAR[mol].F[DISP][XDIR][O]-VAR[mol].F[DISP][XDIR][H2])  ) +
	     ( (VAR[mol].F[DISP][YDIR][O]-VAR[mol].F[DISP][YDIR][H1]) *
	       (VAR[mol].F[DISP][YDIR][O]-VAR[mol].F[DISP][YDIR][H2])  ) +
	     ( (VAR[mol].F[DISP][ZDIR][O]-VAR[mol].F[DISP][ZDIR][H1]) *
	       (VAR[mol].F[DISP][ZDIR][O]-VAR[mol].F[DISP][ZDIR][H2])  );
    	R1=sqrt(R1);
    	R2=sqrt(R2);
    	COS=RX/(R1*R2);
    	DT=(acos(COS)-ANGLE)*ROH;
    	DR1=R1-ROH;
    	DR2=R2-ROH;
    	DR1S=DR1*DR1;
    	DR2S=DR2*DR2;
    	DRP=DR1+DR2;
    	DTS=DT*DT;
    	LPOTA += (FC11*(DR1S+DR2S)+FC33*DTS)*0.5
	       +FC12*DR1*DR2+FC13*DRP*DT
               +(FC111*(DR1S*DR1+DR2S*DR2)+FC333*DTS*DT+FC112*DRP*DR1*DR2+
	         FC113*(DR1S+DR2S)*DT+FC123*DR1*DR2*DT+FC133*DRP*DTS)*ROHI;
        LPOTA += (FC1111*(DR1S*DR1S+DR2S*DR2S)+FC3333*DTS*DTS+
	 FC1112*(DR1S+DR2S)*DR1*DR2+FC1122*DR1S*DR2S+
	 FC1113*(DR1S*DR1+DR2S*DR2)*DT+FC1123*DRP*DR1*DR2*DT+
	 FC1133*(DR1S+DR2S)*DTS+FC1233*DR1*DR2*DTS+
	 FC1333*DRP*DTS*DT)*ROHI2;
    } /* for mol */

    {
	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_BarEnter;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->PotengBar;	/* data_id */


	} else
		trace_overflow_error();
}  

	

	{
#ifdef SPARC
	unsigned status;
	do {
		register unsigned regvar;
		asm volatile ("ldstub %1,%0" : "=r" (regvar) : "m" ((gl->PotengBar[0].lock)) ); 
		status = regvar;
		if ( status ) thisProcess()->relinquishProcessor();
	} while( status );
/*
#define LOAD_REG_FROM_MEM(reg, var) \
({	register unsigned fake_register_variable asm(#reg); \
	asm volatile ("ld %1,%0" : "=r" (fake_register_variable) : "m" (var) ); })
*/
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->PotengBar[0].lock)));
	asm("L2222:");
	asm("cmpqb	$0, 0(_spinlock_addr)");
	asm("bne 	L2222");
	asm("sbitib	$0, 0(_spinlock_addr)");
	asm("bfs 	L2222"); 
	
#endif
} ;

	if (gl->PotengBar[0].count[0] < (NumProcs - 1)) {
		gl->PotengBar[0].count[0]++;
		{
#ifdef SPARC
	(gl->PotengBar[0].lock) = 0;
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->PotengBar[0].lock)));
	asm("movqb	$0, 0(_spinlock_addr)");
#endif
} ;
		{
#ifdef SPARC
	unsigned status;
	do {
		register unsigned regvar;
		asm volatile ("ldstub %1,%0" : "=r" (regvar) : "m" ((gl->PotengBar[0].queue[0])) ); 
		status = regvar;
		if ( status ) thisProcess()->relinquishProcessor();
	} while( status );
/*
#define LOAD_REG_FROM_MEM(reg, var) \
({	register unsigned fake_register_variable asm(#reg); \
	asm volatile ("ld %1,%0" : "=r" (fake_register_variable) : "m" (var) ); })
*/
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->PotengBar[0].queue[0])));
	asm("L2223:");
	asm("cmpqb	$0, 0(_spinlock_addr)");
	asm("bne 	L2223");
	asm("sbitib	$0, 0(_spinlock_addr)");
	asm("bfs 	L2223"); 
	
#endif
} ;
	}

	if (gl->PotengBar[0].count[0] == 0) {
		{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_BarExitLast;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->PotengBar;	/* data_id */


	} else
		trace_overflow_error();
}  
		{
#ifdef SPARC
	(gl->PotengBar[0].lock) = 0;
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->PotengBar[0].lock)));
	asm("movqb	$0, 0(_spinlock_addr)");
#endif
} ;
	} else {
		(gl->PotengBar[0].count[0])--;
		{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_BarExit;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->PotengBar;	/* data_id */


	} else
		trace_overflow_error();
}  
		{
#ifdef SPARC
	(gl->PotengBar[0].queue[0]) = 0;
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->PotengBar[0].queue[0])));
	asm("movqb	$0, 0(_spinlock_addr)");
#endif
} ;
	}
 
} 

    /*  compute inter-molecular potential energy */
    LPOTR=0.0;
    LPTRF=0.0;
    for (mol = StartMol[ProcID]; mol < StartMol[ProcID+1]; mol++) {
	int comp_last = mol + NMOL/2;
	int icomp;
	if (NMOL%2==0 && mol%2) comp_last -= 1;
        for (icomp = mol+1; icomp <= comp_last; icomp++) {
	    comp = icomp;
	    if (comp > NMOL1) comp = comp%NMOL;
	    {
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcEnter;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 24;	/* data_id */


	} else
		trace_overflow_error();
}  
	    CSHIFT(VAR[mol].F[DISP][XDIR],VAR[comp].F[DISP][XDIR],
		   VAR[mol].VM[XDIR], VAR[comp].VM[XDIR],XL,BOXH,BOXL);
	    {
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcExit;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 24;	/* data_id */


	} else
		trace_overflow_error();
}  

	    {
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcEnter;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 24;	/* data_id */


	} else
		trace_overflow_error();
}  
	    CSHIFT(VAR[mol].F[DISP][YDIR],VAR[comp].F[DISP][YDIR],
		   VAR[mol].VM[YDIR], VAR[comp].VM[YDIR],YL,BOXH,BOXL);
	    {
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcExit;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 24;	/* data_id */


	} else
		trace_overflow_error();
}  

	    {
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcEnter;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 24;	/* data_id */


	} else
		trace_overflow_error();
}  
	    CSHIFT(VAR[mol].F[DISP][ZDIR],VAR[comp].F[DISP][ZDIR],
		   VAR[mol].VM[ZDIR], VAR[comp].VM[ZDIR],ZL,BOXH,BOXL);
	    {
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcExit;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 24;	/* data_id */


	} else
		trace_overflow_error();
}  

            KC=0;
            for (K = 0; K < 9; K++) {
           	RS[K]=XL[K]*XL[K]+YL[K]*YL[K]+ZL[K]*ZL[K];
		if (RS[K] > CUT2) 
		    KC=KC+1;
      	    } /* for k */
      	    if (KC != 9) {
      		for (K = 0; K < 9; K++) {
      		    if (RS[K] <= CUT2)
	 		RL[K]=sqrt(RS[K]);
		    else {
		    	RL[K]=CUTOFF;
	  		RS[K]=CUT2;
		    } /* else */
      	      	} /* for K */
      		LPOTR= LPOTR-QQ2/RL[1]-QQ2/RL[2]-QQ2/RL[3]-QQ2/RL[4]
	               +QQ /RL[5]+QQ /RL[6]+QQ /RL[7]+QQ /RL[8]
	       	       +QQ4/RL[0];
      		LPTRF= LPTRF-REF2*RS[0]-REF1*((RS[5]+RS[6]+RS[7]+RS[8])*0.5
		       -RS[1]-RS[2]-RS[3]-RS[4]);
                if (KC <= 0) {
      		    for (K = 9; K <  14; K++) 
			RL[K]=sqrt(XL[K]*XL[K]+YL[K]*YL[K]+ZL[K]*ZL[K]);
       		    LPOTR= LPOTR+A1* exp(-B1*RL[9])
	       		   +A2*(exp(-B2*RL[ 5])+exp(-B2*RL[ 6])
			   +exp(-B2*RL[ 7])+exp(-B2*RL[ 8]))
	       		   +A3*(exp(-B3*RL[10])+exp(-B3*RL[11])
			   +exp(-B3*RL[12])+exp(-B3*RL[13]))
	       		   -A4*(exp(-B4*RL[10])+exp(-B4*RL[11])
			   +exp(-B4*RL[12])+exp(-B4*RL[13]));
		} /* if KC <= 0 */
	    } /* if KC != 9 */
	} /* for comp */
    } /* for mol */

        /* update shared sums from computed  private sums */
    {
	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_Lock;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->PotengSumLock;	/* data_id */


	} else
		trace_overflow_error();
}  
	{
	{
#ifdef SPARC
	unsigned status;
	do {
		register unsigned regvar;
		asm volatile ("ldstub %1,%0" : "=r" (regvar) : "m" ((gl->PotengSumLock[0].lock)) ); 
		status = regvar;
		if ( status ) thisProcess()->relinquishProcessor();
	} while( status );
/*
#define LOAD_REG_FROM_MEM(reg, var) \
({	register unsigned fake_register_variable asm(#reg); \
	asm volatile ("ld %1,%0" : "=r" (fake_register_variable) : "m" (var) ); })
*/
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->PotengSumLock[0].lock)));
	asm("L2224:");
	asm("cmpqb	$0, 0(_spinlock_addr)");
	asm("bne 	L2224");
	asm("sbitib	$0, 0(_spinlock_addr)");
	asm("bfs 	L2224"); 
	
#endif
} ;
} 
	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_LockDone;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->PotengSumLock;	/* data_id */


	} else
		trace_overflow_error();
}  
} 
    *POTA = *POTA + LPOTA;
    *POTR = *POTR + LPOTR;
    *PTRF = *PTRF + LPTRF;
    {
	{
	gl->PotengSumLock[0].lock = 0;
} 
	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_Unlock;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->PotengSumLock;	/* data_id */


	} else
		trace_overflow_error();
}  
}
} /* end of subroutine POTENG */
