#ifdef __GNUC__
#pragma implementation
#endif


/**********************************************************************
$Id: predcor.cc 2 2004-06-03 01:58:28Z jcarlyle $

$Source: /projects/choices/old/cvsroot/choices/Applications/Water/predcor.cc,v $

$Log: predcor.cc,v $
Revision 1.1  2004/06/03 01:58:30  jcarlyle
Initial revision

Revision 2.1  1992/06/03  19:52:23  zhang
Ansified version, initial checkin

Revision 1.2  1991/11/22  14:21:32  zhang
Added EXTERN_ENV and header;


Predicts and corrects new values for displacement and its five derivatives.
**********************************************************************/




#ifndef BUFSIZ
#include "stdio.h"
#include "stdlib.h"
#endif
#include "Math.h"



				/* Main thread start and end times. */
extern unsigned start_time, end_time;
extern unsigned thread_num;
 


extern char *spinlock_addr;	/* For spinlock macro. */
 




#include "Assert.h"
#include "FreeRunningTimer.h"
#include "NameServer.h"
#include "Process.h"
#include "ThisProcess.h"
#include "UnixApplication.h"

extern FreeRunningTimer* choices_clock;


extern void *shnew(long n);
extern void initializeNewSharedAllocator(void);

#define MAX_NUM_PROCS 128
extern Process* child_procs[MAX_NUM_PROCS];
extern unsigned num_child_procs;
 
 



#include "String.h"


#include "ParmacsTrace.h"

extern char trace_fname[];
extern FILE *trace_file;
#define TRACE_BUFF_SIZE 262144
extern unsigned trace_buff[TRACE_BUFF_SIZE * 4];
extern unsigned *trace_loc;
extern unsigned *final_trace_loc;
unsigned *trace_buff_end;
extern unsigned thread_pid;
extern unsigned num_children;
extern char *ParmacsTrace_version;

extern void trace_io_error(char *func);
extern void trace_overflow_error(void);
 
 

#include "mdvar.h"
#include "parameters.h"
#include "mddata.h"
#include "split.h"
#include "water.h"

/*
Predicts new values for displacement and its five derivatives.
*/

void 
PREDIC(double C[],
       int NOR1)	/* NOR1 = NORDER + 1 = 7 (for a sixth-order
			   method) */
{
/*   this routine calculates predicted F(X), F'(X), F''(X), ... */

    int JIZ;
    int  JI;
    int  L;
    double S;
    int func, mol, dir, atom;

    JIZ=2;
    /* .....loop over F(X), F'(X), F''(X), ..... */
    for (func = 0; func < NORDER; func++) {
	for (mol = StartMol[ProcID]; mol < StartMol[ProcID+1]; mol++)
	    for ( dir = 0; dir < NDIR; dir++)
		for ( atom = 0; atom < NATOM; atom++ ) {
		    JI = JIZ;
		    /* sum over Taylor Series */
		    S = 0.0;
		    for ( L = func; L < NORDER; L++) {
			S += C[JI] * VAR[mol].F[L+1][dir][atom];
			JI++;
		    } /* for */
		    VAR[mol].F[func][dir][atom] += S;
		} /* for atom */
	JIZ += NOR1;
    } /* for func */
} /* end of subroutine PREDIC */

/*
Corrects the predicted values, based on forces etc. computed in the
interim.
*/

void
CORREC(double PCC[],	/* the predictor-corrector constants */
       int NOR1)	/* NORDER + 1 = 7 for a sixth-order method) */
{
/*
.....this routine calculates corrected F(X), F'(X), F"(X), ....
     from corrected F(X) = predicted F(X) + PCC(1)*(FR-SD)
     where SD is predicted accl. F"(X) and FR is computed 
     accl. (force/mass) at predicted position
*/

    double Y;
    int mol, dir, atom, func;

    for (mol = StartMol[ProcID]; mol < StartMol[ProcID+1]; mol++) {
/*
    for (mol = 0; mol < NMOL; mol++) {
*/
	for (dir = 0; dir < NDIR; dir++) {
	    for (atom = 0; atom < NATOM; atom++) {
		Y = VAR[mol].F[FORCES][dir][atom] - VAR[mol].F[ACC][dir][atom];
		for ( func = 0; func < NOR1; func++) 
		    VAR[mol].F[func][dir][atom] += PCC[func] * Y;   
	    } /* for atom */		
	} /* for dir */
    } /* for mol */
} /* end of subroutine CORREC */

