#ifdef __GNUC__
#pragma implementation
#endif


/**********************************************************************
$Id: redsync.cc 2 2004-06-03 01:58:28Z jcarlyle $

$Source: /projects/choices/old/cvsroot/choices/Applications/Water/redsync.cc,v $

$Log: redsync.cc,v $
Revision 1.1  2004/06/03 01:58:30  jcarlyle
Initial revision

Revision 2.1  1992/06/03  19:52:23  zhang
Ansified version, initial checkin

Revision 1.3  1991/12/06  17:41:03  zhang
Added trace macros calls to procedure entry and exit

Revision 1.2  91/10/30  11:10:22  zhang
Added log message


Routine that implements the time-steps.
**********************************************************************/





#ifndef BUFSIZ
#include "stdio.h"
#include "stdlib.h"
#endif
#include "Math.h"



				/* Main thread start and end times. */
extern unsigned start_time, end_time;
extern unsigned thread_num;
 


extern char *spinlock_addr;	/* For spinlock macro. */
 




#include "Assert.h"
#include "FreeRunningTimer.h"
#include "NameServer.h"
#include "Process.h"
#include "ThisProcess.h"
#include "UnixApplication.h"

extern FreeRunningTimer* choices_clock;


extern void *shnew(long n);
extern void initializeNewSharedAllocator(void);

#define MAX_NUM_PROCS 128
extern Process* child_procs[MAX_NUM_PROCS];
extern unsigned num_child_procs;
 
 



#include "String.h"


#include "ParmacsTrace.h"

extern char trace_fname[];
extern FILE *trace_file;
#define TRACE_BUFF_SIZE 262144
extern unsigned trace_buff[TRACE_BUFF_SIZE * 4];
extern unsigned *trace_loc;
extern unsigned *final_trace_loc;
unsigned *trace_buff_end;
extern unsigned thread_pid;
extern unsigned num_children;
extern char *ParmacsTrace_version;

extern void trace_io_error(char *func);
extern void trace_overflow_error(void);
 
 

#include "comments.h"	/* moves leading comments to another file for now */

#include "parameters.h"
#include "mdvar.h"
#include "water.h"
#include "wwpot.h"
#include "cnst.h"
#include "mddata.h"
#include "fileio.h"
#include "split.h"
#include "global.h"


    /* routine that implements the time-steps. Called by main routine 
        and calls others */

double
MDMAIN(int NFSV, int NFRST, int NSTEP, int NRST, int NPRINT, int NSAVE,
       int LKT, int NORD1)
{
    double XTT;
    int I, i;
    double POTA,POTR,POTRF;
    double XVIR,AVGT,TEN;

        /* wait till everyone gets to beginning; not necessary */

    {
	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_BarEnter;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->start;	/* data_id */


	} else
		trace_overflow_error();
}  

	

	{
#ifdef SPARC
	unsigned status;
	do {
		register unsigned regvar;
		asm volatile ("ldstub %1,%0" : "=r" (regvar) : "m" ((gl->start[0].lock)) ); 
		status = regvar;
		if ( status ) thisProcess()->relinquishProcessor();
	} while( status );
/*
#define LOAD_REG_FROM_MEM(reg, var) \
({	register unsigned fake_register_variable asm(#reg); \
	asm volatile ("ld %1,%0" : "=r" (fake_register_variable) : "m" (var) ); })
*/
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->start[0].lock)));
	asm("L2222:");
	asm("cmpqb	$0, 0(_spinlock_addr)");
	asm("bne 	L2222");
	asm("sbitib	$0, 0(_spinlock_addr)");
	asm("bfs 	L2222"); 
	
#endif
} ;

	if (gl->start[0].count[0] < (NumProcs - 1)) {
		gl->start[0].count[0]++;
		{
#ifdef SPARC
	(gl->start[0].lock) = 0;
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->start[0].lock)));
	asm("movqb	$0, 0(_spinlock_addr)");
#endif
} ;
		{
#ifdef SPARC
	unsigned status;
	do {
		register unsigned regvar;
		asm volatile ("ldstub %1,%0" : "=r" (regvar) : "m" ((gl->start[0].queue[0])) ); 
		status = regvar;
		if ( status ) thisProcess()->relinquishProcessor();
	} while( status );
/*
#define LOAD_REG_FROM_MEM(reg, var) \
({	register unsigned fake_register_variable asm(#reg); \
	asm volatile ("ld %1,%0" : "=r" (fake_register_variable) : "m" (var) ); })
*/
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->start[0].queue[0])));
	asm("L2223:");
	asm("cmpqb	$0, 0(_spinlock_addr)");
	asm("bne 	L2223");
	asm("sbitib	$0, 0(_spinlock_addr)");
	asm("bfs 	L2223"); 
	
#endif
} ;
	}

	if (gl->start[0].count[0] == 0) {
		{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_BarExitLast;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->start;	/* data_id */


	} else
		trace_overflow_error();
}  
		{
#ifdef SPARC
	(gl->start[0].lock) = 0;
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->start[0].lock)));
	asm("movqb	$0, 0(_spinlock_addr)");
#endif
} ;
	} else {
		(gl->start[0].count[0])--;
		{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_BarExit;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->start;	/* data_id */


	} else
		trace_overflow_error();
}  
		{
#ifdef SPARC
	(gl->start[0].queue[0]) = 0;
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->start[0].queue[0])));
	asm("movqb	$0, 0(_spinlock_addr)");
#endif
} ;
	}
 
} ;

    /* MOLECULAR DYNAMICS LOOP OVER ALL TIME-STEPS */

    for (i=1;i <= NSTEP; i++) {
       	TTMV=TTMV+1.00;

            /* initialize various shared sums */
	if (ProcID == 0) {
	    int dir;
	    gl->VIR = 0.0;
	    gl->POTA = 0.0;
	    gl->POTR = 0.0;
	    gl->POTRF = 0.0;
	    for (dir = XDIR; dir <= ZDIR; dir++)
		gl->SUM[dir] = 0.0;
	}

	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcEnter;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 17;	/* data_id */


	} else
		trace_overflow_error();
}  
    	PREDIC(TLC,NORD1);
	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcExit;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 17;	/* data_id */


	} else
		trace_overflow_error();
}  

	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcEnter;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 22;	/* data_id */


	} else
		trace_overflow_error();
}  
     	INTRAF(&gl->VIR);
	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcExit;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 22;	/* data_id */


	} else
		trace_overflow_error();
}  

	{
	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_BarEnter;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->start;	/* data_id */


	} else
		trace_overflow_error();
}  

	

	{
#ifdef SPARC
	unsigned status;
	do {
		register unsigned regvar;
		asm volatile ("ldstub %1,%0" : "=r" (regvar) : "m" ((gl->start[0].lock)) ); 
		status = regvar;
		if ( status ) thisProcess()->relinquishProcessor();
	} while( status );
/*
#define LOAD_REG_FROM_MEM(reg, var) \
({	register unsigned fake_register_variable asm(#reg); \
	asm volatile ("ld %1,%0" : "=r" (fake_register_variable) : "m" (var) ); })
*/
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->start[0].lock)));
	asm("L2224:");
	asm("cmpqb	$0, 0(_spinlock_addr)");
	asm("bne 	L2224");
	asm("sbitib	$0, 0(_spinlock_addr)");
	asm("bfs 	L2224"); 
	
#endif
} ;

	if (gl->start[0].count[0] < (NumProcs - 1)) {
		gl->start[0].count[0]++;
		{
#ifdef SPARC
	(gl->start[0].lock) = 0;
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->start[0].lock)));
	asm("movqb	$0, 0(_spinlock_addr)");
#endif
} ;
		{
#ifdef SPARC
	unsigned status;
	do {
		register unsigned regvar;
		asm volatile ("ldstub %1,%0" : "=r" (regvar) : "m" ((gl->start[0].queue[0])) ); 
		status = regvar;
		if ( status ) thisProcess()->relinquishProcessor();
	} while( status );
/*
#define LOAD_REG_FROM_MEM(reg, var) \
({	register unsigned fake_register_variable asm(#reg); \
	asm volatile ("ld %1,%0" : "=r" (fake_register_variable) : "m" (var) ); })
*/
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->start[0].queue[0])));
	asm("L2225:");
	asm("cmpqb	$0, 0(_spinlock_addr)");
	asm("bne 	L2225");
	asm("sbitib	$0, 0(_spinlock_addr)");
	asm("bfs 	L2225"); 
	
#endif
} ;
	}

	if (gl->start[0].count[0] == 0) {
		{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_BarExitLast;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->start;	/* data_id */


	} else
		trace_overflow_error();
}  
		{
#ifdef SPARC
	(gl->start[0].lock) = 0;
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->start[0].lock)));
	asm("movqb	$0, 0(_spinlock_addr)");
#endif
} ;
	} else {
		(gl->start[0].count[0])--;
		{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_BarExit;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->start;	/* data_id */


	} else
		trace_overflow_error();
}  
		{
#ifdef SPARC
	(gl->start[0].queue[0]) = 0;
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->start[0].queue[0])));
	asm("movqb	$0, 0(_spinlock_addr)");
#endif
} ;
	}
 
} ;

	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcEnter;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 23;	/* data_id */


	} else
		trace_overflow_error();
}  
    	INTERF(FORCES,&gl->VIR);
	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcExit;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 23;	/* data_id */


	} else
		trace_overflow_error();
}  

	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcEnter;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 18;	/* data_id */


	} else
		trace_overflow_error();
}  
   	CORREC(PCC,NORD1);
	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcExit;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 18;	/* data_id */


	} else
		trace_overflow_error();
}  

	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcEnter;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 19;	/* data_id */


	} else
		trace_overflow_error();
}  
    	BNDRY();
	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcExit;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 19;	/* data_id */


	} else
		trace_overflow_error();
}  

	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcEnter;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 20;	/* data_id */


	} else
		trace_overflow_error();
}  
    	KINETI(NMOL,gl->SUM,HMAS,OMAS);
	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcExit;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 20;	/* data_id */


	} else
		trace_overflow_error();
}  

    	TKIN=TKIN+gl->SUM[0]+gl->SUM[1]+gl->SUM[2];
        TVIR=TVIR-gl->VIR;

        /*  check if potential energy is to be computed, and if
            printing and/or saving is to be done, this time step.
            Note that potential energy is computed once every NPRINT
            time-steps */

        if (((i % NPRINT) == 0) || ( (NSAVE > 0) && ((I % NSAVE) == 0))){

                /*  call potential energy computing routine */
	    {
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcEnter;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 21;	/* data_id */


	} else
		trace_overflow_error();
}  
	    POTENG(&gl->POTA,&gl->POTR,&gl->POTRF);
	    {
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_ProcExit;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) 21;	/* data_id */


	} else
		trace_overflow_error();
}  

	    {
	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_BarEnter;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->start;	/* data_id */


	} else
		trace_overflow_error();
}  

	

	{
#ifdef SPARC
	unsigned status;
	do {
		register unsigned regvar;
		asm volatile ("ldstub %1,%0" : "=r" (regvar) : "m" ((gl->start[0].lock)) ); 
		status = regvar;
		if ( status ) thisProcess()->relinquishProcessor();
	} while( status );
/*
#define LOAD_REG_FROM_MEM(reg, var) \
({	register unsigned fake_register_variable asm(#reg); \
	asm volatile ("ld %1,%0" : "=r" (fake_register_variable) : "m" (var) ); })
*/
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->start[0].lock)));
	asm("L2226:");
	asm("cmpqb	$0, 0(_spinlock_addr)");
	asm("bne 	L2226");
	asm("sbitib	$0, 0(_spinlock_addr)");
	asm("bfs 	L2226"); 
	
#endif
} ;

	if (gl->start[0].count[0] < (NumProcs - 1)) {
		gl->start[0].count[0]++;
		{
#ifdef SPARC
	(gl->start[0].lock) = 0;
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->start[0].lock)));
	asm("movqb	$0, 0(_spinlock_addr)");
#endif
} ;
		{
#ifdef SPARC
	unsigned status;
	do {
		register unsigned regvar;
		asm volatile ("ldstub %1,%0" : "=r" (regvar) : "m" ((gl->start[0].queue[0])) ); 
		status = regvar;
		if ( status ) thisProcess()->relinquishProcessor();
	} while( status );
/*
#define LOAD_REG_FROM_MEM(reg, var) \
({	register unsigned fake_register_variable asm(#reg); \
	asm volatile ("ld %1,%0" : "=r" (fake_register_variable) : "m" (var) ); })
*/
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->start[0].queue[0])));
	asm("L2227:");
	asm("cmpqb	$0, 0(_spinlock_addr)");
	asm("bne 	L2227");
	asm("sbitib	$0, 0(_spinlock_addr)");
	asm("bfs 	L2227"); 
	
#endif
} ;
	}

	if (gl->start[0].count[0] == 0) {
		{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_BarExitLast;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->start;	/* data_id */


	} else
		trace_overflow_error();
}  
		{
#ifdef SPARC
	(gl->start[0].lock) = 0;
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->start[0].lock)));
	asm("movqb	$0, 0(_spinlock_addr)");
#endif
} ;
	} else {
		(gl->start[0].count[0])--;
		{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_BarExit;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->start;	/* data_id */


	} else
		trace_overflow_error();
}  
		{
#ifdef SPARC
	(gl->start[0].queue[0]) = 0;
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->start[0].queue[0])));
	asm("movqb	$0, 0(_spinlock_addr)");
#endif
} ;
	}
 
} ;

                /* modify computed sums */
	    POTA=gl->POTA*FPOT;
	    POTR=gl->POTR*FPOT;
	    POTRF=gl->POTRF*FPOT;

                /* compute some values to print */
	    XVIR=TVIR*FPOT*0.50/TTMV;
	    AVGT=TKIN*FKIN*TEMP*2.00/(3.00*TTMV);
	    TEN=(gl->SUM[0]+gl->SUM[1]+gl->SUM[2])*FKIN;
	    XTT=POTA+POTR+POTRF+TEN;
	    if ((i % NPRINT) == 0 && ProcID == 0) {
	  	fprintf(six,"     %5d %14.5lf %12.5lf %12.5lf  \
                    %12.5lf\n %16.3lf %16.5lf %16.5lf\n",
                    i,TEN,POTA,POTR,POTRF,XTT,AVGT,XVIR);
	    }
        }

            /* wait for everyone to finish time-step */
	{
	{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_BarEnter;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->start;	/* data_id */


	} else
		trace_overflow_error();
}  

	

	{
#ifdef SPARC
	unsigned status;
	do {
		register unsigned regvar;
		asm volatile ("ldstub %1,%0" : "=r" (regvar) : "m" ((gl->start[0].lock)) ); 
		status = regvar;
		if ( status ) thisProcess()->relinquishProcessor();
	} while( status );
/*
#define LOAD_REG_FROM_MEM(reg, var) \
({	register unsigned fake_register_variable asm(#reg); \
	asm volatile ("ld %1,%0" : "=r" (fake_register_variable) : "m" (var) ); })
*/
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->start[0].lock)));
	asm("L2228:");
	asm("cmpqb	$0, 0(_spinlock_addr)");
	asm("bne 	L2228");
	asm("sbitib	$0, 0(_spinlock_addr)");
	asm("bfs 	L2228"); 
	
#endif
} ;

	if (gl->start[0].count[0] < (NumProcs - 1)) {
		gl->start[0].count[0]++;
		{
#ifdef SPARC
	(gl->start[0].lock) = 0;
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->start[0].lock)));
	asm("movqb	$0, 0(_spinlock_addr)");
#endif
} ;
		{
#ifdef SPARC
	unsigned status;
	do {
		register unsigned regvar;
		asm volatile ("ldstub %1,%0" : "=r" (regvar) : "m" ((gl->start[0].queue[0])) ); 
		status = regvar;
		if ( status ) thisProcess()->relinquishProcessor();
	} while( status );
/*
#define LOAD_REG_FROM_MEM(reg, var) \
({	register unsigned fake_register_variable asm(#reg); \
	asm volatile ("ld %1,%0" : "=r" (fake_register_variable) : "m" (var) ); })
*/
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->start[0].queue[0])));
	asm("L2229:");
	asm("cmpqb	$0, 0(_spinlock_addr)");
	asm("bne 	L2229");
	asm("sbitib	$0, 0(_spinlock_addr)");
	asm("bfs 	L2229"); 
	
#endif
} ;
	}

	if (gl->start[0].count[0] == 0) {
		{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_BarExitLast;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->start;	/* data_id */


	} else
		trace_overflow_error();
}  
		{
#ifdef SPARC
	(gl->start[0].lock) = 0;
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->start[0].lock)));
	asm("movqb	$0, 0(_spinlock_addr)");
#endif
} ;
	} else {
		(gl->start[0].count[0])--;
		{
	if (trace_loc < trace_buff_end) {	/* Test for overflow */
		*trace_loc++ = (unsigned) PTT_BarExit;	/* type */
		*trace_loc = choices_clock->time();		/* time */
		trace_loc++;
		*trace_loc++ = thread_pid;	/* pid */


		*trace_loc++ = (unsigned) &gl->start;	/* data_id */


	} else
		trace_overflow_error();
}  
		{
#ifdef SPARC
	(gl->start[0].queue[0]) = 0;
#endif
#ifdef MULTIMAX
	spinlock_addr = (&((gl->start[0].queue[0])));
	asm("movqb	$0, 0(_spinlock_addr)");
#endif
} ;
	}
 
} ;

    } /* for i */
    return(XTT);
} /* end of subroutine MDMAIN */
