#ifdef TEST
#define UNIX
#include "Support.h"
#endif
extern "C" {
int oc_cksum(char * buffer, unsigned count, unsigned startval);
}

#ifdef sparc
int oc_cksum(char * buffer, unsigned count, unsigned startval)
{
	long val = startval;

	int offset = ((unsigned) buffer & 0x3);
	if ( offset != 0 ) {
		/* not aligned case, first try to align */
		int len = 4 - offset;
		if ( count < len )
			len = count;
		unsigned mask = ( 0xffffffff << ( (4 - len) << 3 ) ) 
				>> (offset << 3);
		unsigned i = (*(unsigned long *)(buffer - offset)) & mask;
		asm volatile( "addcc %0,%1,%0" : "=r" (val) : "r" (i) );
		asm volatile( "addxcc %0,0,%0" : "=r" (val) );
		buffer += len;
		count -= len;	
	}

	/* already 4 byte aligned, going for 32 byte aligned */

	while ( ((unsigned) buffer & 0x1f) && count >= 4 ) {
		unsigned i = *(unsigned long *)buffer;
		asm volatile( "addcc %0,%1,%0" : "=r" (val) : "r" (i) );
		asm volatile( "addxcc %0,0,%0" : "=r" (val) );
		buffer += 4;
		count -= 4;	
	}

	/* 32 byte aligned make best use of cache line */

	while ( count >= 32 ) {
		union ll {
			long long i;
			struct {
				unsigned long high;
				unsigned long low;
			} l;
		};
		ll x1; 
		ll x2;
		asm volatile( "ldd [%1],%0" : "=r" (x1.i) : "r" (buffer) );
		asm volatile( "ldd [%1+8],%0" : "=r" (x2.i) : "r" (buffer) );
		asm volatile( "addcc %0,%1,%0" : "=r" (val) : "r" (x1.l.high) );
		asm volatile( "addxcc %0,%1,%0" : "=r" (val) : "r" (x1.l.low) );
		asm volatile( "ldd [%1+16],%0" : "=r" (x1.i) : "r" (buffer) );
		asm volatile( "addxcc %0,%1,%0" : "=r" (val) : "r" (x2.l.high) );
		asm volatile( "addxcc %0,%1,%0" : "=r" (val) : "r" (x2.l.low) );
		asm volatile( "ldd [%1+24],%0" : "=r" (x2.i) : "r" (buffer) );
		asm volatile( "addxcc %0,%1,%0" : "=r" (val) : "r" (x1.l.high) );
		asm volatile( "addxcc %0,%1,%0" : "=r" (val) : "r" (x1.l.low) );
		asm volatile( "addxcc %0,%1,%0" : "=r" (val) : "r" (x2.l.high) );
		asm volatile( "addxcc %0,%1,%0" : "=r" (val) : "r" (x2.l.low) );
		asm volatile( "addxcc %0,0,%0" : "=r" (val) );
		buffer += 32;
		count -= 32;	
	}

	/* do remainder */

	while (count >= 4) {
		unsigned i = *(unsigned long *) buffer;
		asm volatile( "addcc %0,%1,%0" : "=r" (val) : "r" (i) );
		asm volatile( "addxcc %0,0,%0" : "=r" (val) );
		buffer += 4;
		count -= 4;
	}

	/* less than 4 bytes left */

	if (count > 0) { 
		unsigned mask = 0xffffffff << ( ( 4 - count ) << 3 );
		unsigned i = (*(unsigned long *) buffer) & mask;
		asm volatile( "addcc %0,%1,%0" : "=r" (val) : "r" (i) );
		asm volatile( "addxcc %0,0,%0" : "=r" (val) );
	}
		
	/* convert 32 bit check sum to 16 bits */

	unsigned t = val << 16;
	asm volatile( "addcc %0,%1,%0" : "=r" (val) : "r" (t) );
	asm volatile( "srl %0,16,%0" : "=r" (val) );
	asm volatile( "addxcc %1,0,%0" : "=r" (t) : "r" (val) );
	return t;
}

#endif

#ifdef TEST

int old_oc_cksum(char * buffer, unsigned count, unsigned startval)
{
	startval = (startval >> 16) + (startval & 0x0000ffff);

	if (((unsigned) buffer & 0x1) == 1) {
		startval += (unsigned short)(unsigned char)(*buffer);
		buffer++;
		count--;
	}

	unsigned short * s = (unsigned short *) buffer;
	while (count >= 2) {
		startval += *s;
		s++;
		count -= 2;
	}

	if (count) {
		unsigned char * b = (unsigned char *) s;
		startval += ((unsigned short)(*b) << 8); // this is for sparc
	}
		
	startval = (startval >> 16) + (startval & 0x0000ffff);
	startval = (startval >> 16) + (startval & 0x0000ffff);
	return startval;
}

#define BUFSIZE 8192
char buf[BUFSIZE];

typedef int (*cksum)(char * buffer, unsigned count, unsigned startval);

int compare(cksum p)
{
	for (int i = 16; i <= BUFSIZE; i += i ) {
		unsigned start = systemtime();
		for( int r = 0; r < 100; r++ ) {
			(*p)(buf, i, 0);
		}
		unsigned stop = systemtime();
		printf("size %d time %d us\n", i, (stop-start)/100);
	}
	return 0;
}

int main( int argc, char *argv[] )
{
	for (int i = 0; i < BUFSIZE; i++) {
		buf[i] = i;
	}
	compare(&oc_cksum);
	compare(&old_oc_cksum);
	
/*
	int off = 0;
	int count = BUFSIZE;
	if ( argc >= 2 ) off = atoi(argv[1]);
	if ( argc >= 3 ) count = atoi(argv[2]);
	printf("offset %d, count %d\n", off, count);
	unsigned newsum = oc_cksum( buf + off, count, 0 );
	unsigned oldsum = old_oc_cksum( buf + off, count, 0 );
	if ( newsum != oldsum ) {
		printf("bug new %x, old %x\n", i, newsum, oldsum);
		return 1;
	}
 */
	printf("starting pass 1\n");
	for (i = 0; i < BUFSIZE; i++) {
		unsigned newsum = oc_cksum( buf, BUFSIZE - i, 0 );
		unsigned oldsum = old_oc_cksum( buf, BUFSIZE - i, 0 );
		if ( newsum != oldsum ) {
			printf("pass 1: bug i = %d, new %x, old %x\n", 
				i, newsum, oldsum);
			return 1;
		}
	}
	printf("starting pass 2\n");
	for (i = 0; i < BUFSIZE; i++) {
		unsigned newsum = oc_cksum( buf + i, BUFSIZE - i, 0 );
		unsigned oldsum = old_oc_cksum( buf + i, BUFSIZE - i, 0 );
		if ( newsum != oldsum ) {
			printf("pass 2: bug i = %d, new %x, old %x\n", 
				i, newsum, oldsum);
			return 1;
		}
	}
	for (i = 0; i < BUFSIZE; i++) {
		buf[i] = BUFSIZE - i;
	}
	printf("starting pass 3\n");
	for (i = 0; i < BUFSIZE; i++) {
		unsigned newsum = oc_cksum( buf, BUFSIZE - i, 0 );
		unsigned oldsum = old_oc_cksum( buf, BUFSIZE - i, 0 );
		if ( newsum != oldsum ) {
			printf("pass 3: bug i = %d, new %x, old %x\n", 
				i, newsum, oldsum);
			return 1;
		}
	}
	printf("starting pass 4\n");
	for (i = 0; i < BUFSIZE; i++) {
		unsigned newsum = oc_cksum( buf + i, BUFSIZE - i, 0 );
		unsigned oldsum = old_oc_cksum( buf + i, BUFSIZE - i, 0 );
		if ( newsum != oldsum ) {
			printf("pass 4: bug i = %d, new %x, old %x\n", 
				i, newsum, oldsum);
			return 1;
		}
	}
	printf("passed all tests\n");
	return 0;
}

#endif
