#ifndef SocketKernel_h
#define SocketKernel_h

#include "SocketDef.h"

class Process;

extern "C" {

void process_intr(Process * p);
/*
 *
 ************** MAP METHODS TO BSD SYSCALLS **************
 *
 */

int sys_socket(struct proc * p, struct sys_socket_args * uap, int * retval);
struct sys_socket_args {
	int	domain;
	int	type;
	int	protocol;
};
int sys_bind(struct proc * p, struct sys_bind_args * uap, int * retval);
struct sys_bind_args {
	int	s;
	caddr_t	name;
	int	namelen;
};
int sys_listen(struct proc * p, struct sys_listen_args * uap, int * retval);
struct sys_listen_args {
	int	s;
	int	backlog;
};
int sys_accept(struct proc * p, struct sys_accept_args * uap, int * retval);
struct sys_accept_args {
	int	s;
	caddr_t	name;
	int	*anamelen;
};
int sys_connect(struct proc * p, struct sys_connect_args * uap, int * retval);
struct sys_connect_args {
	int	s;
	caddr_t	name;
	int	namelen;
};
int sys_socketpair(struct proc * p, struct sys_socketpair_args * uap, int * retval);
struct sys_socketpair_args {
	int	domain;
	int	type;
	int	protocol;
	int	*rsv;
};
int sys_sendto(struct proc * p, struct sys_sendto_args * uap, int * retval);
struct sys_sendto_args {
	int	s;
	caddr_t	buf;
	int	len;
	int	flags;
	caddr_t	to;
	int	tolen;
};
int sys_sendmsg(struct proc * p, struct sys_sendmsg_args * uap, int * retval);
struct sys_sendmsg_args {
	int	s;
	caddr_t	msg;
	int	flags;
};
int sys_recvfrom(struct proc * p, struct sys_recvfrom_args * uap, int * retval);
struct sys_recvfrom_args {
	int	s;
	caddr_t	buf;
	int	len;
	int	flags;
	caddr_t	from;
	int	*fromlenaddr;
};
int sys_recvmsg(struct proc * p, struct sys_recvmsg_args * uap, int * retval);
struct sys_recvmsg_args {
	int	s;
	struct	msghdr *msg;
	int	flags;
};
int sys_shutdown(struct proc * p, struct sys_shutdown_args * uap, int * retval);
struct sys_shutdown_args {
	int	s;
	int	how;
};
int sys_setsockopt(struct proc * p, struct sys_setsockopt_args * uap, int * retval);
struct sys_setsockopt_args {
	int	s;
	int	level;
	int	name;
	caddr_t	val;
	int	valsize;
};
int sys_getsockopt(struct proc * p, struct sys_getsockopt_args * uap, int * retval);
struct sys_getsockopt_args {
	int	s;
	int	level;
	int	name;
	caddr_t	val;
	int	*avalsize;
};
int sys_getsockname(struct proc * p, struct sys_getsockname_args * uap, int * retval);
struct sys_getsockname_args {
	int	fdes;
	caddr_t	asa;
	int	*alen;
};
int sys_getpeername(struct proc * p, struct sys_getpeername_args * uap, int * retval);
struct sys_getpeername_args {
	int	fdes;
	caddr_t	asa;
	int	*alen;
};
int sys_close(struct proc * p, struct sys_close_args * uap, int * retval);
struct sys_close_args {
	int     fdes;
};

}

/*
 *
 ******************** Other useful routines ***************
 *
 */

u_long inet_addr(char *cp);
u_long inet_myaddr();

inline
int socket(int domain, int type, int protocol, int & errno)
{
	int error;
	int retval;
	struct sys_socket_args args = { domain, type, protocol };
	error = sys_socket(&proc0, &args, &retval);
	if ( error ) {
		errno = error;
		return -1;
	} else {
		return retval;
	}
}

inline
int socket(int domain, int type, int protocol)
{
	return socket(domain, type, protocol, errno);
}

inline
int bind(int s, struct sockaddr * name, int namelen, int & errno)
{
	int error;
	int retval;
	struct sys_bind_args args = { s, (caddr_t) name, namelen };
	error = sys_bind(&proc0, &args, &retval);
	if ( error ) {
		errno = error;
		return -1;
	} else {
		return 0;
	}
}

inline
int bind(int s, struct sockaddr * name, int namelen)
{
	return bind(s, name, namelen, errno);
}

inline
int listen(int s, int backlog, int & errno)
{
	int error;
	int retval;
	struct sys_listen_args args = { s, backlog };
	error = sys_listen(&proc0, &args, &retval);
	if ( error ) {
		errno = error;
		return -1;
	} else {
		return 0;
	}
}
inline
int listen(int s, int backlog)
{
	return listen(s, backlog, errno);
}

inline
int accept(int s, struct sockaddr * name, int *anamelen, int & errno)
{
	int error;
	int retval;
	struct sys_accept_args args = { s, (caddr_t) name, anamelen };
	error = sys_accept(&proc0, &args, &retval);
	if ( error ) {
		errno = error;
		return -1;
	} else {
		return retval;
	}
}

inline
int accept(int s, struct sockaddr * name, int *anamelen)
{
	return accept(s, name, anamelen, errno);
}

inline
int connect(int s, struct sockaddr * name, int namelen, int & errno)
{
	int error;
	int retval;
	struct sys_connect_args args = { s, (caddr_t) name, namelen };
	error = sys_connect(&proc0, &args, &retval);
	if ( error ) {
		errno = error;
		return -1;
	} else {
		return 0;
	}
}

inline
int connect(int s, struct sockaddr * name, int namelen)
{
	return connect(s, name, namelen, errno);
}

inline
int socketpair(int domain, int type, int protocol, int *rsv, int & errno)
{
	int error;
	int retval;
	struct sys_socketpair_args args = { domain, type, protocol, rsv };
	error = sys_socketpair(&proc0, &args, &retval);
	if ( error ) {
		errno = error;
		return -1;
	} else {
		return 0;
	}
}

inline
int socketpair(int domain, int type, int protocol, int *rsv)
{
	return socketpair(domain, type, protocol, rsv, errno);
}

inline
int sendto(int s, caddr_t buf, int len, int flags, 
	   struct sockaddr * to, int tolen, int & errno)
{
	int error;
	int retval;
	struct sys_sendto_args args = 
		{ s, buf, len, flags, (caddr_t) to, tolen };
	error = sys_sendto(&proc0, &args, &retval);
	if ( error ) {
		errno = error;
		return -1;
	} else {
		return retval;
	}
}

inline
int sendto(int s, caddr_t buf, int len, int flags, 
	   struct sockaddr * to, int tolen)
{
	return sendto(s, buf, len, flags, to, tolen, errno);
}

inline
int send(int s, caddr_t buf, int len, int flags, int & errno)
{
	return sendto(s, buf, len, flags, 0, 0, errno);
}

inline
int send(int s, caddr_t buf, int len, int flags)
{
        return sendto(s, buf, len, flags, 0, 0, errno);
}

inline
int sendmsg(int s, struct msghdr * msg, int flags, int & errno)
{
	int error;
	int retval;
	struct sys_sendmsg_args args = { s, (caddr_t) msg, flags };
	error = sys_sendmsg(&proc0, &args, &retval);
	if ( error ) {
		errno = error;
		return -1;
	} else {
		return retval;
	}
}

inline
int sendmsg(int s, struct msghdr * msg, int flags)
{
	return sendmsg(s, msg, flags, errno);
}

inline
int recvfrom(int s, caddr_t buf, int len, int flags, 
	     struct sockaddr * from, int * fromlenaddr, int & errno)
{
	int error;
	int retval;
	struct sys_recvfrom_args args = 
		{ s, buf, len, flags, (caddr_t) from, fromlenaddr };
	error = sys_recvfrom(&proc0, &args, &retval);
	if ( error ) {
		errno = error;
		return -1;
	} else {
		return retval;
	}
}

inline
int recvfrom(int s, caddr_t buf, int len, int flags, 
	     struct sockaddr * from, int * fromlenaddr)
{
	return recvfrom(s, buf, len, flags, from, fromlenaddr, errno);
}

inline
int recv(int s, caddr_t buf, int len, int flags, int & errno)
{
	return recvfrom(s, buf, len, flags, 0, 0, errno);
}

inline
int recv(int s, caddr_t buf, int len, int flags)
{
	return recvfrom(s, buf, len, flags, 0, 0, errno);
}

inline
int recvmsg(int s, struct msghdr * msg, int flags, int & errno)
{
	int error;
	int retval;
	struct sys_recvmsg_args args = { s, msg, flags };
	error = sys_recvmsg(&proc0, &args, &retval);
	if ( error ) {
		errno = error;
		return -1;
	} else {
		return retval;
	}
}

inline
int recvmsg(int s, struct msghdr *msg, int flags)
{
	return recvmsg(s, msg, flags, errno);
}

inline
int shutdown(int s, int how, int & errno)
{
	int error;
	int retval;
	struct sys_shutdown_args args = { s, how };
	error = sys_shutdown(&proc0, &args, &retval);
	if ( error ) {
		errno = error;
		return -1;
	} else {
		return 0;
	}
}

inline
int shutdown(int s, int how)
{
	return shutdown(s, how, errno);
}

inline
int setsockopt(int s, int level, int name, caddr_t val, int valsize, int & errno)
{
	int error;
	int retval;
	struct sys_setsockopt_args args = { s, level, name, val, valsize };
	error = sys_setsockopt(&proc0, &args, &retval);
	if ( error ) {
		errno = error;
		return -1;
	} else {
		return 0;
	}
}

inline
int setsockopt(int s, int level, int name, caddr_t val, int valsize)
{
	return setsockopt(s, level, name, val, valsize, errno);
}

inline
int getsockopt(int s, int level, int name, caddr_t val, int * valsize, int & errno)
{
	int error;
	int retval;
	struct sys_getsockopt_args args = { s, level, name, val, valsize };
	error = sys_getsockopt(&proc0, &args, &retval);
	if ( error ) {
		errno = error;
		return -1;
	} else {
		return 0;
	}
}

inline
int getsockopt(int s, int level, int name, caddr_t val, int * valsize)
{
	return getsockopt(s, level, name, val, valsize, errno);
}

inline
int getsockname(int fdes, struct sockaddr * asa, int * alen, int & errno)
{
	int error;
	int retval;
	struct sys_getsockname_args args = { fdes, (caddr_t) asa, alen };
	error = sys_getsockname(&proc0, &args, &retval);
	if ( error ) {
		errno = error;
		return -1;
	} else {
		return 0;
	}
}

inline
int getsockname(int fdes, struct sockaddr * asa, int * alen)
{
	return getsockname(fdes, asa, alen, errno);
}

inline
int getpeername(int fdes, struct sockaddr * asa, int * alen, int & errno)
{
	int error;
	int retval;
	struct sys_getpeername_args args = { fdes, (caddr_t) asa, alen };
	error = sys_getpeername(&proc0, &args, &retval);
	if ( error ) {
		errno = error;
		return -1;
	} else {
		return 0;
	}
}

inline
int getpeername(int fdes, struct sockaddr * asa, int * alen)
{
	return getpeername(fdes, asa, alen, errno);
}

inline
int close(int fdes, int & errno)
{
	int error;
	int retval;
	struct sys_close_args args = { fdes };
	error = sys_close(&proc0, &args, &retval);
	if ( error ) {
		errno = error;
		return -1;
	} else {
		return 0;
	}
}

inline
int close(int fdes)
{
	return close(fdes, errno);
}

#endif SocketKernel_h
