#include "Support.h"

#define SETSOCKOPT

int debugTcpEcho = 0;
#define DebugTcp if (debugTcpEcho) Console()

#ifdef UNIX
int TcpEchoServer( int argc, char * argv[] );
int TcpEchoClient( int argc, char * argv[] );
char * asHex(unsigned long i)
{
	char * buf = new char[32];
	sprintf(buf, "%x", i);
	return buf;
}

int main( int argc, char * argv[] )
{
	argc--;
	argv++;
	if ( argc == 3 ) {
		TcpEchoServer( argc, argv );
	} else if ( argc == 5 ) {
		TcpEchoClient( argc, argv );
	} else {
		Console() << "arguments: server_port count size\n" << eor;
		Console() << "arguments: server_internet_address server_port client_port count size\n" << eor;
	}
	return 0;
}

u_long inet_myaddr()
{
	static u_long addr = 0;
	struct hostent * host;
	char hostname[128];
	if (addr) return addr;
	gethostname(hostname, sizeof(hostname));
	host = gethostbyname(hostname);
	if (host == 0) {
		Console() << "gethostbyname error\n" << eor;
		exit(1);
	}
	printf("%x\n", *((unsigned *) host->h_addr));
	addr = *((unsigned long *) host->h_addr);
	return addr;
}
#endif

int sendn( int s, caddr_t buf, int len )
{
	int remaining = len;
	while ( remaining > 0 ) {
		int result = send( s, buf, remaining, 0 );
		if ( result < 0 ) {
			Console() << "send failed errno " << errno 
				  << "\n" << eor;
			return -1;
		}
		if ( result == 0 ) {
			Console() << "send result == 0\n" << eor;
			break;
		}
		remaining -= result;
		buf += result;
	}
	return len - remaining;
}

int recvn( int s, caddr_t buf, int len )
{
	int remaining = len;
	while ( remaining > 0 ) {
		int result = recv( s, buf, remaining, 0 );
		if ( result < 0 ) {
			Console() << "recv failed errno " << errno 
				  << "\n" << eor;
			return -1;
		}
		if ( result == 0 ) {
			Console() << "recv result == 0\n" << eor;
			break;
		}
		remaining -= result;
		buf += result;
	}
	return len - remaining;
}


int TcpEchoServer( int argc, char * argv[] )
{
	if ( argc != 3 ) {
		Console() << "arguments: server_port count size\n" << eor;
		return 0;
	}
	u_long hostaddr = inet_myaddr();
	u_short port = atoi( argv[0] );	
	int count = atoi( argv[1] );
	int size = atoi( argv[2] );
	int reserve = (size * 15)/10;
	Console() << "TcpEchoServer: hostaddr " << asHex( hostaddr )
		  << " port " << port << " count " << count 
		  << " size " << size << "\n" << eor;
	struct sockaddr_in mysockaddr;
	bzero(&mysockaddr, sizeof(mysockaddr));
	mysockaddr.sin_family = AF_INET;
	mysockaddr.sin_port = port;
	mysockaddr.sin_addr.s_addr = hostaddr;
	struct sockaddr * sa = (struct sockaddr *) &mysockaddr;
	int salen = sizeof(mysockaddr);
	
	/* the following should be in subroutine but who cares for now */

	int i;
	int s = -1;
	int result;
	int alen;
	int orgsize;
	char * buf = new char[size];
	char addrbuf[32];
	struct sockaddr_in clientaddr;
	int clientaddrlen = sizeof( clientaddr );
	int listensoc = socket( AF_INET, SOCK_STREAM, IPPROTO_TCP );
	if ( listensoc < 0 ) {
		Console() << "socket failed errno " << errno << "\n" << eor;
		goto done;
	}
	result = bind( listensoc, sa, salen );
	if ( result < 0 ) {
		Console() << "bind failed errno " << errno << "\n" << eor;
		goto done;
	}
	result = listen( listensoc, SOMAXCONN );
	if ( result < 0 ) {
		Console() << "listen failed errno " << errno << "\n" << eor;
		goto done;
	}
	s = accept( listensoc, (struct sockaddr *) &clientaddr, &clientaddrlen );
	if ( s < 0 ) {
		Console() << "accept failed errno " << errno << "\n" << eor;
		goto done;
	}
	Console() << "accepted connection from hostaddr " 
		  << asHex(clientaddr.sin_addr.s_addr) 
		  << " port " << clientaddr.sin_port << "\n" << eor;
	alen = sizeof(orgsize);
	result = getsockopt( s, SOL_SOCKET, SO_SNDBUF, 
			     (caddr_t) &orgsize, &alen );
	if ( result < 0 ) {
		Console() << "getsockopt send buf failed\n" << eor;
		goto done;
	}
	Console() << "getsockopt send buf orig reserved " << orgsize 
		  << "\n" << eor;
#ifdef SETSOCKOPT
	if ( orgsize < reserve ) {
		result = setsockopt( s, SOL_SOCKET, SO_SNDBUF, 
				     (caddr_t) &reserve, sizeof(reserve) );
		if ( result < 0 ) {
			Console() << "setsockopt send buf failed\n" << eor;
			goto done;
		}
	}
	alen = sizeof(orgsize);
	result = getsockopt( s, SOL_SOCKET, SO_SNDBUF, 
			     (caddr_t) &orgsize, &alen );
	if ( result < 0 ) {
		Console() << "getsockopt send buf failed\n" << eor;
		goto done;
	}
	Console() << "getsockopt send buf new reserved " << orgsize 
		  << "\n" << eor;
#endif
	alen = sizeof(orgsize);
	result = getsockopt( s, SOL_SOCKET, SO_RCVBUF, 
			     (caddr_t) &orgsize, &alen );
	if ( result < 0 ) {
		Console() << "getsockopt recv buf failed\n" << eor;
		goto done;
	}
	Console() << "getsockopt recv buf orig reserved " << orgsize 
		  << "\n" << eor;
#ifdef SETSOCKOPT
	if ( orgsize < reserve ) {
		result = setsockopt( s, SOL_SOCKET, SO_RCVBUF, 
				     (caddr_t) &reserve, sizeof(reserve) );
		if ( result < 0 ) {
			Console() << "setsockopt recv buf failed\n" << eor;
			goto done;
		}
	}
	alen = sizeof(orgsize);
	result = getsockopt( s, SOL_SOCKET, SO_RCVBUF, 
			     (caddr_t) &orgsize, &alen );
	if ( result < 0 ) {
		Console() << "getsockopt recv buf failed\n" << eor;
		goto done;
	}
	Console() << "getsockopt recv buf new reserved " << orgsize 
		  << "\n" << eor;
#endif
	for ( i = 0; i < count; i ++ ) {
		int addrlen = sizeof( addrbuf );
		result = recvn( s, buf, size );
		if ( result < 0 ) {
			Console() << "recvn failed errno " << errno << "\n" 
				  << eor;
			goto done;
		}
		if ( result < size ) {
			Console() << "readn returns " << result 
				  << " which is < size " << size 
				  << "\n" << eor;
		}
		DebugTcp << "count " << i+1 << "\n" << eor;
		int sendresult = sendn( s, buf, result );
		if ( sendresult < 0 ) {
			Console() << "sendn failed errno " << errno << "\n" 
				  << eor;
			goto done;
		}
		if ( sendresult < result ) {
			Console() << "sendn returns " << sendresult 
				  << " which is < recv result " << result
				  << "\n" << eor;
		}
	}
	
done:
	if ( s >= 0 ) {
		int result = close( s );
		if ( result < 0 ) {
			Console() << "close failed errno " << errno << "\n" 
				  << eor;
		}
	}
	if ( listensoc >= 0 ) {
		int result = close( listensoc );
		if ( result < 0 ) {
			Console() << "close failed errno " << errno << "\n" 
				  << eor;
		}
	}
	delete buf;
	return 0;
}

int TcpEchoClient( int argc, char * argv[] )
{
	if ( argc != 5 ) {
		Console() << "arguments: server_internet_address server_port client_port count size\n" << eor;
		return 0;
	}
	u_long serverhostaddr = inet_addr( argv[0] );
	u_short serverport = atoi( argv[1] );	
	u_long clienthostaddr = inet_myaddr();
	u_short clientport = atoi( argv[2] );	
	int count = atoi( argv[3] );
	int size = atoi( argv[4] );
	Console() << "TcpEchoClient:" 
		  << " server hostaddr " << asHex( serverhostaddr )
		  << " server port " << serverport 
		  << " client hostaddr " << asHex( clienthostaddr )
		  << " client port " << clientport
		  << " count " << count 
		  << " size " << size << "\n" << eor;
	struct sockaddr_in serversockaddr;
	bzero(&serversockaddr, sizeof(serversockaddr));
	serversockaddr.sin_family = AF_INET;
	serversockaddr.sin_port = serverport;
	serversockaddr.sin_addr.s_addr = serverhostaddr;
	struct sockaddr_in clientsockaddr;
	bzero(&clientsockaddr, sizeof(clientsockaddr));
	clientsockaddr.sin_family = AF_INET;
	clientsockaddr.sin_port = clientport;
	clientsockaddr.sin_addr.s_addr = clienthostaddr;
	struct sockaddr * serversa = (struct sockaddr *) &serversockaddr;
	int serversalen = sizeof(serversockaddr);
	struct sockaddr * clientsa = (struct sockaddr *) &clientsockaddr;
	int clientsalen = sizeof(clientsockaddr);
	
	/* the following should be in subroutine but who cares for now */

	int i;
	int result;
	int alen;
	int orgsize;
	unsigned sum = 0;
	unsigned volume = 0;
	char * buf = new char[size];
	struct sockaddr_in recvsa;
	int s = socket( AF_INET, SOCK_STREAM, IPPROTO_TCP );
	if ( s < 0 ) {
		Console() << "socket failed errno " << errno << "\n";
		goto done;
	}
	alen = sizeof(orgsize);
	result = getsockopt( s, SOL_SOCKET, SO_SNDBUF, 
			     (caddr_t) &orgsize, &alen );
	if ( result < 0 ) {
		Console() << "getsockopt send buf failed\n" << eor;
		goto done;
	}
	Console() << "getsockopt send buf orig reserved " << orgsize 
		  << "\n" << eor;
#ifdef SETSOCKOPT
	if ( orgsize < size ) {
		result = setsockopt( s, SOL_SOCKET, SO_SNDBUF, 
				     (caddr_t) &size, sizeof(size) );
		if ( result < 0 ) {
			Console() << "setsockopt send buf failed\n" << eor;
			goto done;
		}
	}
	alen = sizeof(orgsize);
	result = getsockopt( s, SOL_SOCKET, SO_SNDBUF, 
			     (caddr_t) &orgsize, &alen );
	if ( result < 0 ) {
		Console() << "getsockopt send buf failed\n" << eor;
		goto done;
	}
	Console() << "getsockopt send buf new reserved " << orgsize 
		  << "\n" << eor;
#endif
	alen = sizeof(orgsize);
	result = getsockopt( s, SOL_SOCKET, SO_RCVBUF, 
			     (caddr_t) &orgsize, &alen );
	if ( result < 0 ) {
		Console() << "getsockopt recv buf failed\n" << eor;
		goto done;
	}
	Console() << "getsockopt recv buf orig reserved " << orgsize 
		  << "\n" << eor;
#ifdef SETSOCKOPT
	if ( orgsize < size ) {
		result = setsockopt( s, SOL_SOCKET, SO_RCVBUF, 
				     (caddr_t) &size, sizeof(size) );
		if ( result < 0 ) {
			Console() << "setsockopt recv buf failed\n" << eor;
			goto done;
		}
	}
	alen = sizeof(orgsize);
	result = getsockopt( s, SOL_SOCKET, SO_RCVBUF, 
			     (caddr_t) &orgsize, &alen );
	if ( result < 0 ) {
		Console() << "getsockopt recv buf failed\n" << eor;
		goto done;
	}
	Console() << "getsockopt recv buf new reserved " << orgsize 
		  << "\n" << eor;
#endif
	result = bind( s, clientsa, clientsalen );
	if ( result < 0 ) {
		Console() << "bind failed errno " << errno << "\n";
		goto done;
	}
	result = connect( s, serversa, serversalen );
	if ( result < 0 ) {
		Console() << "connect failed errno " << errno << "\n";
		goto done;
	}
	for ( i = 0; i < count; i ++ ) {
		unsigned start = systemtime();
		int sendresult = sendn( s, buf, size );
		if ( sendresult < 0 ) {
			Console() << "sendn failed errno " << errno << "\n" 
				  << eor;
			goto done;
		}
		if ( sendresult < size ) {
			Console() << "sendn returns " << sendresult 
				  << " which is < size " << size
				  << "\n" << eor;
		}
		volume += sendresult;
		DebugTcp << "sent " << sendresult 
			 << " waiting for reply\n" << eor;
		result = recvn( s, buf, sendresult );
		if ( result < 0 ) {
			Console() << "recvn failed errno " << errno << "\n" 
				  << eor;
			goto done;
		}
		if ( result < sendresult ) {
			Console() << "recvn returns " << result
				  << " which is < sendn result " 
				  << sendresult << "\n" << eor;
		}
		volume += result;
		DebugTcp << "count " << i+1 << "\n" << eor;
		unsigned stop = systemtime();
		Console() << i+1 << ": round trip (us) " << stop - start
			  << "\n" << eor;
		sum += stop - start;
	}
	
	Console() << "Volume " << volume << " bytes in " << sum << " us\n"
                  << eor;

done:
	if ( s >= 0 ) {
		int result = close(s);
		if ( result < 0 ) {
			Console() << "close failed errno " << errno << "\n" 
				  << eor;
		}
	}
	delete buf;
	return 0;
}
