#include "Support.h"

int debugUdpEcho = 0;
#define DebugUdp if (debugUdpEcho) Console()

#define CHECKCONTENTS
#define ROUNDTRIP
#define SETSOCKOPT

#ifdef UNIX
int UdpEchoServer( int argc, char * argv[] );
int UdpEchoClient( int argc, char * argv[] );
char * asHex(unsigned long i)
{
	char * buf = new char[32];
	sprintf(buf, "%x", i);
	return buf;
}

int main( int argc, char * argv[] )
{
	argc--;
	argv++;
	if ( argc == 3 ) {
		UdpEchoServer( argc, argv );
	} else if ( argc == 5 ) {
		UdpEchoClient( argc, argv );
	} else {
		Console() << "arguments: server_port count size\n" << eor;
		Console() << "arguments: server_internet_address server_port client_port count size\n" << eor;
	}
	return 0;
}

u_long inet_myaddr()
{
	static u_long addr = 0;
	struct hostent * host;
	char hostname[128];
	if (addr) return addr;
	gethostname(hostname, sizeof(hostname));
	host = gethostbyname(hostname);
	if (host == 0) {
		Console() << "gethostbyname error\n" << eor;
		exit(1);
	}
	printf("%x\n", *((unsigned *) host->h_addr));
	addr = *((unsigned long *) host->h_addr);
	return addr;
}
#endif

int UdpEchoServer( int argc, char * argv[] )
{
	if ( argc != 3 ) {
		Console() << "arguments: server_port count size\n" << eor;
		return 0;
	}
	u_long hostaddr = inet_myaddr();
	u_short port = atoi( argv[0] );	
	int count = atoi( argv[1] );
	int size = atoi( argv[2] );
	int reserve = size + 8000;
	Console() << "UdpEchoServer: hostaddr " << asHex( hostaddr )
		  << " port " << port << " count " << count 
		  << " size " << size << "\n" << eor;
	struct sockaddr_in mysockaddr;
	bzero(&mysockaddr, sizeof(mysockaddr));
	mysockaddr.sin_family = AF_INET;
	mysockaddr.sin_port = port;
	mysockaddr.sin_addr.s_addr = hostaddr;
	struct sockaddr * sa = (struct sockaddr *) &mysockaddr;
	int salen = sizeof(mysockaddr);
	
	/* the following should be in subroutine but who cares for now */

	int i;
	int result;
	int orgsize;
	int alen;
	char * buf = new char[size];
	struct sockaddr_in addrbuf;
	int s = socket( AF_INET, SOCK_DGRAM, IPPROTO_UDP );
	if ( s < 0 ) {
		Console() << "socket failed errno " << errno << "\n" << eor;
		goto done;
	}
	alen = sizeof(orgsize);
	result = getsockopt( s, SOL_SOCKET, SO_SNDBUF, 
			     (caddr_t) &orgsize, &alen );
	if ( result < 0 ) {
		Console() << "getsockopt send buf failed\n" << eor;
		goto done;
	}
	Console() << "getsockopt send buf orig reserved " << orgsize 
		  << "\n" << eor;
#ifdef SETSOCKOPT
	if ( orgsize < reserve ) {
		result = setsockopt( s, SOL_SOCKET, SO_SNDBUF, 
				     (caddr_t) &reserve, sizeof(reserve) );
		if ( result < 0 ) {
			Console() << "setsockopt send buf failed\n" << eor;
			goto done;
		}
	}
	alen = sizeof(orgsize);
	result = getsockopt( s, SOL_SOCKET, SO_SNDBUF, 
			     (caddr_t) &orgsize, &alen );
	if ( result < 0 ) {
		Console() << "getsockopt send buf failed\n" << eor;
		goto done;
	}
	Console() << "getsockopt send buf new reserved " << orgsize 
		  << "\n" << eor;
#endif
	alen = sizeof(orgsize);
	result = getsockopt( s, SOL_SOCKET, SO_RCVBUF, 
			     (caddr_t) &orgsize, &alen );
	if ( result < 0 ) {
		Console() << "getsockopt recv buf failed\n" << eor;
		goto done;
	}
	Console() << "getsockopt recv buf orig reserved " << orgsize 
		  << "\n" << eor;
#ifdef SETSOCKOPT
	if ( orgsize < reserve ) {
		result = setsockopt( s, SOL_SOCKET, SO_RCVBUF, 
				     (caddr_t) &reserve, sizeof(reserve) );
		if ( result < 0 ) {
			Console() << "setsockopt recv buf failed\n" << eor;
			goto done;
		}
	}
	alen = sizeof(orgsize);
	result = getsockopt( s, SOL_SOCKET, SO_RCVBUF, 
			     (caddr_t) &orgsize, &alen );
	if ( result < 0 ) {
		Console() << "getsockopt recv buf failed\n" << eor;
		goto done;
	}
	Console() << "getsockopt recv buf new reserved " << orgsize 
		  << "\n" << eor;
#endif
	result = bind( s, sa, salen );
	if ( result < 0 ) {
		Console() << "bind failed errno " << errno << "\n" << eor;
		goto done;
	}
	for ( i = 0; i < count; i ++ ) {
		int addrlen = sizeof( addrbuf );
		result = recvfrom( s, buf, size, 0,
				   (struct sockaddr *) &addrbuf, &addrlen );
		if ( result < 0 ) {
			Console() << "recvfrom failed errno " << errno << "\n" 
				  << eor;
			goto done;
		}
		DebugUdp << "count " << i+1 << "\n" 
		         << "recv packet of length " << result << "\n" << eor;
#ifdef ROUNDTRIP
		int sendresult = sendto( s, buf, result, 0, 
				 	 (struct sockaddr *) &addrbuf, 
					 addrlen );
		if ( sendresult < 0 ) {
			Console() << "sendto failed errno " << errno << "\n" 
				  << eor;
			goto done;
		}
		if ( result != sendresult ) {
			Console() << "sendto want to send " << result
				  << " sent " << sendresult << "\n" << eor;
		}
#endif
	}
	
done:
	if ( s >= 0 ) {
		int result = close(s);
		if ( result < 0 ) {
			Console() << "close failed errno " << errno << "\n" 
				  << eor;
		}
	}
	delete buf;
	return 0;
}

int UdpEchoClient( int argc, char * argv[] )
{
	if ( argc != 5 ) {
		Console() << "arguments: server_internet_address server_port client_port count size\n" << eor;
		return 0;
	}
	int alen;
	int orgsize;
	u_long serverhostaddr = inet_addr( argv[0] );
	u_short serverport = atoi( argv[1] );	
	u_long clienthostaddr = inet_myaddr();
	u_short clientport = atoi( argv[2] );	
	int count = atoi( argv[3] );
	int size = atoi( argv[4] );
	int reserve = size + 8000;
	Console() << "UdpEchoClient: server hostaddr " 
		  << asHex( serverhostaddr )
		  << " server port " << serverport 
		  << " client hostaddr " << asHex( clienthostaddr )
		  << " client port " << clientport
		  << " count " << count 
		  << " size " << size << "\n" << eor;
	struct sockaddr_in serversockaddr;
	bzero(&serversockaddr, sizeof(serversockaddr));
	serversockaddr.sin_family = AF_INET;
	serversockaddr.sin_port = serverport;
	serversockaddr.sin_addr.s_addr = serverhostaddr;
	struct sockaddr_in clientsockaddr;
	bzero(&clientsockaddr, sizeof(clientsockaddr));
	clientsockaddr.sin_family = AF_INET;
	clientsockaddr.sin_port = clientport;
	clientsockaddr.sin_addr.s_addr = clienthostaddr;
	struct sockaddr * serversa = (struct sockaddr *) &serversockaddr;
	int serversalen = sizeof(serversockaddr);
	struct sockaddr * clientsa = (struct sockaddr *) &clientsockaddr;
	int clientsalen = sizeof(clientsockaddr);
	
	/* the following should be in subroutine but who cares for now */

	int i;
	int result;
	char * buf = new char[size];
	struct sockaddr_in recvsa;
	unsigned sum = 0;
	unsigned volume = 0;
#ifdef CHECKCONTENTS
	for ( i = 0; i < size; i++ ) 
		buf[i] = ( i & 0x7f );
#endif 
	int s = socket( AF_INET, SOCK_DGRAM, IPPROTO_UDP );
	if ( s < 0 ) {
		Console() << "socket failed errno " << errno << "\n" << eor;
		goto done;
	}
	alen = sizeof(orgsize);
	result = getsockopt( s, SOL_SOCKET, SO_SNDBUF, 
			     (caddr_t) &orgsize, &alen );
	if ( result < 0 ) {
		Console() << "getsockopt send buf failed\n" << eor;
		goto done;
	}
	Console() << "getsockopt send buf orig reserved " << orgsize 
		  << "\n" << eor;
#ifdef SETSOCKOPT
	if ( orgsize < reserve ) {
		result = setsockopt( s, SOL_SOCKET, SO_SNDBUF, 
				     (caddr_t) &reserve, sizeof(reserve) );
		if ( result < 0 ) {
			Console() << "setsockopt send buf failed\n" << eor;
			goto done;
		}
	}
	alen = sizeof(orgsize);
	result = getsockopt( s, SOL_SOCKET, SO_SNDBUF, 
			     (caddr_t) &orgsize, &alen );
	if ( result < 0 ) {
		Console() << "getsockopt send buf failed\n" << eor;
		goto done;
	}
	Console() << "getsockopt send buf new reserved " << orgsize 
		  << "\n" << eor;
#endif
	alen = sizeof(orgsize);
	result = getsockopt( s, SOL_SOCKET, SO_RCVBUF, 
			     (caddr_t) &orgsize, &alen );
	if ( result < 0 ) {
		Console() << "getsockopt recv buf failed\n" << eor;
		goto done;
	}
	Console() << "getsockopt recv buf orig reserved " << orgsize 
		  << "\n" << eor;
#ifdef SETSOCKOPT
	if ( orgsize < reserve ) {
		result = setsockopt( s, SOL_SOCKET, SO_RCVBUF, 
				     (caddr_t) &reserve, sizeof(reserve) );
		if ( result < 0 ) {
			Console() << "setsockopt recv buf failed\n" << eor;
			goto done;
		}
	}
	alen = sizeof(orgsize);
	result = getsockopt( s, SOL_SOCKET, SO_RCVBUF, 
			     (caddr_t) &orgsize, &alen );
	if ( result < 0 ) {
		Console() << "getsockopt recv buf failed\n" << eor;
		goto done;
	}
	Console() << "getsockopt recv buf new reserved " << orgsize 
		  << "\n" << eor;
#endif
	result = bind( s, clientsa, clientsalen );
	if ( result < 0 ) {
		Console() << "bind failed errno " << errno << "\n" << eor;
		goto done;
	}
	for ( i = 0; i < count; i ++ ) {
		unsigned start = systemtime();
		int sendresult = sendto( s, buf, size, 0, 
					 serversa, serversalen );
		if ( sendresult < 0 ) {
			Console() << "sendto failed errno " << errno << "\n" 
				  << eor;
			goto done;
		}
		if ( sendresult < size ) {
			Console() << "sendto want to send " << size
		        	  << " sent " << sendresult << "\n" << eor;
		}
		volume += sendresult;
		DebugUdp << "sendto want to send " << size
		      	 << " sent " << sendresult << "\n" << eor;
#ifdef ROUNDTRIP
		int recvsalen = sizeof( recvsa );
		result = recvfrom( s, buf, size, 0, 
				   (struct sockaddr *) &recvsa, &recvsalen );
		if ( result < 0 ) {
			Console() << "recvfrom failed errno " << errno << "\n" 
				  << eor;
			goto done;
		}
		if ( result < size ) {
			Console() << "recvfrom want to recv " << size
		        	  << " recvd " << result << "\n" << eor;
		}
		volume += result;
		DebugUdp << "count " << i+1 << "\n" 
		         << "recv packet of length " << result << "\n" << eor;
		unsigned stop = systemtime();
		Console() << i+1 << ": round trip (us) " << stop - start
			  << "\n" << eor;
#ifdef CHECKCONTENTS
		for ( int k = 0; k < result; k++ ) 
			if ( buf[k] != ( k & 0x7f ) ) {
				Console() << "contents error at " << k 
					  << "\n" << eor;
			}
#endif 
		sum += (stop - start);
		if ( recvsa.sin_port != serverport ||
		     recvsa.sin_addr.s_addr != serverhostaddr ) {
			Console() << "recv packet did not come from expected server\n" << eor;
		}
#endif
	}
	
	Console() << "Volume " << volume << " bytes in " << sum << " us\n" 
		  << eor;

done:
	if ( s >= 0 ) {
		int result = close(s);
		if ( result < 0 ) {
			Console() << "close failed errno " << errno << "\n" 
				  << eor;
		}
	}
	delete buf;
	return 0;
}
