/*
#ifdef __GNUC__
#pragma implementation
#endif

 * Copyright (c) 1982, 1986, 1990 Regents of the University of California.
 * All rights reserved.  The Berkeley software License Agreement
 * specifies the terms and conditions for redistribution.
 *
 *	@(#)kern_synch.c	7.11 (Berkeley) 4/3/90
 */

#include "Process.h"
#include "ThisProcess.h"

extern "C" {

#include "machine/pte.h"
/* #include "machine/psl.h" */
#include "machine/mtpr.h"

#include "param.h"
#include "systm.h"
#include "user.h"
#include "proc.h"
#include "vm.h"
#include "kernel.h"
#include "buf.h"

typedef void (*PFVDP)(struct dummy_proc *);
int tsleep(caddr_t chan, int pri, char *wmesg, int timo);
void endtsleep(struct dummy_proc *p);
void unsleep(struct dummy_proc *p);
void sleep(caddr_t chan, int pri);
void setrun(struct dummy_proc *p);
void wakeup(caddr_t chan);
void untimeout(PFVDP fun, caddr_t arg);
void timeout(PFVDP fun, caddr_t arg, int t);
void process_intr(Process * p);

int splhigh();
int spl0();
int splx(int);
void panic(char *string);
void printf(...);

}

struct dummy_proc {
	struct dummy_proc * p_link;
	struct dummy_proc * p_rlink;
	char *p_wmesg;
	caddr_t p_wchan;
	int p_flag;
	Process * p_process;
	char p_slptime;
	char p_stat;
	int p_sig;
};

#define SQSIZE 0100	/* Must be power of 2 */
#define HASH(x)	(( (int) x >> 5) & (SQSIZE-1))
struct slpque_t {
	struct dummy_proc *sq_head;
	struct dummy_proc **sq_tailp;
} slpque[SQSIZE];

/*
 * General sleep call.
 * Suspends current process until a wakeup is made on chan.
 * The process will then be made runnable with priority pri.
 * Sleeps at most timo/hz seconds (0 means no timeout).
 * If pri includes PCATCH flag, signals are checked
 * before and after sleeping, else signals are not checked.
 * Returns 0 if awakened, EWOULDBLOCK if the timeout expires.
 * If PCATCH is set and a signal needs to be delivered,
 * ERESTART is returned if the current system call should be restarted
 * if possible, and EINTR is returned if the system call should
 * be interrupted by the signal (return EINTR).
 */

int tsleep(caddr_t chan, int pri, char *wmesg, int timo)
{
	register struct dummy_proc *rp;
	register struct slpque_t *qp;
	register int s;
	int sig;
	int pcatch = pri & PCATCH;
	extern int cold;
	Process * process;

	process = thisProcess();
	rp = (dummy_proc *) process->blockedInfo();
	s = splhigh();
	rp->p_rlink = 0;
#ifdef DIAGNOSTIC
	if (chan == 0 || rp->p_stat != SRUN || rp->p_rlink)
		panic("tsleep");
#endif
	rp->p_process = process;
	rp->p_wchan = chan;
	rp->p_wmesg = wmesg;
	rp->p_slptime = 0;
	rp->p_sig = 0;
#ifdef ORIGINAL
	rp->p_pri = pri & PRIMASK;
#endif
	rp->p_flag = 0;
	qp = &slpque[HASH(chan)];
	if (qp->sq_head == 0)
		qp->sq_head = rp;
	else
		*qp->sq_tailp = rp;
	*(qp->sq_tailp = &rp->p_link) = 0;
	/*
	 * If we stop in CURSIG/issig(), wakeup may already
	 * have happened when we return.
	 * rp->p_wchan will then be 0.
	 */
	if (pcatch) {
#ifdef ORIGINAL
		// can't handle signals yet

		if (sig = CURSIG(rp)) {
			if (rp->p_wchan)
				unsleep(rp);
			rp->p_stat = SRUN;
			splx(s);
			if (u.u_sigintr & sigmask(sig))
				return (EINTR);
			return (ERESTART);
		}
#endif
		if (rp->p_wchan == 0) {
			splx(s);
			return (0);
		}
		rp->p_flag |= SSINTR;
	}
	rp->p_stat = SSLEEP;
	if (timo)
		timeout((PFVDP) &endtsleep, (caddr_t)rp, timo);
#ifdef ORIGINAL
	(void) spl0();
	u.u_ru.ru_nvcsw++;
	swtch();
	curpri = rp->p_usrpri;
#endif 
	process->block((Object *) slpque);
	splx(s);
	rp->p_flag &= ~SSINTR;
	if (rp->p_flag & STIMO) {
		rp->p_flag &= ~STIMO;
		return (EWOULDBLOCK);
	}
	if (timo)
		untimeout((PFVDP) &endtsleep, (caddr_t)rp);
	if (pcatch && rp->p_sig ) {
		return (EINTR);
	}
#ifdef ORIGINAL
	if (pcatch && (sig = CURSIG(rp))) {
		if (u.u_sigintr & sigmask(sig))
			return (EINTR);
		return (ERESTART);
	}
#endif
	return (0);
}

/*
 * Implement timeout for tsleep.
 * If process hasn't been awakened (wchan non-zero),
 * set timeout flag and undo the sleep.  If proc
 * is stopped, just unsleep so it will remain stopped.
 */

class ProcessCheat : public Process {
public:
	ProcessState state() { return _state; }
};

void endtsleep(struct dummy_proc *p)
{
	int s = splhigh();

	if (p->p_wchan) {
		if (p->p_stat == SSLEEP)
			setrun(p);
		else
			unsleep(p);
	}
	splx(s);
}

/*
 * Short-term, non-interruptable sleep.
 */
void sleep(caddr_t chan,int pri)
{
	register struct dummy_proc *rp;
	register struct slpque_t *qp;
	register s;
	extern int cold;

#ifdef DIAGNOSTIC
	if (pri > PZERO) {
		printf("sleep called with pri %d > PZERO, wchan: %x\n",
			pri, chan);
		panic("old sleep");
	}
#endif

#ifdef ORIGINAL
	rp = u.u_procp;
	s = splhigh();
	if (cold || panicstr) {
		/*
		 * After a panic, or during autoconfiguration,
		 * just give interrupts a chance, then just return;
		 * don't run any other procs or panic below,
		 * in case this is the idle process and already asleep.
		 */
		(void) spl0();
		splx(s);
		return;
	}
#endif

	Process * process = thisProcess();
	rp = (dummy_proc *) process->blockedInfo();
	s = splhigh();

	rp->p_rlink = 0;
#ifdef DIAGNOSTIC
	if (chan==0 || rp->p_stat != SRUN || rp->p_rlink)
		panic("sleep");
#endif
	rp->p_wchan = chan;
	rp->p_wmesg = NULL;
	rp->p_slptime = 0;
#ifdef ORIGINAL
	rp->p_pri = pri;
#endif
	qp = &slpque[HASH(chan)];
	if (qp->sq_head == 0)
		qp->sq_head = rp;
	else
		*qp->sq_tailp = rp;
	*(qp->sq_tailp = &rp->p_link) = 0;
	rp->p_stat = SSLEEP;
#ifdef ORIGINAL
	(void) spl0();
	u.u_ru.ru_nvcsw++;
	swtch();
	curpri = rp->p_usrpri;
#endif
	process->block((Object *) slpque);
	splx(s);
}

/*
 * Remove a process from its wait queue
 */
void unsleep(struct dummy_proc *p)
{
	register struct slpque_t *qp;
	register struct dummy_proc **hp;
	int s;

	s = splhigh();
	if (p->p_wchan) {
		hp = &(qp = &slpque[HASH(p->p_wchan)])->sq_head;
		while (*hp != p)
			hp = &(*hp)->p_link;
		*hp = p->p_link;
		if (qp->sq_tailp == &p->p_link)
			qp->sq_tailp = hp;
		p->p_wchan = 0;
	}
	splx(s);
}

/*
 * Wake up all processes sleeping on chan.
 */
void wakeup(caddr_t chan)
{
	register struct slpque_t *qp;
	register struct dummy_proc *p, **q;
	int s;

	s = splhigh();
	qp = &slpque[HASH(chan)];
restart:
	for (q = &qp->sq_head; p = *q; ) {
#ifdef DIAGNOSTIC
		if (p->p_rlink || p->p_stat != SSLEEP && p->p_stat != SSTOP)
			panic("wakeup");
#endif
		if (p->p_wchan==chan) {
			p->p_wchan = 0;
			*q = p->p_link;
			if (qp->sq_tailp == &p->p_link)
				qp->sq_tailp = q;
			if (p->p_stat == SSLEEP) {
				/* OPTIMIZED INLINE EXPANSION OF setrun(p) */
#ifdef ORIGINAL
				if (p->p_slptime > 1)
					updatepri(p);
				p->p_stat = SRUN;
				if (p->p_flag & SLOAD)
					setrq(p);
				/*
				 * Since curpri is a usrpri,
				 * p->p_pri is always better than curpri.
				 */
				runrun++;
				aston();
				if ((p->p_flag&SLOAD) == 0) {
					if (runout != 0) {
						runout = 0;
						wakeup((caddr_t)&runout);
					}
					wantin++;
				}
#endif
				p->p_stat = SRUN;
				Assert(((ProcessCheat *)(p->p_process))->state()
				       == ProcessBlocked);
				p->p_process->ready();
				/* END INLINE EXPANSION */
				goto restart;
			}
		} else
			q = &p->p_link;
	}
	splx(s);
}

#ifdef ORIGINAL
/*
 * Initialize the (doubly-linked) run queues
 * to be empty.
 */
rqinit()
{
	register int i;

	for (i = 0; i < NQS; i++)
		qs[i].ph_link = qs[i].ph_rlink = (struct proc *)&qs[i];
}
#endif

/*
 * Set the process running;
 * arrange for it to be swapped in if necessary.
 */
void setrun(struct dummy_proc *p)
{
	register int s;

	s = splhigh();
	switch (p->p_stat) {

	case 0:
	case SWAIT:
	case SRUN:
	case SZOMB:
	default:
		panic("setrun");

	case SSTOP:
	case SSLEEP:
		unsleep(p);		/* e.g. when sending signals */
		break;

	case SIDL:
		break;
	}
	p->p_stat = SRUN;
#ifdef ORIGINAL
	if (p->p_flag & SLOAD)
		setrq(p);
	splx(s);
	if (p->p_slptime > 1)
		updatepri(p);
	if (p->p_pri < curpri) {
		runrun++;
		aston();
	}
	if ((p->p_flag&SLOAD) == 0) {
		if (runout != 0) {
			runout = 0;
			wakeup((caddr_t)&runout);
		}
		wantin++;
	}
#endif
	p->p_stat = SRUN;
	Assert(((ProcessCheat *)(p->p_process))->state() == ProcessBlocked);
	p->p_process->ready();
}

#ifdef ORIGINAL

/*
 * Set user priority.
 * The rescheduling flag (runrun)
 * is set if the priority is better
 * than the currently running process.
 */
setpri(pp)
	register struct proc *pp;
{
	register int p;

	p = (pp->p_cpu & 0377)/4;
	p += PUSER + 2 * pp->p_nice;
	if (pp->p_rssize > pp->p_maxrss && freemem < desfree)
		p += 2*4;	/* effectively, nice(4) */
	if (p > 127)
		p = 127;
	if (p < curpri) {
		runrun++;
		aston();
	}
	pp->p_usrpri = p;
	return (p);
}
#endif

void 
process_intr(Process * p)
{
	int s = splhigh();
	struct dummy_proc * rp = (dummy_proc *) p->blockedInfo();
	rp->p_sig = 1;
	setrun( rp );
	splx(s);
}

