#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@@cs.uiuc.edu)
 * Contributing Author: Vincent F. Russo (russo@@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ClassInstaller.cc -
 *
 *	$Header: /projects/choices/old/cvsroot/choices/Common/ClassInstaller.cc,v 1.1 2004/06/03 01:58:31 jcarlyle Exp $
 */
#include "ClassInstaller.h"
#include "Assert.h"
#include "Byte.h"
#include "Class.h"
#include "PersistentClass.h"
#include "ClassConstructor.h"
#include "NameServer.h"


#ifdef __GNUC__
#pragma implementation
#endif

static struct instEntry {
		ClassStar * theClass;
		ClassStar * superClass;
		int size;
		char * name;
		POFPO ctor;
		iFPO supportsObject;
		char  tsort_field;
		int  flag;
	} _info[ MaxNumberOfClasses ];

static int       _nextIndex = 0;
static int       _alreadyCalled = 0;

extern ClassStar NullClass;

ClassInstaller::ClassInstaller( ClassStar * theClass, ClassStar * superclass, int size, char * name, int flag, POFPO ctor, iFPO supportsObject)
{

    Assert (this != 0);

    if( _alreadyCalled ){
	if( superclass != 0 ){
           if ( flag ) // define PersistentClass
		*theClass = new PersistentClass( *superclass, size, name, 
		    ctor == 0 ? 0 : new ClassConstructor( ctor ) );
           else
		*theClass = new Class( *superclass, size, name, 
		    ctor == 0 ? 0 : new ClassConstructor( ctor ) );
	}
	else{
           if ( flag ) // define PersistentClass
		*theClass = new PersistentClass( 0, size, name, 
		    ctor == 0 ? 0 : new ClassConstructor( ctor ) );
           else
		*theClass = new Class( 0, size, name, 
		    ctor == 0 ? 0 : new ClassConstructor( ctor ) );
	}
        StandardNameServer->bind(  name, *theClass );
    }
    _info[ _nextIndex ].theClass = theClass;
    _info[ _nextIndex ].superClass = superclass;
    _info[ _nextIndex ].size = size;
    _info[ _nextIndex ].name = (char *)name;
    _info[ _nextIndex ].ctor = ctor;	
    _info[_nextIndex ].supportsObject = supportsObject;
    _info[_nextIndex ].flag = flag;
    if( superclass != &NullClass ) _info[ _nextIndex ].tsort_field = 1;
    else _info[ _nextIndex ].tsort_field = 0;	
    
    _nextIndex++;
    Assert( _nextIndex < MaxNumberOfClasses );	
}

void 
ClassInstaller::swap( int i, int j )
{

    int size = sizeof( struct instEntry );
    char * buffer = new char[ size ];

    ByteCopy( (void *)(&_info[i]), (void *)buffer,      size );
    ByteCopy( (void *)(&_info[j]), (void *)(&_info[i]), size );
    ByteCopy( (void *)buffer,      (void *)(&_info[j]), size );
}

void
ClassInstaller::tsort()
{

    /* The algorithm: (single inheritance)
     * The table _info is partitioned into two subtables by a variable
     * secondHalf which always points to the first element of the second
     * table. initially  secondHalf = 0. The algprithm is:
     */
    int secondHalf = 0;

    while( 1 ){
    /*  1] Look the second half of the table for an element with tsort_field=0
     *              (terminate if not finding one) */
	int i = secondHalf;
	while( i < _nextIndex ){
	    if( _info[i].tsort_field == 0 ){
		/* 2] If you find one swap with the secondHalf and increase
		 *    the upper table by 1. */
		swap( secondHalf, i );
		secondHalf++;
		/* 3] Find all the subclasse of the class and make their
	 	 *    field 0. Then start again. */
		for( int j = secondHalf; j<_nextIndex; j++)
		    if( _info[j].superClass == _info[secondHalf-1].theClass)
			 _info[j].tsort_field = 0;
		break;
	    }
	    i++;
	}	
	if( i >= _nextIndex ) break;
    }
		
}

void
ClassInstaller::install()
{

    _alreadyCalled = 1;

    /*** 1 Do a topological sort first so that superclasses be built
     * befor subclasses
     */
    tsort();

    /*** 2 Now build the classes */
    for( int i = 0; i < _nextIndex ; i++ ) {
	// Console() << "installing " << _info[i].name << "\n" << eor;
	if( _info[i].superClass != 0 )
          if (_info[i].flag ) //define PersistentClass
	    *(_info[i].theClass) = new PersistentClass( *(_info[i].superClass), 
		    _info[i].size, _info[i].name, _info[i].ctor == 0 ? 0 :
				new ClassConstructor( _info[i].ctor ) );
          else 
{
	    *(_info[i].theClass) = new Class( *(_info[i].superClass), 
		    _info[i].size, _info[i].name, _info[i].ctor == 0 ? 0 :
				new ClassConstructor( _info[i].ctor ) );
}
	else
          if ( _info[i].flag ) //define PersistentClass
	    *(_info[i].theClass) = new PersistentClass( 0,
		    _info[i].size, _info[i].name, _info[i].ctor == 0 ? 0 :
				new ClassConstructor( _info[i].ctor ) );
          else
	    *(_info[i].theClass) = new Class( 0,
		    _info[i].size, _info[i].name, _info[i].ctor == 0 ? 0 :
				new ClassConstructor( _info[i].ctor ) );
	}

}

void
ClassInstaller::uninstall()
{

    Assert( _alreadyCalled == 1 );

    for( int i = 0; i < _nextIndex ; i++ ) {
	  *(_info[i].theClass) = (Class *)0;
    }

}
