#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Bjorn A. Helgaas (helgaas@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * InputStream.h: definition of an output stream of characters.
 */

#ifdef DJGPP
#pragma implementation "InputStream.h"
#endif
#include "InputStream.h"
#include "ClassStar.h"
#include "Ctype.h"
#include "Convert.h"
#include "Assert.h"
#if defined (linux)
#include <stdlib.h>  // To get definition of NULL
#endif


#ifdef __GNUC__
#pragma implementation
#endif

InputStream::InputStream()
{
    	readBufferIndex = 0;
    	readBuffer [0] = 0;
    	readBackBufferIndex = 0;
}

// this can only accept pointers to args
// anything else is an error

int
InputStream::formattedRead( const char * format, const formatArgs & a1,
	const formatArgs & a2, const formatArgs & a3, const formatArgs & a4 )
{
    void const *args [4];
    args [0] = (&a1 == 0) ? 0 : a1._formatArgs.p;
    args [1] = (&a2 == 0) ? 0 : a2._formatArgs.p;
    args [2] = (&a3 == 0) ? 0 : a3._formatArgs.p;
    args [3] = (&a4 == 0) ? 0 : a4._formatArgs.p;
    return _doscan(format, (int **)args);
}

int
InputStream::multiFormattedRead (const char *format, int **args)
{
    return _doscan (format, args);
}

int
InputStream::readBackChar (char ch)
{
    return ungetc (ch);
}

char
InputStream::getc ()
{
    if (readBackBufferIndex > 0)
    	return readBackBuffer [readBackBufferIndex--];    
    else if (readBuffer [readBufferIndex] != 0)
    	return readBuffer [readBufferIndex++];
    else
    {
    	readBufferIndex = read (readBuffer, inputStreamBufferSize);
	Assert (readBufferIndex > 0);
	readBuffer [readBufferIndex] = 0;
	readBufferIndex = 1;
	return readBuffer [0];
    }
}

char
InputStream::ungetc (char c)
{
    readBackBufferIndex++;
		// Francis - What is the correct return value here?
    if (readBackBufferIndex >= inputStreamBufferSize) return 255;
    readBackBuffer [readBackBufferIndex] = c;
    return c;
}


/*
 * Copyright (c) 1988 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted provided
 * that: (1) source distributions retain this entire copyright notice and
 * comment, and (2) distributions including binaries display the following
 * acknowledgement:  ``This product includes software developed by the
 * University of California, Berkeley and its contributors'' in the
 * documentation or other materials provided with the distribution and in
 * all advertising materials mentioning features or use of this software.
 * Neither the name of the University nor the names of its contributors may
 * be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

//#define NULL 0
#define EOF -1

#define	SPC	01
#define	STP	02

#define	SHORT	0
#define	REGULAR	1
#define	LONG	2
#define	INT	0
#define	FLOAT	1

static char _sctab[256] = {
	0,0,0,0,0,0,0,0,
	0,SPC,SPC,0,0,0,0,0,
	0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,
	SPC,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,
};

int
InputStream::_doscan(const char *fmt, int **argp)
{
	register int ch;
	int nmatch, len, ch1;
	int **ptr, fileended, size;

	nmatch = 0;
	fileended = 0;
	for (;;) switch (ch = *fmt++) {
	case '\0': 
		return (nmatch);
	case '%': 
		if ((ch = *fmt++) == '%')
			goto def;
		ptr = 0;
		if (ch != '*')
			ptr = argp++;
		else
			ch = *fmt++;
		len = 0;
		size = REGULAR;
		while (isdigit(ch)) {
			len = len*10 + ch - '0';
			ch = *fmt++;
		}
		if (len == 0)
			len = 30000;
		if (ch=='l') {
			size = LONG;
			ch = *fmt++;
		} else if (ch=='h') {
			size = SHORT;
			ch = *fmt++;
		} else if (ch=='[')
			fmt = _getccl(fmt);
		if (isupper(ch)) {
			ch = tolower(ch);
			size = LONG;
		}
		if (ch == '\0')
			return(-1);
		if (_innum(ptr, ch, len, size, &fileended) && ptr)
			nmatch++;
		if (fileended)
			return(nmatch? nmatch: -1);
		break;

	case ' ':
	case '\n':
	case '\t': 
		while ((ch1 = getc())==' ' || ch1=='\t' || ch1=='\n')
			;
		if (ch1 != EOF)
			ungetc(ch1);
		break;

	default: 
	def:
		ch1 = getc();
		if (ch1 != ch) {
			if (ch1==EOF)
				return(-1);
			ungetc(ch1);
			return(nmatch);
		}
	}
}

#ifndef NULL
#define NULL 0
#define __INPUTSTREAM_NULL 1
#endif


int
InputStream::_innum(int **ptr, int type, int len, int size, int *eofptr)
{
	register char *np;
	char numbuf[64];
	register int c, base;
	int expseen, scale, negflg, c1, ndigit;
	long lcval;

	if (type=='c' || type=='s' || type=='[')
		return(_instr(ptr? *(char **)ptr: (char *)NULL, type, len, eofptr));
	lcval = 0;
	ndigit = 0;
	scale = INT;
	if (type=='e'||type=='f')
		scale = FLOAT;
	base = 10;
	if (type=='o')
		base = 8;
	else if (type=='x')
		base = 16;
	np = numbuf;
	expseen = 0;
	negflg = 0;
	while ((c = getc())==' ' || c=='\t' || c=='\n');
	if (c=='-') {
		negflg++;
		*np++ = c;
		c = getc();
		len--;
	} else if (c=='+') {
		len--;
		c = getc();
	}
	for ( ; --len>=0; *np++ = c, c = getc()) {
		if (isdigit(c)
		 || base==16 && ('a'<=c && c<='f' || 'A'<=c && c<='F')) {
			ndigit++;
			if (base==8)
				lcval <<=3;
			else if (base==10)
				lcval = ((lcval<<2) + lcval)<<1;
			else
				lcval <<= 4;
			c1 = c;
			if (isdigit(c))
				c -= '0';
			else if ('a'<=c && c<='f')
				c -= 'a'-10;
			else
				c -= 'A'-10;
			lcval += c;
			c = c1;
			continue;
		} else if (c=='.') {
			if (base!=10 || scale==INT)
				break;
			ndigit++;
			continue;
		} else if ((c=='e'||c=='E') && expseen==0) {
			if (base!=10 || scale==INT || ndigit==0)
				break;
			expseen++;
			*np++ = c;
			c = getc();
			if (c!='+'&&c!='-'&&('0'>c||c>'9'))
				break;
		} else
			break;
	}
	if (negflg)
		lcval = -lcval;
	if (c != EOF) {
		ungetc(c);
		*eofptr = 0;
	} else
		*eofptr = 1;
 	if (ptr==NULL || np==numbuf || (negflg && np==numbuf+1) )/* gene dykes*/
		return(0);
	*np++ = 0;
	switch((scale<<4) | size) {

	case (FLOAT<<4) | SHORT:
	case (FLOAT<<4) | REGULAR:
		**(float **)ptr = atof(numbuf);
		break;

	case (FLOAT<<4) | LONG:
		**(double **)ptr = atof(numbuf);
		break;

	case (INT<<4) | SHORT:
		**(short **)ptr = (short) lcval;
		break;

	case (INT<<4) | REGULAR:
		**(int **)ptr = (int) lcval;
		break;

	case (INT<<4) | LONG:
		**(long **)ptr = lcval;
		break;
	}
	return(1);
}

#ifdef __INPUTSTREAM_NULL
#undef NULL
#undef __INPUTSTREAM_NULL
#endif

int
InputStream::_instr(char *ptr, int type, int len, int *eofptr)
{
	register int ch;
	register char *optr;
	int ignstp;

	*eofptr = 0;
	optr = ptr;
	if (type=='c' && len==30000)
		len = 1;
	ignstp = 0;
	if (type=='s')
		ignstp = SPC;
	while ((ch = getc()) != EOF && _sctab[ch] & ignstp)
		;
	ignstp = SPC;
	if (type=='c')
		ignstp = 0;
	else if (type=='[')
		ignstp = STP;
	while (ch!=EOF && (_sctab[ch]&ignstp)==0) {
		if (ptr)
			*ptr++ = ch;
		if (--len <= 0)
			break;
		ch = getc();
	}
	if (ch != EOF) {
		if (len > 0)
			ungetc(ch);
		*eofptr = 0;
	} else
		*eofptr = 1;
	if (ptr && ptr!=optr) {
		if (type!='c')
			*ptr++ = '\0';
		return(1);
	}
	return(0);
}

const char *
InputStream::_getccl(const char *s)
{
	register unsigned c, t;

	t = 0;
	if (*s == '^') {
		t++;
		s++;
	}
	for (c = 0; c < (sizeof _sctab / sizeof _sctab[0]); c++)
		if (t)
			_sctab[c] &= ~STP;
		else
			_sctab[c] |= STP;
	if ((c = *s) == ']' || c == '-') {	/* first char is special */
		if (t)
			_sctab[c] |= STP;
		else
			_sctab[c] &= ~STP;
		s++;
	}
	while ((c = *s++) != ']') {
		if (c==0)
			return((char *)--s);
		else if (c == '-' && *s != ']' && s[-2] < *s) {
			for (c = s[-2] + 1; c < (unsigned)*s; c++)
				if (t)
					_sctab[c] |= STP;
				else
					_sctab[c] &= ~STP;
		} else if (t)
			_sctab[c] |= STP;
		else
			_sctab[c] &= ~STP;
	}
	return((const char *)s);
}

#include "ClassInstaller.h"
DEFINECLASS("InputStream",InputStream,ProxiableObject,0,0);
