#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801-2910
 *
 * Copyright (c) 1991 The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 *
 * Author: Amit Parghi (parghi@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

int
match( char * string, char * pattern )
{
    char   ok;
    char   negate;
    char   left;
    char   right;
    int    stackPtr = 0;
    int    stackSize = 0;
    for( char * p = pattern; *p; p++ ) if( *p == '*' ) stackSize++;

    char ** patternStack = new char * [stackSize];
    char ** stringStack  = new char * [stackSize];

    for(;;) {
        switch( *pattern ) {
    
        case '*':
            if( *++pattern == '\0' ) {
                delete patternStack;
                delete stringStack;
                return( 1 );
            }
            patternStack[stackPtr] = pattern;
            stringStack[stackPtr++] = string + 1;
            continue;
    
        case '[':
            ok = 0;
            left = '\0';
            if( *++pattern == '^' ) {
                negate = 1;
                ++pattern;
            }
            else negate = 0;
            while( ( *pattern != '\0' ) && ( *pattern != ']' ) ) {
                switch( *pattern ) {
                case '-':
                    if( left != '\0' ) {
                        right = *++pattern;
                        ok |= ( left <= *string ) && ( *string <= right );
                        left = '\0';
                        break;
                    }
                default:
                    if( *string == ( left = *pattern ) ) ok = 1;
                }
                ++pattern;
            }
            if( negate ) ok = !ok;
            if( ok ) {
                ++string;
                ++pattern;
                continue;
            }
            else goto BackTrack;
    
        case '\0':
            if( *string == '\0' ) {
                delete patternStack;
                delete stringStack;
                return( 1 );
            }
    
        default:
            if( *pattern != *string ) goto BackTrack;
    
        case '?':
            if( *string != '\0' ) {
                ++string;
                ++pattern;
                continue;
            }
        }
    
BackTrack:
        if( --stackPtr >= 0 ) {
            pattern = patternStack[stackPtr];
            string = stringStack[stackPtr];
            if( *string != '\0' ) {
                patternStack[stackPtr] = pattern;
                stringStack[stackPtr++] = string + 1;
                continue;
            }
        }
    
        delete patternStack;
        delete stringStack;
        return( 0 );
    }
}
