#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * Object.c -  object class to assist in debugging.
 */
#include "Assert.h"
#include "Raid.h"
#include "Object.h"
#include "OutputStream.h"
#include "Class.h"


#ifdef __GNUC__
#pragma implementation
#endif

/*
 * Set the CatchObject with a debugger and set a breakpoint at
 *   CaughtObject() to catch when a particular object is created or deleted.
 */
Object *CatchObject = 0;
void
CaughtObject() { }

Object::Object()
{
	_magicNumber = ObjectMagicNumber;
	_debugMask = 0;
	_debugMessages = 0;

	if ( this == CatchObject )
		CaughtObject();
}

Object::~Object()
{
	Raids( RaidConstructor ) << this << "::~Object()\n" << eor;

	if ( this == CatchObject )
		CaughtObject();

	if( ! alive() ) {
		Console() << this << "::~Object(): not alive()\n" << eor;
	}

	Assert( alive() );

	_magicNumber = 0;

}

int
Object::isProxy()
{
	return( 0 );
}

unsigned int
Object::identity()
{
	return( (unsigned int) this );
}

void
Object::writeName( OutputStream * os )
{
	Assert( os != 0 );

	char buf[64];
	if( classOf() != 0 ) *os << classOf()->name(64, buf);
	*os << "[" << asHex( (unsigned int)this ) << "]";
}

void
Object::inspect( OutputStream * os )
{
	writeName( os );
}

int
Object::initDebug()
{
	//_debugMask = 0;
	//_debugMessages = 0;
	if( classOf() != 0 ) {
		if( classOf()->initDebugObjects() > 0 ) {
			_debugMask = classOf()->initDebugMask();
			_debugMessages = classOf()->initDebugMessages();
		}
	}
	return( 0 ); //FIX this function should be a void
}

int
Object::isMemberOf( Class * aClass )
{
	Assert( classOf() != 0 );
	Assert( aClass != 0 );
	return( classOf()->identity() == aClass->identity() );
}

int
Object::isKindOf( Class * aClass )
{
	Assert( classOf() != 0 );
	Assert( aClass != 0 );
	return( classOf()->isASubclassOf( aClass ) );
}

int
Object::debugMask()
{
	return( _debugMask );
}

void
Object::setDebugMask( unsigned mask )
{
	Assert( mask <= MaxObjectDebugMask );
	_debugMask = mask;
}

void
Object::setDebugMessages( unsigned messages )
{
	Assert( messages <= MaxObjectDebugMessages );
	_debugMessages = messages;
}



#include "ClassInstaller.h"
ClassStar NullClass = 0;
DEFINECLASS("Object",Object,Null,0,0);
