#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Contributing Author: Bjorn A. Helgaas (helgaas@cs.uiuc.edu)
 * Contributing Author: Lee Lup Yuen (lup@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
#ifdef DJGPP
#pragma implementation "OutputStream.h"
#endif
#include "Assert.h"
#include "OutputStream.h"
#include "Process.h"
#include "Semaphore.h"
#include "String.h"
#include "Ctype.h"


#ifdef __GNUC__
#pragma implementation
#endif

//  Please read comments in OutputStream.h regarding g++ bug with passing
//  of doubles.  - Lup

extern OutputStream * ConsoleOutputStream;

OutputStream :: OutputStream ()
{
    writeBufferIndex = 0;
}

/*
 * Formatted write to an OutputStream (using old C "printf" style formats).
 */
int
OutputStream :: formattedWrite (const char * format, const formatArgs & a1,
                                const formatArgs & a2, const formatArgs & a3, const formatArgs & a4)
{
    formatArgs args[4] = {(formatArgs) 0, (formatArgs) 0, (formatArgs) 0,
                          (formatArgs) 0};
    
    if (&a1 != 0) args[0] = a1;
    if (&a2 != 0) args[1] = a2;
    if (&a3 != 0) args[2] = a3;
    if (&a4 != 0) args[3] = a4;

    //  Print the args using the format string, buffering output into
    //  writeBuffer.
    int charsPrinted = doprnt (format, (formatArgs *) args);

    //  Flush out remaining characters in the writeBuffer.
    if (writeBufferIndex > 0)
    {
        write (writeBuffer, writeBufferIndex);
        writeBufferIndex = 0;
    }
    return charsPrinted;
}

void
OutputStream::emitChar( char c, int width, OutputStreamPadSide padSide )
{
    int padLength = ( width > 0 ) ? ( width - 1 ) : 0;

    if ( padSide == padLeft )   emitSpaces( padLength );
    write( &c, 1 );
    if ( padSide == padRight )  emitSpaces( padLength );
}

void
OutputStream::emitString( const char * string, int width,
                          OutputStreamPadSide padSide )
{
    /*
     * Determine pad length.
     */
    int padLength = width;
    for ( const char * p = string; *p != 0; p++ ) {
        if ( padLength == 0 )
            break;
        padLength--;
    }

    if ( padSide == padLeft )   emitSpaces( padLength );
    write( string, strlen( string ) );
    if ( padSide == padRight )  emitSpaces( padLength );
}

void
OutputStream::emitSpaces( int count )
{
    if( count == 0 ) return;

    static char spaces[] = { ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ',
                             ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '};

    unsigned i;
    for( i = count; i > ( sizeof( spaces ) );
         i -= ( sizeof( spaces ) ) ) {
        write( spaces, sizeof( spaces ) );
    }
    Assert( i > 0 );
    Assert( i <= sizeof( spaces ) - 1 );
    write( spaces, i );
}

void
OutputStream::emitZeros( int count )
{
    if( count == 0 ) return;

    static char zeros[] = { '0', '0', '0', '0', '0', '0', '0', '0',
                            '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0'};

    unsigned i;
    for( i = count; i > ( sizeof( zeros ) );
         i -= ( sizeof( zeros ) ) ) {
        write( zeros, sizeof( zeros ) );
    }
    Assert( i > 0 );
    Assert( i <= sizeof( zeros ) - 1 );
    write( zeros, i );
}


/*
 * emitNumber - converts and prints an int-sized argument in a given base
 *
 * NOTE: Currently only bases 2, 8, 10, and 16 work.  Calls write().
 *  Assumes *caller* has acquired the stream's exclusive use if
 *  necessary.
 */
void
OutputStream::emitNumber( unsigned int arg, int base, int width,
                          OutputStreamPadSide padSide, OutputStreamPadChar padChar )
{
    /*
     * Determine prefix.
     * Update/maintain pad length.
     * Negate value if signed decimal.
     */
    int padLength = width;
    char * prefix = 0;
    int prefixLength = 0;
    int sarg ;
    switch ( base ) {
        case 2:     /* binary */
            prefix = "0b";
            prefixLength = 2;
            break;
        case 8:     /* octal */
            if ( arg != 0 ) {
                prefix = "0";
                prefixLength = 1;
            }
            break;
        case -10:   /* signed decimal */
            base = 10;
            sarg = (int) arg;
            if ( sarg < 0 ) {
                prefix = "-";
                prefixLength = 1;
                sarg = -sarg;
                arg = (unsigned int) sarg;
            }
            break;
        case 10:    /* unsigned decimal */
            break;
        default:    /* unknown - treat as hexadecimal */
            base = 16;
            /* Falls through... */
        case 16:    /* hexadecmial */
            prefix = "0x";
            prefixLength = 2;
            break;
    }
    padLength -= prefixLength;

    /*
     * Convert value to digits into buffer.
     * Update/maintain pad length.
     * Note that this is in reverse order (i.e., LSD first).
     * We test for completion at the bottom of the
     * loop in order to assure that we always get at least
     * one digit in case the value is initially zero.
     */
    unsigned int val = arg;
    static const int bufferSize = 32;
    char buffer[bufferSize];
    char * p = buffer;
    do {
        *p++ = "0123456789abcdef"[val % base];
        padLength--;
    } while ( ( val /= base ) != 0 );
    p--;

    /*
     * Emit left space padding, if appropriate.
     */
    if ( ( padLength > 0 )   &&
         ( padSide == padLeft ) &&
         ( padChar == aSpace ) ) {
        emitSpaces( padLength );
    }

    /*
     * Emit the prefix.
     */
    if( prefix != 0 ) {
        Assert( prefixLength != 0 );
        write( prefix, prefixLength );
    }

    /*
     * Emit left zero padding, if appropriate.
     */
    if ( ( padLength > 0 )   &&
         ( padSide == padLeft ) &&
         ( padChar == aZero ) ) {
        emitZeros( padLength );
    }

    /*
     * Emit all the digits in the buffer.
     * Copy them into an inverted buffer since this is probably
     * faster than writing them one at a time.
     */
    char invertBuffer[bufferSize];
    char * q = invertBuffer;
    int len = p - buffer + 1;
    while( p >= buffer ) {
        *q++ = *p--;
    }
    write( invertBuffer, len );

    /*
     * Emit right padding, if appropriate.
     * This will always be space padding,
     * since zero padding here makes no sense.
     */
    if ( ( padLength > 0 ) && ( padSide == padRight ) ) {
        emitSpaces( padLength );
    }
}

void
OutputStream::flush()
{
    //  Flush out remaining characters in the writeBuffer.
    if (writeBufferIndex > 0)
    {
        write (writeBuffer, writeBufferIndex);
        writeBufferIndex = 0;
    }
    return;
}

OutputStream &
OutputStream::operator << ( char c )
{
    write( &c, 1 );
    return( *this );
}

OutputStream &
OutputStream::operator << ( const char * s )
{
    write( s, strlen( s ) );
    return( *this );
}

OutputStream &
OutputStream::operator << ( const void * s )
{
    write( "Cannot output untyped void*", 28 );
    return( *this );
}

OutputStream &
OutputStream::operator << ( int i )
{
    emitNumber( i, -10, 0, padLeft, aZero );
    return( *this );
}

OutputStream &
OutputStream::operator << ( unsigned int i )
{
    emitNumber( i, 10, 0, padLeft, aZero );
    return( *this );
}

OutputStream &
OutputStream::operator << ( long i )
{
    emitNumber( i, -10, 0, padLeft, aZero );
    return( *this );
}

OutputStream &
OutputStream::operator << ( unsigned long i )
{
    emitNumber( i, 10, 0, padLeft, aZero );
    return( *this );
}

OutputStream &
OutputStream::operator << ( Object * o )
{
    if( o == 0 ) *this << "<NULL>";
    else if( !o->alive() ) {
        *this << "DEAD[" << asHex( (unsigned int)o ) << "]";
    }
    else o->writeName( this );
    return( *this );
}

OutputStream &
OutputStream::operator << ( Object & o )
{
    if( &o == 0 ) *this << "<NULL>";
    else if( !o.alive() ) {
        *this << "DEAD[" << asHex( (unsigned int)&o ) << "]";
    }
    else o.writeName( this );
    return( *this );
}

OutputStream &
OutputStream::operator << ( ProxiableObjectRef & o )
{
    if( o == 0 ) *this << "<NULL>";
    else if( !o->alive() ) {
        *this << "DEAD[" << asHex( (unsigned int)(ProxiableObject *)o ) << "]";
    }
    else o->writeName( this );
    return( *this );
}

OutputStream &
OutputStream::operator << ( ProxiableObjectStar & o )
{
    if( o == 0 ) *this << "<NULL>";
    else if( !o->alive() ) {
        *this << "DEAD[" << asHex( (unsigned int)(ProxiableObject *)o ) << "]";
    }
    else o->writeName( this );
    return( *this );
}

/*
 * The following methods are temporary for compatibility with the old
 * OutputStream class.
 */

/*
 * formatted print.
 */
void
OutputStream::printf( const char * format, const formatArgs & a1,
                      const formatArgs & a2, const formatArgs & a3, const formatArgs & a4 )
{
    formattedWrite( format,
                    ( &a1 != 0) ? a1._formatArgs.l : 0,
                    ( &a2 != 0) ? a2._formatArgs.l : 0,
                    ( &a3 != 0) ? a3._formatArgs.l : 0,
                    ( &a4 != 0) ? a4._formatArgs.l : 0 );
}

/*
 * CPUPrintf - print a message prefaced by the processor id.
 */
void
OutputStream::cpuPrintf( const char * format, const formatArgs & a1,
                         const formatArgs & a2, const formatArgs & a3, const formatArgs & a4 )
{
    formattedWrite( format,
                    ( &a1 != 0) ? a1._formatArgs.l : 0,
                    ( &a2 != 0) ? a2._formatArgs.l : 0,
                    ( &a3 != 0) ? a3._formatArgs.l : 0,
                    ( &a4 != 0) ? a4._formatArgs.l : 0 );
}

/*
 * CriticalPrintf - desperate print to console.
 */
void
OutputStream::criticalPrintf( const char * format, const formatArgs & a1,
                              const formatArgs & a2, const formatArgs & a3, const formatArgs & a4 )
{
    write( "\n", 1 );
    formattedWrite( "CRITICAL: ", 0, 0, 0, 0 );
    formattedWrite( format,
                    ( &a1 != 0) ? a1._formatArgs.l : 0,
                    ( &a2 != 0) ? a2._formatArgs.l : 0,
                    ( &a3 != 0) ? a3._formatArgs.l : 0,
                    ( &a4 != 0) ? a4._formatArgs.l : 0 );
}

/*
 * UitimatePrintf - print with extreme prejudice... only use by something
 *          called by Panic.
 */
void
OutputStream::ultimatePrintf( const char * format, const formatArgs & a1,
                              const formatArgs & a2, const formatArgs & a3, const formatArgs & a4 )
{
    write( "\n", 1 );
    formattedWrite( "ULTIMATE: ", 0, 0, 0, 0 );
    formattedWrite( format,
                    ( &a1 != 0) ? a1._formatArgs.l : 0,
                    ( &a2 != 0) ? a2._formatArgs.l : 0,
                    ( &a3 != 0) ? a3._formatArgs.l : 0,
                    ( &a4 != 0) ? a4._formatArgs.l : 0 );
}

/*
 * Panic - print a panic message and halt.
 *
 * I want to move this to Kernel sometime.  Then we can halt all the
 * processors in the system.
 */
void
OutputStream::panic( const char * format, const formatArgs & a1,
                     const formatArgs & a2, const formatArgs & a3, const formatArgs & a4 )
{
    write( "\n", 1 );
    formattedWrite( "PANIC: ", 0, 0, 0, 0 );
    formattedWrite( format,
                    ( &a1 != 0) ? a1._formatArgs.l : 0,
                    ( &a2 != 0) ? a2._formatArgs.l : 0,
                    ( &a3 != 0) ? a3._formatArgs.l : 0,
                    ( &a4 != 0) ? a4._formatArgs.l : 0 );

    extern void Halt();

    Halt();
    /* Assert( NOTREACHED ); */ // this would result in a
    // recursive call to panic.
}

void
OutputStream::acquireExclusiveUse()
{
    Console().formattedWrite( "error! OutputStream::acquireExclusiveUse() called\n" );
    Assert(NOTREACHED);
}

void
OutputStream::releaseExclusiveUse()
{
    Console().formattedWrite( "error! OutputStream::releaseExclusiveUse() called\n" );
    Assert(NOTREACHED);
}

/*
 * This variable should *never* be declared 1, otherwise it will cause
 * Raid statements to fail when the console has not been created yet!
 *              - Stan
 */
unsigned char DebugGlobalVariable = (char) 0;

char *
OutputStream :: memchr (char *str, char ch, int length)
{
    //  Look for character ch in the first length characters of str.
    //  Return the address of the character found, or return 0
    //  if not found.
    for (int i = 0; i <= length - 1; i++)
    {
        if (str [i] == ch)
            return str + i;
    }
    return 0;
}


double
OutputStream :: modf (/* double x, */ double *i)
{
    double x = doubleParameter;  //  Kludge
#define MAXLONG 0x7fffffffL
    if (x >= 0)
    {
        if (x > MAXLONG)
        {
            *i = x;  /*  Loss of precision here.  */
            return 0.0;
        }    
        else
        {
            long l = (long) x;
            *i = (double) l;
            return x - l;
        }  
    }
    else  
    {
        if (x < -MAXLONG)
        {
            *i = x;  /*  Loss of precision here.  */
            return 0.0;
        }    
        else
        {
            long l = (long) x;
            *i = l;
            return x - l;
        }  
    }
#undef MAXLONG
}

//  Code for formatted write ("printf") from BSD Reno.

/*
 * Copyright (c) 1988 Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted provided
 * that: (1) source distributions retain this entire copyright notice and
 * comment, and (2) distributions including binaries display the following
 * acknowledgement:  ``This product includes software developed by the
 * University of California, Berkeley and its contributors'' in the
 * documentation or other materials provided with the distribution and in
 * all advertising materials mentioning features or use of this software.
 * Neither the name of the University nor the names of its contributors may
 * be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#if defined(LIBC_SCCS) && !defined(lint)
static char sccsid[] = "@(#)doprnt.c    5.39 (Berkeley) 6/28/90";
#endif /* LIBC_SCCS and not lint */

/* 11-bit exponent (VAX G floating point) is 308 decimal digits */
#define MAXEXP      308
/* 128 bit fraction takes up 39 decimal digits; max reasonable precision */
#define MAXFRACT    39

#define DEFPREC     6

#define BUF     (MAXEXP+MAXFRACT+1) /* + decimal point */

void
OutputStream :: putChar (char ch)
{
    if (writeBufferIndex < outputStreamWriteBufferSize)
        writeBuffer [writeBufferIndex++] = ch;
    else
    {
        write (writeBuffer, outputStreamWriteBufferSize);
        writeBuffer [0] = ch;
        writeBufferIndex = 1;
    }
}

#define ARG(basetype) \
    _ulong = flags&LONGINT ? (long) (args [argsIndex++]._formatArgs.l) : \
        flags&SHORTINT ? (short basetype) (int) (args [argsIndex++]._formatArgs.l) : \
        (int) (args [argsIndex++]._formatArgs.l)

#define todigit(c)  ((c) - '0')
#define tochar(n)   ((n) + '0')

#define LONGINT     0x01        /* long integer */
#define LONGDBL     0x02        /* long double; unimplemented */
#define SHORTINT    0x04        /* short integer */
#define ALT     0x08        /* alternate form */
#define LADJUST     0x10        /* left adjustment */
#define ZEROPAD     0x20        /* zero (as opposed to blank) pad */
#define HEXPREFIX   0x40        /* add 0x or 0X prefix */

int
OutputStream :: doprnt (const char *fmt0, formatArgs *args)
{
    const char *fmt;    /* format string */
    int ch;             /* character from fmt */
    int cnt =0;         /* return value accumulator */
    int n;              /* random handy integer */
    char *t;            /* buffer pointer */
    double dNum=0;      /* double precision arguments %[eEfgG] */
    unsigned long _ulong;   /* integer arguments %[diouxX] */
    int base;       /* base for [diouxX] conversion */
    int dprec;      /* decimal precision in [diouxX] */
    int fieldsz;    /* field size expanded by sign, etc */
    int flags;      /* flags as above */
    int fpprec;     /* `extra' floating precision in [eEfgG] */
    int prec;       /* precision from format (%.3d), or -1 */
    int realsz;     /* field size expanded by decimal precision */
    int size;       /* size of converted field or string */
    int width;      /* width from format (%8d), or 0 */
    char sign;      /* sign prefix (' ', '+', '-', or \0) */
    char softsign;  /* temporary negative sign for floats */
    char *digs;     /* digits for [diouxX] conversion */
    char buf[BUF];      /* space for %c, %[diouxX], %[eEfgG] */
    int argsIndex = 0;  /* next value in args to be used */

    fmt = fmt0;
    digs = "0123456789abcdef";
    Object * o;
    for (;;) {
        for (;;)
        {
            ch = *fmt;
            if (ch == '%' || ch == 0)
                break;
            putChar (ch);
            cnt++;
            fmt++;
        }
        
        if (ch == 0)
            return (cnt);

        flags = 0; dprec = 0; fpprec = 0; width = 0;
        prec = -1;
        sign = '\0';
        
    rflag: 
        switch (*++fmt) {
            case ' ':
                /*
                 * ``If the space and + flags both appear, the space
                 * flag will be ignored.''
                 *  -- ANSI X3J11
                 */
                if (!sign)
                    sign = ' ';
                goto rflag;
            case '#':
                flags |= ALT;
                goto rflag;
            case '*':
                /*
                 * ``A negative field width argument is taken as a
                 * - flag followed by a  positive field width.''
                 *  -- ANSI X3J11
                 * They don't exclude field widths read from args.
                 */
                if ((width = (int) (args [argsIndex++]._formatArgs.l)) >= 0)
                    goto rflag;
                width = -width;
                /* FALLTHROUGH */
            case '-':
                flags |= LADJUST;
                goto rflag;
            case '+':
                sign = '+';
                goto rflag;
            case '.':
                if (*++fmt == '*')
                    n = (int) (args [argsIndex++]._formatArgs.l);
                else {
                    n = 0;
                    while (isascii(*fmt) && isdigit(*fmt))
                        n = 10 * n + todigit(*fmt++);
                    --fmt;
                }
                prec = n < 0 ? -1 : n;
                goto rflag;
            case '0':
                /*
                 * ``Note that 0 is taken as a flag, not as the
                 * beginning of a field width.''
                 *  -- ANSI X3J11
                 */
                flags |= ZEROPAD;
                goto rflag;
            case '1': case '2': case '3': case '4':
            case '5': case '6': case '7': case '8': case '9':
                n = 0;
                do {
                    n = 10 * n + todigit(*fmt);
                } while (isascii(*++fmt) && isdigit(*fmt));
                width = n;
                --fmt;
                goto rflag;
            case 'L':
                flags |= LONGDBL;
                goto rflag;
            case 'h':
                flags |= SHORTINT;
                goto rflag;
            case 'l':
                flags |= LONGINT;
                goto rflag;
            case 'c':
                *(t = buf) = (int) (args [argsIndex++]._formatArgs.l);
                size = 1;
                sign = '\0';
                goto pforw;
            case 'D':
                flags |= LONGINT;
                /*FALLTHROUGH*/
            case 'd':
            case 'i':
                ARG(int);
                if ((long)_ulong < 0) {
                    _ulong = -_ulong;
                    sign = '-';
                }
                base = 10;
                goto number;
            case 'e':
            case 'E':
            case 'f':
            case 'g':
            case 'G':
                //  Kludge: g++ with -mregparm sometimes crashes
                //  when "*1.0" is omitted.
                dNum = args [argsIndex++]._formatArgs.d * 1.0;
                /*
                 * don't do unrealistic precision; just pad it with
                 * zeroes later, so buffer size stays rational.
                 */
                if (prec > MAXFRACT) {
                    if (*fmt != 'g' && *fmt != 'G' || (flags&ALT))
                        fpprec = prec - MAXFRACT;
                    prec = MAXFRACT;
                }
                else if (prec == -1)
                    prec = DEFPREC;
                /*
                 * softsign avoids negative 0 if dNum is < 0 and
                 * no significant digits will be shown
                 */
                if (dNum < 0) {
                    softsign = '-';
                    dNum = -dNum;
                }
                else
                    softsign = 0;
                /*
                 * convert may have to round up past the "start" of the
                 * buffer, i.e. ``intf("%.2f", (double)9.999);'';
                 * if the first char isn't NULL, it did.
                 */
                *buf = 0;
                doubleParameter = dNum;  //  Kludge
                size = convert(/* dNum, */ prec, flags, &softsign,
                               *fmt, buf, buf + sizeof(buf));
                if (softsign)
                    sign = '-';
                t = *buf ? buf : buf + 1;
                goto pforw;
            case 'n':
                if (flags & LONGINT)
                    *(long *) (args [argsIndex++]._formatArgs.p) = cnt;
                else if (flags & SHORTINT)
                    *(short *) (args [argsIndex++]._formatArgs.p) = cnt;
                else
                    *(int *) (args [argsIndex++]._formatArgs.p) = cnt;
                break;
            case 'O':
                flags |= LONGINT;
                /*FALLTHROUGH*/
            case 'o':
                ARG(unsigned);
                base = 8;
                goto nosign;
            case 'p':
                /*
                 * ``The argument shall be a pointer to void.  The
                 * value of the pointer is converted to a sequence
                 * of printable characters, in an implementation-
                 * defined manner.''
                 *  -- ANSI X3J11
                 */
                /* NOSTRICT */
                _ulong = (unsigned long) (void *)
                (args [argsIndex++]._formatArgs.p);
                base = 16;
                goto nosign;
            case 's':
                if (!(t = (char *) (args [argsIndex++]._formatArgs.p)))
                    t = "(null)";
                if (prec >= 0) {
                    /*
                     * can't use strlen; can only look for the
                     * NUL in the first `prec' characters, and
                     * strlen() will go further.
                     */
                    char *p;

                    if ((p = memchr(t, 0, prec))) {
                        size = p - t;
                        if (size > prec)
                            size = prec;
                    } else
                        size = prec;
                } else
                    size = strlen(t);
                sign = '\0';
                goto pforw;
            case 'U':
                flags |= LONGINT;
                /*FALLTHROUGH*/
            case 'u':
                ARG(unsigned);
                base = 10;
                goto nosign;
            case 'X':
                digs = "0123456789ABCDEF";
                /* FALLTHROUGH */
            case 'x':
                ARG(unsigned);
                base = 16;
                /* leading 0x/X only if non-zero */
                if (flags & ALT && _ulong != 0)
                    flags |= HEXPREFIX;

                /* unsigned conversions */
        nosign:         sign = '\0';
                /*
                 * ``... diouXx conversions ... if a precision is
                 * specified, the 0 flag will be ignored.''
                 *  -- ANSI X3J11
                 */
        number:         if ((dprec = prec) >= 0)
            flags &= ~ZEROPAD;

                /*
                 * ``The result of converting a zero value with an
                 * explicit precision of zero is no characters.''
                 *  -- ANSI X3J11
                 */
                t = buf + BUF;
                if (_ulong != 0 || prec != 0) {
                    do {
                        *--t = digs[_ulong % base];
                        _ulong /= base;
                    } while (_ulong);
                    digs = "0123456789abcdef";
                    if (flags & ALT && base == 8 && *t != '0')
                        *--t = '0'; /* octal leading 0 */
                }
                size = buf + BUF - t;

        pforw:
                /*
                 * All reasonable formats wind up here.  At this point,
                 * `t' points to a string which (if not flags&LADJUST)
                 * should be padded out to `width' places.  If
                 * flags&ZEROPAD, it should first be prefixed by any
                 * sign or other prefix; otherwise, it should be blank
                 * padded before the prefix is emitted.  After any
                 * left-hand padding and prefixing, emit zeroes
                 * required by a decimal [diouxX] precision, then print
                 * the string proper, then emit zeroes required by any
                 * leftover floating precision; finally, if LADJUST,
                 * pad with blanks.
                 */

                /*
                 * compute actual size, so we know how much to pad
                 * fieldsz excludes decimal prec; realsz includes it
                 */
                fieldsz = size + fpprec;
                if (sign)
                    fieldsz++;
                if (flags & HEXPREFIX)
                    fieldsz += 2;
                realsz = dprec > fieldsz ? dprec : fieldsz;

                /* right-adjusting blank padding */
                if ((flags & (LADJUST|ZEROPAD)) == 0 && width)
                    for (n = realsz; n < width; n++)
                        putChar(' ');
                /* prefix */
                if (sign)
                    putChar(sign);
                if (flags & HEXPREFIX) {
                    putChar('0');
                    putChar((char)*fmt);
                }
                /* right-adjusting zero padding */
                if ((flags & (LADJUST|ZEROPAD)) == ZEROPAD)
                    for (n = realsz; n < width; n++)
                        putChar('0');
                /* leading zeroes from decimal precision */
                for (n = fieldsz; n < dprec; n++)
                    putChar('0');

                /* the string or number proper */
                n = size;
                while (--n >= 0)
                    putChar(*t++);
                /* trailing f._formatArgs.p. zeroes */
                while (--fpprec >= 0)
                    putChar('0');
                /* left-adjusting padding (always blank) */
                if (flags & LADJUST)
                    for (n = realsz; n < width; n++)
                        putChar(' ');
                /* finally, adjust cnt */
                cnt += width > realsz ? width : realsz;
                break;
            case '\0':  /* "%?" prints ?, unless ? is NULL */
                return (cnt);
                /*
                 * NOTE: Below are non-standard extensions for Choices.
                 */

            case 'b':
                /*
                 * Convert to binary.
                 */
                OutputStream :: flush ();
                emitNumber((unsigned int) (args [argsIndex++]._formatArgs.l),
                           2, width, padLeft, aSpace );
                break;

            case 'A':
                /*
                 * An address.
                 * Print as "@%010x".
                 */
                OutputStream :: flush ();
                write( "@", 1 );
                emitNumber( (unsigned int) (args [argsIndex++]._formatArgs.l),
                            16, 10, padLeft, aZero );
                break;
            
            case 'N':
                /*
                 * Send the writeName() message
                 * to the argument.
                 */
                OutputStream :: flush ();
                o = (Object *) (args [argsIndex++]._formatArgs.p);
                if( o == 0 ) {
                    write( "<NULL>", 6 );
                }
                else if( !o->alive() ) {
                    write( "DEAD[", 5 );
                    emitNumber( (unsigned int) o,
                                16, 0, padLeft, aSpace );
                    write( "]", 1 );
                    //o->writeName( this );
                }
                else {
                    o->writeName( this );
                }
                break;
            
            case 'I':
                /*
                 * Send the inspect() message
                 * to the argument.
                 */
                OutputStream :: flush ();
                o = (Object *) (args [argsIndex++]._formatArgs.p);
                if( o == 0 ) {
                    write( "*(NULL)", 7 );
                }
                else if( !o->alive() ) {
                    write( "*(DEAD)", 7 );
                }
                else {
                    o->inspect( this );
                }
                break;

            case 'T':
                /*
                 * A 'this'.
                 * Print as "[@%010x]".
                 */
                OutputStream :: flush ();
                write( "[@", 2 );
                emitNumber( (unsigned int) (args [argsIndex++]._formatArgs.l),
                            16, 10, padLeft, aZero );
                write( "]", 1 );
                break;
            
            default:
                putChar((char)*fmt);
                cnt++;
        }
        fmt++;
    }
    //Assert (NOTREACHED);
    return 0;
}

int
OutputStream :: convert (/* double number, */ int prec, int flags, char *signp,
                         unsigned char fmtch, char *startp, char *endp)
{
    double number = doubleParameter;  //  Kludge
    char *p, *t;
    double fract;
    int dotrim, expcnt, gformat;
    double integer, tmp;

    dotrim = expcnt = gformat = 0;
    doubleParameter = number;  //  Kludge
    fract = modf(/* number, */ &integer);

    /* get an extra slot for rounding. */
    t = ++startp;

    /*
     * get integer portion of number; put into the end of the buffer; the
     * .01 is added for modf(356.0 / 10, &integer) returning .59999999...
     */
    for (p = endp - 1; integer; ++expcnt) {
        doubleParameter = integer / 10;  //  Kludge
        tmp = modf(/* integer / 10, */ &integer);
        *p-- = tochar((int)((tmp + .01) * 10));
    }
    switch(fmtch) {
        case 'f':
            /* reverse integer into beginning of buffer */
            if (expcnt)
                for (; ++p < endp; *t++ = *p);
            else
                *t++ = '0';
            /*
             * if precision required or alternate flag set, add in a
             * decimal point.
             */
            if (prec || flags&ALT)
                *t++ = '.';
            /* if requires more precision and some fraction left */
            if (fract) {
                if (prec)
                    do {
                        doubleParameter = fract * 10;// Kludge
                        fract = modf(/* fract * 10, */ &tmp);
                        *t++ = tochar((int)tmp);
                    } while (--prec && fract);
                if (fract)
                {
                    doubleParameter = fract;  //  Kludge
                    startp = round(/* fract, */ (int *) 0, startp,
                                   t - 1, (char)0, signp);
                }
            }
            for (; prec--; *t++ = '0');
            break;
        case 'e':
        case 'E':
    eformat:    if (expcnt) {
        *t++ = *++p;
        if (prec || flags&ALT)
            *t++ = '.';
        /* if requires more precision and some integer left */
        for (; prec && ++p < endp; --prec)
            *t++ = *p;
        /*
         * if done precision and more of the integer component,
         * round using it; adjust fract so we don't re-round
         * later.
         */
        if (!prec && ++p < endp) {
            fract = 0;
            doubleParameter = 0;  //  Kludge
            startp = round(/* (double)0,*/ &expcnt, startp,
                           t - 1, *p, signp);
        }
        /* adjust expcnt for digit in front of decimal */
        --expcnt;
    }
        /* until first fractional digit, decrement exponent */
    else if (fract) {
        /* adjust expcnt for digit in front of decimal */
        for (expcnt = -1;; --expcnt) {
            doubleParameter = fract * 10;  //  Kludge
            fract = modf(/* fract * 10, */ &tmp);
            if (tmp)
                break;
        }
        *t++ = tochar((int)tmp);
        if (prec || flags&ALT)
            *t++ = '.';
    }
    else {
        *t++ = '0';
        if (prec || flags&ALT)
            *t++ = '.';
    }
        /* if requires more precision and some fraction left */
        if (fract) {
            if (prec)
                do {
                    doubleParameter = fract * 10; // Kludge
                    fract = modf(/* fract * 10, */ &tmp);
                    *t++ = tochar((int)tmp);
                } while (--prec && fract);
            if (fract)
            {
                doubleParameter = fract;  //  Kludge
                startp = round(/* fract, */ &expcnt, startp,
                               t - 1, (char)0, signp);
            }
        }
        /* if requires more precision */
        for (; prec--; *t++ = '0');

        /* unless alternate flag, trim any g/G format trailing 0's */
        if (gformat && !(flags&ALT)) {
            while (t > startp && *--t == '0');
            if (*t == '.')
                --t;
            ++t;
        }
        t = exponent(t, expcnt, fmtch);
        break;
        case 'g':
        case 'G':
            /* a precision of 0 is treated as a precision of 1. */
            if (!prec)
                ++prec;
            /*
             * ``The style used depends on the value converted; style e
             * will be used only if the exponent resulting from the
             * conversion is less than -4 or greater than the precision.''
             *  -- ANSI X3J11
             */
            if (expcnt > prec || !expcnt && fract && fract < .0001) {
                /*
                 * g/G format counts "significant digits, not digits of
                 * precision; for the e/E format, this just causes an
                 * off-by-one problem, i.e. g/G considers the digit
                 * before the decimal point significant and e/E doesn't
                 * count it as precision.
                 */
                --prec;
                fmtch -= 2;     /* G->E, g->e */
                gformat = 1;
                goto eformat;
            }
            /*
             * reverse integer into beginning of buffer,
             * note, decrement precision
             */
            if (expcnt)
                for (; ++p < endp; *t++ = *p, --prec);
            else
                *t++ = '0';
            /*
             * if precision required or alternate flag set, add in a
             * decimal point.  If no digits yet, add in leading 0.
             */
            if (prec || flags&ALT) {
                dotrim = 1;
                *t++ = '.';
            }
            else
                dotrim = 0;
            /* if requires more precision and some fraction left */
            if (fract) {
                if (prec) {
                    do {
                        doubleParameter = fract * 10; // Kludge
                        fract = modf(/* fract * 10, */ &tmp);
                        *t++ = tochar((int)tmp);
                    } while(!tmp);
                    while (--prec && fract) {
                        doubleParameter = fract * 10; // Kludge
                        fract = modf(/* fract * 10, */ &tmp);
                        *t++ = tochar((int)tmp);
                    }
                }
                if (fract)
                {
                    doubleParameter = fract;  //  Kludge
                    startp = round(/* fract, */ (int *) 0, startp,
                                   t - 1, (char)0, signp);
                }
            }
            /* alternate format, adds 0's for precision, else trim 0's */
            if (flags&ALT)
                for (; prec--; *t++ = '0');
            else if (dotrim) {
                while (t > startp && *--t == '0');
                if (*t != '.')
                    ++t;
            }
    }
    return(t - startp);
}

char *
OutputStream :: round (/* double fract, */ int *exp, char *start, char *end,
                       char ch, char *signp)
{
    double fract = doubleParameter;  //  Kludge
    double tmp;

    if (fract)
    {
        doubleParameter = fract * 10;  //  Kludge
        modf(/* fract * 10, */ &tmp);
    }
    else
        tmp = todigit(ch);
    if (tmp > 4)
        for (;; --end) {
            if (*end == '.')
                --end;
            if (++*end <= '9')
                break;
            *end = '0';
            if (end == start) {
                if (exp) {  /* e/E; increment exponent */
                    *end = '1';
                    ++*exp;
                }
                else {      /* f; add extra digit */
                    *--end = '1';
                    --start;
                }
                break;
            }
        }
    /* ``"%.3f", (double)-0.0004'' gives you a negative 0. */
    else if (*signp == '-')
        for (;; --end) {
            if (*end == '.')
                --end;
            if (*end != '0')
                break;
            if (end == start)
                *signp = 0;
        }
    return(start);
}

char *
OutputStream :: exponent (char *p, int exp, unsigned char fmtch)
{
    char *t;
    char expbuf[MAXEXP];

    *p++ = fmtch;
    if (exp < 0) {
        exp = -exp;
        *p++ = '-';
    }
    else
        *p++ = '+';
    t = expbuf + MAXEXP;
    if (exp > 9) {
        do {
            *--t = tochar(exp % 10);
        } while ((exp /= 10) > 9);
        *--t = tochar(exp);
        for (; t < expbuf + MAXEXP; *p++ = *t++);
    }
    else {
        *p++ = '0';
        *p++ = tochar(exp);
    }
    return(p);
}

#include "ClassInstaller.h"
DEFINECLASS("OutputStream",OutputStream,ProxiableObject,0,0);
