#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * PersistentObjectStar.c - 
 */
#include "Assert.h"
#include "PersistentObject.h"
#include "PersistentObjectStar.h"
#include "PersistentClass.h"
#include "ProxiableObject.h"


#ifdef VCHOICES
const unsigned mask = 0x00000001;
#else
#ifdef CHORUS
const unsigned mask = 0x08000000;
#else
const unsigned mask = 0xf0000000;
#endif
#endif

inline
long int
in_mem(void *ptr)
{
	return ( ((unsigned)ptr & mask) != mask);
}

inline
ProxiableObject *
mask_ptr(unsigned long p)
{
	return ( (ProxiableObject *)(p | mask));
}

inline
int
unmask_ptr(void *p)
{
	return ((unsigned long)p & ~mask);
}

void
PersistentObjectStar::cache( PersistentClass * aClass ) 
{
        if( _pointer != 0 ) {
                if(!in_mem(_pointer)) {
                        _pointer = aClass->retrieveObject(unmask_ptr(_pointer));
                        Assert( _pointer != 0 );
                }
        }
}

void
PersistentObjectStar::stash( ProxiableObject *po, int ref ) 
{
        if( _pointer != 0 ) {
                if( in_mem(po)) {
                        _pointer->unreference();
                }
        }
        _pointer = po;
        if( !ref && ( _pointer != 0 ) ) {
                if( in_mem(_pointer)) {
                        _pointer->reference();
                }
        }
}

void
PersistentObjectStar::flushStar()
{
        if( _pointer != 0 ) {
                if( in_mem(_pointer)) {
                        PersistentObject * po = (PersistentObject *)_pointer;
                        int id = po->idNumber();
                        _pointer = mask_ptr(id); 
                        po->unreference();
                }
        }
}

PersistentObjectStar::PersistentObjectStar( ProxiableObject *const  po )
{
	_pointer = 0;
        stash( po );
}


PersistentObjectStar::PersistentObjectStar( const ProxiableObjectRef & po )
{
        _pointer = 0;
        stash( po, 1 );
}

PersistentObjectStar::PersistentObjectStar( const ProxiableObjectStar & po )
{
        _pointer = 0;
        stash( po );
}

PersistentObjectStar &
PersistentObjectStar::operator =( ProxiableObject *const  po )
{
	_pointer = 0;
        stash( po );
	return *this;
}

PersistentObjectStar & 
PersistentObjectStar::operator =( const ProxiableObjectRef & po )
{
        _pointer = 0;
        stash( po, 1 );
	return *this;
}

PersistentObjectStar & 
PersistentObjectStar::operator =( const ProxiableObjectStar & po )
{
        _pointer = 0;
        stash( po );
	return *this;
}

PersistentObjectStar::PersistentObjectStar( PersistentObject *const  po )
{
        _pointer = 0;
        stash( po );
}

PersistentObjectStar::PersistentObjectStar( const PersistentObjectRef & po )
{
        _pointer = 0;
        stash( po, 1 );
}

PersistentObjectStar::PersistentObjectStar( const PersistentObjectStar & po )
{
        _pointer = 0;
        stash( po );
}

PersistentObjectStar & 
PersistentObjectStar::operator=( PersistentObject *const  po )
{
    stash( po );
    return( *this );
}

PersistentObjectStar & 
PersistentObjectStar::operator=( const PersistentObjectRef & po )
{
	stash( po, 1 );
	return( *this );
}

PersistentObjectStar & 
PersistentObjectStar::operator=( const PersistentObjectStar & po )
{
    stash( po );
    return( *this );
}

PersistentObjectStar::operator PersistentObject*()
{
        cache( PersistentObjectClass );
        return( (PersistentObject*)_pointer );
}

PersistentObject*
PersistentObjectStar::operator->() 
{
        cache( PersistentObjectClass );
        return( (PersistentObject*)_pointer );
}

void
PersistentObjectRef::cache( PersistentClass * aClass ) 
{
        if( _pointer != 0 ) {
                if( ! in_mem(_pointer)) {
                        _pointer = aClass->retrieveObject(unmask_ptr(_pointer));
                        Assert( _pointer != 0 );
                }
        }
}

void
PersistentObjectRef::stash( ProxiableObject *po, int ref ) 
{
        if( _pointer != 0 ) {
                if(in_mem(_pointer)) {
                        _pointer->unreference();
                }
        }
        _pointer = po;
        if( !ref && ( _pointer != 0 ) ) {
                if (in_mem(_pointer)) {
                        _pointer->reference();
                }
        }
}

PersistentObjectRef::PersistentObjectRef( ProxiableObject *const  po )
{
	stash( po );
}

PersistentObjectRef::PersistentObjectRef( const ProxiableObjectRef & po )
{
	stash( po, 1 );
}

PersistentObjectRef::PersistentObjectRef( const ProxiableObjectStar & po )
{
	stash( po );
}

PersistentObjectRef &
PersistentObjectRef::operator =( ProxiableObject *const  po )
{
	stash( po );
	return *this;
}

PersistentObjectRef &
PersistentObjectRef::operator =( const ProxiableObjectRef & po )
{
	stash( po, 1 );
	return *this;
}

PersistentObjectRef &
PersistentObjectRef::operator =( const ProxiableObjectStar & po )
{
	stash( po );
	return *this;
}

PersistentObjectRef::PersistentObjectRef( PersistentObject *const  po )
{
	stash( po );
}

PersistentObjectRef::PersistentObjectRef( const PersistentObjectRef & po )
{
	stash( po, 1 );
}

PersistentObjectRef::PersistentObjectRef( const PersistentObjectStar & po )
{
	stash( po );
}

PersistentObjectRef & 
PersistentObjectRef::operator=( PersistentObject *const  po )
{
    stash( po );
    return( *this );
}

PersistentObjectRef & 
PersistentObjectRef::operator=( const PersistentObjectRef & po )
{
	stash( po, 1 );
	return( *this );
}

PersistentObjectRef & 
PersistentObjectRef::operator=( const PersistentObjectStar & po )
{
    stash( po );
    return( *this );
}

PersistentObjectRef::operator PersistentObject*()
{
	cache( PersistentObjectClass );
	return( (PersistentObject*)_pointer );
}

PersistentObject*
PersistentObjectRef::operator->()
{
	cache( PersistentObjectClass );
	return( (PersistentObject*)_pointer );
}
