#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ProxiableObject.c -  object class to assist in debugging.
 */
#include "Assert.h"
#include "Raid.h"
#include "ProxiableObject.h"
#include "ProxiableObjectStar.h"
#include "Class.h"
#include "OutputStream.h"
#include "String.h"
#include "NameServer.h"
#include "Lock.h"


#ifdef __GNUC__
#pragma implementation
#endif

Lock ProxiableObject::ProxiableObjectLock;
ProxiableObject * ProxiableObject::ProxiableObjectsList = 0;
int ProxiableObject::ProxiableObjectsStored = 0;

ProxiableObject::ProxiableObject()
{
	_real = 0;

	ProxiableObjectLock.acquire();
	ProxiableObjectsStored++;
	proxiableObjectName = 0;
	proxiableObjectNext = ProxiableObjectsList;
	proxiableObjectList = &ProxiableObjectsList;
	if ( ProxiableObjectsList ) {
		ProxiableObjectsList->proxiableObjectList = 
			&proxiableObjectNext;
	}
	ProxiableObjectsList = this;
	ProxiableObjectLock.release();
}

ProxiableObject::~ProxiableObject()
{
	Raids( RaidConstructor ) << this << "::~ProxiableObject()\n" << eor;

	Assert( alive() );

	ProxiableObjectLock.acquire();

	char * name = proxiableObjectName;
	ProxiableObject * next = proxiableObjectNext;
	*proxiableObjectList = next;
	if ( next ) {
		next->proxiableObjectList = proxiableObjectList;
	}
	ProxiableObjectsStored--;
	ProxiableObjectLock.release();

	Assert( _referenceCount.value() == 0 );

	delete name;
}

void
ProxiableObject::noRemainingReferences()
{
 	/*
 	 * NOTE: this function is duplicated in ObjectProxy
 	 */
	delete this;
}

void
ProxiableObject::reference()
{
	/*
	 * NOTE: this function is duplicated in ObjectProxy
	 */
	Raids( RaidReference ) << this << "::reference()\n" << eor;

	_referenceCount.increment();
}

void
ProxiableObject::unreference()
{
	/*
	 * NOTE: this function is duplicated in ObjectProxy
	 */
	Raids( RaidReference ) << this << "::unreference()\n" << eor;

	Assert( alive() );

	if( _referenceCount.decrement() == 0 ) {
		noRemainingReferences();
	}
}

void
ProxiableObject::initialize()
{
}

ProxiableObjectRef
ProxiableObject::asA( Class * aClass, int & status )
{
	status = 0;
	if( isKindOf( aClass ) ) {
		return( this );
	}
	else {
		aClass->constructor( this );
		return( 0 );
	}
}

ProxiableObjectRef
ProxiableObject::asA( char * className, int & status )
{
	if( className == 0 ) className = "Class";

	ClassStar aClass = StandardNameServer->lookup( className, ClassClass );

	return( asA( aClass, status ) );
}

char *
ProxiableObject::objectName()
{
	return proxiableObjectName;
}

void
ProxiableObject::setName( const char * str )
{
	if( str != 0 ) {
		ProxiableObjectLock.acquire();
		if ( proxiableObjectName != 0 ) {
			delete proxiableObjectName;
		}
		proxiableObjectName = new char[ strlen( str ) + 1 ];
		strcpy( proxiableObjectName, (char *) str );
		ProxiableObjectLock.release();
	}
}

void
ProxiableObject::writeName( OutputStream * os )
{
	Object::writeName( os );
	os->formattedWrite( "{%d}", _referenceCount.value() );
	if( objectName() != 0 ) {
		os->formattedWrite( "(%s)", objectName() );
	}
}

void
ProxiableObject::inspect( OutputStream * os )
{
	Object::inspect( os );
	if( objectName() != 0 ) {
		os->formattedWrite( "(%s)", objectName() );
	}
}



#include "ClassInstaller.h"
DEFINECLASS("ProxiableObject",ProxiableObject,Object,0,0);
