#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David K. Raila (raila@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * Raid.cc - Object debugging interface
 *
 */
#include "Class.h"
#include "FileSystem.h"	// For error return codes
#include "NameServer.h"	
#include "Raid.h"
#include "String.h"
#include "Convert.h"


#ifdef __GNUC__
#pragma implementation
#endif

int
parseMask( char * s )
{
	char * codeMasks[6];
	codeMasks[0] = "Error";
	codeMasks[1] = "Constructor";
	codeMasks[2] = "Reference";
	codeMasks[3] = "Method";
	codeMasks[4] = "Protected";
	codeMasks[5] = "Detail";

	int mask = 0;
	int len = strlen( s );

	char * t = s;
	for( int i = 0; i <= len; i++, s++ ) {
		if( *s == '|' ) *s = '\0';
		if( *s == '\0' ) {
			for( int j = 0; j < 6; j++ ) {
				if( ( strprefix( t, codeMasks[j] ) == 0 ) ||
				    ( ( strncmp( t, "Raid", 4 ) == 0 ) &&
				    ( strprefix( t+4,codeMasks[j] ) == 0 ) ) ) {
					mask |= 1 << j;
				}
			}
			t = s+1;
		}
	}

	return( mask );
}


void
debugClass( Class * aClass, int mask, int messages, int objects )
{
	Assert( aClass != 0 );
	aClass->setInitDebugMask( mask );
	aClass->setInitDebugMessages( messages );
	aClass->setInitDebugObjects( objects );
}

void
debugChildren( Class * aClass, int mask, int messages, int objects )
{
	Assert( aClass != 0 );
	ClassStar child = aClass->child();
	while( child != 0 ) {
		debugClass( child, mask, messages, objects );
		debugChildren( child, mask, messages, objects );
		child = child->sibling();
	}
}

int
cdb( int argc, char *argv[] )
{
	if( argc < 4 ) {
		Console().formattedWrite( "usage: %s (member|kind) ClassName mask [#messages [#objects]]\n", argv[0] );
	    return( NumberOfArguments );
	}
	
	else {
		int kind = 0;
		if( strprefix( argv[1], "kindred" ) == 0 ) kind = 1;
		else if( strprefix( argv[1], "members" ) != 0 ) {
			Console().formattedWrite( "enter either member or kind\n" );
			return( InvalidArgument );
		}

		int mask = 0;
		if( ( argv[3][0] >= '0' ) && ( argv[3][0] <= '9' ) ) {
			mask = atoi( argv[3] );
		}
		else mask = parseMask( argv[3] );
		int messages = MaxObjectDebugMessages;
		int objects =  0xfffffff;
		if( argc > 4 ) {
			messages = atoi( argv[4] );
			if( argc > 5 ) {
				objects = atoi( argv[5] );
			}
		}

		ClassStar aClass = 
			StandardNameServer->lookup( argv[2], ClassClass );
		if( aClass != 0 ) {
			debugClass( aClass, mask, messages, objects );
			if( kind ) {
			    debugChildren( aClass, mask, messages, objects );
			}
		}
	}
	return( 0 );
}


int
odb( int argc, char *argv[] )
{
	int mask = 0;
	int messages = 0;
	int kind = 0;

	if ( argc > 3 ) {
		if( ( argv[3][0] >= '0' ) && ( argv[3][0] <= '9' ) ) {
			mask = atoi( argv[3] );
		}
		else mask = parseMask( argv[3] );
		if ( argc > 4 ) messages = atoi( argv[4] );
		else messages = MaxObjectDebugMessages;
		if( strprefix( argv[1], "kindred" ) == 0 ) kind = 1;
		else if( strprefix( argv[1], "members" ) != 0 ) {
			Console().formattedWrite( "use either m or k for 2nd argument\n" );
			return( InvalidArgument );
		}
	}
	else {
		Console().formattedWrite( "usage: %s (member|kind) ClassName [mask [#messages]]\n", argv[0] );
		return( NumberOfArguments );
	}

	ClassStar aClass =
		StandardNameServer->lookup( argv[2], ClassClass );
	if( aClass != 0 ) {
		if( kind ) aClass->debugKindred( mask, messages );
		else	   aClass->debugMembers( mask, messages );
	}
	return( 0 );
}


void
printClass( Class * aClass, const char * tag = "" )
{
	Assert( aClass != 0 );
        char namebuf[64];
                Console().formattedWrite( "%s %d %s\n",
                       aClass->name(64, namebuf), aClass->sizeOf(), tag  );
/*
	for( int i = 0; i < aClass->level(); i++ ) {
		Console().formattedWrite( "  " );
	}
	char namebuf[64];
	unsigned char ring = aClass->ring();
	if ( ring == 0 )
		Console().formattedWrite( "%s %d %s\n",
                       aClass->name(64, namebuf), aClass->sizeOf(), tag  );
	else
		Console().formattedWrite( "%s %d (ring %d) %s\n",
		    aClass->name(64, namebuf), aClass->sizeOf(), ring, tag );
*/
}

void
printAncestors( Class * aClass )
{
	Assert( aClass != 0 );
	ClassStar parent = aClass->parent();
	if( parent != 0 ) {
		printAncestors( parent );
		printClass( parent );
	}
}

void
printDescendants( Class * aClass )
{
	Assert( aClass != 0 );
	ClassStar child = aClass->child();
	while( child != 0 ) {
		printClass( child );
		printDescendants( child );
		child = child->sibling();
	}
}

int
hierarchy( int argc, char *argv[] )
{
	if( argc < 2 ) {
	    Console().formattedWrite( "usage: %s ClassName...\n", argv[0] );
	    return( NumberOfArguments );
	}
	
	for( int i = 1; i < argc; i++ ) {
		ClassStar aClass = 
			StandardNameServer->lookup( argv[i], ClassClass );
		if( aClass != 0 ) {
			if( strcmp( argv[0], "hierarchy" ) == 0 ) {
				printAncestors( aClass );
				printClass( aClass, " <<<" );
				printDescendants( aClass );
			}
			else if( strcmp( argv[0], "ancestors" ) == 0 ) {
				printAncestors( aClass );
				printClass( aClass );
			}
			else if( strcmp( argv[0], "descendants" ) == 0 ) {
				printClass( aClass );
				printDescendants( aClass );
			}
			else {
				Assert( NOTREACHED );
			}
		}
		else {
			Console().formattedWrite( "Cannot find class %s\n", argv[i] );
		}
		if( i < argc - 1 ) Console().formattedWrite( "\n" );
	}
	return( 0 );
}


int
instances( int argc, char *argv[] )
{
	int kind = 0;

	if( argc > 1 ) {
		if( strcmp( argv[0], "kindred" ) == 0 ) kind = 1;
		else Assert( strcmp( argv[0], "members" ) == 0 );
	}
	else {
		Console().formattedWrite( "usage: %s Class [...]\n", argv[0] );
		return( NumberOfArguments );
	}

	for( int i = 1; i < argc; i++ ) {
		ClassStar aClass =
			StandardNameServer->lookup( argv[i], ClassClass );
		if( aClass != 0 ) {
			int instances = kind ? aClass->displayKindred()
					     : aClass->displayMembers();
			Console().formattedWrite( "%d instances.\n", instances );
		}
	}
	return( 0 );
}

int
instance( int argc, char *argv[] )
{
	if( argc > 1 ) {
		Assert( strcmp( argv[0], "instance" ) == 0 );
	}
	else {
		Console().formattedWrite( "usage: %s hex-adress(es)\n", argv[0] );
		return( NumberOfArguments );
	}

	for( int i = 1; i < argc; i++ ) {
		int address = 0;
		
		for( char * c = argv[i]; ; c++ ) {
			if( ( address == 0 ) && ( *c == 'x' ) ) {
			}
			else if( ( address == 0 ) && ( *c == 'X' ) ) {
			}
			else if( ( 'a' <= *c ) && ( *c <= 'f' ) ) {
				address *= 16;
				address += *c - 'a';
				address += 10;
			}
			else if( ( 'A' <= *c ) && ( *c <= 'F' ) ) {
				address *= 16;
				address += *c - 'A';
				address += 10;
			}
			else if( ( '0' <= *c ) && ( *c <= '9' ) ) {
				address *= 16;
				address += *c - '0';
			}
			else break;
		}
		ClassClass->displayMember( (ProxiableObject *)address );
	}
	return( 0 );
}
