#
# This file is part of the Choices Operating System
# Developed by: The TAPESTRY Parallel Computing Laboratory
#		 University of Illinois at Urbana-Champaign
#		 Department of Computer Science
#		 1304 W. Springfield Ave.
#		 Urbana, IL	61801
#
# Copyright (c) 1987, 1988, 1989, 1990, 1991 The University of Illinois 
# Board of Trustees.  All Rights Reserved.
#
# CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
#
# Author: David K. Raila (raila@cs.uiuc.edu)
# Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
#
# Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
#   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
#
# Common makefile for ARM-based Choices products
# This should not be invoked directly, but included in other makefiles.
#
#

PROCESSOR = ARM
ARCH =      arm
CROSS = arm-none-eabi-

CPP =  $(CROSS)cpp
CC =   $(CROSS)gcc
C++ =  $(CROSS)g++
LD =   $(CROSS)g++
NM =   $(CROSS)nm
SIZE = $(CROSS)size
AS =   $(CROSS)as
AR =   $(CROSS)ar
RANLIB = $(CROSS)ranlib
OBJDUMP = $(CROSS)objdump
OBJCOPY = $(CROSS)objcopy
GDB = $(CROSS)gdb
SED =	sed
FLEX =	flex
YACC =	byacc
INSTALL = 	install -c -s -m 775
GENCLASS = 	./genclass
CONSTCOLLECT =  NM=$(NM) $(TOP)/Configure/Tools/Misc/$(HOST)/Constcollect
PROXIFY = CPP="$(CPP)" $(TOP)/Configure/Tools/Proxify++/OMAP/Proxify++

#
# Various flags for cpp, c++, cc, and ld.
#
ifdef VERBOSE
VERBOSEFLAG = -v
endif

ifdef DEBUG
DEBUGFLAGS = -DDEBUG
endif

DEFINES =	$(OPTIONALDEFINES) $(DEBUGFLAGS) $(EXTRADEFINES) -DARM
ARMOPTS = -msoft-float -mpoke-function-name -mapcs -march=armv5te -mfpu=fpa -fnon-call-exceptions -fasynchronous-unwind-tables

CPPFLAGS = -undef -D__GNUC__ -D_POSIX_C_SOURCE -Darm
COPTS =	-g -pipe -Wall -O2 $(ARMOPTS) $(VERBOSEFLAG)

CFLAGS = $(COPTS) $(DEFINES) $(patsubst %,-iquote %,$(SPECIALHVPATH)) $(patsubst %,-iquote %,$(HVPATH)) 
#CFLAGS = $(COPTS) $(DEFINES) $(patsubst %,-I %,$(HVPATH)) $(patsubst %,-I %,$(SPECIALHVPATH))

C++FLAGS =	$(CFLAGS)
LDFLAGS = 	 $(CFLAGS) $(DEBUG_LD) -Wl,--wrap,malloc -Wl,--wrap,free $(EXTRALDFLAGS)
ASDEFINES = -DASM -D_ASM $(DEFINES)
ASFLAGS = -mno-fpu -gdwarf2 -march=armv5te

#
# Make a .o from C++ source.
#
ifdef NODEPENDENCIES
%.o: %.cc
	@echo "Compiling $< ";  \
	$(RM) $*.o; \
	$(C++) $(C++FLAGS) -c $<
else
%.o: %.cc
	@echo "Compiling $< ";  \
	$(RM) D/$*.dep $*.o; \
	> D/$*.dep; \
	DEPENDENCIES_OUTPUT="D/$*.dep $*.o";\
	export DEPENDENCIES_OUTPUT; \
	$(C++) $(C++FLAGS) -c $<
endif # NODEPENDENCIES

.PHONY: ALWAYS
#
# Pre-process a C++ source
#
%.cpp: %.cc ALWAYS
	@echo "Pre-processing $<";  \
	$(C++) -E $(C++FLAGS) -c $< > $*.cpp

#
# assemble a c++ file
#
%.s: %.cc ALWAYS
	@echo "Making $*.s from $<";  \
	$(C++) -S $(C++FLAGS) $< 

#
# assemble a c file
#
%.s: %.c ALWAYS
	@echo "Making $*.s from $<";  \
	$(CC) -S $(CFLAGS) $< 

#
# Make a .o from C source.
#
%.o: %.c
	@echo "Compiling $<"; \
	$(RM) D/$*.dep $*.o; \
	> D/$*.dep; \
	DEPENDENCIES_OUTPUT="D/$*.dep $*.o";\
	export DEPENDENCIES_OUTPUT; \
	$(CC) $(CFLAGS) -c $<

#
# Make a .o from assembler source.
#
%.o: %.s
	@echo "Assembling $<";  \
	$(RM) D/$*.dep $*..s $*.o; \
	> D/$*.dep; \
	DEPENDENCIES_OUTPUT="D/$*.dep $*.o"; \
	export DEPENDENCIES_OUTPUT; \
	$(CPP) $(CPPFLAGS) $(ASDEFINES) $< | $(AS) $(ASFLAGS) - -o $*.o

#
# Make a .cpp from assembler source.
#
%.cpp: %.s ALWAYS
	@echo "Pre-processing $< to $*.cpp";  \
	$(RM) $*.cpp
	> D/$*.dep; \
	DEPENDENCIES_OUTPUT="D/$*.dep $*.o"; \
	export DEPENDENCIES_OUTPUT; \
	$(CPP) $(CPPFLAGS) -DASM $< > $*.cpp

#
# Make a .cc from lex source.
#
%.lex.cc: %.l
	@echo "Flexing $<"; \
	$(RM) D/$*.dep; \
	(echo "$*.lex.cc: $<";echo "$*.lex.o: $*.lex.cc") >D/$*.dep; \
	$(FLEX) $<
	@mv lex.yy.c $*.lex.cc
#
# Make a .cc from yacc source.
#
%.tab.cc: %.y
	@echo "Yaccing $<"; \
	$(RM) D/$*.dep; \
	(echo "$*.tab.cc: $<";echo "$*.tab.o: $*.tab.cc") >D/$*.dep; \
	$(YACC) -dl $<
	mv y.tab.h $*.tab.h;
	mv y.tab.c $*.tab.cc
