#
# This file is part of the Choices Operating System
# Developed by: The TAPESTRY Parallel Computing Laboratory
#		 University of Illinois at Urbana-Champaign
#		 Department of Computer Science
#		 1304 W. Springfield Ave.
#		 Urbana, IL	61801
#
# Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
#	All Rights Reserved.
# CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
#
# Author: David W. Dykstra (dykstra@cs.uiuc.edu)
# Author: Peter W. Madany (madany@cs.uiuc.edu)
# Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
#
# Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
#   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
#
#
# makefile for FiSh under UNIX
# This should not be invoked directly, but included in other makefiles.
#
#	$Header: /projects/choices/old/cvsroot/choices/Configure/Applications/UNIXFiSh/FiShCommon.mk,v 1.1 2004/06/03 01:58:32 jcarlyle Exp $

all: FiSh 

UFISH = $(TOP)/Applications/UNIXFiSh
FISH = $(TOP)/Applications/FiSh
LIBS = $(TOP)/Configure/Libraries/GeneralPurpose/$(MACHINE)/libG.a

EXTRADEFINES = -DASSERT -DRAID -DUNIX 
EXTRAHVPATH = $(UFISH) $(FISH) $(TOP)/Includes/MachineDependent/Multimax \
	$(TOP)/Includes \
	$(TOP)/Includes/FileSystems \
	$(TOP)/Includes/Memory \
	$(TOP)/Includes/IODevices

EXTRAVPATH = $(UFISH) $(FISH) $(TOP)/MachineDependent/Multimax \
	$(TOP)/Kernel \
	$(TOP)/FileSystems \
	$(TOP)/Memory \
	$(TOP)/Common \
	$(TOP)/IODevices

include Config.mk

ifdef ArFile
ArIncludes=	ArContainer.cc ArDictionary.cc ArEntry.cc ArRootEntry.cc 
ArSources=	ArContainer.cc ArDictionary.cc ArEntry.cc ArRootEntry.cc 
endif

ifdef BSDFile
BSDIncludes=	BSDContainer.cc BSDDirectory.cc \
		BSDInode.cc HashedBSDDirectory.cc BSDAllocator.cc
BSDSources=	BSDContainer.cc BSDDirectory.cc \
		BSDInode.cc HashedBSDDirectory.cc BSDAllocator.cc
endif

ifdef COFFFile
COFFIncludes=	COFFContainer.cc COFFDictionary.cc 
COFFSources=	COFFContainer.cc COFFDictionary.cc 
endif

ifdef LogFile
LogIncludes=	LogContainer.cc LogInode.cc
LogSources=	LogContainer.cc LogInode.cc
endif

ifdef MadFile
MadIncludes=	MadContainer.cc MadFileNode.cc MadBlockAllocator.cc
MadSources=	MadContainer.cc MadFileNode.cc MadBlockAllocator.cc
endif

ifdef MailFile
MailIncludes=	MailContainer.cc MailDictionary.cc MailEntry.cc \
		MailRootEntry.cc MailMessageContainer.cc \
		MailMessageDictionary.cc
MailSources=	MailContainer.cc MailDictionary.cc MailEntry.cc \
		MailRootEntry.cc MailMessageContainer.cc \
		MailMessageDictionary.cc
endif

ifdef MSDOSFile
MSDOSIncludes=	MSDOSContainer.cc MSDOSDirectory.cc \
		MSDOSEntry.cc MSDOSRootEntry.cc MSDOSFAT.cc
MSDOSSources=	MSDOSContainer.cc MSDOSDirectory.cc \
		MSDOSEntry.cc MSDOSRootEntry.cc MSDOSFAT.cc
endif

ifdef SVIDFile
SVIDIncludes=	SVIDContainer.cc SVIDDirectory.cc SVIDVersionedDirectory.cc \
		SVIDInode.cc SVIDAllocator.cc
SVIDSources=	SVIDContainer.cc SVIDDirectory.cc SVIDVersionedDirectory.cc \
		SVIDInode.cc SVIDAllocator.cc
endif

ifdef NFSFile
NFSIncludes=	Server.cc NFSDictionary.cc NFSDisk.cc
NFSSources=	Server.cc NFSDictionary.cc NFSDisk.cc
endif

ifdef AIXFile
AIXIncludes=	AIXContainer.cc AIXInode.cc AIXAllocator.cc AIXDirectory.cc
AIXSources=	AIXContainer.cc AIXInode.cc AIXAllocator.cc AIXDirectory.cc
endif

INCLUDES=	Object.h Class.h ClassInstaller.h NameServer.h FileSystem.h \
	LOCK.h Semaphore.h MountItem.h DictionaryStack.h ClassConstructor.h \
	ProxiableObject.h StoredObject.h MemoryRange.h MemoryObject.h \
	PersistentMemoryObject.h FileObject.h MemoryObjectPartition.h \
	FormatAssists.h RootEntry.h \
	BlockAllocator.h MemoryObjectContainer.h MemoryObjectDictionary.h \
	FileStream.h ReadFileStream.h WriteFileStream.h SymbolicLink.h \
	FileSystemInterface.h MountTable.h Disk.h DiskContainer.h \
	MultimaxEMCDisk.h CRQ.h MultimaxInterrupts.h \
	Domain.h SystemInterface.h Process.h ThisProcess.h \
	REFERENCE_COUNT.h stdio.h Tools.h \
	ObjectProxy.h ObjectProxyManager.h InputStream.h OutputStream.h \
	UNIXInode.h UNIXContainer.h \
	COFFLoader.h COFF.h Ar.h \
	PersistentObject.h ProcessStar.h ClassConstructorStar.h \
	BlockAllocatorStar.h ClassStar.h FileStreamStar.h \
	FileSystemInterfaceStar.h LogContainerStar.h LogInodeStar.h \
	MSDOSFATStar.h MemoryObjectContainerStar.h \
	MemoryObjectDictionaryStar.h MemoryObjectStar.h MemoryRangeStar.h \
	MountTableStar.h OutputStreamStar.h PersistentMemoryObjectStar.h \
	NameServerStar.h PersistentObjectStar.h ProxiableObjectStar.h \
	ReadFileStreamStar.h StoredObjectStar.h SymbolicLinkStar.h \
	FileObjectStar.h WriteFileStreamStar.h \
	ProxyTableCopy.h ProxyTableCopyStar.h ProcessAssert.h \
	$(ArIncludes) $(BSDIncludes) $(COFFIncludes) \
	$(LogIncludes) $(MadIncludes) $(MailIncludes) $(MSDOSIncludes) \
	$(SVIDIncludes) $(AIXIncludes) $(NFSIncludes)

SOURCES=	Object.cc Class.cc ClassInstaller.cc NameServer.cc \
	FileSystem.cc MountItem.cc DictionaryStack.cc ClassConstructor.cc \
	ProxiableObject.cc StoredObject.cc MemoryRange.cc MemoryObject.cc \
	PersistentMemoryObject.cc FileObject.cc MemoryObjectPartition.cc \
	FormatAssists.cc RootEntry.cc \
	BlockAllocator.cc MemoryObjectContainer.cc MemoryObjectDictionary.cc \
	FileStream.cc ReadFileStream.cc WriteFileStream.cc SymbolicLink.cc \
	FileSystemInterface.cc MountTable.cc Disk.cc DiskContainer.cc \
	MultimaxEMCDisk.cc \
	UNIXInode.cc UNIXContainer.cc COFFLoader.cc \
	Other.cc Hierarchy.cc CreateFSI.cc Tools.cc Shell.cc \
	PersistentObject.cc ProcessStar.cc InputStream.cc \
	BlockAllocatorStar.cc ClassStar.cc FileStreamStar.cc \
	FileSystemInterfaceStar.cc LogContainerStar.cc LogInodeStar.cc \
	MSDOSFATStar.cc MemoryObjectContainerStar.cc ClassConstructorStar.cc \
	MemoryObjectDictionaryStar.cc MemoryObjectStar.cc MemoryRangeStar.cc \
	MountTableStar.cc OutputStreamStar.cc PersistentMemoryObjectStar.cc \
	NameServerStar.cc PersistentObjectStar.cc ProxiableObjectStar.cc \
	ReadFileStreamStar.cc StoredObjectStar.cc SymbolicLinkStar.cc \
	FileObjectStar.cc WriteFileStreamStar.cc SemaphoreStar.cc \
	ProxyTableCopyStar.cc ProxyTableCopy.cc ProcessAssert.cc \
	$(ArSources) $(BSDSources) $(COFFSources) \
	$(LogSources) $(MadSources) $(MailSources) $(MSDOSSources) \
	$(SVIDSources) $(AIXSources) $(NFSSources)

OBJECTS= $(addsuffix .o, $(basename $(SOURCES)))

include ../../ApplicationsCommon.mk

%.o: %.cc
	@echo "Compiling $<"; \
	$(RM) D/$*.dep $*.o; \
	> D/$*.dep; \
	DEPENDENCIES_OUTPUT="D/$*.dep $*.o";\
	export DEPENDENCIES_OUTPUT; \
	$(C++) -I- $(C++FLAGS) -c $< 

%.s: %.cc
	@echo "Compiling $<"; \
	$(C++) -I- $(C++FLAGS) -S $< 

FiSh: $(OBJECTS) $(LIBS)
	@echo "Linking FiSh"; \
	$(RM) _ctdt.[so]; touch _ctdt.s; \
	$(COLLECT) -o _ctdt.s $(OBJECTS) $(LIBS)
	@$(AS) -o _ctdt.o _ctdt.s
	@$(RM) FiSh; \
	$(CC) -o FiSh $(OBJECTS) _ctdt.o $(LIBS)

clobber: clobberFiSh
clobberFiSh:
	$(RM) FiSh

mad: makeMadFileSystem checkMadFileSystem

makeMadFileSystem: makeMadFileSystem.cc
	@echo Making makeMadFileSystem
	@$(C++) -I- $(C++FLAGS) -o makeMadFileSystem makeMadFileSystem.cc

checkMadFileSystem: checkMadFileSystem.cc
	@echo Making checkMadFileSystem
	@$(C++) -I- $(C++FLAGS) -o checkMadFileSystem checkMadFileSystem.cc


LEFTOVERS = bsd4 bsd8 checkMadFileSystem demo fscks load loadscript log \
	mad makeMadFileSystem mm msdos script svid x.ar passwd

clean: localClean

localClean:
	$(RM) $(LEFTOVERS)
