#
# This file is part of the Choices Operating System
# Developed by: The TAPESTRY Parallel Computing Laboratory
#		 University of Illinois at Urbana-Champaign
#		 Department of Computer Science
#		 1304 W. Springfield Ave.
#		 Urbana, IL	61801
#
# Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
#	All Rights Reserved.
# CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
#
# Author: Aamod Sane (sane@cs.uiuc.edu)
# Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
#
# Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
#   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
#
#
# makefile for VM under UNIX
# This should not be invoked directly, but included in other makefiles.
#

ifndef $(PROG)
PROG=VM
endif

ALLCLASSES=/mnt/tapestry/Tools/Bin/allClasses
EXTRACOPTS=-g


all: $(PROG)

# FISH = $(TOP)/Applications/FiSh
UVM = $(TOP)/Applications/UNIXVM
LIBS = $(TOP)/Configure/Libraries/GeneralPurpose/$(MACHINE)/libG.a

EXTRADEFINES = -DASSERT -DRAID -DUNIX 
EXTRAHVPATH = $(UVM) $(TOP)/Includes/MachineDependent/Multimax
EXTRAVPATH = $(UVM) $(TOP)/MachineDependent/Multimax

include Config.mk

# Unixification include files

INCLUDEU = stdio.h Other.h

# Kernel include files (Includes/)

INCLUDEK = Object.h Class.h NameServer.h \
	ObjectProxy.h ObjectProxyManager.h InputStream.h OutputStream.h 

INCLUDEF = NameServer.h FileSystem.h \
	LOCK.h Semaphore.h MountItem.h DictionaryStack.h \
	ProxiableObject.h StoredObject.h MemoryRange.h MemoryObject.h \
	PersistentMemoryObject.h FileObject.h MemoryObjectPartition.h \
	RootEntry.h MemoryObjectCache.h \
	PhysicalMemoryObject.h SOURCE_OF_ZEROS_MEMORY_OBJECT.h \
	BlockAllocator.h MemoryObjectContainer.h MemoryObjectDictionary.h \
	FileStream.h ReadFileStream.h WriteFileStream.h SymbolicLink.h \
	FileSystemInterface.h MountTable.h \
	MultimaxEMCDisk.h MultimaxEMCContainer.h CRQ.h MultimaxInterrupts.h \
	Domain.h Kernel.h PhysicalMemoryChain.h Process.h \
	REFERENCE_COUNT.h Tools.h \
	ObjectProxy.h ObjectProxyManager.h OutputStream.h \
	UNIXInode.h UNIXContainer.h \
	BSDInode.h BSDContainer.h BSDAllocator.h BSDDirectory.h \
	HashedBSDDirectory.h \
	MadContainer.h MadFileNode.h MadBlockAllocator.h \
	SVIDInode.h SVIDContainer.h SVIDAllocator.h SVIDDirectory.h \
	COFFLoader.h COFF.h COFFContainer.h COFFDictionary.h \
	Ar.h ArContainer.h ArDictionary.h ArEntry.h ArRootEntry.h \
	MailContainer.h MailDictionary.h MailEntry.h MailRootEntry.h \
	MailMessageContainer.h MailMessageDictionary.h \
	LogInode.h LogContainer.h \
	MSDOSEntry.h MSDOSRootEntry.h \
	MSDOSContainer.h MSDOSDirectory.h MSDOSFAT.h \
	Reference.h RefThing.h PersistentObject.h Thing.h \
	AIXContainer.h AIXInode.h AIXAllocator.h AIXDirectory.h

# Portable include files (Includes/Libraries/GeneralPurpose/)

INCLUDEP = Link.h LinkIterator.h List.h LightList.h HashTable.h \
	MultiHashTable.h

COMMONINCLUDES = $(INCLUDEU) $(INCLUDEK) $(INCLUDEP)

INCLUDES= $(COMMONINCLUDES)

# Special unixification files

SOURCEU = Other.cc

# Kernel source files ()

SOURCEK = Object.cc Class.cc NameServer.cc ProxiableObject.cc

# Portable source files (Libraries/GeneralPurpose/)

# SOURCEP = Link.cc LinkIterator.cc List.cc LightList.cc \
# HashTable.cc MultiHashTable.cc

SOURCEF = FileSystem.cc \
	MountItem.cc DictionaryStack.cc \
	ProxiableObject.cc StoredObject.cc MemoryRange.cc MemoryObject.cc \
	PersistentMemoryObject.cc FileObject.cc MemoryObjectPartition.cc \
	RootEntry.cc MemoryObjectCache.cc \
	PhysicalMemoryObject.cc SOURCE_OF_ZEROS_MEMORY_OBJECT.cc \
	BlockAllocator.cc MemoryObjectContainer.cc MemoryObjectDictionary.cc \
	FileStream.cc ReadFileStream.cc WriteFileStream.cc SymbolicLink.cc \
	FileSystemInterface.cc MountTable.cc \
	MultimaxEMCDisk.cc MultimaxEMCContainer.cc \
	UNIXInode.cc UNIXContainer.cc \

SOURCEOPTIONAL = BSDInode.cc BSDContainer.cc BSDAllocator.cc BSDDirectory.cc \
	HashedBSDDirectory.cc \
	MadContainer.cc MadFileNode.cc MadBlockAllocator.cc \
	SVIDInode.cc SVIDContainer.cc SVIDAllocator.cc SVIDDirectory.cc \
	COFFLoader.cc COFFContainer.cc COFFDictionary.cc \
	ArContainer.cc ArDictionary.cc ArEntry.cc ArRootEntry.cc \
	MailContainer.cc MailDictionary.cc MailEntry.cc MailRootEntry.cc \
	MailMessageContainer.cc MailMessageDictionary.cc \
	LogInode.cc LogContainer.cc \
	MSDOSEntry.cc MSDOSRootEntry.cc \
	MSDOSContainer.cc MSDOSDirectory.cc MSDOSFAT.cc \
	Reference.cc RefThing.cc PersistentObject.cc Thing.cc \
	AIXContainer.cc AIXInode.cc AIXAllocator.cc AIXDirectory.cc


COMMONSOURCES = $(SOURCEK) $(SOURCEU)

include ../../ApplicationsCommon.mk

Ar.o: ArContainer.o ArDictionary.o ArEntry.o ArRootEntry.o 
	ld -r -o Ar.o ArContainer.o ArDictionary.o ArEntry.o ArRootEntry.o 

ifdef ArFile
ArFiles=	ArContainer.o ArDictionary.o ArEntry.o ArRootEntry.o 
endif

ifdef BSDFile
BSDFiles=	BSDContainer.o BSDDirectory.o \
		BSDInode.o HashedBSDDirectory.o BSDAllocator.o
endif

ifdef COFFFile
COFFFiles=	COFFContainer.o COFFDictionary.o 
endif

ifdef LogFile
LogFiles=	LogContainer.o LogInode.o
endif

ifdef MadFile
MadFiles=	MadContainer.o MadFileNode.o MadBlockAllocator.o
endif

ifdef MailFile
MailFiles=	MailContainer.o MailDictionary.o MailEntry.o MailRootEntry.o \
		MailMessageContainer.o MailMessageDictionary.o
endif

ifdef MSDOSFile
MSDOSFiles=	MSDOSContainer.o MSDOSDirectory.o \
		MSDOSEntry.o MSDOSRootEntry.o MSDOSFAT.o
endif

ifdef SVIDFile
SVIDFiles=	SVIDContainer.o SVIDDirectory.o \
		SVIDInode.o SVIDAllocator.o
endif

ifdef POFile
POFiles=	Reference.o RefThing.o PersistentObject.o Thing.o
endif

ifdef AIXFile
AIXFiles=	AIXContainer.o AIXInode.o AIXAllocator.o AIXDirectory.o
endif

SOURCES = $(COMMONSOURCES)

OBJECTS = $(addsuffix .o, $(basename $(SOURCES))) \
	$(ArFiles) $(BSDFiles) $(COFFFiles) \
	$(LogFiles) $(MadFiles) $(MailFiles) $(MSDOSFiles) \
	$(POFiles) $(SVIDFiles) $(AIXFiles) $(DiskFiles) \
        $(RAMDiskFiles) $(OptionalSchedulerFiles)

DEPFILES = $(EXTRADEPFILES) $(SOURCES) ClassInstantiation.cc

%.o: %.cc
	@echo "Compiling $<"; \
	$(RM) D/$*.dep $*.o; \
	> D/$*.dep; \
	DEPENDENCIES_OUTPUT="D/$*.dep $*.o";\
	export DEPENDENCIES_OUTPUT; \
	$(C++) -I- $(C++FLAGS) -c $< 

%.s: %.cc
	@echo "Compiling $<"; \
	$(C++) -I- $(C++FLAGS) -S $< 

$(PROG): $(PROG).o $(OBJECTS) $(LIBS)
	@echo Linking $(PROG); \
	echo Linking $(PROG).o; \
	$(RM) $(PROG); \
	$(C++) -o $(PROG) $(PROG).o $(OBJECTS) $(LIBS)

clobber: clobberVM
clobberVM:
	$(RM) VM
#
# Create First Class Classes code by reading all include files.
#
Class.o: ClassInstantiation.cc

ClassInstantiation.cc: ClassIncludes.cc Config.mk
	@echo Creating definitions of Class objects; \
	$(RM) D/ClassInstantiation.dep;\
	DEPENDENCIES_OUTPUT="D/ClassInstantiation.dep ClassInstantiation.cc";\
	export DEPENDENCIES_OUTPUT; > D/ClassInstantiation.dep; \
	$(C++) -I- $(C++FLAGS) -E ClassIncludes.cc | $(ALLCLASSES)

ClassDefinitions.cc:
	@echo "ClassDefinitions.cc not found, continuing"; \
	echo $(ALLCLASSES)

ClassIncludes.cc: Makefile ../VMCommon.mk
	@echo Creating ClassIncludes.cc; \
	$(RM) ClassIncludes.cc; \
	echo $(INCLUDES)|tr ' ' '\012'|sed 's/^/#include "/;s/$$/"/' >ClassIncludes.cc
