#
# This file is part of the Choices Operating System
# Developed by: The TAPESTRY Parallel Computing Laboratory
#		 University of Illinois at Urbana-Champaign
#		 Department of Computer Science
#		 1304 W. Springfield Ave.
#		 Urbana, IL	61801
#
# Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
#	All Rights Reserved.
# CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
#
# Author: David W. Dykstra (dykstra@cs.uiuc.edu)
# Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
#
# Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
#   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
#
#
# Common makefile for all Choices products.
# This should not be invoked directly, but included in other makefiles.
#
#	$Header: /projects/choices/old/cvsroot/choices/Configure/ChoicesCommon.mk,v 1.4 2004/08/18 08:01:38 jcarlyle Exp $
#

.SUFFIXES:

ifndef TOP
ifdef CHOICESNODE
TOP = $(CHOICESNODE)
else
TOP = ../../..
endif # CHOICESNODE
endif # TOP

echohvpath:
	echo $(HVPATH)

echoccvpath:
	echo $(CCVPATH)

PRINT =	lpr
RM = rm -f
MV = mv

#
# Where to find all the include files.
#

STANDARDHVPATH += \
	$(TOP)/Configure/System/$(MACHINE) \
	$(TOP)/Includes/Common \
	$(TOP)/Configure/System/$(MACHINE) \
	$(TOP)/Includes/Libraries/GeneralPurpose
#
HVPATH += . \
	$(EXTRAHVPATH) \
	$(STANDARDHVPATH) \
	$(COMMONHVPATH) \
	$(OPTIONSHVPATH) \
	$(PERSONALHVPATH)
#
HVPATH += . \
#
# Where to find all the source files.
#
CCVPATH += . \
	$(OPTIONSVPATH) \
	$(EXTRAVPATH) \
	$(COMMONCCVPATH) \
	$(PERSONALCCVPATH)

space += $(empty) $(empty)
VPATH += $(subst $(space),:,$(CCVPATH))

#
# This rule is here to prevent the first common rule from being called by
#   default if the machine-dependent makefile doesn't supply a default rule.
#
choices_default:
	@echo "No Default Rule"


#
# Only warn if a .h file does not exist.  This prevents gnumake from
#  complaining when a file name gets changed and the old name is still
#  referenced in a .dep file.
#
%.h:
	@echo "$@ not found, continuing"

#
# Forget about these rules (for speed)
#
%.mk: ;
Makefile: ;

.PHONY: all

% : RCS/%,v
% : %,v
% : SCCS/s.%
% : s.%

#
# Empty command for .dep
#
%.dep: ;


ifndef NODEPENDENCIES
#
# Rule for generating dependencies
# 
Make.dependencies: D/.exists Makefile
	@echo "Creating Make.dependencies"
	@$(RM) Make.dependencies
#
# The following line adds dependencies to 'Make.dependencies' of all
#  included files in 'Makefile' and all the files they include.  This
#  only works for two levels but that is sufficient for us because
#  'DEPFILES' is never modified more than two levels above the bottom.
#
	@sed -n '/Make.dependencies/d; s/^include /Make.dependencies: /p' \
				Makefile >Make.dependencies
	@INCS="`sed -n '/Make.dependencies/d; s/^include //p' Makefile`"; \
	     sed -n '/Make.dependencies/d; s/^include /Make.dependencies: /p' \
				$$INCS >>Make.dependencies
#
# Now add the include commands for all the 'DEPFILES' and 'DEPFILES2'
#
	@echo $(basename $(DEPFILES)) | tr ' ' '\012' | \
	    awk '/..*/{ print "include D/"$$1".dep" }' >>Make.dependencies
	@echo $(basename $(DEPFILES2)) | tr ' ' '\012' | \
	    awk '/..*/{ print "include D/"$$1".dep" }' >>Make.dependencies
	@echo "Finished creating Make.dependencies"

include Make.dependencies
endif # NODEPENDENICES

# we want a D directory
D/.exists:
	mkdir -p D && touch D/.exists

#
# Rules for cleaning up
# Including makefiles can add other rules as dependencies to clean and
#   clobber to clean up other things.
# 
clean:
	$(RM) *.map *.out core *_ctdt.s *..c Choices Choices.bin
	$(RM) [_a-m]*.[ilyCSso] [A-M]*.[ilyCSso] 
	$(RM) [_n-z]*.[ilyCSso] [N-Z]*.[ilyCSso] 

clobber: clean
	#$(RM) D/[_a-m]* D/[A-M]*
	#$(RM) D/[_n-z]* D/[N-Z]*
	$(RM) D/*.dep
	$(RM) Make.dependencies



##############################################################################
#
# Make a class browser file for ebrowse
#
##############################################################################
EFAKES += $(addsuffix .b, $(SOURCES) $(INCLUDES))

%.b: %
	echo $< >> browsefiles

.PHONY: ebrowse
ebrowse:
	-$(MAKE) -k $(EFAKES)
	ebrowse -f browsefiles
	rm -f browsefiles

clobber: clobberebrowse
clobberebrowse:
	$(RM) BROWSE browsefiles
#############################################################################

##############################################################################
#
# Make a etags file for emacs
#
##############################################################################
TFAKES += $(addsuffix .t, $(SOURCES) $(INCLUDES))

%.t: %
	echo $< >> tagfiles

.PHONY: etags
etags:
	-$(MAKE) -k $(TFAKES)
	etags `cat tagfiles`
	rm -f tagfiles

clobber: clobbertags
clobbertags:
	$(RM) tagfiles etags TAGS
#############################################################################

##############################################################################
#
# codemtrics
#
##############################################################################
MSRCFAKES += $(addsuffix .ms, $(SOURCES) )
MINCFAKES += $(addsuffix .mi, $(INCLUDES) )

%.ms: %
	@echo $< >> msfiles

%.mi: %
	@echo $< >> mifiles

.PHONY: codecount
codecount:
	@-$(MAKE) -k $(MSRCFAKES) $(MINCFAKES)
	echo -n "Includes: "
	wc -l mifiles
	cat `cat mifiles` | wc 
	echo -n "Sources: "
	wc -l msfiles
	cat `cat msfiles` | wc
	echo -n "Total: "
	cat `cat mifiles msfiles` | wc
	$(RM) msfiles mifiles

clobber: clobbercount
clobbercount:
	$(RM) msfiles mifiles
#############################################################################

