#
# This file is part of the Choices Operating System
# Developed by: The TAPESTRY Parallel Computing Laboratory
#		 University of Illinois at Urbana-Champaign
#		 Department of Computer Science
#		 1304 W. Springfield Ave.
#		 Urbana, IL	61801
#
# Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
# The University of Illinois Board of Trustees.
#	All Rights Reserved.
# CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
#
# Author: Swee Boon Lim (sblim@cs.uiuc.edu)
# Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
#
# Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
#   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
#
#
# makefile for the UnixApplication library
# This should not be invoked directly, but included in other makefiles.
#
#	$Header: /projects/choices/old/cvsroot/choices/Configure/Libraries/Unix/UnixCommon.mk,v 1.1 2004/06/03 01:58:32 jcarlyle Exp $

all:	libU.a libUM.a libUCompat.a 

EXTRADEFINES = -nostdinc

CHOICESINCDIR = $(TOP)/Includes/Libraries/Unix/include/choices $(TOP)/Includes/Libraries/SystemInterface/$(PROCESSOR)
ifdef VCHOICES
ADDTLDIR = $(TOP)/Libraries/Unix/libc/choices/SPARC
endif
CHOICESSRCDIR = $(TOP)/Libraries/Unix/libc/choices/Portable \
	$(TOP)/Libraries/Unix/libc/choices/$(PROCESSOR) $(ADDTLDIR)
CHOICESSOURCES = __bssinit.c atof.c strtod.c \
	$(CHOICESPROCESSORDEPENDENTSOURCES)
CHOICESOBJECTS = $(addsuffix .o, $(basename $(CHOICESSOURCES)))

STDIOSRCDIR = $(TOP)/Libraries/Unix/libc/stdio
STDIOSOURCES = clrerr.c doprnt.c doscan.c fdopen.c fgetc.c filbuf.c findiop.c \
        flsbuf.c fopen.c fprintf.c fputc.c fread.c freopen.c fseek.c \
        ftell.c fwrite.c getchar.c gets.c getw.c printf.c putchar.c \
        putw.c rew.c scanf.c setbuf.c setbuffer.c sprintf.c ungetc.c \
        vfprintf.c vprintf.c vsprintf.c \
	fgets.c fputs.c puts.c
STDIOOBJECTS = $(addsuffix .o, $(basename $(STDIOSOURCES)))

STDLIBSRCDIR = $(TOP)/Libraries/Unix/libc/stdlib
STDLIBSOURCES= abort.c atexit.c atoi.c atol.c bsearch.c calloc.c div.c exit.c \
        getenv.c labs.c ldiv.c malloc.c putenv.c qsort.c rand.c setenv.c \
        strtol.c strtoul.c system.c \
	abs.c 
STDLIBNOTUSED = atof.c
STDLIBOBJECTS = $(addsuffix .o, $(basename $(STDLIBSOURCES)))

STRINGSRCDIR = $(TOP)/Libraries/Unix/libc/string
STRINGSOURCES =  memccpy.c strcasecmp.c strcoll.c strdup.c strerror.c \
        strftime.c strmode.c strtok.c strxfrm.c swab.c \
	bcmp.c bcopy.c bzero.s ffs.c index.c memchr.c memcmp.c memset.c \
        rindex.c strcat.c strcmp.c strcpy.c strcspn.c strlen.c \
        strncat.c strncmp.c strncpy.c strpbrk.c strsep.c \
        strspn.c strstr.c \
	memmove.c memcpy.c strchr.c strrchr.c
STRINGOBJECTS = $(addsuffix .o, $(basename $(STRINGSOURCES)))

GENSRCDIR = $(TOP)/Libraries/Unix/libc/gen \
	$(TOP)/Libraries/Unix/libc/gen/regexp
GENSOURCES = alarm.c clock.c closedir.c crypt.c ctime.c ctype_.c difftime.c \
        errlst.c execvp.c fnmatch.c fstab.c fts.c \
        getgrent.c getlogin.c getmntinfo.c getopt.c getpass.c \
        getpwent.c getttyent.c getusershell.c getwd.c glob.c initgroups.c \
        isatty.c isctype.c mktemp.c ndbm.c nice.c nlist.c opendir.c \
        pause.c perror.c popen.c psignal.c random.c readdir.c regex.c \
        remove.c rewinddir.c scandir.c seekdir.c setjmperr.c setmode.c \
        setrgid.c setruid.c siginterrupt.c siglist.c signal.c \
        sigsetops.c sleep.c syslog.c telldir.c termios.c time.c timezone.c \
        tmpnam.c ttyname.c ttyslot.c ualarm.c unvis.c usleep.c valloc.c vis.c \
        wait.c wait3.c waitpid.c \
	regerror.c regexp.c regsub.c
GENNOTUSED = disklabel.c fakcu.c sigsetjmp.c
GENOBJECTS = $(addsuffix .o, $(basename $(GENSOURCES)))

LIBUTILSRCDIR = $(TOP)/Libraries/Unix/libutil
LIBUTILSOURCES = pwcache.c
LIBUTILOBJECTS = $(addsuffix .o, $(basename $(LIBUTILSOURCES))

LIBUCOMPATSRCDIR = $(TOP)/Libraries/Unix/libcompat/4.1 \
	$(TOP)/Libraries/Unix/libcompat/4.3
LIBUCOMPATSOURCES = ftime.c getpw.c gtty.c stty.c \
	times.c utime.c vlimit.c vtimes.c \
	ecvt.c gcvt.c lsearch.c ruserpass.c sibuf.c sobuf.c strout.c
LIBUCOMPATNOTUSED = rexec.c tell.c
LIBUCOMPATOBJECTS = $(addsuffix .o, $(basename $(LIBUCOMPATSOURCES)))

SRCDIR = $(TOP)/Libraries/Unix/Emulation $(CHOICESSRCDIR) \
	$(STDIOSRCDIR) $(STRINGSRCDIR) $(STDLIBSRCDIR) \
        $(GENSRCDIR) $(LIBUTILSRCDIR) $(DBHASHSRCDIR) \
	$(LIBUMSRCDIR) $(LIBUCOMPATSRCDIR)

INCDIR = $(TOP)/Includes/Libraries/Unix/Emulation \
	$(STDIOSRCDIR) $(STRINGSRCDIR) $(STDLIBSRCDIR) \
	$(GENSRCDIR) $(DBHASHSRCDIR) $(LIBUMSRCDIR) \
	$(LIBUCOMPATSRCDIR) 

COMDIR = $(TOP)/Common
CONFSYSTEM = $(TOP)/Configure/System/$(MACHINE)
INCSYSTEM = $(TOP)/Includes/System/$(MACHINE) 
INCMACHDEP = $(TOP)/Includes/MachineDependent/$(MACHINE)

EXTRAVPATH = $(SRCDIR) $(COMDIR)
EXTRAHVPATH = $(INCDIR) $(TOP)/Includes/Libraries/SystemInterface/Portable \
    $(INCSYSTEM) $(INCMACHDEP)
SPECIALHVPATH = $(CHOICESINCDIR) \
	../../../../Includes/Libraries/Unix/include \
	$(TOP)/Includes/Libraries/Unix/include/sys 

#
# List includes from the $(INCDIR) directory here
#
INCLUDES = ExecInfo.h FileDescriptor.h SysCalls.h \
	UnixConstants.h UnixApplication.h

#
# List sources from the $(SRCDIR) directory here
#
PORTSOURCES = ExecInfo.cc FileDescriptor.cc UnixApplication.cc \
	access.cc chdir.cc chmod.cc chown.cc chroot.cc close.cc creat.cc \
	dup.cc dup2.cc execve.cc _exit.cc fcntl.cc fchdir.cc fchmod.cc \
	fchown.cc fchroot.cc flock.cc fork.cc fstat.cc fsync.cc \
	ftruncate.cc getdirentries.cc getdtablesize.cc getegid.cc \
	geteuid.cc getgid.cc getgroups.cc getpagesize.cc getpid.cc \
	getrusage.cc gettimeofday.cc getuid.cc \
	ioctl.cc link.cc lseek.cc lstat.cc mkdir.cc mkfifo.cc mknod.cc \
	open.cc pipe.cc read.cc readlink.cc readv.cc rename.cc rmdir.cc \
	sbrk.cc setgroups.cc setitimer.cc settimeofday.cc sigaction.cc \
	sigblock.cc sigpause.cc \
	sigprocmask.cc stat.cc symlink.cc \
	sync.cc tell.cc truncate.cc unlink.cc umask.cc utimes.cc vfork.cc \
	wait4.cc write.cc writev.cc 

#
# List full path of sources copied from system here
#

LIBUSOURCES = $(PORTSOURCES) $(EXTRASOURCES) \
	$(CHOICESSOURCES) $(STDIOSOURCES) $(STRINGSOURCES) \
	$(STDLIBSOURCES) $(GENSOURCES) $(LIBUTILSOURCES) \
	$(DBHASHSOURCES)
LIBUOBJECTS = $(addsuffix .o, $(basename $(LIBUSOURCES)))

#
# Machine dependent makefiles must define LIBUMSRCDIR and LIBUMSOURCES
#

LIBUMOBJECTS = $(addsuffix .o, $(basename $(LIBUMSOURCES)))

SOURCES = $(LIBCSOURCES) $(LIBUMSOURCES) $(LIBUCOMPATSOURCES)

DEPFILES = $(SOURCES)

include ../../LibrariesCommon.mk

libU.a: $(LIBUOBJECTS)
	$(RM) libU.a
	$(AR) cr libU.a $(LIBUOBJECTS)
ifdef RANLIB
	$(RANLIB) libU.a
endif

libUM.a: $(LIBUMOBJECTS)
	$(RM) libUM.a
	$(AR) cr libUM.a $(LIBUMOBJECTS)
ifdef RANLIB
	$(RANLIB) libUM.a
endif

libUCompat.a: $(LIBUCOMPATOBJECTS)
	$(RM) libUCompat.a
	$(AR) cr libUCompat.a $(LIBUCOMPATOBJECTS)
ifdef RANLIB
	$(RANLIB) libUCompat.a
endif


clobber: localClobber

localClobber:
	$(RM) libUCompat.a $(LIBUCOMPATOBJECTS)




