#
# This file is part of the Choices Operating System
# Developed by: The TAPESTRY Parallel Computing Laboratory
#		 University of Illinois at Urbana-Champaign
#		 Department of Computer Science
#		 1304 W. Springfield Ave.
#		 Urbana, IL	61801
#
# Copyright (c) 1987, 1988, 1989, 1990, 1991, 2005
# The University of Illinois Board of Trustees.
#	All Rights Reserved.
# CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
#
# Author: David W. Dykstra (dykstra@cs.uiuc.edu)
# Contributing Author: Jeffrey C. Carlyle (jcarlyle@cs.uiuc.edu)
# Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
#
# Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
#   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
#
#
# Common makefile for FileSystems portion of Choices
#
#

#
# Optional sources/includes
#

ifdef CramfsFile
CramfsIncludes = CramfsFileObject.h CramfsContainer.h CramfsDictionary.h \
		 Cramfs.h crc32.h zconf.h zutil.h inffixed.h inftrees.h \
		 zlib.h inffast.h inflate.h CramfsDirObject.h
CramfsSources = CramfsFileObject.cc CramfsContainer.cc CramfsDictionary.cc \
		inflate.c uncompr.c zutil.c adler32.c crc32.c inffast.c \
		inftrees.c CramfsDirObject.cc
CramfsDefines = -DCramfsFile
DiskFile=yes
COMMONHVPATH += $(TOP)/Includes/FileSystems/zlib
COMMONCCVPATH += $(TOP)/FileSystems/zlib
endif # CramfsFile

ifdef Ext2File
Ext2Includes = Ext2Container.h Ext2Directory.h \
                Ext2Inode.h Ext2Allocator.h
Ext2Sources =  Ext2Container.cc Ext2Directory.cc \
                Ext2Inode.cc Ext2Allocator.cc
Ext2Defines = -DExt2File
DiskFile=Yes
endif

ifdef ArFile
ArIncludes=     Ar.h ArContainer.h ArDictionary.h ArEntry.h ArRootEntry.h
ArSources=      ArContainer.cc ArDictionary.cc ArEntry.cc ArRootEntry.cc
ArFileDefines = -DArFile
DiskFile=Yes
endif

ifdef BSDFile
BSDIncludes=    BSDContainer.h BSDDirectory.h \
		BSDInode.h HashedBSDDirectory.h BSDAllocator.h
BSDSources=     BSDContainer.cc BSDDirectory.cc \
		BSDInode.cc HashedBSDDirectory.cc BSDAllocator.cc
BSDFileDefines = 	-DBsdFile
DiskFile=Yes
endif

ifdef COFFFile
COFFIncludes=   COFFContainer.h COFFDictionary.h
COFFSources=    COFFContainer.cc COFFDictionary.cc
COFFFileDefines= -DCoffFile
DiskFile=Yes
endif

ifdef ELFFile
#Added for ELF file support (LKM adaptor for choices project)
ELFIncludes = ELFContainer.h ELFDictionary.h ELFLoader.h
ELFSources = ELFContainer.cc ELFDictionary.cc ELFLoader.cc
ELFStars = ELFDictionaryStar.cc ELFContainerStar.cc
ELFStarIncludes = ELFDictionaryStar.h ELFContainerStar.h
DiskFile=Yes
endif

ifdef LKMAdaptor
#Added for LKM adaptor support
LKMIncludes = LKMAdaptor.h HelloWorldDevice.h HelloWorldLKM.h EthernetDriverLKM.h
LKMSources = LKMAdaptor.cc HelloWorldLKM.cc EthernetDriverLKM.cc
DiskFile=Yes
endif

ifdef COFFLoader
COFFIncludes=    COFFLoader.cc
COFFLoaderSources=    COFFLoader.cc
CoffLoaderDefines = -DCoffFile -DCODELOADER
DiskFile=Yes
endif

ifdef LogFile
LogIncludes=    LogContainer.h LogInode.h LogContainerStar.h
LogSources=     LogContainer.cc LogInode.cc LogContainerStar.cc
LogStars=       LogInodeStar.cc
LogStarIncludes=	LogInodeStar.h
LogFileDefines = -DLogFile
DiskFile=Yes
endif

ifdef MadFile
MadIncludes=    MadContainer.h MadFileNode.h MadBlockAllocator.h \
					MadFileSystem.cc
MadSources=     MadContainer.cc MadFileNode.cc MadBlockAllocator.cc
MadFileDefines = -DMadFile
DiskFile=Yes
endif

ifdef MailFile
MailIncludes=   MailContainer.h MailDictionary.h MailEntry.h \
		MailRootEntry.h MailMessageContainer.h \
		MailMessageDictionary.h
MailSources=    MailContainer.cc MailDictionary.cc MailEntry.cc \
		MailRootEntry.cc MailMessageContainer.cc \
		MailMessageDictionary.cc
MailFileDefines = -DMailFile
DiskFile=Yes
endif

ifdef MSDOSFile
MSDOSSources=   MSDOSContainer.h MSDOSDirectory.h \
		MSDOSEntry.h MSDOSRootEntry.h MSDOSFAT.h
MSDOSSources=   MSDOSContainer.cc MSDOSDirectory.cc \
		MSDOSEntry.cc MSDOSRootEntry.cc MSDOSFAT.cc
MSDOSStars=   MSDOSFATStar.cc
MSDOSStarIncludes=   MSDOSFATStar.h
MsDosFileDefines = -DMsDosFile
DiskFile=Yes
endif

ifdef SVIDFile
SVIDIncludes=   SVIDContainer.h SVIDDirectory.h SVIDVersionedDirectory.h \
		SVIDInode.h SVIDAllocator.h
SVIDSources=    SVIDContainer.cc SVIDDirectory.cc SVIDVersionedDirectory.cc \
		SVIDInode.cc SVIDAllocator.cc
SVIDFileDefines = -DSVIDFile
DiskFile=Yes
endif

ifdef AIXFile
AIXIncludes=    AIXContainer.h AIXDirectory.h AIXAllocator.h AIXInode.h
AIXSources=     AIXContainer.cc AIXDirectory.cc AIXAllocator.cc AIXInode.cc
AIXFileDefines = -DAIXFile
DiskFile=Yes
endif

ifdef TarFile
TarIncludes=    Tar.h TarContainer.h TarDictionary.h TarEntry.h TarRootEntry.h
TarSources=     TarContainer.cc TarDictionary.cc TarEntry.cc TarRootEntry.cc
TarFileDefines = -DTarFile
DiskFile=Yes
endif

ifdef RAMDiskFile
RAMDiskSources= RAMDisk.h
RAMDiskSources= RAMDisk.cc
DiskFile=Yes
endif

ifdef RecordFile
ifndef IndexedRecordFile
RecordIncludes = AutoloadPersistentObject.h RecordFile.h FixedRecordFile.h \
	RelativeRecordFile.h VariableRecordFile.h
else
RecordIncludes = AutoloadPersistentObject.h RecordFile.h FixedRecordFile.h \
	RelativeRecordFile.h IndexedRecordFile.h VariableRecordFile.h
endif
RecordSources = AutoloadPersistentObject.cc RecordFile.cc \
	FixedRecordFile.cc  RelativeRecordFile.cc IndexedRecordFile.cc \
	VariableRecordFile.cc
RecordStars = RecordStreamStar.cc FixedRecordFileStar.cc \
	FixedRecordStreamStar.cc VariableRecordFileStar.cc \
	VariableRecordStreamStar.cc RelativeRecordFileStar.cc \
	RecordFileStar.cc IndexedRecordFileStar.cc
RecordStarIncludes = RecordStreamStar.h FixedRecordFileStar.h \
	FixedRecordStreamStar.h VariableRecordFileStar.h \
	VariableRecordStreamStar.h  AutoloadPersistentObjectStar.h \
	RelativeRecordFileStar.h RecordFileStar.h
DiskFile=Yes
endif

ifdef POSHFile
POSHIncludes  =  POSHExpression.h POSHBuiltin.h POSHVariable.h \
	POSHFileNode.h POSHExpressionList.h POSHIfStatement.h POSHApply.h \
	POSHContainer.h
POSHSources   =  POSHExpression.cc POSHBuiltin.cc POSHVariable.cc \
	POSHFileNode.cc POSHExpressionList.cc POSHIfStatement.cc \
	POSHApply.cc POSHString.cc POSHInteger.cc POSHLambda.cc \
	POSHContainer.cc POSHBlockAllocator.cc
DiskFile=Yes
endif

ifdef RemoteFileSystem
RemoteFileSystemIncludes = RemoteFileSystem.h RemoteFileSustemRPC.h \
	RemoteFileSystemMessages.h \
	RemoteContainer.h RemoteDictionary.h RemoteFileObject.h \
	RemoteServer.h RemoteTables.h ResourceLock.h
RemoteFileSystemSources = RemoteFileSystem.cc RemoteFileSystemRPC.cc \
	RemoteFileSystemMessages.cc \
	RemoteContainer.cc RemoteDictionary.cc RemoteFileObject.cc \
	RemoteServer.cc RemoteTables.cc ResourceLock.cc
RemoteFileSystemDefines = -DRemoteFileSystem
DiskFile=Yes
endif

ifdef DiskFile
DiskIncludes=   Disk.h DiskContainer.h
DiskSources=    Disk.cc DiskContainer.cc
DiskFileDefines = -DDiskFile
endif

OPTIONALFILESYSTEMINCLUDES = $(ArIncludes) $(BSDIncludes) $(COFFIncludes) \
	$(COFFLoaderIncludes) $(LogIncludes) $(MadIncludes) $(MailIncludes) \
	$(MSDOSIncludes) $(SVIDIncludes) $(AIXIncludes) $(DiskIncludes) \
	$(RAMDiskIncludes) $(TarIncludes) $(RecordIncludes) \
	$(POSHIncludes) $(RemoteFileSystemIncludes) $(ELFIncludes) \
	$(Ext2Includes) $(CramfsIncludes) \
	$(LKMIncludes)

OPTIONALFILESYSTEMSOURCES = $(ArSources) $(BSDSources) $(COFFSources) \
	$(Ext2Sources) \
	$(COFFLoaderSources) $(LogSources) $(MadSources) $(MailSources) \
	$(MSDOSSources) $(SVIDSources) $(AIXSources) $(DiskSources) \
	$(RAMDiskSources) $(OptionalSchedulerSources) $(TarSources) \
	$(RecordSources) $(POSHSources) $(RemoteFileSystemSources) \
	$(ELFSources) $(CramfsSources) $(LKMSources)

OPTIONALFILESYSTEMSTARS = $(ArStars) $(BSDStars) $(COFFStars) \
	$(COFFLoaderStars) $(LogStars) $(MadStars) $(MailStars) \
	$(MSDOSStars) $(SVIDStars) $(AIXStars) $(DiskStars) \
	$(RAMDiskStars) $(OptionalSchedulerStars) $(TarStars) \
	$(RecordStars) $(CramfsStars) $(POSHStars) $(ELFStars)

OPTIONALFILESYSTEMSTARINCLUDES = \
	$(ArStarIncludes) $(BSDStarIncludes) \
	$(COFFStarIncludes) $(COFFLoaderStarIncludes) \
	$(LogStarIncludes) $(MadStarIncludes) $(MailStarIncludes) \
	$(MSDOSStarIncludes) $(SVIDStarIncludes) $(AIXStarIncludes) \
	$(DiskStarIncludes)  $(RAMDiskStarIncludes) \
	$(OptionalSchedulerStarIncludes) $(TarStarIncludes) \
	$(RecordStarIncludes) $(CramfsStarIncludes) $(POSHStarIncludes) \
	$(ELFStarIncludes)

OPTIONALFILESYSTEMDEFINES = \
	$(TarFileDefines) \
	$(CoffLoaderDefines) \
	$(DiskFileDefines) \
	$(AIXFileDefines) \
	$(Ext2Defines) \
	$(ArFileDefines) \
	$(BSDFileDefines) \
	$(COFFFileDefines) \
	$(MadFileDefines) \
	$(MailFileDefines) \
	$(MsDosFileDefines) \
	$(SVIDFileDefines) \
	$(DiskFileDefines) \
	$(AIXFileDefines) \
	$(CramfsDefines) \
	$(RemoteFileSystemDefines)

PROXYFILESYSTEMINCLUDES = FileObject.h FileObjectView.h \
	RecordStream.h FileSystem.h \
        FixedRecordStream.h PersistentArrayStream.h VariableRecordStream.h \
	FileSystemInterface.h MemoryObjectContainer.h \
	MemoryObjectDictionary.h PersistentMemoryObject.h \
	SymbolicLink.h PersistentObject.h \
        RecordFile.h FixedRecordFile.h VariableRecordFile.h \
        PersistentArray.h PersistentClass.h
 

FILESYSTEMINCLUDES = \
	DictionaryStack.h DiskContainer.h RootEntry.h \
	MountItem.h MountTable.h UNIXContainer.h UNIXInode.h BlockAllocator.h \
	UNIXInodeIndirect.h \
	COFF.h elF.h MemoryObjectPartition.h \
        PersistentIntArray.h \
        FileIndex.h \
	FileSystemLocks.h \
	ContainerIndex.h \
	ZombieInfo.h

FILESYSTEMSTARINCLUDES = \
	$(OPTIONALFILESYSTEMSTARINCLUDES) \
	DiskStar.h FileObjectStar.h FileSystemInterfaceStar.h \
	MemoryObjectContainerStar.h MemoryObjectDictionaryStar.h \
	MountTableStar.h PersistentMemoryObjectStar.h \
	SymbolicLinkStar.h PersistentArrayStar.h PersistentArrayStreamStar.h \
	PersistentClassStar.h FileIndexStar.h PersistentIntArrayStar.h


FILESYSTEMSOURCES = \
	DictionaryStack.cc RecordStream.cc \
	FileSystemInterface.cc RootEntry.cc MountItem.cc MountTable.cc \
	SymbolicLink.cc UNIXContainer.cc UNIXInode.cc \
	UNIXInodeIndirect.cc \
	BlockAllocator.cc \
	PersistentObject.cc FileObject.cc FileObjectView.cc \
	MemoryObjectContainer.cc \
	MemoryObjectCachingContainer.cc \
	MemoryObjectDictionary.cc MemoryObjectPartition.cc \
	PersistentMemoryObject.cc \
	PersistentIntArray.cc \
	FileIndex.cc \
	PersistentArray.cc FixedRecordStream.cc PersistentArrayStream.cc \
	VariableRecordStream.cc PersistentClass.cc \
	FileSystemLocks.cc \
	ContainerIndex.cc \
	ZombieInfo.cc

FILESYSTEMSTARSOURCES = \
	$(OPTIONALFILESYSTEMSTARS) \
	DiskStar.cc FileObjectStar.cc FileSystemInterfaceStar.cc \
	MemoryObjectContainerStar.cc MemoryObjectDictionaryStar.cc \
	MountTableStar.cc PersistentMemoryObjectStar.cc \
	PersistentArrayStar.cc \
	PersistentArrayStreamStar.cc AutoloadPersistentObjectStar.cc \
	PersistentClassStar.cc FileIndexStar.cc \
	PersistentIntArrayStar.cc

