#include "Kernel.h"
#include "IntegratorKernel.h"
#include "KernelCmdInstaller.h"
#include "RecklessDriver.h"
#include "RecklessDriverDD.h"
#include "Exception.h"
#include "NameServer.h"
#include "FreeRunningTimer.h"
#include "FreeRunningTimerStar.h"

/*
 * Accessing an array beyond its bounds
 */
void outOfBounds() {
	try
	{
		RecklessDriver * badDriver = new RecklessDriver();
		badDriver->accessOutOfBounds();
		delete badDriver;
	}
	catch(Exception * ex)
	{
		Console() << "Exception returned from RecklessDriver::accessOutOfBounds(): \n" << eor;
		Console() << ex->getMessage() << "\n" << eor;
	}
}

void outOfBoundsDD() {
	try
	{
		RecklessDriverDD * badDriver = new RecklessDriverDD();
		badDriver->accessOutOfBounds();
		delete badDriver;
	}
	catch(Exception * ex)
	{
		Console() << "Exception returned from RecklessDriverDD::accessOutOfBounds(): \n" << eor;
		Console() << ex->getMessage() << "\n" << eor;
	}
}

static KernelCmd* testAccessOutOfBounds = Kernel::installCmd (
	new KernelCmd(
		outOfBounds,
		"DD_outOfBounds", 
		"DefensiveDriving: access beyond the bounds of an array." 
	));

static KernelCmd* testAccessOutOfBoundsDD = Kernel::installCmd (
	new KernelCmd(
		outOfBoundsDD,
		"DD_outOfBoundsDD", 
		"DefensiveDriving: access beyond the bounds of an array with DD." 
	));

/*
 * Kernel Access Tests
 */
void kernelAccess() {
	try
	{
		RecklessDriver * badDriver = new RecklessDriver();
		badDriver->accessKernelObjects();
		delete badDriver;
	}
	catch(Exception * ex)
	{
		Console() << "Exception returned from RecklessDriver::accessKernelObjects(): \n" << eor;
		Console() << ex->getMessage() << "\n" << eor;
	}
}

void kernelAccessDD() {
	try
	{
		RecklessDriverDD * badDriver = new RecklessDriverDD();
		badDriver->accessKernelObjects();
		delete badDriver;
	}
	catch(Exception * ex)
	{
		Console() << "Exception returned from RecklessDriverDD::accessKernelObjects(): \n" << eor;
		Console() << ex->getMessage() << "\n" << eor;
	}
}

static KernelCmd* testAccessKernelObjects = Kernel::installCmd (
	new KernelCmd(
		kernelAccess,
		"DD_kernelAccess", 
		"DefensiveDriving: accessing kernel objects." 
	));

static KernelCmd* testAccessKernelObjectsDD = Kernel::installCmd (
	new KernelCmd(
		kernelAccessDD,
		"DD_kernelAccessDD", 
		"DefensiveDriving: accessing kernel objects with DD." 
	));

/*
 * Dereferencing an invalid pointer
 */
void invalidPtrDereference() {
	try
	{
		RecklessDriver * badDriver = new RecklessDriver();
		badDriver->invalidPointerDereference();
		delete badDriver;
	}
	catch(Exception * ex)
	{
		Console() << "Exception returned from RecklessDriver::invalidPointerDereference();: \n" << eor;
		Console() << ex->getMessage() << "\n" << eor;
	}
}

void invalidPtrDereferenceDD() {

	try
	{
		RecklessDriverDD * badDriver = new RecklessDriverDD();
		badDriver->invalidPointerDereference();
		delete badDriver;
	}
	catch(Exception * ex)
	{
		Console() << "Exception returned from RecklessDriver::invalidPointerDereference();: \n" << eor;
		Console() << ex->getMessage() << "\n" << eor;
	}
}



static KernelCmd* testInvalidPointerDereference = Kernel::installCmd (
	new KernelCmd(
		invalidPtrDereference,
		"DD_invalidPtrDereference", 
		"DefensiveDriving: dereferencing an invalid pointer." 
	));

static KernelCmd* testInvalidPointerDereferenceDD = Kernel::installCmd (
	new KernelCmd(
		invalidPtrDereferenceDD,
		"DD_invalidPtrDereferenceDD", 
		"DefensiveDriving: dereferencing an invalid pointer with DD." 
	));

/*
 * Setting kernel data
 */
void setKernelData() {
	try
	{
		RecklessDriver * badDriver = new RecklessDriver();
		badDriver->setKernelObjects();
		delete badDriver;
	}
	catch(Exception * ex)
	{
		Console() << "Exception returned from RecklessDriver::setKernelObjects();: \n" << eor;
		Console() << ex->getMessage() << "\n" << eor;
	}
}

void setKernelDataDD() {
	try
	{
		RecklessDriverDD * badDriver = new RecklessDriverDD();
		badDriver->setKernelObjects();
		delete badDriver;
	}
	catch(Exception * ex)
	{
		Console() << "Exception returned from RecklessDriverDD::setKenelDataDD();: \n" << eor;
		Console() << ex->getMessage() << "\n" << eor;
	}
}

static KernelCmd* testSetKernelObjects = Kernel::installCmd (
	new KernelCmd(
		setKernelData,
		"DD_setKernelData", 
		"DefensiveDriving: attempt to set kernel data." 
	));

static KernelCmd* testSetKernelObjectsDD = Kernel::installCmd (
	new KernelCmd(
		setKernelDataDD,
		"DD_setKernelDataDD", 
		"DefensiveDriving: attempt to set kernel data with DD." 
	));


void overhead(){
  FreeRunningTimerStar SystemTimer;
  unsigned int start = 0;
  unsigned int end = 0;
#define NTIMES 10000
  SystemTimer = (FreeRunningTimer *)(ProxiableObject *)
                StandardNameServer->lookup( "SystemTimer", "FreeRunningTimer" );

  try
    {
      Console()<<"loop iteration for "<<NTIMES<<" times\n"<<eor;
      RecklessDriverDD * badDriver = new RecklessDriverDD();
      start = SystemTimer->time();
      for(int i = 0; i < NTIMES; i++){
      }
      end = SystemTimer->time();
      int over = end-start;
      Console()<<"Loop Overhead = "<<over<<"\n"<<eor;
      
      start = SystemTimer->time();
      for(int i=0; i< NTIMES; i++){
	badDriver->invalidPointerDereference();
      }
      end = SystemTimer -> time();
      
      Console() << "Time " << end - start << "\n" << eor;
      // delete badDriver;
    }
  catch(Exception * ex)
    {
      Console() << "Exception returned from RecklessDriverDD::invalidPointerDereference();: \n" << eor;
      Console() << ex->getMessage() << "\n" << eor;
    }
}

static KernelCmd* testOverhead = Kernel::installCmd (
	new KernelCmd(
		overhead,
		"DD_overhead", 
		"DefensiveDriving: test overhead." 
	));
