#include "Kernel.h"
#include "IntegratorKernel.h"
#include "KernelCmdInstaller.h"
#include "Object.h"
#include "ThisCPU.h"
#include "ARMTranslation.h"
#include "ARMCPU.h"
#include "AsmBridge.h"
#include "GenericARMGdb.h"
#include "IntegratorSerial.h"
#include "ARMInterrupts.h"
#include "IntegratorInterrupts.h"
#include "IntegratorInterruptManager.h"

void myCode ()
{
  Console() << "Called student code\n" << eor;
}

void myTest ()
{
  Console() << "Running test case...\n" << eor;
  Console() << "Passed!\n" << eor;
}

class MyClass : public Object
{
public:
  MyClass()
  {
//    Console() << "Creating instance of MyClass\n" << eor;
  }

  void myCode ()
  {
    Console() << "Called student code within object\n" << eor;
  }
};

void MMUOn() {
	Console() << "Turning MMU on\n" << eor;
	if(TheKernel == 0) {
		Console() << "Please initialize the kernel first...\n" << eor;
		return;
	}

	thisCPU()->MMU()->activate(thisCPU()->idleProcess()->domain());
	thisCPU()->MMU()->enable();
}

void showPTraw(unsigned* pointerTable) {
	Console() << "Pointer tables begin at:" << asHex((void*)pointerTable) << "\n" 
		<< "" << "\n" 
		<<  eor;

	unsigned entry;

	for( int i=0; i<4096; i++) {
		entry = pointerTable[i];
		if(entry != 0) {
			Console() << "PointerTable[" << asHex(i) << "] =" << asHex(entry) << "   " << eor;
			unsigned type = entry & 0x3; 
			unsigned domain = (entry >> 5) & 0xf; 

			switch(type) {
				case 0:
					Console() << "Translation fault\r\n" << eor;
					break;
				case 1:
                {
					unsigned base = entry &  ~((10 << 1) - 1);
					Console() << "Coarse page table:  domain= " << asHex(domain) << " base=" << asHex(base) << "\n" << eor;
					for( int j=0; j < 256; j++) {
						unsigned ptentry = *((unsigned*)base+j);
						unsigned ptype = ptentry & 0x3; 
						if( ptentry == 0 )
							continue;

						Console() << "    entry[" << asHex(i) << "][" << asHex(j) << "] = " << asHex(ptentry) << " ";
						switch(ptype) {
							case 0:
								Console() << "Translation fault\r\n" << eor;
								continue;
							case 1:
								Console() << "Large 1k page " << eor;
								break;
							case 2:
								Console() << "Small 4k page " << eor;
								break;
							case 3:
								Console() << "Extended small page " << eor;
								break;
							default:
								Console() << "Unknown page table entry kind " << asHex(ptype) << "\r\n";
								continue;
						}
						Console() << "    Maps 4k or 16k from " << asHex(ptentry & (~4095)) 
						  << " to VA " << asHex(((1 << 20) * i) + ((1 << 12) *j)) << "\n" << eor;
					}
					Console() << "\r\n";
					break;
                }
				case 2:
					Console() << "Section table: domain= " << asHex(domain) << " maps 1 mb from " << asHex(entry & ~(( 1 << 20) - 1)) 
						  << " to VA " << asHex( (1 << 20) * i) << "\n" << eor;
					break;
				case 3:
					Console() << "Reserved entry\r\n" << eor;
					break;
				default:
					Console() << "Unknown pointer table entry kind " << asHex(type) << "\r\n";
			}
		}
	}
}
	
void showPT() {
	if(TheKernel == 0) {
		Console() << "Please initialize the kernel first...\n" << eor;
		return;
	}
	ARMTranslation* trans = (ARMTranslation*) TheKernel->domain()->translation();
	ARMPointerTableEntry* pointerTable = trans->pointerTable();
	showPTraw((unsigned*)pointerTable);
}

void defaultPT() {
	Console() << "Creating default page table...\n" << eor;
	ARMTranslation* trans = (ARMTranslation*) TheKernel->domain()->translation();
	ARMPointerTableEntry* pointerTable = trans->pointerTable();
	unsigned* ttb = (unsigned*) pointerTable;
	for( int i=0; i<4096; i++) {
		*(ttb + i) = 0x00000c1e + i*0x00100000;  
	}
	Console() << "done.\n" << eor;
}

MyClass installerDummy;
static KernelCmd* mycmd = Kernel::installCmd (new KernelCmd(myCode, "mycode1", "Invoke My Kernel Code" ));
static KernelCmd* myTestCase = Kernel::installCmd (new KernelCmd(myTest, "testcase1", "Run a test case" ));
static KernelCmd* mycmd2 = Kernel::installCmd (new KernelCmd(new MyClass(), (OPFV)&MyClass::myCode, "mycode2", "Invoke My Kernel Object's Code3"));
static KernelCmd* mmuon = Kernel::installCmd (new KernelCmd(MMUOn, "mmu", "Turn MMU on"));
static KernelCmd* showpt = Kernel::installCmd (new KernelCmd(showPT, "showpt", "Dump the contents of the page tables"));
static KernelCmd* defpt = Kernel::installCmd (new KernelCmd(defaultPT, "defpt", "Create a default 1-1 mapped page table"));

void setPTE(unsigned int ttb, unsigned int real, unsigned int virt,
    bool dbg = false)
{
  unsigned int *table = (unsigned int *)ttb;
  unsigned int index = (virt >> 20); 

  if(dbg)
    Console() << "[in] virt " << asHex(virt) << "->" << asHex(real)
      << " currently: table[" << asHex(index) << "]=" <<
      asHex(table[index]) << "\n" << eor;

  // bits 0-1  : identify PTE as section entry
  // bit 2     : B
  // bit 3     : C
  // bit 4     : 1
  // bits 5-8  : domain
  // bit 9     : 0
  // bits 10-11: AP
  // bits 12-19: 0
  //
  // 1100 0001 0010
  //   11   0   0 000    1           0,0      | 10
  // | AP | P | domain | SB1 | cached, buffed | section table 
  unsigned int entry = 0xC12;

  // upper 12 bits of PTE replace upper 12 bits of virtual address
  // to generate physical address
  entry |= ( real & 0xFFF00000 );

  table[index] = entry;


  if(dbg)
    Console() << "[out] virt " << asHex(virt) << "->" << asHex(real)
      << " currently: table[" << asHex(index) << "]=" <<
      asHex(table[index]) << "\n" << eor;
}

// Sets page table entry in ttb for a 1 mb section
void setPTBase(unsigned int ttb, unsigned int ptb, unsigned int real, unsigned int virt,
    bool dbg = false)
{
  unsigned int *table = (unsigned int *)ttb;
  unsigned int *ptable = (unsigned int *)ptb;
  unsigned int index = (virt >> 20); 
  unsigned int ptindex; 

  if(dbg)
    Console() << "[in] virt " << asHex(virt) << "->" << asHex(real)
      << " currently: table[" << asHex(index) << "]=" <<
      asHex(table[index]) << "\n" << eor;

  // bits 0-1  : identify PTE as a coarse PT base address
  // bit 2-4   : SBZ
  // bits 5-8  : domain
  // bit 9-P   : 0
  // bits 10-31: PT base address
  //
  // 1100 0001 0010
  // | PT base | P | domain | SBZ | Coarse page table 
  //   PT base   0 00   00 00  0 | 01
  unsigned int entry = 0x001;

  // upper 22 bits identify page table base
  entry |= ( ptb & 0xFFFFFC00 );

  // write base page table
  table[index] = entry;

  // figure out second-level PT index
  // Keep index within last mb (section table)
  // Divide by 12 (page table index
  ptindex = (virt & 0xFFFFF) >> 12; 

  // figure out entry info
  // Using small page ( 4 kb )
  // Bit 31-12   : base address
  // Bit 11-4    : 4 groups of 2 access permission bits (11)
  // Bit 2,3	 : Cached-3, Buffered-2
  // Bit 1,0	 : 10 (as a small page)
  // base | 1111 1111 00 10
  entry = 0xFF2;
  // Lop off last 4 kb of real address
  entry |= real & 0xFFFFF000;
  ptable[ptindex] = entry;

  if(dbg)
    Console() << "[out] virt " << asHex(virt) << "->" << asHex(real)
      << " currently: table[" << asHex(index) << "]=" <<
      asHex(table[index]) << ", ptable[" << ptindex << "] = " << asHex(ptable[ptindex]) << "\n" << eor;
}



void mmuOn() {
  /* 
     VM Mappings (1M sections)
     Virtual    Real       Purpose
     0x00000000 0x10000000 Interrupts (Low)
     0x10000000 0x10000000 Choices-chunk1
     0x10100000 0x10100000 Choices-chunk2
     0x10200000 0x10200000 Choices-chunk3
     0x10300000 0x10300000 Choices-chunk4
     0x10800000 0x10800000 Pointer Table Base 
     0x10a00000 0x10a00000 Page Table Base 
     0xFFF00000 0xFFF00000 Serial I/O
  */

  unsigned int ttb = 0x10800000;
  unsigned int ptb = 0x10a00000;

  unsigned* ptr = (unsigned*) ttb;

  for( int i=0; i < 4096; i++ ) 
	  ptr[i] = 0;

  ptr = (unsigned*) ptb;
  for( int i=0; i < 16384; i++ ) 
	  ptr[i] = 0;
  // put the exception tables at real address 0x10000000
  // (the beginning of choices)
  // setPTE(ttb, 0x10000000, 0x00000000, true);


  // Choices
  setPTE(ttb, 0x10000000, 0x10000000, true);
  setPTE(ttb, 0x10100000, 0x10100000, true);
  setPTE(ttb, 0x10200000, 0x10200000, true);
  setPTE(ttb, 0x10300000, 0x10300000, true);

  // Pointer Table
  setPTE(ttb, 0x10800000, 0x10800000, true);

  // Serial I/O 0xfffb0000
  setPTBase(ttb, ptb, 0xFFFB0000, 0xFFFB0000, true);

  //For high interrupt vectors
  setPTBase(ttb, ptb, 0x10000000, 0xFFFF0000, true);

  // except for this stuff. :-)
  // (used to test that VM is actually on)
  setPTE(ttb, 0x10000000, 0x10900000, true);
  setPTE(ttb, 0x10A00000, 0x10A00000, true);
  setPTE(ttb, 0x10A00000, 0x10B00000, true);
  setPTBase(ttb, ptb+4096, 0xB1111111, 0xC1111111, true);
  setPTBase(ttb, ptb+8192, 0xBEBE1111, 0xC1111111, true);
  setPTBase(ttb, ptb+12288, 0xF331dead, 0xF331dead, true);
  setPTBase(ttb, ptb+16384, 0xE111cc00, 0xE111cc00, true);

  // Write ttb into cp15:c2 (Sloss 504)
  ttb &= 0xffffc000;

  unsigned int ttbOut = 0xDEADBEEF;

  // check the original value
  asm ( "mrc p15, 0, %0, c2, c0, 0"
      : "=r" (ttbOut)
      :
      : );
  Console() << "Original ttb=" << asHex(ttbOut) << "\n" << eor;


  // write then check the new value
  asm ( "mcr p15, 0, %0, c2, c0, 0"
      :
      : "r" (ttb)
      : );
  asm ( "mrc p15, 0, %0, c2, c0, 0"
      : "=r" (ttbOut)
      :
      : );

  Console() << "Wrote ttb= " << asHex(ttbOut) << "\n" << eor;

  // set page table permissions (Sloss 512)
  int permsIn = 0xCAFEBABE, permsOut = 0xFFFFFFFF;

  // check original value
  asm ( "mrc p15, 0, %0, c3, c0, 0"
      : "=r" (permsIn)
      :
      : );
  Console() << "Perms=" << asHex(permsIn) << "\n" << eor;

  
  // set new value
  asm ( "mcr p15, 0, %0, c3, c0, 0"
      :
      : "r" (permsOut)
      : );
  
  // check new value
  asm ( "mrc p15, 0, %0, c3, c0, 0"
      : "=r" (permsIn)
      :
      : );
  Console() << "Perms=" << asHex(permsIn) << "\n" << eor;


  // adjust MMU control word (Sloss 514)
  // read current MMU control word
  int control;
  asm ( "mrc p15, 0, %0, c1, c0, 0"
      : "=r" (control)
      :
      : );
  Console() << "Control word=" << asHex(control) << "\n" << eor;

  // enable MMU, ICACHE, and DCACHE
  control &= ~0x00001005;
  control |= 0x00001005;
  control |= 1 << 13; //high exception vectors

  Console() << "New control word will be: " << asHex(control) << "\n" << eor;

  // set the new MMU control word
  asm ( "mcr p15, 0, %0, c1, c0, 0"
      :
      : "r" (control)
      : );

  Console() << "Set new control word.\n" << eor;

  // reread to see if change actually happend
  asm ( "mrc p15, 0, %0, c1, c0, 0"
      : "=r" (control)
      :
      : );
  Console() << "Control word=" << asHex(control) << "\n" << eor;

  // these should point to the same location in physical memory
  unsigned int a = *((unsigned int *)0x10000000);
  unsigned int b = *((unsigned int *)0x10900000);

  Console() << "a: " << asHex(a) << " b: " << asHex(b) << "\n" << eor;
  
  // these should point to the same location in physical memory
  volatile unsigned int *c = (unsigned int *)0x10A00000;
  volatile unsigned int *d = (unsigned int *)0x10B00000;

  c[0] = 0x00000000;
  c[1] = 0x00000000;
  d[0] = 0x00000000;
  d[1] = 0x00000000;

  c[0] = 0xCAFEBABE;
  d[1] = 0xDEADBEEF;

  Console() << "d[0]: " << asHex((unsigned int)d[0])
    << " c[1]: " << asHex((unsigned int)c[1]) << "\n" << eor;

  Console() << "c[0]: " << asHex((unsigned int)c[0])
    << " d[1]: " << asHex((unsigned int)d[1]) << "\n" << eor;

	return;
}

void mySWI ()
{
	Gdb *gdb = new GenericARMGdb();

	mmuOn();

  asm( "swi #3" );
}

void runUboot() {
	void (*f)() = (void (*)()) 0x0;
	f();
}

void runChoices() {
	void (*f)() = (void (*)()) 0x10000000;
	f();
}

int testfun() {
	volatile int a = 0xdeaddea1;
	volatile int b = 0xbeefbee2;
	volatile int c = 0xdeaddea3;
	volatile int d = 0xbeefbee4;
	volatile int e = 0xdeaddea5;
	volatile int f = 0xbeefbee6;
	volatile int g = 0xdeaddea7;
	volatile int h = 0xbeefbee8;
	while(1) {
		a = b + c;   
		b = b + c;   
		c = b + a;   
		a = b + a;   
		c = b + c;   
	}
	return a;
}

InterruptManager *interruptManager;
extern IntegratorSerial* GenericARMSerialPort;

void testInt() {
  Console() << "Testing interrupts\n" << eor;

	mmuOn();

  showPTraw((unsigned*)0x10800000);

  interruptManager = new IntegratorInterruptManager();
	Gdb *gdb = new GenericARMGdb();

	// Enable serial on IH2
	interruptManager->enableInterrupt(54);

  Console() << "Enabling serial interrupts\n" << eor;
  GenericARMSerialPort->enableInterrupts();
  Console() << "Enabled serial interrupts\n" << eor;

	PROCESSOR::enableInterrupts();
	Assert(PROCESSOR::interruptsEnabled());
  asm("mov r0, #111");
  asm("mov r7, #117");

	testfun();
}



static KernelCmd* testint = Kernel::installCmd (new KernelCmd(testInt, "testint", "Test interrupt code" ));

static KernelCmd* mycmd3 = Kernel::installCmd (new KernelCmd(mySWI, "swi", "Invoke My Kernel SWI Code" ));

static KernelCmd* uboot = Kernel::installCmd (new KernelCmd(runUboot, "uboot",
      "Jump straight into U-Boot" ));

static KernelCmd* choices = Kernel::installCmd (new KernelCmd(runChoices,
      "choices", "restart choices" ));

void assertNotReached() {
	//use this to debug cpu registers&flags
	Assert(NOTREACHED);
}

static KernelCmd* notReachedCmd = Kernel::installCmd (new KernelCmd(assertNotReached,
															  "assert", "generate an assertion and show cpu state" ));


void intOn() {
	asm volatile (" mrs r2, cpsr ; 	       \
	bic r2, r2, #0x80 ;    \
	msr cpsr_c, r2 " : : : "r2");
	Debug << "Turning int on\n" << eor;
}

static KernelCmd* intOnCmd = Kernel::installCmd (new KernelCmd(intOn,
																	"inton", "turn interrupts on" ));

void intOff() {
	unsigned long old,temp;
	__asm__ __volatile__("mrs %0, cpsr\n"
						 "orr %1, %0, #0xc0\n"
						 "msr cpsr_c, %1"
						 : "=r" (old), "=r" (temp)
						 :
						 : "memory");

	Debug << "Turning int off\n" << eor;
}

static KernelCmd* intOffCmd = Kernel::installCmd (new KernelCmd(intOff,
															   "intoff", "turn interrupts off" ));

void intShow() {
	volatile unsigned int prevState;
	
	asm volatile (" mrs r1, cpsr ; " : "=r" (prevState) : : "r1");
	
	Debug << "Interrupt state is: " << asHex(prevState) << "\n" << eor;
}

static KernelCmd* intStateCmd = Kernel::installCmd (new KernelCmd(intShow,
																"intshow", "show interrupts" ));

void permTest() {
	asm volatile ("eor r1, r1, r1\n"
				  "add r1, r1, #0xd3\n"
				  "msr cpsr_c, r1\n"
				  : : :);
}

static KernelCmd* permTestCmd = Kernel::installCmd (new KernelCmd(permTest,
																  "permtest", "test permissions" ));

void breakpoint() {
	mmuOn();
	new GenericARMGdb();
	TrapToDebugger();
}

static KernelCmd* breakCmd = Kernel::installCmd( new KernelCmd(breakpoint, "b", "Break to debugger"));

extern void throwFunc();
void exception_test() {
	unsigned bad = 0x235263;

	Console() << "ABOUT TO FAULT\n" << eor;
	int a = 5;
	int b = 0xcafebabe;
	int c = 0;
	
	try {
		a *= 2;

		b = a/c;
		Console() << "After div by 0\n" << asHex(b) << "\n" << eor;

		intShow();
		asm( "mov r4, %0" :: "r"(a) : "r4" );
		asm( "mov r0, #0" ::: "r0" );
		asm( "mov r1, #1" ::: "r1" );
		asm( "orr r0, r0, r1" ::: "r0" );
		asm( "mov r3, %0" :: "r"(bad) : "r3" );
		asm( "mov r3, #2" );
		asm( "str r3, [r11, #-88]" );
		asm( "ldr r0, [r3]" ::: "r0" );
		Console() << "DID NOT HAPPEN\n" << asHex(b) << "\n" << eor;
	} catch (unsigned e) {
		Console() << "GOT EXCEPTION1\n" << eor;
	}
/*	try {
		traceStack();
	} catch (unsigned e) {
		Console() << "GOT EXCEPTION2\n" << eor;
		}*/
}

static KernelCmd* exceptionsCmd = Kernel::installCmd( new KernelCmd(exception_test, "e", "Test Exceptions"));
