#
# This file is part of the Choices Operating System
# Developed by: The TAPESTRY Parallel Computing Laboratory
#		 University of Illinois at Urbana-Champaign
#		 Department of Computer Science
#		 1304 W. Springfield Ave.
#		 Urbana, IL	61801
#
# Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
# The University of Illinois Board of Trustees.
#	All Rights Reserved.
# CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
#
# Author: Johnny Zweig (zweig@cs.uiuc.edu)
# Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
#
# Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
#   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
#
#
# Common makefile for Networking portions of Choices
#
#

#
# Optional sources/includes
#



ifdef Networking

# Networking include files (Includes/Networks, Includes/Networks/(NETTYPE) )

NETWORKVPATH =	$(TOP)/Networks/Administrator \
		$(TOP)/Networks/Conduit \
		$(TOP)/Networks/IP \
		$(TOP)/Networks/TCP \
		$(TOP)/Networks/UDP \
		$(TOP)/Networks/Ethernet \
		$(TOP)/Networks/Other

NETWORKHVPATH =	$(TOP)/Includes/Networks/Administrator \
		$(TOP)/Includes/Networks/Conduit \
		$(TOP)/Includes/Networks/IP \
		$(TOP)/Includes/Networks/TCP \
		$(TOP)/Includes/Networks/UDP \
		$(TOP)/Includes/Networks/Ethernet \
		$(TOP)/Includes/Networks/Other

NETWORKINCLUDES = $(MACHNETINCLUDES) \
	NetworkStatistics.h NetworkBuffer.h NetworkBufferManager.h \
	NetworkBufferQueue.h \
	NetworkAdministrator.h NetworkDriver.h \
	NetworkTest.h NetworkWalker.h \
	Ethernet.h EthernetAdministrator.h EthernetDMUX.h \
	EthernetDriver.h StubEthernetDriver.h \
	ZootReturnCodes.h

# Networking source files (Networks/Administrator, Networks/Ethernet)

NETWORKSOURCES = $(MACHNETSOURCE) \
	NetworkStatistics.cc NetworkBuffer.cc NetworkBufferManager.cc \
	NetworkBufferQueue.cc \
	NetworkAdministrator.cc NetworkDriver.cc \
	NetworkTest.cc NetworkWalker.cc \
	Ethernet.cc EthernetAdministrator.cc EthernetDMUX.cc \
	EthernetDriver.cc StubEthernetDriver.cc \
	ZootReturnCodes.cc

NETWORKDEFINES = -DNETWORKING

endif
###########################################################################
ifdef ZOOT

# Conduit/ZOOT include files (Networks/{Conduit,Ethernet,IP,TCP,UDP,Other})

# Hyper-important fact:  the file MachineByteOrder.h must be a sym-link to
#   one or the other of /Includes/Networks/Conduit/LittleEndianByteOrder.h
#   or BigEndianByteOrder.h! -JohnnyZ

PROXYCONDUITINCLUDES = Conduit.h ApplicationConduitStream.h \
	ApplicationConduitInputStream.h ApplicationConduitOutputStream.h \
	UserSessionManager.h

CONDUITINCLUDES = \
	Word.h ConduitTypes.h MachineByteOrder.h ZootConfig.h \
	ConduitMessage.h IPMessage.h NetworkAddress.h ZootAddress.h \
	ConduitProtocolIdentifiers.h \
	ConduitMessageBuffer.h AutoRetransmitBuffer.h MessageFIFO.h \
	IPChecksum.h ZootClock.h  Zoot_IP_Output.h \
	ZootStateMachine.h UDPConduit.h \
	TtyBuffer.h DumbConduit.h AdapterConduit.h \
	CCCP.h ElvisConduit.h \
	UserSession.h UserSessionManager.h AboriginalTelnet.h FTP.h \
	AboriginalPing.h

# Conduit/ZOOT source files (Networks/Administrator, Networks/Ethernet)

CONDUITSOURCES = Conduit.cc \
	EthernetConduit.cc IPConduit.cc TCPConduit.cc UDPConduit.cc \
	Zoot_IP_Output.cc ZootStateMachine.cc \
	TCPClose_WaitState.cc TCPFin_Wait_2State.cc \
	TCPClosedState.cc TCPListenState.cc \
	TCPClosingState.cc TCPSyn_ReceivedState.cc \
	TCPEstablishedState.cc TCPSyn_SentState.cc \
	TCPTime_WaitState.cc TCPLast_AckState.cc \
	TCPFin_Wait_1State.cc \
	ApplicationConduit.cc ZootAddress.cc ConduitMessageBuffer.cc \
	MessageFIFO.cc \
	IPChecksum.cc ZootClock.cc \
	ConduitTypes.cc ZootController.cc ConduitMessage.cc \
	IPMessage.cc DumbConduit.cc AdapterConduit.cc TtyBuffer.cc \
	CCCP.cc ElvisConduit.cc ICMPConduit.cc\
	ApplicationConduitStream.cc ApplicationConduitInputStream.cc \
	ApplicationConduitOutputStream.cc UserSession.cc \
	UserSessionManager.cc AboriginalTelnet.cc FTP.cc \
	AboriginalPing.cc 

NETWORKDEFINES = -DNETWORKING -DZOOT -DQUIET

endif
