#
# This file is part of the Choices Operating System
# Developed by: The TAPESTRY Parallel Computing Laboratory
#		 University of Illinois at Urbana-Champaign
#		 Department of Computer Science
#		 1304 W. Springfield Ave.
#		 Urbana, IL	61801
#
# Copyright (c) 1987, 1988, 1989, 1990, 1991
# The University of Illinois Board of Trustees.
#	All Rights Reserved.
# CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
#
# Author: David W. Dykstra (dykstra@cs.uiuc.edu)
# Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
#
# Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
#   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
#
#
# Makefile to include for all modules that want files proxified
#
#

PROXIFIEDDIR = $(TOP)/Includes/$(MODULENAME)/$(MACHINE)
PROXIFIEDCOMDIR = $(TOP)/Includes/$(MODULENAME)/Common

#
# Having the 'New' files allows us to only update *Includes.cc if
#    the include list really changes
#
NewAllIncludes.cc: Make.dependencies
	@echo Creating NewAllIncludes.cc; \
	$(RM) NewAllIncludes.cc; \
	echo $(ALLINCLUDES)|tr ' ' '\012'| \
		sed '/^$$/d;s/^/#include "/;s/$$/"/' >NewAllIncludes.cc

AllIncludes.cc : NewAllIncludes.cc
	@if cmp NewAllIncludes.cc AllIncludes.cc >/dev/null 2>&1; \
	then :; \
	else echo "Creating AllIncludes.cc"; \
	    $(RM) AllIncludes.cc; cp NewAllIncludes.cc AllIncludes.cc; \
	fi

NewTableOnlyProxyIncludes.cc : Make.dependencies
	@echo Creating NewTableOnlyProxyIncludes.cc; \
	$(RM) NewTableOnlyProxyIncludes.cc; \
	echo $(TABLEONLYPROXYINCLUDES)|tr ' ' '\012'| \
		sed '/^$$/d;s/^/#include "/;s/$$/"/' >NewTableOnlyProxyIncludes.cc

TableOnlyProxyIncludes.cc : NewTableOnlyProxyIncludes.cc
	@if cmp NewTableOnlyProxyIncludes.cc TableOnlyProxyIncludes.cc >/dev/null 2>&1; \
	then :; \
	else echo "Creating TableOnlyProxyIncludes.cc"; \
	    $(RM) TableOnlyProxyIncludes.cc; \
	    cp NewTableOnlyProxyIncludes.cc TableOnlyProxyIncludes.cc; \
	fi

NewProxyIncludes.cc : Make.dependencies
	@echo Creating NewProxyIncludes.cc; \
	$(RM) NewProxyIncludes.cc; \
	echo TableOnlyProxyIncludes.cc $(PROXYINCLUDES)|tr ' ' '\012'| \
		sed '/^$$/d;s/^/#include "/;s/$$/"/' >NewProxyIncludes.cc

ProxyIncludes.cc : NewProxyIncludes.cc
	@if cmp NewProxyIncludes.cc ProxyIncludes.cc >/dev/null 2>&1; \
	then :; \
	else echo "Creating ProxyIncludes.cc"; \
	    $(RM) ProxyIncludes.cc; cp NewProxyIncludes.cc ProxyIncludes.cc; \
	fi

#
# proxify the header files
#

PROXIFIEDSOURCES = PROXYTABLES.cc CONSTRUCTORINSTALLERS.cc PROVIDEDVERSIONS.cc
PROXIFIEDOBJECTS = $(addsuffix .o, $(basename $(PROXIFIEDSOURCES)))

# $(PROXIFIEDSOURCES): $(STARSOURCES)

PROXIFIEDLIB = lib$(MODULENAME).a
PROXIFIEDLIBSOURCES = Constructors.s $(MODULENAME)STUBS.cc \
			$(MODULENAME)VERSIONS.cc $(STARSOURCES) \
			ProxiableObjectStar.cc PersistentObjectStar.cc
PROXIFIEDLIBOBJECTS = $(addsuffix .o, $(basename $(PROXIFIEDLIBSOURCES)))

proxify: $(PROXIFIEDOBJECTS) $(PROXIFIEDLIB)

# use default rules to build these sources created by the makefile
PROXYTABLES.o : PROXYTABLES.cc
CONSTRUCTORINSTALLERS.o : CONSTRUCTORINSTALLERS.cc
PROVIDEDVERSIONS.o : PROVIDEDVERSIONS.cc
$(MODULENAME)VERSIONS.o : $(MODULENAME)VERSIONS.cc

#
# These rules run Proxify++ on all of the $(PROXYINCLUDES) and
#    $(TABLEONLYPROXYINCLUDES) header files at once.   All header files that
#    those header files include are also proxified, and the generated header
#    files are only saved if they are in $(ALLINCLUDES).  The files in
#    $(TABLEONLYPROXYINCLUDES) have only proxy tables generated for them where
#    those in $(PROXYINCLUDES) have stubs and constructors generated for them
#    as well.
#
PROXYDEPFILES = PROXYTABLESSRC.c $(PROXIFIEDLIBSOURCES)

ifeq "$(MODULENAME)" "System"
TABLEONLYPROXYVERSION = TableOnlyProxyVersion.cc
TableOnlyProxyVersion.cc : $(TABLEONLYPROXYINCLUDES) $(STARSOURCES)
	@VERS=`date`; echo "New TableOnlyProxyVersion $$VERS"; \
	$(RM) TableOnlyProxyVersion.cc; \
	echo "VERSION( TableOnlyProxy, \"TableOnlyProxyVersion\", \"$$VERS\" );" \
					>TableOnlyProxyVersion.cc

endif

PROXYTABLES.cc : ProxyIncludes.cc TableOnlyProxyIncludes.cc AllIncludes.cc \
				$(TABLEONLYPROXYVERSION)
	@$(RM) -r H; mkdir H; \
	$(RM) D/PROXYTABLESSRC.dep PROXYTABLES.cc; \
	DEPENDENCIES_OUTPUT="D/PROXYTABLESSRC.dep PROXYTABLES.cc"; \
	export DEPENDENCIES_OUTPUT; \
	$(PROXIFY) $(CPPFLAGS) -all -version ProxyIncludes.cc
	@HDIR=$(PROXIFIEDDIR); \
	if [ ! -d "$$HDIR" ]; then mkdir $$HDIR; fi; \
	echo "Updating header files in $$HDIR"; \
	OLDP=`pwd`; cd $$HDIR; \
	for H in *.h; do \
	    if test -f "$$OLDP/H/$$H" && \
		  grep "\"$$H\"" $$OLDP/AllIncludes.cc >/dev/null; then : ; \
		else echo "Removing old $$HDIR/$$H"; $(RM) "$$H"; \
	    fi; \
	done
	@if [ ! -f $(MODULENAME)VERSIONS.cc ]; \
	then touch $(MODULENAME)VERSIONS.cc; fi; \
	$(RM) NEWVERSIONS.cc; \
	echo '#include "VersionInstaller.h"' >NEWVERSIONS.cc
ifdef TABLEONLYPROXYVERSION
	@cat $(TABLEONLYPROXYVERSION) >>NEWVERSIONS.cc
endif
	@cd H; HDIR=../$(PROXIFIEDDIR); \
	for H in *.h; do \
	    if grep "\"$$H\"" ../AllIncludes.cc >/dev/null; then \
		B=`basename $$H .h`; OV=""; \
		if cmp $$H $$HDIR/$$H >/dev/null 2>&1; then \
		    OV="`grep \^VERSION\(\ $$B, ../$(MODULENAME)VERSIONS.cc`"; \
		else echo "Creating $$HDIR/$$H"; \
		     $(RM) $$HDIR/$$H ../$$V; cp $$H $$HDIR; \
		fi; \
		if [ "$$OV" != "" ]; then \
		    echo "$$OV" >> ../NEWVERSIONS.cc; \
		elif grep "^REQUIREDVERSION" $$H >/dev/null; then \
		    VERS=`date`; echo "New $${B}Version $$VERS"; \
		    echo "VERSION( $$B, \"$${B}Version\", \"$$VERS\" );" \
						    >>../NEWVERSIONS.cc; \
		fi; \
	    fi; \
	done; cd ..; \
	$(RM) -rf H; \
	echo "Finished updating header files"; \
	$(MV) ProxyIncludesT.cc PROXYTABLES.cc; \
	touch PROXYTABLES.cc

NEWVERSIONS.cc : PROXYTABLES.cc
	@touch NEWVERSIONS.cc

$(MODULENAME)VERSIONS.cc: NEWVERSIONS.cc
	@if cmp $(MODULENAME)VERSIONS.cc NEWVERSIONS.cc >/dev/null 2>&1; \
	then : ; \
	else echo "Creating $(MODULENAME)VERSIONS.cc"; \
	     $(RM) $(MODULENAME)VERSIONS.cc; \
	     cp NEWVERSIONS.cc $(MODULENAME)VERSIONS.cc; \
	fi

PROVIDEDVERSIONS.cc : $(MODULENAME)VERSIONS.cc
	@echo "Creating PROVIDEDVERSIONS.cc"; \
	$(RM) PROVIDEDVERSIONS.cc; \
	sed 's/^VERSION/PROVIDEDVERSION/' $(MODULENAME)VERSIONS.cc \
						>PROVIDEDVERSIONS.cc

NEWSTUBS.cc : PROXYTABLES.cc
	@echo "Creating NEWSTUBS.cc"; \
	$(RM) NEWSTUBS.cc; \
	mv ProxyIncludesS.cc NEWSTUBS.cc; \
	touch NEWSTUBS.cc

$(MODULENAME)STUBS.cc : NEWSTUBS.cc
	@if cmp NEWSTUBS.cc $(MODULENAME)STUBS.cc >/dev/null 2>&1; \
	then : ; \
	else echo "Creating $(MODULENAME)STUBS.cc"; \
	     $(RM) $(MODULENAME)STUBS.cc; \
	     cp NEWSTUBS.cc $(MODULENAME)STUBS.cc; \
	fi

CONSTRUCTORS.cc : PROXYTABLES.cc
	@$(RM) CONSTRUCTORS.cc; \
	mv ProxyIncludesC.cc CONSTRUCTORS.cc
	@touch CONSTRUCTORS.cc

ProxyIncludesP.cc: PROXYTABLES.cc

ifeq "$(MODULENAME)" "System"
LIBHVPATH = \
	. \
	$(TOP)/Includes/Libraries/SystemInterface/Portable \
	$(TOP)/Includes/Libraries/SystemInterface/$(PROCESSOR)
else
LIBHVPATH = $(COMMONHVPATH)
endif
LIBINCFLAGS = $(patsubst %,-I%,$(LIBHVPATH) $(PROXIFIEDDIR) $(PROXIFIEDCOMDIR) \
				$(HVPATH))

NEWCONSTRUCTORS.h : CONSTRUCTORS.cc
	@echo Creating NEWCONSTRUCTORS.h; \
	$(RM) D/CONSTRUCTORS.dep CONSTRUCTORS.o; \
	DEPENDENCIES_OUTPUT="D/CONSTRUCTORS.dep CONSTRUCTORS.h"; \
	export DEPENDENCIES_OUTPUT; \
	$(C++) $(COPTS) $(LIBINCFLAGS) -c CONSTRUCTORS.cc
	@$(RM) $@; \
	$(CONSTCOLLECT) CONSTRUCTORS.o >$@
# 	@$(RM) CONSTRUCTORS.o

CONSTRUCTORS.h : NEWCONSTRUCTORS.h
	@if cmp NEWCONSTRUCTORS.h CONSTRUCTORS.h >/dev/null 2>&1; \
	then : ; \
	else echo "Creating CONSTRUCTORS.h"; \
	     $(RM) CONSTRUCTORS.h; \
	     cp NEWCONSTRUCTORS.h CONSTRUCTORS.h; \
	fi

CONSTRUCTORINSTALLERS.cc :
	@echo "Creating CONSTRUCTORINSTALLERS.cc"; \
	$(RM) CONSTRUCTORINSTALLERS.cc; \
	(echo '#include "ConstructorInstaller.h"'; \
	 echo '#include "CONSTRUCTORS.h"') >CONSTRUCTORINSTALLERS.cc

CONSTRUCTORINSTALLERS.o : CONSTRUCTORS.h

Constructors.o : Constructors.s # use default rule to compile
Constructors.o : CONSTRUCTORS.h

$(MODULENAME)STUBS.o : $(MODULENAME)STUBS.cc
	@echo Compiling $(MODULENAME)STUBS.cc; \
	$(RM) D/$(MODULENAME)STUBS.dep $(MODULENAME)STUBS.o; \
	DEPENDENCIES_OUTPUT="D/$(MODULENAME)STUBS.dep $(MODULENAME)STUBS.o"; \
	export DEPENDENCIES_OUTPUT; \
	$(C++) $(COPTS) -DDONTREQUIREVERSION $(LIBINCFLAGS) -c \
				$(MODULENAME)STUBS.cc

$(PROXIFIEDLIB) : $(PROXIFIEDLIBOBJECTS)
	@echo "Creating $(PROXIFIEDLIB)"; \
	$(RM) $(PROXIFIEDLIB); \
	$(AR) cr $(PROXIFIEDLIB) $(PROXIFIEDLIBOBJECTS)
ifdef RANLIB
	$(RANLIB) $(PROXIFIEDLIB)
endif
	@echo "$(PROXIFIEDLIB) created"
    
clobber: clobberproxyheaders

clobberproxyheaders:
	cd $(PROXIFIEDDIR); $(RM) *.h
