
# This file is part of the Choices Operating System
# Developed by: The TAPESTRY Parallel Computing Laboratory
#		 University of Illinois at Urbana-Champaign
#		 Department of Computer Science
#		 1304 W. Springfield Ave.
#		 Urbana, IL	61801
#
# Copyright (c) 1987, 1988, 1989, 1990, 1991
# The University of Illinois Board of Trustees.
#	All Rights Reserved.
# CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
#
# Author: David W. Dykstra (dykstra@cs.uiuc.edu)
# Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
#
# Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
#   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
#
#
# Common makefile to make all versions of the Choices system
#
#


include Config.mk
#
# include a personal makefile for testing, projects, MPs, etc.
# skip if it's not there.
#
ifneq "$(CHOICESPERSONAL)" ""
PERSONALPATH=$(dir $(CHOICESPERSONAL))
-include $(CHOICESPERSONAL)
PERSONALHVPATH += $(PERSONALPATH)
PERSONALCCVPATH += $(PERSONALPATH)

endif

ifndef CONFIGROOT
	CONFIGROOT=../..
endif

include $(CONFIGROOT)/ChoicesCommon.mk

ifdef SpecialTop
	TOP=$(SpecialTop)
endif

include $(CONFIGROOT)/System/FileSystemsCommon.mk
ifdef Networking
include $(CONFIGROOT)/System/NetworkingCommon.mk
endif
ifdef Distributed
include $(CONFIGROOT)/System/DistributedCommon.mk
endif
ifdef SunOSUnixCompatibility
include $(CONFIGROOT)/System/UNIXCompatibility.mk
endif
ifdef UMaxUnixCompatibility
include $(CONFIGROOT)/System/UNIXCompatibility.mk
endif
ifdef DVM
include $(CONFIGROOT)/System/DVM.mk
endif
ifdef BSDNet
include $(CONFIGROOT)/System/BSDNetCommon.mk
endif
include $(CONFIGROOT)/System/CommonSources.mk

all: Applications Choices 

LIBSTAR_A := libStar.a

Choices: $(LIBSTAR_A) $(TOP)/Configure/Libraries/GeneralPurpose/$(ARCH)/libG.a

Applications: $(LIBSTAR_A) $(TOP)/Configure/Libraries/GeneralPurpose/$(ARCH)/libG.a $(TOP)/Configure/Libraries/SystemInterface/$(ABSTRACT_MACHINE)/libC.a
	@make -C $(TOP)/Configure/Applications/ARM

System.map: Choices
	$(NM) -n Choices > System.map

$(TOP)/Configure/Libraries/GeneralPurpose/$(ARCH)/libG.a:
	$(MAKE) -C $(TOP)/Configure/Libraries/GeneralPurpose/$(ARCH)

$(TOP)/Configure/Libraries/SystemInterface/$(ABSTRACT_MACHINE)/libC.a:
	$(MAKE) -C $(TOP)/Configure/Libraries/SystemInterface/$(ABSTRACT_MACHINE)

ifneq ($(ABSTRACT_MACHINE),)
COMMONHVPATH += \
	$(TOP)/Includes/ConfigurationFiles/$(ABSTRACT_MACHINE) \
	$(TOP)/Includes/MachineDependent/$(ABSTRACT_MACHINE) \
	$(TOP)/Includes/ConfigurationFiles/$(ABSTRACT_MACHINE)/$(ARCH) \
	$(TOP)/Includes/MachineDependent/$(ABSTRACT_MACHINE)/$(ARCH) \
	$(TOP)/Configure/System/$(ABSTRACT_MACHINE)/DefineHeaders \
	$(TOP)/Configure/System/$(ABSTRACT_MACHINE)/$(ARCH)/DefineHeaders \
	$(TOP)/Includes/MachineDependent/$(ABSTRACT_MACHINE)/unix \
	$(TOP)/Includes/MachineDependent/$(ABSTRACT_MACHINE)/$(ARCH)/unix

COMMONCCVPATH += \
	$(TOP)/MachineDependent/$(ABSTRACT_MACHINE) \
	$(TOP)/MachineDependent/$(ABSTRACT_MACHINE)/$(ARCH)

STANDARDHVPATH += \
	$(TOP)/Configure/System/$(ABSTRACT_MACHINE)
endif # $(ABSTRACT_MACHINE)


COMMONHVPATH += \
	$(TOP)/Includes/ConfigurationFiles/$(MACHINE) \
	$(TOP)/Includes/MachineDependent/$(MACHINE) \
	$(TOP)/Includes/ProcessorDependent/$(PROCESSOR) \
	$(TOP)/Includes/ConfigurationFiles/$(MACHINE)/$(ARCH) \
	$(TOP)/Includes/MachineDependent/$(MACHINE)/$(ARCH) \
	$(TOP)/Includes/ProcessorDependent/$(PROCESSOR)/$(ARCH) \
	$(TOP)/Includes \
	$(TOP)/Includes/Schedulers \
	$(TOP)/Includes/FileSystems \
	$(TOP)/Includes/Memory \
	$(DVMHPATH) \
	$(TOP)/Includes/IODevices \
	$(TOP)/Includes/Instrument \
	$(TOP)/Configure/System/$(MACHINE)/DefineHeaders \
	$(TOP)/Configure/System/$(MACHINE)/$(ARCH)/DefineHeaders \
	$(LOADERSHVPATH) \
	$(TOP)/Includes/MachineDependent/$(MACHINE)/unix \
	$(TOP)/Includes/MachineDependent/$(MACHINE)/$(ARCH)/unix \
	$(TOP)/Includes/Libraries/Unix/Emulation


COMMONCCVPATH += \
	$(TOP)/MachineDependent/$(MACHINE) \
	$(TOP)/ProcessorDependent/$(PROCESSOR) \
	$(TOP)/MachineDependent/$(MACHINE)/$(ARCH) \
	$(TOP)/ProcessorDependent/$(PROCESSOR)/$(ARCH) \
	$(TOP)/Kernel \
	$(TOP)/Memory \
	$(DVMCCPATH) \
	$(TOP)/Schedulers \
	$(TOP)/FileSystems \
	$(TOP)/IODevices \
	$(TOP)/Instrument \
	$(LOADERSCCVPATH) \
	$(TOP)/Common 


Make.dependencies: Config.mk

LIBS += $(TOP)/Configure/Libraries/GeneralPurpose/$(ARCH)/libG.a $(LIBSTAR_A)

#
# Optional sources/includes
#

ifdef OptionalSchedulerFile
OptionalSchedulerIncludes += HRNSchedulerUSS.h LIFOSchedulerUSS.h SchedList.h \
		MLFSchedulerUSS.h RRSchedulerUSS.h SRTSchedulerUSS.h \
		SchedulerUSS.h UNIXSchedulerUSS.h VMSSchedulerUSS.h
OptionalSchedulerSources += HRNSchedulerUSS.cc LIFOSchedulerUSS.cc SchedList.cc \
		MLFSchedulerUSS.cc RRSchedulerUSS.cc SRTSchedulerUSS.cc \
		SchedulerUSS.cc UNIXSchedulerUSS.cc VMSSchedulerUSS.cc
endif

################################################################################
# 	ADD OPTIONAL STUFF BEFORE HERE!!!!!
################################################################################
OPTIONALDEFINES += \
	$(OPTIONALFILESYSTEMDEFINES) \
	$(UNIXCOMPATIBILITYDEFINES) \
	$(NETWORKDEFINES) \
	$(DISTRIBUTEDDEFINES) \
	$(DVMDEFINES) \
	$(SIMPLEINSTRUMENTDEFINES) \
	$(BSDNETDEFINES) \
	$(PERSONALDEFINES)

OPTIONALINCLUDES += $(OPTIONALFILESYSTEMINCLUDES) \
	$(OptionalSchedulerIncludes) \
	$(UNIXCOMPATINCLUDES)  \
	$(NETWORKINCLUDES) $(CONDUITINCLUDES) \
	$(LOADERINCLUDES) \
	$(DISTRIBUTEDINCLUDES) \
	$(BSDNETINCLUDES) \
	$(PERSONALINCLUDES)

OPTIONALSOURCES += $(OPTIONALFILESYSTEMSOURCES) \
	$(OptionalSchedulerSources) \
	$(UNIXCOMPATSOURCE) \
	$(NETWORKSOURCES) $(CONDUITSOURCES) \
	$(LOADERSOURCES) \
	$(DISTRIBUTEDSOURCES) \
	$(BSDNETSOURCES) \
	$(PERSONALSOURCES)

OPTIONALSTARSOURCES += \
	$(FILESYSTEMSTARSOURCES) \
	$(UNIXCOMPATSTARSOURCE) \
	$(DISTRIBUTEDSTARSOURCES) \
	$(MACHSTARSOURCES) \
	$(BSDNETSTARSOURCES)

OPTIONALSTARINCLUDES += \
	$(FILESYSTEMSTARINCLUDES)  \
	$(UNIXSTARINCLUDES) \
	$(DISTRIBUTEDSTARINCLUDES) \
	$(BSDNETSTARINCLUDES)

OPTIONSVPATH += $(UNIXCOMPATVPATH) $(NETWORKVPATH) \
	$(DISTRIBUTEDVPATH) $(BSDNETVPATH)

OPTIONSHVPATH += $(UNIXCOMPATHVPATH) $(NETWORKHVPATH) \
	$(DISTRIBUTEDHVPATH) $(BSDNETHVPATH)


#
# VPATH, HVPATH and CCVPATH are overriding ChoicesCommon.mk
#
HVPATH = . \
	$(OPTIONSHVPATH) \
	$(EXTRAHVPATH) \
	$(COMMONHVPATH) \
	$(STANDARDHVPATH) \
	$(PERSONALHVPATH)

CCVPATH = . \
	$(OPTIONSVPATH) \
	$(EXTRAVPATH) \
	$(COMMONCCVPATH) \
	$(TOP)/Libraries/SystemInterface/$(PROCESSOR) \
	$(TOP)/Libraries/SystemInterface/$(ARCH)  \
	$(PERSONALCCVPATH)

# need to add the HVPATH to the VPATH so Proxify++ rule can find .h files
VPATH = $(subst $(space),:,$(CCVPATH) $(HVPATH))

################################################################################
# Kernel include files (Includes/)
################################################################################

KERNELINCLUDES += AddressTranslation.h AddressTranslator.h Allocator.h \
	Async.h AwaitedInterruptInterrupt.h Barrier.h \
	BlockedInfo.h CPU.h CPUSet.h \
	CharacterQueue.h Interrupt.h FaultErrorCode.h Gdb.h \
	InterruptManager.h Interrupts.h Kernel.h KernelCmdInstaller.h Lock.h \
	MachineConfiguration.h MadAllocator.h MemoryBlockAllocator.h \
	NonReentrant.h \
	ObjectProxy.h ObjectProxyManager.h PageTableConstants.h \
	ProcessContainer.h ProcessEvents.h ProcessorContext.h ProcessorException.h \
	RaidOutputStream.h \
	SoftInt.h SoftClock.h \
	StackSize.h Stream.h StupidAllocator.h \
	SystemTime.h ThisProcess.h TimeSliceInterrupt.h TimerManager.h \
	VersionObject.h WatchdogTimer.h AppInterface.h

PROXYKERNELINCLUDES += Class.h FreeRunningTimer.h \
	NameServer.h PeriodicTimer.h Pipe.h Process.h \
	Semaphore.h SystemInterface.h \
	TimeOfDay.h TimeoutTimer.h Timer.h TimingInterface.h \
	ConstructorDescriptor.h ProxyTableCopy.h 

MEMORYINCLUDES += AccessType.h AddressFlushQueue.h Domain.h DummyMemoryObject.h \
	MemoryObjectCache.h \
	SpecialPurposeCache.h \
	MemoryObjectView.h PagedMemoryObjectCache.h \
	PhysicalMemoryChain.h PhysicalMemoryManager.h \
	PhysicalMemoryRange.h \
	PhysicallyAddressableUnit.h PremappedMemoryObjectCache.h \
	ProtectionLevel.h \
	RootMemoryObjectCache.h Store.h \
	VirtualMemoryRange.h VMInterface.h \
	CacheUnitInfo.h Cache.h DummyCache.h RealCache.h UNIXInodeCache.h \
	Map.h Use.h TranslationMap.h CheckSum.h RangeLocks.h \
	SourceOfZerosMemoryObject.h ReplacementPolicy.h \
	LockHolder.h GlobalLRUReplacementPolicy.h CacheAsync.h \
	MemoryObjectAsync.h PageOutAsync.h \
	IOWindow.h

PROXYMEMORYINCLUDES += MemoryObject.h MemoryRange.h VMInterface.h

# Schedulers necessary include files (Includes/Schedulers/)

SCHEDULERSINCLUDES += FIFOScheduler.h RRScheduler.h \
	SemaphoreRegulatedFIFOScheduler.h SystemScheduler.h

# Instrument include files (Includes/Instrument/)

INSTRUMENTINCLUDES += CounterInstrument.h \
	EventerInstrument.h Instrument.h InstrumentBuffer.h InstrumentEvent.h \
	InstrumentFormat.h InstrumentManager.h SystemInstrumentManager.h \
	UserInstrumentManager.h InstrumentTranscript.h \
	InstrumentTypes.h TimerInstrument.h PerCpuTimerInstrument.h

PROXYINSTRUMENTINCLUDES += Instrument.h InstrumentManager.h \
	UserInstrumentManager.h

# I/O Devices include files (Includes/IODevices)

# PROXYIODEVICESINCLUDES += Disk.h
PROXYIODEVICESINCLUDES += CharacterInputStream.h

IODEVICESINCLUDES += Disk.h DiskScheduler.h \
	SerialLine.h MultiplexedOutputStream.h NullStreams.h


# ConfigurationFiles include files (Includes/ConfigurationFiles/$(MACHINE)/

CONFINCLUDES += \
    ADDRESS_TRANSLATION.h CPU_CONFIGURATION.h MACHINEKERNEL.h LOCK.h MAGIC.h \
    MAIN.h PAGE_TABLE_CONSTANTS.h PHYSICAL_MEMORY_OBJECT.h \
    PROCESSOR_CONTEXT.h PROCESSOR_ID.h REFERENCE_COUNT.h \
    TIMER_MANAGER.h TIMESLICE_TIMER.h \
    TYPES.h USER_MEMORY_RANGE.h

#
# Include the PROXYFILESYSTEMISINCLUDES with the EXTRATABLEONLYPROXYINCLUDES
#   rather than the PROXYINCLUDES because Proxify++ will be run in full on
#   those in the intermediate load module.
#
SYSTEMINCLUDES += $(KERNELINCLUDES) $(MEMORYINCLUDES) $(SCHEDULERSINCLUDES) \
	$(INSTRUMENTINCLUDES) $(IODEVICESINCLUDES) $(CONFINCLUDES) \
	$(FILESYSTEMINCLUDES) $(OPTIONALFILESYSTEMINCLUDES) \
	$(DISTRIBUTEDINCLUDES) $(DVMINCLUDES)

PROXYINCLUDES += $(PROXYKERNELINCLUDES) $(PROXYMEMORYINCLUDES) \
	$(PROXYSCHEDULERSINCLUDES) $(PROXYINSTRUMENTINCLUDES) \
	$(PROXYIODEVICESINCLUDES) $(PROXYFILESYSTEMINCLUDES) \
	$(PROXYCONDUITINCLUDES) $(DISTRIBUTEDPROXYINCLUDES) \
	$(PROXYBSDNETINCLUDES)
#
# ALLINCLUDES is used in ProxifyCommon.mk.  It should contain all includes
#  except those that we do not want to keep generated header files for,
#  namely the COMMONINCLUDES, the TABLEONLYPROXYINCLUDES and the STARINCLUDES
#
ALLINCLUDES += $(SYSTEMINCLUDES) $(EXTRAINCLUDES) \
		$(PROXYINCLUDES) $(EXTRAPROXYINCLUDES) $(OPTIONALINCLUDES)

STARINCLUDES += \
	$(EXTRASTARINCLUDES) $(COMMONSTARINCLUDES) $(OPTIONALSTARINCLUDES)

INCLUDES += $(ALLINCLUDES) $(COMMONINCLUDES) $(TABLEONLYPROXYINCLUDES) \
		$(STARINCLUDES) 

#
# List all the source files in the various source directories.
# A source file can be in any of the directories in the vpaths above but
# things will probably break if two directories contain a file of the
# same name.
#

# Kernel sources (except ObjectProxyVtable.cc because it needs to be
#   placed in a special section that's readable by everyone)

KERNELSOURCES += Main.cc Allocator.cc Assert.cc \
	Async.cc AwaitedInterruptInterrupt.cc Barrier.cc CPU.cc \
	CPUSet.cc CharacterQueue.cc Class.cc \
	ConstructorDescriptor.cc Interrupt.cc FreeRunningTimer.cc Gdb.cc \
	InterruptManager.cc \
	Kernel.cc KernelCmdInstaller.cc BootConsole.cc Lock.cc \
	NonReentrant.cc \
	Misc.cc MachineConfiguration.cc MadAllocator.cc \
	MemoryBlockAllocator.cc MemoryBlockRoutines.cc NameServer.cc \
	NewAndDelete.cc ObjectProxyKernel.cc ObjectProxyManager.cc \
	PeriodicTimer.cc Pipe.cc Process.cc Semaphore.cc \
	ProcessContainer.cc ProcessorContext.cc ProcessorException.cc ProxyTableCopy.cc \
	RaidOutputStream.cc \
	SoftInt.cc SoftClock.cc \
	StupidAllocator.cc SystemInterface.cc \
	SystemTime.cc TimeOfDay.cc \
	TimeSliceInterrupt.cc TimeoutTimer.cc Timer.cc TimerManager.cc \
	TimingInterface.cc VersionObject.cc WatchdogTimer.cc \
	AppInterface.cc

MEMORYSOURCES += AddressFlushQueue.cc AddressTranslation.cc \
	AddressTranslator.cc Domain.cc DummyMemoryObject.cc \
	MemoryObject.cc \
	SpecialPurposeCache.cc \
	MemoryObjectCache.cc MemoryObjectView.cc MemoryRange.cc \
	PagedMemoryObjectCache.cc \
	PhysicalMemoryChain.cc PhysicalMemoryManager.cc \
	PhysicalMemoryRange.cc \
	PremappedMemoryObjectCache.cc \
	RootMemoryObjectCache.cc Store.cc StoreInfo.cc \
	VirtualMemoryRange.cc VMInterface.cc \
	CacheUnitInfo.cc Cache.cc RealCache.cc DummyCache.cc \
	UNIXInodeCache.cc Map.cc Use.cc TranslationMap.cc CheckSum.cc \
	RangeLocks.cc SourceOfZerosMemoryObject.cc ReplacementPolicy.cc \
	GlobalLRUReplacementPolicy.cc CacheAsync.cc \
	MemoryObjectAsync.cc PageOutAsync.cc \
	IOWindow.cc PhysicallyAddressableUnit.cc 

# Schedulers necessary sources

SCHEDULERSSOURCES += FIFOScheduler.cc RRScheduler.cc \
	SemaphoreRegulatedFIFOScheduler.cc SystemScheduler.cc

# Instrument sources

ifdef Instrumentation
INSTRUMENTSOURCES += Instrument.cc InstrumentBuffer.cc \
	InstrumentManager.cc SystemInstrumentManager.cc \
	UserInstrumentManager.cc \
	InstrumentTranscript.cc \
	PerCpuTimerInstrument.cc
endif

ifdef SimpleInstrumentation
SIMPLEINSTRUMENTSOURCES += \
	SimpleInstruments.cc MemorySimpleInstruments.cc \
	FileSystemSimpleInstruments.cc
SIMPLEINSTRUMENTDEFINES += -DSIMPLE_INSTRUMENTATION
endif

# I/O Devices source files (IODevices)

IODEVICESSOURCES += SerialLine.cc DiskScheduler.cc \
	CharacterInputStream.cc \
	MultiplexedOutputStream.cc NullStreams.cc 

#
# $(PROXIFIEDSOURCES) should be last in this list so Proxify++ will only
#   run after compilations are done.  Proxify++ isn't as robust at catching
#   errors as the compiler is.
#
SYSTEMSOURCES += $(KERNELSOURCES) $(MEMORYSOURCES) $(SCHEDULERSSOURCES) \
	$(INSTRUMENTSOURCES) $(IODEVICESSOURCES) $(OPTIONALSOURCES) \
	$(FILESYSTEMSOURCES) $(DVMSOURCES) $(PROXIFIEDSOURCES) \
	$(SIMPLEINSTRUMENTSOURCES) 


STARSOURCES += \
	$(EXTRASTARSOURCES) $(COMMONSTARSOURCES) $(OPTIONALSTARSOURCES)


NONSTARSOURCES += $(SYSTEMSOURCES) $(EXTRASOURCES) $(COMMONSOURCES) 

SOURCES += $(NONSTARSOURCES) $(STARSOURCES)

#
# Putting all the Deps in one is too long, can get '*** Killed' on babym
#
DEPFILES += $(NONSTARSOURCES) $(PROCESSOR)asmdefs.h CONSTRUCTORS.h
DEPFILES2 += $(EXTRADEPFILES) $(PROXYDEPFILES) $(STARSOURCES)

$(PROCESSOR)asmdefs.h: $(LIBSTAR_A)

#
# Objects in a Choices system
#
NONSTAROBJECTS += $(addsuffix .o, $(basename $(NONSTARSOURCES)))
STAROBJECTS += $(addsuffix .o, $(basename $(STARSOURCES))) \
		ProxiableObjectStar.o PersistentObjectStar.o
OBJECTS += $(NONSTAROBJECTS) $(BSDNETOBJ)

###############################################################################
# Rules for generating Star classes from the prototype files
# in Includes/Common
#
# This uses the DEFINECLASS statements in the corresponding .cc files
# to create the hierarchy of stars.
#
#
ifdef MAKINGSTARS
$(LIBSTAR_A): $(STAROBJECTS)
	$(RM) $@
	$(AR) cr $@ $(STAROBJECTS)
ifdef RANLIB
	$(RANLIB) $@
endif

$(STAROBJECTS): $(STARSOURCES)

$(STARSOURCES): %Star.cc: %.cc Star.ccP Star.hP
	@echo "Generating $*Star"
	PARENT=`egrep "^DEFINECLASS|^DEFINEPERSISTENTCLASS" $< | awk -F, '$$2 == "$*" {print $$3}'` ;\
	if test "$$PARENT" = "Object" ;\
	then  \
		PARENT=ProxiableObject; \
	fi; \
	$(RM) $*Star.cc $*Star.h ; \
	$(TOP)/Tools/Misc/genstar $*Star $(TOP)/Includes/Common/Star "<T>" $* "<P>" $$PARENT 

else
$(LIBSTAR_A):
	@$(MAKE) -s MAKINGSTARS=y $@
endif


#PROXYTABLES.cc $(OBJECTS): $(LIBSTAR_A)
$(NONSTAROBJECTS): $(LIBSTAR_A) $(PROCESSOR)asmdefs.h

###############################################################################
#
# Things to help porting, make this and you'll only build non platform/machine 
# stuff.  (Hopefully portable easily)
# then come back to the specific (hard) codes...
#
##############################################################################
PORTOBJECTS += $(addsuffix .o, $(basename $(EXTRASOURCES)))
touchPortObjects:
	touch $(PORTOBJECTS)


#
# Rules to clean up after ourselves.
#
clobber: clobberchoices

clobberchoices:
	$(RM) Choices *Star.cc *Star.h *.a

#
# This is a Proxified system
#
MODULENAME+=System
#include $(CONFIGROOT)/System/ProxifyCommon.mk


#include $(CONFIGROOT)/Support.mk
