#
# This file is part of the Choices Operating System
# Developed by: The TAPESTRY Parallel Computing Laboratory
#		 University of Illinois at Urbana-Champaign
#		 Department of Computer Science
#		 1304 W. Springfield Ave.
#		 Urbana, IL	61801
#
# Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
#	All Rights Reserved.
# CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
#
# Author: Swee Boon Lim (sblim@cs.uiuc.edu)
# Author: Lee Lup Yuen (lup@cs.uiuc.edu)
# Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
#
# Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
#   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
#
#
# makefile for Choices applications
# Revision 1.0
#
# This is really a template that should be tailored for each Choices
# application.  The "CHANGE:" tag indicates which variables are customizable.
# Use gnumake to run this file.

EXTRADEFINES = -nostdinc -DCHOICES $(UNIXDEFINES)
EXTRAHVPATH = $(UNIXHVPATH)
EXTRAVPATH = $(UNIXVPATH)

DEPFILES2 = $(PROXYDEPFILES)
ALLINCLUDES = $(INCLUDES)

ifndef UNIX
INTERFACE = SystemInterface

ifdef CHOICESNODE
COMMONHVPATH = \
	$(CHOICESNODE)/Includes/Libraries/$(INTERFACE)/Portable \
	$(CHOICESNODE)/Includes/Libraries/$(INTERFACE)/$(PROCESSOR) \
	$(CHOICESNODE)/Includes/System/$(MACHINE) 
else
COMMONHVPATH = \
	../../../../Includes/Libraries/$(INTERFACE)/Portable \
	../../../../Includes/Libraries/$(INTERFACE)/$(PROCESSOR) \
	../../../../Includes/System/$(MACHINE)
endif # CHOICESNODE
else
COMMONHVPATH =  ../../../../Includes/System/$(MACHINE)

endif

ifdef CHOICESNODE
include $(CHOICESNODE)/Configure/ChoicesCommon.mk
TOP = $(CHOICESNODE)
else
include ../../../ChoicesCommon.mk
TOP = ../../../..
endif # CHOICESNODE

OBJECTS = $(addsuffix .o, $(basename $(SOURCES)))

DEPFILES = $(EXTRADEPFILES) $(SOURCES)

#
# Everything that cant somehow be remade.
#
EVERYTHING = Makefile $(SOURCES) $(INCLUDES)

#

ifdef CHOICESNODE
SYSTEM = $(CHOICESNODE)/Configure/System/$(MACHINE)
CONFLIBRARIES = $(CHOICESNODE)/Configure/Libraries
else
SYSTEM = ../../../System/$(MACHINE)
CONFLIBRARIES = ../../../Libraries
endif # CHOICESNODE
INTLIB = $(CONFLIBRARIES)/$(INTERFACE)/$(MACHINE)
GENLIB = $(CONFLIBRARIES)/GeneralPurpose/$(MACHINE)

SPECIALHVPATH = $(TOP)/Includes/Libraries/Unix/include/choices \
	$(TOP)/Includes/Libraries/Unix/include \
	$(TOP)/Includes/Libraries/Unix/include/sys 

APPLIBS = $(UNIXLIBS) $(CONFLIBRARIES)/Unix/$(MACHINE)/libU.a \
	$(CONFLIBRARIES)/Unix/$(MACHINE)/libUM.a

ifndef UNIX
#
# The LoaderOptions.mk file should set $(EXTRALDFLAGS),  $(EXTRASTARTS)
# $(EXTRALIBS)
# This is needed because some linkers need to specify command line options
#   and some need to specify the options in a file. Others also need
#   special libraries
# The application's makefile should set $(APPLIBS) to the list of extra
# libraries it requires.
#
include $(INTLIB)/LoaderOptions.mk
endif

STARTS = $(EXTRASTARTS) $(INTLIB)/crt0.o
ifdef SPARC
STDLIBS = $(INTLIB)/libC.a $(SYSTEM)/libSystem.a $(GENLIB)/libG.a \
	$(INTLIB)/libC.a $(GENLIB)/libG.a $(GCCLIB)
else
STDLIBS = $(INTLIB)/libC.a $(SYSTEM)/libSystem.a $(GENLIB)/libG.a $(GCCLIB)
endif
LIBS =  $(APPLIBS) $(STDLIBS) $(EXTRALIBS)

% : %.o $(STARTS) $(EXTRAOBJECTS) $(LIBS) 
ifndef NODEPENDENCIES
	@if [ ! -w D/$*.dep ]; then \
	    echo "Breaking the link on D/$*.dep"; \
	    cp D/$*.dep .; \
	    $(RM) D/$*.dep; \
	    cat $*.dep >D/$*.dep; \
	    $(RM) $*.dep; \
	else : ; \
	fi
	@echo "$@: $(STARTS) $(EXTRAOBJECTS) $(LIBS)" >>D/$*.dep
ifdef COLLECT
	@echo "Collecting $*.o"; \
	$(RM) _$@_ctdt.[cso]; \
	$(COLLECT) -o _$@_ctdt.s $*.o $(LIBS); \
	touch _$@_ctdt.s; \
	$(AS) -o _$@_ctdt.o _$@_ctdt.s
	@echo "Linking $@"; \
	$(RM) $@; \
	$(LD) -o $@ $(LDFLAGS) $(STARTS) $*.o _$@_ctdt.o $(EXTRAOBJECTS) $(LIBS)
	@$(RM) _$@_ctdt.[cso]
else
	@echo "Linking $@"; \
	$(LD) -o $@ $(LDFLAGS) $(EXTRALDFLAGS) $(STARTS) $*.o $(EXTRAOBJECTS) $(LIBS)
endif

else

% : %.o
	@echo "Linking $^"; \
	$(C++) -o $@ $^ $(LIBS)

endif


clobber: clobberProg

clobberProg:
	rm -rf $(UNIXPROG) $(OBJECTS)

install:
	$(INSTALL) $(UNIXPROG) $(TARGETDIR)
