#ifdef __GNUC__
#pragma implementation
#endif

#include "AsyncMessageContainer.h"
#include "Semaphore.h"
#include "AsyncMessage.h"
#include "iPSCMessage.h"
#include "Message.h"
#include "OutputStream.h"
#include "Byte.h"

#ifdef __GNUC__
#pragma implementation
#endif

ROQMessage::ROQMessage()
{
sem = 0;
satisfied = 0;
mesg = 0;
type = 0;
}


AsyncMessageContainer::AsyncMessageContainer(char*name)  : MessageContainer(name)
{
InQSem = new MutexSemaphore;
}

/*
 * Is there a message of the required type 
 * in the message queue. This is asynchronous.
 * If not message of the type is in the messageBuffer
 * then just return.
 */

int
AsyncMessageContainer::probe( int typemask)
{
#ifdef AS_DEBUG
Console() << " head " << head << " tail " << tail << "\n" << eor;
#endif

int i;
i = tail;
while(1) {
	if(messageBuffer[i] && typeMatch(((iPSCMessage*)messageBuffer[i])->getType(),typemask)) 
		return 1;
	i = (i+1) % messageBufferSize;
	if (i == head) return 0; 
}	
}
/*
 * This is synchronous version of the
 * message passing system.
 */

void
AsyncMessageContainer::cprobe( int typemask)
{
while(!probe(typemask)) {}
}

// increment head mod messageBufferSize
// if it equals tail then error 

void
AsyncMessageContainer::nextIndex(int* index) {
 mutwait.acquire();
*index = head = (head +1 ) % messageBufferSize;

if( ((head + 1) % messageBufferSize) == tail)  {
	compact();
}
 mutwait.release();
}

/*
 * compact the message queue 
 */

void
AsyncMessageContainer::compact()
{
// generate buffer full if this still does not
// reduce the size of the wasted space
}
/*
 * This is synchronous receive that waits
 * until a message  of the appropriate type
 * is available. Fill in the message structure.
 */

void
AsyncMessageContainer::receive(Message* message)
{
#ifdef AS_DEBUG
Console() << "AsyncMessageContainer::recieve \n" << eor;
#endif

cprobe(((iPSCMessage*)message)->getType());

InQSem->P();
getMessage(message);
InQSem->V();

}

int 
AsyncMessageContainer::ireceive(Message* message)
{
#ifdef AS_DEBUG
Console() << "AsyncMessageContainer::irecv \n" << eor;
#endif

int ret = -1;
	InQSem->P();
if(probe(((iPSCMessage*)message)->getType())) {
	getMessage(message);
} else {
	ret = addToROQ(message);
}
	InQSem->V();
	return ret;
}

int
AsyncMessageContainer::getMessage(Message *message) {

long typemask = ((iPSCMessage*)message)->getType();
int len = message->size();

int i;
i = tail;

while(1) {
        if(messageBuffer[i] && typeMatch(((iPSCMessage*)messageBuffer[i])->getType(),typemask)) {
// increment tail if has been removed. 
                if(i == tail) 
                        tail = (tail +1) % messageBufferSize;
                LastMessage.setType(((iPSCMessage*)messageBuffer[i])->getType());
                LastMessage.setSize(messageBuffer[i]->size());
 LastMessage.setSourceId(((iPSCMessage*)messageBuffer[i])->getSourceId());
                if(LastMessage.size() > 0) {
                        if( len >= LastMessage.size())
                        CopyBlock((char *)messageBuffer[i]->data(),(char *)message->data(),LastMessage.size());
                        else CopyBlock((char *)messageBuffer[i]->data(),(char *)message->data(),len);
                }
 	message->setUserData(message->data(), messageBuffer[i]->size());
    	message->setcType( messageBuffer[i]->getcType());
    	((iPSCMessage*)message)->setType(((iPSCMessage*) messageBuffer[i])->getType());

        messageBuffer[i] = 0;
        qlen--;
                return 1;
        }
        i = (i + 1) % messageBufferSize;
        if (i == head) return 0;
}
return 0;
}
int
AsyncMessageContainer::getACopyFromInQ( long typemask) {

int  i = tail;

while(1) {
	if(messageBuffer[i] && typeMatch(((iPSCMessage*)messageBuffer[i])->getType(),typemask)) {
				LastMessage.setType(((iPSCMessage*)messageBuffer[i])->getType());
				LastMessage.setSize(messageBuffer[i]->size());  
                LastMessage.setSourceId(((iPSCMessage*)messageBuffer[i])->getSourceId());

				return 1;
	}
	i = (i +1) % messageBufferSize;
	if (i == head) return 0; 
}	
}

// for receives only
long
AsyncMessageContainer::checkMask(long type,long typesel) {
  long          i;              /* loop variable */
  long          tsel;           /* temp typesel */

  /* determine if this type is in the typesel */

  tsel = typesel & 0x7fffffff;

  for (i = (long)0; tsel != (long)0; tsel >>= (long)1) {
    if ((tsel & LOWBIT) && i == type)
      return 1;
    i++;
    if ((i == BIGTYPE) && (type >= BIGTYPE))
      return 1;
  }

  return 0;
}

// This function determines whether or not there is a match
// between the typemask and type. type is a message type and
// typemask maybe a type, an or of typemasks or -1 to match all types

long
AsyncMessageContainer :: typeMatch(long type,long typesel) 
{

long match = 0;

if (type < -1 )
        match = checkMask(type,typesel);

return ( match || (type == typesel) || typesel == ANYTYPE);
}

int
AsyncMessageContainer::infoCount()
{
return LastMessage.size();
}

int
AsyncMessageContainer::infoType()
{
return LastMessage.getType();
}

int
AsyncMessageContainer::infoNode()
{
return LastMessage.getDestId();
}
#include "ClassInstaller.h"
DEFINECLASS("AsyncMessageContainer",AsyncMessageContainer,MessageContainer,0, 0);
