#ifdef __GNUC__
#pragma implementation
#endif


/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Labora ory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted und
er license agreement.
 *
 * Author: Nayeem Islam (nislam@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campb
ell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICL
ASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Gra
nt No. 1-5-37411.
 */


#include "OutputStream.h"
#include "Transport.h"
#include "EthernetTransfer.h"
#include "Header.h"
#include "DistEthernet.h"
#include "Assert.h"
#include "TimeoutTimer.h"
#include "AtLeastOnce.h"
#include "BitArray.h"
#include "Semaphore.h"


AtLeastOnce::AtLeastOnce(Transport* trans) : Reliability(trans)
{
timeout = 1000000; // 10 seconds
MaxNumTimeouts = 10;
gate =  new Semaphore(window);
mutex =  new MutexSemaphore;
sequenceId = 0;
ind = new BitArray(window);
}

int
AtLeastOnce::send(char* base,char *prev,Address *dest,int len)
{
char* temPkt = prev - sizeof(ReliabilityHeader);
ReliabilityHeader* relPkt =(ReliabilityHeader*) temPkt; 
relPkt->ack = 0;

gate->P();
mutex->P();

relPkt->sequenceNum = ++sequenceId;
int index = ind->bitSearch(0,0,1); // inset a get index here 
ind->set(index); // inset a get index here 
Console() << "Got index" << index << eor;
mutex->V();
Ack[index] = 0; // add an array
timer[index] = new TimeoutTimer; 

int numTimeOuts = 0;

while((numTimeOuts < MaxNumTimeouts) && Ack[index] !=1 ){
	numTimeOuts++;
	//inspect(relPkt);
	transfer->send(base, (char* )relPkt,dest,len);
	timer[index]->start (timeout);// 10 seconds
	timer[index]->await();
	if(Ack[index] == 0) Console() << "Resending \n" << eor;
}
Ack[index] = 0;
ind->clear(index);
gate->V();
Console() << "sent packet" << eor; 
}


void
AtLeastOnce::receive( char *packet, Address *etherAddr,Address *dest,NetworkBuffer *nb)
{
	ReliabilityHeader *ato =  (ReliabilityHeader *)packet;
	//inspect(ato);

// if it is an ack stop timer.
if(ato->ack) {
	Console() << "Got Ack with index " << ato->delmask<<eor;
	// Unused! int time = timer[ato->delmask]->stop();
	timer[ato->delmask]->start(1);
	Ack[ato->delmask] = 1; // need to change this to an array
} else {
	Console() << "Send Ack with delmask " << ato->delmask<<eor;
	int length = sizeof(AckHeader);
	if(length < 64 ) length = 64;
	char * np = new char[length];

	AckHeader *ack = (AckHeader*) np;	

	ack->reliabilityHeader.sequenceNum = ato->sequenceNum;
	ack->reliabilityHeader.ack = 1;
// send ack
	ack->reliabilityHeader.delmask = ato->delmask;
	
	transfer->send((char *)ack,(char*)&(ack->reliabilityHeader),etherAddr,length);

// pass to transport layer.

	char * newpos = packet + sizeof(ReliabilityHeader);
	transport->receive(newpos,etherAddr,dest,nb);
    }
}
void
AtLeastOnce::inspect(ReliabilityHeader *reliabilityHeader)
{
        Console() << "\n  ack      : " << reliabilityHeader->ack;
        Console() << "    sequenceNum      : " << reliabilityHeader->sequenceNum;
        Console() << "    delmask      : " << reliabilityHeader-> delmask;
        Console() << "\n" << eor;
}
