#ifdef __GNUC__
#pragma implementation
#endif

/* 
 * All VMTP client managemnet routines are 
 * handled in this module.
 */
#include "ClientCSR.h"
#include "Address.h"
#include "Header.h"
#include "Message.h"
#include "Container.h"
#include "ReceivingTransport.h"
#include "TimeoutTimer.h"
#include "EthernetTransfer.h"
#include "Reliability.h"
#include "BitArray.h"
#include "NetworkBuffer.h"
#include "NameServer.h"
#include "Byte.h"

#ifdef __GNUC__
#pragma implementation
#endif

//#define CL_DEBUG 1
ClientCSR::ClientCSR (Container* loc, Address* e) {
etherAddr = new EthernetAddress;
csr_host = new EthernetAddress;
ByteCopy(e->bytes(),etherAddr->bytes(),6);
*etherAddr = *e;
client_id = loc;

response = 0; done =0;

#ifdef CL_DEBUG
Console() << "New ClientCSR" << eor;
#endif
}

/*
 * We have a chain of data pointers and 
 * a chain of pointers to network buffers.
 * 3 things may happen
 *
 * 1) if time_limit expires --> resend packets.
 * 2) if receive a probe --> resend unreceived packets
 * 3) if receive a response --> done. return
 *
 */

void
ClientCSR::await_response(Address *dest,int np, Reliability *rel)
{
reliability = rel;
numPackets = np;
numTimeOuts = 0;
response = 0;
state = AWAIT_RESPONSE;
csr_host =dest;
#ifdef CL_DEBUG
Console() << "about to enter loop\n" << eor;
#endif
while((numTimeOuts < MaxNumTimeouts) && !response ){
        timer->start (time_limit);// 1 millisecond
        timer->await();
        numTimeOuts++;
#ifdef CL_DEBUG
	Console() << "ClientCSR::await_response::take action \n" << eor;
#endif
	if(state == RECV_RESPONSE) break;
	if((timer->residual() == 0) && !response && (numTimeOuts < MaxNumTimeouts) ) // send unsent packets
            send(); // Means we timed our or got a 
			// probe. 
	//Console() << "ClientCSR::try again\n" << eor;
}
	if(state == RECV_RESPONSE) multiPacketResponse();
//Console() << "ClientCSR::await_response::exiting loop\n" << eor;
}

/*
 * We have received a response and are in the process
 * or receiving the response.
 */

void ClientCSR::recv_response( )
{
#ifdef CL_DEBUG
Console() << "ClientCSR::recv_response" << eor;
#endif
state = RECV_RESPONSE;
done = 0;
unlock(); // i do not like this unlocking without matching lock
timer->stop(); // reschedule the process waiting in await_response
}

void
ClientCSR::multiPacketResponse()
{

#ifdef CL_DEBUG
Console() << "ClientCSR::multiPacketResponse\n" << eor;
#endif

while((numTimeOuts < MaxNumTimeouts) && !response ){
        timer->start (time_limit);// 1 millisecond
        timer->await();
        numTimeOuts++;
// Console() << "ClientCSR::multiPacketResponse::something happended \n" << eor;
	if((timer->residual() == 0) && !response && (numTimeOuts < MaxNumTimeouts) ) // send unsent packets
            sendProbe(); // Means we timed our or got a 
			// probe. 
}

}

void
ClientCSR::probe(int mask)
{
	transmission_mask = mask;
        //Unused! int time = timer->stop();
}
void
ClientCSR::responseDone()
{
	response = 1;
        int time = timer->stop();
	//Console() << "response done time left " << time << eor;
// free all the network buffers
	for ( int j=0; j< numPackets; j++) {
		//Console() << "Releasing buffer" << eor;
        	nblist[j]->release();
	}
	
}

void ClientCSR::flush(int numP) 
{ 
trans_id++;
delete message; // is this ok.
retrans_count = 0;
transmission_mask = 0;
tm->set(0,numP);
}

void ClientCSR::send()
{
Console() << "Resending information" << eor;
Console() << "num packets " <<numPackets << eor;
for ( int j=0; j< numPackets; j++) {
#ifdef VCHOICES
//    if ( !tm->get( j ) )
        reliability->send(nblist[j]->packet(), chain[j],csr_host,nblist[j]->length(),nblist[j]);
#else     if ( !tm->get( j ) )
        reliability->send(nblist[j]->kpacket(), chain[j],csr_host,nblist[j]->length(),nblist[j]);
#endif
}

}


void ClientCSR::sendProbe() {
	Console() << "SendingProbe" << eor;
/*
 * Used with response to describe when packets
 * are not received
 */
        int length = sizeof(Header);
        if(length < 64 ) length = 64;

        NetworkBuffer *nb =  netFree->getBuffer();

#ifdef VCHOICES
        char *pkt = nb->packet();
#else
        char *pkt = nb->kpacket();
#endif
        nb->setLength(length);
        Header *probe = (Header*) pkt;
        probe->transportHeader.probe = 1;
        probe->reliabilityHeader.delmask = transmission_mask; 
        transfer->send((char*) probe,(char*)probe,csr_host,length,nb);
}

void
ClientCSR::endTrans() {
        state = IDLE;
        first_packet = 0;
        currSize = 0;
}

