#ifdef __GNUC__
#pragma implementation
#endif


/*
 * Dispathing and message creation routines.
 */

#include "OutputStream.h"
#include "MessageContainer.h"
#include "ContainerRepresentative.h"
#include "GroupRepresentative.h"
#include "UniRepresentative.h"
#include "SendGroupRepresentative.h"
#include "Reliability.h"
#include "AtLeastOnce.h"
#include "AsyncMessage.h"
#include "Message.h"
#include "ContainerManager.h"
#include "Assert.h"
#include "Byte.h"
#include "Header.h"
#include "NetworkBuffer.h"
#include "DispatchMessage.h"
#include "DistributedNameServer.h"
#include "EthernetTransfer.h"
#include "Reliability.h"
#include "Semaphore.h"
#include "ContainerGroup.h"
#include "RemoteCSR.h"

#ifdef __GNUC__
#pragma implementation
#endif

extern Transport *NetTransport ;

extern int dumpBuf(unsigned char *c, int offset, int len);
extern int dumpHex(unsigned char *c, int len);

DispatchMessage::DispatchMessage()
{
reliability = new Reliability(this);
//transfer = new EthernetTransfer(reliability);
Console() << "DispatchMessage::DispatchMessage" <<
	"Created " << eor; 

}

/* 
 * Routines to deliver messages to the appropriate
 * MessageContainers. Knowledge of ContainerGroups are 
 * necessary in this part of the dispatching process.
 */

void
DispatchMessage::dispatch(Message *msg,Address *destAddr, ProxiableObject* destMessagePort)
{
	MessageContainer* dest=0;
	if(destAddr->isGroup()) {
		if(!MPManager) {
			MPManager = StandardNameServer->lookup("containerManager",ContainerManagerClass);
		}	
		Assert(MPManager != 0);
		ContainerGroup* pg = MPManager->find(destAddr->bytes());
		Assert( pg != 0);
		pg->put(msg);
	} else {
		dest = StandardDistributedNameServer->lookup(destMessagePort);
		Assert ( dest != 0);
		dest->put(msg);
/*
		msg->setDestination(dest->cr);
		putPort(msg);
*/
	}
}


void
DispatchMessage::dispatch(char *newpos,TransportHeader *header,Address *remote_machine,Address *destAddr,NetworkBuffer *nb)
{
	MessageContainer* dest=0;
	if(destAddr->isGroup()) {
#ifdef T_MESSAGE_DEBUG
	Console() << "received group message " << eor;
        dumpHex((unsigned char*)destAddr->bytes(),6);
	Console() << "\n" << eor;
#endif
		if(!MPManager) {
			MPManager = StandardNameServer->lookup("containerManager",ContainerManagerClass);
		}	
		Assert(MPManager != 0);
		ContainerGroup* pg = MPManager->find(destAddr->bytes());
		Assert( pg != 0);
		pg->put(makeMessage(newpos,pg, header,remote_machine,header->size,nb));
	} else {
		dest = StandardDistributedNameServer->lookup(header->destMessagePort);
		Assert ( dest != 0);
// changed from putport to put
		dest->put(makeMessage(newpos,dest,header,remote_machine,header->size,nb));
	}
}

Message*
DispatchMessage::getMessage(TransportHeader *header,Address *remote_machine)
{
        Message *msg;
        if(header->ctype == RPC_MESSAGE) {
                        RemoteCSR* rcsr = findRemoteCSR( (MessageContainer*)header->replyMessagePort,remote_machine,(MessageContainer*)header->destMessagePort); // use reply port
			if (rcsr->cacheReply == 0) rcsr->cacheReply =  new UniRepresentative(remote_machine, NetTransport,(MessageContainer *) header->replyMessagePort );
                        RPCMessage *rmsg = new RPCMessage;
                        rmsg->setReplyToPort(rcsr->cacheReply);
			rmsg->setServer((MessageContainer*) header->destMessagePort);
                        msg = rmsg;
                } else if (header->ctype == AR_MESSAGE)
{
                        AsyncMessage *amsg = new AsyncMessage;
                        amsg->setType(header->type);
                        msg = amsg;
                } else {
                        msg = new Message(0);
                }
                msg->setSize(header->size);
        return msg;
}

Message*
DispatchMessage::makeMessage(char* newpos, Container* dest, TransportHeader *header,Address* remote_machine, int Size,NetworkBuffer *nb)
{
#ifdef T_MESSAGE_DEBUG
                Console() << "DispatchMessage::passUp " << eor;
#endif
                Message *msg = getMessage(header, remote_machine);
                //msg->setUserData(newpos + sizeof(TransportHeader), header->size);
// ATTN: Remove this copy
                ByteCopy( newpos + sizeof(TransportHeader), msg->data(),Size);
                if(dest) msg->setDestination(dest->cachedCR());
                msg->setcType(header->ctype);
                msg->setTime(nb->getTime());
#ifdef T_MESSAGE_DEBUG
                Console() << "DispatchMessage::about to putport" << eor;
#endif
                return msg;
}

#include "ClassInstaller.h"
DEFINECLASS("DispatchMessage",DispatchMessage,Transport,0, 0);
