#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Nayeem Islam (nislam@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * DistributedNameServer.cc: implementation of the DistributedNameServer class.
 */

#include "Assert.h"
#include "Class.h"
#include "Domain.h"
#include "AssocTable.h"
#include "NameServer.h"
#include "Process.h"
#include "ProxiableObject.h"
#include "Raid.h"
#include "Byte.h"
#include "String.h"
#include "DistributedNameServer.h"
#include "ContainerManager.h"
#include "DistEthernet.h"
#include "MessageContainer.h"
#include "ContainerGroup.h"
#include "UniRepresentative.h"
#include "Message.h"
#include "NameServer.h"
#include "ContainerGroup.h"
#include "SendGroupRepresentative.h"
#include "GroupRepresentative.h"
#include "NetworkTable.h"
#include "Address.h"
#include "Ran.h"

#ifdef __GNUC__
#pragma implementation
#endif

DistributedNameServer *StandardDistributedNameServer = 0;

extern int dumpHex(unsigned char *c, int len);
extern void Seed(int,int);
extern double Random(int);
extern Address* LocalAddress;

// #define DNS_DEBUG 1
DistributedNameServer::DistributedNameServer( NameServer*_parent , Transport *_transport, Transport *_localTransport): NetworkNameServer(_parent, _transport, _localTransport)
{
#ifdef DNS_DEBUG 
	Console() << "About to create ContainerManager " << eor;
#endif
        MPManager = new ContainerManager;
        StandardNameServer->bind("containerManager",MPManager);
        int seed;
        ByteCopy(LocalAddress->bytes()+2,&seed,4);
        srandom(seed);
// well know address reserved for name server
#ifndef MULTIMAX
	EthernetAddress *n = new EthernetAddress(0x01,0x01,0x01,0x01,0x01,0x01);
	n->makeGroup();
#else
	EthernetAddress *n = new EthernetAddress(0xFF,0xFF,0xFF,0xFF,0xFF,0xFF);
#endif
	ContainerGroup * cg = new ContainerGroup(0,n);
	cg->add(localContainer);
        broadcastMessageContainer = new UniRepresentative(n, _transport,localContainer );
	//localContainer->cr = broadcastMessageContainer;
#ifdef DNS_DEBUG
	Console() << "DNS created" << eor;
#endif
}

ProxiableObject*
DistributedNameServer:: manage(MessageContainer* obj)
{
return MPManager->manage(obj);
}

ProxiableObjectRef
DistributedNameServer::check(MessageContainer* o)
{
    return MPManager->check(o); // check o 
}
void
DistributedNameServer::add(ContainerGroup* o)
{
    MPManager->add(o); // add o
}

// Current a broadcast should be changed to proper multicast
void
DistributedNameServer::multicast(char *name,ProxiableObject *obj,NSMessageType type)
{
	Message *msg = new Message; // an async message 
        int size = strlen(name)+1+sizeof(BroadcastMessage);
	char *buf = new char[size];
	BroadcastMessage *bmsg = (BroadcastMessage *) buf;
  	bmsg->type = type; 
	if(type == BIND){
		LocalAddress->copyTo(bmsg->ethernetAddress);
#ifdef DNS_DEBUG
		Console () << " DNS regular Address " << eor; 
#endif
	} else { 
		ByteCopy(((ContainerGroup*)obj)->getAddress()->bytes(),bmsg->ethernetAddress,6); // multicast address
#ifdef DNS_DEBUG
		Console () << " DNS Multicast Address " << eor; 
#endif
		dumpHex(bmsg->ethernetAddress,6);
	}
  	bmsg->mport = (MessageContainer*)obj; 
  	strcpy(buf+sizeof(BroadcastMessage), name);

  	msg->setSize(size);
  	msg->setData(buf);
  	msg->setDestination(broadcastMessageContainer);
  	msg->send();
#ifdef DNS_DEBUG
	Console() << "Done Sending" << eor;
#endif
  	//delete msg;
  	//delete buf;
}

void
DistributedNameServer::checkMessageContainer() 
{
 while (!localContainer->isEmpty()) {
 Console() << "DistributedNameServer::checkMessageContainer\n" << eor;
#ifdef DNS_DEBUG
#endif

  char *buf = new char[1024];
  Message *msg = new Message(buf,1024,1);
  msg->setReceivePort(localContainer);
  msg->receive();

  BroadcastMessage *bmsg = (BroadcastMessage *) msg->data();
#ifdef DNS_EBUG
  Console() << "bmsg->type\n" << eor;
  Console() << bmsg->type << eor;
  Console() << "bmsg->ethernetAddress\n" << eor;
  dumpHex(bmsg->ethernetAddress,sizeof(bmsg->ethernetAddress));
Console() << (char*)msg->data() + sizeof(BroadcastMessage) << eor;
#endif
  if (ByteCompare(bmsg->ethernetAddress, LocalAddress->bytes() ,sizeof(DistEthernetAddress))!= 0) {
	switch (bmsg->type) {
		case BIND:
		Console() << "binding" << eor;
#ifdef DNS_EBUG
#endif
                ContainerRepresentative *newPort = new UniRepresentative(new EthernetAddress(bmsg->ethernetAddress), aTransport, (MessageContainer*)bmsg->mport);
        	NameServer::bind((char *)msg->data() + sizeof(BroadcastMessage), newPort);
		break;
		case UNBIND:
		NameServer::unbind((char *)msg->data() + sizeof(BroadcastMessage));
		break;
		case PGBIND:
// send rights only
#ifdef DNS_EBUG
		Console() << " ====  PG ====" << eor;
#endif
// create with no pointers to a local container group
		GroupRepresentative* pg = new GroupRepresentative(MPManager->makeAddress((unsigned char*)bmsg->ethernetAddress),aTransport,(ContainerGroup*) 0);
        	NameServer::bind((char *)msg->data() + sizeof(BroadcastMessage), pg);
		//MPManager->add(pg);
#ifdef DNS_DEBUG
		Console() << " added portgroup named "
 		<< pg->getName() << " address\n" << eor;
			dumpHex((unsigned char*)pg->getAddress()->bytes(),6);
#endif
		break;
/*
		case RPGBIND:
#ifdef DNS_EBUG
		Console() << "  RPG BIND " << eor;
#endif
		RPCRPortGroup* rpg = new RPCRPortGroup(msg->data() + sizeof(BroadcastMessage),MPManager->makeAddress((unsigned char*)bmsg->ethernetAddress),bmsg->mport,aTransport);
        	NameServer::bind(msg->data() + sizeof(BroadcastMessage), rpg);
		MPManager->add(rpg);
#ifdef DNS_DEBUG
		Console() << " added portgroup named "
 		<< rpg->getName() << " address\n" << eor;
			dumpHex((unsigned char*)rpg->getAddress()->bytes(),6);
#endif
		break;
*/
		default:
			Console() << "should never get here" << eor;
		break;
	}
  }
  delete msg;
}
}
#include "ClassInstaller.h"
DEFINECLASS("DistributedNameServer",DistributedNameServer,NetworkNameServer,0,0);
