#ifdef __GNUC__
#pragma implementation
#endif

#include "HypercubeMessageContainer.h"
#include "AsyncMessageContainer.h"
#include "iPSCMessage.h"
#include "Lock.h"
#include "Semaphore.h"
#include "OutputStream.h"
#include "Byte.h"

#ifdef __GNUC__
#pragma implementation
#endif

// #define HMP_DEBUG 1

/*
 * I have done away with the request satisfied queue.
 * I think it is unncessary
 */

HypercubeMessageContainer::HypercubeMessageContainer(char* name) :
 AsyncMessageContainer(name) 
{
	ROQhead = ROQtail = 0;
}

void
HypercubeMessageContainer::nextMessageId(int* index) {
 mutwait.acquire();
*index = ROQhead;
ROQhead = (ROQhead +1 ) % requestStorageSize;
 Assert (( (ROQhead + 1) % requestStorageSize) != ROQtail) ("HypercubeMessageContainer Overflow");
 mutwait.release();
}

int
HypercubeMessageContainer::addToROQ(Message *message)
{
int index;
nextMessageId(&index);

ROQ[index].mesg = 0;
ROQ[index].userm = message;
ROQ[index].sem = new Semaphore(0);
ROQ[index].satisfied = 0;
ROQ[index].type = ((iPSCMessage*)message)->getType();
#ifdef HMP_DEBUG
Console() << "HypercubeMessageContainer::addToROQ " <<
	"index is " << index << " type " << ROQ[index].type << eor;
#endif
return index;
}

/*
 * Remove any requests out standing for this
 * message. If the message has arrived discard it.
 */

void
HypercubeMessageContainer::cancel( int messageId)
{
        updateROQ(messageId);
}
/*
 * wait for a message of the appropriate type to
 * arrive at this port.
 */
int
HypercubeMessageContainer::wait(int messageId){

#ifdef HMP_DEBUG
Console() << "HypercubeMessageContainer::wait "  << eor;
#endif

        while(ROQ[messageId].satisfied == 0){}
// for multi - with not semaphores.

        ROQ[messageId].sem->P();
#ifdef HMP_DEBUG

Console() << "GOT Message" << eor ;
#endif

        ROQ[messageId].lock.acquire();
        updateROQ(messageId);
}


int
HypercubeMessageContainer::done(int messageId)
{
        // ROQ[messageId].sem->P(); QUO VADIS
        ROQ[messageId].lock.release();
        if(ROQ[messageId].satisfied){
                updateROQ(messageId); // does unlocking
                return 1;
        } else {
                ROQ[messageId].lock.release();

                return 0;
        }
}

void
HypercubeMessageContainer::put(Message *message){

#ifdef HMP_DEBUG
Console() << "In HypercubeMessageContainer::put" << 
	"ROQTail " << ROQtail << " ROQHead " << ROQhead << eor;
#endif

if(ROQprobe( ((iPSCMessage*)message)->getType() ) ){
#ifdef HMP_DEBUG
Console() << "probe successful         \n" << eor;
#endif
	int i;
	i = ROQtail;
	while(1) {
		ROQ[i].lock.acquire();
       		 if((ROQ[i].satisfied == 0) && typeMatch(ROQ[i].type,((iPSCMessage*)message)->getType())) {
#ifdef HMP_DEBUG

		Console() << "found request" << eor;
#endif
			ROQ[i].mesg = 0;	
			Message *nextMessage = message;
                        Message *msg = ROQ[i].userm;
                        Assert (nextMessage != 0);
                        Assert (msg->maxSize() >= nextMessage-> size())
                                ("user buffer too small for message data");
                        ByteCopy (nextMessage -> data (), msg->data(), nextMessage -> size ());
                        msg->setUserData(msg->data(), nextMessage->size());
                        msg->setcType( nextMessage->getcType());
                        ((iPSCMessage*)msg)->setType( (( iPSCMessage*)nextMessage)->getType());
                        delete nextMessage;
			ROQ[i].satisfied = 1;
			ROQ[i].lock.release();
			ROQ[i].sem->V();
		} else ROQ[i].lock.release();

        	i = (i+1) % requestStorageSize;
       		if (i == ROQhead) return;
	}
	
} else MessageContainer::put(message);

}

int
HypercubeMessageContainer::ROQprobe( long typemask)
{
int i;
i = ROQtail;
while(1) {
        if(!ROQ[i].satisfied && (ROQ[i].type != 0 ) && typeMatch(ROQ[i].type,typemask))
                return 1;
        i = (i+1) % requestStorageSize;
        if (i == ROQhead ) return 0;
}
}

void
HypercubeMessageContainer::updateROQ(int i) {
// increment tail only if the next one element was read
                if(i == ROQtail)
                        ROQtail = (ROQtail +1) % requestStorageSize;
	delete ROQ[i].sem;
	ROQ[i].sem = 0; 
	ROQ[i].sem = new Semaphore(0); 
	ROQ[i].mesg = 0;
	ROQ[i].satisfied = 0;
    	ROQ[i].lock.release();
		
}
/*
 * print the list of valid entries
 * in messageBuffer
 */

void
HypercubeMessageContainer::validBufferEntries() {
        for(int i=0;i< messageBufferSize; i++) {
                if(messageBuffer[i])
                        Console() << i << " 1, type " << ((iPSCMessage*)messageBuffer[i])->getType() << "\n" << eor;
                else Console() << i << " 0 \n" << eor;
        }
}
/*
 * print the list of valid entries
 * in ROQ
 */

void
HypercubeMessageContainer::validROQEntries() {
        for(int i=0;i< requestStorageSize; i++) {
                if(ROQ[i].mesg)
                        Console() << i << " 1, type " << ROQ[i].type << " \n" << eor;
                else Console() << i << " 0 \n" << eor;
        }
}

#include "ClassInstaller.h"
DEFINECLASS("HypercubeMessageContainer",HypercubeMessageContainer,AsyncMessageContainer,0,0);
