#ifdef __GNUC__
#pragma implementation
#endif

#include "DistributedNameServer.h"
#include "MessageContainer.h"
#include "UniRepresentative.h"
#include "Message.h"
#include "Semaphore.h"
#include "Transport.h"
#include "Byte.h"
#include "LocalTransport.h"

#ifdef __GNUC__
#pragma implementation
#endif

/*
 * for remote portgroup created by name server
 * do not add this into the name server again
 * This is not really needed for the 
 * anyone. Another reason to separate MessagePorts
 * from MessageContainers
 */
// #define RMP_DEBUG 1
extern Address *LocalAddress;

MessageContainer :: MessageContainer (char *name)
{
    _rpcMsg = 0;
    qlen = tail = head  = 0;
    for (int i = 0; i <= messageBufferSize - 1; i++)
    	messageBuffer [i] = 0;
    messageAvailable = new Semaphore (0);
    rpcSem = new Semaphore (0);
    if (name != 0) {
        StandardDistributedNameServer -> bind (name, this);
	cr = StandardDistributedNameServer->lookup(name,ContainerRepresentativeClass);
	} else 
	cr = new UniRepresentative(LocalAddress,0,this);
#ifdef RMP_DEBUG
    Console() << "RMP -- done " << eor;
#endif
}

/*
 * This method is called by the appropriate
 * Transport Object to insert a message into 
 * a queue
 */

void MessageContainer :: put (Message *msg)
{
#ifdef RMP_DEBUG
 Console() << "MessageContainer :: put" << eor;
#endif

    int index;
    if(msg->getcType() == 2) {
#ifdef RMP_DEBUG
	Console() << "MessageContainer :: put rpc " << eor;
#endif
	_rpcMsg = (RPCMessage *)msg;
	((RPCMessage*)_rpcMsg)->setcType(1);
	rpcSem->V();
    } else {
#ifdef RMP_DEBUG
	Console() << "MessageContainer :: on main queue" << eor;
#endif
	nextPutIndex(&index);
	messageBuffer [index] = msg;
    	messageAvailable -> V ();
   }
#ifdef RMP_DEBUG
Console() << "MessageContainer :: done" << eor;
#endif
}

ContainerRepresentative *
MessageContainer :: rpcGet (RPCMessage *msg)
{
#ifdef RMP_DEBUG
    Console() << "RMP -- rpcGet \n" << eor;
#endif
    messageAvailable -> P ();
#ifdef RMP_DEBUG
    Console() << "RMP -- rpcGet got mesage \n" << eor;
#endif
    mutwait.acquire();
    Message *nextMessage = messageBuffer [tail];

    Assert (nextMessage != 0);
    //Assert (msg->maxSize() >= nextMessage -> size()) ("user buffer too small for message data");
	
    ByteCopy (nextMessage -> data (), msg->data(), nextMessage -> size ());
    msg->setUserData(msg->data(), nextMessage->size());
    msg->setcType( nextMessage->getcType());
    msg->setTime( nextMessage->getTime());
    msg->setServer( ((RPCMessage*)nextMessage)->getServer());
    messageBuffer [tail] = 0;
    tail = (tail +1) % messageBufferSize;
    ContainerRepresentative *ret = ((RPCMessage*) nextMessage)->getReplyToPort();
    mutwait.release();
    delete nextMessage;
   return  ret;

}
/*
 * wait for a message of a upto a specific size
 * then fill in the message buffer
 */

void MessageContainer :: get (Message *msg)
{
    messageAvailable -> P ();
#ifdef RMP_DEBUG
    Console() << "MessageContainer :: get \n" << " tail " << tail << "  head " << head << "\n" << eor;
#endif
    mutwait.acquire();
    Message *nextMessage = messageBuffer[tail];
    Assert (nextMessage != 0);
    Assert (msg->maxSize() >= nextMessage -> size()) 
	("user buffer too small for message data");
	
    ByteCopy (nextMessage -> data (), msg->data(), nextMessage -> size ());
    msg->setUserData(msg->data(), nextMessage->size());
    msg->setcType( nextMessage->getcType());
    msg->setTime( nextMessage->getTime());
    messageBuffer [tail] = 0;
    tail = (tail +1) % messageBufferSize;
    mutwait.release();
    delete nextMessage;

}
int MessageContainer :: isEmpty ()
{
	if(tail == head) return 1 ;
	else return 0;
}
int
MessageContainer:: wait (Message *msg)
{
#ifdef RMP_DEBUG
    Console() << "MessageContainer::wait" <<
	this << " \n" << eor;
#endif
    rpcSem->P(); // wait for the reply.
    Assert(_rpcMsg != 0);
    Assert (msg->maxSize() >= _rpcMsg -> size());
    ByteCopy (_rpcMsg -> data (), msg->data(), _rpcMsg -> size ());
    msg->setUserData(msg->data(), _rpcMsg->size());
	return 0;
}

/*
 * This is a tight little routine for
 * handing out indices into
 * the circular message queue
 * for getting messages
 */

void
MessageContainer::dump() {
	for(int i = tail; i = ( i + 1) % messageBufferSize; i <= head)
{
	if(messageBuffer[i] != 0) {
	Console() <<	"\n i is " << i 
	<< " size " << messageBuffer[i]->size() <<  eor;
	}
}

}
void
MessageContainer::nextGetIndex(int* index) {

Assert (tail != head ) ("No message ");
 mutwait.acquire();
*index = tail; 
 tail  =(tail +1 ) % messageBufferSize;
 mutwait.release();
}
/*
 * This is a tight little routine for
 * handing out indices into
 * the circular message queue
 * for putting messages
 */

void
MessageContainer::nextPutIndex(int* index) {
 mutwait.acquire();
*index = head; 
head = (head +1 ) % messageBufferSize;

 Assert (( (head + 1) % messageBufferSize) != tail)
	("MessageContainer Overflow");

 mutwait.release();
}

ContainerRepresentative*
MessageContainer::cachedCR () {return cr;}

#include "ClassInstaller.h"
DEFINECLASS("MessageContainer",MessageContainer,ProxiableObject,0,0);
