#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Nayeem Islam (nislam@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * NetworkNameServer.cc: implementation of the NetworkNameServer class.
 */

#include "Assert.h"
#include "Class.h"
#include "Domain.h"
#include "AssocTable.h"
#include "NameServer.h"
#include "Process.h"
#include "ProxiableObject.h"
#include "Raid.h"
#include "Byte.h"
#include "String.h"
#include "NameServer.h"
#include "NetworkNameServer.h"
#include "ContainerManager.h"
#include "MessageContainer.h"
#include "UniRepresentative.h"
#include "SendGroupRepresentative.h"
#include "GroupRepresentative.h"
#include "ContainerGroup.h"
#include "Message.h"
#include "NameServer.h"
#include "NetworkTable.h"
#include "Address.h"
#include "Transport.h"
#include "SendingTransport.h"
#include "ReceivingTransport.h"

#ifdef __GNUC__
#pragma implementation
#endif

extern int dumpHex(unsigned char *c, int len);

Transport* NetTransport =0;

/*
 * DNS is short for Network Name Server 
 */
MessageContainer *StandardDNSMessagePort = 0;
NetworkTable *StandardNetworkTable = 0;
extern Address *LocalAddress;

// #define NNS_DEBUG 1 

NetworkNameServer::NetworkNameServer( NameServer *_parent , Transport *_transport, Transport *_localTransport): NameServer(_parent)
{
	StandardNetworkTable = new NetworkTable;

	Console() << "NetworkTable  created \n" << eor;

        NetTransport = aTransport = _transport;
        aTransport = _transport;
        StandardNameServer->bind("LocalTransport",_localTransport);
        localContainer = new MessageContainer(0);
	StandardDNSMessagePort = localContainer;
#ifdef NNS_DEBUG
	Console() << "NNS created \n" << eor;
#endif
}

NetworkNameServer::~NetworkNameServer()
{

}

static void
unbindNetworkNameServerEntry( int type, void * name, unsigned int entry )
{
	( (ProxiableObject *) entry )->unreference();
}
void
NetworkNameServer::bind( char * name, ProxiableObject * o )
{
#ifdef NNS_DEBUG
	Console() << "In bind \n";
	Console().flush();
#endif
	Raid( RaidMethod )( "%N::bind( %s, %N )\n", this, name, o );
	checkMessageContainer();
	Class * objClass = o->classOf();
	if (objClass->isASubclassOf(ContainerGroupClass)){
                NameServer::bind(name,new SendGroupRepresentative(LocalAddress,(Transport* )0,(ContainerGroup *)o));
		multicast(name,o,PGBIND);
		add((ContainerGroup*) o); // add to local machine list
	} else if (objClass->isASubclassOf(MessageContainerClass)){
                NameServer::bind(name,new UniRepresentative(LocalAddress,(Transport* )0,(MessageContainer *)o));
	        multicast(name,o,BIND);
 } else { multicast(name,o,BIND); } //generic
}

ProxiableObjectRef
NetworkNameServer::lookup( ProxiableObject * obj )
{
	return(check((MessageContainer*)obj));
}

ProxiableObjectRef
NetworkNameServer::lookup( char * name, Class * aClass )
{
	setDebugMask(RaidMethod);
	Raid( RaidMethod )( "%N::lookup( (char*)%s, (Class*)%N )\n", this, name, aClass );
	checkMessageContainer();
	if( name == 0 ) {
		Raid( RaidMethod )( "%N::lookup: bad name (name == 0)\n", this);
		return( 0 );
		}

	/*
	 * aClass == 0 defaults to ClassClass to allow a user program
	 * to lookup a class when it doesn't have anything else.
	 */
	if( aClass == 0 ) {
		aClass = ClassClass;
	}

	ProxiableObject * obj = 0;
	//lock.acquire();
	mutex.P();
	if ( hashTable != 0 ) {
		unsigned int value;
		if (hashTable->lookup( name , value ))
		    obj = (ProxiableObject *)value;
// 		obj = (ProxiableObject *) hashTable->lookup( name, status );
// 		if ( status == -1 )
// 			obj = 0;
	}
	//lock.release();
	mutex.V();
	if(obj != 0) {
/*
		obj = sendToObjectManager(obj);
*/
	}
	Raid( RaidMethod )( "%N::lookup: obj = %N from hash table\n", 
		this, obj );
	if ( obj != 0 ) {
		Class * objClass = obj->classOf();
		if( !objClass->isASubclassOf( aClass ) ) {
			setDebugMask(RaidMethod);
			Raid( RaidMethod )( "%N::lookup: bad class\n", this );
			return( 0 );
		}
	}
	else if ( parent != 0 ) {
		setDebugMask(RaidMethod);
		Raid( RaidMethod )( "%N::lookup: trying %s in parent(%N)\n", this, name, parent );

	return(parent->lookup( name, aClass ));
	// should really query other if not found.
	}
	
	Raid( RaidMethod )( "%N::lookup: returning %N\n", this, obj );

	/*
	 * The next statement can cause an outcall if obj is a proxy;
	 *   it calls reference() on obj.
	 */
	ProxiableObjectRef ret = obj;

	return( ret );
}

ProxiableObjectRef
NetworkNameServer::lookup( char * name, char * className )
{
	setDebugMask(RaidMethod);
	Raid( RaidMethod )( "%N::lookup( (char*)%s, (char*)%s )\n",
		this, name, className );
	if( className == 0 ) className = "Class";

	ClassStar aClass = StandardNameServer->lookup( className, ClassClass );

	if( aClass == 0 )
		Console() << " aClass Messageport does not exist";


	ProxiableObjectRef ret;
	if( aClass != 0 ) ret = lookup( name, aClass );
	else ret = 0;

	setDebugMask(RaidMethod);
	Raid( RaidMethod )( "%N::lookup( %s, %s ) returns %N\n", this, name, 
		className, ret );
	return ret;
}

int
NetworkNameServer::unbind( char * name )
{
	int status;

	Raid( RaidMethod )( "%N::unbind(%s)\n", this, name );
	checkMessageContainer();
	status = NameServer::unbind(  name);
	multicast(name,0,UNBIND);
	return status;

}

ProxiableObject*
NetworkNameServer::sendToObjectManager(ProxiableObject *obj)
{
	Class * objClass = obj->classOf();
	if (objClass->isASubclassOf(MessageContainerClass))
		return(manage((MessageContainer*)obj));
}

#include "ClassInstaller.h"
DEFINECLASS("NetworkNameServer",NetworkNameServer,ProxiableObject,0,0);
