#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Nayeem Islam (nislam@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */


#include "OutputStream.h"
#include "Assert.h"
#include "Byte.h"
#include "String.h"
#include "HexDump.h"
#include "NetworkBuffer.h"
#include "EthernetAdministrator.h"
#include "NetworkTable.h"
#include "Node.h"

#ifdef __GNUC__
#pragma implementation
#endif

static EthernetAddress deviusAddress(0x8,0x0,0x20,0x10,0xde,0xc9);
static Node deviusNode(&deviusAddress,"devius",0);

static EthernetAddress deliriusAddress(0x8,0x0,0x20,0x10,0xdf,0xf3);
static Node deliriusNode(&deliriusAddress,"delirius",0);

static EthernetAddress ss2t8Address(0x8,0x0,0x20,0x0f,0xc5,0x3f);
static Node ss2t8Node(&ss2t8Address,"ss2t8",0);

static EthernetAddress ss2t7Address(0x8,0x0,0x20,0xb,0x38,0x72);
static Node ss2t7Node(&ss2t7Address,"ss2t7",0);

static EthernetAddress ss2t6Address(0x8,0x0,0x20,0xb,0x26,0xda);
static Node ss2t6Node(&ss2t6Address,"ss2t6",0);

static EthernetAddress ContagiusAddress(0x8,0x0,0x20,0xb,0xf,0x4a);
static Node ContagiusNode(&ContagiusAddress,"contagius",0);
static EthernetAddress AnxiusAddress(0x8,0x0,0x20,0xb,0x9,0x64);
static Node AnxiusNode(&AnxiusAddress,"anxius",0);
static EthernetAddress CascaAddress(0x8,0x0,0x20,0x13,0x17,0x5e);
static Node CascaNode(&CascaAddress,"casca",0);
static EthernetAddress JuliusAddress(0x8,0x0,0x20,0xb,0x30,0x58);
static Node JuliusNode(&JuliusAddress,"julius",0);
static EthernetAddress ss2t1Address(0x8,0x0,0x20,0xb,0x26,0x73);
static Node ss2t1Node(&ss2t1Address,"ss2t1",0);

static EthernetAddress AuntiemAddress(0x8,0x0,0x4c,0x0,0x20,0x04);
static Node AuntiemNode(&AuntiemAddress,"auntiem",0);
static EthernetAddress CousinmAddress(0x8,0x0,0x4c,0x0,0x23,0xf0);
static Node CousinmNode(&CousinmAddress,"cousinm",0);
static EthernetAddress TediusAddress(0x8,0x0,0x20,0x10,0xdf,0x8);
static Node TediusNode(&TediusAddress,"tedius",0);
static EthernetAddress EnviusAddress(0x8,0x0,0x20,0xe,0xe6,0x1f);
static Node EnviusNode(&EnviusAddress,"envius",0);
static EthernetAddress StudiusAddress(0x8,0x0,0x20,0x10, 0xa9,0xde);
static Node StudiusNode(&StudiusAddress,"studius",0);
static EthernetAddress AllAddress(0xff,0xff,0xff,0xff,0xff,0xff);
static EthernetAddress NullAddress(0x0,0x0,0x0,0x0,0x0,0x0);

static struct {
	int index;
	char * name;
	EthernetAddress * address;
	Node *node;
} EtherHost[] = {
{ 0, "casca", &CascaAddress ,&CascaNode},
{ 1, "anxius", &AnxiusAddress ,&AnxiusNode},
{ 2, "contagius", &ContagiusAddress ,&ContagiusNode},
{ 3, "ss2t1", &ss2t1Address, &ss2t1Node},
{ 4, "cousinm", &CousinmAddress ,&CousinmNode},
{ 5, "auntiem", &AuntiemAddress , &AuntiemNode},
{ 6, "tedius", &TediusAddress , &TediusNode},
{ 7, "envius", &EnviusAddress , &EnviusNode},
{ 8, "studius", &StudiusAddress , &StudiusNode},
{ 9, "julius", &JuliusAddress , &JuliusNode},
{ 10, "ss2t6", &ss2t6Address , &ss2t6Node},
{ 11, "ss2t7", &ss2t7Address , &ss2t7Node},
{ 12, "ss2t8", &ss2t8Address , &ss2t8Node},
{ 13, "devius", &deviusAddress , &deviusNode},
{ 14, "delirius", &deliriusAddress , &deliriusNode},
{ 15, (char *) 0, &NullAddress ,0 }
};
const int maxIndex = 15;

extern EthernetAdministrator *  _THE_EthernetAdministrator;
extern ATable* StandardNetworkTable;

NetworkTable::NetworkTable()
{
    Assert(_THE_EthernetAdministrator != 0);
    _localAddress = _THE_EthernetAdministrator->address();
    _localIndex = machineIndex(_localAddress);
    _localName = machineName(_localIndex);
}
void
NetworkTable::list()
{
    for( int i = 0; EtherHost[i].name != 0; i++ )
    {
	Console() <<
	" Node Id " << EtherHost[i].index << 
	" Name  " << EtherHost[i].name << 
	" Address " << *(EtherHost[i].address) 
		<< "\n" <<  eor;
    }
}

Address *
NetworkTable::machineAddress(unsigned index)
{
//    index--;
    Assert(index < maxIndex);
    return EtherHost[index].address;
}

char *
NetworkTable::machineName(unsigned index)
{
 //   index--;
    Assert(index < maxIndex);
    return EtherHost[index].name;
}

unsigned
NetworkTable::machineIndex(char * name)
{
    for( int i = 0; EtherHost[i].name != 0; i++ )
    {
	if (strcmp(EtherHost[i].name,name) == 0)
	    return EtherHost[i].index;
    }
    return 0;
}

char*
NetworkTable::machineName(Address * address)
{
    for( int i = 0; *(EtherHost[i].address) != NullAddress; i++ )
    {
	if (ByteCompare((EtherHost[i].address)->bytes(),address->bytes(),6) == 0)
	    return EtherHost[i].name;
    }
    return 0;
}
Node*
NetworkTable::node(Address * address)
{
    for( int i = 0; *(EtherHost[i].address) != NullAddress; i++ )
    {
	if (ByteCompare((EtherHost[i].address)->bytes(),address->bytes(),6) == 0)
	    return EtherHost[i].node;
    }
    return 0;
}
unsigned
NetworkTable::machineIndex(Address * address)
{
    for( int i = 0; *(EtherHost[i].address) != NullAddress; i++ )
    {
	if (ByteCompare((EtherHost[i].address)->bytes(),address->bytes(),6) == 0)
	    return EtherHost[i].index;
    }
    return 0;
}
/*
#include "ClassInstaller.h"
DEFINECLASS("NetworkTable",NetworkTable,ATable,0, 0);
*/
