#ifdef __GNUC__
#pragma implementation
#endif

#include "OutputStream.h"
#include "MessageContainer.h"
#include "ReceivingTransport.h"
#include "Reliability.h"
#include "AtLeastOnce.h"
#include "AsyncMessage.h"
#include "Message.h"
#include "Assert.h"
#include "Byte.h"
#include "Header.h"
#include "NetworkBuffer.h"
#include "EthernetTransfer.h"
#include "DistEthernet.h"
#include "DistributedNameServer.h"
#include "Semaphore.h"
#include "ContainerGroup.h"
#include "ClientCSR.h"
#include "RemoteCSR.h"
#include "MultiCCSR.h"

#ifdef __GNUC__
#pragma implementation
#endif

//#define T_MESSAGE_DEBUG 1
//#define TF_MESSAGE_DEBUG 1

extern Transport *NetTransport ;

extern int dumpBuf(unsigned char *c, int offset, int len);
extern int dumpHex(unsigned char *c, int len);

ReceivingTransport::ReceivingTransport()
{
	clSem = new Lock;
	cl2Sem = new Lock;
	remSem = new Lock;
	rem2Sem = new Lock;
        for(int i=0;i<tempBufferSize;i++) {
		RemoteCSRArray[i] = 0;
		ClientCSRArray[i] = 0;
	}
Console() << "ReceivingTransport::ReceivingTransport"
	<< "created" << eor;

}

/*
 * called for packets, by the reciever of a message.
 * This is called for remote processing.
 * at this point it is either an request being received
 * or a response being received.
 */

void
ReceivingTransport::receive(char *newpos,Address *sender,Address *destAddr,NetworkBuffer* nb)
{
	TransportHeader *header = (TransportHeader *) newpos;
#ifdef TF_MESSAGE_DEBUG
	Console() << "ReceivingTransport::receive \n" << eor;
		inspect(header);
#endif
	// find csr for sender
	if(header ->probe)
		HandleProbe(newpos,sender ,destAddr,nb);
	if(header->fragment != 0 ) { // a large message
		if(header->ctype == REPLY_MESSAGE )	
			assembleResponse(newpos,header->size,sender,destAddr,nb);
		else if (header->ctype == RPC_MESSAGE)
		assembleRequest(newpos,header->size,sender,destAddr,nb);
		else if (header->ctype == ASYNC_MESSAGE)
		assembleAsync(newpos,header->size,sender,destAddr,nb);
	} else { // if small go ahead and passup
		switch(header->ctype) {
		case REPLY_MESSAGE : // HandleResponse 
               	 	ClientCSR* csr = findClientCSR((MessageContainer*)(header->destMessagePort),LocalAddress); // should only be one guy waiting
			Assert( csr !=0);
			csr->responseDone(); // unblock 
			csr->endTrans(); // of transaction
			break;
		case  RPC_MESSAGE:  // HandleRequest 
                        RemoteCSR* rcsr = findRemoteCSR((MessageContainer*)header->replyMessagePort,sender,(MessageContainer*)header->destMessagePort); // use reply port
			Assert(rcsr != 0);
			rcsr->flush(header->messageId); // flush all previous transactions 
			rcsr->incr_packets();
			break;
		case  ASYNC_MESSAGE:  // HandleRequest 
			//Console() << "got async" << eor;
			break;
		} // switch
/*
 * Now add message to the port
 */
		dispatch(newpos,header,sender,destAddr,nb);
	} // if
}
/*
 * Response handler, should onle deal with 
 * ClientCSR.
 */

void
ReceivingTransport::assembleResponse(char* packet, int len, Address *etherAddr,Address *destAddr,NetworkBuffer *nb)
{
#ifdef TF_MESSAGE_DEBUG
	Console() << "ReceivingTransport::assembleResponse " << eor;
#endif
	TransportHeader *header = (TransportHeader *) packet;
 	MessageContainer* dest=0;
        ClientCSR* csr = findClientCSR((MessageContainer*)( header->destMessagePort),LocalAddress); // should only be one
	csr->lock();
	if( (csr -> state ==  AWAIT_RESPONSE) && (csr->first_packet == 0) ) { // transaction in progress 
		csr->putMessage (makeMessage(packet,dest,header,etherAddr,header->fragmentSize,nb));// new Message;
		csr->currSize = header->fragmentSize;
		csr->incr_packets(); // increment packet count
		csr->setFrag(header->fragment -1); // set fragment position
		csr->first_packet = 1; // wait for more packets.
		csr->recv_response(); // wait for more packets.
#ifdef TF_MESSAGE_DEBUG
		Console() << " first Packet " << csr->currSize << eor;
#endif
	} else {
#ifdef TF_MESSAGE_DEBUG
		Console() << " Another fragment " << eor;
#endif
		Message *tm  = csr->getMessage();
#ifdef T_MESSAGE_DEBUG
		Console() << " currSize " << csr->currSize << eor;
		Console().printf( " %x \n",(char*)tm->data()); 
		Console().printf( " %x \n",(char*)tm->data() + tm->currSize); 
#endif
		ByteCopy( packet + sizeof(TransportHeader), (char*)tm->data() + csr->currSize, header->fragmentSize);
		csr->currSize += header->fragmentSize;
		csr->setFrag(header->fragment-1);
		csr->incr_packets();
#ifdef TF_MESSAGE_DEBUG
		Console() << " fragment " <<  header->fragment << eor;
		Console() << " latest \n" <<  csr->currSize << eor;
#endif
		if(csr->currSize == header->size) {
			csr->responseDone(); // unset timer
			csr->endTrans(); // unset timer
			csr->unlock();
			dispatch(csr->getMessage(),destAddr,header->destMessagePort);	
		} else csr->unlock();
	}
}

/* 
 * This is the assemble method for a request to
 * a server
 */
/*
 * Handle the Request
 * Deal only with RemoteCSR's
 */

void
ReceivingTransport::assembleRequest(char* packet, int len, Address *etherAddr,Address *destAddr,NetworkBuffer *nb)
{
#ifdef TF_MESSAGE_DEBUG
	Console() << "ReceivingTransport::assembleRequest " << eor;
#endif
	TransportHeader *header = (TransportHeader *) packet;
 	MessageContainer* dest=0;
        RemoteCSR* rcsr = findRemoteCSR((MessageContainer*)header->replyMessagePort,etherAddr,(MessageContainer*)header->destMessagePort); // use reply port
	rcsr->lock();
	if( !(rcsr -> state == RECV_REQ) && (rcsr->first_packet == 0)) { // new transaction 
		rcsr->trans_id = header->messageId;
		rcsr->flush(header->messageId); // increment packet count
		rcsr -> putMessage ( makeMessage(packet,dest,header,etherAddr,header->fragmentSize,nb));// new Message;
#ifdef TF_MESSAGE_DEBUG
		Console() << " first Packet " << rcsr->currSize << eor;
#endif
		rcsr->currSize = header->fragmentSize;
		rcsr->setFrag(header->fragment-1); // set fragment position
		rcsr->incr_packets(); // increment packet count
		rcsr->first_packet = 1; // wait for more packets.
		rcsr->recv_request(); // wait for more packets.
	} else {
		Assert( rcsr->trans_id == header->messageId);
#ifdef TF_MESSAGE_DEBUG
		Console() << " Another fragment " << eor;
#endif
		Message * tm  = rcsr->getMessage();
#ifdef T_MESSAGE_DEBUG
		Console() << " currSize " << tm->currSize << eor;
		Console().printf( " %x \n",(char*)tm->message->data()); 
		Console().printf( " %x \n",(char*)tm->message->data() + tm->currSize); 
#endif
		ByteCopy( packet + sizeof(TransportHeader), (char*)tm->data() + rcsr->currSize, header->fragmentSize);
		rcsr->currSize += header->fragmentSize;
		rcsr->setFrag(header->fragment-1);
		rcsr->incr_packets();
#ifdef TF_MESSAGE_DEBUG
		Console() << " fragment " <<  header->fragment << eor;
		Console() << " latest \n" <<  rcsr->currSize << eor;
#endif
		if(rcsr->currSize == header->size) {
			rcsr->requestDone(); // unset timer
			rcsr->endTrans(); // unset timer
			rcsr->unlock();
			dispatch(rcsr->getMessage(),destAddr,header->destMessagePort);	
		} else rcsr->unlock();
	}
}

void
ReceivingTransport::assembleAsync(char* packet, int len, Address *etherAddr,Address *destAddr,NetworkBuffer *nb)
{
#ifdef TF_MESSAGE_DEBUG
	Console() << "ReceivingTransport::assembleAsync " << eor;
#endif
	TransportHeader *header = (TransportHeader *) packet;
 	MessageContainer* dest=0;
        RemoteCSR* rcsr = findRemoteCSR((MessageContainer*)header->replyMessagePort,etherAddr,(MessageContainer*)header->destMessagePort); // use reply port
	rcsr->lock();
	if( !(rcsr -> state == RECV_REQ) && (rcsr->first_packet == 0)) { // new transaction 
		rcsr->trans_id = header->messageId;
		rcsr->flush(header->messageId); // increment packet count
		rcsr -> putMessage ( makeMessage(packet,dest,header,etherAddr,header->fragmentSize,nb));// new Message;
#ifdef TF_MESSAGE_DEBUG
		Console() << " first Packet " << rcsr->currSize << eor;
#endif
		rcsr->currSize = header->fragmentSize;
		rcsr->setFrag(header->fragment-1); // set fragment position
		rcsr->incr_packets(); // increment packet count
		//rcsr->recv_request(); // wait for more packets.
		rcsr->first_packet = 1; // wait for more packets.
		rcsr->setState(RECV_REQ); // wait for more packets.
		rcsr->unlock();
	} else {
		Assert( rcsr->trans_id == header->messageId);
#ifdef TF_MESSAGE_DEBUG
		Console() << " Another fragment " << eor;
#endif
		Message * tm  = rcsr->getMessage();
#ifdef T_MESSAGE_DEBUG
		Console() << " currSize " << tm->currSize << eor;
		Console().printf( " %x \n",(char*)tm->message->data()); 
		Console().printf( " %x \n",(char*)tm->message->data() + tm->currSize); 
#endif
		ByteCopy( packet + sizeof(TransportHeader), (char*)tm->data() + rcsr->currSize, header->fragmentSize);
		rcsr->currSize += header->fragmentSize;
		rcsr->setFrag(header->fragment-1);
		rcsr->incr_packets();
#ifdef TF_MESSAGE_DEBUG
		Console() << " fragment " <<  header->fragment << eor;
		Console() << " latest \n" <<  rcsr->currSize << eor;
#endif
		if(rcsr->currSize == header->size) {
			//rcsr->requestDone(); // unset timer
			rcsr->endTrans();
			rcsr->numPackets=0;
			rcsr->unlock();
			dispatch(rcsr->getMessage(),destAddr,header->destMessagePort);	
		} else rcsr->unlock();
	}
}

void
ReceivingTransport::saveClientCSR(ClientCSR* csr)
{
	int foundSpot = 0;
//        clSem->acquire();
	for(int i=0;i<tempBufferSize;i++)
		if(ClientCSRArray[i] == 0) {
			ClientCSRArray[i] = csr;
			foundSpot = 1; 
 //       		clSem->release();
			return;
		}
  //      clSem->release();
	Assert( foundSpot ) ("Out of temporary storage ");
}

void
ReceivingTransport::saveRemoteCSR(RemoteCSR* rcsr)
{
        //remSem->acquire();
	int foundSpot = 0;
	for(int i=0;i<tempBufferSize;i++)
		if(RemoteCSRArray[i] == 0) {
			RemoteCSRArray[i] = rcsr;
			foundSpot = 1; 
        	//	remSem->release();
			return;
		}
	Assert( foundSpot ) ("Out of temporary storage ");
        //remSem->release();

}

RemoteCSR*    
ReceivingTransport::findRemoteCSR(Container* client,Address*cl_machine,Container*server)
{
rem2Sem->acquire();
int i;
for(i=0;i<tempBufferSize;i++){
	if(RemoteCSRArray[i] && !ByteCompare((RemoteCSRArray[i]->client_machine)->bytes(),cl_machine->bytes(),6) && (RemoteCSRArray[i]->server == server) && RemoteCSRArray[i]->client_id == client)
	{	
#ifdef TF_MESSAGE_DEBUG
		Console() << " resuing old " << eor;
		Console() << " trans id\n " << RemoteCSRArray[i]->trans_id << eor;
		Console() << " State " <<  RemoteCSRArray[i]->state  << eor;
		Console() << " \n fp " <<  RemoteCSRArray[i]->first_packet  << eor;
#endif
		rem2Sem->release();
		return RemoteCSRArray[i];
	}
}
	Console() << "Creating new RCSR" << eor;
        RemoteCSR* rcsr = new RemoteCSR(client,cl_machine,server);
	saveRemoteCSR(rcsr);	
	rem2Sem->release();
	return rcsr;
	// return 0;
}

ClientCSR*    
ReceivingTransport::findClientCSR(Container* mp,Address* etherAddr,int type)
{
cl2Sem->acquire();
int i;
for(i=0;i<tempBufferSize;i++){
	if(ClientCSRArray[i] && !ByteCompare((ClientCSRArray[i]->etherAddr)->bytes(), etherAddr->bytes(),6) && ClientCSRArray[i]->client_id == mp)
	{	
#ifdef TF_MESSAGE_DEBUG
		Console() << " found temp mess  " << eor;
#endif
		cl2Sem->release();
		return ClientCSRArray[i];
	}
}
	//Console() << " Creating new CCSR\n" << eor;
	ClientCSR *ccsr;
        if (type ==0) ccsr = new ClientCSR(mp,etherAddr); 
	else ccsr = new MultiCCSR(mp,etherAddr,type);
	saveClientCSR(ccsr);
	cl2Sem->release();
	return ccsr;
}

void
ReceivingTransport::inspect(TransportHeader *transportHeader)
{
        Console().printf(" destMessageContainer  %x :\n", transportHeader->destMessagePort);
        Console() << "\n  ctype      : " << transportHeader->ctype;
        Console() << "    size      : " << transportHeader->size;
        Console() << "    messageId      : " << transportHeader-> messageId;
        Console() << "    fragment      : " << transportHeader-> fragment;
        Console() << "    fragmentSize      : " << transportHeader-> fragmentSize;
        Console() << "    probe      : " << transportHeader-> probe;
        Console() << "\n" << eor;

//     dumpBuf((unsigned char *) transportHeader, sizeof(TransportHeader), transportHeader->fragmentSize);
}

void
ReceivingTransport::HandleProbe(char *newpos,Address *sender,Address *destAddr,NetworkBuffer* nb)
{
//	We have a probe on our hands
// 	find 	
        TransportHeader *header = (TransportHeader *) newpos;
		if(header->ctype == REPLY_MESSAGE ) // HandleResponse 
		{
               	 	ClientCSR* csr = findClientCSR((MessageContainer*)(header->destMessagePort),LocalAddress); // should only be one guy waiting
			csr->probe(((Header *)header)->reliabilityHeader.delmask);
			// endtrans is suspect
		} else { // HandleRequest 
                        RemoteCSR* rcsr = findRemoteCSR((MessageContainer*)header->replyMessagePort,sender,(MessageContainer*)header->destMessagePort); // use reply port
			if(rcsr == 0) rcsr = new RemoteCSR((MessageContainer*)header->replyMessagePort,sender,(MessageContainer*)header->destMessagePort);

			rcsr->handleResponseProbe(header->messageId);
		}
}

#include "ClassInstaller.h"
DEFINECLASS("ReceivingTransport",ReceivingTransport,DispatchMessage,0,0);
