#ifdef __GNUC__
#pragma implementation
#endif

/* 
 * All VMTP client managemnet routines are 
 * handled in this module.
 */
#include "RemoteCSR.h"
#include "Address.h"
#include "Header.h"
#include "Process.h"
#include "ThisProcess.h"
#include "Semaphore.h"
#include "Message.h"
#include "MessageContainer.h"
#include "ReceivingTransport.h"
#include "TimeoutTimer.h"
#include "EthernetTransfer.h"
#include "BitArray.h"
#include "NetworkBuffer.h"
#include "NameServer.h"
#include "Byte.h"
#include "DistributedNameServer.h"



extern int dumpHex(unsigned char *c, int len);

/*
 * This is categorized by 
 * three aspects. The MessageContainer of the 
 * client. The address of the clients machine,
 * and the messageport of the server or the 
 * recipient of the message
 *
 */
// #define REMOTE_CSR_DEBUG 1
RemoteCSR::RemoteCSR (Container* client,Address* e,Container* serv) {

timer = new TimeoutTimer;
client_machine = new EthernetAddress;
ByteCopy(e->bytes(),client_machine->bytes(),6);
client_id = client;
server = serv; // server id, to identify which server this remote csr belong to. 

state = RIDLE;
cacheReply = 0;
dest =0;
response = 0;
done = 0;

#ifdef REMOTE_CSR_DEBUG
Console() << "Created RemoteCSR " << 
	client_machine << eor;
	Console().printf( " client_id %x server %x",client_id,server);
#endif
/*
 * This process is used to handle multipacket
 * request packets by the receiver.
 */
sem = new Semaphore(0);
Process* m_process =  new SystemProcess( (PFV) &RemoteCSR::timerLoop, thisProcess()->domain(), small, (int) this);
m_process->ready();
}

void
RemoteCSR::timerLoop()
{
#ifdef REMOTE_CSR_DEBUG
Console() << "RemoteCSR::timerLoop ready" << "\n" << eor;
#endif
 while(1)
        {
        sem->P(); // wait for a multipacket request
        // Console() << "sem ok" << eor;
        numTimeOuts = 0;
        done = 0;
#ifdef REMOTE_CSR_DEBUG
        Console() << "RemoteCSR::recv_request() " << eor
;
#endif
        while((numTimeOuts < MaxNumTimeouts) && !done ){
                timer->start (time_limit);// 1 millisecond
                timer->await();
                numTimeOuts++;
                if(timer->residual() && !done && (numTimeOuts < MaxNumTimeouts) ) // send unsent packets
                sendProbe(); // send a probe.
        }
 
}
}


/*
 * We have a chain of data pointers and 
 * a chain of pointers to network buffers.
 * We will for an acknowledgement and then 
 * we destroy the timer.
 * in CSR::wait the sender is awaiting responses, and 
 * after the first ack, receiving reponses if 
 * packets are dropped.
 */
void
RemoteCSR::recv_request()
{
        state = RECV_REQ;
        unlock();
        sem->V(); // have some one else wait.

/*
numTimeOuts = 0;
done = 0;
state = RECV_REQ;
unlock();
#ifdef REMOTE_CSR_DEBUG
Console() << "RemoteCSR::recv_request() " << eor;
#endif
while((numTimeOuts < MaxNumTimeouts) && !done ){
        timer->start (time_limit);// 1 millisecond
        timer->await();
        numTimeOuts++;
        if(timer->residual() && !done && (numTimeOuts < MaxNumTimeouts) ) // send unsent packets
            sendProbe(); // send a probe.
}
*/

}
void
RemoteCSR::setState(int st)
{
state = st;
}


void 
RemoteCSR::handleResponseProbe(int request_id ) {
	if(state == RESPONDED && request_id == trans_id) 
		send();	
}

void RemoteCSR::send()
{
for ( int j=0; j< numPackets; j++)
    if ( tm->get( j ) )
        transfer->send(nblist[j]->packet(), chain[j],client_machine,nblist[j]->length(),nblist[j]);
}

void 
RemoteCSR::endTrans() {
	state = RIDLE;
	first_packet = 0;
	currSize = 0;
}

void RemoteCSR::incr_packets() 
{
	numPackets++;
}

/*
 * set the transaction
 * id field for the rcsr
 */
void RemoteCSR::flush(int tid) 
{
for ( int j=0; j< numPackets; j++) {
	//Console() << "releasing buffer\n" << eor;
        nblist[j]->release();
}
	trans_id = tid;	 // new transaction
	numPackets = 0; // number of packets received
	currSize = 0;
}


void
RemoteCSR::requestDone()
{
	done = 1;
        int time = timer->stop();
}
void RemoteCSR::sendProbe() {
#ifdef REMOTE_CSR_DEBUG
Console() << "Sending a Probe" << eor;
#endif
/*
 * Used with response to describe when packets
 * are not received
 */
        int length = sizeof(Header);
        if(length < 64 ) length = 64;
 
        NetworkBuffer *nb =  netFree->getBuffer();
 
#ifdef SPARC
#ifndef VCHOICES
                char *pkt = nb->kpacket();
#else
                char *pkt = nb->packet();
#endif
#else
                char *pkt = nb->packet();
#endif
 
        nb->setLength(length);
        Header *probe = (Header*) pkt;
        probe->transportHeader.probe = 1;
        probe->reliabilityHeader.delmask = transmission_mask;
        transfer->send((char*) probe,(char*)probe,client_machine,length);
}

