#ifdef __GNUC__
#pragma implementation
#endif

#include "OutputStream.h"
#include "MessageContainer.h"
#include "ContainerRepresentative.h"
#include "AsyncMessage.h"
#include "MultiRPCMessage.h"
#include "Reliability.h"
#include "SendingTransport.h"
#include "NetworkBuffer.h"
#include "NameServer.h"
#include "EthernetTransfer.h"
#include "DistEthernet.h"
#include "NetworkBuffer.h"
#include "Assert.h"
#include "Byte.h"
#include "ClientCSR.h"
#include "RemoteCSR.h"
#include "MultiCCSR.h"

#ifdef __GNUC__
#pragma implementation
#endif

// #define SEND_MESSAGE_DEBUG 1

SendingTransport::SendingTransport()
{

/* 
 * only one subclass should be registered 
 * with the nameserver
 */ 

nbfl = StandardNameServer->lookup("nbfl","NetworkBufferFreeList");
Assert(nbfl !=0)("SendingTransport::SendingTransport -- No Network Buffer Manager found");
MaxPacketLength = 1400;
#ifdef SEND_MESSAGE_DEBUG
	Console() << 
		"SendingTransport::SendingTransport" <<
		" Creation Done " << eor;
#endif
}

void
SendingTransport::initTransportHeader(TransportHeader *header,Message *message)
{
        header->destMessagePort = message->destination() ->remoteContainer(); // remote MessageContainer
        if(message->getcType() == RPC_MESSAGE || message->getcType() == MULTI_MESSAGE)
                header->replyMessagePort = ((RPCMessage*)message)->getReplyPort();
        else header->replyMessagePort = 0;
 
        header->size = message->size();
        header->ctype = message->getcType();
        if(message->getcType() == 3) // now get type acc ording to the asynchronous protocol
                header->type = ((AsyncMessage*)message)->getType();
 
        header->messageId= header->fragment = header->fragmentSize= header->probe= 0;
#ifdef SEND_MESSAGE_DEBUG
        Console() << "DONE Transport" << eor;
#endif
}

/*
 * This may be a Request out or a Response
 * back. The differentiation is made internally.
 * In side the procedure. 
 */

int
SendingTransport::send(Message *message)
{
#ifdef SEND_MESSAGE_DEBUG
        Console() << "SendingTransport::send" << eor;
#endif
        int MaxData = MaxPacketLength  - sizeof(Header);
        int size = message->size();
	Assert( size <=  RunLength*MaxPacketLength) ("Packet is too Big to be sent");

        if(size > MaxData) {
                if(message->getcType() == ASYNC_MESSAGE)
			fragmentAsync(message);
		else fragment(message);
        } else { // all this code is re-entrant
	// allocate everything in one big chunk.
                int newlen = sizeof(Header)+size;
                if(newlen < 64 ) newlen = 64;
	//      char *pkt =new char[newlen];
		NetworkBuffer *nb =  nbfl->getBuffer(); 

#ifdef SPARC 
#ifdef VCHOICES
		char *pkt = nb->packet();
#else
		char *pkt = nb->kpacket();
#endif
#else
                char *pkt = nb->packet();
#endif

		nb->setLength(newlen);
        // copy data from user or kernel message
       Console() << "about to copy\n" << eor;
                ByteCopy(message->data(), pkt + sizeof(Header),size);
                Header * header = (Header *) pkt;
                initTransportHeader(&(header->transportHeader), message);
       Console() << "DONE init data segment\n" << eor;
/*
 * Invoke reliability and
 * flow control objects.
 */
#ifdef SEND_MESSAGE_DEBUG
        inspect(&(header->transportHeader));
#endif
	// each message has a different CSR
	char **chain= 0;
	NetworkBuffer** nblist= 0;
	int type =0;
		switch (header->transportHeader.ctype) {
		case ASYNC_MESSAGE:
			Console() << " async \n" << eor;
	              reliability->send( pkt, (char *)&(header->transportHeader),message->destination()->getAddress(), newlen,nb);
			nb->release();
			break;
		case MULTI_MESSAGE:
			Console() << "Multi message " << eor;
			type = ((MultiRPCMessage*)message)->getNumMesg();
			message->setcType(RPC_MESSAGE);
			header->transportHeader.ctype = RPC_MESSAGE;
			header->transportHeader.destMessagePort= 0;
			Console() << "type " << type << eor;
		case RPC_MESSAGE: 
/*
			Console() << "RPC_MESSAGE" << eor;
*/
			ClientCSR* ccsr = findClientCSR(((RPCMessage*) message)->getReplyPort(),LocalAddress,type);
			if(type) ((MultiCCSR*)ccsr)->setResponses(type);
			chain = ccsr ->getChain();
			nblist = ccsr ->getNblist();
			chain[0] = (char *)&(header->transportHeader);
			nblist[0] = nb;
                	ccsr->flush(0);
			header->transportHeader.messageId = ccsr->trans_id; // set the transaction id.	
                	reliability->send( nb->packet(), chain[0],message->destination()->getAddress(), newlen,nb);
                	//ccsr->send();
                	ccsr->await_response(message->destination()->getAddress(), 1,reliability);
			break;
		case REPLY_MESSAGE: 
/*
		Console() << "REPLYMESSAGE looking for " <<
        (((RPCMessage*) message)->getReplyToPort())->getAddress()  << eor;
        Console().printf( " client_id %x server %x",
	((RPCMessage*)message)->getReplyToPort(),
	((RPCMessage*)message)->getServer());
*/

		// response
			RemoteCSR* rcsr = findRemoteCSR(((RPCMessage*)message)->getReplyToPort()->remoteContainer(),(((RPCMessage*)message)->getReplyToPort())->getAddress(),((RPCMessage*)message)->getServer()); // use reply port
			if(rcsr == 0) { rcsr = new RemoteCSR(((RPCMessage*)message)->getReplyToPort()->remoteContainer(),((RPCMessage*)message)->getReplyToPort()->getAddress(),((RPCMessage*)message)->getServer());
				saveRemoteCSR(rcsr);
			} 
// save chain in local stash
                        chain = rcsr ->getChain();
			nblist = rcsr ->getNblist();
			chain[0] = (char *)&(header->transportHeader);
                        nblist[0] = nb;
			header->transportHeader.messageId = rcsr->trans_id; // set the transaction id.	
			rcsr->setState(RESPONDED);
			rcsr->numPackets = 1;
                	reliability->send( nb->packet(), chain[0],message->destination()->getAddress(), newlen,nb);
/*
                	reliability->send( nb->packet(),(char *)&(header->transportHeader),message->destination()->getAddress(), newlen,nb);
*/
			//Console() << "Sent reply" << eor;
			break;
        } // switch
	}
}

void
SendingTransport::fragment(Message *message)
{
#ifdef SEND_MESSAGE_DEBUG
        Console() << "SendingTransport::fragment " << eor;
	Console() << " about to send " << eor;
	Console() << (char*) message->data() << eor;
#endif SEND_MESSAGE_DEBUG

        int MaxData = MaxPacketLength  - sizeof(Header);
        int sendlen = message->size();
	int firstlen = 0;
	int transid=0;

	char **chain =0;
	NetworkBuffer **nblist =0;

	int numPacket = 0;
	switch (message->getcType()) {
 		case RPC_MESSAGE :
                        ClientCSR* ccsr = findClientCSR(((RPCMessage*) message)->getReplyPort(), LocalAddress);
                        if(ccsr == 0) {
                                ccsr = new ClientCSR(((RPCMessage*)message)->getReplyPort(),LocalAddress);
				saveClientCSR(ccsr);
			}
                        chain = ccsr ->getChain();
                        nblist = ccsr ->getNblist();
			transid = ccsr -> trans_id;
                	ccsr->flush(0);
		break;
	case  REPLY_MESSAGE:
              RemoteCSR* rcsr = findRemoteCSR( ((RPCMessage*)message)->getReplyToPort()->remoteContainer(),(((RPCMessage*)message)->getReplyToPort())->getAddress(),((RPCMessage*)message)->getServer()); // use reply port
               if(rcsr == 0) {
 			rcsr = new RemoteCSR(((RPCMessage*)message)->getReplyToPort()->remoteContainer(),((RPCMessage*)message)->getReplyToPort()->getAddress(),((RPCMessage*)message)->getServer());
			saveRemoteCSR(rcsr);
		}
                        chain = rcsr ->getChain();
                        nblist = rcsr ->getNblist();
			transid = rcsr -> trans_id;
			rcsr->setState(RESPONDED);
		break;
	}
        Address *etherAddr = message->destination()->getAddress();
        char *p = (char *)message->data();
        int rem = sendlen % MaxData;
	if(rem == 0) rem = MaxData;
	firstlen = rem;
                // send remainder first
        int newlen = sizeof(Header)+rem;
        if(newlen < 64 ) newlen = 64;
       // char *pkt = new char[newlen];
// ::NEW check 
	NetworkBuffer *nb =  nbfl->getBuffer(); 

#ifdef SPARC 
#ifdef VCHOICES
		char *pkt = nb->packet();
#else
		char *pkt = nb->kpacket();
#endif
#else
                char *pkt = nb->packet();
#endif

		nb->setLength(newlen);
                ByteCopy(p, pkt + sizeof(Header),rem);
                Header * header = (Header *) pkt;
                // fragment the packet
                int fragment = 1;
// message specific
                header->transportHeader.destMessagePort = message->destination()->remoteContainer();
		if(message->getcType() == 1)
                	header->transportHeader.replyMessagePort = ((RPCMessage*)message)->getReplyPort();
		else header->transportHeader.replyMessagePort = 0;
        	header->transportHeader.ctype = message->getcType();

                header->transportHeader.messageId = transid;
                header->transportHeader.size = sendlen;
// fragment specific
                header->transportHeader.fragment = fragment;
                header->transportHeader.fragmentSize = rem;
#ifdef SEND_MESSAGE_DEBUG
                inspect(&(header->transportHeader));
#endif
        	chain[fragment -1] = pkt;
        	nblist[fragment -1] = nb;
		numPacket++;
                reliability->send(pkt,(char *)&(header->transportHeader) ,etherAddr,newlen,nb);

                p += rem;
                rem = sendlen - rem;
                newlen = MaxPacketLength;
                while(rem) {
        //        char *pkt =new char[newlen];
// :: NEW check
	NetworkBuffer *nb =  nbfl->getBuffer(); 

#ifdef SPARC 
#ifdef VCHOICES
		char *pkt = nb->packet();
#else
		char *pkt = nb->kpacket();
#endif
#else
                char *pkt = nb->packet();
#endif

		nb->setLength(newlen);
                ByteCopy(p, pkt + sizeof(Header),MaxData);
                Header * header = (Header *) pkt;
                header->transportHeader.destMessagePort = message->destination()->remoteContainer();
		if(message->getcType() == 1)
                	header->transportHeader.replyMessagePort = ((RPCMessage*)message)->getReplyPort();
		else header->transportHeader.replyMessagePort = 0;
        	header->transportHeader.ctype = message->getcType();
                header->transportHeader.messageId = transid;
                header->transportHeader.size = sendlen;
// fragment specific
                header->transportHeader.fragment = ++fragment;
                header->transportHeader.fragmentSize = MaxData;
#ifdef SEND_MESSAGE_DEBUG
                inspect(&(header->transportHeader));
		Console() << "\n" << eor;
		for(int i=0;i<MaxData;i++)
                	Console() << p[i] << " " << eor;
#endif
        	chain[fragment  -1] = pkt;
        	nblist[fragment  -1] = nb;
		numPacket++;
                reliability->send(pkt,(char *)&(header->transportHeader) ,etherAddr,newlen,nb);
                        rem -= MaxData;
                        p += MaxData;
                }
 	if(message->getcType() == RPC_MESSAGE ) {
  		ClientCSR* ccsr = findClientCSR( ((RPCMessage*) message)->getReplyPort(),LocalAddress);
 		ccsr->await_response(etherAddr,numPacket,reliability);
	} else  if (message->getcType() == REPLY_MESSAGE)
	{
			RemoteCSR* rcsr = findRemoteCSR(((RPCMessage*)message)->getReplyToPort()->remoteContainer(),(((RPCMessage*)message)->getReplyToPort())->getAddress(),((RPCMessage*)message)->getServer()); // use reply port
		rcsr->numPackets = numPacket;
	} 
}

void
SendingTransport::fragmentAsync(Message *message)
{
#ifdef SEND_MESSAGE_DEBUG
        Console() << "SendingTransport::fragment " << eor;
	Console() << " about to send " << eor;
	Console() << (char*) message->data() << eor;
#endif SEND_MESSAGE_DEBUG

        int MaxData = MaxPacketLength  - sizeof(Header);
        int sendlen = message->size();
	int transid=0;

        Address *etherAddr = message->destination()->getAddress();
        char *p = (char *)message->data();
        int rem = sendlen % MaxData;
	if(rem == 0) rem = MaxData;
                // send remainder first
        int newlen = sizeof(Header)+rem;
        if(newlen < 64 ) newlen = 64;
       // char *pkt = new char[newlen];
// ::NEW check 
	NetworkBuffer *nb =  nbfl->getBuffer(); 

#ifdef SPARC 
#ifdef VCHOICES
		char *pkt = nb->packet();
#else
		char *pkt = nb->kpacket();
#endif
#else
                char *pkt = nb->packet();
#endif

		nb->setLength(newlen);
                ByteCopy(p, pkt + sizeof(Header),rem);
                Header * header = (Header *) pkt;
                // fragment the packet
                int fragment = 1;
// message specific
                header->transportHeader.destMessagePort = message->destination()->remoteContainer();
		if(message->getcType() == 1)
                	header->transportHeader.replyMessagePort = ((RPCMessage*)message)->getReplyPort();
		else header->transportHeader.replyMessagePort = 0;
        	header->transportHeader.ctype = message->getcType();

                header->transportHeader.messageId = transid;
                header->transportHeader.size = sendlen;
// fragment specific
                header->transportHeader.fragment = fragment;
                header->transportHeader.fragmentSize = rem;
#ifdef SEND_MESSAGE_DEBUG
                inspect(&(header->transportHeader));
#endif
                reliability->send(pkt,(char *)&(header->transportHeader) ,etherAddr,newlen,nb);
		nb->release();

                p += rem;
                rem = sendlen - rem;
                newlen = MaxPacketLength;
                while(rem) {
        //        char *pkt =new char[newlen];
// :: NEW check
	NetworkBuffer *nb =  nbfl->getBuffer(); 

#ifdef SPARC 
#ifdef VCHOICES
		char *pkt = nb->packet();
#else
		char *pkt = nb->kpacket();
#endif
#else
                char *pkt = nb->packet();
#endif

		nb->setLength(newlen);
                ByteCopy(p, pkt + sizeof(Header),MaxData);
                Header * header = (Header *) pkt;
                header->transportHeader.destMessagePort = message->destination()->remoteContainer();
		if(message->getcType() == 1)
                	header->transportHeader.replyMessagePort = ((RPCMessage*)message)->getReplyPort();
		else header->transportHeader.replyMessagePort = 0;
        	header->transportHeader.ctype = message->getcType();
                header->transportHeader.messageId = transid;
                header->transportHeader.size = sendlen;
// fragment specific
                header->transportHeader.fragment = ++fragment;
                header->transportHeader.fragmentSize = MaxData;
#ifdef SEND_MESSAGE_DEBUG
                inspect(&(header->transportHeader));
		Console() << "\n" << eor;
		for(int i=0;i<MaxData;i++)
                	Console() << p[i] << " " << eor;
#endif
                reliability->send(pkt,(char *)&(header->transportHeader) ,etherAddr,newlen,nb);
		 nb->release();
                        rem -= MaxData;
                        p += MaxData;
                }
}

#include "ClassInstaller.h"
DEFINECLASS("SendingTransport",SendingTransport,ReceivingTransport,0, 0);

