#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ArContainer.cc - Ar Inode System class
 */

#include "Ar.h"
#include "ArContainer.h"
#include "ArEntry.h"
#include "ArRootEntry.h"
#include "FileSystem.h"
#include "Convert.h"
#include "String.h"
#include "Raid.h"

#ifdef __GNUC__
#pragma implementation
#endif

FileObject *
ArContainer::basicOpen( int id, int & status, int flags )
{
	Raid( RaidMethod )( "%N::basicOpen( inumber %d )\n", this, id );

	FileObject * fo = 0;
	if( id < 1 ) status = IdNumberOutOfRange;
	else {
		status = 0;
		if( id == 1 ) fo = new ArRootEntry( id, this, _source );
		else fo = new ArEntry(  _lengths[id], 0, id, this, _source,
					_offsets[id] + ArchiveHeaderSize,
					&_ar[id] );
	}

	Raid( RaidMethod )( "%N::basicOpen returns %N %d\n", this, fo, status );

	return( fo );
}

ArContainer::~ArContainer()
{
	Raid( RaidConstructor )( "%N::~ArContainer()\n", this );

	if( _valid ) {
		for( int i = 2; i < _numberOfFileObjects; i++ ) {
			_byteSource->write( _offsets[i], ArchiveHeaderSize,
					(char *)&_ar[i], ArchiveHeaderSize );
		}
		delete _ar;
		delete _lengths;
		delete _offsets;
	}

	Raid( RaidConstructor )( "%N::~ArContainer done\n", this );
}

ArContainer::ArContainer( PersistentMemoryObject * source )
: MemoryObjectContainer( source ), _byteSource( 0 )
{
	initDebug();
	_valid = 1;

	Raid( RaidConstructor )( "%N::ArContainer( %N )\n", this, _source );
	int bytes = _source->numberOfBytes();
	if( bytes == 0 ) {
		_valid = 0;
		_offsets = 0;
		_lengths = 0;
		_ar = 0;
		initializeArrays( 0 );
	}
	else {
		_byteSource = new FileObject( bytes, 0, 0, 0, _source );

		Assert( ArchiveMagicSize <= ArchiveHeaderSize );

		char buffer[ArchiveMagicSize];

		_byteSource->read( 0, ArchiveMagicSize, buffer,
							ArchiveMagicSize );
		if( strncmp( buffer, ArchiveMagic, ArchiveMagicSize ) != 0 ) {
			_valid = 0;
		}
		int pos = ArchiveMagicSize;
		for( int objects = 2; ( pos < bytes ) && _valid; objects++ ) {
			ar_hdr ar;
			_byteSource->read( pos, ArchiveHeaderSize,
					    (char *)&ar, ArchiveHeaderSize );
			if( strncmp( ar.ar_fmag, ArchiveFileMagic, 2 ) != 0 ) {
				_valid = 0;
			}
			else {
				int size = atoi( ar.ar_size );
				if( size & 1 ) size++;
				pos += ArchiveHeaderSize + size;
			}
		}
		if( !_valid ) objects = 0;

		initializeArrays( objects );
		
		_offsets = new int[objects];
		_lengths = new int[objects];
		_ar = new ar_hdr[objects];
		pos = ArchiveMagicSize;
		_offsets[0] = 0;	_lengths[0] = 0;
		_offsets[1] = 1;	_lengths[1] = 0;
		for( objects = 2; objects < _numberOfFileObjects; objects++ ) {
			Assert( pos < bytes );

			_offsets[objects] = pos;
			ar_hdr * ar = &_ar[objects];
			_byteSource->read( pos, ArchiveHeaderSize,
					(char *)ar, ArchiveHeaderSize );
			Assert(strncmp(ar->ar_fmag, ArchiveFileMagic, 2) == 0);

			int size = atoi( ar->ar_size );
			_lengths[objects] = size;
			if( size & 1 ) size++;
			pos += ArchiveHeaderSize + size;
		}
	}
}

ProxiableObject * 
ArContainerConstructor( ProxiableObject * source )
{
	if( source->isKindOf( PersistentMemoryObjectClass ) ) {
		return( new ArContainer( (PersistentMemoryObject *)source ) );
	}
	else return( 0 );
}



#include "ClassInstaller.h"
DEFINEPERSISTENTCLASS("ArContainer",ArContainer,MemoryObjectContainer,&ArContainerConstructor,0);
