#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ArDictionary.cc - Ar Dictionary class
 */

#include "Raid.h"
#include "Ar.h"
#include "ArDictionary.h"
#include "FileSystem.h"
#include "Convert.h"
#include "String.h"
#include "PersistentMemoryObject.h"

#ifdef __GNUC__
#pragma implementation
#endif

int
ArDictionary::findKey( char * key, int )
{
	_emptyOffset = -1;
	int len = strlen( key );
	if( ( strcmp( key, "." ) == 0 ) || ( strcmp( key, ".." ) == 0 ) ) {
		_idNumberOfKey = 1;
		return( 1 );
	}
	else for( int i = 2; i < _objects; i++ ) {
		ar_hdr * ar = (ar_hdr *)&_buffer[_offsets[i]];
		if( strncmp( key, ar->ar_name, len ) == 0 ) {
			if( (ar->ar_name[len]==' ') || (ar->ar_name[len]=='/')) {
				_idNumberOfKey = i;
				_offset = _offsets[i];
				return( 1 );
			}
		}
	}
	return( 0 );
}

int
ArDictionary::basicKeys( char * keyBuffer, int bufferSize, int & count )
{
	Raid( RaidMethod )( "%N::basicKeys( buffer %x ) _size %d\n", 
			   this, keyBuffer, bufferSize, _size );
	
	Assert( bufferSize > 6 );
	int bytes = bufferSize - 1;
	char * t = keyBuffer;
	*t++ = '.';
	*t++ = '\n';
	*t++ = '.';
	*t++ = '.';
	*t++ = '\n';
	char * next = t;
	count = 2;
	bytes -= 5;
	
	for( int i = 2; i < _objects && bytes > 0; i++ ) {
		ar_hdr * ar = (ar_hdr *)&_buffer[_offsets[i]];
		char * s = ar->ar_name;
		while( (*s != ' ') && (*s != '/') && bytes > 0 ) {
			*t++ = *s++;
			bytes--;
		}
		if (bytes == 0)
			break;
		*t++ = '\n';
		bytes--;
		next = t;
		count++;
	}

	*next = 0;
	if (i == _objects) return( 0 );
	else return( TooManyKeys );
}

ArDictionary::~ArDictionary()
{
	Raid( RaidConstructor ) ( "%N::~ArDictionary()\n", this );
	delete _offsets;
}

ArDictionary::ArDictionary( PersistentMemoryObject * source )
: MemoryObjectDictionary( source )
{
	initDebug();
	Raid( RaidConstructor ) ( "%N::ArDictionary( %N )\n", this, source );

	_maxKeyLength = 16;

	_offsets = new int[_size/ArchiveHeaderSize];
	int pos = ArchiveMagicSize;
	_offsets[0] = 0;
	_offsets[1] = 1;
	//ar_hdr * ar = new ar_hdr;
	ar_hdr * ar;
	for( _objects = 2; pos < _size; _objects++ ) {
		_offsets[_objects] = pos;
		ar = (ar_hdr *)&_buffer[pos];
		if( strncmp( ar->ar_fmag, ArchiveFileMagic, 2 ) != 0 ) {
			_valid = 0;
		}

		int size = atoi( ar->ar_size );
		if( size & 1 ) size++;
		pos += ArchiveHeaderSize + size;
	}
	//delete ar;
}

ProxiableObject * 
ArDictionaryConstructor( ProxiableObject * source )
{
	if( source->isKindOf( PersistentMemoryObjectClass ) ) {
		return( new ArDictionary( (PersistentMemoryObject *)source ) );
	}
	else return( 0 );
}



#include "ClassInstaller.h"
DEFINEPERSISTENTCLASS("ArDictionary",ArDictionary,MemoryObjectDictionary,&ArDictionaryConstructor,0);
