#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ArEntry.c - Ar  Directory Entry (inode) code
 */

#include "Assert.h"
#include "FileSystem.h"
#include "UNIXFile.h"
#include "ArEntry.h"
#include "Ar.h"
#include "Convert.h"
#include "String.h"
#include "Class.h"
#include "Raid.h"

#ifdef __GNUC__
#pragma implementation
#endif

extern ClassStar PersistentArrayClass;

ArEntry::ArEntry( unsigned int numberOfUnits,
		  int log2UnitSize,
                  int objectNumber,
		  MemoryObjectContainer * container,
		  PersistentMemoryObject * source,
		  int offset,
		  ar_hdr * ar )
: FileObject( numberOfUnits, log2UnitSize, objectNumber, container, source, offset ) 
{
	initDebug();
	Raid( RaidConstructor )( "%N::ArEntry()\n", this );
	_ar = ar;
}

Class *
ArEntry::supports( Class * aClass )
{
	Raid( RaidProtected )( "%N::supports( %N )\n", this, aClass );

	Class * supportedClass = 0;

	if( PersistentArrayClass->isASubclassOf( aClass ) ) {
            supportedClass = PersistentArrayClass;
        }

	Raid( RaidProtected )( "%N::supports() returns %N\n",
				this, supportedClass );

	return( supportedClass );
}

int
ArEntry::numberOfBytes()
{
    int bytes = unitSize() * numberOfUnits();
    Raid( RaidMethod )( "%N::numberOfBytes() %d\n", this, bytes );
    return( bytes );
}

int
ArEntry::setMode( int mode )
{
    Raid( RaidMethod )( "%N::setMode( %o )\n", this, mode );
    for( int i = 0; i < 4; i++ ) {
	int shift = i * 3;
	int mask = 7 << shift;
	_ar->ar_mode[5 - i] = '0' + ( ( mode & mask ) >> shift );
    }
    return( 0 );
}

int
ArEntry::accessMode()
{
    return( atoi( _ar->ar_mode, 8 ) & UNIXAccessMask );
}

int
ArEntry::userId()
{
    return( atoi( _ar->ar_uid ) );
}

int
ArEntry::groupId()
{
    return( atoi( _ar->ar_gid ) );
}

void
ArEntry::info( UNIXFileInfo * infoBuffer )
{
    PersistentMemoryObject::info( infoBuffer );
    infoBuffer->mode = atoi( _ar->ar_mode, 8 );
}


#include "ClassInstaller.h"
DEFINECLASS("ArEntry",ArEntry,FileObject,0,0);
