#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * AutoloadPersistentObject.c - 
 */
#include "Assert.h"
#include "Raid.h"
#include "AutoloadPersistentObject.h"
#include "PersistentMemoryObject.h"
#include "Byte.h"
#include "PersistentClass.h"

#ifdef __GNUC__
#pragma implementation
#endif

AutoloadPersistentObject::AutoloadPersistentObject( PersistentMemoryObject * source )
: PersistentObject( source )
{
	initDebug();

	Raid( RaidConstructor )( "%N::AutoloadPersistentObject( %N )\n",
				  this, source );

	_arrayBuffer = 0;
}

AutoloadPersistentObject::~AutoloadPersistentObject()
{
	Raid( RaidConstructor )( "%N::~AutoloadPersistentObject\n", this );
}

void
AutoloadPersistentObject::noRemainingReferences()
{
	Raid( RaidReference )( "%N::noRemainingReferences\n", this );

	flush();

	int size = classOf()->sizeOf() - PersistentObjectClass->sizeOf();
	char * source = ((char *)this) + PersistentObjectClass->sizeOf();

	if( _modified ) {
		int units = _source->numberOfUnits();
		Assert( size >= 0 );

		if( ( units == 0 ) && ( size > 0 ) ) {
			_source->setNumberOfBytes( size );
			units = _source->numberOfUnits();
		}

		int bufferSize = units << _source->log2UnitSize();

		char * buffer = new char[bufferSize];
		ByteCopy( source, buffer, size );
		{
			_source->write( 0, units, buffer, bufferSize );
		}

		delete buffer;
	}

	ByteZero( source, size );

	_source->close( this );
}

/*
int
sizeOfAutoloadPersistentObject()
{
	return( sizeof( AutoloadPersistentObject ) );
}

Class *
AutoloadPersistentObject::classOf()
{
	return( AutoloadPersistentObjectClass );
}

*/

#include "ClassInstaller.h"
DEFINEPERSISTENTCLASS("AutoloadPersistentObject",AutoloadPersistentObject,PersistentObject,0,0);
