#ifdef __GNUC__
#pragma implementation
#pragma implementation "PersistentClassStar.h"
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Contributing Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * BSDContainer.c - 
 */

//HACK BY SCOTT KIRCHER
//(Why doesn't this get defined when I set it in Config.mk?)
#ifndef COFFFile
#define COFFFile
#endif

#include "Assert.h"
#include "FileSystem.h"
#include "BSDInode.h"
#include "BSDContainer.h"
#include "BSDAllocator.h"
#include "FileObject.h"
#include "UNIXFile.h"
#include "UNIXInodeIndirect.h"
#include "Class.h"
#include "Raid.h"
extern ClassStar HashedBSDDirectoryClass;
extern ClassStar MemoryObjectDictionaryClass;
extern ClassStar SymbolicLinkClass;

#ifdef __GNUC__
#pragma implementation
#endif

extern int BSDInodeDefaultCacheHints;

BSDContainer::BSDContainer( PersistentMemoryObject * source )
: UNIXContainer( source )
{
    initDebug();
    Raid( RaidConstructor )( "%N::BSDContainer( %N )\n", this, source );

    /*
     *	Create super block buffer, chainize it, read it from the disk,
     *	and check its magic number.
     */
    _superBlock = (fs *) new char[SBSIZE];
    int count = SBSIZE >> _source->log2UnitSize();
    int unitsRead = _source->read( SBLOCK, count, (char *) _superBlock, SBSIZE );
    Assert( unitsRead == count );
    _groupMask = ~_superBlock->fs_cgmask;
    _fragmentsPerBlock = _superBlock->fs_frag;

    if( _superBlock->fs_magic != FbMAGIC ) {
        Raid( RaidError )( "fsmagic %x!=%x\n", _superBlock->fs_magic, FbMAGIC );
	Assert( _superBlock->fs_magic == FbMAGIC );
    }

    /*
     *	Calculate constants needed by UNIXContainer.
     */
    _log2FragmentToSector = _superBlock->fs_fsbtodb;
    _dinodeSize = sizeof( dinode );
    _inodesPerBlock = _superBlock->fs_inopb;
    _inodeBlockSize = _inodesPerBlock * _dinodeSize;

    /*
     *  Create Indirect Block Cache
     */

    _indirect = UNIXInodeIndirect::setup
		( _source->log2UnitSize() + _log2FragmentToSector + 
		  _superBlock->fs_fragshift );

    /*
     *	Create cylinder group summary information buffer, chainize it,
     *	and read it from the disk.
     */
    _cylSumMem = (csum *) new char[_superBlock->fs_cssize];
    count = _superBlock->fs_cssize >> _source->log2UnitSize();
    if( ( count << _source->log2UnitSize() ) != _superBlock->fs_cssize ) {
	count++;
    }
    unitsRead = _source->read( _superBlock->fs_csaddr << _log2FragmentToSector,
		count, (char *) _cylSumMem, count << _source->log2UnitSize() );
    Assert( unitsRead == count );

    /*
     *	Create cylinder group buffers, chainize them,
     *	read them from the disk, and check their magic numbers.
     */
    _cylGroup = new cg * [_superBlock->fs_ncg];
    _allocatorGroup = new ag * [_superBlock->fs_ncg];
    _iused = new unsigned char * [_superBlock->fs_ncg];
    _cgCache = new cgCache [_superBlock->fs_ncg];
    for( int i = 0; i < _superBlock->fs_ncg; i++ ) {
	unsigned int fragment = mapCylGroup( i ) + _superBlock->fs_cblkno;
	unsigned int sector = fragment << _log2FragmentToSector;
	_cgCache[i]._info = 0;
	_cgCache[i]._buf = 0;
	_cgCache[i]._sector = sector;
	_allocatorGroup[i] = new ag;
    }

    /*
     *	Allocate and clear in memory and on disk inode pointers.
     */
    int containerSize = _superBlock->fs_ipg * _superBlock->fs_ncg;
    UNIXContainer::initializeArrays( containerSize );
    _allocator = new BSDAllocator( _source, this, _allocatorGroup,
				   _superBlock->fs_size,
				   &( _superBlock->fs_nbfree ),
				   &( _superBlock->fs_nffree ),
				   _log2FragmentToSector, _fragmentsPerBlock,
				   _superBlock->fs_fpg, _superBlock->fs_spc,
				   _superBlock->fs_ncg, _superBlock->fs_nsect,
				   _superBlock->fs_ipg );
}

BSDContainer::~BSDContainer()
{
    Raid( RaidConstructor )( "%N::~BSDContainer\n", this );

    synchronize();

    /*
     *	Delete everything allocated in the constructor.
     */
    Raid( RaidDetail )( "%N::~BSDContainer numMO %d num CG %d\n",
			this, _numberOfFileObjects, _superBlock->fs_ncg );
    for( int i = 0; i < _superBlock->fs_ncg; i++ ) {
	delete _allocatorGroup[i];
    }

    delete _cgCache;
    delete _iused;
    delete _allocatorGroup;
    delete _cylGroup;
    delete _cylSumMem;
    delete _superBlock;

    Raid( RaidConstructor )( "%N::~BSDContainer done\n", this );
}

void BSDContainer::getCG( int group )
{
    cgCache * cgc = _cgCache + group;
    cgc->_info = _indirect->getIndirect( _source, cgc->_sector, 0, cgc->_info );
    char * buf = (char *) cgc->_info->buf();
    if ( buf == cgc->_buf ) {
	return;
    }
    static Lock localLock;
    localLock.acquire();
    if ( buf == cgc->_buf ) {
	localLock.release();
	return;
    }
    _cylGroup[group] = (cg *) buf;
    if( _cylGroup[group]->cg_magic == CGbMAGIC ) {
	_iused[group] =(unsigned char*) buf + _cylGroup[group]->cg_iusedoff;
	_allocatorGroup[group]->ag_free = 
		(unsigned char *)buf + _cylGroup[group]->cg_freeoff;
	_allocatorGroup[group]->ag_btot = 
		(int *)(buf + _cylGroup[group]->cg_btotoff);
	_allocatorGroup[group]->ag_b = 
		(short *)(buf + _cylGroup[group]->cg_boff);
    }
    else {
	ocg * oldGroup = (ocg *)_cylGroup[group];
	Assert( oldGroup->cg_magic == CGbMAGIC );
	_iused[group] = oldGroup->cg_iused;
	_allocatorGroup[group]->ag_free = oldGroup->cg_free;
	_allocatorGroup[group]->ag_btot = oldGroup->cg_btot;
	_allocatorGroup[group]->ag_b = oldGroup->cg_b;
    }
    _allocatorGroup[group]->ag_frsum = _cylGroup[group]->cg_frsum;
    _allocatorGroup[group]->ag_ndblk = &(_cylGroup[group]->cg_ndblk);
    _allocatorGroup[group]->ag_nbfree = &(_cylGroup[group]->cs_nbfree);
    _allocatorGroup[group]->ag_rotor = &(_cylGroup[group]->cg_rotor);
    cgc->_buf = buf;
    localLock.release();
}

void BSDContainer::releaseCG( int group, int dirty )
{
    _indirect->releaseIndirect( _cgCache[group]._info, dirty );
}

/*
 * Find block number containing this inode.
 */
int
BSDContainer::mapInumber( int inumber )
{
    int group = inumber / _superBlock->fs_ipg;
    int block = ( inumber % _superBlock->fs_ipg ) / _inodesPerBlock;
    int fragment =  block << _superBlock->fs_fragshift;
    int firstFragment = mapCylGroup( group ) + _superBlock->fs_iblkno;
    return( firstFragment + fragment );
}

/*
 * Find first inode in block containing this inode.
 */
int
BSDContainer::firstInumber( int inumber )
{
    return( ( inumber / _inodesPerBlock ) * _inodesPerBlock );
}

/*
 * 	Find suitable place to put a new directory
 */

int
BSDContainer::directoryPref()
{
    int avgifree = _superBlock->fs_ndir / _superBlock->fs_ncg;
    int minndir = _superBlock->fs_ipg;
    int mincg = 0;
    for ( int cg = 0; cg < _superBlock->fs_ncg; cg++ ) {
	getCG(cg);
	if ( _cylGroup[cg]->cs_ndir < minndir &&
	     _cylGroup[cg]->cs_nifree >= avgifree ) {
		mincg = cg;
		minndir = _cylGroup[cg]->cs_ndir;
	}
	releaseCG(cg, 0);
    }
    return _superBlock->fs_ipg * mincg;
}

/*
 *	Find an unused inode, start looking in cylinder group "cyl".
 *	Returns the inumber of the inode found, or zero if none found.
 *	The inode is marked in use by this function.
 */

int
BSDContainer::getUnusedInodeCyl( int cyl, int inumber )
{
    Assert( cyl >= 0 && cyl < _superBlock->fs_ncg );

    const unsigned char allUsed = 0xff;
    int i;
    int len;
    int start;
    unsigned char uc;
    unsigned char * cp;

    getCG( cyl );
    cg * cylgrp = _cylGroup[cyl];

    if ( cylgrp->cs_nifree == 0 ) {
	releaseCG( cyl, 0 );
	return 0;
    }

    unsigned offset;
    if ( inumber ) {
    	offset = inumber % _superBlock->fs_ipg;
        if ( bitIsClear( _iused[cyl], offset ) )
	    goto done;
    } 
  
    // assume 8 bits per byte

    start = cylgrp->cg_irotor >> 3;
    len = ( _superBlock->fs_ipg - cylgrp->cg_irotor + 7 ) >> 3;
    cp = &(_iused[cyl][start]);
    int loc;
    for( loc = len; loc > 0; loc-- ) {
	if ( *cp != allUsed ) 
	    break;
	cp++;
    }
    if ( loc == 0 ) {
	len = start + 1;
	start = 0;
        cp = &(_iused[cyl][0]);
        for( loc = len; loc > 0; loc-- ) {
   	    if ( *cp != allUsed ) 
	         break;
	    cp++;
	}
  	Assert( loc != 0 );
    }
    offset = (start + len - loc) << 3;
    uc = *cp;
    for ( i = 1; i < ( 1 << 8 ); i <<= 1, offset++ ) {
 	if ( ( uc & i ) == 0 ) {
	    cylgrp->cg_irotor = offset;
	    goto done;
	}
    }
    Assert( NOTREACHED );
done:

    // Make sure the one found is free, mark it in use.
    setBit( _iused[cyl], offset );
    cylgrp->cs_nifree--;
    _superBlock->fs_nifree--;
    
    // The superblock is now modified.
    _superBlock->fs_time = 0;

    inumber = offset + cyl * _superBlock->fs_ipg;
	
    releaseCG( cyl, 1 );

    return( inumber );
}

int
BSDContainer::getUnusedInode( int inumber )
{
    Raid( RaidMethod )( "%N::getUnusedInode( inumber %d )\n", this, inumber );
    
    if ( _superBlock->fs_nifree == 0 ) {
	/* no more inodes */
	return 0;
    }
    if ( inumber >= _superBlock->fs_ncg * _superBlock->fs_ipg)
	inumber = 0;
    int cyl = inumber / _superBlock->fs_ipg;
    int orgCyl = cyl;

    int result = getUnusedInodeCyl( cyl, inumber );
    if ( result )
	return result;

    int i;
    for ( i = 1; i < _superBlock->fs_ncg; i *= 2 ) {
	cyl += i;
	if ( cyl >= _superBlock->fs_ncg ) 
	     cyl -= _superBlock->fs_ncg;
	result = getUnusedInodeCyl( cyl, inumber );
	if ( result )
	     return result;
    }

    cyl = ( orgCyl + 2 ) % _superBlock->fs_ncg;
    for ( i = 2; i < _superBlock->fs_ncg; i++ ) {
	result = getUnusedInodeCyl( cyl, inumber );
	if ( result )
	     return result;
	cyl++;
	if ( cyl == _superBlock->fs_ncg ) 
	     cyl = 0;
    }
    Assert( NOTREACHED );
    return 0;
}

/*
 *	Open returns a pointer to a sub-memory-object ( inode )
 *	with the given number.
 *	If the inumber is negative, an unused one is allocated an initialized.
 *	Also, the new inumber is passed back thru the reference parameter.
 *	If the inode is already in use,
 *	update its refernce count.
 *	Otherwise create a new in memory inode and return a pointer to it.
 *	Creating a new memory inode will also read in the disk inode if needed.
 */
FileObject *
BSDContainer::basicOpen( int inumber, int & status, int flags )
{
    Raid( RaidMethod )( "%N::basicOpen( inumber %d )\n", this, inumber );

    if( inumber < basicRootId() ) {
	status = IdNumberOutOfRange;
	return( 0 );
    }

    /*
     *	Read this inode from the disk,
     *	Check to see if it exists,
     *	and construct its in memory structure.
     */
    acquire();
    bind( inumber, (FileObject *) ~0 );
    UNIXInodeIndirectInfo ** info;
    dinode * thisDinode = (dinode *) getDinode( inumber, info );

    if( thisDinode->di_nlink == 0 ) {
	status = UnusedIdNumber;
	unbind( inumber );
	writeDinode( inumber, 1, 1 );
    	release();
	return( 0 );
    }
    release();

    status = 0;
    return( new BSDInode( numFragments( thisDinode->di_size, NDADDR ),
			  _source->log2UnitSize() + _log2FragmentToSector,
			  inumber, this, _source,
			  _superBlock->fs_fragshift, thisDinode, info ) );
}

/*
 *	create returns a pointer to a new sub-memory-object ( inode )
 * 	The inumber passed in is just used as a hint to the file system.
 *	The Class parameter is primarily used to set the type of the
 *	file, it should not change during the life of the file.
 *	Uig, gid, and file permissions can be set via methods on an Inode.
 */
FileObject *
BSDContainer::basicCreate( int inumber, Class * aClass, int & status, 
			   int flags )
{
    Raid( RaidMethod )( "%N::basicCreate( inumber %d, %N, )\n",
			this, inumber, aClass );

    acquire();
    if( inumber < basicRootId() ) inumber = 0;
    if( aClass->isASubclassOf( MemoryObjectDictionaryClass ) ) 
	inumber = directoryPref();
    inumber = getUnusedInode( inumber );
    if( inumber <= 0 ) {
	status = ContainerFull;
	release();
	return( 0 );
    }

    /*
     *	Read the inode from disk, and reset all of its fields.
     */

    bind( inumber, (FileObject *) ~0 );
    UNIXInodeIndirectInfo ** info;
    dinode * thisDinode = (dinode *) getDinode( inumber, info );

    thisDinode->di_flags = 0;
    thisDinode->di_blocks = 0;
    thisDinode->di_nlink = 0;
    thisDinode->di_mode = UNIXAccessMask;
    thisDinode->di_spare[2] = BSDInodeDefaultCacheHints;

    if( aClass->isASubclassOf( SymbolicLinkClass ) ) {
	thisDinode->di_mode |= UNIXTypeSymbolicLink;
    }
    else if( aClass->isASubclassOf( MemoryObjectDictionaryClass ) ) {
        if( aClass->isASubclassOf( HashedBSDDirectoryClass ) ) {
	    thisDinode->di_flags |= IC_HASHED_DIR;
        }
	thisDinode->di_mode |= UNIXTypeDirectory;
    }
    else {
#ifdef ArFile
	extern ClassStar ArContainerClass;
#endif
#ifdef MailFile
	extern ClassStar MailContainerClass;
#endif
#ifdef TarFile
	extern ClassStar TarContainerClass;
#endif
#ifdef COFFFile
	extern ClassStar COFFContainerClass;
#endif
#ifdef ArFile
        if( aClass->isASubclassOf( ArContainerClass ) ) {
            thisDinode->di_spare[3] = 2;
        }
        else 
#endif
#ifdef MailFile
	if( aClass->isASubclassOf( MailContainerClass ) ) {
            thisDinode->di_spare[3] = 3;
        }
        else 
#endif
#ifdef TarFile
	if( aClass->isASubclassOf( TarContainerClass ) ) {
            thisDinode->di_spare[3] = 4;
        }
        else 
#endif
#ifdef COFFFile
	if( aClass->isASubclassOf( COFFContainerClass ) ) {
            thisDinode->di_spare[3] = 5;
        }else
#endif
/*	extern ClassStar ELFContainerClass;
	if( aClass->isASubclassOf( ELFContainerClass ) ) {
			thisDinode->di_spare[3] = 6;
		}*/

	thisDinode->di_mode |= UNIXTypeRegular;
    }

    thisDinode->di_uid = 0;
    thisDinode->di_gid = 0;
    thisDinode->di_size[0] = 0;
    thisDinode->di_size[1] = 0;
    // FIX: what time is it?
    thisDinode->di_atime =
    thisDinode->di_mtime =
    thisDinode->di_ctime = 0;
    int i;
    for( i = 0; i < NDADDR; i++ ) thisDinode->di_db[i] = 0;
    for(     i = 0; i < NIADDR; i++ ) thisDinode->di_ib[i] = 0;

    if( aClass->isASubclassOf( MemoryObjectDictionaryClass ) ) {
        int cyl = inumber / _superBlock->fs_ipg;
        _cylGroup[cyl]->cs_ndir++;
	_superBlock->fs_ndir++;
    }

    /*
     *	Create the new in memory inode.
     */
    status = 0;

    *((*info)->dirtyFlag()) = 1;

    release();

    return( new BSDInode( numFragments( thisDinode->di_size, NDADDR ),
			  _source->log2UnitSize() + _log2FragmentToSector,
			  inumber, this, _source,
			  _superBlock->fs_fragshift, thisDinode, info ) );
}

void
BSDContainer::basicClose( FileObject * thisInode )
{
    Raid( RaidMethod )( "%N::basicClose( %N )\n", this, thisInode );

    int inumber = thisInode->idNumber();

    Raid( RaidMethod )( "%N::basicClose() inumber %d \n", this, inumber );

    /*
     *	If their are no links to this inode, then this call will free 
     *  its space.
     *	FIX: this isnt ready yet.
     */
    int links = thisInode->links( 0 );
    Raid( RaidMethod )( "%N::basicClose links %d\n", this, links );

    int directory = thisInode->isDictionary();

    MemoryObjectContainer::basicClose( thisInode );

    /*
     *	Write the disk inode back to disk.
     */

    if ( links > 0 ) {
    	writeDinode( inumber, 0, 1 );
    }

    acquire();
    if( links == 0 ) {
        int cg = inumber / _superBlock->fs_ipg;
        int cg_inumber = inumber % _superBlock->fs_ipg;
        Raid( RaidMethod )( "%N::basicClose cg_inumber %d cg %d\n",
		   this, cg_inumber, cg );

	getCG( cg );
        Assert( bitIsSet( _iused[cg], cg_inumber ) );

        clearBit( _iused[cg], cg_inumber );
	if ( directory ) {
            _cylGroup[cg]->cs_ndir--;
	    _superBlock->fs_ndir--;
        }
        _cylGroup[cg]->cs_nifree++;
	_superBlock->fs_nifree++;
        _modified = 1;
	releaseCG( cg, 1 );
    }
    release();
}

int
BSDContainer::basicRootId()
{
    return( 2 );
}

/*
 *	Synchronize the in-core data structures with the disk.
 */
void
BSDContainer::basicSynchronize()
{
    Raid( RaidMethod )( "%N::basicSynchronize() modified %d\n",
		        this, _modified );
    
    basicSynchronizeInodes();

    Assert( _superBlock->fs_magic == FbMAGIC );

    int fs_nffree = _superBlock->fs_nffree;
    // _superBlock->fs_nbfree = 0;
    _superBlock->fs_nffree = 0;
    // _superBlock->fs_nifree = 0;
    // _superBlock->fs_ndir  = 0;
    for( int i = 0; i < _superBlock->fs_ncg; i++ ) {
	getCG( i );
        _cylGroup[i]->cs_nffree = 0;
        for( int f = 0; f < MAXFRAG; f++ ) {
		_cylGroup[i]->cs_nffree += _cylGroup[i]->cg_frsum[f] * f;
	}
        // _superBlock->fs_nbfree += _cylGroup[i]->cs_nbfree;
        _superBlock->fs_nffree += _cylGroup[i]->cs_nffree;
        // _superBlock->fs_nifree += _cylGroup[i]->cs_nifree;
        // _superBlock->fs_ndir   += _cylGroup[i]->cs_ndir;
        _cylSumMem[i].cs_nbfree = _cylGroup[i]->cs_nbfree;
        _cylSumMem[i].cs_nffree = _cylGroup[i]->cs_nffree;
        _cylSumMem[i].cs_nifree = _cylGroup[i]->cs_nifree;
        _cylSumMem[i].cs_ndir   = _cylGroup[i]->cs_ndir;
	releaseCG( i, 1 );
    }
    if ( fs_nffree != _superBlock->fs_nffree ) {
	Console() << this << "::basicSynchronize() potential inconsistency "
		  << " fs_nffree " << fs_nffree << " != "
	  	  << " fs_nffree computed from cul groups " 
		  << _superBlock->fs_nffree << "\n" << eor;
    }
    _superBlock->fs_fmod = 0;
    _source->write( SBLOCK, SBSIZE >> _source->log2UnitSize(),
					(char *) _superBlock, SBSIZE );

    // Write out the cylinder summary information.
    int units = _superBlock->fs_cssize >> _source->log2UnitSize();
    if( ( units << _source->log2UnitSize() ) != _superBlock->fs_cssize ) {
	units++;
    }
    _source->write( _superBlock->fs_csaddr << _log2FragmentToSector,
	    units, (char *) _cylSumMem, units << _source->log2UnitSize() );

    // Write out the cylinder group information.
    _indirect->synchronize( _source, 0 );
}

int
BSDContainer::numFragments( int sizes[], int ndaddr )
{
    union {
	int     i;
	char    c[4];
    } littleEndian;
    littleEndian.i = 1;
    int size = sizes[!littleEndian.c[0]];

    Raid( RaidMethod )( "%N::numFragments( size %d, ndaddr %d )\n",
	 	        this, size, ndaddr );

    if( size == 0 ) return( 0 );

    int roundSize;
    if ( ( size >> _superBlock->fs_bshift ) >= ndaddr ) {
	roundSize = ( size + _superBlock->fs_bsize - 1) &
		    _superBlock->fs_bmask;
    }
    else {
	roundSize = ( size + _superBlock->fs_fsize - 1) &
		    _superBlock->fs_fmask;
    }

    roundSize >>= _superBlock->fs_fshift;
    
    Raid( RaidMethod )( "%N::numFragment returns %d\n", this, roundSize );
    return( roundSize );
}

int
BSDContainer::blockPref( int inumber, int * buf, int start, int startBlock )
{
    if ( start % _superBlock->fs_maxbpg == 0 ||
	 buf[ start - 1 ] == 0 ) {
	if ( start < NDADDR ) {
	    int cg = inumber / _superBlock->fs_ipg;
	    return ( _superBlock->fs_fpg * cg + _superBlock->fs_frag );
	}
	int startcg;
	if ( start == 0 || buf[ start - 1 ] == 0 ) {
	    startcg = inumber / _superBlock->fs_ipg +
		      startBlock / _superBlock->fs_maxbpg;
	} else {
	    startcg = buf[ start - 1 ] / _superBlock->fs_fpg + 1;
	}
	startcg %= _superBlock->fs_ncg;
 	int avgbfree = _superBlock->fs_nbfree / _superBlock->fs_ncg;
	int cg;
	for ( cg = startcg; cg < _superBlock->fs_ncg; cg++ ) {
	    getCG( cg );
	    if ( *(_allocatorGroup[cg]->ag_nbfree) >= avgbfree ) {
		_superBlock->fs_cgrotor = cg;
		releaseCG( cg, 0 );
		return ( _superBlock->fs_fpg * cg + _superBlock->fs_frag );
	    }
	    releaseCG( cg, 0 );
	}
	for ( cg = 0; cg <= startcg; cg++ ) {
	    getCG( cg );
	    if ( *(_allocatorGroup[cg]->ag_nbfree) >= avgbfree ) {
		_superBlock->fs_cgrotor = cg;
		releaseCG( cg, 0 );
		return ( _superBlock->fs_fpg * cg + _superBlock->fs_frag );
	    }
	    releaseCG( cg, 0 );
	}
	return 0;
    }
    int nextBlock = buf[ start - 1 ] + _fragmentsPerBlock;
#ifdef MAXCONTIG
    if ( _superBlock->fs_rotdelay != 0 &&
	 ( start <= _superBlock->fs_maxcontig || 
	   ( buf[ start - _superBlock->fs_maxcontig ] +
	     ( _superBlock->fs_maxcontig << _superBlock->fs_fragshift )
	     == nextBlock ) ) ) {
	int inc = _superBlock->fs_rotdelay * _superBlock->fs_rps *
		  _superBlock->fs_nsect / ( _superBlock->fs_nspf * 1000 );
	inc = ( ( inc + _fragmentsPerBlock - 1 ) / _fragmentsPerBlock ) * 
	      _fragmentsPerBlock;
	nextBlock += inc;
    }
#endif
    return nextBlock; 
}

ProxiableObject * 
BSDContainerConstructor( ProxiableObject * source )
{
    if( source->isKindOf( PersistentMemoryObjectClass ) ) {
        return( new BSDContainer( (PersistentMemoryObject *)source ) );
    }
    else return( 0 );
}

OutputStream & operator<< ( OutputStream & stream, ag & allocatorGroup )
{
    stream  << "ag_b " << *(allocatorGroup.ag_b)
	    << " ag_free " << asAddress(allocatorGroup.ag_free)
	    << " ag_btot " << *(allocatorGroup.ag_btot)
	    << " ag_frsum " << *(allocatorGroup.ag_frsum)
	    << " ag_ndblk " << *(allocatorGroup.ag_ndblk)
	    << " ag_nbfree " << *(allocatorGroup.ag_nbfree);
    return stream;
}

#include "ClassInstaller.h"
DEFINEPERSISTENTCLASS("BSDContainer",BSDContainer,UNIXContainer,&BSDContainerConstructor,0);
