#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Contributing Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Contributing Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * BSDInode.c - 
 */

//HACK BY SCOTT KIRCHER
//(Why doesn't this get defined when I set it in Config.mk?)
#ifndef COFFFile
#define COFFFile
#endif

#include "Assert.h"
#include "Byte.h"
#include "BSDAllocator.h"
#include "BSDContainer.h"
#include "BSDInode.h"
#include "Class.h"
#include "FileSystem.h"
#include "FileSystemHints.h"
#include "MemoryObjectAsync.h"
#include "MemoryObjectContainer.h"
#include "MemoryObjectDictionary.h"
#include "Raid.h"
#include "RemoteFileSystem.h"
#include "SoftClock.h"
#include "Time.h"
#include "UNIXFile.h"
#include "UNIXInodeIndirect.h"

#ifdef __GNUC__
#pragma implementation
#endif

extern ClassStar PersistentArrayClass;
extern ClassStar SymbolicLinkClass;

int BSDInodeDefaultCacheHints = (H_LCACHE << 16);

int debugHints = 0;

inline
void BSDInode::initDinode()
{
    direct = _diskInode->di_db;
    singleIndex = _diskInode->di_ib;
    doubleIndex = _diskInode->di_ib + 1;
    tripleIndex = _diskInode->di_ib + 2;
    _modifiedFlag = (*(_inodeData->_inodeIndirectInfo))->dirtyFlag();
//    Assert( _diskInode->di_nlink == _inodeData->_di_nlink );
}

void BSDInode::getDinode()
{
    *(_inodeData->_inodeIndirectInfo) = indirect->getIndirect
	( _inodeData->_partition, 
	  _inodeData->_inodeBlock, 
	  0xffffffff,
	  *(_inodeData->_inodeIndirectInfo) );
    dinode * newDiskInode = 
	( (dinode *) ( (*(_inodeData->_inodeIndirectInfo))->buf() ) ) + 
	_inodeData->_inodeOffset;
    if ( _diskInode != newDiskInode ) {
	_diskInode = newDiskInode;
	initDinode();
    }
}

void BSDInode::releaseDinode()
{
    indirect->releaseIndirect( *(_inodeData->_inodeIndirectInfo), 0 );
}

/* 
 * Initialize the in-memory inode
 */
BSDInode::BSDInode( unsigned int numOfUnits,
		    int log2UnitSize,
		    int inumber,
		    MemoryObjectContainer * container,
		    PersistentMemoryObject * source,
		    int log2blocksize,
		    dinode * thisDinode, 
		    UNIXInodeIndirectInfo ** inodeIndirectInfo )
	: UNIXInode( numOfUnits, log2UnitSize, inumber, 
		     container, source, log2blocksize )
{
    initDebug();
 
    Raid( RaidConstructor )( "%N::BSDInode()\n", this );

    /*
     *	Initialize variables from parameters.
     */
    _diskInode = thisDinode;
    _inodeData = new _InodeData;
    _version = _diskInode->di_version;
    _hints = _diskInode->di_spare[2];
    if (debugHints) {
   	 Console() 
		<< this << "::ctor _hints " << asHex(_hints) << "\n" << eor;
    }
    _inodeData->_di_mode = _diskInode->di_mode;
    _inodeData->_di_nlink = _diskInode->di_nlink;
    _inodeData->_di_flags = _diskInode->di_flags;
    _inodeData->_di_spare3 = _diskInode->di_spare[3];
    
    union {
	int	i;
	char	c[4];
    } littleEndian;
    littleEndian.i = 1;
    _inodeData->_di_size = _diskInode->di_size[!littleEndian.c[0]];

    _inodeData->_inodeIndirectInfo = inodeIndirectInfo;
    _inodeData->_partition = (*inodeIndirectInfo)->source();
    _inodeData->_inodeBlock = (*inodeIndirectInfo)->unit();
    _inodeData->_inodeOffset = 
	_diskInode - ( (dinode *) ( (*inodeIndirectInfo)->buf() ) );

    initDinode();

    /*
     *	Set up indirect variables for mapUnit function.
     */
    singleIndirect = NDADDR;
    doubleIndirect = singleIndirect + ( 1 << indexShift );
    tripleIndirect = doubleIndirect + ( 1 << ( indexShift + indexShift ) );

    /*
     * Used by allocBlock to allocate blocks near each other.
     */
    nearBlock = 0;

    /*
     * Calculate the number of fragments in the last logical block of the file.
     */
    if( !( _diskInode->di_flags & IC_FASTLINK ) ) {
        fragmentsInLastBlock = numFragments();
        if( fragmentsInLastBlock > 0 ) {
	    nearBlock = mapUnit( lastBlockNo(), 0 );
            if( nearBlock == 0 ) fragmentsInLastBlock = 0;
	}
    }

    /*
     * Set access time
     */
    _diskInode->di_atime = _theTime.sec();

    releaseDinode();

    MemoryObject::basicSetNumberOfUnits( _numberOfUnits );
}

BSDInode::~BSDInode()
{
    Raid( RaidConstructor )( "%N::~BSDInode() fragsInLastBlk %d\n",
			     this, fragmentsInLastBlock );


    if (debugHints) {
   	 Console() 
		<< this << "::dtor _hints " << asHex(_hints) 
		<< " zombieDelay " << zombieDelay() << "\n" << eor;
    }
    if( _inodeData->_di_nlink == 0 ) {
    	getDinode();
	Assert( _inodeData->_di_nlink == _diskInode->di_nlink );
	Assert( zombieInfo()->type() == ZOMBIE_NOT );
	updateInode( 1 );
	_diskInode->di_mode = 0;
	_diskInode->di_atime = 0;
	_diskInode->di_ctime = 0;
	_diskInode->di_mtime = 0;
	_diskInode->di_size[0] = 0;
	_diskInode->di_size[1] = 0;
	_diskInode->di_version++;
	_diskInode->di_spare[0] = 0;
	_diskInode->di_spare[1] = 0;
	_diskInode->di_spare[2] = 0;
	_diskInode->di_spare[3] = 0;
	_version = _diskInode->di_version;
	setModified();
        releaseDinode(); 
    } else {
	if ( zombieInfo()->type() == ZOMBIE_NOT ||
	     zombieInfo()->type() == ZOMBIE_NOTSYNC )
    	    synchronizeInode();
    }

    _diskInode = 0;
    delete _inodeData;
    _inodeData = 0;
}

Class *
BSDInode::supports( Class * aClass )
{
    Raid( RaidProtected )( "%N::supports( %N )\n", this, aClass );

Console()<<"BSDInode::supports()!\n";
 Console()<<"aClass="<<aClass<<"\n";

    Class * supportedClass = 0;

    if( ( _inodeData->_di_spare3 == 0 ) &&
	( ( _inodeData->_di_mode & UNIXTypeMask ) == UNIXTypeRegular ) ) {
 	if( PersistentArrayClass->isASubclassOf( aClass ) ) {
                supportedClass = PersistentArrayClass;
        }
    }
    else if( ( _inodeData->_di_mode & UNIXTypeMask ) == UNIXTypeDirectory ) {
	extern ClassStar BSDDirectoryClass;
	extern ClassStar HashedBSDDirectoryClass;
	if( _inodeData->_di_flags & IC_HASHED_DIR ) {
                if( HashedBSDDirectoryClass->isASubclassOf( aClass ) ) {
                        supportedClass = HashedBSDDirectoryClass;
                }
        }
	else {
		if( BSDDirectoryClass->isASubclassOf( aClass ) ) {
			supportedClass = BSDDirectoryClass;
		}
	}
    }
    else if( ( _inodeData->_di_mode & UNIXTypeMask ) == UNIXTypeSymbolicLink ) {
	if( SymbolicLinkClass->isASubclassOf( aClass ) ) {
		supportedClass = SymbolicLinkClass;
	}
    }
    else if( _inodeData->_di_spare3 == 2 ) {
        if( PersistentArrayClass->isASubclassOf( aClass ) ) {
                supportedClass = PersistentArrayClass;
        }
#ifdef ArFile
        else {
		extern ClassStar ArContainerClass;
		if( ArContainerClass->isASubclassOf( aClass ) ) {
			supportedClass = ArContainerClass;
		}
        }
#endif
    }
    else if( _inodeData->_di_spare3 == 3 ) {
        if( PersistentArrayClass->isASubclassOf( aClass ) ) {
                supportedClass = PersistentArrayClass;
        }
#ifdef MailFile
        else {
	extern ClassStar MailContainerClass;
		if( MailContainerClass->isASubclassOf( aClass ) ) {
			supportedClass = MailContainerClass;
		}
        }
#endif
    }
    else if( _inodeData->_di_spare3 == 4 ) {
        if( PersistentArrayClass->isASubclassOf( aClass ) ) {
                supportedClass = PersistentArrayClass;
        }
#ifdef TarFile
        else {
		extern ClassStar TarContainerClass;
		if( TarContainerClass->isASubclassOf( aClass ) ) {
			supportedClass = TarContainerClass;
	     }
        }
#endif
    }
    else if( _inodeData->_di_spare3 == 5 ) {
        if( PersistentArrayClass->isASubclassOf( aClass ) ) {
                supportedClass = PersistentArrayClass;
        }
#ifdef COFFFile
        else {
	  Console()<<"Got here!\n";
		extern ClassStar COFFContainerClass;
		if( COFFContainerClass->isASubclassOf( aClass ) ) {
			supportedClass = COFFContainerClass;
		}
        }
#endif
    }    
    else if( _inodeData->_di_spare3 == 6 ) {
        if( PersistentArrayClass->isASubclassOf( aClass ) ) {
                supportedClass = PersistentArrayClass;
        }
#ifdef ELFFile
        else {
	  Console()<<"Got here (ELF)!\n";
		extern ClassStar ELFContainerClass;
		if( ELFContainerClass->isASubclassOf( aClass ) ) {
			supportedClass = ELFContainerClass;
		}
        }
#endif

    }
    Console()<<"_di_spare3="<<_inodeData->_di_spare3<<"\n";
    Raid( RaidProtected )( "%N::supports() returns %N %d\n",
			   this, supportedClass );

    //HACK BY SCOTT KIRCHER
    //cpon doesn't know the types of objects, so it doesn't set
    //the _di_spare3 hint flag, so we need a seperate check for
    //COFF files
#ifdef COFFFile
		extern ClassStar COFFContainerClass;
		if( COFFContainerClass->isASubclassOf( aClass ) ) {
			Console()<<"Recognized an COFFContainer\n";
			supportedClass = COFFContainerClass;
		}
#endif

    //HACK BY SCOTT KIRCHER
    //cpon doesn't know the types of objects, so it doesn't set
    //the _di_spare3 hint flag, so we need a seperate check for
    //ELF files
#ifdef ELFFile
		extern ClassStar ELFContainerClass;
		if( ELFContainerClass->isASubclassOf( aClass ) ) {
			Console()<<"Recognized an ELFContainer\n";
			supportedClass = ELFContainerClass;
		}
#endif


    return( supportedClass );
}

void
BSDInode::hintsChanged( unsigned oldHints )
{
    getDinode();
    if ( _hints != (unsigned)_diskInode->di_spare[2] ) {
        _diskInode->di_spare[2] = _hints;
        setModified();
    }
    releaseDinode();
}

int
BSDInode::basicRead( unsigned startUnit, int count, char * addr,
                     unsigned nbytes, IOAsyncClosure * c )
{
    getDinode();
    _diskInode->di_atime = _theTime.sec();
    releaseDinode();
    return UNIXInode::basicRead( startUnit, count, addr, nbytes, c );
}

int 
BSDInode::rawRead( unsigned startUnit, unsigned count, char * addr, unsigned nbytes,
		   IOAsyncClosure * c )
{
    Raids( RaidProtected ) 
	<< this << "::rawRead( " << startUnit << ", " << count 
	<< ", " << asAddress( addr ) << ", " << nbytes 
	<< ", " << asAddress( c ) << " )\n" << eor;

Debug<< this << "::rawRead( " << startUnit << ", " << count 
	<< ", " << asAddress( addr ) << ", " << nbytes 
	<< ", " << asAddress( c ) << " )\n" << eor;
   
    _mutex.P();

    if( ( ( _inodeData->_di_mode & UNIXTypeMask ) == UNIXTypeSymbolicLink ) &&
	  ( _inodeData->_di_flags & IC_FASTLINK ) ) {

        getDinode();

	Assert( numberOfUnits() <= 1 );
	if( startUnit >= 1 ) return( 0 );

	unsigned bytes = numberOfBytes();
	Assert( bytes < unitSize() );
	Assert( bytes <= nbytes );

	ByteCopy( (char *) _diskInode->di_db, addr, bytes );
	
	releaseDinode();

	_mutex.V();

	if ( c ) {
		c->activate( 1 );
	}		
	return 1;
    }
    else {
	_mutex.V();
Debug<<"Calling UNIXInode::rawRead\n";
        return UNIXInode::rawRead( startUnit, count, addr, nbytes, c );
    }
}

int
BSDInode::basicWrite( unsigned startUnit, unsigned count, char * addr,
                      unsigned nbytes, IOAsyncClosure * c )
{
    getDinode();
    _diskInode->di_mtime = _theTime.sec();
    releaseDinode();
    return UNIXInode::basicWrite( startUnit, count, addr, nbytes, c );
}

int
BSDInode::basicWriteThrough( unsigned startUnit, unsigned count, char * addr,
                             unsigned nbytes, IOAsyncClosure * c )
{
    getDinode();
    _diskInode->di_mtime = _theTime.sec();
    releaseDinode();
    return UNIXInode::basicWriteThrough( startUnit, count, addr, nbytes, c );
}

int 
BSDInode::rawWrite( unsigned startUnit, unsigned count, char * src, unsigned nbytes,
		    int writeThrough, IOAsyncClosure * c )
{
    Raids( RaidProtected ) 
	<< this << "::rawWrite( " << startUnit << ", " << count 
	<< ", " << asAddress( src ) << ", " << nbytes 
        << ", writeThrough " << writeThrough 
	<< ", " << asAddress( c ) << " )\n" << eor;

    /*
     * Dont perpetuate fast links.
     * I.e. writes to fast links should reset the fast link bit and
     * clear the direct and indirect block pointers,
     * since the fast link data were stored there. 
     * FIX: perpetuate fast links
     */
    _mutex.P();
    if( ( ( _inodeData->_di_mode & UNIXTypeMask ) == UNIXTypeSymbolicLink ) &&
	  ( _inodeData->_di_flags & IC_FASTLINK    ) ) {
	
	getDinode();

	_inodeData->_di_flags = ( _diskInode->di_flags &= ~IC_FASTLINK );

	// FIX: these assertions are restrictive
	Assert( startUnit == 0 );
	Assert( count == 1 );

	int i;
	for( i = 0; i < NDADDR; i++ ) _diskInode->di_db[i] = 0;
	for(     i = 0; i < NIADDR; i++ ) _diskInode->di_ib[i] = 0;
	setModified();

	releaseDinode();
    }
    _mutex.V();

    return( UNIXInode::rawWrite( startUnit, count, src, nbytes, writeThrough, c ) );
}

unsigned
BSDInode::numberOfBytes()
{
    return _inodeData->_di_size;
}

unsigned
BSDInode::basicSetNumberOfBytes(unsigned size )
{
    Raid( RaidProtected )( "%N::basicSetNumberOfBytes( %d ) %d\n",
			   this, size, numberOfBytes() );

    if( size < 0 ) size = 0;
    if( size == _inodeData->_di_size ) return size;
    getDinode();
    _diskInode->di_mtime = _theTime.sec();
    union {
	int	i;
	char	c[4];
    } littleEndian;
    littleEndian.i = 1;
    _diskInode->di_size[!littleEndian.c[0]] = size;
    _inodeData->_di_size = size;
    unsigned units;

Debug<<"size = "<<size<<"\n"<<eor;

    if( size == 0 ) 
	units = 0;
    else
	units = ( size + unitSize() - 1 ) >> log2UnitSize();
    if( ( units >> log2BlockSize ) >= singleIndirect ) {
        int blocks = ( units + blockSize() - 1 ) >> log2BlockSize;
        units = blocks << log2BlockSize;
    }
Debug<<"units = "<<units<<"\n"<<eor;
    if ( numberOfUnits() != units ) {
        unsigned  result = basicSetNumberOfUnits( units );
Debug<<"result = "<<result<<"\n"<<eor;
        Assert( result == units );
    }
    setModified();
    releaseDinode();
    return( size );
}

unsigned
BSDInode::basicSetNumberOfUnits( unsigned units )
{
    Raid( RaidProtected )
	    ( "%N::basicSetNumberOfUnits(BSD)( units %d ) was %d\n",
	      this, units, numberOfUnits() );
    unsigned numUnits = numberOfUnits();
    
    if( units == numUnits ) return( numUnits );

    getDinode();

#ifdef NEVER
    if( units == 0 ) {
	_diskInode->di_version++;
	_version = _diskInode->di_version;
	setModified();
    }
#endif

    if( _diskInode->di_flags & IC_FASTLINK ) {
	Assert( units == 0 );
	Assert( numUnits == 1 );

	int i;
	for( i = 0; i < NDADDR; i++ ) _diskInode->di_db[i] = 0;
	for(     i = 0; i < NIADDR; i++ ) _diskInode->di_ib[i] = 0;
	setModified();
    }

    // If the last block is fragmented, extend it out to a full block.
    // Unless it will soon be freed; if so, just free it now.
    if( fragmentsInLastBlock > 0 ) {

	int logicalBlock = lastBlockNo();
	int endBlock = ( units + blockSize() - 1 ) >> log2BlockSize;
	int physicalBlock = direct[ logicalBlock ];

        Assert( physicalBlock != 0 );

        if( endBlock <= logicalBlock ) {
            _container->free( _diskInode->di_blocks, physicalBlock, 
			      fragmentsInLastBlock );
	    direct[ logicalBlock ] = 0;
	    numUnits -= fragmentsInLastBlock;
	    MemoryObject::basicSetNumberOfUnits( numUnits );
        } else if ( ! ((BSDContainer *) _container)->allocator()->
		      extendFragmentsToBlock( _diskInode->di_blocks,
			  	  	      physicalBlock, 
					      fragmentsInLastBlock ) ) {
	    Raids( RaidProtected ) 
		<< this << "::basicSetNumberOfUnits() extend failed\n"
		<< eor;
            char * buffer = new char [unitSize() << log2BlockSize];

            ByteZero( buffer, unitSize() << log2BlockSize );
            fileObjectRead( physicalBlock, fragmentsInLastBlock,
			    buffer, fragmentsInLastBlock << log2UnitSize(), 0 );
            _container->free( _diskInode->di_blocks, physicalBlock, 
			      fragmentsInLastBlock );
            // FIX: check physicalBlock
	    int newPhysicalBlock = 0;
            allocBlocks( &newPhysicalBlock, 0, 1, 
			 physicalBlock & ~(blockMask()) );
	    Assert( newPhysicalBlock != 0 );
	    direct[ logicalBlock ] = newPhysicalBlock;
            fileObjectWrite( newPhysicalBlock, blockSize(),
			     buffer, unitSize() << log2BlockSize, 0, 0 );
            delete buffer;
        }
	setModified();
        fragmentsInLastBlock = 0;
    }

    releaseDinode();

    int result = UNIXInode::basicSetNumberOfUnits( units );

    return result;
}

/*
 *	this function allows other objects to access and modify the
 *	link count ( or reference count ) of this inode.
 *	links returns the new number of links for this inode.
 */
int
BSDInode::links( int delta )
{
    Raid( RaidMethod )( "%N::links( delta %d )\n", this, delta );
    int result;
    if ( delta == 0 ) {
	result = _inodeData->_di_nlink;
    } else {
    	_mutex.P();
    	getDinode();
	Assert( _diskInode->di_nlink == _inodeData->_di_nlink );
	_inodeData->_di_nlink += delta;
    	result = _diskInode->di_nlink = _inodeData->_di_nlink;
        _diskInode->di_ctime = _theTime.sec();
	setModified();
	releaseDinode();
        _mutex.V();
	if ( delta > 0 && _creatingDictionary ) {
	    synchronizeCreatingDictionary();
	}
    }
    Raid( RaidMethod )( "%N::links = %d\n", this, result );
    return( result );
}

int
BSDInode::setMode( int mode )
{
    Raid( RaidMethod )( "%N::setMode( mode %o ) %o\n",
		this, mode, _inodeData->_di_mode );

    short new_mode = ( _inodeData->_di_mode & ~07777 ) | ( mode & 07777 );
    if ( new_mode == _inodeData->_di_mode ) 
	return 0;
    _mutex.P();
    getDinode();
    _inodeData->_di_mode = new_mode;
    _diskInode->di_mode = new_mode;
    _diskInode->di_ctime = _theTime.sec();
    setModified();
    releaseDinode();
    _mutex.V();
    return( 0 );
}

int
BSDInode::setUserId( int uid )
{
    getDinode();
    Raid( RaidMethod )( "%N::setUserId( uid %d ) %d\n",
		this, uid, _diskInode->di_uid );
    _diskInode->di_uid = uid;
    _diskInode->di_ctime = _theTime.sec();
    setModified();
    releaseDinode();
    return( 0 );
}

int
BSDInode::setGroupId( int gid )
{
    getDinode();
    Raid( RaidMethod )( "%N::setGroupId( gid %d ) %d\n",
		this, gid, _diskInode->di_gid );
    _diskInode->di_gid = gid;
    _diskInode->di_ctime = _theTime.sec();
    setModified();
    releaseDinode();
    return( 0 );
}

void
BSDInode::info( UNIXFileInfo * infoBuffer )
{
    Raid( RaidMethod )( "%N::info( %x ) %d\n", this, infoBuffer, _idNumber );
    getDinode();
    infoBuffer->device	= 0;
    infoBuffer->inumber	= _idNumber;
    infoBuffer->mode	= _diskInode->di_mode;
    infoBuffer->links	= _inodeData->_di_nlink;
    infoBuffer->uid	= _diskInode->di_uid;
    infoBuffer->gid	= _diskInode->di_gid;
    infoBuffer->size	= _inodeData->_di_size;
    infoBuffer->atime	= _diskInode->di_atime;
    infoBuffer->mtime	= _diskInode->di_mtime;
    infoBuffer->ctime	= _diskInode->di_ctime;
    infoBuffer->blocks	= _diskInode->di_blocks;
    infoBuffer->blockSize = blockSize() << _log2UnitSize;
    infoBuffer->flags	= _inodeData->_di_flags;
    releaseDinode();
}

void
BSDInode::getAttributes( RFSAttributes * attr )
{
    getDinode();
    attr->mode = _diskInode->di_mode;
    attr->links = _inodeData->_di_nlink;
    attr->uid = _diskInode->di_uid;
    attr->gid = _diskInode->di_gid;
    attr->size = _inodeData->_di_size;
    attr->atime = _diskInode->di_atime;
    attr->mtime = _diskInode->di_mtime;
    attr->ctime = _diskInode->di_ctime;
    attr->type.set( this );
    attr->log2BlockSize = log2BlockSize;
    attr->log2UnitSize = _log2UnitSize;
    attr->flags = RFSAttributes::AttrDirty;
    attr->cacheHints = _hints;
    releaseDinode();
}

void
BSDInode::setAttributes( RFSAttributes * attr )
{
    if ( !(attr->flags & RFSAttributes::AttrDirty) )
	return;
    getDinode();
    if ( !isDictionary() && _inodeData->_di_size != (unsigned)attr->size )
        setNumberOfBytes( attr->size );
    if ( _diskInode->di_uid != attr->uid ) {
        _diskInode->di_uid = attr->uid;
        _diskInode->di_ctime = _theTime.sec();
        setModified();
    }
    if ( _diskInode->di_gid != attr->gid ) {
        _diskInode->di_gid = attr->gid;
        _diskInode->di_ctime = _theTime.sec();
        setModified();
    }
    if ( (_diskInode->di_mode & UNIXAccessMask) !=
         (attr->mode & UNIXAccessMask) ) {
        short new_mode = ( _inodeData->_di_mode & ~07777 ) | 
			 ( attr->mode & 07777 );
        _inodeData->_di_mode = new_mode;
        _diskInode->di_mode = new_mode;
        _diskInode->di_ctime = _theTime.sec();
        setModified();
    }
    releaseDinode();
    unsigned cacheHints = attr->cacheHints;
    if ( (_hints & 0xffff0000) != (cacheHints & 0xffff0000) ) {
	_hintLock.acquire();
	unsigned oldHints = _hints;
	_hints = (_hints & 0x0000ffff) | (cacheHints & 0xffff0000);
	_hintLock.release();
	hintsChanged( oldHints );
    }
}

int
BSDInode::allocBlock()
{
    Assert( NOTREACHED );
    return 0;
}

void
BSDInode::freeBlock( int physicalBlock )
{
    Raid( RaidDetail )( "%N::freeBlock( physicalBlock %d ) id %d\n",
		        this, physicalBlock, idNumber() );
    _container->free( _diskInode->di_blocks, physicalBlock, blockSize() );
    setModified();
}

void
BSDInode::freeFragments( int physicalBlock, int count )
{
    Raid( RaidDetail )
	( "%N::freeFragments( physicalBlock %d, count %d ) id %d\n",
	  this, physicalBlock, count, idNumber() );
    _container->free( _diskInode->di_blocks, physicalBlock, count );
    setModified();
}

int
BSDInode::allocBlocks( int * buf, int start, int count, int pref )
{
#ifdef FANCY
    return ((BSDContainer *) _container)->allocator()->allocateBlocks
	( _diskInode->di_blocks, _idNumber, buf, start, count, pref );
#else
    int result = ((BSDContainer *) _container)->allocator()->allocateBlocks
	( _diskInode->di_blocks, _idNumber, buf, start, count, pref );
    if ( result > 0 )
	nearBlock = buf[ result - 1 ];
    return result;
#endif
}

int
BSDInode::blockPref( int * buf, int start, int startBlock )
{
#ifdef FANCY
    return ((BSDContainer *) _container)->blockPref
	( _idNumber, buf, start, startBlock );
#else
    if( nearBlock == 0 ) {
	nearBlock = ((BSDContainer *) _container)->blockPref
		    ( _idNumber, buf, start, startBlock );
	return nearBlock;
    }
    return nearBlock + blockSize();
#endif
}

void 
BSDInode::updateInode( int finalSync )
{
    Raids( RaidProtected ) 
	<< this << "::updateInode( " << finalSync << " )\n" << eor;
    getDinode();
    unsigned units;
    Assert( _inodeData->_di_nlink == _diskInode->di_nlink );
    if ( finalSync && _inodeData->_di_nlink == 0 ) {
	units = 0;
    } else {
        int bytes = numberOfBytes();
	if ( bytes == 0 ) {
	    units = 0;
	} else {
	    units = ( bytes + unitSize() - 1 ) >> log2UnitSize();
    	    if( ( units >> log2BlockSize ) >= singleIndirect ) {
       	        int blocks = ( units + blockSize() - 1 ) >> log2BlockSize;
                units = blocks << log2BlockSize;
	    }
	}
    }
  
    if ( units != numberOfUnits() ) 
	basicSetNumberOfUnits( units );

    /*	If the file satisfies the following three conditions:
     *		1 ) Its last block is not yet fragmented
     *		2 ) It is not an abominable fast link
     *		3 ) It is not an empty file
     *	then the last block must be checked to see if it needs to be fragmented
     */
    if( ( fragmentsInLastBlock == 0 ) &&
         ( !( _diskInode->di_flags & IC_FASTLINK ) ) &&
	 ( ( fragmentsInLastBlock = numFragments() ) > 0 ) ) {

	// Calculate last logical and physical block in file.
        int logicalBlock = lastBlockNo();
        int physicalBlock = direct[ logicalBlock ];

	// Only existing physical blocks can ever need fragmentation
        if( physicalBlock != 0 ) {

    	    Raids( RaidDetail ) 
		<< this << "::updateInode() fragmentsInLastBlock " 
		<< fragmentsInLastBlock << " )\n" << eor;

#ifdef SLOW
            char * buffer = new char [unitSize() << log2BlockSize];

	    // NOTE: these operations vvvv are assumed to be atomic 
            int status = fileObjectRead( physicalBlock, blockSize(), 
					 buffer, unitSize() << log2BlockSize,
					 0 );
	    Assert( status == blockSize() );
            freeBlock( physicalBlock );

            physicalBlock = _container->allocate( _diskInode->di_blocks,
					  physicalBlock, fragmentsInLastBlock );
	    Assert( physicalBlock != 0 );

            direct[ logicalBlock ] = physicalBlock;
            status = fileObjectWrite( physicalBlock, fragmentsInLastBlock, 
				      buffer, 
				      fragmentsInLastBlock << log2UnitSize(), 
				      1, 0 );
	    Assert( status == fragmentsInLastBlock );
	    // NOTE: these operations ^^^^ are assumed to be atomic 

            delete buffer;
    	    setModified();
#else /* not SLOW */
	    // this is a lot faster but may excessively fragment disk.
 	    _container->free( _diskInode->di_blocks,
			      physicalBlock + fragmentsInLastBlock,
			      blockSize() - fragmentsInLastBlock );
	    setModified();
#endif /* SLOW */
        }
    }
    releaseDinode();
}

void
BSDInode::synchronizeInode()
{
    if ( _diskInode != 0 ) {
	UNIXInode::synchronizeInode();
        updateInode( 0 );
    }
}

int
BSDInode::accessMode()
{
    return( _inodeData->_di_mode & UNIXAccessMask );
}

int
BSDInode::userId()
{
    getDinode();
    int result = _diskInode->di_uid;
    releaseDinode();
    return result;
}

int
BSDInode::groupId()
{
    getDinode();
    int result = _diskInode->di_gid;
    releaseDinode();
    return result;
}

int
BSDInode::isSymbolicLink()
{
    return( (_inodeData->_di_mode & UNIXTypeMask) == UNIXTypeSymbolicLink );
}

int
BSDInode::isDictionary()
{
    return( (_inodeData->_di_mode & UNIXTypeMask) == UNIXTypeDirectory );
}


int
BSDInode::isContainer()
{
// FIX: for now ok.
//IMPORTANT CODE CHANGE BY SCOTT KIRCHER
//Need to be able to identify files that are recursive filesystems
//This should be more selective somehow. It used to always return 0
//Now I made it always return 1, but neither is really correct
    return( 1 ); 
}

#include "ClassInstaller.h"
DEFINECLASS("BSDInode",BSDInode,UNIXInode,0,0);
