#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * BlockAllocator.c - 
 */
#include "Assert.h"
#include "Raid.h"
#include "BlockAllocator.h"
#include "Semaphore.h"

#ifdef __GNUC__
#pragma implementation
#endif

BlockAllocator::BlockAllocator( MemoryObject * source )
{
	initDebug();
	Raid( RaidConstructor )( "%N::BlockAllocator( %N )\n", this, _source );

	_source = source;
	_mutex = new Semaphore( 0 );
	_mutex->V();
}

BlockAllocator::~BlockAllocator()
{
	Raid( RaidConstructor )( "%N::~BlockAllocator()\n", this );
        //delete _mutex;
}

int
BlockAllocator::basicAllocate( int &, int, int )
{
	Assert( NOTREACHED );
	return( 0 );
}

void
BlockAllocator::basicFree( int &, int, int )
{
	Assert( NOTREACHED );
}

int
BlockAllocator::allocate( int & blocks, int nearPhysicalUnit, int units )
{
	Raid( RaidMethod )( "%N::basicAllocate( %d, %d, %d )\n",
			    this, blocks, nearPhysicalUnit, units );

	_mutex->P();
	int block = basicAllocate( blocks, nearPhysicalUnit, units );
	_mutex->V();

	if ( block == 0 ) {
		Console() 
			<< "=== WARNING ===\n"
			<< asAddress(this) << "::allocate() " << asAddress(_source) 
			<< " is full, allocate failed\n" << eor;
	}
	Raid( RaidMethod )( "%N::basicAllocate returns %d\n", this, block );
	return( block );
}

void
BlockAllocator::free( int & blocks, int physicalUnit, int units )
{
	Raid( RaidMethod )( "%N::basicFree( %d, %d, %d )\n",
			    this, blocks, physicalUnit, units );

	_mutex->P();
	basicFree( blocks, physicalUnit, units );
	_mutex->V();
}



#include "ClassInstaller.h"
DEFINECLASS("BlockAllocator",BlockAllocator,ProxiableObject,0,0);
