#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * COFFDictionary.cc - COFF Dictionary class
 */

#include "Raid.h"
#include "COFF.h"
#include "COFFDictionary.h"
#include "FileSystem.h"
#include "String.h"
#include "PersistentMemoryObject.h"

#ifdef __GNUC__
#pragma implementation
#endif

int
COFFDictionary::findKey( char * key, int )
{
	Raid( RaidProtected )( "%N::findKey( %s, )\n", this, key );

	_idNumberOfKey = 0;
	_emptyOffset = -1;
	_offset = -1;

	if( ( strcmp( key, "." ) == 0 ) || ( strcmp( key, ".." ) == 0 ) ) {
		_idNumberOfKey = RootId;
	}
	else if( strcmp( key, "Symbol.Table" ) == 0 ) {
		_idNumberOfKey = SymbolTableId;
	}
	else if( strcmp( key, "String.Table" ) == 0 ) {
		_idNumberOfKey = StringTableId;
	}
	else if( strcmp( key, "File.Header" ) == 0 ) {
		_idNumberOfKey = FileHeaderId;
	}
	else if( strcmp( key, "A.Out.Header" ) == 0 ) {
		_idNumberOfKey = AOutHeaderId;
	}
	else for( int i = 0; i < _filehdr->f_nscns; i++ ) {
		if( strcmp( key, _scnhdrs[i].s_name ) == 0 ) {
			_idNumberOfKey = SectionIds + i * 4 + Section;
			break;
		}
		int nameLength = strlen( _scnhdrs[i].s_name );
		if( strncmp( key, _scnhdrs[i].s_name, nameLength ) == 0 ) {
			if( strcmp( &key[nameLength], ".Header" ) == 0 ) {
				_idNumberOfKey = SectionIds + i * 4 + Header;
				break;
			}
			else if( strcmp( &key[nameLength], ".Reloc" ) == 0 ) {
				_idNumberOfKey = SectionIds + i * 4 + Reloc;
				break;
			}
			else if( strcmp( &key[nameLength], ".Lineno" ) == 0 ) {
				_idNumberOfKey = SectionIds + i * 4 + LineNo;
				break;
			}
		}
	}

	Raid( RaidProtected )( "%N::findKey returns %d\n",
				this, _idNumberOfKey  );
	return( _idNumberOfKey != 0 );
}

int
COFFDictionary::basicKeys( char * keyBuffer, int bufferSize, int & count )
{
    Raid( RaidMethod )( "%N::basicKeys( buffer %x, size %d ) _size %d\n", 
			this, keyBuffer, bufferSize, _size );

    strcpy( keyBuffer, ".\n..\nFile.Header\n" );
    count = 4;
    if( _filehdr->f_opthdr != 0 ) {
	strcat( keyBuffer, "A.Out.Header\n" );
	count++;
    }
    if( _filehdr->f_nsyms != 0 ) {
	strcat( keyBuffer, "Symbol.Table\n" );
	count++;
	strcat( keyBuffer, "String.Table\n" );
	count++;
    }
    char * t = &keyBuffer[strlen( keyBuffer )];

    for( int i = 0; i < _filehdr->f_nscns; i++ ) {
	char * s = _scnhdrs[i].s_name;
	while( *s != '\0' ) *t++ = *s++;
	s = ".Header";
	while( *s != '\0' ) *t++ = *s++;
	*t++ = '\n';
	count++;

	s = _scnhdrs[i].s_name;
	while( *s != '\0' ) *t++ = *s++;
	*t++ = '\n';
	count++;

	if( _scnhdrs[i].s_nreloc > 0 ) {
		s = _scnhdrs[i].s_name;
		while( *s != '\0' ) *t++ = *s++;
		s = ".Reloc";
		while( *s != '\0' ) *t++ = *s++;
		*t++ = '\n';
		count++;
	}

	if( _scnhdrs[i].s_nlnno > 0 ) {
		s = _scnhdrs[i].s_name;
		while( *s != '\0' ) *t++ = *s++;
		s = ".Lineno";
		while( *s != '\0' ) *t++ = *s++;
		*t++ = '\n';
		count++;
	}
    }
    *t++ = '\0';

    return( 0 );
}

COFFDictionary::~COFFDictionary()
{
	Raid( RaidConstructor ) ( "%N::~COFFDictionary()\n", this );
}

COFFDictionary::COFFDictionary( PersistentMemoryObject * source )
: MemoryObjectDictionary( source )
{
	initDebug();
	Raid( RaidConstructor ) ( "%N::COFFDictionary( %N )\n", this, source );

	_maxKeyLength = 32;

	_filehdr = (filehdr *)_buffer;

	_scnhdrs = (scnhdr *)&_buffer[sizeof( filehdr ) + _filehdr->f_opthdr];
}

ProxiableObject * 
COFFDictionaryConstructor( ProxiableObject * source )
{
	if( source->isKindOf( PersistentMemoryObjectClass ) ) {
		return( new COFFDictionary( (PersistentMemoryObject *)source ) );
	}
	else return( 0 );
}



#include "ClassInstaller.h"
DEFINEPERSISTENTCLASS("COFFDictionary",COFFDictionary,MemoryObjectDictionary,&COFFDictionaryConstructor,0);
