#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

//COMMENTS/DOCUMENTATION ADDED BY SCOTT KIRCHER
//LINUX MODULE ADAPTOR FOR CHOICES PROJECT, CS423
//All such comment blocks are ended by: -SK
//Anything I didn't understand will have a question above it of the form ??Why is this here??
//-SK

#include "Assert.h"
#include "Raid.h"
#include "COFFLoader.h"
#include "NameServer.h"
#include "PersistentMemoryObject.h"
#include "MemoryObjectDictionary.h"
#include "OutputStream.h"
#include "String.h"
#include "Byte.h"
#include "FileSystem.h"
#include "FileSystemInterface.h"
#include "RecordStream.h"
#include "Class.h"
#include "ClassStar.h"
#include "Convert.h"
#include "Ctype.h"
#include "COFF.h"
#include "COFFDictionary.h"
#include "ClassConstructor.h"

#ifdef __GNUC__
#pragma implementation
#endif

//Function pointer type (void->int) -SK
typedef int (* PFI)();


//swapBytes
//Reverses the endianness of a 32-bit word
//(swaps the first and last bytes, swaps the middle two bytes)
// -SK
void
swapBytes( int * location )
{
	char * cp = (char *)location;
	cp[0] ^= cp[3];
	cp[3] ^= cp[0];
	cp[0] ^= cp[3];
	cp[1] ^= cp[2];
	cp[2] ^= cp[1];
	cp[1] ^= cp[2];
}

//COFFLoader C++ constructor
//Takes a class name of a class to be loaded
//class must reside in a COFF file named "/Code/"+clsName+".o"
// -SK
COFFLoader::COFFLoader( char * clsName )
	//Initialize the file-system interface by querying the name server for the current
	//fileSystemInterface. -SK
: _fsi( StandardNameServer->lookup( "fileSystemInterface",
				FileSystemInterfaceClass ) ), _root( 0 )
{
	initDebug();

	Raid( RaidConstructor )( "%N::COFFLoader( %s )\n", this, clsName );

	//Initialize pointers -SK
	_text = 0;
	_data = 0;
	_textRelocs = 0;
	_dataRelocs = 0;
	_mainSymbols = 0;
	_mainStrings = 0;
	_loadSymbols = 0;
	_loadStrings = 0;

	//Construct file name from class name - SK
	//??Why does i start from 1 instead of 0?? - SK
	//??Why are only lower case letters allowed?? - SK
        int i;
	for( i = 1; ( clsName[i] >= 'a' ) && ( clsName[i] <= 'z' ); i++ );  
	
        strcpy( _loadObjectName, "/Code/" );
	strncpy( &_loadObjectName[6], clsName, i );
	strcpy( &_loadObjectName[i+6], ".o" );

	//We're going to be treating the COFF file as a file system itself
	//So, we can access various parts of it by appending /SectionID to the
	//file name. We're doing that here for the symbol table:
	//-SK
	strcpy( _loadSymbolTableName, _loadObjectName );
	strcat( _loadSymbolTableName, "/Symbol.Table" );

	//And also for the string table -SK
	strcpy( _loadStringTableName, _loadObjectName );
	strcat( _loadStringTableName, "/String.Table" );

	//And also for the .text section of the COFF -SK
	strcpy( _loadTextName, _loadObjectName );
	strcat( _loadTextName, "/.text" );

	//This is for the Reloc subsection of the text section -SK
	strcpy( _loadTextRelocName, _loadTextName );
	strcat( _loadTextRelocName, ".Reloc" );

	//This is for the .data section of the COFF -SK
	strcpy( _loadDataName, _loadObjectName );
	strcat( _loadDataName, "/.data" );

	//This is for the Reloc subsection of the data section -SK
	strcpy( _loadDataRelocName, _loadDataName );
	strcat( _loadDataRelocName, ".Reloc" );

	Raid( RaidDetail )( "%N::COFFLoader Looking for new code in %s\n",
			    this, _loadObjectName );

	Assert( _fsi != 0 );

	int status = 0;
	//Open the COFF file -SK

	//HACK BY SCOTT KIRCHER
	//Changed MemoryObjectDictionaryClass to COFFDictionaryClass
	//to force handling of file as a COFF file
 	_root = _fsi->open( _loadObjectName, COFFDictionaryClass, status );
        if(status!=Success) Console()<<StatusCodes[status]<<"\n";
	Console()<<_root<<"\n";

	//Open the symbol table (part of the COFF file) -SK
	RecordStreamStar loadSymbolTable = _fsi->open( _loadSymbolTableName,
					RecordStreamClass, status );

	//Open the string table (part of the COFF file) -SK
	RecordStreamStar loadStringTable = _fsi->open( _loadStringTableName,
					RecordStreamClass, status );

	//Open the main symbol table. Symbols exported by the OS. -SK
	RecordStreamStar mainSymbolTable =
					_fsi->open( (char *)MainSymbolTableName,
					RecordStreamClass, status );
	//Open the main string table. Strings (long symbols) exported by the OS. -SK
	RecordStreamStar mainStringTable =
					_fsi->open( (char *)MainStringTableName,
					RecordStreamClass, status );

	//Note that, we haven't opened the .text or .data sections. This is accomplished
	//in the readSection function instead. - SK

	//Make sure we were able to open everything -SK
	if( ( _root == 0 ) ||
	    ( loadSymbolTable == 0 ) ||
	    ( loadStringTable == 0 ) ||
	    ( mainSymbolTable == 0 ) ||
	    ( mainStringTable == 0 ) ) {
		Raid( RaidDetail )( "%N::COFFLoader: Opens failed.\n", this );
                Console()<<"Unable to open some necessary file\n";
                Console()<<"_root="<<_root<<"\n";
		Console()<<"loadSymbolTable="<<loadSymbolTable<<"\n";
		Console()<<"loadStringTable="<<loadStringTable<<"\n";
		Console()<<"mainStringTable="<<mainStringTable<<"\n";
		Console()<<"mainSymbolTable="<<mainSymbolTable<<"\n";
		return;
	}

	Raid( RaidDetail )( "%N::COFFLoader: Opens succeeded.\n", this );

	//read the text section into a buffer (_text) -SK
	status = readSection( _loadTextName, _text, _textSize );
	Assert( status == 0 );
	//read the data section into a buffer (_data) -SK
	status = readSection( _loadDataName, _data, _dataSize );
	Assert( status == 0 );

	//read the text relocation section into a buffer -SK
	status = readSection( _loadTextRelocName, _textRelocs, _textRelocCount);
	_textRelocCount /= RELOCSIZE;
	Assert( status == 0 );
	//read the data relocation section into a buffer -SK
	status = readSection( _loadDataRelocName, _dataRelocs, _dataRelocCount);
	_dataRelocCount /= RELOCSIZE;
	Assert( status == 0 );

	//read the main symbol and string tables (OS exported symbols) -SK
	status = readSymbols( mainSymbolTable, _mainSymbols, _mainSymbolCount,
			      mainStringTable, _mainStrings );
	Assert( status == 0 );
	//read the COFF file's symbol and string tables -SK
	status = readSymbols( loadSymbolTable, _loadSymbols, _loadSymbolCount,
			      loadStringTable, _loadStrings );
	Assert( status == 0 );

	//Record the original address of the .text (executable code) section -SK
	Assert( strcmp( _loadSymbols[2]._n._n_name, ".text" ) == 0 );
	_textPrelocated = _loadSymbols[2].n_value;
	
	//Funky pointer stuff! We're finding the location of the .text section in "real" memory here,
	//and subtracting the file-offset address, to get the amount we need to offset all references into
	//the .text section by, so that they will reference "real" memory. -SK
	_textRelocated = (int)_text - _textPrelocated; 

	//Record the original address of the .data (initialized data) section -SK
	Assert( strcmp( _loadSymbols[4]._n._n_name, ".data" ) == 0 );
	_dataPrelocated = _loadSymbols[4].n_value;
	//Same as above for .text, except this time it's for .data -SK
	_dataRelocated = (int)_data - _dataPrelocated;

	//resolve undefined symbols (get external reference symbol's addresses from main symbol table) -SK
	status = resolveUndefined();
	Assert( status == 0 );
	status = relocateSymbols();
	Assert( status == 0 );
	status = resolveDefined();
	Assert( status == 0 );
	Console()<<"COFFLoader completed all operations\n";
}

//COFFLoader destructor
//release non reference counted dynamic memory
// -SK
COFFLoader::~COFFLoader()
{
	Raid( RaidConstructor )( "%N::~COFFLoader()\n", this );

	//??Why aren't there checks to make sure these things were really allocated?? -SK
	//??Is it because we require (Assert) that the object creation succeeded?? -SK

	//Note that we do NOT delete _text and _data. So, the loaded class is still available
	//even after the COFFLoader class that loaded it dies. -SK

	delete _textRelocs;
	delete _dataRelocs;
	delete _mainSymbols;
	delete _mainStrings;
	delete _loadSymbols;
	delete _loadStrings;
}

//readSection
//Reads a section of the COFF file, as specified by the path argument
//Since we treat the COFF file as another filesystem, we can access
//its sections by using something like "filename/Section" -SK
int
COFFLoader::readSection( char * path, char * & buffer, int & size )
{
	Raid( RaidProtected )( "%N::readSection( %s, )\n", this, path );

	int status = 0;

	//Open the section as a record stream (basically will just read in all the bytes of the section in order) -SK
	RecordStreamStar rfs = _fsi->open( path, RecordStreamClass, status );

	if( rfs == 0 ) {
		Assert( status != 0 );
		return( status );
	}

	//Actually read the section (note each section behaves like its own file, and knows how long it is) -SK
	//(Ah the wonders of OOP. This is what Professor Campbell would call "kinda cool") -SK
	size = rfs->numberOfBytes();
	if( size != 0 ) {
		buffer = new char[size];
		rfs->read( buffer, size );
	}
	else buffer = 0;

	Raid( RaidProtected )( "%N::readSection loaded %s @ %x\n",
				this, path, buffer );

	return( 0 );
}

//readSymbols
//Reads both the symbol and the string table. Record streams for both of these must
//be open already, but don't need to allocate the buffers for the data, that will be
//allocated in this function. The symbol table is read into an array of syment structures,
//whereas the string table is read into a simple char buffer. -SK
int
COFFLoader::readSymbols( RecordStream * rfs1, syment * & buf1,
			 int & symCount,
			 RecordStream * rfs2, char   * & buf2 )
{
	Raid( RaidProtected )( "%N::readSymbols( ,,,, )\n", this );

	//read the symbol table -SK
	int size = rfs1->numberOfBytes();
	Assert( size != 0 );
	symCount = size / sizeof( syment );
	buf1 = (syment *)new char[size];
	rfs1->read( (char *)buf1, size );

	//read the string table -SK
	size = rfs2->numberOfBytes();
	if( size != 0 ) {
		buf2 = new char[size];
		rfs2->read( buf2, size );
	}
	else buf2 = 0;

	return( 0 );
}

//resolveUndefined
//This function looks big and complicated, but its extremely simple, actually.
//All it does is search the main symbol table for symbol names that match up with each
//undefined external reference in the COFF file's symbol table, and copy over the value (address)
//of the main symbol into the loaded symbol. Most of the rigamorole has to do with finding
//the names of symbols when they exceed 8 characters (it has to look in the string table then). -SK
//Also sets the _staticBase member to the address of the main table symbol called "p_glbl" -SK
int
COFFLoader::resolveUndefined()
{
	Raid( RaidProtected )( "%N::resolveUndefined()\n", this );

	int status = 0;
	char * key = new char[1024];

	char ** mainKeys = new char * [_mainSymbolCount];
	int * mainIndices = new int[_mainSymbolCount];
	int mainKeyCount = 0;

	//loop through the main symbol table (OS exported symbols)
        int i;
	for( i = 0; i < _mainSymbolCount; i++ ) {
		//For each main symbol that is not a filename, not a private (static) symbol
		//and NOT an undefined external reference...
	    if( ( _mainSymbols[i].n_sclass != C_FILE ) &&
		( _mainSymbols[i].n_sclass != C_STAT ) &&
		( _mainSymbols[i].n_scnum != 0 ) ) {

		//Allocate space for the new symbol. NOTE symbols must be less than 256 characters long! -SK
		mainKeys[mainKeyCount] = new char[256];
		mainIndices[mainKeyCount] = i; //Store the index into the symbol table for this symbol, for quick access later -SK

		char * t = mainKeys[mainKeyCount++];
		char * s = _mainSymbols[i]._n._n_name;
		
		//if the first byte is zero (this should really be a check of all of _n_nptr[0] to see if its zero...
		//but I think this is probably equivelant)... -SK
		if( *s == '\0' ) {
			//get the name from the main string table, at the offset specified in the symbol table _n_nptr[1] -SK
			s = &_mainStrings[_mainSymbols[i]._n._n_nptr[1]];
			//copy the symbol name into the mainKeys array -SK
			while( *s != '\0' ) *t++ = *s++;
		}
		else {
			//otherwise, the name is inlined -SK
			int limit = SYMNMLEN;
			//and we can copy it directly into the mainKeys array (making sure to only copy at most SYMNLEN bytes... it won't necessarily be null-terminated) -SK
			while( ( *s != '\0' ) && ( limit-- ) ) *t++ = *s++;
		}
		*t = '\0'; //null-terminate the symbol name in the in the mainKeys array -SK
	    }
	    i += _mainSymbols[i].n_numaux; //skip all the auxillary data that comes after this symbol -SK
	}

	//Search for a symbol in the main symbol table called "p_glbl" -SK
        int k;
	for( k = 0; k < mainKeyCount; k++ ) {
		if( strcmp( "p_glbl", mainKeys[k] ) == 0 ) break;
	}
	Assert( k < mainKeyCount ); //fail if we can't find it -SK

	//record the address of "p_glbl" -SK
	_staticBase = _mainSymbols[mainIndices[k]].n_value;
	Assert( _staticBase != 0 );

	//loop through all the symbols loaded from the COFF file
	for( i = 0; i < _loadSymbolCount; i++ ) {
		//for each symbol that is not a filename, not private (static), and IS an undefined external reference
	    if( ( _loadSymbols[i].n_sclass != C_FILE ) &&
		( _loadSymbols[i].n_sclass != C_STAT ) &&
		( _loadSymbols[i].n_scnum == 0 ) ) {
		char * t = key;
		char * s = _loadSymbols[i]._n._n_name;
		
		//Retrieve symbol name:
		//if the first byte is zero (this should really be a check of all of _n_nptr[0] to see if its zero...
		//but I think this is probably equivelant)... -SK
		if( *s == '\0' ) {
			//get the name from the COFF string table, at the offset specified in the symbol table _n_nptr[1] -SK
			s = &_loadStrings[_loadSymbols[i]._n._n_nptr[1]];
			//copy the symbol name into the mainKeys array -SK
			while( *s != '\0' ) *t++ = *s++;
		}
		else {
			//otherwise, the name is inlined -SK
			int limit = SYMNMLEN;
			//and we can copy it directly into the mainKeys array (making sure to only copy at most SYMNLEN bytes... it won't necessarily be null-terminated) -SK
			while( ( *s != '\0' ) && ( limit-- ) ) *t++ = *s++;
		}
		*t = '\0'; //null terminate the symbol name -SK

		//search through the main symbol table for any symbol name matching this one -SK
		int k;
                for( k = 0; k < mainKeyCount; k++ ) {
			if( strcmp( key, mainKeys[k] ) == 0 ) break;
		}
		//if we couldn't find it, return failure -SK
		if( k == mainKeyCount ) return( 1 );

		Raid( RaidDetail )( "%N::resolveUndefined() %s %d\n",
				    this, key, mainIndices[k] );
		
		//store the symbol's address, as retrieved from the main symbol table -SK
		_loadSymbols[i].n_value = _mainSymbols[mainIndices[k]].n_value;
	    }
	    i += _loadSymbols[i].n_numaux; //Skip auxillary data, as we did with the main symbol table -SK
	}

	//??Why aren't they deleting mainKeys and mainIndices? Major memory leak here!!?? -SK
	//??Plus, its bad practice to use "delete" on an array, it should be "delete []"... even though, in this case, there are no destructors to call?? -SK
	delete key;

	return( status );
}

//installSizeOf
//tells the Class object that represents the loaded class what the size of the
//object is (by calling the sizeof function in the loaded code). This
//function is used only by resolveDefined. -SK
int
COFFLoader::installSizeOf( char * key, syment * sym )
{
	Raid( RaidProtected )( "%N::installSizeOf( %s )\n", this, key );

	int status = 0;

	Assert( sym->n_scnum == 1 );
	Assert( sym->n_type == 0 );
	Assert( sym->n_numaux == 0 );

	//A very hacky way of getting the name of the class.
	//Assumes the key begins with "_sizeOf" followed by
	//the name of the class, followed by "_"...
	//Its done this way, I think, so that multiple classes can be
	//installed from a single file. But its still a hack. -SK
	key = &key[7];
        int i;
	for( i = 0; key[i] != '_'; i++ );
	key[i] = '\0';

	//use the address of the symbol representing the sizeof function
	//and map it into a void->int function pointer (this points to the
	//code for the sizeof -SK
	PFI pfi = (PFI)&_text[sym->n_value];

	//Look up the Class object that represents this object -SK
	ClassStar aClass = StandardNameServer->lookup( key, ClassClass );
	Raid( RaidDetail )( "%N::installSizeOf %N\n", this, aClass );
	
	//call the sizeof function and pass the result to the Class object -SK
	if( aClass != 0 ) aClass->setSizeOf( (*pfi)() );

	return( status );
}

//installConstructor
//tells the Class object that represents the loaded class what the
//address of the addressable constructor for the class is. (Recall,
//the addressable constructor is called by the Class object's virtual Constructor
//method which in turn calls the real constructor of the object via
//the "new" operator). -SK
int
COFFLoader::installConstructor( char * key, syment * sym )
{
	Raid( RaidProtected )( "%N::installConstructor( %s )\n", this, key );

	int status = 0;

	Assert( sym->n_scnum == 1 );
	Assert( sym->n_type == 0 );
	Assert( sym->n_numaux == 0 );

	//Just like the hacky hack in the installSizeOf function, this is
	//how they get the name of the class -SK
	key = &key[1];
        int i;
	for( i = 0; strncmp( &key[i], "Constructor", 11 ) != 0; i++ );
	key[i] = '\0';

	//Translate the address of the symbol (an index into the _text table)
	//into a "real" address (which is, of course, virtual) for the ClassConstructor
	//function -SK
	POFPO pf = (POFPO)&_text[sym->n_value];

	//Get the Class object that represents the loaded class -SK
	ClassStar aClass = StandardNameServer->lookup( key, ClassClass );
	Raid( RaidDetail )( "%N::installConstructor %N\n", this, aClass );
	
	//Inform the Class object what the address of the addressable constructor is -SK
	if( aClass != 0 ) aClass->setConstructor( new ClassConstructor( pf ) );

	return( status );
}

//resolveDefined
//Basically all this function does is install the sizes of all loaded classes
//and the addressable constructors for all loaded classes into the corresponding
//dynamic Class objects, so that the loaded classes can be instantiated -SK
int
COFFLoader::resolveDefined()
{
	Raid( RaidProtected )( "%N::resolveDefined()\n", this );

	int status = 0;
	char * key = new char[1024];

	//loop through all the loaded symbols
        int i;
	for( i = 0; i < _loadSymbolCount; i++ ) {
		//for all symbols that are not filenames or private, and ARE
		//defined, internal, references...
	    if( ( _loadSymbols[i].n_sclass != C_FILE ) &&
		( _loadSymbols[i].n_sclass != C_STAT ) &&
		( _loadSymbols[i].n_scnum != 0 ) ) {
		
		//get the full symbol name in the usual fashion (see resolveUndefined())
		char * t = key;
		char * s = _loadSymbols[i]._n._n_name;
		if( *s == '\0' ) {
			s = &_loadStrings[_loadSymbols[i]._n._n_nptr[1]];
			while( *s != '\0' ) *t++ = *s++;
		}
		else {
			int limit = SYMNMLEN;
			while( ( *s != '\0' ) && ( limit-- ) ) *t++ = *s++;
		}
		*t = '\0';

		//if the symbol is a sizeOf function for some class, install it
		if( match( key, "_sizeOf*__Fv" ) ) {
			status = installSizeOf( key, &_loadSymbols[i] );
			if( status != 0 ) break;
		}
		//if the symbol is an addressable constructor for some class, isntall it
		else if( match( key, "_*Constructor__*" ) ) {
			status = installConstructor( key, &_loadSymbols[i] );
			if( status != 0 ) break;
		}
	    }
	    i += _loadSymbols[i].n_numaux; //skip the symbol's auxillary data
	}
	delete key;

	return( status );
}


//relocateSymbols
//uses the relocation tables to translate all references in the text and data section from
//the file-relative virtual addresses to the loaded memory relative virtual addresses -SK
//The basic issue here is that anytime a symbol is referred to in the .text or .data sections,
//it is referred to by its address in some section. This address is initially file-relative for internal symbols
//and unknown for external symbols. Relocation fixes up these address so that they are memory relative. 
//This includes specifying the addresses of external symbols that we resolved with resolveUndefined() -SK
int
COFFLoader::relocateSymbols()
{
	Raid( RaidProtected )( "%N::relocateSymbols() %x %x %x\n", this,
				_textRelocated, _dataRelocated, _staticBase );

	Raid( RaidProtected )( "%N::relocateSymbols text symbols %d\n",
				this, _textRelocCount );

	//loop through all the relocation entries for the .text section -SK
	int i;
        for( i = 0; i < _textRelocCount; i++ ) {
		//get a pointer to the current relocation (we have to do this extra casting because we loaded the relocation tables as character buffers -SK
		reloc * rp = (reloc *)&_textRelocs[i * RELOCSIZE];

		//the only type of .text relocation supported is 32-bit Address Displacement -SK
		Assert( ( rp->r_type & R_Type ) == R_Address );
		Assert( ( rp->r_type & R_Size ) == R_Four );
		Assert( ( rp->r_type & R_Format ) == R_Displacement );

		//make sure the symbol referred to by the relocation entry is actually valid -SK
		Assert( rp->r_symndx >= 0 );
		Assert( rp->r_symndx < _loadSymbolCount );

		//make sure the virtual address (location in file) is actually in the .text section -SK
		Assert( rp->r_vaddr >= _textPrelocated );
		rp->r_vaddr -= _textPrelocated; //transform file offset into an offset into the .text section (array index) -SK
		Assert( rp->r_vaddr < _textSize );

		//get a pointer to the 4 bytes (32-bits) stored at the symbol's location in the .text section -SK
		int * location = (int *)&_text[rp->r_vaddr];
		
		//get the section number of the symbol from the symbol table
		//scnum=0 means its an external reference, scnum=1 means the symbol resides in the .text section
		//scnum=2 means the symbol resides in the .data section -SK
		int section = _loadSymbols[rp->r_symndx].n_scnum;

		//reverse the bytes so our pointer math will work -SK
		swapBytes( location );
		Raid( RaidDetail )( "%N::relocateSymbols %x %d %x\n",
                                    this, *location, section, rp->r_type );

		//if the symbol we're pointing at is external (section==0), we should add in the
		//address for that external symbol that we found during the resolveUndefined() stage -SK
		if( section == 0 ) {
		    	*location += _loadSymbols[rp->r_symndx].n_value;
		}

		//Do different things depending on what the address should be relative to -SK
		switch( rp->r_type & R_RelativeTo ) {
		case R_Absolute:
			//Not relative to anything.
			//Only valid for .text and external symbols -SK
 			Assert( ( section == 0 ) ||
                                ( section == 1 ) );
			//if it was a .text symbol, we need to add in the memory address of the begining of the
			//text section, so that the address is now absolute in memory space. -SK
			if ( section == 1 ) *location += _textRelocated;
			break;
		case R_PC_Relative:
			//Relative to the program counter
			//Valid for .text, .data., and external symbols -SK
  			Assert( ( section == 0 ) ||
                                ( section == 1 ) ||
                                ( section == 2 ) );
			//Since the program counter will be a memory address pointing to something inside the .text section,
			//if the reference is referring to anything outside of the .text section, we need to subtract off the begining address of the .text section -SK
			if( section != 1 ) *location -= _textRelocated;
			//if the reference is referring to something in the .data section, we need to translate that to a real memory address by adding in the begining memory address of the .data section -SK
			if( section == 2 ) *location += _dataRelocated;
			break;
		case R_SB_Relative:
			//Relative to the static base ("p_glbl" symbol in main symbol table)
			//Only valid for external references -SK
 			Assert( section == 0 );
			//make address relative to the location of the "p_glbl" symbol
            *location -= _staticBase;
			break;
		default:
			Assert( NOTREACHED );
		}

		//set the top two bits to 1...
		//??Why do we need to do this?? -SK
		*location |= 0xc0000000;

		swapBytes( location );
	}

	Raid( RaidProtected )( "%N::relocateSymbols data symbols %d\n",
				this, _dataRelocCount );

	//loop through all relocation entries for the .data section -SK
	for( i = 0; i < _dataRelocCount; i++ ) {
		//get a pointer to the current relocation (we have to do this extra casting because we loaded the relocation tables as character buffers -SK
		reloc * rp = (reloc *)&_dataRelocs[i * RELOCSIZE];

		//make sure the symbol referred to by the relocation entry is actually valid -SK
		Assert( rp->r_symndx >= 0 );
		Assert( rp->r_symndx < _loadSymbolCount );

		//the only type of .data relocation supported is 32-bit Absolute Address Number -SK
		Assert( rp->r_type == ( R_Address | R_Four |
					R_Number  | R_Absolute ) );

		//make sure the address (file offset) of the reference is in actually in the .data section -SK
		Assert( rp->r_vaddr >= _dataPrelocated );
		rp->r_vaddr -= _dataPrelocated; //transform file offset into an array index into the _data array -SK
		Assert( rp->r_vaddr < _dataSize );

		//get the four bytes at that location (this is the reference we need to alter) -SK
		int * location = (int *)&_data[rp->r_vaddr];

		if( _loadSymbols[rp->r_symndx].n_scnum == 0 ) {
			Assert( *location == 0 );
			//if the reference is to an external symbol, set the reference to the address we found for that symbol during resolveUndefined() -SK
			*location = _loadSymbols[rp->r_symndx].n_value;
		}
		else if( _loadSymbols[rp->r_symndx].n_scnum == 1 ) {
			//if the reference is to a .text symbol, add in the memory location of the begining of _text -SK
			*location += _textRelocated;
		}
		else Assert( NOTREACHED );
	}

	return( 0 );
}

//Standard Choices Class object installation -SK
#include "ClassInstaller.h"
DEFINECLASS("COFFLoader",COFFLoader,Object,0,0);
