#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Panos Kougiouris (kougiour@cs.uiuc.edu)
 * Contributing Author: See-mong Tan (stan@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * 1. Member methods of class DiskContainer (file DiskContainer.h)
 */

#include "Assert.h"
#include "Class.h"
#include "Disk.h"
#include "DiskContainer.h"
#include "FileSystem.h"
#include "MemoryObjectPartition.h"
#include "Object.h"
#include "OutputStream.h"
#include "Raid.h"

#ifdef __GNUC__
#pragma implementation
#endif


/************************************************************************/
/*  Member methods for class DiskContainer				*/
/************************************************************************/

FileObject *
DiskContainer::basicOpen( int idNumber, int & status, int flags )
{
    Raid( RaidMethod )( "%NDiskContainer::basicOpen( %d )\n", this, idNumber );

    Disk * disk = (Disk *) (ProxiableObject *) _source;

    int start, units;

    if ( disk->locatePartition( idNumber, start, units ) == -1 ) {
        Raid( RaidMethod )(
           "%N::DiskContainer::basicOpen: empty partition %d\n", this,
				idNumber );
         status = UnusedIdNumber;
         return( 0 );
    }

    return ( new MemoryObjectPartition( units, _source->log2UnitSize(),
					idNumber, this, _source, start ) );
}


DiskContainer::DiskContainer( PersistentMemoryObject * source,
					    int partitions )
	: MemoryObjectContainer( source )
{
	initDebug();
	Raid( RaidConstructor )( "%NDiskContainer::DiskContainer( %N, )\n",
				 this, _source );

	_partitions = partitions;
	initializeArrays( partitions );
}


ProxiableObject *
DiskContainerConstructor( ProxiableObject * source )
{
	Debug.printf( "DiskContainerConstructor( %N )\n", source );

        if( source->isKindOf( DiskClass ) ) {
		Disk * disk = (Disk *)source;
                return( new DiskContainer( disk, disk->partitions() ) );
        }
        else return( 0 );
}


#include "ClassInstaller.h"
DEFINECLASS("DiskContainer",DiskContainer,MemoryObjectContainer,&DiskContainerConstructor,0);
