#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ElFDictionary.cc - ELF Dictionary class : dthomas1@cs.uiuc.edu 
 */

#include "Raid.h"
#include "elF.h"
#include "ELFDictionary.h"
#include "FileSystem.h"
#include "String.h"
#include "PersistentMemoryObject.h"

#ifdef __GNUC__
#pragma implementation
#endif

using namespace llvm::ELF;

int
ELFDictionary::findKey( char * key, int )
{
	Raid( RaidProtected )( "%N::findKey( %s, )\n", this, key );

#ifdef JEFF
	Console()<<this<<"::findKey( "<<key<<")\n";
#endif
	
	_idNumberOfKey = 0;
	_emptyOffset = -1;
	_offset = -1;

	if( ( strcmp( key, "." ) == 0 ) || ( strcmp( key, ".." ) == 0 ) ) {
		_idNumberOfKey = RootId;
	}
	else if( strcmp( key, "Symbol.Table" ) == 0 ) {
		_idNumberOfKey = SymbolTableId;
	}
	else if( strcmp( key, "String.Table" ) == 0 ) {
		_idNumberOfKey = StringTableId;
	}
	else if( strcmp( key, "File.Header" ) == 0 ) {
		_idNumberOfKey = FileHeaderId;
	}
	else if( strcmp( key, "String.Symbol" ) == 0 ) {
		_idNumberOfKey = StringSymbolId;
	}
	else for( int i = 0; i < _filehdr->e_shnum; i++ ) {
		if( strcmp( key, &_strTable[_scnhdrs[i].sh_name] ) == 0 ) {
			_idNumberOfKey = SectionIds + i * 4 + Section;
			break;
		}
		int nameLength = strlen(  &_strTable[_scnhdrs[i].sh_name] );
		if( strncmp( key,  &_strTable[_scnhdrs[i].sh_name] , nameLength ) == 0 ) {
			if( strcmp( &key[nameLength], ".Header" ) == 0 ) {
				_idNumberOfKey = SectionIds + i * 4 + Header;
				break;
			}
			else if( strcmp( &key[nameLength], ".Reloc" ) == 0 ) {
				_idNumberOfKey = SectionIds + i * 4 + Reloc;
				break;
			}
			else if( strcmp( &key[nameLength], ".Lineno" ) == 0 ) {
				_idNumberOfKey = SectionIds + i * 4 + LineNo;
				break;
			}
		}
	}

	Raid( RaidProtected )( "%N::findKey returns %d\n",
				this, _idNumberOfKey  );

	//normally, this just returns 1 or 0 depending on whether it succeeded or not,
	//but for our purposes, we actually want to know the id number that was returned -SK
	return( _idNumberOfKey );
}

int
ELFDictionary::basicKeys( char * keyBuffer, int bufferSize, int & count )
{
    Raid( RaidMethod )( "%N::basicKeys( buffer %x, size %d ) _size %d\n", 
			this, keyBuffer, bufferSize, _size );

#ifdef JEFF
    Console()<<this<<"::basicKeys()\n";
#endif

    strcpy( keyBuffer, ".\n..\nFile.Header\n" );
    count = 4;
    if( _filehdr->e_shstrndx != 0 ) {
	strcat(keyBuffer,"String.Symbol\n");
	count++;
	strcat( keyBuffer, "Symbol.Table\n" );
	count++;
	strcat( keyBuffer, "String.Table\n" );
	count++;
    }
    char * t = &keyBuffer[strlen( keyBuffer )];

    for( int i = 0; i < _filehdr->e_shnum; i++ ) {
			char * s = &_strTable[_scnhdrs[i].sh_name] ;
			while( *s != '\0' ) *t++ = *s++;
			s = ".Header";
			while( *s != '\0' ) *t++ = *s++;
			*t++ = '\n';
			count++;

			s =  &_strTable[_scnhdrs[i].sh_name] ;
			while( *s != '\0' ) *t++ = *s++;
			*t++ = '\n';
			count++;

			if(isRelocatable(_scnhdrs[i].sh_name)) {
				s = &_strTable[_scnhdrs[i].sh_name] ;
				while( *s != '\0' ) *t++ = *s++;
					s = ".Reloc";
				while( *s != '\0' ) *t++ = *s++;
					*t++ = '\n';
		count++;
	}

#if 0
	if( _scnhdrs[i].s_nlnno > 0 ) {
		s = _scnhdrs[i].s_name;
		while( *s != '\0' ) *t++ = *s++;
		s = ".Lineno";
		while( *s != '\0' ) *t++ = *s++;
		*t++ = '\n';
		count++;
	}

#endif
    }
    *t++ = '\0';

    return( 0 );
}

bool
ELFDictionary::isRelocatable(int strIndex)
{
	char strName[256];
	char *origSection = &_strTable[strIndex];
	strcpy(strName, ".rela");
	strcat(strName, origSection);
	for (int i=0; i<_filehdr->e_shnum; i++){
		if (strcmp(&_strTable[_scnhdrs[i].sh_name],strName) == 0){
			return (true);
		}
	}
  return (false); /* Not all sections have reloc sections */
}


ELFDictionary::~ELFDictionary()
{
	Raid( RaidConstructor ) ( "%N::~ELFDictionary()\n", this );
}

ELFDictionary::ELFDictionary( PersistentMemoryObject * source )
: MemoryObjectDictionary( source )
{
	initDebug();
	Raid( RaidConstructor ) ( "%N::ELFDictionary( %N )\n", this, source);

#ifdef JEFF
	Console()<<"ELFDictionary constructor called.\n";
#endif

	_maxKeyLength = 32;

	Assert(_size!=0);
	reMap();

	_filehdr = (filehdr *)_buffer;

	Assert( _filehdr!=0 );

#ifdef JEFF
	Console() << "_filehdr->e_shoff: " << _filehdr->e_shoff << "\n" << eor;
#endif
	_scnhdrs = (scnhdr *)&_buffer[_filehdr->e_shoff];
#ifdef JEFF
	Console()<<"_scnhdrs assignment succeeded.\n";
#endif

 	 _strTable = (char*)&_buffer[_scnhdrs[_filehdr->e_shstrndx].sh_offset];
#ifdef JEFF
   	Console()<<"_strTable assignment succeeded.\n";
#endif
}

ProxiableObject * 
ELFDictionaryConstructor( ProxiableObject * source )
{
	if( source->isKindOf( PersistentMemoryObjectClass ) ) {
		return( new ELFDictionary( (PersistentMemoryObject *)source ) );
	}
	else return( 0 );
}

#include "ClassInstaller.h"
DEFINEPERSISTENTCLASS("ELFDictionary",ELFDictionary,MemoryObjectDictionary,&ELFDictionaryConstructor,0);
