#ifdef __GNUC__
#pragma implementation
#endif

/*****************************
*ELFLoader                   *
*Like the COFFLoader, but for*
*ELF files. Specifically     *
*designed to load Linux ELF  *
*binaries (for OS modules)   *
******************************
*Linux adaptor for Choices   *
*Project, CS423, Spring 2004 *
*****************************/
//SCOTT KIRCHER
//Note, this code is machine dependent (on SPARCS)
//even though it isn't in the MachineDependent directory

#include "Assert.h"
#include "Raid.h"
#include "ELFLoader.h"
#include "NameServer.h"
#include "PersistentArrayStream.h"
#include "PersistentMemoryObject.h"
#include "MemoryObjectDictionary.h"
#include "OutputStream.h"
#include "String.h"
#include "Byte.h"
#include "FileSystem.h"
#include "FileSystemInterface.h"
#include "RecordStream.h"
#include "Class.h"
#include "ClassStar.h"
#include "Convert.h"
#include "Ctype.h"
#include "elF.h"
#include "ELFDictionary.h"
#include "ClassConstructor.h"

#ifdef __GNUC__
#pragma implementation
#endif


using namespace llvm::ELF; 

//temporary function to hack printk calls into Console calls
//Note that right now it doesn't do formatted output, just prints whatever's in f
int printkHack( const char * f, ...)
{
	Console().printf(f);
	return 0;
}

//reverseEndianness
//Reverses the endianness of a 32-bit word
//(swaps the first and last bytes, swaps the middle two bytes)
void
reverseEndianness( int * location )
{
	char * cp = (char *)location;
	cp[0] ^= cp[3];
	cp[3] ^= cp[0];
	cp[0] ^= cp[3];
	cp[1] ^= cp[2];
	cp[2] ^= cp[1];
	cp[1] ^= cp[2];
}


//define the size of a relocation entry
//SPARC machines use only the addend form of relocation entry
#define RELOCSIZE (sizeof(Elf32_Rela))
typedef Elf32_Rela reloc;

//Function pointer type (void->int)
typedef int (* PFI)();


//function to convert from a persistent memory object to a record stream
//code pretty much stolen from file-system-interface
RecordStreamStar ELFLoader::convertToRecordStream(PersistentMemoryObjectRef mo)
{
	int status=Success;
	ProxiableObjectRef po;

            extern PersistentClassStar PersistentArrayClass;
            extern PersistentClassStar FixedRecordFileClass;
            extern PersistentClassStar VariableRecordFileClass;
            extern ClassStar FixedRecordStreamClass;
            extern ClassStar VariableRecordStreamClass;
            PersistentObjectStar p = mo->asA( PersistentArrayClass, status );
            if( p != 0 ) {
               po  = PersistentArrayStreamClass->constructor( p );
            }
            else {
                p = mo->asA( FixedRecordFileClass, status );
                if( p != 0 ) {
                    po = FixedRecordStreamClass->constructor( p );
                }
                else {
                    p = mo->asA( VariableRecordFileClass, status );
                    if( p != 0 ) {
                        po = VariableRecordStreamClass->constructor( p );
                    }
                }
            }
            if( ( po != 0 ) && ( !po->isKindOf( RecordStreamClass ) ) ) po = 0;
	if(status!=Success) 
	{
		Console()<<this<<"::convertToRecordStream failed: "<<StatusCodes[status]<<"\n";
	}
	return po;
}

//ELFLoader C++ constructor
//Takes a file name of an ELF file to be loaded
//Unlike its COFFLoader brother, you can specify an explicit path
//If you DON'T specify an extension, .o is assumed
// -SK
ELFLoader::ELFLoader( const char * clsName, int export_symbol_size, ExportTuple *export_symbol_table, int import_symbol_size, ImportTuple *import_symbol_table )
	//Initialize the file-system interface by querying the name server for the current
	//fileSystemInterface. -SK
: _fsi( StandardNameServer->lookup( "fileSystemInterface",
				FileSystemInterfaceClass ) ), _root( 0 )
{
	initDebug();

	Raid( RaidConstructor )( "%N::ELFLoader( %s )\n", this, clsName );

	error_code=Success;
	//Initialize pointers
	_text = 0;
	_data = 0;
	_bss = 0;
	_rodata = 0;
	_rodatastr18 = 0;
	_textRelocs = 0;
	_dataRelocs = 0;
	_bssRelocs = 0;
	_rodataRelocs = 0;
	_rodatastr18Relocs = 0;

	_loadSymbols = 0;
	_loadStrings = 0;

	//add default extension if necessary
    int i;
	for( i = 0; clsName[i]!='.' && clsName[i]!=0; i++ );  
	
	strcpy( _loadObjectName, clsName);
	if(clsName[i]!='.')	strcat( _loadObjectName, ".o" );

	//Specify the symbol table's "name"
	strcpy( _loadSymbolTableName, "Symbol.Table" );

	//And also for the string table
	strcpy( _loadStringTableName, "String.Symbol" );

	//And also for the .text section of the ELF
	strcpy( _loadTextName, ".text" );

	//This is for the Reloc subsection of the text section
	strcpy( _loadTextRelocName, ".text.Reloc" );

	//This is for the .data section of the ELF
	strcpy( _loadDataName, ".data" );

	//This is for the Reloc subsection of the data section
	strcpy( _loadDataRelocName, ".data.Reloc" );

	//This is for the .bss section of the ELF
	strcpy( _loadBssName, ".bss" );

	//This is for the Reloc subsection of the bss section
	strcpy( _loadBssRelocName, ".bss.Reloc" );

	//This is for the .rodata (readonly data) section of the ELF
	strcpy( _loadRODataName, ".rodata" );

	//This is for the Reloc subsection of the data section
	strcpy( _loadRODataRelocName, ".rodata.Reloc" );

	//This is for the .rodata.str1.8 (string readonly data) section of the ELF
	strcpy( _loadRODatastr18Name, ".rodata.str1.8" );

	//This is for the Reloc subsection of the data section
	strcpy( _loadRODatastr18RelocName, ".rodata.str1.8.Reloc" );

	Raid( RaidDetail )( "%N::ELFLoader Looking for new code in %s\n",
			    this, _loadObjectName );

	Assert( _fsi != 0 );
        Console()<<"*******DCT**********"<<"\n";

	int status = 0;
	//Open the ELF file

	//Using ELFDictionaryClass
	//to force handling of file as a ELF file

 	_root = _fsi->open( _loadObjectName, ELFDictionaryClass, status );
        if(status!=Success) Console()<<StatusCodes[status]<<"\n";
	Console()<<_root<<"\n";
	if(!_root)
	{
		Console()<<"Unable to open "<<_loadObjectName<<"\n";
		error_code=status;
		return;
	}

	//Supposidely, in Choices, we should have been able to access
	//portions of the ELF file as a file system (i.e. passing in a name
	//like "module.o/Symbol.Table"
	//but the ELF files were not being recognized as "directories"
	//So, instead, we directly use the _root (which is the ELF file itself)
	//to open the subsections. We then have to convert the resulting
	//PersistentMemoryObjects into RecordStreams, so we can use them

	Console()<<"Opening Symbol.Table.\n";
	PersistentMemoryObjectRef mo=_root->open(_loadSymbolTableName,status,0);	    
	RecordStreamStar loadSymbolTable=convertToRecordStream(mo);
	Console()<<StatusCodes[status]<<"\n";

	
	//Open the string table (part of the ELF file) -SK
	Console()<<"Opening String.Table.\n";
	mo=_root->open(_loadStringTableName,status,0);
        Assert (status  == 0);	    
	RecordStreamStar loadStringTable=convertToRecordStream(mo);
	Console()<< "**************" << StatusCodes[status]<<"\n";



//TODO: Main Choices Symbol table
/*	//Open the main symbol table. Symbols exported by the OS. -SK
	RecordStreamStar mainSymbolTable =
					_fsi->open( (char *)MainSymbolTableName,
					RecordStreamClass, status );

	Assert(status == Success);	
	//Open the main string table. Strings (long symbols) exported by the OS. -SK
	RecordStreamStar mainStringTable =
					_fsi->open( (char *)MainStringTableName,
					RecordStreamClass, status );*/

	Assert(status == Success);	
	//Note that, we haven't opened the .text or .data sections. This is accomplished
	//in the readSection function instead.

	//Make sure we were able to open everything -SK
	if( ( _root == 0 ) ||
	    ( loadSymbolTable == 0 ) ||
	    ( loadStringTable == 0 ) 
//TODO: Main Choices Symbol Table
	    /*|| ( mainSymbolTable == 0 ) ||
	    ( mainStringTable == 0 )*/ ) {
		Raid( RaidDetail )( "%N::ELFLoader: Opens failed.\n", this );
        
	    Console()<<"Unable to open some necessary file\n";
                Console()<<"_root="<<_root<<"\n";
		Console()<<"loadSymbolTable="<<loadSymbolTable<<"\n";
		Console()<<"loadStringTable="<<loadStringTable<<"\n";
/*		Console()<<"mainStringTable="<<mainStringTable<<"\n";
		Console()<<"mainSymbolTable="<<mainSymbolTable<<"\n";*/
		error_code=1;
		return;
	}

	Raid( RaidDetail )( "%N::ELFLoader: Opens succeeded.\n", this );

	//read the text section into a buffer (_text)
	status = readSection( _loadTextName, _text, _textSize );
	Assert( status == Success && _textSize>0);
	Console()<<_root->findKey(_loadTextName,0)<<"\n";
	_textSectionNumber = (_root->findKey(_loadTextName,0)-SectionIds)>>2;

	//read the rodata section into a buffer (_rodata)
	status = readSection( _loadRODataName, _rodata, _rodataSize );
	if(status == Success && _rodataSize>0)
                _rodataSectionNumber = (_root->findKey(_loadRODataName,0)-SectionIds)>>2;
        else
                _rodataSectionNumber = -1;

	//read the rodatastr18 section into a buffer (_rodatastr18)
	status = readSection( _loadRODatastr18Name, _rodatastr18, _rodatastr18Size );
	if(status == Success && _rodatastr18Size>0)
                _rodatastr18SectionNumber = (_root->findKey(_loadRODatastr18Name,0)-SectionIds)>>2;
        else
                _rodatastr18SectionNumber = -1;

	Console()<<"*****CONTENTS OF .rodata.str1.8 SECTION******\n";
	for(int c=0;c<_rodatastr18Size;c++)
	Console()<<_rodatastr18[c];
	Console()<<"\n*****END CONTENTS OF .rodata.str1.8 SECTION******\n";	

	//read the data section into a buffer (_data)
	status = readSection( _loadDataName, _data, _dataSize );
	//data section doesn't have to exist
	if(status == Success && _dataSize>0)
		_dataSectionNumber = (_root->findKey(_loadDataName,0)-SectionIds)>>2;
	else
		_dataSectionNumber = -1;

	//read the bss section into a buffer (_bss)
	status = readSection( _loadBssName, _bss, _bssSize );
	//bss section doesn't have to exist
	if(status == Success && _bssSize>0)
		_bssSectionNumber = (_root->findKey(_loadBssName,0)-SectionIds)>>2;
	else
		_bssSectionNumber = -1;


	//read the text relocation section into a buffer
	status = readSection( _loadTextRelocName, _textRelocs, _textRelocCount);
	if(status == Success)
		_textRelocCount /= RELOCSIZE;
	else
	{
		_textRelocCount=0;
		Console()<<"ELFLoader: Warning! couldn't find relocation info for .text section\n";
	}

	//read the data relocation section into a buffer
	status = readSection( _loadDataRelocName, _dataRelocs, _dataRelocCount);
	if(status==Success)	
		_dataRelocCount /= RELOCSIZE;
	else
		_dataRelocCount=0;

	//read the bss relocation section into a buffer
	status = readSection( _loadBssRelocName, _bssRelocs, _bssRelocCount);
	if(status==Success)	
		_bssRelocCount /= RELOCSIZE;
	else
		_bssRelocCount=0;

	//read the rodata relocation section into a buffer
	status = readSection( _loadRODataRelocName, _rodataRelocs, _rodataRelocCount);
	if(status==Success)
		_rodataRelocCount /= RELOCSIZE;
	else
		_rodataRelocCount=0;

	//read the rodatastr18 relocation section into a buffer
	status = readSection( _loadRODatastr18RelocName, _rodatastr18Relocs, _rodatastr18RelocCount);
	if(status==Success)
		_rodatastr18RelocCount /= RELOCSIZE;
	else
		_rodatastr18RelocCount=0;


	//read the ELF file's symbol and string tables 
	status = readSymbols( loadSymbolTable, _loadSymbols, _loadSymbolCount,
			      loadStringTable, _loadStrings );

	Assert( status == 0 );

/*	//Record the original address of the .text (executable code) section
	Assert( strcmp( _loadSymbols[2]._n._n_name, ".text" ) == 0 );
	_textPrelocated = _loadSymbols[2].n_value;	*/
	//For ELF files, unlike COFF files, reference offsets are relative to sections
	//so _textPrelocated is always zero (I think)
	_textPrelocated = 0;
	
	//Funky pointer stuff! We're finding the location of the .text section in "real" memory here,
	//and subtracting the file-offset address, to get the amount we need to offset all references into
	//the .text section by, so that they will reference "real" virtual memory.
	_textRelocated = (int)_text - _textPrelocated; 

/*	//Record the original address of the .data (initialized data) section -SK
	Assert( strcmp( _loadSymbols[4]._n._n_name, ".data" ) == 0 );
	_dataPrelocated = _loadSymbols[4].n_value;*/
	//For ELF files, unlike COFF files, reference offsets are relative to sections
	//so _dataPrelocated is always zero (I think)
	_dataPrelocated=0;

	//Same as above for .text, except this time it's for .data -SK
	_dataRelocated = (int)_data - _dataPrelocated;

	_bssPrelocated=0;

	//Same as above for .text, except this time it's for .bss -SK
	_bssRelocated = (int)_bss - _bssPrelocated;

	_rodataPrelocated=0;

	//Same as above for .text, except this time it's for .rodata -SK
	_rodataRelocated = (int)_rodata - _rodataPrelocated;

	_rodatastr18Prelocated=0;

	//Same as above for .text, except this time it's for .rodata.str1.8 -SK
	_rodatastr18Relocated = (int)_rodatastr18 - _rodatastr18Prelocated;

	//resolve undefined symbols (get external reference symbol's addresses from main symbol table) -SK
	status = resolveUndefined(export_symbol_size, export_symbol_table);
	Assert( status == 0 );
	//relocate symbols
	status = relocateSymbols();
	Assert( status == 0 );
	//find things like init_module, or constructors...
	status = resolveDefined( import_symbol_size, import_symbol_table);
	Assert( status == 0 );
	Console()<<"ELFLoader completed all operations\n";
	error_code = status;
}

//ELFLoader destructor
//release non reference counted dynamic memory
ELFLoader::~ELFLoader()
{
	Raid( RaidConstructor )( "%N::~ELFLoader()\n", this );

	//Note that we do NOT delete _text and _data. So, the loaded class is still available
	//even after the ELFLoader class that loaded it dies.

	if(_textRelocs) delete _textRelocs;
	if(_dataRelocs) delete _dataRelocs;
	if(_bssRelocs) delete _bssRelocs;
	if(_rodataRelocs) delete _rodataRelocs;
	if(_rodatastr18Relocs) delete _rodatastr18Relocs;
}

//readSection
//Reads a section of the ELF file, as specified by the path argument
//Since we treat the ELF file as another filesystem, we can access
//its sections by using something like "filename/Section"
int
ELFLoader::readSection( char * path, char * & buffer, int & size )
{
	Raid( RaidProtected )( "%N::readSection( %s, )\n", this, path );

	int status = 0;

	Console()<<"Reading Section: "<<path<<"\n";

	//Open the section as a record stream (basically will just read in all the bytes of the section in order) -SK
	PersistentMemoryObjectRef mo=_root->open(path,status,0);
        if (status != Success) return status;	    
	RecordStreamStar rfs=convertToRecordStream(mo);
	

	if( rfs == 0 ) {
		Assert( status != 0 );
		return( status );
	}

	//Actually read the section (note each section behaves like its own file, and knows how long it is) -SK
	size = rfs->numberOfBytes();
	if( size != 0 ) {
		buffer = new char[size];
		rfs->read( buffer, size );
	}
	else buffer = 0;

	Raid( RaidProtected )( "%N::readSection loaded %s @ %x\n",
				this, path, buffer );

	return( 0 );
}

//readSymbols
//Reads both the symbol and the string table. Record streams for both of these must
//be open already, but don't need to allocate the buffers for the data, that will be
//allocated in this function. The symbol table is read into an array of syment structures,
//whereas the string table is read into a simple char buffer.
int
ELFLoader::readSymbols( RecordStream * rfs1, Elf32_Sym * & buf1,
			 int & symCount,
			 RecordStream * rfs2, char   * & buf2 )
{
	Console()<<"********Read Symbols**********\n";
	Raid( RaidProtected )( "%N::readSymbols( ,,,, )\n", this );

	//read the symbol table -SK
	int size = rfs1->numberOfBytes();
	Assert( size != 0 );
	symCount = size / sizeof( Elf32_Sym );
	buf1 = (Elf32_Sym *)new char[size];
	rfs1->read( (char *)buf1, size );

	//read the string table -SK
	size = rfs2->numberOfBytes();
	Assert (size !=0);
	if( size != 0 ) {
		buf2 = new char[size];
		rfs2->read( buf2, size );
	}
	else buf2 = 0;

	Console()<<"*****CONTENTS OF String.Symbol SECTION******\n";
	for(int c=0;c<size;c++)
		Console()<<buf2[c];
	Console()<<"\n*****END CONTENTS OF String.Symbol SECTION******\n";	

	return( 0 );
}

//Currently our resolveUndefined function looks specifically for
//references to printk
int ELFLoader::resolveUndefined(int export_symbol_size, ExportTuple *export_symbol_table)
{
	int i,c;
	Console()<<"Resolving External Symbols...\n";
	//loop through all the symbols looking for undefined globals
	for(i=0;i<_loadSymbolCount;i++)
	{
		//check to see if the symbol is not local, and is undefined
		Console()<<"Found symbol "<<getString(_loadSymbols[i].st_name)<<"\n";
		if(_loadSymbols[i].getBinding()!=STB_LOCAL
			&& _loadSymbols[i].st_shndx==SHN_UNDEF)
		{
			Console()<<"Attempting to resolve "<<getString(_loadSymbols[i].st_name)<<"\n";
			//if so, it is a candidate for resolution
			//we need to search through all the exported symbols for a match
			for(c=0;c<export_symbol_size;c++)
			{
				if(match(getString(_loadSymbols[i].st_name),export_symbol_table[c].name))
				{
					//once the symbol is found, we must update its address
					_loadSymbols[i].st_value=(Elf32_Addr)export_symbol_table[c].address;				
					Console()<<"Resolved external symbol: "<<getString(_loadSymbols[i].st_name)<<" ("<<_loadSymbols[i].st_value<<")\n";
					break;
				}
			}
			if(c==export_symbol_size)
			{
				Console()<<"Unresolved external symbol: "<<getString(_loadSymbols[i].st_name)<<"\n";
				return 1;
			}
		}
	}

	return 0;
}


//resolveDefined is very much like resolveUndefined, except it works in the opposite direction, in a sense
//it looks for defined symbols in the loaded ELF file that match the import_symbols, and grabs their address
//so that Choices can call such functions
int ELFLoader::resolveDefined(int import_symbol_size, ImportTuple *import_symbol_table)
{
	int i,c;
	int status=Success;

	//loop through all the symbols looking for a function named init_module 
	//search through all the import symbols to see if this is a symbol we were looking for
	for(c=0;c<import_symbol_size;c++)
	{
		for(i=0;i<_loadSymbolCount;i++)
		{
			if((_loadSymbols[i].getType()==STT_FUNC || _loadSymbols[i].getType()==STT_OBJECT)
				&& _loadSymbols[i].st_shndx!=SHN_UNDEF)
			{
				//see if this is the init_module function
				if(match(getString(_loadSymbols[i].st_name),import_symbol_table[c].name))
				{
					//if it is, be happy, grab its address
					//since we've already done the relocation, the symbol's
					//st_value is the address we need
					//place that address in the location specified by the import_symbol_table
					*import_symbol_table[c].address=(void *)_loadSymbols[i].st_value;
				
					Console()<<"Resolved defined symbol: " << _loadSymbols[i].st_value << " -- " << getString(_loadSymbols[i].st_name) << "\n";
					break;
				}
			}			
		}
		if(i==_loadSymbolCount)
		{
			Console()<<"Required symbol not found: "<<import_symbol_table[c].name<<"\n";
			status=1;
		}
	}

	return status;
}

//Supported SPARC relocation types:
#define R_SPARC_NONE 0
#define R_SPARC_32   3
#define R_SPARC_DISP32 6
#define R_SPARC_WDISP30 7
#define R_SPARC_HI22 9
#define R_SPARC_LO10 12

//relocateSymbols
//uses the relocation tables to translate all references in the text and data section from
//the file-relative virtual addresses to the loaded memory relative virtual addresses
//The basic issue here is that anytime a symbol is referred to in the .text or .data sections,
//it is referred to by its address in some section. This address is initially file-relative for internal symbols
//and unknown for external symbols. Relocation fixes up these address so that they are memory relative. 
//This includes specifying the addresses of external symbols that we resolved with resolveUndefined()
int ELFLoader::relocateSymbols()
{
	int i;
	int section;
	Console() << "Tot num of symbols " << _loadSymbolCount << " \n";
	Console() << "Relocating Symbols...\n";
	Console() << "Handling only symbols for: "<<"\n";
	Console() << "  .text (section "<<_textSectionNumber<<")\n";
	Console() << "  .data (section "<<_dataSectionNumber<<")\n";
	Console() << "  .bss (section "<<_bssSectionNumber<<")\n";
	Console() << "  .rodata (section "<<_rodataSectionNumber<<")\n";
	Console() << "  .rodata.str1.8 (section "<<_rodatastr18SectionNumber<<")\n";
	//before doing the relocation, we need to update all symbol values with their new locations in memory
	for( i = 0; i < _loadSymbolCount; i++)
	{
            //add in offset of section the pointer is in to
            section=_loadSymbols[i].st_shndx;
	    //we've already placed the true address of undefined symbols into the symbol table
	    if(section==SHN_UNDEF || section==SHN_ABS) {;/*do nothing*/ }
            else
            //we only handle text, data, bss, and rodata, for now
                if(section==_textSectionNumber)
				{	
                       _loadSymbols[i].st_value+=_textRelocated; //pointer into the text section
					   Console()<<".text."<<getString(_loadSymbols[i].st_name)<<" relocated to "<<_loadSymbols[i].st_value<<"\n";
				}
                else if(section==_dataSectionNumber)
				{
                       _loadSymbols[i].st_value+=_dataRelocated; //pointer into the data section
					   Console()<<".data."<<getString(_loadSymbols[i].st_name)<<" relocated to "<<_loadSymbols[i].st_value<<"\n";
				}
                else if(section==_bssSectionNumber)
				{
                       _loadSymbols[i].st_value+=_bssRelocated; //pointer into the bss section
					   Console()<<".bss."<<getString(_loadSymbols[i].st_name)<<" relocated to "<<_loadSymbols[i].st_value<<"\n";
				}
                else if(section==_rodataSectionNumber)
				{
                       _loadSymbols[i].st_value+=_rodataRelocated; //pointer into the rodata section
					   Console()<<".rodata."<<getString(_loadSymbols[i].st_name)<<" relocated to "<<_loadSymbols[i].st_value<<"\n";
				}
                else if(section==_rodatastr18SectionNumber)
				{
                       _loadSymbols[i].st_value+=_rodatastr18Relocated; //pointer into the rodata section
					   Console()<<".rodata.str1.8."<<getString(_loadSymbols[i].st_name)<<" relocated to "<<_loadSymbols[i].st_value<<"\n";
				}
                else
                {
					Console()<<"ELFLoader: Warning! Unhandled section #"<<section<<" during "<<getString(_loadSymbols[i].st_name)<<" relocation\n";                     
                }
	
	}


	reloc * rp; //pointer to current relocation entry
	//loop through all relocation entries for the _text section (could be none)
        for( i = 0; i < _textRelocCount; i++ )
	{
		//retrieve a pointer to the current relocation entry
		rp = (reloc *)&_textRelocs[i * RELOCSIZE];
	
		if(processRelocation(rp,_text)!=Success) return 1;
	
	}

	//loop through all relocation entries for the _data section (could be none)
	for( i = 0; i < _dataRelocCount; i++ )
	{
		//retrieve a pointer to the current relocation entry
		rp = (reloc *)&_dataRelocs[i * RELOCSIZE];

		if(processRelocation(rp,_data)!=Success) return 1;

	}

	//loop through all relocation entries for the _bss section (could be none)
	for( i = 0; i < _bssRelocCount; i++ )
	{
		//retrieve a pointer to the current relocation entry
		rp = (reloc *)&_bssRelocs[i * RELOCSIZE];

		if(processRelocation(rp,_bss)!=Success) return 1;

	}

	//loop through all relocation entries for the _rodata section (could be none)
	for( i = 0; i < _rodataRelocCount; i++ )
	{
		//retrieve a pointer to the current relocation entry
		rp = (reloc *)&_rodataRelocs[i * RELOCSIZE];

		if(processRelocation(rp,_rodata)!=Success) return 1;

	}

	//loop through all relocation entries for the _rodatastr18 section (could be none)
	for( i = 0; i < _rodatastr18RelocCount; i++ )
	{
		//retrieve a pointer to the current relocation entry
		rp = (reloc *)&_rodatastr18Relocs[i * RELOCSIZE];

		if(processRelocation(rp,_rodatastr18)!=Success) return 1;

	}

	return 0;
}

//function that actually process relocation entries
//rp is a pointer to the relocation entry, and buffer is a pointer to the loaded section where the relocation should take place

int ELFLoader::processRelocation(Elf32_Rela *rp,char *buffer)
{
	int type;
	int symbol;
	int addend;
	int offset;
	int place;
	unsigned int *location;
	int temp;
	if(!rp || !buffer) return 1;
               //retrieve the relocation entry's type
                type=rp->getType();
                symbol=rp->getSymbol();
                addend=rp->r_addend;
				offset=rp->r_offset;
                place=((int)buffer)+offset;

				char tempbuffer[64];
				if(_loadSymbols[symbol].st_shndx==_textSectionNumber) strcpy(tempbuffer,".text");
				else if(_loadSymbols[symbol].st_shndx==_dataSectionNumber) strcpy(tempbuffer,".data");
				else if(_loadSymbols[symbol].st_shndx==_bssSectionNumber) strcpy(tempbuffer,".bss");
				else if(_loadSymbols[symbol].st_shndx==_rodataSectionNumber) strcpy(tempbuffer,".rodata");
				else if(_loadSymbols[symbol].st_shndx==_rodatastr18SectionNumber) strcpy(tempbuffer,".rodata.str1.8");
				else strcpy(tempbuffer,"[UNKNOWN]");

				Console()<<"Relocating reference to "<<tempbuffer<<"+"<<addend<<" ("<<getString(_loadSymbols[symbol].st_name)<<")\n";

                //switch based on relocation type
                switch(type)
                {
                case R_SPARC_NONE: break;
                                                                                                                                                                                                     
                case R_SPARC_32:
                        //get a pointer to the 4 bytes (32-bits) stored at the symbol's location in the .text section
                        location = (unsigned int *)&buffer[offset];
                        //assign the relocated value
                        *location = _loadSymbols[symbol].st_value + addend;
                        break;
                                                                                                                                                                                                     
                case R_SPARC_DISP32:
                        //get a pointer to the 4 bytes (32-bits) stored at the symbol's location in the .text section
                        location = (unsigned int *)&buffer[offset];
                        //assign the relocated value
                        *location = _loadSymbols[symbol].st_value + addend - place;
                        break;
		
				case R_SPARC_WDISP30:
                        //get a pointer to the 4 bytes (32-bits) stored at the symbol's location in the .text section
                        location = (unsigned int *)&buffer[offset];
 						//compute the relocated value
						temp = (_loadSymbols[symbol].st_value + addend - place)>>2;
						*location = ((*location)&0xc0000000) + (temp&0x3fffffff);
				break;

		case R_SPARC_HI22:
                        //get a pointer to the 4 bytes (32-bits) stored at the symbol's location in the .text section
                        location = (unsigned int *)&buffer[offset];
 			//compute the relocated value
			temp = (_loadSymbols[symbol].st_value + addend)>>10;
			*location = ((*location)&0xffc00000) + (temp&0x003fffff);
			break;

		case R_SPARC_LO10:
                        //get a pointer to the 4 bytes (32-bits) stored at the symbol's location in the .text section
                        location = (unsigned int *)&buffer[offset];
 			//compute the relocated value
			temp = (_loadSymbols[symbol].st_value + addend)&0x3ff;
			*location = ((*location)&0xffffe000) + temp;
			break;
			

                default:
                        Console()<<"Relocation type "<<type<<" not supported\n";
                        return 1;
                }


	return 0;
}

//Standard Choices Class object installation
#include "ClassInstaller.h"
DEFINECLASS("ELFLoader",ELFLoader,Object,0,0);
