#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Adapted from Linux kernel ext2 fs sources
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * EXT2Allocator.c - 
 */
#include "Assert.h"
#include "Raid.h"
#include "Ext2Allocator.h"
#include "Ext2Container.h"

int nibblemap[] = {4, 3, 3, 2, 3, 2, 2, 1, 3, 2, 2, 1, 2, 1, 1, 0};

inline int 
Ext2Allocator::set_bit(int nr, unsigned long *addr)
{
    int mask, retval;
    unsigned char *ADDR = (unsigned char *)addr;
        
    ADDR += nr >> 3;
    mask = 1 << (nr & 0x07);
    retval = (mask & *ADDR) != 0;
    *ADDR |= mask;
    return retval;
}
inline unsigned long 
Ext2Allocator::ffz(unsigned long word)
{  
    unsigned long result = 0; 
 
    while(word & 1) {
        result++;
        word >>= 1;
    }
    return result;
}


int
Ext2Allocator::test_bit(int nr, const void * addr)
{
    __const__ unsigned char *ADDR = (__const__ unsigned char *) addr;
    return (ADDR[nr >> 3] >> (nr & 7)) & 1;

}

int 
Ext2Allocator::find_next_zero_bit (void * addr, int size, int offset)
{
    unsigned long *p =(unsigned long *) addr + (offset >> 5);
    unsigned long result = offset & ~31UL;
    unsigned long tmp;
 
    if (offset >= size)
        return size;
    size -= result;
    offset &= 31UL;
    if (offset) {
        tmp = *(p++);
        tmp |= ~0UL >> (32-offset);
        if (size < 32)
            goto found_first;
        if (~tmp)
            goto found_middle;
        size -= 32; 
        result += 32;
    }
    while (size & ~31UL) {
        if (~(tmp = *(p++)))
            goto found_middle; 
        result += 32; 
        size -= 32;
        result += 32;
    }
    while (size & ~31UL) {
        if (~(tmp = *(p++)))
            goto found_middle;
        result += 32;
        size -= 32;
    }
    if (!size)
        return result;
    tmp = *p;
 
 found_first:
    tmp |= ~0UL << size;
    if (tmp == ~0UL)        /* Are any bits zero? */
        return result + size; /* Nope. */
 found_middle:
    return result + ffz(tmp);
}

int 
Ext2Allocator::find_first_zero_bit(void * addr, unsigned size)
{
    return find_next_zero_bit(addr,size, 0);
}

void 
Ext2Allocator::brelse(struct buffer_head * buf)
{
//  wait_on_buffer(buf);
    /* If dirty, mark the time this buffer should be written back */
//  set_writetime(buf, 0);
//  refile_buffer(buf);
    if (buf->b_count) {
        buf->b_count--;
        return;
    }
}
inline int 
Ext2Allocator::clear_bit(int nr, unsigned long *addr)
{
    int mask, retval;
    unsigned char *ADDR = (unsigned char *)addr;
        
    ADDR += nr >> 3;
    mask = 1 << (nr & 0x07);
    retval = (mask & *ADDR) != 0;
    *ADDR &= ~mask;
    return retval;

}

struct buffer_head * 
Ext2Allocator::bread(int block, int size)
{
    struct buffer_head *bh = new buffer_head;
    bh->b_data = new char[size];

    // FRANCIS TODO: These buffers need to be managed. Who should free them? Flush dirty to disk?
    Debug << "Request to read block: " << block << " size: " << size << "\n" << eor;
    _source->read(block * size/_source->unitSize(), size >> _source->log2UnitSize(), bh->b_data, size);

//  if (buffer_uptodate(bh))
//      return bh;
//  ll_rw_block(READ, 1, &bh);
//  wait_on_buffer(bh);
//  if (buffer_uptodate(bh))
//      return bh;
    brelse(bh);
    return bh;
}


Ext2Allocator::Ext2Allocator( MemoryObject * source, ext2filsys *sb,  
                              int log2FragmentToSector)
    : BlockAllocator( source )
{
    initDebug();
    _sb = sb;
}

unsigned long 
Ext2Allocator::ext2_count_free (struct buffer_head * map, unsigned int numchars)
{
    unsigned int i;
    unsigned long sum = 0;

    if (!map)
        return (0);   
    for (i = 0; i < numchars; i++)
        sum += nibblemap[map->b_data[i] & 0xf] +
        nibblemap[(map->b_data[i] >> 4) & 0xf];
    return (sum);
}

struct ext2_group_desc *
Ext2Allocator::get_group_desc(  ext2filsys * sb,
                                unsigned int block_group, struct buffer_head * bh )
{
    unsigned long group_desc;
    unsigned long desc;
    struct ext2_group_desc * gdp;

    if (block_group >= sb->u.ext2_sb.s_groups_count)
        Debug.printf("Ext2Allocator::get_group_desc():block_group >= s_groups_count\n");

    group_desc = block_group / sb->u.ext2_sb.s_desc_per_block;
    desc = block_group % sb->u.ext2_sb.s_desc_per_block;

    if (!sb->u.ext2_sb.s_group_desc[group_desc])
        Debug << this << "Group descriptor not loaded\n" << eor;

    gdp = (struct ext2_group_desc *)sb->u.ext2_sb.s_group_desc[group_desc]->b_data;
    if (bh)
        bh = sb->u.ext2_sb.s_group_desc[group_desc];
    return gdp + desc;
}

void 
Ext2Allocator::read_block_bitmap( ext2filsys * sb, unsigned int
                                  block_group, unsigned long bitmap_nr)
{
    struct ext2_group_desc * gdp;
    struct buffer_head * bh;
    
    gdp = get_group_desc (sb, block_group, NULL);
    bh = bread (gdp->bg_block_bitmap, sb->s_blocksize);
    if (!bh)
        Debug << this << "Cannot read block bitmap\n" << eor;
    sb->u.ext2_sb.s_block_bitmap_number[bitmap_nr] = block_group;
    sb->u.ext2_sb.s_block_bitmap[bitmap_nr] = bh;
}

int 
Ext2Allocator::load__block_bitmap( ext2filsys * sb, unsigned int
                                   block_group)
{
    int i, j;
    unsigned long block_bitmap_number;
    struct buffer_head * block_bitmap;

    if (block_group >= sb->u.ext2_sb.s_groups_count)
        Debug << "load_block_bitmap, block_group >= groups_count \n" << eor;

    if (sb->u.ext2_sb.s_groups_count <= EXT2_MAX_GROUP_LOADED) {
        if (sb->u.ext2_sb.s_block_bitmap[block_group]) {
            if  (sb->u.ext2_sb.s_block_bitmap_number[block_group] !=block_group)
                Debug.printf ("load_block_bitmap, block_group != block_bitmap_number");
            else
                return block_group;
        } else {
            read_block_bitmap (sb, block_group, block_group);
            return block_group;
        }
    }

    for (i = 0; i < sb->u.ext2_sb.s_loaded_block_bitmaps &&
         sb->u.ext2_sb.s_block_bitmap_number[i] != block_group; i++);

    if (i < sb->u.ext2_sb.s_loaded_block_bitmaps &&
        sb->u.ext2_sb.s_block_bitmap_number[i] == block_group) {
        block_bitmap_number = sb->u.ext2_sb.s_block_bitmap_number[i];
        block_bitmap = sb->u.ext2_sb.s_block_bitmap[i];
        for (j = i; j > 0; j--) {
            sb->u.ext2_sb.s_block_bitmap_number[j] =
            sb->u.ext2_sb.s_block_bitmap_number[j - 1];
            sb->u.ext2_sb.s_block_bitmap[j] =
            sb->u.ext2_sb.s_block_bitmap[j - 1];
        }
        sb->u.ext2_sb.s_block_bitmap_number[0] = block_bitmap_number;
        sb->u.ext2_sb.s_block_bitmap[0] = block_bitmap;
    } else {
        if (sb->u.ext2_sb.s_loaded_block_bitmaps < EXT2_MAX_GROUP_LOADED)
            sb->u.ext2_sb.s_loaded_block_bitmaps++;
        else
            brelse (sb->u.ext2_sb.s_block_bitmap[EXT2_MAX_GROUP_LOADED - 1]);
        for (j = sb->u.ext2_sb.s_loaded_block_bitmaps - 1; j > 0;  j--) {
            sb->u.ext2_sb.s_block_bitmap_number[j] =
            sb->u.ext2_sb.s_block_bitmap_number[j - 1];
            sb->u.ext2_sb.s_block_bitmap[j] =
            sb->u.ext2_sb.s_block_bitmap[j - 1];
        }
        read_block_bitmap (sb, block_group, 0);
    }
    return 0;
}

inline
int
Ext2Allocator::load_block_bitmap( ext2filsys * sb, unsigned int
                                  block_group)
{
    if (sb->u.ext2_sb.s_loaded_block_bitmaps > 0 &&
        sb->u.ext2_sb.s_block_bitmap_number[0] == block_group)
        return 0;
    
    if (sb->u.ext2_sb.s_groups_count <= EXT2_MAX_GROUP_LOADED && 
        sb->u.ext2_sb.s_block_bitmap_number[block_group] == block_group &&
        sb->u.ext2_sb.s_block_bitmap[block_group]) 
        return block_group;

    return load__block_bitmap (sb, block_group);
}

long
Ext2Allocator::ext2_count_free_blocks ( ext2filsys * sb)
{
    return sb->u.ext2_sb.s_es->s_free_blocks_count;
}


inline int Ext2Allocator::block_in_use( unsigned int block,
                                        ext2filsys * sb, unsigned char *map) {
    return test_bit ((block - sb->u.ext2_sb.s_es->s_first_data_block) %
                     sb->u.ext2_sb.s_blocks_per_group, map);
}

void
Ext2Allocator:: ext2_check_blocks_bitmap ( ext2filsys * sb)
{
    struct buffer_head * bh;
    unsigned long desc_count, bitmap_count, x=0;
    unsigned long desc_blocks;
    int bitmap_nr;
    struct ext2_group_desc * gdp;
    unsigned int i, j;

    desc_count = 0;
    bitmap_count = 0;
    gdp = NULL;
    desc_blocks = (sb->u.ext2_sb.s_groups_count + sb->u.ext2_sb.s_desc_per_block - 1) /
    sb->u.ext2_sb.s_desc_per_block;
    for (i = 0; i < sb->u.ext2_sb.s_groups_count; i++) {
        gdp = get_group_desc (sb, i, NULL);
        desc_count += gdp->bg_free_blocks_count;
        bitmap_nr = load_block_bitmap (sb, i);
        bh = sb->u.ext2_sb.s_block_bitmap[bitmap_nr];

        if (!test_bit (0, bh->b_data))
            Debug.printf ("Superblock in group %d is marked free", i);

        for (j = 0; j < desc_blocks; j++)
            if (!test_bit (j + 1, bh->b_data))
                Debug.printf("Descriptor block #%d in group %d is marked free", j, i);

        if (!block_in_use (gdp->bg_block_bitmap, sb,(unsigned char*) bh->b_data))
            Debug.printf("Block bitmap for group %d is marked free",i);

        if (!block_in_use (gdp->bg_inode_bitmap, sb, (unsigned char *)bh->b_data))
            Debug.printf("Inode bitmap for group %d is marked free",i);

        for (j = 0; j < sb->u.ext2_sb.s_itb_per_group; j++)
            if (!block_in_use (gdp->bg_inode_table + j, sb, (unsigned char *)bh->b_data))
                Debug.printf("Block #%d of the inode table in group %d is marked free", j, i);

        x = ext2_count_free (bh, sb->s_blocksize);
        if (gdp->bg_free_blocks_count != x)
            Debug.printf("Wrong free blocks count for group %d, stored = %d, counted = %lu", i,
                    gdp->bg_free_blocks_count, x);
        bitmap_count += x;
    }
    if (sb->u.ext2_sb.s_es->s_free_blocks_count != bitmap_count)
        Debug.printf("Wrong free blocks count in super block, stored = %lu, counted = %lu",
                (unsigned long) sb->u.ext2_sb.s_es->s_free_blocks_count, bitmap_count);
}

int 
Ext2Allocator::basicAllocate( int & blocks, int nearPhysicalUnit, int 
                              units )
{
    struct buffer_head * bh;
    struct buffer_head * bh2=NULL;
    char * p, * r=NULL;
    int  tmp;
    int bitmap_nr;
    unsigned int goal,k,i,j;
    int near = nearPhysicalUnit;
    struct ext2filsys * sb;/*assumed name*/
    struct ext2_group_desc * gdp;
    struct ext2_super_block * es;

    goal = near;

    sb = _sb;
    es = sb->u.ext2_sb.s_es; /*assuming we use only ext2 type superblock*/
    if (es->s_free_blocks_count <= es->s_r_blocks_count 
        //  &&
        //  (!fsuser() && (sb->u.ext2_sb.s_resuid != current->fsuid) &&
        //   (sb->u.ext2_sb.s_resgid == 0 ||
        //    !in_group_p (sb->u.ext2_sb.s_resgid)))
        ) {
        return 0;
    }

 repeat:
    /*
     * First, test whether the goal block is free.
     */
    if (goal < es->s_first_data_block || goal >= es->s_blocks_count)
        goal = es->s_first_data_block;
    i = (goal - es->s_first_data_block) / sb->u.ext2_sb.s_blocks_per_group;
    gdp = get_group_desc (sb, i, bh2);
    if (gdp->bg_free_blocks_count > 0) {
        j = ((goal - es->s_first_data_block) % sb->u.ext2_sb.s_blocks_per_group);
        bitmap_nr = load_block_bitmap (sb, i);
        bh = sb->u.ext2_sb.s_block_bitmap[bitmap_nr];

        Debug.printf ("goal is at %d:%d.\n", i, j);

        if (!test_bit(j, bh->b_data)) {
            goto got_block;
        }
        if (j) {
            /*
             * The goal was occupied; search forward for a free 
             * block within the next XX blocks.
             *
             * end_goal is more or less random, but it has to be
             * less than EXT2_BLOCKS_PER_GROUP. Aligning up to the
             * next 64-bit boundary is simple..
             */
            unsigned int end_goal = (j + 63) & ~63;
            j = find_next_zero_bit(bh->b_data, end_goal, j);
            if (j < end_goal)
                goto got_block;
        }
    
        Debug.printf ("Bit not found near goal\n");

        /*
         * There has been no free block found in the near vicinity
         * of the goal: do a search forward through the block groups,
         * searching in each group first for an entire free byte in
         * the bitmap and then for any free bit.
         * 
         * Search first in the remainder of the current group; then,
         * cyclicly search through the rest of the groups.
         */
        p = ((char *) bh->b_data) + (j >> 3);
//      r = memscan(p, 0, (sb->u.ext2_sb.s_blocks_per_group - j + 7) >> 3);
        k = (r - ((char *) bh->b_data)) << 3;
        if (k < sb->u.ext2_sb.s_blocks_per_group) {
            j = k;
            goto search_back;
        }
        k = find_next_zero_bit ((unsigned long *) bh->b_data, 
                                sb->u.ext2_sb.s_blocks_per_group,
                                j);
        if (k < sb->u.ext2_sb.s_blocks_per_group) {
            j = k;
            goto got_block;
        }
    }

    Debug.printf ("Bit not found in block group %d.\n", i);

    /*
     * Now search the rest of the groups.  We assume that 
     * i and gdp correctly point to the last group visited.
     */
    for (k = 0; k < sb->u.ext2_sb.s_groups_count; k++) {
        i++;
        if (i >= sb->u.ext2_sb.s_groups_count)
            i = 0;
        gdp = get_group_desc (sb, i, bh2);
        if (gdp->bg_free_blocks_count > 0)
            break;
    }
    if (k >= sb->u.ext2_sb.s_groups_count) {
        return 0;
    }
    bitmap_nr = load_block_bitmap (sb, i);
    bh = sb->u.ext2_sb.s_block_bitmap[bitmap_nr];
    // TODO: This needs to be fixed
    //  r = memscan(bh->b_data, 0, sb->u.ext2_sb.s_blocks_per_group >> 3);
    j = (r - bh->b_data) << 3;
    if (j < sb->u.ext2_sb.s_blocks_per_group)

        goto search_back;
    else
        j = find_first_zero_bit ((unsigned long *) bh->b_data,
                                 sb->u.ext2_sb.s_blocks_per_group);
    if (j >= sb->u.ext2_sb.s_blocks_per_group) {
        Debug.printf("Free blocks count corrupted for block group %d", i);
        return 0;
    }

 search_back:
    /* 
     * We have succeeded in finding a free byte in the block
     * bitmap.  Now search backwards up to 7 bits to find the
     * start of this group of free blocks.
     */
    for (k = 0; k < 7 && j > 0 && !test_bit (j - 1, bh->b_data); k++, j--);
    
 got_block:

    Debug.printf ("using block group %d(%d)\n", i, gdp->bg_free_blocks_count);

    /*
     * Check quota for allocation of this block.
     */
    /*not sure this part will work*/
    //if (sb->dq_op)
    //  if (sb->dq_op->alloc_block (inode, fs_to_dq_blocks(1, sb->s_blocksize))) {
    //      return 0;
    //  }

    tmp = j + i * sb->u.ext2_sb.s_blocks_per_group + es->s_first_data_block;

    if (set_bit (j,(long unsigned int *) bh->b_data)) {
        Debug.printf("bit already set for block %d", j);
        //  if (sb->dq_op)
        //      sb->dq_op->free_block(inode, fs_to_dq_blocks(1, sb->s_blocksize));
        goto repeat;
    }

    Debug.printf("found bit %d\n", j);

    j = tmp;

    /*not sure if we need following*/
//  mark_buffer_dirty(bh, 1);
/*  if (sb->s_flags & MS_SYNCHRONOUS) {
    ll_rw_block (WRITE, 1, &bh);
    wait_on_buffer (bh);
    }
*/
    if (j >= es->s_blocks_count) {
        Debug.printf("block >= blocks count - block_group = %d, block=%d", i, j);
        return 0;
    }
//  if (!(bh = getblk (sb->s_dev, j, sb->s_blocksize))) {
//      Debug.printf("cannot get block %d", j);
//      return 0;
//  }
//  memset(bh->b_data, 0, sb->s_blocksize);
//  mark_buffer_uptodate(bh, 1);
//  mark_buffer_dirty(bh, 1);
    brelse (bh);

    gdp->bg_free_blocks_count--;
//  mark_buffer_dirty(bh2, 1);
    es->s_free_blocks_count--;
//  mark_buffer_dirty(sb->u.ext2_sb.s_sbh, 1);
    sb->s_dirt = 1;
    return j;
}

void
Ext2Allocator::basicFree (int & blocks, int physicalUnit, int units )
{
    struct buffer_head * bh;
    struct buffer_head * bh2=NULL;
    unsigned long block_group;
    unsigned long bit;
    unsigned long i;
    int bitmap_nr;
    struct ext2filsys * sb; /*assumed name*/
    struct ext2_group_desc * gdp;
    struct ext2_super_block * es;
    unsigned int block = physicalUnit;
    unsigned long count = units;
    sb = _sb;
    if (!sb) {
        Debug.printf ("ext2_free_blocks: nonexistent device");
        return;
    }
    es = sb->u.ext2_sb.s_es;
    if (block < es->s_first_data_block || 
        (block + count) > es->s_blocks_count) {
        Debug.printf("Freeing blocks not in datazone - block = %u, count = %lu", block, count);
        return;
    }

    Debug.printf ("freeing block %lu\n", block);

    block_group = (block - es->s_first_data_block) /
    sb->u.ext2_sb.s_blocks_per_group;
    bit = (block - es->s_first_data_block) % sb->u.ext2_sb.s_blocks_per_group;
    if (bit + count > sb->u.ext2_sb.s_blocks_per_group)
        Debug.printf("Freeing blocks across group boundary Block = %u, count = %lu",block, count);
    bitmap_nr = load_block_bitmap (sb, block_group);
    bh = sb->u.ext2_sb.s_block_bitmap[bitmap_nr];
    gdp = get_group_desc (sb, block_group, bh2);

    for (i = 0; i < count; i++) {
        if (!clear_bit ((int)bit + i,(long unsigned int *) bh->b_data))
            Debug.printf("bit already cleared for block %u", block);
        else {
            //if (sb->dq_op)
            //  sb->dq_op->free_block(inode, fs_to_dq_blocks(1, sb->s_blocksize));
            gdp->bg_free_blocks_count++;
            es->s_free_blocks_count++;
        }
    }
    
    //mark_buffer_dirty(bh2, 1);
    //mark_buffer_dirty(sb->u.ext2_sb.s_sbh, 1);

    //mark_buffer_dirty(bh, 1);
    //if (sb->s_flags & MS_SYNCHRONOUS) {
    //  ll_rw_block (WRITE, 1, &bh);
    //  wait_on_buffer (bh);
    //}
    sb->s_dirt = 1;
    return;
}

#include "ClassInstaller.h"
DEFINECLASS("Ext2Allocator",Ext2Allocator,BlockAllocator,0,0);

