#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Contributing Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Contributing Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * FileSystemInterface.c - 
 */

#include "Assert.h"
#include "Raid.h"
#include "FileSystem.h"
#include "FileSystemInterface.h"
#include "FileSystemSimpleInstruments.h"
#include "MemoryObjectContainer.h"
#include "MemoryObjectDictionary.h"
#include "SymbolicLink.h"
#include "SymbolicLinkStar.h"
#include "DictionaryStack.h"
#include "MountItem.h"
#include "MountTable.h"
#include "String.h"
#include "PersistentMemoryObject.h"
#include "PersistentObjectStar.h"
#include "Class.h"
#include "ClassStar.h"
#include "PersistentArrayStream.h"
#ifdef RemoteFileSystem
#include "RemoteFileSystem.h"
#endif
#ifdef VCFILE
#include "VCDictionary.h"
#include "VCFileObject.h"
#endif
#ifdef COFFFile
#include "COFF.h"
#include "COFFDictionary.h"
#include "COFFContainer.h"
#endif

#ifdef ELFFile
//CODE ADDED BY SCOTT KIRCHER
#include "ELFDictionary.h"
#include "ELFContainer.h"
#endif


#ifdef __GNUC__
#pragma implementation
#endif

/*
 * Someday these may have some meaning.  They should be in the Process object.
 */
int UID = 0;
int GID = 0;
int UMASK = 0;

// #define JEFF

FileSystemInterface::FileSystemInterface( MemoryObjectDictionary * od,
                                          MemoryObjectContainer * oc )
    : 
    _rootContainer( oc ), 
    _root( od ), 
    _current( od ),
    _mountTable( new MountTable ),
    _openFlagsSetMask( 0 ), 
    _openFlagsClearMask( 0 )
{
    initDebug();

    Raids( RaidConstructor ) << this << "::FSI( " << od << ", "
                             << oc << " )\n" << eor;
    Assert( _root != 0 );
    Assert( _current != 0 );
    // Assert( _rootContainer != 0 );
    Assert( _mountTable != 0 );
}

FileSystemInterface::FileSystemInterface( FileSystemInterface * fsi,
                                          int copy )
    :
    _rootContainer( fsi->_rootContainer ),
    _root( fsi->_root ),
    _current( copy ? fsi->_current : fsi->_root ),
    _mountTable( (MountTable *)fsi->_mountTable ),
    _openFlagsSetMask( fsi->_openFlagsSetMask ),
    _openFlagsClearMask( fsi->_openFlagsClearMask )
//_mountTable( copy ? new MountTable( fsi->_mountTable )
//                  :   (MountTable *)fsi->_mountTable )
{
    initDebug();

    Raids( RaidConstructor ) << this << "::FSI( " << fsi << ", "
                             << copy << " )\n" << eor;
    Raids( RaidConstructor ) << this << "::FSI: _current " << _current
                             << "_root" << _root << "\n" << eor;
    Raids( RaidConstructor ) << this << "::FSI: fsi->_current "
                             << fsi->_current << "fsi->_root" << fsi->_root
                             << "\n" << eor;
    Assert( _root != 0 );
    Assert( _current != 0 );
//  Assert( _rootContainer != 0 );
    Assert( _mountTable != 0 );
}

FileSystemInterface::~FileSystemInterface()
{
    Raids( RaidConstructor ) << this << "::~FileSystemInterface()\n" << eor;
}

void
FileSystemInterface::substituteMount( PersistentMemoryObjectRef & pmo )
{
    _mountTable->substitute( pmo );
}

void
FileSystemInterface::substituteMountUp( MemoryObjectDictionaryRef & moc,
                                        char * key )
{
    _mountTable->substituteUp( moc, key );
}

FileSystemInterfaceTimer 
FileSystemInterfaceBasicFindDictionary("FileSystemInterface::basicFindDictionary() timer");

MemoryObjectDictionaryRef
FileSystemInterface::basicFindDictionary( char * path, char * & key,
                                          MemoryObjectDictionaryRef & odd,
                                          int links )
{
    SInstr( int istart = FileSystemInterfaceBasicFindDictionary.start() );

    MemoryObjectDictionaryRef od = odd;

    Raid( RaidProtected )( "%N::basicFindDictionary( path %s in %N )\n",
                           this, path, od );
#ifdef JEFF
    Console() << this << "::" << __FUNCTION__ << "(path " <<
    path << " in " << od << " )\n" << eor;
#endif /* JEFF */

    Assert( od != (MemoryObjectDictionary*)0 );

    for( ;; ) {
        Assert( od != 0);

        while( *path == '/' ) path++;

        _mountTable->substituteUp( od, path );

        key = path;
 
        char tmpKey[256];
        int i;
        for( i = 0; ( ( *path != '/' ) && ( *path != '\0' ) ); ) {
            tmpKey[i++] = *path++;
        }
        if( *path == '\0' ) break;

        tmpKey[i] = '\0';

        if( ( strcmp( tmpKey, "..") != 0 ) ||
            ( od->identity() != _root->identity() ) ) {
            int status = 0;
            PersistentMemoryObjectRef mor = od->open( tmpKey, status );

#ifdef JEFF
            Console() << this << "::" << __FUNCTION__ << "(tmpKey " <<
            tmpKey << " status " << status << " mor " << mor << " )\n" << eor;
#endif /* JEFF */

            if( mor != 0 ) {
                od = 0;
                break;
            }

            _mountTable->substitute( mor );
            substituteLink( od, mor, status, links );

            PersistentMemoryObjectStar mo = mor;
            if( mo == 0 ) break;

            if( mo->isDictionary() ) {
                od = mo->persistentObject();
                if ( od == 0 ) {
                    od = mo->asA( MemoryObjectDictionaryClass, status );
                    Assert( od != 0 );
                }
            } else if ( mo->isContainer() ) {
                MemoryObjectContainerStar oc = mo->persistentObject();
                if ( oc == 0 ) {
                    oc = mo->asA( MemoryObjectContainerClass, status );
                    Assert( oc != 0 );
                }
                od = oc->rootDictionary();
            }
        }
    }
    Raid( RaidProtected )( "%N::basicFindDictionary returns %N key %s\n",
                           this, od, key );
#ifdef JEFF
    Console() << this << "::" << __FUNCTION__ << " returns " <<
    od << " key " << key << "\n" << eor;
#endif /* JEFF */

    _mountTable->substituteUp( od, key );

    SInstr( FileSystemInterfaceBasicFindDictionary.stop( istart ) );

    return( od );
}

MemoryObjectDictionaryRef
FileSystemInterface::findDictionary( char * path, char * & key,
                                     MemoryObjectDictionary * od,
                                     int links )
{

    Raid( RaidProtected )( "%N::findDictionary( %s, , %d, %N )\n",
                           this, path, links, od );

    Assert( od != 0 );
    Assert( path != 0 );
    MemoryObjectDictionaryRef startDictionary = 0;
    if( path[0] == '/' ) startDictionary = _root;
    else startDictionary = od;
    return( basicFindDictionary( path, key, startDictionary, links ) );
}

FileSystemInterfaceTimer 
FileSystemInterfacePathOpen("FileSystemInterface::pathOpen() timer");

PersistentMemoryObjectRef
FileSystemInterface::pathOpen( char * path, int & status,
                               MemoryObjectDictionary * sd,
                               int links, int flags )
{
    SInstr( int istart = FileSystemInterfacePathOpen.start() );
    Raid( RaidProtected )( "%N::pathOpen( %s, , %N, %d )\n",
                           this, path, sd, links );

    PersistentMemoryObjectRef mo = 0;
    status = 0;
    char * key;

    MemoryObjectDictionaryStar od = findDictionary( path, key, sd, links );

    if( od == 0 ) {
        status = InvalidPath;
        SInstr( FileSystemInterfacePathOpen.stop( istart ) );
#ifdef JEFF
        Console() << this << "::" << __FUNCTION__ << " returns status " <<
        status << ", od " << od << "\n" << eor;
#endif /* JEFF */
        return( 0 );
    }
    else if( ( strcmp( key, "..") == 0 ) &&
             ( od->identity() == _root->identity()  ) ) {
        mo = od->source();
#ifdef JEFF
        Console() << __FUNCTION__ << ":1\n" << eor;
#endif /* JEFF */
    }
    else if( *key == '\0' ) {
        mo = od->open( ".", status );
#ifdef JEFF
        Console() << __FUNCTION__ << ":2\n" << eor;
#endif /* JEFF */
    }
    else  {
        mo = od->open( key, status, flags );
#ifdef JEFF
        Console() << __FUNCTION__ << ":3\n" << eor;
#endif /* JEFF */
    }

    _mountTable->substitute( mo );
    if( links > 0 ) substituteLink( od, mo, status, links );

    Raid( RaidProtected )( "%N::pathOpen return( %N ) %d\n", this, mo, status );
    SInstr( FileSystemInterfacePathOpen.stop( istart ) );
#ifdef JEFF
    Console() << this << "::" << __FUNCTION__ << " returns status " <<
    status << ", od " << od << ", mo " << mo << "\n" << eor;
#endif /* JEFF */
    return( mo );
}

FileSystemInterfaceTimer 
FileSystemInterfacePathCreate("FileSystemInterface::pathCreate() timer");

PersistentMemoryObjectRef
FileSystemInterface::pathCreate( char * path, Class * aClass, int & status,
                                 int flags )
{
    SInstr( int istart = FileSystemInterfacePathCreate.start() );
    Raid( RaidProtected )( "%N::pathCreate( %s, %N )\n", this, path, aClass );

    PersistentMemoryObjectRef mo = 0;
    status = 0;
    char * key;

    MemoryObjectDictionaryStar dictionary = findDictionary( path, key, _current );

    if( dictionary == 0 ) {
        status = InvalidPath;
        SInstr( FileSystemInterfacePathCreate.stop( istart ) );
        return( 0 );
    }
    else if( ( strcmp( key, "..") == 0 ) || ( *key == '\0' ) ) {
        status = CantCreateDot;
    }
    else {
        Debug << this << "::pathCreate() Creating " << key << " in " << dictionary <<"\n"<<eor;
        mo = dictionary->create( key, aClass, status, flags );
    }

    _mountTable->substitute( mo );

    Raid( RaidProtected )( "%N::pathCreate returns %N\n", this, mo );
    SInstr( FileSystemInterfacePathCreate.stop( istart ) );
    return( mo );
}

FileSystemInterfaceTimer 
FileSystemInterfaceOpen("FileSystemInterface::open() timer");

#ifdef MULTIMAX
extern unsigned int getKernelStackPointer();
#endif
ProxiableObjectRef
FileSystemInterface::open( char * path, Class * iClass, int & status,
                           int flags, int mode, Class * bClass )
{
    SInstr( int istart = FileSystemInterfaceOpen.start() );
    Raid( RaidMethod )( "%N::open( %s, %N, %o )\n",
                        this, path, iClass, mode );
#ifdef MULTIMAX
    Raids( RaidDetail ) << this << "::open(): stack pointer " <<
    asHex(getKernelStackPointer()) << "\n" << eor;
#endif

#ifdef JEFF
    Console()<<"++++1++++\n"<<eor;
#endif /* JEFF */

    if( iClass == 0 ) {
        status = ProtocolMismatch;
        SInstr( FileSystemInterfaceOpen.stop( istart ) );
        return( 0 );
    }
#ifdef JEFF
    Console()<<"++++2++++\n"<<eor;
#endif /* JEFF */
    flags = ( flags & ~_openFlagsClearMask ) | _openFlagsSetMask;
    PersistentMemoryObjectStar mo = 0;
#ifdef JEFF
    Console()<<"++++2a++++\n"<<eor;
#endif /* JEFF */
    if( flags & O_CREAT ) {
#ifdef JEFF
        Console()<<"++++2b++++\n"<<eor;
#endif /* JEFF */
        if( bClass == 0 ) bClass = iClass;
#ifdef JEFF
        Console()<<"++++2c++++\n"<<eor;
#endif /* JEFF */
        mo = pathCreate( path, bClass, status, flags );
    }
    else {
#ifdef JEFF
        Console()<<"++++2d++++\n"<<eor;
#endif /* JEFF */
        int follow = ( flags & O_NFOLLOW ) ? 0 : 8;
#ifdef JEFF
        Console()<<"++++2e++++\n"<<eor;
#endif /* JEFF */
        mo = pathOpen( path, status, _current, follow, flags );
    }
#ifdef JEFF
    Console()<<"++++2f++++\n"<<eor;
#endif /* JEFF */
    if( mo == 0 ) {
        SInstr( FileSystemInterfaceOpen.stop( istart ) );
        return( 0 );
    }
#ifdef JEFF
    Console()<<"++++3++++\n"<<eor;
#endif /* JEFF */

    int keyExisted = ( status == KeyExists );

    Raid( RaidDetail )( "%N::open status %d keyEx %d\n",
                        this, status, keyExisted );
#ifdef JEFF
    Console()<<StatusCodes[status]<<"\n";
    Console()<<mo<<"\n";
#endif /* JEFF */
    ProxiableObjectRef po = mo->asA( iClass, status );
#ifdef JEFF
    Console()<<StatusCodes[status]<<"\n";
    Console()<<"++++4++++\n"<<eor;
#endif /* JEFF */

    if( po == 0 )
    {
        Console()<<"Onwards!\n";
      
        if( iClass->isASubclassOf( MemoryObjectDictionaryClass ) &&
            mo->isContainer() )
        {
            Console()<<"Hither?\n";
    
            // IMPORTANT CODE CHANGE BY SCOTT KIRCHER
            // Hack to get COFF files to be recognized
            Console()<<"Checking for COFFDictionaryClass\n";

#ifdef COFFFile
            if(iClass->isASubclassOf( COFFDictionaryClass))
            {
                Console()<<"Twas!\n";
                MemoryObjectContainerStar oc = mo->persistentObject();
      
                if( oc == 0 )
                    oc = mo->asA( COFFContainerClass, status );
      
                if( oc != 0 )
                    po = oc->rootDictionary();
      
                Console()<<po<<"\n";
        
                if( ( po != 0 ) && ( !po->isKindOf( iClass ) ) ) po = 0;        
            } /* COFFDictionaryClass */
            else
#endif /* COFFFile */
#ifdef ELFFile
                if(iClass->isASubclassOf( ELFDictionaryClass))
                {
                    Console()<<"Twas!\n";
      
                    MemoryObjectContainerStar oc = mo->persistentObject();
      
                    if( oc == 0 )
                        oc = mo->asA( ELFContainerClass, status );
      
                    if( oc != 0 )
                        po = oc->rootDictionary();
      
                    Console()<<po<<"\n";
      
                    if( ( po != 0 ) && ( !po->isKindOf( iClass ) ) )
                        po = 0;       
                } /* ELFDictionaryClass */
                else
#endif /* ELFFile */
                {
                    MemoryObjectContainerStar oc = mo->persistentObject();
      
                    if( oc == 0 )
                        oc = mo->asA( MemoryObjectContainerClass, status );
      
                    if( oc != 0 )
                        po = oc->rootDictionary();
      
                    if( ( po != 0 ) && ( !po->isKindOf( iClass ) ) )
                        po = 0;
                } /* not an ELFDictionaryClass or a COFFDictionaryClass */
        } /* MemoryObjectDictionaryClass */
        else if( iClass->isASubclassOf( RecordStreamClass ) ) {
            extern PersistentClassStar PersistentArrayClass;
            extern PersistentClassStar FixedRecordFileClass;
            extern PersistentClassStar VariableRecordFileClass;
            extern ClassStar FixedRecordStreamClass;
            extern ClassStar VariableRecordStreamClass;
            Console()<<"Yon?\n";
            PersistentObjectStar p = mo->asA( PersistentArrayClass, status );
            if( p != 0 ) {
                po = PersistentArrayStreamClass->constructor( p );
            }
            else {
                p = mo->asA( FixedRecordFileClass, status );
                if( p != 0 ) {
                    po = FixedRecordStreamClass->constructor( p );
                }
                else {
                    p = mo->asA( VariableRecordFileClass, status );
                    if( p != 0 ) {
                        po = VariableRecordStreamClass->constructor( p );
                    }
                }
            }
            if( ( po != 0 ) && ( !po->isKindOf( iClass ) ) ) po = 0;
        }
    }

#ifdef JEFF
    Console()<<"++++5++++\n"<<eor;
#endif /* JEFF */

    if( ( po != 0 ) && !keyExisted && ( flags & O_CREAT ) ) {
        mo->setUserId( UID );
        mo->setGroupId( GID );
        mo->setMode( mode & ~UMASK );
    }
    else if( ( po != 0 ) && ( flags & O_TRUNC ) ) {
        mo->setNumberOfBytes( 0 );
    }

    SInstr( FileSystemInterfaceOpen.stop( istart ) );
#ifdef JEFF
    Console()<<"++++6++++\n"<<eor;
#endif /* JEFF */

    return( po );
}

FileSystemInterfaceTimer 
FileSystemInterfaceOpen1("FileSystemInterface::open1() timer");
PersistentObjectRef
FileSystemInterface::open( char * path, int & status, int & returnType,
                           int flags, int mode )
{
    SInstr( int istart = FileSystemInterfaceOpen1.start() );
    Raid( RaidMethod )( "%N::open( %s, %o )\n",
                        this, path, mode );
#ifdef MULTIMAX
    Raids( RaidDetail ) << this << "::open(): stack pointer " <<
    asHex(getKernelStackPointer()) << "\n" << eor;
#endif

    flags = ( flags & ~_openFlagsClearMask ) | _openFlagsSetMask;
    PersistentMemoryObjectStar mo = 0;
    if( flags & O_CREAT ) {
        mo = pathCreate( path, PersistentMemoryObjectClass, status, flags );
    }
    else {
        int follow = ( flags & O_NFOLLOW ) ? 0 : 8;
        mo = pathOpen( path, status, _current, follow, flags );
    }
    if( mo == 0 ) {
        SInstr( FileSystemInterfaceOpen1.stop( istart ) );
        return( (ProxiableObject*)0 );
    }

    int keyExisted = ( status == KeyExists );

    Raid( RaidDetail )( "%N::open status %d keyEx %d\n",
                        this, status, keyExisted );

    ProxiableObjectRef po = mo->asA( PersistentObjectClass, status );

    if( ( po != 0 ) && !keyExisted && ( flags & O_CREAT ) ) {
        mo->setUserId( UID );
        mo->setGroupId( GID );
        mo->setMode( mode & ~UMASK );
    }
    else if( ( po != 0 ) && ( flags & O_TRUNC ) ) {
        mo->setNumberOfBytes( 0 );
    }
    PersistentObject * p = (PersistentObject *) (ProxiableObject*)po;
    if ( p->source()->isDictionary() )
        returnType = MemoryObjectDictionaryType;
    else if ( p->isKindOf( PersistentArrayClass ) )
        returnType = PersistentArrayType;
    else 
        returnType = UnknownType;

    SInstr( FileSystemInterfaceOpen1.stop( istart ) );
    return( po );
}

int
FileSystemInterface::chdir( char * path )
{
    return( changeDictionary( _current, path ) );
}

int
FileSystemInterface::chroot( char * path )
{
    return( changeDictionary( _root, path ) );
}

int
FileSystemInterface::changeDictionary( MemoryObjectDictionaryStar & od,
                                       char * path )
{
    Raid( RaidMethod )( "%N::changeDictionary( %N, %s )\n", this, od, path );

    int status = 0;
    PersistentMemoryObjectStar mo = pathOpen( path, status, _current );
    if( mo == 0 ) return( status );
    return changeDictionary( od, mo );
}

int
FileSystemInterface::chdir( PersistentMemoryObject * mo )
{
    return( changeDictionary( _current, mo ) );
}

int
FileSystemInterface::chroot( PersistentMemoryObject * mo )
{
    return( changeDictionary( _root, mo ) );
}

int
FileSystemInterface::changeDictionary( MemoryObjectDictionaryStar & od,
                                       PersistentMemoryObject * mo )
{
    int status = 0;
    MemoryObjectDictionaryStar odDot = mo->asA( MemoryObjectDictionaryClass,
                                                status );
    if( odDot == 0 ) {
        MemoryObjectContainerStar oc = mo->asA( MemoryObjectContainerClass,
                                                status );
        if( oc != 0 ) {
            odDot = oc->rootDictionary();
        }
    }
    if( odDot == 0 ) return( status );

// LOCK HERE
    od = (MemoryObjectDictionary *)odDot->identity();
// UNLOCK HERE

    return( 0 );
}

int
FileSystemInterface::add( char * path, PersistentObject * po )
{
    Raid( RaidMethod )( "%N::add( %s, %N )\n", this, path, po );

    int status =  0;
    PersistentMemoryObjectStar mo = po->asA( PersistentMemoryObjectClass,
                                             status );
    if( mo == 0 ) return( status );

    char * key;
    MemoryObjectDictionaryStar dictionary = findDictionary( path, key,
                                                            _current );
    if( dictionary == 0 ) {
        status = InvalidPath;
    }
    else if ( !dictionary->sharesContainer( mo ) ) {
        status = DifferentContainers;
    }
    else if( ( strcmp( key, "..") == 0 ) || ( *key == '\0' ) ) {
        status = CantCreateDot;
    }
    else {
        status = dictionary->add( key, mo );
        if( status == KeyExists ) {
            status = dictionary->remove( key );
            status = dictionary->add( key, mo );
        }
    }

    return( status );
}

int
FileSystemInterface::link( char * inpath, char * outpath, int overwrite )
{
    Raid( RaidMethod )( "%N::link( %s, %s, %d )\n",
                        this, inpath, outpath, overwrite );

    int status =  0;
    PersistentMemoryObjectStar mo = pathOpen( inpath, status, _current );
    if( mo == 0 ) return( status );

    char * key;
    MemoryObjectDictionaryStar dictionary = findDictionary( outpath, key,
                                                            _current );
    if( dictionary == 0 ) {
        status = InvalidPath;
    }
    else if ( !dictionary->sharesContainer( mo ) ) {
        status = DifferentContainers;
    }
    else if( ( strcmp( key, "..") == 0 ) || ( *key == '\0' ) ) {
        status = CantCreateDot;
    }
    else {
        status = dictionary->add( key, mo );
        if( status == KeyExists ) {
            status = dictionary->remove( key );
            status = dictionary->add( key, mo );
        }
    }

    return( status );
}

int
FileSystemInterface::unlink( char * path )
{
    Raid( RaidMethod )( "%N::unlink( %s )\n", this, path );

    char * key;
    MemoryObjectDictionaryStar dictionary = findDictionary( path, key, _current );
    if( dictionary == 0 ) return( InvalidPath );

    int status = dictionary->remove( key );

    return( status );
}

int
FileSystemInterface::info( char * path, UNIXFileInfo * buffer, int follow,
                           unsigned * hints )
{
    Raid( RaidMethod )( "%N::info( %s, %x )\n", this, path, buffer );

    int status = 0;
    PersistentMemoryObjectStar mo = pathOpen( path, status, _current, follow ? 8 : 0 )
    ;
    if( mo != 0 ) {
        mo->info( buffer );
        if ( hints ) {
            *hints = mo->getCacheHints();
        }
    }
    else Assert( status != 0 );

    Raid( RaidMethod )( "%N::info return( %d )\n", this, status );

    return( status );
}

int
FileSystemInterface::mount( char * special, char * path, int rwflag )
{
    Raid( RaidMethod )( "%N::mount( %s, %s, %o )\n",
                        this, special, path, rwflag );
    
    Assert( special != 0 );
    Assert( path != 0 );

    int status = 0;

    PersistentMemoryObjectStar mp = pathOpen( path, status, _current );
    if( mp == 0 ) return( status );
#ifdef RemoteFileSystem
    extern ClassStar RemoteFileObjectClass;
    if( mp->isKindOf( RemoteFileObjectClass ) ) 
        return NotImplemented;
#endif
    MemoryObjectDictionaryStar d = mp->asA( MemoryObjectDictionaryClass, 
                                            status );
    if( d == 0 ) return( status );

    Raid( RaidDetail )( "%N::mount point %N\n", this, mp );

    PersistentMemoryObjectStar m = 0;

    switch( *special ) {
        case '!': {
            if ( _rootContainer == 0 ) {
                return( NoSuchContainer );
            }
            char * p = special + 1;
            int objectNumber;
            for( objectNumber = 0; ( '0' <= *p ) && ( *p <= '9' ); p++ ) {
                objectNumber *= 10;
                objectNumber += *p - '0';
            }

            MemoryObjectContainerStar container = 0;
            PersistentMemoryObjectStar mo = _rootContainer->open( objectNumber,
                                                                  status,
                                                                  O_NCACHE );
            Raid( RaidDetail )( "%N::mount mo %N\n", this, mo );
            if( mo != 0 ) {
                container = mo->asA( MemoryObjectContainerClass, status );
            }
            Raid( RaidDetail )( "%N::mount container %N\n", this, container );
            if( container == 0 ) {
                return( NoSuchContainer );
            }

            MemoryObjectDictionaryRef modr = container->rootDictionary();
            Raid( RaidDetail )( "%N::mount root %N\n", this, modr );
            if( modr == 0 ) return( InvalidPath );

            char * key;
            MemoryObjectDictionaryStar mod = basicFindDictionary( p, key, modr );
            Raid( RaidDetail )( "%N::mount dict %N\n", this, mod );
            if( mod == 0 ) {
                return( InvalidPath );
            }

            if( *key == '\0' ) {
                m = mod->asA( PersistentMemoryObjectClass, status );
            }
            else m = mod->open( key, status );
            Raid( RaidDetail )( "%N::mount mounted %N\n", this, m );

            if( m == 0 ) {
                return( status );
            }
        }
            break;
#ifdef VCFILE
        case '#': 
            ++special;
            char * p = special;
            m = new VCFileObject ( p, 0 );
            break;
#endif /*VCFILE*/
#ifdef RemoteFileSystem
        case ':': {
            extern PersistentMemoryObjectRef clientMount( char *, char * );

            ++special;
            int status = 0;
            m = RemoteFileSystemMount( special, path, status );
            if (status != 0) {
                return status;
            }
        }
            break;
#endif
        default: {
            m = pathOpen( special, status, _current );
            if( m == 0 ) return( status );
        }
    }

    Raid( RaidDetail )( "%N::mounted %N %N\n", this, mp, m );

    PersistentMemoryObjectRef mpr = mp;
    status = _mountTable->addMount( mpr, m );

    Raid( RaidDetail )( "%N::mount done %d\n", this, status );

    return( status );
}

int
FileSystemInterface::unmount( char * path )
{
    Raid( RaidMethod )( "%N::unmount( %s )\n", this, path );
    
    int status = 0;
    PersistentMemoryObjectStar mo = pathOpen( path, status, _current );
    if( mo != 0 ) status = _mountTable->deleteMount( mo );
    return( status );
}

int
FileSystemInterface::synchronize()
{
    Raid( RaidMethod )( "%N::synchronize()\n", this );
    _mountTable->synchronize();
    if ( _rootContainer != 0 )
        _rootContainer->synchronize();

    //Unused! int status = 0;
    PersistentMemoryObjectStar pmo = _root->source();
    Assert( pmo != 0 );
    
    MemoryObjectContainerStar oc = pmo->container();
    if (oc) 
        oc->synchronize();

    return( 0 );
}

FileSystemInterfaceTimer 
FileSystemInterfaceSubstituteLink("FileSystemInterface::substituteLink() timer");

void
FileSystemInterface::substituteLink( MemoryObjectDictionary * od,
                                     PersistentMemoryObjectRef & mo,
                                     int & status, int links )
{
    SInstr( int istart = FileSystemInterfaceSubstituteLink.start() );

    Raid( RaidProtected )( "%N::substituteLink( %N, %N, , %d )\n",
                           this, od, mo, links );

    if( mo != 0 && mo->isSymbolicLink() ) {
        SymbolicLinkStar sl = mo->persistentObject();
        if( sl == 0 ) sl = mo->asA( SymbolicLinkClass, status );
        Assert( sl != 0 );
        if( links > 0 ) {
            int size = sl->numberOfBytes();
            char * path = new char[size+1];
            //  Is this a bug? - Lup
            //sl->at( path, size );
            sl->at( 0, path, size );
            path[size] = 0;
            mo = pathOpen( path, status, od, --links );
            delete path;
        }
        else {
            status = TooManySymbolicLinks;
            mo = 0;
        }
    } 

    Raid( RaidProtected )( "%N::substituteLink = %N\n", this, mo );
    SInstr( FileSystemInterfaceSubstituteLink.stop( istart ) );
}

void
FileSystemInterface::openFlagsSetMask( unsigned mask )
{
    _openFlagsSetMask = mask;
}

unsigned
FileSystemInterface::openFlagsSetMask()
{
    return _openFlagsSetMask;
}

void
FileSystemInterface::openFlagsClearMask( unsigned mask )
{
    _openFlagsClearMask = mask;
}

unsigned
FileSystemInterface::openFlagsClearMask()
{
    return _openFlagsClearMask;
}

#include "ClassInstaller.h"
DEFINECLASS("FileSystemInterface",FileSystemInterface,ProxiableObject,0,0);
