#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * FileSystemLocks.cc
 */

#include "FileSystemLocks.h"
#include "Lock.h"
#include "Semaphore.h"
#include "ThisProcess.h"

#ifdef __GNUC__
#pragma implementation
#endif

NestedLock::NestedLock( int count, int maxNest )
     : semaphore( count )
{
	Assert( count == 0 || count == 1 );
	if (count == 0) {
		nesting = 1;
		process = thisProcess();
	} else {
		nesting = 0;
		process = 0;
	}
	maxNesting = maxNest;
	Assert( maxNesting >= 1 );
}

void NestedLock::acquire()
{
	lock.acquire();

	Process * callerProcess = thisProcess();
	if ( nesting == 0 ) {
		lock.release();
		semaphore.P();            // not expected to block
		lock.acquire();
		Assert( nesting == 0 && process == 0 );
		nesting = 1;
		process = callerProcess;
		lock.release();
		return;
	}
		
	Assert( nesting > 0 && process != 0 );
	if ( process == callerProcess ) {
		nesting++;
		lock.release();
		Assert( nesting < maxNesting );
		return;
	}

	lock.release();
	semaphore.P();                    // expected to block
	lock.acquire();
	Assert( nesting == 0 && process == 0 );
	nesting = 1;
	process = callerProcess;
	lock.release();
	return;
}

void NestedLock::release()
{
	lock.acquire();
	Process * callerProcess = thisProcess();

	Assert( process == callerProcess );
	Assert( nesting > 0 );

	nesting--;
	if ( nesting == 0 ) {
		process = 0;
		semaphore.V();
	}
	lock.release();
}

MRSWLock::MRSWLock()
     : readerSem( 0 ), writerSem( 0 )
{
	readers = 0;
	writers = 0;
	readersWaiting = 0;
	writersWaiting = 0;
}

void
MRSWLock::acquireRead()
{
	lock.acquire();
	if ( writers > 0 && writersWaiting == 0 ) {
		readersWaiting++;
		lock.release();
		readerSem.P();
	} else {
		Assert( writers == 0 );
		readers++;
		lock.release();
	}
}

void
MRSWLock::acquireWrite()
{
	lock.acquire();
	if ( readers > 0 || writers > 0 ) {
		writersWaiting++;
		lock.release();
		writerSem.P();
	} else {
		writers++;
		lock.release();
	}
}

void
MRSWLock::releaseRead()
{
	lock.acquire();
	readers--;
	Assert( writers == 0 );
	if ( readers == 0 && writersWaiting > 0 ) {
		writersWaiting--;
		writers++;
		writerSem.V();
	}
	lock.release();
}

void
MRSWLock::releaseWrite()
{
	lock.acquire();
	writers--;
	Assert( writers == 0 && readers == 0 );
	while ( readersWaiting > 0 ) {
		readersWaiting--;
		readers++;
		readerSem.V();
	}
	lock.release();
}
