#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * FixedRecordFile.c - 
 */
#include "Assert.h"
#include "Raid.h"
#include "FixedRecordFile.h"
#include "PersistentMemoryObject.h"
#include "Convert.h"
#include "PersistentClass.h"
#include "PersistentIntArray.h"
#include "PersistentArray.h"
#include "FileSystem.h"

#ifdef __GNUC__
#pragma implementation
#endif

class FileSystemInterface;

FixedRecordFile::FixedRecordFile( PersistentMemoryObject * source )
: RecordFile( source )
{
	initDebug();

	Raids( RaidConstructor ) << this << "::FixedRecordFile( "
				 << source << " )\n" << eor;
}

FixedRecordFile::~FixedRecordFile()
{
	Raids( RaidConstructor ) << this << "::~FixedRecordFile()\n" << eor;
}

int
FixedRecordFile::init( int argc, char *argv[], FileSystemInterface * )
{
	Raids( RaidMethod ) << this << "::init()\n" << eor;

	_data = PersistentArrayClass->newObject();
	_data->persist();

	if( argc == 0 ) {
		char * args[1];
		args[0] = "1";
		_data->init( 1, args );
		_recordSize = 1;
	}
	else {
		_data->init( 1, argv );
		_recordSize = atoi( argv[0] );
	}

	_source->setRecordSize( _recordSize );

	_minRecordSize = _recordSize;
	_maxRecordSize = _recordSize;

	return( 0 );
}

int
FixedRecordFile::read( unsigned int record, char * buffer,
		       unsigned int size, int & status )
{
	Raid( RaidMethod )( "%N::read( %d, , %d, )\n", this, record, size );

	if( size < _recordSize ) status = RecordSizeMismatch;
	else if( record < numberOfRecords() ) {
		_data->at( record, buffer, 1 );
		status = 0;
	}
	else status = EndOfFile;

	Raid( RaidMethod )( "%N::read return( %d ) status %d\n",
			    this, ( ( status == 0 ) ? _recordSize : 0 ), status );
	return( ( status == 0 ) ? _recordSize : 0 );
}

int
FixedRecordFile::write( unsigned int record, char * buffer,
			   unsigned int size, int & status )
{
	Raid( RaidMethod )( "%N::write( %d, , %d, )\n", this, record, size );

	if (size < _recordSize ) status = RecordSizeMismatch;
	else {
		_data->atPut( record, buffer, 1 );
		status = 0;
	}

	Raid( RaidMethod )( "%N::write return( %d ) status %d\n",
			    this, ( ( status == 0 ) ? _recordSize : 0 ), status );
	return( ( status == 0 ) ? _recordSize : 0 );
}

unsigned 
FixedRecordFile::numberOfRecords()
{
	Raid( RaidMethod )( "%N::numberOfRecords() %d\n",
			    this, _data->size() );

	return( _data->size() );
}

unsigned
FixedRecordFile::recordSize()
{
    Raids( RaidMethod ) << this << "::recordSize() "
			<< _recordSize << "\n" << eor;
    return( _recordSize );
}

ProxiableObject *
FixedRecordFileConstructor( ProxiableObject * source )
{
        if( source->isKindOf( PersistentMemoryObjectClass ) ) {
		return( new FixedRecordFile( (PersistentMemoryObject *)source ) );
        }
        else return( 0 );
}

/*
int
sizeOfFixedRecordFile()
{
	return( sizeof( FixedRecordFile ) );
}

Class *
FixedRecordFile::classOf()
{
	return( FixedRecordFileClass );
}
*/


#include "ClassInstaller.h"
DEFINEPERSISTENTCLASS("FixedRecordFile",FixedRecordFile,RecordFile,&FixedRecordFileConstructor,0);
