#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * HashedBSDDirecotry.c - 
 */
#include "Assert.h"
#include "Raid.h"
#include "FileSystem.h"
#include "PersistentMemoryObject.h"
#include "HashedBSDDirectory.h"
#include "Byte.h"
#include "String.h"
#include "Semaphore.h"

#ifdef __GNUC__
#pragma implementation
#endif

class HD_DIRECT {
	friend class HashedBSDDirectory;

	int	hd_next_hd;
	unsigned int hd_ino;
	unsigned short	hd_reclen;
	unsigned short	hd_namlen;
	union {
		char	hd_name[MaxEntrySize+1];
		int	hd_back_free;
	};
};

class direct {
	friend class HashedBSDDirectory;

	unsigned int	d_ino;
	unsigned short	d_reclen;
	unsigned short	d_namlen;
	char	d_name[MaxEntrySize + 1];
};

static const int sizeHD_Direct = 3 + sizeof( HD_DIRECT ) - MaxEntrySize;
static const int NW_HASH =  8;
static const int HALF_NW_HASH = 4;
static const int DIR_HASHSIZE = 123;
static const int DH_MAGIC = 0x22057;

class DIR_HASHTBL {
	friend class HashedBSDDirectory;

	int	dh_next_free;
	int	dh_magic;
	int	dh_reserved1;
	int	dh_back_free;
	int	dh_first_hd[DIR_HASHSIZE];
	int	dh_reserved2;
};

HashedBSDDirectory::HashedBSDDirectory( PersistentMemoryObject * source )
: MemoryObjectDictionary( source )
{
    initDebug();
    Raid( RaidConstructor )( "%N::HashedBSDDirectory( %N )\n", this, source );
    Raid( RaidDetail )( "%N::HashedBSDDirectory idNumber %d _size %d\n",
		this, source->idNumber(), _size );

    _maxKeyLength = MaxEntrySize;

    _chainNumber = -1;
    _hashTable = (DIR_HASHTBL *)_buffer;
    /* make directory file size a multiple of the directory size */
    int roundedSize = ( _size + DirectoryBlockSize - 1 ) / DirectoryBlockSize
			* DirectoryBlockSize;
    if( _size != roundedSize ) {
	_size = roundedSize;
	_source->setNumberOfBytes( _size );
    }
    if( _size == 0 ) {
	_source->setNumberOfBytes( sizeof( DIR_HASHTBL ) + DirectoryBlockSize );
        _units = _source->numberOfUnits();
	_size = _source->numberOfBytes();
	reMap();
	_hashTable = (DIR_HASHTBL *)_buffer;
	_hashTable->dh_next_free = DirectoryBlockSize;
	_hashTable->dh_magic = DH_MAGIC;
	_hashTable->dh_reserved1 = 0;
	_hashTable->dh_back_free = DirectoryBlockSize;
	for( int i = 0; i < DIR_HASHSIZE; i++ ) _hashTable->dh_first_hd[i] = 0;
	_hashTable->dh_reserved2 = 0;
	HD_DIRECT * dp = (HD_DIRECT *)&_buffer[sizeof( DIR_HASHTBL )];
	dp->hd_ino = 0;
	dp->hd_reclen = DirectoryBlockSize;
	dp->hd_namlen = 0;
	dp->hd_next_hd = 0;
	dp->hd_back_free = 0;
	
	add( ".", source );
    } else {
	reMap();
    }

    Assert( _hashTable->dh_magic == DH_MAGIC );
}

HashedBSDDirectory::~HashedBSDDirectory()
{
    Raid( RaidConstructor )( "%N::~HashedBSDDirectory\n", this );
}

// Peters function (studlier than me!!) :
int
HashedBSDDirectory::dirHashName( char * key )
{
    int i;
    Raid( RaidMethod )( "%N::dirHashName( %s )\n", this, key );

    unsigned int hash = 0;
    int nameLen = strlen( key );
    int mask;
/*  BIG-LITTLE ENDIAN PROBLEMS HERE */
    switch( nameLen % sizeof( int ) ) {
	case 0: mask = 0xffffffff;
		break;
	case 1: mask = 0xff;
		break;
	case 2: mask = 0xffff;
		break;
	case 3: mask = 0xffffff;
		break;
	default: mask = 0;
		Assert( NOTREACHED );
    }
    int len = ( nameLen + sizeof( int ) - 1) /sizeof( int );
    unsigned int * wordPtr = (unsigned int *) key;

    // Hash is the sum of the words in the filename
    if( len <= NW_HASH ) {
	for( i = 0; i < len - 1; i++ ) hash += *wordPtr++;
	hash += *wordPtr & mask;
	Raid( RaidDetail )( "%N::dirHashName %x %x\n", this, mask, *wordPtr );
    }

    // Hash is the sum of the first and last 4 words in the filename
    else {

	// Add in the first four words
	for( i = 0; i < HALF_NW_HASH; i++ ) hash += *wordPtr++;

	// Add in the last four words
	wordPtr = (unsigned int *) (key + ((len-4) * sizeof( int )));
	for( i = 0; i < HALF_NW_HASH-1; i++ ) hash += *wordPtr++;
	hash += *wordPtr & mask;
    }

    // Return result modulo 123
    Raid( RaidDetail )( "%N::dirHashName returns %d\n", this, hash % 123 );
    return( hash % 123 );
}

// search the directory file for the occurrence of filename, if it
// is found, return 1; otherwise 0.
// side effects: set previousOffset, offset, and _chainNumber
int
HashedBSDDirectory::findKey( char * key, int ) 
{
    Raid( RaidMethod )( "%N::findKey( key %s)\n", this, key );
    if( _size == 0 ) return( 0 );

    /* determine chain containing this filename */
    _chainNumber = dirHashName( key );
    Raid( RaidDetail )( "%N::findKey _chainNumber=%d\n", this, _chainNumber );

    /* search chain until found or end */
    _previousOffset = 0;
    _offset = _hashTable->dh_first_hd[_chainNumber];
    while( _offset != 0 ) {
        HD_DIRECT * nextEntry = (HD_DIRECT *) &_buffer[_offset];
        Raid( RaidDetail )( "%N::findKey id=%d reclen=%d ",
		this, nextEntry->hd_ino, nextEntry->hd_reclen );
        Raid( RaidDetail )( " namlen=%d name=%s\n",
		nextEntry->hd_namlen, nextEntry->hd_name );
        	
 	/* if found then return the inumber */
	if( (nextEntry->hd_ino != 0) 
		&& (strcmp( key, nextEntry->hd_name ) == 0) ) {
	    _idNumberOfKey = nextEntry->hd_ino;
	    Raid( RaidDetail )( "%N::findKey found @ %d\n", this, _offset );
	    return( 1 );
	}

	/* update to next entry in the chain */
	_previousOffset = _offset;
	_offset = nextEntry->hd_next_hd;
        Raid( RaidDetail )( "%N::findKey offset=%d\n", this, _offset );
    }
    return( 0 );
}

int
HashedBSDDirectory::isSpace( int size ) 
{
    Raid( RaidMethod )( "%N::isSpace(%d)\n", this, size );
    _emptyOffset = -1;

    /* if not found then locate a big enough empty space */
    if( size > 0 ) {
	_offset = _hashTable->dh_next_free;
	HD_DIRECT * nextEntry = (HD_DIRECT *) &_buffer[_offset];
	int done = 0;
	while( !done ) {

	    /* return if this space is large enough */
	    if( nextEntry->hd_reclen >= size ) {
		Raid( RaidDetail )( "%N::isSpace found %d bytes at offset %d\n",
			this, nextEntry->hd_reclen, _offset );
		_emptyOffset = _offset;
		return( 1 );
	    }

	    /* update to next free entry */
	    _offset = nextEntry->hd_next_hd;
	    nextEntry = (HD_DIRECT *) &_buffer[_offset];

	    /* done if we have finished traversing the circular free list */
	    if( _offset == _hashTable->dh_next_free ) done = 0;
	}
    }
    return( 0 );
}

int
HashedBSDDirectory::clearEntry()
{
    Raid( RaidMethod )( "%N::clearEntry()\n", this );

    /* find the entry in the hash chain */
    HD_DIRECT * freeMe = (HD_DIRECT *) &_buffer[_offset];
    int nextOffset = freeMe->hd_next_hd;

    /* remove the entry from the list of hash chains */
    if( _previousOffset == 0 ) {
	_hashTable->dh_first_hd[_chainNumber] = nextOffset;
	Raid( RaidDetail )( "%N::remove previousOffset=0\n", this );
    }

    /* otherwise remove it from the specific hash chain */
    else {
	HD_DIRECT * previousEntry = (HD_DIRECT *) &_buffer[_previousOffset];
	previousEntry->hd_next_hd = nextOffset;
	Raid( RaidDetail )( "%N::remove previousOffset=%d\n", 
			this, _previousOffset );
    }

    /* add the entry to the free list and coelesce with surrounding space */
    insertInFreeList( freeMe->hd_reclen );

    return( writeChanges() );
}

int
HashedBSDDirectory::writeChanges()
{
    int count = _source->write( 0, _units, _buffer, _size );
    if( count != _units ) return( WriteError );
    return( 0 );
}

int
HashedBSDDirectory::insertEntry( char * key, int keyLength, int idNumber )
{
    HD_DIRECT dir;
    HD_DIRECT * dp = &dir;
    HD_DIRECT * freeEntry;

    Raid( RaidMethod )( "%N::insertEntry( key %s, keyLength %d idNumber %d)\n", 
		this, key, keyLength, idNumber );

    int size = entrySize( keyLength );
    if( size > MaxEntrySize ) return( 0 );

    dir.hd_ino = idNumber;
    dir.hd_namlen = keyLength;
    dir.hd_reclen = size;

    // there was not enough room in any of the free entries so get more space
    if( !isSpace( size ) ) {
	Raid( RaidMethod )( "%N::insertEntry, no space, adding space\n", this );

	_offset = _source->numberOfBytes();
	_source->setNumberOfBytes( _offset + DirectoryBlockSize );
        int newUnits = _source->numberOfUnits();
	if( _units != newUnits )  {
	    reMap();
	    _units = newUnits;
	    _hashTable = (DIR_HASHTBL *)_buffer;
	}

	// put the entire block in free list
	insertInFreeList( DirectoryBlockSize );
	_emptyOffset = _offset;
    }
 
    /* there should now be a big enough free entry */
    Raid( RaidMethod )( "%N::insertEntry space=%d\n", this, _emptyOffset );

    // Get the free list entry 
    freeEntry = (HD_DIRECT *) &_buffer[_emptyOffset];
    Assert( freeEntry->hd_ino == 0 );

    // Determine bytes needed by new entry.  The leftover will still
    // be a part of the free list as long as it is at least 16 bytes.
    if( freeEntry->hd_reclen >= size + entrySize( 1 ) ) {
        _offset = _emptyOffset + size;
	insertInFreeList( freeEntry->hd_reclen - size );
        freeEntry->hd_reclen = size;
    }

    _offset = _emptyOffset;
    dir.hd_reclen = freeEntry->hd_reclen;
    HD_DIRECT * entry = 0;
    if( freeEntry->hd_next_hd != 0 ) {
            entry = (HD_DIRECT *) &_buffer[freeEntry->hd_next_hd];
            entry->hd_back_free = freeEntry->hd_back_free;
    }
    else {
            _hashTable->dh_back_free = freeEntry->hd_back_free;
    }
    if( freeEntry->hd_back_free != 0 ) {
            entry = (HD_DIRECT *) &_buffer[freeEntry->hd_back_free];
            entry->hd_next_hd = freeEntry->hd_next_hd;
    }
    else {
            _hashTable->dh_next_free = freeEntry->hd_next_hd;
    }

    /* move new entry into the block */
    HD_DIRECT * allocEntry = (HD_DIRECT *) &_buffer[_offset];
    allocEntry->hd_ino = dp->hd_ino;
    allocEntry->hd_reclen = dp->hd_reclen;
    allocEntry->hd_namlen = dp->hd_namlen;
    strcpy( allocEntry->hd_name, key );
    Raid( RaidDetail )( "%N::put new entry ino=%d reclen=%d ",
    		this, allocEntry->hd_ino, allocEntry->hd_reclen );
    Raid( RaidDetail )( "fn=%s offset=%d hashChain=%d\n",
		allocEntry->hd_name, _offset, _chainNumber );

    /* add the entry to its chain in the hash table */
    allocEntry->hd_next_hd = _hashTable->dh_first_hd[_chainNumber];
    _hashTable->dh_first_hd[_chainNumber] = _offset;

    return( writeChanges() );
}

int 
HashedBSDDirectory::basicKeys( char * keyBuffer, int bufferSize, int & count )
{
    Raid( RaidMethod )( "%N::keys( buffer %x, bufferSize %d ) _size %d\n",
		this, keyBuffer, bufferSize, _size );

    count = 0;
    int limit = ( bufferSize >= _size ) ? _size : bufferSize;
    char * t = keyBuffer;
    HD_DIRECT * dp = ( HD_DIRECT * ) _buffer;
    for( _offset = 512; _offset < limit; _offset += dp->hd_reclen ) {

        dp = ( HD_DIRECT * ) &_buffer[_offset];

	Raid( RaidDetail )( "%N::keys _offset %d ", this, _offset );
	Raid( RaidDetail )( "ino %d reclen %d namlen %d name %s\n",
		   dp->hd_ino, dp->hd_reclen, dp->hd_namlen, dp->hd_name );
	Assert( dp->hd_reclen > dp->hd_namlen );

	if( dp->hd_ino != 0) {
		Assert( dp->hd_namlen > 0 );

		if( t != 0 ) {
			for( char * s = dp->hd_name; *s; ) *t++ = *s++; 
			*t++ = '\n';
		}
		count++;
	}
    }
    if( t != 0 ) *t = '\0';
    if( limit == _size ) return( 0 );
    else return( TooManyKeys );
}

int 
HashedBSDDirectory::basicAssociations( char * keyBuffer, int bufferSize,
				      int & count )
{
    Raid( RaidMethod )( "%N::basicAssociations( buf %x, bufSize %d )_size %d\n",
			this, keyBuffer, bufferSize, _size );

    count = 0;
    int limit = ( bufferSize >= _size ) ? _size : bufferSize;
    char * t = keyBuffer;
    direct * u;
    HD_DIRECT * dp = ( HD_DIRECT * ) _buffer;
    for( _offset = 512; _offset < limit; _offset += dp->hd_reclen ) {

        dp = ( HD_DIRECT * ) &_buffer[_offset];

	Raid( RaidDetail )( "%N::keys _offset %d ", this, _offset );
	Raid( RaidDetail )( "ino %d reclen %d namlen %d name %s\n",
		   dp->hd_ino, dp->hd_reclen, dp->hd_namlen, dp->hd_name );
	Assert( dp->hd_reclen > dp->hd_namlen );

	if( dp->hd_ino != 0) {
		Assert( dp->hd_namlen > 0 );

		if( t != 0 ) {
		        u = (direct *)t;
				// Skip hd_next_hd, which we don't want
		        for( int i = sizeof(dp->hd_next_hd);
			    i < dp->hd_reclen; ) *t++ =
			  _buffer[_offset+i++];
			u->d_reclen -= sizeof(dp->hd_next_hd);
		    }

		count++;
	}
    }
    count = t - keyBuffer;
    if( limit == _size ) return( 0 );
    else return( TooManyKeys );
}
int 
HashedBSDDirectory::basicAssociations( char * keyBuffer, int bufferSize,
				      int & count, int & offset )
{
    Raid( RaidMethod )( "%N::basicAssociations( buf %x, bufSize %d )_size %d\n",
			this, keyBuffer, bufferSize, _size );

    count = 0;
    int limit = ( bufferSize >= _size ) ? _size : bufferSize;
    char * t = keyBuffer;
    direct * u;
    HD_DIRECT * dp = ( HD_DIRECT * ) _buffer;
    if( offset == 0 ) offset = 512;
    for( _offset = offset; _offset < limit; _offset += dp->hd_reclen ) {

        dp = ( HD_DIRECT * ) &_buffer[_offset];

	Raid( RaidDetail )( "%N::keys _offset %d ", this, _offset );
	Raid( RaidDetail )( "ino %d reclen %d namlen %d name %s\n",
		   dp->hd_ino, dp->hd_reclen, dp->hd_namlen, dp->hd_name );
	Assert( dp->hd_reclen > dp->hd_namlen );

	if( dp->hd_ino != 0) {
		Assert( dp->hd_namlen > 0 );

		if( t != 0 ) {
		        u = (direct *)t;
				// Skip hd_next_hd, which we don't want
		        for( int i = sizeof(dp->hd_next_hd);
			    i < dp->hd_reclen; ) *t++ =
			  _buffer[_offset+i++];
			u->d_reclen -= sizeof(dp->hd_next_hd);
		    }

		count++;
	}
    }
    count = t - keyBuffer;
    offset = _offset;
    if( limit == _size ) return( 0 );
    else return( TooManyKeys );
}

int
HashedBSDDirectory::entrySize( int keyLength )
{
    return( ( keyLength + sizeHD_Direct ) & ~3 );
}

int
HashedBSDDirectory::hint()
{
	return( _source->idNumber() );
}

ProxiableObject * 
HashedBSDDirectoryConstructor( ProxiableObject * source )
{
        if( source->isKindOf( PersistentMemoryObjectClass ) ) {
                return( new HashedBSDDirectory( (PersistentMemoryObject *)source ) );
        }
        else return( 0 );
}

int
HashedBSDDirectory::add( char * key, PersistentMemoryObject * mo )
{
    Raid( RaidMethod )( "%N::add( key %s, %N )\n", this, key, mo );

    int status = 0;
    int keyLength = strlen( key );
    int entryLength = entrySize( keyLength );

    _mutex->P();

    if     ( !_valid )				status = DirectoryInvalid;
    else if( keyLength > _maxKeyLength )	status = KeyTooBig;
    else {
	findKey( key, entryLength );
        status = insertEntry( key, keyLength, mo->idNumber() );
    }
    if( status == 0 ) mo->links( 1 );

    _mutex->V();
    return( status );
}


void
HashedBSDDirectory::insertInFreeList( int size )
{
	Raid( RaidMethod )( "%N::insertInFreeList( size %d )\n", this, size );

	HD_DIRECT * freeEntry = (HD_DIRECT *) &_buffer[_offset];
	freeEntry->hd_ino = 0;
	freeEntry->hd_namlen = 0;
	freeEntry->hd_reclen = size;
	freeEntry->hd_next_hd = _hashTable->dh_next_free;
	freeEntry->hd_back_free = 0;

	int oldNextFree = _hashTable->dh_next_free;
	_hashTable->dh_next_free = _offset;

	if( _hashTable->dh_back_free == 0 ) {
		_hashTable->dh_back_free = _offset;
	}
	else {
		freeEntry = (HD_DIRECT *) &_buffer[oldNextFree];
		freeEntry->hd_back_free = _offset;
	}
}



#include "ClassInstaller.h"
DEFINEPERSISTENTCLASS("HashedBSDDirectory",HashedBSDDirectory,MemoryObjectDictionary,&HashedBSDDirectoryConstructor,0);
