#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * IndexedRecordFile.c - 
 */
#include "Assert.h"
#include "Raid.h"
#include "IndexedRecordFile.h"
#include "PersistentMemoryObject.h"
#include "Byte.h"
#include "PersistentClass.h"
#include "FileIndex.h"
#include "PersistentArray.h"
#include "PersistentIntArray.h"
#include "FileSystem.h"
#include "Convert.h"

#ifdef __GNUC__
#pragma implementation
#endif

class FileSystemInterface;

IndexedRecordFile::IndexedRecordFile( PersistentMemoryObject * source )
: RelativeRecordFile( source )
{
	initDebug();

	Raids( RaidConstructor ) << this << "::IndexedRecordFile( "
				 << source << " )\n" << eor;
}

IndexedRecordFile::~IndexedRecordFile()
{
	Raids( RaidConstructor ) << this << "::~IndexedRecordFile()\n" << eor;
}

void
IndexedRecordFile::flush()
{
	Raids( RaidReference ) << this << "::flush()\n" << eor;
	_data.flushStar();
	_control.flushStar();
	for( int i = 0; i < 8; i++ ) _index[i].flushStar();
}

int
IndexedRecordFile::init( int argc, char *argv[], FileSystemInterface * )
{
	Raids( RaidMethod ) << this << "::init( " << argc << " )\n" << eor;

	char * args[1];
	args[0] = "1";

	_data = PersistentArrayClass->newObject();
	_data->persist();
	if( argc == 0 ) {
		_data->init( 1, args );
		_recordSize = 1;
	}
	else {
		_data->init( 1, argv );
		_recordSize = atoi( argv[0] );
	}

	_minRecordSize = _recordSize;
	_maxRecordSize = _recordSize;

	_control = PersistentIntArrayClass->newObject();
	_control->persist();
	_control->init( 1, args );

	if( argc > 1 ) {
		char string[12];
		args[0] = string;

		argc--;
		argv++;
		_indices = argc / 2;
		for( unsigned i = 0; i < _indices; i++ ) {
			_start[i] = atoi( *argv++ );
			_length[i] = atoi( *argv++ );
			if( ( ( _start[i] + _length[i] ) > _recordSize ) ||
			    ( _length[i] == 0 ) ) {
				_indices--;
				i--;
			}
			else {
				_index[i] = FileIndexClass->newObject();
				_index[i]->persist();
        			int d = digits( _length[i] );
        			itoa( _length[i], &string[d] );
				_index[i]->init( 1, args );
			}
		}
	}

	return( 0 );
}

int
IndexedRecordFile::read( unsigned int record, char * buffer,
			  unsigned int size, int & status )
{
	Raid( RaidMethod )( "%N::read( %d, , %d, )\n", this, record, size );

        if( size < _recordSize ) status = RecordSizeMismatch;
        else if( record < numberOfRecords() ) {
                if( _control->at( record ) ) {
			_data->at( record, buffer, 1 );
                	status = 0;
		}
		else status = RecordNotFound;
        }
        else status = EndOfFile;

	Raid( RaidMethod )( "%N::read return( %d ) status %d\n",
			    this, ( ( status == 0 ) ? _recordSize : 0 ), status );
	return( ( status == 0 ) ? _recordSize : 0 );
}

int
IndexedRecordFile::write( unsigned int record, char * buffer,
			   unsigned int size, int & status )
{
	Raid( RaidMethod )( "%N::write( %d, , %d, )\n", this, record, size );

	status = 0;

	if( size < _recordSize ) status = RecordSizeMismatch;
	else {
		if( !_control->at( record ) ) {
			for( unsigned i = 0; i < _indices; i++ ) {
				char * key = buffer + _start[i];
				_index[i]->atPut( key, record, status );
				if( status != 0 ) break;
			}
			if( status == 0 ) {
				_control->atPut( record, 1 );
				_data->atPut( record, buffer, 1 );
			}
		}
        	else status = RecordExists;
	}

	Raid( RaidMethod )( "%N::write return( %d ) status %d\n",
			    this, ( ( status == 0 ) ? _recordSize : 0 ), status );
	return( ( status == 0 ) ? _recordSize : 0 );
}

ProxiableObject *
IndexedRecordFileConstructor( ProxiableObject * source )
{
        if( source->isKindOf( PersistentMemoryObjectClass ) ) {
		return( new IndexedRecordFile( (PersistentMemoryObject *)source ) );
        }
        else return( 0 );
}

/*
int
sizeOfIndexedRecordFile()
{
	return( sizeof( IndexedRecordFile ) );
}

Class *
IndexedRecordFile::classOf()
{
	return( IndexedRecordFileClass );
}
*/


#include "ClassInstaller.h"
DEFINEPERSISTENTCLASS("IndexedRecordFile",IndexedRecordFile,RelativeRecordFile,&IndexedRecordFileConstructor,0);
