#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Author: Daniel E. Weber (weber@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * MailDictionary.cc -
 */

#include "Assert.h"
#include "Raid.h"
#include "FileSystem.h"
#include "PersistentMemoryObject.h"
#include "MailDictionary.h"
#include "String.h"

#ifdef __GNUC__
#pragma implementation
#endif

const int DirNameSize = 14;

class direct {
        friend class MailDictionary;

        short   d_ino;
        char    d_name[DirNameSize];
};

/*
 * MailDictionary allows directory entries to be located in,
 * added to, and removed from a Mail directory file.
 */
MailDictionary::MailDictionary( PersistentMemoryObject * source )
: ( source )
{
    initDebug();
    Raid( RaidConstructor )( "%N::MailDictionary( %N )\n", this, source );
    Raid( RaidDetail )( "%N::MailDictionary idNumber %d _size %d\n",
             this, source->idNumber(), _size );

    _maxKeyLength = DirNameSize;
}


MailDictionary::~MailDictionary()
{
    Raid( RaidConstructor )( "%N::~MailDictionary\n", this );
}


/*
 * search the entire directory for an occurrence of key
 * if found, return 1; otherwise 0
 * side effects: set _offset dir entry ofs and
 * findKey will also try to find an empty hole to hold a new directory entry,
 * if needSize is positive.
 */
int
MailDictionary::findKey( char * key, int needSize )
{
    direct * dp;

    Raid( RaidMethod )( "%N::findKey( key %s, needSize %d )\n",
                this, key, needSize );

    int StillLooking = ( needSize > 0 );
    _emptyOffset = -1;

    for( _offset = 0; _offset < _size; _offset += sizeof( direct ) ) {
        dp = ( direct * ) &_buffer[_offset];

        Raid( RaidDetail )( "%N::findKey ofs %d ", this, _offset);
        Raid( RaidDetail )( "ino %d key %s\n", dp->d_ino, dp->d_name );

        /* check to see if a hole */
        if( ( StillLooking ) && ( dp->d_ino == 0 ) ) {
            _emptyOffset = _offset;
            StillLooking = 0;
            Raid( RaidDetail )( "%N::findKey space @ %d\n", this, _offset );
        }

        /* return if key is found */
        if( ( dp->d_ino != 0 ) && ( strcmp( key, dp->d_name ) == 0 ) ) {
            _idNumberOfKey = dp->d_ino;
            Raid( RaidDetail )( "%N::findKey found @ %d\n", this, _offset );
            return( 1 );
        }
    }
    return( 0 );
}


/*
 *      return positive count of keys if the buffer was big enough,
 *      negative count otherwise.
 *      optimize and improve later, perhaps via memory-mapping a precalculated
 *      list-o-keys.
 */
int
MailDictionary::basicKeys( char * keyBuffer, int bufferSize, int & count )
{
    Raid( RaidMethod )( "%N::keys( buffer %x, bufferSize %d ) _size %d\n",
                this, keyBuffer, bufferSize, _size );

    count = 0;
    int limit = ( bufferSize >= _size ) ? _size : bufferSize;
    char * t = keyBuffer;
    direct * dp = ( direct * ) _buffer;
    for( _offset = 0; _offset < limit; _offset += sizeof( direct ) ) {

        dp = ( direct * ) &_buffer[_offset];

        Raid( RaidDetail )( "%N::keys ofs %d ", this, _offset);
        Raid( RaidDetail )( "ino %d key %s\n", dp->d_ino, dp->d_name );

        if( dp->d_ino != 0) {
                if( t != 0 ) {
                        for( char * s = dp->d_name; *s; ) *t++ = *s++;
                        *t++ = '\n';
                }
                count++;
        }
    }
    if( t != 0 ) *t = '\0';
    if( limit == _size ) return( 0 );
    else return( TooManyKeys );
}


int
MailDictionary::entrySize( int )
{
    return( sizeof( direct ) );
}

ProxiableObject * 
MailDictionaryConstructor( ProxiableObject * source )
{
        if( source->isKindOf( PersistentMemoryObjectClass ) ) {
                return( new MailDictionary( (PersistentMemoryObject *)source ) );
        }
        else return( 0 );
}


#include "ClassInstaller.h"
DEFINEPERSISTENTCLASS("MailDictionary",MailDictionary,MemoryObjectDictionary,&MailDictionaryConstructor,0);
//DEFINECLASS("MailDictionary",MailDictionary,MemoryObjectDictionary,&MailDictionaryConstructor,0);
