#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Author: Daniel E. Weber (weber@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
#include "MailMessageContainer.h"
#include "FileObject.h"
#include "RootEntry.h"
#include "FileSystem.h"
#include "String.h"
#include "Raid.h"

#ifdef __GNUC__
#pragma implementation
#endif

FileObject *
MailMessageContainer::basicOpen( int id, int & status, int flags )
{
   Raid( RaidMethod )( "%N::basicOpen( inumber %d )\n", this, id );

   if ( id < 1 ) {
	 status = IdNumberOutOfRange;
	 return ( 0 );
   }
   else {
	status = 0;
	if( id == 1 ) {
		extern ClassStar MailMessageDictionaryClass;
		return( new RootEntry( _source->numberOfBytes(), 0,
				       id, this, _source,
				       MailMessageDictionaryClass ) );
	}
   	else {
		return( new FileObject( _sizes[id], 0,
					id, this,
					_source, _offsets[id] ) );
	}
   }
}


/*
 * Destroy all of the stored information.  First, each of the individual
 * attributes for each message.  Then the entire message structure itself.
 */
MailMessageContainer::~MailMessageContainer()
{
	Raid( RaidConstructor )( "%N::~MailMessageContainer()\n", this );

	if( _valid ) {
		delete _offsets;
		delete _sizes;
	}
}

MailMessageContainer::MailMessageContainer( PersistentMemoryObject * source )
: ( source )
{
   initDebug();

   Raid( RaidConstructor )( "%N::MailMessageContainer( %N )\n", this, _source );

/*
 * If the file is empty, set all information to be NULL.
 */
    unsigned int bytes = _source->numberOfBytes();
    if( bytes == 0 ) {
        _valid = 0;
        initializeArrays( 0 );
    }
    else {
        int units = _source->numberOfUnits();
        int bufferSize = units << _source->log2UnitSize();
        char * buffer = new char[bufferSize];
        _source->read( 0, units, (char *)buffer, bufferSize);

	int objects = 0;
        if( strncmp( buffer, "From ", 5 ) != 0 ) {
            _valid = 0;
            initializeArrays( 0 );
        }
        else {
	    _valid = 1;
	    for( int pos = 0; pos < bytes; pos++ ) {
		if( buffer[pos] == '\n' ) {
		    if( buffer[++pos] == '\n' ) break;
		    if( buffer[pos] > ' ' ) objects++;
		}
	    }
	    int endOfItems = pos;

	    objects += 3; // empty, root, ..., message body
	    _offsets = new int[objects];
	    _sizes = new int[objects];
	    initializeArrays( objects );

	    for( int i = 0; i < objects; i++ ) {
		_offsets[i] = 0;
		_sizes[i] = 0;
	    }

	    objects = 2;
	    int foundItem = 0;
	    for( pos = 0; pos < endOfItems; pos++ ) {
		if( buffer[pos] == '\n' ) {
		    if( buffer[++pos] > ' ' ) {
			_sizes[objects-1] = pos - _offsets[objects-1];
			foundItem = 1;
		    }
		}
		if( foundItem && ( buffer[pos] == ':' ) ) {
		    _offsets[objects++] = pos + 2;
		    foundItem = 0;
		}
	    }
	    _sizes[objects-1] = pos - 1 - _offsets[objects-1];
	    _offsets[objects] = pos;
	    _sizes[objects] = bytes - pos;
	}

	delete buffer;
   }
}

/*
 * The source will be a new MailMessageContainer.
 */
ProxiableObject * 
MailMessageContainerConstructor( ProxiableObject * source )
{
    if ( source->isKindOf( PersistentMemoryObjectClass ) ) {
	return( new MailMessageContainer( (PersistentMemoryObject *)source ) );
    }
    else return( 0 );
}


#include "ClassInstaller.h"
DEFINEPERSISTENTCLASS("MailMessageContainer",MailMessageContainer,MemoryObjectContainer,&MailMessageContainerConstructor,0);
//DEFINECLASS("MailMessageContainer",MailMessageContainer,MemoryObjectContainer,&MailMessageContainerConstructor,0);
