#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Author: Daniel E. Weber (weber@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * MailMessageDictionary.cc -
 */
#include "Assert.h"
#include "Raid.h"
#include "FileSystem.h"
#include "MailMessageDictionary.h"
#include "PersistentMemoryObject.h"
#include "MemoryObjectContainer.h"
#include "String.h"

#ifdef __GNUC__
#pragma implementation
#endif

/*
 * MailMessageDictionary allows directory entries to be located in,
 * added to, and removed from a MailMessage directory file.
 */
MailMessageDictionary::MailMessageDictionary( PersistentMemoryObject * source )
: ( source )
{
    initDebug();
    Raid( RaidConstructor )( "%N::MailMessageDictionary( %N )\n", this, source );

    _maxKeyLength = 128;
}

MailMessageDictionary::~MailMessageDictionary()
{
    Raid( RaidConstructor )( "%N::~MailMessageDictionary\n", this );
}

/*
 * search the entire directory for an occurrence of key
 * if found, return 1; otherwise 0
 * side effects: set _offset dir entry ofs and
 * findKey will also try to find an empty hole to hold a new directory entry,
 * if needSize is positive.
 */
int
MailMessageDictionary::findKey( char * key, int )
{
    Raid( RaidMethod )( "%N::findKey( key %s, )\n", this, key );

    _idNumberOfKey = 0;
    _emptyOffset = -1;
    _offset = -1;

    if( strcmp( key, "." ) == 0 ) {
	_idNumberOfKey = _source->idNumber();
    }

    int line = 1;
    int keyLen = strlen( key ); 
    for( int i = 0; i < _size; i++ ) {
	if( _buffer[i] == '\n' ) {
	    if( _buffer[++i] == '\n' ) break;
	    if( _buffer[i] > ' ' ) {
	    	line++;
		if( strncmp( key, &_buffer[i], keyLen ) == 0 ) {
		    _idNumberOfKey = line;
		    break;
		}
	    }
	}
    }
    if( ( _idNumberOfKey == 0 )  && ( strcmp( key, "MessageBody" ) == 0 ) ) {
	_idNumberOfKey = line + 1;
    }
    return( _idNumberOfKey != 0 );
}

PersistentMemoryObjectRef
MailMessageDictionary::open( char * key, int & status )
{
	Raid( RaidMethod )( "%N::open( %s )\n", this, key );

	PersistentMemoryObjectRef mo = 0;

	if( strcmp( key, ".." ) == 0 ) {
	    PersistentMemoryObjectStar m = _container->asA(
					PersistentMemoryObjectClass, status );
	    if( m != 0 ) {
		MemoryObjectContainerStar oc = m->container();
		mo = oc->asA( PersistentMemoryObjectClass, status );
	    }
	}
	else mo = MemoryObjectDictionary::open( key, status );

	return( mo );
}

/*
 *      return positive count of keys if the buffer was big enough,
 *      negative count otherwise.
 *      optimize and improve later, perhaps via memory-mapping a precalculated
 *      list-o-keys.
 */
int
MailMessageDictionary::basicKeys( char * keyBuffer, int bufferSize, int & count )
{
    Raid( RaidMethod )( "%N::keys( buffer %x, bufferSize %d ) _size %d\n",
			this, keyBuffer, bufferSize, _size );

    int limit = bufferSize >= _size ? _size : bufferSize;
// FIX check for extra 5 chars
    strcpy( keyBuffer, ".\n..\nMessageBody\n" );
    count = 3;
    char * t = &keyBuffer[strlen( keyBuffer )];

    for( int i = 0; i < _size; i++ ) {
	if( _buffer[i] == '\n' ) {
	    if( _buffer[++i] == '\n' ) break;
	    if( _buffer[i] > ' ' ) {
		count++;
		while( ( i < _size ) && ( _buffer[i] != ':' ) ) {
		    *t++ = _buffer[i++]; //if( _buffer[i] == '\n' ) break;
		}
		*t++ = '\n';
	    }
	}
    }
    *t++ = '\0';

//FIX check this
    if( limit == _size ) return( 0 );
    else return( TooManyKeys );
}

ProxiableObject * 
MailMessageDictionaryConstructor( ProxiableObject * source )
{
        if( source->isKindOf( PersistentMemoryObjectClass ) ) {
                return( new MailMessageDictionary( (PersistentMemoryObject *)source ) );
        }
        else return( 0 );
}


#include "ClassInstaller.h"
DEFINEPERSISTENTCLASS("MailMessageDictionary",MailMessageDictionary,MemoryObjectDictionary,&MailMessageDictionaryConstructor,0);
//DEFINECLASS("MailMessageDictionary",MailMessageDictionary,MemoryObjectDictionary,&MailMessageDictionaryConstructor,0);
