#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Author: Daniel E. Weber (weber@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * MailRootEntry.cc - Mail Root Directory Entry (inode) code
 */

#include "Assert.h"
#include "Byte.h"
#include "Raid.h"
#include "FileSystem.h"
#include "MailRootEntry.h"

#ifdef __GNUC__
#pragma implementation
#endif

extern ClassStar MailDictionaryClass;

MailRootEntry::MailRootEntry( int inumber,
			      MemoryObjectContainer * container,
			      PersistentMemoryObject * source,
			      int messages )
: ( ( messages + 2 ) * sizeof(direct), 0, inumber, container,
      source, MailDictionaryClass, 0 )
{
    initDebug();
    Raid( RaidConstructor )( "%N::MailRootEntry( %d )\n", this, messages );

    _entries = messages + 2;

/*
 * Create the direct array for individual messages.
 */
    _mailDirectoryArray = new direct[ _entries ];

    for( int i = 0; i < _entries; i++ ) {
	_mailDirectoryArray[i].d_ino = 0;
	for (int l = 0; l < DirNameSize; l++) {
	    _mailDirectoryArray[i].d_name[l] = '\0';
	}
    }

/*
 * Set first two entries to be "." and ".." corresponding to root and parent.
 */
    _mailDirectoryArray[0].d_ino = 1;
    _mailDirectoryArray[0].d_name[0] = '.';
    _mailDirectoryArray[1].d_ino = 1;
    _mailDirectoryArray[1].d_name[0] = '.';
    _mailDirectoryArray[1].d_name[1] = '.';
 
/*
 * Transform each message number into the message name.  Messages start at 
 * position 2 in the direct array.  First the inode number is set, then the
 * number of characters in the name is determined.  Finally, the number is
 * stored as a character by filling up the name backwards (least significant
 * digit -> most significant).
 */
    for( int msg = 2; msg < _entries; msg++ ) {
	int i,j;
	int number = msg - 1;

	_mailDirectoryArray[ msg ].d_ino = number + 1;
	for (i = 1, j = -1; i <= number; i *= 10, j++);
	for ( ; j >= 0; j--) {
	    _mailDirectoryArray[ msg ].d_name[j] = "0123456789"[number % 10];
	    number /= 10;
	}
    }
}


/*
 * Delete the _mailDirectoryArray containing the id number and the name of 
 * the file.
 */
MailRootEntry::~MailRootEntry()
{
   Raid( RaidConstructor )( "%N::~MailRootEntry()\n", this );

   delete _mailDirectoryArray;
}


/*
 * read pulls bytes from the _MailDictionaryArray (the array containing
 * the individual mail names).
 */
int
MailRootEntry::read( unsigned int offset, int length, char * buf, int nbytes )
{
   Raid( RaidMethod )( "%N::read( %d, %d, %x )\n",
			this, offset, length, buf );

   int bytes = numberOfBytes();
   char *mailDirectoryArray = (char *) _mailDirectoryArray;
 
   if (offset > bytes) return( 0 );
   else {
      if ( (offset + length) > bytes ) length = bytes - offset;

      if ( length > nbytes )
	    length = nbytes;

      Raid( RaidDetail )( "%N::read %x %d\n",
			  this, &mailDirectoryArray[offset], length );

      ByteCopy( (char *) buf, &mailDirectoryArray[offset], length );
      return( length ); // by Xiao
   }
}




#include "ClassInstaller.h"
DEFINECLASS("MailRootEntry",MailRootEntry,RootEntry,0,0);
