#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * MemoryObjectPartition.c - a view of another memory object.
 */

#include "AIXContainer.h"
#include "Assert.h"
#include "BSDContainer.h"
#include "Class.h"
#include "LogContainer.h"
#include "MSDOSContainer.h"
#include "MadContainer.h"
#include "POSHContainer.h"
#include "MemoryObjectPartition.h"
#include "Ext2Container.h"
#include "Cramfs.h"
#include "NameServer.h"
#include "Raid.h"
//.#pragma implementation "SVIDContainer.h"
#include "SVIDContainer.h"

#ifdef __GNUC__
#pragma implementation
#endif

extern void *__VDISK_BEGIN__;

// #define USELOCK

MemoryObjectPartition::MemoryObjectPartition( unsigned int numberOfUnits,
                                              int log2UnitSize,
                                              int idNumber,
                                              MemoryObjectContainer * container,
                                              PersistentMemoryObject * source,
                                              int offset )
    : FileObject( numberOfUnits, log2UnitSize, idNumber, container, source, offset )
   
{
    initDebug();
    Raid( RaidConstructor )( "%N::MemoryObjectPartition()\n", this );

#ifndef USELOCK
    Assert( _moLock != 0 );
    delete _moLock;
    _moLock = 0;
#endif

#ifdef FLATTEN
    _baseOffset = _offset;
    _baseShift = _unitShift;
    _baseSource = source;
    Assert( _baseShift >= 0 );
/*
  Console() << this << " baseSource " << _baseSource << " baseOffset " 
  << _baseOffset << " baseShift " << (int) _baseShift 
  << "\n" << eor;
*/
#endif /* FLATTEN */
 
    char * buffer = new char[unitSize()];
    int unitMask = unitSize() - 1;

    int status = read( offsetToUnit( FsMAGICOFFSET ), 1, 
                       buffer, unitSize() );
    int magic = * (int *) &buffer[FsMAGICOFFSET & unitMask];
    if( magic == FsMAGIC )
        _supports = StandardNameServer-> lookup( "SVIDContainer", ClassClass);
    else {
        magic = *((int *)buffer);
        Debug << "Magic: " << asHex(magic) << "\n" << eor;
        if(magic == (int)CramfsMagicNumber) {
            Debug << "Cramfs detected.\n" << eor;
            _supports = StandardNameServer->lookup("CramfsContainer", ClassClass);
        } else {
            magic = * (int *) &buffer[FlMAGICOFFSET & unitMask];
            if( magic == FlMAGIC ) 
                _supports = StandardNameServer-> lookup( "LogContainer", ClassClass);
            else {
                status = read( offsetToUnit( FaMAGICOFFSET ), 1, 
                               buffer, unitSize() );
                magic = * (int *) &buffer[FaMAGICOFFSET & unitMask];
                if( magic == FaMAGIC ) _supports = StandardNameServer->
                                       lookup( "AIXContainer", ClassClass);
                else {
                    status = read( offsetToUnit( FbMAGICOFFSET ), 1, 
                                   buffer, unitSize() );
                    magic = * (int *) &buffer[FbMAGICOFFSET & unitMask];
                    if( magic == FbMAGIC ) _supports = StandardNameServer->
                                           lookup( "BSDContainer", ClassClass);
                    else {
                        status = read( offsetToUnit( FmMAGICOFFSET ), 1,
                                       buffer, unitSize() );
                        magic = * (int *) &buffer[FmMAGICOFFSET & unitMask];
                        if( magic == FmMAGIC ) _supports = StandardNameServer->
                                               lookup( "MadContainer", ClassClass);
                        else {
                            status = read( offsetToUnit( FpMAGICOFFSET ),1,
                                           buffer, unitSize() );
                            magic = * (int *) &buffer[FpMAGICOFFSET & unitMask];
                            if( magic == FpMAGIC )
                                _supports = StandardNameServer->lookup( "POSHContainer",
                                                                        ClassClass); 
                            else
                            {
                                status = read( offsetToUnit( FeMAGICOFFSET ),1,
                                               buffer, unitSize() );
                                magic = * (int *) &buffer[FeMAGICOFFSET & unitMask];

                                unsigned short smagic;
                                smagic = * (short *) &buffer[FeMAGICOFFSET & unitMask];

                                if( smagic == FeMAGIC )
                                {
                                    Debug << "Ext2 recognized!" << eor;
                                    _supports = StandardNameServer->lookup(
                                        "Ext2Container", ClassClass); 
                                }
                                else
                                { 
                                    Debug << "No Ext2!" << eor;
                                    _supports = StandardNameServer->lookup(
                                        "MSDOSContainer", ClassClass);
                                }
                            }
                        }
                    }
                }
            }
        }
    }

    delete buffer;
}

Class *
MemoryObjectPartition::supports( Class * aClass )
{
    Raid( RaidProtected )( "%N::supports( %N )\n", this, aClass );

    Class * supportedClass = 0;

    if( _supports != 0 && _supports->isASubclassOf( aClass ) ) {
        supportedClass = _supports;
    }

    Raid( RaidProtected )( "%N::supports() returns %N\n",
                           this, supportedClass );

    return( supportedClass );
}

int
MemoryObjectPartition::isContainer()
{
    return( _supports != 0 );
}

void
MemoryObjectPartition::basicSynchronizeClosure
( PersistentMemoryObject::BasicSynchronizeAsyncClosure * c )
{
    Assert( NOTREACHED );
}

void
MemoryObjectPartition::basicSynchronize( IOAsyncClosure * c )
{
    MemoryObject::basicSynchronize( c );
}

void
MemoryObjectPartition::basicSynchronize( unsigned from, unsigned to,
                                         IOAsyncClosure * c )
{
    MemoryObject::basicSynchronize( from, to, c );
}

#include "ClassInstaller.h"
DEFINECLASS("MemoryObjectPartition",MemoryObjectPartition,FileObject,0,0);
