#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
#include "Assert.h"
#include "PersistentMemoryObject.h"
#include "MountItem.h"

#ifdef __GNUC__
#pragma implementation
#endif

MountItem::MountItem( MountItem * next,
		      PersistentMemoryObjectRef & mountPoint,
		      PersistentMemoryObjectRef & mounted )
: _mountPoint( mountPoint ), _mounted( mounted )
{
	Assert( mountPoint != 0);
	Assert( mounted != 0);
	_next = next;
	if( _next != 0 ) _next->setPrevious( this );
	_previous = 0;
}

MountItem::~MountItem()
{
	Assert( _mountPoint != 0 );
	Assert( _mounted != 0 );
	if( _previous != 0 ) _previous->setNext( _next );
	if( _next != 0 ) _next->setPrevious( _previous );
}

MountItem *
MountItem::lookupMountPoint( PersistentMemoryObject * mo )
{
	if     ( _mountPoint->identity() == mo->identity() ) return( this );
	else if( _next       != 0  ) return( _next->lookupMountPoint( mo ) );
	else 			     return( 0 );
}

MountItem *
MountItem::lookupMounted( PersistentMemoryObject * mo )
{
	if     ( _mounted->identity() == mo->identity() ) return( this );
	else if( _next    != 0  ) return( _next->lookupMounted( mo ) );
	else 			  return( 0 );
}
