#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * POSHApply.c - 
 */
#include "POSHApply.h"
#include "POSHInteger.h"
#include "POSHString.h"

#ifdef __GNUC__
#pragma implementation
#endif

POSHApply::~POSHApply() { Raids( RaidConstructor ) << this << "::~POSHApply()\n" << eor; }

ProxiableObject * POSHApplyConstructor( ProxiableObject * source )
{
	if( source->isKindOf( PersistentMemoryObjectClass ) ) {
		return( new POSHApply( (PersistentMemoryObject *)source ) );
	}
	else return( 0 );
}

/*
int sizeOfPOSHApply() { return( sizeof( POSHApply ) ); }

Class * POSHApply::classOf() { return( POSHApplyClass ); }
*/
/*************************************************************************/

POSHApply::POSHApply( PersistentMemoryObject * source )
: ( source )
{
	initDebug();
	Raids( RaidConstructor ) << this << "::POSHApply( "
				 << source << " )\n" << eor;
	_argc = 0;
}

void
POSHApply::flush()
{
	Raids( RaidReference ) << this << "::flush()\n" << eor;
	for( int i = 0; i < _argc; i++ ) _argv[i].flushStar();
}

void
POSHApply::printOn( OutputStream * os )
{
	Raids( RaidMethod ) << this << "::printOn( " << os << " )\n" << eor;
	*os << this << eor;
}

POSHExpressionRef
POSHApply::eval( FileSystemInterface * fsi, int & status )
{
	Raids( RaidMethod ) << this << "::eval( " << fsi << ", "
			    << status << " )\n" << eor;

	POSHExpressionRef p = 0;

	if( status != Success ) return( p );

	if( _argv[0] == 0 ) status = ApplyNotInitialized;
	else {
		char name[2];
		name[1] = '\0';
		POSHExpressionStar a[7];
		for( int i = 1; i < _argc; i++ ) {
			a[i] = _argv[i]->eval( fsi, status );
			if( ( status == Success ) && ( a[i] == 0 ) ) {
				status = ApplyArgumentEvaluationFailed ;
			}
			if( status != Success ) return( p );
		}
		MemoryObjectDictionaryStar od = fsi->open( "env",
						MemoryObjectDictionaryClass,
						status, O_CREAT, 0777,
						MemoryObjectDictionaryClass );
		fsi->chdir( "env" );
		for( i = 1; i < _argc; i++ ) {
			name[0] = '0' + i - 1;
			Assert( a[i] != 0 );
			fsi->add( name, a[i] ); 
		}
		p = _argv[0]->eval( fsi, status );
		for( i = 1; i < _argc; i++ ) {
			name[0] = '0' + i - 1;
			od->remove( name );
		}
		fsi->chdir( ".." );
		fsi->unlink( "env" );
		od = 0;
	}
	return( p );
}

int
POSHApply::init( int argc, char *argv[], FileSystemInterface * fsi )
{
	Raids( RaidMethod ) << this << "::init( " << argc << ", "
			    << argv[0] << " )\n" << eor;

	_modified = 1;

	int status = 0;

	_argc = argc;

	if ( _argc == 0 ) status = TooFewArguments;
	else if ( _argc > 7 ) {
		status = TooManyArguments;
		_argc = 7;
	}
	for( int a = 0; a < _argc; a++ ) {
		_argv[a] = convertArg( a, argv, status, fsi );
		if( status != Success ) break;
	}
	return( status );
}
 

#include "ClassInstaller.h"
DEFINEPERSISTENTCLASS("POSHApply",POSHApply,POSHExpression,&POSHApplyConstructor,0);
