#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * POSHBlockAllocator.c - 
 */
#include "Assert.h"
#include "Raid.h"
#include "POSHBlockAllocator.h"
#include "POSHContainer.h"
#include "POSHFileSystem.h"

#ifdef __GNUC__
#pragma implementation
#endif

POSHBlockAllocator::POSHBlockAllocator( MemoryObject * source,
				      POSHSuperBlock * superBlock,
				      unsigned char * blockMap,
				      int log2BlockToSector )
: (source )
{
    initDebug();
    Raid( RaidConstructor )( "%N::POSHBlockAllocator( %N, %A, %d )\n",
			     this, source, superBlock, log2BlockToSector );

    _superBlock = superBlock;
    _blockMap = blockMap;
    _log2BlockToSector = log2BlockToSector;
}

/*
 * Allocate a single new block.
 */
int
POSHBlockAllocator::basicAllocate( int &, int near, int needed )
{
    Raid( RaidProtected )( "%N::basicAllocate( , %d, %d )\n",
			   this, near, needed );

    int newBlock = 0;

    if( _superBlock->_freeBlocks != 0 ) {
	for( int i = 0; i < _superBlock->_dataBlocks; i++ ) {
		if( _blockMap[i] == 0 ) break;
	}
	Assert( i < _superBlock->_dataBlocks );
	Assert( _blockMap[i] == 0 );

	_superBlock->_freeBlocks--;
	_blockMap[i] = 1;

	newBlock = i + _superBlock->_dataBlockStart;
	_superBlock->_modified = 1;
    }

    Raid( RaidProtected )( "POSHBlockAllocator::basicAllocate = %d\n",newBlock );
    return( newBlock );
}

/*
 *	Free the specified block.
 */
void
POSHBlockAllocator::basicFree( int &, int oldBlock, int needed ) 
{
    Raid( RaidProtected )( "%N::basicFree( , %d, %d )\n",
			   this, oldBlock, needed );

    oldBlock -= _superBlock->_dataBlockStart;

    Assert( oldBlock < _superBlock->_dataBlocks );

    Assert( _blockMap[oldBlock] != 0 );
    _blockMap[oldBlock]--;

    if( _blockMap[oldBlock] == 0 ) _superBlock->_freeBlocks++;

    _superBlock->_modified = 1;
}

/*
int
sizeOfPOSHBlockAllocator()
{
	return( sizeof( POSHBlockAllocator ) );
}

Class *
POSHBlockAllocator::classOf()
{
	return( POSHBlockAllocatorClass );
}
*/


#include "ClassInstaller.h"
DEFINECLASS("POSHBlockAllocator",POSHBlockAllocator,BlockAllocator,0,0);
