#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *         University of Illinois at Urbana-Champaign
 *         Department of Computer Science
 *         1304 W. Springfield Ave.
 *         Urbana, IL    61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *    All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * POSHBuiltin.c - 
 */
#include "POSHInteger.h"
#include "POSHBuiltin.h"
#include "OutputStream.h"

#ifdef __GNUC__
#pragma implementation
#endif

extern OutputStream * stdout;

POSHBuiltin::~POSHBuiltin() { Raids( RaidConstructor ) << this << "::~POSHBuiltin()\n" << eor; }

ProxiableObject * POSHBuiltinConstructor( ProxiableObject * source )
{ if( source->isKindOf( PersistentMemoryObjectClass ) ) {
        return( new POSHBuiltin( (PersistentMemoryObject *)source ) );
  }
  else return( 0 );
}

/*
int sizeOfPOSHBuiltin() { return( sizeof( POSHBuiltin ) ); }

Class * POSHBuiltin::classOf() { return( POSHBuiltinClass ); }
*/
/*************************************************************************/

POSHBuiltin::POSHBuiltin( PersistentMemoryObject * source )
: ( source )
{
    initDebug();
    Raids( RaidConstructor ) << this << "::POSHBuiltin( "
                 << source << " )\n" << eor;
    _value[0] = '\0';
}

POSHExpressionRef
POSHBuiltin::eval( FileSystemInterface * fsi, int & status )
{
    Raids( RaidMethod ) << this << "::eval( " << fsi << ", "
			<< status << " )\n" << eor;

    Raids( RaidDetail ) << this << "::eval " << _value << "\n" << eor;

    POSHIntegerRef i = 0;

    if( status != Success ) return( i );

    i = POSHIntegerClass->newObject();
    if( i == 0 ) {
	status = NewObjectFailed;
	return( i );
    }
    i->setValue( 0 );

    {
        POSHExpressionStar argv[7];
        for( int argc = 0; argc < 7; argc++ ) {
            int status = 0;
            char name[2];
            name[0] = '0' + argc;
            name[1] = '\0';
            argv[argc] = fsi->open( name, POSHExpressionClass, status );
            if( argv[argc] == 0 ) break;
        }
        if( argc == 0 ) {
            if( strcmp( _value, "writeln" ) == 0 ) *stdout << "\n" << eor;
            else status = UnknownBuiltinFunction0;
        }
        else if( argc == 1 ) {
            switch( _value[0] ) {
            case '+':
                if( strcmp( _value, "++" ) == 0 ) {
                    i->setValue( argv[0]->asInt() + 1 );
                }
                else status = UnknownBuiltinFunction1;
                break;
            case '-':
                if( strcmp( _value, "-" ) == 0 ) {
                    i->setValue( - argv[0]->asInt() );
                }
                else if( strcmp( _value, "--" ) == 0 ) {
                    i->setValue( argv[0]->asInt() - 1 );
                }
                else status = UnknownBuiltinFunction1;
                break;
            case '~':
                if( strcmp( _value, "~" ) == 0 ) {
                    i->setValue( ~ argv[0]->asInt() );
                }
                else status = UnknownBuiltinFunction1;
                break;
            case '!':
                if( strcmp( _value, "!" ) == 0 ) {
                    i->setValue( ! argv[0]->asInt() );
                }
                else status = UnknownBuiltinFunction1;
                break;
            case 'w':
                if( strcmp( _value, "write" ) == 0 ) {
                    argv[0]->printOn( stdout );
                    i->setValue( argv[0]->asInt() );
                }
                else if( strcmp( _value, "writeln" ) == 0 ) {
                    argv[0]->printOn( stdout );
                    *stdout << "\n" << eor;
                    i->setValue( argv[0]->asInt() );
                }
                else status = UnknownBuiltinFunction1;
                break;
            default:
                status = UnknownBuiltinFunction1;
            }
        }
        else if( argc == 2 ) {
            switch( _value[0] ) {
            case '+':
                if( strcmp( _value, "+" ) == 0 ) {
                    i->setValue( argv[0]->asInt() + argv[1]->asInt() );
                }
                else status = UnknownBuiltinFunction2;
                break;
            case '-':
                if( strcmp( _value, "-" ) == 0 ) {
                    i->setValue( argv[0]->asInt() - argv[1]->asInt() );
                }
                else status = UnknownBuiltinFunction2;
                break;
            case '*':
                if( strcmp( _value, "*" ) == 0 ) {
                    i->setValue( argv[0]->asInt() * argv[1]->asInt() );
                }
                else status = UnknownBuiltinFunction2;
                break;
            case '/':
                if( strcmp( _value, "/" ) == 0 ) {
                    i->setValue( argv[0]->asInt() / argv[1]->asInt() );
                }
                else status = UnknownBuiltinFunction2;
                break;
            case '<':
                if( strcmp( _value, "<" ) == 0 ) {
                    i->setValue( argv[0]->asInt() < argv[1]->asInt() );
                }
                else if( strcmp( _value, "<=" ) == 0 ) {
                    i->setValue( argv[0]->asInt() <= argv[1]->asInt() );
                }
                else if( strcmp( _value, "<<" ) == 0 ) {
                    i->setValue( argv[0]->asInt() << argv[1]->asInt() );
                }
                else status = UnknownBuiltinFunction2;
                break;
            case '>':
                if( strcmp( _value, ">" ) == 0 ) {
                    i->setValue( argv[0]->asInt() > argv[1]->asInt() );
                }
                else if( strcmp( _value, ">=" ) == 0 ) {
                    i->setValue( argv[0]->asInt() >= argv[1]->asInt() );
                }
                else if( strcmp( _value, ">>" ) == 0 ) {
                    i->setValue( argv[0]->asInt() >> argv[1]->asInt() );
                }
                else status = UnknownBuiltinFunction2;
                break;
            case '%':
                if( strcmp( _value, "%" ) == 0 ) {
                    i->setValue( argv[0]->asInt() % argv[1]->asInt() );
                }
                else status = UnknownBuiltinFunction2;
                break;
            case '&':
                if( strcmp( _value, "&" ) == 0 ) {
                    i->setValue( argv[0]->asInt() & argv[1]->asInt() );
                }
                else if( strcmp( _value, "&&" ) == 0 ) {
                    i->setValue( argv[0]->asInt() && argv[1]->asInt() );
                }
                else status = UnknownBuiltinFunction2;
                break;
            case '|':
                if( strcmp( _value, "|" ) == 0 ) {
                    i->setValue( argv[0]->asInt() | argv[1]->asInt() );
                }
                else if( strcmp( _value, "||" ) == 0 ) {
                    i->setValue( argv[0]->asInt() || argv[1]->asInt() );
                }
                else status = UnknownBuiltinFunction2;
                break;
            case '^':
                if( strcmp( _value, "^" ) == 0 ) {
                    i->setValue( argv[0]->asInt() ^ argv[1]->asInt() );
                }
                else status = UnknownBuiltinFunction2;
                break;
            case '=':
                if( strcmp( _value, "==" ) == 0 ) {
                    i->setValue( argv[0]->asInt() == argv[1]->asInt() );
                }
                else status = UnknownBuiltinFunction2;
                break;
            case '!':
                if( strcmp( _value, "!=" ) == 0 ) {
                    i->setValue( argv[0]->asInt() != argv[1]->asInt() );
                }
                else status = UnknownBuiltinFunction2;
                break;
            case 'w':
                if( strcmp( _value, "write" ) == 0 ) {
                    argv[0]->printOn( stdout );
                    argv[1]->printOn( stdout );
                    i->setValue( argv[1]->asInt() );
                }
                else if( strcmp( _value, "writeln" ) == 0 ) {
                    argv[0]->printOn( stdout );
                    argv[1]->printOn( stdout );
                    *stdout << "\n" << eor;
                    i->setValue( argv[1]->asInt() );
                }
                else status = UnknownBuiltinFunction2;
                break;
            default:
                status = UnknownBuiltinFunction2;
            }
        }
        else {
            switch( _value[0] ) {
            case 'w':
                if( strcmp( _value, "write" ) == 0 ) {
    		for( int a = 0; a < argc; a++ ) {
                        argv[a]->printOn( stdout );
                    }
                    i->setValue( argv[argc-1]->asInt() );
                }
                else if( strcmp( _value, "writeln" ) == 0 ) {
    		for( int a = 0; a < argc; a++ ) {
                        argv[a]->printOn( stdout );
                    }
                    *stdout << "\n" << eor;
                    i->setValue( argv[argc-1]->asInt() );
                }
                else status = UnknownBuiltinFunction;
                break;
            default:
                status = UnknownBuiltinFunction;
            }
        }
    }
    return( i );
}

void
POSHBuiltin::printOn( OutputStream * os )
{
    Raids( RaidMethod ) << this << "::printOn( " << os << " )\n" << eor;
    *os << this << eor;
}

int
POSHBuiltin::init( int argc, char *argv[], FileSystemInterface * )
{
	Raids( RaidMethod ) << this << "::init( " << argc << ", "
			    << argv[0] << " )\n" << eor;

	_modified = 1;

	int status = Success;

	if ( argc == 0 ) status = TooFewArguments;
	else {
		strcpy( _value, argv[0] ); // FIX, check size
		if( argc > 1 ) status = TooManyArguments;
	}

	return( status );
}


#include "ClassInstaller.h"
DEFINEPERSISTENTCLASS("POSHBuiltin",POSHBuiltin,POSHExpression,&POSHBuiltinConstructor,0);
