#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * PersistentArrayStream.c - 
 */
#include "Assert.h"
#include "FileSystem.h"
#include "Raid.h"
#include "PersistentArrayStream.h"
#include "PersistentArray.h"

#ifdef __GNUC__
#pragma implementation
#endif

PersistentArrayStream::PersistentArrayStream( PersistentArray * po )
: RecordStream( po )
{
    initDebug();
    Raids( RaidConstructor ) << this <<  "::PersistentArrayStream( "
			     << po << " )\n" << eor;

    // UNUSED ??? int status = 0;

    Assert( po->isKindOf( PersistentArrayClass ) );
    _file = po;
}

PersistentArrayStream::~PersistentArrayStream()
{
    Raids( RaidConstructor ) << this << "::~PersistentArrayStream()\n" << eor;
}

int
PersistentArrayStream::read( char * buffer, int size, int & status )
{
    Raids( RaidMethod ) << this << "::read( , " << size << " , )\n" << eor;

    status = 0;

    int records = _file->numberOfRecords();
    int recSize = _file->recordSize();

    size /= recSize;

    if( ( _recordNumber + size ) > records ) {
	size = records - _recordNumber;
	if( size < 0 ) size = 0;
	if( size == 0 ) status = EndOfFile;
    }
    _file->at( _recordNumber, buffer, size );
    _recordNumber += size;

    return( size * recSize );
}

int
PersistentArrayStream::write( char * buffer, int size, int & status )
{
    Raids( RaidMethod ) << this << "::write( , " << size << " )\n" << eor;

    status = 0;

    int recSize = _file->recordSize();
    size /= recSize;
    _file->atPut( _recordNumber, buffer, size );
    _recordNumber += size;
    return( size * recSize );
}

ProxiableObject *
PersistentArrayStreamConstructor( ProxiableObject * source )
{
	if( source->isKindOf( PersistentArrayClass ) ) {
		return( new PersistentArrayStream( (PersistentArray *)source ) );
	}
	else return( 0 );
}

/*
int
sizeOfPersistentArrayStream()
{
	return( sizeof( PersistentArrayStream ) );
}

Class *
PersistentArrayStream::classOf()
{
	return( PersistentArrayStreamClass );
}
*/


#include "ClassInstaller.h"
DEFINECLASS("PersistentArrayStream",PersistentArrayStream,RecordStream,&PersistentArrayStreamConstructor,0);
