#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * PersistentClass.cc -
 */
#include "Assert.h"
#include "Raid.h"
#include "PersistentClass.h"
#include "PersistentClassStar.h"
#include "String.h"
//#include "Kernel.h"
//#include "Domain.h"
//#include "ObjectProxy.h"
//#include "ObjectProxyManager.h"
#include "PersistentMemoryObject.h"
#include "PersistentMemoryObjectStar.h"
#include "MemoryObjectContainer.h"
#include "MemoryObjectContainerStar.h"
#include "MemoryObjectDictionary.h"
#include "MemoryObjectDictionaryStar.h"
#include "FileSystemInterface.h"
#include "FileSystemInterfaceStar.h"
#include "NameServer.h"
#include "COFFLoader.h"
#include "PersistentObjectStar.h"
#include "OutputStream.h"

#ifdef __GNUC__
#pragma implementation
#endif

extern int UID;
extern int GID;
extern int UMASK;

//#include "PersistentClassDefinitions.cc"

void
PersistentClass::initialize() {
//#include "PersistentClassInstantiation.cc"
}


PersistentClass::PersistentClass( Class * parent, int size,
				  const char * name, ClassConstructor * ctor )
				  //const char * name, POFPO ctor )
: Class( parent, size, name, ctor )
{
	initDebug();
	Raids( RaidConstructor ) << this << "::PersistentClass( "
				 << parent << " )\n" << eor;
}

PersistentClass::~PersistentClass()
{
	/*
	 * Applications destruct these things!, dkr 4-16-92
	 */
	// Assert( NOTREACHED );
}

void
PersistentClass::cleanup()
{
        Raids( RaidMethod ) << this << "::cleanup( " << _dictionary << ", "
	                    << _container << " )\n" << eor;
        ClassStar mychild = child();
	ClassStar mysibling = sibling();

	if( mychild != 0 ) {
		mychild->cleanup();
	}
	if( mysibling != 0 ) {
		mysibling->cleanup();
	}
/*
	if( _child != 0 ) {
		_child->cleanup();
	}
	if( _sibling != 0 ) {
		_sibling->cleanup();
	}
*/

	if( _dictionary != 0 ) _dictionary = 0;
	if( _container  != 0 ) _container  = 0;
}

PersistentObjectRef
PersistentClass::newObject()
{
    Raids( RaidMethod ) << this << "::newObject()\n" << eor;
    int status = 0;
    PersistentObjectRef po = (PersistentObject *)0;

    if( _container == 0 ) {
	FileSystemInterfaceStar fsi = StandardNameServer->lookup("fileSystemInterface","FileSystemInterface");
	//FileSystemInterfaceStar fsi = TheKernel->fileSystemInterface();
	Assert( fsi != 0 );
	if( _dictionary == 0 ) {
		_dictionary = fsi->open( "/ObjectStore",
					  MemoryObjectDictionaryClass,
					  status );
	}
	Assert( _dictionary != 0 );
	PersistentMemoryObjectStar pmo = _dictionary->asA(
					 PersistentMemoryObjectClass, status );
	Assert( pmo != 0 );
	_container = pmo->container();
	Assert( _container != 0 );
    }
    PersistentMemoryObjectStar mo = _container->create( 0, this, status );
    if( mo == 0 ) return( po );

    mo->setUserId( UID );
    mo->setGroupId( GID );
    mo->setMode( 0777 );
    //mo->setNumberOfBytes( sizeOf() );

    po = mo->asA( this, status );

    return( po );
}

PersistentObjectRef
PersistentClass::retrieveObject( int id )
{
    Raids( RaidMethod ) << this << "::retrieveObject( " << id << " )\n" << eor;
    int status = 0;
    PersistentObjectRef po = (PersistentObject *)0;

    if( _container == 0 ) {
	FileSystemInterfaceStar fsi = StandardNameServer->lookup("fileSystemInterface", "FileSystemInterface");
	//FileSystemInterfaceStar fsi = TheKernel->fileSystemInterface();
	Assert( fsi != 0 );
	MemoryObjectDictionaryStar od = fsi->open( "/ObjectStore",
						  MemoryObjectDictionaryClass,
						  status );
	if( od == (MemoryObjectDictionary *)0 ) return( po );
	PersistentMemoryObjectStar pmo = od->asA( PersistentMemoryObjectClass,
						  status );
	Assert( pmo != 0 );
	_container = pmo->container();
	Assert( _container != 0 );
    }
    PersistentMemoryObjectStar mo = _container->open( id, status );
    if( mo == (PersistentMemoryObject *)0 ) return( po );

    po = mo->asA( this, status );
    return( po );
}

MemoryObjectDictionaryRef
PersistentClass::rootDictionary()
{
	if( _dictionary == 0 ) {
		int status = 0;
		FileSystemInterfaceStar fsi = StandardNameServer->lookup("fileSystemInterface", "FileSystemInterface");
		//FileSystemInterfaceStar fsi = TheKernel->fileSystemInterface();
		Assert( fsi != 0 );
		_dictionary = fsi->open( "/ObjectStore",
					  MemoryObjectDictionaryClass,
					  status );
	}
	return( _dictionary );
}

/*
Class *
PersistentClass::classOf()
{ return( PersistentClassClass ); }

int
sizeOfPersistentClass()
{ return( sizeof( PersistentClass ) ); }

*/

#include "ClassInstaller.h"
DEFINECLASS("PersistentClass",PersistentClass,Class,0,0);
