#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * PersistentIntArray.c - 
 */
#include "Assert.h"
#include "Raid.h"
#include "PersistentIntArray.h"
#include "PersistentMemoryObject.h"
#include "Convert.h"

#ifdef __GNUC__
#pragma implementation
#endif

PersistentIntArray::PersistentIntArray( PersistentMemoryObject * source )
: PersistentArray( source )
{
	initDebug();

	Raid( RaidConstructor )( "%N::PersistentIntArray( %N )\n", this, source );
}

PersistentIntArray::~PersistentIntArray()
{
	Raid( RaidConstructor )( "%N::~PersistentIntArray\n", this );
}

int
PersistentIntArray::init( int argc, char *argv[], FileSystemInterface * )
{
	if( argc > 0 ) {
		Raids( RaidMethod ) << this << "::init( " << argc << ", "
				    << argv[0] << " )\n" << eor;
		int recordSize = atoi( argv[0] );
		Assert( recordSize >= (int)sizeof( char ) );
		Assert( recordSize <= (int)sizeof( int ) );
		_source->setRecordSize( recordSize );
	}
	else {
		Raids( RaidMethod ) << this << "::init()\n" << eor;
		_source->setRecordSize( sizeof( int ) );
	}
	return( 0 );
}

int
PersistentIntArray::at( unsigned int index )
{
	int value = 0;

	switch( _source->recordSize() ) {
	case sizeof( char ):	char c;
				PersistentArray::at( index, &c, 1 );
				value = c;
				break;
	case sizeof( short ):	short s;
				PersistentArray::at( index, (char *)&s, 1 );
				value = s;
				break;
	case sizeof( int ):	PersistentArray::at( index, (char *)&value, 1 );
				break;
	default:		Assert( NOTREACHED );
	}


	Raids( RaidMethod ) << this << "::at( " << index << " ) return( "
			    << value << " )\n" << eor;

	return( value );
}

int
PersistentIntArray::atPut( unsigned int index, int value )
{
	Raids( RaidMethod ) << this << "::atPut( " << index << ", "
			    << value << " )\n" << eor;

	PersistentArray::atPut( index, (char *)&value, 1 );

	switch( _source->recordSize() ) {
	  char c;
	  short s;
	  int i;
	case sizeof( char ):	c = value;
				PersistentArray::atPut( index, &c, 1 );
				break;
	case sizeof( short ):	s = value;
				PersistentArray::atPut( index, (char *)&s, 1 );
				break;
	case sizeof( int ): 	i = value;
				PersistentArray::atPut( index, (char *)&i, 1 );
				break;
	default:		Assert( NOTREACHED );
	}

	return( value );
}

ProxiableObject *
PersistentIntArrayConstructor( ProxiableObject * source )
{
        if( source->isKindOf( PersistentMemoryObjectClass ) ) {
		return( new PersistentIntArray( (PersistentMemoryObject *)source ) );
        }
        else return( 0 );
}

/*
int
sizeOfPersistentIntArray()
{
	return( sizeof( PersistentIntArray ) );
}

Class *
PersistentIntArray::classOf()
{
	return( PersistentIntArrayClass );
}
*/


#include "ClassInstaller.h"
DEFINEPERSISTENTCLASS("PersistentIntArray",PersistentIntArray,PersistentArray,&PersistentIntArrayConstructor,0);
