#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * PersistentObject.c - 
 */
#include "Assert.h"
#include "FileSystem.h"
#include "Raid.h"
#include "PersistentClass.h"
#include "PersistentObject.h"
#include "PersistentObjectStar.h"
#include "PersistentMemoryObject.h"
#include "Class.h"
#include "String.h"
#include "MemoryObjectDictionary.h"
#include "FileSystemInterface.h"
#include "Lock.h"
#include "Byte.h"
#include "OutputStream.h"

#ifdef __GNUC__
#pragma implementation
#endif

PersistentObject::PersistentObject( PersistentMemoryObject * source )
: _source( source )
{
        initDebug();

        Raid( RaidConstructor )( "%N::PersistentObject( %N )\n", this, source );

        _arrayBuffer = 0;
        _modified = 1;
}

PersistentObject::~PersistentObject()
{
	Raid( RaidConstructor )( "%N::~PersistentObject\n", this );
        _source = 0;
}

void
PersistentObject::flush()
{
        Raid( RaidReference )( "%N::flush()\n", this );
}

void
PersistentObject::noRemainingReferences()
{
	Raid( RaidReference )( "%N::noRemainingReferences\n", this );
	
        if( _arrayBuffer != 0 ) flush();
	_source->close( this );
}

int
PersistentObject::persist()
{
        Raid( RaidReference )( "%N::persist() %d\n", this );

        MemoryObjectDictionaryStar od = ((PersistentClass *)classOf())->
                                                            rootDictionary();
        Assert( od != 0 );

        char string[16];

        int d = digits( _source->idNumber() );
        Assert( d != 0 );

        itoa( _source->idNumber(), &string[d] );

        int status = od->add( string, _source );

        return( status );
}

int
PersistentObject::desist()
{
        Raid( RaidReference )( "%N::desist() %d\n", this );

        MemoryObjectDictionaryStar od = ((PersistentClass *)classOf())->
                                                            rootDictionary();
        Assert( od != 0 );

        char string[16];

        int d = digits( _source->idNumber() );
        Assert( d != 0 );

        itoa( _source->idNumber(), &string[d] );

        int status = od->remove( string );

        return( status );
}

unsigned
PersistentObject::numberOfBytes()
{
        Raid( RaidMethod )( "%N::numberOfBytes() %d\n",
                            this, _source->numberOfBytes() );

        return( _source->numberOfBytes() );
}

unsigned
PersistentObject::numberOfRecords()
{
        Raid( RaidMethod )( "%N::numberOfRecords() %d\n",
                            this, _source->numberOfRecords() );

        return( _source->numberOfRecords() );
}

unsigned
PersistentObject::recordSize()
{
        Raid( RaidMethod )( "%N::recordSize() %d\n",
                            this, _source->recordSize() );

        return( _source->recordSize() );
}

int
PersistentObject::execute()
{
        Raids( RaidMethod ) << this << "::execute()\n" << eor;
        return( 0 );
}

int
PersistentObject::init( int argc, char *argv[], FileSystemInterface * )
{
        Raids( RaidMethod ) << this << "::init( " << argc << ", "
                            << argv[0] << " )\n" << eor;
        return( 0 );
}

int
PersistentObject::copy( PersistentObject * po )
{
    Raid( RaidMethod )( "%N::copy( %N )\n", this, po );

    int status = 0;
    PersistentMemoryObjectStar mo = po->asA( PersistentMemoryObjectClass,
                                             status );
    if( mo != 0 ) status = _source->copy( mo );
    return( status );
}

void
PersistentObject::info( UNIXFileInfo * buffer )
{
    Raid( RaidMethod )( "%N::info( %x )\n", this, buffer );

    _source->info( buffer );
}
 
void
PersistentObject::writeName( OutputStream * os )
{
        ProxiableObject::writeName( os );
        if( _source != 0 ) {
                os->formattedWrite( "(" );
                _source->writeIdNumber( os );
                os->formattedWrite( ")" );
        }
}

int
PersistentObject::executable()
{
        return( _source->access( AccessExecute ) );
}

int
PersistentObject::idNumber()
{
        return( _source->idNumber() );
}

ProxiableObjectRef
PersistentObject::asA( Class * aClass, int & status )
{
        return( _source->asA( aClass, status ) );
}

int
PersistentObject::links()
{
        return( _source->links( 0 ) );
}

int
PersistentObject::setUserId( int uid )
{
        return( _source->setUserId( uid ) );
}

int
PersistentObject::setGroupId( int gid )
{
        return( _source->setGroupId( gid ) );
}

int
PersistentObject::setMode( int mode )
{
        return( _source->setMode( mode ) );
}

int
PersistentObject::unmount()
{
        return Success;
}

int 
PersistentObject::notify( int event )
{
	return event;
}

ProxiableObject *
PersistentObjectConstructor( ProxiableObject * source )
{
        if( source->isKindOf( PersistentMemoryObjectClass ) ) {
		return( new PersistentObject( (PersistentMemoryObject *)source ) );
        }
        else return( 0 );
}


#include "ClassInstaller.h"
DEFINEPERSISTENTCLASS("PersistentObject",PersistentObject,ProxiableObject,&PersistentObjectConstructor,0);
//DEFINECLASS("PersistentObject",PersistentObject,ProxiableObject,&PersistentObjectConstructor,0);
