#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * RecordStream.c - 
 */

//HACK BY SCOTT KIRCHER
//(Why doesn't this get defined when I set it in Config.mk?)
#ifndef COFFFile
#define COFFFile
#endif

#include "Assert.h"
#include "FileSystem.h"
#include "Raid.h"
#include "RecordStream.h"
#include "PersistentMemoryObject.h"
#include "VariableRecordFile.h"
#include "PersistentArray.h"
#include "PersistentObjectStar.h"
#include "SystemInterface.h"
#include "Process.h"
#include "ThisProcess.h"

#ifdef __GNUC__
#pragma implementation
#endif


RecordStream::RecordStream( PersistentObject * po )
{
    initDebug();
    Raids( RaidConstructor ) << this <<  "::RecordStream( "
			     << po << " )\n" << eor;

Console()<<"RecordStream constructor called.\n";

    _recordNumber = 0;
    _file = po;
}

RecordStream::~RecordStream()
{
    Raids( RaidConstructor ) << this << "::~RecordStream()\n" << eor;
}

int
RecordStream::init( int argc, char * argv[] )
{
    Raids( RaidMethod ) << this << "::init( " << argc << " )\n" << eor;

    return( _file->init( argc, argv ) );
}

int
RecordStream::read( char *b, int size )
{
    Raids( RaidMethod ) << this << "::read( , " << size << " )\n" << eor;
    int dummyStatus;

    return( read(b, size, dummyStatus) );
}

#ifdef NEVER
int
RecordStream::read( char *, int size, int & status )
{
    Raids( RaidMethod ) << this << "::read( , " << size << " , )\n" << eor;

    status = NotImplemented;
    return( 0 );
}
#endif

int
RecordStream::write( char *b, int size )
{
    Raids( RaidMethod ) << this << "::write( , " << size << " )\n" << eor;
    int dummyStatus;

    return( write(b, size, dummyStatus) );
}

#ifdef NEVER
int
RecordStream::write( char *, int size, int & status )
{
    Raids( RaidMethod ) << this << "::write( , " << size << " )\n" << eor;

    status = NotImplemented;
    return( 0 );
}
#endif

int
RecordStream::setMode( int mode )
{
    Raids( RaidMethod ) << this << "::setMode( " << mode << " ) return( "
			<< mode << " )\n" << eor;
    Assert( NOTREACHED ); // FIX
    return( mode );
}

int
RecordStream::setUserId( int uid )
{
    Raids( RaidMethod ) << this << "::setUserId( " << uid << " ) return( "
			<< uid << " )\n" << eor;
    Assert( NOTREACHED ); // FIX
    return( uid );
}

int
RecordStream::setGroupId( int gid )
{
    Raids( RaidMethod ) << this << "::setGroupId( " << gid << " ) return( "
			<< gid << " )\n" << eor;
    Assert( NOTREACHED ); // FIX
    return( gid );
}

int
RecordStream::recordNumber()
{
    Raids( RaidMethod ) << this << "::recordNumber() return( "
			<< _recordNumber << " )\n" << eor;
    return( _recordNumber );
}

/*
 *	change the current offset
 *	mode	0 == beginning of file
 *		1 == current offset
 *		2 == end of file
 */
int
RecordStream::setRecordNumber( int delta, int mode )
{
    Raids( RaidMethod ) << this << "::setRecordNumber( , " << delta
			<< ", " << mode << ", )\n" << eor;

    if( mode == 0 )      _recordNumber = delta;
    else if( mode == 1 ) _recordNumber += delta;
    else if( mode == 2 ) _recordNumber = _file->numberOfRecords() + delta;
    else return( -1 );

    if( _recordNumber < 0 ) _recordNumber = 0;
    return( _recordNumber );
}

int
RecordStream::numberOfBytes()
{
    Raids( RaidMethod ) << this << "::numberOfBytes() "
			<< _file->numberOfBytes() << "\n" << eor;
    return( _file->numberOfBytes() );
}

int
RecordStream::numberOfRecords()
{
    Raids( RaidMethod ) << this << "::numberOfRecords() "
			<< _file->numberOfRecords() << "\n" << eor;
    return( _file->numberOfRecords() );
}

int
RecordStream::recordSize()
{
    Raids( RaidMethod ) << this << "::recordSize() "
			<< _file->recordSize() << "\n" << eor;
    return( _file->recordSize() );
}

int
RecordStream::minimumRecordSize()
{
    Raids( RaidMethod ) << this << "::minimumRecordSize() "
			<< _file->recordSize() << "\n" << eor;
    return( _file->recordSize() );
}

int
RecordStream::maximumRecordSize()
{
    Raids( RaidMethod ) << this << "::maximumRecordSize() "
			<< _file->recordSize() << "\n" << eor;
    return( _file->recordSize() );
}

void
RecordStream::info( UNIXFileInfo * )
{
    Raids( RaidMethod ) << this << "::info()\n" << eor;
}

int 
RecordStream::executable()
{
	return(_file->executable());
}

ProxiableObjectRef
RecordStream::asA( Class * aClass, int & status )
{
	return( _file->asA( aClass, status ));
}

int
RecordStream::runFile( int argc, char ** argv )
{
    Raids( RaidMethod ) << this << "::runFile( argc " << argc
			<< ", argv[0] " << argv[0] << ")\n" << eor;

    int status;

    if( !executable() ) return( -1 );


    char * buffer = new char[recordSize()*2];

    setRecordNumber( 0, 0 );
    read( buffer, 2, status);
    setRecordNumber( 0, 0 );

    int header = buffer[0];
    Raids( RaidDetail ) << this << "::runFile header = "
			<< header << "\n" << eor;

    delete buffer;

    Process * process = 0;

#include "defUNIXCOMPATIBILITY.h"
#ifdef UNIXCOMPATIBILITY
    extern Process * BuildBSDProcessFromCOFFFile( PersistentMemoryObject *,int,char **);

    if( header == 0124 ) {
	Raids( RaidDetail ) << this << "::runFile BSD\n" << eor;
	process = BuildBSDProcessFromCOFFFile( this, argc, argv );
    }
    else
#endif /*UNIXCOMPATIBILITY*/
    {
	Raids( RaidDetail ) << this << "::runFile Choices\n" << eor;
	PersistentMemoryObjectStar pmo =
		this->asA(PersistentMemoryObjectClass, status);
	if (status) return status;
	process = StandardSystemInterface->buildProcessFromObjectFile(
				pmo, argc, argv );
    }

    if( process != 0 ) {
	Raids( RaidDetail ) << this << "::runFile " << process << "\n" << eor;

	process->ready();
	thisProcess()->waitForProcess( process );
    }

    Raids( RaidDetail ) << this << "::runFile return( 0 )\n" << eor;
    return( 0 );
}

#include "ClassInstaller.h"
DEFINECLASS("RecordStream",RecordStream,ProxiableObject,0,0);
