#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * RelativeRecordFile.c - 
 */
#include "Assert.h"
#include "Raid.h"
#include "RelativeRecordFile.h"
#include "PersistentMemoryObject.h"
#include "Byte.h"
#include "PersistentClass.h"
#include "PersistentIntArray.h"
#include "PersistentArray.h"
#include "FileSystem.h"
#include "Convert.h"

#ifdef __GNUC__
#pragma implementation
#endif

class FileSystemInterface;

RelativeRecordFile::RelativeRecordFile( PersistentMemoryObject * source )
: FixedRecordFile( source )
{
	initDebug();

	Raids( RaidConstructor ) << this << "::RelativeRecordFile( "
				 << source << " )\n" << eor;
}

RelativeRecordFile::~RelativeRecordFile()
{
	Raids( RaidConstructor ) << this << "::~RelativeRecordFile()\n" << eor;
}

void
RelativeRecordFile::flush()
{
	Raids( RaidReference ) << this << "::flush()\n" << eor;
	_data.flushStar();
	_control.flushStar();
}

int
RelativeRecordFile::init( int argc, char *argv[], FileSystemInterface * )
{
	Raids( RaidMethod ) << this << "::init()\n" << eor;

	char * args[1];
	args[0] = "1";

	_data = PersistentArrayClass->newObject();
	_data->persist();
	if( argc == 0 ) {
		_data->init( 1, args );
		_recordSize = 1;
	}
	else {
		_data->init( 1, argv );
		_recordSize = atoi( argv[0] );
	}

	_minRecordSize = _recordSize;
	_maxRecordSize = _recordSize;

	_control = PersistentIntArrayClass->newObject();
	_control->persist();
	_control->init( 1, args );

	return( 0 );
}

int
RelativeRecordFile::read( unsigned int record, char * buffer,
			  unsigned int size, int & status )
{
	Raid( RaidMethod )( "%N::read( %d, , %d, )\n", this, record, size );

        if( size < _recordSize ) status = RecordSizeMismatch;
        else if( record < numberOfRecords() ) {
                if( _control->at( record ) ) {
			_data->at( record, buffer, 1 );
                	status = 0;
		}
		else status = RecordNotFound;
        }
        else status = EndOfFile;

	Raid( RaidMethod )( "%N::read return( %d ) status %d\n",
			    this, ( ( status == 0 ) ? size : 0 ), status );
	return( ( status == 0 ) ? size : 0 );
}

int
RelativeRecordFile::write( unsigned int record, char * buffer,
			   unsigned int size, int & status )
{
	Raid( RaidMethod )( "%N::write( %d, , %d, )\n", this, record, size );

	if( size < _recordSize ) status = RecordSizeMismatch;
	else {
                if( !_control->at( record ) ) {
                        _control->atPut( record, 1 );
                        _data->atPut( record, buffer, 1 );
                        status = 0;
                }
                else status = RecordExists;
	}

	Raid( RaidMethod )( "%N::write return( %d ) status %d\n",
			    this, ( ( status == 0 ) ? size : 0 ), status );
	return( ( status == 0 ) ? size : 0 );
}

ProxiableObject *
RelativeRecordFileConstructor( ProxiableObject * source )
{
        if( source->isKindOf( PersistentMemoryObjectClass ) ) {
		return( new RelativeRecordFile( (PersistentMemoryObject *)source ) );
        }
        else return( 0 );
}

/*
int
sizeOfRelativeRecordFile()
{
	return( sizeof( RelativeRecordFile ) );
}

Class *
RelativeRecordFile::classOf()
{
	return( RelativeRecordFileClass );
}
*/


#include "ClassInstaller.h"
DEFINEPERSISTENTCLASS("RelativeRecordFile",RelativeRecordFile,FixedRecordFile,&RelativeRecordFileConstructor,0);
