#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 *RemoteDictionary.c -
 */

#include "Assert.h"
#include "Byte.h"
#include "Class.h"
#include "ContainerIndex.h"
#include "FileSystem.h"
#include "OutputStream.h"
#include "PersistentMemoryObject.h"
#include "PersistentMemoryObjectStar.h"
#include "Raid.h"
#include "RemoteDictionary.h"
#include "RemoteContainer.h"
#include "RemoteFileObject.h"
#include "RemoteFileSystem.h"
#include "RemoteFileSystemRPC.h"
#include "RemoteTables.h"
#include "Semaphore.h"
#include "SemaphoreStar.h"
#include "String.h"

#ifdef __GNUC__
#pragma implementation
#endif

RemoteDictionary::RemoteDictionary
	( PersistentMemoryObject * source )
	: MemoryObjectDictionary( source )
{
	initDebug();
	Raids( RaidConstructor ) 
		<< this << "::RemoteDictionary( source " << source << " ) \n" 
		<< eor;
	Assert( _source->isKindOf( RemoteFileObjectClass ) );
}

RemoteDictionary::~RemoteDictionary( )
{
	Raids( RaidConstructor ) 
		<< this << "::~RemoteDictionary" << "\n" << eor;
	TheRemotePathCache->unbind( this );
}

void
RemoteDictionary::reMap()
{
	/* do nothing */
}

PersistentMemoryObjectRef
RemoteDictionary::open( char * key, int & status, int flags )
{
	Raids( RaidMethod ) 
		<< this << "::open( key " << key << ", flags " << flags 
		<< " )\n" << eor;

	RFSHandle fileHandle;
	RFSAttributes attributes;
	FileObject * fo;
	RemoteContainer * container;

	status = 0;
	PersistentMemoryObjectRef pmo = 0;

	// lookup path name cache
	int success = TheRemotePathCache->lookup( this, key, &fileHandle );
	if ( !success ) 
		goto doRPC;

	// found in name cache
	container = TheRemoteContainerRegistry->lookup
		( fileHandle._containerHandle );

	// check if exists on this host
	Raids( RaidDetail ) 
		<< this << "::open() " << key << " found in name cache\n"
		<< eor;
	fo = TheContainerIndex->lookup( container, fileHandle._idNumber );
	if ( fo != 0 ) {
		// file object exists in this host
		Raids( RaidDetail ) 
			<< this << "::open() " << key << " exists on host\n"
			<< eor;
		pmo = container->open( fileHandle._idNumber, status, flags );
		goto done;
	}

doRPC:
	// files object does not exists on host
	Raids( RaidDetail ) 
		<< this << "::open() doing RPC\n" << eor;
	status = RemoteFileSystemRPC::open
		( *(handle()), key, flags, &fileHandle, &attributes );	
	Raids( RaidDetail ) 
		<< this << "::open() RPC status " << status << "\n" << eor;
	if ( status != 0 )
		goto done; 
	container = TheRemoteContainerRegistry->lookup
		( fileHandle._containerHandle );
	pmo = container->fastOpen( fileHandle, attributes, status, flags, 0 );
	TheRemotePathCache->bind( this, key, fileHandle );
#ifdef RAID
	if ( !( key[0] == '.' &&
	        ( key[1] == 0 || ( key[1] == '.' && key[2] == 0 ) ) ) )
		pmo->setName( key );	
#endif

done:
	Raids( RaidMethod )
		<< this << "::open() result pmo " << pmo << " status " 
		<< status << "\n" << eor;
	return pmo;
}

PersistentMemoryObjectRef
RemoteDictionary::create( char * key, Class * aClass, int & status, int flags )
{
	Raids( RaidMethod ) 
		<< this << "::create( key " << key << ", aClass " << aClass 
		<< " )\n" << eor;

	RFSHandle fileHandle;
	RFSAttributes attributes;
	FileObject * fo;
	RemoteContainer * container;

	status = 0;
	PersistentMemoryObjectRef pmo = 0;

	// lookup path name cache
	int success = TheRemotePathCache->lookup( this, key, &fileHandle );
	if ( !success ) 
		goto doRPC;

	// found in name cache
	Raids( RaidDetail ) 
		<< this << "::create() " << key << " found in name cache\n"
		<< eor;
	container = TheRemoteContainerRegistry->lookup
		( fileHandle._containerHandle );

	// check if exists on this host
	fo = TheContainerIndex->lookup( container, fileHandle._idNumber );
	if ( fo != 0 ) {
		// file object exists in this host
		Raids( RaidDetail ) 
			<< this << "::create() " << key << " exists on host\n"
			<< eor;
		pmo = container->open( fileHandle._idNumber, status, flags );
		ProxiableObjectStar po = pmo->asA( aClass, status );
		if ( po != 0 ) {
			pmo->setName( key );
			status = KeyExists;
		} else {
			pmo = 0;
		}
		goto done;
	}

doRPC:
	// files object does not exists on host
	Raids( RaidDetail ) 
		<< this << "::create() doing RPC\n" << eor;
	{
	RFSObjectType objectType( aClass );
	status = RemoteFileSystemRPC::create
		( *(handle()), objectType, key, flags,
		  &fileHandle, &attributes );
	}
	Raids( RaidDetail ) 
		<< this << "::create() RPC status " << status << "\n" << eor;
	if ( status == 0 || status == KeyExists ) {
		container = TheRemoteContainerRegistry->lookup
			( fileHandle._containerHandle );
		pmo = container->fastOpen
			( fileHandle, attributes, status, flags, 1 );
		TheRemotePathCache->bind( this, key, fileHandle );
#ifdef RAID
		if ( !( key[0] == '.' &&
		        ( key[1] == 0 || ( key[1] == '.' && key[2] == 0 ) ) ) )
			pmo->setName( key );	
#endif
	}

done:
	Raids( RaidMethod ) 
		<< this << "::create( key " << key << " ) returns " << pmo
		<< " status " << status << "\n" << eor;
	return pmo;
}

int RemoteDictionary::add( char * key, PersistentMemoryObject * pmo )
{
	Raids( RaidMethod ) << this
		<< "::add"
		<< " this = " << asHex((unsigned) this)
		<< " key = " << key
		<< " pmo = " << pmo
		<< "\n" << eor;

	int status = 0;

	if ( !sharesContainer( pmo ) ) {
		status = DifferentContainers;
	} else {
		int links;
  	        status = RemoteFileSystemRPC::add
			( *(handle()), key, pmo->idNumber(), &links );
		if ( status == 0 ) {
			((RemoteFileObject *) pmo)->setLinks( links );
			TheRemotePathCache->bind
				( this, key, 
				  *(((RemoteFileObject *) pmo)->handle()) );
		}
	}
	return status;
}

int RemoteDictionary::remove( char * key, int * remainingLinks )
{
	Raids( RaidMethod ) << this
		<< "::remove"
		<< " this = " << asHex((unsigned) this)
		<< " key = " << key
		<< "\n" << eor;

	int links;
        int status = RemoteFileSystemRPC::remove( *(handle()), key, &links );

	if ( status != 0 )
		return status;

	RFSHandle fileHandle;
	int success = TheRemotePathCache->lookup( this, key, &fileHandle );
	if ( !success ) 
		return status;

	// found in name cache, first unbind from name cache
	TheRemotePathCache->unbind( this, key );
	RemoteContainer * container = 
		TheRemoteContainerRegistry->lookup
			( fileHandle._containerHandle );
	FileObject * fo = TheContainerIndex->lookup
			( container, fileHandle._idNumber );
	if ( fo != 0 ) {
		// file object exists in this host
		PersistentMemoryObjectStar pmo = 
			container->open( fileHandle._idNumber, status, 0 );
		((RemoteFileObject *) pmo)->setLinks( links );
	}
	if ( remainingLinks ) 
		*remainingLinks = links;
	return status;
}

int RemoteDictionary::keys( char * buffer, int bufferSize, int & count )
{
	Raids( RaidMethod ) << this
		<< "::keys"
		<< " this = " << asHex((unsigned) this)
		<< " buffer = " << asHex((unsigned) buffer)
		<< " bufferSize = " << bufferSize
		<< " count = " << asHex((unsigned) &count)
		<< "\n" << eor;

        int status = RemoteFileSystemRPC::keys
		( *(handle()), buffer, bufferSize, &count );
				  
        Raids( RaidMethod ) << this
		<< "::keys"
		<< " count = " << count
		<< " buffer = " << buffer
                << "\n" << eor;

	return status;
}

int RemoteDictionary::associations( char * buffer, int bufferSize,
				    int & count ) 
{
	Raids( RaidMethod ) << this
		<< "::associations"
		<< " this = " << asHex((unsigned) this)
		<< " buffer = " << asHex((unsigned) buffer)
		<< " bufferSize = " << bufferSize
		<< " count = " << asHex((unsigned) &count)
		<< "\n" << eor;

	int startOffset = -1;
	int resultOffset;
        int status = RemoteFileSystemRPC::associations
		( *(handle()), buffer, bufferSize, startOffset, 
		  &count, &resultOffset );
				  
        Raids( RaidMethod ) << this
		<< "::keys"
		<< " count = " << count
		<< " buffer = " << buffer
                << "\n" << eor;

	return status;
}

int RemoteDictionary::associations( char * buffer, int bufferSize,
				    int & count, int & offset ) 
{
	Raids( RaidMethod ) << this
		<< "::associations"
		<< " this = " << asHex((unsigned) this)
		<< " buffer = " << asHex((unsigned) buffer)
		<< " bufferSize = " << bufferSize
		<< " count = " << asHex((unsigned) &count)
		<< " offset = " << offset
		<< "\n" << eor;

	int startOffset = offset;
	int resultOffset;
        int status = RemoteFileSystemRPC::associations
		( *(handle()), buffer, bufferSize, startOffset, 
		  &count, &resultOffset );
	offset = resultOffset;
				  
        Raids( RaidMethod ) << this
		<< "::keys"
		<< " count = " << count
		<< " buffer = " << buffer
		<< " offset = " << offset
                << "\n" << eor;

	return status;
}

int RemoteDictionary::keySize()
{
        Raids( RaidMethod ) << this
		<< "::keySize"
                << " this = " << asHex((unsigned) this)
                << "\n" << eor;
	Assert( NOTREACHED );
}

ProxiableObject *
RemoteDictionaryConstructor( ProxiableObject * source )
{
        if( source->isKindOf( RemoteFileObjectClass ) ) 
                return( new RemoteDictionary
			( (PersistentMemoryObject *) source ) );
        else return( 0 );
}

int 
RemoteDictionary::sharesContainer( PersistentMemoryObject * pmo )
{
	Assert( _source->isKindOf( RemoteFileObjectClass ) );
	if ( ! pmo->isKindOf( RemoteFileObjectClass ) ) 
		return 0;
	if ( handle()->_containerHandle != 
	     ( (RemoteFileObject *) pmo )->handle()->_containerHandle )
		return 0;
	return 1;
}

#include "ClassInstaller.h"
DEFINEPERSISTENTCLASS("RemoteDictionary",RemoteDictionary,MemoryObjectDictionary,&RemoteDictionaryConstructor,0);
