#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * RemoteFileSystem.cc 
 */

#include "Assert.h"
#include "Class.h"
#include "ClassStar.h"
#include "FileSystem.h"
#include "Log2.h"
#include "PersistentClass.h"
#include "PersistentClassStar.h"
#include "PersistentMemoryObject.h"
#include "PersistentMemoryObjectStar.h"
#include "RemoteContainer.h"
#include "RemoteDictionary.h"
#ifdef __GNUC__
#pragma implementation "RemoteFileSystem.h"
#endif
#include "RemoteFileSystem.h"
#include "RemoteFileSystemRPC.h"
#include "RemoteTables.h"
#include "UNIXFile.h"

#ifdef __GNUC__
#pragma implementation
#endif

/*
 *  remote mount method
 */

unsigned RemoteFileSystemMaxDataSize = 48 * 1024;

PersistentMemoryObjectRef
RemoteFileSystemMount( char * special, char * mountPoint, int & status, 
		       int flags )
{
	RemoteFileSystemRPC::clientInit();
	Assert( special != 0 );
	char * serverName = special;
	char * p = special;
        for( ; ( *p != '\0' ) && ( *p != '/' ) && ( *p != '!' ); p++ );
	char temp = *p;
	*p = 0;
	extern unsigned long inet_addr( char * p );
	unsigned long server = inet_addr( serverName );

	extern unsigned long inet_myaddr();
	if ( server == inet_myaddr() ) {
		status = NotImplemented;
		return 0;
	}

	char * serverMountPoint = p;
	*serverMountPoint = temp;

	status = 0;

	Assert( mountPoint != 0 );

	if ( mountPoint[0] != '/' && mountPoint[0] != 0 ) {
		status = DirectoryInvalid;
		return 0;
	}
					
	RFSHandle handle;
	RFSAttributes attributes;
        status = RemoteFileSystemRPC::mount
		( server, serverMountPoint, &handle, &attributes );
	if (status != 0) 
		return 0;

        RemoteContainer * container = TheRemoteContainerRegistry->lookup
                ( handle._containerHandle );
        PersistentMemoryObjectRef pmo = 
		container->fastOpen( handle, attributes, status, 0, 0 );
	return pmo;
}

extern PersistentClassStar MemoryObjectDictionaryClass;
extern PersistentClassStar SymbolicLinkClass;
extern PersistentClassStar PersistentArrayClass;
extern ClassStar PersistentMemoryObjectClass;

static struct RFSClassTable {
	RFSObjectTypeEnum value;
	ClassStar * aClass;
} rfsClassTable[] = {
	{ NormalFile, &PersistentMemoryObjectClass },
	{ NormalFile, &PersistentArrayClass },
	{ Dictionary, &MemoryObjectDictionaryClass },
	{ SymbolicLink, &SymbolicLinkClass },
	{ Unknown, 0 } 
};

RFSObjectType::RFSObjectType( Class * aClass )
{
	RFSClassTable * p = rfsClassTable;
	while ( !aClass->isASubclassOf( *p->aClass ) ) {
		Assert( p->value != Unknown );
		p++;
	}
	_type = p->value;
}

Class *
RFSObjectType::objectClass()
{
	RFSClassTable * p = rfsClassTable;
	while ( p->value != _type ) {
		Assert( p->value != Unknown );
		p++;
	}
	return (Class *) ( *p->aClass );
}

void
RFSObjectType::set( PersistentMemoryObject * pmo )
{
	if ( pmo->isDictionary() )
		_type = Dictionary;
	else if ( pmo->isSymbolicLink() )
		_type = SymbolicLink;
	else if ( pmo->isContainer() ) {
		_type = Unknown; // not supported for now
		Assert( NOTREACHED );
	} else
		_type = NormalFile;
}
