#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ResourceLock.cc
 */

#include "BlockedInfo.h"
#include "ResourceLock.h"
#include "Process.h"
#include "PROCESSOR.h"
#include "ThisProcess.h"

#ifdef __GNUC__
#pragma implementation
#endif

ResourceLock::ResourceLock( int max )
        : _max( max ), _inUse( 0 ), _waiting( 0 ), _active( 0 ), 
	  _maxActive( 0 ), _maxWaiting( 0 ) 
{
}

void
ResourceLock::acquire( int amount )
{
	IntStorage intState ; PROCESSOR::disableInterrupts(intState);
	_lock.acquire();
	if ( _inUse + amount > _max || _waiting > 0 ) {
		Process * process = thisProcess();
		Assert( process != 0 );
		_WaitInfo * info = (_WaitInfo *) process->blockedInfo();
		info->setAmount( amount );
		_blocked.addTail( process );
		_waiting++;
		if ( _waiting > _maxWaiting )
			_maxWaiting = _waiting;
		_lock.release();
		process->block( this );
	} else {
		_inUse += amount;
		_active++;
		if ( _active > _maxActive )
			_maxActive = _active;
		_lock.release();
	}
	PROCESSOR::restoreInterrupts( intState );
}

void
ResourceLock::release( int amount )
{
	_lock.acquire();
	_inUse -= amount;
	_active--;
	Assert( _inUse >= 0 );
	Process * process = _blocked.head();
	while( process != 0 ) {
		_WaitInfo * info = ( _WaitInfo * ) process->blockedInfo();
		if ( _inUse + info->amount() > _max ) 
			break;
		_blocked.remove( process );
		_waiting--;
		_inUse += info->amount();
		_active++;
		if ( _active > _maxActive )
			_maxActive = _active;
		process->ready();
		process = info->next();
	}
	_lock.release();
}

