#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * RootEntry.c - Ar Root Directory Entry (inode) code
 */
#include "RootEntry.h"
#include "Assert.h"
#include "Raid.h"
#include "UNIXFile.h"
#include "Class.h"

#ifdef __GNUC__
#pragma implementation
#endif

RootEntry::RootEntry( unsigned int numberOfUnits,
		      int log2UnitSize,
		      int inumber,
		      MemoryObjectContainer * container,
		      PersistentMemoryObject * source,
		      Class * supports,
		      int offset )
: FileObject( numberOfUnits, log2UnitSize, inumber, container, source, offset ),
  _supports( supports )
{
	initDebug();
	Raid( RaidConstructor )( "%N::RootEntry( %N )\n", this, supports );

	Assert( _supports != 0 );
}

Class *
RootEntry::supports( Class * aClass )
{
	Raid( RaidProtected )( "%N::supports( %N )\n", this, aClass );

	Class * supportedClass = 0;

        if( _supports->isASubclassOf( aClass ) ) supportedClass = _supports;

	Raid( RaidProtected )( "%N::supports() returns %N\n",
				this, supportedClass );

	return( supportedClass );
}

int
RootEntry::isDictionary()
{
	return 1;
}

void
RootEntry::info( UNIXFileInfo * infoBuffer )
{
    Raid( RaidMethod )( "%N::info( ) %d\n", this, _idNumber );
    infoBuffer->device	= 0;
    infoBuffer->inumber	= _idNumber;
    infoBuffer->mode	= UNIXTypeDirectory | accessMode();
    infoBuffer->links	= 1;
    infoBuffer->uid	= userId();
    infoBuffer->gid	= groupId();
    infoBuffer->flags	= 0;	//FIX
    infoBuffer->rdevice	= 0;	//FIX
    infoBuffer->size	= numberOfBytes();
    infoBuffer->atime	= 0;
    infoBuffer->mtime	= 0;
    infoBuffer->ctime	= 0;
    infoBuffer->blockSize = unitSize();
    infoBuffer->blocks	= numberOfUnits();
}

int
RootEntry::accessMode()
{
	return( UNIXReadMask|UNIXExecuteMask );
}


#include "ClassInstaller.h"
DEFINECLASS("RootEntry",RootEntry,FileObject,0,0);
