#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SVIDAllocator.c - 
 */
#include "Assert.h"
#include "Raid.h"
#include "SVIDAllocator.h"
#include "SVIDContainer.h"

#ifdef __GNUC__
#pragma implementation
#endif

SVIDAllocator::SVIDAllocator( MemoryObject * source, filsys * superBlock,
			      int log2FragmentToSector ) : BlockAllocator(source )
{
    initDebug();
    Raid( RaidConstructor )( "%N::SVIDAllocator( %N, %A, %d )\n",
			     this, source, superBlock, log2FragmentToSector );

    _superBlock = superBlock;
    _log2FragmentToSector = log2FragmentToSector;
}

/*
 * Allocate a single new block.
 */
int
SVIDAllocator::basicAllocate( int &, int, int )
{
    Raid( RaidProtected )( "%N::basicAllocate( 0, 0, 1 )\n", this );

    // Get a block off the free list
    int newBlock = _superBlock->s_free[--_superBlock->s_nfree];

    // There are no free blocks left
    if( newBlock == 0 ) return( 0 );

    // If last of free list then rebuild free list.  Free list contains
    // pointer to first block in chain of free list followed by up to
    // 50 free blocks.  When these free blocks are used up, the free list
    // is constructed by copying the blocks indicated in the first entry
    // of the chain.  Each block in the chain contains the count of free
    // block pointers, pointer to the next block in the chain, and then
    // the free block pointers.
    if( _superBlock->s_nfree == 0 ) {
	int * block = new int [( _source->unitSize() << _log2FragmentToSector ) >> 2];
	_source->read( newBlock, 1, (char *) block,
			_source->unitSize() << _log2FragmentToSector );
	_superBlock->s_nfree = block[0];
	for( int i = 1; i <= NICFREE; i++ )
	    _superBlock->s_free[i-1] = block[i];
	delete block;
    }

    // Decrement count of free blocks
    _superBlock->s_tfree--;

    Raid( RaidProtected )( "SVIDAllocator::basicAllocate = %d\n", newBlock );
    return( newBlock );
}

/*
 *	Free the specified block.
 */
void
SVIDAllocator::basicFree( int &, int oldBlock, int ) 
{
    Raid( RaidProtected )( "%N::basicFree( 1, %d, 0 )\n", this, oldBlock );

    // If the free list in the superblock is full then put it in this
    // block, put this block at the front of the chain, and have the 
    // free list point to this block.
    if( _superBlock->s_nfree == NICFREE ) {
	int * buffer = new int [( _source->unitSize() << _log2FragmentToSector ) >> 2];

	// Create block containing the current free list.
	buffer[0] = NICFREE;
	for( int i = 0; i < NICFREE; i++ )
	    buffer[i+1] = _superBlock->s_free[i];

	// Write the block to disk.
	_source->write( oldBlock, 1, (char *) buffer, 
			_source->unitSize() << _log2FragmentToSector );

	// Update the free list to point to the block just written.
	_superBlock->s_nfree = 0;
	delete buffer;
    }

    // Put this block in the free list
    _superBlock->s_free[_superBlock->s_nfree++] = oldBlock;
    _superBlock->s_tfree++;
}


#include "ClassInstaller.h"
DEFINECLASS("SVIDAllocator",SVIDAllocator,BlockAllocator,0,0);
