#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Lun Xiao (xiao@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * TarContainer.cc - Tar Inode System class
 */

#include "Tar.h"
#include "TarContainer.h"
#include "TarEntry.h"
#include "TarRootEntry.h"
#include "FileSystem.h"
#include "Convert.h"
#include "String.h"
#include "Raid.h"

#ifdef __GNUC__
#pragma implementation
#endif

FileObject *
TarContainer::basicOpen( int id, int & status, int flags )
{
	Raid( RaidMethod )( "%N::basicOpen( inumber %d )\n", this, id );

	FileObject * fo = 0;
	if( id < 1 ) status = IdNumberOutOfRange;
	else {
		status = 0;
		if( id == 1 ) fo = new TarRootEntry( id, this, _source );
		else fo = new TarEntry(  _lengths[id], 9, id, this, _source,
					_offsets[id] + (TarchiveHeaderSize/TBLOCK),
					&_tar[id] );
	}

	Raid( RaidMethod )( "%N::basicOpen returns %N %d\n", this, fo, status );

	return( fo );
}

TarContainer::~TarContainer()
{
	Raid( RaidConstructor )( "%N::~TarContainer()\n", this );

	if( _valid ) {
		for( int i = 2; i < _numberOfFileObjects; i++ ) {
			_byteSource->write( _offsets[i], TarchiveHeaderSize,
					 (char *)&_tar[i], TarchiveHeaderSize );
		}
		delete _tar;
		delete _lengths;
		delete _offsets;
	}

	Raid( RaidConstructor )( "%N::~TarContainer done\n", this );
}

TarContainer::TarContainer( PersistentMemoryObject * source )
: ( source ), _byteSource( 0 )
{
	initDebug();
	_valid = 1;

	Raid( RaidConstructor )( "%N::TarContainer( %N )\n", this, _source );
	//int units = _source->numberOfUnits();
	int bytes = _source->numberOfBytes();
	if( bytes == 0 ) {
		_valid = 0;
		_offsets = 0;
		_lengths = 0;
		_tar = 0;
		initializeArrays( 0 );
	}
	else {
		_byteSource = new FileObject( bytes, 0, 0, 0, _source );

		int pos = 0;
		for( int objects = 2; ( pos < bytes  ) && _valid; objects++ ) {
			tar_hblock tar;
			_byteSource->read( pos, TarchiveHeaderSize, 
					(char *)&tar, TarchiveHeaderSize );

			int size = atoi( tar.dbuf.tar_size, 8 );
			if (( size % TBLOCK) != 0)
				size = ((size / TBLOCK) + 1) * TBLOCK;
			pos += TarchiveHeaderSize + size;
		}
		if( !_valid ) objects = 0;

		initializeArrays( objects );
		
		_offsets = new int[objects];
		_lengths = new int[objects];
		_tar = new tar_hblock[objects];

		pos = 0;
		_offsets[0] = 0;	_lengths[0] = 0;
		_offsets[1] = 1;	_lengths[1] = 0;

		for( objects = 2; objects < _numberOfFileObjects; objects++ ) {
			Assert( pos < bytes );

			_offsets[objects] = (pos/ TBLOCK);
			//_offsets[objects] = pos;
			tar_hblock * tar = &_tar[objects];
			_byteSource->read( pos, TarchiveHeaderSize, 
					(char *)tar, TarchiveHeaderSize );
		//Assert(strncmp(ar->ar_fmag, ArchiveFileMagic, 2) == 0);

			int size = atoi( tar->dbuf.tar_size, 8 );
			_lengths[objects] = (size / TBLOCK) + 1;
			if (( size % TBLOCK) != 0)
				size = ((size / TBLOCK) + 1) * TBLOCK;
			pos += TarchiveHeaderSize + size;
		}
	}
}

ProxiableObject *
TarContainerConstructor( ProxiableObject * source )
{
	if( source->isKindOf( PersistentMemoryObjectClass ) ) {
		return( new TarContainer( (PersistentMemoryObject *)source ) );
	}
	else return( 0 );
}

#include "ClassInstaller.h"
DEFINEPERSISTENTCLASS("TarContainer",TarContainer,MemoryObjectContainer,&TarContainerConstructor,0);
