#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Lun Xiao (xiao@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * TarDictionary.cc - Tar Dictionary class
 */

#include "Raid.h"
#include "Tar.h"
#include "TarDictionary.h"
#include "FileSystem.h"
#include "Convert.h"
#include "String.h"
#include "PersistentMemoryObject.h"

#ifdef __GNUC__
#pragma implementation
#endif

int
TarDictionary::findKey( char * key, int )
{
	_emptyOffset = -1;
	int len = strlen( key );
	if( ( strcmp( key, "." ) == 0 ) || ( strcmp( key, ".." ) == 0 ) ) {
		_idNumberOfKey = 1;
		return( 1 );
	}
	else for( int i = 2; i < _objects; i++ ) {
		tar_hblock * tar = (tar_hblock *)&_buffer[_offsets[i]];
		if( strncmp( key, (tar->dbuf).tar_name, len ) == 0 ) {
			if( (tar->dbuf).tar_name[len] == '\0' ) {
				_idNumberOfKey = i;
				_offset = _offsets[i];
				return( 1 );
			}
		}
	}
	return( 0 );
}

int
TarDictionary::basicKeys( char * keyBuffer, int bufferSize, int & count )
{
    Raid( RaidMethod )( "%N::basicKeys( buffer %x ) _size %d\n", 
		this, keyBuffer, bufferSize, _size );

    int limit = bufferSize >= _size ? _size : bufferSize;
    char * t = keyBuffer;
// FIX check for extra 5 chars
    *t++ = '.';
    *t++ = '\n';
    *t++ = '.';
    *t++ = '.';
    *t++ = '\n';
    count = 2;

    for( int i = 2; i < _objects; i++ ) {
		tar_hblock * tar = (tar_hblock *)&_buffer[_offsets[i]];
		char * s = (tar->dbuf).tar_name;
		if (*s == '\0') break;
		while( *s != '\0' ) *t++ = *s++;
		*t++ = '\n';
    }
    *t++ = '\0';

//FIX check this
    if( limit == _size ) return( 0 );
    else return( TooManyKeys );
}

TarDictionary::~TarDictionary()
{
	Raid( RaidConstructor ) ( "%N::~TarDictionary()\n", this );
	delete _offsets;
}

TarDictionary::TarDictionary( PersistentMemoryObject * source )
: ( source )
{
	initDebug();
	Raid( RaidConstructor ) ( "%N::TarDictionary( %N )\n", this, source );

	_maxKeyLength = 16;

	_offsets = new int[_size/TarchiveHeaderSize];
	//int pos = TarchiveMagicSize;
	int pos = 0;
	_offsets[0] = 0;
	_offsets[1] = 1;

        tar_hblock * tar;
        //tar_hblock * tar = new tar_hblock;
	for( _objects = 2; pos < _size; _objects++ ) {
		tar = (tar_hblock *)&_buffer[pos];
		char * s = (tar->dbuf).tar_name;
		if (*s == '\0') break;
		_offsets[_objects] = pos;

		int size = atoi( tar->dbuf.tar_size, 8 );
Raid( RaidDetail) ( "%N::TarDictionary objects = %d pos = %d size=%d\n", this, _objects, pos, size); 
		if ((size % TBLOCK) != 0)
			size = ((size / TBLOCK) + 1) * TBLOCK;
		pos += TarchiveHeaderSize + size;
	}
	//delete tar;
}

ProxiableObject *
TarDictionaryConstructor( ProxiableObject * source )
{
	if( source->isKindOf( PersistentMemoryObjectClass ) ) {
		return( new TarDictionary( (PersistentMemoryObject *)source ) );
	}
	else return( 0 );
}


#include "ClassInstaller.h"
DEFINEPERSISTENTCLASS("TarDictionary",TarDictionary,MemoryObjectDictionary,&TarDictionaryConstructor,0);
