#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Lun Xiao (xiao@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * TarEntry.c - Tar  Directory Entry (inode) code
 */

#include "Assert.h"
#include "FileSystem.h"
#include "UNIXFile.h"
#include "TarEntry.h"
#include "Tar.h"
#include "Convert.h"
#include "String.h"
#include "Class.h"
#include "Raid.h"

#ifdef __GNUC__
#pragma implementation
#endif

extern ClassStar PersistentArrayClass;

TarEntry::TarEntry( unsigned int numberOfUnits,
		  int log2UnitSize,
                  int objectNumber,
		  MemoryObjectContainer * container,
		  PersistentMemoryObject * source,
		  int offset,
		  tar_hblock * tar )
: ( numberOfUnits, log2UnitSize, objectNumber, container, source, offset ) 
{
	initDebug();
	Raid( RaidConstructor )( "%N::TarEntry()\n", this );
	_tar = tar;
}

Class *
TarEntry::supports( Class * aClass )
{
	Raid( RaidProtected )( "%N::supports( %N )\n", this, aClass );

	Class * supportedClass = 0;

        if( PersistentArrayClass->isASubclassOf( aClass ) ) {
            supportedClass = PersistentArrayClass;
        }

	Raid( RaidProtected )( "%N::supports() returns %N\n",
				this, supportedClass );

	return( supportedClass );
}

int
TarEntry::numberOfBytes()
{
    int bytes = ( atoi( _tar->dbuf.tar_size, 8 ) );
    Raid( RaidMethod )( "%N::numberOfBytes() %d\n", this, bytes );
    return( bytes );
}


int
TarEntry::setMode( int mode )
{
    Raid( RaidMethod )( "%N::setMode( %o )\n", this, mode );
    for( int i = 0; i < 4; i++ ) {
	int shift = i * 3;
	int mask = 7 << shift;
	_tar->dbuf.tar_mode[5 - i] = '0' + ( ( mode & mask ) >> shift );
    }
    return( 0 );
}


int
TarEntry::accessMode()
{
    _tar->dbuf.tar_mode[0]= '1';
    _tar->dbuf.tar_mode[1]= '0';
    _tar->dbuf.tar_mode[2]= '0';
    
    return( atoi( _tar->dbuf.tar_mode, 8 ) & UNIXAccessMask );
}

int
TarEntry::userId()
{
    return( atoi( _tar->dbuf.tar_uid, 8 ) );
}

int
TarEntry::groupId()
{
    return( atoi( _tar->dbuf.tar_gid, 8 ) );
}

void
TarEntry::info( UNIXFileInfo * infoBuffer )
{
    PersistentMemoryObject::info( infoBuffer );
    infoBuffer->mode = atoi( _tar->dbuf.tar_mode, 8 );
}


#include "ClassInstaller.h"
DEFINECLASS("TarEntry",TarEntry,FileObject,0,0);
