#ifdef __GNUC__
#pragma implementation
#endif

#define DEBUG

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *VCDictionary.c -
 */

#include "Assert.h"
#include "Class.h"
#include "FileSystem.h"
// #include "FileObjectStar.h"
#include "FileObject.h"
#include "FileObject.h"
#include "OutputStream.h"
#include "Raid.h"
#include "VCDictionary.h"
#include "VCDictionaryStar.h"
#include "VCFileObject.h"
#include "PersistentMemoryObject.h"
// #include "PersistentMemoryObjectStar.h"
#include "MemoryObjectDictionary.h"
#include "PersistentClass.h"
#include "Raid.h"
#include "Byte.h"
#include "String.h"
extern "C" {
#include <sys/types.h>
#include <sys/stat.h>
#include <dirent.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
int getdents(int, struct dirent*, int);
};

#ifdef __GNUC__
#pragma implementation
#endif

class direct {
	friend class VCDictionary;

	long    d_off;
	long    d_ino;
	short	d_reclen;
	short	d_namlen;
	char	d_name[256];
};


class Table{
public:
	int fpointer;
	PersistentMemoryObject *  _fileObject;
	char fileName[256];
	Table(){_fileObject = 0;}
};

/*
 * This should only be called from FileSystemInterface
 * to create a mount point
 */
VCDictionary::VCDictionary( PersistentMemoryObject * source)
	: MemoryObjectDictionary( source ), _source((VCFileObject*)source)
{
	Raid( RaidConstructor )("%N::VCDictionary()\n", this);
	Assert (_source != 0);
	_maxKeyLength = 255;
}

char *
VCDictionary::mypath()
{
	return _source->path();
}

VCDictionary::~VCDictionary( )
{
	Raid( RaidConstructor )("%N::~VCDictionary()\n", this);
}

PersistentMemoryObjectRef
VCDictionary::open( char * key, int & status, int flags )
{
	Raids( RaidMethod ) << this << "::open"
		<< " key = " << key << " \n" << eor;
	status = 0;

	if (strcmp(".", key) == 0 || strlen(key) == 0)
		return _source;


	char wholePath[ strlen(mypath()) + strlen(key) + 1];
	strcpy(wholePath, mypath() );
	if (strcmp("..", key) == 0){
		char *p = &wholePath[strlen(mypath()) - 1];
		while (*p != '/' && p > wholePath)
			p--;
		if (p <= wholePath)
			return _source->root();
		else 
			*p = 0;
		}
	else {
		Assert (strlen(key) > 0);
		strcat(wholePath, "/"); 
		strcat(wholePath, key); 
	}

	Assert (strlen(wholePath) > 0);
	struct stat buf;
	if (stat(wholePath, &buf) != 0) {
		status = KeyNotFound;
		return( 0 );
	}

	Assert (_source != 0);
	VCFileObject * _fileObject = new VCFileObject(wholePath,
		_source->root());
	_fileObject->setName(wholePath);
	Raids( RaidMethod ) << this << "::open done" << " path = " << key
		<< " status = " << status << "\n" << eor;

	return _fileObject;
}

PersistentMemoryObjectRef
VCDictionary::create( char * key, Class * aClass, int & status, int flags)
{
	Raids( RaidMethod ) << this
		<< "::create"
		<< " this = " << asHex((unsigned) this)
		<< " key = " << key
		<< " aClass = " << aClass
		<< "\n" << eor;
        status = 0;

	char wholePath[1000];
        strcpy(wholePath, mypath());
	strcat(wholePath, "/"); 
        strcat(wholePath, key); 

        if ( MemoryObjectDictionaryClass->isASubclassOf( aClass ) ){
		int r = ::mkdir(wholePath, 0666 );
		Assert (r != -1);
		VCFileObject * fo = new VCFileObject(wholePath,
			_source->root());
		fo->setName(wholePath);
		VCDictionaryStar d = new VCDictionary(fo);
		d->reference();
		d->setName(wholePath);
		return (d);
		}
        else {
		int r = ::creat(wholePath, 0666 );
		Assert (r != -1);
		VCFileObject * fo = new VCFileObject(wholePath,
			_source->root());
		fo->setName(wholePath);
		return (fo);
	}

}

int 
VCDictionary::remove( char * key, int * links )
{
	Raids( RaidMethod ) << this
		<< "::remove"
		<< " this = " << asHex((unsigned) this)
		<< " key = " << key
		<< "\n" << eor;

	char wholePath[strlen(mypath()) + strlen(key) + 1];
        strcpy(wholePath, mypath());
	strcat(wholePath, "/"); 
        strcat(wholePath, key); 

	if ( links ) {
		struct stat buf;
		int i = ::stat(wholePath, &buf);
		if ( i != -1 ) {
			*links = buf.st_nlink - 1;
		}
	}
        int i = ::unlink(wholePath);
	if ( i == -1 ) Console().printf("VCDictionary::remove(%s): unlink error %d \n", key, errno);

	return( 0 );
}

int 
VCDictionary::keys( char * buffer, int bufferSize, int & count )
{
	Raids( RaidMethod ) << this
		<< "::keys"
		<< " this = " << asHex((unsigned) this)
		<< " buffer = " << asHex((unsigned) buffer)
		<< " bufferSize = " << bufferSize
		<< " count = " << asHex((unsigned) &count)
		<< "\n" << eor;
	count = 0;
	char tmpBuff[bufferSize];
	int _fp = ::open(mypath(), O_RDONLY, 0);
	Assert (_fp != -1);
	int dentbytes = getdirentries(_fp, tmpBuff, bufferSize, 0);
	Assert (dentbytes != -1);
	::close(_fp);


	int limit = ( bufferSize >= dentbytes ) ? dentbytes : bufferSize;
        char * t = buffer;
        direct * dp = ( direct * ) tmpBuff;
        for( _offset = 0; _offset < limit; _offset += dp->d_reclen ) {

            dp = ( direct * ) &tmpBuff[_offset];

            Raid( RaidDetail )( "%N::keys ofs %d ", this, _offset);
            Raid( RaidDetail )( "ino %d rl %d nl %d key %s\n",
                       dp->d_ino, dp->d_reclen, dp->d_namlen, dp->d_name );

            Assert( dp->d_reclen > dp->d_namlen );

            if( dp->d_ino != 0) {
                    Assert( dp->d_namlen > 0 );

                    if( t != 0 ) {
                            for( char * s = dp->d_name; *s; ) *t++ = *s++;
                            *t++ = '\n';
                    }
                    count++;
            }
        }
        if( t != 0 ) *t = '\0';
        if( limit == dentbytes ) {
		return( 0 );
	}
        else return( TooManyKeys );
}

void
VCDictionary::noRemainingReferences()
{
        Raids( RaidMethod ) << this
		<< "::noRemainingReferences"
                << "\n" << eor;
	// delete this;
}

ProxiableObject *
VCDictionaryConstructor( ProxiableObject * source )
{
        if( source->isKindOf( VCFileObjectClass ) ) {
                return( new VCDictionary( (PersistentMemoryObject *)source) );
        }
        else return( 0 );
}

#include "ClassInstaller.h"
DEFINECLASS("VCDictionary",VCDictionary,MemoryObjectDictionary,&VCDictionaryConstructor,0);
