#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * VCFileObject.c - 
 */
#include "Assert.h"
#include "Byte.h"
#include "Class.h"
#include "FileSystem.h"
#include "FileSystemInterface.h"
#include "FileSystemInterfaceStar.h"
#include "Kernel.h"
#include "MemoryObjectAsync.h"
#include "NameServer.h"
#include "PersistentObject.h"
#include "PersistentObjectStar.h"
#include "PersistentClass.h"
#include "Raid.h"
#include "RecordStream.h"
#include "VCDictionary.h"
#include "VCFileObject.h"
//#include "String.h"
#include "UNIXFile.h"
#include "Byte.h"
extern "C" {
#include <sys/types.h>
  //#include <sys/dirent.h>
#include <sys/errno.h>
#include <unistd.h>
#include <fcntl.h>
};

#ifdef __GNUC__
#pragma implementation
#endif

void
VCFileObject::noRemainingReferences()
{
	delete this;
}

VCFileObject::VCFileObject(char * path, VCFileObject * root)
	: FileObject( 0, 0, 0, 0, 0, 0 )
{
	initDebug();
	Raids( RaidConstructor ) << this
		  << "::VCFileObject"
		  << "\n" << eor;

	_path = new char[strlen(path) + 1];
	strcpy(_path, path);
	_fd = -1;
	restat();
	_log2UnitSize = 10;
	_numberOfUnits = (_buf.st_size + unitSize() - 1) >> _log2UnitSize;

	if (root == 0)
		_root = this;
	else
		_root = root;

}

VCFileObject::~VCFileObject( )
{
	Raids( RaidConstructor ) << this
		<< "::~VCFileObject"
		<< "\n" << eor;
        delete _path;
}

Class *
VCFileObject::supports( Class * aClass )
{
        Raid( RaidProtected )( "%N::supports( %N )\n", this, aClass );

        Class * supportedClass = 0;

	extern ClassStar PersistentArrayClass;

	if (PersistentArrayClass->isASubclassOf(aClass))
		supportedClass = PersistentArrayClass;
	else if (VCDictionaryClass->isASubclassOf(aClass))
		supportedClass = VCDictionaryClass;

        Raid( RaidProtected )( "%N::supports() returns %N \n",
                                this, supportedClass );

        return( supportedClass );
}

int 
VCFileObject::isDictionary()
{
	return( supports( MemoryObjectDictionaryClass ) != 0 );
}

int
VCFileObject::rawRead( unsigned startUnit, int count, char * dest, int nbytes,
		       IOAsyncClosure * c )
{
        Raids( RaidProtected ) << this 
		<< "::rawRead( startUnit " << startUnit
		<< ", count " << count 
		<< ", dest " << asHex( (int) dest )
		<< ", nbytes " << nbytes 
		<< " )\n" << eor;

	// do not support true async operation for now

	_fd = ::open(_path, O_RDONLY, 0);
	Assert (_fd != -1);
	int offset = startUnit*unitSize();
	int r = lseek(_fd, offset, SEEK_SET);
	Assert (r == offset);
        int b = ::read(_fd, dest, nbytes);	
	::close(_fd);

	int retval = (b + unitSize() - 1) >> log2UnitSize();

	if ( c ) {
		c->activate( retval );
	}

	Raids( RaidProtected ) << this
		<< "::rawRead done"
		<< " return = " << retval 
		<< "\n" << eor;
	return retval;
}

int
VCFileObject::rawWrite( unsigned startUnit, int count, char * dest, 
		        int nbytes, int writeThrough, IOAsyncClosure * c )
{
        Raids( RaidProtected ) << this 
		<< "::rawWrite( startUnit " << startUnit
		<< ", count " << count 
		<< ", dest " << asHex( (int) dest )
		<< ", nbytes " << nbytes 
		<< ", writeThrough " << writeThrough
		<< " )\n" << eor;

	// do not support true async operation for now

	_fd = ::open(_path, O_RDWR, 0);
	Assert (_fd != -1);
	int offset = startUnit*unitSize();
	int r = lseek(_fd, offset, SEEK_SET);
	Assert (r == offset);
	int b = ::write(_fd, dest, nbytes);
	::close(_fd);

        int retval = (b + unitSize() - 1) >> log2UnitSize();

	if ( c ) {
		c->activate( retval );
	}

	Raids( RaidProtected ) << this
		<< "::rawWrite done"
		<< " return = " << retval 
		<< "\n" << eor;

	return retval;
}

void
VCFileObject::restat()
{
	_fd = ::open(_path, O_RDONLY, 0);
	Assert (_fd != -1);
	int r = fstat(_fd, &_buf);
	Assert (r == 0);
	::close(_fd);
}

int
VCFileObject::numberOfBytes()
{
	restat();
	return(_buf.st_size);
}

int
VCFileObject::accessMode()
{
	restat();
	return(_buf.st_mode & UNIXAccessMask) ;
}

int
VCFileObject::access( int mask )
{
	restat();
	return ((_buf.st_mode & mask) == (unsigned)mask);
}

int
VCFileObject::userId()
{
	restat();
	return(_buf.st_uid) ;
}

int
VCFileObject::groupId()
{
	restat();
	return(_buf.st_gid) ;
}

void
VCFileObject::info( UNIXFileInfo * infoBuffer )
{
    	Raid( RaidMethod )( "%N::info( ) %d\n", this, _idNumber );
    	restat();
    	infoBuffer->device  = _buf.st_dev;
    	infoBuffer->inumber = _buf.st_ino;
    	infoBuffer->mode    = _buf.st_mode;
    	infoBuffer->links   = _buf.st_nlink;
    	infoBuffer->uid     = _buf.st_uid;
    	infoBuffer->gid     = _buf.st_gid;
    	infoBuffer->size    = _buf.st_size;
    	infoBuffer->atime   = _buf.st_atime;
    	infoBuffer->mtime   = _buf.st_mtime;
    	infoBuffer->ctime   = _buf.st_ctime;
    	infoBuffer->blocks  = _buf.st_blocks;
    	infoBuffer->blockSize = _buf.st_blksize;
    	infoBuffer->flags   = 0;
}

int 
VCFileObject::fixedSize()
{
	return( 0 );
}

int
VCFileObject::basicSetNumberOfBytes( int bytes )
{
	restat();
	if (bytes == 0) {
		int i = ::creat(_path, O_TRUNC);
		Assert (i != -1);
		int r = chmod(_path, _buf.st_mode);
		Assert (r != -1);
		::close(i);
		}
	else
		{
		int r = ::truncate(_path, bytes);
		Assert (r == 0);
		}
	unsigned units = ( bytes + unitSize() - 1 ) >> log2UnitSize();
	basicSetNumberOfUnits( units );
        return bytes;
}

int
VCFileObject::transient()
{
        return 1;
}

unsigned
VCFileObject::basicSetNumberOfUnits( unsigned units )
{
	if ( _numberOfUnits != units )
	MemoryObject::basicSetNumberOfUnits( units );
	return units;
}

int
VCFileObject::setMode( int accessMode )
{
        return (chmod(_path, accessMode));
}

int
VCFileObject::setUserId( int uid )
{
	restat();
        return chown(_path, uid, _buf.st_gid);
}

int
VCFileObject::setGroupId( int gid )
{
	restat();
        return chown(_path, _buf.st_uid, gid);
}

void
VCFileObject::writeIdNumber( OutputStream * os )
{
	PersistentMemoryObject::writeIdNumber( os );
}

#include "ClassInstaller.h"
DEFINECLASS("VCFileObject",VCFileObject,FileObject,0,0);
