#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * VariableRecordFile.c - 
 */
#include "Assert.h"
#include "Raid.h"
#include "VariableRecordFile.h"
#include "PersistentMemoryObject.h"
#include "Byte.h"
#include "PersistentClass.h"
#include "PersistentIntArray.h"
#include "PersistentArray.h"
#include "FileSystem.h"

#ifdef __GNUC__
#pragma implementation
#endif

class FileSystemInterface;

VariableRecordFile::VariableRecordFile( PersistentMemoryObject * source )
: RecordFile( source )
{
	initDebug();

	Raids( RaidConstructor ) << this << "::VariableRecordFile( "
				 << source << " )\n" << eor;
}

VariableRecordFile::~VariableRecordFile()
{
	Raids( RaidConstructor ) << this << "::~VariableRecordFile()\n" << eor;
}

void
VariableRecordFile::flush()
{
	Raids( RaidReference ) << this << "::flush()\n" << eor;
	_data.flushStar();
	_control.flushStar();
}

int
VariableRecordFile::init( int, char *[], FileSystemInterface * )
{
	Raids( RaidMethod ) << this << "::init()\n" << eor;

	char * args[1];
	args[0] = "1";

	_data = PersistentArrayClass->newObject();
	_data->persist();
	_data->init( 1, args );

	_control = PersistentIntArrayClass->newObject();
	_control->persist();
	_control->init( 0, args );

	return( 0 );
}

int
VariableRecordFile::read( unsigned int record, char * buffer,
			  unsigned int size, int & status )
{
	Raid( RaidMethod )( "%N::read( %d, , %d, )\n", this, record, size );

	if( record < _control->size() ) {
		unsigned position = _control->at( record++ );
		unsigned nextPosition = ( record < _control->size() ) ?
				   _control->at( record ) : _data->size();

		unsigned recordSize = nextPosition - position;
		Assert( recordSize >= 0 );

		if( size > recordSize ) size = recordSize; 

		_data->at( position, buffer, size );

		status = 0;
	}
	else status = EndOfFile;

	Raid( RaidMethod )( "%N::read return( %d ) status %d\n",
			    this, ( ( status == 0 ) ? size : 0 ), status );
	return( ( status == 0 ) ? size : 0 );
}

int
VariableRecordFile::write( unsigned int record, char * buffer,
			   unsigned int size, int & status )
{
	Raid( RaidMethod )( "%N::write( %d, , %d, )\n", this, record, size );

	if( record == _control->size() ) {
		int position = _data->size();

		if( position == 0 ) {
			_minRecordSize = size;
			_maxRecordSize = size;
		}
		else {
			if( size < _minRecordSize ) _minRecordSize = size;
			if( size > _maxRecordSize ) _maxRecordSize = size;
		}

		_data->atPut( position, buffer, size );
		_control->atPut( record, position );

		status = 0;
	}
	else status = NotEndOfFile;

	Raid( RaidMethod )( "%N::write return( %d ) status %d\n",
			    this, ( ( status == 0 ) ? size : 0 ), status );
	return( ( status == 0 ) ? size : 0 );
}

unsigned
VariableRecordFile::numberOfRecords()
{
	Raid( RaidMethod )( "%N::numberOfRecords() %d\n",
			    this, _control->size() );

	return( _control->size() );
}

ProxiableObject *
VariableRecordFileConstructor( ProxiableObject * source )
{
        if( source->isKindOf( PersistentMemoryObjectClass ) ) {
		return( new VariableRecordFile( (PersistentMemoryObject *)source ) );
        }
        else return( 0 );
}

/*
int
sizeOfVariableRecordFile()
{
	return( sizeof( VariableRecordFile ) );
}

Class *
VariableRecordFile::classOf()
{
	return( VariableRecordFileClass );
}
*/


#include "ClassInstaller.h"
DEFINEPERSISTENTCLASS("VariableRecordFile",VariableRecordFile,RecordFile,&VariableRecordFileConstructor,0);
