#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * VariableRecordStream.c - 
 */
#include "Assert.h"
#include "FileSystem.h"
#include "Raid.h"
#include "VariableRecordStream.h"
#include "VariableRecordFile.h"
#include "PersistentMemoryObject.h"

#ifdef __GNUC__
#pragma implementation
#endif

VariableRecordStream::VariableRecordStream( PersistentObject * po )
: RecordStream( po )
{
    initDebug();
    Raids( RaidConstructor ) << this <<  "::VariableRecordStream( "
			     << po << " )\n" << eor;

    // UNUSED ??? int status = 0;

    _file = po;
}

VariableRecordStream::~VariableRecordStream()
{
    Raids( RaidConstructor ) << this << "::~VariableRecordStream()\n" << eor;
}

int
VariableRecordStream::read( char * buffer, int size, int & status )
{
    Raids( RaidMethod ) << this << "::read( , " << size << " , )\n" << eor;

    int result = _file->read( _recordNumber, buffer, size, status );
    if( status == 0 ) _recordNumber++;

    return( result );
}

int
VariableRecordStream::write( char * buffer, int size, int & status )
{
    Raids( RaidMethod ) << this << "::write( , " << size << " )\n" << eor;

    int result = _file->write( _recordNumber, buffer, size, status );
    if( status == 0 ) _recordNumber++;

    return( result );
}

int
VariableRecordStream::minimumRecordSize()
{
    Raids( RaidMethod ) << this << "::minimumRecordSizerecordSize() "
			<< _file->minimumRecordSize() << "\n" << eor;
    return( _file->minimumRecordSize() );
}

int
VariableRecordStream::maximumRecordSize()
{
    Raids( RaidMethod ) << this << "::maximumRecordSize() "
			<< _file->maximumRecordSize() << "\n" << eor;
    return( _file->maximumRecordSize() );
}


ProxiableObject *
VariableRecordStreamConstructor( ProxiableObject * source )
{
	if( source->isKindOf( VariableRecordFileClass ) ) {
		return( new VariableRecordStream( (PersistentObject *)source ) );
	}
	else return( 0 );
}

/*
int
sizeOfVariableRecordStream()
{
	return( sizeof( VariableRecordStream ) );
}

Class *
VariableRecordStream::classOf()
{
	return( VariableRecordStreamClass );
}
*/


#include "ClassInstaller.h"
DEFINECLASS("VariableRecordStream",VariableRecordStream,RecordStream,&VariableRecordStreamConstructor,0);
