#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Panagiotis Kougiouris (kougiour@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "OutputStream.h"
#include "BufferedInputLogicalDevice.h"
#include "CharacterQueue.h"
#include "InputLogicalDevice.h"


#ifdef __GNUC__
#pragma implementation
#endif


BufferedInputLogicalDevice::BufferedInputLogicalDevice( SerialDevice * dev, unsigned short queueType  ) : ( dev )
{
	Debug.printf( "[%x]::BufferedInputLogicalDevice::BufferedInputLogicalDevice( serialDevice=%x, type= %d )\n", this, dev, queueType );
	inputCharQueue = 0;
	setAttributes( (int) queueType );
}

void
BufferedInputLogicalDevice::setAttributes( int type )
{

	Debug.printf( "[%x]::BufferedInputLogicalDevice::setAttributes( type= %d )\n", this, type );
	CharacterQueue * oldQueue = inputCharQueue;
        switch ( type ){
        case 0:
                inputCharQueue = new LineBufferedEchoingCharacterQueue( device, "LBE" );
                break;
        case 1:
                inputCharQueue = new RawCharacterQueue( "Raw" );
                break;
        default:
                Assert( NOTREACHED );
        }
	Debug.printf( "[%x]::BufferedInputLogicalDevice::setAttributes( type= %d ) ready to delete the oldQueue\n", this, type );
	// WARNING: if ( oldQueue != 0 ) delete oldQueue;
	Debug.printf( "[%x]::BufferedInputLogicalDevice::setAttributes( type= %d ) done\n", this, type );
}	

unsigned short
BufferedInputLogicalDevice::charPending(){

        return !(inputCharQueue->empty());

}

char
BufferedInputLogicalDevice::getChar(){


	Debug.printf( "[%x]::BufferedInputLogicalDevice::getChar()\n",this); 
        char c;
	inputCharQueue->dequeue( &c );
	Debug.printf( "[%x]::BufferedInputLogicalDevice::getChar() returns %c \n", this, c); 
        return c;

}

int
BufferedInputLogicalDevice::read( char * bufr, int maxlen ){

	char c;
        int count = 0;

        Debug.printf( "[%A]::BufferedInputLogicalDevice::Read( bufr:%A maxlen:%d )\n", this, bufr, maxlen );
        for( ;; ) {

                /* Get a character by sending a message to self 
		 */
                c = getChar();

                /* if no more characters in queue, return a short read. 
		 */
                if( c == (char) -1 ) {
                        *bufr = '\0';
                        return( count );
                }

                /* Otherwise stuck the character in the buffer
		 * until full.
		 */
                Debug.printf( "[%A]BufferedInputLogicalDevice::Read: got %c\n", this, c );
                *(bufr++) = c;
                if( ++count == maxlen ) return( count );
                
        }
}


void
BufferedInputLogicalDevice::inputInterruptHandle(){

        /*
         * If there is a character waiting, handle it.
         */
        if( device->charPending() ) {
                char c = device->getChar();
                inputCharQueue->enqueue( c );
//		Debug << this << "::inputInterruptHandle(): read '" << c <<
//			"'\n" << eor;
        } else {
		Console() << this << "::inputInterruptHandle: NO WORK?\n"
			<< eor;
        }
}

