#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Bjorn A. Helgaas (helgaas@cs.uiuc.edu)
 * Contributing Author: Lee Lup Yuen (lup@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * CharacterInputStream.h: definition of an output stream of characters.
 */

#include "Assert.h"
#include "CharacterInputStream.h"
#include "Process.h"
#include "ThisProcess.h"
#include "Raid.h"


#ifdef __GNUC__
#pragma implementation
#endif


CharacterInputStream::CharacterInputStream( OutputStream * echoStream )
{
	Assert( echoStream != 0 );
	_echoStream = echoStream;
	_queue = new LineBufferedEchoingCharacterQueue( _echoStream );
	Assert( _queue != 0 );
}


int
CharacterInputStream::read( char * buffer, int limit )
{
	extern int consoleInputInitialized;
	while( consoleInputInitialized == 0 )
		thisProcess()->relinquishProcessor();

	Debug << "CharacterInputStream::read( buffer: " << asAddress( buffer )
				<< ", limit: " << limit << " )\n" << eor;
	
	int count = _queue->dequeue( buffer, limit );
	
	Debug << "CharacterInputStream::read: got '" << buffer
				<< "', length: " << count << "\n" << eor;
	
	return( count );
}

void
CharacterInputStream::switchCharacterQueue( int type )
{
	CharacterQueue * newQueue = 0;
	
	switch( type ) {
		case CS_LINEBUFFERED:
			newQueue = new LineBufferedEchoingCharacterQueue( _echoStream );
			break;
		case CS_RAW:
			newQueue = new RawCharacterQueue();
			break;
		case CS_CHARACTERBLOCK:
			newQueue = new BlockCharacterQueue(16384);
			break;
		default:
			Console().cpuPrintf( "Invalid CharacterQueue type\n" );
			return;
	}
	
	Assert( newQueue != 0 );
	CharacterQueue * oldQueue = _queue;
	_queue = newQueue;
	delete oldQueue;
}


#include "ClassInstaller.h"
DEFINECLASS("CharacterInputStream",CharacterInputStream,InputStream,0,0);
