#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Bjorn A. Helgaas (helgaas@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
#include "Assert.h"
#include "CharacterQueue.h"
#include "MultiplexedOutputStream.h"



#ifdef __GNUC__
#pragma implementation
#endif

/*
 * Multiplex classes.  Note that this implementation is non-blocking
 * because we use only locks.
 */
MultiplexedOutputStream::MultiplexedOutputStream( OutputStream * sink,
	int bufferSize )
{
	Assert( sink != 0 );
	_sink = sink;
	_queue = new RawCharacterQueue( bufferSize );
	Assert( _queue != 0 );
	_flushBuffer = new char[ bufferSize ];
	Assert( _flushBuffer != 0 );
	_bufferSize = bufferSize;
}

MultiplexedOutputStream::~MultiplexedOutputStream()
{
	Assert( _sink != 0 );
	Assert( _queue != 0 );
	Assert( _flushBuffer != 0 );

	_sink = 0;
	delete _queue;
	delete _flushBuffer;
}

void
MultiplexedOutputStream::basicFlush()
{
	int n = _queue->dequeue( _flushBuffer, _bufferSize );
	_sink->write( _flushBuffer, n );
}

void
MultiplexedOutputStream::acquireExclusiveUse()
{
	Assert( ! _lock.heldByMe() );
	_lock.acquire();
}

void
MultiplexedOutputStream::releaseExclusiveUse()
{
	_lock.release();
}

void
MultiplexedOutputStream::flush()
{
	_sink->acquireExclusiveUse();
	basicFlush();
	_sink->releaseExclusiveUse();
}

int
MultiplexedOutputStream::write( const char * buffer, int count )
{
	int charactersLeft = count;
	while( charactersLeft > 0 ) {
		if( _queue->full() ) {
			flush();
		}
		int n = _queue->enqueue( buffer, count );
		buffer += n;
		charactersLeft -= n;
	}
	return( count );
}


IdentifyingMultiplexedOutputStream::IdentifyingMultiplexedOutputStream(
	OutputStream * sink, Object * owner, int bufferSize )
	: MultiplexedOutputStream( sink, bufferSize )
{
	_owner = owner;
}

void
IdentifyingMultiplexedOutputStream::flush()
{
	_sink->acquireExclusiveUse();
	*(OutputStream *)_sink << "<" << _owner << ">: ";
	basicFlush();
	_sink->releaseExclusiveUse();
}

