#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1989
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Roland G Ouellette
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * TtyBuffer.cc
 */

#include "OutputStream.h"
#include "ConduitMessage.h"
#include "MessageFIFO.h"
#include "DumbConduit.h"
#include "TtyBuffer.h"


#ifdef __GNUC__
#pragma implementation
#endif


status
UserTtyBuffer::read(octet *s, int l) {
  Assert(NOTREACHED);
  return( 0 );
}

status
UserTtyBuffer::write(octet *s, int l) {
  Assert(NOTREACHED);
  return( 0 );
}


TtyBuffer::TtyBuffer(int max_msg, int max_chars) {
  rawInput = new SemaphoreRegulatedMessageFIFO(max_msg);
  myConduit = new DumbConduit(this);
  current_message = new ConduitMessage();
  chars_left_input = max_chars;
  chars_left_current_msg = 0;
}

TtyBuffer::~TtyBuffer( ) {
  delete rawInput;
  delete myConduit;
}

void
TtyBuffer::next_message() {
  Assert( chars_left_current_msg == 0 );

  // dispose of old message...
  int number_of_chars = current_message->dataLength();
  delete current_message;
  Assert( ! lock.heldByMe() );
  lock.acquire();
  chars_left_input -= number_of_chars;
  lock.release();
  myConduit->expandWindow(number_of_chars);

  // new message...
  current_message = rawInput->remove();
  Assert( current_message != 0 );
  if( ((TCPMessage *) current_message)->FIN() ) {
      chars_left_current_msg = 0;
      current_char = 0;
  }
  chars_left_current_msg = current_message->dataLength();
  current_char = current_message->data();
  Assert( current_char != 0 );
}

status
TtyBuffer::read(octet *s, int l) {
  // Better make damn well sure that *s to *(s + l) are in the user's
  // address space...  Do that later.
  Assert( s != 0 );
  int i = 0;
  for ( ;i < l; i++ ) {
    while (chars_left_current_msg == 0)	// skip over mt messages too...
      if( current_char )		// if 0, current message is FIN
	next_message();			// get more chars
      else return(0);
    if( chars_left_current_msg == -1 )
      return(i);
    Assert( chars_left_current_msg > 0 );
    *s++ = *current_char++;
    chars_left_current_msg--;
  }
  return(i);				// number of bytes read
}

status
TtyBuffer::write(octet *s, int l) {
  // Better make damn well sure that *s to *(s + l) are in the users
  // address space...  Do that later.
  Assert( s != 0 );
  octet *out_bound = new octet[l];
  move_bytes(out_bound, s, l);		// copy to kernel space...
  myConduit->insertFromAbove(l, out_bound);
  return( 0 );
}

status
TtyMessageBuffer::read(octet *s, int l) {
  // Like TtyBuffer but only return one message

  // Better make damn well sure that *s to *(s + l) are in the user's
  // address space...  Do that later.
  Assert( s != 0 );
  int i = 0;
  while( chars_left_current_msg == 0 ) {
      next_message();
      if( current_char == 0)
	return(0);
  }
  for ( ;i < l && chars_left_current_msg > 0; i++ ) {
    Assert( chars_left_current_msg > 0 );
    *s++ = *current_char++;
    chars_left_current_msg--;
  }
  return(i);				// number of bytes read
}




#include "ClassInstaller.h"
DEFINECLASS("UserTtyBuffer",UserTtyBuffer,ProxiableObject,0,0);
DEFINECLASS("TtyBuffer",TtyBuffer,UserTtyBuffer,0,0);
DEFINECLASS("TtyMessageBuffer",TtyMessageBuffer,TtyBuffer,0,0);
