/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * Async.h: 
 */

#ifndef Async_h
#define Async_h

#ifdef __GNUC__
#pragma interface
#endif

#include "BlockedInfo.h"
#include "Lock.h"
#include "Object.h"

class AsyncClosure;
typedef void (* AsyncFunc)( AsyncClosure * closure );

class AsyncClosure {
	friend class AsyncSupport;
	AsyncClosure * _next;
	AsyncClosure * _prev;
  static void doNothing(AsyncClosure *p) {Console()<<"DONOTHING called\n"<<eor;}
 protected:
	AsyncFunc _func;
  AsyncClosure();
  public:
	unsigned _urgency;
};

class AsyncBlockedInfo : public BlockedInfo {
  public:
	AsyncClosure * asyncClosure() { return (AsyncClosure *) _buf[0]; }
	void setAsyncClosure( AsyncClosure *c ) { _buf[0] = (unsigned) c; }
};

class AsyncSupport : public Object {

	Lock _lock;
	unsigned _concurrency;
	unsigned _idling;
	unsigned _active;
	unsigned _waiting;
	BlockedInfoList _waitList;
	AsyncClosure * _head;
	AsyncClosure * _tail;

	void acquire() { _lock.acquire(); }
	void release() { _lock.release(); }

	static void daemon( int arg );

      public:

	unsigned active() { return _active; }
	unsigned waiting() { return _waiting; }

	Class * classOf();

	AsyncSupport( unsigned concurrency );
	void schedule( AsyncClosure * c );
};

class ClassStar;

extern ClassStar AsyncSupportClass;

extern AsyncSupport * TheAsyncSupport;

#endif /* Async_h */
