/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef Barrier_h
#define Barrier_h

#ifdef __GNUC__
#pragma interface
#endif

#include "Lock.h"
#include "Process.h"
#include "PROCESSOR.h"
#include "ThisProcess.h"

class Barrier {
	Lock _lock;
	int _count;
	Process * _process;
public:
	Barrier( int count = 0 );
	void setCount( unsigned count );
	void done();
	void wait();
};

inline
Barrier::Barrier( int count )
	: _count( count ), _process( 0 )
{
}

inline
void
Barrier::setCount( unsigned count )
{
	_lock.acquire();
	_count += count;
	_lock.release();
}

inline
void
Barrier::done()
{
	_lock.acquire();
	_count--;
	if ( _count == 0 && _process != 0 ) {
		_process->ready();
	}
	_lock.release();
}

inline
void
Barrier::wait()
{
	IntStorage intState ;
	PROCESSOR::disableInterrupts(intState);
	_lock.acquire();
	if ( _count != 0 ) {
		Assert( _count > 0 );
		_process = thisProcess();
		_lock.release();
		_process->block( (Object *) this );
	} else {
		_lock.release();
	}
	PROCESSOR::restoreInterrupts( intState );
}

#endif /*Barrier_h*/
