/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * BlockedInfo.h: 
 */

#ifndef BlockedInfo_h
#define BlockedInfo_h

/*

   This file actually provides impementation so the #pragma interface
   causes problems! (or something)

#ifdef __GNUC__
#pragma interface
#endif
*/

#include "Process.h"

class BlockedInfoList {
protected:
	Process * _head;
	Process * _tail;
public:
	BlockedInfoList() : _head( 0 ), _tail( 0 ) {}
	int isEmpty() { return _head == 0; }
	Process * head() { return _head; }
	Process * tail() { return _tail; }
	void addHead( Process * p );
	void addTail( Process * p );
	void addAfter( Process * p, Process * prev );
	void addBefore( Process * p, Process * next );
	void remove( Process * p );
	Process * removeHead();
	Process * removeTail();
};

inline 
void 
BlockedInfoList::addHead( Process * p ) 
{
	BlockedInfo * info = p->blockedInfo();
	info->setPrev( 0 );
	if ( _head ) {
		_head->blockedInfo()->setPrev( p );
		info->setNext( _head );
	} else {
		_tail = p;
		info->setNext( 0 );
	}
	_head = p;
}

inline
void 
BlockedInfoList::addTail( Process * p )
{
        BlockedInfo * info = p->blockedInfo();
        info->setNext( 0 ); 
        if ( _tail ) {
                _tail->blockedInfo()->setNext( p );  
                info->setPrev( _tail ); 
        } else { 
                _head = p; 
                info->setPrev( 0 ); 
        }
        _tail = p; 
}

inline
void 
BlockedInfoList::addAfter( Process * p, Process * prev )
{
	if ( prev == 0 ) {
		addHead( p );
		return;
	}
        BlockedInfo * info = p->blockedInfo();
	BlockedInfo * prevInfo = prev->blockedInfo();
	Process * next = prevInfo->next();
	if ( next ) {
		info->setNext( next );
		next->blockedInfo()->setPrev( p );
	} else {
	 	info->setNext( 0 );
		_tail = p;
	}
	info->setPrev( prev );
	prevInfo->setNext( p );
}

inline
void
BlockedInfoList::addBefore( Process * p, Process * next )
{
	if ( next == 0 ) {
		addTail( p );
		return;
	}
        BlockedInfo * info = p->blockedInfo();
        BlockedInfo * nextInfo = next->blockedInfo();
        Process * prev = nextInfo->prev();  
        if ( prev ) {
                info->setPrev( prev );
                prev->blockedInfo()->setNext( p ); 
        } else {
                info->setPrev( 0 );
                _head = p;
        }
        info->setNext( next );
        nextInfo->setNext( p );
}


inline
Process * 
BlockedInfoList::removeHead()
{
	Process * p = _head;
        if ( p == 0 ) {
                return 0;
        }
        Process * next = p->blockedInfo()->next();
	if ( next ) {
        	next->blockedInfo()->setPrev( 0 );
		_head = next;
	} else {
		_tail = _head = 0;
	}
	return p;
}

inline   
Process *
BlockedInfoList::removeTail()
{
        Process * p = _tail;
        if ( p == 0 ) {
                return 0;
        }
        Process * prev = p->blockedInfo()->prev();
        if ( prev ) { 
                prev->blockedInfo()->setNext( 0 );
                _tail = prev;
        } else { 
                _tail = _head = 0;
        }
        return p;
}
 
inline
void
BlockedInfoList::remove( Process * p )
{
	BlockedInfo * info = p->blockedInfo();
	Process * prev = info->prev();
	Process * next = info->next();
	if ( prev ) {
		prev->blockedInfo()->setNext( next );
	} else {
		_head = next;
	}
	if ( next ) {
		next->blockedInfo()->setPrev( prev );
	} else {
		_tail = prev;
	}

}

#endif /* BlockedInfo_h */
