/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Author: David Raila (raila@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * CPUSet.h: sets of CPU.s descriminated by their processor id number.
 */

#ifndef CPUSet_h
#define CPUSet_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Assert.h"
#include "Lock.h"
#include "CPU.h"
#include "Object.h"
#include "OutputStream.h"


extern ClassStar CPUSetClass;
class CPUSet : public Object {
protected:
	friend class CPUSetIterator;

	Lock lock;
	CPU* _entry[ MAX_PROCESSORS ];
	unsigned _count;			// How many we have

public:
	CPUSet();
	~CPUSet(){}

	int contains( CPU * cpu );
	void add( CPU * cpu );
	void remove( CPU * cpu );
	CPU *cpu(CpuId);		// Return CPU or 0 for id
	int isEmpty();

	virtual Class * classOf();
};

inline
CPUSet::CPUSet()
{ 
	for (_count = 0; _count < MAX_PROCESSORS; _count++)
		_entry[_count] = 0; 
	_count = 0;
}

/*
 * An iterator for a CPUSet.
 */

class CPUSetIterator {
protected:
	CPUSet * _mySet;
	CpuId _current;
public:
	CPUSetIterator( CPUSet * cpuSet );
	CPU * next();
};

inline
CPUSetIterator::CPUSetIterator( CPUSet * set )
{
	Assert( set != 0 );
	_mySet = set;
	_current = 0;
}

#endif /* CPUSet_h */
