#ifndef Assert_h
#define Assert_h

#ifdef __GNUC__
#pragma interface
#endif

#include "FormatAssists.h"
#include "Exception.h"

const int NOTREACHED = 0;

#ifdef ASSERT

class AssertionFailure {
 public:
    AssertionFailure(const char *exp , char * file, char *baseFile, int line);
    ~AssertionFailure();    // Does the trap
    AssertionFailure& operator()(const char * format,
                                 const formatArgs & a1 = *(formatArgs *)0,
                                 const formatArgs & a2 = *(formatArgs *)0,
                                 const formatArgs & a3 = *(formatArgs *)0,
                                 const formatArgs & a4 = *(formatArgs *)0 );
};

#define Assert(exp) if(exp) ; else (void)AssertionFailure( #exp, __FILE__, __BASE_FILE__, __LINE__ )

class AssertionException : public Exception {
 public:
    AssertionException(const char *exp, const char *file,
                       const char *baseFile, int line);
};

#define AssertOrThrow(exp) if (!(exp)) throw new AssertionException( #exp, __FILE__, __BASE_FILE__, __LINE__ )

#else /* !ASSERT */
class
NullAssertion
{
 public:
    ~NullAssertion() {}
    NullAssertion (char *exp , char * file, char *baseFile, int line){}
    void operator()(char * format,
                    const formatArgs & a1 = *(formatArgs *)0,
                    const formatArgs & a2 = *(formatArgs *)0,
                    const formatArgs & a3 = *(formatArgs *)0,
                    const formatArgs & a4 = *(formatArgs *)0 ){}
};

#define Assert(exp) if(exp) ; else (void)NullAssertion( #exp, __FILE__, __BASE_FILE__, __LINE__ )
#define AssertOrThrow(exp) (void)NullAssertion()

#endif /* ASSERT */

#endif /* Assert_h */
