/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Nayeem Islam (nislam@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * AsyncMessageContainer.h - supports asynchronous recieves 
 */

#ifndef AsyncMessageContainer_h
#define AsyncMessageContainer_h

#ifdef __GNUC__
#pragma interface
#endif

#include "Lock.h"
#include "MessageContainer.h"
#include "iPSCMessage.h"

const long LOWBIT  = ((long)0x1);
const long BIGTYPE = ((long)30);
const long MINTYPESEL = ((long)-100);
const long MAXTYPESEL = ((long)-2);
const long ANYTYPE = ((long)-1);

extern ZeroBlock(char* , unsigned int);
extern void CopyBlock(const char *, char*, unsigned int);

extern ClassStar AsyncMessageContainerClass;

class Message;
class iPSCMessage;
class ROQMessage;
class Transfer;
class MessageContainer;
class MutexSemaphore;

/*
 * This currently preserves the 
 * original hyper implementation for
 * the perfromance experiments.
 */

class ROQMessage {
public:
	Lock lock;
	int satisfied;
	int type;
	Semaphore* sem;
	Message *mesg;
	Message *userm;
	ROQMessage();
};

class AsyncMessageContainer : public MessageContainer {
protected:
	virtual int addToROQ(Message *message)=0; 
	void compact();
	MutexSemaphore* InQSem;
	virtual void nextIndex(int*);
	iPSCMessage LastMessage; // last message received on port
/* Just update last message os that it can be queried */
	virtual int getACopyFromInQ( long typemask);
	
	virtual long checkMask(long type, long typesel);
	virtual long typeMatch(long gtype, long typesel);
	virtual int getMessage(Message* message);
public:
 	Class *classOf ();
    	proxiable AsyncMessageContainer (char*);

    	proxiable virtual void put( Message *message)=0; 

	proxiable virtual int ireceive (Message *message);
	proxiable virtual void receive (Message *message);
	proxiable virtual int probe (int type);
	proxiable virtual void cprobe (int type);
	proxiable virtual int infoNode ();
	proxiable virtual int infoType ();
	proxiable virtual int infoCount ();
	
	proxiable virtual int wait ( int messageId)=0; // block
	proxiable virtual int done ( int messageId)=0; // ret if not there
        proxiable virtual void cancel ( int messageId)=0;


};

#endif AsyncMessageContainer_h
