/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Contributing Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * Class.h -
 */
#ifndef Class_h
#define	Class_h

#ifdef __GNUC__
#pragma interface
#endif

#include "ProxiableObject.h"
#include "ClassStar.h"
#include "ProxyTableCopyStar.h"
#include "ClassConstructorStar.h"
class NameServer;
class Class;

/*
 * A pointer to a function taking an object of a Class.
 * Used by the applyTo... methods to do something to each object.
 */
typedef void (*ClassApplicationFunction)(ProxiableObject *);

extern ClassStar ClassClass;
class Class : public ProxiableObject {
/*
 *  A Class is the run-time representation of a C++ class.
 */

protected:
	char *		_name;

	/* 
	 * Cannot reference both parent and child; that would cause a loop
	 */
	Class *		_parent;
	ClassStar	_sibling;
	ClassStar	_child;

	int		_sizeOf;

	int		_initDebugObjects;
	int		_initDebugMask;
	int		_initDebugMessages;

	ClassConstructorStar _constructor;

	ProxyTableCopyStar _proxyTable;

	/* Method addChild: */
	void		addChild( Class * );

	/* Method setSibling: */
	void		setSibling( Class * );

	~Class();
public:
	Class * classOf();
	proxiable Class( Class * parent, int size, const char * name,
	       ClassConstructor * ctor = 0 );

	proxiable virtual void	cleanup();

	/*
	 * there are two 'name'.  The first one is not useful to applications
	 *   because they can't access kernel memory.  The second copies the
	 *   name into the buffer that's passed in.
	 */
	virtual const char *	name();
	proxiable virtual char *	name(int, char *);
	proxiable virtual ClassRef	parent();
	proxiable virtual ClassRef	child();
	proxiable virtual ClassRef	sibling();
	proxiable virtual Class *	real();
	proxiable virtual int		isASubclassOf( Class * );
	proxiable virtual int		isClass( Class * );
	proxiable virtual int		sizeOf();

	proxiable virtual void		bind( NameServer * );
	proxiable virtual void		unbind( NameServer * );

	proxiable virtual int	initDebugObjects();
	proxiable virtual int	initDebugMask();
	proxiable virtual int	initDebugMessages();

	proxiable virtual void	setInitDebugObjects( int );
	proxiable virtual void	setInitDebugMask( int );
	proxiable virtual void	setInitDebugMessages( int );

	proxiable virtual ProxiableObject * constructor( ProxiableObject * );
        proxiable virtual int             concrete();

	proxiable virtual int	debugMembers( int mask, int messages );
	proxiable virtual int	debugKindred( int mask, int messages );
	proxiable virtual int	displayMember( ProxiableObject * );
	proxiable virtual int	displayMembers();
	proxiable virtual int	displayKindred( int leaksOnly = 0 );

	/*
	 * Apply the ClassApplicationFunction to members of areas 
	 * of the hierarchy, return the number of objects applied to.
	 */
	proxiable virtual int	applyToMembers( ClassApplicationFunction );
	proxiable virtual int	applyToKindred( ClassApplicationFunction );

	virtual void		setSizeOf( int size );
	virtual void		setConstructor( ClassConstructor * ctor );

	proxiable virtual void		setProxyTable( ProxyTableCopy * );
	proxiable virtual ProxyTableCopy * proxyTable();
};

#endif	/*Class_h*/
