/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Panagiotis Kougiouris (kougiour@cs.uiuc.edu)
 * Contributing Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ClassInstaller.h -
 *
 *	$Header: /projects/choices/old/cvsroot/choices/Includes/Common/ClassInstaller.h,v 1.1 2004/06/03 01:58:40 jcarlyle Exp $
 */
#ifndef ClassInstaller_h
#define	ClassInstaller_h


#ifdef __GNUC__
#pragma interface
#endif

#include "ClassStar.h"
#include "PersistentClassStar.h"

class ProxiableObject;
typedef int  (*iFPO)( ProxiableObject * );
typedef ProxiableObject * (*POFPO)( ProxiableObject * );


#define MaxNumberOfClasses  350
class ClassInstaller {
public:

    /* No object is constrcuted. Simply the static tables are updated about
     * a new class. The class will be built later---unless if it is called
     * after the initialization
     * ARGUMENTS:
     * 1] A pointer to the pointer pointing to the object representing the class
     * 2] A pointer to the pointer pointing to the object representing the superclass
     * 3] the size of an object of this class
     * 4] the name of the class
     * 5] a pointer to a function that constructs object of this class
     * 6] a pointer to function that takes an object and answers if this class
     *    supports this object
     */
    ClassInstaller( ClassStar *, ClassStar *, int, char *,int flag, POFPO=0, iFPO = 0);

    /* The method that builds all classes. It calls tsort first. */
    static void install();
    static void uninstall();

protected:

    /* Topologically sorts the _info so that superclasses are in front
     * of subclasses */
    static void tsort();

    /* Swap two entries of the table * info */
    static void swap( int, int );
};

/* 
 * Define the ::classOf function, the ClassInstaller, and the ClassStar
 *   variable itself.
 * The definition of the CLASS##Class is deliberately put after the
 *   definition of the CLASS##Installer to require that the programmers
 *   put an extern for the CLASS##Class in the CLASS header file.
 */
#define DEFINECLASS(NAME, CLASS, PARENT, CONST, SUPPORTS) \
	Class * CLASS::classOf() { return( CLASS##Class ); } \
	ClassInstaller Install##CLASS ( & CLASS##Class, & PARENT##Class, \
		sizeof( CLASS ), NAME, 0, CONST, SUPPORTS ); \
	ClassStar CLASS##Class;

/*
 * Define PersistentClass
 */
#define DEFINEPERSISTENTCLASS(NAME, CLASS, PARENT, CONST, SUPPORTS) \
	Class * CLASS::classOf() { return( CLASS##Class ); } \
	ClassInstaller Install##CLASS ( & CLASS##Class, & PARENT##Class, \
		sizeof( CLASS ), NAME, 1, CONST, SUPPORTS ); \
	PersistentClassStar CLASS##Class;

/*
 * Just define the installer and the star variable, for use by Proxify++
 */
#define COPYCLASS(NAME, CLASS, PARENT) \
	ClassInstaller Install##CLASS ( & CLASS##Class, & PARENT##Class, \
		sizeof( CLASS ), NAME, 0, 0, 0 ); \
	ClassStar CLASS##Class;

#define COPYPERSISTENTCLASS(NAME, CLASS, PARENT) \
	ClassInstaller Install##CLASS ( & CLASS##Class, & PARENT##Class, \
		sizeof( CLASS ), NAME, 1, 0, 0 ); \
	PersistentClassStar CLASS##Class;

#endif	/*ClassInstaller_h*/
