/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * FileSystemHints.h - 
 */

#ifndef	FileSystemHints_h
#define	FileSystemHints_h


#ifdef __GNUC__
#pragma interface
#endif

const int O_LCACHE = 0x08000000;
const int O_NLCACHE= 0x04000000;
const int O_SINGLE = 0x00800000;
const int O_MULTI  = 0x00400000;
const int O_LARGE  = 0x00200000;
const int O_SMALL  = 0x00100000;
const int O_RANDOM = 0x00080000;
const int O_SEQ    = 0x00040000;
const int O_TEMP   = 0x00020000;
const int O_PERM   = 0x00010000;

const int O_HINTMASK    = 0x0fff0000;
const int O_SETHINTMASK = 0x0AAA0000;
const int O_CLRHINTMASK = 0x05550000;
const int O_HINTSHIFT	= 16;

const int H_LCACHE = 0x00000010;
const int H_SINGLE = 0x00000008;
const int H_LARGE  = 0x00000004;
const int H_RANDOM = 0x00000002;
const int H_TEMP   = 0x00000001;

inline 
int h_single(const int hint)
{
	return (hint & H_SINGLE);
}

inline
int h_mult(const int hint)
{
	return !h_single(hint);
}

inline 
int h_large(const int hint)
{
	return (hint & H_LARGE);
}

inline
int h_small(const int hint)
{
	return !h_large(hint);
}

inline 
int h_random(const int hint)
{
	return (hint & H_RANDOM);
}

inline
int h_seq(const int hint)
{
	return !h_random(hint);
}

inline 
int h_temp(const int hint)
{
	return (hint & H_TEMP);
}

inline
int h_ntemp(const int hint)
{
	return !h_temp(hint);
}

inline
int h_lcache(const int hint)
{
	return (hint & H_LCACHE);
}

inline
int h_nlcache(const int hint)
{
	return !h_lcache(hint);
}

#endif /*FileSystemHints_h*/
