/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Contributing Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Contributing Author: Bjorn A. Helgaas (helgaas@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * FormatAssists.h: used in OutputStream
 */

#ifndef FormatAssists_h
#define FormatAssists_h


#ifdef __GNUC__
#pragma interface
#endif

class OutputStream;
class ProxiableObjectRef;
class ProxiableObjectStar;

/*
 * An assist class for formatted printing to an OutputStream.
 */

class formatArgs {
 public:
    union {
        void const * p;
        long l;
        double d;
    } _formatArgs;


    formatArgs();
    formatArgs( const ProxiableObjectRef & );
    formatArgs( const ProxiableObjectStar & );
    formatArgs( void const * );
    formatArgs( int );
    formatArgs( unsigned int );
    formatArgs( long );
    formatArgs( unsigned long );
    formatArgs( double );
};

#include "ProxiableObjectStar.h"
inline
formatArgs::formatArgs()
{
    _formatArgs.p = 0;
}

inline
formatArgs::formatArgs( const ProxiableObjectRef & pos )
{
    _formatArgs.p = pos._pointer;
}

inline
formatArgs::formatArgs( const  ProxiableObjectStar & pos )
{
    _formatArgs.p = (ProxiableObject *)pos;
}       

inline
formatArgs::formatArgs( void const * pp )
{
    _formatArgs.p = pp;
}

inline
formatArgs::formatArgs( int ii )
{
    _formatArgs.l = ii;
}

inline
formatArgs::formatArgs( unsigned int ui )
{
    _formatArgs.l = ui;
}

inline
formatArgs::formatArgs( long ll )
{
    _formatArgs.l = ll;
}

inline
formatArgs::formatArgs( unsigned long ul )
{
    _formatArgs.l = ul;
}

inline
formatArgs::formatArgs( double dd )
{
    _formatArgs.d = dd;
}


/*
 * The following classes (asHex, asAddress, etc) are defined for
 * convenience in using OutputStreams.  They allow use to say,
 * for instance:
 *
 *  stream << "The number is " << asHex( n ) << eor;
 *  stream << "In binary, its " << asBits( n ) << eor;
 *
 * The classes don't do anything themselves; they merely cause the
 * right version of
 *
 *  OutputStream & operator <<
 *
 * to be selected.  (See OutputStream.cc for the definitions of these
 * functions.)
 */

class Flush {
 public:
    Flush(){}
};

#define eor Flush()

class asAddress {
    friend OutputStream & operator << ( OutputStream &, asAddress & );
 protected:
    const void * const addr;
 public:
    asAddress( const void * const );
    asAddress( void * );
    operator unsigned long() {return (unsigned long)addr;}
};

inline asAddress::asAddress( const void * const a ) : addr( a ) {}
inline asAddress::asAddress( void * a ) : addr( a ) {}


class asHex {
    friend OutputStream & operator << ( OutputStream &, asHex & );
 protected:
    const unsigned int val;
 public:
    asHex( void * const);
    asHex( const unsigned int );
    operator unsigned() {return val;}
};

inline asHex::asHex( const unsigned int i ) : val(i) {}
inline asHex::asHex( void * const i ) : val((unsigned)i) {}


class asOctal {
    friend OutputStream & operator << ( OutputStream &, asOctal & );
 protected:
    const unsigned int val;
 public:
    asOctal( const unsigned int );
    operator unsigned() {return val;}
};

inline asOctal::asOctal( const unsigned int i ) : val(i) {}


class asBits {
    friend OutputStream & operator << ( OutputStream &, asBits & );
 protected:
    const unsigned int val;
 public:
    asBits( const unsigned int );
    operator unsigned() {return val;}
};

inline asBits::asBits( const unsigned int i ) : val(i) {}

#endif /* FormatAssists_h */
