
/* 
 *
 * This file defines a set of struictures that can 
 * used for distributed communication. 
 * The base message class is a
 *
 */

#ifndef Message_h
#define Message_h


#ifdef __GNUC__
#pragma interface
#endif

class ContainerRepresentative;
class MessageContainer;
class EthernetTransport;
class EthernetTransfer;

/* 
 * This sends an asynchronous message through the system
 */

class Message {
protected:
  unsigned int _time;
  ContainerRepresentative* destinationMessagePort;      /* send to this  mport         */
  MessageContainer* receiverMessagePort;      /* get a messae pfrom this port */
  unsigned int maxDataSize;     /* length of data segment            */
  unsigned int dataSize;     /* length of data segment            */
  char* dataPtr;             /* data segment of message          */
  int deleteData;		/* whether or not data should be deleted */
  int ctype;
public:
  Message(char* initData,int initSize,int delData=1); /* default is to delete related data */
  Message(int delData=1); /* default is to delete related data */
  ~Message();

    void setTime(unsigned int);
    unsigned int getTime();

  void print();
  int getcType();
  void setcType(int );
  void setReceivePort(MessageContainer *rport);/* set receiver port */
  void setDestination(ContainerRepresentative *sport);/* set destination field        */
  void setSize(unsigned int size);/* set size field of message  */
  void setData(void *data); /* set pointer to data area   */
  void setMaxSize(void *data, unsigned int); /* set pointer to data area   */
  void setUserData(void *indata,int size); /* set pointer to data area   */

 ContainerRepresentative* destination();           /* destination of messageport */
  MessageContainer* receiverPort();           /* destination of messageport */
  long maxSize();                  /* size of message              */
  long size();                  /* size of message              */
  void* data();              /* pointer to data area of msg  */
  void send();  /* send the message */
  void receive();  /* send the message */
};

/*
 *  this message forces a reply to the system
 */

/*
 * There are some hacks here put in for 
 * some specific reasons.
 * In particular, replyToMessagePort, is used 
 * on the receiving side.
 * where as replyMessagePort is used on the 
 * sending  side to refer to the 
 * the reply port. The use of exact types 
 * is helps debugging the code and
 * the eventual separation of messageports form
 * receiver message ports. 
 */

class RPCMessage : public Message {
protected:
	MessageContainer* server;
        ContainerRepresentative* replyToMessagePort; // on remote machine
	MessageContainer* replyMessagePort;// on the local machine
public:
  RPCMessage(char* initData,int initSize,int delData=1); /* default is to delete related data */
	RPCMessage();
	void rpc(); // client calls this function 
	void reply(); // server calls this function 

	void rpcReceive(); // server calls this function 
	void setReplyPort(MessageContainer*); // to set reply port 
	MessageContainer* getReplyPort(); // to set reply port 
	ContainerRepresentative* getReplyToPort();
	void setReplyToPort(ContainerRepresentative*); // to set reply port 

	MessageContainer* getServer(); // server calls this function 
	void setServer(MessageContainer*); // server calls this function 
	void print(); // to set reply port 
};

#endif Message_h

