/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ProxiableObjectStar.h - User level file class
 */

#ifndef	ProxiableObjectStar_h
#define	ProxiableObjectStar_h


#ifdef __GNUC__
#pragma interface
#endif

class ProxiableObjectRef;
class ProxiableObject;

class ProxiableObjectStar {
public:
    ProxiableObjectStar( ProxiableObject *const  = 0 );
    ProxiableObjectStar( const ProxiableObjectRef & );
    ProxiableObjectStar( const ProxiableObjectStar & );

    ProxiableObjectStar& operator=( ProxiableObject *const);
    ProxiableObjectStar& operator=( const ProxiableObjectRef & );
    ProxiableObjectStar& operator=( const ProxiableObjectStar & );

    operator ProxiableObject *() const;
    ProxiableObject * operator->() const;

    // friend int operator==( const ProxiableObjectStar & const, void * );
    // friend int operator!=( const ProxiableObjectStar & const, void * );

    ~ProxiableObjectStar();

    ProxiableObject *	_pointer;
};

class ProxiableObjectRef {
public:
    ProxiableObjectRef( ProxiableObject *const  = 0 );
    ProxiableObjectRef( const ProxiableObjectRef & );
    ProxiableObjectRef( const ProxiableObjectStar & );

    ProxiableObjectRef& operator=( ProxiableObject *const  );
    ProxiableObjectRef& operator=( const ProxiableObjectRef & );
    ProxiableObjectRef& operator=( const ProxiableObjectStar & );

    operator ProxiableObject *() const;
    ProxiableObject * operator->() const;

    // friend int operator==( const ProxiableObjectRef & const, void * );
    // friend int operator!=( const ProxiableObjectRef & const, void * );

    ProxiableObject *	_pointer;
};

#ifndef PROXIFY
inline 
ProxiableObjectStar::ProxiableObjectStar( const ProxiableObjectRef & pos )
{
	_pointer = pos._pointer;
}

inline
ProxiableObject *
ProxiableObjectStar::operator->() const
{
	return _pointer;
}

inline 
ProxiableObjectStar::operator ProxiableObject *() const
{
	return _pointer;
}

#ifdef NEVER
inline
int
operator==( const ProxiableObjectStar & const s1, void * s2 )
{
	return( s1._pointer == s2 );
}

inline
int
operator!=( const ProxiableObjectStar & const s1, void * s2 )
{
	return( s1._pointer != s2 );
}
#endif

inline
ProxiableObjectRef::ProxiableObjectRef( const ProxiableObjectRef & pos )
{
	_pointer = pos._pointer;
}

inline
ProxiableObject *
ProxiableObjectRef::operator->() const
{
	return _pointer;
}

inline
ProxiableObjectRef::operator ProxiableObject *() const
{
	return _pointer;
}

#ifdef NEVER
inline
int
operator==( const ProxiableObjectRef & const s1, void * s2 )
{
	return( s1._pointer == s2 );
}

inline
int
operator!=( const ProxiableObjectRef & const s1, void * s2 )
{
	return( s1._pointer != s2 );
}
#endif

#endif

#endif /*ProxiableObjectStar_h*/
