/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Contributing Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * Raid.h - support for run-time controllable debugging.
 */

#ifndef Raid_h
#define	Raid_h


#ifdef __GNUC__
#pragma interface
#endif


/*
 * One can turn on six different types of debugging for either existing
 * objects or future objects belonging to specific classes or kinds:
 *
 * Most of the debugging messages either print out method invocations
 * and their arguments or they print out return values.
 *
 *	Bit	Mask	    Type
 *
 * 	0)	  1	Error Conditions (Not used much yet)
 *	1)	  2	Constructor/Destructor Methods
 *	2)	  4	reference/unreference/noRemainingReferences Methods
 *	3)	  8	Public Methods
 *	4)	 16	Protected Methods
 *	5)	 32	Detailed Information within any type of method
 */
const int RaidError	 = 1;
const int RaidConstructor = 2;
const int RaidReference	 = 4;
const int RaidMethod	 = 8;
const int RaidProtected	 = 16;
const int RaidDetail	 = 32;

	/*
	 * See Raid.cc for these
	 */
extern int parseMask( char* );
extern int odb(int, char *[]);
extern int cdb(int, char *[]);
extern int hierarchy(int, char *[]);
extern int instance(int, char *[]);
extern int instances(int, char *[]);


#ifdef RAID
#ifdef KERNEL
#include "RaidOutputStream.h"
extern RaidOutputStream StandardRaidStream;
inline
OutputStream & RaidStream()
{
	return StandardRaidStream;
}

#else

#include "OutputStream.h"
inline 
OutputStream & 
RaidStream() 
{ 
	return Console();
}
#endif

/*
 * If RAID is defined, then the Raid and Raids macros are defined,
 * and debugging can be controlled at run-time.
 * Raid( mask )() supports printf-style debugging.
 * Raids( mask )() supports <<-style debugging.
 */

extern unsigned char DebugGlobalVariable;
extern unsigned char _debugMask;

#define Raid(mask)  if (( DebugGlobalVariable ) || (( _debugMask & mask ) && ( _debugMessages-- > 0 ))) RaidStream().formattedWrite
#define Raids(mask)  if (( DebugGlobalVariable ) || (( _debugMask & mask ) && ( _debugMessages-- > 0 ))) RaidStream()

#else
#include "Debug.h"

/*
 * If RAID is undefined, then the Raid and Raids macros are defined as Debugs,
 * and debugging cannot be controlled at run-time.
 */
#define Raid(mask) Debug.printf
#define Raids(mask) Debug

#endif /*RAID*/

#endif /*Raid_h*/
