#ifndef ReferenceCount_h
#define ReferenceCount_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Assert.h"
#include "Lock.h"


class ReferenceCount {
protected:
	Lock _lock;
	unsigned _count;
public:
	ReferenceCount();
	~ReferenceCount();
	int value();
	int increment();
	int decrement();
};

inline
ReferenceCount::ReferenceCount()
{
	_count = 0;
}

inline
ReferenceCount::~ReferenceCount()
{
	Assert(this != 0);
	Assert( _count == 0 );
}

inline int
ReferenceCount::value()
{
	Assert( this != 0);
	return( _count );
}

inline int
ReferenceCount::increment()
{
	Assert( this != 0);
	_lock.acquire();
	int ret = ++_count;
	_lock.release();
	return ret;
}

inline int
ReferenceCount::decrement()
{
	Assert( this != 0);
	Assert( _count != 0);
	int retval;
	_lock.acquire();
	retval = --_count;
	_lock.release();
	return retval ;
}

#endif	/* ReferenceCount_h */
