/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Template Author: David K. Raila (raila@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
#ifndef	Star_h
#define	Star_h

#ifdef __GNUC__
#pragma interface
#endif

#include "ProxiableObjectStar.h"
#include "PersistentObjectStar.h"

template<class T>
class Star : public ProxiableObjectStar {
public:
	Star( T *const t = 0  ) : ProxiableObjectStar(t){};
	Star& const operator=( T *const  );
	Star( ProxiableObject *const  = 0 );
	Star& operator=( ProxiableObject *const  );
	Star( const ProxiableObjectRef & );
	Star& operator=( const ProxiableObjectRef & );
	Star( const ProxiableObjectStar & );
	Star& operator=( const ProxiableObjectStar & );
	operator T *() const {return (T *)_pointer;}
	T * operator->() const {return (T *)_pointer;}
};

template <class T>
class Ref : public ProxiableObjectRef {
public:
	Ref( T *const t = 0 ) : ProxiableObjectRef(t){};
	Ref& const operator=( T * const );
	Ref( ProxiableObject *const  = 0 );
	Ref& const operator=( ProxiableObject *const  );
	Ref( const ProxiableObjectRef & );
	Ref& const operator=( const ProxiableObjectRef & );
	Ref( const ProxiableObjectStar & );
	Ref& const operator=( const ProxiableObjectStar & );
	operator T *() const;
	T * operator->() const;
};

template <class T>
class PersistentStar : public Star<T> {
protected:
	void stash( ProxiableObject *, int ref = 0 );
	void cache( PersistentClass * );
public:
        void flushStar();

	PersistentStar( ProxiableObject *const  ) ;
	PersistentStar( const ProxiableObjectRef & );
	PersistentStar( const ProxiableObjectStar & );

	PersistentStar& operator=( ProxiableObject *const  );
	PersistentStar& operator=( const ProxiableObjectRef & );
	PersistentStar& operator=( const ProxiableObjectStar & );

	PersistentStar( PersistentObject *const = 0 );
	PersistentStar( const PersistentObjectRef & );
	PersistentStar( const PersistentObjectStar & );

	PersistentStar& operator=( PersistentObject *const  );
	PersistentStar& operator=( const PersistentObjectRef & );
	PersistentStar& operator=( const PersistentObjectStar & );


        operator PersistentObject *() ;
	operator PersistentObjectRef() const;
        PersistentObject * operator->();
};

template <class T>
class PersistentRef : public Ref<T> {
protected:
	void stash( ProxiableObject *, int ref = 0 );
	void cache( PersistentClass * );

public:

	PersistentRef( ProxiableObject  *const  );
	PersistentRef( const ProxiableObjectRef & );
	PersistentRef( const ProxiableObjectStar & );

	PersistentRef& operator=( ProxiableObject *const  );
	PersistentRef& operator=( const ProxiableObjectRef & );
	PersistentRef& operator=( const ProxiableObjectStar & );

	PersistentRef( PersistentObject *const  = 0 );
	PersistentRef( const PersistentObjectRef & );
	PersistentRef( const PersistentObjectStar & );

	PersistentRef& operator=( PersistentObject *const  );
	PersistentRef& operator=( const PersistentObjectRef & );
	PersistentRef& operator=( const PersistentObjectStar & );

        operator PersistentObject *();
        operator PersistentObjectStar () const;
        PersistentObject * operator->();
};

#endif PersistentObjectStar_h
