/* 
 * Abstract client state record.
 */

#ifndef CSR_h
#define CSR_h

#ifdef __GNUC__
#pragma interface
#endif

#include "DistEthernet.h"
#include "Lock.h"

class Address;
class TimeoutTimer;
class Message;
class NetworkBuffer ;
class NetworkBufferFreeList ;
class EthernetTransfer ;
class IntBitArray;

/*
 * This is an abstract class for client state records. 
 */

class CSR {
public:
/*
 * Information used in storage of records and 
 * retrieval.
 */

	CSR();

	Lock mutwait;
	void lock();
	void unlock();

	char** getChain();
	char* chain[RunLength];
        NetworkBuffer * nblist[RunLength];
        NetworkBuffer **getNblist();

	TimeoutTimer *timer;
	NetworkBufferFreeList *netFree;
	int numPackets;

        void putMessage(Message*);
        Message* getMessage();
	Message* message; // message cache

	EthernetTransfer* transfer;

	int first_packet;
	int trans_id; // monotonically increasing
	int retrans_count;
	int transmission_mask;
	int time_limit;

        IntBitArray *tm;
	int MaxNumTimeouts;
	int numTimeOuts;

/*
 * Temporary messge that will be assembled and 
 * then delivered
 */

/*
 * for large messages, that exceed the underlying
 * network packet size.
 */
        int currSize;
/* 
 * Keeping data for retransmissions 
 */

	int num_pack_recv;

	void setState(int);
        virtual void endTrans();

/*
 * Temporary message for delivery.
 */

        virtual void incr_packets();
        void setFrag(int);
	virtual void probe(int);
};

inline void CSR::lock() { mutwait.acquire(); }
inline void CSR::unlock() { mutwait.release(); }

#endif
