/* 
 * There is only one per process.
 * This is used for remote communication.
 */

/*
 *States of the client state record.
 */


/*
 * Client may be in one of any five states.
 *
 * IDLE -- idling or processing, 
 * AWAIT_RESPONSE -- waiting for a response, 
 * RECV_RESPONSE -- in the process of receiving a response,
 * GETREPLY -- get a large number of replies
 * ASYNC - sending an async response.
 */
#ifndef ClientCSR_h
#define ClientCSR_h

#ifdef __GNUC__
#pragma interface
#endif


#include "DistEthernet.h"
#include "Lock.h"

class Address;
class TimeoutTimer;
class Message;
class NetworkBuffer;
class IntBitArray;
class NetworkBufferFreeList;
class MessageContainer;
class Container;

#include "CSR.h"

enum CCSRState {IDLE,AWAIT_RESPONSE,RECV_RESPONSE,GETREPLY,ASYNC};

/*
 * These records are indexed by destination or 
 * receiver port and machine address.
 */

class Reliability;

class ClientCSR : public CSR {
public:
	ClientCSR(Container*,Address*);
	CCSRState state;

	Address *etherAddr; // part of client id
	Container *client_id; // part of client id
	Container* dest; // destnation, message port

	int response;
	int done;
	Address *csr_host; // address of recipient 
	Reliability *reliability;

	virtual void flush(int);
	virtual void probe(int);

	virtual void multiPacketResponse();
	virtual void responseDone();
	virtual void await_response(Address *dest, int numPackets,Reliability*);
	virtual void recv_response();
	virtual void sendProbe();
	virtual void send();
	virtual void endTrans();

};


#endif ClientCSR_h
