/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Nayeem Islam (nislam@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ContainerGroup.h - grouping of MessageContainers 
 */

#ifndef ContainerGroup_h
#define ContainerGroup_h

#ifdef __GNUC__
#pragma interface
#endif

#include "Byte.h"
#include "String.h"
#include "Assert.h"
#include "Container.h"
#include "Address.h"

extern ClassStar ContainerGroupClass;

class Message;
class Transfer;
class EthernetTransfer;
class MessageContainer;
class Transport;
class IteratingList;
class Link;
class ContainerManager;
class SendGroupRepresentative;

/*
 * this class is the representative for the real 
 * class on the
 * other end of the connection 
 * It is primarily used for SENDING messages
 * Ports are kernel entities so they are reachable from
 * everywhere
*/

class ContainerGroup : public Container { 
protected:
	Address* Id;
	char *name;
	SendGroupRepresentative* sgr;
	ContainerManager* MP;
	int numMembers;

    proxiable virtual int wait (Message*){}


// notifications to be sent across the net
   void notifyRemove(Link*);
   void notifyAdd(Link*);

// number of local ports associated with this message port
    enum {listSize = 16}; // limit the number of local MessageContainer
    IteratingList *localList;

public:
    ContainerGroup (char*, Address*);
    Class *classOf ();
	Address *getAddress();
/*
 * Add a message port to the group
 */
   virtual ContainerRepresentative *cachedCR(); // cache
    proxiable virtual void add(MessageContainer*);
    proxiable virtual void add(char*);

/*
 * Remove a message port to the group
 */
    proxiable virtual void remove(MessageContainer*);
    proxiable virtual void remove(char*);

/*
 * Is the given MessageContainer a member of the group 
 */
    int isMember(MessageContainer*);
    int isMember(char*);
/*
 *
 */
	char *getName();
   void listMembers();

    void reset();
    MessageContainer* next();

     void put(Message*);

};


#endif ContainerGroup_h
