/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Nayeem Islam (nislam@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ContainerRepresentative.h - `capability' or 
 * send rights for a MessageContainer. 
 *      $Header: /mnt/tapestry/Choices/nislam.9.29.1990/Includes/MessagePorts
 *      /RCS/MessagePort.h,v 1.2 90/08/18 13:02:47 nislam Exp $
 */


#ifndef ContainerRepresentative_h
#define ContainerRepresentative_h

#ifdef __GNUC__
#pragma interface
#endif

#include "Assert.h"
#include "ProxiableObject.h"
#include "Address.h"

extern ClassStar ContainerRepresentativeClass;

class Message;
class RPCMessage;
class Transfer;
class EthernetTransfer;
class Domain;
class MessageContainer;
class Transport;
class Container;

/* 
 * ABSTRACT CLASS: do not instantiate.
 * ----------------------------------
 * This class specifies the inteface for
 * intelligent capabilities, and 
 * is the representative for object(s). 
 * Subclasses employ various means to 
 * locate recipients of messages.
 * The name server administers these.
 * It has the sole responsibility of 
 * creating these, and reference counts them
 * in conjunction with special servers. 
 */

class ContainerRepresentative : public ProxiableObject
{
protected:

// All the transfer mechanisms are friends of
// the message port. This is a performance hack.

    friend class Transport;
    friend class BufferedTransport;

    friend class NetworkTransport;
    friend class iPSCTransport;
    friend class EthernetTransport;
    friend class ContainerManager;
    friend class SendingTransport;
    friend class ReceivingTransport;
    char *name;

    Address *location; // location of port
    Transport* transport;
    Domain *domain;
public:
    proxiable ContainerRepresentative (char*);
    proxiable ContainerRepresentative (Address*, Transport * trans=0);
    /*  Only Message is supposed to call this.  */

// ABSTRACT
    proxiable virtual void send (Message *);
// ABSTRACT
    proxiable virtual int rpcSend (RPCMessage *, MessageContainer *mp)=0;
// default implementations

    Address *getAddress ();
    void setAddress (Address *);

    void setName(char*);
    char* getName();
    virtual Container* localContainer()=0;
    virtual Container* remoteContainer()=0;
    Class *classOf ();
};

#endif ContainerRepresentative_h
