/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted und
er license agreement.
 *
 * Author: Nayeem Islam (nislam@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 * No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef Header_h
#define Header_h

#ifdef __GNUC__
#pragma interface
#endif

class Message;

#define NAMESERVERPORT 1

#define ASYNC_REQUEST 0x0001
#define RPC_REQUEST 0x0002
#define RPC_RESPONSE 0X0004
#define PROB 0x0008 // do we need request and reply probes

static const int ASYNC_MESSAGE = 0 ; // send part is tagged as an ASYNC_MESSAGE
static const int RPC_MESSAGE = 1 ; // send part is tagged as an RPC_MESSAGE,
static const int REPLY_MESSAGE = 2  ; // multi packet or response (same thing);
static const int AR_MESSAGE = 3  ; // multi packet;
static const int BIG_MESSAGE = 4  ; // multi packet;
static const int PROBE = 5  ; // a probe or ack message
static const int MULTI_MESSAGE = 6  ; // a probe or ack message

class TransportHeader {
        friend class    CSR;
        friend class    ClientCSR;
        friend class    RemoteCSR;
        friend class    ReceivingTransport;
        friend class    SendingTransport;
        friend class    NetworkTransport;
        friend class    EthernetTransport;
        friend class    iPSCTransport;
        friend class    DispatchMessage;
        friend class    DispatchPacket;
        friend class    Transport;

        ProxiableObject *destMessagePort;
        ProxiableObject *replyMessagePort;
        unsigned int    pid; // process id
        unsigned int    ctype; // class type
        unsigned int    type; // hypercube message type
        unsigned int    size;

        unsigned int    messageId; // this is the transaction id
        unsigned int    fragment;
        unsigned int    fragmentSize;
        unsigned int    probe;
};

class ReliabilityHeader {
        friend class    ReceivingTransport;
        friend class    Reliability;
        friend class    AtLeastOnce;
        friend class    ExactlyOnce;
        friend class    DispatchMessage;
        friend class    ClientCSR;
        friend class    RemoteCSR;

	int	ack;
	int	sequenceNum;
	int	delmask; // delivery mask
};

#endif Header_h
