/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Nayeem Islam (nislam@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * MessageContainer.h - user access to the message system
 *
 *      $Header: /mnt/tapestry/Choices/nislam.9.29.1990/Includes/MessageContainer
 *      /RCS/MessageContainer.h,v 1.2 90/08/18 13:02:47 nislam Exp $
 */

/* 
 * This is the inteface for the MessageContainer.
 * This is the main communication entity in the 
 * Choices operating sytem.  It is named and it
 * buffers messages.
 * ContainerRepresentatives server as representatives 
 * for MessageContainers.
 */	

#ifndef MessageContainer_h
#define MessageContainer_h

#ifdef __GNUC__
#pragma interface
#endif

#include "Lock.h"
#include "Assert.h"
#include "ProxiableObject.h"
#include "SemaphoreStar.h"
#include "Container.h"
#include "Address.h"

class Message;
class MessageContainer;
class Transport;
class RPCMessage;
class Semaphore;
class ContainerRepresentative;


// This is used primarily for receiving 

extern ClassStar MessageContainerClass;

class MessageContainer : public Container
{
   friend class BufferedTransport;
   friend class DispatchMessage;
protected:
    enum {messageBufferSize = 128};

    RPCMessage*         _rpcMsg;
    ContainerRepresentative * cr; // cached cr for localcommunication
    Lock mutwait; 
    SemaphoreStar rpcSem; 

    int qlen;
    int tail, head;

    Message *messageBuffer [messageBufferSize];
    Semaphore* messageAvailable; 
/* 
 *  Internal assist functions
 */
    void nextPutIndex (int *index);
    void nextGetIndex (int *index);
   char *name;
 // 	void updateBuffer(int i);
public:
    int qlength();
    proxiable MessageContainer (char *name);

   virtual ContainerRepresentative *cachedCR(); // cached cr for local
    proxiable virtual void get (Message *msg);
    proxiable virtual ContainerRepresentative* rpcGet (RPCMessage *msg);
    proxiable virtual int isEmpty ();
    proxiable virtual void put (Message *);
    proxiable virtual int wait (Message*);
    proxiable virtual void dump (); // contents of queue

    Class *classOf ();
};

inline int
MessageContainer::qlength() {return qlen;}

#endif MessageContainer_h
