/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted und
er license agreement.
 *
 * Author: Nayeem Islam (nislam@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 * No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef NetworkTransport_h
#define NetworkTransport_h

#ifdef __GNUC__
#pragma interface
#endif

/* 
 * The transfor class knows about the 
 * various methods for flow control
 * and reliability
 */

#include "Transport.h"
#include "ProxiableObject.h"

extern ClassStar NetworkTransportClass;

class Message;
class Semaphore;
class TransportHeader;
class Reliability;
class MessagePort;
class Address;
class NetworkBuffer;

class TempMessage {
public:
	TempMessage ();
	Address *etherAddr;
	int messageId;
	int currSize;
	Message *message;
	
};

class NetworkTransport : public Transport {
protected:

	int MaxPacketLength;
	Reliability  *reliability;
	int messageIdentifier;
	int bigArrayIndex;
	int getMessageId();
	Semaphore* messageMutex;
	Semaphore* arrayMutex;
 	enum {tempBufferSize = 128};

	TempMessage* 	MessageArray[tempBufferSize];
	TempMessage*	findTempMessage(Address*,int);
	void	putTempMessage(TempMessage*);
	void	removeTempMessage(TempMessage*);
	void 	assemble(char* packet, int len, Address*,Address *,NetworkBuffer*);
	Message* 	makeMessage(  char*, MessagePort*, TransportHeader* ,Address*, int ,NetworkBuffer*);
	virtual void 	fragment( Message*)=0;
	void initTransportHeader(TransportHeader*, Message *message);
	Message* getMessage(TransportHeader*,Address*);

	void dispatch(Message* ,Address *, ProxiableObject* );
	void dispatch(char *,TransportHeader *,Address *,Address *,NetworkBuffer*);
public:
	NetworkTransport();
	Class *classOf ();
	virtual int send(Message *message)=0;
	virtual void receive(char *packet, Address *etherAddr,Address* dest,NetworkBuffer *);
	virtual void inspect(TransportHeader *packet);
};

#endif NetworkTransport_h
