/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted und
er license agreement.
 *
 * Author: Nayeem Islam (nislam@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 * No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef ReceivingTransport_h
#define ReceivingTransport_h

/* 
 * The transfor class knows about the 
 * various methods for flow control
 * and reliability
 */

#ifdef __GNUC__
#pragma interface
#endif

#include "ReceivingTransport.h"
#include "ProxiableObject.h"
#include "ContainerManager.h"
#include "DispatchMessage.h"

extern ClassStar ReceivingTransportClass;

class Message;
class Semaphore;
class TransportHeader;
class Reliability;
class MessageContainer;
class Address;
class NetworkBuffer;
class ClientCSR;
class RemoteCSR;
class EthernetTransfer;
class Container;

class ReceivingTransport : public DispatchMessage {
protected:

	int MaxPacketLength;

	int messageIdentifier;
	int bigArrayIndex;

	Lock* clSem;
	Lock* cl2Sem;
	Lock* remSem;
	Lock* rem2Sem;
 	enum {tempBufferSize = 128};

	void saveClientCSR(ClientCSR*);
	void saveRemoteCSR(RemoteCSR*);

        ClientCSR*      ClientCSRArray[tempBufferSize];
        RemoteCSR*      RemoteCSRArray[tempBufferSize];

	virtual ClientCSR*	findClientCSR(Container*,Address*,int type =0);
	virtual RemoteCSR*	findRemoteCSR(Container*,Address*,Container*);

	void 	assemble(char* packet, int len, Address*,Address *,NetworkBuffer*);
	void 	assembleResponse(char* packet, int len, Address*,Address *,NetworkBuffer*);
	void 	assembleRequest(char* packet, int len, Address*,Address *,NetworkBuffer*);
	void 	assembleAsync(char* packet, int len, Address*,Address *,NetworkBuffer*);

public:
	ReceivingTransport();
	Class *classOf ();

	virtual void receive(char *packet, Address *etherAddr,Address* dest,NetworkBuffer *);
	void HandleProbe(char *newpos,Address *sender,Address *destAddr,NetworkBuffer* nb);

	virtual void inspect(TransportHeader *packet);
};

#endif ReceivingTransport_h
