/* 
 * There is only one per process.
 * This is used for remote communication.
 */

/*
 * States of the remote client state record
 * The remote client state record may be in any one 
 * of the following states.
 * 
 * AWAIT_REQ -- awaiting request
 * RECVREQ -- receiving request
 * RESPDISC -- response discarded 
 * RESPONDED -- responded
 *
 */
#ifndef RemoteCSR_h
#define RemoteCSR_h

#ifdef __GNUC__
#pragma interface
#endif

#include "DistEthernet.h"
#include "Lock.h"

class Address;
class Semaphore;
class TimeoutTimer;
class Message;
class NetworkBuffer ;
class NetworkBufferFreeList ;
class EthernetTransfer ;
class IntBitArray;
class MessageContainer;
class Container;
class ContainerRepresentative;

#include "CSR.h"

enum RCSRState {RIDLE,AWAIT_REQ,RECV_REQ,RESPDISC,RESPONDED};

/*
 * These are ordered and searched according
 * to pid and machine id
 */

class RemoteCSR :public CSR{
public:
	RemoteCSR(Container*,Address*,Container*);
	RCSRState state; // states of the remote client
/*
 * Information used in storage of records and 
 * retrieval.
 */
	ContainerRepresentative* cacheReply;
	Container* server;
	Address *client_machine;
        Container *client_id; // part of client id
        Container* dest; // destnation, message port
	int response;
	int done;

/* 
 * Keeping data for retransmissions 
 */

	void setState(int);
	void send();

/*
 * Temporary message for delivery.
 */
	Semaphore *sem;
	void timerLoop();

        void incr_packets();
	void requestDone();
	virtual void endTrans();


	void flush(int);
        void recv_request(); 
	void handleResponseProbe(int);
	void sendProbe();
};

#endif
