/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Nayeem Islam (nislam@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * UniRepresentative.h - `capability' or 
 * send rights for a MessageContainer. 
 *      $Header: /mnt/tapestry/Choices/nislam.9.29.1990/Includes/UniRepresentative
 *      /RCS/UniRepresentative.h,v 1.2 90/08/18 13:02:47 nislam Exp $
 */

#ifndef UniRepresentative_h
#define UniRepresentative_h

#ifdef __GNUC__
#pragma interface
#endif

#include "Assert.h"
#include "ContainerRepresentative.h"
#include "Address.h"

extern ClassStar UniRepresentativeClass;

class Message;
class RPCMessage;
class Transfer;
class Transport;
class EthernetTransfer;
class Domain;
class MessageContainer;
class Contianer;

/* 
 * CONCRETE CLASS: 
 * ----------------------------------
 * Intelligent capability to locate single.
 */

class UniRepresentative : public ContainerRepresentative
{
protected:

// All the transfer mechanisms are friends of
// the message port. This is a performance hack.

    friend class Transport;
    friend class BufferedTransport;

    friend class NetworkTransport;
    friend class iPSCTransport;
    friend class EthernetTransport;
    friend class ContainerManager;
    friend class SendingTransport;
    friend class ReceivingTransport;

    unsigned int   localMC :1;   // Is it local or remote?
    MessageContainer *theMC; //pointer to `real' Message
public:
    proxiable UniRepresentative (char*);
    proxiable UniRepresentative (Address*, Transport * trans=0, MessageContainer * mc = 0);

    virtual Container* localContainer();
    virtual Container* remoteContainer();
    proxiable virtual int rpcSend (RPCMessage *,MessageContainer *mp);
    Class *classOf ();
};

#endif UniRepresentative_h
