/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * Copyright (c) 1990 IBM Corporation. All Rights reserved
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Author: Panos Kougiouris ( kougiour@cs.uiuc.edu )
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * AIXContainer.h - AIX File System super block
 *
 */

#ifndef AIXContainer_h
#define AIXContainer_h


#ifdef __GNUC__
#pragma interface
#endif

#include "UNIXContainer.h"
#include "PersistentMemoryObject.h"

const int FaMAGIC = 0xffeeddcd;		/* s_magic number */
const int FaMAGICOFFSET = 4096;		/* Magic location on disk */
const int AIXNICFREE=600;
const int AIXNICINOD=325;

class aixfilsys {
	friend class AIXContainer;
	friend class AIXAllocator;

	long	s_magic;		/* 0 mgic number */
	long 	s_gfs;			/* 4 global file system number */
	long	s_fsize;		/* 8 entire volume size */
	long	s_lwm;			/* 12 */
	long	s_hwm;			/* 16 */
	long	s_llst;			/* 20 */
	long	f_store;		/* 24 */

	long	s_time;			/* 28 last superblock update */
	long	s_tfree;		/* 32 total free blocks */

	long	s_isize;		/* 36 size in blocks of i-list */
	short	s_nfree;		/* 40 number of addresses in s_free */
	unsigned short s_flags;		/* 42 file system flags */
	long	 s_tinode;		/* 44 total free i-nodes */
	long	 s_lasti;		/* 48 */
	long	 s_nbehind;		/* 52 */

	short	s_gfspack;		/* 56 */
	short	s_ninode;		/* 58 number of i-nodes i s_inode */
	short	s_dinfo[4];		/* 60 interleave stuff */
	char	s_flock;		/* 68 lock of free list manipulation */
	char	s_ilock;		/* 69 lock of i-list manipulation */
	char 	s_fmod;			/* 70 superblock modified flag */
	char	s_version;		/* 71 version of the data format */
	char	s_fsmnt[32];		/* 72 name of the files system */
	char	s_fpack[8];		/* 104 name of the volume */
	long	s_inode[AIXNICINOD];	/* 112 free i-node list */
	int	s_free[AIXNICFREE];	/* 1412 free block list */
	char	s_byteorder;		/* 3812 */
};

extern ProxiableObject * AIXContainerConstructor( ProxiableObject * );
extern PersistentClassStar AIXContainerClass;
class AIXContainer : public UNIXContainer {
protected:
	aixfilsys * _superBlock;

	int	numFragments( int size );
	int	mapInumber( int inum );
	int	firstInumber( int inum );
	int	getUnusedInode( int notUsedInThisSubClass );

	FileObject * basicOpen( int idNumber, int & status, int flags );
	FileObject * basicCreate( int idNumber, Class *, int &, int flags );
	void basicClose( FileObject * mo );
	int  basicRootId();
	void basicSynchronize();

	~AIXContainer();
public:
	Class * classOf();

	AIXContainer( PersistentMemoryObject * source );

};

#endif /*AIXContainer_h*/
