/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * Copyright (c) 1990 IBM Corporation. All Rights reserved
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Author: Panos Kougiouris ( kougiour@cs.uiuc.edu )
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * AIXInode.h - 
 */

#ifndef AIXInode_h
#define AIXInode_h


#ifdef __GNUC__
#pragma interface
#endif


#ifdef __GNUC__
#pragma interface
#endif

#include "UNIXInode.h"

#define  NADDR 		13
#define  SMBLKSZ 	384
#define DIDEL 0x01
#define DISTORE 0x02
#define DIALLOC 0x10
#define DIHIDDEN 0x20
#define DILONGDIR 0x40
#define DILINK 0x80


class AIXdinode {
	friend class AIXInode;
	friend class AIXContainer;

        unsigned short  di_mode;        /* mode and type of file */
        short           di_nlink;       /* number of links to file */
        unsigned short  di_uid;         /* owners user id */
        unsigned short  di_gid;         /* owners group id */
	long		di_uniqueid;
	short		di_filler;
	short		di_dflag;
        int             di_size;        /* number of bytes in file */
        int             di_mtime;       /* time last accessed */
        int             di_atime;       /* time last modified */
        int             di_ctime;       /* time of last file status change */
	long		di_cmtcnt;
	long		di_fstore;
	long		di_version;
	long		di_blocks;
	char		di_pad[27];
	char		di_sbflag;
	long		di_addr[NADDR];
	char		di_sbbuf[SMBLKSZ];
};

extern ClassStar AIXInodeClass;
class AIXInode : public UNIXInode {
protected:
	// Structures and functions that almost could be inherited
	AIXdinode	* _diskInode;

	int		allocBlock();
	void		freeBlock( int physicalBlock );
	int		getDirect( int index );
	void		setDirect( int index, int value );
	int		getIndirect( int level );
	void		setIndirect( int level, int value );
	int		accessMode();
	int		userId();
	int		groupId();
	int		basicMapUnit( int logicalBlock, int addMapping );

	~AIXInode();

	Class *         supports( Class * );

	int		basicSetNumberOfBytes( int size );
	int		rawRead( unsigned, int, char *, int, IOAsyncClosure * );
	int		rawWrite( unsigned, int, char *, int, int,
				  IOAsyncClosure * );
	int     	basicCopy( MemoryObject * dest );

public:
	Class * classOf();
	AIXInode( unsigned int numberOfUnits,
		  int log2UnitSize,
		  int inumber,
		  MemoryObjectContainer *,
		  PersistentMemoryObject * source,
		  AIXdinode * );

	int		isDictionary();
	int		numberOfBytes();
	int		links( int delta );
	int		setMode( int mode );
	int		setUserId( int uid );
	int		setGroupId( int gid );
	void		info( UNIXFileInfo * );
};

#endif	AIXInode_h
