/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Contributing Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * BSDAllocator.h - inode system superClass
 */

#ifndef BSDAllocator_h
#define BSDAllocator_h


#ifdef __GNUC__
#pragma interface
#endif

#include "BlockAllocator.h"
#include "MemoryObjectStar.h"

class fs;
class ag;
class BSDContainer;

extern ClassStar BSDAllocatorClass;
class BSDAllocator : public BlockAllocator {
protected:

    Lock	_lock;
    ag **	_allocatorGroup;
    int *	_totalFreeBlocks;
    int *	_totalFreeFragments;
    int 	_size;
    int		_log2FragmentToSector;
    int		_fragmentsPerBlock;
    int		_blockMask;
    int		_fragmentsPerGroup;
    unsigned		_sectorsPerCylinder;
    unsigned		_cylinderGroups;
    int		_sectors;
    int		_inodesPerGroup;
    BSDContainer * _container;

    void markBlockInUse( int cg, int block );
    int allocateBlocksFromGroup( int & blocks, int cg, int * buf, 
				 int count, int pref );

    int	blockIsFree( unsigned int cg, unsigned int fragment );
    void updateDiskInfo( unsigned int cg, unsigned int fragment , int delta );

    void updateBlocks( int & blocks, int units );
    int	basicAllocate( int & blocks, int nearPhysicalUnit, int units );
    void basicFree( int & blocks, int physicalUnit, int units );

public:
    Class * classOf();

    BSDAllocator( MemoryObject * source, BSDContainer * container,
		  ag ** cylGroup, int size,
		  int * totalFreeBlocks, int * totalFreeFragments,
		  int log2FragmentToSector, int fragmentsPerBlock,
		  int fragmentsPerGroup, int sectorsPerCylinder,
		  int cylinderGroups, int sectors, int inodesPerGroup );

    int	allocate( int & blocks, int nearPhysicalUnit, int units );
    void free( int & blocks, int physicalUnit, int units );

    int allocateBlocks( int & blocks, int inumber, int *buf, int start, 
			int count, int pref );
    int extendFragmentsToBlock( int &blocks, int physicalBlock, int fragments );
};

#endif /*BSDAllocator_h*/
