/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * BSDContainer.h - BSD Inode System Class
 */

#ifndef BSDContainer_h
#define BSDContainer_h


#ifdef __GNUC__
#pragma interface
#endif

#include "UNIXContainer.h"

class BSDAllocator;
class csum;
class ag;
class cg;
class fs;
class cgCache;
class OutputStream;

const int FbMAGICOFFSET = 9564;		/* Magic location on disk */
const int FbMAGIC = 0x11954;		/* fs_magic number */

extern ProxiableObject * BSDContainerConstructor( ProxiableObject * );
extern PersistentClassStar BSDContainerClass;
class BSDContainer : public UNIXContainer {
protected:
	fs *	_superBlock;
	csum *	_cylSumMem;
	cg **	_cylGroup;
	ag **	_allocatorGroup;
	cgCache * _cgCache;
	unsigned char ** _iused;
	int	_fragmentsPerBlock;
	int	_groupMask;

	int	mapCylGroup( int group );
	int	mapInumber( int inum );
	int	firstInumber( int inum );
	int	directoryPref();
	int 	getUnusedInodeCyl( int cyl, int inumber );
	int	getUnusedInode( int inumber );
	int	numFragments( int sizes[], int ndaddr );

	FileObject * basicOpen( int idNumber, int & status, int flags );
	FileObject * basicCreate( int idNumber, Class *, int &, int flags );
	void basicClose( FileObject * mo );
	int  basicRootId();
	void basicSynchronize();
	~BSDContainer();

	friend	class BSDInode;
	
	int 	blockPref( int inumber, int * buf, int start, int startBlock );
  BSDAllocator * allocator() { return (BSDAllocator*)(BlockAllocator*)_allocator; }

	friend  class BSDAllocator;

	void	getCG( int group );
	void	releaseCG( int group, int dirty );

public:
	Class * classOf();

	BSDContainer( PersistentMemoryObject * source );
};

const int CGbMAGIC =0x090255;
const int DEV_BSIZE = 512;
const int MAXFRAG = 8;
const int BBSIZE = 8192;
const int SBSIZE = 8192;
const unsigned int BBLOCK = 0;
const unsigned int SBLOCK = BBLOCK + BBSIZE / DEV_BSIZE;
const unsigned NRPOS  = 8;
const int MAXIPG = 2048;
const int MINBSIZE = 4096;
const unsigned MAXCPG = 32;
const int MAXMNTLEN = 512;
const int MAXCSBUFS = 32;

const int Log2BitsPerByte = 3;
const int BitsPerByteMask = 7;

static inline void
setBit( unsigned char * array, int bit )
{
	array[bit >> Log2BitsPerByte] |= 1 << ( bit & BitsPerByteMask );
}

static inline void
clearBit( unsigned char * array, int bit )
{
	array[bit >> Log2BitsPerByte] &= ~( 1 << ( bit & BitsPerByteMask ) );
}

static inline int
bitIsSet( unsigned char * array, int bit )
{
return( array[bit >> Log2BitsPerByte] & ( 1 << ( bit & BitsPerByteMask ) ) );
}

static inline int
bitIsClear( unsigned char * array, int bit )
{
	return( !bitIsSet( array, bit ) );
}

class csum {
	friend class BSDContainer;

	int	cs_ndir;
	int	cs_nbfree;
	int	cs_nifree;
	int	cs_nffree;
};

class fs {
	friend class BSDContainer;
	friend class BSDAllocator;
	friend class BSDInode;

	fs *	fs_link;
	fs *	fs_rlink;
	unsigned int	fs_sblkno;
	unsigned int	fs_cblkno;
	unsigned int	fs_iblkno;
	unsigned int	fs_dblkno;
	int	fs_cgoffset;
	int	fs_cgmask;
	unsigned int 	fs_time;
	int	fs_size;
	int	fs_dsize;
	int	fs_ncg;
	int	fs_bsize;
	int	fs_fsize;
	int	fs_frag;
	int	fs_minfree;
	int	fs_rotdelay;
	int	fs_rps;
	int	fs_bmask;
	int	fs_fmask;
	int	fs_bshift;
	int	fs_fshift;
	int	fs_maxcontig;
	int	fs_maxbpg;
	int	fs_fragshift;
	int	fs_fsbtodb;
	int	fs_sbsize;
	int	fs_csmask;
	int	fs_csshift;
	int	fs_nindir;
	int	fs_inopb;
	int	fs_nspf;
	int	fs_sparecon[6];
	unsigned int fs_csaddr;
	int	fs_cssize;
	int	fs_cgsize;
	int	fs_ntrak;
	int	fs_nsect;
	int  	fs_spc;
	int	fs_ncyl;
	int	fs_cpg;
	int	fs_ipg;
	int	fs_fpg;
	int	fs_ndir;
	int	fs_nbfree;
	int	fs_nifree;
	int	fs_nffree;
	char   	fs_fmod;
	char   	fs_clean;
	char   	fs_ronly;
	char   	fs_flags;
	char	fs_fsmnt[MAXMNTLEN];
	int	fs_cgrotor;
	csum *	fs_csp[MAXCSBUFS];
	int	fs_cpc;
	short	fs_postbl[MAXCPG][NRPOS];
	int	fs_magic;
	unsigned char	fs_rotbl[1];
};

struct	cg {
	friend class BSDContainer;

	struct  cg *cg_link;
	int	cg_magic;
	unsigned int	cg_time;
	int	cg_cgx;
	short	cg_ncyl;
	short	cg_niblk;
	int	cg_ndblk;
	int	cs_ndir;
	int	cs_nbfree;
	int	cs_nifree;
	int	cs_nffree;
	int	cg_rotor;
	int	cg_frotor;
	int	cg_irotor;
	int	cg_frsum[MAXFRAG];
	int	cg_btotoff;
	int	cg_boff;
	int	cg_iusedoff;
	int	cg_freeoff;
	int	cg_nextfreeoff;
	int	cg_sparecon[16];
	unsigned char	cg_space[1];
};

class ocg {
	friend class BSDContainer;

	ocg *	cg_link;
	ocg *	cg_rlink;
	unsigned int	cg_time;
	int	cg_cgx;
	short	cg_ncyl;
	short	cg_niblk;
	int	cg_ndblk;
	int	cs_ndir;
	int	cs_nbfree;
	int	cs_nifree;
	int	cs_nffree;
	int	cg_rotor;
	int	cg_frotor;
	int	cg_irotor;
	int	cg_frsum[MAXFRAG];
	int	cg_btot[MAXCPG];
	short	cg_b[MAXCPG*NRPOS];
	unsigned char	cg_iused[MAXIPG>>3];
	int	cg_magic;
	unsigned char	cg_free[1];
};

class ag {
	friend class BSDAllocator;
	friend class BSDContainer;
	friend class BSDInode;
	friend OutputStream & operator<< ( OutputStream &, ag & );

	short *	ag_b;
	unsigned char *	ag_free;
	int *	ag_btot;
	int *	ag_frsum;
	int *	ag_ndblk;
	int *	ag_nbfree;
	int *	ag_rotor;
};

class cgCache {
	friend class BSDContainer;

	UNIXInodeIndirectInfo * _info;
	unsigned _sector;
	char * _buf;
};

OutputStream & operator<< ( OutputStream &, ag &);

inline int
BSDContainer::mapCylGroup( int group )
{
        int fragment = _superBlock->fs_fpg * group;
	int offset = _superBlock->fs_cgoffset * ( group & _groupMask );
	return( fragment + offset );
}


#endif /*BSDContainer_h*/



