/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Contributing Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * BSDInode.h - BSD Inode class
 */

#ifndef BSDInode_h
#define BSDInode_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Semaphore.h"
#include "UNIXInode.h"

const int  NDADDR = 12;
const int  NIADDR = 3;

class fs;
class cg;
class UNIXInodeIndirectInfo;

class dinode {
	friend class BSDInode;
	friend class BSDContainer;

	short	di_mode;		/*  0: mode and type of file */
	short	di_nlink;		/*  2: number of links to file */
	short	di_uid;			/*  4: owners user id */
	short	di_gid;			/*  6: owners group id */
	int	di_size[2];		/*  8: number of bytes in file */
	unsigned int	di_atime;	/* 16: time last accessed */
	unsigned int	di_atime2;	/* 20: */
	unsigned int	di_mtime;	/* 24: time last modified */
	unsigned int	di_mtime2;	/* 28: */
	unsigned int	di_ctime;	/* 32: last time inode changed */
	unsigned int	di_ctime2;	/* 36: */
	unsigned int	di_db[NDADDR];	/* 40: direct block    */
	unsigned int	di_ib[NIADDR];	/* 88: indirect blocks */
	unsigned int	di_flags;	/* 100: status */
	int	di_blocks;		/* 104: blocks actually held */
	int	di_version;		/* 108: */
	int	di_spare[4];		/* 112: reserved, currently unused */
};

extern ClassStar BSDInodeClass;

class BSDInode : public UNIXInode {
protected:

	/*
	 *	Structures and functions that almost could be inherited
	 */
    struct _InodeData {
	UNIXInodeIndirectInfo ** _inodeIndirectInfo;
	PersistentMemoryObject * _partition;
	unsigned _inodeBlock;
	unsigned _inodeOffset;
	unsigned _di_spare3;
	unsigned _di_flags;
	unsigned _di_size;
	short    _di_mode;
	short	 _di_nlink;
    };

	dinode *_diskInode;
	_InodeData *_inodeData;

	void	initDinode();
	void	getDinode();
	void 	releaseDinode();

	int	allocBlock();
	void	freeBlock( int physicalBlock );
	void	freeFragments( int physicalBlock, int count );
	int	blockPref( int * buf, int start, int startBlock );
	int	allocBlocks( int * buf, int start, int count, int pref );

	int	accessMode();
	int	userId();
	int	groupId();

	/*
	 *	The following variables and functions handle BSD fragmentation
	 */
	int	numFragments();

	/*
	 *	nearBlock helps with BSDs block allocation optimizations
	 */
	// int	nearBlock;
	~BSDInode();

        Class * supports( Class * );

	unsigned	basicSetNumberOfBytes( unsigned size );
	unsigned    basicSetNumberOfUnits( unsigned numberOfUnits );

	int     basicRead( unsigned start, int count, 
		           char * addr, unsigned nbytes, IOAsyncClosure * c );
	int 	basicWrite( unsigned start, unsigned count,
			    char * addr, unsigned nbytes, IOAsyncClosure * c );
	int 	basicWriteThrough( unsigned start, unsigned count,
				   char * addr, unsigned nbytes, 
				   IOAsyncClosure * c );	
	int	rawRead( unsigned startUnit, unsigned count, char * src, unsigned nbytes,
			 IOAsyncClosure * c );
	int	rawWrite( unsigned startUnit, unsigned count, 
			  char * dest, unsigned n,
			  int writeThrough, IOAsyncClosure * c );
	void	updateInode( int finalSync );
	void	synchronizeInode();
	void	hintsChanged( unsigned oldHints );

public:
	Class * classOf();
	BSDInode( unsigned int numberOfUnits,
		  int log2UnitSize,
		  int inumber,
		  MemoryObjectContainer *,
		  PersistentMemoryObject * source,
		  int log2BlockSize,
		  dinode *, UNIXInodeIndirectInfo ** );

	unsigned	numberOfBytes();
	int	links( int delta );
	int	setMode( int mode );
	int	setUserId( int uid );
	int	setGroupId( int gid );
	int	isSymbolicLink();
	int	isContainer();
	int	isDictionary();
	void	info( UNIXFileInfo * );
	void	getAttributes( RFSAttributes * );
	void	setAttributes( RFSAttributes * );
};

inline int
BSDInode::numFragments()
{
	int ret = _inodeData->_di_size;
	if( ret > 0 ) {
		unsigned  units = ( ret + unitSize() - 1 ) >> log2UnitSize();
		if( ( units >> log2BlockSize ) >= singleIndirect ) ret = 0;
		else ret = units & blockMask();
	}
	return( ret );
}

#endif	/*BSDInode_h*/
