/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * COFF.h -
 */

//COMMENTS/DOCUMENTATION ADDED BY SCOTT KIRCHER
//LINUX MODULE ADAPTOR FOR CHOICES PROJECT, CS423
//All such comment blocks are ended by: -SK
//Anything I didn't understand will have a question above it of the form ??Why is this here??
//-SK

//My COFF format references used in documenting this file were:
// http://docsrv.sco.com:507/en/topics/COFF.html
// http://www2.randomlogic.com/linux_html/679004.html
// http://www.delorie.com/djgpp/doc/coff/
//-SK

#ifndef COFF_h
#define COFF_h


#ifdef __GNUC__
#pragma interface
#endif

//Standard COFF file header - SK
class filehdr {
	friend class COFFContainer;
	friend class COFFDictionary;

	unsigned short	f_magic; //Magic number indicating target machine for which the object was compiled -SK
	unsigned short	f_nscns; //Number of sections in the COFF file -SK
	int		f_timdat; //Timestamp indicating when file was created (number of seconds since 00:00:00GMT Jan. 1, 1970 -SK
	int		f_symptr; //File pointer (file offset) indicating the start of the symbol table -SK
	int		f_nsyms; //Number of entries in that symbol table -SK
	unsigned short	f_opthdr; //Number of bytes in the optional header (the optional header stores system-dependent information) -SK
	unsigned short	f_flags; //Flags (stored in a bitmask manner). See below for defined flags. -SK
};

//These are the basic COFF flags (in octal) -SK
const int F_RELFLG	= 0000001; //Indicates the relocation information has been stripped from the file -SK
const int F_EXEC		= 0000002; //Indicates there are no undefined references, and object is ready to execute -SK
const int F_LNNO		= 0000004; //Indicates line numbers have been stripped from the file -SK
const int F_LSYMS	= 0000010; //Indicates local symbols have been stripped from the file -SK

//These are more specialized COFF flags -SK
const int F_MINMAL	= 0000020; //Indicates this is a minimal file (*.m) -SK
const int F_UPDATE	= 0000040; //Indicates this is a fully bound update file -SK
const int F_SWABD	= 0000100; //??Indicates bytes have been swabbed, but what does THAT mean?? -SK
const int F_AR16WR	= 0000200; //16 bit words, little endian -SK
const int F_AR32WR	= 0000400; //32 bit words, little endian (i386) -SK
const int F_AR32W	= 0001000; //32 bit words ??big endian?? -SK
const int F_PATCH	= 0002000;
const int F_NODF		= 0002000;
const int F_NS32K_ID	= 0174000;
const int F_APC_CONE	= 0004000;
const int F_XPC532	= 0010000;
const int F_XPC381	= 0020000;
const int F_XPC580	= 0040000;
const int F_32000	= 0100000;

//Various magic numbers for some systems (in octal) -SK
//Note, i386's magic number is 0514, which is obviously not listed below
//Not too important for our purposes, we just need to know which one of these corresponds to
//the particular machine we're developing on. -SK
const int NS32GMAGIC	= 0524; //According to COFFContainer.cc, this magic number...
const int NS32SMAGIC	= 0525; //...and this one are the only ones supported by Choices -SK
const int MC68MAGIC	= 0520; //So, we can ignore all the rest -SK
const int MC68TVMAGIC	= 0521;
const int M68MAGIC	= 0210;
const int M68TVMAGIC	= 0211;
const int U370WRMAGIC	= 0530;
const int U370ROMAGIC	= 0535;

//a.out header
//This is often in the "optional header" area mentioned above (size is given by f_opthdr in the filehdr structure)
//Linux, for example, requires an a.out header in its COFF files, because it can specify an entry point address
//(default COFF assumes the entry point is the first location in the .text section) -SK
/*??Who can access this data? It looks like everything is private here, and no friend classes are specified?? -SK
??According to COFFContainer.cc, the f_opthdr value must be 0 or equal to sizeof(aouthdr), however the actual
  data stored in the aouthdr doesn't appear to be used anywhere?? -SK */
//To further complicate things, the a.out file header doesn't appear to be fully standardized. The one required
//by Linux is NOT this one: -SK
class aouthdr {
	short	magic;
	short	vstamp;
	int	tsize;
	int	dsize;
	int	bsize;
	int	msize;
	int	mod_start;
	int	entry;
	int	text_start;
	int	data_start;
	short	entry_mod;
	unsigned short	flags;
};

//??Are these for aouthdr?? -SK
const int U_AL		= 0x07;
const int U_AL_NONE	= 0x00;
const int U_AL_512	= 0x01;
const int U_AL_1024	= 0x02;
const int U_AL_2048	= 0x03;
const int U_AL_4096	= 0x04;
const int U_AL_8192	= 0x05;
const int U_PR		= 0x38;
const int U_PR_DATA	= 0x08;
const int U_PR_TEXT	= 0x10;
const int U_PR_MOD	= 0x20;
const int U_SYS_5	= 0x100;
const int U_SYS_42	= 0x200;
const int U_UMAX_4	= 0x400;
const int U_UMAX_5	= 0x800;

//Section header
//One of these headers appears before each section (i.e. .text, .data, etc...) in the COFF file -SK
class scnhdr {
	friend class COFFContainer;
	friend class COFFDictionary;

	char		s_name[8]; //The name of the section (".text", ".name", etc...) -SK
	int		s_paddr; //I don't think this,
	int		s_vaddr; //or this are used by the COFFloader -SK
	int		s_size; //Size (in bytes) of the data for this section -SK
	int		s_scnptr; //File offset of where the data begins for this section -SK
	int		s_relptr; //File offset of where relocation entries begin for this section -SK
	int		s_lnnoptr; //File offset of where line number entries begin for this section -SK
	unsigned short	s_nreloc; //Number of relocation entries -SK
	unsigned short	s_nlnno; //Number of line number entries -SK
	int		s_flags; //section header flags (see below) -SK
	int		s_symptr; //??What is this?? -SK
	unsigned short	s_modno; //??What is this?? -SK
	short		s_pad; //Padding, I assume, to make the structure a certain size. -SK
};

//Section header flags -SK
const int STYP_REG	= 0x00; //regular section, to be allocated, relocated, loaded -SK
const int STYP_DSECT	= 0x01; //dummy section, to be not-allocated, relocated, not-loaded -SK
const int STYP_NOLOAD	= 0x02; //no-load section, to be allocated, relocated, not-loaded -SK
const int STYP_GROUP	= 0x04; //Grouped section, formed from several inputs -SK
const int STYP_PAD	= 0x08; //Padding section, to be not-allocated, not-relocated, loaded -SK
const int STYP_COPY	= 0x10; //Copy section (??extra relocation info??), to be not-allocated, not-relocated (but relocation info processed), loaded -SK
const int STYP_INFO	= 0x200; //Comment section, section should be ignored -SK
const int STYP_LIB	= 0x800; //.lib section, section should be ignored -SK
const int STYP_OVER	= 0x400; //Overlay section, to be not-allocated, relocated, not-loaded -SK
const int STYP_SHARE	= 0x400; //??Not sure with this, but for some reason it uses the same bit as STYP_OVER?? -SK
const int STYP_TEXT	= 0x20; //Section contains executable code -SK
const int STYP_DATA	= 0x40; //Section contains initialized data -SK
const int STYP_BSS	= 0x80; //Section contains uninitialized data -SK

const int S_NEWFCN	= 0x10;
const int S_SHRSEG	= 0x20;

//Storage class identifiers for symbols
//Tells where and what the symbol represents -SK
const int C_EFCN		= -1; //physical end of function -SK
const int C_NULL		= 0; //Nothing -SK
const int C_AUTO		= 1; //Automatic variable -SK
const int C_EXT		= 2; //external (public) symbol (globals and externs) -SK
const int C_STAT		= 3; //static (private) symbol -SK
const int C_REG		= 4; //register variable -SK
const int C_EXTDEF	= 5; //external definition -SK
const int C_LABEL	= 6; //label -SK
const int C_ULABEL	= 7; //undefined label -SK
const int C_MOS		= 8; //Member of a structure or class -SK
const int C_ARG		= 9; //Function argument -SK
const int C_STRTAG	= 10; //Structure tag -SK
const int C_MOU		= 11; //Member of a union -SK
const int C_UNTAG	= 12; //Union tag -SK
const int C_TPDEF	= 13; //Type definition -SK
const int C_USTATIC	= 14; //Undefined static -SK
const int C_ENTAG	= 15; //Enumeration tag -SK
const int C_MOE		= 16; //Member of an enumeration -SK
const int C_REGPARM	= 17; //Register parameter -SK
const int C_FIELD	= 18; //Bit field -SK
const int C_SHARE	= 19; //??shared/autoargument?? -SK

const int C_BLOCK	= 100; //Begining or end of a block -SK
const int C_FCN		= 101; //begining or end of a function -SK
const int C_EOS		= 102; //end of a structure -SK
const int C_FILE		= 103; //file name -SK
const int C_LINE		= 104; //line number, reformatted as a symbol -SK
const int C_ALIAS	= 105; //duplicate tag -SK
const int C_HIDDEN	= 106; //??Not sure about this one?? -SK
const int C_SHADOW	= 107; //??What is this?? -SK

const int SYMNMLEN	= 8; //Maximum length of an inlined symbol name -SK
const int FILNMLEN	= 14;
const int DIMNUM		= 4;

//Symbol entries have the following structure -SK
class syment {
	friend class COFFLoader;

	//the first field can either be the inlined name of the function
	//or a flag and a pointer to the location in the string table that
	//contains the true name -SK
	union {
		char		_n_name[SYMNMLEN]; //inlined name -SK
		int		_n_nptr[2]; //if _n_nptr[0] is 0, then the symbol name can be found at the offset _n_nptr[1] into the string table -SK
	} _n;
	int		n_value; //the value of the symbol (address (in the section it belongs to) or a constant value) -SK
	short		n_scnum; //section number of the section the symbol's address is relative to. Or one of the negative values shown below -SK
	unsigned short	n_type; //symbol type (see below) -SK
	char		n_sclass; //storage class (see above) -SK
	char		n_numaux; //number of sizeof(syment) sized auxillary data following this symbol data -SK
	char		n_env; //??What is this?? -SK
	char		n_dummy; //??What is this?? -SK
};

//additional flags for n_scnum
const int N_UNDEF	= 0; //Undefined (extern) symbol. No section corresponds, needs to be resolved. -SK
const int N_ABS		= -1; //Absolute symbol (n_value is constant, not address) -SK
const int N_DEBUG	= -2; //A debugging symbol -SK
const int N_TV		= (unsigned short)-3; //??What is this?? -SK
const int P_TV		= (unsigned short)-4; //??What is this?? -SK

//These are the values that can go into the n_type field
//Should be obvious what most of them represent -SK
const int T_NULL		= 0;
const int T_ARG		= 1;
const int T_CHAR		= 2;
const int T_SHORT	= 3;
const int T_INT		= 4;
const int T_LONG		= 5;
const int T_FLOAT	= 6;
const int T_DOUBLE	= 7;
const int T_STRUCT	= 8;
const int T_UNION	= 9;
const int T_ENUM		= 10;
const int T_MOE		= 11; //Member of Enumeration -SK
const int T_UCHAR	= 12; //Unsigned Char... -SK
const int T_USHORT	= 13;
const int T_UINT		= 14;
const int T_ULONG	= 15;

//Derived types, these go in the high byte of n_type
const int DT_NON		= 0; 
const int DT_PTR		= 1; //Pointer to _ -SK
const int DT_FCN		= 2; //Function returning _ -SK
const int DT_ARY		= 3; //Array of _ -SK

const int N_BTMASK	= 017;
const int N_TMASK	= 060;
const int N_TMASK1	= 0300;
const int N_TMASK2	= 0360;
const int N_BTSHFT	= 4;
const int N_TSHIFT	= 2;

//The auxilary structure is what can follow a regular symbol entry
//providing additional information about the symbol (such as debugger info).
//despite the fact that COFFLoader is listed as a friend, it doesn't appear to
//use this at all. -SK
//This appears to be very system specific, and as yet I haven't found
//documentation for this particular format. Though it appears to be mostly
//debugging info (like line numbers, sizes, etc...). -SK
union auxent {
	friend class COFFLoader;

	struct {
		int		x_tagndx;
		union {
			struct {
				unsigned short	x_lnno;
				unsigned short	x_size;
			} x_lnsz;
			int	x_fsize;
		} x_misc;
		union {
			struct {
				int	x_lnnoptr;
				int	x_endndx;
			} x_fcn;
			struct {
				unsigned short	x_dimen[DIMNUM];
			} x_ary;
		} x_fcnary;
		union {
			struct {
				unsigned short x_callseq;
				unsigned short x_level;
			} x_fend;
			struct {
				unsigned short x_tvindx;
				unsigned short x_plude;
			} x_befn;
		} x_fnend;
	} x_sym;
	struct {
		char		x_fname[FILNMLEN];
		unsigned short	x_fdummy;
		int		x_foff;
	} x_file;
	struct {
		int		x_scnlen;
		unsigned short	x_nreloc;
		unsigned short	x_nlinno;
		int		x_linoptr;
	} x_scn;
};

//Relocation entries needed when translating the file addresses
//to "true" (i.e. in memory) virtual addresses (relocation) -SK
class reloc {
	friend class COFFLoader;

	int	r_vaddr; //virtual address (that is, file offset) of a reference to some symbol -SK
	int	r_symndx; //index into the symbol table for that symbol -SK
	unsigned short	r_type; //relocation flags (see below) -SK
};

//masks and flags for use in the r_type field -SK
const int R_Type		= 0xf; //mask to obtain relocation type -SK
const int R_Address	= 0x1; //address type (the only type supported by COFFLoader. Not sure what other types there might be) -SK

const int R_RelativeTo	= 0xf0; //mask to obtain relativity flag -SK
const int R_Absolute	= 0x00; //address is not relative to program counter or static base ("p_glbl") -SK
const int R_PC_Relative	= 0x10; //address is relative to program counter -SK
const int R_SB_Relative	= 0x20; //address is relative to static base ("p_glbl" symbol in main symbol table) -SK

const int R_Format	= 0xf00; //mask to obtain format flag -SK
const int R_Number	= 0x000; //The only kind of relocation allowed for references from .data ??what is this, though?? -SK
const int R_Displacement = 0x100; //relocation format required for .text ??What does it mean, exactly?? -SK

const int R_Size		= 0xf000; //mask to obtain size flag -SK
const int R_Four		= 0x2000; //reference is a 32-bit pointer -SK

const int RELOCSIZE = 10; //Size of the relocation structure -SK
const int LINENOSIZE = 6; //??Size of the line number entries?? -SK

//COFFDictionary/COFFContainer flags to identify different sections -SK
const int RootId = 1;
const int FileHeaderId = 2;
const int AOutHeaderId = 3;
const int SymbolTableId = 4;
const int StringTableId = 5;
const int SectionIds = 6;

const int Header  = 0;
const int Section = 1;
const int Reloc   = 2;
const int LineNo  = 3;

#endif /*COFF_h*/
