/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
#ifndef COFFLoader_h
#define COFFLoader_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Object.h"
#include "COFF.h"
#include "FileSystemInterfaceStar.h"
#include "MemoryObjectDictionaryStar.h"
#include "RecordStreamStar.h"

extern int match( char * string, char * pattern );

const char * const MainSymbolTableName = "/Code/FiSh/Symbol.Table";
const char * const MainStringTableName = "/Code/FiSh/String.Table";

extern ClassStar COFFLoaderClass;
class COFFLoader : public Object {
protected:
	FileSystemInterfaceStar		_fsi;

	MemoryObjectDictionaryStar	_root;

	char *		_text;
	int		_textSize;
	char *		_data;
	int		_dataSize;

	int		_textPrelocated;
	int		_textRelocated;
	int		_dataPrelocated;
	int		_dataRelocated;

	int		_staticBase;

	syment *	_mainSymbols;
	int		_mainSymbolCount;
	char *		_mainStrings;

	syment *	_loadSymbols;
	int		_loadSymbolCount;
	char *		_loadStrings;

	char *		_textRelocs;
	int		_textRelocCount;
	char *		_dataRelocs;
	int		_dataRelocCount;

	char		_loadObjectName[48];
	char		_loadSymbolTableName[64];
	char		_loadStringTableName[64];
	char		_loadTextName[64];
	char		_loadDataName[64];
	char		_loadTextRelocName[72];
	char		_loadDataRelocName[72];

	int		readSection( char * path, char * & buffer, int & size );
        int             readSymbols( RecordStream * rfs1,
				     syment * & buf1, int & symCount,
                                     RecordStream * rfs2,
				     char * & buf2 );
	int		resolveUndefined();
	int		resolveDefined();
	int		relocateSymbols();

	int		installSizeOf( char * key, syment *);
	int		installConstructor( char * key, syment *);

public:
	COFFLoader( char * className );
	~COFFLoader();

	Class * classOf();
};

#endif /*COFFLoader_h*/
