/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ContainerIndex.h
 */

#ifndef ContainerIndex_h
#define ContainerIndex_h

#ifdef __GNUC__
#pragma interface
#endif

#include "Lock.h"

class FileObject;
class MemoryObjectContainer;

/* should be using const int _maxOpen and const int _hashTableSize
 * in ContainerIndex class
 * but Proxify does not like it.
 */

// #define DEBUG_CONTAINERINDEX 
#define CONTAINERINDEX_HASHTABLESIZE 3019

class ContainerIndex {
	struct _IndexEntry {
		MemoryObjectContainer * _container;
		int _index;
		FileObject * _fileObject;
		_IndexEntry * _next;
#ifdef DEBUG_CONTAINERINDEX
		_IndexEntry ** _list;
#endif
	};
#ifdef DEBUG_CONTAINERINDEX
	int _hashTableCount[CONTAINERINDEX_HASHTABLESIZE];
#endif
	_IndexEntry * _hashTable[CONTAINERINDEX_HASHTABLESIZE];
	_IndexEntry * _free;
	int _freeCount;
	static Lock _lock;

	static void acquire() { _lock.acquire(); }
	static void release() { _lock.release(); }
	int hash(MemoryObjectContainer *c, int index); 
	void errfull();
public:
	ContainerIndex();
	FileObject * lookup(MemoryObjectContainer * c, int index);
	void bind(MemoryObjectContainer *c, int index, FileObject * f);
	void unbind(MemoryObjectContainer *c, int index);
	FileObject ** getActiveFileObjects(MemoryObjectContainer *c);
	void freeActiveFileObjects(FileObject **);
};

extern ContainerIndex * TheContainerIndex;

inline
int 
ContainerIndex::hash(MemoryObjectContainer *c, int index) 
{ 
	return ((unsigned) c + index) % CONTAINERINDEX_HASHTABLESIZE;
}

#endif /*ContainerIndex_h*/

