/*
 * This file is part of the Choices Operating System
 * Developed by: The Mobile Operating Systems Group
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 201 N. Goodwin Ave.
 *		 Urbana, IL	61801
 *
 * Copyright 2005 University of Illinois Board of Trustees. All Rights Reserved.
 *
 * Author: Jeffrey Carlyle (jcarlyle@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 */
/*
 * Cramfs.h - Data structures and types used by cramfs implementation.
 */

#ifndef Cramfs_h
#define Cramfs_h

#ifdef __GNUC__
#pragma interface
#endif

#include "OutputStream.h"
#include "MachineTypes.h"

const int8 CramfsModeWidth = 16;
const int8 CramfsUidWidth = 16;
const int8 CramfsSizeWidth = 24;
const int8 CramfsGidWidth = 8;
const int8 CramfsNameLenWidth = 6;
const int8 CramfsOffsetWidth = 26;
const int8 CramfsSignatureLength = 16;
const int8 CramfsNameLength = 16;

const int16 CramfsMaxPath = (((1 << CramfsNameLenWidth) - 1) << 2);

const int16 CramfsBlockSize = 4096;
const int16 CramfsLog2UnitSize = 12;

enum CramfsFlags {
  CramfsFlagFsinfoVersion2    = 0x001,
  CramfsFlagSortedDirs	      = 0x002,
  CramfsFlagHoles             = 0x100,
  CramfsFlagWrongSignature    = 0x200,
  CramfsFlagShiftedRootOffset = 0x400
};

const int32 CramfsMagicNumber = 0x28cd3d45;

const int32 CramfsIsDirMode = 0x4000;

struct CramfsInode
{
  int32 mode:CramfsModeWidth;
  int32 uid:CramfsUidWidth;
  int32 size:CramfsSizeWidth;
  int32 gid:CramfsGidWidth;
  int32 namelen:CramfsNameLenWidth;
  int32 offset:CramfsOffsetWidth;

  void dumpTo(OutputStream &out);
};

struct CramfsFSInfo
{
  int32 crc;
  int32 edition;
  int32 blocks;
  int32 files;
};

struct CramfsSuperblock
{
  int32 magic;
  int32 size;
  int32 flags;
  int32 future;
  int8  signature[16];
  CramfsFSInfo	fsinfo;
  int8  name[16];
  CramfsInode root;
};

#endif /* Cramfs_h */
