/*
 * This file is part of the Choices Operating System
 * Developed by: The Mobile Operating Systems Group
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 201 N. Goodwin Ave.
 *		 Urbana, IL	61801
 *
 * Copyright 2005 University of Illinois Board of Trustees. All Rights Reserved.
 *
 * Author: Jeffrey Carlyle (jcarlyle@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 */
/*
 * CramfsDirObject.h - Cramfs Directory Entry (inode) class
 */

#ifndef CramfsDirObject_h
#define CramfsDirObject_h

#ifdef __GNUC__
#pragma interface
#endif

#include "FileObject.h"
#include "Cramfs.h"
#include "Class.h"

extern ClassStar CramfsDirObjectClass;

class CramfsDirObject : public FileObject
{
  public:
    Class *classOf();
    
    CramfsDirObject(int idNumber, MemoryObjectContainer *container,
	FileObject *source, CramfsInode *pInode);

    virtual ~CramfsDirObject();

    virtual Class *supports(Class *aClass);

    int isDictionary(void);
    int userId(void);
    int groupId(void);
    int accessMode(void);
    void info(UNIXFileInfo *infoBuffer);

  protected:
    CramfsInode *_pInode;
    Class *_supports;
};

#endif /* CramfsDirObject_h */

