/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * DictionaryStack.h - to be implemented
command		dirstack
		/
cd a		/ a
cd ..		/
cd /a		/ a
cd ..		/
cd x -> /a/b/c	/ c
cd ..		/ b
cd x -> /a/b/c	/ b c
cd ^		/


	token			action

	/			stack()			zero stack
	sym-link		push()			push base directory
	^	if (stackSize>1)pop()  otherwise top()	pop if possible
	..			top()			replace top
	.			top()			replace top (i.e. noop)
	other			top()			replace top
 */

#ifndef DictionaryStack_h
#define DictionaryStack_h


#ifdef __GNUC__
#pragma interface
#endif

class MemoryObjectDictionary;

class DictionaryStack {
protected:
	int			_top;
	int			_size;
	MemoryObjectDictionary **	_stack;
public:
	DictionaryStack( MemoryObjectDictionary * od );
	~DictionaryStack();

	MemoryObjectDictionary *	push( MemoryObjectDictionary * );
	MemoryObjectDictionary *	pop();
	MemoryObjectDictionary *	top( MemoryObjectDictionary * );
};

inline MemoryObjectDictionary *	
DictionaryStack::push( MemoryObjectDictionary * od )
{
	_top++;
	Assert( _top < _size );

	_stack[_top] = od;
	Assert( od != 0 );

	return( _stack[_top] );
}

inline MemoryObjectDictionary *
DictionaryStack::pop()
{
	Assert( _top >= 0 );
	if( _top > 0 ) _top--;

	return( _stack[_top] );
}

inline MemoryObjectDictionary *
DictionaryStack::top( MemoryObjectDictionary * od )
{
	Assert( _top >= 0 );
	Assert( _top < _size );

	if( od != 0 ) _stack[_top] = od;

	return( _stack[_top] );
}

#endif /*DictionaryStack_h*/
