/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Panos Kougiouris (kougiour@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/* The file has three parts:
 * 1. Class DiskContainer - MemoryObjectContainer representing the layaout of
 *                       a disk
 */

#ifndef DiskContainer_h
#define	DiskContainer_h


#ifdef __GNUC__
#pragma interface
#endif


/**********************************************************************/
/*    CLASS DISKConatiner                                             */
/**********************************************************************/


#include "MemoryObjectContainer.h"

/*
 * A Class to represent each diskette  as a MemoryObjectContainer.
 */
extern ClassStar DiskContainerClass;

class DiskContainer : public MemoryObjectContainer {
public:
	/* DESCR: 	A function to return a PersistentMemoryObject
	 *		representing the given partition.
	 * INPUT:	1] The number of the partition we want to open.
	 *		   Here this number cannot be other than one
	 *		   but subclasses can overload the method.
	 * OUTPUT:	1] A metaobject representing our class.
	 *		2] A variable status that can be:
	 *			0 meaning ok
         *			UnuseIdNumber if the idnumber is out of range.
	 */
	virtual FileObject * basicOpen( int idNumber, int & status,
				        int flags );

	/* DESCR: 	A function to return a metaobject representing our
	 *		class.
	 * OUTPUT:	A metaobject representing our class.
	 */
	Class *
	classOf();

	/* DESCR: 	The constructor of a disk container. It sets the number
	 *		of the partitions. A Disk can only have on partition.
	 *		However MachineDependent subclasses might have more.
	 *
	 * INPUT:	1] The source of the Container.
	 *		2] The number of the partitions.
	 */
	DiskContainer( PersistentMemoryObject * source, int partitions );

protected:

	/* DESCR: An instance variable that remembers how many partition
	 *        the disk has.
         * INITIAL VALUE: The initial value is passed as argument to the
	 *	          constructor.
	 */
	int _partitions;
};


#endif /*DiskContainer_h*/
