/*****************************
*ELFLoader                   *
*Like the COFFLoader, but for*
*ELF files. Specifically     *
*designed to load Linux ELF  *
*binaries (for OS modules)   *
******************************
*Linux adaptor for Choices   *
*Project, CS423, Spring 2004 *
*****************************/
//SCOTT KIRCHER
//Note, this code is machine dependent (on SPARCS)
//even though it isn't in the MachineDependent directory


#ifndef ELFLoader_h
#define ELFLoader_h

#ifdef __GNUC__
#pragma interface
#endif

#include "elF.h"
#include "Object.h"
#include "FileSystemInterfaceStar.h"
#include "MemoryObjectDictionaryStar.h"
#include "RecordStreamStar.h"
#include "PersistentMemoryObjectStar.h"

extern int match( char * string, char * pattern );

extern ClassStar ELFLoaderClass;
class ELFLoader : public Object {
public:
	//exported symbol table (basically contains names of ELF symbols and the address that should go with it)
	//this will be used in a call to match(), so can use wildcards: *
	//pass an array of these in to the constructor to perform undefined symbol linking
	struct ExportTuple
	{
		char *name; //the symbol's name
		void *address; //the address to GIVE to the loaded module
	};
	//required symbol table (this contains a list of names for symbols that are required
	//to be defined in the LKM (such as init_module, cleanup_module). They will be found
	//and added to this table when the module is loaded
	class ImportTuple
	{
	public:
		ImportTuple() {name=0;address=0;}
		char *name; //the symbol's name
		void **address; //WHERE to PLACE the address FOUND in the loaded LKM
	};

protected:
	FileSystemInterfaceStar		_fsi;

	MemoryObjectDictionaryStar	_root;

	char *		_text;
	int		_textSize;
	char *		_data;
	int		_dataSize;
	char *		_bss; //unitialized data
	int		_bssSize;
	char *		_rodata;
	int		_rodataSize;
	char *		_rodatastr18;
	int		_rodatastr18Size;

	int		_textPrelocated;
	int		_textRelocated;
	int		_dataPrelocated;
	int		_dataRelocated;
	int		_bssPrelocated;
	int		_bssRelocated;
	int		_rodataPrelocated;
	int		_rodataRelocated;
	int		_rodatastr18Prelocated;
	int		_rodatastr18Relocated;

	llvm::ELF::Elf32_Sym *	_loadSymbols;
	int		_loadSymbolCount;
	char *		_loadStrings;

	char *		_textRelocs;
	int		_textRelocCount;
	char *		_dataRelocs;
	int		_dataRelocCount;
	char *		_bssRelocs;
	int		_bssRelocCount;
	char *		_rodataRelocs;
	int		_rodataRelocCount;
	char *		_rodatastr18Relocs;
	int		_rodatastr18RelocCount;

	char		_loadObjectName[48];
	char		_loadSymbolTableName[64];
	char		_loadStringTableName[64];
	char		_loadTextName[64];
	char		_loadDataName[64];
	char		_loadBssName[64];
	char		_loadRODataName[64];
	char		_loadRODatastr18Name[64];
	char		_loadTextRelocName[72];
	char		_loadDataRelocName[72];
	char		_loadBssRelocName[72];
	char		_loadRODataRelocName[72];
	char		_loadRODatastr18RelocName[72];

	int _textSectionNumber;
	int _dataSectionNumber;
	int _bssSectionNumber;
	int _rodataSectionNumber;
	int _rodatastr18SectionNumber;

	char * getString(int i) {return &_loadStrings[i];}

	int		readSection( char * path, char * & buffer, int & size );
    
	int     readSymbols( RecordStream * rfs1, llvm::ELF::Elf32_Sym * & buf1, int & symCount, RecordStream * rfs2, char * & buf2 );
	int		resolveUndefined(int export_symbol_size, ExportTuple *export_symbol_table);
	int		resolveDefined(int import_symbol_size, ImportTuple *import_symbol_table);
	int		relocateSymbols();

	int	processRelocation(llvm::ELF::Elf32_Rela *rp,char *buffer);
	RecordStreamStar convertToRecordStream(PersistentMemoryObjectRef mo);
	
	int error_code;

public:

	ELFLoader( const char * className, int export_symbol_size, ExportTuple *export_symbol_table, int import_symbol_size, ImportTuple *import_symbol_table );
	int getStatus() {return error_code;}

	~ELFLoader();

	Class * classOf();
};

#endif /*ELFLoader_h*/
