/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Contributing Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * EXT2Allocator.h - inode system superClass
 */

#ifndef Ext2Allocator_h
#define Ext2Allocator_h


#ifdef __GNUC__
#pragma interface
#endif

#include "BlockAllocator.h"
#include "MemoryObjectStar.h"
#include "Ext2Container.h"
class fs;
class ag;
class ext2superblock;/*assume ext2 super block struct name*/
class EXT2Container;

struct ext2_group_desc
{
    unsigned int bg_block_bitmap;
    unsigned int bg_inode_bitmap;
    unsigned int bg_inode_table;
    unsigned short bg_free_blocks_count;
    unsigned short bg_free_inodes_count;
    unsigned short bg_used_dirs_count;
    unsigned short bg_pad;
    unsigned int bg_reserved[3];
};


extern ClassStar Ext2AllocatorClass;
class Ext2Allocator : public BlockAllocator {
 protected:
    ext2filsys * _sb;

    /*following are ported from linux ext2 balloc.c*/
    struct ext2_group_desc * get_group_desc( ext2filsys * sb,
                                             unsigned int block_group, struct buffer_head * bh );
    unsigned long ext2_count_free (struct buffer_head * map, unsigned int numchars);
    void read_block_bitmap( ext2filsys * sb, unsigned int
                            block_group, unsigned long bitmap_nr);
    int load__block_bitmap( ext2filsys * sb, unsigned int
                            block_group);
    inline int load_block_bitmap( ext2filsys * sb, unsigned int
                                  block_group);
    long ext2_count_free_blocks ( ext2filsys * sb);
    inline int block_in_use( unsigned int block,
                             ext2filsys * sb, unsigned char *map);        
    void ext2_check_blocks_bitmap ( ext2filsys * sb); 
    int basicAllocate( int & blocks, int nearPhysicalUnit, int units );
    void basicFree( int & blocks, int physicalUnit, int units );
    inline unsigned long ffz(unsigned long word);
    int test_bit(int nr, const void * addr);
    inline int clear_bit(int nr, unsigned long *addr);
    inline int set_bit(int nr, unsigned long *addr);
    int find_next_zero_bit (void * addr, int size, int offset);
    int find_first_zero_bit(void * addr, unsigned size);
    struct buffer_head * bread(int block, int size);
    void brelse(struct buffer_head * buf);

 public:
    Class * classOf();

    /* Don't need that many parameters compared to BSD, all we need are source, 
      containter and superblock*/
    Ext2Allocator( MemoryObject * source, ext2filsys *sb,
                   int log2FragmentToSector);

//    int allocateBlocks( int & blocks, int inumber, int *buf, int start, 
//          int count, int pref );
//    int extendFragmentsToBlock( int &blocks, int physicalBlock, int fragments );
};

#endif /*EXT2Allocator_h*/
