/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * Copyright (c) 1990 IBM Corporation. All Rights reserved
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Author: Panos Kougiouris ( kougiour@cs.uiuc.edu )
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * Ext2Container.h - Ext2 File System super block
 *
 */

#ifndef Ext2Container_h
#define Ext2Container_h


#ifdef __GNUC__
#pragma interface
#endif

#include "UNIXContainer.h"
#include "PersistentMemoryObject.h"


const int FeMAGICOFFSET = 0x438;        /* Magic location on disk */

/* Ext2FS is little-endian. Need to swap bytes on a big endian-machine */
#ifdef NEED_ENDIAN_SWAP
const unsigned short EXT2_SUPER_MAGIC=0x53EF;         /* Linux's magic number */
#else
const unsigned short EXT2_SUPER_MAGIC=0xEF53;         /* Linux's magic number */
#endif

const unsigned short FeMAGIC=EXT2_SUPER_MAGIC;

#define EXT2_MAX_GROUP_LOADED   8


#define EXT2_DIDEL 0x01
#define EXT2_DISTORE 0x02
#define EXT2_DIALLOC 0x10
#define EXT2_DIHIDDEN 0x20
#define EXT2_DILONGDIR 0x40
#define EXT2_DILINK 0x80

struct buffer_head {
    /* First cache line: */
    unsigned long b_blocknr;            /* block number */
    // kdev_t b_dev;                    /* device (B_FREE = free) */
    // kdev_t b_rdev;                   /* Real device */
    unsigned long b_rsector;            /* Real buffer location on disk */
    struct buffer_head * b_next;        /* Hash queue list */
    struct buffer_head * b_this_page;   /* circular list of buffers in one page */

    /* Second cache line: */
    unsigned long b_state;              /* buffer state bitmap (see above) */
    struct buffer_head * b_next_free;
    unsigned int b_count;               /* users using this block */
    unsigned long b_size;               /* block size */

    /* Non-performance-critical data follows. */
    char * b_data;                      /* pointer to data block (1024 bytes) */
    unsigned int b_list;                /* List that this buffer appears */
    unsigned long b_flushtime;          /* Time when this (dirty) buffer
                                         * should be written */
    unsigned long b_lru_time;           /* Time when this buffer was 
                                         * last used. */
    struct wait_queue * b_wait;
    struct buffer_head * b_prev;        /* doubly linked list of hash-queue */
    struct buffer_head * b_prev_free;   /* doubly linked list of buffers */
    struct buffer_head * b_reqnext;     /* request queue */
};

struct ext2_super_block {
    unsigned int    s_inodes_count;         /* Inodes count */
    unsigned int    s_blocks_count;         /* Blocks count */
    unsigned int    s_r_blocks_count;       /* Reserved blocks count */   
    unsigned int    s_free_blocks_count;    /* Free blocks count */
    unsigned int    s_free_inodes_count;    /* Free inodes count */
    unsigned int    s_first_data_block;     /* First Data Block */
    unsigned int    s_log_block_size;       /* Block size */
    int             s_log_frag_size;        /* Fragment size */
    unsigned int    s_blocks_per_group;     /* # Blocks per group */
    unsigned int    s_frags_per_group;      /* # Fragments per group */  
    unsigned int    s_inodes_per_group;     /* # Inodes per group */
    unsigned int    s_mtime;                /* Mount time */
    unsigned int    s_wtime;                /* Write time */
    unsigned short  s_mnt_count;            /* Mount count */
    short           s_max_mnt_count;        /* Maximal mount count */
    unsigned short  s_magic;                /* Magic signature */
    unsigned short  s_state;                /* File system state */
    unsigned short  s_errors;               /* Behaviour when detecting errors */
    unsigned short  s_minor_rev_level;      /* minor revision level */
    unsigned int    s_lastcheck;            /* time of last check */
    unsigned int    s_checkinterval;        /* max. time between checks */
    unsigned int    s_creator_os;           /* OS */
    unsigned int    s_rev_level;            /* Revision level */
    unsigned short  s_def_resuid;           /* Default uid for reserved blocks */
    unsigned short  s_def_resgid;           /* Default gid for reserved blocks */

    unsigned int    s_first_ino;            /* First non-reserved inode */
    unsigned short  s_inode_size;           /* size of inode structure */
    unsigned short  s_block_group_nr;       /* block group # of this superblock */
    unsigned int    s_feature_compat;       /* compatible feature set */
    unsigned int    s_feature_incompat;     /* incompatible feature set */
    unsigned int    s_feature_ro_compat;    /* readonly-compatible feature set */
    unsigned int    s_reserved[230];        /* Padding to the end of the block */
};

struct ext2_sb_info {
    unsigned long s_frag_size;              /* Size of a fragment in bytes */
    unsigned long s_frags_per_block;        /* Number of fragments per block */
    unsigned long s_inodes_per_block;       /* Number of inodes per block */
    unsigned long s_frags_per_group;        /* Number of fragments in a group */
    unsigned long s_blocks_per_group;       /* Number of blocks in a group */
    unsigned long s_inodes_per_group;       /* Number of inodes in a group */
    unsigned long s_itb_per_group;          /* Number of inode table blocks per group */
    unsigned long s_db_per_group;           /* Number of descriptor blocks per group */
    unsigned long s_desc_per_block;         /* Number of group descriptors per block */
    unsigned long s_groups_count;           /* Number of groups in the fs */
    struct buffer_head * s_sbh;             /* Buffer containing the super block */
    struct ext2_super_block * s_es;         /* Pointer to the super block in the buffer */
    struct buffer_head ** s_group_desc;
    unsigned short s_loaded_inode_bitmaps;
    unsigned short s_loaded_block_bitmaps;
    unsigned long s_inode_bitmap_number[EXT2_MAX_GROUP_LOADED];
    struct buffer_head * s_inode_bitmap[EXT2_MAX_GROUP_LOADED];
    unsigned long s_block_bitmap_number[EXT2_MAX_GROUP_LOADED];
    struct buffer_head * s_block_bitmap[EXT2_MAX_GROUP_LOADED];
    int s_rename_lock;
    struct wait_queue * s_rename_wait;
    unsigned long  s_mount_opt;
    unsigned short s_resuid;
    unsigned short s_resgid;
    unsigned short s_mount_state;
    unsigned short s_pad;
    int s_addr_per_block_bits;
    int s_desc_per_block_bits;
    int s_inode_size;
    int s_first_ino;
};

class ext2filsys {
    friend class Ext2Container;
    friend class Ext2Allocator;

    unsigned long s_blocksize;
    unsigned char s_blocksize_bits;
    unsigned char s_lock;
    unsigned char s_rd_only;
    unsigned char s_dirt;
    struct file_system_type *s_type;
    struct super_operations *s_op;
    struct dquot_operations *dq_op;
    unsigned long s_flags;
    unsigned long s_magic;
    unsigned long s_time;
    struct inode * s_covered;
    struct inode * s_mounted;
    struct wait_queue * s_wait;
    union {
        struct ext2_sb_info ext2_sb;
    } u;
};



/* What to do with these two? */
extern ProxiableObject * Ext2ContainerConstructor( ProxiableObject * );
extern PersistentClassStar Ext2ContainerClass;


class Ext2Container : public UNIXContainer {
 protected:
    ext2filsys * _superBlock;
    
    unsigned numFragments( unsigned size );
    int mapInumber( int inum );
    int firstInumber( int inum );
    int getUnusedInode( int notUsedInThisSubClass );

    FileObject * basicOpen( int idNumber, int & status, int flags );
    FileObject * basicCreate( int idNumber, Class *, int &, int flags );
    void basicClose( FileObject * mo );
    int  basicRootId();
    void basicSynchronize();

    ~Ext2Container();
 public:
    Class * classOf();

    Ext2Container( PersistentMemoryObject * source );
    int blockPref( int inumber, int * buf, int start, int startBlock );

};

#endif /*Ext2Container_h*/
