/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * FileObject.h - 
 */

#ifndef FileObject_h
#define FileObject_h

#ifdef __GNUC__
#pragma interface
#endif

#define FLATTEN 

#include "PersistentMemoryObject.h"
#include "PersistentMemoryObjectStar.h"
#include "MemoryObjectContainerStar.h"
#include "MemoryObjectStar.h"

extern ClassStar FileObjectClass;

class FileObject : public PersistentMemoryObject {

 protected:
    PersistentMemoryObject *    _source;
    int     _offset;
    int     _unitShift;
    char  _buffered;

#ifdef FLATTEN
    int     _baseShift;
    int     _baseOffset;
    PersistentMemoryObject *    _baseSource;
#endif /* FLATTEN */

    ~FileObject();

    virtual int basicCopy( MemoryObject * destination );

    virtual int rawRead( unsigned startUnit, unsigned count,
                         char * addr, unsigned nbytes, IOAsyncClosure * c );
    virtual int rawWrite( unsigned startUnit, unsigned count,
                          char * addr, unsigned nbytes, int writeThrough,
                          IOAsyncClosure * c );

    virtual int fileObjectRead( unsigned startUnit, int count,
                                char * addr, int nbytes,
                                IOAsyncClosure * c );
    virtual int fileObjectWrite( unsigned startUnit, int count,
                                 char * addr, int nbytes,
                                 int writeThrough,
                                 IOAsyncClosure * c );

    friend class MemoryObjectCachingContainer;
    virtual int     transient();

 public:
    Class * classOf();

    proxiable FileObject( unsigned int numberOfUnits,
                          int log2UnitSize,
                          int idNumber,
                          MemoryObjectContainer * container,
                          PersistentMemoryObject * source,
                          int offset = 0 );

    proxiable void  writeIdNumber( OutputStream * );

    int     userId();
    int     groupId();

#ifdef FLATTEN
    PersistentMemoryObject * source() { return _source; }
    PersistentMemoryObject * baseSource() { return _baseSource; }
    int shift() { return _unitShift; } 
    int baseShift() { return _baseShift; }
    int offset() { return _offset; }
    int baseOffset() { return _baseOffset; }
    int buffered() { return _buffered; }
#endif /* FLATTEN */
};

#endif /*FileObject_h*/
