/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Contributing Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Contributing Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * FileSystemInterface.h - user file system inteface class
 */

#ifndef FileSystemInterface_h
#define FileSystemInterface_h


#ifdef __GNUC__
#pragma interface
#endif

#include "FileSystemInterfaceStar.h"
#include "ProxiableObject.h"
#include "MemoryObjectDictionaryStar.h"
#include "MemoryObjectContainerStar.h"
#include "PersistentMemoryObjectStar.h"
#include "PersistentObjectStar.h"
#include "MountTableStar.h"

class MountItem;
class PersistentObject;
class UNIXFileInfo;
class DictionaryStack;

const int O_RDONLY  = 0;
const int O_WRONLY  = 1;
const int O_RDRW    = 2;
const int O_NDELAY  = 4;
const int O_APPEND  = 8;

const int O_CREAT   = 0x0100;
const int O_TRUNC   = 0x0200;
const int O_EXCL    = 0x0400;

const int O_NFOLLOW = 0x1000;
const int O_NSYNC   = 0x2000;

const int O_NCACHE  = 0x80000000;

const int O_C8K     = 0x10000000;
const int O_C16K    = 0x20000000;
const int O_C32K    = 0x30000000;
const int O_C64K    = 0x40000000;
const int O_C128K   = 0x50000000;
const int O_C256K   = 0x60000000;
const int O_CSIZE   = 0x70000000;

const int O_CMASK   = 0x70000000;

enum PersistentObjectType {
    UnknownType = 0,
    PersistentArrayType = 1,
    MemoryObjectDictionaryType = 2
};

extern ClassStar FileSystemInterfaceClass;
class FileSystemInterface : public ProxiableObject {
 protected:
    MemoryObjectContainerStar   _rootContainer;
    MemoryObjectDictionaryStar  _root;
    MemoryObjectDictionaryStar  _current;
    DictionaryStack *   _dictionaries;
    MountTableStar  _mountTable;
    unsigned        _openFlagsSetMask;
    unsigned        _openFlagsClearMask;

    ~FileSystemInterface();

    MemoryObjectDictionaryRef basicFindDictionary( char * path, char * & key,
                                                   MemoryObjectDictionaryRef &,
                                                   int links = 8 );
    MemoryObjectDictionaryRef findDictionary( char * path, char * & key,
                                              MemoryObjectDictionary *,
                                              int links = 8 );
    PersistentMemoryObjectRef pathOpen( char * path, int & status,
                                        MemoryObjectDictionary *,
                                        int links = 8,
                                        int flags = 0 );
    PersistentMemoryObjectRef pathCreate( char * path, Class *,
                                          int & status,
                                          int flags = 0 );

    void substituteLink( MemoryObjectDictionary *, PersistentMemoryObjectRef &,
                         int & status, int links );
    int     changeDictionary( MemoryObjectDictionaryStar &, char * path );
    int     changeDictionary( MemoryObjectDictionaryStar &, 
                              PersistentMemoryObject *pmo );

 public:
    Class * classOf();

    FileSystemInterface( MemoryObjectDictionary *,
                         MemoryObjectContainer * );
    FileSystemInterface( FileSystemInterface *, int copyMountTable );

    proxiable virtual ProxiableObjectRef open( char * path, 
                                               Class * interfaceClass,
                                               int & status, int flags = 0,
                                               int mode = 0, 
                                               Class * baseClass = 0 );
    proxiable virtual PersistentObjectRef open( char * path, int & status,
                                                int & returnType,
                                                int flags = 0, int mode = 0 );

    proxiable virtual int add( char * path, PersistentObject * );

    proxiable virtual int info( char * dirname, UNIXFileInfo *, int follow,
                                unsigned * hints = 0 );

    proxiable virtual int link( char * inpath, char * outpath,int overwrite=0 );
    proxiable virtual int unlink( char * pathname );

    proxiable virtual int chdir( char * dirname );
    proxiable virtual int chroot( char * dirname );
    proxiable virtual int chdir( PersistentMemoryObject * pmo );
    proxiable virtual int chroot( PersistentMemoryObject * pmo );

    proxiable virtual int mount( char * special, char * pathname, int rwflag );
    proxiable virtual int unmount( char * pathname );

    proxiable virtual int synchronize();

    proxiable virtual void substituteMount( PersistentMemoryObjectRef & pmo );
    proxiable virtual void substituteMountUp( MemoryObjectDictionaryRef & moc, 
                                              char * key );

    proxiable virtual void openFlagsSetMask( unsigned mask );
    proxiable virtual void openFlagsClearMask( unsigned mask );
    proxiable virtual unsigned openFlagsSetMask();
    proxiable virtual unsigned openFlagsClearMask();

};

#endif /*FileSystemInterface_h*/
