/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David K. Raila (raila@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
#ifndef FileSystemSimpleInstruments_h
#define FileSystemSimpleInstruments_h

#ifdef __GNUC__
#pragma interface
#endif

#include "SimpleInstruments.h"

	/*
	 **************************************************************
	 * Simple instruments for the FileSystem subsystem
	 * There is one class each for when SIMPLE_INSTRUMENTATION is defined
	 * and not.  Be sure to add/delete both!
	 **************************************************************
	 */
#ifdef SIMPLE_INSTRUMENTATION

extern ClassStar PersistentMemoryObjectTimerClass;
extern ClassStar MemoryObjectDictionaryTimerClass;
extern ClassStar FileSystemInterfaceTimerClass;
extern ClassStar MountTableTimerClass;
extern ClassStar MemoryObjectContainerTimerClass;

class PersistentMemoryObjectTimer : public FileSystemTimer
{
public:
	PersistentMemoryObjectTimer(const char * const name) : FileSystemTimer(name) {}
	Class * classOf();
};
class MemoryObjectDictionaryTimer : public FileSystemTimer
{
public:
	MemoryObjectDictionaryTimer(const char * const name) : FileSystemTimer(name) {}
	Class * classOf();
};
class FileSystemInterfaceTimer : public FileSystemTimer
{
public:
	FileSystemInterfaceTimer(const char * const name) : FileSystemTimer(name) {}
	Class * classOf();
};
class MountTableTimer : public FileSystemTimer
{
public:
        MountTableTimer(const char * const name) : FileSystemTimer(name) {}
        Class * classOf();
};
class MemoryObjectContainerTimer : public FileSystemTimer
{
public:
        MemoryObjectContainerTimer(const char * const name) : FileSystemTimer(name) {}
        Class * classOf();
};
#else /* NOT SIMPLE_INSTRUMENTATION*/

class PersistentMemoryObjectTimer : public FileSystemTimer
{
public: 
	PersistentMemoryObjectTimer(const char *const,SimpleInstrumentState = ACTIVE){};
};
class MemoryObjectDictionaryTimer : public FileSystemTimer 
{
public: 
	MemoryObjectDictionaryTimer(const char *const,SimpleInstrumentState = ACTIVE){};
};
class FileSystemInterfaceTimer : public FileSystemTimer 
{
public: 
	FileSystemInterfaceTimer(const char *const,SimpleInstrumentState = ACTIVE){};
};
class MountTableTimer : public FileSystemTimer 
{
public: 
	MountTableTimer(const char *const,SimpleInstrumentState = ACTIVE){};
};
class MemoryObjectContainerTimer : public FileSystemTimer
{
public:
        MemoryObjectContainerTimer(const char *const,SimpleInstrumentState = ACTIVE){};
};

#endif /*SIMPLE_INSTRUMENTATION*/

#endif /* FileSystemSimpleInstruments_h*/
