/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * MadContainer.h - mad file system abstract Class
 */

#ifndef MadContainer_h
#define MadContainer_h


#ifdef __GNUC__
#pragma interface
#endif

#include "MemoryObjectContainer.h"
#include "MadFileSystem.h"
class MadNode;

const int FmMAGIC = 0xf106beef;
const int FmMAGICOFFSET = 8192+sizeof(MadSuperBlock)-sizeof(int);

extern ProxiableObject * MadContainerConstructor( ProxiableObject * );
extern PersistentClassStar MadContainerClass;
class MadContainer : public MemoryObjectContainer {
protected:
	MadNode **	_nodes;
	int		_classes;
	ClassStar *	_classList;
	char **		_classNameList;
	char *		_classNames;
	MadSuperBlock * _superBlock;
	unsigned char *	_nodeMap;
	unsigned char *	_blockMap;
	int		_log2BlockToSector;
	int		_blockSize;

	int	blockNumberOfBlock( int );
	int	mapNodeNumber( int );
	int	firstNodeNumber( int );

	void	readNode( int fileNodeNumber );
	void	writeNode( int fileNodeNumber );
	int	getUnusedNode( int nearFileNodeNumber );

	FileObject * basicOpen( int idNumber, int & status, int flags );
	FileObject * basicCreate( int idNumber, Class *, int &, int flags );
	void	basicClose( FileObject * mo );
	void	basicSynchronize();

	~MadContainer();
public:
	Class * classOf();

	MadContainer( PersistentMemoryObject * source );
};

#endif /*MadContainer_h*/
