/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * MemoryObjectContainer.h - inode system superClass
 */

#ifndef MemoryObjectCachingContainer_h
#define MemoryObjectCachingContainer_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Lock.h"
#include "MemoryObjectContainer.h"

#define SAFESYNC

extern PersistentClassStar MemoryObjectCachingContainerClass;

class ZombieInfo;

class MemoryObjectCachingContainer : public MemoryObjectContainer {
/*
 * A MemoryObjectCachingContainer is like MemoryObjectContainer, except
 * it delays actual closing to keep the FileObjects and their cache
 * alive for awhile.
 */
public:
    Class * classOf();

    MemoryObjectCachingContainer( PersistentMemoryObject * source );

    FileObjectRef open( int id, int & status, int flags = 0 );
    void close( FileObject * fo );
    void synchronize();
    int  unmount();

    void killZombie( int id );

    static int shutdown();

protected:

    static int blocked;
    static Lock slock;

    static ZombieInfoList * SyncZombieList;
    static unsigned MaxSyncZombieCount;

    static ZombieInfoList * NotSyncZombieList;
    static unsigned MaxNotSyncZombieCount;

    static ZombieInfoList * ExternZombieList;

    static void shutdown( ZombieInfoList * );

    static void daemon();
    static void spawnHelpDaemon();

    friend class SpawnHelpDaemonClosure;

    static Lock ActiveHelpLock;
    static unsigned ActiveHelpDaemonCount;
    static unsigned MaxActiveHelpDaemon;

    static void helpDaemon();

    friend class Store;

    static int killZombies();
};

#endif /*MemoryObjectCachingContainer_h*/


