/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * MountItem.h - part of a mount table
 */

#ifndef MountItem_h
#define MountItem_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Object.h"
#include "PersistentMemoryObjectStar.h"

class MountItem : public Object {
protected:
	MountItem *		_next;
	MountItem *		_previous;
	PersistentMemoryObjectStar _mountPoint;
	PersistentMemoryObjectStar _mounted;

	void			setNext( MountItem * );
	void			setPrevious( MountItem * );
public:
	MountItem( MountItem *, PersistentMemoryObjectRef & mountPoint,
			        PersistentMemoryObjectRef & mounted );
	~MountItem();

	MountItem *		lookupMounted( PersistentMemoryObject * );
	MountItem *		lookupMountPoint( PersistentMemoryObject * );
	PersistentMemoryObject * mountPoint();
	PersistentMemoryObject * mounted();
	MountItem *		next();
};

inline void
MountItem::setPrevious( MountItem * p )
{
	_previous = p;
}

inline void
MountItem::setNext( MountItem * n )
{
	_next = n;
}

inline MountItem *
MountItem::next()
{
	return( _next );
}

inline PersistentMemoryObject *
MountItem::mountPoint()
{
	return( _mountPoint );
}

inline PersistentMemoryObject *
MountItem::mounted()
{
	return( _mounted );
}

#endif /*MountItem_h*/
