/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Contributing Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * NFSDictionary.h - abstract superclass for directories
 */

#ifndef NFSDictionary_h
#define NFSDictionary_h


#ifdef __GNUC__
#pragma interface
#endif

#include "PersistentMemoryObjectStar.h"
#include "MemoryObjectContainerStar.h"
#include "MemoryObjectDictionary.h"

extern PersistentClassStar NFSDictionaryClass;
extern ProxiableObject * NFSDictionaryConstructor(ProxiableObject *);
class NFSDictionary : public MemoryObjectDictionary {
protected:
    int			_socket;
    int *		_fileHandle;

    /* Method NFSDictionary: destructor. */
    ~NFSDictionary();

public:
    /* Method classOf: returns the class of this object. */
    Class * classOf();

    /* Method NFSDictionary: contructor. */
    NFSDictionary( PersistentMemoryObject * source );

    /* Method open: returns the MemoryObject corresponding to the given key,
     *              if the key is in the dictionary. */
    virtual PersistentMemoryObjectRef open( char * key, int & status );

    /* Method create: returns the MemoryObject corresponding to the given key
     *                if the key is in the dictionary.
     *                Otherwise a new MemoryObject is created
     *                and the key is stored in the dictionary. */
    virtual PersistentMemoryObjectRef create( char * key, Class *, int & sts );

    /* Method add: inserts the given key into the dictionary only if it is not
     *             already in the dictionary. */
    virtual int add( char * key, PersistentMemoryObject * );

    /* Method remove: deletes the given key from the dictionary. */
    virtual int remove( char * key, int * links = 0 );

    /* Method keys: returns the list of keys in the dictionary.
     *              Also returns the number of keys. */
    virtual int keys( char * buffer, int bufferSize, int & count );

    /* Method keySize: returns the sum of the length of the keys
     *                 in the dictionary.  */
    virtual int keySize();
};

#define NFSopenOp		'o'
#define NFScreateOp		'c'
#define NFSaddOp		'a'
#define NFSremoveOp		'r'
#define NFSkeysOp		'k'

#endif NFSDictionary_h
