/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * NFSDisk.h: NFSDisk class descriptions.
 */

#ifndef NFSDisk_h
#define NFSDisk_h


#ifdef __GNUC__
#pragma interface
#endif

#include "PersistentMemoryObject.h"
#include "PersistentMemoryObjectStar.h"

extern ClassStar NFSDiskClass;
class NFSDisk : public PersistentMemoryObject {
/*
 */
protected:
	int		_socket;
	int *		_fileHandle;

	~NFSDisk();

	int	accessMode();
public:
	Class * classOf();

	NFSDisk( unsigned int numberOfUnits, int log2UnitSize,
		 int socket, int * fileHandle );

	int socket();
	int * fileHandle();

	int	rawRead( unsigned int startUnit, int count, char * dest, 
			 int n );
	int	rawWrite( unsigned int startUnit, int count, char * src, 
			  int n );

	Class *	supports( Class * aClass );
	int	numberOfBytes();
	int	setNumberOfBytes( int numberOfBytes );
	int	setNumberOfUnits( unsigned int numberOfUnits );
	void	info( UNIXFileInfo * );
	int	isSymbolicLink();
	int	userId();
	int	groupId();
	int	setUserId( int uid );
	int	setGroupId( int gid );
	int	setMode( int mode );
	int	links( int delta = 0 );

	void	writeIdNumber( OutputStream * );
};

#define NFSmountOp		'm'
#define NFSunmountOp		'u'

#define NFSreadOp		'R'
#define NFSwriteOp		'W'
#define NFSinfoOp		'S'
#define NFSsetNumberOfBytesOp	'B'
#define NFSsetGroupIdOp		'G'
#define NFSsetModeOp		'M'
#define NFSsetNumberOfUnitsOp	'N'
#define NFSsetUserIdOp		'I'

#endif NFSDisk_h
