/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * POSHExpression.h - User level file class
 */

#ifndef	POSHExpression_h
#define	POSHExpression_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Assert.h"
#include "Raid.h"
#include "PhysicalMemoryChain.h"
#include "Process.h"
#include "Domain.h"
#include "PersistentMemoryObject.h"
#include "MemoryObjectDictionary.h"
#include "FileSystemInterface.h"
#include "PersistentClass.h"
#include "AutoloadPersistentObject.h"
#include "POSHExpressionStar.h"
#include "Convert.h"
#include "String.h"

enum POSHErrorCodes {
	Success = 0,
	ApplyNotInitialized,
	ApplyArgumentEvaluationFailed,
	IfNotInitialized,
	NewObjectFailed,
	ObjectOpenFailed,
	SubclassResponsibility,
	TooFewArguments,
	TooManyArguments,
	UnknownBuiltinFunction0,
	UnknownBuiltinFunction1,
	UnknownBuiltinFunction2,
	UnknownBuiltinFunction
};

extern PersistentClassStar POSHExpressionClass;
class POSHExpression : public AutoloadPersistentObject {
protected:
	~POSHExpression();

	POSHExpressionRef convertArg( int argn, char * argv[], int & status,
				      FileSystemInterface * fsi );

public:
	Class * classOf();

	POSHExpression( PersistentMemoryObject * mo );

	int init( int argc, char *argv[], FileSystemInterface * );
	int execute();

	void flush();

	virtual int		  asInt();
	virtual int		  isTrue();
	virtual POSHExpressionRef eval( FileSystemInterface *, int & status );
	virtual void		  printOn( OutputStream * );
};

#endif POSHExpression_h
