/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *         University of Illinois at Urbana-Champaign
 *         Department of Computer Science
 *         1304 W. Springfield Ave.
 *         Urbana, IL    61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *    All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Author: Lee Lup Yuen (lup@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * PersistentArray.h - User level file class
 */

#ifndef PersistentArray_h
#define PersistentArray_h


#ifdef __GNUC__
#pragma interface
#endif

#include "MemoryObjectViewStar.h"
#include "PersistentObject.h"
#include "PersistentArrayStar.h"

class FileSystemInterface;

extern ProxiableObject * PersistentArrayConstructor( ProxiableObject * );

extern PersistentClassStar PersistentArrayClass;

class PersistentArray : public PersistentObject {

protected:
	~PersistentArray();
	void noRemainingReferences();
	char *_address;
	unsigned _log2UnitSize;
	unsigned _cacheLog2UnitSize;
	unsigned _unitSize;
	unsigned _unitSizeMask;
	unsigned _log2ViewSize;
	char * _buffer;
	MemoryObjectViewStar _view;

	void reMap();
	int useMap(unsigned length);
	
public:
	Class * classOf();
	
	proxiable PersistentArray( PersistentMemoryObject * mo );
	
	int init( int argc, char *argv[], FileSystemInterface * = 0 );
	
	proxiable virtual int at( unsigned int index );
	proxiable virtual int at( unsigned int index, char * values, 
				  unsigned int count ); 
	proxiable virtual int at( char * index, int & status );

	proxiable virtual int atPut( unsigned int index, int value );
	proxiable virtual int atPut( unsigned int index, char * values, 
				     unsigned int count ); 
	proxiable virtual int atPut( char * index, int value, int & status );

	proxiable virtual unsigned size();

	virtual 	  int notify( int );
};

#endif /*PersistentArray_h*/
