/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Contributing Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * PersistentClass.h -
 */
#ifndef PersistentClass_h
#define	PersistentClass_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Class.h"
#include "MemoryObjectDictionaryStar.h"
#include "MemoryObjectContainerStar.h"
#include "PersistentObjectStar.h"

extern ClassStar PersistentClassClass;
class PersistentClass : public Class {
/*
 *  A PersistentClass is the run-time representation of a C++ class.
 */

protected:
	MemoryObjectDictionaryStar	_dictionary;
	MemoryObjectContainerStar	_container;

	~PersistentClass();
public:
	Class * classOf();
	proxiable PersistentClass( Class * parent, int size, const char * name, ClassConstructor * ctor=0 );
	//PersistentClass( Class * parent, int size, const char * name, POFPO=0 );

	static void		initialize();
	void			cleanup();

	proxiable virtual MemoryObjectDictionaryRef rootDictionary();
	proxiable virtual PersistentObjectRef	newObject();
	proxiable virtual PersistentObjectRef	retrieveObject( int id );
};

#endif	/*PersistentClass_h*/
