/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * PersistentObject.h - User level file class
 */

#ifndef	PersistentObject_h
#define	PersistentObject_h


#ifdef __GNUC__
#pragma interface
#endif

#include "ProxiableObject.h"
#include "PersistentClassStar.h"

class UNIXFileInfo;
class FileSystemInterface;
class PersistentObjectStar;

extern PersistentClassStar PersistentObjectClass;

#include "PersistentMemoryObjectStar.h"

const int PersistentObjectTableSize = 2311;

class PersistentObject : public ProxiableObject {
/*
 * The PersistentObject class defines objects that are based on and tracked
 * by an underlying PersistentMemoryObject.  It can be instantiated,
 * but it is primarily used as a superclass for MemoryObjectDictionary,
 * MemoryObjectContainer, etc.
 */
    friend                      class PersistentMemoryObject;
    friend                      class NFSDisk;
    friend                      class FileObject;

protected:
        short           _modified;

        /* The underlying PersistentMemoryObject */
        PersistentMemoryObject *     _source;
        char *          _arrayBuffer;

        /* Method ~PersistentObject: destructor */
        ~PersistentObject();

        /* Method noRemainingReferences: actions taken when no longer used */
        void            noRemainingReferences();

public:
        /* Method classOf: returns the Class of this object */
        Class * classOf();

	/* Method source: returns _source */
	proxiable PersistentMemoryObject * source();

        /* Method PersistentObject: constructor */
        proxiable PersistentObject( PersistentMemoryObject * mo );

        proxiable virtual int  init( int argc, char *argv[], FileSystemInterface * = 0 );
        proxiable virtual int  execute();

        proxiable virtual int  persist();
        proxiable virtual int  desist();

        /* Method numberOfBytes: return the size of the underlying
         *                       PersistentMemoryObject in bytes.
         */
        proxiable virtual unsigned   numberOfBytes();
        proxiable virtual unsigned   numberOfRecords();
        proxiable virtual unsigned   recordSize();

        proxiable virtual int  copy( PersistentObject * po );

        proxiable virtual void flush();

    /* Method writeName: write the name of the object to the OutputStream */
    proxiable virtual void                writeName( OutputStream * );

    /* Method executable: return true iff this object is a runable program */
    proxiable virtual int                 executable();

    /* Method idNumber: return the container index
     *                  for the underlying PersistentMemoryObject
     */
    proxiable virtual int                 idNumber();

    /* Method asA: return an object of the given Class based on the same
     *             PersistentMemoryObject as this object.  Return 0 if the
     *             PersistentMemoryObject does not support the given Class.
     */
    proxiable virtual ProxiableObjectRef  asA( Class *, int & status );


    /* Method links: return the number of links (as in UNIX hard links)
     *               to the underlying PersistentMemoryObject.
     */
    proxiable virtual int         links();

    /* Method info: return lots of information about the underlying
     *              PersistentMemoryObject. (Similar to UNIX fstat().)
     */
    proxiable virtual void        info( UNIXFileInfo * );

    /* Method setUserId: set the UID of the underlying PersistentMemoryObject.
     *                   (Similar to UNIX fchown().)
     */
    proxiable virtual int         setUserId( int uid );

    /* Method setGrouptId: set the GID of the underlying PersistentMemoryObject.
     *                     (Similar to UNIX fchown().)
     */
    proxiable virtual int         setGroupId( int gid );

    /* Method setMode: set the mode bits (read, write, and execute
     *                 for user, group, and others) of the underlying
     *                 PersistentMemoryObject. (Similar to UNIX fchmod().)
     */
    proxiable virtual int         setMode( int mode );

    /* Method unmount: prepare for unmounting 
     */
    virtual int			  unmount();
    
    /* Method for notifying certain events
     */
    virtual int			  notify( int );
};

inline PersistentMemoryObject * 
PersistentObject::source()
{
	return( _source ); 
}

inline int
digits( int i )
{
        int d;

        if( i >= 100000 ) {
                if     ( i >= 1000000000 ) d = 10;
                else if( i >= 100000000  ) d =  9;
                else if( i >= 10000000   ) d =  8;
                else if( i >= 1000000    ) d =  7;
                else                       d =  6;
        }
        else {
                if     ( i >= 10000 ) d = 5;
                else if( i >= 1000  ) d = 4;
                else if( i >= 100   ) d = 3;
                else if( i >= 10    ) d = 2;
                else                  d = 1;
        }
        return( d );
}


inline void
itoa( int i, char * a )
{
        *a-- = '\0';
        if( i == 0 ) *a-- = '0';
        else while( i ) {
                *a-- = "0123456789"[i % 10];
                i /= 10;
        }
}

#endif /*PersistentObject_h*/
