/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * RecordStream.h 
 */

#ifndef	RecordStream_h
#define	RecordStream_h


#ifdef __GNUC__
#pragma interface
#endif

#include "ProxiableObject.h"
#include "RecordStreamStar.h"
#include "PersistentObjectStar.h"

class UNIXFileInfo;
class PersistentObject;

extern ClassStar RecordStreamClass;
class RecordStream : public ProxiableObject {
/*
 * RecordStream defines sequential access to various kinds
 * of record-oriented files.
 */

protected:
	int			_recordNumber;
	PersistentObjectStar	_file;

	/* Method ~RecordStream: destructor */
	~RecordStream();

public:
	/* Method classOf: returns the Class of this object */
	Class * classOf();

	/* Method RecordStream: constructor */
	proxiable RecordStream( PersistentObject * po );
	
	/* Method read: read multiple records, return number read */
	proxiable virtual int read( char * buffer, int size );

	/* Method read: read the next record */
	proxiable virtual int read( char * buffer, int size, int & status ) = 0;

	/* Method write: write multiple records, return number written */
	proxiable virtual int write( char * buffer, int size );

	/* Method write: write record of size bytes if at end of file */
	proxiable virtual int write( char * buffer, int size, int & status ) =0;

	proxiable virtual int recordNumber();
	proxiable virtual int setRecordNumber( int delta, int mode );

	proxiable virtual int numberOfBytes();
	proxiable virtual int numberOfRecords();
	proxiable virtual int recordSize();
	proxiable virtual int minimumRecordSize();
	proxiable virtual int maximumRecordSize();

	proxiable virtual int  setMode( int mode );
	proxiable virtual int  setUserId( int uid );
	proxiable virtual int  setGroupId( int gid );
	proxiable virtual void info( UNIXFileInfo * );
	proxiable virtual int  init( int argc, char *argv[] );
	proxiable virtual int  executable();
	proxiable virtual ProxiableObjectRef  asA( Class *, int & status );

	/* Method runFile: execute this file with the given arguments */
	proxiable virtual int   runFile( int argc, char ** argv );

};

#endif /*RecordStream_h*/
