/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * RemoteFileObject.h 
 */

#ifndef RemoteFileObject_h
#define RemoteFileObject_h


#ifdef __GNUC__
#pragma interface
#endif

#include "FileObject.h"
#include "PersistentMemoryObjectStar.h"
#include "RemoteFileSystem.h"

extern ClassStar RemoteFileObjectClass;

class RemoteFileObject : public FileObject {
	
protected:
	RFSHandle _handle;
	RFSAttributes _attributes;
	int _unusedUnit;
	int _modified;

protected:

	friend class RemoteContainer;

	int accessMode();
	
	Class * supports( Class * aClass );

	MemoryObjectCache * buildCache();

	void basicSynchronizeClosure( BasicSynchronizeAsyncClosure * c );
	void basicSynchronize( IOAsyncClosure * c );
	void basicSynchronize( unsigned from, unsigned to,
			       IOAsyncClosure * c );
	unsigned basicSetNumberOfBytes( unsigned size );
	unsigned basicSetNumberOfUnits( unsigned numberOfUnits );
	int basicCopy( MemoryObject * dest );
	int rawRead( unsigned startUnit, int count, char * dest, int nbytes,
		     IOAsyncClosure * c );
	int rawWrite( unsigned startUnit, int count, char * src, int nbytes, 
		      int writeThrough, IOAsyncClosure * c );
	int parseFlags( int flags );
	void hintsChanged( unsigned oldHints );
	void setModified();
	void clearModified();

public:	
	Class * classOf();

	RemoteFileObject( MemoryObjectContainer * container, 
			  RFSHandle & handle, RFSAttributes & attributes,
		          int create );
	~RemoteFileObject();

	int isSymbolicLink();
	int isContainer();
	int isDictionary();
	int fixedSize();
	int setUserId( int uid );
	int setGroupId( int gid );
	int setMode( int accessMode );
	int userId();
	int groupId();
	int transient();
	int modified();
	int links( int delta );
	unsigned numberOfBytes();
	void writeIdNumber( OutputStream * os );
	void info( UNIXFileInfo * infoBuffer );
	int isRemote();

	void setName( const char * str ); // for debugging

	void syncAttributes( int commit, int close, IOAsyncClosure * c );
	void getAttributes( RFSAttributes * );
	void setAttributes( RFSAttributes * );

	void setLinks( int links ) 
		{ _attributes.links = links; }
	RFSHandle * handle() 
		{ return & _handle; }
	RFSAttributes * attributes()
		{ return & _attributes; }
	int referenceCount() 
		{ return PersistentMemoryObject::referenceCount(); }
	ZombieInfo * zombieInfo() 
		{ return PersistentMemoryObject::zombieInfo(); }
};

#endif	RemoteFileObject_h
