/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * RemoteFileSystem.h 
 */

#ifndef RemoteFileSystem_h
#define RemoteFileSystem_h

#ifdef __GNUC__
#pragma interface
#endif

#define RFS_UPDATE_ATTRIBUTES

extern unsigned RemoteFileSystemMaxDataSize;

struct RFSContainerHandle {
	unsigned _host;
	unsigned _containerId;
	unsigned _size;
	unsigned _rootId;
	unsigned hash() 
		{ return _host + _containerId; }
	int operator== ( RFSContainerHandle & handle )
		{ return ( _host == handle._host && 
			   _containerId == handle._containerId ); }
	int operator!= ( RFSContainerHandle & handle )
		{ return !( *this == handle ); }
};

struct RFSHandle {
	RFSContainerHandle _containerHandle;
	unsigned _idNumber;
	unsigned _version;

	unsigned hash()
		{ return _containerHandle.hash() + _idNumber + _version; }
	int operator== ( RFSHandle & handle )
		{ return ( _containerHandle == handle._containerHandle &&
			   _idNumber == handle._idNumber &&
			   _version == handle._version ); }
	int operator!= ( RFSHandle & handle ) 
		{ return !(*this == handle); }
};

enum RFSObjectTypeEnum {
	Unknown,
	NormalFile,
	Dictionary,
	SymbolicLink,
	Container
};

class Class;
class PersistentMemoryObject;

class RFSObjectType {
	RFSObjectTypeEnum _type;	
public:
	RFSObjectType() : _type( Unknown ) {}
	RFSObjectType( RFSObjectTypeEnum value ) : _type( value ) {}
	RFSObjectType( Class * aClass );
	RFSObjectTypeEnum type() { return _type; }
#ifdef RemoteFileSystem
	void set( PersistentMemoryObject * );
#else
	void set( PersistentMemoryObject * ){}
#endif
	Class * objectClass();
};

class RFSAttributes {
    public:
	short mode;
	short links;
	short uid;
	short gid;
	int size;
	int atime;
	int mtime;
	int ctime;
	char log2BlockSize;
	char log2UnitSize;
	RFSObjectType type;
	unsigned short cacheHints;

	unsigned short flags;
	static const unsigned long AttrDirty = 0x1;
	static const unsigned long FixedSize = 0x2;
};

class PersistentMemoryObjectRef;

PersistentMemoryObjectRef
RemoteFileSystemMount( char * special, char * mountPoint, int & status,
		       int flags = 0 );

#endif /*RemoteFileSystem_h*/
