/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * RemoteFileSystemMessage.h 
 */

#ifndef RemoteFileSystemMessage_h
#define RemoteFileSystemMessage_h

#ifdef __GNUC__
#pragma interface
#endif

#include "RemoteFileSystem.h"

enum RFSMessageType {
	RFSMount = 0,
	RFSContainerOpen,
	RFSContainerCreate,
	RFSOpen,
	RFSCreate,
	RFSKeys,
	RFSRemove,
	RFSAdd,
	RFSRead,
	RFSWrite,
	RFSSync,
	RFSAssociations,
	RFSMountReply,
	RFSContainerOpenReply,
	RFSContainerCreateReply,
	RFSOpenReply,
	RFSCreateReply,
	RFSKeysReply,
	RFSRemoveReply,
	RFSAddReply,
	RFSReadReply,
	RFSWriteReply,
	RFSSyncReply,
	RFSAssociationsReply,
	RFSLast
};

struct RFSMessage {
	unsigned _checkSum;
	unsigned _sequenceNumber;
	unsigned _headerSize;
	unsigned _dataSize;
  	RFSMessageType _messageType;
	unsigned checkSum() 
		{ return _sequenceNumber + _headerSize + _dataSize; }
};

struct RFSMountMessage : public RFSMessage {
};

struct RFSRequestMessage : public RFSMessage {
	RFSHandle _handle;
};

struct RFSContainerOpenMessage : public RFSRequestMessage {
	int _idNumber;
	int _flags;
};

struct RFSContainerCreateMessage : public RFSRequestMessage {
	int _hint;
	RFSObjectType _objectType;
	int _flags;
};

struct RFSOpenMessage : public RFSRequestMessage {
	int _flags;
};

struct RFSCreateMessage : public RFSRequestMessage {
	RFSObjectType _objectType;
	int _flags;
};

struct RFSKeysMessage : public RFSRequestMessage {
	int _bufferSize;
};

struct RFSAssociationsMessage : public RFSRequestMessage {
	int _bufferSize;
	int _startOffset;
};

struct RFSRemoveMessage : public RFSRequestMessage {
};

struct RFSAddMessage : public RFSRequestMessage {
	int _idNumber;
};

struct RFSSyncMessage : public RFSRequestMessage {
	RFSAttributes _attributes;
	char _commit;
	char _close;
};

struct RFSReadMessage : public RFSRequestMessage {
	int _startUnit;
	int _noOfUnits;
	int _noOfBytes;
#ifdef RFS_UPDATE_ATTRIBUTES
	RFSAttributes _attributes;
#endif
};

struct RFSWriteMessage : public RFSRequestMessage {
	int _startUnit;
	int _noOfUnits;
	int _writeThrough;
#ifdef RFS_UPDATE_ATTRIBUTES
	RFSAttributes _attributes;
#endif
};

struct RFSReplyMessage : public RFSMessage {
	int _status;
};

struct RFSMountReplyMessage : public RFSReplyMessage {
	RFSHandle _handle;
	RFSAttributes _attributes;
};

struct RFSContainerOpenReplyMessage : public RFSReplyMessage {
	RFSHandle _handle;
	RFSAttributes _attributes;
};

struct RFSContainerCreateReplyMessage : public RFSReplyMessage {
	RFSHandle _handle;
	RFSAttributes _attributes;
};

struct RFSOpenReplyMessage : public RFSReplyMessage {
	RFSHandle _handle;
	RFSAttributes _attributes;
};

struct RFSCreateReplyMessage : public RFSReplyMessage {
	RFSHandle _handle;
	RFSAttributes _attributes;
};

struct RFSKeysReplyMessage : public RFSReplyMessage {
	int _count;
};

struct RFSAssociationsReplyMessage : public RFSReplyMessage {
	int _count;
	int _resultOffset;
	unsigned _dummy[2];	// padding to avoid net error
};

struct RFSRemoveReplyMessage : public RFSReplyMessage {
	int _links;
};

struct RFSAddReplyMessage : public RFSReplyMessage {
	int _links;
};

struct RFSSyncReplyMessage : public RFSReplyMessage {
};

struct RFSReadReplyMessage : public RFSReplyMessage {
	int _actualNoOfUnits;
#ifdef RFS_UPDATE_ATTRIBUTES
	RFSAttributes _attributes;
#endif
};

struct RFSWriteReplyMessage : public RFSReplyMessage {
	int _actualNoOfUnits;
#ifdef RFS_UPDATE_ATTRIBUTES
	RFSAttributes _attributes;
#endif
};

class OutputStream;

OutputStream & operator<< ( OutputStream & s, RFSMessage & msg );

#endif RemoteFileSystemMessage_h
