/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * RemoteFileSystemRPC.h
 */

#ifndef RemoteFileSystemRPC_h
#define RemoteFileSystemRPC_h


#ifdef __GNUC__
#pragma interface
#endif

#include "RemoteFileSystem.h"

class IOAsyncClosure;

const unsigned short RemoteFileSystemClientPort = 1204;
const unsigned short RemoteFileSystemServerPort = 1966;
const unsigned RemoteFileSystemMaxOutstandingRPC = 128;

class RemoteFileObject;

class RemoteFileSystemRPC {
protected:
	static int _debugMask;
	static int _debugMessages;

public:
	static int clientInit();

	static int mount( unsigned server, char * path,	
			  RFSHandle * handle, RFSAttributes * attributes );
	static int containerOpen( RFSContainerHandle & handle, int idNumber,
			 	  int flags, RFSHandle * fileHandle, 
			 	  RFSAttributes * attributes );
	static int containerCreate( RFSContainerHandle & handle, int hint, 
			   	    RFSObjectType type,
			   	    int flags, RFSHandle * fileHandle, 
			   	    RFSAttributes * attributes );
	static int open( RFSHandle & handle, char * key, int flags, 
			 RFSHandle * fileHandle, RFSAttributes * attributes );
	static int create( RFSHandle & handle, RFSObjectType type,
			   char * key, int flags, 
			   RFSHandle * fileHandle, RFSAttributes * attributes );
	static int keys( RFSHandle & handle,
			 char * buffer, int bufferSize,
			 int * count );
	static int associations( RFSHandle & handle,
				 char * buffer, int bufferSize,
				 int startOffset,
				 int * count, int * resultOffset );
	static int remove( RFSHandle & handle, char * key, int * links );
	static int add( RFSHandle & handle, char * key, int idNumber, 
			int * links );
	static int read( RemoteFileObject * rfo, RFSHandle & handle,
			 int startUnit, int noOfUnits, 
			 char * buffer, int noOfBytes, 
			 int * actualNoOfUnits, IOAsyncClosure * c );
	static int write( RemoteFileObject * rfo, RFSHandle & handle,
			  int startUnit, int noOfUnits, 
			  char * buffer, int noOfBytes, int writeThrough,
			  int * actualNoOfUnits, IOAsyncClosure * c );
	static int sync( RFSHandle & handle, 
			 RFSAttributes & attributes, int commit, int close,
			 IOAsyncClosure * );
};

extern int buildSocket( unsigned long host, int port );

#endif RemoteFileSystemRPC_h


