/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * RemoteServer.h 
 */

#ifndef RemoteServer_h
#define RemoteServer_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Semaphore.h"
#include "FileSystemInterface.h"
#include "FileSystemInterfaceStar.h"

class FileObjectRef;
class MemoryObjectContainerRef;
class Process;
class RFSContanerHandle;
class RFSHandle;
class RFSMessage;

extern ClassStar RemoteServerClass;

class RemoteServer {
	
protected:
	typedef void (*DispatchFunc)( RFSMessage * );
	static int _noOfProcess;
	static Semaphore _mutex;
	static Process ** _processes;
	static int _socket;
	static FileSystemInterfaceStar _fsiStar;
	static FileSystemInterface * _fsi;
	static DispatchFunc _dispatchTable[32];

	static void handle( PersistentMemoryObject * pmo, RFSHandle & );
	static MemoryObjectContainerRef container( RFSContainerHandle & handle,
						   int & status );
	static FileObjectRef fileObject( RFSHandle & handle, int & status );

	static void daemon( int i );

	static void mount( RFSMessage * rfshdr );
	static void containerOpen( RFSMessage * rfshdr );
	static void containerCreate( RFSMessage * rfshdr );
	static void open( RFSMessage * rfshdr );
	static void create( RFSMessage * rfshdr );
	static void keys( RFSMessage * rfshdr );
	static void remove( RFSMessage * rfshdr );
	static void add( RFSMessage * rfshdr );
	static void read( RFSMessage * rfshdr );
	static void write( RFSMessage * rfshdr );
	static void sync( RFSMessage * rfshdr );
	static void associations( RFSMessage * rfshdr );
	static void invalid( RFSMessage * rfshdr );
		    
public:	
	static int _debugMask;
	static int _debugMessages;
	static void serverInit( int noOfProcess );
};

#endif	RemoteServer_h
