/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * RemoteTables.h - unix inode system abstract Class
 */

#ifndef RemoteTables_h
#define RemoteTables_h

#ifdef __GNUC__
#pragma interface
#endif

#include "Lock.h"

class RemoteContainer;
class RFSHandle;

class RemoteContainerRegistryInfo {
	friend class RemoteContainerRegistry;

	RemoteContainer * _container;
	RemoteContainerRegistryInfo * _next;
};

class RemoteContainerRegistry {
protected:
	unsigned _size;
	Lock _lock;
	RemoteContainerRegistryInfo ** _info;
public:
	RemoteContainerRegistry( unsigned size = 67 );
	void unbind( RemoteContainer * remoteContainer );
	RemoteContainer * lookup( RFSContainerHandle & handle );
};

extern RemoteContainerRegistry * TheRemoteContainerRegistry;

class RemoteDictionary;

class RemotePathCacheInfo {
	friend class RemotePathCache;
	RemoteDictionary * _dictionary;
	unsigned _age;
	RFSHandle _handle;
	RemotePathCacheInfo * _next;
	char * key() { return (char *)( this + 1 ); }
};

class RemotePathCache {
	const unsigned _defaultTimeout = 30; /* seconds */
	const unsigned _defaultHashTableSize = 1223;
	unsigned _timeout;
	unsigned _limit;
	Lock _lock;
	unsigned _size;
	unsigned _noOfEntries;
	RemotePathCacheInfo ** _info;
	void garbageCollect();
	unsigned hash( RemoteDictionary * dictionary, char * key );
public:
	RemotePathCache( unsigned size = _defaultHashTableSize );
	void bind( RemoteDictionary * dictionary, char * key, 
		   RFSHandle & handle );
	void unbind( RemoteDictionary * dictionary, char * key );
	void unbind( RemoteDictionary * dictionary );
	int lookup( RemoteDictionary * dictionary, char * key, 
		    RFSHandle * handle );
};

extern RemotePathCache * TheRemotePathCache;

#endif RemoteTables_h
