/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SVIDContainer.h - SVID Inode System Class
 */

#ifndef SVIDContainer_h
#define SVIDContainer_h


#ifdef __GNUC__
#pragma interface
#endif

#include "PersistentMemoryObject.h"
#include "UNIXContainer.h"

const int FsMAGIC = 0xfd187e20;		/* s_magic number */
const int FsMAGICOFFSET = 1016;		/* Magic location on disk */
const int NICFREE=50;
const int NICINOD=100;

class filsys {
	friend class SVIDContainer;
	friend class SVIDAllocator;

	unsigned short	s_isize;	/* 0 size in blocks of i-list */
	int	s_fsize;		/* 2 size in blocks of entire volume */
	short	s_nfree;		/* 6 number of addresses in s_free */
	int	s_free[NICFREE];	/* 8 free block list */
	short	s_ninode;		/* 208 number of i-nodes i s_inode */
	unsigned short s_inode[NICINOD];/* 210 free i-node list */
	char	s_flock;		/* 410 lock of free list manipulation */
	char	s_ilock;		/* 411 lock of i-list manipulation */
	char 	s_fmod;			/* 412 superblock modified flag */
	char 	s_ronly;		/* 413 mounted read-only flag */
	int	s_time;			/* 414 last superblock update */
	short	s_dinfo[4];		/* 418 device information */
	int	s_tfree;		/* 426 total free blocks */
	unsigned short s_tinode;	/* 430 total free i-nodes */
	char	s_fname[6];		/* 432 file system name */
	char	s_fpack[6];		/* 438 file system pack name */
	int	s_fill[12];		/* 444 ADJUST to make filsys size 512 */
	int	s_state;		/* 492 file system state */
	int	s_magic;		/* 496 magic number of file system */
	int	s_type;			/* 500 type of new file system */
};

extern ProxiableObject * SVIDContainerConstructor( ProxiableObject * );
extern PersistentClassStar SVIDContainerClass;
class SVIDContainer : public UNIXContainer {
protected:
	filsys * _superBlock;

	int	numFragments( int size );
	int	mapInumber( int inum );
	int	firstInumber( int inum );
	int	getUnusedInode( int notUsedInThisSubClass );

	FileObject * basicOpen( int idNumber, int & status, int flags );
	FileObject * basicCreate( int idNumber, Class *, int &, int flags );
	void basicClose( FileObject * mo );
	int  basicRootId();
	void basicSynchronize();

	~SVIDContainer();
public:
	Class * classOf();

	SVIDContainer( PersistentMemoryObject * source );
};

inline int
SVIDContainer::numFragments( int size )
{
	int fragmentSize = _source->unitSize() << _log2FragmentToSector;
	return( ( size + fragmentSize - 1 ) / fragmentSize ); 	
}

#endif /*SVIDContainer_h*/
