/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SVIDInode.h - 
 */

#ifndef SVIDInode_h
#define SVIDInode_h


#ifdef __GNUC__
#pragma interface
#endif

#include "UNIXInode.h"

class SVdinode {
	friend class SVIDInode;
	friend class SVIDContainer;

        unsigned short  di_mode;        /* mode and type of file */
        short           di_nlink;       /* number of links to file */
        unsigned short  di_uid;         /* owners user id */
        unsigned short  di_gid;         /* owners group id */
        int             di_size;        /* number of bytes in file */
        char            di_addr[40];    /* disk block addresses */
        int             di_atime;       /* time last accessed */
        int             di_mtime;       /* time last modified */
        int             di_ctime;       /* time of last file status change */
};

extern ClassStar SVIDInodeClass;
class	SVIDInode : public UNIXInode {
protected:
	// Structures and functions that almost could be inherited
	SVdinode	* _diskInode;

	int		allocBlock();
	void		freeBlock( int physicalBlock );
	int		getDirect( int index );
	void		setDirect( int index, int value );
	int		getIndirect( int level );
	void		setIndirect( int level, int value );
	int		accessMode();
	int		userId();
	int		groupId();
	~SVIDInode();

	Class *	supports( Class * );

	int		basicSetNumberOfBytes( int size );

public:
	Class * classOf();
	SVIDInode( unsigned int numberOfUnits,
		   int log2UnitSize,
		   int inumber,
		   MemoryObjectContainer *,
		   PersistentMemoryObject * source,
		   SVdinode * );

	int		isDictionary();
	int		numberOfBytes();
	int		links( int delta );
	int		setMode( int mode );
	int		setUserId( int uid );
	int		setGroupId( int gid );
	void		info( UNIXFileInfo * );
};

#endif	SVIDInode_h
