/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Contributing Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Contributing Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * UNIXContainer.h - unix inode system abstract Class
 */

#ifndef UNIXContainer_h
#define UNIXContainer_h

#ifdef __GNUC__
#pragma interface
#endif

#include "MemoryObjectCachingContainer.h"
#include "Semaphore.h"

class UNIXInodeIndirect;
class UNIXInodeIndirectInfo;


extern ClassStar UNIXContainerClass;
class UNIXContainer : public MemoryObjectCachingContainer {
 protected:
    UNIXInodeIndirect * _indirect;
    UNIXInodeIndirectInfo ** _diskInodeCacheInfo;
    int             _noDiskInodeBufs;
    int     _log2FragmentToSector;
    int     _inodesPerBlock;
    int     _dinodeSize;
    int     _inodeBlockSize;

    void        writeDinode( int inum, int release, unsigned delay=0 );
    void *      getDinode( int inum, UNIXInodeIndirectInfo ** & info );
    void        releaseDinode( UNIXInodeIndirectInfo * info );

    virtual int mapInumber( int inum );
    virtual int firstInumber( int inum );

    void            initializeArrays( int containerSize );
    void            basicSynchronize( int idNumber );
    void            basicSynchronizeInodes();

    ~UNIXContainer();
    
    friend class UNIXInode;

 public:
    Class * classOf();

    UNIXContainer( PersistentMemoryObject * source );
};

#endif  /*UNIXContainer_h*/

