/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * UNIXInodeIndirect.h -
 */

#ifndef UNIXInodeIndirect_h
#define UNIXInodeIndirect_h

#ifdef __GNUC__
#pragma interface
#endif

#include "Assert.h"
#include "Lock.h"

class PersistentMemoryObject;
class OutputStream;

class UNIXInodeIndirectInfo;
class UNIXInodeIndirect;
class UNIXInodeIndirectSynchronizeClosure : public IOAsyncClosure {
 protected:
    static void func( UNIXInodeIndirectSynchronizeClosure * c );
    static void funcWriteDone( UNIXInodeIndirectSynchronizeClosure * c);
 public:
    UNIXInodeIndirect * _indirect;
    UNIXInodeIndirectInfo * _info;
    int _release;

    UNIXInodeIndirectSynchronizeClosure
    ( UNIXInodeIndirect * indirect, UNIXInodeIndirectInfo * info,
      int release )
    : IOAsyncClosure(),
    _indirect( indirect ), _info( info ), _release( release )
    {_func= (AsyncFunc)&func;}
    // ), _urgency( 0 ) { }
};

class UNIXInodeIndirectInfo {
    friend class UNIXInodeIndirect;

    PersistentMemoryObject * _source;
    int _lock;
    unsigned _idNumber;
    unsigned _unit;
    unsigned _age;
    int _dirty;
    unsigned _pending;
    int * _buf;
    UNIXInodeIndirectInfo * _prev;
    UNIXInodeIndirectInfo * _next;
    UNIXInodeIndirectInfo ** _list;
    UNIXInodeIndirectInfo * _neighbor;
 public:
    UNIXInodeIndirectInfo() 
    { _source = 0; _unit = 0; _age = 0; _lock = 0; _pending = 0;
    _dirty = 0; _prev = 0; _next = 0; _list = 0; }
    unsigned lock() 
    { return _lock; }
    unsigned unit()
    { return _unit; }
    unsigned idNumber() 
    { return _idNumber; }
    int * buf( unsigned i ) 
    { return _buf + i; }
    int * buf() 
    { return _buf; }
    int * dirtyFlag()
    { return &_dirty; }
    void setPending( unsigned pending )
    { _pending = pending; }
    PersistentMemoryObject * source() 
    { return _source; }
};

class UNIXInodeIndirect {
 protected:
    static UNIXInodeIndirect * _indirect[32];

    static Lock _lock;
    static void acquire() { _lock.acquire(); }
    static void release() { _lock.release(); }
    
    unsigned _size;
    unsigned _blockSize;
    unsigned _log2BlockSize;
    unsigned _hashSize;
    unsigned _free;
    unsigned _unlocked;
    unsigned _locked;
    unsigned _busy;

    unsigned _fastHit;
    unsigned _slowHit;
    unsigned _miss;
    
    char * _bufs;
    UNIXInodeIndirectInfo * _infos;
    UNIXInodeIndirectInfo * _freeList;
    UNIXInodeIndirectInfo ** _hashTable;

    static unsigned _age;
    
    friend class UNIXInodeIndirectSynchronizeClosure;

    static unsigned age() 
    { return _age++; }
    unsigned hash( PersistentMemoryObject * source, unsigned unit )
    { return ( ( ((unsigned) source) >> 2 ) + unit ) % _hashSize; }
    unsigned empty( UNIXInodeIndirectInfo * infoList )
    { return infoList == 0; }
    void removeFromList( UNIXInodeIndirectInfo ** list,
                         UNIXInodeIndirectInfo * item );
    void addToList( UNIXInodeIndirectInfo ** list,
                    UNIXInodeIndirectInfo * item );
    unsigned read( PersistentMemoryObject * source, unsigned unit,
                   unsigned * buf );
    unsigned write( UNIXInodeIndirectInfo * info );
    void write( UNIXInodeIndirectInfo * info, IOAsyncClosure * c );
    void writeDone( UNIXInodeIndirectInfo * info, unsigned result );
    void allocateMore();
    UNIXInodeIndirectInfo * victimize();
    UNIXInodeIndirectInfo * basicGetIndirect( PersistentMemoryObject * src,
                                              unsigned unit, 
                                              unsigned idNumber );
    static void timeout( UNIXInodeIndirectSynchronizeClosure * c );

 public:
    static void infoAll(OutputStream &);
    static UNIXInodeIndirect * setup( unsigned log2BlockSize );
    static UNIXInodeIndirect * indirect( unsigned log2BlockSize );
    UNIXInodeIndirect( unsigned size, unsigned log2blockSize );
    UNIXInodeIndirectInfo * newIndirect( PersistentMemoryObject * source,
                                         unsigned unit, unsigned idNumber );
    UNIXInodeIndirectInfo * getIndirect( PersistentMemoryObject * source,
                                         unsigned unit, 
                                         unsigned idNumber,
                                         UNIXInodeIndirectInfo * hint );
    void releaseIndirect( UNIXInodeIndirectInfo * info, int modified );
    void freeIndirect( UNIXInodeIndirectInfo * info );
    void synchronize( UNIXInodeIndirectInfo * info );
    // void synchronize( UNIXInodeIndirectInfo * info, unsigned delay,
    //           int release );
    void synchronize( PersistentMemoryObject * source, unsigned unit,
                      unsigned idNumber, UNIXInodeIndirectInfo * hint,
                      unsigned delay, int release );
    void synchronize( PersistentMemoryObject * source, unsigned idNumber );
    void synchronize( PersistentMemoryObject * source );
    void free( PersistentMemoryObject * source );
    void info(OutputStream &);
};

inline
UNIXInodeIndirect *
UNIXInodeIndirect::indirect( unsigned log2BlockSize )
{
    return _indirect[ log2BlockSize ];
}

inline
void 
UNIXInodeIndirect::removeFromList( UNIXInodeIndirectInfo ** list,
                                   UNIXInodeIndirectInfo * item )
{ 
    Assert( item->_list == list );
    item->_list = 0;
    UNIXInodeIndirectInfo * prev = item->_prev;
    if ( item->_prev ) {
        Assert( item->_prev->_next == item );
        item->_prev->_next = item->_next;
        item->_prev = 0;
    } else {
        Assert( *list == item );
        *list = item->_next;
    }
    if ( item->_next ) {
        Assert( item->_next->_prev == item );
        item->_next->_prev = prev;
        item->_next = 0;
    }
}

inline
void 
UNIXInodeIndirect::addToList( UNIXInodeIndirectInfo ** list,
                              UNIXInodeIndirectInfo * item )
{
    Assert( item->_list == 0 && item->_next == 0 && item->_prev == 0 );
    item->_prev = 0;
    item->_next = *list;
    if (item->_next) {
        Assert( item->_next->_prev == 0 );
        Assert( item->_next->_list == list );
        item->_next->_prev = item;
    }
    *list = item;
    item->_list = list;
}

inline
UNIXInodeIndirectInfo *
UNIXInodeIndirect::getIndirect( PersistentMemoryObject * source,
                                unsigned unit, unsigned idNumber,
                                UNIXInodeIndirectInfo * hint )
{
    acquire();
    if ( hint != 0 && hint->_source == source && hint->_unit == unit &&
         hint->_lock != -1 ) 
    {
        // Francis: TODO I don't think this assertion is correct
        //Assert( hint->_idNumber == idNumber );
        Assert( hint->_list ==
                _hashTable + hash( hint->_source, hint->_unit ) );
        if ( hint->_lock == 0 ) {
            _locked++;
            _unlocked--;
        }
        hint->_lock ++;
        hint->_age = age();
        _fastHit++;
    } else {
        hint = basicGetIndirect( source, unit, idNumber );
        Assert( hint->_list ==
                _hashTable + hash( hint->_source, hint->_unit ) );
    }
    release();
    return hint;
}

inline 
void
UNIXInodeIndirect::releaseIndirect( UNIXInodeIndirectInfo * info,
                                    int modified )
{
    Assert( info->_lock > 0 );
    Assert( info->_list == _hashTable + hash( info->_source, info->_unit ) );
    acquire(); 
    info->_lock--;
    if ( info->_lock == 0 ) {
        _locked--;
        _unlocked++;
    }
    if ( modified ) info->_dirty = 1;
    info->_age = age();
    release(); 
}
 
inline 
void
UNIXInodeIndirect::freeIndirect( UNIXInodeIndirectInfo * info )
{
    acquire(); 
    Assert( info->_lock == 1 );
    Assert( info->_list ==
            _hashTable + hash( info->_source, info->_unit ) );
    removeFromList( _hashTable + hash( info->_source, info->_unit ), info );
    _free++;
    _locked--;
    info->_lock--;
    info->_source = 0;
    info->_unit = 0;
    info->_dirty = 0;
    info->_age = 0;
    addToList( &_freeList, info );
/*
  Debug
  << "UNIXInodeIndirect[" << _log2BlockSize 
  << "]::freeIndirect( " 
  << asAddress(info) << " )\n" << eor; 
*/
    release(); 
}

#endif // UNIXInodeIndirect_h
