/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * VCFileObject.h 
 */

#ifndef VCFileObject_h
#define VCFileObject_h


#ifdef __GNUC__
#pragma interface
#endif

#include "FileObject.h"

extern "C" {
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
};


class UNIXFileInfo;

extern ClassStar VCFileObjectClass;

class VCFileObject : public FileObject {

        char * _path;
	int _fd;		// For when it's open
	struct stat _buf;	// For getting file info
	VCFileObject *		_root;	// The root FO in my partition

protected:
	void	invalidateCache( );

	void	noRemainingReferences();

	int	transient();
	int	accessMode();
	int	access(int mask);
	int	userId();
	int	groupId();
	void    restat();    	// Update stat buffer
//	int blockSize(){return 0;}
//	int mapUnit(int i, int){return i;}

	Class * supports( Class * );

	int     basicSetNumberOfBytes( int size );
	unsigned	basicSetNumberOfUnits( unsigned units );
	int	rawRead( unsigned startUnit, int count, char *, int, 
		  	 IOAsyncClosure * );
	int	rawWrite( unsigned startUnit, int count, char *, int, int,
			  IOAsyncClosure * );

public:
	VCFileObject * root(){return _root;}
	Class * classOf();

	VCFileObject(char * path, VCFileObject *);
	~VCFileObject( );

	int	isDictionary();

	int     fixedSize();

	int	setUserId(int);
	int	setGroupId(int);
	int	setMode(int);

	int	numberOfBytes();

	void	writeIdNumber( OutputStream * os );
	void	info( UNIXFileInfo * infoBuffer );
	char *	path() {return _path;}
};

#endif	//VCFileObject_h

