/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ZombieInfo.h - 
 */

#ifndef ZombieInfo_h
#define ZombieInfo_h

#ifdef __GNUC__
#pragma interface
#endif

#include "Assert.h"

enum ZombieInfoType {
	ZOMBIE_NOT,
	ZOMBIE_NOTSYNC,
	ZOMBIE_SYNC,
	ZOMBIE_EXTERN
};
	
class FileObject;

class ZombieInfoList;

class MemoryObjectCachingContainer;

class ZombieInfo {
friend class ZombieInfoList;
protected:
	ZombieInfoType _type;
	unsigned int _time;
	unsigned int _expire;
	FileObject * _fo;
	ZombieInfoList * _list;
	ZombieInfo * _next;
	ZombieInfo * _prev;
public:
	ZombieInfo() : _type( ZOMBIE_NOT ), _time( 0 ), _expire( 0 ) {}
	void unlink();
	void init( FileObject * fo, unsigned time );
	void insertAfter( ZombieInfo * );
	void insertBefore( ZombieInfo * );
	FileObject * fo() { return _fo; }
	void setFO( FileObject * fo ) { _fo = fo; }
	ZombieInfoType type() { return _type; }
	void setType( ZombieInfoType type ) { _type = type; }
	unsigned time() { return _time; }
	void setTime( unsigned time ) { _time = time; }
	unsigned expire() { return _expire; }
	void setExpire( unsigned time ) { _expire = time; }
	ZombieInfo * next() { return _next; }
	ZombieInfo * prev() { return _prev; }
};

class ZombieInfoList : public ZombieInfo
{
protected:
	friend class ZombieInfo;
	int _count;
public:
	ZombieInfoList( ZombieInfoType type );
	ZombieInfo * head() { return _next; }
	int count() { return _count; }
	int isEmpty() { return _count == 0; }
	void insertByTime( ZombieInfo * info );
};

inline
ZombieInfoList::ZombieInfoList( ZombieInfoType type )
{
	_list = this;
	_next = this; 
	_prev = this; 
	_fo = 0; 
	_type = type;
	_count = 0;
}

inline
void 
ZombieInfo::init( FileObject * fo, unsigned time )
{
	Assert( _type == ZOMBIE_NOT );
	_fo = fo;
	_time = time;
}

inline 
void
ZombieInfo::unlink()
{
	Assert( _list != this );
	_next->_prev = _prev;
	_prev->_next = _next;
	_next = _prev = 0;
	_list->_count--;
	Assert( _list->_count >= 0 );
	_list = 0;
	_type = ZOMBIE_NOT;
}

inline
void
ZombieInfo::insertAfter( ZombieInfo * info )
{
	Assert( _type == ZOMBIE_NOT );
	ZombieInfo * prev = info;
	ZombieInfo * next = info->_next;
	_prev = prev;
	_next = next;
	prev->_next = this;
	next->_prev = this;
	_list = info->_list;
	_list->_count++;
	_type = _list->_type;
}

inline
void
ZombieInfo::insertBefore( ZombieInfo * info )
{
	Assert( _type == ZOMBIE_NOT );
	ZombieInfo * next = info;
	ZombieInfo * prev = info->_prev;
	_next = next;
	_prev = prev;
	next->_prev = this;
	prev->_next = this;
	_list = info->_list;
	_list->_count++;
	_type = _list->_type;
}

#endif /*ZombieInfo_h*/
