/*
 * This file is part of the Choices Operating System
 * Author: Yevgeniy Rabinovich <rabinvch@uiuc.edu>,
 *                             <yrabinov+uiuc@cs.uchicago.edu>
 * Copyright (c) 2004
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 */
/*
 * endian_swap.h - Functions for endiannes-swapping
 */

#ifndef ENDIAN_SWAP_H
#define ENDIAN_SWAP_H

#include "Libraries/Unix/include/choices/machine/endian.h"

#if BYTE_ORDER == BIG_ENDIAN
#define NEED_ENDIAN_SWAP
#endif // BYTE_ORDER

/* Function to convert a 32-bit value into different endianness */
inline void swapl(void *addr)
{
#ifdef NEED_ENDIAN_SWAP
  unsigned char temp = *((unsigned char *)addr);
  /* Swap first and last digits */
  *((unsigned char *)addr) = *(((unsigned char *)addr)+3);
  *(((unsigned char *)addr)+3) = temp;
  /* Swap the middle digits */
  temp = *(((unsigned char *)addr)+1);
  *(((unsigned char *)addr)+1) = *(((unsigned char *)addr)+2);
  *(((unsigned char *)addr)+2) = temp;
#endif // NEED_ENDIAN_SWAP
}

/* Function to convert a 16-bit value into different endianness */
inline void swaps(void *addr)
{
#ifdef NEED_ENDIAN_SWAP
  unsigned char temp = *((unsigned char *)addr);
  /* Swap the digits */
  *((unsigned char *)addr) = *(((unsigned char *)addr)+1);
  *(((unsigned char *)addr)+1) = temp;
#endif // NEED_ENDIAN_SWAP
}


/* Example:
int main(int argc, char **argv)
{
  int a = 0x12345678;
  short b = 10;

  std::cout << a << std::endl;
  swapl((void *) &a);
  std::cout << a << std::endl;

  std::cout << b << std::endl;
  swaps((void *) &b);
  std::cout << b << std::endl;

  return 0;
}
*/

#endif // ENDIAN_SWAP_H
