#ifndef AdapterConduit_h
#define AdapterConduit_h


#ifdef __GNUC__
#pragma interface
#endif

#include "NetworkAddress.h"
#include "ConduitTypes.h"
#include "Conduit.h"

class DumbConduit;

class AdapterConduit : public Conduit {
  /* AdapterConduits are used so that the DumbConduit in TtyBuffer
   * can genericly insert message & not have to deal with the port
   * specific garbage.  The subclasses know about the port specific
   * garbage.  The methods here are just placeholders...
   */
public:
  /* Method insertFromAbove: outbound message */
  virtual Return_Code insertFromAbove(int l, octet *s);

  /* Method expandWindow: give some flow control tokens back so we can
   * get some more messages.
   */
  virtual Return_Code expandWindow(int count);

  /* Method connectFromAbove: Meet the guy to whom you'll be
   * forwarding messages.
   */
  virtual Return_Code connectFromAbove( DumbConduit * c );
};

class TCPAdapterConduit : public AdapterConduit {
protected:
  TCPConduit *thingBelowMe;
  DumbConduit *thingAboveMe;
  TCPAddress mysession;
public:
  /* Method insertFromAbove: forwards an outbound message */
  virtual Return_Code insertFromAbove(int l, octet *s);

  /* Method insertFromBelow: forwards an inbound message */
  virtual Return_Code insertFromBelow(ConduitMessage * msg);

  /* Method expandWindow: give some flow control tokens back so we can
   * get some more messages.
   */
  virtual Return_Code expandWindow(int count);

  /* Method connectTo: introduce yourself to the TCPConduit to whom
   * you will send messages.
   */
  virtual Return_Code connectTo ( TCPConduit * c );

  /* Method connectTo: if it isn't a TCPConduit Barf bitch wail & let
   * everyone know.
   */
  virtual Return_Code connectTo ( Conduit * c );

  /* Method connectFromAbove: Meet the guy to whom you'll be
   * forwarding messages.
   */
  virtual Return_Code connectFromAbove( DumbConduit * c );

  /* Method connectFromBelow: TCPConduit will handshake using this;
   * we'll put up with him & throw away the welcome wagon fliers.
   */
  virtual Return_Code connectFromBelow(Conduit * c,
				       ProtocolID proto,
				       int his_handy,
				       int & my_handy );

  /* Method session: TCP needs an address, so we'll keep track of it here. */
  TCPAddress *session();

  /* Method setSession: set this bit of state that tcp needs... */
  void setSession( TCPAddress & newsession );
};

#endif /* AdapterConduit_h */
