/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Contributing Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Contributing Author: Lee Lup Yuen (lup@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * CharacterInputStream.h: definition of an input stream of characters.
 */

#ifndef CharacterInputStream_h
#define CharacterInputStream_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Assert.h"
#include "CharacterQueue.h"
#include "InputStream.h"
#include "CharacterInputStreamStar.h"

class OutputStream;

const int CS_LINEBUFFERED = 0;
const int CS_RAW = 1;
const int CS_CHARACTERBLOCK = 2;

extern ClassStar CharacterInputStreamClass;
class CharacterInputStream : public InputStream {
protected:
	OutputStream * _echoStream;
	CharacterQueue * _queue;

public:
	CharacterInputStream( OutputStream * echoStream );

	proxiable virtual int read( char * buffer, int bufSize );
	proxiable virtual void switchCharacterQueue( int type );
	int enqueue( const char * buffer, int count );
	CharacterQueue * queue();

	Class * classOf();
};

inline int
CharacterInputStream::enqueue( const char * buffer, int count )
{
	Assert( _queue != 0 );
	return( _queue->enqueue( buffer, count ) );
}

inline CharacterQueue *
CharacterInputStream::queue()
{
	Assert( _queue != 0 );
	return( _queue );
}

/*
 * This is defined in Kernel/Kernel.cc
 */
extern CharacterInputStreamStar ConsoleInputStream;

#endif	/* CharacterInputStream_h */
