/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef DiskScheduler_h
#define	DiskScheduler_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Lock.h"
#include "Object.h"

class Disk;
class IOAsyncClosure;
class OutputStream;
class Process;
class Semaphore;
class AsyncSupport;

class DiskIORequest {
public:
	int _op;
	unsigned _start;
	int _count;
	char * _buf;
	int _nbytes;
	int _through;
	unsigned * _result;
	DiskIORequest * _next;
	DiskIORequest * _prev;
	IOAsyncClosure * _closure;
	Semaphore * _sem;

	DiskIORequest( int op, unsigned start, unsigned count, char * buf, 
		       unsigned nbytes, int through, unsigned * result,
		       IOAsyncClosure * c, Semaphore * sem );
};

inline
DiskIORequest::DiskIORequest( int op, unsigned start, unsigned count, char * buf, 
			      unsigned nbytes, int through, unsigned * result,
			      IOAsyncClosure * c, Semaphore * sem )
{
	_op = op; 
	_start = start; 
	_count = count; 
	_buf = buf;
	_nbytes = nbytes; 
	_through = through; 
	_result = result;
	_next = _prev = 0;
	_closure = c;
	_sem = sem;
}

extern OutputStream & operator<< ( OutputStream & stream, DiskIORequest & ior );

class DiskScheduler : public Object {
protected:
	Disk * _disk;
	Process * _process;
	Semaphore * _sem;
	Lock _lock;
	int _queueLength;
	DiskIORequest * _head;
	DiskIORequest * _tail;
	DiskIORequest * _current;
	AsyncSupport * _asyncSupport;
	void daemon();
public:
	DiskScheduler( Disk * disk );
	virtual void add( DiskIORequest * ior ) = 0;
	void startDaemon();
};

class FIFODiskScheduler : public DiskScheduler {
public:
	FIFODiskScheduler( Disk * disk ) : DiskScheduler( disk ) {}
	void add( DiskIORequest * ior );
};

class CSCANDiskScheduler : public DiskScheduler {
public:
	CSCANDiskScheduler( Disk * disk ) : DiskScheduler( disk ) {}
	void add( DiskIORequest * ior );
};

class OneAtATimeDiskScheduler : public DiskScheduler {
	Semaphore * _oneAtATimeMutex;
public:
	OneAtATimeDiskScheduler( Disk * disk );
	void add( DiskIORequest * ior );
};

#endif /*DiskScheduler_h*/
