/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Roland Ouellette
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * dumbConduit.h
 */


#ifndef DumbConduit_h
#define DumbConduit_h


#ifdef __GNUC__
#pragma interface
#endif

class ConduitMessage;
class TtyBuffer;
#include "Conduit.h"
#include "AdapterConduit.h"

class DumbConduit : public Conduit {
  /*
   * This thing is a friend of TtyBuffer.  It knows how to put
   * messages that are inserted into it into its associated TtyBuffer.
   * It could really be combined with adapter conduit...
   * Note that the insert top method takes octet *'s, rather than messages.
   */
protected:
  TtyBuffer *myTtyBuffer;
  AdapterConduit *belowMeConduit;

public:

  /* Method DumbConduit: constructor */
  DumbConduit(TtyBuffer *t);

  /* Method expandWindow: The method of flow control implemented here
   * is token based, like TCP.  ExpandWindow is used to give tokens
   * back to the stuff under us, so that it will send us more
   * messages.  We count input in characters and not messages
   * because otherwise not be able to get bazillions of little
   * messages.  Bazillions of big messages could be a problem
   * though...
   */
  void expandWindow(int n);

  /* Method insertFromBelow: Take an inbound message */
  Return_Code insertFromBelow( ConduitMessage * msg );

  /* Message insertFromAbove: Take an outbound message... but not a message yet */
  Return_Code insertFromAbove( int l, octet *s );
  
  /* Message connectTo: Meet the guy to whom you send outbound messages
   * and expand windows.
   */
  Return_Code connectTo( AdapterConduit * c );
};

#endif /* DumbConduit_h */
