/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Panagiotis Kougiouris (kougiour@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef InputLogicalDevice_h
#define InputLogicalDevice_h


#ifdef __GNUC__
#pragma interface
#endif

#include "SerialDevice.h"

class CharacterQueue;

class InputLogicalDevice  {
protected:

	/* Since it is a logical device it should contain an
	 * actual device
	 */

	 SerialDevice * device;

public:
	/* The constructor of a serial device. The SerialDevice is the
	 * actual device. 
	 */
	InputLogicalDevice( SerialDevice * );

	/* A function to set attributes of the Logical device */
	virtual void
	setAttributes( int );

	/* A function to check if any character waits
	 * last time a char was getChared()---see below.
	 */
	virtual unsigned short charPending();

	/* Get a character from the input. In case of buffer empty 
	 * a -1 is returned.
	 */
	virtual char getChar();

	/* A function to read more than one chars up to a terminator */
	virtual int read( char * buffer, int size );

	/* Function to enable input interrupts */
	void enableInputInterrupt();

	/* A function to be called when an input interrupt
	 * comes from this device.
	 */
	virtual void inputInterruptHandle();

	/* A method to return the input interrupt vector number
	 * stored at the physical device.
	 */
	int inputInterruptVector();


	/* A method to return the hardware device */
	SerialDevice * asADevice();

};

inline int
InputLogicalDevice::inputInterruptVector(){

	/* device is an instance variable of this*/

	return	device->inputInterruptVector();
}

inline void
InputLogicalDevice::enableInputInterrupt()
{

	device->enableInputInterrupt();
}

inline SerialDevice *
InputLogicalDevice::asADevice(){

	return device;
}

#endif	/* InputLogicalDevice_h */
