/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Bjorn A. Helgaas (helgaas@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * OutputStream.h: definition of an output stream of characters.
 */

#ifndef MultiplexedOutputStream_h
#define MultiplexedOutputStream_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Lock.h"
#include "OutputStream.h"
#include "OutputStreamStar.h"

class CharacterQueue;


/*
 * Classes to help multiplex a stream.
 */

class MultiplexedOutputStream : public OutputStream {
protected:
	Lock _lock;

	CharacterQueue * _queue;
	OutputStreamStar _sink;
	char * _flushBuffer;
	int _bufferSize;

	void basicFlush();

public:
	MultiplexedOutputStream( OutputStream * sink, int bufferSize = 512 );
	~MultiplexedOutputStream();

	virtual void acquireExclusiveUse();
	virtual void releaseExclusiveUse();
	virtual void flush();
	virtual int write( const char * buffer, int charCount );
};


class IdentifyingMultiplexedOutputStream : public MultiplexedOutputStream {
protected:
	Object * _owner;

public:
	IdentifyingMultiplexedOutputStream( OutputStream * sink, Object * owner,		int bufferSize = 512 );

	virtual void flush();
};

#endif /* MultiplexedOutputStream_h */
