/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Panagiotis Kougiouris (kougiour@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef OutputLogicalDevice_h
#define OutputLogicalDevice_h


#ifdef __GNUC__
#pragma interface
#endif

#include "SerialDevice.h"


class OutputLogicalDevice {
protected:

	/* Since it is a logical device it should contain an
	 * actual device
	 */

	 SerialDevice * device;

public:
	/* The constructor of a serial device. The SerialDevice is the
	 * actual device. 
	 */
	OutputLogicalDevice( SerialDevice * );

	/* Write a character
	 */
	void putChar( char c );

	/* Function to enable  output interrupts */
	void enableOutputInterrupt();

	/* A method to be called when an output interrupt comes for the device
	 */
        void outputInterruptHandle();

	/* A method to return the output interrupt vector number
	 * stored at the physical device
	 */
	 int outputInterruptVector();

	/* A method to return the device */
	SerialDevice * asADevice();

};

inline int 
OutputLogicalDevice::outputInterruptVector(){

	/* device is an instance variable of this */
	
	return device->outputInterruptVector();
}


inline void
OutputLogicalDevice::enableOutputInterrupt()
{

	device->enableOutputInterrupt();
}

inline SerialDevice * 
OutputLogicalDevice::asADevice(){

	return device;
}
#endif	/* OutputLogicalDevice_h */
