/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Panagiotis Kougiouris (kougiour@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef SerialDevice_h
#define SerialDevice_h


#ifdef __GNUC__
#pragma interface
#endif

class SerialDevice {
protected:

	/* The device should know the vector numbers assigned to
	 * it in order to be able to change the interrupt handlers.
	 * It seems reasonable that the vector numbers should not change
	 * on the fly. So these numbers are initialized during
	 * object creation and they do not change any more.
	 */
	int inputInterruptVectorNumber;
	int outputInterruptVectorNumber;    

public:
	/* The constructor of a serial device */
	SerialDevice( int, int );

	/* A function that brings the device in a predefined state.
	 * The specialization of this function should specify exactly
	 * that state
	 */
	virtual void reset();

	/* A function which just puts the char c to the output register
	 * to be printed. the function does not check if a pending
	 * operation exists.
	 */
	virtual void putChar( char c);

	/* A function to check if any character has arrived since the
	 * last time a char was getChared()---see below.
	 */
	virtual unsigned short charPending();

	/* Get a character from the input register and reset the char pending
	 * flag if it is set. The function does not
	 * check whether the character just arrived or used to exist there
	 * for ever. 
	 */
	virtual char getChar();

	/* A function to be called when an input interrupt
	 * comes from this device.
	 */
	void inputInterruptHandler();

	/* A method to be called when an output interrupt comes for the device
	 */
        void outputInterruptHandler();

	/* Methods to read the input and output interrupt
	 * vectors 
	 */
       int inputInterruptVector();

       int outputInterruptVector();

       /* Methods to enable the chip interrupts */
       virtual void enableInputInterrupt();

       virtual void enableOutputInterrupt();


};


inline int
SerialDevice::inputInterruptVector()
{
	return inputInterruptVectorNumber;
}

inline int
SerialDevice::outputInterruptVector()
{
	return outputInterruptVectorNumber;
}

#endif	/* SerialDevice_h */
