/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef EventerInstrument_h
#define EventerInstrument_h


#ifdef __GNUC__
#pragma interface
#endif

#ifdef INSTRUMENT

#include "Instrument.h"
#include "InstrumentTypes.h"
#include "InstrumentFormat.h"

extern class InstrumentManager;

#include "InstrumentManager.h"

extern ClassStar EventerInstrumentClass;

class EventerInstrument : public Instrument {
/*
 * Just an event logger.
 */

private:

protected:
	struct {
		iIdentifierType id;
	} _header;

	/* Method birthLog:	Log instrument creation.		*/
	void birthLog ();

	/* Method deathLog:	Log instrument destruction.		*/
	virtual void deathLog ();

	/* Method log:		Log an event.				*/
	void log (unsigned int eventID,
		  char *message,
		  unsigned int messageLength);
	
public:

	/* Method EventerInstrument:		The constructor.	*/
	proxiable EventerInstrument (InstrumentManager *iowner,
				     char *title,
				     iStateType state = DEAD);
	
	/* Method ~EventerInstrument:	The destructor.			*/
	proxiable ~EventerInstrument ();

	/* Method reset:	Set all instrument data to 0.		*/
	proxiable virtual void reset ();

	/* Method getID:	Return the assigned instrument ID.	*/
	proxiable virtual unsigned int getID ();

	/* Method event:	Cause an event to occur.		*/
	proxiable virtual void event (unsigned int eventID,
				      char *message = NULL,
				      unsigned int messageLength = 0);
	Class * classOf ();
};

#endif INSTRUMENT
#endif
