/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * InstrumentEvent.h: Event IDs used by the Kernel
 *
 */
/*
 * This file defines the kernel instrument event IDs.  All the kernel events
 * logged by instruments should be placed here for easy lookup during
 * post-processing if necessary.
 *
 */ 
	
#ifndef InstrumentEvent_h
#define InstrumentEvent_h


#ifdef __GNUC__
#pragma interface
#endif

static const unsigned int GenericEvent  = (unsigned int) 0;
static const unsigned int InstrumentBirth  = (unsigned int) 1;
static const unsigned int InstrumentDeath = (unsigned int) 2;

static const unsigned int InstrumentBufferBirth = (unsigned int) 3;
static const unsigned int InstrumentBufferDeath = (unsigned int) 4;
static const unsigned int InstrumentBufferClear  = (unsigned int) 5;

static const unsigned int InstrumentNoOp  = (unsigned int) 6;

static const unsigned int CounterCheckEvent  = (unsigned int) 7;
static const unsigned int EventerCheckEvent  = (unsigned int) 8;
static const unsigned int TimerCheckEvent  = (unsigned int) 9;
static const unsigned int LatencyCheckEvent  = (unsigned int) 10;

static const unsigned int CPU_IdleLoopEvent  = (unsigned int) 11;
static const unsigned int Interrupt_RaiseEvent  = (unsigned int) 12;

static const unsigned int Domain_RepairFaultLookupEvent = (unsigned int) 13;
static const unsigned int Domain_RepairFaultEvent = (unsigned int) 14;
static const unsigned int
  Domain_RepairFaultMakeResidentEvent = (unsigned int) 15;
static const unsigned int
  Domain_RepairFaultAddMappingEvent = (unsigned int) 16;

static const unsigned int
  SystemInstrumentManager_DumpLogBufferEvent = (unsigned int) 17;

static const unsigned int
  InstrumentBuffer_InterruptEvent = (unsigned int) 18;
#ifdef MULTIMAX
static const unsigned int
  SVCTrap_Event = (unsigned int) 19;
#endif

/* Process Events */

static const unsigned int Process_ConstructorIn = (unsigned int) 101;
static const unsigned int Process_ConstructorOut = (unsigned int) 102;
static const unsigned int Process_DestructorIn = (unsigned int) 103;
static const unsigned int Process_DestructorOut = (unsigned int) 104;
static const unsigned int Process_GiveProcessorToSaveStart = (unsigned int)105;
static const unsigned int Process_GiveProcessorToSaveEnd = (unsigned int) 106;
static const unsigned int Process_GiveProcessorToRestoreStart = 
                                                           (unsigned int) 107;
static const unsigned int Process_GiveProcessorToRestoreEnd = 
                                                           (unsigned int) 108;
static const unsigned int Process_GiveProcessorTo =(unsigned int) 109;
static const unsigned int Process_Restore =(unsigned int) 110;

/* IO Events */
static const unsigned int IO_ReadStart = (unsigned int) 111;
static const unsigned int IO_ReadEnd = (unsigned int) 112;
static const unsigned int IO_WriteStart = (unsigned int) 113;
static const unsigned int IO_WriteEnd = (unsigned int) 114;

#endif
