/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * InstrumentManager.h: basic instrument manager
 */

// ----------------------------------------------------------------------------


#ifdef __GNUC__
#pragma interface
#endif

#ifndef InstrumentManager_h
#define InstrumentManager_h

#ifdef INSTRUMENT

#include "InstrumentTypes.h"

extern class Instrument;
#include "Instrument.h"

extern ClassStar InstrumentManagerClass;

class InstrumentManager : public ProxiableObject{

	friend class SystemInstrumentManager;
	
private:
	
protected:

	struct {
		iIdentifierType id;
	} _header;
	
	char *_title;

	// All of the instruments so we can dump/change/info them
	Instrument **_instrumentList;
	unsigned int _numInstruments, _maxInstruments;

	Boolean _suspendNewInstruments;

	/* Method birthLog:		Log manager creation.		*/
	virtual void birthLog (char *title);

	/* Method deathLog:		Log manager destruction.	*/
	virtual void deathLog ();

public:	

	/* Method InstrumentManager:	The constructor.		*/
	InstrumentManager (char *title,
			   unsigned int instrumentAlloc);

	/* Method ~InstrumentManager:	The destructor.			*/
	virtual ~InstrumentManager ();

	/* Method getID:	Return the assigned manager ID.		*/
	unsigned int getID ();
	
	/* Method getMode:		Return mode of instrument	*/
	virtual iModeType getMode ();

	/* Method suspendNewInstruments:	Is it suspending them?	*/
	Boolean suspendNewInstruments ();
	
	/* Method addInstrument:	Register it and return its ID.	*/
	unsigned int addInstrument (Instrument *i);

	/* Method removeInstrument:	Remove it from the active list.	*/
	void removeInstrument (Instrument *i);

	/* Method logAllInstruments:	Command everyone to log.	*/
	void logAllInstruments ();

	/* Method resetAllInstruments:	Command everyone to reset.	*/
	void resetAllInstruments ();

	/* Method suspendAllInstruments:	Suspend everyone.	*/
	void suspendAllInstruments ();

	/* Method resumeAllInstruments:		Resume everyone.	*/
	void resumeAllInstruments ();
	
	/* Method time:			Returns the system time.	*/
	virtual timeType time () = 0;
	
	/* Method writeLogBuffer:	Write data to the buffer.	*/
	virtual void writeLogBuffer (unsigned int dataLength,
				     char *data,
				     unsigned int eventID,
				     unsigned int messageLength = 0,
				     char *message = NULL) = 0;

	Class * classOf ();	

	/* Method dummy: 		required for proxify++ (bug)	*/
	proxiable virtual void dummy() {}
};

#endif INSTRUMENT
#endif
