/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * InstrumentTypes.h: Some types that the instrumentation needs.
 */

	
#ifndef InstrumentTypes_h
#define InstrumentTypes_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Types.h"

typedef unsigned int countType;
typedef unsigned int timeType;

// ----------------------------------------------------------------------------
// Flags whether the instrument is a system or a user level instrument
//   SYSTEM -- Kernel instrument
//   USER -- User (application) instrument
enum iModeType { SYSTEM_INSTRUMENT = 0, USER_INSTRUMENT };

// ----------------------------------------------------------------------------
// Describes whether the instrument is updated and/or printed to the log
//   DEAD -- do nothing
//   ASLEEP -- print only without optional messages
//   SNORING -- print only with messages
//   MUTE -- update only
//   AWAKE -- update and print without messages
//   TALKING -- update and print with messages
enum iStateType { DEAD, ASLEEP, SNORING, MUTE, AWAKE, TALKING };

// ----------------------------------------------------------------------------
// An attempt to allow user-process-only timers, ignored for now
//   SYSTEM -- timer only runs when in system
//   PROCESS -- timer only runs in the instrumented process
//   ALL -- timer always runs
//   SYSTEMPERCPU -- timer is "per cpu"
enum iScopeType { ALL, SYSTEM, PROCESS, SYSTEMPERCPU };

enum iInstrumentType { EVENTER, COUNTER, TIMER };
enum iTimerStateType { STOPPED, SUSPENDED, RUNNING };

// ----------------------------------------------------------------------------

enum iLogOpenResult { OKNEW, OKAPPEND, NOFILESPECIFIED, CANTCLOSE, CANTOPEN };

static const countType EVENT = 1;
static const countType SUMMARY = (countType) -1;

static const int MaxInstrumentLogNameSize = 128;

#endif
