/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1992 
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef PerCpuTimerInstrument_h
#define PerCpuTimerInstrument_h


#ifdef __GNUC__
#pragma interface
#endif

#ifdef INSTRUMENT

#include "Instrument.h"
#include "InstrumentTypes.h"
#include "InstrumentFormat.h"

extern class InstrumentManager;

#include "InstrumentManager.h"

extern ClassStar PerCpuTimerInstrumentClass;

class PerCpuTimerInstrument : public TimerInstrument {
/*
 * This is similar to a counter, but with a notion of change in time.
 */

private:
	TimerInstrument **_theInstruments; // one per CPU
	iInstrumentStateType *_theStates;
 	unsigned int _activeCPUs;
	
protected:
	
	/* Method birthLog:	Log instrument creation.		*/
	void birthLog ();

	/* Method deathLog:	Log instrument destruction.		*/
	virtual void deathLog ();

	/* Method log:		Log an event.				*/
	void log (unsigned int eventID,
		  char *message,
		  unsigned int messageLength);
		
public:

	 PerCpuTimerInstrument (SystemInstrumentManager *iowner,
				   char *title,
				   iStateType state = DEAD,
				   unsigned int freq = SUMMARY,
				   iScopeType scope = ALL);

	 ~PerCpuTimerInstrument ();
	
	 virtual void reset ();

	 virtual unsigned int getID ();

	 virtual void suspend ();

 	 virtual void resume ();

 	 virtual void begin ();

	 virtual void changeFreq (unsigned int newFreq);

	 virtual void changeScope (iScopeType newScope);

	 virtual void changeState (iStateType newState);
	
	 virtual void start ();

	 virtual void stop (unsigned int eventID,
				     char *message = NULL,
				     unsigned int messageLength = 0);

 	 void startAll ();

	Class * classOf ();
};

#endif INSTRUMENT
#endif
