/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SystemInstrumentManager.h: OS-only instrument manager
 */

#ifndef SystemInstrumentManager_h
#define SystemInstrumentManager_h


#ifdef __GNUC__
#pragma interface
#endif

#ifdef INSTRUMENT

#include "Interrupt.h"
#include "CPU.h"
#include "FreeRunningTimer.h"

#include "InstrumentManager.h"
#include "InstrumentBuffer.h"
#ifndef iPSC2
#include "RecordStreamStar.h"
#include "FileSystemInterfaceStar.h"
#endif

extern CPU *thisCPU ();

class UserInstrumentManager;

extern void SystemInstrumentManagerConstructInstruments ();

extern ClassStar InstrumentBufferInterruptClass;

class InstrumentBufferInterrupt : public Interrupt {
	
    friend class SystemInstrumentManager;
private:
        unsigned int *_raised;
	
protected:

	/* Method basicRaise:	Raise the interrupt.			*/
	void basicRaise (char * info);

public:
	InstrumentBufferInterrupt();
	~InstrumentBufferInterrupt();

};

extern ClassStar SystemInstrumentManagerClass;

class SystemInstrumentManager : public InstrumentManager {

	friend class Kernel;
	friend class InstrumentBufferInterrupt;
	friend void  InstrumentBufferDumpProcessEntry (int);

private:
	
	static Boolean _loggingIsOn;
#ifndef iPSC2
	// The logDump file
	static RecordStreamStar _logDumpFile;
	
	static FileSystemInterfaceStar _FSI;
#endif

protected:

	// The associated manager list
	InstrumentManager **_iManagerList;
	unsigned int _numIManagers, _maxIManagers;

	/* Method deathLog:		Log manager destruction.	*/
	virtual void deathLog ();
public:

	/* Method SystemInstrumentManager:	The constructor.	*/
	SystemInstrumentManager (char *title,
				      unsigned int instrumentAlloc,
				      unsigned int managerAlloc);

	/* Method ~SystemInstrumentManager:	The destructor.	*/
	~SystemInstrumentManager ();
	
	unsigned int _numInstrumentedCPUs;

	/* Method numInstrumentedCPUs:	Return the num of active CPUs.	*/
	unsigned int numInstrumentedCPUs ();

	/* Method addInstrumentManager:	Register a new user manager.	*/
	unsigned int addInstrumentManager (UserInstrumentManager *manager);

	/* Method removeInstrumentManager:	Remove a user manager.	*/
	void removeInstrumentManager (UserInstrumentManager *manager);

	/* Method logAllInstrumentManagers:	Log all other managers.	*/
	void logAllInstrumentManagers ();

	/* Method openLogDumpFile:	Open the file to dump to.	*/
	Boolean openLogDumpFile (char *fileName, unsigned int mode = 0666);

	/* Method openInitialLogDumpFile:  Open the file to dump to.	*/
	Boolean openInitialLogDumpFile();
	
	/* Method closeLogDumpFile:	Close the dump file.		*/
	Boolean closeLogDumpFile ();

	/* Method dumpLogBuffer:	Dump the buffer to the file.	*/
	Boolean dumpLogBuffer ();

	/* Method time:			Returns the system time.	*/
	timeType time ();
	
	/* Method writeLogBuffer:	Write data to the buffer.	*/
	void writeLogBuffer (unsigned int dataLength, char *data,
		unsigned int eventID,
		unsigned int messageLength = 0, char *message = NULL);

	void suspendAllInstruments ();
	void resumeAllInstruments ();

	void beginAllInstruments ();

	Class *classOf();
};

#endif // INSTRUMENT
#endif 

