
#ifndef InterruptManager_h
#define InterruptManager_h

#include "Assert.h"
#include "Interrupt.h"
#include "ProcessorContext.h"

#ifdef __GNUC__
#pragma interface
#endif

// Every interrupt in a machine should be wired to exactly one interrupt manager
// An interrupt manager may be shared between multiple CPUs
// Primary CPU should instantiate the manager and secondary CPUs may access this object
// through pointers
class InterruptManager {
public:
	Interrupt *setInterrupt(unsigned number, Interrupt *);
	Interrupt *getInterrupt(unsigned number);
	void dispatchInterrupt(unsigned number, InterruptContext *context);
	void interruptReceived(InterruptContext *context);
	void interruptReceived(unsigned number, InterruptContext *context);
	
	// Priority levels run from 1 .. NumPriorityLevels in decreasing order of priorities
	// Subclasses which implement interrupt priorities may redefine these functions
	virtual unsigned getNumPriorityLevels() { return 1; }
	virtual void setPriority(unsigned number, unsigned level) { Assert(level == 1); }
	virtual unsigned getPriority(unsigned number) { return 1; }

	// This is to enable interrupts on external interrupt hardware (interrupt controllers)
	// The processor has a separate enable/disable flag which can mask all interrupts
	virtual void enableInterrupts() = 0;
	virtual void disableInterrupts() = 0;
	virtual void enableInterrupt(unsigned number) = 0;
	virtual void disableInterrupt(unsigned number) = 0;

	virtual ~InterruptManager();

protected:
	InterruptManager();
	InterruptManager(unsigned numInterrupts);

	virtual unsigned getInterruptNumber() = 0;
	virtual void ackInterrupt(unsigned number) = 0;

	Interrupt **interrupts;
	unsigned numInterrupts;
};

#endif
