/***********************************
*EthernetDriverLKM                 *
*Ethernet driver "implementation   *
*Actual implementation is provided *
*by the loaded Linux Kernel Module *
***********************************/
//Scott Kircher
//CS424
//LKM Adaptor for Choices Project
//a.k.a. "TooManyChoices"
//a.k.a. "Frankenstein"

#ifndef __ETHERNETDRIVERLKM_H__
#define __ETHERNETDRIVERLKM_H__


#ifdef __GNUC__
#pragma interface
#endif

#include 	<sys/stropts.h>
#include    	"LinuxNetStructures.h"
#include	"EthernetDriver.h"
#include	"Ethernet.h"
#include	"NetworkBuffer.h"
#include    	"LKMAdaptor.h"

typedef void (*LKMRxFunc) (LinuxNetStructures::sock *sk, int len);

class Semaphore;
class NetworkBufferFreeList;

extern ClassStar EthernetDriverLKMClass;
class EthernetDriverLKM : public EthernetDriver, protected LKMAdaptor 
{
protected:
	
	//stuff to emulate the Linux register_netdev and friends
	static LinuxNetStructures::net_device *_device_info; 
	//currently we allow just one, but this normally would be a linked list
	static int adaptor_register_netdev(LinuxNetStructures::net_device *device);
	static void adaptor_unregister_netdev(LinuxNetStructures::net_device *device);
	static void adaptor_ether_setup(LinuxNetStructures::net_device *device);
	static int adaptor_dev_get(const char *name);
	static LinuxNetStructures::sock *adaptor_netlink_kernel_create(int unit, void(*input)(LinuxNetStructures::sock *sk, int len));
    static int adaptor_netlink_broadcast(LinuxNetStructures::sock *ssk, LinuxNetStructures::sk_buff *skb, unsigned int pid, int nonblock);
	static LinuxNetStructures::sk_buff* adaptor_skb_dequeue();
	static struct LinuxNetStructures::sk_buff * adaptor_skb_clone (struct LinuxNetStructures::sk_buff * skb, int gfp_mask);
	static void adaptor_kfree_skb (LinuxNetStructures::sk_buff * skb);
	static struct LinuxNetStructures::sk_buff * adaptor_skb_realloc_headroom(LinuxNetStructures::sk_buff *skb, unsigned int headroom);
	static void adaptor_netif_rx(LinuxNetStructures::sk_buff* skb);
	static void adaptor_atomic_sub();
	static unsigned short adaptor_eth_type_trans(struct LinuxNetStructures::sk_buff *skb, struct LinuxNetStructures::net_device *dev);
	static void adaptor_out_of_line_bug(int line);
	static volatile unsigned long adaptor_jiffies;
	static void adaptor_sock_release(void *sock);
	static LinuxNetStructures::sk_buff*		_skbRx;
	static LKMRxFunc	_lkmRecvFunc;
	
	int initDLPI();

	int			_fd;
	int			_serverSocket;
	int			_flags;
	strbuf _data;
	
	//Semaphore 		_rxInterrupt;


	bool		connectToLocalServer(); 

private:
	int		doTransmit(NetworkBuffer *);
	LinuxNetStructures::sk_buff* convertBuffer(NetworkBuffer *buf);

protected:
	unsigned int		_inputPackets;
	unsigned int		_outputPackets;
	unsigned int		_interrupts;
	bool _ready;

public:
	Class * classOf();

	EthernetDriverLKM(const char *LKMmodulename,unsigned char debugMask=0,NetworkBufferFreeList* nf =0);
	virtual ~EthernetDriverLKM();

	bool    isReady() {return _ready;}

	void getDescriptors(int *file, int *server)
	{
		*file = _fd;
		*server = _serverSocket;
	}

	virtual void interruptHandler();	// The interrupt handler */
	virtual void printInfo();		// Print stats to console

	virtual int 	transmitPacket(NetworkBuffer *);// To send a packet
	virtual int 	transmitPacketAsync(NetworkBuffer *);

	//multicast not currently supported
	virtual int	addMulticast(EthernetAddress *) {return 0;}
	virtual int	removeMulticast(EthernetAddress *) {return 0;}
};


#endif /* ! LKMEthernet_h */
