/*****************************
*LKMAdaptor                  *
*Uses ELFLoader to load an   *
*ELF binary version of a     *
*Linux Kernel Module, and    *
*provides the (static) syms  *
*that the LKM expects to     *
*link with(i.e. kernel calls)*
*Also provides the "member"  *
*interface for Choices.      *
*There should probably be one*
*Of these classes for each   *
*kind of driver base class,  *
*that provides the interface *
*that Choices expects for the*
*particular driver.          *
******************************
*Linux adaptor for Choices   *
*Project, CS423, Spring 2004 *
*****************************/
//SCOTT KIRCHER


#ifndef LKMAdaptor_h
#define LKMAdaptor_h

#ifdef __GNUC__
#pragma interface
#endif

#include "Assert.h"
#include "ELFLoader.h"

//because we plan to use multiple inheretence to
//derive classes that are children of both LKMAdaptor
//and some Driver class, we cannot have LKMAdaptor
//be a child of Object

class LKMAdaptor {
private:
	//static kernel interface adaptor functions, etc...
	//each of these is designed to "look like" a kernel
	//function or object that the LKM expects to be able
	//to link with:
	static int adaptor_printk(char *s, unsigned x1, unsigned x2, unsigned x3, unsigned x4);
	static void *adaptor_memcpy(void *s1, const void *s2, int n); 
	static void *adaptor_kmalloc(int size, int flags);
	static void adaptor_bzero(char *s,int n);
	static void adaptor_kfree (const void *objp);
	static void *adaptor_memset(void *s, int c, int n);
	static int adaptor_sprintf(char * buffer, const char *format, unsigned x1, unsigned x2, unsigned x3, unsigned x4); 
	static void adaptor_set_bit (int nr, volatile void * addr);
	static void adaptor_clear_bit (int nr, volatile void * addr);

	//exported symbol table (basically contains names of ELF symbols and the address that should go with it)
	//this will be used in a call to match(), so can use wildcards: *
	//pass an array of these in to the constructor to perform undefined symbol linking
	static const int number_of_exported_symbols;
	static ELFLoader::ExportTuple exported_symbols[];

	//specialized exported symbol table (so child classes can add exported symbols as well)
	int number_of_special_exported_symbols;
	int special_exported_symbols_storage_size;
	ELFLoader::ExportTuple *special_exported_symbols;

	//required symbol table (this contains a list of names for symbols that are required
	//to be defined in the LKM (such as init_module, cleanup_module). They will be found
	//and added to this table when the module is loaded
	int number_of_required_symbols;
	int required_symbols_storage_size;
	ELFLoader::ImportTuple *required_symbols;

protected:
	//child classes need to be able to register new required symbols
	void addRequiredSymbol(char *name,void **who_wants_it);

	//child classes need to be able to register new exported symbols
	void addExportedSymbol(char *name,void *where_is_it);

	//and load an ELF file
	int loadELF(const char * moduleName );

//specific required LKM symbols
//child classes will expand these considerably,
//but every LKM must have an init_module and cleanup_module
	void (*init_module)();
	void (*cleanup_module)();

public:
	LKMAdaptor(const char * moduleName ); //init and load module immediately
	LKMAdaptor(); //init but don't load a module
	~LKMAdaptor();

//Choices interface functions:
	void initModule(); 
	void cleanupModule();
};

#endif /*LKMAdaptor_h*/
