/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef AssocTable_h
#define AssocTable_h


#ifdef __GNUC__
#pragma interface
#endif

#include "MultiHashTable.h"


// Provide string,number associations, also number,number
// associations. 

typedef void (*PFIVUV)(int,void*,unsigned int);

    class _at_entry : public _mht_entry {
      protected:
	unsigned int _value;
	unsigned int _copied;
      public:
	_at_entry(int i, unsigned int v);
	_at_entry(char* i, unsigned int v, unsigned copied);
	_at_entry(void* i, unsigned int v);
	~_at_entry();

	unsigned int value();
	void inspect();
    };

    class _at_list : public _mht_list {
      public:
	virtual void mapcar(PFLV f);
	~_at_list() {};
    };

    class _at_table : public _mht_table {
      public:
	_at_table(HintList * t, int _n) : _mht_table(t,_n) {};
	virtual ~_at_table() {};
	virtual HintList * operator[](int i);
    };

class AssocTable : public MultiHashTable {
  protected:
    
  public:
    AssocTable( int size, PFIVUV f=0);
    ~AssocTable();
    
    // For bind and lookup, the return value indicates whether the
    // operation failed. If so, the reason is the absence of the
    // string or number in the table.
    
    int bind( char * string, unsigned int value );
    int unbind( char * string, unsigned int& value , int n=0 );
    int lookup( char * string, unsigned int & value , int n=0 );

    int bind( int number, unsigned int value );
    int unbind( int number, unsigned int& value , int n=0 );
    int lookup( int number, unsigned int & value , int n=0 );
    
    int bind( void * pointer, unsigned int value );
    int unbind( void * pointer, unsigned int& value , int n=0 );
    int lookup( void * pointer, unsigned int & value , int n=0 );

};

#endif /* AssocTable_h */
