/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Lee Lup Yuen (lup@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * StringOutputStream.h
 */
 
#ifndef StringOutputStream_h
#define StringOutputStream_h

#ifdef __GNUC__
#pragma interface
#endif


#include "FormatAssists.h"

static const stringOutputStreamWriteBufferSize = 200;

class StringOutputStream
{
    /* A StringOutputStream is for performing formatted writes to a string.
     * The object is created without any arguments, then several formatted
     * writes may be called on the object, and the resulting string can
     * retrieved using the str() method.  The StringOutputStream is
     * restricted to length 200.
     */
public:
    /* Method formattedWrite: Write a1 to a10 using specified format into
         the string buffer.  Works exactly like sprintf. */
    int formattedWrite (const char * format,
	const formatArgs & a1 = *(formatArgs *)0,
	const formatArgs & a2 = *(formatArgs *)0,
	const formatArgs & a3 = *(formatArgs *)0,
	const formatArgs & a4 = *(formatArgs *)0,
	const formatArgs & a5 = *(formatArgs *)0,
	const formatArgs & a6 = *(formatArgs *)0,
	const formatArgs & a7 = *(formatArgs *)0,
	const formatArgs & a8 = *(formatArgs *)0,
	const formatArgs & a9 = *(formatArgs *)0,
	const formatArgs & a10= *(formatArgs *)0);

    /* Method str: Return a pointer to the string buffer. */
    char *str ();

private:
    //  formattedWrite uses writeBuffer to buffer the characters written.
    char writeBuffer [stringOutputStreamWriteBufferSize];
    int writeBufferIndex;

    int doprnt (const char *format, const formatArgs *args);
    void putChar (char ch);
    int convert (/* double,*/ int, int, char *, unsigned char, char *, char *);
    char *round (/* double, */ int *, char *, char *, char, char *);
    char *exponent (char *, int, unsigned char);
    char *memchr (char *, char, int);
    double modf (/* double, */ double *);
    double doubleParameter;  // Kludge: variable used for passing parameter.
};                           //         Read comments in StringOutputStream.cc.

#endif
