/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Aamod Sane sane@cs.uiuc.edu
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef Partition_h
#define Partition_h

#ifdef __GNUC__
#pragma interface
#endif

#include "BitArray.h"


// range bits indicate that a range is allocated.
// screen delimits the ranges by changing value for adjacent ranges
// range == 1 and screen == 0|1 indicates allocation
// range == 0 and screen == 1 is illegal and should never occur

class BitRanges {
  protected:
    
    BitArray _range;
    BitArray _screen;
    
    int borderSearch(const unsigned pos, int dir);
    
  public:
    
    BitRanges(int n);
    virtual ~BitRanges();
    
    int alloc(const unsigned pos);
    int alloc(const unsigned from, const unsigned to);
    
    int free(const unsigned pos);
    int free(const unsigned from, const unsigned to);
    
    int isFree(const unsigned pos);
    int isFree(const unsigned from, const unsigned to);
    
    int inRange(const unsigned pos, unsigned& from, unsigned& to);
    
    virtual void inspect();
};

inline
BitRanges::BitRanges(int n)
: _range(n), _screen(n)
{
}

inline
BitRanges::~BitRanges()
{
}

inline int
BitRanges::isFree(const unsigned pos)
{
    return !_range.get(pos);
}

inline int
BitRanges::isFree(const unsigned from, const unsigned to)
{
    return !_range.get(from,to);
}

#endif
