/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef HashTable_h
#define HashTable_h

#ifdef __GNUC__
#pragma interface
#endif


#include "GPInclude.h"
#include "Link.h"
#include "List.h"

// Abstract hash table defined for Link subclasses that have a hash()
// function. An interface can define a proper subclass and hash
// functions for numbers, strings and pointers.

    class _ht_table {
      protected:
	unsigned _numBuckets;
	HintList* __table;
      public:
	_ht_table(HintList * _t, unsigned _n) { __table = _t; _numBuckets = _n; };
	virtual ~_ht_table() {
            if (__table)
                delete[] __table;
            __table = 0;
	    }
	virtual HintList* operator[](int i);
    };

class HashTable {
    unsigned nextPrime( unsigned from );
    int isPrime( unsigned number );
    int divides( unsigned divisor, unsigned number );
  protected:
    
    // Internal classes and variables get prepended by underscore
    _ht_table* _table;

    unsigned    _numBuckets;
    PFLV   _destructFunction; // Function called on all elements when
			      // destructing, if defined in ctor

    HashTable(int size, PFLV f, int derived);
  public:
    // Estimated size or nearabouts.
    HashTable(int size, PFLV f=0);
    virtual ~HashTable();
    
    // find and remove will return a pointer to the found/removed link
    // resp. or 0 for failure. Note,of course, that the Link class
    // defined such a member function
    
    virtual void insert( Link * entry );
    // For nonunique members, search for the nth element. The first
    // element is 0
    virtual Link * find( Link * entry, int n=0 );
    virtual Link * remove( Link * entry, int n=0 );
    virtual void mapcar(PFLV f); // PFLV defined in Link.h

    virtual void inspect();
};

#endif
