/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef LightHashTable_h
#define LightHashTable_h

#ifdef __GNUC__
#pragma interface
#endif

#include "GPInclude.h"
#include "LightList.h"


// Abstract hash table defined for SLink subclasses that have a hash()
// function. An interface can define a proper subclass and hash
// functions for numbers, strings and pointers.

class LightHashTable;

    class HashVirtualiser : public LightListVirtualiser {
	LightHashTable* _t;
      public:
	HashVirtualiser( LightHashTable * h ) { _t = h; };
	virtual ~HashVirtualiser() {};
	virtual int compare(SLink * a, SLink * b);
	virtual void destroy(SLink * a);
	virtual void inspect(SLink * a);
	virtual SLink * end(SLink *);
    };
    
    // Internal classes and variables get prepended by underscore
    class _lht_table {
      protected:
	unsigned _numBuckets;
	LightList* __table;
      public:
	_lht_table(LightList * _t, int _n) { __table = _t; _numBuckets = _n; };
	virtual ~_lht_table() {
	    if (__table)
              delete __table;
	    __table = 0;
            }
	virtual LightList* operator[](int i);
    };

class LightHashTable {
    friend class HashVirtualiser;
    unsigned nextPrime( unsigned from );
    int isPrime( unsigned number );
    int divides( unsigned divisor, unsigned number );
  protected:
    
    _lht_table* _table;

    unsigned    _numBuckets;
    PFSV   _destructFunction; // Function called on all elements when
			      // destructing, if defined in ctor
    
    // Virtualiser is used to virtualise the functions for the
    // lightlist. What it actually does is to dispatch these to the
    // functions defined in the hashtable. Subclasses of hashtable
    // never have to bother about the virtualiser again.
    HashVirtualiser* _v;
    
    virtual int hash( SLink * entry, int buckets );
    virtual int compare( SLink * a, SLink * b );
    virtual void deleteEntry( SLink * entry );
    virtual void inspectEntry( SLink * entry );
    virtual SLink * listType(SLink *);

  public:
    // Estimated size or nearabouts.
    LightHashTable(int size, PFSV f=0);
    LightHashTable(int size, PFSV f, int derived);
    virtual ~LightHashTable();
    
    // find and remove will return a pointer to the found/removed link
    // resp. or 0 for failure. Note,of course, that the SLink class
    // defined such a member function
    
    virtual void insert( SLink * entry );
    // For nonunique members, search for the nth element. The first
    // element is 0
    virtual SLink * find( SLink * entry, int n=0 );
    virtual SLink * remove( SLink * entry, int n=0 );
    virtual void mapcar(PFSV f); // PFSV defined in SLink.h

    virtual void inspect();
};

#endif
