/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: See-Mong Tan (stan@cs.uiuc.edu)
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * Link.h:  Linked lists
 *
 *	$Header: /projects/choices/old/cvsroot/choices/Includes/Libraries/GeneralPurpose/Link.h,v 1.1 2004/06/03 01:58:43 jcarlyle Exp $
 */

#ifndef Link_h
#define	Link_h

#ifdef __GNUC__
#pragma interface
#endif


#include "GPInclude.h"

// Testing Link - Links and lists get together by the program Lists.cc
// Tests, solutions etc. are all in Applications/UNIXGP

class SLink;

// Used to provide virtual functions without using the vtable pointer.
// A virtualiser has virtual functions, vtable pointer and is passed
// explicitly as a parameter. Functions from the Virtualiser are then
// called instead of directly calling class functions. A virtualiser
// serves as a explicit vtable pointer.

class LinkVirtualiser {
  public:
    virtual int compare( SLink *, SLink * );
    virtual void destroy( SLink * );
    virtual void inspect( SLink * );
    virtual int hash( SLink *, int );
    virtual SLink * end( SLink * l );

    // squash g++ 4.0 warning
    virtual ~LinkVirtualiser() {}
};

//
// General assumption for all Links:
// 	Links that do not point anywhere point to themselves
// Assumption for lists
//	By default, lists are closed, i.e. the pointer of the "last"
// 	member points to the first
//

// Single Link
class SLink {
  protected:
    SLink * _next;
  public:
    SLink();
    ~SLink();
    
    SLink * next();
    void setNext( SLink * s );
    

    // methods to manipulate link
    void addAfter( SLink * list ); // Add myself after the list
    void remove( SLink * prev );   // Remove myself from the list,
    				   // splicing neighbours
    
    void inspect();
    
    // List functions: Do not use these, not yet tested. See
    // comparable functions for class List.
    
    SLink * find( SLink * e, LinkVirtualiser * v );
    SLink * find( SLink * e, int nth, LinkVirtualiser * c );
    SLink * nthElem(int n, LinkVirtualiser * v);
    int sublist(int start, int fin, SLink*& a, SLink*& b,
		LinkVirtualiser * v);
    void destroyAll(LinkVirtualiser * v);
    void inspectAll(LinkVirtualiser * v);
    
};

inline
SLink::SLink()
{
    _next = this;
}

inline
SLink::~SLink()
{
}

inline SLink *
SLink::next()
{
    return _next;
}

inline void
SLink::setNext( SLink * s )
{
    _next = s;
}

inline void
SLink::addAfter( SLink * list )
{
    setNext( list->next() );
    list->setNext( this );
}

inline void
SLink::remove( SLink * prev )
{
    prev->setNext( next() );
    setNext( this );
}

class NVLinkVirtualiser : public LinkVirtualiser {
  public:
    virtual void destroy( SLink * );
    virtual void inspect( SLink * );
};

// Non virtual double link
class NVLink : public SLink {
protected:
   NVLink * _prev;
public:
    NVLink();
    ~NVLink();
    
    // methods for accessing
    NVLink * next();
    NVLink * previous();

    // methods for setting fields
    void setNext( NVLink * );
    void setPrevious( NVLink * );

    // methods for adding
    void addBefore( NVLink * );     // add myself before the link
    void addAfter( NVLink * );      // add myself after the link

    void remove();                  // remove myself from the linked
				    // list. Compare with remove for SLink.

    void inspect();
};

inline
NVLink::NVLink()
{
    _next = _prev = this;
}

inline
NVLink::~NVLink()
{
}

inline NVLink *
NVLink::next()
{
    return (NVLink *)_next;
}

inline NVLink *
NVLink::previous()
{
    return _prev;
}

inline void
NVLink::setNext( NVLink * linkObj )
{
    _next = linkObj;
}

inline void
NVLink::setPrevious( NVLink * linkObj )
{
    _prev = linkObj;
}

// Double link with virtual members.
// Virtualisers are not necessary for subclasses

class Link : public NVLink {
  public:
    Link();
    virtual ~Link();
    
    Link * next();
    Link * previous();
    
    virtual int compare(Link * l);	// compare pointers to self
    virtual void inspect();		// print contents
    virtual int hash(int);		// hash pointer to self.
					// Useful for the HashTables.

    // List functions: without the virtualiser.
    // Do not use now - untested. See comparable functions for class
    // List.
    
    virtual Link * find( Link * e );
    virtual Link * find( Link * e, int nth );
    virtual Link * nthElem(int n );
    virtual int sublist(int start, int fin, Link*& a, Link*& b);
    virtual void destroyAll();
    virtual void inspectAll();
    // is list circular
    virtual Link * end( Link * l );
};

inline Link *
Link::next()
{
    return (Link*)NVLink::next();
}

inline Link *
Link::previous()
{
    return (Link*)NVLink::previous();
}

#endif /*Link_h*/
