/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef MultiLevelTable_h
#define MultiLevelTable_h

#ifdef __GNUC__
#pragma interface
#endif


class MultiLevelTable
{
	int _log2ChunkSize;
	int _chunkSize;
	int _chunkMask;

	int _log2IndexSize;
	int _indexSize;
	int _indexMask;

	int _shiftCount;
	int _maxForLevel;
        int _numberOfLevels;

        int ** _root;

protected:
	int ** newIndex();
	int * newChunk();
	void deleteLevel( int ** index, int level );

	void addLevel();

	int log2ChunkSize();
	int log2IndexSize();
public:
        MultiLevelTable( int log2ChunkSize = 0, int log2IndexSize = 0 );
	~MultiLevelTable();

	int at( int i );
	void atPut( int i, int value );
	int * where( int i, int alloc );
};

inline int MultiLevelTable::log2ChunkSize()
{
	return _log2ChunkSize;
}

inline int MultiLevelTable::log2IndexSize()
{
	return _log2IndexSize;
}

inline void MultiLevelTable::atPut( int i, int v )
{
        *( where( i, 1 ) ) = v;
}

inline int MultiLevelTable::at( int i ) 
{
	int * p = where( i, 0 );
	return ( p ? *p : 0 );
}

class BitsMultiLevelTable : public MultiLevelTable
{
        int _log2EntrySize;
	int _log2EntriesPerWord;
	int _log2EntriesPerChunk;
	int _entryMask;
	int _entriesPerWordMask;
	int _entriesPerChunkMask;

public:
        BitsMultiLevelTable( int log2EntrySize, int log2EntriesPerChunk = 0,
			     int log2IndexSize = 0 );
	int at( int i );
	void atPut( int i, int value );
};

inline int BitsMultiLevelTable::at( int i )
{
        int * addr = where( i >> _log2EntriesPerWord, 0 );
	if ( addr == 0 ) {
		return 0;
		}
	else {
		int shifts = ( i & _entriesPerWordMask ) << _log2EntrySize;
		return (( *addr >> shifts ) & _entryMask);
	}
}

inline void BitsMultiLevelTable::atPut( int i, int v )
{
        int * addr = where( i >> _log2EntriesPerWord, 1 );
        int shifts = ( i & _entriesPerWordMask ) << _log2EntrySize;
        *addr = ( *addr & ~( _entryMask << shifts ) ) | 
	        ( ( v & _entryMask ) << shifts );
}

#endif /*MultiLevelTable_h*/

