/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * NVList.h - List varieties
 *
 *	$Header: /projects/choices/old/cvsroot/choices/Includes/Libraries/GeneralPurpose/NVList.h,v 1.1 2004/06/03 01:58:43 jcarlyle Exp $
 */

#ifndef NVList_h
#define NVList_h

#ifdef __GNUC__
#pragma interface
#endif


#include "Link.h"
#include "LightList.h"

//
// A doubly linked list without using virtual functions
// We can use most of the LightList, modify the basic
// add/remove to do the right thing to the second link
// and we are in business. See LightList for further comments
//

class NVListVirtualiser : public LightListVirtualiser {
  public:
    virtual void destroy( SLink * );
    virtual void inspect( SLink * );
    virtual void add( LightList * l,
		     SLink * newEntry, SLink * oldEntry, int head = 0 );
    virtual SLink * remove( LightList * l, 
		     SLink * oldEntry, int nth=0);
};

class NVList : public LightList {
  public:
    NVList();
    NVList(NVLink * l);
    ~NVList();

    // add which takes care of the second link
    void add( SLink * newEntry, SLink * oldEntry, int head = 0 );
    // remove which takes care of the second link
    SLink * remove( SLink * oldEntry, int nth=0, NVListVirtualiser * v=0);
};

inline
NVList::NVList()
: LightList()
{
}

inline
NVList::NVList( NVLink * l )
: LightList(l)
{
}

inline
NVList::~NVList()
{
}

#endif	/* NVList_h */
