/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Contributing Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * String.h
 */

#ifndef String_h
#define	String_h

#ifdef __GNUC__
#pragma interface
#endif



#if (!defined(SPARC) && !defined(linux) && !defined(__APPLE__) && !defined(ARM))
#ifndef CFRONT
	/*
	 * cfront can't handle inline loops
	 */

inline int
strlen( const char * s )
{
	const char *p = s;
	for ( ; *p; p++ ) ;
	return p - s;
}

/*
 * Copy s2 to s1.
 * Return s1.
 */
inline char *
strcpy( char * s1, const char * s2 )
{
	for ( char * p = s1; *p = *s2; p++, s2++ ) ;
	return s1;
}

/*
 * Copy s2 to s1, copying at most n characters.
 * Return s1.
 */

inline char *
strncpy( char * s1, const char * s2, int n )
{
	for ( char * p = s1; ( n > 0 ) && ( *p = *s2 ); p++, s2++, n-- ) ;
	return s1;
}

/*
 * Compare s1 to s2.
 * Return negative/zero/positive if s1 is lexically less/equal/greater s2.
 */
inline int
strcmp( const char * s1, const char * s2 )
{
	for ( ; *s1 && *s2 && ( *s1 == *s2 ); s1++, s2++ ) ;
	return *s1 - *s2;
}

/*
 * Compare s1 to s2, comparing at most n characters.
 * Return negative/zero/positive if s1 is lexically less/equal/greater s2.
 */

inline int
strncmp( const char * s1, const char * s2, int n )
{
	for ( ; n > 0; s1++, s2++, n-- )
		if ( ! ( *s1 && *s2 && ( *s1 == *s2 ) ) )
			return *s1 - *s2;
	return 0;
}

/*
 * Return pointer to c in s, if present.
 * Null, otherwise.
 */
inline char *
index( const char * s, char c )
{
	for ( ; *s; s++ )
		if ( *s == c )
			return (char *) s;
	return (char *) 0;
}

/*
 * Return pointer to last c in s, if present.
 * Null, otherwise.
 */
inline char *
rindex( const char * s, char c )
{
	char * ret = 0;
	for ( const char * p = s; *p; p++ )
		if ( *p == c )
			ret = (char *)p;
	return ret;
}

/*
 * Replace leftmost occurrence of c in string with a null character.
 * (Primarily intended for chopping newlines from input strings.)
 * Return string.
 */
inline int
strprefix( char * str, char * prefix )
{
	int len = strlen( str );

	int ret;
	if( len == 0 ) 
		ret = strcmp( str, prefix );
	else
		ret = strncmp( str, prefix, len );
	return ret;
}


/*
 * Append s2 to s1.
 * Return s1.
 */
inline
char * strcat( char * s1, const char * s2 )
{
	char * p = s1 + strlen( s1 );
	(void) strcpy( p, s2 );
	return s1;
}

#else
int strlen( const char * s );
char * strcpy( char * s1, const char * s2 );
char * strncpy( char * s1, const char * s2, int n );
int strcmp( const char * s1, const char * s2 );
int strncmp( const char * s1, const char * s2, int n );
char * index( const char * s, char c );
char * rindex( const char * s, char c );
#endif /*CFRONT*/
#else /*SPARC*/
#include <string.h>
inline int
strprefix( char * str, char * prefix )
{
	int len = strlen( str );

	int ret;
	if( len == 0 ) 
		ret = strcmp( str, prefix );
	else
		ret = strncmp( str, prefix, len );
	return ret;
}

/*
 * Replace leftmost occurrence of c in string with a null character.
 * (Primarily intended for chopping newlines from input strings.)
 * Return string.
 */
inline
char * chop( char * s, char c )
{
	char * pos = strchr( s, c );
	if( pos != 0 )
		*pos = '\0';
	return( s );
}
#endif /*SPARC*/

#endif /*String_h*/
