/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Lee Lup Yuen (lup@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * StringInputStream.h
 */
 
#ifndef StringInputStream_h
#define StringInputStream_h

#ifdef __GNUC__
#pragma interface
#endif


#include "FormatAssists.h"

static const unsigned stringInputStreamBufferSize = 200;  //  Read-back buffer size.

class StringInputStream
{
    /* A StringInputStream lets the user perform formatted reads from a
     * string.  The object records the number of characters read for
     * each formatted read, so that consecutive formatted reads will
     * continue from where the previous read stopped.
     */
public:
    StringInputStream (char *str);
    ~StringInputStream ();

    /* Method formattedRead: Read from string into arguments according to
         format.  Works exactly like sscanf. */
    int formattedRead (char * format,
	const formatArgs & a1 = *(formatArgs *)0,
	const formatArgs & a2 = *(formatArgs *)0,
	const formatArgs & a3 = *(formatArgs *)0,
	const formatArgs & a4 = *(formatArgs *)0,
	const formatArgs & a5 = *(formatArgs *)0,
	const formatArgs & a6 = *(formatArgs *)0,
	const formatArgs & a7 = *(formatArgs *)0,
	const formatArgs & a8 = *(formatArgs *)0,
	const formatArgs & a9 = *(formatArgs *)0,
	const formatArgs & a10= *(formatArgs *)0);

    /* Method reclaim: Move the remaining characters not used by previous
         formattedReads into buffer, subject to a maximum of bufferSize
	 characters. */
    int reclaim (char *buffer, int bufferSize);

private:
    int _doscan (char *, int **);
    int _innum (int **, int , int , int , int*);
    int _instr (char *,  int , int , int*);
    char * _getccl (char *);
    int getc ();
    char ungetc (char);

    char *readBuffer;
    int readBufferIndex;
    // ungetc characters placed here.
    char readBackBuffer [stringInputStreamBufferSize + 1];
    int readBackBufferIndex;
};

#endif
