/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * FileSystem.h - file system error condition constants
 */

#ifndef FileSystem_h
#define FileSystem_h


#ifdef __GNUC__
#pragma interface
#endif

enum Status {
    Success = 0,
    BadUnixFilename,
    CantAcceptSocket,
    CantConnectToSocket,
    CantCreateDot,
    ContainerFull,
    DifferentContainers,
    DirectoryBusy,
    DirectoryHasKeys,
    DirectoryInvalid,
    EmptyFd,
    EndOfFile,
    FileAccessViolation,
    FileExists,
    IdNumberOutOfRange,
    InvalidArgument,
    InvalidPath,
    KeyExists,
    KeyNotFound,
    KeyTooSmall,
    KeyTooBig,
    MemoryObjectFull,
    MountPointBusy,
    NotAProgram,
    NoSuchContainer,
    NoSuchFd,
    NoSuchFile,
    NotEndOfFile,
    NotImplemented,
    NotSpecial,
    NumberOfArguments,
    ObjectNotMounted,
    OperationNotSupported,
    OutOfMemory,
    OutOfSpace,
    PartialRecordRead,
    ProtocolMismatch,
    ReadError,
    RecordExists,
    RecordNotFound,
    RecordSizeMismatch,
    SocketReadError,
    SocketWriteError,
    TooManyFiles,
    TooManyKeys,
    TooManySymbolicLinks,
    UnbalancedParentheses,
    UnknownNFSOp,
    UnusedIdNumber,
    WriteError,
    Zz
};

extern const char * StatusCodes[];

const int AccessExecute = 01;
const int AccessWrite = 02;
const int AccessRead  = 04;
const int AccessOwner = 010;
const int AccessMask  = AccessExecute|AccessWrite|AccessRead|AccessOwner;

const int O_RDONLY  = 0;
const int O_WRONLY  = 1;
const int O_RDRW    = 2;
const int O_NDELAY  = 4;
const int O_APPEND  = 8;

const int O_CREAT   = 0x0100;
const int O_TRUNC   = 0x0200;
const int O_EXCL    = 0x0400;

const int O_NFOLLOW = 0x1000;
const int O_NSYNC   = 0x2000;

const int O_NCACHE  = 0x80000000;

const int O_C8K     = 0x10000000;
const int O_C16K    = 0x20000000;
const int O_C32K    = 0x30000000;
const int O_C64K    = 0x40000000;
const int O_C128K   = 0x50000000;
const int O_C256K   = 0x60000000;
const int O_CSIZE   = 0x70000000;

const int O_CMASK   = 0x70000000;

enum PersistentObjectType {
    UnknownType = 0,
    PersistentArrayType = 1,
    MemoryObjectDictionaryType = 2
};

#endif /*FileSystem_h*/
