/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Contributing Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Contributing Author: David Dykstra (dykstra@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * Time.h - Time class; just a place holder for subclasses.
 */

#ifndef Time_h
#define Time_h

#include "OutputStream.h"

class TimeValue {
	int _sec;
	int _usec;
public:
	TimeValue();
	TimeValue( int sec, int usec );
	int sec();
	int usec();
	unsigned time();
	void setSec( int sec );
	void setUsec( int usec );
	void setTime( unsigned usec );
	TimeValue operator+( const int usec );
	TimeValue operator-( const int usec );
	TimeValue operator+( const TimeValue & t );
	TimeValue operator-( const TimeValue & t );
	TimeValue & operator+=( const int usec );
	TimeValue & operator-=( const int usec );
	TimeValue & operator+=( const TimeValue & t );
	TimeValue & operator-=( const TimeValue & t );
};

inline 
TimeValue::TimeValue() 
{
	_sec = 0; _usec = 0;
}

inline
TimeValue::TimeValue( int sec, int usec )
{
	_sec = sec; _usec = usec;
}

inline
int TimeValue::sec()
{
	return _sec;
}

inline
int TimeValue::usec()
{
	return _usec;
}

inline
OutputStream & operator<< ( OutputStream & s, TimeValue  t )
{
	s << t.sec() << " secs " << t.usec() << " usecs";
	return s;
}

inline
unsigned TimeValue::time()
{
	return _sec * 1000000 + _usec;
}

inline
void TimeValue::setSec( int sec )
{
	_sec = sec;
}

inline
void TimeValue::setUsec( int usec )
{
	_usec = usec;
}

inline
void TimeValue::setTime( unsigned usec )
{
	_sec = usec / 1000000;
	_usec = usec % 1000000;
}

inline
TimeValue TimeValue::operator+( const int usec )
{
	TimeValue ret( *this );
	ret._usec += usec;
	if ( ret._usec >= 1000000 ) {
		ret._sec++;
		ret._usec -= 1000000;	
	}
	return ret;
}

inline
TimeValue TimeValue::operator-( const int usec )
{
	TimeValue ret( *this );
	ret._usec -= usec;
	if ( ret._usec < 0 ) {
		ret._sec--;
		ret._usec += 1000000;	
	}
	return ret;
}

inline
TimeValue TimeValue::operator+( const TimeValue & t )
{
	TimeValue ret( *this );
	ret._usec += t._usec;
	ret._sec += t._sec;
	if ( ret._usec >= 1000000 ) {
		ret._sec++;
		ret._usec -= 1000000;	
	}
	return ret;
}

inline 
TimeValue TimeValue::operator-( const TimeValue & t )
{
	TimeValue ret( *this );
	ret._usec -= t._usec;
	ret._sec -= t._sec;
	if ( ret._usec < 0 ) {
		ret._sec--;
		ret._usec += 1000000;
	}
	return ret;
}

inline
TimeValue & TimeValue::operator+=( const int usec )
{
	_usec += usec;
	if ( _usec >= 1000000 ) {
		_sec++;
		_usec -= 1000000;
	}
	return *this;
}

inline
TimeValue & TimeValue::operator-=( const int usec )
{
	_usec -= usec;
	if ( _usec < 0 ) {
		_sec--;
		_usec += 1000000;
	}
	return *this;
}

inline
TimeValue & TimeValue::operator+=( const TimeValue & t )
{
	_usec += t._usec;
	_sec += t._sec;
	if ( _usec >= 1000000 ) {
		_sec++;
		_usec -= 1000000;
	}
	return *this;
}

inline
TimeValue & TimeValue::operator-=( const TimeValue & t )
{
	_usec -= t._usec;
	_sec -= t._sec;
	if ( _usec < 0 ) {
		_sec--;
		_usec += 1000000;
	}
	return *this;
}


#endif /* Time_h */
