
#ifndef __COMMON__
#define __COMMON__

#include "ProtectionLevel.h"
#include "FormatAssists.h"
#include "ClassStar.h"
#include "PersistentClassStar.h"
#include "Assert.h"
class ProxiableObject;
class OutputStream;
class InputStream;
class SystemInterface;
class FileSystemInterface;
class NameServer;

/*
 * A pointer to a function taking an object of a Class.
 * Used by the applyTo... methods to do something to each object.
 */
typedef void (*ClassApplicationFunction)(ProxiableObject *);

extern ClassStar NullClass;
extern OutputStream * StandardOutput;
extern InputStream * StandardInput;
extern SystemInterface * StandardSystemInterface;
extern FileSystemInterface * StandardFileSystemInterface;
extern NameServer * StandardNameServer;

typedef void (*PFV)(void *);
typedef unsigned int Offset;
typedef ProxiableObject *(*PConstructor)(char *, unsigned int, unsigned int,
						unsigned int, unsigned int);
typedef ProxiableObject * (*POFPO)( ProxiableObject * );
typedef void (*ClassApplicationFunction)(ProxiableObject *);

#define proxiable
#define Console() (*StandardOutput)

enum ProcessState {
  ProcessNew= 0,
  ProcessReady = 1,
  ProcessRunning = 2,
  ProcessBlocked = 3,
  ProcessDead = 4
};

typedef int StackSize;
const StackSize heapBootStack = -1;
const StackSize none=0;
const StackSize tiny=2;
const StackSize small=4;
const StackSize normal=8;
const StackSize application=16;
const StackSize large=32;
const StackSize huge=64;
const StackSize gigantic=128;
	
enum Return_Code {
	OK = 0,
	ERROR = -1,
			// Just generally wrong. Unspecified error.
	ERROR_IN_SPECIFICATION = -2,
			// E.g. illegal protocol specification
	NO_SUCH_ADDRESS = -3,
			// The given handle doesn't specify anything
	NO_SUCH_CONDUIT = -4,
			//       "            "            "
	NO_SUCH_DESTINATION = -5,
			//       "            "            "
	NO_SUCH_MESSAGE = -6,
			//       "            "            "
	NO_SUCH_CONNECTION = -7,
			// TCP error
	PORT_IN_USE = -8,
			// E.g. passive open on a port that someone else
			//   is already connected to.
	PROTOCOL_IN_USE = -9,
			// For Ethernet, specifies that someone else wants
			//   packets with that ethernet type/protocol field
	ILLEGAL_PORT = -10,
			// E.g. sending to a port to which you're not
			//   connected.
	ILLEGAL_OPERATION = -11,
			// For example, trying to connectFromBelow to a
			//   conduit that has no bottom.
	TOO_MANY_CONNECTIONS = -12,
			// I.e. no more space in the tables.
	BUFFER_FULL = -13,
	CHECKSUM_ERROR = -14,
	CONNECTION_CLOSING = -15,
			// TCP error
	CONNECTION_RESET = -16,
			// I.e. refused/aborted by foreign host
	CANNOT_SEND = -17,
			// For instance, if the window is too small to send, or
			//   some other resource is unavailable. It means the
			//   message has _not_ been accepted, so responsibility
			//   for re-trying to send or deleting the thing is
			//   left with the sender.
	CANNOT_DELIVER = -18,
			// For instance, if there is no conduit connected to
			//   this one, insertFromBelow could return this code.
	CANNOT_CONNECT = -19,
			// Conduit unable to connect for unspecified reason
	INVALID_PROTOCOL = -20,
			// Conduit cannot connect to Conduit of that type
	INVALID_MESSAGE_TYPE = -21,
			// Conduit cannot accept messages of that type
			//   (protocol number).
	INVALID_ENCAPSULATION = -22,
			// UDP/TCP message does not have IP message as holder
	NOT_TRANSMITTED = -23,
			// Conduit of Buffer (or whatever) accepted the
			//   message, but was unable to send it out yet
			//   (for example, if the network isn't quite
			//   initialized yet, or a connection is still in
			//   the process of being initialized) and will keep
			//   trying until it is successful. Informational.
	INCOMPLETE_ADDRESS = -24,
			// Message missing (some of) its address, and is
			//   thus not deliverable.
	NOT_FOUND = -25,
			// MachineConfiguration::lookup() uses this
	END_OF_THE_WORLD = -26
			// Some extraordinarily serious error has
			//   occured. Should never be seen.
};
	
#endif
