/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * Allocator.h: definition of a Big Bag-o-Pages heap class.
 * Much of this copied from MadAllocator in the kernel.
 */
/*
 */

#ifndef Allocator_h
#define Allocator_h


#ifdef __GNUC__
#pragma interface
#endif

#include "SemaphoreStar.h"

/*
 * The generic allocator class.
 */
class Allocator {
protected:
public:
	virtual void * allocate( unsigned int nbytes );
	virtual void free( void * cp );
	Allocator();
	virtual ~Allocator();
};

static const int NumberOBuckets = 32;

class BucketAllocator : public Allocator {
protected:
	int		_log2PageSize;
	char *		_baseAddress;
	char *		_upperBoundAddress;
	char *		_pageToBucket;
	SemaphoreStar	_bucketMutex[NumberOBuckets];
	char *		_bucketToPage[NumberOBuckets];
	char		_sizeToBucket[4100];
	char		_sizeToBuckets[1024];
	SemaphoreStar	_topMutex;
	int		_top;
	int		_pageSize;
	int		_pageMask;
	int		_numberOfPages;

	unsigned int	findBucket( unsigned int size );
	char *		moreSpace( int bucket );
public:
	BucketAllocator( void * baseAddress, int numberOfPages,
			int log2PageSize, int noMultiProcessing = 1 );
	~BucketAllocator();
	void		enableMultiProcessing();
	void *		allocate( unsigned int size );
	void		free( void * cp );
};

#endif
