/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef SysCalls_h
#define SysCalls_h


#ifdef __GNUC__
#pragma interface
#endif

extern "C" {

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/time.h>
#include <sys/signal.h>
#include <sys/uio.h>
#include <sys/resource.h>

int access(const char * path, int mode);
int chdir(const char * path);
int chmod(const char * path, mode_t mode);
int chown(const char * path, int owner, int group);
int chroot(const char * path);
int close(int fd);
int creat(const char * path, mode_t mode);
int dup(int fd);
int dup2(int fd1, int fd2);
int execve(char * path, char * argv[], char * envp[] );
int _exit(int i);
int fcntl(int fd, int cmd, int arg);
int fchdir(int fd);
int fchmod(int fd, mode_t mode);
int fchown(int fd, int owner, int group);
int fchroot(int fd);
int flock(int fd, int op);
int fork();
int fstat(int fd, struct stat * buf);
int fsync(int fd);
int ftruncate(int fd, off_t length);
int getdirentries(int fd, const char * buf, int nbytes, long * basep);
int getdtablesize();
int getegid();
int geteuid();
int getgid();
int getgroups(int gidsetlen, int * gidset);
int getpagesize();
int getpid();
int getrusage(int pid, struct rusage *);
int gettimeofday(struct timeval * tp, struct timezone * tzp);
int getuid();
int ioctl(int fd, unsigned long request, caddr_t arg);
int link(char *, char *);
off_t lseek(int fd, off_t offset, int whence);
int lstat(const char * path, struct stat * buf);
int mkdir(const char * path, mode_t mode);
int mkfifo(const char * path, mode_t mode);
int mknod(const char * path, int mode, int dev);
// int open(const char * path, int flags, mode_t mode);
int pipe(int fd[]);
int read(int fd, const char * buf, int nbyte);
int readlink(const char *path, char *buf, int bufsize);
int readv(int fd, struct iovec *iov, int iovcnt);
int rename(const char * oldname, const char * newname);
int rmdir(char * path);
char * sbrk(int);
int setgroups(int ngroups, int * gidset);
int setitimer(int which, struct itimerval * value, struct itimerval * ovalue);
int settimeofday(const struct timeval * tp, const struct timezone * tzp);
int sigaction(int sig, const struct sigaction * act, struct sigaction * oact);
int sigblock(int mask);
int sigpause(int mask);
int sigprocmask(int how, const sigset_t * set, sigset_t * oset);
int stat(const char * path, struct stat * buf);
int symlink (const char *name1, const char *name2);
int sync();
long tell(int fd);
int truncate(const char * path, off_t length);
int unlink(const char * path);
unsigned short umask(unsigned short mask);
int utimes(const char * path, const struct timeval * tvp);
int vfork();
int wait4(int pid, int * statusp, int options, struct rusage * usage);
int write(int fd, const char * buf, int nbyte);
int writev(int fd, struct iovec *iov, int iovcnt);
}
#endif
