/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * UnixApplication.h
 *
 */

//  This file declares general variables used by the Unix application
//  libraries.
 
#ifndef UnixApplication_h
#define UnixApplication_h
#ifdef __GNUG__
#pragma interface
#endif

//#define UA_DEBUG

//  errno stores the Unix error number, while UA_errno stores the Choices
//  equivalent of the the error number.  Unix error numbers are defined in
//  errno.h while Choices error numbers are defined in FileSystem.h.
//  translateErrno converts UA_errno into errno.  Choices applications
//  should never refer to UA_errno explicitly - use errno instead.

extern int errno;
extern int UA_errno;
extern int UA_debug;
extern int UA_initialized;
extern "C" void UA_init();
extern "C" void UA_translateErrno ();
extern "C" int  UA_notImplemented();

extern int UA_negativePid;
extern int UA_lastChild;
extern int UA_sigmask;
extern int UA_umask;
extern int UA_auid;
extern int UA_egid;
extern int UA_euid;
extern int UA_gid;
extern int UA_uid;

#ifdef UA_DEBUG
#define UA_Debug if (UA_debug) Console()
#else
#define UA_Debug if (0) Console()
#endif

//  These are internal variables meant to be used by the Unix libraries
//  only.  Do not use these variables in Choices applications.

class ClassStar;
class FileSystemInterface;
class FileDescriptorTable;

extern ClassStar UA_PersistentArrayStreamClass;
extern FileSystemInterface * UA_FileSystemInterface;
extern FileDescriptorTable * UA_FileDescriptorTable;

extern "C" {
typedef int (* exitproc_t)(int);
extern exitproc_t _exitproc;
extern int _exit(int);
};

extern exitproc_t UA_old_exitproc;

#endif
