/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David Raila (raila@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef Lock_h
#define Lock_h


#ifdef __GNUC__
#pragma interface
#endif

#include "MachineTypes.h"
#include "Assert.h"
class Process;

	/*
	 * BusyWait does spin locking, whatever the interrupt level
	 */
class BusyWait{
	int _held;			// 1 if held, 0 else, used to testandset
	int _state;			// For subclass CPU use only.
	unsigned _holder;			// CPU ID of holder
	Process * _process;		// Process ID of holder
public:
	BusyWait();
	~BusyWait();
	int held(){return _held;}
	int heldByMe();
	void acquire();
	void release();
};

	/*
	 * Lock disables interrupts and spins.
	 */
class Lock : public BusyWait {
	IntStorage _interruptState;
public:
	Lock(){}
	Lock(char*){}
	void	acquire();
	void	release();
};

/**
 * Helper object. Aquires lock when the object is instantiated and releases the
 * lock when the object is destroyed.
 */
class LockKeeper {
    private:
	Lock *_lock;

    public:
	LockKeeper(Lock *lock);
	virtual ~LockKeeper();
};

#endif /*Lock_h*/
