/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801-2910
 *
 * Copyright (c) 1991 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Amit Parghi (parghi@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *	MachineConfiguration.h
 *
 *	Object which encapsulates and stores startup configuration
 *	data for the local machine.
 *
 *	$Id: MachineConfiguration.h 2 2004-06-03 01:58:28Z jcarlyle $
 *
 * $Log: MachineConfiguration.h,v $
 * Revision 1.1  2004/06/03 01:58:40  jcarlyle
 * Initial revision
 *
 * Revision 1.1  1992/05/12  14:53:39  mcgrath
 * Initial revision
 *
 *
 */


#ifndef MachineConfiguration_h
#define MachineConfiguration_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Object.h"

class AssocTable;
class FileSystemInterface;

class MachineConfiguration : public Object {
protected:
    AssocTable *_table;

    char *_identification;
    char * pastEnd;
    char * current;
    const char * base;
    int read( char * );
    int loadFromFile(char *name, FileSystemInterface *);    // read file data
    int readConfiguration(char *, char *);

public:
    MachineConfiguration(FileSystemInterface *,		// pointer to our FSI
			 unsigned short debugMask = 0,	// Raid debug bitmask
			 unsigned int tableSize = 16);	// max size of table
    virtual ~MachineConfiguration();

    int bind(char *key, char *value);		// store {key,value} in table
    int bind(char *key, unsigned int &value);

    int unbind(char *key, int &status);		// remove pair from table

    int lookup(char *key, char *&value);
    char *	identification();	// String where it came from
};

inline 
char *
MachineConfiguration::identification()
{
return _identification;
}

inline int MachineConfiguration::bind(char *key, unsigned int &value)
{
    return bind(key, (char *) value);		// is this wise or even safe??
}



const int MACHINECONFIGURATION_MAXKEYLENGTH = 100;
const int MACHINECONFIGURATION_MAXVALUELENGTH = 100;

#endif /* MachineConfiguration_h */
