/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * GenericARMConsole.h - Sparcstation-1 console class.
 */

#ifndef GenericARMConsole_h
#define GenericARMConsole_h


#ifdef __GNUC__
#pragma interface
#endif


#include "Lock.h"	/* could be a problem since locks use assertions */
#include "OutputStream.h"
#include "GenericARMSerial.h"
#include "Interrupt.h"

extern int consoleInitialized;
extern void rawConsolePrintf(char *);	// Non-locking

class GenericARMConsole : public OutputStream {
	/*
	 * The serial chip we talk to.
	 */
	GenericARMSerial *serialCOM;

protected:
	Lock lowPriorityLock;
	Lock highPriorityLock;

	/*
	 * The parent class expects these!
	 */
	void acquireExclusiveUse();
	void releaseExclusiveUse();
	void acquireHighPriorityUse();
	void releaseHighPriorityUse();

	void emitIdentifierOfCurrentProcessor(){};
	void emitNewline();

public:

	virtual void panic( const char * f,
											const formatArgs & a1 = *(formatArgs *)0,
											const formatArgs & a2 = *(formatArgs *)0,
											const formatArgs & a3 = *(formatArgs *)0,
											const formatArgs & a4 = *(formatArgs *)0 );

	GenericARMConsole(GenericARMSerial* ser);
	~GenericARMConsole();
	void putChar( char c );
	int getChar();

	int nonBlockingGetChar();
	int write( const char * buf, int count );
	void setInterruptMode();
};

extern OutputStream * ConsoleOutputStream;

#endif	/* GenericARMConsole_h */
