/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987-2006
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef GenericARMGdb_h
#define GenericARMGdb_h

#include "Gdb.h"
#include "PROCESSOR.h"

#ifdef __GNUC__
#pragma interface
#endif

class GenericARMGdb: public Gdb {
 public:
	GenericARMGdb();
	virtual ~GenericARMGdb() {}
 protected:
	// ARM Gdb expects 26 words, the B* registers are ignored.
	enum regnames {
		R0=0,R1,R2,R3,R4,R5,R6,R7,R8,R9,R10,
		FP,IP,SP,LR,PC,
		B0,B1,B2,B3,B4,B5,B6,B7,B8,
		CPSR
	};

	virtual void basicHandle(int vector, InterruptContext *context);
	virtual unsigned basicCausedInterrupt(int vector, InterruptContext *context);

	virtual unsigned getPC() { return registers[PC]; }
	virtual void setPC(unsigned address);
	virtual void setBreakInsn(unsigned address);

	virtual char GetDebugChar();
	virtual void PutDebugChar(char);
	virtual int ComputeSignal(int);
	virtual void GdbKill();     // Kill & reboot.  

	virtual void GdbMachineHandlerStart(int, InterruptContext *context);
	virtual void GdbMachineHandlerEnd(int, InterruptContext *context);

	virtual int GdbMachineMem_ok( unsigned acc_type, char *addr, unsigned len);
};


extern void traceStack();

inline void TrapToDebugger() {
	Console() << "Please type ctrl-alt-2, then 'gdbserver'. Connect with gdb by 'target remote localhost:1234'\n" << eor;
//	asm volatile ("bkpt #0");
}

#endif /*GenericARMGdb_h*/
