/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * GenericARMKernel.h: Machine description class for the Integrator Kernel.
 */

#ifndef GenericARMKernel_h
#define GenericARMKernel_h


#ifdef __GNUC__
#pragma interface
#endif


#include "Kernel.h"
#include "MACHINE_CONFIGURATION.h"
#include "AsmBridge.h"
#include "ARMPageTableConstants.h"
#include "MachPROM.h"
#include "elF.h"

class ESP;
class RAMDisk;
class MemoryObjectContainer;
class PhysicalMemoryRange;
class PersistentMemoryObject;

extern ClassStar GenericARMKernelClass;
class GenericARMKernel : public Kernel {
 protected:

    ESP *           _SCSIController;
    RAMDisk * theDisk;
    MemoryObjectContainer * theRootContainer;
    unsigned _availableMemory;
    unsigned _physicalMemorySize;

    /*
     * Generic ARM stuff.
     */
    virtual void DisplayCpuInfo(GenericARMPROM *prom);
    virtual void EnableConsoleInterrupts(void);
    virtual void InstallTimeOfDayClock(void);
    virtual void InstallSoftwareInterruptManager(void);
    virtual void InstallSoftwareClockManager(void);
    virtual void InstallTimerProcesses(void);
    virtual void InstallTheRamDisk(char *baseAddress);
    
    /* pure virtual functions */
    virtual void basicAddMemoryObjects() = 0;
    
    
    /*
     * Overloads of parent class methods.
     */
    virtual void buildRunningCPU();
    virtual void basicInitializeDevicesAndManagers();
    virtual void initialize( Allocator * bootHeap );

    virtual void shutdown();

    DiskRef rootDisk();
    int isObjectFile( PersistentMemoryObject * objectFile );
    const char * loadObjectFile( PersistentMemoryObject * objectFile,
                                 Domain * newProcessesDomain );
    void mountNonRootFileSystems();
    void initializeBackingStore();
    void runBootConsole(Allocator * bootHeap);
    bool doCommand(char* cmd, Allocator * bootHeap);
    void printMenu();
    int loadObjectHeader(PersistentMemoryObject * file,
                         llvm::ELF::Elf32_Ehdr *header);

 public:
    /* pure virtual functions */
    virtual const unsigned          kernelHeapSize() = 0;

    virtual PhysicalMemoryRange *   physicalMemoryMap() ;
    virtual const unsigned      physicalMemorySize();
    virtual const char *            kernelStart()
    {return KERNEL_START_VIRTUAL;}
    virtual const char *            applicationStart()
    {return USERMEMORYSTART;}
    virtual const unsigned          kernelSize() 
    {return KERNEL_SIZE;}
    virtual const unsigned          applicationSize() 
    {return ((unsigned)USERMEMORYEND - (unsigned)USERMEMORYSTART);}

    // Francis - TODO Is this correct?
    virtual const unsigned          addressTranslationMemorySize()
    {return (PointerTableSize + PageTableSize*50);}

    virtual PersistentMemoryObjectRef allocateMemoryObjectForBackingStore();
    GenericARMKernel();

    virtual int tryCommand(int argc, char *argv[], int partnumber);

    Class * classOf();
};

#endif /* GenericARMKernel_h */
