/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-2006
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * GenericARMTimesliceTimer.h - Timeslice timer for the GenericARM
 */

#ifndef GenericARMTimesliceTimer_h
#define GenericARMTimesliceTimer_h


#ifdef __GNUC__
#pragma interface
#endif


#include "Lock.h"
#include "Timer.h"	/* parent class */
#include "OutputStream.h"
#include "GenericARMTimeSliceInterrupt.h"

extern GenericARMTimeSliceInterrupt TSliceInterrupt;

class GenericARMTimesliceTimer : public Timer {
	Lock *lock;
	unsigned int tickTime;
	unsigned int expireTime;
	char running;

public:
	GenericARMTimesliceTimer();
	~GenericARMTimesliceTimer();

	unsigned int start( unsigned int microseconds );
	void set( unsigned int microseconds );
	unsigned int stop();
	unsigned int residual();
	int idle();
	/*
	 * tick should be called directly from the hardware timer interrupt.
	 *   every 1000000/ClockFrequency microseconds.
	 */
	void tick();
};

inline
GenericARMTimesliceTimer::GenericARMTimesliceTimer( )
{
	lock = new Lock("GenericARMTimesliceTimerLock");
	tickTime = 0;
	running = 0;
	expireTime = 0;
}

inline 
GenericARMTimesliceTimer::~GenericARMTimesliceTimer()
{
	delete lock;
}


inline void
GenericARMTimesliceTimer::tick()
{
	lock->acquire();

	if ((++tickTime == expireTime) && running) {
		running = 0;
		lock->release();
		TSliceInterrupt.raise();
	}
	else
		lock->release();
}

#endif /* GenericARMTimesliceTimer_h */
