/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef IntegratorConfiguration_h
#define IntegratorConfiguration_h

/*
 *      ARM Choices VM layout
 *
 *       ---------------    0x0
 *      |    invalid    |
 *       ---------------    _kernelStart = 0x04000
 *      |    Kernel |
 *       ---------------    &end
 *      |   Boot Heap   |
 *       --------------- 
 *      |       |
 *       ---------------
 *      |  Kernel Heap  |
 *       ---------------    KERNELSIZE (SHARED_START) = 0x10000000
 *      |    VA HOLE    |   (no mans land)
 *       ---------------    0xe0000000  APPLICATION_START
 *      |       |
 *      |       |
 *       ---------------    0xff000000  APPLICATION_END
 *      |     dvma  |
 *       ---------------    0xffd00000
 *      |    eeprom |
 *       ---------------    0xfff00000
 *      |  spare segptr |
 *       ---------------    IO_VBASE = 0xffff0000
 *      |   hardware    |
 *       ---------------
 */

#define         PHYSICALMEMORYSTART     0x00000000;

extern void * end;
#define HEAP_START() ((char*)&end)
const char *const    USERMEMORYSTART    = (char *) 0xe0000000;
const char *const    USERMEMORYEND      = (char *) 0xff000000;

/*
 * The size of the 1-1 mapped image that the boot code sets up
 * and that the VM initialization expects to be in place.
 * Range of virtual address that kernel lives in.
 */
char *const     KERNEL_START_VIRTUAL = (char *)0x10000;
char *const     KERNEL_END_VIRTUAL   = (char *)0x800000;
const unsigned  KERNEL_SIZE          = KERNEL_END_VIRTUAL - KERNEL_START_VIRTUAL;

#include "IntegratorInterrupts.h"
#include "IntegratorFreeRunningTimer.h"
#include "IntegratorTimerManager.h"
#include "IntegratorInterruptManager.h"

#ifdef RELIABILITY
#include "IntegratorSerialDD.h"
#define MACHINESERIAL           IntegratorSerialDD
#else
#include "IntegratorSerial.h"
#define MACHINESERIAL           IntegratorSerial
#endif

#define MACHINE_UART_BASE   UART_BASE
#define INITRD              0x00800000  

#define MACHINEINTERRUPTMANAGER IntegratorInterruptManager
#define MACHINEPROM             IntegratorPROM

#endif /* IntegratorConfiguration_h */
