#ifndef IntegratorSerial_h
#define IntegratorSerial_h

/*
 * This file is part of the Choices Operating System
 * Developed by:
 *      The Mobile Operating Systems Group
 *      Department of Computer Science
 *      University of Illinois at Urbana-Champaign
 *      201 N. Goodwin Ave.
 *      Urbana, IL 61801
 *
 * Copyright 1987-2006 University of Illinois Board of Trustees. All rights
 * reserved. 
 *
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 */

/*
 * Integrator/IntegratorSerial.h: Integrator serial port interaction.
 */

#include "GenericARMSerial.h"

#define UART_BASE 0x16000000
#define UART_CR (UART_BASE + 0x30)
#define UART_RXE (1 << 9)
#define UART_TXE (1 << 8)
#define UARTEN (1 << 0)
#define UARTDR (UART_BASE)
#define UARTFR (UART_BASE + 0x18)
#define UART_BUSY (0x3)          

/** Interrupts **/

/* mask/ set clear */
#define UARTIMSC (UART_BASE + 0x38)
#define UART_TXIM (1 << 5)
#define UART_RXIM (1 << 4)

/* raw interrupt status */
#define UARTTRIS (UART_BASE + 0x3c)
#define UART_TXRIS (1 << 5)
#define UART_RXRIS (1 << 4)

/* Masked interrupts */
#define UARTMIS (UART_BASE + 0x40)

/* Interrupt clear register */
#define UARTICR (UART_BASE + 0x44)

// Overrun	
#define UART_OEIC (1 << 10)
// Break
#define UART_BEIC (1 << 9)
// Parity
#define UART_PEIC (1 << 8)
// Framing error
#define UART_FEIC (1 << 7)
// Receive timeout
#define UART_RTIC (1 << 6)
// Transmit clear for UARTTXINTR
#define UART_TXIC (1 << 5)
// Receive clear for UARTRXINTR
#define UART_RXIC (1 << 4)

/* UARTFR status */
#define UART_RXFF (1 << 6)
#define UART_TXFF (1 << 5)

class IntegratorSerial : public GenericARMSerial {
public:
	IntegratorSerial(unsigned base);

	virtual void Initialize();
	virtual void putChar(char);
	virtual char getChar();
	virtual int  tstChar();
	virtual void enableInterrupts();
	virtual void disableInterrupts();

protected:
	unsigned base;
};

#endif
