/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * Vectors.h: vector numbers for processor traps/interrupts.
 */

#ifndef Vectors_h
#define Vectors_h


#ifdef __GNUC__
#pragma interface
#endif


/*
 * Interrupt Vectors for this machine
 */
//ARM p. 33 Sloss
const unsigned VEC_RESET    = 0; //reset
const unsigned VEC_UNDEF_INSN   = 1; //undefined instruction
const unsigned VEC_SWI      = 2; //software interrupt 
const unsigned VEC_PF_ABORT     = 3; //prefetch abort
const unsigned VEC_DATA_ABORT   = 4; //data abort
const unsigned VEC_RESERVED     = 5; //reserved
const unsigned VEC_IRQ      = 6; //irq request
const unsigned VEC_FIQ      = 7; //fast irq request

#define ARM_PIC_VEC_BASE    8
#define ARM_PIC_VEC_CNT     32


#define ARM_PIC_SOFTINT     0
#define ARM_PIC_UARTINT0    1
#define ARM_PIC_UARTINT1    2
#define ARM_PIC_KBDINT      3
#define ARM_PIC_MOUSEINT    4
#define ARM_PIC_TIMERINT0   5
#define ARM_PIC_TIMERINT1   6
#define ARM_PIC_TIMERINT2   7
#define ARM_PIC_RTCINT      8
#define ARM_PIC_LM_LLINT0   9
#define ARM_PIC_LM_LLINT1   10
#define ARM_PIC_CLCDCINT    22
#define ARM_PIC_MMCIINT0    23
#define ARM_PIC_MMCIINT1    24
#define ARM_PIC_AACIINT     25
#define ARM_PIC_CPPLDINT    26
#define ARM_PIC_ETH_INT     27
#define ARM_PIC_TS_PENINT   28

/*
 * Peripheral interrupts
 */
const unsigned VEC_ETHER    = 21; //?
const unsigned VEC_UART     = ARM_PIC_VEC_BASE + ARM_PIC_UARTINT0;
// For timer manager
const unsigned VEC_TIMER2   = ARM_PIC_VEC_BASE + ARM_PIC_TIMERINT0;
// For FRT
const unsigned VEC_TIMER1   = ARM_PIC_VEC_BASE + ARM_PIC_TIMERINT1;

const unsigned VEC_TIMER3   = ARM_PIC_VEC_BASE + ARM_PIC_TIMERINT2;
const unsigned VEC_ASYNCH_MEM   = 31; //?

#endif /* Vectors_h */
