/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Contributing Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * CRQ related code/structure definitions derived mostly from Encore sources
 *	and proprietary technical documents.
 *
 *  Portions of this file are Copyright 1985 Encore Computer Corporation
 */

/*
 * This file assumes a lot about the C++ compiler. Especially that structures/
 * classes are packed without holes, that sizeof( int ) == sizeof( long ) == 4
 * bytes, and that sizeof( short ) == 2 bytes.
 */

#ifndef CRQ_h
#define CRQ_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Object.h"
#include "Lock.h"
#include "Semaphore.h"
#include "SemaphoreStar.h"

class PhysicalMemoryChain;	// forward reference...
class CRQ_MSG;			// DITTO!

/*
 * CRQ Message Op-codes:
 */
enum CRQ_MSG_OP_CODES {
	/*
	 * The following codes define the standard commands recognized by
	 * all CRQ systems.
	 */
	CRQOP_CREATE_CHANNEL = 1,
	CRQOP_DELETE_CHANNEL = 2,
	CRQOP_ABORT_REQ = 3,
	CRQOP_ABORT_ALL = 4,
	CRQOP_LOAD = 5,
	CRQOP_DUMP = 6,
	CRQOP_EXEC_PROG = 7,
	CRQOP_WARM = 8,
	CRQOP_COLD = 9,
	CRQOP_NOP = 10,
	CRQOP_DUMP_STATE = 11,
	CRQOP_INQUIRE = 12,
	/*
	 * The following commands are specific to general masstore operations.
	 */
	CRQOP_MS_READ = 64,
	CRQOP_MS_WRITE = 65,
	CRQOP_MS_READ_CONFIG = 66,
	CRQOP_MS_READ_STAT = 67,
	CRQOP_MS_READ_RESET_STAT = 68,
	CRQOP_MS_DISABLE_WRITE = 69,
	CRQOP_MS_ENABLE_WRITE = 70,
	CRQOP_MS_TEST_UNIT_READY = 71,
	/*
	 * Commands for masstore disks
	 */
	CRQOP_MD_RECALIBRATE = 72,
	CRQOP_MD_FORMAT_DRIVE = 73,
	CRQOP_MD_REPLACE_TRACK = 74,
	CRQOP_MD_READ_GEOMETRY = 75,
	CRQOP_MD_READ_BAD_BLOCK	= 76,
	CRQOP_MD_SEEK = 77,
	CRQOP_MD_READ_LONG = 78,
	CRQOP_MD_WRITE_LONG = 79,
	CRQOP_MD_CHECK_TRACK = 80,
	CRQOP_MD_READ_DT_PROM = 81,
	CRQOP_MD_READ_SECT_ID = 82,
	/*
	 * Commands for masstore tapes
	 */
	CRQOP_MT_WRITE_FILE_MARK = 83,
	CRQOP_MT_REWIND	= 84,
	CRQOP_MT_SPACE_BLOCK = 85,
	CRQOP_MT_SPACE_FMARK = 86,
	CRQOP_MT_SPACE_SQN_FMARK = 87,
	CRQOP_MD_REASSIGN_BLOCK = 88,
	/*
	 * EMC Ethernet commands
	 */
	CRQOP_EN_XMIT_FRAME = 256,
	CRQOP_EN_RCV_FRAME = 257,
	CRQOP_EN_READ_STAT = 258,
	CRQOP_EN_READ_RST_STAT = 259,
	CRQOP_EN_CONTROL = 260,
	CRQOP_EN_XMIT_FRAME_STS = 261,
	/*
	 * SCC control commands
	 */
	CRQOP_SCC_SYS_STATE = 320,
	CRQOP_SCC_SET_TOY = 321,
	CRQOP_SCC_GET_TOY = 322,
	CRQOP_SCC_SET_DCT = 323,
	CRQOP_SCC_GET_DCT = 324,
	CRQOP_SCC_GET_BBRAM_AVAIL = 325,
	CRQOP_SCC_REBOOT = 326,
	CRQOP_SCC_START_PROFILE = 327,
	CRQOP_SCC_STOP_PROFILE = 328,
	CRQOP_SCC_GET_AD = 329,
	/*
	 * These are for the System Exerciser only.
	 * They allow the exerciser to start and stop Memory CSR scans,
	 * Soft Error attentions, NMI reports, and any future diagnostic
	 * output for obvious reasons.
	 */
	CRQOP_SCC_FREEZE = 330,
	CRQOP_SCC_MEM_SCAN = 331,
	CRQOP_SCC_SOFT_ERROR = 332,
	CRQOP_SCC_NMI = 333,
	/*
	 * SCC serial line commands
	 */
	CRQOP_SL_SET_MODE = 336,
	CRQOP_SL_GET_MODE = 337,
	CRQOP_SL_INIT = 338,
	CRQOP_SL_PAUSE = 339,
	CRQOP_SL_RESUME = 340,
	CRQOP_SL_READ = 341,
	CRQOP_SL_WRITE = 342,
	/*
	 * SL attentions for the SCC
	 */
	CRQATTN_SL_READ = 360,
	CRQATTN_SL_WRITE = 361,
	/*
	 * Other SCC attentions.
	 */
	CRQATTN_ENV = 362,
	CRQATTN_MEM_CSR = 363,
	CRQATTN_PS_FAIL = 364,
	CRQATTN_SOFTERR = 365,
	CRQATTN_ALIVE = 366,
	/*
	 * More SCC commands
	 */
	CRQOP_SCC_SHTDWN_RDY = 400,
	CRQOP_SCC_SHUTDOWN = 401,
	/*
	 * Standard and device-specific attention message code
	 */
	CRQATN_BASE = 0x8000,
	CRQATN_LOCK_TMO	= (CRQATN_BASE + 1),
	CRQATN_VECT_NOT_XMIT = (CRQATN_BASE + 2),
	CRQATN_VECT_NOT_RCV = (CRQATN_BASE + 3),
	CRQATN_NO_FREE_MSG = (CRQATN_BASE + 4),
	CRQATN_INVLD_CRQ = (CRQATN_BASE + 5),

	CRQATN_EMC_ATN = (CRQATN_BASE + 64),
	CRQATN_MSC_ATN = (CRQATN_BASE + 65),
};

/*
 * Linked lists of "things" in the Multimax I/O system.
 */
class DBL_LINK {
protected:
	DBL_LINK * dbl_fwd;
	DBL_LINK * dbl_bwd;

public:
	DBL_LINK();

	void enqueue( CRQ_MSG * );
	CRQ_MSG * dequeue();
	int isEmpty();
};

inline int
DBL_LINK::isEmpty()
{
	return( dbl_fwd == (DBL_LINK *) &dbl_fwd );	 // or == this ??
}

inline void
DBL_LINK::enqueue( CRQ_MSG * msg )
{
	DBL_LINK * queue = dbl_bwd;
	DBL_LINK * message = (DBL_LINK *) msg;

	message->dbl_fwd = queue->dbl_fwd;
	message->dbl_bwd = queue;
	queue->dbl_fwd = queue->dbl_fwd->dbl_bwd = message;
}

inline CRQ_MSG *
DBL_LINK::dequeue()
{
	if( isEmpty() ) return( 0 );

	DBL_LINK * queue = dbl_fwd;
	CRQ_MSG * message = (CRQ_MSG *) queue;

	queue->dbl_bwd->dbl_fwd = queue->dbl_fwd;
	queue->dbl_fwd->dbl_bwd = queue->dbl_bwd;
	return( message );
}

class SYSHDR : public DBL_LINK {
protected:
	unsigned int	hdr_size:14;	/* Structure size in bytes */
	unsigned int	hdr_type:11;	/* Structure type */
	unsigned int	hdr_version:6;	/* Structure version */
	unsigned int	hdr_alloc:1;	/* Allocator flag */
public:
	SYSHDR( int size, int type, int version );
};

/*
 * The only 2 Encore "objects" (subclasses of SYSHDR) that we use are
 * CRQ's and CRQ_MSG's. There types for the hdr_type field in the SYSHDR
 * follow.
 */
static const int TYPE_CRQ = 5;		// I/O controller cmd/resp queue (CRQ)
static const int TYPE_CRQMSG = 6;	// A CRQ message

/* Time in micro-seconds. */
class D_TIME {
protected:
	unsigned int t_low;		/* Low 32-bits. */
	unsigned int t_high;		/* High 32-bits. */
};

class TIMING {
protected:
	unsigned int t_samples;		/* Number of samples */
	unsigned int t_min;		/* Minimum time  */
	unsigned int t_max;		/* Maximum time */
	D_TIME t_total;			/* Total time */
};

class MPLOCK_STATS {
protected:
	TIMING mpls_hold;		/* Hold time statistics */
	TIMING mpls_wait;		/* Wait time statistics */
	unsigned int mpls_collisions;	/* Number of collisions */
};

class MPLOCK {
public:
	char lock_state;		/* Lock state (and byte) */
	char lock_level;		/* Lock level */
	char lock_owner;		/* Lock owner (or previous) cpuid */
	char lock_type;			/* Lock type */
	MPLOCK_STATS * lock_stats;	/* Pointer to lock statistics */
};

/*
 * Lock state values
 */
static const char LOCK_HELD = 0xff;		// Lock is held
static const char LOCK_FREE = 0x00;		// Lock is free

/*
 * Lock type fields
 */
static const char LOCK_NORMAL = 0;
static const char LOCK_INSTRUMENTED = 1;

/*
 * We only use one lock level in Choices (for now).
 */
static const char ChoicesIOLockLevel = 0;

class CRQ_STATS : public SYSHDR {
public:
	short crs_maxcmd;		/* Max size of cmd queue */
	short crs_curcmd;		/* Current size of cmd queue */
	short crs_maxrsp;		/* Max size of response queue */
	short crs_currsp;		/* Current size of response queue */
	short crs_maxattn;		/* Max size of attention queue */
	short crs_curattn;		/* Current size of attention queue */
	short crs_maxfree;		/* Max size of free list queue */
	short crs_curfree;		/* Current size of free list queue */
	long crs_vectattempts;		/* # attempts to send a vector */
	MPLOCK_STATS crs_lockstats;	/* Mplock statistics data structure */
};

/*
 * A Command/Response Queue Object.
 */
class CRQ : public SYSHDR {
protected:
	MPLOCK crq_mplock;		/* Spin lock */
public:
	char crq_mode;			/* Polled vs interrupt value */
	char crq_opt;			/* Option bit field */
	short crq_unitid;		/* Unitid associated with crq */
	long crq_slave_vect;		/* Vector to interrupt device/slave */
	long crq_master_vect;		/* Vector to interrupt host/master */
	DBL_LINK crq_cmd;		/* Queue of outstanding commands */
	DBL_LINK crq_immedcmd;		/* Queue of high-priority commands */
	DBL_LINK crq_rsp;		/* Queue of completed responses */
	DBL_LINK crq_attn;		/* Queue of attentions to master */
	DBL_LINK crq_free;		/* Queue of free cmd/attn packets */
	long crq_totcmds;		/* Total # commands issued (master) */
	long crq_totrsps;		/* Tot. # responses returned (slave) */
	long crq_totattns;		/* Total # attentions sent (slave) */
	long crq_totvects;		/* Total # vectors sent (both) */
	CRQ_STATS * crq_stats;		/* Ptr to extended sts blk or NULL */

	/*
	 * Methods.
	 */
	CRQ( unsigned int unitId, unsigned int master_vector, int mode );

	void lock();
	void unlock();
	long slaveVector();

	void dump();
	void sendCommand( CRQ_MSG * command );
	void sendCommandAndAwaitResponse( CRQ_MSG * command );
	unsigned int changeMasterVector( unsigned int newVector );

	CRQ_MSG * dequeueAttentionMessage();
	CRQ_MSG * dequeueResponse();
	void handleStandardAttention( CRQ_MSG * attn );
	void enqueueFreeMessage( CRQ_MSG * msg );
};

inline long
CRQ::slaveVector()
{
	return( crq_slave_vect );
}

inline void
CRQ::lock()
{
	/*
	 * get the lock.
	 */
	char oldState = 1;
	while( oldState == 1 ) {
		/*
		 * Try an interlocked test-and-set on the lock.
		 */
		asm volatile( "sbitib $0,%1; sfsb %0" : "=g" (oldState) : "g" (crq_mplock.lock_state) );

		/*
		 * Spin only reading if that failed to avoid cache
		 * contention until its possible that the test-and-set
		 * might succeed.
		 */
		if( oldState == 1 ) {
			do {} while( crq_mplock.lock_state == LOCK_HELD );
		}
	}
}

inline void
CRQ::unlock()
{
	crq_mplock.lock_state = LOCK_FREE;
}

static const int CRQ_REV_LEVEL = 1;

/*
 * Mode definitions
 */
enum {	CRQMODE_POLL = 0,	// Polled
	CRQMODE_INTR = 1,	// Interrupt driven
};

/*
 * Option definitions
 */
enum {	CRQOPT_NULL = 0,
	CRQOPT_EXTSTATS = 1,	// Keep extended CRQ statistics
};

/*
 * Message status codes.
 */
enum {	STS_SUCCESS = 0,	// Operation successful
	STS_WARNING = 1,	// Operation successful: status to report
	STS_PENDING = 2,	// Used when an I/O command is in progress
	STS_QUEUED = 3,		// Command queued, waiting for device
	STS_FREE = 4,		// Message packet on free list
	STS_ABORTED = 5,	// Operation aborted
	STS_BADARG = 6,		// Invalid argument
	STS_INVOPCODE = 7,	// Unrecognized opcode field contents
	STS_ERROR = 8,		// Operation unsuccessful
	STS_TIMEOUT = 9,	// Operation timed out
};

/*
 * All CRQ commands are subclasses of CRQ_MSG. Each adds a set of
 * variables specific to the particular message.  These variables provide for
 * the arguments and return status values for the particular command.
 *
 * Justification for this division is that some of the processing of
 * any CRQ command does not depend on the format of the particular
 * command.  All of this command independent information is collected into
 * the CRQ_MSG class.
 */
static const int CRQMSG_VERSION = 1;

class CRQ_MSG : public SYSHDR {
	friend class CRQ;
protected:
	CRQ * crq_msg_crq;		/* Back ptr to msg's CRQ */
	unsigned short crq_msg_code;	/* Message type code */
	unsigned short crq_msg_unitid;	/* Unit id of message target */
	SemaphoreStar crq_msg_semaphore; /* Completion Semaphore */
	long crq_msg_timestamp[2];	/* Time message was queued */
	long crq_msg_status;		/* Overall message status */

	void setStatus( long status );
public:
	// some of the defaults should go away once all CRQ_MSG subclasses
	// have constructors. Especially size!
	CRQ_MSG( int size = sizeof( SYSHDR ) + 24, int opCode = 0,
		int unitID = 0, Semaphore * sem = 0 );
	long status();
	unsigned short code();
	Semaphore * semaphore();
};

inline long
CRQ_MSG::status()
{
	return( crq_msg_status );
}

inline unsigned short
CRQ_MSG::code()
{
	return( crq_msg_code );
}

inline Semaphore *
CRQ_MSG::semaphore()
{
	return( crq_msg_semaphore );
}

inline void
CRQ_MSG::setStatus( long status )
{
	crq_msg_status = status;
}

/*
 * The maximum size of any attention message in the system
 */
static const int MAX_ATTN_SIZE = sizeof( CRQ_MSG ) + 24;

static const int FreeAttentions = 2;	// Number of free attentions to give
					// each CRQ.

class CRQ_GENERIC_ATTN_MSG : public CRQ_MSG {	/* A GENERIC ATTENTION */
public:
	char data[ 24 ];	// see above...

	CRQ_GENERIC_ATTN_MSG();
};

inline
CRQ_GENERIC_ATTN_MSG::CRQ_GENERIC_ATTN_MSG()
	: CRQ_MSG( sizeof( CRQ_GENERIC_ATTN_MSG ), 0, 0, 0 )
{
}

/*
 * The CRQ_MSG class used to create new channels.
 */
class CreateChannelRequest : public CRQ_MSG {
protected:
	/*
	 * The next two fields are expected to be EXACTLY as the are
	 * by the CRQ firmware.
	 */
	CRQ * newCRQAddress;
	long resultStatusCode;	// set by slave side.

	/*
	 * Stuff added for Choices.
	 */
public:
	CreateChannelRequest();

	void setup( CRQ * newCRQ );
	long resultStatus();
};

inline
CreateChannelRequest::CreateChannelRequest()
	: CRQ_MSG( sizeof( CreateChannelRequest ), CRQOP_CREATE_CHANNEL,
		0, new BinarySemaphore( 0 ) )
{
}

inline void
CreateChannelRequest::setup( CRQ * newCRQ )
{
	newCRQAddress = newCRQ;
}

inline long
CreateChannelRequest::resultStatus()
{
	return( resultStatusCode );
}

/*
 * Buffer descriptors (buf_desc) are 64-bit structs understood by the
 * EMC DMA engine.  Each describes a region of physical memory to do
 * DMA to/from. An array of them describes an entire block transfer.
 * The last one in the array should have bd_buff_ptr = bd_invalid_addr and
 * bi_v = !bd_valid. Be careful to avoid the strange insert/extract bug
 * when referencing the buf_info subfields.
 */
class buf_desc {
public:
        const char * bd_buff_ptr;	// Pointer to data (64-bit aligned)
	struct buf_info {
		unsigned int bi_bcnt : 24;	// Byte count
        	unsigned int bi_lll  :  3;	// Last byte offset
		unsigned int bi_fff  :  3;	// First byte offset
        	unsigned int bi_c    :  1;	// Chained descriptor
        	unsigned int bi_v    :  1;	// Descriptor valid
	} bd_bi;

	buf_desc();
	void map( const char * addr, int nbytes );
	void invalidate();
};

static const unsigned int QUAD_ALIGN_MASK = 0xfffffff8;
static const unsigned int QUAD_OFF_MASK = 0x7;
static const int bd_chained = 1;	// Value for setting bi_c
static const int bd_valid = 1;		// Value for setting bi_v
/*
 * Invalid physical address constant for last buffer descriptor
 */
static const char * const bd_invalid_addr = (char * const) 0xffffffff;


inline
buf_desc::buf_desc()
{
}

inline void
buf_desc::map( const char * addr, int nbytes )
{
	buf_info info;		// do inserts/extracts on stack
				// to avoid the crossing a page bug.

	bd_buff_ptr = (const char *) ( (unsigned int) addr & QUAD_ALIGN_MASK);
	info.bi_bcnt = ( ( ( (unsigned int) addr & QUAD_OFF_MASK ) + nbytes
		+ 7 ) & QUAD_ALIGN_MASK ) - 4;
	info.bi_fff = (unsigned int) addr & QUAD_OFF_MASK;
	info.bi_lll = ( (unsigned int) addr + nbytes - 1 ) & QUAD_OFF_MASK;
	info.bi_c = ! bd_chained;
	info.bi_v = bd_valid;
	bd_bi = info;
}

inline void
buf_desc::invalidate()
{
	buf_info info;

	info.bi_v = ! bd_valid;
	bd_buff_ptr = bd_invalid_addr;
	bd_bi = info;
}

/*
 * A BufferDescriptor is basically a way to store four buf_desc objects
 * and ensure their alignment. In reality only three of the buf_desc objects
 * are usable since the forth is needed as a terminator.
 * Three was chosen since it allows a guarantee of 8k to be transfered in
 * one descriptor .
 *
 * This allows 8k virtual buffers to occupy non-contiguous physical pages
 * and still be DMA'ed into. 
 * 8k will hold any ethernet buffers that might cross a page boundry as
 * well as up to 16 contiguous disk blocks for file system transfers.
 */
class BufferDescriptor {
protected:
	buf_desc * firstDescriptor;
	long buf_descArray[9];

public:
	BufferDescriptor();

	void mapTo( const char * data_paddr, int length );
	void mapTo( PhysicalMemoryChain * chain );
	buf_desc * descriptors();
};

inline
BufferDescriptor::BufferDescriptor()
{
	/*
	 * Make sure that the first buf_desc is quadword aligned.
	 * Do this by pointing firstDescriptor at the first or second buf_desc
	 * int the buf_descArray array depending on which is quadword aligned.
	 */
	if( ( (unsigned int) &buf_descArray[0] & QUAD_OFF_MASK ) == 0 ) {
		firstDescriptor = (buf_desc *) &buf_descArray[0];
	}
	else {
		firstDescriptor = (buf_desc *) &buf_descArray[1];
	}
}

inline buf_desc *
BufferDescriptor::descriptors()
{
	return( firstDescriptor );
}

/*
 * Disk Layout (from layout.h)
 */
static const int DISK_PARTNAM_LEN = 255;
static const int MAXPARTITIONS = 64;
static const int ROOT_PART = 0;
static const int PAGE_PART = 1;
static const int ALL_PART = 2;
static const int HEADER_PART = 3;
static const int MIN_HP_PI = 2048;	// Min hdr part size with program images
static const int MIN_HP_NO_PI = 36; // Min hdr part size without program images
static const int VOL_LABEL_LEN = 32;

static const int FORMAT_VERSION_MAJ = 1;
static const int SCC_MAGIC_NUMBER = 0x05ac369f;

/*
 * Disk Format Revision Structure
 */
class FORMAT_REV {
public:
	short format_major;	/* Format revision, major */
	short format_minor;	/* Format revision, minor */
};

/*
 * Volume Number Structure
 */
class VOL_NUMBER {
public:
	short vol_num;		/* Volume number */
	short vol_total;	/* Total # of volumes in set*/
};

/* 
 * Program Image Area Descriptor Structure 
 */
class IMAGE_DESC {
public:
	int image_size;		/* Image data size in bytes */
	int image_addr;		/* Byte address of start of images */
};

/*
 * Disk Geometry Structure
 */
class DK_GEOM {
public:
	int total_blocks;		/* Total blocks on disk */
	int byte_sector;		/* Bytes per sector */
	int sector_track;		/* Sectors per track */
	int track_cylinder;		/* Tracks per cylinder */
	int cylinders;			/* Total # of cylinders */
	int total_avail_blocks;		/* Total OS addressable blocks */
	int avail_sect_cylinder;	/* Addressable sectors/cylinder */
	int end_cylinders;		/* End cylinders reserved */
	int reserved_1;			/* Reserved */
	int reserved_2;			/* Reserved */
};

/*
 * Partition Namespace Definition
 */
class PART_NAMESP {
public:
	struct {
		char p_name[ DISK_PARTNAM_LEN + 1 ];	/* Partition name */
	} part[ MAXPARTITIONS ];
};

/*
 * Definition for the on-disk layout structure.  The layout is followed on the
 * the next sector boundary by a namespace which is a MAXPARTITIONS by 
 * DISK_PARTNAM_LEN+1 structure, the pointers in the l_partitions array point
 * to the corresponding index in the name space.
 */
static const int L_IA_DEFINED = 0x0001;		/* Image area defined */

class LAYOUT {
public:
	int l_SCC_magic_number;			/* SCC layout validation # */
	FORMAT_REV l_format_rev;		/* Format revision */
	char l_vol_set_label[VOL_LABEL_LEN];	/* Volume set unique id */
	VOL_NUMBER l_vol_number;		/* Volume number in set */
	int l_layout_flag;			/* Flag word */
	IMAGE_DESC l_image_desc;		/* Program image area desc */
	DK_GEOM lay_geom;			/* Disk geometry */
	int l_reserved_1;			/* Reserved */
	int l_reserved_2;			/* Reserved */
	short l_current_part;			/* Current # of partitions */
	short l_maximum_part;			/* Maximum # of partitions */
	struct {
		int part_size;		/* Partition size in blocks */
		int part_off;		/* Partition offset in blks */
		int part_type;		/* Type of partition */
		int part_name_off;	/* Partition name byte offset */
					/*    in l_part_namesp */
		int part_reserved;	/* Reserved */
	} l_partitions[MAXPARTITIONS];
	PART_NAMESP l_part_namesp;		/* Partition name space */
	int l_reserved[10];			/* Reserved */
	int l_check_sum;			/* Check sum for layout */
};

/*
 * The check sum for the layout is calculated by looking at the layout as
 * an array of unsigned characters and adding each one (up to but not 
 * including l_check_sum) to an integer 'sum' which starts as zero.
 */

/*
 * Definitions for part_type
 */
enum {	ROOT_TYPE = 1,
	PAGE_TYPE = 2,
	ALL_TYPE = 3,
	HEADER_TYPE = 4,
	STANDARD_TYPE = 5,
	NOFORMAT_TYPE = 6,
	DIAGNOSTIC_TYPE = 7,
};

/*
 * EMC Message classes.
 */
class EM_MSG : public CRQ_MSG {		/* Standard header for Ethernet */
					/*	and Masstore messages */
public:
	short em_status_code;		/* Command status code */
	short em_dev_errcode;		/* Device specific error code */
	long em_compltn_cnt;		/* Completion count (bytes or else)*/
	long em_xtnd_status;		/* Extended status */

	EM_MSG();
	EM_MSG( int size, int opCode, int unitID, Semaphore * sem = 0 );
};

inline
EM_MSG::EM_MSG()
{
}

inline
EM_MSG::EM_MSG( int size, int opCode, int unitID, Semaphore * sem )
	: CRQ_MSG( size, opCode, unitID, sem )
{
}

/*
 * A MassStoreTransferRequest is a transfer message the way the EMC expects to
 * see it plus a BufferDescriptor.
 */
class MassStoreTransferRequest : public EM_MSG {
protected:
	/*
	 * This much (until END_EMC_SPECIFIC) the EMC knows about and has to
	 * be EXACTLY the way it is.  (see documentation on CRQ_MS_XFER_MSG).
	 */
	buf_desc * firstDescriptor;
	int totalByteCount;
	unsigned long startingBlock;
	int sortkey;			// could be used for seek optimization
					// but we currently ignore it
	/*
	 * END_EMC_SPECIFIC.
	 * This stuff was added for Choices and the EMC is unaware of.
	 */
	BufferDescriptor bd;

public:
	MassStoreTransferRequest();

	void map( int opcode, PhysicalMemoryChain * chain,
		int count, unsigned long block );
	void map( int opcode, const char * paddr, int count,
		unsigned long block );
};

/*
 * Initialize a MassStoreTransferRequest message for reading/writing into
 * a chain from a specified block number.
 */
inline void
MassStoreTransferRequest::map( int opcode, PhysicalMemoryChain * chain,
	int count, unsigned long block )
{
	bd.mapTo( chain );
	totalByteCount = count;
	startingBlock = block;

	crq_msg_code = opcode;
}

inline void
MassStoreTransferRequest::map( int opcode, const char * paddr, int count,
	unsigned long block )
{
	bd.mapTo( paddr, count );
	totalByteCount = count;
	startingBlock = block;

	crq_msg_code = opcode;
}


class CRQ_MS_READ_CONFIG_MSG : public EM_MSG {	/* READ CONFIGURATION */
public:
	char * ms_config_buf;		/* Buffer addr to store config data */
	int ms_config_bcnt;		/* Size of configuration buffer */
};

class CRQ_MS_RDSTAT_MSG : public EM_MSG {	/* READ CHANNEL STATISTICS */
public:
	char * ms_rdstat_buf;		/* Data buffer to receive stats */
	int ms_rdstat_bcnt;		/* Size of data buffer */
};

class CRQ_MS_RRSTAT_MSG : public EM_MSG {/* READ AND RESET CHANNEL STATISTICS */
public:
	char * ms_rrstat_buf;		/* Data buffer to receive stats */
	int ms_rrstat_bcnt;		/* Size of data buffer */
};

class CRQ_MS_WRTDSB_MSG : public EM_MSG {	/* DISABLE WRITE PROTECTION */
};

class CRQ_MS_WRTENB_MSG : public EM_MSG {	/* ENABLE WRITE PROTECTION */
};

class CRQ_MS_TSTRDY_MSG : public EM_MSG {	/* TEST UNIT READY */
};

class CRQ_MD_RECAL_MSG : public EM_MSG {	/* RECALIBRATE DISK */
};

class CRQ_MD_FMT_MSG : public EM_MSG {		/* FORMAT DISK */
public:
	int	md_fmt_disk_type;	/* Type of disk (see commdkio.h) */
	int	md_fmt_badblk_cnt;	/* # of bad blocks */
	buf_desc * md_fmt_desc;		/* First buffer descriptor for bad
					 * block data	*/
};

class CRQ_MD_RPLTRK_MSG : public EM_MSG {	/* REPLACE BAD TRACK */
public:
	unsigned long md_rpltrk_lbn;	/* Logical block address of bad trk */
	buf_desc * md_rpltrk_desc;	/* First buf descriptor of buf to use
					 * to store trk data during replace */
};

class CRQ_MD_RDGEOM_MSG : public EM_MSG {	/* READ DISK GEOMETRY */
public:
	DK_GEOM *md_rdgeom_dkgm; /* Addr of buf to receive geom info */
	int	md_rdgeom_bcnt;		/* Size of geometry buffer */
};

class CRQ_MD_RDBAD_MSG : public EM_MSG {	/* READ BAD BLOCK DATA */
public:
	buf_desc * md_rdbad_desc;	/* First buffer descriptor to receive
					 * bad block data */
	int	md_rdbad_bcnt;		/* Size of bad block buffer */
};

class CRQ_MD_SEEK_MSG : public EM_MSG {	/* SEEK TO TRACK */
public:
	unsigned long md_seek_pbn;	/* Physical block of track to seek */ 
};

class CRQ_MD_CHKTRK_MSG : public EM_MSG {	/* CHECK TRACK */
public:
	unsigned long md_chktrk_pbn;	/* Physical block of track to check */
};

class CRQ_MD_RDTYP_MSG : public EM_MSG {	/* READ DRIVE TYPE */
public:
	buf_desc * md_rdtyp_desc;	/* Ptr to first buffer descriptor */
	int	md_rdtyp_bcnt;		/* Total requested byte count */
};

class CRQ_MD_RDSID_MSG : public EM_MSG {	/* READ SECTOR ID */
public:
	buf_desc * md_rdsid_desc;	/* Ptr to first buffer descriptor */
	int	md_rdsid_bcnt;		/* Total requested byte count */
	unsigned long md_rdsid_pbn;	/* Physical block # of track to read */
};

/*
 * EMC Attention stuff.
 */
class EMC_ATN_MSG : public CRQ_MSG {
public:
	long emc_atn_status;		// Attention status
	long emc_atn_xtnd_stat;		// Extended attention status
};

/* Most Attention Messages will be sent to the Requestor Channel with the
   following interrupts, which will be sent to the Unit Channel:

	EMCATN_SCSI_ILLMSG - if the unit channel exists
	EMCATN_SCSI_SENSE_ERROR
*/
enum {	EMCATN_POWER_FAIL = 1,		// NBI Powerfail signal asserted
	EMCATN_CPU_RAM_PE = 2,		// EMC CPU RAM Read parity error
	EMCATN_CPU_LOCAL_PE = 3,	// EMC CPU Local Bus Read parity error
	EMCATN_CPU_NBI_PE = 4,		// EMC CPU NBI Read parity error
	EMCATN_CPU_BAD_ADDR = 5,	// EMC CPU bad Nanobus Memory Address
	EMCATN_CPU_LAN_PE = 6,		// EMC CPU LAN Memory Read parity error
	EMCATN_LAN_CTLR_PE = 7,		// 82586 LAN Memory Read parity error
	EMCATN_SCSI_FIFO_PE = 8,	// SCSI Data FIFO Output parity error
	EMCATN_LOCAL_BUS_PE = 9,	// EMC Local Bus parity error with Abort
	EMCATN_NBI_NO_GRANT = 10,	// NBI No Grant with Timeout
	EMCATN_NBI_NOT_ACCEPTED	= 11,	// NBI Address not Accepted or
					//	Stallwrite with Timeout
	EMCATN_NBI_RW_DATA = 12,	// NBI Write Data parity error or no
					//	Read Acknowledge with Timeout
	EMCATN_NBI_PROTOCOL = 13,	// NBI Protocol or Logic error with
					//	Timeout
	EMCATN_NBI_DLW_READ = 14,	// NBI Double Long Word error
					// xtnd_stat = NBI Master for all above

	EMCATN_CASCADE_INT = 21,	// Cascade interrupt in non-cascade mode
	EMCATN_ILL_VECT_INT = 22,	// Illegal Vector interrupt from ICU,
					// xtnd_stat = ICU IR No.
	EMCATN_ILL_TRAP	= 23,		// Illegal Trap, xtnd_stat=Trap Vector
	EMCATN_ILL_DEV_VECT = 24,	// Illegal slave vector received,
					// xtnd_stat = vector

	EMCATN_SCSI_ILLMSG = 31,	// Illegal SCSI message, xtnd_stat =
					// 1st four message bytes
	EMCATN_SCSI_SENSE_ERROR = 32,	// Error on a SCSI Request Sense command
					// xtnd_stat = EMC Unit Number
	EMCATN_SCSI_BUS_PE = 33,	// SCSI Bus parity error
	EMCATN_SCSI_CPU_TIMEOUT = 34,	// SCSI CPU Handshaking Timeout
	EMCATN_SCSI_CPU_FAIL = 35,	// SCSI CPU failed self test

	EMCATN_82586_SCB_TMO = 41,	// 82586 SCB interrupt acknowledgement
					// timeout
	EMCATN_82586_CMD_TMO = 42,	// 82586 Action Command completion
					// timeout
	EMCATN_82586_CMD_ERR = 43,	// 2586 Action Command completed with
					// error
};

/*
 * SCC message status codes.
 */
enum {	SCC_CRQ_CHAN_UNDEF = 1,
	SCC_CRQ_CHAN_EXITS = 2,
	SCC_CRQ_CHAN_CREATED = 3,
	SCC_CRQ_CHAN_DELETED = 4,
	SCC_CRQ_CHAN_ACTIVE = 5,
};

/*
 * Number of slots and things in slots
 */
const int NUM_SLOT = 16;
const int NUM_SLOT_DEV = 4;
const int NUM_MEM = 16;
const int OLD_NUM_MEM = 8;
const unsigned int REQ_MASK = 0x23ff;
const unsigned int MEM_MASK = 0x22ff;

/*
 * SCC shared memory layout
 */
const int SCC_SLOT = 12;
const int SCC_CRQS_SLOT	= 4;
const int SCC_CRQS_OFFSET = 1;
const unsigned int SCCMEM_BASE =	0xfffc0000;
const unsigned int SCCMEM_TOP = 0xfffc8000;
const unsigned int SCCREG_BASE = SCCMEM_BASE + 0x10000;
const unsigned int SCCREG_TOP = SCCREG_BASE + 0x300;

/*
 * Macro to calculate the address of a CRQ in SCC shared memory given the
 * slot and device in slot
 */
inline CRQ *
RequestorCRQ( int slot, int device )
{
	return( (CRQ *) ( SCCMEM_BASE + ( ((( slot + SCC_CRQS_OFFSET )
	   * SCC_CRQS_SLOT ) + device ) * sizeof( CRQ ) ) ) );
}

/*
 * Convert lamp/slot/dev/lun into a "unitid" as defined by the Multimax
 * I/O subsystem.
 */
inline unsigned short
UnitID( int lamp, int slot, int dev, int lun )
{
	return( ((lamp & 0xf) << 12) | ((slot & 0xf) << 8)
		| ((dev & 0x3) << 6) | (lun & 0x3f) );
}

/*
 * Generic logical units.
 */
const int REQ_LUN = 0;			// requestor logical unit.
const int SLOT_LUN = 1;			// slot logical unit.


/*
 * Pools of CRQ_MSG's
 */
class CRQ_MSGPool : public Object {
protected:
	DBL_LINK links;
	Lock mutex;
	SemaphoreStar resourceCountSemaphore;
public:
	CRQ_MSGPool();
	~CRQ_MSGPool();

	void addMessage( CRQ_MSG * msg );
	CRQ_MSG * removeMessage();
};

inline void
CRQ_MSGPool::addMessage( CRQ_MSG * msg )
{
	/*
	 * Return the message to the pool.
	 * Lock the links to avoid corruption.
	 */
	mutex.acquire();
	links.enqueue( msg );
	mutex.release();

	/*
	 * Indicate that the message is available to any waiters.
	 */
	resourceCountSemaphore->V();
}

#endif /* CRQ_h */
