/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Contributing Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Contributing Author: Gary M. Murakami (murakami@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * MultimaxEMCDisk.h - Memory Object representing a disk connected to
 *		       an EMC on the Multimax.
 */

#ifndef MultimaxEMCDisk_h
#define	MultimaxEMCDisk_h


#ifdef __GNUC__
#pragma interface
#endif

#include "CRQ.h"
#include "MultimaxInterrupts.h"
#include "PersistentMemoryObject.h"
#include "Disk.h"
#include "Raid.h"
#include "SemaphoreStar.h"


/*
 * Class to represent each EMC-connected disk drive as a PersistentMemoryObject.
 */
extern ClassStar MultimaxEMCDiskClass;
class MultimaxEMCDisk : public Disk {
	friend class MultimaxKernel;
	friend class MultimaxEMCDiskInterrupt;
protected:
	int slot;
	int logicalUnit;
	int responseVector;

	SemaphoreStar alertSemaphore;
	MultimaxEMCDiskInterrupt myInterrupt;

	volatile int initDone;

	int doio( unsigned int start, int count, char * addr, int nbytes,
								int op );
	CRQ crq;		// A CRQ to talk to it with.
				// Some free attention messages to queue up.
	CRQ_GENERIC_ATTN_MSG attention[ FreeAttentions ];	
	LAYOUT	_layout;
	int	_partitions;

	void	initialize();
	void	handleAttention();
	void	handleInterrupt();

	static void diskInterruptProcessEntry( MultimaxEMCDisk * );

	int	transient();

	~MultimaxEMCDisk();
public:
	Class * classOf();

	MultimaxEMCDisk( int slot, int logicalUnit, int responseVector );

	int rawRead( unsigned int start, int count, char * addr, int nbytes );
	int rawWrite( unsigned int start, int count, char * addr, int nbytes );

	int basicDoio( unsigned int, char *, int );
	int reset();
	int partitions();
	int locatePartition( int idNumber, int &start, int &units );
        LAYOUT * layout();

};

inline int
MultimaxEMCDisk::partitions()
{
      Raid( RaidMethod )( "%N::partitions() %d\n", this, _partitions );
      return( _partitions );
}

inline LAYOUT *
MultimaxEMCDisk::layout()
{
      Raid( RaidMethod )( "%N::layout()\n", this );
      return( &_layout );
}

#endif MultimaxEMCDisk_h
