/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801-2910
 *
 * Copyright (c) 1991 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Amit Parghi (parghi@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *	MultimaxEthernetDriver.h
 *
 *	This code attempts to provide an interface between the standard
 *	networking interface and the atrocity known as EMCEthernet.cc.
 *
 * $Id: MultimaxEthernetDriver.h 2 2004-06-03 01:58:28Z jcarlyle $
 *
 * $Log: MultimaxEthernetDriver.h,v $
 * Revision 1.1  2004/06/03 01:58:44  jcarlyle
 * Initial revision
 *
 *
 */


#ifndef MultimaxEthernetDriver_h
#define MultimaxEthernetDriver_h


#ifdef __GNUC__
#pragma interface
#endif

class NetworkAdministrator;
class Process;
class NetworkBuffer;

#include "Types.h"
#include "EthernetDriver.h"

class MultimaxEthernetDriver : public EthernetDriver {
private:
    Process *_daemonProcess;		// daemon: waits for incoming packets

    unsigned char *_emcBuffer;		// pointer to input buffer in EMC
    int _emcBufferSize;			// length of input buffer

public:
    MultimaxEthernetDriver(unsigned char debugMask = 0);
    virtual ~MultimaxEthernetDriver();

    virtual void ready(NetworkAdministrator *);

    virtual int transmitPacket(NetworkBuffer *);
    virtual int transmitPacketAsync(NetworkBuffer *);
    virtual int	addMulticast(EthernetAddress *);
    virtual int removeMulticast(EthernetAddress *);

    void recvLoop();	// Loops and never returns

    void setEMCBuffer(char *, int);
};

inline int
MultimaxEthernetDriver::transmitPacketAsync(NetworkBuffer *)
{
  Assert(NOTREACHED);
  return(0);
}


inline int
MultimaxEthernetDriver::addMulticast(EthernetAddress *)
{
  Assert(NOTREACHED);
  return(0);
}


inline int
MultimaxEthernetDriver::removeMulticast(EthernetAddress *)
{
  Assert(NOTREACHED);
  return(0);
}


    inline void
MultimaxEthernetDriver::setEMCBuffer(char *buff, int size)
{						// used by daemon
    _emcBuffer = (unsigned char *) buff;
    _emcBufferSize = size;
}

#endif /* MultimaxEthernetDriver_h */
