/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef OMAPConfiguration_h
#define OMAPConfiguration_h

/*
 *      ARM Choices VM layout
 *
 *       ---------------    0x0
 *      |    invalid    |
 *       ---------------    _kernelStart = 0x04000
 *      |    Kernel     |
 *       ---------------    &end
 *      |   Boot Heap   |
 *       --------------- 
 *      |               |
 *       ---------------
 *      |  Kernel Heap  |
 *       ---------------    KERNELSIZE (SHARED_START) = 0x10000000
 *      |    VA HOLE    |   (no mans land)
 *       ---------------    0xe0000000  APPLICATION_START
 *      |               |
 *      |               |
 *       ---------------    0xff000000  APPLICATION_END
 *      |     dvma      |
 *       ---------------    0xffd00000
 *      |    eeprom     |
 *       ---------------    0xfff00000
 *      |  spare segptr |
 *       ---------------    IO_VBASE = 0xffff0000
 *      |   hardware    |
 *       ---------------
 */

#define         PHYSICALMEMORYSTART     0x10000000;

extern void * end;
#define HEAP_START() ((char*)&end)
const char *const   USERMEMORYSTART    = (char *) 0xe0000000;
const char *const   USERMEMORYEND      = (char *) 0xff000000;

/*
 * The size of the 1-1 mapped image that the boot code sets up
 * and that the VM initialization expects to be in place.
 * Range of virtual address that kernel lives in.
 */
char *const         KERNEL_START_VIRTUAL= (char *)0x10000000;
char *const         KERNEL_END_VIRTUAL  = (char *)0x11F00000; // 1 meg at end reserved for vdisk
const unsigned      KERNEL_SIZE         = KERNEL_END_VIRTUAL - KERNEL_START_VIRTUAL;

/*
#define OMAP_CONFIG_REGISTERS_BASE 0xFFFE1000
#define OFFSET_MOD_CONF_CTRL_1 0x110

unsigned *MOD_CONF_CTRL_1 = (unsigned *)(OMAP_CONFIG_REGISTERS_BASE + OFFSET_MOD_CONF_CTRL_1);
// Bits 1:0 and 3:2 select clock source for GPTIMER1 and GPTIMER2 respectively
// 00 (default) 12MHz system clock in posted mode, 19.2MHz in non-posted mode
// 01 32KHz clock (posted mode)
// 10 External Clock (non posted mode)
*/

#include "OMAPDevices.h"
#include "OMAPFreeRunningTimer.h"
#include "OMAPInterruptManager.h"

#ifdef RELIABILITY
#include "OMAPSerialDD.h"
#define MACHINESERIAL       OMAPSerialDD
#else
#include "OMAPSerial.h"
#define MACHINESERIAL       OMAPSerial
#endif

#define MACHINE_UART_BASE   CFG_NS16550_COM1

#define MACHINEINTERRUPTMANAGER OMAPInterruptManager

#define MACHINEPROM         OMAPPROM

#endif /* OMAPConfiguration_h */
