/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987-2006
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef OMAPDevices_h
#define OMAPDevices_h

#ifdef __GNUC__
#pragma interface
#endif

/*
 * contains IO virtual addresses of OMAP Devices 
 */

// MPU subsystem clocks
#define CLKGEN_REG_BASE         (0xfffece00)
#define ARM_CKCTL               (CLKGEN_REG_BASE + 0x00)
#define ARM_IDLECT1             (CLKGEN_REG_BASE + 0x04)
#define ARM_IDLECT2             (CLKGEN_REG_BASE + 0x08)
#define ARM_EWUPCT              (CLKGEN_REG_BASE + 0x0C)
#define ARM_RSTCT1              (CLKGEN_REG_BASE + 0x10)
#define ARM_RSTCT2              (CLKGEN_REG_BASE + 0x14)
#define ARM_SYSST               (CLKGEN_REG_BASE + 0x18)
#define ARM_IDLECT3             (CLKGEN_REG_BASE + 0x24)

#define CK_RATEF                1
#define CK_IDLEF                2
#define CK_ENABLEF              4
#define CK_SELECTF              8

/* ARM_CKCTL bit shifts */
#define PERDIV                  0
#define LCDDIV                  2
#define ARMDIV                  4
#define DSPDIV                  6
#define TCDIV                   8
#define DSPMMUDIV               10
#define ARM_TIMXO               12
#define EN_DSPCK                13
#define ARM_INTHCK_SEL          14 /* REVISIT: Where is this used? */

/* ARM_IDLECT1 bit shifts */
#define IDLWDT_ARM      0
#define IDLXORP_ARM     1
#define IDLPER_ARM      2
#define IDLLCD_ARM      3
#define IDLLB_ARM       4
#define IDLHSAB_ARM     5
#define IDLIF_ARM       6
#define IDLDPLL_ARM     7
#define IDLAPI_ARM      8
#define IDLTIM_ARM      9
#define SETARM_IDLE     11

/* ARM_IDLECT2 bit shifts */
#define EN_WDTCK        0
#define EN_XORPCK       1
#define EN_PERCK        2
#define EN_LCDCK        3
#define EN_LBCK         4
#define EN_HSABCK       5
#define EN_APICK        6
#define EN_TIMCK        7
#define DMACK_REQ       8
#define EN_GPIOCK       9
#define EN_LBFREECK     10

// Serial Port
#define CFG_NS16550_COM1     0xfffb0000
#define CFG_NS16550_CLK      (48000000)      /* can be 12M/32Khz or 48Mhz */

// MPU Timers
#define OMAP_MPU_TIMER1_BASE    (0xfffec500)
#define OMAP_MPU_TIMER2_BASE    (0xfffec600)
#define OMAP_MPU_TIMER3_BASE    (0xfffec700)
#define MPU_TIMER_FREE          (1 << 6)
#define MPU_TIMER_CLOCK_ENABLE  (1 << 5)
#define MPU_TIMER_AR            (1 << 1)
#define MPU_TIMER_ST            (1 << 0)

// Interrupts
#define OMAP_IH1_BASE 0xfffecb00
#define OMAP_IH2_BASE 0xfffe0000

#define IH1_ITR       0x00
#define IH1_MIR       0x04
#define IH1_SIR_IRQ   0x10
#define IH1_SIR_FIQ   0x14
#define IH1_CONTROL   0x18
#define IH1_ILR0      0x1C
#define IH1_SISR      0x9C
#define IH1_GMR       0xA0

#define IH2_ITR       0x00
#define IH2_MIR       0x04
#define IH2_SIR_IRQ   0x10
#define IH2_SIR_FIQ   0x14
#define IH2_CONTROL   0x18
#define IH2_ILR0      0x1C
#define IH2_SISR      0x9C
#define IH2_STATUS    0xA0
#define IH2_OCP_CFG   0xA4
#define IH2_INTH_REV  0xA8

#define nInterrupts 256

// Watchdog
#define OMAP_WATCHDOG_BASE             0xfffeb000

#define OMAP_WATCHDOG_CLK_REGISTER     ARM_IDLECT2
#define OMAP_WATCHDOG_CLK_ENABLE       EN_WDTCK

// ULPD (Ultra Low Power Device)
#define OMAP_ULPD_BASE                  0xfffe0800
#define OMAP_ULPD_RESET_STATUS_REGISTER (OMAP_ULPD_BASE + 0x6C)

#endif /* OMAPDevices_h */
