/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * OMAPTimer.h - defines for the ARMstation1 timer
 */

#ifndef OMAPTimer_h
#define OMAPTimer_h

#ifdef __GNUC__
#pragma interface
#endif

#include "OMAPDevices.h"

inline void loop(unsigned i) {
    while (i > 0) i--;
}

class OMAPMPUTimer {
 private:
    // This class cannot have any other member variables. An object of this class
    // is designed to be directly placed on memory-mapped hardware. There should not
    // be any virtual functions because there cannot be any vptr.
    // The order of the member variables below has to match hardware.
    // This is a compiler dependent class. Do not modify this unless you are sure
    // you know what you are doing
    volatile unsigned CNTL_TIMER; // R/W
    volatile unsigned LOAD_TIM;   // W
    volatile unsigned READ_TIM;   // R
 public:
    inline void start(unsigned value) {
        CNTL_TIMER = MPU_TIMER_CLOCK_ENABLE;
        loop(10000);
        LOAD_TIM = value;
        loop(10000);
        CNTL_TIMER = (MPU_TIMER_CLOCK_ENABLE | MPU_TIMER_AR | MPU_TIMER_ST);
    }
    inline unsigned read() {
        return READ_TIM;
    }
};

extern OMAPMPUTimer *omapTimer1;
extern OMAPMPUTimer *omapTimer2;
extern OMAPMPUTimer *omapTimer3;

// Default is 48MHz Clock for MPU timers
#define MPU_TIMER_TICKS_PER_SEC   (48000000)

/*
 * These are for the FreeRunningTimer, it 
 * interrupts every 4 seconds.
 */
const unsigned FRTuSecPerInterrupt = 4000000;
const unsigned FRTTickPeriod = ((MPU_TIMER_TICKS_PER_SEC / 1000000 * FRTuSecPerInterrupt) - 1);

/*
 * These are for the timer manager, it
 * interrupts every 30 milliseconds.
 */
const unsigned TimerManageruSecPerInterrupt = 30000;
const unsigned TimerManagerTickPeriod = ((MPU_TIMER_TICKS_PER_SEC / 1000000 * TimerManageruSecPerInterrupt) - 1);


#endif /* OMAPTimer_h */
