/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Francis David (fdavid@uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * OMAPWatchdogTimer.h - OMAPWatchdogTimer class; just a place holder for subclasses.
 */

#ifndef OMAPWatchdogTimer_h
#define OMAPWatchdogTimer_h


#ifdef __GNUC__
#pragma interface
#endif

#include "OMAPDevices.h"
#include "WatchdogTimer.h"

#define TIMER_MAX          32      /* Default is 32 seconds */
#define TIMER_MIN          1

class OMAPWatchdog {
 private:
    // This class cannot have any other member variables. An object of this class
    // is designed to be directly placed on memory-mapped hardware. There should not
    // be any virtual functions because there cannot be any vptr.
    // The order of the member variables below has to match hardware.
    // This is a compiler dependent class. Do not modify this unless you are sure
    // you know what you are doing
    volatile unsigned REV;         // R
    volatile unsigned reserved0[3];
    volatile unsigned CONFIG;      // R/W
    volatile unsigned STATUS;      // R
    volatile unsigned reserved1[3];
    volatile unsigned CLR;         // R/W
    volatile unsigned CRR;         // R/W
    volatile unsigned LDR;         // R/W
    volatile unsigned TGR;         // R/W
    volatile unsigned WPS;         // R
    volatile unsigned reserved2[4];
    volatile unsigned SPR;         // R/W
 public:
    inline void enable() {
        SPR = 0xbbbb;
        while (WPS & 0x10);   // wait for posted write to complete
        SPR = 0x4444;
        while (WPS & 0x10);   // wait for posted write to complete
    }
    inline void disable() {
        SPR = 0xaaaa;
        while (WPS & 0x10);   // wait for posted write to complete
        SPR = 0x5555;
        while (WPS & 0x10);   // wait for posted write to complete
    }
    inline void ping() {
        TGR = ~TGR;
        while (WPS & 0x08);   // wait for posted write to complete
    }
    inline void setTimeout(unsigned seconds) {
        LDR = (0xffffffff - ((seconds) * 32768)) + 1;
        while (WPS & 0x04);   // wait for posted write to complete
    }
};

extern OMAPWatchdog *omapWatchdog;

class OMAPWatchdogTimer : public WatchdogTimer {
public:
	/*
	 * "start" starts timer and returns actual number of seconds set,
	 *  possibly rounded up to the granularity of the hardware timer.
	 */
	unsigned start( unsigned seconds );
	/*
	 * "stop" returns the residual number of seconds after stopping.
	 */
	unsigned stop();
	/*
	 * "residual" returns the remaining number of seconds.
	 */
	unsigned residual();
    /*
     * Pings resets the timer on the watchdog
     */
    void ping();
};

#endif /* OMAPWatchdogTimer_h */
