/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *			   University of Illinois at Urbana-Champaign
 *			   Department of Computer Science
 *			   1304 W. Springfield Ave.
 *			   Urbana, IL	 61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Contributing Author: See-Mong Tan (stan@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * DatakitChannel.h: Per-channel Datakit processing;
 *	  switch and urp protocol implementation
 */

#ifndef DatakitChannel_h
#define DatakitChannel_h
#ifdef CFRONT
#define CONST const
#define STATIC static
#else /* must be g++ */
#pragma interface
#if (__GNUG__ >= 2)
#if (!defined CONST)||(!defined STATIC)
#define CONST
#define STATIC
#endif
#else
#pragma once
#define CONST const
#define STATIC static
#endif
#endif

#include "Assert.h"
#include "ProxiableObject.h"
#include "Urp.h"
#include "Lock.h"
#include "Semaphore.h"
#include "DatakitDriver.h"

#define MAXCHAN 512

extern ClassStar DatakitChannelClass;
class DatakitChannel : public ProxiableObject {
protected:
	// filled in by the subclass 
	int chan;		// channel number
	char ctlcalls;		// should incoming ctls invoke calls?

public:
	// proxiable objects need this
	proxiable virtual int channel();
	
	char ctlCalls();	// receive ctl as data or calls to ReceiveControl
	proxiable virtual int channelOpened();
	void closeChannel();

	// the following are to be called only from the interrupt handler 
	virtual int ReceivePacket( DatakitPacket * );
	virtual void ReceiveControl( int );

	Class * classOf();
};

#endif  /* DatakitChannel_h */
