/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *             University of Illinois at Urbana-Champaign
 *             Department of Computer Science
 *             1304 W. Springfield Ave.
 *             Urbana, IL    61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *    All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Contributing Authors: See-Mong Tan (stan@cs.uiuc.edu)
 *			Sean M. Dorward (smd@research.att.com)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * DatakitDriver.c: AT&T 6586 CPM-HS Datakit driver
 */

#ifndef DatakitDriver_h
#define DatakitDriver_h
#ifdef CFRONT
#define CONST const
#define STATIC static
#else /* must be g++ */
#pragma interface
#if (__GNUG__ >= 2)
#if (!defined CONST)||(!defined STATIC)
#define CONST
#define STATIC
#endif
#else
#pragma once
#define CONST const
#define STATIC static
#endif
#endif

const int MAXCHAN = 512;

const int DKPSIZ = 16;	// packet length

class DatakitPacket {
public:
	/*
	 * NOTE: eventually performance should be improved by
	 *   overloading the 'new' and 'delete' operators to not
	 *   actually delete the space but to put it on a free list.
	 */

	DatakitPacket *next;
	unsigned char buf[DKPSIZ];
	unsigned char len;
	unsigned short hibits;
	unsigned char ack;

	DatakitPacket();
};

STATIC inline
DatakitPacket::DatakitPacket()
{
        len = 0;
	ack = 0;
	hibits = 0;
	next = 0;
}

// Send functions send only the information in their arguments
void DatakitSendRawPacket(DatakitPacket *);
void DatakitSendPacket(int, DatakitPacket *, int);
void DatakitSendControl(int chan, int val);

// Transmit functions buffer all the data up to a DatakitFinishTransmit
//	return values are the number of bytes transmitted, or -1 for an error
int DatakitTransmitStart(int chan);
int DatakitTransmitData(unsigned char *buf, int len);
int DatakitTransmitControl(int);
int DatakitTransmitEnd();

void DatakitInterruptHandler();

class DatakitChannel;
extern DatakitChannel *DatakitChan[];

#endif
