/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *			   University of Illinois at Urbana-Champaign
 *			   Department of Computer Science
 *			   1304 W. Springfield Ave.
 *			   Urbana, IL	 61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Contributing Author: See-Mong Tan (stan@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * DatakitSwitch.h: Per-channel Datakit processing;
 *	  switch implementation
 */

#ifndef DatakitSwitch_h
#define DatakitSwitch_h
#ifdef CFRONT
#define CONST const
#define STATIC static
#else /* must be g++ */
#pragma interface
#if (__GNUG__ >= 2)
#if (!defined CONST)||(!defined STATIC)
#define CONST
#define STATIC
#endif
#else
#pragma once
#define CONST const
#define STATIC static
#endif
#endif

#include "Assert.h"
#include "ProxiableObject.h"
#include "Urp.h"
#include "LOCK.h"
#include "SemaphoreStar.h"
#include "DatakitDriver.h"
#include "DatakitChannel.h"

extern ClassStar DatakitSwitchClass;
class DatakitSwitch : public DatakitChannel {
protected:
	SemaphoreStar mutex;		// mutual exclusion semaphore
	int waiting;			// a reader or writer is waiting
	DatakitPacket sendPacket;	// packet to use for sending to switch
	DatakitPacket *returnedPacket;	// packet returned from switch

	void sendAndWait();
	SemaphoreStar  waitSemaphore;      // wait on this for reply
	int getSwitchAddress(int);
	int switchParity(int);
	void getphys( int, int, int & , int & );

	void timerHandlerEntry();
	SemaphoreStar timerWakeup;        // wake up handler when needed

public:
	proxiable DatakitSwitch();
	proxiable ~DatakitSwitch();

	// The following routine is only for use from the interrupt handler
	int ReceivePacket(DatakitPacket *);

	int setDevice( int mod, int eaddr, int oaddr, int nrte );
	int Rammify();
	int Rommify();
	proxiable virtual int ReadLocation(int addr, int evenodd = 0);
	proxiable virtual int WriteLocation(int addr, int val, int evenodd = 0);
	int GetCount(int module, int channel, int ctrldata = 0);
	proxiable virtual int GetChannel(int &module, int &channel, int &iscommand, int ctrldata = 0);
	int SetCount(int module, int channel, int count, int ctrldata = 0);
	proxiable virtual int SetChannel(int sourcemodule, int sourcechannel,
		int destmodule, int destchannel, int iscommand = 0, int ctrldata = 0);
	Class * classOf();
};

#endif  /* DatakitSwitch_h */
