/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-90, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: John L. Coolidge (coolidge@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef TYPES_h
#define TYPES_h

#ifdef CFRONT
#  define CONST const
#  define STATIC static
#else /* must be g++ */
#  pragma interface
#  if (__GNUG__ >= 2)
#    if (!defined STATIC)||(!defined CONST)
#      define CONST
#      define STATIC
#    endif
#  else
#    define CONST const
#    define STATIC static
#  endif
#endif

typedef unsigned char		int8;
typedef unsigned short		int16;
typedef unsigned long		int32;
//typedef unsigned long long	int64;
typedef unsigned char IntStorage;

#define WORDS_BIG_ENDIAN 0
#define BITS_PER_UNIT 8

typedef struct longlong { unsigned long low, high; };

class int64
{
private:
	friend class SS1TimeOfDay;
	union {
		struct longlong s;
		long i[2];
		unsigned int ui[2];
		unsigned short us[2][2];
	} _value;

public:
	int64()
		{ _value.ui[0] = 0; _value.ui[1] = 0; }

	int64( const int64& value )
		{ _value.s = value._value.s; }

	int64( int value )
		{ _value.s.low = value; _value.s.high = 0; }

	int64( int low, int high )
		{ _value.s.low = low; _value.s.high = high; }

	int64( unsigned int low, unsigned int high )
		{ _value.s.low = low; _value.s.high = high; }

	operator int() const
		{ return _value.s.low; }

	operator unsigned int() const
		{ return _value.s.low; }

	friend int64 operator+(const int64&, const int64&);
	friend int64 operator-(const int64&, const int64&);
	friend int64 operator*(const int64&, const int64&);
	friend int64 operator/(const int64&, const int64&);
	friend int64 operator%(const int64&, const int64&);
};

#ifndef CFRONT
extern void badd(int16 *a, int16 *b, int16 *c, int n);
inline int64 operator+(const int64& a,const int64& b)
{
	int64 c;
	badd((int16*)&(a._value), (int16*)&(b._value),
		(int16*)&(c._value), sizeof c._value);
	return c;
}

extern void bsub(int16* a, int16* b, int16* c, int n);
inline int64 operator-(const int64& a, const int64& b)
{
	int64 c;
	bsub( (int16*)&(a._value), (int16*)&(b._value),
		(int16*)&(c._value), sizeof (c._value) );
	return c;
}

extern void bdiv (int16* a, int16* b, int16* q, int16* r, int m, int n);
inline int64 operator/(const int64& a, const int64& b)
{
	unsigned long s[4];
	s[0] = a._value.s.low;
	s[1] = a._value.s.high;
	s[2] = 0;
	s[3] = 0;
	int64 q;
	unsigned long r[2];
	bdiv( (int16*)&s, (int16*)&(b._value), (int16*)&(q._value),
		(int16*)&r, sizeof s, sizeof (b._value));
	return q;
}

extern void bmul (int16* a, int16* b, int16* c, int m, int n);
inline int64 operator*(const int64& a, const int64& b)
{
	unsigned int s[4];
	bmul( (int16*)&(a._value), (int16*)&(b._value),
		(int16*)&s, sizeof (a._value),
		sizeof (b._value) );
	int64 c(s[0],s[1]);
	return c;
}

inline int64 operator%(const int64& a, const int64& b)
{
	unsigned long s[4];
	s[0] = a._value.s.low;
	s[1] = a._value.s.high;
	s[2] = 0;
	s[3] = 0;
	int64 r;
	unsigned long q[2];
	bdiv( (int16*)&s, (int16*)&(b._value), (int16*)&q,
		(int16*)&(r._value), sizeof s, sizeof (b._value));
	return r;
}
#else
extern int64 operator-(const int64& a, const int64& b);
extern int64 operator+(const int64& a, const int64& b);
extern int64 operator/(const int64& a, const int64& b);
extern int64 operator*(const int64& a, const int64& b);
extern int64 operator%(const int64& a, const int64& b);
#endif




#endif /* TYPES_h */
