/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801-2910
 *
 * Copyright (c) 1990,1991,1992 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: David Pointer (dpointer@csrd.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *	PCi586EthernetConstants.h
 *
 *	Constants used by the PCi586EthernetDriver code.
 */


#ifndef PCi586EthernetConstants_h
#define PCi586EthernetConstants_h


const int ioBase = 0x300;  // must match io base jumper on card

//--------------------------------------------------------------------------
// gate array I/O registers (offset from base address)
#define GA_PSTR    ioBase+0x400  // (RW) page start register
#define GA_PSPR    ioBase+0x401  // (RW) page stop register
#define GA_DQTR    ioBase+0x402  // (RW) DMA DRQ timer register
#define GA_BCFR    ioBase+0x403  // (RO) base config register (jumper J2)
#define GA_PCFR    ioBase+0x404  // (RO) prom/ram configuration register (J1)
#define GA_GACFR   ioBase+0x405  // (RW) gate array configuration register
#define GA_CTRL    ioBase+0x406  // (RW) gate array control register
#define GA_STREG   ioBase+0x407  // (RO) gate array status register
#define GA_IDCFR   ioBase+0x408  // (RW) interrupt/DMA config register
#define GA_DAMSB   ioBase+0x409  // (RW) DMA address Reg MSB
#define GA_DALSB   ioBase+0x40a  // (RW) DMA address Reg LSB
#define GA_VPTR2   ioBase+0x40b  // (RW) Vector Ptr Reg 2
#define GA_VPTR1   ioBase+0x40c  // (RW) Vector Ptr Reg 1
#define GA_VPTR0   ioBase+0x40d  // (RW) Vector Ptr Reg 0
#define GA_RFMSB   ioBase+0x40e  // (RW) Register File Access MSB
#define GA_RFLSB   ioBase+0x40f  // (RW) Register File Access LSB

// gate array register bits and values

// receive buffer pagfe start and stop boundaries
#define GA_PSTR_pgstrt 0x26  // allows full size 802.3 packets: 
#define GA_PSPR_pgstop 0x40  //   space for 1 tx and 4 rx packets

// gate array configuration register
#define GA_GACFR_nim   0x80  // NIC intr mask (block intr = 1)
#define GA_GACFR_tcm   0x40  // dma terminal count mask
#define GA_GACFR_0ws   0x20  // assert 0 wait state for gate array access
#define GA_GACFR_rsel  0x08  // ram select (share packet buffer = 1)
// For a 3c503 with 8k of memory, GA_GACFR_msb<2:0> must equal <001>
// From the point of view of the NIC, this 8k of memory is located
// at addresses 0x2000-0x3fff.
#define GA_GACFR_msb2  0x04  // memory bank select<2>
#define GA_GACFR_msb1  0x02  // memory bank select<1>
#define GA_GACFR_msb0  0x01  // memory bank select<0>

// gate array control register
#define GA_CTRL_start  0x80  // starts the DMA controller in the gate array
#define GA_CTRL_ddir   0x40  // dma direction (1=system to packet buffer)
#define GA_CTRL_dbsel  0x20  // double buffer select
#define GA_CTRL_share  0x10  // enables interrupt sharing
#define GA_CTRL_eahi   0x08  // selects ethernet address prom high
#define GA_CTRL_ealo   0x04  // selects ethernet address prom low
#define GA_CTRL_xsel   0x02  // 1 = BNC, 0 = DIX (external xceiver)
#define GA_CTRL_rst    0x01  // software reset

// gate array status register
#define GA_STREG_dprdy 0x80  // register files data port ready
#define GA_STREG_uflw  0x40  // read op on data files when data n/a
#define GA_STREG_oflw  0x20  // write op on data files when rf full
#define GA_STREG_dtc   0x10  // dma terminal count signal received
#define GA_STREG_dip   0x08  // dma in progress
#define GA_STREG_rev2  0x04  // gate array revision level bit 2
#define GA_STREG_rev1  0x02  // gate array revision level bit 1
#define GA_STREG_rev0  0x01  // gate array revision level bit 0

// gate array interrupt/DMA register
#define GA_IDCFR_irq5  0x80  // selects interrupt request 5
#define GA_IDCFR_irq4  0x40  // selects interrupt request 4
#define GA_IDCFR_irq3  0x20  // selects interrupt request 3
#define GA_IDCFR_irq2  0x10  // selects interrupt request 2
#define GA_IDCFR_drq3  0x04  // selects dma request 3
#define GA_IDCFR_drq2  0x02  // selects dma request 2
#define GA_IDCFR_drq1  0x01  // selects dma request 1

// vector pointer registers
#define GA_VPTR2_value 0xff  // resets memory mapped mode on a cold
#define GA_VPTR1_value 0xff  //   system reset
#define GA_VPTR0_value 0x00

// recommeded values for DMA registers
#define GA_DQTR_value  0x08  // 8 byte burst transfers
#define GA_DAMSB_value 0x20  // allows 1.5k transmit buffer
#define GA_DALSB_value 0x00

//--------------------------------------------------------------------------
// National Semiconductor DP8390 LAN Controller (NIC) register addresses
//
// These addresses can *ONLY* be accessed if GA_CTRL_<eahi:ealo> = <00>
//
// #define [reg name] [reg addr] // (NIC_CR_ps<1:0>) (access) [description]
#define NIC_CR      ioBase+0x0   // (xx) (RW) command register
#define NIC_DCR     ioBase+0xe   // (00) (W) data configuration register
                                 // (10) (R) data configuration register
#define NIC_FIFO    ioBase+0x6   // (00) (R) fifo
#define NIC_BNDY    ioBase+0x3   // (00) (RW) boundary pointer
#define NIC_CURR    ioBase+0x7   // (01) (RW) current page register
#define NIC_ACU     ioBase+0x6   // (10) (RW) address counter (upper)
#define NIC_ACL     ioBase+0x7   // (10) (RW) address counter (lower)

#define NIC_PSTART  ioBase+0x1   // (00) (W) rx page start register
                                 // (10) (R) rx page start register
#define NIC_PSTOP   ioBase+0x2   // (00) (W) rx page stop register
                                 // (10) (R) rx page stop register
#define NIC_RCR     ioBase+0xc   // (00) (W) receiver configuration register
                                 // (10) (R) receiver configuration register
#define NIC_RSR     ioBase+0xc   // (00) (R) receiver status register

#define NIC_TPSR    ioBase+0x4   // (00) (W) transmit page start addr
                                 // (10) (R) transmit page start addr
#define NIC_TSR     ioBase+0x4   // (00) (R) transmit status register
#define NIC_TBCR0   ioBase+0x5   // (00) (W) transmit byte count register 0
#define NIC_TBCR1   ioBase+0x6   // (00) (W) transmit byte count register 1
#define NIC_TCR     ioBase+0xd   // (00) (W) transmitter config register
                                 // (10) (R) transmitter config register

#define NIC_ISR     ioBase+0x7   // (00) (RW) interrupt status register
#define NIC_IMR     ioBase+0xf   // (00) (W) interrupt mask register
                                 // (10) (R) interrupt mask register

#define NIC_CRDA0   ioBase+0x8   // (00) (R) current remote dma addr 0
#define NIC_CRDA1   ioBase+0x9   // (00) (R) current remote dma addr 1
#define NIC_RNPP    ioBase+0x3   // (10) (RW) remote next packet pointer
#define NIC_RSAR0   ioBase+0x8   // (00) (W) remote start address register 0
#define NIC_RSAR1   ioBase+0x9   // (00) (W) remote start address register 1
#define NIC_RBCR0   ioBase+0xa   // (00) (W) remote byte counter register 0
#define NIC_RBCR1   ioBase+0xb   // (00) (W) remote byte counter register 1

#define NIC_CLDA0   ioBase+0x1   // (00) (R) current local dma addr 0
                                  // (10) (W) current local dma addr 0
#define NIC_CLDA1   ioBase+0x2   // (00) (R) current local dma addr 1
                                  // (10) (W) current local dma addr 1
#define NIC_LNPP    ioBase+0x5   // (10) (RW) local next packet pointer

#define NIC_NCR     ioBase+0x5   // (00) (R) number of collisions register
#define NIC_CNTR0   ioBase+0xd   // (00) (R) frame alignment errors counter
#define NIC_CNTR1   ioBase+0xe   // (00) (R) crc errors counter
#define NIC_CNTR2   ioBase+0xf   // (00) (R) missed packet errors counter

#define NIC_PAR0    ioBase+0x1   // (01) (RW) physical address register 0
#define NIC_PAR1    ioBase+0x2   // (01) (RW) physical address register 1
#define NIC_PAR2    ioBase+0x3   // (01) (RW) physical address register 2
#define NIC_PAR3    ioBase+0x4   // (01) (RW) physical address register 3
#define NIC_PAR4    ioBase+0x5   // (01) (RW) physical address register 4
#define NIC_PAR5    ioBase+0x6   // (01) (RW) physical address register 5

#define NIC_MAR0    ioBase+0x8   // (01) (RW) multicast address register 0
#define NIC_MAR1    ioBase+0x9   // (01) (RW) multicast address register 1
#define NIC_MAR2    ioBase+0xa   // (01) (RW) multicast address register 2
#define NIC_MAR3    ioBase+0xb   // (01) (RW) multicast address register 3
#define NIC_MAR4    ioBase+0xc   // (01) (RW) multicast address register 4
#define NIC_MAR5    ioBase+0xd   // (01) (RW) multicast address register 5
#define NIC_MAR6    ioBase+0xe   // (01) (RW) multicast address register 6
#define NIC_MAR7    ioBase+0xf   // (01) (RW) multicast address register 7

// LAN Controller register bits and values
#define NIC_PSTART_pgstrt  0x26      // same as GA_PSTR_pgstrt
#define NIC_PSTOP_pgstop   0x40      // same as GA_PSPR_pgstop
#define NIC_TPSR_start     0x20      // a<15:8> of start of transmit buffer
#define NIC_BNDY_init      NIC_PSTART_pgstrt      // initial value of rd ptr
#define NIC_CURR_init      NIC_PSTART_pgstrt + 1  // initial value of wr ptr

// NIC control register bits
#define NIC_CR_ps1  0x80    // NIC register page select 1
#define NIC_CR_ps0  0x40    // NIC register page select 0
#define NIC_CR_rd2  0x20    // remote DMA cmd bit 2: aborts any DMA
#define NIC_CR_rd1  0x10    // remote DMA cmd bit 1    
#define NIC_CR_rd0  0x08    // remote DMA cmd bit 0
#define NIC_CR_txp  0x04    // initiate transmission of packet
#define NIC_CR_sta  0x02    // start: activates NIC
#define NIC_CR_stp  0x01    // STOP: software reset

// NIC interrupt status register bits
#define NIC_ISR_rst  0x80   // current reset status
#define NIC_ISR_rdc  0x40   // remote dma complete
#define NIC_ISR_cnt  0x20   // counter overflow
#define NIC_ISR_ovw  0x10   // overwrite warning
#define NIC_ISR_txe  0x08   // transmit error
#define NIC_ISR_rxe  0x04   // receive errro
#define NIC_ISR_ptx  0x02   // packet transmitted
#define NIC_ISR_prx  0x01   // packet received
#define NIC_ISR_resetISR  0xff   // reset the interrupt status reg

// NIC interrupt mask register bits
#define NIC_IMR_rdce  0x40  // dma complete interrupt enable
#define NIC_IMR_cnte  0x20  // counter overflow interrupt enable
#define NIC_IMR_ovwe  0x10  // overwrite warning interrupt enable
#define NIC_IMR_txee  0x08  // transmit error interrupt enable
#define NIC_IMR_rxee  0x04  // receive error interrupt enable
#define NIC_IMR_ptxe  0x02  // packet transmitted interrupt enable
#define NIC_IMR_prxe  0x01  // packet received interrupt enable
#define NIC_IMR_all   0x3f  // enable all interrupts but remote DMA

// NIC data configuration register bits
#define NIC_DCR_ft1  0x40   // fifo threshhold bit 1
#define NIC_DCR_ft0  0x20   // fifo threshhold bit 0
#define NIC_DCR_arm  0x10   // autoinit remote
#define NIC_DCR_bms  0x08   // burst mode select
#define NIC_DCR_las  0x04   // long address select
#define NIC_DCR_bos  0x02   // byte order select
#define NIC_DCR_wts  0x01   // word transfer select

// NIC transmit configuration register
#define NIC_TCR_ofst 0x10   // collision offset enable
#define NIC_TCR_atd  0x08   // auto transmit enable
#define NIC_TCR_lb1  0x04   // encoded loopback control bit 1
#define NIC_TCR_lb0  0x02   // encoded loopback control bit 0
#define NIC_TCR_crc  0x01   // inhibit crc
#define NIC_TCR_operation   0   // normal operation

// NIC transmit status register bits
#define NIC_TSR_owc  0x80  // out of window collision
#define NIC_TSR_cdh  0x40  // collision detect heartbeat
#define NIC_TSR_fu   0x20  // fifo underrun
#define NIC_TSR_crs  0x10  // carrier sense lost
#define NIC_TSR_abt  0x08  // transmit aborted
#define NIC_TSR_col  0x04  // transmit collided
#define NIC_TSR_dfr  0x02  // non-deferred transmission
#define NIC_TSR_ptx  0x01  // packet transmitted

// NIC receive configuration register
#define NIC_RCR_mon  0x20   // monitor mode
#define NIC_RCR_pro  0x10   // promiscuous physical
#define NIC_RCR_am   0x08   // accept multicast
#define NIC_RCR_ab   0x04   // accept broadcast
#define NIC_RCR_ar   0x02   // accept runt packets
#define NIC_RCR_sep  0x01   // save errored packets
#define NIC_RCR_operation 0 // normal operation

// NIC receive status register bits
#define NIC_RSR_dfr  0x80  // deferring
#define NIC_RSR_dis  0x40  // receiver disabled
#define NIC_RSR_phy  0x20  // physical/multicast address
#define NIC_RSR_mpa  0x10  // missed packet
#define NIC_RSR_fo   0x08  // fifo overrun
#define NIC_RSR_fae  0x04  // frame alignment error
#define NIC_RSR_crc  0x02  // crc error
#define NIC_RSR_prx  0x01  // packet received intact

//--------------------------------------------------------------------------
// Ethernet Station Address Bytes
//
// These addresses can *ONLY* be accessed if GA_CTRL_<eahi:ealo> = <01>
//
#define EA_BYTE5   ioBase+0x5      // most sig byte of station address
#define EA_BYTE4   ioBase+0x4
#define EA_BYTE3   ioBase+0x3
#define EA_BYTE2   ioBase+0x2
#define EA_BYTE1   ioBase+0x1
#define EA_BYTE0   ioBase+0x0      // least sig byte of station address



#endif /* PCi586EthernetConstants_h */
