/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 *
 * Copyright (c) 1987,1988,1989,1990,1991,1992  The University of Illinois
 *						Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 * 
 * Author: David B. Pointer (dpointer@csrd.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * PCi586EthernetDriver.h: Description class for the 3com 3c503 Etherlink II
 *                          ethernet controller card in the PCi586 machine.
 *
 *   $Header: /usr/local/cvsroot/Choices/vcnew/Includes/MachineDependent/PCi586/PCi586EthernetDriver.h,v 1.3 2004/05/03 17:09:19 neelakan Exp $
 */

#ifndef PCi586EthernetDriver_h
#define PCi586EthernetDriver_h

#include "Types.h"
#include "NetworkDriver.h"
#include "Ethernet.h"
#include "EthernetDriver.h"
#include "ProxiableObject.h"
#include "NetworkStatistics.h"

#include "PCi586EthernetConstants.h"

const unsigned short maxEthernetDataSize = (256*6);
const unsigned short minEthernetDataSize = 46;
const int ethernetAddressSize = 6;
const int multicastAddressSize = 8;

const int noMemoryError = 1;
const int memoryError = 0;


//--------------------------------------------------------------------------
class EthernetPacket {
public:
  unsigned char destinationAddress[ethernetAddressSize];  // [0] == lsb
  unsigned char sourceAddress[ethernetAddressSize];       // [0] == lsb
  unsigned short packetDataLength;
  unsigned char packetData[maxEthernetDataSize];          // [0] == lsb
};  

/* class PCi586EthernetDriver
 *
 */
extern ClassStar PCi586EthernetDriverClass;

class PCi586EthernetDriver : public EthernetDriver {

  unsigned char etherAddress[ethernetAddressSize];
  unsigned char multicastAddress[multicastAddressSize];
  unsigned char *memoryBase;
  unsigned char nextPacket;    // read ptr to next packet (not NIC_BNDY!)

  int testMemory();
  void stopNIC();
  void restartNIC();

  virtual void processRecv();

  // this function fulfills an EthernetAdministrator function that
  // is used for driver testing
  EthernetPacket * getNewEthernetPacketPtr();
	// ***** WARNING ... THIS IS OBSOLETE AND SHOULD BE REMOVED // -- Amit

public:

  PCi586EthernetDriver( unsigned char debugMask = 0 );
  virtual ~PCi586EthernetDriver()
	{}

  void oldGetEthernetAddress( unsigned char * buffer );  	// -- Amit
  EthernetAddress * getEthernetAddress();  			// -- Amit

  virtual int transmitPacket( NetworkBuffer * );		// -- Amit

  // should *only* be called from a member of the class
  // PCi586EthernetDriverInterrupt
  void ethernetInterruptHandler( unsigned char interruptStatusRegister );

  Class * classOf();               // proxiable objects need this
};

#endif /* PCi586EthernetDriver_h */
