/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * PCi586FreeRunningTimer.h: free running timer for the PCi586
 */

#ifndef PCi586FreeRunningTimer_h
#define PCi586FreeRunningTimer_h
#ifdef CFRONT
#define CONST const
#define STATIC static
#else /* must be g++ */
//#pragma interface
#if (__GNUG__ >= 2)
#ifndef CONST
#define CONST
#define STATIC
#endif
#else
#pragma once
#define CONST const
#define STATIC static
#endif
#endif

#include "FreeRunningTimer.h"	/* parent class */
#include "Lock.h"
#include "PCi586IntervalTimer.h"
#include "ThisCPU.h"

/* The ClockFrequency should be even divisor of 1000000 */
const unsigned int ClockFrequency = 100; /* ticks per second */

/* on the PCi586, can't be more than ~50 milliseconds per tick */
const unsigned int PITCount = PIT_HZ / ClockFrequency;

extern ClassStar PCi586FreeRunningTimerClass;
class PCi586FreeRunningTimer : public FreeRunningTimer {
	unsigned int lastTime;
	Lock lock;
public:
	Class * classOf();
	PCi586FreeRunningTimer();

	virtual unsigned int time();
	/*
	 * This should be called directly from the hardware interrupt every
	 *   clock tick.
	 */
	void tick();
};

STATIC inline
PCi586FreeRunningTimer::PCi586FreeRunningTimer()
{
	lastTime = 0;
}

STATIC inline void
PCi586FreeRunningTimer::tick()
{
	lastTime += 1000000/ClockFrequency;
}

#endif /* PCi836FreeRunningTimer_h */
