/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * PCI586IntervalTimer.h - defines for the AT&T 6586 PC programmable
 *	interval timer (Intel 8254)
 */

#ifndef PCi586IntervalTimer_h
#define PCi586IntervalTimer_h
#ifdef CFRONT
#define CONST const
#define STATIC static
#else /* must be g++ */
#pragma interface
#if (__GNUG__ >= 2)
#ifndef CONST
#define CONST
#define STATIC
#endif
#else
#pragma once
#define CONST const
#define STATIC static
#endif
#endif

const int PITCTR0_PORT = 0x40;	/* counter 0 port */
const int PITCTR1_PORT = 0x41;	/* counter 1 port */
const int PITCTR2_PORT = 0x42;	/* counter 2 port */
const int PITCTL_PORT  = 0x43;	/* PIT control port */

/*
 * Control port bits
 */
/* select conter bits */
const int PIT_SEL0 = 0x00;	/* select counter 0 */
const int PIT_SEL1 = 0x40;	/* select counter 1 */
const int PIT_SEL2 = 0x80;	/* select counter 2 */
const int PIT_READBACK = 0xc0;	/* read-back command */
/* read/write bits */
const int PIT_LATCH = 0x00;	/* counter latch command */
const int PIT_LEAST = 0x10;	/* read/write least significant byte */
const int PIT_MOST = 0x20;	/* read/write most significant byte */
const int PIT_BOTH = 0x30;	/* read/write both bytes */
/* mode bits */
const int PIT_MODE0 = 0x00;	/* interrupt on terminal count */
const int PIT_MODE1 = 0x02;	/* hardware retriggerable one-shot */
const int PIT_MODE2 = 0x04;	/* rate generator */
const int PIT_MODE3 = 0x06;	/* square wave mode */
const int PIT_MODE4 = 0x08;	/* software triggered strobe */
const int PIT_MODE5 = 0x0a;	/* hardware triggered strobe */
/* bcd bit */
const int PIT_BINARY = 0x00;	/* 16-bit binary counter */
const int PIT_BCD = 0x01;	/* binary-coded decimal, 4 decades */



const int PIT_HZ = 1193167;	/* ticks per second on the counters */
/* PIT_HZ = PIT_HZFACT1 * PIT_HZFACT2 */
const int PIT_HZFACT1 = 113;
const int PIT_HZFACT2 = 10559;


#endif /* PCi586IntervalTimer_h */
