/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * PCi586Kernel.h: Machine description class for the PCi586 Kernel.
 */

#ifndef PCi586Kernel_h
#define PCi586Kernel_h
#ifdef CFRONT
#define CONST const
#define STATIC static
#else /* must be g++ */
//#pragma interface
#if (__GNUG__ >= 2)
#if (!defined CONST)
#define CONST
#define STATIC
#endif
#else
#pragma once
#define CONST const
#define STATIC static
#endif
#endif

#include "Kernel.h"
#include "Process.h"
#include "PCi586Disk.h"
#include "PCi586DiskStar.h"

class MemoryObjectContainer;
class PhysicalMemoryRange;
class PCi586EthernetDriver;

extern ClassStar PCi586KernelClass;
class PCi586Kernel : public Kernel {

protected:

        PCi586DiskStar theDisk;
	MemoryObjectContainer * theRootContainer;

	PCi586EthernetDriver * theEthernetDriver;

	PhysicalMemoryRangeRef configure();

	/*
	 * Overloads of parent class methods.
	 */
	virtual void buildRunningCPU();
	virtual void configureHardware(){};
	virtual PhysicalMemoryRange *	physicalMemoryMap() ;
	virtual const unsigned          physicalMemorySize();
	virtual const char *		kernelStart() ;
	virtual const char *		applicationStart() ;
	virtual const unsigned		kernelHeapSize() ;
	virtual const unsigned		kernelSize() ;
	virtual const unsigned		applicationSize() ;
	virtual const unsigned		addressTranslationMemorySize() ;

	virtual void basicInitializeDevicesAndManagers();
	virtual void basicInitializeNetworkProcesses();

	virtual void mountNonRootFileSystems();
	virtual void initializeBackingStore();

	PersistentMemoryObjectRef allocateMemoryObjectForBackingStore();

	virtual int isObjectFile( PersistentMemoryObject * objectFile );
	virtual const char * loadObjectFile(
				PersistentMemoryObject * objectFile,
				Domain * newProcessesDomain );	

public:
	PCi586Kernel();
	~PCi586Kernel();

	DiskRef rootDisk();

	Class * classOf();
};

extern unsigned int begin;
//JMT HACK FIXME
const unsigned int KERNELSTART = 0x100000;
const int KERNELSIZE = 0x2000000-KERNELSTART;	// 32 Mb - KERNELSTART
const unsigned int APPLICATIONSTART = KERNELSTART+KERNELSIZE;
const unsigned int APPLICATIONSIZE = 4 * KERNELSIZE;
const int KERNELHEAPSIZE = 0xa00000; 		// 10 Mb
const int ADDRESSTRANSLATIONMEMORYSIZE = 4096 * 75;

inline
const char *		
PCi586Kernel::kernelStart() {return (const char *)KERNELSTART;}

inline
const char *		
PCi586Kernel::applicationStart(){return (const char *)APPLICATIONSTART;}

inline
const unsigned		
PCi586Kernel::kernelHeapSize(){return KERNELHEAPSIZE;} 

inline
const unsigned		
PCi586Kernel::kernelSize(){return KERNELSIZE;} 

inline
const unsigned		
PCi586Kernel::applicationSize(){return APPLICATIONSIZE;} 

inline
const unsigned		
PCi586Kernel::addressTranslationMemorySize()
{
	return ADDRESSTRANSLATIONMEMORYSIZE;
}

#endif /* PCi586Kernel_h */
