/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 */

#ifndef PCi586Multiboot_h
#define PCi586Multiboot_h

 /* The magic number for the Multiboot header.  */
#define MULTIBOOT_HEADER_MAGIC		0x1BADB002

/* The flags for the Multiboot header.  */
#define MULTIBOOT_HEADER_FLAGS		0x00000003

#define MULTIBOOT_HEADER_CHECKSUM   -(MULTIBOOT_HEADER_MAGIC + MULTIBOOT_HEADER_FLAGS)

/* The magic number passed by a Multiboot-compliant boot loader.  */
#define MULTIBOOT_BOOTLOADER_MAGIC	0x2BADB002

/* Macro to check if the bit BIT in FLAGS is set.  */
#define MULTIBOOT_CHECK_FLAG(flags,bit)	((flags) & (1 << (bit)))

/* We statically allocate space to copy the memory, drive and module linked lists
   passed from the multiboot-compliant boot loader */
#define MAX_MULTIBOOT_ENTRIES 16

#ifndef ASM
/* Do not include the following in boot.s  */

/* The Multiboot header.  */
typedef struct multiboot_header
{
  unsigned long magic;
  unsigned long flags;
  unsigned long checksum;

  /* Optional (depending on value of flags) */
//  unsigned long header_addr;
//  unsigned long load_addr;
//  unsigned long load_end_addr;
//  unsigned long bss_end_addr;
//  unsigned long entry_addr;
} multiboot_header_t;

/* The section header table for ELF.  */
typedef struct elf_section_header_table
{
  unsigned long num;
  unsigned long size;
  unsigned long addr;
  unsigned long shndx;
} elf_section_header_table_t;

/* The Multiboot information.  */
typedef struct multiboot_info
{
  unsigned long flags;
  unsigned long mem_lower;
  unsigned long mem_upper;
  unsigned long boot_device;
  unsigned long cmdline;
  unsigned long mods_count;
  unsigned long mods_addr;
  elf_section_header_table_t elf_sec;
  unsigned long mmap_length;
  unsigned long mmap_addr;
  unsigned long drives_length;
  unsigned long drives_addr;
} multiboot_info_t;

/* The module structure.  */
typedef struct module
{
  unsigned long mod_start;
  unsigned long mod_end;
  unsigned long string;
  unsigned long reserved;
} module_t;

/* The memory map. Be careful that the offset 0 is base_addr_low (not size).  */
typedef struct memory_map
{
  unsigned long size; // offset -4
  unsigned long base_addr_low; // offset 0
  unsigned long base_addr_high;
  unsigned long length_low;
  unsigned long length_high;
  unsigned long type;
} memory_map_t;

/* The drive info structure.  */
typedef struct drive_info
{
  unsigned long size;
  unsigned char drive_number;
  unsigned char drive_mode;
  unsigned short drive_cylinders;
  unsigned char drive_heads;
  unsigned char drive_sectors;
} drive_info_t;

extern unsigned long multiboot_magic;
extern multiboot_info_t multiboot_info;
extern unsigned num_modules;
extern module_t modules[MAX_MULTIBOOT_ENTRIES];

extern unsigned num_memory_maps;
extern memory_map_t memory_map[MAX_MULTIBOOT_ENTRIES];

extern unsigned num_drives;
extern drive_info_t drive_info[MAX_MULTIBOOT_ENTRIES];

#endif /* ASM */

#endif /* PCi586Multiboot_h */
