/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * PCi586ReferenceCount.h - An atomic integer on the MAX.
 */

#ifndef PCi586ReferenceCount_h
#define PCi586ReferenceCount_h
#ifdef CFRONT
#define CONST const
#define STATIC static
#else /* must be g++ */
#pragma interface
#if (__GNUG__ == 2)
#define CONST
#define STATIC
#else
#pragma once
#define CONST const
#define STATIC static
#endif
#endif

#include "Assert.h"

class PCi586ReferenceCount {
protected:
	short count;
public:
	PCi586ReferenceCount();
	~PCi586ReferenceCount();
	int value();
	void increment();
	int decrement();
};

extern unsigned int DisableInterrupts();
extern void EnableInterrupts();

inline
PCi586ReferenceCount::PCi586ReferenceCount()
{
	count = 0;
}

inline
PCi586ReferenceCount::~PCi586ReferenceCount()
{
	Assert( count == 0 );
}

inline int
PCi586ReferenceCount::value()
{
	return( count );
}

inline void
PCi586ReferenceCount::increment()
{
	unsigned int intstate;

	intstate = DisableInterrupts();

	count++;

	if ( intstate )
		/* reenable if they were enabled */
		EnableInterrupts();
}

inline int
PCi586ReferenceCount::decrement()
{
	int retval;
	unsigned int intstate;

	intstate = DisableInterrupts();

	retval = --count;

	if ( intstate ) 
		/* reenable if they were enabled */
		EnableInterrupts();

	return( retval );
}

#endif	/* PCi586ReferenceCount_h */
