/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Contributing Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Contributing Author: David Dykstra (dykstra@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * PCi586TimesliceTimer.h - Timeslice timer for the PCi586
 */

#ifndef PCi586TimesliceTimer_h
#define PCi586TimesliceTimer_h
#ifdef CFRONT
#define CONST const
#define STATIC static
#else /* must be g++ */
//#pragma interface
#if (__GNUG__ >= 2)
#if (!defined CONST)||(!defined STATIC)
#define CONST
#define STATIC
#endif
#else
#pragma once
#define CONST const
#define STATIC static
#endif
#endif

#include "Timer.h"	/* parent class */
#include "TimeSliceInterrupt.h"
#include "Lock.h"	/* do get definition of Lock */

extern TimeSliceInterrupt TSliceInterrupt;

class PCi586TimesliceTimer : public Timer {
	// these can be static because there's only one
	//  TimesliceTimer on the PCi586

	static Lock *lock;
	static unsigned int tickTime;

	static unsigned int expireTime;
	static char running;

public:
	PCi586TimesliceTimer();
	~PCi586TimesliceTimer();

	unsigned int start( unsigned int microseconds );
	void set( unsigned int microseconds );
	unsigned int stop();
	unsigned int residual();
	int idle();
	/*
	 * tick should be called directly from the hardware timer interrupt.
	 *   every 1000000/ClockFrequency microseconds.
	 */
	static void tick();
};

STATIC inline void
PCi586TimesliceTimer::tick()
{
    lock->acquire();
    if ((++tickTime == expireTime) && running) {
	running = 0;
	lock->release();
	TSliceInterrupt.raise();
    }
    else
	lock->release();
}

#endif /* PCi586TimesliceTimer_h */
