/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Bjorn A. Helgaas (helgaas@cs.uiuc.edu)
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * Vectors.h: vector numbers for processor traps/interrupts.
 */

#ifndef Vectors_h
#define Vectors_h
#ifdef CFRONT
#define CONST const
#define STATIC static
#else /* must be g++ */
#pragma interface
#if (__GNUG__ >= 2)
#if (!defined CONST)
#define CONST
#define STATIC
#endif
#else
#pragma once
#define CONST const
#define STATIC static
#endif
#endif

/*
 * NOTE:  This file is included in assembler source so must use #defines
 *  instead of const ints.  For the same reason, there can't be
 *  parentheses around the expressions because the assembler doesn't
 *  allow them.
 */

/*
 * Interrupt Vectors
 */
#define GENERALPROTECTION_Vector 13
#define PAGEFAULT_Vector  	14

#define HARDWARE_Vectors  	0x18

#define INTERVAL_Vector 	HARDWARE_Vectors + 0
#define KEYBOARD_Vector 	HARDWARE_Vectors + 1
#define COM1_Vector 		HARDWARE_Vectors + 3
#define COM0_Vector 		HARDWARE_Vectors + 4

#define ETH_Vector		HARDWARE_Vectors + 5  /* 3c503 Ethernet */

#define DK_Vector 		HARDWARE_Vectors + 15

#define CONSOLEIN_Vector 	KEYBOARD_Vector
#define CONSOLEOUT_Vector 	COM0_Vector

/*
 * The following are not interrupts, they're in the Gdt.
 */
#define SYSTEM_CS		0x08	/* 1<<3 */
#define SYSTEM_DS		0x10	/* 2<<3 */
#define SYSTEM_TSS		0x18	/* 3<<3 */

#define PROXY_Vector 		4
#define PROXYRETURN_Vector 	5
/*
 * The next (APPLICATIONRING * 2) segments are used for
 *   segments for code and data for the rings above kernel.
 */
#define USERSEG_Vectors		6

/* Entries 0xe through 0x2f are used by CDB */
#define MAXGdt_Vector		0xe

#endif /* Vectors_h */
