/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef Gdb_h
#define Gdb_h


#ifdef __GNUC__
#pragma interface
#endif

#define GDB
#include "MACHINE_GDB.h"

/* 
 * Interface definitions for the machine/processor dependent
 * portions of GDB.  Include this in your machine dependent
 * stuff.
 */
extern char 		GetDebugChar();
extern void 			PutDebugChar(char);
extern int 			ComputeSignal(int);
extern void			GdbKill(); 		// Kill & reboot.
extern unsigned			GdbBreakInsn;		// Opcode for break
/*
 * These are called at entry/exit of handler.  You can use them to set
 * up things like info to tell page fault handlers that faults should be
 * handled in user mode, etc.
 */
extern void			GdbMachineHandlerStart(int, char*); 
extern void			GdbMachineHandlerEnd(int, char*);

        /*
         * Provided per machine:
         * Tell if Memory Access at addr -> addr+len is ok
         * for the type of access requested
	 * return 0 for success, non-zero for failure
         */
const unsigned GDB_READ = 0;
const unsigned GDB_WRITE = 1;
extern int
GdbMachineMem_ok( unsigned acc_type, char *addr, unsigned len);


/* 
 * Use C linkage so we can use _GDBhandler in the assembly 
 * code without name munging.
 */
extern "C" {
	void GdbHandler(int, char *);
}

/*
 * Stuff that is internal to the GDB handler, but you might want to
 * use it somewhere else, like to send a packet or something strange.
 */
extern void			GdbGetPacket(char *);
extern void			GdbPutPacket(char *);
extern int			GdbRemoteDebugging;

#endif /*Gdb_h*/
