/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
#ifndef SCSICommand_h
#define SCSICommand_h


#ifdef __GNUC__
#pragma interface
#endif

/* Group 0 Commands for Direct Access Devices */
 const unsigned char C0_TEST_UNIT_READY = 0x00; /* optional */
 const unsigned char C0_REZERO_UNIT     = 0x01; /* optional */
 const unsigned char C0_REQUEST_SENSE   = 0x03; /* mandatory */
 const unsigned char C0_FORMAT_UNIT     = 0x04; /* mandatory */
 const unsigned char C0_REASSGN_BLCKS   = 0x07; /* optional */
 const unsigned char C0_READ            = 0x08; /* mandatory */
 const unsigned char C0_WRITE           = 0x0a; /* mandatory */
 const unsigned char C0_SEEK            = 0x0b; /* optional */
 const unsigned char C0_INQUIRY         = 0x12; /* extended */
 const unsigned char C0_MODE_SELECT     = 0x15; /* optional */
 const unsigned char C0_RESERVE         = 0x16; /* optional */
 const unsigned char C0_RELEASE         = 0x17; /* optional */
 const unsigned char C0_COPY            = 0x18; /* optional */
 const unsigned char C0_MODE_SENSE      = 0x1a; /* optional */
 const unsigned char C0_START_STOP_UNIT = 0x1b; /* optional */
 const unsigned char C0_REC_DIAG_RES    = 0x1c; /* optional */
 const unsigned char C0_SEND_DIAG       = 0x1d; /* optional */
 const unsigned char C0_PREVENT_ALLOW   = 0x1e; /* optional */

/* Group 1 Commands for Direct Access Devices */
 const unsigned char C1_READ_CAP        = 0x25; /* extended */
 const unsigned char C1_READ            = 0x28; /* extended */
 const unsigned char C1_WRITE           = 0x2a; /* extended */
 const unsigned char C1_SEEK            = 0x2b; /* optional */
 const unsigned char C1_WRITE_VER       = 0x2e; /* optional */
 const unsigned char C1_VERIFY          = 0x2f; /* optional */
 const unsigned char C1_SEARCH_HI       = 0x30; /* optional */
 const unsigned char C1_SEARCH_EQ       = 0x31; /* optional */
 const unsigned char C1_SEARCH_LO       = 0x32; /* optional */
 const unsigned char C1_SET_LIMITS      = 0x33; /* optional */
 const unsigned char C1_COMPARE         = 0x39; /* optional */
 const unsigned char C1_COPY_VER        = 0x3a; /* optional */

/* Definitions for the control Byte */
 const unsigned char NO_LINK         = 0x00;
 const unsigned char LINKED          = 0x01;
 const unsigned char LINKED_W_FLAG   = 0x03;



/* The most general SCSICommand class */
class SCSICommand {
 protected:
  int myLengthInBytes;
  unsigned char byteArray[12];
 public:
  SCSICommand(int);
  void init(int);
  int length();
  unsigned char byte(int);
  unsigned char * dataLoc();
};

inline
unsigned char *
 SCSICommand::dataLoc()
{
  return (byteArray);
}

inline
 SCSICommand::SCSICommand (int len)
{
  myLengthInBytes = len;
}

inline void
 SCSICommand::init(int len)
{
  myLengthInBytes = len;
}

inline int
 SCSICommand::length()
{
  return myLengthInBytes;
}

inline unsigned char 
 SCSICommand::byte(int i)
{
  return byteArray[i];
}


/* SCSICommand group subclasses */


class SCSICommandGroup0: public SCSICommand {
 public:
  SCSICommandGroup0(unsigned char opcode, unsigned char logicalUnitNumber,
		unsigned int logicalBlockAddress, unsigned char transferLength,
		unsigned char controlByte);
  void init(unsigned char opcode, unsigned char logicalUnitNumber,
	    	unsigned int logicalBlockAddress, unsigned char transferLength,
		unsigned char controlByte);
};

inline
SCSICommandGroup0::SCSICommandGroup0
  (unsigned char opcode, unsigned char logicalUnitNumber,
   unsigned int logicalBlockAddress, unsigned char transferLength,
   unsigned char controlByte) : SCSICommand(6)
{
  byteArray[0] = opcode;
  byteArray[1] = (logicalUnitNumber << 5) | (logicalBlockAddress >> 16);
  byteArray[2] = logicalBlockAddress >> 8;
  byteArray[3] = logicalBlockAddress;
  byteArray[4] = transferLength;
  byteArray[5] = controlByte;
}

inline void
SCSICommandGroup0::init
  (unsigned char opcode, unsigned char logicalUnitNumber,
   unsigned int logicalBlockAddress, unsigned char transferLength,
   unsigned char controlByte)
{
  SCSICommand::init(6);
  byteArray[0] = opcode;
  byteArray[1] = (logicalUnitNumber << 5) | (logicalBlockAddress >> 16);
  byteArray[2] = logicalBlockAddress >> 8;
  byteArray[3] = logicalBlockAddress;
  byteArray[4] = transferLength;
  byteArray[5] = controlByte;
}

/*************************************************************
 *                                                           *
 *            Mode Sense Command                             *
 *                                                           *
 *************************************************************/
class ModeSenseCommand: public SCSICommandGroup0 {
 public:
  ModeSenseCommand(unsigned char logicalUnitNumber,
                 unsigned int allocationLength,
                 unsigned char controlByte = NO_LINK);
  void init(unsigned char logicalUnitNumber,
                 unsigned int allocationLength,
                 unsigned char controlByte = NO_LINK);
};
inline
 ModeSenseCommand::ModeSenseCommand
  (unsigned char logicalUnitNumber,unsigned int allocationLength,
   unsigned char controlByte)
: SCSICommandGroup0(C0_MODE_SENSE, logicalUnitNumber,0,allocationLength, controlByte)
{
}
inline void
 ModeSenseCommand::init
  (unsigned char logicalUnitNumber,unsigned int allocationLength,
   unsigned char controlByte)
{
  SCSICommandGroup0::init(C0_MODE_SENSE, logicalUnitNumber,0,allocationLength, controlByte);
}

/*************************************************************
 *                                                           *
 *                Test Unit Ready Command                    *
 *                                                           *
 *************************************************************/
class TestUnitReadyCommand: public SCSICommandGroup0 {
 public:
  TestUnitReadyCommand(unsigned char logicalUnitNumber,
                 unsigned int allocationLength,
                 unsigned char controlByte = NO_LINK);
  void init(unsigned char logicalUnitNumber,
                 unsigned int allocationLength,
                 unsigned char controlByte = NO_LINK);
};
inline
 TestUnitReadyCommand::TestUnitReadyCommand
  (unsigned char logicalUnitNumber,unsigned int allocationLength,
   unsigned char controlByte)
: SCSICommandGroup0(C0_TEST_UNIT_READY, logicalUnitNumber,0,allocationLength, controlByte)
{
}
inline void
 TestUnitReadyCommand::init
  (unsigned char logicalUnitNumber,unsigned int allocationLength,
   unsigned char controlByte)
{
  SCSICommandGroup0::init(C0_TEST_UNIT_READY, logicalUnitNumber,0,allocationLength, controlByte);
}
 
/*************************************************************
 *                                                           *
 *                Read Command -- Group 0                    *
 *                                                           *
 *************************************************************/
class ReadCommandGroup0: public SCSICommandGroup0 {
 public:
  ReadCommandGroup0(unsigned char logicalUnitNumber,
		    unsigned int logicalBlockAddress,
		    unsigned char transferLength,
		    unsigned char controlByte = NO_LINK);
  void init(unsigned char logicalUnitNumber,
		    unsigned int logicalBlockAddress,
		    unsigned char transferLength,
		    unsigned char controlByte = NO_LINK);
};
inline
 ReadCommandGroup0::ReadCommandGroup0
  (unsigned char logicalUnitNumber, 
   unsigned int logicalBlockAddress,
   unsigned char transferLength,
   unsigned char controlByte)
: SCSICommandGroup0(C0_READ, logicalUnitNumber,logicalBlockAddress,
  transferLength,controlByte)
{
}
inline void
 ReadCommandGroup0::init
  (unsigned char logicalUnitNumber,
   unsigned int logicalBlockAddress,
   unsigned char transferLength,
   unsigned char controlByte)
{
  SCSICommandGroup0::init(C0_READ, logicalUnitNumber,logicalBlockAddress,
  transferLength,controlByte);
}

/*************************************************************
 *                                                           *
 *                Write Command -- Group 0                   *
 *                                                           *
 *************************************************************/
class WriteCommandGroup0: public SCSICommandGroup0 {
 public:
  WriteCommandGroup0(unsigned char logicalUnitNumber,
   unsigned int logicalBlockAddress,
   unsigned char transferLength,
   unsigned char controlByte = NO_LINK);
  void init(unsigned char logicalUnitNumber,
   unsigned int logicalBlockAddress,
   unsigned char transferLength,
   unsigned char controlByte = NO_LINK);
};
inline
 WriteCommandGroup0::WriteCommandGroup0
  (unsigned char logicalUnitNumber, 
   unsigned int logicalBlockAddress,
   unsigned char transferLength,
   unsigned char controlByte )
: SCSICommandGroup0(C0_WRITE, logicalUnitNumber,logicalBlockAddress,
  transferLength,controlByte)
{
}
inline void
 WriteCommandGroup0::init
  (unsigned char logicalUnitNumber,
   unsigned int logicalBlockAddress,
   unsigned char transferLength,
   unsigned char controlByte)
{
  SCSICommandGroup0::init(C0_WRITE, logicalUnitNumber,logicalBlockAddress,
  transferLength,controlByte);
}

#ifdef NEVER
class SCSICommandGroup1: public SCSICommand{
 public:
  SCSICommandGroup1(unsigned char opcode, unsigned char logicalUnitNumber, 
		unsigned char relAddressBit, long logicalBlockAddress, 
		unsigned int transferLength, unsigned char controlByte);
};


class SCSICommandGroup5: public SCSICommand{
 public:
  SCSICommandGroup5(unsigned char opcode, unsigned char logicalUnitNumber,
                unsigned char relAddressBit, long logicalBlockAddress,
                unsigned int transferLength, unsigned char controlByte);
};



/*************************************************************
 *                                                           *
 *            Inquiry Command                                *
 *                                                           *
 *************************************************************/

class InquiryCommand: public SCSICommandGroup0 {
 public:
  InquiryCommand(unsigned char logicalUnitNumber, 
		 unsigned int allocationLength,
		 unsigned char controlByte = NO_LINK);
};

inline
 InquiryCommand::InquiryCommand
  (unsigned char logicalUnitNumber,unsigned int allocationLength,
   unsigned char controlByte = NO_LINK)
:SCSICommandGroup0(C0_INQUIRY, logicalUnitNumber,0,allocationLength, controlByte)
{
}
  
/*************************************************************
 *                                                           *
 *               Request Sense Command                       *
 *                                                           *
 *************************************************************/

class RequestSenseCommand: public SCSICommandGroup0 {
 public:
  RequestSenseCommand(unsigned char logicalUnitNumber,
                 unsigned int allocationLength,
                 unsigned char controlByte = NO_LINK);
};

inline
 RequestSenseCommand::RequestSenseCommand
  (unsigned char logicalUnitNumber,unsigned int allocationLength, 
   unsigned char controlByte = NO_LINK)
:SCSICommandGroup0(C0_REQUEST_SENSE, logicalUnitNumber,0,allocationLength, controlByte)
{
}


/*************************************************************
 *                                                           *
 *                Read Command -- Group 1                    *
 *                                                           *
 *************************************************************/
class ReadCommandGroup1: public SCSICommandGroup1 {
 public:
  ReadCommandGroup1(unsigned char logicalUnitNumber,
                unsigned char relAddressBit, int logicalBlockAddress,
                unsigned int transferLength, unsigned char controlByte);
};


inline
 ReadCommandGroup1::ReadCommandGroup1(unsigned char logicalUnitNumber,
                unsigned char relAddressBit, int logicalBlockAddress,
                unsigned int transferLength, unsigned char controlByte):
  SCSICommandGroup1(C1_READ,logicalUnitNumber,relAddressBit,logicalBlockAddress,
   transferLength,controlByte)
{
}


/*************************************************************
 *                                                           *
 *                Write Command -- Group 1                   *
 *                                                           *
 *************************************************************/
class WriteCommandGroup1: public SCSICommandGroup1 {
 public:
  WriteCommandGroup1(unsigned char logicalUnitNumber,
                unsigned char relAddressBit, int logicalBlockAddress,
                unsigned int transferLength, unsigned char controlByte);
};


inline
 WriteCommandGroup1::WriteCommandGroup1(unsigned char logicalUnitNumber,
                unsigned char relAddressBit, int logicalBlockAddress,
                unsigned int transferLength, unsigned char controlByte):
  SCSICommandGroup1(C1_WRITE,logicalUnitNumber,relAddressBit,logicalBlockAddress,
   transferLength,controlByte)
{
}


/*************************************************************
 *                                                           *
 *                Read Capacity Command                      *
 *                                                           *
 *************************************************************/

class ReadCapacityCommand: public SCSICommandGroup1 {
 public:
  ReadCapacityCommand(unsigned char logicalUnitNumber);
};

inline
 ReadCapacityCommand::ReadCapacityCommand (unsigned char logicalUnitNumber):
  SCSICommandGroup1(C1_READ_CAP,logicalUnitNumber,0,0,0,0)
{
}
#endif /* NEVER*/
#endif 
