/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
#ifndef SCSIDevice_h
#define SCSIDevice_h

#ifdef __GNUC__
#pragma interface
#endif


#include "SemaphoreStar.h"
class SCSICommand;

	/*
	 * Generic things for targets and controllers
	 */
class SCSIDevice {
 protected:
  unsigned char myBusId;

  unsigned char myTimeout;
  unsigned char mySyncOffset;
  unsigned char mySyncPeriod;
  unsigned char *myDataPointer;
  unsigned 	dataPointerLen;

  SCSICommand *myCommand;
  
 public:
  SCSIDevice(unsigned char busId);
  virtual ~SCSIDevice(){};

  unsigned char *dataPointer() {return myDataPointer;}
  unsigned dataLen() {return dataPointerLen;}

  unsigned char busId() {return myBusId;}

  unsigned char timeout() {return myTimeout;}
  void timeout(unsigned char time) {myTimeout = time;}
  unsigned char syncOffset() {return mySyncOffset;}
  void syncOffset(unsigned char offset) {mySyncOffset = offset;}
  unsigned char syncPeriod() {return mySyncPeriod;}
  void syncPeriod(unsigned char period) {mySyncPeriod = period;}
};

enum SCSIError {
  ERR_NO_ERROR = 0, ERR_DMA, ERR_DMA_REQ_PEND, ERR_PARITY, ERR_GROSS,
  ERR_RESET, ERR_NO_COMPL, ERR_NO_RESPONSE
  };

class SCSIController: public SCSIDevice {
const unsigned DEFAULT_CONTROLLER_BUS_ID = 7;
const unsigned MAX_TARGETS = 8;

protected:
	char *_dataBuf;
	unsigned _dataLen;
	char *_commandBuf;
	unsigned _commandLen;

  SemaphoreStar _mutex;
  SemaphoreStar _signalSemaphore;

  /*
   * The table of targets on our bus.
   * They are initialized in initializeTargetTable();
   */
  SCSIDevice *targets[MAX_TARGETS];
  void initializeTargetTable();

  SCSIDevice *currentDevice;
  SCSIError currentError;
  unsigned char messageToSend;
  int 	acceptMessage();
  void 	abortCommand();
  virtual unsigned 	waitForCompletion();
  void 	complete();

 public:

  // Run command and return error code
  unsigned	doCommand(unsigned char unit, SCSICommand &, char *buf,
			  unsigned buflen, int read);	
  unsigned	read(unsigned unit, unsigned startblock, unsigned blocks, char* buf,
	  unsigned buflen);
  unsigned	write(unsigned unit, unsigned startblock, unsigned blocks,
			char* buf, unsigned buflen);
  SCSIController();
  virtual ~SCSIController(){}

  /*
   * Run the SCSI command
   */
  virtual void 		initControllerAction(SCSICommand &, unsigned char unit,
				char *databuf, unsigned buflen ) = 0;

  virtual void 		initTarget(unsigned char unit) = 0;

 int	diskInfo(int, unsigned *blks, unsigned *blksize, char *vendor,
	unsigned vendorlen);
};
 

/* SCSI Messages */
const unsigned char MSG_CMD_COMPLETE          = 0x00;
const unsigned char MSG_EXTEND                = 0x01;
const unsigned char MSG_SV_PTR                = 0x02;
const unsigned char MSG_RS_PTR                = 0x03;
const unsigned char MSG_DISCON                = 0x04;
const unsigned char MSG_INIT_DETECT_ERR       = 0x05;
const unsigned char MSG_ABORT                 = 0x06;
const unsigned char MSG_REJECT                = 0x07;
const unsigned char MSG_NOP                   = 0x08;
const unsigned char MSG_PAR_ERR               = 0x09;
const unsigned char MSG_LCMD_COMPLETE         = 0x0a;
const unsigned char MSG_LCMD_W_FLAGS          = 0x0b;
const unsigned char MSG_BUS_DEV_RST           = 0x0c;
/*
 * Template for identify msg, fill in target
 */
const unsigned char MSG_IDENTIFY           	= 0x80;
const unsigned char MSG_IDENTIFY_ENABLE_RES	= 0xc0;

#endif SCSIDevice_h
