/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef SS1Am7990_h
#define SS1Am7990_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Am7990.h"

/*
 * Am7990 Lance ethernet driver machine dependent routines.
 *
 * On the SPARC the lance data structures have to be 8 byte aligned.
 * Also, the DMA vaddr area for the chip is 0xff****** on the SS1
 *
 * This subclass takes care of mapping packets to the DMA are on send
 * and recieve.  It communicates with the driver in DMA addresses and with
 * the higher levels in kernel virtual addresses for reception.
 */

	/*
	 * Allocate triple the amount for alignment waste
	 */
const sizeofBase = ( (12 + TransmitBuffers + ReceiveBuffers) 
	* sizeof(Am7990Word) * 4);


class SS1Am7990 : public Am7990 {
private:
	/* 
	 * The base of the memory that the chip uses.  Maps it to DMA area.
	 */
	static char _base [sizeofBase] ;
	char	*_dmaBase;	// The addr when _base is dma mapped
	/*
	 * How much past base is used
	 */
	unsigned offset;
protected:
	/* getBuffer allocates a packet for recv ring and maps it into
	 * DMA
	 */
	virtual NetworkBuffer *		getBuffer(unsigned int len) ;
	/*
	 * processPacket is called when a packet arrives
	 */
	virtual void 		receivePacket(NetworkBuffer *);

public:
	SS1Am7990( volatile Am7990Word *rap,  volatile Am7990Word *rdp,
		   unsigned char debugMask = 0 );
	~SS1Am7990();

	virtual void ready(NetworkAdministrator *);	// override parent

	/*
	 * interruptHandler() flushes the cache on the SS1
	 * since the cache ignore supervisor only tags, so the
	 * chip would never see the CPU disown the buffer if
	 * the CPU did so in another context, sigh.
	 * Then it calls the real interruptHandler.
	 * It flushes the DVMA side (context 0) 
	 * before the real handler, and flushes the current
	 * context just after it.
	 */
	void interruptHandler();

	virtual int transmitPacket(NetworkBuffer *b);
};

#endif /* SS1Am7990_h */
