/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef SS1Configuration_h
#define SS1Configuration_h


#ifdef __GNUC__
#pragma interface
#endif

/*
 *		SPARCstation Choices VM layout
 *
 *		 ---------------	0x0
 *		|    invalid	|
 *		 ---------------	_kernelStart = 0x04000
 *		|    Kernel	|
 *	 	 --------------- 	&end
 *		|   Boot Heap	|
 *	 	 --------------- 
 *		|		|
 *	 	 ---------------
 *		|  Kernel Heap	|
 *	 	 --------------- 	KERNELSIZE (SHARED_START) = 0x10000000
 *		|    VA HOLE	|	(no mans land)
 *	 	 ---------------	0xe0000000	APPLICATION_START
 *		|		|
 *		|		|
 *	 	 ---------------	0xff000000	APPLICATION_END
 *		|     dvma	|
 *	 	 ---------------	0xffd00000
 *		|    eeprom	|
 *	 	 ---------------	0xfff00000
 *		|  spare segptr	|
 *	 	 ---------------	IO_VBASE = 0xffff0000
 *		|   hardware	|
 *	 	 ---------------
 */
#define SUN_ROM

/*
 * The size of the 1-1 mapped image that the boot code sets up
 * and that the VM initialization expects to be in place.
 */
#define 		KERNEL_VADDR_START	0x00000000
#define 		KSIZE			0x00400000 /* 4M */

#define			DVMA_BASE		0xff000000

/* 
 * Where we map the OBIO HW things to 
 */
#define IO_VBASE	0xFFFF0000 
#define IO_VSIZE	0x00010000

#ifndef ASM

char *const 		PHYSICALMEMORYSTART 	= 0;

/*
 * The SPARCstation has a hole in the virtual address space.
 */
const char *const 	VA_HOLE_START		= (char *)0x20000000;
const char *const	 VA_HOLE_END		= (char *)0xe0000000;

extern unsigned end;
#define HEAP_START() ((char*)&end)
const char *const	 USERMEMORYSTART 	= (char *) 0xe0000000;
const char *const	 USERMEMORYEND 		= (char *) DVMA_BASE;

/*
 * Range of virtual address that kernel lives in.
 */
char *const		KERNEL_START_VIRTUAL 	= (char *)0x4000;
char *const		KERNEL_END_VIRTUAL 	= (char *)0x10000000;
// const unsigned	KERNEL_HEAPSIZE		= 0x00800000; // 8M
// will be computed from available memory
const unsigned		KERNEL_SIZE		= KERNEL_END_VIRTUAL - KERNEL_START_VIRTUAL;

/* Where the Lance ethernet and SCSI does DMA */
char *const	 	DMA_BASE 		= (char *)DVMA_BASE;
char *const 	 	DMA_END			= (char *)0xffd00000;

char *const	 	EEPROM_VADDR 		= (char *)0xffd00000;
const unsigned		EEPROM_VSIZE 		= 0x00200000;


char *const 		SPARE_SEGPTR		= (char *)0xfff00000;

/*
 * Some miscellanious thangs...
 */
const unsigned	SCSIBUS_ID		= 7;

#endif /* ASM */
#endif /* SS1Configuration_h */
