/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SS1Console.h - Sparcstation-1 console class.
 */

#ifndef SS1Console_h
#define SS1Console_h


#ifdef __GNUC__
#pragma interface
#endif


#include "Lock.h"	/* could be a problem since locks use assertions */
#include "OutputStream.h"
class zs;

extern int consoleInitialized;
extern void rawConsolePrintf(char *);	// Non-locking

class SS1Console : public OutputStream {
	/*
	 * The serial chip we talk to.
	 */
	zs * _myzs;
protected:
	Lock lowPriorityLock;
	Lock highPriorityLock;

	/*
	 * The parent class expects these!
	 */
	void acquireExclusiveUse();
	void releaseExclusiveUse();
	void acquireHighPriorityUse();
	void releaseHighPriorityUse();

	void emitIdentifierOfCurrentProcessor(){};
	void emitNewline();

public:

        virtual void panic( const char * f,
                const formatArgs & a1 = *(formatArgs *)0,
                const formatArgs & a2 = *(formatArgs *)0,
                const formatArgs & a3 = *(formatArgs *)0,
                const formatArgs & a4 = *(formatArgs *)0 );

	SS1Console(volatile unsigned char * const); 
	void putChar( char c );
	int getChar();
	int nonBlockingGetChar();
	int write( const char * buf, int count );
	void setInterruptMode();
};

extern OutputStream * ConsoleOutputStream;

#endif	/* SS1Console_h */
