/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SS1Kernel.h: Machine description class for the SS1 Kernel.
 */

#ifndef SS1Kernel_h
#define SS1Kernel_h


#ifdef __GNUC__
#pragma interface
#endif


#include "Kernel.h"
#include "SS1Configuration.h"

class ESP;
class SS1Disk;
class MemoryObjectContainer;
class PhysicalMemoryRange;
class PersistentMemoryObject;

extern ClassStar SS1KernelClass;
class SS1Kernel : public Kernel {
protected:

	ESP *			_SCSIController;
        SS1Disk * theDisk;
	MemoryObjectContainer * theRootContainer;
	unsigned _availableMemory;
	unsigned _physicalMemorySize;

	/*
	 * Overloads of parent class methods.
	 */
	virtual void buildRunningCPU();
	virtual void basicInitializeDevicesAndManagers();
	virtual void basicInitializeNetworkProcesses();
	virtual void shutdown();


	DiskRef rootDisk();
	int isObjectFile( PersistentMemoryObject * objectFile );
	const char * loadObjectFile( PersistentMemoryObject * objectFile,
						Domain * newProcessesDomain );
	void mountNonRootFileSystems();
	void initializeBackingStore();
	int tryCommand(int argc, char *argv[], int partnumber);

public:

	virtual PhysicalMemoryRange *   physicalMemoryMap() ;
	virtual const unsigned		physicalMemorySize();
	virtual const char *            kernelStart()
				{return KERNEL_START_VIRTUAL;}
	virtual const char *            applicationStart()
				{return USERMEMORYSTART;}
	virtual const unsigned          kernelHeapSize();
	virtual const unsigned          kernelSize() 
				{return KERNEL_SIZE;}
	virtual const unsigned          applicationSize() 
				{return ((unsigned)USERMEMORYEND - (unsigned)USERMEMORYSTART);}
	virtual const unsigned          addressTranslationMemorySize()
				{return 0;} // Memory comes from kernel Heap.

	SS1Kernel();
	Class * classOf();
};

#endif /* SS1Kernel_h */
