/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SS1Timer.h - defines for the SPARCstation1 timer
 */

#ifndef SS1Timer_h
#define SS1Timer_h


#ifdef __GNUC__
#pragma interface
#endif


#include "SS1devices.h"

/*
 * Timer register things common to both timers
 */
const unsigned 	COUNTER_SHIFT	= 10;
const unsigned	LIMIT_MASK	= 0x80000000;
const unsigned 	COUNTER_MASK	= ((~0 << COUNTER_SHIFT) & (~LIMIT_MASK));
const unsigned	COUNTER_HZ 	= 1000000;

/*
 * These are for the FreeRunningTimer, its limit is set to zero
 * so it free_runs and interrupts about every 2 seconds.
 */
const unsigned FRTuSecPerInterrupt = 0x1 << 21;

/*
 * These are for the timer manager
 */
const unsigned int ClockFrequency = 10; 	/* ticks per second */
const unsigned int TimerManageruSecPerInterrupt = COUNTER_HZ / ClockFrequency;


#endif /* SS1Timer_h */
