/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SS1TimesliceTimer.h - Timeslice timer for the SS1
 */

#ifndef SS1TimesliceTimer_h
#define SS1TimesliceTimer_h


#ifdef __GNUC__
#pragma interface
#endif


#include "Lock.h"
#include "Timer.h"	/* parent class */
#include "TimeSliceInterrupt.h"

extern TimeSliceInterrupt TSliceInterrupt;

class SS1TimesliceTimer : public Timer {
	Lock *lock;
	unsigned int tickTime;

	unsigned int expireTime;
	char running;

public:
	SS1TimesliceTimer();
	~SS1TimesliceTimer();

	unsigned int start( unsigned int microseconds );
	void set( unsigned int microseconds );
	unsigned int stop();
	unsigned int residual();
	int idle();
	/*
	 * tick should be called directly from the hardware timer interrupt.
	 *   every 1000000/ClockFrequency microseconds.
	 */
	void tick();
};

inline
SS1TimesliceTimer::SS1TimesliceTimer( )
{
	lock = new Lock("SS1TimesliceTimerLock");
	tickTime = 0;
	running = 0;
}

inline 
SS1TimesliceTimer::~SS1TimesliceTimer()
{
	delete lock;
}


inline void
SS1TimesliceTimer::tick()
{
    lock->acquire();
    if ((++tickTime == expireTime) && running) {
	running = 0;
	lock->release();
	TSliceInterrupt.raise();
    }
    else
	lock->release();
}

#endif /* SS1TimesliceTimer_h */
