/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef SS1devices_h
#define SS1devices_h


#ifdef __GNUC__
#pragma interface
#endif


/*
 *
 * contains IO virtual addresses of SS1 devices 
 *
 */

#include <sys/intreg.h>
#include "SS1Configuration.h"

/*
 * pre - defined:
 * AUXIO_ADDR		(IO_VBASE + 0x00006000)
 * COUNTER_ADDR		(IO_VBASE + 0x00007000)
 * EEPROM_ADDR			0xFFFF8000
 * CLOCK_ADDR			0xFFFF87F8
 * MEMERR_ADDR			0xFFFF9000
 * INTREG_ADDR			0xFFFFA000
 * IDPROM_ADDR			EEPROM_ADDR+EEPROM_SIZE
 */

#define SERIAL_ADDR		(IO_VBASE + 0x00001000)
#define LE0_ADDR		(IO_VBASE + 0x00003000)
#define TIMER_ADDR 		(IO_VBASE + 0x00007000)
#define SCSI_ADDR		(IO_VBASE + 0x0000B000)
#define DMA_ADDR		(IO_VBASE + 0x0000C000)

/*
 * This is for use in paging code, where the disk is called upon to read
 * into a pau, but the SPARC needs a vaddr to do dvma...  Since there is
 * extra room in the hardware PMEG we can use it (for now, hack hack)
 */

/* The following are only for C code.  
 * Define ASM for assembly code omission.
 */
#ifndef ASM

/* Vaddress constants of the register bases */
/*volatile unsigned char *const INTREGVAL = (unsigned char *)INTREG_ADDR;*/
typedef unsigned char u_char;

volatile unsigned short *const LE0_RDP =	(unsigned short *)(LE0_ADDR);
volatile unsigned short *const LE0_RAP =	(unsigned short *)(LE0_ADDR + 2);
volatile unsigned *const COUNTER0 =  (unsigned *)TIMER_ADDR;
volatile unsigned *const LIMIT0 =    (unsigned *)(TIMER_ADDR + 4);
volatile unsigned *const COUNTER1 =  (unsigned *)(TIMER_ADDR + 8);
volatile unsigned *const LIMIT1 =    (unsigned *)(TIMER_ADDR + 0x0C);
volatile unsigned *const DMA =       (unsigned *)DMA_ADDR;
volatile unsigned *const SCSI =      (unsigned *)SCSI_ADDR;
volatile unsigned char *const SERIAL_B_CTL =	(unsigned char*)(SERIAL_ADDR);
volatile unsigned char *const SERIAL_B_DATA = (unsigned char*)(SERIAL_ADDR + 2);
volatile unsigned char *const SERIAL_A_CTL =	(unsigned char*)(SERIAL_ADDR + 4);
volatile unsigned char *const SERIAL_A_DATA =	(unsigned char*)(SERIAL_ADDR + 6);

#endif /* ASM */


#endif /* SS1devices_h */
