/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *a e Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * VCDisk.h - Memory Object representing a disk connected to
 *		       an EMC on the Multimax.
 */

#ifndef VCDisk_h
#define	VCDisk_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Disk.h"
#include "Raid.h"
#include "Semaphore.h"

#if defined(linux) || defined(__APPLE__)
#include<sys/types.h>
/* linux_port: borrowed from solaris sys/asynch.h */
typedef struct aio_result_t {
    ssize_t aio_return;             /* return value of read or write */
    int aio_errno;          /* errno generated by the IO */
} aio_result_t;
#else
extern "C" {
#include <sys/asynch.h>
};
#endif

/*
 * A simple disk class, one partition per disk.
 * open more disks if you wnat more partitions.
 */
extern ClassStar VCDiskClass;
class VCDisk : public Disk {
	friend class VCKernel;
	friend class VCDiskInterrupt;

	BinarySemaphore	_wait;
	aio_result_t _resultP;
	int _unixFile;
	int _expectedBytes;
protected:
	~VCDisk();
public:
	Class * classOf();
	Class *supports(Class *);
	// // // // // // // // // Class * supports( Class * );

	VCDisk( char * unixFileName);

	int doio(unsigned start, int count, char *buf, int bufsiz,
		int direction);

	void handler();	// Interrupt handler
	int partitions(){return 1;}
	int locatePartition( int idNumber, int &start, int &units )
		{start = 0; units = _numberOfUnits; return 0;}
	int basicDoio( unsigned int, char *, int ){return DiskOperFailed;}
	int reset(){return 0;}
};
#endif /*VCDisk_h*/
