/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 WSpringfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
#ifndef VCInterrupts_h
#define VCInterrupts_h

#ifdef __GNUC__
#pragma interface
#endif

#include "Interrupt.h"		/* parent class */

class VCFPInterrupt : public Interrupt
{
public:
        void basicRaise(char *);
};

class VCMemoryInterrupt : public Interrupt {
public:
        void basicRaise(char *);
};

class VCIllegalInstructionInterrupt : public Interrupt {
public:
        void basicRaise(char *);
};

#ifdef NEVER
class VCProxyInterrupt : public Interrupt
{
protected:
        void basicRaise(char *);
};
#endif

class VCDisk;
class VCDiskInterrupt : public Interrupt
{
	VCDisk *_disk;
friend class VCDisk;
        VCDiskInterrupt(VCDisk *which){ _disk = which;}
protected:
        void basicRaise(char *);
};

class VCIOInterrupt : public Interrupt
{
private:
	static const int		nIOInterrupts = 8;
	static int		ioInterrupts;
	static int		fdTable[ nIOInterrupts ];
	static Interrupt	*interruptTable[ nIOInterrupts ];

protected:
 	void basicRaise(char *);

public:
	VCIOInterrupt(void) { ioInterrupts = 0; }
	static Interrupt *setIOInterrupt(int fd, Interrupt * interrupt, 
					 PFV f = 0);
};


#endif /* VCInterrupts_h */
