/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1990,1991 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Theron Tock (theron@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *	VCEthernet.h
 *
 *	Interface for the Virtual Choices Ethernet driver.
 *
 * $Id: 
 *
 */

#ifndef VCEthernet_h
#define VCEthernet_h


#ifdef __GNUC__
#pragma interface
#endif

#include	"EthernetDriver.h"
#include	"Ethernet.h"

/*
 * The port number for the socket based virtual network
 */
const int  networkPort = 2863;     /* My favorite number */

class Semaphore;
class NetworkBufferFreeList;

extern ClassStar VCEthernetClass;
class VCEthernet : public EthernetDriver {
private:
	int			_fd;
	int			_serverSocket;
#ifdef DOESNT_WORK
	int			_multicastCount[MulticastAddresses];
	EthernetAddress		_multicastAddress[MulticastAddresses];
	Semaphore		*_multicastMutex;
#endif

	void		connectToLocalServer();
	int		doTransmit(NetworkBuffer *);

protected:
	unsigned int		_inputPackets;
	unsigned int		_outputPackets;
	unsigned int		_interrupts;


public:
	Class * classOf();

	VCEthernet(unsigned char debugMask = 0,NetworkBufferFreeList* nf = 0);
	virtual ~VCEthernet();

	void	getDescriptors(int *file, int *server);

	virtual void interruptHandler();	// The interrupt handler
	virtual void printInfo();		// Print stats to console

	virtual int 	transmitPacket(NetworkBuffer *);// To send a packet
	virtual int 	transmitPacketAsync(NetworkBuffer *);
	virtual int	addMulticast(EthernetAddress *);
	virtual int	removeMulticast(EthernetAddress *);
};


inline
void
VCEthernet::getDescriptors(int *file, int *server)
{
  *file = _fd;
  *server = _serverSocket;
}


#endif /* ! VCEthernet_h */
