/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef VCKernel_h
#define VCKernel_h


#ifdef __GNUC__
#pragma interface
#endif


#include "Kernel.h"
#include "VCDisk.h"

extern ClassStar VCKernelClass; class VCKernel : public Kernel {
protected:
    /*
     * These are virtual functions that need to be defined
     */
    DiskRef rootDisk();
    PhysicalMemoryRange * configure();
    void buildRunningCPU();
    void configureHardware();
    void basicInitializeDevicesAndManagers();
    void initializeBackingStore();
    void initialize(Allocator *bootHeap);
    void basicInitializeNetworkProcesses();
    void startOtherProcessors(){}
    void mountNonRootFileSystems();

	/*
	 * Used for mmap to get the kernel and application
	 * VM ranges
	 */
    VCDisk * theDisk;

	void cponCommand(int argc, char *argv[]);
public:
    VCKernel();
    ~VCKernel(){}

    const unsigned kernelHeapSize();
    const char * kernelStart();
    const unsigned kernelSize();
    const unsigned applicationSize();
    const unsigned addressTranslationMemorySize(){return 32;}
    const char * applicationStart();
    PhysicalMemoryRange *         physicalMemoryMap();
	 const unsigned int physicalMemorySize();

    int isObjectFile( PersistentMemoryObject * objectFile );
    const char * loadObjectFile( PersistentMemoryObject * objectFile,
				Domain * newProcessesDomain );
    int tryCommand(int argc, char *argv[], int );

    Class * classOf();
};

#endif
