/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * MadAllocator.h: definition of a Big Bag-o-Pages heap class.
 */

#ifndef MadAllocator_h
#define MadAllocator_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Allocator.h"
#include "Lock.h"

class MemoryObjectCache;

class Domain;

const unsigned NumberOBuckets = 32;

extern ClassStar MadAllocatorClass;
class MadAllocator : public Allocator {
protected:
	unsigned	_log2PageSize;
	int		_canReclaim;
	char *		_baseAddress;
	char *		_upperBoundAddress;
	char *		_pageToBucket;
	Lock *		_bucketMutex[NumberOBuckets];
	unsigned	_bucketFreeCount[NumberOBuckets];
	unsigned	_bucketMaxFreeCount[NumberOBuckets];
	char *		_bucketToPage[NumberOBuckets];
	char *		_bucketEnd[NumberOBuckets];
	char		_sizeToBucket[4100];
	char		_sizeToBuckets[1024];
	Lock *		_topMutex;
	int		_top;
	int		_pageSize;
	int		_pageMask;
	Domain *	_domain;
	unsigned	_numberOfPages;
	void *		_startOkRange;
	void *		_endOkRange;

	MemoryObjectCache * _cache;
	unsigned int	findBucket( unsigned int size );
	char *		moreSpace( unsigned int bucket );
public:
	static int	stateSize( int numberOfPages );

	MadAllocator( void * stateAddress, void * baseAddress,
		      int numberOfPages, int log2PageSize, Domain *,
		      void * startOkRange = 0, void * endOkRange = 0,
		      int noMultiProcessing = 0 );
	~MadAllocator();
	Class *		classOf();
	void		enableMultiProcessing();
	void *		allocate( unsigned int size );
	void		free( void * cp );
	int		pagesUsed();
	void *		base( void * cp );
	int		sizeInPages(){return _numberOfPages;};
	int		reclaimSpace();
};

inline void *
MadAllocator::base( void * cp )
{
	void * returnValue = 0;
	unsigned page = ( (char *)cp - _baseAddress ) >> _log2PageSize;
	if( ( (char *)cp >= _baseAddress ) && ( page < _numberOfPages ) ) {
		unsigned bucket = _pageToBucket[page];
		if( ( bucket > 1 ) && ( bucket <= _log2PageSize ) ) {
			returnValue = (void *)( (int)cp & ~((1<<bucket)-1) );
		}
	}

	return( returnValue );
}

#endif /*MadAllocator_h*/
