/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Contributing Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Contributing Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * AddressTranslator.h: Hardware AddressTranslator abstract class.
 */

#ifndef AddressTranslator_h
#define AddressTranslator_h


#ifdef __GNUC__
#pragma interface
#endif

#include "AccessType.h"
#include "AddressTranslation.h"
#include "Object.h" /* parent class */

class Domain;
class ProcessorContext;

typedef void (*EnableFunction)( void * );

extern ClassStar AddressTranslatorClass;
class AddressTranslator : public Object {
 protected:
    Domain * _currentDomain;
 public:
    AddressTranslator();

    void activate( Domain *);
    virtual void enable() = 0;
    virtual int enabled() = 0;

    virtual void basicActivate( AddressTranslation *) = 0;
    virtual void flushCache( const char * address,
                             unsigned int length ) = 0;

    Domain * currentlyActiveDomain();

    Class * classOf();
};

inline
AddressTranslator::AddressTranslator()
{
    _currentDomain = 0;
}

inline Domain *
AddressTranslator::currentlyActiveDomain()
{
    return( _currentDomain );
}

#endif /* AddressTranslator_h */
