/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * CacheAsync.h 
 */

#ifndef CacheAsync_h
#define CacheAsync_h
#ifdef __GNUC__
#pragma interface
#endif

#include "Barrier.h"
#include "MemoryObjectAsync.h"

class CacheMOIOClosure : public IOAsyncClosure {
	unsigned _expected;
	unsigned _log2MoUnitSizeRatio;
	IOAsyncClosure * _parent;
public:
	static void func( CacheMOIOClosure * c );
	CacheMOIOClosure
		( IOAsyncClosure * parent, unsigned log2MoUnitSizeRatio,
		  unsigned expected )
		: _expected( expected ), 
		  _log2MoUnitSizeRatio( log2MoUnitSizeRatio ),
		  _parent( parent )
		  {_func = (AsyncFunc) &func ;}
};

class CacheIOState;

class CacheIOClosure : public IOAsyncClosure {
protected:
	static void func( CacheIOClosure * c );
public:
	CacheIOState * _state;
	CacheIOClosure() { _func= (AsyncFunc)&func;}
};

class CacheIOState {
public:
	static const int _prealloc = 8;
	
	struct _Operation {
		unsigned _startUnit;
		unsigned _count;
		char * _buf;
		unsigned _spare;
		CacheIOClosure _closure;
	};

	static Lock _lock;
	int _expectedResult;
	int _result;
	unsigned _spare;
	IOAsyncClosure * _parent;
	unsigned _count;
	unsigned _completed;
	_Operation * _ops;
	unsigned _max;
	Barrier _barrier;
	_Operation _preallocatedOps[_prealloc];

	CacheIOState( IOAsyncClosure * parent ) 
		: _parent( parent ), _count( 0 ), _completed( 0 ),
		  _ops( _preallocatedOps ), _max( _prealloc ),
	 	  _barrier( 1 ) {}
	~CacheIOState() { if ( _ops != _preallocatedOps ) delete _ops; }
	void add( unsigned startUnit, unsigned count, char * buf );
	void add( unsigned startUnit, unsigned count, char * buf, 
		  unsigned spare );
};

inline
void 
CacheIOState::add( unsigned startUnit, unsigned count, char * buf )
{
	if ( _count == _max ) {
		int newMax = _max << 1;
		_Operation * newOps = new _Operation[newMax];
		for( unsigned i = 0; i < _max; i++ ) {
			newOps[i] = _ops[i];
		}
		if ( _ops != _preallocatedOps ) 
			delete _ops;
		_ops = newOps;
		_max = newMax;
	}
	Assert( _count < _max );
	_Operation * op = _ops + _count;
	_count++;
	op->_startUnit = startUnit;
	op->_count = count;
	op->_buf = buf;
	op->_closure._state = this;
}

inline
void 
CacheIOState::add( unsigned startUnit, unsigned count, char * buf, 
		   unsigned spare )
{
	if ( _count == _max ) {
		int newMax = _max << 1;
		_Operation * newOps = new _Operation[newMax];
		for( unsigned i = 0; i < _max; i++ ) {
			newOps[i] = _ops[i];
		}
		if ( _ops != _preallocatedOps ) 
			delete _ops;
		_ops = newOps;
		_max = newMax;
	}
	Assert( _count < _max );
	_Operation * op = _ops + _count;
	_count++;
	op->_startUnit = startUnit;
	op->_count = count;
	op->_buf = buf;
	op->_closure._state = this;
	op->_spare = spare;
}

#endif /*CacheAsync_h*/
