/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * CacheUnitInfo.h

 */

#ifndef CacheUnitInfo_h
#define CacheUnitInfo_h

#include "MultiLevelTable.h"

#ifdef __GNUC__
#pragma interface
#endif

// #define DEBUG_CACHE_ALLOCATE

enum CUIFlag { 
	CUIValidBit = 0x01,
	CUIModifiedBit = 0x02,

	CUIInvalid = 0x00,
	CUIValid = 0x01,
	CUIModified = 0x03
};

class CacheUnitInfo {
	char _info;
public:
	CacheUnitInfo();
	CacheUnitInfo( int flag );	
	CacheUnitInfo( CUIFlag flag );
	void is( CUIFlag flag );
	int valid();
	int modified();
	int info();
};

OutputStream & operator<< ( OutputStream & os, CacheUnitInfo info );

inline CacheUnitInfo::CacheUnitInfo() {}
inline CacheUnitInfo::CacheUnitInfo( int flag )
        : _info( flag ) {}
inline CacheUnitInfo::CacheUnitInfo( CUIFlag flag )
        : _info( flag ) {}
inline void CacheUnitInfo::is( CUIFlag flag ) { _info = flag; }
inline int CacheUnitInfo::valid() { return _info & CUIValidBit; }
inline int CacheUnitInfo::modified() { return _info & CUIModifiedBit; }
inline int CacheUnitInfo::info() { return _info; }

class CacheUnitInfoTable {
protected:
	int _size;
	int _allocatedSize;
#ifdef DEBUG_CACHE_ALLOCATE
	int _validSize;
#endif

	virtual void basicSetUnitInfo( int unit, CacheUnitInfo info );
	virtual void basicSetSize( int size );
public:
	CacheUnitInfoTable( int size );
	virtual ~CacheUnitInfoTable();

	int size()		{ return _size; }
	int allocatedSize()	{ return _allocatedSize; }
#ifdef DEBUG_CACHE_ALLOCATE
	int validSize()		{ return _validSize; }
#endif

	void setUnitInfo( int unit, CacheUnitInfo info );
	virtual CacheUnitInfo unitInfo( int unit );
	virtual void setSize( int size );
};

inline
void CacheUnitInfoTable::setUnitInfo( int unit, CacheUnitInfo info )
{
#ifdef DEBUG_CACHE_ALLOCATE
	int oldValid = unitInfo( unit ).valid();
	int newValid = info.valid();
	if ( !oldValid && newValid )
		_validSize++;
	else if ( oldValid && !newValid )
		_validSize--;
	basicSetUnitInfo( unit, info );
#else
	basicSetUnitInfo( unit, info );
#endif
}

class ArrayCacheUnitInfoTable : public CacheUnitInfoTable {
protected:
	int _log2BitsPerInt;
	int _log2EntriesPerInt;
	unsigned * _array;

	virtual void basicSetUnitInfo( int unit, CacheUnitInfo info );
	virtual void basicSetSize( int size );
public:
	ArrayCacheUnitInfoTable( int size );
	virtual ~ArrayCacheUnitInfoTable();
	virtual CacheUnitInfo unitInfo( int unit );
};

class BitsMultiLevelTable;

class MultiLevelCacheUnitInfoTable : public CacheUnitInfoTable {
protected:
	BitsMultiLevelTable _table;

	virtual void basicSetUnitInfo( int unit, CacheUnitInfo info );
	virtual void basicSetSize( int size );
public:
	MultiLevelCacheUnitInfoTable( int size );
	virtual ~MultiLevelCacheUnitInfoTable();
	virtual CacheUnitInfo unitInfo( int unit );
};

#endif /*CacheUnitInfo_h*/
