/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef COWCache_h
#define COWCache_h


#ifdef __GNUC__
#pragma interface
#endif

#include "SimpleVMCache.h"
#include "MemoryObjectStar.h"

class MemoryObject;

extern ClassStar COWCacheClass;
class COWCache: public SimpleVMCache {
  protected:
    
    MemoryObjectStar _sourceMO;
    MemoryObjectCache * _source;
    MemoryObjectCache * _shadow;
    
  public:
  
    Class * classOf();
    
    void setShadow(MemoryObjectCache * cache);
    
    
    COWCache(MemoryObject * memobj, unsigned int length,
	     MemoryObject * source = 0);
    ~COWCache();
    
    int isACopy();
    
    PhysicallyAddressableUnit * pageFromShadow(unsigned unit);
    PhysicallyAddressableUnit * pageToShadow(unsigned unit);
    
    PhysicallyAddressableUnit * pageFromSource(unsigned unit);
    
    PhysicallyAddressableUnit * dupPageSelf(unsigned unit);

    MemoryObjectCache * createShadow();

    int pagesToStore(unsigned from, unsigned to = 0);
    void flush();
    void saveModified(unsigned from, unsigned to = 0);
};

inline void
COWCache::setShadow(MemoryObjectCache * cache)
{
    _shadow = cache;
}

inline int
COWCache::isACopy()
{
    return _source != 0;

}


#endif /* COWCache_h */
