
/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted und
er license agreement.
 *
 * Author: Nayeem Islam (nislam@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 * No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef DVMCommunicator_h
#define DVMCommunicator_h


#ifdef __GNUC__
#pragma interface
#endif

#include "NetworkWalker.h"
#include "DVMMessage.h"
#include "Link.h"

/* 
 * Pure Data Transfer twiddling
 */

static const int NumMachines = 10;

class DVMWalker : public NetworkWalker {
/*
 * Walker is the network walker used by etheradministrator for DVM
 * packets. The function walk should not block. In our case, walk
 * simply calls the DVMCommunicator.
 */
public:
    DVMWalker();
    
    virtual void walk(NetworkBuffer * buff);
};

class List;

class DVMCommunicator {
/*
 * DVMCommunicator is the lowest level network interface for DVM.
 * It supports machine naming -- a machine has an index, a name, and
 * an ethernet address. DVM communication takes place in terms of
 * machine indices. The locations of a page can thus be expressed as a
 * bit array per page.
 * Currently, the names and addresses are in a hardcoded static array.
 */
  protected:
    DVMWalker TheDVMWalker;
    unsigned _localIndex;		// index for this machine
    EthernetAddress * _localAddress;	// address for this machine
    char * _localName;			// name of this machine

    class _dvmIdEntry : public Link {
      public:
	unsigned index;
	EthernetAddress *address;
	char *name;

	_dvmIdEntry(unsigned i, EthernetAddress * a, char * n)
	{ index = i; address = a; name = n; };
    };
    
    List * _etherHost;
    unsigned maxHosts;
    
  public:
    DVMCommunicator();
    
// functions intended for testing
    
    void networkSendTest(char * packet, int len, char * destname);
    void networkReceiveTest(char *packet, int len);
    void makeEtherHeader(char * destname, EthernetHeader *);
    
// debugging
    
    void dumpPacket(char *packet, int len);

// Actual DVM communication functions
    
    /*
     * method networkSend: send packet to machine id index.
     * Used by DVMMessages for transmission.
     */
    void networkSend(unsigned * packet, unsigned length, int index,
		     DVMMsgType type=0);
    /*
     * method networkReceive: handle packet of given length
     * Used by the ethernet administrator
     */
    void networkReceive(char * packet, int length);

// Nameserver functions. Hook up eventually to the real nameserver.

    EthernetAddress * machineAddress(unsigned index);
    char * machineName(unsigned index);
    unsigned machineIndex(char * name);
    unsigned machineIndex(EthernetAddress * address);

    unsigned localIndex() { return _localIndex; };
    char * localName() { return _localName; };
    EthernetAddress * localAddress() { return _localAddress; };

    unsigned headerSize() { return sizeof(EthernetHeader); };
};

extern DVMCommunicator* TheDVMCommunicator;

#endif DVMCommunicator_h
