/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef SimpleVM_h
#define SimpleVM_h


#ifdef __GNUC__
#pragma interface
#endif

#include "State.h"
#include "FaultErrorCode.h"

class MemoryObjectCache;
class AddressTranslation;


// No Page, but a Disk Copy exists.

class SimpleVMState: public State {
  public:
    SimpleVMState(StateMachine * m) : State(m) {};
    SimpleVMState(StateMachine * m, State * d) : State(m,d) {};
    
    virtual State * localRead(MemoryObjectCache * cache, unsigned unit,
			      AddressTranslation * adt) = 0;
    virtual State * localWrite(MemoryObjectCache * cache, unsigned unit,
			       AddressTranslation * adt) = 0;
    virtual State * pageOut(MemoryObjectCache * cache, unsigned unit) = 0;
    virtual FaultErrorCode code() = 0;
};

class VM_NPDC: public SimpleVMState {
  public:
    VM_NPDC(StateMachine * machine) : SimpleVMState(machine) {};
    
    virtual State * localRead(MemoryObjectCache * cache, unsigned unit,
			      AddressTranslation * adt);
    virtual State * localWrite(MemoryObjectCache * cache, unsigned unit,
			       AddressTranslation * adt);
    virtual State * pageOut(MemoryObjectCache * cache, unsigned unit);
    FaultErrorCode code();
};

class VM_P: public SimpleVMState {
  public:
    VM_P(StateMachine * machine) : SimpleVMState(machine) {};
    
    virtual State * localRead(MemoryObjectCache * cache, unsigned unit,
			      AddressTranslation * adt);
    virtual State * localWrite(MemoryObjectCache * cache, unsigned unit,
			       AddressTranslation * adt);
    virtual State * pageOut(MemoryObjectCache * cache, unsigned unit);
    FaultErrorCode code();
};

class SimpleVMMachine: public StateMachine {
  public:
    SimpleVMMachine();
    SimpleVMMachine(int numStates) : StateMachine(numStates) {};
};

extern SimpleVMMachine * TheSimpleVMMachine;
extern SimpleVMState * STATE_VM_NPDC;
extern SimpleVMState * STATE_VM_P;

#endif /* SimpleVM_h */
