/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SimpleVMCache.h: A MemoryObjectCache for paged harware.
 */

#ifndef SimpleVMCache_h
#define SimpleVMCache_h


#ifdef __GNUC__
#pragma interface
#endif

#include "PagedMemoryObjectCache.h"	/* parent class */

class State;
class SimpleVMState;

/*
 * Define the size of units or "pages" in a SimpleVMCache.
 * The size of the pages has to be a power of 2 multiple of the hardware
 * page size.
 * Should be consistent with RMOC..Size.
 */
static const unsigned int SimpleVMCacheLog2UnitSize = 12;

extern ClassStar SimpleVMCacheClass;
class SimpleVMCache : public PagedMemoryObjectCache {
/*
 * A SimpleVMCache is the normal cache for use by user
 * programs. It is implemented using a state machine, whereas
 * PagedMemoryObjectCache contains normal inlined code.
 */
    
  protected:
    
    State ** _pageState;

    SimpleVMState * pageState(unsigned unit);
    
  public:
  
    Class * classOf();
    
    SimpleVMCache( MemoryObject * memobj, unsigned int length );
    SimpleVMCache( MemoryObject * memobj, unsigned int length, int derived );
    ~SimpleVMCache();
    
    /*
     * Versions of cache functions coded using page state machines
     */
    
    virtual PhysicalMemoryChain
	cache(unsigned int unit,
	      ProtectionLevel requiredProtection, FaultErrorCode & code,
	      AddressTranslation * adt, const char * vaddr, int addToAdt=0 );
    
    virtual PhysicalMemoryChain
	mcache(unsigned int unit, int nunits,
	       ProtectionLevel requiredProtection, FaultErrorCode& code,
	       AddressTranslation * adt, const char * vaddr, int addToAdt=0 );
    
    virtual PhysicallyAddressableUnit * pageOut();
    
};

inline SimpleVMState *
SimpleVMCache::pageState(unsigned unit)
{
    return (SimpleVMState *)_pageState[unit];
}

#endif /* SimpleVMCache_h */
