/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * DummyMemoryObject.h:  A brain dead memory object
 */

#ifndef DummyMemoryObject_h
#define DummyMemoryObject_h


#ifdef __GNUC__
#pragma interface
#endif

#include "MemoryObject.h"
#include "PremappedMemoryObjectCache.h"
#include "ProtectionLevel.h"
#include "RangeLocks.h"
#include "MemoryObjectView.h"

extern ClassStar DummyMemoryObjectClass;
class DummyMemoryObject : public MemoryObject {
	SpecialRangeLock _specialLock;
	NormalMemoryObjectView _view;

      protected:
	int rawRead( unsigned startUnit, unsigned count, char * addr, unsigned nbytes,
		     IOAsyncClosure * c );
	int rawWrite( unsigned startUnit, unsigned count, char * addr, unsigned nbytes,
		      int writeThrough, IOAsyncClosure * c );

      public:
	Class * classOf();

	DummyMemoryObject( unsigned int numberOfUnits, int log2UnitSize );
	~DummyMemoryObject();

	MemoryObjectCache * buildCache();
	MemoryObjectView * view() {return & _view;}
};

extern ClassStar PhysicallyResidentMemoryObjectClass;

class PhysicallyResidentMemoryObject : public DummyMemoryObject {
        PremappedMemoryObjectCache _premappedCache;
	
      protected:
	const char * residentAddress;

      public:
	Class * classOf();

	PhysicallyResidentMemoryObject( unsigned int numberOfUnits,
		unsigned log2UnitSize, const char * address, int preAllocated );
	PhysicallyResidentMemoryObject( unsigned log2UnitSize, const char * base,
	        const char * top, int preAllocated );
	~PhysicallyResidentMemoryObject();
};

extern ClassStar RootMemoryObjectClass;

class RootMemoryObject : public DummyMemoryObject {
      protected:
      public:
	Class * classOf();

	RootMemoryObject( unsigned int numberOfUnits, unsigned log2UnitSize );
	~RootMemoryObject();
};

#endif /* DummyMemoryObject_h */
