/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * IOWindow.h -
 */

#ifndef IOWindow_h
#define IOWindow_h

#ifdef __GNUC__
#pragma interface
#endif

#include "Assert.h"
#include "Lock.h"
#include "MemoryObjectView.h"

class MemoryObjectCache;

class IOWindowInfo {
	friend class IOWindowCache;

	unsigned _lock;
	unsigned _age;
	IOWindowInfo * _prev;
	IOWindowInfo * _next;
//	IOWindowInfo ** _list;
	
	char * _baseAddr;
	unsigned _from;
	unsigned _log2WindowSize;
	MemoryObjectCache * _cache;
	MemoryObjectView * _view;
	MemoryObjectView _realView;

public:
	IOWindowInfo() : _realView(0, 0, 0, 0)
		{ _lock = 0; _age = 0; _prev = 0; _next = 0; _from = 0;
		  _baseAddr = 0; _cache = 0; _view = &_realView; 
		  _log2WindowSize = 0; _view->reference(); }
	char * baseAddr() { return _baseAddr; }
	MemoryObjectCache * cache() { return _cache; }
	unsigned from() { return _from; }
	MemoryObjectView * view() { return _view; }
};

class AddressTranslation;

class IOWindowCache {
protected:
	static Lock _lock;
	static void acquire() { _lock.acquire(); }
	static void release() { _lock.release(); }
	
	unsigned _size;
	unsigned _hashSize;
	
	AddressTranslation * _adt;
	IOWindowInfo * _infos;
	IOWindowInfo * _freeList;
	IOWindowInfo ** _hashTable;

	static unsigned _age;

	unsigned age() 
		{ return _age++; }
	unsigned hash( MemoryObjectCache * cache, unsigned from )
		{ return ( ( ((unsigned) cache) >> 2 ) + from ) % _hashSize; }
	unsigned empty( IOWindowInfo * infoList )
		{ return infoList == 0; }
	void removeFromList( IOWindowInfo ** list,
			     IOWindowInfo * item );
	void addToList( IOWindowInfo ** list,
			IOWindowInfo * item );
	void free( IOWindowInfo * info, int untranslate );
	void freeExcess( MemoryObjectCache * cache );
	IOWindowInfo * victimize( MemoryObjectCache * cache );
	IOWindowInfo * basicAcquireIOWindow( MemoryObjectCache * cache,
					     unsigned from );

public:
        IOWindowCache( unsigned size );
	IOWindowInfo * acquireIOWindow( MemoryObjectCache * cache,
			   	        unsigned from, IOWindowInfo * hint );
	void releaseIOWindow( IOWindowInfo * info );
	void freeIOWindow( MemoryObjectCache * cache, int untranslate );
};

extern IOWindowCache * TheIOWindowCache;

inline
void 
IOWindowCache::removeFromList( IOWindowInfo ** list, IOWindowInfo * item )
{ 
//	Assert( item->_list == list );
//	item->_list = 0;
	IOWindowInfo * prev = item->_prev;
	if ( item->_prev ) {
		item->_prev->_next = item->_next;
		item->_prev = 0;
	} else {
		Assert( *list == item );
		*list = item->_next;
	}
	if ( item->_next ) {
		item->_next->_prev = prev;
		item->_next = 0;
	}
}

inline
void 
IOWindowCache::addToList( IOWindowInfo ** list, IOWindowInfo * item )
{
//	Assert( item->_list == 0 && item->_next == 0 && item->_prev == 0 );
	item->_prev = 0;
	item->_next = *list;
	if (item->_next) {
		Assert( item->_next->_prev == 0 );
//		Assert( item->_next->_list == list );
		item->_next->_prev = item;
	}
	*list = item;
//	item->_list = list;
}

inline
IOWindowInfo *
IOWindowCache::acquireIOWindow( MemoryObjectCache * cache, unsigned from,
				IOWindowInfo * hint )
{
        acquire();
        if ( hint != 0 && hint->_cache == cache && hint->_from == from ) {
                hint->_lock ++;
                hint->_age = age();
        } else {
		hint = basicAcquireIOWindow( cache, from );
	}
        release();
	return hint;
}

inline
void
IOWindowCache::releaseIOWindow( IOWindowInfo * info )
{
        Assert( info->_lock > 0 );
        acquire(); 
        info->_lock--;
        info->_age = age();
        release(); 
}
 
#endif // IOWindow_h
