/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Contributing Author: Vince Russo (russo@cs.uiuc.edu)
 * Contributing Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * LockHolder.h
 */

#ifndef LockHolder_h
#define LockHolder_h

#include "MemoryObjectCache.h"

class PhysicallyAddressableUnit;

class LockHolder {
private:
	int _referenceCount;
	unsigned _from;
	unsigned _to;
	unsigned _unlock;
	MemoryObjectCache * _cache;
	PhysicallyAddressableUnit * _chain;
public:
	LockHolder( unsigned from, unsigned to, MemoryObjectCache * cache,
		    PhysicallyAddressableUnit * chain, int unlock );	
	~LockHolder();
	void reference();
	void unreference();
	PhysicallyAddressableUnit * chain();
	int size();
};

inline 
LockHolder::LockHolder( unsigned from, unsigned to, MemoryObjectCache * cache,
			PhysicallyAddressableUnit * chain, int unlock )
{
	_referenceCount = 0;
	_from = from;
	_to = to;
	_cache = cache;
	_chain = chain;
	_unlock = unlock;
}

inline
LockHolder::~LockHolder()
{
	if (_unlock)
		_cache->unlock(_from, _to);
}

inline void
LockHolder::reference()
{
	_referenceCount++;
}

inline void
LockHolder::unreference()
{
	_referenceCount--;
	if (_referenceCount == 0)
		delete this;
}

inline PhysicallyAddressableUnit *
LockHolder::chain()
{
	return _chain;
}

inline int
LockHolder::size()
{
	return ( _to - _from + 1 ) << _cache->log2UnitSize();
}

#endif	/* LockHolder_h */
