/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * MemoryObjectAsync.h: 
 */

#ifndef MemoryObjectAsync_h
#define MemoryObjectAsync_h

#ifdef __GNUC__
#pragma interface
#endif

#include "Async.h"

class IOAsyncClosure : public AsyncClosure {
      public:
	unsigned _result;
	unsigned _rightShift;
  IOAsyncClosure(); // : _rightShift( 0 ) {}
	void init() { _rightShift = 0; }
  void activate() ;
  //{ (*_func)( this ); }
  void activate( int result ) ;
  //{ _result = ( result >> _rightShift ) ; (*_func)( this ); }
};

class SumResultIOAsyncClosure : public IOAsyncClosure {
      protected:
	IOAsyncClosure * _parent;
	int _sum;
	static void func( SumResultIOAsyncClosure * c );
      public:
	SumResultIOAsyncClosure( IOAsyncClosure * parent, int sum ) 
		: _parent( parent ), _sum(sum) {_func = (AsyncFunc)&func; }
};

class ReleaseIOAsyncClosure : public IOAsyncClosure {
      protected:
	IOAsyncClosure * _parent;
	MemoryObject * _memoryObject;
	unsigned _from;
	unsigned _to;
	static void func( ReleaseIOAsyncClosure * c );
	
      public:
	ReleaseIOAsyncClosure( IOAsyncClosure * parent, MemoryObject * mo, 
			       unsigned from, unsigned to )
		: 
		  _parent( parent ), _memoryObject( mo ),
		  _from( from ), _to( to ) {_func = (AsyncFunc)&func; }
};

class SynchronizeIOAsyncClosure : public IOAsyncClosure {
      protected:
	int _state;
	IOAsyncClosure * _parent;
	MemoryObject * _memoryObject;
	unsigned _from;
	unsigned _to;
	static void func( SynchronizeIOAsyncClosure * c );
	
      public:
	SynchronizeIOAsyncClosure( IOAsyncClosure * parent, MemoryObject * mo, 
				   unsigned from, unsigned to )
		:  
		        _state( 0 ),_parent( parent ),_memoryObject( mo ), _from( from ), _to( to ){_func = (AsyncFunc)&func; }
};

class TranslateBytesToUnitsAsyncClosure : public IOAsyncClosure {
      protected:
	IOAsyncClosure * _parent;
	unsigned _expected;
	unsigned _log2UnitSize;
	static void func( TranslateBytesToUnitsAsyncClosure * c );

      public:
	TranslateBytesToUnitsAsyncClosure( IOAsyncClosure * parent,
					   unsigned log2UnitSize,
					   unsigned expected )
		: _parent( parent ), _expected( expected ), 
		  _log2UnitSize( log2UnitSize ) {_func = (AsyncFunc)&func; }
};

class DummyAsyncClosure : public IOAsyncClosure
{
	static void func( DummyAsyncClosure * c );
public:
	DummyAsyncClosure() {_func = (AsyncFunc)&func; }
};

extern DummyAsyncClosure * theDummyAsyncClosure;

#endif /*MemoryObjectAsync_h*/
