/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * MemoryRange.h - Basic memory management class.
 */

#ifndef MemoryRange_h
#define MemoryRange_h 


#ifdef __GNUC__
#pragma interface
#endif

#include "Assert.h"
#include "ProxiableObject.h"

typedef unsigned int Offset;
class ClassStar;

extern ClassStar MemoryRangeClass;
class MemoryRange : public ProxiableObject {
protected:
	unsigned int _numberOfUnits;
	unsigned _log2UnitSize;

public:
	proxiable MemoryRange( unsigned int numberOfUnits, unsigned  log2UnitSize );
	virtual Class * classOf();

	/*
	 * Return the size of a unit in bytes.
	 */
	proxiable virtual unsigned  unitSize();

	/*
	 * Return the log base 2 of the size of a unit.
	 */
	proxiable virtual unsigned  log2UnitSize();

	/*
	 * Return how many units are in this range.
	 */
	proxiable virtual unsigned int numberOfUnits();

	/*
	 * Convert a byte offset into the range to a unit number.
	 */
	proxiable virtual unsigned int offsetToUnit( Offset offset );

	/*
	 * Convert a unit number into a byte offset into the range.
	 */
	proxiable virtual Offset unitToOffset( unsigned int unit );

	/*
	 * Is a unit within the range?
	 */
	proxiable virtual int isIn( unsigned unit );

	/*
	 * Floor and Ceiling an arbitrary number or address
	 */
	unsigned floorUnit( unsigned addr );
	const char * floorUnit( const char * addr );

	unsigned ceilingUnit( unsigned addr );
	const char * ceilingUnit( const char * addr );
	
	/*
	 * Alignment testing
	 */
	int unitAligned( unsigned addr );
	int unitAligned( const char * addr );
};

inline unsigned
MemoryRange::floorUnit( unsigned addr )
{
    return addr & (~( (((unsigned) 1) << _log2UnitSize) - 1));
}

inline const char *
MemoryRange::floorUnit( const char * addr )
{
    return (const char *) (((unsigned) addr ) &
			   (~( (((unsigned) 1) << _log2UnitSize) - 1)));
}

inline unsigned
MemoryRange::ceilingUnit( unsigned addr )
{
    unsigned i = (((unsigned) 1) << _log2UnitSize) - 1;
    return (addr + i) & (~i);
}

inline const char *
MemoryRange::ceilingUnit( const char * addr )
{
    unsigned i = (((unsigned) 1) << _log2UnitSize) - 1;
    return (const char *) ( (((unsigned)addr) + i) & (~i) );
}

inline int
MemoryRange::unitAligned( unsigned addr )
{
    return (addr & ((((unsigned) 1) << _log2UnitSize) - 1)) == 0;
}

inline int
MemoryRange::unitAligned( const char * addr )
{
    return (((unsigned)addr) & ((((unsigned) 1) << _log2UnitSize) - 1)) == 0;
}

#endif	/* MemoryRange_h */
