/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David K. Raila (raila@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
#ifndef MemorySimpleInstruments_h
#define MemorySimpleInstruments_h


#ifdef __GNUC__
#pragma interface
#endif

#include "SimpleInstruments.h"

	/*
	 **************************************************************
	 * Simple instruments for the Memory subsystem
	 * There is one class each for when SIMPLE_INSTRUMENTATION is defined
	 * and not.  Be sure to add/delete both!
	 **************************************************************
	 */
#ifdef SIMPLE_INSTRUMENTATION
extern ClassStar DomainTimerClass;
extern ClassStar MMUTimerClass;
extern ClassStar ADTTimerClass;
extern ClassStar StoreTimerClass;
extern ClassStar MemoryObjectCacheTimerClass;
extern ClassStar CacheTimerClass;

class DomainTimer : public MemoryTimer
{
public:
	DomainTimer(const char * const name) : MemoryTimer(name) {}
	Class * classOf();
};
class MMUTimer : public MemoryTimer
{
public:
	MMUTimer(const char * const name) : MemoryTimer(name) {}
	Class * classOf();
};
class ADTTimer : public MemoryTimer
{
public:
	ADTTimer(const char * const name) : MemoryTimer(name) {}
	Class * classOf();
};
class StoreTimer : public MemoryTimer
{
public:
	StoreTimer(const char * const name) : MemoryTimer(name) {}
	Class * classOf();
};
class MemoryObjectCacheTimer : public MemoryTimer
{
public:  
        MemoryObjectCacheTimer(const char * const name) : MemoryTimer(name) {}
        Class * classOf();
};
class CacheTimer : public MemoryTimer
{
public:  
        CacheTimer(const char * const name) : MemoryTimer(name) {}
        Class * classOf();
};
#else /* NOT SIMPLE_INSTRUMENTATION*/

class DomainTimer : public MemoryTimer 
{
public:
	DomainTimer( const char *const, SimpleInstrumentState = ACTIVE ) {}
};
class MMUTimer : public MemoryTimer
{
public:
	MMUTimer( const char *const, SimpleInstrumentState = ACTIVE ) {}
};
class ADTTimer : public MemoryTimer 
{
public:
	ADTTimer( const char *const, SimpleInstrumentState = ACTIVE ) {}
};
class StoreTimer : public MemoryTimer 
{
public:
	StoreTimer( const char *const, SimpleInstrumentState = ACTIVE ) {}
};
class MemoryObjectCacheTimer : public MemoryTimer 
{
public:
	MemoryObjectCacheTimer( const char *const, SimpleInstrumentState = ACTIVE ) {}
};
class CacheTimer : public MemoryTimer 
{
public:
	CacheTimer( const char *const, SimpleInstrumentState = ACTIVE ) {}
};
#endif /*SIMPLE_INSTRUMENTATION*/

#endif /* MemorySimpleInstruments_h*/
