/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Contributing Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * PagedMemoryObjectCache.h: A MemoryObjectCache for paged harware.
 */

#ifndef PagedMemoryObjectCache_h
#define PagedMemoryObjectCache_h


#ifdef __GNUC__
#pragma interface
#endif

#include "MemoryObjectCache.h"	/* parent class */

class MemoryObject;	
class MultiLevelTable;
class Cache;

extern ClassStar PagedMemoryObjectCacheClass;

class ReplacementPolicy;

class PagedMemoryObjectCache : public MemoryObjectCache {
/*
 * A PagedMemoryObjectCache is the normal cache for use by user
 * programs. It supports basic pagein/pageout, and multiple simultaneous 
 * pagefaults using bitmap locks.
 */
      protected:
	
	/*
         * These methods include those that were undefined in
	 * MemoryObjectCache.
	 */
	virtual PhysicallyAddressableUnit * storeAlloc(unsigned units);
	virtual void storeDealloc( PhysicallyAddressableUnit *, unsigned q );
	virtual unsigned memoryObjectAlloc( unsigned from, unsigned to );
	
	friend	class ReplacementPolicy;
	virtual void unitsToStoreQueued( unsigned nunits );
	
      public:
	
	Class * classOf();
	
	PagedMemoryObjectCache( MemoryObject * memobj );
	~PagedMemoryObjectCache();
	
	virtual int setNumberOfUnits( unsigned );
	virtual void setReplacementPolicyClass( Class * policyClass );
	virtual unsigned pageOut( unsigned noOfPaus );
	virtual void addCache( Class * cacheClass );
};

#endif /* PagedMemoryObjectCache_h */
