/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Bjorn A. Helgaas (helgaas@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * PhysicalMemoryRange.h - Physical memory management
 */

#ifndef PhysicalMemoryRange_h
#define PhysicalMemoryRange_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Assert.h"
#include "MemoryRange.h"


extern ClassStar PhysicalMemoryRangeClass;
class PhysicalMemoryRange : public MemoryRange {
protected:
	PhysicalMemoryRange * _next;

	const char *	_physicalAddress;
	const char *	_virtualAddress;
	int		_transparentlyMapped;
	int		_preAllocated;
	char *		_objectName;

public:
	PhysicalMemoryRange( const char * physicalAddress,
		unsigned int units, int log2UnitSize,
		char * name = "PhysicalMemoryRange",
		int preAllocated = 0 );
	PhysicalMemoryRange( const char * physicalAddress,
		const char * virtualAddress,
		unsigned int units, int log2UnitSize,
		char * name = "PhysicalMemoryRange",
		int preAllocated = 0 );
	~PhysicalMemoryRange();

	PhysicalMemoryRange * next();
	void setNext( PhysicalMemoryRange * range );

	const char *	physicalAddress();
	const char *	virtualAddress();
	int		transparentlyMapped();
	int		preAllocated();
	char *		name();

	void		add( PhysicalMemoryRange * newRange );

	virtual Class * classOf();
};


inline PhysicalMemoryRange *
PhysicalMemoryRange::next()
{
	return( _next );
}

inline void
PhysicalMemoryRange::setNext( PhysicalMemoryRange * range )
{
	/*
	 * Once the SystemMemoryMap linked list of PhysicalMemoryRange's
	 * is built, it better not be changed, because various objects
	 * have copies of the head pointer (i.e., Store,
	 * PhysicalMemoryManager, machine-dependent Translation, etc).
	 */

	Assert( _next == 0 );

	_next = range;
}

inline const char *
PhysicalMemoryRange::physicalAddress()
{
	return( _physicalAddress );
}

inline const char *
PhysicalMemoryRange::virtualAddress()
{
	return( _virtualAddress );
}

inline int
PhysicalMemoryRange::transparentlyMapped()
{
	return( _transparentlyMapped );
}

inline int
PhysicalMemoryRange::preAllocated()
{
	return( _preAllocated );
}

inline char *
PhysicalMemoryRange::name()
{
	return( _objectName );
}

#endif	/* PhysicalMemoryRange_h */
