/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * PremappedMemoryObjectCache.h: A physical memory cache of a memory objects
 *				 that is already in physical memory at
 *				 boot time data. (E.g. the I/O registers).
 */

#ifndef PremappedMemoryObjectCache_h
#define PremappedMemoryObjectCache_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Map.h"
#include "PhysicallyAddressableUnit.h"
#include "SpecialPurposeCache.h"	/* parent class */
#include "TranslationMap.h"

class MemoryObjectViewRef;

extern ClassStar PremappedMemoryObjectCacheClass;

class PremappedMemoryObjectCache : public SpecialPurposeCache {
/*
 * A PremappedMemoryObjectCache is used for the BootHeap, One-to-one
 * mapped memory objects and other special purpose kernel uses. This
 * cache does not have a map. It uses the MainMemory directly to find paus
 * PremappedCache faults occur only due to missing translation mappings.
 * Neither conventional pageIn nor pageOut is supported
 */

	PremappedMap _premappedMap;
	ConstantSizeTranslationMap _constantSizeTranslationMap;
	
 protected:
	int _preAllocated;

	void noRemainingReferences();
    
 public:
	
	virtual Class * classOf();
	
	PremappedMemoryObjectCache( MemoryObject * memobj,
				    const char * physBase,
				    int preAllocated );
	~PremappedMemoryObjectCache();
	
	/*
	 * Special versions of cache functions that support only missing
	 * translation mappings
	 */
    
	virtual LockHolder *
		cache( unsigned int unit,
		       ProtectionLevel requiredProtection, 
	 	       ProtectionLevel & grantedProtection, 
		       FaultErrorCode & code,
		       AddressTranslation * adt, const char * vaddr, 
		       CacheReason reason, MemoryObjectView * view,
		       const char * baseAddr );
	
	virtual LockHolder *
		mcache( unsigned int unit, int nunits,
		        ProtectionLevel requiredProtection, 
		        ProtectionLevel & grantedProtection, 
		        FaultErrorCode& code,
		        AddressTranslation * adt, const char * vaddr, 
		        CacheReason reason, MemoryObjectView * view,
		        const char * baseAddr);

};

#endif /* PremappedMemoryObjectCache_h */
