/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Bjorn Helgaas (helgaas@cs.uiuc.edu)
 * Contributing Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * RandomPagedMemoryObjectCache.h: A MemoryObjectCache for paged hardware.
 */

#ifndef RandomPagedMemoryObjectCache_h
#define RandomPagedMemoryObjectCache_h


#ifdef __GNUC__
#pragma interface
#endif

#include "FreeRunningTimer.h"
#include "OutputStream.h"
#include "PagedMemoryObjectCache.h"	/* parent class */


static const int defaultRandomWSS = 10;

extern ClassStar RandomPagedMemoryObjectCacheClass;
class RandomPagedMemoryObjectCache : public PagedMemoryObjectCache {
protected:
	unsigned int * residentUnit;

	unsigned int victim();
	void newArrival( unsigned int unit );

public:
	RandomPagedMemoryObjectCache( MemoryObject * memobj, unsigned int length );
	~RandomPagedMemoryObjectCache();
	Class * classOf();
};

inline
RandomPagedMemoryObjectCache::RandomPagedMemoryObjectCache(
	MemoryObject * memoryObject, unsigned int length )
	: ( memoryObject, length )
{
	Console() << this << "::RandomPagedMemoryObjectCache( length: "
		<< length << " )\n" << eor;
	residentUnit = new unsigned int[length];
	_workingSetSize = (length < defaultRandomWSS) ? length : defaultRandomWSS;
}

inline
RandomPagedMemoryObjectCache::~RandomPagedMemoryObjectCache()
{
	Console() << this << "::~RandomPagedMemoryObjectCache()\n" << eor;
	Assert( this != 0 );
	delete residentUnit;
}

#endif	/* RandomPagedMemoryObjectCache.h */
