/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * RangeLock.h
 */

#ifndef RangeLocks_h
#define RangeLocks_h

#ifdef __GNUC__
#pragma interface
#endif

#include "BlockedInfo.h"
#include "Link.h"
#include "List.h"
#include "Lock.h"
// #include "Object.h"
#include "Use.h"

class Process;
class ProxiableObject;

class RangeLock : public Object
{
      protected:
	class _FaultInfo : public BlockedInfo {
	      public:
		unsigned from() { return _buf[0]; }
		unsigned to() { return _buf[1]; }
		void setFrom( unsigned i ) { _buf[0] = i; }
		void setTo( unsigned i ) { _buf[1] = i; }
	};

	Lock _lock;
	Use * _use;
	int _exclusive;
	BlockedInfoList _blocked;
	ProxiableObject * _object;

	virtual void block( unsigned from, unsigned to );
	virtual void unblock();

      public:
	RangeLock( ProxiableObject * o ) { _object = o; _exclusive = 0; }
	void acquire() { _lock.acquire(); }
	void release() { _lock.release(); }
	virtual Process * acquire( unsigned from, unsigned to, int block = 1,
				   int locked = 0 );
	virtual Process * release( unsigned from, unsigned to, 
				   int locked = 0 );
	virtual void acquireAll();
	virtual void releaseAll();
	virtual void downgradeFromAll( unsigned from, unsigned to );
};

class SpecialRangeLock : public RangeLock {
      protected:
	ConstantSizeTableUse _constantSizeUse;
      
	Lock _myLock;
	virtual void block( unsigned from, unsigned to );
	virtual void unblock();
      public:
	SpecialRangeLock( ProxiableObject * );
};

class NormalRangeLock : public RangeLock {
      protected:
        VariableSizeTableUse _variableSizeUse;

      public:
	NormalRangeLock( ProxiableObject * );
};

class MutexLock 
{
      protected:
	class _FaultInfo : public BlockedInfo {
	      public:
	};

	static Lock _lock;
	Process * _holder;
	BlockedInfoList _blocked;
	ProxiableObject * _object;

      public:
	MutexLock( ProxiableObject * o ) : _holder(0), _object( o ) {}
	void acquire();
	void release();
	void P() { acquire(); }
	void V() { release(); }
};

#endif /*RangeLocks_h*/

