/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SpecialPurposeCache.h: Special purpose kernel caches that do not
 * support pageout and require fixed data structures.
 */

#ifndef SpecialPurposeCache_h
#define SpecialPurposeCache_h


#ifdef __GNUC__
#pragma interface
#endif

#include "MemoryObjectCache.h"		/* parent class */
#include "AddressTranslation.h"

extern ClassStar SpecialPurposeCacheClass;

class SpecialPurposeCache: public MemoryObjectCache {
/*
 * A SpecialPurposeCache is an abstract superclass of caches for the
 * kernel heap (RootMemoryObjectCache) and the kernel one-to-one
 * mapped objects cache (PremappedMemoryObjectCache). These caches do
 * not support pageout for any purpose. They also cannot have dynamic
 * structures. Specifically, the adt list, for these caches is simply
 * an array. These caches also have a special interface to the Store
 * does not awaken the pager. 
 */
    
  protected:
    
    /*
     * Methods defined in this class
     */
    
    virtual PhysicallyAddressableUnit * storeAlloc( unsigned );
    virtual void storeDealloc( PhysicallyAddressableUnit *, unsigned queued );
    virtual unsigned memoryObjectAlloc( unsigned from, unsigned to );
    virtual void unitsToStoreQueued( unsigned nunits );

    /*
     * These methods are defined to do nothing so that general purpose
     * algorithms can be written in the superclass.
     */
    
    virtual void saveModified(unsigned int from, unsigned to, int code, 
			      IOAsyncClosure * c);
    virtual void toNet(unsigned from, unsigned to);
    virtual void fromNet(unsigned from, unsigned to);

    /* Method ~SpecialPurposeCache: null destructor. */
    
    ~SpecialPurposeCache(){};
    
  public:
    
    Class * classOf();
    
    /* Method SpecialPurposeCache: null constructor. */

    SpecialPurposeCache(MemoryObject * mo);

    virtual int setNumberOfUnits( unsigned size );

    /*
     * Public methods that should not be called
     */

    virtual void setReplacementPolicyClass( Class * policyClass );
    virtual void flush();
    virtual void flush(unsigned from, unsigned to);
    virtual void synchronize( IOAsyncClosure * c );
    virtual void synchronize( unsigned from, unsigned to, IOAsyncClosure * c );
    virtual void addCache( Class * cacheClass );
    virtual unsigned pageOut( unsigned noOfPaus );
};

#endif /* SpecialPurposeCache_h */
