/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * StoreInfo.h - 
 */

#ifndef StoreInfo_h
#define StoreInfo_h

#ifdef __GNUC__
#pragma interface
#endif

#include "Assert.h"

enum StoreInfoType {
	STOREINFO_NOT,
	STOREINFO_LISTHEAD,
	STOREINFO_ITEM
};
	
class MemoryObjectCache;

class StoreInfo {
protected:
	friend class Store;

	StoreInfoType _type;
	MemoryObjectCache * _cache;
	unsigned _paus;
	unsigned _queued;
	StoreInfo * _next;
	StoreInfo * _prev;
public:
	StoreInfo( MemoryObjectCache * cache );
	void unlink();
	void insertAfter( StoreInfo * );
	void insertBefore( StoreInfo * );
	int  isEmpty();
	int  queued();
};

inline
StoreInfo::StoreInfo( MemoryObjectCache * cache ) 
	: _type( STOREINFO_NOT ), _cache( cache ), _paus( 0 ), _queued( 0 ) 
{
}

inline 
void
StoreInfo::unlink()
{
	Assert( _type != STOREINFO_LISTHEAD );
	_next->_prev = _prev;
	_prev->_next = _next;
	_next = _prev = 0;
}

inline
void
StoreInfo::insertAfter( StoreInfo * info )
{
	Assert( _type == STOREINFO_ITEM );
	StoreInfo * prev = info;
	StoreInfo * next = info->_next;
	_prev = prev;
	_next = next;
	prev->_next = this;
	next->_prev = this;
}

inline
void
StoreInfo::insertBefore( StoreInfo * info )
{
	Assert( _type == STOREINFO_ITEM );
	StoreInfo * next = info;
	StoreInfo * prev = info->_prev;
	_next = next;
	_prev = prev;
	next->_prev = this;
	prev->_next = this;
}

inline
int
StoreInfo::isEmpty()
{
	Assert( _type == STOREINFO_LISTHEAD );
	return ( _next == _prev );
}

inline
int 
StoreInfo::queued()
{
	return _queued;
}

#endif /*StoreInfo_h*/

