/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * TranslationMap.h
 */

#ifndef TranslationMap_h
#define TranslationMap_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Lock.h"
#include "ProtectionLevel.h"

class Map;
class MemoryObjectView;

class _adt {

protected:
	AddressTranslation * _a;
	const char * _vadd;
	unsigned _offset;
	unsigned _size;
	_adt * _next;
public:
	_adt(AddressTranslation * a, const char * v, 
	     unsigned offset, unsigned size);
	_adt();
	~_adt();
	AddressTranslation * adt();
	const char * addr();
	unsigned offset();
	unsigned size();
	_adt * nextLink();

	void setAdt(AddressTranslation * a);
	void setAddr(const char * v);
	void setOffset(unsigned offset);
	void setSize(unsigned size);
	void setNextLink(_adt * a);

	int operator==(_adt & a);
	void init(AddressTranslation * a, const char * v, 
		  unsigned offset, unsigned size);

};

inline
_adt::_adt(AddressTranslation * a, const char * v, 
	   unsigned offset, unsigned size)
{
	_a = a; 
	_vadd = v;
	_offset = offset;
	_size = size;
}

inline
_adt::_adt()
{
}

inline
_adt::~_adt()
{
}

inline AddressTranslation *
_adt::adt()
{
	return _a;
}

inline void
_adt::setAdt(AddressTranslation * a)
{
	_a = a;
}

inline const char *
_adt::addr()
{
	return _vadd;
}

inline void
_adt::setAddr(const char * v)
{
	_vadd = v;
}

inline unsigned
_adt::offset()
{
	return _offset;
}

inline _adt *
_adt::nextLink()
{
	return _next;
}

inline void
_adt::setOffset(unsigned offset)
{
	_offset = offset;
}

inline unsigned
_adt::size()
{
	return _size;
}

inline void
_adt::setSize(unsigned size)
{
	_size = size;
}

inline void
_adt::setNextLink(_adt * next)
{
	_next = next;
}

inline int
_adt::operator==(_adt & a)
{
	return ( adt() == a.adt() && addr() == a.addr() );
}

inline void
_adt::init(AddressTranslation * a, const char * v, 
	   unsigned offset, unsigned size)
{
	_a = a; 
	_vadd = v;
	_offset = offset;
	_size = size;
}

class TranslationMap : public Object {
protected:
	Lock * _lock;

	Map * _map;

	_adt * _head;
	_adt * _prev;
	_adt * _current;

	virtual _adt * newEntry( AddressTranslation * a, const char * baseAddr,
				 MemoryObjectView * view ) = 0;
	virtual void deleteEntry( _adt * a ) = 0;

        virtual _adt * first();
	virtual _adt * next();
	virtual _adt * remove();

	virtual void add( AddressTranslation * a, const char * baseAddr, 
			  MemoryObjectView * view );
public:
	TranslationMap( Map * map );
	~TranslationMap();
	void acquire();
	void release();
        virtual void translatable( unsigned from, unsigned to, 
				   ProtectionLevel prot, 
				   AddressTranslation * adt,
				   const char * vaddr,
				   MemoryObjectView * view,
				   const char * baseAddr,
				   PhysicallyAddressableUnit * pau );
        virtual void untranslatable( unsigned from, unsigned to,
				     AddressTranslation * adt,
				     const char * baseAddr );
        virtual int  referenced( unsigned from, unsigned to, int clear );
};

inline 
void 
TranslationMap::acquire()
{
	_lock->acquire();
}

inline 
void
TranslationMap::release()
{
	_lock->release();
}

class LinkedListTranslationMap : public TranslationMap
{
protected:
	static int _freeCount;
	static _adt * _free;
	static Lock _staticLock;
	virtual _adt * newEntry( AddressTranslation * a, const char * baseAddr,
				 MemoryObjectView * view );
	virtual void deleteEntry( _adt * a );

public:
	LinkedListTranslationMap( Map * map );
};

const int MaxConstantSizeTranslationMapAdt = 32;

class ConstantSizeTranslationMap : public LinkedListTranslationMap
{
protected:
	Lock _instanceLock;
	_adt * _freeList;
	_adt _array[ MaxConstantSizeTranslationMapAdt ];

	virtual _adt * newEntry( AddressTranslation * a, const char * baseAddr,
				 MemoryObjectView * view );
	virtual void deleteEntry( _adt * a );

public:
	ConstantSizeTranslationMap( Map * map );
};

#endif /*TranslationMap_h*/
