/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * UNIXInodeCache.h
 */

#ifndef UNIXInodeCache_h
#define UNIXInodeCache_h


#ifdef __GNUC__
#pragma interface
#endif

#include "UNIXInode.h"
#include "RealCache.h"

class MemoryObjectContainer;

extern ClassStar UNIXInodeCacheClass;

class UNIXInodeCache;

class UNIXInodeCacheLink {
protected:
	UNIXInodeCacheLink * _next;
	UNIXInodeCacheLink * _prev;
	UNIXInodeCache * _cache;	
public:
	UNIXInodeCacheLink( UNIXInodeCache * c ) : _cache( c ) {}
	void insertAfter( UNIXInodeCacheLink * );
	void insertBefore( UNIXInodeCacheLink * );
	void unlink();
	UNIXInodeCacheLink * next() { return _next; }
	UNIXInodeCacheLink * prev() { return _prev; }
	UNIXInodeCache * cache()    { return _cache; }
};

inline
void
UNIXInodeCacheLink::insertAfter( UNIXInodeCacheLink * link )
{
        UNIXInodeCacheLink * prev = link;
        UNIXInodeCacheLink * next = link->_next;
        _prev = prev;
        _next = next;
        prev->_next = this;
        next->_prev = this;
}
 
inline
void
UNIXInodeCacheLink::insertBefore( UNIXInodeCacheLink * link )
{
        UNIXInodeCacheLink * next = link;
        UNIXInodeCacheLink * prev = link->_prev;
        _next = next;
        _prev = prev;
        next->_prev = this;
        prev->_next = this;
}

inline
void
UNIXInodeCacheLink::unlink()
{
	Assert( _cache != 0 );
        _next->_prev = _prev;
        _prev->_next = _next;
        _next = _prev = 0;
}

class UNIXInodeCacheList : public UNIXInodeCacheLink {
public:
	UNIXInodeCacheList() 
	: UNIXInodeCacheLink( 0 ) { _next = _prev = this; }
	int isEmpty() { return _next == _prev; }
	UNIXInodeCacheLink * head() { return _next; }
	UNIXInodeCacheLink * tail() { return _prev; }
};

class UNIXInodeCache : public RealCache {
      protected:	
	static UNIXInodeCacheList _LRUList;
	static int _cachedBytes;
	static int _cachedFiles;
	static int _maxCachedBytes;
	static int _maxCachedFiles;
	static int _thresholdCachedBytes;
	static int _thresholdCachedFiles;
	static UNIXInodeCacheLink * _cursor;

	int _inodeLog2UnitSize;
	UNIXInode * _inode;
	PersistentMemoryObjectStar _pmoStar;
	UNIXInodeCacheLink _link;

	void updateLRU();

	virtual unsigned basicSetNumberOfUnits( unsigned int noOfUnits ); 
	virtual void basicAllocate( unsigned int count );
	virtual void basicFreeUnits( unsigned int startUnit, unsigned int count );
	virtual int basicRead ( unsigned int startUnit, unsigned int count, char * addr,
			        IOAsyncClosure * c );
	virtual int basicWrite( unsigned int startUnit, unsigned int count, char * addr,
			        IOAsyncClosure * c );

	static void cleaner();
	static void updateThresholds();
	static void exceedThresholds();
	static void printParams( OutputStream & );

      public:
	Class * classOf();

	UNIXInodeCache( MemoryObjectCache * moc, int & status );
	~UNIXInodeCache();

	virtual int read ( unsigned int startUnit, unsigned int count, char * addr,
			   IOAsyncClosure * c );
	virtual int write( unsigned int startUnit, unsigned int count, CacheUnitInfo * info,
			   char * addr, IOAsyncClosure * c );
	virtual int rawWrite( unsigned int startUnit, unsigned int count, CacheUnitInfo * info,
			      char * addr, IOAsyncClosure * c );

	static void cmd( int argc, char * argv[] );
};

#endif /*UNIXInodeCache_h*/
