/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * Use.h
 */

#ifndef Use_h
#define Use_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Lock.h"

static const unsigned MaxUse = 0xffffffff;

class Use {
public:
	Use();
	virtual ~Use();
	virtual Process * use( unsigned f, unsigned t, Process * p = 0 ) = 0;
	virtual Process * unuse( unsigned from, unsigned to ) = 0;
	virtual Process * lessUse( unsigned from, unsigned to ) = 0;
	virtual int noOfUsers() = 0;
};

inline
Use::Use() 
{ 
}

inline 
Use::~Use()
{
}

class _UseInfo {
      public:
	unsigned _from;
	unsigned _to;
	Process * _process;

	void init( unsigned from, unsigned to, Process * process = 0 );
	int member( unsigned i );
	int intersect( unsigned from, unsigned to );
	int subset( unsigned from, unsigned to );
	int equal( unsigned from, unsigned to );
};

inline void
_UseInfo::init( unsigned from, unsigned to, Process * process )
{
	_from = from;
	_to = to;
	_process = process;
}

inline int
_UseInfo::member( unsigned i )
{
	return i >= _from && i <= _to;
}

inline int
_UseInfo::intersect( unsigned from, unsigned to )
{
	return ! ( to < _from || from > _to );
}

inline int
_UseInfo::subset( unsigned from, unsigned to )
{
	return ( from >= _from && to <= _to );
}

inline int
_UseInfo::equal( unsigned from, unsigned to )
{
	return from == _from && to == _to;
}

class TableUse : public Use {
protected:

	_UseInfo * _table;
	int _allocatedSize;
	int _last;

	virtual void doubleTable() = 0;
public:
	TableUse( int concurrentFaults );
	Process * use( unsigned from, unsigned to, Process * process = 0 );
	Process * unuse( unsigned from, unsigned to );
	Process * lessUse( unsigned from, unsigned to );
	int noOfUsers();
};

class VariableSizeTableUse : public TableUse
{
protected:
	virtual void doubleTable();
public:
	VariableSizeTableUse( int concurrentFaults );
	~VariableSizeTableUse();
};

class ConstantSizeTableUse : public TableUse
{
	_UseInfo _preAllocatedTable[ 64 ];

protected:
	virtual void doubleTable();
public:
	ConstantSizeTableUse( int concurrentFaults );
};

#endif /*Use_h*/



