/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Bjorn A. Helgaas (helgaas@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * VirtualMemoryRange.h - Virtual memory management
 */

#ifndef VirtualMemoryRange_h
#define VirtualMemoryRange_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Assert.h"
#include "MemoryRange.h"


extern ClassStar VirtualMemoryRangeClass;
class VirtualMemoryRange : public MemoryRange {
protected:
	VirtualMemoryRange * _next;

	const char *	_virtualAddress;
	char *	_objectName;
	unsigned char _ring;

public:
	VirtualMemoryRange( const char * virtualAddress, unsigned int units,
		int log2UnitSize, char * name = "VirtualMemoryRange",
	    				unsigned char myRing = 0 );
	~VirtualMemoryRange();

	VirtualMemoryRange * next();
	void setNext( VirtualMemoryRange * range );

	const char *	virtualAddress();
	char *		name();
	unsigned char	ring();

	void		add( VirtualMemoryRange * newRange );

	Class * classOf();
};


inline VirtualMemoryRange *
VirtualMemoryRange::next()
{
	return( _next );
}

inline void
VirtualMemoryRange::setNext( VirtualMemoryRange * range )
{
	/*
	 * Once the SystemMemoryMap linked list of VirtualMemoryRange's
	 * is built, it better not be changed, because various objects
	 * have copies of the head pointer (i.e., Store,
	 * PhysicalMemoryManager, machine-dependent Translation, etc).
	 */

	Assert( _next == 0 );

	_next = range;
}

inline const char *
VirtualMemoryRange::virtualAddress()
{
	return( _virtualAddress );
}

inline char *
VirtualMemoryRange::name()
{
	return( _objectName );
}

inline unsigned char
VirtualMemoryRange::ring()
{
	return( _ring );
}

#endif	/* VirtualMemoryRange_h */
