/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Bjorn A. Helgaas (helgaas@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * MemoryBlockAllocator.h: definition of a generic heap class.
 */
#ifndef MemoryBlockAllocator_h
#define MemoryBlockAllocator_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Allocator.h"  /* the parent class */
#include "Lock.h"

extern ClassStar MemoryBlockAllocatorClass;
class MemoryBlockAllocator : public Allocator {
 protected:
    Lock            _mutex;
    unsigned        _blockSize;
    unsigned        _blockCount;
    char *          _firstBlock;
    char *          _lastBlock;
    unsigned        _freeListSize;
    unsigned char * _freeList;
    unsigned        _totalBytesFree;
 public:
    Class * classOf();

    MemoryBlockAllocator( int size, int blockSize );
    ~MemoryBlockAllocator();
    virtual void * allocateAligned( unsigned nbytes, unsigned align = 0 );
    virtual void * allocate( unsigned nbytes ) { return allocateAligned( nbytes, 0 ); };
    virtual void free( void * cp );
};

#endif /*MemoryBlockAllocator_h*/
