/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * NameServer.h: definition of a Choices persistent object name server.
 */

#ifndef NameServer_h
#define NameServer_h


#ifdef __GNUC__
#pragma interface
#endif

#include "ProxiableObject.h"
#include "ProxiableObjectStar.h"
#include "ClassStar.h"
#include "NameServerStar.h"
#include "Semaphore.h"

class AssocTable;

const int KERNELNAMESERVERHASHSIZE = 1024;
const int OTHERNAMESERVERHASHSIZE = 256;

extern ClassStar NameServerClass;
class NameServer : public ProxiableObject {
protected:
	MutexSemaphore mutex;
	AssocTable *hashTable;
	NameServerStar parent;

public:
	NameServer( NameServer * _parent );
	~NameServer();

	/*
	 * bind name in this NameServer to a ProxiableObject.
	 * If the name is already in the NameServer, override it.
	 */
	proxiable virtual void bind( char * name, ProxiableObject * );

	/*
	 * Lookup a name in the NameServer, and recursively in parent
	 *   if not found.  If the object looked up is not of the class
	 *   of the second parameter or if not found at all, returns 0,
	 *   otherwise returns a Ref to the ProxiableObject.
	 */
	proxiable virtual ProxiableObjectRef lookup( char * name, Class * );
	proxiable virtual ProxiableObjectRef lookup( char * name, char * );

	/*
	 * unbind name in this NameServer.
	 * returns 1 if successful, 0 if failed
	 */
	proxiable virtual int unbind( char * name );

	/*
	 * add all mappings from another nameserver
	 */
	proxiable virtual void clone( NameServer * nameServer );

	Class * classOf();
};

extern NameServer * StandardNameServer;

#endif /* NameServer_h */
