/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801-2910
 *
 * Copyright (c) 1991 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Amit Parghi (parghi@cs.uiuc.edu)
 * Contributing Author: Nayeem Islam (nislam@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *	NetworkAdministrator.h
 *
 *	Superclass for network administrator processes.
 *	This is just used for dispatching.
 *
 * $Id: NetworkAdministrator.h 2 2004-06-03 01:58:28Z jcarlyle $
 *
 * $Log: NetworkAdministrator.h,v $
 * Revision 1.1  2004/06/03 01:58:47  jcarlyle
 * Initial revision
 *
 * Revision 1.2  2004/04/21 22:47:29  dthomas1
 *
 * Compiler fixes
 *   Changed the file to add DLPI support
 * Changed the makefile to reflect changes in vcethernetu
 *
 * Revision 1.1.1.1  2004/03/10 23:01:52  kircher
 *
 *
 */


#ifndef NetworkAdministrator_h
#define NetworkAdministrator_h


#ifdef __GNUC__
#pragma interface
#endif


#include "ProxiableObject.h"		// super class definition


#include "NetworkBuffer.h"
#include "NetworkBufferQueue.h"
#include "NetworkDriver.h"// needed to inline getDriverStats()

class NetworkStatistics;

extern ClassStar NetworkAdministratorClass;

class NetworkAdministrator : public ProxiableObject {
private:
    NetworkBufferQueue *_receiveQueue;		// for packets rec'd by driver
    void receiveLoop();		//fetch incoming packets
protected:
	// performance statistics
	unsigned walkerTime;
	unsigned walkers;
    NetworkDriver *_driver;			// needed by subclasses

    NetworkStatistics *_stats;			// statistics for debugging

    // Network-specific method to map a type code to a Walker.
    // self subclassResponsibility
    virtual NetworkWalker *findWalker(NetworkBuffer *) = 0; 

    // Constructor and destructor are made protected in order to prevent
    // direct instantiation and to force instantiation only of subclasses.
    NetworkAdministrator(char *, unsigned char debug = 0);
    virtual ~NetworkAdministrator();

public:
	void printStats();
	void clearStats();
    void transmit(NetworkBuffer *);		// called by protocol code
    void receive(NetworkBuffer *);		// called by our driver

    NetworkStatistics *getStats();		// return administrator's stats
    NetworkStatistics *getDriverStats();	// return driver's stats

    virtual void ready(NetworkDriver *);	// set _driver value and go ...

    Class *classOf();
};


inline NetworkStatistics *NetworkAdministrator::getStats()
{
    return _stats;
}

inline NetworkStatistics *NetworkAdministrator::getDriverStats()
{
    return _driver->getStats();
}

#endif /* NetworkAdministrator_h */
