/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1990,1991 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Amit Parghi (parghi@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *	NetworkBufferManager.h
 *
 *	Class definition of the NetworkBufferManager.
 *
 * $Id: NetworkBufferManager.h 2 2004-06-03 01:58:28Z jcarlyle $
 *
 * $Log: NetworkBufferManager.h,v $
 * Revision 1.1  2004/06/03 01:58:47  jcarlyle
 * Initial revision
 *
 * Revision 1.1  1991/12/01  06:10:38  parghi
 * Initial revision
 *
 */


#ifndef NetworkBufferManager_h
#define NetworkBufferManager_h


#ifdef __GNUC__
#pragma interface
#endif


#include "Object.h"
#include "SemaphoreStar.h"
#include "ProcessStar.h"

class NetworkBuffer;
class Process;

const unsigned int BUFFER_SIZE_0 = 2048;

    //	Note that in the design of the NetworkBufferManager, the code
    //	should be machine-INdependent!!  The only machine-dependent parts
    //	should be the constants defining the buffer sizes and the high-
    //	and low-water marks for the sizes of the respective free lists.
    //							// Amit

class NetworkBufferManager;					// forward
void _NetworkBufferManagerDaemon(NetworkBufferManager *);	// forward


class NetworkBufferManager : public Object {
    friend void _NetworkBufferManagerDaemon(NetworkBufferManager *);

protected:
    NetworkBufferFreeList * _list;		// free list
    unsigned int _size;				// size of buffers in list

    ProcessStar _daemonProcess;
    SemaphoreStar _semaphore;			// used for synchronization

public:
    NetworkBufferManager(unsigned char debugMask = 0);
    virtual ~NetworkBufferManager();

    virtual NetworkBufferFreeList * getFreeList();
    virtual void launchDaemon();
};



	//  NetworkGenericBufferManager ... used for "general" applications.
	//
class NetworkGenericBufferManager : public NetworkBufferManager {
    friend void _NetworkBufferManagerDaemon(NetworkBufferManager *);

public:
    NetworkGenericBufferManager(unsigned int size = BUFFER_SIZE_0);
    ~NetworkGenericBufferManager();
};


#endif /* NetworkBufferManager_h */
