/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801-2910
 *
 * Copyright (c) 1991 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Amit Parghi (parghi@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *	NetworkBufferQueue.h
 *
 *	Maintain a queue of NetworkBuffers.  This is used to pass the
 *	NetworkBuffers to the NetworkAdministrator (which, being single-
 *	threaded, may not be able to deal with requests immediately).
 *
 * $Id: NetworkBufferQueue.h 2 2004-06-03 01:58:28Z jcarlyle $
 *
 * $Log: NetworkBufferQueue.h,v $
 * Revision 1.1  2004/06/03 01:58:47  jcarlyle
 * Initial revision
 *
 */


#ifndef NetworkBufferQueue_h
#define NetworkBufferQueue_h


#ifdef __GNUC__
#pragma interface
#endif


#include "Object.h"
#include "Types.h"


class List;
class Lock;
class NetworkBuffer;
class Semaphore;


class NetworkBufferQueue : public Object {
private:
    Lock	*_lock;	// mutex for adding/deleting
    Semaphore	*_semaphore;	// counter for synchronization
    List	*_queue;

public:
    NetworkBufferQueue();
    ~NetworkBufferQueue();

    void put(NetworkBuffer *);	
    NetworkBuffer *get();
};

#endif /* NetworkBufferQueue_h */
