/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801-2910
 *
 * Copyright (c) 1991,1992 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Amit Parghi (parghi@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *	NetworkDriver.h
 *
 *	This is the machine independent superclass for all network
 *	device drivers.
 *
 * $Id: NetworkDriver.h 2 2004-06-03 01:58:28Z jcarlyle $
 *
 * $Log: NetworkDriver.h,v $
 * Revision 1.1  2004/06/03 01:58:47  jcarlyle
 * Initial revision
 *
 * Revision 1.1  1991/12/01  06:11:23  parghi
 * Initial revision
 *
 */


#ifndef NetworkDriver_h
#define NetworkDriver_h


#ifdef __GNUC__
#pragma interface
#endif


#include "ProxiableObject.h"
#include "NetworkBuffer.h"
#include "Types.h"

class NetworkAdministrator;
class NetworkBuffer;
class NetworkStatistics;
class NetworkBufferFreeList;

extern ClassStar NetworkDriverClass;

class NetworkDriver : public ProxiableObject {
protected:
    NetworkBufferFreeList* flist;
    NetworkAdministrator *_administrator;	// our NetworkAdministrator
    NetworkStatistics *_driverStats;		// keep useful statistics

	// receivePacket() is the method invoked by the interrupt handler
	// when a network packet has been received.  Setting the
	// length of the packet is up to receivePacket(), and
	// using private instance variables known to the interrupt handler
	 // is certainly considered kosher.
	
    virtual void receivePacket(NetworkBuffer *packet);

	// The constructor is protected to prevent direct instantiation.
    NetworkDriver(unsigned char debugMask,NetworkBufferFreeList* fl=0);	// debugMask is used by Raid
    virtual ~NetworkDriver();

public:
    NetworkStatistics *getStats();		// retrieve our stats

	//  Last stage of initialization: set the _administrator pointer and
	//	start any daemons, etc. required.
	//  **N.B. the driver MUST NOT call the administrator, nor attempt
	//  to receive packets, before the Administrator calls ready().
	//
    virtual void ready(NetworkAdministrator *);

	// Called by the Administrator when transmitting a packet (obviously!).
    	// Yet another case of "self subclassResponsibility".
    virtual int transmitPacket(NetworkBuffer *) = 0;
   NetworkBuffer* getBuffer();

    Class * classOf();				// cruft for ProxiableObject...
};

inline NetworkBuffer *NetworkDriver::getBuffer()
{
    return flist->getBuffer();
}


#endif /* NetworkDriver_h */
