/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Johnny Zweig (zweig@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 *
 *
 * FIle: BigEndianByteOrder.h
 *
 */
#ifndef BigEndianByteOrder_h
#define BigEndianByteOrder_h


#ifdef __GNUC__
#pragma interface
#endif

#include "ConduitTypes.h"

//-----------------------------------------------------------------------------
	// If the local machine's byte order is the same as network
	//   byte order, the constant HOFFSET should be set to 0
	//   since we can use the same copy of tha value for both
	//   orderings. Otherwise, HOFFSET=1 since the value array
	//   will need to store both a nbo and a hbo version.
// BIG-ENDIAN SETUP:
const int	NOFFSET = 0;
		// offset in value-array of nbo copy of value
const int	HOFFSET = 0;
		// offset in value-array of hbo copy of value;
//-----------------------------------------------------------------------------
class ByteSwapper {
protected:
	inline int16	ntoh( int16 nval ) const; // Convert int from nbo to hbo
	inline int16	hton( int16 hval ) const; // Convert int from hbo to nbo
	inline int32	ntoh( int32 nval ) const;
	inline int32	hton( int32 hval ) const;
					    // (See definitions at the end
					    //   of this file.)
};
//-------------------- UTILITIES: ---------------------------------------------
//-----------------------------------------------------------------------------
//  The definitions of these silly inline functions to transpose byte order
//    are machine dependent. There's no way of getting around that....
//-----------------------------------------------------------------------------
// Since Network Byte Order is Big Endian, there is no need to swap:

inline int16 
ByteSwapper::ntoh( int16 val ) const {
	return( val );
}
inline int32 
ByteSwapper::ntoh( int32 val ) const {
	return( val );
}
inline int16 
ByteSwapper::hton( int16 val ) const {
	return( val );
}
inline int32 
ByteSwapper::hton( int32 val ) const {
	return( val );
}
//-----------------------------------------------------------------------------
#endif
