/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Adapted by: Johnny Zweig
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * MessageFIFO.h: a FIFO Message container.
 *
 */

#ifndef MessageFIFO_h
#define MessageFIFO_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Assert.h"
#include "OutputStream.h"
#include "Lock.h"
#include "Semaphore.h"
#include "ConduitTypes.h"
#include "Word.h"
#include "ZootReturnCodes.h"
class ConduitMessage;

/*
 * MessageFIFO -- a FIFO list of pointers to messages
 */

class MessageFIFO {
	/* Self-explanatory. */
	/* Method add: insert message into FIFO. */
	/* Method remove: obtain next message from FIFO. */
	/* Method isEmpty: handy, since remove may be blocking. */
protected:
	int sz;		// size, since this is a bounded-length buffer
	Lock lock;
	ConduitMessage * * msgs;	// array of messages being held
	int head;
	int tail;
public:
	MessageFIFO( int initialsize );
	virtual	~MessageFIFO();
	Return_Code add( ConduitMessage * );
		/* Will return OK unless there is no room to hold the
		 * message, in which case it returns BUFFER_FULL.
		 */
	ConduitMessage * remove();
	Boolean isEmpty();
};

inline
MessageFIFO::MessageFIFO( int initialsize ) : lock()
{
	Assert( this != 0 );
	head = tail = 0;
	sz = initialsize;
	msgs = new ConduitMessage *[initialsize]; // array of pointers to msgs
}
//----------------------------------------------------------------------
/*
 * A Semaphore regulated Message-FIFO.
 */
class SemaphoreRegulatedMessageFIFO : public MessageFIFO {
protected:
	Semaphore semaphore;
public:
	SemaphoreRegulatedMessageFIFO( int initialsize );
	~SemaphoreRegulatedMessageFIFO();
	Return_Code add( ConduitMessage * );
	ConduitMessage * remove();
};

inline
SemaphoreRegulatedMessageFIFO::SemaphoreRegulatedMessageFIFO( int initialsize )
: MessageFIFO( initialsize ), semaphore( 0 )
{
	Assert( this != 0 );
}

#endif /* MessageFIFO */
