/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Johnny Zweig (zweig@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * NetworkAddress.h
 *
 */

#ifndef NetworkAddress_h
#define NetworkAddress_h


#ifdef __GNUC__
#pragma interface
#endif

#include "ConduitTypes.h"
#include "Word.h"

class NetworkAddress {

/* An Address is just whatever information needs to accompany a message that
 *   is being inserted into a conduit in order to specify where it needs to
 *   go. So an IPAddress is 32 bits (such as 128.174.238.4), while a TCPAddress
 *   is just an offset into array that can be used to specify which connection
 *   a message is part of. The base class doesn't really do anything except
 *   root the chain of the important operations. (They don't need to be
 *   virtual, I don't think.... -JZ)
 */
	/* Method ptr: returns pointer to array of contiguous octets forming
		the address (where appropriate). */
	/* Method equals: copy value pointed to by given pointer. */
	/* Method cpInto: deposit value into memory pointed to by
		given pointer. */

protected:
public:
	NetworkAddress();
	~NetworkAddress();

	octet * ptr();
				/* pointer to the bytes that make up
				   the address (stored in network byte
				   order, of course). May not make sense
				   for some complicated subclasses. */
	void	copyFrom( octet * source );
				/* copy address right out of datagram */
	void	copyTo( octet * dest );
				/* copy address right into datagram */
};
inline
NetworkAddress::NetworkAddress()
{
}

#endif /* NetworkAddress_h */
