/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Johnny Zweig (zweig@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * ZootAddress.h
 *
 */

#ifndef ZootAddress_h
#define ZootAddress_h


#ifdef __GNUC__
#pragma interface
#endif

#include "ConduitTypes.h"
#include "Word.h"
#include "Ethernet.h"
#include "NetworkAddress.h"


class IPAddress : public NetworkAddress {
    friend OutputStream & operator << (OutputStream &, IPAddress &);
protected:
	union { octet _addr[4]; int32 force_alignment_thus; };
public:
	IPAddress();	/* just sets _addr[] = 0.0.0.0 */
	IPAddress( int a1, int a2, int a3, int a4 );
			/* sets _addr[] = a1.a2.a3.a4 */
	IPAddress( const IPAddress & source );
			/* copies the IPAddress */
	IPAddress( const Word32 & source );
			/* takes value out of word (so byte order is dealt
			 * with sensibly). */
	IPAddress( octet * source );
			 /* copies value out of a datagram (in nbo) */
	~IPAddress();

	Word32 val() const;	/* best way to obtain value -- foo.val().val()
				 * thus is the colloquial way to get an
				 * IP-address foo as an int32 */
	octet *	ptr();	/* pointer to addr[] */
	int	addrclass() const;	/* 0 = Class A;
					 * 1 = Class B;
				 	 * 2 = Class C (an enum would
					 *    probably be more appropriate) */
	void	copyTo( octet * dest ) const;		/* see above */
	void	copyFrom( octet * source );		/* see above */
	void	equals( int a1, int a2, int a3, int a4 ); /* see above */
	void	equals( Word32 & source );		/* see above */
	void	equals( const IPAddress & source );	/* see above */
	void	equals( int32 source );		/* converts from host- to
						 * network byte-order */
	Boolean	compare( const IPAddress & compareto ) const;
			/* return TRUE if they are the same,
			 * FALSE otherwise. */
	int operator==( const IPAddress & compareto ) const;
			/* same thing */
};

//-----------------------------------------------------------------------------
class TCPAddressSpecifier : public IPAddress {
	/* Not used at the moment */
protected:
	Word16	srcportnumber;	// Source port number (i.e. mine)
	Word16	destportnumber;	// Destination port #
	// Options need to be set up "by hand" in each TCP message (LATER)
	// Also, Urgent and Push data need to be handled manually
public:
	TCPAddressSpecifier();
	~TCPAddressSpecifier();

	Word16	sourcePort();
	Word16	destinationPort();
	void	setSourcePort( int16 newport );
	void	setSourcePort( Word16 & newport );
	void	setDestinationPort( int16 newport );
	void	setDestinationPort( Word16 & newport );
	// Note that equals() will set the IPAddress portion, but not the
	//   port, etc.!!! (Similarly, copyTo(octet*) will only scribble down
	//   the IP address part.)
};

/* omit
**********************Broken since G++ can't pass small structs!***************
//-----------------------------------------------------------------------------
// In a similar manner to message-ids, Handles are a handy thing that
//   you can give to a conduit, along with a message, that is required for
//   finding the correct connection that the message belongs to. For now, it's
//   an offset into an array, but later it will probably be a pointer to an
//   object that keeps track of a connection, or something.
class Handle {
protected:
	int	Offset;	// usually, a handle is used to store an offset into
			//   an array (say, of connection-descriptors)
public:
	Handle();
	Handle(int newoffset);
	~Handle();
	int	offset();
	void	setOffset(int newoffset);
};
inline
Handle::Handle() { Offset = 0; }
inline
Handle::Handle( int newoffset ) { Offset = newoffset; }
inline
Handle::~Handle() { }
inline int
Handle::offset() { return( Offset ); }
inline void
Handle::setOffset(int newoffset) { Offset = newoffset; }
*******************************************************************************
INSTEAD I NEED TO DO:
*/
typedef	int	Handle;
	/* A Handle is just an int, since it is currently just an offset
	   into a table of connection-descriptors in the TCP Conduit. */

//-----------------------------------------------------------------------------
class TCPAddress : public NetworkAddress {
	/* A TCP address is a handle, pure and simple. The only way to insert
	 * a TCP-message into a TCP-conduit that makes any sense is if you have
	 * already opened a connection and have a handle for the connection
	 * carrying the message.
	 */
protected:
	Handle myhandle;
public:
	TCPAddress();
	TCPAddress( Handle newhandle );
			   /* Useful to be able to assign a Handle to a
			    * TCPAddress. */
	~TCPAddress();
	Handle		handle();
			   /* All a TCP Address is is a handle; it just needs
			    * to tell the TCP Conduit which connection
			    * descriptor to use -- saves unnecessary searching
			    * to find the CD. */
	void		setHandle( Handle newhandle );
			   /* self-explanatory */
};

/***********************
class ZootEthernetAddress : public NetworkAddress {	// LATER!!!
protected:
	octet	_addr[6];
public:
	ZootEthernetAddress();
	ZootEthernetAddress( int a1, int a2, int a3, int a4, int a5, int a6 );
		// Sets _addr[]=a1:a2:a3:a4:a5:a6
	~ZootEthernetAddress();

	octet * ptr(); // points to _addr[]

	void	equals( int a1, int a2, int a3, int a4, int a5, int a6 );
	void	copyTo( octet * dest );
	void	copyFrom( octet * source );
	Boolean	compare( ZootEthernetAddress & compareto );
	int operator==( ZootEthernetAddress & compareto );
};
*****************/
//-----------------------------------------------------------------------------
//				inliners:
//-----------------------------------------------------------------------------
#ifndef PROXIFY
inline
IPAddress::IPAddress()
{
	for( int i = 0; i < 4; i++ ) _addr[i] = 0;
}

inline
IPAddress::IPAddress( int a1, int a2, int a3, int a4 )
{
	_addr[0] = a1;
	_addr[1] = a2;
	_addr[2] = a3;
	_addr[3] = a4;
}

inline
IPAddress::IPAddress(const Word32 & source)
{
	source.copyTo( & _addr[0] );
}

inline
IPAddress::IPAddress(const IPAddress & source)
{
	source.copyTo( & _addr[0] );
}

inline
IPAddress::IPAddress( octet * source )
{
	for( int i = 0; i < 4; i++ ) _addr[i] = source[i];
}

inline
IPAddress::~IPAddress()
{}


inline Word32
IPAddress::val() const
{
	Word32 tmp;
	tmp.equals( & _addr[0] );
	return( tmp );
}

inline octet *
IPAddress::ptr()
{
	return( & _addr[0] );
}

inline void
IPAddress::copyTo( octet * dest ) const
{
	for( int i = 0; i < 4; i++ ) dest[i] = _addr[i];
}

inline void
IPAddress::copyFrom( octet * source )
{
	for( int i = 0; i < 4; i++ ) _addr[i] = source[i];
}

inline void
IPAddress::equals( int a1, int a2, int a3, int a4 )
{
	_addr[0] = a1;
	_addr[1] = a2;
	_addr[2] = a3;
	_addr[3] = a4;
}

inline void
IPAddress::equals( Word32 &source )
{
	source.copyTo( & _addr[0] );
}

inline void
IPAddress::equals( const IPAddress &source )
{
	source.copyTo( & _addr[0] );
}

inline void
IPAddress::equals( int32 source )
{
	Word32 tmp;
	tmp.equals( source );
	tmp.copyTo( & _addr[0] );
}

inline OutputStream &operator << (OutputStream &output,
				  IPAddress &address)
{
    output << (int) address._addr[0] << "." << (int) address._addr[1] << 
      "." << (int) address._addr[2] << "." << (int) address._addr[3];
    return output;
}


inline
TCPAddress::TCPAddress()
{
	myhandle = 0;
}

inline
TCPAddress::TCPAddress( Handle newhandle )
{
	myhandle = newhandle;
}

inline
TCPAddress::~TCPAddress()
{}

inline Handle
TCPAddress::handle()
{
	return( myhandle );
}

inline void
TCPAddress::setHandle( Handle newhandle )
{
	myhandle = newhandle;
}

/**********************
inline
ZootEthernetAddress::ZootEthernetAddress()
{
	for( int i = 0; i < 6; i++ ) _addr[i] = 0;
}

inline
ZootEthernetAddress::ZootEthernetAddress( int a1,int a2,int a3,int a4,int a5,int a6 )
{
	_addr[0] = a1;
	_addr[1] = a2;
	_addr[2] = a3;
	_addr[3] = a4;
	_addr[4] = a5;
	_addr[5] = a6;
}

inline
ZootEthernetAddress::~ZootEthernetAddress()
{}

inline octet *
ZootEthernetAddress::ptr()
{
	return( & _addr[0] );
}

inline void
ZootEthernetAddress::equals( int a1, int a2, int a3, int a4, int a5, int a6 )
{
	_addr[0] = a1;
	_addr[1] = a2;
	_addr[2] = a3;
	_addr[3] = a4;
	_addr[4] = a5;
	_addr[5] = a6;
}

inline void
ZootEthernetAddress::copyTo( octet * dest )
{
	dest[0] = _addr[0];
	dest[1] = _addr[1];
	dest[2] = _addr[2];
	dest[3] = _addr[3];
	dest[4] = _addr[4];
	dest[5] = _addr[5];
}

inline void
ZootEthernetAddress::copyFrom( octet * source )
{
	_addr[0] = source[0];
	_addr[1] = source[1];
	_addr[2] = source[2];
	_addr[3] = source[3];
	_addr[4] = source[4];
	_addr[5] = source[5];
}

inline Boolean
ZootEthernetAddress::compare( ZootEthernetAddress & compareto )
{
	octet * finger = compareto.ptr();

	if(     finger[0] == _addr[0] &&
		finger[1] == _addr[1] &&
		finger[2] == _addr[2] &&
		finger[3] == _addr[3] &&
		finger[4] == _addr[4] &&
		finger[5] == _addr[5]    ) {
	    return( TRUE );
	}
	else {
	    return( FALSE );
	}
}

inline int
ZootEthernetAddress::operator==( ZootEthernetAddress & compareto )
{
	octet * finger = compareto.ptr();

	if(     finger[0] == _addr[0] &&
		finger[1] == _addr[1] &&
		finger[2] == _addr[2] &&
		finger[3] == _addr[3] &&
		finger[4] == _addr[4] &&
		finger[5] == _addr[5]    ) {
	    return( TRUE );
	}
	else {
	    return( FALSE );
	}
}

*******************/
//-----------------------------------------------------------------------------
inline
TCPAddressSpecifier::TCPAddressSpecifier()
{

        short unsigned int temp = 0;

	srcportnumber.equals( temp );
	destportnumber.equals( temp );
}

inline
TCPAddressSpecifier::~TCPAddressSpecifier()
{
}

inline Word16
TCPAddressSpecifier::sourcePort()
{
	return( srcportnumber );
}

inline Word16
TCPAddressSpecifier::destinationPort()
{
	return( destportnumber );
}

inline void
TCPAddressSpecifier::setSourcePort( int16 newport )
{
	srcportnumber.equals( newport );
}

inline void
TCPAddressSpecifier::setSourcePort( Word16 & newport )
{
	srcportnumber.equals( newport );
}

inline void
TCPAddressSpecifier::setDestinationPort( int16 newport )
{
	destportnumber.equals( newport );
}

inline void
TCPAddressSpecifier::setDestinationPort( Word16 & newport )
{
	destportnumber.equals( newport );
}

#endif


/******************************************************************************
	For reasons I won't go into now, these local addresses of interest
	get figured out by some code in ConduitTypes.cc, which
	is dumb and I will move them around in some big code reorganization
	Real Soon Now(tm). -JZ
******************************************************************************/
extern IPAddress		TheLocalIPAddress;
extern IPAddress		TheLocalGatewayIPAddress;
extern EthernetAddress	TheLocalEthernetAddress;
extern EthernetAddress	TheLocalGatewayEthernetAddress;

#endif /* ZootAddress_h */
