/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801-2910
 *
 * Copyright (c) 1990,1991 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Amit Parghi (parghi@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *	Ethernet.h
 *
 *	$Id: Ethernet.h 2 2004-06-03 01:58:28Z jcarlyle $
 *
 */

#ifndef Ethernet_h
#define Ethernet_h


#ifdef __GNUC__
#pragma interface
#endif

#include "OutputStream.h"
#include "Types.h"
#include "HexDump.h"
#include "Address.h"


const unsigned int MinEthernetPacketLength = 60;
const unsigned int MaxEthernetPacketLength = 1514;
				// This measures the number of octets between
				//   the source-addr and the end of data only

	//  Typedef used to define the type of the Ethernet protocol
	//  type code.  For a partial list of currently assigned type
	//  codes, see the list at the bottom of this file.
typedef int16 EthernetProtocolID;

		// Ethernet packet type code used for our tests.
const EthernetProtocolID EthernetSpecialTestType = 0xFACE;


const unsigned int EthernetAddressLength = 6;
const unsigned int MulticastAddresses = 64;

class EthernetAddress : public Address {
	// Format assist that allows EthernetAddresses to be shifted
	//   onto output streams and look sensible
    //friend OutputStream & operator << (OutputStream &, EthernetAddress &);

private:
    octet _addr[EthernetAddressLength];		// packed binary form

    int isEqual(const EthernetAddress &) const;	// comparison!

public:
    EthernetAddress();
    EthernetAddress(const octet *);		// initialize with copyFrom
    EthernetAddress(const EthernetAddress &);
    EthernetAddress(const int, const int, const int,
		    const int, const int, const int);
    virtual ~EthernetAddress();

    inline void copyFrom(const octet *);	// convert to/from binary
    inline void copyTo(octet *) const;

    int operator == (const EthernetAddress &);	// comparison
    int operator != (const EthernetAddress &);
    Address & operator = (const EthernetAddress &);  // assignment
    void equals(const int a0, const int a1, const int a2,
		const int a3, const int a4, const int a5);  // in-place assign!

    int isBroadcast() const;			// non-zero ==> broadcast addr

    inline int isGroup() const;
    inline int isMulticast() const;
    inline void makeGroup();
	 inline operator octet*() { return _addr;}

    octet *bytes() {
	return _addr;
    }
};



	//	Note that Cfront is too stupid to understand the in-lining
	//	of these functions and their invocation within the
	//	constructors immediately below .  Wotta pain.		// Amit
#ifndef CFRONT
inline void
EthernetAddress::copyTo(octet *destination) const //copy into a buffer
{
    for (int i = 0 ; i < EthernetAddressLength ; i++) {
	*destination = _addr[i];
	destination++;
    }
}


inline void
EthernetAddress::copyFrom(const octet *source)	// copy out of a buffer
{
    for (int i = 0 ; i < EthernetAddressLength ; i++) {
	_addr[i] = *source;
	source++;
    }
}
#endif /* ! CFRONT */


inline
EthernetAddress::EthernetAddress(const octet *buffer)
{
    copyFrom(buffer);
}

inline
EthernetAddress::EthernetAddress(const EthernetAddress &address)
{
    address.copyTo(_addr);
}

inline
EthernetAddress::EthernetAddress(const int a0, const int a1,
				 const int a2, const int a3,
				 const int a4, const int a5)
{
    _addr[0] = (octet) a0;		// WARNING!!  POSSIBLE TRUNCATION!!
    _addr[1] = (octet) a1;
    _addr[2] = (octet) a2;
    _addr[3] = (octet) a3;
    _addr[4] = (octet) a4;
    _addr[5] = (octet) a5;
}

    inline
EthernetAddress::~EthernetAddress()
{
}


inline int EthernetAddress::operator == (const EthernetAddress &addr2)
{
    return isEqual( addr2);
}

inline int EthernetAddress::operator != (const EthernetAddress &addr2)
{
    return (! isEqual(addr2));
}

inline Address & EthernetAddress::operator = (const EthernetAddress & addr2)
{
    addr2.copyTo(_addr);

    return *this;
}

inline void EthernetAddress::equals(const int a0, const int a1, const int a2,
				    const int a3, const int a4, const int a5)
{
    _addr[0] = (octet) a0;
    _addr[1] = (octet) a1;
    _addr[2] = (octet) a2;
    _addr[3] = (octet) a3;
    _addr[4] = (octet) a4;
    _addr[5] = (octet) a5;
}

inline OutputStream &operator << (OutputStream &output,
				  EthernetAddress &address)
{
    HexDump *hd = new HexString(address, EthernetAddressLength,':');
    //output << HexString(address, EthernetAddressLength, ':');
    output << hd; 
    return output;
}

inline int EthernetAddress::isGroup() const
{
  return((_addr[0] & 0x01) && !isBroadcast() );
}

inline int EthernetAddress::isMulticast() const
{
  return(_addr[0] & 0x01);
}


inline void EthernetAddress::makeGroup()
{
  _addr[0] |= 0x01;
}


/*
 *	PARTIAL UNOFFICIAL LIST OF ETHERNET PROTOCOL TYPE CODES
 * (Taken from RFC 1060.)
 *
 * The following list is contributed unverified information from various
 * sources.
 *
 * Assignments:
 *
 *    Ethernet          Exp. Ethernet    Description          References
 *    -------------     -------------   -----------           ----------
 *    decimal  Hex      decimal  octal
 *       000   0000-05DC   -       -    IEEE802.3 Length Field   [XEROX]
 *       257   0101-01FF   -       -    Experimental             [XEROX]
 *       512   0200        512   1000   XEROX PUP (see 0A00)   [8,XEROX]
 *       513   0201        -      -     PUP Addr Trans (see 0A01)[XEROX]
 *      1536   0600       1536   3000   XEROX NS IDP         [133,XEROX]
 *      2048   0800        513   1001   DOD IP                 [105,JBP]
 *      2049   0801        -      -     X.75 Internet            [XEROX]
 *      2050   0802        -      -     NBS Internet             [XEROX]
 *      2051   0803        -      -     ECMA Internet            [XEROX]
 *      2052   0804        -      -     Chaosnet                 [XEROX]
 *      2053   0805        -      -     X.25 Level 3             [XEROX]
 *      2054   0806        -      -     ARP                     [88,JBP]
 *      2055   0807        -      -     XNS Compatability        [XEROX]
 *      2076   081C        -      -     Symbolics Private         [DCP1]
 *      2184   0888-088A   -      -     Xyplex                   [XEROX]
 *      2304   0900        -      -     Ungermann-Bass net debugr[XEROX]
 *      2560   0A00        -      -     Xerox IEEE802.3 PUP      [XEROX]
 *      2561   0A01        -      -     PUP Addr Trans           [XEROX]
 *      2989   0BAD        -      -     Banyan Systems           [XEROX]
 *      4096   1000        -      -     Berkeley Trailer nego    [XEROX]
 *      4097   1001-100F   -      -     Berkeley Trailer encap/IP[XEROX]
 *      5632   1600        -      -     Valid Systems            [XEROX]
 *     16962   4242        -      -     PCS Basic Block Protocol [XEROX]
 *     21000   5208        -      -     BBN Simnet               [XEROX]
 *     24576   6000        -      -     DEC Unassigned (Exp.)    [XEROX]
 *     24577   6001        -      -     DEC MOP Dump/Load        [XEROX]
 *     24578   6002        -      -     DEC MOP Remote Console   [XEROX]
 *     24579   6003        -      -     DEC DECNET Phase IV Route[XEROX]
 *     24580   6004        -      -     DEC LAT                  [XEROX]
 *     24581   6005        -      -     DEC Diagnostic Protocol  [XEROX]
 *     24582   6006        -      -     DEC Customer Protocol    [XEROX]
 *     24583   6007        -      -     DEC LAVC, SCA            [XEROX]
 *     24584   6008-6009   -      -     DEC Unassigned           [XEROX]
 *     24586   6010-6014   -      -     3Com Corporation         [XEROX]
 *     28672   7000        -      -     Ungermann-Bass download  [XEROX]
 *     28674   7002        -      -     Ungermann-Bass dia/loop  [XEROX]
 *     28704   7020-7029   -      -     LRT                      [XEROX]
 *     28720   7030        -      -     Proteon                  [XEROX]
 *     28724   7034        -      -     Cabletron                [XEROX]
 *     32771   8003        -      -     Cronus VLN            [131,DT15]
 *     32772   8004        -      -     Cronus Direct         [131,DT15]
 *     32773   8005        -      -     HP Probe                 [XEROX]
 *     32774   8006        -      -     Nestar                   [XEROX]
 *     32776   8008        -      -     AT&T                     [XEROX]
 *     32784   8010        -      -     Excelan                  [XEROX]
 *     32787   8013        -      -     SGI diagnostics            [AXC]
 *     32788   8014        -      -     SGI network games          [AXC]
 *     32789   8015        -      -     SGI reserved               [AXC]
 *     32780   8016        -      -     SGI bounce server          [AXC]
 *     32783   8019        -      -     Apollo Computers         [XEROX]
 *     32815   802E        -      -     Tymshare                 [XEROX]
 *     32816   802F        -      -     Tigan, Inc.              [XEROX]
 *     32821   8035        -      -     Reverse ARP             [48,JXM]
 *     32822   8036        -      -     Aeonic Systems           [XEROX]
 *     32824   8038        -      -     DEC LANBridge            [XEROX]
 *     32825   8039-803C   -      -     DEC Unassigned           [XEROX]
 *     32829   803D        -      -     DEC Ethernet Encryption  [XEROX]
 *     32830   803E        -      -     DEC Unassigned           [XEROX]
 *     32831   803F        -      -     DEC LAN Traffic Monitor  [XEROX]
 *     32832   8040-8042   -      -     DEC Unassigned           [XEROX]
 *     32836   8044        -      -     Planning Research Corp.  [XEROX]
 *     32838   8046        -      -     AT&T                     [XEROX]
 *     32839   8047        -      -     AT&T                     [XEROX]
 *     32841   8049        -      -     ExperData                [XEROX]
 *     32859   805B        -      -     Stanford V Kernel exp.   [XEROX]
 *     32860   805C        -      -     Stanford V Kernel prod.  [XEROX]
 *     32861   805D        -      -     Evans & Sutherland       [XEROX]
 *     32864   8060        -      -     Little Machines          [XEROX]
 *     32866   8062        -      -     Counterpoint Computers   [XEROX]
 *     32869   8065-8066   -      -     Univ. of Mass. @ Amherst [XEROX]
 *     32871   8067        -      -     Veeco Integrated Auto.   [XEROX]
 *     32872   8068        -      -     General Dynamics         [XEROX]
 *     32873   8069        -      -     AT&T                     [XEROX]
 *     32874   806A        -      -     Autophon                 [XEROX]
 *     32876   806C        -      -     ComDesign                [XEROX]
 *     32877   806D        -      -     Computgraphic Corp.      [XEROX]
 *     32878   806E-8077   -      -     Landmark Graphics Corp.  [XEROX]
 *     32890   807A        -      -     Matra                    [XEROX]
 *     32891   807B        -      -     Dansk Data Elektronik    [XEROX]
 *     32892   807C        -      -     Merit Internodal           [HWB]
 *     32893   807D-807F   -      -     Vitalink Communications  [XEROX]
 *     32896   8080        -      -     Vitalink TransLAN III    [XEROX]
 *     32897   8081-8083   -      -     Counterpoint Computers   [XEROX]
 *     32923   809B        -      -     Appletalk                [XEROX]
 *     32924   809C-809E   -      -     Datability               [XEROX]
 *     32927   809F        -      -     Spider Systems Ltd.      [XEROX]
 *     32931   80A3        -      -     Nixdorf Computers        [XEROX]
 *     32932   80A4-80B3   -      -     Siemens Gammasonics Inc. [XEROX]
 *     32960   80C0-80C3   -      -     DCA Data Exchange Cluster[XEROX]
 *     32966   80C6        -      -     Pacer Software           [XEROX]
 *     32967   80C7        -      -     Applitek Corporation     [XEROX]
 *     32968   80C8-80CC   -      -     Intergraph Corporation   [XEROX]
 *     32973   80CD-80CE   -      -     Harris Corporation       [XEROX]
 *     32974   80CF-80D2   -      -     Taylor Instrument        [XEROX]
 *     32979   80D3-80D4   -      -     Rosemount Corporation    [XEROX]
 *     32981   80D5        -      -     IBM SNA Service on Ether [XEROX]
 *     32989   80DD        -      -     Varian Associates        [XEROX]
 *     32990   80DE-80DF   -      -     Integrated Solutions TRFS[XEROX]
 *     32992   80E0-80E3   -      -     Allen-Bradley            [XEROX]
 *     32996   80E4-80F0   -      -     Datability               [XEROX]
 *     33010   80F2        -      -     Retix                    [XEROX]
 *     33011   80F3        -      -     AppleTalk AARP (Kinetics)[XEROX]
 *     33012   80F4-80F5   -      -     Kinetics                 [XEROX]
 *     33015   80F7        -      -     Apollo Computer          [XEROX]
 *     33023   80FF-8103   -      -     Wellfleet Communications [XEROX]
 *     33031   8107-8109   -      -     Symbolics Private        [XEROX]
 *     33072   8130        -      -     Waterloo Microsystems    [XEROX]
 *     33073   8131        -      -     VG Laboratory Systems    [XEROX]
 *     33079   8137-8138   -      -     Novell, Inc.             [XEROX]
 *     33081   8139-813D   -      -     KTI                      [XEROX]
 *     33100   814C        -      -     SNMP                      [JKR1]
 *     36864   9000        -      -     Loopback                 [XEROX]
 *     36865   9001        -      -     3Com(Bridge) XNS Sys Mgmt[XEROX]
 *     36866   9002        -      -     3Com(Bridge) TCP-IP Sys  [XEROX]
 *     36867   9003        -      -     3Com(Bridge) loop detect [XEROX]
 *     65280   FF00        -      -     BBN VITAL-LanBridge cache[XEROX]
 *
 */


#endif /* Ethernet_h */
