/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801-2910
 *
 * Copyright (c) 1991 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Amit Parghi (parghi@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *	EthernetAdministrator.h
 *
 *	Declaration of the EthernetAdministrator class.
 *
 * $Id: EthernetAdministrator.h 2 2004-06-03 01:58:28Z jcarlyle $
 *
 * $Log: EthernetAdministrator.h,v $
 * Revision 1.1  2004/06/03 01:58:47  jcarlyle
 * Initial revision
 *
 * Revision 1.2  1991/12/01  06:15:12  parghi
 * Misc. changes.  Last revision before the Great Networking Rewrite.
 *
 * Revision 1.1  91/05/06  16:04:13  parghi
 * Initial revision
 *
 */


#ifndef EthernetAdministrator_h
#define EthernetAdministrator_h


#ifdef __GNUC__
#pragma interface
#endif


#include "NetworkAdministrator.h"
#include "Ethernet.h"

class EthernetDMUXTable;
class EthernetDriver;
class NetworkWalker;
class NetworkBufferFreeList;


extern ClassStar EthernetAdministratorClass;
class EthernetAdministrator : public NetworkAdministrator {
private:
    EthernetAddress _address;		// local interface's Ethernet address
    EthernetDMUXTable *_table;		// table of {packet-type,walker} pairs

				// pull the protocol type code from a packet
    EthernetProtocolID getProtocolID(octet *);

    void transmitDaemon();		// used for transmission tests

protected:
    virtual NetworkWalker *findWalker(NetworkBuffer *);

public:
    EthernetAdministrator(char *name,
			  unsigned char debugMask = 0);
    virtual ~EthernetAdministrator();

    virtual void ready(NetworkDriver *);

    int registerType(EthernetProtocolID,	// add {Ethertype, Walker}
		     NetworkWalker *);		//	pair to Admin's table
    int unregisterType(EthernetProtocolID);	// remove pair from table

    EthernetAddress *address();			// get local Ethernet address

    Class *classOf();
};


#endif /* EthernetAdministrator_h */
