/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801-2910
 *
 * Copyright (c) 1991 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Amit Parghi (parghi@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *	EthernetDMUX.h
 *
 *	Definition of the EthernetDMUX classes:  for the tables which map
 *	Ethernet type codes to Walker classes in the EthernetAdministrator.
 *
 *	$Id: EthernetDMUX.h 2 2004-06-03 01:58:28Z jcarlyle $
 *
 */


#ifndef EthernetDMUX_h
#define EthernetDMUX_h


#ifdef __GNUC__
#pragma interface
#endif


#include "Link.h"
#include "List.h"
#include "SemaphoreStar.h"

#include "Ethernet.h"
#include "Object.h"

class NetworkWalker;
class NetworkBufferFreeList;



    //	The EthernetDMUX... classes are used within the EthernetAdministrator
    //	to keep track of the EthernetChannels corresponding to the various
    //	Ethernet packet types, e.g. AppleTalk, IP, ARP, DVM, etc.
    //
class EthernetDMUXData : public Link {	// each entry is really a linked list
    friend class EthernetDMUXTable;
private:
    EthernetProtocolID _type;		// Ethernet packet type code
    NetworkWalker *_walker;		// pointer to appropriate Queue/Channel
public:
    EthernetDMUXData();
    EthernetDMUXData(EthernetProtocolID, NetworkWalker *);
};


    //	EthernetDMUXTable holds the individual EthernetDMUXData entries.
    //	This class is arranged as an array (ordered on a hash of the
    //	EthernetProtocolID) of EthernetDMUXData items.

    //	NOTE that this value must be changed appropriately if the hashing
    //	technique in hash() is changed!
const unsigned int EthernetDMUXTableSize = 255 + 255 + 1;

class EthernetDMUXTable : public Object {  // holds {type code, Walker*} pairs

private:
    SemaphoreStar _semaphore;				// mutex lock the table
    List *_array;

    EthernetDMUXData * getEntry(EthernetProtocolID);
    unsigned short hash(EthernetProtocolID);		// used for ordering

public:
    EthernetDMUXTable(unsigned char debugMask = 0);	// Raid debug option
    virtual ~EthernetDMUXTable();

    NetworkWalker *findWalker(EthernetProtocolID);	// find walker 

    int add(EthernetProtocolID, NetworkWalker *);	// add to table
    void remove(EthernetProtocolID);			// delete from table
};


    inline unsigned short
EthernetDMUXTable::hash(EthernetProtocolID t)
{
    unsigned short returnValue;

    returnValue = ((t >> 8) & 0xff) + (t & 0xff);	// add the two bytes;

    return returnValue;
}

#endif /* EthernetDMUX_h */
