/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801-2910
 *
 * Copyright (c) 1991 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Amit Parghi (parghi@cs.uiuc.edu)
 * Contributing Author: Nayeem Islam (nislam@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *	EthernetDriver.h
 *
 *	Definitions for the machine-independent, Ethernet-specific
 *	portions of a Driver.
 *
 * $Id: EthernetDriver.h 2 2004-06-03 01:58:28Z jcarlyle $
 *
 * $Log: EthernetDriver.h,v $
 * Revision 1.1  2004/06/03 01:58:47  jcarlyle
 * Initial revision
 *
 * Revision 1.2  1991/12/01  06:15:12  parghi
 * Misc. changes.  Last revision before the Great Networking Rewrite.
 *
 * Revision 1.1  91/05/06  16:06:11  parghi
 * Initial revision
 *
 */


#ifndef EthernetDriver_h
#define EthernetDriver_h


#ifdef __GNUC__
#pragma interface
#endif



#include "NetworkDriver.h"
#include "Ethernet.h"

class NetworkBuffer;
class NetworkBufferFreeList;
class HashTable;
class Lock;

extern ClassStar EthernetDriverClass;
class EthernetDriver : public NetworkDriver {
private:
    HashTable	*_multicastAddresses;
    Lock	*_lock;
    NetworkBufferFreeList* flist;

protected:
    EthernetAddress * _ethernetAddress;
    EthernetDriver(unsigned char debugMask,NetworkBufferFreeList* lf =0);
    virtual ~EthernetDriver();

    virtual int registerMulticast(EthernetAddress *);
    virtual int unregisterMulticast(EthernetAddress *);
    virtual int isValidAddress(EthernetAddress *);
    int		hashAddressTo64(EthernetAddress *);

public:
    virtual int transmitPacket(NetworkBuffer *) = 0; // subclassResponsibility
    virtual int transmitPacketAsync(NetworkBuffer *) = 0;
    virtual int	addMulticast(EthernetAddress *) = 0;
    virtual int removeMulticast(EthernetAddress *) = 0;

    EthernetAddress * getEthernetAddress();
    virtual void printStats(){}
    virtual void clearStats(){}

    Class * classOf();
};



#endif /* ! EthernetDriver_h */
