/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Lee Lup Yuen (lup@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * ApplicationConduitOutputStream.h
 */
 
#ifndef ApplicationConduitOutputStream_h
#define ApplicationConduitOutputStream_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Assert.h"
#include "OutputStream.h"

class ApplicationConduitStream;
extern ClassStar ApplicationConduitOutputStreamClass;

class ApplicationConduitOutputStream : public OutputStream
{
    /* An ApplicationConduitOutputStream provides an OutputStream interface to
     * an ApplicationConduitStream.
     */
public:
    proxiable ApplicationConduitOutputStream (ApplicationConduitStream
    	*conduit);
    ~ApplicationConduitOutputStream ();
    //  Not really proxiable, but proxify++ complains about write being an
    //  abstract virtual function.
    proxiable int write (const char *buffer, int count);
    OutputStream &flush ();
    void acquireExclusiveUse () { Assert (0); }
    void releaseExclusiveUse () { Assert (0); }
    Class *classOf ();
private:
    ApplicationConduitStream *conduitStream;
public:
    //  This must be put here to prevent proxify++ from complaining about
    //  conduitStream.
    ApplicationConduitStream *getConduitStream () { return conduitStream; }
};

#endif
