/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Lee Lup Yuen (lup@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * UserSession.h
 */
 
#ifndef UserSession_h
#define UserSession_h


#ifdef __GNUC__
#pragma interface
#endif

#include "ProxiableObject.h"

class ApplicationConduitStream;
class ClassStar;
extern ClassStar UserSessionClass;

class UserSession : public ProxiableObject
{
    /* A UserSession maintains information about a single remote user of
     * the networking services.  Operations are provided to write messages
     * to that user and to spy on that user.
     */
public:
    UserSession (int sessionType, ApplicationConduitStream *sessionStream,
    	unsigned char *remoteAddress);
    ~UserSession ();
    void setUserName (const char *userName);
    char *getUserName ();
    int getSessionType ();
    unsigned long getCreationTime ();
    unsigned char *getRemoteAddress ();
    ApplicationConduitStream *getSessionStream ();
    void write (const char *buffer, int count);
    void spyOn (ApplicationConduitStream *spyStream);
    void spyOff ();
    Class *classOf ();
private:
    unsigned char remoteAddress [4];  //  IP address of user.
    unsigned long creationTime;       //  Time of creation of this session.
    char *userName;                   //  Name of user.
    int sessionType;                  //  Type of session. See UserSessionInfo.
    ApplicationConduitStream *sessionStream;  //  Conduit used by this session.

    //  These fields should be used by the UserSessionManager only.
    friend class UserSessionManager;
    int sessionID;
    int status;
    UserSession *nextUserSession;
    
    /* Method setUserName: Set user name for this session.  If user name
         has already been set, do nothing. */
    /* Method getUserName: Return the user name of this session. */
    /* Method getSessionType: Return the session type. */
    /* Method getCreationTime: Return the time at which this session was
         created.  Units: seconds since Jan 1 1970, GMT 0. */
    /* Method getRemoteAddress: Return IP address of this session. */
    /* Method getSessionStream: Get the conduit stream used by this session. */
    /* Method write: Write count bytes from buffer into this session's
         conduit stream. */
    /* Method spyOn: Copy all future output to this session's conduit stream
         to spyStream. */
    /* Method spyOff: Stop copying output to this session's conduit stream
         to another stream. */
};

#endif
