/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Lee Lup Yuen (lup@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * UserSessionManager.h
 */
 
#ifndef UserSessionManager_h
#define UserSessionManager_h


#ifdef __GNUC__
#pragma interface
#endif

#include "ProxiableObject.h"

class UserSession;
class ClassStar;
struct UserSessionInfo;
extern ClassStar UserSessionManagerClass;

class UserSessionManager : public ProxiableObject
{
    /* A UserSessionManager provides an application-level interface for
     * querying user sessions and for writing to or spying on the sessions.
     * There should only be a single UserSessionManager in the system,
     * accessed as "userSessionManager" from the StandardNameServer.
     */
public:
    UserSessionManager ();
    ~UserSessionManager ();
    void add (UserSession *userSession);
    void remove (UserSession *userSession);
    proxiable virtual unsigned long getCreationTime ();
    proxiable virtual int getMySession ();
    proxiable virtual int getSessions (int *sessionID, int count);
    proxiable virtual int getSessionInfo (int sessionID,
        UserSessionInfo *info);
    proxiable virtual void writeSession (int sessionID, const char *buffer,
    	int count);
    proxiable virtual void spyOnSession (int sessionID);
    proxiable virtual void spyOffSession (int sessionID);
    proxiable virtual void setSessionUserName (const char *name);
    Class *classOf ();
private:
    unsigned long creationTime;  //  Time of creation of the session manager.
    int nextSessionID;           //  Next ID to be assigned to a new session.
    UserSession *firstUserSession; //  List of user sessions.
    UserSession *findUserSession (int sessionID); // Locate user session.
    
    /* Method add: Add a user session. */
    /* Method remove: Remove a user session. */
    /* Method getCreationTime: Return time of creation of the session
         manager. */
    /* Method getMySession: Return the session ID of the caller. */
    /* Method getSessions: Store all session ID's into array sessionID.
         Restricted to count entries. */
    /* Method getSessionInfo: Write into info the information about
         the session with the specified session ID.  Return -1 if session ID
	 is invalid. */
    /* Method writeSession: Write count bytes from buffer to the indicated
         session. */
    /* Method spyOnSession: Copy all future output from the indicated session
         to the session of the caller. */
    /* Method spyOffSession: Stop copying all output from the indicated session
         to the session of the caller. */
    /* Method setSessionUserName: Set the user name of the session of the
         caller.  Works only if the user name has not already been set. */
};

#endif
