/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ObjectProxy.h
 */

#ifndef ObjectProxy_h
#define ObjectProxy_h


#ifdef __GNUC__
#pragma interface
#endif

/*
 * IMPORTANT NOTICE: if any virtual functions are added to or moved around
 *   in Object or ProxiableObject, corresponding changes need to be made to
 *   class ObjectProxy and to every Processor-dependent ObjectProxyStubs.s,
 *   and changes may need to be made to the ObjectProxyCatchTable in
 *   ObjectProxy.h.
 */

#include "Object.h"
#include "ProxiableObject.h"
#include "ReferenceCount.h"
#include "Types.h"


class ObjectProxyManager;
class ProxyTableCopy;
class Domain;
class Process;
class ObjectProxy;

/*
 * structure used to pass information between a proxy call and a proxy return
 */
const int MAXPARAMPROXIES = 5;
class ProxyReturnFlags {
public:
	unsigned char returnType;
	unsigned char callType;
	unsigned char numParamProxies;
	PFV calledMethod;
	Class * returnClass;
	Process * thisProc;
	Domain * callingDomain;
	Domain * calledDomain;
	Domain * previousDomain;
	ProxiableObject ** savedState;
	ObjectProxy * paramProxies[ MAXPARAMPROXIES ];
};

/*
 * These make up the hash key for the proxy
 */
class ObjectProxyKey {
public:
	ProxiableObject * _proxiedObject;/* pointer to proxied object */
	Domain * _proxiedDomain;	/* domain proxied object is in */
	ProxyTableCopy *_table;		/* table for proxied object */
};


class ObjectProxy : public Object {
	friend class ObjectProxyManager;
	friend class ProxyPoolManager;
protected:
	ReferenceCount	_referenceCount; /* reference count for the proxy */
					 /* this must be first to correspond */
					 /*   with the referenceCount in */
					 /*   ProxiableObject */

	ObjectProxyKey key;

	ObjectProxy *prev, *next;	/* list pointers */

	char notReferenced;		/* non-zero if not referenced */

	/*
	 * Need as many fake functions here as any Proxiable class will ever
	 *   have (virtual) functions.   This is defined in MAXMETHODS in
	 *   ProxyTable.h.
	 */
	/*
	 * NOTE: isProxy() must be first, because CFRONT generates the vtable
	 *  for the first virtual function in a class (and Object specifies
	 *  that these first functions are virtual).   It still is the
	 *  second function in the vtable (after the destructor), but it
	 *  needs to be listed first in this class.
	 */
	/* from Object.h */
	virtual int isProxy();
	virtual ~ObjectProxy();
	virtual unsigned int identity();
	virtual int isKindOf( Class * );
	virtual int isMemberOf( Class * );
	virtual Class * classOf();
	virtual int debugMask();
	virtual void setDebugMask( unsigned mask );
	virtual void setDebugMessages( unsigned messages );
	virtual void writeName( OutputStream * );
	virtual void inspect( OutputStream * );

	/* from ProxiableObject.h */
	virtual void noRemainingReferences();
	virtual void reference();
	virtual void unreference();
	/* don't care about the rest from ProxiableObject.h; 		*/
	/*  only care through the ones that need to be caught specially */

	virtual int f14( int, int, int, int, int );
	virtual int f15( int, int, int, int, int );
	virtual int f16( int, int, int, int, int );
	virtual int f17( int, int, int, int, int );
	virtual int f18( int, int, int, int, int );
	virtual int f19( int, int, int, int, int );
	virtual int f20( int, int, int, int, int );
	virtual int f21( int, int, int, int, int );
	virtual int f22( int, int, int, int, int );
	virtual int f23( int, int, int, int, int );
	virtual int f24( int, int, int, int, int );
	virtual int f25( int, int, int, int, int );
	virtual int f26( int, int, int, int, int );
	virtual int f27( int, int, int, int, int );
	virtual int f28( int, int, int, int, int );
	virtual int f29( int, int, int, int, int );
	virtual int f30( int, int, int, int, int );
	virtual int f31( int, int, int, int, int );
	virtual int f32( int, int, int, int, int );
	virtual int f33( int, int, int, int, int );
	virtual int f34( int, int, int, int, int );
	virtual int f35( int, int, int, int, int );
	virtual int f36( int, int, int, int, int );
	virtual int f37( int, int, int, int, int );
	virtual int f38( int, int, int, int, int );
	virtual int f39( int, int, int, int, int );
	virtual int f40( int, int, int, int, int );
	virtual int f41( int, int, int, int, int );
	virtual int f42( int, int, int, int, int );
	virtual int f43( int, int, int, int, int );
	virtual int f44( int, int, int, int, int );
	virtual int f45( int, int, int, int, int );
	virtual int f46( int, int, int, int, int );
	virtual int f47( int, int, int, int, int );
	virtual int f48( int, int, int, int, int );
	virtual int f49( int, int, int, int, int );
	virtual int f50( int, int, int, int, int );
	virtual int f51( int, int, int, int, int );
	virtual int f52( int, int, int, int, int );
	virtual int f53( int, int, int, int, int );
	virtual int f54( int, int, int, int, int );
	virtual int f55( int, int, int, int, int );
	virtual int f56( int, int, int, int, int );
	virtual int f57( int, int, int, int, int );
	virtual int f58( int, int, int, int, int );
	virtual int f59( int, int, int, int, int );
	virtual int f60( int, int, int, int, int );
	virtual int f61( int, int, int, int, int );
	virtual int f62( int, int, int, int, int );
	virtual int f63( int, int, int, int, int );
	virtual int f64( int, int, int, int, int );
	virtual int f65( int, int, int, int, int );
	virtual int f66( int, int, int, int, int );
	virtual int f67( int, int, int, int, int );
	virtual int f68( int, int, int, int, int );
	virtual int f69( int, int, int, int, int );
	virtual int f70( int, int, int, int, int );
	virtual int f71( int, int, int, int, int );
	virtual int f72( int, int, int, int, int );
	virtual int f73( int, int, int, int, int );
	virtual int f74( int, int, int, int, int );
	virtual int f75( int, int, int, int, int );
	virtual int f76( int, int, int, int, int );
	virtual int f77( int, int, int, int, int );
	virtual int f78( int, int, int, int, int );
	virtual int f79( int, int, int, int, int );
	virtual int f80( int, int, int, int, int );
	virtual int f81( int, int, int, int, int );
	virtual int f82( int, int, int, int, int );
	virtual int f83( int, int, int, int, int );
	virtual int f84( int, int, int, int, int );
	virtual int f85( int, int, int, int, int );
	virtual int f86( int, int, int, int, int );
	virtual int f87( int, int, int, int, int );
	virtual int f88( int, int, int, int, int );
	virtual int f89( int, int, int, int, int );
	virtual int f90( int, int, int, int, int );
	virtual int f91( int, int, int, int, int );
	virtual int f92( int, int, int, int, int );
	virtual int f93( int, int, int, int, int );
	virtual int f94( int, int, int, int, int );
	virtual int f95( int, int, int, int, int );
	virtual int f96( int, int, int, int, int );
	virtual int f97( int, int, int, int, int );
	virtual int f98( int, int, int, int, int );
	virtual int f99( int, int, int, int, int );

public:
	ObjectProxy();

	/*
	 * None of the rest of these can be virtual!
	 * Non-inline functions are defined in ObjectProxyKernel.cc.
	 */
	ProxiableObject * proxiedObject();
	void setProxiedObject( ProxiableObject *, Class *, Domain *, int );
	Domain * proxiedDomain();
	ProxyTableCopy * table();

	/*
	 * The following are used to check of a proxy call references
	 *    other proxies that are passed in as parameters, and to free
	 *    the parameter-proxies upon return from the proxy call if they
	 *    have not been referenced.
	 */
	int setNotReferenced();
	void freeIfNotReferenced( ObjectProxyManager * );

	/*
	 * The following methods are used to treat some method calls specially
	 *  so as to not go directly through to the methods on the real object.
	 */

	int catchDestructor();
	int catchNoRemainingReferences();
	int catchReference();
	int catchUnreference();
	/*
	 * The 'noCatch' method is put in the ObjectProxyCatchTable when
	 *   a method is not to be caught.  Zero would be used instead
	 *   but it causes CFRONT to generate bad code.
	 */
	int noCatch();
};

typedef int (ObjectProxy::*CATCHTYPE)();

const int UNREFERENCE_METHOD_NUMBER = 13;

const CATCHTYPE ObjectProxyCatchTable[] = {
	&ObjectProxy::catchDestructor,
	&ObjectProxy::noCatch,
	&ObjectProxy::noCatch,
	&ObjectProxy::noCatch,
	&ObjectProxy::noCatch,
	&ObjectProxy::noCatch,
	&ObjectProxy::noCatch,
	&ObjectProxy::noCatch,
	&ObjectProxy::noCatch,
	&ObjectProxy::noCatch,
	&ObjectProxy::noCatch,
	&ObjectProxy::catchNoRemainingReferences,
	&ObjectProxy::catchReference,
	&ObjectProxy::catchUnreference,
	&ObjectProxy::noCatch
};

const int SizeOfObjectProxyCatchTable = 
	(sizeof( ObjectProxyCatchTable ) / sizeof( CATCHTYPE ));


inline ProxiableObject *
ObjectProxy::proxiedObject()
{
	return( key._proxiedObject );
}

inline Domain *
ObjectProxy::proxiedDomain()
{
	return( key._proxiedDomain );
}

inline ProxyTableCopy *
ObjectProxy::table()
{
	return( key._table );
}

inline int
ObjectProxy::setNotReferenced()
{
	if ( _referenceCount.value() == 0 ) {
		notReferenced = 1;
		return( 1 );
	}
	else
		return( 0 );
}

#endif /* ObjectProxy_h */
