/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ObjectProxy.h
 */

#ifndef ObjectProxyManager_h
#define ObjectProxyManager_h


#ifdef __GNUC__
#pragma interface
#endif

#include "ProxiableObject.h"
#include "ObjectProxy.h"
#include "MemoryObjectStar.h"
#include "Semaphore.h"
class Domain;
class HashTable;

extern ClassStar ObjectProxyManagerClass;
class ObjectProxyManager : public ProxiableObject {
protected:
	MutexSemaphore mutex;

	HashTable * hash;

	ObjectProxy *domainProxyMemory;
	ObjectProxy *domainProxyMemoryLast;
	ObjectProxy *kernelProxyMemory;
	ObjectProxy *kernelProxyMemoryLast;
	unsigned int kernelToDomainProxyMemoryOffset;

	ObjectProxy *freeList;

	ObjectProxy *firstObjectProxy;
	ObjectProxy *nextObjectProxy;
	ObjectProxy *lastObjectProxy;

	unsigned int sizeOfAllocatedProxies;

	MemoryObjectStar proxyMemoryObject;

	ObjectProxy * usedList;
	int proxiesAllocated;

	ProxiableObject * firstToFree;

	~ObjectProxyManager();

public:
	ObjectProxyManager( int maxProxies, ObjectProxy * proxyMemory,
		MemoryObject * proxyMO, ProxiableObject * freeFirst );
	
	void clone( ObjectProxyManager * opm );
	int isValidProxy( ObjectProxy * );

	ObjectProxy * domainToKernel( ObjectProxy * );
	ObjectProxy * kernelToDomain( ObjectProxy * );

	Class * classOf();

	ObjectProxy * allocate( ProxiableObject *, Class *, Domain *, int );
	void free( ObjectProxy * );

	void freeAll();

	//  Increment the reference counts of the ObjectProxies.
	void incrementAllReferenceCounts ();

};

#endif /* ObjectProxyManager_h */

