/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Bjorn A. Helgaas (helgaas@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * PageTableConstants.h: include machine-dependent constants and define a few functions
 */

#ifndef PageTableConstants_h
#define PageTableConstants_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Assert.h"
#include "PAGE_TABLE_CONSTANTS.h"

/*
 * Some miscellaneous functions used by TwoLevelPageTables and
 * possibly others.
 */

inline unsigned int
AddressToPage( const char * address )
{
	return( (unsigned int) address >> Log2PhysicalPageSize );
}

inline unsigned int
PageAligned( const char * address )
{
	return( ((unsigned int) address % PhysicalPageSize) == 0 );
}


inline unsigned int
FloorPage( unsigned int address )
{
	return( address & (~ ( PhysicalPageSize - 1 )) );
}

inline const char *
FloorPage( const char * address )
{
	return( (const char *) FloorPage( (unsigned int) address ) );
}


inline unsigned int
CeilingPage( unsigned int address )
{
	return( FloorPage( address + ( PhysicalPageSize - 1 ) ) );
}

inline const char *
CeilingPage( const char * address )
{
	return( (const char *) CeilingPage( (unsigned int) address ) );
}

#endif	/* PageTableConstants_h */
