/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Contributing Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Contributing Author: David Dykstra (dykstra@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * PeriodicTimer.h - global Periodic timer.
 */

#ifndef PeriodicTimer_h
#define PeriodicTimer_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Timer.h"	/* parent class */


class Lock;

#include "ClassStar.h"
#include "SemaphoreStar.h"

extern ClassStar PeriodicTimerClass;
class PeriodicTimer : public Timer {
protected:
	/*
	 * This is the per-instance data
	 */
	SemaphoreStar periodicSemaphore; /* semaphore to V when expires	*/
	char running;			/* timer running flag		*/
	int maxCount;			/* number of ticks in period	*/
	int count;
	PeriodicTimer *nextDifferent;	/* single link for different counts */
	PeriodicTimer *nextSame;	/* single link for same counts	*/

	/*
	 * Now for the class data and functions
	 */
	static Lock *lock;		/* lock to protect lists */
	static PeriodicTimer *first;	/* pointer to first timer */

public:
	static void initialize();

	static int perTickBookKeeping();
	static void expiryProcessing();

	/* 
	 * Now back to regular public functions
	 */

	proxiable PeriodicTimer();
	proxiable ~PeriodicTimer();

	proxiable unsigned int start( unsigned int microseconds );
	proxiable unsigned int stop();
	proxiable unsigned int residual();
	proxiable void await();

	Class *classOf();
};


#endif /* PeriodicTimer_h */
