/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * Pipe.h
 */

#ifndef Pipe_h
#define Pipe_h


#ifdef __GNUC__
#pragma interface
#endif

#include "ProxiableObject.h"
#include "Semaphore.h"

extern ClassStar PipeClass;

class Pipe : public ProxiableObject {
      protected:
	int _size;
	int _count;
	int _head;
	int _tail;
	int _readPending;
	int _writePending;
	int _readQueueLength;
	int _writeQueueLength;
	int _readBlocked;
	int _writeBlocked;
	Lock _lock;
	char * _buffer;
	Semaphore _copyOutSem;
	Semaphore _copyInSem;
	Semaphore _readQueue;
	Semaphore _writeQueue;

	int used() { return _count; }
	int free() { return _size - _count; }
	void insertIntoBuffer( char * buf, int count );
	void removeFromBuffer( char * buf, int count );
      public:
	Class * classOf();
	proxiable Pipe( int size = 8192 );
	proxiable ~Pipe();
	proxiable virtual int read( char * buf, int count, int block = 1 );
	proxiable virtual int write( char * buf, int count, int block = 1 );
};

#endif 
















