/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ProcessContainer.h: description of the class to hold Processes.
 */

#ifndef ProcessContainer_h
#define ProcessContainer_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Object.h"
#include "Lock.h"

class Process;


extern ClassStar ProcessContainerClass;
class ProcessContainer : public Object {
public:
	virtual void add( Process * ) = 0;
	virtual Process * remove() = 0;
	virtual int isEmpty() = 0;
	Class * classOf();
};


/*
 * Kind of like the /dev/null of ProcessContainers. All adds just throw
 * the process on the floor and all removes return empty.
 */
extern ClassStar NullProcessContainerClass;
class NullProcessContainer : public ProcessContainer {
public:
	void add( Process * );
	Process * remove();
	int isEmpty();
	Class * classOf();
};

/*
 * SingleProcessHolder - a non-locked scheduler that holds a single Process.
 */
extern ClassStar SingleProcessHolderClass;
class SingleProcessHolder : public ProcessContainer {
protected:
	Process * theProcess;
public:
	SingleProcessHolder();
	~SingleProcessHolder();
	Class * classOf();
	void add( Process * process );
	Process * remove();
	int isEmpty();
};

/*
 * a locked version of the above.
 */
class LockedSingleProcessHolder : public SingleProcessHolder {
protected:
	Lock lock;
public:
	LockedSingleProcessHolder();
	~LockedSingleProcessHolder();
	void add( Process * process );
	Process * remove();
};


#endif /* ProcessContainer_h */
