/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Author: David Raila (raila@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ProcessorContext.h: Parent class for all machine dependent process context
 *           information classes.
 */

#ifndef ProcessorContext_h
#define ProcessorContext_h


#ifdef __GNUC__
#pragma interface
#endif

#include <setjmp.h>
#include "MemoryObjectStar.h"
#include "PhysicalMemoryChain.h"
#include "Object.h"     /* parent class */
#include "Types.h"

class CPU;
class Process;
class ProxiableObject;

class InterruptContext {};

extern ClassStar ProcessorContextClass;
class ProcessorContext : public Object {
    friend class Process;
    friend class ApplicationProcess;
    friend class TimeSliceInterrupt;
    friend class CPU;
    Process * _resuming; // To track the restore
    int _checkPointed;
 protected:

    virtual Process * basicRestore( Process * process) = 0;

    Process * setResuming(Process *p) { return (_resuming = p);}
    Process * getResuming() { return _resuming;}


    Process * const owner;  // temporary until the two are merged...

    int hasEverBeenDispatched;

    /*
     * Methods to restore and activate the context on the calling processor.
     */
    virtual volatile Process * checkpoint() = 0;
    void restore( Process * process);

    /*
     * Methods that the switching code uses to lock down the kernel
     * stacks.
     *
     * This stuff is in here for testing.  It doesn't really do
     * anything.  The VM code needs to be fixed first.  See
     * ProcessorContext.cc  -raila 5-21-92
     */
    PhysicalMemoryChain     _kernelChain;
    MemoryObjectStar    _kernelStackMO;
    char            * _kernelStack;
    unsigned        _kernelStackSize;

    /*
     * The machine-independent code to assist in context restoration.
     */
    void restoreAssist( Process * previousProcess );

 public:
    virtual void restoreFromInitialContext( Process * process) = 0;
    ProcessorContext( Process * owner );
    virtual ~ProcessorContext();
    Process * process();

    virtual char        * kernelStackBase(){return _kernelStack;}
    virtual unsigned    kernelStackSize(){return _kernelStackSize;}

    virtual void        createKernelStack();
    virtual void        cacheKernelStack();
    virtual void        releaseKernelStack();

    virtual void traceStack(){}

    Class * classOf();

};


inline Process *
ProcessorContext::process()
{
    return( owner );
}

#endif /* ProcessorContext_h */
