/*
 * This file is part of the Choices Operating System
 *
 * Copyright (c) 2006
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Francis David (fdavid@uiuc.edu)
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 */
/*
 * ARMException.h: event class description.
 */

#ifndef ARMException_h
#define ARMException_h


#ifdef __GNUC__
#pragma interface
#endif

#include "ProcessorException.h" /* parent class */
#include "ARMContext.h"

extern ClassStar ARMExceptionClass;
class ARMException : public ProcessorException {
  public:
    ARMException(ARMInterruptContext *context);
    ARMException(ARMInterruptContext *context, const char *msg);

    virtual ~ARMException();

    virtual void displayContext();
    virtual void displayStackTrace();
    
    ARMInterruptContext _context;
    ARMStackTrace _trace;

    Class * classOf();
};

class ARMUndefinedInstructionException: public ARMException {
 public:
    ARMUndefinedInstructionException(ARMInterruptContext *context)
    :ARMException(context,"ARM Undefined Instruction Exception") {}
    ARMUndefinedInstructionException(ARMInterruptContext *context, const char *msg)
    :ARMException(context,msg) {}
};

class ARMInstructionAccessException: public ARMException {
 public:
    ARMInstructionAccessException(ARMInterruptContext *context, ARMStackTrace *trace)
    :ARMException(context,"ARM Instruction Access Exception") {}
    ARMInstructionAccessException(ARMInterruptContext *context, const char *msg) 
    :ARMException(context,msg) {}
};

class ARMDataAccessException: public ARMException {
 public:
    ARMDataAccessException(ARMInterruptContext *context)
    :ARMException(context,"ARM Data Access Exception") {}
    ARMDataAccessException(ARMInterruptContext *context, const char *msg) 
    :ARMException(context,msg) {}
};

class ARMWatchdogTimeoutException: public ARMException {
 public:
    ARMWatchdogTimeoutException(ARMInterruptContext *context)
    :ARMException(context,"ARM Watchdog Timeout Exception") {}
    ARMWatchdogTimeoutException(ARMInterruptContext *context, const char *msg)
    :ARMException(context,msg) {}
};

#endif /* ARMException_h */
