/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 WSpringfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ARMInterrupts.h: hardware interrupt sub-class descriptions for the
 *	ARM processor.
 */

#ifndef ARMInterrupts_h
#define ARMInterrupts_h

#ifdef __GNUC__
#pragma interface
#endif

#include "Exception.h"
#include "Interrupt.h"		/* parent class */


#define FSR_VECTOR                    0x0
#define FSR_ALIGNMENT1                0x1
#define FSR_ALIGNMENT2                0x3
#define FSR_EXT_ABORT_TRANSLATION_L1  0xC
#define FSR_EXT_ABORT_TRANSLATION_L2  0xE
#define FSR_TRANSLATION_SECTION       0x5
#define FSR_TRANSLATION_PAGE          0x7
#define FSR_DOMAIN_SECTION            0x9
#define FSR_DOMAIN_PAGE               0xB
#define FSR_PERMISSION_SECTION        0xD
#define FSR_PERMISSION_PAGE           0xF
#define EXT_ABORT_LINEFETCH_SECTION   0x4
#define EXT_ABORT_LINEFETCH_PAGE      0x6
#define EXT_ABORT_N_LINEFETCH_SECTION 0x8
#define EXT_ABORT_N_LINEFETCH_PAGE    0xA

/*
 * On the ARM the level is passed in info so the handler can reinstate
 * interrupts if it desires.
 */

class ARMDataAccessInterrupt : public Interrupt {
protected:
	void basicRaise( char * info );
public:
};

class ARMInstructionAccessInterrupt : public Interrupt {
protected:
	void basicRaise( char * info );
public:
};

class ARMUndefinedInstructionInterrupt : public Interrupt {
protected:
	void basicRaise( char * info );
public:
};

class ARMDivideByZeroInterrupt : public Interrupt {
protected:
	void basicRaise( char * info );
};

class ARMProxyInterrupt : public Interrupt {
protected:
	void basicRaise( char * info );
};

void ARMWatchdogTimeoutInterrupt(ARMInterruptContext *context);

#endif /* ARMInterrupts_h */

