/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David K. Raila (raila@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
#ifndef ARMSimpleInstrument_h
#define ARMSimpleInstrument_h


#ifdef __GNUC__
#pragma interface
#endif

#include "SimpleInstruments.h"

	/*
	 **************************************************************
	 * Simple instruments for the ARM architecture defined here.
	 * There is one class each for when SIMPLE_INSTRUMENTATION is defined
	 * and not.  Be sure to add/delete both!
	 **************************************************************
	 */
#ifdef SIMPLE_INSTRUMENTATION
extern ClassStar ARMSimpleTimerClass;
extern ClassStar ARMCacheFlushTimerClass;
extern ClassStar ARMMMUTimerClass;
class ARMSimpleTimer : public SimpleTimer
{
public:
	ARMSimpleTimer(const char * const name) : SimpleTimer(name) {}
	Class * classOf();
};
class ARMCacheFlushTimer : public ARMSimpleTimer
{
public:
	ARMCacheFlushTimer(const char * const name) : ARMSimpleTimer(name) {}
	Class * classOf();
};
class ARMMMUTimer : public ARMSimpleTimer
{
public:
	ARMMMUTimer(const char * const name) : ARMSimpleTimer(name) {}
	Class * classOf();
};
#else /* NOT SIMPLE_INSTRUMENTATION*/
class ARMCacheFlushTimer : public SimpleTimer 
{
public:
	ARMCacheFlushTimer(const char * const, SimpleInstrumentState = ACTIVE) {}
};
class ARMMMUTimer : public SimpleTimer 
{
public:
	ARMMMUTimer(const char * const, SimpleInstrumentState = ACTIVE) {}
};
#endif /*SIMPLE_INSTRUMENTATION*/

#endif /*ARMSimpleInstrument_h*/
