/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Bjorn A. Helgaas (helgaas@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * MC68030CPU.h: per-cpu private information class for the MC68030 processor.
 */

#ifndef MC68030CPU_h
#define MC68030CPU_h


#ifdef __GNUC__
#pragma interface
#endif

#include "CPU.h"
#include "MC68030MMU.h"
#include "MC68030Context.h"

class Interrupt;


const int nInterrupts = 256;
const int bootStackSize = 2048;

extern ClassStar MC68030CPUClass;
class MC68030CPU : public CPU {
	friend void debugInitialize();
protected:

        /*
         * These are assembler assists for interrupt/trap handling.
         */
        static void interruptAssist();
	static void debugInterruptAssist();
	static void debugReturnFromInterrupt();

	/*
	 * Pointers to the interrupt vector table and the corresponding
	 * interrupt table.  When these are allocated, they MUST be
	 * allocated such that they are contiguous.  interruptAssist()
	 * relies on this fact in order to quickly associate a vector
	 * with an interrupt.  These should be filled in by a subclass.
         */
        PFV * interruptVectorTable;
        Interrupt ** interruptTable;

        MC68030MMU myMMU;

	/*
	 * This is called as the first process is being dispatched on
	 * this processor.  Since it is executed with (this == thisCPU())
	 * it can do things like load the vector base register, etc.
	 */
        virtual void chipInitialize();

        /*
         * Called by the kernel to install all the interrupts.  This may
	 * be executed by another processor on behalf of this one, so
	 * we can't use on-chip resources likes the vector base register
	 * here.
         */
        virtual void installInterrupts();

        /*
         * Helpers for trap/interrupt processing.
         */
        static int getContextSaveHint( Interrupt *, char * );

public:
	MC68030CPU( unsigned long identifier,
		ProcessContainer * initialIdleContainer );
	~MC68030CPU();

        /*
         * The subclasses define these to bind and lookup the binding of
         * interrupt (hardware) vectors to Interrupts.
         */
        virtual Interrupt * setInterrupt( int vector, Interrupt * interrupt );
	virtual Interrupt * interrupt( int vector );

        /*
         * Preempt the processor.
         */
        virtual void preempt();

        /*
         * Stuff for sanity checks. Mainly used in Assertions.
         */
        virtual int interruptsEnabled();
        virtual int interruptsDisabled();

	virtual void debugBreakpoint();

	virtual Class * classOf();
};

#endif /* MC68030CPU_h */
