/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Bjorn A. Helgaas (helgaas@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * MC68030ReferenceCount.h - An atomic integer.
 */

#ifndef MC68030ReferenceCount_h
#define MC68030ReferenceCount_h


#ifdef __GNUC__
#pragma interface
#endif

class MC68030ReferenceCount {
protected:
	volatile unsigned char state;
	short count;
public:
	inline MC68030ReferenceCount();
	inline ~MC68030ReferenceCount();
	inline int value();
	inline void increment();
	inline int decrement();
};

inline
MC68030ReferenceCount::MC68030ReferenceCount()
{
	state = 0;
	count = 0;
}

inline
MC68030ReferenceCount::~MC68030ReferenceCount()
{
	Assert( count == 0 );
}

inline int
MC68030ReferenceCount::value()
{
	return( count );
}

inline void
MC68030ReferenceCount::increment()
{
	/*
	 * Save the current interrupt state, then disable interrupts.
	 */
	unsigned short oldSR, newSR;
	asm( "movew %%sr,%0;\
	      oriw  #0x0700,%%sr" : "=g" (oldSR) );
	oldSR &= 0x0700;
	
	/*
	 * Now acquire the lock.  If we don't get the lock the first time,
	 * we spin only reading until it is released.  (Each instruction
	 * is 2 bytes long.)
	 */
	asm( "tas  %0@;\
	      beq  .+8;\
	      tstb %0@;\
	      bne  .-2;\
	      beq  .-8" : : "a" (this) );

	/*
	 * Now that we hold the lock, increment the count, release the
	 * lock, and restore the previous interrupt state.  (Don't touch
	 * the other parts of the SR just in case someone cares about
	 * a flag value, etc.)
	 */
	count++;
	state = 0;

	asm( "movew %%sr,%0" : "=g" (newSR) );
	newSR &= ~0x0700;
	newSR |= oldSR;
	asm( "movew %0,%%sr" : : "g" (newSR) );
}

inline int
MC68030ReferenceCount::decrement()
{
	/*
	 * See comments above; this is very similar.
	 */

	unsigned short oldSR, newSR;
	asm( "movew %%sr,%0;\
	      oriw  #0x0700,%%sr" : "=g" (oldSR) );
	oldSR &= 0x0700;
	
	asm( "tas  %0@;\
	      beq  .+8;\
	      tstb %0@;\
	      bne  .-2;\
	      beq  .-8" : : "a" (this) );

	int retval = --count;
	state = 0;

	asm( "movew %%sr,%0" : "=g" (newSR) );
	newSR &= ~0x0700;
	newSR |= oldSR;
	asm( "movew %0,%%sr" : : "g" (newSR) );

	return( retval );
}

#endif	/* MC68030ReferenceCount_h */
