/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Contributing Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Contributing Author: Bjorn A. Helgaas (helgaas@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * MC68030Translation.h:  Pages tables for a MC68030 MMU.
 */

#ifndef MC68030Translation_h
#define MC68030Translation_h


#ifdef __GNUC__
#pragma interface
#endif

#include "PhysicalMemoryRange.h"
#include "TwoLevelPageTable.h"


extern ClassStar MC68030TranslationClass;
class MC68030Translation : public AddressTranslation {
	friend void PhysicalMemorySetup();

protected:
	TwoLevelPageTable * systemTranslation;
	TwoLevelPageTable * userTranslation;

public:
	/*
	 * Used by MC68030MMU to get the value to load the PTBs with.
	 * This used to be private, but then the MMU class has to be a
	 * friend, which led to a problem with incomplete base types.
	 * This way we don't even need to include MMU.h.
	 */
	const POINTER_TABLE_ENTRY * systemPointerTable();
	const POINTER_TABLE_ENTRY * userPointerTable();

	/*
	 * Used by AddressTranslator (MC68030MMU) to see what happened when
	 * a page fault occurs.
	 */
	AddressTranslationFailureCode determineError( const char * address,
		AccessType attemptedAccess );

	static void init( unsigned int tableAreaSize,
		VirtualMemoryRange * systemMemoryMap );
	
	Class * classOf();

	MC68030Translation();
	~MC68030Translation();

	void addMapping( const char * vaddr, PhysicalMemoryChain * chain,
		ProtectionLevel prot );
	void basicRemoveMapping( const char * vaddr, unsigned int nbytes );
	int basicChangeProtection( const char * vaddr,
		unsigned int nbytes, ProtectionLevel protection );

	const char * mapping( const char * vaddr );
};

#endif /* MC68030Translation_h */
