/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Contributing Author: See-Mong Tan (stan@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * NS32332Context.h: Process context for a NS32332.
 */

#ifndef NS32332Context_h
#define NS32332Context_h


#ifdef __GNUC__
#pragma interface
#endif

#include "ProcessorContext.h"	/* parent class */
#include "StackSize.h"
#include "MemoryObjectStar.h"
class ObjectProxy;
class Domain;

struct Module {
	void * StaticBasePointer;
	unsigned int LinkBasePointer;
	void * ProgramBasePointer;
	// NOTE: Even though the fourth field is reserved
	// we use it to stash an Interrupt pointer in.
	void * InterruptPointer;
};

/*
 * Traps get a pointer to this in their "info" argument.
 */
struct stackFrame {
        unsigned int r2;
        unsigned int r1;
        unsigned int r0;
	unsigned int fp;
        unsigned int pc;
        unsigned short mod;
        unsigned short psr;
};

class RettFrame {
public:
	/*
	 * These form a stack frame suitable for a rett instruction
	 * and must remain in this order.
	 */
	PFV entryPoint;
	unsigned short mod;
	unsigned short psr;

};

class UserProxyState {
public:

	/*
	 * These next are saved/restored by the proxy trap assembly code.
	 * They are all the compiler non-volatile registers that the user
	 *    can set: we can't trust that the user hasn't written his own
	 *    assembly code to mess with them.
	 */
	unsigned	r3;
	unsigned	r4;
	unsigned	r5;
	unsigned	r6;
	unsigned	r7;
	unsigned	fp;
};

class UserProxyStack {
	friend class NS32332GenericContext;
protected:
	/*
	 * These items must stay in this order
	 */
	RettFrame rettFrame;

	UserProxyState state;
};

class asmdefs;

extern ClassStar NS32332GenericContextClass;
class NS32332GenericContext : public ProcessorContext {
friend class asmdefs;
protected:

	MemoryObjectStar _kernelStackMO;
	char * stackPointer[ 2 ];
	char * stackBase[ 2 ];
	char * stackBound[ 2 ];


	/*
	 * Generic Register Save Area.
	 */
	unsigned	r3;
	unsigned	r4;
	unsigned	r5;
	unsigned	r6;
	unsigned	r7;
	unsigned	fp;
	unsigned	sp;
	unsigned	pc;
	unsigned	usp;
	unsigned	sb;
	unsigned short		mod;
	unsigned short		psr;

	/*
	 * Generic checkpoint/restore methods.
	 */
	Process * checkpoint();
	void basicRestore( Process * );

public:
	NS32332GenericContext( Process * owner, StackSize stackSize );
	~NS32332GenericContext();

	/*
	 * Attempt to do a stack backtrace.
	 */
	void traceStack();

	Class * classOf();
};

class InitialApplicationContextStackDescription {
	friend class NS32332ApplicationContext;
protected:
	const char * usp;
	PFV returnPoint;
	int arg0;
	int arg1;
	ObjectProxy *defaultProxy;

	RettFrame rettFrame;
};

class asmdefs;
extern ClassStar NS32332ApplicationContextClass;
class NS32332ApplicationContext : public NS32332GenericContext {
friend class asmdefs;
protected:

	/*
	 * Additional registers saved by an application context.
	 */
	int	fsr;
	double	f0;
	double	f2;
	double	f4;
	double	f6;

	/*
	 * Overloaded to checkpoint/restore the additional registers above.
	 */
	Process * checkpoint();
	void basicRestore( Process * );

	void restoreFromInitialContext( Process * );
public:
#ifdef UNIXCOMPATIBILITY
	NS32332ApplicationContext( Process * owner, PFV entryPoint,
		StackSize stackSize, Domain *initialDomain, int arg0, int arg1,
		        int arg2, StackSize userStackSize,
			char * userStackPointer = 0 );
#else
	NS32332ApplicationContext( Process * owner, PFV entryPoint,
		StackSize stackSize, Domain *initialDomain, int arg0, int arg1,
		        StackSize userStackSize, char * userStackPointer = 0 );
#endif

	void initialRestoreAssist( Process * );

	Class * classOf();
};

extern ClassStar NS32332SystemContextClass;
class NS32332SystemContext : public NS32332GenericContext {
	friend void idleProcessInitialRestoreAssist( ProcessorContext *,
		Process * );
protected:
	void restoreFromInitialContext( Process * );
	void initialRestoreAssist( Process * );
public:
	NS32332SystemContext( Process * owner, PFV entryPoint,
		StackSize stackSize, int arg0, int arg1 );
	Class * classOf();
};

extern ClassStar NS32332UninterruptableSystemContextClass;
class NS32332UninterruptableSystemContext : public NS32332SystemContext {
protected:
public:
	NS32332UninterruptableSystemContext( Process * owner, PFV entryPoint,
		StackSize stackSize, int arg0, int arg1 );
	Class * classOf();
};

#endif /* NS32332Context_h */
