/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * Vectors.h: vector numbers for processor traps/interrupts.
 */

#ifndef Vectors_h
#define Vectors_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Assert.h"
#include "OutputStream.h"

/*
 * Trap Vectors
 */
static const int NVI_Vector = 0;	// Non-Vectored Interrupt
static const int NMI_Vector = 1;	// Non-Maskable Interrupt
static const int ABT_Vector = 2;	// Abort (VM Error) Trap
static const int FPU_Vector = 3;	// Floating Point Interrupt Trap
static const int ILL_Vector = 4;	// Illegal Instruction Trap
static const int SVC_Vector = 5;	// Supervisor Call Instruction Trap
static const int DVZ_Vector = 6;	// Divide by Zero Trap
static const int FLG_Vector = 7;	// Trap on Flag
static const int BPT_Vector = 8;	// Breakpoint Trap
static const int TRC_Vector = 9;	// Trace Trap
static const int UND_Vector = 10;	// Undefined Instruction Trap
static const int RESERVED_11_Vector = 11;	// Reserved
static const int BER_Vector = 12;		// Bus Error Trap
static const int RESERVED_13_Vector = 13;	// Reserved
static const int RESERVED_14_Vector = 14;	// Reserved
static const int RESERVED_15_Vector = 15;	// Reserved

static const int PROXY_Vector = ILL_Vector; // proxy uses privileged instruction

/*
 * Master ICU Vectors (0x10 - 0x1f).
 */
static const int POWERFAIL_Vector =		0x10;
static const int SYSTEMNMI_Vector =		0x11;
static const int HARDNANOBUSERROR0_Vector =	0x12;
static const int HARDNANOBUSERROR1_Vector =	0x13;
static const int HARDNANOBUSERROR2_Vector =	0x14;
static const int HARDNANOBUSERROR3_Vector =	0x15;
static const int DESTSELPARITYERROR_Vector =	0x16;
static const int OPEN_Vector =			0x17;
static const int SLAVEICU_Vector =		0x18;
static const int SOFTNANOBUSERROR0_Vector =	0x19;
static const int MASTERFREE1_Vector =		0x1A;
static const int TIMESLICE_Vector =		0x1B; // Time Slice Counter
static const int VECTORBUSINTERRUPT_Vector =	0x1C;
static const int MASTERFREE3_Vector =		0x1D;
static const int MASTERFREE4_Vector =		0x1E;
static const int MASTERFREE5_Vector =		0x1F;

/*
 * Slave ICU Vectors (0x20 - 0x2f).
 */
static const int SOFTNANOBUSERROR1_Vector =	0x20;
static const int SOFTNANOBUSERROR2_Vector =	0x21;
static const int RCVDADDRPARITYERRORB4_Vector =	0x22;
static const int RCVDADDRPARITYERRORB0_Vector =	0x23;
static const int RCVDADDRPARITYERRORB1_Vector =	0x24;
static const int RCVDADDRPARITYERRORB2_Vector =	0x25;
static const int RCVDADDRPARITYERRORB3_Vector =	0x26;
static const int BTAGPARITYERRORBYTE0_Vector =	0x27;
static const int BTAGPARITYERRORBYTE1_Vector =	0x28;
static const int VECTORBUSNAKERROR_Vector =	0x29;
static const int VECTORBUSPARITYERROR_Vector =	0x2A;
static const int VECTORBUSSYNCERROR_Vector =	0x2B;
static const int CACHEPARITYERROR_Vector =	0x2C;
static const int DUARTINTERRUPT_Vector =	0x2D;
static const int SLAVEFREE1_Vector =		0x2E;
static const int SLAVEFREE2_Vector =		0x2F;

/*
 * 0x30-0x3f are reserved for requestor CRQ's. One per slot.
 */
static const int SLOT0REQUESTOR_Vector	= 0x30;	// Slot 0's Requestor CRQ
static const int SLOT1REQUESTOR_Vector	= 0x31;	// Slot 1's Requestor CRQ
static const int SLOT2REQUESTOR_Vector	= 0x32;	// Slot 2's Requestor CRQ
static const int SLOT3REQUESTOR_Vector	= 0x33;	// Slot 3's Requestor CRQ
static const int SLOT4REQUESTOR_Vector	= 0x34;	// Slot 4's Requestor CRQ
static const int SLOT5REQUESTOR_Vector	= 0x35;	// Slot 5's Requestor CRQ
static const int SLOT6REQUESTOR_Vector	= 0x36;	// Slot 6's Requestor CRQ
static const int SLOT7REQUESTOR_Vector	= 0x37;	// Slot 7's Requestor CRQ
static const int SLOT8REQUESTOR_Vector	= 0x38;	// Slot 8's Requestor CRQ
static const int SLOT9REQUESTOR_Vector	= 0x39;	// Slot 9's Requestor CRQ
static const int SLOT10REQUESTOR_Vector	= 0x3a;	// Slot 10's Requestor CRQ
static const int SLOT11REQUESTOR_Vector	= 0x3b;	// Slot 11's Requestor CRQ
static const int SLOT12REQUESTOR_Vector	= 0x3c;	// Slot 12's Requestor CRQ
static const int SLOT13REQUESTOR_Vector	= 0x3d;	// Slot 13's Requestor CRQ
static const int SLOT14REQUESTOR_Vector	= 0x3e;	// Slot 14's Requestor CRQ
static const int SLOT15REQUESTOR_Vector	= 0x3f;	// Slot 15's Requestor CRQ

/*
 * Vector Bus Interrupt Vectors (0x40 - 0xff)
 */
static const int CONSOLEIN_Vector =	0x40;	// Console Input Interrupt
static const int CONSOLEOUT_Vector =	0x41;	// Console Output Interrupt
static const int DISK_Vector =		0x42;	// Disk Interrupt
static const int ETHER_Vector =		0x43;	// Ethernet Control Interrupt
static const int ETHERL_Vector =	0x44;	// Ethernet Large Packet
static const int ETHERS_Vector =	0x45;	// Ethernet Small Packet
static const int TAPE_Vector =		0x46;	// Tape Interrupt
static const int MMU_Vector =		0x47;	// MMU flush Interrupt
static const int INTERVAL_Vector =	0x48;	// Interval Timer Interrupt
static const int PREEMPT_Vector	=	0x49;	// CPU Preempt Vector
#ifdef INSTRUMENT
static const int INSTRUMENT_Vector =	0x50;	// Dump instrumentation buffers
#endif

/*
 * Vector bus classes.
 */
static const int CPUCLASS_HALTED = 0;	// Halted CPU class
static const int CPUCLASS_RUNNING = 1;	// Normal, running CPU class
static const int CPUCLASS_MAX =	1;	// Highest class used

/*
 * Vector manipulation routines.
 */

extern unsigned int SetCPUVectorClass( unsigned int vectorClass );
extern unsigned int CPUVectorClass();
extern unsigned int DirectedVector( unsigned int number, unsigned int cpuid );
extern unsigned int ClassVector( unsigned int number, unsigned int classvec );
extern unsigned int VectorNumber( unsigned int vector );
extern void TransmitVector( unsigned int vector );

static long * const APCSTSREG = (long *) 0xffffff54;
static const unsigned int STSOUTRDY = 1 << 13;
static unsigned char * const VECBUSFIFO = (unsigned char *) 0xffffff48;

inline void
TransmitDirectedVector( unsigned int number, unsigned int dest )
{
	TransmitVector( DirectedVector( number, dest ) );
}

inline void
TransmitClassVector( unsigned int number, unsigned int vectorClass )
{
	TransmitVector( ClassVector( number, vectorClass ) );
}

/*
 * This is only called with interrupts disabled. Although, I'm not sure it
 * really matters anyway.
 */
inline int
FlushVector()
{
	int number = -1;
	if ( *APCSTSREG & STSOUTRDY ) {
		number = (int) *VECBUSFIFO;
		Assert( number != -1 );
	}
	Debug.printf( "FlushVector: %x\n", number );
	Assert( number >= -1 );
	return ( number );
}

#endif /* Vectors_h */
