/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SPARCContext.h: Process context for a SPARC.
 *
 */

#ifndef SPARCContext_h
#define SPARCContext_h


#ifdef __GNUC__
#pragma interface
#endif


#include "MemoryObjectStar.h"
#include "ProcessorContext.h"	/* parent class */
#include "StackSize.h"
#include <sys/psw.h>
class ObjectProxy;
class Domain;

extern ClassStar SPARCGenericContextClass;
class SPARCGenericContext : public ProcessorContext {
protected:
	virtual Process * checkpoint();
	Process* basicRestore( Process * );

	static void defaultReturn();


	virtual void restoreFromInitialContext( Process * );
	/*
	 * Instance data
	 */
	unsigned long long globals[4];			 // 7 global regs + %y
	unsigned long long volatileRegs[8];		// 16 volatile regs
	unsigned _entryPoint;			// PC

	/*
	 * Stack related things
	 */
	MemoryObjectStar _kernelStackMO;
	char * _kernelStackBase;
	unsigned _kernelStackSize;

        char * _stackPointer;

        char * _userStackTop;
	unsigned _stackBytes;

	unsigned int	psr;
	// In initial context, interrupt state is in inputs[0]
	// pc is in inputs[1], and arg0 is in inputs[2]
	// arg1 in inputs[3] 
	// initial proxy in inputs[4]

	/* 
	 * Allocate a stack of size in domain, set the MO star
	 */
        void allocateStack(StackSize size, Domain *domain,
		char * &, unsigned &, MemoryObjectStar &, char * &); 

	/* 
	 * Floating point stuff (only valid for applications )
	 */
	unsigned fsr;
	unsigned long long floats[16];
public:
	fpEnabled() {return (psr & PSR_EF);}

public:
	char *	_kernelStackTop;
	unsigned _kernelStackBytes;
	SPARCGenericContext( Process * owner, PFV entry, StackSize stackSize, 
		int arg0, int arg1 );
	virtual ~SPARCGenericContext();
        Class * classOf();

	void traceStack();

};

extern ClassStar SPARCSystemContextClass;
class SPARCSystemContext : public SPARCGenericContext {
protected:
public:
	SPARCSystemContext( Process * owner, PFV entryPoint,
		StackSize stackSize, int arg0 , int arg1 = 0) ;
	Class * classOf();
};

inline
SPARCSystemContext::SPARCSystemContext( Process * owner, PFV entry,
		StackSize stackSize, int arg0 , int arg1) :
			SPARCGenericContext( owner, entry, stackSize, arg0, arg1)
{}

extern ClassStar SPARCUninterruptableSystemContextClass;
class SPARCUninterruptableSystemContext : public SPARCSystemContext {
protected:
public:
	SPARCUninterruptableSystemContext( Process * owner, PFV entryPoint,
		StackSize stackSize, int arg0, int arg1 = 0);
	Class * classOf();
};


extern ClassStar SPARCApplicationContextClass;
class SPARCApplicationContext : public SPARCGenericContext {
protected:

	MemoryObjectStar userStackMO;
	char * userStackBase;

public:
	~SPARCApplicationContext(){}
	SPARCApplicationContext( Process * owner, PFV entryPoint, 
		StackSize = application, Domain *initialDomain = 0, 
		int arg0 = 0, int arg1 = 0, StackSize = application, 
		char *userStackPointer = 0);
	Class * classOf();
};

#endif /* SPARCContext_h */
