/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 WSpringfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-2021
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SPARCInterrupts.h: hardware interrupt sub-class descriptions for the
 *	SPARC processor.
 */

#ifndef SPARCInterrupts_h
#define SPARCInterrupts_h


#ifdef __GNUC__
#pragma interface
#endif


#include "Interrupt.h"		/* parent class */

/*
 * These are the levels that the particular harware devices interrupt
 * at, useful in assertions, etc. within drivers.
 */
const unsigned char INTERRUPT_TIMER = 10;
const unsigned char INTERRUPT_FREETIMER = 14;

/*
 * Lock out EVERYTHING, ie. disable ALL interrupts
 */
const unsigned char INTERRUPT_NO_INTERRUPTS = 0xf;
const unsigned char INTERRUPT_LOCK = INTERRUPT_NO_INTERRUPTS;

/*
 * These are the levels that interruptable and uninterruptable contexts
 * start out at, and what the levels are set to when a context gets the
 * becomeUnInterruptable or becomeInterruptable methods.
 */
const unsigned char INTERRUPT_HIGH = INTERRUPT_FREETIMER - 1; 
const unsigned char INTERRUPT_LOW = 0;
const unsigned char INTERRUPT_CSWTCH = INTERRUPT_NO_INTERRUPTS;

/*
 * On the SPARC the level is passed in info so the handler can reinstate
 * interrupts if it desires.
 */

class SPARCFPInterrupt : public Interrupt
{
protected:
        void basicRaise(char *);
};

class SPARCFPDisabledInterrupt : public Interrupt
{
protected:
        void basicRaise(char *);
};

class SPARCAsynchronousMemoryInterrupt : public Interrupt {
protected:
	void basicRaise( char * info );
public:
};

class SPARCMemoryAlignmentInterrupt : public Interrupt {
protected:
	void basicRaise( char * info );
public:
};

class SPARCDataAccessInterrupt : public Interrupt {
protected:
	void basicRaise( char * info );
public:
};

class SPARCInstructionAccessInterrupt : public Interrupt {
protected:
	void basicRaise( char * info );
public:
};

class SPARCIllegalInstructionInterrupt : public Interrupt {
protected:
	void basicRaise( char * info );
public:
};

class SPARCDivideByZeroInterrupt : public Interrupt {
protected:
	void basicRaise( char * info );
};

class SPARCProxyInterrupt : public Interrupt {
protected:
	void basicRaise( char * info );
};


#endif /* SPARCInterrupts_h */
