/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SPARCasmutil.h: Process context for a SPARC.
 */

#ifndef SPARCasmutil_h
#define SPARCasmutil_h


#ifdef __GNUC__
#pragma interface
#endif


#ifdef ASM

/*
 * Things to make gas happy
 */
#ifdef GAS
#define bz be
#define bnz bne
#define bnz bne
#undef ASI_CTL
#define ASI_CTL 2
#undef ASI_SM
#define ASI_SM 3
#undef ASI_PM
#define ASI_PM 4
#endif /*GAS*/

#else /*ASM*/

extern "C" void flushWindows();

#define BSTRINGIZE(a...) #a
#define STRINGIZE(a...) BSTRINGIZE( a )

#define LOAD_REG_FROM_MEM(reg, var, clobber...)  \
	asm volatile ( STRINGIZE(ld %0,%reg) : : "m" (var)); 

#define LOAD_REG_FROM_MEM_DOUBLE(reg1, var) \
	asm volatile ( STRINGIZE(ldd %0,%reg1) : : "m" (var) );

#ifdef NEVER
#define LOAD_REG_FROM_MEM(reg, var, clobber...)  \
	asm volatile ( STRINGIZE(ld %0,%reg) : : "m" (var) : STRINGIZE(reg)); 

#define LOAD_REG_FROM_MEM_DOUBLE(reg1, reg2, var) \
	asm volatile ( STRINGIZE(ldd %0,%reg1) : : "m" (var) : STRINGIZE(reg1), STRINGIZE(reg2)); 
#endif

#define STORE_REG_TO_MEM(reg, var, clobber...) \
	asm volatile(STRINGIZE(st %reg,%0) : : "m" (var));\

#define STORE_REG_TO_MEM_DOUBLE(reg, var, clobber...) \
	asm volatile(STRINGIZE(std %reg,%0) : : "m" (var));\

	/*
	 * Useful for interrupt routines that read registers to clear,
	 * but never use the results.  Avoids compiler optimizations
	 */
#define FORCE_USE(var) \
	asm volatile("mov %0,%%g0" : : "r" (var));


inline unsigned
readWordASI(register unsigned *addr, unsigned asi)
{
	unsigned result;
	asm ("lda [%1]%2,%0" : "=g"(result) : "r"(addr) , "i"(asi));
	return result;
}

inline void
writeWordASI(unsigned *const addr, const unsigned value, const unsigned asi)
{
	asm volatile("sta %1,[%0]%2" : : "r" (addr), "r" (value), "i"(asi));
}

inline unsigned char
readByteASI(volatile unsigned char *const addr, const unsigned asi)
{
	register unsigned char result;
	asm volatile("lduba [%1]%2,%0" : "=r" (result) : "r" (addr), "i" (asi));
	return result;
}

inline void
writeByteASI(unsigned char *const addr, const unsigned char value, const unsigned asi)
{
	asm volatile("stba %1,[%0]%2" : : "r" (addr), "r" (value), "i"(asi));
}

#endif /*ASM*/
#endif /* SPARCasmutil_h */
