/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef VCCPU_h
#define VCCPU_h


#ifdef __GNUC__
#pragma interface
#endif

#include "MachineTypes.h"
#include "CPU.h"
#include <signal.h>
#include <ucontext.h>

#if defined(linux)
#   include <bits/siginfo.h>
    typedef __caddr_t caddr_t;
#else 
#ifdef EMCHAN
#include <siginfo.h>
#endif
#endif





/*
 * These objects are passed to handlers by the signal system.
 * Interrupts get them in the info ptr and can find out what happened.
 */
class VCSignalInfo {
	siginfo_t *_si;
	ucontext_t *_uc;
	caddr_t _faddr; 
public:
	VCSignalInfo(int, siginfo_t *si, ucontext_t *uc, caddr_t faddr) : _si(si), _uc(uc), _faddr(faddr) {}
	int code() { return _si->si_code;}
	void *address() { return _faddr;}
};

extern "C"
int
vcmain( int argc, char ** argv );

extern ClassStar VCCPUClass;
class VCCPU : public CPU {
friend int main(int argc, char **argv);
friend int vcmain(int argc, char **argv);
private:
	static sigset_t	_interrupts;		// The set of sigs to en/dis/able.
	static void initSignalStuff();	// THIS MUST BE THE VERY FIRST CODE
												// CALLED EVEN BEFORE STATIC CTORS
protected:
	static const int nInterrupts = 32;
	static Interrupt * interruptTable[ nInterrupts ];
	Interrupt *interrupt(int n) {return interruptTable[n];}

	static void         hardwareInterruptAssist(int vector, siginfo_t *,
				void *);
public:
    VCCPU( ProcessContainer * initialContainer );
    ~VCCPU(){}

    void installInterrupts();
    void chipInitialize(){}

    Interrupt * setInterrupt( int vector, Interrupt * interrupt, PFV f );

    void preempt(){}

    void debugBreakpoint();

    void sendFlushRequestTo( unsigned long dest ){}
    void idleFunction();

    static inline void disableInterrupts(IntStorage &arg)
		{
		sigprocmask(SIG_BLOCK, &_interrupts, &arg);
		}
    static inline void disableInterrupts()
		{
		sigprocmask(SIG_BLOCK, &_interrupts, 0);
		}
    static inline void restoreInterrupts(IntStorage &old){
			sigprocmask(SIG_SETMASK, &old, 0);
		}
    static inline void enableInterrupts(){
		sigprocmask(SIG_UNBLOCK, &_interrupts, 0);
		}
    static int interruptsEnabled();
    static void initMutexState(MutexStateAddr addr){*addr = 0;}
    static void spinMutex(MutexStateAddr);
    static void releaseMutex(MutexStateAddr addr){*addr = 0;}
    static unsigned cpuID(){return 0;}

    Class * classOf();
};

inline void
VCCPU::spinMutex(MutexStateAddr addr)
{
	Assert (addr != 0);
	Assert (*addr == 0);
	*addr = 1;
}


	/*
	 * A class to encapsulate the UNIX signal information since
	 * Choices interrupts prefer a single argument cast to a char *
	 */
class VCISignalInfo {
protected:
	siginfo_t *_siginfo;
	ucontext_t *_ctxt;
public:
	VCISignalInfo(siginfo_t *si, ucontext_t *uc)
		{_siginfo = si; _ctxt = uc;}
	int vector() {return _siginfo->si_signo;}
	int code(){ 
#if !defined(linux) || defined(__APPLE__)
#ifdef EMCHAN
		if (_siginfo->si_code == SI_NOINFO) 
			return 0;
		else
#endif
#endif
			return _siginfo->si_code;
	}
	siginfo_t *siginfo(){return _siginfo;}
#if defined(linux) || defined(__APPLE__)
	// linux_port: char* to void*, si_faddr to si_addr 
	void *address(){ return _siginfo->si_addr; }
#else 
	char *address(){ return _siginfo->si_faddr; }
#endif
};

#endif
