/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
#ifndef VCPageTable_h
#define VCPageTable_h


#ifdef __GNUC__
#pragma interface
#endif

#include "AddressTranslation.h"
class Allocator;
#include "VCPageTableConstants.h"
class VirtualMemoryRange;
class PhysicallyAddressableUnit;


/*
 * Each VCPageMapping maps a single page within an ADT.
 * It is basically like an extensible page table entry.
 */
class VCPageMapping {
	const char * 			_vaddress;	// virtual address
	unsigned			_unit;		// unit in store file
	PhysicallyAddressableUnit *	_pau;		// pau mapped to
	ProtectionLevel			_prot;		// protection mapped at
public:
	VCPageMapping() {_vaddress = 0; _pau = 0; _unit = 0; _prot = NoAccess;}
	VCPageMapping(const char *vaddr, unsigned unit,
		PhysicallyAddressableUnit *pau, ProtectionLevel prot) 
		{_vaddress = vaddr; _pau = pau; _unit = unit; _prot = prot;}

	const char * vaddress() {return _vaddress;}
	unsigned unit() {return _unit;}
	ProtectionLevel prot() {return _prot;}
	PhysicallyAddressableUnit * pau() {return _pau;}
};

/*
 * VCPageTable objects keep a list of page mappings.
 * Keep them sorted and a count of mappings just for fun.
 * Kinda like the code in Domain
 */
extern ClassStar VCPageTableClass;
class VCPageTable : public AddressTranslation {
protected:
	static Allocator * 	_allocator;
	static Lock *	 	_lock;
	VCPageMapping *		_mappings;
	unsigned		_numMappings;
	char *			_baseAddr;	// vaddr of first mapped pg
	char *			_endAddr;	// vaddr past last mapped pg
	void		allocatePageTable();
public:
    VCPageTable( VCPageTable *p ) ;
    virtual ~VCPageTable();

    static void init( unsigned int, VirtualMemoryRange * );

    void addMapping( const char * vaddr,
		    PhysicalMemoryChain * chain, ProtectionLevel prot );
    void addPageMapping( const char * vaddr,
		    PhysicallyAddressableUnit * unit, ProtectionLevel prot);
    void removePageMapping( const char * vaddr, int imActive);
    void basicRemoveMapping( const char * vaddr,
			    unsigned int nbytes );

    int basicReferenced ( const char * vaddr, unsigned int nbytes, int clear )
	{return 1;};

    int basicChangeProtection( const char * vaddr, unsigned int nbytes, 
			      ProtectionLevel protection );
	int basicChangeStatus( const char * vaddr, unsigned int nbytes,
			  AddressTranslationStatus clear,
			  AddressTranslationStatus set );

    /* 
     * These are used by the domain switching to mmap
     * and unmap the current translation
     */
    void activate();

    Class * classOf();
};

#endif
