/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * VCPageTableConstants.h:
 */

#ifndef VCPageTableConstants_h
#define VCPageTableConstants_h


#ifdef __GNUC__
#pragma interface
#endif

/*
 * The following constants are used primarily by the TwoLevelPageTable class.
 */

static const int PhysicalPageSize         = 4096;
static const int Log2PhysicalPageSize     = 12;

static const unsigned int Log2SegmentSize            = 18;
static const unsigned int SegmentSize                = 1 << Log2SegmentSize;


static const int Log2PointerTableEntries = 10;
static const int PointerTableEntries     = 1024;
static const int PointerTableSize        = 4096;

static const int Log2PageTableEntries    = 10;
static const int PageTableEntries        = 1024;
static const int PageTableSize           = 4096;

static const int Pte_PROTECT_LEVELS	 = 4;

enum VCProtection { SYSTEMRO = 0, SYSTEMRW = 1, USERRO = 2, USERRW = 3 };

inline char *
segmentAlign(void *const  addr)
{
	return (char *)(
		(unsigned long)addr & (~0 << Log2SegmentSize) );
}

inline char * 
pageAlign(void *  addr)
{
	return (char *) (
		(unsigned long)addr & (~0 << Log2PhysicalPageSize) );
}

inline unsigned
segmentOffset(void * addr)
{
	return  (unsigned long)addr % SegmentSize;
}

inline unsigned
pageOffset(void *const addr)
{
	return (unsigned long)addr % PhysicalPageSize;
}

#endif	/* VCPageTableConstants_h */
