/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef VCContext_h
#define VCContext_h

#ifdef __GNUC__
#pragma interface
#endif

#include <stdlib.h>
#include <setjmp.h>

#include "ProcessorContext.h"
#include "StackSize.h"
#include "Types.h"



class ObjectProxy;
class Domain;
class MemoryObject;

/* Union to double align the register memory for context switching */
union _union { long unsigned u; double d;};
typedef _union doublereg;


extern ClassStar VCGenericContextClass;
class VCGenericContext : public ProcessorContext {
public:
    doublereg globals[4];	        // 7 global regs + %y
    doublereg volatileRegs[8];		// 16 volatile regs
    char * stackPointer;		// Can be folded into inputs
    char * stackBase;		        // Necessary?/*
    MemoryObject *	_stackMO;

#if defined(linux)
    __jmp_buf_tag my_jmp_buf;

virtual    __jmp_buf_tag* getJmpBuf() {
	return &my_jmp_buf;
}
#endif

protected:
    PFV _entryPoint;
    OPFV _opfvEntryPoint;

    int _interruptable;

    /*
     * basicRestore() restores this context.
     * restoreFromInitialContext() is called first time round.
     */
    Process * basicRestore( Process * processGivingUpProcessor );
    void restoreFromInitialContext( Process * processGivingUpProcessor  );
    
    Process * checkpoint();

  void initialize(StackSize);
public:
    VCGenericContext( Process * owner, PFV entry,
				 StackSize stacksize,
				 int arg0, int arg1 );
    VCGenericContext( Process * owner, PFV entry,
				 StackSize stacksize,
				 OPFV arg0, int arg1 );
    ~VCGenericContext();

    int		addrInStack(void *);
    
    Class * classOf();
};

extern ClassStar VCApplicationContextClass;
class VCApplicationContext : public VCGenericContext {
protected:
    void startOff();
public:
    VCApplicationContext( Process * owner, PFV entryPoint,
				     StackSize stackSize, 
				     Domain *initialDomain,
				     int arg0, int arg1,
				     StackSize userStackSize, 
				     char * userStackPointer = 0);
    
    Class * classOf();
};

extern ClassStar VCSystemContextClass;
class VCSystemContext : public VCGenericContext {
protected:

public:
    VCSystemContext( Process * owner, PFV entryPoint,
				StackSize stackSize, int arg0, int arg1 );
    VCSystemContext( Process * owner, PFV entryPoint,
				StackSize stackSize, OPFV arg0, int arg1 );
    Class * classOf();
};

extern ClassStar VCUninterruptableSystemContextClass;
class VCUninterruptableSystemContext : public VCSystemContext {
protected:
    void startOff();
public:
    VCUninterruptableSystemContext( Process * owner, 
					       PFV entryPoint,
					       StackSize stackSize, 
					       int arg0, int arg1 );
    Class * classOf();
};

#endif
