/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Ruth A. Aydt (aydt@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 *	SegmentDescriptor.h:  Header file defining classes for the 
 *	various 80586 Segment Descriptors that may appear in the Global
 *	Descriptor Table.
 *	
 *	Task State Segment Descriptor: 
 *			(page 7-3 in 80586 Programmer's Reference Manual)
 *	type = BusyTask or NonBusyTask 
 *	accessed = 1
 */

#ifndef SegmentDescriptor_h
#define SegmentDescriptor_h
#ifdef CFRONT
#define CONST const
#define STATIC static
#else /* must be g++ */
#pragma interface
#if (__GNUG__ >= 2)
#ifndef CONST
#define CONST
#define STATIC
#endif
#else
#pragma once
#define CONST const
#define STATIC static
#endif
#endif

#include "i586DataTypes.h"
#include "TaskStateSegment.h"
 
enum    DescriptorType { BusyTask=0x05, NonBusyTask=0x04 };
enum	PrivilegeLevel { LevelZero, LevelOne, LevelTwo, LevelThree };
enum 	SDGranularity { OneByte=0x00, FourKilobytes=0x01 };

class SegmentDescriptor {
private:
    unsigned int low_limit	:16;
    unsigned int low_base	:16;
    unsigned int mid_base	:8 ;
    unsigned int accessed	:1 ;
    unsigned int type		:4 ;
    unsigned int dpl		:2 ;
    unsigned int present	:1 ;
    unsigned int high_limit	:4 ;
    unsigned int b20		:1 ;
    unsigned int b21		:1 ;
    unsigned int b22		:1 ;
    unsigned int granularity	:1 ;
    unsigned int high_base	:8 ;
public:
    SegmentDescriptor();
    SegmentDescriptor( DescriptorType dtype, TaskStateSegment * base, PrivilegeLevel priv);
    TaskStateSegment * base();
};

inline
SegmentDescriptor::SegmentDescriptor()
{
}

/*
 * This is the constructor for a TSS descriptor		
 */
inline
SegmentDescriptor::SegmentDescriptor( DescriptorType dtype, TaskStateSegment * base, PrivilegeLevel priv)
{
    union doubleword dbl;
    dbl.all = (unsigned) base;

    low_limit = sizeof( TaskStateSegment );
    low_base = dbl.word.n0;
    mid_base = dbl.byte.n2;
    accessed = 1;
    type = dtype;
    dpl = priv;
    present = 1;
    high_limit = 0;
    b21 = b22 = 0;
    granularity = OneByte;
    high_base = dbl.byte.n3;
}

/*
 * Return pointer to the Task State Segment for this descriptor
 */
STATIC inline TaskStateSegment * 
SegmentDescriptor::base()
{
    union doubleword dbl;
    dbl.word.n0 = low_base;
    dbl.byte.n2 = mid_base;
    dbl.byte.n3 = high_base;
    return ( (TaskStateSegment *) dbl.all );
}

#endif //SegmentDescriptor_h
