/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Ruth A. Aydt (aydt@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 *	TaskStateSegment.h:  Header file defining class for the 
 *	80586 Task State Segment.  See page 7-2 in the 80383 Programmer's
 *	Reference Manual.  Also, see 4-2 for eflags.
 */

#ifndef TaskStateSegment_h
#define TaskStateSegment_h
#ifdef CFRONT
#define CONST const
#define STATIC static
#else /* must be g++ */

#ifdef I586SYM
#pragma interface
#endif

#if (__GNUG__ >= 2)
#if (!defined CONST)||(!defined STATIC)
#define CONST
#define STATIC
#endif
#else
#pragma once
#define CONST const
#define STATIC static
#endif
#endif

#include "AddressTranslation.h"
#include "OutputStream.h"
#include "EflagsRegister.h"

class TaskStateSegment {
public:
    unsigned short back_link;	// back link to previous tss
    unsigned short fill1;
    unsigned int esp0;		// stack offset - priv level 0
    unsigned short ss0;		// stack selector - priv level 0
    unsigned short fill2;
    unsigned int esp1;		// stack offset - priv level 1
    unsigned short ss1;		// stack selector - priv level 1
    unsigned short fill3;
    unsigned int esp2;		// stack offset - priv level 2
    unsigned short ss2;		// stack selector - priv level 2
    unsigned short fill4;
    unsigned int pdbr;		// phys addr of page directory table
    unsigned int eip;		// instruction pointer
    EflagsRegister eflags;	// system flags
    unsigned int eax;		// general register
    unsigned int ecx;		// general register
    unsigned int edx;		// general register
    unsigned int ebx;		// general register
    unsigned int esp;		// stack pointer
    unsigned int ebp;		// stack-frame base pointer
    unsigned int esi;		// general register
    unsigned int edi;		// general register
    unsigned short es;		// extra data segment
    unsigned short fill5;	
    unsigned short cs; 		// code segment
    unsigned short fill6;	
    unsigned short ss; 		// stack segment
    unsigned short fill7;	
    unsigned short ds; 		// data segment
    unsigned short fill8;	
    unsigned short fs; 		// extra data segment
    unsigned short fill9;	
    unsigned short gs; 		// extra data segment
    unsigned short fill10;	
    unsigned short ldt;		// selector for local descriptor table
    unsigned short fill11;	
    unsigned short t;		// debug trap bit
    unsigned short iobase;	// i/o permission map base
public:
    void setPdbr( const char * );
    AddressTranslation * valuOfPdbr( );
    void setEspEbp( int, void *, void *);
    void * valuOfEsp( );
    void * valuOfEbp( );
};

STATIC inline void
TaskStateSegment::setPdbr( const char * baseAddr)
{
    pdbr = (unsigned int) baseAddr;
}

STATIC inline AddressTranslation *
TaskStateSegment::valuOfPdbr( )
{
    return ( (AddressTranslation *) pdbr );
}

STATIC inline void
TaskStateSegment::setEspEbp( int protLevel, void * stackPtr, void * basePtr)
{
    esp = (unsigned int) stackPtr;

    if (protLevel == 0)
	esp0 = esp;
    else if (protLevel == 1)
	esp1 = esp;
    else if (protLevel == 2)
	esp2 = esp;
    else
	Console().panic( "TSS::setEspEbp - invalid protection level for sp\n");

    ebp = (unsigned int) basePtr;
}

STATIC inline void *
TaskStateSegment::valuOfEsp( )
{
    return ( (void *) esp );
}

STATIC inline void *
TaskStateSegment::valuOfEbp( )
{
    return ( (void *) ebp );
}

#endif //TaskStateSegment_h
