/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * FIFOScheduler.h: a FIFO Process container.
 */

#ifndef FIFOScheduler_h
#define FIFOScheduler_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Assert.h"
#include "Debug.h"
#include "Lock.h"
#include "OutputStream.h"
#include "ProcessContainer.h"


/*
 * FIFOScheduler - a non-preempt FIFO run-to-completion scheduler.
 */

class FIFOScheduler : public ProcessContainer {
protected:
	Lock lock;
	Process * head;
	Process * tail;
public:
	FIFOScheduler();
	~FIFOScheduler();
	void add( Process * );
	Process * remove();
	int isEmpty();
};

inline
FIFOScheduler::FIFOScheduler()
{
	Debug << this << "::FIFOScheduler()\n" << eor;
	Assert( this != 0 );
	head = tail = 0;
}

/*
 * A FIFOScheduler without a lock. It assumes and user of it somehow
 * guarentees mutual exclusion for it.
 */
class NonLockedFIFOScheduler : public ProcessContainer {
protected:
	Process * head;
	Process * tail;
public:
	NonLockedFIFOScheduler();
	~NonLockedFIFOScheduler();

	void add( Process * );
	Process * remove();
	int isEmpty();
};

inline
NonLockedFIFOScheduler::NonLockedFIFOScheduler()
{
	Assert( this != 0 );
	head = tail = 0;
}
 
#endif /* FIFOScheduler_h */
