/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *	SchedList data structures used by the USS scheduler implementations.
 *	Concrete subclasses are SchedQueue, SchedStack, SchedFeedbackQueues, 
 *	SchedPriorityList, and SchedSearchList.  SchedList is an abstract class.
 */

#ifndef List_h 
#define List_h 


#ifdef __GNUC__
#pragma interface
#endif

#include "ProcessContainer.h"		/* parent class */
#include "Lock.h"

class Process;		// forward reference...


// This class is used by the SchedList classes and schedulers to set and get
// the process priority.  It is stored in the schedulerInfo field of a process.
class USSSchedulerInfo;

// Abstract list class defines the framework for the subclasses
class SchedList : public ProcessContainer {
public:
	virtual void add( Process * ) = 0;
	virtual Process * remove() = 0;
	virtual int isEmpty() = 0;
};

// SchedQueue data structure used by FIFO and RR schedulers
class SchedQueue : public SchedList {
protected:
	Lock lock;
	USSSchedulerInfo * head;
	USSSchedulerInfo * tail;
public:
	SchedQueue( char * name );
	void add( Process * process );
	Process * remove();
	int isEmpty();
};

// SchedStack data structure used by LIFO scheduler
class SchedStack : public SchedQueue {
public:
	SchedStack( char * name );
	void add( Process * process );
};

// FeebackQueues data structure used by MLF and VMS schedulers.
class SchedFeedbackQueues : public SchedList {
private:
	int numLevels;
	SchedQueue ** Queues;
public:
	SchedFeedbackQueues( int levels, char * name );
	~SchedFeedbackQueues();
	void add( Process * process );
	Process * remove();
	int isEmpty();
};

// Searchlist data structure used by HRN and UNIX schedulers 
class SchedSearchList : public SchedList {
protected:
	Lock lock;
	USSSchedulerInfo * first;
public:
	SchedSearchList( char * name );
	void add( Process * process );
	Process * remove();
	int isEmpty();
};

// SchedPriorityList data structure used by SRT, Deadline and VMS schedulers
class SchedPriorityList : public SchedList {
protected:
	Lock lock;
	USSSchedulerInfo * first;
	USSSchedulerInfo * last;		// (Preemptive Schedulers) 
public:
	SchedPriorityList( char * name );
	void add( Process * process );
	Process * remove();
	int isEmpty();
	void premove( Process * );		// (Preemptive Schedulers) 
	Process * findPreempt( Process * );	// (Preemptive Schedulers) 
};

#endif /* List_h */
