/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Contributing Author: Roy H. Campbell (roy@cs.uiuc.edu)
 * Contributing Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Contributing Author: Johnny Zweig (zweig@cs.uiuc.edu)
 * Contributing Author: Roland Ouellette (ouellette@tarkin.enet.dec.com)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * Semaphore.h - Semaphore class description.
 */

#ifndef Semaphore_h
#define Semaphore_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Assert.h"
#include "OutputStream.h"
#include "Object.h"
#include "Lock.h"
#include "FIFOScheduler.h"


class Process;
extern ClassStar SemaphoreClass;
class Semaphore : public ProxiableObject {
protected:
	Process *_holder;
	BusyWait lock;
	int count;
	int maxCount;
	NonLockedFIFOScheduler queue;
public:
	// use something big for the max as a default.
	proxiable Semaphore( int initialCount, int maxCount = 65535 );
	proxiable ~Semaphore();

	proxiable virtual void P();
	proxiable virtual void V();

	Class * classOf();
};

extern ClassStar BinarySemaphoreClass;
class BinarySemaphore : public Semaphore {
public:
	proxiable BinarySemaphore( int initialCount );
	proxiable ~BinarySemaphore();

	Class * classOf();
};


/*
 * A semaphore that rather than enqueueing a waiting process upon a V()
 * operation immediately runs the process and enqueues the invoking process
 * instead. The default maxCount is 1 since this is usually used in interrupt
 * processing to re-start an interrupt handler process which was awaiting
 * the interrupt.
 */
extern ClassStar GraciousSemaphoreClass;
class GraciousSemaphore : public Semaphore {
public:
	GraciousSemaphore( int initialCount, int maxCount = 65535 );
	~GraciousSemaphore();
	void P();
	void V();

	Class * classOf();
 private:
    Process *givingProcess;
};


/*
 * A MutexSemaphore is a BinarySemaphore that starts out with
 * the resource available.
 */
class MutexSemaphore : public BinarySemaphore {
public:
	MutexSemaphore();
	~MutexSemaphore();
};


inline
MutexSemaphore::MutexSemaphore() : BinarySemaphore( 1 )
{
}

inline
MutexSemaphore::~MutexSemaphore()
{
}

/*
 * A LazySemaphore is a BinarySemaphore that can be V'd more times
 * then it's P'd safely
 */
class LazySemaphore: public BinarySemaphore {
public:
    LazySemaphore( int initialCount );
    ~LazySemaphore();
    void V();
};

class MultiVSemaphore : public Semaphore {
public:
    MultiVSemaphore( int initialCount, int maxCount = 65535 );
    ~MultiVSemaphore();
    void V();			/* just to avoid CFRONT warning */
    void V(int times_to_v);
    const char * className();
};

inline void
MultiVSemaphore::V()
{
    Semaphore::V();		/* to avoid CFRONT warning */
}

#endif /* Semaphore_h */


