/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1994
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SoftClock.h
 */

#ifndef SoftClock_h
#define SoftClock_h

#ifdef __GNUC__
#pragma interface
#endif

#include "Lock.h"

typedef void (*SoftClockFunc)(char *, int);

class SoftwareClockManager {
protected:
	static const int MaxSoftCalls = 256;
	struct SoftCall {
		SoftCall * _next;
		SoftClockFunc _func;
		char * _arg;
		int _ticks;
	} _softcalls[ MaxSoftCalls ];

	Lock _lock;
	int _hz;
	int _usecsPerTick;
	SoftCall _todo;
	SoftCall * _free;

	static int hardClockInterrupt();
	void softClockInterrupt(char*, int);
	
	void start();
  friend void doSoftint(char *a);
public:
	SoftwareClockManager();
	void add( SoftClockFunc func, char * arg, int ticks );
	int remove( SoftClockFunc func, char * arg );
	int hz() { return _hz; }
	int usecsPerTick() { return _usecsPerTick; }
};

class TimeValue;
extern SoftwareClockManager * TheSoftwareClockManager;
extern TimeValue _theTime;

#endif /* SoftClock_h */
