/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1994
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SoftInt.h
 */

#ifndef SoftInt_h
#define SoftInt_h

#ifdef __GNUC__
#pragma interface
#endif

#include "Lock.h"

typedef void (*SoftCallFunc)(char *);

class SoftwareInterruptManager {
protected:
	static const int MaxSoftCalls = 32;
	Lock _lock;
	struct SoftCall {
		SoftCallFunc _func;
		char * _arg;
		SoftCall * _next;
	} _softcalls[ MaxSoftCalls ];
	SoftCall * _head;
	SoftCall * _free;
	SoftCall * _tail;

	/* defined by subclass must enable software interrupt */

	virtual void enableSoftwareInterrupt() = 0;

	/* called by subclass when software interrupt occurs, shold
	   disable software interrupt and be called with interrupts
	   disabled */

	int softwareInterrupt();
public:
	SoftwareInterruptManager();
	virtual ~SoftwareInterruptManager(){}
	void add( SoftCallFunc func, char * arg );
};

extern SoftwareInterruptManager * TheSoftwareInterruptManager;

#endif /* SoftInt_h */
