/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SystemInterface.h - class for miscellaneous system calls
 */

#ifndef SystemInterface_h
#define SystemInterface_h


#ifdef __GNUC__
#pragma interface
#endif

#include "ProxiableObject.h"
#include "SystemInterfaceStar.h"
#include "NameServerStar.h"
#include "ProcessStar.h"
#include "DiskStar.h"
#include "OutputStreamStar.h"
#include "InputStreamStar.h"

class ExecInfo;
class Process;
class ProvidedVersionInstaller;
class RequiredVersionInstaller;
class PersistentMemoryObject;

extern ClassStar SystemInterfaceClass;
class SystemInterface : public ProxiableObject {
public:
	proxiable virtual void *initialCall( void * returnFunct );
	proxiable virtual void versions
		( ProvidedVersionInstaller *, RequiredVersionInstaller * );
	proxiable virtual Process * getThisProcess();
	proxiable virtual void killThisProcess();
	proxiable virtual Process * buildProcessFromObjectFile
		( PersistentMemoryObject *, int argc, char ** argv );
	proxiable virtual DiskRef rootDisk();
	proxiable virtual int execFromObjectFile
		( PersistentMemoryObject *, int, char **, ExecInfo * );
	proxiable virtual int isObjectFile( PersistentMemoryObject * );
	proxiable virtual OutputStreamRef outputStream( ProxiableObject * );
	proxiable virtual InputStreamRef inputStream( ProxiableObject * );
	proxiable virtual int pageSize();
	proxiable virtual void controlSimpleInstruments
		( int argc, char **argv );
	Class * classOf();
};

extern SystemInterfaceStar TheSystemInterface;

extern SystemInterface * StandardSystemInterface;

#endif /*SystemInterface_h*/
