/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: John L. Coolidge (coolidge@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifndef SystemTime_h
#define SystemTime_h


#ifdef __GNUC__
#pragma interface
#endif

#include "MachineTypes.h"

const int64 MICROSECOND_TO_SECOND = 1000000;	// Number of us in a second
const int64 SYSTEM_TO_SECOND = 10000000;	// Number of 100ns in a second
const int64 TIMEZONE_TO_SYSTEM = 0x61c46800;	// Number of 100ns in an hour
const int64 SYSTEM_SEC_TO_UNIX_SEC = 0xd1044080;// Number of seconds between
						// Nov 17, 1858 and Jan 1, 1970
inline int
LeapYear( int year )
{
    return( ( (year % 4 == 0) && (year % 100 != 0)) || (year % 400 == 0) );
}

inline int
DaysPerYear( int year )
{
    return( ( LeapYear( year ) )? 366 : 365 );
}

inline int64
convertfromunix( const int64& time )
{
    return( (time + SYSTEM_SEC_TO_UNIX_SEC) * SYSTEM_TO_SECOND );
}

class UnixTime;

class SystemTime
{
  private:
    int64 _time;		// Time in 100ns since Nov 17, 1858
    int64 _timeZoneOffset;	// Time zone offset (same format as above)
    int8  _dst;			// Daylight savings time flag (see Unix)

  public:
    SystemTime( )
      { }
    SystemTime( int64 time, int64 timeZoneOffset, int8 dst )
      { _time = time; _timeZoneOffset = timeZoneOffset; _dst = dst; }

    UnixTime unixTime();

    int64 time()
      { return _time; }
    int64 timeZoneOffset()
      { return _timeZoneOffset; }
    int8 dst()
      { return _dst; }

    void setTime( int64 time )
      { _time = time; }
    void setTimeZoneOffset( int64 timeZoneOffset )
      { _timeZoneOffset = timeZoneOffset; }
    void setDst( int8 dst )
      { _dst = dst; }
};

#endif /* SystemTime_h */
