/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Contributing Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Contributing Author: David Dykstra (dykstra@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * TimeoutTimer.h - global Timeout timer.
 */

#ifndef TimeoutTimer_h
#define TimeoutTimer_h


#ifdef __GNUC__
#pragma interface
#endif

#define ONELock

#include "Timer.h"	/* parent class */

class Lock;

#include "ClassStar.h"
#include "SemaphoreStar.h"

class Semaphore;	// forward reference...

const int WHEELHASHPOWER = 10;
const int WHEELHASHSIZE = 1024;	/* WHEELHASHSIZE = 2 ^ WHEELHASHPOWER */
const unsigned int WHEELHASHMASK = WHEELHASHSIZE - 1;


extern ClassStar TimeoutTimerClass;
class TimeoutTimer : public Timer {
protected:
	/*
	 * This is the per-instance data and protected functions
	 */
	SemaphoreStar timeoutSemaphore; /* semaphore to V when expires	*/
	SemaphoreStar raceSemaphore;    /* semaphore for stop/expire races */
	char running;		/* timer running flag			*/
	char expiring;		/* timer expiring flag			*/
	char waiting;		/* timer waiting to be stopped flag	*/
	int _tableIndex;	/* index into timeout table		*/
	unsigned int tickCount;	/* upper bits of count left in timer	*/
	unsigned int expireTick; /* tickTime of expiration		*/
	TimeoutTimer *prev;	/* linked list backward pointer		*/
	TimeoutTimer *next;	/* linked list forward pointer		*/

	void waitExpire();

	/*
	 * Now for the class data and functions
	 */
	static TimeoutTimer *table[WHEELHASHSIZE];
	static unsigned int tickTime;
	static int index();
#ifdef ONELock
	static Lock *lock;
#else
	static Lock *lock[WHEELHASHSIZE+1];
#endif
	static void acquireLock(int);
	static void releaseLock(int);

	static TimeoutTimer ** bucket(int indx);
	static int hashIndex( unsigned int index );
	static int hashRemainder( unsigned int index );
	static int hashValue( unsigned int index );

	static unsigned int timeResidual( TimeoutTimer * ptr );

public:

	static void initialize();

	static int perTickBookKeeping();
	static void expiryProcessing();

	/* 
	 * Now back to regular public functions
	 */

	proxiable TimeoutTimer();
	proxiable ~TimeoutTimer();

	proxiable unsigned int start( unsigned int microseconds );
	proxiable unsigned int stop();
	proxiable unsigned int residual();
	proxiable void await();

	Class *classOf();
};

#endif /* TimeoutTimer_h */
