/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Contributing Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Contributing Author: David Dykstra (dykstra@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * Timer.h - Timer class; just a place holder for subclasses.
 */

#ifndef Timer_h
#define Timer_h


#ifdef __GNUC__
#pragma interface
#endif

#include "ProxiableObject.h"	/* parent class */


extern ClassStar TimerClass;
class Timer : public ProxiableObject {
public:
	/*
	 * "start" starts timer and returns actual number of microseconds set,
	 *     possibly rounded up to the granularity of the hardware timer.
	 */
	proxiable virtual unsigned int start( unsigned int microseconds ) = 0;
	/*
	 * "stop" returns the residual number of microseconds after stopping.
	 */
	proxiable virtual unsigned int stop() = 0;
	/*
	 * "residual" returns the remaining number of microseconds.
	 */
	proxiable virtual unsigned int residual() = 0;
	/*
	 * "await" waits for timer to expire.
	 */
	proxiable virtual void await();

#if 0
	/*
	 * "tick" advances the timer for timers which don't automatically
	 * advance.  Timers which DO advance automatically can just
	 * inherit the null method defined here.
	 */
	virtual void tick();
#endif

	Class * classOf();
};

#endif /* Timer_h */
