/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * TimerManager.h - machine independent clock manager
 */

#ifndef TimerManager_h
#define TimerManager_h


#ifdef __GNUC__
#pragma interface
#endif

/* #define TIMERMANAGERDEBUG */

#include "Assert.h"
#include "Interrupt.h"
#include "Lock.h"
#include "Object.h"
#include "OutputStream.h"
#include "SemaphoreStar.h"
#include "Types.h"


class TimerManager: public Object {
private:
	Lock busyLock;
	int busyFlag;
	SemaphoreStar semaphore;
	PFI perTickBookKeeping;
	PFV expiryProcessing;

	unsigned int lastFreeRunningTime;

protected:
	int microsecondsPerTick;

public:
	TimerManager(PFI, PFV);
	unsigned int microsecondsToTicks( unsigned int microseconds );
	unsigned int ticksToMicroseconds( unsigned int ticks );
	void interruptHandler();
	void processHandler();

	virtual void initialize();  /* subclass may redefine to start up clock */
	/*
	 * setFreeRunningTime is only for machine-dependent subclass to call.
	 *  It should be protected but it is not allowed for a subclass
	 *  to access protected data through a pointer; it can only access
	 *  protected data in its own instance.
	 */
	void setFreeRunningTime(unsigned int);
};

inline unsigned int
TimerManager::microsecondsToTicks( unsigned int microseconds )
{
	/* Round up to the nearest number of ticks */
	return((microseconds+microsecondsPerTick-1)/microsecondsPerTick);
}

inline unsigned int
TimerManager::ticksToMicroseconds( unsigned int ticks )
{
	return(ticks * microsecondsPerTick);
}

inline void
TimerManager::initialize()
{
	/* if the subclass does not redefine, does nothing */
}

inline void
TimerManager::setFreeRunningTime(unsigned int t)
{
	lastFreeRunningTime = t;
}

class TimerManagerInterrupt : public Interrupt {
protected:
public:
	/* must be defined by machine dependent portion of timer manager */
	void basicRaise( char * );
};

#endif /* TimerManager_h */
