/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * TimingInterface.h - class for timing system calls
 */

#ifndef TimingInterface_h
#define TimingInterface_h


#ifdef __GNUC__
#pragma interface
#endif

#include "ProxiableObject.h"
#include "TimingInterfaceStar.h"

extern ClassStar TimingInterfaceClass;
proxiable class TimingInterface : public ProxiableObject {
protected:
	TimingInterfaceStar ointfStar;
	TimingInterface * ointf;
	TimingInterfaceStar anotherStar;
	TimingInterface * another;
	TimingInterfaceStar yetanotherStar;
	TimingInterface * yetanother;
public:
	proxiable TimingInterface();
	proxiable TimingInterface( int );
	proxiable ~TimingInterface();
	proxiable virtual void nullCall();
	proxiable virtual void intParam( int );
	proxiable virtual void twoIntParams( int, int );
	proxiable virtual void ptrParam( char * );
	proxiable virtual void proxyParam( TimingInterface * );
	proxiable virtual TimingInterfaceRef returnOldProxy();
	proxiable virtual TimingInterfaceRef returnNewProxy();
	proxiable virtual TimingInterface * returnOldProxyAsterisk();
	proxiable virtual TimingInterface * returnNewProxyAsterisk();

	proxiable virtual void setOutCallInterface( TimingInterface * );
	proxiable virtual void setOutProxyInterface( TimingInterface * );
	/*
	 * The following time outcalls.  The subclass of TimingInterface
	 *  set by setOutCallInterface is called the number of tries passed
	 *  in as the parameter.  The average elapsed time in microseconds is
	 *  returned.
	 */
	proxiable virtual unsigned int outNullCalls( unsigned );
	proxiable virtual unsigned int outIntParams( unsigned );
	proxiable virtual unsigned int outTwoIntParams( unsigned );
	proxiable virtual unsigned int outPtrParams( unsigned );
	proxiable virtual unsigned int outProxyParams( unsigned, unsigned );
	proxiable virtual unsigned int outRefUnref( unsigned );
	proxiable virtual unsigned int outReturnOldProxyAsterisks( unsigned );
	proxiable virtual unsigned int outReturnNewProxyAsterisks( unsigned );

	Class * classOf();
};

#endif /*TimingInterface_h*/
