/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Francis David (fdavid@uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * WatchdogTimer.h - WatchdogTimer class; just a place holder for subclasses.
 */

#ifndef WatchdogTimer_h
#define WatchdogTimer_h


#ifdef __GNUC__
#pragma interface
#endif

#include "ProxiableObject.h"	/* parent class */

extern ClassStar WatchdogTimerClass;
class WatchdogTimer : public ProxiableObject {
public:
	/*
	 * "start" starts timer and returns actual number of seconds set,
	 *  possibly rounded up to the granularity of the hardware timer.
	 */
	proxiable virtual unsigned start( unsigned int seconds ) = 0;
	/*
	 * "stop" returns the residual number of seconds after stopping.
	 */
	proxiable virtual unsigned stop() = 0;
	/*
	 * "residual" returns the remaining number of seconds.
	 */
	proxiable virtual unsigned residual() = 0;
    /*
     * Pings resets the timer on the watchdog
     */
    proxiable virtual void ping() = 0;

	Class * classOf();
};

#endif /* WatchdogTimer_h */
