#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * InstrumentBuffer.cc: buffer for use by system instrument manager
 */

#ifdef INSTRUMENT
#include "Assert.h"
#include "OutputStream.h"
#include "InstrumentBuffer.h"
#include "InstrumentEvent.h"


#ifdef __GNUC__
#pragma implementation
#endif

char * MemoryCopy (char *dest, const char *source, unsigned int length);

/*
Buffer constructor.
*/
InstrumentBuffer::InstrumentBuffer (unsigned int bufferAlloc,
				    unsigned long cpuNumber)
{

	bufferIdentifierType cpuBufferID;

	cpuBufferID.dumpTime = SystemTimer->time();
	cpuBufferID.id.special = FALSE;
	cpuBufferID.id.count = FALSE;
	cpuBufferID.id.time = FALSE;
	cpuBufferID.id.message = TRUE;
	cpuBufferID.id.percpu = TRUE;
	cpuBufferID.id.perprocess = FALSE;
	cpuBufferID.id.ownerid = 0;
	cpuBufferID.id.selfid = 0;
	cpuBufferID.id.mode = SYSTEM_INSTRUMENT;
	cpuBufferID.eventID = InstrumentBufferBirth;
	cpuBufferID.cpuLength = sizeof (cpuNumber);
	cpuBufferID.cpu = cpuNumber;
	
	// total buffer size
	_mxSz = bufferAlloc;

	// _size is the number of bytes available for non-buffer instrument
	// events.  The very first event records is reserved for use by this
	// buffer for logging dump times.
	_size = sizeof (bufferIdentifierType);

	Debug << this << "::InstrumentBuffer() _mxSz "<< _mxSz << "\n" << eor;

	_buffer = new char[bufferAlloc];

	// the following adds the first event in the buffer - this event is
	// updated as the buffer is dumped to disk.  This *always* remains the
	// first event in the buffer.

	// had to split up the memcopy to insert eventID - should go
	// cpuBufferID once things are verified
	MemoryCopy (_buffer, (const char *) (&cpuBufferID),
		    sizeof (cpuBufferID));

}

/*
Buffer destructor.
*/
InstrumentBuffer::~InstrumentBuffer ()
{
	Debug << this << "::~InstrumentBuffer()\n" << eor;

	delete _buffer;
}

char * 
MemoryCopy (char *dest, const char *source, unsigned int length)
{
        // copy length number of bytes from source to destination

	for ( ; length != 0; --length)
		*(dest++) = *(source++);
	return dest;
};

char * 
InstrumentBuffer::getBufferLocation ()
{
        // return the location of the actual buffer

	return (_buffer);
};

unsigned int 
InstrumentBuffer::getBufferSize ()
{
        // return the USEABLE SIZE of the buffer - useable means the buffer
        // which can be used for event logging other than recording the buffer
        // creation/clear event 
  
	return (_size);
};

unsigned int 
InstrumentBuffer::getBufferMaxSize ()
{
        // return the total size of the buffer

	return (_mxSz);
};

unsigned int 
InstrumentBuffer::getBufferSizeLeft ()
{
        // return the useable number of bytes left in the buffer

	return (_mxSz - _size);
};

void 
InstrumentBuffer::clear ()
{
        // clear the buffer for writing future events and reset the size to
	// the number of bytes useable for logging events
        // Note that the dump time is the current time

	bufferIdentifierType *buffer = (bufferIdentifierType *) _buffer;

	_size = sizeof (bufferIdentifierType);		// reset logging size
	buffer->dumpTime = SystemTimer -> time();	// time following dump
	buffer->eventID = InstrumentBufferClear;	// update event ID

};

void 
InstrumentBuffer::timeStamp ()
{
        // append the curent time to the buffer

	Assert (_size + sizeof (timeType) <= _mxSz);

	timeType t = SystemTimer -> time();
	MemoryCopy ((char *) (_buffer + _size), (char *)&t, sizeof(timeType));
	_size += sizeof (timeType);
};

/*
Put the specified data in the buffer and update the size.
*/
void 
InstrumentBuffer::append (unsigned int dataLength, char *data)
{
        // append data of size datalength to the buffer and update the bumber
	// of bytes left in the buffer
  
	Assert ((_size + dataLength) <= _mxSz);

	MemoryCopy ((char *) (_buffer + _size), data, dataLength);
	_size += dataLength;
};

#endif INSTRUMENT
