#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * InstrumentManager.cc: basic instrument manager
 */

#ifdef INSTRUMENT

#include "Assert.h"
#include "String.h"
#include "InstrumentManager.h"
#include "InstrumentEvent.h"


#ifdef __GNUC__
#pragma implementation
#endif

/*
The constructor.
*/
InstrumentManager::InstrumentManager (char *title,
				      unsigned int instrumentAlloc)				  
{
	// Check for valid range!!! (What IS valid???)

	_title = new char[strlen (title) +1];
	strcpy (_title, title);
	_instrumentList = new Instrument *[instrumentAlloc];
	_maxInstruments = instrumentAlloc;
	_numInstruments = 0;
	_header.id.special = FALSE;
	_header.id.count = FALSE;
	_header.id.time = FALSE;
	_header.id.message = FALSE;
	_header.id.percpu = FALSE;
	_header.id.perprocess = FALSE;
	_header.id.selfid = 0;
}

/*
The destructor.
All the interesting work is done in the subclass destructors.
*/
InstrumentManager::~InstrumentManager ()
{
	delete _instrumentList;	
}

void 
InstrumentManager::birthLog (char *title)
{
	_header.id.special = TRUE;
	_header.id.message = TRUE;
	writeLogBuffer (sizeof(_header),
			(char *)&_header,
			InstrumentBirth,
			strlen (title),
			title);
	_header.id.special = FALSE;
}

void 
InstrumentManager::deathLog ()
{
	logAllInstruments ();
}

/*
return SYSTEM_INSTRUMENT if in system mode or USER_INSTRUMENT if created
by user
*/
iModeType 
InstrumentManager::getMode ()
{
  return (_header.id.mode);

}	// InstrumentManager::getMode ()

/*
Assign a unique instrument ID which is used when sending to the
log.  It allows a shorthand representation of events in the log
(which will be expanded by the dumpfile-viewing application).
Somehow, this information must get to the other processors for
system instruments on non-shared-memory machines.
*/
unsigned int 
InstrumentManager::addInstrument (Instrument *iObject)
{
	if (_numInstruments < _maxInstruments) {
		_instrumentList[_numInstruments++] = iObject;

		return (_numInstruments);
	}
	else {
		register unsigned int i;

		// Find an unused entry
		for (i = 0;
		     ((i < _maxInstruments) &&
		      (_instrumentList[i] != (Instrument *) NULL));
		     i++)
			;

		// If all entries are used
		if (i == _maxInstruments) {
			return (FALSE);
		}
		// We found an entry
		else {
			_instrumentList[i] = iObject;

			return (i);
		}
	}
}

/*
Remove our pointer to it from the list.
*/
void 
InstrumentManager::removeInstrument (Instrument *iObject)
{
	if (_instrumentList == NULL) {
	}
	else {
		_instrumentList[(iObject->getID ()) - 1] = NULL;
	}
}

/*
For every instrument of this instrumentation, log it.
*/
void 
InstrumentManager::logAllInstruments ()
{
	register Instrument **i = _instrumentList;
	
	for ( ; i != (_instrumentList + _numInstruments); i++) {
		if ((*i) != NULL)
			(*i)->deathLog ();
	}
}

void 
InstrumentManager::resetAllInstruments ()
{
	register Instrument **i = _instrumentList;
	
	for ( ; i != (_instrumentList + _numInstruments); i++) {
		if ((*i) != NULL)
			(*i)->reset ();
	}
}

void 
InstrumentManager::suspendAllInstruments ()
{
	register Instrument **i = _instrumentList;
	
	for ( ; i != (_instrumentList + _numInstruments); i++) {
		if ((*i) != NULL)
			(*i)->suspend ();
	}
}

void 
InstrumentManager::resumeAllInstruments ()
{
	register Instrument **i = _instrumentList;
	
	for ( ; i != (_instrumentList + _numInstruments); i++) {
		if ((*i) != NULL)
			(*i)->resume ();
	}
}


Boolean 
InstrumentManager :: suspendNewInstruments ()
{
	return (_suspendNewInstruments);
}


/*
Return the instrumentation id number.
*/
unsigned 
int InstrumentManager :: getID ()
{
	return (_header.id.ownerid);
}

#include "ClassInstaller.h"

DEFINECLASS("InstrumentManager",InstrumentManager,ProxiableObject,0,0);

#endif INSTRUMENT
