#ifdef __GNUC__
#pragma implementation
#endif


#include "Assert.h"
#include "Class.h"
#include "ClassStar.h"
#include "NameServer.h"
#include "SimpleInstruments.h"
#include "FreeRunningTimer.h"
#include "FreeRunningTimerStar.h"
#include "PROCESSOR.h"
#include "OutputStream.h"
#include "Process.h"
#include "String.h"
#include "ThisProcess.h"

#ifdef __GNUC__
#pragma implementation
#endif

int SimpleTimerUseProcessTime = 1;
int SimpleInstrumentVerbose = 0;

extern FreeRunningTimerStar SystemTimer;

void
activateSimpleInstrument( ProxiableObject * obj)
{
	if (!obj->isKindOf(SimpleInstrumentClass)) {
		Console() << obj << " is not a SimpleInstrument\n";
		return;
	}
	SimpleInstrument *si = (SimpleInstrument*)obj;
	if (SimpleInstrumentVerbose)
		Console() << obj << "\n";
	si->activate();
}

void
deactivateSimpleInstrument( ProxiableObject * obj)
{
	if (!obj->isKindOf(SimpleInstrumentClass)) {
		Console() << obj << " is not a SimpleInstrument\n";
		return;
	}
	SimpleInstrument *si = (SimpleInstrument*)obj;
	if (SimpleInstrumentVerbose)
		Console() << obj << "\n";
	si->deactivate();
}

void
resetSimpleInstrument( ProxiableObject * obj)
{
	if (!obj->isKindOf(SimpleInstrumentClass)) {
		Console() << obj << " is not a SimpleInstrument\n";
		return;
	}
	SimpleInstrument *si = (SimpleInstrument*)obj;
	if (SimpleInstrumentVerbose)
		Console() << obj << "\n";
	si->reset();
}

void
displaySimpleInstrument( ProxiableObject * obj)
{
	if (!obj->isKindOf(SimpleInstrumentClass)) {
		Console() << obj << " is not a SimpleInstrument\n";
		return;
	}
	SimpleInstrument *si = (SimpleInstrument*)obj;
	si->displayOnConsole();
}

/*
 * Support for "inst [k|m] [Class] [on|off|d|r]
 */
void
simpleInstrumentControl( int argc, char *argv[])
{
	int result = 0;
	if (argc != 4){
		Console() << "Usage: instrument [member|kind] [Class ][on|off|display]\n" << eor;
		return;
		}
	ClassStar cl = StandardNameServer->lookup(argv[2], ClassClass);
	int on = (strcmp(argv[3], "on") == 0);
	int display = (strcmp(argv[3], "d") == 0);
	int reset = (strcmp(argv[3], "r") == 0);
	int member = (strcmp(argv[1], "m") == 0);
	ClassApplicationFunction func ; 
	if (on) 
		func = &activateSimpleInstrument;
	else if (display)
		func = &displaySimpleInstrument; 
	else if (reset)
		func = &resetSimpleInstrument;
	else 
		func = &deactivateSimpleInstrument;
	if (cl){
		if (member)
			result = cl->applyToMembers(func);
		else
			result = cl->applyToKindred(func);
		}
	Console() << result << " instruments " << (display ? "displayed" : (on ? "enabled" : (reset ? "reset" : "disabled")))
		<< "\n" << eor;
}

unsigned 
SimpleTimer::basicStart()
{
	if (SimpleTimerUseProcessTime) {
		return (unsigned) (thisProcess()->processTime());
	} else {
		return (unsigned) (SystemTimer->time());
	}
}

unsigned
SimpleTimer::basicStop(const unsigned start)
{
	Assert ( start != 0) ;
	unsigned startTime = start;
	unsigned endTime;	
	if (SimpleTimerUseProcessTime) {
		endTime = thisProcess()->processTime(); 
	} else {
		endTime = SystemTimer->time();
	}
	unsigned time = endTime - startTime;
	Assert( time < 0x80000000 );
	lock();
	_totalTime += time;
	if (time < _min){
		_min = time;
		_minSample = _count;
	}
	if (time > _max) {
		_max = time;
		_maxSample = _count;
	}
	_count++;
	unlock();
	return time;
}

unsigned
SimpleTimer::basicSuspend(const unsigned start)
{	
	Assert ( start != 0) ;
	unsigned startTime = ( unsigned int ) start;
	unsigned endTime;	
	if (SimpleTimerUseProcessTime) {
		endTime = thisProcess()->processTime(); 
	} else {
		endTime = SystemTimer->time();
	}
	unsigned time = endTime - startTime;
	Assert( time < 0x80000000 );
	return time;
}

unsigned
SimpleTimer::basicResume(const unsigned timeUsed)
{
	Assert ( start != 0) ;
	unsigned timeNow;
	if (SimpleTimerUseProcessTime) {
		timeNow = thisProcess()->processTime(); 
	} else {
		timeNow = SystemTimer->time();
	}
	unsigned fakeStartTime = timeNow - timeUsed;
	return fakeStartTime;
}

void
SimpleTimer::displayOnConsole()
{
	if (count() == 0 && !SimpleInstrumentVerbose)
		return;

	Console() << this ;
	if (active()) {
		if (count() == 0)
			Console() << ": 0 samples" ;
		else
			Console() << ": \n" << "\t" <<
				count() << " timings, min = " << min() <<
				"(sample " << minSample() << ") " <<
				" max = " << max() << 
				"(sample " << maxSample() << ") " <<
				" avg = " << avg() << 
				" total = " << _totalTime ;
	}
	else Console() << ": inactive" ;
	Console() << "\n" << eor;
}

#ifdef NEVER
void
SimpleTimer::writeName( OutputStream *s)
{
	ProxiableObject::writeName(s);
	if (active()) {
		if ( count() == 0)
			*s << ": 0 samples" ;
		else
			*s << ": \n" << "\t" <<
				count() << " timings, min = " << min() <<
				"(sample " << minSample() << ") " <<
				" max = " << max() << 
				"(sample " << maxSample() << ") " <<
				" avg = " << avg() << 
				" total = " << _totalTime ;
	}
	else *s << ": inactive" ;
}
#endif

#include "ClassInstaller.h"
DEFINECLASS("SimpleInstrument",SimpleInstrument,ProxiableObject,0,0);
DEFINECLASS("SimpleEventCounter",SimpleEventCounter,SimpleInstrument,0,0);
DEFINECLASS("SimpleTimer",SimpleTimer,SimpleInstrument,0,0);

DEFINECLASS("MemoryTimer",MemoryTimer,SimpleTimer,0,0);
DEFINECLASS("ProcessTimer",ProcessTimer,SimpleTimer,0,0);
DEFINECLASS("FileSystemTimer",FileSystemTimer,SimpleTimer,0,0);
