#ifdef __GNUC__
#pragma implementation
#endif

#include "Class.h"
#include "CPU.h"
#include "FormatAssists.h"
#include "OutputStream.h"
#include "Process.h"
#include "PROCESSOR_CONTEXT.h"
#include "String.h"
#include "ThisProcess.h"
#include "ThisCPU.h"
#include "Assert.h"
#include "Raid.h"

#ifdef ASSERT
AssertionFailure::AssertionFailure(const char *exp , char * file, char *baseFile, int line)
{
#ifdef RAIDOUTPUTSTREAM
	RaidStream().write("Assert failed\n",14);
#endif
	Console().write("Assert failed\n",14);

	if( ! strcmp( file, baseFile ) ) {
#ifdef RAIDOUTPUTSTREAM
	        RaidStream() << "Assertion (" << exp << ") failed\n\tin "
		    << file << ", line " << line << "\n" << eor;
#endif
	        Console() << "Assertion (" << exp << ") failed\n\tin "
		    << file << ", line " << line << "\n" << eor;
	} else {
#ifdef RAIDOUTPUTSTREAM
		RaidStream() << "Assertion (" << exp << ") failed\n\tin " << 
			file 
                     << " (#include'd in " << baseFile << 
			 "), line " << line << "\n" << eor;
#endif
		Console() << "Assertion (" << exp << ") failed\n\tin " << 
			file 
                     << " (#include'd in " << baseFile << 
			 "), line " << line << "\n" << eor;
	}
	Console().flush();
}

AssertionFailure::~AssertionFailure()
{
	extern void Halt();
	Console() << "Debug Trap\n" << eor;
	CPU *cpu = thisCPU();
	if (cpu) cpu->debugBreakpoint();
	Process *p = thisProcess();
        if (p){((PROCESSOR_CONTEXT *)p->context())->traceStack();}

	/*
	 * If the process is an application process then kill it and
	 * continue, otherwise we are in trouble.
	 */
#ifdef RAIDOUTPUTSTREAM
	extern void RaidDump();
	RaidDump();
#endif
	if (p != 0) {
		if ( ((Object*)p)->isKindOf( ApplicationProcessClass )) {
			p->die();
			}
		else Halt();
		}
	else Halt();
}

AssertionFailure&
AssertionFailure::operator()(const char * format, const formatArgs & a1,
		const formatArgs & a2, const formatArgs & a3,
		const formatArgs & a4)
{
	Console().formattedWrite( format, a1, a2, a3, a4);
	Console() << "\n" << eor;
	return *this;
}

/**
 * Create an assertion failed exception.
 *
 * @param exp Assetion clause.
 * @param file File where exception occured.
 * @param baseFile File that included file where exception occured.
 * @param line Line number where exception occured.
 */
AssertionException::AssertionException(const char *exp, const char *file,
    const char *baseFile, int line)
{
  _msg = new char[strlen(exp) + strlen(file) + strlen(baseFile) + 120];

  strcpy(_msg, "Assert failed\n");

  if(strcmp(file, baseFile)==0) /* strings are equal */ {
    strcat(_msg, "Assertion (");
    strcat(_msg, exp);
    strcat(_msg, " ) failed\n\tin ");
    strcat(_msg, file);
    strcat(_msg, ", line ");
    strcat(_msg, "(ha ha ha)");
    strcat(_msg, "\n");
  } else {
    strcat(_msg, "Assertion (");
    strcat(_msg, exp);
    strcat(_msg, " ) failed\n\tin ");
    strcat(_msg, file);
    strcat(_msg, " (#include'd in ");
    strcat(_msg, baseFile);
    strcat(_msg, "), line ");
    strcat(_msg, "(ha ha ha)");
    strcat(_msg, "\n");
  }
}


#endif /* ASSERT */
