#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * Async.cc: asynchronous support class stubs.
 */

//#define DEBUG
//#define NOASYNC
const unsigned ASYNCSUPPORTWARNING  = 6;

#include "Assert.h"
#include "Async.h"
#include "Kernel.h"
#include "Process.h"
#include "String.h"
#include "ThisProcess.h"

#ifdef __GNUC__
#pragma implementation
#endif

AsyncClosure::AsyncClosure()
{
  _func = &doNothing;
  _next = 0;
  _prev = 0;
  _urgency = 0;
}

AsyncSupport * TheAsyncSupport = 0;

AsyncSupport::AsyncSupport( unsigned concurrency )
	: _concurrency( concurrency )
{
#ifdef NOASYNC
	return;
#endif // NOSYNC
	Assert( _concurrency > 0 );
	Debug << "AsyncSupport[" << asHex( (unsigned) this )
		  << "] starting " << concurrency << " daemons\n" << eor;
	acquire();
	_waiting = 0;
	_active = 0;
	_idling = 0;
	_head = 0;
	_tail = 0;
	for( unsigned i = 0; i < concurrency; i++ ) {
		Process * p = new SystemProcess
			( (PFV) daemon, TheKernel->domain(),
			  small, (int) this );
		p->reference();
		p->ready();
	}
	release();
	thisProcess()->relinquishProcessor();
}

#ifdef ASYNCSUPPORTWARNING
int AsyncSupportWarning = ASYNCSUPPORTWARNING;
#endif // ASYNCSUPPORTWARNING

void 
AsyncSupport::schedule( AsyncClosure * c )
{
#ifdef NOASYNC
	(*(c->_func))( c );
	return;
#endif // NOASYNC
        IntStorage intState ; PROCESSOR::disableInterrupts(intState);
	acquire();
	Debug 	<< this << "::schedule( " << asAddress( c ) << " ) _idling "
		<< _idling << "\n" << eor;
	if ( _idling > 0 ) {
		_idling--;
		_active++;
		Process * p = _waitList.removeHead();
		((AsyncBlockedInfo *) p->blockedInfo())->setAsyncClosure( c );
		release();
		if ( c->_urgency > 0 ) {
			Process * process = thisProcess();
			Debug 	<< this << "::schedule() ready " << process 
				<< " giveProcessorTo " << p << "\n" << eor;
			process->ready();
                        process->giveProcessorTo( p, ProcessReady );
		} else {
			Debug 	<< this << "::schedule() ready " << p
				<< "\n" << eor;
			p->ready();
		}
	} else {
		_waiting++;
#ifdef ASYNCSUPPORTWARNING
		if ( _waiting > AsyncSupportWarning ) {
			Console()
		 		<< this << "::schedule() waiting "
				<< _waiting << " adding after current _tail " 
				<< asAddress( _tail ) << "\n" << eor;
			AsyncSupportWarning = _waiting;
		}
#endif // ASYNCSUPPORTWARNING
		if ( _tail ) {
			_tail->_next = c;
			c->_prev = _tail;
			c->_next = 0;
			_tail = c;
		} else {
			Assert( _waiting == 1 );
			_tail = _head = c;
			c->_next = c->_prev = 0;
		}
		release();
	}
	PROCESSOR::restoreInterrupts( intState );
}

void
AsyncSupport::daemon( int arg )
{
	static char * generalName = "#AsyncSupportDaemon-";

	AsyncSupport * async = (AsyncSupport *) arg;
	Process * me = thisProcess();

        IntStorage intState ; PROCESSOR::disableInterrupts(intState);
	async->acquire();
	unsigned myId = async->_idling; 
	char code[3];
	code[0] = ( myId / 10 ) + '0';
	code[1] = ( myId % 10 ) + '0';
	code[2] = 0;
	char * myName = new char[ strlen( generalName ) + 3 ];
	strcpy( myName, generalName );
	strcat( myName, code );
	me->setName( myName );
	if ( myId == async->_concurrency - 1 ) {
		Debug << "  " << code << " done\n" << eor;
	} else {
		Debug << "  " << code << eor;
	}
	delete myName;

	async->_idling++;
	async->_waitList.addTail( me );
	async->release();
        me->block( async );
        PROCESSOR::restoreInterrupts( intState );

	// main loop

	AsyncClosure * myClosure = 0;
	while(1) {

		if ( myClosure == 0 ) 
 		    	myClosure = ((AsyncBlockedInfo *) me->blockedInfo())
				    ->asyncClosure();
		Debug 	<< me << ": calling " << asAddress( (void*)myClosure->_func )
		      	<< " with closure " << asAddress( myClosure )
			<< "\n" << eor;
#ifdef PROCESSBLOCKTRAP
		me->setProcessEvent( (unsigned) TheAsyncSupport );
#endif // PROCESSBLOCKTRAP
		(*(myClosure->_func))( myClosure );
#ifdef PROCESSBLOCKTRAP
		me->setProcessEvent( 0 );
#endif // PROCESSBLOCKTRAP
		Debug 	<< me << ": done\n" << eor;
        	IntStorage intState ; PROCESSOR::disableInterrupts(intState);
		async->acquire();
		Debug 	<< me << ": _waiting " << async->_waiting 
			<< "\n" << eor;
		if ( async->_waiting > 0 ) {
			Assert( async->_idling == 0 );
			async->_waiting--;
			myClosure = async->_head;
			async->_head = async->_head->_next;
			if ( async->_head == 0 ) {
				async->_tail = 0;
			} else {
				async->_head->_prev = 0;
			}	
			async->release();
		} else {
			async->_idling++;
			async->_active--;
			async->_waitList.addHead( me );
			async->release();
        		me->block( async );
			myClosure = 0;
		}
        	PROCESSOR::restoreInterrupts( intState );
	}

	Assert( NOTREACHED );
}

#include "ClassInstaller.h"
DEFINECLASS("AsyncSupport",AsyncSupport,Object,0,0);
