/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *     University of Illinois at Urbana-Champaign
 *     Department of Computer Science
 *     1304 W. Springfield Ave.
 *     Urbana, IL 61801
 *
 * Copyright (c) 1987-2005
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 */

#include "NewAndDelete.h"
#include "OutputStream.h"
#include "CharacterInputStream.h"
#include "String.h"
#include "KernelCmdInstaller.h"
#include "ThisCPU.h"

#ifdef __GNUC__
#pragma implementation
#endif
#include "BootConsole.h"

BootConsole::~BootConsole()
{
}

void BootConsole::printMenu() {
    Console() << "\nMenu:\n"
              << "reset - reset back to bootloader\n"
              << "c - continue boot process\n"
              << "? - list custom boot commands\n"
              << "about - about this build of Choices\nchoices:\\> " << eor;
}

int BootConsole::readBuffer(unsigned len, char* buf, unsigned timeout)
{
    unsigned int i = 0, res = 1;
    char c;
    do {
        if( !timeout ) { 
            c = getChar();
        } else {
            while( (timeout-- > 0) && res ) {
                res = getRawChar(&c);
            }
            
            if( res ) {
                return res;
            } else {
                buf[0] = 0;
                return 0;
            }
        }
        
        buf[i++] = (char)c;
        /* if you are getting input showing up twice on your side, make sure
         * input echo is turned off in your terminal emulator. */
        Console() << c << eor;
        /* 0xd, 0xa at newline */
    } while ((c != 0xD)&&(c != 0xA)&&(i < len));
    
    if((c==0xD)||(c==0xA))    
        buf[i-1] = 0;
    
    // 0 means no timeout has occurred
    return 0;
}

void BootConsole::run()
{
    char buf[255];

    while(1) {
        printMenu();

        if( readBuffer(255, buf) ) {
            // timeout
            if( doDefault() )
                break;
        }

        //process command
        if( doCommand(buf) )
            break;
    }
}


bool BootConsole::doCommand(char *cmd) {
    if(strlen(cmd) == 0)
        return 1;

    if( !strcmp(cmd, "reset") ) {
        Console() << "Rebooting...\n" << eor;
        thisCPU()->reset();
    } else if( !strcmp(cmd, "c") ) {
        return 1;
    } else if( !strcmp(cmd, "about") ) {
        Console() << "Choices, see original SRG copyright in boot message\n" << eor;
    } else {
        KernelCmdInstaller* installer = KernelCmdInstaller::getCmdInstaller();
        if(installer == NULL) {
            Console() << "No user commands defined...\n" << eor;
        } else {
            KernelCmd* cmdi = installer->find((char*)cmd);

            if(cmdi == NULL) {
                Console() << "No such user command found\n" << eor;
            } else {
                Console() << "Loading address " << asHex(cmdi) << "\n" << eor;
                //Console() << "Executing '" << cmdi->getName() << ": " << cmdi->getDescription() << "'\n" << eor;
                cmdi->execute();
                //Console() << "Done executing " << cmd << eor;
            }
        }
    }
    return 0;
}
