#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * CPUSet.c: CPU set class implementation.
 */

#include "Assert.h"
#include "CPUSet.h"
#include "OutputStream.h"


#ifdef __GNUC__
#pragma implementation
#endif


int
CPUSet::isEmpty()
{
	return( _count == 0 );
}


int
CPUSet::contains( CPU * cpu )
{
	CpuId id = cpu->id();
	lock.acquire();
	int result = (cpu == _entry[id]);
	lock.release();
	return( result );
}

void
CPUSet::add( CPU * cpu )
{
	CpuId cpuid = cpu->id();
	Debug.printf( "CPUSet%T::add( cpu:%N ) cpuid=%d\n", this,
		cpu, cpuid );
	lock.acquire();
	_entry[ cpuid ] = cpu;
	_count++;
	lock.release();
}

void
CPUSet::remove( CPU * cpu )
{
	CpuId id = cpu->id();
	Debug.printf( "CPUSet%T::remove( id:%d )\n", this, id );
	lock.acquire();
	_entry[ id ] = 0;
	_count--;
	lock.release();
}

CPU *
CPUSet::cpu( CpuId id )
{
	lock.acquire();
	CPU * r = _entry[id];
	lock.release();
	return r;
}

CPU *
CPUSetIterator::next()
{
	Debug.printf( "CPUSetIterator%T::next()\n", this );
	Assert( this != 0 );
	Assert( _mySet != 0 );
	CPU *cpu;

	for( ; _current < MAX_PROCESSORS; _current++ ) {
		cpu = _mySet->cpu( _current );
		if( cpu != 0) {
			Debug.printf( "CPUSetIterator::next: contains %P\n",
				_current ); 
			Assert( cpu != 0 );
			Debug.printf( "CPUSetIterator::next: return %A\n",
				cpu );
			_current++;	// Not a bug, because it returns!
			return( cpu );
		}
	}
	return( 0 );
}


#include "ClassInstaller.h"
DEFINECLASS("CPUSet",CPUSet,Object,0,0);
