#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Contributing Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * Class.cc -
 */
#ifdef CODELOADER
#include "COFFLoader.h"
#endif
#include "Class.h"
#include "ClassConstructor.h"
#include "NameServer.h"
#include "OutputStream.h"
#include "Assert.h"
#include "Raid.h"
#include "String.h"


#ifdef __GNUC__
#pragma implementation
#endif

Class::Class( Class * parent, int size, const char * name,
	      ClassConstructor * ctor  )
:  _parent( parent ), _sibling( 0 ), _child( 0 ) 
{
	_initDebugObjects = 0;
	_initDebugMask = 0;
	_initDebugMessages = 0;
	initDebug();
	Assert( name != 0 ) ( "Class::ctor(): name is zero\n" );
	setName( name );
	_name = new char[ strlen( name ) + 1];
	strcpy( _name, name );
	_sizeOf = size;
	_constructor = ctor;
	if ( ( ctor != 0 ) && ! ctor->isProxy() ) {
	    /* 
	     * to keep it from showing up at shut down; kernel classes
	     *   are not deleted at shut down.
	     */
	    ctor->setName("#classConstructor");
	}
	_proxyTable = (ProxyTableCopy *) 0;
	if( _parent != 0 ) {
		_parent->addChild( this );
	}
	Raids( RaidConstructor ) << this << "::Class( "
				 << parent << " )\n" << eor;
}

Class::~Class()
{
	Raid( RaidConstructor )( "%N::dtor()\n", this);
	delete _name;
}

void
Class::cleanup()
{
	Raid( RaidMethod )( "%N::cleanup\n", this);
	_proxyTable = (ProxyTableCopy *) 0;
	if( _child != 0 ) {
		_child->cleanup();
	}
	if( _sibling != 0 ) {
		_sibling->cleanup();
	}
	Raid( RaidMethod )( "%N::cleanup returns\n", this);
}

void
Class::addChild( Class * child )
{
	Raid( RaidMethod )( "%N::addChild( %N )\n", this, child);
	Assert( child != 0 ) ( "%N::addChild: child is zero\n", this );

	if( _child == 0 ) {
		_child = child;
	}
	else {
		child->setSibling( _child );
		_child = child;
	}
	Raid( RaidMethod )( "%N::addChild( %N ) returns\n", this, child);
}

void
Class::setSibling( Class * sibling )
{
	Raid( RaidMethod )( "%N::setSibling( %N )\n", this, sibling);
	Assert( _sibling == 0 )
		( "%N::setSibling: sibling already set\n", this );
	_sibling = sibling;
	Raid( RaidMethod )( "%N::setSibling( %N ) returns\n", this, sibling);
}

int
Class::isASubclassOf( Class * aClass )
{
	Raid( RaidMethod )( "%N::isASubclassOf( %N )\n", this, aClass);
	Assert( aClass != 0 )
		( "%N::isASubclassOf: aClass is zero\n" );

	int ret = 0;
	Class * me = this;
	Class * r;
	if ( ( ( r = (Class *)(ProxiableObject *) me->_real ) != 0 ))
		me = r;
	else if ( ( ( r = (Class *)(ProxiableObject *) aClass->_real ) != 0))
		aClass = r;

	for( Class * p = me; p != 0; p = p->_parent ) {
		if( p == aClass ) ret = 1;
	}

	Raid( RaidMethod )( "%N::isASubclassOf( %N ) returns ( %d )\n", this, aClass, ret);
	return ret;
}

int
Class::isClass( Class * aClass )
{
	Raid( RaidMethod )( "%N::isClass( %N )\n", this, aClass);
	Assert( aClass != 0 )
		( "%N::isClass: aClass is zero\n" );

	Class * me = this;
	Class *r;

	if ( ( ( r = (Class *)(ProxiableObject *) me->_real ) != 0 ))
		me = r;
	else if ( ( ( r = (Class *)(ProxiableObject *) aClass->_real ) != 0))
		aClass = r;

	int ret =  me == aClass ;
	Raid( RaidMethod )( "%N::isClass( %N ) returns ( %d )\n", 
		this, aClass, ret);
	return ret;
}

int
Class::concrete()
{
	Raid( RaidMethod )( "%N::concrete( %N )\n", this);
        int ret = ( !_sizeOf || (int)(ClassConstructor *)_constructor );
	Raid( RaidMethod )( "%N::concrete( %N ) returns ( %d )\n", this, ret);
	return ret;
}

ProxiableObject *
Class::constructor( ProxiableObject * source )
{
	Raids( RaidMethod ) << this << "::constructor( "
			    << source << ")\n" << eor;

#ifdef CODELOADER
	COFFLoader * loader = 0;
	if( _constructor == 0 ) {
		loader = new COFFLoader( objectName() );
		delete loader;
		Raids( RaidMethod ) << this << "::constructor _constructor "
				    << _constructor << "\n" << eor;
		if( _constructor == 0 ) return( 0 );
	}
#else
	if ( _constructor == 0 ) return( 0 );
#endif

	ProxiableObject * po = _constructor->construct( source );
	Raids( RaidMethod ) << this << "::constructor return( "
			    << po << " )\n" << eor;

	return( po );
}

int
Class::sizeOf()
{
	return( _sizeOf );
}

void
Class::bind( NameServer * ns )
{
	Raid( RaidMethod )( "%N::bind( %N )\n", this, ns);
	Assert( ns != 0 ) ( "%N::bind: ns is zero\n" );

	Assert( _real == 0 ) ( "%N:bind: _real already set\n" );

	_real = ns->lookup( objectName(), "Class" );

	ns->bind( objectName(), this );
	if( _child != 0 )	_child->bind( ns );
	if( _sibling != 0 )	_sibling->bind( ns );
	Raid( RaidMethod )( "%N::bind( %N ) returns\n", this, ns);
}

void
Class::unbind( NameServer * ns )
{
	Raid( RaidMethod )( "%N::unbind( %N )\n", this, ns);
	Assert( ns != 0 ) ( "%N::unbind: ns is zero\n" );

	Assert( _real != 0 ) ( "%N:bind: _real not already set\n" );

	_real = 0;
	ns->unbind( objectName() );
	if( _child != 0 )	_child->unbind( ns );
	if( _sibling != 0 )	_sibling->unbind( ns );
	Raid( RaidMethod )( "%N::unbind( %N ) returns\n", this, ns);
}

const char *
Class::name()
{
	return( _name );
}

char *
Class::name(int max, char *buf)
{
	if ( ( _name == 0 ) || ( buf == 0 ) || ( max == 0 ) )
		return( 0 );

	strncpy( buf, _name, max );
	buf[ max - 1 ] = '\0';

	return( buf );
}

ClassRef
Class::parent()
{
	if( _parent == 0 ) return( 0 );

	return( _parent );
}

ClassRef
Class::child()
{
	if( _child == 0 ) return( 0 );

	return( _child );
}

ClassRef
Class::sibling()
{
	if( _sibling == 0 ) {
		if ( _real == 0 )
			return( 0 );
		ClassStar par =
		    ( ( Class * ) ( ProxiableObject * )_real )->parent();
		if ( par == 0 )
			return( 0 );
		ClassRef chi = par->child();
		return( chi );
	}

	return( _sibling );
}

Class *
Class::real()
{
	if ( _real != 0 )
		return( ( ( Class * ) ( ProxiableObject * )_real )->real() );
	else
		return( this );
}

int
Class::initDebugObjects()
{
	if( _initDebugObjects == 0 ) return( 0 );
	else return( _initDebugObjects-- );
}

int
Class::initDebugMask()
{
	return( _initDebugMask );
}

int
Class::initDebugMessages()
{
	return( _initDebugMessages );
}

void
Class::setInitDebugObjects( int objects )
{
	Raids( RaidMethod ) << this << "::setInitDebugObjects( "
			    << objects << " )\n" << eor;
	_initDebugObjects = objects;
}

void
Class::setInitDebugMask( int mask )
{
	Raids( RaidMethod ) << this << "::setInitDebugMask( "
			    << asHex( mask ) << " )\n" << eor;
	_initDebugMask = mask;
}

void
Class::setInitDebugMessages( int messages )
{
	Raids( RaidMethod ) << this << "::setInitDebugMessages( "
			    << messages << " )\n" << eor;
	_initDebugMessages = messages;
}

int
Class::debugMembers( int mask, int messages )
{
	int instances = 0;

	ProxiableObjectLock.acquire();
	for( ProxiableObject * po = ProxiableObjectsList; 
	     po != 0; po = po->proxiableObjectNext ) {
		if( po->isMemberOf( this ) ) {
			instances++;
			po->setDebugMask( mask );
			po->setDebugMessages( messages );
		}
	}
	ProxiableObjectLock.release();
	return( instances );
}

int
Class::debugKindred( int mask, int messages )
{
	int instances = 0;

	ProxiableObjectLock.acquire();
	for( ProxiableObject * po = ProxiableObjectsList; 
	     po != 0; po = po->proxiableObjectNext ) {
		if( po->isKindOf( this ) ) {
			instances++;
			po->setDebugMask( mask );
			po->setDebugMessages( messages );
		}
	}
	ProxiableObjectLock.release();
	return( instances );
}

static void
displayObject(ProxiableObject * po )
{
	Console() << po << "\n" << eor;
}

int
Class::displayMember( ProxiableObject * src )
{
	int result = 0;
	if( src != 0 ) {
		ProxiableObjectLock.acquire();
		for( ProxiableObject * po = ProxiableObjectsList; 
		     po != 0; po = po->proxiableObjectNext ) {
			if( src == po ) {
				Console() << src << "\n" << eor;
				result = 1;
				break;
			}
		}
		ProxiableObjectLock.release();
	}
	return( result );
}

int
Class::displayMembers()
{
	ClassApplicationFunction func = &displayObject;
	return (applyToMembers(func));
}

int
Class::displayKindred( int leaksOnly )
{
	ClassApplicationFunction func = &displayObject;
	return (applyToKindred(func));
}

int
Class::applyToMembers(ClassApplicationFunction func)
{
	int count = 0;

	ProxiableObjectLock.acquire();
	for( ProxiableObject * po = ProxiableObjectsList; 
	     po != 0; po = po->proxiableObjectNext ) {
		if( po->isMemberOf( this ) ) {
			count++;
			(*func)(po);
		}
	}
	ProxiableObjectLock.release();
	return( count );
}

int
Class::applyToKindred(ClassApplicationFunction func)
{
	int count = 0;

	ProxiableObjectLock.acquire();
	for( ProxiableObject * po = ProxiableObjectsList; 
	     po != 0; po = po->proxiableObjectNext ) {
		if( po->isKindOf( this ) ) {
			count++;
			(*func)(po);
		}
	}
	ProxiableObjectLock.release();
	return( count );
}

void
Class::setSizeOf( int size )
{
	_sizeOf = size;
}

void
Class::setConstructor( ClassConstructor * ctor )
{
	_constructor = ctor;
}

void
Class::setProxyTable( ProxyTableCopy * table )
{
	_proxyTable = table;
}

ProxyTableCopy *
Class::proxyTable( )
{
	if ( _proxyTable == 0 ) {
		if ( _real != 0 )
	    		_proxyTable = ( ( Class * )
				( ProxiableObject * ) _real )->proxyTable();
		else if ( _parent != 0 )
	    		_proxyTable = _parent->proxyTable();
	}
	return _proxyTable;
}

#include "ClassInstaller.h"
DEFINECLASS("Class",Class,ProxiableObject,0,0);
