#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ConstructorDescriptor.cc: descriptor of a proxiable constructor
 */

#include "ConstructorDescriptor.h"
#include "Class.h"
#include "NameServer.h"
#include "ThisProcess.h"
#include "Process.h"
#include "Assert.h"
#include "ProxyTableCopy.h"


#ifdef __GNUC__
#pragma implementation
#endif

ConstructorDescriptor::ConstructorDescriptor( char * name,
		char * clss, unsigned  number, PConstructor funct )
{
	NameServer * ns = thisProcess()->nameServer();

	ClassStar myClass = ns->lookup( clss, ClassClass );

	Assert( myClass != 0 )
		( "Class %s not found for constructor descriptor %s\n",
				clss, name );

	ProxyTableCopy *table = myClass->proxyTable();

	Assert( table != 0 )
		( "No proxy table for class %N for constructor descriptor %s\n",
						(ProxiableObject*)myClass, name );

	Assert( number < table->numConstructors )
	    ( "Invalid constructor number %d for %N\n", number, (ProxiableObject*)myClass );

	meth = table->constructors[ number ];

	domain = thisProcess()->domain();

	ctor = funct;

	size = myClass->sizeOf();

	ns->bind( name, this );

	setName( name );
}

#include "ClassInstaller.h"
DEFINECLASS("ConstructorDescriptor",ConstructorDescriptor,ProxiableObject,0,0);
