#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * Interrupt.cc: interrupt class stubs.
 */

#include "Assert.h"
#include "Raid.h"
#include "Interrupt.h"
#include "ClassStar.h"


#ifdef __GNUC__
#pragma implementation
#endif

#ifdef INSTRUMENT_INTERRUPT
#define Instr(exp) exp
extern class SystemInstrumentManager : public InstrumentManager;
extern SystemInstrumentManager *TheInstrumentManager;

PerCpuTimerInstrument *iInterruptRaise = NULL;

void 
InterruptConstructInstruments ()
{
	iInterruptRaise = new PerCpuTimerInstrument
		(TheInstrumentManager, "InterruptRaise", TALKING, 1000, ALL);
}

void 
InterruptDeleteInstruments ()
{
        delete iInterruptRaise;
        iInterruptRaise = NULL;
}

#else
#define Instr(exp)
void 
InterruptConstructInstruments () {}
void 
InterruptDeleteInstruments () {}
#endif  //  INSTRUMENT_INTERRUPT

Interrupt::Interrupt()
{
	_next = 0;
}

Interrupt *
Interrupt::stack(Interrupt *e)
{
	// tack it on the end and return the same list
	Interrupt * current = this;
	while (current->_next) current = current->next();
	current->setNext(e);
	return this;
}


/*
 * This is mainly here so assembler assists for interrupt handling don't
 * have to know how to do C++ method lookup and dispatch.
 * It could be inlined but I would need some way to FORCE the compiler
 * to generate a callable version as well. I'll worry about it when
 * I really try to optimize things. --Vince
 */
void
Interrupt::raise( char * info )
{
	Instr (iInterruptRaise->start ());
	Raid(RaidMethod)("%N::raise( %d )", this, info);
	basicRaise( info );
	Raid(RaidMethod)("%N::raise( %d ): done", this, info);
	Instr (iInterruptRaise->stop (Interrupt_RaiseEvent));
}


#include "ClassInstaller.h"
DEFINECLASS("Interrupt",Interrupt,Object,0,0);
