#ifdef __GNUC__
#pragma implementation
#endif

#include "OutputStream.h"
#include "InterruptManager.h"

InterruptManager::InterruptManager() { 
    Assert(NOTREACHED); 
}

InterruptManager::InterruptManager(unsigned numInterrupts):numInterrupts(numInterrupts) {
    interrupts = reinterpret_cast<Interrupt **>(new void*[numInterrupts]);
    for (unsigned i = 0; i < numInterrupts; i++) {
        interrupts[i] = 0;
    }
}

InterruptManager::~InterruptManager() {
    delete [] interrupts;
}

Interrupt *InterruptManager::setInterrupt(unsigned number, Interrupt *interrupt) {
    Interrupt *oldInterrupt = 0;
    if (number < numInterrupts) {
        oldInterrupt = interrupts[number];
        interrupts[number] = interrupt;
        enableInterrupt(number);
    } else {
        Assert(NOTREACHED);
    }
    return oldInterrupt;
}

Interrupt *InterruptManager::getInterrupt(unsigned number) {
    if (number < numInterrupts) {
        return interrupts[number];
    } else {
        return 0;
    }
}

void InterruptManager::dispatchInterrupt(unsigned number, InterruptContext *context) {
    if (number < numInterrupts) {
        Interrupt *interrupt = interrupts[number];
        if (interrupt) {
            interrupt->raise((char *)context);
        } else {
            Debug << "Spurious interrupt #" << number << " received.\n" << eor;
        }
    } else {
        Assert(NOTREACHED);
    }
}

void InterruptManager::interruptReceived(InterruptContext *context) {
    unsigned number = getInterruptNumber();

    dispatchInterrupt(number, context);

    // Interrupts are disabled now - This ACK goes to Interrupt
    // handling hardware. The device or hardware generating the interrupt (interrupt source) 
    // might need a separate ACK which should be done by the dispatch routine
    ackInterrupt(number);
}

void InterruptManager::interruptReceived(unsigned number, InterruptContext *context) {
    dispatchInterrupt(number, context);

    // Interrupts are disabled now - This ACK goes to Interrupt
    // handling hardware. The device or hardware generating the interrupt (interrupt source) 
    // might need a separate ACK which should be done by the dispatch routine
    ackInterrupt(number);
}
