#ifdef __GNUC__
#pragma implementation
#pragma implementation "CharacterInputStreamStar.h"
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * Kernel.cc
 */

/*
 * PUT THE FOLLOWING in the machine configuration file:
 */

#if defined(LINUXNET)
#ifdef NONETWORKING
#undef NONETWORKING
#endif
#endif
#if defined(BSDNET)
#ifndef NONETWORKING
#define NONETWORKING
#endif
#endif

// #define DUMP_OBJECTS_ON_EXIT

static const int MAXKERNELPROXIES = 4096;

#include "defNETWORKING.h"
#include "defZOOT.h"
#include "ADDRESS_TRANSLATION.h"
#include "Assert.h"
#include "Async.h"
#include "Class.h"
#include "CharacterInputStream.h"
#include "ConstructorInstaller.h"
#include "Convert.h"
#include "CPU.h"
#include "ThisCPU.h"
#include "ThisProcess.h"
#include "Domain.h"
#include "DummyMemoryObject.h"
#include "FIFOScheduler.h"
#include "FileSystem.h"
#include "FileSystemInterface.h"
#include "VMInterface.h"
#include "Raid.h"
#include "FreeRunningTimer.h"
#include "FreeRunningTimerStar.h"
#include "InputStream.h"
#include "IOWindow.h"
#include "Kernel.h"
#include "MachineConfiguration.h"
#include "MadAllocator.h"
#include "NameServer.h"
#include "MemoryObjectCache.h"
#include "MemoryObjectCachingContainer.h"
#include "MemoryObjectContainer.h"
#include "NullStreams.h"
#include "OutputStream.h"
#include "PhysicalMemoryManager.h"
#include "PhysicallyAddressableUnit.h"
#include "PersistentMemoryObject.h"
#include "ProcessContainer.h"
#include "ProxyTable.h"
#include "PROCESSOR.h"
#ifdef RemoteFileSystem
#include "RemoteFileSystemRPC.h"
#include "RemoteServer.h"
#endif // RemoteFileSystem
#include "RRScheduler.h"
#include "SemaphoreRegulatedFIFOScheduler.h"
#include "SimpleInstruments.h"
#include "Store.h"
#include "String.h"
#include "SystemInterface.h"
#include "SystemScheduler.h"
#include "ThisProcess.h"
#include "TimeoutTimer.h"
#include "VersionInstaller.h"
#include "VersionObject.h"
//#include "PhysicalMemoryObject.h"
#include "VirtualMemoryRange.h"
#include "KernelCmdInstaller.h"
#include "Disk.h"
#include "ELFDictionary.h"
#include "AppInterface.h"
#include "PeriodicTimer.h"

#ifdef DVM
#include "MachineBuilder.h"
#endif
#ifdef VCHOICES
#ifndef __APPLE__
extern "C" void * sbrk(int);    // VCHOICES
#endif
#ifdef __GNUC__
#pragma implementation "VCStupidAllocator.h"
#endif
#include "VCStupidAllocator.h"
#endif
#ifdef DISTRIBUTED
#include "EthernetTransport.h"
#include "LocalTransport.h"
#include "DistributedNameServer.h"
#endif
#ifdef INSTRUMENT
#define Instr(exp) exp
#include "SystemInstrumentManager.h"
#include "PerCpuTimerInstrument.h"
SystemInstrumentManager *TheInstrumentManager = NULL;
EventerInstrument *iEventCheck = NULL;
CounterInstrument *iCounterCheck = NULL;
TimerInstrument *iTimerCheck = NULL;
TimerInstrument *iTimerLatencyCheck = NULL;
#ifdef INSTRUMENT_IO
EventerInstrument *iIO = NULL;
#endif INSTRUMENT_IO

#ifdef __GNUC__
#pragma implementation
#endif

extern PerCpuTimerInstrument *iProcessRestore;

void 
KernelConstructInstruments ()
{
    iEventCheck = new EventerInstrument
    (TheInstrumentManager, "EventCheck", TALKING);
    iCounterCheck = new CounterInstrument
    (TheInstrumentManager, "CounterCheck", TALKING, 1);
    iTimerCheck = new TimerInstrument
    (TheInstrumentManager, "TimerCheck", TALKING, 1, ALL);
    iTimerLatencyCheck = new TimerInstrument
    (TheInstrumentManager, "TimerLatencyCheck", TALKING, 1, ALL);
}
 
void 
KernelDeleteInstruments ()
{
    delete iEventCheck;
    iEventCheck = NULL;
    delete iCounterCheck;
    iCounterCheck = NULL;
    delete iTimerCheck;
    iTimerCheck = NULL;
    delete iTimerLatencyCheck;
    iTimerLatencyCheck = NULL;
}

// void startInstruments();
void setupInstrumentation();
void cleanupInstrumentation();
#ifdef INSTRUMENT_KERNEL
void instrumentTimes ();
#endif INSTRUMENT_KERNEL

#else
#define Instr(exp)
#endif /*INSTRUMENT*/


PhysicalMemoryManager * MainMemory;
Store * MainStore;      // Main physical memory manager.
BootConsole * bootConsole;

/*
 * The constructor for a Kernel. Almost nothing should actually be done here
 * or in any subclass constructors. In particlar, the Class and ProxiableObject
 * stuff isnt set up yet, so creating any new objects would probably
 * be a bad idea.
 *
 * The *real* work of initializing the machine and the kernel should be done
 * in Kernel::initialize(), and Kernel::basicInitialize().
 */
Kernel::Kernel()
{
    /*
     * Print a banner to the REAL console to show signs of life.
     */
    Console() << "\nChoices [Version 0.9 beta] $Revision: 674 $\n" ;
    Console() << "(C) Copyright 1987-2007 by The University of Illinois Board of Trustees.\n";
    Console() << " All rights reserved.\n";
    Console() << " Information at http://choices.cs.uiuc.edu/\n\n";
    Console() << " This product includes software developed by\n" ;
    Console() << " the University of California, Berkeley and its contributors.\n";
    Console() << "\n" << eor;


    /*
     * This should have an assertion to check that it was only
     * called one in the entire lifetime of the system.
     * This is probably good enough.
     */
    Assert( TheKernel == 0 );

    _initialScheduler = (ProcessContainer *) 0;
    _kernelDomain = 0;

    bootConsole = 0;
}

/*
 * Never delete a Kernel.
 */
Kernel::~Kernel()
{
    Assert( NOTREACHED );
}

/*
 * This is only called by the first processor. The bootHeap argument is
 * the allocator all the things allocated in this method are being allocated
 * from via operator new. This argument is used to solve the chicken-and-egg
 * problem of how much VM space is needed to map all the initially allocated
 * stuff 1-to-1.
 * 
 * One "feature" to be aware of (and the will probably be very useful) is
 * that any memory allocated, before the new kernel heap is installed at
 * the end of this routine, will be mapped 1-to-1.
 *
 * Subclasses MUST setup the thisCPU() function to lookup the CPU object 
 * pointer to use for the invoking processor in ::basicInitialize().
 * The CPU object CANNOT be allocated off the heap with a "new" when a new
 * processor starts, it must have exist already. This is due to the fact that
 * there is no useable heap at the point this method is invoked.
 * On uni-processors its simple. The Kernel initialization should create
 * a CPU instance for the particular processor type and keep a pointer to it
 * around to return when this method is invoked.
 * On multi-processors, the CPU objects for each configured
 * processor should have been built during the Kernel initialization
 * if the number of processor was know. If not, things are bad. I have
 * a few ideas but until we get a machine that has this problem (if
 * one even exists) I'm not going to worry about it.
 *      --Vince
 */

CharacterInputStreamStar ConsoleInputStream;
OutputStream * ConsoleOutputStream;

void
Kernel::initialize( Allocator * bootHeap )
{
    Debug << this << "::initialize()\n" << eor;

    PROCESSOR::disableInterrupts();     // Just to be sure

    /*
     * Initialize some local vm variables from the
     * subclass
     */
    const char * _kernelStart = kernelStart();
    const unsigned _kernelSize = kernelSize();

    /*
     * Build the kernel Domain and set kernel standard nameserver
     */
    _kernelDomain = new Domain( 0, (char *)_kernelStart, _kernelSize);
    Assert (_kernelDomain != 0);
    domain()->setName( "#KernelDomain" );
    StandardNameServer = domain()->nameServer();

    /*
     * Initialize the ready queue scheduler so we can build the
     * running CPU object.
     * We are using a round-robin timeslicing container
     */
    ProcessContainer * readyQueue = new RRScheduler( 1 << 16 );
    Assert( readyQueue != 0 );
    Debug << "Ready Queue " << readyQueue << "\n" << eor;
    /*
     * A SystemScheduler is actually two queues. A high priority
     * run-to-completion FIFO queue, plus whatever was
     * created above.
     */
    // THE SUBCLASS SHOULD SET THE SCHEDULER!
    _initialScheduler = new SystemScheduler( readyQueue );
    Assert( _initialScheduler != 0 );

    /*
     * Build the first CPU
     */
    buildRunningCPU();

    //TheZeroObject->setName( "#TheZeroObject" );
    //ThePhysicalMemoryObject->setName( "#ThePhysicalMemoryObject" );

    TheNullInputStream.setName( "#TheNullInputStream" );
    TheNullOutputStream.setName( "#TheNullOutputStream" );

    /*
     *  Initialize the input and output streams for the console
     */
    ConsoleOutputStream->setName( "#ConsoleOutputStream" );
    ConsoleOutputStream->reference(); //created too early for Stars
    StandardNameServer->bind( "StandardOutput", ConsoleOutputStream );
    StandardNameServer->bind( "ConsoleOutput", ConsoleOutputStream );

    ConsoleInputStream = new CharacterInputStream( &Console() );
    Assert( ConsoleInputStream != 0 );
    ConsoleInputStream->setName( "#ConsoleInputStream" );
    StandardNameServer->bind( "StandardInput", ConsoleInputStream );

    /*
     * Build the garbageCan.
     */
    _garbageCan = new SemaphoreRegulatedFIFOScheduler();
    Assert( _garbageCan != 0 );
    Debug << "GarbageCan " << _garbageCan << " built\n" << eor;

#ifdef DVM
    // Build VM state machines before diddling with VM stuff
    TheMachineBuilder.build();
#endif

    Process *p = new SystemProcess(
        (PFV) &Kernel::setupProcessTrampolineEntry, domain(), 
        heapBootStack, (int) this );
    p->setName( "#Kernel Setup Trampoline Process" );
    Assert( _initialScheduler != 0 );
    p->changeReadyContainer( _initialScheduler );
    p->ready();
    
    /*
     * Figure out all the things that need to be in the shared
     * virtual address space.  These regions will be shared by
     * all domains.  These include the kernel text, data, and
     * heap areas, which begin at _kernelStart, and the (preallocated)
     * I/O areas.
     */

    unsigned unitsOfKernelVM = _kernelSize / PhysicalPageSize + 1;
    VirtualMemoryRange * kernelVirtualMemoryMap = new VirtualMemoryRange(
        _kernelStart, unitsOfKernelVM, Log2PhysicalPageSize,
        "#Kernel RAM" );


    PhysicalMemoryRange * _physicalMemoryMap = physicalMemoryMap();
    Assert( _physicalMemoryMap != 0 );
    PhysicalMemoryRange * current = _physicalMemoryMap;
    while( current != 0 ) {
        if( current->preAllocated() ) {
            kernelVirtualMemoryMap->add( new VirtualMemoryRange(
                                             current->virtualAddress(),
                                             current->numberOfUnits(),
                                             current->log2UnitSize(),
                                             current->name() ) );
        }
        current = current->next();
    }

    /*
     * Initialize the AddressTranslation class.  The memory
     * allocated will be mapped one-to-one when VM is enabled
     * since we are still using the bootHeap.
     * This is *imperative* to the proper functioning of
     * some instances of AddressTranslation.
     */
    Debug << "Kernel:: initializing address translation class\n" << eor;
    ADDRESS_TRANSLATION::init( addressTranslationMemorySize(), kernelVirtualMemoryMap );

    /*
     * Set up the machine independent initial persistent objects.
     */
    Debug << "Kernel:: setting up machine independent initial persistent objects\n" << eor;

    extern FreeRunningTimerStar SystemTimer;
    SystemTimer->setName( "#SystemTimer" );
    StandardNameServer->bind( "SystemTimer", SystemTimer );

    Debug << this << "::initialize() begin memory config\n" << eor;

    /*
     * Build the PhysicalMemoryManager Object (the collection of all 
     * PhysicallyAddressableUnits for all of addresable memory).
     */
    MainMemory = new PhysicalMemoryManager( _physicalMemoryMap );
    Assert( MainMemory != 0 );
    MainMemory->setName( "#MainMemory" );

    MainStore = new Store( _physicalMemoryMap );
    Assert( MainStore != 0 );
    MainStore->setName( "#MainStore" );

    /*
     * Build the Store, then we can build processes
     * without stacks on the heap
     */
    Debug << "MainStore " << MainStore << " ("
          << MainStore->numberOfUnits() << " units, "
          << MainStore->numberOfUnits() * MainStore->unitSize()
          << " bytes)\n" << eor;

    /*
     * Lay out the initial VM mappings in the Kernel domain.
     * These initial mappings are the Kernel, bootHeap, 1-1
     * memory mappings, etc.
     *
     * First lay down the kernel mappings, then go back over the
     * physical memory map and see what parts of it we missed and
     * add them 1-1.
     *
     */

    /*
     * Build MemoryObjects to represent everything that's already in
     * physical memory. This includes all the stuff loaded from
     * the image of the system itself plus the stuff allocated
     * past KERNELHEAPSTART.  Allow for a piece in the middle
     * which is marked 'ReadOnly' to enable the ProxyAssist code
     * to be accessible to Applications.  The variables 
     * 'KernelReadOnlyStart' and 'KernelReadOnlyEnd' should be
     * set by all versions of the kernel, possibly in the
     * 'LoaderDirectives' file.
     * Once the memory objects are built, add them to the Domain of
     * the kernel.
     */
    extern const char KernelReadOnlyStart[], KernelReadOnlyEnd[];
    /* 
     * Round the start down and the end up to be sure to get it all.
     */
    const char *kernelReadOnlyStart = (const char *)
    ((int) KernelReadOnlyStart & ~(PhysicalPageSize - 1));
    const char *kernelReadOnlyEnd = (const char *)
    (((int) KernelReadOnlyEnd + PhysicalPageSize - 1) &
     ~(PhysicalPageSize - 1));
/*****************************************************************************
 * Insert the low kernel memory object (if necessary)
 *****************************************************************************/
    if (_kernelStart < kernelReadOnlyStart) {
        _lowMemoryObject = new
        PhysicallyResidentMemoryObject( ((kernelReadOnlyStart -
                                          _kernelStart) / PhysicalPageSize),
                                        Log2PhysicalPageSize, _kernelStart, 1 );
        Assert( _lowMemoryObject != 0 );
        domain()->add( _lowMemoryObject, _kernelStart, NoAccess );
        Debug << "Added " << _lowMemoryObject << " for\n\t"
              << "_lowMemoryObject to\n\t" << domain() << " at "
              << asAddress( _kernelStart ) << " to "
              << asAddress( kernelReadOnlyStart ) << "\n" << eor;
        _lowMemoryObject->setName( "#_lowMemoryObject" );

    }
/*****************************************************************************
 * Insert the read-only part of the kernel
 *****************************************************************************/
    _readOnlyMemoryObject = new
    PhysicallyResidentMemoryObject( ((kernelReadOnlyEnd -
                                      kernelReadOnlyStart) / PhysicalPageSize),
                                    Log2PhysicalPageSize, kernelReadOnlyStart, 1 );
    Assert( _readOnlyMemoryObject != 0 );
    domain()->add( _readOnlyMemoryObject, kernelReadOnlyStart, ReadOnly );
    Debug << "Added " << _readOnlyMemoryObject << " for\n\t"
          << "_readOnlyMemoryObject to\n\t" << domain() << " at "
          << asAddress( kernelReadOnlyStart ) << " to "
          << asAddress( kernelReadOnlyEnd ) << "\n" << eor;
    _readOnlyMemoryObject->setName( "#_readOnlyMemoryObject" );
    MemoryObjectCache * aCache = _readOnlyMemoryObject->cache();
    if( aCache != 0 ) aCache->setName( "#_readOnlyMemoryObjectCache" );


/*****************************************************************************
 * Create the Kernel Heap MO, stick it in the domain later after 
 * all other initial mappings are set up.
 *****************************************************************************/
    /*
     * Build a RootMemoryObject for the kernelHeap.
     */
    unsigned int kernelPages = kernelHeapSize() / PhysicalPageSize;
    _heapMemoryObject = new RootMemoryObject(kernelPages, Log2PhysicalPageSize );
    Debug << "_heapMemoryObject = " << _heapMemoryObject
          << "(" << _heapMemoryObject->numberOfUnits()
          << " units)\n" << eor;
    Assert( _heapMemoryObject != 0 );
    _heapMemoryObject->setName( "#_heapMemoryObject" );
    aCache = _heapMemoryObject->cache();
    if( aCache != 0 ) aCache->setName( "#_heapMemoryObjectCache" );

/*****************************************************************************
 * Allocate anything that need be before we find the end of the bootheap
 * and map the high kernel in 1-1.
 *****************************************************************************/
    /*
     * Put aside some memory to later build the kernelHeap Allocator
     * in.  (We know that its constructor does no memory allocation
     * of its own.)
     */
    void * kernelHeapMemory = new char[ sizeof( MadAllocator ) ];
    int allocatorStateSize = MadAllocator::stateSize(
        _heapMemoryObject->numberOfUnits() );
    void * allocatorStateAddress = new char[ allocatorStateSize ];
    Debug << "Allocated " << allocatorStateSize << " bytes at "
          << asAddress( allocatorStateAddress )
          << " for MadAllocator state\n" << eor;

    /*
     * Allocate space for the _highKernelMemoryObject
     */
    void * highKernelMemoryObjectSpace =
    new char[ sizeof(PhysicallyResidentMemoryObject) ];

#ifdef ONE_TO_ONE
    /*
     * Allocate space for memory objects that 1-1 map physical
     * memory not mapped in the kernel.  Allocate one MO per
     * piece of the physicalMemoryMap.  One of the pieces in
     * the physicalMemory map is assumed to hold the kernel,
     * but there still may be parts of that area left over that 
     * need to be 1-1 mapped.  Preallocated areas need no action.
     */
    _numOneToOneMemObjs = 0;
    for ( current = _physicalMemoryMap;
          current != 0;
          current = current->next() ) {
        if ( current->preAllocated() )
            _numOneToOneMemObjs++;
    }
    Debug << "Found " << asHex(_numOneToOneMemObjs) 
          << " memory objects to map one-one\n" << eor;

    void ** oneToOneMemoryObjectSpaces = 0;
    if ( _numOneToOneMemObjs > 0 ) {
        oneToOneMemoryObjectSpaces = new void * [_numOneToOneMemObjs];
        for (int i = 0; i < _numOneToOneMemObjs; i++) {
            /*
             * Create the spaces to new them into
             */
            oneToOneMemoryObjectSpaces[i] = new 
            char[ sizeof(PhysicallyResidentMemoryObject) ];
        }

        /*
         * And create the stars in the kernel to reference them
         * so they don't go away
         */
        _oneToOneMemoryObjects = 
        new MemoryObjectStar[_numOneToOneMemObjs];
    }
#endif

/*****************************************************************************
 * ALL ALLOCATION FROM BOOTHEAP SHOULD BE DONE BY THIS POINT (but it's not)
 *****************************************************************************/
    /*
     * allocate to the next physical page boundary, then set topSoFar
     * to the next page since the creation of the following memory
     * objects will allocated the ProxiableObject parts from the 
     * bootHeap.  This is generally 2 bytes per MO, so a page should
     * handle it.  What a HACK.
     */
    char * topSoFar = (char *) bootHeap->allocate( 0 );
    // the point of this appears to be to make sure the stupid
    //   allocator has rounded off top to be a multiple of 8. -JZ

    (void) bootHeap->allocate(
        PhysicalPageSize -
        ( ( (unsigned int) topSoFar ) & ( PhysicalPageSize - 1 ) ) );
    topSoFar = (char *) bootHeap->allocate( 0 );
#ifdef VCHOICES
    topSoFar = (char *)sbrk(0);
    topSoFar  = (char *) (((int)topSoFar + 5*PhysicalPageSize) 
                          & ~(PhysicalPageSize -1));
#endif
    Debug << "topSoFar is " << asAddress(topSoFar) << "\n" << eor;

    Assert( ( ( (unsigned int ) topSoFar ) & ( PhysicalPageSize - 1 ) ) == 0 );
    topSoFar += PhysicalPageSize - 1;


/*****************************************************************************
 * Create the high kernel MO now that we know where the top of memory is.
 *****************************************************************************/
    Debug << "Creating high kernel memory object (start/topSoFar: "
          << asAddress( kernelReadOnlyEnd ) << "/"
          << asAddress( topSoFar ) << ")\n" << eor;
    _highMemoryObject = new
    ( highKernelMemoryObjectSpace )
    PhysicallyResidentMemoryObject( Log2PhysicalPageSize,
                                    kernelReadOnlyEnd, topSoFar, 1 );
    Assert( _highMemoryObject != 0 );
    domain()->add( _highMemoryObject, kernelReadOnlyEnd, NoAccess );
    Debug << "Added " << _highMemoryObject << " for\n\t"
          << "allocatedMemoryObject to\n\t" << domain() << " at "
          << asAddress( kernelReadOnlyEnd ) << "\n"
          << eor;

/*****************************************************************************
 * Cache all the kernel areas 
 *****************************************************************************/
    if (_kernelStart < kernelReadOnlyStart) {
        domain()->makeAddressable( _kernelStart,
                                   kernelReadOnlyStart - _kernelStart );
        Debug << "Forced cache of Low Kernel\n" << eor;
    }
    domain()->makeAddressable( kernelReadOnlyStart, kernelReadOnlyEnd - kernelReadOnlyStart );
    Debug << "Forced cache of ReadOnly Kernel\n" << eor;
    domain()->makeAddressable( kernelReadOnlyEnd, 
                               (topSoFar - (char *)kernelReadOnlyEnd) );
    Debug << "Forced cache of HighKernel\n" << eor;

#ifdef ONE_TO_ONE
/*****************************************************************************
 * 1-1 map whatever is left.
 * This assumes that whatever is below topSoFar has been mapped already.
 *****************************************************************************/
    unsigned currentSlot = 0;
    for( current = _physicalMemoryMap;
         current != 0;
         current = current->next() ) {
        if ( !current->preAllocated() )
            continue;
        Assert ( currentSlot < _numOneToOneMemObjs );
        char *start = (char*)current->physicalAddress();
        char *end = start + 
        current->numberOfUnits() * current->unitSize() - 1 ;
        if (start < topSoFar) {
            Debug << "Start of this MO at end of Kernel\n" 
                  << eor;
            start = topSoFar + 1;
        }
        Debug << "Creating oneToOneMemoryObject (start/end: "
              << asAddress( start ) << "/"
              << asAddress( end ) << ")\n" << eor;
        Assert (start >= topSoFar);
        Assert ( (end - 1) > start);
        _oneToOneMemoryObjects[currentSlot] = new
        ( oneToOneMemoryObjectSpaces[currentSlot] )
        PhysicallyResidentMemoryObject( 
            Log2PhysicalPageSize, start, end, 
            current->preAllocated() );
        Assert( _oneToOneMemoryObjects[currentSlot] != 0 );
        domain()->add( _oneToOneMemoryObjects[currentSlot],
                       start, NoAccess );
        Debug << "Added " << _oneToOneMemoryObjects[currentSlot]
              << " for\n\t" << current << " to\n\t" 
              << domain() << " at " << asAddress(start) 
              << "\n" << eor;
        domain()->makeAddressable( start, end - start);
        Debug << "Forced cache of 1-1 MemoryObject\n" 
              << eor;
        currentSlot++;
    }
    Assert( currentSlot == _numOneToOneMemObjs );
#endif

    // Allow subclasses to add their own MemoryObjects if necessary
    basicAddMemoryObjects();

/*****************************************************************************
 * Now we can insert the Heap MO and verify that it is in the
 * kernel VM region.
 *****************************************************************************/
    char * kernelHeapAddress =
    (char *) domain()->add( _heapMemoryObject, NoAccess );
    Debug << "Added " << _heapMemoryObject << " for\n\t"
          << "_heapMemoryObject to\n\t" << domain() << " at "
          << asAddress( kernelHeapAddress ) << "\n"
          << eor;
    Assert ( kernelHeapAddress < _kernelStart + _kernelSize );
    Assert ( kernelHeapAddress + kernelHeapSize() < _kernelStart + _kernelSize );

    /*
     * Build the kernelHeap object now that we know the virtual address
     * of the _heapMemoryObject.
     */
    char *endBootHeap = (char*)bootHeap->allocate( 0 );
    Assert (topSoFar > endBootHeap);
    void * startBootHeap = (void *) this;
    endBootHeap = topSoFar;

    _kernelHeap = new
    ( kernelHeapMemory )
    MadAllocator( allocatorStateAddress, kernelHeapAddress,
                  _heapMemoryObject->numberOfUnits(),
                  _heapMemoryObject->log2UnitSize(), domain(),
                  startBootHeap, endBootHeap, 1 );
    Assert( _kernelHeap != 0 );
    Debug << "KernelHeap ready\n" << eor;

    /*
     * Don't actually do a "new" until VM is enabled!!
     */
    extern Allocator * SetNewAllocator( Allocator * );
    Allocator * oldAllocator = SetNewAllocator( _kernelHeap );

    /*
     * Make sure the oldAllocator was the right one.
     */
    Assert( oldAllocator == bootHeap );

    /*
     * The kernel is ready to use now so we can start the other
     * processors if there are any.
     */
    startOtherProcessors();
    Debug << this << "::initialize() done\n" << eor;
}


/* 
 * The setupProcessEntry is used by a process with a boot stack on the boot
 * heap.
 * It simply builds the setProcess with a stack as a memoryObject.
 * This is necessary due to avoid perturbing VM initialization any
 * more.
 * -raila 3-19-93
 */
void
Kernel::setupProcessEntry( Kernel *k)
{
    Assert( k != 0 );
    k->main();
}

/*
 * Stub for the Kernels "setup" Process to enter at.
 */
void
Kernel::setupProcessTrampolineEntry( Kernel * k )
{
    Assert (k != 0);
    /*
     * Build a startup process and stick it into the initial scheduler.
     * It will be run by the first processor to enter its idleLoop.
     */
    Debug << "Building kernel setup process\n" << eor;
    /*
     * Like the CPU idle Processes, we set the domain for the new
     * process even though, at this point, the Domain is empty.
     * It won't be empty by the time we use it, however.
     */
    Process *p = new SystemProcess(
        (PFV)&Kernel::setupProcessEntry, k->domain(), large, (int) k );
    p->setName( "#Kernel Setup Process" );
    //Assert( _initialScheduler != 0 );
    //p->changeReadyContainer( _initialScheduler );
    p->ready();
    thisProcess()->die();
}

void
Kernel::main()
{
    ProcessResourceUsageOn = 0;

    ((MadAllocator *) _kernelHeap)->enableMultiProcessing();

    MainStore->startPager();

    /*
     * Configure hardware, additional CPU's, memory, etc.
     */
    configureHardware();    
    Debug << "Kernel::configureHardware() done\n" << eor;

    /*
     * Name the memory objects we couldn't do before because the
     *   allocator wasn't ready.
     */
    if (_lowMemoryObject != 0){
        _lowMemoryObject->setName( "#_lowMemoryObject");
        MemoryObjectCache * aCache = _lowMemoryObject->cache();
        if( aCache != 0 ) 
            aCache->setName( "#_lowMemoryObjectCache" );
    }
    Assert (_readOnlyMemoryObject != 0);
    _readOnlyMemoryObject->setName( "#_readOnlyMemoryObject" );
    MemoryObjectCache * aCache = _readOnlyMemoryObject->cache();
    if( aCache != 0 ) aCache->setName( "#_readOnlyMemoryObjectCache" );
    Assert (_highMemoryObject != 0);
    _highMemoryObject->setName( "#_highMemoryObject" );
    aCache = _highMemoryObject->cache();
    if( aCache != 0 ) aCache->setName( "#_highMemoryObjectCache" );
#ifdef ONE_TO_ONE
    int i;  // Keep it out of the conditionals
    for (i = 0; i < _numOneToOneMemObjs; i++) {
        Assert (_oneToOneMemoryObjects[i] != 0);
        _oneToOneMemoryObjects[i]->setName("#_aOneToOneMemoryObject");
        aCache = _oneToOneMemoryObjects[i]->cache();
        if( aCache != 0 ) 
            aCache->setName( "#_aOneToOneMemoryObjectCache" );
    }
#endif

    StandardNameServer->setName( "#KernelNameServer" );

    ObjectClass->bind( StandardNameServer );
    Debug << "Classes bound\n" << eor;

    /*
     * Initialize the proxy tables.  This has to be done after the
     *   classes are bound into the nameServer.
     */
    ProxyTableInstaller::install();

    /*
     * Initialize the constructor descriptors.  This has to be done
     *   after the ProxyTableInstaller::install().
     */
    ConstructorInstaller::install();

    /*
     * Initialize the provided versions
     */
    VersionObject::install( ProvidedVersionInstaller::list() );

    /*
     * Initialize VM IO Window Cache
     */
    TheIOWindowCache = new IOWindowCache( 0 );

    /*
     * Initialize asynchronous support
     */
    TheAsyncSupport = new AsyncSupport( 6 );

    /*
     * Setup all the interrupt handling and process management
     * processes and start them running.
     */
    Debug << "Kernel::main() initializeDevicesAndManagers\n" << eor;
    initializeDevicesAndManagers();

    Debug << "Kernel::main() createFileSystemInterface\n" << eor;
    createFileSystemInterface();

    Debug << "Kernel::main() createVMInterface\n" << eor;
    createVMInterface();
    
#ifndef ARM
    Debug << this << "::main(): build machine configuration\n" << eor;
    buildConfiguration();
    Assert (getConfiguration() != 0);
#endif

#ifndef NONETWORKING  
#ifdef NETWORKING
    Debug << "Kernel::main() initializeNetworkProcesses\n" << eor;
    initializeNetworkProcesses();
#endif
#endif

#ifdef BSDNET
    extern int net_init();
    net_init();
#endif

#ifdef DVM
    TheMachineBuilder.buildDVM();
#endif

#ifdef INSTRUMENT
    setupInstrumentation();
#endif /*INSTRUMENT*/

    /*
     * Build application-interface system objects
     */
    TheSystemInterface = new SystemInterface;  // hang
    StandardSystemInterface = TheSystemInterface;
    TheSystemInterface->setName( "TheSystemInterface" );
    Debug << "TheSystemInterface = " << TheSystemInterface << "\n" << eor;
    TheAppInterface = new AppInterface;

    /*
     * Loop executing top level applications.
     */
    dispatchApplications();


    Instr(cleanupInstrumentation());

    _fileSystemInterface = 0;
    // THIS CRASHES VCHOICES! dkr-11-11-94 // _backingStoreSource = 0;

    PROCESSOR::disableInterrupts();
     
    this->shutdown();     // Machine dependent shutdown (no more interrupts)

    TheSystemInterface = 0;
    TheAppInterface = 0;

    ObjectClass->cleanup();

#ifdef DUMP_OBJECTS_ON_EXIT
    int instances = ObjectClass->displayKindred( 1 );
    Console() << "Objects still alive " << instances << "\n" << eor;
#endif

    extern void Halt();
    Halt();
    Assert( NOTREACHED );
}

void
Kernel::mountNonRootFileSystems()
{
    /* default do nothing */
    Console() << "Kernel::mountNonRootFileSystems(): nothing mounted\n"
              << eor;
}

void
Kernel::buildConfiguration()
{
    Debug << "Kernel::main() initialize MachineConfiguration\n" << eor;
    _configuration = new MachineConfiguration(_fileSystemInterface);
}

void
Kernel::initializeBackingStore()
{
    /* default do nothing */
    Console() << "Kernel::initializeBackingStore(): not initialized\n"
              << eor;
}

void
Kernel::createFileSystemInterface()
{
    /*
     * Get a source of swapping memory object. ::rootDisk() is
     * implemented by the subclasses to return the primary disk
     * for this machine.
     */
    MemoryObjectStar disk = rootDisk();
    Assert( disk != 0 );
    disk->setName( "#rootDisk" );

    int status = 0;
    MemoryObjectContainerStar rootContainer =
    disk->asA( MemoryObjectContainerClass, status );
    Assert( rootContainer != 0 );
    rootContainer->setName( "rootContainer" );

    Debug << "rootContainer = " << rootContainer << "\n" << eor;

    status = 0;
    MemoryObjectStar mo = rootContainer->open( 0, status, O_NCACHE );
    Assert( mo != 0 );
    MemoryObjectContainerStar oc = mo->asA( MemoryObjectContainerClass,
                                            status );
    Assert( oc != 0 );

    MemoryObjectDictionaryStar mod = oc->rootDictionary();
    Assert( mod != 0 );
    _fileSystemInterface = new FileSystemInterface( mod, rootContainer );
    _fileSystemInterface->setName( "KernelFileSystemInterface" );

    Debug << "_fileSystemInterface = " << _fileSystemInterface << "\n" << eor;

    mountNonRootFileSystems();

    StandardNameServer->bind( "fileSystemInterface", _fileSystemInterface );

    initializeBackingStore();
}

void
Kernel::createVMInterface()
{
    _VMInterface = new VMInterface();
    _VMInterface->setName( "KernelVMInterface" );
    StandardNameServer->bind( "vmInterface", _VMInterface );
}

FileSystemInterfaceRef
Kernel::fileSystemInterface()
{
    return( _fileSystemInterface );
}

/*
 * The Kernels "setup" Process finishes by running this method.
 * Basically, it just keeps building, dispatching, and waiting for top level
 * application Processes.
 */
void
Kernel::dispatchApplications()
{

    Debug << this << "dispatchApplications()\n" << eor;

#ifndef NONETWORKING    
#ifdef DISTRIBUTED
    StandardDistributedNameServer = new DistributedNameServer(0,new SendingTransport, new BufferedTransport);
    StandardNameServer->bind( "StandardDistributedNameServer", StandardDistributedNameServer);

#endif
#endif


    PersistentMemoryObjectStar applicationObjectFile ;
    while( ( applicationObjectFile =
             getMemoryObjectOfInitialApplicationProcess() ) != 0 ) {

        /*
         * Convert the MemoryObject into a Process.
         */
        Process * application = buildProcessFromObjectFile(
            applicationObjectFile, 0, (char **) 0 );

        if( application == 0 )
            continue;

        /*
         * Mark the process as ready to run.
         */
        Debug << "Readying top level application " << application
              << "\n" << eor;
        Assert( application != 0 );
        application->ready();
        Debug << "waiting for top level application to exit\n" << eor;
        applicationObjectFile  = (PersistentMemoryObject*)0;
        thisProcess()->waitForProcess( application );
        Debug << "application exited\n" << eor;
    }

    Debug << this << "::dispatchApplications(): done\n" << eor;
}

/*
 * The waste management process runs this code to delete terminated Processes.
 * (It finds them in the garbageCan). This is a class function not a member
 * function.
 */
void
Kernel::wasteManagementProcessEntry( Kernel * k )
{
    Debug << thisProcess() << " started\n" << eor;
    while(1) {
        Debug << "WasteManagementProcess: garbageCan->remove()\n"
              << eor;
        Debug << "_garbageCan = " << k->_garbageCan << "\n" << eor;
        Process * deadProcess = k->_garbageCan->remove();
        Assert( deadProcess != 0 );
        Debug << "WasteManagementProcess: delete " << deadProcess
              << "\n" << eor;
        deadProcess->unreference();
    }
}

/*
 * Setup machine-independent devices and managers.
 * Subclasses should overload basicInitializeDevicesAndManagers
 * to do machine-dependent stuff.
 */
void
Kernel::initializeDevicesAndManagers()
{
    Debug << this << "::initializeDevicesAndManagers()\n" << eor;

    /*
     * Start up the process responsible for emptying the garbageCan
     * whenever it is full.
     */
    Debug << "Building waste management process\n" << eor;

    Process * process = new SystemProcess(
        (PFV) Kernel::wasteManagementProcessEntry, domain(),
        normal, (int) this );
    Assert( process != 0 );
    process->setName( "#WasteManagerProcess" );
    process->ready();

    Debug << "Waste manager process: " << process << "\n" << eor;

    basicInitializeDevicesAndManagers();
}

#ifdef NETWORKING
void
Kernel::initializeNetworkProcesses()
{
    Debug << this << "::initializeNetworkProcesses()\n" << eor;

    //  Start up the machine-dependent portion of the networking
    //  subsystem (e.g. the drivers).
    basicInitializeNetworkProcesses();

#ifndef LINUXNET
#ifdef ZOOTCONTROLLER
    /*
     * Now launch ZOOTCONTROLLER:
     */
    extern volatile int ZootControllerInitialized;
    extern void ZootController(ProcessContainer *);
    Debug << "Building ZootController stack/process\n" << eor;

    Assert( _initialScheduler != 0 );
    Process * process = new SystemProcess( (PFV) ZootController, domain(),
                                           huge, (int)_initialScheduler );
    Assert( process != 0 );
    process->setName( "#ZootController" );

    process->ready();
    /* We _may_ have the only CPU. Give it up for the moment. */
    while( ZootControllerInitialized == 0 )
        thisProcess()->relinquishProcessor();

    Debug << "ZootController process done\n" << eor;
#endif /* ZOOTCONTROLLER */
#endif /* no LINUXNEXT */
}
#endif /*NETWORKING*/


/*
 * Method to build top level applications with. The arguments are the
 * MemoryObject containing the process binary and (FOR NOW!) a UNIX like
 * argc/argv argument count/vector.
 */

Process *
Kernel::buildProcessFromObjectFile( PersistentMemoryObject * ObjectFile,
                                    int argc, char * argv[] )
{
    DomainStar newDomain =
    new Domain ( TheKernel->domain(),
                 (char*)applicationStart(), applicationSize());

    Assert( newDomain != 0 );
    return( buildProcessFromObjectFile(
                ObjectFile, newDomain, argc, argv ) );
}

Process *
Kernel::buildProcessFromObjectFile( PersistentMemoryObject * ObjectFile,
                                    Domain *dom, int argc, char * argv[] )
{
    Debug << this << "::buildProcessFromObjectFile( ObjectFile:" 
          << ObjectFile << " argc:" << argc << " )\n";
#ifdef DEBUG
    for( int y = 0; y < argc; y++ ) {
        Debug << "argv[" << y << "] = " << argv[y] << "\n";
    }
#endif
    Debug << eor;

    /*
     * Add to the domain a mapped cache for each section in the file.
     * We dont reference the ObjectFile before passing it one because
     * were done with it here.
     */
    PFV entryPoint = (PFV) loadObjectFile( ObjectFile, dom );
    Debug << this << "::buildProcessFromObjectFile: entry = "
          << asAddress( (void*)entryPoint ) << "\n" << eor;

    /*
     * FIXME: This is certainly machine-dependent!
     */
    if( entryPoint == 0 ) {
        Console() << "Executable file format is not recognized\n" << eor;
        return( 0 );
    }

    /*
     * Handle the arguments.
     * Get a memory object to put them in and then add it to the current
     * domain long enough to stuff the args into it.
     * Once this is done remove it from the current domain and put
     * it in the sub-processes domain. Pass its address and argc
     * on to the sub-process.
     */

    int argvBytes = sizeof( char * ) * (argc + 1 + 32);
    int argsize = argvBytes;
    for( int j = 0; j < argc; j ++ ) {
        argsize += strlen( argv[j] ) + 1;
    }
    argsize += 1024;
    PersistentMemoryObjectStar memObj = allocateMemoryObjectForBackingStore();
    Assert( memObj != 0 );

    int unitSize = memObj->unitSize();
    memObj->setName( "ArgumentMO" );
    memObj->setNumberOfUnits( ( argsize + unitSize - 1 ) / unitSize );
    Debug << "Argument swap mo = " << memObj << "\n" << eor;

    /*
     * ReadOnly will still allow the kernel to write into it but not
     * allow the application to change it during the period it is
     * addressable.
     */
    //const char * argloc = thisProcess()->domain()->add( memObj, ReadOnly );
    const char * argloc = TheKernel->domain()->add( memObj, ReadOnly );
    TheKernel->domain()->makeAddressable(argloc, memObj->numberOfBytes());
    Debug << this << "::BPFCF temp (local) argloc at " <<
        asAddress( argloc ) << "\n" << eor;

    const char * childArgloc = dom->add( memObj, ReadWrite );
    dom->makeAddressable(childArgloc, memObj->numberOfBytes());
    Debug << this << "::BPFCF child argslog at "
          << asAddress( childArgloc ) << "\n" << eor;

    /*
     * Copy the argments into it one at a time. Save room for the new
     * argv vector first in the memory.
     */

    char ** newargv = (char **) argloc;
    char * argptr = (char *) argloc + argvBytes;

    for( int i = 0; i < argc; i++ ) {
        newargv[i] = (char *) childArgloc + (argptr - (char *) argloc);
        Debug << "Child argv[" << i << "] = "
              << asAddress( newargv[i] ) << "(" << argv[i]
              << ")\n" << eor;
        strcpy( argptr, argv[i] );
        argptr += strlen( argv[i] ) + 1;
    }
    newargv[argc] = 0;

    int arg0 = argc;
    int arg1 = (int) childArgloc;

    Debug << "Childargs " << arg0 << " " << asAddress( (void *) arg1 )
          << "\n" << eor;
    //thisProcess()->domain()->remove( memObj, argloc );
    TheKernel->domain()->remove( memObj, argloc );
    Debug << "memobj removed from Kernel domain\n" << eor;

    /*
     * Create the process.
     */
    Process * process = new ApplicationProcess( entryPoint, dom,
                                                arg0, arg1 );

    process->setName(ObjectFile->objectName());

    Debug << "BuildProcessFromObjectFile: process " << process
          << "\n" << eor;
    return( process );
}

PersistentMemoryObjectRef
Kernel::getMemoryObjectOfInitialApplicationProcess()
{
    Debug << this << "::getMemoryObjectOfInitialApplicationProcess()\n"
          << eor;

    PersistentMemoryObjectStar memobj ;

    while( memobj == 0 ) {
        char inBuf[128];
        Console() << "Enter path of binary "
                  << "executable application file: " << eor;
        ConsoleInputStream->read( inBuf, 128 );
        chop( inBuf, '\n' );
        inBuf[ 128 - 1 ] = '\0';
        /*
         * Parse into separate arguments.  Only blanks are supported
         * to separate arguments.
         */
        int argc;
        char *argvSpace[10];
        char **argv = &argvSpace[ 0 ];
        char *cp = &inBuf[0];
        while( *cp == ' ' )
            cp++;
        for ( argc = 0; ( argc < 10 ) && ( *cp != '\0' ); argc++ ) {
            argv[ argc ] = cp;
            while( ( *cp != '\0' ) && ( *cp != ' ' ) )
                cp++;
            if ( *cp != '\0' )
                *cp++ = '\0';
            while ( *cp == ' ' )
                cp++;
        }

        if (argc == 0)
            continue;

        int partNumber = atoi(argv[0]);
        if (tryCommand(argc, argv, partNumber) == 1)
            continue;

        if (partNumber == 321){ // Shutdown
#ifdef INSTRUMENT
            TheInstrumentManager->closeLogDumpFile();
#endif /*INSTRUMENT*/
            MemoryObjectCachingContainer::shutdown();
            Kernel::tryCommand(0,0,999);    //sync FS's
            this->shutdown();   // MD shutdown capability
            // Kernel::tryCommand(0,0,666);
            return 0;
        }

        Debug << "Reading the file: " << argv[0] << "\n"
              << eor;
        int status = 0;
        memobj = _fileSystemInterface -> open
        (argv [0], PersistentMemoryObjectClass, status);
        if (memobj == 0) {
            Console()
            << StatusCodes[status]  << ": "
            << argv[0]      << "\n" << eor;
        } else {
            if (!memobj -> access (AccessExecute)) {
                memobj = (MemoryObject*)0;
                Console() << argv[0]
                          << " not executable\n" << eor;
            }
        }

    }

    Debug   << "getMemoryObjectOfInitialApplicationProcess: return( "
            << memobj << " )\n" << eor;

    return( memobj );
}

KernelCmd* Kernel::installCmd(KernelCmd* cmd)
{
    KernelCmdInstaller::installCmd(cmd);
    return cmd;
}

typedef struct {
    int buf[20];    
    int head, tail,max;
    Semaphore *mutex;
    Semaphore *full;
    Semaphore *empty;
} queue ; 
    
void producer (void *arg) {
    queue * fifo = (queue *)arg;
    while (1) {
        fifo->empty->P();
        fifo->mutex->P();
        /*produce*/     
        Console() << "\nProducing!\n" << eor;
        fifo->buf[fifo->tail++]=1;
        if (fifo->tail == fifo->max)
            fifo->tail = 0;
        for (int i = 0; i < fifo->max; i++)
            Console() << fifo->buf[i] << eor;
        Console() << "\n" << eor;
        fifo->mutex->V();
        fifo->full->V();
    }   
    thisProcess()->die();
}

void consumer (void *arg) {
    queue * fifo = (queue*)arg;
    while(1) {
        fifo->full->P();
        fifo->mutex->P();
        /*consume*/     
        
        Console() << "\nConsuming!\n" << eor;
        fifo->buf[fifo->head++]=0;
        if (fifo->head == fifo->max)
            fifo->head = 0;
            
        for (int i = 0; i < fifo->max; i++)
            Console() << fifo->buf[i] << eor;
        Console() << "\n" << eor;
        
        fifo->mutex->V();
        fifo->empty->V();
    }
    thisProcess()->die();
}

void hello_world_thread(void *arg) {
    int size = 10;
    queue * fifo = new queue;
    fifo->mutex = new Semaphore(1,1);
    fifo->empty = new Semaphore(size,size);
    fifo->full = new Semaphore(0,size);
    fifo->head = fifo->tail=0;
    fifo->max = size;
    Console() << "\nProducer Consumer!\n" << eor;
    
    Process * process1 = new PreemptableSystemProcess((PFV) consumer, thisProcess()->domain(), normal, (int) fifo );
    Assert( process1 != 0 );
    process1->setName( "#Consumer Process" );
    Process * process2 = new PreemptableSystemProcess((PFV) producer, thisProcess()->domain(), normal, (int) fifo );
    Assert( process2 != 0 );
    process2->setName( "#Producer Process" );
    process2->ready();
    process1->ready();

    /*while(1) {
        fifo->full->P();
        fifo->mutex->P();
        //consume   
        
        Console() << "\nConsuming!\n" << eor;
        fifo->buf[fifo->head++]=0;
        if (fifo->head == fifo->max)
            fifo->head = 0;
            
        for (int i = 0; i < fifo->max; i++)
            Console() << fifo->buf[i] << eor;
        Console() << "\n" << eor;
        
        fifo->mutex->V();
        fifo->empty->V();
    }
    */

    thisProcess()->waitForProcess(process1);
    thisProcess()->waitForProcess(process2);
    Console() << "\nProducer Consumer Ending!\n" << eor;
    
    thisProcess()->die();

}

void hello_world(void *arg) {
    PeriodicTimer *p=new PeriodicTimer();
    Console() << "Starting, with timer: "<<p->start(4096)<<'\n' << eor;
    for(int i=10;i>0;i--){
        Console()<< i <<'\n' << eor;
        p->await();
    }
    Console()<< "Stopping, with timer: "<<p->stop()<<'\n' << eor;
    Console() << "Hello World!\n" << eor;
    delete p;
    thisProcess()->die();
}

void hello_world_loop(void *arg) {
    int a = 0, b = 0;
    while(1) {
        // The delay caused by the following loop needs to be tuned to test different
        // platforms. 20,000 works for ARM-OMAP; 2,000,000 works for ARM-Integrator on 3Ghz Pentium
        for (int i = 0; i < 20000; i++) a++;
        Console() << "Hello World! " << asHex(arg) << ":" << b << "\n" << eor;
        a = 0;
        b++;
        if ( b == 100 ) break;
    }
    thisProcess()->die();
}

void null_pointer_dereference() {
    *(unsigned *)0 = 0;
    // Francis
    // This Console function call is needed because arm-none-eabi from 
    // codesourcery is broken and doesn't generate
    // table entries for leaf functions?
    Console() << "Hello!\n" << eor;
}

void exception_test(void *arg) {
    int a = 0, b = 0, c = 0;

    while(1) {
        for (int i = 0; i < 20000; i++) a++;
        Console() << "Exception Test Loop: " << asHex(arg) << ":" << c << "\n" << eor;
        a = 0;
        c++;
        try {
            // Francis
            // Need to call function that dies.. because arm-none-eabi- from codesourcery
            // doesn't seem to find local catch blocks - maybe because non-call-exceptions is broken?
            if (c == 10) { null_pointer_dereference(); }
        } catch (ProcessorException *e) {
            Console() << "Caught Exception " << *e << ":"
                << e->getMessage() << "\n" << eor;
            e->displayContext();
            e->displayStackTrace();
            c = 0;
            b++;
            delete e;
        }
        if ( b == 10 ) break;
    }
    thisProcess()->die();
}

void microreboot_trace(void *arg) {
    Kernel *kernel = (Kernel *)arg;
    char *data = new char[kernel->rootDisk()->unitSize()];
    AssertOrThrow(data != NULL);

    for(int i = 0; i < 10; i++) {
        MICROREBOOTABLE(kernel->rootDisk(), read(i, 1, data,
                    kernel->rootDisk()->unitSize()), MICROREBOOT_RETRY, 5);

        Console() << "Block " << i << ":\n" << eor;
        for(unsigned j = 0; j < kernel->rootDisk()->unitSize();) {
            Console() << asHex(data[j]) << " " << eor;

            if((++j % 16) == 0)
                Console() << "\n" << eor;
        }
        Console() << "\n\n" << eor;
    }
    thisProcess()->die();
}

void restart_test() {
    int a = 0, b = 0, c = 0;
    while(1) {
        for (int i = 0; i < 20000; i++) a++;
        Console() << "Exception Test Loop: " << c << "\n" << eor;
        a = 0;
        c++;
//      try {
        if (c == 10) { *(unsigned *)0 = 0; }
//      } catch (Exception *e) {
//          Console() << "Caught Exception " << *e << "\n" << eor;
//          c = 0;
//          b++;
//      }
        if ( b == 10 ) break;
    }
    thisProcess()->die();
}

unsigned perfStartTime = 0;

/* To run this test, the data abort exception handler should just increment PC */
void perfRestartOld() {
    static int first = 0;
    unsigned perfEndTime,t;
 
    // Uncomment for spawn performance
    perfEndTime = SystemTimer->time();
    t = perfEndTime - perfStartTime;
    Console() << "Spawn Performance normal end: " << perfEndTime << " start: " << perfStartTime << " took " << t << "\n" << eor;
    thisProcess()->die();

    if (!first) {
        first = 1;
        perfStartTime = SystemTimer->time();
        *(unsigned *)0 = 0;
        Process *process = new SystemProcess(perfRestartOld, TheKernel->domain(), normal, (int) 0 );
        Assert( process != 0 );
        process->setName( "#RestartPerformanceTestProcess" );
        process->ready();
        thisProcess()->die();
    }

    perfEndTime = SystemTimer->time();
    t = perfEndTime - perfStartTime;
    Console() << "Restart Old Performance end: " << perfEndTime << " start: " << perfStartTime << " took " << t << "\n" << eor;
    perfStartTime = SystemTimer->time();
    first = 0;
    thisProcess()->die();
}

void perfRestartNew() {
    static int first = 0;
    unsigned perfEndTime,t;

    // Uncomment for spawn performance
    perfEndTime = SystemTimer->time();
    t = perfEndTime - perfStartTime;
    Console() << "Spawn Performance Restartable end: " << perfEndTime << " start: " << perfStartTime << " took " << t << "\n" << eor;
    thisProcess()->die();

    if (!first) {
        first = 1;
        perfStartTime = SystemTimer->time();
        *(unsigned *)0 = 0;
    }
    perfEndTime = SystemTimer->time();
    t = perfEndTime - perfStartTime;
    Console() << "Restart New Performance end: " << perfEndTime << " start: " << perfStartTime << " took " << t << "\n" << eor;
    perfStartTime = SystemTimer->time();
    first = 0;
    thisProcess()->die();
}

extern int doNothing();
void tryFunc(int mode) {
//    if (mode) {
//        try {
//            doNothing();
//        } catch (...) {}
//    }
}

void perfTry() {
    unsigned NTIMES = 20000;
    unsigned i;
    
    IntStorage intState ; PROCESSOR::disableInterrupts(intState);

    unsigned start = SystemTimer->time();
    for( i = 0; i < NTIMES; i++ ) {
        tryFunc(0);
    }
    unsigned int end = SystemTimer->time();
    
    int over = end - start;
    Console() << "Loop Overhead = " << over << "\n" << eor;
    
    start = SystemTimer->time();
    for( i = 0; i < NTIMES; i++ ) {
        tryFunc(1);
    }
    end = SystemTimer->time();
    
    int t = (end - start - over) / NTIMES;
    
    Console() << "Try Performance end: " << end << " start: " << start << " took " << t << "\n" << eor;
    
    PROCESSOR::restoreInterrupts(intState);
}

/*
  void switchAndInvoke(OPFV method, Object *object) {
  // Save current domain
  Domain *currentDomain = thisProcess()->currentDomain();
  Exception *ex = 0;

  // Get domain of object (All target objects of switch and invoke have an associated domain)
  // Domain *domain = object->domain();
  Domain *domain = TheKernel->domain();

  // Allocate new stack for process and place it into new domain
  // allocateStack()

  // Push current stack into Process's stack of stacks

  try{
  // Copy current frame to new stack
        
  thisProcess()->setCurrentDomain(domain);
  // Activate new domain
  CPU * cpu = thisCPU();
  if( cpu->MMU()->currentlyActiveDomain() != domain ) {
  cpu->MMU()->activate( domain );
  }
        
  // Perform call
  (object->*method)();
  } catch (Exception *e) {
  Console() << "Caught a cross domain exception\n";
  ex = e;
  }

  // We are now on the called domain's stack, switch back
  // Pop stack from Process's stack of stacks

  // Restore domain
  thisProcess()->setCurrentDomain(currentDomain);
  // Activate new domain
  CPU * cpu = thisCPU();
  if( cpu->MMU()->currentlyActiveDomain() != currentDomain ) {
  cpu->MMU()->activate( currentDomain );
  }

  // the exception object is in a area shared with calling domain
  // If an exception was caught Rethrow the exception
  if (ex) throw ex;

  return;
  }
*/

int
Kernel::tryCommand(int argc, char *argv[], int partnumber)
{
    int result = 1;
    int numInstances ;

//try user-defined code first
//  Console() << "Trying... \"" << argv[0] << "\"\n" << eor;
    KernelCmdInstaller* installer = KernelCmdInstaller::getCmdInstaller();
    if(installer == NULL) {
        //Console() << "No user commands defined...\n" << eor;
    } else {
        
        KernelCmd* cmd = installer->find(argv[0]);

        if(cmd == NULL) {
            //      Console() << "No such user command found\n" << eor;
        } else {
            cmd->execute();
            return 1; //success
        }
    }

    switch( partnumber ) {
        case 999:
            Console() << "Syncing filesystems... " << eor;
            if (_fileSystemInterface != 0)
                _fileSystemInterface->synchronize();
            if (_backingStoreSource != 0)
                _backingStoreSource->synchronize();
            Console() << "Wait for 4 secs... " << eor;  
            {
                TimeoutTimer timer;
                timer.start( 4 * 1000000 );
                timer.await();
            }
            Console() << "done.\n" << eor;
            break;
        case 747:
        {
            perfStartTime = SystemTimer->time();
            Process *process = new SystemProcess(perfRestartOld, TheKernel->domain(), normal, (int) 0 );
            Assert( process != 0 );
            process->setName( "#RestartOldPerformanceTestProcess" );
            process->ready();
            break;
        }
        case 748:
        {
            perfStartTime = SystemTimer->time();
            RestartableProcess *p = new RestartableProcess(perfRestartNew, 0 & RESTART_LIMIT);
            Process *process = new SystemProcess((OPFV) &RestartableProcess::run, domain(), normal, (int) p );
            Assert( process != 0 );
            process->setName( "#RestartNewPerformanceTestProcess" );
            process->ready();
            break;
        }
        case 749:
        {
            perfTry();
            break;
        }
        case 1233:
        {
            // Process Preemption
            Process *process = new PreemptableSystemProcess((PFV) hello_world_thread, domain(), normal, 1 );
            Assert( process != 0 );
            process->setName( "#HelloWorldProcess 1" );
            process->ready();
            break;
        }
        case 1234:
        {
            // Hello World
            Process * process = new SystemProcess((PFV) hello_world, domain(), normal, (int) this);
            Assert( process != 0 );
            process->setName( "#HelloWorldProcess" );
            process->ready();
            break;
        }
        case 1235:
        {
            // Process Preemption
            Process *process = new PreemptableSystemProcess((PFV) hello_world_loop, domain(), normal, 1 );
            Assert( process != 0 );
            process->setName( "#HelloWorldProcess 1" );
            process->ready();
            process = new PreemptableSystemProcess((PFV) hello_world_loop, domain(), normal, 2 );
            Assert( process != 0 );
            process->setName( "#HelloWorldProcess 2" );
            process->ready();
            break;
        }
        case 1236:
        {
            // Exception Handling Test
            // Test local exceptions
            try {
                throw 1;
            } catch(...) {
                Console() << "Caught locally thrown exception\n" << eor;
            }
            Process *process = new SystemProcess((PFV) exception_test, domain(), normal, (int) this );
            Assert( process != 0 );
            process->setName( "#ExceptionTestProcess" );
            process->ready();
            break;
        }
        case 1237:
        {
            // Restartable Process Test
            RestartableProcess *p = new RestartableProcess(restart_test, 0 & RESTART_LIMIT);
            Process *process = new SystemProcess((OPFV) &RestartableProcess::run, domain(), normal, (int) p );
            Assert( process != 0 );
            process->setName( "#RestartTestProcess" );
            process->ready();
            break;
        }
        case 1238:
        {
            // Domain switching and cross domain exceptions
            //Process *process = new SystemProcess( (PFV) hello_world,
            //                                     new Domain(TheKernel->domain(),(char*)hello_world, 180000), 
            //                                      normal,
            //                                      (int) this );
            // We just create the process object to make a cross
            // domain call into process
            //switchAndInvoke((OPFV)&Process::ready,process);
            break;
        }
        case 1239:
        {
            try {
                AssertOrThrow(NOTREACHED);
            }
            catch(Exception *e) {
                Console() << "Caught exception " << *e << ": " 
                          << e->getMessage() << "\n" << eor;
            }
            break;
        }
        case 1240: /* microreboot testing */
        {
            Process *process = new SystemProcess((PFV)microreboot_trace,
                                                 domain(), normal, (int) this );
            Assert( process != 0 );
            process->setName( "#DiskBlockReadProcess" );
            process->ready();
            break;
        }
        case 1241: // FileSystem Read/Write Test
        {
            int status;
            try {
                PersistentMemoryObjectStar mo =
                _fileSystemInterface->open("/fstest",
                                           PersistentMemoryObjectClass, status,
                                           O_CREAT,O_RDRW);
                Console() << "Status was " << status << "\n" << eor;
                Assert(mo != NULL);
                mo->setNumberOfBytes(4096);
                char *buf = new char[4096];
                for (int i = 0; i < 1024; i++) ((unsigned *)buf)[i] = 0xcafebabe;
                mo->writeThrough(0,1,buf,4096);
                _fileSystemInterface->synchronize();
                PersistentMemoryObjectStar mo_read =
                _fileSystemInterface->open("/fstest",
                                           PersistentMemoryObjectClass, status,
                                           O_CREAT,O_RDRW);
                Console() << "Status was " << status << "\n" << eor;
                Assert(mo_read != NULL);
                mo_read->read(0,1,buf,4096);
                for(unsigned j = 0; j < 4096;) {
                    Console() << asHex(buf[j]) << " " << eor;

                    if((++j % 16) == 0)
                        Console() << "\n" << eor;
                }
            } catch(Exception *e) {
                Console() << "Caught Exception: " << asAddress(e) << "\n" << eor;
            }
            break;
        }
        case 912:
        {
            // Test case for spawning app
            Console().formattedWrite("Creating Application process\n");
            Process *appProcess = new ApplicationProcess((PFV) hello_world, domain(), (int) this, 0, normal );
            Assert( appProcess != 0 );
            appProcess->setName( "#HelloWorldAppProcess" );
            Console().formattedWrite("Ready to run Application process\n");
            appProcess->ready();
            break;
        }
        case 916:
        {
            int status;
            try {
                PersistentMemoryObjectStar mo =
                _fileSystemInterface->open("/hello.bin",
                                           PersistentMemoryObjectClass, status);
                Assert( mo != NULL );

                Domain *appDomain = new Domain(TheKernel->domain(),
                                               (char*)TheKernel->applicationStart(),
                                               TheKernel->applicationSize());
                appDomain->setName( "#HelloWorldBinaryDomain" );

                //changed to ReadWrite; this is a needed crunch because the loader can't
                //distinguish between .text and .data section
                const char *base = appDomain->add(mo, ReadWrite);

                Console() << "base is " << asAddress(base) << "\r\n"
                          << eor;

                ApplicationProcess *appProcess =
                new ApplicationProcess((PFV)TheKernel->applicationStart(), appDomain,
                                       0, 0, normal);
                Assert( appProcess != 0 );

                appProcess->setName( "#HelloWorldBinaryProcess" );
                Console().formattedWrite(
                    "Ready to run Application process\n");

                try {
                    appProcess->ready();
                }
                catch(...) {
                    Console() << "something bad happened\n" << eor;
                }

                Console() << "hmm... did anything happen?\n" << eor;
            }
            catch(Exception *e) {
                Console() << "Caught another exception: " <<
                asAddress(e) << "\n" << eor;
            }
              
            break;
        }
        case 917:
        {
            volatile int dividend = 99;
            volatile int divisor = 0;
            volatile int quotient = 0xCAFEBABE;

            Console() << "Dividend: " << dividend << " Divisor: "
                      << divisor << "\n" << eor;

            quotient = dividend / divisor;

            Console() << "Quotient: " << quotient << "\n" << eor;

            break;
        }
        case 918:
        {
            int status;

            try {
                MemoryObjectDictionaryStar dictionary =
                _fileSystemInterface->open("/hello.elf",
                                           ELFDictionaryClass, status);
                AssertOrThrow( dictionary != NULL );

                Console() << dictionary << "::keySize() = "
                          << dictionary->keySize() << "\n" << eor;

                int keySize = dictionary->keySize() + 1;
                char *keys = new char[keySize];

                AssertOrThrow(keys != NULL);

                int count = 0;
                dictionary->keys(keys, keySize, count);

                keys[keySize] = '\0';

                Console() << dictionary << "::keys count="
                          << count << " keys=\n" << keys << "\n" << eor;

                delete[] keys;
            }
            catch(Exception *e) {
                Console() << "Caught " << e << ": " << e->getMessage()
                          << "\n" << eor;
            } 

            break;
        }

        case 9999:
            MemoryObjectCachingContainer::shutdown();
            break;

        case 9990:
            extern void UNIXInodeCache_cmd(int argc, char * argv[]);
            UNIXInodeCache_cmd( argc, argv );
            break;

        case 911:
            extern void RaidDump();
            RaidDump();
            break;

        case 888:
            numInstances = ObjectClass->displayKindred();
            Console()   << "Live objects "
                        << numInstances << "\n" << eor;
            break;

        case 777:
            extern void times();
            times();
            break;

        case 778:
            extern void lotsaCalls(int, char *[]);
            argv++;
            argc--;
            lotsaCalls(argc, argv);
            break;

        case 776:
            extern void timeDisks(int, char* []);
            argv++;
            argc--;
            timeDisks(argc, argv);
            break;

#ifdef BSDNET
        case 1966:
            extern int UdpEchoServer(int, char* []);
            argv++;
            argc--;
            UdpEchoServer(argc, argv);
            break;
        case 1967:
            extern int UdpEchoClient(int, char* []);
            argv++;
            argc--;
            UdpEchoClient(argc, argv);
            break;
        case 1968:
            extern int TcpEchoServer(int, char* []);
            argv++;
            argc--;
            TcpEchoServer(argc, argv);
            break;
        case 1969:
            extern int TcpEchoClient(int, char* []);
            argv++;
            argc--;
            TcpEchoClient(argc, argv);
            break;
        case 1970:
            extern int debugBSD;
            if ( argc > 1 ) 
                debugBSD = atoi( argv[1] );
            Console() << "debugBSD = " << debugBSD << "\n" << eor;
            break;
#endif
#ifdef RemoteFileSystem
        case 1204:
            RemoteServer::serverInit( 4 );
            break;
        case 1205:
            RemoteFileSystemRPC::clientInit();
            break;
        case 1212:
            extern void RemoteFileSystemStatistics_cmd
            (int argc, char *argv[]);
            RemoteFileSystemStatistics_cmd(argc, argv);
            break;
#endif

        case 666:
            Console()   << "kernelHeap->pagesUsed(): "
                        << _kernelHeap->pagesUsed()
                        << " of "
                        << _kernelHeap->sizeInPages()
                        << " total "
                        << "\n" << eor;
            MainStore->info( Console() );
            extern void heap_info();
            heap_info();
            extern void UNIXInodeIndirect_info();
            UNIXInodeIndirect_info();
            break;
        
        case 4444:
            extern void testMadAllocator(int, int);
            testMadAllocator(atoi(argv[1]), atoi(argv[2]));
            break;
        case 4445:
            extern void overflowKernelStack(unsigned);
            overflowKernelStack(0);
            break;
#ifdef DISTRIBUTED
        case 7000:
            extern void startRPCServer();
            startRPCServer();
            break;
        case 7001:
            extern void startRPCClient();
            startRPCClient();
            break;
        case 7002:
            extern void startServer();
            startServer();
            break;
        case 7003:
            extern void startClient();
            startClient();
            break;
        case 67:
            extern void  doEthernetTests();
            doEthernetTests();
            break;
#endif
        case 555:
            thisCPU()->debugBreakpoint();
            break;

        case 123:
            void switchtimes();
            switchtimes();
            break;
        case 124:
            Assert (1 == 0)("Ooops");
            break;

#ifdef INSTRUMENT
        case 111:
            TheInstrumentManager->deathLog ();
            TheInstrumentManager->closeLogDumpFile();
            break;
#endif          
#ifdef INSTRUMENT
        case 113:
            TheInstrumentManager->deathLog ();
            break;
#endif          

#ifdef INSTRUMENT
#ifdef INSTRUMENT_KERNEL
        case 112:
            instrumentTimes ();
            break;
#endif
#endif
#ifdef SAMPLEPC
        case 22:
            //  Display 1000 samples of the PC.
            //  See SS1TimerManager.cc.
            extern int pcSampleMax;
            extern char *pcSample [];
            Console () << "pcSampleMax = " << pcSampleMax <<
            "\n" << eor;
            for (int i = 0; i <= pcSampleMax; i++)
                Console () << asAddress (pcSample [i])
                           << "\n" << eor;
            break;
#endif
        default:
            /*
             *  Process Raid commands
             */
            if ( ! strcmp(argv[0], "odb")) odb(argc, argv);
            else if  ( ! strcmp(argv[0], "cdb")) cdb(argc, argv);
            else if  ( ! strcmp(argv[0], "ancestors")) hierarchy(argc, argv);
            else if  ( ! strcmp(argv[0], "descendants")) hierarchy(argc, argv);
            else if  ( ! strcmp(argv[0], "hierarchy")) hierarchy(argc, argv);
            else if  ( ! strcmp(argv[0], "instance")) instance(argc, argv);
            else if  ( ! strcmp(argv[0], "members")) instances(argc, argv);
            else if  ( ! strcmp(argv[0], "kindred")) instances(argc, argv);
#ifdef SIMPLE_INSTRUMENTATION
            else if  (! strcmp(argv[0], "inst")) simpleInstrumentControl(argc, argv);
#endif
            else
                result = 0;
            break;
                
    }

    return result;
}

PersistentMemoryObjectRef
Kernel::allocateMemoryObjectForBackingStore()
{
    Assert( _backingStoreSource != 0 );
    int status = 0;
    PersistentMemoryObjectRef mo = _backingStoreSource->create
    ( 0, PersistentObjectClass, status );
    Debug << this << "::allocateMem...BackingStore()" << mo << "\n" << eor;

    /*
     * Worry about errors later. This whole thing is a hack anyway.
     */
    Assert( status == 0 );
    Assert( mo != (PersistentMemoryObject*)0 );
    return( mo );
}

MemoryObjectContainer *
Kernel::backingStoreSource()
{
    return _backingStoreSource;
}

/*
 * Uni processor kernels can just inherit this.
 */
void
Kernel::startOtherProcessors()
{
}


OutputStream &
Console()
{
    if( thisProcess() != 0 ) {
        return( thisProcess()->console() );
    } else if( ConsoleOutputStream != 0) {
        return( *ConsoleOutputStream );
    } else {
        /*
         * Before even the ConsoleOutputStream exists, any output
         * is discarded.  This should be for just a few hundred
         * instructions after the system boots.
         */
        return( TheNullOutputStream );
    }
}

#ifdef INSTRUMENT
void
setupInstrumentation()
{
    /*
     * Create all instrumentation objects for the system.
     */

    Debug << "Start to make instrument objects.\n" << eor;

    TheInstrumentManager = new SystemInstrumentManager
    ("System", 1000, 100);
    TheInstrumentManager->reference();
    StandardNameServer->bind( "TheSystemInstrumentManager",
                              TheInstrumentManager);

    Debug << "Instrument manager created.\n" << eor;

    extern void SystemInstrumentManagerConstructInstruments ();
    SystemInstrumentManagerConstructInstruments ();

#ifdef INSTRUMENT_KERNEL    
    extern void KernelConstructInstruments ();
    KernelConstructInstruments ();
#endif

#ifdef INSTRUMENT_CPU
    extern void CPUConstructInstruments ();
    CPUConstructInstruments ();
#endif  

#ifdef INSTRUMENT_DOMAIN
    extern void DomainConstructInstruments ();
    DomainConstructInstruments ();
#endif  

#ifdef INSTRUMENT_INTERRUPT
    extern void InterruptConstructInstruments ();
    InterruptConstructInstruments ();
#endif  

#ifdef INSTRUMENT_PROCESS
    ProcessConstructInstruments ();
#endif  

#ifdef INSTRUMENT_IO
    iIO = new EventerInstrument(TheInstrumentManager,"IO",ASLEEP);
#endif

#ifdef MULTIMAX
#ifdef INSTRUMENT_SVCTRAP
    extern void SVCTrapConstructInstruments ();
    SVCTrapConstructInstruments ();
#endif  
#endif MULTIMAX

    Debug << "Done making system instrument objects.\n" << eor;

    // Once the file is created, we can wake up the instruments
    if (TheInstrumentManager->openInitialLogDumpFile() == FALSE) {
        // What if the log doesn't open?
    }

    // Once the file is created, we can wake up the instruments
    TheInstrumentManager->beginAllInstruments ();

#ifdef INSTRUMENT_PROCESS
    iProcessRestore->startAll();
#endif

#ifdef INSTRUMENT_IO
    iIO->changeState(TALKING);
#endif
}

void
cleanupInstrumentation()
{
    Debug << "Deleting instrument objects.\n" << eor;

#ifdef MULTIMAX
#ifdef INSTRUMENT_SVCTRAP
    extern void SVCTrapDeleteInstruments ();
    SVCTrapDeleteInstruments ();
#endif  
#endif MULTIMAX

#ifdef INSTRUMENT_IO
    delete iIO;
    iIO = NULL;
#endif

#ifdef INSTRUMENT_DOMAIN
    extern void DomainDeleteInstruments ();
    DomainDeleteInstruments ();
#endif  

#ifdef INSTRUMENT_PROCESS
    extern void ProcessDeleteInstruments ();
    ProcessDeleteInstruments ();
#endif  

#ifdef INSTRUMENT_INTERRUPT
    extern void InterruptDeleteInstruments ();
    InterruptDeleteInstruments ();
#endif  

#ifdef INSTRUMENT_CPU
    extern void CPUDeleteInstruments ();
    CPUDeleteInstruments ();
#endif  

#ifdef INSTRUMENT_KERNEL    
    extern void KernelDeleteInstruments ();
    KernelDeleteInstruments ();
#endif INSTRUMENT_KERNEL    

    extern void SystemInstrumentManagerDeleteInstruments ();
    SystemInstrumentManagerDeleteInstruments ();

    TheInstrumentManager->unreference();
    Debug << "Instrument manager deleted.\n" << eor;

    Debug << "Done making system instrument objects.\n" << eor;
}

#ifdef INSTRUMENT_KERNEL
void instrumentTimes ()
{
    IntStorage state ; PROCESSOR::disableInterrupts(state);
    for (int bigloop = 0; bigloop < 100; bigloop++) {
        register uint loop;
                
        iTimerLatencyCheck->changeFreq (1);
        iTimerLatencyCheck->reset ();
                        
        // tBetween
        iTimerLatencyCheck->start ();
        iTimerLatencyCheck->stop (LatencyCheckEvent, "tBetween");

        // 1000*loopTime + tBetween
        iTimerLatencyCheck->start ();
        for (loop = 0; loop < 1000; loop++) {
        }
        iTimerLatencyCheck->stop (LatencyCheckEvent,
                                  "1000*loopTime + tBetween");

        // eEvent + eLog + tBetween
        iTimerLatencyCheck->start ();
        iEventCheck->event (EventerCheckEvent);
        iTimerLatencyCheck->stop (LatencyCheckEvent, 
                                  "eEvent + eLog + tBetween");

        // eEvent + eLog + 40*sData + tBetween
        iTimerLatencyCheck->start ();
        iEventCheck->event (EventerCheckEvent,
                            "0123456789012345678901234567890123456789");
        iTimerLatencyCheck->stop (LatencyCheckEvent, 
                                  "eEvent + eLog + 40*sData + tBetween");

        // eEvent + eLog + 40*dData + tBetween
        iTimerLatencyCheck->start ();
        iEventCheck->event (EventerCheckEvent,
                            "0123456789012345678901234567890123456789", 40);
        iTimerLatencyCheck->stop (LatencyCheckEvent,
                                  "eEvent + eLog + 40*dData + tBetween");

        // cEvent + cLog + tBetween
        iCounterCheck->changeFreq (1);
        iCounterCheck->reset ();
        iTimerLatencyCheck->start ();
        iCounterCheck->event (CounterCheckEvent);
        iTimerLatencyCheck->stop (LatencyCheckEvent,
                                  "cEvent + cLog + tBetween");
                        
        // 1000*cEvent + 1000*loopTime + tBetween
        iCounterCheck->changeFreq (1001);
        iCounterCheck->reset ();
        iTimerLatencyCheck->start ();
        for (loop = 0; loop < 1000; loop++) {
            iCounterCheck->event (CounterCheckEvent);
        }
        iTimerLatencyCheck->stop (LatencyCheckEvent,
                                  "1000*cEvent + 1000*loopTime + tBetween");

        // tEvent + tLog + tBetween
        iTimerCheck->changeFreq (1);
        iTimerCheck->reset ();
        iTimerLatencyCheck->start ();
        iTimerCheck->start ();
        iTimerCheck->stop (TimerCheckEvent);
        iTimerLatencyCheck->stop (LatencyCheckEvent,
                                  "tEvent + tLog + tBetween");
                        
        // 1000*tEvent + 1000*loopTime + tBetween
        iTimerCheck->changeFreq (1001);
        iTimerCheck->reset ();
        iTimerLatencyCheck->start ();
        for (loop = 0; loop < 1000; loop++) {
            iTimerCheck->start ();
            iTimerCheck->stop (TimerCheckEvent);
        }
        iTimerLatencyCheck->stop (LatencyCheckEvent,
                                  "1000*tEvent + 1000*loopTime + tBetween");
    }

    PROCESSOR::restoreInterrupts(state);
}
#endif  //  INSTRUMENT_KERNEL
#endif /*INSTRUMENT*/
#include "ClassInstaller.h"
DEFINECLASS("Kernel",Kernel,Object,0,0);
