#include "NewAndDelete.h"
#include "KernelCmdInstaller.h"
#include "OutputStream.h"
#include "ClassStar.h"

KernelCmdInstaller* KernelCmdInstaller::self = 0;

KernelCmd::KernelCmd(Object* obj, OPFV entry, char* name, char* description)
{
	myName = name;
	myDescription = description;
	object = obj;
	entryPoint = entry;
	fentry = 0;	
}

KernelCmd::KernelCmd()
{
	myName = "No Name";
	myDescription = "No description";
	object = NULL;
	entryPoint = NULL;
	fentry = NULL;
}

KernelCmd::KernelCmd(PFV entry, char* name, char* description)
{
	myName = name;
	myDescription = description;
	object = 0;
	entryPoint = 0;
	fentry = entry;	
}

void KernelCmdInstaller::printCommands()
{
	Console() << "List of Commands:--------\n" << eor;
		
	map<string, KernelCmd*>::iterator iter = _kernelCommands.begin();

	while(iter != _kernelCommands.end() ) {
		Console() << (iter->first).c_str() << "\n" << eor;
		iter++;
	} 
	Console() << "End list of Commands:--------\n" << eor;
}

void KernelCmdInstaller::runTests() {
	runWithPrefix("test");
}

void KernelCmdInstaller::runWithPrefix(char* prefix)
{
	Console() << "Running command(s) with prefix:" << prefix << " --------\n" << eor;
	
	map<string, KernelCmd*>::iterator iter = _kernelCommands.begin();
	
	int n = 0;
	char* name;
	
	while(iter != _kernelCommands.end()) {
		name = (char*) (iter->first).c_str();
		if (!strncmp(name, prefix, strlen(prefix))) {
			Console() <<  "Running cmd #" << ++n << " " << name << "\n" << eor;
			_kernelCommands[name]->execute();
		}
		
		iter++;
	}
	
	Console() << "End tests: ran " << n << " tests\n" << eor;
}


KernelCmd* KernelCmdInstaller::installCmd(KernelCmd* cmd)
{
	KernelCmdInstaller* installer = KernelCmdInstaller::getCmdInstaller();  
	if(installer == 0) {
		Debug << "Creating command installer\n" << eor;
		installer = new KernelCmdInstaller(); 
		KernelCmdInstaller::setCmdInstaller(installer);

		installer->_kernelCommands["?"] = new KernelCmd((Object*)installer, (OPFV)&KernelCmdInstaller::printCommands, "?", "Print a list of user-defined commands");
		installer->_kernelCommands["runTests"] = new KernelCmd((Object*)installer, (OPFV)&KernelCmdInstaller::runTests, "runTests", "Run all user-defined test cases");
	}
	Debug << "Installing command \"" << cmd->getName() << "\" at " << asHex(cmd) << "\n" << eor;

	installer->_kernelCommands[cmd->getName()] = cmd;
	
	//KernelCmdInstaller::getCmdInstaller()->printCommands();
	return cmd;
}

void KernelCmd::execute() 
{
	Console() << "Executing command \"" << myName << "\"\n" << eor;
	if(object == 0) { //PFV
		(*fentry)();
	} else {
		(object->*entryPoint)();
	}
}

KernelCmd* KernelCmdInstaller::find(string name) 
{
	map<string, KernelCmd*>::iterator iter = _kernelCommands.find(name);
	return (iter != _kernelCommands.end()) ? iter->second : (KernelCmd*) NULL;
}
#include "ClassInstaller.h"
DEFINECLASS("KernelCmdInstaller",KernelCmdInstaller,Object,0,0);
