#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David Raila (raila@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "Assert.h"
#include "Lock.h"
#include "PROCESSOR.h"
#include "Process.h"
#include "ThisProcess.h"


#ifdef __GNUC__
#pragma implementation
#endif

BusyWait::BusyWait()
{
	_held = 0;
	PROCESSOR::initMutexState(&_state);
}

BusyWait::~BusyWait()
{
	Assert (this != 0);
	Assert (!held());
}

int
BusyWait::heldByMe()
{
	Assert (this != 0);
	return ( _held && PROCESSOR::cpuID() == _holder );
}

void
BusyWait::acquire()
{
	Assert (this != 0);
	PROCESSOR::spinMutex(&_state);
	Assert (! held());
	_held = 1;
	_holder = PROCESSOR::cpuID();
	_process = thisProcess();
	if (_process) _process->acquiredLock(this);
}

void
BusyWait::release()
{
	Assert (this != 0);
	Assert( held() );
	_held = 0;
	_holder = 0;
	if (_process) _process->releasedLock(this);
	PROCESSOR::releaseMutex(&_state);
}

void
Lock::acquire()
{
	Assert( this != 0 );
	PROCESSOR::disableInterrupts(_interruptState);
	BusyWait::acquire();
}

void
Lock::release()
{
	Assert (this != 0);
	Assert (! PROCESSOR::interruptsEnabled());
	Assert ( heldByMe());
	BusyWait::release();
	PROCESSOR::restoreInterrupts(_interruptState);
}

LockKeeper::LockKeeper(Lock *lock)
{
    _lock = lock;
    _lock->acquire();
}

LockKeeper::~LockKeeper()
{
    _lock->release();
}
