#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801-2910
 *
 * Copyright (c) 1991 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Amit Parghi (parghi@cs.uiuc.edu)
 * Contributing Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *	MachineConfiguration.cc
 *
 *	Code for the MachineConfiguration object, which maintains
 *	site-specific configuration data for the kernel.
 *
 */


// #define DEBUG 1
//#define RAID 1

#include "Raid.h"
#include "FileSystem.h"
#include "FileSystemInterface.h"
#include "PersistentMemoryObject.h"
#include "PersistentMemoryObjectStar.h"
#include "String.h"
#include "Process.h"
#include "ThisProcess.h"
#include "Domain.h"
#include "AssocTable.h"
#include "MachineConfiguration.h"
#include "Types.h"


#ifdef __GNUC__
#pragma implementation
#endif


MachineConfiguration::MachineConfiguration(FileSystemInterface *fsi,
					   unsigned short debugMask,
					   unsigned int tableSize)
{
    Raids(RaidMethod) << "(2)MachineConfiguration constructor(fsi="
		      << asHex((int) fsi)
		      << ", tableSize=" << tableSize << ")\n" << eor;

    _table = new AssocTable(tableSize);

#ifndef VCHOICES
    (void) loadFromFile("/Configuration/Local", fsi);
    _identification = "/Configuration/Local";
#else
    (void) loadFromFile("/Configuration", fsi);
    _identification = "/Configuration";
#endif
}


MachineConfiguration::~MachineConfiguration()
{
    delete _table;
}


int
MachineConfiguration::bind(char *key, char *value)
{
    Raids(RaidMethod) << "MachineConfiguration::bind(key=\"" << key
		      << "\", value=\"" << value << "\")\n" << eor;

    char *newKey, *newValue;

    newKey = new char [strlen(key) + 1];
    (void) strcpy(newKey, key);

    newValue = new char [strlen(value) + 1];
    (void) strcpy(newValue, value);

    return _table->bind(newKey, (int) newValue);	// cast to int -- yecch
}


int
MachineConfiguration::unbind(char *key, int & status)
{
  char *string = key;
	// The only reason for overriding the parent method in this case
	// is to deallocate the memory used for the strings.  However, I'm
	// not sure there's any easy way to locate those memory areas
	// before doing a parent unbind, unless this subclass keeps a
	// second list of the allocated strings, which is something I'm
	// really reluctant to do when you consider how unlikely it is
	// for kernel resources to be unbound.  Sigh ... so for the moment,
	// we'll ignore this problem and do nothing, in the hope that our
	// memory leaks will be few.  Talk about laziness.	// Amit

	//  Get the bound string as returned by AssocTable.
    Assert(NOTREACHED); // fix this string = (char *) _table->unbind(k, status);

    Raids(RaidMethod) << "MachineConfiguration::unbind(key=\"" << key
		      << "\"), found at " << asHex((int) string)
		      << '\n' << eor;

    if (string != NULL) {
	delete string;
    }

    return 0;
}

int
MachineConfiguration::lookup(char *key, char *&value)
{			// NB: "value" is a ref to ptr to char, not ptr to ref
    int status;
    unsigned int temp = 0;

    status = _table->lookup(key, temp);
    value = (char *) temp;

    Raids(RaidMethod) << "MachineConfiguration::lookup returns key=\"" << key
		      << "\", value = \"" << value << "\"\n" << eor;

    if (status == 1) {		// used to indicate success by lookup()
	return 0;
    } else {
	return 1;
    }
}



	//  This is the cute one.  Given a file name string, open the file,
	//  read a bunch of specifications from it, and load those into
	//  the kernel configuration.  No problem, right?  Yaaah right.
	//						// Amit
	//
	//  Note that the format of a config file is thus:
	//  config-file ::= config-line
	//			|  config-line config-file
	//  config-line ::= blank-line
	//			|  config-definition
	//  config-definition ::=
	//    optional-whitespace config-keystring whitespace config-value <\n>
	//  whitespace ::=  <space> | <tab> | <^L>
	//  optional-whitespace ::= whitespace | nothing
	//  config-keystring ::= alphanumeric-character
	//			|  alphanumeric-character  config-keystring
	//  config-value ::= alphanumeric-character rest-of-value <\n>
	//  rest-of-value ::= config-value-character
	//			|  config-value-character  rest-of-value
	//			|  nothing
	//  config-value-character ::= <alphanumeric-character>
	//			|  whitespace
	//			|  punctutation-character
	//
	//	Got it?

int
MachineConfiguration::loadFromFile(char *fileName, FileSystemInterface *fsi)
{
    int status;
    char key[MACHINECONFIGURATION_MAXKEYLENGTH + 1],
	 value[MACHINECONFIGURATION_MAXVALUELENGTH + 1];

    Raids(RaidMethod) << "MachineConfiguration::loadFromFile(file=\""
		      << fileName << "\", fsi=" << asHex((int) fsi)
		      << ")\n" << eor;

    // need to open the file ..
    PersistentMemoryObjectStar mo =
	    fsi->open(fileName, PersistentMemoryObjectClass, status);
    
    if (status == Success) {

	Raids(RaidMethod) << "MachineConfiguration::loadFromFile: reading "
			  << fileName << '\n' << eor;

	base = thisProcess()->domain()->add( mo, NoAccess );
	pastEnd = ( char * ) base + mo->numberOfBytes();
	current = ( char * ) base;

	while ((status = readConfiguration(key,value)) == Success){
	    Raids(RaidMethod) << "MachineConfiguration::loadFromFile "
			      << "read (KEY=" << key << ", VALUE=" << value
			      << "), status =" << status << '\n' << eor;
	    Console() << "MachineConfiguration::loadFromFile "
			      << "read (KEY=" << key << ", VALUE=" << value
			      << "), status =" << status << '\n' << eor;
	    (void) bind(key, value);
	}

	status = 0;

	thisProcess()->domain()->remove( mo, base );

    } else {
	Console() << "MachineConfiguration: *** ERROR " << status <<
		" *** when attempting to open " << fileName << '\n' << eor;
    }

    return status;
}

int
MachineConfiguration::read( char * b )
{
    if ( current == pastEnd ) {
        *b = 0;
        return 0;
    } else {
	*b = *current++;
	return 1;
    }
}

int
MachineConfiguration::readConfiguration(char *key,
					char *value)
{
    char input,				// character read from stream
	 *keyPointer,			// point to current character in key
	 *valuePointer;			// point to current character in value
    int keyCounter,			// length of key field read so far
	valueCounter;			// length of value field read so far
    int bytesRead;

	//  Nota bene: this here is a DFA-style input routine ...	// Amit

    NewLine:				// wait for non-blank input
	do {
	    bytesRead = read( &input );

	    Raids(RaidDetail) << "MachineConfiguration::readConfiguration: "
			      << "  read character '" << input << "'\n" << eor;

	} while ((bytesRead == 1) &&		// skip leading whitespace
		 (input == ' ' ||
		  input == '\t' ||
		  input == '\n'));

	if (bytesRead != 1) {			// failure/EOF in read()
	    goto Error;
	} else if (input == '#') {		// ran into comment string
	    goto InComment;
	} else {				// must be a {key,value} pair
	    goto InKey;
	}

    InComment:			    // skip past a comment (up to end of line)
	Raids(RaidDetail)
	<< "MachineConfiguration::readConfiguration:  skipping comment line\n"
	<< eor;

	do {					// swallow everything up to \n
	    bytesRead = read(&input);

	    Raids(RaidDetail) << "MachineConfiguration::readConfiguration: "
			      << "  read character '" << input << "'\n" << eor;
	} while ((bytesRead == 1) && (input != '\n'));

	if (bytesRead != 1) {			// EOF or read() failure
	    goto Error;
	} else {
	    goto NewLine;
	}

    InKey:				// currently reading the "key" field
	keyPointer = key;
	keyCounter = 0;
	while ((bytesRead == 1) &&	// read everything up to first white
	       ((input != ' ') && (input != '\t')) &&
	       (keyCounter < MACHINECONFIGURATION_MAXKEYLENGTH)) {

	    *keyPointer++ = input;	// start with character we just read...
	    keyCounter++;
	    bytesRead = read(&input);	// and read the next

	    Raids(RaidDetail) << "MachineConfiguration::readConfiguration: "
			      << "  read character '" << input << "'\n" << eor;
	}
	*keyPointer = '\0';			// null terminate input string

	if (bytesRead != 1) {			// EOF or read() failure
	    goto Error;
	} else {

	    do {				// skip intermediate whitespace
		bytesRead = read(&input);

		Raids(RaidDetail)
			    << "MachineConfiguration::readConfiguration: "
			    << "  read character '" << input << "'\n" << eor;

	    } while ((bytesRead == 1) &&
		      (input == ' ' || input == '\t'));

	    if (bytesRead == 1) {
		goto InValue;
	    } else {
		goto Error;			// EOF or error in whitespace
	    }
	}


    InValue:					// read "value" from line
	valuePointer = value;
	valueCounter = 0;

	while ((bytesRead == 1) &&		// read everything up to \n
	       (input != '\n') &&
	       (valueCounter < MACHINECONFIGURATION_MAXVALUELENGTH)) {
	    *valuePointer++ = input;
	    valueCounter++;
	    bytesRead = read(&input);

	    Raids(RaidDetail) << "InValue: read character '" << input
			      << "', bytesRead = " << bytesRead << '\n' << eor;
	}
	*valuePointer = '\0';

	if (bytesRead == 1) {			// Successfully read the \n ...
	    goto Done;				// so we're finished this line
	} else {
	    goto Error;				// EOF/error in mid-line
	}

    Done:					// normal exit
	Raids(RaidMethod)
		<< "MachineConfiguration::readConfiguration:  normal exit\n"
		<< eor;
	return Success;

    Error:					// abnormal exit
	Raids(RaidMethod) << "MachineConfiguration::readConfiguration: "
			  << " ERROR abort, last character was '" << input
			  << "', bytesRead=" << bytesRead << '\n' << eor;
	return ReadError;
}
