#ifdef __GNUC__
#pragma implementation
#endif

// #define DEBUG
/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * This is the initial setup code.
 * Main() is called by ALL processors while still in physical memory
 * mode. One processor (the first one to call it Main()) sets up the
 * Kernel via its constructor.
 * All processors join the kernel by invoking the perProcessorEntry() method
 * on it.
 */

#include "Assert.h"
#include "Class.h"
#include "ClassInstaller.h"
#include "MACHINEKERNEL.h"
#include "MACHINE_CONFIGURATION.h"
#include "MAIN.h"
#include "NameServer.h"
#include "OutputStream.h"
#include "StupidAllocator.h"
#include "PersistentObject.h"
#ifdef VCHOICES
#include "VCStupidAllocator.h"
#endif

#ifdef __GNUC__
#pragma implementation
#endif

#ifdef VCHOICES
static char bootHeapStateLocation[ sizeof( VCStupidAllocator ) ];
#else
static char bootHeapStateLocation[ sizeof( StupidAllocator ) ];
#endif


/*
 * THE Kernel.
 */
Kernel * TheKernel = 0;

/*
 * The very first statically constructed allocator that allocates all other
 * object statically.
 */



Allocator * createBootAllocator( )
{
    /*
     * Build a boot heap to use until the Kernel Heap is set
     * up and the VM is enabled. Use a new with a "placement"
     * since there are no allocators to use yet.
     */

    Allocator * bootHeap = new ( bootHeapStateLocation )
#ifdef VCHOICES
    VCStupidAllocator();
#else
    StupidAllocator( HEAP_START() );
#endif //VCHOICES

    return bootHeap;
}



extern 
Allocator *
SetNewAllocator(Allocator *);

void
Main( int argc, char ** argv )
{
    /*
     * This should only be called by one processor (the first to boot in a
     * multiprocessor).
     */
    Allocator *bootHeap = createBootAllocator( ) ;

    /*
     * Setup operator new to use the heap just constructed
     * for the duration of the following few lines of code.
     * This will allow the constructors of statically initialized
     * objects to call operator new.
     *
     * By the way... It makes me nervous too...
     */
    Allocator * oldHeap = SetNewAllocator( bootHeap );
    /*
     * This check assures that nobody else has already done the
     * previous initialization.
     */
    Assert( oldHeap == 0 );

    /*
     * Only the first processor should get here and do the initialization.
     */
    Assert( TheKernel == 0 );

    /*
     * Initialize the ProxiableObject tables.
     * The console hasn't been built yet, so no debug here.
     */
    ProxiableObject::initialize();

    /* Initialize the PersistentObjectLock and 
     * the PersistentObjectTable
     */
    PersistentObject::initialize();
    /*
     * The console will be built here, so debugging should work
     * when we get back.
     */
    MAIN( argc, argv );

    /*
     * Call constructors for all statically allocated objects.
     */
    extern void initializeStaticallyConstructedObjects();
    initializeStaticallyConstructedObjects();

    /*
     * Build the kernel object for this particular architecture.
     * It could just be a statically allocated object, but I
     * build it separately just in case it wants to use
     * any other objects (static).
     */
    Debug << "Build the kernel\n" << eor;
    TheKernel = new MACHINEKERNEL;

    /*
     * Initialize all the Class objects.
     */
    Debug << "call Class::install()\n" << eor;
    ClassInstaller::install();

    /*
     * Initialize the kernel. This will setup the initial VM mappings.
     */
    Debug << "Initialize the kernel\n" << eor;
    TheKernel->initialize( bootHeap);
    Debug << "The kernel is initialized\n" << eor;

    /*
     * Kernel::initialize() should have reset the allocator to
     * use as operator new's heap. It cant, however, be used
     * until VM is enabled. Therefore,
     * dont do any "new"'s until the CPU has beed initialized!
     * In the future I'd like to add code to check this.
     *
     * Also, Kernel::initialize() should have made thisCPU() useable
     * by this and any other processor.
     */
    extern Allocator * GetNewAllocator();
    Assert( GetNewAllocator() != bootHeap );
    Assert( TheKernel != 0 );

    Debug << "Main done\n" << eor;
}

/*
 * Do what the name says. This is compiler dependent.
 */

extern "C" {
    extern PFV __CTOR_LIST__[]; 
}
extern "C" void _init ();

void
initializeStaticallyConstructedObjects()
{
#ifdef VCHOICES
    _init();
#else
#ifdef CFRONT
    for( PFV * pf = _ctors; *pf != 0; pf++ ) {
        (**pf)();
    }
#else
    /*
     * The new "collect" generates CTOR- and DTOR-LISTs with a
     * leading count (they're still null-terminated).
     */
    for( PFV * pf = &__CTOR_LIST__[1]; *pf != 0; pf++ ) {
        (**pf)();
    }

#endif //CFRONT
#endif //VCHOICES
}


#ifndef VCHOICES
/*
 * Used by abstract (pure) virtual functions.
 */
extern "C" void
#ifdef CFRONT
__pure_virtual_called ()
#else
    abort ()
#endif
{
    Assert (NOTREACHED);
    exit(1);
}

#endif
