#ifdef __GNUC__
#pragma implementation
#endif

#include "ProcessContainer.h"
#include "OutputStream.h"
#include "Kernel.h"
#include "Semaphore.h"
#include "SemaphoreStar.h"
#include "Store.h"
#include "String.h"
#include "Convert.h"
#include "ThisProcess.h"
#include "FreeRunningTimer.h"
#include "FreeRunningTimerStar.h"
#include "PROCESSOR.h"


#ifdef __GNUC__
#pragma implementation
#endif

#define NTIMES 10000

Process * first;
Process * second;
Process * top;

void
one( int )
{
	Console() << "one started\n" << eor;
	PROCESSOR::disableInterrupts();

	first->giveProcessorTo( second, ProcessReady );

	unsigned int start = SystemTimer->time();
	int i;
	for( i = 0; i < NTIMES; i++ ) {
	}
	unsigned int end = SystemTimer->time();

	int over = end - start;
	Console() << "Loop Overhead = " << over << "\n" << eor;

	start = SystemTimer->time();
	for( i = 0; i < NTIMES; i++ ) {
		// first->ready();
		first->giveProcessorTo( second, ProcessReady );
	}
	end = SystemTimer->time();

	int t = ((end - start) - over) / (2*NTIMES);

	Console() << "Switch took " << t << "\n" << eor;

	Console() << "one back, give to top\n" << eor;

	first->die();
	Assert( NOTREACHED );
}

void
two( int )
{
	Console() << "two started\n" << eor;
	PROCESSOR::disableInterrupts();

	for( int i = 0; i < NTIMES; i++ ) {
		second->giveProcessorTo( first, ProcessReady );
	}

	Console() << "two done, give to one\n" << eor;
	second->ready();
	second->giveProcessorTo(first, ProcessReady);

	second->die();
	Assert( NOTREACHED );
}

void
switchtimes()
{
	first = new SystemProcess(
		(PFV) one, TheKernel->domain(), normal, 0 );
	first->setName( "first" );
	second = new SystemProcess(
		(PFV) two, TheKernel->domain(), normal, 0 );
	second->setName( "second" );

	top = thisProcess();

	Console() << "\ntop: Timing " << NTIMES << " iterations\n" << eor;
	Console() << "top: give to first\n" << eor;

	IntStorage intState ; PROCESSOR::disableInterrupts(intState);
	top->ready();
	top->giveProcessorTo( first, ProcessReady );

	Console() << "top: back\n";
	PROCESSOR::restoreInterrupts(intState);
}

Store *
aFunction()
{
	return( MainStore );
}

void
times()
{
	char ** testVector = new char * [128];
	Lock * lock = new Lock();
	ReferenceCount * pc = new ReferenceCount();
	Process * process = thisProcess();
	SemaphoreStar sem = new BinarySemaphore( 1 );
	int i;

	Console() << "Timing " << NTIMES << " iterations\n\n" << eor;

	int t0 = SystemTimer->time();
	for( i = 0; i < NTIMES; i++ ) ;
	int t1 = SystemTimer->time();

	int t2 = SystemTimer->time();
	for( i = 0; i < NTIMES; i++ ) aFunction();
	int t3 = SystemTimer->time();

	int t4 = SystemTimer->time();
	for( i = 0; i < NTIMES; i++ ) process->classOf();
	int t5 = SystemTimer->time();

	int t6 = SystemTimer->time();
	for( i = 0; i < NTIMES; i++ ) {
		pc->increment();
		pc->decrement();
	}
	int t7 = SystemTimer->time();

	int t8 = SystemTimer->time();
	for( i = 0; i < NTIMES; i++ ) {
		lock->acquire();
		lock->release();
	}
	int t9 = SystemTimer->time();

	int t10 = SystemTimer->time();
	for( i = 0; i < NTIMES; i++ ) {
		sem->P();
		sem->V();
	}
	int t11 = SystemTimer->time();

	int loop0 = t1 - t0;
	Console() << "Empty loop overhead   " << loop0/NTIMES
		<< "\n" << eor;
	Console() << "Function call   " << (t3-t2-loop0)/NTIMES
		<< "\n" << eor;
	Console() << "Virtual function call   " << (t5-t4-loop0)/NTIMES
		<< "\n" << eor;
	Console() << "Reference count (increment+decrement)   "
		<< (t7-t6-loop0)/NTIMES << "\n" << eor;
	Console() << "Lock (acquire+release)   "
		<< (t9-t8-loop0)/NTIMES << "\n" << eor;
	Console() << "Binary semaphore (P+V)   "
		<< (t11-t10-loop0)/NTIMES << "\n" << eor;
	Console() << "\n\n" << eor;

	delete lock;
	delete pc;
	delete testVector;
}


void
testMadAllocator(int times, int max)
{
	char *buf;
	int size = 2311;
	while (--times > 0) {
		Console() << "Allocate " << size << " bytes\n" << eor;
		buf = new char[size];
		size = SystemTimer->time() % max;
		}
}

void
_overflowKernelStack(unsigned count)
{
	Console() << " " << count ;
	_overflowKernelStack(count+1);
}

void
__overflowKernelStack(unsigned count)
{
	PROCESSOR::enableInterrupts();
	Console() << "overflowing kernel stack, should crash gracefully\n";
	Console() << "Printing depth of calls\n";
	_overflowKernelStack(count);
}

void
overflowKernelStack(unsigned count)
{
	Process *p = new SystemProcess(
		(PFV) __overflowKernelStack, TheKernel->domain(), normal, count );
	p->ready();
	PROCESSOR::disableInterrupts();
	thisProcess()->giveProcessorTo(p, ProcessReady);
}

/*
 * Just do a bunch of pseudo-randomly recursive calls  with
 * allocations off the stack for a period of time.
 *
 * lotsaCalls (seconds, maxdepth, allocsize);
 */


static int allocsize;
static int depth;
static int seconds;

void theCalls(unsigned end, int depth, int alloc, char *p)
{
	if (SystemTimer->time() > end)		// Time expired??
		return;
	if (depth-- == 0)			// Bottomed out??
		return;
	char *heapBytes = 0;
	if (alloc > 0 ) {
		heapBytes = new char [alloc];
		if (p != 0)
			for (int i = 0; i < alloc; i+=4096)
				heapBytes[i] = p[i];    // Touch every page
		p = heapBytes;
		}
	theCalls(end, depth, alloc, p);
	if (heapBytes) delete heapBytes;
}

void
doLotsaCalls()
{
	Console() << "Calling for " << seconds << " seconds, depth: "
		<< depth << " allocsize: " << allocsize << "\n" << eor;

	unsigned time = SystemTimer->time();
	unsigned end = time + (seconds << 20);
	unsigned newtime;
	while ((newtime = SystemTimer->time()) < end){
		if ((newtime - time) > (1 << 20)){
			time = newtime;
			Console() << thisProcess() << ": still calling\n" << eor;
			}
		theCalls(end, depth, allocsize, 0);
		}
	Console() << "Calling done\n" << eor;

	thisProcess()->die();
}

void
lotsaCalls(int argc, char *argv[])
{
	/*
	 * Defaults
	 */
	allocsize = 0;
	depth = 200;
	seconds = 1;
	int howmany = 1;

	switch (argc) {
		case 4:
			howmany = atoi(argv[3]);
			// FALLTHROUGH!
		case 3:
			allocsize = atoi(argv[2]);
			// FALLTHROUGH!
		case 2:
			depth = atoi(argv[1]);
			// FALLTHROUGH!
		case 1:
			seconds = atoi(argv[0]);
			// FALLTHROUGH!
		case 0:
			break;
		default:
			Console() << "Usage: lotsaCalls (seconds, maxdepth, allocsize)\n";
			return;
			break;
	};

	for (int i = 0; i < howmany; i++) {
		Process *p = new PreemptableSystemProcess
			( (PFV) doLotsaCalls, TheKernel->domain(), normal, 0);
		p->ready();
		}
}

int doNothing() { return 1;}
