#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * NonReentrant.cc
 */

#include "Lock.h"
#include "NonReentrant.h"
#include "Process.h"
#include "PROCESSOR.h"
#include "ThisCPU.h"
#include "ThisProcess.h"

static Lock lock;

NonReentrant::NonReentrant(char * name)
{
	_name = name;
	_nesting = 0;
	for (int i = 0; i < _maxProcess; i++) {
		_process[i] = 0;
		_residual[i] = 0;
	}
}

int NonReentrant::enter()
{
	lock.acquire();
	TIMESLICETIMER * timesliceTimer = thisCPU()->timesliceTimer();
	unsigned int resid = timesliceTimer->stop();
	int my_nesting;
	my_nesting = _nesting;
        _process[my_nesting] = thisProcess();
	_residual[my_nesting] = resid;
        _nesting++;
        if (my_nesting != 0) {
		Console()
			<< _name << " enter " << my_nesting 
			<< "\n" << eor;
        }
	lock.release();
	return my_nesting;
}

void NonReentrant::exit(int my_nesting)
{
	lock.acquire();
	if (my_nesting != 0) {
		Console()
			<< _name << " exit " << my_nesting 
			<< ", _nesting = " << _nesting
			<< "\n" << eor;
	}
	unsigned int resid = _residual[my_nesting];
	_process[my_nesting] = 0;
	_residual[my_nesting] = 0;	
	_nesting--;
	lock.release();
	if (resid != 0) {
		TIMESLICETIMER * timesliceTimer = thisCPU()->timesliceTimer();
		timesliceTimer->start(resid);
	}
}
