#ifdef __GNUC__
#pragma implementation
#endif

// #define DEBUG
/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ObjectProxyKernel.cc: assists for Proxy calls which do not need to be in
 *   User-Readable.  The remainder of the ObjectProxy methods are in
 *   ObjectProxyVtable.cc and ObjectProxyStubs.cc where they can be made
 *   to be User-Readable.
 */

#include "Class.h"
#include "ConstructorDescriptor.h"
#include "ConstructorDescriptorStar.h"
#include "Domain.h"
#include "Kernel.h"
#include "NameServer.h"
#include "ObjectProxy.h"
#include "ObjectProxyManager.h"
#include "ProxyTableCopy.h"
#include "ThisProcess.h"
#include "Process.h"


#ifdef __GNUC__
#pragma implementation
#endif

class vtable_entry	/* the same for cfront and g++ */
{
public:
	short a,b;
	unsigned int f;
};

enum ProxyCallType { InCall, OutCall };

/*
 * This is called from the from ProxyCallAssist.
 */
PFV
ProxyConstructAssist( char *name, Domain * callingDomain,
		      MethodTableEntryCopy **methPtr, 
		      Domain ** calledDomainPtr, int & size )
{
#ifdef DEBUG
	Console() << "ProxyConstructAssist ( "
			<< asHex( (int) name) << ", "
			<< callingDomain << " )\n" << eor;
#endif

	Assert( name != 0 ) ( "ProxyConstructAssist: null name" );
// NAYEEM fix later
/*
	Assert( TheKernel->lookupRing( name ) >= callingDomain->ring() )
		( "ProxyConstructAssist: pointer to constructor name (%A) out of range\n", name );
*/


	ConstructorDescriptorStar c =
		callingDomain->nameServer()->lookup( name,
					ConstructorDescriptorClass );

	Assert( c != 0 )( "ProxyConstructAssist: bad constructor %s", name);
	*methPtr = c->meth;
	*calledDomainPtr = c->domain;
	size = c->size;

	return( (PFV) c->ctor );
}

/* 
 * this piece pulled out of AllocateProxy to make the compiler
 *   to really make AllocateProxy an inline
 */
void
AllocateProxyParameter( ObjectProxy * proxy, ProxyReturnFlags * returnFlags )
{
	/*
	 * This is for a parameter, not a return value, and the
	 *    proxy has a zero reference count.
	 * Save a pointer to the allocated proxy to check upon return.
	 */
	Assert( returnFlags->numParamProxies < MAXPARAMPROXIES )
	    ( "More than %d zero-count proxies coming in to a method\n", MAXPARAMPROXIES );

	returnFlags->paramProxies[ returnFlags->numParamProxies++ ] =
						proxy;
}

/*
 * Allocate a proxy and keep track of it if it is for a parameter coming in.
 */

inline ProxiableObject *
AllocateProxy( ProxiableObject * object, Class * proxiedClass,
	Domain * proxyDomain, Domain * proxiedDomain, int ref,
					ProxyReturnFlags * returnFlags )
{
	ObjectProxy * proxy = proxyDomain->proxyManager()->allocate(
		object, proxiedClass, proxiedDomain, ref );
	
	if ( proxy->setNotReferenced() && ( returnFlags->returnType == 0 ) )
		AllocateProxyParameter( proxy, returnFlags );

	return( (ProxiableObject *) proxy );
}

/*
 * Since the _referenceCount is protected in ProxiableObject, use this to
 *   get around the compiler check.  Don't want to use the virtual function
 *   reference() in this case because the object is in user space and the
 *   vtable can't be trusted.  The alternative to this would be to make
 *   StripProxy a fried of ProxiableObject and that's even uglier than
 *   this in my opinion.
 */
class SubProxiableObject : public ProxiableObject {
public:
	void catchReference();
};

inline void
SubProxiableObject::catchReference()
{
	_referenceCount.increment();
}

/*
 * A common function used by ProxyInCallParamAssist and ProxyOutCallParamAssist.
 * Strip a proxy off and allocate a new one if necessary.
 */
ProxiableObject *
StripProxy( ProxiableObject * object, Class * proxiedClass,
	Domain * callingDomain, Domain * calledDomain,
				ProxyReturnFlags * returnFlags )
{
	ProxiableObject * returnObject;
	if ( ( (ObjectProxy *) object )->proxiedDomain() == calledDomain ) {
		/*
		 * Proxy is to the called Domain.  Strip off the proxy.
		 */
		returnObject = ( (ObjectProxy *) object )->proxiedObject();

		if ( returnFlags->returnType == 2 ) {
			/*
			 * The caller has erroneously incremented the reference
			 *   count on the proxy rather than on the object
			 *   itself.  Decrement the reference count on the old
			 *   proxy and increment it on the real object.
			 */
			(void) ( (ObjectProxy *) object )->catchUnreference();

			( (SubProxiableObject *) returnObject )->
							catchReference();
		}
	}
	else {
	Assert(NOTREACHED) ("cannot proxy to other domains");
		/* 
		 * Proxy is to a different Domain.  Allocate a new one
		 *   for the called Domain.
		 */
	
		returnObject = AllocateProxy(
			( (ObjectProxy *) object )->proxiedObject(),
			    proxiedClass, calledDomain,
				( (ObjectProxy *) object )->proxiedDomain(), 0,
				    returnFlags );

		if ( returnFlags->returnType == 2 ) {
			/*
			 * The caller has erroneously incremented the reference
			 *   count on the proxy rather than on the object
			 *   itself.  Decrement the reference count on the old
			 *   proxy and increment it on the new one.
			 */
			(void) ( (ObjectProxy *) object )->catchUnreference();

			(void) ( (ObjectProxy *) returnObject )->
							catchReference();
		}

	}

	if ( returnFlags->returnType == 1 ) {
		/*
		 * If this is a return of an asterisk and the proxy has
		 *  never been referenced, free it because the returner
		 *  would only want to have used the proxy temporarily.
		 * No need to check for notReferenced if the return type
		 *  is a Ref because it will always have been referenced.
		 */
		( (ObjectProxy *) object )->freeIfNotReferenced(
					callingDomain->proxyManager() );
	}

	return( returnObject );
}

/*
 * This is called on incalls for each parameter that is a pointer
 *    and on the return from outcalls that return a pointer.
 */

inline int
ProxyRangeCheck(Domain *d, char *addr, int strict)
{
#ifdef VCHOICES
	return ( 1 );
#endif

	if( addr == 0 )
		return( 1 );
	if( d == TheKernel->domain()) {
		if(strict) return 1;
		if (TheKernel->domain()->validAddress(addr))
			return( 1 );
	} else  {
		if (d->validAddress(addr))
			return( 1 );
	}
	return 0;
}

inline ProxiableObject *
ProxyInCallParamAssist( ProxiableObject *object, Class *expectedClass,
    			Domain * callingDomain, Domain * calledDomain,
					ProxyReturnFlags * returnFlags )
{
#ifdef DEBUG
	Console() << "ProxyInCallParamAssist ( "
			<< asHex( (int) object ) << ", "
			<< asHex( (int) expectedClass ) << ", "
			<< callingDomain << ", "
			<< calledDomain << ", "
			<< returnFlags << " )\n" << eor;
#endif

	if ((unsigned int) expectedClass == 1)
	{
		/*
		 * A pointer to a non-proxiable object.  Range check it
		 *   to make sure the caller is not trying to get the
		 *   elevated-privilege code to access privileged space.
		 */
		if ( ProxyRangeCheck(callingDomain,(char*)object,1) )
			return( object );
		else
			return( 0 );
	}

	if ( callingDomain->proxyManager()->isValidProxy(
						(ObjectProxy *) object ) )
	{

		object = (ProxiableObject *) 
			 callingDomain->proxyManager()->domainToKernel 
				( (ObjectProxy *) object );

		/*
		 * A valid Proxy coming in.
		 * Get the class from the Proxy's table rather than from
		 *   object->proxiedObject()->classOf() because that might
		 *   require an outcall from the kernel.
		 */
// NAYEEM -- fix later
/*
		if ( ( ( (ObjectProxy *) object )->proxiedDomain()->ring() <=
					calledDomain->ring() ) &&
*/
		if( !( (ObjectProxy *) object )->table()->tableClass->
		       isASubclassOf ( expectedClass ) )
			return( 0 );
		
		return( StripProxy( object, expectedClass, callingDomain,
				calledDomain, returnFlags ) );
	}

	/*
	 * Not a valid proxy coming in.
	 */
	ProxyTableCopy * table;
	if ( ( ( table = expectedClass->proxyTable() ) == 0 ) ||
						!table->isProxiable ) {
		/*
		 * The class is not a proxiable class
		 */
		return( 0 );
	}
	if ( ProxyRangeCheck(callingDomain, (char*)object, 0) ){
		/*
		 * The object is not in the caller's address space
		 */
		return( 0 );
	}
	/*
	 * Allocate a proxy for the called domain into the caller's
	 *   space.
	 */
	return( AllocateProxy( object, expectedClass, calledDomain,
		callingDomain, returnFlags->returnType == 2, returnFlags ) );
}

/*
 * This is called on outcalls for each parameter that is a pointer to a
 *    proxiable object and on the return from incalls if the return
 *    is a pointer to a proxiable object.
 */

inline ProxiableObject *
ProxyOutCallParamAssist( ProxiableObject *object, Class * expectedClass,
    			 Domain * callingDomain, Domain * calledDomain,
			 ProxyReturnFlags * returnFlags )
{
#ifdef DEBUG
	Console() << "ProxyOutCallParamAssist ( "
			<< asHex( (int) object ) << ", "
			<< asHex( (int) expectedClass ) << ", "
			<< callingDomain << ", "
			<< calledDomain << ", "
			<< returnFlags << " )\n" << eor;
#endif

	Class * proxiedClass;
	if ((unsigned int) expectedClass == 1) {
		/*
		 * We don't care where a non-proxiable object points, although
		 *   the less-privileged code will not be able to access the
		 *   memory if the pointer points to  more-privileged space.
		 */
		return( object );
	}

	if ( callingDomain == TheKernel->domain() ) {
		/*
		 * This is the most common case so make it as fast as possible.
		 * A call to the 'isProxy()' virtual function can be trusted 
		 *   because the object (or Proxy) is in the kernel.
		 */
		Assert (object != 0);
		if ( !object->isProxy() ) {
			/*
			 * Allocate a proxy to kernel domin for the called Domain
			 */
			return( AllocateProxy( object, object->classOf(),
				calledDomain, callingDomain,
				returnFlags->returnType == 2, returnFlags ) );
		}

		/*
		 * Make an outcall to the object to find out it's actual
		 *   class since the class in the proxy is only the class that
		 *   was expected when it was passed in to the kernel.
		 */
		proxiedClass = object->classOf();
	}
	else if ( callingDomain->proxyManager()->isValidProxy(
						(ObjectProxy *) object ) ) {
		ProxiableObject * proxiedObject =
			      ( (ObjectProxy *) object )->proxiedObject();
		Domain * proxiedDomain =
			      ( (ObjectProxy *) object )->proxiedDomain();
		Domain * kernelDomain = TheKernel->domain();

		if ( proxiedDomain == kernelDomain )
			proxiedClass = proxiedObject->classOf();
		else
		{
			/*
			 *  Use temporary proxy to ask the proxied object what
			 *    its class is.  To save time and complication
			 *    don't touch the reference counts.
			 */
			ObjectProxy * tmpProxy = returnFlags->thisProc->
					temporaryProxy();
			
			tmpProxy->setProxiedObject(  proxiedObject,
				ProxiableObjectClass, proxiedDomain, 2 );

			proxiedClass = ( (ProxiableObject *) tmpProxy )->
						classOf();

			tmpProxy->setProxiedObject( 0, 0, 0, 0 );
		}
	}
	else {
		/*
		 * Not calling from Kernel and object is not a valid proxy.
		 */
	if ( ProxyRangeCheck(callingDomain,(char *)object, 0) ) {
			/*
			 * The object is not in the caller's address space
			 */
			return( 0 );
		}

		/*
		 *  Use temporary proxy to ask the proxied object what
		 *    its class is.  To save time and complication
		 *    don't touch the reference counts.
		 */
		ObjectProxy * tmpProxy = returnFlags->thisProc->
						temporaryProxy();
		
		tmpProxy->setProxiedObject( object, ProxiableObjectClass,
					callingDomain, 2 );

		proxiedClass = ( (ProxiableObject *) tmpProxy )->classOf();

		tmpProxy->setProxiedObject( 0, 0, 0, 0 );

		/*
		 * Allocate a proxy for the called domain into the caller's
		 *   space.
		 */

		return( AllocateProxy( object, proxiedClass, calledDomain,
		   callingDomain, returnFlags->returnType == 2, returnFlags ) );
	}

	/*
	 * A proxy going out.  
	 */
	return( StripProxy( object, proxiedClass, callingDomain,
					calledDomain, returnFlags ) );
}

/*
 * This is called directly from the Proxy trap.
 *   Convert the user's proxy and copy the arguments, translating each one.
 * Args:
 *   methodNumber - number of method that caused trap
 *   fromKernel - non-zero if proxy call came from the kernel, zero otherwise
 *   returnFlags - pointer to scratch memory large enough to hold a structure
 *		of type ProxyReturnFlags.  This will be used to pass 
 *		information to ProxyReturnAssist.
 *   refPointer - pointer to where return value will be written into if
 *		method returns Ref or Star.  If 0, will assume that Ref or
 *		Star is returned in a register.
 *   from - pointer to user stack; the first word has the return address
 *		to the proxy stub, followed by the parameters
 *   to - pointer to scratch space on kernel stack to copy method arguments
 *  		into if call is to kernel or state if call is not to kernel
 * Return value:
 *   0 - all work already done
 *   1 - call to kernel mode and don't need to call ProxyReturnAssist
 *   2 - call to kernel mode and do need to call ProxyReturnAssist
 *   3 - call to user mode, memory at 'to' filled with state needed to
 *	    return to kernel mode, and user stack has copied args.
 */


extern "C"
int
ProxyCallAssist( unsigned int methodNumber, int fromKernel, 
	ProxyReturnFlags *returnFlags, void *refPointer,
		ProxiableObject **from, ProxiableObject **to )
{

	Assert (! fromKernel);
#ifdef REFBEFORETHIS
	int skippedRef = 0;
#endif

#ifdef DEBUG
	Console() << "ProxyCallAssist ( " << (int) methodNumber << ", "
		  << asHex( fromKernel ) << ", "
		  << asHex( (int) returnFlags ) << ", "
		  << asHex( (int) refPointer ) << ", "
		  << asHex( (int) from ) << ", "
		  << asHex( (int) to ) << " )\n" << eor;
#endif

	Process *thisProc = thisProcess();

	Domain *callingDomain;
	if ( fromKernel )
		callingDomain = TheKernel->domain();
	else {
		callingDomain = thisProc->currentDomain();
	}

	ObjectProxyManager *callingDomainProxyManager =
		callingDomain->proxyManager();
	from++;		/* skip over return address */

	PFV method = 0;
	MethodTableEntryCopy *meth = 0;
	Domain *calledDomain = 0;
	ObjectProxy *proxy = 0;
	ProxiableObject *object = 0;
	if ( (int) methodNumber == -1 )
	{
		/*
		 * A constructor.  First argument is actually pointer to
		 *   string name of constructor, not a proxy.
		 */
		int size;
		method = ProxyConstructAssist( (char *) *from, callingDomain,
						&meth, &calledDomain, size );
		object = 0; /* constructors should start with 'this' of null */
#ifdef __GNUC__ 
		/* without -fthis-is-variable must call new first */
		object = (ProxiableObject *) new char[size];
#endif
		from += 2; /* skip string name and return address */
	}
	else
	{
		/*
		 * A regular method.
		 * Validate that the object belongs to the current manager
		 */

		proxy = (ObjectProxy *) *from;

#ifdef REFBEFORETHIS
		if ( !callingDomainProxyManager->isValidProxy( proxy ) )
		{
			/*
			 * first parameter is probably the pointer to
			 *  the ref, but can't tell yet.
			 */
			proxy = (ObjectProxy *) *(++from);
			skippedRef = 1;
#endif

		Assert( callingDomainProxyManager-> isValidProxy( proxy ) )
		    ( "ProxyCallAssist: invalid proxy %A for domain %N",
		    proxy, callingDomain  );

#ifdef REFBEFORETHIS
		}
#endif

		proxy = callingDomainProxyManager->domainToKernel( proxy );
		object = proxy->proxiedObject();
		calledDomain = proxy->proxiedDomain();

		/*
		 * Continued below after determining call type.  The
		 *   processing is split because we want to catch attempted
		 *   calls to domains that aren't mapped in before trying to
		 *   look at the object, and determination of call type is also
		 *   applicable to constructors.
		 */
	}

	ProxyCallType callType = InCall;
	if ( calledDomain == TheKernel->domain() ) {
		/*
		 * This is special cased because it's the most common case and
		 *   we want speed.  thisProc->domain()->parent(0) of every
		 *   Domain is the Kernel Domain; that is, there's only one
		 *   Domain in ring 0.
		 */
		Assert( callingDomain != TheKernel->domain() )
		  (" ProxyCallAssist: kernel-to-kernel proxy calls should never happen\n");

		callType = InCall;
	}
	else {
		Assert(NOTREACHED) ("can only call kernel domain \n"); 
	}
	Assert (callType == InCall);

	if ( (int) methodNumber != -1 ) {
		/*
		 * continuation of processing for regular method
		 */

		Assert( object != 0 )
		    ( "ProxyCallAssist: no proxiedObject for proxy %A", proxy );
		Assert( object->alive() )
		    ( "ProxyCallAssist: dead proxiedObject %A for proxy %A", object, proxy );

		ProxyTableCopy *table = proxy->table();

		Assert( ( methodNumber < table->numMethods ) )
		    ( "ProxyCallAssist: invalid proxy method %d on object %A",
		    methodNumber, object );

		meth = table->methods[ methodNumber ];

		if ( meth->realMethodNumber < SizeOfObjectProxyCatchTable ) {
			if ( ObjectProxyCatchTable[ methodNumber ] == 
						    &ObjectProxy::noCatch )  {
				/*
				 * nothing; need to reverse the sense of the
				 *  comparison to avoid CFRONT bug.
				 */
			}
			else {

				/*
				 * This is a method that must be caught and
				 *   processed specially.
				 */
				CATCHTYPE catchMethod =
					ObjectProxyCatchTable[ methodNumber ];
				if ( !( proxy->*catchMethod )() ) {
#ifdef DEBUG
					Console() <<
					    "ProxyCallAssist caught method,"
					      << " nothing more to do\n" << eor;
#endif
					returnFlags->calledMethod = 
						*((PFV *)(&catchMethod));
					return( 0 );
				}

				/*
				 * Need to free the proxy and then call
				 *   unreference() in the calling domain.
				 */
				callingDomain->proxyManager()->free( proxy );
				
				methodNumber = UNREFERENCE_METHOD_NUMBER;
				meth = table->methods[ methodNumber ];
			}
		}

		/*
		 * The vtable pointer is always at the end of the object
		 *  structure.
		 */
		vtable_entry *vtableptr = *(vtable_entry **) (((char *) object)
				+ sizeof(Object) - sizeof(vtable_entry *));

		method = (PFV) ( vtableptr + meth->realMethodNumber + 1 )->f;
	}

	returnFlags->calledMethod = method;

	if ( calledDomain == TheKernel->domain() ) {
		/*
		 * Check for optimization for null calls to kernel
		 */
		if ( ( meth->numParams == 0 ) && ( meth->returnClass == 0 ) ) {
#ifdef REFBEFORETHIS
			Assert( !skippedRef )
			  ( "ProxyCallAssist: proxy not found where needed\n" );
#endif
			*to = object;
			return( 1 );
		}
	}

#ifdef REFBEFORETHIS
	if ( skippedRef )
		*to++ = *(from - 1);
#endif

	*to++ = object;

	from++; /* skip 'this' */

	if ( meth->returnModifier == 'R' ) {
#ifdef REFBEFORETHIS
		Assert( skippedRef )
		    ( "ProxyCallAssist: Ref Pointer not found where needed\n" );
#endif

#ifdef CFRONT
		*to++ = *from++;
#endif
	}
#ifdef REFBEFORETHIS
	else {
		Assert( !skippedRef )
		    ( "ProxyCallAssist: proxy not found where needed\n" );
	}
#endif
	
	returnFlags->returnType = 0;
	returnFlags->numParamProxies = 0;
	returnFlags->thisProc = thisProc;

	if ( meth->numParams > 0 ) {
	    /*
	     * Copy/translate parameters
	     */
	    Class **classes = meth->paramClasses;
	    Class **endclasses = classes + meth->numParams;
	    switch( callType )
	    {
	    case InCall:
		for (; classes != endclasses; classes++)
		{
		    if ((*classes == 0) || (*from == 0))
			*to++ = *from++;
		    else
		    {
			*to = ProxyInCallParamAssist( *from, *classes,
			    callingDomain, calledDomain, returnFlags );
			Assert( *to != 0 )
			    ( "ProxyCallAssist: invalid parameter %d on incall to method %A: %A\n", classes - meth->paramClasses, (void*)method, *from );
			from++;
			to++;
		    }
		}
		break;
	default:
		Assert (NOTREACHED)("NO Other calls supported");
		break;
	    }
	}

	if ( meth->returnClass != 0 )
	{
		if (meth->returnModifier == 'R')
			returnFlags->returnType = 2;
		else
			returnFlags->returnType = 1;
		returnFlags->returnClass = meth->returnClass;
		returnFlags->callType = callType;
	}
	else if ( ( calledDomain == TheKernel->domain() ) && ( returnFlags->numParamProxies == 0 ) )
		/* another optimization */
		return( 1 );

	returnFlags->callingDomain = callingDomain;
	returnFlags->calledDomain = calledDomain;

	if ( calledDomain == TheKernel->domain() )
		return( 2 );
	else
		return( 3 );
}

/*
 * This is called from the proxy trap handler after the proxy method has
 *    returned.
 */

extern "C" 
ProxiableObject *
ProxyReturnAssist( ProxiableObject *returnValue, ProxyReturnFlags *returnFlags )
{
#ifdef DEBUG
	Console() << "ProxyReturnAssist ( "
		  << asHex( (int) returnValue ) << ", "
		  << asHex( (int) returnFlags ) << " )\n" << eor;
#endif

	Assert(returnFlags->calledDomain == TheKernel->domain());

	if ( returnFlags->numParamProxies > 0 ) {
		/*
		 * Free any incoming proxies that have never been referenced.
		 */
		ObjectProxyManager * m =
			returnFlags->calledDomain->proxyManager();
		for (int i = 0; i < returnFlags->numParamProxies; i++)
			returnFlags->paramProxies[i]->freeIfNotReferenced( m );
	}

	if (!(( returnFlags->returnType == 0 ) || ( returnValue == 0 ) )) {
		ObjectProxyManager * m = 
			returnFlags->callingDomain->proxyManager();
		switch( returnFlags->callType )
		{
		case InCall:
			/*
			 * An InCall return is like an OutCall parameter.
			 */
			returnValue = ProxyOutCallParamAssist( returnValue, 
				    returnFlags->returnClass,
					returnFlags->calledDomain,
					    returnFlags->callingDomain,
						returnFlags );
			
			Assert( returnValue != 0 )
			    ( "ProxyReturnAssist: invalid return value from incall to %A: %A\n",
				(unsigned int) returnFlags->calledMethod, returnValue );
			if ( m->isValidProxy( (ObjectProxy *) returnValue ) ) {
				returnValue = (ProxiableObject *) m->kernelToDomain
					( (ObjectProxy *) returnValue );
			}

			break;

		case OutCall:
			/*
			 * An OutCall return is like an InCall parameter.
			 */
			Assert(NOTREACHED) ("Outcalls not allowed");
			returnValue = ProxyInCallParamAssist( returnValue, 
				    returnFlags->returnClass,
					returnFlags->calledDomain,
					    returnFlags->callingDomain,
						returnFlags );
			
			Assert( returnValue != 0 )
			    ( "ProxyReturnAssist: invalid return value from outcall to %A: %A\n", (unsigned int) returnFlags->calledMethod, returnValue );
			if ( m->isValidProxy( (ObjectProxy *) returnValue ) ) {
				returnValue = (ProxiableObject *) m->kernelToDomain
					( (ObjectProxy *) returnValue );
			}

			break;

		default:
			/*
			 * Other types not implemented yet
			 */
			Assert( NOTREACHED )("Unknown proxy call type");
			break;
		}
	}

	return( returnValue );
}

class Domain;

int
ObjectProxy::noCatch()
{
	Assert( NOTREACHED );
	return( 0 );
}

int
ObjectProxy::catchReference()
{
	/*
	 * If someone references an ObjectProxy, want to reference the
	 *   ObjectProxy itself instead of the proxiedObject.
	 *   Similarly for unreference() and noRemainingReferences().
	 */

	_referenceCount.increment();

	notReferenced = 0;

	return( 0 );
}

int
ObjectProxy::catchUnreference()
{
	Assert( alive() )
	    ( "ObjectProxy::catchUnreference(): unreferencing dead proxy %A\n", this );

	if( _referenceCount.decrement() == 0 ) {
		return( catchNoRemainingReferences() );
	}

	return( 0 );
}

int
ObjectProxy::catchNoRemainingReferences()
{
	/*
	 * NOTE: this functionality is duplicated in ProxiableObject
	 */
	return( catchDestructor() );
}

int
ObjectProxy::catchDestructor()
{
	/*
	 * Free the proxy if the reference count is zero.
	 */
	int count = _referenceCount.value();

	Assert( count == 0 )
	    ( "%A: destructing proxy with non-zero reference count (%d)\n", this, count );

	return( 1 );
}

void
ObjectProxy::freeIfNotReferenced( ObjectProxyManager * thisMan )
{
	if ( notReferenced ) {

		/*
		 * Decrement the reference count on the proxied object and
		 *  free the proxy.  Even if the reference count becomes
		 *  zero, don't delete the proxied object; this is called
		 *  for parameters of type 'ProxiableObject *' (or a sub-class
		 *  of ProxiableObject) and if the proxy hadn't been in the
		 *  way then the parameter would never have been referenced.
		 *  If it hadn't been referenced, of course it would never
		 *  get unreferenced, which means that it wouldn't have gotten
		 *  deleted even if the count started out at zero.
		 */
		(void) proxiedObject()->_referenceCount.decrement();
		
		thisMan->free( this );
	}
}

void
ObjectProxy::setProxiedObject( ProxiableObject * who, Class * whoClass,
				Domain * dom, int returnsRef )
{
	Assert (returnsRef == 1 || returnsRef == 0);
	/*
	 * Don't use any virtual functions on 'who' because it may
	 *   not be in the current Domain!
	 */
	if ( who == 0 ) {
		key._proxiedObject = 0;
		return;
	}

	if ( key._proxiedObject == who ) {
		/*
		 * Resetting the same proxiedObject
		 */
		if ( returnsRef == 1 ) {
			/*
			 * Client now has an extra reference().
			 * Take it away from the proxiedObject and put it on
			 *     the proxy instead.
			 * We know the reference count is greater than 1
			 *    because this proxy has already referenced it.
			 * Can't use unreference() because that's a virtual
			 *    function which may not be in the kernel.
			 */
			(void) who->_referenceCount.decrement();
			(void) _referenceCount.increment();
		}
		return;
	}

#ifdef DEBUG
	Console() << "ObjectProxy(" << asHex( (unsigned int) this ) <<
	 ")::setProxiedObject( " << asHex( (unsigned int) who) << " )\n" << eor;
#endif
	key._proxiedObject = who;

	key._proxiedDomain = dom;

	if ( returnsRef == 1 )
	{
		/*
		 * Reference count on proxiedObject is already incremented.
		 * Start out proxy count also incremented.
		 */
		(void) _referenceCount.increment();
	}
	else 
	{
		/*
		 * Reference count on proxiedObject not incremented.  Reference
		 *  it and assume user will reference the proxy. (if they
		 *  don't, the proxy will still get deleted when the
		 *  Domain dies anyway).  Can't use reference() because that's
		 *  a virtual function which may not be in the kernel.
		 */
		who->_referenceCount.increment();
	}

	Assert( whoClass != 0 ) ( "%N: no class\n", who );

	key._table = whoClass->proxyTable();

	Assert( key._table != 0 )
	    ( "%A: no proxy table for class %s\n", who, whoClass->name() );

}

