#ifdef __GNUC__
#pragma implementation
#endif


//#define DEBUG_GIVEPROCESSORTO
/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * Process.c: Implements a generic thread of execution to build
 *            higher level processes out of.
 */

#include "AddressTranslator.h"
#include "Assert.h"
#include "Class.h"
#include "ClassStar.h"
#include "CPU.h"
#include "Domain.h"
#include "FreeRunningTimer.h"
#include "FreeRunningTimerStar.h"
#include "Kernel.h"
#include "MultiplexedOutputStream.h"
#include "NameServer.h"
#include "ObjectProxyManager.h"
#include "OutputStream.h"
#include "Process.h"
#include "ProcessorContext.h"
#include "PROCESSOR.h"
#include "PROCESSOR_CONTEXT.h"
#include "Raid.h"
#include "Semaphore.h"
#include "ThisCPU.h"
#include "ThisProcess.h"
#include "Timer.h"
#include "AppInterface.h"
//#define PROCESSBLOCKTRAP
#define PRINTSYSTEMPROCESSTIME

#ifdef PRINTSYSTEMPROCESSTIME
#include "GlobalLRUReplacementPolicy.h"
#include "Store.h"
#endif // PRINTSYSTEMPROCESSTIME

#ifdef __GNUC__
#pragma implementation
#endif

#ifdef DEBUG_GIVEPROCESSORTO
#define DebugGiveProcessorTo Console()
#else
#define DebugGiveProcessorTo Debug
#endif

int ProcessResourceUsageOn = 0;
int ProcessResourceUsageReportOn = 0;

#ifdef INSTRUMENT_PROCESS
#define Instr(exp) exp
#include "StringOutputStream.h"
#include "Instrument.h"
#include "SystemInstrumentManager.h"
#include "InstrumentEvent.h"
#include "PerCpuTimerInstrument.h"

extern SystemInstrumentManager *TheInstrumentManager;

PerCpuTimerInstrument *iProcessRestore = NULL;

void 
ProcessConstructInstruments()
{
    iProcessRestore = new PerCpuTimerInstrument
    ( TheInstrumentManager, "ProcessRestore", TALKING, 1, ALL);
}
void 
ProcessDeleteInstruments()
{
    delete iProcessRestore;
    iProcessRestore = NULL;
}

#else
#define Instr(exp)
#endif

/*
 * These should be moved or re-moved.
 */
SchedulerInfo::~SchedulerInfo() 
{
    Debug << this << "::~SchedulerInfo()\n" << eor;
}

/*
 * Create a new process, initialize all machine independent fields.
 */
Process::Process( Domain * initialDomain, ProcessorContext * context )
{
    initDebug();
    Raid( RaidConstructor )( "%N::Process( %N, %x, ... )\n",
                             this, initialDomain );
    Assert( this != 0 );
    Assert( initialDomain != 0 );

    _blockedOn = 0;
    _numLocksHeld = 0;

#ifdef BUST_LOCKS
    for(lockIndex = 0; lockIndex < MAX_LOCKS_HELD; lockIndex++) _held[lockIndex] = 0;
    lockIndex = 0;
#endif

    _inGPT = false;

    /*
     * Until there is always a process...
     */
    Process * process = thisProcess();
    if( process == 0 ) {
        _readyContainer = 0;
    } else {
        _readyContainer = process->_readyContainer;
    }

    _domain = initialDomain;
    _domainCleaner = initialDomain->cleaner();

    _context = context;
    _newContext = 0;
    _oldContext = 0;
    _state = ProcessNew;

    _quantum = 0;

    _serviceTime = 0;
    _processorStartTime = 0;
    _processorTimeInt = 0;
    _userStartTime = 0;

    _schedulerInfo = 0;
    _completionStatus = -1;
    _notifySemaphore = 0;
    _notifyStatus = 0;

    _console = ConsoleOutputStream;
    // _console = 0;
    _console = new MultiplexedOutputStream( _console );
    _console->setName( objectName() );

    _stackBusy = 0;

    _temporaryProxy = 0;

    /*
     * All processes start out running in the kernel.
     */
    _currentDomain = TheKernel->domain();

    /*
     * Because the kernel waste-management process thinks it owns
     * every process, reference it here.  It will be unreferenced
     * after it dies.
     */
    reference();

    if (thisProcess () != 0)
    {
        //  Inherit the current process' streams and FSI.
        standardInput = thisProcess () -> standardInput;
        standardOutput = thisProcess () -> standardOutput;
        fileSystemInterface = thisProcess () -> fileSystemInterface;
    }
    else
    {
        //  This is the very first process; initialize everything to 0.
        standardInput = 0;
        standardOutput = 0;
        fileSystemInterface = 0;
    }

    Raids( RaidConstructor ) << this << "::Process() done.\n" << eor;
}

/*
 * Free up all the resources owned by a Process and then destroy it.
 */
Process::~Process()
{
    /*
     * Should deallocate stacks and other resources here.
     * and what if its on a queue somewhere?
     */
    Raid( RaidConstructor )( "%N::~Process()\n", this );
    Assert( this != 0 );
    Assert( this != thisProcess() );

    /*
     * Wake up the waiting process if there is one.
     */
    // wakeUpWaiter( _completionStatus );
    Assert( dead() );
    Assert( _notifyStatus == 0 );

    /*
     * Delete scheduler info object if it exists.
     */
    if( _schedulerInfo != 0 ) {
        delete _schedulerInfo;
    }

    delete _context;
    Assert( _context != _newContext );
    Assert( _context != _oldContext );
    if ( _newContext ) 
        delete _newContext;
    if ( _oldContext )
        delete _oldContext;
}

void
Process::getResourceUsage( ProcessResourceUsage * usage )
{
    if ( ProcessResourceUsageOn ) {
        _usage.systemTime =  _processorTime - _usage.userTime;
        if ( _state == ProcessRunning ) {
            unsigned int time = SystemTimer->time();
            unsigned int elapsed = time - _processorStartTime;
            // Assert( elapsed < 0x80000000 );
            _usage.systemTime += elapsed;
        }
    }
    if ( usage )
        *usage = _usage;
}

void Process::exec(void (Object::*)(), int, char**)
{ Assert(NOTREACHED);}


/*
 * A non-preemptable process running in system mode.
 */
SystemProcess::SystemProcess( OPFV startAddress, Domain * initialDomain,
                              StackSize stackSize, int argument )
    : Process( initialDomain, new SYSTEM_CONTEXT( this,
                                                  //&trampoline, stackSize, startAddress, argument ) )
                                                  startAddress, stackSize, argument , 0) )
{
    initDebug();
    Raids( RaidConstructor ) << this << "::SystemProcess()\n" << eor;
    Assert( this != 0 );
}


/*
 * A non-preemptable process running in system mode.
 */
SystemProcess::SystemProcess( PFV startAddress, Domain * initialDomain,
                              StackSize stackSize, int argument )
    : Process( initialDomain, new SYSTEM_CONTEXT( this,
                                                  startAddress, stackSize, argument, 0 ) )
{
    initDebug();
    Raids( RaidConstructor ) << this << "::SystemProcess()\n" << eor;
    Assert( this != 0 );
}
void
Process::SVCHandler(stackFrame*)
{
    /* A normal process doesn't understand SVC traps */
    Assert(NOTREACHED);
}

NameServer *
Process::nameServer()
{
    return domain()->nameServer();
}

NameServerRef
Process::domainNameServer()
{
    return domain()->nameServer();
}

ObjectProxy *
Process::temporaryProxy()
{
    /*
     * return a proxy that can be used for temporary purposes from
     *    the kernel.
     */
    if ( _temporaryProxy == 0 ) {
        _temporaryProxy = TheKernel->domain()->proxyManager()->
        allocate( 0, 0, 0, 0 );
        Assert( _temporaryProxy != 0 );
    }
    return( _temporaryProxy );
}

/*
 * Dump the internal contents of a process
 */
void
Process::inspect( OutputStream * os )
{
    writeName( os );
    os->write( "(", 1 );
    domain()->inspect( os );
    os->write( ")", 1 );
}

void
Process::ready()
{
    Assert( _readyContainer != 0 );
    Assert( _state != ProcessReady );
    _state = ProcessReady;
    _readyContainer->add( this );
}

void
Process::relinquishProcessor()
{
    // for now, we can only tell ourself to relinquish. Noone else can
    // tell us.
    Assert( this == thisProcess() );
    Assert( this != 0 );

    Raids( RaidMethod ) 
    << this << "::relinquishProcessor()\n" << eor;

    if (_oldContext) {
        _oldContext = 0;
        delete _oldContext;
    }

    /*
     * NEVER remove from the idleContainer with interrupts on.
     */
    IntStorage intState ; PROCESSOR::disableInterrupts(intState);

    /*
     * Give to the next ready process or the idle Process
     * (CPU::getNextReadyProcess returns the idleProcess if
     * there's nothing in the queue).
     * Changed to give to idleProcess from returning to the current
     * process to avoid deadlocking with interupts off.
     * -raila 3-10-93
     */
    Process * nextProcess = thisCPU()->getNextReadyProcess();

    ready();
    Assert (! PROCESSOR::interruptsEnabled());
    giveProcessorTo( nextProcess, ProcessReady );

    PROCESSOR::restoreInterrupts(intState);
}

#ifdef PROCESSBLOCKTRAP
int ProcessBlockTrap = 0;
#endif // PROCESSBLOCKTRAP

void
Process::block( Object *s)
{
    /* block suspends this process and runs whatever is on the readyq*/
    /* whatever calls block should have put this process on some queue*/
    // for now, we can only tell ourself to block. Noone else can
    // tell us.  Also, idleProcesses cannot block.
    Assert( this == thisProcess() );
    Assert( this != 0 );

    Raids( RaidMethod ) << this << "::block()\n" << eor;

    if ( _oldContext ) {
        _oldContext = 0;
        delete _oldContext;
    }
#ifdef PROCESSBLOCKTRAP
    if ( ProcessBlockTrap && ProcessBlockTrap == getProcessEvent() ) {
        Console() << this << "::block trap\n" << eor;
    }
#endif // PROCESSBLOCKTRAP

    /*
     * Get another process ready to run on this CPU.
     */
    Process * nextProcess = thisCPU()->getNextReadyProcess();
    Assert( nextProcess != 0 );
    /*
     * Tell the other Process where to save the current process.
     */
    Assert( ! PROCESSOR::interruptsEnabled() );
    _blockedOn = s;
    giveProcessorTo( nextProcess, ProcessBlocked );
    _blockedOn = 0;
}

void
Process::die( int status )
{
    // for now, we can only tell ourself to die. Noone else can
    // tell us.
    Assert( this == thisProcess() );
	
    TheAppInterface->cleanup(this);

    Raids( RaidConstructor ) << this << "::die()\n" << eor;


    /*
     * We will no longer use the domain.  Unreference the domainCleaner
     * so the proxies get deleted while the domain is still activated.
     */
    _domainCleaner = 0;

    if ( _temporaryProxy != 0 ) {
        TheKernel->domain()->proxyManager()->free( _temporaryProxy );
        _temporaryProxy = 0;
    }

    _completionLock.acquire();
    _completionStatus = ( status < 0 ? -status : status );
    wakeUpWaiter( _completionStatus );
    _completionLock.release();

    if (_oldContext) {
        delete _oldContext;
        _oldContext = 0;
    }

    if ( ProcessResourceUsageReportOn ) {
        getResourceUsage( 0 );
        Console() << this << "::die()"
                  << "  systemTime = " << _usage.systemTime
                  << "  userTime = " << _usage.userTime << "\n" << eor;
    }

    /*
     * We arrange to give the processor to someone else.
     * This is because someone (maybe the only other process in the system)
     * might be waiting for us. They can't be restarted until our destructor
     * is called. This can't happen until were put into the Garbage by
     * dispatching some other process.
     *
     */
    PROCESSOR::disableInterrupts();
    changeReadyContainer( TheKernel->_garbageCan );

    /*
     * Get another process to run, and switch to it.
     */
    Process * p = thisCPU()->getNextReadyProcess();

    Assert (p != this);		// Can't give to ourself after dying

    /*
     * Since this process is terminating, we don't want to leave its
     * Domain active on the current processor. This is since the process
     * being given the processor might not have a preference which Domain
     * is running (a SystemProcess does this) and attempt to keep using
     * the current process' Domain. A race could occur when the
     * current process is removed from the garbage can and deleted,
     * potentially causing the Domain to be deleted while it is still
     * being used!
     *
     * The solution is to switch the processor over to the Domain of
     * the process being given the processor. For system processes
     * this will switch over to TheKernel->domain().
     */
    Raids( RaidConstructor ) << this << "::die() " << p << " "
                             << p->domain() << "\n" << eor;
    Assert( p->domain() != 0 );
    if( p->domain() != domain() ) {
        thisCPU()->MMU()->activate( p->domain());
    }
    _domain = 0;
	
    releaseStack();

    if (_numLocksHeld) Console() << "Process dying holding locks!\n" << eor;

#ifdef BUST_LOCKS
    // Release all locks that are held
    for(lockIndex = 0; lockIndex < MAX_LOCKS_HELD; lockIndex++)
        if (_held[lockIndex]) _held[lockIndex]->release();
    lockIndex = 0;
#endif

    ready(); /* This puts this Process in the garbageCan queue */
    giveProcessorTo( p, ProcessDead );
    Assert( NOTREACHED );
}

void
Process::restore( Process * previousProcess )
{
    Assert (this != 0);
    Instr(StringOutputStream comment);

    /* This gets called by restarted interrupt processes and by method
     * giveProcessorTo						   */
    Raids( RaidProtected ) << this << "::restore( " << previousProcess << " )\n" << eor;
    Debug << this << "::restore( " << previousProcess << " )\n" << eor;

    Assert( domain() != 0 );

#ifdef INSTRUMENT_PROCESS

    // make sure we have the right process address/name

    if (previousProcess == NULL) {
        // This has so far never happened

        Assert(NOTREACHED);
        comment.formattedWrite("%d,%d,NULL,%s", NULL, this, 
                               this->objectName());
    } else {
        // a regular context switch

        comment.formattedWrite("%d,%d,%s,%s", previousProcess, this, 
                               previousProcess->objectName(), this->objectName());
    }
#endif



    /* 
     * Release old process's stack for use by other processor's
     * Note Interrrupt processes get here but previousProcess is nil 
     */
    if (previousProcess != 0) {
        previousProcess->releaseStack();  
//		if (previousProcess->dead())
//			previousProcess->_domain = 0;
    }

    /*
     * First, do process management housekeeping for the process
     * that gave us the processor.
     */


#ifdef INSTRUMENT_PROCESS
    if (iProcessRestore != NULL) {
        iProcessRestore->stop(Process_Restore, comment.str());
        iProcessRestore->start();
    }
#endif

    /*
     * Next, do any subclass specific restoring. This includes changing the
     * VM (via activating another Domain) if this process is the
     * kind which cares (usually an ApplicationProcess).
     */
    basicRestore();
}

void
Process::wakeUpWaiter( int status )
{
    Raids( RaidMethod ) << this << "::wakeUpWaiter( " 
                        << status << ")\n" << eor;
    if( _notifySemaphore != 0 ) {
        Raids( RaidDetail ) 
        << this << "::wakeUpWaiter: 'V'ing " 
        << _notifySemaphore
        << "\n" << eor;
        Assert( _notifyStatus != 0 );
        * _notifyStatus = status;
        _notifySemaphore->V();
        _notifyStatus = 0;
        _notifySemaphore = 0;
    }
}

int
Process::waitForProcess( Process * other )
{
    Raids( RaidMethod ) 
    << this << "::waitForProcess( " << other << " )\n" << eor;
    Assert( this != 0 );
    Assert( other != 0 );
    int status;
    _completionLock.acquire();
    if ( _completionStatus >= 0 ) {
        status = completionStatus();
        _completionLock.release();
    } else {
        Semaphore * sem = new BinarySemaphore( 0 );
        other->notifyUponCompletion( sem, & status );
        _completionLock.release();
        sem->P();
        delete sem;	// Now that we're done with it
    }
    return status;
}

void
Process::giveProcessorTo( Process * anotherProcess, ProcessState newState )
{
    Assert( this != 0 );
    Assert( thisProcess() == 0 || this == thisProcess() );
    Assert( ! PROCESSOR::interruptsEnabled() )
    ("%X::giveProcessorTo(%X)\n", this, anotherProcess);
//	Assert (numLocksHeld() == 0 || newState == ProcessDead ) ("%X::giveProcessorTo(%X), holding %d locks!\n", this, anotherProcess, numLocksHeld(), 0);

    Raids( RaidProtected ) 
    << this << "::giveProcessorTo( " << anotherProcess << " )\n"
    << eor;
    DebugGiveProcessorTo 
    << this << "::giveProcessorTo( " << anotherProcess << " )\n"
    << eor;

    Assert( anotherProcess->context()->alive() );

#ifdef DEBUG_SOFTINT
    extern int softint_nest;
    extern int debug_softint;
    if (softint_nest > 0) {
        if (debug_softint)
            Console() 
            << this << "giveProcessTo( " << anotherProcess 
            << " ) softint_nest " << softint_nest << "\n" << eor;
    }
#endif

    /*
     * Do anything which is subclass specific during a save.
     */
    save();

    /*
     * Checkpoint the current state of this process/context.
     * context()->checkpoint() will return a 0 when the checkpoint
     * is first taken and the process who resumed this process
     * when the checkpoint is resumed.
     */
    // Can't put this on the stack in VCHoices volatile Process * processWhoResumedUs;
    Assert(context()->alive());
    Assert ( ! context()->_checkPointed );
    context()->_checkPointed = 1;
    _inGPT = true;

    if( ( context()->setResuming((Process *)context()->checkpoint()) ) == 0 ) {
        /*
         * This is where you get after the checkpoint is first
         * taken.
         */
        Assert( context()->alive() );
        Assert( thisProcess() == 0 || this == thisProcess() );
        Raids( RaidDetail )
        << this << ": checkpoint made\n" 
        << eor;
        DebugGiveProcessorTo 
        << this << ": checkpoint made\n" 
        << eor;


        if ( ProcessResourceUsageOn ) {
            unsigned int time = SystemTimer->time();
            unsigned int elapsed = time - _processorStartTime;
            // Assert( elapsed < 0x80000000 );
            _processorTime += elapsed;
            _processorTimeInt += elapsed;
            anotherProcess->_processorStartTime = 
            anotherProcess->_userStartTime = time;
            if ( newState == ProcessReady ) 
                _usage.involContextSwitch++;
            else
                _usage.volContextSwitch++;
        }

        /*
         * Lock the stack of the Process to which this Processor is switching 
         * in order to make sure that no other processor is using that stack.
         * The Processor busy waits
         * if another Processor is still using the stack.
         */
        anotherProcess->getStack();
        _state = newState;
        anotherProcess->_state = ProcessRunning;
        if (!anotherProcess->_inGPT && anotherProcess->context()->hasEverBeenDispatched) {
            anotherProcess->restore(this);
            anotherProcess->context()->_checkPointed = 0;
        }
        anotherProcess->context()->restore( this );
#ifndef VCHOICES
        Assert( NOTREACHED );
#endif
    }

    /*
     * This is where you get after the process is rolled
     * back to the spot it was checkpointed; this is not reached
     * when a process is restored from initial context
     */
    Assert( ! PROCESSOR::interruptsEnabled() )
    ("%X::giveProcessorTo(%X)\n", this, anotherProcess);
    Assert( this == thisProcess() );
    context()->_checkPointed = 0;

    Raids( RaidDetail )
    << this << ": checkpoint resumed\n" << eor;
    DebugGiveProcessorTo 
    << this << ": checkpoint resumed\n" << eor;

    /* Since we are now running on a new stack, free the old process
     * stack and let another processor use it, if required.
     */
    Assert (thisProcess()->context()->getResuming() != 0);
    restore( thisProcess()->context()->getResuming() );
	
    _inGPT = false;
}

void
Process::forcedSwitchTo(Process *nextProcess) {
    Assert (! PROCESSOR::interruptsEnabled());
    Assert( nextProcess->context()->alive() );
	
    context()->_checkPointed = 1;
    nextProcess->context()->_checkPointed = 0;
	
    save();
    Assert(context()->alive());

    if ( ProcessResourceUsageOn ) {
        unsigned int time = SystemTimer->time();
        unsigned int elapsed = time - _processorStartTime;
		
        _processorTime += elapsed;
        _processorTimeInt += elapsed;
        nextProcess->_processorStartTime = nextProcess->_userStartTime = time;
        _usage.involContextSwitch++;
    }
    nextProcess->getStack();
    nextProcess->_state = ProcessRunning;

    CPU::_runningProcesses[PROCESSOR::cpuID()] = nextProcess;
    if( nextProcess->context()->hasEverBeenDispatched ) {
        Debug << this << "::restore: not first time\n" << eor;
#ifdef NEW_SWITCH
        cacheKernelStack();
#endif
        if (!nextProcess->_inGPT)
            nextProcess->restore(this);
        else
            nextProcess->context()->setResuming(this);
    }
    else {
        Debug << this << "::restore: first time\n" << eor;
#ifdef NEW_SWITCH
        createKernelStack();
        cacheKernelStack();
#endif
        // This essentially does the same thing as PROCESSORCONTEXT::restoreFromInitialContext
        nextProcess->setCurrentDomain( nextProcess->domain() );
        nextProcess->context()->restoreAssist( this );
    }
}

ProcessRef 
Process :: clone (const char *entry, const char *stackPointer)
{
    Raids( RaidMethod )
    << this << "::clone( " << asAddress( entry ) << ", "
    << asAddress( stackPointer ) << " )\n" << eor;

    Assert( NOTREACHED );
    return 0;
}

void 
Process :: exec( PFV startAddr, int arg1, char ** arg2 )
{
    Assert( NOTREACHED );
}

/*
 * Constructor for use by applications themselves
 */

#ifdef PRINTSYSTEMPROCESSTIME

int PrintSystemProcessTime = 1;

void printSystemProcessTime( Process * process )
{
    if ( !PrintSystemProcessTime )
        return;

    CPU ** cpus = thisCPU()->_cpus;
    while ( *cpus != 0 ) {
        Debug << *cpus << " idleProcess processor time " 
                  << (*cpus)->idleProcess()->processorTime()
                  << "\n" << eor;
        cpus++;
    }
#ifdef SS1
    extern Process * diskControllerInterruptProcess;
    if ( diskControllerInterruptProcess ) {
        Debug << diskControllerInterruptProcess << " "
                  << diskControllerInterruptProcess->processorTime()
                  << "\n" << eor;
    }
#endif /*SS1*/
    Process * storePager = MainStore->pager();
    Debug << storePager << " " << storePager->processorTime()
              << "\n" << eor;
    Process * globalLRUPager = GlobalLRUReplacementPolicy::pager();
    if ( globalLRUPager ) Debug 
    << globalLRUPager << " " 
    << globalLRUPager->processorTime() << "\n" << eor;
}

#else

inline void printSystemProcessTime( Process * ) 
{
}

#endif // PRINTSYSTEMPROCESSTIME

ApplicationProcess::ApplicationProcess( ENTRYTYPE entry, int arg1, char **arg2,
                                        StackSize userStackSize )
    : Process( thisProcess()->domain(), new APPLICATION_CONTEXT( this,
                                                                 (PFV) entry, application, thisProcess()->domain(),
                                                                 arg1, (int) arg2, userStackSize ) ),
      _userStackSize( userStackSize )
{
    initDebug();
    Raid( RaidConstructor )( "%N::ApplicationProcess( %N, ... )\n",
                             this, thisProcess()->domain() );
    Assert( this != 0 );
    printSystemProcessTime( this );
}

/*
 * For an application process, both entryPoint and logicalEntryPoint are
 * the address specified in the executable file.
 */
ApplicationProcess::ApplicationProcess( PFV startAddr, Domain * initialDomain,
                                        int arg1, int arg2, StackSize userStackSize )
    : Process( initialDomain, new APPLICATION_CONTEXT( this,
                                                       startAddr, application, initialDomain,
                                                       arg1, arg2, userStackSize ) ),
      _userStackSize( userStackSize )
{
    initDebug();
    Raid( RaidConstructor )( "%N::ApplicationProcess( %N, ... )\n",
                             this, initialDomain );
    Assert( this != 0 );
    printSystemProcessTime( this );
}

ApplicationProcess::ApplicationProcess( PFV startAddr, Domain * initialDomain,
                                        int arg1, int arg2, char * userStackPointer )
    : Process( initialDomain, new APPLICATION_CONTEXT( this,
                                                       startAddr, application, initialDomain,
                                                       arg1, arg2, none, userStackPointer ) )
{
    initDebug();
    Raid( RaidConstructor )( "%N::ApplicationProcess( %N, ... )\n",
                             this, initialDomain );
    Assert( this != 0 );
    Assert( thisProcess()->isKindOf( ApplicationProcessClass ) );
    _userStackSize = 
    ( ( ApplicationProcess * ) thisProcess() )->userStackSize();
    printSystemProcessTime( this );
}

void ApplicationProcess::exec(void (Object::*)(), int, char**)
{ Assert(NOTREACHED);}

ApplicationProcess::~ApplicationProcess()
{
    Raid( RaidConstructor )( "%N::~ApplicationProcess()\n", this );
    printSystemProcessTime( this );
}

void
ApplicationProcess::basicRestore()
{

    /*
     * See if the processor is currently using the proper domain
     * and if not, switch it to the right one.
     */
    CPU * cpu = thisCPU();
    if( cpu->MMU()->currentlyActiveDomain() != domain() ) {
        cpu->MMU()->activate( domain() );
    }

    /*
     * Prime the timer running for this process. It isnt actually started
     * until the process is fully dispatched.
     */
    Raids( RaidProtected ) 
    << this << "::basicDispatch(): Prime timer\n" << eor;
    thisCPU()->timesliceTimer()->start( quantum() );
}

/*
 * ApplicationProcess's stop the timeSlice timer and set their residual.
 */
void
ApplicationProcess :: save()
{
    Raids( RaidProtected ) << this << "::save()\n" << eor;

    /*
     * Stop the timer for this processor and set the
     * processes quantum to the timers residual.
     */
    CPU * cpu = thisCPU();
    Assert( cpu->timesliceTimer() != 0 );
    setResidual( cpu->timesliceTimer()->stop() );
}

ProcessRef 
ApplicationProcess :: clone (const char *entry, const char *stackPointer)
{
    Raids( RaidMethod )
    << this << "::clone( " << asAddress( entry ) << ", "
    << asAddress( stackPointer ) << " )\n" << eor;

    DomainStar myDomain = thisProcess () -> domain ();
	
    //  Create a new domain.
    DomainStar newDomain = new Domain ( TheKernel->domain(),
                                        (char*)TheKernel->applicationStart(), 
                                        TheKernel->applicationSize());
	

    //  Initialize the new domain (eg. ObjectProxyManager).
    newDomain -> setDataEqual (myDomain);

    //  For every memory object in the current domain: if the memory object
    //  is writeable, copy it to the new domain, otherwise insert it
    //  directly into the new domain.  Obliterate everything in the new
    //  domain before copying anything.
    newDomain -> copy (myDomain);

    //  Create a new process and context for that domain.  Set the context
    //  of the new process so that it uses the given entry point and stack
    //  pointer.

    ProcessStar newProcess = 
    new ApplicationProcess ((PFV) entry, newDomain, 0, 0, 
                            (char *) stackPointer);
						     
    newDomain->setName("Cloned domain");
    newProcess->setName("Cloned process");

    return newProcess;
}


void 
ApplicationProcess :: exec( PFV startAddr, int arg1, char ** arg2 )
{
    if ( _oldContext ) {
        _oldContext = 0;
        delete _oldContext;
    }

    _newContext = new APPLICATION_CONTEXT
    ( this, startAddr, application, thisProcess()->domain(),
      arg1, (int) arg2, userStackSize() );
    PROCESSOR::disableInterrupts();
    save();
    _oldContext = _context;
    _context = _newContext;
    _newContext = 0;
    _context->restore( 0 );
    Assert( NOTREACHED );
}

StackSize ApplicationProcess::userStackSize()
{
    return _userStackSize;
}

/*
 * Processes which run with interrupts disabled and non-preemptable.
 */
InterruptProcess::InterruptProcess( PFV startAddress, Domain * initialDomain, 
                                    StackSize stackSize, int argument )
    : Process( initialDomain, new UNINTERRUPTABLE_SYSTEM_CONTEXT( this,
                                                                  startAddress, stackSize, argument, 0 ) )
{
    initDebug();
    Raid( RaidConstructor )( "%N::InterruptProcess( %N, ... )\n",
                             this, initialDomain );
    Assert( this != 0 );
}

InterruptProcess::~InterruptProcess()
{
    Console() << this << "::~InterruptProcess()\n" << eor;
}

/*
 * Each Processor runs this when it is idle.
 */
PerCPUIdleProcess::PerCPUIdleProcess( PFV startAddress, Domain * initialDomain,
                                      StackSize stackSize, int argument )
    : Process( initialDomain, new PER_CPU_IDLE_CONTEXT( this,
                                                        startAddress, stackSize, argument, 0 ) )
{
    initDebug();
    Raids( RaidConstructor ) << this << "::PerCPUIdleProcess()\n" << eor;
    Assert( this != 0 );

    /*
     * This process is the first to be dispatched on a processor.
     * There is a short period of time after this process is
     * dispatched but before virtual address translation is enabled,
     * It is important not to attempt to allocate any new objects
     * during this period, so we set up the console for this process
     * here rather than allocating it at the first use.
     */
    // _console = buildConsole();
    // _console->setName( "#PerCPUIdleProcessConsole" );
    Raids( RaidConstructor ) 
    << this << "::PerCPUIdleProcess() done\n" << eor;
}

PerCPUIdleProcess::~PerCPUIdleProcess()
{
    Raids( RaidConstructor ) << this << "::~PerCPUIdleProcess\n" << eor;
    Assert( NOTREACHED );
}

SystemProcess::~SystemProcess()
{
    Raids( RaidConstructor ) << this << "::~SystemProcess\n" << eor;
}

/*
 * Process running in system mode but preemptable.
 */
PreemptableSystemProcess::PreemptableSystemProcess( PFV startAddress, Domain * initialDomain, 
                                                    StackSize stackSize, int argument )
    : SystemProcess( startAddress, initialDomain, stackSize, argument )
        
{       
    initDebug();
    Raids( RaidConstructor ) 
    << this << "::PreemptableSystemProcess()\n" << eor;
}       

/*
 * A preemptable process running in system mode.
 */
PreemptableSystemProcess::PreemptableSystemProcess( OPFV startAddress, Domain * initialDomain,
                                                    StackSize stackSize, int argument )
    : SystemProcess( startAddress, initialDomain, stackSize, argument)
{
    initDebug();
    Raids( RaidConstructor ) << this << "::PreemptableSystemProcess()\n" << eor;
    Assert( this != 0 );
}

PreemptableSystemProcess::~PreemptableSystemProcess()
{       
    Raids( RaidConstructor ) 
    << this << "::~PreemptableSystemProcess\n" << eor;
}       

void
PreemptableSystemProcess::basicRestore()
{
    /*
     * Prime the timer running for this process. It isnt actually started
     * until the process is fully dispatched.
     */
    Raids( RaidProtected ) 
    << this << "::basicDispatch(): Prime timer\n" << eor;
    thisCPU()->timesliceTimer()->start( quantum() );
}

/*
 * PreemptableSystemProcess's stop the timeSlice timer and set their residual.
 */
void
PreemptableSystemProcess :: save()
{
    Raids( RaidProtected ) << this << "::save()\n" << eor;

    /*
     * Stop the timer for this processor and set the
     * processes quantum to the timers residual.
     */
    CPU * cpu = thisCPU();
    Assert( cpu->timesliceTimer() != 0 );
    setResidual( cpu->timesliceTimer()->stop() );
}

#include "ClassInstaller.h"
DEFINECLASS("Process",Process,ProxiableObject,0,0);
DEFINECLASS("ApplicationProcess",ApplicationProcess,Process,0,0);
DEFINECLASS("InterruptProcess",InterruptProcess,Process,0,0);
DEFINECLASS("SystemProcess",SystemProcess,Process,0,0);
DEFINECLASS("PerCPUIdleProcess",PerCPUIdleProcess,Process,0,0);
DEFINECLASS("PreemptableSystemProcess",PreemptableSystemProcess,SystemProcess,0,0);
DEFINECLASS("SchedulerInfo",SchedulerInfo,Object,0,0);
