#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ProcessContainer.c: ProcessContainer base class.
 */

#include "Assert.h"
#include "OutputStream.h"
#include "Process.h"
#include "ProcessContainer.h"


#ifdef __GNUC__
#pragma implementation
#endif

/*
 * ProcessContainer is the base class (surprise...).
 * The base class doesn't implement anything.
 */

void
NullProcessContainer::add( Process * )
{
}

Process *
NullProcessContainer::remove()
{
	return( 0 );
}

int
NullProcessContainer::isEmpty()
{
	return( 1 );
}


SingleProcessHolder::SingleProcessHolder()
{
	theProcess = 0;
}

SingleProcessHolder::~SingleProcessHolder()
{
	Debug << this << "::~SingleProcessHolder()\n" << eor;
	Assert( theProcess == 0 );
}

void
SingleProcessHolder::add( Process * process )
{
	if( theProcess != 0 ) {
		Console().panic( "SingleProcessHolder%T::add( %A ): %A already on\n",
			this, process, theProcess );
	}
	Assert( theProcess == 0 );
	theProcess = process;
}

Process *
SingleProcessHolder::remove()
{
	Process * process = theProcess;
	theProcess = 0;
	return( process );
}

int
SingleProcessHolder::isEmpty()
{
	return( theProcess == 0 );
}


LockedSingleProcessHolder::LockedSingleProcessHolder()
{
}

LockedSingleProcessHolder::~LockedSingleProcessHolder()
{
	Debug << this << "::~LockedSingleProcessHolder()\n" << eor;
	Assert( ! lock.heldByMe() );
	Assert( theProcess == 0 );
}

void
LockedSingleProcessHolder::add( Process * process )
{
	Assert( theProcess == 0 );
	Assert( ! lock.heldByMe() );
	lock.acquire();
	theProcess = process;
	lock.release();
}

Process *
LockedSingleProcessHolder::remove()
{
	Assert( ! lock.heldByMe() );
	lock.acquire();
	Process * process = theProcess;
	theProcess = 0;
	lock.release();
	return( process );
}


#include "ClassInstaller.h"
DEFINECLASS("ProcessContainer",ProcessContainer,Object,0,0);
DEFINECLASS("NullProcessContainer",NullProcessContainer,ProcessContainer,0,0);
DEFINECLASS("SingleProcessHolder",SingleProcessHolder,ProcessContainer,0,0);
