#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ProcessorContext.c: Processor context class.
 */

	/*
	 *	NEW-SWITCH is for the new Process switching code
	 *	test infrastructure.  It doesn't do anything but 
	 *	slow things down due to VM inefficiencies.
	 *	Only use it to test Domain::constructChain
	 *	and Domain::makeAddressable().
	 *	-raila 5-21-92
	 */
// #define NEW_SWITCH

#include "Assert.h"
#include "CPU.h"
#include "Class.h"
#include "ThisCPU.h"
#include "OutputStream.h"
#include "ThisProcess.h"
#include "Process.h"
#include "ProcessorContext.h"
#include "Domain.h"
#include "PagedMemoryObjectCache.h"
#include "SourceOfZerosMemoryObject.h"
#include "PAGE_TABLE_CONSTANTS.h"
#include "StackSize.h"
#include "Kernel.h"
#include "PROCESSOR.h"


#ifdef __GNUC__
#pragma implementation
#endif

const unsigned STACK_FACTOR = 1;

ProcessorContext::ProcessorContext( Process * process )
	: owner( process ), hasEverBeenDispatched( 0 )
{
	Assert( process != 0 );
	_checkPointed = 0;
}

ProcessorContext::~ProcessorContext()
{
	// _kernelStackMO = 0;
}

void
ProcessorContext::createKernelStack( )
{
	_kernelStackMO = new SourceOfZerosMemoryObject;
	Assert (_kernelStackMO != 0);
	_kernelStackMO->setName("aKernelStackSource");
	unsigned stackPages = STACK_FACTOR * small;
	_kernelStackSize = stackPages * PhysicalPageSize;
	_kernelStackMO->setNumberOfUnits(_kernelStackSize);
	_kernelStackMO->setCacheClass( PagedMemoryObjectCacheClass );
	MemoryObjectCache * cache = _kernelStackMO->buildCache();
	Assert (cache != 0);
	cache->setName("aKernelStackCache");
	_kernelStack = (char *) TheKernel->domain()->add
		                 (_kernelStackMO, NoAccess);
}

void
ProcessorContext::cacheKernelStack( )
{
	char *base = kernelStackBase();
	unsigned size = kernelStackSize();
	Assert (size != 0);

	// _kernelChain = process()->domain()->constructChain(base, size);
	process()->domain()->makeAddressable(base, size);
}

void
ProcessorContext::releaseKernelStack( )
{
	// _kernelChain = PhysicalMemoryChain();
}


/*
 * NOTE: Be aware that if thisProcess() is called here, it returns the process
 * giving up the processor not the process being restored.
 */
void
ProcessorContext::restore( Process * processGivingUpProcessor )
{
	Assert (! PROCESSOR::interruptsEnabled());
	CPU::_runningProcesses[PROCESSOR::cpuID()] = owner;
	if( hasEverBeenDispatched ) {
		Debug << this << "::restore: not first time\n" << eor;
#ifdef NEW_SWITCH
		cacheKernelStack();
#endif
		basicRestore( processGivingUpProcessor );
	}
	else {
		Debug << this << "::restore: first time\n" << eor;
#ifdef NEW_SWITCH
		createKernelStack();
		cacheKernelStack();
#endif
		restoreFromInitialContext( processGivingUpProcessor );
	}
	Assert( NOTREACHED );
}

/*
 * This is used when a process is restore from initial context
 * This is (has to) to be called within the context of the new process since it
 * might add the previous process to a ProcessContainer somewhere.
 */
void
ProcessorContext::restoreAssist( Process * previousProcess )
{
	Debug << this << "::restoreAssist( " << previousProcess << " )\n"
		<< eor;

	Assert( this != 0 );
	Assert( thisProcess() == 0 || this == thisProcess()->context() );
	Assert( isKindOf( ProcessorContextClass ) );
	Assert( process() != 0 );
#ifdef OLDINT
	Assert( !isInterruptable() );
#else
	Assert(! PROCESSOR::interruptsEnabled() );
#endif

	/*
	 * See if this is the first time this process is ever been dispatched
	 * and if so make sure from now on it doesn't get the initial dispatch
	 * code called. You can do this by unconditionally setting the flag
	 * to 'true' since the mere fact a process got here means it has
	 * been dispatched.
	 */
	hasEverBeenDispatched = 1;

#ifdef NEW_SWITCH
	 if (previousProcess != 0) 
		previousProcess->context()->releaseKernelStack();
#endif
	 process()->restore( previousProcess );
}


#include "ClassInstaller.h"
DEFINECLASS("ProcessorContext",ProcessorContext,Object,0,0);
