#ifdef __GNUC__
#pragma implementation
#endif

/* #define PROXYDEBUG */
/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ProxyTableCopy.cc
 */

#include "ProxyTableCopy.h"
#include "Assert.h"
#include "Class.h"
#include "ClassStar.h"
#include "NameServer.h"
#include "OutputStream.h"
#include "Process.h"
#include "ThisProcess.h"
#include "Assert.h"


#ifdef __GNUC__
#pragma implementation
#endif

#ifdef PROXYDEBUG
void
MethodTableEntryCopy::dump()
{
    Console().printf(" return type: ");
    if (returnClass) {
	if (returnClass == (Class *) 1)
	    Console().printf("something * ");
	else if (returnModifier)
	{
	    switch(returnModifier)
	    {
	    case 'R':
		Console().printf("%sRef ",
			returnClass->name()); break;
	    case 'S':
		Console().printf("%sStar ", returnClass->name());
		break;
	    default:
		Console().printf("%s %c ", returnClass->name(),
				    returnModifier);
		break;
	    }
	}
	else
	    Console().printf("%s ", returnClass->name());
	Console().printf("\n");
    }
    else
	Console().printf("none\n");
    for (int j = 0; j < numParams; j++) {
	Console().printf("  param %d type: ",j);
	if (paramClasses[j])
	{
	    if (paramClasses[j] == (Class *) 1)
		Console().printf("something *\n");
	    else
		Console().printf("%s\n", paramClasses[j]->name());
	}
	else
	    Console().printf("none\n");
    }
}
#endif

void
ProxyTableCopy::dump()
{
#ifdef PROXYDEBUG
    if (isProxiable)
	Console().printf("Table for proxiable class ");
    else
	Console().printf("Table for non-proxiable class ");
    if (tableClass != 0)
	Console().printf("%s\n",tableClass->name());
    else
	Console().printf("unknown\n");

    int i;
    for (i = 0; i < numMethods; i++) {
	Console().printf("method %d->%d",i, methods[i]->realMethodNumber);
	methods[i]->dump();
    }
    for (i = 0; i < numConstructors; i++) {
	Console().printf("constructor %d",i);
	constructors[i]->dump();
    }
#endif
}

MethodTableEntryCopy::MethodTableEntryCopy( MethodTableEntry *m )
{
    ClassStar proxiableObjectClass =
	thisProcess()->nameServer()->lookup( "ProxiableObject", ClassClass );

    if (m->returnClass != 0)
    {
	ClassStar retClass =
	    thisProcess()->nameServer()->lookup( m->returnClass, ClassClass );
	if ((retClass != 0) &&
		    retClass->isASubclassOf( proxiableObjectClass ))
	    returnClass = retClass;
	else
	    /* set to '1' to indicate that there was a
	     *   non-proxiable-object pointer
	     */
	    returnClass = (Class *) 1;
    }
    else
	returnClass = 0;
    switch( m->returnModifier )
    {
    case 'S':
	Assert( NOTREACHED ) ( "return Star not supported\n" );
	break;
    case '&':
	returnModifier = '*';
	break;
    default:
	returnModifier = m->returnModifier;
	break;
    }
    realMethodNumber = m->realMethodNumber;
    Assert( realMethodNumber >= 0 && realMethodNumber < MAXMETHODS )
			( "Bad realMethodNumber: %d\n", realMethodNumber );
    numParams = m->numParams;
    Assert( numParams >= 0 && numParams < MAXPARAMS )
			( "Bad numParams: %d\n", numParams );

    if ( numParams > 0 ) {
	    paramClasses = new Class * [ numParams ];
	    Assert( paramClasses != 0 );
    }
    else
	    paramClasses = 0;

    for (int j = 0; j < numParams; j++)
    {
	char *p = m->paramClasses[j];
	if (p != 0)
	{
	    ClassStar pClass = thisProcess()->nameServer()->lookup( p,
								ClassClass );
	    if ((pClass != 0) &&
			    pClass->isASubclassOf( proxiableObjectClass ))
		paramClasses[j] = (Class *) pClass;
	    else
		/* set to '1' to indicate that there was a
		 *   non-proxiable-object pointer
		 */
		paramClasses[j] = (Class *) 1;
	}
	else
	    paramClasses[j] = 0;
    }
}

MethodTableEntryCopy::~MethodTableEntryCopy()
{
    if ( numParams > 0 )
    	delete paramClasses;
}

ProxyTableCopy::ProxyTableCopy( ProxyTable * t )
{
    unsigned i;
    ClassStar myClass = thisProcess()->nameServer()->lookup( t->tableClass,
								ClassClass );
    if (myClass == 0)
    {
	Console() << "Warning: cannot find " << t->tableClass << "Class,\n" <<
		    "    Proxies will not work on that class\n" << eor;
	return;
    }

    tableClass = myClass;
    isProxiable = t->isProxiable;
    numMethods = t->numMethods;
    Assert( numMethods > 0 && numMethods < MAXMETHODS )
	( "Bad numMethods on class %s: %d\n", t->tableClass, numMethods );
    numConstructors = t->numConstructors;
    Assert( numConstructors >=0 && numConstructors < MAXMETHODS )
	( "Bad numConstructors on class %s: %d\n", t->tableClass,
							numConstructors );

    if ( numMethods > 0 ) {
	methods = new MethodTableEntryCopy * [ numMethods ];
	Assert( methods != 0 );
	for (i = 0; i < numMethods; i++) {
	    methods[ i ] = new MethodTableEntryCopy( t->methods[i] );
	    Assert( methods[ i ] != 0 );
	}
    }
    else
	methods = 0;

    if ( numConstructors > 0 ) {
	constructors = new MethodTableEntryCopy * [ numConstructors ];
	Assert( constructors != 0 );

	for (i = 0; i < numConstructors; i++) {
	    constructors[ i ] = new MethodTableEntryCopy( t->constructors[i] );
	    Assert( constructors[ i ] != 0 );
	}
    }
    else
	constructors = 0;

    dump();

    myClass->setProxyTable( this );
}

ProxyTableCopy::~ProxyTableCopy()
{
    unsigned i;
    if ( numMethods > 0 ) {
	for (i = 0; i < numMethods; i++)
	    delete methods[i];
	delete methods;
    }

    if ( numConstructors > 0 ) {
	for (i = 0; i < numConstructors; i++)
	    delete constructors[i];
	delete constructors;
    }
}

#include "ClassInstaller.h"
DEFINECLASS("ProxyTableCopy",ProxyTableCopy,ProxiableObject,0,0);
