#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1994
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SoftClock.cc
 */

#include "Assert.h"
#include "OutputStream.h"
#include "SoftClock.h"
#include "SoftInt.h"
#include "PROCESSOR.h"
#include "Time.h"
#include "TimeOfDay.h"
#include "UnixTime.h"

#ifdef __GNUC__
#pragma implementation
#endif

TimeValue _theTime;

SoftwareClockManager * TheSoftwareClockManager = 0;

SoftwareClockManager::SoftwareClockManager()
	: _hz( 0 ), _usecsPerTick( 0 )
{
	Assert( TheSoftwareClockManager == 0 );
	_free = _softcalls;
	for ( int i = 1; i < MaxSoftCalls; i++ ) {
		_softcalls[ i - 1 ]._next = _softcalls + i;
	}
	_todo._next = 0;
}

void
SoftwareClockManager::start()
{
	Assert( _hz != 0 && _usecsPerTick != 0 );
	UnixTime unixTime;
	theSystemClock->getUnixTime( unixTime );
	_theTime.setSec( unixTime.seconds() + 1);
	_theTime.setUsec( 0 );
}

void doSoftint(char *a)
{
  TheSoftwareClockManager->softClockInterrupt(a, 0);
}

int
SoftwareClockManager::hardClockInterrupt()
{
    if (TheSoftwareClockManager == 0) return 0;
	TheSoftwareClockManager->_lock.acquire();
	int softint = 0;
	SoftCall * p1 = TheSoftwareClockManager->_todo._next;
	while( p1 ) {
		if ( --(p1->_ticks) > 0 )
			break;
		softint = 1;
		if ( p1->_ticks == 0 )
			break;
		p1 = p1->_next;
	}
	_theTime += TheSoftwareClockManager->_usecsPerTick;
	if ( softint ) {
		TheSoftwareInterruptManager->add
			( &doSoftint, 
		          (char *) TheSoftwareClockManager);
	}
	TheSoftwareClockManager->_lock.release();
	return 0;
}

void
SoftwareClockManager::softClockInterrupt(char * p_unused, int i_unused)
{
	while(1) {
		_lock.acquire();
		SoftCall * p1 = _todo._next;
		if ( p1 == 0 || p1->_ticks > 0 ) {
			_lock.release();
			break;
		}
		char * arg = p1->_arg;
		SoftClockFunc func = p1->_func;
		unsigned ticks = p1->_ticks;
		_todo._next = p1->_next;
		p1->_next = _free;
		_free = p1;
		_lock.release();
		(*func)(arg, ticks);
	}
}

void
SoftwareClockManager::add( SoftClockFunc func, char * arg, int ticks )
{
	if ( ticks < 0 ) 
		ticks = 1;
	_lock.acquire();
	SoftCall * newsoftcall = _free;
	if ( newsoftcall == 0 ) {
		Console() 
		<< "SoftClockManager::add() too many timeout entries\n" << eor;
		Assert( NOTREACHED );
	}
	_free = newsoftcall->_next;
	newsoftcall->_func = func;
	newsoftcall->_arg = arg;
	SoftCall * p1;
	SoftCall * p2;
	for ( p1 = &_todo; ( p2 = p1->_next ) && p2->_ticks < ticks; p1 = p2 ) {
		if ( p2->_ticks > 0 ) {
			ticks -= p2->_ticks;
		}
	}
	p1->_next = newsoftcall;
	newsoftcall->_next = p2;
	newsoftcall->_ticks = ticks;
	if ( p2 )
		p2->_ticks -= ticks;
	_lock.release();
}

int
SoftwareClockManager::remove( SoftClockFunc func, char * arg ) 
{
	_lock.acquire();
	SoftCall * p1;
	SoftCall * p2;
	for ( p1 = &_todo; ( p2 = p1->_next ) != 0; p1 = p2 ) {
		if ( p2->_func == func && p2->_arg == arg ) {
			if ( p2->_next && p2->_ticks > 0 ) {
				p2->_next->_ticks += p2->_ticks;
			}
			p1->_next = p2->_next;
			p2->_next = _free;
			_free = p2;
			_lock.release();
			return 1;
		}
	}
	_lock.release();
	return 0;
}

