#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1994
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SoftInt.cc
 */

#include "Assert.h"
#include "OutputStream.h"
#include "SoftInt.h"

#ifdef __GNUC__
#pragma implementation
#endif

SoftwareInterruptManager * TheSoftwareInterruptManager = 0;

SoftwareInterruptManager::SoftwareInterruptManager()
	: _head( 0 ), _free( 0 ), _tail( 0 )
{
	for( SoftCall * sc = _softcalls; sc < _softcalls + MaxSoftCalls; sc++ ){
		sc->_next = _free;
		_free = sc;
	} 
}

void
SoftwareInterruptManager::add( SoftCallFunc func, char * arg )
{
	_lock.acquire();
	SoftCall *sc;
	for( sc = _head; sc != 0; sc = sc->_next ) {
		if ( sc->_func == func && sc->_arg == arg ) 
			goto done;
	}
	sc = _free;
	if ( sc == 0 ) {
		Console() 
		<< "SoftwareInterruptManager::add() too many softcalls\n" 
		<< eor;
		Assert( NOTREACHED );
	}
	_free = sc->_next;
	sc->_func = func;
	sc->_arg = arg;
	sc->_next = 0;
	if ( _head ) {
		_tail->_next = sc;
		_tail = sc;
	} else {
		_tail = _head = sc;
	}
done:	
	enableSoftwareInterrupt();
	_lock.release();
}

int
SoftwareInterruptManager::softwareInterrupt()
{
	if ( _head == 0 )
		return 0;
	_lock.acquire();
	while( SoftCall * sc = _head ) {
		_head = sc->_next;
		SoftCallFunc func = sc->_func;
		char * arg = sc->_arg;
		sc->_next = _free;
		_free = sc;
		_lock.release();
		(*func)(arg);
		_lock.acquire();
	}
	_lock.release();
	return 1;
}
