#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Author: Panagiotis Kougiouris (kougiour@cs.uiuc.edu)
 * Contributing Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * TerminalProcess.cc: the process that awaits terminal characters
 *
 */

#include "Assert.h"
#include "CPU.h"
#include "ThisCPU.h"
#include "InputLogicalDevice.h"
#include "AwaitedInterruptInterrupt.h"
#include "OutputLogicalDevice.h"
#include "ThisProcess.h"
#include "Vectors.h"


#ifdef __GNUC__
#pragma implementation
#endif

static void
TerminalInputInterruptProcessEntry( int dev )
{
	/*
	 * The argument is passed as an int because the process constructor
	 * takes ints as initial arguments to the entry point.
	 */
	InputLogicalDevice * device = (InputLogicalDevice *) dev;
	Debug.printf( "TerminalInputInterruptProcessEntry( device:%x \n", device );

	AwaitedInterruptInterrupt * interrupt =
		new AwaitedInterruptInterrupt( "Terminal Input", 0 );

	/* get the vector number from the device */
	int vectorNumber = device->inputInterruptVector();
	thisCPU()->setInterrupt( vectorNumber , interrupt );
	Console().cpuPrintf( "TerminalInput vector set\n" );

	/*
	 * Loop forever waiting for a character interrupt and then processing
	 * the input character.
	 */
	while( 1 ) {
		Debug.printf( "TerminalInputInterruptProcessEntry: awaiting interrupt\n" );
		interrupt->await();
		Debug.printf( "TerminalInputInterruptProcessEntry: await returned\n" );

		/*
		 * Process the input.
		 */
		device->inputInterruptHandle();
	}
}

static void
TerminalOutputInterruptProcessEntry( OutputLogicalDevice * device )
{
	Debug.printf( "TerminalOutputInterruptProcessEntry( device:%x )\n", device );

	AwaitedInterruptInterrupt * interrupt =
		new AwaitedInterruptInterrupt( "Terminal Output", 0 );

	/* get the output vector number from the device */
	int vectorNumber = device->outputInterruptVector();
	thisCPU()->setInterrupt( vectorNumber, interrupt );

	/*
	 * Loop forever processing interrupts.
	 */
	while( 1 ) {
		Debug.printf( "TerminalOutputInterruptProcess: awaiting interrupt\n" );
		interrupt->await();
		Debug.printf( "TerminalOutputInterruptProcess: got interrupt\n" );

		/*
		 * Process the interrupt.
		 */
		device->outputInterruptHandle();
	}
}

void
TerminalManagerProcessEntry( int arg )
{
	InputLogicalDevice * inDev = ( InputLogicalDevice *) arg;
	OutputLogicalDevice * outDev = 0 ; // DANGER: It should have passed
					   // as parameter.
	Debug.printf( "TerminalManagerProcessEntry( in: %x  out: %x  )\n",inDev, outDev );

	/*
	 * Process to await characters from serial port.
	 */
	Process * process = new InterruptProcess(
		(PFV) TerminalInputInterruptProcessEntry,
		thisProcess()-> domain(), normal,
		(int) inDev );
	Assert( process != 0 );
	process->setName( "#TerminalInputInterrupt" );
	Debug.printf( "TerminalManagerProcess: Console interrupt process at %A\n", process );

	process->ready();
	thisProcess()->relinquishProcessor();
	inDev->enableInputInterrupt();

#ifdef 0
	/*
	 * Next, start the output interrupt process.
	 */
	process = new InterruptProcess(
		(PFV) TerminalOutputInterruptProcessEntry,
		thisProcess()->domain(), normal,
		(int) outDev,  0, 0, 0 );
	Assert( process != 0 );
	process->setName( "#TerminalOutputInterrupt" );
	Debug.printf( "TerminalManagerProcess: output process at %A\n",
		process );

	process->ready();
	thisProcess()->relinquishProcessor();
	outDev->enableOutputInterrupt();
#endif

	/*
	 * Call the terminal initialization code.
	 */
	Debug.printf( "TerminalManagerProcess: interrupt processes started\n" );
}
