#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * TimeSliceInterrupt.c: what to do when a timeslice interrupt occurs.
 */

#include "Assert.h"
#include "CPU.h"
#include "Class.h"
#include "ThisCPU.h"
#include "OutputStream.h"
#include "ThisProcess.h"
#include "Process.h"
#include "ProcessContainer.h"
#include "Raid.h"
#include "TimeSliceInterrupt.h"
#include "Timer.h"
#include "PROCESSOR.h"


#ifdef __GNUC__
#pragma implementation
#endif

/*
 * Just like a thisProcess()->relinquishProcessor(), except reset the
 * timesliceTimer to make sure that its residual returns 0.
 * This is method usually called from the actual hardware interrupt.
 */

TimeSliceInterrupt::TimeSliceInterrupt()
{
}

void
TimeSliceInterrupt::basicRaise( char * )
{
	Debug << this << "::basicRaise()\n" << eor;

	Process * current = thisProcess();
	CPU * currentCPU = thisCPU();

#ifdef WARNING
	if (current->isKindOf(SystemProcessClass) 
		&& current != currentCPU->idleProcess()) {
		Console() << "\nWARNING: " << current 
			<< " exceeded timeslice!\n";
		Console().flush();
		}
#endif
	/*
	 * Force the timer to return a 0 residual when the process
	 * reads it next.
	 */
	currentCPU->timesliceTimer()->set( 0 );

	IntStorage intState ; PROCESSOR::disableInterrupts(intState);

	/*
	 * See if there is another process ready to run on this CPU.
	 */
	Process * nextProcess = currentCPU->idleContainer()->remove();

	/*
	 * If there is another process, give the processor to it;
	 * put this process in
	 * its idleContainer(). Otherwise, just continue running
	 * who was running.
	 */
	if( nextProcess != 0 ) {
		Assert( current != 0 );
		Raids( RaidDetail ) << "give to " << nextProcess << "\n" << eor;
		current -> ready();
		current->giveProcessorTo( nextProcess, ProcessReady );
	}
	else {
		/*
		 * Start the timer for another timeslice.
		 */
		currentCPU->timesliceTimer()->start( current->quantum() );
		Debug << "::basicRaise(): returning to caller\n" << eor;
	}

	PROCESSOR::restoreInterrupts(intState);
}


#include "ClassInstaller.h"
DEFINECLASS("TimeSliceInterrupt",TimeSliceInterrupt,Interrupt,0,0);
