#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * TimerManager.c - machine independent portion of timer managing
 */

#include "Assert.h"
#include "CPU.h"
#include "FreeRunningTimer.h"
#include "FreeRunningTimerStar.h"
#include "OutputStream.h"
#include "Semaphore.h"
#include "TimerManager.h"


#ifdef __GNUC__
#pragma implementation
#endif


TimerManager::TimerManager(PFI ptb, PFV ep)
{
	Debug << "TimerManager::TimerManager()\n" << eor;
	semaphore = new Semaphore(0);
	semaphore->setName("#TimerManagerSemaphore");
	busyFlag = 0;
	perTickBookKeeping = ptb;
	expiryProcessing = ep;
}

void
TimerManager::interruptHandler()
{
	//Debug << "TimerManager::interruptHandler()\n" << eor;

	busyLock.acquire(); /* lock is needed to make test/set atomic */

	if (busyFlag)
	{
		/* skip this tick */
		busyLock.release();
	}
	else
	{
		busyFlag = 1;

		busyLock.release();

		lastFreeRunningTime += microsecondsPerTick;
		if ((*perTickBookKeeping)())
		{
	    /* a timer is expiring this tick, start the process */
	    semaphore->V();
		}
		else
		{
	    /* don't need to start the process this tick */
	    busyFlag = 0;
		}
	}

	//Debug << "Exiting TimerManager::interruptHandler()\n" << eor;
}

void
TimerManager::processHandler()
{
	Debug << "Entering TimerManager::processHandler()..... \n" << eor;

	while(1)
	{
		semaphore->P();

		Debug << "TimerManager::processHandler(): Timer Expired\n" << eor;

		(*expiryProcessing)();

		while ((int) (SystemTimer->time() - lastFreeRunningTime)
					 >= (int) microsecondsPerTick)
		{
	    lastFreeRunningTime += microsecondsPerTick;
	    if ((*perTickBookKeeping)())
				(*expiryProcessing)();
		}
		busyFlag = 0;
	}
}
