#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * TimingInterface.cc - class for timings
 */

#include "TimingInterface.h"
#include "FreeRunningTimer.h"
#include "FreeRunningTimerStar.h"
#include "Semaphore.h"


#ifdef __GNUC__
#pragma implementation
#endif

TimingInterface::TimingInterface()
{
	anotherStar = new TimingInterface( 0 );
	another = anotherStar;
}

TimingInterface::TimingInterface( int )
{
}

TimingInterface::~TimingInterface()
{
}

void
TimingInterface::nullCall()
{
}

void
TimingInterface::intParam( int )
{
}

void
TimingInterface::twoIntParams( int, int )
{
}

void
TimingInterface::ptrParam( char * )
{
}

void
TimingInterface::proxyParam( TimingInterface * )
{
}

TimingInterfaceRef
TimingInterface::returnOldProxy()
{
	return( this );
}

TimingInterfaceRef
TimingInterface::returnNewProxy()
{
	return( another );
}

TimingInterface *
TimingInterface::returnOldProxyAsterisk()
{
	return( this );
}

TimingInterface *
TimingInterface::returnNewProxyAsterisk()
{
	return( another );
}

void
TimingInterface::setOutCallInterface( TimingInterface * intf )
{
	ointfStar = intf;
	ointf = intf;
}

void
TimingInterface::setOutProxyInterface( TimingInterface * intf )
{
	yetanotherStar = intf;
	yetanother = intf;
}

unsigned int
TimingInterface::outNullCalls( unsigned ntries )
{
	unsigned int start = SystemTimer->time();
	for ( unsigned int i = 0; i < ntries; i++ )
	    ointf->nullCall();
	return( SystemTimer->time() - start );
}

unsigned int
TimingInterface::outIntParams( unsigned ntries )
{
	unsigned int start = SystemTimer->time();
	for ( unsigned int i = 0; i < ntries; i++ )
	    ointf->intParam( 0 );
	return( SystemTimer->time() - start );
}

unsigned int
TimingInterface::outTwoIntParams( unsigned ntries )
{
	unsigned int start = SystemTimer->time();
	for ( unsigned int i = 0; i < ntries; i++ )
	    ointf->twoIntParams( 0, 0 );
	return( SystemTimer->time() - start );
}

unsigned int
TimingInterface::outPtrParams( unsigned ntries )
{
	unsigned int start = SystemTimer->time();
	char c;
	for ( unsigned int i = 0; i < ntries; i++ )
	    ointf->ptrParam( &c );
	return( SystemTimer->time() - start );
}

unsigned int
TimingInterface::outProxyParams( unsigned ntries, unsigned flag )
{
	TimingInterface * param = 0;
	switch(flag)
	{
	case 0:
	    param = ointf;
	    break;
	case 1:
	    param = another;
	    break;
	case 2:
	    param = this;
	    break;
	case 3:
	    param = yetanother;
	    break;
	}
	unsigned int start = SystemTimer->time();
	for ( unsigned int i = 0; i < ntries; i++ )
	    ointf->proxyParam( param );
	return( SystemTimer->time() - start );
}

unsigned int
TimingInterface::outRefUnref( unsigned ntries )
{
	unsigned int start = SystemTimer->time();
	TimingInterfaceStar o;
	for ( unsigned int i = 0; i < ntries; i++ ) {
	    o = ointf;
	    o = 0;
	}
	return( SystemTimer->time() - start );
}

unsigned int
TimingInterface::outReturnOldProxyAsterisks( unsigned ntries )
{
	unsigned int start = SystemTimer->time();
	for ( unsigned int i = 0; i < ntries; i++ ) {
	    (void) ointf->returnOldProxyAsterisk();
	}
	return( SystemTimer->time() - start );
}

unsigned int
TimingInterface::outReturnNewProxyAsterisks( unsigned ntries )
{
	unsigned int start = SystemTimer->time();
	TimingInterface * oAsterisk;
	for ( unsigned int i = 0; i < ntries; i++ ) {
	    oAsterisk = ointf->returnNewProxyAsterisk();
	    delete oAsterisk;
	}
	return( SystemTimer->time() - start );
}

#include "ClassInstaller.h"
DEFINECLASS("TimingInterface",TimingInterface,ProxiableObject,0,0);
