#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * VersionObject.cc
 */

#include "NameServer.h"
#include "Process.h"
#include "ThisProcess.h"
#include "Assert.h"
#include "String.h"
#include "VersionInstaller.h"
#include "VersionObject.h"
#include "VersionObjectStar.h"


#ifdef __GNUC__
#pragma implementation
#endif

VersionObject::VersionObject( char * s )
{
    Assert( strlen(s) < 64 )
	( " Version string too long: %d\n", strlen(s) );

    version = new char[ strlen(s) + 1 ];
    strcpy( version, s );

}

VersionObject::~VersionObject()
{
    delete version;
}

void
VersionObject::install( ProvidedVersionInstaller * list )
{
    Process * thisProc = thisProcess();
    VersionObjectStar v;
    VersionInstaller * p;
    for (; list != 0; list = list->next ) {
	p = list->versionPtr;
	if ( p->visited )
	    continue;
	p->visited = 1;
	v = new VersionObject( p->version );
	thisProc->nameServer()->bind( p->name, v );
    }
}

void
VersionObject::check( RequiredVersionInstaller * list )
{
    Process * thisProc = thisProcess();
    VersionObjectStar v;
    VersionInstaller * p;
    for (; list != 0; list = list->next ) {
	p = list->versionPtr;
	if ( p->visited )
	    continue;
	p->visited = 1;
	v = thisProc->nameServer()->lookup( p->name, VersionObjectClass );
	if( v != 0 ) {
	    thisProc->console() << "Warning: required " << p->name <<
		    " not found\n" << eor;
	    continue;
	}
	if ( strcmp( v->version, p->version ) != 0 )
	    thisProc->console() << "Warning: required " << p->name <<
	      " " << p->version << " does not match\n" <<
	      "     provided " << p->name << " " << v->version << "\n" << eor;
    }
}

#include "ClassInstaller.h"
DEFINECLASS("VersionObject",VersionObject,ProxiableObject,0,0);
