#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "GPInclude.h"
#include "String.h"
#include "AssocTable.h"

#ifdef __GNUC__
#pragma implementation
#endif


_at_entry::_at_entry(int i, unsigned int v)
: _mht_entry(i), _copied(0)
{ _value = v;}

_at_entry::_at_entry(char* i, unsigned int v, unsigned copied)
: _mht_entry(i), _copied(copied)
{ _value = v;}

_at_entry::_at_entry(void* i, unsigned int v)
: _mht_entry(i), _copied(0)
{ _value = v;}

_at_entry::~_at_entry()
{
    if (_copied) {
	delete _string;
    }
}

unsigned int
_at_entry::value()
{
    return _value;
}

void
_at_entry::inspect()
{
    _mht_entry::inspect();
    Console() << "," << value() << eor;
}

// call a function "void f(int, void*, int)"

void
_at_list::mapcar(PFLV f)
{
    if ( _head == 0 )
	return;

    Link *first,*follow;
    first = follow = _head;
    do {
	first = first->next();
	(*((PFIVUV)f))(((_at_entry *)follow)->type(),
		       ((_at_entry *)follow)->pointer(),
		       ((_at_entry *)follow)->value());
	follow = first;
    }
    while ( first != _head );
}

 //_at_table::_at_table()
 //{
     //if (__table)
 	//delete (_at_list *)__table;
     //__table = 0;
 //}    

HintList*
_at_table::operator[]( int i )
{
    Assert((0 <= i) && ((unsigned)i < _numBuckets));
    return &((_at_list *)__table)[i];
}

AssocTable::AssocTable( int size, PFIVUV f)
: MultiHashTable(size,(PFIVV)f,1)
{
    _table = new _at_table( new _at_list[_numBuckets], _numBuckets );
    Assert(_table != 0);
}

AssocTable::~AssocTable()
{
}

// Regrettably, MHT methods cannot be used easily...

// In the interests of efficieny, string is not copied in the ctor.
// That way the temp entries in lookup etc. do not have to traverse
// the allocator.
int
AssocTable::bind( char * string,  unsigned int value )
{
    char * c = new char[strlen(string)+1];
    (void)strcpy(c,string);
    Link * e = new _at_entry(c,value,1);
    HashTable::insert(e);
    return value;
}
int
AssocTable::lookup( char * string,  unsigned int& value , int n)
{
    _at_entry e(string,value,0);
    _at_entry * r = (_at_entry *)find(&e,n);
    if (r) {
	value = r->value();
	return 1;
    }
    else
	return 0;
}
int
AssocTable::unbind( char * string,  unsigned int& value , int n)
{
    _at_entry e(string,value,0);
    _at_entry * r = (_at_entry *)HashTable::remove(&e,n);
    if (r) {
	value = r->value();
	delete r;
	return 1;
    }
    else
	return 0;
}

int
AssocTable::bind( int number,  unsigned int value )
{
    Link * e = new _at_entry(number,value);
    HashTable::insert(e);
    return value;
}
int
AssocTable::lookup( int number,  unsigned int& value , int n)
{
    _at_entry e(number,value);
    _at_entry * r = (_at_entry *)find(&e,n);
    if (r) {
	value = r->value();
	return 1;
    }
    else
	return 0;
}
int
AssocTable::unbind( int number,  unsigned int& value , int n)
{
    _at_entry e(number,value);
    _at_entry * r = (_at_entry *)HashTable::remove(&e,n);
    if (r) {
	value = r->value();
	delete r;
	return 1;
    }
    else
	return 0;
}

int
AssocTable::bind( void* pointer,  unsigned int value )
{
    Link * e = new _at_entry(pointer,value);
    HashTable::insert(e);
    return value;
}
int
AssocTable::lookup( void* pointer,  unsigned int& value , int n)
{
    _at_entry e(pointer,value);
    _at_entry * r = (_at_entry *)find(&e,n);
    if (r) {
	value = r->value();
	return 1;
    }
    else
	return 0;
}
int
AssocTable::unbind( void* pointer,  unsigned int& value , int n)
{
    _at_entry e(pointer,value);
    _at_entry * r = (_at_entry *)HashTable::remove(&e,n);
    if (r) {
	value = r->value();
	delete r;
	return 1;
    }
    else
	return 0;
}
