#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
#include "BitArray.h"

#ifdef __GNUC__
#pragma implementation
#endif


const unsigned BitsInInt = (32);
const unsigned LSB       = ((unsigned)0x1);
const unsigned MSB       = (~(~((unsigned)0x0)>>1));
const unsigned ZERO      = ((unsigned)0x0);
const unsigned FFFF      = (~((unsigned)0x0));

int
IntBitArray::get(const unsigned from, const unsigned to)
{
    unsigned m = mask(from,to);
    return ((*theInt) & m)?
	    ((m == ((*theInt) & m)) ? 1 : -1)
	    : 0;
}

/*
 * Search for the next bit of type 1 or 0 in increasing (dir== 1)
 * or decreasing (dir==1) direction. Return -1 for not found, else
 * the position.
 */
int
IntBitArray::bitSearch(unsigned pos, int type, int dir)
{
    Assert(pos < BitsInInt);
    
    if (dir) {
	if (type) {
	    for(;pos < BitsInInt;pos++)
		if ((*theInt) & mask(pos))
		    return pos;
	} else {
	    for(;pos < BitsInInt;pos++)
		if (!((*theInt) & mask(pos)))
		    return pos;
	}
    } else {
	if (type) {
	    do {
		if ((*theInt) & mask(pos))
		    return pos;
	    } while( pos-- > 0 );
	} else {
	    do {
		if (!((*theInt) & mask(pos)))
		    return pos;
	    } while( pos-- > 0 );
	}
    }
    return -1;
}

/*
 * find the contiguos extent around pos of bits that have the same
 * value as bits. Record the extent in from and to - return the value of
 * the bit itself as the return value.
 */
int
IntBitArray::getExtent(const unsigned pos, unsigned& from, unsigned& to)
{
    int type = !((*theInt) & mask(pos));
    int f, t;
    f = bitSearch(pos,type,0);
    t = bitSearch(pos,type,1);
    f = (f < 0) ? 0 : f+1;
    t = (t < 0) ? BitsInInt-1 : t-1;
    from = f;
    to   = t;
    return type;
}

BitArray::BitArray(int n)
{
    bits  = n;
    ints  = inInt(bits) + 1;
    array = new unsigned[ints];
    for( unsigned i=0; i < ints; i++)
	array[i] = ZERO;
}

void
BitArray::set()
{
    for( unsigned i=0; i < ints; i++)
	array[i] = FFFF;
}
    
void
BitArray::set(const unsigned pos)
{
    Assert(pos < bits);
    IntBitArray(array+(inInt(pos))).set(offsetInInt(pos));
}

void
BitArray::set(const unsigned from, const unsigned to)
{
    Assert(from <= to && to < bits);
    unsigned i = inInt(from);
    unsigned j = inInt(to);
    if (i == j) {
	IntBitArray(array+(i)).set(offsetInInt(from),offsetInInt(to));
	return;
    }
    IntBitArray(array+(i)).set(offsetInInt(from),BitsInInt-1);
    IntBitArray(array+(j)).set(0,offsetInInt(to));
    for (unsigned k=i+1; k < j;k++)
	array[k] = FFFF;
}

int
BitArray::get()
{
    unsigned test = array[0] ? FFFF : 0;
    for( unsigned i=0; i < ints; i++)
	if (array[i] != test)
	    return -1;
    return test ? 1 : 0 ;
}
    
int
BitArray::get(const unsigned pos)
{
    Assert(pos < bits);
    return IntBitArray(array+(inInt(pos))).get(offsetInInt(pos));
}

int
BitArray::get(const unsigned from, const unsigned to)
{
    Assert(from <= to && to < bits);
    
    unsigned i = inInt(from);
    unsigned j = inInt(to);

    if (i == j)
	return IntBitArray(array+(i)).get(offsetInInt(from),offsetInInt(to));
    
    int ret = IntBitArray(array+(i))
			.get(offsetInInt(from),BitsInInt-1);
    if (ret >= 0)
	if (ret == IntBitArray(array+(j)).get(0,offsetInInt(to))) {
	    unsigned k,test;
	    for (k=i+1, test = ret ? FFFF : ZERO ; k < j ; k++)
		if( array[k] != test)
		    return -1;
	    return ret;
	}
    return -1;
}

void
BitArray::clear()
{
    for( unsigned i=0; i < ints; i++)
	array[i] = ZERO;
}
    
void
BitArray::clear(const unsigned pos)
{
    Assert(pos < bits);
    IntBitArray(array+(inInt(pos))).clear(offsetInInt(pos));
}

void
BitArray::clear(const unsigned from, const unsigned to)
{
    Assert(from <= to && to < bits);
    unsigned i = inInt(from);
    unsigned j = inInt(to);

    if (i == j) {
	IntBitArray(array+(i)).clear(offsetInInt(from),offsetInInt(to));
	return;
    }
    IntBitArray(array+(i)).clear(offsetInInt(from),BitsInInt-1);
    IntBitArray(array+(j)).clear(0,offsetInInt(to));
    for (unsigned k=i+1; k < j;k++)
	array[k] = ZERO;
}

void
BitArray::invert()
{
    for( unsigned i=0; i < ints; i++)
	array[i] = ~array[i];
}
    
void
BitArray::invert(const unsigned pos)
{
    Assert(pos < bits);
    IntBitArray(array+(inInt(pos))).invert(offsetInInt(pos));
}

void
BitArray::invert(const unsigned from, const unsigned to)
{
    Assert(from <= to && to < bits);
    unsigned i = inInt(from);
    unsigned j = inInt(to);

    if (i == j) {
	IntBitArray(array+(i)).invert(offsetInInt(from),offsetInInt(to));
	return;
    }
    IntBitArray(array+(i)).invert(offsetInInt(from),BitsInInt-1);
    IntBitArray(array+(j)).invert(0,offsetInInt(to));
    for (unsigned k=i+1; k < j;k++)
	array[k] = ~array[k];
}

/*
 * Search for the next bit of type 1 or 0 in increasing (dir== 1)
 * or decreasing (dir==1) direction. Return -1 for not found, else
 * the position.
 */
int
BitArray::bitSearch(unsigned pos, int type, int dir)
{
    Assert(pos < bits);
    Assert(type == 0 || type == 1);
    
    // to find 1, test for ZERO
    unsigned test = type ? ZERO : FFFF ;
    
    int add, lim, ipos;
    
    if (dir) { 	// increasing bit index
	add = 1;
	lim = ints;
	ipos = 0;
    } else { 	// decreasing bit index
	add = -1;
	lim = -1;
	ipos = BitsInInt-1;
    }
    
    int i = inInt(pos);
    
    // ideally bitSearch should throw an exception if the bit isn't found
    // since we are searching for unsigned int data, but returning a -1
    // for an error; we could also to the STL thing and return a 0xFFFF
    int ret = IntBitArray(array+(i)).bitSearch(offsetInInt(pos), type, dir);

    if (ret < 0) {
	while ( (i+=add) != lim && array[i] == test )
	    ;
	if (i != lim) 
	    ret = IntBitArray(array+(i)).bitSearch(ipos,type,dir);
    }
    if (ret >= 0) {
	ret = bitpos(i,ret);
	if ((unsigned)ret >= bits) ret = -1;
    }
    
    return ret;
}

/*
 * find the contiguos extent around pos of bits that have the same
 * value as bits. Record the extent in from and to - return the value of
 * the bit itself as the return value.
 */
int
BitArray::getExtent(const unsigned pos, unsigned& from, unsigned& to)
{
    int type = !get(pos);
    int f = bitSearch(pos,type,0);
    int t = bitSearch(pos,type,1);
    from = (f < 0) ? 0 : f+1;
    to = (t < 0) ? bits-1 : t-1;
    return type;
}

void
BitArray::inspect()
{
    for( unsigned i=0; i < bits; i++ ) {
	Console() << (IntBitArray(array+(inInt(i))).get(offsetInInt(i)) ? 1 : 0);
	if (!((i+1)%BitsInInt)) Console() << "\n";
    }
}
