#ifdef __GNUC__
#pragma implementation
#endif

#include "Convert.h"
#include "Ctype.h"

#ifdef __GNUC__
#pragma implementation
#endif


char *
SkipWhite(char * a)
{
	// Avoid null pointers.
	if (a == 0)
		return((char *) 0);

	// Skip white space.
	// Return pointer to first character past white space.
	while (*a && isascii(*a) && isspace(*a))
		a++;
	return(a);
}

/*
 * this only works for bases <= 10
 */
int
atoi(char * a, int base)
{
	int i = 0;			// Value.
	int sign = 1;			// Sign of value (presume positive).

	// Avoid null pointers.
	if (a == 0)
		return(0);

	// Skip initial blanks.
	a = SkipWhite(a);

	// Handle possible initial sign.
	if (*a == '-') {		// Value is negative.
		sign = -1;
		a++;
	} else if (*a == '+') {		// Value is positive.
		a++;
	}

	// Get value.
	while (*a && isascii(*a) && isdigit(*a)) {
		i = (i * base) + (int) (*a - '0');
		a++;
	}

	// Adjust value for sign.
	i *= sign;

	// Return value.
	return(i);
}

double
atof(char * a)
{
	double value = 0.0;			// Value to return.
	double sign =  1.0;			// Sign of value.
	int exponent = 0;			// Exponent of value.

	// Avoid null pointers.
	if (a == 0)
		return(0.0);

	// Skip initial white space.
	a = SkipWhite(a);

	// Handle initial sign.
	if (*a == '-') {
		sign = -1.0;
		a++;
	} else if (*a == '+') {
		a++;
	}

	// Get value (left of decimal).
	while (*a && isascii(*a) && isdigit(*a)) {
		value = (10.0 * value) + (double) (*a - '0');
		a++;
	}

	// Get value (right of decimal).
	if (*a == '.') {
		a++;
		double d = 0.1;
		while (*a && isascii(*a) && isdigit(*a)) {
			value = value + (d * (double) (*a - '0'));
			d /= 10.0;
			a++;
		}
	}

	// Get exponent.
	if ((*a == 'e') || (*a == 'E')) {
		a++;
		exponent = atoi(a);
	}

	// Adjust value due to exponent.
	if (exponent != 0) {
		int increment = (exponent > 0) ?   -1  :  1;
		double factor = (exponent > 0) ? 10.0  : 0.1;
		while (exponent != 0) {
			exponent += increment;
			value *= factor;
		}
	}

	// Adjust value due to sign.
	value  *= sign;

	// Return value.
	return(value);
}
