#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
#include "HashTable.h"

#ifdef __GNUC__
#pragma implementation
#endif


HintList*
_ht_table::operator[]( int i )
{
    Assert((0 <= i) && ((unsigned int)i < _numBuckets));
    return &__table[i];
}

HashTable::HashTable(int size, PFLV f)
{
    _numBuckets = nextPrime(size);
    _destructFunction = f;
    _table = new _ht_table( new HintList[_numBuckets], _numBuckets );
}

HashTable::HashTable(int size, PFLV f, int derived)
{
    _numBuckets = nextPrime(size);
    _destructFunction = f;
}

HashTable::~HashTable()
{
    if (_destructFunction)
	mapcar((PFLV)_destructFunction);
    if (_table)
	delete _table;
    _table = 0;
}

unsigned
HashTable::nextPrime( unsigned from )
{
    for( int i = from; ; i++ ) {
        if ( isPrime( i ) )
            return i;
    }
}

int
HashTable::isPrime( unsigned number )
{
    for( unsigned i = 2; i < number; i++ )
        if ( divides( i, number ) )
            return 0;
    return 1;
}

int
HashTable::divides( unsigned divisor, unsigned number )
{
    int result = number / divisor;
    return divisor * result == number ? 1 : 0;
}

void
HashTable::insert( Link * entry )
{
    Assert(entry != 0);
    (*_table)[entry->hash(_numBuckets)]->addFirst(entry);
}

Link *
HashTable::find( Link * entry, int n )
{
    if (!entry) return 0;
    if (n)
	return (*_table)[entry->hash(_numBuckets)]->find(entry,n);    
    else
	return (*_table)[entry->hash(_numBuckets)]->find(entry);    
}

Link *
HashTable::remove( Link * entry, int n )
{
    if ( !entry ) return 0;
    return (*_table)[entry->hash(_numBuckets)]->remove(entry,n);    
}

void
HashTable::mapcar(PFLV f)
{
    for(unsigned i=0; i < _numBuckets; i++) {
	(*_table)[i]->mapcar(f);
    }
}

// actually, inspect could be a special case of mapcar..
void
HashTable::inspect()
{
    for(unsigned i=0; i < _numBuckets; i++) {
 	Console() << i << "\t" << eor;
	(*_table)[i]->inspect();
 	Console() << "\n" << eor;
    }
}
