#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
#include "LightHashTable.h"

#ifdef __GNUC__
#pragma implementation
#endif


int
HashVirtualiser::compare( SLink * a, SLink * b )
{
    return _t->compare(a,b);
}

void
HashVirtualiser::destroy( SLink * a )
{
    _t->deleteEntry(a);
}

void
HashVirtualiser::inspect( SLink * a )
{
    _t->inspectEntry(a);
}

SLink *
HashVirtualiser::end(SLink * l)
{
    return _t->listType(l);
}

LightList*
_lht_table::operator[]( int i )
{
    Assert(0 <= i && (unsigned)i < _numBuckets);
    return &__table[i];
}

#include "LightList.h"

LightHashTable::LightHashTable(int size, PFSV f)
{
    _numBuckets = nextPrime( size );
    _destructFunction = f;

    LightList *l = new LightList[_numBuckets];
    _table = new _lht_table( l , _numBuckets );
    Assert(_table != 0);
    _v = new HashVirtualiser(this);
}

LightHashTable::LightHashTable(int size, PFSV f, int derived)
{
    _numBuckets = nextPrime( size );
    _destructFunction = f;
    _v = new HashVirtualiser(this);
}

LightHashTable::~LightHashTable()
{
    if (_destructFunction)
	mapcar((PFSV)_destructFunction);
    if (_table)
	delete _table;
    _table = 0;
    delete _v;
}

unsigned
LightHashTable::nextPrime( unsigned from )
{
    for( int i = from; ; i++ ) {
        if ( isPrime( i ) )
            return i;
    }
}

int
LightHashTable::isPrime( unsigned number )
{
    for( unsigned i = 2; i < number; i++ )
        if ( divides( i, number ) )
            return 0;
    return 1;
}

int
LightHashTable::divides( unsigned divisor, unsigned number )
{
    int result = number / divisor;
    return divisor * result == number ? 1 : 0;
}

int
LightHashTable::hash( SLink * entry, int buckets )
{
    return ((int)entry)%buckets;
}

int
LightHashTable::compare( SLink * a, SLink * b )
{
    return a == b;
}

void
LightHashTable::deleteEntry( SLink * entry )
{
    delete entry;
}

void
LightHashTable::inspectEntry( SLink * entry )
{
    entry->inspect();
}

SLink *
LightHashTable::listType(SLink * l)
{
    // default closed lists
    return l;
}

void
LightHashTable::insert( SLink * entry )
{
    Assert(entry != 0);
    (*_table)[hash(entry,_numBuckets)]->addFirst(entry,_v);
}

SLink *
LightHashTable::find( SLink * entry, int n )
{
    if (!entry) return 0;
    if (n)
	return (*_table)[hash(entry,_numBuckets)]->find(entry,n,_v);
    else
	return (*_table)[hash(entry,_numBuckets)]->find(entry,_v);
}

SLink *
LightHashTable::remove( SLink * entry, int n )
{
    if ( !entry ) return 0;
    return (*_table)[hash(entry,_numBuckets)]->remove(entry,n,_v);    
}

void
LightHashTable::mapcar(PFSV f)
{
    for(unsigned i=0; i < _numBuckets; i++) {
	(*_table)[i]->mapcar(f);
    }
}

// actually, inspect could be a special case of mapcar..
void
LightHashTable::inspect()
{
    for(unsigned i=0; i < _numBuckets; i++) {
 	Console() << i << "\t" << eor;
	(*_table)[i]->inspect(_v);
 	Console() << "\n" << eor;
    }
}

