#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
#include "GPInclude.h"
#include "Assert.h"
#include "LightList.h"

#ifdef __GNUC__
#pragma implementation
#endif


void
LightListVirtualiser::add( LightList * l, SLink * newEntry,
			  SLink * oldEntry, int head )
{
    l->add( newEntry, oldEntry, head );
}

SLink *
LightListVirtualiser::remove( LightList * l, SLink * oldEntry,
			     int nth)
{
    return l->remove( oldEntry, nth, this );
}

SLink *
LightList::find( SLink * e, LightListVirtualiser * v )
{
    Assert( v != 0 );

    if ( e == 0 || _tail == 0 ) return 0;

    SLink* t = head();
    do 
	if (v->compare(t,e)) return t;
    while( (t = t->next()) != head() );

    return 0;
}

SLink *
LightList::find( SLink * e, int nth, LightListVirtualiser * v )
{
    Assert( v != 0 && nth >= 0 );

    if ( e == 0 || _tail == 0 ) return 0;

    SLink* t = head();
    do 
	if (v->compare(t,e))
	    if (--nth < 0)
		return t;
    while( (t = t->next()) != head() );
    
    return 0;
}
    
// oldEntry must be a pointer to an existing entry. This is not checked!!!
// If it is 0, and list already exists, you lose.
void
LightList::add( SLink * newEntry, SLink * oldEntry, int head )
{
    Assert( newEntry != 0 );
    Assert( head ? oldEntry == _tail: 1 );

    if ( oldEntry == 0 ) {
	Assert( _tail == 0 );
	_tail = newEntry;
    }
    else {
	newEntry->addAfter( oldEntry );
	if ( oldEntry == _tail && !head )
	    _tail = newEntry;

    }
}

SLink *
LightList::remove( SLink * e, int nth, LightListVirtualiser * v )
{
    Assert( e != 0 && nth >= 0 && v != 0 );
    
    if (_tail == 0 )			// no element
	return 0;
    
    SLink * t = _tail;
    
    if (t == t->next()) {		// one element
	if (nth > 0)
	    return 0;
	else {
	    if (v->compare(t,e)) {
		_tail = 0;
		return t;
	    }
	    else
		return 0;
	}
    }
    
    do 					// more than one element
	if (v->compare(t->next(),e))
	    if (--nth < 0)
		break;
    while( (t = t->next()) != _tail );
    
    if (t == _tail) 
	if (!v->compare(t->next(),e))
	    return 0;
    SLink * ret = t->next();
    if (ret == _tail)
	_tail = t;
    ret->remove(t);
    return ret;
	
}

// elements numbered from 0
SLink *
LightList::nthElem(int n)
{
    Assert( n >= 0 );

    if (!_tail)
	return 0;
    
    SLink *t;
    for (t = head(); --n >= 0 && t != _tail; t = t->next());
    if ( n >= 0 )
	return 0;
    else
	return t;
}

// sublist inclusive from start to end
int
LightList::sublist(int start, int end, SLink*& a, SLink*& b)
{
    Assert( start >= 0 && end >= start );

    b = a = nthElem(start);
    if (!a) return 0;
    if ( end > start ) {
	b = nthElem(end);
	if (!b) return 0;
    }
    return 1;
}
	

void
LightList::destroy(LightListVirtualiser * v)
{
    if ( _tail == 0 )
	return;

    SLink *first,*follow;
    first = follow = _tail;
    do {
	first = first->next();
	v->destroy(follow);
	follow = first;
    }
    while ( first != _tail );
    _tail = 0;
}

void
LightList::inspect(LightListVirtualiser * v)
{
    if ( _tail == 0 )
	return;

    SLink *first,*follow;
    first = follow = _tail;
    do {
	first = first->next();
	v->inspect(follow);
	Console() << " " << eor;
	follow = first;
    }
    while ( first != _tail );
}

void
LightList::mapcar(PFSV f)
{
    if ( _tail == 0 )
	return;

    SLink *first,*follow;
    first = follow = _tail;
    do {
	first = first->next();
	(*f)(follow);
	follow = first;
    }
    while ( first != _tail );
}
