#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: See-Mong Tan (stan@cs.uiuc.edu)
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * Link.cc:  Linked lists
 *
 *	$Header: /projects/choices/old/cvsroot/choices/Libraries/GeneralPurpose/Portable/Link.cc,v 1.1 2004/06/03 01:58:48 jcarlyle Exp $
 */
#include "Assert.h"
#include "Link.h"

#ifdef __GNUC__
#pragma implementation
#endif


SLink *
LinkVirtualiser::end( SLink * l )
{
    // default is closedList, so end is starting link
    return l;
}

int
LinkVirtualiser::compare( SLink * a, SLink * b )
{
    return a == b;
}

void
LinkVirtualiser::destroy( SLink * a )
{
    delete a;
}

void
LinkVirtualiser::inspect( SLink * a )
{
    a->inspect();
}

int
LinkVirtualiser::hash( SLink * a, int buckets )
{
    return ((int)a)%buckets;
}

void
NVLinkVirtualiser::destroy( SLink * a )
{
    delete (NVLink*)a;
}

void
NVLinkVirtualiser::inspect( SLink * a )
{
    ((NVLink*)a)->inspect();
}

void
SLink::inspect()
{
     Console() << (unsigned)next() << eor;
}

void
NVLink::addBefore( NVLink * list )
{
    // point to previous object
    setPrevious( list->previous() );
    setNext( list );

    // set previous object and list to point to me
    list->previous()->setNext( this );
    list->setPrevious( this );
}

void
NVLink::addAfter( NVLink * list )
{
    // point to next object
    setNext( list->next() );
    setPrevious( list );
    
    // set next object and list to point to me
    list->next()->setPrevious( this );
    list->setNext( this );
}

void
NVLink::remove()
{
    previous()->setNext( next() );
    next()->setPrevious( previous() );
    setNext( this );
    setPrevious( this );
}

void
NVLink::inspect()
{
    Console() << (const char *)previous() << "," << (const char *)next() << eor;
}

Link::Link()
{
}

Link::~Link()
{
}

void
Link::inspect()
{
    NVLink::inspect();
}

int
Link::compare( Link * l )
{
    return (this == l);
}

int
Link::hash(int buckets)
{
    return ((int)this)%buckets;
}

/*
 * This whole kaboodle (below) not tested
 */

/*
 * complicated functions:
 *	Links as lists
 *	See the OpenLinkList class to use links as lists.
 *	It does a very special kind of list management -- a link subclass
 *	which can tell whether it is head or tail is used so that any element
 *	can be removed knowing the element itself
 *      Alternatively, you can provide your own management
 *
 *	Someday, I will make the standard linked lists use functions from
 *	here.
 *		Aamod
 */

SLink *
SLink::find( SLink * e, LinkVirtualiser * v )
{
    Assert( e != 0 );

    if ( v == 0 )
	v = new LinkVirtualiser;

    SLink* t = this;
    do 
	if (v->compare(t,e)) return t;
    while( (t = t->next()) != v->end(this) );

    return 0;
}

SLink *
SLink::find( SLink * e, int nth, LinkVirtualiser * v )
{
    Assert( e != 0 && nth >= 0 );

    if ( v == 0 )
	v = new LinkVirtualiser;

    SLink* t = this;
    do 
	if (v->compare(t,e))
	    if (--nth < 0)
		return t;
    while( (t = t->next()) != v->end(this) );
    
    return 0;
}

// elements numbered from 0
SLink *
SLink::nthElem(int n, LinkVirtualiser * v)
{
    Assert( n >= 0 );

    SLink *t;
    for ( t = this; --n >= 0 && t != v->end(this); t = t->next());
    if ( n >= 0 )
	return 0;
    else
	return t;
}
    
int
SLink::sublist(int start, int fin, SLink*& a, SLink*& b,
	       LinkVirtualiser * v)
{
    Assert( start >= 0 && fin >= start );

    a = nthElem((start ? start-1 : start),v);
    if (!a) return 0;
    if ( fin > start ) {
	b = nthElem(fin,v);
	if ( !b) return 0;
    }
    return 1;
}
	

void
SLink::destroyAll(LinkVirtualiser * v)
{
    SLink *first,*follow;
    first = follow = this;
    do {
	first = first->next();
	v->destroy(follow);
	follow = first;
    }
    while ( first != v->end(this) );
}

void
SLink::inspectAll(LinkVirtualiser * v)
{
    for (SLink * t = this; t != v->end(this); t = t->next()) {
	v->inspect(t);
 	Console() << " " << eor;
    }
}

/*
 * Exactly the same code for Link except for the virtualisers
 */

Link *
Link::end( Link * l )
{
    // default circular
    return l;
}

Link *
Link::find( Link * e )
{
    Assert( e != 0 );

    Link* t = this;
    do 
	if (t->compare(e)) return t;
    while( (t = t->next()) != end(this) );

    return 0;
}

Link *
Link::find( Link * e, int nth )
{
    Assert( e != 0 && nth >= 0 );

    Link* t = this;
    do 
	if (t->compare(e))
	    if (--nth < 0)
		return t;
    while( (t = t->next()) != end(this) );
    
    return 0;
}

// elements numbered from 0
Link *
Link::nthElem(int n )
{
    Assert( n >= 0 );

    Link *t;
    for (t = this; --n >= 0 && t != end(this); t = t->next());
    if ( n >= 0 )
	return 0;
    else
	return t;
}
    
int
Link::sublist(int start, int fin, Link*& a, Link*& b)
{
    Assert( start >= 0 && fin >= start );

    a = nthElem(start ? start-1 : start);
    if (!a) return 0;
    if ( fin > start ) {
	b = nthElem(fin);
	if ( !b) return 0;
    }
    return 1;
}
	

void
Link::destroyAll()
{
    Link *first,*follow;
    first = follow = this;
    do {
	first = first->next();
	delete follow;
	follow = first;
    }
    while ( first != end(this) );
}

void
Link::inspectAll()
{
    for (Link * t = this; t != end(this); t = t->next()) {
	inspect();
 	Console() << " " << eor;
    }
}
