#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * List.cc - List varieties
 *
 *	$Header: /projects/choices/old/cvsroot/choices/Libraries/GeneralPurpose/Portable/List.cc,v 1.1 2004/06/03 01:58:48 jcarlyle Exp $
 */

#include "GPInclude.h"
#include "Assert.h"
#include "List.h"


#ifdef __GNUC__
#pragma implementation
#endif

// List Definitions

Link *
List::find( Link * e )
{
    Assert( e != 0 );

    if ( start() == 0 ) return 0;

    Link* t = start();
    do 
	if (t->compare(e)) return t;
    while( (t = t->next()) != start() );

    return 0;
}

// nth starts at zero
Link *
List::find( Link * e, int nth )
{
    Assert( e != 0 && nth >= 0 );

    if ( start() == 0 ) return 0;

    Link* t = start();
    do 
	if (t->compare(e))
	    if (--nth < 0)
		return t;
    while( (t = t->next()) != start() );

    return 0;
}

/*
 * NOTE - oldEntry IS EXPECTED TO BE A POINTER TO AN EXISTING ENTRY
 * Use find to get the required entry if neccessary
 */

void
List::add( Link * newEntry, Link * oldEntry, int before )
{
    Assert( newEntry != 0 );
    
    if ( oldEntry == 0 ) {
	Assert( _head == 0 );
	_head = newEntry;
	return;
    }
    if (before) {
	newEntry->addBefore(oldEntry);
	if (_head == oldEntry)
	    _head = newEntry;
	Assert( _head->next() == oldEntry );
    }
    else
	newEntry->addAfter(oldEntry);
}

Link *
List::remove( Link * entry, int nth )
{
    Link * _found;
    Assert( entry != 0 );
    if (_head == 0 )
	return 0;
    else {
	_found = nth ? find(entry,nth) : find(entry);
	if (_found) {
	    if ( _found == _head )
		_head = _head->next();
	    _found->remove();
	    if ( _found == _head )
		_head = 0;
	}
	return _found;
    }
}

Link *
List::removeFirst()
{
    Link * ret = _head;
    if (ret) {
	_head = _head->next();
	if (ret == _head)
	    _head = 0;
	else
	    ret->remove();
    }
    return ret;
}

Link *
List::removeLast()
{
    Link * ret = _head;
    if (ret) {
	if (_head == _head->next())
	    _head = 0;
	else
	    (ret = _head->previous())->remove();
    }
    return ret;
}

void
List::destroy()
{
    if ( _head == 0 )
	return;

    Link *first,*follow;
    first = follow = _head;
    do {
	first = first->next();
	delete follow;
	follow = first;
    }
    while ( first != _head );
    _head = 0;
}

void
List::mapcar(PFLV f)
{
    if ( _head == 0 )
	return;

    Link *first,*follow;
    first = follow = _head;
    do {
	first = first->next();
	(*f)(follow);
	follow = first;
    }
    while ( first != _head );
}

void
List::inspect()
{
    if ( _head == 0 )
	return;

    Link *first,*follow;
    first = follow = _head;
    do {
	first = first->next();
	if ( follow == start() ) 
	    Console() << "<" << eor;
	follow->inspect();
	if ( follow == start() ) 
	    Console() << ">" << eor;
 	Console() << " " << eor;
	follow = first;
    } 
    while ( first != _head );
}

void
CountingList::add( Link * newEntry, Link * oldEntry, int before )
{
    _numItems++;
    List::add(newEntry,oldEntry,before);
}
    
Link *
CountingList::remove( Link * oldEntry, int nth )
{
    Link * tmp = List::remove(oldEntry, nth);
    if (tmp) // if the remove succeeded..
	_numItems--;
    return tmp;
}

void
CountingList::inspect()
{
    Console() << "[" << _numItems << "] " << eor;
    List::inspect();
}

Link *
HintList::find( Link * e )
{
    Link * tmp = CountingList::find(e);
    if (tmp)
	_hint = tmp;
    return tmp;
}

Link *
HintList::find( Link * e, int nth )
{
    Link * tmp = CountingList::find(e, nth);
    if (tmp)
	_hint = tmp;
    return tmp;
}

Link *
HintList::remove( Link * oldEntry, int nth )
{
    Link * tmp = CountingList::remove( oldEntry, nth );
    if (tmp)
	_hint = _head;
    return tmp;
}

void
HintList::add( Link * newEntry, Link * oldEntry, int before )
{
    CountingList::add(newEntry,oldEntry,before);
    if (numItems() == 1)
	_hint = _head;
}
