#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#ifdef CHOICES
#include "String.h"
#else
extern "C" {
    int strcmp(const char *, const char *);
    char * strcpy(char *, const char *);
    int strlen(const char *);
}
#endif
#include "MultiHashTable.h"

#ifdef __GNUC__
#pragma implementation
#endif


int 
_mht_entry::hash(int buckets)
{
    switch( _type ) {
      case EntryNumber:
	return number()%buckets;
      case EntryString: {
	  unsigned int key = 0;
	  char * s = string();
	  while (*s != 0) {
	      key <<= 1;
	      key += *s++;
	  }
	  
	  return key % buckets;
      }
      case EntryPointer:
	return ((int)pointer())%buckets;
      default:
	Assert( NOTREACHED );
	return -1; // shut up warning;
    }
}

int
_mht_entry::compare(Link* b)
{
    if (_type != ((_mht_entry *)b)->type())
	return 0;

    switch( _type ) {
      case EntryNumber:
	return number() == ((_mht_entry*)b)->number();
      case EntryString: 
	return (strcmp(string(),((_mht_entry*)b)->string()) == 0);
      case EntryPointer:
	return pointer() == ((_mht_entry*)b)->pointer();

      default:
	Assert( NOTREACHED );
	return -1; // shut up warning;
    }
}

void
_mht_entry::inspect()
{
    switch( _type ) {
      case EntryNumber:
 	Console() << number() << eor;
	break;
      case EntryString: 
 	Console() << string() << eor;
	break;
      case EntryPointer:
 	Console() << pointer() << eor;
	break;
      default:
	Assert( NOTREACHED );
    }
}

// call a function "void f(int, void*)"

void
_mht_list::mapcar(PFLV f)
{
    if ( _head == 0 )
	return;

    Link *first,*follow;
    first = follow = _head;
    do {
	first = first->next();
	(*((PFIVV)f))(((_mht_entry *)follow)->type(),
		      ((_mht_entry *)follow)->pointer());
	follow = first;
    }
    while ( first != _head );
}

HintList*
_mht_table::operator[]( int i )
{
    Assert((0 <= i) && ((unsigned)i < _numBuckets));
    return &((_mht_list *)__table)[i];
}

MultiHashTable::MultiHashTable(int size, PFIVV f)
: HashTable(size,(PFLV)f,1)
{
    _table = new _mht_table( new _mht_list[_numBuckets], _numBuckets );
    Assert(_table != 0);
}

MultiHashTable::MultiHashTable(int size, PFIVV f, int derived)
: HashTable(size,(PFLV)f,derived)
{
}

MultiHashTable::~MultiHashTable()
{
}

int
MultiHashTable::isIn(_mht_entry * e, int n)
{
    _mht_entry * found = (_mht_entry*)find(e,n);
    if (found) {
	Assert( e->compare(found) );
	return 1;
    }
    else
	return 0;
}

// insert, remove and isIn for numbers
// The whole trick is storage management. Regretfully, in this version
// there are no checks to see if new succeeded; maybe later.

void
MultiHashTable::insert(int e)
{
    Link * _e = new _mht_entry(e);
    HashTable::insert(_e);
}
int
MultiHashTable::remove(int e, int n)
{
    _mht_entry _e(e);
    _mht_entry * r = (_mht_entry*)HashTable::remove(&_e,n);
    if (!r) return 0;
    delete r;
    return 1;
}
int
MultiHashTable::isIn(int e, int n)
{
    _mht_entry _e(e);
    return isIn(&_e,n);
}

// insert, remove and isIn for strings

void
MultiHashTable::insert(char* e)
{
    // Make your own copy of string
    
    char * c = new char[strlen(e)+1];
    (void)strcpy(c,e);
    Link * _e = new _mht_entry(c);
    HashTable::insert(_e);
}
int
MultiHashTable::remove(char* e, int n)
{
    _mht_entry _e(e);
    _mht_entry * r = (_mht_entry*)HashTable::remove(&_e,n);
    if (!r) return 0;
    
    // Delete, since you have copied the string
    
    delete r->string();
    delete r;
    return 1;
}
int
MultiHashTable::isIn(char* e, int n)
{
    _mht_entry _e(e);
    return isIn(&_e,n);
}

// insert, remove and isIn for pointers 

void
MultiHashTable::insert(void* e)
{
    Link * _e = new _mht_entry(e);
    HashTable::insert(_e);
}
void*
MultiHashTable::remove(void* e, int n)
{
    _mht_entry _e(e);
    _mht_entry * r = (_mht_entry*)HashTable::remove(&_e,n);
    if (!r) return 0;
    void * ret = r->pointer();
    delete r;
    return ret;
}
int
MultiHashTable::isIn(void* e, int n)
{
    _mht_entry _e(e);
    return isIn(&_e,n);
}
