#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "Assert.h"
#include "MultiLevelTable.h"


#ifdef __GNUC__
#pragma implementation
#endif

MultiLevelTable::MultiLevelTable( int log2ChunkSize, int log2IndexSize )
        : _log2ChunkSize( log2ChunkSize ), _log2IndexSize( log2IndexSize )
{
        if ( _log2ChunkSize == 0 )
	        _log2ChunkSize = 6;
        if ( _log2IndexSize == 0 )
	        _log2IndexSize = _log2ChunkSize - 1;

	_chunkSize = 1 << _log2ChunkSize;
	_chunkMask = _chunkSize - 1;
	_indexSize = 1 << _log2IndexSize;
	_indexMask = _indexSize - 1;

	_numberOfLevels = 1;
	_root = ( int ** ) newChunk();
	_shiftCount = 0;
	_maxForLevel = 1 << _log2ChunkSize;
}

MultiLevelTable::~MultiLevelTable()
{
        deleteLevel( _root, 1 );
}

int * MultiLevelTable::newChunk()
{
	int * array = new int[ _chunkSize ];
	Assert( array != 0 );
	for ( int i = 0; i < _chunkSize; i++ ) 
		array[ i ] = 0;
	return array;
}

int ** MultiLevelTable::newIndex()
{
	int ** array = new int * [ _indexSize ];
	Assert( array != 0 );
	for ( int i = 0; i < _indexSize; i++ ) 
		array[ i ] = 0;
	return array;  
}

void MultiLevelTable::deleteLevel( int ** index, int level )
{
        if ( level < _numberOfLevels ) {
	        int nextLevel = level + 1;
		for ( int i = 0; i < _indexSize; i++ ) {
		        if ( index[ i ] != 0 )
			        deleteLevel( (int **) index[ i ], nextLevel );
		}
	}
        delete index;
}

void MultiLevelTable::addLevel()
{
        int ** oldRoot = _root;
	_root = newIndex();
	_root[ 0 ] = ( int * ) oldRoot;
	_numberOfLevels++;
	_maxForLevel <<= _log2IndexSize;
	if ( _numberOfLevels == 2 )
	        _shiftCount += _log2ChunkSize;
	else
	        _shiftCount += _log2IndexSize;
}

int * MultiLevelTable::where( int i, int alloc )
{
	while ( i >= _maxForLevel ) {
	        if ( !alloc ) 
                        return 0;
		addLevel();
	}
	
	int ** index = _root;
	int ** addr;

	int offset;
	int shiftCount = _shiftCount;

        for( int level = 1; level < _numberOfLevels; level++ ) {
	        offset = ( i >> shiftCount ) & _indexMask;
		addr = index + offset; 
		if ( *addr == 0 ) {
  		        if ( !alloc )
		                return 0;
			if ( level == ( _numberOfLevels - 1 ) )
			        *addr = newChunk();
			else
			        *addr = ( int * ) newIndex();
		}
		index = ( int ** ) *addr;
		shiftCount -= _log2IndexSize;
	 }

	 int * chunk = ( int * ) index;
	 return chunk + ( i & _chunkMask );
}

BitsMultiLevelTable::BitsMultiLevelTable( int log2EntrySize, 
					  int log2ChunkSize_,
					  int log2IndexSize_ )
        : MultiLevelTable( log2ChunkSize_, log2IndexSize_ ),
          _log2EntrySize( log2EntrySize )
{
	int log2WordSize = 0;
	while( 1 << log2WordSize != sizeof( int ) << 3 )
	        log2WordSize++;
	  
	_log2EntriesPerWord = log2WordSize - _log2EntrySize;
        Assert( _log2EntriesPerWord > 0 );
	_log2EntriesPerChunk = log2ChunkSize() + _log2EntriesPerWord;
	
        _entryMask = ( 1 << ( 1 << _log2EntrySize ) ) - 1;
	_entriesPerWordMask = ( 1 << _log2EntriesPerWord ) - 1;
	_entriesPerChunkMask = ( 1 << _log2EntriesPerChunk ) - 1;
}


