#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * NVList.cc - List varieties
 *
 *	$Header: /projects/choices/old/cvsroot/choices/Libraries/GeneralPurpose/Portable/NVList.cc,v 1.1 2004/06/03 01:58:48 jcarlyle Exp $
 */

#include "GPInclude.h"
#include "Assert.h"
#include "NVList.h"


#ifdef __GNUC__
#pragma implementation
#endif

void
NVListVirtualiser::destroy( SLink* l )
{
    delete ((NVLink *)l);
}

void
NVListVirtualiser::inspect( SLink* l )
{
    ((NVLink *)l)->inspect();
}

void
NVListVirtualiser::add( LightList * l, SLink * newEntry,
			  SLink * oldEntry, int head )
{
    ((NVList *)l)->add( newEntry, oldEntry, head );
}

SLink *
NVListVirtualiser::remove( LightList * l, SLink * oldEntry,
			     int nth )
{
    return ((NVList *)l)->remove( oldEntry, nth, this );
}

// oldEntry must be a pointer to an existing Entry. This is not checked!!
void
NVList::add( SLink * newEntry, SLink * oldEntry, int head )
{
    // add after old entry, so set oldEntry->next()
    if (oldEntry) {
	((NVLink*)oldEntry->next())->setPrevious( ((NVLink *)newEntry) );
	((NVLink*)newEntry)->setPrevious( ((NVLink*)oldEntry) );
    }
    // set nexts as per singly linked list
    LightList::add( newEntry, oldEntry, head );
}

// Reuse by copying - same as SLink function, except for the remove
// which is for double Link
SLink *
NVList::remove( SLink * e, int nth, NVListVirtualiser * v )
{
    Assert( e != 0 && nth >= 0 && v != 0 );
    
    if (_tail == 0 )			// no element
	return 0;
    
    SLink * t = _tail;
    
    if (t == t->next()) {		// one element
	if (nth > 0)
	    return 0;
	else {
	    if (v->compare(t,e)) {
		_tail = 0;
		return t;
	    }
	    else
		return 0;
	}
    }
    
    do 					// more than one element
	if (v->compare(t->next(),e))
	    if (--nth < 0)
		break;
    while( (t = t->next()) != _tail );
    
    if (t == _tail) 
	if (!v->compare(t->next(),e))
	    return 0;
    SLink * ret = t->next();
    if (ret == _tail)
	_tail = t;
    ((NVLink *)ret)->remove();
    return ret;
}
