#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Lee Lup Yuen (lup@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
 
#include "../../../../Includes/Lock.h"
#include "OutputStream.h"
#include "Math.h"

#ifdef __GNUC__
#pragma implementation
#endif


#define MAXLONG 0x7fffffffL
char domainMsg [] = "Argument domain error";
char singMsg [] = "Argument singularity error";
char overflowMsg [] = "Overflow range error";
char underflowMsg [] = "Underflow range error";
char tlossMsg [] = "Total loss of significance";
char plossMsg [] = "Partial loss of significance";
char unknownMsg [] = "Unknown error";

int mtherr (char *str, int i)
{
    char *msg;
    switch (i)
    {
        case DOMAIN: msg = domainMsg; break;
        case SING: msg = singMsg; break;
	case OVERFLOW: msg = overflowMsg; break;
	case UNDERFLOW: msg = underflowMsg; break;
	case TLOSS: msg = tlossMsg; break;
        case PLOSS: msg = plossMsg; break; 
        default: msg = unknownMsg;
    }
    StandardOutput->formattedWrite ("mtherr: %s - %s\n", str, msg);
    return 0;
}


int abs (int x)
{
    if (x < 0)
        return -x;
    else
    	return x;
}


double floor (double x)
{
    if (x >= 0)
    {
        if (x > MAXLONG)
	    return x;  /*  Loss of precision here.  */
	else
	    return (long) x + 0.0;
    }  
    else
    {
        if (x < -MAXLONG)
	    return x;  /*  Loss of precision here.  */
	else
	    return (long) x - 1.0;
    }
}


double ceil (double x)
{
    if (x >= 0)
    {
        if (x > MAXLONG)
	    return x;  /*  Loss of precision here.  */
	else
        {
	    long c = (long) x;
	    if (c == x)
	        return c + 0.0;
	    else
	        return c + 1.0;
	}    
    }  
    else
    {
        if (x < -MAXLONG)
	    return x;  /*  Loss of precision here.  */
	else
	    return (long) x + 0.0;
    }
}


double fabs (double x)
{
    if (x < 0) return -x;
    else return x;
}


double modf (double x, double *i)
{
    if (x >= 0)
    {
        if (x > MAXLONG)
	{
	    *i = x;  /*  Loss of precision here.  */
            return 0.0;
	}    
	else
	{
	    *i = (long) x;
	    return x - *i;
	}  
    }
    else  
    {
        if (x < -MAXLONG)
	{
	    *i = x;  /*  Loss of precision here.  */
            return 0.0;
	}    
	else
	{
	    *i = (long) x;
	    return x - *i;
	}  
    }
}

