/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * Allocator.cc: implementation of a Big Bag-o-Pages heap class.
 * Much of this copied from MadAllocator in the kernel
 *
 */

#include "Semaphore.h"
#include "Allocator.h"

/*Allocator::Allocator()
{
}

Allocator::~Allocator()
{
}

void *
Allocator::allocate( unsigned )
{
	return( 0 );
}

void
Allocator::free( void * )
{

}*/


static const int NumberOBuckets = 32;

BucketAllocator::BucketAllocator( void * baseAddress, int numberOfPages,
				  int log2PageSize, int noMultiProcessing )
{
	/*
	 *	FIX: _sizeToBucket should be static class data.
	 */
	_sizeToBuckets[0] = 11;
	int element = 1;
	for( int bit = 0; bit < 10; bit++ ) {
		for( int i = 0; i < ( 1 << bit ); i++ ) {
			_sizeToBuckets[element++] = bit + 12;
		}
	}

	_sizeToBucket[0] = 2;
	_sizeToBucket[1] = 2;
	_sizeToBucket[2] = 2;
	element = 3;
	for( int bit = 1; bit < 12; bit++ ) {
		for( int i = 0; i < ( 1 << bit ); i++ ) {
			_sizeToBucket[element++] = bit + 1;
		}
	}

	//Assert( numberOfPages > 4 );
	//Assert( log2PageSize >= 8 );
	//Assert( log2PageSize <= 16 );

	/*
	 * Take space out for the _pageToBucket map.
	 */
	_pageToBucket = (char *) baseAddress;
	baseAddress = (void *) ((char **) baseAddress + numberOfPages);
	numberOfPages -= ( (numberOfPages * sizeof(char *)) >> log2PageSize )
							+ 1;

	/*
	 *	Set up instance variables based on constructor parameters.
	 *	Make sure the base address for allocations is page-aligned.
	 *	If re-alignment is needed, then we lose one page.
	 */
	_log2PageSize = log2PageSize;
	_pageSize = 1 << _log2PageSize;
	_pageMask = _pageSize - 1;
	if( (int) baseAddress & _pageMask ) {
		_numberOfPages = numberOfPages - 1;
		int alignedAddress = (int)baseAddress + _pageMask;
		_baseAddress = (char *)( alignedAddress & ~_pageMask );
	}
	else {
		_numberOfPages = numberOfPages;
		_baseAddress = (char *) baseAddress;
	}
	_upperBoundAddress = (char *) ((int) _baseAddress +
		(_numberOfPages * _pageSize));

	/*
	 *	Check to make sure aligment worked and
	 *	_baseAddress address is non-null.
	 */
	//Assert( ( (int) _baseAddress & _pageMask ) == 0 );
	//Assert( _baseAddress != 0 );
	//Assert( _pageToBucket != 0 );

	/*
	 *	Initialize the buckets to page array,
	 *	set to zeros since no bucket points to a page with free memory.
	 */
	for( int i = 0; i < 16; i++ ) {
		_bucketToPage[i] = 0;
		_bucketMutex[i] = 0;
	}

	/*
	 *	Initialize the page information array,
	 *	set to zeros since no pages are in use.
	 */
	for( int i = 0; i < _numberOfPages; i++ ) _pageToBucket[i] = 0;

	_top = 1; // waste a page cause I am lazy and to be safe
	_topMutex = 0;

	if ( !noMultiProcessing )
		enableMultiProcessing();
}

/*
 *	Nothing to do in the destructor.
 */
BucketAllocator::~BucketAllocator()
{
	//Assert( NOTREACHED );
}

void
BucketAllocator::enableMultiProcessing()
{
	for( int i = 0; i < NumberOBuckets; i++ ) {
		_bucketMutex[i] = new Semaphore( 1,1 );
	}
	_topMutex = new Semaphore( 1,1 );
}

/*
 *	findBucket quickly rounds nbytes up to the nearest power
 *	of 2, which is the bucket number.  A request for zero
 *	bytes is basically ignored.
 */
inline unsigned int
BucketAllocator::findBucket( unsigned int size )
{
	unsigned int bucket;
	if( size <= 0x1000 ) {
		bucket = _sizeToBucket[size];
	}
	else {
		if( size < 0x400000 )	bucket = _sizeToBuckets[size>>12];
		else			bucket = _sizeToBuckets[size>>22] + 10;

		/* Adjust if size is not a power of two */
		if( size & (size - 1) )	bucket++;
	}
	return( bucket );
}

/*
 *	allocate	given the size of a desired object,
 *			return a pointer to a block of memory whose
 *			size is the smallest power of 2 greater than or equal
 *			to nbytes and whose allignment is either on a page
 *			boundary or on a multiple of the region size.
 *
 *			return 0 if either the requestor asked for a 0 byte
 *			block-o-memory or we are out-o-memory.
 *
 */
void *
BucketAllocator::allocate( unsigned int nbytes )
{
	char *	returnValue;

	int bucket = findBucket( nbytes );

	//Assert( bucket > 1 );
	//Assert( bucket < NumberOBuckets );

	/*
	 *	Get the appropriate bucket descriptor out of the
	 *	_bucket[i].toPage table.  A null descriptor means there is
	 *	currently no free space for this bucket.
	 */
	Semaphore *mutex = _bucketMutex[bucket];
	if ( mutex != 0 )
		mutex->P();

	if( ( returnValue = _bucketToPage[bucket] ) == 0 ) {
		returnValue = moreSpace( bucket );
	}
	else {
		_bucketToPage[bucket] = *(char **)returnValue;
		//Assert( (_bucketToPage[bucket] == 0) ||
		//	(_bucketToPage[bucket] >= _baseAddress) );
		//Assert( _bucketToPage[bucket] < _upperBoundAddress );
	}

	if ( mutex != 0 )
		mutex->V();

	//if ( (void *)returnValue == CatchBlock )
	//	CaughtBlock();

	return( (void *)returnValue );
}

char *
BucketAllocator::moreSpace( int bucket )
{
	//Assert( bucket > 1 );
	//Assert( bucket < NumberOBuckets );

	int pages = 1;
	if( bucket > _log2PageSize ) pages = 1 << ( bucket - _log2PageSize );

	Semaphore *mutex = _topMutex;
	if ( mutex != 0 )
		mutex->P();

	int oldTop = _top;

	if( _top + pages > _numberOfPages ) {
		//StandardOutput->printf(
		//	"%N::moreSpace out of free pages.\n", this );
		//Assert( NOTREACHED );
		return( 0 );
	}

	_top += pages + 1;	// FIX: The + 1 puts unmapped wholes in heap

	if ( mutex != 0 )
		mutex->V();

	_pageToBucket[oldTop] = bucket;

	int units;
	if( bucket < _log2PageSize ) units = _pageSize >> bucket;
	else {
		units = 1;
		for( int i = 1; i < pages; i++ ) _pageToBucket[oldTop + i] = 1;
	}

	char * head = &_baseAddress[oldTop<<_log2PageSize];
	//Assert( head >= _baseAddress );
	//Assert( head < _upperBoundAddress );

	char * tail = head;
	int inc = 1 << bucket;
	for( ; units > 1; units-- ) {
		//Assert( tail >= _baseAddress );
		//Assert( tail < _upperBoundAddress );
		*(int *)tail = (int)tail + inc;
		tail += inc;
	}

	*(char **)tail = _bucketToPage[bucket];
	_bucketToPage[bucket] = *(char **)head;
	//Assert( ( _bucketToPage[bucket] == 0 ) ||
	//	( _bucketToPage[bucket] >= _baseAddress ) );
	//Assert( _bucketToPage[bucket] < _upperBoundAddress );
	
	return( head );
}

/*
 *	Given a pointer to a block of memory, return that memory to the
 *	heap.  Perform various checks to make sure the pointer is in
 *	the heap's range, has a valid alignment, and isn't returned 
 *	multiple times between allocations.
 *	(Not all checks performed all the time).
 *	Freeing address 0 is legal and a no-op.
 */
void
BucketAllocator::free( void * vp )
{
	//if ( vp == CatchBlock )
	//	CaughtBlock();

	char * cp = (char *)vp;
	//if( ( cp != 0 ) && ( cp < _baseAddress ) ) {
		//StandardOutput->printf(
			//"%N::free( %A ) Object below heap base: %A\n",
				//this, cp, _baseAddress );
	//}

	/*
	 *	Nothing to do if passed a null or an out-of-range pointer .
	 */
	if( cp < _baseAddress ) return;

	/*
	 * 	Calculate the following:
	 *		1 ) pointer offset from base address
	 *		2 ) page number relative to address
	 *		3 ) offset from beginning of page
	 *	Make sure the pointer obeys the following: 
	 *		1 ) pointer is within managed region of memory
	 *		2 ) page number is within range
	 *		3 ) this page is in use
	 *		4 ) offset has the proper alignment for this page
	 */
	int offset = cp - _baseAddress;

	//Assert( ( offset >> _log2PageSize ) < _numberOfPages );

	int bucket = _pageToBucket[offset >> _log2PageSize];
	//Assert( bucket > 1 );
	//Assert( bucket < NumberOBuckets );

	int pageOffset = (int)cp & _pageMask;
	//Assert( ! ( ( ( 1 << bucket ) - 1 ) & pageOffset ) );

	Semaphore *mutex = _bucketMutex[bucket];
	if ( mutex != 0 )
		mutex->P();

	*(char **)cp = _bucketToPage[bucket];
	_bucketToPage[bucket] = cp;
	//Assert( _bucketToPage[bucket] >= _baseAddress );
	//Assert( _bucketToPage[bucket] < _upperBoundAddress );

	if ( mutex != 0 )
		mutex->V();
} 

BucketAllocator * _new_allocator;
new_handler _current_handler;
