#include "ClassConstructor.h"
#include "NameServer.h"
#include "ProxiableObject.h"

#include "Class.h"
#include "exception.h"

Class::Class(void* k) :ProxiableObject((void*)0) {kernelObj=k;}

Class::Class(Class* parent, int size, char* name, ClassConstructor* ctor) :ProxiableObject((void*)0)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(parent)+sizeof(size)+sizeof(name)+sizeof(ctor)];
  *(Class**)(paramBuffer) = (Class*)(parent->kernelObj);
  *(int*)(paramBuffer+sizeof(parent)) = size;
  *(char**)(paramBuffer+sizeof(parent)+sizeof(size)) = name;
  *(ClassConstructor**)(paramBuffer+sizeof(parent)+sizeof(size)+sizeof(name)) = (ClassConstructor*)(ctor->kernelObj);

  asm volatile (
		"mov r0, #0\n\t"
		"mov r2, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (paramBuffer), "r"(&kernelObj) : "r0", "r1", "r2", "r3" 
		);
  kernel_exception::throw_exception(code);
  
}

void Class::cleanup()
{
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #2\n\t"
		"mov r1, %1\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

char* Class::name(int arg1, char* arg2)
{
  char* ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)+sizeof(arg2)];
  *(int*)(paramBuffer) = arg1;
  *(char**)(paramBuffer+sizeof(arg1)) = arg2;

  asm volatile ("mov r0, #3\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

Class* Class::parent()
{
  Class* ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #4\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new Class( (void*)ret );

  return ret;
}

Class* Class::child()
{
  Class* ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #5\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new Class( (void*)ret );

  return ret;
}

Class* Class::sibling()
{
  Class* ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #6\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new Class( (void*)ret );

  return ret;
}

Class* Class::real()
{
  Class* ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #7\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new Class( (void*)ret );

  return ret;
}

int Class::isASubclassOf(Class* arg1)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(Class**)(paramBuffer) = (Class*)(arg1->kernelObj);

  asm volatile ("mov r0, #8\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int Class::isClass(Class* arg1)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(Class**)(paramBuffer) = (Class*)(arg1->kernelObj);

  asm volatile ("mov r0, #9\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int Class::sizeOf()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #10\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

void Class::bind(NameServer* arg1)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(NameServer**)(paramBuffer) = (NameServer*)(arg1->kernelObj);

  asm volatile ("mov r0, #11\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void Class::unbind(NameServer* arg1)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(NameServer**)(paramBuffer) = (NameServer*)(arg1->kernelObj);

  asm volatile ("mov r0, #12\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

int Class::initDebugObjects()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #13\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int Class::initDebugMask()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #14\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int Class::initDebugMessages()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #15\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

void Class::setInitDebugObjects(int arg1)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(int*)(paramBuffer) = arg1;

  asm volatile ("mov r0, #16\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void Class::setInitDebugMask(int arg1)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(int*)(paramBuffer) = arg1;

  asm volatile ("mov r0, #17\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void Class::setInitDebugMessages(int arg1)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(int*)(paramBuffer) = arg1;

  asm volatile ("mov r0, #18\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

int Class::concrete()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #19\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int Class::debugMembers(int mask, int messages)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(mask)+sizeof(messages)];
  *(int*)(paramBuffer) = mask;
  *(int*)(paramBuffer+sizeof(mask)) = messages;

  asm volatile ("mov r0, #20\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int Class::debugKindred(int mask, int messages)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(mask)+sizeof(messages)];
  *(int*)(paramBuffer) = mask;
  *(int*)(paramBuffer+sizeof(mask)) = messages;

  asm volatile ("mov r0, #21\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int Class::displayMember(ProxiableObject* arg1)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(ProxiableObject**)(paramBuffer) = (ProxiableObject*)(arg1->kernelObj);

  asm volatile ("mov r0, #22\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int Class::displayMembers()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #23\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int Class::displayKindred(int leaksOnly)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(leaksOnly)];
  *(int*)(paramBuffer) = leaksOnly;

  asm volatile ("mov r0, #24\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int Class::applyToMembers(ClassApplicationFunction arg1)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(ClassApplicationFunction*)(paramBuffer) = arg1;

  asm volatile ("mov r0, #25\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int Class::applyToKindred(ClassApplicationFunction arg1)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(ClassApplicationFunction*)(paramBuffer) = arg1;

  asm volatile ("mov r0, #26\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3864\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

Class::~Class() {}
#include "ClassInstaller.h"
DEFINECLASS("Class",Class,ProxiableObject,0,0);

