#include "ProxiableObject.h"
#include "Class.h"
#include "PersistentObject.h"
#include "UNIXFileInfo.h"
#include "PersistentMemoryObject.h"

#include "FileSystemInterface.h"
#include "exception.h"

FileSystemInterface::FileSystemInterface(void* k) :ProxiableObject((void*)0) {kernelObj=k;}

ProxiableObject* FileSystemInterface::open(char* path, Class* interfaceClass, int& status, int flags, int mode, Class* baseClass)
{
  ProxiableObject* ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(path)+sizeof(interfaceClass)+sizeof(void*)+sizeof(flags)+sizeof(mode)+sizeof(baseClass)];
  *(char**)(paramBuffer) = path;
  *(Class**)(paramBuffer+sizeof(path)) = (Class*)(interfaceClass->kernelObj);
  *(int**)(paramBuffer+sizeof(path)+sizeof(interfaceClass)) = &status;
  *(int*)(paramBuffer+sizeof(path)+sizeof(interfaceClass)+sizeof(void*)) = flags;
  *(int*)(paramBuffer+sizeof(path)+sizeof(interfaceClass)+sizeof(void*)+sizeof(flags)) = mode;
  *(Class**)(paramBuffer+sizeof(path)+sizeof(interfaceClass)+sizeof(void*)+sizeof(flags)+sizeof(mode)) = (Class*)(baseClass->kernelObj);

  asm volatile ("mov r0, #2\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3858\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new ProxiableObject( (void*)ret );

  return ret;
}

PersistentObject* FileSystemInterface::open(char* path, int& status, int& returnType, int flags, int mode)
{
  PersistentObject* ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(path)+sizeof(void*)+sizeof(void*)+sizeof(flags)+sizeof(mode)];
  *(char**)(paramBuffer) = path;
  *(int**)(paramBuffer+sizeof(path)) = &status;
  *(int**)(paramBuffer+sizeof(path)+sizeof(void*)) = &returnType;
  *(int*)(paramBuffer+sizeof(path)+sizeof(void*)+sizeof(void*)) = flags;
  *(int*)(paramBuffer+sizeof(path)+sizeof(void*)+sizeof(void*)+sizeof(flags)) = mode;

  asm volatile ("mov r0, #3\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3858\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new PersistentObject( (void*)ret );

  return ret;
}

int FileSystemInterface::add(char* path, PersistentObject* arg2)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(path)+sizeof(arg2)];
  *(char**)(paramBuffer) = path;
  *(PersistentObject**)(paramBuffer+sizeof(path)) = (PersistentObject*)(arg2->kernelObj);

  asm volatile ("mov r0, #4\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3858\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int FileSystemInterface::info(char* dirname, UNIXFileInfo* arg2, int follow, unsigned int* hints)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(dirname)+sizeof(arg2)+sizeof(follow)+sizeof(hints)];
  *(char**)(paramBuffer) = dirname;
  *(UNIXFileInfo**)(paramBuffer+sizeof(dirname)) = arg2;
  *(int*)(paramBuffer+sizeof(dirname)+sizeof(arg2)) = follow;
  *(unsigned int**)(paramBuffer+sizeof(dirname)+sizeof(arg2)+sizeof(follow)) = hints;

  asm volatile ("mov r0, #5\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3858\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int FileSystemInterface::link(char* inpath, char* outpath, int overwrite)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(inpath)+sizeof(outpath)+sizeof(overwrite)];
  *(char**)(paramBuffer) = inpath;
  *(char**)(paramBuffer+sizeof(inpath)) = outpath;
  *(int*)(paramBuffer+sizeof(inpath)+sizeof(outpath)) = overwrite;

  asm volatile ("mov r0, #6\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3858\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int FileSystemInterface::unlink(char* pathname)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(pathname)];
  *(char**)(paramBuffer) = pathname;

  asm volatile ("mov r0, #7\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3858\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int FileSystemInterface::chdir(char* dirname)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(dirname)];
  *(char**)(paramBuffer) = dirname;

  asm volatile ("mov r0, #8\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3858\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int FileSystemInterface::chroot(char* dirname)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(dirname)];
  *(char**)(paramBuffer) = dirname;

  asm volatile ("mov r0, #9\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3858\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int FileSystemInterface::chdir(PersistentMemoryObject* pmo)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(pmo)];
  *(PersistentMemoryObject**)(paramBuffer) = (PersistentMemoryObject*)(pmo->kernelObj);

  asm volatile ("mov r0, #10\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3858\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int FileSystemInterface::chroot(PersistentMemoryObject* pmo)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(pmo)];
  *(PersistentMemoryObject**)(paramBuffer) = (PersistentMemoryObject*)(pmo->kernelObj);

  asm volatile ("mov r0, #11\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3858\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int FileSystemInterface::mount(char* special, char* pathname, int rwflag)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(special)+sizeof(pathname)+sizeof(rwflag)];
  *(char**)(paramBuffer) = special;
  *(char**)(paramBuffer+sizeof(special)) = pathname;
  *(int*)(paramBuffer+sizeof(special)+sizeof(pathname)) = rwflag;

  asm volatile ("mov r0, #12\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3858\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int FileSystemInterface::unmount(char* pathname)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(pathname)];
  *(char**)(paramBuffer) = pathname;

  asm volatile ("mov r0, #13\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3858\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int FileSystemInterface::synchronize()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #14\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3858\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

void FileSystemInterface::openFlagsSetMask(unsigned int mask)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(mask)];
  *(unsigned int*)(paramBuffer) = mask;

  asm volatile ("mov r0, #15\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3858\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void FileSystemInterface::openFlagsClearMask(unsigned int mask)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(mask)];
  *(unsigned int*)(paramBuffer) = mask;

  asm volatile ("mov r0, #16\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3858\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

unsigned int FileSystemInterface::openFlagsSetMask()
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #17\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3858\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

unsigned int FileSystemInterface::openFlagsClearMask()
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #18\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3858\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

FileSystemInterface::~FileSystemInterface() {}
#include "ClassInstaller.h"
DEFINECLASS("FileSystemInterface",FileSystemInterface,ProxiableObject,0,0);

