#include "Class.h"

#include "MemoryObject.h"
#include "exception.h"

MemoryObject::MemoryObject(void* k) :MemoryRange((void*)0) {kernelObj=k;}

int MemoryObject::fixedSize()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #2\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3846\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int MemoryObject::read(unsigned int startUnit, int count, char* addr, int nbytes)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(startUnit)+sizeof(count)+sizeof(addr)+sizeof(nbytes)];
  *(unsigned int*)(paramBuffer) = startUnit;
  *(int*)(paramBuffer+sizeof(startUnit)) = count;
  *(char**)(paramBuffer+sizeof(startUnit)+sizeof(count)) = addr;
  *(int*)(paramBuffer+sizeof(startUnit)+sizeof(count)+sizeof(addr)) = nbytes;

  asm volatile ("mov r0, #3\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3846\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int MemoryObject::write(unsigned int startUnit, int count, char* addr, int nbytes)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(startUnit)+sizeof(count)+sizeof(addr)+sizeof(nbytes)];
  *(unsigned int*)(paramBuffer) = startUnit;
  *(int*)(paramBuffer+sizeof(startUnit)) = count;
  *(char**)(paramBuffer+sizeof(startUnit)+sizeof(count)) = addr;
  *(int*)(paramBuffer+sizeof(startUnit)+sizeof(count)+sizeof(addr)) = nbytes;

  asm volatile ("mov r0, #4\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3846\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int MemoryObject::writeThrough(unsigned int startUnit, int count, char* addr, int nbytes)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(startUnit)+sizeof(count)+sizeof(addr)+sizeof(nbytes)];
  *(unsigned int*)(paramBuffer) = startUnit;
  *(int*)(paramBuffer+sizeof(startUnit)) = count;
  *(char**)(paramBuffer+sizeof(startUnit)+sizeof(count)) = addr;
  *(int*)(paramBuffer+sizeof(startUnit)+sizeof(count)+sizeof(addr)) = nbytes;

  asm volatile ("mov r0, #5\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3846\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

void MemoryObject::synchronize()
{
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #6\n\t"
		"mov r1, %1\n\t"
		"swi #3846\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void MemoryObject::flush()
{
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #7\n\t"
		"mov r1, %1\n\t"
		"swi #3846\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

int MemoryObject::setNumberOfUnits(unsigned int numberOfUnits)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(numberOfUnits)];
  *(unsigned int*)(paramBuffer) = numberOfUnits;

  asm volatile ("mov r0, #8\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3846\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int MemoryObject::copy(MemoryObject* dest)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(dest)];
  *(MemoryObject**)(paramBuffer) = (MemoryObject*)(dest->kernelObj);

  asm volatile ("mov r0, #9\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3846\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

unsigned int MemoryObject::cacheLog2UnitSize()
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #10\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3846\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

unsigned int MemoryObject::setCacheLog2UnitSize(unsigned int arg1)
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(unsigned int*)(paramBuffer) = arg1;

  asm volatile ("mov r0, #11\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3846\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

Class* MemoryObject::cacheClass()
{
  Class* ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #12\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3846\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new Class( (void*)ret );

  return ret;
}

void MemoryObject::setCacheClass(Class* arg1)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(Class**)(paramBuffer) = (Class*)(arg1->kernelObj);

  asm volatile ("mov r0, #13\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3846\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

Class* MemoryObject::replacementPolicyClass()
{
  Class* ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #14\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3846\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new Class( (void*)ret );

  return ret;
}

void MemoryObject::setReplacementPolicyClass(Class* arg1)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(Class**)(paramBuffer) = (Class*)(arg1->kernelObj);

  asm volatile ("mov r0, #15\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3846\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

unsigned int MemoryObject::maxResidentCacheUnits()
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #16\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3846\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

unsigned int MemoryObject::setMaxResidentCacheUnits(unsigned int units)
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(units)];
  *(unsigned int*)(paramBuffer) = units;

  asm volatile ("mov r0, #17\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3846\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

void MemoryObject::addCache(Class* cacheClass)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(cacheClass)];
  *(Class**)(paramBuffer) = (Class*)(cacheClass->kernelObj);

  asm volatile ("mov r0, #18\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3846\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

MemoryObject::~MemoryObject() {}
#include "ClassInstaller.h"
DEFINECLASS("MemoryObject",MemoryObject,MemoryRange,0,0);

