#include "PersistentMemoryObject.h"
#include "FileObject.h"
#include "Class.h"
#include "MemoryObjectDictionary.h"

#include "MemoryObjectContainer.h"
#include "exception.h"

MemoryObjectContainer::MemoryObjectContainer(void* k) :PersistentObject((void*)0) {kernelObj=k;}

MemoryObjectContainer::MemoryObjectContainer(PersistentMemoryObject* source) :PersistentObject((void*)0)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(source)];
  *(PersistentMemoryObject**)(paramBuffer) = (PersistentMemoryObject*)(source->kernelObj);

  asm volatile (
		"mov r0, #0\n\t"
		"mov r2, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3852\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (paramBuffer), "r"(&kernelObj) : "r0", "r1", "r2", "r3" 
		);
  kernel_exception::throw_exception(code);
  
}

FileObject* MemoryObjectContainer::open(int id, int& status, int flags)
{
  FileObject* ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(id)+sizeof(void*)+sizeof(flags)];
  *(int*)(paramBuffer) = id;
  *(int**)(paramBuffer+sizeof(id)) = &status;
  *(int*)(paramBuffer+sizeof(id)+sizeof(void*)) = flags;

  asm volatile ("mov r0, #2\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3852\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new FileObject( (void*)ret );

  return ret;
}

FileObject* MemoryObjectContainer::create(int id, Class* arg2, int& sts, int flags)
{
  FileObject* ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(id)+sizeof(arg2)+sizeof(void*)+sizeof(flags)];
  *(int*)(paramBuffer) = id;
  *(Class**)(paramBuffer+sizeof(id)) = (Class*)(arg2->kernelObj);
  *(int**)(paramBuffer+sizeof(id)+sizeof(arg2)) = &sts;
  *(int*)(paramBuffer+sizeof(id)+sizeof(arg2)+sizeof(void*)) = flags;

  asm volatile ("mov r0, #3\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3852\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new FileObject( (void*)ret );

  return ret;
}

void MemoryObjectContainer::close(FileObject* fo)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(fo)];
  *(FileObject**)(paramBuffer) = (FileObject*)(fo->kernelObj);

  asm volatile ("mov r0, #4\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3852\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

int MemoryObjectContainer::allocate(int& blocks, int nearPhysicalUnit, int units)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(void*)+sizeof(nearPhysicalUnit)+sizeof(units)];
  *(int**)(paramBuffer) = &blocks;
  *(int*)(paramBuffer+sizeof(void*)) = nearPhysicalUnit;
  *(int*)(paramBuffer+sizeof(void*)+sizeof(nearPhysicalUnit)) = units;

  asm volatile ("mov r0, #5\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3852\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

void MemoryObjectContainer::free(int& blocks, int physicalUnit, int units)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(void*)+sizeof(physicalUnit)+sizeof(units)];
  *(int**)(paramBuffer) = &blocks;
  *(int*)(paramBuffer+sizeof(void*)) = physicalUnit;
  *(int*)(paramBuffer+sizeof(void*)+sizeof(physicalUnit)) = units;

  asm volatile ("mov r0, #6\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3852\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void MemoryObjectContainer::synchronize()
{
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #7\n\t"
		"mov r1, %1\n\t"
		"swi #3852\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void MemoryObjectContainer::synchronize(int idNumber)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(idNumber)];
  *(int*)(paramBuffer) = idNumber;

  asm volatile ("mov r0, #8\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3852\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

MemoryObjectDictionary* MemoryObjectContainer::rootDictionary()
{
  MemoryObjectDictionary* ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #9\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3852\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new MemoryObjectDictionary( (void*)ret );

  return ret;
}

int MemoryObjectContainer::size()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #10\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3852\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int MemoryObjectContainer::rootIdNumber()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #11\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3852\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

void MemoryObjectContainer::acquire()
{
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #12\n\t"
		"mov r1, %1\n\t"
		"swi #3852\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void MemoryObjectContainer::release()
{
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #13\n\t"
		"mov r1, %1\n\t"
		"swi #3852\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void MemoryObjectContainer::acquire(int idNumber)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(idNumber)];
  *(int*)(paramBuffer) = idNumber;

  asm volatile ("mov r0, #14\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3852\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void MemoryObjectContainer::release(int idNumber)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(idNumber)];
  *(int*)(paramBuffer) = idNumber;

  asm volatile ("mov r0, #15\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3852\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

MemoryObjectContainer::~MemoryObjectContainer() {}
#include "ClassInstaller.h"
DEFINEPERSISTENTCLASS("MemoryObjectContainer",MemoryObjectContainer,PersistentObject,0,0);

