#include "PersistentMemoryObject.h"
#include "Class.h"

#include "MemoryObjectDictionary.h"
#include "exception.h"

MemoryObjectDictionary::MemoryObjectDictionary(void* k) :PersistentObject((void*)0) {kernelObj=k;}

MemoryObjectDictionary::MemoryObjectDictionary(PersistentMemoryObject* source) :PersistentObject((void*)0)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(source)];
  *(PersistentMemoryObject**)(paramBuffer) = (PersistentMemoryObject*)(source->kernelObj);

  asm volatile (
		"mov r0, #0\n\t"
		"mov r2, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3859\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (paramBuffer), "r"(&kernelObj) : "r0", "r1", "r2", "r3" 
		);
  kernel_exception::throw_exception(code);
  
}

PersistentMemoryObject* MemoryObjectDictionary::open(char* key, int& status, int flags)
{
  PersistentMemoryObject* ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(key)+sizeof(void*)+sizeof(flags)];
  *(char**)(paramBuffer) = key;
  *(int**)(paramBuffer+sizeof(key)) = &status;
  *(int*)(paramBuffer+sizeof(key)+sizeof(void*)) = flags;

  asm volatile ("mov r0, #2\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3859\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new PersistentMemoryObject( (void*)ret );

  return ret;
}

PersistentMemoryObject* MemoryObjectDictionary::create(char* key, Class* arg2, int& sts, int flags)
{
  PersistentMemoryObject* ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(key)+sizeof(arg2)+sizeof(void*)+sizeof(flags)];
  *(char**)(paramBuffer) = key;
  *(Class**)(paramBuffer+sizeof(key)) = (Class*)(arg2->kernelObj);
  *(int**)(paramBuffer+sizeof(key)+sizeof(arg2)) = &sts;
  *(int*)(paramBuffer+sizeof(key)+sizeof(arg2)+sizeof(void*)) = flags;

  asm volatile ("mov r0, #3\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3859\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new PersistentMemoryObject( (void*)ret );

  return ret;
}

int MemoryObjectDictionary::add(char* key, PersistentMemoryObject* arg2)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(key)+sizeof(arg2)];
  *(char**)(paramBuffer) = key;
  *(PersistentMemoryObject**)(paramBuffer+sizeof(key)) = (PersistentMemoryObject*)(arg2->kernelObj);

  asm volatile ("mov r0, #4\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3859\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int MemoryObjectDictionary::remove(char* key, int* links)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(key)+sizeof(links)];
  *(char**)(paramBuffer) = key;
  *(int**)(paramBuffer+sizeof(key)) = links;

  asm volatile ("mov r0, #5\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3859\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int MemoryObjectDictionary::keys(char* buffer, int bufferSize, int& count)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(buffer)+sizeof(bufferSize)+sizeof(void*)];
  *(char**)(paramBuffer) = buffer;
  *(int*)(paramBuffer+sizeof(buffer)) = bufferSize;
  *(int**)(paramBuffer+sizeof(buffer)+sizeof(bufferSize)) = &count;

  asm volatile ("mov r0, #6\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3859\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int MemoryObjectDictionary::associations(char* buffer, int bufferSize, int& count)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(buffer)+sizeof(bufferSize)+sizeof(void*)];
  *(char**)(paramBuffer) = buffer;
  *(int*)(paramBuffer+sizeof(buffer)) = bufferSize;
  *(int**)(paramBuffer+sizeof(buffer)+sizeof(bufferSize)) = &count;

  asm volatile ("mov r0, #7\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3859\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int MemoryObjectDictionary::associations(char* buffer, int bufferSize, int& count, int& offset)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(buffer)+sizeof(bufferSize)+sizeof(void*)+sizeof(void*)];
  *(char**)(paramBuffer) = buffer;
  *(int*)(paramBuffer+sizeof(buffer)) = bufferSize;
  *(int**)(paramBuffer+sizeof(buffer)+sizeof(bufferSize)) = &count;
  *(int**)(paramBuffer+sizeof(buffer)+sizeof(bufferSize)+sizeof(void*)) = &offset;

  asm volatile ("mov r0, #8\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3859\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int MemoryObjectDictionary::keySize()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #9\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3859\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int MemoryObjectDictionary::sharesContainer(PersistentMemoryObject* arg1)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(PersistentMemoryObject**)(paramBuffer) = (PersistentMemoryObject*)(arg1->kernelObj);

  asm volatile ("mov r0, #10\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3859\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int MemoryObjectDictionary::synchronize()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #11\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3859\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

MemoryObjectDictionary::~MemoryObjectDictionary() {}
#include "ClassInstaller.h"
DEFINEPERSISTENTCLASS("MemoryObjectDictionary",MemoryObjectDictionary,PersistentObject,0,0);

