
#include "MemoryRange.h"
#include "exception.h"

MemoryRange::MemoryRange(void* k) :ProxiableObject((void*)0) {kernelObj=k;}

MemoryRange::MemoryRange(unsigned int numberOfUnits, unsigned int log2UnitSize) :ProxiableObject((void*)0)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(numberOfUnits)+sizeof(log2UnitSize)];
  *(unsigned int*)(paramBuffer) = numberOfUnits;
  *(unsigned int*)(paramBuffer+sizeof(numberOfUnits)) = log2UnitSize;

  asm volatile (
		"mov r0, #0\n\t"
		"mov r2, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3844\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (paramBuffer), "r"(&kernelObj) : "r0", "r1", "r2", "r3" 
		);
  kernel_exception::throw_exception(code);
  
}

unsigned int MemoryRange::unitSize()
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #2\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3844\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

unsigned int MemoryRange::log2UnitSize()
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #3\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3844\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

unsigned int MemoryRange::numberOfUnits()
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #4\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3844\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

unsigned int MemoryRange::offsetToUnit(Offset offset1)
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(offset1)];
  *(Offset*)(paramBuffer) = offset1;

  asm volatile ("mov r0, #5\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3844\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

Offset MemoryRange::unitToOffset(unsigned int unit)
{
  Offset ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(unit)];
  *(unsigned int*)(paramBuffer) = unit;

  asm volatile ("mov r0, #6\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3844\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int MemoryRange::isIn(unsigned int unit)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(unit)];
  *(unsigned int*)(paramBuffer) = unit;

  asm volatile ("mov r0, #7\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3844\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

MemoryRange::~MemoryRange() {}
#include "ClassInstaller.h"
DEFINECLASS("MemoryRange",MemoryRange,ProxiableObject,0,0);

