//  Please read comments in OutputStream.h regarding g++ bug with passing
//  of doubles.  - Lup

int
strlen( const char * s )
{
    const char *p = s;
    for ( ; *p; p++ ) ;
    return p - s;
}

void
OutputStream::emitChar( char c, int width, OutputStreamPadSide padSide )
{
    int padLength = ( width > 0 ) ? ( width - 1 ) : 0;

    if ( padSide == padLeft )   emitSpaces( padLength );
    write( &c, 1 );
    if ( padSide == padRight )  emitSpaces( padLength );
}

void
OutputStream::emitString( const char * string, int width,
                          OutputStreamPadSide padSide )
{
    /*
     * Determine pad length.
     */
    int padLength = width;
    for ( const char * p = string; *p != 0; p++ ) {
        if ( padLength == 0 )
            break;
        padLength--;
    }

    if ( padSide == padLeft )   emitSpaces( padLength );
    write( string, strlen( string ) );
    if ( padSide == padRight )  emitSpaces( padLength );
}

void
OutputStream::emitSpaces( int count )
{
    if( count == 0 ) return;

    static char spaces[] = { ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ',
                             ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '};

    unsigned i;
    for( i = count; i > ( sizeof( spaces ) );
         i -= ( sizeof( spaces ) ) ) {
        write( spaces, sizeof( spaces ) );
    }
    //Assert( i > 0 );
    //Assert( i <= sizeof( spaces ) - 1 );
    write( spaces, i );
}

void
OutputStream::emitZeros( int count )
{
    if( count == 0 ) return;

    static char zeros[] = { '0', '0', '0', '0', '0', '0', '0', '0',
                            '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0', '0'};

    unsigned i;
    for( i = count; i > ( sizeof( zeros ) );
         i -= ( sizeof( zeros ) ) ) {
        write( zeros, sizeof( zeros ) );
    }
    //Assert( i > 0 );
    //Assert( i <= sizeof( zeros ) - 1 );
    write( zeros, i );
}


/*
 * emitNumber - converts and prints an int-sized argument in a given base
 *
 * NOTE: Currently only bases 2, 8, 10, and 16 work.  Calls write().
 *  Assumes *caller* has acquired the stream's exclusive use if
 *  necessary.
 */
void
OutputStream::emitNumber( unsigned int arg, int base, int width,
                          OutputStreamPadSide padSide, OutputStreamPadChar padChar )
{
    /*
     * Determine prefix.
     * Update/maintain pad length.
     * Negate value if signed decimal.
     */
    int padLength = width;
    char * prefix = 0;
    int prefixLength = 0;
    int sarg ;
    switch ( base ) {
        case 2:     /* binary */
            prefix = "0b";
            prefixLength = 2;
            break;
        case 8:     /* octal */
            if ( arg != 0 ) {
                prefix = "0";
                prefixLength = 1;
            }
            break;
        case -10:   /* signed decimal */
            base = 10;
            sarg = (int) arg;
            if ( sarg < 0 ) {
                prefix = "-";
                prefixLength = 1;
                sarg = -sarg;
                arg = (unsigned int) sarg;
            }
            break;
        case 10:    /* unsigned decimal */
            break;
        default:    /* unknown - treat as hexadecimal */
            base = 16;
            /* Falls through... */
        case 16:    /* hexadecmial */
            prefix = "0x";
            prefixLength = 2;
            break;
    }
    padLength -= prefixLength;

    /*
     * Convert value to digits into buffer.
     * Update/maintain pad length.
     * Note that this is in reverse order (i.e., LSD first).
     * We test for completion at the bottom of the
     * loop in order to assure that we always get at least
     * one digit in case the value is initially zero.
     */
    unsigned int val = arg;
    static const int bufferSize = 32;
    char buffer[bufferSize];
    char * p = buffer;
    do {
        *p++ = "0123456789abcdef"[val % base];
        padLength--;
    } while ( ( val /= base ) != 0 );
    p--;

    /*
     * Emit left space padding, if appropriate.
     */
    if ( ( padLength > 0 )   &&
         ( padSide == padLeft ) &&
         ( padChar == aSpace ) ) {
        emitSpaces( padLength );
    }

    /*
     * Emit the prefix.
     */
    if( prefix != 0 ) {
        //Assert( prefixLength != 0 );
        write( prefix, prefixLength );
    }

    /*
     * Emit left zero padding, if appropriate.
     */
    if ( ( padLength > 0 )   &&
         ( padSide == padLeft ) &&
         ( padChar == aZero ) ) {
        emitZeros( padLength );
    }

    /*
     * Emit all the digits in the buffer.
     * Copy them into an inverted buffer since this is probably
     * faster than writing them one at a time.
     */
    char invertBuffer[bufferSize];
    char * q = invertBuffer;
    int len = p - buffer + 1;
    while( p >= buffer ) {
        *q++ = *p--;
    }
    write( invertBuffer, len );

    /*
     * Emit right padding, if appropriate.
     * This will always be space padding,
     * since zero padding here makes no sense.
     */
    if ( ( padLength > 0 ) && ( padSide == padRight ) ) {
        emitSpaces( padLength );
    }
}

OutputStream &
OutputStream::operator << ( char c )
{
    write( &c, 1 );
    return( *this );
}

OutputStream &
OutputStream::operator << ( const char * s )
{
    write( s, strlen( s ) );
    return( *this );
}

OutputStream &
OutputStream::operator << ( const void * s )
{
    write( "Cannot output untyped void*", 28 );
    return( *this );
}

OutputStream &
OutputStream::operator << ( int i )
{
    emitNumber( i, -10, 0, padLeft, aZero );
    return( *this );
}

OutputStream &
OutputStream::operator << ( unsigned int i )
{
    emitNumber( i, 10, 0, padLeft, aZero );
    return( *this );
}

OutputStream &
OutputStream::operator << ( long i )
{
    emitNumber( i, -10, 0, padLeft, aZero );
    return( *this );
}

OutputStream &
OutputStream::operator << ( unsigned long i )
{
    emitNumber( i, 10, 0, padLeft, aZero );
    return( *this );
}
