
#include "PeriodicTimer.h"
#include "exception.h"

PeriodicTimer::PeriodicTimer(void* k) :Timer((void*)0) {kernelObj=k;}

PeriodicTimer::PeriodicTimer() :Timer((void*)0)
{
  kernel_exception::EXCODE code;
  asm volatile (
		"mov r0, #0\n\t"
		"mov r3, %1\n\t"
		"swi #3855\n\t"
		"str r0, %0"
		: "=m"(code) :"r"(&kernelObj) : "r0", "r1", "r2", "r3" 
		);
  kernel_exception::throw_exception(code);
}

unsigned int PeriodicTimer::start(unsigned int microseconds)
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(microseconds)];
  *(unsigned int*)(paramBuffer) = microseconds;

  asm volatile ("mov r0, #2\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3855\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

unsigned int PeriodicTimer::stop()
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #3\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3855\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

unsigned int PeriodicTimer::residual()
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #4\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3855\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

void PeriodicTimer::await()
{
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #5\n\t"
		"mov r1, %1\n\t"
		"swi #3855\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

PeriodicTimer::~PeriodicTimer() 
{
  kernel_exception::EXCODE code;
  asm volatile (
		"mov r0, #1\n\t"
		"mov r1, %1\n\t"
		"swi $3855\n\t"
		"str r0, %0"
		: "=m" (code) :"r" (kernelObj) : "r0", "r1", "r2", "r3"
		);
  kernel_exception::throw_exception(code);
}
#include "ClassInstaller.h"
DEFINECLASS("PeriodicTimer",PeriodicTimer,Timer,0,0);

