#include "PersistentMemoryObject.h"

#include "PersistentArray.h"
#include "exception.h"

PersistentArray::PersistentArray(void* k) :PersistentObject((void*)0) {kernelObj=k;}

PersistentArray::PersistentArray(PersistentMemoryObject* mo) :PersistentObject((void*)0)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(mo)];
  *(PersistentMemoryObject**)(paramBuffer) = (PersistentMemoryObject*)(mo->kernelObj);

  asm volatile (
		"mov r0, #0\n\t"
		"mov r2, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3850\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (paramBuffer), "r"(&kernelObj) : "r0", "r1", "r2", "r3" 
		);
  kernel_exception::throw_exception(code);
  
}

int PersistentArray::at(unsigned int index)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(index)];
  *(unsigned int*)(paramBuffer) = index;

  asm volatile ("mov r0, #2\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3850\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentArray::at(unsigned int index, char* values, unsigned int count)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(index)+sizeof(values)+sizeof(count)];
  *(unsigned int*)(paramBuffer) = index;
  *(char**)(paramBuffer+sizeof(index)) = values;
  *(unsigned int*)(paramBuffer+sizeof(index)+sizeof(values)) = count;

  asm volatile ("mov r0, #3\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3850\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentArray::at(char* index, int& status)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(index)+sizeof(void*)];
  *(char**)(paramBuffer) = index;
  *(int**)(paramBuffer+sizeof(index)) = &status;

  asm volatile ("mov r0, #4\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3850\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentArray::atPut(unsigned int index, int value)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(index)+sizeof(value)];
  *(unsigned int*)(paramBuffer) = index;
  *(int*)(paramBuffer+sizeof(index)) = value;

  asm volatile ("mov r0, #5\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3850\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentArray::atPut(unsigned int index, char* values, unsigned int count)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(index)+sizeof(values)+sizeof(count)];
  *(unsigned int*)(paramBuffer) = index;
  *(char**)(paramBuffer+sizeof(index)) = values;
  *(unsigned int*)(paramBuffer+sizeof(index)+sizeof(values)) = count;

  asm volatile ("mov r0, #6\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3850\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentArray::atPut(char* index, int value, int& status)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(index)+sizeof(value)+sizeof(void*)];
  *(char**)(paramBuffer) = index;
  *(int*)(paramBuffer+sizeof(index)) = value;
  *(int**)(paramBuffer+sizeof(index)+sizeof(value)) = &status;

  asm volatile ("mov r0, #7\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3850\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

unsigned int PersistentArray::size()
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #8\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3850\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

PersistentArray::~PersistentArray() {}
#include "ClassInstaller.h"
DEFINEPERSISTENTCLASS("PersistentArray",PersistentArray,PersistentObject,0,0);

