#include "Class.h"
#include "ClassConstructor.h"
#include "MemoryObjectDictionary.h"
#include "PersistentObject.h"

#include "PersistentClass.h"
#include "exception.h"

PersistentClass::PersistentClass(void* k) :Class((void*)0) {kernelObj=k;}

PersistentClass::PersistentClass(Class* parent, int size, char* name, ClassConstructor* ctor) :Class((void*)0)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(parent)+sizeof(size)+sizeof(name)+sizeof(ctor)];
  *(Class**)(paramBuffer) = (Class*)(parent->kernelObj);
  *(int*)(paramBuffer+sizeof(parent)) = size;
  *(char**)(paramBuffer+sizeof(parent)+sizeof(size)) = name;
  *(ClassConstructor**)(paramBuffer+sizeof(parent)+sizeof(size)+sizeof(name)) = (ClassConstructor*)(ctor->kernelObj);

  asm volatile (
		"mov r0, #0\n\t"
		"mov r2, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3875\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (paramBuffer), "r"(&kernelObj) : "r0", "r1", "r2", "r3" 
		);
  kernel_exception::throw_exception(code);
  
}

MemoryObjectDictionary* PersistentClass::rootDictionary()
{
  MemoryObjectDictionary* ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #2\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3875\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new MemoryObjectDictionary( (void*)ret );

  return ret;
}

PersistentObject* PersistentClass::newObject()
{
  PersistentObject* ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #3\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3875\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new PersistentObject( (void*)ret );

  return ret;
}

PersistentObject* PersistentClass::retrieveObject(int id)
{
  PersistentObject* ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(id)];
  *(int*)(paramBuffer) = id;

  asm volatile ("mov r0, #4\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3875\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new PersistentObject( (void*)ret );

  return ret;
}

PersistentClass::~PersistentClass() {}
#include "ClassInstaller.h"
DEFINECLASS("PersistentClass",PersistentClass,Class,0,0);

