#include "UNIXFileInfo.h"
#include "MemoryObjectContainer.h"
#include "ProxiableObject.h"
#include "Class.h"
#include "PersistentObject.h"
#include "OutputStream.h"

#include "PersistentMemoryObject.h"
#include "exception.h"

PersistentMemoryObject::PersistentMemoryObject(void* k) :MemoryObject((void*)0) {kernelObj=k;}

void PersistentMemoryObject::unreference()
{
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #2\n\t"
		"mov r1, %1\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void PersistentMemoryObject::synchronize()
{
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #3\n\t"
		"mov r1, %1\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

int PersistentMemoryObject::setNumberOfUnits(unsigned int numberOfUnits)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(numberOfUnits)];
  *(unsigned int*)(paramBuffer) = numberOfUnits;

  asm volatile ("mov r0, #4\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

unsigned int PersistentMemoryObject::numberOfRecords()
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #5\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentMemoryObject::setNumberOfRecords(unsigned int numberOfRecords)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(numberOfRecords)];
  *(unsigned int*)(paramBuffer) = numberOfRecords;

  asm volatile ("mov r0, #6\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

unsigned int PersistentMemoryObject::recordSize()
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #7\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentMemoryObject::setRecordSize(unsigned int recordSize)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(recordSize)];
  *(unsigned int*)(paramBuffer) = recordSize;

  asm volatile ("mov r0, #8\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

unsigned int PersistentMemoryObject::numberOfBytes()
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #9\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentMemoryObject::setNumberOfBytes(unsigned int numberOfBytes)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(numberOfBytes)];
  *(unsigned int*)(paramBuffer) = numberOfBytes;

  asm volatile ("mov r0, #10\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

void PersistentMemoryObject::info(UNIXFileInfo* arg1)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(UNIXFileInfo**)(paramBuffer) = arg1;

  asm volatile ("mov r0, #11\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

int PersistentMemoryObject::isSymbolicLink()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #12\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentMemoryObject::isContainer()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #13\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentMemoryObject::isDictionary()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #14\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentMemoryObject::setUserId(int uid)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(uid)];
  *(int*)(paramBuffer) = uid;

  asm volatile ("mov r0, #15\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentMemoryObject::setGroupId(int gid)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(gid)];
  *(int*)(paramBuffer) = gid;

  asm volatile ("mov r0, #16\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentMemoryObject::setMode(int mode)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(mode)];
  *(int*)(paramBuffer) = mode;

  asm volatile ("mov r0, #17\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentMemoryObject::idNumber()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #18\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentMemoryObject::links(int delta)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(delta)];
  *(int*)(paramBuffer) = delta;

  asm volatile ("mov r0, #19\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

MemoryObjectContainer* PersistentMemoryObject::container()
{
  MemoryObjectContainer* ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #20\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new MemoryObjectContainer( (void*)ret );

  return ret;
}

int PersistentMemoryObject::access(int mask)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(mask)];
  *(int*)(paramBuffer) = mask;

  asm volatile ("mov r0, #21\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

ProxiableObject* PersistentMemoryObject::asA(Class* arg1, int& status)
{
  ProxiableObject* ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)+sizeof(void*)];
  *(Class**)(paramBuffer) = (Class*)(arg1->kernelObj);
  *(int**)(paramBuffer+sizeof(arg1)) = &status;

  asm volatile ("mov r0, #22\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new ProxiableObject( (void*)ret );

  return ret;
}

PersistentObject* PersistentMemoryObject::persistentObject()
{
  PersistentObject* ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #23\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new PersistentObject( (void*)ret );

  return ret;
}

void PersistentMemoryObject::close(PersistentObject* so)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(so)];
  *(PersistentObject**)(paramBuffer) = (PersistentObject*)(so->kernelObj);

  asm volatile ("mov r0, #24\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void PersistentMemoryObject::writeName(OutputStream* arg1)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(OutputStream**)(paramBuffer) = (OutputStream*)(arg1->kernelObj);

  asm volatile ("mov r0, #25\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void PersistentMemoryObject::writeIdNumber(OutputStream* arg1)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(OutputStream**)(paramBuffer) = (OutputStream*)(arg1->kernelObj);

  asm volatile ("mov r0, #26\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

int PersistentMemoryObject::userId()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #27\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentMemoryObject::groupId()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #28\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentMemoryObject::alloc(unsigned int startUnit, int count)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(startUnit)+sizeof(count)];
  *(unsigned int*)(paramBuffer) = startUnit;
  *(int*)(paramBuffer+sizeof(startUnit)) = count;

  asm volatile ("mov r0, #29\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentMemoryObject::free(unsigned int startUnit, int count)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(startUnit)+sizeof(count)];
  *(unsigned int*)(paramBuffer) = startUnit;
  *(int*)(paramBuffer+sizeof(startUnit)) = count;

  asm volatile ("mov r0, #30\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

void PersistentMemoryObject::setCacheHints(unsigned int userHints)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(userHints)];
  *(unsigned int*)(paramBuffer) = userHints;

  asm volatile ("mov r0, #31\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

unsigned int PersistentMemoryObject::getCacheHints()
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #32\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3847\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

PersistentMemoryObject::~PersistentMemoryObject() {}
#include "ClassInstaller.h"
DEFINECLASS("PersistentMemoryObject",PersistentMemoryObject,MemoryObject,0,0);

