#include "PersistentMemoryObject.h"
#include "FileSystemInterface.h"
#include "OutputStream.h"
#include "ProxiableObject.h"
#include "Class.h"
#include "UNIXFileInfo.h"

#include "PersistentObject.h"
#include "exception.h"

PersistentObject::PersistentObject(void* k) :ProxiableObject((void*)0) {kernelObj=k;}

PersistentMemoryObject* PersistentObject::source()
{
  PersistentMemoryObject* ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #2\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3849\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new PersistentMemoryObject( (void*)ret );

  return ret;
}

PersistentObject::PersistentObject(PersistentMemoryObject* mo) :ProxiableObject((void*)0)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(mo)];
  *(PersistentMemoryObject**)(paramBuffer) = (PersistentMemoryObject*)(mo->kernelObj);

  asm volatile (
		"mov r0, #0\n\t"
		"mov r2, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3849\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (paramBuffer), "r"(&kernelObj) : "r0", "r1", "r2", "r3" 
		);
  kernel_exception::throw_exception(code);
  
}

int PersistentObject::init(int argc, char** argv, FileSystemInterface* arg3)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(argc)+sizeof(argv)+sizeof(arg3)];
  *(int*)(paramBuffer) = argc;
  *(char***)(paramBuffer+sizeof(argc)) = argv;
  *(FileSystemInterface**)(paramBuffer+sizeof(argc)+sizeof(argv)) = (FileSystemInterface*)(arg3->kernelObj);

  asm volatile ("mov r0, #3\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3849\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentObject::execute()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #4\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3849\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentObject::persist()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #5\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3849\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentObject::desist()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #6\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3849\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

unsigned int PersistentObject::numberOfBytes()
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #7\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3849\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

unsigned int PersistentObject::numberOfRecords()
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #8\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3849\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

unsigned int PersistentObject::recordSize()
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #9\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3849\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentObject::copy(PersistentObject* po)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(po)];
  *(PersistentObject**)(paramBuffer) = (PersistentObject*)(po->kernelObj);

  asm volatile ("mov r0, #10\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3849\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

void PersistentObject::flush()
{
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #11\n\t"
		"mov r1, %1\n\t"
		"swi #3849\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void PersistentObject::writeName(OutputStream* arg1)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(OutputStream**)(paramBuffer) = (OutputStream*)(arg1->kernelObj);

  asm volatile ("mov r0, #12\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3849\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

int PersistentObject::executable()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #13\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3849\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentObject::idNumber()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #14\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3849\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

ProxiableObject* PersistentObject::asA(Class* arg1, int& status)
{
  ProxiableObject* ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)+sizeof(void*)];
  *(Class**)(paramBuffer) = (Class*)(arg1->kernelObj);
  *(int**)(paramBuffer+sizeof(arg1)) = &status;

  asm volatile ("mov r0, #15\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3849\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new ProxiableObject( (void*)ret );

  return ret;
}

int PersistentObject::links()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #16\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3849\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

void PersistentObject::info(UNIXFileInfo* arg1)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(UNIXFileInfo**)(paramBuffer) = arg1;

  asm volatile ("mov r0, #17\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3849\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

int PersistentObject::setUserId(int uid)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(uid)];
  *(int*)(paramBuffer) = uid;

  asm volatile ("mov r0, #18\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3849\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentObject::setGroupId(int gid)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(gid)];
  *(int*)(paramBuffer) = gid;

  asm volatile ("mov r0, #19\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3849\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int PersistentObject::setMode(int mode)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(mode)];
  *(int*)(paramBuffer) = mode;

  asm volatile ("mov r0, #20\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3849\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

PersistentObject::~PersistentObject() {}
#include "ClassInstaller.h"
DEFINECLASS("PersistentObject",PersistentObject,ProxiableObject,0,0);

