#include "Domain.h"
#include "Semaphore.h"
#include "NameServer.h"
#include "ProcessResourceUsage.h"

#include "Process.h"
#include "exception.h"

Process::Process(void* k) :ProxiableObject((void*)0) {kernelObj=k;}

Domain* Process::currentDomain()
{
  Domain* ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #2\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3865\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new Domain( (void*)ret );

  return ret;
}

int Process::waitForProcess(Process* other)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(other)];
  *(Process**)(paramBuffer) = (Process*)(other->kernelObj);

  asm volatile ("mov r0, #3\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3865\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

void Process::notifyUponCompletion(Semaphore* sem, int* arg2)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(sem)+sizeof(arg2)];
  *(Semaphore**)(paramBuffer) = (Semaphore*)(sem->kernelObj);
  *(int**)(paramBuffer+sizeof(sem)) = arg2;

  asm volatile ("mov r0, #4\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3865\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void Process::wakeUpWaiter(int status)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(status)];
  *(int*)(paramBuffer) = status;

  asm volatile ("mov r0, #5\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3865\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

int Process::dead()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #6\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3865\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int Process::completionStatus()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #7\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3865\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

void Process::giveProcessorTo(Process* anotherProcess, ProcessState newState)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(anotherProcess)+sizeof(newState)];
  *(Process**)(paramBuffer) = (Process*)(anotherProcess->kernelObj);
  *(ProcessState*)(paramBuffer+sizeof(anotherProcess)) = newState;

  asm volatile ("mov r0, #8\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3865\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void Process::die(int status)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(status)];
  *(int*)(paramBuffer) = status;

  asm volatile ("mov r0, #9\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3865\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void Process::relinquishProcessor()
{
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #10\n\t"
		"mov r1, %1\n\t"
		"swi #3865\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void Process::ready()
{
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #11\n\t"
		"mov r1, %1\n\t"
		"swi #3865\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

NameServer* Process::domainNameServer()
{
  NameServer* ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #12\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3865\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new NameServer( (void*)ret );

  return ret;
}

Process* Process::clone(char* entry, char* stackPointer)
{
  Process* ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(entry)+sizeof(stackPointer)];
  *(char**)(paramBuffer) = entry;
  *(char**)(paramBuffer+sizeof(entry)) = stackPointer;

  asm volatile ("mov r0, #13\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3865\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new Process( (void*)ret );

  return ret;
}

void Process::getResourceUsage(ProcessResourceUsage* buf)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(buf)];
  *(ProcessResourceUsage**)(paramBuffer) = buf;

  asm volatile ("mov r0, #14\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3865\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

Process::~Process() {}
#include "ClassInstaller.h"
DEFINECLASS("Process",Process,ProxiableObject,0,0);

