#include "OutputStream.h"
#include "Class.h"

#include "ProxiableObject.h"
#include "exception.h"

ProxiableObject::ProxiableObject(void* k)  {kernelObj=k;}

ProxiableObject::ProxiableObject() 
{
  kernel_exception::EXCODE code;
  asm volatile (
		"mov r0, #0\n\t"
		"mov r3, %1\n\t"
		"swi #3841\n\t"
		"str r0, %0"
		: "=m"(code) :"r"(&kernelObj) : "r0", "r1", "r2", "r3" 
		);
  kernel_exception::throw_exception(code);
}

void ProxiableObject::reference()
{
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #2\n\t"
		"mov r1, %1\n\t"
		"swi #3841\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void ProxiableObject::unreference()
{
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #3\n\t"
		"mov r1, %1\n\t"
		"swi #3841\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

char* ProxiableObject::objectName()
{
  char* ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #4\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3841\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

void ProxiableObject::setName(char* str)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(str)];
  *(char**)(paramBuffer) = str;

  asm volatile ("mov r0, #5\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3841\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void ProxiableObject::writeName(OutputStream* arg1)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(OutputStream**)(paramBuffer) = (OutputStream*)(arg1->kernelObj);

  asm volatile ("mov r0, #6\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3841\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void ProxiableObject::inspect(OutputStream* arg1)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(OutputStream**)(paramBuffer) = (OutputStream*)(arg1->kernelObj);

  asm volatile ("mov r0, #7\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3841\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

ProxiableObject* ProxiableObject::asA(Class* arg1, int& status)
{
  ProxiableObject* ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)+sizeof(void*)];
  *(Class**)(paramBuffer) = (Class*)(arg1->kernelObj);
  *(int**)(paramBuffer+sizeof(arg1)) = &status;

  asm volatile ("mov r0, #8\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3841\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new ProxiableObject( (void*)ret );

  return ret;
}

ProxiableObject* ProxiableObject::asA(char* arg1, int& status)
{
  ProxiableObject* ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)+sizeof(void*)];
  *(char**)(paramBuffer) = arg1;
  *(int**)(paramBuffer+sizeof(arg1)) = &status;

  asm volatile ("mov r0, #9\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3841\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new ProxiableObject( (void*)ret );

  return ret;
}

ProxiableObject::~ProxiableObject() {}
#include "ClassInstaller.h"
DEFINECLASS("ProxiableObject",ProxiableObject,Null,0,0);

