#include "UNIXFileInfo.h"
#include "ProxiableObject.h"
#include "Class.h"

#include "RecordStream.h"
#include "exception.h"

RecordStream::RecordStream(void* k) :ProxiableObject((void*)0) {kernelObj=k;}

int RecordStream::read(char* buffer, int size)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(buffer)+sizeof(size)];
  *(char**)(paramBuffer) = buffer;
  *(int*)(paramBuffer+sizeof(buffer)) = size;

  asm volatile ("mov r0, #2\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3863\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int RecordStream::read(char* buffer, int size, int& status)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(buffer)+sizeof(size)+sizeof(void*)];
  *(char**)(paramBuffer) = buffer;
  *(int*)(paramBuffer+sizeof(buffer)) = size;
  *(int**)(paramBuffer+sizeof(buffer)+sizeof(size)) = &status;

  asm volatile ("mov r0, #3\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3863\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int RecordStream::write(char* buffer, int size)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(buffer)+sizeof(size)];
  *(char**)(paramBuffer) = buffer;
  *(int*)(paramBuffer+sizeof(buffer)) = size;

  asm volatile ("mov r0, #4\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3863\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int RecordStream::write(char* buffer, int size, int& status)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(buffer)+sizeof(size)+sizeof(void*)];
  *(char**)(paramBuffer) = buffer;
  *(int*)(paramBuffer+sizeof(buffer)) = size;
  *(int**)(paramBuffer+sizeof(buffer)+sizeof(size)) = &status;

  asm volatile ("mov r0, #5\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3863\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int RecordStream::recordNumber()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #6\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3863\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int RecordStream::setRecordNumber(int delta, int mode)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(delta)+sizeof(mode)];
  *(int*)(paramBuffer) = delta;
  *(int*)(paramBuffer+sizeof(delta)) = mode;

  asm volatile ("mov r0, #7\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3863\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int RecordStream::numberOfBytes()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #8\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3863\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int RecordStream::numberOfRecords()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #9\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3863\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int RecordStream::recordSize()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #10\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3863\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int RecordStream::minimumRecordSize()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #11\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3863\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int RecordStream::maximumRecordSize()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #12\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3863\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int RecordStream::setMode(int mode)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(mode)];
  *(int*)(paramBuffer) = mode;

  asm volatile ("mov r0, #13\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3863\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int RecordStream::setUserId(int uid)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(uid)];
  *(int*)(paramBuffer) = uid;

  asm volatile ("mov r0, #14\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3863\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int RecordStream::setGroupId(int gid)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(gid)];
  *(int*)(paramBuffer) = gid;

  asm volatile ("mov r0, #15\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3863\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

void RecordStream::info(UNIXFileInfo* arg1)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(UNIXFileInfo**)(paramBuffer) = arg1;

  asm volatile ("mov r0, #16\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3863\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

int RecordStream::init(int argc, char** argv)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(argc)+sizeof(argv)];
  *(int*)(paramBuffer) = argc;
  *(char***)(paramBuffer+sizeof(argc)) = argv;

  asm volatile ("mov r0, #17\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3863\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

int RecordStream::executable()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #18\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3863\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

ProxiableObject* RecordStream::asA(Class* arg1, int& status)
{
  ProxiableObject* ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)+sizeof(void*)];
  *(Class**)(paramBuffer) = (Class*)(arg1->kernelObj);
  *(int**)(paramBuffer+sizeof(arg1)) = &status;

  asm volatile ("mov r0, #19\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3863\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new ProxiableObject( (void*)ret );

  return ret;
}

int RecordStream::runFile(int argc, char** argv)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(argc)+sizeof(argv)];
  *(int*)(paramBuffer) = argc;
  *(char***)(paramBuffer+sizeof(argc)) = argv;

  asm volatile ("mov r0, #20\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3863\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

RecordStream::~RecordStream() {}
#include "ClassInstaller.h"
DEFINECLASS("RecordStream",RecordStream,ProxiableObject,0,0);

