#include "ProvidedVersionInstaller.h"
#include "RequiredVersionInstaller.h"
#include "Process.h"
#include "PersistentMemoryObject.h"
#include "Disk.h"
#include "OutputStream.h"
#include "ProxiableObject.h"
#include "InputStream.h"

#include "SystemInterface.h"
#include "exception.h"

SystemInterface::SystemInterface(void* k) :ProxiableObject((void*)0) {kernelObj=k;}

void SystemInterface::initialCall(void* returnFunct)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(returnFunct)];
  *(void**)(paramBuffer) = returnFunct;

  asm volatile ("mov r0, #2\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3857\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void SystemInterface::versions(ProvidedVersionInstaller* arg1, RequiredVersionInstaller* arg2)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)+sizeof(arg2)];
  *(ProvidedVersionInstaller**)(paramBuffer) = arg1;
  *(RequiredVersionInstaller**)(paramBuffer+sizeof(arg1)) = arg2;

  asm volatile ("mov r0, #3\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3857\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

Process* SystemInterface::getThisProcess()
{
  Process* ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #4\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3857\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new Process( (void*)ret );

  return ret;
}

void SystemInterface::killThisProcess()
{
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #5\n\t"
		"mov r1, %1\n\t"
		"swi #3857\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

Process* SystemInterface::buildProcessFromObjectFile(PersistentMemoryObject* arg1, int argc, char** argv)
{
  Process* ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)+sizeof(argc)+sizeof(argv)];
  *(PersistentMemoryObject**)(paramBuffer) = (PersistentMemoryObject*)(arg1->kernelObj);
  *(int*)(paramBuffer+sizeof(arg1)) = argc;
  *(char***)(paramBuffer+sizeof(arg1)+sizeof(argc)) = argv;

  asm volatile ("mov r0, #6\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3857\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new Process( (void*)ret );

  return ret;
}

Disk* SystemInterface::rootDisk()
{
  Disk* ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #7\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3857\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new Disk( (void*)ret );

  return ret;
}

int SystemInterface::isObjectFile(PersistentMemoryObject* arg1)
{
  int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(PersistentMemoryObject**)(paramBuffer) = (PersistentMemoryObject*)(arg1->kernelObj);

  asm volatile ("mov r0, #8\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3857\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

OutputStream* SystemInterface::outputStream(ProxiableObject* arg1)
{
  OutputStream* ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(ProxiableObject**)(paramBuffer) = (ProxiableObject*)(arg1->kernelObj);

  asm volatile ("mov r0, #9\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3857\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new OutputStream( (void*)ret );

  return ret;
}

InputStream* SystemInterface::inputStream(ProxiableObject* arg1)
{
  InputStream* ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(ProxiableObject**)(paramBuffer) = (ProxiableObject*)(arg1->kernelObj);

  asm volatile ("mov r0, #10\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3857\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new InputStream( (void*)ret );

  return ret;
}

int SystemInterface::pageSize()
{
  int ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #11\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3857\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

void SystemInterface::controlSimpleInstruments(int argc, char** argv)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(argc)+sizeof(argv)];
  *(int*)(paramBuffer) = argc;
  *(char***)(paramBuffer+sizeof(argc)) = argv;

  asm volatile ("mov r0, #12\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3857\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

SystemInterface::~SystemInterface() {}
#include "ClassInstaller.h"
DEFINECLASS("SystemInterface",SystemInterface,ProxiableObject,0,0);

