#include "common.h"
#include "exception.h"
#include "ApplicationProcess.h"
#include "Semaphore.h"
#include "SystemInterface.h"
#include "Thread.h"


#define MAX_THREADS 65535

Thread::Thread()
{
  AppProcObj = new ApplicationProcess(_entry,
				      StandardSystemInterface->getThisProcess()->currentDomain(),
				      (int)this,0, normal);
  join_sem=new Semaphore(0,MAX_THREADS);
  mutex=new Semaphore(1,MAX_THREADS);
  join_sem_counter=0;
  dead=0;
}

void Thread::Start() 
{
  AppProcObj->ready();
}


// WaitForProcess(other)
// other is the joined thread.
// calling thread calls WaitForProcess
int Thread::Join() 
{
  int already_dead=0;

  mutex->P();
  if (!dead) 
    join_sem_counter++;
  else
    already_dead=1;
  mutex->V();

  if (!already_dead)
    join_sem->P();

  return 0;

  // The following doesn't quite work. If AppProcObj is dead, the
  // kernel will not be able to find it and exception thrown if that's
  // the case. Thus, use semaphore instead.

  // return Interface::thisProcess()->waitForProcess(AppProcObj);
}

void Thread::_entry(void *args) 
{
  int status=0;
  Thread* thrd_ptr;
  asm volatile ("str r0, %0" 
		: "=m"(thrd_ptr)
		::"r0");

  thrd_ptr->Run();
  thrd_ptr->_die();
}

void Thread::_die() {

  mutex->P();
  for (int i=0; i<join_sem_counter; i++)
    join_sem->V();
 
  dead=1;
  mutex->V();
  AppProcObj->die(0);
}

Thread::~Thread() {
  
}
