
#include "TimingInterface.h"
#include "exception.h"

TimingInterface::TimingInterface(void* k) :ProxiableObject((void*)0) {kernelObj=k;}

TimingInterface::TimingInterface() :ProxiableObject((void*)0)
{
  kernel_exception::EXCODE code;
  asm volatile (
		"mov r0, #0\n\t"
		"mov r3, %1\n\t"
		"swi #3877\n\t"
		"str r0, %0"
		: "=m"(code) :"r"(&kernelObj) : "r0", "r1", "r2", "r3" 
		);
  kernel_exception::throw_exception(code);
}

void TimingInterface::nullCall()
{
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #2\n\t"
		"mov r1, %1\n\t"
		"swi #3877\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void TimingInterface::intParam(int arg1)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(int*)(paramBuffer) = arg1;

  asm volatile ("mov r0, #3\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3877\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void TimingInterface::twoIntParams(int arg1, int arg2)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)+sizeof(arg2)];
  *(int*)(paramBuffer) = arg1;
  *(int*)(paramBuffer+sizeof(arg1)) = arg2;

  asm volatile ("mov r0, #4\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3877\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void TimingInterface::ptrParam(char* arg1)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(char**)(paramBuffer) = arg1;

  asm volatile ("mov r0, #5\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3877\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void TimingInterface::proxyParam(TimingInterface* arg1)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(TimingInterface**)(paramBuffer) = (TimingInterface*)(arg1->kernelObj);

  asm volatile ("mov r0, #6\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3877\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

TimingInterface* TimingInterface::returnOldProxy()
{
  TimingInterface* ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #7\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3877\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new TimingInterface( (void*)ret );

  return ret;
}

TimingInterface* TimingInterface::returnNewProxy()
{
  TimingInterface* ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #8\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3877\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new TimingInterface( (void*)ret );

  return ret;
}

TimingInterface* TimingInterface::returnOldProxyAsterisk()
{
  TimingInterface* ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #9\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3877\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new TimingInterface( (void*)ret );

  return ret;
}

TimingInterface* TimingInterface::returnNewProxyAsterisk()
{
  TimingInterface* ret;
  kernel_exception::EXCODE code;
  
  asm volatile ("mov r0, #10\n\t"
		"mov r1, %1\n\t"
		"mov r3, %2\n\t"
		"swi #3877\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new TimingInterface( (void*)ret );

  return ret;
}

void TimingInterface::setOutCallInterface(TimingInterface* arg1)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(TimingInterface**)(paramBuffer) = (TimingInterface*)(arg1->kernelObj);

  asm volatile ("mov r0, #11\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3877\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void TimingInterface::setOutProxyInterface(TimingInterface* arg1)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(TimingInterface**)(paramBuffer) = (TimingInterface*)(arg1->kernelObj);

  asm volatile ("mov r0, #12\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3877\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

unsigned int TimingInterface::outNullCalls(unsigned int arg1)
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(unsigned int*)(paramBuffer) = arg1;

  asm volatile ("mov r0, #13\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3877\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

unsigned int TimingInterface::outIntParams(unsigned int arg1)
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(unsigned int*)(paramBuffer) = arg1;

  asm volatile ("mov r0, #14\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3877\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

unsigned int TimingInterface::outTwoIntParams(unsigned int arg1)
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(unsigned int*)(paramBuffer) = arg1;

  asm volatile ("mov r0, #15\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3877\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

unsigned int TimingInterface::outPtrParams(unsigned int arg1)
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(unsigned int*)(paramBuffer) = arg1;

  asm volatile ("mov r0, #16\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3877\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

unsigned int TimingInterface::outProxyParams(unsigned int arg1, unsigned int arg2)
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)+sizeof(arg2)];
  *(unsigned int*)(paramBuffer) = arg1;
  *(unsigned int*)(paramBuffer+sizeof(arg1)) = arg2;

  asm volatile ("mov r0, #17\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3877\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

unsigned int TimingInterface::outRefUnref(unsigned int arg1)
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(unsigned int*)(paramBuffer) = arg1;

  asm volatile ("mov r0, #18\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3877\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

unsigned int TimingInterface::outReturnOldProxyAsterisks(unsigned int arg1)
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(unsigned int*)(paramBuffer) = arg1;

  asm volatile ("mov r0, #19\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3877\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

unsigned int TimingInterface::outReturnNewProxyAsterisks(unsigned int arg1)
{
  unsigned int ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)];
  *(unsigned int*)(paramBuffer) = arg1;

  asm volatile ("mov r0, #20\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3877\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

TimingInterface::~TimingInterface() 
{
  kernel_exception::EXCODE code;
  asm volatile (
		"mov r0, #1\n\t"
		"mov r1, %1\n\t"
		"swi $3877\n\t"
		"str r0, %0"
		: "=m" (code) :"r" (kernelObj) : "r0", "r1", "r2", "r3"
		);
  kernel_exception::throw_exception(code);
}
#include "ClassInstaller.h"
DEFINECLASS("TimingInterface",TimingInterface,ProxiableObject,0,0);

