#include "MemoryObjectCache.h"
#include "RecordStream.h"

#include "VMInterface.h"
#include "exception.h"

VMInterface::VMInterface(void* k)  {kernelObj=k;}

void VMInterface::test(char* message)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(message)];
  *(char**)(paramBuffer) = message;

  asm volatile ("mov r0, #2\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3860\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

MemoryObjectCache* VMInterface::createCache(RecordStream* arg1, char* cacheType)
{
  MemoryObjectCache* ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)+sizeof(cacheType)];
  *(RecordStream**)(paramBuffer) = (RecordStream*)(arg1->kernelObj);
  *(char**)(paramBuffer+sizeof(arg1)) = cacheType;

  asm volatile ("mov r0, #3\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3860\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  ret = new MemoryObjectCache( (void*)ret );

  return ret;
}

char* VMInterface::addToDomain(RecordStream* arg1, ProtectionLevel p)
{
  char* ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(arg1)+sizeof(p)];
  *(RecordStream**)(paramBuffer) = (RecordStream*)(arg1->kernelObj);
  *(ProtectionLevel*)(paramBuffer+sizeof(arg1)) = p;

  asm volatile ("mov r0, #4\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3860\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

void VMInterface::setupCOW(RecordStream* origin, RecordStream* copy)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(origin)+sizeof(copy)];
  *(RecordStream**)(paramBuffer) = (RecordStream*)(origin->kernelObj);
  *(RecordStream**)(paramBuffer+sizeof(origin)) = (RecordStream*)(copy->kernelObj);

  asm volatile ("mov r0, #5\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3860\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

void VMInterface::networkTest(char* to)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(to)];
  *(char**)(paramBuffer) = to;

  asm volatile ("mov r0, #6\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3860\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

char* VMInterface::openDVM(char* filename)
{
  char* ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(filename)];
  *(char**)(paramBuffer) = filename;

  asm volatile ("mov r0, #7\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3860\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

void VMInterface::closeDVM(char* filename)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(filename)];
  *(char**)(paramBuffer) = filename;

  asm volatile ("mov r0, #8\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"swi #3860\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
}

char* VMInterface::createSharedRegion(unsigned int size)
{
  char* ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0+sizeof(size)];
  *(unsigned int*)(paramBuffer) = size;

  asm volatile ("mov r0, #9\n\t"
		"mov r1, %1\n\t"
		"mov r2, %2\n\t"
		"mov r3, %3\n\t"
		"swi #3860\n\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  
  return ret;
}

VMInterface::~VMInterface() {}
#include "ClassInstaller.h"
DEFINECLASS("VMInterface",VMInterface,Null,0,0);

