#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
#include "Assert.h"
#include "BufferedOutputStream.h"
#include "Byte.h"
#include "String.h"


#ifdef __GNUC__
#pragma implementation
#endif

BufferedOutputStream::BufferedOutputStream( OutputStream * sink,
		    int flushOnNewline, int bufferSize )
{
	Assert( sink != 0 );
	_sink = sink;
	_flushBuffer = new char[ bufferSize ];
	Assert( _flushBuffer != 0 );
	_bufferSize = bufferSize;
	_flushOnNewline = flushOnNewline;
	_bytesInBuffer = 0;
}

BufferedOutputStream::~BufferedOutputStream()
{
	flush();

	Assert( _sink != 0 );
	Assert( _flushBuffer != 0 );

	_sink = 0;
	delete _flushBuffer;
}

OutputStream &
BufferedOutputStream::flush()
{
	if ( _bytesInBuffer != 0 ) {
		(void) _sink->write( _flushBuffer, _bytesInBuffer );
		_bytesInBuffer = 0;
	}

	(void) _sink->flush();

	return( *this );
}

int
BufferedOutputStream::write( const char * buffer, int count )
{
	if ( ( _bytesInBuffer != 0 ) &&
				( _bytesInBuffer + count > _bufferSize ) ) {
		(void) _sink->write( _flushBuffer, _bytesInBuffer );
		_bytesInBuffer = 0;
	}
	
	int retcount = count;

	while ( count > _bufferSize ) {
		_sink->write( buffer, _bufferSize );
		buffer += _bufferSize;
		count -= _bufferSize;
	}

	ByteCopy( (void *) buffer, (void *) &_flushBuffer[ _bytesInBuffer ],
									count );

	_bytesInBuffer += count;

	if ( _flushOnNewline )
	{
		while (count-- > 0)
		{
			if (*buffer++ == '\n')
			{
			        (void) flush();
				break;
			}
		}
	}

	return( retcount );
}


#include "ClassInstaller.h"
DEFINECLASS("BufferedOutputStream",BufferedOutputStream,OutputStream,0,0);
