#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

// #define DEBUG
#include "Debug.h"
#include "Process.h"
#include "ProcessStar.h"
#include "ThisProcess.h"
#include "setjmp.h"


#ifdef __GNUC__
#pragma implementation
#endif

//  cloneThisProcess() uses this variable to store the registers to be
//  restored after the child begins execution.
static jump_buf cloneEnv;

char * getDummyStackPointer()
{
    // use dynamic arrays to figure out how the stack grows (i.e. up or down)
    // also use them to figure out a safe place in the stack to use on 
    // initial entry

    int d1[ 256 ];
    int * current = d1;

    // make sure the stack did not overflow
    current[0] = 1;
    current[255] = 1;

    int * next = 0;
    if ( *current ) {
	int d2[ 256 ];
	next = d2;

	// make sure the stack did not overflow
	next[0] = 1;
	next[255] = 1;
    }
    Assert( next != 0 );

    // check which way the stack grows and select a dummy sp
    int * dummysp = ( next < current ? current : next );
    
    // sanity check
#ifdef SPARC
    Assert( dummysp == current );
#endif

    // align to be sure
    return ( char * ) ( ( unsigned ) dummysp & ~0x7 );
}

void 
cloneAssist ()
{
    //  thisProcess()->clone() causes the new process to begin execution
    //  here.  We have to restore the registers.

    longjmp1 (&cloneEnv, 1);
}

ProcessRef cloneThisProcess ()
{
    //  Create and execute a new process with a copy of the current domain
    //  and begin execution in exactly the same environment as the current
    //  process.  The current process will return a pointer to the new process
    //  when the current process exits this function.  The new process will
    //  return 0 when the new process exits this function.  Only the
    //  writeable unshared memory objects in the current domain are copied
    //  to the new domain.  The other memory objects are directly inserted
    //  into the new domain.
    //  Save the registers.

    if (setjmp1 (&cloneEnv) == 0)
    {
    	//  If this is the parent process, perform the clone.  The cloned child
    	//  should start running at cloneAssist().  Return the child pointer.
	//  Reserve some stack space for the child.  Put the stack pointer
	//  in the middle of the stack because we don't know whether the
	//  stack grows upwards or downwards (ie. we want machine
	//  independence).

        char * stackPointer = getDummyStackPointer();
	Debug << thisProcess() << "->clone()\n" << eor;
	ProcessStar child = thisProcess () -> clone 
            ( (const char *) cloneAssist, stackPointer );
	Debug << thisProcess() << "->clone() done, calling " 
	      << child << "->ready()\n" << eor;
	child -> ready ();
	Debug << child << "->ready() done\n" << eor;
	return child;
    }
    else {

    	//  If this is the child process, return 0.
	return 0;
    }
}
