#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * _Start.cc
 */

#include "Assert.h" 
#include "BufferedOutputStream.h"
#include "Class.h"
#include "ClassInstaller.h"
#include "ConstructorInstaller.h"
#include "InputStream.h"
#include "NameServer.h"
#include "OutputStream.h"
#include "ThisProcess.h"
#include "ProxyTable.h"
#include "VersionInstaller.h"
#include "NewAndDelete.h"


#ifdef __GNUC__
#pragma implementation
#endif

/*
 * Standard global variables available to all applications
 */
SystemInterface * StandardSystemInterface;
OutputStream * StandardOutput;
InputStream * StandardInput;
NameServer * StandardNameServer;

OutputStream &
Console()
{
    return ( *StandardOutput );
}

/*
 *
 */

class Class;

typedef void (*PFV)();

extern "C" void _defaultReturn();
//extern "C" void _main();

PFV ConstructProxyPtr;

REQUIREDVERSION( TableOnlyProxy );

extern void initializeNewAllocator();


void
initializeStaticallyConstructedObjects()
{
	typedef void (*PFV)();
#ifdef CFRONT
	extern PFV _ctors[];
	for( PFV * pf = _ctors; *pf != 0; pf++ ) {
		(**pf)();
	}
#else
	extern PFV __CTOR_LIST__[];

	/*
	 * The new "collect" generates CTOR- and DTOR-LISTs with a
	 * leading count (they're still null-terminated).
	 */
	for( PFV * pf = &__CTOR_LIST__[1]; *pf != 0; pf++ ) {
		(**pf)();
	}
#endif
}


char **environ;

extern "C" void
_Start( SystemInterface * theInterface )
{
	initializeNewAllocator();

	ProxiableObject::initialize();

	StandardSystemInterface = theInterface;
	StandardSystemInterface->reference();

	ConstructProxyPtr = (PFV) StandardSystemInterface->
			    	initialCall( (void *)&_defaultReturn );

	/*
	 * Call the static constructors.  Do this after the initial call
	 *   because some of them may construct proxies.
	 */
	initializeStaticallyConstructedObjects();

	/*
	 * Check/install the versions.  This assumes the static constructors
	 *   have been called.
	 */
	StandardSystemInterface->versions( ProvidedVersionInstaller::list(),
			    RequiredVersionInstaller::list() );

	/*
	 * Get the standard proxies
	 */
	Process * thisProc = thisProcess();

	StandardNameServer = thisProc->domainNameServer();

	StandardOutput = (OutputStream *)(ProxiableObject *)StandardNameServer->
			lookup( "StandardOutput", "OutputStream" );

	StandardInput = (InputStream *)(ProxiableObject *)StandardNameServer->
			lookup( "StandardInput", "InputStream" );

	/*
	 * Make the StandardOutput to be locally buffered.
	 * Reference it once so it won't get deleted until we die.
	 */

//	StandardOutput = new BufferedOutputStream( StandardOutput, 1 );
//	StandardOutput->reference();


	/*
	 * Initialize the classes, proxy tables, and constructors
	 *    (in that order!)
	 */
        ClassClass = StandardNameServer->lookup( "Class", "Class" );
        ObjectClass = StandardNameServer->lookup( "Object", "Class" );

	ClassInstaller::install();
	ObjectClass->bind( StandardNameServer );

	ProxyTableInstaller::install();

	ConstructorInstaller::install();
}

extern void
Halt()
{
	/*
	 * Avoid thisProcess()->die() because that may allocate a proxy
	 *  where we don't need one and prevent the process object from
	 *  going away.  This is especially a problem if the current
	 *  process began in a different domain than this one.
	 */
	StandardSystemInterface->killThisProcess();
}

/*
 * The following functions are assembler assists
 */
extern "C" void
_KillThisProcess( unsigned int )
{
	/* there's an error code passed in if we ever want to use it */
	Halt();
}

/*
 * _PostConstructProxy is called by proxy constructor stubs after creating the
 *    new object.  If the oldThis is zero, that means somebody did a 'new'
 *    directly.  If it is nonZero but points to a live object, that means
 *    somebody is trying to "subclass" a proxied object; set the '_real'
 *    pointer to point to the proxy.   If it does not point to a live
 *    object, then someone tried to allocate their own space for the proxy,
 *    an error.
 */

/*
 * Hack to set _real which is protected.  Notice how easily compile-time
 *   checks can be circumvented!
 */
class SubProxiableObject : public ProxiableObject {
public:
	void setReal( ProxiableObject * real ) { _real = real; };
};

class vtable_entry;

extern "C" ProxiableObject *
_PostConstructProxy( ProxiableObject * oldThis, ProxiableObject * newThis,
					vtable_entry * vtbl )
{
	if ( oldThis != 0 )
	{
		// Console() << "oldThis != 0\n" << eor;
		/*
		 * Some memory was already assigned for this object.
		 * This can be called either by a sub-class constructor
		 *   or if the user assigns his own space for the object.
		 * Call the ProxiableObject constructor on the old 'this'
		 *   to initialize the local portions of the object, set
		 *   the vtbl to the correct class, and set the '_real'
		 *   pointer to the constructed proxy.
		 * Stub functions will then forward calls to the proxy.
		 */
		(void) new ( (void *) oldThis ) ProxiableObject();
		/*
		 * The vtable pointer is always at the end of the object
		 *  structure.
		 */
		*((vtable_entry **) (((char *) oldThis) + sizeof(Object) -
			sizeof(vtable_entry *))) = vtbl;


		((SubProxiableObject *)oldThis)->setReal( newThis );

		return( oldThis );
	}
	return( newThis );
}

/*
 * Used by true virtual functions.
 */
extern "C" char
#ifdef CFRONT
__pure_virtual_called ()
#else
abort ()
#endif
{
	Assert (NOTREACHED);
	return (0);
}
