#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ExecInfo.cc
 */

#include "Assert.h"
#include "Byte.h"
#include "ExecInfo.h"
#include "FileDescriptor.h"
#include "String.h"
#include "UnixApplication.h"


#ifdef __GNUC__
#pragma implementation
#endif

extern "C" int putenv(char *);

ExecInfo * UA_ExecInfoEncode(char * environ[])
{
	/* compute environment length */

	int envsize = 0;
    char **s;
	for (s = environ; *s; s++) {
		envsize = strlen(*s) + 1;
	}
	int length = sizeof(ExecInfo) + envsize;
	length += sizeof(int) - (length % sizeof(int));
	ExecInfo * execInfo = (ExecInfo *) new char[length];
	ByteCopy(EXEC_SIGNATURE, execInfo->signature, 8);
	execInfo->length = length;
	execInfo->negativePid = UA_negativePid;
	execInfo->lastChild = UA_lastChild;
	execInfo->sigmask = UA_sigmask;
	execInfo->umask = UA_umask;
	execInfo->auid = UA_auid;
	execInfo->euid = UA_euid;
	execInfo->egid = UA_egid;
	execInfo->gid = UA_gid;
	execInfo->uid = UA_uid;
	ByteCopy((char *) UA_FileDescriptorTable, 
		 (char *) &(execInfo->fileDescriptorTable),
		 sizeof(FileDescriptorTable));
	char * p = &execInfo->env[0];
	execInfo->envCount = 0;
	for (s = environ; *s; s++ ) {
		strcpy( p, *s );
		p += strlen( *s ) + 1;
		execInfo->envCount++;
	}
	return execInfo;
}

void UA_ExecInfoDecode(ExecInfo * execInfo)
{
	/* compute environment length */

	Assert(ByteCompare(EXEC_SIGNATURE, execInfo->signature, 8) == 0);
	UA_negativePid = execInfo->negativePid;
	UA_lastChild = execInfo->lastChild;
	UA_sigmask = execInfo->sigmask;
	UA_umask = execInfo->umask;
	UA_auid = execInfo->auid;
	UA_euid = execInfo->euid;
	UA_egid = execInfo->egid;
	UA_gid = execInfo->gid;
	UA_uid = execInfo->uid;
	UA_FileDescriptorTable = new FileDescriptorTable();
	Assert(UA_FileDescriptorTable != 0);
	UA_FileDescriptorTable = &(execInfo->fileDescriptorTable);
/*
	ByteCopy((char *) &(execInfo->fileDescriptorTable),
		 (char *) UA_FileDescriptorTable, 
		 sizeof(FileDescriptorTable));
 */
	char * p = &execInfo->env[0];
	for (int i = 0; i < execInfo->envCount; i++) {
		putenv(p);
		p += strlen( p ) + 1;
	}
}

