#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
 
#include "Assert.h"
#include "ExecInfo.h"
#include "FileDescriptor.h"
#include "FileSystem.h"
#include "FileSystemHints.h"
#include "FileSystemInterface.h"
#include "Class.h"
#include "ClassStar.h"
#include "InputStream.h"
#include "NameServer.h"
#include "OutputStream.h"
#include "UnixApplication.h"
extern "C" {
#include <errno.h>
}

#ifdef __GNUC__
#pragma implementation
#endif


int UA_debug = 1;
int UA_initialized = 0;
int UA_errno = 0;

int UA_negativePid = -1;
int UA_lastChild = -1;
int UA_sigmask = 0;
int UA_umask = 0;
int UA_auid = 0;
int UA_egid = 0;
int UA_euid = 0;
int UA_gid = 0;
int UA_uid = 0;

exitproc_t UA_old_exitproc;

extern ExecInfo * UA_ExecInfo;
ClassStar UA_PersistentArrayStreamClass;

FileDescriptorTable * UA_FileDescriptorTable = 0;
FileSystemInterface * UA_FileSystemInterface = 0;

int UA_notImplemented()
{
	UA_Debug << "UA_notImplemented()\n" << eor;
	errno = ENOSYS;
	return -1;
}

void UA_init()
{
	/*
	 * Must be called before using any Unix functions!
	 * Need to reinitialize stdin, stdout and stderr again because
	 * StandardInput/Output may be set after the static constructors for
	 * stdin/stdout/stderrAdapter have been called.
	 */

	UA_Debug << "UA_init() called\n" << eor;
	
	if (UA_initialized) {
		UA_Debug << "UA_init() already called\n" << eor;
		return;
	}

	UA_old_exitproc = _exitproc;
	_exitproc = &_exit;

	UA_FileSystemInterface = (FileSystemInterface *)StandardNameServer->lookup
    ("fileSystemInterface", "FileSystemInterface");
	Assert(UA_FileSystemInterface != 0);
	UA_PersistentArrayStreamClass = StandardNameServer->lookup
    ("PersistentArrayStream", "Class");
	Assert(UA_PersistentArrayStreamClass != 0);
	UA_initialized = 1;
	UA_errno = 0;
	errno = 0;

	if (UA_ExecInfo != 0) {
		UA_Debug << "UA_ExecInfo not null\n" << eor;
		UA_ExecInfoDecode( UA_ExecInfo );
		UA_Debug << "UA_ExecInfoDecode done\n" << eor;
		return;
	}

	UA_Debug << "UA_FileDescriptorTable being initialized\n" << eor;

	int f;
	FileDescriptor * fd;
	UA_FileDescriptorTable = new FileDescriptorTable();
	Assert(UA_FileDescriptorTable != 0);

	f = UA_FileDescriptorTable->allocFileDescriptor();
	Assert(f == 0);
	fd = UA_FileDescriptorTable->fileDescriptor(f);
#ifndef STAR
	StandardInput->reference();
#endif // STAR
	fd->init(StandardInput, 0, InputStreamFileDescriptorClass);

	f = UA_FileDescriptorTable->allocFileDescriptor();
	Assert(f == 1);
	fd = UA_FileDescriptorTable->fileDescriptor(f);
#ifndef STAR
	StandardOutput->reference();
#endif // STAR
	fd->init(StandardOutput, 0, OutputStreamFileDescriptorClass);

	f = UA_FileDescriptorTable->allocFileDescriptor();
	Assert(f == 2);
	fd = UA_FileDescriptorTable->fileDescriptor(f);
#ifndef STAR
	StandardOutput->reference();
#endif // STAR
	fd->init(StandardOutput, 0, OutputStreamFileDescriptorClass);
	
	UA_Debug << "UA_init done\n" << eor;

	return;
}

void UA_translateErrno()
{
	/*
	 * Translate the Choices filesystem error code into the appropriate
	 * Unix error code, and stick it into errno.
	 */

	Assert (UA_errno != 0);
	switch (UA_errno) {
	      case DirectoryInvalid: errno = ENOTDIR; break;
	      case KeyExists: errno = EEXIST; break;
	      case KeyNotFound: errno = ENOENT; break;
	      case NotAProgram: errno = ENOEXEC; break;
	      case FileAccessViolation: errno = EACCES; break;
	      case DifferentContainers: errno = EXDEV; break;
	      case UnknownNFSOp: errno = EISDIR; break;
		/* FIXTHIS: Need more error codes! */
	      default: 
		UA_Debug << "UA_translateErrno " << UA_errno 
			<< " unknown\n" << eor;
		errno = EPERM;
	}
}

extern "C" {
void hintstring( unsigned hints, char * hintstr );
}

void hintstring( unsigned hints, char * hintstr )
{
	unsigned rhints;
	rhints = hints >> 16;
	*hintstr++ = (h_lcache(rhints) ? 'l' : '-');
	*hintstr++ = (h_single(rhints) ? 's' : '-');
	*hintstr++ = (h_large(rhints) ? 'h' : '-');
	*hintstr++ = (h_random(rhints) ? 'r' : '-');
	*hintstr++ = (h_temp(rhints) ? 't' : '-');
	*hintstr++ = ' ';
	*hintstr++ = (h_lcache(hints) ? 'l' : '-');
	*hintstr++ = (h_single(hints) ? 's' : '-');
	*hintstr++ = (h_large(hints) ? 'h' : '-');
	*hintstr++ = (h_random(hints) ? 'r' : '-');
	*hintstr++ = (h_temp(hints) ? 't' : '-');
	*hintstr = 0;
}

