#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "FileSystemInterface.h"
#include "PersistentMemoryObject.h"
#include "PersistentMemoryObjectStar.h"
#include "SysCalls.h"
#include "UnixApplication.h"
#include "OutputStream.h"
#include "UNIXFile.h"
extern "C" {
#include <sys/errno.h>
}

#ifdef __GNUC__
#pragma implementation
#endif


int access(const char * path, int mode)
{
	UA_Debug << "access(" << path << ", " << mode << ")\n" 
		<< eor;
	if (!UA_initialized) UA_init();
	if (mode & ~(R_OK | W_OK | X_OK)) {
		errno = EINVAL;
		UA_Debug << "returns -1 errno " << errno << "\n" << eor;
		return -1;
	}
    Assert(PersistentMemoryObjectClass != 0);
	PersistentMemoryObjectStar so = (PersistentMemoryObject *)UA_FileSystemInterface->open 
    ((char *) path, PersistentMemoryObjectClass, UA_errno);
	if (UA_errno || so == 0) { 
		UA_translateErrno (); 
		UA_Debug << "returns -1 errno " << errno 
			<< " UA_errno " << UA_errno << "\n" << eor;
		return -1; 
	}
	if (mode == F_OK) {
		UA_Debug << "returns 0\n" << eor;
		return 0;
	}
	UNIXFileInfo fileInfo;
        so->info(&fileInfo);
	int realmode = fileInfo.mode;
	int i = ((realmode >> UNIXOwnerShift) & mode) |
		((realmode >> UNIXGroupShift) & mode) |
		((realmode >> UNIXPublicShift) & mode);
	if (i != mode) {
		errno = EACCES;
		UA_Debug << "returns -1 errno " << errno << "\n" << eor;
		return -1;
	}
	UA_Debug << "returns 0\n" << eor;
	return 0;
}
