#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "FileDescriptor.h"
#include "OutputStream.h"
#include "SysCalls.h"
#include "UnixApplication.h"
#ifndef STAR
#include "PersistentMemoryObject.h"
#endif

extern "C" {
#include <sys/errno.h>
}

#ifdef __GNUC__
#pragma implementation
#endif


int dup2(int fd1, int fd2)
{
	UA_Debug << "dup2(" << fd1 << ", " << fd2 << ")\n" << eor;
	if (!UA_initialized) UA_init();
	FileDescriptor *filedes1 = UA_FileDescriptorTable->fileDescriptor(fd1);
	if (filedes1 == 0 || !filedes1->used()) { 
		errno = EBADF; 
		UA_Debug << "returns -1 errno " << errno << "\n" << eor;
		return -1; 
	}
	FileDescriptor *filedes2 = UA_FileDescriptorTable->fileDescriptor(fd2);
	if (filedes2 == 0) {
		errno = EBADF; 
		UA_Debug << "returns -1 errno " << errno << "\n" << eor;
		return -1; 
	}
	/* 
	 * should move the following to FileDescriptorTable to
         * avoid concurrency problem if more than one thread allowed
	 * ok for now
	 */
	if (filedes2->used()) {
		filedes2->close();
	}
    Assert(!filedes2->used());
#ifndef STAR
	PersistentMemoryObject *pmo = filedes2->persistentMemoryObject();
	ProxiableObject *po = filedes2->proxiableObject();
	if (pmo) pmo->reference();
	if (po) po->reference();	
#endif // STAR
	filedes2->init(filedes1);
	UA_Debug << "returns " << fd2 << "\n" << eor;
	return fd2;
}
