#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "ExecInfo.h"
#include "FileDescriptor.h"
#include "FileSystemInterface.h"
#include "NameServer.h"
#include "OutputStream.h"
#include "PersistentMemoryObject.h"
#include "PersistentMemoryObjectStar.h"
#include "SysCalls.h"
#include "SystemInterface.h"
#include "Process.h"
#include "UnixApplication.h"
extern "C" {
#include <sys/errno.h>
}

#ifdef __GNUC__
#pragma implementation
#endif



int execve(char * path, char * argv[], char * envp[])
{
	UA_Debug << "execve(" << asHex((int) path) << ", "
		<< asHex((int) argv) << ", "
		<< asHex((int) envp) << ")\n" << eor;

	if (!UA_initialized) UA_init();
	PersistentMemoryObjectStar pmo = (PersistentMemoryObject *)UA_FileSystemInterface->open
		(path, PersistentMemoryObjectClass, UA_errno);
	if (UA_errno || pmo == 0) { 
		UA_translateErrno (); 
		UA_Debug << "returns -1 errno " << errno 
			<< " UA_errno " << UA_errno << "\n" << eor;
		return -1; 
	}
	if (!StandardSystemInterface->isObjectFile(pmo)) {
		errno = ENOEXEC;
		UA_Debug << "returns -1 errno " << errno << "\n" << eor;
		return -1;
	}

	UA_FileDescriptorTable->closeOnExec();

	FileDescriptor *filedes = UA_FileDescriptorTable->fileDescriptor(0);
    Assert( StandardNameServer != 0 );
	if (filedes == 0) {
		StandardNameServer->unbind("StandardInput");
	} else {
		StandardNameServer->bind
			("StandardInput", filedes->proxiableObject());
	}
	filedes = UA_FileDescriptorTable->fileDescriptor(1);
	if (filedes == 0) {
		StandardNameServer->unbind("StandardOutput");
	} else {
		StandardNameServer->bind
			("StandardOutput", filedes->proxiableObject());
	}

	int argc = 0;
	for (char ** arg = argv; *arg; arg++)
		argc++;

	ExecInfo * execInfo = UA_ExecInfoEncode(envp);

    // Francis TODO: Fix
	// ObjectClass->unbind(StandardNameServer);
	// ClassInstaller::uninstall();
	
    // Francis: TODO fix execInfo isn't being used
	Process *process = StandardSystemInterface->buildProcessFromObjectFile
		(pmo, argc, argv);
    process->ready();
	Assert(UA_errno != 0);
	UA_translateErrno (); 
	UA_Debug << "returns -1 errno " << errno 
		<< " UA_errno " << UA_errno << "\n" << eor;
	delete execInfo;
	return -1; 
}
