#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "FileDescriptor.h"
#include "FileSystemInterface.h"
#include "OutputStream.h"
#include "SysCalls.h"
#include "UnixApplication.h"
extern "C" {
#include <sys/errno.h>
}

#ifdef __GNUC__
#pragma implementation
#endif

int fcntl(int fd, int cmd, int arg)
{
	int result;

	UA_Debug << "fcntl(" << fd << ", " << cmd << ", " 
		<< asHex((int) arg) << ")\n" << eor;
	if (!UA_initialized) UA_init();
	FileDescriptor *filedes = UA_FileDescriptorTable->fileDescriptor(fd);
	if (filedes == 0 || !filedes->used()) { 
		errno = EBADF; 
		UA_Debug << "returns -1 errno " << errno << "\n" << eor;
		return -1; 
	}
	switch(cmd) {
        case F_DUPFD:
          {
              if (arg < 0) { 
                  errno = EBADF; 
                  UA_Debug << "returns -1 errno " << errno 
                           << "\n" << eor;
                  return -1; 
              }
              int newfd = UA_FileDescriptorTable->allocFileDescriptor(arg);
              FileDescriptor * newfiledes = 
              UA_FileDescriptorTable->fileDescriptor(newfd);
              newfiledes->init(filedes);
              UA_Debug << "returns " << newfd << "\n" << eor;
              return newfd;
          }
        case F_GETFD:
            result = filedes->flag(FDCloseOnExec) ? 1 : 0;
            UA_Debug << "returns " << result << "\n" << eor;
            return result;
        case F_SETFD:
            filedes->setFlag(FDCloseOnExec, arg);
            result = 0;
            UA_Debug << "returns " << result << "\n" << eor;
            return result;
        case F_GETFL:
            result = filedes->fileFlags();
            UA_Debug << "returns " << result << "\n" << eor;
            return result;
        case F_SETFL:
            filedes->setFileFlags(arg);
            result = 0;
            UA_Debug << "returns " << result << "\n" << eor;
            return result;
	}
	return UA_notImplemented();
}
