#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "FileDescriptor.h"
#include "OutputStream.h"
#include "SysCalls.h"
#include "UnixApplication.h"
#include "UNIXFile.h"
extern "C" {
#include <unistd.h>
#include <sys/errno.h>
}

#ifdef __GNUC__
#pragma implementation
#endif


int fstat(int fd, struct stat * buf) 
{
	UA_Debug << "fstat(" << fd << ")\n" << eor;
	if (!UA_initialized) UA_init();
	FileDescriptor * filedes = UA_FileDescriptorTable->fileDescriptor(fd);
	if (filedes == 0 || !filedes->used()) {
		errno = EBADF; 
		UA_Debug << "returns -1 errno " << errno << "\n" << eor;
		return -1; 
	};
	UNIXFileInfo fileInfo;
	int result = filedes->stat(&fileInfo);
	if (result < 0) { 
		errno = EPERM; 
		UA_Debug << "returns -1 errno " << errno << "\n" << eor; 
		return result; 
	}
	buf->st_dev = fileInfo.device;
	buf->st_ino = fileInfo.inumber;
	buf->st_mode = fileInfo.mode;
	buf->st_nlink = fileInfo.links;
	buf->st_uid = fileInfo.uid;
	buf->st_gid = fileInfo.gid;
	buf->st_rdev = fileInfo.rdevice;
	buf->st_size = fileInfo.size;
	buf->st_atime = fileInfo.atime;
	buf->st_spare1 = fileInfo.aspare;
	buf->st_mtime = fileInfo.mtime;
	buf->st_spare2 = fileInfo.mspare;
	buf->st_ctime = fileInfo.ctime;
	buf->st_spare3 = fileInfo.cspare;
	buf->st_blksize = fileInfo.blockSize;
	buf->st_blocks = fileInfo.blocks;
	UA_Debug << "returns 0\n" << eor;
	return 0;
}
