#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "FileDescriptor.h"
#include "PersistentMemoryObject.h"
#include "PersistentMemoryObjectStar.h"
#include "SysCalls.h"
#include "UnixApplication.h"
#include "OutputStream.h"
extern "C" {
#include <sys/errno.h>
}

#ifdef __GNUC__
#pragma implementation
#endif


int ftruncate(int fd, off_t length)
{
	UA_Debug << "ftruncate(" << fd << ", " << length << ")\n" << eor;
	if (!UA_initialized) UA_init();
	FileDescriptor * filedes = UA_FileDescriptorTable->fileDescriptor(fd);
	if (filedes == 0 || !filedes->used()) { 
		errno = EBADF; 
		UA_Debug << "returns -1 errno " << errno << "\n" << eor;
		return -1; 
	}
	PersistentMemoryObject * so = filedes->persistentMemoryObject();
	if (UA_errno || so == 0) { 
		UA_translateErrno (); 
		UA_Debug << "returns -1 errno " << errno 
			<< " UA_errno " << UA_errno << "\n" << eor;
		return -1; 
	}
	int result = so->setNumberOfBytes(length);
	if (result != 0) { 
		errno = EINVAL; 
		UA_Debug << "returns -1 errno " << errno << "\n" << eor;
		return -1; 
	}
	return 0; 
}
