#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "OutputStream.h"
#include "SysCalls.h"
#include "NameServer.h"
#include "TimeOfDay.h"
#include "UnixApplication.h"
#include "UnixTime.h"
extern "C" {
#include <unistd.h>
}

#ifdef __GNUC__
#pragma implementation
#endif


int gettimeofday(struct timeval * tp, struct timezone * tzp)
{
	static struct timeval tv = { 0, 0 };
	static struct timezone tz = { 0, 0 };
	static int initialized = 0;
	static TimeOfDay * theSystemClock = 0;

	UA_Debug << "gettimeofday(" << asHex((int) tp) << ", "
		<< asHex((int) tzp) << ")\n" << eor;
	if (initialized == 0) {
		initialized = 1;
		UA_Debug << "looking for TimeOfDay class\n" << eor;
		Class * TimeOfDayClass = (Class *)StandardNameServer->lookup
			("TimeOfDay", "Class");
		if (TimeOfDayClass != 0) {
			UA_Debug << "looking for timeOfDay object\n" << eor;
			theSystemClock = (TimeOfDay *)StandardNameServer->lookup
				("timeOfDay", TimeOfDayClass);
		}
	}

	if (theSystemClock) {
		UnixTime unixTime;
		theSystemClock->getUnixTime(unixTime);
		tp->tv_sec = unixTime.seconds();
		tp->tv_usec = unixTime.useconds();
		tzp->tz_minuteswest = unixTime.minuteswest();
		tzp->tz_dsttime = unixTime.dsttime();
		UA_Debug << "returns 0 with secs " << tp->tv_sec 
			 << " usecs " << tp->tv_usec << "\n" << eor;
		return 0;
	} else {
		*tp = tv;
		*tzp = tz;
		UA_Debug << "returns 0 with bogus time\n" << eor;
		return 0;
	}
}
