#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "FileDescriptor.h"
#include "FileSystemInterface.h"
#include "FileSystem.h"
#include "MemoryObjectDictionary.h"
#include "PersistentArray.h"
//#include "PersistentArrayStream.h"
#include "PersistentMemoryObject.h"
#include "PersistentMemoryObjectStar.h"
#include "PersistentObject.h"
#include "PersistentObjectStar.h"
#include "RecordStream.h"
#include "RecordStreamStar.h"
// #include "SysCalls.h"
#include "UnixApplication.h"
#include "OutputStream.h"

extern "C" {
#include <sys/errno.h>
#include <sys/types.h>
#include <sys/unistd.h>
int open(const char * path, int flags, mode_t mode);
}

#ifdef __GNUC__
#pragma implementation
#endif

#define OLDOPEN

int open(const char * path, int flags, mode_t mode)
{
/*
	va_list ap;
	va_start(ap, flags);
	mode_t mode = va_arg(ap, mode_t);
	va_end(ap);
	if (mode == 0) mode = 0777;
 */
	UA_Debug << "open(" << path << ", " << asHex((int) flags) << ", "
		<< asHex((int) mode) << ")\n" << eor;
	mode &= ~UA_umask;
	if (!UA_initialized) UA_init();
#ifdef OLDOPEN
	Class * cls = 0;
	int fd = UA_FileDescriptorTable->allocFileDescriptor();
	FileDescriptor * filedes = UA_FileDescriptorTable->fileDescriptor(fd);
	if (flags & O_CREAT) cls = PersistentMemoryObjectClass;
#ifdef STAR
	PersistentObjectStar po = 
#else 
	PersistentObject * po = 
#endif // STAR
    (PersistentObject *)UA_FileSystemInterface->open 
		((char *) path, PersistentObjectClass, UA_errno, 
		 flags, mode, cls);
#else // OLDOPEN
        int fd = UA_FileDescriptorTable->allocFileDescriptor();
        FileDescriptor * filedes = UA_FileDescriptorTable->fileDescriptor(fd);
	int returnType;
#ifdef STAR
        PersistentObjectStar po =
#else   
        PersistentObject * po =
#endif // STAR
                UA_FileSystemInterface->open((char *) path, UA_errno, 
					     returnType, flags, mode);
#endif // OLDOPEN
	UA_Debug << "po = " << po << "\n" << eor;
	if (UA_errno || po == 0) {
		UA_translateErrno (); 
		UA_Debug << "returns -1 errno " << errno 
			<< " UA_errno " << UA_errno << "\n" << eor;
		filedes->setFree(); 
		return -1; 
	}

#ifdef EAGER
        int err;
#ifdef STAR
        PersistentMemoryObjectStar pmo =
#else // STAR
        PersistentMemoryObject * pmo =
#endif // STAR
                po->asA(PersistentMemoryObjectClass,err);
#else // EAGER
	PersistentMemoryObject * pmo = 0;
#endif // EAGER
#ifdef OLDOPEN

// Francis TODO fix
#ifdef FRANCIS
        filedes->init(po, pmo, 
		      (po->isKindOf(MemoryObjectDictionaryClass) ?
			DirectoryStreamFileDescriptorClass :
			RecordStreamFileDescriptorClass) );
#endif

#else // OLDOPEN
	Assert(returnType != UnknownType);
        filedes->init(po, pmo, 
		      (returnType == MemoryObjectDictionaryType ?
			DirectoryStreamFileDescriptorClass :
			RecordStreamFileDescriptorClass) );

#endif // OLDOPEN
        filedes->setFileFlags(flags);
	if ( flags & O_APPEND ) {
		filedes->lseek(0, SEEK_END); 
	} else {
		filedes->lseek(0, SEEK_SET);
	}
        UA_Debug << "returns " << fd << "\n" << eor;
        return fd;
}
