#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "FileSystemInterface.h"
#include "PersistentMemoryObject.h"
#include "PersistentMemoryObjectStar.h"
#include "SysCalls.h"
#include "UnixApplication.h"
#include "OutputStream.h"

#ifdef __GNUC__
#pragma implementation
#endif

int rename(const char *oldName, const char *newName)
{
	UA_Debug << "rename(" << oldName << ", " << newName << ")\n" << eor;
	if (!UA_initialized) UA_init();
	/*
	 * Make sure that the file already exists.  Check that by opening
	 * the file for reading.
	 */
	PersistentMemoryObjectStar r = (PersistentMemoryObject *)UA_FileSystemInterface->
		open((char *) oldName, PersistentMemoryObjectClass, 
		     UA_errno);
	if (UA_errno || r == 0) { 
		UA_translateErrno (); 
		UA_Debug << "returns -1 errno " << errno 
			<< " UA_errno " << UA_errno << "\n" << eor;
		return -1; 
	}
	r = 0;  /*  Close the file. */
	/*  Link the files, deleting the new one if it already exists. */
	UA_errno = UA_FileSystemInterface->link
		((char *) oldName, (char *) newName, 1);
	if (UA_errno ) { 
		UA_translateErrno (); 
		UA_Debug << "returns -1 errno " << errno 
			<< " UA_errno " << UA_errno << "\n" << eor;
		return -1; 
	}
	/*  Delete the old file. */
	UA_errno = UA_FileSystemInterface->unlink((char *) oldName);
	if (UA_errno) { 
		UA_translateErrno (); 
		UA_Debug << "returns -1 errno " << errno 
			<< " UA_errno " << UA_errno << "\n" << eor;
		return -1; 
	}
        UA_Debug << "returns 0\n" << eor;
	return 0;
}
