#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "OutputStream.h"
#include "Process.h"
#include "SysCalls.h"
#include "ThisProcess.h"
#include "UnixApplication.h"
extern "C" {
#include <sys/wait.h>
#include <sys/errno.h>
}

#ifdef __GNUC__
#pragma implementation
#endif


int wait4(int pid, int * statusp, int options, struct rusage * usage)
{
	static struct rusage dummy = {
		{ 0, 0 },	/* user time used */
		{ 0, 0 },       /* system time used */
		0,		/* max resident set size */
		0,		/* integral shared memory size */
		0,		/* integral unshared data " */
		0,		/* integral unshared stack " */
		0,		/* page reclaims */
		0,              /* page faults */
		0,		/* swaps */
		0,		/* block input operations */
		0,		/* block output operations */
		0,		/* messages sent */
		0,		/* messages received */
		0,		/* signals received */
		0,		/* voluntary context switches */
		0,		/* involuntary " */
	};

	UA_Debug << "wait4(" << pid << ", " << asAddress(statusp)
		<< ", " << options << ", " << asAddress(usage)
		<< ")\n" << eor;
	Process * process = 0;
	if (pid <= 0) {
		Console() 
			<< "wait4( pid " << pid  
			<< " <= 0 ) assumed to be last process forked, pid "
			<< (UA_negativePid ? -UA_lastChild : UA_lastChild)
			<< "\n" << eor;
		if (UA_lastChild == -1) {
			errno = ECHILD;
			UA_Debug << "returns -1 errno " << errno 
				<< "\n" << eor;
			return -1;
		}
		process = (Process *) UA_lastChild;
	} else {
		if (UA_negativePid)
			process = (Process *) -pid;
		else
			process = (Process *) pid;
	}
    
    //Francis TODO FIX
	//Assert(process->isKindOf(ProcessClass));
	if ((options & WNOHANG) && !process->dead()) {
		UA_Debug << "return 0 WNOHANG process " 
			<< process << " not dead yet\n" << eor;
		return 0;
	}

	thisProcess()->waitForProcess(process);
	if (statusp) {
		*statusp = (process->completionStatus() & 0xff) << 8;
	}
	if (usage) {
		*usage = dummy;
	}
	process->unreference();
	int result = UA_negativePid ? -pid : pid;
	UA_Debug << "return " << result << "\n" << eor;
}
