/*
 * This file is part of the Choices Operating System
 * Developed by:
 *      The Mobile Operating Systems Group
 *      Department of Computer Science
 *      University of Illinois at Urbana-Champaign
 *      201 N. Goodwin Ave.
 *      Urbana, IL 61801
 *
 * Copyright 1987-2006 University of Illinois Board of Trustees. All rights
 * reserved. 
 *
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 */

/*
 * GenericARM/AsmBridge.cc: Interface to assembly functions for Generic ARM
 *  machine type.
 */

#include "AsmBridge.h"
#include "MachAsmBridge.h"
#include "ARMCPU.h"
#include "Process.h"
#include "ThisProcess.h"

VABFuncV *  GenericARMSerialInitialize  = &MACH_SERIALINITIALIZE;
VABFuncCS * GenericARMSerialPutS        = &MACH_SERIALPUTS;
VABFuncB *  GenericARMSerialPutChar     = &MACH_SERIALPUTCHAR;
BABFuncV *  GenericARMSerialGetChar     = &MACH_SERIALGETCHAR;
NABFuncPC * GenericARMSerialGetRawChar  = &MACH_SERIALGETRAWCHAR;

extern "C" {
    void undef_handler(ARMInterruptContext *context)
    {
#ifdef DEBUG
        GenericARMSerialPutS("\r\n### UNDEFINED INSTRUCTION HANDLER CALLED ###\r\n");
#endif
        ARMCPU::hardwareInterruptAssist(1,context);
#ifdef DEBUG
        GenericARMSerialPutS("\r\n### UNDEFINED INSTRUCTION HANDLER RETURN ###\r\n");
#endif
    }
}

extern "C" {
    void swi_handler(ARMInterruptContext *context)
    {
        ARMCPU::hardwareInterruptAssist(2,context);
    }
}

extern "C" {
    void prefetch_abort_handler(ARMInterruptContext *context)
    {
#ifdef DEBUG
        GenericARMSerialPutS("\r\n### PREFETCH ABORT HANDLER CALLED ###\r\n");
#endif
        ARMCPU::hardwareInterruptAssist(3,context);
#ifdef DEBUG
        GenericARMSerialPutS("\r\n### PREFETCH ABORT HANDLER RETURN ###\r\n");
#endif
    }
}

extern "C" {
    void data_abort_handler(ARMInterruptContext *context)
    {
#ifdef DEBUG
        GenericARMSerialPutS("\r\n### DATA ABORT HANDLER CALLED ###\r\n");
#endif
        ARMCPU::hardwareInterruptAssist(4,context);
#ifdef DEBUG
        GenericARMSerialPutS("\r\n### DATA ABORT HANDLER RETURN ###\r\n");
#endif
    }
}

extern "C" {
    /**
     * Handler for generic IRQ.
     *
     * @param   context Pseudo-object on the interrupt stack.
     */
    void irq_handler(ARMInterruptContext *context)
    {
        Process *process = thisProcess();

#ifdef DEBUGINT
        GenericARMSerialPutS("\r\n### IRQ HANDLER CALLED ###\r\n");
#endif
        ARMCPU::hardwareInterruptAssist(6,context);
#ifdef DEBUGINT
        GenericARMSerialPutS("\r\n### IRQ HANDLER RETURN ###\r\n");
#endif
        Process *newProcess = thisProcess();
        if (process != newProcess) {
            ((ARMGenericContext *)process->context())->loadFromInterruptContext(context);
            ((ARMGenericContext *)newProcess->context())->restoreToInterruptContext(context);
        }
    }
}

extern "C" {
    /**
     * Handler for fast IRQ.
     *
     * @param   context Pseudo-object on the interrupt stack.
     */
    void fiq_handler(ARMInterruptContext *context)
    {
#ifdef DEBUG
        GenericARMSerialPutS("\r\n### FIQ HANDLER CALLED ###\r\n");
#endif
        ARMCPU::hardwareInterruptAssist(7,context);
#ifdef DEBUG
        GenericARMSerialPutS("\r\n### FIQ HANDLER RETURN ###\r\n");
#endif
    }
}

extern "C" {
    unsigned get_stack(void)
    {
        // This function should not be called in ARM_SYS or ARM_USR mode
        ProcessorContext *context = thisProcess()->context();
        return ((ARMGenericContext *)context)->getInterruptStack();
    }
}
