#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987-2006
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * GenericARMTimerManager.c - machine dependent portion of timer managing
 */

#include "Assert.h"
#include "CPU.h"
#include "ThisCPU.h"
#include "MACHINE_TIMER_MANAGER.h"
#include "MACHINE_CONFIGURATION.h"
#include "OutputStream.h"
#include "FreeRunningTimerStar.h"
#include "Lock.h"

#ifdef __GNUC__
#pragma implementation
#endif

void (*GenericARMTickInterruptFunc)() = 0;

#ifdef SAMPLEPC
//  Code to capture 1000 samples of the PC.
#include "GenericARMGdb.h"
const pcSampleSize = 1000;
char *pcSample [pcSampleSize];
int pcSampleNext = 0, pcSampleMax = 0;

char *getPC (char *trapSP)
{
    /*
     * 1.  Allocate and fill in the registers structure.
     * 2.  Point the global registers at it.
     * 3.  Call the gdb handler
     * 4.  Restore the registers
     */
    unsigned *source;

    source = (unsigned *)trapSP;

    // Francis - TODO Verify if correct
    return (char *) source[14];
}
#endif

extern TimerManager *TimeoutManager;
extern TimerManager *PeriodicManager;

static Lock *TimerManagerLock;
extern void loop(unsigned);

extern void AcknowledgeTimer(void);
extern void StartTimer(void);

void
TimerManagerInterrupt::basicRaise( char *stackPointer )
{
    //Debug << "TimerManagerInterrupt::basicRaise()\n" << eor;
    Assert( this != 0 );

    /*
     * We get some timer interrupts before the Managers are completely
     * initialized so need to make sure the pointers are non-null
     * before calling the handlers
     */
    if (TimeoutManager != 0) TimeoutManager->interruptHandler();

    if (PeriodicManager != 0) PeriodicManager->interruptHandler();

    if (GenericARMTickInterruptFunc != 0) (*GenericARMTickInterruptFunc)();
    
    CPU * currentCPU = thisCPU();
    Assert (currentCPU != 0);
    TIMESLICETIMER *ts = currentCPU->timesliceTimer();
    Assert (ts != 0);
    ts->tick();

#ifdef SAMPLEPC
    char *address = getPC (stackPointer);
    if (pcSampleNext > pcSampleMax) pcSampleMax = pcSampleNext;
    pcSample [pcSampleNext++] = address;
    if (pcSampleNext >= pcSampleSize) pcSampleNext = 0;
#endif

    AcknowledgeTimer();
    // Debug << "TimerManagerInterrupt::basicRaise() returning \n" << eor;
}

GenericARMTimerManager::GenericARMTimerManager(PFI ptb, PFV ep)
    : TimerManager(ptb, ep)
{
    Debug << "GenericARMTimerManager::GenericARMTimerManager()\n" << eor;
    static int intervalTimerStarted = 0;

    microsecondsPerTick = TimerManageruSecPerInterrupt;
    TimerManagerLock = new Lock;
    Assert( TimerManagerLock != 0 );


    unsigned int microTicks = TimerManageruSecPerInterrupt;
    Assert(microTicks > 0);

    /*
     * Initialize the hardware clock when the first GenericARMTimerManager 
     * is constructed.
     */
    TimerManagerLock->acquire();

    // Francis TODO: needs cleanup
    if (!intervalTimerStarted) {
        intervalTimerStarted = 1;
        Debug << this << ": initializing timer\n" << eor;
        StartTimer();
    }

    setFreeRunningTime(SystemTimer->time());
    TimerManagerLock->release();

    Debug << "GenericARMTimerManager::GenericARMTimerManager() done\n" << eor;
}
