#ifdef __GNUC__
#pragma implementation
#pragma implementation "FreeRunningTimerStar.h"
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * IntegratorFreeRunningTimer.cc
 */
#include "Assert.h"
#include "FreeRunningTimerStar.h"
#include "OutputStream.h"
#include "IntegratorFreeRunningTimer.h"
#include "FreeRunningTimerStar.h"
#include "GenericARMTimeOfDay.h"
#include "IntegratorTimer.h"


#ifdef __GNUC__
#pragma implementation
#endif


IntegratorMPUTimer *iTimer1 = (IntegratorMPUTimer *)Integrator_MPU_TIMER1_BASE;
IntegratorMPUTimer *iTimer2 = (IntegratorMPUTimer *)Integrator_MPU_TIMER2_BASE;
IntegratorMPUTimer *iTimer3 = (IntegratorMPUTimer *)Integrator_MPU_TIMER3_BASE;

IntegratorFreeRunningTimer theSystemTimer;
FreeRunningTimerStar SystemTimer = &theSystemTimer;

unsigned IntegratorFreeRunningTimer::constructed = 0;
unsigned IntegratorFreeRunningTimer::timesCalled;
unsigned IntegratorFreeRunningTimer::interrupts;
unsigned IntegratorFreeRunningTimer::prevTime;
Lock * IntegratorFreeRunningTimer::FreeRunningTimerLock;

void
IntegratorFreeRunningTimerInterrupt::basicRaise(char *)
{
    Debug << "IntegratorFreeRunningTimerInterrupt::basicRaise()\n" << eor;
    Assert (SystemTimer != 0);
    theSystemTimer.tick();
    iTimer2->clear();
}

IntegratorFreeRunningTimer::IntegratorFreeRunningTimer()
{
    Assert ( !constructed );
    lastTime = 0;
    timesCalled = 0;
    interrupts = 0;
    prevTime = 0;
    FreeRunningTimerLock = new Lock();
    FreeRunningTimerLock->acquire();
    constructed++;
    FreeRunningTimerLock->release();
}

void
IntegratorFreeRunningTimer::tick()
{
    //Debug << "IntegratorFreeRunningTimer::tick()\n" << eor;
    FreeRunningTimerLock->acquire();
    unsigned int prevTime = lastTime;
    lastTime += FRTuSecPerInterrupt;
    interrupts++;
    if (theSystemClock != 0 && prevTime > lastTime) 
        ((GenericARMTimeOfDay *) theSystemClock)->systemTimerOverflow();
    FreeRunningTimerLock->release();
}


unsigned int
IntegratorFreeRunningTimer::time()
{
    Debug << "IntegratorFreeRunningTimer::time()\n" << eor;
    Assert (FreeRunningTimerLock != 0);
    FreeRunningTimerLock->acquire();
    timesCalled++;
    unsigned t;
    unsigned timerval;
    unsigned correctedLastTime;
    unsigned timertime;
    int pending = 0;

    timerval = iTimer2->read();
    correctedLastTime = lastTime;

    timertime = timerval;

    // Francis TODO What?
//  if (timerval & LIMIT_MASK) {
    if (0) {
        /* 
         * interrupt pending, clock rolled over but lastTime not
         * updated yet.
         */
        correctedLastTime += FRTuSecPerInterrupt;
        pending = 1;
    }


    t = correctedLastTime + timertime; 
    Assert ((t - correctedLastTime) == timertime);

    Debug << "IntegratorFreeRunningTimer::time() correctedLast = " << asHex(correctedLastTime) << "\n" << eor;
    Debug << "IntegratorFreeRunningTimer::time() counter says " << asHex(timerval) << "(" << asHex(timerval) << ")\n" << eor;
    Debug << "IntegratorFreeRunningTimer::time() returns " << asHex(t) << "\n" << eor;

    prevTime = t;
    FreeRunningTimerLock->release();
    return (t);
}

#include "ClassInstaller.h"
DEFINECLASS("IntegratorFreeRunningTimer",IntegratorFreeRunningTimer,FreeRunningTimer,0,0);
