#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987-2005
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Ellick Chan (emchan@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * IntegratorSerial.cc: Integrator UART driver class implementation.
 */

#include "Assert.h"
#include "IntegratorSerial.h"

#define BAUD_DIVISOR    26  /* 115200 bps */

IntegratorSerial::IntegratorSerial(unsigned base) : GenericARMSerial(),base(base)
{
    Initialize();
}

void IntegratorSerial::Initialize()
{
    // ARM Primecell PL011 UART
    unsigned* uart_cr = (unsigned*) UART_CR;
    *uart_cr = 0;
    *uart_cr = UART_RXE | UART_TXE;
    *uart_cr |= UARTEN; 
}

void IntegratorSerial::enableInterrupts()
{
    unsigned* uart_cr = (unsigned*) UART_CR;
    unsigned* uartimsc = (unsigned*) UARTIMSC;
    *uart_cr = 0;
    *uart_cr = UART_RXE | UART_TXE;
    *uart_cr |= UARTEN; 

    // enable interrupts
    *uartimsc |= UART_RXIM;
    //*uartimsc |= UART_TXIM;
}

void IntegratorSerial::disableInterrupts()
{
    unsigned* uartimsc = (unsigned*) UARTIMSC;
    *uartimsc &= ~UART_RXIM;
}

void IntegratorSerial::putChar(char c)
{
    unsigned *uart_dr = (unsigned*) UARTDR;
    unsigned *uart_fr = (unsigned*) UARTFR;
    while(*uart_fr & UART_BUSY);
    *uart_dr = c;       
}

char IntegratorSerial::getChar()
{
    unsigned *uart_dr = (unsigned*) UARTDR;
    return (char) *uart_dr;       
}

int IntegratorSerial::tstChar()
{
    return 1;
// Francis: This doesn't seem to be working right
//    unsigned* uartfr = (unsigned*)UARTFR;
//    return *uartfr & UART_RXFF;
}
