#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Michael J. North (north@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */


// THIS HASNT BEEN TESTED IN A LOOOONG TIME.


/*
 * EMCTape.c: Multimax EMC Tape device.
 */
#include "Assert.h"
#include "OutputStream.h"
#include "CRQ.h"
#include "Vectors.h"
#include "Lock.h"
#include "Semaphore.h"
#include "SemaphoreStar.h"
#include "PhysicalMemoryChain.h"
#include "Process.h"
#include "ThisProcess.h"
#include "Domain.h"


#ifdef __GNUC__
#pragma implementation
#endif

static const int EMC_SLOT = 4;		// MAKE THIS A PARAMETER SOMEWHERES

/*
 * EMC Logical units for the tape.
 */
static const int TAPE0_LUN = 8;

/*
 * A class to represent each physical tape device in the system connected to
 * an EMC.
 */
class MultimaxEMCTape {
protected:
	int slot;
	int logicalUnit;

	int doio( unsigned int start, int count, char * buffer, int op );
public:
	CRQ crq;		// A CRQ to talk to it with.
	CRQ_GENERIC_ATTN_MSG attention[ FreeAttentions ];	

	MultimaxEMCTape( int slot, int logicalUnit );
	void init();
	void handleInterrupt();

	int getData( unsigned int start, int count, char * dest );
	int putData( unsigned int start, int count, char * dest );

};

MultimaxEMCTape::MultimaxEMCTape( int slot, int logicalUnit )
	: crq( UnitID( 0, slot, 0, logicalUnit ),
		ClassVector( TAPE_Vector, CPUCLASS_RUNNING ), CRQMODE_INTR )
{
	this->slot = slot;
	this->logicalUnit = logicalUnit;

	/*
	 * Stick the free attention messages on the new CRQ.
	 */
	for( int i = 0; i < FreeAttentions; i++ ) {
		crq.enqueueFreeMessage( &attention[i] );
	}
}

/*
 * EMC defined tape CRQ_MSG's
 */
class CRQ_MT_RWND_MSG : public EM_MSG { /* REWIND TAPE */
public:
	CRQ_MT_RWND_MSG( int unitid, Semaphore * sem );
};

inline
CRQ_MT_RWND_MSG::CRQ_MT_RWND_MSG( int unitid, Semaphore * sem )
	: EM_MSG( sizeof( CRQ_MT_RWND_MSG ), CRQOP_MT_REWIND, unitid, sem )
{
}

class CRQ_MT_WRTFMK_MSG : public EM_MSG {	/* WRITE FILE MARK */
public:
	int mt_wrtfmk_cnt;	// # file mark bytes to write

	CRQ_MT_WRTFMK_MSG( int unitid, Semaphore * sem, int count );
};

inline
CRQ_MT_WRTFMK_MSG::CRQ_MT_WRTFMK_MSG( int unitid, Semaphore * sem, int count )
	: EM_MSG( sizeof( CRQ_MT_WRTFMK_MSG ), CRQOP_MT_WRITE_FILE_MARK,
		unitid, sem )
{
	mt_wrtfmk_cnt = count;
}

class CRQ_MT_SPC_MSG : public EM_MSG {	/* SPACE */
public:
	int mt_spc_cnt;		// # of items to skip

	CRQ_MT_SPC_MSG( int unitid, Semaphore * sem, int count );
};

inline
CRQ_MT_SPC_MSG::CRQ_MT_SPC_MSG( int unitid, Semaphore * sem, int count )
	: EM_MSG( sizeof( CRQ_MT_SPC_MSG ), CRQOP_MT_SPACE_FMARK, unitid, sem )
{
	mt_spc_cnt = count;
}


static CreateChannelRequest CreateChannelMessage;

class MT_RWND_PKT : public CRQ_MT_RWND_MSG {
public:
	MT_RWND_PKT( int unitid = 0 );
};

inline
MT_RWND_PKT::MT_RWND_PKT( int unitid )
	: CRQ_MT_RWND_MSG( unitid, new BinarySemaphore( 0 ) )
{
}

class MT_SPC_PKT : public CRQ_MT_SPC_MSG {
public:
	MT_SPC_PKT( int unitid = 0, int nitems = 0 );
};

inline
MT_SPC_PKT::MT_SPC_PKT( int unitid, int nitems )
	: CRQ_MT_SPC_MSG( unitid, new BinarySemaphore( 0 ), nitems )
{
}

class MT_WRTFMK_PKT : public CRQ_MT_WRTFMK_MSG {
public:
	MT_WRTFMK_PKT( int unitid = 0, int count = 0 );
};

inline
MT_WRTFMK_PKT::MT_WRTFMK_PKT( int unitid, int count )
	: CRQ_MT_WRTFMK_MSG( unitid, new BinarySemaphore( 0 ), count )
{
}

class MT_SPACE_BLOCK_PKT : public CRQ_MT_SPC_MSG {
public:
	MT_SPACE_BLOCK_PKT( int unitid = 0, int nitems = 0 );
};

inline
MT_SPACE_BLOCK_PKT::MT_SPACE_BLOCK_PKT( int unitid, int nitems )
	: CRQ_MT_SPC_MSG( unitid, new BinarySemaphore( 0 ), nitems )
{
}

class MT_SPACE_SQN_FMARK_PKT : public CRQ_MT_SPC_MSG {
public:
	MT_SPACE_SQN_FMARK_PKT( int unitid = 0, int nitems = 0 );
};

inline
MT_SPACE_SQN_FMARK_PKT::MT_SPACE_SQN_FMARK_PKT( int unitid, int nitems )
	: CRQ_MT_SPC_MSG( unitid, new BinarySemaphore( 0 ), nitems )
{
}

/*
 * The transfer message pool stuff.
 */
static const int num_pool = 10;			// # xfer messages in pool
static MassStoreTransferRequest PoolMsg[ num_pool ];	// tape pool objects
static Lock PoolLock;				// Lock for the pool.
static DBL_LINK Pool;				// tape pool queue
static SemaphoreStar PoolCount;			// tape pool counting semaphore

/*
 *  The rewind message and a semaphore for blocking it's use
 */
static MT_RWND_PKT rewindPacket;
static SemaphoreStar rewindSemaphore;

static MT_SPC_PKT forwardPacket;
static SemaphoreStar forwardSemaphore;

static MT_WRTFMK_PKT wrtfmkPacket;
static SemaphoreStar wrtfmkSemaphore;

static MT_SPACE_BLOCK_PKT spaceblockPacket;
static SemaphoreStar spaceblockSemaphore;

static MT_SPACE_SQN_FMARK_PKT spacesqnfmkPacket;
static SemaphoreStar spacesqnfmkSemaphore;

static MultimaxEMCTape theTape( EMC_SLOT, TAPE0_LUN );
MultimaxEMCTape * TAPE = &theTape;

void
InitTape()
{
	theTape.init();
	rewindSemaphore = new BinarySemaphore( 1 );
	forwardSemaphore = new BinarySemaphore( 1 );
	wrtfmkSemaphore = new BinarySemaphore( 1 );
	spaceblockSemaphore = new BinarySemaphore( 1 );
	spacesqnfmkSemaphore = new BinarySemaphore( 1 );

}

void
TapeInterruptHandler()
{
	theTape.handleInterrupt();
}

void
MultimaxEMCTape::init()
{
	Debug.printf( "MultimaxEMTape::init()\n" );

	/*
	 * Stick all the free Xfer messages in the Pool.
	 */
	for( int i = 0; i < num_pool; i++ ) {
		Pool.enqueue( &PoolMsg[i] );
	}

 	/*
	 * Get a pointer to the EMC's requestor CRQ.
	 */
	CRQ * EMCCRQ = RequestorCRQ( slot, REQ_LUN );

	Debug.printf( "InitTape: Tape CRQ/UnitID %A(%x) EMCCRQ %A\n", &crq,
		crq.crq_unitid, EMCCRQ );
	/*
	 * Initialize the CRQ as a channel for channel for tape requests
	 */
	CreateChannelMessage.setup( &crq );

	Debug.printf( "InitTape: sending create chan and blocking\n" );
	EMCCRQ->sendCommand( &CreateChannelMessage );
	CreateChannelMessage.semaphore()->P();

	/*
	 * Verify the reply.
	 */
	Debug.printf( "InitTape: message/channel status: %x/%x\n",
		CreateChannelMessage.status(),
		CreateChannelMessage.resultStatus() );
	Assert( CreateChannelMessage.status() == STS_SUCCESS );

	/*
	 * At this point we have a CRQ to the tape and can start asking it
	 * questions.
	 */
	Debug.printf( "Tape initialized\n" );
}

/*
 * TapeInterruptHandler
 */
void
MultimaxEMCTape::handleInterrupt()
{
	CRQ_MSG * rsp;
	CRQ_MSG * attn;

	int didWork = 0;

	Debug.printf( "MultimaxEMCTape::handleInterrupt()\n" );
	do {
		/*
		 * If there is an attention message waiting, handle it.
		 */
		if( ( attn = crq.dequeueAttentionMessage() ) != 0 ) {
			switch( attn->code() ) {
			   default:
				crq.handleStandardAttention( attn );
				break;
			}
			crq.enqueueFreeMessage( attn );
			didWork = 1;
		}
	
		/*
		 * If there is a response message waiting, handle it.
		 */
		if( ( rsp = crq.dequeueResponse() ) != 0 ) {
			switch( rsp->code() ) {
			   case CRQOP_MS_READ:
			   case CRQOP_MS_WRITE:
			   case CRQOP_MT_REWIND:
			   case CRQOP_MT_WRITE_FILE_MARK:
			   case CRQOP_MT_SPACE_BLOCK:
				Assert( rsp->semaphore() != 0 );
				rsp->semaphore()->V();
				break;
			   default:
				Debug.printf( "Tape: BAD RESPONSE\n" );
				break;
			}
			didWork = 1;
		}
	} while( ( rsp != 0 ) || ( attn != 0 ) );

	if ( ! didWork ) Debug.printf( "MultimaxEMCTape::handleInt: NO WORK?\n" );
	Debug.printf( "MultimaxEMCDIsk::handleInterrupt: done\n" );
}


int
MultimaxEMCTape::getData( unsigned int start, int count, char * buffer )
{
	return( doio( start, count, buffer, CRQOP_MS_READ ) );
}

int
MultimaxEMCTape::putData( unsigned int start, int count, char * buffer )
{
	return( doio( start, count, buffer, CRQOP_MS_WRITE ) );
}

int
MultimaxEMCTape::doio( unsigned int start, int count, char * buffer, int op )
{
	Debug.printf( "MultimaxEMCTape::doio( start:%d, count:%d, buffer:%A op:%d )\n",
		start, count, buffer, op );
	Assert( ( op == CRQOP_MS_READ ) || ( op == CRQOP_MS_WRITE ) );

	/*
	 * Get a message from the pool to use for transfer requests.
	 */
	PoolCount->P();
	PoolLock.acquire();
	MassStoreTransferRequest * bmesg =
		(MassStoreTransferRequest *) Pool.dequeue();
	PoolLock.release();
	Assert( bmesg != 0 );

	int i = 0;
	while( i < count ) {
		/*
		 * Transfer the request in 4k (8 block) chunks.
		 */
		int nblocks = 8;
		if( ( i + nblocks ) > count ) nblocks = count - i;

		/*
		 * set up the xfer command message.
		 * 512 = number of bytes in blk
		 */
		int nbytes = nblocks * 512;
		PhysicalMemoryChain chain = thisProcess()->domain()->
			constructChain( buffer, nbytes );

		bmesg->map( op, &chain, nbytes, start + i );

		crq.sendCommand( bmesg );
		Debug.printf( "EMCTape::doio: Semaphore sem:%A->P() xfer %d blocks\n",
			bmesg->semaphore(), nblocks );
		bmesg->semaphore()->P();	// wait-synchronize

		/*
		 * TapeInterruptHandler has already removed msg from the
		 * response queue so just check status.
		 */
		if( bmesg->status() != STS_SUCCESS ) break;
		buffer += nbytes;
		i += nblocks;
	}

	/*
	 * Return the message to the free list.
	 */
	PoolLock.acquire();
	Pool.enqueue( bmesg );
	PoolLock.release();
	PoolCount->V();

	return( i );
}

int
TapeIO(int op, void * msg, int len)
{
	Debug.printf( "TapeIO(op:%d msg:%A len:%d)\n", op, msg, len );
	Assert( msg != 0 );
	Assert( len > 0);
	switch( op ) {
		case 0:
			void Rewind();
			Rewind();
			break;
		case 1:
			void SpaceForward(int nfiles);
			SpaceForward( 1 );
			break;
		case 2:
			void WriteFileMark( int nmarks );
			WriteFileMark( 1 );
			break;
		case 3:
			void SpaceBlock( int nblocks );
			SpaceBlock( 124 );
			break;
		case 4:
			void SpaceSqnFmk( int nfmks );
			SpaceSqnFmk( 3 );
			break;
		default:
			Console().printf("TapeIO Request Unknown (at this time)\n");
			break;
	}
	return( 0 );
}

void
Rewind()
{
	rewindSemaphore->P();
	Console().printf("I would like to Rewind the Tape\n");

	(void) new ( &rewindPacket ) MT_RWND_PKT( theTape.crq.crq_unitid );

	theTape.crq.sendCommand( &rewindPacket );
	rewindPacket.semaphore()->P();
	rewindSemaphore->V();
}

void
SpaceForward( int nfiles )
{
	forwardSemaphore->P();
	Console().printf("I would like to Fast Forward the Tape\n");

	(void) new ( &forwardPacket ) MT_SPC_PKT( theTape.crq.crq_unitid,
		nfiles );

	theTape.crq.sendCommand( &forwardPacket );
	forwardPacket.semaphore()->P();
	forwardSemaphore->V();
}

void
WriteFileMark( int nmarks )
{
	wrtfmkSemaphore->P();
	Console().printf("I would like to Write a few File Marks to the Tape\n");

	(void) new ( &wrtfmkPacket ) MT_WRTFMK_PKT( theTape.crq.crq_unitid,
		nmarks );

	theTape.crq.sendCommand( &wrtfmkPacket );
	wrtfmkPacket.semaphore()->P();
	wrtfmkSemaphore->V();
}

void
SpaceBlock( int nblocks )
{
	spaceblockSemaphore->P();
	Console().printf("I would like to Fast Forward the Tape\n");

	(void) new ( &spaceblockPacket ) MT_SPACE_BLOCK_PKT(
		theTape.crq.crq_unitid, nblocks );

	theTape.crq.sendCommand( &spaceblockPacket );
	spaceblockPacket.semaphore()->P();
	spaceblockSemaphore->V();
}

void
SpaceSqnFmk( int nsqnfmks )
{
	spacesqnfmkSemaphore->P();
	Console().printf("I would like to Fast Forward the Tape\n");

	(void) new ( &spacesqnfmkPacket ) MT_SPACE_SQN_FMARK_PKT(
		theTape.crq.crq_unitid, nsqnfmks );

	theTape.crq.sendCommand( &spacesqnfmkPacket );
	spacesqnfmkPacket.semaphore()->P();
	spacesqnfmkSemaphore->V();
}
