#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Paul Petersen
 * Contributing Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 *	EtherProcess.c: ethernet processes.
 */

#include "Assert.h"
#include "OutputStream.h"
#include "CPU.h"
#include "ThisCPU.h"
#include "AwaitedInterruptInterrupt.h"
#include "Process.h"
#include "ThisProcess.h"
#include "Vectors.h"


#ifdef __GNUC__
#pragma implementation
#endif

static const int EMC_SLOT = 4;		// TEMPORARY

static void
EtherControlProcessEntry( int vector )
{
	Debug.printf( "EtherControlProcessEntry( vector:%x )\n", vector );

	AwaitedInterruptInterrupt * interrupt = new AwaitedInterruptInterrupt();
	thisCPU()->setInterrupt( vector, interrupt );

	/*
	 * Loop forever waiting for a message from the EMC
	 */

	while( 1 ) {
		interrupt->await();

		/*
		 * Process the input.
	 	 */
		extern void EtherControlInterruptHandler();
		EtherControlInterruptHandler();
	}
}

static void
EtherLargeProcessEntry( int vector )
{
	Debug.printf( "EtherLargeProcessEntry( vector:%x )\n", vector );

	AwaitedInterruptInterrupt * interrupt = new AwaitedInterruptInterrupt();
	thisCPU()->setInterrupt( vector, interrupt );

	/*
	 * Loop forever waiting for a message from the EMC
	 */

	while( 1 ) {
		interrupt->await();

		/*
		 * Process the input.
	 	 */
		extern void EtherLargeInterruptHandler();
		EtherLargeInterruptHandler();
	}
}

static void
EtherSmallProcessEntry( int vector )
{
	Debug.printf( "EtherSmallProcessEntry( vector:%x )\n", vector );

	AwaitedInterruptInterrupt * interrupt = new AwaitedInterruptInterrupt();
	thisCPU()->setInterrupt( vector, interrupt );

	/*
	 * Loop forever waiting for the next message from the EMC
	 */

	while( 1 ) {
		interrupt->await();

		/*
		 * Process the input.
	 	 */
		extern void EtherSmallInterruptHandler();
		EtherSmallInterruptHandler();
	}
}

volatile int ethernetInitialized = 0;

void
EthernetManagerProcessEntry( int arg )
{
	Debug.printf( "EthernetManagerProcessEntry( %x )\n", arg );

	/*
	 * Before trying to initialize the ethernet device start up the
	 * processes to wait for interrupts from it. Start with the control
	 * interrupt process.
	 */
	Process * process = new InterruptProcess( 
		(PFV) EtherControlProcessEntry, thisProcess()->domain(), normal,
		ETHER_Vector );
	Debug.printf( "EtherManagerProcess: control interrupt at %A\n",
		process );
	Assert( process != 0 );
	process->setName( "#EtherControl" );

	process->ready();
	thisProcess()->relinquishProcessor();

	Debug.printf( "EtherManagerProcess: control interrupt process started\n" );

	/*
	 * Next do the small packet interrupt process.
	 */
	process = new InterruptProcess(
		(PFV) EtherSmallProcessEntry, thisProcess()->domain(), normal,
		 ETHERS_Vector );
	Debug.printf( "EtherManagerProcess: small interrupt at %A\n",
		process );
	Assert( process != 0 );
	process->setName( "#EtherManager" );

	process->ready();
	thisProcess()->relinquishProcessor();
	Debug.printf( "EtherManagerProcess: small interrupt process started\n" );

	/*
	 * Finally do the large packet interrupt process.
	 */
	process = new InterruptProcess(
		(PFV) EtherLargeProcessEntry, thisProcess()->domain(), normal,
		 ETHERL_Vector );
	Debug.printf( "EtherManagerProcess: large interrupt at %A\n",
		process );
	process->setName( "#EtherLargePacketProcess" );
	Assert( process != 0 );

	process->ready();
	thisProcess()->relinquishProcessor();
	Debug.printf( "EtherManagerProcess: large interrupt process started\n" );

	/*
	 * Initialize the ethernet interface.
	 */	
	extern void InitEther();

	InitEther();
	Debug.printf( "Ethernet Initilization: done\n" );
        ethernetInitialized = 1;
}
