#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * MultimaxConsole.c: Multimax Diagnostic console output routines.
 */

#include "FreeRunningTimer.h"
#include "MultimaxConsole.h"


#ifdef __GNUC__
#pragma implementation
#endif


MultimaxConsole::MultimaxConsole()
{
}

/*
 * These violate one of my personal pet peeves of acquiring and releasing
 * a lock in two different function!!  I really can't see any better way.
 * We have to pray the parent class always calls them in pairs.
 */
void
MultimaxConsole::acquireExclusiveUse()
{
	_exclusiveUseLock.acquire();
}

void
MultimaxConsole::releaseExclusiveUse()
{
	_exclusiveUseLock.release();
}

/*
 * Write a character to the console via the SCC. All locking mut be handled
 * at a higher level.
 */
int SCCWriteFailed = 0;

static void
sccWrite( char c )
{
	volatile char * const sccreq = (char *) 0xfffc0000; // Transmit request register.
	volatile char * const sccbuf = (char *) 0xfffc0001; // Transmit buffer.
	int spin = 1000000;
	while ( *sccreq && spin-- );
	if ( *sccreq ) {
		SCCWriteFailed++;
	}
	*sccbuf = c;
	*sccreq = 1;
/*
	volatile char * const auxreq = (char *) 0xfffc0000; // Transmit request register.
	volatile char * const auxbuf = (char *) 0xfffc0001; // Transmit buffer.
	spin = 1000000;
	while ( *auxreq && spin-- );
	if ( *auxreq ) {
		SCCWriteFailed++;
	}
	*auxbuf = ( c & 040 ) ? ( c & ~040 ) : ( c | 040 );
	*auxreq = 1;
*/
}

int
MultimaxConsole::write( const char * buffer, int count )
{
	int charsWritten = 0;
	while( count-- > 0 ) {
		_mutexLock.acquire();

		char c = *buffer++;
		sccWrite( c );
		if( c == '\n' ) {
			sccWrite( '\r' );
		}

		_mutexLock.release();
		charsWritten++;
	}
	return( charsWritten );
}


int
MultimaxConsoleForUserOutput::write( const char * buffer, int count )
{
	extern int SendToConsole( char * buffer, int count );
	return( SendToConsole( (char *) buffer, count ) );
}

/*
 * Really incredibly stupid "put a string on the console function".
 * It is used only for debugging VERY early in the runtime of the system
 * before even locks are initialized. Basically its only use is to see if the
 * machine even booted.
 * This is used ONLY by machine dependent code.
 */
void
BootPrint( char * string )
{
	char c;
	while( c = *string++ ) {
		sccWrite( c );
	}
}
