#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801-2910
 *
 * Copyright (c) 1991 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Amit Parghi (parghi@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *	MultimaxEthernetDriver.cc
 *
 *	EthernetDriver interface for the Multimax.
 *	This object is simply an interface between the real networking
 *	code and the EMCEthernet code.
 *
 * $Id: MultimaxEthernetDriver.cc 2 2004-06-03 01:58:28Z jcarlyle $
 *
 * $Log: MultimaxEthernetDriver.cc,v $
 * Revision 1.1  2004/06/03 01:58:55  jcarlyle
 * Initial revision
 *
 */

//SYSIN DD *

// #define DEBUG 1
#define RAID 1

#include "OutputStream.h"
#include "Process.h"
#include "Semaphore.h"
#include "ThisProcess.h"
#include "Types.h"
#include "NetworkBuffer.h"
#include "EthernetAdministrator.h"
#include "MultimaxEthernetDriver.h"
#include "Raid.h"


#ifdef __GNUC__
#pragma implementation
#endif

	//	defined in EMCEthernet.cc ...
extern void EtherAddr(char *);
extern int  EtherXmit(char *, unsigned int );
extern int  EtherRecv(char *&);
extern void EtherRelease(char *&);


void _MultimaxEthernetDaemon(MultimaxEthernetDriver *);		// forward


MultimaxEthernetDriver::MultimaxEthernetDriver(unsigned char debug)
    : EthernetDriver(debug)
{
	//  Get this interface's Ethernet address from the EMC and
	//  place it in the _ethernetAddress variable, as required.
    char tmp[8];

    Raids(RaidMethod)
	    << "MultimaxEthernetDriver::MultimaxEthernetDriver() starting.\n"
	    << eor;

    EtherAddr(tmp);
    _ethernetAddress = new EthernetAddress((unsigned char *) tmp);

    Raids(RaidMethod) << "MultimaxEthernetDriver: EthernetAddress is "
		      << *_ethernetAddress << '\n' << eor;

	//  Now, make the receiver daemon ready to go (but don't ready() it!).
    _daemonProcess = new
		     PreemptableSystemProcess((PFV) _MultimaxEthernetDaemon,
					      thisProcess()->domain(),
					      normal,		// stack size
					      (int) this);
    _daemonProcess->setName("MultimaxEthernetDaemon");

    setEMCBuffer(NULL, 0);
}

MultimaxEthernetDriver::~MultimaxEthernetDriver()
{
    delete _ethernetAddress;
}


void MultimaxEthernetDriver::ready(NetworkAdministrator *admin)
{
    Console() << "MultimaxEthernetDriver::ready("
	      << asHex((int) admin) << ")\n" << eor;

    EthernetDriver::ready(admin);		// super ready: admin

    _daemonProcess->ready();			// NOW we can start the daemon!
}

int
MultimaxEthernetDriver::transmitPacket(NetworkBuffer *buff)
{
    Raids(RaidDetail) << "MultimaxEthernetDriver::transmitPacket(): buffer="
		      << asHex((int) buff->packet()) << ", length="
		      << asHex((int) buff->length()) << "\n" << eor;

	//  Since EtherXmit() doesn't stuff the sender's address into
	//  the Ethernet packet before transmitting it, we have to do
	//  this ourselves.  (This is, of course, a security hole of
	//  serious dimensions, but, hey, if you can't trust your kernel,
	//  then who CAN you trust, right?			// Amit
    _ethernetAddress->copyTo(buff->packet() + EthernetAddressLength);

    EtherXmit( (char *) buff->packet(), (int) buff->length() );
    return 0;
}


void
MultimaxEthernetDriver::recvLoop()
{
    octet *emcBuffer;
    int size;

#   ifdef DEBUG
	Debug << "\t*** _MultimaxEthernetDaemon starting up ...\n" << eor;
#   endif

    while (1) {					// loop forever
	    // Note that emcBuffer *must* be initialized to NULL before
	    // calling EtherRecv(), because an Assert in that routine
	    // requires it.  As to why this is, who knows?	// Amit

	NetworkBuffer *buffer = NetworkDriver::getBuffer();
	Assert (buffer != 0);
	emcBuffer = NULL;
	size = EtherRecv(emcBuffer);		// wait for a packet to come in
	Assert (emcBuffer != NULL);

#	ifdef DEBUG
	    Debug << "_MultimaxEthernetDaemon: received packet, length = "
		  << size << "\n" << eor;
#	endif

	buffer->copyFrom(emcBuffer, size);	// *** COPY ***
	EtherRelease(emcBuffer);		// return buffer to EMC list
	setEMCBuffer(NULL, 0);
	NetworkDriver::receivePacket(buffer);	// pass on to Driver & Admin
    }
	Assert (NOTREACHED);
}

void
_MultimaxEthernetDaemon(MultimaxEthernetDriver *driver)
{
	driver->recvLoop();
	Assert (NOTREACHED);
}
