#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Contributing Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * MultimaxInterrupts.cc: Multimax hardware interrupt classes implementations.
 */
#include "Assert.h"
#include "OutputStream.h"
#include "ThisProcess.h"
#include "ProcessContainer.h"
#include "CPU.h"
#include "ThisCPU.h"
#include "Vectors.h"
#include "AddressFlushQueue.h"
#include "MultimaxInterrupts.h"
#include "Timer.h"
#include "PROCESSOR.h"


#ifdef __GNUC__
#pragma implementation
#endif

void
VectorBusInterrupt::basicRaise( char * info )
{
	// ADD AN ASSERT TO CHECK THE APC CSR TO SEE IF A VECTOR IS THERE
	// INSIDE FlushVector.

	unsigned int vec = FlushVector();
	Assert( vec != -1 );
	Debug.printf( "VectorBusInterrupt::basicRaise(): vector %x\n", vec );

	CPU * cpu = thisCPU();

	Interrupt * handler = cpu->interrupt( vec );
	Assert( handler != 0 );
	Debug.printf( "VectorBusInterrupt::basicRaise: %A\n", handler );

	handler->raise( info );
	Debug.printf( "VectorBusInterrupt: interrupt handler returned\n" );
}

void
NanoBusInterrupt::basicRaise( char * )
{
	Console().panic( "NanoBus Interrupt (%s) PAR=%A process--%I\n", message,
		*(unsigned volatile int *) 0xffffff40, thisProcess() );
	Assert( NOTREACHED );
}

/*
 * Flush the AddressTranslator entry specified be the interrupt.
 */
void
AddressTranslatorFlushInterrupt::basicRaise( char * )
{
	Assert( ! PROCESSOR::interruptsEnabled() );

	/*
	 * Flush all entries in the AddressFlushQueue.
	 */
	AddressFlushQueue * flushQueue = thisCPU()->flushQueue();
	Assert( flushQueue != 0 );
	flushQueue->doFlush();
}
