#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Author: John L. Coolidge (coolidge@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */


/*
 */

/*
 * RealTimeClock.c: Code to set and read the time-of-day clock. 
 *
 */

#include "Assert.h"
#include "OutputStream.h"
#include "CRQ.h"
#include "NS32202.h"
#include "RealTimeClock.h"
#include "Vectors.h"
#include "NewAndDelete.h"
#include "FreeRunningTimerStar.h"
#include "FreeRunningTimer.h"
#include "SystemTime.h"


#ifdef __GNUC__
#pragma implementation
#endif

/*
 * There is only a single instance.
 */

class CRQ_TOYMSG : public CRQ_MSG {
public:
	TOY * addr;
	short status;

	CRQ_TOYMSG( int op = CRQOP_SCC_GET_TOY );
};

inline
CRQ_TOYMSG::CRQ_TOYMSG( int op )
	: CRQ_MSG( sizeof( CRQ_TOYMSG ), op, UnitID( 0, SCC_SLOT, 0, 0 ), 0 )
{
}

static CRQ_TOYMSG toyMsg;
static TOY systemTOY;

static unsigned int NormalDaysInMonth[12] = 
			{31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
static unsigned int LeapDaysInMonth[12] = 
			{31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };

MultimaxTimeOfDay::MultimaxTimeOfDay()
{
    // Initialize the global time-in-seconds to the current hardware time.
    getHardwareTime( _baseTime );
    _previousTime = _baseTime.time();

    // Initialize the current notion of delta time to the current free
    // running timer value
    _previousFreeTime = SystemTimer->time();

	Console() << "Current time " << asHex(_previousTime._value.ui[0]) <<
		" " << asHex(_previousTime._value.ui[1]) << "\n" << eor;
}

void MultimaxTimeOfDay::getTime(SystemTime& resulttime)
{
    // Get the current free-running time
    int32 currentFreeTime = SystemTimer->time();

    // Find out the delta time
    int64 deltaTime = ( ((currentFreeTime > _previousFreeTime) ?
			 (currentFreeTime - _previousFreeTime) :
			 (currentFreeTime -
			  (0xffffffff - _previousFreeTime) + 1))
		       * 10);

    _previousFreeTime = currentFreeTime;

//    _previousTime += deltaTime;
    _previousTime = _previousTime + deltaTime;
 
//	Console() << "Current time " << asHex(_previousTime._value.ui[0]) <<
//		" " << asHex(_previousTime._value.ui[1]) << "\n" << eor;
    resulttime.setTime( _previousTime );
    resulttime.setTimeZoneOffset( _baseTime.timeZoneOffset() );
    resulttime.setDst( _baseTime.dst() );
}

// For debugging purposes
void
printToy( TOY * toyData )
{
	Console().cpuPrintf( "DayOfWeek=%d\n", toyData->toy_day_of_week );
	Console().cpuPrintf( "Date=%d%d/%d%d/",
	toyData->toy_ten_month, toyData->toy_unit_month,
	toyData->toy_ten_day, toyData->toy_unit_day);
	Console().cpuPrintf( "%d%d%d%d\n",
	toyData->toy_thou_year, toyData->toy_hun_year,
	toyData->toy_ten_year, toyData->toy_unit_year );
	Console().cpuPrintf( "Time=%d%d:%d%d.",
	toyData->toy_ten_hour, toyData->toy_unit_hour,
	toyData->toy_ten_min, toyData->toy_unit_min);
	Console().cpuPrintf( "%d%d.%d\n",
	toyData->toy_ten_sec, toyData->toy_unit_sec,
	toyData->toy_tenths_sec );
	Console().cpuPrintf( "Time zone offset=%d\n",
	toyData->toy_offset );
}

static int64 convertfromoffset( int offsettime )
{
	return(offsettime * TIMEZONE_TO_SYSTEM);
}

// Return the time of day from the system clock (in system time units)
void GetTimeFromSCCToy( TOY& toytime )
{
	Debug.printf( "MultimaxTimeOfDay::getHardwareTime()\n" );

	// Get a pointer to the proper CRQ for the processor to be started
	CRQ * crq = RequestorCRQ( SCC_SLOT, 0 );
	Assert( crq != 0 );
	crq->crq_mode = CRQMODE_POLL;

	// Initialize command
	CRQ_TOYMSG * cmd = new ( &toyMsg ) CRQ_TOYMSG( CRQOP_SCC_GET_TOY );
	cmd->addr = &toytime;
	Debug.printf( "MultimaxTimeOfDay::getHardwareTime: cmd: %A\n", cmd );

	Debug.printf( "MultimaxTimeOfDay::getHardwareTime: enqueue command %A on %A\n",
		cmd, crq );
	crq->sendCommand( cmd );
	TransmitVector( (unsigned int) crq->crq_slave_vect );
	Debug.printf( "MultimaxTimeOfDay::getHardwareTime: request enqueued\n" );

	// Poll response queue waiting for completion
	CRQ_MSG * rsp = 0;
	while( rsp == 0 ) rsp = crq->dequeueResponse();

	Debug.printf( "MultimaxTimeOfDay::getHardwareTime: rsp=%x code=%d toy status=%d\n",
		rsp, rsp->code(), cmd->status );
	// Check response status codes
	if( rsp->code() != CRQOP_SCC_GET_TOY )
	    Console().cpuPrintf( "MultimaxTimeOfDay::getHardwareTime response status=%d\n", 
			rsp->code() );
	crq->crq_mode = CRQMODE_INTR;
};
	
void MultimaxTimeOfDay::getHardwareTime( SystemTime& time )
{
	GetTimeFromSCCToy(systemTOY);	
	int64 newTime = toSystemTime( &systemTOY );
	time.setTime( newTime );
	Console() << "Base time " << asHex(time.time()._value.ui[0]) <<
		" " << asHex(time.time()._value.ui[1]) << "\n" << eor;
	printToy( &systemTOY );
	time.setTimeZoneOffset( convertfromoffset(systemTOY.toy_offset) );
	time.setDst( 1 );	// Default to using dst
}

// Set the time of day on the system clock (given seconds since Vincemas)
void
MultimaxTimeOfDay::setTime( SystemTime& time )
{
    unsigned int seconds = 0; //shut compiler up. This function BROKEN. FIX.
    Assert( NOTREACHED );
	Debug.printf( "MultimaxTimeOfDay::setRealTime( %d )\n", seconds );

	// Get a pointer to the proper CRQ for the processor to be started
	CRQ * crq = RequestorCRQ( SCC_SLOT, 0 );
	Assert( crq != 0 );
	crq->crq_mode = CRQMODE_POLL;

	// Initialize command
	CRQ_TOYMSG * cmd = new ( &toyMsg ) CRQ_TOYMSG( CRQOP_SCC_SET_TOY );
	Debug.printf( "MultimaxTimeOfDay::setRealTime: cmd: %A\n", cmd );

	// Convert the seconds to toy
	fromSeconds( toyMsg.addr, seconds );

	Debug.printf( "MultimaxTimeOfDay::setRealTime: enqueue command %A on %A\n",
		cmd, crq );
	crq->sendCommand( cmd );

	TransmitVector( (unsigned int) crq->crq_slave_vect );
	Debug.printf( "MultimaxTimeOfDay::setRealTime: request enqueued\n" );

	// Poll response queue waiting for completion
	CRQ_MSG * rsp = 0;
	while( rsp == 0 ) rsp = crq->dequeueResponse();

	Debug.printf( "MultimaxTimeOfDay::setRealTime: rsp=%x code=%d toy status=%d\n",
		rsp, rsp->code(),  cmd->status );

	// Check response status codes
	if( rsp->code() != CRQOP_SCC_GET_TOY ) 
	    Console().cpuPrintf( "MultimaxTimeOfDay::getRealTime response status=%d\n", 
			rsp->code() );
	//printToy( cmd->addr );
}

// Convert from toy's Greenwich mean time to System time (Nov 17, 1858, 0:00 AM
// Greenwich Time, 100ns units)
int64
MultimaxTimeOfDay::toSystemTime( TOY * toyData )
{
	Debug.printf( "MultimaxTimeOfDay::toSeconds( %x )\n", toyData );

 	// Convert the toy numbers to usuable integers
	unsigned int myYear = toyData->toy_unit_year 
		+ (toyData->toy_ten_year * 10) + (toyData->toy_hun_year * 100) 
		+ (toyData->toy_thou_year * 1000);
	unsigned int myMonth = toyData->toy_unit_month 
		+ (toyData->toy_ten_month * 10);
	//unsigned int myDayOfWeek = toyData->toy_day_of_week;
	unsigned int myDay = toyData->toy_unit_day 
		+ (toyData->toy_ten_day * 10);
	unsigned int myHour = toyData->toy_unit_hour 
		+ (toyData->toy_ten_hour * 10);
	unsigned int myMinute = toyData->toy_unit_min 
		+ (toyData->toy_ten_min * 10);
	unsigned int mySecond = toyData->toy_unit_sec 
		+ (toyData->toy_ten_sec * 10);
	unsigned int dayOfYear = 0;

	Assert( (myMonth >= 1) && (myMonth <= 12) );
	Assert( (myDay >= 1) && (myDay <= 31) );
	Assert( myHour <= 23 );
	Assert( myMinute <= 59) ;
	Assert( mySecond <= 59) ;

	// Calculate the day of the year
	int month = myMonth;
	while( --month ) dayOfYear += NormalDaysInMonth[month - 1];
	dayOfYear += (myDay - 1);

	// Determine the total number of days since 1970, Jan 1, 0:00 AM
	int32 total = dayOfYear;
	int year = myYear;	
	for( int i = 1970; i < year; i++ )
		total += DaysPerYear( i );
	if( (DaysPerYear( year ) == 366) && myMonth >= 3 ) {
		dayOfYear += 1;
		total += 1;
	}

	// Convert the days to seconds
	total = (total * (24 * 60 * 60));

	// Add in hours for today
	total += (myHour * (60 * 60));

	// Add in minutes for today
	total += (myMinute * 60);

	// Add in seconds for today
	total += mySecond;

	// Convert from seconds to 100ns
//	total *= SYSTEM_TO_SECOND;
	int64 temptotal = total;
	int64 systotal = convertfromunix( temptotal );

//	Console() << "toSystemTime: unix-format time " << asHex(total) <<
//	  " system-format time " << asHex(systotal._value.s.low) << " " <<
//	    asHex(systotal._value.s.high) << "\n" << eor;

//	Debug.printf( "MultimaxTimeOfDay::toSeconds returns %d\n", total );
	return( systotal );
}

// Convert from toy's Greenwich mean time to Vincemas (Aug 9, 1962, 5:41 AM
// Greenwich Time)
unsigned int
MultimaxTimeOfDay::toVinceSeconds( TOY * toyData )
{
	Debug.printf( "MultimaxTimeOfDay::toVinceSeconds( %x )\n", toyData );

 	// Convert the toy numbers to usuable integers
	unsigned int myYear = toyData->toy_unit_year 
		+ (toyData->toy_ten_year * 10) + (toyData->toy_hun_year * 100) 
		+ (toyData->toy_thou_year * 1000);
	unsigned int myMonth = toyData->toy_unit_month 
		+ (toyData->toy_ten_month * 10);
	//unsigned int myDayOfWeek = toyData->toy_day_of_week;
	unsigned int myDay = toyData->toy_unit_day 
		+ (toyData->toy_ten_day * 10);
	unsigned int myHour = toyData->toy_unit_hour 
		+ (toyData->toy_ten_hour * 10);
	unsigned int myMinute = toyData->toy_unit_min 
		+ (toyData->toy_ten_min * 10);
	unsigned int mySecond = toyData->toy_unit_sec 
		+ (toyData->toy_ten_sec * 10);
	unsigned int dayOfYear = 0;

	Assert( (myMonth >= 1) && (myMonth <= 12) );
	Assert( (myDay >= 1) && (myDay <= 31) );
	Assert( myHour <= 23) ;
	Assert( myMinute <= 59) ;
	Assert( mySecond <= 59) ;

	// Calculate the day of the year
	int month = myMonth;
	while( --month ) dayOfYear += NormalDaysInMonth[month - 1];
	dayOfYear += (myDay - 1);

	// Determine the total number of days since 1962, August 9, 5:41 AM
	unsigned int total = dayOfYear;
	int year = myYear;	
	for( int i = 1962; i < year; i++ )
		total += DaysPerYear( i );
	if( (DaysPerYear( year ) == 366) && myMonth >= 3 ) {
		dayOfYear += 1;
		total += 1;
	}

	// Convert the days to seconds
	total = (total * (24 * 60 * 60));

	// Add in hours for today
	total += (myHour * (60 * 60));

	// Add in minutes for today
	total += (myMinute * 60);

	// Add in seconds for today
	total += mySecond;

	// Subtract seconds from Jan1 1962-Aug6 1962, 5:41AM Greenwich Time
	total -= 19028460;

	Debug.printf( "MultimaxTimeOfDay::toVinceSeconds returns %d\n", total );
	return( total );
}


// Take the number of seconds since Vincemas and convert to toy format
// and load into toy structure.
void
MultimaxTimeOfDay::fromSeconds( TOY * toyData, unsigned int totalSeconds )
{
  	Debug.printf( "MultimaxTimeOfDay::fromSeconds( %x, %d )\n", toyData,
		totalSeconds );

	// Add in bias from beginning of 1962 for easy calculations
	totalSeconds += 19028460;

	// determine hours:minutes:seconds
	unsigned int secondsToday = totalSeconds % 86400;
	unsigned int seconds = secondsToday % 60;
	toyData->toy_ten_sec = (seconds / 10);
	toyData->toy_unit_sec = (seconds % 10);

	unsigned int days = totalSeconds / 86400;
	unsigned int minutesToday = secondsToday / 60;
	unsigned int minutes = minutesToday % 60;
	toyData->toy_ten_min = (minutes / 10);
	toyData->toy_unit_min = (minutes % 10);

	unsigned int hours = minutesToday / 60;
	toyData->toy_ten_hour = (hours / 10);
	toyData->toy_unit_hour = (hours % 10);

	// determine the day of the week
	toyData->toy_day_of_week = (days + 4) % 7;

	// determine the year, month, and day 
	for( unsigned year = 62; days >= DaysPerYear( year ); year++ )
	    days -= DaysPerYear( year );

	// determine month
	unsigned int * daysInMonth = ( LeapYear( year ) ) ? LeapDaysInMonth : 
		NormalDaysInMonth;
	for( unsigned int month = 0; days >= daysInMonth[month]; month++ )
	    days -= daysInMonth[month];

	month++;
	days++;

	toyData->toy_ten_month = (month / 10);
	toyData->toy_unit_month = (month % 10);
	toyData->toy_ten_day = (days / 10);
	toyData->toy_unit_day = (days % 10);

	year += 1900;
	toyData->toy_thou_year = (year / 1000);
	toyData->toy_hun_year = ((year / 100) % 10);
	toyData->toy_ten_year = ((year / 10) % 10);
	toyData->toy_unit_year = (year % 10);

	toyData->toy_offset = 0;

  	Debug.printf( "MultimaxTimeOfDay::fromSeconds done\n" );
}

#include "ClassInstaller.h"
DEFINECLASS("MultimaxTimeOfDay",MultimaxTimeOfDay,TimeOfDay,0,0);
