#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Contributing Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */



/*
 * Requestor.c: Requestor CRQ management functions.
 */

#include "Assert.h"
#include "OutputStream.h"
#include "Semaphore.h"
#include "Vectors.h"
#include "CRQ.h"


#ifdef __GNUC__
#pragma implementation
#endif

void
RequestorInit( int slot )
{
	Assert( (slot >= 0) && (slot < NUM_SLOT) );
	CRQ * crq = RequestorCRQ( slot, 0 );
	unsigned int vector = ClassVector( 0x30 + slot, CPUCLASS_RUNNING );
	crq->changeMasterVector( vector );
}

/*
 * RequestorInterruptHandler. Called from a Requestor process when an
 * interrupt occurs.
 */
void
RequestorInterruptHandler( int slot )
{
	CRQ_MSG * rsp;
	CRQ_MSG * attn;

	int didWork = 0;

	Debug.printf( "RequestorInterruptHandler( slot:%d )\n", slot );

	CRQ * crq = RequestorCRQ( slot, 0 );
	do {
		/*
		 * If there is an attenetion message waiting, handle it.
		 */
		if( ( attn = crq->dequeueAttentionMessage() ) != 0 ) {
			switch( attn->code() ) {
			   default:
				crq->handleStandardAttention( attn );
				break;
			}
			crq->enqueueFreeMessage( attn );
			didWork = 1;
		}

		/*
		 * If there is a response message waiting, handle it.
		 */
		if( ( rsp = crq->dequeueResponse() ) != 0 ) {
			Debug.printf( "RequestorCRQ: there's a response there\n" );
			switch( rsp->code() ) {
			   case CRQOP_CREATE_CHANNEL:
				Assert( rsp->semaphore() != 0 );
				Debug.printf( "RequestorCRQ interrupt slot %d: %A->V()\n",
					slot, rsp->semaphore() );
				rsp->semaphore()->V();
				break;
			   default:
				Console().cpuPrintf( "RequestorCRQ Interrupt: BAD RESPONSE %d\n",
					rsp->code() );
				break;
			}
			didWork = 1;
		}

	} while( ( attn != 0 ) || ( rsp != 0 ) );
	if( ! didWork ) {
		Console().cpuPrintf( "RequestorInterruptHandler: slot %d NO WORK\n",
			slot );
	}
	Debug.printf( "RequestorInterruptHandler: done\n" );
}
