#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * SCCSerialLine.c - SCC Serial Line (especially the console) I/O.
 *
 * There is still some ENCORE stuff in here that should be nuked!
 */

#include "Assert.h"
#include "CRQ.h"
#include "CharacterInputStream.h"
#include "CharacterQueue.h"
#include "FreeRunningTimer.h"
#include "OutputStream.h"
#include "Semaphore.h"
#include "SemaphoreStar.h"
#include "Vectors.h"
#include "NewAndDelete.h"


#ifdef __GNUC__
#pragma implementation
#endif

//static MultimaxConsoleForUserOutput theConsoleOutputStream;
//OutputStream * ConsoleOutputStream = &theConsoleOutputStream;

/*
 * SCC defined serial line logical units.
 */
static const int SL_READ_LUN = 0;
static const int SL_WRITE_LUN = 1;

/*
 * Convert a line/channel into a SCC serial line logical unit.
 */
static inline int
SerialLineLogicalUnit( int line, int channel )
{
        return( (line << 2) | channel | 0x20 );
}

/*
 * Tuneable parameters.
 */
static const int InputBufferSize = 80;		// Read packet buffer size.
static const int InputDelay = 10000;		// Read packet delay (in usec).
static const int OutputBufferSize = 256;	// Write packet buffer size.
static const int ReadPacketCount = 3;

static const int SCC_RDPKT_REV_LEVEL = 1;

static const unsigned int SCC_ERR_INDEX	= 0xffff;
static const unsigned int SCC_ERR_FE = (1 << 21);
static const unsigned int SCC_ERR_PE = (1 << 19);


/*
 * A read request message the way the SCC expects to see it on a serial line
 * CRQ.
 */
class CRQ_SL_READ_MSG : public CRQ_MSG {
public:
	short	sl_read_data_limit;	/* maximum number of characters */
	short	sl_read_error_limit;	/* maximum number of errors */
	long	sl_read_delay; 		/* defined by free run counter */
	char *	sl_read_data;		/* address of data buffer */
	long *	sl_read_error;		/* address of error buffer */
	/*	sl_read_error[n] bits mean the following:
	 *	.[0,15]	index of error into receive buffer
	 *	.[16,18]
  	 *	.[19] = parity error
  	 *	.[20] = rcv uart overflow
  	 *	.[21] = framming error
  	 *	.[22] = break detect
	 *	.[23,31]
	 */
	short	sl_read_data_actual;		/* Actual characters read */
	short	sl_read_data_errors;		/* Actual number of errors */

	CRQ_SL_READ_MSG( int unitid, int limit, int delay, char * buffer,
		long * errorPtr );
};

inline
CRQ_SL_READ_MSG::CRQ_SL_READ_MSG( int unitid, int limit, int delay,
	char * buffer, long * errorPtr )
	: CRQ_MSG( sizeof( CRQ_SL_READ_MSG ), CRQOP_SL_READ, unitid, 0 )
{
	sl_read_data_limit = limit;
	sl_read_error_limit = 1;
	sl_read_delay = delay;
	sl_read_data = buffer;
	sl_read_error = errorPtr;
	sl_read_data_actual = 0;
	sl_read_data_errors = 0;
}

/*
 * For Choices, we add an error buffer and a data buffer to store the read
 * characters and any errors in.
 */
class SCCSL_RDPKT : public CRQ_SL_READ_MSG {
public:
	unsigned int scc_rd_error;		// Error buffer
	char scc_rd_buf[ InputBufferSize ];	// Data buffer

	SCCSL_RDPKT( int unitid = 0 );
};

inline
SCCSL_RDPKT::SCCSL_RDPKT( int unitid )
	: CRQ_SL_READ_MSG( unitid, InputBufferSize, InputDelay, &scc_rd_buf[0],
		(long *) &scc_rd_error )
{
	scc_rd_error = 0;
}

/*
 * A write request message the way the SCC expects to see it on a serial line
 * CRQ.
 */
static const int SL_WRITE_MSG_REV_LEVEL = 1;

class CRQ_SL_WRITE_MSG : public CRQ_MSG {
public:
	short	sl_write_char_count;
	short	sl_where_the_hell_did_this_come_from;
	char *	sl_write_buff;
	short	sl_write_actual;

	CRQ_SL_WRITE_MSG( int unitid, Semaphore * sem, char * buf, int count );
};

inline
CRQ_SL_WRITE_MSG::CRQ_SL_WRITE_MSG( int unitid, Semaphore * sem, char * buf,
	int count )
	: CRQ_MSG( sizeof( CRQ_SL_WRITE_MSG ), CRQOP_SL_WRITE, unitid, sem )
		
{
	sl_write_buff = buf;
	sl_write_char_count = count;
	sl_write_actual = 0;		// I have no idea why!
}

/*
 * For Choices, we add a semaphore to synchronize with the completion and
 * a buffer to store the characters to write in.
 */
class SCCSL_WRITEPKT : public CRQ_SL_WRITE_MSG {
public:
	char writebuf[ OutputBufferSize ];	// Data buffer

	SCCSL_WRITEPKT();
	SCCSL_WRITEPKT( int unitid, char * buffer, int count );
};

inline
SCCSL_WRITEPKT::SCCSL_WRITEPKT()
	: CRQ_SL_WRITE_MSG( 0, new BinarySemaphore( 0 ), 0, 0 )
{
}

inline
SCCSL_WRITEPKT::SCCSL_WRITEPKT( int unitid, char * buffer, int count )
    : CRQ_SL_WRITE_MSG( unitid, new BinarySemaphore( 0 ), &writebuf[0], count )
{
	/*
	 * Copy the data into the buffer.
	 */
	Assert( count <= OutputBufferSize );		// temporary until
							// I fix it to just do
							// multiple writes
							// or DMA or something..
	for( int i = 0; i < count; i++ ) {
		writebuf[i] = buffer[i];
	}
}

/*
 * Attentions from the SCC serial line CRQ's are all the same and supposedly
 * only show up on the recieve CRQ. I put some on the transmit CRQ just
 * in case.
 */
static const int SL_ATTN_REV_LEVEL = 1;

class CRQ_SL_ATTN_MSG : public CRQ_MSG {
public:
	long sl_attn_status;

	CRQ_SL_ATTN_MSG( int unitid = 0 );
};

inline
CRQ_SL_ATTN_MSG::CRQ_SL_ATTN_MSG( int unitid )
	: CRQ_MSG( sizeof( CRQ_SL_ATTN_MSG ), 0, unitid, 0 )
{
}
	
static const int SL_ATTN_RBUF =	0x04;	// Attention on ring buffer overflow
static const int SL_ATTN_REXSTS = 0x02;	// Attn on rcv chan on extern
					// status change

static const int SL_ATTN_RESET = 0x8000;	// Reset attention mode
static const int SL_ATTN_MODE = (SL_ATTN_RBUF | SL_ATTN_REXSTS | SL_ATTN_RESET);

/*
 * CRQ, Messages, etc. to talk to a serial line with.
 */
static char ReceiveCRQMem[ sizeof( CRQ ) ];		// Receive CRQ.
static char TransmitCRQMem[ sizeof( CRQ ) ];		// Transmit CRQ.

static CRQ * ReceiveCRQ = 0;
static CRQ * TransmitCRQ = 0;


static CreateChannelRequest CreateChannelMessage;
static CRQ_SL_ATTN_MSG ReceiveAttn[FreeAttentions];	// Attention messages.
static CRQ_SL_ATTN_MSG TransmitAttn[FreeAttentions];	// some for sends.
static SCCSL_RDPKT ReadBuffer[ReadPacketCount];		// Read packets.
static SCCSL_WRITEPKT WritePacket;			// A single write pkt.
static SemaphoreStar WritePacketMutex;

int
SendToConsole( char * buff, int count )
{
//	Debug.printf( "SendToConsole( buff:%A count:%d )\n", buff, count );
	if( count == 0 ) return( 0 );

	/*
	 * Lock the WritePacket. If necessary, there could be a pool of
	 * messages, but I dont see the need yet.
	 */
	WritePacketMutex->P();

	/*
	 * Initialize the message.
	 * This will copy the data into the buffer.
	 * Remember to use the physical address of the buffer.
	 * Later on add the code to lock the memory resident or reuse the
	 * bdl code that will have to do it for the ethernet.
	 */
	SCCSL_WRITEPKT * mesg = new ( &WritePacket ) SCCSL_WRITEPKT(
		TransmitCRQ->crq_unitid, buff, count );

	/*
	 * Send the command and await its completion.
	 */
//	Debug.printf( "SendToConsole: Semaphore sem:%A->P()\n", &mesg->semaphore() );
	TransmitCRQ->sendCommand( mesg );
	mesg->semaphore()->P();	// wait-synchronize

	/*
	 * InterruptHandler has already removed msg from the
	 * response queue so just check status.
	 */
//	Debug.printf( "SendToConsole: data sent\n" );
	Assert( mesg->status() == STS_SUCCESS );
	Assert( mesg->sl_write_actual >= count );

	/*
	 * Release the message.
	 */
	WritePacketMutex->V();

	return( count );
}

/*
 * These are routines implemented elsewhere.
 */
extern unsigned int ClassVector( unsigned int, unsigned int );

volatile int consoleInputInitialized;

void
InitConsole()
{
	Debug.printf( "InitConsole()\n" );

	CRQ * SCCCRQ = RequestorCRQ( SCC_SLOT, REQ_LUN );   // Ptr to SCC's CRQ.

	/*
	 * Set global variables that aren't be statically initialized.
	 */
	WritePacketMutex = new Semaphore( 1 );
	unsigned short rcvUnitId = UnitID(0, SCC_SLOT, 0,
		SerialLineLogicalUnit( 0, SL_READ_LUN ) );
	unsigned short xmitUnitId = UnitID(0, SCC_SLOT, 0,
		SerialLineLogicalUnit( 0, SL_WRITE_LUN ) );
	unsigned int receiveVector =
		ClassVector( CONSOLEIN_Vector, CPUCLASS_RUNNING );
	unsigned int transmitVector =
		ClassVector( CONSOLEOUT_Vector, CPUCLASS_RUNNING );
	Debug.printf( "InitConsoleInput: rcvUnitId: %x xmitUnitId: %x\n",
		rcvUnitId, xmitUnitId );
	Debug.printf( "InitConsoleInput: receiveVector: %x transmitVector: %x\n",
		receiveVector, transmitVector );
	Debug.printf( "InitConsoleInput: %d receive packets at %x\n",
		ReadPacketCount, &ReadBuffer[0] );

	/*
	 * Initialize the receive and transmit CRQs.
	 */
	ReceiveCRQ = new ( ReceiveCRQMem ) CRQ( rcvUnitId, receiveVector,
		CRQMODE_INTR );
	TransmitCRQ = new ( TransmitCRQMem ) CRQ( xmitUnitId, transmitVector,
		CRQMODE_INTR );

	Debug.printf( "InitConsoleInput: ReceiveCRQ: %A  TransmitCRQ: %A\n",
		ReceiveCRQ, TransmitCRQ );
	/*
	 * Initialize and enqueue the attention messages on the free list.
	 */
	for( int j = 0; j < FreeAttentions; j++ ) {
		(void) new ( &ReceiveAttn[j] ) CRQ_SL_ATTN_MSG( rcvUnitId );
		ReceiveCRQ->enqueueFreeMessage( &ReceiveAttn[j] );
	}
	for( j = 0; j < FreeAttentions; j++ ) {
		(void) new ( &TransmitAttn[j] ) CRQ_SL_ATTN_MSG( xmitUnitId );
		TransmitCRQ->enqueueFreeMessage( &TransmitAttn[j] );
	}

	/*
	 * Create a CRQ channel for console input requests.
	 */
	CreateChannelMessage.setup( ReceiveCRQ );
	
	/*
	 * Send the command and wait for a reply.
	 */
	Debug.printf( "InitConsole() sending input CRQ create request\n" );
	SCCCRQ->sendCommand( &CreateChannelMessage );
	CreateChannelMessage.semaphore()->P();

	/*
	 * Verify successful channel creation.
	 */
	Debug.printf( "InitConsole: INPUT message/channel status: %x/%x\n",
		CreateChannelMessage.status(),
		CreateChannelMessage.resultStatus() );
	Assert( CreateChannelMessage.status() == STS_SUCCESS );
	Assert( CreateChannelMessage.resultStatus() == SCC_CRQ_CHAN_CREATED );

	/*
	 * Create a CRQ channel for console output requests.
	 */
	CreateChannelMessage.setup( TransmitCRQ );

	/*
	 * Send the command and wait for a reply.
	 */
	Debug.printf( "InitConsole() sending output CRQ create request\n" );
	SCCCRQ->sendCommand( &CreateChannelMessage );
	CreateChannelMessage.semaphore()->P();

	/*
	 * Verify successful channel creation.
	 */
	Debug.printf( "InitConsole: OUTPUT message/channel status: %x/%x\n",
		CreateChannelMessage.status(),
		CreateChannelMessage.resultStatus() );
	Assert( CreateChannelMessage.status() == STS_SUCCESS );
	Assert( CreateChannelMessage.resultStatus() == SCC_CRQ_CHAN_CREATED );

	/*
	 * Initialize and enqueue a couple of read packets on the command list.
	 */
	for( j = 0; j < ReadPacketCount; j++ ) {
		(void) new ( &ReadBuffer[j] ) SCCSL_RDPKT( rcvUnitId );
		ReceiveCRQ->sendCommand( &ReadBuffer[j] );
	}

	Debug.printf( "Console initialized\n" );

	consoleInputInitialized = 1;
}

/*
 * ConsoleOutputInterruptHandler. Called from the console output interrupt
 * process when a TransmitCRQ vector interrupt occurs.
 */
void
ConsoleOutputInterruptHandler()
{
//	Debug.printf( "ConsoleOutputInterruptHandler()\n" );

	CRQ_MSG * rsp;
	CRQ_MSG * attn;
	CRQ * crq = TransmitCRQ;

	int didWork = 0;

	do {
		/*
		 * If there is an attenetion message waiting, handle it.
		 */
		if( ( attn = crq->dequeueAttentionMessage() ) != 0 ) {
			switch( attn->code() ) {
			   default:
				crq->handleStandardAttention( attn );
				break;
			}
			crq->enqueueFreeMessage( attn );
			didWork = 1;
		}

		/*
		 * If there is a response message waiting, handle it.
		 */
		if( ( rsp = crq->dequeueResponse() ) != 0 ) {
			switch( rsp->code() ) {
			   case CRQOP_SL_WRITE:
//				Debug.printf( "ConsoleOutputInterrupt: WRITE\n" );
				Assert( rsp->status() == STS_SUCCESS );
				Assert( rsp->semaphore() != 0 );
//				Debug.printf( "ConsoleOutputInterrupt: %A->V()\n",
//					rsp->semaphore() );
	
	SCCSL_WRITEPKT * pkt = (SCCSL_WRITEPKT *) rsp;
//	Debug.printf( "buffer = %s\n", pkt->writebuf );
				//rsp->semaphore()->V();
	pkt->semaphore()->V();
//				Debug.printf( "ConsoleOutputInterrupt: V'ed\n" );
				break;
			   default:
				Console().cpuPrintf( "ConsoleOutputInterrupt: BAD RESPONSE\n" );
				break;
			}
			didWork = 1;
		}
	} while( ( attn != 0 ) || ( rsp != 0 ) );

	if( ! didWork ) Console().cpuPrintf( "ConsoleOutputInterrupt: NO WORK?\n" );
//	Debug.printf( "ConsoleOutputInterrupt: done\n" );
}

/*
 * ConsoleInputInterruptHandler. Called from the console input interrupt
 * process when an ReceiveCRQ vector interrupt occurs.
 */
void
ConsoleInputInterruptHandler()
{
	CRQ_MSG * rsp;
	CRQ_MSG * attn;
	CRQ * crq = ReceiveCRQ;

	int didWork = 0;

	Debug.printf( "ConsoleInputInterruptHandler()\n" );
	do {
		/*
		 * If there is an attenetion message waiting, handle it.
		 */
		if( ( attn = crq->dequeueAttentionMessage() ) != 0 ) {
			switch( attn->code() ) {
			   default:
				crq->handleStandardAttention( attn );
				break;
			}
			crq->enqueueFreeMessage( attn );
			didWork = 1;
		}

		/*
		 * If there is a response message waiting, handle it.
		 */
		if( ( rsp = crq->dequeueResponse() ) != 0 ) {
			switch( rsp->code() ) {
			   case CRQOP_SL_READ:
				/*
				 * Check for weird conditions.
				 */
				Debug.printf( "ConsoleInterrupt:status: %x\n",
					rsp->status() );
				Assert( rsp->status() == STS_SUCCESS );

				void enqueueConsoleInput( SCCSL_RDPKT * );
				enqueueConsoleInput( (SCCSL_RDPKT *) rsp );

				/*
				 * Re-initialize and re-send the read command.
				 */
				SCCSL_RDPKT * pkt = new ( rsp )
					SCCSL_RDPKT( crq->crq_unitid );
				crq->sendCommand( pkt );
				break;
			   default:
				Console().cpuPrintf( "ConsoleInput: BAD RESPONSE\n" );
				break;
			}
			didWork = 1;
		}
	} while( ( attn != 0 ) || ( rsp != 0 ) );

	if( ! didWork ) Console().cpuPrintf( "ConsoleInputInterruptHandler: NO WORK?\n" );
	Debug.printf( "ConsoleInputInterruptHandler: done\n" );
}

extern CharacterQueue * consoleInputQueue;
/*
 * enqueueConsoleInput - enqueue console input characters for reading.
 */
void
enqueueConsoleInput( SCCSL_RDPKT * rdpkt )
{
	int char_cnt;
	int err_cnt;
	/*
	 * Get characters from read packet.
	 */
	Debug.printf( "enqueueConsoleInput: actual/errors: %d/%d\n",
		rdpkt->sl_read_data_actual, rdpkt->sl_read_data_errors );
	for ( char_cnt = 0, err_cnt = rdpkt->sl_read_data_errors;
	      char_cnt < rdpkt->sl_read_data_actual; char_cnt++) {
		/*
		 * Get next character.
		 */
		char nxt_char = rdpkt->scc_rd_buf[char_cnt] & 0x7f;

		/*
		 * Handle an erroneous character.
		 */
		if( err_cnt > 0 ) {
			if((rdpkt->scc_rd_error & SCC_ERR_INDEX) == char_cnt) {
				err_cnt--;
				if(rdpkt->scc_rd_error & SCC_ERR_FE) {
					Console().cpuPrintf( "Console framing error.\n" );
					continue;
				}
				if(rdpkt->scc_rd_error & SCC_ERR_PE) {
					Console().cpuPrintf( "Console parity error.\n" );
					continue;
				}
				Assert( NOTREACHED );
			}

			/*
			 * If we got here, then this wasn't one of the bad ones.
			 */
		}

		/*
		 * Put the character in the console input character queue.
		 */
		ConsoleInputStream->enqueue( &nxt_char, 1 );
	}
	Debug.printf( "enqueueConsoleInput: done\n" );
}
