#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Michael J. North (north@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * TapeProcess.c: the process that awaits disk interrupts
 */
#include "Assert.h"
#include "OutputStream.h"
#include "AwaitedInterruptInterrupt.h"
#include "Process.h"
#include "ThisProcess.h"
#include "CPU.h"
#include "ThisCPU.h"
#include "Vectors.h"
#include "Domain.h"


#ifdef __GNUC__
#pragma implementation
#endif

static const int EMC_SLOT = 4;		// NUKE THIS
volatile int tapeInitialized = 0;

static void
TapeInterruptProcessEntry( int vector )
{
	Debug.printf( "TapeInterrupProcessEntry( vector:%x )\n", vector );

	AwaitedInterruptInterrupt * interrupt = new AwaitedInterruptInterrupt();
	thisCPU()->setInterrupt( vector, interrupt );

	/*
	 * Loop forever waiting for tape interrupts and then processing
	 * them.
	 */
	while( 1 ) {
		interrupt->await();

		/*
		 * Process the interrupt.
		 */
		Debug.printf( "TapeInterruptProcess: awoken\n" );
		extern void TapeInterruptHandler();
		TapeInterruptHandler();
	}
}

void
TapeManagerProcessEntry( int arg )
{
	Debug.printf( "TapeManagerProcessEntry( arg:%x )\n", arg );

	/*
	 * Before trying to initialize the tape drive start up a process to wait
	 * for the tape interrupts since the initialization code will try
	 * to read the disk to get the header.
	 */
	Process * process = new InterruptProcess( 
		(PFV) TapeInterruptProcessEntry, thisProcess()->domain(),
		normal, TAPE_Vector );
	Assert( process != 0 );
	Debug.printf( "TapeManagerProcess: interrupt process %A\n", process );
	process->setName( "#TapeInterrrupt" );

	/*
	 * Ready the process to run.
	 * Since it's an interrupt process it should have high enough
	 * priority to run before this process is rescheduled. (I hope).
	 */
	process->ready();
	thisProcess()->relinquishProcessor();

	/*
	 * Call the tape initialization code.
	 */
	extern void InitTape();

	Debug.printf( "TapeManagerProcess: interrupt process started\n" );
	InitTape();
	Debug.printf( "TapeProcessEntry: disk initialized\n" );
	tapeInitialized = 1;
}
