#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * OMAPFreeRunningTimer.cc
 */
#include "Assert.h"
#include "FreeRunningTimerStar.h"
#include "OutputStream.h"
#include "OMAPFreeRunningTimer.h"
#include "FreeRunningTimerStar.h"
#include "GenericARMTimeOfDay.h"
#include "OMAPTimer.h"
#include "OMAPInterruptManager.h"
#include "ThisCPU.h"
#include "Vectors.h"

// Pings are temporarily issued directly to hardware
// until the watchdog timer gets its own thread
// This is needed because the caller stack needs to be RW for the component
//#ifdef RELIABILITY
//#include "OMAPWatchdogTimerDD.h"
//extern OMAPWatchdogTimerDD watchdog;
//#else
#include "OMAPWatchdogTimer.h"
extern OMAPWatchdogTimer watchdog;
//#endif

#ifdef __GNUC__
#pragma implementation
#endif


OMAPMPUTimer *omapTimer1 = (OMAPMPUTimer *)OMAP_MPU_TIMER1_BASE;
OMAPMPUTimer *omapTimer2 = (OMAPMPUTimer *)OMAP_MPU_TIMER2_BASE;
OMAPMPUTimer *omapTimer3 = (OMAPMPUTimer *)OMAP_MPU_TIMER3_BASE;

OMAPFreeRunningTimer theSystemTimer;
FreeRunningTimerStar SystemTimer = &theSystemTimer;

unsigned OMAPFreeRunningTimer::constructed = 0;
unsigned OMAPFreeRunningTimer::timesCalled;
unsigned OMAPFreeRunningTimer::interrupts;
unsigned OMAPFreeRunningTimer::prevTime;
Lock * OMAPFreeRunningTimer::FreeRunningTimerLock;

extern unsigned long getCRC32(unsigned address, unsigned numBytes);
extern unsigned reloadKernelFromFlash(unsigned address);

void
OMAPFreeRunningTimerInterrupt::basicRaise(char *)
{
    // Debug << "OMAPFreeRunningTimerInterrupt::basicRaise()\n" << eor;
    Assert (SystemTimer != 0);

    // Kick the watchdog
    watchdog.ping();

#ifdef RELIABILITY
    // CRC boot.s (4K)
    static unsigned bootcrc = 0;
    unsigned newcrc = getCRC32(0x10000000,0x1000);
    if(bootcrc) {
        if (bootcrc != newcrc) {
            // CRC failed. Reload kernel
            Console() << "CRC failed. Reloading kernel...";
            reloadKernelFromFlash(0);
            Console() << "done!\n" << eor;
        }
    } else {
        bootcrc = newcrc;
    }
#endif

    theSystemTimer.tick();
}

OMAPFreeRunningTimer::OMAPFreeRunningTimer()
{
    Assert ( !constructed );
    lastTime = 0;
    timesCalled = 0;
    interrupts = 0;
    prevTime = 0;
    FreeRunningTimerLock = new Lock();
    FreeRunningTimerLock->acquire();
    constructed++;
    FreeRunningTimerLock->release();
}

void
OMAPFreeRunningTimer::tick()
{
    // Debug << "OMAPFreeRunningTimer::tick()\n" << eor;
    FreeRunningTimerLock->acquire();
    unsigned int prevTime = lastTime;
    lastTime += FRTuSecPerInterrupt;
    interrupts++;
    if (theSystemClock != 0 && prevTime > lastTime) 
        ((GenericARMTimeOfDay *) theSystemClock)->systemTimerOverflow();
    FreeRunningTimerLock->release();
}


unsigned int
OMAPFreeRunningTimer::time()
{
    Assert (FreeRunningTimerLock != 0);
    FreeRunningTimerLock->acquire();
    timesCalled++;
    unsigned t;
    unsigned timerval;
    unsigned correctedLastTime;
    unsigned timertime;

    timerval = omapTimer1->read();
    correctedLastTime = lastTime;

    timertime = (unsigned)((float)(FRTTickPeriod - timerval) / FRTTickPeriod * FRTuSecPerInterrupt);

    // If there is a pending interrupt, add FRTuSecPerInterrupt to correctedLastTime
    // Note: If interrupts are off for more than FRTuSecPerInterrupt, we could have missed a tick
    // Therefore, interrupts should NEVER be off for more than FRTuSecPerInterrupt
    if (dynamic_cast<OMAPInterruptManager *>(thisCPU()->getInterruptManager())->isPending(VEC_TIMER1)) {
        correctedLastTime += FRTuSecPerInterrupt;
    }
    
    t = correctedLastTime + timertime; 
    
    Debug << this << " returns " << t << " microseconds\n" << eor;

    FreeRunningTimerLock->release();
    return (t);
}

#include "ClassInstaller.h"
DEFINECLASS("OMAPFreeRunningTimer",OMAPFreeRunningTimer,FreeRunningTimer,0,0);
