#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Francis David <fdavid@uiuc.edu>
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * OMAPWatchdogTimer.cc
 */
#include "Assert.h"
#include "OutputStream.h"
#include "OMAPWatchdogTimer.h"

OMAPWatchdog *omapWatchdog = (OMAPWatchdog *)OMAP_WATCHDOG_BASE;

unsigned OMAPWatchdogTimer::start( unsigned seconds ) {
    // Enable Watchdog Clock
    unsigned *control = (unsigned *)ARM_IDLECT2;
    *control |= (1 << EN_WDTCK);

    omapWatchdog->disable();
    omapWatchdog->setTimeout(seconds);
    omapWatchdog->enable();
    OMAPWatchdogTimer::ping(); // Avoid problems with wrappers and virtual functions
    return seconds;
}

unsigned OMAPWatchdogTimer::stop() {
    omapWatchdog->disable();

    // Disable Watchdog Clock
    unsigned *control = (unsigned *)ARM_IDLECT2;
    *control &= ~(1 << EN_WDTCK);

    return 0;
}

unsigned OMAPWatchdogTimer::residual() {
    Assert(NOTREACHED);
    return 0;
}

void OMAPWatchdogTimer::ping() {
    omapWatchdog->ping();
}
