#ifdef __GNUC__
#pragma implementation
#endif

#include "OMAPWatchdogTimerDD.h"
#include "MACHINE_CONFIGURATION.h"
#include "ARMPageTableConstants.h"

void OMAPWatchdogTimerDD::basicInit() {
    AddressTranslation* translation = domain_->translation();
    translation->changeProtection((char *)OMAP_WATCHDOG_BASE, PhysicalPageSize, ReadWrite);
    translation->changeProtection((char *)CLKGEN_REG_BASE, PhysicalPageSize, ReadWrite);

#ifdef DEBUG
    translation->changeProtection((char *)MACHINE_UART_BASE, PhysicalPageSize, ReadWrite);
#endif
}

unsigned OMAPWatchdogTimerDD::start(unsigned seconds) {
    Exception* ex = 0;
    unsigned ret = 0;
    char *oldStackPointer_;
    
    init();

    IntStorage intState ; PROCESSOR::disableInterrupts(intState);

    // Save current stack
    asm( "mov %0, sp":"=r"(oldStackPointer_) );
    switchToDriverDomain();
    asm( "mov r13, %0"::"r"(newStackPointer_) );

    try{
        ret = OMAPWatchdogTimer::start(seconds);
    } catch (Exception* e){
        ex = e;
    }

    switchToKernelDomain();

    // Restore stack
    asm("mov sp, %0"::"r"(oldStackPointer_));

    PROCESSOR::restoreInterrupts(intState);

    if(ex) throw ex;

    return ret;
}

unsigned OMAPWatchdogTimerDD::stop() {
    Exception* ex = 0;
    unsigned ret = 0;
    char *oldStackPointer_;

    init();

    IntStorage intState ; PROCESSOR::disableInterrupts(intState);

    // Save current stack
    asm( "mov %0, sp":"=r"(oldStackPointer_) );
    switchToDriverDomain();
    asm( "mov r13, %0"::"r"(newStackPointer_) );

    try{
        ret = OMAPWatchdogTimer::stop();
    } catch (Exception* e){
        ex = e;
    }

    switchToKernelDomain();

    // Restore stack
    asm("mov sp, %0"::"r"(oldStackPointer_));

    PROCESSOR::restoreInterrupts(intState);

    if(ex) throw ex;

    return ret;
}

unsigned OMAPWatchdogTimerDD::residual() {
    Exception* ex = 0;
    unsigned ret = 0;
    char *oldStackPointer_;

    init();

    IntStorage intState ; PROCESSOR::disableInterrupts(intState);

    // Save current stack
    asm( "mov %0, sp":"=r"(oldStackPointer_) );
    switchToDriverDomain();
    asm( "mov r13, %0"::"r"(newStackPointer_) );

    try{
        ret = OMAPWatchdogTimer::residual();
    } catch (Exception* e){
        ex = e;
    }

    switchToKernelDomain();

    // Restore stack
    asm("mov sp, %0"::"r"(oldStackPointer_));

    PROCESSOR::restoreInterrupts(intState);

    if(ex) throw ex;

    return ret;
}

void OMAPWatchdogTimerDD::ping() {
    Exception* ex = 0;
    char *oldStackPointer_;

    init();

    IntStorage intState ; PROCESSOR::disableInterrupts(intState);

    // Save current stack
    asm( "mov %0, sp":"=r"(oldStackPointer_) );
    switchToDriverDomain();
    asm( "mov r13, %0"::"r"(newStackPointer_) );

    try{
        OMAPWatchdogTimer::ping();
    } catch (Exception* e){
        ex = e;
    }

    switchToKernelDomain();

    // Restore stack
    asm("mov sp, %0"::"r"(oldStackPointer_));

    PROCESSOR::restoreInterrupts(intState);

    if(ex) throw ex;
}
