/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Contributing Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ConsoleProcess.c: the process that awaits console characters
 */

#include "Assert.h"
#include "OutputStream.h"
#include "CPU.h"
#include "AwaitedInterruptInterrupt.h"
#include "ThisProcess.h"
#include "ThisCPU.h"
#include "Vectors.h"
#include "Process.h"

static const int SCC_SLOT = 12;		// TEMPORARY

static void
ConsoleInterruptProcessEntry( int vector )
{
	Debug.printf( "ConsoleInterruptProcessEntry( vector:%x )\n", vector );

	AwaitedInterruptInterrupt * interrupt = new AwaitedInterruptInterrupt();
	thisCPU()->setInterrupt( vector, interrupt );

	/*
	 * Loop forever waiting for a character interrupt and then processing
	 * the input character.
	 */
	while( 1 ) {
		Debug.printf( "ConsoleProcessEntry: awaiting interrupt\n" );
		interrupt->await();
		Debug.printf( "ConsoleProcessEntry: await returned\n" );

		/*
		 * Process the input.
		 */
		extern void ConsoleInputInterruptHandler();
		ConsoleInputInterruptHandler();
	}
}

static void
ConsoleOutputInterruptProcessEntry( int vector )
{
	Debug.printf( "ConsoleOutputInterruptProcessEntry( vector:%x )\n",
		vector );

	AwaitedInterruptInterrupt * interrupt = new AwaitedInterruptInterrupt();
	thisCPU()->setInterrupt( vector, interrupt );

	/*
	 * Loop forever processing interrupts.
	 */
	while( 1 ) {
		Debug.printf( "ConsoleOutputInterruptProcess: awaiting interrupt\n" );
		interrupt->await();
		Debug.printf( "ConsoleOutputInterruptProcess: got interrupt\n" );

		/*
		 * Process the interrupt.
		 */
		extern void ConsoleOutputInterruptHandler();
		ConsoleOutputInterruptHandler();
	}
}

void
ConsoleManagerProcessEntry( int arg )
{
	Debug.printf( "ConsoleManagerProcessEntry( arg:%x )\n", arg );

	/*
	 * Before initializing the console input, start up processes to
	 * wait for and handle the console interrupts.
	 */
	Process * process = new InterruptProcess( 
		(PFV) ConsoleInterruptProcessEntry, thisProcess()->domain(),
		normal, CONSOLEIN_Vector );
	Assert( process != 0 );
	process->setName( "#ConsoleInterrrupt" );
	Debug.printf( "ConsoleManagerProcess: interrupt process at %A\n",
		process );

	/*
	 * Put the interrupt process in its ready container and then
	 * relinquish the processor and hope the interrupt process runs.
	 */
	process->ready();
	thisProcess()->relinquishProcessor();

	/*
	 * Next, start the output interrupt process.
	 */
	process = new InterruptProcess( 
		(PFV) ConsoleOutputInterruptProcessEntry,
		thisProcess()->domain(),
		normal, CONSOLEOUT_Vector );
	Assert( process != 0 );
	process->setName( "#ConsoleOutputInterrrupt" );
	Debug.printf( "ConsoleManagerProcess: output process at %A\n",
		process );

	process->ready();
	thisProcess()->relinquishProcessor();

	/*
	 * Call the console initilization code.
	 */
	Debug.printf( "ConsoleManagerProcess: interrupt processes started\n" );
	extern void InitConsole();
	InitConsole();
	Debug.printf( "ConsoleProcessEntry: console initialized\n" );

	/*
	 * Just fall off the end to terminate the process.
	 */
}
