#define DEBUG
/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Contributing Author: See-Mong Tan (stan@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * DatakitCon.cc - Datakit con program
 */

#include "Assert.h"
#include "OutputStream.h"
#include "ThisProcess.h"
#include "DatakitURPChannel.h"
#include "Semaphore.h"
#include "DatakitUnixHost.h"
#include "DatakitCon.h"

extern void DatakitConConsoleReadEntry( int );
extern void DatakitConURPChannelReadEntry( int );

extern int ConsoleRead( char *, int );

DatakitCon::DatakitCon( char * dialstring )
{
    Debug << "DatakitCon::DatakitCon( dialstring=" << dialstring << " )\n"
	<< eor;
    Assert( unixHostManager != 0 );
    myChannel = unixHostManager->open( dialstring );
    Assert( myChannel != 0 );

    // spawn process to listen on console
    Process * process = new SystemProcess( (PFV) &DatakitConConsoleReadEntry,
					  thisProcess()->domain(),
					  normal, (int) myChannel );
    Assert( process != 0 );
    process->ready();
    
    // Spawn process to listen on URP channel
    process = new SystemProcess( (PFV) &DatakitConURPChannelReadEntry,
				thisProcess()->domain(),
				normal, (int) myChannel );
    Assert( process != 0 );
    process->ready();
}
    
void
DatakitConConsoleReadEntry( int c )
{
    DatakitURPChannel * chan = (DatakitURPChannel *) c;
    Assert( chan != 0 );
    Debug << "DatakitConConsoleReadEntry( chan=" << chan << " )\n" << eor;

    for(;;) {
	char buf[512];

	int rlen = ConsoleRead( (char *) &buf, 512 );
	// Add a newline to the back of the string, and increment length
	buf[rlen++] = '\n';
	chan->Write( (unsigned char *) &buf, rlen );
    }
}

void
DatakitConURPChannelReadEntry( int c )
{
    DatakitURPChannel * chan = (DatakitURPChannel *) c;
    Assert( chan != 0 );
    Debug << "DatakitConURPChannelReadEntry( chan=" << chan << " )\n" << eor;

    for(;;) {
	char buf[512];
	
	int rlen = chan->Read( (unsigned char *) &buf, 512 );
	Console().write( (const char *) &buf, rlen );
    }
}
    


#include "ClassInstaller.h"
DEFINECLASS DatakitCon
