/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Contributing Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * PCi586FreeRunningTimer.cc
 */
#include "Assert.h"
#include "FreeRunningTimerStar.h"
#include "PCi586FreeRunningTimer.h"
#include "PCi586IntervalTimer.h"
#include "PCi586InterruptController.h"
#include "Util.h"
#include "ClassStar.h"

/*
 * There is only a single instance.
 */
PCi586FreeRunningTimer theSystemTimer;
FreeRunningTimerStar SystemTimer = &theSystemTimer;

unsigned int
PCi586FreeRunningTimer::time()
{
	unsigned int t;
	unsigned short timerval;

	/*
	 * This lock is just to disable interrupts; on a multiprocessor it
	 *  wouldn't be enough to protect the timer port, but this code is
	 *  machine-specific.
	 */
	lock.acquire();
	Out(PITCTL_PORT, PIT_SEL0|PIT_LATCH);
	timerval = In(PITCTR0_PORT);
	timerval |= In(PITCTR0_PORT) << 8;
	/*
	 * Want to multiply by 1 million and divide by PIT_HZ to get
	 *  microseconds, but that would cause overflow.  Instead,
	 *  divide each of those factors by 20.
	 */
	t = lastTime + (((unsigned int)(PITCount - timerval)) *
					(1000000/20) / (PIT_HZ/20));

	if (In(PIC_CTRL_REG) & PIC_TIMER_MASK) {
		/*
		 * Timer interrupt pending.  If the timer hasn't wrapped
		 *   around since reading timerval, add a tick's worth
		 *   of time to the return value.
		 */
		unsigned short timerval2;
		Out(PITCTL_PORT, PIT_SEL0|PIT_LATCH);
		timerval2 = In(PITCTR0_PORT);
		timerval2 |= In(PITCTR0_PORT) << 8;
		if (timerval2 < timerval)
			t += 1000000/ClockFrequency;
	}
	lock.release();
	return (t);
}



#include "ClassInstaller.h"
DEFINECLASS("PCi586FreeRunningTimer",PCi586FreeRunningTimer,FreeRunningTimer,0,0);
