#define USEPCI586BACKINGSTORE
#define  NEW
/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Contributing Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * PCi586Kernel.cc: PCi586 kernel class implementation.
 */

#include "Assert.h"
//#include "PCi586EthernetDriver.h"
#include "PCi586Kernel.h"
#include "PCi586Multiboot.h"
#include "PCi586i586CPU.h"
#include "CharacterInputStream.h"
#include "CharacterQueue.h"
//#include "CPUCollection.h"
#include "Interrupt.h"
#include "FileSystemInterface.h"
#include "NameServer.h"
#include "OutputStream.h"
#include "PhysicalMemoryRange.h"
#include "ThisProcess.h"
#include "Store.h"
#include "Vectors.h"
#include "PROCESSOR.h"

#include "i586PageTableConstants.h"

PCi586Kernel::PCi586Kernel()
{
	Console() << "(PC i586 Version)\n" << eor;
}

PCi586Kernel::~PCi586Kernel()
{
	Assert( NOTREACHED );
}

void
PCi586Kernel::buildRunningCPU()
{
	/*
	 * Build a CPU object for this processor.
	 */
	CPU * current = new PCi586i586CPU( PROCESSOR::cpuID(),
		_initialScheduler );
	current->setName( "#CPU" );
	((i586CPU *)current)->setRingRange(0, kernelStart(), 
		kernelStart() + kernelSize());
	((i586CPU *)current)->setRingRange(1, applicationStart(), 
		applicationStart() + applicationSize());
	/*
	 * Initialize the Interrupt Descriptor Table entries
	 */
	current->installInterrupts();
	Assert( !PROCESSOR::interruptsEnabled() );
}


const unsigned          
PCi586Kernel::physicalMemorySize()
{
  return 0;
}

/*
 * Figure out and display our machine configuration.
 * Return the map of physical memory.
 */
PhysicalMemoryRange *
PCi586Kernel::physicalMemoryMap()
{
  Debug << "PCi586Kernel::physicalMemoryMap()\n" << eor;
  
  unsigned i;
  
  Debug << "multiboot flags: " << asHex(multiboot_info.flags) << "\n" << eor;
  
  for(i=0; i < num_drives; i++) {
    Debug << "drive[" << i << "]:" <<
      " cyl: " << drive_info[i].drive_cylinders <<
      ", head: " << drive_info[i].drive_heads <<
      ", sect: " << drive_info[i].drive_sectors <<
      ", mode: " << ((drive_info[i].drive_mode == 0) ? "CHS" : "LBA") <<
      "\n" << eor;
  }
  
  for(i = 0; i < num_memory_maps; i++) {
    Debug << "memory_map[" << i << "]:" <<
      " base: " << asHex(memory_map[i].base_addr_low) <<
      ", length: " << asHex(memory_map[i].length_low) <<
      ((memory_map[i].type == 1) ? ", AVAILABLE" : ", RESERVED") <<
      "\n" << eor;
  }
  
  /**** NOTE: Memory must be mapped 1-1 through memory_map[1].length_low 
   *    so we can skip past the PC screen memory
   *****/
  int bytesOfPhysicalMemory;
  unsigned int unitsOfPhysicalMemory;
  
  bytesOfPhysicalMemory =
    (int) (memory_map[0].length_low);
  unitsOfPhysicalMemory = bytesOfPhysicalMemory / PhysicalPageSize;
  
  PhysicalMemoryRange * memoryMap = 
    new PhysicalMemoryRange( (const char*) memory_map[0].base_addr_low, 
			     unitsOfPhysicalMemory,  Log2PhysicalPageSize,
			     "#Memory (RAM)",0);
  
  //NOTE: NN Hack
  // Mapping range between low and high mmap passed by multiboot-compliant bootloader.
  // Includes Video memory
  
  unsigned int beginAddr = memory_map[0].base_addr_low + memory_map[0].length_low;
  beginAddr = memoryMap->ceilingUnit( beginAddr );
  
  bytesOfPhysicalMemory =
    (int) (memory_map[1].base_addr_low - beginAddr);
  unitsOfPhysicalMemory = bytesOfPhysicalMemory / PhysicalPageSize;
  
  memoryMap->add ( new PhysicalMemoryRange(
	  (const char*)(beginAddr), unitsOfPhysicalMemory, 
	  Log2PhysicalPageSize,"#Memory (RAM)",1) );
  
  // End NN Hack
  
  bytesOfPhysicalMemory =
    (int) (memory_map[1].length_low);
  unitsOfPhysicalMemory = bytesOfPhysicalMemory / PhysicalPageSize;
  
  memoryMap->add ( new PhysicalMemoryRange(
					   (const char*)memory_map[1].base_addr_low, unitsOfPhysicalMemory, 
					   Log2PhysicalPageSize,"#Memory (RAM)",0) );
  
  return( memoryMap );
}

/*
 * Setup machine dependent devices and device managers.
 */
void
PCi586Kernel::basicInitializeDevicesAndManagers()
{
	Debug << "PCi586Kernel::basicInitializeDevicesAndManagers()\n" << eor;
	Assert( _initialScheduler != 0 );

	// configure the disk driver, argument is 0 for drive 0
	theDisk = new PCi586Disk( 0 );

	/* 
	 * Set the backspace character for the console input stream to
	 * be ^H instead of delete.  (WHAT A HACK!)
	 */
	LineBufferedEchoingCharacterQueue * queue =
		(LineBufferedEchoingCharacterQueue *)
			ConsoleInputStream->queue();
	queue->setSpecialChar( Backspace, '\b' );

	/*
	 * Set up machine dependent (Multimax) manager processes and
	 * devices.
	 */
	Process * process;

	/*
	 * Build a stack for and construct the console manager process,
	 */
	Debug << "Building console manager process\n" << eor;
	extern void ConsoleManagerProcessEntry( int );
	process = new SystemProcess( (PFV) ConsoleManagerProcessEntry,
		domain(), normal, 0 );
	Debug << "console process " << process << "\n" << eor;
	Assert( process != 0 );
	process->ready();
	process->setName( "#ConsoleManagerProcess" );

	Debug << "Console taken care of\n" << eor;

	/*
	 * Next start up the disk.
	 */
	Debug << "Starting up the Disk\n" << eor;
	theDisk->initialize();
	Debug << "Disk taken care off\n" << eor;

#ifdef DATAKIT
	Debug << "Starting Datakit Driver Process\n" << eor;
	extern void DatakitDriverProcessEntry( int );
	process = new SystemProcess( (PFV) DatakitDriverProcessEntry,
				    domain(), normal, 0 );
	Assert( process != 0 );
	process->ready();
	process->setName( "#DatakitDriverProcess" );
	Debug << "Datakit Driver taken care of\n" << eor;
#endif

	/*
	 * Now start the Timeout timer & Periodic timer helper processes.
	 *
	 */
	Debug << "Build Timeout Manager process\n" << eor;
	extern void TimeoutTimerEntry( int );
	process = new SystemProcess( (PFV) TimeoutTimerEntry,
		domain(), normal, 0 );
	Debug << "Timeout manager process " << process << "\n" << eor;
	Assert( process != 0 );
	process->setName( "#TimeoutTimerManager" );
	process->ready();

	Debug << "Build Periodic Manager process\n" << eor;
	extern void PeriodicTimerEntry( int );
	process = new SystemProcess( (PFV) PeriodicTimerEntry,
		domain(), normal, 0 );
	Debug << "Periodic manager process " << process << "\n" << eor;
	Assert( process != 0 );
	process->setName( "#PeriodicTimerManager" );
	process->ready();

}


void
PCi586Kernel::basicInitializeNetworkProcesses()
{
#if 0
         /*
         * Start up the 3c503 ethernet card driver.
         */
        Debug << "Starting Ethernet Driver\n" << eor;
        Assert( theEthernetDriver == 0 );
        theEthernetDriver = new PCi586EthernetDriver();
        Assert( theEthernetDriver != 0 );
        Debug << "Ethernet Driver started\n" << eor;
	theEthernetDriver->setName( "ethernetDriver" );
	// add to name server so we can access it
	// temporary measure, till the EthernetAdministrator works
	StandardNameServer->bind( "ethernetDriver", theEthernetDriver );
#endif
}

DiskRef
PCi586Kernel::rootDisk()
{
	return(theDisk);
}


void
PCi586Kernel::mountNonRootFileSystems()
{
	/* none for now */
}

void
PCi586Kernel::initializeBackingStore()
{
#ifndef USEPCI586BACKINGSTORE
	/*
	 * Use the root file system for backing store objects
	 */
  /*Comment this out for now, since we don't want to deal with the backing store yet. 
    MDB: 4/29/04
    
	_backingStoreSource = oc;
  */
#endif
}

#ifdef SNEAKYMO

#include "PersistentMemoryObject.h"
#include "PhysicalMemoryChain.h"

const int SNEAKYBUFSIZE = 81920;
const int NUMSNEAKYBUFS = 8;

class SneakyCopyMemoryObject : public PersistentMemoryObject {
private:
	static unsigned char buf[NUMSNEAKYBUFS][SNEAKYBUFSIZE];
	static unsigned int inuse[NUMSNEAKYBUFS];
	int mynum;
public:
	SneakyCopyMemoryObject(int);
	~SneakyCopyMemoryObject();
	int read(unsigned int, int, PhysicalMemoryChain *);
	int write(unsigned int, int, PhysicalMemoryChain *);
};

const int SneakyPageSize = 1024;
const int log2SneakyPageSize = 10;


SneakyCopyMemoryObject::SneakyCopyMemoryObject(int size)
  : (size, log2SneakyPageSize, 0, 0 )
{
    /*
     * Can't do 'new' because it wouldn't allocate in physical memory
     */
    for (int i = 0; i < NUMSNEAKYBUFS; i++) {
	if (!inuse[i]) {
	    inuse[i] = 1;
	    mynum = i;
	    break;
	}
    }
    Assert(i < NUMSNEAKYBUFS);
}

SneakyCopyMemoryObject::~SneakyCopyMemoryObject()
{
    inuse[mynum] = 0;
}

extern "C" {
   void PhysCopyBlock(unsigned char *, unsigned char *, int);
};

int
SneakyCopyMemoryObject::read(unsigned int startUnit, int count,
					PhysicalMemoryChain * chain)
{
	unsigned char * psrc =  &buf[ mynum ][ startUnit * SneakyPageSize ];

	PhysicalMemoryChainContiguousBlockIterator iterator( chain, count );
	unsigned int length = 0;
	const char * nextBlock = 0;

	unsigned int total = 0;

	while( ( nextBlock = iterator.next( length ) ) != 0 ) {
		Debug.printf( "SneakyCMO::read: CopyBlock( src:%A dest:%A count:%d )\n",
			psrc, nextBlock, length );

		total += length;
		Assert((startUnit + total) * SneakyPageSize <= SNEAKYBUFSIZE);

		PhysCopyBlock( psrc, (unsigned char *) nextBlock,
						length * SneakyPageSize);
		psrc += length * SneakyPageSize;
	}
	Assert( total == count );
	return( total );
}

int
SneakyCopyMemoryObject::write(unsigned int startUnit, int count,
					PhysicalMemoryChain * chain)
{
	Assert((startUnit * SneakyPageSize) < SNEAKYBUFSIZE);
	unsigned char * pdest =  &buf[ mynum ][ startUnit * SneakyPageSize ];

	PhysicalMemoryChainContiguousBlockIterator iterator( chain, count );
	unsigned int length = 0;
	const char * nextBlock = 0;

	unsigned int total = 0;

	while( ( nextBlock = iterator.next( length ) ) != 0 ) {
		Debug.printf( "SneakyCMO::write: CopyBlock( src:%A dest:%A count:%d )\n",
			nextBlock, pdest, length );
		total += length;
		Assert((startUnit + total) * SneakyPageSize <= SNEAKYBUFSIZE);

		PhysCopyBlock( (unsigned char *) nextBlock, pdest,
						length * SneakyPageSize);
		pdest += length * SneakyPageSize;
	}
	Assert( total == count );
	return( total );
}


MemoryObject *
PCi586Kernel::getMemoryObjectOfInitialApplicationProcess()
{
    Debug << "PCi586Kernel::getMemoryObjectOfInitialApplicationProcess()\n"
	    << eor;
    MemoryObject * m = new SneakyMemoryObject (SizeDummyCoffFile);

    int status;
    StoredObject * so;
    for(;;) {
	char applicationName[ 512 ];

	Console().printf("Enter Name of Initial Application Process: ");
	
	extern int ConsoleRead( char *, int );
	(void) ConsoleRead( applicationName, 512 );

	so = _fileSystemInterface->open( applicationName,
					StoredObjectClass,
					status );
	if ( status == 0 )
	    break;                   // got it!
    }
  
    MemoryObject * m = (MemoryObject *) so->asA( MemoryObjectClass, status );
    Assert( status == 0 );
    Assert( m != 0 );

    return( m );
}
#endif

PersistentMemoryObjectRef
PCi586Kernel::allocateMemoryObjectForBackingStore()
{
    Debug << "PCi586Kernel::allocateMemoryObjectForBackingStore()\n"
	<< eor;
#ifdef USEPCI586BACKINGSTORE
#ifdef SNEAKYMO
    PersistentMemoryObject * mo = new SneakyCopyMemoryObject (SNEAKYBUFSIZE);
#endif

    const int NumberOfPartitions = theDisk->myVTOC.v_nparts;
    int i;


    // partition information part of volume table of contents
    PCi586DiskPartitionHeader * partitionHdr = theDisk->myVTOC.v_part;

    for(i = 0; i < NumberOfPartitions; i++) {
	if ( partitionHdr->p_tag == V_SWAP )
	    break;                     // found it
	else
	    partitionHdr++;            // try next partition
    }
    Assert( i < NumberOfPartitions );

    PersistentMemoryObject * mo = 
	new PCi586DiskBackingStore( theDisk,
				    (int) partitionHdr->p_start,
				    (int) partitionHdr->p_size );

    Assert( mo != 0 );

    return( mo );
#else
    /*
     * just use parent
     */
    return ( Kernel::allocateMemoryObjectForBackingStore() );
#endif
}

int PCi586Kernel::isObjectFile( PersistentMemoryObject * objectFile )
{
  return 0;
}

const char * PCi586Kernel::loadObjectFile(
				PersistentMemoryObject * objectFile,
				Domain * newProcessesDomain )
{
  return 0;
}	

#include "ClassInstaller.h"
DEFINECLASS("PCi586Kernel",PCi586Kernel,Kernel,0,0);
