/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 * Copyright (c) 1990, IBM corporation. 
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Panos Kougiouris (kougiour@cs.uiuc.edu, kougiour at kgnvmc)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * 80586DebugSupport.s: assembly language debug routines.
 *
 */


	.text
/* The function gdbint moves the register values from the stack to the
 * structure registers and then calls the command processing function
 * passingthe interrupt number as argument.
 * INPUT:  In the stack:
 *   old esp-->   [old ss	    (long)]             []  only in level
 *		  [old esp          (long)]		    transition
 *          	  old eflags        (long)
 *		  old cs            (long)
 *                old eip           (long)
 *		  errror code	    (long)
 *   new esp-->   interrupt number  (long)
 * SIDE EFFECTS:
 *	   The registers data structure are loaded with the values 
 *	   the actual registers had just before the interrupt.
 * OUTPUT: None
 * BUGS:   The function assumes no priv. level transistion. This
 *	   should be changed later.
 * INFO:   The variable privTransition is set to  1  if we had level transition
 *					      to  0  otherwise.
 */
	.globl	gdbint__Fv
gdbint__Fv:
	cli				/* disable interrupts */
	pushl   %ds			/* push the old ds */
	pushl	%eax			/* push the old eax */
	movw	$8, %ax			/* load ds with the priv data segment */
	movw	%ax, %ds

	/* Save some of the regs  to be able to use them */
	popl	%eax
	movl 	%eax, registers		/* Move eax to the registers */
	movl	%ecx, registers+4

	/* save the debug registers now */
        movl    %db0, %eax
        movl    %eax, dreg
        movl    %db1, %eax
        movl    %eax, dreg+4
        movl    %db2, %eax
        movl    %eax, dreg+8
        movl    %db3, %eax
        movl    %eax, dreg+12
        movl    %db6, %eax
        movl    %eax, dreg+24
        movl    %db7, %eax
        movl    %eax, dreg+28

	/* store ds */
	popl	%eax			/* now the old ds in eax */
	andl	$0xFFFF, %eax
	movl	%eax, registers+48	/* store the old ds */

	/* Now check if we had priv. level transition. */
	movl	$0, privTransition		/* Default no level Trans. */
	movw	%cs, %ax			/* Get the new CS */
	andl	$3, %eax			/* Get the new cs RPL */
	movl	12(%esp), %ecx			/* Get the old cs */
	andl	$3, %ecx			/* Get the old cs RPL */
	subl	%ecx, %eax			/* Compare them */
	jz	continue1			/* Jmp on no transition */
	movl	$1, privTransition
continue1:
	/* Save the rest of the regs  here */
	movl	%edx, registers+8
	movl    %ebx, registers+12     

	/* The %esp and ss will be found differently depending on 
	 * if we had level transition  or not. */
	movl 	privTransition, %eax
	subl	$0, %eax
	jz	noTransitionESP
	movl	20(%esp), %eax		/* save esp from the stack */
	movl 	%eax, registers+16
	movl	24(%esp), %eax		/* save ss from the stack */
	movl	%eax, registers+44
	jmp 	continue2
noTransitionESP:
	movl    %esp, %eax		/* Find the value of the old */
	addl	$20, %eax		/* %esp by adding 20 */
	movl 	%eax, registers+16
	xor	%eax, %eax		/* just move the ss */
	movw	%ss, %ax
	movl	%eax,registers+44
continue2:
	movl	%ebp, registers+20
	movl	%esi, registers+24
	movl	%edi, registers+28
	movl	8(%esp), %eax		/* old eip */
	movl	%eax, registers+32		
	movl	16(%esp), %eax		/* old flags */
	movl	%eax, registers+36	 
	movl	12(%esp), %eax		/* old cs */
	andl	$0x0000ffff, %eax
	movl	%eax,registers+40 
	xor	%eax, %eax
	movw	%es, %ax
	movl	%eax, registers+52
	movw	%fs, %ax
	movl	%eax,registers+56
	movw	%gs, %ax
	movl	%eax, registers+60
	popl 	%eax			/* get the interrupt number */
	movl	debugStackPointer, %esp /* set new stack */
	pushl	%eax			/* push the interrupt number */
	movl	%eax, interruptNumber   /* save it to be accessed from debug */
	call	processCommands__Fi	/* this should never return */
	hlt				/* or a trap here? */

/* The following routine is called when we return from an
 * interrupt.
 *
 * INPUT:  In the stack:
 *   old esp-->
 *   		  interrupt number  (long)
 *		
 *
 * SIDE EFFECTS:
 *	   The registers  are loaded from the registers 
 *	   data structure.
 * OUTPUT: None
 * BUGS:   The function assumes no priv. level transistion. This
 *	   should be changed later. It does not set the cs
 */

	.globl	debugReturnFromInterrupt__Fv
debugReturnFromInterrupt__Fv:
        /* Restore the debug registers */
        movl    dreg, %eax
        movl    %eax, %db0
        movl    dreg+4, %eax
        movl     %eax, %db1
        movl    dreg+8, %eax
        movl     %eax, %db2
        movl    dreg+12, %eax
        movl     %eax, %db3
        movl    $0x0, %eax
        movl    %eax, %db6
        movl    dreg+28, %eax
        movl     %eax, %db7
	/* restore the registers */
	movl 	registers, %eax
	movl	registers+4, %ecx
	movl	registers+8, %edx
	movl    registers+12, %ebx     
	movl	registers+20, %ebp
	movl	registers+24, %esi
	movl	registers+28, %edi
	movl	privTransition, %eax	/* restore ss only if no tranisition */
	subl	$0, %eax
	jne	TransitionRestore		
	movl	registers+44, %eax		/* If (not transition) then */
	movw	%ax, %ss
	movl 	registers+16, %esp
	jmp	continue3
TransitionRestore:				/* else */
	pushl	registers+44			/* push the old ss */
	pushl	registers+16			/* push the old stack pointer */
continue3:					/* endif */
	movl	registers+52, %eax
	movw	%ax, %es
	movl	registers+56, %eax
	movw	%ax, %fs
	movl	registers+60, %eax
	movw	%ax, %gs

	movl 	registers, %eax		/* because we changed its value */
	pushl	registers+36		/* the flags */
	pushl	registers+40		/* the cs */
	pushl	registers+32		/* push the eip */	
	pushl	registers+48		/* push the old ds temporarily */
	popl	%ds			/* restore the old ds */
	iret


	.data
	.globl privTransition
privTransition: 
		.long 0

	.globl interruptNumber
interruptNumber:
		.long 0
