/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Bjorn A. Helgaas (helgaas@cs.uiuc.edu)
 * Contributing Author: See-Mong Tan (stan@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * This is the initial setup code.
 * Main() is called by ALL processors while still in physical memory
 * mode. One processor (the first one to call it Main()) sets up the
 * Kernel via its constructor.
 * All processors join the kernel by invoking the perProcessorEntry() method
 * on it.
 */

#include "Assert.h"
#include "PCi586Console.h"
#include "PCi586Multiboot.h"
#include "OutputStream.h"
#include "CPU.h"

unsigned long multiboot_magic;
multiboot_info_t multiboot_info;
unsigned num_modules;
module_t modules[MAX_MULTIBOOT_ENTRIES];

unsigned num_memory_maps;
memory_map_t memory_map[MAX_MULTIBOOT_ENTRIES];

unsigned num_drives;
drive_info_t drive_info[MAX_MULTIBOOT_ENTRIES];

char * nameOfInitialApplication;
int useDebugger = 0;

extern "C" {
void atexit(){}
}

/* provides functionality of cstdlib memcpy */
void
memcpy(void *dest, void *src, unsigned long count) {
  unsigned long i;
  for(i = 0; i < count; i++) {
    ((unsigned char *)dest)[i] = ((unsigned char *)src)[i];
  }
};

/*
 * This should only be called by one processor (the first to boot in a
 * multiprocessor).
 */
void
PCi586Main( int, char ** )
{
	extern OutputStream * ConsoleOutputStream;

	ConsoleOutputStream = new PCi586Console();

	Assert( ConsoleOutputStream != 0 );
	Console() << "Console built\n" << eor;

}


unsigned int *
PCi586HeapStart()
{
	/*
	 * The HeapStart is the starting address at which the kernel can
	 * allocate memory.  Start the heap after the 1MB or the end of the
    * kernel.
	 *
	 * 0x100000 == 1Mb, after display memory
	 */

	extern int end;
	if( (int *) 0x100000 < &end ) {
		return( (unsigned int *) &end );
	} else {
		return( (unsigned int *) 0x100000 );
	}
}

extern void Main(int argc, char **argv);

extern "C" {
void KernelStart(void) {
  /* If necessary, command line arguments could be parsed out of the multiboot_info
     structure, but because we don't currently support command line args simply pass 0's */
  Main(0, 0);
  CPU::processorStart();
  Assert ( NOTREACHED );
}

/* Copy multiboot header info passed in by the bootloader */
void CopyMultibootInfo(unsigned long mb_magic, unsigned long mb_addr) {
  /* Copy the multiboot info passed in by the bootloader into
     statically allocated memory (so that it isn't lost if we
     accidentally overwrite it) */

  multiboot_magic = mb_magic;

  memcpy((void *)&multiboot_info, (void *)mb_addr, sizeof(multiboot_info_t));

  /* Is mods_* flag valid? */
  if(MULTIBOOT_CHECK_FLAG (multiboot_info.flags, 3)) {
    memcpy((void *)&modules[0], (void *)multiboot_info.mods_addr,
           sizeof(module_t) * multiboot_info.mods_count);
    num_modules = multiboot_info.mods_count;
  }

  /* Is mmap_* flag valid? */
  if(MULTIBOOT_CHECK_FLAG (multiboot_info.flags, 6)) {
    int i;
    memory_map_t *curr_map;
    for(i = 0, curr_map = (memory_map_t *)multiboot_info.mmap_addr;
        (unsigned long)curr_map < (multiboot_info.mmap_addr + multiboot_info.mmap_length);
        i++, curr_map = (memory_map_t *) ((unsigned long)curr_map
                                          + curr_map->size + sizeof(curr_map->size))) {
      memcpy((void *)&memory_map[i], (void *)curr_map, sizeof(memory_map_t));
      num_memory_maps++;
    }
  }

  /* Is drives_* flag valid? */
  if(MULTIBOOT_CHECK_FLAG (multiboot_info.flags, 7)) {
    int i;
    drive_info_t *curr_drive;
    for(i = 0, curr_drive = (drive_info_t *)multiboot_info.drives_addr;
        (unsigned long)curr_drive < (multiboot_info.drives_addr + multiboot_info.drives_length);
        i++, curr_drive = (drive_info_t *) ((unsigned long)curr_drive + curr_drive->size)) {
      memcpy((void *)&drive_info[i],(void *)curr_drive, sizeof(drive_info_t));
      num_drives++;
    }
  }
}
}
