/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * PCi586TimerManager.c - machine dependent portion of timer managing
 */

#include "OutputStream.h"
#include "Assert.h"
#include "TIMER_MANAGER.h"
#include "CPU.h"
#include "Lock.h"
#include "FreeRunningTimerStar.h"
#include "PCi586FreeRunningTimer.h"
#include "PCi586InterruptController.h"
#include "PCi586IntervalTimer.h"
#include "Util.h"

extern TimerManager *TimeoutManager;
extern TimerManager *PeriodicManager;

void
TimerManagerInterrupt::basicRaise( char * )
{
    /*
    Debug.printf( "TimerManagerInterrupt%T::basicRaise()\n", this );
    */
    Assert( this != 0 );
    //Assert( thisCPU()->interruptsDisabled() );

    ((PCi586FreeRunningTimer *)(FreeRunningTimer *)SystemTimer)->tick();

    /*
     * We get some timer interrupts before the Managers are completely
     *   initialized so need to make sure the pointers are non-null
     *   before calling the handlers
     */
    if (TimeoutManager != 0)
	TimeoutManager->interruptHandler();

    if (PeriodicManager != 0)
	PeriodicManager->interruptHandler();

    Out(PIC_CTRL_REG, PIC_EOI);

    PCi586TimesliceTimer::tick();
}

static Lock initTimerLock;	/* these two could be within the next routine */
static int intervalTimerStarted = 0;  /* but it gets "sorry, not implemented" */

PCi586TimerManager::PCi586TimerManager(PFI ptb, PFV ep)
	: TimerManager(ptb, ep)
{
    Debug.printf("PCi586TimerManager::ctor()\n");

    /* QUESTION: should this be rounded up, or always be evenly divisible?*/
    unsigned int microTicks = 1000000/ClockFrequency;
    Assert((microTicks * ClockFrequency) == 1000000);
    Assert(microTicks > 0);

    /*
     * The microsecondsPerTick could conceivably be different for
     *   timeout timers and periodic timers, but they're the same here;
     *   in fact, there's no way to tell the difference at this point.
     */
    microsecondsPerTick = microTicks;

    /*
     * Initialize the hardware clock when the first PCi586TimerManager 
     *   is constructed.
     */
    initTimerLock.acquire();

    if (!intervalTimerStarted)
    {
	intervalTimerStarted = 1;

	Out(PITCTL_PORT, PIT_SEL0|PIT_BOTH|PIT_MODE2);
	Assert(PITCount == (unsigned short) PITCount);

	Out(PITCTR0_PORT, PITCount & 0xff);
	Out(PITCTR0_PORT, PITCount >> 8);
	Out(PIC_MASK_REG, In(PIC_MASK_REG) & ~PIC_TIMER_MASK);
    }

    initTimerLock.release();

    setFreeRunningTime(SystemTimer->time());
}
