/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Gary Johnston (johnston@cs.uiuc.edu)
 * Contributing Author: Dave Dykstra (dykstra@cs.uiuc.edu)
 * Contributing Author: Doug Leyens (leyens@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * PCi586TimesliceTimer.cc
 */

#include "Assert.h"
#include "OutputStream.h"
#include "PCi586TimesliceTimer.h"
#include "PCi586FreeRunningTimer.h"  /* for definition of ClockFrequency */
#include "Lock.h"

static const int microsecondsPerTick = 1000000/ClockFrequency;

TimeSliceInterrupt TSliceInterrupt;

Lock *PCi586TimesliceTimer::lock;
unsigned int PCi586TimesliceTimer::tickTime;

unsigned int PCi586TimesliceTimer::expireTime;
char PCi586TimesliceTimer::running;

PCi586TimesliceTimer::PCi586TimesliceTimer( )
{
	Debug << "PCi586TimesliceTimer::ctor()\n" << eor;

	//JMT HACK TBD
	lock = new Lock("PCi586TimesliceTimerLock");
	tickTime = 0;
	running = 0;
}

PCi586TimesliceTimer::~PCi586TimesliceTimer()
{
	delete lock;
}


unsigned int
PCi586TimesliceTimer::start( unsigned int microseconds )
{
	Debug << "PCi586TimesliceTimer::start( " << microseconds << " )\n";

	unsigned int ticks = (microseconds + (microsecondsPerTick - 1)) /
						microsecondsPerTick;

	if (ticks != 0) {
		lock->acquire();

		running = 1;
		expireTime = tickTime + ticks;

		lock->release();
	}

	return(ticks * microsecondsPerTick);
}

void
PCi586TimesliceTimer::set( unsigned int )
{
	/*
	 * This method should not have to exist; it is only here
	 *   for compatibility with the Multimax.
	 */
	Debug.printf( "PCi586TimesliceTimer::set()\n" );
}

unsigned int
PCi586TimesliceTimer::stop()
{
	Debug << "PCi586TimesliceTimer::stop()\n" << eor;

	unsigned int resid;

	lock->acquire();

	if (running) {
		running = 0;
		resid = expireTime - tickTime;
	}
	else
		resid = 0;

	lock->release();

	return(resid * microsecondsPerTick);

}

/*
 * residual - Returns the number of microseconds left before interrupt.
 */

unsigned int
PCi586TimesliceTimer::residual()
{
	Debug.printf( "PCi586TimesliceTimer::residual() entering \n" );

	unsigned int resid;

	lock->acquire();

	if (running)
		resid = expireTime - tickTime;
	else
		resid = 0;

	lock->release();

	return(resid * microsecondsPerTick);
}

/*
 * idle - returns non-zero iff the timer is not running.
 */

int
PCi586TimesliceTimer::idle()
{
	Debug.printf( "PCi586TimesliceTimer::idle()\n" );
	return ( !running );
}
