	.file	"boot.s"
	.globl	_choicesstart

#undef DEBUG

#include "PCi586Multiboot.h"
                	
/* reserve initial kernel stack space */
.set STACKSIZE, 0x4000          /* that's 16k. */
.comm stack, STACKSIZE, 32      /* reserve 16k stack on a quadword boundary */

#define data16	.byte 0x66
#define addr16	.byte 0x67
#define sstol	stosl
#define sstob	stosb

/****************************************************************************
/*  GDTSLOTS are the number of Global Descriptor Tables slots to allow.
/****************************************************************************/
GDTSLOTS =	0x40

/*************************************************************************
/*    beginning of code                                                  
/*************************************************************************/
	.text
	.global	_choicesstart

_choicesstart:
	jmp _choices

	.align 4

/* For GRUB compliance */
multibootheader:

	.long MULTIBOOT_HEADER_MAGIC
	.long MULTIBOOT_HEADER_FLAGS
	.long MULTIBOOT_HEADER_CHECKSUM

_choices:
	cli
	cld
	
        /* Initialize the stack pointer. */
        movl    $(stack + STACKSIZE), %esp

        /* Reset EFLAGS */
        pushl   $0
        popf

        pushl   %ebx
        pushl   %eax
        call    CopyMultibootInfo

        /* Setup our Global Descriptor Table (GDT) */
	lgdt	gdtptr

        /* clear extra segment registers (point them to NULL descriptor) */
	xor	%eax, %eax
	movw	%ax, %es
	movw	%ax, %fs
	movw	%ax, %gs

        /* Clear the Local Descriptor Table (LDT) (mark as invalid) */
        lldt    %ax
        
        /* the code segment (CS), data segment (DS), and stack segment (SS) */
        /* are setup for us by the multiboot bootloader.  CS = entry 1 in the
        /* GDT.  both DS and SS = entry 2 in the GDT */

        /* load Interrupt Descriptor Table (IDT) */
	lidt	IDTR+2

        /* clear base pointer (marks end of frame pointer chain) */
	mov	%eax, %ebp

        /* far jump to the tss in the gdt.  This will cause a task switch */
	ljmp     $24,$0

/*****************************************************************************
/ GDT - Global Descriptor Table.  
/  NOTE: these must be in the same order as the defines in Vectors.h
/*****************************************************************************/
#ifdef GAS
	.align	16
#else
	.align	16
#endif

	.globl	Gdt
	.globl  gdt
Gdt:
gdt:
/* 0 */
	.long	0, 0		/* Null descriptor */

CS =	. - Gdt			/* System CS */
/* 1 */
	.value	0xFFFF		/* low word of limit */
	.value	0		/* low word of base */
	.byte	0		/* mid byte of base */
	.byte	0x9B		/* access */
	.byte	0xCF		/* G, D bits and high nibble of limit */
	.byte	0x00		/* high byte of base */

DS =	. - Gdt			/* System DS */
/* 2 */
	.value	0xFFFF		/* low word of limit */
	.value	0		/* low word of base */
	.byte	0		/* mid byte of base */
	.byte	0x93		/* access */
	.byte	0xCF		/* G, B bits and high nibble of limit */
	.byte	0x00		/* high byte of base */

CHTR =  . - Gdt			/* CH TSS - Task segment descriptor we use  */
/* 3 */				/* when we start Choices (THIS IS IT!) */
	.value tsssize-1	/* low word of limit */
	.long	chtss + 0x89000000 /* low 24 bits of base plus access byte */
	.byte	0x00		/* G, D bits and high nibble of limit */
	.byte	0x00		/* high byte of base */

/* 4 */ .long	0, 0		/* PROXY_Vector */
/* 5 */ .long	0, 0		/* PROXYRETURN_Vector */
/* 6 */	.long	0, 0		/* beginning of USERSEG_Vectors */
/* 7 */	.long	0, 0
/* 8 */	.long	0, 0
/* 9 */	.long	0, 0
/* a */	.long	0, 0
/* b */	.long	0, 0
/* c */	.long	0, 0
/* d */	.long	0, 0


/*
 * NOTICE: Gdt entries 0xe through 0x2f are used by CDB
 */

#ifndef GAS
gdtend:
	.=8\*GDTSLOTS + Gdt
#endif
bgdtend:

/******************************************************************************
/
/	Table pointers
/
/******************************************************************************/

gdtptr:					/* Physical limit and address of GDT */
	.value	bgdtend - Gdt -1 
	.long	Gdt


/****************************************************************************
/  data segment
/****************************************************************************/
	.data

/****************************************************************************
/   Choices Task State Segment 
/****************************************************************************/

	.globl	chtss
chtss: 				/* TSS for initial Choices task */
	.long	0		/* Back link */
	.long	stack + STACKSIZE  /* esp0 */
	.value	DS, 0		/* ss0 */
	.long	0		/* esp1 */
	.value	0, 0		/* ss1 */
	.long	0		/* esp2 */
	.value	0, 0		/* ss2 */
	.long	0		/* pdbr */
	.long	KernelStart     /* eip - FIXME: pointer to kernel task? */
	.long	0x1000		/* flags */
	.long	0		/* eax */
	.long	0		/* ecx */
	.long	0		/* edx */
	.long	0		/* ebx */
	.long	stack + STACKSIZE  /* esp */
	.long	0		/* ebp */
	.long	0		/* esi */
	.long	0		/* edi */
	.value	DS, 0		/* es */
	.value	CS, 0		/* cs */
	.value	DS, 0		/* ss */
	.value	DS, 0		/* ds */
	.value	DS, 0		/* fs */
	.value	DS, 0		/* gs */
	.value	0, 0		/* ldt */
	.value	0		/* t bit */
	.value	tsssize		/* i/o map */

tsssize = . - chtss

	.globl __fltused  /* defined to show that floating point is ok */
__fltused:

/******************************************************************************/
/ bss 
/******************************************************************************/
#ifndef GAS
	.bss
#endif
	.align	4
