#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ConsoleProcess.c: the process that awaits console characters
 */

#include "Assert.h"
#include "AwaitedInterruptInterrupt.h"
#include "CharacterInputStream.h"
#include "CPU.h"
#include "ThisCPU.h"
#include "SS1Console.h"		// For ROM stuff
#include "Process.h"
#include "ThisProcess.h"
#include "Vectors.h"


#ifdef __GNUC__
#pragma implementation
#endif


static void
ConsoleInputProcessEntry()
{
	Process * process = thisProcess();
	char ch;

	extern int consoleInputInitialized;
	consoleInputInitialized = 1;

	Debug << process << "ConsoleInputProcessEntry()\n" << eor;
	while (1) {
		ch = ((SS1Console*)ConsoleOutputStream)->getChar();
		Assert ( ConsoleInputStream != 0 );
		ConsoleInputStream->enqueue( &ch, 1 );
		}
	Assert (NOTREACHED);
}

void
UartInterruptProcess()
{
	AwaitedInterruptInterrupt * interrupt =
		new AwaitedInterruptInterrupt();

        thisCPU()->setInterrupt(VEC_UART, interrupt);

	while (1) {
		Console() << thisProcess() << ": awaiting interrupt\n" << eor;
		interrupt->await();
		Console() << thisProcess() << ": interrupt\n" << eor;
		/*
		 * reset interrupt for now
		 */
		}
	Assert (NOTREACHED);
}


void
ConsoleManagerProcessEntry( int arg )
{
	Debug << "ConsoleManagerProcessEntry( arg: " << arg << ")\n" << eor;

	/*
	 * start the interrupt handler
	 */
	Process * process = new InterruptProcess( 
		(PFV) UartInterruptProcess, thisProcess()->domain(),
		small, 0 );
	Assert( process != 0 );
	process->setName( "#UartInterruptProcess" );
	Debug <<  "ConsoleManagerProcess: UartInterruptProcess at " 
		<< asAddress(process) << "\n" << eor;
	process->ready();
	thisProcess()->relinquishProcessor();

	/*
	 * start the input process.
	 */
	process = new SystemProcess( (PFV) ConsoleInputProcessEntry,
		thisProcess()->domain(), small, 0);
	Assert( process != 0 );
	process->setName( "#ConsoleInputProcess" );
	Debug << "ConsoleManagerProcess: Input process at " 
		<< asAddress(process) << "\n" << eor;

	process->ready();
	thisProcess()->relinquishProcessor();

#ifndef SPARC
	/*
	 * Put the interrupt process in its ready container and then
	 * relinquish the processor and hope the interrupt process runs.
	 */
	process->ready();
	thisProcess()->relinquishProcessor();

	/*
	 * Next, start the output interrupt process.
	 */
	process = new InterruptProcess( 
		(PFV) ConsoleOutputInterruptProcessEntry,
		thisProcess()->domain(),
		normal, CONSOLEOUT_Vector );
	Assert( process != 0 );
	process->setName( "#ConsoleOutputInterrrupt" );
	Debug << ( "ConsoleManagerProcess: Output process at " 
		<< asAddress(process) << "\n" << eor;

	process->ready();
	thisProcess()->relinquishProcessor();

	/*
	 * Call the console initilization code.
	 */
	Debug << "ConsoleManagerProcess: interrupt processes started\n" << eor;

	extern InitConsole();
	InitConsole();
	Debug <<  "ConsoleProcessEntry: console initialized\n" << eor;

	/*
	 * Just fall off the end to terminate the process.
	 */

#endif /* SPARC */
}
