#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include <ctype.h>
#include "Gdb.h"
#include "OutputStream.h"
#include "String.h"


#ifdef __GNUC__
#pragma implementation
#endif

enum regnames {
        G0,G1,G2,G3,G4,G5,G6,G7,
        O0,O1,O2,O3,O4,O5,O6,O7,
        L0,L1,L2,L3,L4,L5,L6,L7,
        I0,I1,I2,I3,I4,I5,I6,I7,
        F0,F1,F2,F3,F4,F5,F6,F7,
        F8,F9,F10,F11,F12,F13,F14,F15,
        F16,F17,F18,F19,F20,F21,F22,F23,
        F24,F25,F26,F27,F28,F29,F30,F31,
        Y,PSR,WIM,TBR,PC,NPC,FPSR,CPSR
        };

int GdbRemoteDebugging = 1;

/*
 * scanhex and scan2hex are replacements for sscanf'ing hex values
 * so libc.a is not needed.
 */
int
scanhex(char *p, unsigned int *var)
{
	char upchar;
	int result = 0;
        int value = 0;
        while (isxdigit(*p)) {
		result++;
                if (isdigit(*p)) {
                        value = 16 * value + (*p - '0');
		}
                else {
			if (islower(*p))
				upchar = (*p - 'a') + 'A';
			else
				upchar = *p;

                        value = 16 * value + (upchar - 'A' + 10);
                }
        p++;
        }
	*var = value;
	return(result);
}

int
scan2hex(char *p, unsigned int *var1, unsigned int *var2)
{
	int result;

	if ((result = scanhex(p, var1)) == 0)
		return(0);
	if (*(p+result++) != ',')
		return(0);
	if (scanhex((p+result), var2) == 0)
		return(0);
	return(2);
}

/*
 * The structure that the trap routine fills in with register values
 * at trap time and restores (possibly changed) register values from
 * after the trap.
 */
extern unsigned *registers ;

char hexchars[]="0123456789abcdef";

char 
hex(char ch)
{
  if ((ch >= 'a') && (ch <= 'f')) return (ch-'a'+10);
  if ((ch >= '0') && (ch <= '9')) return (ch-'0');
  return (0);
}

void GdbGetPacket(char *buffer)
{
	unsigned char checksum;
	unsigned char xmitcsum;
	char *current;
	int  i;
	int  count;
	char ch;

	do{
		/* Wait for the start char */
		while ( (GetDebugChar()) != '$' ) ;
		checksum = 0;
		count = 0;
		current = buffer;

		while ( (ch = GetDebugChar()) != '#') {
			*current++ = ch;
			checksum += ch;
			count++;
			}
		*current = '\0';

        xmitcsum = hex(GetDebugChar()) << 4;
        xmitcsum |= hex(GetDebugChar());
	// checksum &= 0xff;	// Mask to 8 bits.
	// xmitcsum &= 0xff;
	if (xmitcsum != checksum) {
		PutDebugChar('-');  /* failed checksum */
		Console().printf("GdbGetPacket: checksum error! 0x%x != 0x%x\n",
			checksum, xmitcsum);
		Console() << "Packet: " << buffer << "\n" << eor;

		Debug << asHex(xmitcsum) << " != " << asHex(checksum) << "\n" << eor;
		Debug << "Packet: " << buffer << "\n" << eor;
		}	
	else {
		PutDebugChar('+');  /* successful transfer */
		/* if a sequence char is present, reply the sequence ID */
		if (buffer[2] == ':') {
			    PutDebugChar( buffer[0] );
			    PutDebugChar( buffer[1] );
			    /* remove sequence chars from buffer */
			    count = strlen(buffer);
			    for (i=3; i <= count; i++) buffer[i-3] = buffer[i];
		     }
		}
	} while (checksum != xmitcsum);

        Debug << "GdbGetPacket: got packet: " << buffer << "\n" <<eor;

}


/* send the packet in buffer.  The host get's one chance to read it.  
   This routine does not wait for a positive acknowledge.  */
void 
GdbPutPacket(char *buffer)
{
  unsigned char checksum;
  int  count;
  char ch;
  
  /*  $<packet info>#<checksum>. */
  do {
  PutDebugChar('$');
  checksum = 0;
  count    = 0;
  
  Debug << "GdbPutPacket(" << buffer << ")\n" << eor;

  while (ch=buffer[count]) {
    PutDebugChar(ch);
    checksum += ch;
    count += 1;
  }

  PutDebugChar('#');
  PutDebugChar(hexchars[checksum >> 4]);
  PutDebugChar(hexchars[checksum % 16]);

  } while (1 == 0);  /* (GetDebugChar() != '+'); */
  
}

static char  inbuffer[BUFMAX];
static char  outbuffer[BUFMAX];
static short error;

/* convert the memory pointed to by mem into hex, placing result in buf */
/* return a pointer to the last char put in buf (null) */
char* mem2hex(char *mem, char *buf, int count)
{
      int i;
      unsigned char ch;
      for (i=0;i<count;i++) {
          ch = *mem++;
          *buf++ = hexchars[ch >> 4];
          *buf++ = hexchars[ch % 16];
      }
      *buf = 0; 
      return(buf);
}

/* convert the hex array pointed to by buf into binary to be placed in mem */
/* return a pointer to the character AFTER the last byte written */
char* hex2mem(char *buf, char *mem, int count)
{
      int i;
      unsigned char ch;
      for (i=0;i<count;i++) {
          ch = hex(*buf++) << 4;
          ch = ch + hex(*buf++);
          *mem++ = ch;
      }
      return(mem);
}


/*
 * This function does all command procesing for interfacing to gdb.
 */
void GdbHandler(int interruptVector, char *stackptr)
{
  int    sigval;
  unsigned int    addr, length;
  char * ptr;
  static int stepping = 0;
  static unsigned int stepped_insn;
  static unsigned *steppedInsnPointer;
  int done=0;

 Debug << "handle_interrupt(" << interruptVector << ")\n" << eor;

 /*
  * This can be used to set things in the VM system so we know that
  * we are running on behalf of a user process.  So that our memory
  * faults are translated into user faults.
  */
 GdbMachineHandlerStart(interruptVector, stackptr);

	if (stepping) { 
		/*
		 * We got here after a "s" message.
		 * Replace the stepped insn and return
		 */
		*steppedInsnPointer = stepped_insn;
		stepping = 0;
		done = 1;
		}

  /* reply to host that an interrupt has occurred */
  sigval = ComputeSignal( interruptVector );
		outbuffer[0] = 'S';
		outbuffer[1] = hexchars[sigval/16];
		outbuffer[2] = hexchars[sigval];
		outbuffer[3] = '\0';
  GdbPutPacket(outbuffer); 

  while (!done) { 
    error = 0;
    outbuffer[0] = 0;
    GdbGetPacket(inbuffer);
    Debug << "handle_interrupt: got packet: " << inbuffer << "\n" << eor;
    switch (inbuffer[0]) {
      case '?' : 
		outbuffer[0] = 'S';
		outbuffer[1] = hexchars[sigval/16];
		outbuffer[2] = hexchars[sigval];
		outbuffer[3] = '\0';
                 break; 
      case 'd' : GdbRemoteDebugging = !(GdbRemoteDebugging);  
			/* toggle debug flag */
                 break; 
      case 'g' : /* return the value of the CPU registers */
                mem2hex((char*) registers, outbuffer, NUMREGBYTES);
                break;
      case 'G' : /* set the value of the CPU registers - return OK */
                hex2mem(&inbuffer[1], (char*) registers, NUMREGBYTES);
                strcpy(outbuffer,"OK");
                break;
      
      /* mAA..AA,LLLL  Read LLLL bytes at address AA..AA */
      case 'm' : 
		if (2 == scan2hex(&inbuffer[1],&addr,&length) &&
			GdbMachineMem_ok(GDB_READ, (char *)addr, length) == 0) {
			mem2hex((char*) addr, outbuffer, length);
			}
		else {
			strcpy(outbuffer,"E01");
			Debug << "malformed read memory command: " << 
							inbuffer << eor;
			}     
		break;

/* MAA..AA,LLLL: Write LLLL bytes at address AA.AA return OK */
	case 'M' : 
		if (2 == scan2hex(&inbuffer[1],&addr,&length) &&	
			GdbMachineMem_ok(GDB_WRITE, (char*)addr, length) == 0) {
			ptr = index(inbuffer,':');
			ptr += 1; /* point 1 past the colon */
			hex2mem(ptr, (char*) addr, length);
			strcpy(outbuffer,"OK");
			}
		else {
			strcpy(outbuffer,"E02");
			Debug << "malformed write memory command: " << inbuffer
								<< eor;
			}     
		break;

/* cAA..AA    Continue at address AA..AA(optional) */
case 'c' : 
	if (0 != scanhex(&inbuffer[1],&registers[ PC ]))
		registers[NPC] = registers[PC] + 4;
	done = 1;
	break;

/* sAA..AA   Step one instruction from AA..AA(optional) */
case 's' : 
	if (0 != scanhex(&inbuffer[1],&registers[ PC ]))
		;

	static unsigned int break_insn = 0x91d02001;
	stepped_insn = *((unsigned int*)registers[NPC]);
	stepping = 1;
	steppedInsnPointer = ((unsigned int *)registers[NPC]);
	//*((unsigned int*)registers[NPC]) = break_insn;
	*steppedInsnPointer = break_insn;
	done = 1;
	break;

/* kill the program */
      case 'k' :  /* do nothing */
		Console().printf("GdbGetPacket: K packet\n");
		GdbKill();
                break;

	default:	
		Debug << "GdbHandler: Unsupported request: " <<  
						inbuffer << eor;
      } /* switch */ 
    
    /* reply to the request */
    if (!done) GdbPutPacket(outbuffer); 
    }

	GdbMachineHandlerEnd(interruptVector, stackptr);
}
