#ifdef __GNUC__
#pragma implementation
#endif

/*
 * Routines to get things from the machine.
 * Some of these routines a PROPRIETARY Sun source code and are not to be distributed
 */

#include "Assert.h"
#include "OutputStream.h"
#include "PROM.h"
#include "String.h"
#include "SS1devices.h"
//#include "SPARCTranslation.h"

extern "C" {
#include <sys/types.h>
#include <sys/pte.h>
//#undef EEPROM_ADDR
#include <sys/eeprom.h>
#include <sys/cpu.h>
#include <sys/comvec.h>
//#undef LOCORE
#include <sys/mon/idprom.h>
};
struct idprom *idromp = (struct idprom *) (EEPROM_ADDR + EEPROM_SIZE);


#ifdef __GNUC__
#pragma implementation
#endif

#define v_exit_to_mon obp.op_exit
#define v_config_ops obp.op_config_ops
#define op_alloc obp.op2_alloc
#define v_romvec_version obp.op_romvec_version
#define v_physmemory obp.v_physmemory
#define v_virtmemory obp.v_virtmemory
#define v_availmemory obp.v_availmemory
#define v_mac_address obp.v_mac_address

unsigned _pagesize;

inline 
void 
bzero(char *source, int length)
{
	while (length--)
		*source++ = 0;
}



/*
 * This routine returns the length of the property value for the named
 * property.
 */
int
getproplen(int id, char *name)
        /* int id;
        char *name;*/
{
        return ((*romp->v_config_ops->devr_getproplen)(id, name));
}


int
searchpromtree(int node, char *name, char *buf)
	/* int node;
	char *name;
	char *buf;*/
{
	int id;
	*buf = '\0';
	(void)(*romp->v_config_ops->devr_getprop)(node, "name", buf);
	if (strcmp(buf, name) == 0) {
		return (node);
	}
	id = (*romp->v_config_ops->devr_next)(node);
	if (id && (id = searchpromtree(id, name, buf)))
		return (id);
	id = (*romp->v_config_ops->devr_child)(node);
	if (id && (id = searchpromtree(id, name, buf)))
		return (id);
	return (0);
}

/*
 * This routine allows drivers to easily retrieve the value of a property
 * for int-sized values. The value of 'name' in the property list for 'devi'
 * is returned. If no value is found, 'def' is returned.
 *
 * As a special case, if a zero length property is found, 1 is returned.
 */
int
getprop(int id, char *name, int def)
	/*int id;
	char *name;
	int def;*/
{
	int value;
	switch (getproplen(id, name)) {
	case 0:
		return (1);
	case sizeof (int):
		(void)(*romp->v_config_ops->devr_getprop)(id, name, &value);
		// (void) prom_getprop((dnode_t)id, (caddr_t)name, (caddr_t)&value);
		return (value);
	case -1:
	default:
		return (def);
	}
	/*NOTREACHED*/
}

/*
 * This structure represents one piece of bus space occupied by a given
  * device. It is used in an array for devices with multiple address windows.
  */
typedef  char *   addr_t;

  struct dev_reg {
	  int reg_bustype;     /* cookie for bus type it's on */
		  addr_t reg_addr;     /* address of reg relative to bus */
			  u_int reg_size;         /* size of this register set */
			  };

struct memlist *
getmemlist(char *name, char *prop)
	/* char *name, *prop;*/
{
	int nodeid;
	int i, j, k, chunks;
	u_int propsize;
	struct dev_reg *rp;
	struct memlist *pmem;
	static caddr_t bufp = (caddr_t)0;
	static u_int left;

	if (bufp == (caddr_t)0) {
		bufp = (caddr_t)(*romp->op_alloc)((caddr_t)0, PAGESIZE);
		left = PAGESIZE;
	}

	/*
	 * First find the right node.
	 */
	nodeid =
	    searchpromtree((*romp->v_config_ops->devr_next)(0), name, bufp);
	if (nodeid == 0)
		return ((struct memlist *)0);
	//dprintf("Found %s:%s at node %x\n", name, prop, nodeid);
	Debug << "Found " << name << ":" << prop << " at node " <<
		asHex(nodeid) << "\n" << eor;

	/*
	 * Allocate memlist elements, one per dev_reg struct.
	 */
	propsize = (u_int)getproplen(nodeid, prop);
	chunks = propsize / sizeof (struct dev_reg);
	if (chunks == 0)
		return ((struct memlist *)0);

	Assert ( left >= (chunks * sizeof (struct memlist)) );
	pmem = (struct memlist *)bufp;
	bzero((caddr_t)pmem, (u_int) chunks * sizeof (struct memlist));
	left -= (u_int) chunks * sizeof (struct memlist);
	bufp += (u_int) chunks * sizeof (struct memlist);

	Assert (left >= propsize);
	rp = (struct dev_reg *)bufp;
	bzero((caddr_t)rp, propsize);
	left -= propsize;
	bufp += propsize;

	/*
	 * Fill in memlist chunks.
	 */
	(void)(*romp->v_config_ops->devr_getprop)(nodeid, prop, rp);
	for (i = 0; i < chunks; ++i) {
		Debug << "Chunk " << i << " addr " <<
			asAddress(rp[i].reg_addr) << " bustype " <<
			asHex(rp[i].reg_bustype) << " size " 
			<< asHex(rp[i].reg_size) << "\n" << eor;
		if (rp[i].reg_bustype != OBMEM)
			continue;

		/*
		 * Insert the new element into the array of list nodes
		 * so that the array remains sorted in ascending order.
		 */

		/* Find the first element with a larger address */
		for (j = 0; j < i; j++)
			if (pmem[j].address > (u_int)rp[i].reg_addr)
				break;

		/* Copy following elements up to make room for the new one */
		for (k = i; k > j; --k)
			pmem[k] = pmem[k-1];

		/* Insert the new element */
		pmem[j].address = (u_int)rp[i].reg_addr;
		pmem[j].size = rp[i].reg_size;
	}
	for (i = 1; i < chunks; ++i) {
		if (pmem[i].address)
			pmem[i-1].next = &pmem[i];
	}
	return (pmem);
}

int cpu;
/*
 * We set the cpu type from the idprom machine ID.
 * XXX CPU name should be PROM property.
 */
void 
setcputype()
{

	cpu = -1;

	if (idromp->id_format == 1) {
		cpu = idromp->id_machine;
		if ((cpu & CPU_ARCH) != SUN4C_ARCH) {
			//printf("Unknown machine type 0x%x in NVRAM\n", cpu);
			cpu = -1;
		}
	} else {
		//printf("Invalid format type in NVRAM\n");
	}

	if (cpu == -1) {
		//printf("Machine type set to Sun-4/60\n");
		cpu = CPU_SUN4C_60;
	}
}

/*
 * Written by raila
 */


void
ethernetAddress(unsigned char *buffer)
{
	if (romp->v_romvec_version > 0) {
		Assert (idromp != 0);
		Assert (idromp->id_format == IDFORM_1);
		for (int i = 0; i < 6; i++)
			buffer[i] = idromp->id_ether[i];
		}
	else	{
		(*romp->v_mac_address)(0, buffer);
		}
}

void
SS1PROM::setmemlists( struct memlist * &phys_available, struct memlist * &virt_available,
	struct memlist * &phys_total)
{
	if (romp->v_romvec_version > 0) {
		phys_available = getmemlist("memory", "available");
		virt_available = getmemlist("virtual-memory", "available");
		phys_total = getmemlist("memory", "reg");
		}
	else {
		phys_available = *romp->v_availmemory;
		virt_available = *romp->v_virtmemory;
		phys_total = *romp->v_physmemory;
		}
	Assert (phys_available != 0);
	Assert (virt_available != 0);
	Assert (phys_total != 0);
}

unsigned npmegs = 0;
unsigned nctxts = 0;
unsigned vac_size = 0;
int vac_hwflush = 0;
unsigned vac_linesize = 0;

extern "C"
void
initNpmegs()
{
	npmegs = getprop(0, "mmu-npmg", 0);
}

void
SS1PROM::getHWInfo()
{
	vac_size = getprop(0, "vac-size", 0);
	vac_hwflush = getprop(0, "vac-hwflush", 0);
	vac_linesize = getprop(0, "vac-linesize", 0);
	setcputype();

/*
 * Sun  Proprietary portion of code from machdep.c in SunOS src
 */
	/*
	 * Workaround for bugid 1067719: vac-hwflush is mis-named
	 * vac_hwflush, and it has the wrong value for SS1+ anyway.
	 * (This is the bugfix for 1068462.)
	 */
	switch (cpu) {
	case CPU_SUN4C_75:	/* SS-2 */
	case CPU_SUN4C_50:	/* IPX */
	case CPU_SUN4C_25:	/* ELC */
		if (vac_hwflush == 0)
			vac_hwflush = 1;
	}
}

void
SS1PROM::exit_to_mon()
{
	(*romp->v_exit_to_mon)();
}
