#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "Types.h"
#include "AwaitedInterruptInterrupt.h"
#include "CPU.h"
#include "ThisCPU.h"
#include "Domain.h"
#include "NetworkBuffer.h"
#include "NetworkAdministrator.h"
#include "OutputStream.h"
#include "SPARCInterrupts.h"
#include "SPARCInterrupts.h"
#include "SPARCTranslation.h"
#include "SS1Am7990.h"
#include "SS1devices.h"
#include "SS1util.h"
#include "Semaphore.h"
#include "Process.h"
#include "ThisProcess.h"


#ifdef __GNUC__
#pragma implementation
#endif

/*
 * static defns
 */
char SS1Am7990::_base[sizeofBase];

const PacketBuffers = ReceiveBuffers ;

static _nextPacket = 0;

SS1Am7990 * le0 = 0; 
	/* 
	 * The stupid SPARCstation Lance high byte is wired to ones.
	 * So we have pages allocated and mapped to DMA_BASE
	 * in the VM initialization code, so we have to use placement for
	 * the buffers, etc.
	 * There are also alignment requirements (8 byte),
	 * see the constructor for the alignment...
	 * We surround the data with two pointers just in case all this
	 * stuff doesn't get allocated on a single page..... sigh
	 */

/* Used to force alignment to 8 byte */
inline unsigned Alignoffset(unsigned o)
{
	unsigned ret = o + (8 - (o % 8));
	Assert (ret > o);
	return ret;
}

/*******************************************************************/
void
Am7990InterruptProcessEntry()
{
	AwaitedInterruptInterrupt * interrupt =
		new AwaitedInterruptInterrupt();
	thisCPU()->setInterrupt( 21, interrupt);

	while (1) {
		Debug << thisProcess() << ": awaiting interrupt\n" << eor;
		interrupt->await();
		Debug << thisProcess() << ": interrupt\n" << eor;

		Assert (le0 != 0);
		le0->interruptHandler();
	}
}

#ifdef ECHO_TEST
Semaphore  sendSem(0);

void
Am7990SendProcessEntry()
{
	octet *packet = new octet [100];
	for (int i = 0; i < 100; i++)
		packet[i] = i;

	while(1) {
		sendSem.P();
		Console() << "." << eor;
		// le0->send(packet, 100);
		}
}
#endif

/*******************************************************************/

void
SS1Am7990::interruptHandler()
{
	SPARCTranslation::rangeFlush(_dmaBase, sizeofBase);
	Am7990::interruptHandler();
}

SS1Am7990::SS1Am7990(volatile Am7990Word *rap,
		     volatile Am7990Word *rdp,
		     unsigned char debugMask) :
	Am7990(rap, rdp, debugMask)
{
#ifdef BSDNET
	Assert( NOTREACHED );
#endif
	/*
	 * 1. Create the interrupt process and install it in the CPU.
	 * 2. Create a new driver le0.  The constructor causes an interrupt.
	 */
	Process * p = new InterruptProcess ( Am7990InterruptProcessEntry,
					     thisProcess()->domain(),
					     normal, 21);
	Assert (p != 0);
	p->setName("#Am7990InterruptProcess");
	p->ready();
	thisProcess()->relinquishProcessor(); // get it running

	Assert (le0 == 0);
	le0 = this; // barf, global variable...YOU'LL PAY FOR THIS, DAVE RAILA!

	/*
	 * 1. Allocate and map the DMA space, new the preallocated variables.
	 * 3. Allocate send and receive buffers, set superclass vars.
	 * 4. Set up initialization block and set _initBlock in superclass
	 */
	Debug << "SS1Am7990::SS1Am7990(" << asAddress((char*)rap) << ", " 
		<< asAddress((char*)rdp) << ")\n" << eor;


	_dmaBase = SPARCTranslation::mapToDMA(&_base[0], sizeofBase, 1);
	offset = 0;
	/*
	 * Create the init block in the preallocated memory, align 8
	 */
	offset = 0;
	_initBlock = new ( _dmaBase + offset ) Am7990Word;
	offset += sizeof(Am7990Word);
	Assert (offset < sizeofBase);
	for (int j = 1; j < 12; j++){
		new (_dmaBase + offset) Am7990Word;
		offset += sizeof(Am7990Word);
		Assert (offset < sizeofBase);
		}
		
	Debug << "le: Init Block allocated at " << asAddress(_initBlock)
	      << "\n" << eor;

		/*
		 * Allocate receive bufs
		 * Force 8 byte alignment:
		 */
	offset = Alignoffset(offset);
	Assert (offset < sizeofBase);
	_receiveRing = new (_dmaBase+offset) Am7990BufferDescriptor;
	offset += sizeof(Am7990BufferDescriptor);
	Assert (offset < sizeofBase);
	for (int i = 1; i < ReceiveBuffers; i++) {
		new (_dmaBase + offset) Am7990BufferDescriptor;
		offset += sizeof(Am7990BufferDescriptor);
		Assert (offset < sizeofBase);
		}
	Debug << "le: receive ring at " << asAddress(_receiveRing)
		<< "\n" << eor;

	_initBlock[8] = (unsigned)_receiveRing & 0xffff;
	_initBlock[9] = (((unsigned)_receiveRing >> 16) & 0xff)
			| (Log2ReceiveBuffers << 13) ;

	/*
	 * Allocate send bufs
	 * Force 8 byte alignment:
	 */
	offset = Alignoffset(offset);
	Assert (offset < sizeofBase);
	_transmitRing = new (_dmaBase+offset) Am7990BufferDescriptor;
	offset += sizeof(Am7990BufferDescriptor);
	Assert (offset < sizeofBase);
	for (i = 1; i < TransmitBuffers; i++) {
		new (_dmaBase + offset) Am7990BufferDescriptor;
		offset += sizeof(Am7990BufferDescriptor);
		Assert (offset < sizeofBase);
		}
	Debug << "le0: transmit ring at " << asAddress(_transmitRing)
	      << "\n" << eor;

	_initBlock[10] = (unsigned)_transmitRing & 0xffff;
	_initBlock[11] = (((unsigned)_transmitRing >> 16) & 0xff)
			 | (Log2TransmitBuffers << 13) ;

	/* 
	 * Set up enet address in init Block 
	 */
	octet enet[6];
	ethernetAddress(enet);
	_ethernetAddress = new EthernetAddress(enet);
	Console() << "Ethernet address: " << *_ethernetAddress << '\n' << eor;
	_initBlock[1] = (enet[1] << 8) | enet[0];
	_initBlock[2] = (enet[3] << 8) | enet[2];
	_initBlock[3] = (enet[5] << 8) | enet[4];
		
	/* 
	 * Set up logical address filter in init Block 
	 * Here is the algorithm, but Choices doesn't use
	 * multicast (yet).
	 */

#ifdef MULTICAST
	/*
 	 * Pass all our multicast addresses through the CRC and
 	 * set 1 bits.  We still have to check when the packet comes
	 * in, but this filters out a hell of a lot of 'em.
	 *
	 * The high order 6 bits of the CRC result is and index into the
	 * table, if the bit is 1 than the interface accepts the packet.
	 */

	foreach (multicast ethernet address) {
		unsigned char *p = (pointer to address);

                register u_long crc;

                crc = 0xffffffff;
                int len = 6;
                while (len-- > 0) {
                        unsigned c = *p;
                        for (int i = 0; i < 8; i++) {
                                if ((c & 0x01) ^ (crc & 0x01)) {
                                        crc >>= 1;
                                        crc = crc ^ 0xedb88320; /* polynomial */
                                } else
                                        crc >>= 1;
                                c >>= 1;
                        }
                        p++;
                }
                crc = crc >> 26;
                _initBlock[crc >> 4] |= 1 << (crc & 0xf);

        }
#endif /* MULTICAST */
}

SS1Am7990::~SS1Am7990()
{
}

void
SS1Am7990::ready(NetworkAdministrator *anAdministrator)
{
	Am7990::ready(anAdministrator);		// super ready: anAdministrator

	initChip();

	if (_initialized) {
		le0 = this;
		startChip();
		}

#ifdef ECHO_TEST
	Process *echo =  new SystemProcess( (PFV) Am7990SendProcessEntry,
		thisProcess()->domain(), normal, 0);
	Assert( echo != 0);
	echo->setName( "#SS1Am7990EchoProcess");
	echo->ready();
	thisProcess()->relinquishProcessor();
#endif

#ifdef TEST
	extern void ethernetTestProcessEntry();
	Process *p = new SystemProcess( (PFV) ethernetTestProcessEntry,
		thisProcess()->domain(), large, 0);
	Assert( p != 0);
	p->setName( "#ethernetTestProcess");
	p->ready();
	thisProcess()->relinquishProcessor();
#endif
}

int
SS1Am7990::transmitPacket(NetworkBuffer *buf)
{
	char *pkt = (char *)(buf->packet());
	unsigned len = buf->length();

	/*
	 * Lock down callers memory
	 * Map the packet to the DMA area
	 * Call the superclass send
	 * Unmap the packet
	 * return value returned to us
	 * chain destructor unlocks memory
	 */
/*
	PhysicalMemoryChain chain = 
		thisProcess()->domain()->constructChain((char *)pkt,len,1);
 */
	buf->setPacket((octet *)SPARCTranslation::mapToDMA((char *)pkt, len)); 
	int retval = Am7990::transmitPacket(buf);
	buf->setPacket((octet *)SPARCTranslation::unMapDMA((char *)(buf->packet())));
	return retval;
}

NetworkBuffer *
SS1Am7990::getBuffer(unsigned int len) 
{
	NetworkBuffer *buf = Am7990::getBuffer(len);
	buf->setPacket((octet *)SPARCTranslation::mapToDMA((char *)(buf->packet()), 
		buf->length())); 
	return buf;
}


#ifdef LINUXNET
void (*SS1Am7990ReceiveOverride)(NetworkBuffer *) = 0;
#endif

void
SS1Am7990::receivePacket(NetworkBuffer *buf) 
{
#ifdef ECHO_TEST
	sendSem.V();
#endif
	buf->setPacket((octet *)SPARCTranslation::unMapDMA((char *)(buf->packet())));
#ifdef LINUXNET
	if (SS1Am7990ReceiveOverride)
		(*SS1Am7990ReceiveOverride)(buf);
	else {
/*
		buggy code, skip
		NetworkDriver::receivePacket(buf);
 */
		buf->release();
	}
#else
	NetworkDriver::receivePacket(buf);
#endif
	
}

#ifdef TEST
#include "FreeRunningTimer.h"
#include "FreeRunningTimerStar.h"

#define PACKETS 1000
void
ethernetTestProcessEntry()
{

	char *packet = new char[MaxPacket];
	for (int i = 0; i < MaxPacket; i++)
		packet[i] = (unsigned char)i;

	unsigned totalbytes = 0;
	unsigned start = SystemTimer->time();
	for(int p = 0; p < PACKETS; p++) {
		int len = 100+(i++ % 300);
		le0->send(packet, len);
		totalbytes+=len;
		};
	unsigned end = SystemTimer->time();
	Console() << "\nSending " << PACKETS << "packets\n" ;
	Console() << "Sent " << PACKETS << " packets of average size "
		<< totalbytes/PACKETS << " in " << (end - start) 
		<< " microseconds (" << ((end-start) / PACKETS) 
		<< "microsec/pckt)\n" << eor;
}


#endif /* TEST */
