#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David Raila (raila@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * SS1Benchmarks
 */

#include "Byte.h"
#include "Domain.h"
#include "DummyMemoryObject.h"
#include "FreeRunningTimerStar.h"
#include "FreeRunningTimer.h"
#include "MemoryObjectViewStar.h"
#include "OutputStream.h"
#include "PhysicalMemoryChain.h"
#include "Process.h"
#include "ProxiableObject.h"
#include "ProxiableObjectStar.h"
#include "RootMemoryObjectCache.h"
#include "SimpleInstruments.h"
#include "SourceOfZerosMemoryObject.h"
#include "SPARCTranslation.h"
#include "String.h"
#include "ThisProcess.h"

class VMBenchmarks {
public:
	static int useTimers;
	static int main(int argc, char *argv[]);
	static int dmaMapUnmapFlush();
	static int memoryObjectAddRemove();
	static int madAllocator();
	static void onMemoryTimers();
	static void offMemoryTimers();
	static void resetMemoryTimers();
	static void displayMemoryTimers();
};

int VMBenchmarks::useTimers;

void SS1Benchmarks(int argc, char *argv[])
{
	int useTimers = 0;
	if (argc > 1 && strcmp(argv[1],"timer") == 0) 
		useTimers = 1;
		
	if (useTimers) {
		VMBenchmarks::useTimers = 1;
		VMBenchmarks::onMemoryTimers();
	} else {
		VMBenchmarks::useTimers = 1;
		VMBenchmarks::offMemoryTimers();
		VMBenchmarks::useTimers = 0;
	}
	VMBenchmarks::main(argc,argv);
	return;
}

int VMBenchmarks::main(int argc, char *argv[])
{
        VMBenchmarks::madAllocator();
        VMBenchmarks::dmaMapUnmapFlush();
	VMBenchmarks::memoryObjectAddRemove();
	return 0;
}

void VMBenchmarks::onMemoryTimers()
{
#ifdef SIMPLE_INSTRUMENTATION
	static char *on[] = {"inst", "k", "MemoryTimer", "on"};
	if (useTimers) simpleInstrumentControl(4, on);
#endif // SIMPLE_INSTRUMENTATION
}

void VMBenchmarks::offMemoryTimers()
{
#ifdef SIMPLE_INSTRUMENTATION
	static char *off[] = {"inst", "k", "MemoryTimer", "off"};
	if (useTimers) simpleInstrumentControl(4, off);
#endif // SIMPLE_INSTRUMENTATION
}

void VMBenchmarks::resetMemoryTimers()
{
#ifdef SIMPLE_INSTRUMENTATION
	static char *reset[] = {"inst", "k", "MemoryTimer", "r"};
	if (useTimers) simpleInstrumentControl(4, reset);
#endif // SIMPLE_INSTRUMENTATION
}

void VMBenchmarks::displayMemoryTimers()
{
#ifdef SIMPLE_INSTRUMENTATION
	static char *display[] = {"inst", "k", "MemoryTimer", "d"};
	if (useTimers) simpleInstrumentControl(4, display);
#endif // SIMPLE_INSTRUMENTATION
}

int VMBenchmarks::madAllocator()
{
	const int NTIMES = 50;
	char * array[NTIMES];
	for( int log2Size = 2; log2Size <= 12; log2Size++ ) {
		int size = 1 << log2Size;
		int t1 = SystemTimer->time();
		for (int i = 0; i < NTIMES; i++) {
			array[i] = new char[size];
		}
		int t2 = SystemTimer->time();
		for (i = 0; i < NTIMES; i++) {
			delete array[i];
		}
		int t3 = SystemTimer->time();
		Console() 
		<< "Size = " << size << " ntimes = " << NTIMES
		<< " alloc = " << (t2-t1)/NTIMES 
		<< " free = " << (t3-t2)/NTIMES
		<< "\n" << eor;
	}
	Console() << "\n" << eor;
}

int VMBenchmarks::dmaMapUnmapFlush()
{
	Console() << "VMBenchmarks::dmaMapUnmapFlush()\n" << eor;

	int loop0 = 0;
	int NTIMES = 1;
	//int units =  1;
        char * pkt  = new char[4096];
 
        for(int i=0; i< 4095;i++)
                pkt[i] = 0;
 
        PhysicalMemoryChain chain =
                thisProcess()->domain()->constructChain((char *)pkt,4096);
 
        int t0 = SystemTimer->time();
        char *adr =(char *)SPARCTranslation::mapToDMA((char *)pkt, 4096);
        int t1 = SystemTimer->time();
 
        int t2 = SystemTimer->time();
	chain = PhysicalMemoryChain();
        SPARCTranslation::unMapDMA((char *)adr);
        int t3 = SystemTimer->time();
	IntStorage state ; PROCESSOR::disableInterrupts(state);
        SPARCTranslation::rangeFlush((char *)pkt,4096);
        PROCESSOR::restoreInterrupts(state);
 
        int t4 = SystemTimer->time();
	PROCESSOR::disableInterrupts(state);
        SPARCTranslation::rangeFlush((char *)pkt,4096);
        PROCESSOR::restoreInterrupts(state);
        int t5 = SystemTimer->time();
 
        for(i=0;i< 4095;i++)
                pkt[i] = 1;
 
        int t6 = SystemTimer->time();
	PROCESSOR::disableInterrupts(state);
        SPARCTranslation::rangeFlush((char *)pkt,4096);
        PROCESSOR::restoreInterrupts(state);
        int t7 = SystemTimer->time();
 
        int t8 = SystemTimer->time();
	PROCESSOR::disableInterrupts(state);
        SPARCTranslation::rangeFlush((char *)pkt,4096);
        PROCESSOR::restoreInterrupts(state);
        int t9 = SystemTimer->time();
 
	Console() 
		<< "mapTODMA "
                << (t1-t0-loop0)/NTIMES << "\n"
		<< "umMapDMA "
                << (t3-t2-loop0)/NTIMES << "\n"
		<< "empty flush "
                << (t5-t4-loop0)/NTIMES << "\n"
                << "dirty flush "
                << (t7-t6-loop0)/NTIMES << "\n"
                << "clean flush "
                << (t9-t8-loop0)/NTIMES << "\n\n" 
                << eor;

        return 0;
}

int VMBenchmarks::memoryObjectAddRemove()
{
	Console() << "VMBenchmarks::memoryObjectAddRemove()\n" << eor;

	const unsigned int NTIMES = 25;
	MemoryObjectStar mo[NTIMES];
	MemoryObjectCache * cache[NTIMES];
	char * baseAddr[NTIMES];
	unsigned int i;
	Domain * domain = thisProcess()->domain();

	onMemoryTimers();

	for( unsigned log2Size = 12; log2Size < 17; log2Size++ ) {
		
		unsigned size = 1 << log2Size;
	        MemoryObjectViewStar mov[NTIMES];

		Console() 
		<< "Size = " << size << " ntimes " << NTIMES << "\n" << eor;
		
		/* create source of zeros memory object */

		int t1 = SystemTimer->time();
		for( i = 0; i < NTIMES; i++ ) {
			mo[i] = new SourceOfZerosMemoryObject;
			mo[i]->setNumberOfUnits( size );
			mo[i]->setCacheClass( RootMemoryObjectCacheClass );
		}
		int t1e = SystemTimer->time();

		/* build cache */

		int t2 = SystemTimer->time();
		for( i = 0; i < NTIMES; i++) {
			cache[i] = mo[i]->buildCache();
			Assert( cache[i] != 0 );
		}
		int t2e = SystemTimer->time();

		/* add to domain */
 
		Console() << "Domain::add()\n" << eor;
		resetMemoryTimers();
		int t3 = SystemTimer->time();
		for( i = 0; i < NTIMES; i++ ) {
			baseAddr[i] = (char *) domain->add( mo[i], NoAccess );
			Assert(baseAddr[i] != 0);
		}
		int t3e = SystemTimer->time();
		displayMemoryTimers();

		/* make addressable */

		Console() << "Domain::makeAddressable()\n" << eor;
		resetMemoryTimers();
		int t4 = SystemTimer->time();
		for( i = 0; i < NTIMES; i++ ) {
			domain->makeAddressable( baseAddr[i], size );
		}
		int t4e = SystemTimer->time();
		displayMemoryTimers();

		/* construct chain */

		Console() << "Domain::constructChain()\n" << eor;
		resetMemoryTimers();
		int t5 = SystemTimer->time();
		for( i = 0; i < NTIMES; i++ ) { 
			(void)domain->constructChain( baseAddr[i], size, 0 );
		}
		int t5e = SystemTimer->time();
		displayMemoryTimers();

		/* make unaddressable */

		Console() << "Domain::makeUnAddressable()\n" << eor;
		resetMemoryTimers();
		int t6 = SystemTimer->time();
		for( i = 0; i < NTIMES; i++ ) {
			domain->makeUnAddressable( baseAddr[i], size );
		}
		int t6e = SystemTimer->time();
		displayMemoryTimers();

		/* free */

		Console() << "MemoryObjectCache::free()\n" << eor;
		resetMemoryTimers();
		int last = cache[0]->numberOfUnits() - 1;
		int t7 = SystemTimer->time();
		for( i = 0; i < NTIMES; i++ ) {
			cache[i]->free(0, last);
		}
		int t7e = SystemTimer->time();
		displayMemoryTimers();
		
		/* construct chain */

		Console() << "domain::constructChain()\n" << eor;
		resetMemoryTimers();
		int t8 = SystemTimer->time();
		for( i = 0; i < NTIMES; i++ ) {
			(void)domain->constructChain( baseAddr[i], size, 1 );
		}
		int t8e = SystemTimer->time();
		displayMemoryTimers();

		/* untranslatable */
		
		Console() << "MemoryObjectCache::untranslatable()\n" << eor;
		resetMemoryTimers();
		int t9= SystemTimer->time();
		for( i = 0; i < NTIMES; i++ ) {
			cache[i]->untranslatable(0, last, 
						 domain->translation(),
						 baseAddr[i]);
		}
		int t9e = SystemTimer->time();
		displayMemoryTimers();
		
		/* lookup views */

		Console() << "Domain::lookup()\n" << eor;
		resetMemoryTimers();
		int t10 = SystemTimer->time();
		for( i = 0; i < NTIMES; i++ ) {
			ProtectionLevel level;
		        char * addr;
			unsigned length;
		        mov[i] = domain->lookup( baseAddr[i], addr,
						 length, level );
		}
		int t10e = SystemTimer->time();
		displayMemoryTimers();

		/* translatable */

		Console() << "MemoryObjectCache::translatable()\n" << eor;
		resetMemoryTimers();
		int t11 = SystemTimer->time();
		for( i = 0; i < NTIMES; i++ ) {
			cache[i]->translatable(0, last, NoAccess,
					       domain->translation(),
					       baseAddr[i], mov[i],
					       baseAddr[i], 0);
		}
		int t11e = SystemTimer->time();
		displayMemoryTimers();

		/* unref views */

		resetMemoryTimers();
		int t12 = SystemTimer->time();
		for( i = 0; i < NTIMES; i++ ) {
		        mov[i] = 0;
		}
		int t12e = SystemTimer->time();

		/* other tests */


		/* remove from domain */

		resetMemoryTimers();
		int t90 = SystemTimer->time();
		for( i = 0; i < NTIMES; i++ ) {
			char * addr = (char *)
				domain->remove( mo[i], baseAddr[i] );
			Assert( addr == baseAddr[i] );
		}
		int t90e = SystemTimer->time();
		
		/* delete memory object and cache */
		
		resetMemoryTimers();
		int t91 = SystemTimer->time();
		for( i = 0; i < NTIMES; i++ ) {
			mo[i] = 0;
		}
		int t91e = SystemTimer->time();
		
		/* new and delete */

		char *source = new char[size];
		char *array[NTIMES];
		ProxiableObjectStar po[NTIMES];
		int t100 = SystemTimer->time();
		for ( i = 0; i < NTIMES; i++ ) {
			array[i] = new char[sizeof(ProxiableObject)];
		}
		int t100e = SystemTimer->time();

		int t101 = SystemTimer->time();
		for ( i = 0; i < NTIMES; i++ ) {
			delete array[i];
		}
		int t101e = SystemTimer->time();

		int t102 = SystemTimer->time();
		for ( i = 0; i < NTIMES; i++ ) {
			po[i] = new ProxiableObject;
		}
		int t102e = SystemTimer->time();

		int t103 = SystemTimer->time();
		for ( i = 0; i < NTIMES; i++ ) {
			po[i] = 0;
		}
		int t103e = SystemTimer->time();

		int t104 = SystemTimer->time();
		for ( i = 0; i < NTIMES; i++ ) {
			array[i] = new char[size];
		}
		int t104e = SystemTimer->time();

		int t105 = SystemTimer->time();
		for ( i = 0; i < NTIMES; i++ ) {
			ByteZero( array[i], size );
		}
		int t105e = SystemTimer->time();

		int t106 = SystemTimer->time();
		for ( i = 0; i < NTIMES; i++ ) {
			ByteCopy( source, array[i], size );
		}
		int t106e = SystemTimer->time();

		int t107 = SystemTimer->time();
		for ( i = 0; i < NTIMES; i++ ) {
			delete array[i];
		}
		int t107e = SystemTimer->time();

		int t108 = SystemTimer->time();
		for ( i = 0; i < NTIMES; i++ ) {
			po[i] = new DummyMemoryObject(size>>12, 12);
		}
		int t108e = SystemTimer->time();

		int t109 = SystemTimer->time();
		for ( i = 0; i < NTIMES; i++ ) {
			po[i] = 0;
		}
		int t109e = SystemTimer->time();

		delete source;
		
		Console() 
		<< "Size = " << size << " ntimes " << NTIMES << "\n"
		<< "  create MO        : " << (t1e - t1)/NTIMES << "\n"
		<< "  create MOC       : " << (t2e - t2)/NTIMES << "\n"
		<< "  domain add       : " << (t3e - t3)/NTIMES << "\n"
	        << "  make addressable : " << (t4e - t4)/NTIMES << "\n"
		<< "  construct chain  : " << (t5e - t5)/NTIMES << "\n"
		<< "  make unaddr      : " << (t6e - t6)/NTIMES << "\n"
		<< "  free             : " << (t7e - t7)/NTIMES << "\n"
		<< "  construct chain  : " << (t8e - t8)/NTIMES << "\n"
		<< "  cache unstrans   : " << (t9e - t9)/NTIMES << "\n"
		<< "  lookup view      : " << (t10e - t10)/NTIMES << "\n"
		<< "  cache trans      : " << (t11e - t11)/NTIMES << "\n"
		<< "  unref view       : " << (t12e - t12)/NTIMES << "\n"
		<< eor;
		Console()
		<< "  domain remove    : " << (t90e - t90)/NTIMES << "\n"
		<< "  destroy MO & MOC : " << (t91e - t91)/NTIMES << "\n"
		<< eor;
		Console() 
		<< "  alloc char[PO]   : " << (t100e-t100)/NTIMES << "\n"
		<< "  free char[PO]    : " << (t101e-t101)/NTIMES << "\n"
		<< "  new PO           : " << (t102e-t102)/NTIMES << "\n"
		<< "  delete PO        : " << (t103e-t103)/NTIMES << "\n"
		<< "  alloc char[Size] : " << (t104e-t104)/NTIMES << "\n"
		<< "  zero char[Size]  : " << (t105e-t105)/NTIMES << "\n"
		<< "  copy char[Size]  : " << (t106e-t106)/NTIMES << "\n"
		<< "  free char[Size]  : " << (t107e-t107)/NTIMES << "\n"
		<< "  new MO           : " << (t108e-t108)/NTIMES << "\n"
		<< "  delete MO        : " << (t109e-t109)/NTIMES << "\n"
		<< eor;
	}

	Console() << "\n" << eor;
	offMemoryTimers();
}
