#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * SS1Console.c: SS1 Diagnostic (polling) console output routines.
 */

#include "Assert.h"
#include "Process.h"
#include "ThisProcess.h"
#include "AwaitedInterruptInterrupt.h"
#include "CharacterInputStream.h"
#include "CharacterQueue.h"
#include "CPU.h"
#include "ThisCPU.h"
#include "Gdb.h"
#include "OutputStream.h"
#include "SS1Console.h"
#include "zs.h"
#include "Vectors.h"
#include "TimeoutTimer.h"

#ifdef __GNUC__
#pragma implementation
#endif


const unsigned TIMEOUT = 10000;


// 1 when initialized
int consoleInputInitialized = 0;

SS1Console::SS1Console(volatile unsigned char *const ctlAddr)
{
	extern consINit;
	consINit = 1;
	_myzs = new zs(ctlAddr);

}

void
SS1Console::acquireExclusiveUse()
{
	Assert( ! lowPriorityLock.heldByMe() );
	lowPriorityLock.acquire();
}

void
SS1Console::releaseExclusiveUse()
{
	lowPriorityLock.release();
}

void
SS1Console::acquireHighPriorityUse()
{
	Assert( ! highPriorityLock.heldByMe() );
	highPriorityLock.acquire();
}

void
SS1Console::releaseHighPriorityUse()
{
	highPriorityLock.release();
}

/*
 * SS1Console::putChar - write a single character to the console
 */
void
SS1Console::putChar( char c )
{
	int i;
	for (i = 0; i < TIMEOUT; i++)
		if ((_myzs->readCTL(0) & 0x04) == 0x04) break;
	Assert ( i < TIMEOUT);

	_myzs->writeDATA(c);

	if ( c == '\n' ) {
		for (i = 0; i < TIMEOUT; i++)
			if ((_myzs->readCTL(0) & 0x04) == 0x04) break;
		Assert ( i < TIMEOUT);

		_myzs->writeDATA('\r');
		}
}

int
SS1Console::nonBlockingGetChar()
{
	return ( (_myzs->readCTL(0) & 0x01) == 0x01);
}

int
SS1Console::getChar()
{
#ifdef DISABLECONSOLE
	static int i = 0;
	if (i == 0)
	{
	    extern int TimeoutManager;
	    while (TimeoutManager == 0)
	    	thisProcess () -> relinquishProcessor ();
    	    Console () << "Console timer started\n" << eor;		
    	    TimeoutTimer *t = new TimeoutTimer;
	    t -> start (3 * 60 * 1000000);
	    t -> await ();
	}

	char *str = "22\n321\n";
	if (str [i] != 0)
	    return str [i++];
	
    	Semaphore *s = new Semaphore (0);
	s -> P ();  //  Wait forever.
#endif
	extern void Halt();

	while ((_myzs->readCTL(0) & 0x01) != 0x01)
		if (consoleInputInitialized)
			thisProcess()->relinquishProcessor();

	int ch = _myzs->readDATA();
	if (ch == 16)
		Halt();
	if (ch == 26)
		TrapToDebugger();
	return ch;
}

int
SS1Console::write( const char * buf, int count )
{
	for( int i = 0; i < count; i++ ) {
		putChar( *buf++ );
	}
	return(count);
}

void panic( const char * format,
                const formatArgs & a1 = *(formatArgs *)0,
                const formatArgs & a2 = *(formatArgs *)0,
                const formatArgs & a3 = *(formatArgs *)0,
                const formatArgs & a4 = *(formatArgs *)0 )
{
	if (consoleInitialized) 
		Console().panic(format, a1, a2, a3, a4);
	else
		TrapToDebugger();
}

void SS1Console::panic( const char * format,
                const formatArgs & a1,
                const formatArgs & a2,
                const formatArgs & a3,
                const formatArgs & a4)

{

	if (consoleInitialized) {
		acquireHighPriorityUse();
		emitNewline();
		emitIdentifierOfCurrentProcessor();
		formattedWrite( "PANIC: " );
		formattedWrite( format,
			( &a1 != 0) ? a1._formatArgs.l : 0,
			( &a2 != 0) ? a2._formatArgs.l : 0,
			( &a3 != 0) ? a3._formatArgs.l : 0,
			( &a4 != 0) ? a4._formatArgs.l : 0 );
		}
	TrapToDebugger();
}

void
SS1Console::emitNewline()
{
	putChar('\n');
}

void
ConsoleInputProcess()
{
	AwaitedInterruptInterrupt * interrupt =
		new AwaitedInterruptInterrupt();

        thisCPU()->setInterrupt(VEC_UART, interrupt);

	while (1) {
		Debug << thisProcess() << ": awaiting interrupt\n" << eor;
		interrupt->await();
		Debug << thisProcess() << ": interrupt\n" << eor;
		char ch = ((SS1Console*)ConsoleOutputStream)->getChar();
                Assert ( ConsoleInputStream != 0 );
                ConsoleInputStream->enqueue( &ch, 1 );
		}
	Assert (NOTREACHED);
}

void
SS1Console::setInterruptMode()
{
	/*
	 * start the interrupt handler
	 */
	Process * process = new InterruptProcess( 
		(PFV) ConsoleInputProcess, thisProcess()->domain(),
		small, 0 );
	Assert( process != 0 );
	process->setName( "#ConsoleInputProcess" );
	Debug <<  "ConsoleManagerProcess: ConsoleInputProcess at " 
		<< asAddress(process) << "\n" << eor;
	process->ready();
	thisProcess()->relinquishProcessor();

	_myzs->enableRint();
	consoleInputInitialized = 1;
}

