#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: See-Mong Tan (stan@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * SS1Disk.c: SS1 Disk Driver
 *
 */

#include "Assert.h"
#include "Domain.h"
#include "Class.h"
#include "DummyMemoryObject.h"
#include "ESP.h"
#include "FileSystemSimpleInstruments.h"
#include "Kernel.h"
#include "MemoryObjectPartition.h"
#include "PhysicalMemoryChain.h"
#include "Process.h"
#include "Raid.h"
#include "SS1Configuration.h"
#include "SS1Disk.h"
#include "SS1DiskPartition.h"
#include "ThisProcess.h"

extern "C" {
#include <sys/types.h>
#include <sys/dklabel.h>
};

#ifdef __GNUC__
#pragma implementation
#endif

SS1Disk::SS1Disk( ESP *controller, unsigned drive=0 )
     : Disk(1), mutex(),
     _myController(controller),	_unit(drive)
{
    Debug << "SS1Disk::SS1Disk()\n" << eor;

    _segmentSize = 1 << 16; // 64 kb
    char *vendor = new char[200] ;
    _blocks = 0;
    _blocksize = 0;
    int status = _myController->diskInfo(_unit, &_blocks, &_blocksize, vendor, 200);
    Assert (status == 0);
    Debug << "SS1Disk::SS1Disk() (unit " << _unit << ")\n\t"
	<< vendor << "\n\t" << _blocks << " blocks of size " << _blocksize
	<< "\n" << eor;
    delete vendor;

	_numberOfUnits = _blocks;
	// Assert (_blocksize == 512);
	_log2UnitSize = 9;

    parts = new SS1DiskPartition*[NDKMAP];
    char * label = new char [sizeof(struct dk_label)];
    _myController->read(_unit, 0, 1, label, sizeof(struct dk_label));
    struct dk_label *lab = (struct dk_label *)label;
    struct dk_map *dk_part = lab->dkl_map;
    Assert (lab->dkl_magic == DKL_MAGIC);
    Console() << "SS1Disk: unit " << drive << ": " << label << "\n" << eor;
    for (int i = 0; i < NDKMAP; i++) {
	if (i == 2) {
		parts[i] = 0;
		_numberOfUnits = dk_part[i].dkl_nblk;
		continue;	// Partition 2 is the whole disk for suns
		}
	Debug << "Partition " << i << " start: " << dk_part[i].dkl_cylno
		<< " blocks: " << dk_part[i].dkl_nblk << "\n" << eor;
	if (dk_part[i].dkl_nblk)
		parts[i] = new SS1DiskPartition(dk_part[i].dkl_nblk,
			lab->dkl_nhead*lab->dkl_nsect*dk_part[i].dkl_cylno );
	else
		parts[i] = 0;
    }
    delete label;
}

int
SS1Disk::locatePartition( int idNumber, int &start, int &units )
{
	start = 0;
	units = 0;
	if (! parts[idNumber])
		return -1;
	start = parts[idNumber]->_offset;
	units = parts[idNumber]->_size;
	return 0;
}

/*
 * doio - read/write to/from disk.
 *
 * When called the address pointed to is guaranteed to be locked
 * down in memory.  We have to remap it to the DMA area and do the
 * operation.
 */

//FileSystemTimer SS1DiskTimer("SS1DiskTimer");

int
SS1Disk::doio(unsigned int start, int count, char * addr, int nbytes, int op)
{
//	SInstr(int start = SS1DiskTimer.start());
	Assert (nbytes == (count << 9));
	unsigned ret = 0;

#ifdef NEVER
	/*
	 * Chainize the buffer to lock it in memory, 
	 */
	static PhysicalMemoryChain bufferChain;
	if (nbytes > 0) 
		bufferChain = thisProcess()->domain()->constructChain
			      ( addr, nbytes, 1);
#endif
 

	switch (op) {
		case DiskReadOperation:
			ret = _myController->read(_unit, start, count, 
						  addr, nbytes);
			break;
		case DiskWriteOperation:
			ret = _myController->write(_unit, start, count, 
						   addr, nbytes);
			break;
		default:
			Assert (NOTREACHED);
			break;
		}
	//bufferChain = PhysicalMemoryChain();
//	SInstr(SS1DiskTimer.stop(start));
	return ret;
}

#include "ClassInstaller.h"
DEFINECLASS("SS1Disk",SS1Disk,Disk,0,0);

