#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SS1FreeRunningTimer.cc
 */
#include "Assert.h"
#include "FreeRunningTimerStar.h"
#include "OutputStream.h"
#include "SPARCasmutil.h"
#include "SS1FreeRunningTimer.h"
#include "SS1TimeOfDay.h"
#include "SS1Timer.h"


#ifdef __GNUC__
#pragma implementation
#endif

SS1FreeRunningTimer theSystemTimer;
FreeRunningTimerStar SystemTimer = &theSystemTimer;

unsigned SS1FreeRunningTimer::constructed = 0;
unsigned SS1FreeRunningTimer::timesCalled;
unsigned SS1FreeRunningTimer::interrupts;
unsigned SS1FreeRunningTimer::prevTime;
Lock * SS1FreeRunningTimer::FreeRunningTimerLock;

void
SS1FreeRunningTimerInterrupt::basicRaise(char *)
{
	Debug << "SS1FreeRunningTimerInterrupt::basicRaise()\n" << eor;

	Assert (SystemTimer != 0);
	((SS1FreeRunningTimer *)SystemTimer)->tick();
}

SS1FreeRunningTimer::SS1FreeRunningTimer()
{
	Debug << "SS1FreeRunningTimer::SS1FreeRunningTimer()\n" << eor;

	Assert ( !constructed );
	lastTime = 0;
	timesCalled = 0;
	interrupts = 0;
	prevTime = 0;
	FreeRunningTimerLock = new Lock();
	FreeRunningTimerLock->acquire();
	constructed++;
	*LIMIT1 = 0;
	FreeRunningTimerLock->release();
	Debug << "INTREG :" << asHex(*INTREG) << "\n" << eor;
}

void
SS1FreeRunningTimer::tick()
{
	Debug << "SS1FreeRunningTimer::tick()\n" << eor;
	FreeRunningTimerLock->acquire();
	unsigned limit = *LIMIT1;
	FORCE_USE(limit);
	unsigned int prevTime = lastTime;
	lastTime += FRTuSecPerInterrupt;
	interrupts++;
	if (theSystemClock != 0 && prevTime > lastTime) 
		((SS1TimeOfDay *) theSystemClock)->systemTimerOverflow();
	FreeRunningTimerLock->release();
}


#ifdef DEBUG
static unsigned PreviusResult = 0;
static int previousPending = 0;
static int previousPreviousPending = 0;
static unsigned previousTimertime = 0;
static unsigned previouscorrected = 0;
#endif

unsigned int
SS1FreeRunningTimer::time()
{
	Assert (FreeRunningTimerLock != 0);
	FreeRunningTimerLock->acquire();
	timesCalled++;
	unsigned t;
	unsigned timerval;
	unsigned correctedLastTime;
	unsigned timertime;
	int pending = 0;

	timerval = *COUNTER1 ;
	correctedLastTime = lastTime;

	timertime = ((timerval & COUNTER_MASK) >> COUNTER_SHIFT);

	if (timerval & LIMIT_MASK) {
		/* 
		 * interrupt pending, clock rolled over but lastTime not
		 * updated yet.
		 */
		correctedLastTime += FRTuSecPerInterrupt;
		pending = 1;
		}


	t = correctedLastTime + timertime; 
	Assert ((t - correctedLastTime) == timertime);


	Debug << "SS1FreeRunningTimer::time() correctedLast = " << 
		asHex(correctedLastTime) << "\n" << eor;
	Debug << "SS1FreeRunningTimer::time() counter says " << 
		asHex(timerval) << "(" << asHex( (timerval&COUNTER_MASK) >>
		COUNTER_SHIFT) << ")\n" << eor;
	Debug << "INTREG :" << asHex(*INTREG) << "\n" << eor;
	Debug << "SS1FreeRunningTimer::time() returns " << asHex(t) << "\n" << eor;

#ifdef  DEBUG
	Assert (t > PreviusResult);
	PreviusResult = t;
	previousPreviousPending = previousPending;
	previousPending = pending;
	previousTimertime = timertime;
	previouscorrected = correctedLastTime;
#endif

/*
	extern int ProcessResourceUsageOn;
	Assert( !ProcessResourceUsageOn || ( t - prevTime ) < 0x80000000 );
 */
	prevTime = t;
	FreeRunningTimerLock->release();
	return (t);
}

#include "ClassInstaller.h"
DEFINECLASS("SS1FreeRunningTimer",SS1FreeRunningTimer,FreeRunningTimer,0,0);
