#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

// #define DEBUG_REGISTERS

#include "Gdb.h"
#include "OutputStream.h"
#include "SPARCasmutil.h"
#include "SS1Console.h"
#include "SS1devices.h"
#include "SS1Gdb.h"
#include "SS1util.h"
#include "SPARCPageTableConstants.h"
#include "SPARCTranslation.h"
#include "zs.h"
#include <sys/psw.h>
#include <sys/reg.h>
#include <sys/asm_linkage.h>
//#include <sys/mmu.h>


#ifdef __GNUC__
#pragma implementation
#endif

volatile unsigned char *const GDB_CTL = SERIAL_A_CTL;
volatile unsigned char *const GDB_DATA = SERIAL_A_DATA;


/*******************************************************************
*   FROM THE GDB SOURCE
*   #define NUM_REGS 72
*   #define REGISTER_NAMES  \
* 	{ "g0", "g1", "g2", "g3", "g4", "g5", "g6", "g7",       \
*   "o0", "o1", "o2", "o3", "o4", "o5", "sp", "o7",       \
*   "l0", "l1", "l2", "l3", "l4", "l5", "l6", "l7",       \
*   "i0", "i1", "i2", "i3", "i4", "i5", "fp", "i7",       \
*                                                                 \
*   "f0", "f1", "f2", "f3", "f4", "f5", "f6", "f7",       \
*   "f8", "f9", "f10", "f11", "f12", "f13", "f14", "f15", \
*   "f16", "f17", "f18", "f19", "f20", "f21", "f22", "f23",       \
*   "f24", "f25", "f26", "f27", "f28", "f29", "f30", "f31",       \
*                                                                 \
*   "y", "psr", "wim", "tbr", "pc", "npc", "fpsr", "cpsr" };
********************************************************************/

unsigned GdbBreakInsn = 0x91d02001;


void 
GdbKill()
{
	extern void Halt();
	Halt();
}

unsigned debuggingUser = 0;
unsigned * registers = 0;

int consINit = 0;
void                     
GdbMachineHandlerStart(int vector, char* trapSP)
{
	debuggingUser = ((unsigned*)trapSP)[0] & PSR_PS;
}

int
GdbMachineMem_ok(unsigned type, char *addr, unsigned len)
{
	register char * basepage = pageAlign(addr);
	register char * lastpage = pageAlign(addr + len - 1);
	unsigned mask;
	switch (type) {
		case GDB_WRITE:
			mask = PG_V|PG_W;
			break;
		case GDB_READ:
			mask = PG_V;
			break;
		default:
			mask = 0;
			Assert(NOTREACHED);
			break;
		}
	while (basepage <= lastpage) {
		unsigned pte;
		asm volatile("lda [%1]4,%0" : "=r"(pte) : "r"(basepage) );
		if ( (pte & mask) != mask )
			return 1;
		basepage += PhysicalPageSize;
		}
	return 0;
}



void
SPARCGdbHandler(int vector, char* trapSP)
{

	flushWindows();			// Flush everything out to the stack
	unsigned * lastRegisters = registers;	// Save for later restore
	/*
         * 1.  Allocate and fill in the registers structure.
	 * 2.  Point the global registers at it.
	 * 3.  Call the gdb handler
	 * 4.  Restore the registers
	 */
	unsigned myregisters[NUM_REGISTERS];
	char *userSP = ((char**)trapSP)[14];
	unsigned *source;
	unsigned *dest;

/******************************************************************************
 * Store away the registers (from the stack) into myregisters
 ******************************************************************************/
	/*
	 * Get the saved globals
	 */
	source = (unsigned *)(trapSP + MINFRAME + 3*4);
	dest = myregisters;
	for (int i = 0; i < 8; i++)
		*dest++ = *source++;
	/*
	 * Get the saved outs, the trap window ins.
	 */
	source = (unsigned *) (trapSP + 8*4);
	dest = &myregisters[8];
	for (i = 0; i < 8; i++)
		*dest++ = *source++;
	/*
	 * Get the saved locals
	 */
	source = (unsigned *) userSP;
	dest = &myregisters[16];
	for (i = 0; i < 8; i++)
		*dest++ = *source++;
	/*
	 * Get the saved ins
	 */
	source = (unsigned *) (userSP + 8*4);
	dest = &myregisters[24];
	for (i = 0; i < 8; i++)
		*dest++ = *source++;
	/*
	 * Get the special registers, zero out currently unused regs
	 */
	source = (unsigned *)( trapSP + MINFRAME);
	unsigned y = 0;
	asm volatile("mov %%y, %0" : "=r"(y) );
	myregisters[64] = y;	// %y
	myregisters[65] = source[PSR];	// psr
	//unsigned wim = 0;
	//asm volatile("mov %%wim, %0" : "=r"(wim) );
	myregisters[66] = * ((unsigned *)(trapSP + 3*4));	// %wim
	myregisters[67] = 0;	// %tbr
	myregisters[68] = source[PC];	// %pc
	myregisters[69] = source[nPC];	// %npc
	myregisters[70] = 0;	// %fpsr
	myregisters[71] = 0;	// %cpsr

/******************************************************************************
 * Point the registers structure there and go to the handler
 ******************************************************************************/
	registers = myregisters;
#ifdef DEBUG_REGISTERS
	unsigned pre_registers[NUM_REGISTERS];	// Copy before call 
	unsigned *pre_reg_ptr = registers;
	for (i = 0; i < NUM_REGISTERS; i++)
		pre_registers[i] = registers[i];

#endif
	unsigned * pcptr = (unsigned*)registers[68]; 
	GdbHandler(vector, trapSP);
	source = (unsigned*)trapSP;
	if (*pcptr == 0x91d02001){	// Fix, PC and nPC to skip ta 1 insn
#ifdef DEBUG_REGISTERS
		Console() << "Fixing BP PC\n" <<  eor;
#endif
		registers[68] = registers[69];
		registers[69] = registers[69] + 4;
		}
	else
		registers[68] = (unsigned)pcptr;

#ifdef DEBUG_REGISTERS
	if (registers != pre_reg_ptr)
		Console() << "registers ptr changed from " 
			<< asAddress(pre_reg_ptr) << " to " 
			<< asAddress(registers) << "\n" << eor;
	for (i = 0; i < NUM_REGISTERS; i++)
		if (registers[i] != pre_registers[i]) {
			Console() << "registers[" << i << "] changed from "
				<< asHex(pre_registers[i]) << " to "
				<< asHex(registers[i]) << "\n" << eor;
			}
	Console() << "returning from handler\n" << eor;
#endif


/******************************************************************************
 * Restore the (possibly changed) registers structures to their sources
 ******************************************************************************/
	flushWindows();	// So the window code restores them from memory written
	/*
	 * Restore the saved globals
	 */
	dest = (unsigned *)(trapSP + MINFRAME + 3*4);
	source = myregisters;
	for (i = 0; i < 8; i++)
		*dest++ = *source++;
	/*
	 * Restore the saved outs, the windows ins.
	 */
	dest = (unsigned *) (trapSP + 8*4);
	source = &myregisters[8];
	for (i = 0; i < 8; i++)
		*dest++ = *source++;
	/*
	 * Restore the saved locals
	 */
	dest = (unsigned *) userSP;
	source = &myregisters[16];
	for (i = 0; i < 8; i++)
		*dest++ = *source++;
	/*
	 * Restore the saved ins
	 */
	dest = (unsigned *) (userSP + 8*4);
	source = &myregisters[24];
	for (i = 0; i < 8; i++)
		*dest++ = *source++;

	/*
	 * Restore the special registers, zero out currently unused regs
	 */
	dest = (unsigned *)( trapSP + MINFRAME);
	dest[PC] = myregisters[68];	// %pc
	dest[nPC] = myregisters[69];	// %npc

/******************************************************************************
 * Restore the old registers pointer and get outta here
 ******************************************************************************/
	registers = lastRegisters;		// Restore old value
}




void                     
GdbMachineHandlerEnd(int vector, char* stack)
{
	debuggingUser = 0;
}

char
GetDebugChar()
{
        char inch;

        while((zsreadreg(GDB_CTL, 0) & 0x01) != 0x01) ;
        inch = zsreaddata(GDB_DATA);
        Debug << "getDebugChar got : " << inch << "\n" << eor;

        return(inch);
}

void PutDebugChar(char outch)
{
        Debug << "putDebugChar(" << outch << ")\n" << eor;
        while((zsreadreg(GDB_CTL, 0) & 0x04) != 0x04) ;
                zswritedata(GDB_DATA, outch);
}

/* this function takes the interrupt number and attempts to
   translate this number into a unix compatible signal value */
int ComputeSignal( int interruptVector )
{
  int sigval;

Debug << "ComputeSignal(" << interruptVector <<")\n" << eor;
if (interruptVector &128)       /* software interrupt */
        sigval = 5;
else
  switch (interruptVector - 128) {
    case 1 : sigval = 11; break; /* seg. violation      */
    case 2 : sigval = 2;  break; /* illegal instruction */
    case 3 : sigval = 3; break; /* privilege violation  */
    case 4 : sigval = 8; break; /* FP interrupt         */
    case 7 : sigval = 10;break; /* alignment            */
    case 8 : sigval = 8; break; /* FP interrupt         */
    case 9 : sigval = 11;break; /* data access          */
    case 10 : sigval = 11;break; /* data access         */
    default:
      sigval = 5;         /* breakpoint */
  }
  return (sigval);
}
