#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David Raila (raila@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * SS1Kernel.cc: SS1 kernel class implementation.
 */

#include "defNETWORKING.h"
#include "defDISTRIBUTED.h"
#include "ADDRESS_TRANSLATION.h"
#include "Assert.h"
#include "Byte.h"
#include "CPU.h"
#include "ESP.h"
#include "Interrupt.h"
#ifdef NETWORKING
#include "EthernetAdministrator.h"
#endif
#include "MemoryObjectView.h"
#include "FileSystemInterface.h"
#include "InputStream.h"
#include "MACHINE_CONFIGURATION.h"
#include "MemoryObjectContainer.h"
#include "NameServer.h"
#include "OutputStream.h"
#include "PAGE_TABLE_CONSTANTS.h"
#include "PROM.h"
#include "PhysicalMemoryRange.h"
#include "Process.h"
#include "SPARCCPU.h"
#include "SPARCInterrupts.h"
#ifdef NETWORKING
#include "SS1Am7990.h"
#include "Am7990MachineConstants.h"
#endif
#include "SS1Console.h"
#include "SS1Disk.h"
#include "SS1Kernel.h"
#include "SS1DMA.h"
#include "SS1devices.h"
#include "SS1TimeOfDay.h"
#include "SS1util.h"
#include "SS1SoftClock.h"
#include "SS1SoftInt.h"
#include "ThisProcess.h"
#include "TimeoutTimer.h"
#include "VirtualMemoryRange.h"
#include "PROCESSOR.h"

extern "C" {
//#include <limits.h>
#include <sys/exechdr.h>
//#include <sun4c/cpu.h>
#include <sys/openprom.h>
};

#define NONETWORKING 

#ifdef LINUXNET
#ifdef NONETWORKING
#undef NONETWORKING
#endif
#endif
#ifdef BSDNET
#ifndef NONETWORKING
#define NONETWORKING
#endif
#endif

#ifdef __GNUC__
#pragma implementation
#endif

#include "SS1Gdb.h"

SS1Kernel::SS1Kernel()
{
	Console() << "(Sun SPARCstation Version)\n" << eor;

	SS1PROM::getHWInfo();

	Console() << "CPU: " ;
	extern int cpu;
	extern int vac_size;
	extern int vac_linesize;
	extern int vac_hwflush;
	switch (cpu) {
		case CPU_SUN4C_75:      /* SS-2 */
			Console() << "SUN4C_75: SS2";
			break;
		case CPU_SUN4C_50:      /* IPX */
			Console() << "SUN4C_50: IPX";
			break;
		case CPU_SUN4C_25:      /* ELC */
			Console() << "SUN4C_25: ELC";
			break;
		case CPU_SUN4C_60:      /* SS1 */
			Console() << "SUN4C_60: SS1";
			break;
		case CPU_SUN4C_40:      /* IPC */
			Console() << "SUN4C_40: IPC";
			break;
		case CPU_SUN4C_65:      /* SS1+ */
			Console() << "SUN4C_65: SS1+";
			break;
		case CPU_SUN4C_20:      /* SLC */
			Console() << "SUN4C_20: SLC";
			break;
		default:
			Console() << "Unknown CPU type!";
			Assert (NOTREACHED);
			break;
		}
	Console() << "\n";
	Console() << "vac_size: " << vac_size << "\n";
	Console() << "vac_linesize: " << vac_linesize << "\n";
	Console() << "vac_hwflush: " << vac_hwflush << "\n";

	_availableMemory = 0;
	_physicalMemorySize = 0;
}

void
SS1Kernel::buildRunningCPU()
{
	/*
	 * Build a CPU object for this processor and install interrupts.
	 */
	CPU * current = new SPARCCPU( PROCESSOR::cpuID(), _initialScheduler );
	current->setName( "#CPU" );
	current->setIdleContainer(_initialScheduler);
	current->installInterrupts();
}

PhysicalMemoryRange *
SS1Kernel::physicalMemoryMap()
{
	Debug << "SS1Kernel::physicalMemoryMap()\n" << eor;

	/*
	 * Get the available physical and virtual memory lists from the
	 * EEPROM
	 */
	unsigned physMemory = 0;
	struct memlist *phys_available = 0;
	struct memlist *virt_available = 0;
	struct memlist *phys_total = 0;
	SS1PROM::setmemlists(phys_available, virt_available, phys_total);


	/*
	 * Find out how much memory we have on board to print
	 * out a nice message.
	 */
	struct memlist *chunk = phys_total;
	while (chunk)	{
		physMemory += chunk->size;
		chunk = chunk->next;
		}

	/*
	 * Configure the Physical memory map, also find the amount
	 * of memory available.
	 *
	 * The physicalMemoryMap needs to be laid out with the following
	 * separate pieces on the SPARC:
	 *	1. Kernel
	 *	2. Any other physical memory chunks not taken by the PROM
	 */
	chunk = phys_available;
	Debug << "Starting physical memory map: " << asAddress((char*)chunk->address)
		<< " size: " << asHex(chunk->size) << "\n" << eor;
	PhysicalMemoryRange * _physicalMemoryMap = new 
		PhysicalMemoryRange( (char *)chunk->address,
		chunk->size/PhysicalPageSize, Log2PhysicalPageSize, "#Memory (RAM)" );
	_availableMemory += chunk->size;
	chunk = chunk->next;
	while (chunk)	{
		Debug << "Adding to physical memory map: " 
			<< asAddress((char*)chunk->address) << " size: " 
			<< asHex(chunk->size) << "\n" << eor;
			_physicalMemoryMap->add( 
				new PhysicalMemoryRange ((char*)chunk->address,
				chunk->size/PhysicalPageSize, 
				Log2PhysicalPageSize ));
			_availableMemory += chunk->size;
			chunk = chunk->next;
		}

	_physicalMemorySize = physMemory;

	Console() << "Total memory : " << physMemory 
		<< " (" << asHex(physMemory) << ")\n" << eor;
	Console() << "Available memory : " << _availableMemory 
		<< " (" << asHex(_availableMemory) << ")\n" << eor;

	Debug << "Exiting SS1Kernel::physicalMemoryMap()\n" << eor;
	return _physicalMemoryMap;
}

const unsigned 
SS1Kernel::kernelHeapSize()
{
	return (const unsigned) ((_physicalMemorySize * 8) / 10);
}

const unsigned
SS1Kernel::physicalMemorySize()
{
	return (const unsigned) _physicalMemorySize;
}

void
SS1Kernel::mountNonRootFileSystems(){
	_fileSystemInterface->mount( "!6/", "usr", 0 );
// for ss2t1 ss2t2
	_fileSystemInterface->mount( "!7/", "mnt/local", 0 );
}

void SS1Kernel::initializeBackingStore()
{
	MemoryObjectStar disk = rootDisk();
	Assert( disk != 0 );
	int status = 0;
	MemoryObjectContainerStar rootContainer =
		disk->asA( MemoryObjectContainerClass, status );
	Assert( status == 0 );
	Assert( rootContainer != 0 );
	MemoryObjectStar mo = rootContainer->open( 6, status, O_NCACHE );
	Assert( status == 0 );
	Assert( mo != 0 );
	_backingStoreSource = mo->asA( MemoryObjectContainerClass, status );
	Debug << "backingStoreSource: " << _backingStoreSource << "\n" << eor;
	Assert( _backingStoreSource != 0 );
	_backingStoreSource->setName( "backingStoreSource" );
}

int 
SS1Kernel::isObjectFile( PersistentMemoryObject * file )
{
	Debug << this << "::isObjectFile( file: " << file 
		<< ")\n" << eor;

	Assert ( file != 0 );
	unsigned unitSize = file->unitSize();
	exec *hdr = new exec[unitSize/sizeof(exec) + 1];
	
	/* Read the a.out header into headerBuf */
	int i = file->read(0, 1, (char*)hdr, unitSize);
	if (i != 1) {
		Console() << this << "::loadObjectFile(): can't read first \
			block of file!\n" << eor;
		delete hdr;
		return 0;
		}
	if (hdr->a_dynamic) {
		Console() << this << "::loadObjectFile(): Dynamic file\n" << eor;
		delete hdr;
		return 0;
		}
	if (hdr->a_machtype != M_SPARC) {
		Console() << this << "::loadObjectFile(): Not a SPARC executable\n" << eor;
		delete hdr;
		return 0;
		}
	if (hdr->a_magic != ZMAGIC) {
		Console() << this << "::loadObjectFile(): File not ZMAGIC\n" << eor;
		delete hdr;
		return 0;
		}
	if ( (hdr->a_trsize + hdr->a_drsize) != 0 ) {
		Console() << this << "::loadObjectFile(): Bad trsize +
		drsize\n" << eor;
		delete hdr;
		return 0;
		}

	Assert (! hdr->a_dynamic);
	Assert (hdr->a_machtype == M_SPARC);
	Assert (hdr->a_magic == ZMAGIC);
	Assert (hdr->a_trsize + hdr->a_drsize == 0);

	delete hdr;
	return 1;
}

const char *
SS1Kernel::loadObjectFile( PersistentMemoryObject * file,
				Domain * newDomain )
{
	Debug << this << "::loadObjectFile( file: " << file 
		<< ", domain: " << newDomain << ")\n" << eor;
	
	Assert ( file != 0 );
	Assert ( newDomain != 0 );
	unsigned unitSize = file->unitSize();
	exec *hdr = new exec[unitSize/sizeof(exec) + 1];
	
	/* Read the a.out header into headerBuf */
	int i = file->read(0, 1, (char*)hdr, unitSize);
	if (i != 1) {
		Console() << this << "::loadObjectFile(): can't read first \
			block of file!\n" << eor;
		delete hdr;
		return 0;
		}
	if (hdr->a_dynamic) {
		Console() << this << "::loadObjectFile(): Dynamic file\n" << eor;
		delete hdr;
		return 0;
		}
	if (hdr->a_machtype != M_SPARC) {
		Console() << this << "::loadObjectFile(): Not a SPARC executable\n" << eor;
		delete hdr;
		return 0;
		}
	if (hdr->a_magic != ZMAGIC) {
		Console() << this << "::loadObjectFile(): File not ZMAGIC\n" << eor;
		delete hdr;
		return 0;
		}
	if ( (hdr->a_trsize + hdr->a_drsize) != 0 ) {
		Console() << this << "::loadObjectFile(): Bad trsize +
		drsize\n" << eor;
		delete hdr;
		return 0;
		}

	Assert (! hdr->a_dynamic);
	Assert (hdr->a_machtype == M_SPARC);
	Assert (hdr->a_magic == ZMAGIC);
	Assert (hdr->a_trsize + hdr->a_drsize == 0);
		/*
		 * See sys/exec.h for Maigc numbers, man a.out too for
		 * file format
		 */

	Debug << "building text section: size " << asHex(hdr->a_text) 
		<< "\n"  << eor;
	unsigned textSize = hdr->a_text ;
	char *textAddr = (char *) (hdr->a_entry & ~(PhysicalPageSize -1));
	unsigned textUnit = 0;	// Where it starts in the file
	unsigned units = file->offsetToUnit(textSize+unitSize-1);
	PersistentMemoryObjectStar mo = 0;
	mo = new NormalMemoryObjectView
		( file, units, file->log2UnitSize(), textUnit );
	mo->setName( ".text" ); 
	const char *where = newDomain->add( mo, textAddr, ReadOnly );
	Assert (where == textAddr);
	// newDomain->makeAddressable(where, units << file->log2UnitSize());
	Debug << "building data section: \n" << eor;
	unsigned dataSize = hdr->a_data ;
	unsigned bssSize = hdr->a_bss;
	char *dataAddr = pageAlign(textAddr + textSize);
	unsigned dataUnit = file->offsetToUnit(hdr->a_text + unitSize - 1);
	units = file->offsetToUnit(dataSize+bssSize+unitSize-1);
	unsigned dataOffset = dataUnit << file->log2UnitSize();
	MemoryObjectViewStar view = new NormalMemoryObjectView
		( file, dataSize, 0, dataOffset );
	mo = TheKernel->allocateMemoryObjectForBackingStore();
	Assert (mo != 0);
	mo->setNumberOfUnits( units );
	mo->setName(".bss");
	mo->buildCache();
	mo->cache()->setName(".bss");
	// mo->setDebugMask(63);
	// mo->setDebugMessages(32767);
	// mo->cache()->setDebugMask(8);
	// mo->cache()->setDebugMessages(32767);
	const char * source = TheKernel->domain()->add( view, NoAccess );
	const char * dest = TheKernel->domain()->add( mo, NoAccess );
	ByteCopy( ( char * ) source, ( char * ) dest, dataSize );
	TheKernel->domain()->remove( view, source );
	TheKernel->domain()->remove( mo, dest );
	where = newDomain->add( mo, dataAddr, ReadWrite);

	const char *entry = (char*)hdr->a_entry;

	delete hdr;
	return entry;
}


/*
 * Setup machine dependent devices and device managers.
 */
void
SS1Kernel::basicInitializeDevicesAndManagers()
{

	Debug << "SS1Kernel::basicInitializeDevicesAndManagers()\n" << eor;
	Assert( _initialScheduler != 0 );

//	*INTREG = 0;	// All interrupts off/disabled
//	*INTREG = 0x01;	// Enable, but don't accept any

	/*
	 * Now that interrupts are available, set up the Console
	 * for interrupts.
	 */
	Debug << "Enabling Console interrupts\n" << eor;
	((SS1Console*)ConsoleOutputStream)->setInterruptMode();

	/*
	 * Start the Timeout timer & Periodic timer helper processes.
	 *
	 */
	Debug << "Build Timeout Manager process\n" << eor;
	extern void TimeoutTimerEntry( int );
	Process *process = new SystemProcess( (PFV) TimeoutTimerEntry,
		domain(), normal, 0 );
	Debug << "Timeout manager process " << process << "\n" << eor;
	Assert( process != 0 );
	process->setName( "#TimeoutTimerManager" );
	process->ready();
	thisProcess()->relinquishProcessor();

	Debug << "Build Periodic Manager process\n" << eor;
	extern void PeriodicTimerEntry( int );
	process = new SystemProcess( (PFV) PeriodicTimerEntry,
		domain(), normal, 0 );
	Debug << "Periodic manager process " << process << "\n" << eor;
	Assert( process != 0 );
	process->setName( "#PeriodicTimerManager" );
	process->ready();
	thisProcess()->relinquishProcessor();

#ifndef NONETWORKING
#ifdef ETHERNET
	/*
	 * Start up ethernet
	 */
	extern void SS1EthernetConfig();
	Debug << "Build Ethernet\n" << eor;
	SS1EthernetConfig();
#endif
#endif

	*INTREG = 0x01;
	*INTREG |= 0x20;
	*INTREG |= 0x80;

	/*
	 * Start up the disk.
	 */
	Debug << "SCSI Disk Initialization\n" << eor;

	DMAController * dmaController = new SS1DMAController(DMA);
	_SCSIController = new ESP((unsigned char *)SCSI,4,dmaController);
	extern ESP * esp0;
	esp0 = _SCSIController;	// For the Interrupt
	_SCSIController->configure(SCSIBUS_ID); 

	unsigned whichDisk = 3 ;
	Console() << "SS1Kernel: using SCSI unit " <<  whichDisk << "\n" << eor;
	theDisk = new SS1Disk( _SCSIController, whichDisk);
	// theDisk->diskScheduler()->startDaemon();

        /*
         * Build the time of day clock
         */

	theSystemClock = new SS1TimeOfDay();
        StandardNameServer->bind( "timeOfDay", theSystemClock );

	/*
	 * Now install software interrupt manager
	 */

	TheSoftwareInterruptManager = new SS1SoftwareInterruptManager();

	/*
	 * Now install software clock manager
	 */

	TheSoftwareClockManager = new SS1SoftwareClockManager();

}

void
SS1Kernel::basicInitializeNetworkProcesses()
{
#ifndef NONETWORKING
#ifdef NETWORKING
	// Start up the EthernetDriver and EthernetAdministrator.	// Amit
	Debug << "SS1Kernel:\t\tstarting EthernetDriver\n" << eor;

	//
	// Start up ethernet
	//

	//  Start up the Administrator.
	Debug << "SS1Kernel:\t\tstarting EthernetAdministrator\n" << eor;
	_ethernetAdministrator = new EthernetAdministrator(
					"ethernetAdministrator", 0x3f);
	Assert(_ethernetAdministrator != 0);

#	if defined(NETWORK_TEST)
		_ethernetDriver = new StubEthernetDriver();
		Debug << "\t\t\t\tStarted *STUB* EthernetDriver\n" << eor;
#	else
		_ethernetDriver = new SS1Am7990(LE0_RAP, LE0_RDP);
#	endif /* NETWORK_TEST */

	Assert(_ethernetDriver != 0);
	_ethernetDriver->setName("ethernetDriver");
	StandardNameServer->bind("ethernetDriver", _ethernetDriver);

	Debug << "SS1Kernel: _ethernetDriver = " << asAddress(_ethernetDriver)
	      << ", _ethernetAdministrator = "
	      << asAddress(_ethernetAdministrator) << '\n' << eor;

	_ethernetDriver->ready(_ethernetAdministrator);
	_ethernetAdministrator->ready(_ethernetDriver);

	Debug << "SS1Kernel: driver and administrator are ready...\n" << eor;

#ifdef LINUXNET
	extern int net_init();
	net_init();
#endif

#endif NETWORKING
#endif NONETWORKING
}

void
SS1Kernel::shutdown()
{
	// *INTREG = 0;    // All interrupts off/disabled
}

int
SS1Kernel::tryCommand( int argc, char *argv[], int partnumber)
{
	extern debugApplications;
	int result = 1;
	unsigned debugMask;
	switch (partnumber) {
		case 444:
			debugMask = 0x1;
			if (debugApplications & debugMask)
				debugApplications &= ~debugMask;
			else
				debugApplications |= debugMask;
			Console() << this << ": debugApplications = " <<
				asHex(debugApplications) << "\n" << eor;
			Console() << this << ": Stop Application in _start\n" 
				<< eor;
			break;
		case 445:
			debugMask = 0x2;
			if (debugApplications & debugMask)
				debugApplications &= ~debugMask;
			else
				debugApplications |= debugMask;
			Console() << this << ": debugApplications = " <<
				asHex(debugApplications) << "\n" << eor;
			Console() << this << ": Stop Application before _main\n" 
				<< eor;
			break;
		case 446:
			debugMask = 0x4;
			if (debugApplications & debugMask)
				debugApplications &= ~debugMask;
			else
				debugApplications |= debugMask;
			Console() << this << ": debugApplications = " <<
				asHex(debugApplications) << "\n" << eor;
			Console() << this << ": Stop Application after _main\n" 
				<< eor;
			break;
		case 500:
			Assert (NOTREACHED);
			break;
		case 222:
#ifndef NONETWORKING
#ifdef NETWORKING
			extern SS1Am7990 * le0;
			if (le0 == 0){
				Console() << "le0 == 0!\n" << eor;
				}
			else	{
				le0->printInfo();
				}
#else
			Console() << "Networking is disabled\n" << eor;
#endif
#endif
			break;
  		case 7000:
#ifdef DISTRIBUTED
                        extern void startServer();
                        startServer();
#endif DISTRIBUTED
                        break;
		case 7001:
#ifdef DISTRIBUTED
                        extern void startClient();
                        startClient();
#endif DISTRIBUTED
                        break;
		case 8000:
			extern void SS1Benchmarks(int argc, char *argv[]);
			SS1Benchmarks(argc, argv);
			break;
		default:
			result = Kernel::tryCommand(argc, argv, partnumber);
			break;
		}
	return result;
}

DiskRef
SS1Kernel::rootDisk()
{
	return(theDisk);
}

#include "ClassInstaller.h"
DEFINECLASS("SS1Kernel",SS1Kernel,Kernel,0,0);
