#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1994
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SS1SoftInt.cc
 */

#include "Assert.h"
#include "Interrupt.h"
#include "PROCESSOR.h"
#include "SS1SoftInt.h"
#include "SS1devices.h"
#include "ThisCPU.h"
#include "ThisProcess.h"

#ifdef __GNUC__
#pragma implementation
#endif

const IntStorage SS1SoftwareInterruptLevel = 6;

inline 
int
SS1SoftwareInterruptManager::softwareInterrupt()
{
	return SoftwareInterruptManager::softwareInterrupt();
}

class SoftCallInterrupt : public Interrupt {
protected:
        virtual void basicRaise( char * info );
public:
        SoftCallInterrupt() : Interrupt() {}
};
 
#define DEBUG_SOFTINT
#ifdef DEBUG_SOFTINT
int softint_nest = 0;
int debug_softint = 1;
Process * softint_process[32];
#endif 

void SoftCallInterrupt::basicRaise(char * info)
{
	IntStorage intState ; PROCESSOR::disableInterrupts(intState);
	TIMESLICETIMER * timesliceTimer = thisCPU()->timesliceTimer();
	unsigned int resid = timesliceTimer->stop();
#ifdef DEBUG_SOFTINT
	int my_nesting;
	my_nesting = softint_nest;
        softint_process[my_nesting] = thisProcess();
        softint_nest++;
        if (my_nesting != 0) {
		if (debug_softint)
        	        Console() 
			<< "SoftCallInterrupt enter " << my_nesting 
			<< "\n" << eor;
        }
#endif // DEBUG_SOFTINT
	*INTREG = *INTREG & ~INTREG_SOFT_INT1;
	PROCESSOR::restoreInterrupts( SS1SoftwareInterruptLevel );
	( (SS1SoftwareInterruptManager *) TheSoftwareInterruptManager )
		->softwareInterrupt();
#ifdef DEBUG_SOFTINT
	PROCESSOR::disableInterrupts();
	if (my_nesting != 0) {
		if (debug_softint)
			Console()
			<< "SoftCallInterrupt exit " << my_nesting 
			<< ", softint_nest = " << softint_nest
			<< "\n" << eor;
	}
	softint_process[my_nesting] = 0;
	softint_nest--;
#endif // DEBUG_SOFTINT
	PROCESSOR::restoreInterrupts( intState );
	if (resid != 0) 
		timesliceTimer->start(resid);
}

SS1SoftwareInterruptManager::SS1SoftwareInterruptManager()
	: SoftwareInterruptManager()
{
	Assert( TheSoftwareInterruptManager == 0 );
        Interrupt * interrupt = new SoftCallInterrupt();
        thisCPU()->setInterrupt(17, interrupt);
}

void
SS1SoftwareInterruptManager::enableSoftwareInterrupt()
{
	Assert( !PROCESSOR::interruptsEnabled() );
	*INTREG = *INTREG | INTREG_SOFT_INT1;
}
