#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "FreeRunningTimer.h"
#include "FreeRunningTimerStar.h"
#include "Raid.h"
#include "SS1TimeOfDay.h"
#include "SystemTime.h"
#include "Time.h"
#include "UnixTime.h"

#ifdef __GNUC__
#pragma implementation
#endif

static int daysInMonth[2][12] =
{ { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 },
  { 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 } };
static int daysInYear[2] =
  { 365, 366 };
static TimeValue carryOver(4294, 967296);
  
SS1TimeOfDay::SS1TimeOfDay()
{
	initDebug();
	Raids( RaidConstructor ) << this << "::SS1TimeOfDay()" << eor;
	SS1Time ss1Time;
	getHardwareTime( ss1Time );
	Console() << "SS1TimeOfDay " 
		<< ss1Time.month << "/" 
		<< ss1Time.date << "/"
		<< ss1Time.year << " "
		<< ss1Time.hour << ":"
		<< ss1Time.min << ":"
		<< ss1Time.sec << " GMT\n" << eor;
	unsigned long unixTime32 = unixTime( ss1Time );
	unsigned int systemTimerTime = SystemTimer->time();
	unsigned int sec = systemTimerTime / 1000000;
	unsigned int usec = systemTimerTime % 1000000;
	TimeValue t(sec,usec);
	_lastTime.setSec(unixTime32);
	_lastTime = _lastTime - t;
	Console() << "SS1TimeOfDay Unix Time " << unixTime32 
		<< "\n" << eor;
	UnixTime ut;
	getUnixTime(ut);
	Console() << "SS1TimeOfDay Unix " << ut.seconds() << " secs "
		<< ut.useconds() << " usecs\n" << eor; 	
}

void
SS1TimeOfDay::systemTimerOverflow()
{
	_lastTime += carryOver;
}

void
SS1TimeOfDay::getUnixTime( UnixTime & uTime )
{
	TimeValue lt = _lastTime;
	unsigned int systemTimerTime = SystemTimer->time();
	unsigned int sec = systemTimerTime / 1000000;
	unsigned int usec = systemTimerTime % 1000000;
	TimeValue t(sec, usec);
	lt = lt + t;
	uTime.setSeconds(lt.sec());
	uTime.setUSeconds(lt.usec());
	uTime.setMinuteswest(0);
	uTime.setDsttime(0);
}

void
SS1TimeOfDay::getTime( SystemTime & sysTime )
{
	UnixTime uTime;
	getUnixTime(uTime);
	int64 result = (uTime.seconds() + SYSTEM_SEC_TO_UNIX_SEC) *
		       SYSTEM_TO_SECOND + uTime.useconds() * 10;
	sysTime.setTime(result);
	sysTime.setTimeZoneOffset(0);
	sysTime.setDst(0);
}

char SS1TimeOfDay::regValue( volatile unsigned char * const reg )
{
        unsigned char val = * reg;
        return (val >> 4) * 10 + (val & 0x0f);
}

void 
SS1TimeOfDay::getHardwareTime( SS1Time & time )
{
	Raids( RaidProtected ) << this << "::getHardwareTime()" << eor;

	time.sec = regValue( reg_sec );
	time.min = regValue( reg_min );
	time.hour = regValue( reg_hour );
	time.day = regValue( reg_day );
	time.date = regValue( reg_date );
	time.month = regValue( reg_month );
	time.year = regValue( reg_year );

	Assert( time.sec >= 0 && time.sec <= 59 );
	Assert( time.min >= 0 && time.min <= 59 );
	Assert( time.hour >= 0 && time.hour <= 23 );
	Assert( time.day >= 1 && time.day <= 7 );
	Assert( time.date >= 1 && time.date <= 31 );
	Assert( time.month >= 1 && time.month <= 12 );
	Assert( time.year >= 0 && time.year <= 99);

	time.year += 1968;
}

unsigned long
SS1TimeOfDay::unixTime( SS1Time & time )
{
	int month = time.month;
	int dayOfYear = time.date - 1;
	int leapYear = LeapYear( time.year ) ? 1 : 0;
	while( --month ) dayOfYear += daysInMonth[ leapYear ][ month - 1 ];
	int daysSince1970 = dayOfYear;
	for ( int i = 1970; i < time.year; i++ ) {
		daysSince1970 += daysInYear[ LeapYear( i ) ? 1 : 0 ];
	}
	unsigned long secsSince1970 =
		daysSince1970 * ( 24 * 60 * 60 ) +
		time.hour * ( 60 * 60 ) +
		time.min * 60 +
 		time.sec;
	return secsSince1970;
}

#include "ClassInstaller.h"
DEFINECLASS("SS1TimeOfDay",SS1TimeOfDay,TimeOfDay,0,0);
