#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SS1TimesliceTimer.cc
 */

#include "Assert.h"
#include "OutputStream.h"
#include "SS1Timer.h"
#include "SS1TimesliceTimer.h"


#ifdef __GNUC__
#pragma implementation
#endif

TimeSliceInterrupt TSliceInterrupt;

const unsigned microsecondsPerTick = TimerManageruSecPerInterrupt;

unsigned int
SS1TimesliceTimer::start( unsigned int microseconds )
{
	Debug << "SS1TimesliceTimer::start( " << microseconds << " )\n";

	unsigned int ticks = (microseconds + (microsecondsPerTick - 1)) /
						microsecondsPerTick;

	if (ticks != 0) {
		lock->acquire();

		running = 1;
		expireTime = tickTime + ticks;

		lock->release();
	}

	return(ticks * microsecondsPerTick);
}

void
SS1TimesliceTimer::set( unsigned int )
{
}

unsigned int
SS1TimesliceTimer::stop()
{
	Debug << "SS1TimesliceTimer::stop()\n" << eor;

	unsigned int resid;

	lock->acquire();

	if (running) {
		running = 0;
		resid = expireTime - tickTime;
	}
	else
		resid = 0;

	lock->release();

	return(resid * microsecondsPerTick);

}

/*
 * residual - Returns the number of microseconds left before interrupt.
 */

unsigned int
SS1TimesliceTimer::residual()
{
	Debug <<  "SS1TimesliceTimer::residual() entering \n" << eor;

	unsigned int resid;

	lock->acquire();

	if (running)
		resid = expireTime - tickTime;
	else
		resid = 0;

	lock->release();

	return(resid * microsecondsPerTick);
}

/*
 * idle - returns non-zero iff the timer is not running.
 */

int
SS1TimesliceTimer::idle()
{
	Debug <<  "SS1TimesliceTimer::idle()\n" << eor;
	return ( !running );
}
