
#define BOOTSTACK_SIZE 0x2000
#define NW 8
#define PROBE
#define REGSIZE 76

#include "MACHINE_CONFIGURATION.h"
#include "MMU_CONSTANTS.h"
#include "SS1devices.h"
#include "SPARCasmutil.h"
#include "SPARCasmdefs.h"
#include <sys/psw.h>
#include <sys/reg.h>
#include <sys/asm_linkage.h>
#include <sys/mmu.h>
#include <sys/pte.h>
#include <sys/auxio.h>
#include <sys/clock.h>
#include <sys/eeprom.h>
#include <sys/intreg.h>
#include <sys/memerr.h>
#include <sys/machparam.h>

#define OBIO_KBD_MOUSE_ADDR     0xF0000000
#define OBIO_SERIAL_ADDR        0xF1000000
#define OBIO_CLOCK_ADDR         0xF2000000
#define OBIO_TIMER_ADDR         OBIO_COUNTER_ADDR
#define OBIO_EDP_PRIVATE_ADDR   0xF7000000
#define OBIO_ECC_ADDR           0xF7100000
#define OBIO_FLOPPY_ADDR        0xF7200000
#define OBIO_AUDIO_ISDN_ADDR    0xF7201000
#define OBIO_VME_ADDR           0xF7F00000
#define OBIO_DMA_ADDR           0xF8400000
#define OBIO_SCSI_ADDR          0xF8800000
#define OBIO_LE0_ADDR           0xF8C00000



!*************************************************************************
!
!    Choices startup code 
!
!*************************************************************************

!/**************************************************************************
!
! Trap table setup.
! Trap 0 is at _choicesstart, which branches to _bootStart.
!
!*************************************************************************
!*
!* Trap vector table.
!* This must be the first text in the boot image.
!*
!* When a trap is taken, we vector to KBASE+TT*16 and we have
!* the following state:
!*      2) traps are disabled
!*      3) the previous state of PSR_S is in PSR_PS
!*      4) the CWP has been bumped into a trap window
!*      5) the previous pc and npc is in %l1 and %l2 respectively.
!*
!* Registers:
!*      %l0 - trap number
!*      %l1 - trapped pc
!*      %l2 - trapped npc
!*
!*
!*      %l3 - wim (sys_trap only)
!*      %l4 - system trap number (sys_trap only)
!*      %l6 - number of windows - 1
!*      %l7 - stack pointer (interrupts and system calls)
!*
!* Note: UNIX receives control at vector 0 (trap)
!*/

#define LOCAL_TRAP(H) \
        mov %psr,%l0; b (H); nop; nop;

#define EMU_TRAP(N) \
	mov %psr,%l0; mov (N),%l4; b emu_trap; mov (NW-1), %l6;

#define RES_TRAP(N) \
	mov %psr,%l0; mov (N),%l4; b res_trap; mov (NW-1), %l6;

#define WIN_TRAP(T) \
	mov %psr,%l0; rd %wim,%l3; b (T); mov (NW-1),%l6;

.section ".text"
	.align 8
	.globl	_choicesstart
	.type _choicesstart,#function
	.proc 020
_choicesstart:
_traptable:
	LOCAL_TRAP(_bootStart);		!00
	RES_TRAP(0x1);			!01
	EMU_TRAP(0x2);			!02
	EMU_TRAP(0x3);			!03
	RES_TRAP(0x4);			!04
	WIN_TRAP(vac_w_overflow);		!05
	WIN_TRAP(vac_w_underflow);		!06
	RES_TRAP(0x7);			!07
	EMU_TRAP(0x8);			!08
	RES_TRAP(0x9);			!09
	EMU_TRAP(0xa);			!10
	EMU_TRAP(0xb);			!11
EMU_TRAP(0xc);	EMU_TRAP(0xd);	EMU_TRAP(0xe);	EMU_TRAP(0xf);	! c - f
EMU_TRAP(0x10);	RES_TRAP(0x11);	RES_TRAP(0x12);	RES_TRAP(0x13);	! 10 - 13
RES_TRAP(0x14);	RES_TRAP(0x15);	RES_TRAP(0x16);	RES_TRAP(0x17);	! 14 - 17
RES_TRAP(0x18);	RES_TRAP(0x19);	RES_TRAP(0x1a);	RES_TRAP(0x1b);	! 18 - 1b
RES_TRAP(0x1c);	RES_TRAP(0x1d);	RES_TRAP(0x1e);	RES_TRAP(0x1f);	! 1c - 1f
EMU_TRAP(0x20);	EMU_TRAP(0x21);	EMU_TRAP(0x22);	EMU_TRAP(0x23);	! 20 - 23
EMU_TRAP(0x24);	EMU_TRAP(0x25);	EMU_TRAP(0x26);	EMU_TRAP(0x27);	! 24 - 27
EMU_TRAP(0x28);	EMU_TRAP(0x29);	EMU_TRAP(0x2a);	EMU_TRAP(0x2b);	! 28 - 2b
EMU_TRAP(0x2c);	EMU_TRAP(0x2d);	EMU_TRAP(0x2e);	EMU_TRAP(0x2f);	! 2c - 2f
EMU_TRAP(0x30);	EMU_TRAP(0x31);	EMU_TRAP(0x32);	EMU_TRAP(0x33);	! 30 - 33
EMU_TRAP(0x34);	EMU_TRAP(0x35);	EMU_TRAP(0x36);	EMU_TRAP(0x37);	! 34 - 37
EMU_TRAP(0x38);	EMU_TRAP(0x39);	EMU_TRAP(0x3a);	EMU_TRAP(0x3b);	! 38 - 3b
EMU_TRAP(0x3c);	EMU_TRAP(0x3d);	EMU_TRAP(0x3e);	EMU_TRAP(0x3f);	! 3c - 3f
EMU_TRAP(0x40);	EMU_TRAP(0x41);	EMU_TRAP(0x42);	EMU_TRAP(0x43);	! 40 - 43
EMU_TRAP(0x44);	EMU_TRAP(0x45);	EMU_TRAP(0x46);	EMU_TRAP(0x47);	! 44 - 47
EMU_TRAP(0x48);	EMU_TRAP(0x49);	EMU_TRAP(0x4a);	EMU_TRAP(0x4b);	! 48 - 4b
EMU_TRAP(0x4c);	EMU_TRAP(0x4d);	EMU_TRAP(0x4e);	EMU_TRAP(0x4f);	! 4c - 4f
EMU_TRAP(0x50);	EMU_TRAP(0x51);	EMU_TRAP(0x52);	EMU_TRAP(0x53);	! 50 - 53
EMU_TRAP(0x54);	EMU_TRAP(0x55);	EMU_TRAP(0x56);	EMU_TRAP(0x57);	! 54 - 57
EMU_TRAP(0x58);	EMU_TRAP(0x59);	EMU_TRAP(0x5a);	EMU_TRAP(0x5b);	! 58 - 5b
EMU_TRAP(0x5c);	EMU_TRAP(0x5d);	EMU_TRAP(0x5e);	EMU_TRAP(0x5f);	! 5c - 5f
EMU_TRAP(0x60);	EMU_TRAP(0x61);	EMU_TRAP(0x62);	EMU_TRAP(0x63);	! 60 - 63
EMU_TRAP(0x64);	EMU_TRAP(0x65);	EMU_TRAP(0x66);	EMU_TRAP(0x67);	! 64 - 67
EMU_TRAP(0x68);	EMU_TRAP(0x69);	EMU_TRAP(0x6a);	EMU_TRAP(0x6b);	! 68 - 6b
EMU_TRAP(0x6c);	EMU_TRAP(0x6d);	EMU_TRAP(0x6e);	EMU_TRAP(0x6f);	! 6c - 6f
EMU_TRAP(0x70);	EMU_TRAP(0x71);	EMU_TRAP(0x72);	EMU_TRAP(0x73);	! 70 - 73
EMU_TRAP(0x74);	EMU_TRAP(0x75);	EMU_TRAP(0x76);	EMU_TRAP(0x77);	! 74 - 77
EMU_TRAP(0x78);	EMU_TRAP(0x79);	EMU_TRAP(0x7a);	EMU_TRAP(0x7b);	! 78 - 7b
EMU_TRAP(0x7c);	EMU_TRAP(0x7d);	EMU_TRAP(0x7e);	EMU_TRAP(0x7f);	! 7c - 7f

EMU_TRAP(0x80);	RES_TRAP(0x81);	EMU_TRAP(0x82);	EMU_TRAP(0x83);	! 80 - 83
EMU_TRAP(0x84); EMU_TRAP(0x85);	EMU_TRAP(0x86);	EMU_TRAP(0x87);	! 84 - 87
EMU_TRAP(0x88);	EMU_TRAP(0x89);	EMU_TRAP(0x8a);	EMU_TRAP(0x8b);	! 88 - 8b
EMU_TRAP(0x8c);	EMU_TRAP(0x8d);	EMU_TRAP(0x8e);	EMU_TRAP(0x8f);	! 8c - 8f
EMU_TRAP(0x90);	EMU_TRAP(0x91);	EMU_TRAP(0x92);	EMU_TRAP(0x93);	! 90 - 93
EMU_TRAP(0x94);	EMU_TRAP(0x95);	EMU_TRAP(0x96);	EMU_TRAP(0x97);	! 94 - 97
EMU_TRAP(0x98);	EMU_TRAP(0x99);	EMU_TRAP(0x9a);	EMU_TRAP(0x9b);	! 98 - 9b
EMU_TRAP(0x9c);	EMU_TRAP(0x9d);	EMU_TRAP(0x9e);	EMU_TRAP(0x9f);	! 9c - 9f
EMU_TRAP(0xa0);	EMU_TRAP(0xa1);	EMU_TRAP(0xa2);	EMU_TRAP(0xa3);	! a0 - a3
EMU_TRAP(0xa4);	EMU_TRAP(0xa5);	EMU_TRAP(0xa6);	EMU_TRAP(0xa7);	! a4 - a7
EMU_TRAP(0xa8);	EMU_TRAP(0xa9);	EMU_TRAP(0xaa);	EMU_TRAP(0xab);	! a8 - ab
EMU_TRAP(0xac);	EMU_TRAP(0xad);	EMU_TRAP(0xae);	EMU_TRAP(0xaf);	! ac - af
EMU_TRAP(0xb0);	EMU_TRAP(0xb1);	EMU_TRAP(0xb2);	EMU_TRAP(0xb3);	! b0 - b3
EMU_TRAP(0xb4);	EMU_TRAP(0xb5);	EMU_TRAP(0xb6);	EMU_TRAP(0xb7);	! b4 - b7
EMU_TRAP(0xb8);	EMU_TRAP(0xb9);	EMU_TRAP(0xba);	EMU_TRAP(0xbb);	! b8 - bb
EMU_TRAP(0xbc);	EMU_TRAP(0xbd);	EMU_TRAP(0xbe);	EMU_TRAP(0xbf);	! bc - bf
EMU_TRAP(0xc0);	EMU_TRAP(0xc1);	EMU_TRAP(0xc2);	EMU_TRAP(0xc3);	! c0 - c3
EMU_TRAP(0xc4);	EMU_TRAP(0xc5);	EMU_TRAP(0xc6);	EMU_TRAP(0xc7);	! c4 - c7
EMU_TRAP(0xc8);	EMU_TRAP(0xc9);	EMU_TRAP(0xca);	EMU_TRAP(0xcb);	! c8 - cb
EMU_TRAP(0xcc);	EMU_TRAP(0xcd);	EMU_TRAP(0xce);	EMU_TRAP(0xcf);	! cc - cf
EMU_TRAP(0xd0);	EMU_TRAP(0xd1);	EMU_TRAP(0xd2);	EMU_TRAP(0xd3);	! d0 - d3
EMU_TRAP(0xd4);	EMU_TRAP(0xd5);	EMU_TRAP(0xd6);	EMU_TRAP(0xd7);	! d4 - d7
EMU_TRAP(0xd8);	EMU_TRAP(0xd9);	EMU_TRAP(0xda);	EMU_TRAP(0xdb);	! d8 - db
EMU_TRAP(0xdc);	EMU_TRAP(0xdd);	EMU_TRAP(0xde);	EMU_TRAP(0xdf);	! dc - df
EMU_TRAP(0xe0);	EMU_TRAP(0xe1);	EMU_TRAP(0xe2);	EMU_TRAP(0xe3);	! e0 - e3
EMU_TRAP(0xe4);	EMU_TRAP(0xe5);	EMU_TRAP(0xe6);	EMU_TRAP(0xe7);	! e4 - e7
EMU_TRAP(0xe8);	EMU_TRAP(0xe9);	EMU_TRAP(0xea);	EMU_TRAP(0xeb);	! e8 - eb
EMU_TRAP(0xec);	EMU_TRAP(0xed);	EMU_TRAP(0xee);	EMU_TRAP(0xef);	! ec - ef
EMU_TRAP(0xf0);	EMU_TRAP(0xf1);	EMU_TRAP(0xf2);	EMU_TRAP(0xf3);	! f0 - f3
EMU_TRAP(0xf4);	EMU_TRAP(0xf5);	EMU_TRAP(0xf6);	EMU_TRAP(0xf7);	! f4 - f7
EMU_TRAP(0xf8);	EMU_TRAP(0xf9);	EMU_TRAP(0xfa);	EMU_TRAP(0xfb);	! f8 - fb
EMU_TRAP(0xfc);	EMU_TRAP(0xfd);	EMU_TRAP(0xfe);	EMU_TRAP(0xff);	! fc - ff

traptable_end:

vac_w_overflow:
	sethi   %hi(CACHE_TAGS+0x4050), %l5
	or      %l5, %lo(CACHE_TAGS+0x4050), %l5
	sta     %g0, [%l5]ASI_CTL
	b,a     w_overflow

vac_w_underflow:
	sethi   %hi(CACHE_TAGS+0x4060), %l5
	or      %l5, %lo(CACHE_TAGS+0x4060), %l5
	sta     %g0, [%l5]ASI_CTL
	b,a     w_underflow


.section ".data"
	.align 8
	.global nwindows
nwindows:
	.word   0
_stacktop:
	.skip BOOTSTACK_SIZE
	.globl _stack
_stack:
	.globl romp
romp:
	.word 0
	.globl _stackOVF
_stackOVF:
	.word 0

!*************************************************************************
!
! System trap handler.
!	1. Save globals and check for overflow condition, fix if necessary.
!	2. Enable traps and call SPARCCPU::hardwareInterruptassist()int with
!	the trap number as its argument.
!	3. Restore everything that we saved and disable traps again.
!	4. Return from trap.
!*
!* Registers:
!*      %l0 - %psr at trap
!*      %l1 - trapped pc (adjusted)
!*      %l2 - trapped npc (adjusted)
!*      %l4 - system trap number
!*      %l6 - number of windows - 1
!*
!
!*************************************************************************
.section ".text"
	.align 4
emu_trap:
	mov %l2, %l1
	add %l1, 4, %l2
res_trap:
				!
				! Work around bug in SS2 cache
				!
	mov     %tbr, %l5               ! get tbr
        sethi   %hi(vac_size), %l3     ! get size of cache
        ld      [%l3 + %lo(vac_size)], %l3
	cmp	%l3, %g0
	bnz	1f
	nop
	set	0x10000, %l3
1:
        sub     %l3, 1, %l3             ! turn cache size into bitmask
        and     %l3, %l5, %l5           ! get offset of vector within cache
                                        ! hardware folks say lsb's don't matter
        sethi   %hi(CACHE_TAGS), %l3    ! cheat, no low bits
        sta     %g0, [%l3 + %l5]ASI_CTL ! invalidate that cache line!
				!
				! End of bug workaround
				!
	rd	%wim, %l3		! get WIM
	andcc   %l0, PSR_PS, %g0        ! test pS
	bnz      trap_gotstack
	sub %fp, MINFRAME+REGSIZE, %l7
user_trap:
	sethi	%hi(SS1ThisProcessorContext), %l7
	ld	[%l7 + %lo(SS1ThisProcessorContext)], %l7
	ld	[%l7 + SPARCCONTEXT_KERNELSTACK_OFFSET], %l7	! kernel stack for this process
	sub %l7, MINFRAME+REGSIZE, %l7

trap_gotstack:
#ifdef PROBE
			!
			! Check stack alignment
			!
	btst	0x7, %l7
	bnz _stackAlign
	nop
			!
			! Check stack residency
			!
	add	%l7, MINFRAME, %l5
	lda 	[%l7]ASI_PM, %l5
	srl	%l5, 24, %l5
	or	%l5, 0x33, %l5		! turn on all necessary bits
	subcc	%l5, 0xf3, %g0		! Should be zero
	bnz	_stackPageFaultL5
	mov	%l7, %l5
					! Need Check end writable?
	add	%l7, MINFRAME, %l5
	and	%l5, 0xfff, %l5
	subcc   %l5, 0x1000-((SP-1)*4), %g0
	bl 1f
					! End on next page, check it
	add	%l7, MINFRAME+((SP-1)*4), %l5
	lda 	[%l5]ASI_PM, %l5
	srl	%l5, 24, %l5
	or	%l5, 0x33, %l5		! turn on all necessary bits
	subcc	%l5, 0xf3, %g0		! Should be zero
	bnz	_stackPageFaultL5
	add	%l7, MINFRAME+((SP-1)*4), %l5
1:
#endif

		!
		! Stack is OK to save globals and trap info
		!
	SAVE_GLOBALS(%l7 + MINFRAME)
	st	%fp, [%l7 + MINFRAME + SP*4] ! stack pointer
	st	%l0, [%l7 + MINFRAME + PSR*4] ! psr
	st	%l1, [%l7 + MINFRAME + PC*4] ! pc
		!
		! Check for overflow now
		!
	mov	0x01, %l5		! CWM = 0x01 << CWP
	sll	%l5, %l0, %l5
	andcc    %l5, %l3, %g0          ! delay slot, compare WIM and CWM
	bz _trap_call		! Windows OK, call higher level
	st      %l2, [%l7 + MINFRAME + nPC*4] ! npc

trap_overflow:
	!
	! Globals have been saved.
	! Process the overflow condition
	!
	srl     %l3, 1, %g1
	sll     %l3, %l6, %l6           ! %l6 == NW-1
	or      %l6, %g1, %g1
	save 	%g0,%g0,%g0
	wr 	%g0, %g1, %wim

#ifdef PROBE
	!
	! Check stack residency
	!
	lda 	[%sp]ASI_PM, %g5
	srl	%g5, 24, %g5
	or	%g5, 0x33, %g5		! turn on all necessary bits
	subcc	%g5, 0xf3, %g0		! Should be zero
	bnz	_stackPageFaultG5
	mov	%sp, %g5
		!
		! Check end writable if necessary
		!
	and     %sp, 0xfff, %g1
	subcc   %g1, 0x1000-(15*4), %g0
	bl 1f
	add	%sp, 15*4, %g5
	lda 	[%g5]ASI_PM, %g5
	srl	%g5, 24, %g5
	or	%g5, 0x33, %g5		! turn on all necessary bits
	subcc	%g5, 0xf3, %g0		! Should be zero
	bnz	_stackPageFaultG5
	add	%sp, (15*4), %g5
1:
#endif
	SAVE_WINDOW(%sp);
	restore

	.globl _trap_call
_trap_call:				 
	mov 	%l7, %sp		! Set up stack

#define INTLEVEL
#ifdef INTLEVEL
	!
	! Fiddle with interrupt level if necessary
	!
	or	%l4, 0x0f, %g1
	xorcc	%g1, 0x1f, %g0		! Check for trap 0x1x (interrupt)
	bnz	1f
	and	%l4, 0x0f, %g1		! Adjust interrupt level in %g1
	sll	%g1, 8, %g1
	or	%l0, %g1, %l0		! 
#else
	or %l0, PSR_PIL, %l0
#endif
	mov	%l0, %psr		! re-enable them if necessary

	! Done messing with interrupt level, make the call to HWInterruptAssist
1:		
	or	%l0, PSR_ET, %l0
	mov	%l0, %psr		! enable traps to call higher level code
	mov	%l7, %o1		! arg1= trap %sp
		mov	%i7, %l7
		mov	%l1, %i7
	call hardwareInterruptAssist__8SPARCCPUiPc,2
	mov 	%l4, %o0		! Set up arg0 = vector
		mov	%l7, %i7


	.globl sys_rtt
sys_rtt:
#ifdef PROBE
#endif
	ld      [%sp + MINFRAME + PSR*4], %l0 ! get saved psr
	andcc    %l0, PSR_PS, %g0             ! test pS for return to supervisor
	bnz     sr_sup
	mov	%psr, %g1		! Get CWP

sr_user:
        !
	! Return to user. Turn off traps using the current CWP (because
	! we are returning to user).
	and     %g1, PSR_CWP, %g1       ! insert current CWP in old psr
	andn    %l0, PSR_CWP, %l0
	or      %l0, %g1, %l0
	! nop; nop;                       ! psr delay
	b sr_samecwp
	mov     %l0, %psr               ! install old psr, disable traps

sr_sup:
	set	nwindows, %g5			! %g5 gets NW
#ifdef PROBE
#endif
	ld [%g5], %g5
  	ld      [%sp + MINFRAME + SP*4], %fp ! get sys sp

	xor	%g1, %l0, %g1		! test for CWP change
	andcc	%g1, PSR_CWP, %g0
	bz,a	sr_samecwp
	mov	%l0, %psr		! install old psr, disable traps

	!
	! The CWP will be changed. We must save sp and the ins
	! and recompute WIM. We know we need to restore the next
	! window in this case.
	!
	mov	%l0, %g3		! save old psr
	mov	%sp, %g4		! save sp, ins for new window
#ifdef PROBE
#endif
	std	%i0, [%sp +(8*4)]	! normal stack save area
	std	%i2, [%sp +(10*4)]
	std	%i4, [%sp +(12*4)]
	std	%i6, [%sp +(14*4)]
#ifdef PROBE
#endif
	mov	%g3, %psr		! old psr, disable traps, CWP, PSR_CC
	mov	0x4, %g1		! psr delay, compute mask for CWP + 2
	sll	%g1, %g3, %g1		! psr delay, wont work for NW == 32
	srl	%g1, %g5, %g2		! psr delay
	or	%g1, %g2, %g1
	wr	%g0, %g1, %wim		! install new wim
	mov	%g4, %sp		! reestablish sp
#ifdef PROBE
#endif
	ldd	[%sp + (8*4)], %i0	! reestablish ins
	ldd	[%sp + (10*4)], %i2
	ldd	[%sp + (12*4)], %i4
	ldd	[%sp + (14*4)], %i6
	restore				! restore return window
#ifdef PROBE
#endif
	RESTORE_WINDOW(%sp)
	b	sr_out
	save

sr_samecwp:
nop
nop
	!
	! There is no CWP change.
	! We must make sure that there is a window to return to.
	!
	mov	0x2, %g1		! compute mask for CWP + 1
	sll	%g1, %l0, %g1		! XXX wont work for NW == 32
	set nwindows, %g5
#ifdef PROBE
#endif
	ld [%g5], %g5
	srl	%g1, %g5, %g2		! %g5 == NW, from above
	or	%g1, %g2, %g1
	rd	%wim, %g2		! cmp with wim to check for underflow
	andcc	%g2, %g1, %g0
	bz	sr_out
	mov	%l0, %psr		! install old PSR_CC
	!
	! No window to return to. Restore it.
	!
	sll	%g2, 1, %g1		! compute new WIM = rol(WIM, 1, NW)
	sub %g5, 1, %g5
	srl	%g2, %g5, %g2
	or	%g1, %g2, %g1
	wr	%g0, %g1, %wim		! install it
	nop; nop; nop;			! wim delay
	restore				! get into window to be restored
#ifdef PROBE
#endif
	RESTORE_WINDOW(%sp);
	save %g0,%g0,%g0		! get back to original window
sr_out:
#ifdef PROBE
#endif
	RESTORE_GLOBALS(%sp + MINFRAME);
	ld	[%sp + MINFRAME + PC*4], %l1 ! delay slot, restore sys pc
	ld	[%sp + MINFRAME + nPC*4], %l2 ! sys npc
	jmpl %l1,%g0		! Return to or after trapper with one style.
	rett %l2		! Registers changed on trap entry to do it.
	nop

!*************************************************************************
!
!  THe first code to get executed
!
!*************************************************************************
_bootStart:
ba		_resetMachine
nop
	mov     %o0, %g7                ! save arg (romp) until bss is clear
	mov     %o1, %g6                ! save argc
	mov     %o2, %g5                ! save aargv
	set     PSR_S|PSR_PIL, %g1       ! setup psr, leave traps
	mov     %g1, %psr
	or      %g1, PSR_ET, %g1       ! setup psr, leave traps
	mov     %g1, %psr
	set     CONTEXT_REG, %g1        ! setup kernel context (0)
	stba    %g0, [%g1]ASI_CTL
	wr     %g0, 0x02, %wim              ! setup wim

/******************************************************
 * Intialize memory mappings by 1-1 mapping up to KSIZE
 ******************************************************/
	set	0xfffc0000, %l0		! segment extent mask
        mov 	%g0, %l1		! address pointer
        set     KSIZE, %l2		! ending address
        set     0x1000, %l3		! Page extent
	mov	%g0, %l4		! PMEG
	set	PG_V|PG_W|PG_S|PG_NC, %l5	! page # with protections

sloop:
        stba    %l4, [%l1]ASI_SM	/* Write segment map */
        sta     %l5, [%l1]ASI_PM	/* Write page map */
        add     %l1, %l3, %l1		/* Point to next page */
	andncc	%l1, %l0, %g0		/* segment boundary? */
	bnz	inc_pmeg
	nop
	inc	%l4			/* next pmeg */
inc_pmeg:
        cmp     %l1, %l2
        ble	sloop
	inc	%l5			/* delay, set next pte */

/**********************************************************
 * Set page 0 to be invalid, assumes kernel starts on page 1 
 **********************************************************/
	sta	%g0,[%g0]ASI_PM

	!
	! Find the the number of implemented register windows.
	! The last byte of every trap vector must be equal to
	! the number of windows in the implementation minus one.
	! The trap vector macros (above) depend on it!
	!
	wr	%g0, %g0, %wim		! note psr has cwp = 0
	set	_traptable, %g2
	mov	256, %g3		! number of trap vectors
	sethi	%hi(nwindows), %g4	! initialize _nwindows

	save %g0, %g0, %g0		! decrement cwp, wraparound to NW-1
	mov	%psr, %g1
	and	%g1, PSR_CWP, %g1       ! we now have nwindows-1
	restore				! get back to orignal window
	wr	%g0, 2, %wim			! reset initial wim
0:
	stb	%g1, [%g2 + 15]		! write last byte of trap vectors
	subcc	%g3, 1, %g3		! with nwindows-1
	bnz	0b
	add 	%g2, 16, %g2

	inc	%g1			! inialize the nwindows variable
	st	%g1, [%g4 + %lo(nwindows)]

/**************************************************************
 * Setup the flush routine, it has 6 saves and 6 restores which
 * is proper for 8 register windows.
 * null out the last one if nwindows == 7.
 **************************************************************/
nop_insn:
	nop
	cmp %g1, 8
	be eight_windows
	nop
seven_windows:
	set nop_insn, %g2
	ld [%g2], %g2			! %g2 now has nop insn
	set flush_nop_slots, %g1
	st %g2, [%g1]			! nop the last save
	add 4, %g1, %g1
	st %g2, [%g1]			! nop the first restore
eight_windows:
		! nothing to do for eight windows

/***********************************************************************
 * 		clear the bss section and store the romp pointer
 ***********************************************************************/
	set edata, %g1
	set end, %g2
	st %g0, [%g1]
bssloop:
	add %g1, 4, %g1
	cmp %g1, %g2
	bl bssloop
	st %g0, [%g1]

	set romp, %l0			
	st %g7,[%l0]

/**************************************************************************
 * Initialize _npmegs
 *************************************************************************/
	call	initNpmegs, 0
	nop
	set 	npmegs, %g2
	ld 	[%g2], %g1

	!
	! Pick a PMEG to map on board devices and set it up
	!
	srl	%g1, 1, %g1	! %g1 gets npmegs/2 for HW PMEG
	set	IO_VBASE, %g2
	stba	%g1, [%g2]ASI_SM
				! Null it out
	set 	IO_VBASE+IO_VSIZE, %g3
	set 	0x1000, %g4
null_io_pmeg:
	sta 	%g0, [%g2]ASI_PM
	add	%g2, %g4, %g2
	cmp 	%g2, %g3
	bl	null_io_pmeg
	nop

/*
* Now map in the hardware devices
*/

CLOCK_PTE = 	(PG_V | PG_KW | PGT_OBIO | PG_NC | (OBIO_CLOCK_ADDR >> PAGESHIFT))
SERIAL_PTE = 	(PG_V | PG_KW | PGT_OBIO | PG_NC | (OBIO_SERIAL_ADDR >> PAGESHIFT))
INTREG_PTE = 	(PG_V | PG_KW | PGT_OBIO | PG_NC | (OBIO_INTREG_ADDR >> PAGESHIFT))
EEPROM_PTE = 	(PG_V | PG_KW | PGT_OBIO | PG_NC | (OBIO_EEPROM_ADDR >> PAGESHIFT))
TIMER_PTE = 	(PG_V | PG_KW | PGT_OBIO | PG_NC | (OBIO_TIMER_ADDR >> PAGESHIFT))
SCSI_PTE = 	(PG_V | PG_KW | PGT_OBIO | PG_NC | (OBIO_SCSI_ADDR >> PAGESHIFT))
DMA_PTE = 	(PG_V | PG_KW | PGT_OBIO | PG_NC | (OBIO_DMA_ADDR >> PAGESHIFT))
LE0_PTE = 	(PG_V | PG_KW | PGT_OBIO | PG_NC | (OBIO_LE0_ADDR >> PAGESHIFT))


        set     TIMER_ADDR, %g1       ! map in counters
        set     TIMER_PTE, %g2
        sta     %g2, [%g1]ASI_PM

        set     EEPROM_ADDR, %g1        ! map in eeprom/clock/idprom
        set     EEPROM_PTE, %g2
        sta     %g2, [%g1]ASI_PM

        set     LE0_ADDR, %g1        ! map in le0 ethernet 
        set     LE0_PTE, %g2
        sta     %g2, [%g1]ASI_PM

        set     INTREG_ADDR, %g1        ! map in interrupt control reg
        set     INTREG_PTE, %g2
        sta     %g2, [%g1]ASI_PM
	st	%g0, [%g1]		! and disable interrupts

	set SERIAL_ADDR, %g1		! map in serial ports
	set SERIAL_PTE, %g2
	sta %g2, [%g1]ASI_PM

	set SCSI_ADDR, %g1		! Map in SCSI register
	set SCSI_PTE, %g2
	sta %g2, [%g1]ASI_PM

	set DMA_ADDR, %g1		! Map in DMA register
	set DMA_PTE, %g2
	sta %g2, [%g1]ASI_PM

/*
 * Install trap table
 */
	set 	_traptable, %g1
	mov 	%g1, %tbr
	nop; nop; nop;

/*
 * Start the kernel
 */
	set 	KernelStart__Fv, %g1
	mov	%g6, %g2
	b	_restart
	mov	%g5, %g3


/*
 * Flush windows routine
 * The boot code patches two slots with nops depending on the
 * number of windows.
 */
.section ".text"
	.align 8
	.globl flushWindows
flushWindows:
	save %g0,%g0,%g0
	save %g0,%g0,%g0
	save %g0,%g0,%g0
	save %g0,%g0,%g0
	save %g0,%g0,%g0
flush_nop_slots:
	save %g0,%g0,%g0
	restore
	restore
	restore
	restore
	restore
	ret
	restore


			!
			! Jump to this after putting a function
			! pointer in %g1
			! It will re-install the boot stack
			! and call that function, thoroghly destroying
			! whatever is cached in the register windows,
			! so dont expect it to return.
			! You are responsible for stuffing args
			! in %g2-%g7.  Dont put them in %o since
			! _restart may change the window.
			!
	.globl _restart
_restart:
	wr	%g0, 2, %wim			! reset initial wim
	nop; nop; nop;
	set	PSR_S|PSR_PIL, %g2       	! No interrupts
	mov	%g2, %psr
	or	%g2, PSR_ET, %g2		! Enable traps
	mov	%g2, %psr
	set 	_stack, %fp			! Install boot stack
	sub 	%fp, SA(MINFRAME + REGSIZE), %sp
	mov	%g2, %o0
	mov	%g3, %o1
	mov	%g4, %o2
	mov	%g5, %o3
	mov	%g6, %o4
	call	%g1
	mov	%g7, %o5
	call	restartReturnedAssertion
	nop
_resetMachine:					! And in case that fails
	set 	0x40000000, %l7 
	set 	0x04, %g6
	stba 	%g6, [%l7]ASI_CTL 
	b 	_resetMachine
	nop


	!
	! Stack Page fault trampolines
	!
	.globl _stackPageFaultG1, _stackPageFaultG5, _stackPageFaultL5
_stackPageFaultG1:
	b _stackPageFault
	mov %g1, %g6
_stackPageFaultG5:
	b _stackPageFault
	mov %g5, %g6
_stackPageFaultL5:
	b _stackPageFault
	mov %l5, %g6
_stackPageFault:
	mov 	%l0, %g2
	mov 	%l1, %g3
	mov	%sp, %g4
	mov 	%wim, %g5
	set	stackOverflow, %g1
	b	_restart
	nop

	.globl _stackAlign
_stackAlign:
	set	stackAlignment, %g1
	b	_restart
	nop

