#include <sys/asm_linkage.h>
#include <sys/param.h>
#include <sys/psw.h>
#include <sys/trap.h>
#include <sys/mmu.h>
#include <sys/pte.h>
#include <sys/reg.h>
#include <sys/buserr.h>
#include <sys/pcb.h>

#include "SPARCasmutil.h"

#define PROBE
/* #define COUNT */


#ifdef COUNT
	.seg "data"
	.align 8
	.align 4
	.globl _numOverflows
_numOverflows:
	.word 0
#endif


	! When called:
	! %l0 = %psr
	! %l1 = trapping pc
	! %l2 = trapping npc
	! %l3 = %wim
	! %l6 = NWINDOWS-1
	!
	.seg	"text"
	.align	4
	.global	w_overflow
w_overflow:
#ifdef COUNT
	sethi 	%hi(_numOverflows), %l7
	ld	[%l7 + %lo(_numOverflows)], %l5
	inc	%l5
	st	%l5, [%l7 + %lo(_numOverflows)]
#endif
	sll     %l3, %l6, %l6           ! trap vector set %l6 = NW - 1
	srl     %l3, 1, %l3 		! l3 = %wim >> 1
	mov     %g1, %l7                ! save %g1
	or      %l6, %l3, %g1           ! next WIM = %g1 = ror(WIM, 1, NW
        save	%g0, %g0, %g0           ! get into window to be saved
        mov     %g1, %wim               ! install new wim
#ifdef PROBE
	!
	! Check that stack is aligned 8
	!
	btst	0x7, %sp
	bnz _stackAlign
	!
	! Check that the stack base is on a writable page
	!
	lda	[%sp]ASI_PM, %g1
	srl	%g1, 24, %g1
	or	%g1, 0x33, %g1
	subcc	%g1, 0xf3, %g0
	bnz,a 	overflowFault
	mov 	%sp, %g1
	!
	! Check that the end is writable too
	!
	and	%sp, 0xfff, %g1
	subcc	%g1, 0x1000-(15*4), %g0
	bl	1f
	add	%sp, 15*4, %g1
	lda	[%g1]ASI_PM, %g1
	srl	%g1, 24, %g1
	or	%g1, 0x33, %g1
	subcc	%g1, 0xf3, %g0
	bnz,a 	overflowFault
	add	%sp, 15*4, %g1
1:
#endif PROBE
noFault:
	SAVE_WINDOW(%sp)
        restore                         ! go back to trap window
        mov     %l0, %psr               ! reinstall system PSR_CC
	nop; nop; nop;
        mov     %l7, %g1                ! restore g1
        jmpl    %l1, %g0                ! reexecute save
        rett    %l2

overflowFault:
	bnz 	_stackPageFaultG1
	nop

#ifdef STACK_PAGEFAULT
		!
		! * Save globals and overflow window to current process buffer
		! * Install kernel stack (if necessary)
		! * Enable traps and call pagefault
		! * Restore globals and overflow window
		! * Return from trap
		! 
	restore				! back to trap window
	set	_SS1ThisProcessorContext, %l3
	ld	[%l3], %l3
	set	_SPARCContextBufferOffset, %l5
	ld	[%l5], %l5
	add	[%l3 + %l5], %l5	! l5 has base of Buffers
	set	_SPARCContextBufferCountOffset, %l4
	ld	[%l4], %l6		! l6 has current buffer
	inc	%l6
	st	%l6, [%l4]		! update current buffer
	dec	%l6
	cmp	%l6, %g0
loop:
	bz	1f
	dec	%l6
	add	%l5, (24*4), %l5
	b	loop
	cmp	%l6, %g0
1:
					! l5 now has current buffer
	SAVE_GLOBALS(%l5)
	mov	%l5, %g5
	save	%g0,%g0,%g0
	SAVE_WINDOW(%g5 + (8*4))
					! Globals and locals now saved 
					! Call pagefault for the user
					! stack address
					!
	restore				! back to trap window
	andcc   %l0, PSR_PS, %g0	! From supervisor?
	sub 	%fp, MINFRAME+REGSIZE, %sp
	or	%l0, PSR_PIL, %g1
	mov	%g1, %psr		! No interrupts
	or	%g1, PSR_ET, %g1
	mov	%g1, %psr		! Enable traps

	bz	1f
	nop
	bnz 	_stackPageFaultG1
	nop
1:
	mov	%g1, %l6			!l6 has fault address
	set	_SS1ThisProcessorContext, %l3
	ld	[%l3], %l3
	set	_SPARCContextSystemStackOffset, %l5
	ld	[%l5], %l5
	ld	[%l3 + %l5], %l6	! l6 has kernel stack for this process
	mov	%fp, %l5
	sub 	%l6, MINFRAME+REGSIZE, %g1	! g1 has next window %sp
        save	%g0, %g0, %g0           ! get into window to be saved
	SAVE_WINDOW(%g1)
	restore
	MOV	
	mov	%psr,	
#endif
