#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David Raila (raila@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "Assert.h"
#include "Class.h"
#include "VCDisk.h"
#include "VCDiskContainer.h"
#include "FileSystem.h"
#include "Object.h"
#include "OutputStream.h"
#include "Raid.h"
#include "String.h"
#include "MemoryObjectPartition.h"
extern "C" {
#include <sys/types.h>
#include <sys/stat.h>
};


#ifdef __GNUC__
#pragma implementation
#endif


PersistentMemoryObjectRef
VCDiskContainer::open( int idNumber, int & status )
{
    Raid( RaidMethod )( "%NVCDiskContainer::basicOpen( %d )\n", this, idNumber );

    Assert (idNumber < (int)MAX_VDISKS);
    if (_disks[idNumber] == 0) {
	char unixFile[100];
	strcpy(unixFile, _prefix);
	int index = strlen(_prefix);
	Assert (idNumber < 10);
	int digit = idNumber % 10 ;
	unixFile[index++] = '0' + digit;
	unixFile[index] = 0;
	struct stat buf;
	if (::stat(unixFile, &buf) != -1)
		_disks[idNumber] = new VCDisk(unixFile);
	else
		return (PersistentMemoryObject*)0;
	}
	return _disks[idNumber];

	Assert (_disks[idNumber] != 0);
	int start = 0;
	int count = 0;
	_disks[idNumber]->locatePartition(idNumber, start, count);
	return (new MemoryObjectPartition(count, 
		_disks[idNumber]->log2UnitSize(),
		idNumber, this, _disks[idNumber], start));
}


VCDiskContainer::VCDiskContainer( PersistentMemoryObject * source)
	: MemoryObjectContainer( source )
{
	initDebug();
	Raid( RaidConstructor )( "%NVCDiskContainer::VCDiskContainer( %N, )\n",
				 this, _source );
	Assert (source->isMemberOf(VCDiskClass));
	_disks[0] = (VCDisk*)source;
	_prefix = "vdisk";
	initializeArrays(MAX_VDISKS);
}


ProxiableObject *
VCDiskContainerConstructor( ProxiableObject * source )
{
	Debug.printf( "VCDiskContainerConstructor( %N )\n", source );

        if( source->isKindOf( DiskClass ) ) {
		Disk * disk = (Disk *)source;
                return( new VCDiskContainer( disk) );
        }
        else return( 0 );
}


#include "ClassInstaller.h"
DEFINECLASS("VCDiskContainer",VCDiskContainer,MemoryObjectContainer,&VCDiskContainerConstructor,0);
