#ifdef __GNUC__
#pragma implementation
#endif

#include "VCEthernet.h"

extern "C" {
#include	<memory.h>
#include	<signal.h>
#include	<stdio.h>
#include	<fcntl.h>
#include	<malloc.h>
#include	<stdlib.h>
#include	<unistd.h>
#include	<sys/param.h>
#include	<sys/file.h>
#include	<sys/ioctl.h>
#include	<sys/time.h>
#include	<net/nit_pf.h>
#include	<net/nit_buf.h>
#include	<net/packetfilt.h>
#include	<stropts.h>
#include	<sys/socket.h>
#include	<net/if.h>
#include	<net/nit_if.h>
#include	<netinet/in.h>
#include	<netinet/if_ether.h>
#include	<netdb.h>
int	putmsg(int fd, struct strbuf *ctl, struct strbuf *data, int flags);
int	setpgid(pid_t, pid_t);
int	ioctl(int, int, void *);
int	vfork();
int	socket(int, int, int);
int	gethostname(char *, int);
void	bcopy(char *, char *, int);
void	bzero(char *, int);
int	connect(int, struct sockaddr *, int);
int	bind(int, struct sockaddr_in *, int);
int	listen(int, int);
int	select(int, fd_set *, fd_set *, fd_set *, struct timeval * );
int	accept(int, struct sockaddr *, int *);
}

main()
{
  int			i, j;
  int			newSocket, mySocket;
  int			*connectedSockets;
  int 			connectedCount = 0;
  struct sockaddr_in	address;
  fd_set		files;
  char			*packet;
  int			packetSize, addrLen;
  const int		maxPacketSize = 4096;

	/* Create our own process group */
  setpgid(0, 0);

  	/* Close all open file descriptors */
  for (i = 0; i < NOFILE; i++)
    close(i);

  	/* Restore default behavior for all signals, since we don't
	   want to take a VCInterrupt */
  for (i = 0; i < 32; i++)
    signal(i, SIG_DFL);

  	/* Ignore SIGPIPE, which we get when we try to write to a
	   closed socket.  We'll eventually get around to closing it,
	   but it might not be right away */
  signal(SIGPIPE, SIG_IGN);

  	/* Create the socket we use to accept connections */
  if ((mySocket = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
    perror("socket: ");
    exit(0);
    }

  	/* Set it to non-blocking mode */
  fcntl(mySocket, F_SETFL, O_NDELAY);

  	/* Accept connections from anyone.  This conceivably allows
	   VChoices on remote machines to talk to us */
  address.sin_addr.s_addr = INADDR_ANY;
  address.sin_port = htons(networkPort);
  address.sin_family = AF_INET;

  	/* Bind and listen on the socket */
  if (bind(mySocket, &address, sizeof(address)) < 0) {
    perror("bind: ");
    exit(0);
    }

  if (listen(mySocket, SOMAXCONN) < 0) {
    perror("listen: ");
    exit(0);
    }


#ifdef Hmm
  	/* If this is defined, the server will exit if no one
	   connects within the first two minutes.  Otherwise,
	   the server could wait indefinitely for the first
	   connection */
  struct timeval	initialWait;

  FD_ZERO(&files);
  FD_SET(mySocket, &files);

  initialWait.tv_sec = 120;	/* two minutes */
  initialWait.tv_usec = 0;
  if (select(FD_SETSIZE, &files, NULL, NULL, &initialWait) <= 0){
    perror("select: ");
    exit(0);
    }
#endif

  	/* Initialize our record of who is connected */
  connectedSockets = (int *) malloc(0);
  packet = (char *)malloc(maxPacketSize);
  do
  {
  	/* Initialize the list of descriptors to listen on */
    FD_ZERO(&files);
    FD_SET(mySocket, &files);
    for (i = 0; i < connectedCount; i++)
      FD_SET(connectedSockets[i], &files);

  	/* Wait until there is data to read */
    select(FD_SETSIZE, &files, NULL, NULL, NULL);

  	/* Go through all connections, and process those that have data */
    for (i = 0; i < connectedCount; i++)
      if (FD_ISSET(connectedSockets[i], &files))
      {
	int	err;

  		/* Read the size of the upcoming packet */
	err = read(connectedSockets[i], &packetSize, sizeof(int));

		/* If select marks the socket as being ready to read,
		   but there is no data, then the semantics state
		   that this is a closed socket.  So remove it from
		   the list of connections */
	if (err <= 0)
	{
	  if (connectedCount <= 0){
		printf("Count <= 0!\n");
		exit(1);
		}
	  connectedCount--;
	  bcopy((char*)&connectedSockets[i+1], (char*)&connectedSockets[i],
		(connectedCount - i) * sizeof(int));
	}
  		/* Something is wrong, so flush all input */
	else if (packetSize > maxPacketSize)
	{
	  while (read(connectedSockets[i], packet, maxPacketSize))
	    ;
	}
  		/* Read and distribute the packet */
	else
	{
		/* The socket is blocking, so we will wait until
		   the packet actually arrives. Otherwise there is
		   trouble if we are faster than the sender */
	  read(connectedSockets[i], packet, packetSize);

  		/* For all connections, send the packet */
	  for (j = 0; j < connectedCount; j++)
	    if (i != j)
	    {
		/* Beware: the receiving side should use a blocking
		   socket, in case the reader is faster than these
		   two writes */
	      write(connectedSockets[j], &packetSize, sizeof(int));
	      write(connectedSockets[j], packet, packetSize);
	    }
	}
      }

  	/* Check to see if there are any new connections */
    addrLen = sizeof(address);
    if ((newSocket = accept(mySocket, (sockaddr *) &address, &addrLen)) >= 0)
    {
  	/* Adjust the list and fill in the new connection */
      connectedSockets = (int *) realloc(connectedSockets,
					 ++connectedCount * sizeof(int));
      connectedSockets[connectedCount - 1] = newSocket;
      fcntl(newSocket, F_SETFL, 0);
    }
  } while (connectedCount > 0);

  	/* Since we're a child, we exit */
  exit(0);
}
