//#define DEBUG
//#define AIO
#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * VCDisk.cc: Multimax EMC Disk driver.
 */

#ifdef __GNUC__
#pragma implementation
#endif

#include "Assert.h"
#include "Raid.h"
#include "VCDisk.h"
#include "VCDiskContainer.h"
#include "VCInterrupts.h"
#include "MemoryObjectPartition.h"
#include "String.h"
#include "Class.h"
#include "ClassStar.h"
#include "CPU.h"
#include "ThisCPU.h"
#include "Process.h"
#include "ThisProcess.h"
#include "Domain.h"


extern ClassStar StoredObjectClass;
extern ClassStar VCContainerClass;

extern "C" {
#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>

#if defined(linux)
#define AIO_INPROGRESS -2
#include <aio.h>
//#include <linux/timespec.h>
#else
#include <sys/asynch.h>
#endif

#include <sys/time.h>
#include <sys/file.h>
#include <fcntl.h>
//int open( char *, int = 0, int = 0 );
//int lockf( int, int, long);
//int aioread(int, char*, int, int, int, aio_result_t*);
//int aiowrite(int, char*, int, int, int, aio_result_t*);
//int aiowait(timeval *);
};

VCDisk::VCDisk( char * diskname)
: Disk( 1 ), _wait(0)
{
	initDebug();
	_unixFile = -1;
	Assert( AIO_INPROGRESS != 0);
	_resultP.aio_return = 0;

	Raid( RaidConstructor )( "%N::VCDisk()\n", this );


	_log2UnitSize = 9;
	_unixFile = ::open( diskname, 2 /*O_RDWR*/ );
	Assert (_unixFile != -1)("Can't open VDisk file %s for read/write\n", diskname);

	int end = ::lseek( _unixFile, 0, SEEK_END);
	_numberOfUnits = end >> _log2UnitSize;

	/*
	 * Make sure we are the only process using the file
	 */
	int ret = lockf(_unixFile, F_TLOCK, end);
	Assert (ret == 0) ("VDisk file %s already locked!\n", diskname);
	ret = lockf(_unixFile, F_LOCK, end);
	Assert (ret == 0) ("Can't lock VDisk file %s, already in use?\n", 
	diskname);

#ifdef AIO
	ret = fcntl(_unixFile, F_SETFL, FASYNC);
	Assert (ret != -1);
	//ret = fcntl(_unixFile, F_SETOWN, getpid());
	//Assert (ret != -1);

	Interrupt *ex = new VCDiskInterrupt(this);
	//VCIOInterrupt::setIOInterrupt(_unixFile, ex);
	thisCPU()->setInterrupt(SIGIO, ex);
#endif
}

VCDisk::~VCDisk()
{
	::close( _unixFile );
}


int
VCDisk::doio(unsigned int start, int count, char * addr, int nbytes,	int direction)
{
  Debug << "VCDIsk::doIO(" << start << ", " << count << ", " << asHex((unsigned)addr) 
	<< ", " << nbytes << ", " << direction << ")" << eor;
	Assert (start < numberOfUnits());
	Assert (start + count <= numberOfUnits());
	_expectedBytes = nbytes;
	int i;
#ifdef AIO
	Assert ( _resultP.aio_return == 0);
	_resultP.aio_return = AIO_INPROGRESS;
	if (direction == DiskWriteOperation)
		i = ::aiowrite( _unixFile, addr, nbytes, start*unitSize(),
			SEEK_SET, &_resultP);
	else
		i = ::aioread( _unixFile, addr, nbytes, start*unitSize(),
			SEEK_SET, &_resultP);

	Assert( i == 0);
	_wait.P();
#else
	IntStorage old ; PROCESSOR::disableInterrupts(old);
	i = lseek( _unixFile, start*unitSize(), 0 );
	Assert( i != -1);
	if (direction == DiskWriteOperation)
		i = ::write( _unixFile, addr, nbytes );
	else
		i = ::read( _unixFile, addr, nbytes );
	PROCESSOR::restoreInterrupts(old);
	Assert (i == nbytes);
#endif
  Debug << "VCDIsk::doIO(" << start << ", " << count << ", " << asHex((unsigned)addr) 
	<< ", " << nbytes << ", " << direction << ") returns" << eor;
	return( count );
}

void
VCDiskInterrupt::basicRaise (char *)
{
	_disk->handler();
}

void
VCDisk::handler()
{
	if ( _resultP.aio_return != 0 && 
			_resultP.aio_return != AIO_INPROGRESS ) {
		Assert (_resultP.aio_return  == _expectedBytes);
#if defined(linux)
		timespec tv; tv.tv_sec = 0; tv.tv_nsec = 0;
		int ret = (int)aio_suspend(NULL, 0, &tv);	// Snarf off the queue
#else
		timeval tv; tv.tv_sec = 0; tv.tv_usec = 0;
		int ret = (int)aiowait(&tv);	// Snarf off the queue
#endif
		Assert (ret != 0 && ret != -1);
		Assert (&_resultP  == (aio_result_t *)ret);
		_resultP.aio_return = 0;	// reset signal
		_wait.V();
		}
	else Console() << this << "::handler(): not for me\n" << eor;
}

#ifdef NEVER
Class *
VCDisk::supports( Class *aClass )
{
        Raid( RaidProtected )( "%NDisk::supports( %N )\n", this, aClass );

        Class * supportedClass = 0;

        if( VCDiskContainerClass->isASubclassOf( aClass ) ) {
            supportedClass = VCDiskContainerClass;
        }

        Raid( RaidProtected )( "%NDIsk::supports() returns %N\n",
                                this, supportedClass);

        return( supportedClass );
}
#endif

#include "ClassInstaller.h"
DEFINECLASS("VCDisk",VCDisk,Disk,0,0);
