#ifdef __GNUC__
#pragma implementation
#endif

/* This file is part of the Choices Operating System
n
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "VCConsole.h"
#include "CharacterInputStream.h"
#include "OutputStream.h"
#include "Process.h"
#include "ThisProcess.h"
#include "AwaitedInterruptInterrupt.h"
#include "VCCPU.h"
#include "VCInterrupts.h"
#include "ThisCPU.h"


#ifdef __GNUC__
#pragma implementation
#endif

int consoleInputInitialized = 0;

extern "C" { 
#include <signal.h>
#include <unistd.h>
#include <sys/types.h>
#include <stropts.h>
#include <fcntl.h>
#include <sys/file.h>
#include <sys/ttold.h>
#include <sgtty.h>
#include <stdlib.h>
#include <stdio.h>
//#include <sys/termios.h>
//int on_exit( void *, int);
// int ioctl( int, int, void *);
int sigpause( int);
};

	/*
	 * VChoices uses the console in non-blocking mode and
	 * gets signals for characters to simulate interrupt
	 * driven console I/O.
	 *
	 * Since VChoices may be run from UNIX signals that kill,
	 * start, or stop VChoices these signals must be handled and used
	 * to set the appropriate mode of the stdin file descriptor.
	 *
	 * setUNIXConsoleMode() sets it to be blocking, cooked.
	 * setChoicesConsoleMode() set it to be non-blocking and raw.
	 * stopHandler() sets UNIX mode and re-sends the stop signal
	 * so we don't have to hit ^Z twice.
	 */
static struct sgttyb origTerm;	
static int origFlags;
extern void Halt();

static
void
callHalt(int sig)
{
	Halt();
}


void
setUNIXConsoleMode()
{
	int r = ::ioctl (0, TIOCSETP, &origTerm);
	r = ::fcntl( 0, F_SETFL, origFlags );
	signal(SIGHUP, SIG_DFL);
	signal(SIGINT, SIG_DFL);
	signal(SIGQUIT, SIG_DFL);
	signal(SIGTRAP, SIG_DFL);
	signal(SIGABRT, SIG_DFL);
	signal(SIGEMT, SIG_DFL);
	signal(SIGSYS, SIG_DFL);
	signal(SIGTERM, SIG_DFL);
	signal(SIGTSTP, SIG_DFL);
	// signal(SIGCONT, setChoicesConsoleMode);
	// signal(SIGCONT, SIG_DFL);
	signal(SIGTTIN, SIG_DFL);
	signal(SIGTTOU, SIG_DFL);
	signal(SIGXCPU, SIG_DFL);
	signal(SIGXFSZ, SIG_DFL);
}

static void
reposter(int sig)
{
	setUNIXConsoleMode();
	kill(getpid(), sig);
}


static void
setChoicesConsoleMode(int = 0);

static void
debugTrap(int)
{
	Console() << 
		"VCCPU::debugBreakpoint(): Waiting for debugger attach or UNIX default signal\n" << "UNIX process ID is " << getpid() << "\n" << eor;

		setUNIXConsoleMode();
		sigset_t _waitDebug;
		sigfillset(&_waitDebug);
		sigdelset(&_waitDebug, SIGQUIT);
		sigdelset(&_waitDebug, SIGTRAP);
		sigdelset(&_waitDebug, SIGINT);
		sigdelset(&_waitDebug, SIGABRT);
		sigdelset(&_waitDebug, SIGTSTP);
		sigdelset(&_waitDebug, SIGCONT);

		sigsuspend(&_waitDebug);
		setChoicesConsoleMode();
}

static void
setChoicesConsoleMode(int)
{
	signal(SIGHUP, callHalt);
	signal(SIGINT, callHalt);
	signal(SIGQUIT, reposter);
	signal(SIGTRAP, debugTrap);
	signal(SIGABRT, reposter);
	signal(SIGEMT, callHalt);
	signal(SIGSYS, callHalt);
	signal(SIGTERM, callHalt);
	signal(SIGTSTP, reposter);
	signal(SIGCONT, setChoicesConsoleMode);
	signal(SIGTTIN, reposter);
	signal(SIGTTOU, reposter);
	signal(SIGXCPU, callHalt);
	signal(SIGXFSZ, callHalt);
	struct sgttyb sg;
	int r =  ioctl (0, TIOCGETP, &sg) ;
	//Assert (r == 0);
	sg.sg_flags |= O_CBREAK | O_ANYP;
	sg.sg_flags &= ~O_ECHO;
	sg.sg_flags |= O_NOFLSH;
	r =  ioctl (0, TIOCSETP, &sg);
	//Assert (r == 0);
	r =  fcntl( 0, F_SETFL, FNDELAY );
	//Assert (r >= 0 );
}


void
ConsoleInputProcessEntry()
{
    Debug << thisProcess() << "::ConsoleInputProcessEntry()\n" << eor;

	origFlags = ::fcntl(0, F_GETFL, 0);
	int r =  ioctl (0, TIOCGETP, &origTerm);
	//Assert (r == 0);
	r = atexit(setUNIXConsoleMode);
	//Assert ( r == 0);
	setChoicesConsoleMode();

	/*
	 * Catch input ready on descriptor signal on SIGIO
	 */
	r =   ioctl( 0, I_SETSIG, (caddr_t)S_INPUT);
	//Assert (r == 0);
	AwaitedInterruptInterrupt * interrupt =
			new AwaitedInterruptInterrupt();

	VCIOInterrupt::setIOInterrupt(0, interrupt);

    for(;;) {
	char buf;
	int nbytes;

	Debug << thisProcess() << ": awaiting interrupt\n" << eor;
	interrupt->await();
	Debug << thisProcess() << ": interrupt\n" << eor;
	do 	{
		nbytes = ::read( 0, &buf, 1 );
		if (nbytes > 0){
			Assert (nbytes == 1);
			Assert ( ConsoleInputStream != 0 );
			ConsoleInputStream->enqueue( &buf, 1 );
			}
		} while (nbytes > 0);
	}
}
