#define DEBUG 1
#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

//#define TEST

#include "Byte.h"
#include "String.h"
#include "PhysicalMemoryRange.h"
#include "VCKernel.h"
#include "VCPageTableConstants.h"
#include "FileSystemInterface.h"
#include "MemoryObjectContainer.h"
#include "VCConsole.h"
#include "Process.h"
#include "ThisProcess.h"
#include "RecordStream.h"
#include "RecordStreamStar.h"
#include "NameServer.h"
#include "PersistentArrayStream.h"
#include "PersistentArray.h"
#include "FileObject.h"
#include "MachineConfiguration.h"
#include "MemoryObjectView.h"
#include "CharacterInputStream.h"
#include "SourceOfZerosMemoryObject.h"
#include "PagedMemoryObjectCache.h"
#ifdef NETWORKING
#include	"EthernetAdministrator.h"
#include	"VCEthernet.h"
#include        "Conduit.h"
const unsigned int BUFFER_COUNT = 100;
const unsigned int SIZE = 2000;
#endif

//CODE ADDED BY SCOTT KIRCHER
#include "COFFLoader.h"
#include "ELFLoader.h"
#include "LKMAdaptor.h"
#include "HelloWorldLKM.h"
#include "EthernetDriverLKM.h"
#include "Class.h"

extern "C" {
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/stat.h>
// int mprotect(const char *, int, int);
// int munmap(const char *, int);
int mlockall(int);

/*
 * a.out support
 */
//#include <limits.h>
#include <sys/exechdr.h>
//#include <sys/exec.h>
};


#ifdef __GNUC__
#pragma implementation
#endif

const int PHYS_MEMORY_SIZE = 0x02000000; // 32 megs
//char * const KSTART = (char * const) 0x10000000;
char * const KSTART = (char * const)0xf0000000;
const int KERNELHEAPSIZE = 0x00800000; // 8 megs 
const int KERNELSIZE = 0x04000000; // 64 megs 
char * const ASTART = (char * const) 0xf8000000;
const int APPLICATIONSIZE = 0x04000000; // 64 megs

int StoreFD = -1;

const unsigned
VCKernel::kernelHeapSize()
	{return KERNELHEAPSIZE;}

const unsigned
VCKernel::kernelSize()
	{return KERNELSIZE;}

const unsigned
VCKernel::applicationSize()
	{return APPLICATIONSIZE;}

const char *
VCKernel::kernelStart()
	{return KSTART;}

const char *
VCKernel::applicationStart()
	{return ASTART;}

void
VCKernel::initialize(Allocator *bootHeap)
{
	/*
	 * Allocate the kernel VM working range
	 */
	int kfd = ::open("KernelVM",  O_RDWR|O_CREAT|O_TRUNC|O_SYNC, 0700);
	Assert (kfd != -1);
	::unlink("KernelVM");	// so it goes away
	int r =  ::lseek(kfd, kernelSize()-1, SEEK_SET);
	Assert (r != -1);
	r =  ::write(kfd, " ", 1);
	Assert (r != -1);
	char *p = (char*)mmap((char*)kernelStart(), kernelSize(), PROT_NONE, MAP_PRIVATE|MAP_FIXED|MAP_ANON, -1, 0);
	Assert ((int)p != -1);
	mprotect(p, kernelSize(), PROT_NONE);
	Debug << "VCKernel: KernelVM at " << asHex((unsigned)p) << " PROT_NONE: " << PROT_NONE << "\n";
	//Assert( munmap(p, kernelSize()) == 0);


	/*
	 * Allocate the Application VM range
	 */
	int afd = ::open("ApplicationVM",  O_RDWR|O_CREAT|O_TRUNC|O_SYNC, 0700);
	Assert (afd != -1);
	::unlink("ApplicationVM");	// so it goes away
	r =  ::lseek(afd, applicationSize()-1, SEEK_SET);
	Assert (r != -1);
	r =  ::write(afd, " ", 1);
	Assert (r != -1);
	p = (char*)mmap((char *)applicationStart(), applicationSize(), 
		PROT_NONE, MAP_SHARED|MAP_FIXED, afd, 0);
	Assert ((int)p != -1);
	Assert (p == applicationStart());
	Debug << "VCKernel: ApplicationVM at " << asHex((unsigned)p) << "\n";
	//Assert( munmap(p, applicationSize()) == 0);

#ifdef NEVER
	int ml = mlockall(MCL_CURRENT|MCL_FUTURE);		// Lock mappings down, now and forever
	if (ml != 0){
		Console() << "*********** WARNING ***********\n";
		Console() << "mlockall failed!  Memory mapping may misbehave...\n";
		Console() << "Run as root to avoid memory mapping problems\n";
		Console() << "*********** END WARNING ***********\n";
		}
#endif


	/*
	 * Allocate the Store 1-1 mapped VM range
	 */
	StoreFD = ::open("MemoryImage", O_RDWR|O_CREAT|O_TRUNC|O_SYNC, 0700);
	Assert (StoreFD != -1);
		::unlink("MemoryImage");	// so it goes away
	r =  ::lseek(StoreFD, PHYS_MEMORY_SIZE-1, SEEK_SET);
	Assert (r != -1);
	r =  ::write(StoreFD, " ", 1);
	Assert (r != -1);
	//StoreVirtualAddress = mmap(0, PHYS_MEMORY_SIZE,
			//PROT_READ|PROT_WRITE, MAP_SHARED, StoreFD, 0);
	//Assert ((int)StoreVirtualAddress != -1);
	Kernel::initialize(bootHeap);
}

VCKernel::VCKernel()
{
	Console() << "(Virtual Choices on UNIX)\n" << eor;
}




DiskRef
VCKernel::rootDisk()
{
    return theDisk;
}

char * diskUnit = "0" ;		// Default vdisk0 and swap0

void
VCKernel::initializeBackingStore()
{
	char swapFile[100];
	strcpy(swapFile, "swap");
	strcat(swapFile, diskUnit);
	MemoryObjectStar disk = new VCDisk(swapFile);
	Assert( disk != 0 );
	int status = 0;
	MemoryObjectContainerStar rootContainer =
		disk->asA( MemoryObjectContainerClass, status );
	Assert( status == 0 );
	Assert( rootContainer != 0 );
	MemoryObjectStar mo = rootContainer->open( 0, status, O_NCACHE );
	Assert( status == 0 );
	Assert( mo != 0 );
	_backingStoreSource = mo->asA( MemoryObjectContainerClass, status );
	Debug << "backingStoreSource: " << _backingStoreSource << "\n" << eor;
	Assert( _backingStoreSource != 0 );
	_backingStoreSource->setName( "backingStoreSource" );
}

void
VCKernel::mountNonRootFileSystems()
{
	// _fileSystemInterface->mount( "!6/", "usr", 0 );
}

/*
 * boot information that is set by main()
 */
struct bootinfo {
    int textstart;
    int datastart;
    int bssstart;
    int end;
};

extern struct bootinfo * BOOTINFO;
void
VCKernel::buildRunningCPU()
{
    /*
     * Build a CPU object for this processor.
     */
    CPU * current = new VCCPU( _initialScheduler );
    current->setName( "#CPU" );
}



void
VCKernel::configureHardware()
{

	/*
	 * Configure the machine and determine the map of physical memory.
	 */
	char diskFile[100];
	strcpy( diskFile, "vdisk");
	strcat(diskFile, diskUnit);
	theDisk = new VCDisk( diskFile );
	Assert (theDisk != 0);
}

PhysicalMemoryRange * 
VCKernel::physicalMemoryMap()
{
    PhysicalMemoryRange * memoryMap = 
	//new PhysicalMemoryRange(StoreVirtualAddress,
	new PhysicalMemoryRange(0,
				PHYS_MEMORY_SIZE / PhysicalPageSize, 
				Log2PhysicalPageSize, 
				"#Memory (RAM)" );

	return( memoryMap );
}

const unsigned int
VCKernel::physicalMemorySize()
{
	return PHYS_MEMORY_SIZE;
}

void
VCKernel::basicInitializeDevicesAndManagers()
{
    // Create console manager process
    extern void ConsoleInputProcessEntry();

    Process * process = new SystemProcess( &ConsoleInputProcessEntry,
					  domain(), normal, 0 );
    Assert( process != 0 );
    process->setName("# VC Console Input");
    process->ready();

	/*
	 * Start the Timeout timer & Periodic timer helper processes.
	 */
	Debug << "Build Timeout Manager process\n" << eor;
	extern void TimeoutTimerEntry( int );
	process = new SystemProcess( (PFV) TimeoutTimerEntry,
		domain(), normal, 0 );
	Debug << "Timeout manager process " << process << "\n" << eor;
	Assert( process != 0 );
	process->setName( "#TimeoutTimerManager" );
	process->ready();
	thisProcess()->relinquishProcessor();

	Debug << "Build Periodic Manager process\n" << eor;
	extern void PeriodicTimerEntry( int );
	process = new SystemProcess( (PFV) PeriodicTimerEntry,
		domain(), normal, 0 );
	Debug << "Periodic manager process " << process << "\n" << eor;
	Assert( process != 0 );
	process->setName( "#PeriodicTimerManager" );
	process->ready();
	thisProcess()->relinquishProcessor();
}

int
VCKernel::isObjectFile( PersistentMemoryObject * file )
{
	Debug << this << "::isObjectFile( file: " << file 
		<< ")\n" << eor;

	Assert ( file != 0 );
	int ret = 1;

	/*
	 * Map the file to read the exec header and copy from it, etc.
	 */
	const char * fileAddr = TheKernel->domain()->add( file, NoAccess );
	exec *hdr = (exec *)fileAddr;

	if (hdr->a_dynamic) {
		Console() << this << "::isObjectFile(): Dynamic file\n" << eor;
		ret = 0;
		}
	if (hdr->a_machtype != M_SPARC) {
		Console() << this << "::isObjectFile(): Not a SPARC executable\n" << eor;
		ret = 0;
		}
	if (hdr->a_magic != ZMAGIC) {
		Console() << this << "::isObjectFile(): File not ZMAGIC\n" << eor;
		ret = 0;
		}
	if ( (hdr->a_trsize + hdr->a_drsize) != 0 ) {
		Console() << this << "::isObjectFile(): Bad trsize + drsize\n" << eor;
		ret = 0;
		}

	TheKernel->domain()->remove( file, fileAddr );
	return ret;
}

char * textAddr; 	// global so gdb can use it.

const char * 
VCKernel::loadObjectFile( PersistentMemoryObject * file,
				     Domain * newDomain )
{
#ifdef TEST
	newDomain = TheKernel->domain();
#endif
	Debug << this << "::loadObjectFile( file: " << file 
			<< ", domain: " << newDomain << ")\n" << eor;
	
	Assert ( file != 0 );
	Assert ( newDomain != 0 );

	/*
	 * Map the file to read the exec header and copy from it, etc.
	 */
	const char * fileAddr = TheKernel->domain()->add( file, NoAccess );
	exec *hdr = (exec *)fileAddr;

	int ret = 1;
	if (hdr->a_dynamic) {
		Console() << this << "::loadObjectFile(): Dynamic file\n" << eor;
		ret = 0;
		}
	if (hdr->a_machtype != M_SPARC) {
		Console() << this << "::loadObjectFile(): Not a SPARC executable\n" << eor;
		ret = 0;
		}
	if (hdr->a_magic != ZMAGIC) {
		Console() << this << "::loadObjectFile(): File not ZMAGIC\n" << eor;
		ret = 0;
		}
	if ( (hdr->a_trsize + hdr->a_drsize) != 0 ) {
		Console() << this << "::loadObjectFile(): Bad trsize + drsize\n" << eor;
		ret = 0;
		}
	if (ret == 0){
		TheKernel->domain()->remove( file, fileAddr );
		return 0;
		}

	Assert (! hdr->a_dynamic);
	Assert (hdr->a_machtype == M_SPARC);
	Assert (hdr->a_magic == ZMAGIC);
	Assert (hdr->a_trsize + hdr->a_drsize == 0);

	textAddr =  (char*) (hdr->a_entry & ~(PhysicalPageSize - 1));
	unsigned textSize = hdr->a_text ;
	char *dataAddr = textAddr + textSize;
	unsigned dataSize = hdr->a_data ;
//	char *bssAddr = dataAddr + dataSize;
	unsigned bssSize = hdr->a_bss;
	const char *entry = (char*)(hdr->a_entry);

	/*
	 * SunOS ld doesn't count the exec header in the offsets in
	 * the file, so map starting after that.
	 * gcc ld didn't need this.   MOViews's don't quite support
	 * a non-page boundary offset, so dont -DMOVIEW in the mean time.
	 * Map the file once, copy the parts into the data, text, etc.
	 * MO's, then remove the file.
	 */
#define MOVIEW

	Debug << "building text section: size " << asHex(hdr->a_text) 
		<< "\n"  << eor;
	PersistentMemoryObjectStar mo = (PersistentMemoryObject*)0;
#ifdef MOVIEW
	mo = new NormalMemoryObjectView
		( file, textSize, 0, 0 );
#else
	mo = TheKernel->allocateMemoryObjectForBackingStore();
	mo->setNumberOfUnits( textSize >> mo->log2UnitSize());
	const char *taddr = TheKernel->domain()->add( mo, NoAccess );
	ByteCopy( ( char * ) (fileAddr+(sizeof(struct exec))),
		( char * ) taddr, textSize );
	TheKernel->domain()->remove( mo, taddr );
#endif
	mo->setName( ".text" ); 
	const char *where = newDomain->add( mo, textAddr, ReadOnly );
	Assert (where == textAddr);
#ifdef TEST
	newDomain->makeAddressable(textAddr, textSize);
#endif

	Debug << "building data section: \n" << eor;
	unsigned dataOffset = (unsigned) pageAlign((void*)(textSize + PhysicalPageSize - 1));
	mo = TheKernel->allocateMemoryObjectForBackingStore();
	mo->setNumberOfUnits( (dataSize + bssSize) >> mo->log2UnitSize());
	const char *daddr = TheKernel->domain()->add( mo, NoAccess );
	ByteCopy( ( char * ) (fileAddr+dataOffset),
		( char * ) daddr, dataSize );
	TheKernel->domain()->remove( mo, daddr );
	mo->setName( ".data+.bss" ); 
	where = newDomain->add( mo, dataAddr, ReadWrite );
	Assert (where == dataAddr);

	TheKernel->domain()->remove( file, fileAddr );
	return entry;
}

int
VCKernel::tryCommand(int argc, char *argv[], int number)
{
	if ( strncmp( argv[0], "cpon", 4 ) == 0 ) {
		cponCommand(argc, argv);
		return 1;
		}else
	//CODE ADDED BY SCOTT KIRCHER
	//COFFLoader kernel command
        if ( strncmp( argv[0], "coffload", 8 ) == 0) 
	  {
	    coffloadCommand(argc,argv);
            return 1;
	  }else
	//CODE ADDED BY SCOTT KIRCHER
	//ELFLoader kernel command
        if ( strncmp( argv[0], "elfload", 8 ) == 0) 
	  {
	    elfloadCommand(argc,argv);
            return 1;
	  }
	//CODE ADDED BY SCOTT KIRCHER
	//start
        if ( strncmp( argv[0], "start", 5 ) == 0) 
	  {
	    startCommand(argc,argv);
            return 1;
	  }
	//CODE ADDED BY TANYA L. CRENSHAW
	//ping
	if (strncmp( argv[0], "ping", 4) == 0)
	  {
	    pingCommand(argc, argv);
	    return 1;
	  }
	else {
	  switch (number) {
	        default:
		    return Kernel::tryCommand(argc, argv, number);
	      }
	    }
	return(1);
}

#ifdef NEVER
int 
read( int fd, char *buf, int length)
{
	aio_result_t resultp;
	aioread(fd, buf, length, 0, SEEK_CUR, &resultp);
	int res = aiowait(0);
	Assert (res != 0 && res != -1);
	lseek(fd, resultp.aio_return, SEEK_CUR);
	return resultp.aio_return;
}
#endif

//CODE ADDED BY SCOTT KIRCHER
//COFFLoader kernel command
void VCKernel::coffloadCommand(int argc, char **argv)
{
  if(argc<2) {
    Console().printf("usage: coffload classname");
    return;
  }

  COFFLoader *loader=new COFFLoader(argv[1]);
  delete loader;
  
  //try to instantiate the class we just loaded
  ClassStar aClass = StandardNameServer->lookup( argv[1], ClassClass );
  if((Class*)aClass)
  {
    Console()<<"Load succeeded\n";
    if(aClass->constructor(0)) Console()<<"Constructor successfull\n";
    else Console()<<"Constructor failed\n";
  }else
    Console()<<"Load failed\n";
}

//CODE ADDED BY SCOTT KIRCHER
//ELFLoader kernel command
void VCKernel::elfloadCommand(int argc, char **argv)
{
  if(argc<2) {
    Console().printf("usage: elfload modulename\n");
    return;
  }

  LKMAdaptor *loader=new LKMAdaptor(argv[1]);
  delete loader;
}

//CODE ADDED BY SCOTT KIRCHER
//start (some LKM device) command
void VCKernel::startCommand(int argc, char **argv)
{
  if(argc<3) {
    Console().printf("usage: start devicename modulename\n");
    return;
  }

  if(!strcmp(argv[1],"HelloWorldLKM"))
  {
	HelloWorldLKM *device=new HelloWorldLKM(argv[2]);
	device->initialize();
	delete device;
  }

  if(!strcmp(argv[1],"EthernetDriverLKM"))
  {
	IntStorage state ; PROCESSOR::disableInterrupts(state);
	//  Start up the Administrator.
	_ethernetAdministrator = new EthernetAdministrator(
					"ethernetAdministrator");
	Assert(_ethernetAdministrator != 0);

   NetworkBufferFreeList *fl = new NetworkBufferFreeList(SIZE,BUFFER_COUNT);
                fl->setName("nbfl");
                //fl->listElems();
                StandardNameServer->bind("nbfl",fl);
				EthernetDriverLKM *device=new EthernetDriverLKM(argv[2],0,fl);
			    if(device->isReady())
				{
					_ethernetDriver=device;
				}else
				{
					_ethernetDriver=0;
					PROCESSOR::restoreInterrupts(state);
					return;
				}

	Assert(_ethernetDriver != 0);
	_ethernetDriver->setName("ethernetDriver");
	StandardNameServer->bind("ethernetDriver", _ethernetDriver);

	_ethernetDriver->ready(_ethernetAdministrator);
	_ethernetAdministrator->ready(_ethernetDriver);
	PROCESSOR::restoreInterrupts(state);
	Console()<<"EthernetDriverLKM("<<argv[2]<<") started OK.\n";
	Console()<<"Trying to transmit test packet\n";
	_ethernetDriver->transmitPacket(new NetworkBuffer((octet *)"This is a test",15,0));
  }

}


void
VCKernel::cponCommand( int argc, char ** argv )
{
	int fd;
	const int buffersize = 0x4000;

	if( argc < 2 ) {
		Console().printf( "usage: cpon unixfile filename\n" );
		return;
	}
	char * Path1 = argv[1];
	char * Path2 = argv[2];
	if( argc < 3 ) Path2 = Path1;
	if( ( fd = ::open( Path1, 0 ) ) < 0 ) {
		Console().printf( "cannot open %s\n", Path1 );
		return;
	}
	// Find the file size....
	struct stat sb;
	fstat(fd, &sb);
	int fsize = sb.st_size;
	
	int status = 0;
	char * buf = new char [buffersize];
	RecordStreamStar OFile = 0;

	//ClassStar RFSC = StandardNameServer->lookup( "RecordStream", "Class");
	//Assert (RFSC != 0);

Debug<<"****1****\n"<<eor;

	FileSystemInterfaceStar FSI = fileSystemInterface();
	unsigned int flags = O_CREAT|O_TRUNC|O_APPEND;
Debug<<"****2****\n"<<eor;
	OFile = FSI->open( Path2, RecordStreamClass, status, flags, 0755, PersistentArrayClass);
Debug<<"****3****\n"<<eor;

	if( ( OFile == 0 ) || ( status != 0 ) ) {
		Console().printf( "cannot create %s: status %d\n", Path2, status);
		::close( fd );
		return;
	}
	Assert (OFile->recordSize() == 1);
Debug<<"****4****\n"<<eor;
Debug<<OFile<<"\n"<<eor;
	int amt = read( fd, buf, buffersize );
	while( amt > 0 ) {
	  int status;
Debug<<"****4a****\n"<<eor;
		int oamt = OFile->write( buf, amt, status );
Debug<<"****4b****\n"<<eor;
		if( oamt != amt ) {
			Console().printf("Write error\n");
			break;
		}
		amt = read( fd, buf, buffersize );
	}
Debug<<"****5****\n"<<eor;

	//::close( fd );
	delete buf;
Debug<<"****6****\n"<<eor;

}

// CODE ADDED BY TANYA L. CRENSHAW
// Invoke the AborginalPingController
void
VCKernel::pingCommand(int argc, char ** argv )
{
  extern int AboriginalPingControllerInitialized;
  extern void AboriginalPingController();

  Console() << argv[1] << "\n" ;

  Process * process = new SystemProcess( (PFV) AboriginalPingController, domain(),
					 huge, 0);
  Assert( process != 0 );
  process->setName( "#AboriginalPingController" );

  process->ready();

  /*  The SystemProcess is a non-preemptable one
   *  We _may_ have the only CPU. Give it up for the moment. 
   */
  while( AboriginalPingControllerInitialized == 0 )
    thisProcess()->relinquishProcessor();

  Debug << "Aboriginal process done\n" << eor;
  
  return;
}

void
VCKernel::basicInitializeNetworkProcesses()
{
#if 0
#ifdef NETWORKING
#include	"defNETWORKING.h"
	IntStorage state ; PROCESSOR::disableInterrupts(state);
	//  Start up the Administrator.
	_ethernetAdministrator = new EthernetAdministrator(
					"ethernetAdministrator");
	Assert(_ethernetAdministrator != 0);

#	if defined(NETWORK_TEST)
		_ethernetDriver = new StubEthernetDriver();
		Debug << "\t\t\t\tStarted *STUB* EthernetDriver\n" << eor;
#	else
        NetworkBufferFreeList *fl = new NetworkBufferFreeList(SIZE,BUFFER_COUNT);
                fl->setName("nbfl");
                //fl->listElems();
                StandardNameServer->bind("nbfl",fl);
                VCEthernet *newVCethernet = new VCEthernet(0,fl);
			    if(newVCethernet->isReady())
				{
					_ethernetDriver=newVCethernet;
				}else
				{
					_ethernetDriver=0;
					PROCESSOR::restoreInterrupts(state);
					return;
				}
			//_ethernetDriver = new VCEthernet;
#	endif /* NETWORK_TEST */

	Assert(_ethernetDriver != 0);
	_ethernetDriver->setName("ethernetDriver");
	StandardNameServer->bind("ethernetDriver", _ethernetDriver);

	_ethernetDriver->ready(_ethernetAdministrator);
	_ethernetAdministrator->ready(_ethernetDriver);
	PROCESSOR::restoreInterrupts(state);
#endif
#endif
}

#ifdef NEVER
void
VCKernel::buildConfiguration()
{
	_configuration = new MachineConfiguration;
#ifdef NETWORKING
	char	localAddr[80];
	Console() << "Please enter my IP address: " << eor;
	ConsoleInputStream->read(localAddr, sizeof(localAddr));
	_configuration->bind("hostIPaddress", localAddr);

	Console() << "Please enter the IP address of my router: " << eor;
	ConsoleInputStream->read(localAddr, sizeof(localAddr));
	_configuration->bind("gatewayIPaddress", localAddr);

	Console() << "Please enter the ethernet address of my router: " << eor;
	ConsoleInputStream->read(localAddr, sizeof(localAddr));
	_configuration->bind("gatewayEthernetAddress", localAddr);

	Console() << "Using default netmask of 255.255.255.0\n" << eor;
	_configuration->bind("IPsubnetMask", "FFFFFF00");
#endif
}
#endif

#include "ClassInstaller.h"
DEFINECLASS("VCKernel",VCKernel,Kernel,0,0);
