#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "AddressFlushQueue.h"
#include "AddressTranslator.h"
#include "Assert.h"
#include "CPU.h"
#include "ThisCPU.h"
#include "OutputStream.h"
#include "Process.h"


#ifdef __GNUC__
#pragma implementation
#endif

AddressFlushQueue::AddressFlushQueue( unsigned long identifier )
{
	cpuid = identifier;
	queueHead = 0;
	queueTail = 0;
	queueLength = 0;
}

void
AddressFlushQueue::add( const char * vaddr, unsigned int len )
{
	Debug << asAddress(this) << "::add( " << asAddress( vaddr ) << ", length "
		<< len << " )\n" << eor;

	//Assert( cpuid != PerMemoryProcessorID() );

	lock.acquire();

	Debug << asAddress(this) << "::add()"
		<< " queueHead " << queueHead << ", "
		<< " queueLength " << queueLength << "\n" << eor;

	Assert( queueLength < AddressFlushQueueMaxSize );
	queue[ queueTail ].addr = vaddr;
	queue[ queueTail ].length = len;
	queueLength++;
	queueTail++;
	if ( queueTail == AddressFlushQueueMaxSize ) 
		queueTail = 0;
	lock.release();

	Assert( cpuid != 0 );
	Debug << asAddress(this) << "::add: vector sent to cpuid: " << (unsigned)cpuid
		<< " from " << thisCPU()->id() << "\n" << eor;
	thisCPU()->sendFlushRequestTo( cpuid );
	
	Debug << asAddress(this) << "::add: done\n" << eor;
}

void
AddressFlushQueue::doFlush()
{
	Debug << asAddress(this) << "::doFlush()"
		<< " queueHead " << queueHead << ", "
		<< " queueLength " << queueLength << "\n" << eor;

	lock.acquire();
	Assert( queueLength > 0 );
	const char * virtualAddress = queue[ queueHead ].addr;
	unsigned length = queue[ queueHead ].length;
	queueHead++;
	if ( queueHead == AddressFlushQueueMaxSize )
		queueHead = 0;
	queueLength--;
	lock.release();

	Debug << asAddress(this) << "::doFlush( " << asAddress( virtualAddress ) 
		<< ", length " << length << " )\n" << eor;		
	thisCPU()->MMU()->flushCache( virtualAddress, length );
	Debug << asAddress(this) << "::doFlush: done\n" << eor;
}
