#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Contributing Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * AddressTranslation.c: hardware dependent address translation information
 *           classes.
 */

#include "Assert.h"
#include "AddressTranslation.h"
#include "CPUSet.h"
#include "CPU.h"


#ifdef __GNUC__
#pragma implementation
#endif

void
AddressTranslation::flushFromAllActiveTranslationCaches( const char * vaddr,
                                                         unsigned int nbytes )
{
    Debug << this << "::flushFromAllActiveTranslationCaches( vaddr: "
          << asAddress( vaddr ) << " nbytes: " << nbytes << " )\n" << eor;
    CPUSetIterator cpuSetIterator( &cpusActiveOn );
    for( CPU * cpu = cpuSetIterator.next(); cpu != 0;
         cpu = cpuSetIterator.next() ) {
        Debug << "Looping, cpu=" << cpu << "\n" << eor;
        cpu->flushAddressTranslationCache( vaddr, nbytes );
    }
}

void
AddressTranslation::removeMapping( const char * base, unsigned int length )
{
    /*
     * First remove it from any in memory tables.
     */
    Debug << this << "::removeMapping(): flush incore tables\n" << eor;
    basicRemoveMapping( base, length );

    /*
     * Next, flush it from any TLB caches.
     */
    Debug << this << "::removeMapping(): flush TLB's\n" << eor;
    flushFromAllActiveTranslationCaches( base, length );
}

int
AddressTranslation::changeProtection( const char * vaddr, unsigned int nbytes,
                                      ProtectionLevel newLevel )
{
    /*
     * First change the mappings in any incore tables.
     */
    if( ! basicChangeProtection( vaddr, nbytes, newLevel ) ) {
        return( 0 );
    }

    /*
     * Next, flush any old mappings out of any TLB caches.
     */
    Debug << this << "::changeProtection(): flush TLB's\n" << eor;
    flushFromAllActiveTranslationCaches( vaddr, nbytes );

    return( 1 );
}

int
AddressTranslation::changeStatus( const char * vaddr, unsigned int nbytes,
                                  AddressTranslationStatus clear, 
                                  AddressTranslationStatus set )
{
    /*
     * First change the mappings in any incore tables.
     */
    if( ! basicChangeStatus( vaddr, nbytes, clear, set ) ) {
        return( 0 );
    }

    /*
     * Next, flush any old mappings out of any TLB caches.
     */
    Debug << this << "::changeStatus(): flush TLB's\n" << eor;
    flushFromAllActiveTranslationCaches( vaddr, nbytes );

    return( 1 );
}

int
AddressTranslation::referenced( const char * vaddr, unsigned int nbytes,
                                int clear )
{         
    int ret = basicReferenced( vaddr, nbytes, clear );
 
    /*
     * Next, flush any old mappings out of any TLB caches.
     */
    Debug << this << "::referenced(): flush TLB's\n" << eor;
    flushFromAllActiveTranslationCaches( vaddr, nbytes );
 
    return ret;
}

#include "ClassInstaller.h"
DEFINECLASS("AddressTranslation",AddressTranslation,Object,0,0);
