#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Contributing Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * AddressTranslator.c: hardware dependent address translator (MMU) classes.
 */

#include "AddressTranslator.h"
#include "Domain.h"
#include "ThisCPU.h"


#ifdef __GNUC__
#pragma implementation
#endif

void
AddressTranslator::activate( Domain * domain)
{
    Debug << this << "::activate( " << domain << " )\n" << eor;

    Assert( domain != 0 );
    Assert( domain->translation() != 0 );

    /*
     * Start the hardware using the new translation and keep translations
     * informed of who's using them for use in TLB consistency algorithms.
     */
    CPU * cpu = thisCPU();
    if( _currentDomain != 0 ) {
        Debug << this << "::activate: stop using " << _currentDomain
              << "\n" << eor;
        Assert( _currentDomain->translation() != 0 );
        _currentDomain->translation()->noLongerActiveOn( cpu );
    }
    _currentDomain = domain;
    domain->translation()->activeOn( cpu );
    basicActivate( domain->translation());
}


#include "ClassInstaller.h"
DEFINECLASS("AddressTranslator",AddressTranslator,Object,0,0);
