#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * Cache.cc
 */

#include "Assert.h"
#include "CacheAsync.h"
#include "OutputStream.h"

#ifdef __GNUC__
#pragma implementation
#endif

Lock CacheIOState::_lock;

void
CacheMOIOClosure::func( CacheMOIOClosure * c )
{
	unsigned noOfUnits = c->_result;
	if ( noOfUnits != c->_expected ) {
		Console() << "CacheMOIOClosure::func() " 
			<< noOfUnits << " != " << c->_expected << "\n" << eor;
		Assert( NOTREACHED );
	}
	if ( noOfUnits > 0 )
		noOfUnits = 
			( ( noOfUnits - 1 ) >> c->_log2MoUnitSizeRatio ) + 1; 
	if ( c->_parent ) {
		c->_parent->activate( noOfUnits );
	}
	delete c;
}

void
CacheIOClosure::func( CacheIOClosure * c )
{
	CacheIOState * s = c->_state;
	CacheIOState::_lock.acquire();
	s->_result += c->_result;
	s->_completed++;
	CacheIOState::_lock.release();
	if ( s->_completed == s->_count ) {
		Assert( s->_result == s->_expectedResult );
		if ( s->_parent != 0 ) {
			s->_parent->activate( s->_result );
			delete s;
		} else {
			s->_barrier.done();
		}
	}
}
