#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * CacheUnitInfo.cc
 */

#include "Assert.h"
#include "Byte.h"
#include "CacheUnitInfo.h"
#include "Log2.h"
#include "Raid.h"


#ifdef __GNUC__
#pragma implementation
#endif

OutputStream & operator<< ( OutputStream & os, CacheUnitInfo info )
{
	if ( !info.valid() ) {
		os << "[invalid]";
	} else if ( !info.modified() ) {
		os << "[valid]";
	} else {
		os << "[modified]";
	}
	return os;
}

CacheUnitInfoTable::CacheUnitInfoTable( int size )
	: _size( size ), _allocatedSize( 0 )
{
#ifdef DEBUG_CACHE_ALLOCATE
	_validSize = 0;
#endif
}

CacheUnitInfoTable::~CacheUnitInfoTable()
{
}
 
CacheUnitInfo CacheUnitInfoTable::unitInfo( int unit )
{
	Assert( NOTREACHED );
	return CacheUnitInfo( 0 );
}

void CacheUnitInfoTable::basicSetUnitInfo( int unit, CacheUnitInfo info )
{
	Assert( NOTREACHED );
}

void CacheUnitInfoTable::setSize( int size )
{
	if ( _size == size ) {
		return;
	} else if ( _size > size ) {
		CacheUnitInfo invalid( CUIInvalid );
		for ( int unit = size; unit < _size; unit++ ) {
			setUnitInfo( unit, invalid );
		}
	} else if ( _allocatedSize < size ) {
		basicSetSize( size );
	}
	_size = size;
	Assert( _allocatedSize >= _size );
}

void CacheUnitInfoTable::basicSetSize( int size )
{
	Assert( NOTREACHED );
}

ArrayCacheUnitInfoTable::ArrayCacheUnitInfoTable( int size )
	: CacheUnitInfoTable( size ), _array( 0 )
{
	_log2BitsPerInt = log2Low( sizeof( unsigned ) ) + 3;
	_log2EntriesPerInt = _log2BitsPerInt - 1;
	basicSetSize( size );
}

ArrayCacheUnitInfoTable::~ArrayCacheUnitInfoTable()
{
	delete _array;
}

CacheUnitInfo ArrayCacheUnitInfoTable::unitInfo( int unit )
{
	return CacheUnitInfo( (int)( _array[ unit >> _log2EntriesPerInt ] >> 
		                ( ( unit & ~_log2EntriesPerInt ) << 1 ) ) 
				& 0x03 );
}

void ArrayCacheUnitInfoTable::basicSetUnitInfo( int unit, CacheUnitInfo info )
{
	int shift = ( unit & ~_log2EntriesPerInt ) << 1;
	int mask = 0x03 << shift;
	int index = unit >> _log2EntriesPerInt;
	_array[ index ] = ( _array[ index ] & ~mask ) |
			  ( ( info.info() ) << shift );
}

void ArrayCacheUnitInfoTable::basicSetSize( int size )
{
	Assert( size > _allocatedSize );

	int oldArraySize = _allocatedSize >> _log2EntriesPerInt; 

	if ( size <= 128 ) {
		_allocatedSize = 128; 
	} else {
		_allocatedSize = 1 << log2High( size );
	}

	int newArraySize = _allocatedSize >> _log2EntriesPerInt;
	unsigned * newArray = new unsigned[ newArraySize ];

	if ( oldArraySize ) {
		ByteCopy( _array, newArray, oldArraySize * sizeof(unsigned) );
		ByteZero( &newArray[ oldArraySize ], 
			  ( newArraySize - oldArraySize ) * sizeof(unsigned) );
		delete _array;
	} else {
		ByteZero( newArray, newArraySize * sizeof( unsigned ) );
	}
	_array = newArray;
}

MultiLevelCacheUnitInfoTable::MultiLevelCacheUnitInfoTable( int size )
	: CacheUnitInfoTable( size ), _table( 1, 4, 4 )
{
	_allocatedSize = size;
}

MultiLevelCacheUnitInfoTable::~MultiLevelCacheUnitInfoTable()
{
}

CacheUnitInfo MultiLevelCacheUnitInfoTable::unitInfo( int unit )
{
	return CacheUnitInfo( _table.at( unit ) );
}

void MultiLevelCacheUnitInfoTable::basicSetUnitInfo( int unit, 
						     CacheUnitInfo info )
{
	_table.atPut( unit, info.info() );
}

void MultiLevelCacheUnitInfoTable::basicSetSize( int size )
{
	Assert( size > _allocatedSize );
	_allocatedSize = size;
}

