#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * CheckSum.cc
 */

#include "Assert.h"
#include "Cache.h"
#include "CheckSum.h"
#include "ClassStar.h"
#include "MemoryRange.h"
#include "Semaphore.h"


#ifdef __GNUC__
#pragma implementation
#endif

int debugCheckSum = 1;

static Semaphore csmutex( 1 );

void
CheckSum::init( Object * _obj, char * _message, int _startUnit, 
	        int _numberOfUnits, char * _buffer, int _actualNoOfUnits )
{
	debugMask = _obj->debugMask();

	if ( !debugCheckSum || !( debugMask & 64 ) )
		return;

	obj = _obj;
	message = _message;
	startUnit = _startUnit;
	numberOfUnits = _numberOfUnits;
	buffer = _buffer;

	csmutex.P();

	if ( obj->isKindOf( MemoryRangeClass) )
		unitSize = ( ( MemoryRange * ) obj )->unitSize();
	else if ( obj->isKindOf( CacheClass ) )
		unitSize = ( ( Cache * ) obj )->unitSize();
	else
		Assert( NOTREACHED );

	if ( _actualNoOfUnits < 0 ) {
		Console() << obj << "::" << message << " ( "
			<< startUnit << ", " << numberOfUnits << ", "
			<< asHex( (unsigned) buffer )
			<< " )\n" << eor;
	} else {
		Console() << obj << "::" << message << " ( "
			<< startUnit << ", " << numberOfUnits << ", "
			<< asHex( (unsigned) buffer )
			<< " ) actual = " << _actualNoOfUnits 
			<< "\n" << eor;
	}
	Console().flush();

	csmutex.V();

	if ( _actualNoOfUnits >= 0 ) 
		evaluate( _actualNoOfUnits, 0 );
}

void 
CheckSum::evaluate( int actualNoOfUnits, int printMsg )
{
	if ( !debugCheckSum || !( debugMask & 64 ) )
		return;

	csmutex.P();

	char * p = buffer;

	if ( printMsg ) {
		Console() << obj << "::" << message << " actual = " 
			<< actualNoOfUnits << "\n" << eor;
		Console().flush();
	}

	int endUnit = startUnit + actualNoOfUnits - 1;
	int unit;
	for ( unit = startUnit; unit <= endUnit; unit++, p += unitSize ) {
		Console().printf( "\t%4d = %8x\n", 
				  unit, ByteSum( p, unitSize ) );
	}

	endUnit = startUnit + numberOfUnits - 1;
	for ( ;unit <= endUnit; unit ++,  p += unitSize ) {
		Console().printf( "\t%4d = %8x ??\n", 
				  unit, ByteSum( p, unitSize ) );
	}

	Console().flush();

	csmutex.V();
}

unsigned 
ByteSum( char * p, int count )
{
	unsigned wordSize = sizeof( unsigned );
	Assert( ( ( unsigned ) p % wordSize ) == 0 && 
	        ( count % wordSize ) == 0 );
	unsigned * ip = ( unsigned * ) p;
	count = count / wordSize ;
	unsigned sum = 0;
	while ( count-- > 0 )
		sum += *ip++;
	return sum;
}
