#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */


#include "OutputStream.h"
#include "Assert.h"
#include "Byte.h"
#include "String.h"
#include "HexDump.h"
#include "NetworkBuffer.h"
#include "EthernetAdministrator.h"
#include "DVMCommunicator.h"
#include "DVMManager.h"
#include "DVMMessage.h"
#include "NewAndDelete.h"
#include "Kernel.h"
#include "MachineConfiguration.h"
#include "List.h"
#include "LinkIterator.h"


#ifdef __GNUC__
#pragma implementation
#endif

// static EthernetAddress ContagiusAddress(0x8,0x0,0x20,0xb,0xf,0x4a);
// static EthernetAddress AnxiusAddress(0x8,0x0,0x20,0xb,0x9,0x64);
// static EthernetAddress CascaAddress(0x8,0x0,0x20,0xe,0x9e,0xa8);
// static EthernetAddress ConfuciusAddress(0x8,0x0,0x20,0xb,0x26,0x73);
// static EthernetAddress AuntiemAddress(0x8,0x0,0x4c,0x0,0x20,0x04);
// static EthernetAddress CousinmAddress(0x8,0x0,0x4c,0x0,0x23,0xf0);
// static EthernetAddress TediusAddress(0x8,0x0,0x20,0x10,0xdf,0x8);
// static EthernetAddress EnviusAddress(0x8,0x0,0x20,0xe,0xe6,0x1f);
// static EthernetAddress StudiusAddress(0x8,0x0,0x20,0x10,0xa9,0xde);
// static EthernetAddress AllAddress(0xff,0xff,0xff,0xff,0xff,0xff);
// static EthernetAddress NullAddress(0x0,0x0,0x0,0x0,0x0,0x0);
// 
// static struct {
// 	int index;
// 	const char * name;
// 	EthernetAddress * address;
// } EtherHost[] = {
// { 0x1 << 0, "anxius", &AnxiusAddress },
// { 0x1 << 1, "auntiem", &AuntiemAddress },
// { 0x1 << 2, "casca", &CascaAddress },
// { 0x1 << 3, "confucius", &ConfuciusAddress },
// { 0x1 << 4, "cousinm", &CousinmAddress },
// { 0x1 << 5, "contagius", &ContagiusAddress },
// { 0x1 << 6, "envius", &EnviusAddress },
// { 0x1 << 7, "studius", &StudiusAddress },
// { 0x1 << 8, "tedius", &TediusAddress },
// { 0, (char *) 0, &NullAddress }
// };

const int maxHosts = (0x1 << 8);

int 				receiveTransferDebugFlag = 0;
int 				sendTransferDebugFlag = 0;
int				networkTestFlag = 0;
DVMCommunicator * 		TheDVMCommunicator = 0;
static EthernetProtocolID	ETHERNET_DVM_TYPE = 0x7070;
extern EthernetAdministrator *	_THE_EthernetAdministrator;

EthernetAddress *
ethernetAddress(char * string)
{
    int hex_table[] = {	0, 1, 2, 3, 4, 5, 6, 7, 8, 9,   // '0' - '9'
			0, 0, 0, 0, 0, 0, 0,	// skip ':' - '@'
			10, 11, 12, 13, 14, 15 };	// 'A' - 'F'
    
    int n0 = 16*(hex_table[ string[0] - '0' ]) +
	hex_table[ string[1] - '0' ];
    int n1 = 16*(hex_table[ string[3] - '0' ]) +
	hex_table[ string[4] - '0' ];
    int n2 = 16*(hex_table[ string[6] - '0' ]) +
	hex_table[ string[7] - '0' ];
    int n3 = 16*(hex_table[ string[9] - '0' ]) +
	hex_table[ string[10] - '0' ];
    int n4 = 16*(hex_table[ string[12] - '0' ]) +
	hex_table[ string[13] - '0' ];
    int n5 = 16*(hex_table[ string[15] - '0' ]) +
	hex_table[ string[16] - '0' ];
    
    return new EthernetAddress( n0, n1, n2, n3, n4, n5);
}
    

DVMCommunicator::DVMCommunicator()
{
    Assert(0 == TheDVMCommunicator);
    Assert(_THE_EthernetAdministrator != 0);
    
    _etherHost = new List;
    
    char *id_table[] = { "0","1","2","3","4","5","6","7","8","9",
			 "10","11","12","13","14","15","16","17","18","19",
			 "20","21","22","23","24","25","26","27","28","29",
			 "30","31" };
    char * name = 0;
    char * address = 0;
    MachineConfiguration * config = TheKernel->getConfiguration();
    for (unsigned i=0; i < 32; i++) {
	config->lookup(id_table[i],name);
	if (name != 0) {
	    config->lookup(name,address);
	    Assert(address != 0);
	    // Do not replicate name, since both tables are in the
	    // kernel. The Config table is over the lifetime of the
	    // System, so no problem
	    _etherHost->addFirst(new _dvmIdEntry(0x1 << i,
						 ethernetAddress(address),
						 name));
	}
	else
	    break;
    }
    maxHosts = 0x1 << i-1;
    
    _localAddress = _THE_EthernetAdministrator->address();
    _localIndex = machineIndex(_localAddress);
    _localName = machineName(_localIndex);
}
	
EthernetAddress *
DVMCommunicator::machineAddress(unsigned index)
{
    Assert(index <= maxHosts);
    LinkIterator i(_etherHost->theLink());
    _dvmIdEntry * id;
    while(id = (_dvmIdEntry *)i.next()) {
	if (id->index == index)
	    return id->address;
    }
    Assert(NOTREACHED);
    return 0;
}

char *
DVMCommunicator::machineName(unsigned index)
{
    Assert(index <= maxHosts);
    LinkIterator i(_etherHost->theLink());
    _dvmIdEntry * id;
    while(id = (_dvmIdEntry *)i.next()) {
	if (id->index == index)
	    return id->name;
    }
    Assert(NOTREACHED);
    return 0;
}

unsigned
DVMCommunicator::machineIndex(char * name)
{
    LinkIterator i(_etherHost->theLink());
    _dvmIdEntry * id;
    while(id = (_dvmIdEntry *)i.next()) {
	if (strcmp(id->name,name) == 0)
	    return id->index;
    }
    Assert(NOTREACHED);
    return 0;
}

unsigned
DVMCommunicator::machineIndex(EthernetAddress * address)
{
    LinkIterator i(_etherHost->theLink());
    _dvmIdEntry * id;
    while(id = (_dvmIdEntry *)i.next()) {
	if (*id->address == *address)
	    return id->index;
    }
    Assert(NOTREACHED);
    return 0;
}

void
DVMCommunicator::networkReceiveTest(char *packet, int len)
{
    
    Console() << "In DVMCommunicator::networkReceive->" << eor;
    char * tempbuf = new char[1024];
    ByteCopy(packet,tempbuf,len);
    EthernetHeader *etherHeader = (EthernetHeader *) packet;
    if (ByteCompare(etherHeader->type, &ETHERNET_DVM_TYPE, 2) == 0) 
    {
	dumpPacket(packet, len);
	delete tempbuf;
    }
}

void
DVMCommunicator::networkSendTest(char * packet, int len, char * destname)
{
    /* 
     * Package TransportPacket so that Ethernet drivers understand
     * what to do with it. 
     */
    Console() << "DVMCommunicator::networkSend->" << eor;
    
    unsigned newlen = sizeof(EthernetHeader) + len;
    char* etherPkt = new char[newlen];
    makeEtherHeader(destname,(EthernetHeader *)etherPkt);
    ByteCopy(packet, 
	     etherPkt + sizeof(EthernetHeader),
	     len);
    dumpPacket(etherPkt, newlen); 
    
    /*
     * Create A new pkt and ask the administrator to 
     * send it.
     */
    NetworkBuffer * buff = new NetworkBuffer( etherPkt,newlen ) ;
    _THE_EthernetAdministrator->transmit( buff );
    delete buff;
    delete etherPkt;
}

void
DVMCommunicator::makeEtherHeader(char * destname, EthernetHeader * space)
{
    Assert( destname != 0);
    Assert( space != 0 );

    space->dest = *machineAddress(machineIndex(destname));
    space->source = *_THE_EthernetAdministrator->address();
    ByteCopy(&ETHERNET_DVM_TYPE, space->type, 2);
}

void
DVMCommunicator::networkReceive(char *packet, int length)
{
    Console() << "<<<-------------------------\n" << eor;
    /*
     * Make a fresh copy first --- if we can avoid this somehow...
     */
    unsigned * copy = new unsigned[(length+sizeof(unsigned)-1)/sizeof(unsigned)];
    ByteCopy(packet,(char *)copy,length);
    EthernetHeader *etherHeader = (EthernetHeader *)copy;
    /*
     * If packet is our type, create a DVMMsg and put into the
     * DVMManager receive queue.
     */
    if (ByteCompare(etherHeader->type, &ETHERNET_DVM_TYPE, 2) == 0) {
	if (receiveTransferDebugFlag) dumpPacket(packet, length);
//	DVMRcvMsg * rmsg = 0;
	DVMServiceMsg * smsg = 0;
	DVMCacheMsg * cmsg = 0;
	unsigned * servicePacket = copy +
	    ((headerSize() + sizeof(unsigned)-1)/ sizeof(unsigned));
	switch(etherHeader->_DVMType) {
	  case GetPage:
#ifdef CFRONT
	    cmsg = new (servicePacket) DVMGetPageMsg(copy,length);
#else
	    cmsg = new {servicePacket} DVMGetPageMsg(copy,length);
#endif
	    break;
	  case HerePage:
#ifdef CFRONT
	    cmsg = new (servicePacket) DVMHerePageMsg(copy,length);
#else
	    cmsg = new {servicePacket} DVMHerePageMsg(copy,length);
#endif
	    break;
	  case AckPage:
#ifdef CFRONT
	    cmsg = new (servicePacket) DVMAckPageMsg(copy,length);
#else
	    cmsg = new {servicePacket} DVMAckPageMsg(copy,length);
#endif
	    break;
	  case OwnerHint:
#ifdef CFRONT
	    cmsg = new (servicePacket) DVMOwnerHintMsg(copy,length);
#else
	    cmsg = new {servicePacket} DVMOwnerHintMsg(copy,length);
#endif
	    break;
	  case GetRead:
#ifdef CFRONT
	    cmsg = new (servicePacket) DVMGetReadMsg(copy,length);
#else
	    cmsg = new {servicePacket} DVMGetReadMsg(copy,length);
#endif
	    break;
	  case HereRead:
#ifdef CFRONT
	    cmsg = new (servicePacket) DVMHereReadMsg(copy,length);
#else
	    cmsg = new {servicePacket} DVMHereReadMsg(copy,length);
#endif
	    break;
	  case GetUp:
#ifdef CFRONT
	    cmsg = new (servicePacket) DVMGetUpMsg(copy,length);
#else
	    cmsg = new {servicePacket} DVMGetUpMsg(copy,length);
#endif
	    break;
	  case HereUp:
#ifdef CFRONT
	    cmsg = new (servicePacket) DVMHereUpMsg(copy,length);
#else
	    cmsg = new {servicePacket} DVMHereUpMsg(copy,length);
#endif
	    break;
	  case AckUp:
#ifdef CFRONT
	    cmsg = new (servicePacket) DVMAckUpMsg(copy,length);
#else
	    cmsg = new {servicePacket} DVMAckUpMsg(copy,length);
#endif
	    break;
	  case Inv:
#ifdef CFRONT
	    cmsg = new (servicePacket) DVMInvMsg(copy,length);
#else
	    cmsg = new {servicePacket} DVMInvMsg(copy,length);
#endif
	    break;
	  case AckInv:
#ifdef CFRONT
	    cmsg = new (servicePacket) DVMAckInvMsg(copy,length);
#else
	    cmsg = new {servicePacket} DVMAckInvMsg(copy,length);
#endif
	    break;
	  case Join:
#ifdef CFRONT
	    smsg = new (servicePacket) DVMJoinMsg(copy,length);
#else
	    smsg = new {servicePacket} DVMJoinMsg(copy,length);
#endif
	    break;
	  case AckJoin:
#ifdef CFRONT	    
	    smsg = new (servicePacket) DVMAckJoinMsg(copy,length);
#else	    
	    smsg = new {servicePacket} DVMAckJoinMsg(copy,length);
#endif	    
	    break;
	  case Leave:
#ifdef CFRONT	    
	    smsg = new (servicePacket) DVMLeaveMsg(copy,length);
#else	    
	    smsg = new {servicePacket} DVMLeaveMsg(copy,length);
#endif	    
	    break;
	  case AckLeave:
#ifdef CFRONT	    
	    smsg = new (servicePacket) DVMAckLeaveMsg(copy,length);
#else	    
	    smsg = new {servicePacket} DVMAckLeaveMsg(copy,length);
#endif	    
	    break;
	  default:
	    Assert(NOTREACHED);
	    break;
	}
	switch(etherHeader->_DVMType) {
	  case GetPage:
	  case HerePage:
	  case AckPage:
	  case OwnerHint:
	  case GetRead:
	  case HereRead:
	  case GetUp:
	  case HereUp:
	  case AckUp:
	  case Inv:
	  case AckInv:
	    TheDVMManager->enqRcv(cmsg);
	    break;
	  case Join:
	  case AckJoin:
	  case Leave:
	  case AckLeave:
	    TheDVMManager->enqSrv(smsg);
	    break;
	  default:
	    Assert(NOTREACHED);
	    break;
	}
    }
}

void
DVMCommunicator::networkSend(unsigned * packet, unsigned length, int
			     index, DVMMsgType type)
{
    /* 
     * Setup ethernet headers.
     * Someday, we can do some pagemapping magic and stick a ethernet
     * from another page, esp. for fddi.
     */
    Console() << "----------------------->>>\n" << eor;
    
    EthernetHeader * header = (EthernetHeader *)packet;
    header->dest = *machineAddress(index);
    header->source = *_THE_EthernetAdministrator->address();
    ByteCopy(&ETHERNET_DVM_TYPE, header->type, 2);
    header->_DVMType = type;
    
    if (sendTransferDebugFlag)
	dumpPacket((char *)packet, length); 
    
    /*
     * Create A new pkt and ask the administrator to 
     * send it.
     */
    NetworkBuffer * buff = new NetworkBuffer((char *)packet,length) ;
    _THE_EthernetAdministrator->transmit(buff);
    delete buff;
}

void
DVMCommunicator::dumpPacket(char *packet, int )
{
    Console() << "Packet Dump:\n" << HexDump(packet,64) << "\n" << eor;
}

    
DVMWalker::DVMWalker()
{
    Assert(_THE_EthernetAdministrator != 0);
    _THE_EthernetAdministrator->registerType(ETHERNET_DVM_TYPE,this);
}

void
DVMWalker::walk( NetworkBuffer * buff )
{
    Assert(TheDVMCommunicator != 0);
//     if (networkTestFlag)
// 	TheDVMCommunicator->networkReceiveTest( buff->packet(),buff->length() ); 
//     else
    TheDVMCommunicator->networkReceive( buff->packet(),buff->length() ); 
    buff->release();
}
