#ifdef __GNUC__
#pragma implementation
#endif

//#define DEBUG
/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "Assert.h"
#include "OutputStream.h"
#include "DVMManager.h"
#include "Kernel.h"
#include "Process.h"
#include "ThisProcess.h"	// For debugging
#include "OneCopyDVMCache.h"
#include "DVMCommunicator.h"
#include "PhysicallyAddressableUnit.h"
#include "String.h"
#include "MemoryObject.h"
#include "NameServer.h"
#include "PersistentMemoryObject.h"
#include "PersistentMemoryObjectStar.h"
#include "FileSystemInterface.h"
#include "FileSystemInterfaceStar.h"


#ifdef __GNUC__
#pragma implementation
#endif

DVMManager * TheDVMManager = 0;

const long ONE_SECOND = 1000000;
const long ONE_MINUTE = ONE_SECOND*60;

OutputStream&
operator<<(OutputStream& os, DVMManager * mgr)
{
    return os << "DVMManager[" << asHex((int)mgr) << "]";
}

    
static void
DVMXmitProcess(int arg)
{
    DVMManager * mgr = (DVMManager *)arg;
    Console() << thisProcess() << "(" << mgr << ")\n" << eor;
    mgr->xmitProcessEntry();
}

static void
DVMRcvProcess(int arg)
{
    DVMManager * mgr = (DVMManager *)arg;
    Console() << thisProcess() << "(" << mgr << ")\n" << eor;
    mgr->rcvProcessEntry();
}

static void
DVMActionProcess(int arg)
{
    DVMManager * mgr = (DVMManager *)arg;
    Console() << thisProcess() << "(" << mgr << ")\n" << eor;
    mgr->actionProcessEntry();
}

static void
DVMTimeoutProcess(int arg)
{
    DVMManager * mgr = (DVMManager *)arg;
    Console() << thisProcess() << "(" << mgr << ")\n" << eor;
    mgr->timeoutProcessEntry();
}

static void
DVMSrvProcess(int arg)
{
    DVMManager * mgr = (DVMManager *)arg;
    Console() << thisProcess() << "(" << mgr << ")\n" << eor;
    mgr->srvProcessEntry();
}

static void
DVMCacheProcess(int arg)
{
    DVMManager * mgr = (DVMManager *)arg;
    Console() << thisProcess() << "(" << mgr << ")\n" << eor;
    mgr->cacheProcessEntry();
}

//#define FLUSHTEST
#ifdef FLUSHTEST

static void
FlushTestProcess(int)
{
    Console() << "FlushTestProcess: ready\n" << eor;
    TimeoutTimer _timer;
    for(;;) {
	_timer.start(ONE_MINUTE);
	_timer.await();
	Console() << "FlushTestProcess: awakened\n" << eor;
	int i,j;
	for (i=0; i < 10; i++) {
	    int size = 4096*(i+2);
	    char * test = new char[size];

	    for (j=0; j < 40960; j+=1000)
		test[j] = 't';

	    Console() << "FlushTestProcess: MakeUn and write (iteration "
		<< i << ", array " << asHex((unsigned)test) << ")\n" << eor;
	    thisProcess()->domain()->makeUnAddressable(test,size);
	    
	    for (j=0; j < 40960; j+=1000)
		test[j] = 't';

	    Console() << "FlushTestProcess: Deleting (iteration "
		<< i << ", array " << asHex((unsigned)test) << ")\n" << eor;
	    delete test;
	    thisProcess()->domain()->makeUnAddressable(test,size);

	    test = 0;
	}
	Console() << "Waiting\n" << eor;
	_timer.start(ONE_MINUTE);
	_timer.await();
    }
}

#endif

DVMManager::DVMManager()
: _awaitXmit(0),_awaitRcv(0),_awaitAction(0),_awaitSrv(0),_awaitCache(0),
  _moTableMutex(1),
  _remoteMutex(1),
  _remoteAwait(0),
  _serviceTimer(MaxMOs,ONE_MINUTE*2,(void *)this,DVMManager::DVMServiceTimeout)
{
    Assert(TheDVMManager == 0);

#ifdef FLUSHTEST
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Stuff for FLUSHTEST
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    Process * flushProcess = new
	PreemptableSystemProcess((PFV)::FlushTestProcess,TheKernel->domain(),
				 normal, (int)this);
    Assert(flushProcess != 0);
    flushProcess->setName("#FlushTestProcess");
    flushProcess->ready();
#endif
    

    Assert(TheDVMCommunicator != 0);
    _thisMachine = TheDVMCommunicator->localIndex();
    
//     _xmitProcess = new PreemptableSystemProcess((PFV)::DVMXmitProcess, TheKernel->domain(),
// 				     normal, (int)this);
//     Assert(_xmitProcess != 0);
//     _xmitProcess->setName("#DVMManagerXmitProcess");
//     _xmitProcess->ready();
// 
//     _rcvProcess = new PreemptableSystemProcess((PFV)::DVMRcvProcess, TheKernel->domain(),
// 				     normal, (int)this);
//     Assert(_rcvProcess != 0);
//     _rcvProcess->setName("#DVMManagerRcvProcess");
//     _rcvProcess->ready();

//     _actionProcess = new PreemptableSystemProcess((PFV)::DVMActionProcess, TheKernel->domain(),
// 				     normal, (int)this);
//     Assert(_actionProcess != 0);
//     _actionProcess->setName("#DVMManagerActionProcess");
//     _actionProcess->ready();
// 
//     _timeoutProcess = new PreemptableSystemProcess((PFV)::DVMTimeoutProcess,
// 					TheKernel->domain(), normal, (int)this);
//     Assert(_timeoutProcess != 0);
//     _timeoutProcess->setName("#DVMManagerTimeoutProcess");
//     _timeoutProcess->ready();
//     _timeoutQueue = new List();
//     _backupQueue = new List();
//     
//     _cacheProcess = new PreemptableSystemProcess((PFV)::DVMCacheProcess,
// 						 TheKernel->domain(),
// 						 normal, (int)this);
//     Assert(_cacheProcess != 0);
//     _cacheProcess->setName("#DVMManagerCacheProcess");
//     _cacheProcess->ready();

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Stuff for Setup/Teardown. Might separate this later into a separate class
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    for (int i=0; i < MaxMOs; i++) {
	_moTable[i].mo = 0;
	_moTable[i].name = 0;
	_moTable[i].machines = 0;
	_moTable[i].localrefs = 0;
	_moTable[i].remoteid = 0;
	_moTable[i].state = STATE_DVM_SRV_EMPTY;
    }
    
    _srvProcess = new PreemptableSystemProcess((PFV)::DVMSrvProcess,
					       TheKernel->domain(),
					       normal, (int)this);
    Assert(_srvProcess != 0);
    _srvProcess->setName("#DVMManagerSrvProcess");
    _srvProcess->ready();

    for (i=0; i < 10; i++) {
	_outgoing[i] = 1;
	_incoming[i] = 0;
    }
}

DVMManager::~DVMManager()
{
    _xmitProcess->die();
    _rcvProcess->die();
//     _actionProcess->die();
//     _timeoutProcess->die();
}

unsigned
DVMManager::tsindex(unsigned machine)
{
    switch(machine) {
      case 0x1 << 0:
	return 0;
	break;
      case 0x1 << 1:
	return 1;
	break;
      case 0x1 << 2:
	return 2;
	break;
      case 0x1 << 3:
	return 3;
	break;
      case 0x1 << 4:
	return 4;
	break;
      case 0x1 << 5:
	return 5;
	break;
      case 0x1 << 6:
	return 6;
	break;
      case 0x1 << 7:
	return 7;
	break;
      case 0x1 << 8:
	return 8;
	break;
      case 0x1 << 9:
	return 9;
	break;
      case 0x1 << 10:
	return 10;
	break;
      default:
	Assert(NOTREACHED);
	return 0;
	break;
    }
}


// Managing the queues:
// The Cache puts DVMMsgs on the xmit q, the DVMCommunicator puts
// them on the rcv q. The xmit process eventually transmits the
// messages and the rcv process eventually receives the messages.
// In both cases, the faulting process is waiting on a semaphore embedded
// in the
// messages. When the message exchange is complete, the faulting
// process is reawakened.
// Acks are handled within the manager --- upon transmission, the
// transmit message can be queued into the transmit queue if desired.

// I have tried to make things object oriented by having messages
// support transmit, receive and timeout. However, messages need to be
// moved among the various queues, they need to be destroyed and
// concurrenct access to the queues has to be supported. So the
// functions get tangled.

// Xmit
// The message transmits itself. If transmit returns a message, it is
// queued into the timeout queue.
// The message destroys itself if it does not need an ack.

void
DVMManager::xmitProcessEntry()
{
    DVMMsg * message = 0;
    
    for(;;) {
	message = deqXmit();
	
	if (message != 0) {
	    Debug << "DVMManager::xmit doit\n" << eor;
	    message->transmit();
// 	    if (message != 0) {
// 		Assert(message->type() != Join);
// 		Assert(message->type() != Leave);
// 		Assert(message->type() != AckJoin);
// 		Assert(message->type() != AckLeave);
// 		enqTimeout((DVMCacheMsg *)message);
// 	    }
	}
	else {
	    Debug << "DVMManager::xmit wait\n" << eor;
	    _awaitXmit.P();
	}
    }
}


// Receive:
// Rcv needs a case statement since we have a raw representation of
// the message. It is possible to convert it into a class elsewhere
// using a table of types, and then a case statement may not be required.
// There are possible efficiency hassles in such a case.

// GetPage messages cause invocation of remote methods on the cache.
// Such a method should not need message reception for completion.
// So we hand it over to the action process.

// HerePage and AckPage find the waiting message on the timeout queue
// and call the receive method. If the receive method returns a
// message, it is an ack and is sent back to the xmit queue.
// If there is no receiving message, the received message must answer
// an ack if any. This is fairly ugly.

void
DVMManager::rcvProcessEntry()
{
    DVMCacheMsg * message = 0;
    
    for(;;) {
	message = deqRcv();
	
	if (message != 0) {
#ifdef ASSERT
	    switch (message->type()) {
	      case GetPage:
	      case HerePage:
	      case AckPage:
	      case OwnerHint:
	      case GetRead:
	      case HereRead:
	      case GetUp:
	      case HereUp:
	      case AckUp:
	      case Inv:
	      case AckInv:
		_cache->enqAction(message);
		break;
	      default:
		Assert(NOTREACHED);
		break;
	    }
#else
	    _cache->enqAction(message);
#endif /* ASSERT */    
	}
	else {
	    Debug << "DVMManager::rcv wait\n" << eor;
	    _awaitRcv.P();
	}
    }
}

void
DVMManager::enqRcv(DVMCacheMsg * msg)
{
    _cache->enqAction(msg);
}

void
DVMManager::srvProcessEntry()
{
    DVMServiceMsg * message = 0;
    
    for(;;) {
	message = deqSrv();
	
	if (message != 0) {
	    unsigned * packet = message->top();
	    Console() << "Srv(" << message << ")\n" << eor;
	    switch (message->type()) {
	      case Join:
	      case Leave:
		if (message->timestamp() >=
		    _incoming[tsindex(message->source())]) {
		    message->action(this);
		}
		break;
	      case AckJoin:
	      case AckLeave:
		if (message->timestamp() >=
		    _outgoing[tsindex(message->source())]) {
		    message->action(this);
		}
		break;
	      default:
		Assert(NOTREACHED);
		break;
	    }
// 	    switch (message->type()) {
// 	      case Join:
// 	      case Leave:
// 	      case AckJoin:
// 	      case AckLeave:
// 		message->action(this);
// 		break;
// 	      default:
// 		Assert(NOTREACHED);
// 		break;
// 	    }
	    delete packet;
	}
	else {
	    Debug << "DVMManager::srv wait\n" << eor;
	    _awaitSrv.P();
	}
    }
}

void
DVMManager::cacheProcessEntry()
{
    DVMCacheMsg * message = 0;
    
    for(;;) {
	message = deqCache();
	
	if (message != 0) {
	    Assert(NOTREACHED);
	    unsigned * packet = message->top();
	    DVMMsg * waiter = 0;
	    DVMMsg * ack = 0;
	    switch (message->type()) {
// 	      case GetPage:
// 		Debug << "DVMManager::cache GetPage\n" << eor;
// 		Console() << "Cache(" << message << ")\n" << eor;
//  		_cache->remoteCache(message->unit(),
//  				    RemoteRead, message->source());
// 		//message->action(this);
// 		break;
	      default:
		Assert(NOTREACHED);
		break;
	    }
	    delete packet;
	}
	else {
	    Debug << "DVMManager::cache wait\n" << eor;
	    _awaitCache.P();
	}
    }
}

// Action
// Invoke cache actions that may cause message transmission reception
// etc. This process can block. Only one cache action will be invoked
// at a time.

void
DVMManager::actionProcessEntry()
{
    DVMCacheMsg * message = 0;
    unsigned * packet = 0;
    DVMServiceMsg * nmsg = 0;
    
    for(;;) {
	message = deqAction();
	if (message != 0) {
	    Assert(NOTREACHED);
	    switch (message->type()) {
	      default:
		Assert(NOTREACHED);
		break;
	    }
	}
	else {
	    Debug << "DVMManager::action wait\n" << eor;
	    _awaitAction.P();
	}
    }
}

// Timeout:
// a message typically returns a copy of itself when timeouting. The
// copy is put on to transmit queue. It is possible that an ack is
// received after timeout but before the message is re-sent.
// The actual message on the timeout q will then be destroyed.
// In this case, for a HerePage message, the contents of the pau can be
// garbage, and a GetPage message no longer relevant. That's fine,
// since any messages that do not make sense get dropped properly and
// their contents are irrelevant.

// Notice that making things oo is even harder here because of the
// split synchronization calls acquire and release.

void
DVMManager::timeoutProcessEntry()
{
    DVMMsg * message = 0;
    
    for(;;) {
	acqTimeout();
	if (message = deqTimeout()) {
	    Assert(NOTREACHED);
	    DVMMsg * xmit = message->timeout();
	    enqBackup((DVMCacheMsg *)message);
	    rlsTimeout();
	    if (xmit) {
		Debug << "DVMManager::time doit\n" << eor;
		enqXmit(xmit);
	    }
	}
	else {
	    exchangeTOQ();
	    rlsTimeout();
	    Console() << "DVMManager::time wait\n" << eor;
	    _timer.start(ONE_MINUTE*2);
	    _timer.await();
	}
    }
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
// Stuff for Setup/Teardown. Might separate this later into a separate class
//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

#define Debug Console()

// used by join requests and local requests
unsigned
DVMManager::find(char * path)
{
    Assert(path != 0);
    // Find if name matches
    for (unsigned i=1; i < MaxMOs; i++) {
	if (_moTable[i].name != 0 && (strcmp(path,_moTable[i].name) == 0))
	    return i;
    }
    return 0;
}

// Used by leave requests
unsigned
DVMManager::find(unsigned remote)
{
    Assert(remote > 0);
    for (unsigned i=1; i < MaxMOs; i++) {
	if (_moTable[i].remoteid == remote)
	    return i;
    }
    return 0;
}

unsigned
DVMManager::installName(char * path)
{
     // If not, return first empty slot
    for (unsigned i=1; i < MaxMOs; i++) {
	if (_moTable[i].name == 0) {
	    _moTable[i].name = new char[strlen(path)+1];
	    strcpy(_moTable[i].name,path);
	    return i;
	}
    }
    Assert(NOTREACHED);
    return 0;
}

void
DVMManager::enableTimeout(unsigned slot)
{
    Assert(slot > 0);
    _serviceTimer.enable(slot);
}

void
DVMManager::disableTimeout(unsigned slot)
{
    Assert(slot > 0);
    _serviceTimer.disable(slot);
}


unsigned
DVMManager::hostId(char * path)
{
    unsigned hostnum;
    
    char * colon = index(path,':');
    Assert(colon != 0);
    if (colon) {
	*colon = '\0';
	hostnum = TheDVMCommunicator->machineIndex(path);
	*colon = ':';
    }
    else {
	hostnum = TheDVMCommunicator->localIndex();
    }
    
    return hostnum;
}

void
DVMManager::createNCDVM(unsigned slot)
{
    Assert(slot > 0);
    Assert(slot < MaxMOs);
    Assert(_moTable[slot].mo == 0);
    
    char * path = _moTable[slot].name;
    unsigned host = hostId(path);
    int isCopy = (localId() != host);
    char * filename = index(path,':') + 1;

    Debug << "::createNCDVM(" << filename << "," << host << ","
	<< isCopy << ")\n" << eor;
    
    // Open file for local pageout.
    
    FileSystemInterfaceStar FSI =
	StandardNameServer->lookup("fileSystemInterface",
				   FileSystemInterfaceClass);
    Assert (FSI != 0);
    unsigned err;
    MemoryObjectStar mo = FSI->open(filename,
				    PersistentMemoryObjectClass,
				    err,
				    O_RDRW);
    if(err) {
	Console() << "::createNCDVM(" << mo << ") *** error " << err
	    << "while opening\n" << eor;
	Assert(NOTREACHED);
	return;
    }
    
    // Create a NCDVMCache and a DVM manager for the
    // MemoryObject. Cache and Manager should refer to each other
    
    Assert(PagedMemoryObjectCacheLog2UnitSize >= mo->log2UnitSize());
    int unitSizeRatio = 1 << (PagedMemoryObjectCacheLog2UnitSize
			      - mo->log2UnitSize());
    int unitsInCache = (mo->numberOfUnits()+unitSizeRatio-1)/unitSizeRatio;
    
    Debug << "::createNCDVM(" << mo << ": unitsize=" << mo->unitSize()
	<< ",numunits=" << mo->numberOfUnits() << ")\n" << eor;
    
    // MO points to cache, cache points to MO
    
    RCDVMCache * cache = new RCDVMCache(mo, unitsInCache,
						  host, isCopy);
    mo->setMemoryObjectCache( cache );
    
    // Manager and cache point to each other
    
    TheDVMManager->addCache( cache );
    cache->setManager( TheDVMManager );

    _moTable[slot].mo = mo;
    if (isCopy)
	_moTable[slot].machines = host;
}

void
DVMManager::destroyDVM(unsigned slot)
{
    Assert(slot > 0);
    Assert(slot < MaxMOs);
    Assert(_moTable[slot].localrefs == 0);
    Assert(_moTable[slot].name != 0);
    _moTable[slot].mo = 0;
    delete _moTable[slot].name;
    _moTable[slot].name = 0;
    _moTable[slot].machines = 0;
    _moTable[slot].remoteid = 0;
}
    
MemoryObjectRef
DVMManager::setupDVM(char * path)
{
    Assert(path != 0);
    
    int isRemote = (localId() != hostId(path));
    
    // if Join Remote, check remoteMutex. Mutex is released when
    // response arrives.
    if (isRemote) {		
	_remoteMutex.P();
    }
    
    _moTableMutex.P();
    
    unsigned slot = find(path);
    if (slot == 0)
	slot = installName(path);
    
    if (isRemote)
	_moTable[slot].state =
	    (DVMServiceState *)_moTable[slot].state->joinRemote(this,slot);
    else
	_moTable[slot].state =
	    (DVMServiceState *)_moTable[slot].state->joinLocal(this,slot);
	
    _moTableMutex.V();

    if (isRemote) {
	_remoteAwait.P();
	_remoteMutex.V();
    }

    // It is safe to return this since we are holding a ref by now.
    
    return _moTable[slot].mo;
}

void
DVMManager::remoteSetupDone(char * path, DVMServiceMsg * msg)
{
    Assert(path != 0);
    Assert(msg != 0);
    
    _moTableMutex.P();

    unsigned slot = find(path);
    Assert(slot > 0);

    _moTable[slot].state =
	(DVMServiceState *)_moTable[slot].state->remoteJoinDone(this,slot,msg);
    
    _moTableMutex.V();
    
    _remoteAwait.V();
}

int
DVMManager::setupDVM(char * path, DVMServiceMsg * msg)
{
    Assert(path != 0);
    Assert(msg != 0);
    
   _moTableMutex.P();
    
    unsigned slot = find(path);
    if (slot == 0)
	slot = installName(path);
    
    _moTable[slot].state =
	(DVMServiceState *)_moTable[slot].state->remoteRequestJoin(this,slot,msg);
	
    _moTableMutex.V();

    return slot;
}

void
DVMManager::teardownDVM(char * path)
{
    Assert(path != 0);
    
    int isRemote = (localId() != hostId(path));
    
    // if Join Remote, check remoteMutex. Mutex is released when
    // response arrives.
    if (isRemote) {		
	_remoteMutex.P();
    }
    
    _moTableMutex.P();
    
    unsigned slot =  find(path);
    Assert(slot > 0);
    
    if (isRemote)
	_moTable[slot].state =
	    (DVMServiceState *)_moTable[slot].state->leaveRemote(this,slot);
    else
	_moTable[slot].state =
	    (DVMServiceState *)_moTable[slot].state->leaveLocal(this,slot);
	
    _moTableMutex.V();

    if (isRemote) {
	_remoteAwait.P();
	_remoteMutex.V();
    }
}

void
DVMManager::teardownDVM(char * path, DVMServiceMsg * msg)
{
    Assert(path != 0);
    Assert(msg != 0);
    
    _moTableMutex.P();
    
    unsigned slot = find(path);
    
    // If path is not found, method is invoked on state in slot 0,
    // which is DVM_SRV_EMPTY as needed.
    
    _moTable[slot].state = (DVMServiceState *)_moTable[slot].state->
	remoteRequestLeave(this,slot,msg);
	
    _moTableMutex.V();
}

void
DVMManager::remoteTeardownDone(char * path, DVMServiceMsg * msg)
{
    Assert(path != 0);
    Assert(msg != 0);
    
    _moTableMutex.P();

    unsigned slot = find(path);
    Assert(slot > 0);

    _moTable[slot].state =
	(DVMServiceState *)_moTable[slot].state->remoteLeaveDone(this,slot,msg);
    
    _moTableMutex.V();
    
    _remoteAwait.V();
}

void
DVMManager::DVMServiceTimeout(void * mgr, int slot)
{
    ((DVMManager *)mgr)->serviceTimeout((unsigned)slot);
}

void
DVMManager::serviceTimeout(unsigned slot)
{
    Assert(slot > 0);
    Assert(slot < MaxMOs);
    
    _moTableMutex.P();
    
    _moTable[slot].state =
	(DVMServiceState *)_moTable[slot].state->timeout(this,slot);
	
    _moTableMutex.V();
}

State *
DVM_SRV_EMPTY::joinLocal(DVMManager * mgr, unsigned slot)
{
    mgr->createNCDVM(slot);
    mgr->refMemobj(slot);
    return returnState(STATE_DVM_SRV_LOCAL);
}

State *
DVM_SRV_EMPTY::joinRemote(DVMManager * mgr, unsigned slot)
{
    mgr->createNCDVM(slot);
    mgr->refMemobj(slot);
    mgr->enableTimeout(slot);
    STATE_DVM_SRV_TOREMOTE->timeout(mgr,slot);
    return returnState(STATE_DVM_SRV_TOREMOTE);
}

State *
DVM_SRV_EMPTY::remoteRequestJoin(DVMManager * mgr, unsigned slot,
				 DVMServiceMsg * msg)
{
    Assert(msg->timestamp() > mgr->incoming(msg->source()));
    Assert((mgr->machines(slot) & msg->source()) == 0);
    mgr->setIncoming(msg->source(),msg->timestamp());
    mgr->createNCDVM(slot);
    mgr->setMachine(slot,msg->source());
    mgr->enqXmit(new DVMAckJoinMsg(mgr, slot, msg));
    return returnState(STATE_DVM_SRV_LOCAL);
}

State *
DVM_SRV_EMPTY::remoteRequestLeave(DVMManager * mgr,
				  unsigned slot, DVMServiceMsg * msg)
{
    Assert(msg->timestamp() == mgr->incoming(msg->source()));
    mgr->enqXmit(new DVMAckLeaveMsg(mgr, slot, msg));
    return returnState(STATE_DVM_SRV_EMPTY);
}

State *
DVM_SRV_EMPTY::remoteLeaveDone(DVMManager *, unsigned,
				    DVMServiceMsg *)
{
    Assert(NOTREACHED);
    return returnState(STATE_DVM_SRV_EMPTY);
}

State *
DVM_SRV_REMOTE::joinRemote(DVMManager * mgr, unsigned slot)
{
    mgr->refMemobj(slot);
    return returnState(STATE_DVM_SRV_REMOTE);
}

State *
DVM_SRV_REMOTE::leaveRemote(DVMManager * mgr, unsigned slot)
{
    if (mgr->unrefMemobj(slot) == 0) {
	mgr->enableTimeout(slot);
	STATE_DVM_SRV_TOEMPTY->timeout(mgr,slot);
	return returnState(STATE_DVM_SRV_TOEMPTY);
    }
    else
	return returnState(STATE_DVM_SRV_REMOTE);
}

State *
DVM_SRV_LOCAL::joinLocal(DVMManager * mgr, unsigned slot)
{
    mgr->refMemobj(slot);
    return returnState(STATE_DVM_SRV_LOCAL);
}

State *
DVM_SRV_LOCAL::remoteRequestJoin(DVMManager * mgr,
				 unsigned slot,
				 DVMServiceMsg * msg)
{
    Assert(((mgr->machines(slot) & msg->source()) != 0) ?
	   (msg->timestamp() == mgr->incoming(msg->source())) :
	   (msg->timestamp() > mgr->incoming(msg->source())));
    mgr->setIncoming(msg->source(),msg->timestamp());
    mgr->setMachine(slot,msg->source());
    mgr->enqXmit(new DVMAckJoinMsg(mgr, slot, msg));
    return returnState(STATE_DVM_SRV_LOCAL);
}

State *
DVM_SRV_LOCAL::leaveLocal(DVMManager * mgr, unsigned slot)
{
    if (mgr->unrefMemobj(slot) == 0 && mgr->machines(slot) == 0) {
	mgr->destroyDVM(slot);
	return returnState(STATE_DVM_SRV_EMPTY);
    }
    else
	return returnState(STATE_DVM_SRV_LOCAL);
}

State *
DVM_SRV_LOCAL::remoteRequestLeave(DVMManager * mgr,
				  unsigned slot,
				  DVMServiceMsg * msg)
{
    Assert(((mgr->machines(slot) & msg->source()) == 0) ?
	   (msg->timestamp() == mgr->incoming(msg->source())) :
	   (msg->timestamp() > mgr->incoming(msg->source())));
    
    // Terminate only if local has terminated
    if (mgr->refs(slot) == 0) {
	mgr->setIncoming(msg->source(),msg->timestamp());
	mgr->enqXmit(new DVMAckLeaveMsg(mgr, slot, msg));
	if (mgr->unsetMachine(slot,msg->source()) == 0) {
	    mgr->destroyDVM(slot);
	    return returnState(STATE_DVM_SRV_EMPTY);
	}
    }
    return returnState(STATE_DVM_SRV_LOCAL);
}

State *
DVM_SRV_TOREMOTE::remoteJoinDone(DVMManager * mgr,
				 unsigned slot, DVMServiceMsg * msg)
{
    Assert(msg->timestamp() == mgr->outgoing(msg->source()));
    mgr->nextStamp(msg->source());
    mgr->setRemoteId(slot,msg->monumber());
    mgr->disableTimeout(slot);
    return returnState(STATE_DVM_SRV_REMOTE);
}

State *
DVM_SRV_TOREMOTE::timeout(DVMManager * mgr, unsigned slot)
{
    mgr->enqXmit(new DVMJoinMsg(mgr, slot));
    return returnState(STATE_DVM_SRV_TOREMOTE);
}

State *
DVM_SRV_TOEMPTY::remoteLeaveDone(DVMManager * mgr, unsigned slot,
				    DVMServiceMsg * msg)
{
    Assert(msg->timestamp() == mgr->outgoing(msg->source()));
    mgr->nextStamp(msg->source());
    mgr->destroyDVM(slot);
    mgr->disableTimeout(slot);
    return returnState(STATE_DVM_SRV_EMPTY);
}

State *
DVM_SRV_TOEMPTY::timeout(DVMManager * mgr, unsigned slot)
{
    mgr->enqXmit(new DVMLeaveMsg(mgr, slot));
    return returnState(STATE_DVM_SRV_TOEMPTY);
}

DVMServiceMachine::DVMServiceMachine()
: StateMachine(5)
{
    NEWSTATE(DVM_SRV_EMPTY);
    NEWSTATE(DVM_SRV_LOCAL);
    NEWSTATE(DVM_SRV_REMOTE);
    NEWSTATE(DVM_SRV_TOEMPTY);
    NEWSTATE(DVM_SRV_TOREMOTE);
}

GroupTimer::GroupTimer(int max, long microSecondQuantum,
		       void * clientObject, PFVPIV clientFunction)
: _timeoutMutex(1)
{
    // Ideally, we would like to limit max based on speed of
    // processor, quantum etc.
    Assert(max > 0);
    _maxEvents = max;
    _eventTable = new char[_maxEvents];
    for (int i=0; i < _maxEvents; i++) {
	_eventTable[i] = 0;
    }
    
    _timeoutProcess =
	new PreemptableSystemProcess((PFV)GroupTimerEntry,
				     TheKernel->domain(), normal, (int)this);
    Assert(_timeoutProcess != 0);
    _timeoutProcess->setName("#GroupTimerTimeoutProcess");
    _timeoutProcess->ready();

    Assert(microSecondQuantum > 0);
    
    _microSecondQuantum = microSecondQuantum;

    Assert(clientObject != 0);
    Assert(clientFunction != 0);
    
    _clientObject = clientObject;
    _clientFunction = clientFunction;
}

GroupTimer::~GroupTimer()
{
    delete _eventTable;
    _timeoutProcess->die();
}

void
GroupTimer::GroupTimerEntry(int groupTimer)
{
    GroupTimer * timer = (GroupTimer *)groupTimer;
    Console() << thisProcess() << "(" << timer << ")\n" << eor;
    timer->timeoutProcessEntry();
}

void
GroupTimer::timeoutProcessEntry()
{
    for (;;) {
// 	// Timer wakes up four times as often as quantum requires so
// 	// that possible error is 25% only
// 	_timer.start(_microSecondQuantum >> 2); 
 	_timer.start(_microSecondQuantum); 
	_timer.await();
	_timeoutMutex.P();
	for(int i=0; i < _maxEvents; i++) {
	    switch(_eventTable[i]) {
	      // Not registered
	      case 0:
		break;
// 	      // Increment for next time
// 	      case 1:
// 	      case 2:
// 	      case 3:
// 	      case 4:
// 		_eventTable[i]++;
// 		break;
// 	      // Quantum expired, reset and call client. Client
// 	      // function should be free to disable timeout.
// 	      case 5:
	      case 1:
		_eventTable[i] = 1;
		_timeoutMutex.V();
		_clientFunction(_clientObject,i);
		_timeoutMutex.P();
		break;
	      default:
		Assert(NOTREACHED);
		break;
	    }
	}
	_timeoutMutex.V();
    }
}

void
GroupTimer::enable(int eventNum)
{
    Assert(eventNum >= 0);
    Assert(eventNum < _maxEvents);
    _timeoutMutex.P();
    _eventTable[eventNum] = 1;
    _timeoutMutex.V();
}

void
GroupTimer::disable(int eventNum)
{
    Assert(eventNum >= 0);
    Assert(eventNum < _maxEvents);
    _timeoutMutex.P();
    _eventTable[eventNum] = 0;
    _timeoutMutex.V();
}

DVMServiceMachine * TheDVMServiceMachine = 0;
DVMServiceState * STATE_DVM_SRV_EMPTY = 0;
DVMServiceState * STATE_DVM_SRV_LOCAL = 0;
DVMServiceState * STATE_DVM_SRV_TOEMPTY = 0;
DVMServiceState * STATE_DVM_SRV_REMOTE = 0;
DVMServiceState * STATE_DVM_SRV_TOREMOTE = 0;

#define Debug BlackHole
