#ifdef __GNUC__
#pragma implementation
#endif

#define DEBUG
/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "OutputStream.h"
#include "Assert.h"
#include "DVMMessage.h"
#include "DVMCommunicator.h"
#include "Semaphore.h"
#include "String.h"
#include "DVMManager.h"
#include "NewAndDelete.h"
#include "OneCopyDVM.h"
#include "OneCopyDVMCache.h"


#ifdef __GNUC__
#pragma implementation
#endif

inline unsigned
FloorUnsigned( unsigned int i )
{
    return( i & (~ ( sizeof(unsigned) - 1 )) );
}

inline unsigned
CeilingUnsigned(unsigned i)
{
    return FloorUnsigned(i + sizeof(unsigned) - 1);
}

inline unsigned
MultipleUnsigned(unsigned i)
{
    return CeilingUnsigned(i)/sizeof(unsigned);
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

// 
// Calculate fragment start and size. Move this to DVMCommunicator later.
// 

static inline void
etherFragment(unsigned fragnum, unsigned& start, unsigned& size)
{
    switch (fragnum) {
      case 0x1:
	start = 0;
	// end = 349;
	size = 350;
	break;
      case 0x2:
	start = 350;
	// end = 699;
	size = 350;
	break;
      case 0x4:
	start = 700;
	// end = 1023;
	size = 324;
	break;
      default:
	Assert(NOTREACHED);
	break;
    }
}

// Used by DVMsg::inspect

static struct {
    const char * name;
} MessageNames[] = {
    "NULL",			// index 0
    "GP",
    "HP",
    "AP",
    "J",
    "AJ",
    "L",
    "AL",
    "OH",
    "GR",
    "HR",
    "GU",
    "HU",
    "AU",
    "I",
    "AI",
    "INF"			// index greater than existing messages
};

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

OutputStream&
operator<<(OutputStream& os, DVMMsg * msg)
{
    os << "DVMMsg[" << asHex((int)msg) << "](";
    msg->inspect(os);
    os << ")";
    return os;
}

void
DVMMsg::inspect(OutputStream& os)
{
    os << MessageNames[type()].name << "," << destination() << ","
       << source() << "," << timestamp() ;
}

// 
// Link defines pointer equality for comparison. Message comparsion,
// however is intended to identify acks. Messages that do not wait for
// acks should never be compared. We make sure of that by undefining
// compare for such messages.
// 

int
DVMMsg::compare(Link *)
{
    Assert(NOTREACHED);
    return 0;
}

//
// Used for checks in ::receive. Will fail for messages that do not
// wait for receive
//

DVMMsgType
DVMMsg::rcvType()
{
    Assert(NOTREACHED);
    return 0;
}

// 
// Timeout for messages returns a copy with (_sem == 0). A proper copy
// can be made in the subclass.
// Timeout messages themselves, acks etc. do not timeout. Make sure
// that timeout is not called for such messages.
// 

DVMMsg *
DVMMsg::timeout()
{
    Assert(NOTREACHED);
    return 0;
}

DVMMsg *
DVMMsg::transmit()
{
    Debug << "CacheXmit(" << this << ")\n" << eor;
    unsigned _length = length()+TheDVMCommunicator->headerSize();
    if (_length < 64) _length = 64; // SPARC HACK
    unsigned * packet = new unsigned[(_length+sizeof(unsigned)-1)/
				     sizeof(unsigned)];
    rawCopyTo(packet+MultipleUnsigned(TheDVMCommunicator->headerSize()));
    TheDVMCommunicator->networkSend(packet,_length,_destination,_type); 
    delete packet;

    Assert(_sem == 0);
    DVMMsg * ret = this;
    if (_sem == 0) {
	delete this;
	ret = 0;
    }
    return ret;
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

int
DVMCacheMsg::compare(Link * l)
{
    Assert(NOTREACHED);
    Assert(_type != AckPage);
    
    switch (((DVMMsg *)l)->type()) {
      case GetPage:
	Console() << "GP COMPARED\n" << eor;
      case AckPage:
      case HerePage:
	return (((DVMCacheMsg *)l)->unit() == _unit)
	    && (((DVMCacheMsg *)l)->fragment() == _fragment);
	break;
      default:
	return 0;
	break;
    }
}

void
DVMCacheMsg::inspect(OutputStream& os)
{
    DVMMsg::inspect(os);
    os << "," << _unit << "," << _fragment;
}

DVMCacheMsg *
DVMCacheMsg::receive(DVMCacheMsg * msg)
{
    Assert(NOTREACHED);
    Assert(msg != 0);
    Assert(rcvType() == msg->type());
    Debug << "Rcv(" << msg << ")\n" << eor;
    Assert(msg->destination() == _source);
    Assert(msg->source() == _destination);

    cacheCopyFrom(msg);
    Assert(_sem != 0);
    _sem->V();
    DVMCacheMsg * ret = response();
    delete this;
    return ret;
}



//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

void
DVMGetPageMsg::cacheCopyFrom(DVMCacheMsg * msg)
{
    Assert(NOTREACHED);
    Assert(msg != 0);
    Assert(msg->type() == HerePage);
    Assert(msg->fragment() == _fragment);

    unsigned fstart = 0;
    unsigned fsize = 0;
    
    etherFragment(_fragment, fstart, fsize);
    
    unsigned * page = msg->address();
    for(unsigned i=0; i < fsize; i++)
	_address[fstart+i] = page[i];
}

// DVMMsg *
// DVMGetPageMsg::transmit()
// {
//     if (0x1 == _fragment) {
// 	return DVMMsg::transmit();
//     }
//     else {
// 	DVMMsg * ret = this;
// 	if (_sem == 0) {
// 	    delete this;
// 	    ret = 0;
// 	}
// 	return ret;
//     }
// }

unsigned
DVMGetPageMsg::length()
{
    return CeilingUnsigned(sizeof(DVMGetPageMsg));
}

void
DVMGetPageMsg::rawCopyTo(unsigned * packet)
{
    DVMGetPageMsg * msg = (DVMGetPageMsg *)packet;
    *msg = *this;
}

// Same as ::transmit above

DVMMsg *
DVMGetPageMsg::timeout()
{
    Assert(NOTREACHED);
    Assert(_sem != 0);
    if (0x1 == _fragment)
	return new DVMGetPageMsg(_destination,_source,_timestamp,
				 _unit,_address,_fragment);
    else
	return 0;
}

// When GetPage receives a HerePage
DVMCacheMsg *
DVMGetPageMsg::response()
{
    return new DVMAckPageMsg(_destination,_source,_timestamp,
			     _unit,_fragment);
}

void
DVMGetPageMsg::action(NCDVMCache * cache)
{
    Assert(cache != 0);
    cache->stateAction(&NCDVMState::remoteWrite,this);
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

void
DVMHerePageMsg::rawCopyTo(unsigned * packet)
{
    DVMHerePageMsg * msg = (DVMHerePageMsg *)packet;
    *msg = *this;

    unsigned fstart = 0;
    unsigned fsize = 0;
    
    etherFragment(_fragment, fstart, fsize);
    
    packet += MultipleUnsigned(sizeof(DVMHerePageMsg));
    for(unsigned i=0; i < fsize; i++)
	packet[i] = _address[i];
}


unsigned
DVMHerePageMsg::length()
{
    Assert(sizeof(DVMCacheMsg) == sizeof(DVMHerePageMsg));
    return 1400+CeilingUnsigned(sizeof(DVMHerePageMsg));
}

DVMMsg *
DVMHerePageMsg::timeout()
{
    Assert(NOTREACHED);
    Assert(_sem != 0);
    return new DVMHerePageMsg(_destination,_source,_timestamp,
			      _unit,_address,_fragment);
}


// When I receive an AckPage
DVMCacheMsg *
DVMHerePageMsg::response()
{
    return 0;
}

// When I am received, but have no waiter
DVMCacheMsg *
DVMHerePageMsg::selfack()
{
    return new DVMAckPageMsg(_source,_destination,_timestamp,
			     _unit,_fragment);
}

void
DVMHerePageMsg::action(NCDVMCache * cache)
{
    Assert(cache != 0);
    cache->stateAction(&NCDVMState::hereRemoteWrite,this);
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


unsigned
DVMAckPageMsg::length()
{
    return CeilingUnsigned(sizeof(DVMAckPageMsg));
}

void
DVMAckPageMsg::rawCopyTo(unsigned * packet)
{
    DVMAckPageMsg * msg = (DVMAckPageMsg *)packet;
    *msg = *this;
}

DVMCacheMsg *
DVMAckPageMsg::response()
{
    return 0;
}

DVMCacheMsg *
DVMAckPageMsg::receive(DVMCacheMsg *)
{
    Assert(NOTREACHED);
    return 0;
}

void
DVMAckPageMsg::action(NCDVMCache * cache)
{
    Assert(cache != 0);
    cache->stateAction(&NCDVMState::remoteWriteDone,this);
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

unsigned
DVMOwnerHintMsg::length()
{
    return CeilingUnsigned(sizeof(DVMOwnerHintMsg));
}

void
DVMOwnerHintMsg::rawCopyTo(unsigned * packet)
{
    DVMOwnerHintMsg * msg = (DVMOwnerHintMsg *)packet;
    *msg = *this;
}

DVMCacheMsg *
DVMOwnerHintMsg::response()
{
    return 0;
}

DVMCacheMsg *
DVMOwnerHintMsg::receive(DVMCacheMsg *)
{
    Assert(NOTREACHED);
    return 0;
}

void
DVMOwnerHintMsg::action(NCDVMCache * cache)
{
    Assert(cache != 0);
    cache->stateAction(&NCDVMState::ownerHint,this);
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

void
DVMGetReadMsg::cacheCopyFrom(DVMCacheMsg * msg)
{
    Assert(NOTREACHED);
    Assert(msg != 0);
    Assert(msg->type() == HereRead);
    Assert(msg->fragment() == _fragment);

    unsigned fstart = 0;
    unsigned fsize = 0;
    
    etherFragment(_fragment, fstart, fsize);
    
    unsigned * page = msg->address();
    for(unsigned i=0; i < fsize; i++)
	_address[fstart+i] = page[i];
}

unsigned
DVMGetReadMsg::length()
{
    return CeilingUnsigned(sizeof(DVMGetReadMsg));
}

void
DVMGetReadMsg::rawCopyTo(unsigned * packet)
{
    DVMGetReadMsg * msg = (DVMGetReadMsg *)packet;
    *msg = *this;
}

// Same as ::transmit above

DVMMsg *
DVMGetReadMsg::timeout()
{
    Assert(NOTREACHED);
    Assert(_sem != 0);
    if (0x1 == _fragment)
	return new DVMGetReadMsg(_destination,_source,_timestamp,
				 _unit,_address,_fragment);
    else
	return 0;
}

// When GetRead receives a HereRead
DVMCacheMsg *
DVMGetReadMsg::response()
{
    Assert(NOTREACHED);
    return 0;
}

void
DVMGetReadMsg::action(NCDVMCache * cache)
{
    Assert(cache != 0);
    ((RCDVMCache *)cache)->stateAction(&RCDVMState::remoteRead,this);
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

void
DVMHereReadMsg::rawCopyTo(unsigned * packet)
{
    DVMHereReadMsg * msg = (DVMHereReadMsg *)packet;
    *msg = *this;

    unsigned fstart = 0;
    unsigned fsize = 0;
    
    etherFragment(_fragment, fstart, fsize);
    
    packet += MultipleUnsigned(sizeof(DVMHereReadMsg));
    for(unsigned i=0; i < fsize; i++)
	packet[i] = _address[i];
}


unsigned
DVMHereReadMsg::length()
{
    Assert(sizeof(DVMCacheMsg) == sizeof(DVMHereReadMsg));
    return 1400+CeilingUnsigned(sizeof(DVMHereReadMsg));
}

DVMMsg *
DVMHereReadMsg::timeout()
{
    Assert(NOTREACHED);
    Assert(_sem != 0);
    return new DVMHereReadMsg(_destination,_source,_timestamp,
			      _unit,_address,_fragment);
}


// When I receive an AckRead
DVMCacheMsg *
DVMHereReadMsg::response()
{
    return 0;
}

// When I am received, but have no waiter
DVMCacheMsg *
DVMHereReadMsg::selfack()
{
    Assert(NOTREACHED);
    return 0;
}

void
DVMHereReadMsg::action(NCDVMCache * cache)
{
    Assert(cache != 0);
    ((RCDVMCache *)cache)->stateAction(&RCDVMState::hereRemoteRead,this);
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

unsigned
DVMGetUpMsg::length()
{
    return CeilingUnsigned(sizeof(DVMGetUpMsg));
}

void
DVMGetUpMsg::rawCopyTo(unsigned * packet)
{
    DVMGetUpMsg * msg = (DVMGetUpMsg *)packet;
    *msg = *this;
}

DVMCacheMsg *
DVMGetUpMsg::response()
{
    return 0;
}

DVMCacheMsg *
DVMGetUpMsg::receive(DVMCacheMsg *)
{
    Assert(NOTREACHED);
    return 0;
}

void
DVMGetUpMsg::action(NCDVMCache * cache)
{
    Assert(cache != 0);
    ((RCDVMCache *)cache)->stateAction(&RCDVMState::remoteUpgrade,this);
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

unsigned
DVMHereUpMsg::length()
{
    return CeilingUnsigned(sizeof(DVMHereUpMsg));
}

void
DVMHereUpMsg::rawCopyTo(unsigned * packet)
{
    DVMHereUpMsg * msg = (DVMHereUpMsg *)packet;
    *msg = *this;
}

DVMCacheMsg *
DVMHereUpMsg::response()
{
    return 0;
}

DVMCacheMsg *
DVMHereUpMsg::receive(DVMCacheMsg *)
{
    Assert(NOTREACHED);
    return 0;
}

void
DVMHereUpMsg::action(NCDVMCache * cache)
{
    Assert(cache != 0);
    ((RCDVMCache *)cache)->stateAction(&RCDVMState::hereRemoteUpgrade,this);
}


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

unsigned
DVMAckUpMsg::length()
{
    return CeilingUnsigned(sizeof(DVMAckUpMsg));
}

void
DVMAckUpMsg::rawCopyTo(unsigned * packet)
{
    DVMAckUpMsg * msg = (DVMAckUpMsg *)packet;
    *msg = *this;
}

DVMCacheMsg *
DVMAckUpMsg::response()
{
    return 0;
}

DVMCacheMsg *
DVMAckUpMsg::receive(DVMCacheMsg *)
{
    Assert(NOTREACHED);
    return 0;
}

void
DVMAckUpMsg::action(NCDVMCache * cache)
{
    Assert(cache != 0);
    ((RCDVMCache *)cache)->stateAction(&RCDVMState::remoteUpgradeDone,this);
}


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

unsigned
DVMInvMsg::length()
{
    return CeilingUnsigned(sizeof(DVMInvMsg));
}

void
DVMInvMsg::rawCopyTo(unsigned * packet)
{
    DVMInvMsg * msg = (DVMInvMsg *)packet;
    *msg = *this;
}

DVMCacheMsg *
DVMInvMsg::response()
{
    return 0;
}

DVMCacheMsg *
DVMInvMsg::receive(DVMCacheMsg *)
{
    Assert(NOTREACHED);
    return 0;
}

void
DVMInvMsg::action(NCDVMCache * cache)
{
    Assert(cache != 0);
    ((RCDVMCache *)cache)->stateAction(&RCDVMState::invalidate,this);
}


//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

unsigned
DVMAckInvMsg::length()
{
    return CeilingUnsigned(sizeof(DVMAckInvMsg));
}

void
DVMAckInvMsg::rawCopyTo(unsigned * packet)
{
    DVMAckInvMsg * msg = (DVMAckInvMsg *)packet;
    *msg = *this;
}

DVMCacheMsg *
DVMAckInvMsg::response()
{
    return 0;
}

DVMCacheMsg *
DVMAckInvMsg::receive(DVMCacheMsg *)
{
    Assert(NOTREACHED);
    return 0;
}

void
DVMAckInvMsg::action(NCDVMCache * cache)
{
    Assert(cache != 0);
    ((RCDVMCache *)cache)->stateAction(&RCDVMState::ackInvalidate,this);
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


// Msg needs Mgr and vice versa, solve that problem by making this
// ctor non-inline

DVMServiceMsg::DVMServiceMsg(DVMMsgType type, DVMManager * mgr, unsigned slot,
			     DVMServiceMsg * msg)   // if msg, msg is reply
: DVMMsg(0,type,				    // type
	 msg?msg->source():mgr->machines(slot), // destination
	 mgr->localId(),			    // source
	 msg?msg->timestamp():mgr->timestamp(mgr->machines(slot))) // timestamp
{
    _monumber = slot;
    _top = 0;
    _rawLength = 0;
}

void
DVMServiceMsg::inspect(OutputStream& os)
{
    DVMMsg::inspect(os);
    os << "," << _monumber << "," << namecopy;
}

DVMServiceMsg::~DVMServiceMsg()
{
    delete namecopy;
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

DVMJoinMsg::DVMJoinMsg(DVMManager * mgr, unsigned slot)
: DVMServiceMsg (Join,mgr,slot)
{
    char * moname = mgr->name(slot);
    Assert(moname != 0);
    namecopy = new char[strlen(moname)+1];
    strcpy(namecopy,moname);
}

void
DVMJoinMsg::rawCopyTo(unsigned * packet)
{
    DVMJoinMsg * msg = (DVMJoinMsg *)packet;
    *msg = *this;
    packet += MultipleUnsigned(sizeof(DVMJoinMsg));
    strcpy((char *)packet,namecopy);
}

unsigned
DVMJoinMsg::length()
{
    return CeilingUnsigned(sizeof(DVMJoinMsg))+strlen(namecopy)+1;
}

void
DVMJoinMsg::action(DVMManager * mgr)
{
    Debug << "DVMJoinMsg::action\n" << eor;
    mgr->setupDVM(moname(), this);
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

DVMAckJoinMsg::DVMAckJoinMsg(DVMManager * mgr, unsigned slot,
			     DVMServiceMsg * msg)
: DVMServiceMsg (AckJoin,mgr,slot,msg)
{
    char * moname = mgr->name(slot);
    Assert(moname != 0);
    namecopy = new char[strlen(moname)+1];
    strcpy(namecopy,moname);
}

void
DVMAckJoinMsg::rawCopyTo(unsigned * packet)
{
    DVMAckJoinMsg * msg = (DVMAckJoinMsg *)packet;
    *msg = *this;
    packet += MultipleUnsigned(sizeof(DVMAckJoinMsg));
    strcpy((char *)packet,namecopy);
}

unsigned
DVMAckJoinMsg::length()
{
    return CeilingUnsigned(sizeof(DVMAckJoinMsg))+strlen(namecopy)+1;
}

void
DVMAckJoinMsg::action(DVMManager * mgr)
{
    Debug << "DVMAckJoinMsg::action\n" << eor;
    mgr->remoteSetupDone(moname(), this);
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
DVMLeaveMsg::DVMLeaveMsg(DVMManager * mgr, unsigned slot)
: DVMServiceMsg(Leave,mgr,slot)
{
    char * moname = mgr->name(slot);
    Assert(moname != 0);
    namecopy = new char[strlen(moname)+1];
    strcpy(namecopy,moname);
   _monumber = mgr->remoteId(slot);
}

void
DVMLeaveMsg::rawCopyTo(unsigned * packet)
{
    DVMLeaveMsg * msg = (DVMLeaveMsg *)packet;
    *msg = *this;
    packet += MultipleUnsigned(sizeof(DVMLeaveMsg));
    strcpy((char *)packet,namecopy);
}

unsigned
DVMLeaveMsg::length()
{
    return CeilingUnsigned(sizeof(DVMLeaveMsg))+strlen(namecopy)+1;
}

void
DVMLeaveMsg::action(DVMManager * mgr)
{
    Debug << "DVMLeaveMsg::action\n" << eor;
    mgr->teardownDVM(moname(), this);
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

DVMAckLeaveMsg::DVMAckLeaveMsg(DVMManager * mgr, unsigned slot,
			       DVMServiceMsg * msg)
: DVMServiceMsg(AckLeave,mgr,slot,msg)
{
    char * moname = msg->moname();
    Assert(moname != 0);
    namecopy = new char[strlen(moname)+1];
    strcpy(namecopy,moname);
   _monumber = mgr->remoteId(slot);
}

void
DVMAckLeaveMsg::rawCopyTo(unsigned * packet)
{
    DVMAckLeaveMsg * msg = (DVMAckLeaveMsg *)packet;
    *msg = *this;
    packet += MultipleUnsigned(sizeof(DVMAckLeaveMsg));
    strcpy((char *)packet,namecopy);
}

unsigned
DVMAckLeaveMsg::length()
{
    return CeilingUnsigned(sizeof(DVMAckLeaveMsg))+strlen(namecopy)+1;
}

void
DVMAckLeaveMsg::action(DVMManager * mgr)
{
    Console() << "::DVMAckLeaveMsg action\n" << eor;
    mgr->remoteTeardownDone(moname(),this);
}
