#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "OneCopyDVMCache.h"
#include "OneCopyDVM.h"
#include "DVMAccessType.h"
#include "DVMManager.h"
#include "ThisProcess.h"


#ifdef __GNUC__
#pragma implementation
#endif

OneCopyDVMCache::OneCopyDVMCache( MemoryObject * memoryObject,
	unsigned int nunits, int copy, int originId )
	: ( memoryObject, nunits, 1 )
{
    _manager = 0;
    int id = copy ? originId : 0;
    _machines = new int [ _numberOfUnits ];
    for (int i=0; i < _numberOfUnits; i++) {
	_machines[i]=id;
    }
    State * state = copy ? STATE_DVM_NPD : STATE_DVM_NPDC;
    for (i = 0; i < _numberOfUnits; i++) {
 	_pageState[i] = state;
    }
}

// Send out a page and record the new owner
void
OneCopyDVMCache::pagesToNet(int requester, unsigned from, unsigned to)
{
    if (to == 0) to = from;
    Assert( from <= to && to < _numberOfUnits );
    Assert(to == from);
    
    _manager->send(requester,from,getPlist(from,to));
    _machines[from]=requester;
}

// Fetch pages from the net and record ourselves as the new owner
void
OneCopyDVMCache::pagesFromNet(unsigned from, unsigned to)
{
    if (to == 0) to = from;
    Assert( from <= to && to < _numberOfUnits );
    Assert(to == from);
    
    putPlist(_manager->rcv(_machines[from],from),from,to);
    _machines[from]=_manager->id();
}

// Handle a remote event by dispatching it to the state
void
OneCopyDVMCache::remoteCache(unsigned unit, DVMAccessType attemptedAccess,
			     int requester)
{
    Debug << this << "::remoteCache( unit: " << unit << ", attemptedAccess: "
	<< attemptedAccess << ", requester" << requester << " )\n" << eor;
    Assert( isIn( unit ) );
    
    preFault(unit);
    
    if (attemptedAccess == RemoteRead)
	_pageState[unit] = pageState(unit)->remoteRead(this, unit, requester);
    else
	_pageState[unit] = pageState(unit)->remoteWrite(this, unit, requester);
    
    _machines[unit] = requester;
    
    postFault(unit);
}

/*
 * Free data structures and paus. Assume that the MemoryObject has
 * called appropriate methods and there are no references anymore to
 * any pages that were in here. Also assume that dirty pages do not
 * matter.
 */
OneCopyDVMCache::~OneCopyDVMCache()
{
    delete _machines;
}

#include "ClassInstaller.h"
DEFINECLASS("OneCopyDVMCache",OneCopyDVMCache,SimpleVMCache,0,0);
