#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "SimpleVM.h"
#include "MemoryObjectCache.h"
#include "OutputStream.h"
#include "PhysicallyAddressableUnit.h"


#ifdef __GNUC__
#pragma implementation
#endif

// FaultErrorCode
// SimpleVMState::code()
// {
//     // Return fatal code just to make sure...
//     // Really, I should return something like InvalidCode...
//     return AccessOfUnmanagedMemory;
// }

State *
VM_NPDC::localRead(MemoryObjectCache * cache, unsigned unit,
		   AddressTranslation * adt)
{
    cache->pagesFromStore(unit);
    cache->fromDisk(unit);
    cache->unitToPau(unit)->setProtection(ReadOnly);
    cache->postPageIn( unit );
//     cache->nprotect(adt,ReadOnly,unit);
    cache->translatable(unit,unit,ReadOnly,adt);
    return returnState(STATE_VM_P);
}

State *
VM_NPDC::localWrite(MemoryObjectCache * cache, unsigned unit,
		    AddressTranslation * adt)
{
    cache->pagesFromStore(unit);
    cache->unitToPau(unit)->setProtection(ReadWrite);
    cache->fromDisk(unit);
//     cache->nprotect(adt,ReadWrite,unit);
    cache->translatable(unit,unit,ReadWrite,adt);
    return returnState(STATE_VM_P);
}

State *
VM_NPDC::pageOut(MemoryObjectCache * cache, unsigned unit)
{
    Assert(NOTREACHED);
    return 0;
}

FaultErrorCode
VM_NPDC::code()
{
    return MissingPhysicalMemory;
}

State *
VM_P::localRead(MemoryObjectCache * cache, unsigned unit,
		AddressTranslation * adt)
{
    if (cache->unitToPau(unit)->protection() < ReadOnly) {
	if (cache->nprotect(adt,ReadOnly,unit) == 0)
	    cache->translatable(unit,unit,ReadOnly,adt);
    }
    else {
	cache->translatable(unit,unit,ReadOnly,adt);
    }
	    
    return returnState(STATE_VM_P);
}

State *
VM_P::localWrite(MemoryObjectCache * cache, unsigned unit,
		 AddressTranslation * adt)
{
    if (cache->unitToPau(unit)->protection() < ReadWrite) {
	if (cache->nprotect(adt,ReadWrite,unit) == 0)
	    cache->translatable(unit,unit,ReadWrite,adt);
    }
    else {
	cache->translatable(unit,unit,ReadWrite,adt);
    }
    return returnState(STATE_VM_P);
}

State *
VM_P::pageOut(MemoryObjectCache * cache, unsigned unit)
{
    cache->untranslatable(unit,unit,NoAccess);
    cache->saveModified(unit);
    return returnState(STATE_VM_NPDC);
}

FaultErrorCode
VM_P::code()
{
    return MissingAddressTranslationMapping;
}

SimpleVMMachine::SimpleVMMachine()
: StateMachine(2)
{
    NEWSTATE( VM_NPDC );
    NEWSTATE( VM_P );
}

SimpleVMMachine * TheSimpleVMMachine = 0;

SimpleVMState * STATE_VM_NPDC = 0;
SimpleVMState * STATE_VM_P = 0;
