#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
// THIS FILE INCLUDES A TEMPLATE FOR DEFINING AND USING STATE MACHINES.

#include "Assert.h"
#include "State.h"


#ifdef __GNUC__
#pragma implementation
#endif

StateMachine::StateMachine(int numStates)
{
    _newStatesStarted = 0;
    _numStates = numStates+1;
    _states = new State * [ _numStates ];
    for (int i = 0; i < _numStates; i++)
	_states[i] = 0;
}

StateMachine::~StateMachine()
{
    Assert(NOTREACHED);

// Someday, when the kernel destructs
//     for (int i=0; i < _numStates; i++)
//     {
// 	delete _states[i];
// 	_states[i] = 0;
//     }
//     delete _states;

}

void
StateMachine::newState(State * s)
{
    Assert(s != 0);
    Assert(s->number() == 0);
    for(unsigned i=1; i < _numStates; i++)
	if (_states[i] == 0)
	    break;
    Assert(i < _numStates);
    s->setNumber(i);
    _states[i] = s;
    _newStatesStarted = 1;
}

// place the state in its place
void
StateMachine::replaceState(State * s)
{
    Assert(s != 0);
    Assert(_newStatesStarted == 0);
    Assert( s->number() > 0 );
    Assert( s->number() < _numStates );
    Assert( _states[s->number()] == 0 );
    _states[s->number()] = s;
}

// return the State with identical number from this machine
State *
StateMachine::returnState(State * s)
{
    Assert(s != 0);
    Assert(s->number() < _numStates);
    return _states[s->number()];
}

//%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

State *
State::error()
{
    Assert(NOTREACHED);
    return 0;
}

State *
State::null()
{
    return returnState(this);
}

State::State(StateMachine * m)
:_delegatee(0), _number(0), _machine(m)
{
    Assert(m != 0);
};

State::State(StateMachine * m, State * d)
:_delegatee(d), _number(d->number()), _machine(m)
{
    Assert(d != 0);
    Assert(m != 0);
};

State::~State()
{
    Assert(NOTREACHED);
}
