#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * DummyCache.cc
 */

#include "Assert.h"
#include "Barrier.h"
#include "CacheAsync.h"
#include "DummyCache.h"
#include "MemoryObject.h"
#include "MemoryObjectCache.h"
#include "MemorySimpleInstruments.h"
#include "Raid.h"

#ifdef __GNUC__
#pragma implementation
#endif

DummyCache::DummyCache( MemoryObjectCache * moc )
	: Cache( moc )
{
	initDebug();
	Raids( RaidConstructor ) << this << "::DummyCache( " 
		<< moc << " )\n" << eor;

	_numberOfUnits = moc->numberOfUnits();
}

int DummyCache::basicRead ( unsigned int startUnit, unsigned int count, char * addr,
			    IOAsyncClosure * c )
{
	_stats._basicRead.add( count );
	return moRead( startUnit, count, addr, c );
}

int DummyCache::basicWrite( unsigned int startUnit, unsigned int count, char * addr,
			    IOAsyncClosure * c )
{
	Assert( NOTREACHED );

	Raids( RaidProtected ) << this << "::basicWrite( " 
		<< startUnit << ", " 
		<< count << ", " << asAddress( addr ) << ", "
		<< asAddress( c )
		<< " )\n" << eor;

	_stats._basicWrite.add( count );
	int noOfUnits = moWrite( startUnit, count, addr, 0, c );

	return noOfUnits;
}

unsigned DummyCache::basicSetNumberOfUnits( unsigned int noOfUnits )
{
	return noOfUnits;
}

unsigned int DummyCache::setNumberOfUnits( unsigned int noOfUnits )
{
	_numberOfUnits = noOfUnits;
	return noOfUnits;
}

int DummyCache::exists( unsigned unit )
{
	return 0;
}

int DummyCache::read( unsigned int startUnit, unsigned int count, char * addr, 
	    	      IOAsyncClosure * c )	
{
Debug<<this<<"::read("<<startUnit<<", "
                 << count << ", " << asAddress( addr ) << ", "
                 << asAddress( c ) << " )\n" << eor;
	_stats._read.add( count );
	int result = moRead( startUnit, count, addr, c );

	return result;
}

int DummyCache::write( unsigned int startUnit, unsigned int count, CacheUnitInfo * info, 
		       char * addr, IOAsyncClosure * c )
{
	Raids( RaidMethod ) << this << "::write( " << startUnit << ", " 
		<< count << ", " << asAddress( info ) << ", " 
		<< asAddress( addr ) << ", " << asAddress( c )
		<< " )\n" << eor;

	_stats._write.add( count );
	writeMOModified( startUnit, count, info, addr, 0, c );

	return count;
}

int DummyCache::rawWrite( unsigned int startUnit, unsigned int count, CacheUnitInfo * info, 
		          char * addr, IOAsyncClosure * c )
{
	Raids( RaidMethod ) << this << "::write( " << startUnit << ", " 
		<< count << ", " << asAddress( info ) << ", " 
		<< asAddress( addr ) << ", " << asAddress( c )
		<< " )\n" << eor;

	_stats._rawWrite.add( count );
	writeMOModified( startUnit, count, info, addr, 1, c );

	return count;
}


int DummyCache::dirty()
{
	return 0;
}

#include "ClassInstaller.h"
DEFINECLASS("DummyCache",DummyCache,Cache,0,0);
