#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David A. Ladd (ladd@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "AddressTranslation.h"
#include "Assert.h"
#include "FutureMemoryObjectCache.h"
#include "OutputStream.h"
#include "Semaphore.h"


#ifdef __GNUC__
#pragma implementation
#endif

	/* I'll come back and fix this one later...  --Vince*/


void
SyncBlock::fault()
{
	wait->P();
	Debug << "Did P\n" << eor;
	wait->V();
	Debug << "Did V\n" << eor;
}

FutureMemoryObjectCache::FutureMemoryObjectCache( MemoryObject * obj, 
	unsigned int length )
	: ( obj, length )
{
	Debug << this << "::FutureMemoryObjectCache( " << obj << ", "
		<< length << " )\n" << eor;
	syncTable = new SyncBlock * [length];
	for (int i = 0; i < length; i++) {
		syncTable[i] = 0;
	}
	invals = 0;
	faults = 0;
}

FutureMemoryObjectCache::~FutureMemoryObjectCache()
{
	Debug << this << "::~FutureMemoryObjectCache()\n" << eor;
#ifdef FUTURESTATS
	Console() << this << ": " invals << " futures, " << faults
		<< " faults\n" << eor;
#endif
	delete syncTable;
}

int
FutureMemoryObjectCache::invalPage( unsigned int offset )
{
	Debug << this << "::invalPage( offset " << asHex( offset ) << ")\n"
		<< eor;

	unsigned int unit = offsetToUnit( (Offset) offset);

	/*
	 * Release the unit from the cache.
	 */
	//PhysicallyAddressableUnit * pau =
					    release( unit );
	// shouldnt I deallocate the unit here....

	/*
	 * set up semaphore
	 */
	Debug << this << "::invalPage : inval'd, setting up SyncBlock\n" << eor;
	if ( syncTable[unit] ) {
		delete syncTable[unit];
	}

	syncTable[unit] = new SyncBlock;
	Assert ( syncTable[unit] != 0 );

	invals++;

	Debug << this << "::invalPage : return\n" << eor;
	return( 0 );
}

int
FutureMemoryObjectCache::mergePage( unsigned int /*home*/, unsigned int /*offset*/)
{
	Console() << this << "::mergePage: BROKEN!!\n" << eor;

//	unsigned int map_unit = offsetToUnit ( (Offset) offset);
//	unsigned int base_unit = offsetToUnit ( (Offset) home);
	
			//   used to be:
			//   map(map_unit, physicalAddress(base_unit) );
			//   
			//   thought: could I copy PTE?

//	faultSem.P();

//	PTE * mapPte = getPTEFor ( map_unit );
//	if( mapPte == 0 )
//		mapPte = buildMappings( map_unit );

//	Assert ( mapPte != 0 );

//	PTE * basePte = getPTEFor( base_unit );

//	Assert ( basePte != 0 && basePte->validated() );

//	mapPte->validate( basePte->physicalAddress() );

//	faultSem.V();
			// for now, let this new mapping fault in

//	Assert (syncTable[map_unit] != 0);


//	Console->debug( "FutureMemoryObjectCache::mergePage() - WAKING UP [%d]\n", 
//			map_unit);
//	syncTable[map_unit]->merge();

	return (0);
}

PhysicalMemoryChain
FutureMemoryObjectCache::cache( unsigned int unit,
	ProtectionLevel requestedProtection, FaultErrorCode & code )
{
	Debug << this << "::cache( unit: " << unit << " )\n" << eor;

	Assert( isIn ( unit ) );

	if ( syncTable[unit] ) {
		faults++;
		Debug << this << "::cache() - WAITING [" << unit << "]\n"
			<< eor;
		syncTable[unit]->fault();
		Debug << this << "::cache() - WAKING\n" << eor;
	}
	else {
		Debug << this << "::cache() - Using parent\n" << eor;
	}

	return( PagedMemoryObjectCache::cache( unit, requestedProtection,
		code ) );	
}


#include "ClassInstaller.h"
DEFINECLASS("FutureMemoryObjectCache",FutureMemoryObjectCache,PagedMemoryObjectCache,0,0);
