#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * MemoryObjectAsync.cc 
 */

#include "MemoryObject.h"
#include "MemoryObjectAsync.h"

#ifdef __GNUC__
#pragma implementation
#endif



IOAsyncClosure::IOAsyncClosure()
  : _rightShift( 0 ) 
{
    
}

void 
IOAsyncClosure::activate() 
{ 
   (*_func)( this ); 
}

void 
IOAsyncClosure::activate( int result ) 
{ 
  _result = ( result >> _rightShift ) ;
  (*_func)( this ); 
}

void
SumResultIOAsyncClosure::func( SumResultIOAsyncClosure * c )
{
	c->_parent->activate( c->_result + c->_sum );
	delete c;
}

void
ReleaseIOAsyncClosure::func( ReleaseIOAsyncClosure * c ) 
{
	c->_memoryObject->release( c->_from, c->_to );
	IOAsyncClosure * parent = c->_parent;
	if ( parent ) {
		parent->activate( c->_result );
	}
	delete c;
}

void
SynchronizeIOAsyncClosure::func( SynchronizeIOAsyncClosure * c ) 
{
	if ( c->_state == 0 ) {
		c->_state = 1;
		c->_memoryObject->basicSynchronize( c->_from, c->_to, c );
		return;
	}
	if ( c->_state == 1 ) {
		c->_state = 2;
		Assert( c->_parent != 0 );
		c->_parent->activate( c->_result );
		delete c;
		return;
	}
	Assert( NOTREACHED );
}

void
TranslateBytesToUnitsAsyncClosure::func( TranslateBytesToUnitsAsyncClosure * c )
{
	unsigned result = c->_result;
        result = ( result + ( ( 1 << c->_log2UnitSize ) - 1 ) )  
		 >> c->_log2UnitSize;
        Assert( result >= c->_expected );
	if ( c->_parent ) {
		c->_parent->activate( result );
	}
	delete c;
}

static DummyAsyncClosure staticDummyAsyncClosure;

DummyAsyncClosure * theDummyAsyncClosure = &staticDummyAsyncClosure;

void DummyAsyncClosure::func( DummyAsyncClosure * c )
{
	return;
}
