#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * MemoryObjectView.c 
 */

#include "ClassStar.h"
#include "MemoryObjectCache.h"
#include "MemoryObjectView.h"
#include "PhysicallyAddressableUnit.h"
#include "Raid.h"
#include "RangeLocks.h"


#ifdef __GNUC__
#pragma implementation
#endif

MemoryObjectView::MemoryObjectView( MemoryObject * source,
                                    unsigned int numberOfUnits, 
                                    int log2UnitSize,
                                    unsigned startUnit )
    : MemoryObject( numberOfUnits, log2UnitSize ), 
      _source( source ), _startUnit( startUnit )
         
{
    initDebug();
    // Assert( source != 0 );
    Raids( RaidConstructor ) << this << "::MemoryObjectView( " 
                             << source << ", numberOfUnits " << numberOfUnits 
                             << ", log2UnitSize " << log2UnitSize
                             << ", startUnit " << startUnit
                             << " )\n" << eor;

    _viewOffset = startUnit << _log2UnitSize;
    if ( source ) {
        Assert( !_source->isKindOf( MemoryObjectViewClass ) );
        MemoryObjectCache * moc = buildCache();
        Assert( moc == 0 || 
                ( _viewOffset & ( PAUUnitSize - 1 ) ) == 0 );
        // setName( _source->objectName() );
    }
}

int
MemoryObjectView::setNumberOfUnits( unsigned units )
{
    _numberOfUnits = units;
    return units;
}

MemoryObjectCache * 
MemoryObjectView::buildCache()
{
    return _source == 0 ? 0 : _source->buildCache();
}

MemoryObjectCache * 
MemoryObjectView::cache()
{
    return _source == 0 ? 0 : _source->cache();
}

ProtectionLevel
MemoryObjectView::logicalProtectionLevel()
{
    return _source->logicalProtectionLevel();
}

/* These functions should not be reached */

int
MemoryObjectView::read( unsigned startUnit, int count, 
                        char * addr, int nbytes )
{
    Assert( NOTREACHED ); return 0;
}

int
MemoryObjectView::rawRead( unsigned startUnit, unsigned count, 
                           char * addr, unsigned nbytes, IOAsyncClosure * c )
{
    Assert( NOTREACHED ); return 0;
}

int
MemoryObjectView::write( unsigned startUnit, int count, 
                         char * addr, int nbytes )
{
    Assert( NOTREACHED ); return 0;
}

int
MemoryObjectView::rawWrite( unsigned startUnit, unsigned count, 
                            char * addr, unsigned nbytes, int writeThrough,
                            IOAsyncClosure * c )
{
    Assert( NOTREACHED ); return 0;
}

int
MemoryObjectView::writeThrough( unsigned startUnit, int count, char * addr, 
                                int nbytes )
{
    Assert( NOTREACHED ); return 0;
}

void
MemoryObjectView::asyncRead( unsigned startUnit, int count, 
                             char * addr, int nbytes, IOAsyncClosure * c )
{
    Assert( NOTREACHED ); return;
}

void
MemoryObjectView::asyncWrite( unsigned startUnit, int count, 
                              char * addr, int nbytes, IOAsyncClosure * c )
{
    Assert( NOTREACHED ); return;
}

void
MemoryObjectView::asyncWriteThrough( unsigned startUnit, int count, 
                                     char * addr, int nbytes, 
                                     IOAsyncClosure * c ) 
{
    Assert( NOTREACHED ); return;
}

void
MemoryObjectView::synchronize()
{
    Assert( NOTREACHED );
}

void
MemoryObjectView::flush()
{
    Assert( NOTREACHED );
}

int
MemoryObjectView::copy( MemoryObject * dest )
{
    Assert( NOTREACHED ); return 0;
}

RangeLock * 
MemoryObjectView::moLock()
{
    Assert( NOTREACHED ); return 0;
}

void
MemoryObjectView::setMemoryObjectCache( MemoryObjectCache * cache )
{
    Assert( NOTREACHED ); 
}

unsigned 
MemoryObjectView::cacheLog2UnitSize()
{
    Assert( NOTREACHED ); return 0;
}

unsigned
MemoryObjectView::setCacheLog2UnitSize( unsigned log2Size )
{
    Assert( NOTREACHED ); return 0;
}

Class *
MemoryObjectView::cacheClass()
{
    Assert( NOTREACHED ); return 0;
}

void
MemoryObjectView::setCacheClass( Class * cacheClass )
{
    Assert( NOTREACHED ); 
}

Class *
MemoryObjectView::cachePolicyClass()
{
    Assert( NOTREACHED ); return 0;
}

void
MemoryObjectView::setCachePolicyClass( Class * policyClass )
{
    Assert( NOTREACHED );
}

/*
 * NormalMemoryObjectView
 */
int
NormalMemoryObjectView::read( unsigned int startUnit, int count, 
                              char * addr, int nbytes )
{
    Assert( cache() != 0 );
    unsigned offset = ( startUnit << log2UnitSize() ) + _viewOffset;
    int maxBytes = numberOfUnits() << log2UnitSize();
    if ( maxBytes < nbytes ) 
        nbytes = maxBytes;
    unsigned from = offset >> _source->log2UnitSize();
    unsigned to = ( offset + nbytes - 1 ) >> _source->log2UnitSize();
    _source->acquire( from, to );
    unsigned actual = cache()->cacheRead( offset, nbytes, addr, 0 );
    _source->release( from, to );
    actual = ( actual + ( unitSize() - 1 ) ) >> log2UnitSize();
    return actual;
}

int
NormalMemoryObjectView::write( unsigned int startUnit, int count, 
                               char * addr, int nbytes )
{
    Assert( cache() != 0 );
    unsigned offset = ( startUnit << log2UnitSize() ) + _viewOffset;
    int maxBytes = numberOfUnits() << log2UnitSize();
    if ( maxBytes < nbytes ) 
        nbytes = maxBytes;
    unsigned from = offset >> _source->log2UnitSize();
    unsigned to = ( offset + nbytes - 1 ) >> _source->log2UnitSize();
    _source->acquire( from, to );
    unsigned actual = cache()->cacheWrite( offset, nbytes, addr, 0 );
    _source->release( from, to );
    actual = ( actual + ( unitSize() - 1 ) ) >> log2UnitSize();
    return actual;
}

#include "ClassInstaller.h"
DEFINECLASS("MemoryObjectView",MemoryObjectView,MemoryObject,0,0);
DEFINECLASS("NormalMemoryObjectView",NormalMemoryObjectView,MemoryObjectView,0,0);
DEFINECLASS("LockedMemoryObjectView",LockedMemoryObjectView,MemoryObjectView,0,0);
