#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * MemoryRange.cc - 
 */
#include "MemoryRange.h"
#include "ClassStar.h"


#ifdef __GNUC__
#pragma implementation
#endif

MemoryRange::MemoryRange( unsigned int numberOfUnits, unsigned log2UnitSize )
{
	initDebug();

	Assert( this != 0 );
	Assert( log2UnitSize >= 0 );
	Assert( log2UnitSize <= 32 );

	// maybe I should some assertion about number of units??
	_numberOfUnits = numberOfUnits;
	_log2UnitSize = log2UnitSize;
}

unsigned
MemoryRange::unitSize()
{
	return( (unsigned) 1 << _log2UnitSize );
}

unsigned
MemoryRange::log2UnitSize()
{
	return( _log2UnitSize );
}

unsigned int
MemoryRange::numberOfUnits()
{
	return( _numberOfUnits );
}

unsigned int
MemoryRange::offsetToUnit( Offset offset )
{
	return( (unsigned int) offset >> _log2UnitSize );
}

Offset
MemoryRange::unitToOffset( unsigned int unit )
{
	return( unit << _log2UnitSize );
}

int
MemoryRange::isIn( unsigned unit )
{
	return( unit < _numberOfUnits );
}

#include "ClassInstaller.h"
DEFINECLASS("MemoryRange",MemoryRange,ProxiableObject,0,0);
