#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * PhysicalMemoryManager.cc 
 */

#include "Assert.h"
#include "OutputStream.h"
#include "PhysicalMemoryRange.h"
#include "PhysicalMemoryManager.h"
#include "NewAndDelete.h"

#ifdef __GNUC__
#pragma implementation
#endif

class PhysicallyAddressableUnit;	// forward reference...

PhysicalMemoryManager::PhysicalMemoryManager( PhysicalMemoryRange *
	systemMemoryMap )
{
	Debug << this << "::PhysicalMemoryManager()\n" << eor;
	Assert( this != 0 );

	map = systemMemoryMap;

	PhysicalMemoryRange * region;
	int totalRegions = 0;
	for( region = systemMemoryMap; region != 0; region = region->next() ) {
		totalRegions++;
	}
	Debug << this << "::PhysicalMemoryManager(): " << totalRegions
		<< " regions\n" << eor;

	theUnits = (PhysicallyAddressableUnit **) new char
		[ sizeof( PhysicallyAddressableUnit ** ) * totalRegions ];

	region = systemMemoryMap;
	for( int i = 0; i < totalRegions; i++ ) {
		/*
		 * Allocate and initialize the PAUs for this region.
		 */
		theUnits[i] = (PhysicallyAddressableUnit *) new char
			[ sizeof( PhysicallyAddressableUnit ) *
				region->numberOfUnits() ];

		char * address = (char *) region->physicalAddress();
		for( unsigned j = 0; j < region->numberOfUnits(); j++ ) {
			(void) new (&theUnits[i][j])
				PhysicallyAddressableUnit( address );
			address += region->unitSize();
		}
		Debug << region << ", " << region->numberOfUnits()
			<< " units at "
			<< asAddress( region->physicalAddress() ) << "\n"
			<< eor;
		region = region->next();
	}
}



/*
 * This should never get called.
 */
PhysicalMemoryManager::~PhysicalMemoryManager()
{
	Assert( NOTREACHED );
	// how would I free the PAUs anyway?
}

PhysicallyAddressableUnit *
PhysicalMemoryManager::lookup( const char * physicalAddress )
{
//	Debug << this << "::lookup( " << asAddress( physicalAddress ) << " )\n" << eor;

	PhysicalMemoryRange * region;
	unsigned int unitOffset;
	int i = 0;
	for( region = map; region != 0; region = region->next() ) {
//		Debug << "Checking " << region << "("
//			<< asAddress( region->physicalAddress() )
//			<< ")\n" << eor;
		if( physicalAddress >= region->physicalAddress() ) {
			unitOffset = region->offsetToUnit(
				physicalAddress - region->physicalAddress() );
//			Debug << "unitOffset: " << unitOffset
//				<< ", numberOfUnits: "
//				<< region->numberOfUnits() << "\n" << eor;
			if( region->isIn( unitOffset ) ) {
				return( &theUnits[i][unitOffset] );
			}
		}
		i++;
	}

	Console() << this << "::lookup: " << asAddress( physicalAddress )
		<< " is not in memory map!\n\n" << eor;

	Assert( NOTREACHED );
	return( 0 );
}

#include "ClassInstaller.h"
DEFINECLASS("PhysicalMemoryManager",PhysicalMemoryManager,ProxiableObject,0,0);
