#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *       University of Illinois at Urbana-Champaign
 *       Department of Computer Science
 *       1304 W. Springfield Ave.
 *       Urbana, IL 61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *  All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Bjorn A. Helgaas (helgaas@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * PhysicalMemoryRange.cc - 
 */

#include "Assert.h"
#include "PhysicalMemoryRange.h"
#include "ClassStar.h"


#ifdef __GNUC__
#pragma implementation
#endif

PhysicalMemoryRange::PhysicalMemoryRange( const char * physicalAddress,
    unsigned int units, int log2UnitSize,  char * name,
    int preAllocated )
    : MemoryRange( units, log2UnitSize )
{
    _next = 0;

    _physicalAddress = physicalAddress;
    _virtualAddress = physicalAddress;
    _transparentlyMapped = 1;
    _preAllocated = preAllocated;

    _objectName = name;
    setName( (const char *) name );
}

PhysicalMemoryRange::PhysicalMemoryRange( const char * physicalAddress,
    const char * virtualAddress, unsigned int units, int log2UnitSize,
    char * name, int preAllocated )
    : MemoryRange( units, log2UnitSize )
{
    _next = 0;

    _physicalAddress = physicalAddress;
    _virtualAddress = virtualAddress;
    _transparentlyMapped = (physicalAddress == virtualAddress);
    _preAllocated = preAllocated;

    setName( (const char *) name );
}

PhysicalMemoryRange::~PhysicalMemoryRange()
{
    Assert( NOTREACHED );
}

void
PhysicalMemoryRange::add( PhysicalMemoryRange * newRange )
{
    Assert(newRange != 0);

    PhysicalMemoryRange * current = this;
    while( current->next() != 0 ) {
        current = current->next();
    }
    current->setNext( newRange );
}

/*
 * This function really should be inline, but is in the .cc file to force
 * Cfront to generate a vtable for the class.
 */



#include "ClassInstaller.h"
DEFINECLASS("PhysicalMemoryRange",PhysicalMemoryRange,MemoryRange,0,0);
