#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * RangeLocks.cc
 */

#include "BlockedInfo.h"
#include "Raid.h"
#include "RangeLocks.h"
#include "Process.h"
#include "ProcessEvents.h"
#include "PROCESSOR.h"
#include "ThisProcess.h"

#if defined(SS1) || defined(VCHOICES)
#define SINGLEPROCESSOR
#endif

#ifdef __GNUC__
#pragma implementation
#endif

//#define RELINQUISH

void 
RangeLock::block( unsigned from, unsigned to )
{
	Process * process = thisProcess();

//	Raids( RaidProtected ) 
//		<< this << "::block( " 
//		<< from << ", " << to << " ) " << _object << " " 
//		<< process << "\n"  << eor;

	Assert( process != 0 );
	_FaultInfo * info = (_FaultInfo *) process->blockedInfo();
	info->setFrom( from );
	info->setTo( to );
	_blocked.addTail( process );
	release();
	process->block( _object );
}

void 
RangeLock::unblock()
{
//	Raids( RaidProtected ) 
//		<< this << "::unblock()\n" << eor;

	Process * process = _blocked.head();
	while( process != 0 ) {
		_FaultInfo * info = ( _FaultInfo * ) process->blockedInfo();
		if ( _use->use( info->from(), info->to(), process ) == 0 ) {
			_blocked.remove( process );
			if ( info->to() == MaxUse ) {
				release();
        			IntStorage intState ; PROCESSOR::disableInterrupts(intState);
        			thisProcess()->ready();
//				nextProcess->setProcessEvent( INODE );
        			Assert (! PROCESSOR::interruptsEnabled());
        			thisProcess()->giveProcessorTo
					( process, ProcessReady );
        			PROCESSOR::restoreInterrupts(intState);
				acquire();
				_exclusive = 1;
				break;
			} else {
				process->ready();
			}
		}
		process = info->next();
	}
}

void
RangeLock::acquireAll()
{
//	Raids( RaidMethod ) 
//		<< this << "::acquireAll()\n" << eor;
	acquire( 0, MaxUse, 1 );
}

void
RangeLock::releaseAll()
{
//	Raids( RaidMethod ) 
//		<< this << "::releaseAll()\n" << eor;
	release( 0, MaxUse );
}

int debugRangeLockExclusive = 0;

Process * 
RangeLock::acquire( unsigned from, unsigned to, int blocking, int locked )
{
//	Raids( RaidMethod ) 
//		<< this << "::acquire( " << from << ", " << to << ", " 
//		<< blocking << " ) " << _object << "\n" << eor;
	IntStorage intState ; PROCESSOR::disableInterrupts(intState);
	Process * acquiredBy;
	Assert( !( locked && blocking ) );
	if ( ! locked )
		acquire();
	acquiredBy = _use->use( from, to );
	if ( acquiredBy != 0 ) {
		if ( blocking ) {
			block( from, to );
			if ( locked )
				acquire();
			acquiredBy = 0;
		} else {
			if ( _exclusive ) {
//				if ( debugRangeLockExclusive ) Console() 
//					<< this << "::acquire( " 
//					<< from << ", " << to 
//					<< ", non-blocking ) " 
//					<< _object 
//					<< " held exclusive\n" 
//					<< eor;
				release();
				thisProcess()->relinquishProcessor();
				acquire();
			}
			if ( ! locked ) 
				release();
		}
	} else {
		if ( to == MaxUse ) {
			_exclusive = 1;
//		        thisProcess()->setProcessEvent( INODE );
		}
		if ( ! locked )
			release();
	}
	PROCESSOR::restoreInterrupts( intState );
//	Raids( RaidMethod )
//		<< this << "::acquire( " << from << ", " << to 
//		<< ", " << blocking
//		<< " ) " << _object << " done " << acquiredBy << "\n" << eor;
	return acquiredBy;
}

Process *
RangeLock::release( unsigned from, unsigned to, int locked )
{
//	Raids( RaidMethod ) 
//		<< this << "::release( " << from << ", " << to 
//		<< " ) " << _object << "\n" << eor;
	if ( !locked ) 
		acquire();
	Process * process = _use->unuse( from, to );
	Assert( process != 0 );
	if ( _exclusive ) {
		_exclusive = 0;
//		process->setProcessEvent( 0 );
	}
	unblock();
	if ( !locked )
		release();
	return process;
}

void
RangeLock::downgradeFromAll( unsigned from, unsigned to )
{
//	Raids( RaidMethod )
//		<< this << "::downgradeFromAll( " << from << ", " << to
//		<< " )\n" << eor;
	acquire();
	Process * process = _use->lessUse( from, to );
	Assert( process != 0 );
	_exclusive = 0;
//	process->setProcessEvent( 0 );
	unblock();
	release();
}

SpecialRangeLock::SpecialRangeLock( ProxiableObject * mo )
     : RangeLock( mo ), _constantSizeUse( 64 )
{
	_use = & _constantSizeUse;
}

#ifndef RELINQUISH

void 
SpecialRangeLock::block( unsigned from, unsigned to )
{
        RangeLock::block( from, to );
}

void
SpecialRangeLock::unblock()
{
        RangeLock::unblock();
}

#else // RELINQUISH

void
SpecialRangeLock::block( unsigned from, unsigned to )
{
//	Raids( RaidProtected ) 
//		<< this << "::block( " 
//		<< from << ", " << to << " ) " << _object << "\n"  << eor;

	Process * myProcess = thisProcess();
	unsigned count = 0;
	while ( 1 ) {
		Process * process = _use->use( from, to, myProcess );
		if ( process == 0 )
			break;
		release();
		unsigned lastCount = count;
		count++;
		if ( count % 1000 ) {
			Console()
				<< this << "::block( " << from << ", "
				<< to << " ) still waiting " << count 
				<< "\n" << eor;
		}
		Assert( lastCount < count );
#ifdef SINGLEPROCESSOR
		myProcess->relinquishProcessor();
#endif
		acquire();
	}
	release();
}

void
SpecialRangeLock::unblock()
{
//	Raids( RaidProtected ) 
//		<< this << "::unblock()\n" << eor;
}

#endif // RELINQUISH

NormalRangeLock::NormalRangeLock( ProxiableObject * mo )
     : RangeLock( mo ), _variableSizeUse( 5 )
{
	_use = & _variableSizeUse;
}

Lock MutexLock::_lock;

void
MutexLock::acquire()
{
//	Raids( RaidMethod ) 
//		<< this << "::acquire()\n" << eor;
	IntStorage intState ; PROCESSOR::disableInterrupts(intState);
	Process * process = thisProcess();
	_lock.acquire();
	if ( _holder != 0 ) {
		_blocked.addTail( process );
		_lock.release();
		process->block( (Object *) this );
	} else {
		_holder = process;
		_lock.release();
	}
	Assert( _holder == process );
	PROCESSOR::restoreInterrupts( intState );
//	Raids( RaidMethod )
//		<< this << "::acquire() done\n" << eor;
}

void
MutexLock::release()
{
//	Raids( RaidMethod ) 
//		<< this << "::release()\n" << eor;
	_lock.acquire();
	Process * process = _blocked.head();
	if ( process != 0 ) {
		_blocked.remove( process );
		_holder = process;
		process->ready();
	} else {
		_holder = 0;
	}
	_lock.release();
}

