#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1993
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * ReplacementPolicy.cc
 */

//#define DEBUG_NEW_STUFF

#include "Class.h"
#include "Map.h"
#include "PagedMemoryObjectCache.h"
#include "PageOutAsync.h"
#include "Raid.h"
#include "RangeLocks.h"
#include "ReplacementPolicy.h"
#include "Store.h"
#include "ThisCPU.h"

#ifdef __GNUC__
#pragma implementation
#endif

ReplacementPolicy::ReplacementPolicy( PagedMemoryObjectCache * cache )
{
	initDebug();
	Raids( RaidConstructor ) 
		<< this << "::ReplacementPolicy( " << cache 
		<< " )\n" << eor;
	Assert( this != 0 );
	Assert( cache != 0 && cache->isKindOf( PagedMemoryObjectCacheClass ) );
	_cache = cache;
	_map = _cache->_map;
	_numberOfUnits = _cache->numberOfUnits();
	Assert( _map );
	setName( _cache->objectName() );
}

unsigned
ReplacementPolicy::maxResidentUnits()
{
	Raids( RaidMethod ) 
		<< this << "::maxResidentUnits()" << "\n" << eor;
	return _numberOfUnits;
}

unsigned
ReplacementPolicy::setMaxResidentUnits( unsigned units )
{
	Raids( RaidMethod ) 
		<< this << "::setMaxResidentUnits( " << units 
		<< " )" << "\n" << eor;
	Assert( units == _numberOfUnits );
	return _numberOfUnits;
}

void 
ReplacementPolicy::setNumberOfUnits( unsigned noOfUnits )
{
	Raids( RaidMethod ) 
		<< this << "::setNumberOfUnits( " << noOfUnits
		<< " )\n" << eor;
	_numberOfUnits = noOfUnits;
}

void
ReplacementPolicy::unitsFromStore( unsigned from, unsigned to )
{
	Raids( RaidMethod ) << this << "::unitsFromStore( "
		<< from << ", " << to << " )\n" << eor;
}

void
ReplacementPolicy::unitsToStore( unsigned from, unsigned to )
{
	Raids( RaidMethod ) << this << "::unitsToStore( "
		<< from << ", " << to << " )\n" << eor;
}

void
ReplacementPolicy::postPageIn( unsigned from, unsigned to )
{
	Raids( RaidMethod ) << this << "::postPageIn( "
		<< from << ", " << to << " )\n" << eor;
}

void
ReplacementPolicy::preCacheRead( unsigned from, unsigned to )
{
        Raids( RaidMethod ) << this << "::preCacheRead( "
                << from << ", " << to << " )\n" << eor;
}
 
void
ReplacementPolicy::preCacheWrite( unsigned from, unsigned to )
{
        Raids( RaidMethod ) << this << "::preCacheWrite( "
                << from << ", " << to << " )\n" << eor;
}

void
ReplacementPolicy::postCacheRead( unsigned from, unsigned to )
{
        Raids( RaidMethod ) << this << "::postCacheRead( "
                << from << ", " << to << " )\n" << eor;
}
 
void
ReplacementPolicy::postCacheWrite( unsigned from, unsigned to )
{
        Raids( RaidMethod ) << this << "::postCacheWrite( "
                << from << ", " << to << " )\n" << eor;
}

void
ReplacementPolicy::notify( int )
{
}

void
ReplacementPolicy::doPageOutAsync( unsigned firstUnit, unsigned lastUnit, 
				   PhysicallyAddressableUnit * firstPau, 
				   PostFaultType postFaultType, 
				   int unitsToStoreCode )
{
/*
	Console() << this << "::doPageOut( " << firstUnit << ", " 	
		<< lastUnit << " ) memoryObject " << _cache->memoryObject()
		<< "\n" << eor;
 */

	Assert( ( unitsToStoreCode & UNITSTOSTORE_QUEUED ) == 0 );
	cacheUnitsToStoreQueued( lastUnit - firstUnit + 1 );
	unitsToStoreCode |= UNITSTOSTORE_QUEUED;

        cacheUntranslatable( firstUnit, lastUnit );
 
	// PageOutAsyncClosure::_countSem.P();

        PageOutAsyncClosure * closure =
                new PageOutAsyncClosure( this, firstUnit, lastUnit,
                                         firstPau, postFaultType, 
					 unitsToStoreCode );

	cacheToCache( firstUnit, lastUnit, 1, firstPau, closure );

        // TheAsyncSupport->schedule( closure );
}

unsigned
ReplacementPolicy::pageOut( unsigned noOfUnits )
{
	Raids( RaidMethod ) 
		<< this << "::pageOut( " << noOfUnits << " ) mo "
		<< asAddress(_cache->memoryObject()) << " cpu " << asAddress(thisCPU()) << "\n"
		<< eor;

	Assert( noOfUnits > 0 );

	PhysicallyAddressableUnit * pauList;
	unsigned start;
	unsigned end;
	unsigned found = victim( noOfUnits, start, end, pauList );

	unsigned foundUnits;
	for ( foundUnits = 0; found && foundUnits < noOfUnits; ) {
		
		unsigned firstUnit = start;
		PhysicallyAddressableUnit * firstPau = pauList;
		foundUnits += ( end - start + 1 );

		/* at least one victim found, check for consecutive victims */

		unsigned lastUnit = end;
		found = 0;
		while ( foundUnits < noOfUnits && 
		        ( found = victim( noOfUnits - foundUnits, 
				          start, end, pauList ) ) &&
		        start == ( lastUnit + 1 ) ) {
			lastUnit = end;
			foundUnits += ( end - start + 1 );
		}
		
		Assert( firstUnit <= lastUnit && lastUnit < _numberOfUnits );

		/* page out firstUnit to lastUnit */

#ifdef DEBUG_NEW_STUFF
		PhysicallyAddressableUnit * checkList = 
			_map->pauList( firstUnit, lastUnit );
		PhysicallyAddressableUnit * tmp = firstPau;
		unsigned count = ( lastUnit - firstUnit + 1 ) << 
				 ( _cache->log2UnitSize() - Log2PAUUnitSize );
		while ( count -- ) {
			Assert( tmp != 0 );
			Assert( tmp == checkList );
			tmp = tmp->nextLink();
			checkList = checkList->nextLink();
		}
#endif // DEBUG_NEW_STUFF

                doPageOutAsync( firstUnit, lastUnit, firstPau, 
				PostFaultIndividual, 0 );

	}

	Raids( RaidDetail )
		<< this << "::pageOutUnits() completed found "
	        << foundUnits << "\n" << eor;
	
	return foundUnits;	
}

BoundedReplacementPolicy::BoundedReplacementPolicy( PagedMemoryObjectCache * 
						    cache )
     : ReplacementPolicy( cache ), _pageOutMutex( 1 )
{
	initDebug();
	Raids( RaidConstructor ) 
		<< this << "::BoundedReplacementPolicy( " << cache 
		<< " )\n" << eor;
	_maxResidentUnits = _numberOfUnits;
	if ( _maxResidentUnits == 0 ) {
		/* default 64K or unit size if unit > 64K */
		if ( _cache->log2UnitSize() < 16 ) 
			_maxResidentUnits = 1 << (16 - _cache->log2UnitSize());
		else 
			_maxResidentUnits = 1;
	}
	_residentUnits = _map->residentSize();
}

unsigned
BoundedReplacementPolicy::maxResidentUnits()
{
	Raids( RaidMethod ) << this << "::maxResidentUnits()" << "\n" << eor;
	return _maxResidentUnits;
}

unsigned
BoundedReplacementPolicy::setMaxResidentUnits( unsigned units )
{
	Raids( RaidMethod ) << this << "::setMaxResidentUnits( " << units 
		<< " )" << "\n" << eor;
	unsigned old = _maxResidentUnits;
	_lock.acquire();
	_maxResidentUnits = units;
	_lock.release();
	return old;
}

void
BoundedReplacementPolicy::unitsFromStore( unsigned from, unsigned to )
{
	Raids( RaidMethod ) 
		<< this << "::unitsFromStore( "
		<< from << ", " << to << " )\n" << eor;

	Assert( from <= to && to < _numberOfUnits );
	int count = to - from + 1;

	_pageOutMutex.P();
	_lock.acquire();
	_residentUnits += count;
	int excess = _residentUnits - _maxResidentUnits;
	for ( int tries = 0; tries < 2 && excess > 0; tries ++ ) {	      
		_lock.release();
		// unsigned unitsPagedOut = 
		pageOut( excess );
		_lock.acquire();
		excess = _residentUnits - _maxResidentUnits;
	}
	_lock.release();
	Raids( RaidDetail ) 
		<< this << "::unitsFromStore( " << from << ", " << to
		<< " ) excess " << excess << "\n" << eor;
	_pageOutMutex.V();
}

void
BoundedReplacementPolicy::unitsToStore( unsigned from, unsigned to )
{
	Raids( RaidMethod ) 
		<< this << "::unitsDeallocated( "
		<< from << ", " << to << " )\n" << eor;
	Assert( from <= to && to < _numberOfUnits );
	_lock.acquire();
	unsigned count = to - from + 1;
	_residentUnits -= count;
	_lock.release();
}

SimpleReplacementPolicy::SimpleReplacementPolicy
	( PagedMemoryObjectCache * cache ) : ReplacementPolicy( cache )
{
	initDebug();
	Raids( RaidConstructor ) 
		<< this << "::SimpleReplacementPolicy( " << cache 
		<< " )\n" << eor;
	_lastVictim = ~0;
}

unsigned 
SimpleReplacementPolicy::victim(unsigned noOfUnits, 
			        unsigned &start, unsigned &end, 
				PhysicallyAddressableUnit * &pau)
{
	Raids( RaidMethod ) << this << "::victim()\n" << eor;

	acquireMoLock();
	unsigned nextVictim = _lastVictim + 1;
	for( unsigned i = 0; i < _cache->numberOfUnits(); i++, nextVictim++ ) {
		if ( nextVictim >= _cache->numberOfUnits() ) 
			nextVictim = 0;
		if ( _map->pauList( nextVictim, nextVictim ) && 
		     acquireMo( nextVictim, nextVictim, 0, 1 ) == 0 ) {
			pau = _map->pauList( nextVictim, nextVictim );
			if ( !pau ) {
				Raids( RaidDetail ) 
					<< this 
					<< "::victim() victim " << nextVictim
					<< " pre-empted\n" 
					<< eor;
			        releaseMo( nextVictim, nextVictim, 1 );
				continue;
			}
			releaseMoLock();
			Raids( RaidMethod ) 
				<< this << "::victim() is "
				<< nextVictim << "\n" << eor;
			_lastVictim = nextVictim;
			start = end = _lastVictim;
			return 1;
		}
	}
	releaseMoLock();

	Raids( RaidMethod ) << this << "::victim() none found\n" << eor;

	return 0;
}

ProxiableObject *
SimpleReplacementPolicyConstructor( ProxiableObject * source )
{
        if( !source->isKindOf( PagedMemoryObjectCacheClass ) ) 
		return 0;
	return new 
		SimpleReplacementPolicy( ( PagedMemoryObjectCache * ) source );
}

FIFOReplacementPolicy::FIFOReplacementPolicy
	( PagedMemoryObjectCache * cache ) : BoundedReplacementPolicy( cache )
{
	initDebug();
	Raids( RaidConstructor ) 
		<< this << "::FIFOReplacementPolicy( " << cache 
		<< " )\n" << eor;
	Assert( _numberOfUnits == _cache->numberOfUnits() );
	if ( _map->residentSize() != 0 ) {
		for ( unsigned i = 0; i < _numberOfUnits; i++ ) {
			if ( _map->exists( i ) ) {
				Raids( RaidDetail ) 
					<< this << "::FIFOReplacementPolicy() "
					<< i << " exists\n" << eor;
				FIFOReplacementPolicyLink * link = 
					new FIFOReplacementPolicyLink( i );
				_fifo.addLast( link );
			}
		}
	}
	Assert( _numberOfUnits == _cache->numberOfUnits() );
}

FIFOReplacementPolicy::~FIFOReplacementPolicy()
{
	Raids( RaidConstructor ) 
		<< this << "::~FIFOReplacementPolicy()\n" << eor;
	FIFOReplacementPolicyLink * link;
	while ( ( link = ( FIFOReplacementPolicyLink * ) _fifo.removeFirst() )
	        != 0 )
		delete link;
}

void
FIFOReplacementPolicy::postPageIn( unsigned from, unsigned to )
{
	Raids( RaidMethod ) << this << "::postPageIn( "
		<< from << ", " << to << " )\n" << eor;
	Assert( from <= to && to < _numberOfUnits );
	_fifoLock.acquire();
	for ( unsigned i = from; i <= to; i++ ) {
		FIFOReplacementPolicyLink * link =
			new FIFOReplacementPolicyLink( i );
		_fifo.addLast( link );
	}
	_fifoLock.release();
}

void
FIFOReplacementPolicy::unitsToStore( unsigned from, unsigned to )
{
	Raids( RaidMethod ) 
		<< this << "::unitsToStore( "
		<< from << ", " << to << " )\n" << eor;

	Assert( from <= to && to < _numberOfUnits );
	Assert( _residentUnits >= _map->residentSize() );
	_fifoLock.acquire();
	FIFOReplacementPolicyLink * firstLink = 
		( FIFOReplacementPolicyLink * ) _fifo.first();
	FIFOReplacementPolicyLink * nextLink = firstLink;
	unsigned removed = 0;
	unsigned count = to - from + 1;
	for ( unsigned i = 0; removed < count && i < _residentUnits; i++ ) {
		unsigned unit = nextLink->unit();
		if ( unit >= from && unit <= to ) {
			Raids( RaidDetail ) 
				<< this << "::unitsToStore() found "
				<< unit << "\n" << eor;
			FIFOReplacementPolicyLink * prevLink = nextLink;
			nextLink = ( FIFOReplacementPolicyLink * ) 
				   prevLink->next();
			if ( prevLink == firstLink ) {
				_fifo.removeFirst();
				firstLink = ( FIFOReplacementPolicyLink * )
					    _fifo.first();
			} else {
				prevLink->remove();
			}
			delete prevLink;
			removed++;
		} else {
			nextLink = ( FIFOReplacementPolicyLink * ) 
				   nextLink->next();
		}
	} 
	Assert( removed == count );
	_fifoLock.release();
        BoundedReplacementPolicy::unitsToStore( from, to );
}

unsigned 
FIFOReplacementPolicy::victim(unsigned noOfUnits, 
			      unsigned &start, unsigned &end, 
			      PhysicallyAddressableUnit * &pau)
{
	Raids( RaidMethod ) << this << "::victim()\n" << eor;

	acquireMoLock();
	_fifoLock.acquire();
	FIFOReplacementPolicyLink * firstLink = 
		( FIFOReplacementPolicyLink * ) _fifo.first();
	FIFOReplacementPolicyLink * nextLink = firstLink;
	Assert( _residentUnits >= _map->residentSize() );
	for ( unsigned i = 0; 
	      i < _residentUnits; 
	      nextLink = ( FIFOReplacementPolicyLink * ) nextLink->next(),
	      i++ ) {
		unsigned nextVictim = nextLink->unit();
		if ( _map->exists( nextVictim ) && 
		     acquireMo( nextVictim, nextVictim, 0, 1 ) == 0 ) {
			pau = _map->pauList( nextVictim, nextVictim );
			if ( !pau ) {
				Raids( RaidDetail ) 
					<< this 
					<< "::victim() victim " << nextVictim
					<< " pre-empted\n" 
					<< eor;
			        releaseMo( nextVictim, nextVictim, 1 );
				continue;
			}

			Raids( RaidMethod ) 
				<< this << "::victim() is "
				<< nextVictim << "\n" << eor;
			
			if ( firstLink == nextLink ) {
				_fifo.removeFirst();
			} else {
				nextLink->remove();
			}
			delete nextLink;
			_fifoLock.release();
			releaseMoLock();
			start = end = nextVictim;
			return 1;
		}
	}
	_fifoLock.release();
	releaseMoLock();

	Raids( RaidMethod ) << this << "::victim() none found\n" << eor;

	return 0;
}

ProxiableObject *
FIFOReplacementPolicyConstructor( ProxiableObject * source )
{
        if( !source->isKindOf( PagedMemoryObjectCacheClass ) ) 
		return 0;
	return new 
		FIFOReplacementPolicy( ( PagedMemoryObjectCache * ) source );
}

#include "ClassInstaller.h"
DEFINECLASS("ReplacementPolicy",ReplacementPolicy,ProxiableObject,0,0);
DEFINECLASS("BoundedReplacementPolicy",BoundedReplacementPolicy,ReplacementPolicy,0,0);
DEFINECLASS("SimpleReplacementPolicy",SimpleReplacementPolicy,ReplacementPolicy,&SimpleReplacementPolicyConstructor,0);
DEFINECLASS("FIFOReplacementPolicy",FIFOReplacementPolicy,BoundedReplacementPolicy,&FIFOReplacementPolicyConstructor,0);




