#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "Assert.h"
#include "Class.h"
#include "Map.h"
#include "MemoryObject.h"
#include "Raid.h"
#include "RootMemoryObjectCache.h"
#include "PhysicallyAddressableUnit.h"
#include "TranslationMap.h"


#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This class of cache preallocates any needed state info in the ctor.
 * This is since it is the cache that most other caches get their state from.
 */
RootMemoryObjectCache::RootMemoryObjectCache( MemoryObject * mo )
	: SpecialPurposeCache( mo )
{
	initDebug();
	Raids( RaidConstructor ) << this << "::RootMemoryObjectCache( " 
		<< mo << " )\n" << eor;

	Assert( this != 0 );
	Assert( _memoryObject != 0 );
	Assert( log2UnitSize() == Log2PAUUnitSize );
	_map = new ConstantSizeTableMap( _numberOfUnits, log2UnitSize() );
	_adtMap = new ConstantSizeTranslationMap( _map );
	Assert( _map );
	Assert( _adtMap );
}

RootMemoryObjectCache::~RootMemoryObjectCache()
{
	Raids( RaidConstructor ) << this << "::~RootMemoryObjectCache( " 
		<< this << " )\n" << eor;

	cleanUp();

	delete _map;
	delete _adtMap;
}

ProxiableObject *
RootMemoryObjectCacheConstructor( ProxiableObject * source )
{
        if( !source->isKindOf( MemoryObjectClass ) ) 
		return 0;
	
	MemoryObject * mo = ( MemoryObject * ) source;
	return new RootMemoryObjectCache( mo );
}

#include "ClassInstaller.h"
DEFINECLASS("RootMemoryObjectCache",RootMemoryObjectCache,SpecialPurposeCache,&RootMemoryObjectCacheConstructor,0);

