#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Bjorn A. Helgaas (helgaas@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SourceOfZerosMemoryObject.c - fill a physical buffer with zeros.
 */

/*
 * NOTE: this file is used only for the Mac so far, but I think we should
 * get rid of the
 * SOZMOs for each processor and replace them all with this file.  Each
 * processor would have to define zeroBlock() in its Util.h and Util.s
 * files.  Then we could get rid of the SOURCE_OF_ZEROS_MO.h files also.
 * Similarly for the PhysicalMemoryObjects.  -Bjorn
 */

#include "Assert.h"
#include "Byte.h"
#include "MemoryObjectAsync.h"
#include "Raid.h"
#include "SourceOfZerosMemoryObject.h"


#ifdef __GNUC__
#pragma implementation
#endif

SourceOfZerosMemoryObject::SourceOfZerosMemoryObject()
	: MemoryObject( 0xffffffff, 0 ), _specialLock( this )
{
	Assert( this != 0 );
	initDebug();
	Raids( RaidConstructor ) 
		<< this << "::SourceOfZerosMemoryObject()\n" << eor;
	_moLock = & _specialLock;
}

int
SourceOfZerosMemoryObject::fixedSize()
{
	return 0;
}

int
SourceOfZerosMemoryObject::rawRead( unsigned startUnit, unsigned count,
				    char * addr, unsigned nbytes, 
				    IOAsyncClosure * c )
{
	Raids( RaidProtected ) 
		<< this << "::rawRead( " << startUnit 
		<< ", " << count 
		<< ", " << asAddress( addr ) 
		<< ", " << nbytes 
		<< ", " << asAddress( c )
		<< " )\n" << eor;

	Assert( count == nbytes );
	if ( count != 0 ) 
		ByteZero( addr, nbytes );
	if ( c ) 
		c->activate( count );
	return count;
}

int
SourceOfZerosMemoryObject::rawWrite( unsigned startUnit, unsigned count,
				     char * addr, unsigned nbytes, 
				     int writeThrough, IOAsyncClosure * c )
{
	Raids( RaidProtected ) 
		<< this << "::rawWrite( " << startUnit 
		<< ", " << count 
		<< ", " << asAddress( addr ) 
		<< ", " << nbytes 
		<< ", " << writeThrough 
		<< ", " << asAddress( c )
		<< " )\n" << eor;

	Assert( NOTREACHED );
	return 0;
}


#include "ClassInstaller.h"
DEFINECLASS("SourceOfZerosMemoryObject",SourceOfZerosMemoryObject,MemoryObject,0,0);
