#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * UNIXInodeCache.cc
 */

#include "Assert.h"
#include "CacheAsync.h"
#include "ClassStar.h"
#include "Convert.h"
#include "FileObject.h"
#include "FileObjectStar.h"
#include "FileSystemHints.h"
#include "FileSystemInterface.h"
#include "FileSystemInterfaceStar.h"
#include "Kernel.h"
#include "Log2.h"
#include "MemoryObjectCachingContainer.h"
#include "MemoryObjectContainer.h"
#include "MemoryObjectContainerStar.h"
#include "Process.h"
#include "ProcessStar.h"
#include "Raid.h"
#include "ThisProcess.h"
#include "TimeoutTimer.h"
#include "UNIXContainer.h"
#include "UNIXInode.h"
#include "UNIXInodeCache.h"

#ifdef __GNUC__
#pragma implementation
#endif

#define NEEDNOT_LOCK

char * UNIXInodeCachePath = "/mnt/local";
UNIXContainer * UNIXInodeCacheContainer = 0;

UNIXInodeCacheList UNIXInodeCache::_LRUList;
UNIXInodeCacheLink * UNIXInodeCache::_cursor = &_LRUList;
int UNIXInodeCache::_cachedBytes = 0;
int UNIXInodeCache::_cachedFiles = 0;
int UNIXInodeCache::_maxCachedBytes = 256 << 20;	
int UNIXInodeCache::_maxCachedFiles = 16384;
int UNIXInodeCache::_thresholdCachedBytes;
int UNIXInodeCache::_thresholdCachedFiles;

int cleanerBlocked = 1;
ProcessStar cleanerProcess = 0;
static Semaphore cleanerSem( 0 );
unsigned updateLRUCount = 0;


extern ClassStar UNIXContainerClass;

static Semaphore sem(1);

int debugUNIXInodeCache = 0;

inline
void UNIXInodeCache::updateThresholds()
{
	_thresholdCachedBytes = ( _maxCachedBytes / 10 ) * 9 ;
	_thresholdCachedFiles = ( _maxCachedFiles / 10 ) * 9 ;
}

inline 
void UNIXInodeCache::printParams( OutputStream & s )
{
	s 
	<< "UNIXInodeCache\n"
	<< "    max cached files " << _maxCachedFiles 
	<< " threshold " << _thresholdCachedFiles 
	<< " cached files " << _cachedFiles << "\n"
	<< "    max cached bytes " << _maxCachedBytes
	<< " threshold " << _thresholdCachedBytes 
	<< " cached bytes " << _cachedBytes
	<< "\n" << eor;
}

inline
void UNIXInodeCache::exceedThresholds()
{
	if ( debugUNIXInodeCache & 0x1 ) Console()
		<< "UNIXInodeCache::exceedThresholds()"
		<< " cachedFiles " << _cachedFiles
		<< " cachedBytes " << _cachedBytes
		<< "\n" << eor;
	cleanerBlocked = 0;
	cleanerSem.V();
}

UNIXInodeCache::UNIXInodeCache( MemoryObjectCache * moc, int & status )
	: RealCache( moc ), _link( this )
{
	initDebug();
	Raids( RaidConstructor ) << this << "::UNIXInodeCache( " 
		<< moc << " )\n" << eor;
	Assert( UNIXInodeCachePath != 0 );
	status = 0;
	if ( UNIXInodeCacheContainer == 0 ) {
	    	sem.P();
		if ( UNIXInodeCacheContainer == 0 ) {
			FileSystemInterfaceStar fsi = 
				TheKernel->fileSystemInterface();
			Assert( fsi != 0 );
			int openStatus;
			FileObjectStar fo = fsi->open( UNIXInodeCachePath, 
						       FileObjectClass,
						       openStatus,
						       O_NCACHE );
			Assert( fo != 0 );
			Assert( fo->fastContainer()->isKindOf
				( UNIXContainerClass ) );
			UNIXInodeCacheContainer = 
				(UNIXContainer *) fo->fastContainer();
			UNIXInodeCacheContainer->reference();
			cleanerProcess = new SystemProcess
				( (PFV) &UNIXInodeCache::cleaner,
				  TheKernel->domain(), normal, 0 );
			cleanerProcess->setName
				( "UNIXInodeCache::cleanerProcess" );
			cleanerProcess->ready();
			updateThresholds();
			printParams( Console() );
		}
		sem.V();
	}
	if ( _cachedFiles >= _maxCachedFiles ) {
		if ( debugUNIXInodeCache & 0x2 ) Console()
			<< this << "::UNIXInodeCache() caching not enabled\n"
			<< eor;
		status = -1;
		return;
	}
	_pmoStar = UNIXInodeCacheContainer->create
		( 0, UNIXInodeClass, status, O_NCACHE );
	Assert( status == 0 && _pmoStar != 0 );
	Assert( _pmoStar->isKindOf( UNIXInodeClass ) );
	_inode = ( UNIXInode * ) (void*)_pmoStar;
	_inodeLog2UnitSize = _inode->log2UnitSize();
	int log2InodeUnitSizeRatio = _log2UnitSize - _inodeLog2UnitSize;
	Assert( log2InodeUnitSizeRatio >= 0 );
	_log2SegmentSize = log2Low( _inode->segmentSize() );
	acquire();
	_link.insertBefore( &_LRUList );
	_cachedFiles++;
	if ( cleanerBlocked && _cachedFiles >= _thresholdCachedFiles )
		exceedThresholds();
	release();
	extern ClassStar PersistentMemoryObjectClass;
	Assert( _mo->isKindOf( PersistentMemoryObjectClass ) );
}

UNIXInodeCache::~UNIXInodeCache()
{
	if ( _pmoStar == 0 ) 
		return;
	acquire();
	if ( &_link == _cursor ) {
		_cursor = _link.next();
	}
	_link.unlink();
#ifdef DEBUG_CACHE_ALLOCATE
	if ( ! ( _cachedUnits <= _numberOfUnits &&
	         _cachedUnits == _unitInfoTable->validSize() ) ) {
		Console() << "UNIXInodeCache::~UNIXInodeCache inconsistent "
			<< _cachedUnits << " and " 
			<< _unitInfoTable->validSize() << "\n" << eor;
	}
#endif // DEBUG_CACHE_ALLOCATE
	Assert( _cachedUnits >= 0 && _cachedUnits <= _numberOfUnits );
	_cachedFiles--;
	_cachedBytes -= ( _cachedUnits << _log2UnitSize );
unsigned prevCachedUnits = _cachedUnits;
	_cachedUnits = 0;
	release();
if (prevCachedUnits != _numberOfUnits && (debugUNIXInodeCache & 0x8)) {
	Console() 
		<< this << "::~UNIXInodeCache cacheUnits " << prevCachedUnits
		<< " numberOfUnits " << _numberOfUnits << "\n" << eor;
}
	updateLRUCount++;
	if ( ( updateLRUCount & 0x1ff ) == 0 && ( debugUNIXInodeCache & 0x2 ) )
		printParams( Console() );
}

unsigned UNIXInodeCache::basicSetNumberOfUnits( unsigned noOfUnits )
{
	Raids( RaidProtected ) 
		<< this << "::basicSetNumberOfUnits( " << noOfUnits 
		<< " )\n" << eor;

	int log2InodeUnitSizeRatio = _log2UnitSize - _inodeLog2UnitSize;
#ifdef NEEDNOT_LOCK
	_inode->basicSetNumberOfUnits( noOfUnits << log2InodeUnitSizeRatio );
#else
	_inode->setNumberOfUnits( noOfUnits << log2InodeUnitSizeRatio );
#endif
	return RealCache::basicSetNumberOfUnits( noOfUnits );
}

void UNIXInodeCache::basicAllocate( unsigned count )
{
	_cachedBytes += ( count << _log2UnitSize );
	if ( cleanerBlocked && _cachedBytes >= _thresholdCachedBytes ) 
		exceedThresholds();
#ifdef DEBUG_CACHE_ALLOCATE
	if ( ! ( _cachedUnits <= _numberOfUnits &&
	         _cachedUnits == _unitInfoTable->validSize() ) ) {
		Console() << "UNIXInodeCache::basicAllocate inconsistent "
			<< _cachedUnits << " and " 
			<< _unitInfoTable->validSize() << "\n" << eor;
	}
#endif // DEBUG_CACHE_ALLOCATE
}

void UNIXInodeCache::basicFreeUnits( unsigned startUnit, unsigned count )
{
	Raids( RaidProtected ) 
		<< this << "::basicFreeUnits( " << startUnit << ", " << count
		<< " )\n" << eor;

	_cachedBytes -= ( count << _log2UnitSize );
	Assert( _cachedBytes >= 0 );
#ifdef DEBUG_CACHE_ALLOCATE
	if ( ! ( _cachedUnits <= _numberOfUnits &&
	         _cachedUnits == _unitInfoTable->validSize() ) ) {
		Console() << "UNIXInodeCache::basicFreeUnits inconsistent "
			<< _cachedUnits << " and " 
			<< _unitInfoTable->validSize() << "\n" << eor;
	}
#endif
	release();
	int log2InodeUnitSizeRatio = _log2UnitSize - _inodeLog2UnitSize;
	int inodeStartUnit = startUnit << log2InodeUnitSizeRatio;
	int inodeCount = count << log2InodeUnitSizeRatio;
	_inode->free( inodeStartUnit, inodeCount );
	acquire();
}

inline
void UNIXInodeCache::updateLRU()
{
	if ( &_link == _cursor ) {
		_cursor = _link.next();
	}	
	_link.unlink();
	_link.insertBefore( &_LRUList );
}

int UNIXInodeCache::read( unsigned  startUnit, unsigned  count, char * addr, 
		          IOAsyncClosure * c )
{
	updateLRUCount++;
	if ( ( updateLRUCount & 0x1ff ) == 0 && ( debugUNIXInodeCache & 0x2 ) )
		printParams( Console() );
	acquire();
	updateLRU();
	release();
	return RealCache::read( startUnit, count, addr, c );
}

int UNIXInodeCache::write(unsigned int startUnit, unsigned  count, CacheUnitInfo * cu,
			   char * addr, IOAsyncClosure * c )
{
	updateLRUCount++;
	if ( ( updateLRUCount & 0x1ff ) == 0 && ( debugUNIXInodeCache & 0x2 ) )
		printParams( Console() );
	acquire();
	updateLRU();
	if ( _cachedBytes >= _maxCachedBytes ) {
		release();
		if ( debugUNIXInodeCache & 0x2 ) Console()
			<< this << "::write bypass\n"
			<< eor;
		invalidate( startUnit, count, 0 );
		_stats._write.add( count );
		return writeMOModified( startUnit, count, cu, addr, 0, c );
	} else {
		release();
		return RealCache::write( startUnit, count, cu, addr, c );
	}
}

int UNIXInodeCache::rawWrite( unsigned startUnit, unsigned count, CacheUnitInfo * cu,
			      char * addr, IOAsyncClosure * c )
{
	updateLRUCount++;
	if ( ( updateLRUCount & 0x1ff ) == 0 && ( debugUNIXInodeCache & 0x2 ) )
		printParams( Console() );
	acquire();
	updateLRU();
	if ( _cachedBytes >= _maxCachedBytes ) {
		release();
		if ( debugUNIXInodeCache & 0x2 ) Console()
			<< this << "::rawWrite bypass\n"
			<< eor;
		invalidate( startUnit, count, 0 );
		_stats._rawWrite.add( count );
		return writeMOModified( startUnit, count, cu, addr, 1, c );
	} else {
		release();
		return RealCache::rawWrite( startUnit, count, cu, addr, c );
	}
}

int UNIXInodeCache::basicRead ( unsigned startUnit, unsigned count, char * addr,
			 	IOAsyncClosure * c )
{
	Raids( RaidProtected ) 
		<< this << "::basicRead( " << startUnit << ", " << count
		<< " )\n" << eor;

	int log2InodeUnitSizeRatio = _log2UnitSize - _inodeLog2UnitSize;
	int inodeStartUnit = startUnit << log2InodeUnitSizeRatio;
	int inodeCount = count << log2InodeUnitSizeRatio;
	
	int actualCount;

	_stats._basicRead.add( count );

	if ( c == 0 ) {
#ifdef NEEDNOT_LOCK
		actualCount = _inode->rawRead
			( inodeStartUnit, inodeCount, addr,
			  inodeCount << _inodeLog2UnitSize, c ); 
#else
		actualCount = _inode->read
			( inodeStartUnit, inodeCount, addr,
			  inodeCount << _inodeLog2UnitSize );
#endif
		if ( actualCount > 0 )
			actualCount = 
			( ( actualCount - 1 ) >> log2InodeUnitSizeRatio ) + 1;
	} else {
		actualCount = inodeCount;
		CacheMOIOClosure * myClosure =
			new CacheMOIOClosure
			( c, log2InodeUnitSizeRatio, actualCount );
#ifdef NEEDNOT_LOCK
		_inode->rawRead( inodeStartUnit, inodeCount, addr,
			         inodeCount << _inodeLog2UnitSize, myClosure );
#else
		_inode->asyncRead( inodeStartUnit, inodeCount, addr,
			           inodeCount << _inodeLog2UnitSize, myClosure);
#endif
	}

	return actualCount;
}

int UNIXInodeCache::basicWrite( unsigned startUnit, unsigned count, char * addr,
				IOAsyncClosure * c )
{
	Raids( RaidProtected ) 
		<< this << "::basicWrite( " << startUnit << ", " 
		<< count << " )\n" << eor;

	int log2InodeUnitSizeRatio = _log2UnitSize - _inodeLog2UnitSize;
	int inodeStartUnit = startUnit << log2InodeUnitSizeRatio;
	int inodeCount = count << log2InodeUnitSizeRatio;
	
	Assert( _inode->cache() == 0 );

	int actualCount;

	_stats._basicWrite.add( count );

	if ( c == 0 ) {
#ifdef NEEDNOT_LOCK
		actualCount = _inode->rawWrite
			( inodeStartUnit, inodeCount, addr,
			  inodeCount << _inodeLog2UnitSize, 0, c ); 
#else
		actualCount = _inode->write
			( inodeStartUnit, inodeCount, addr,
			  inodeCount << _inodeLog2UnitSize );
#endif
		if ( actualCount > 0 )
			actualCount = 
			( ( actualCount - 1 ) >> log2InodeUnitSizeRatio ) + 1;
	} else {
		actualCount = inodeCount;
		CacheMOIOClosure * myClosure = new CacheMOIOClosure
			( c, log2InodeUnitSizeRatio, actualCount );
#ifdef NEEDNOT_LOCK
		_inode->rawWrite( inodeStartUnit, inodeCount, addr,
			          inodeCount << _inodeLog2UnitSize, 0,
				  myClosure );
#else
		_inode->asyncWrite( inodeStartUnit, inodeCount, addr,
			            inodeCount << _inodeLog2UnitSize, 
				    myClosure );
#endif
	}

	return actualCount;	
}

void
UNIXInodeCache::cleaner()
{

    Console() << "UNIXInodeCache::cleaner " << thisProcess() << "\n" << eor;

    cleanerSem.P();

    while (1) {

	if ( debugUNIXInodeCache & 0x1 ) {
		Console()
		<< "UNIXInodeCache::cleaner() running\n" << eor;
		printParams( Console() );
	}

	UNIXInodeCache * cache;
	PersistentMemoryObject * pmo;
	ZombieInfo * info;
	acquire();
	_cursor = _LRUList.next();
	while ( ( _cachedFiles >= _thresholdCachedFiles ||
		  _cachedBytes >= _thresholdCachedBytes ) &&
		_cursor != &_LRUList ) {

		cache = _cursor->cache();
		pmo = (PersistentMemoryObject *) cache->_mo;
		info = pmo->zombieInfo();
		ZombieInfoType zombieType = info->type();
		if ( zombieType != ZOMBIE_NOT &&
		     !( zombieType == ZOMBIE_NOTSYNC && info->expire() == 0 ) )
		{
			if ( debugUNIXInodeCache & 0x4 ) Console() 
				<< "UNIXInodeCache::cleaner() killing "
				<< pmo << " cached units " 
				<< cache->_cachedUnits << "\n" << eor;
			MemoryObjectCachingContainer * container = 
				(MemoryObjectCachingContainer *)
				pmo->fastContainer();
			int idNumber = pmo->idNumber();
			release();
			container->killZombie( idNumber );
			acquire();
		} else {
			if ( debugUNIXInodeCache & 0x4 ) Console()
				<< "UNIXInodeCache::cleaner() "
				<< pmo << " in use cached units "
				<< cache->_cachedUnits << "\n" << eor;
			_cursor = _cursor->next();
		}
	};
	_cursor = &_LRUList;
	cleanerBlocked = 1;
	if ( debugUNIXInodeCache & 0x1 ) { 
		Console()
		<< "UNIXInodeCache::cleaner() blocked\n" << eor;
		printParams( Console() );
	}
	release();

	cleanerSem.P();
    }
}

void
UNIXInodeCache::cmd( int argc, char *argv[] )
{
	if ( argc == 1 ) {
		printParams( Console() );
		return;
	}
	if ( argc == 2 ) {
		debugUNIXInodeCache = atoi( argv[1] );
		Console() << "UNIXInodeCache debug 0x" 
			<< asHex( debugUNIXInodeCache )
			<< "\n" << eor;	
		return;
	}
	if ( argc != 3 ) {
		Console() 
		<< "Usage: " 
		<< "    " << argv[0] << "\n"
		<< "    " << argv[0] << " debug\n"
		<< "    " << argv[0] << " max-files max-megabytes\n" << eor;
		return;
	}
	_maxCachedFiles = atoi( argv[1] );
	_maxCachedBytes = atoi( argv[2] ) << 20;
	updateThresholds();
	printParams( Console() );
}

void
UNIXInodeCache_cmd( int argc, char *argv[] )
{
	UNIXInodeCache::cmd( argc, argv );
}

#include "ClassInstaller.h"
DEFINECLASS("UNIXInodeCache",UNIXInodeCache,RealCache,0,0);
