#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Swee Boon Lim (sblim@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * Use.cc
 */

#include "ThisProcess.h"
#include "Use.h"


#ifdef __GNUC__
#pragma implementation
#endif

TableUse::TableUse( int concurrentFaults )
{
	_allocatedSize = concurrentFaults;
	_last = 0;
}

Process *
TableUse::unuse( unsigned from, unsigned to )
{
	for ( int i = 0; i < _last; i++ ) {
		_UseInfo * useInfo = _table + i;
		if ( useInfo->equal( from, to ) ) {
			Process * process = useInfo->_process;
		        *useInfo = _table[ --_last ];
			return process;
		}
	}
	return 0;
}

Process *
TableUse::use( unsigned from, unsigned to, Process * process )
{
	for ( int i = 0; i < _last; i++ ) {
		if ( _table[ i ].intersect( from, to ) ) 
			return _table[ i ]._process;
	}
	if ( _last == _allocatedSize )
		doubleTable();
	if ( process == 0 )
		process = thisProcess();
	if ( process == 0 )
		process = (Process *) ~0;
	_table[ _last++ ].init( from, to, process );
	return 0;
}

Process *
TableUse::lessUse( unsigned from, unsigned to )
{
	Assert( _last == 1 );
	_UseInfo * useInfo = _table;
	Assert( useInfo->_from == 0 && useInfo->_to == MaxUse );
	Process * process = thisProcess();
	if ( process == 0 )
		process = (Process *) ~0;
	Assert( useInfo->_process == process );
	useInfo->_from = from;
	useInfo->_to = to;	
	return process;
}

int 
TableUse::noOfUsers()
{
	return _last;
}

VariableSizeTableUse::VariableSizeTableUse( int concurrentFaults )
	: TableUse( concurrentFaults )
{
	_table = new _UseInfo[ _allocatedSize ];
}

VariableSizeTableUse::~VariableSizeTableUse()
{
	delete _table;
}

#ifdef NEVER
int debugVariableTableUse = 1;
#endif

void
VariableSizeTableUse::doubleTable()
{
#ifdef NEVER
	if (debugVariableTableUse) {
		Console() 
		<< "VariableSizeTableUse[" << asHex((unsigned) this)
		<< "]::doubleTable() new size " << ( _allocatedSize << 1 )
		<< "\n" << eor;
	}	
#endif
	_allocatedSize <<= 1;
	_UseInfo * newTable = new _UseInfo[ _allocatedSize ];
	for ( int i = 0; i < _last; i++ )
		newTable[ i ] = _table[ i ];
	delete _table;
	_table = newTable;
}

ConstantSizeTableUse::ConstantSizeTableUse( int concurrentFaults )
     : TableUse( concurrentFaults )
{
	int preAllocatedSize = sizeof( _preAllocatedTable ) / 
	                       sizeof( _UseInfo );
	Assert( preAllocatedSize >= concurrentFaults );
	_table = _preAllocatedTable;
}

void 
ConstantSizeTableUse::doubleTable()
{
	Assert( NOTREACHED );
}

