#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Aamod Sane (sane@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "OutputStream.h"
#include "VMInterface.h"
#include "MemoryObject.h"
#include "PagedMemoryObjectCache.h"
#ifdef DVM
#include "SimpleVMCache.h"
#include "OneCopyDVMCache.h"
#include "DVMManager.h"
#include "COWCache.h"
#include "DVMCommunicator.h"
#endif
#include "Kernel.h"
#include "ThisProcess.h"
#include "Process.h"
#include "Domain.h"
#include "RecordStream.h"
#include "PersistentMemoryObject.h"
#include "PersistentMemoryObjectStar.h"
#include "String.h"
#include "FileSystemInterface.h"
#include "FileSystemInterfaceStar.h"
#include "NameServer.h"


#ifdef __GNUC__
#pragma implementation
#endif


void
VMInterface::test(char * message)
{
    Console() << this << "::test(" << message << ")\n" << eor;
}

// MemoryObjectCacheRef
// VMInterface::createCache(RecordStream * ro, Class * cacheClass)
// {
//     int status;
//     PersistentMemoryObjectStar mo =
// 	ro->asA(PersistentMemoryObjectClass, status);
// 
//     Assert( cacheClass->isASubclassOf(MemoryObjectCacheClass) );
// 
//     return cacheClass->construct( mo );
// }
    
// Create a cache for the given MemoryObject. The cache and the mo
// should refer to each other
MemoryObjectCache *
VMInterface::createCache(RecordStream * ro, char * cacheType)
{
    int status;
    PersistentMemoryObjectStar mo =
	ro->asA(PersistentMemoryObjectClass, status);
    
    MemoryObjectCache * cache = 0;
    ClassStar cacheClass = 
	    thisProcess()->domain()->nameServer()->lookup( cacheType, (char*)0 );
    mo->setCacheClass( cacheClass );
    cache = mo->buildCache();

/*
    if ( strcmp(cacheType,"PagedMemoryObjectCache") == 0 )
	cache = new PagedMemoryObjectCache( mo, unitsInCache );
#ifdef DVM
    else if ( strcmp(cacheType,"SimpleVMCache") == 0 )
	cache = new SimpleVMCache( mo, unitsInCache );
    else if ( strcmp(cacheType,"OneCopyDVMCache") == 0 )
	cache = new RCDVMCache( mo, unitsInCache, 0 ,0 );
    else if ( strcmp(cacheType,"COWCache") == 0 )
	cache = new COWCache( mo, unitsInCache );
#endif
    else
	Assert( NOTREACHED );

    mo->setMemoryObjectCache( cache );
*/
    return cache;
}

// Add to domain at an address chosen by the Domain
char *
VMInterface::addToDomain(RecordStream * ro, ProtectionLevel p)
{
    int status;
    PersistentMemoryObjectStar mo =
	ro->asA(PersistentMemoryObjectClass, status);

    Domain * myDomain = thisProcess()->domain();
    Assert( myDomain != 0 );
    // Fix - remove char * later
    return (char *)myDomain->add(mo,p);
}

// Create Caches and DVMManagers for origin and copy MemoryObjects.
// Caches and MO's should refer to each other, Caches and Managers
// should refer to each other, and the managers should refer to each other
// as the remote manager.
// Managers should have distinct ID's
// void
// VMInterface::setupLDVM(RecordStream *origin, RecordStream *copy)
// {
//     int status;
// 
//     // Create a OneCopyDVMCache and a DVM manager for the original
//     // MemoryObject. Cache and Manager should refer to each other
//     
//     PersistentMemoryObjectStar originmo =
// 	origin->asA(PersistentMemoryObjectClass, status);
//     
//     Assert(PagedMemoryObjectCacheLog2UnitSize >= originmo->log2UnitSize());
//     int unitSizeRatio = 1 << (PagedMemoryObjectCacheLog2UnitSize
// 			      - originmo->log2UnitSize());
//     int unitsInCache = (originmo->numberOfUnits()+unitSizeRatio- 1)/unitSizeRatio;
// 
//     Console() << originmo << ": unitsize=" << originmo->unitSize() << ",numunits="
// 	<< originmo->numberOfUnits() << "\n" << eor;
//     
//     // MO points to cache, cache points to MO
//     
//     OneCopyDVMCache * origincache =
// 	new OneCopyDVMCache( originmo, unitsInCache );
//     originmo->setMemoryObjectCache( origincache );
//     
//     // Manager points to cache, cache points to manager
//     // Number of machines in mgr = 2, mgr id = 1
//     DVMManager * originmgr = new DVMManager(2, 1, origincache);
//     origincache->setManager( originmgr );
// 
// 
//     // In a similar vein, create a OneCopyDVMCache and a DVM manager
//     // for the copy MemoryObject.
// 
//     
//     PersistentMemoryObjectStar copymo =
// 	copy->asA(PersistentMemoryObjectClass, status);
//     
//     Assert(PagedMemoryObjectCacheLog2UnitSize >= copymo->log2UnitSize());
//     unitSizeRatio = 1 << (PagedMemoryObjectCacheLog2UnitSize
// 			      - copymo->log2UnitSize());
//     unitsInCache = (copymo->numberOfUnits()+unitSizeRatio- 1)/unitSizeRatio;
// 
//     Console() << copymo << ": unitsize=" << copymo->unitSize() << ",numunits="
// 	<< copymo->numberOfUnits() << "\n" << eor;
//     
//     // MO points to cache, cache points to MO
//     // Cache is a copy cache, and the id of the remote mgr is 1
//     OneCopyDVMCache * copycache =
// 	new OneCopyDVMCache( copymo, unitsInCache, 1, 1 );
//     copymo->setMemoryObjectCache( copycache );
// 
//     // Manager points to cache, cache points to manager
//     // Number of machines in mgr = 2, mgr id = 2
//     DVMManager * copymgr = new DVMManager(2, 2, copycache);
//     copycache->setManager( copymgr );
// 
// 
//     // The managers should refer to each other as the remote mgr
//     
//     originmgr->add(copymgr);
//     copymgr->add(originmgr);
// }

void
VMInterface::setupCOW(RecordStream *origin, RecordStream *copy)
{
#ifdef DVM
    int status;

    // Create a OneCopyDVMCache and a DVM manager for the original
    // MemoryObject. Cache and Manager should refer to each other
    
    PersistentMemoryObjectStar originmo =
	origin->asA(PersistentMemoryObjectClass, status);
    
    Assert(PagedMemoryObjectCacheLog2UnitSize >= originmo->log2UnitSize());
    int unitSizeRatio = 1 << (PagedMemoryObjectCacheLog2UnitSize
			      - originmo->log2UnitSize());
    int unitsInCache = (originmo->numberOfUnits()+unitSizeRatio- 1)/unitSizeRatio;

    
    // MO points to cache, cache points to MO
    
    COWCache * origincache = (COWCache *)originmo->cache();
    Assert(origincache != 0);
    Console() << "Original: " << originmo << "," << origincache
	<< "\n" << eor;
    MemoryObjectCache * shadow = origincache->createShadow();
    Assert(shadow != 0);

    Console() << "Shadow created: " << shadow << "\n" << eor;
    
    PersistentMemoryObjectStar copymo =
	copy->asA(PersistentMemoryObjectClass, status);
    
    Assert(PagedMemoryObjectCacheLog2UnitSize >= copymo->log2UnitSize());
    unitSizeRatio = 1 << (PagedMemoryObjectCacheLog2UnitSize
			      - copymo->log2UnitSize());
    unitsInCache = (copymo->numberOfUnits()+unitSizeRatio- 1)/unitSizeRatio;

    COWCache * copycache =
	new COWCache( originmo, unitsInCache, originmo );
    Assert(copycache != 0);
    Console() << "Copy: " << copymo << "," << copycache << "\n" << eor;
    
    copycache->setShadow(shadow);
    copymo->setMemoryObjectCache(copycache);
    
#endif
}

void
VMInterface::networkTest(char * to)
{
#ifdef DVM
    if (TheDVMCommunicator == 0)
	TheDVMCommunicator = new DVMCommunicator();
    Assert( 0 != TheDVMCommunicator );

    extern int networkTestFlag;

    networkTestFlag = ~networkTestFlag;

    if (to) {
	char buf[256];
	for (int i=0; i < 256; i++)
	    buf[i]=(char)0xf;
	strncpy(buf,to,255);
	TheDVMCommunicator->networkSendTest(buf,256,to);
    }
#endif
}

char *
VMInterface::openDVM(char * path)
{
#ifndef DVM
	return 0;
#else
    Assert(path != 0);
    
    MemoryObjectStar mo = TheDVMManager->setupDVM(path);
    Assert(mo != 0);
    
    Domain * myDomain = thisProcess()->domain();
    Assert( myDomain != 0 );
    
    // FIX - remove char * later
    return (char *)myDomain->add(mo,ReadWrite);
#endif
}

void
VMInterface::closeDVM(char * path)
{
#ifdef DVM
    Assert(path != 0);
    TheDVMManager->teardownDVM(path);
#endif
}

char *
VMInterface::createSharedRegion( unsigned size )
{
     Assert( size > 0 );
     PersistentMemoryObjectStar pmo = 
	     TheKernel->allocateMemoryObjectForBackingStore();
     unsigned nunits = ( size + pmo->unitSize() - 1 ) >> pmo->log2UnitSize();
     Assert( nunits > 0 );
     pmo->setNumberOfUnits( nunits );
     const char * baseAddr = thisProcess()->domain()->add( pmo, ReadWrite, 1 );
     return ( char * ) baseAddr;
}

#include "ClassInstaller.h"
DEFINECLASS("VMInterface",VMInterface,ProxiableObject,0,0);
