#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801-2910
 *
 * Copyright (c) 1991 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Amit Parghi (parghi@cs.uiuc.edu)
 * Contributing Author: Nayeem Islam (nislam@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *	NetworkAdministrator.cc
 *
 *	Implement the architecture- and network-independent processing
 *	portions of the network administrator.
 *
 * $Id: NetworkAdministrator.cc 2 2004-06-03 01:58:28Z jcarlyle $
 *
 * $Log: NetworkAdministrator.cc,v $
 * Revision 1.1  2004/06/03 01:58:57  jcarlyle
 * Initial revision
 *
 * Revision 1.2  2004/04/21 22:47:47  dthomas1
 * Compiler fixes
 *   Changed the file to add DLPI support
 * Changed the makefile to reflect changes in vcethernetu
 *
 * Revision 1.1.1.1  2004/03/10 23:02:18  kircher
 *
 *
 */

//DD SYSIN *

//#define STATS 1
// #define ADD_TEST 1
//#define DEBUG 1
// #define RAID 1

#include "OutputStream.h"
#include "Raid.h"
#include "NameServer.h"
#include "Process.h"
#include "ThisProcess.h"
#include "NetworkStatistics.h"
#include "NetworkBuffer.h"
#include "NetworkDriver.h"
#include "NetworkBufferQueue.h"
#include "NetworkWalker.h"
#include "NetworkAdministrator.h"
#include "FreeRunningTimer.h"
#include "FreeRunningTimerStar.h"

#ifdef __GNUC__
#pragma implementation
#endif




const unsigned INITIAL_PROCESS_COUNT	= 1;

NetworkAdministrator::NetworkAdministrator(char *name,
					   unsigned char debugMask)
{
//    setDebugMessages(MaxObjectDebugMessages);
 setDebugMask(debugMask);

	walkerTime = 0;
	walkers = 0;

    Raids(RaidConstructor) << "NetworkAdministrator constructor (name=\""
		      << name << "\")\n" << eor;

    _receiveQueue = new NetworkBufferQueue();

    Raids(RaidDetail) << "NetworkAdministrator constructor: _receiveQueue = "
		      << asAddress(_receiveQueue) << '\n' << eor;

    setName(name);				// register self w/ NameServer
    StandardNameServer->bind(name, this);

    Raids(RaidDetail) << "NetworkAdministrator constr:NameServer bind OK\n"
		      << eor;

    Raids(RaidDetail) << "NetworkAdministrator constructor: "
		      << '\n' << eor;

    _stats = new NetworkStatistics();

    Process * receivers;
    for(int i=0;i<INITIAL_PROCESS_COUNT;i++) {
    receivers = new SystemProcess( (OPFV) &NetworkAdministrator::receiveLoop,thisProcess()->domain(), huge, (int) this);
    receivers->setName("NetworkAdministrator daemon");
    receivers->ready();
    //thisProcess()->relinquishProcessor();
    Raids(RaidDetail) << this << ": receiveLoop process started, proc * ="
                      << asAddress(receivers) << '\n' << eor;
   }

    Raids(RaidDetail) 
	<< "NetworkAdministrator constructor(\"" << name << "\") done,\n\t_bufferManager = " 
	<< "\n\t_receiveQueue = " << asAddress(_receiveQueue) << '\n' << eor;
	//  NOTE that any processes or initialization that require use
	//  of the driver **MUST** be delayed until ready() is called.
    _driver = NULL;
}


NetworkAdministrator::~NetworkAdministrator()
{
    Raids(RaidMethod) << "NetworkAdministrator destructor...\n" << eor;

    Raids(RaidMethod) << "NetworkAdministrator destructor done...\n" << eor;
}


void
NetworkAdministrator::ready(NetworkDriver *aDriver)
{
	//  Start up any driver-specific activities.  Also see subclasses.

    if (_driver != NULL) {
	Console() << this << "**** ready() called twice, driver="
		  << asAddress(aDriver) << " !!\n" << eor;
	Assert(NOTREACHED);
    } else {
	_driver = aDriver;		// assume driver already started up
    }
}


void
NetworkAdministrator::transmit(NetworkBuffer *aBuffer)
{
    Raids(RaidMethod) << "NetworkAdministrator::transmit(buffer = "
		      << asAddress(aBuffer) << ")\n" << eor;

    /*  Check the alignment of the buffer */
    Assert(((unsigned int)aBuffer & 3) == 0);

    _driver->transmitPacket(aBuffer);
#ifdef STATS
    _stats->transmitPacketCount++;
    _stats->transmitOctetCount += aBuffer->length();
#endif
}

/*
 * this function access the local queue.
 */

void
NetworkAdministrator::receive(NetworkBuffer *aBuffer)
{
   HexDump *hd = new HexDump(aBuffer->packet(),32);
    Raids(RaidMethod) << "NetworkAdministrator::receive(buffer = "
		      << asAddress(aBuffer) << ")\n"
		      << *hd << '\n' << eor;


    _receiveQueue->put(aBuffer);
/*
// try  something new: go all the way; should
// change the nature of the process at this point.
// without running on the old processes stack
// run all the way up 
        NetworkWalker *walker;
        walker = findWalker(aBuffer);
        if(walker) {
                walker -> walk (aBuffer);
        } else {
        aBuffer -> release();
        }
*/
#ifdef STATS
    _stats->receivePacketCount++;
    _stats->receiveOctetCount += aBuffer->length();
#endif
}


void
NetworkAdministrator::receiveLoop()
{
    NetworkBuffer *buffer;
    NetworkWalker *walker;

    Raids(RaidMethod) << "NetworkAdministrator::receiveLoop: starting.\n"
		      << eor;
    while (1) {
	    Raids(RaidDetail) << "NetworkAdmin loop: getting next buffer\n" << eor;
	buffer = _receiveQueue->get();// next buffer
#ifdef STATS 
	unsigned start = SystemTimer->time();
#endif
#ifdef ADD_TEST
// looks like this should be an assert
	if (buffer == 0) {
	    Raids(RaidDetail) << "NetworkAdmin loop: got null buffer, exiting\n"
			      << eor;
	    break;
	    }
#endif
	    Raids(RaidDetail) << "NetworkAdmin loop: got buffer, pointer ="
			      << asAddress(buffer) << '\n' << eor;

	walker = findWalker(buffer);
 	if(walker) {
	 	Debug << "walker found  " << eor;
		walker -> walk (buffer); 
	}// let walker release buffer
        else {
	buffer -> release();// release buffer
	Debug << "no walker buffer released" << eor;
	}
#ifdef STATS
	unsigned end = SystemTimer->time();
	walkerTime += (end- start);
	walkers++;
#endif
    }
    Raids(RaidDetail) << "NetworkAdmin loop: done\n" << eor;
}

void
NetworkAdministrator::clearStats()
{
walkerTime = 0;
walkers = 0;
}
void
NetworkAdministrator::printStats()
{
Console() << " Total time spent in walkers " << walkerTime << "\n" << eor;
Console() << " Total number of walkers called " << walkers << "\n" << eor;
}

#include "ClassInstaller.h"
DEFINECLASS("NetworkAdministrator",NetworkAdministrator,ProxiableObject,0,0);
