#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801-2910
 *
 * Copyright (c) 1990,1991 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Amit Parghi (parghi@cs.uiuc.edu)
 * Contributing Author: Nayeem Islam (nislam@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *	NetworkBuffer.cc
 *
 *	Methods for the NetworkBuffer and related classes.
 *
 * $Id: NetworkBuffer.cc 2 2004-06-03 01:58:28Z jcarlyle $
 *
 * $Log: NetworkBuffer.cc,v $
 * Revision 1.1  2004/06/03 01:58:57  jcarlyle
 * Initial revision
 *
// Revision 1.1  1992/12/02  02:59:07  nislam
// Initial revision
//
 */
// #define DEBUG 1

#include "OutputStream.h"
#include "Semaphore.h"
#include "Process.h"
#include "ThisProcess.h"
#include "Domain.h"
#include "Kernel.h"
#include "Byte.h"
#include "NetworkStatistics.h"
#include "NetworkBuffer.h"
#include "NetworkBufferManager.h"
#include "NetworkWalker.h"
#include "Types.h"
#include "NameServer.h"


#ifdef __GNUC__
#pragma implementation
#endif


	//  Use this method when we already have the data sitting in a
	//  char array somewhere, i.e. when we want to encapsulate existing
	//  data in a NetworkBuffer object (e.g. when transmitting).
NetworkBuffer::NetworkBuffer(octet *pk,
			     unsigned int len,
			     NetworkBufferFreeList *p)
{
    constructorCommon(pk, len, p);

    _flags = 0;
}


	//	Use this method to create a new NetworkBuffer out of nothing;
	//	this means new allocation of the actual char array for the
	//	packet.
NetworkBuffer::NetworkBuffer(unsigned int len, NetworkBufferFreeList *p)
{
    constructorCommon(new octet[len], len, p);

#   if defined(DEBUG)
	Debug << "NetworkBuffer constructor(len=" << len
	      << ", list=" << asHex((int) p)
	      << ", this = " << asHex((int) this)
	      << ")\n" << eor;
#   endif

    _flags = NetworkBufferFlags_DeletePacket;
}


void NetworkBuffer::constructorCommon(octet *pk,
				      unsigned int len,
				      NetworkBufferFreeList *p)
{
    _packet = pk;
    _length = len;
    _maxLength = len;
    _freeList = p;
}

	//	Destructor has to know whether to delete the char array or not;
	//	this depends on which of the above constructors was called.
NetworkBuffer::~NetworkBuffer()
{
    if (_flags & NetworkBufferFlags_DeletePacket) {
	delete _packet;
    }
}


    void
NetworkBuffer::release()		// put myself back on my free list
{
    if (freeList() != NULL) {
	freeList()->releaseBuffer(this);
    }
}

void			// copy into NetworkBuffer from user's data area
NetworkBuffer::copyFrom(const octet *source, unsigned int length)
{
    if (packet() != NULL) {
	copy(source, packet(), length);
	setLength(length);
    }
}

    unsigned int
NetworkBuffer::copyTo(octet *destination)
{
    copy(packet(), destination, length());
    return length();
}

    void		// block copy of length bytes from source to dest
NetworkBuffer::copy(const octet *source,
		    octet *destination,
		    unsigned int length)
{

	// And finish off the remaining bytes, one by one ...
    while (length > 0) {
	*destination = *source;
	++source;
	++destination;
	--length;
    }
}


NetworkBufferFreeList::NetworkBufferFreeList(unsigned int bufferSize, unsigned int listSize ,int init)
{
     _length = bufferSize;
    _count = listSize;
    _totalCount = listSize;

    _mutex = new Lock;

	Debug << "NetworkBufferFreeList constructor:\n"
	      << ", bufferSize = " << bufferSize
	      << ", listSize = " << listSize << ")\n" << eor;

	if(!init) {
		initialize(bufferSize,listSize);
	}
}

void
NetworkBufferFreeList::initialize(unsigned int bufferSize, unsigned int listSize)
{
	Debug << "NetworkBufferFreeList::initialize" << eor;
	Debug << " WRONG " << eor;


	NetworkBuffer *tmp;
    _list = new NetworkBuffer(_length, this);	// set first item

    for (int i = 0 ; i < _count ; i++) {// put empty buffers in list
    tmp = new NetworkBuffer(_length, this);	// set first item
        tmp->addAfter(_list);
        // Unused! char *pkt = (char *)(tmp->packet());
        // Unused! unsigned len = tmp->length();
    }

}

void
NetworkBufferFreeList::listElems() {
	NetworkBuffer* t;
	t = _list;

    for (int i = 0 ; i < _count ; i++) {
	Debug << " addr in loop " <<  i << " "<< eor;
	Debug << asAddress(t->packet()) << "\n" << eor;
	Debug << asAddress(t->kpacket()) << "\n" << eor;
	Debug << ((unsigned)t->packet()) -((unsigned)t->kpacket()) % (1 << 16)  << " " << eor;
	Debug << ((unsigned)t->packet()) -((unsigned)t->kpacket()) % (1 << 18)  << "\n" << eor;
	t = (NetworkBuffer*) t->next();
	}
}

NetworkBufferFreeList::~NetworkBufferFreeList()
{
    delete _list;
}


    NetworkBuffer *
NetworkBufferFreeList::getBuffer()	// get NetworkBuffer from the free list
{
    NetworkBuffer *returnValue;

#   ifdef DEBUG
	Debug << "NetworkBufferFreeList::getBuffer:  this = "
	      << asHex((unsigned int) this)
	      << ", _list = " << asHex((unsigned int) _list)
	      << "\n\t_mutex =" << asHex((unsigned int) _mutex)
	      << '\n' << eor;
#   endif

    _mutex->acquire();// critical section: lock free list
    Assert (_count > 0);
    returnValue = (NetworkBuffer *) _list->next();

    Debug << "NetworkBufferFreeList::getBuffer:  returnValue = "
	  << returnValue << '\n' << eor;
	returnValue->remove(); 

    Debug << "NetworkBufferFreeList::getBuffer: removed from list\n" << eor;
    _count--;
    _mutex->release();

    return returnValue;
}

   void               // return buffer to free list
NetworkBufferFreeList::releaseBuffer(NetworkBuffer *buff)
{
    _mutex->acquire();  // critical section: modifying list
    Assert (_count <= _totalCount );

    buff->addAfter(_list); // add it to the end of our list
    ++_count;

    _mutex->release();

}
#include "ClassInstaller.h"
DEFINECLASS("NetworkBufferFreeList",NetworkBufferFreeList,ProxiableObject,0,0);
