#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1990,1991 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Amit Parghi (parghi@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *	NetworkBufferManager.cc
 *
 *	Maintain the NetworkBufferFreeLists for the NetworkWalker(s).
 *
 * $Id: NetworkBufferManager.cc 2 2004-06-03 01:58:28Z jcarlyle $
 *
 * $Log: NetworkBufferManager.cc,v $
 * Revision 1.1  2004/06/03 01:58:57  jcarlyle
 * Initial revision
 *
// Revision 1.1  1993/03/22  20:01:36  raila
// Initial revision
//
 * Revision 1.3  1991/12/01  05:57:30  parghi
 * Misc. changes.  Last revision before networking redesign.
 *
 * Revision 1.2  91/09/25  16:20:49  parghi
 * Some clean-ups and major fixes prior to Stable-izing.
 * 
 * Revision 1.1  91/04/14  23:10:47  parghi
 * Initial revision
 *
 */

//DD SYSIN *

// #define RAID 1
// #define DEBUG 1

#include "Process.h"
#include "ThisProcess.h"
#include "Semaphore.h"
#include "OutputStream.h"
#include "Raid.h"
#include "Types.h"
#include "NetworkBuffer.h"
#include "NetworkBufferManager.h"



#ifdef __GNUC__
#pragma implementation
#endif
void _NetworkBufferManagerDaemon(NetworkBufferManager *);

#ifdef FIXTHIS 
const unsigned BUFFER_COUNT = 32;
#else
const unsigned BUFFER_COUNT = 42;
#endif


NetworkBufferManager::NetworkBufferManager(unsigned char debugMask)
{
    // setDebugMask(debugMask);				// for Raid
    // setDebugMessages(MaxObjectDebugMessages);

    Raids(RaidMethod) << "NetworkBufferManager constructor starting.\n" << eor;

// setting freelist size
    _list = new NetworkBufferFreeList(2000, BUFFER_COUNT);
     _list->listElems();
    _semaphore = new Semaphore(0);

    Raids(RaidDetail) << "NetworkBufferManager: _semaphore = "
		      << asHex((int) _semaphore._pointer)
		      << ", _list = " << asHex((int) _list) << '\n' << eor;

/*
    _daemonProcess = new PreemptableSystemProcess(
					(PFV) _NetworkBufferManagerDaemon,
					thisProcess()->domain(),
					normal,
					(int) this);
    _daemonProcess->setName("NetworkBufferManagerDaemon");
    _daemonProcess->ready();

    Raids(RaidDetail) << "NetworkBufferManager: started _daemonProcess = "
		      << asHex((int) _daemonProcess._pointer) << '\n' << eor;
*/
}


NetworkBufferManager::~NetworkBufferManager()
{
    delete _list;
}


    NetworkBufferFreeList *
NetworkBufferManager::getFreeList()
{
    return _list;
}


    void		// use synchronization semaphore to start the Daemon
NetworkBufferManager::launchDaemon()
{
    _semaphore->V();
}


NetworkGenericBufferManager::NetworkGenericBufferManager(unsigned int size)
{
    _size = size;

#   ifdef DEBUG
	Debug <<
	    "NetworkGenericBufferManager: constructor starting, using size "
	    << _size << '\n' << eor;
#   endif

    _list = new NetworkBufferFreeList(2000, _size, BUFFER_COUNT);

}

NetworkGenericBufferManager::~NetworkGenericBufferManager()
{
}


static const int NetworkBufferMinBufferCount = 5;
static const int NetworkBufferMaxBufferCount = 20;
// currently this daemon does nothing
    void
_NetworkBufferManagerDaemon(NetworkBufferManager *mgr)
{
#   ifdef DEBUG
	Debug << "NetworkBufferManager Daemon starting up, BufferManager = "
	      << asHex((unsigned int) mgr) << '\n' << eor;
#   endif

    while (1) {

	mgr->_semaphore->P();		// wait for synchronization
	// mgr->_semaphore->V();

#	ifdef DEBUG
	    Debug << "NetworkBufferManagerDaemon: checking free list...\n"
		  << eor;
#	endif

	    //	Check the current size of each list.  If necessary,
	    //	expand or contract the list ... this is a policy module.

	if (mgr->_list->_count < NetworkBufferMinBufferCount) {
#	    ifdef DEBUG
		Debug << "NetworkBufferManagerDaemon: extending free list,"
		      << " _count = " << mgr->_list->_count
		      << ", mincount = " << NetworkBufferMinBufferCount
		      << '\n' << eor;
#	    endif

	    NetworkBuffer *buf;
	    for ( int i = 0 ; i < 5 ; i++ ) {      // extend the list ...
		buf = new NetworkBuffer(mgr->_list->_length, mgr->_list);
		mgr->_list->releaseBuffer(buf);
	    }

	} else if (mgr->_list->_count > NetworkBufferMaxBufferCount) {
						//	shrink the free list

#	    ifdef DEBUG
		Debug << "NetworkBufferManagerDaemon: reducing free list,"
		      << " _count = " << mgr->_list->_count
		      << ", maxcount = " << NetworkBufferMaxBufferCount
		      << '\n' << eor;
#	    endif
	}

    }
}
