#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801-2910
 *
 * Copyright (c) 1991 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Amit Parghi (parghi@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *	NetworkBufferQueue.cc
 *
 *	
 *
 * $Id: NetworkBufferQueue.cc 2 2004-06-03 01:58:28Z jcarlyle $
 *
 * $Log: NetworkBufferQueue.cc,v $
 * Revision 1.1  2004/06/03 01:58:57  jcarlyle
 * Initial revision
 *
 */

//DD SYSIN *

// #define DEBUG 1
//#define RAID 1


#include "Semaphore.h"
#include "Types.h"
#include "NetworkBuffer.h"
#include "NetworkBufferQueue.h"
#include "OutputStream.h"
#include "List.h"
#include "Lock.h"
#include "Semaphore.h"
#include "Raid.h"


#ifdef __GNUC__
#pragma implementation
#endif


NetworkBufferQueue::NetworkBufferQueue()
{
    _lock = new Lock;
    _semaphore = new Semaphore(0);

    _queue = new List;

    Debug << "NetworkBufferQueue[" << asAddress(this) 
	  << "]::NetworkBufferQueue()\n" << eor;
}


NetworkBufferQueue::~NetworkBufferQueue()
{
  NetworkBuffer		*entry;
  int			count;

  Debug << "NetworkBufferQueue destructor...\n" << eor;

	/*
	 * empty out the queue, V all the waiters so
	 * _semaphore can be destroyed.
	 * When we awaken them they get 0 returns.
	 */
  _lock->acquire();				// should we even bother??
  for (count = 0; entry = (NetworkBuffer *) _queue->removeFirst(); count++)
    delete entry;

  _lock->release();
  while (count--)
    _semaphore->V();
}


void
NetworkBufferQueue::put(NetworkBuffer *buffer)
{
    Debug << "NetworkBufferQueue::put(buffer="
	  << asAddress(buffer) << ")\n" << eor;

    _lock->acquire();
    _queue->addLast(buffer);
    _lock->release();

    _semaphore->V();				// increment counting semaphore
}


NetworkBuffer *
NetworkBufferQueue::get()
{
    NetworkBuffer *returnValue;

    _semaphore->P();				// block if the queue is empty

    _lock->acquire();
    returnValue = (NetworkBuffer *) _queue->removeFirst();
    _lock->release();

    Debug << "NetworkBufferQueue::get() returning "
	  << asAddress(returnValue) << '\n' << eor;

    return returnValue;
}
