#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801-2910
 *
 * Copyright (c) 1991 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Amit Parghi (parghi@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *	NetworkDriver.cc
 *
 * $Id: NetworkDriver.cc 2 2004-06-03 01:58:28Z jcarlyle $
 *
 * $Log: NetworkDriver.cc,v $
 * Revision 1.1  2004/06/03 01:58:57  jcarlyle
 * Initial revision
 *
 * Revision 1.3  1991/12/01  05:58:48  parghi
 * Cleaned up as abstract superclass.  Last revision prior to rewrite.
 *
 * Revision 1.2  91/09/25  16:20:55  parghi
 * Cleaned up for Stable and minor changes.
 * 
 * Revision 1.1  91/04/14  23:09:23  parghi
 * Initial revision
 *
 */

//DD SYSIN *


#include "NameServer.h"
#include "NetworkStatistics.h"
#include "NetworkDriver.h"
#include "NetworkAdministrator.h"
#include "NetworkBuffer.h"
#include "Raid.h"


#ifdef __GNUC__
#pragma implementation
#endif


NetworkDriver::NetworkDriver(unsigned char mask, NetworkBufferFreeList* list)
{
    setDebugMask(mask);					// for Raid
    setDebugMessages(MaxObjectDebugMessages);

    Raids(RaidMethod) << "NetworkDriver constructor (RAID mask = "
		      << asHex(mask) << ")\n" << eor;

    _driverStats = new NetworkStatistics;
    flist = list;
    if(!list) {
	flist = new NetworkBufferFreeList(2000,32);
        flist->setName("nbfl");
	StandardNameServer->bind("nbfl",flist);
	}
    //flist->listElems();
}


NetworkDriver::~NetworkDriver()
{
    delete _driverStats;
}


    NetworkStatistics *
NetworkDriver::getStats()		// return statistics
{
    return _driverStats;
}

		// set the pointer to the NetworkAdministrator
void NetworkDriver::ready(NetworkAdministrator *admin)
{
    Raids(RaidMethod) << "NetworkDriver::ready(admin=" << asHex((int) admin)
		      << ")\n" << eor;

    _administrator = admin;
}


void 
NetworkDriver::receivePacket(NetworkBuffer *buf)
{
	Debug << "NetworkDriver::receivePacket" << eor; 
	_administrator->receive(buf);
}


#include "ClassInstaller.h"
DEFINECLASS("NetworkDriver", NetworkDriver, ProxiableObject, 0, 0);
