#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-92
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Johnny Zweig (zweig@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * ConduitMessage.cc
 *
 */

#include "OutputStream.h"
#include "Assert.h"
#include "ThisProcess.h"
#include "ConduitTypes.h" 
#include "Word.h"
#include "IPChecksum.h"
#include "ZootAddress.h"
#include "ConduitMessage.h"
#include "ConduitProtocolIdentifiers.h"


#ifdef __GNUC__
#pragma implementation
#endif


#ifndef QUIET
#undef Debug
#define Debug Console()
#endif

/* ---------- Class ConduitMessage member functions ---------- */

ConduitMessage::ConduitMessage(int hsz, int dsz) 
{
	octet * finger;

	_proto = Undefined;
	_control = 0;
	_header = 0;
	_headerLength = 0;
	_data = 0;
	_dataLength = 0;
	_link = 0;
	_holder = 0;

	_status = 0;	// Maybe nothing needs to be deleted
	if( hsz != 0 && dsz != 0 ) {
		_header = new octet[ ( hsz + dsz ) ];
		_data = _header + hsz;
		for (finger = _header; finger < _header + hsz + dsz; finger++) {
			*finger = 0;
		}
		_headerLength = hsz;
		_dataLength = dsz;
		_status |= 0x01;// set bit 0 so that everything eventually
				// gets deleted (_header points to both).
	}
	else if (hsz != 0) {
		_header = new octet[hsz];
		for (finger = _header; finger < _header + hsz; finger++) {
			*finger = 0;
		}
		_headerLength = hsz;
		_status |= 0x01;	// set bit 0 so that header eventually
					// gets deleted.
	}
	else if (dsz != 0) {
		_data = new octet[dsz];
		for (finger = _data; finger < _data + dsz; finger++) {
			*finger = 0;
		}
		_dataLength = dsz;
		_status |= 0x02; // set bit 1 so data gets deleted.
	}
}

ConduitMessage::ConduitMessage(int hsz, int dsz, ConduitMessage * pld) 
{
	octet * finger;

	_proto = Undefined;
	_control = 0;
	_header = 0;
	_headerLength = 0;
	_data = 0;
	_dataLength = 0;
	_link = 0;
	_holder = 0;

        _status = 0;     // Maybe nothing needs to be deleted
	if( hsz != 0 && dsz != 0 ) {
		_header = new octet[ ( hsz + dsz ) ];
		_data = _header + hsz;
		for (finger = _header; finger < _header + hsz + dsz; finger++) {
			*finger = 0;
		}
		_headerLength = hsz;
		_dataLength = dsz;
		_status |= 0x01;// set bit 0 so that everything eventually
				// gets deleted (_header points to both).
	}
        else if (hsz != 0) {
                _header = new octet[hsz];
		for (finger = _header; finger < _header + hsz; finger++) {
			*finger = 0;
		}
                _headerLength = hsz;
                _status = _status | 0x01; // set bit 0 so that header
                                    // eventually gets deleted.
        }
        else if (dsz != 0) {
                _data = new octet[dsz];
		for (finger = _data; finger < _data + dsz; finger++) {
			*finger = 0;
		}
                _dataLength = dsz;
                _status |= 0x02; // set bit 1 so data gets deleted.
        }
	if (pld != 0) {
		_link = pld;
		_status |= 0x04;	// set bit 2 so payload will get
				//  deleted (so if payload wants to
				//  delete enclosing message, it needs
				//  to twiddle the right status bit or
				//  something....)
	}
}

ConduitMessage::~ConduitMessage() 
{	// Just have to run through status bits, deleting
			//  as necessary.
	if (_status & 0x01) {	// need to delete header
		Assert( 0 != _header );
		delete _header;
	}
	if (_status & 0x02) {	// need to delete data
		Assert( 0 != _data );
		delete _data;
	}
	if (_status & 0x04) {	// need to delete payload
		Assert( 0 != _link );
		delete _link;
	}
}

int
ConduitMessage::length() 
{
	if( _link == 0 ) {
	   return( _headerLength + _dataLength );
	}
	else {
	   return( _headerLength + _dataLength + _link->length() );
			// Aren't virtual-functions swell?
	}
}

void
ConduitMessage::del() 
{
	ConduitMessage *m = _holder;
	if( m == 0 ) {
	    delete this;
	    return;
	}
	while( m->_holder != 0 ) m = m->_holder;
	delete m;
}

ConduitMessage *
ConduitMessage::clone()
{
	ConduitMessage * msg;

	msg = new ConduitMessage( _headerLength, _dataLength );
	PhotoCopy( this, msg );
	return( msg );
}
