#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Johnny Zweig (zweig@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * MessageBuffer.cc
 *
 */


#include "Assert.h"
#include "Lock.h"
#include "CPU.h"
#include "Semaphore.h"
#include "OutputStream.h"
#include "Domain.h"
#include "Process.h"
#include "ThisProcess.h"
#include "TimeoutTimer.h"
#include "ZootConfig.h"
#include "ZootReturnCodes.h"
#include "ConduitTypes.h"
#include "Word.h"
#include "ZootClock.h"
#include "ZootAddress.h"
#include "ConduitMessage.h"
#include "Conduit.h"
#include "ZootStateMachine.h"
#include "ConduitMessageBuffer.h"
#include "ConduitMessageBuffer.h"

#ifdef __GNUC__
#pragma implementation
#endif


#define put(x) Debug << x << eor;

// ------------------ ConduitMessageBuffer member functions: ------------------ 
ConduitMessageBuffer::~ConduitMessageBuffer()
{
	for( int i = 0; i < _size; i++ ){
	   if( msgs[i] != 0 ) delete msgs[i];
	}
	delete msgs;
	delete addrs;
}
MessageID
ConduitMessageBuffer::insert( Return_Code & ret, ConduitMessage * newmsg,
			NetworkAddress * newaddr )
{//JZ
put("a")
	if( 0 == newmsg ) {
	   ret = ERROR;
	   return( -1 );
	}

	//grab();
put("b")

	if( _count >= _size ) {
put("c")
	    //release();
	    ret = BUFFER_FULL;
	    return( -1 );
	}

	if( _next == _size ) _next = 0; // Wrap around

	Assert( 0 <= _next && _size > _next );
	Assert( 0 <= _count );

	msgs[ _next ] = newmsg;
	addrs[ _next ] = newaddr;
	_count++;
	_next++;

	//release();
put("d")

	ret = OK;
	return( _next - 1 ); // -1 because I post-incremented _next
}
Return_Code
ConduitMessageBuffer::del( MessageID handle )
{
	if( handle == -1 || msgs[ handle ] == 0 ) {
	   return( NO_SUCH_MESSAGE );
	}
	//grab();
	msgs[ handle ]->del();
	msgs[ handle ] = 0;
	addrs[ handle ] = 0;
	_count--;
	//release();
	return( OK );
}
Return_Code
ConduitMessageBuffer::connect( Conduit * target )
{
	Assert( NOTREACHED );
	// Connecting a buffer that doesn't know how is an error
	return( ERROR );
}
Return_Code
ConduitMessageBuffer::disconnect()
{
	Assert( NOTREACHED );
	// Disconnecting a buffer that doesn't know how is an error
	return( Return_Code(0) );
}
Return_Code
ConduitMessageBuffer::send( MessageID handle )
{
	Assert( NOTREACHED );
	// Sending a message from a buffer that doesn't know how is an error
	return( ERROR );
}
void
ConduitMessageBuffer::flush()
{
	Assert( NOTREACHED );
	// Flushing a buffer that doesn't know how is an error
}
void
ConduitMessageBuffer::reset()
{
	//grab();

	for( int i = 0; i < _size; i++ ){
	   if( msgs[i] != 0 ) {
		msgs[i]->del();
		msgs[i] = 0;
		addrs[i] = 0;
	   }
	}
	_count = 0;
	_next = 0;
	//release();
}
// ---------------- TCPWaitBuffer member functions: ----------------
TCPWaitBuffer::TCPWaitBuffer()
{
	//Debug << "TCPWaitBuffer::TCPWaitBuffer() called...\n" << eor;
}
TCPWaitBuffer::TCPWaitBuffer( int sz ) : ConduitMessageBuffer( sz )
{
        Debug << this << "::ctor( sz=" << sz << " )\n" << eor;
}
TCPWaitBuffer::~TCPWaitBuffer()
{
	// Nothing needs to be done above and beyond what the CMB destructor
	// does.
}
Return_Code
TCPWaitBuffer::del( MessageID handle )
{
	// Do NOT actually delete the message!
	if( handle == -1 ) return( ERROR );

	if( handle < 0 || handle > _size ) return( ERROR );

	//grab();
	msgs[ handle ]  = 0;
	addrs[ handle ]  = 0;
	_count--;
	//release();
	return ( OK );
}

//--------------------------------------------------------------------
// OK, here's the deal:  according to the Conduit paradigm, whenever
//   a message gets inserted into another conduit, that conduit _will_
//   eventually delete the message.  That's just life.  Since a retransmit
//   buffer (or ARB) doesn't want this to happen, it needs to insert
//   _copies_ of its messages, marked pleaseDontDeletePayload, in a tricky
//   way, so that the original is still lying around.  The sensible solution
//   to this problem is to use first-class classes -- just ask the class of
//   a message to produce a new instance, and copy the stuff over. This
//   bogus kludge with msg->clone() circumvents that.  Anyway, this function
//   makes a copy of the message, linked to whatever the message is linked to,
//   that can safely be inserted into abother conduit and deleted without
//   clobbering the original message. "Copy" here means message of the same
//   type with copies of the header and data -- it won't have the same _holder,
//   _link, for instance.
ConduitMessage *
COPY( ConduitMessage * msg )
{
	ConduitMessage * newmessage = msg->clone();
	newmessage->setLink( msg->link() );
	newmessage->pleaseDontDeletePayload();
	return( newmessage );
}
// ---------------- RetransmitBuffer member functions: ----------------
RetransmitBuffer::RetransmitBuffer()
{
}
RetransmitBuffer::RetransmitBuffer( int sz ) : ConduitMessageBuffer( sz )
{
}
RetransmitBuffer::RetransmitBuffer( int sz, 
				Conduit * initialDestination ) : ConduitMessageBuffer( sz )
{
	_dest = initialDestination;
}
RetransmitBuffer::~RetransmitBuffer()
{
}
MessageID
RetransmitBuffer::insert( Return_Code & ret,
			ConduitMessage * newmsg, NetworkAddress * newaddr )
{
	MessageID tmp;

	tmp = this->ConduitMessageBuffer::insert( ret, newmsg, newaddr );
	if( ret != OK ) return( tmp );

	Assert( _dest != 0 );
	// Makes sense to send it once immediately upon being buffered:
	ret = _dest->insertFromAbove( COPY(newmsg), newaddr );
	return( tmp );
}
void
RetransmitBuffer::reset()
{
	int i;

	for( i = 0; i < _size; i++ ) {
	   if( msgs[i] != 0 ) {
		delete msgs[i];
		msgs[i] = 0;
		addrs[i] = 0;
	   }
	}
	_count = 0;
	_next = 0;
}
Return_Code
RetransmitBuffer::connect( Conduit * target )
{
	_dest = target;
	return( OK );
}
Return_Code
RetransmitBuffer::send( MessageID handle )
{
	if( _dest == 0 ) return( ERROR );

	if( msgs[ handle ] != 0 ) {
	   return( _dest->insertFromAbove( COPY(msgs[handle]),
				    addrs[handle] ) );
	}
	else {
	   return( NO_SUCH_MESSAGE );
	}
}
void
RetransmitBuffer::flush()
{
	int i;

	if( _dest == 0 ) return; // ERROR!

	//grab();
	_next = _next % _size; // Since _next may have been incremented
	for( i = (_next + _size - _count) % _size;
	     i != _next;
	     i = (i + 1) % _size ) {
	   if( msgs[i] != 0 ) {
	   // A redundant check, but safe....
	      Assert( 0 != addrs[i] );
	      _dest->insertFromAbove( COPY(msgs[i]), addrs[i] );
	   }
	}
	//release();
}
Return_Code
RetransmitBuffer::disconnect()
{
	_dest = 0;

	return( OK );
}


// ------------------ AutoRetransmitBuffer member functions: ------------------ 

static Lock ARBInitLock;
SemaphoreStar TheARBMutexSemaphore; // Ugly ugly ugly
AutoRetransmitBuffer *	AutoRetransmitBuffer::_ARBList;

AutoRetransmitBuffer::AutoRetransmitBuffer()
{
	Process * process;
	int32 tmp;
	int i;

	Debug << this << "::ctor()" << eor;

	// Parent constructor intializes _size=100, _count=0, _dest=0
	_lastsent = new int32[100];
	tmp = now32().val();
	for( i = 0; i < 100; i++ ) {
		_lastsent[i] = tmp;
	}
	_delay = ARB_RETRANSMIT_INTERVAL;
		//   (The clock ticks once per 4 microseconds)

	if ( TheARBMutexSemaphore == (Semaphore *) 0 ) {
		ARBInitLock.acquire();
		TheARBMutexSemaphore = new MutexSemaphore;
		TheARBMutexSemaphore->setName( "#TheARBMutexSemaphore" );
		ARBInitLock.release();
	}
	// Brashly put myself at the head of the list of ARBs:
	TheARBMutexSemaphore->P();

	_nextARB = _ARBList;
	if( _ARBList == 0 ) {	// I must be the first ARB to be created, so
				//  it's my job to launch the daemon.
	    process = new 
		PreemptableSystemProcess((PFV) AutoRetransmitBufferDaemon, 
					 thisProcess()->domain(), small, 0 );
	    Assert( process != 0 );
	    process->setName( "#ARB retransmit daemon" );
	    
	    process->ready();
	}
	_ARBList = this;

	TheARBMutexSemaphore->V();

}
AutoRetransmitBuffer::AutoRetransmitBuffer( int sz ) : RetransmitBuffer( sz )
{
	Process * process;
	int32 tmp;
	int i;

	Debug << "AutoRetransmitBuffer::AutoRetransmitBuffer(size=" << sz << ") called... this=" << this << "\n" << eor;

	_delay = ARB_RETRANSMIT_INTERVAL;
	_lastsent = new int32[sz];
	tmp = now32().val();
	for( i = 0; i < sz; i++ ) {
		_lastsent[i] = tmp;
	}
	if ( TheARBMutexSemaphore == (Semaphore *) 0 ) {
		ARBInitLock.acquire();
		TheARBMutexSemaphore = new MutexSemaphore;
		TheARBMutexSemaphore->setName( "#TheARBMutexSemaphore" );
		ARBInitLock.release();
	}
	// Brashly put myself at the head of the list of ARBs:
	TheARBMutexSemaphore->P();

	_nextARB = _ARBList;
	if( _ARBList == 0 ) {	// I must be the first ARB to be created, so
				//  it's my job to launch the daemon.
	process = new PreemptableSystemProcess(
		(PFV) AutoRetransmitBufferDaemon, thisProcess()->domain(),
		small, 0 );
	Assert( process != 0 );
	process->setName( "#ARB retransmit daemon" );

	process->ready();
	}
	_ARBList = this;
	TheARBMutexSemaphore->V();
}
AutoRetransmitBuffer::AutoRetransmitBuffer( int sz, int32 dlay ) : RetransmitBuffer( sz )
{
	Process * process;
	int32 tmp;
	int i;

	//Debug << "AutoRetransmitBuffer::AutoRetransmitBuffer(size=" << sz << ", delay=" << dlay << ") called... this=" << this << "\n" << eor;

	_delay = dlay;
	_lastsent = new int32[sz];
	tmp = now32().val();
	for( i = 0; i < sz; i++ ) {
		_lastsent[i] = tmp;
	}
	if ( TheARBMutexSemaphore == (Semaphore *) 0 ) {
		ARBInitLock.acquire();
		TheARBMutexSemaphore = new MutexSemaphore;
		TheARBMutexSemaphore->setName( "#TheARBMutexSemaphore" );
		ARBInitLock.release();
	}
	// Brashly put myself at the head of the list of ARBs:
	TheARBMutexSemaphore->P();

	_nextARB = _ARBList;
	if( _ARBList == 0 ) {	// I must be the first ARB to be created, so
				//  it's my job to launch the daemon.
	process = new PreemptableSystemProcess(
		(PFV) AutoRetransmitBufferDaemon, thisProcess()->domain(),			small, 0 );
	Assert( process != 0 );
	process->setName( "#ARB retransmit daemon" );

	process->ready();
	}
	_ARBList = this;
	TheARBMutexSemaphore->V();
}
AutoRetransmitBuffer::AutoRetransmitBuffer( int sz, int32 dlay,
		Conduit * initialDestination ) : RetransmitBuffer( sz, initialDestination )
{
	Process * process;
	int32 tmp;
	int i;

	//Debug << "AutoRetransmitBuffer::AutoRetransmitBuffer(size=" << sz << ", delay=" << dlay << ", initial destination=" << initialDestination << ") called... this=" << this << "\n" << eor;

	_delay = dlay;
	_lastsent = new int32[sz];
	tmp = now32().val();
	for( i = 0; i < sz; i++ ) {
		_lastsent[i] = tmp;
	}
	if ( TheARBMutexSemaphore == (Semaphore *) 0 ) {
		ARBInitLock.acquire();
		TheARBMutexSemaphore = new MutexSemaphore;
		TheARBMutexSemaphore->setName( "#TheARBMutexSemaphore" );
		ARBInitLock.release();
	}
	// Brashly put myself at the head of the list of ARBs:
	TheARBMutexSemaphore->P();

	_nextARB = _ARBList;
	if( _ARBList == 0 ) {	// I must be the first ARB to be created, so
				//  it's my job to launch the daemon.
	process = new PreemptableSystemProcess(
		(PFV) AutoRetransmitBufferDaemon, thisProcess()->domain(),
		small, 0 );
	Assert( process != 0 );
	process->setName( "#ARB retransmit daemon" );

	process->ready();
	}
	_ARBList = this;
	TheARBMutexSemaphore->V();
}
AutoRetransmitBuffer::~AutoRetransmitBuffer()
{
	delete _lastsent;

	// Ought to kill daemon(s), too, I suppose.... -JZ
	// Also Ought to remove myself from the ARBList!!!
}

void
AutoRetransmitBuffer::AutoRetransmitBufferDaemon()
{
// I think running through all the messages in all the ARB's every second
// ought to be the Right THing(tm).  I don't think having a separate daemon
// for each ARB is a good idea. -JZ
	AutoRetransmitBuffer * arb;
	TimeoutTimer * timer = new TimeoutTimer; // delete when we die

	Debug << "AutoRetransmitBufferDaemon: Hello!" << eor;
	// Might as well wait until there are some ARBs to talk to:
	while( _ARBList == 0 )
	{
	    timer->start( 1000000 );	// 1 second
	    timer->await();
	}

	Debug << "AutoRetransmitBufferDaemon: Hello!!" << eor;
	for(;;) { // Forever
	    timer->start( 1000000 );	// 1 second
	    timer->await();

	    TheARBMutexSemaphore->P();
	    for( arb = _ARBList; arb != 0; arb = arb->_nextARB ) {
		arb->check();
	    }
	    TheARBMutexSemaphore->V();
	}
}
void
AutoRetransmitBuffer::check()
{
	// Comparing "times" is tricky, since the counter wraps around. There
	//   are two cases: if now32() > delay, then (now32()-delay) will be
	//   positive, and we need to resend anything with a timestamp
	//   smaller than (now32()-delay) _OR_ greater than now32(). Otherwise,
	//   we send anything whose value (recall these are _unsigned_ numbers;
	//   it matters!) is smaller than (now32()-delay) _AND_ greater than
	//   now32(). A picture helps:
	//	0					2**32
	//	+--------+-----------+--------------------+
	//	         |<--delay-->|   <- don't re-send yet -- it's too soon!
	//                           ^
	//	                    now
	//
	//
	//	0					2**32
	//	+-+------------------------------+--------+
	//      ->|                              |<--delay-
	//        ^
	//	 now

	Word32 Now, Then;
	int i;

	if( _count == 0 	// No point checking what to re-send out of
	    ||			//   an empty buffer.
	    _dest == 0 ) return;// No point checking what to send out of
				//   a buffer with nowhere to send to....

	//Debug << "ARB::check() called....\n" << eor;
	Now = now32();
	Then = Now.val() - _delay;

	grab(); // Grab ARB

	if( _delay < Now.val() ) {
	   for( i = 0; i < _size; i++ ) {
	      if( msgs[i] != 0 &&
			( _lastsent[i] > Now.val() ||
			  _lastsent[i] < Then.val() ) ) {
		 Debug << "Retransmitting msg=" << msgs[i] << 
		   " addr=" << addrs[i] << " handle=" << i << " in IP\n" <<eor;
		 Assert( 0 != addrs[i] );
	         _dest->insertFromAbove( COPY(msgs[i]), addrs[i] );
	         _lastsent[i] = Now.val();	// Update timestamp
	      }
	   }
	}
	else {
	   for( i = 0; i < _size; i++ ) {
	      if( msgs[i] != 0 &&
			( _lastsent[i] > Now.val() &&
			  _lastsent[i] < Then.val()  ) ) {
		 Debug << "Retransmitting mesg=" << msgs[i] << 
		   " addr=" << addrs[i] << " handle=" << i << " in IP\n" <<eor;
		 Assert( 0 != addrs[i] );
	         _dest->insertFromAbove( COPY(msgs[i]), addrs[i] );
	         _lastsent[i] = Now.val();	// Update timestamp
	      }
	   }
	}

	release(); // Release ARB

}
MessageID
AutoRetransmitBuffer::insert( Return_Code & ret, ConduitMessage * newmsg,
			NetworkAddress * newaddr )
{//JZ
	MessageID tmp;

	//Debug << "\tARB::insert() called.\n" << eor;

	Assert( 0 != newmsg && 0 != newaddr /* MessageBuffer.cc l.540 */ );

put("A")
	tmp = this->ConduitMessageBuffer::insert( ret, newmsg, newaddr );
put("B")
	if( ret != OK ) return( tmp );

	//grab();
put("C")
	_lastsent[ tmp ] = now32().val();
	//release();
put("D")

	//Debug << "ARB::insert: about to put into IP conduit....\n" << eor;
	if( _dest == 0 ) {
put("E")
	    ret = NOT_TRANSMITTED;
	    Debug << "\tARB::insert: unable to transmit.\n" << eor;
	    return( tmp );
	}

	// Makes sense to send it once immediately upon being buffered:
put("F")
	ret = _dest->insertFromAbove( COPY(newmsg), newaddr );
put("G")

	return( tmp );
}


void
AutoRetransmitBuffer::kick()
{
   if( _dest == 0 ) {
	return; // Boo hoo!
   }
   grab();
   // Retransmit everything right away:
   Word32 Now = now32();
   for( int i = 0; i < _size; i++ ) {
      if( msgs[i] != 0 ) {
	 Debug << "Retransmitting msg=" << msgs[i] << 
	   " addr=" << addrs[i] << " handle=" << i << " in IP\n" <<eor;
	 Assert( 0 != addrs[i] );
         _dest->insertFromAbove( COPY(msgs[i]), addrs[i] );
         _lastsent[i] = Now.val();	// Update timestamp
      }
   }
   release();
}




#include "ClassInstaller.h"
DEFINECLASS("ConduitMessageBuffer",ConduitMessageBuffer,ProxiableObject,0,0);
DEFINECLASS("AutoRetransmitBuffer",AutoRetransmitBuffer,RetransmitBuffer,0,0);
DEFINECLASS("TCPWaitBuffer",TCPWaitBuffer,ConduitMessageBuffer,0,0);
DEFINECLASS("RetransmitBuffer",RetransmitBuffer,ConduitMessageBuffer,0,0);
