#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Johnny Zweig (zweig@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 *
 *
 * ConduitTypes.cc
 *
 */

#include "Assert.h"
#include "Kernel.h"
#include "MachineConfiguration.h"
#include "OutputStream.h"
#include "ConduitTypes.h"
#include "ZootAddress.h"
#include "ZootReturnCodes.h"
#include "EthernetAdministrator.h"
#include "InputStream.h"
#include "CharacterInputStream.h"
#include "NetworkBuffer.h"
#include "NetworkWalker.h"
#include "Process.h"
#include "ThisProcess.h"
#include "TimeoutTimer.h"


#ifdef __GNUC__
#pragma implementation
#endif


//
//    THIS IS COMPILER-DEPENDENT MORE THAN MACHINE-DEPENDENT
//
/* octetcopy is basically strncpy(), but guaranteed to manipulate octets */
void
octetcopy( octet * dest, const octet * src, int len ) 
	// [Recall that const octet * is a pointer to a constant octet, not
	//  a constant pointer to an octet....]
{
	const octet * p1;
	octet * p2;
	p1 = src;
	p2 = dest;
	for( int i = 0; i < len; i++ ) {
		p2[i] = p1[i];
	}
}

//  OctetCopy does 4 byte copies if the alignment allows.
void
OctetCopy( const octet * source, octet * destination, int length ) 
{
    if ((((unsigned int)source & 3) != 0) ||
        (((unsigned int)destination & 3) != 0)) {
        octetcopy( destination, source, length );
        return;
    }

    int32 *longSource = (int32 *) source;
    int32 *longDestination = (int32 *) destination;

	// Use longword copies for as much of the block as we can ...
    while (length >= sizeof(*longDestination)) {
	*longDestination = *longSource;
	++longSource;
	++longDestination;
	length -= sizeof(*longDestination);
    }

	// And finish off the remaining bytes, one by one ...
    if (length > 0) {
	source = (unsigned char *) longSource;
	destination = (unsigned char *) longDestination;

	while (length > 0) {
	    *destination = *source;
	    ++source;
	    ++destination;
	    --length;
	}
    }
}
//-----------------------------------------------------------------------------
//
//	MULTIMAXEN-SPECIFIC (well, not really...):
#include "defZOOT.h"
#ifdef ZOOT
extern EthernetAdministrator * _THE_EthernetAdministrator;  //gross


EthernetAddress		TheLocalEthernetAddress;
IPAddress		TheLocalIPAddress;
				// Stupid: there will be more than one if
				// there are >1 interface cards....
IPAddress		TheLocalGatewayIPAddress;
				// Ditto!
Word32			TheLocalIPSubnetMask;
EthernetAddress		TheLocalGatewayEthernetAddress;
				// Ditto again!
char *			TheLocalHostName = 0;


int		haveWantedAddress;
EthernetAddress	wantedEthernet;
IPAddress	wantedIP;


class ARPWalker : public NetworkWalker {
public:
  virtual void walk(NetworkBuffer *);
};


void
ARPWalker::walk(NetworkBuffer *buff)
{
  IPAddress	senderIP;
  octet		*data;

  data = buff->packet();

  if (data[20] == 0x00 && data[21] == 0x02) {
    senderIP.copyFrom(&data[28]);
    if (senderIP == wantedIP) {
      wantedEthernet.copyFrom(&data[22]);
      haveWantedAddress = TRUE;
    }
  }

  buff->release();
}


int doesIPExist(IPAddress ipAddr, EthernetAddress *etherAddr)
{
  ARPWalker		getReply;
  int			i;
  octet *               packet;
  TimeoutTimer		timer;

  packet = new octet[80];
  Assert( packet != 0 );
  NetworkBuffer	buff(packet, 80);

  haveWantedAddress = FALSE;
  wantedIP = ipAddr;
  _THE_EthernetAdministrator->registerType(0x0806, &getReply);

  for (i = 0; i < 6; i++)
    packet[i] = 0xFF;

  TheLocalEthernetAddress.copyTo(&packet[6]);
  packet[12] = 0x08;		/* Arp type = 0x0806 */
  packet[13] = 0x06;
  packet[14] = 0x00;		/* Ethernet medium = 0x0001 */
  packet[15] = 0x01;
  packet[16] = 0x08;		/* Protocol = 0x0800 */
  packet[17] = 0x00;
  packet[18] = 0x06;		/* Hardware addr length = 6 */
  packet[19] = 0x04;		/* Protocol addr length = 4 */
  packet[20] = 0x00;		/* Arp request op = 0x0001 */
  packet[21] = 0x01;
  TheLocalEthernetAddress.copyTo(&packet[22]);
  ipAddr.copyTo(&packet[38]);

  _THE_EthernetAdministrator->transmit(&buff);

  for (i = 0; i < 10 && !haveWantedAddress; i++) {
    timer.start(100000);  /* Wait for .1 seconds */
    timer.await();
  }

  _THE_EthernetAdministrator->unregisterType(0x0806);
  *etherAddr = wantedEthernet;

  delete packet;

  return(haveWantedAddress);
}

#endif ZOOT

IPAddress
ParseIPAddress( char * str )
{
	// NOTICE: This code assumes that the IP addres in str is formatted
	//   more or less like "128.174.238.2"

	int n0, n1, n2, n3;

	// Look for digits up to the first dot:
	n0 = 0;
	n1 = 0;
	n2 = 0;
	n3 = 0;
	while (*str && *str == ' ')
	  str++;

	while( 0 != *str && '.' != *str ) {
	    if( '0' <= *str && '9' >= *str ) {
		n0 = (n0 * 10) + (*str - '0');
	    }
	    str++;
	}
	if( '.' == *str ) {
	    str++;
	    // Look for stuff up to the next dot:
	    while( 0 != *str && '.' != *str ) {
		if( '0' <= *str && '9' >= *str ) {
		    n1 = (n1 * 10) + (*str - '0');
		}
		str++;
	    }
	}
	if( '.' == *str ) {
	    str++;
	    // Look for stuff up to the next dot:
	    while( 0 != *str && '.' != *str ) {
		if( '0' <= *str && '9' >= *str ) {
		    n2 = (n2 * 10) + (*str - '0');
		}
		str++;
	    }
	}
	if( '.' == *str ) {
	    str++;
	    // Look for stuff up to the next dot:
	    while( 0 != *str && '.' != *str && *str != ' ' && *str != '\n') {
		if( '0' <= *str && '9' >= *str ) {
		    n3 = (n3 * 10) + (*str - '0');
		}
		str++;
	    }
	}
	return( IPAddress( n0, n1, n2, n3 ) );
}


void
InitializeLocalNetworkAddresses()
{
	int ret;
	int hex_table[] = {	0, 1, 2, 3, 4, 5, 6, 7, 8, 9,   // '0' - '9'
				0, 0, 0, 0, 0, 0, 0,	// skip ':' - '@'
				10, 11, 12, 13, 14, 15 };	// 'A' - 'F'

	// Get Local Ethernet Address from the EthernetAdministrator:
	Assert( _THE_EthernetAdministrator != 0 );
	extern EthernetAdministrator * _THE_EthernetAdministrator;

//Console() << "InitializeLocalNetworkAddresses(): Local Ethernet Address " <<
//*(_THE_EthernetAdministrator->address()) << ".\n" << eor;

	_THE_EthernetAdministrator->address()->copyTo(
		TheLocalEthernetAddress.bytes() );

	// Get the Machine Configuration database:
	MachineConfiguration * config = TheKernel->getConfiguration();

	/*
	 * Take IP, router IP, and router enet from
	 * Configuration object
	 */

	ret = config->lookup( "hostname", TheLocalHostName );

	Console() << "Local hostname: " << TheLocalHostName; 

	// Determine the local gateway's IP address:
	//   If it supported router-discovery protocol, a simple ICMP
	//   message would do. This should be in a file. Hard-coding is evil.

	char * string;
	ret = config->lookup( "gatewayIPaddress", string );
	if( OK != ret ) {
	  string = "128.174.238.21";
	  Console() << "Yikes! Cannot find gateway IP address from" <<
			config->identification() <<
			": Defaulting to " << string << ".\n" << eor;
	}
	TheLocalGatewayIPAddress = ParseIPAddress( string );

	// Determine the local gateway's Ethernet address:
	//   An ARP packet would be the right thing, but I'll hard code
	//   it for now since I am lazy and stupid:

	if (!doesIPExist(TheLocalGatewayIPAddress, 
			&TheLocalGatewayEthernetAddress)) {
	  ret = config->lookup( "gatewayEthernetAddress", string );
	  if (OK != ret) {
	    string = "08:00:20:10:DF:F3";
	    Console() << "Yikes! Cannot find gateway ethernet address" <<
			  " in /Configuration/Local!\n" <<
			  "Defaulting to " << string << ".\n" << eor;
	  }
	
	    // "Parse" the ethernet-address string which ABSOLUTELY MUST BE
	    //   IN THE FORM "00:A2:F0:0C:00:07":

	    int n0 = 16*(hex_table[ string[0] - '0' ]) +
			hex_table[ string[1] - '0' ];
	    int n1 = 16*(hex_table[ string[3] - '0' ]) +
			hex_table[ string[4] - '0' ];
	    int n2 = 16*(hex_table[ string[6] - '0' ]) +
			hex_table[ string[7] - '0' ];
	    int n3 = 16*(hex_table[ string[9] - '0' ]) +
			hex_table[ string[10] - '0' ];
	    int n4 = 16*(hex_table[ string[12] - '0' ]) +
			hex_table[ string[13] - '0' ];
	    int n5 = 16*(hex_table[ string[15] - '0' ]) +
			hex_table[ string[16] - '0' ];

	    TheLocalGatewayEthernetAddress = 
		EthernetAddress( n0, n1, n2, n3, n4, n5);
	}

	// Determine the local IP address:
	ret = config->lookup( "hostIPaddress", string );
	if (OK != ret) {
	  string = "128.174.238.100";
	  Console() << "Yikes! Cannot find local host IP address" <<
		       " in /Configuration/Local!\n" <<
		       "Defaulting to " << string << ".\n" << eor;
	}
	TheLocalIPAddress = ParseIPAddress(string);


	ret = config->lookup( "IPsubnetMask", string );

	if (OK != ret) {
	  string = "000000FF";
	  Console() << "Yikes! Cannot find IP subnet mask" << 
		       " in /Configuration/Local!\n" <<
		       "Defaulting to " << string << ".\n" << eor;
	}

    int32 mask = 0;
    for( int i = 0; i < 8; i++ )
	mask = (16*mask) + (hex_table[ string[i] - '0' ]);
	    TheLocalIPSubnetMask = Word32( mask );
#ifdef NEVER
#ifdef VCHOICES
		char	localAddr[40];
		Console() << "Please enter an IP address: " << eor;
		ConsoleInputStream->read(localAddr, sizeof(localAddr));
		localAddr[sizeof(localAddr) - 1] = '\0';
		TheLocalIPAddress = ParseIPAddress(localAddr);
#endif
#endif

	for ( ; ; ) {
	  EthernetAddress	hostUsingIP;
	  char			ipAddr[40];

	  if (!doesIPExist(TheLocalIPAddress, &hostUsingIP))
	    break;

	  Console() << "IP address " << TheLocalIPAddress << " is currently" <<
	               " in use by " << hostUsingIP << ".\nPlease" <<
		       " enter an IP address: " << eor;

	  ConsoleInputStream->read(ipAddr, 40);
	  ipAddr[39] = '\0';
	  TheLocalIPAddress = ParseIPAddress(ipAddr);
	}

}
