#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-92
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Johnny Zweig (zweig@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * ConduitMessage.cc
 *
 */

#include "OutputStream.h"
#include "Assert.h"
#include "ThisProcess.h"
#include "ConduitTypes.h" 
#include "Word.h"
#include "IPChecksum.h"
#include "ZootAddress.h"
#include "IPMessage.h"
#include "ConduitProtocolIdentifiers.h"


#ifdef __GNUC__
#pragma implementation
#endif


#ifndef QUIET
#undef Debug
#define Debug Console()
#endif

//-------------------------------------------------------------------------
//--------------------Class IPMessage member functions --------------------
IPMessage::IPMessage()
{
	_proto = IP;
	_header = new octet[20];
	_headerLength = 20;
	octetcopy(_header, DefaultIPHeader, 20);
	_status |= 0x01;
}
IPMessage::IPMessage(int hsz, int dsz) : ConduitMessage(hsz, dsz) 
{
	_proto = IP;
	if( hsz >= 20 ) {
	    octetcopy(_header, DefaultIPHeader, 20);
	}
}
IPMessage::IPMessage(int hsz, int dsz, ConduitMessage * pld) : ConduitMessage(hsz, dsz, pld) 
{
	_proto = IP;
	if( hsz >= 20 ) {
	    octetcopy(_header, DefaultIPHeader, 20);
	}
}

#define printDebug Debug

void
IPMessage::print()
{
	printDebug << "IPMessage (header:" << this << "\n(";
	printDebug << asHex( _header[0] )<< "." ;
		printDebug << asHex( _header[1] )<< "." ;
		printDebug << asHex( _header[2] )<< "." ;
		printDebug << asHex( _header[3] )<< ".\t" ;
		printDebug << _header[0] << "." ;
		printDebug << _header[1] << "." ;
		printDebug << _header[2] << "." ;
		printDebug << _header[3] << ")\n" ;
		printDebug << asHex( _header[4] )<< "." ;
		printDebug << asHex( _header[5] )<< "." ;
		printDebug << asHex( _header[6] )<< "." ;
		printDebug << asHex( _header[7] )<< "." ;
		printDebug << asHex( _header[8] )<< "." ;
		printDebug << asHex( _header[9] )<< "." ;
		printDebug << asHex( _header[10] )<< "." ;
		printDebug << asHex( _header[11] )<< "." ;
		printDebug << asHex( _header[12] )<< "." ;
		printDebug << asHex( _header[13] )<< "." ;
		printDebug << asHex( _header[14] )<< "." ;
		printDebug << asHex( _header[15] )<< "." ;
		printDebug << asHex( _header[16] )<< "." ;
		printDebug << asHex( _header[17] )<< "." ;
		printDebug << asHex( _header[18] )<< "." ;
		printDebug << asHex( _header[19] )<< ").\n" << eor;
/****************
	// Print out hex-dump of rest of packet (_header[2-3] is length):
	for( int i = 0; i < 256*_header[2] + _header[3] - 4*(_header[0] & 0x0f);
							i++ ) {
		printDebug << asHex( _data[i] ) << ".";
	}
	printDebug << "\n" << eor;
****************/
}

// ---------- THE DEFAULT IP HEADER: ----------
const octet
DefaultIPHeader[20] =
{
	0x45,	// VER = 4, IHL = 5 32-bit words
	0x00,	// Normal TOS (all zeroes)
	0,20,	// Total Length = 20 octets with no data
	0,0,	// No Identification
	0,0,	// May fragment, last fragment, fragment offset = 0
	30,	// Time To Live = 30 seconds/hops
	0,	// No default protocol
	0,0,	// Not really the right header checksum!!!
	0,0,0,0,	// No default source IP address
	0,0,0,0		// No default destination IP address
	// No options
};
// Random Thought: how about having the controller fill in some details?

ConduitMessage *
IPMessage::clone()
{
	ConduitMessage * msg;

	msg = new IPMessage( _headerLength, _dataLength );
	PhotoCopy( this, msg );
	return( msg );
}

//--------------------Class TCPMessage member functions --------------------
TCPMessage::TCPMessage()
{
	_proto = TCP;
	myIPmsg = 0;
}
TCPMessage::TCPMessage(int hsz, int dsz) : ConduitMessage(hsz, dsz)
{
	_proto = TCP;
	myIPmsg = 0;
	setOffset( (nibble) hsz / 4 );	// offset/header-length is measured
					//   in 32-bit words!
}
TCPMessage::TCPMessage(int hsz, int dsz, ConduitMessage *pld) : ConduitMessage(hsz, dsz, pld)
{
	_proto = TCP;
	myIPmsg = 0;
	setOffset( (nibble) hsz / 4 );
}
TCPMessage::TCPMessage(int iphsz, int ipdsz, int hsz, int dsz, ConduitMessage * pld)
	: ConduitMessage(hsz, dsz, pld)
{
	_proto = TCP;
	myIPmsg = new IPMessage( iphsz, ipdsz, this );
	myIPmsg->setPROTO( (octet) 6 );
	setOffset( (nibble) hsz / 4 );
}
void
TCPMessage::print()
{
	int i;

	myIPmsg->print();

	printDebug << "\n---------- TCP header: ---------------------------\n";
	printDebug << "Source Port:\t" <<
		_header[0] << "." << _header[1] << "\n";
	printDebug << "Destination Port:\t" <<
		_header[2] << "." << _header[3] << "\n";
	printDebug << "Sequence Number:\t" <<
		_header[4] << "." << _header[5] <<
		_header[6] << "." << _header[7] << "\n";
	printDebug << "Acknowledgement Number:\t" <<
		_header[8] << "." << _header[9] <<
		_header[10] << "." << _header[11] << "\n";
	printDebug << "Data offset (header-length):\t"<< (_header[12]>>4) << "\n";
	printDebug << "   (measured in 32-bit words)\n";
	printDebug << "CODE bits:\n";
	printDebug << "\tURG:\t" << ((_header[13]&0x20)>>5) << "\n";
	printDebug << "\tACK:\t" << ((_header[13]&0x10)>>4) << "\n";
	printDebug << "\tPSH:\t" << ((_header[13]&0x08)>>3) << "\n";
	printDebug << "\tRST:\t" << ((_header[13]&0x04)>>2) << "\n";
	printDebug << "\tSYN:\t" << ((_header[13]&0x02)>>1) << "\n";
	printDebug << "\tFIN:\t" << (_header[13]&0x01) << "\n";
	printDebug << "Window:\t" <<
		asHex( _header[14] ) << "." << asHex( _header[15] ) << "\n";
	printDebug << "CHECKSUM:\t" <<
		asHex( _header[16] ) << "." << asHex( _header[17] ) << "\n";
	printDebug << "Urgent Pointer:\t" <<
		asHex( _header[18] ) << "." << asHex( _header[19] ) << "\n";
	printDebug << "Options:\t";
	for(i=0; i<(_header[12]>>4)*4 - 20; i++) {
		printDebug << asHex( _header[20+i] ) << " ";
	}
	printDebug << "\n";
	printDebug << "TCP Data Length: "  << _dataLength;
	printDebug << "\n-----------------------------------------------\n"<< eor;
}
ConduitMessage *
TCPMessage::clone()
{
	ConduitMessage * msg;

	msg = new TCPMessage( _headerLength, _dataLength );
	PhotoCopy( this, msg );
	return( msg );
}
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
TCPPseudoHeader::TCPPseudoHeader()
{
	int i;
	for( i = 0; i < 12; i++ ) { pseudoheader[i] = 0; }
	pseudoheader[9] = 0x06;		// TCP's protocol number

	short unsigned int tmp1 = 0;

	chksum = tmp1;
}

TCPPseudoHeader::TCPPseudoHeader( IPAddress & src,
				 IPAddress & dest, Word16 & len )
{
	pseudoheader[9] = 0x06;		// TCP's protocol number
	src.copyTo(  &pseudoheader[0]  );
	dest.copyTo( &pseudoheader[4]  );
	len.copyTo(  &pseudoheader[10] );

	short unsigned int tmp1 = 0;

	chksum = tmp1;
}

TCPPseudoHeader::TCPPseudoHeader( IPAddress & src, IPAddress & dest, int16 len )
{
	Word16 tmp;
	tmp.equals( len );
	pseudoheader[9] = 0x06;		// TCP's protocol number
	src.copyTo(  &pseudoheader[0]  );
	dest.copyTo( &pseudoheader[4]  );
	tmp.copyTo(  &pseudoheader[10] );

	short unsigned int tmp1 = 0;

	chksum = tmp1;
}

IPAddress
TCPPseudoHeader::sourceAddress()
{
	IPAddress tmp;
	tmp.equals( &pseudoheader[0] );
	return ( tmp );
}

IPAddress
TCPPseudoHeader::destinationAddress()
{
	IPAddress tmp;
	tmp.equals( &pseudoheader[4] );
	return ( tmp );
}

Word16
TCPPseudoHeader::length()
{
#ifdef FIXTHIS
	Word16 tmp;
#else
	static Word16 tmp;
#endif
	tmp.equals( &pseudoheader[10] );
	return ( tmp );
}

Word16
TCPPseudoHeader::checksum()
{
	chksum.equals( IPChecksum(&pseudoheader[0], 12) );
	return ( chksum );
}

void
TCPPseudoHeader::setSourceAddress(const IPAddress & newsrcaddr)
{
	newsrcaddr.copyTo( &pseudoheader[0] );

	short unsigned int tmp1 = 0;

	chksum = tmp1;
}

void
TCPPseudoHeader::setDestinationAddress(const IPAddress & newdestaddr)
{
	newdestaddr.copyTo( &pseudoheader[4] );

	short unsigned int tmp1 = 0;

	chksum = tmp1;
}

void
TCPPseudoHeader::setLength(const Word16 & newlen)
{
	newlen.copyTo( &pseudoheader[10] );

	short unsigned int tmp1 = 0;

	chksum = tmp1;

}

void
TCPPseudoHeader::setLength(int16 newlen)
{
	Word16 tmp;
	tmp.equals( newlen );	// to have byte-order dealt with right
	tmp.copyTo( &pseudoheader[10] );

	short unsigned int tmp1 = 0;

	chksum = tmp1;
}

void
TCPPseudoHeader::doChecksum( TCPMessage * msg )
{
	Word16 datacheck, headercheck, tmp1, tmp2;

	Assert( 0 != msg );

	short unsigned int tmp4 = 0;

	msg->setChecksum( tmp4);	// Just to be sure -- must be 0 for computation

	setLength( msg->length() );
	chksum.equals( IPChecksum( &pseudoheader[0], 12 ) );

	headercheck = IPChecksum( msg->header(), msg->headerLength() );
	if( msg->dataLength() > 0 ) {
	    datacheck = IPChecksum( msg->data(), msg->dataLength() );
	    tmp1 = combineIPChecksums( chksum, datacheck );
	    tmp2 = combineIPChecksums( headercheck.val(), tmp1.val() );
	    msg->setChecksum( tmp2 );
	}
	else {
	    tmp1 = combineIPChecksums( headercheck, chksum );
	    msg->setChecksum( tmp1 );
	}
}

//------------------------------- UDPMessage stuff ---------------
UDPMessage::UDPMessage(int iphsz, int ipdsz, int hsz, int dsz, ConduitMessage * pld)
	: ConduitMessage(hsz, dsz, pld)
{
	_proto = UDP;
	_holder = new IPMessage( iphsz, ipdsz, this );
	((IPMessage*)_holder)->setPROTO( (octet) 17 );
}

ConduitMessage *
UDPMessage::clone()
{
	ConduitMessage * msg;

	msg = new UDPMessage( _headerLength, _dataLength );
	PhotoCopy( this, msg );
	return( msg );
}

//--------------------Class ICMPMessage member functions --------------------
ICMPMessage::ICMPMessage()
{
	_proto = ICMP;
	icmpIPmsg = 0;
}
ICMPMessage::ICMPMessage(int hsz, int dsz) : ConduitMessage(hsz, dsz)
{
	_proto = ICMP;
	icmpIPmsg = 0;
}
ICMPMessage::ICMPMessage(int hsz, int dsz, ConduitMessage *pld) : ConduitMessage(hsz, dsz, pld)
{
	_proto = ICMP;
	icmpIPmsg = 0;
}
ICMPMessage::ICMPMessage(int iphsz, int ipdsz, int hsz, int dsz, ConduitMessage * pld)
	: ConduitMessage(hsz, dsz, pld)
{
	_proto = ICMP;
	icmpIPmsg = new IPMessage( iphsz, ipdsz, this );
	icmpIPmsg->setPROTO( (octet) ICMP );
}

//#endif
