#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Johnny Zweig (zweig@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * ZootController.cc -- That which giveth TCP, and that which taketh away
 *
 */
/*
	The function ZootController() is the biggie, here. Right now, it
	just calls ZootControllerTest() which creates some conduits and
	monkeys around with messages (i.e. it tries to talk to the
	sendmail daemon on cato -- snicker!) and stuff.

	The ZootReceiverDaemons that are launched are what grab the msgs
	IP deposits in the TCP incoming message FIFO and walks them through
	protocol processing. The Echoing Application Canduit has a FIFO
	and a process that snarfs messages from the FIFO and sends them
	back out (being careful to scrog the sequence numbers so they will
	be ignored by the other end that just telnetted in). Sigh!
	There is also a process associated with each AutoRetransmit Buffer
	(ARB) -- there is only one for now. That's what TCP uses to get
	its outgoing messages retransmitted.
 */

/* Be sure to include everything!!! Unincluded include-files hinder. */
#include "OutputStream.h"
#include "CPU.h"
#include "Process.h"
#include "ThisProcess.h"
#include "Semaphore.h"
//#include "TimeoutTimer.h"
#include "ConduitTypes.h"
#include "Word.h"
#include "ZootReturnCodes.h"
#include "Assert.h"
#include "ZootAddress.h"
#include "Ethernet.h"
#include "ConduitMessage.h"
#include "ZootClock.h"
#include "ConduitMessageBuffer.h"
#include "ZootStateMachine.h"
#include "Conduit.h"
#ifdef NOTUSED
#include "UDPConduit.h"
#include "CCCP.h"
#include "ElvisConduit.h"
#endif
#include "UserSessionManager.h"
#include "NameServer.h"
#include "AboriginalTelnet.h"
#include "FTP.h"


#ifdef __GNUC__
#pragma implementation
#endif

UserSessionManager *TheUserSessionManager;
volatile EthernetConduit * TheEthernetConduit;
volatile IPConduit * TheIPConduit;	// Global variables are evil! Should
					//   use a nameserver
volatile TCPConduit * TheTCPConduit;	// Globally-accessible
volatile Conduit * TheFakeARPConduit;	// Globally-accessible
volatile Boolean ZootNotReady = TRUE;
volatile int ZootControllerInitialized; // see MultimaxKernel.cc

// These are in ConduitTypes.cc:
extern IPAddress		TheLocalGatewayIPAddress;
extern Word32			TheLocalIPSubnetMask;
extern EthernetAddress	TheLocalGatewayEthernetAddress;

class FakeARPConduit : public Conduit {
protected:
public:
	FakeARPConduit();
	~FakeARPConduit();
	Return_Code	connectTo( Conduit * c );
	Return_Code	connectFromBelow( Conduit * c,
					ProtocolID proto,
					int   his_handy,
					int & my_handy );
	Return_Code   insertFromBelow( ConduitMessage * msg );
};


FakeARPConduit::FakeARPConduit()
{
}
FakeARPConduit::~FakeARPConduit()
{
}
Return_Code
FakeARPConduit::connectTo( Conduit * c )
{
	if( c != (Conduit*)TheEthernetConduit ) return( CANNOT_CONNECT );
	return( c->connectFromAbove( this, Undefined ) );
}
Return_Code
FakeARPConduit::connectFromBelow( Conduit * c, ProtocolID proto,
	int his_handy, int & my_handy )
{
	my_handy = 0x0806;	// so I get ARP ethernet packets
	return( OK );
}
Return_Code
FakeARPConduit::insertFromBelow( ConduitMessage * msg )
{
	IPAddress TargetIA;
	IPAddress SenderIA;
	EthernetAddress SenderHA;
	ConduitMessage * ARPReply;

	// If it's not an ARP request, do nothing (or update the ARP table):
	Word16 operation;
	operation.copyFrom( msg->data() + 6 );
	if( operation.val() == 0x0002 ) {
	    // Check if it's a response from the local gateway:
	    SenderIA.copyFrom( msg->data() + 14 );
	    if(SenderIA.val().val()==TheLocalGatewayIPAddress.val().val()) {
		SenderHA.copyFrom( msg->data() + 8 );
		//SenderHA.copyTo( TheLocalGatewayEthernetAddress.ptr() );
		TheLocalGatewayEthernetAddress = SenderHA;
//Console() << "FakeARP: setting TLGEA to " << SenderHA << "\n" << eor;
	    }
	    msg->del();
	    return( OK );
	}

	if( operation.val() != 0x0001 ) {
		// It's not a request, so forget it.
		msg->del();
		return( OK );
	}

	// Check who the target of the request is:
	TargetIA.copyFrom( msg->data() + 24 );

	if( TargetIA.val().val() == TheLocalIPAddress.val().val() ) {
	    // It's for me!!

	    // Set up ARPreply to tell anyone asking for my address who I am;
	    //   when we get an ARP request, we can just fill in the
	    //   destination address and target hardware address and spit it
	    //   at them:
	    ARPReply = new EthernetMessage( 0, 64, 0 );
	    Word16 atype;
	    Word32 anumber;

	    // Set up known-in-advance fields of ARPReply ethernet header:
	    TheLocalEthernetAddress.copyTo( ARPReply->header() + 6 );
		// This is probably unnecessary, since the ethernet driver will
		//   probably fill it in again....
	    short unsigned int tmp1 = 0x0806;
	    
	    atype.equals( tmp1 );
	    atype.copyTo( ARPReply->header() + 12 );

	    // Set up known-in-advance fields of ARPReply ARP message:
	    tmp1 = 0x00010800;

	    anumber.equals( tmp1 ); // Hardware=1,proto=IP
	    anumber.copyTo( ARPReply->data() );

	    tmp1 = 0x06040002;

	    anumber.equals( tmp1 ); // Hlen=6,Plen=4,Operation=2 (reply)
	    anumber.copyTo( ARPReply->data() + 4 );
	    TheLocalEthernetAddress.copyTo( ARPReply->data() + 8 );
	    TheLocalIPAddress.copyTo( ARPReply->data() + 14 );
	    // Now set up stuff specific to this reply:
	    SenderHA.copyFrom( msg->data() + 8 );
	    SenderIA.copyFrom( msg->data() + 14 );
	    SenderHA.copyTo( ARPReply->header() );
	    SenderHA.copyTo( ARPReply->data() + 18 );
	    SenderIA.copyTo( ARPReply->data() + 24 );

	    ((Conduit *)TheEthernetConduit)->insertFromAbove( ARPReply, 0 );
	}
	// Tell the ZootController to try sending the ARP reply (I see
	//   that there is a race, but I don't care -- the person sending
	//   the ARP request will try again soon, probably):
	msg->del();
	return( OK );
}

void
ZootControllerTest(ProcessContainer * initialScheduler) 
{
#ifdef NOTUSED
	Word16 srcport;
#endif
	int i;
//	int32 time;
	octet Foo[6];
#ifdef NOTUSED
	UDPPortMonster * udp;
	ApplicationConduit * appl, * appl2;
	EchoingApplicationConduit * echoappl;
#endif
	Return_Code ret;
#ifdef NOTUSED
	TCPAddress * tcpaddr;
	CCCPConduit * cccp;
	ElvisConduit * theKing;
#endif

	/**** Test code to find union-initializer bug: ************
        register union { int16 number; char bytes[2]; } temp = { 0xabcd };
	octet _octet = temp.bytes[0];
	temp.bytes[0] = temp.bytes[1];
	temp.bytes[1] = _octet;
	Console() << "0xabcd?" << asHex( temp.number ) << "\n" << eor;

	Word32 josephine;
	Console() << "josephine: " << asHex( josephine.val() ) << "\n" << eor;
	josephine = 7;
	Console() << "josephine: " << asHex( josephine.val() ) << "\n" << eor;
	josephine.copyFrom( (octet*) "0123" );
	Console() << "josephine: " << asHex( josephine.val() ) << "\n" << eor;
	Console() << "Foo: " << asHex( Foo[0] ) << asHex( Foo[1] )
	<< asHex( Foo[2] ) << asHex( Foo[3] ) << "\n" << eor;
	josephine.copyTo( Foo );
	Console() << "Foo: " << asHex( Foo[0] ) << asHex( Foo[1] )
	<< asHex( Foo[2] ) << asHex( Foo[3] ) << "\n" << eor;
	******************/

/*** crashes (page 0 is no fair to look at now)

	Word32 zilch;
	zilch.equals( (octet*) 0 );
	Debug << "The word at location 0x00000 is:" << zilch.val()
		<< "\nIt's vtable is: " << eor;
	for(i=0;i<20;i++) {
		Debug << asHex( *( *( (int32**)0  ) + i ) ) << " ";
	}
	Debug << "\n" << eor;
****/
	// NAMES AND STUFF (assumes TheLocalNetworkAddresses were alrerady
	//   initialized!):
//extern char *			TheLocalHostName;

	TheLocalIPAddress.copyTo(&Foo[0]);
	Console() << "\tLocal IP address: ";
	for(i=0; i<4; i++) {
		Console() << (int)Foo[i];
		if( i != 3 ) Console() << ".";
		}
	Console() << "\n" << eor;

//	Word32 local_ip_addr;
//	local_ip_addr.copyFrom( &Foo[0] );
//	Console() << "again:" << asHex( local_ip_addr.val() ) << "\n" << eor;


	TheLocalEthernetAddress.copyTo( &Foo[0] );
	Debug << "local Ethernet address:  ";
	for(i=0; i<6; i++)
		Debug<< asHex( Foo[i] ) << ":";
	Debug << "\n" << eor;

	// CONDUIT STUFF
	Debug << "About to make an Ethernet Conduit...\n" << eor;
	TheEthernetConduit = new EthernetConduit();
	Debug << "Made an Ethernet Conduit.\n"
		<< eor;
	Debug << "About to make a FakeARPConduit...\n" << eor;
	TheFakeARPConduit = new FakeARPConduit();
	Debug << "Made a FakeARPConduit.\nAbout to make an IP Conduit...\n"
		<< eor;
	TheIPConduit = new IPConduit();
	Debug << "Made an IP Conduit.\n" << eor;

	Debug << "About to connect the IP- to the Ethernet Conduit...\n" << eor;
	ret = ((IPConduit *)TheIPConduit)->connectTo( (Conduit*)TheEthernetConduit );
	Debug << "\tReturn Code: " << ret << ".\n" << eor;

	Debug << "About to connect the FakeARP- to the Ethernet Conduit...\n"
		<< eor;
	ret = ((Conduit *)TheFakeARPConduit)->connectTo( (Conduit*)TheEthernetConduit );
	Debug << "\tReturn Code: " << ret << ".\n" << eor;

	Debug << "About to make TCP Conduit...\n" << eor;
	TheTCPConduit = new TCPConduit();
	Debug << "Made a TCP Conduit.\n" << eor;

#ifdef NOTUSED
	Debug << "About to make UDP Monster...\n" << eor;
	udp = new UDPPortMonster();
	Debug << "Made a UDP Monster.\n"
		 "About to make other assorted Conduits..." << eor;

	cccp = new CCCPConduit();
	theKing = new ElvisConduit();
	appl = new ApplicationConduit();
	appl2 = new ApplicationConduit();
	echoappl = new EchoingApplicationConduit(initialScheduler);
		// passing in the scheduler is a hack!
#endif
	Assert( TheIPConduit != 0 );
	Assert( TheTCPConduit != 0 );
#ifdef NOTUSED
	Assert( udp != 0 );
	Assert( appl != 0 );
	Assert( appl2 != 0 );
	Assert( echoappl != 0 );

	Debug << "Made CCCP, Elvis, Echoing Conduits.\n" << eor;
#endif

        TheUserSessionManager = new UserSessionManager;
	Assert (TheUserSessionManager != 0);
	StandardNameServer -> bind ("userSessionManager",
            TheUserSessionManager);

// this ought to go....
	// Spawn ZootReceiverDaemons here (so initialization is complete)
	/*
	 * Now launch ZOOT ReceiverDaemons:
	 ************************************************
	extern void ZootReceiverDaemon(); // in Conduit.cc
	Debug  << "Building ZootReceiverDaemon stacks/processes\n" << eor;
	for( i = 0; i < 4; i++ ) {

	    Process * process;
	    process = new SystemProcess( 
		    (PFV) ZootReceiverDaemon, thisProcess()->domain(),
		    small, 0 );
	    Assert( process != 0 );
	    process->setName( "#Zoot Rec'v Daemon" );
	    process->ready();
	}
	Console() << "ZootReceiverDaemon processes launched....\n";
	*******************************************/
// up through this point. I think ZRDs are unnecessary when using Walkers.  -JZ

	Debug << "About to connect tcp to top of ip...\n" << eor;
	ret = ((TCPConduit *)TheTCPConduit)->connectTo( (Conduit*)TheIPConduit );
	Debug << "\tReturn Code: " << ret << ".\n" << eor;

#ifdef NOTUSED
	Debug << "About to connect appl to top of TCP....\n" << eor;
	ret = appl->connectTo( (TCPConduit*)TheTCPConduit );
	Debug << "\tReturn Code: " << ret << ".\n" << eor;

	Debug << "About to connect appl2 to top of TCP....\n" << eor;
	ret = appl2->connectTo( (TCPConduit*)TheTCPConduit );
	Debug << "\tReturn Code: " << ret << ".\n" << eor;

	Debug << "About to connect echoappl to top of TCP....\n" << eor;
	ret = echoappl->connectTo( (TCPConduit*)TheTCPConduit );
	Debug << "\tReturn Code: " << ret << ".\n" << eor;

	Debug << "About to connect CCCP to top of TCP....\n" << eor;
	ret = cccp->connectTo( (TCPConduit*)TheTCPConduit );
	Debug << "\tReturn Code: " << ret << ".\n" << eor;

	cccp = new CCCPConduit();
	Assert( 0 != cccp );

	Debug << "About to connect CCCP to top of TCP....\n" << eor;
	ret = cccp->connectTo( (TCPConduit*)TheTCPConduit );
	Debug << "\tReturn Code: " << ret << ".\n" << eor;

	cccp = new CCCPConduit();
	Assert( 0 != cccp );

	Debug << "About to connect CCCP to top of TCP....\n" << eor;
	ret = cccp->connectTo( (TCPConduit*)TheTCPConduit );
	Debug << "\tReturn Code: " << ret << ".\n" << eor;

	cccp = new CCCPConduit();
	Assert( 0 != cccp );

	Debug << "About to connect CCCP to top of TCP....\n" << eor;
	ret = cccp->connectTo( (TCPConduit*)TheTCPConduit );
	Debug << "\tReturn Code: " << ret << ".\n" << eor;

	cccp = new CCCPConduit();
	Assert( 0 != cccp );

	Debug << "About to connect CCCP to top of TCP....\n" << eor;
	ret = cccp->connectTo( (TCPConduit*)TheTCPConduit );
	Debug << "\tReturn Code: " << ret << ".\n" << eor;

	Debug << "About to connect ELVIS to top of TCP....\n" << eor;
	ret = theKing->connectTo( (TCPConduit*)TheTCPConduit );
	Debug << "\tReturn Code: " << ret << ".\n" << eor;
#endif

        AboriginalTelnet *telnet = new AboriginalTelnet;
        telnet -> setName ("Aboriginal Telnet Server");
        telnet -> passiveStart ();
        Console() << "\tChoices Aboriginal Telnet Server (CATS 0.1) running "
            << "on port 23\n" << eor;
	    
        FTP *ftp = new FTP;
        ftp -> setName ("FTP Server");
        ftp -> passiveStart ();
        Console() << "\tFTP Server (0.1) running on port 21\n" << eor;
	    
	Console() << "\n-----> ZOOT ALL CONNECTED....\n\n" << eor;
	ZootNotReady = FALSE;

#ifdef NOTUSED
	// This is just a dumb kludge I like to leave lying around to
	//   test whether a machine is alive by telnetting to port 666 or 7
	srcport = 666;
	tcpaddr = new TCPAddress;
	Assert( 0 != tcpaddr );
	ret = TheTCPConduit->open( appl2, srcport, 1234, *tcpaddr );
	appl2->setSession( *tcpaddr );
	Debug << "Just opened a connection on TCP port 666 (return code: " <<
	    ret << ", handle: " << tcpaddr->handle() <<
	    ")....\n------\n" << eor;

	srcport = 7;
	ret = TheTCPConduit->open( echoappl, srcport, 1234, *tcpaddr );
	echoappl->setSession( *tcpaddr );
	Debug << "Just opened a connection on TCP port 7   (return code: " <<
	    ret << ", handle: " << tcpaddr->handle() <<
	    ")....\n------\n" << eor;

	Conduit * port520;
	ret = udp->getPort( port520, 520 );
	Debug << "Just opened a connection on UDP port 520   (return code: " <<
	    ret << ", Port-conduit: " << 
		asAddress( port520 ) << ")....\n------\n" << eor;
#endif

	ZootControllerInitialized = 1; // signals MultimaxKernel.cc to go on

	Console() << "\tZoot is up and running.\n" << eor;

	SemaphoreStar sleepingBeauty = new Semaphore(0);
	sleepingBeauty->P();		// Never to awaken.
	// while (1) { thisProcess()->relinquishProcessor(); }

	Assert( NOTREACHED );

}


//------------------------ ZootController(): -------------------------------
void
ZootController(ProcessContainer * initialScheduler) 
{
#include "defZOOT.h"
#ifdef ZOOT
        Console() << "\tZootController starting.\n" << eor;

	Debug << "\tInitializing Local Network Addresses/Hostname...\n" << eor;

	extern void InitializeLocalNetworkAddresses();
		// see ConduitTypes.cc
	InitializeLocalNetworkAddresses();

	ZootControllerTest( initialScheduler );
		// this is now a misnomer, since it "tests" Zoot by actually
		// turning on IP, TCP, UDP, etc.
#endif
}
