#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801-2910
 *
 * Copyright (c) 1990,1991 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Amit Parghi (parghi@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *	EthernetAdministrator.cc
 *
 *	The Ethernet Administrator.
 *
 * $Id: EthernetAdministrator.cc 2 2004-06-03 01:58:28Z jcarlyle $
 *
 * $Log: EthernetAdministrator.cc,v $
 * Revision 1.1  2004/06/03 01:58:57  jcarlyle
 * Initial revision
 *
 * Revision 1.5  1991/12/01  06:02:13  parghi
 * Minor changes, brought up-to-date with Stable.9.23.1991.
 * Last revision to networking redesign/rewrite.
 *
 * Revision 1.4  91/09/25  16:08:49  parghi
 * minor changes, made better, faster, stronger.
 * Will still need minor changes before Stable-ifying.
 * 
 * Revision 1.3  91/05/06  16:11:39  parghi
 * misc changes (nothing major)
 * 
 * Revision 1.2  91/04/14  22:54:01  parghi
 * many updates ... working version prior to munging to allow custom buffers
 * 
 * Revision 1.1  91/03/07  22:11:59  parghi
 * Initial revision
 *
 */

//DD SYSIN *

	//  DEBUG is used to compile runtime diagnostics for debugging.
//#define DEBUG 1
#define RAID 1

	//  TEST is used to compile additional runtime diagnostics and
	//  special testing behaviour (esp. not using the Walkers).
// #define TEST

	//  TEST2 is used to compile the Administrator--->Administrator
	//  sequenced packet test.
//#define TEST2

	//  TRANSMIT_TEST is used to run the transmission testing process,
	//  which periodically tranmits a broadcast (!) packet ...
//#define TRANSMIT_TEST 1

	//  TARGET_POPILIUS is used in conjunction with TEST2 to select the
	//  host which will act as the destination for our test messages.
	//  If TARGET_POPILIUS is set, "popilius" will be the destination;
	//  if TARGET_COUSINM is set, "cousinm" will be the destination;
	//  if TARGET_AUNTIEM is set, "auntiem" will be the destination;
	//  if none of these is set, the destination host will be "publius".
//#define TARGET_COUSINM

#ifdef TEST2
# if defined(TARGET_POPILIUS)
    static const char * EthernetSpecialTestTarget = "popilius";
# elif defined(TARGET_COUSINM)
    static const char * EthernetSpecialTestTarget = "cousinm";
# elif defined(TARGET_AUNTIEM)
    static const char * EthernetSpecialTestTarget = "auntiem";
# else
    static const char * EthernetSpecialTestTarget = "publius";
# endif
#endif /* TEST2 */


#include "Process.h"
#include "ThisProcess.h"
#include "Kernel.h"
#include "MachineConfiguration.h"
#include "String.h"
#include "OutputStream.h"
#include "Raid.h"
#include "NetworkBuffer.h"
#include "NetworkTest.h"
#include "NetworkStatistics.h"
#include "NetworkDriver.h"
#include "Ethernet.h"
#include "EthernetDMUX.h"
#include "EthernetDriver.h"
#include "EthernetAdministrator.h"
#include "TimeoutTimer.h"


#ifdef __GNUC__
#pragma implementation
#endif


EthernetAdministrator* _THE_EthernetAdministrator;


    //	Now, the definitions for the EthernetAdministrator
    //
EthernetAdministrator::EthernetAdministrator(char *name,
					     unsigned char debugMask)
    : NetworkAdministrator(name, debugMask)
{
    Raids(RaidConstructor) << "EthernetAdministrator constructor (name=\""
			   << name << "\")\n" << eor;

	    //  Create a new DMUX table, initialized as empty
    _table = new EthernetDMUXTable();
    _THE_EthernetAdministrator = this;		// WARNING: GROSS HACK
						// should use NameServer
}


EthernetAdministrator::~EthernetAdministrator()
{}


void
EthernetAdministrator::ready(NetworkDriver *aDriver)
{
    Raids(RaidMethod) << "EthernetAdministrator::ready(driver="
		      << asAddress(aDriver) << ")\n" << eor;

    NetworkAdministrator::ready(aDriver);		// super ready: aDriver

    _address = *(((EthernetDriver *)_driver)->getEthernetAddress());

    Debug << "EthernetAdministrator started on hardware address "
	      << _address << '\n' << eor;

#ifdef TEST2
    Return_Code status;

	//  Get the local hostname from the MachineConfiguration database.
    MachineConfiguration *config = TheKernel->getConfiguration();
    char *myHostName;
    // config->setDebugMask(0x28);
    status = config->lookup("hostname", myHostName);

    Debug << "EthernetAdmin::: config->lookup() returns status = " << status
	  << ", hostname = " << asAddress(myHostName) << '\n' << eor;

	//  Check our host name against that of the intended destination
	//  for the test packets.  If we are the destination, set up the
	//  NetworkWalker for the test packets; if not, set up the daemon
	//  to transmit the test packets.
    if ((status == OK) && strcmp(myHostName, EthernetSpecialTestTarget)) {
			// check that local host name != target's host name
	Debug << "TEST2: starting EthernetAdministratorTestEntry on host \""
	      << myHostName << "\", sending to " << EthernetSpecialTestTarget
	      << '\n' << eor;

	    //	This is the Level 2 test of the EthernetAdministrator.
	    //	In this test, we pass packets back and forth between the
	    //	two EthernetAdministrators.
	extern void EthernetAdministratorTestEntry(int);	// prototype
	Process *testProcess = new PreemptableSystemProcess(
					(PFV) EthernetAdministratorTestEntry,
					thisProcess()->domain(),
					normal,
					(int) _driver);
	testProcess->setName("EthernetAdministrator Test Transmitter");
	testProcess->ready();
	thisProcess()->relinquishProcessor();
    }
#endif /* TEST2 */

	// Start up a NetworkTestWalker to field incoming test packets ...
    // UNUSED??? NetworkWalker *tmp = new NetworkTestWalker();

#   if defined(TRANSMIT_TEST)
	extern void EthernetTransmitDaemon(int);		// forward
	Process *transmitDaemon = new PreemptableSystemProcess(
			(PFV) &EthernetAdministrator::transmitDaemon,
			thisProcess()->domain(),
			small,
			(int) this);
	Raids(RaidDetail) << this << ": transmitDaemon is process "
			  << asAddress(transmitDaemon) << '\n' << eor;
	transmitDaemon->setName("EthernetAdministrator Transmit Daemon");
	transmitDaemon->ready();
	thisProcess()->relinquishProcessor();
	Raids(RaidDetail) << this << ": transmitDaemon ready() done.\n" <<eor;
#   endif /* TRANSMIT_TEST */

}


NetworkWalker *
EthernetAdministrator::findWalker(NetworkBuffer *buffer)
{
    Raids(RaidDetail) << "EthernetAdministrator::findWalker(buffer="
		      << asHex((unsigned int) buffer) << ")\n" << eor;

    EthernetProtocolID id = getProtocolID((unsigned char *) buffer->packet());

    return _table->findWalker(id);
}


int					// add an entry to our DMUX table
EthernetAdministrator::registerType(EthernetProtocolID t, NetworkWalker *w)
{
    Raids(RaidMethod) << "EthernetAdministrator::registerType("
		      << asHex((int) t) << "," << asAddress( w )
		      << ")\n" << eor ;

    return _table->add(t, w);
}

int 
EthernetAdministrator::unregisterType(EthernetProtocolID t)
{
    Raids(RaidMethod) << "EthernetAdministrator::unregisterType("
		      << asHex((int) t) << ")\n" << eor;

    _table->remove(t);

    return 0;
}




EthernetAddress *
EthernetAdministrator::address() // return Ethernet address
{
    return & _address;				// is this reasonable???
}


EthernetProtocolID 
EthernetAdministrator::getProtocolID(octet *buffer)
{
    EthernetProtocolID type;
    octet *typePointer;

	// Code to pull the ProtocolID out of the packet.
	//	Recall that the data in the inbound packet is laid out
	//	in the following format:
	//	start->	 6 bytes	destination Ethernet address
	//		 6 bytes	source Ethernet address
	//	data->	 2 bytes	Ethernet protocol type
	//		 data ...
	//	Note also that the ProtocolID is stored in MSB,LSB format.
	//	Note also that the above format is for DIX Ethernet, not for
	//	IEEE 802.3.  This code will also parse 802.3, but it doesn't
	//	have to *like* it ...

    typePointer = buffer + 2 * EthernetAddressLength;
    // Clever host byte-order independent way to grab the type field value:
    type = ((unsigned int) *typePointer) * 256 + *(typePointer+1);

	// Check for IEEE 802.3 format.  If this packet is in 802.3 format,
	// then we must hope and pray that it uses the SNAP convention.

    if (type < MaxEthernetPacketLength) {
	Raids(RaidDetail) << "EthernetDriver::getProtocolID:  802.3 packet\n"
			  << eor ;

	typePointer += 2;
	if (	*typePointer++ == 0xAA  &&	// need 802.3 DEST SAP == 0xAA
		*typePointer++ == 0xAA		// need 802.3  SRC SAP == 0xAA
		&&  typePointer++ 		// skip 802.3 TYPE field
		&&  *typePointer++ == 0 &&  *typePointer++ == 0
		&&  *typePointer++ == 0 ) {	// check 802.3 ORG field == 0

			// We can get Ethertype from its new position in
			// the SNAP encoding... yay
	    type = ((unsigned int) *typePointer) * 256 + *(typePointer+1);
	    Raids(RaidDetail) << "EthernetDriver: received 802.3 packet, type "
			      << asHex(type) << '\n' << eor;

	} else {
	    Raids(RaidDetail) << "EthernetDriver::getProtocolID\n"
		    << "\tIEEE 802.3 packet not in SNAP format...discarded\n"
		    << eor ;
	    _stats->droppedPacketCount++;

	    return 0;
	}
    } else {
	Raids(RaidDetail) << "EthernetDriver: received normal packet, type is "
			  << asHex(type) << '\n' << eor;
    }	// if (802.3)

    return type;
}



    //	The truly astute observer will notice that the following routine
    //	bears a strong similarity to Dave Pointer's "EthernetTestEntry"
    //	routine in Att6386EthernetDriver.cc.  This is not an accident.
    //
#ifdef TEST2
void
EthernetAdministratorTestEntry(int foo)
{
    EthernetDriver *drv = (EthernetDriver *) foo;

    const unsigned short testPacketDataLength = 64;
    NetworkBuffer *testPacket = new NetworkBuffer(testPacketDataLength, 0);
    unsigned char *pack = testPacket->packet();

	// Stuff destination Ethernet address into buffer.
#if defined(TARGET_POPILIUS)
    pack[0] = 0x02;
    pack[1] = 0x60;
    pack[2] = 0x8c;
    pack[3] = 0x0e;
    pack[4] = 0x66;	// address 02:60:8c:0e:66:fb = popilius
    pack[5] = 0xfb;
#elif defined(TARGET_COUSINM)
    pack[0] = 0x08;
    pack[1] = 0x00;
    pack[2] = 0x4c;
    pack[3] = 0x00;
    pack[4] = 0x23;	// address 08:00:4c:00:23:f0 = cousinm
    pack[5] = 0xf0;
#elif defined(TARGET_AUNTIEM)
    pack[0] = 0x08;
    pack[1] = 0x00;
    pack[2] = 0x4c;
    pack[3] = 0x00;
    pack[4] = 0x27;	// address 08:00:4c:00:27:ac = auntiem
    pack[5] = 0xac;
#elif defined(TARGET_PUBLIUS)
    pack[0] = 0x02;
    pack[1] = 0x60;
    pack[2] = 0x8c;
    pack[3] = 0x3c;
    pack[4] = 0x26;	// address 02:60:8c:3c:26:45 = publius
    pack[5] = 0x45;
#else			// broadcast!
    pack[0] = pack[1] = pack[2] =
    pack[3] = pack[4] = pack[5] = 0xff;
#endif /* TARGET_POPILIUS */

	// Also fill the Ethernet packet type field.
    pack[12] = (EthernetSpecialTestType >> 8);
    pack[13] = (EthernetSpecialTestType & 0xFF);

	//  Add sequence number to each packet so as to
	//  check what packets we get at the other end.		-- Amit
    unsigned short *sequenceNumber = (unsigned short *)
				     (testPacket->packet() + 14);

    for (int i = 16 ; i < testPacketDataLength ; i++) {
	pack[i] = 0x55;
    }

    unsigned short counter = 1;			// tracks our sequence number
    for(;;) {
	for (int burst = 0 ; burst < 2 ; burst++) {
	    Debug << "TestEntry:: transmitting packet w/sequence number "
		  << asHex(counter) << " ******\n" << eor;
	    *sequenceNumber = counter++;
	    drv->transmitPacket(testPacket);
	}
	// for (unsigned int j = 0 ; j < 800000 ; j += 1);		// delay
	TimeoutTimer *timer = new TimeoutTimer();
	timer->start(1000000);	// 1 sec.
	timer->await();
	delete timer;

    }

    // not really reached
    //delete testPacket;
}
#endif  /* TEST2 */


void
EthernetAdministrator::transmitDaemon()
{
    NetworkBuffer *aBuffer;
    octet *aCharacter;
    int i;

    Debug << "EthernetTransmitDaemon starting (admin="
	      << asAddress(this) << '\n' << eor;

    //  Now begin filling up the packet ...
    aBuffer = new NetworkBuffer(256);
    aCharacter = aBuffer->packet();

    for (i = 0 ; i < EthernetAddressLength ; i++) {	// set dest. address
	*aCharacter++ = i;
    }

    aCharacter += EthernetAddressLength;		// skip sender address

    *aCharacter++ = 0x99;				// ethertype
    *aCharacter++ = 0x99;

    for (i = 0 ; i < 100 ; i++) {			// fill packet with
	*aCharacter++ = 0x5a;				// test pattern ...
	*aCharacter++ = 0xa5;
    }

    while (1) {
	TimeoutTimer *timer = new TimeoutTimer();
	timer->start(1000000);	// 1 sec.
	timer->await();
	delete timer;
	transmit(aBuffer);
#ifdef NEVER
	for (i = 0 ; i < 1000000 ; i++) {		// delay ...
	    aCharacter++;				// This ought to use a
	    i++;					// TimeoutTimer instead
	    aCharacter--;
	    i--;
	}
#endif
    }
}


#include "ClassInstaller.h"
DEFINECLASS("EthernetAdministrator",EthernetAdministrator,NetworkAdministrator,0,0);
