#ifdef __GNUC__
#pragma implementation
#endif

#include "ConduitMessage.h"
#include "Conduit.h"
#include "NetworkBuffer.h"
		// defines class NetworkBuffer
#include "NetworkWalker.h"
		// defines class NetworkWalker
#include "EthernetAdministrator.h"
		// The EA is how I actuall send/arrange to receive packets


#ifdef __GNUC__
#pragma implementation
#endif

//extern Conduit *	TheEthernetConduit; // see ZootController.cc
extern EthernetAdministrator * _THE_EthernetAdministrator;  //gross

//---------------------------------------------------------------------------

#define ETHERNET_HEADER_LENGTH	14
	// That is, source-ethernet-address, destination-address, type.

EthernetMessage::EthernetMessage(int header_length, int data_length,
					ConduitMessage * link )
			: ConduitMessage( ETHERNET_HEADER_LENGTH, data_length, link )
{
	// Notice that we ignore header_length. Maybe it's not necessary
	//   to even have it as an arguement, except to make it look like
	//   most of the other ConduitMessage constructors.
	_buff = 0;
}

EthernetMessage::EthernetMessage( NetworkBuffer * buff ) : ConduitMessage( 0, 0 )
{
	// Notice we call the parent constructor with 0's, so we
	//   should be all set not to delete _header or _data, so there's
	//   no need to diddle the status bits with pleaseDontDeleteHeader,
	//   etc.  This constructor is really only for use by instances of
	//   ConduitEtherWalker.
	Assert( 0 != buff );
	_buff = buff;
	octet * p = buff->packet();
	_header = p;
	_data = p + ETHERNET_HEADER_LENGTH;
}

EthernetMessage::~EthernetMessage()
{
	if( 0 != _buff ) _buff->release(); // puts it back on the NB free list.
	// NOTA BENE:  this assumes that the only time an EthernetMessage
	//   has a nonzero NetworkBugger is when an ethernet packet arrived
	//   and one of the EthernetAdministrator's buffers (which know which
	//   free list to put themselves onto) was passed in to the
	//   constructor.  This code, for example would be WRONG:
	//
	//	NetworkBugger * buff = new NetworkBugger( 1024 );
	//	EthernetMessage * msg = new EthernetMessage( buff );
	//	msg->del();	// ERROR! Will attempt to do buff->release()

	// Let parent destructor worry about whether to delete _link, etc....
}
ConduitMessage *
EthernetMessage::clone()
{
	ConduitMessage * msg;

	msg = new EthernetMessage( _headerLength, _dataLength, 0 );
	extern void PhotoCopy( ConduitMessage*, ConduitMessage* );
		// see ConduitMessage.cc
	PhotoCopy( this, msg );
	return( msg );
}
//---------------------------------------------------------------------------
// Walker class that gives ethernet NetworkBuffers to the EthernetConduit
//   as EthernetMessages:

class ConduitEtherWalker : public NetworkWalker {
protected:
	Conduit *	_aConduit;
public:
	ConduitEtherWalker( Conduit * aConduit );
	~ConduitEtherWalker();
	void	walk( NetworkBuffer * buff );
};

ConduitEtherWalker::ConduitEtherWalker( Conduit * aConduit )
{
	_aConduit = aConduit;
}
ConduitEtherWalker::~ConduitEtherWalker()
{
}
void
ConduitEtherWalker::walk( NetworkBuffer * buff )
{
	// Let the EthernetMessage constructor do the translation/setup:
	EthernetMessage * msg = new EthernetMessage( buff );
	_aConduit->insertFromBelow( msg ); // ignore return code
}
//---------------------------------------------------------------------------
EthernetConduit::EthernetConduit()
{
	my_proto = Ethernet;
}

EthernetConduit::~EthernetConduit()
{
}
Return_Code
EthernetConduit::connectTo( Conduit * c )
{
	Assert( NOTREACHED );	// Nobody down there to connect to right now
	return( ERROR );
}
Return_Code
EthernetConduit::connectFromAbove( Conduit * c, ProtocolID proto )
{
	Return_Code ret;
	int his_handy;
	EthernetProtocolID epid;

	// First, do c->cFB to get c's handy number (ethernet type field)
	//   (ignoring his protocol):
	ret = c->connectFromBelow( this, my_proto, 0, his_handy );
	if( ret == OK ) {
	    // Now, create a ConduitWalker that will get packets for that type
	    //   field and deliver them, as EthernetMessages, to c:
	    NetworkWalker * walker = new ConduitEtherWalker( c );
	    epid = his_handy;
	    Assert( 0 != _THE_EthernetAdministrator );
	    _THE_EthernetAdministrator->registerType( epid, walker );
	}
	return( ret );
}
Return_Code
EthernetConduit::connectFromBelow( Conduit * c, ProtocolID proto, int his_handy,
				int & my_handy )
{
	Assert( NOTREACHED );	// Nobody there right now...
	return( ERROR );
}
Return_Code
EthernetConduit::disconnectFrom( Conduit * c )
{
	Assert( NOTREACHED );	// Asking to disconnect from a conduit Ethernet
				// can't conceivably be connected to...
	return( NO_SUCH_CONDUIT );
}
Return_Code
EthernetConduit::disconnectFromAbove( Conduit * c )
{
	// Ugh.  Need to tell the appropriate walker to fuck off and die:
	// So, I guess I need some goddamn linked-list of ConduitWalkers.
	return( ERROR );
}
Return_Code
EthernetConduit::insertFromAbove( ConduitMessage * msg, NetworkAddress * addr )
{
	// Ignore addr -- the message should already have the ethernet
	//   header all filled in (including the source ether-addr)!!!

	NetworkBuffer * buff;
	octet * p;
	ConduitMessage * finger;

	// Create a new NetworkBuffer to send the packet in:
	buff = new NetworkBuffer( ETHERNET_HEADER_LENGTH + msg->length() );

	// Now, run my finger down the list of linked ConduitMessages and
	//   copy them all into the NetworkBuffer:
	p = buff->packet();
	finger = msg;
	while( 0 != finger ) {
	    if( 0 != finger->header() ) {
		OctetCopy( finger->header(), p, finger->headerLength() );
		p += finger->headerLength();
	    }
	    if( 0 != finger->data() ) {
		OctetCopy( finger->data(), p, finger->dataLength() );
		p += finger->dataLength();
	    }
	    finger = finger->link();
	}

	// Send the buffer:
	_THE_EthernetAdministrator->transmit( buff );

	delete buff;
	msg->del();

	return( OK ); // since EA->transmit() is void
}

Return_Code
EthernetConduit::insertFromBelow( ConduitMessage * msg )
{
	Assert( NOTREACHED );	// Should never be called -- the walkers
				//   will skip the EC and deliver Ethernet-
				//   Messages directly to their conduits.
	return( ERROR );
}
void
EthernetConduit::Reset()
{
}
