#ifdef __GNUC__
#pragma implementation
#endif

/*
 * Revision 1.1  91/05/06  16:12:10  parghi
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801-2910
 *
 * Copyright (c) 1991 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Amit Parghi (parghi@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *	EthernetDMUX.cc
 *
 *	Classes to handle Ethernet-type decoding and table lookup.
 *
 *	$Id: EthernetDMUX.cc 2 2004-06-03 01:58:28Z jcarlyle $
 *
 * $Log: EthernetDMUX.cc,v $
 * Revision 1.1  2004/06/03 01:58:57  jcarlyle
 * Initial revision
 *
 * Revision 1.2  1991/09/25  16:09:21  parghi
 * minor changes, made better, faster, stronger.
 * Will still need minor changes before Stable-ifying.
 *
 * Revision 1.1  91/05/06  16:12:10  parghi
 * Initial revision
 *
 */

#define DEBUG 1
#define RAID 1

#include "Semaphore.h"
#include "OutputStream.h"
#include "Raid.h"
#include "Ethernet.h"
#include "EthernetDMUX.h"


#ifdef __GNUC__
#pragma implementation
#endif


EthernetDMUXData::EthernetDMUXData()
{
    _type = 0;
    _walker = NULL;
}

EthernetDMUXData::EthernetDMUXData(EthernetProtocolID t, NetworkWalker *w)
{
    _type = t;
    _walker = w;
}


EthernetDMUXTable::EthernetDMUXTable(unsigned char debugMask)
{
    setDebugMessages(MaxObjectDebugMessages);
    setDebugMask(debugMask);

    _semaphore = (Semaphore *) new MutexSemaphore();
    _array = new List[EthernetDMUXTableSize];

    Raids(RaidMethod) << "EthernetDMUXTable::ctor:  this=" << asHex((int) this)
		      << ", _array=" << asHex((int) _array) << '\n' << eor;
}

EthernetDMUXTable::~EthernetDMUXTable()
{
    //delete[EthernetDMUXTableSize] _array;
    delete [] _array;
}


				// given packet type, locate its DMUXData
EthernetDMUXData *
EthernetDMUXTable::getEntry(EthernetProtocolID t)
{
    List *list;
    EthernetDMUXData *current;

    Raids(RaidMethod) << "EthernetDMUXTable::getEntry(type=" << asHex(t)
		      << ", _array = " << asHex((unsigned int) _array)
		      << ", this = " << asHex((int) this) << ")\n" << eor;

    list = _array + hash(t);			// find hash chain

    Raids(RaidDetail) << "EthernetDMUXTable::getEntry: got list "
		      << asHex((unsigned int) list)
		      << " for ethertype " << asHex(t)
		      << " (_array=" << asHex((int) _array) << ")\n" << eor;

    current = (EthernetDMUXData *) list->first();

    if (current != NULL) {
	_semaphore->P();
	while ((current != NULL) && (current->_type != t)
	       && (current != (EthernetDMUXData *) list->last())) {
	    current = (EthernetDMUXData *) current->next();
	}
	_semaphore->V();

	if (current->_type == t) {		// found
	    return current;
	} else {				// not found
	    return NULL;
	}

    } else {					// list is empty
	Raids(RaidDetail) << "EthernetDMUXTable::getEntry: list is empty.\n"
			  << eor;

	return NULL;
    }
}

NetworkWalker *
EthernetDMUXTable::findWalker(EthernetProtocolID t)
{
    EthernetDMUXData *pointer = getEntry(t);

    if (pointer != NULL) {
	return pointer->_walker;
    } else {
	return NULL;
    }
}



int 
EthernetDMUXTable::add(EthernetProtocolID t, NetworkWalker *w)
{					// add {protocol,Walker} tuple to table
    EthernetDMUXData *current;		// pointer to EthernetDMUXData
    int returnCode;

    Raids(RaidMethod) << "EthernetDMUXTable::add(type=" << asHex(t)
		      << ", walker=" << asHex((int) w) << ")\n" << eor;

    if (t == 0) {			// ethertype of zero: No Good
	returnCode = 1;

    } else {
	current = getEntry(t);		// check for it in table

	if (current != NULL) {		// entry already exists
	    returnCode = 1;

	} else {
	    current = new EthernetDMUXData(t, w);
	    List *list = _array + hash(t);	// point to array entry

	    _semaphore->P();
	    list->addLast(current);		// and add it to the chain
	    _semaphore->V();

	    returnCode = 0;
	}
    }

    return returnCode;
}


void 
EthernetDMUXTable::remove(EthernetProtocolID t)	// currently unused??
{
    EthernetDMUXData *current;

    Raids(RaidMethod) << "EthernetDMUXTable::remove(type=" << asHex(t)
		      << ")\n" << eor;

    current = getEntry(t);			// find it, if it exists

    if (current != NULL) {			// found in table
	List *list = _array + hash(t);

	_semaphore->P();
	list->remove(current);
	_semaphore->V();

	delete current;
    }
}
