#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801-2910
 *
 * Copyright (c) 1991 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Amit Parghi (parghi@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *	StubEthernetDriver.cc
 *
 *	Fake EthernetDriver to be used for testing.  This EthernetDriver
 *	doesn't transmit anything, but it pretends to receive packets
 *	at regular intervals.  The packets are all of the type specified
 *	in the special value "EthernetSpecialTestType" and are made to
 *	appear as if sent from 00:01:02:03:04:05 to FF:FE:FD:FC:FB:FA.
 *	The shortword immediately following the ethernet type (i.e. the
 *	first byte of the packet proper) contains a sequence number.
 *
 */

//DD SYSIN *

#define DEBUG  1

#include "OutputStream.h"
#include "StackSize.h"
#include "Process.h"
#include "ThisProcess.h"
#include "NetworkBuffer.h"
#include "NetworkDriver.h"
#include "EthernetDriver.h"
#include "EthernetAdministrator.h"
#include "StubEthernetDriver.h"


#ifdef __GNUC__
#pragma implementation
#endif

StubEthernetDriver::StubEthernetDriver(unsigned char mask)
    : EthernetDriver(mask)
{
    unsigned char buff[7];

    Console() << "StubEthernetDriver ... constructor starting up.\n" << eor;

	//  Set the _ethernetAddress
    for (int i = 0 ; i < 7 ; i++) {
	buff[i] = (unsigned char) i;	// set address to 00:01:02:03:04:05
    }
    _ethernetAddress = new EthernetAddress(buff);

	//  Since this is a stub thingie, we have to fork a daemon process
	//  which will pretend to receive packets at regular intervals.
    _dummyProcess = new PreemptableSystemProcess(
					(OPFV) &StubEthernetDriver::daemon,
					thisProcess()->domain(),// domain
					small,			// stacksize
					(int) this);		// argument
    _dummyProcess->setName("StubEthernetDriver Daemon");
    _dummyProcess->ready();
    thisProcess()->relinquishProcessor();

    Console() << "StubEthernetDriver: started dummyProcess, process ptr = "
	      << asAddress(_dummyProcess) << '\n' << eor;

    _sequenceNumber = 1;
}

StubEthernetDriver::~StubEthernetDriver()
{
    delete _ethernetAddress;
}

int
StubEthernetDriver::transmitPacket(NetworkBuffer *buff)
{
    // this is a stub ... so we don't do anything
    Console() << "StubEthernetDriver::transmitPacket called with packet:\n\t"
	      << buff << '\n' << eor;
	return 0;
}

void
StubEthernetDriver::receivePacket(octet *, unsigned int)
{
//    unsigned short *ptr;
    NetworkBuffer *buffer;

    static const unsigned int TestPacketLength = 64;
    buffer = getBuffer();

    buffer->setLength(TestPacketLength);
    unsigned char *pack = buffer->packet();
    pack[ 0] = 0xff;				// destination Ethernet address
    pack[ 1] = 0xfe;
    pack[ 2] = 0xfd;
    pack[ 3] = 0xfc;
    pack[ 4] = 0xfb;
    pack[ 5] = 0xfa;

    pack[ 6] = 0x00;				// source Ethernet address
    pack[ 7] = 0x01;
    pack[ 8] = 0x02;
    pack[ 9] = 0x03;
    pack[10] = 0x04;
    pack[11] = 0x05;

    pack[12] = (EthernetSpecialTestType >> 8);	// protocol type code
    pack[13] = (EthernetSpecialTestType & 0xFF);

    pack[14] = (_sequenceNumber >> 8);		// use network byte order
    pack[15] = _sequenceNumber & 0xff;
    _sequenceNumber++;

    for (unsigned int i = 16 ; i < TestPacketLength ; i++) {
	pack[i] = 0x55;	// fill rest of buffer
    }

    Console() << "StubEthernetDriver::receivePacket: \"received\" packet # "
	      << asHex(_sequenceNumber) << '\n' << eor;

    _administrator->receive(buffer);
}


    void
StubEthernetDriver::daemon()
{
    Console() << "StubEthernetDriver::daemon() starting up.\n" << eor;

    for (unsigned int i = 0 ; i < 4000000 ; i++);	// delay

    for (;;) {					// "receive" forever
	receivePacket(0,0);
	for (unsigned int i = 0 ; i < 2000000 ; i++);
    }
}
