#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Johnny Zweig (zweig@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * IPChecksum.cc
 *
 */

#include "OutputStream.h"
// REMINDER: When checking the checksum on an incoming message, 0x0000 and 
//   0xffff are  both the same number! 
//   (Plus or minus zero, in 1's-complement....)
#include "ConduitTypes.h"
#include "Word.h"
#include "IPChecksum.h"


#ifdef __GNUC__
#pragma implementation
#endif


#ifndef QUIET
#undef Debug
#define Debug Console()
#endif


Word16
IPChecksum( octet * datagram, int length ) {
	// Add all the 16-bit words (in HOST byte-order!) inside a 32-bit word,
	//   so that the carries can all be added in at the end of the 
	//   calculation. Notice that, regardless of host byte order, if we 
	//   assign the sum to an int16, the byte with the lower address will 
	//   be the one with the high order bits (i.e. we never switched them, 
	//   even though the host might have regarded the high order 
	//   bits as low order when doing the sum -- the end-around carry
	//   takes care of everything!), so casting a pointer to an int16 
	//   as an octet * will cause the assignment to a Word16 to do the
	//   Right Thing. (See RFC 1071 for lots about checksum-computation.)

	int32	sum = 0;
	int16	tmp;
	Word16	num;
	int16 *	foo = (int16 *)datagram;

	while ( length > 1 ) {
		sum += *foo++;
		length -= 2;
	}

	// Add left-over byte, if any
	if ( length > 0 ) {
		int16 holder;	// Make it an int16 so alignment is right
		if (HOFFSET == 1)
		{
		    //  This is for the Multimax.
		    char * holderarray = (char*) & holder;
		    holderarray[0] = *foo;
		    holderarray[1] = 0x00;
		}
		else
		{
		    //  This is for the SPARC.
    	    	    int8 b = *(int8 *) foo;
    	    	    holder = ((int16) b) << 8;
		}
		sum += holder;
			// To make sure there is a zero after that last byte!
	}

	// Fold 32-bit sum to 16 bits (while-loop since may have to add twice
	//   if the first addition causes overflow out of lo-order 16 bits)
	while ( sum >> 16 )
		sum = ( sum & 0x0000ffff ) + ( sum >> 16 );

	tmp = sum;
	tmp = ~tmp;

	num.equals((octet *) &tmp);
		// Here, then, we've basically casted from octet * to int16 *
		//   and back to octet *; it ends up with the right byte order
	return( num );
}

