#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Johnny Zweig (zweig@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * IPConduit.cc 
 *
 */
/*
	See Conduit.h for much better comments about what all this stuff
	does....
 */

#include "OutputStream.h"
#include "ConduitTypes.h"
#include "Word.h"
#include "CPU.h"
#include "Process.h"
#include "ThisProcess.h"
#include "Semaphore.h"
#include "defZOOT.h"
#include "ZootConfig.h"
#include "ZootReturnCodes.h"
#include "Assert.h"
#include "ZootAddress.h"
#include "ConduitMessage.h"
#include "ZootClock.h"
#include "ConduitMessageBuffer.h"
#include "MessageFIFO.h"
#include "IPChecksum.h"
#include "ZootStateMachine.h"
#include "Zoot_IP_Output.h"
#include "Conduit.h"


#ifdef __GNUC__
#pragma implementation
#endif

// -------------------- Class IPConduit stuff: -------------------
IPConduit::IPConduit()
{
	int i;

	my_proto = IP;
	Debug << "IPConduit() -- my_proto = " << my_proto << "\n" << eor;
	_size = 10;
	_count = 0;
	_top = new Conduit *[256];
		// I'll just use the PROTO field to index directly into the
		//   table.
	for( i = 0; i < 256; i++ ) {
	   _top[i] = 0;
	}
}

IPConduit::~IPConduit()
{
	delete _top;
}
Return_Code
IPConduit::connectTo( Conduit * c )
{
	Return_Code ret;

	_ethernet = 0;

	// Do a double-dispatch with the other Conduit:
	ret = c->connectFromAbove( this, my_proto );

	if( ret != OK )
	{
	    // Undo the connectFromBelow() that c did
	    _ethernet = 0;
	}
	return( ret );
}
Return_Code
IPConduit::connectFromAbove( Conduit * c, ProtocolID proto )
{
	Return_Code ret;
	int his_handy;

	// Ignore other guy's protocol -- just interested in his IP-handy#
	Debug << "IPConduit::cFA() protocol:" << proto << "\n" << eor;

	ret = c->connectFromBelow( this, my_proto, 0, his_handy );
	if( ret == OK ) {
	    // Notice that this other conduit will set his_handy to be the
	    //   IP protocol field he wishes to use.
	    // First, it would be terribly wise to look through the table and
	    //   make sure nobody is already using the protocol specified by
	    //   his_handy at this point....
	    if( 0 > his_handy || 255 < his_handy ) return( ILLEGAL_PORT );
			// this seems as good an error to use as any
	    if( 0 != _top[his_handy] ) return( PORT_IN_USE );
			// this seems as good an error to use as any
	    if( _count == _size ) {
		return( TOO_MANY_CONNECTIONS );
	    }
	    else {
		// Update the appropriate pointer in the array, so that
		//   incoming messages can be delivered to this conduit (see
		//   IPConduit::insertFromBelow):
		_top[his_handy] = c;
	   	_count = _count + 1;
	    }
	}
	return( ret );
}
Return_Code
IPConduit::connectFromBelow( Conduit * c, ProtocolID proto, int his_handy,
				int & my_handy )
{
	Debug << "IPConduit::cFB() protocol:" << proto << "\n" << eor;
	if( proto != Ethernet ) return( INVALID_PROTOCOL );

	if( _ethernet != 0 ) return( CANNOT_CONNECT );
		// Already connected to an EthernetConduit....

	_ethernet = c;
	my_handy = 0x0800;	// See RFC 1010 for IP-protocol-numbers
	return( OK );
}
Return_Code
IPConduit::disconnectFrom( Conduit * c )
{
	Return_Code ret;

	if( _ethernet != c ) return( NO_SUCH_CONDUIT );

	ret = c->disconnectFromAbove( this );
	if( ret == OK ) {
	    _ethernet = 0;
	}
	return( ret );
}
Return_Code
IPConduit::disconnectFromAbove( Conduit * c )
{
	int i;

	if( _count == 0 ) {
	   return( NO_SUCH_CONDUIT );	// in fact, no conduits at all....
	}
	else {
	   // Find the pointer to this guy (if there is one):
	   for( i = 0; i < _size && _top[i] != c; i++ ) { }
	   if( i == _size ) {
	      return( NO_SUCH_CONDUIT );
	   }
	   else {
	      _top[i] = 0;
	      _count--;
	   }
	}
	return( OK );
}


Return_Code
IPConduit::insertFromAbove( ConduitMessage * msg, NetworkAddress * addr )
{
	// The following IP header fields need to be specified by the sender:
	//   PROTOCOL (since IP can't tell / doesn't care who inserted the 
	//   message), Type Of Service (TOS), and FLAGS (such as Don'tFragment)
	//
	// The IP conduit will set Time To Live, source and destination
	//   IP-addresses, IDENT, checksum, Version, HLEN, TotalLength
	Return_Code ret;
	IPMessage * ipmsg;
	IPAddress * ipaddr;
	// int i; // temporaries
	nibble tmp;
	Word16 tmp16;

	if( IP != msg->protocol() ) return( INVALID_MESSAGE_TYPE );

	ipmsg = (IPMessage*)msg;
	ipaddr = (IPAddress*)addr;
	ipmsg->setDestADDR( ipaddr->val() );
	ipmsg->setSourceADDR( TheLocalIPAddress.val() );
	ipmsg->setVERS( (nibble) 4 );
	tmp = ipmsg->headerLength() / 4;
		// HLEN encoded in # of 32-bit words in the header...
	ipmsg->setHLEN( tmp );

	short unsigned int tmp2 = ipmsg->length();

	tmp16.equals( tmp2 );
	ipmsg->setTotalLength( tmp16 );

	// ...and send the fucker along its merry way:    {{{JZ}}}
	// THIS IS OLD CODE AND TOTALLY BOGUS! I will not talk to my address
	//   resolution server at this point. I will not calculate the
	//   checksum, and I will not insert it into an EthernetConduit.
	//   Instead, I will fake it all by calling this utterly bogus
	//   function:
	ret = Zoot_ip_output( ipmsg );

	return( ret );
}
void
IPConduit::Reset()
{
	int i;
	// Disconnect everything:
	for(i=0; i < _size; i++) {
	   _top[i] = 0;
	}
	_count = 0;
}

volatile SemaphoreRegulatedMessageFIFO * TCPIncomingMessageFIFO = 0;
	// Gets initialized by the TCPConduit constructor....
volatile SemaphoreRegulatedMessageFIFO * ICMPIncomingMessageFIFO = 0;
	// Gets initialized by the TCPConduit constructor....


Return_Code
IPConduit::insertFromBelow( ConduitMessage * msg )
{
//   msg should be an EthernetMessage, with header/data fields
//   set up appropriately.
	int len;
	octet * finger;
	IPMessage * ipmsg;
	Word16 smack;
	Return_Code ret = ERROR;

//#define Debug Console()
	Debug << "*** IP Packet received by ZOOT!!! ***\n" << eor;

	// Calculate total length of incoming packet:
	smack.equals( msg->data() + 2 );
	len = smack.val();
	Debug << "\tappears to be " << asHex(len) << " octets long(IP-data)\n"
			<<eor;

	ipmsg = new IPMessage( len, 0 );
	msg->setLink( ipmsg );
	// Copy from ethernet packet to IP-message:
	finger = ipmsg->header();
	octetcopy( finger, msg->data(), len );

	// Set up the IP Message sensibly:
	ipmsg->setData(   (ipmsg->TotalLength().val() - ipmsg->HLEN()*4 ),
			ipmsg->header() + ipmsg->HLEN()*4 );
	ipmsg->pleaseDontDeleteData();
	// Set header to itself with the correct length:
	ipmsg->setHeader( ipmsg->HLEN()*4, ipmsg->header() );

	// Check the Checksum:
	Word16 checksum = IPChecksum( ipmsg->header(), ipmsg->headerLength() );
	if( 0 != checksum.val() ) {
	    ipmsg->del();
	    Debug << "IP Checksum Error!!! (" << asHex( checksum.val() )
			<< ")\n" << eor;
	    return( CHECKSUM_ERROR );
	}

	// GACK!!! I think there is a race if _top is being updated. We need
	//   to make sure that while doing _top[whatever]->insertFromBelow()
	//   that we are _not_ holding any mutex lock, since that call may
	//   take arbitrarily long.  Shit. This is vexing, and requires
	//   further thought (Johnny Z 16APR92).

	// DEMULTIPLEX the packet, based on its PROTO field:
	if( _top[ ipmsg->PROTO() ] != 0 ) {
//this should be zapped, I think....
/******
	    if( 6 == ipmsg->PROTO() ) {
		// Queue TCP msg for reception:
		if( TCPIncomingMessageFIFO != 0 ) {
//Debug << "TCP message\n" << eor;
			TCPIncomingMessageFIFO->add( ipmsg );
			ret = OK;
		}
		else {
		    ipmsg->del(); // TCP isn't awake yet....
		    Debug << "IP Cannot Deliver!!!\n" << eor;
		    ret = CANNOT_DELIVER;
		}
	    }
	    else {
//up through here....
********************/
//Debug << "IP message\n" << eor;
		ret = _top[ ipmsg->PROTO() ]->insertFromBelow( ipmsg );
//	    }
//and don't forget to delete this closing-brace here....
	}
	else {
	    ipmsg->del();	// boo-hoo!
	    Debug << "Unable to deliver packet.\n" << eor;
	    ret = NO_SUCH_CONDUIT;
	}
	return( ret );

}

//------------------------ ZootReceiverDaemon(): ---------------------------
// These need to be launched by the Zoot Controller at system initialization.
//   This is how IP and TCP are decoupled processwise.
extern volatile TCPConduit * TheTCPConduit; // defined in ZootController.cc
extern volatile int ZootNotReady;
void
ZootReceiverDaemon()
{
#ifdef ZOOT
	IPMessage * msg = 0;
	
	while( ZootNotReady )
		thisProcess()->relinquishProcessor();
	for(;;) {	// forever
	    if( TCPIncomingMessageFIFO != 0 )
	    {
		msg =
		  (IPMessage *)(((SemaphoreRegulatedMessageFIFO *)TCPIncomingMessageFIFO)->remove());
		// I had to cast the thing without the volatile * so the
		// compiler wouldn't die. I really hope that works. - jcarlyle
	    }
//	    Debug << "ZootReceiverDaemon: just nabbed msg....\n" << eor;
	    if( msg != 0 && TCPIncomingMessageFIFO != 0 )
	    {
	        ((TCPConduit *)TheTCPConduit)->insertFromBelow( msg );
		// I had to cast the thing to a TCPConduit * so the compiler
		// wouldn't err about a type mismatch for the 'this' argument
		// - jcarlyle
	    }
	    else {
		msg->del();
		Debug << "ZRD: cannot deliver message\n" << eor;
	    }
	}
#endif
}
//JZ: The Daemon should be told about the TCP Conduit so it doesn't have to
//fuck around with this global variable. Also, IP shouldn't be ready to hand
//messages to TCP until they've been connected by the controller. Also, 
//zoot_input() ought to know about the IP conduit and/or TCP Conduit.
//  Actually, none of this ought to exist at all. It should go from the
//EthernetConduitWalker through IP, TCP, whatever, and fuck this daemon
//nonsense!
