#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Johnny Zweig (zweig@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * Zoot_IP_Output.cc
 *
 */

#include "OutputStream.h"
#include "Assert.h"
#include "ZootConfig.h"
#include "ZootReturnCodes.h"
#include "ZootAddress.h"
#include "Conduit.h"
#include "ConduitMessage.h"
#include "IPChecksum.h"
#include "ZootClock.h"
#include "Ethernet.h"
#include "EthernetAdministrator.h"
#include "Zoot_IP_Output.h"


#ifdef __GNUC__
#pragma implementation
#endif

unsigned short ip_id;
static const unsigned int EthernetHeaderSize = 14;
extern volatile Conduit * TheEthernetConduit; // see ZootController.cc

#ifndef QUIET
#undef Debug
#define Debug Console()
#endif

Return_Code
Zoot_ip_output( IPMessage * msg ) {

	Assert( msg != 0 );
	// UNUSED int length = msg->length();
	int hlen = msg->HLEN() * 4;
//	unsigned char * rawEthernetPacket = 0;
//	octet * packetPointer;
	ConduitMessage * ethermsg;

	Debug << "\nZ_IP_O(" << asAddress( msg ) << ")\n" << eor;
//	msg->print();

//	rawEthernetPacket = new unsigned char[ MaxEthernetPacketLength ];
//	packetPointer = &rawEthernetPacket[0];

	msg->setIDENT( ip_id++ );

	// Checksum:
	msg->setHChecksum( IPChecksum(msg->header(), hlen) );

	ethermsg = new EthernetMessage( 0, 0, msg );
		// Note: EthernetMessages always have a header, so this
		//   first 0-argument is misleading.

	TheLocalGatewayEthernetAddress.copyTo( ethermsg->header() );
	TheLocalEthernetAddress.copyTo( ethermsg->header() + 6 );
	*(ethermsg->header() + 12) = 0x08;	// ethertype = 0x0800 (IP)
	*(ethermsg->header() + 13) = 0x00;

	// Check if it's for a machine on this subnet, and send directly
	//   there in that case:  (LATER -JZ)

	// Bon voyage!
	((Conduit *)TheEthernetConduit)->insertFromAbove( ethermsg, 0 );
	// Had to cast so the compipler wouldn't die complaing about 'this' arg
	// -- jcarlyle

/*******************************
	for( i = 0; i < length + 14; i++ )
		rawEthernetPacket[i] = 0;

	// Set up rawEthernetPacket's source-addr:

	TheLocalEthernetAddress.copyTo( rawEthernetPacket + 6 );

	//rawEthernetPacket[6] = 0x8; // FROM: unclem.cs.uiuc.edu
	//rawEthernetPacket[7] = 0x0;
	//rawEthernetPacket[8] = 0x4c;
	//rawEthernetPacket[9] = 0x0;
	//rawEthernetPacket[10] = 0x27;
	//rawEthernetPacket[11] = 0xac;

	// Set up rawEthernetPacket's destination-addr:
	//  (ugly hack)




	// With the lack of any known ethernet addresses I'll just fake it 
	// hosts that we have here in our lab.....  I'll fix this 
	// as they say,  at a later date.....


	switch( msg->DestADDR().val() ) {
*****
		case 0x80aeee04:   // brutus:
			rawEthernetPacket[0] = 0x8;
			rawEthernetPacket[1] = 0x0;
			rawEthernetPacket[2] = 0x20;
			rawEthernetPacket[3] = 0x01;
			rawEthernetPacket[4] = 0x5d;
			rawEthernetPacket[5] = 0x1c;
			break;
		case 0x80aeee04:   // casca:
			rawEthernetPacket[0] = 0x8;
			rawEthernetPacket[1] = 0x0;
			rawEthernetPacket[2] = 0x20;
			rawEthernetPacket[3] = 0x6;
			rawEthernetPacket[4] = 0x16;
			rawEthernetPacket[5] = 0xfa;
			break;
		case 0x80aeee03: // cassius:
			rawEthernetPacket[0] = 0x8;
			rawEthernetPacket[1] = 0x0;
			rawEthernetPacket[2] = 0x20;
			rawEthernetPacket[3] = 0x6;
			rawEthernetPacket[4] = 0x11;
			rawEthernetPacket[5] = 0x50;
			break;
		case 0x80aeee34:   // cato:
			rawEthernetPacket[0] = 0x2;
			rawEthernetPacket[1] = 0x60;
			rawEthernetPacket[2] = 0x8c;
			rawEthernetPacket[3] = 0x83;
			rawEthernetPacket[4] = 0x77;
			rawEthernetPacket[5] = 0x5;
			break;
		case 0x80aeee30:   // babym:
			rawEthernetPacket[0] = 0x8;
			rawEthernetPacket[1] = 0x0;
			rawEthernetPacket[2] = 0x4c;
			rawEthernetPacket[3] = 0x0;
			rawEthernetPacket[4] = 0x27;
			rawEthernetPacket[5] = 0xb7;
			break;
*****
// UP TO DATE AS OF 23 MAR 1992 -JZ
		case 0x80aeee02:   // cousinm:
			rawEthernetPacket[0] = 0x8;
			rawEthernetPacket[1] = 0x0;
			rawEthernetPacket[2] = 0x4c;
			rawEthernetPacket[3] = 0x0;
			rawEthernetPacket[4] = 0x23;
			rawEthernetPacket[5] = 0xf0;
			break;
		case 0x80aeee01:   // auntiem:
			rawEthernetPacket[0] = 0x8;
			rawEthernetPacket[1] = 0x0;
			rawEthernetPacket[2] = 0x4c;
			rawEthernetPacket[3] = 0x0;
			rawEthernetPacket[4] = 0x20;
			rawEthernetPacket[5] = 0x04;
			break;
		case 0x80aeee03:   // unclem:
			rawEthernetPacket[0] = 0x8;
			rawEthernetPacket[1] = 0x0;
			rawEthernetPacket[2] = 0x4c;
			rawEthernetPacket[3] = 0x0;
			rawEthernetPacket[4] = 0x27;
			rawEthernetPacket[5] = 0xb7;
			break;
		case 0x80aeee04:   // envius:
			rawEthernetPacket[0] = 0x8;
			rawEthernetPacket[1] = 0x0;
			rawEthernetPacket[2] = 0x20;
			rawEthernetPacket[3] = 0x0e;
			rawEthernetPacket[4] = 0xe6;
			rawEthernetPacket[5] = 0x1f;
			break;
		default: // Copy the local gateway's ethernet address:
			TheLocalGatewayEthernetAddress.
				copyTo( & rawEthernetPacket[0] );
			break;
	}
	// ethernet packet type is IP
	rawEthernetPacket[12] = 0x8;
	rawEthernetPacket[13] = 0x0;

	m = msg;
	packetPointer += EthernetHeaderSize;
	// Assume msg has an IP header, and copy it:
	octetcopy( packetPointer, m->header(), hlen );
	packetPointer += hlen;
	// Copy any data:
	if( m->dataLength() != 0) {
	    octetcopy( packetPointer, m->data(), m->dataLength() );
	    packetPointer += m->dataLength();
	}
	// Copy all the headers and data and stuff from linked messages:
	while( 0 != m->link() ) {
	    m = m->link();
	    if( 0 != m->headerLength() )
		octetcopy( packetPointer,
			m->header(),
			m->headerLength() );
	    packetPointer += m->headerLength();
	    if( 0 != m->dataLength() )
		octetcopy( packetPointer,
			m->data(),
			m->dataLength() );
	    packetPointer += m->dataLength();
	}
	// Note that m no longer points to the original IP message!!!!


	// SEND THE LITTLE ANGEL OFF:

	//DEBUG:
*
	Debug << "DUMP:";
	for( i = 0; i < packetPointer - &rawEthernetPacket[0]; i++ )
	    Debug << (int)rawEthernetPacket[i] << " ";
	Debug << "\n" << eor;
*

//	EtherXmit( (char *)rawEthernetPacket, length + EthernetHeaderSize );

	delete rawEthernetPacket;	// EtherXmit() is synchronous -- the
					// data in the packet have been
					// transmitted now.
*******************************/

	return( OK );
}

