#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Tanya L. Crenshaw (tcrensha@uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "AboriginalTelnet.h"
#include "Kernel.h"
#include "Process.h"
#include "ThisProcess.h"
#include "Conduit.h"
#include "CharacterInputStream.h"
#include "PersistentMemoryObject.h"
#include "PersistentMemoryObjectStar.h"
#include "RecordStream.h"
#include "RecordStreamStar.h"
#include "FileSystemInterface.h"
#include "FileSystemInterfaceStar.h"
#include "Assert.h"
#include "String.h"
#include "Byte.h"
#include "NullStreams.h"
#include "UserSession.h"
#include "UserSessionInfo.h"
#include "NameServer.h"
#include "SystemInterface.h"
#include "AboriginalPing.h"

#ifdef __GNUC__
#pragma implementation
#endif


AboriginalPing :: AboriginalPing ()
{
    setName ("Aboriginal Ping");
}

AboriginalPing :: ~AboriginalPing ()
{

}

void 
AboriginalPingController()
{
  Return_Code ret;

  Console() << "PING: " << eor ;

#if 0
  //Initialize the Local Network addresses.  This is
  //done by Zoot Controller and assumes that a
  //Ethernet Admin is already in place.  We are ommitting
  // Zoot Controller and therefore this must be done here.
  extern void InitializeLocalNetworkAddresses();
  InitializeLocalNetworkAddresses();
#endif

  // Create the Conduits required for ping
  myICMPConduit = new ICMPConduit();
  myIPConduit = new IPConduit();
  myEthernetConduit = new EthernetConduit();

  Assert (myIPConduit != 0);
  Assert (myEthernetConduit != 0);
  Assert (myICMPConduit != 0);

  // Build a communication pathway with the
  // newly created Conduits

  Debug << "Connect ICMP to IP..." ;
  ret = ((Conduit*)myICMPConduit)->connectTo( (Conduit*)myIPConduit);
  Debug << "done.\n";

  Debug << "Connect IP to Ethernet..." ;
  ret = ((Conduit*)myIPConduit)->connectTo( (Conduit*)myEthernetConduit);
  Debug << "done.\n" ;

  // Create a default ICMP Message
  ICMPMessage * myICMPMessage;
  myICMPMessage = new ICMPMessage();

  // Set type to ECHO REQUEST
  myICMPMessage->setMessageType(8);

  // TODO: How to make a NetworkAddress from the parameter?
  // TODO: Paramter in SystemProcess can only be an int??
  //myICMPConduit.insertFromAbove(myICMPMessage, 0);

  // Indicate that the process is complete so that
  // the processor may be relinquished.
  AboriginalPingControllerInitialized = 1;
}



