#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Lee Lup Yuen (lup@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "AboriginalTelnet.h"
#include "Kernel.h"
#include "Process.h"
#include "ThisProcess.h"
#include "Conduit.h"
#include "CharacterInputStream.h"
#include "PersistentMemoryObject.h"
#include "PersistentMemoryObjectStar.h"
#include "RecordStream.h"
#include "RecordStreamStar.h"
#include "FileSystemInterface.h"
#include "FileSystemInterfaceStar.h"
#include "Assert.h"
#include "String.h"
#include "Byte.h"
#include "NullStreams.h"
#include "ApplicationConduitInputStream.h"
#include "ApplicationConduitOutputStream.h"
#include "UserSession.h"
#include "UserSessionInfo.h"
#include "NameServer.h"
#include "SystemInterface.h"


#ifdef __GNUC__
#pragma implementation
#endif

//  IAC codes (see RFC 854)
enum {IAC_SE   = 240, IAC_NOP = 241, IAC_DATAMARK = 242, IAC_BRK  = 243,
      IAC_IP   = 244, IAC_AO  = 245, IAC_AYT      = 246, IAC_EC   = 247,
      IAC_EL   = 248, IAC_GA  = 249, IAC_SB       = 250, IAC_WILL = 251,
      IAC_WONT = 252, IAC_DO  = 253, IAC_DONT     = 254, IAC_IAC  = 255,
      IAC      = 255};
enum {IAC_ECHO = 1, IAC_SUPPRESS_GO_AHEADS = 3, IAC_TERMINAL_TYPE = 24};

AboriginalTelnet :: AboriginalTelnet ()
{
    setName ("Aboriginal Telnet Server");
    iacMode = 0;
    iacCommandIndex = 0;
}

AboriginalTelnet :: ~AboriginalTelnet ()
{
}

void AboriginalTelnet :: aboriginalTelnetDaemon (AboriginalTelnet *client)
{
    TCPMessage *msg = 0;
    Assert( client != 0 );
    
    // Wait for someone to call up.
    msg = (TCPMessage *) client -> incoming.remove ();
    unsigned char clientAddr [4];
    msg -> SourceAddress ().val ().copyTo (clientAddr);
    Console () << "[Incoming Telnet connection from " <<
    	(int) clientAddr [0] << "." << (int) clientAddr [1] << "." <<
	(int) clientAddr [2] << "." << (int) clientAddr [3] <<
	" established]\n" << eor;

    // Make another conduit to handle the next connection.
    AboriginalTelnet *telnet = new AboriginalTelnet;
    telnet -> passiveStart ();

#ifdef NOTUSED
    // Make sure that client is an authorized host.
    if (clientAddr [0] != 128 || clientAddr [1] != 174 ||
    	clientAddr [2] != 240)  //  Tapestry net
#else
    if (0)
#endif
    {
    	const char badClient [] = "Unauthorized host.  "
	    "Contact choices@cs.uiuc.edu for assistance.\r\n";
    	client -> rawWrite (badClient, strlen (badClient));
    	client -> close ();
    	Console () << "[UNAUTHORIZED Telnet connection from " <<
    	    (int) clientAddr [0] << "." << (int) clientAddr [1] << "." <<
	    (int) clientAddr [2] << "." << (int) clientAddr [3] <<
	    " closed]\n" << eor;
    	thisProcess () -> die ();
    }

    // Make a session for this connection.
    client -> setUserSession (new UserSession (UserSessionInfo::TelnetSession,
    	client, clientAddr));

    // Startup a shell for this connection, and set up stdin and stdout.
    PreemptableSystemProcess *shellProcess = new PreemptableSystemProcess
    	((PFV) startupShell, thisProcess () -> domain(), huge, (int) client);
    Assert (shellProcess != 0);
    shellProcess -> setName ("#Aboriginal Telnet Shell Process");
    shellProcess -> ready ();

    // Queue up all messages read, watching out for the finish condition.
    for (;;)
    {
    	// Strip null characters and interpret IAC commands.
    	int length;
    	char *characters = client -> processIncomingMessage (msg, length);
    	if (length > 0) {
    	    client -> characterQueue -> enqueue (characters, length);
		delete characters;
		client -> tcpConduit () -> expandWindow (*(client -> session ()),
	    msg -> dataLength ());
	    }
	if (msg -> FIN ())
	{
    	    msg -> del ();
	    break;
	}
    	msg -> del ();
	msg = (TCPMessage *) client -> incoming.remove ();
    }
    //  Disconnect myself and prepare to die.
    client -> close ();
    UserSession *sess = client -> getUserSession ();
    if (sess != 0) delete sess;
    Console () << "[Telnet connection from " <<
    	(int) clientAddr [0] << "." << (int) clientAddr [1] << "." <<
	(int) clientAddr [2] << "." << (int) clientAddr [3] <<
	" closed]\n" << eor;
    thisProcess () -> die ();
}

char *AboriginalTelnet :: processIncomingMessage (TCPMessage *msg, int &length)
{
    //  Return the data bytes in msg after stripping off the null characters
    //  and interpreting the IAC commands.  Return in length the number of
    //  data bytes returned.
    length = msg -> dataLength ();
    unsigned char *dataSrc = (unsigned char *) msg -> data ();
    unsigned char *dataDest = new unsigned char [length];
    int dest = 0;
    for (int src = 0; src <= length - 1; src++)
    {
	if (iacMode == 0 && dataSrc [src] == 0)
	    continue;
    	else if (iacMode == 0 && dataSrc [src] == '\r')
	{
	    dataDest [dest++] = '\n';
	    continue;
	}
	else if (iacMode == 0 && dataSrc [src] == IAC)
	{
	    //  This is an IAC command.  Enter IAC mode.
	    iacMode = 1;
	}
	else if (iacMode == 0)
    	    dataDest [dest++] = dataSrc [src];
	else if (iacMode)
	{
	    iacCommand [iacCommandIndex++] = dataSrc [src];
	    switch (iacMode)
	    {
	    case 1:
	    	switch (dataSrc [src])
	    	{
	    	//  Commands with no arguments.
	    	case IAC_NOP: case IAC_DATAMARK: case IAC_BRK: case IAC_IP:
	    	case IAC_AO: case IAC_AYT: case IAC_EC: case IAC_EL:
		case IAC_GA:
	    	    iacMode = 0; break;
		case IAC_IAC:
		    dataDest [dest++] = IAC;
		    iacMode = 0; break;
	    	//  Commands with one argument.
	    	case IAC_WILL: case IAC_WONT: case IAC_DO: case IAC_DONT:
	    	    iacMode = 2; break;
	    	//  Commands with variable arguments, terminated by IAC_SE.
	    	case IAC_SB:
	    	    iacMode = 3; break;
	    	default:
	    	    Console () << "Unrecognized IAC command\n" << eor;
		    iacMode = 0;
	    	}
		break;
	    case 2:
	    	//  A single argument has been read; exit IAC mode.
	    	iacMode = 0; break;
	    case 3:
	    	//  If we see an IAC_SE, exit IAC mode.
	    	if (dataSrc [src] == IAC_SE) iacMode = 0;
		break;
	    default:
	    	Console () << "IAC internal error\n" << eor;
	    }
	    if (iacMode == 0)
	    {
		processIACCommand ();
	    }
	}
	else
	    Assert (0);
    }
    length = dest;
    return (char *) dataDest;
}

char *AboriginalTelnet :: processOutgoingMessage (const char *buffer,
    int &count)
{
    //  Return buffer with all \n characters prepended with \r.  Also
    //  escape all IAC characters.  Store the number of characters returned
    //  in count.
    unsigned char *dataSrc = (unsigned char *) buffer;
    unsigned char *dataDest = new unsigned char [count * 2];
    int dest = 0;
    for (int src = 0; src <= count - 1; src++)
    {
    	if (dataSrc [src] == '\n')
	{
	    dataDest [dest++] = '\r';
	    dataDest [dest++] = '\n';
	}
    	else if (dataSrc [src] == IAC)
	{
	    dataDest [dest++] = IAC;
	    dataDest [dest++] = IAC;
	}
	else
	    dataDest [dest++] = dataSrc [src];
    }
    count = dest;
    return (char *) dataDest;
}

void AboriginalTelnet :: processIACCommand ()
{
    unsigned char reply [100];
    if (iacCommand [0] == IAC_DO && iacCommand [1] == IAC_SUPPRESS_GO_AHEADS)
    {
	reply [0] = IAC;
	reply [1] = IAC_WILL;
	reply [2] = IAC_SUPPRESS_GO_AHEADS;
	rawWrite ((char *) reply, 3);
    }
    else if (iacCommand [0] == IAC_DO && iacCommand [1] == IAC_ECHO)
    {
	reply [0] = IAC;
	reply [1] = IAC_DO;
	reply [2] = IAC_ECHO;
	rawWrite ((char *) reply, 3);
    }
    iacCommandIndex = 0;
}

const char telnetWelcome [] =
"Welcome to the Choices Aboriginal Telnet Server (CATS 0.0)\n"
"For assistance contact lup@cs.uiuc.edu\n\n"
"Why is this Telnet server aboriginal to Choices?  Well, according to OED2,\n"
"it's the \"first or earliest so far as history or science gives record;\n"
"primitive; strictly native\" (i.e. it contains native C++ code, uses native\n"
"Choices objects, and it's not a hacked version of Unix telnetd).\n\n"
"This Telnet server is to be used only by researchers working on the Choices\n"
"project.  (Actually, the people who are in charge of the Choices project\n"
"are also permitted to use the server, but I'm sure they have better things\n"
"to do with their time than toying around with the server and hoping to find\n"
"a way to crash it.)  If you are not authorized to use this server, or if\n"
"you are one of those pesky people who roam the Internet with the intention\n"
"of abusing somebody else's computing resources, please remove yourself from\n"
"the server immediately, or you will be forever haunted by nightmares of\n"
"grotesque and malformed objects that have been created by all past and\n"
"present C++ programmers in the world who do not fully understand the\n"
"object-oriented programming paradigm.\n\n";

void AboriginalTelnet :: startupShell (AboriginalTelnet *client)
{
    const unsigned char echo [] = {IAC, IAC_WILL, IAC_ECHO};
    client -> rawWrite ((const char *) echo, 3);
    client -> write (telnetWelcome, strlen (telnetWelcome));
    client -> flush ();
    int err;
    userLogin (client);
    FileSystemInterfaceStar kernelFSI = StandardNameServer -> lookup
    	("fileSystemInterface", "FileSystemInterface");
    Assert (kernelFSI != 0);
    PersistentMemoryObjectStar pmo = kernelFSI -> open ("/Bin/bash",
    	PersistentMemoryObjectClass, err);
    if (err || pmo == 0)
    {
    	const char msg [] = "Can't open /Bin/bash.  "
	    "Contact choices@cs.uiuc.edu for assistance.\r\n";
    	client -> rawWrite (msg, strlen (msg));
	client -> close ();
    	Console () << "AboriginalTelnet: Can't open /Bin/bash\n" << eor;
	thisProcess () -> die ();
    }
    {
    	InputStreamStar iss_stdin = new ApplicationConduitInputStream (client);
    	OutputStreamStar oss_stdout = new ApplicationConduitOutputStream (client);
    	FileSystemInterfaceStar fileSystemInterface = new FileSystemInterface
	    (kernelFSI, 0);
	iss_stdin -> setName ("Aboriginal Telnet stdin");
	oss_stdout -> setName ("Aboriginal Telnet stdout");
	fileSystemInterface -> setName ("Aboriginal Telnet file system "
	    "interface");
    	char *argv [] = {0};
    	Process *proc = StandardSystemInterface -> buildProcessFromObjectFile
	    (pmo, 0, argv);
	if (proc == 0)
	{
    	    const char msg [] = "Can't run /Bin/bash.  "
	    	"Contact choices@cs.uiuc.edu for assistance.\r\n";
    	    client -> rawWrite (msg, strlen (msg));
	    client -> close ();
    	    Console () << "AboriginalTelnet: Can't run /Bin/bash\n" << eor;
	}
	NameServerStar procNameServer = proc -> nameServer ();
	procNameServer -> bind ("StandardInput", iss_stdin);
	procNameServer -> bind ("StandardOutput", oss_stdout);
	procNameServer -> bind ("fileSystemInterface", fileSystemInterface);
	proc -> ready ();
	thisProcess () -> waitForProcess (proc);
    }
    pmo = 0;
    client -> finish ();
    thisProcess () -> die ();
}

void AboriginalTelnet :: userLogin (AboriginalTelnet *client)
{
    //  Prompt user for a username.
    for (;;)
    {
    	const char loginMessage [] = "Choices Login: ";
	client -> write (loginMessage, strlen (loginMessage));
	client -> flush ();
	char userName [1000];
	int charsRead = client -> read (userName, 1000);
	client -> flush ();
	charsRead--;
	if (charsRead > 0)
	{
	    userName [charsRead] = 0;
	    client -> getUserSession () -> setUserName (userName);
	    break;
	}
    }
    //  Display the message at /Etc/motd.
    int err;
    FileSystemInterfaceStar kernelFSI = TheKernel -> fileSystemInterface ();
    RecordStreamStar rfs = kernelFSI -> open ("/Etc/motd",
    	RecordStreamClass, err);
    if (err || rfs == 0) return;
    int charsRead = 1;
    while (charsRead > 0)
    {
    	char buffer [1024];
    	charsRead = rfs -> read (buffer, 1024);
	client -> write (buffer, charsRead);
    }
    client -> flush ();
    rfs = 0;
}

#include "ClassInstaller.h"
DEFINECLASS("AboriginalTelnet",AboriginalTelnet,ApplicationConduitStream,0,0);


