#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Johnny Zweig (zweig@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * Conduit.cc 
 *
 */
/*
	See Conduit.h for much better comments about what all this stuff
	does....
 */

#include "OutputStream.h"
#include "ConduitTypes.h"
#include "Word.h"
#include "CPU.h"
#include "Process.h"
#include "ThisProcess.h"
#include "Semaphore.h"
#include "ZootConfig.h"
#include "ZootReturnCodes.h"
#include "Assert.h"
#include "ZootAddress.h"
#include "ConduitMessage.h"
#include "ZootClock.h"
#include "ConduitMessageBuffer.h"
#include "MessageFIFO.h"
#include "ZootStateMachine.h"
#include "Zoot_IP_Output.h"
#include "Conduit.h"


#ifdef __GNUC__
#pragma implementation
#endif

// -------------------- Class ApplicationConduit stuff: -------------------
ApplicationConduit::ApplicationConduit()
{
	// Nothing much to do. 
	my_proto = Undefined;
	mysession = 100000; // nonsense
	myTCPConduit = 0;
}
ApplicationConduit::~ApplicationConduit()
{
	// Nothing to do.
}
Return_Code
ApplicationConduit::connectTo( Conduit * c )
{
	Return_Code ret;

	// Do a double-dispatch with the other Conduit:
	ret = c->connectFromAbove( this, my_proto );
	if( ret != OK )
	    myTCPConduit = 0;
	return( ret );
}
Return_Code
ApplicationConduit::connectFromAbove( Conduit * c, ProtocolID proto )
{
	Assert( NOTREACHED );	// I guess....
	return( Return_Code(0) );
}
Return_Code
ApplicationConduit::connectFromBelow( Conduit * c, ProtocolID proto,
				int his_handy,
				int & my_handy )
{
	//Return_Code ret;

	if( proto != TCP ) return( INVALID_PROTOCOL );

	if( myTCPConduit != 0 ) return( CANNOT_CONNECT );
		// Already connected to a TCP Conduit....

	myTCPConduit = (TCPConduit *)c;
	my_handy = 0;
	return( OK );
}
Return_Code
ApplicationConduit::disconnectFrom( Conduit * c )
{
	Return_Code ret;

	if( myTCPConduit != c ) return( NO_SUCH_CONDUIT );

	ret = c->disconnectFromAbove( this );
	if( ret == OK ) {
	    myTCPConduit = 0;
	}
	return( ret );
}
Return_Code
ApplicationConduit::disconnectFromAbove( Conduit * c )
{
	Assert( NOTREACHED );	// Nobody up there
	return( Return_Code(0) );
}

Return_Code  
ApplicationConduit::insertFromBelow( ConduitMessage * msg )
{
	TCPMessage * tcpmsg;

	Assert( msg != 0 );
	tcpmsg = (TCPMessage*)msg;

	// Ought to have stub code to actually transfer data to user. Might
	//   have to repackage message....

	// Instead, this version just throws the message into the bit-bucket:
	//Console().OldStyleDebug("ApplicationConduit::iB(%x)\n",tcpmsg);
	msg->del();

	return( OK );
}
void
ApplicationConduit::setSession( TCPAddress & newsession )
{
	mysession = newsession;
}

// -------------------- Class EchoingApplicationConduit stuff: ------------

EchoingApplicationConduit::EchoingApplicationConduit() : _queue( 50 )
{
	Debug << "Hey Stupid! You just created an Echoing Application Conduit that won't echo anything...\n" << eor;
	my_proto = EchoingApplication;
}

// Having to pass in a scheduler is a termporary hack since this might be
//   launched before thisCPU()->scheduler() is meaningful...
EchoingApplicationConduit::EchoingApplicationConduit(ProcessContainer * sched)
		: _queue( 50 )
{
	Process * process;

	my_proto = EchoingApplication;
	EAnotReady = TRUE;
	Debug << "About to launch an EchoingApplicationConduit daemon...\n(this:"<< this << ")\n" << eor;
	process = new PreemptableSystemProcess(
		(PFV) eacDaemon,			// Name of fn/process
		thisProcess()->domain(),	// New process' domain
		small,					// Stack size
		(int)this );				// 1st argument
	Assert( process != 0 );
	process->setName( "#Echoing Application Conduit Daemon" );

        // @@@ Changed to process->ready() from old syle sched->add( process )
        //  ... Stan
	process->ready();
	//Console().OldStyleDebug("Launched an EchoingApplicationConduit process.\n");
}

EchoingApplicationConduit::~EchoingApplicationConduit()
{
}
Return_Code
EchoingApplicationConduit::insertFromBelow( ConduitMessage * msg )
{
	EAnotReady = FALSE;	// Ugly hack to keep the fucker from doing
				//   anything before it's too soon.
	//Console().OldStyleDebug("EchoingApplicationConduit::iB(%x)\n",msg);
	_queue.add( msg );
	return( OK );
}
void
EchoingApplicationConduit::eacDaemon(EchoingApplicationConduit * eac)
{
	TCPMessage * msg;
	TCPMessage * reply;

	Debug << "\tEchoingApplicationConduitDaemon: Hello!  (eac:"<< eac << ")\n" << eor;
	Assert( eac != 0 );

	// Wait for everything to get hooked-up/initialized:
	volatile int notReady;
	while ( (notReady = eac->EAnotReady) == 0 )
		thisProcess()->relinquishProcessor();

	// Wait for messages and throw them back at sender:
	for(;;) {	// forever:
	    //Console().OldStyleDebug("\tEchoingApplicationConduitDaemon about to grab stuff out of the queue....\n");
	    msg = (TCPMessage *)eac->queue()->remove();


	    //Console().OldStyleDebug("\tEchoingApplicationConduitDaemon echoing msg @%x....\n", msg);
	    if( msg != 0) {
		//Console().OldStyleDebug("\tmsg's dataLength: %d\n", msg->dataLength());
		//msg->print();
		reply = new TCPMessage(  20,	// IP-header size
					   0,	// IP data size
					   20,	// TCP header size
					   msg->dataLength(),// TCP data size
					   0 );	// No payload
		if( reply->data() )
			octetcopy( reply->data(), msg->data(), msg->dataLength() );
		// Update TCP receive window:
		int16 amount = msg->dataLength();
		msg->del();
		TCPAddress sess = *(eac->session());

		eac->tcpConduit()->expandWindow( sess, amount );

		if( reply->data() && (*( reply->data() ) == 'Q') ) {
		    eac->tcpConduit()->insertFromAbove( reply, &sess );
		    Debug << "\tEACDaemon closing connection #" << eac->session()->handle() << "\n" << eor;
		    eac->tcpConduit()->close( *eac->session() );
		}
		else
		    eac->tcpConduit()->insertFromAbove( reply, eac->session() );
	    }
	    else Debug << "EAC: no message (lost race?)\n" << eor;
	}
}

#include "ClassInstaller.h"
DEFINECLASS("ApplicationConduit",ApplicationConduit,Conduit,0,0);
