#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Lee Lup Yuen (lup@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "ApplicationConduitStream.h"
#include "Kernel.h"
#include "Process.h"
#include "Conduit.h"
#include "CharacterInputStream.h"
#include "ApplicationConduitOutputStream.h"
#include "FileSystemInterface.h"
#include "ThisProcess.h"
#include "Assert.h"
#include "String.h"
#include "Byte.h"
#include "List.h"
#include "Semaphore.h"
#include "NullStreams.h"
#include "UserSession.h"


#ifdef __GNUC__
#pragma implementation
#endif

#undef Debug
#define Debug TheNullOutputStream
//#define Debug Console()

//  Max number of data bytes in an IP packet.
const int maxDataLength = 1024;

class OutgoingMessage : public Link
{
public:
    char *data;
    int count;
};

ApplicationConduitStream :: ApplicationConduitStream () : incoming (50)
{
    my_proto = EchoingApplication; // "He's Lying."
    myTCPConduit = 0;
    echoStream = new ApplicationConduitOutputStream (this);
    characterQueue = new CharacterInputStream (echoStream);
    quickBufferPos = quickBuffer;
    quickBufferCount = 0;
    quickBufferFull = 0;
    outgoing = new List;
    outgoingCharacterCount = 0;
    outgoingSemaphore = new Semaphore (1);
    closed = new Semaphore (0);
    name = "ApplicationConduitStream";
    daemon = 0;
    spyStream = 0;
    userSession = 0;
}

ApplicationConduitStream :: ~ApplicationConduitStream ()
{
    delete outgoing;
    //  Wait for this conduit to be closed via a FIN message to the daemon.
    closed -> P ();
//  Can't do this: If daemon is still running while we delete it, we
//  will thrash the whole system.
//    if (daemon != 0) daemon -> unreference ();
}

void ApplicationConduitStream :: setUserSession (UserSession *session)
{
    userSession = session;
}

UserSession *ApplicationConduitStream :: getUserSession ()
{
    return userSession;
}

Return_Code ApplicationConduitStream :: passiveStart (int port)
{
    //  Start a passive connection at the indicated port number.
    if (objectName () != 0);
    	name = objectName ();
    extern TCPConduit *TheTCPConduit;
    Return_Code ret = this -> connectTo (TheTCPConduit);
    Debug << "\tConnected new " << name << ": " << Return_Code_Table [-ret]
    	<< "\n" << eor;
    if (ret < 0) return ret;

    if (port == -1)
    	port = passivePort ();
    Debug << name << " opening passive conn on port " << port << "\n" << eor;
    TCPAddress sess = -1;

    short unsigned int tmp1 = port;

    ret = myTCPConduit -> open (this, tmp1, 4095, sess);
    if (ret < 0) return ret;
    setSession (sess);
    startDaemon (passiveDaemon ());
    return Return_Code(0);
}

Return_Code ApplicationConduitStream :: activeStart (long ipAddress,
    int localPort, int remotePort)
{
    //  Start an active connection at the indicated address and ports.
    if (objectName () != 0)
    	name = objectName ();
    extern TCPConduit *TheTCPConduit;
    Return_Code ret = this -> connectTo (TheTCPConduit);
    Debug << "\tConnected new " << name << ": " << Return_Code_Table [-ret]
    	<< "\n" << eor;
    if (ret < 0) return ret;

    TCPAddress sess = -1;
    short unsigned int tmp4 = ipAddress;
    IPAddress addr = (IPAddress)tmp4;

    Debug.formattedWrite ("Connecting %s to %lx from port %d to port %d\n",
    	name, ipAddress, localPort, remotePort);

    short unsigned int tmp1 = 4095;
    short unsigned int tmp2 = localPort;
    short unsigned int tmp3 = remotePort;

    ret = myTCPConduit -> open (this, 
				tmp2, 
				tmp3, 
				addr, 
				tmp1, 
				sess);

    Debug << "\t" << name << " connected to remote port: " <<
	Return_Code_Table [-ret] << "\n" << eor;
    if (ret < 0) return ret;
    setSession (sess);
    startDaemon (activeDaemon ());
    return Return_Code(0);
}

void ApplicationConduitStream :: startDaemon (PFV *conduitDaemon)
{
    //  Start daemon to send and receive network data.
    Assert (conduitDaemon != 0);
    daemon = new PreemptableSystemProcess(
	(PFV) conduitDaemon,	    	// Name of fn/process
	thisProcess() -> domain(),	// New process' domain
	huge,				// Stack size
	(int) this);			// 1st argument
    Assert (daemon != 0);
    char daemonName [1000];
    strcpy (daemonName, name);
    strcat (daemonName, " daemon");
    daemon -> setName (daemonName);
    daemon -> ready ();
}

void ApplicationConduitStream :: finish ()
{
    //  Send the daemon a FIN message so that it will close the connection
    //  and kill itself.
    TCPMessage *msg = new TCPMessage (20, 0, 20, 0, 0);
    msg -> setFIN (TRUE);
    insertFromBelow (msg);
    //  Gross hack - don't let the application delete me.  Closing messages
    //  may still be coming after I'm dead.
    reference ();
    if (myTCPConduit != 0) myTCPConduit -> abort (*session ());
}

void ApplicationConduitStream :: close ()
{
    if (myTCPConduit == 0) return;
    //  Close TCP connection.
    myTCPConduit -> close (*session ());
    
    //  Disconnect myself from the TCP conduit.
    Return_Code ret = disconnectFrom (myTCPConduit);
    myTCPConduit = 0;
    Debug << "\tDisconnected " << name << " from TCP conduit: " <<
    	Return_Code_Table [-ret] << "\n" << eor;
    setEndOfStream ();
    closed -> V ();
}

Return_Code ApplicationConduitStream :: insertFromBelow (ConduitMessage *msg)
{
    Return_Code ret = incoming.add (msg);
    thisProcess () -> relinquishProcessor ();
    return ret;
}

int ApplicationConduitStream :: read (char *buffer, int count)
{
    return characterQueue -> read (buffer, count);
}

Return_Code ApplicationConduitStream :: write (const char *buffer, int count)
{
    if (count == 0) return Return_Code(0);
    if (!quickBufferFull && quickBufferCount + count <= quickBufferSize)
    {
    	for (int i = 0; i <= count - 1; i++)
	    *(quickBufferPos++) = buffer [i];
	quickBufferCount = quickBufferCount + count;
	outgoingCharacterCount = outgoingCharacterCount + count;
    	if (outgoingCharacterCount < maxDataLength) return Return_Code(0);
	return flush ();
    }
    quickBufferFull = 1;
    OutgoingMessage *msg = new OutgoingMessage;
    msg -> data = new char [count];
    for (int i = 0; i <= count - 1; i++)
    	msg -> data [i] = buffer [i];
    msg -> count = count;
    
    outgoingSemaphore -> P ();
    outgoing -> addLast (msg);
    outgoingCharacterCount = outgoingCharacterCount + count;
    outgoingSemaphore -> V ();
    if (outgoingCharacterCount < maxDataLength) return Return_Code(0);
    return flush ();
}

Return_Code ApplicationConduitStream :: flush ()
{
    if (outgoingCharacterCount == 0) return Return_Code(0);
    
    outgoingSemaphore -> P ();
    char *data = new char [outgoingCharacterCount];
    int dataIndex = 0;
    for (int i = 0; i <= quickBufferCount - 1; i++)
    	data [dataIndex++] = quickBuffer [i];
    quickBufferPos = quickBuffer;
    quickBufferCount = 0;
    quickBufferFull = 0;
    OutgoingMessage *msg;
    while ((msg = (OutgoingMessage *) outgoing -> removeFirst ()) != 0)
    {
    	ByteCopy (msg -> data, data + dataIndex, msg -> count);
	dataIndex = dataIndex + msg -> count;
	delete msg -> data;
	delete msg;
    }
    Assert (dataIndex == outgoingCharacterCount);
    outgoingCharacterCount = 0;
    outgoingSemaphore -> V ();

    char *processedData = processOutgoingMessage (data, dataIndex);
    Return_Code ret = Return_Code(0);
    if (dataIndex != 0 && processedData != 0)
    	ret = rawWrite (processedData, dataIndex);
    if (processedData != 0) delete processedData;
    delete data;
    return ret;
}

Return_Code ApplicationConduitStream :: rawWrite (const char *buffer,
    int count)
{
    while (count > 0)
    {
    	int charsToSend = count;
	if (charsToSend > maxDataLength) charsToSend = maxDataLength;
    	Return_Code status = rawWriteBlock (buffer, charsToSend);
	if (spyStream != 0) spyStream -> rawWriteBlock (buffer, charsToSend);
	if (status < 0) return status;
	buffer = buffer + charsToSend;
	count = count - charsToSend;
    }
    return Return_Code(0);
}

Return_Code ApplicationConduitStream :: rawWriteBlock (const char *buffer,
    int count)
{
    TCPMessage *output = new TCPMessage (20, 0, 20, count, 0);
    if (count > 0)
    	octetcopy (output -> data (), (octet *) buffer, count);
    if (myTCPConduit == 0)
    {
    	Console () << "ApplicationConduitStream error: "
	    "Writing to an unconnected conduit\n" << eor;
	thisProcess () -> die ();
    }
    Return_Code ret = myTCPConduit -> insertFromAbove (output, session ());
    if (ret < 0)
    {
        output -> del ();
	TCPAddress *sess = session ();
	while (count > 0)
	{
	    Console () << "@" << eor;
	    for (int i = 1; i <= 100000; i++)
	    	thisProcess () -> relinquishProcessor ();
	    TCPMessage *output = new TCPMessage (20, 0, 20, count, 0);
	    octetcopy (output -> data (), (octet *) buffer, count);
	    ret = myTCPConduit -> insertFromAbove (output, sess);
	    if (ret >= 0) break;
	    output -> del ();
	}
    }
    return ret;
}

void ApplicationConduitStream :: setEndOfStream ()
{
    //  Indicate that characterQueue will not receive any more characters.
    characterQueue -> queue () -> terminate ();
}

void ApplicationConduitStream :: switchCharacterQueue (int type)
{
    characterQueue -> switchCharacterQueue (type);
}

char *ApplicationConduitStream :: processIncomingMessage (TCPMessage *msg,
    int &length)
{
    length = msg -> dataLength ();
    char *dataSrc = (char *) msg -> data ();
    char *dataDest = new char [length];
    for (int i = 0; i <= length - 1; i++)
    	dataDest [i] = dataSrc [i];
    return dataDest;
}

char *ApplicationConduitStream :: processOutgoingMessage (const char *buffer,
    int &count)
{
    char *dataDest = new char [count];
    for (int i = 0; i <= count - 1; i++)
	dataDest [i] = buffer [i];
    return dataDest;
}

void ApplicationConduitStream :: applicationConduitStreamDaemon
    (ApplicationConduitStream *client)
{
    Debug << "\t" << client -> name << " Daemon: Hello!\n" << eor;
    Assert (client != 0);
    TCPMessage *msg = 0;
    
    // Queue up all characters in received messages, watching out for the
    // finish condition.
    for (;;)
    {
    	msg = (TCPMessage *) client -> incoming.remove ();
    	int length;
    	char *characters = client -> processIncomingMessage (msg, length);
    	if (characters != 0 && length > 0)
	{
    	    int charsEnqueued = client -> characterQueue -> enqueue
	    	(characters, length);
	    char *remainder = characters + charsEnqueued;
	    while (charsEnqueued < length)
	    {
	    	thisProcess () -> relinquishProcessor ();
	    	Debug << "#" << eor;
		int remainderEnqueued = client -> characterQueue -> enqueue
		    (remainder, length - charsEnqueued);
	    	charsEnqueued = charsEnqueued + remainderEnqueued;
		remainder = remainder + remainderEnqueued;
	    }
	    Assert (charsEnqueued == length);
	}
	if (characters != 0) delete characters;
	client -> tcpConduit () -> expandWindow (*(client -> session ()),
	    msg -> dataLength ());
	if (msg -> FIN ())
	{
    	    msg -> del ();
	    break;
	}
    	msg -> del ();
    }
    client -> close ();
    thisProcess () -> die ();
}

#include "ClassInstaller.h"
DEFINECLASS("ApplicationConduitStream", ApplicationConduitStream,
    ApplicationConduit,0,0);
