#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990
 *   The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Johnny Zweig (zweig@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * CCCP.cc 
 * (Choices to Choices Copy Program)
 */
/*
 *	Right now, this is a hack to get crude file-transfer from
 *	UNIX-->Choices working soonest.
 */

#include "OutputStream.h"
#include "ConduitTypes.h"
#include "Class.h"
#include "Word.h"
#include "CPU.h"
#include "Process.h"
#include "ThisProcess.h"
#include "Semaphore.h"
#include "Interrupt.h"
#include "Kernel.h"
#include "RecordStream.h"
#include "RecordStreamStar.h"
#include "FileSystem.h"
#include "FileSystemInterface.h"
#include "FileSystemInterfaceStar.h"
#include "ZootConfig.h"
#include "ZootReturnCodes.h"
#include "Assert.h"
#include "ZootAddress.h"
#include "ConduitMessage.h"
#include "ZootClock.h"
#include "ConduitMessageBuffer.h"
#include "MessageFIFO.h"
#include "ZootStateMachine.h"
#include "Zoot_IP_Output.h"
#include "Conduit.h"
#include "CCCP.h"


#ifdef __GNUC__
#pragma implementation
#endif

#ifndef QUIET
#undef Debug
#define Debug Console()
#endif

int
AsciiToInteger( const char * s )
{
	int sum = 0;
	int i = 0;

	/* Skip whitespace: */
	while( s[i] == ' ' || s[i] == '\t' || s[i] == '\n' ) ++i;
	/* Figure out whether it's decimal/hex/octal: */
	if( s[i] == '0' ) {
	    if( s[i] == 'x' ) {
		/* Hex */
		i++; // Skip over the 'x'
		while( ( s[i] >= '0' && s[i] <= '9' ) ||
		       ( s[i] >= 'a' && s[i] <= 'f' ) ||
		       ( s[i] >= 'A' && s[i] <= 'F' ) ) {
		    if( s[i] >= '0' && s[i] <= '9' )
			sum = ( sum * 0x10 ) + ( s[i] - '0' );
		    if( s[i] >= 'a' && s[i] <= 'f' )
			sum = ( sum * 0x10 ) + ( s[i] - 'f' + 0x10 );
		    if( s[i] >= 'A' && s[i] <= 'F' )
			sum = ( sum * 0x10 ) + ( s[i] - 'F' + 0x10 );
		    i++;
		}
	    }
	    else {
		/* Octal -- yuck! */
		while( s[i] >= '0' && s[i] <= '7' ) {
		    sum = ( sum * 010 ) + ( s[i] - '0' );
		    i++;
		}
	    }
	}
	else {
	    /* It's decimal */
	    while( s[i] >= '0' && s[i] <= '9' ) {
		sum = ( sum * 10 ) + ( s[i] - '0' );
		i++;
	    }
	}
	return( sum );
}

/******
class CCCPConduit : public ApplicationConduit {
protected:
	static void cccpDaemon( CCCPConduit * client );
	SemaphoreRegulatedMessageFIFO _incoming;
public:
	CCCPConduit();
	~CCCPConduit();

	Return_Code	insertFromBelow( ConduitMessage * msg );
	SemaphoreRegulatedMessageFIFO *	queue();
	TCPConduit * tcpConduit();
};

inline SemaphoreRegulatedMessageFIFO  *
CCCPConduit::queue()
{
	return( & _incoming );
}
inline TCPConduit *
CCCPConduit::tcpConduit()
{
	return( myTCPConduit );
}
*******/

CCCPConduit::CCCPConduit() : _incoming( 50 )
{

	my_proto = EchoingApplication; // "He's Lying."

	/* Start demon to suck in data and write to disk: */
	_daemon = new PreemptableSystemProcess(
		(PFV) cccpDaemon,		// Name of fn/process
		thisProcess()->domain(),	// New process' domain
		small,				// Stack size
		(int)this );			// 1st argument

	Assert( _daemon != 0 );
	_daemon->setName( "#CCCP Receiver Daemon" );
	_daemon->ready();

}


CCCPConduit::~CCCPConduit()
{
	_daemon->die();
}
Return_Code
CCCPConduit::insertFromBelow( ConduitMessage * msg )
{
	return( _incoming.add( msg ) );
}

volatile int CCCP_NAME;	// used to get the last digit of the filename

void
CCCPConduit::cccpDaemon( CCCPConduit * client )
{//JZ
	Return_Code ret;
	TCPAddress sess = -1;
	TCPMessage * msg   = 0;
	TCPMessage * reply = 0;
	char filename[11];
	octet * p;

	CCCP_NAME = 0;
	Debug << "\tCCCP receiver Daemon: Hello!  (at:"<< client
		    << ")\n" << eor;
	Assert( client != 0 );

	// Wait for client to get connected to TCP if necessary:
	volatile Conduit * notReady;
	while( (notReady = client->tcpConduit()) == 0 )
	    thisProcess()->relinquishProcessor();


	// Wait for messages and throw them back at sender:
	for(;;) {	// forever:
	    Debug << "Opening passive conn. on port 1826.\n" << eor;
	    short unsigned int tmp1 = 1826;
	    short unsigned int tmp2 = 4095;

	    client->tcpConduit()->open( client, tmp1, tmp2, sess );
	    client->setSession( sess );

	    // WAIT FOR SOMEONE TO CALL UP:
	    msg = (TCPMessage *)client->queue()->remove();
	    //Debug << "msg:" << asAddress( msg ) << eor;

	    // CREATE FILE:
	    filename[0] = '/';
	    filename[1] = 't';
	    filename[2] = 'm';
	    filename[3] = 'p';
	    filename[4] = '/';
	    filename[5] = 'C';
	    filename[6] = 'C';
	    filename[7] = 'C';
	    filename[8] = 'P';
	    filename[9] = '0' + CCCP_NAME++;	// /tmp/CCCP0, CCCP1 and so on.
	    filename[10] = 0;

	    int status = 0;

	    FileSystemInterfaceStar fsi = TheKernel->fileSystemInterface();
	    RecordStreamStar wfs = fsi->open( (char *)filename,
			RecordStreamClass, status, 0, 0666 );
	    Debug << "fsi->open:" << status << " = "
			<< StatusCodes[-status] << "\n" << eor;

	    if( wfs == 0 ) { //ERROR! Cannot write to file!
		    Debug << "ERROR! Cannot write to file!\n" << eor;
		    // ABORT CONNECTION AND LOOP:
		    reply = new TCPMessage(  20, 0, 20, 17, 0 );
		    octetcopy( reply->data(),
			       (octet*)"[CCCP: ERROR!]\r\n", 17 );
		    ret = client->tcpConduit()->
			    	     insertFromAbove( reply, &sess );
		    Debug << "\tinsertFromAbove() ret=" <<
			 Return_Code_Table[-ret] << "\n";
		    client->tcpConduit()->close( *client->session() );
		    continue;
	    }
	    else {
		    // WRITE the sucker:
		    // TELL CLIENT TO GO AHEAD:
		    reply = new TCPMessage(  20,	// IP-header size
					     0,		// IP data size
					     20,	// TCP header size
					     5,   	// TCP data size
					     0 );	// No payload
		    octetcopy(	reply->data(),
				(octet*)"OK\r\n", 5 );
		    ret = client->tcpConduit()->insertFromAbove( reply, &sess );
		    Debug << "\tinsertFromAbove() ret=" <<
			 Return_Code_Table[-ret] << "\n" << eor;

		    // LOOP GRABBING FILE-DATA
		    while( msg != 0 ) {
		        client->tcpConduit()->
			    expandWindow( sess, msg->dataLength() );
		        p = msg->data();

		        //Debug << "File data (" << asAddress(p)
			//    << ") follows.\n\t";
		        if( msg->dataLength() == 0 ) {
			    //Debug << "(none)" << eor;
		        }
		        else {
			    //for( i = 0; i < msg->dataLength(); i++ )
			    //    Debug << p[i];
			    wfs->write( (char*)p, msg->dataLength() );
			}
		        //Debug << "\n" << eor;

			if( msg->FIN() ) {
		            Debug << "CLOSE\n" << eor;
		    	     reply = new TCPMessage(  20, 0, 20, 19, 0 );
		    	     octetcopy( reply->data(),
				        (octet*)"[CCCP: GOODBYE.]\r\n", 19 );
		    	     ret = client->tcpConduit()->
			    	     insertFromAbove( reply, &sess );
		    	     Debug<<"insertFromAbove() ret="<<
			 	    Return_Code_Table[-ret] << "\n" << eor;
				// Notice that, since the other end already
				// closed the goddamned connection, he will
				// in all likelihood never see this (it will
				// get delivered, but probably ignored)....
			    // CLOSE CONNECTION:
			    client->tcpConduit()->close( *client->session() );
			    // CLOSE FILE:
			    msg->del();
			    msg = 0; // Cheesy way to break out of loop....
			}
			else{
			    msg->del(); // HOCHWICHTIG!
			    msg = (TCPMessage *)client->queue()->remove();
			    //Debug << "msg:" << asAddress( msg ) << eor;
			}
		    } // END OF DATA-GRABBING LOOP
		} // end of check for file-hipness
	    } // END OF INFINITE LOOP
}
