#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990
 *   The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Johnny Zweig (zweig@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * Elvis.cc 
 * (Choices Elvis Protocol Server)
 */

#include "OutputStream.h"
#include "ConduitTypes.h"
#include "Class.h"
#include "Word.h"
#include "CPU.h"
#include "Process.h"
#include "ThisProcess.h"
#include "Semaphore.h"
#include "Interrupt.h"
#include "Kernel.h"
#include "RecordStream.h"
#include "RecordStreamStar.h"
#include "FileSystem.h"
#include "FileSystemInterface.h"
#include "FileSystemInterfaceStar.h"
#include "ZootConfig.h"
#include "ZootReturnCodes.h"
#include "Assert.h"
#include "ZootAddress.h"
#include "ConduitMessage.h"
#include "ZootClock.h"
#include "ConduitMessageBuffer.h"
#include "MessageFIFO.h"
#include "ZootStateMachine.h"
#include "Zoot_IP_Output.h"
#include "Conduit.h"
#include "ElvisConduit.h"


#ifdef __GNUC__
#pragma implementation
#endif

#ifndef QUIET
#undef Debug
#define Debug Console()
#endif


/******  For reference:
class ElvisConduit : public ApplicationConduit {
protected:
	static void cccpDaemon( ElvisConduit * client );
	SemaphoreRegulatedMessageFIFO _incoming;
	Process * _daemon;
public:
	ElvisConduit();
	~ElvisConduit();

	Return_Code	insertFromBelow( ConduitMessage * msg );
	SemaphoreRegulatedMessageFIFO *	queue();
	TCPConduit * tcpConduit();
};

inline SemaphoreRegulatedMessageFIFO  *
ElvisConduit::queue()
{
	return( & _incoming );
}
inline TCPConduit *
ElvisConduit::tcpConduit()
{
	return( myTCPConduit );
}
*******/

ElvisConduit::ElvisConduit() : _incoming( 50 )
{

	my_proto = EchoingApplication; // "He's Lying."

	/* Start demon to suck in data and read to disk: */
	_daemon = new PreemptableSystemProcess(
		(PFV) ElvisDaemon,		// Name of fn/process
		thisProcess()->domain(),	// New process' domain
		small,				// Stack size
		(int)this );			// 1st argument

	Assert( _daemon != 0 );
	_daemon->setName( "#Elvis Transmitter Daemon" );
	_daemon->ready();

}


ElvisConduit::~ElvisConduit()
{
	_daemon->die();
}
Return_Code
ElvisConduit::insertFromBelow( ConduitMessage * msg )
{
	return( _incoming.add( msg ) );
}

void
ElvisConduit::ElvisDaemon( ElvisConduit * client )
{//JZ
	Return_Code ret;
	TCPAddress sess = -1;
	TCPMessage * msg   = 0;
	TCPMessage * reply = 0;
	int i, count = 0;
	char filename[11];
	char buffer[1024];

	Debug << "\tElvis receiver Daemon: Hello!  (at:"<< client
		    << ")\n" << eor;
	Assert( client != 0 );

	// Wait for client to get connected to TCP if necessary:
	while( client->tcpConduit() == 0 )
	    thisProcess()->relinquishProcessor();


	// Wait for messages and throw them back at sender:
	for(;;) {	// forever:
	    Debug << ">>>>> Opening passive conn. on port 1842.\n" << eor;

	    short unsigned int tmp1 = 1842;
	    short unsigned int tmp2 = 4095;

	    ret = client->tcpConduit()->open( client, tmp1, tmp2, sess );
	    client->setSession( sess );
	    Debug << "\t(return_code: " << Return_Code_Table[-ret] <<
		" handle: " << sess.handle() << " )\n" << eor;

	    // WAIT FOR SOMEONE TO CALL UP:
	    msg = (TCPMessage *)client->queue()->remove();
	    Debug << "msg:" << asAddress( msg ) << eor;

	    // OPEN FILE:
	    octetcopy( (octet*)filename, (const octet *)"/Elvis", 7);

	    int status = 0;

	    FileSystemInterfaceStar fsi = TheKernel->fileSystemInterface();
	    RecordStreamStar rfs = fsi->open( (char *)filename,
			RecordStreamClass, status, 0, 0666 );
	    Debug << "fsi->open:" << status << " = "
			<< StatusCodes[-status] << "\n" << eor;

	    if( rfs == 0 ) { //ERROR! Cannot read from file!
		    Debug << "ERROR! Cannot read from file!\n" << eor;
		    // ABORT CONNECTION AND LOOP:
		    reply = new TCPMessage(  20, 0, 20, 64, 0 );
		    octetcopy( reply->data(),
			(octet*)"[ELVIS: ERROR! There is no Elvis in this system anywhere...]\r\n", 64 );
		    ret = client->tcpConduit()->
			    	     insertFromAbove( reply, &sess );
		    Debug << "\tinsertFromAbove() ret = " <<
			 Return_Code_Table[-ret] << "\n*** NO ELVIS!!! ***\n";
		    client->tcpConduit()->close( *client->session() );
		    continue;
	    }
	    else {
		    // READ from /etc/Elvis into this socket:
		    // TELL CLIENT TO GO AHEAD:
		    reply = new TCPMessage(  20,	// IP-header size
					     0,		// IP data size
					     20,	// TCP header size
					     17,   	// TCP data size
					     0 );	// No payload
		    octetcopy(	reply->data(),
				(octet*)"ELVIS IS KING!\r\n", 17 );
		    ret = client->tcpConduit()->insertFromAbove( reply, &sess );
		    Debug << "\tinsertFromAbove() ret = " <<
			 Return_Code_Table[-ret] << "\n" << eor;

		    if( ret != OK ) {
			client->tcpConduit()->close( *client->session() );
			Debug << "\n***** GACK! *****\n" << eor;
			continue;
		    }

		    // LOOP GRABBING FILE-DATA

		    count = rfs->read( &buffer[0], 1024 );
			// count will be 0 when there is no more stuff to read

		    while( count != 0 ) {
		        client->tcpConduit()->
			    expandWindow( sess, msg->dataLength() );

			for( i = 0; i < count; i++ )
			        Debug << buffer[i];
		        Debug << "\n$$$$$" << eor;

			reply = new TCPMessage(  20, 0, 20, count, 0 );
			octetcopy( reply->data(), (octet*) &buffer[0], count );
			ret = client->tcpConduit()->
				insertFromAbove( reply, &sess );
			Debug<<"insertFromAbove() ret = " <<
				Return_Code_Table[-ret] << "\n" << eor;

			count = rfs->read( buffer, 1024 );

		    } // END OF DATA-GRABBING LOOP

		    reply = new TCPMessage(  20, 0, 20, 20, 0 );
		    octetcopy( reply->data(),
			        (octet*)"[ELVIS: GOODBYE.]\r\n", 20 );
		    ret = client->tcpConduit()->
		    	     insertFromAbove( reply, &sess );
		    Debug<<"insertFromAbove() ret = " <<
			    Return_Code_Table[-ret] << "\n" << eor;

		    // WAIT UNTIL OTHER END HANGS UP:
			// (unnecessary, no? -JZ)
		    msg->del();	// toss old message
		    msg = (TCPMessage *)client->queue()->remove();
		    while( msg->FIN() != TRUE ) {
			msg->del();
			msg = (TCPMessage *)client->queue()->remove();
		    }
		    Debug << "\nCLOSE\n" << eor;
		    // CLOSE CONNECTION:
		    client->tcpConduit()->close( *client->session() );
		    // CLOSE FILE:

		} // end of check for file-hipness
	    } // END OF INFINITE LOOP
}
