#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Lee Lup Yuen (lup@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "FTP.h"
#include "Kernel.h"
#include "Process.h"
#include "ThisProcess.h"
#include "Conduit.h"
#include "CharacterInputStream.h"
#include "FileSystemInterface.h"
#include "FileSystemInterfaceStar.h"
#include "PersistentMemoryObject.h"
#include "PersistentMemoryObjectStar.h"
#include "Semaphore.h"
#include "SystemInterface.h"
#include "Assert.h"
#include "String.h"
#include "Byte.h"
#include "NullStreams.h"
#include "ApplicationConduitInputStream.h"
#include "ApplicationConduitOutputStream.h"
#include "UserSession.h"
#include "UserSessionInfo.h"
#include "NameServer.h"


#ifdef __GNUC__
#pragma implementation
#endif

FTP :: FTP ()
{
    setName ("FTP Server");
}

FTP :: ~FTP ()
{
}

void FTP :: FTPDaemon (FTP *client)
{
    Assert( client != 0 );
    Return_Code ret;
    TCPMessage *msg = 0;
    
    // Wait for someone to call up.
    msg = (TCPMessage *) client -> incoming.remove ();
    client -> clientAddress = (unsigned long) msg -> SourceAddress ().val ().
    	val ();
    client -> clientPort = (unsigned int ) msg -> SourcePort ().val ();
    
    unsigned char clientAddress [4];
    msg -> SourceAddress ().val ().copyTo (clientAddress);
    Console () << "[Incoming FTP connection from " <<
    	(int) clientAddress [0] << "." << (int) clientAddress [1] << "." <<
    	(int) clientAddress [2] << "." << (int) clientAddress [3] <<
	" established]\n" << eor;

    // Make another conduit to handle the next connection.
    FTP *shell = new FTP;
    ret = shell -> passiveStart ();

#ifdef NOTUSED
    // Make sure that client is an authorized host.
    if (clientAddress [0] != 128 || clientAddress [1] != 174 ||
    	clientAddress [2] != 240)  //  Tapestry net
#else
    if (0)
#endif
    {
    	const char badClient [] = "Unauthorized host.  "
	    "Contact choices@cs.uiuc.edu for assistance.\r\n";
    	client -> rawWrite (badClient, strlen (badClient));
    	client -> close ();
    	Console () << "[UNAUTHORIZED FTP connection from " <<
    	    (int) clientAddress [0] << "." << (int) clientAddress [1] << "." <<
	    (int) clientAddress [2] << "." << (int) clientAddress [3] <<
	    " closed]\n" << eor;
    	thisProcess () -> die ();
    }

    // Make a session for this connection.
    client -> setUserSession (new UserSession (UserSessionInfo::FTPSession,
    	client, clientAddress));
	
    // Startup the program for this connection, and set up stdin and stdout.
    PreemptableSystemProcess *commandProcess = new PreemptableSystemProcess
    	((PFV) executeCommand, thisProcess () -> domain(), huge, (int) client);
    Assert (commandProcess != 0);
    commandProcess -> setName ("#FTP Command Process");
    commandProcess -> ready ();

    // Queue up all messages read, watching out for the finish condition.
    for (;;)
    {
    	int length;
    	char *characters = client -> processIncomingMessage (msg, length);
    	if (characters != 0 && length > 0)
    	    client -> characterQueue -> enqueue (characters, length);
	if (characters != 0) delete characters;
	client -> tcpConduit () -> expandWindow (*(client -> session ()),
	    msg -> dataLength ());
	if (msg -> FIN ())
	{
    	    msg -> del ();
	    break;
	}
    	msg -> del ();
	msg = (TCPMessage *) client -> incoming.remove ();
    }
    client -> close ();
    UserSession *sess = client -> getUserSession ();
    if (sess != 0) delete sess;
    Console () << "[FTP connection from " <<
    	(int) clientAddress [0] << "." << (int) clientAddress [1] << "." <<
    	(int) clientAddress [2] << "." << (int) clientAddress [3] <<
	" closed]\n" << eor;
    thisProcess () -> die ();
}

void FTP :: executeCommand (FTP *client)
{
    int err;
    FileSystemInterfaceStar kernelFSI = StandardNameServer -> lookup
    	("fileSystemInterface", "FileSystemInterface");
    Assert (kernelFSI != 0);
    PersistentMemoryObjectStar pmo = kernelFSI -> open ("/Etc/ftpd",
    	PersistentMemoryObjectClass, err);
    if (err || pmo == 0)
    {
    	const char msg [] = "Can't open /Etc/ftpd.  "
	    "Contact choices@cs.uiuc.edu for assistance.\r\n";
    	client -> rawWrite (msg, strlen (msg));
	client -> close ();
    	Console () << "FTP: Can't open /Etc/ftpd\n" << eor;
	thisProcess () -> die ();
    }
    
    char clientAddressStr [16];
    strcpy (clientAddressStr, "000.000.000.000");
    char *addr = clientAddressStr + 12;
    int i;
    for (i = 1; i <= 4; i++)
    {
    	int num = (client -> clientAddress) & 255;
	client -> clientAddress = (client -> clientAddress) >> 8;
    	*(addr + 2) = (num % 10) + '0';  num = num / 10;
    	*(addr + 1) = (num % 10) + '0';  num = num / 10;
    	*addr = (num % 10) + '0';
    	addr = addr - 4;
    }
    char clientPortStr [5];
    addr = clientPortStr + 3;
    for (i = 1; i <= 4; i++)
    {
    	*addr = ((client -> clientPort) % 10) + '0';
	client -> clientPort = (client -> clientPort) / 10;
    	addr--;	
    }
    clientPortStr [4] = 0;
    
    const int argc = 3;
    char *argv [argc + 1];
    argv [0] = "/Etc/ftpd";
    argv [1] = clientAddressStr;
    argv [2] = clientPortStr;
    argv [3] = 0;
    {
    	InputStreamStar iss_stdin = new ApplicationConduitInputStream (client);
    	OutputStreamStar oss_stdout = new ApplicationConduitOutputStream (client);
    	FileSystemInterfaceStar fileSystemInterface = new FileSystemInterface
	    (kernelFSI, 0);
	Process *proc = StandardSystemInterface -> buildProcessFromObjectFile
	    (pmo, argc, argv);
	Assert (proc != 0);
	proc -> nameServer () -> bind ("StandardInput", iss_stdin);
	proc -> nameServer () -> bind ("StandardOutput", oss_stdout);
	proc -> nameServer () -> bind ("fileSystemInterface",
	    fileSystemInterface);
	proc -> ready ();
	thisProcess () -> waitForProcess (proc);
    }
    pmo = 0;
    client -> flush ();
    client -> finish ();
    thisProcess () -> die ();
}

#include "ClassInstaller.h"
DEFINECLASS("FTP",FTP,ApplicationConduitStream,0,0);
