#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Lee Lup Yuen (lup@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "OutputStreamStar.h"
#include "UserSessionManager.h"
#include "UserSession.h"
#include "UserSessionInfo.h"
#include "String.h"
#include "Process.h"
#include "ApplicationConduitOutputStream.h"
#ifdef NOTUSED
#include "RealTimeClock.h"
#endif
#include "ThisProcess.h"
#include "NameServer.h"


#ifdef __GNUC__
#pragma implementation
#endif

UserSessionManager :: UserSessionManager ()
{
    firstUserSession = 0;
    nextSessionID = 1000;
#ifdef NOTUSED
    TimeOfDay timer;
    creationTime = timer.getRealTime ();
#else
    creationTime = 0;
#endif
}

UserSessionManager :: ~UserSessionManager ()
{
}

void UserSessionManager :: add (UserSession *userSession)
{
    userSession -> sessionID = nextSessionID++;
    userSession -> status = 0;
    userSession -> nextUserSession = firstUserSession;
    firstUserSession = userSession;
}

void UserSessionManager :: remove (UserSession *userSession)
{
    UserSession *prev = 0, *sess = firstUserSession;
    while (sess != 0)
    {
    	if (sess -> sessionID == userSession -> sessionID)
	{
	    if (prev == 0) firstUserSession = userSession -> nextUserSession;
	    else prev -> nextUserSession = userSession -> nextUserSession;
	    return;
	}
    	prev = sess;
    	sess = sess -> nextUserSession;
    }
    Assert (0);
}

unsigned long UserSessionManager :: getCreationTime ()
{
    return creationTime;
}

int UserSessionManager :: getMySession ()
{
    UserSession *sess = firstUserSession;
    OutputStreamStar os = thisProcess () -> nameServer () -> lookup
    	("StandardOutput", "ApplicationConduitOutputStream");
    if (os == 0) return 0;
		OutputStream *x = (OutputStream *) os;
    ApplicationConduitStream *myStream = ((ApplicationConduitOutputStream *) x) -> getConduitStream ();
    while (sess != 0)
    {
    	if (myStream == sess -> getSessionStream ())
	    return sess -> sessionID;
    	sess = sess -> nextUserSession;
    }
    return 0;
}

int UserSessionManager :: getSessions (int *sessionID, int count)
{
    int sessions = 0;
    UserSession *sess = firstUserSession;
    while (sess != 0)
    {
    	if (sessions >= count) break;
    	sessionID [sessions] = sess -> sessionID;
	sessions++;
    	sess = sess -> nextUserSession;
    }
    return sessions;
}

int UserSessionManager :: getSessionInfo (int sessionID, UserSessionInfo
    *info)
{
    UserSession *sess = firstUserSession;
    while (sess != 0)
    {
    	if (sessionID == sess -> sessionID)
	{
	    char *name = sess -> getUserName ();
	    if (name != 0) strncpy (info -> userName, name, maxUserNameLength);
	    else info -> userName [0] = 0;
	    info -> creationTime = sess -> getCreationTime ();
	    unsigned char *addr = sess -> getRemoteAddress ();
	    for (int i = 0; i <= 3; i++)
	    	info -> remoteAddress [i] = addr [i];
	    info -> status = sess -> status;
	    info -> sessionType = sess -> getSessionType ();
    	    return 0;
	}
	sess = sess -> nextUserSession;
    }
    return -1;
}

void UserSessionManager :: writeSession (int sessionID, const char *buffer,
    int count)
{
    UserSession *sess = findUserSession (sessionID);
    if (sess == 0) return;
    sess -> write (buffer, count);
}

void UserSessionManager :: spyOnSession (int sessionID)
{
    UserSession *sess = findUserSession (sessionID);
    UserSession *mySess = findUserSession (getMySession ());
    if (sess == 0) return;
    sess -> spyOn (mySess -> getSessionStream ());
}

void UserSessionManager :: spyOffSession (int sessionID)
{
    UserSession *sess = findUserSession (sessionID);
    if (sess == 0) return;
    sess -> spyOff ();
}

void UserSessionManager :: setSessionUserName (const char *name)
{
    UserSession *sess = findUserSession (getMySession ());
    if (sess == 0) return;
    if (sess -> getUserName () != 0) return;
    sess -> setUserName (name);
}

UserSession *UserSessionManager :: findUserSession (int sessionID)
{
    UserSession *sess = firstUserSession;
    while (sess != 0)
    {
    	if (sess -> sessionID == sessionID) return sess;
    	sess = sess -> nextUserSession;
    }
    return 0;
}

#include "ClassInstaller.h"
DEFINECLASS("UserSessionManager", UserSessionManager, ProxiableObject, 0, 0);
