#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Johnny Zweig (zweig@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * TCPClose_WaitState.cc
 *
 */


#include "Semaphore.h"
#include "Assert.h"
#include "ThisProcess.h"
#include "OutputStream.h"
#include "ConduitTypes.h"
#include "Word.h"
#include "ZootConfig.h"
#include "ZootReturnCodes.h"
#include "Assert.h"
#include "ZootAddress.h"
#include "ConduitMessage.h"
#include "ZootClock.h"
#include "Conduit.h"
#include "ConduitMessageBuffer.h"
#include "ZootStateMachine.h"


#ifdef __GNUC__
#pragma implementation
#endif

#ifndef QUIET
#undef Debug
#define Debug thisProcess()->console()
#endif

//-----------------------------------------------------------------------------
//				 Close_WaitState
//-----------------------------------------------------------------------------
Close_WaitState::Close_WaitState()
{
}
Close_WaitState::~Close_WaitState()
{
}
TCPState *
Close_WaitState::closeConnection( Return_Code & ret,
				TCPConnectionDescriptor & cd,
				TCPConduit * carrier )
{
	ret = sendCloseRequest( cd, carrier );
	cd.gotoState( LAST_ACK ); // Should this be CLOSING or LAST_ACK??
	Debug <<"C_WS::cC entering LAST_ACK state.\n" << eor;
		// I am assuming that there is an error on page 61 of
		// RFC793 stating that we should go to the CLOSING
		// state at this point.  There is no need to go into
		// CLOSING (and subsequently to TIME_WAIT) since the other
		// end will get an acknowledgement of his FIN-segment in
		// the FIN-segment we just sent -- acking our close-request
		// implicitly means he knows his close-request got through. -JZ

	return( LAST_ACK );
}
TCPState *
Close_WaitState::abortConnection( Return_Code & ret,
				TCPConnectionDescriptor & cd,
				TCPConduit * carrier )
{
	// I assume the source-addr and dest-addr have already been
	//   set up, since we are in the FIN-WAIT-1 state.

	ret = sendAbort( cd, carrier );

	short unsigned int tmp1 = 0;

	cd.setSourcePort( tmp1 );
	cd.setDestinationPort( tmp1 );
	cd.setDestinationIPAddress( IPAddress() ); // i.e. 0.0.0.0
	cd.gotoState( CLOSED );
	Debug <<"C_WS::aC entering CLOSED state.\n" << eor;
	return( CLOSED );
}
TCPState *
Close_WaitState::processIncomingMessage( Return_Code & ret,
				TCPConnectionDescriptor & cd,
				TCPConduit * carrier, TCPMessage * msg )
{
	MessageID id1, id2;

	Assert( carrier != 0 );
	Assert( msg != 0 );

	// Check whether the segment is in the receive window:
	ret = checkIncomingMessage( cd, carrier, msg );
	if( ret == OK ) {
	    if( msg->RST() ) {
		// Connection has been aborted. So blast outstanding msgs, and
		//   enter CLOSED state:
		// Delete outstanding messages:
		id1 = cd.outstanding();
		while( id1 != -1 ) {
		    id2 = cd.outgoingBuffer()->previous( id1 );
		    cd.outgoingBuffer()->del( id1 );
		    id1 = id2;
		}
		msg->del();
		cd.gotoState( CLOSED );
		Debug <<"C_WS::pIM entering CLOSED state!\n" << eor;
		ret = CONNECTION_RESET;
		return( CLOSED );
	    }
	    // Punt security/precedence....
	    if( msg->SYN() ) {
		// Error! Connection reset! (A SYN in the window is bad news.)
		sendAbort( cd, carrier );
		// Delete outstanding messages:
		id1 = cd.outstanding();
		while( id1 != -1 ) {
		    id2 = cd.outgoingBuffer()->previous( id1 );
		    cd.outgoingBuffer()->del( id1 );
		    id1 = id2;
		}
		// Note that we have just deleted the abort-request from
		//   the ARB, so it only will get sent once. In the (hopefully
		//   unlikely) event that it gets lost, the other end will
		//   time out and get bad news about trying to talk on a non-
		//   existent connection....

		msg->del();

		short unsigned int tmp1 = 0;

		cd.setSourcePort( tmp1 );
		cd.setDestinationPort( tmp1 );
		cd.setDestinationIPAddress( IPAddress() );	// i.e. 0.0.0.0
		cd.gotoState( CLOSED );
		Debug <<"C_WS::pIM entering CLOSED state...\n" << eor;
		ret = CONNECTION_RESET;
		return( CLOSED );
	    }
	    if( msg->ACK() ) {
		// Want to acknowledge regardless of whether it's the same
		// sequence number or not (there might be empty packets that
		// need to be zapped from the rexmit-buffer):
		acknowledge( cd, carrier, msg->AckNumber() );

		// Do Send Window stuff (see TCPEstablishedState.cc for expln):
		if( isBetween( cd.SendWL2().val(),
			       msg->AckNumber().val(),
			       cd.SendNext().val() )     &&
		    isBetween( cd.SendWL1().val(),
	    		       msg->SequenceNumber().val(),
			       cd.ReceiveNext().val() +
		       		cd.ReceiveWindow().val() ) ) {
		    // Update send window:
		    cd.setSendWL1( msg->SequenceNumber() );
		    cd.setSendWL2( msg->AckNumber() );
		    // If it's a duplicate of the last segment used to update
		    //   the window, only allow increases in window size,
		    //   since identical-looking ACK segments may have been
		    //   reordered:
		    if( cd.SendWL1().val() == msg->SequenceNumber().val()   &&
			cd.SendWL2().val() == msg->AckNumber().val()        &&
			cd.SendWindow().val() < msg->Window().val() )  {
	    		cd.setSendWindow( msg->Window() );
		    }
		    // Otherwise, update the window:
		    if( cd.SendWL1().val() != msg->SequenceNumber().val()   ||
			cd.SendWL2().val() != msg->AckNumber().val() )  {
	    		cd.setSendWindow( msg->Window() );
		    }
		}

		if( cd.SendUnacknowledged().val() != msg->AckNumber().val() ) {
		    // Update acknowledgements received:
		    cd.setSendUnacknowledged( msg->AckNumber() );
		}

	    	// Punt URG flag and urgent/push nonsense....

		// There cannot be more data following FIN-segment. If there
		//   is, it is an error which we will silently ignore.

		// No need to check FIN bit, since already in Close-Wait state
	    } // end of ack-check
	}
	// Nuke this puppy! (Since it cannot be handed to anyone else)
	msg->del();
	return( CLOSE_WAIT );	// No change.
}
