#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Johnny Zweig (zweig@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * TCPClosedState.cc
 *
 */


#include "Semaphore.h"
#include "Assert.h"
#include "ThisProcess.h"
#include "OutputStream.h"
#include "ConduitTypes.h"
#include "Word.h"
#include "ZootConfig.h"
#include "ZootReturnCodes.h"
#include "Assert.h"
#include "ZootAddress.h"
#include "ConduitMessage.h"
#include "ZootClock.h"
#include "Conduit.h"
#include "ConduitMessageBuffer.h"
#include "ZootStateMachine.h"


#ifdef __GNUC__
#pragma implementation
#endif

#ifndef QUIET
#undef Debug
#define Debug thisProcess()->console()
#endif

//-----------------------------------------------------------------------------
//				   ClosedState
//-----------------------------------------------------------------------------
ClosedState::ClosedState()
{
}
ClosedState::~ClosedState()
{
}
TCPState *
ClosedState::openConnection( Return_Code & ret, TCPConnectionDescriptor & cd,
				TCPConduit * carrier )
{
	// First, need to determine whether it's passive-promiscuous/passive-
	//   restricted/active by checking which parts of the socket are
	//   zero (recall that anIPaddr.val() returns a Word32, not an int32):

	if( cd.sourcePort().val() == 0 ) {
	    ret  =  ERROR_IN_SPECIFICATION;	// Port 0 illegal as source
	    return( CLOSED );
	}

	if( cd.destinationIPAddress().val().val() == 0 ) {
	    // ---- PASSIVE OPEN: ----
	    // Not much to do for passive open-requests; the tables are mostly
	    //   set up already:
	    if( cd.destinationPort().val() == 0 ) {
		// ---- Promiscuous ----
		cd.gotoState( LISTEN );
		Debug <<"CdS::oC entering LISTEN state.\n" << eor;
		ret  =  OK;
		Debug <<"ClosedState: Passive open....\n" << eor;
		Debug <<"\tCD.DestIPAddr: " <<
			cd.destinationIPAddress().val().val() ;
		Debug <<", SourcePort: " << cd.sourcePort().val() ;
		Debug <<", DestPort: " << cd.destinationPort().val() <<
			"\n" << eor;
		return( LISTEN );
	    }
	    else {
		// ---- Restricted ----
		cd.gotoState( LISTEN );
		Debug <<"CdS::oC entering LISTEN state...\n" << eor;
		ret  =  OK;
		return( LISTEN );
	    }
	}

	// ---- ACTIVE OPEN: ----
	Debug <<"ClosedState: Active open....\n" << eor;
	Debug <<"\tCD.DestIPAddr: " <<
		cd.destinationIPAddress().val().val() ;
	Debug <<", SourcePort: " << cd.sourcePort().val() ;
	Debug <<", DestPort: " << cd.destinationPort().val() <<
		"\n" << eor;

	// Just send off a connection-request datagram ("SYN segment"):
	ret = sendConnectRequest( cd, carrier );
	// (Note the slick way ret will return whatever the ARB said,
	//   indirectly through sendConnectRequest()....)
	
	cd.gotoState( SYN_SENT );
	Debug <<"CdS::oC entering SYN_SENT state..\n" << eor;
	return( SYN_SENT );
}
TCPState *
ClosedState::processIncomingMessage( Return_Code & ret,
				TCPConnectionDescriptor & cd,
				TCPConduit * carrier, TCPMessage * msg )
{
	if( msg->RST() ) {
	    // Do nothing -- this is a redundant request to close an
	    //   already closed connection.
	    ret = OK; 
	}
	else {
	    // Following RFC 793, p.65:
	    // Send an abort-request the other end will understand:
	    if( msg->ACK() ) {
		ret = sendAbort( cd, carrier, msg->AckNumber() );
	    }
	    else {
		// Set acknowledgement # to be the msg's sequence # plus
		//   its data-length so the other end will accept this
		//   reset-request. Sequence number can be left at 0.
	      short unsigned int tmp1 = msg->SequenceNumber().val() + msg->dataLength();
	      short unsigned int tmp2 = 0;

		ret = sendAbort( cd, carrier, tmp2, tmp1);
	    }
	}
	msg->del();	// If we don't, who will?
	return( CLOSED );
}



