#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Johnny Zweig (zweig@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * TCPClosingState.cc
 *
 */


#include "Semaphore.h"
#include "Assert.h"
#include "OutputStream.h"
#include "ConduitTypes.h"
#include "Word.h"
#include "ZootConfig.h"
#include "ZootReturnCodes.h"
#include "Assert.h"
#include "ZootAddress.h"
#include "ConduitMessage.h"
#include "ZootClock.h"
#include "Conduit.h"
#include "ConduitMessageBuffer.h"
#include "ZootStateMachine.h"


#ifdef __GNUC__
#pragma implementation
#endif

#ifndef QUIET
#undef Debug
#define Debug Console()
#endif

//-----------------------------------------------------------------------------
//				   ClosingState
//-----------------------------------------------------------------------------
ClosingState::ClosingState()
{
}
ClosingState::~ClosingState()
{
}
TCPState *
ClosingState::abortConnection( Return_Code & ret,
				TCPConnectionDescriptor & cd,
				TCPConduit * carrier )
{
       short unsigned int tmp1 = 0;

	cd.setSourcePort( tmp1 );
	cd.setDestinationPort( tmp1 );
	cd.setDestinationIPAddress( IPAddress() ); // i.e. 0.0.0.0
	cd.gotoState( CLOSED );
	Debug << "CgS::aC entering CLOSED state.\n" << eor;
	ret = OK;
	return( CLOSED );
}
TCPState *
ClosingState::processIncomingMessage( Return_Code & ret,
				TCPConnectionDescriptor & cd,
				TCPConduit * carrier, TCPMessage * msg )
{
	MessageID id1, id2;
	TCPMessage * reply;

	Assert( carrier != 0 );
	Assert( msg != 0 );

	// Check whether the segment is in the receive window:
	ret = checkIncomingMessage( cd, carrier, msg );
	if( ret == OK ) {
	    if( msg->RST() ) {
		// That's all, folks -- a mostly-dead connection was aborted:
		// Delete outstanding messages:
		id1 = cd.outstanding();
		while( id1 != -1 ) {
		    id2 = cd.outgoingBuffer()->previous( id1 );
		    cd.outgoingBuffer()->del( id1 );
		    id1 = id2;
		}
		msg->del();

		short unsigned int tmp1 = 0;

		cd.setSourcePort( tmp1 );
		cd.setDestinationPort( tmp1 );
		cd.setDestinationIPAddress( IPAddress() );	// i.e. 0.0.0.0
		cd.gotoState( CLOSED );
		Debug << "CgS::pIM entering CLOSED state.\n" << eor;
		return( CLOSED );
	    }
	    if( msg->SYN() ) {
		// Error! Connection reset! (A SYN in the window is bad news.)
		sendAbort( cd, carrier );
		// Delete outstanding messages:
		id1 = cd.outstanding();
		while( id1 != -1 ) {
		    id2 = cd.outgoingBuffer()->previous( id1 );
		    cd.outgoingBuffer()->del( id1 );
		    id1 = id2;
		}
		// Note that we have just deleted the abort-request from
		//   the ARB, so it only will get sent once. In the (hopefully
		//   unlikely) event that it gets lost, the other end will
		//   time out and get bad news about trying to talk on a non-
		//   existent connection....

		short unsigned int tmp2 = 0;

		cd.setSourcePort( tmp2 );
		cd.setDestinationPort( tmp2 );
		cd.setDestinationIPAddress( IPAddress() );	// i.e. 0.0.0.0
		cd.gotoState( CLOSED );
		Debug << "CgS::pIM entering CLOSED state....\n" << eor;
		msg->del();
		ret = CONNECTION_RESET;
		return( CLOSED );
	    }
	    if( msg->ACK() ) {
		acknowledge( cd, carrier, msg->AckNumber() );
		// Do Send Window stuff (see TCPEstablishedState.cc for expln):
		if( isBetween( cd.SendWL2().val(),
			       msg->AckNumber().val(),
			       cd.SendNext().val() )     &&
		    isBetween( cd.SendWL1().val(),
	    		       msg->SequenceNumber().val(),
			       cd.ReceiveNext().val() +
		       		cd.ReceiveWindow().val() ) ) {
		    // Update send window:
		    cd.setSendWL1( msg->SequenceNumber() );
		    cd.setSendWL2( msg->AckNumber() );
		    // If it's a duplicate of the last segment used to update
		    //   the window, only allow increases in window size,
		    //   since identical-looking ACK segments may have been
		    //   reordered:
		    if( cd.SendWL1().val() == msg->SequenceNumber().val()   &&
			cd.SendWL2().val() == msg->AckNumber().val()        &&
			cd.SendWindow().val() < msg->Window().val() )  {
	    		cd.setSendWindow( msg->Window() );
		    }
		    // Otherwise, update the window:
		    if( cd.SendWL1().val() != msg->SequenceNumber().val()   ||
			cd.SendWL2().val() != msg->AckNumber().val() )  {
	    		cd.setSendWindow( msg->Window() );
		    }
		}

		if( cd.SendUnacknowledged().val() != msg->AckNumber().val() ) {
		    // Update acknowledgements received:
		    cd.setSendUnacknowledged( msg->AckNumber() );
		}

	    	// Punt URG flag and urgent/push nonsense....

		// Check whether the ACK acknowledges our FIN. If so,
		//   everything's done and we enter TIME-WAIT, otherwise,
		//   ignore this message:
		if(  msg->AckNumber().val() == cd.SendNext().val() &&
		     msg->SequenceNumber().val() == cd.ReceiveNext().val() ) {
		    cd.gotoState( TIME_WAIT );
		    Debug << "CgS::pIM entering TIME_WAIT state....\n" << eor;
		    cd.setTimestamp( now32() );
		    msg->del();
		    // Delete outstanding messages:
		    // Delete everything from the rexmit buffer:
		    cd.outgoingBuffer()->reset();
		    cd.linkOutstanding( -1 );
		    return( TIME_WAIT );
		}
		else {
		    Debug << "CgS::pIM _NOT_ entering TIME_WAIT state....\n" << eor;
		}
		if( msg->FIN() ) {
		    // Send empty segment to acknowledge the FIN:
		    reply = new TCPMessage( 20, 0, 20, 0, 0 );
		    cd.pseudoheader()->setLength( 20 );
		    sendMessage( cd, carrier, reply );
		    cd.setReceiveNext( cd.ReceiveNext().val() + 1 );
		    // (Since the FIN takes up one sequence number)

		    // Ignore. Stay in CLOSING state:
		    msg->del();
		    return( CLOSING );
		}
	    } // end of ack-check
	}
	// No ACK or not OK means ignore the addled piece of shit:
	msg->del();
	return( CLOSING );	// No change.
}

