#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Johnny Zweig (zweig@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * TCPConduit.cc 
 *
 */
/*
	See Conduit.h for much better comments about what all this stuff
	does....
 */

#include "OutputStream.h"
#include "ConduitTypes.h"
#include "Word.h"
#include "CPU.h"
#include "Semaphore.h"
#include "ZootConfig.h"
#include "ZootReturnCodes.h"
#include "Assert.h"
#include "ZootAddress.h"
#include "ConduitMessage.h"
#include "ZootClock.h"
#include "IPChecksum.h"
#include "ConduitMessageBuffer.h"
#include "MessageFIFO.h"
#include "ZootStateMachine.h"
#include "Zoot_IP_Output.h"
#include "Conduit.h"



#ifdef __GNUC__
#pragma implementation
#endif

// ----------------------------------------------------------------------------
// -------------------------- Class TCPConduit stuff: -------------------------
// ----------------------------------------------------------------------------
// Since there (for now) will only be one TCP conduit in a system, it seems 
//   dumb to have a constructor that takes _appsize, msgsize, waitsize, and
//   a few conduit *'s as arguments. Change later -- for now, just edit 
//   defaults below....

TCPConduit::TCPConduit()
{
	int i;

	my_proto = TCP;
	_appsize = DEFAULT_NUMBER_OF_APPLICATIONS;
		// Selected at Random
	_appcount = 0;
	_IPConduit = 0;
	Debug << this << ":1\n" << eor;
	_connections = new TCPConnectionDescriptor[_appsize];
	// all are automatically set up in CLOSED state...
	Debug << this << ":2\n" << eor;
	for( i = 0; i < _appsize; i++ ) {
	   _connections[i].setCarrier( this );
	}
	Debug << this << ":3\n" << eor;
	_mutexes = new MutexSemaphore[_appsize];
		// (Get initialized to 1 by default.)
	// _mutex gets default-contructed groovily (I hope!)
	Debug << this << ":4\n" << eor;
	TCPIncomingMessageFIFO = new SemaphoreRegulatedMessageFIFO( 100 );
		// 100 should be plenty of space so that messages never get
		//  thrown away just because the receive-processes can't eat
		//  them fast enough. (If there might be more than 1 TCP
		//  Conduit, better check that TCPIncomingMessageFIFO only gets
		//  initialized once! )
	Debug << this << ":5\n" << eor;
}
TCPConduit::~TCPConduit()
{
	// Deleting a buffer automatically deletes any msgs still in there.
	// CHECK delete[_appsize] _connections;
	delete [] _connections;
	// CHECK delete[_appsize] _mutexes;
	delete [] _mutexes;
	delete   TCPIncomingMessageFIFO;
}

Return_Code
TCPConduit::connectTo( Conduit * c )
{
	Return_Code ret;

	// Do a double-dispatch with the other Conduit:
	ret = c->connectFromAbove( this, my_proto );
	if( ret != OK )
	{
	    // Undo the connectFromBelow() that c did
	    _IPConduit = 0;
	    for( int i = 0; i < _appsize; i++ ) {
		_connections[ i ].connect( 0 );
	    }
	}
	return( ret );
}
Return_Code
TCPConduit::connectFromAbove( Conduit * c, ProtocolID proto )
{
	int i = 0;

	// Just need to tell c what kind of Conduit I am -- TCP keeps
	// no list of attached Conduits:
	return( c->connectFromBelow( this, my_proto, 0, i ) );
}
Return_Code
TCPConduit::connectFromBelow( Conduit * c, ProtocolID proto, int his_handy,
				int & my_handy )
{
	//Return_Code ret;
	int i;

	Debug << "TCPConduit::cFB() protocol:" << proto << "\n" << eor;
	if( proto != IP ) return( INVALID_PROTOCOL );

	if( _IPConduit != 0 ) return( CANNOT_CONNECT );
		// Already connected to an IP Conduit....

	_IPConduit = (IPConduit *)c;
	for( i = 0; i < _appsize; i++ ) {
	    _connections[ i ].connect( c );
	    // Connects all the buffers up. Important!
	}
	my_handy = 6;	// See RFC 1010 for IP-protocol-numbers
	return( OK );
}
Return_Code
TCPConduit::disconnectFrom( Conduit * c )
{
	Return_Code ret;

	if( _IPConduit != c ) return( NO_SUCH_CONDUIT );

	ret = c->disconnectFromAbove( this );
	if( ret == OK ) {
	    _IPConduit = 0;
	    for( int i = 0; i < _appsize; i++ ) {
		_connections[ i ].connect( 0 );
	    }
	}
	return( ret );
}
Return_Code
TCPConduit::disconnectFromAbove( Conduit * c )
{
	// Should check if c is participating in any _connections:
	      //_connections[i].abortConnection();
	      //_connections[i].setUser( 0 );
	return( Return_Code(0) );
}


Return_Code
TCPConduit::open( Conduit * cond, const Word16 & srcport, int16 initialWindow,
			TCPAddress & conn )
{
	// "PASSIVE" OPEN (promiscuous)
	TCPConnectionDescriptor * cd;
	Return_Code ret;
	int i;
	Boolean NOTFREE = TRUE;
	int32 t, now;

	_mutex.P();
	// First, check if there's room:
	if ( _appcount == _appsize ) { // No room for connection record....
	    _mutex.V();
	    ret = TOO_MANY_CONNECTIONS; 
	    conn = -1;
	}
	else {
	    // Next, find a spare connection-descriptor (either unused or in
	    //   TIME-WAIT state with a sufficiently old timestamp):
	    //   (TIME_WAIT_DURATION is defined in ZootConfig.h)
	    // Note that there may no _be_ a spare CD if too many of them are
	    //   are still in TIME_WAIT state.
	    now = now32().val();
	    for( i = 0; NOTFREE && i < _appsize; i++ ) {
		t = _connections[ i ].timestamp().val();
		if( _connections[ i ].user() == 0 )
		    NOTFREE = FALSE;
		if( _connections[ i ].state() == TIME_WAIT ) {
		    if( t < (0xffffffff - TIME_WAIT_DURATION) ) {
			if( t > now || ( t + TIME_WAIT_DURATION < now ) )
			    NOTFREE = FALSE;
			    _connections[ i ].gotoState( CLOSED );
		    }
		    else {
			if( t > now &&
			    ( TIME_WAIT_DURATION - (0xffffffff - t) < now ) )
			    NOTFREE = FALSE;
			    // (Note the super-hip way of wrapping around what
			    //  the timestamp will be in 100 seconds without
			    //  relying on unsigned 32-bit addition to wrap
			    //  around sensibly. Neat, eh?)
			    _connections[ i ].gotoState( CLOSED );
		    }
		}
	    }
	    if( ! NOTFREE ) {
		i--;	// The loop ++'ed i once too often
		_connections[ i ].setUser( cond );
		_appcount++;
		_mutex.V();

		// Now, set up the connection-descriptor:
		_mutexes[ i ].P();
		cd = & _connections[ i ];
		cd->setSourcePort( srcport );
		
		short unsigned int tmp1 = 0;

		cd->setDestinationPort( tmp1 );
		cd->setDestinationIPAddress( IPAddress() );
		cd->setReceiveWindow( Word16( initialWindow ) );
		cd->outgoingBuffer()->reset();
			// Just in case anything is left over from a previous
			// incarnation -- you can't be too careful!
		// Now, tell the CD that an open request has occured; its state
		//   (you and I know it's CLOSED, in fact) will inspect the C-D
		//   and take whatever action is appropriate (see StateMachine
		//   stuff):
		ret = cd->openConnection();
		_mutexes[ i ].V();
		conn = i;
	    }
	    else {
		_mutex.V();
		ret = TOO_MANY_CONNECTIONS;
		conn = -1;
	    }
	}
	Debug << "\tTCPConduit::open(1) ret=" << (int)ret << eor;
	return ( ret );
}
Return_Code
TCPConduit::open( Conduit * cond, const Word16 & srcport,
                        const Word16 & destport,
			int16 initialWindow, TCPAddress & conn )
{
	TCPConnectionDescriptor * cd;
	Return_Code ret;
	int i;
	Boolean NOTFREE = TRUE;
	int32 t, now;

	_mutex.P();
	// First, check if there's room:
	if ( _appcount == _appsize ) { // No room for connection record....
	    _mutex.V();
	    ret = TOO_MANY_CONNECTIONS; 
	    conn = -1;
	}
	else {
	    // Next, find a spare connection-descriptor (either unused or in
	    //   TIME-WAIT state with a sufficiently old timestamp):
	    now = now32().val();
	    for( i = 0; NOTFREE && i < _appsize; i++ ) {
		t = _connections[ i ].timestamp().val();
		if( _connections[ i ].user() == 0 )
		    NOTFREE = FALSE;
		if( _connections[ i ].state() == TIME_WAIT ) {
		    if( t < (0xffffffff - TIME_WAIT_DURATION) ) {
			if( t > now || ( t + TIME_WAIT_DURATION < now ) )
			    NOTFREE = FALSE;
			    _connections[ i ].gotoState( CLOSED );
		    }
		    else {
			if( t > now &&
			    ( TIME_WAIT_DURATION - (0xffffffff - t) < now ) )
			    NOTFREE = FALSE;
			    _connections[ i ].gotoState( CLOSED );
		    }
		}
	    }
	    if( ! NOTFREE ) {
		i--;	// The loop ++'ed i once too often
		_connections[ i ].setUser( cond );
		_appcount++;
		_mutex.V();

		// Now, set up the connection-descriptor:
		_mutexes[ i ].P();
		cd = & _connections[ i ];
		cd->setSourcePort( srcport );
		cd->setDestinationPort( destport );
		cd->setDestinationIPAddress( IPAddress() );
		cd->setReceiveWindow( Word16( initialWindow ) );
		// Now, tell the CD that an open request has occured; its state
		//   (you and I know it's CLOSED, in fact) will inspect the C-D
		//   and take whatever action is appropriate (see StateMachine
		//   stuff):
		ret = cd->openConnection();
		_mutexes[ i ].V();
		conn = i;
	    }
	    else {
		_mutex.V();
		ret = TOO_MANY_CONNECTIONS;
		conn = -1;
	    }
	}
	Debug << "\tTCPConduit::open(2) ret=" << (int)ret << eor;
	return ( ret );
}
Return_Code
TCPConduit::open( Conduit * cond, const Word16 & srcport,
                        const Word16 & destport,
			const IPAddress & destipaddr,
			int16 initialWindow, 
			TCPAddress & conn )
{
	TCPConnectionDescriptor * cd;
	Return_Code ret;
	int i;
	Boolean NOTFREE = TRUE;
	int32 t, now;

	_mutex.P();
	// First, check if there's room:
	if ( _appcount == _appsize ) { // No room for connection record....
	    _mutex.V();
	    ret = TOO_MANY_CONNECTIONS; 
	    conn = -1;
	}
	else {
	    // Next, find a spare connection-descriptor (either unused or in
	    //   TIME-WAIT state with a sufficiently old timestamp):
	    now = now32().val();
	    for( i = 0; NOTFREE && i < _appsize; i++ ) {
		t = _connections[ i ].timestamp().val();
		if( _connections[ i ].user() == 0 )
		    NOTFREE = FALSE;
		if( _connections[ i ].state() == TIME_WAIT ) {
		    if( t < (0xffffffff - TIME_WAIT_DURATION) ) {
			if( t > now || ( t + TIME_WAIT_DURATION < now ) )
			    NOTFREE = FALSE;
			    _connections[ i ].gotoState( CLOSED );
		    }
		    else {
			if( t > now &&
			    ( TIME_WAIT_DURATION - (0xffffffff - t) < now ) )
			    NOTFREE = FALSE;
			    _connections[ i ].gotoState( CLOSED );
		    }
		}
	    }
	    if( ! NOTFREE ) {
		i--;	// The loop ++'ed i once too often
		_connections[ i ].setUser( cond );
		_appcount++;
		_mutex.V();

		// Now, set up the connection-descriptor:
		_mutexes[ i ].P();
		cd = & _connections[ i ];
		cd->setSourcePort( srcport );
		cd->setDestinationIPAddress( destipaddr );
		cd->setDestinationPort( destport );
		cd->setReceiveWindow( Word16( initialWindow ) );
		// Now, tell the CD that an open request has occured; its state
		//   (you and I know it's CLOSED, in fact) will inspect the C-D
		//   and take whatever action is appropriate (see StateMachine
		//   stuff):
		ret = cd->openConnection();
		_mutexes[ i ].V();
		conn = i;
	    }
	    else {
		_mutex.V();
		ret = TOO_MANY_CONNECTIONS;
		conn = -1;
	    }
	}
	Debug << "\tTCPConduit::open(3) ret=" << (int)ret << eor;
	return ( ret );
}
Return_Code
TCPConduit::insertFromAbove( ConduitMessage * msg, NetworkAddress * addr )
{
	TCPConnectionDescriptor * cd;
	Return_Code ret;
	TCPMessage * tcpmsg;
	TCPAddress * tcpaddr;

	Assert( msg != 0 );
	Assert( addr != 0 );

	Debug << "TCPConduit::insertFromAbove( msg:" << msg << " addr:" << addr
		<< "\n" << eor;
	tcpmsg = (TCPMessage*)msg;
	tcpaddr = (TCPAddress*)addr;
	if( tcpaddr->handle() >= 0 && tcpaddr->handle() < _appsize)
		cd = & _connections[ tcpaddr->handle() ];
	else cd = 0;
	//Console().OldStyleDebug("\tabout to process on connection descriptor %x\n",
	//	cd );
	//Console().OldStyleDebug("\twhich is the %dth connection...\n",
	//	tcpaddr->handle() );
	if( cd != 0 ) {

            if( tcpmsg->headerLength() == 0 )
                tcpmsg->setHeader( 20, new octet[ 20 ] );
            if( tcpmsg->IPmsg() == 0 )
                tcpmsg->setIPmsg( new IPMessage( 20, 0, tcpmsg ) );

	    _mutexes[ tcpaddr->handle() ].P();
	    ret = cd->processOutgoingMessage( tcpmsg );
	    _mutexes[ tcpaddr->handle() ].V();
	}
	else {
	    ret = NO_SUCH_CONNECTION;
	    Debug << "\tTCPConduit::insertFromAbove: NO_SUCH_CONNECTION\n" << eor;
	}
	return ( ret );
}
TCPMessage *
TCPConduit::prepareIncomingMessage( IPMessage * ipmsg )
{
	TCPMessage * msg;
	int hlen;
	int dlen;

	Assert( ipmsg != 0 );

	// The checksum has already been checked by the TCP conduit by the time
	//   this function has been called....
	// The first job is to brew up a TCPMessage that thinks it lives
	//   in this IPMessage, which will be returned:
	msg = new TCPMessage();
		// Don't want to allocate any more storage -- just set up
		//   pointers.
	// Temporarily set up TCP message to have a header containing
	//   everything (so that I can use TCPMessage::Offset() to nab the
	//   TCP header's length in 32-bit words):
	//msg->setHeader( ipmsg->dataLength(), ipmsg->data() );
	  // Note: this changes the offset field and fucks things up!!***JZ***
	hlen = ((*(ipmsg->data()+12) & 0xf0) >> 2);
	    // Yikes! This is a grungy bit-manipulation to grab the high-order
	    //   4 bits out of the TCP-header's 13th byte (header-length
	    //   measured in 32-bit words) and multiply it by 4 in one go. -JZ
	dlen = ipmsg->dataLength() - hlen;
	// Now set it up properly:
	msg->setHeader( hlen, ipmsg->data() ); // Get the length right
	msg->pleaseDontDeleteHeader();
	msg->setData( dlen, ( ipmsg->data() + hlen ) );
	msg->pleaseDontDeleteData();
	// Don't foget to tell the IP message who it's carrying:
	ipmsg->setLink( msg );
	msg->setIPmsg( ipmsg );	// Hochwichtig! Otherwise, can't ask for
				//   source-Internet-address.
	ipmsg->setData( 0, 0 );
	ipmsg->pleaseDontDeleteData();
		// Since now the IP-message's data belongs to the TCP-message,
		//   need to clobber the (now superfluous) pointer so that
		//   length-calculations are correct. (Having msg->header()
		//   and ipmsg->data() point to the same thing is bad...)
	// *** Don't want to call pleaseDontDeletePayload() since this TCP
	//     message does need to be deleted -- the stuff it points to
	//     will get deleted when the IP message gets destroyed. ***

	return( msg );
}
Return_Code
TCPConduit::insertFromBelow( ConduitMessage * msg )
{
	IPMessage * ipmsg;
	TCPMessage * tcpmsg;
	TCPPseudoHeader ph;
	IPAddress ip;
	Word16 len;
	Word16 check;
	TCPConnectionDescriptor * cd = 0;
	int i;
	Word16 destport;
	Word16 srcport;
	Return_Code ret;

	Assert( msg != 0 );
	ipmsg = (IPMessage*)msg;

#define put(x) Debug << (x) << "\n" << eor;
put("TCPC::iFB");
	_mutex.P();
put("=");
	  //Console().OldStyleDebug("TCPC::iB(): received msg @ %x\n", ipmsg);
	  //ipmsg->print();

	// First, need to check checksum:
	ip.equals( ipmsg->SourceADDR() );
	  //Console().OldStyleDebug("\tSetting the ph-srcAddr to %x\n",ip.val().val());
	  //Console().OldStyleDebug("\tmsg->srcADDR: %x\n", ipmsg->SourceADDR().val());
	ph.setSourceAddress( ip );
	ip.equals( ipmsg->DestADDR() );
	  //Console().OldStyleDebug("\tSetting the ph-dstAddr to %x\n",ip.val().val());
	ph.setDestinationAddress( ip );

	short unsigned int tmp2 = ipmsg->TotalLength().val() - ipmsg->HLEN()*4;
	len.equals( tmp2 );
	// Total length is measured in octets, header length in 32-bit words
	//   (See RFC 791 p.11, 13)
	  //Console().OldStyleDebug("\tSetting the ph-length to %x\n",len.val());
	ph.setLength( len );
	
	short unsigned int tmp3 = ~( ~ph.checksum().val() + 
				     ~IPChecksum(ipmsg->data(), ipmsg->dataLength() ).val() );
	check.equals( tmp3 );

	if( check.val() != 0x0000 && check.val() != 0xffff ) {
	   Debug << "\tTCP Checksum error.(chksum:" << check.val() <<
			"\n" << eor;
	   _mutex.V();
put("CHKSUMERR!");
	   ipmsg->del();
	   return( CHECKSUM_ERROR );
	}
	
	// Next, need to find the C-D for this message's connection:
	//   (the msg->data() + 2 bit is because this is an IP message, not
	//   a TCP message. If it proves to belong to any of the _connections,
	//   we will turn it into a TCP message later.
	destport.equals( ipmsg->data() + 2 ); // 3rd, 4th bytes of _TCP-header_
	for( i = 0; i < _appsize && cd == 0; i++ ) {
/***DEBUG
	   Console().OldStyleDebug("\t_connections[%d]:SrcP:%d DestP: %d DestAd: %d\n",
			i,
			_connections[i].sourcePort().val(),
			_connections[i].destinationPort().val(),
			_connections[i].destinationIPAddress().val().val() );
DEBUG***/
	   // Is it directed at this connection's port?
	   if( _connections[i].sourcePort().val() == destport.val() ) {
	      // Is it from this connection's other end's port?
	      srcport.equals( ipmsg->data() + 0 );
	      if( _connections[i].destinationPort().val() == srcport.val() ||
		  _connections[i].destinationPort().val() == 0 ) {
		 // Is it actually from the other end?
		 if( ipmsg->SourceADDR().val() == 
		    _connections[i].destinationIPAddress().val().val() ||
		    _connections[i].destinationIPAddress().val().val() == 0 ) {
			cd = & _connections[i];
		 }
	      }
	   }
	}
	_mutex.V();
put("=");
		// I think this is the right place to unlock the CD table -JZ
	if( cd == 0 ) {
	   Debug << "\tUnable to find connection for this message.\n" << eor;
	   ipmsg->del();
put("NO SUCH CONNECTION");
	   return( NO_SUCH_CONNECTION );
	}
	// Okay, since it's a groovy message, now is the time to whip up a TCP
	//   message to put the thing into:
	tcpmsg = prepareIncomingMessage( ipmsg );

	// Process the message:
put("%");
	_mutexes[i].P();
	//Console().OldStyleDebug("\tProcessing message @%x on cd @%x:\n", tcpmsg, cd);
	//Console() << "\tprocessing message " << asHex( (unsigned int)tcpmsg ) << " on CD #" << i << "\n" << eor;
	ret = cd->processIncomingMessage( tcpmsg );
	_mutexes[i].V();

	Debug << "\tTCPC::iFB: returning\n" << eor;

	return( ret );
}

Return_Code
TCPConduit::close( TCPAddress & conn )
{
	TCPConnectionDescriptor * cd;
	Return_Code ret;

	_mutexes[ conn.handle() ].P();
	cd = & _connections[ conn.handle() ];
	ret = cd->closeConnection();
	_mutexes[ conn.handle() ].V();
	return( ret );
}
Return_Code
TCPConduit::expandWindow( TCPAddress & conn, int16 additional )
{
	TCPConnectionDescriptor * cd;
	int16 current_receive_window;

	_mutexes[ conn.handle() ].P();
	cd = & _connections[ conn.handle() ];
	current_receive_window = cd->ReceiveWindow().val();
	if( (int32)current_receive_window + additional > 0x0000ffff )
		return( ERROR ); // Overflow -- try again later.

	short unsigned int tmp1 = current_receive_window + additional;

	cd->setReceiveWindow( tmp1 );
	_mutexes[ conn.handle() ].V();
	
	return( OK );
}
Return_Code
TCPConduit::setTimeout( int32 newtimeout )
{
//****	myOutgoingBuffer->setDelay( newtimeout );
	return( OK );
}
Return_Code
TCPConduit::abort( TCPAddress & conn )
{
	TCPConnectionDescriptor * cd;
	Return_Code ret;

	_mutexes[ conn.handle() ].P();
	cd = & _connections[ conn.handle() ];
	ret = cd->abortConnection();
	_mutexes[ conn.handle() ].V();
	//Console().OldStyleDebug("TCPC::abort(): just aborted (%dth) connection %x\n", conn.handle(), cd );
	//Console().OldStyleDebug("\tReturn Code: %d\n", ret );

	return ( ret );
}
void
TCPConduit::Reset()
{	// THOROUGHLY EVIL!!!
	int i;
	for( i = 0; i < _appsize; i++ ) {
	   _connections[i].abortConnection();
	   _connections[i].setUser( 0 );
	}
	_appcount = 0;
//*****	myOutgoingBuffer->reset();
//*****	waitinglist->reset();
}
