#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Johnny Zweig (zweig@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * TCPListenState.cc
 *
 */


#include "Semaphore.h"
#include "Assert.h"
#include "ThisProcess.h"
#include "OutputStream.h"
#include "ConduitTypes.h"
#include "Word.h"
#include "ZootConfig.h"
#include "ZootReturnCodes.h"
#include "Assert.h"
#include "ZootAddress.h"
#include "ConduitMessage.h"
#include "ZootClock.h"
#include "Conduit.h"
#include "ConduitMessageBuffer.h"
#include "ZootStateMachine.h"


#ifdef __GNUC__
#pragma implementation
#endif

#ifndef QUIET
#undef Debug
#define Debug thisProcess()->console()
#endif
//-----------------------------------------------------------------------------
//				   ListenState
//-----------------------------------------------------------------------------
ListenState::ListenState()
{
}
ListenState::~ListenState()
{
}
TCPState *
ListenState::openConnection( Return_Code & ret, TCPConnectionDescriptor & cd,
				TCPConduit * carrier )
{
	// May be a redundant request for passive open, or a switch to an
	//   active open. Can't be an active-open retry, since passive open
	//   is the only way to get into LISTEN state.
	Debug << "LS::oC() called (redundant?)" << eor;

	if( cd.sourcePort().val() == 0 ) {
	    ret  =  ERROR_IN_SPECIFICATION;
	    return( LISTEN );	// Fuck up -- don't change state
	}
	if( cd.destinationIPAddress().val().val() == 0 ) {
	    // ---- PASSIVE OPEN: ----
	    if( cd.destinationPort().val() == 0 ) {
		// ---- Promiscuous ----
		ret  =  OK;
		return( LISTEN );
	    }
	    else {
		// ---- Restricted ----
		ret  =  OK;
		return( LISTEN );
	    }
	}

	// ---- ACTIVE OPEN: ----
	Debug << "ListenState: Passive turned into Active open....\n" << eor;

	ret = sendConnectRequest( cd, carrier );
	
	cd.gotoState( SYN_SENT );
	return( SYN_SENT );
}
TCPState *
ListenState::closeConnection( Return_Code & ret, TCPConnectionDescriptor & cd,
				TCPConduit * carrier )
{
        short unsigned int tmp1 = 0;

	cd.setSourcePort( tmp1 );
	cd.gotoState( CLOSED );
	ret = OK;
	return( CLOSED );

}
TCPState *
ListenState::abortConnection( Return_Code & ret, TCPConnectionDescriptor & cd,
				TCPConduit * carrier )
{
        short unsigned int tmp1 = 0;

	cd.setSourcePort( tmp1 );
	cd.gotoState( CLOSED );
	ret = OK;
	return( CLOSED );
}
TCPState *
ListenState::processOutgoingMessage( Return_Code & ret,
				TCPConnectionDescriptor & cd,
				TCPConduit * carrier, TCPMessage * msg )
{
	MessageID tmp;

	// I think it's evil, but RFC793 allows sending data on a passively
	//   open connection as a mechanism for switching to active-open;
	//   since msg has been provided, I'll check there to see if the
	//   foreign socket has been specified -- there's no way (since we're
	//   in LISTEN state) that it could have been set up in the CD.

	Assert( carrier != 0 );
	Assert( msg != 0 );

	if( msg->DestinationPort().val() == 0 ||
		msg->DestinationAddress().val().val() == 0 ) {
	   Debug << "ListenState: Erroneous attempt to send...\n" << eor;
	   ret  =  ERROR_IN_SPECIFICATION;
	   return( LISTEN );	// Fuck up -- don't change state
	}
	// ---- ACTIVE OPEN: ----
	// Tell the CD about the new destination port/address:
	cd.setDestinationPort( msg->DestinationPort() );
	cd.setDestinationIPAddress( msg->DestinationAddress() );
	Debug << "ListenState: Passive turned into Active open by sending...\n" << eor;

	// Queue up msg to be sent when we get to ESTABLISHED state:
	tmp = cd.waiting();
	cd.linkWaiting( cd.waitingBuffer()->insert( ret, msg, 0 ) );
		// Notice the fake address since insert() wants both....
		// Note that this value of ret will probably get stomped...
	if( ret == BUFFER_FULL ) {
	    cd.linkWaiting( tmp );
	    // Pass on the bad new to the application:
	    return( LISTEN );
	}

	ret = sendConnectRequest( cd, carrier );
	
	cd.gotoState( SYN_SENT );
	return( SYN_SENT );
}
TCPState *
ListenState::processIncomingMessage( Return_Code & ret,
				TCPConnectionDescriptor & cd,
				TCPConduit * carrier, TCPMessage * msg )
{
	IPAddress tmpIPAddr;
	int32 tmpPort;
	TCPMessage * reply;

	Assert( carrier != 0 );
	Assert( msg != 0 );

	// Check the control-flags:
	if( msg->RST() ) {
	    // Ignore -- asking to delete some other incarnation of this
	    //   connection.... N.B.: NEVER send anything in response to a
	    //   segment with the reset-bit set! It would be bad...
	    msg->del();
	    ret = OK;
	    return( LISTEN );
	}
	if( msg->ACK() ) {
	    // An Acknowledgement must be acknowledging some data from a
	    //   different incarnation. Send a Reset-segment to the person
	    //   trying to talk to this port (need to stash old socket
	    //   info, esp. if it was 0, and fill in this msg's socket so
	    //   the abort-request goes to the right place):
	    tmpPort = cd.destinationPort().val();
	    tmpIPAddr = cd.destinationIPAddress();
	    cd.setDestinationPort( msg->SourcePort() );
	    cd.setDestinationIPAddress( msg->SourceAddress() );
		// Temporary -- just so the abort-message goes to the right
		//   place...
	    cd.pseudoheader()->setLength( 20 );	// RST-segment carries no data

	    ret = sendAbort( cd, carrier, msg->AckNumber() );

	    cd.setDestinationPort( tmpPort );
	    cd.setDestinationIPAddress( tmpIPAddr );

	    msg->del();
	    ret = OK;
	    return( LISTEN );
	}
	if( msg->SYN() ) {
	    // A-ha! The other end wants to connect:
	    // Record newly-received socket information about other end:
	    cd.setDestinationPort( msg->SourcePort() );
	    cd.setDestinationIPAddress( msg->SourceAddress() );


	    Debug << "\tmsg: %x, ip-msg:%x\n"<< msg<< msg->IPmsg();
	    Debug << "\tincoming msg from: "<< asHex( msg->SourceAddress().val().val() ) << "\n" << eor;

	    // Ignore precedence/security....

	    // Initialize Receive sequence number expectation, etc.:
	    cd.setReceiveNext(
		msg->SequenceNumber().val() + msg->dataLength() + 1 );
		// SYN-segment could contain data from an evil person.
	    cd.setReceiveInitialSequenceNumber( msg->SequenceNumber() );
	    cd.setSendInitialSequenceNumber( now32() );
	    cd.setSendNext( cd.SendInitialSequenceNumber().val() );
	    cd.setSendUnacknowledged(
			cd.SendInitialSequenceNumber().val() );
	    // Send acknowledgement of connect-request:
	    reply = new TCPMessage(  20,	// IP-header size
				       0,	// IP data size
				       20,	// TCP header size
				       0,	// TCP data size
				       0 );	// No payload
	    Assert( 0 != reply );
	    cd.pseudoheader()->setLength( 20 );	// SYN-segment carries no data
	    reply->setSYN( TRUE ); // IMPORTANT!
Debug << "\t\treplying...\n" << eor;
	    ret = sendMessage( cd, carrier, reply,
			       cd.SendInitialSequenceNumber() );
Debug << "\t\tdone.\n" << eor;
		// sendMessage() should set everything else up right...
		// This is the moral equivalent of a sendConnectRequest().
	    cd.setSendNext( cd.SendInitialSequenceNumber().val() + 1 );

	    // Ignore any data present -- nice people don't send data in SYN
	    //   segments anyway. A retransmission of the SYN segment will
	    //   get handled properly in the SYN_REC'D state.... (Actually,
	    //   it won't unless the SYN is stripped and the sequence-# is
	    //   incremented. Sigh!)

	    msg->del();
	    cd.gotoState( SYN_RECEIVED );
	    return( SYN_RECEIVED );
	}
	// Otherwise, punt this message. It's useless.
	msg->del();
	return( LISTEN );	// No change...
}
