#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Johnny Zweig (zweig@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * TCPSyn_ReceivedState.cc
 *
 */


#include "Semaphore.h"
#include "Assert.h"
#include "ThisProcess.h"
#include "OutputStream.h"
#include "ConduitTypes.h"
#include "Word.h"
#include "ZootConfig.h"
#include "ZootReturnCodes.h"
#include "Assert.h"
#include "ZootAddress.h"
#include "ConduitMessage.h"
#include "ZootClock.h"
#include "Conduit.h"
#include "ConduitMessageBuffer.h"
#include "ZootStateMachine.h"


#ifdef __GNUC__
#pragma implementation
#endif

#ifndef QUIET
#undef Debug
#define Debug thisProcess()->console()
#endif
//-----------------------------------------------------------------------------
//				Syn_ReceivedState
//-----------------------------------------------------------------------------
Syn_ReceivedState::Syn_ReceivedState()
{
}
Syn_ReceivedState::~Syn_ReceivedState()
{
}
TCPState *
Syn_ReceivedState::closeConnection( Return_Code & ret,
				TCPConnectionDescriptor & cd,
				TCPConduit * carrier )
{
	MessageID id1, id2;

	// Need to delete any waiting messages:
	if( cd.waiting() != -1 ) {
	    id1 = cd.waiting();
	    id2 = cd.waitingBuffer()->previous( id1 );
	    while( id1 != -1 ) {
		cd.waitingBuffer()->del( id1 );
		id1 = id2;
		id2 = cd.waitingBuffer()->previous( id1 );
	    }
	}
	// I assume the source-addr and dest-addr have already been
	//   set up, since we are in the SYN-RECEIVED state.
	ret = sendCloseRequest( cd, carrier );	// Send FIN-segment (auto-
						//   matically gets queued)

	short unsigned int tmp4 = 0;
	cd.setSourcePort( tmp4 );
	cd.gotoState( FIN_WAIT_1 );
	Debug <<"S_RS::cC entering FIN_WAIT_1 state\n" << eor;
	return( FIN_WAIT_1 );
}
TCPState *
Syn_ReceivedState::abortConnection( Return_Code & ret,
				TCPConnectionDescriptor & cd,
				TCPConduit * carrier )
{
	MessageID id1, id2;

	// Need to delete any waiting messages:
	if( cd.waiting() != -1 ) {
	    id1 = cd.waiting();
	    id2 = cd.waitingBuffer()->previous( id1 );
	    while( id1 != -1 ) {
		cd.waitingBuffer()->del( id1 );
		id1 = id2;
		id2 = cd.waitingBuffer()->previous( id1 );
	    }
	    cd.linkWaiting( MessageID( -1 ) );
	}
	// I assume the source-addr and dest-addr have already been
	//   set up, since we are in the SYN-RECEIVED state.
	ret = sendAbort( cd, carrier );

	short unsigned int tmp5 = 0;

	cd.setSourcePort( tmp5 );
	cd.setDestinationPort( tmp5 );
	cd.setDestinationIPAddress( IPAddress() ); // i.e. 0.0.0.0
	cd.gotoState( CLOSED );
	Debug << "S_RS:aC entering CLOSED state\n" << eor;
	return( CLOSED );
}
TCPState *
Syn_ReceivedState::processOutgoingMessage( Return_Code & ret,
				TCPConnectionDescriptor & cd,
				TCPConduit * carrier, TCPMessage * msg )
{
	MessageID tmp;

	Assert( carrier != 0 );
	Assert( msg != 0 );

	// Queue up msg to be sent when we get to ESTABLISHED state:
	// (If the wait-buffer fills up, sadness happens.)
	tmp = cd.waiting();
	cd.linkWaiting( cd.waitingBuffer()->insert( ret, msg, 0 ) );
	if( ret != OK ) cd.linkWaiting( tmp );
	return( SYN_RECEIVED );	// No Change
}
TCPState *
Syn_ReceivedState::processIncomingMessage( Return_Code & ret,
				TCPConnectionDescriptor & cd,
				TCPConduit * carrier, TCPMessage * msg )
{
	TCPMessage * reply;
	MessageID waitingid = -1;
	TCPMessage * waiting = 0;
	Boolean WindowHasEnufRoom = TRUE;
	Word32 sequence_number, ack_number;
	int data_length;
	Word16 window;
	Boolean fin_bit_set;


	Assert( carrier != 0 );
	Assert( msg != 0 );

	// Check whether the segment is in the receive window:
	ret = checkIncomingMessage( cd, carrier, msg );
	if( ret == OK ) {
	    // First, check for abort-request (just close if so -- I'm blowing
	    //   off the bit with going back to LISTEN state if the connection
	    //   was originally open passively. Aborted is aborted, and the
	    //   result is a closed connection.)
	    if( msg->RST() ) {
		msg->del();
		
		short unsigned int tmp1 = 0;

		cd.setSourcePort( tmp1 );
		cd.setDestinationPort( tmp1 );
		cd.setDestinationIPAddress( IPAddress() );	// i.e. 0.0.0.0
		cd.gotoState( CLOSED );
		Debug << "S_RS:pIM entering CLOSED state\n" << eor;
		ret = CONNECTION_RESET;
		return( CLOSED );
	    }
	    if( msg->SYN() ) {
		// Error! Connection reset! (A SYN in the window is bad news.)
		sendAbort( cd, carrier );
		// sendAbort() deletes outstanding messages, too....

		short unsigned int tmp2 = 0;

		msg->del();
		cd.setSourcePort( tmp2 );
		cd.setDestinationPort( tmp2 );
		cd.setDestinationIPAddress( IPAddress() );	// i.e. 0.0.0.0
		cd.gotoState( CLOSED );
		Debug << "S_RS:pIM entering CLOSED state\n" << eor;
		ret = CONNECTION_RESET;
		return( CLOSED );
	    }
	    // Copy important stuff out of message before attempting to
	    //   deliver it to the user (since delivering it may destroy
	    //   it):
	    sequence_number = msg->SequenceNumber();
	    ack_number = msg->AckNumber();
	    data_length = msg->dataLength();
	    window = msg->Window();
	    fin_bit_set = msg->FIN();

	    if( msg->ACK() ) {
	      ret = cd.user()->insertFromBelow( msg );

	      if( ret == OK ) {
		if( isBetween(  cd.SendUnacknowledged().val(),
				ack_number.val(),
				cd.SendNext().val() ) ) {
		    // Initialize the window:
		    cd.setSendWindow( window );
		    cd.setSendWL1( sequence_number );
		    cd.setSendWL2( ack_number );
		    
		    cd.gotoState( ESTABLISHED );
		    Debug <<"S_RS::pIM entering ESTABLISHED state\n" << eor;

		    // Continue processing the same way as EstablishedState:

		    // Send anything in the wait-buffer now:
		    if( cd.waitingBuffer()->count() > 0 &&
					 cd.waiting() != -1 ) {
			waitingid = cd.waitingBuffer()->
				previous( cd.waiting() );
			waiting = (TCPMessage*)cd.waitingBuffer()->
				lookAt( waitingid );
		    }
		    while( waiting != 0 && WindowHasEnufRoom ) {

			if( isBetween( cd.SendUnacknowledged().val(),
			      cd.SendNext().val()           +
				  waiting->dataLength(),
			      cd.SendUnacknowledged().val() +
				  cd.SendWindow().val() ) ) {
			    // Send the waiting message off:
			    cd.pseudoheader()->setLength( waiting->length() );
			    ret = sendMessage( cd, carrier, waiting );
			    // Snip off from linked list and delete:
			    cd.linkWaiting( cd.waitingBuffer()->
						previous( waitingid ) );
			    cd.waitingBuffer()->del( waitingid );
	        	    waitingid = cd.waitingBuffer()->
						previous( cd.waiting() );
	        	    waiting = (TCPMessage*)cd.waitingBuffer()->
						lookAt( waitingid );
			    // lookAt() returns 0 if you try to look at garbage
			}
			else {
			    // Boo Hoo! Message is too big to be sent now....
			    WindowHasEnufRoom = FALSE;
			}
		    }

		    acknowledge( cd, carrier, ack_number );

		    // Do Send Window stuff (see TCPEstablishedState.cc):
		    if( isBetween( cd.SendWL2().val(),
			       ack_number.val(),
			       cd.SendNext().val() )     &&
		        isBetween( cd.SendWL1().val(),
	    		       sequence_number.val(),
			       cd.ReceiveNext().val() +
		       		cd.ReceiveWindow().val() ) ) {
		        // Update send window:
		        cd.setSendWL1( sequence_number );
		        cd.setSendWL2( ack_number );
		        // If it's a dupe of the last segment used to update
		        //   the window, only allow increases in window size,
		        //   since identical-looking ACK segments may have been
		        //   reordered:
		        if( cd.SendWL1().val() == sequence_number.val()   &&
			    cd.SendWL2().val() == ack_number.val()        &&
			    cd.SendWindow().val() < window.val() )  {
	    		    cd.setSendWindow( window );
		        }
		        // Otherwise, update the window:
		        if( cd.SendWL1().val() != sequence_number.val()   ||
			    cd.SendWL2().val() != ack_number.val() )  {
	    		    cd.setSendWindow( window );
		        }
		    }

		    if( cd.SendUnacknowledged().val() != ack_number.val() ) {
		        // Update acknowledgements received:
		        cd.setSendUnacknowledged( ack_number );
		    }

	    	    // Punt URG flag and urgent/push nonsense....

		    // Process the contents of the message:
		    // (1) Update Receive-Next:
		    cd.setReceiveNext( cd.ReceiveNext().val() +
						data_length );
		    short unsigned int tmp3 = cd.ReceiveWindow().val() - data_length; 
		    cd.setReceiveWindow( tmp3);

		    // Check FIN bit:
		    if( fin_bit_set ) {
			// Send empty segment to acknowledge the FIN:
			reply = new TCPMessage( 20, 0, 20, 0, 0 );
		        cd.pseudoheader()->setLength( 20 );
			sendMessage( cd, carrier, reply );
		        cd.setReceiveNext( cd.ReceiveNext().val() + 1 );
		        // (Since the FIN takes up one sequence number)

			cd.gotoState( CLOSE_WAIT );
			Debug <<"S_RS::pIM entering CLOSE_WAIT state\n" << eor;
			return( CLOSE_WAIT );
		    }
		    return( ESTABLISHED );
		}	// End of ack-in-range check

		else Debug <<"S_RS::pIM NOT entering ESTABLISHED state (bad ack number:" << ack_number.val() << ", not " <<  cd.SendUnacknowledged().val() <<
")\n" << eor;

	      } // end of ret==OK check
	    }	// end of ack-check
	    // If we've made it here, either the ACK bit wasn't set (so we
	    //   should discard the message now) or it was out of range.
	    //   (Data-carrying segments always have ACK set....)  In any case:
	    msg->del();
	    ret = OK;
	    return( SYN_RECEIVED );	// No change
	}
	Debug <<"\tS_RS: bad message seq-num\n" << eor;
	// ret will be ERROR since that's what checkIncomingMessage(--) returns
	msg->del();
	return( SYN_RECEIVED );	// No change
}

