#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Johnny Zweig (zweig@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * TCPTime_WaitState.cc
 *
 */


#include "Semaphore.h"
#include "Assert.h"
#include "ThisProcess.h"
#include "OutputStream.h"
#include "ConduitTypes.h"
#include "Word.h"
#include "ZootConfig.h"
#include "ZootReturnCodes.h"
#include "Assert.h"
#include "ZootAddress.h"
#include "ConduitMessage.h"
#include "ZootClock.h"
#include "Conduit.h"
#include "ConduitMessageBuffer.h"
#include "ZootStateMachine.h"


#ifdef __GNUC__
#pragma implementation
#endif

#ifndef QUIET
#undef Debug
#define Debug thisProcess()->console()
#endif

//-----------------------------------------------------------------------------
//				  Time_WaitState
//-----------------------------------------------------------------------------
// The idea of TIME-WAIT state is to leave the CD around long enough that any
//   old duplicates won't cause confusion due to lost information because of
//   the CD getting erased. I implement it by setting the timsetamp on the CD
//   and going to TIME_WAIT state. If the conduit needs a connection descriptor
//   it can harvest any CDs that are CLOSED or in TIME_WAIT and have suff-
//   iciently old timestamps.
Time_WaitState::Time_WaitState()
{
}
Time_WaitState::~Time_WaitState()
{
}
TCPState *
Time_WaitState::processIncomingMessage( Return_Code & ret,
			TCPConnectionDescriptor & cd,
			TCPConduit * carrier, TCPMessage * msg )
{
	MessageID id1, id2;
	TCPMessage * reply;

	Assert( carrier != 0 );
	Assert( msg != 0 );

	// Check whether the segment is in the receive window:
	ret = checkIncomingMessage( cd, carrier, msg );
	if( ret == OK ) {
	    if( msg->RST() ) {
		// Connection has been aborted. So blast outstanding msgs, and
		//   enter CLOSED state:
		// Delete outstanding messages:
		cd.outgoingBuffer()->reset();
		msg->del();
		cd.gotoState( CLOSED );
		ret = CONNECTION_RESET;
		return( CLOSED );
	    }
	    // Punt security/precedence....
	    if( msg->SYN() ) {
		// Error! Connection reset! (A SYN in the window is bad news.)
		sendAbort( cd, carrier );
		// Delete outstanding messages:
		id1 = cd.outstanding();
		while( id1 != -1 ) {
		    id2 = cd.outgoingBuffer()->previous( id1 );
		    cd.outgoingBuffer()->del( id1 );
		    id1 = id2;
		}
		// Note that we have just deleted the abort-request from
		//   the ARB, so it only will get sent once. In the (hopefully
		//   unlikely) event that it gets lost, the other end will
		//   time out and get bad news about trying to talk on a non-
		//   existent connection....

		msg->del();
		
		short unsigned int tmp1 = 0;

		cd.setSourcePort( tmp1 );
		cd.setDestinationPort( tmp1 );
		cd.setDestinationIPAddress( IPAddress() );	// i.e. 0.0.0.0
		cd.gotoState( CLOSED );
		ret = CONNECTION_RESET;
		return( CLOSED );
	    }
	    if( msg->ACK() ) {
		// Must be a retransmission of the other guy's FIN. So
		//   acknowledge it and reset the timestamp:
		reply = new TCPMessage( 20, 0, 20, 0, 0 );
		cd.pseudoheader()->setLength( 20 );
		sendMessage( cd, carrier, reply );
		cd.setTimestamp( now32() );
		msg->del();
		return( TIME_WAIT);
	    } // end of ack-check
	    if( msg->FIN() ) {
		// Retransmission...
		cd.setTimestamp( now32() );
		msg->del();
		return( TIME_WAIT);
	    }
	}
	msg->del();
	return( TIME_WAIT );	// No change.
}
