#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Johnny Zweig (zweig@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * ZootStateMachine.cc -- That which implementeth TCP
 *
 */

/*
	This file implements the TCP state-machine. That is, it implements
	all of the TCP protocol stuff that depends on what state the
	connection is (the conduit takes care of stupid stuff like calculating
	the checksum, setting up the IP header correctly, etc.). That's
	why it's so long.

	I moved a bunch of code into TCP*State.cc to speed up compilation
	when using parallel makes....

	See ZootStateMachine.h (in the Inlcudes directory) for more info.
 */

#include "Semaphore.h"
#include "Assert.h"
#include "OutputStream.h"
#include "ConduitTypes.h"
#include "Word.h"
#include "ZootConfig.h"
#include "ZootReturnCodes.h"
#include "ConduitProtocolIdentifiers.h"
#include "Assert.h"
#include "ZootAddress.h"
#include "ConduitMessage.h"
#include "IPChecksum.h"
#include "ZootClock.h"
#include "Conduit.h"
#include "ConduitMessageBuffer.h"
#include "ZootStateMachine.h"



#ifdef __GNUC__
#pragma implementation
#endif

//-----------------------------------------------------------------------------
//			     TCPConnectionDescriptor
//-----------------------------------------------------------------------------
TCPConnectionDescriptor::TCPConnectionDescriptor()
{
        //Debug << "TCPConnectionDescriptor::ctor()\n" << eor;

	_carrier = 0;	// Dunno who it is yet
	_luser = 0;	// Ditto

	_myIPAddress = TheLocalIPAddress; // bitwise copy should be fine
	_pseudoheader.setSourceAddress( _myIPAddress );
	_destIPAddress.equals( 0 );

	short unsigned int tmp1 = 0;

	_srcPort = tmp1;
	_destPort = tmp1;
	_current_state = CLOSED;
	_outstandingList = -1;
	_waitingList = -1;
	SND_UNA = tmp1;
	SND_NXT = tmp1;
	SND_WND = tmp1;
	SND_UP  = tmp1;
	SND_WL1 = tmp1;
	SND_WL2 = tmp1;
	SND_ISN = tmp1;
	RCV_NXT = tmp1;
	RCV_WND = tmp1;
	RCV_UP  = tmp1;
	RCV_ISN = tmp1;
	// _timestamp is only meaningful if the connection is in TIME-WAIT
	//   state so don't need to touch now.
	// _mutex initialized to 1
	_mutex = new MutexSemaphore;
	_outgoingBuffer = new AutoRetransmitBuffer( DEFAULT_ARB_SIZE );
	_waitBuffer = new TCPWaitBuffer( DEFAULT_WAIT_BUFFER_SIZE );
}
TCPConnectionDescriptor::~TCPConnectionDescriptor()
{
	// Nothing to do....
}
//-----------------------------------------------------------------------------
Return_Code
TCPConnectionDescriptor::openConnection()
{
	Return_Code ret;

	//Debug << "TCPCD::openConn() called.\n");
	_mutex->P();//HOCHWICHTIG!
	_current_state->openConnection( ret, *this, _carrier );
	_mutex->V();
	return( ret );
}
Return_Code
TCPConnectionDescriptor::closeConnection()
{
	Return_Code ret;

	//Debug << "TCPCD::closeConn() called.\n");
	_mutex->P();
	_current_state->closeConnection( ret, *this, _carrier );
	_mutex->V();
	return( ret );
}
Return_Code
TCPConnectionDescriptor::abortConnection()
{
	Return_Code ret;
	_mutex->P();
	_current_state->abortConnection( ret, *this, _carrier );
	_mutex->V();
	return( ret );
}
Return_Code
TCPConnectionDescriptor::processOutgoingMessage(TCPMessage * msg)
{
	Return_Code ret;
	Assert( _current_state != 0 );
	Assert( msg != 0 );
	Assert( _carrier != 0 );
	_mutex->P();
	_current_state->processOutgoingMessage( ret, *this, _carrier, msg );
	_mutex->V();
	return( ret );
}
Return_Code
TCPConnectionDescriptor::processIncomingMessage(TCPMessage * msg)
{
	Return_Code ret;

	Debug << "TCPCD::pIM(): processing msg:" << msg << "\n" << eor;
	Assert( 0 != msg );
	Assert( 0 != _carrier );
		
	_mutex->P();
Debug << "\tdelegating to current state\n" << eor;
	_current_state->processIncomingMessage( ret, *this, _carrier, msg );
	_mutex->V();
	Debug << "\tTCPCD::pIM: done\n" << eor;
	return( ret );
}

//-----------------------------------------------------------------------------
TCPConduit *
TCPConnectionDescriptor::carrier()
{
	return( _carrier );
}
Conduit *
TCPConnectionDescriptor::user()
{
	return ( _luser );
}
AutoRetransmitBuffer *
TCPConnectionDescriptor::outgoingBuffer()
{
	return( _outgoingBuffer );
}
TCPWaitBuffer *
TCPConnectionDescriptor::waitingBuffer()
{
	return( _waitBuffer );
}
IPAddress
TCPConnectionDescriptor::sourceIPAddress()
{
	return ( _myIPAddress );
}
IPAddress
TCPConnectionDescriptor::destinationIPAddress()
{
	return( _destIPAddress );
}
Word16
TCPConnectionDescriptor::sourcePort()
{
	return ( _srcPort );
}
Word16
TCPConnectionDescriptor::destinationPort()
{
	return ( _destPort );
}
TCPState *
TCPConnectionDescriptor::state()
{
	return ( _current_state );
}
MessageID
TCPConnectionDescriptor::outstanding()
{
	return ( _outstandingList );
}
MessageID
TCPConnectionDescriptor::waiting()
{
	return ( _waitingList );
}
Handle
TCPConnectionDescriptor::handle()
{
	return ( _handle );
}
TCPPseudoHeader	* 
TCPConnectionDescriptor::pseudoheader()
{
	return ( & _pseudoheader );
}
Word32
TCPConnectionDescriptor::SendUnacknowledged()
{
	return( SND_UNA );
}
Word32
TCPConnectionDescriptor::SendNext()
{
	return( SND_NXT );
}
Word16
TCPConnectionDescriptor::SendWindow()
{
	return( SND_WND );
}
Word16
TCPConnectionDescriptor::SendUrgentPointer()
{
	return( SND_UP );
}
Word32
TCPConnectionDescriptor::SendWL1()
{
	return( SND_WL1 );
}
Word32
TCPConnectionDescriptor::SendWL2()
{
	return( SND_WL2 );
}
Word32
TCPConnectionDescriptor::SendInitialSequenceNumber()
{
	return( SND_ISN );
}
Word32
TCPConnectionDescriptor::ReceiveNext()
{
	return( RCV_NXT );
}
Word16
TCPConnectionDescriptor::ReceiveWindow()
{
	return( RCV_WND );
}
Word16
TCPConnectionDescriptor::ReceiveUrgentPointer()
{
	return( RCV_UP );
}
Word32
TCPConnectionDescriptor::ReceiveInitialSequenceNumber()
{
	return( RCV_ISN );
}
Word32
TCPConnectionDescriptor::timestamp()
{
	return( _timestamp );
}

//-----------------------------------------------------------------------------
// (Have to keep pseudoheader consistent as things are changed!)
void
TCPConnectionDescriptor::setCarrier( TCPConduit * newcarrier )
{
	_carrier = newcarrier;
}
void
TCPConnectionDescriptor::setUser( Conduit * new_luser )
{
	_luser = new_luser;
}
void
TCPConnectionDescriptor::connect( Conduit * target )
{
	// Assume target is hip, and ignore return-codes from connects:
	_outgoingBuffer->connect( target );
}
void
TCPConnectionDescriptor::setLocalIPAddress( const IPAddress & newlocalipaddr )
{
	_myIPAddress = newlocalipaddr; // member-wise copy should be OK
	_pseudoheader.setSourceAddress( newlocalipaddr );
}
void
TCPConnectionDescriptor::setDestinationIPAddress( const IPAddress & newdestipaddr )
{
	//Debug << "TCPCD::setDestinationIPAddr to %x\n",
	//	newdestipaddr.val().val() );
	_destIPAddress = newdestipaddr;
	_pseudoheader.setDestinationAddress( newdestipaddr );
	//Debug << "TCPCD::setDestIPaddr new ipaddr = %x, (from %x )\n",
	//	_destIPAddress.val().val(),
	//	newdestipaddr.val().val() );
}
void
TCPConnectionDescriptor::setSourcePort( const Word16 & newsrcport )
{
	_srcPort = newsrcport;
}
void
TCPConnectionDescriptor::setDestinationPort( const Word16 & newdestport )
{
	_destPort = newdestport;
}
void
TCPConnectionDescriptor::gotoState( TCPState * newstate )
{
	_current_state = newstate;
}
void
TCPConnectionDescriptor::linkOutstanding( MessageID outstandingmsg )
{
	_outstandingList = outstandingmsg;
}
void
TCPConnectionDescriptor::linkWaiting( MessageID waitingmsg )
{
	_waitingList = waitingmsg;
}
void
TCPConnectionDescriptor::setHandle( Handle newhandle )
{
	_handle = newhandle;
}
void
TCPConnectionDescriptor::setSendUnacknowledged( const Word32 & newSND_UNA )
{
	SND_UNA.equals( newSND_UNA );
}
void
TCPConnectionDescriptor::setSendNext( const Word32 & newSND_NXT )
{
	SND_NXT.equals( newSND_NXT );
}
void
TCPConnectionDescriptor::setSendWindow( const Word16 & newSND_WND )
{
	SND_WND.equals( newSND_WND );
}
void
TCPConnectionDescriptor::setSendUrgentPointer( const Word16 & newSND_UP )
{
	SND_UP.equals( newSND_UP );
}
void
TCPConnectionDescriptor::setSendWL1( const Word32 & newSND_WL1 )
{
	SND_WL1.equals( newSND_WL1 );
}
void
TCPConnectionDescriptor::setSendWL2( const Word32 & newSND_WL2 )
{
	SND_WL2.equals( newSND_WL2 );
}
void
TCPConnectionDescriptor::setSendInitialSequenceNumber( const Word32 & newSND_ISN )
{
	SND_ISN.equals( newSND_ISN );
}
void
TCPConnectionDescriptor::setReceiveNext( const Word32 & newRCV_NXT )
{
	RCV_NXT.equals( newRCV_NXT );
}
void
TCPConnectionDescriptor::setReceiveWindow( const Word16 & newRCV_WND )
{
	RCV_WND.equals( newRCV_WND );
}
void
TCPConnectionDescriptor::setReceiveUrgentPointer( const Word16 & newRCV_UP )
{
	RCV_UP.equals( newRCV_UP );
}
void
TCPConnectionDescriptor::setReceiveInitialSequenceNumber( const Word32 & newRCV_ISN )
{
	RCV_ISN.equals( newRCV_ISN );
}
void
TCPConnectionDescriptor::setTimestamp( const Word32 & newtimestamp )
{
	_timestamp = newtimestamp;
}

//-----------------------------------------------------------------------------
//				     TCPState: 
//-----------------------------------------------------------------------------
TCPState::TCPState()
{
	// Nothing to initialize....
}
TCPState::~TCPState()
{
	// Nothing to delete....
}
//-----------------------------------------------------------------------------
// The idea behind sendConnectRequest(), sendAbort(), sendMessage() and
//   sendCloseRequest() is to embody the state-independent aspects of each of
//   these operations. That is, scribbling on the message and calculating a
//   a checksum and whatever. Each state may need to do slightly different
//   kinds of setup before calling these functions.
Return_Code
TCPState::sendConnectRequest( TCPConnectionDescriptor & cd,
			      TCPConduit * carrier )
{
	IPAddress * connectRequestAddr;
	TCPMessage * connectRequest;
	MessageID tmp;
	Return_Code ret;
	octet * finger;

	// Before an Active-Open, need to set up a few things...:
	cd.setSendInitialSequenceNumber( now32() );
	cd.setSendUnacknowledged( cd.SendInitialSequenceNumber() );
	cd.setSendNext( cd.SendInitialSequenceNumber().val() + 1 );
	// The pseudoheader needs to be set up correctly:
	cd.pseudoheader()->setSourceAddress( cd.sourceIPAddress() );
	cd.pseudoheader()->setDestinationAddress( cd.destinationIPAddress() );
	cd.pseudoheader()->setLength( 20 );	// SYN-segment carries no data

	connectRequestAddr = new IPAddress();
	connectRequestAddr->equals(  cd.destinationIPAddress() );
	connectRequest = new TCPMessage(   20,	// IP-header size
					   0,	// IP data size
					   20,	// TCP header size
					   0,	// TCP data size
					   0 );	// No link
	connectRequest->setSourceAddress( TheLocalIPAddress.val() );
		// recall that IPAddress::val() returns a Word32...
	//connectRequest->setDestinationAddress(
		// cd.destinationIPAddress().val() );
		// Unnecessary, since IP-conduit will reset it from cRAddr
	connectRequest->setSourcePort( cd.sourcePort() );
	connectRequest->setDestinationPort(   cd.destinationPort() );
	connectRequest->setSequenceNumber( cd.SendInitialSequenceNumber() );
	connectRequest->setSYN( TRUE );
	// Don't have to worry about ack field/flag for a connection-req.
	connectRequest->setWindow( cd.ReceiveWindow() );
	// Don't have to worry about urgent field/flag for a connection-req...
	// Calculate Checksum:
	//   (need to add TCP header chksum to pseudoheader chksum -- no data)
	finger = connectRequest->header();
	//Debug << "Dump of connect-request header at %x.\n",
	//	finger );
	//for(i=0;i<20;i++) Debug << "%x", *(finger+i) );
	connectRequest->setChecksum( combineIPChecksums(
		IPChecksum( connectRequest->header(), 20 ),
		cd.pseudoheader()->checksum() ) );

	//Debug << "\nTCPState::sendConnectRequest: Connect-Req Ready:\n");
	//Debug << "\t(ConnReq addr: %x, ConnReqAddr addr: %x\n",
	//	connectRequest, connectRequestAddr );
	//connectRequest->IPmsg()->print();
	//connectRequest->print();

	// "BYE-BYE, little connection-request! Good luck...."
	//Debug << "\t(IP-msg ptr: %x, IP-addr-ptr: %x)\n",
	//		connectRequest->IPmsg(),connectRequestAddr);
	tmp = cd.outstanding();
	cd.outgoingBuffer()->grab();
	cd.linkOutstanding( cd.outgoingBuffer()->
		insert( ret, connectRequest->IPmsg(), connectRequestAddr ) );
	cd.outgoingBuffer()->release();
	// Since this has got to be the first message, there's no need to
	//   link connectRequest to whatever's already there; in general,
	//   inserting a message into the re-xmit-buffer requires linking it
	//   into the chain.
	// (Note the slick way ret will return whatever the ARB returns....)
	if( ret == BUFFER_FULL ) {
	    // restore old link (since the buffer just ignored 
	    //   this insert-attempt):
	    cd.linkOutstanding( tmp );
	    delete connectRequest->IPmsg();
	}

	return( ret );
}
Return_Code
TCPState::sendAbort( TCPConnectionDescriptor & cd,
			TCPConduit * carrier )
{
	IPAddress * connectionResetAddr;
	TCPMessage * connectionReset;
	MessageID tmp;
	Return_Code ret;

	cd.pseudoheader()->setLength( 20 );

	connectionResetAddr = new IPAddress();
	connectionResetAddr->equals( cd.destinationIPAddress().val() );
	connectionReset = new TCPMessage(  20,	// IP-header size
					   0,	// IP data size
					   20,	// TCP header size
					   0,	// TCP data size
					   0 );	// No link
	connectionReset->setSourceAddress(
		TheLocalIPAddress.val() );
	connectionReset->setDestinationAddress(
		cd.destinationIPAddress().val() );
	connectionReset->setSourcePort( cd.sourcePort() );
	connectionReset->setDestinationPort( cd.destinationPort() );
	connectionReset->setSequenceNumber( cd.SendNext() );
	connectionReset->setRST( TRUE );

	// Calculate Checksum:
	cd.pseudoheader()->doChecksum( connectionReset );

	//Debug << "TCPState::sendAbort: Connection-Reset Ready:\n");
	//connectionReset->print();

	tmp = cd.outstanding();
	cd.outgoingBuffer()->grab();
	cd.linkOutstanding( cd.outgoingBuffer()->
		insert( ret, connectionReset->IPmsg(), connectionResetAddr ) );
	cd.outgoingBuffer()->release();
	if( ret == BUFFER_FULL ) {
	    // restore old link (since the buffer just ignored 
	    //   this insert-attempt):
	    cd.linkOutstanding( tmp );
	    delete connectionReset->IPmsg();
	}

	return( ret );
}
Return_Code
TCPState::sendAbort( TCPConnectionDescriptor & cd,
			TCPConduit * carrier, const Word32 & seqnum )
{
	IPAddress * connectionResetAddr;
	TCPMessage * connectionReset;
	MessageID tmp;
	Return_Code ret;

	cd.pseudoheader()->setLength( 20 );

	connectionResetAddr = new IPAddress;
	connectionResetAddr->equals( cd.destinationIPAddress().val() );
	connectionReset = new TCPMessage(  20,	// IP-header size
					   0,	// IP data size
					   20,	// TCP header size
					   0,	// TCP data size
					   0 );	// No link
	connectionReset->setSourceAddress(
		TheLocalIPAddress.val() );
	connectionReset->setDestinationAddress(
		cd.destinationIPAddress().val() );
	connectionReset->setSourcePort( cd.sourcePort() );
	connectionReset->setDestinationPort( cd.destinationPort() );
	connectionReset->setSequenceNumber( seqnum );
	connectionReset->setRST( TRUE );

	// Calculate Checksum:
	cd.pseudoheader()->doChecksum( connectionReset );

	//Debug << "TCPState::sendAbort(2): Connection-Reset Ready:\n");
	//connectionReset->print();

	tmp = cd.outstanding();
	cd.outgoingBuffer()->grab();
	cd.linkOutstanding( cd.outgoingBuffer()->
		insert( ret, connectionReset->IPmsg(), connectionResetAddr ) );
	cd.outgoingBuffer()->release();
	if( ret == BUFFER_FULL ) {
	    // restore old link (since the buffer just ignored 
	    //   this insert-attempt):
	    cd.linkOutstanding( tmp );
	    delete connectionReset->IPmsg();
	}

	return( ret );
}
Return_Code
TCPState::sendAbort( TCPConnectionDescriptor & cd, TCPConduit * carrier,
			const Word32 & seqnum, const Word32 & acknum )
{
	IPAddress * connectionResetAddr;
	TCPMessage * connectionReset;
	MessageID tmp;
	Return_Code ret;

	cd.pseudoheader()->setLength( 20 );

	connectionResetAddr = new IPAddress;
	connectionResetAddr->equals( cd.destinationIPAddress().val() );
	connectionReset = new TCPMessage(  20,	// IP-header size
					   0,	// IP data size
					   20,	// TCP header size
					   0,	// TCP data size
					   0 );	// No link
	connectionReset->setSourceAddress(
		TheLocalIPAddress.val() );
	connectionReset->setDestinationAddress(
		cd.destinationIPAddress().val() );
	connectionReset->setSourcePort( cd.sourcePort() );
	connectionReset->setDestinationPort( cd.destinationPort() );
	connectionReset->setSequenceNumber( seqnum );
	connectionReset->setAckNumber( acknum );
	connectionReset->setRST( TRUE );
	connectionReset->setACK( TRUE );

	// Calculate Checksum:
	cd.pseudoheader()->doChecksum( connectionReset );

	//Debug << "TCPState::sendAbort(3): Connection-Reset Ready:\n");
	//connectionReset->print();

	tmp = cd.outstanding();
	cd.outgoingBuffer()->grab();
	cd.linkOutstanding( cd.outgoingBuffer()->
		insert( ret, connectionReset->IPmsg(), connectionResetAddr ) );
	cd.outgoingBuffer()->release();
	if( ret == BUFFER_FULL ) {
	    // restore old link (since the buffer just ignored 
	    //   this insert-attempt):
	    cd.linkOutstanding( tmp );
	    delete connectionReset->IPmsg();
	}

	return( ret );
}
Return_Code
TCPState::sendMessage( TCPConnectionDescriptor & cd,
			TCPConduit * carrier, TCPMessage * msg)
{
	Assert( carrier != 0 );
	Assert( msg != 0 );

	return( sendMessage( cd, carrier, msg, cd.SendNext() ) );
}
Return_Code
TCPState::sendMessage( TCPConnectionDescriptor & cd, TCPConduit * carrier,
			TCPMessage * msg, const Word32 & seqnum )
{
	IPAddress * msgAddr;
	MessageID tmp;
	Return_Code ret;

	Assert( carrier != 0 );
	Assert( msg != 0 );

#define put(x) Debug << x << eor;
	Debug << "TCPState::sendMessage called ...\n" << eor;
	// Just fills in the message's fields and stuffs it into the ARB.
	//   >>>Assumes msg has an IP-message to live in already!<<<
	msgAddr = new IPAddress();
	msgAddr->equals( cd.destinationIPAddress().val() );
put("\t$")
	msg->setSourceAddress( TheLocalIPAddress.val() );
	msg->setDestinationAddress( cd.destinationIPAddress().val() );
	msg->setSourcePort( cd.sourcePort() );
	msg->setDestinationPort( cd.destinationPort() );
	msg->setSequenceNumber( seqnum );
put("\t%")
	// ACK-field:
	msg->setAckNumber( cd.ReceiveNext() );
	msg->setACK( TRUE );
	msg->setOffset( (nibble)( msg->headerLength() / 4 ) );
	msg->setWindow( cd.ReceiveWindow() );
	// (Not going to worry about urgent field/flag....)

	// Calculate Checksum:
	//Debug << "sM: pseudo-len %d msg-len %d\n", cd.pseudoheader()->length().val(), msg->length() );
	//Assert( cd.pseudoheader()->length().val() == msg->length() );

put("\t^")
	cd.pseudoheader()->doChecksum( msg );
put("\t&")

	//Debug << "TCPState::sendMessage: Message Ready. Here it isn't...\n");
	//msg->print();

	tmp = cd.outstanding();
	cd.outgoingBuffer()->grab();
put("\t*")
	Assert( 0 != msg->IPmsg() );
	cd.linkOutstanding( cd.outgoingBuffer()->
		insert( ret, msg->IPmsg(), msgAddr ) );
	cd.outgoingBuffer()->release();
	if( ret != BUFFER_FULL ) {
	    // Update SND-NXT:
	    cd.setSendNext( cd.SendNext().val() + msg->dataLength() );
put("\t@")
	}
	else {
	    // restore old link (since the buffer just ignored 
	    //   this insert-attempt):
	    cd.linkOutstanding( tmp );
	    // Note that BUFFER_FULL will be returned so the user will know
	    //   to try again later....
put("\t!")
	}
	return( ret );
}
Return_Code
TCPState::sendCloseRequest( TCPConnectionDescriptor & cd,
			TCPConduit * carrier )
{
	IPAddress * closeRequestAddr;
	TCPMessage * closeRequest;
	MessageID tmp;
	Return_Code ret;

	Assert( carrier != 0 );

	closeRequestAddr = new IPAddress();
	closeRequestAddr->equals(  cd.destinationIPAddress().val() );
	closeRequest = new TCPMessage(     20,	// IP-header size
					   0,	// IP data size
					   20,	// TCP header size
					   0,	// TCP data size
					   0 );	// No link
	closeRequest->setSourceAddress(
		TheLocalIPAddress.val() );
		// recall that IPAddress::val() returns a Word32...


	closeRequest->setDestinationAddress( cd.destinationIPAddress().val() );
	closeRequest->setSourcePort( cd.sourcePort() );
	closeRequest->setDestinationPort(   cd.destinationPort() );
	closeRequest->setSequenceNumber( cd.SendNext() );
	closeRequest->setFIN( TRUE );
	// Since the FIN takes up one sequence number (to push it past any
	//   that correspond to actual data), SND_NXT must be incremented:
	cd.setSendNext( cd.SendNext().val() + 1 );
	closeRequest->setAckNumber( cd.ReceiveNext() );
	closeRequest->setACK( TRUE );
	closeRequest->setWindow( cd.ReceiveWindow() );

	// Calculate Checksum:
	cd.pseudoheader()->doChecksum( closeRequest );

	//Debug << "TCPState::sendCloseRequest: Connection-Request Ready:\n");
	//closeRequest->print();

	//Debug << "\t(IP-msg ptr: %x, IP-addr-ptr: %x)\n",
	//		closeRequest->IPmsg(),closeRequestAddr);
	tmp = cd.outstanding();
	cd.outgoingBuffer()->grab();
	cd.linkOutstanding( cd.outgoingBuffer()->
		insert( ret, closeRequest->IPmsg(), closeRequestAddr ) );
	cd.outgoingBuffer()->release();
	if( ret == BUFFER_FULL ) {
	    // restore old link (since the buffer just ignored 
	    //   this insert-attempt):
	    cd.linkOutstanding( tmp );
	    delete closeRequest->IPmsg();
	}


	return( ret );
}
Return_Code
TCPState::checkIncomingMessage( TCPConnectionDescriptor & cd,
			TCPConduit * carrier, TCPMessage * msg)
{ // for ESTABLISHED, etc....
	Return_Code ret;
	TCPMessage * reply;

	Assert( carrier != 0 );
	Assert( msg != 0 );

	if( cd.ReceiveWindow().val() == 0 ) {
	    if( msg->SequenceNumber().val() == cd.ReceiveNext().val() ) {
		ret = OK;
	    }
	    else {
		ret = ERROR;
	    }
	}
	else {
	    if( msg->dataLength() == 0 ) {
		// No data, but might have control-info -- so we should
		//   accept it if it is anywhere within the window (if it is
		//   totally out of the window, accepting would be the Wrong
		//   Thing, since it might be old/bogus/retransmitted/etc/.)
		if( isBetween(  cd.ReceiveNext().val(),
				msg->SequenceNumber().val(),
				( cd.ReceiveNext().val() +
					cd.ReceiveWindow().val() ) ) ) {
		    ret = OK;
		}
		else {
		    ret = ERROR;
		}
	    }
	    else {
		// Since this segment carries data, and I'm not buffering
		//   out-of-sequence segments to process later, it will
		//   only be acceptable if it is short enough to fit in
		//   the window, and starts at the next expected sequence
		//   number:
		if( msg->SequenceNumber().val() == cd.ReceiveNext().val() &&
		    isBetween(  cd.ReceiveNext().val(),
				( msg->SequenceNumber().val() +
					msg->dataLength() - 1 ),
				( cd.ReceiveNext().val() +
					cd.ReceiveWindow().val() ) ) ) {
		    ret = OK;
		}
		else {
		    ret = ERROR;
		}
	    }
	}
	if( ret == ERROR ) {
	    // Need to send an ACK to try to get expected data re-sent:
	    //   (I'll use the standard trick of sending a dataless message)
	    reply = new TCPMessage( 20, 0, 20, 0, 0 );
	    cd.pseudoheader()->setLength( 20 );
	    sendMessage( cd, carrier, reply );
	    //delete msg->IPmsg(); Wrongo! Let the state delete the message!
	}

	return( ret );
}
void
TCPState::acknowledge( TCPConnectionDescriptor & cd, TCPConduit * carrier,
			const Word32 & acknum )
{
	// Just chases down the CD's outstanding message chain, and deletes
	//   any messages that just got acknowledged.
	AutoRetransmitBuffer * arb;
	MessageID id1, id2;
	Boolean DontKill = TRUE;
	TCPMessage * msg;

	Debug << "\tAck:  ";

	Assert( carrier != 0 );

	/*
	 * THIS IS WRONG!!! Consider the case of a 1-way receiving connection:
	 * with each incoming packet the ack-number will be the same, but
	 * there may be new messages in the retransmit queue that need to get
	 * thrown away.  Sigh. You can't win them all.  
	 *
	 * Basically, this is a pain in the ass to deal with since control
	 * information (ack's) is passed out-of-band so there is no sequence
	 * number to keep track of control bytes. Maybe we should make TCP
	 * even more complicated and have two sequence-numbers for data and
	 * control separately. Naw. That'd be dumb...
	 *
	if( acknum.val() == cd.SendUnacknowledged().val() ) {
		// Duplicate of last ACK received....
	    Debug << "\tack: ACK == SND_UNA.\n" << eor;
	    return;
	}
	 *
	 *  END WRONG GARBAGE.
	 */

	// Keep in mind that SND_UNA --> acknum is the window of unack'ed
	// data, so that's why both numbers appear in the following
	// contorted comparisons.

	id1 = cd.outstanding();
	if( id1 == -1 ) {
		// No outstanding messages to acknowledge!
	    Debug << "\tack: nothing to acknowledge.\n" << eor;
	    return;
	}
	arb = cd.outgoingBuffer();
	Assert( arb != 0 );

	arb->grab();

	id2 = arb->previous( id1 );	// id2 is the next older message

	// Skip past messages containing unacknowledged data:
	while( id1 != -1 && DontKill ) {

Debug << "Acking msg(ID=0x" << asHex( id1 ) << ")...\n" << eor;

	    Assert( 0 != arb->lookAt( id1 ) );
	    msg = (TCPMessage *) arb->lookAt( id1 )->link();
		// (Assumes the link is a TCP Message -- it has to be,
		//   dunnit? Since we put in everything that's in the ARB
		//   in there in the first place -- they are all IP msgs....)

Debug << "Ack: arb->lookAt(" << asHex(id1) << ")=" <<
asAddress( arb->lookAt(id1) ) << " linked to " << asAddress(msg)
<< "\n" << eor;
	    Assert( 0 != msg );
	    Assert( msg->protocol() == TCP );

	    if( isBetween( cd.SendUnacknowledged().val(),
			   msg->SequenceNumber().val(),
			   acknum.val() ) ) {
		// Some of this message's data (and certainly all of the
		//   next one in the chain) have been acknowledged.
		if( isBetween( msg->SequenceNumber().val() + ( msg->length() -
					msg->headerLength() ),
				acknum.val(),
				cd.SendNext().val() ) ) {
		    // This whole message has been ack'ed so id1 is the first
		    //   message to be killed -- the seq-num of the next
		    //   message sent would be acked.
		    // Nota bene that msg->length()-msg->headerLength() is
		    //   not necessarily the same as msg->dataLength(),
		    //   since there may be data in a linked message....

		    DontKill = FALSE;
		    //Debug << "\t...about to kill msg %d.\n", id1 );
		}
		else {
		    // Advance pointer past message not all of which was ack'ed
		    //   so id1 will point to what needs to be deleted
		    id1 = id2;
		    id2 = arb->previous( id1 );
		    DontKill = FALSE;
		    //Debug << "\t...about to kill msg %d.\n", id1 );
		}
	    }
	    else {

		//Debug << "\t...skipping msg %d.\n", id1 );

		// Try the next message in the chain:
		id1 = id2;
		id2 = arb->previous( id1 );
	    }
	}

	// If acknowledging all outstanding messages, need to let CD know:
	if( id1 != -1 && id1 == cd.outstanding() ) {
	    cd.linkOutstanding( -1 );
	}

	// Kill off anything that needs to go:
	while( id1 != -1 && DontKill == FALSE) {
	    Debug << "Deleting msg " << id1 << "\n" << eor;
	    if( OK != arb->del( id1 ) )
		Debug << "\tack:NO_SUCH_MESSAGE!\n" << eor;
	    id1 = id2;
	    id2 = arb->previous( id1 );
	}
	Debug << "_" << eor;

	arb->release();
}
//-----------------------------------------------------------------------------
TCPState *
TCPState::openConnection( Return_Code & ret,
				TCPConnectionDescriptor & cd,
				TCPConduit * carrier )
{
	ret = ERROR;	// In most states, it doesn't make sense to open a
	return( 0 );	//   new connection, so they inherit this method
}
TCPState *
TCPState::closeConnection( Return_Code & ret,
				TCPConnectionDescriptor & cd,
				TCPConduit * carrier )
{
	ret = ERROR;	// Similarly, in some states it doesn't make sense to
	return( 0 );	//   try to close the connection
}
TCPState *
TCPState::abortConnection( Return_Code & ret,
				TCPConnectionDescriptor & cd,
				TCPConduit * carrier )
{
	ret = ERROR;	// Actually, it makes sense to abort a connection
	return( 0 );	//   from almost any state, if you want to; everyone
			//   will reimplement this method, just about
}
TCPState *
TCPState::processOutgoingMessage( Return_Code & ret,
				TCPConnectionDescriptor & cd,
				TCPConduit * carrier, TCPMessage * msg )
{
	ret = ERROR;
	return( 0 );	// Some states can't process outgoing messages,
			//   so they inherit this method.
}
TCPState *
TCPState::processIncomingMessage( Return_Code & ret,
				TCPConnectionDescriptor & cd,
				TCPConduit * carrier, TCPMessage * msg )
{
	ret = ERROR;
	return( 0 );	// >>>> ALL STATES need to be able to process
			//      incoming messages -- even though some do
			//      nothing more than send a reset in response.
			// (Of course, in most cases if a connection is
			//   nonexistent, the TCP conduit won't even find a
			//   CD that matches and will drop the packet; tough
			//   luck for the other side....)
}

//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// Global pointers to the (single) instances of the class State objects:
//-----------------------------------------------------------------------------
ClosedState		TheCLOSEDState;
ClosedState       *	CLOSED = & TheCLOSEDState;
ListenState		TheLISTENState;
ListenState       *	LISTEN = & TheLISTENState;
Syn_SentState		TheSYN_SENTState;
Syn_SentState     *	SYN_SENT = & TheSYN_SENTState;
Syn_ReceivedState	TheSYN_RECEIVEDState;
Syn_ReceivedState *	SYN_RECEIVED = & TheSYN_RECEIVEDState;
EstablishedState	TheESTABLISHEDState;
EstablishedState  *	ESTABLISHED = & TheESTABLISHEDState;
Fin_Wait_1State		TheFIN_WAIT_1State;
Fin_Wait_1State   *	FIN_WAIT_1 = & TheFIN_WAIT_1State;
Fin_Wait_2State		TheFIN_WAIT_2State;
Fin_Wait_2State   *	FIN_WAIT_2 = & TheFIN_WAIT_2State;
Close_WaitState		TheCLOSE_WAITState;
Close_WaitState   *	CLOSE_WAIT = & TheCLOSE_WAITState;
ClosingState		TheCLOSINGState;
ClosingState      *	CLOSING = & TheCLOSINGState;
Last_AckState		TheLAST_ACKState;
Last_AckState     *	LAST_ACK = & TheLAST_ACKState;
Time_WaitState		TheTIME_WAITState;
Time_WaitState    *	TIME_WAIT = & TheTIME_WAITState;

//--------------------------------------+--------------------------------------
