#ifdef __GNUC__
#pragma implementation
#endif

#include "ARMTranslation.h"
#include "MemoryBlockAllocator.h"
#include "PROCESSOR.h"

extern int TwoLevelPageTableClassInitialized;

void ARMTranslation::init( unsigned int tableAreaSize, VirtualMemoryRange * systemMemoryMap ) {
    Debug << "ARMTranslation::init(): ";

    /*
     * Make sure this is the first time we're called.
     */
    Assert( ! TwoLevelPageTableClassInitialized );

    kernelVirtualMemoryMap = systemMemoryMap;

    /* this only allows us room for three domains; we should figure out
     * how to work around this in the future */
    tableMemoryAllocator = new MemoryBlockAllocator(
        tableAreaSize * 3, PageTableSize );

    TwoLevelPageTableClassInitialized = 1;
}

ARMTranslation::ARMTranslation(ARMTranslation *translation)
    : TwoLevelPageTable(translation)
{
    Debug << "ARMTranslation::ARMTranslation(): ";
    Debug << asAddress(this) << "\n" << eor;
    Assert( this != 0 );
}

ARMTranslation::~ARMTranslation() {

}

POINTER_TABLE_ENTRY *
ARMTranslation::allocatePointerTable()
{
    Assert( tableMemoryAllocator != 0 );
    POINTER_TABLE_ENTRY * table = (POINTER_TABLE_ENTRY *)
    tableMemoryAllocator->allocateAligned( PointerTableSize, PointerTableAlign );

    for( int i = 0; i < PointerTableEntries; i++ ) {
        ((int *)table)[i] = 0;
    }

    return( table );
}

// Francis - We are using coarse L2 page tables
PAGE_TABLE_ENTRY *
ARMTranslation::allocatePageTable()
{
    PAGE_TABLE_ENTRY * table = (PAGE_TABLE_ENTRY *)
    tableMemoryAllocator->allocateAligned( PageTableSize, PageTableAlign );

    for( int i = 0; i < PageTableEntries; i++ ) {
        ((int *)table)[i] = 0;
    }

    return( table );
}

/**
 * Change the status of a page.
 * @param vaddr
 * @param nbytes
 * @param clear
 * @param set
 * @return Unknown.
 */
int ARMTranslation::basicChangeStatus(const char * vaddr,
                                      unsigned int nbytes, AddressTranslationStatus clear,
                                      AddressTranslationStatus set)
{
    /* TODO: this should do something */
    return 0;
}
