/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Bjorn A. Helgaas (helgaas@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * MC68030ContextSwitching.S: assembly language context switching routines.
 */

#include "MC68030ContextOffsets.h"

/*
 * Process *
 * MC68030ApplicationContext::checkpoint( ProcessorContext * )
 *
 * The application context checkpoint/restore functions save/restore
 * the floating point registers and the user stack pointer as well.
 *
 * On Entry:
 *	top-of-stack = pointer to the context to save in.
 */
	.globl	checkpoint__25MC68030ApplicationContext
checkpoint__25MC68030ApplicationContext:
	movel	%sp@(4),%a0		/* get "this" pointer */
	movec	%usp,%d0		/* save user stack pointer */
	movel	%d0,%a0@(USERSTACKPOINTER_MC68030APPLICATIONCONTEXT)

	/*
	 * Call the parent method to save the rest.
	 */
	
	.globl	checkpoint__21MC68030GenericContext
checkpoint__21MC68030GenericContext:
	movel	%sp@(4),%a0		/* get "this" pointer */
	moveml	%d2-%d7/%a2-%a6,%a0@(REGISTERS_MC68030GENERICCONTEXT)
	lea	%sp@(4),%a1		/* sp after return address popped */
	movel	%a1,%a0@(SUPERVISORSTACKPOINTER_MC68030GENERICCONTEXT)
	movel	%sp@,%a0@(PC_MC68030GENERICCONTEXT)	/* return address */

	clrl	%d0			/* return 0 the first time */
	rts

	.globl	basicRestore__25MC68030ApplicationContextP7Process
basicRestore__25MC68030ApplicationContextP7Process:
	movel	%sp@(4),%a0		/* get "this" pointer */
	movel	%a0@(USERSTACKPOINTER_MC68030APPLICATIONCONTEXT),%a1
	movec	%a1,%usp

	.globl	basicRestore__21MC68030GenericContextP7Process
basicRestore__21MC68030GenericContextP7Process:
	movel	%sp@(4),%a0		/* get "this" pointer */
	moveml	%a0@(REGISTERS_MC68030GENERICCONTEXT),%d2-%d7/%a2-%a6
	movel	%sp@(8),%d0		/* parameter is Process *  */
					/* note that return value in d0 */
					/* must be Process *, which is  */
					/* guaranteed to be non-zero    */
	movel	%a0@(SUPERVISORSTACKPOINTER_MC68030GENERICCONTEXT),%sp
	movel	%a0@(PC_MC68030GENERICCONTEXT),%a0
	jmp	%a0@			/* "checkpoint()" returns again! */


/*
 * void
 * MC68030SystemContext::restoreFromInitialContext( Process * prev )
 */
	.globl	restoreFromInitialContext__20MC68030SystemContextP7Process
restoreFromInitialContext__20MC68030SystemContextP7Process:
	movel	%sp@(4),%a0		/* "this" pointer */
	movel	%a0@(SUPERVISORSTACKPOINTER_MC68030GENERICCONTEXT),%a1

	movel	%sp@(8),%a1@-		/* push Process * on new stack */
	movel	%sp@(4),%a1@-		/* push "this" pointer on new stack */
	movel	%a1,%sp			/* switch to new stack */
	jsr	initialRestoreAssist__20MC68030SystemContextP7Process


/*
 * void
 * MC68030ApplicationContext::restoreFromInitialContext( Process * prev )
 */
	.globl	restoreFromInitialContext__25MC68030ApplicationContextP7Process
restoreFromInitialContext__25MC68030ApplicationContextP7Process:
	movel	%sp@(4),%a0		/* "this" pointer */
	movel	%a0@(SUPERVISORSTACKPOINTER_MC68030GENERICCONTEXT),%a0
	movel	%sp@(8),%a0@-		/* push Process * on new stack */
	movel	%sp@(4),%a0@-		/* push "this" pointer on new stack */
	movel	%a0,%sp			/* switch to new stack */

	/*
	 * First do what the Germ requires of us.
	 */
	jsr	restoreAssist__16ProcessorContextP7Process
	addql	#8,%sp

	/*
	 * Set up the arguments for the calling convention the Choices
	 * application library startoff routine expects.
	 */
	movel	%sp@+,%d0		/* arg0 */
	movel	%sp@+,%d1		/* arg1 */
	movel	%sp@+,%d2		/* usp  */

	/*
	 * Start the application running for real by simultaneously loading
	 * the sr from the interrupt stack and jumping to the entry
	 * point (also on the stack).  The new sr will determine which
	 * stack pointer is active after this instruction.
	 * Our constructor put all this info on the system stack for us.
	 */
	rte

/*
 * Function called upon interrupt or trap occuring on a MC68030.
 * Upon taking an interrupt, the 68030 automatically updates the
 * interrupt mask level to the level of the interrupt being serviced.
 * Thus, interrupts are not fully disabled while an interrupt is
 * being processed, but only higher-priority interrupts will be
 * recognized.
 */
	.globl	interruptAssist__10MC68030CPU
interruptAssist__10MC68030CPU:
	moveml	%d0-%d1/%a0-%a1,%sp@-

	movew	%sp@(VECTOR_STACKFRAME),%d0 /* get interrupt vector offset */
	andil	#0x0fff,%d0		/* mask off format identifier */

	movec	%vbr,%a0		/* find interrupt table at */
	addl	#1024,%a0		/* vbr + nInterrupts * sizeof( PFV ) */
	addaw	%d0,%a0			/* add vector offset into table */
	movel	%a0@,%d0		/* get Interrupt * */

	movel	%sp,%a0
	movel	%a0,%sp@-		/* push pointer to saved context */
	movel	%d0,%sp@-		/* push Interrupt "this" pointer */
	jsr	raise__9InterruptPc
	addql	#8,%sp			/* discard raise() parameters */

	moveml	%sp@+,%d0-%d1/%a0-%a1
	rte


	.globl	debugInterruptAssist__10MC68030CPU
debugInterruptAssist__10MC68030CPU:
	oriw	#0x0700,%sr		/* disable interrupts */

debugGetLock:
	tas	debugLock
	beq	debugGotLock
debugWaitForLock:
	tstb	debugLock
	beq	debugGetLock
	jmp	debugWaitForLock

debugGotLock:
	moveml	%d0-%d7/%a0-%a6,registers
	lea	registers,%a5
	clrl	%d0
        movew	%sp@,%d0			/* sr before interrupt */
	movew	%d0,%a5@(66)
	movel	%sp@(2),%a5@(68)		/* pc before interrupt */
        andiw	#0x2000,%d0
        beq	debugUserMode

debugSupervisorMode:

	/*
	 * We were in supervisor mode before taking the interrupt, so
	 * the interrupt stack frame was saved on our stack (note that
	 * we use only the interrupt stack, not the actual supervisor
	 * stack.)  Therefore, we compute the value of the stack
	 * pointer BEFORE the interrupt frame was saved.
	 */

	clrl	%d0
	movew	%sp@(6),%d0		/* interrupt format & vector offset */
        andiw	#0xf000,%d0		/* mask off vector offset */
        rolw	#6,%d0			/* rotate format */
        lea	interruptStackFrameSize,%a1
        addl	%d0,%a1
	movel	%a1@,%d0		/* size of interrupt frame (in bytes) */
	movel	%sp,%a0
	addl	%d0,%a0			/* compute sp before interrupt */
        movel	%a0,%a5@(60)
        bra	debugSPSaved

debugUserMode:

	/*
	 * We were in user mode before the interrupt, so the stack frame
	 * was saved on the interrupt stack, NOT on the user stack.
	 * Therefore, we can just save the user stack pointer since
	 * it hasn't changed.
	 */

	movec	%usp,%a1
        movel	%a1,%a5@(60)
debugSPSaved:
	movew	%sp@(6),%d0		/* get vector offset */
	andl	#0xfff,%d0
	asrl	#2,%d0			/* compute vector number */

	movel	%sp,debugSupervisorSP
	movel	debugStackPointer,%sp	/* switch to special debug stack */

	pmove	%tc,_tc
	pmove	%srp,_srp
	pmove	%crp,_crp
	movel	%d0,%sp@-
	movec	%vbr,%d0
	movel	%d0,_vbr
	movel	%sp@+,%d0

	movel	%d0,%sp@-
	jsr	processCommands__Fi	/* this should never return */

	trap	#0xf


	.globl	debugReturnFromInterrupt__Fv
debugReturnFromInterrupt__Fv:
	movel	debugSupervisorSP,%sp
	lea	registers,%a0
	movew	%a0@(66),%d0		/* set up sr in case changed */
	movew	%d0,%sp@
	movel	%a0@(68),%d0		/* set up pc in case changed */
	movel	%d0,%sp@(2)
	moveml	registers,%d0-%d7/%a0-%a6
	clrb	debugLock		/* release lock */
	rte

/*
 * This function is here so we can start debugging before a CPU is
 * constructed.
 */
	.globl	debugBreakpoint__Fv
debugBreakpoint__Fv:
	trap	#0xf
	rts

	.data
debugLock:
	.long	0


#if 0
	movel	d0,_d0
	movel	d1,_d1
	movel	d2,_d2
	movel	d3,_d3
	movel	d4,_d4
	movel	d5,_d5
	movel	d6,_d6
	movel	d7,_d7
	movel	a0,_a0
	movel	a1,_a1
	movel	a2,_a2
	movel	a3,_a3
	movel	a4,_a4
	movel	a5,_a5
	movel	a6,_a6

	movec	vbr,d0;		movel	d0,_vbr
	movec	cacr,d0;	movel	d0,_cacr
	movec	caar,d0;	movel	d0,_caar
	pmove	tc,_tc

	lea	debugVectorTable,a5
	movec	a5,vbr

#endif
