#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Bjorn A. Helgaas (helgaas@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * MC68030GnuDebugger.cc:
 */

#include "MC68030GnuDebugger.h"
#include "InputStream.h"
#include "MC68030Interrupts.h"
#include "MC68030Vectors.h"
#include "MacintoshCPU.h"
#include "MacintoshInterrupts.h"
#include "OutputStream.h"
#include "String.h"
#include "Util.h"


#ifdef __GNUC__
#pragma implementation
#endif


/*
 * NUMREGBYTES determines the size of the area needed to save all interesting
 * registers.  This must correspond to the size of the register area that
 * the remote debugger expects.  188 bytes is taken from gdb/sun3.h.
 */
#define NUMREGBYTES 188
int registers[NUMREGBYTES/4];

enum regnames { D0, D1, D2, D3, D4, D5, D6, D7, 
		A0, A1, A2, A3, A4, A5, A6, A7, 
		PS, PC };

int _d0;	/* Data Registers */
int _d1;
int _d2;
int _d3;
int _d4;
int _d5;
int _d6;
int _d7;

int _a0;	/* Address Registers */
int _a1;
int _a2;
int _a3;
int _a4;
int _a5;
int _a6;

int _pc;	/* Program Counter */

int _isp;	/* Interrupt Stack Pointer */
int _msp;	/* Master Stack Pointer */
int _usp;	/* User Stack Pointer */
short _sr;	/* Status Register */
int _vbr;	/* Vector Base Register */
int _cacr;	/* Cache Control Register */
int _caar;	/* Cache Address Register */
int _crp[2];	/* CPU Root Pointer */
int _srp[2];	/* Supervisor Root Pointer */
int _tc;	/* Translation Control Register */
int _tt0;	/* Transparent Translation Register 1 */
int _tt1;	/* Transparent Translation Register 2 */
int _mmusr;	/* MMU Status Register */

/*
 * BUFFER_SIZE defines the maximum number of characters in any packet
 * sent to or received from the remote debugger.  It must be at least
 * NUMREGBYTES * 2 + 5.
 */
#define BUFFER_SIZE 400

static char remoteDebug = 0;

static char hexchars[]="0123456789abcdef";

#define DEBUG_STACK_SIZE	1024

int debugStack[ DEBUG_STACK_SIZE ];
int * debugStackPointer = &debugStack[ DEBUG_STACK_SIZE - 1 ];
int * debugSupervisorSP;

/*
 * The size of the interrupt stack on the 68020 varies with the type of
 * interrupt.  The following table is the number of 8-bit BYTES used
 * for each interrupt format.
 */
int interruptStackFrameSize[] = { 8,8,12,8,8,8,8,8,8,20,32,92,8,8,8,8 };

/*
 * Return the maximum number of characters in any packet sent to or
 * received from the remote debugger.  This must be at least five more
 * than twice the number of register bytes.
 */
int
MC68030GnuDebugger::bufferSize()
{
	return( 400 );
}

/*
 * Map a 68k interrupt vector to a "corresponding" UNIX signal number.
 */
int
MC68030GnuDebugger::vectorToSignal( int vector )
{
	switch( vector ) {
		case 2 : return( 10 ); break; /* bus error           */
		case 3 : return( 10 ); break; /* address error       */
		case 4 : return( 4 );  break; /* illegal instruction */
		case 5 : return( 8 );  break; /* zero divide         */
		case 6 : return( 16 ); break; /* chk instruction     */
		case 7 : return( 16 ); break; /* trapv instruction   */
		case 8 : return( 11 ); break; /* privilege violation */
		case 9 : return( 5 );  break; /* trace trap          */
		case 10: return( 4 );  break; /* line 1010 emulator  */
		case 11: return( 4 );  break; /* line 1111 emulator  */
		case 31: return( 2 );  break; /* interrupt           */
		case 33: return( 5 );  break; /* breakpoint          */
		case 47: return( 5 );  break; /* breakpoint	     */
		case 48: return( 8 );  break; /* floating point err  */
		case 49: return( 8 );  break; /* floating point err  */
		case 50: return( 8 );  break; /* zero divide         */
		case 51: return( 8 );  break; /* underflow           */
		case 52: return( 8 );  break; /* operand error       */
		case 53: return( 8 );  break; /* overflow            */
		case 54: return( 8 );  break; /* NAN                 */
		default: return( 7 );         /* "software generated"*/
	}
}

void
MC68030GnuDebugger::stepOrContinue( int step, int pc )
{
	/*
	 * If the pc is specified, set the pc to it.
	 */
	if( pc != 0 ) {
		_pc = pc;
	}

	/*
	 * Set the trace bit if we're stepping; clear it otherwise.
	 */
	if( step == 1 ) {
		_sr |= 0x8000;
	} else {
		_sr &= 0x7fff;
	}

	resumeExecution();
}
