#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Contributing Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Contributing Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * MC68030SourceOfZerosMemoryObject.c - fill a physical buffer with zeros.
 */

#include "Assert.h"
#include "LOCK.h"
#include "CPU_CONFIGURATION.h"	/* for MAXPROCESSORS */
#include "MC68030SourceOfZerosMemoryObject.h"
#include "OutputStream.h"
#include "PROCESSOR_ID.h"
#include "PhysicalMemoryChain.h"
#include "PhysicallyAddressableUnit.h"


#ifdef __GNUC__
#pragma implementation
#endif


MC68030SourceOfZerosMemoryObject theZeroObject;
MemoryObjectStar TheZeroObject = &theZeroObject;

/*
 * I can't figure out how to make this work.  What a brain-damaged
 * architecture!
 */
#ifdef 0
/*
 * This save area is statically allocated so that it will be
 * transparently-mapped when VM is enabled.  This is essential
 * for the proper operation of zeroBlock().
 */
static int saveTC[ MAXPROCESSORS ];
#endif

static int saveTC;
static int disableTC = 0;

static LOCK zeroBlockLock;

void
MC68030SourceOfZerosMemoryObject::zeroBlock( const char * dest,
	unsigned int count )
{
	if( count == 0 ) return;

#ifdef 0
	/*
	 * This routine requires a word of transparently-mapped space
	 * to save the the MMU state.  Disable interrupts, then find
	 * our processor ID.  We'll use the processor ID to index into
	 * a table of statically-allocated space.
	 */
	short sr;
	asm( "movew sr,%0;
	      oriw  #0x0700,sr" : "=g" (sr) );
	
	unsigned int id = PerMachineProcessorID();
#endif
	zeroBlockLock.acquire();

	/*
	 * Make sure the arguments are in registers, disable the MMU
	 * (saving the previous state in transparently-mapped space),
	 * zero the bytes, and restore the MMU.  Note that we know
	 * there is at least one byte to zero.
	 */
	asm( "movel %0,%%a0" : : "g" (dest) : "%a0" );
	asm( "movel %0,%%d0" : : "g" (count) : "%d0" );
	asm( "pmove %%tc,%0" : "=m" (saveTC) );
	asm( "pmove %0,%%tc" : : "g" (disableTC) );
	asm( "Ltop:	clrb  %a0@+;\
			subql #1,%d0;\
			bne   Ltop" );
	asm( "pmove %0,%%tc" : : "m" (saveTC) );

#ifdef 0
	asm( "movew %0,sr" : : "g" (sr) );
#endif
	zeroBlockLock.release();
}


#include "ClassInstaller.h"
DEFINECLASS MC68030SourceOfZerosMemoryObject
