/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Bjorn A. Helgaas (helgaas@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * Util.s: assembly language routines.
 */

	.text
	.globl	setjmp
setjmp:
	movel	%sp@+,%a1			/* return address */
	movel	%sp@,%a0			/* address of buffer */
	moveml	%d2-%d7/%a1-%a7,%a0@
	clrl	%d0
	jmp	%a1@

	.globl	longjmp
longjmp:
	addql	#4,%sp			/* discard return address */
	movel	%sp@+,%a0			/* get address of buffer */
	movel	%sp@,%d0			/* get value to return */
	bne	_lj0
	movel	#1,%d0			/* can't return 0 this time */
_lj0:	moveml	%a0@,%d2-%d7/%a1-%a7
	jmp	%a1@

/*
 * Return the current status of the processor caches
 * (1 => were enabled, 0 => were disabled).
 */
	.globl	processorCachesEnabled
processorCachesEnabled:
#if 0
	movec	%cacr,%d0
	bne	pro1
	clrl	%d0
	bra	pro2
pro1:	movel	#1,%d0
#endif
pro2:	rts

/*
 * Disable processor caches, returning a flag which indicates whether
 * they were previously enabled (1 => were enabled, 0 => were disabled).
 */
	.globl	disableProcessorCaches
disableProcessorCaches:
#if 0
	movel	%d1,%sp@-
	movec	%cacr,%d0
	bne	dis1
	clrl	%d0
	bra	dis2
dis1:	movel	#1,%d0
dis2:	clrl	%d1
	movec	%d1,%cacr
	movel	%sp@+,%d1
#endif
	rts

/*
 * Enable processor caches, returning the old state.  (1 => were
 * enabled, 0 => were disabled).  We flush the caches before
 * enabling them, since this seems to work better.
 */
	.globl	enableProcessorCaches
enableProcessorCaches:
#if 0
	movel	%d1,%sp@-
	movec	%cacr,%d0
	bne	ena1
	clrl	%d0
	bra	ena2
ena1:	movel	#1,%d0
ena2:	movel	#0x0808,%d1	/* CD, CI */
	movec	%d1,%cacr
	movel	#0x0011,%d1	/* IBE, EI */
	movec	%d1,%cacr
	movel	%sp@+,%d1
#endif
	rts

/*
 * Copy a block of physical memory by disabling interrupts, turning the
 * MMU off, performing the copy, and restore the previous interrupt and
 * MMU state.  The only thing that's complicated is that the 68030
 * doesn't allow you to load the MMU's translation control register
 * from another register.  The old state must be stored in memory
 * somewhere, and since we need to retrieve it when the MMU is disabled,
 * we must know the physical address.  Kernel data is mapped transparently,
 * so we can just allocate a word here, as long as we grab a lock while
 * we're using it.
 */
	.data
MMUstate:
	.long	0
MMUoff:
	.long	0
copyLock:
	.long	0

	.text
	.globl physicalCopy
physicalCopy:
	link	%a6,#-8
	moveml	%d0-%d2/%a0-%a2,%sp@-

	movel 	%a6@(8),%a0		/* source */
	movel 	%a6@(12),%a1		/* destination */
	movel 	%a6@(16),%a2		/* count */

	clrl	%a6@(-4)			/* slot for current MMU state */
	clrl	%a6@(-8)			/* zero for disabling MMU */

	movew  	%sr,%d2			/* disable interrupts */
	oriw 	#0x0700,%sr

copyGetLock:
	tas	copyLock
	beq	copyGotLock
	bra	copyGetLock

copyGotLock:
	clrl	MMUoff
	pmove	%tc,MMUstate		/* get current MMU state */
	pmove	MMUoff,%tc		/* turn off MMU */

	clrl	%d0
loop:	
	cmpw	%d0,%a2
	beq   	cont
	moveb 	%a0@+,%a1@+
	subqw 	#1,%a2
	bra   	loop

cont:
	pmove	MMUstate,%tc		/* restore MMU to previous state */
	clrb	copyLock
	movew 	%d2,%sr			/* restore previous sr */
	moveml	%sp@+,%d0-%d2/%a0-%a2
	unlk	%a6
	rts


	.text
	.globl disableInterrupts
disableInterrupts:
	movew	%sr,%d0		/* save previous state */
	oriw	#0x0700,%sr	/* disable interrupts */
	bra	_ie0		/* then act like interruptsEnabled() */

	.text
	.globl interruptsEnabled
interruptsEnabled:
	movew	%sr,%d0
_ie0:	andiw	#0x0700,%d0
	beq	_ie1
	clrl	%d0		/* return 0 if interrupts disabled */
	bra	_ie2
_ie1:	movel	#1,%d0		/* return 1 if interrupts enabled */
_ie2:	rts

	.text
	.globl enableInterrupts
enableInterrupts:
	andiw	#0xf0ff,%sr	/* enable interrupts */
	rts

