#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Contributing Author: See-Mong Tan (stan@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * NS32332Context.c: Processor context for the NS32332.
 */

#include "NS32332Context.h"
#include "OutputStream.h"
#include "OutputStream.h"
#include "Process.h"
#include "ThisProcess.h"
#include "PersistentMemoryObject.h"
#include "Domain.h"
#include "Kernel.h"
#include "ProxyTable.h"
#include "NS32382MMU.h"
#include "ThisProcess.h"
#include "Raid.h"
#include "PROCESSOR.h"
#include "SourceOfZerosMemoryObject.h"
#include "RootMemoryObjectCache.h"


#ifdef __GNUC__
#pragma implementation
#endif

extern unsigned int getUserStackPointer();
unsigned int
getUserStackPointer()
{
	register unsigned int usp;
	asm volatile( "bispsrw $0x200;addr tos,%0;bicpsrw $0x200":"=r" (usp));
	return(usp);
}

extern unsigned int getKernelStackPointer();
unsigned int getKernelStackPointer()
{
    register unsigned int usp;
    asm volatile( "addr tos,%0":"=r" (usp));
    return(usp);
}

NS32332GenericContext::NS32332GenericContext( Process * owner,
	StackSize stackSize )
	: ProcessorContext( owner )
{
	int heapStack = 0;
	int npages;
	switch( stackSize ) {
	   case small:
           case heapBootStack:
		heapStack = 1;		// fallthrough (no break)
		npages = 2; 
		break;
	   case normal:
	   case application:
		npages = 4; 
		break;
	   case large:
		npages = 8;
		break;
	   case huge:
		npages = 16;
		break;
           case gigantic:
                npages = 32;
	        break;
	   default:
		Assert( NOTREACHED );
		npages = 0;
	}
	int nbytes = npages*PhysicalPageSize;

	Debug << this << " for process " << owner << "\n" << eor;
	if (heapStack) {
		Debug << this << ": Heap stack\n" << eor;
		_kernelStackMO = 0;
		stackBound[ 0 ] = new char[ nbytes ];
		Assert( stackBound[0] != 0 );
		}
	else	{
		Debug << this << ": MO stack\n" << eor;
		_kernelStackMO = new SourceOfZerosMemoryObject;
		Assert (_kernelStackMO != 0);
		_kernelStackMO->setName("StackMo");
		_kernelStackMO->setNumberOfUnits(nbytes);
		Debug << "::NS32332GC: Stack MO: " << _kernelStackMO << "\n" << eor;
		_kernelStackMO->setCacheClass(RootMemoryObjectCacheClass);
		MemoryObjectCache * cache = _kernelStackMO->buildCache();
                Assert (cache != 0);
		Debug << this << ": Stack cache: " << cache << "\n" << eor;
		stackBound[0] = (char *)TheKernel->domain()->add(_kernelStackMO, NoAccess);
		Assert (stackBound[0] != 0);
		Debug << this << ": Make Stack addressable\n" << eor;
		TheKernel->domain()->makeAddressable(stackBound[0], nbytes);
		Debug << this << ": Stack is now addressable\n" << eor;
		}
	stackBound[ 1 ] = 0;
	stackPointer[ 1 ] = 0;
	stackBase[ 1 ] = 0;
	stackBase[ 0 ] = &(stackBound[ 0 ])[nbytes - 1];
	stackPointer[ 0 ] = (char *)((unsigned)(stackBase[0] - 7)  & ~7);
	//sharedStackSize = nbytes;

	r5 = 0;


	Debug << this << ": stackBound: " <<
		asAddress((void *)stackBound[ 0 ]) << " stackBase: " <<
		asAddress((void *)stackBase[ 0 ]) << "\n" << eor;
}

NS32332GenericContext::~NS32332GenericContext()
{
	Debug << this << "::~NS32332GenericContext()\n";
	if  (_kernelStackMO == 0){
		Debug << this << ":" << process() << ": Delete Heap stack\n" << eor;
		// delete stackBound[ 0 ];
		}
	else	{
		Debug << this << ":" << process() << ": Delete MO stack\n" << eor;
		TheKernel->domain()->remove(_kernelStackMO,stackBound[ 0 ]);
		}
}

static char *
allocateUserStack( int size, Domain * domain )
{
	char * stack;

	PersistentMemoryObjectStar memObj =
		TheKernel->allocateMemoryObjectForBackingStore();

	Assert( memObj != 0 );
	memObj->setName( "userStack" );

	memObj->setNumberOfUnits( size / memObj->unitSize() );

	stack = (char *) domain->add( memObj, ReadWrite );

	Assert( stack != 0 );

	return( stack );
}


#define MAXTRACESTACK 20
void
NS32332GenericContext::traceStack()
{
	/*
	 * This assumes that the code is not compiled with -fomit-frame-pointer
	 */
	int i, j;
	unsigned int * fp = 0;

	asm volatile( "sprd fp,%0" : "=g" (fp) );
	Console() << "\nStack back trace\n" << eor;
	for (j = 0; j <= 1; j++) {
		if ( stackBase[j] != 0 )
			Console() <<
			    "Stack ring " << j << " has bound of " <<
			        asHex( (unsigned int) stackBound[j] ) <<
			    " and base of " <<
			        asHex( (unsigned int) stackBase[j] ) << "\n";
	}
	Console() << eor;
	for (i = 0; i < MAXTRACESTACK; i++) {
		for (j = 0; j <= 1; j++) {
			if ( ( stackBase[j] != 0 ) &&
				( (char *) ( fp - 3 ) > stackBound[j] ) &&
					( (char *) ( fp + 4) < stackBase[j] ) )
				break;
		}
		
		if ( j > 1 ) {
		    Console() <<
		     "Stopping trace on fp of " <<
			asHex( (unsigned int) fp ) << "\n" << eor;
		    break;
		}
		Console() << "(" << i << ")\t ("
		    << asHex( *(fp-3) ) << ", "
		    << asHex( *(fp-2) ) << ", "
		    << asHex( *(fp-1) ) << ") "
		    << asHex( *(fp+1) ) << " ("
		    << asHex( *(fp+2) ) << ", "
		    << asHex( *(fp+3) ) << ", "
		    << asHex( *(fp+4) ) << ")\n" << eor;

		fp = (unsigned int *) *fp;
	}
}

/*
 * Allocate a stack of the specified size. Memory won't actually be paged
 *   in until needed.  Backing store space will be used, however.
 * Will probably want to have a 'StackMemoryObject' per domain from
 *   which to allocate stacks instead of creating a whole new memory
 *   object for every stack.
 */
#include "defUNIXCOMPATIBILITY.h"
#ifdef UNIXCOMPATIBILITY
NS32332ApplicationContext::NS32332ApplicationContext( Process * owner,
	PFV entry, StackSize stackSize, Domain *initialDomain, int a0, int a1,
	int a2, StackSize userStackSize, char * userStackPointer )
	: NS32332GenericContext( owner, stackSize )
#else
NS32332ApplicationContext::NS32332ApplicationContext( Process * owner,
	PFV entry, StackSize stackSize, Domain *initialDomain, int a0, int a1,
		StackSize userStackSize, char * userStackPointer )
	: NS32332GenericContext( owner, stackSize )
#endif
{
        initDebug();

	Raids(RaidConstructor) << "NS32332ApplicationContext::ctor( " <<
		owner << ", " << asHex((int)entry) << " ...)\n" << eor; 

	int stackSize;
	switch( userStackSize ) {
		case none:
			stackSize = 0;
			break;
		case tiny:
			stackSize = 4096;
			break;
		case small:
			stackSize = 4096*2;
			break;
		case normal:
		case application:
			stackSize = 4096*8;
			break;
		case huge:
			stackSize = 4096*16;
			break;
		case gigantic:
			stackSize = 4096*1024;
			break;
		default:
			stackSize = (int)userStackSize;
	}
	stackSize *=4;

	if( (! userStackPointer) && stackSize )
		// Don't allocate a stack if the user already has a stack
		// pointer or if the stack size would be zero.
		userStackPointer = allocateUserStack( stackSize,
							initialDomain ); 

/* this should always be 1 */

	stackBound[ 1 ] = userStackPointer;
	userStackPointer += stackSize;
	stackBase[ 1 ] = userStackPointer;
	stackPointer[ 1 ] = userStackPointer;

        InitialApplicationContextStackDescription * isd =
                (InitialApplicationContextStackDescription *) (stackPointer[ 0 ]
		- sizeof( InitialApplicationContextStackDescription ));

	stackPointer[ 0 ] = (char *) isd;

	isd->returnPoint = initialDomain->returnFunct();
	isd->arg0 = a0;
	isd->arg1 = a1;
	isd->usp = userStackPointer;
	Raids( RaidMethod ) << "NS32332AppCon: set userStackPointer to " <<
	  asHex((int)isd->usp) << "\n" << eor;

	isd->rettFrame.entryPoint = entry;
	Raids( RaidMethod ) << "NS32332AppCon: set entry to " <<
	  asHex((int)isd->rettFrame.entryPoint) << "\n" << eor;

	/*
	 * Set the PSR to have (UserStack + UserMode + InterruptsEnabled)
	 * See the NS32000 processor architecture manual...
	 */
	isd->rettFrame.psr = 0xb00;

	Module * moduleTableEntry = initialDomain->moduleTableEntry();

	Assert( moduleTableEntry != 0 );

	isd->rettFrame.mod = (unsigned short) moduleTableEntry;

#ifdef UNIXCOMPATIBILITY
	/* There is a BIG HACK here. If arg2 is 2, it's set here and
	   checked in initialRestoreAssist. This is so that 0 can be
	   given to the user process instead of a default proxy. The
	   right solution is to make a better set of constructors that
	   understand that processes might want a third argument that
	   isn't a proxy. For now, the 2 hack will work. --JLC
	*/
	isd->defaultProxy = a2;
	Raids( RaidMethod ) << "NS32332AppCon: set defaultProxy to " <<
	  a2 << "\n" << eor;
#endif
}


void
NS32332ApplicationContext::initialRestoreAssist( Process * previousProcess )
{
	/*
	 * stuff to do in the context of the new process
	 */
        InitialApplicationContextStackDescription * isd = 
		(InitialApplicationContextStackDescription *) stackPointer[ 0 ];

	/*
	 * call parent method
	 */
	restoreAssist( previousProcess );

#ifdef UNIXCOMPATIBILITY
	if( (int)isd->defaultProxy == 2 )
#endif
	/*
	 * Allocate default proxy and pass it to the new application.
	 */
	{
	    isd->defaultProxy = process()->domain()->defaultProxy();
	    Raids( RaidDetail ) << "NS32332AppContext::initialRestoreAssist: allocated default proxy"
	      << eor;
	}

	/*
	 * Will be switching to new domain soon.
	 */
	process()->setCurrentDomain( process()->domain() );
}

NS32332SystemContext::NS32332SystemContext( Process * owner, PFV entry,
	StackSize stackSize, int a0, int a1 )
	: NS32332GenericContext( owner, stackSize )
{
	/*
	 * Keep a copy of the initial entry point and its arguments in
	 * the save area.
	 */
	pc = (int) entry;
	r3 = a0;
	r4 = a1;

	/*
	 * I stash a flag indicating wheather to enable interrupts or
	 * not in r5. initialRestoreAssist() knows to find it here.
	 * Subclasses can re-assign to this to create a context which
	 * does not start out with interrupts enabled.
	 */
	r5 = 1;
}

void
NS32332SystemContext::initialRestoreAssist( Process * previousProcess )
{
	typedef void (* PFVII) (int, int);
	PFVII entryPoint = (PFVII) pc;

	restoreAssist( previousProcess );

	if( r5 ) {
		PROCESSOR::enableInterrupts();
	}

	(* entryPoint)( r3, r4 );

	Assert( process() != 0 );

	process()->die();
	Assert( NOTREACHED );
}

NS32332UninterruptableSystemContext::NS32332UninterruptableSystemContext(
	Process * owner, PFV entry, StackSize stackSize, int a0, int a1 )
	: NS32332SystemContext( owner, entry, stackSize, a0, a1 )
{
	/*
	 * See comment in SystemContext ctor.
	 */
	r5 = 0;
}

#include "ClassInstaller.h"
DEFINECLASS("NS32332SystemContext",NS32332SystemContext,NS32332GenericContext,0,0);
DEFINECLASS("NS32332UninterruptableSystemContext",NS32332UninterruptableSystemContext,NS32332SystemContext,0,0);
DEFINECLASS("NS32332GenericContext",NS32332GenericContext,ProcessorContext,0,0);
DEFINECLASS("NS32332ApplicationContext",NS32332ApplicationContext,NS32332GenericContext,0,0);
