/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * NS32332ContextSwitching.s: assembly language context switching routines.
 *
 * This file also container the first instructions executed by every processor.
 * when it is booted and a small amount of initialization data.
 */

#include "NS32332asmdefs.h"
#include "ProxyTable.h"

#define STARTSTACKSIZE 0x2000;

/****************************************************************************/
/*				DATA					    */
/****************************************************************************/

	/*
	 * void * p_glbl;
	 * This is used to determine the sb registers contents. The linker
	 * (ld) expects it to be defined here. ld magically fills in its value.
	 */
	.comm	p_glbl,4
	.globl	_p_glbl
_p_glbl:
	.double	0
#ifdef DEBUG_HACK
	/*
	 * void * hackThisProcess;
	 * This is a hack for debugging. It is semi-likely to be the last
	 * running process; this is definitely a guess and not a done deal.
	 */
	.globl  _hackThisProcess
	.globl  _hackThisPC
_hackThisProcess:
	.double 0
_hackThisPC:
	.double 0
#endif

/****************************************************************************/
/*				CODE					    */
/****************************************************************************/

/*
 * These are is the very first instructions executed by the first processor in a
 * Choices system. The code here just does necessary CPU initializations
 * and sets up a stack to use. Main() does the real work. 
 */
	.text
	.globl	_start
	.align 4
_start:
	/*
	 * Disable interrupts, use the system stack, run in system mode.
	 * Don't configure for an MMU or an FPU to make sure no one
	 * tries to use them yet.
	 */
	bicpsrw	$0xffff
	setcfg	[i, p]

	/*
	 * Load the static base register. The loader sets us up so that
	 * p_glbl will be the address of static data.
	 */
	lprd	sb,@_p_glbl	/* sb = &p_glbl */
	lprw	mod,$0x1000	/* Use the first mod table entry at */
				/*   address @0x1000 */

	/*
	 * Set up a the Module Table entry at address 0x1000 to get things
	 * started up with. The kernel uses the entry for all its code.
	 */
	sprd	sb,@0x1000+STATICBASEPOINTER__6MODULE
	movqd	$0,@0x1000+LINKBASEPOINTER__6MODULE
	movqd	$0,@0x1000+PROGRAMBASEPOINTER__6MODULE
	movqd	$0,@0x1000+INTERRUPTPOINTER__6MODULE

	/*
	 * Load the first stack and frame pointers for use while setting up.
	 */
	movzbd @-172,r0
	ashd $11,r0
	addr _startStacks+2044,r1
	addd r1,r0
	lprd	sp,r0
	lprd	fp,$0		/* might help debugging? */

	/*
	 * Zero all of bss
	 */
	
	addr	@_end,r0
	addr	@_edata,r1
	subd	r1,r0
	addr	1(r1),r2
	movqb	$0,0(r1)
	movsb	[]

	/*
	 * Call Main() to do all the processor independent initilization.
	 * Once Main returns, the
	 * processor enters at processorEntry below like any other processor.
	 */
// Amitabh
	movqd   $0,tos
	movqd   $0,tos
//	movqd	$0,r1		/* No arguments to Main() yet */
//	movqd	$0,r0
	bsr	_Main__FiPPc
	adjspb	$-8		/* skip args */
	/*
	 * After returning from Main, fall through and act like all the other
	 * Processors.
	 */
/*
 * Where all newly started processors start running. The first processor
 * started executed the above code first but falls through to here.
 * Get a stack for the current processor and call static void CPU::processorStart().
 */
.comm _startStacks,262144

	.text
	.globl	_processorEntry__Fv
_processorEntry__Fv:
	/*
	 * Disable interrupts, use the system stack, run in system mode.
	 * Also, configure for a MMU, FPU and an Interrupt Controller.
	 */
	bicpsrw	$0xffff
	setcfg	[m, f, i, fm, p]

	/*
	 * Load the static base register. The linker (ld) sets us up so that
	 * p_glbl will be the address of static data.
	 */
	lprd	sb,@_p_glbl	/* sb = &p_glbl */
	lprw	mod,$0x1000	/* Use the first mod table entry at */
				/*   address @0x1000 */

	/*
	 * Load the processor start stack
	 *
	movzbd @-172,r0
	ashd $11,r0
	addr _startStacks+4092,r1
	addd r1,r0
	lprd	sp,r0
	lprd	fp,$0		/* might help debugging? */
	jsr _processorStart__3CPU


/*
 * Function called upon an interrupt (vectors >= 16) occuring on a NS32332.
 *
 * This just acks the interrupt and falls through to the trap code below.
 */
	.globl	_interruptAssist__10NS32332CPU
	.align 4
_interruptAssist__10NS32332CPU:
	/*
	 * Inform external interrupt prioritization hardware about the
	 * servicing of the interrupt. Make sure not to touch any registers
	 * or permanently alter the stack here.
	 */
	sprw    psr,tos
	sprw    mod,tos
	addr    4(pc),tos       /* pretty sneaky... and very tacky */
	reti
	/*
	 * Fall through and continue like it was a trap.
	 */

/*
 * Function called upon a trap (vectors 0-15) occuring on a NS32332.
 */
	.globl	_trapAssist__10NS32332CPU
_trapAssist__10NS32332CPU:

#define HACK
#ifdef HACK
	/*
	 * Massive Hack to stop the timer.
	 */
	tbitw	$0x8,6(sp)
	bfc	around
	/*
	 * Only application processes get here, system process jump around.
	 */
	andd	$0xf7,@0xfffffe58
around:
#endif

	/*
	 * Save compiler volatile registers here. This also frees up
	 * a couple of scratch registers to play with.
	 * Use 'enter' instead of 'save' so the frame pointer will also
	 *  be saved so a stack back trace will show where the trap
	 *    occurred.
	 */
	enter	[r0,r1,r2],0

	/*
	 * Get a pointer to which interrupt just occurred into r0
	 */
	sprd	mod,r0
	movd	INTERRUPTPOINTER__6MODULE(r0),r0

	/*
	 * For the duration of this trap/interrupt, we will be running
	 * on module table entry (and, therefore, sb contents) different
	 * from the normal #0 for the system. It doesnt matter since
	 * all the trap/interrupt module table entries load sb with the
	 * same thing #0 would.
	 */

#ifdef OLD
	addr	tos,r1

	/*
	 * OPTIMIZATION: SVC traps happen a lot and should be fast. Test
         * for SVC and jump directly to the basicRaise function if it's
         * a SVC trap.
	 */
// Amitabh
	movd	r1, tos
#endif
	addr tos, tos
	movd	r0, tos 
/*	cmpd    $5, r0
	beq     SVCTrap*/
	bsr	_raise__9InterruptPc
	adjspb	$-8		/* skip args */
	exit	[r2,r1,r0]

#ifdef HACK
	/*
	 * Restart the timer.
	 */
	ord	$0x88,@0xfffffe58
#endif
	rett	$0		/* "return" to the trapping process */

#ifdef NEVER
// SVCTrap:
	bsr	_basicRaise__7SVCTrapPc
	adjspb	$-8		/* skip args */
	exit	[r0,r1,r2]

	/*
	 * Restart the timer.
	 */
	ord	$0x88,@0xfffffe58
	rett	$0		/* "return" to the trapping process */

around:
	/*
	 * OPTIMIZATION: Repeat the above but don't restart the timer
	 * when done.
	 */
	enter	[r0,r1,r2],0
	sprd	mod,r0
	movd	INTERRUPTPOINTER__6MODULE(r0),r0
	addr	tos,r1
// Amitabh
	movd 	r1,tos
	movd    r0,tos
	bsr	_raise__9InterruptPc
	adjspb	$-8		/* skip args */
	exit	[r2,r1,r0]
	rett	$0
#endif

/*
 * Function called upon a SVC trap (vector 5) occuring on a NS32332.
 */
	.globl	_svcAssist__10NS32332CPU
_svcAssist__10NS32332CPU:

        /*
	 * This entire function is an optimization. We avoid the test for
         * system processes, since the kernel should never use the SVC trap
         * (if it does, there's a big problem somewhere). We also avoid one
         * level of virtual function translation, since we know that we want
         * the SVC handler and not some other interrupt handler.
         */

#ifdef HACK
	/*
	 * Massive Hack to stop the timer.
	 */
	andd	$0xf7,@0xfffffe58
#endif

	/*
	 * Save compiler volatile registers here. This also frees up
	 * a couple of scratch registers to play with.
	 * Use 'enter' instead of 'save' so the frame pointer will also
	 *  be saved so a stack back trace will show where the trap
	 *    occurred.
	 */
	enter	[r0,r1,r2],0

	/*
	 * Get a pointer to which interrupt just occurred into r0
	 */
	sprd	mod,r0
	movd	INTERRUPTPOINTER__6MODULE(r0),r0

	/*
	 * For the duration of this trap/interrupt, we will be running
	 * on module table entry (and, therefore, sb contents) different
	 * from the normal #0 for the system. It doesnt matter since
	 * all the trap/interrupt module table entries load sb with the
	 * same thing #0 would.
	 */

#ifdef OLD
	addr	tos,r1
// Amitabh
	movd    r1,tos
#endif
addr	tos, tos
	movd    r0,tos
	bsr	_basicRaise__7SVCTrapPc
	adjspb  $-8
	exit	[r2,r1,r0]

#ifdef HACK
	/*
	 * Restart the timer.
	 */
	ord	$0x88,@0xfffffe58
#endif
	rett	$0		/* "return" to the trapping process */


/*
 * PrivilegedProxyTrap - jumps here when an ObjectProxy method is called from
 *   privileged mode.
 */
	.globl	_PrivilegedProxyTrap
_PrivilegedProxyTrap:
	addr	4(sp), r1	/* get stack pointer into r1 */

	enter	[r3,r4,r5],0	/* set fp and save scratch regs */
	movd	$1, r4		/* one indicates called from kernel mode */

	jump	privilegedContinue

/*
 * NS32332CPU::proxyAssist().
 * Function called upon a illegal instruction trap occuring on a NS32332.
 */
	.globl	_proxyAssist__10NS32332CPU
_proxyAssist__10NS32332CPU:
	/*
	 * If return address isn't address of proxy return call or proxy
	 *  trap instruction, treat as a normal trap
	 */
	cmpd	$__ProxyReturnCall, 0(sp)
	beq	_proxyReturnAssist__10NS32332CPU

	cmpd	$_ProxyTrapInstruction, 0(sp)
	bne	_trapAssist__10NS32332CPU
	
	enter	[r3,r4,r5],0	/* set fp and save scratch regs */

	movd	$0, r4		/* zero indicates called from user mode */

	bispsrw $0x200		/* switch to user stack */
	movd	tos, r3		/* pop return from trap call */
	sprd	sp, r1		/* get user stack pointer */
	bicpsrw $0x200		/* switch back to system stack */

	movd	r3, 4(fp) 	/* make return from trap to be place where */
				/*    trap call was called from  	   */
				/* (This is needed because the instruction */
				/*   after the trap is used to detect a    */
				/*   proxy call from privileged mode)      */

privilegedContinue:
	/*
	 * The calling stub has pushed the parameters that were in r0 and r1
	 *   onto the calling stack before the return address.
	 */
   	movd	r2, r3		/* g++ has addr of optional return Ref in r2 */
// Where does this go in our convention??????

	/* save space for return flags and copied args */
	adjspb	$SZPROXYRETURNFLAGS+(MAXPARAMS*4)
    
	sprd	sp, tos 	/* pass pointer to copied args as arg 6 */
	movd	r1, tos		/* pass pointer to original stack as arg 5 */
	movd	r3, tos		/* pass addr of optional return Ref as arg 4 */
// ?????
				/* pass pointer to return flags as arg 3 */
	addr	-(12+SZPROXYRETURNFLAGS)(fp), tos /* (12 is for [r3,r4,r5]) */
	movd	r4, r1		/* pass 'fromKernel' as arg 2 */
	/* method number is still in r0 as arg 1 */
// Amitabh
	movd    r1, tos
	movd 	r0,tos
	bsr	_ProxyCallAssist /* returns pointer to method to call */
	adjspb  $-8		/* skip args */
	adjspb	$-16		/* skip args */

	cmpd	r0,$1		/* check return value */
	beq	quickCall	/* call to kernel with no ProxyReturnAssist */
	blt	proxyNull	/* all work done */
	cmpd	r0, $2
	bgt	userProxy	/* calling to user mode */
	/* calling to kernel mode */

	movd	tos, r0		/* get arg 1 off stack */
	movd	tos, r1		/* get arg 2 off stack */
	movd	r3, r2		/* restore pointer to optional return Ref */
// ?????
	movd	-(12+SZPROXYRETURNFLAGS)+CALLEDMETHOD__16PROXYRETURNFLAGS(fp),r5
	movd    r1, tos
	movd    r0, tos
	jsr	r5		/* call method */
        adjspb  $-8

proxyReturn:
				/* pass pointer to return flags as arg 2 */
	addr	-(12+SZPROXYRETURNFLAGS)(fp), r1

	cmpb	$2, RETURNTYPE__16PROXYRETURNFLAGS(r1)
	beq	proxyStarAssist

	/* it did not return Ref or Star */

	/* return value is still in r0 as arg 1 */

	movd    r1, tos
	movd    r0, tos
	bsr	_ProxyReturnAssist
        adjspb  $-8

	br	proxyDone

quickCall:
	/* call to kernel with no call to ProxyReturnAssist */

	movd	tos, r0		/* get arg 1 off stack */
	movd	tos, r1		/* get arg 2 off stack */
	movd	r3, r2		/* restore pointer to optional return Ref */
// ??????
	movd	-(12+SZPROXYRETURNFLAGS)+CALLEDMETHOD__16PROXYRETURNFLAGS(fp),r5
	movd    r1, tos
	movd    r0, tos
	jsr	r5		/* call method */
        adjspb  $-8

proxyDone:
	adjspb	$-(SZPROXYRETURNFLAGS+((MAXPARAMS-2)*4)) /* skip the space */
			/* less the two parameters that have been removed  */

	cmpd	$1, r4
	beq	privilegedDone

	exit	[r5,r4,r3]	/* restore scratch regs and fp */
	rett	$0

privilegedDone:
	exit	[r5,r4,r3]
	ret	$0

proxyNull:
	adjspb	$-8		/* pop top two values like a normal call */
	br	proxyDone

proxyStarAssist:
	/*
	 * it did return Ref or Star
	 */
	movd	0(r3), r0	/* pass value of optional Ref return as arg 1 */
// ????	
	movd    r1, tos
	movd    r0, tos
	bsr	_ProxyReturnAssist
        adjspb  $-8
	movd	r0, 0(r3)	/* write out the return value */
// ?????
	br	proxyDone

userProxy:
	/*
	 * stack is set up by NS32332GenericContext::setNewRing to jump
	 *    to user.  User will return to NS32332CPU::proxyReturnAssist
	 *    below.
	 */

	/*
	 * set the user stack pointer and put first two args in registers
	 */
				/* get new user stack pointer */
	bispsrw $0x200		/* switch to user stack */
	movd	tos, r2		/* temporarily pop return address */
	movd	tos, r0		/* get arg 1 off stack */
	movd	tos, r1		/* get arg 2 off stack */
	movd	r2, tos		/* put return address back */
	bicpsrw $0x200		/* switch back to system stack */
	movd	r3, r2		/* restore pointer to optional return Ref */

	/*
	 * Even though the following are compiler non-volatile, we can't
	 *   trust that the user won't write his own assembly code to
	 *   mess with these registers so we need to save them.
	 */
	movd	r3, R3__14USERPROXYSTATE+SZRETTFRAME(sp)
	movd	r4, R4__14USERPROXYSTATE+SZRETTFRAME(sp)
	/* we've already saved r5 and it's scratch as far as we're concerned
	 * movd	r5, R5__14USERPROXYSTATE+SZRETTFRAME(sp)
	 */
	movd	r6, R6__14USERPROXYSTATE+SZRETTFRAME(sp)
	movd	r7, R7__14USERPROXYSTATE+SZRETTFRAME(sp)
	sprd	fp, FP__14USERPROXYSTATE+SZRETTFRAME(sp)
	
	rett	$0

/*
 * NS32332CPU::proxyReturnAssist().
 * Function called upon a proxy return
 */
	.globl	_proxyReturnAssist__10NS32332CPU
_proxyReturnAssist__10NS32332CPU:
	/*
	 * Restore the compiler non-volatile registers
	 */
	movd	R3__14USERPROXYSTATE+SZRETTFRAME(sp), r3
	movd	R4__14USERPROXYSTATE+SZRETTFRAME(sp), r4
	/* movd	R5__14USERPROXYSTATE+SZRETTFRAME(sp), r5 */
	movd	R6__14USERPROXYSTATE+SZRETTFRAME(sp), r6
	movd	R7__14USERPROXYSTATE+SZRETTFRAME(sp), r7
	lprd	fp, FP__14USERPROXYSTATE+SZRETTFRAME(sp)

	adjspb 	$-8	/* pop top two values like normal call to kernel */

	br	proxyReturn


/* Process *
 * NS32332ApplicationContext::checkpoint( ProcessorContext * )
 *
 * The application context checkpoint/restore functions save/restore
 * the floating point registers and the user stack pointer as well.
 */
	.globl _checkpoint__25NS32332ApplicationContext
	.align 4
_checkpoint__25NS32332ApplicationContext:

	/*
	 * Assert( InSystemMode && OnSystemStack && InterruptsDisabled )
	 */

	/*
	 * Save the stuff the generic checkpoint doesnt.
	 */
	movd	4(sp), r0
	sfsr	FSR__25NS32332APPLICATIONCONTEXT(r0)
	movl	f0,F0__25NS32332APPLICATIONCONTEXT(r0)
	movl	f2,F2__25NS32332APPLICATIONCONTEXT(r0)
	movl	f4,F4__25NS32332APPLICATIONCONTEXT(r0)
	movl	f6,F6__25NS32332APPLICATIONCONTEXT(r0)
	bispsrw	$0x200
	sprd	sp,USP__21NS32332GENERICCONTEXT(r0)
	bicpsrw	$0x200

	/*
	 * Call the parent method to save the rest.
	 *
	 * (as an optimization just fall throught to it to avoid the
	 * extra branch.)
	 */
	


/*
 * Process * NS32332GenericContext::checkpoint()
 * 
 * The Generic checkpoint/restore mechnism doesn't mess with
 * floating point
 * because no kernel process uses it. (At least it better not.). It also
 * doesn't bother with the user stack pointer. 
 * 
 */
	.globl _checkpoint__21NS32332GenericContext
_checkpoint__21NS32332GenericContext:

	/*
	 * Assert( InSystemMode && OnSystemStack && InterruptsDisabled );
	 */
	movd    4(sp), r0 
	movd	r3,R3__21NS32332GENERICCONTEXT(r0)
	movd	r4,R4__21NS32332GENERICCONTEXT(r0)
	movd	r5,R5__21NS32332GENERICCONTEXT(r0)
	movd	r6,R6__21NS32332GENERICCONTEXT(r0)
	movd	r7,R7__21NS32332GENERICCONTEXT(r0)
	movd	0(sp),PC__21NS32332GENERICCONTEXT(r0)
	sprd	fp,FP__21NS32332GENERICCONTEXT(r0)
	sprd	sp,SP__21NS32332GENERICCONTEXT(r0)
	sprd	sb,SB__21NS32332GENERICCONTEXT(r0)
	sprw	mod,MOD__21NS32332GENERICCONTEXT(r0)
	sprw	psr, PSR__21NS32332GENERICCONTEXT(r0)
	movqd	$0,r0
	ret	$0


/*
 * void NS32332ApplicationContext::basicRestore( Process * previousProcess )
 *
 * r0 contains the "this" pointer for the Process to restore.
 */
	.globl	_basicRestore__25NS32332ApplicationContextP7Process
_basicRestore__25NS32332ApplicationContextP7Process:

	/*
	 * Restore the stuff the generic restore doesnt.
	 */
	movd	4(sp), r1
	movl	F0__25NS32332APPLICATIONCONTEXT(r1),f0
	movl	F2__25NS32332APPLICATIONCONTEXT(r1),f2
	movl	F4__25NS32332APPLICATIONCONTEXT(r1),f4
	movl	F6__25NS32332APPLICATIONCONTEXT(r1),f6
	lfsr	FSR__25NS32332APPLICATIONCONTEXT(r1)
	bispsrw	$0x200
	lprd	sp,USP__21NS32332GENERICCONTEXT(r1)
	bicpsrw	$0x200

	/*
	 * Call the parent method to do the rest.
	 *
	 * (like the corresponding save, this is optimized to fall
	 * through to it to avoid the extra branch.)
	 */


/*
 * void NS32332GenericContext::basicRestore( Process * previousProcess )
 *
 * "return" (via a return from trap) to a system process from its saved context.
 *
 * r0 contains the "this" pointer for the Process to restore.
 */
	.globl	_basicRestore__21NS32332GenericContextP7Process
_basicRestore__21NS32332GenericContextP7Process:

	movd    4(sp), r1
	movd    8(sp), r0
	movd	R3__21NS32332GENERICCONTEXT(r1),r3
	movd	R4__21NS32332GENERICCONTEXT(r1),r4
	movd	R5__21NS32332GENERICCONTEXT(r1),r5
	movd	R6__21NS32332GENERICCONTEXT(r1),r6
	movd	R7__21NS32332GENERICCONTEXT(r1),r7
	lprd	fp,FP__21NS32332GENERICCONTEXT(r1)
	lprd	sp,SP__21NS32332GENERICCONTEXT(r1)
	lprd	sb,SB__21NS32332GENERICCONTEXT(r1)
	lprw	mod,MOD__21NS32332GENERICCONTEXT(r1)
	lprw	psr, PSR__21NS32332GENERICCONTEXT(r1)
	movd	PC__21NS32332GENERICCONTEXT(r1), 0(sp)
#ifdef DEBUG_HACK
	movd	PC__21NS32332GENERICCONTEXT(r1),@_hackThisPC /* HACK */
	movd	r1,@_hackThisProcess			/*HACK for debugging*/
#endif
	ret     $0


/*
 * void NS32332SystemContext::restoreFromInitialContext( Process * prev )
 */
	.globl	_restoreFromInitialContext__20NS32332SystemContextP7Process
	.align 4
_restoreFromInitialContext__20NS32332SystemContextP7Process:

	/*
	 * Assert( InSystemMode && InterruptsDisabled && OnSystemStack );
	 */

	/*
	 * Load the system stack pointer for this context.
	 * The assumption made here is that we are already executing in
	 * system mode with the system stack pointer selected. I should
	 * add some asserts to check this eventually.
	 *
	 * Also, the instant we do this load, we "become" the new process for
	 * all intents and purposes. (especially calls to thisProcess()).
	 */
//Amitabh
	movd    4(sp), r0
	movd    8(sp),r1
	lprd	sp,STACKPOINTER__21NS32332GENERICCONTEXT(r0)

	/*
	 * (r0 and r1 still have the right args.)
	 */
// Could be screwing things up

	movd    r1, tos
	movd    r0, tos
	bsr	_initialRestoreAssist__20NS32332SystemContextP7Process
	adjspb  $-8

/*
 * void NS32332ApplicationContext::restoreFromInitialContext( Process * )
 */
	.globl	_restoreFromInitialContext__25NS32332ApplicationContextP7Process
	.align 4
_restoreFromInitialContext__25NS32332ApplicationContextP7Process:

	/*
	 * Load the system stack pointer for this context.
	 * The assumption made here is that we are already executing in
	 * system mode with the system stack pointer selected. I should
	 * add some asserts to check this eventually.
	 */
// Amitabh
	movd	4(sp), r0	
	movd	8(sp), r1	
	lprd	sp,STACKPOINTER__21NS32332GENERICCONTEXT(r0)

	/*
	 * First do what the Kernel requires of us.
	 */
	movd    r1,tos
	movd    r0,tos
	bsr	_initialRestoreAssist__25NS32332ApplicationContextP7Process
	adjspb  $-8
	/*
	 * Set up the user's stack.
	 */

	movd	tos,r0  	/* get pointer to user's stack */
	movd	tos,r1		/* get default return function */
	movd	tos,r2		/* put the first arg into r2 */
	movd	tos,r3		/* put the second arg into r3 */
	movd	tos,r4		/* put the defaultProxy into r4 */
	bispsrw $0x200		/* switch to user stack */
	lprd	sp,r0
	lprd	fp,r0
	movd	r4,tos		/* push args on user stack */
	movd 	r3,tos
	movd	r2,tos
	movd	r1,tos		/* push return function on user's stack */
	bicpsrw $0x200		/* switch back to system stack */

	/*
	 * Set up the arguments for the calling convention the Choices
	 * application library crt0.s expects them to be in.
	 */

// Change by Amitabh
/*	movd	tos,r0		 put the first arg into r0 */
/*	movd	tos,r1		 put the second arg into r1 */
/*	movd	tos,r2		 put the defaultProxy into r2 */

	/*
	 * Start the application running for real by simultaneously loading
	 * the psr/mod from the interrupt stack and jumping to the entry
	 * point (also on the stack). The new psr will determine which
	 * stack pointer is active after this instruction.
	 * Our constructor put all this info on the system stack for us.
	 */
	ord	$0x88,@0xfffffe58
	rett	$0
