#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Contributing Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Contributing Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * NS32332PhysicalMemoryObject.c - memory object class to access physical memory.
 */
#include "Assert.h"
#include "OutputStream.h"
#include "NS32332PhysicalMemoryObject.h"
#include "MemoryObjectStar.h"


#ifdef __GNUC__
#pragma implementation
#endif

static const unsigned int FEW_TS = 0x2;
static const unsigned int FEW_TU = 0x1;

NS32332PhysicalMemoryObject thePhysicalMemoryObject;
MemoryObjectStar ThePhysicalMemoryObject = &thePhysicalMemoryObject;


void
NS32332PhysicalMemoryObject::copyBlock( const char * psrc,
				 const char * pdst,
				 unsigned int count )
{
	if ( count == 0 ) {
		return;
	}

	/*
	 * Disable interrupts no matter what their current state.
	 */
	short oldpsr = 0;
	asm volatile( "sprw psr,%0;bicpsrw $0x800" : "=g" (oldpsr) );

	/*
	 * Load registers (r0 count, r1 source, r2 destination ).
	 * Disable the MMU, copy the block, enable MMU.
	 */
	asm volatile( "movd %1,r0;movd %2,r1;movd %3,r2;"
 		: "=m" (*(char *) 0)
		: "g" (count), "g" (psrc), "g" (pdst)
		: "r0", "r1", "r2" );
	asm volatile( "lmr few,$0;movsb [];lmr few,%1"
 		: "=m" (*(char *) 0)
		: "g" (FEW_TU | FEW_TS)
		: "r0", "r1", "r2" );

        /*
         * Re-enable interrupts if there were previously enabled.
         */
	if( ( oldpsr & 0x800 ) != 0 ) {
		asm volatile( "bispsrw $0x800" );
	}
}


#include "ClassInstaller.h"
DEFINECLASS("NS32332PhysicalMemoryObject",NS32332PhysicalMemoryObject,PhysicalMemoryObject,0,0);
