/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * ObjectProxyStubs.s: Machine dependent object proxy interface support.
 */

/*
 * The ObjectProxy methods trap into the kernel via a privileged instruction.
 * If it is already in the Kernel, the privileged instruction suceeds and
 *   the following instruction is executed.
 */


	.globl	_ProxyTrapInstruction

_ProxyTrapInstruction:
	bicpsrw	$0x100
	jump	_PrivilegedProxyTrap


#define ProxyMethod(methodName,methodNumber)				\
	.globl methodName;						\
methodName:;								\
	movd	$methodNumber, r0; /* proxy number in r0 */		\
	bsr	_ProxyTrapInstruction;					\
	movd	tos, r1;	/* get return addresss */		\
	jump	r1		/* return! */

/*
 * Stub called by functions returning from proxy calls above kernel level.
 */
	.globl	__ProxyReturnCall
__ProxyReturnCall:
	bicpsrw	$0x100
	/* Assert (NOTREACHED) */
	bsr	_Halt__Fv

/*
 * Stub called by constructors.   Most of the work is done in user space,
 *   just jump to the trap after loading the method number.
 */
	.globl _ConstructProxy
_ConstructProxy:
	movd	$-1, r0; /* proxy number in r0 */
	bsr	_ProxyTrapInstruction
	ret	0

/*
 * ObjectProxy::~ObjectProxy()
 */
#define __GNUC__
#ifdef __GNUC__
	ProxyMethod(__$_11ObjectProxy,0)
#else
	ProxyMethod(__$_ObjectProxy,0)
#endif

/*
 * Method #1 is "isProxy()" in ObjectProxy.cc
 */

/*
 * unsigned int
 * ObjectProxy::identity()
 */
	ProxyMethod(_identity__11ObjectProxy,2)

/*
 * int
 * ObjectProxy::isKindOf( Class * )
 */
	ProxyMethod(_isKindOf__11ObjectProxyP5Class,3)

/*
 * int
 * ObjectProxy::isMemberOf( Class * )
 */
	ProxyMethod(_isMemberOf__11ObjectProxyP5Class,4)

/*
 * Should this return a proxy of the class?
 */
/*
 * Class *
 * ObjectProxy::classOf()
 */
	ProxyMethod(_classOf__11ObjectProxy,5)

/*
 * int
 * ObjectProxy::debugMask()
 */
	ProxyMethod(_debugMask__11ObjectProxy,6)

/*
 * void
 * ObjectProxy::setDebugMask( unsigned short )
 */
	ProxyMethod(_setDebugMask__11ObjectProxyUs,7)

/*
 * void
 * ObjectProxy::setDebugMessages( short )
 */
	ProxyMethod(_setDebugMessages__11ObjectProxys,8)

/*
 * void
 * ObjectProxy::writeName( OutputStream * )
 */
	ProxyMethod(_writeName__11ObjectProxyP12OutputStream,9)

/*
 * void
 * ObjectProxy::inspect( OutputStream * )
 */
	ProxyMethod(_inspect__11ObjectProxyP12OutputStream,10)

/*
 * void
 * ObjectProxy::noRemainingReferences()
 */
	ProxyMethod(_noRemainingReferences__11ObjectProxy,11)

/*
 * void
 * ObjectProxy::reference()
 */
	ProxyMethod(_reference__11ObjectProxy,12)

/*
 * void
 * ObjectProxy::unreference()
 */
	ProxyMethod(_unreference__11ObjectProxy,13)

/*
 * void
 * ObjectProxy::f14() etc....
 */
#define StubMethod(methodNumber) \
	ProxyMethod(_f##methodNumber##__11ObjectProxyiiiii,methodNumber)

StubMethod(14)
StubMethod(15)
StubMethod(16)
StubMethod(17)
StubMethod(18)
StubMethod(19)
StubMethod(20)
StubMethod(21)
StubMethod(22)
StubMethod(23)
StubMethod(24)
StubMethod(25)
StubMethod(26)
StubMethod(27)
StubMethod(28)
StubMethod(29)
StubMethod(30)
StubMethod(31)
StubMethod(32)
StubMethod(33)
StubMethod(34)
StubMethod(35)
StubMethod(36)
StubMethod(37)
StubMethod(38)
StubMethod(39)
StubMethod(40)
StubMethod(41)
StubMethod(42)
StubMethod(43)
StubMethod(44)
StubMethod(45)
StubMethod(46)
StubMethod(47)
StubMethod(48)
StubMethod(49)
StubMethod(50)
StubMethod(51)
StubMethod(52)
StubMethod(53)
StubMethod(54)
StubMethod(55)
StubMethod(56)
StubMethod(57)
StubMethod(58)
StubMethod(59)
StubMethod(60)
StubMethod(61)
StubMethod(62)
StubMethod(63)
StubMethod(64)
StubMethod(65)
StubMethod(66)
StubMethod(67)
StubMethod(68)
StubMethod(69)
StubMethod(70)
StubMethod(71)
StubMethod(72)
StubMethod(73)
StubMethod(74)
StubMethod(75)
StubMethod(76)
StubMethod(77)
StubMethod(78)
StubMethod(79)
StubMethod(80)
StubMethod(81)
StubMethod(82)
StubMethod(83)
StubMethod(84)
StubMethod(85)
StubMethod(86)
StubMethod(87)
StubMethod(88)
StubMethod(89)
StubMethod(90)
StubMethod(91)
StubMethod(92)
StubMethod(93)
StubMethod(94)
StubMethod(95)
StubMethod(96)
StubMethod(97)
StubMethod(98)
StubMethod(99)
