#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * SPARCContextSwitching.cc: context switching routines.
 */

#include "Domain.h"
#include "SPARCContext.h"
#include "SPARCInterrupts.h"
#include "SPARCasmutil.h"
#include "SPARCPageTableConstants.h"
#include "ThisProcess.h"
#include "Process.h"
#include "PROCESSOR.h"
#include <sys/psw.h>
#include <sys/reg.h>
#include <sys/asm_linkage.h>
class Process;


#ifdef __GNUC__
#pragma implementation
#endif

SPARCGenericContext * SS1ThisProcessorContext = 0;

extern int nwindows;

inline 
void
flushWindows()
{
	switch (nwindows ) {
	case 7:
		asm volatile("
			save %g0,%g0,%g0;
			save %g0,%g0,%g0;
			save %g0,%g0,%g0;
			save %g0,%g0,%g0;
			save %g0,%g0,%g0;
			restore;
			restore;
			restore;
			restore;
			restore;
		");
		break;
	default:
		asm volatile("
			save %g0,%g0,%g0;
			save %g0,%g0,%g0;
			save %g0,%g0,%g0;
			save %g0,%g0,%g0;
			save %g0,%g0,%g0;
			save %g0,%g0,%g0;
			restore;
			restore;
			restore;
			restore;
			restore;
			restore;
		");
		break;
	};
}


void
SPARCGenericContext::defaultReturn()
{
	Assert (thisProcess() != 0);
	thisProcess()->die();
	Assert(NOTREACHED);
}

void
SPARCGenericContext::restoreFromInitialContext(Process *previousProcess)
{
	/*
	 * We are on the kernel stack of the giving process.
	 */
	flushWindows();

	/* 
	 * Use the kernel stack to call the restoreAssist, since we
	 * may change domains and unmap the stack.  We can't call anything
	 * that may over/underflow without the stack mapped.
	 */ 
	LOAD_REG_FROM_MEM(%sp, _kernelStackTop);

	process()->setCurrentDomain( process()->domain() );
	restoreAssist( previousProcess );
	SS1ThisProcessorContext = this;

	/* 
	 * flush windows before changing the stack! 
	 */
	flushWindows();

	unsigned * const inputs = (unsigned *) &volatileRegs[0];
	register unsigned psr asm ("%g2");
	asm volatile("mov %%psr, %0" : "=r"(psr));
	psr &= ~PSR_ET;
	asm volatile("mov %0, %%psr" : : "r" (psr) );	// disable traps!
	psr &= ~PSR_EF;					// Disable fp
	psr &= ~PSR_PIL;				// Mask off int. level
	psr |= (inputs[0] << 8);			// New int. level
	if (inputs[4])
		psr &= ~PSR_PS;				// Previously user mode
	else
		psr |= PSR_PS;				// Previously S mode

	LOAD_REG_FROM_MEM(%sp, _stackPointer)

	/*
	 * Tell the compiler what regs are clobbered
	 */
	asm volatile("nop" : : : "g1", "o0", "o1", "o2", "o3", "o7");
	LOAD_REG_FROM_MEM(%g1, inputs[1]);		// g1 has entry
	LOAD_REG_FROM_MEM(%o0, inputs[2]);		// arg0
	LOAD_REG_FROM_MEM(%o1, inputs[3]);		// arg1
	LOAD_REG_FROM_MEM(%o2, inputs[4]);		// arg2	defPxy
	LOAD_REG_FROM_MEM(%o3, inputs[5]);		// arg3 debugApps
	LOAD_REG_FROM_MEM(%o7, inputs[7]);		// return area
	asm volatile("mov %0, %%psr" : : "r" (psr) );	// Install new psr
	asm volatile("nop;nop;nop");
	asm volatile("save %g0,%g0,%g0");
	asm volatile("
		jmpl %g1, %g0
		rett %g1 + 4
	");
	Assert (NOTREACHED);
}

Process *
SPARCGenericContext::checkpoint()
{
	//Debug << this << "::checkpoint()\n" << eor;
	//Assert ( ! PROCESSOR::interruptsEnabled() );
	//hasEverBeenDispatched = 1;

	/*
	 * save PC
	 */
	STORE_REG_TO_MEM (%o7, _entryPoint);

	/* 
	 * save globals 
	 */
	/* %g0 is hard wired to zero so we use it for %y  */
	STORE_REG_TO_MEM(%g1, ((unsigned *)globals)[1]);
	asm volatile (" mov	%y, %g1");
	STORE_REG_TO_MEM(%g1, ((unsigned *)globals)[0]);
	STORE_REG_TO_MEM_DOUBLE(%g2, globals[1]);
	STORE_REG_TO_MEM_DOUBLE(%g4, globals[2]);
	STORE_REG_TO_MEM_DOUBLE(%g6, globals[3]);

	/* 
	 * save psr, int level, etc.
	 */
	asm volatile ("mov %psr,%g7");
	STORE_REG_TO_MEM(%g7, psr);

	/* 
	 * save inputs and locals
	 */
	STORE_REG_TO_MEM_DOUBLE(%i0, volatileRegs[0]);
	STORE_REG_TO_MEM_DOUBLE(%i2, volatileRegs[1]);
	STORE_REG_TO_MEM_DOUBLE(%i4, volatileRegs[2]);
	STORE_REG_TO_MEM_DOUBLE(%fp, volatileRegs[3]);
	STORE_REG_TO_MEM_DOUBLE(%l0, volatileRegs[4]);
	STORE_REG_TO_MEM_DOUBLE(%l2, volatileRegs[5]);
	STORE_REG_TO_MEM_DOUBLE(%l4, volatileRegs[6]);
	STORE_REG_TO_MEM_DOUBLE(%l6, volatileRegs[7]);

	/*
	 * Save floats (only if necessary)
	 */
	if (fpEnabled()) {
		STORE_REG_TO_MEM_DOUBLE(%f0, floats[0]);
		STORE_REG_TO_MEM_DOUBLE(%f2, floats[1]);
		STORE_REG_TO_MEM_DOUBLE(%f4, floats[2]);
		STORE_REG_TO_MEM_DOUBLE(%f6, floats[3]);
		STORE_REG_TO_MEM_DOUBLE(%f8, floats[4]);
		STORE_REG_TO_MEM_DOUBLE(%f10, floats[5]);
		STORE_REG_TO_MEM_DOUBLE(%f12, floats[6]);
		STORE_REG_TO_MEM_DOUBLE(%f14, floats[7]);
		STORE_REG_TO_MEM_DOUBLE(%f16, floats[8]);
		STORE_REG_TO_MEM_DOUBLE(%f18, floats[9]);
		STORE_REG_TO_MEM_DOUBLE(%f20, floats[10]);
		STORE_REG_TO_MEM_DOUBLE(%f22, floats[11]);
		STORE_REG_TO_MEM_DOUBLE(%f24, floats[12]);
		STORE_REG_TO_MEM_DOUBLE(%f26, floats[13]);
		STORE_REG_TO_MEM_DOUBLE(%f28, floats[14]);
		STORE_REG_TO_MEM_DOUBLE(%f30, floats[15]);
		asm volatile("st %%fsr, %0" : "=m" (this->fsr) );
		}

	return((Process *) 0);
}

Process *
SPARCGenericContext::basicRestore(Process *processGivingUpProcessor)
{
	SS1ThisProcessorContext = this;

	flushWindows();

	/*
	 * Restore floats (if necessary) before %psr change enables FPU
	 */
	if (fpEnabled()) {
		LOAD_REG_FROM_MEM_DOUBLE(%f0, floats[0]);
		LOAD_REG_FROM_MEM_DOUBLE(%f2, floats[1]);
		LOAD_REG_FROM_MEM_DOUBLE(%f4, floats[2]);
		LOAD_REG_FROM_MEM_DOUBLE(%f6, floats[3]);
		LOAD_REG_FROM_MEM_DOUBLE(%f8, floats[4]);
		LOAD_REG_FROM_MEM_DOUBLE(%f10, floats[5]);
		LOAD_REG_FROM_MEM_DOUBLE(%f12, floats[6]);
		LOAD_REG_FROM_MEM_DOUBLE(%f14, floats[7]);
		LOAD_REG_FROM_MEM_DOUBLE(%f16, floats[8]);
		LOAD_REG_FROM_MEM_DOUBLE(%f18, floats[9]);
		LOAD_REG_FROM_MEM_DOUBLE(%f20, floats[10]);
		LOAD_REG_FROM_MEM_DOUBLE(%f22, floats[11]);
		LOAD_REG_FROM_MEM_DOUBLE(%f24, floats[12]);
		LOAD_REG_FROM_MEM_DOUBLE(%f26, floats[13]);
		LOAD_REG_FROM_MEM_DOUBLE(%f28, floats[14]);
		LOAD_REG_FROM_MEM_DOUBLE(%f30, floats[15]);
		asm volatile("ld %0 ,%%fsr" : : "m" (fsr) );
		}

	/*
	 * restore volatile regs
	 */
	LOAD_REG_FROM_MEM_DOUBLE(%i0, volatileRegs[0]);
	LOAD_REG_FROM_MEM_DOUBLE(%i2, volatileRegs[1]);
	LOAD_REG_FROM_MEM_DOUBLE(%i4, volatileRegs[2]);
	LOAD_REG_FROM_MEM_DOUBLE(%fp, volatileRegs[3]);
	LOAD_REG_FROM_MEM_DOUBLE(%l0, volatileRegs[4]);
	LOAD_REG_FROM_MEM_DOUBLE(%l2, volatileRegs[5]);
	LOAD_REG_FROM_MEM_DOUBLE(%l4, volatileRegs[6]);
	LOAD_REG_FROM_MEM_DOUBLE(%l6, volatileRegs[7]);

	/* 
	 * restore psr, we may return to a different window!
	 */
	register unsigned psr;
	asm volatile ("mov %%psr,%0" : "=r" (psr));
	psr &= PSR_CWP;	/* psr has our window pointer */
	unsigned newpsr = (this->psr & (~ PSR_CWP)) | psr;
	asm volatile ("mov %0,%%psr" : : "r" (newpsr));
	//asm volatile ("nop;nop;nop");

	/* 
	 * restore globals 
	 */
	LOAD_REG_FROM_MEM(%g1, ((unsigned *)globals)[0]);
	asm volatile ("mov %g1, %y");
	LOAD_REG_FROM_MEM(%g1, ((unsigned *)globals)[1]);
	LOAD_REG_FROM_MEM_DOUBLE(%g2, globals[1]);
	LOAD_REG_FROM_MEM_DOUBLE(%g4, globals[2]);
	LOAD_REG_FROM_MEM_DOUBLE(%g6, globals[3]);

	/*
	 * restore PC
	 */
	LOAD_REG_FROM_MEM(%o7, _entryPoint);

	return processGivingUpProcessor;
}

