#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801-2910
 *
 * Copyright (c) 1990,1991 The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Nayeem Islam (nislam@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 *	NetworkBuffer.cc
 *
 *	Methods for the NetworkBuffer and related classes.
 *
 * $Id: SPARCNetworkBufferFreeList.cc 465 2006-01-27 04:08:55Z jcarlyle $
 *
 * $Log: SPARCNetworkBufferFreeList.cc,v $
 * Revision 1.1  2004/06/03 01:58:58  jcarlyle
 * Initial revision
 *
// Revision 1.1  1992/12/02  02:59:07  nislam
// Initial revision
//
 * Revision 1.3  1991/12/01  05:56:48  parghi
 * Misc. changes.  Last revision before networking redesign.
 *
 * Revision 1.2  91/09/25  16:19:30  parghi
 * Some clean-ups and major fixes prior to Stable-izing.
 * 
 * Revision 1.1  91/04/14  23:09:13  parghi
 * Initial revision
 * 
 */


 //#define MMAP 1 
//  #define DEBUG 1

#include "OutputStream.h"
#include "Semaphore.h"
#include "Process.h"
#include "ThisProcess.h"
#include "Domain.h"
#include "Kernel.h"
#include "Byte.h"
#include "NetworkStatistics.h"
#include "NetworkBuffer.h"
#include "SPARCNetworkBufferFreeList.h"
#include "MemoryObject.h"
#include "DummyMemoryObject.h"
#include "RootMemoryObjectCache.h"
#include "SPARCInterrupts.h"
#include "SPARCTranslation.h"

#ifdef __GNUC__
#pragma implementation
#endif

// #define MMAP 1  

// OK: 
// lock buffers.
// preallocate these buffer out of DMA space.

SPARCNetworkBufferFreeList::SPARCNetworkBufferFreeList(
					     unsigned int bufferSize,
					     unsigned int listSize ) : NetworkBufferFreeList(bufferSize,listSize,1)
{
initialize(bufferSize,listSize);
}

void
SPARCNetworkBufferFreeList::initialize(unsigned int bufferSize, unsigned int listSize)
{
#ifdef DEBUG
Console() << "SPARCNetworkBufferFreeList" << eor;
#endif
NetworkBuffer *tmp;
#ifdef MMAP
	int units = (_length+PhysicalPageSize-1)/PhysicalPageSize;
	RootMemoryObject* p = new RootMemoryObject( units  , 12, NoAccess);
	const char* newaddr = TheKernel->domain()->add(p,NoAccess);
	thisProcess()->domain()->makeAddressable((char *) newaddr, _length);

	ByteZero((void*)newaddr,_length);
       	_list = new NetworkBuffer((octet*)newaddr,_length, this);	// set first item
        char *pkt = (char *)(_list->packet());
        unsigned len = _list->length();
	char *adr =(char *)SPARCTranslation::mapToDMA((char *)pkt, len, 1);
	_list->setPacket((octet *)adr);
	_list->setKPacket(pkt);
#else
    _list = new NetworkBuffer(_length, this);	// set first item
        char *pkt = (char *)(_list->packet());
        unsigned len = _list->length();
 	//thisProcess()->domain()->constructChain((char *)pkt,len);
	thisProcess()->domain()->makeAddressable((char *) pkt, len);
	_list->setPacket((octet *)SPARCTranslation::mapToDMA((char *)pkt, 
		len, 1));
	_list->setKPacket(pkt);
#endif

#   ifdef DEBUG
	Debug << "NetworkBufferFreeList constructor: _list = "
	      << asHex((int) _list) << '\n' << eor;
#   endif

    for (int i = 0 ; i < _count ; i++) {// put empty buffers in list
#ifdef MMAP
        units = (_length+PhysicalPageSize-1)/PhysicalPageSize;

	RootMemoryObject* p = new RootMemoryObject( units  , 12, NoAccess);
	const char* newaddr = TheKernel->domain()->add(p,NoAccess);
	thisProcess()->domain()->makeAddressable((char *) newaddr, _length);
	ByteZero((void*)newaddr,_length);
    	tmp = new NetworkBuffer((octet*)newaddr,_length, this);	// set first item
        tmp->addAfter(_list);
        char *pkt = (char *)(tmp->packet());
        unsigned len = tmp->length();

	char *adr =(char *)SPARCTranslation::mapToDMA((char *)pkt, len, 1);
	tmp->setPacket((octet *)adr);
	tmp->setKPacket(pkt);
#ifdef DEBUG
	Console() << " addr in loop " <<  i << " "<< eor;
	Console() << asAddress((char*)tmp->packet()) << "\n" << eor;
#endif
#else
    tmp = new NetworkBuffer(_length, this);	// set first item
        tmp->addAfter(_list);
        char *pkt = (char *)(tmp->packet());
        unsigned len = tmp->length();
	//thisProcess()->domain()->constructChain((char *)pkt,len);
	thisProcess()->domain()->makeAddressable((char *) pkt, len);
	tmp->setPacket((octet *)SPARCTranslation::mapToDMA((char *)pkt, 
		len, 1));
	tmp->setKPacket(pkt);
#endif
    }

#   ifdef DEBUG
	Debug << "NetworkBufferFreeList constructor completed.\n" << eor;
#   endif
}
SPARCNetworkBufferFreeList::~SPARCNetworkBufferFreeList()
{
// should do all the unmapping from DMA space here
}
#include "ClassInstaller.h"
DEFINECLASS("SPARCNetworkBufferFreeList",SPARCNetworkBufferFreeList,NetworkBufferFreeList,0,0);
