#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SPARCPageTableEntry.cc:  Page table entries for a SPARC MMU.
 */

#include "PAGE_TABLE_ENTRY.h"


#ifdef __GNUC__
#pragma implementation
#endif

void
SPARCPageTableEntry::setProtection( ProtectionLevel protection )
{
	Assert( protection < Pte_PROTECT_LEVELS );
	Assert( valid() );

	SPARCProtection realProt;

	entry &= ~protectionMask; 	// mask out current protection

	/*
	 * Convert protection to the corresponding SPARC level.
	 */
	if( protection == ReadWrite ) {
		realProt = USERRW;
	}
	else if( protection == ReadOnly ) {
		realProt = USERRO;
	}
	else if( protection == NoAccess ) {
		realProt = SYSTEMRW;
	}
	else {
		realProt = SYSTEMRO;	// shut up compiler warning.
		Assert( NOTREACHED );
	}

	entry |= realProt;
}

int
SPARCPageTableEntry::okToPerform( AccessType attemptedAccess )
{
	if (! (entry & PG_V) )
		return 0;

	unsigned systempage = entry & PG_S;
	unsigned writepage = entry & PG_W;
	int ret = 0;

	switch( attemptedAccess ) {
		case ApplicationWrite:
			ret = (writepage && ! systempage);
			break;
		case ApplicationRead:
			ret = (!systempage);
			break;
		case SystemWrite:
			ret = writepage;
			break;
		case SystemRead:
			ret = 1;
			break;
		default:
			Assert( NOTREACHED );
	}
	return ret;
}
