#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *               University of Illinois at Urbana-Champaign
 *               Department of Computer Science
 *               1304 W. Springfield Ave.
 *               Urbana, IL     61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *      All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#define SBL_restoreSegmentTable
#define SBL_initializeSegmentTable
#define SBL_newDMA
//#define SBL_OPTIMISTIC
//#define SBL_countPMEG
//#define SBL_assertValidAddressRange
//#define SET_NUM_CTXTS 4

#define ENABLE_VA_CACHE
#define INLINE inline

#define DDebug if (debugSPARCTranslation) Debug
int debugSPARCTranslation = 0;

#include "BitArray.h"
#include "DummyMemoryObject.h"
#include "Kernel.h"
#include "Gdb.h"
#include "MMU_CONSTANTS.h"
#include "OutputStream.h"
#include "PAGE_TABLE_ENTRY.h"
#include "PageTableConstants.h"
#include "PhysicalMemoryChain.h"
#include "PhysicalMemoryManager.h"
#include "PhysicallyAddressableUnit.h"
#include "PROM.h"
#include "SPARCTranslation.h"
#include "SS1util.h"
#include "SS1Configuration.h"
#include "SPARCSimpleInstruments.h"
#include "PROCESSOR.h"


#ifdef __GNUC__
#pragma implementation
#endif

/*
 * Static defns
 */
int                     SPARCTranslation::_classInitialized = 0;
Lock *                  SPARCTranslation::_classLock = 0;
SPARCTranslation **     SPARCTranslation::_contextMap = 0;
SPARCTranslation **     SPARCTranslation::_pmegMap = 0;
char **			SPARCTranslation::_pmegAddr = 0;
Lock *                  SPARCTranslation::_dmaLock = 0;
BitArray *              SPARCTranslation::_dmaMap = 0;
const char **           SPARCTranslation::_dmaMappings = 0;
unsigned *              SPARCTranslation::_dmaLens = 0;
SPARCTranslation *      SPARCTranslation::_kernelADT = 0;
CONTEXT			SPARCTranslation::_nextVictim = 0;



#define NUM_CTXTS nctxts

/*
 * Constant PMEGS that are common to all mappings
 */
extern unsigned npmegs;   // Set up by boot code
SEGMENT_TABLE_ENTRY       invalidPMEG     = npmegs - 1;
static unsigned                PMEG_MASK       = npmegs - 1;
const unsigned int segmentScope = SEGMENT_SIZE;
const unsigned int pageScope = PAGE_SIZE;
const unsigned KERNELBARRIERSIZE        = (char*)0xffffffff-USERMEMORYSTART;
const unsigned VAHOLESIZE               = VA_HOLE_END-VA_HOLE_START;
const unsigned HIGHKERNELSIZE   = 0x00ffffff;
const unsigned 		DMA_SEGS	=(DMA_END-DMA_BASE)/SegmentSize;
const int 		DMA_CONTEXT	= 0;

INLINE
char *
SPARCTranslation::mapping( char *vaddr )
{
	char *result;
	result = pageTableEntry(vaddr).addressMappedTo();
	return result;
}

INLINE
SEGMENT_TABLE_ENTRY
SPARCTranslation::segTableEntry( char *const  address)
{
	unsigned char result;
	asm volatile ( "lduba [%1]3,%0" : "=r" (result) : "r" (address) );
	return result;
}

INLINE
PAGE_TABLE_ENTRY
SPARCTranslation::pageTableEntry( char *const  address)
{
	unsigned result;
	asm volatile ( "lda [%1]4,%0" : "=r" (result) : "r" (address));
	return result;
}

INLINE
void
SPARCTranslation::setSegTableEntry( char *const  address, SEGMENT_TABLE_ENTRY value)
{
	asm volatile ( "stba %1,[%0]3" : : "r" (address), "r" (value));
}

INLINE
void
SPARCTranslation::setPageTableEntry(char * vaddress, const char *paddress,
	SPARCProtection protectionbits)
{
	unsigned pte = 0;	// The new value to store.
	pte = ((unsigned)paddress >> Log2PhysicalPageSize) | 
		protectionbits | PG_V ;
	asm volatile ("sta %1, [%0]4" : : "r" (vaddress), "r" (pte) );
}

INLINE
void
SPARCTranslation::unsetPageTableEntry(char *const vaddr)
{
	asm volatile ("sta %%g0, [%0]4" : : "r" (vaddr));
}

INLINE
void
SPARCTranslation::setContext(CONTEXT value)
{
	Assert (! PROCESSOR::interruptsEnabled());
	Assert ( value >= 0 && (value < NUM_CTXTS || NUM_CTXTS == 0));
	asm volatile ("stba %0, [%1]2" : : "r" (value), "r" ((char *)0x30000000));
}

INLINE
CONTEXT
SPARCTranslation::getContext()
{
	CONTEXT result;
	asm volatile ("lduba [%1]2, %0" : "=r" (result) : "r" ((char *)0x30000000) );
	Assert (result < NUM_CTXTS || NUM_CTXTS == 0);
	return result;
}


/*
 * Copy context zero to the passed context.  Assumes that this
 * code is in the cache already.  Copy 0 to itself first to get it
 * in the cache.  This should only be used at boot time to set up
 * all the contexts in the machine, but it's efficient enought to use
 * anytime I suppose.  Just make sure it's in cache or the context
 * that you're copying to has the kernel mappings already.
 */
int
CopyContext(unsigned char newcontext)
{
	Assert (!PROCESSOR::interruptsEnabled());
	unsigned char value;
	register char * context_reg = (char *)0x30000000;
	register unsigned origContext;
	register unsigned readContext;

	/*
	 * Save the original ctxt
	 */
	asm volatile("lduba [%1]2,%0" : "=r" (origContext) : "r" (context_reg));

	asm volatile("stba %1,[%0]2" : : "r" (context_reg), "r" (newcontext) );
	asm volatile("lduba [%1]2,%0" : "=r" (readContext) : "r" (context_reg));
	asm volatile("stba %1,[%0]2" : : "r" (context_reg), "r" (origContext) );
#ifdef NEVER
	/*  
	 * Attempt Change to the new context 
	 */
	asm volatile("stba %1,[%0]2" : : "r" (context_reg), "r" (newcontext) );
	asm volatile("lduba [%1]2,%0" : "=r" (readContext) : "r" (context_reg));
	/*
	 * See if we indeed got there, bail out if not
	 */
	asm volatile("lduba [%1]2,%0" : "=r" (readContext) : "r" (context_reg));
	if (readContext != newcontext) {
		/* back to calling context */
		asm volatile("stba %0,[%1]2" : : "r" (origContext), "r" (context_reg));
		return -1;
		}
#endif


	/* 
	 * flush windows to avoid overflow, which would save to the stack
	 * which is not mapped.
	 */
	flushWindows();

	/* 
	 * copy the first chunk 
	 */
	for (char *addr = 0; addr < (char*)VA_HOLE_START; addr += segmentScope){
		/* read the segment map for addr */
		asm volatile("lduba [%1]3,%0" : "=r" (value) : "r" (addr));
		/* Change to the new context */
		asm volatile("stba %1,[%0]2" : : "r" (context_reg), 
							"r" (newcontext) );
		/* write the map for addr */
		asm volatile("stba %1,[%0]3" : : "r" (addr), "r" (value) );
		/* back to calling context */
		asm volatile("stba %0,[%1]2" : : "r" (origContext), "r" (context_reg));
		}

	/* 
	 * copy second chunk 
	 */
	unsigned size = (char*)0xffffffff - VA_HOLE_END;
	for (unsigned offset = 0; offset < size; offset += segmentScope){
		addr = (char*)VA_HOLE_END + offset;
		/* read the segment map for addr */
		asm volatile("lduba [%1]3,%0" : "=r" (value) : "r" (addr));
		/* Change to the new context */
		asm volatile("stba %1,[%0]2" : : "r" (context_reg),
							"r" (newcontext) );
		/* write the map for addr */
		asm volatile("stba %1,[%0]3" : : "r" (addr), "r" (value) );
		/* back to calling context */
		asm volatile("stba %0,[%1]2" : : "r" (origContext), "r" (context_reg));
		}
	return readContext;
}

#ifndef ENABLE_VA_CACHE
inline void SPARCTranslation::flushSegment(char *segAddr) { }
inline void SPARCTranslation::flushPage(char *pageAddr) { }
inline void SPARCTranslation::invalidateContext() { }
inline void SPARCTranslation::flushContext() { }
void SPARCTranslation::rangeFlush(char *addr, unsigned len) { }
#else /* VA_CACHE is enabled */

extern unsigned vac_linesize;
extern unsigned vac_size;
extern int vac_hwflush;

INLINE void 
SPARCTranslation::invalidateContext()
{
	Assert (!PROCESSOR::interruptsEnabled());
	char *base 		= (char*)0x80000000;
	char *end  		= (char*)0x8000fff0;
	extern unsigned vac_linesize;
	for (char *addr = base; addr < end; addr+=vac_linesize){
		asm volatile("sta %%g0,[%0]2" : : "r" (addr));
		}
}

static SPARCCacheFlushTimer ctxtFlushInst("SPARCFlushContextInstrument");
INLINE void 
SPARCTranslation::flushContext()
{
	Assert (!PROCESSOR::interruptsEnabled());
	SInstr(int start = ctxtFlushInst.start());

	if (vac_hwflush) {
		int i = vac_size;
 		while (i > 0) {
 			asm volatile ("sta %%g0, [%0]7" : : "r" (i));
 			i -= PhysicalPageSize;
 			}
  		}
	else 	{
		const unsigned flushRange = 0x10000;
		char *base 		= (char*)0x30000000;
		register char *addr1 = base;
		register char *addr2 = base + flushRange/8;
		register char *addr3 = base + 2*flushRange/8;
		register char *addr4 = base + 3*flushRange/8;
		register char *addr5 = base + 4*flushRange/8;
		register char *addr6 = base + 5*flushRange/8;
		register char *addr7 = base + 6*flushRange/8;
		register char *addr8 = base + 7*flushRange/8;
		while (addr8 < base + flushRange){
			asm volatile("sta %%g0,[%0]14" : : "r" (addr1));
			addr1 += vac_linesize;
			asm volatile("sta %%g0,[%0]14" : : "r" (addr2));
			addr2 += vac_linesize;
			asm volatile("sta %%g0,[%0]14" : : "r" (addr3));
			addr3 += vac_linesize;
			asm volatile("sta %%g0,[%0]14" : : "r" (addr4));
			addr4 += vac_linesize;
			asm volatile("sta %%g0,[%0]14" : : "r" (addr5));
			addr5 += vac_linesize;
			asm volatile("sta %%g0,[%0]14" : : "r" (addr6));
			addr6 += vac_linesize;
			asm volatile("sta %%g0,[%0]14" : : "r" (addr7));
			addr7 += vac_linesize;
			asm volatile("sta %%g0,[%0]14" : : "r" (addr8));
			addr8 += vac_linesize;
			}
		}
	SInstr(ctxtFlushInst.stop(start));
}

static SPARCCacheFlushTimer pageFlushInst("SPARCFlushPageInstrument");
// INLINE
void 
SPARCTranslation::flushPage(char *const pageAddr)
{
	SInstr(int start = pageFlushInst.start());
	Assert (!PROCESSOR::interruptsEnabled());
	// DDebug << "flushPage(" << asAddress(pageAddr) << ")\n" << eor;
	if (vac_hwflush) {
 		asm volatile ("sta %%g0, [%0]6" : : "r" (pageAddr));
 	} 
	else	{
		const unsigned flushRange = 0x1000;
		register char *addr1 = pageAddr;
		register char *addr2 = pageAddr + flushRange/8;
		register char *addr3 = pageAddr + 2*flushRange/8;
		register char *addr4 = pageAddr + 3*flushRange/8;
		register char *addr5 = pageAddr + 4*flushRange/8;
		register char *addr6 = pageAddr + 5*flushRange/8;
		register char *addr7 = pageAddr + 6*flushRange/8;
		register char *addr8 = pageAddr + 7*flushRange/8;
		while (addr8 < pageAddr + flushRange){
			asm volatile("sta %%g0,[%0]13" : : "r" (addr1));
			addr1 += vac_linesize;
			asm volatile("sta %%g0,[%0]13" : : "r" (addr2));
			addr2 += vac_linesize;
			asm volatile("sta %%g0,[%0]13" : : "r" (addr3));
			addr3 += vac_linesize;
			asm volatile("sta %%g0,[%0]13" : : "r" (addr4));
			addr4 += vac_linesize;
			asm volatile("sta %%g0,[%0]13" : : "r" (addr5));
			addr5 += vac_linesize;
			asm volatile("sta %%g0,[%0]13" : : "r" (addr6));
			addr6 += vac_linesize;
			asm volatile("sta %%g0,[%0]13" : : "r" (addr7));
			addr7 += vac_linesize;
			asm volatile("sta %%g0,[%0]13" : : "r" (addr8));
			addr8 += vac_linesize;
			}
		}
	SInstr(pageFlushInst.stop(start));
}

static SPARCCacheFlushTimer segFlushInst("SPARCFlushSegmentInstrument");
INLINE void SPARCTranslation::flushSegment(char *const segAddr)
{
	SInstr(int start = segFlushInst.start());
	Assert (!PROCESSOR::interruptsEnabled());
	// DDebug << "flushSegment(" << asAddress(segAddr) << ")\n" << eor;

	if (vac_hwflush) {
 		int i = vac_size;
		while (i > 0) {
			asm volatile ("sta %%g0, [%0]5" : : "r" (segAddr+i));
			i -= PhysicalPageSize;
			}
		}
	else	{
		extern unsigned vac_linesize;
		const unsigned flushRange = 0x10000;
		register char *addr1 = segAddr;
		register char *addr2 = segAddr + flushRange/8;
		register char *addr3 = segAddr + 2*flushRange/8;
		register char *addr4 = segAddr + 3*flushRange/8;
		register char *addr5 = segAddr + 4*flushRange/8;
		register char *addr6 = segAddr + 5*flushRange/8;
		register char *addr7 = segAddr + 6*flushRange/8;
		register char *addr8 = segAddr + 7*flushRange/8;
		while (addr8 < segAddr + flushRange){
			asm volatile("sta %%g0,[%0]12" : : "r" (addr1));
			addr1 += vac_linesize;
			asm volatile("sta %%g0,[%0]12" : : "r" (addr2));
			addr2 += vac_linesize;
			asm volatile("sta %%g0,[%0]12" : : "r" (addr3));
			addr3 += vac_linesize;
			asm volatile("sta %%g0,[%0]12" : : "r" (addr4));
			addr4 += vac_linesize;
			asm volatile("sta %%g0,[%0]12" : : "r" (addr5));
			addr5 += vac_linesize;
			asm volatile("sta %%g0,[%0]12" : : "r" (addr6));
			addr6 += vac_linesize;
			asm volatile("sta %%g0,[%0]12" : : "r" (addr7));
			addr7 += vac_linesize;
			asm volatile("sta %%g0,[%0]12" : : "r" (addr8));
			addr8 += vac_linesize;
			}
		}
	SInstr(segFlushInst.stop(start));
}

static SPARCCacheFlushTimer rangeFlushInst("SPARCFlushRangeInstrument");
void
SPARCTranslation::rangeFlush(char *addr, unsigned len)
{
	SInstr(int start = rangeFlushInst.start());
	Assert (!PROCESSOR::interruptsEnabled());
	extern unsigned vac_linesize;
	char * end = addr + len;
	while (addr < end) {
		asm volatile("sta %%g0,[%0]12" : : "r" (addr));
		addr += vac_linesize;
		}
	SInstr(rangeFlushInst.stop(start));
}

#endif /* VA_CACHE_ENABLED */

INLINE
void
SPARCTranslation::localAcquire()
{
	_lock.acquire();
}

INLINE
void
SPARCTranslation::localRelease()
{
	_lock.release();
}

INLINE
void
SPARCTranslation::classAcquire()
{
	Assert (_classLock != 0);
	_classLock->acquire();
}

INLINE
void
SPARCTranslation::classRelease()
{
	Assert (_classLock != 0);
	_classLock->release();
}

SPARCTranslation::SPARCTranslation(SPARCTranslation *)
{
	Debug << "SPARCTranslation::SPARCTranslation(): ";
	Debug << asAddress(this) << "\n" << eor;
	Assert( this != 0 );
	_myContext = -1;	
	_stored = 0;
	if (_kernelADT == 0){
		_kernelADT = this;
		}
}

/*
 * When we get here:
 * The kernel plus a bunch extra is mapped 1-1.
 * The OBIO Hardware is mapped to HW_PMEG
 * We need to create the invalid PMEG, set up the constants that allow us to
 * invalidate everything quickly, and set everything not in use to be
 * invalid.  Then we copy this context to all the others.
 *
 * Now... in order for the VM initialization to work properly we find the
 * end of what should be mapped 1-1 so far and mark those PMEGS taken
 * by the kernel.  This is because the resident MO's are not added in order
 * and the kernel PMEGS will be re-used and we are running in them at the
 * moment.
 */
void
SPARCTranslation::init(unsigned tableAreaSize, VirtualMemoryRange *map)
{
	extern unsigned _npmegs; // From the assembly boot code.

	Assert (!PROCESSOR::interruptsEnabled());
	Debug << "SPARCTranslation::init()\n" << eor;
	Assert(_classInitialized == 0);
	Assert(_classLock == 0);
	Assert(_contextMap == 0);
	Assert(_pmegMap == 0);
	Assert(_pmegAddr == 0);
	Assert(_dmaLock == 0);
	Assert(_dmaMap == 0);
	Assert(_dmaMappings == 0);
	Assert(_dmaLens == 0);

	_classLock = new Lock;
	Assert (_classLock != 0);
	_pmegMap = new SPARCTranslation*[npmegs];
	_pmegAddr = new char*[npmegs];
	Assert (_pmegMap != 0);
	Assert (_pmegAddr != 0);
	for (int i = 0; i < npmegs; i++) {
		_pmegMap[i] = 0;
		_pmegAddr[i] = 0;
	}
	_dmaLock = new Lock ;
	Assert (_dmaLock != 0);
	_dmaMap = new BitArray(DMA_SEGS);
	_dmaMap->clear(0, DMA_SEGS - 1);
	_dmaMappings = new char*[DMA_SEGS];
	_dmaLens = new unsigned[DMA_SEGS];
	for (i = 0; i < DMA_SEGS; i++){
		_dmaMappings[i] = 0;
		_dmaLens[i] = 0;
		}
	_nextVictim = 0;

	/*
	 * Set up the context we are in, then copy it to the other
	 * contexts.
	 *
	 * We get a new char to find out where the top
	 * of the bootHeap currently is.
	 */
	unsigned char newPMEG;
	char *base;
	char *endmapping;
	ProtectionLevel kernelPageProt = NoAccess;
	base = new char;
	char *const topPageSoFar = pageAlign(base) ;
	char *const topSegmentSoFar = segmentAlign(base) ;

	/*
	 * Invalidate anything below the kernel, this is already mapped 1-1,
	 * and we usually just invalidate the first page to catch zero
	 * pointer references.
	 */
	base = 0;
	endmapping = KERNEL_START_VIRTUAL; 
	Debug << "SPARCTranslation::init: invalidating pages before Kernel at "
		<< asAddress(base) << " to " << asAddress(endmapping) 
		<< "\n" << eor;
	_pmegMap[0] = (SPARCTranslation *)1;;
	_pmegAddr[0] = 0;
	for (base = 0; base < endmapping; base += pageScope) {
		Debug << "SPARCTranslation::init: invalidating page for addr "
			<< asAddress(base) << "\n" << eor;
		unsetPageTableEntry(base);
		}

	base = KERNEL_START_VIRTUAL; 
	endmapping = topPageSoFar;
	Debug << "SPARCTranslation::init: adding mappings for KERNEL from "
		<< asAddress(base) << " to " << asAddress(endmapping) 
		<< "\n" << eor;
	/*
	 * Go through the KERNEL segments, mapping them in 1-1.
	 */
	for (/*nothing*/; base <= endmapping; base += pageScope) {
		unsigned segAlign = (unsigned)segmentAlign(base);
		unsigned pgAlign = (unsigned)pageAlign(base);
		newPMEG = (segAlign  >> Log2SegmentSize) & PMEG_MASK;
		if ( segAlign == pgAlign){
			setSegTableEntry(base, newPMEG);
			_pmegMap[newPMEG] = (SPARCTranslation *)1;
			_pmegAddr[newPMEG] = base;
			Debug << "SPARCTr::init() Kernel PMEG :"
				<< asHex(newPMEG) << "\n" << eor;
			}
		setPageTableEntry(base, base, SYSTEMRW);
		Assert(segTableEntry(base) == newPMEG);
		PAGE_TABLE_ENTRY pte = pageTableEntry(base);
//		Debug << " SPARCTranslation::init(): Mapping for" << asAddress(base) << ": " << asHex(pte) << "\n" << eor;
		Assert(pte.valid());
		Assert(pte.okToPerform(SystemRead));
		Assert(pte.okToPerform(SystemWrite));
		Assert(! pte.okToPerform(ApplicationRead));
		Assert(! pte.okToPerform(ApplicationWrite));
		}

	/*
	 * Set up the invalid PMEG and mark it in the map
	 */
	Debug << "Setting up invalid PMEG\n" << eor;
	clearPMEG(invalidPMEG);
	_pmegMap[invalidPMEG] = (SPARCTranslation *)1;
	Debug << "SPARCTr::init() invalid PMEG :" << 
		asHex(invalidPMEG) << "\n" << eor;

	/*
	 * Mark the parts of the pmeg map the the EEPROM is using,
	 * and mark the harware PMEG as in use.
	 */
	base  = segmentAlign(EEPROM_VADDR);
	endmapping = segmentAlign(EEPROM_VADDR + EEPROM_VSIZE );
	for (/*nothing*/; base != endmapping; base += segmentScope) {
		Debug << "SPARCTr::init(): EEPROM pmeg for " <<
			asAddress(base) << ": " <<
			asHex(segTableEntry(base)) << "\n" << eor;
		_pmegMap[segTableEntry(base)] = (SPARCTranslation *)1;
		_pmegAddr[segTableEntry(base)] = base;
		}
	/*
	 * mark the HW pmegs and put them in the map
	 */
	base  = segmentAlign((char *const)IO_VBASE);
	endmapping = segmentAlign(IO_VBASE + IO_VSIZE );
	for (/*nothing*/; base != endmapping; base += segmentScope) {
		Debug << "SPARCTr::init(): IO pmeg for " <<
			asAddress(base) << ": " <<
			asHex(segTableEntry(base)) << "\n" << eor;
		_pmegMap[segTableEntry(base)] = (SPARCTranslation *)1;
		_pmegAddr[segTableEntry(base)] = base;
		}

	/*
	 * Invalidate everything that is not in use and
	 * null out the rest of the PMEGS which may have
	 * non-invalid page mappings in them.
	 */
	base = topPageSoFar + PhysicalPageSize;
	endmapping = VA_HOLE_START;
	Debug << "invalidating " << asAddress(base) <<
		" through " << asAddress(endmapping) << "\n" << eor;
	/*
	 * Invalidate the rest of the pages in the last kernel
	 * segment
	 */
	for (/*nothing*/; base != segmentAlign(base); base += pageScope){
		Debug << "SPARCTranslation::init: invalidating page at " << asAddress(base) << "\n" << eor;
		unsetPageTableEntry(base);
		}
	/*
	 * Invalidate the rest of the segments up to the hole.
	 * Null out the pmegs along the way since they may have
	 * leftover mappings in them
	 */
	for (/*nothing*/; base < endmapping; base += segmentScope){
		Debug << "SPARCTranslation::init: invalidating segment at " << asAddress(base) << "\n" << eor;
		/*
		 * If the pmeg corresponding to 1-1 mapped base exists
		 * then make sure all its pages are invalid.
		 * This assumes that the machine has less memory
		 * than VA_HOLE_END.
		 */
		unsigned oneToOnePmeg = ((unsigned)base >> Log2SegmentSize);
		if ((oneToOnePmeg < npmegs) && (_pmegMap[oneToOnePmeg] == 0)){
			clearPMEG(oneToOnePmeg);
			}
		setSegTableEntry(base, invalidPMEG);
		}

	/*
	 * Map from the end of the hole to the beginning of the PROM
	 * as invalid.
	 */
	base = VA_HOLE_END;
	endmapping = DMA_END;
	Debug << "SPARCTranslation::init: invalidating segments" 
		<< asAddress(base) << " through " << asAddress(endmapping) 
		<< "\n" << eor;
	for (/*nothing*/; base < endmapping; base += segmentScope) {
		Debug << "SPARCTranslation::init: invalidating segment at " << asAddress(base) << "\n" << eor;
		setSegTableEntry(base, invalidPMEG);
		}

	/*
	 * Copy the initial mappings to all contexts:
	 *
	 * Enable the virtual address cache, prime it so we can run in
	 * other contexts, then do the copying
	 */
	Debug << "Copying to other contexts\n" << eor;
	invalidateContext();
	unsigned ser = readSER();
	ser |= 0x10;
	writeSER(ser);

        //Assert (getContext() == 0);
        for (CONTEXT context = 0; context < 0xff; context++) {
                Debug << "copying to context " << asHex(context) << "\n" << eor;                if (CopyContext(context) != context)
                        break;
                else
                        nctxts = context + 1;   // Count up the contexts
                }
        // Not really in the MMU, but what the hell
#ifdef SET_NUM_CTXTS
	Assert(nctxts >= SET_NUM_CTXTS);
	nctxts = SET_NUM_CTXTS;
#endif
        Console() << "SPARCMMU::SPARCMMU(): " << nctxts << " contexts\n" << eor;        Assert (getContext() == 0);
        _contextMap = new SPARCTranslation*[NUM_CTXTS];
        for (i = 0; i < NUM_CTXTS; i++)
                _contextMap[i] = 0;
#ifndef ENABLE_VA_CACHE
	/*
	 * Disable the virtual address cache (testing, -raila)
	 */
	CONTEXT orig = getContext();
	flushWindows();
	for (CONTEXT c = 0 ; c < NUM_CTXTS; c++) {
		setContext(c);
		flushContext();
		invalidateContext();
		}
	setContext(orig);
	ser = readSER();
	ser &= ~0x10;
	writeSER(ser);
#endif /* ENABLE_VA_CACHE */

	_classInitialized++;

	DummyMemoryObject * vaHole = new DummyMemoryObject(VAHOLESIZE, 0);
	Assert (vaHole != 0);
	TheKernel->domain()->add(vaHole, VA_HOLE_START, NoAccess);
#ifndef BSDNET
	DummyMemoryObject *highKernel = new DummyMemoryObject(HIGHKERNELSIZE, 0);
	Assert (highKernel != 0);
	TheKernel->domain()->add(highKernel, USERMEMORYEND, NoAccess);
#endif /* BSDNET */
	Debug << "SPARCTranslation::init() finished\n" << eor;
}

void 
SPARCTranslation::basicActivate()
{
	Debug << this << "::basicActivate()\n" << eor;
	Assert (this != 0);
	Assert (_classLock != 0);

	if (this == _kernelADT)		// Kernel is always in all ADT's
		return;

	localAcquire();
	flushWindows();

	/*
	 * iF WE are already loaded into a context switch to it.
	 */
	if ( _myContext == -1)
		inhabitMMU();
	else
		setContext(_myContext);

	Assert (getContext() == _myContext);
	localRelease();
	Debug << this << "::basicActivate(): in context " << asHex(getContext()) << "\n" << eor;
}

CONTEXT
SPARCTranslation::findContextVictim()
{
	do {
	_nextVictim++;
	_nextVictim %= NUM_CTXTS;
	} while (_nextVictim == getContext());
	return _nextVictim;
}

void
SPARCTranslation::inhabitMMU()
{
	Debug << this << "::inhabitMMU()\n" << eor;
	/*
	 * Look for an empty context, if we don't find one
	 * pick someone to flush
	 */
	Assert ( this != _kernelADT );
	Assert ( _myContext == -1 );
	CONTEXT c;
	classAcquire();
	for (c = 0; c < NUM_CTXTS; c++) {
		Assert( _contextMap[c] != this);
		if (_contextMap[c] == 0) {
			_myContext = c;
			break;
			}
		}
	if (c == NUM_CTXTS) {	// Didn't find one
		c = findContextVictim();
		_contextMap[c]->relinquishContext();
		_contextMap[c] = this;
		_myContext = c;
		}
	_contextMap[_myContext] = this;
	Debug << this << "::inhabitMMU(): inhabiting context " 
		<< asHex(_myContext) << "\n" << eor;
	flushWindows();
	setContext(_myContext);
	/*
	 * Now we have just entered a context that may have
	 * mappings from the previous ADT in it.  We must set it
	 * up with our saved mappings, or all invalid if we don't
	 * have any mappings.
	 */
	if (_stored){
		restoreSegmentTable();
		_stored = 0;
		}
	else
		initializeSegmentTable();
	classRelease();
}

void
SPARCTranslation::relinquishContext()
{
	Debug << this << "::relinquishContext(): ctxt " 
		<< asHex(_myContext) << "\n" << eor;
	Assert ( this != _kernelADT );
	localAcquire();
	CONTEXT orig = getContext();
	flushWindows();
	setContext(_myContext); 
	flushContext();
	unsigned seg = 0;
	for (char *q = (char*)USERMEMORYSTART; q < USERMEMORYEND; q+= segmentScope)
		_mySegmentPointers[seg++] = segTableEntry(q);
	Assert (seg <= nsegs);
		
	_myContext = -1;
	_stored = 1;
	setContext(orig); 
	localRelease();
}

void
SPARCTranslation::clearPMEG(unsigned which)
{
	Debug << "SPARCTranslation::clearPMEG(" << asHex(which) 
		<< ")\n" << eor;
	/*
	 * Zero out the pte's in the PMEG 
	 */
	char *base = SPARE_SEGPTR;
	char *endmapping = base + segmentScope - 1;
	setSegTableEntry(base, which);
	for (/* nothing*/; base < endmapping; base+=pageScope){
		unsetPageTableEntry(base);
		}
}

void
SPARCTranslation::restoreSegmentTable()
{
	Debug << this << "::restoreSegmentTable()\n" << eor;
	Assert (this != _kernelADT);
	Assert (_myContext == getContext() );
	unsigned seg = 0;
	for (char *q = (char*)USERMEMORYSTART; q < USERMEMORYEND; q+= segmentScope){
#ifdef SBL_restoreSegmentTable
                if (segTableEntry(q) != invalidPMEG)
			flushSegment(q);
#else // SBL_restoreSegmentTable
		flushSegment(q);
#endif // SBL_restoreSegmentTable
		setSegTableEntry(q, _mySegmentPointers[seg++]);
		}
}

void
SPARCTranslation::initializeSegmentTable()
{
	Debug << this << "::initializeSegmentTable()\n" << eor;
	Assert (this != _kernelADT);
	for (char *q = (char*)USERMEMORYSTART; q < USERMEMORYEND; q+= segmentScope){
#ifdef SBL_initializeSegmentTable
                if (segTableEntry(q) != invalidPMEG)
			flushSegment(q);
#else // SBL_initializeSegmentTable
		flushSegment(q);
#endif // SBL_initializeSegmentTable
		setSegTableEntry(q, invalidPMEG);
		}
}

/*
 * Flush the page bits back to the pau's.
 * Mark the PMEGS that we were using as free.
 * Mark our context as Free.
 */
SPARCTranslation::~SPARCTranslation()
{
	Debug << this << "::~SPARCTranslation()\n" << eor;
	Assert( this != 0 );

	localAcquire();
	classAcquire();

	/*
	 * Free up our context 
	 */
	Assert (this != _kernelADT);
	if (_myContext != -1) {
		CONTEXT orig = getContext();
		flushWindows();
		setContext(_myContext); 
		flushContext();
		setContext(orig); 
		_contextMap[_myContext] = 0;
		_myContext = -1;
		}
	for (int i = 0; i < npmegs; i++)
		if ( _pmegMap[i] == this) {
			clearPMEG(i);
			_pmegMap[i] = 0;
			_pmegAddr[i] = 0;
		}
	classRelease();
	localRelease();
}

SEGMENT_TABLE_ENTRY
SPARCTranslation::reclaimPMEG()
{
	unsigned pmegPtr = invalidPMEG;
	unsigned which;
	unsigned count = 0;
	for (which = 0; which < npmegs; which++){
		if (which == invalidPMEG) continue;
		/*
		 * for now don't blow away user segments 
		 */
		if (_pmegMap[which] != _kernelADT) continue;
		/*
		 * Probe the pte's in the PMEG 
		 */
		unsigned probes = 0;
		unsigned busy = 0;
		char *base = SPARE_SEGPTR;
		char *endmapping = base + segmentScope - 1;
		setSegTableEntry(base, which);
		for (/* nothing*/; base < endmapping; base+=pageScope){
			if (pageTableEntry(base) != 0)
				busy++;
			probes++;
			}
/*
		Console() << "SPARCTranslation::reclaimPMEG()"
			<< ": pmeg " << which << ": " << busy
			<< " of " << probes << " pte's busy:"
			<< " owner is " 
			<< asHex((unsigned)_pmegMap[which]) 
			<< "\n" << eor;
 */
		setSegTableEntry(SPARE_SEGPTR, invalidPMEG);
		if (busy != 0) 
			continue;
		/*
		 * blow away this pmeg from owner 
		 */
		SPARCTranslation * owner = _pmegMap[which];
	
		if ( owner != _kernelADT ) {

			Console() << "SPARCTranslation::reclaimPMEG() "
				  << "cannot blow away unused USER segment " 
				  << which << " yet\n" << eor;
			continue;
			/* not kernel */
			/* if not active blow away in stored segment table */
			/* else set context blow away */
			/* set context */
			/*
			for (char *q = (char*)USERMEMORYSTART; 
			     q < USERMEMORYEND; q += segmentScope){
                		if (segTableEntry(q) == which )
					setSegTableEntry(q, invalidPMEG);
				}
			 */
			}
		else {
			char * a = _pmegAddr[which];
			Assert( a < VA_HOLE_START );
			CONTEXT myContext = getContext();
			for ( CONTEXT i = 0; i < NUM_CTXTS; i++ ) {
				setContext( i );
				Assert( segTableEntry(a) == which );
				setSegTableEntry(a, invalidPMEG);
				}
			setContext( myContext );
			}
		_pmegMap[which] = 0;
		_pmegAddr[which] = 0;
		count++;
		if ( pmegPtr == invalidPMEG )
			pmegPtr = which;
		}
	Assert( pmegPtr != invalidPMEG );
	Assert( count > 0 );
/*
	Console() << "SPARCTranslation::reclaimPMEG() reclaimed "
		<< count << " returning " << pmegPtr << "\n" << eor;
 */
	return pmegPtr;
}

SEGMENT_TABLE_ENTRY
SPARCTranslation::findPMEGForAddress(char * const address)
{
	/*
	 * If there isn't a PMEG pointed to already then allocate one.
	 */
	SEGMENT_TABLE_ENTRY pmegPtr = segTableEntry( address ) ;
	if (pmegPtr == invalidPMEG) {
		classAcquire();
		unsigned i;
		for (i = 0; i < npmegs; i++){
			if (_pmegMap[i] == 0){
				break;
				}
			}	
#ifdef SBL_countPMEG
                unsigned freepmegs = 0;
                unsigned j;
                for (j = i + 1; j < npmegs; j++){
                        if (_pmegMap[j] == 0)
                                freepmegs++;
                        }
#endif // SBL_countPMEG
#ifdef SBL_countPMEG
                if (freepmegs <= 5) {
                        Console() << freepmegs << " PMEGS left\n" << eor;
                }
#endif // SBL_countPMEG

		/*
		 * we got here with or without finding a PMEG.
		 */
		if ( i < npmegs) {
			/*
			 * we found one!
			 */
			pmegPtr = i;
			}
		else {
			/*
		 	 * reclaim pmegs
			 */
			pmegPtr = reclaimPMEG();
			}
		_pmegMap[pmegPtr] = this;
		_pmegAddr[pmegPtr] = address;
                classRelease();
		}
	Assert(pmegPtr != invalidPMEG);
	Assert( _pmegMap[pmegPtr] == (SPARCTranslation *)1 || _pmegMap[pmegPtr] == this);
	return pmegPtr;
}

/*
 * Just doublecheck that the range passed is all mapped in
 */
int
SPARCTranslation::assertValidAddressRange(char *base, unsigned len)
{
	base = pageAlign(base);
	char *end = pageAlign(base + len - 1);
	while (base <= end){
		Debug << "assertValidAddressRange(): checking addr " <<
			asAddress(base) << "\n" << eor;
		PAGE_TABLE_ENTRY pte = pageTableEntry(base);
		if (! pte.valid() || ! pte.okToPerform(SystemRead) ||
			! pte.okToPerform(SystemWrite))
			return 1 ;
		base+= pageScope;
		}
	return 0;
}

	/*
	 * returns the DMA virtual address and remembers which segment
	 * it used for unMapDMA to use.  Don't access the pointer returned
	 * since it is in the (possibly different) DMA context.
	 */
static SPARCCacheFlushTimer mapDMAInst("SPARCmapToDMAInstrument");
char *
SPARCTranslation::mapToDMA(char *const userVaddr, unsigned len, int multiCtxt)
{
#ifdef BSDNET
	Assert( NOTREACHED );
#endif /* BSDNET */

	SInstr(int start = mapDMAInst.start());
	Debug << "SPARCTranslation::mapToDMA( " << asAddress(userVaddr) << ", " << asHex(len) << " ) in context " << asHex(getContext()) << "\n" << eor;

	Assert (len > 0);
#ifdef SBL_assertValidAddressRange
	if (assertValidAddressRange((char*)userVaddr, len)){
		Console() << "SPARCTranslation::mapToDMA(" <<
			asAddress(userVaddr) << ", " << asHex(len) <<
			"in ctxt: " << asHex(getContext()) << " failed\n"
			<< eor;
		Assert (NOTREACHED);
		}
#endif // SBL_assertValidAddressRange

	/*
	 * Figure out how many segments we'll need
	 */
	char *segEnd = segmentAlign(userVaddr + len - 1);
	char *segBase = segmentAlign(userVaddr);
	unsigned segments = 1 + ((segEnd - segBase)  / SegmentSize );
	Assert (segments > 0);
#ifdef DMA_DEBUG
	Console() << "SPARCTranslation::mapToDMA( " << segments << " segments necessary \n" << eor;
#endif

	_dmaLock->acquire();
		/* 
		 * Find the next available DMA segment pointer
		 */
	unsigned i;
	unsigned found = 0;
	for (i = 0; i < (DMA_SEGS - segments + 1 ); i++)
		if ( _dmaMap->get(i, i+segments-1)  == 0){
			_dmaMap->set(i, i+segments-1);
			found = 1;
			break;
			}
	Assert (found);
	Assert( (i + segments - 1) < DMA_SEGS );
	unsigned dmaseg = i;
	Assert (_dmaMappings[dmaseg] == 0);
	Assert (_dmaLens[dmaseg] == 0);
	_dmaMappings[dmaseg] = userVaddr;
	_dmaLens[dmaseg] = len;

	CONTEXT context = getContext();
	flushWindows();

	/*
	 * Flush cache
	 */
	char * pageBase = pageAlign(userVaddr);
	char * pageEnd = pageAlign(userVaddr + len - 1);
	for (char * addr = pageBase; addr <= pageEnd; addr += PhysicalPageSize)
	{
               	PAGE_TABLE_ENTRY pte = pageTableEntry(addr);
#ifdef SBL_OPTIMISTIC
		if (pte.referenced())
			flushPage(addr);
#else // SBL_OPTIMISTIC
		flushPage(addr);
#endif // SBL_OPTIMISTIC
		}
	/*
	 * Copy the user segment ptr mappings
	 */
	int endctxt;
	if (multiCtxt) 
		endctxt = NUM_CTXTS;
	else
		endctxt = 1;
	unsigned offset = segmentOffset(userVaddr);
	for (int seg = 0; seg < segments; seg++) {
		char *segaddr =(char *)USERMEMORYEND + ((dmaseg + seg)*SegmentSize);
		setContext(context);
		SEGMENT_TABLE_ENTRY userSegPtr = 
			segTableEntry(segBase + (seg * SegmentSize));
		Assert (userSegPtr != invalidPMEG);
		Assert (segTableEntry(segaddr) == invalidPMEG);
		for (unsigned ctxt = 0; ctxt < endctxt; ctxt++) {
			setContext(ctxt);
			setSegTableEntry(segaddr, userSegPtr);
			}
		}

	setContext(context);
	_dmaLock->release();

	char *dmaAddr = (char *)USERMEMORYEND + (dmaseg * SegmentSize) + offset;
#ifdef DMA_DEBUG
	Console() << " returns " << asAddress(dmaAddr) << "\n" << eor;
#endif
   
	SInstr(mapDMAInst.stop(start));
	return dmaAddr;
}

	/*
	 * Let someone reuse the DMA segment table entry and return the
	 * address we were mapped to
	 */
static SPARCCacheFlushTimer unmapDMAInst("SPARCunmapDMAInstrument");
char *
SPARCTranslation::unMapDMA(char *const dmaAddr)
{
#ifdef BSDNET
	Assert( NOTREACHED );
#endif /* BSDNET */

	SInstr(int start = unmapDMAInst.start());
	Debug << "SPARCTranslation::unMapDMA( " << asAddress(dmaAddr) 
		<< ") in context " << asHex(getContext()) << "\n"  << eor;

	char *segBase = segmentAlign(dmaAddr);
	unsigned dmaseg = (segBase - (char *)USERMEMORYEND) / SegmentSize;
	unsigned len = _dmaLens[dmaseg];
	Assert (len > 0);
	char *segEnd = segmentAlign(dmaAddr + len - 1);
	Assert (segBase >= (char *)USERMEMORYEND );
	unsigned segments = 1 + ((segEnd - segBase)  / SegmentSize );
	Assert(segments > 0);

	_dmaLock->acquire();

	CONTEXT context = getContext();
	flushWindows();
	setContext(0);	// DVMA is in ctxt 0.
	_dmaMap->clear(dmaseg, dmaseg + segments - 1);

	/*
   	 * Remove segment ptr
	 */
	for (int seg = 0; seg < segments; seg++) {
		char *segaddr =(char *)USERMEMORYEND + ((dmaseg + seg)*SegmentSize);
		Assert (segTableEntry(segaddr) != invalidPMEG);
		setSegTableEntry(segaddr, invalidPMEG);
		}

	setContext(context);

	Assert (_dmaMappings[dmaseg] != 0);
	char *userAddr = (char *) _dmaMappings[dmaseg];
	_dmaMappings[dmaseg] = 0;
	_dmaLens[dmaseg] = 0;
	_dmaLock->release();
#ifdef DMA_DEBUG
	Console() << " returns " << asAddress(userAddr) << "\n" << eor;
#endif
	SInstr(unmapDMAInst.stop(start));
	return userAddr;
}

char *
SPARCTranslation::makeUncachable(char *const userVaddr, unsigned len)
{
	Debug << "SPARCTranslation::makeUncachable( " << asAddress(userVaddr) << ", " << asHex(len) << " ) in context " << asHex(getContext()) << "\n" << eor;

	Assert (len > 0);
	Assert ((userVaddr >= KERNEL_START_VIRTUAL && 
		 userVaddr < KERNEL_END_VIRTUAL) ||
		(userVaddr >= DMA_BASE &&
	         userVaddr < DMA_END));
	Assert ( userVaddr == pageAlign( userVaddr ) &&
	         (len & (PhysicalPageSize - 1)) == 0 );

	_dmaLock->acquire();
	CONTEXT context = getContext();
	flushWindows();
	/*
	 * Flush cache
	 */
	char * pageBase = pageAlign(userVaddr);
	char * pageEnd = pageAlign(userVaddr + len - 1);
	for (char * addr = pageBase; addr <= pageEnd; 
	     addr += PhysicalPageSize) {
               	PAGE_TABLE_ENTRY pte = pageTableEntry(addr);
		Assert(pte.valid());
#ifdef SBL_OPTIMISTIC
		if (pte.referenced()) 
			flushPage(addr);
#else SBL_OPTIMISTIC
		flushPage(addr);
#endif // SBL_OPTIMISTIC
                unsigned v = pte.value() | PG_NC;
                asm volatile ("sta %1, [%0]4" : : "r" (addr), "r" (v) );
		}
	setContext(context);
	_dmaLock->release();

#ifdef DMA_DEBUG
	Console() << " returns " << asAddress(userVaddr) << "\n" << eor;
#endif
   
	return userVaddr;
}

	/*
	 * prepare for DMA by flushing cache entries
	 */
static SPARCCacheFlushTimer prepareDMAInst("SPARCprepareForDMAInstrument");
char *
SPARCTranslation::prepareForDMA(char *const userVaddr, unsigned len, int read)
{
	SInstr(int start = prepareDMAInst.start());
	Debug << "SPARCTranslation::prepareForDMA( " << asAddress(userVaddr) << ", " << asHex(len) << " ) in context " << asHex(getContext()) << "\n" << eor;

	Assert (len > 0);
	Assert (userVaddr >= KERNEL_START_VIRTUAL && 
		userVaddr < KERNEL_END_VIRTUAL);

	/* do nothing */

#ifndef SBL_newDMA
	_dmaLock->acquire();
	CONTEXT context = getContext();
	flushWindows();
	/*
	 * Flush cache
	 */
	char * pageBase = pageAlign(userVaddr);
	char * pageEnd = pageAlign(userVaddr + len - 1);
	for (char * addr = pageBase; addr <= pageEnd; 
	     addr += PhysicalPageSize) {
               	PAGE_TABLE_ENTRY pte = pageTableEntry(addr);
#ifdef SBL_OPTIMISTIC
		if (!pte.referenced()) 
			continue;
#endif // SBL_OPTIMISTIC
		flushPage(addr);
		}
	setContext(context);
	_dmaLock->release();

#ifdef DMA_DEBUG
	Console() << " returns " << asAddress(userVaddr) << "\n" << eor;
#endif
   
#endif // not SBL_newDMA

	SInstr(prepareDMAInst.stop(start));
	return userVaddr;
}

	/*
	 * cleanup ref and mod bits after DMA
	 */
static SPARCCacheFlushTimer postDMAInst("SPARCpostDMAInstrument");
char *
SPARCTranslation::postDMA(char *const userVaddr, unsigned len, int read)
{
	SInstr(int start = postDMAInst.start());
	Debug << "SPARCTranslation::postDMA( " << asAddress(userVaddr) 
	      << ", " << asHex(len) << " ) in context " 
	      << asHex(getContext()) << "\n" << eor;

#ifdef NEVER
	if ( userVaddr != pageAlign( userVaddr ) ||
	     (len & (PhysicalPageSize - 1)) != 0 ) {
		SInstr(postDMAInst.stop(start));
		return userVaddr;
	}
#endif /* NEVER */
	Assert (len > 0);
	Assert (userVaddr >= KERNEL_START_VIRTUAL && 
		userVaddr < KERNEL_END_VIRTUAL);

	_dmaLock->acquire();
	flushWindows();
	const unsigned mask = ~( PG_R | PG_M );
	char * pageBase = pageAlign(userVaddr);
	char * pageEnd = pageAlign(userVaddr + len - 1);
	for (char * addr = pageBase; addr <= pageEnd; 
	     addr += PhysicalPageSize) {
               	PAGE_TABLE_ENTRY pte = pageTableEntry(addr);
                unsigned v = pte.value() & mask;
                asm volatile ("sta %1, [%0]4" : : "r" (addr), "r" (v) );
#if defined(SBL_newDMA) && !defined(SBL_OPTIMISTIC)
		if( !read )
			continue;
#endif // defined(SBL_newDMA) && !defined(SBL_OPTIMISTIC)
#ifdef SBL_OPTIMISTIC
                if( !pte.referenced())
                        continue;
#endif // SBL_OPTIMISTIC
                flushPage(addr);
		}
	_dmaLock->release();

#ifdef DMA_DEBUG
	Console() << " returns " << asAddress(userVaddr) << "\n" << eor;
#endif
   
	SInstr(postDMAInst.stop(start));
	return userVaddr;
}

void
SPARCTranslation::addMapping( const char *vaddr,
	PhysicalMemoryChain * chain, ProtectionLevel maxProtection )
{
	char *virtualAddress = (char*)vaddr;
	int userPages = 0;
	if (vaddr >= USERMEMORYSTART && vaddr < USERMEMORYEND) {
		Assert (this != _kernelADT);
		userPages = 1;
	        }

	Debug << this << "::addMapping( vaddr: " << asAddress( virtualAddress ) << " chain: " << asAddress( chain ) << " maxProt: " << maxProtection 
	<< "\n" << eor;
	Assert( this != 0 );
	Assert( chain != 0 );
	Assert( _classInitialized );
	unsigned int nbytes = chain->lengthInBytes();
	if( nbytes == 0 ) {
		return;
	}

	/*
	 * Make sure we are not adding to the VA HOLE
	 */
	Assert (virtualAddress < VA_HOLE_START ||
		virtualAddress >= VA_HOLE_END);
	Assert (virtualAddress + nbytes < VA_HOLE_START || 
		virtualAddress + nbytes >= VA_HOLE_END);

	SPARCProtection protectionbits;
	switch (maxProtection) {
		case NoAccess:
			protectionbits = SYSTEMRW;
			break;
		case ReadOnly:
			protectionbits = USERRO;
			// Assert( userPages );
			break;
		case ReadWrite:
			protectionbits = USERRW;
			// Assert( userPages );
			break;
		default:
			Assert(NOTREACHED);
			break;
		};

	localAcquire();
	flushWindows();
	CONTEXT originalContext = getContext();
	CONTEXT first;
	CONTEXT last;
	if (userPages) {
		if (_myContext == -1) 
			inhabitMMU();
		first = last = _myContext;
		}
	else	{
		first = 0;
		last = NUM_CTXTS - 1;
		}

	/*
	 * Round the virtual address down to the base of the page it falls
	 * on. Then calculate how many pages to map in order to map nbytes.
	 */
	char * vpagebase = pageAlign( virtualAddress );
	char * endmapping = pageAlign( virtualAddress + nbytes - 1);
	PhysicallyAddressableUnit * pau = chain->chain();

	Assert( chain->offset() == (virtualAddress - vpagebase) );

	Debug << this << "::addMapping: " << asAddress( vpagebase ) << 
		" through " <<  asAddress(endmapping) << " -> " << 
		asAddress( pau->address() ) << ", " << asHex(nbytes) 
		<< " bytes" << " in context: " << asHex(getContext()) 
		<< " to context " << asHex(_myContext) 
		<< "prot: " << asHex(maxProtection)
		<< "\n" << eor;

        /*
         * Set up the segment table first
         *
         * We find and existing PMEG mapping or get a new one.
         * If we get the same that we had, check to see that
         * we are mapped to the same place
         */
 
	CONTEXT c;
        char * segment = segmentAlign(vpagebase);
        char * endsegment = segmentAlign(endmapping);
        while ( segment <= endsegment ) {
                SEGMENT_TABLE_ENTRY pmeg = segTableEntry(segment);
                if (pmeg == invalidPMEG) {
                        pmeg = findPMEGForAddress(segment);
                        for (c = first; c <= last; c++) {
                                setContext(c);
                                setSegTableEntry(segment, pmeg);
                                }
                        }
                segment += segmentScope;
                }

	setContext(first);
	while ( vpagebase <= endmapping ) {
		Assert( pau != 0 );

		/*
		 * Now that the segment table is set, set up the
		 * page entry in the pmeg that it points to.
		 * If it is already mapped make sure it points to
		 * the same place. 
		 */
		PAGE_TABLE_ENTRY pte = pageTableEntry(vpagebase);
		if (pte.valid()) {
			Assert (pte.addressMappedTo() == pau->address());
			/*
			 * Check to see if translation entry needs to be
			 * be updated, update if protection changed
			 */
			if ((pte.value() & protectionMask) !=
			    protectionbits) {

				/*
			 	 * Update mod bit in pau, so no need to turn
			 	 * pte mod bit on when changing protection
			 	 */
				if (!pau->modified() && pte.modified()) 
					pau->setModified(1);
				/*
			 	 * If referenced, then update ref bit in pau.
			 	 * No need to set pte ref bit on if cache 
				 * is flushed
			 	 * so pte ref bit can be used to determine 
			 	 * accurately to flush cache or not
			 	 */
#ifdef SBL_OPTIMISTIC
				if (pte.referenced()) {
			  		if (!pau->referenced())
						pau->setReferenced(1);
					flushPage(vpagebase);
					}
#else // SBL_OPTIMISTIC
				if (pte.referenced() && !pau->referenced()) 
					pau->setReferenced(1);
				flushPage(vpagebase);
#endif // SBL_OPTIMISTIC
				setPageTableEntry(vpagebase, pau->address(), 
					  	  protectionbits);
				}
			}
		else {
			setPageTableEntry(vpagebase, pau->address(), 
					  protectionbits);
			}
		/*
		 * Next page and next pau
		 */
		pau = pau->nextLink();
		vpagebase += PhysicalPageSize;
	}
	setContext(originalContext);
	localRelease();
	Debug << this << "::addMapping() done\n" << eor; 
}

/*
 * Remove a mapping and update the machine independent ref/mod information
 * for a range of virtual addresses.
 */
void
SPARCTranslation::basicRemoveMapping( const char *vaddr,
	unsigned int nbytes )
{
	Assert( this != 0 );
	char * virtualAddress = (char*)vaddr;
	Debug << this << "::removeMapping( " << asAddress( virtualAddress ) 
		<< ", " << asHex(nbytes) << " )\n" << eor;

	int userPages = 0;
	if (vaddr >= USERMEMORYSTART && vaddr < USERMEMORYEND) {
		Assert (this != _kernelADT);
		userPages = 1;
	        }

	Assert (virtualAddress < VA_HOLE_START ||
		virtualAddress >= VA_HOLE_END);
	Assert (virtualAddress + nbytes < VA_HOLE_START || 
		virtualAddress + nbytes >= VA_HOLE_END);
	if( nbytes == 0 ) {
		Console() << "SPARCTranslation::basicRemoveMapping(): WARNING: zero bytes!\n" << eor;
		return;
	}

	/*
	 * Round the virtual address down to the base of the page it
	 * falls on. Then calculate how many pages to remove in order
	 * to satisfy "nbytes".  For now, require the the virtual
	 * address be page-aligned.
	 */
	char * vpagebase = pageAlign( virtualAddress );
	char * lastpage = pageAlign( virtualAddress + nbytes - 1);
	Assert( PageAligned( virtualAddress ) );
	Debug << this << "::removeMapping: " << asAddress( vpagebase ) 
		<< " through " << asAddress(lastpage) << ", " << 
		asHex(nbytes) << " bytes\n" << eor;

	CONTEXT ctxt;
	if (userPages) {
		if (_myContext == -1) {	// Just blow the segment from the table
			if (! _stored)
				return;
			char *p = segmentAlign(vpagebase);
			while (p < virtualAddress + nbytes) { 
				unsigned offset = (p - USERMEMORYSTART) / segmentScope;
				if (_mySegmentPointers[offset] != invalidPMEG) {
					clearPMEG(_mySegmentPointers[offset]);
					Assert (_pmegMap[_mySegmentPointers[offset]] == this);
					_pmegMap[_mySegmentPointers[offset]] = 0;
					_pmegAddr[_mySegmentPointers[offset]] = 0;
					_mySegmentPointers[offset] = invalidPMEG;
					}
				p += SegmentSize;
				}
			return;
			}
		ctxt = _myContext;
		}
	else {
		ctxt = 0;
		}
	localAcquire();
	CONTEXT orig = getContext();
	flushWindows();
	setContext(ctxt);
	while (vpagebase <= lastpage) {
		if (segTableEntry(vpagebase) == invalidPMEG){
			vpagebase = segmentAlign(vpagebase) + SegmentSize;
			continue;
			}
		Debug << "Invalidating page at " << asAddress(vpagebase) 
			<< "\n" << eor;

		PAGE_TABLE_ENTRY pte = pageTableEntry(vpagebase);

#ifdef SBL_OPTIMISTIC
		if (pte.referenced())
			flushPage(vpagebase);
#else // SBL_OPTIMISTIC
		flushPage(vpagebase);
#endif // SBL_OPTIMISTIC

		unsetPageTableEntry(vpagebase);

		if( pte.valid() ) {
			Debug << this << "::removeMapping: do it \n" << eor;

			/*
			 * Copy the ref/mod info into the machine independent
			 * pau's tables.
			 */
			Assert( MainMemory != 0 );
			PhysicallyAddressableUnit * pau = MainMemory->lookup(
				pte.addressMappedTo() );
			Assert( pau != 0 );

			if (pte.referenced() && !pau->referenced())
				pau->setReferenced(1);
			if (pte.modified() && !pau->modified())
				pau->setModified(1);

			Debug << this << "::removeMapping: dirty: "
				<< pau->modified() << " ref: "
				<< pau->referenced() << "\n" << eor;
		}
		vpagebase += PhysicalPageSize;
	}
	
	setContext(orig);
	localRelease();
	Debug << this << "::removeMapping: done\n" << eor;
}

int
SPARCTranslation::basicChangeProtection( const char *vaddr,
	unsigned int nbytes, ProtectionLevel newProtection )
{
	int userPages = 0;
	if (vaddr >= USERMEMORYSTART && vaddr < USERMEMORYEND) {
		Assert (this != _kernelADT);
		userPages = 1;
	        }

	char *virtualAddress = (char*)vaddr;
	Debug << this << "::basicChangeProtection( vAddress: "
		<< asAddress( virtualAddress ) << " newProtection: "
		<< newProtection << "\n" << eor;

	char *base = pageAlign(virtualAddress);
	char *lastpage = pageAlign(virtualAddress + nbytes - 1);

	SPARCProtection protectionbits;
	switch (newProtection) {
		case NoAccess:
			protectionbits = SYSTEMRW;
			break;
		case ReadOnly:
			protectionbits = USERRO;
			// Assert( userPages );
			break;
		case ReadWrite:
			protectionbits = USERRW;
			// Assert( userPages );
			break;
		default:
			Assert(NOTREACHED);
			break;
		};

	CONTEXT ctxt;
	if (userPages) {
		if (_myContext == -1) {	// Just blow the segment from the table
			if (! _stored)
				return 0;
			char *p = segmentAlign(virtualAddress);
			while (p < virtualAddress + nbytes) {
				unsigned offset = (p - USERMEMORYSTART) / segmentScope;
				if (_mySegmentPointers[offset] != invalidPMEG){
					clearPMEG(_mySegmentPointers[offset]);
					Assert (_pmegMap[_mySegmentPointers[offset]] == this);
					_pmegMap[_mySegmentPointers[offset]] = 0;
					_pmegAddr[_mySegmentPointers[offset]] = 0;
					_mySegmentPointers[offset] = invalidPMEG;
					}
				p += SegmentSize;
				}
			return 1;
			}
		ctxt = _myContext;
		}
	else
		{
		ctxt = 0;
		}

	Debug << this << "::basicChangeProtection() : " << asAddress( base ) 
		<< " through: " << asAddress(lastpage) << "\n" << eor;
	
	
	localAcquire();
	CONTEXT orig = getContext();
	flushWindows();
	setContext(ctxt);
	int rval = 0;
	while (base <= lastpage) {
		PAGE_TABLE_ENTRY old = pageTableEntry(base);
		if ( !old.valid() ) {
			base += PhysicalPageSize;
			continue;
		    }

		rval = 1;
		Assert (old.valid());

		if ((old.value() & protectionMask) != protectionbits) {
			if (old.referenced()) {
                                PhysicallyAddressableUnit * pau = 
			            MainMemory->lookup( old.addressMappedTo() );
                		Assert( pau != 0 );
				if (!pau->referenced()) 
					pau->setReferenced(1);
#ifdef SBL_OPTIMISTIC
                                flushPage(base);
				}
#else // SBL_OPTIMISTIC
				}			
                        flushPage(base);
#endif // SBL_OPTIMISTIC
			// preserve mod bit
		        SPARCProtection mask;
			if (old.modified())
				mask = PG_M;
			else
				mask = 0;
			setPageTableEntry(base, old.addressMappedTo(), 
					  protectionbits | mask);
			} 

		base += PhysicalPageSize;
		}
	setContext(orig);
	localRelease();
	Debug << this << "::basicChangeProtection() done\n" << eor; 
	return( rval );
}

int
SPARCTranslation::basicChangeStatus( const char *vaddr, unsigned int nbytes, 
				     AddressTranslationStatus clear,
				     AddressTranslationStatus set )
{
#ifdef ASSERT
	int userPages = 0;
	if (vaddr >= USERMEMORYSTART && vaddr < USERMEMORYEND) {
		Assert (this != _kernelADT);
		userPages = 1;
	        }
	Assert( !userPages );
#endif // ASSERT

#ifdef SBL_OPTIMISTIC
	int clearingRef = 0;
#endif // SBL_OPTIMISTIC

	unsigned onbits = 0;
	unsigned offbits = 0xffffffff;
	if (set.value & AddressTranslationStatus::Modified) 
		onbits |= PG_M;
	if (clear.value & AddressTranslationStatus::Modified) 
		offbits &= ~PG_M;
	if (set.value & AddressTranslationStatus::Referenced) 
		onbits |= PG_R;
	if (clear.value & AddressTranslationStatus::Referenced) {
#ifdef SBL_OPTIMISTIC
		clearingRef = 1;
#endif // SBL_OPTIMISTIC
		offbits &= ~PG_R;
	}

	int ret = 0;
	char * pageBase = pageAlign((char *)vaddr);
	char * pageEnd = pageAlign((char*)vaddr + nbytes - 1);
	for (char *addr = pageBase; addr <= pageEnd; addr += PhysicalPageSize){
		PAGE_TABLE_ENTRY pte = pageTableEntry(addr);
		if (!pte.valid()) 
			continue;
#ifdef SBL_OPTIMISTIC
		if ( clearingRef && pte.referenced() ) 
                        flushPage(addr);
#endif // SBL_OPTIMISTIC
		unsigned v = (pte.value() | onbits) & offbits;
		asm volatile ("sta %1, [%0]4" : : "r" (addr), "r" (v) );
		}
	return 1;
}

int
SPARCTranslation::basicReferenced( const char *vaddr, unsigned int nbytes, 
				   int clear ) 
{
	int userPages = 0;
	if (vaddr >= USERMEMORYSTART && vaddr < USERMEMORYEND) {
		Assert (this != _kernelADT);
		userPages = 1;
	        }

	char *virtualAddress = (char*)vaddr;
	Debug << this << "::basicReferenced( vAddress: "
		<< asAddress( virtualAddress ) << " nbytes: " << nbytes
		<< " clear: " << clear << "\n" << eor;

	char *base = pageAlign(virtualAddress);
	char *lastpage = pageAlign(virtualAddress + nbytes - 1);

	localAcquire();
	flushWindows();
	CONTEXT originalContext = getContext();
	CONTEXT context;
	if (userPages) {
		if (_myContext == -1) 
			inhabitMMU();
		context = _myContext;
		}
	else	{
		context = 0;
		}

	int ret = 0;
	char * pageBase = pageAlign((char *)vaddr);
	char * pageEnd = pageAlign((char*)vaddr + nbytes - 1);
	setContext( context );
	for (char *addr = pageBase; addr <= pageEnd; addr += PhysicalPageSize){
		PAGE_TABLE_ENTRY pte = pageTableEntry(addr);
		if (!pte.valid() || !pte.referenced())
			continue;
		ret = 1;
		if (clear) {
#ifdef SBL_OPTIMISTIC
                        flushPage(addr);
#endif // SBL_OPTIMISTIC
			unsigned v = (pte.value() & (~PG_R)) ;
			asm volatile ("sta %1, [%0]4" : : "r" (addr), "r" (v) );
			}
		}
	setContext(originalContext);
	localRelease();
	return ret;
}

void
SPARCTranslation::fixKernelInitFault(char *addr)
{
	/*
	 * This is a page fault in the VM initialization
	 * code.  We know the fault should be in 1-1 mapped
	 * memory, so just stick the mapping in.
	 */
	Debug << "SPARCTranslation::fixKernelInitFault(" << asAddress(addr) 
		<< ")\n" << eor;
	Assert (getContext() == 0);
	addr = pageAlign(addr);

	if (segTableEntry(addr) == invalidPMEG) {
		Debug << "Need a new PMEG\n" << eor;
		SEGMENT_TABLE_ENTRY pmeg = 
			((unsigned)addr >> Log2SegmentSize) & 0x7f;
		Debug << "new PMEG is " << asHex(pmeg) << "\n" << eor;
		flushWindows();
                CONTEXT last = NUM_CTXTS;
#ifdef NEVER
                if (_classInitialized) last = NUM_CTXTS;
                else
                        last = 1;
#endif
		for (CONTEXT c = 0; c < last; c++) {
			setContext(c);
			flushSegment(addr);
			setSegTableEntry(addr, pmeg);
			_pmegMap[pmeg] = (SPARCTranslation*)1;
			_pmegAddr[pmeg] = addr;
			}
		setContext(0);
		}
	flushPage(addr);
	setPageTableEntry(addr, addr, SYSTEMRW);
	Assert (getContext() == 0);
	Debug << "SPARCTranslation::fixKernelInitFault() done\n" << eor;;
}
