#ifdef __GNUC__
#pragma implementation
# endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
#include "Assert.h"
#include "ClassStar.h"
#include "Interrupt.h"
#include "TIMESLICE_TIMER.h"
#include "TimerManager.h"
#include "VCCPU.h"
#include "CPU.h"
#include "ThisCPU.h"
#include "VCMMU.h"
#include "VCInterrupts.h"
#include "OutputStream.h"
#include "PROCESSOR.h"
extern "C" {
#include <stdio.h>
//#include <unistd.h>
#include <stdlib.h>
#include <signal.h>
//#include <sys/trap.h>
};

#define DEBUG
#include "Debug.h"

#ifdef __GNUC__
#pragma implementation
#endif

Interrupt * VCCPU::interruptTable[ nInterrupts ];

#ifdef INSTRUMENT
#include "SystemInstrumentManager.h"
InstrumentBufferInterrupt TheInstrumentBufferInterrupt;
#endif

/*
 * Static class stuff
 */
sigset_t   VCCPU::_interrupts;

extern "C" int getpid();

void
VCCPU::initSignalStuff()
{

	/*
	 * This shit is so gdb doesn't barf, I have no idea why it works!
	 */
	//sigignore(SIGTRAP);
	//kill(getpid(), SIGTRAP);

	/*
	 * Set up the interrupt set once at the beginning
	 */
	 sigaddset(&_interrupts, SIGHUP);
	sigaddset(&_interrupts, SIGINT);
	sigaddset(&_interrupts, SIGQUIT);
	sigaddset(&_interrupts, SIGALRM);
	sigaddset(&_interrupts, SIGTERM);
	//sigaddset(&_interrupts, SIGUSR1);
	//sigaddset(&_interrupts, SIGUSR2);
	sigaddset(&_interrupts, SIGWINCH);
	sigaddset(&_interrupts, SIGURG);
#ifdef EMCHAN
	sigaddset(&_interrupts, SIGPOLL);
#endif
	sigaddset(&_interrupts, SIGTSTP);
	sigaddset(&_interrupts, SIGCONT);
	sigaddset(&_interrupts, SIGTTIN);
	sigaddset(&_interrupts, SIGTTOU);
	//sigaddset(&_interrupts, SIGIO);
	sigaddset(&_interrupts, SIGVTALRM);
}


int
VCCPU::interruptsEnabled()
{
	sigset_t current;
	sigprocmask(SIG_BLOCK, 0, &current);

	int ret = 
		!sigismember(&current, SIGHUP) &&
		!sigismember(&current, SIGINT) &&
		!sigismember(&current, SIGQUIT) &&
		!sigismember(&current, SIGALRM) &&
		!sigismember(&current, SIGTERM) &&
		!sigismember(&current, SIGUSR1) &&
		!sigismember(&current, SIGUSR2) &&
		!sigismember(&current, SIGWINCH) &&
		!sigismember(&current, SIGURG) &&
	  //!sigismember(&current, SIGIO) &&
#ifdef EMCHAN
		!sigismember(&current, SIGPOLL) &&
#endif
		!sigismember(&current, SIGTSTP) &&
		!sigismember(&current, SIGCONT) &&
		!sigismember(&current, SIGTTIN) &&
		!sigismember(&current, SIGTTOU) &&
		!sigismember(&current, SIGVTALRM) ;
// #define DEBUG_INT
#ifdef DEBUG_INT
#include <stdio.h>
	printf("VCCPU::interruptsEnabled(): " );
	if (sigismember(&current, SIGHUP))
		 printf( " SIGHUP" );
	if (sigismember(&current, SIGINT))
		 printf( " SIGINT" );
	if (sigismember(&current, SIGQUIT))
		 printf( " SIGQUIT" );
	if (sigismember(&current, SIGALRM))
		 printf( " SIGALRM" );
	if (sigismember(&current, SIGTERM))
		 printf( " SIGTERM" );
	if (sigismember(&current, SIGUSR1))
		 printf( " SIGUSR1" );
	if (sigismember(&current, SIGIO))
		 printf( " SIGIO" );
	if (sigismember(&current, SIGUSR2))
		 printf( " SIGUSR2" );
	if (sigismember(&current, SIGWINCH))
		 printf( " SIGWINCH" );
	if (sigismember(&current, SIGURG))
		 printf( " SIGURG" );
	if (sigismember(&current, SIGPOLL))
		 printf( " SIGPOLL" );
	if (sigismember(&current, SIGTSTP))
		 printf( " SIGTSTP" );
	if (sigismember(&current, SIGCONT))
		 printf( " SIGCONT" );
	if (sigismember(&current, SIGTTIN))
		 printf( " SIGTTIN" );
	if (sigismember(&current, SIGTTOU))
		 printf( " SIGTTOU" );
	if (sigismember(&current, SIGVTALRM))
		 printf( " SIGVTALRM" );
	printf("returns %d\n", ret);
#endif

	return ret;
}

void
VCCPU::idleFunction()
{
	sigset_t current;
	sigprocmask(SIG_BLOCK, 0, &current);
	sigsuspend(&current);
}


void
VCCPU::debugBreakpoint()
{
	kill(getpid(), SIGTRAP);
}

VCCPU::VCCPU( ProcessContainer * idleContainer )
     : CPU( 0, idleContainer )
{
	_MMU = new VCMMU();
	_timesliceTimer = new TIMESLICETIMER();

	for (int i = 1; i < nInterrupts; i++) 
		interruptTable[i] = 0;

	installInterrupts();
}

void
VCCPU::installInterrupts()
{
	VCFPInterrupt 			* fpE = new VCFPInterrupt;
	VCMemoryInterrupt		* mE = new VCMemoryInterrupt;
	VCIllegalInstructionInterrupt	* vcIE = new VCIllegalInstructionInterrupt;
	//VCProxyInterrupt		* vcPE = new VCProxyInterrupt;
	VCIOInterrupt			* vcIOE = new VCIOInterrupt;
	TimerManagerInterrupt		* timerE = new TimerManagerInterrupt;

	setInterrupt(SIGFPE, fpE, 0);
	setInterrupt(SIGBUS, mE, 0);
	setInterrupt(SIGSEGV, mE, 0);
	setInterrupt(SIGILL, vcIE, 0);
	// setInterrupt(SIGUSR1, vcPE, 0);
	setInterrupt(SIGIO, vcIOE, 0);
	setInterrupt(SIGALRM, timerE, 0);
#ifdef INSTRUMENT
#ifdef notdef
// XXX - figure out what the right vector should be, if any!
        //setInterrupt (???, &TheInstrumentBufferInterrupt);
#endif
#endif
}

#ifdef TEST
int lastTrapCalled;
int trapLevel = 0;
int lastTrapReturned;
#endif
void
VCCPU::hardwareInterruptAssist(int vector, siginfo_t *si, void *uc)
{
     ucontext_t *utc = (ucontext_t*)uc;
	Debug << "VCCPU::hardwareInterruptAssist(" << vector 
	      << " at address " << asHex(si?(unsigned)si->si_addr:0) << ")\n" << eor;

	Assert( (vector >= 0) && (vector < nInterrupts) );
	Interrupt * interrupt = thisCPU()->interrupt(vector);

	Assert (interrupt != 0);
	VCSignalInfo info(vector, si, utc, si?(caddr_t)si->si_addr:0);
	while (interrupt) {
		interrupt->raise((char*)&info);
		interrupt = interrupt->next();
		}
	Debug << "VCCPU::hardwareInterruptAssist(" << vector 
	      << " at address " << asHex(si?(unsigned)si->si_addr:0) << ") returns\n" << eor;
}

Interrupt *
VCCPU::setInterrupt( int vector, Interrupt * interrupt, PFV assist  )
{
	Assert( vector >= 0 );
	Assert( vector < nInterrupts );
	Assert( interrupt != 0 );

	Interrupt * oldInterrupt = interruptTable[ vector ];
	if (oldInterrupt) {
		interruptTable[vector] = oldInterrupt->stack(interrupt);
		}
	else {
		interruptTable[ vector ] = interrupt;
	}
	/*
	 * Install the handler for that vector (signal)
	 */
	struct sigaction sa;
	 sa.sa_sigaction = &VCCPU::hardwareInterruptAssist;
	sa.sa_flags = SA_RESTART|SA_SIGINFO;
	sigset_t &st = sa.sa_mask;
	sigemptyset(&st);
	//sigaddset(&st, vector);
	// = (int) signal(vector, &VCCPU::hardwareInterruptAssist);
	int r = (int) sigaction(vector, &sa, NULL);
	Assert (r != -1);

	return( oldInterrupt );
}

void
Halt()
{
#if defined(linux)
	extern  const char *const strerror[];
#else
	extern  char *strerror[];
	extern int errno;
#endif
	Console() << "Halt(): errno: " << errno << " info: " << strerror[errno] << "\n" <<
			eor;
	abort(); // no dtors!
}


#include "ClassInstaller.h"
DEFINECLASS("VCCPU",VCCPU,CPU,0,0);
