#ifdef __GNUC__
#pragma implementation
#endif


/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 */
//#define DEBUG

#include "Assert.h"
#include "Domain.h"
#include "VCContext.h"
#include "Process.h"
#include "CPU.h"
#include "OutputStream.h"
#include "SPARCasmutil.h"
#include "PROCESSOR.h"
#include "Types.h"
#include "Debug.h"
#include "ThisProcess.h"


#ifdef __GNUC__
#pragma implementation
#endif


int getcontext(ucontext_t *ucp)
{
	Assert(NOTREACHED);
}

int setcontext(const ucontext_t *ucp)
{
	Assert(NOTREACHED);
}


void makecontext(ucontext_t *ucp, void (*func)(), int argc, ...) 
{
	Assert(NOTREACHED);
}


Process *
VCGenericContext::basicRestore( Process * 
					   processGivingUpProcessor )
{
  Debug << "VCGenericContext{"<<asHex(this)<<"}::basicRestore("<<asHex(processGivingUpProcessor)<<")\n";
  Assert(hasEverBeenDispatched != 0);
	  PROCESSOR::disableInterrupts();
	  _processResuming = processGivingUpProcessor;
	setcontext(&_ctx);

	Assert(NOTREACHED);
	/* return's from checkpoint() call */
	return processGivingUpProcessor;
}

static void
killMe()
{
    Assert( thisProcess() != 0 );
    thisProcess()->die();
    Assert(NOTREACHED);
}

struct trampstruct {
  OPFV func;
  int arg;
};

static void trampoline(trampstruct *ts)
{
  Object *o = (Object*)ts->arg;
  OPFV func = ts->func;
  delete ts;
  Assert(o != 0);
  (o->*func)();
}


void
VCGenericContext::restoreFromInitialContext( Process * 
							processGivingUpProcessor )
{
  Debug << "VCGenericContext{"<<asHex(this)<<"::restoreFromInitialContext("<<asHex(processGivingUpProcessor)<<")\n";
	PROCESSOR::disableInterrupts();

	ucontext_t kctx;
	getcontext(&kctx);
	//_ctx.uc_link = &kctx;
	//makecontext(&kctx, (PFV)killMe, 0);
	process()->setCurrentDomain( process()->domain() );
    restoreAssist( processGivingUpProcessor );	// may change domain
	
     if (_interruptable)
		PROCESSOR::enableInterrupts();
	getcontext(&_ctx);
	_ctx.uc_stack.ss_size = _stackSize;
	_ctx.uc_stack.ss_sp = stackBase;
	_ctx.uc_stack.ss_flags = 0;

     if (_opfvEntryPoint != 0) {
       trampstruct *ts = new trampstruct;
       ts->func = _opfvEntryPoint;
       ts->arg = _arg0;
       makecontext(&_ctx,  (PFV)trampoline, 1, ts);
     } else {
       makecontext(&_ctx,  (PFV)_entryPoint, 2, _arg0, _arg1);
     }
     setcontext(&_ctx);
     //swapcontext(&kctx, &_ctx);
     Assert (NOTREACHED);

    Assert( process() != 0 );
    process()->die();
    Assert(NOTREACHED);
}

Process *
VCGenericContext::checkpoint()
{
	  IntStorage intstate;
 PROCESSOR::disableInterrupts(intstate);
  Debug << "VCGenericContext{"<<asHex(this)<<"::checkpoint()\n";
  Assert (_processResuming == 0);
  Assert (hasEverBeenDispatched != 0);
  
	  volatile Process *giver = 0;
	getcontext(&_ctx);
	if (_processResuming != 0) {
	  Debug << "VCGenericContext{"<<asHex(this)<<"::checkpoint() : resumed\n";
	  giver = _processResuming;
	  _processResuming = 0;
	  setResuming((Process*)giver);
	} else {
	  Debug << "VCGenericContext{"<<asHex(this)<<"::checkpoint() : checkpointed\n";
	}
	Debug << "VCGenericContext{"<<asHex(this)<<"::checkpoint() : returns " <<asHex((void*)giver)<< "\n";
	PROCESSOR::restoreInterrupts(intstate);
	return((Process*)giver);
}

