#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

#include "Assert.h"
#include "VCContext.h"
#include "VCPageTableConstants.h"
#include "Process.h"
#include "ThisProcess.h"
#include "ClassStar.h"
#include "PROCESSOR.h"
#include "Kernel.h"
#include "RootMemoryObjectCache.h"
#include "MemoryObject.h"
#include "SourceOfZerosMemoryObject.h"


#ifdef __GNUC__
#pragma implementation
#endif


#ifdef INSTRUMENT_INTERRUPT
void InterruptConstructInstruments () {}
#endif

#ifdef INSTRUMENT_SVCTRAP
void SVCTrapConstructInstruments () {}
#endif


unsigned debugApplications = 0;

VCGenericContext::VCGenericContext( Process * owner,
							   PFV entry,
							   StackSize stackSize,
							   int arg0,
							   int arg1)
     : ProcessorContext( owner )
{
    initialize(stackSize);
	_entryPoint = entry;
    _opfvEntryPoint = 0;
	_arg0 = arg0;
	_arg1 = arg1;
}

VCGenericContext::VCGenericContext( Process * owner,
							   OPFV entry,
							   StackSize stackSize,
							   int arg0,
							   int arg1)
     : ProcessorContext( owner )
{
    initialize(stackSize);
_entryPoint = 0;
 _opfvEntryPoint = entry;
 _arg0 = arg0;
_arg1 = arg1;
}

void
VCGenericContext::initialize(StackSize stackSize)
{
    int npages;
    int heapStack = 0;
    _entryPoint = 0;
    _interruptable = 1;
    _processResuming = 0;

    switch( stackSize ) {
    case heapBootStack:
	heapStack = 1;		// Fallthrough
    case small:
	npages = 2;
	break;
    case normal:
	npages = 8;
	break;
    case huge:
	npages = 32;
	break;
    case gigantic:
	npages = 64;
	break;
    case large:
    case application:
    default:
	npages = 16;
	break;
    }
    npages *=2;
    _stackSize = npages *PhysicalPageSize;

	if (heapStack) {
		_stackMO = 0;
		stackBase = new char [_stackSize];
		}
	else	{
		_stackMO = new SourceOfZerosMemoryObject;
		Assert (_stackMO != 0);
		_stackMO->setName("StackMemoryObject");
		_stackMO->setNumberOfUnits(_stackSize);
                _stackMO->setCacheClass( RootMemoryObjectCacheClass );
                MemoryObjectCache * cache = _stackMO->buildCache();
		Assert( cache != 0 );
		stackBase = (char *)(TheKernel->domain()->add(_stackMO, ReadWrite));
		Assert( stackBase != 0 );
		Assert( TheKernel != 0 );
		Assert( TheKernel->domain() != 0 );
		TheKernel->domain()->makeAddressable(stackBase, _stackSize);
		}

	stackPointer = &stackBase[ _stackSize - 1 ];
	stackPointer =  (char*) ((unsigned)(stackPointer - 7) & ~7); 

}


VCGenericContext::~VCGenericContext()
{
	if (_stackMO) 	
		TheKernel->domain()->remove(_stackMO, stackBase);
}



VCApplicationContext::
VCApplicationContext( Process * owner, PFV entryPoint,
				 StackSize stacksize,
				 Domain *initialDomain,
				 int arg0, int arg1,
				 StackSize userStackSize, 
				 char * userStackPointer)
: VCGenericContext( owner, entryPoint, userStackSize, arg0, arg1 )
{
  //unsigned *inputs = (unsigned*)volatileRegs;
      //inputs[4] = (unsigned) initialDomain->defaultProxy();
}





VCSystemContext::VCSystemContext( Process * owner, 
							 PFV entryPoint,
							 StackSize stackSize, 
							 int arg0, int arg1 )
: VCGenericContext ( owner, entryPoint, stackSize, arg0, arg1 )
{
}

VCSystemContext::VCSystemContext( Process * owner, 
							 OPFV entryPoint,
							 StackSize stackSize, 
							 int arg0, int arg1 )
: VCGenericContext ( owner, entryPoint, stackSize, arg0, arg1 )
{
}



VCUninterruptableSystemContext::
VCUninterruptableSystemContext( Process * owner, 
					   PFV entryPoint,
					   StackSize stackSize, 
					   int arg0, int arg1 )
: VCSystemContext( owner , entryPoint, stackSize, arg0, arg1 )
{
	_interruptable = 0;
}

#include "ClassInstaller.h"
DEFINECLASS("VCGenericContext",VCGenericContext,ProcessorContext,0,0);
DEFINECLASS("VCApplicationContext",VCApplicationContext,VCGenericContext,0,0);
DEFINECLASS("VCSystemContext",VCSystemContext,VCGenericContext,0,0);
DEFINECLASS("VCUninterruptableSystemContext",VCUninterruptableSystemContext,VCSystemContext,0,0);
