#ifdef __GNUC__
#pragma implementation
#endif


/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * VCContextSwitching.cc: assembler context switching routines
 */

#include "Assert.h"
#include "Domain.h"
#include "VCContext.h"
#include "Process.h"
#include "CPU.h"
#include "OutputStream.h"
//#include "SPARCasmutil.h"
#include "PROCESSOR.h"
#include "Types.h"
#include "LLVAExecutionEngine.h"

#ifdef __GNUC__
#pragma implementation
#endif

//asm volatile ( "ldd %0,%%i0" : : "m" (volatileRegs[0]) );;



Process *
VCGenericContext::basicRestore( Process * 
					   processGivingUpProcessor )
{
  /* Flush windows */
  __jmp_buf_tag *local_jump_buf = getJmpBuf();
  longjmp(local_jump_buf, (unsigned int) processGivingUpProcessor);

  /* return's from checkpoint() call */
  return processGivingUpProcessor;
}

void
VCGenericContext::restoreFromInitialContext( Process * 
							processGivingUpProcessor )
{
  PROCESSOR::disableInterrupts();
  llva_push_all_regs(); // asm volatile( "pushal; nop" );

  /* Set up new stack and frame pointers */
  stackPointer -= (168 * 4);
  
  // LOAD_REG_FROM_MEM(%esp, stackPointer);
  llva_store_stackpointer(stackPointer);

  process()->setCurrentDomain( process()->domain() );
  restoreAssist( processGivingUpProcessor );	// may change domain

  llva_push_all_regs(); // asm volatile( "pushal; nop" );
	

  typedef int (* PFUNC) (unsigned, unsigned, unsigned);
  typedef void (* OFUNC) (OPFV, unsigned);

  PFUNC func = (PFUNC) ((unsigned*)volatileRegs)[1];
  OFUNC ofunc = (OFUNC) ((unsigned*)volatileRegs)[1];

  if (_interruptable)
    PROCESSOR::enableInterrupts();

  if (_opfvEntryPoint != 0) {
    (void) (*ofunc)(_opfvEntryPoint, ((unsigned*)volatileRegs)[3]);
  } else {
    (void) (*func)(((unsigned*)volatileRegs)[2], ((unsigned*)volatileRegs)[3], ((unsigned*)volatileRegs)[4]);
  }


  Assert( process() != 0 );

  process()->die();
  
  Assert(NOTREACHED);
}

Process *
VCGenericContext::checkpoint()
{
  return((Process *) 0);
}

