#ifdef __GNUC__
#pragma implementation
#endif


/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * VCContextSwitching.cc: assembler context switching routines
 */

#include "Assert.h"
#include "Domain.h"
#include "VCContext.h"
#include "Process.h"
#include "CPU.h"
#include "OutputStream.h"
#include "SPARCasmutil.h"
#include "PROCESSOR.h"
#include "Types.h"

#ifdef NEVER
extern "C" {
#define KERNEL
#include <sun4/asm_linkage.h>
#include <sun4/reg.h>
};
#endif


#ifdef __GNUC__
#pragma implementation
#endif


Process *
VCGenericContext::basicRestore( Process * 
					   processGivingUpProcessor )
{
	/* Flush windows */
	asm volatile( "ta 3; nop" );

	/*
	 * restore volatile regs
	 */
	LOAD_REG_FROM_MEM_DOUBLE(%i0, volatileRegs[0]);
	LOAD_REG_FROM_MEM_DOUBLE(%i2, volatileRegs[1]);
	LOAD_REG_FROM_MEM_DOUBLE(%i4, volatileRegs[2]);
	LOAD_REG_FROM_MEM_DOUBLE(%fp, volatileRegs[3]);
	LOAD_REG_FROM_MEM_DOUBLE(%l0, volatileRegs[4]);
	LOAD_REG_FROM_MEM_DOUBLE(%l2, volatileRegs[5]);
	LOAD_REG_FROM_MEM_DOUBLE(%l4, volatileRegs[6]);
	LOAD_REG_FROM_MEM_DOUBLE(%l6, volatileRegs[7]);
	/* 
	 * restore globals 
	 */
	LOAD_REG_FROM_MEM(%g1, ((unsigned *)globals)[0]);
	asm volatile ("mov %g1, %y");
	LOAD_REG_FROM_MEM(%g1, ((unsigned *)globals)[1]);
	LOAD_REG_FROM_MEM_DOUBLE(%g2, globals[1]);
	LOAD_REG_FROM_MEM_DOUBLE(%g4, globals[2]);
	LOAD_REG_FROM_MEM_DOUBLE(%g6, globals[3]);

	/*
	 * restore PC
	 */
	LOAD_REG_FROM_MEM(%o7, _entryPoint);

	/* return's from checkpoint() call */
	return processGivingUpProcessor;
}

void
VCGenericContext::restoreFromInitialContext( Process * 
							processGivingUpProcessor )
{
	PROCESSOR::disableInterrupts();
    asm volatile( "ta 3; nop" );         // Flush windows

    /* Set up new stack and frame pointers */
     stackPointer -= (168 * 4);
     LOAD_REG_FROM_MEM(%sp, stackPointer);

    process()->setCurrentDomain( process()->domain() );
    restoreAssist( processGivingUpProcessor );	// may change domain

    asm volatile( "ta 3; nop" );         // Flush windows

typedef int (* PFUNC) (unsigned, unsigned, unsigned);
typedef void (* OFUNC) (OPFV, unsigned);
     PFUNC func = (PFUNC) ((unsigned*)volatileRegs)[1];
    OFUNC ofunc = (OFUNC) ((unsigned*)volatileRegs)[1];

     if (_interruptable)
	PROCESSOR::enableInterrupts();

     if (_opfvEntryPoint != 0) {
       (void) (*ofunc)(_opfvEntryPoint, ((unsigned*)volatileRegs)[3]);
     } else {
	(void) (*func)(((unsigned*)volatileRegs)[2], ((unsigned*)volatileRegs)[3], ((unsigned*)volatileRegs)[4]);
     }


    Assert( process() != 0 );

    process()->die();
    
    Assert(NOTREACHED);
}

Process *
VCGenericContext::checkpoint()
{
	/*
	 * save PC
	 */
	STORE_REG_TO_MEM (%o7, _entryPoint);

	/* 
	 * save globals 
	 */
	/* %g0 is hard wired to zero so we use it for %y  */
	STORE_REG_TO_MEM(%g1, ((unsigned *)globals)[1]);
	asm volatile (" mov	%y, %g1");
	STORE_REG_TO_MEM(%g1, ((unsigned *)globals)[0]);
	STORE_REG_TO_MEM_DOUBLE(%g2, globals[1]);
	STORE_REG_TO_MEM_DOUBLE(%g4, globals[2]);
	STORE_REG_TO_MEM_DOUBLE(%g6, globals[3]);

	/* 
	 * save inputs and locals
	 */
	STORE_REG_TO_MEM_DOUBLE(%i0, volatileRegs[0]);
	STORE_REG_TO_MEM_DOUBLE(%i2, volatileRegs[1]);
	STORE_REG_TO_MEM_DOUBLE(%i4, volatileRegs[2]);
	STORE_REG_TO_MEM_DOUBLE(%fp, volatileRegs[3]);
	STORE_REG_TO_MEM_DOUBLE(%l0, volatileRegs[4]);
	STORE_REG_TO_MEM_DOUBLE(%l2, volatileRegs[5]);
	STORE_REG_TO_MEM_DOUBLE(%l4, volatileRegs[6]);
	STORE_REG_TO_MEM_DOUBLE(%l6, volatileRegs[7]);

#ifdef NEVER
	/*
	 * Save stack pointer
	 */
	STORE_REG_TO_MEM(%sp, stackPointer);
	Assert ( ! ((unsigned)stackPointer & 0x7) );
#endif

	return((Process *) 0);
}

