#ifdef GAS
#define	smovb	movsb
#define smovw	movsw
#define smovl	movsl
#define sstob	stosb
#define sstow	stosw
#define sstol	stosl

#endif

/*************************************************************************
/ This file is part of the Choices Operating System
/ Developed by: The TAPESTRY Parallel Computing Laboratory
/		 University of Illinois at Urbana-Champaign
/		 Department of Computer Science
/		 1304 W. Springfield Ave.
/		 Urbana, IL	61801
/
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
/	All Rights Reserved.
/ CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
/
/ Author: Ruth A. Aydt (aydt@cs.uiuc.edu)
/ Contributing Author: Dave W. Dykstra (dykstra@cs.uiuc.edu)
/ Contributing Author: See-Mong Tan (stan@cs.uiuc.edu)
/ Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
/
/Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
/  No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
/
/ Util.s:   Assembly level utility routines for the 80586
/ *****************************************************************************/

	.file	"Util.s"
	.att_syntax
	
	.align	4

/***********************************************************************;
/	Call:		char * getStackPointer();
/	Description:	return the value of the stack pointer before the call
/	Return:		long (in eax)
/ **********************************************************************/

		.globl getStackPointer
getStackPointer:
		movl	%esp, %eax
		subl	$4, %eax    // the return address pushed in the stack
		ret


/***********************************************************************;
/	Call:		In(port);
/	Description:	Input a Byte from an I/O port
/	Arguments:     	port - 	I/O port address.
/	Return:		Byte Input (in al)
/ **********************************************************************/

		.globl	In
In:
		movw	4(%esp), %dx	// move port into dx
		.byte	0xEC		// input byte from port dx into al
		ret

/***********************************************************************;
/	Call:		Inw(port);
/	Description:	Input a word from an I/O port
/	Arguments:     	port - 	I/O port address.
/	Return:		word Input (in ax)
/ **********************************************************************/

		.globl	Inw
Inw:
		movw	4(%esp), %dx	// move port into dx
		.byte	0x66		// data16
		.byte	0xED		// input word from port dx into ax
		ret


/***********************************************************************;
/	Call:		Out(port, value);
/	Description:	Output to an I/O port.
/	Arguments:	port -	The I/O port address.
/	      		value -	The value to output.
/	Return:		none
/ **********************************************************************/

		.globl	Out
Out:
		movw	4(%esp), %dx	// move port into dx 	
		movb	8(%esp), %al	// move value into al
		.byte	0xEE		// output byte in al to port dx
		ret

/***********************************************************************;
/	Call:		Outw(port, value);
/	Description:	Output to an I/O port.
/	Arguments:	port -	The I/O port address.
/	      		value -	The value to output.
/	Return:		none
/ **********************************************************************/

		.globl	Outw
Outw:
		movw	4(%esp), %dx	// move port into dx 	
		movw	8(%esp), %ax	// move value into ax
		.byte	0x66		// data16
		.byte	0xEF		// output word in ax to port dx
		ret

/***********************************************************************;
/	Call: 	      _Lmsw(msw);
/	Description:  Load CR0.
/	Arguments:    msw - new Machine Status Word to load
/	Returns:      none
/ **********************************************************************/


		.globl	_Lmsw
_Lmsw:
		mov	4(%esp), %eax
		mov	%eax, %cr0
		jmp	.finishlmsw 	// do a jump to ensure setting PG bit
.finishlmsw:
		nop			// Give time for this to take effect.
		nop			//  Im not sure if this is the right
		nop			//  number of nops, but setting a
		nop			//  breakpoint here always makes it
		nop			//  work.
		ret

/***********************************************************************;
/	Call: 	      Smsw();
/	Description:  Read Contents of CR0.
/	Arguments:    none
/	Returns:      Value of CR0
/ **********************************************************************/

		.globl	Smsw
Smsw:
		mov	%cr0, %eax
		ret

/***********************************************************************;
/	Call: 	      GetCR2();
/	Description:  Read Contents of CR2.
/	Arguments:    none
/	Returns:      Value of CR2
/ **********************************************************************/

		.globl	GetCR2
GetCR2:
		mov	%cr2, %eax
		ret

/***********************************************************************;
/  	Call:		_Lcr3( padrpd ) 
/	Description:	Load CR3 (Page Directory Base Register).
/	Arguments:	padrpd - physical address of page directory (level1
/			page table)
/	Returns:	none
/ **********************************************************************/

                .globl  _Lcr3
_Lcr3:
                push    %ebp
                mov     %esp, %ebp
                mov     8(%ebp), %eax
          	mov     %eax, %cr3
                pop     %ebp
	        ret

/***********************************************************************;
/  	Call:		FlushTLB()
/	Description:	Move current value of cr3 into cr3 to flush TLB
/	Arguments:	none
/	Returns:	none
/	Comments:	I'm not convinced that this really works - I may
/			need to do something special to deal with the 
/			80586/80387 problem. Also, can I use eax like
/			this?-Ruth
/ **********************************************************************/

                .globl  FlushTLB
FlushTLB:
          	mov     %cr3, %eax
		mov	%eax, %cr3
	        ret

/***********************************************************************
/	Call: 	      Int3();
/	Description:  cause interrupt 3
/	Arguments:    none
/	Returns:      none
/ **********************************************************************/
		.globl	Int3
Int3:
		.byte 0xCC		// int 3
		ret

/***********************************************************************
/	Call:	      JmpStart( whereto, arg);
/	Description:  jump to "whereto" routine with "arg" on stack
/	Arguments:    whereto - address of routine to jump to
/		      arg - argument (gets left on stack for "whereto" routine)
/	Returns:      none (we never get back even to this code after jmp)
/ **********************************************************************/
		.globl JmpStart
JmpStart:
		popl	%eax		// pop off return address - we never
					// go back & want to present correct
					// args on stack to routine we are 
					// jumping to
		jmp	*(%esp)

/***********************************************************************
/	Call:		Str()
/	Description:	Store Task Register command
/	Arguments:	none
/	Returns:	Returns the contents of the task register (short)
/ **********************************************************************/
		.globl Str
Str:
		str	%ax	
		ret

/***********************************************************************
/	Call:		BreakIt()
/	Description:	Access ridiculous memory location to cause page fault
/	Arguments:	none
/	Returns:	none
/ *********************************************************************/
		.globl HINUM
HINUM:		.value 0xFFFFFFFF

		.globl BreakIt
BreakIt:
		push	%ebp
		mov	%esp, %ebp
		movl HINUM, %eax
		movl (%eax), %eax
		pop %ebp
		ret

/***********************************************************************
/	Call: 	      PhysCopyBlock(from,to,nbytes)
/	Description:  copy 'nbytes' of memory from 'from' to 'to' in physical
/			memory (MMU off)
/	Arguments:    from - from address
/		      to - to address
/		      nbytes - number of bytes to copy
/	Returns:      nbytes or -1 if no Copy done
/ **********************************************************************/
	.globl	PhysCopyBlock
PhysCopyBlock:
	pushl	%edi
	pushl	%esi
	movl	12(%esp), %esi             // from address
	movl	16(%esp), %edi             // to address
	movl	20(%esp), %ecx            // # of bytes
	cmpl    $0x00, %ecx               // check for negative counts
        jge     pmoveok
	movl    $0xffffffff, %eax
	popl	%esi
	popl	%edi
	ret
pmoveok: push	%es                       // save registers
	push	%ds

	push	%ebx

	call	DisableInterrupts
	push	%eax			// save interrupt state on stack

	mov	%cr0, %edx		// save %cr0 in edx
	mov	%cr3, %ebx		// save %cr3 in ebx

	// NOTE: dont use the stack with the MMU turned off because
	//   the stack may not be the same in physical memory
	mov	%cr0, %eax
	and	$0x7fffffff, %eax	// turn off MMU
	mov	%eax, %cr0
	mov	$0, %eax		// clear page-table-base-register
	mov	%eax, %cr3


	movw	%ds, %ax                // make sure move is in data space
	movw	%ax, %es

	orl	%ecx, %ecx        // check for zero count %ecx is set in calling function
	jz	pmovedone
        cmpl    %esi, %edi        // if source and destination are same just quit
	jz      pmovedone
	cld
        cmpl    $15, %ecx         // if less than equal to 15 bytes just move 
	jle     pbytemove
	movl	%edi, %eax        // if source and destination are within 3
	subl    %esi, %eax        // bytes it is faster to do a byte move
	jns     plabel1
	neg     %eax

plabel1: cmpl    $3, %eax          // if less than equal to 3 bytes just move 
	jg      pmoveblks          // one by one and quit

pbytemove:                         // sequential byte move
	rep
	smovb
	jmp	pmovedone

pmoveblks:
	movl    %esi, %eax       // mask the last 2 bits of source address
	andl    $3, %eax         // source is at double word boundary
        jz      pblkmove
	xchg    %eax, %ecx       // move the bytes until we reach word boundary
	subl    %ecx, %eax
	rep     
	smovb
	xchg    %eax, %ecx       // restore value of new ecx

pblkmove:
	movl    %ecx, %eax      
	andl    $3, %eax
	shrl    $2, %ecx
	rep 
	smovl   
	movl    %eax, %ecx
	cmpl    $0, %ecx
	jz      pmovedone
	rep 
	smovb

pmovedone:
	mov	%ebx, %cr3		// restore page-table-base register
	mov	%edx, %cr0 		// restore MMU

	pop	%eax
	test	%eax, %eax		// restore interrupt state
	jz	pfinish
	call	EnableInterrupts

pfinish:
	pop	%ebx

	pop	%ds                     // restore system data segment
	pop	%es
	popl	%esi
	popl	%edi
	movl	8(%esp), %eax		// return third arg
	ret

/***********************************************************************
/	Call: 	      CopyBlock(from,to,nbytes)
/	Description:  copy 'nbytes' of memory from 'from' to 'to'
/	Arguments:    from - from address
/		      to - to address
/		      nbytes - number of bytes to copy
/	Returns:      nbytes or -1 if no Copy done
/ **********************************************************************/
	.globl	CopyBlock
CopyBlock:
	pushl	%edi
	pushl	%esi
	movl	12(%esp), %esi             // from address
	movl	16(%esp), %edi             // to address
	movl	20(%esp), %ecx            // # of bytes
	cmpl    $0x00, %ecx               // check for negative counts
        jge     moveok
	movl    $0xffffffff, %eax
	popl	%esi
	popl	%edi
	ret
moveok: push	%es                       // save %es register
	push	%ds
	movw	%ds, %ax                // make sure move is in data space
	movw	%ax, %es

__domove:
	orl	%ecx, %ecx        // check for zero count %ecx is set in calling function
	jz	movedone
        cmpl    %esi, %edi        // if source and destination are same just quit
	jz      movedone
	cld
        cmpl    $15, %ecx         // if less than equal to 15 bytes just move 
	jle     bytemove
	movl	%edi, %eax        // if source and destination are within 3
	subl    %esi, %eax        // bytes it is faster to do a byte move
	jns     label1
	neg     %eax

label1: cmpl    $3, %eax          // if less than equal to 3 bytes just move 
	jg      moveblks          // one by one and quit

bytemove:                         // sequential byte move
	rep
	smovb
	jmp	movedone

moveblks:
	movl    %esi, %eax       // mask the last 2 bits of source address
	andl    $3, %eax         // source is at double word boundary
        jz      blkmove
	xchg    %eax, %ecx       // move the bytes until we reach word boundary
	subl    %ecx, %eax
	rep     
	smovb
	xchg    %eax, %ecx       // restore value of new ecx

blkmove:
	movl    %ecx, %eax      
	andl    $3, %eax
	shrl    $2, %ecx
	rep 
	smovl   
	movl    %eax, %ecx
	cmpl    $0, %ecx
	jz      movedone
	rep 
	smovb

movedone:
	pop	%ds                     // restore system data segment
	pop	%es
	popl	%esi
	popl	%edi
	movl	8(%esp), %eax		// return third arg
	ret

/************************************************************************
/	Call: 	      CopyBlockBackwards(from,to,nbytes)
/	Description:  same as CopyBlock but start from the end and
/			copy to the beginning.  Used to shift memory
/			to the right in insert-type operations.
/	Arguments:    from - from address
/		      to - to address
/		      nbytes - number of bytes to copy
/	Returns:      nbytes or -1 if no Copy done
/ **********************************************************************/
	.globl	CopyBlockBackwards
CopyBlockBackwards:
	pushl	%edi
	pushl	%esi
	movl    12(%esp), %esi    // from address
	movl	16(%esp), %edi    // to address
	movl	20(%esp), %ecx   // # of bytes
	push    %es
	movw    %ds, %ax
	movw    %ax, %es
	cmpl    $0x00, %ecx      // check for negative counts
        jge     dmoveok
	movl    $0xffffffff, %eax
	popl    %es
	popl    %esi
	popl    %edi
	ret

dmoveok:orl     %ecx, %ecx       // check for zero counts
	jz      dmovedone
        cmpl    %edi, %esi       // if source and destination are same just quit
	jz      dmovedone
	addl    %ecx, %esi       // add count to set source and destination for
	decl    %esi             // backward move
	addl    %ecx, %edi
	decl    %edi
	std
	
        cmpl    $15, %ecx       // if count is <= 15 just do a repeated byte move
	jle     dbytemove
	movl	%edi, %eax      // if source and destination are within 3
	subl    %esi, %eax      // bytes it is faster to do a byte move
	jns     dlabel1
	neg     %eax

dlabel1:cmpl    $3, %eax       // if less than equal to 3 bytes just move 
	jg      dmoveblks      // one bye one and quit

dbytemove:
	rep
	smovb
	jmp     dmovedone

dmoveblks:
	movl    %esi, %eax
	andl    $3, %eax
	jz      ddwrdbry
	movl    %esi, %eax
	andl    $1, %eax
	jz      dwrdbry
	movl    %esi, %eax
	andl    $2, %eax
	jnz     dblkmove
        decl    %esi
	decl    %edi
	smovw
	incl    %esi
	incl    %edi
	subl    $2, %ecx
	jmp     dblkmove

ddwrdbry:
	smovb                   // near a double word boundary so move a byte
	decl    %ecx            // to get to the double word and then move blks
	jmp     dblkmove

dwrdbry:smovb
	decl    %esi
	decl    %edi
	smovw
	subl    $3, %ecx
        incl    %esi
	incl    %edi
dblkmove:
	movl    %ecx, %eax
	andl    $3, %eax
	subl    $3, %esi
	subl    $3, %edi
	shrl    $2, %ecx
	rep
	smovl
	movl    %eax, %ecx
	addl    $3, %edi
	addl    $3, %esi
	cmpl    $0, %ecx
	je      dmovedone
	cmpl    $1, %ecx
	jg      dmvtwomr
	smovb
	jmp     dmovedone
dmvtwomr:
	rep
	smovb

dmovedone:
	pop	%es
	popl	%esi
	popl	%edi
	ret

/***********************************************************************
/	Call: 	      PhysZeroBlock(addr,nbytes)
/	Description:  zero 'nbytes' of physical memory (MMU off)
/	Arguments:    addr - address to start zeroing
/		      nbytes - number of bytes to zero
/	Returns:      nbytes or -1 if no zeroing done
/ **********************************************************************/
	.globl	PhysZeroBlock
PhysZeroBlock:
	pushl	%edi
	push	%es
	movw %ds, %ax
	movw %ax, %es
	movl	12(%esp),%edi	// %edi = dest address
	movl	16(%esp),%ecx	// %ecx = length of string
	orl	%ecx, %ecx              // nbytes = 0
	jz	pzdone
	cmpl    $0x00, %ecx               // check for negative counts
        jge     pzerook
	movl    $0xffffffff, %eax
	popl	%es
	popl	%edi
	ret
pzerook:
	push	%ebx
	push	%esi
	call	DisableInterrupts
	push	%eax			// save interrupt state on stack

	mov	%cr0, %edx		// save %cr0 in edx
	mov	%cr3, %ebx		// save %cr3 in ebx

	// NOTE: dont use the stack with the MMU turned off because
	//   the stack may not be the same in physical memory
	mov	%cr0, %eax
	and	$0x7fffffff, %eax	// turn off MMU
	mov	%eax, %cr0
	mov	$0, %eax		// clear page-table-base-register
	mov	%eax, %cr3

	xor	%eax, %eax
	movl	%ecx,%esi	// %esi = number of bytes to move
	shrl	$2,%ecx		// %ecx = number of words to move
	cld
	rep 
	sstol		// move the words

	movl	%esi,%ecx	// %ecx = number of bytes to move
	andl	$0x3,%ecx	// %ecx = number of bytes left to move
	rep 
	sstob		// move the bytes
pzdone:
	mov	%ebx, %cr3		// restore page-table-base register
	mov	%edx, %cr0 		// restore MMU

	pop	%eax
	test	%eax, %eax		// restore interrupt state
	jz	pzfinish
	call	EnableInterrupts

pzfinish:
	pop	%esi
	pop	%ebx

	pop	%es
	pop	%edi
	ret

/***********************************************************************
/	Call: 	      ZeroBlock(addr,nbytes)
/	Description:  zero 'nbytes' of memory
/	Arguments:    addr - address to start zeroing
/		      nbytes - number of bytes to zero
/	Returns:      nbytes or -1 if no zeroing done
/ **********************************************************************/
	.globl	ZeroBlock
ZeroBlock:
	pushl	%edi
	push	%es
	movw %ds, %ax
	movw %ax, %es
	movl	12(%esp),%edi	// %edi = dest address
	movl	16(%esp),%ecx	// %ecx = length of string
	orl	%ecx, %ecx              // nbytes = 0
	jz	zdone
	cmpl    $0x00, %ecx               // check for negative counts
        jge     zerook
	movl    $0xffffffff, %eax
	popl	%es
	popl	%edi
	ret
zerook:
	xor	%eax, %eax
	movl	%ecx,%edx	// %edx = number of bytes to move
	shrl	$2,%ecx		// %ecx = number of words to move
	cld
	rep 
	sstol	// move the words

	movl	%edx,%ecx	// %ecx = number of bytes to move
	andl	$0x3,%ecx	// %ecx = number of bytes left to move
	rep 
	sstob		// move the bytes
zdone:
	pop	%es
	popl	%edi
	ret

/***********************************************************************
/	Call: 	      InterruptsEnabled()
/	Description:  return whether or not Interrupts are Enabled
/	Arguments:    none
/	Returns:      non-zero if interrupts enabled
/ **********************************************************************/
		
		.globl	InterruptsEnabled
		.globl	InterruptsEnabled__Fv
InterruptsEnabled:
InterruptsEnabled__Fv:
		pushf
		popl	%eax
		andl	$0x200, %eax
		ret

/***********************************************************************
/	Call: 	      DisableInterrupts()
/	Description:  disable interrupts and return whether or not
/			Interrupts were Enabled
/	Arguments:    none
/	Returns:      non-zero if interrupts enabled
/ **********************************************************************/
		
		.globl	DisableInterrupts
		.globl	DisableInterrupts__Fv
DisableInterrupts:
DisableInterrupts__Fv:
		pushf
		popl	%eax
		andl	$0x200, %eax
		cli
		ret

/***********************************************************************
/	Call: 	      EnableInterrupts()
/	Description:  enable interrupts
/	Arguments:    none
/	Returns:      none
/ **********************************************************************/
		
		.globl	EnableInterrupts
		.globl	EnableInterrupts__Fv
EnableInterrupts:
EnableInterrupts__Fv:
		sti
		ret

//   The following routines are necessary for the PC-AT device driver
//   for transferring data to and from the data sector buffer of the
//   drive controller... - STAN

/***********************************************************************
/	Call: 	      PhysCopyInWordBlock(port, addr, nwords)
/	Description:  Copy nwords of data from port buffer to physical addr
/	Arguments:    port - I/O port, address of buffer and # of bytes
/       Precondition: Interrupts MUST be off first!
/	Returns:      none
/ **********************************************************************/

//Need these offsets
		.set PORT, 8
		.set ADDR, 12
		.set COUNT, 16

		.align 4

		.globl PhysCopyInWordBlock

PhysCopyInWordBlock:
	pushl	%ebp                      // save register %ebp on stack
	movl 	%esp, %ebp                // put stack ptr with args in ebp
	pushl	%edx                      // save edx,
	pushl	%edi                      // save edi
	pushl	%ecx                      // save ecx
	movl	PORT(%ebp), %edx          // move args into edx, edi and ecx
	movl	ADDR(%ebp), %edi
	movl 	COUNT(%ebp), %ecx

	// move DS into ES
	pushl	%es
	pushl	%ds
	popl	%es
	
	// disable interrupts and MMU now

	push	%ebx
	push	%esi
	call 	DisableInterrupts
	push	%eax                       // save interrupt state on stack
	mov	%cr0, %esi                 // save %cr0 on stack
	mov	%cr3, %ebx                 // save %cr3 in ebx
	
	// do not use stack with MMU turned off - no function calls please
	mov	%cr0, %eax
	and	$0x7fffffff, %eax          // disable MMU
	mov	%eax, %cr0
	mov	$0, %eax                   // clear page table base register
	mov	%eax, %cr3

	// Can start doing the transfer here now
	rep
	.byte	0x66			  // data16
	insl

	// enable MMU and interrupts
	mov	%ebx, %cr3                // restore PTBR	   
	mov	%esi, %cr0                // restore MMU
	pop     %eax
	test	%eax, %eax                // restore interrupts
	jz 	PhysCopyInWordBlockDone   // only if need to
	call	EnableInterrupts

PhysCopyInWordBlockDone:
	pop	%ebx                      // cannot pop to immediate so use
	mov	%ebx, %esi                // %ebx as temp
	pop	%ebx

	// pop the E segment register
	popl	%es

	popl	%ecx 
	popl	%edi
	popl	%edx
	popl	%ebp
	ret

/***********************************************************************
/	Call: 	      PhysCopyOutWordBlock(port, addr, nwords)
/	Description:  Copy nwords of data from phys addr to port buffer
/	Arguments:    port - I/O port, address of buffer and # of bytes
/       Precondition: Interrupts MUST be off first!
/	Returns:      none
/ **********************************************************************/

		.align 4
		.globl PhysCopyOutWordBlock
PhysCopyOutWordBlock:
	pushl	%ebp
	movl 	%esp, %ebp
	pushl	%edx
	pushl	%esi
	pushl	%ecx
	movl	PORT(%ebp), %edx
	movl	ADDR(%ebp), %esi   // must use edi, not esi!!!
	movl 	COUNT(%ebp), %ecx

	// move DS into ES
	push	%es
	push	%ds
	pop	%es
	
	// disable interrupts and MMU now

	push	%ebx
	push	%edi
	call 	DisableInterrupts
	push	%eax                       // save interrupt state on stack
	mov	%cr0, %edi                 // save %cr0 on stack
	mov	%cr3, %ebx                 // save %cr3 in ebx
	
	// do not use stack with MMU turned off - no function calls please
	mov	%cr0, %eax
	and	$0x7fffffff, %eax          // disable MMU
	mov	%eax, %cr0
	mov	$0, %eax                   // clear page table base register
	mov	%eax, %cr3

	rep
	.byte	0x66			  // data16
	outsl

	// enable MMU and interrupts
	mov	%ebx, %cr3                // restore PTBR	   
	mov	%edi, %cr0                // restore MMU
	pop     %eax
	test	%eax, %eax
	jz	PhysCopyOutWordBlockFinish
	call	EnableInterrupts

PhysCopyOutWordBlockFinish:
	pop	%ebx
	mov	%ebx, %edi
	pop	%ebx

	// restore E segment
	pop	%es

	popl	%ecx
	popl	%esi
	popl	%edx
	popl	%ebp
	ret

		.align 4
		.globl CopyInWordBlock

CopyInWordBlock:
	pushl	%ebp                      // save register %ebp on stack
	movl 	%esp, %ebp                // put stack ptr with args in ebp
	pushl	%edx                      // save edx,
	pushl	%edi                      // save edi
	pushl	%ecx                      // save ecx
	movl	PORT(%ebp), %edx          // move args into edx, edi and ecx
	movl	ADDR(%ebp), %edi
	movl 	COUNT(%ebp), %ecx
	
	// move DS into ES
	push	%es
	push	%ds
	pop	%es
	
	// Can start doing the transfer here now
	rep
	.byte	0x66			  // data16
	insl
	
	// restore es
	pop	%es

	popl	%ecx 
	popl	%edi
	popl	%edx
	popl	%ebp
	ret
