/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Contributing Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Contributing Author: See-Mong Tan (stan@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * i586CPU.c: i586 CPU class implementation.
 */

#include "Assert.h"
#include "OutputStream.h"
#include "Domain.h"
#include "Vectors.h"
#include "Timer.h"
#include "Process.h"
#include "i586CPU.h"
#include "i586DataTypes.h"
#include "Util.h"
#include "i586Context.h"
#include "i586Interrupts.h"
#include "ThisProcess.h"
#include "ThisCPU.h"

bool i586CPU::IDTSetup = false;

GateDescriptor IDT[nInterrupts];
InterruptDescriptorTableRegister IDTR = {
	0,
	sizeof(IDT) - 1,
	(unsigned int) &IDT[0],
};
extern "C" void CallStrayInterrupt();
extern "C" void CallStrayInterruptWithErrorCode();
PFV InterruptHooks[nInterrupts] =
{
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterruptWithErrorCode, /* 8 */
	CallStrayInterrupt,
	CallStrayInterruptWithErrorCode, /* 10 */
	CallStrayInterruptWithErrorCode, /* 11 */
	CallStrayInterruptWithErrorCode, /* 12 */
	CallStrayInterruptWithErrorCode, /* 13 */
	CallStrayInterruptWithErrorCode, /* 14 */
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,	/* 60 */
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
	CallStrayInterrupt,
};

/*
 * StrayInterrupt() prints a message showing the interrupt number and 
 * returns.  This is the default interrupt handler for the i586 when no
 * error code is on the stack.
 */
extern void Halt();
void
StrayInterrupt( int inumber )
{
  Console().cpuPrintf("In StrayInterrupt");
  Console().cpuPrintf("ID = %d\n", inumber);
	Console().cpuPrintf( "Unexpected stray interrupt # %d\n", inumber);
	Halt();
}

/*
 * StrayInterruptWithErrorCode() prints a message showing the interrupt 
 * number and value of the error code and returns. This is the default 
 * interrupt handler for the i586 when an error code is on the stack.
 */
void
StrayInterruptWithErrorCode( int inumber, unsigned int errcode )
{
  Console().cpuPrintf("In StrayInterruptWithErrorCode");
  Console().cpuPrintf("ID = %d\n", inumber);
  Console().cpuPrintf("ErrorCode = %x\n", errcode);
	Console().cpuPrintf( "Unexpected stray interrupt #%d,  error code %x\n",
		inumber, errcode );
	Halt();
}

/* 
 * Default interrupt handler until the kernel installs what it wants.
 * There SHOULD be no interrupts until the kernel is running and decides its 
 * ready.
 */
#include "i586Interrupts.h"
StrayInterrupt uncaught;

static int i586CPUClass_Initialized = 0;

Interrupt * i586CPU::interruptTable[ nInterrupts ];

i586CPU::i586CPU( unsigned long identifier, ProcessContainer * iic )
	: CPU( identifier, iic )
{
	Debug << "i586CPU::i586CPU()\n" << eor;

	/*
	 * We know this constructor is called sequentially (not in parallel)
	 * by all the processors, therefore this doesnt need locked.
	 */
	if( ! i586CPUClass_Initialized ) {
		Debug.printf( "Initializing CPU class\n" );
		for( int i = 0; i < nInterrupts; i++ ) {
			interruptTable[i] = &uncaught;
		}

		i586CPUClass_Initialized = 1;
	}

	/*
	 * Set the MMU pointer and other.
	 */
	_MMU = &myMMU;

	IDTSetup = false; //MDB addition; 4/26/04
}

void
i586CPU::chipInitialize()
{
	Debug << "i586CPU::chipInitialize()\n" << eor;

	/* 
	 * The chip is initialized very early on, don't need to do it here
	 */
}

/*
 * Preempt this CPU.  We want the running process to be put back on the
 * scheduler ready queue so we can get a new one.
 * This function is machine and not processor dependent and needs
 * rethought altogether.
 */
void
i586CPU::preempt()
{
	Console() << "i586CPU::prempt not implemented\n" << eor;
}

/*
 * This is called directly from the assembler interruptAssist.
 * It is declare as 'static' in the class definition so it does not get
 *       a 'this' pointer.
 */

void
i586CPU::hardwareInterruptAssist( stackFrame *info )
{
	
  Assert( info->vector >= 0 );
  Assert( info->vector < nInterrupts );

  	Interrupt * interrupt = ((i586CPU *) thisCPU())->interruptTable[ info->vector ];

		Debug << "hardwareInterruptAssist( " << info->vector << "), interrupt( "
	<< asAddress( interrupt ) << " )\n" << eor;

	Assert( interrupt != 0 );

	if ( ( info->cs & 0x3 ) != LvlZero ) {
		/*
		 * Called from user mode, need to save the user stack pointer
		 *   in case the interrupt does an outcall.
		 */
		i586GenericContext * ctxt =
			(i586GenericContext *) thisProcess()->context();
		// char * oldusp = ctxt->saveUserStack( (char *) info->usp );

		interrupt->raise( (char *) info );

		// (void) ctxt->saveUserStack( oldusp );
	}
	else
		interrupt->raise( (char *) info );

		Debug << "hardwareInterruptAssist( " << info->vector << "), interrupt( "
		<< asAddress( interrupt ) << " ) returned \n" << eor;
}

extern "C" {
  stackFrame *getStackPointer();
}

/*
 * Install an interrupt object to manage a vectored interrupt (trap or
 * interrupt) from the hardware.
 */
Interrupt *
i586CPU::setInterrupt( int vector, Interrupt * interrupt, PFV assist )
{
	Debug.printf( "i586CPU::setInterrupt( vec:%d interrupt:%N )\n",
		vector, interrupt );
	Assert( vector >= 0 );
	Assert( vector < nInterrupts );
	Assert( interrupt != 0 );

	/*
	 * Save the pointer to the interrupt object itself.
	 */
	Interrupt * oldInterrupt = interruptTable[ vector ];
	interruptTable[ vector ] = interrupt;

	/*
	 * Set the interrupt vector table to point to interrupt asssist.
	 */
	if( assist )
		InterruptHooks[ vector ] = assist;
	else
		InterruptHooks[ vector ] = (PFV) &i586CPU::interruptAssist;

	return( oldInterrupt );
}

Interrupt * 
i586CPU::interrupt( int vector ) 
{
  Assert( NOTREACHED );
  return 0;
}

void
i586CPU::initMutexState( MutexStateAddr addr)
{
	Assert ( addr != 0);	
	*addr = 0;
}

void
i586CPU::spinMutex( MutexStateAddr addr)
{
	Assert ( addr != 0);
	Assert ( *addr == 0);
	*addr = 1;
}

void 
i586CPU::sendFlushRequestTo( unsigned long )
{
	Assert( NOTREACHED );
}



/*
 * Halt the CPU. This code probably doesn't belong here but I can't think of
 * any better place to put it.
 */
void
Halt()
{
	/*
	 * Halt CPU via infinite loop.
	 */
	Console().ultimatePrintf( "Halting.\n" );
	for ( ;; );
}

/*
 * i586CPU Class destructor.
 * Obvious side effect is to halt the processor. I doubt this could ever
 * actually be called!
 */
i586CPU::~i586CPU()
{
	Debug.printf( "CPU::~CPU()\n" );
	Halt();
}

/*
 * setGdtGate sets the values for the specified gate in the GDT
 */
void
i586CPU::setGdtGate( int gateNumber, GateType type, unsigned short sel,
		    PFV addr, GatePrivilegeLevel priv, unsigned short count)
{
	extern unsigned int gdt;

	GateDescriptor *gdtent = &(((GateDescriptor *) &gdt)[gateNumber]);

	gdtent->low_offset = ((unsigned int) addr) & 0xffff;
	gdtent->high_offset = (((unsigned int) addr) >> 16) & 0xffff;
	gdtent->selector = sel;
	gdtent->count = count;
	gdtent->zeros = 0;
	gdtent->gate_type = type;
	gdtent->dpl = priv;
	gdtent->present = 1;
}

/*
 * setGdtSegment sets the values for the specified segment in the GDT
 */
void
i586CPU::setGdtSegment( int gateNumber, CodeDataType type,
	GatePrivilegeLevel priv, const char * base, unsigned int size )
{
	extern unsigned int gdt;

	CodeDataSegmentDescriptor *gdtent =
			&(((CodeDataSegmentDescriptor *) &gdt)[gateNumber]);

	size -= 1; /* to make sure limit doesn't go too far */

	gdtent->low_base = ((unsigned int) base) & 0xffff;
	gdtent->mid_base = (((unsigned int) base) >> 16) & 0xff;
	gdtent->high_base = (((unsigned int) base) >> 24) & 0xff;
	gdtent->low_limit = (((unsigned int) size) >> 12) & 0xffff;
	gdtent->high_limit = (((unsigned int) size) >> 28) & 0xff;
	gdtent->gate_type = type;
	gdtent->dpl = priv;
	gdtent->present = 1;
	gdtent->granularity = PageGranularity;
}

/*
 * setIdtGate sets the values for the specified gate in the IDT array
 */
void
i586CPU::setIdtGate( int gateNumber, GateType type, unsigned short sel,
		    PFV addr, GatePrivilegeLevel priv)
{
	Assert( gateNumber < nInterrupts );

	IDT[gateNumber].low_offset = ((unsigned int) addr) & 0xffff;
	IDT[gateNumber].high_offset = (((unsigned int) addr) >> 16) & 0xffff;
	IDT[gateNumber].selector = sel;
	IDT[gateNumber].zeros = 0;
	IDT[gateNumber].gate_type = type;
	IDT[gateNumber].dpl = priv;
	IDT[gateNumber].present = 1;
}

i586PageFaultInterrupt PageFaultInterrupt;
i586GeneralProtectionInterrupt GeneralProtectionInterrupt;
 
void
i586CPU::installInterrupts()
{
	Debug << "i586CPU::installInterrupts()\n" << eor;

	/*
	 * The Proxy callgate is a special case; it doesn't go
	 *  through the default interrupt handler at all.
	 */

	setGdtGate( PROXY_Vector, CallGate, SYSTEM_CS,
			(PFV) &i586CPU::ProxyAssist, LvlThree, 0);

	setGdtGate( PROXYRETURN_Vector, CallGate, SYSTEM_CS,
			(PFV) &i586CPU::ProxyReturnAssist, LvlThree, 0);

	setInterrupt( PAGEFAULT_Vector, &PageFaultInterrupt );
	setInterrupt( GENERALPROTECTION_Vector, &GeneralProtectionInterrupt );
}

unsigned short DataSegments[ 2 ];
unsigned short CodeSegments[ 2 ];

void
i586CPU::setRingRange( unsigned char ring, const char *, const char * end )
{
	if ( ring == 0 ) {
		DataSegments[ 0 ] = SYSTEM_DS;
		CodeSegments[ 0 ] = SYSTEM_CS;
		return;
	}
	
	int gate = USERSEG_Vectors + ( ( ring - 1) * 2 );

	Assert( gate + 1 < MAXGdt_Vector );

	setGdtSegment( gate, DataSegment, LvlThree, 0, (unsigned int) end );
	setGdtSegment( gate + 1, CodeSegment, LvlThree, 0,
							(unsigned int) end );
	
	DataSegments[ ring ] = ( gate << 3 ) + LvlThree;
	CodeSegments[ ring ] = ( (gate + 1) << 3 ) + LvlThree;
}
