/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * i586ContextSwitching.s: assembly language context switching routines.
 */

#include "i586asmdefs.h"
#include "Vectors.h"
#include "ProxyTable.h"
	
	
#define APPLICATIONCONTEXT_VOID_CHECKPOINT	\
	.globl APPLICATIONCONTEXT_VOID_CHECKPOINT_NAME ; \
	APPLICATIONCONTEXT_VOID_CHECKPOINT_NAME:
	
#define GENERICCONTEXT_VOID_CHECKPOINT	\
	.globl GENERICCONTEXT_VOID_CHECKPOINT_NAME ; \
	GENERICCONTEXT_VOID_CHECKPOINT_NAME:

#define APPLICATIONCONTEXT_PROCESS_BASICRESTORE \
	.globl 	APPLICATIONCONTEXT_PROCESS_BASICRESTORE_NAME ; \
	APPLICATIONCONTEXT_PROCESS_BASICRESTORE_NAME:

#define GENERICCONTEXT_PROCESS_BASICRESTORE \
	.globl GENERICCONTEXT_PROCESS_BASICRESTORE_NAME ; \
        GENERICCONTEXT_PROCESS_BASICRESTORE_NAME:

#define SYSTEMCONTEXT_PROCESS_RESTOREFROMINITIALCONTEXT \
	.globl SYSTEMCONTEXT_PROCESS_RESTOREFROMINITIALCONTEXT_NAME ; \
	SYSTEMCONTEXT_PROCESS_RESTOREFROMINITIALCONTEXT_NAME:

#define APPLICATIONCONTEXT_PROCESS_RESTOREFROMINITIALCONTEXT \
	.globl APPLICATIONCONTEXT_PROCESS_RESTOREFROMINITIALCONTEXT_NAME ; \
	APPLICATIONCONTEXT_PROCESS_RESTOREFROMINITIALCONTEXT_NAME:
	
#define CPU_VOID_INTERRUPT_ASSIST \
	.globl CPU_VOID_INTERRUPT_ASSIST_NAME ; \
	CPU_VOID_INTERRUPT_ASSIST_NAME:
	
#define CPU_VOID_PROXY_ASSIST \
	.globl CPU_VOID_PROXY_ASSIST_NAME ; \
	CPU_VOID_PROXY_ASSIST_NAME:
	
#define CPU_VOID_PROXY_RETURN_ASSIST \
	.globl CPU_VOID_PROXY_RETURN_ASSIST_NAME ; \
	CPU_VOID_PROXY_RETURN_ASSIST_NAME:		

	
.extern CPU_STACKFRAME_HARDWAREINTERRUPTASSIST_NAME
			
/*
 * Process *
 * i586ApplicationContext::checkpoint( ProcessorContext * )
 *
 * The application context checkpoint/restore functions save/restore
 * the floating point registers and the user stack pointer as well.
 *
 * On Entry:
 *	top-of-stack = pointer to the context to save in.
 */

/*
APPLICATIONCONTEXT_VOID_CHECKPOINT
*/

	/*
	 * Assert( InSystemMode && OnSystemStack && InterruptsDisabled )
	 */

	/*
	 * Save the stuff the generic checkpoint doesn't.
	 * Nothing currently.
	 */
	
	/*
	 * Fall through to the parent method to save the rest.
	 */
	
GENERICCONTEXT_VOID_CHECKPOINT

	mov	4(%esp), %eax		/* get "this" pointer */

	/*
	 * Copy the kernel stack pointer for this process from the system's
	 *   TSS into this context.  It can get changed by a proxy call.
	 */
	mov	$chtss, %edx
	mov	ESP0__TASKSTATESEGMENT(%edx), %edx
	mov	%edx, STARTKERNELSTACK__I586GENERICCONTEXT(%eax)


	mov	%edi, EDI__I586GENERICCONTEXT(%eax)	/* save registers */
	mov	%esi, ESI__I586GENERICCONTEXT(%eax)
	mov	%ebx, EBX__I586GENERICCONTEXT(%eax)
	mov	%ebp, EBP__I586GENERICCONTEXT(%eax)

	mov	%esp, STACKPOINTER__I586GENERICCONTEXT(%eax)
	addl	$4, STACKPOINTER__I586GENERICCONTEXT(%eax) /* esp after return */
	mov	(%esp), %edx				/* return address */
	mov	%edx, PC__I586GENERICCONTEXT(%eax)

	mov	$0, %eax		/* return 0 the first time */
	ret

/*
APPLICATIONCONTEXT_PROCESS_BASICRESTORE
*/
	/*
	 * Restore the stuff the generic checkpoint doesn't.
	 */
	
	/*
	 * Fall through to the parent method to restore the rest.
	 */
	
GENERICCONTEXT_PROCESS_BASICRESTORE

	mov	4(%esp), %edx		/* get "this" pointer */

	/*
	 * Copy the kernel stack pointer for this process into
	 *   the system's TSS.  This gets read when there's an
	 *   interrupt which moves the processor from the user
	 *   priority level to the system priority level.
	 */
	mov	STARTKERNELSTACK__I586GENERICCONTEXT(%edx), %ecx
	mov	$chtss, %eax
	mov	%ecx, ESP0__TASKSTATESEGMENT(%eax)

	mov	8(%esp), %eax		/* return the Process parameter */

	mov	EDI__I586GENERICCONTEXT(%edx), %edi	/* restore registers */
	mov	ESI__I586GENERICCONTEXT(%edx), %esi
	mov	EBX__I586GENERICCONTEXT(%edx), %ebx
	mov	EBP__I586GENERICCONTEXT(%edx), %ebp

	mov	STACKPOINTER__I586GENERICCONTEXT(%edx), %esp	/* switch stacks */
	mov	PC__I586GENERICCONTEXT(%edx), %edx

	jmp	*%edx			/* checkpoint returns again! */
	/* the return value is in %eax, guaranteed to be non-zero */



/*
 * void
 * i586SystemContext::restoreFromInitialContext( Process * prev )
 */

SYSTEMCONTEXT_PROCESS_RESTOREFROMINITIALCONTEXT

	mov	4(%esp), %eax		/* "this" pointer */
	/*
	 * Copy the kernel stack pointer for this process into
	 *   the system's TSS.  This gets read when there's an
	 *   interrupt which moves the processor from the user
	 *   priority level to the system priority level.
	 */
	mov	STARTKERNELSTACK__I586GENERICCONTEXT(%eax), %edx
	mov	$chtss, %ecx
	mov	%edx, ESP0__TASKSTATESEGMENT(%ecx)

	mov	%esp, %edx		/* save old stack pointer */

	mov	STACKPOINTER__I586GENERICCONTEXT(%eax), %esp	/* switch stacks */
	push	8(%edx)			/* push Process * on new stack */
	push	4(%edx)			/* push "this" pointer on new stack */


	call SYSTEMCONTEXT_PROCESS_INITIALRESTOREASSIST_NAME

	/*  Assert(NOTREACHED) */
	/*call	HALT_NAME JMT HACK */
	
/*
 * void
 * i586ApplicationContext::restoreFromInitialContext( Process * prev )
 */
APPLICATIONCONTEXT_PROCESS_RESTOREFROMINITIALCONTEXT				

	mov	4(%esp), %eax		/* "this" pointer */
	mov	%esp, %edx		/* save old stack pointer */

	mov	STACKPOINTER__I586GENERICCONTEXT(%eax), %esp	/* switch stacks */
	push	8(%edx)			/* push Process * on new stack */
	push	4(%edx)			/* push "this" pointer on new stack */

	/*
	 * Copy the kernel stack pointer for this process into
	 *   the system's TSS.  This gets read when there's an
	 *   interrupt which moves the processor from the user
	 *   priority level to the system priority level.
	 */
	mov	STARTKERNELSTACK__I586GENERICCONTEXT(%eax), %edx
	mov	$chtss, %eax
	mov	%edx, ESP0__TASKSTATESEGMENT(%eax)


	/*
	 * First do what the Kernel requires of us.
	 */

	call APPLICATIONCONTEXT_PROCESS_INITIALRESTOREASSIST_NAME
	addl	$8,%esp			/* discard parameters */

	/*
	 * Set up the user's stack with arguments and a return address
	 */
	mov	32(%esp), %edx		/* the user's stack pointer */
	sub	$4, %edx
	pop	(%edx)			/* arg1 */
	sub	$4, %edx
	pop	(%edx)			/* arg0 */
	sub	$4, %edx
	pop	(%edx)			/* returnPoint */
	mov	%edx, 20(%esp)		/* re-set the user's stack pointer */

	pop	%eax			/* defaultProxy */

	pop	%ds			/* switch to new data segment */

	mov	$0, %ebp		/* clear base pointer */

	/*
	 * Start the application running for real by simultaneously loading
	 * the eflags, jumping to the entry point, and switching to the new
	 * stack. Our constructor put all this info on the system stack for us.
	 */
	iret


/*
 * Function called upon interrupt occuring on an 80386.
 */
CPU_VOID_INTERRUPT_ASSIST

	push	%eax
	push	%ecx
	push	%edx
	push	%es
	push	%fs
	push	%gs
	push	%ds

	movw	$SYSTEM_DS, %ax	/* switch to system data segment */
	movw	%ax, %ds

	mov	%esp, %eax	/* pass pointer to stackFrame */
	push	%eax
	
	call    CPU_STACKFRAME_HARDWAREINTERRUPTASSIST_NAME
	addl	$4, %esp	/* discard parameter */

	pop	%ds
	pop	%gs
	pop	%fs
	pop	%es
	pop	%edx
	pop	%ecx
	pop	%eax
	add	$8, %esp	/* skip vector and error code */
	iret

.privilegedProxy:
	movl	$1, %eax	/* indicates call from kernel */

	/* calculate old esp, stack hasn't changed */
	leal	USERSTACKPOINTER__LRETFRAME(%esp), %edx
	jmp	.privilegedContinue

/*
 * Function called upon Proxy call occurring on 80386.
 * This is called via a long-call callgate which copies no arguments
 *  on privilege transition, just switches stacks.
 */
CPU_VOID_PROXY_ASSIST
	push	%ds		/* save data segment */
	movw	%ds, %dx
	andb	$0x3, %dl
	jz	.privilegedProxy /* privileged proxy call */

	movl	$0, %eax	/* indicate call not from kernel */

	movw	$SYSTEM_DS, %dx	/* switch to system data segment */
	movw	%dx, %ds

	movl	USERSTACKPOINTER__LRETFRAME(%esp), %edx /* get old esp */

.privilegedContinue:

#if (CFRONT || REFBEFORETHIS)
	pushl	%ebx		/* save a compiler non-volatile reg */
#endif

	pushl	%ebp		/* save base pointer on stack for ReturnAssist*/
	movl	%esp,%ebp	/* save stack pointer in base pointer */

	/* save space for return flags and copied args */
	subl	$MAXPARAMS<<2+SZPROXYRETURNFLAGS, %esp

	pushl	%esp		/* pass a pointer to copied args */
	pushl	%edx		/* pass pointer to original stack */
#if (CFRONT || REFBEFORETHIS)
#if CFRONT
	movl	2<<2(%edx), %ebx  /* addr of optional return Ref (second arg) */
#else
	movl	1<<2(%edx), %ebx  /* addr of optional return Ref (first arg) */
#endif
	pushl	%ebx		/* refPointer */
#else
	pushl	$0		/* no refPointer */
#endif

	leal	-SZPROXYRETURNFLAGS(%ebp), %edx
	pushl	%edx		/* pass pointer to return flags */
	pushl	%eax		/* pass fromKernel flag */
	pushl	%ecx		/* pass method number */

	call	ProxyCallAssist

	addl	$6<<2, %esp	/* skip args */

	cmpl	$1, %eax	/* check return value */
	je	.quickCall	/* call to kernel with no ProxyReturnAssist */
	jl	.proxyDone	/* all work done */
	cmpl	$2, %eax
	jg	.userProxy	/* calling to user mode */
	/* calling to kernel mode */

	movl	-SZPROXYRETURNFLAGS+CALLEDMETHOD__PROXYRETURNFLAGS(%ebp),%eax

	call	*%eax		/* call method, args on stack */ 

.proxyReturn:
	leal	-SZPROXYRETURNFLAGS(%ebp), %edx
	pushl	%edx		/* pass pointer to return flags as second arg */

#if (CFRONT || REFBEFORETHIS)
	cmpb	$2, RETURNTYPE__PROXYRETURNFLAGS(%edx)
	jz	.proxyStarAssist

	/* it did not return Ref or Star */
#endif

	pushl	%eax		/* pass return value as first arg */
	call	ProxyReturnAssist
	/* don't need to remove args, the 'leave' will take care of it */
	jmp	.proxyDone

.quickCall:
	/* call to kernel with no call to ProxyReturnAssist */

	movl	-SZPROXYRETURNFLAGS+CALLEDMETHOD__PROXYRETURNFLAGS(%ebp),%eax

	call	*%eax		/* call method, args on stack */ 

.proxyDone:
	leave			/* restore stack pointer from base pointer */
				/* and retore base pointer from top of stack */

#if (CFRONT || REFBEFORETHIS)
	popl	%ebx		/* restore ebx */
#endif
	pop	%ds		/* restore user data segment */
	lret


#if (CFRONT || REFBEFORETHIS)
.proxyStarAssist:
	/* it did return Ref or Star */
	pushl	(%ebx)		/* pass the Ref value */
	call	ProxyReturnAssist
	movl	%eax, (%ebx)	/* write out return value */
#ifdef REFBEFORETHIS
	/* need to remove an extra parameter off user stack, what a pain */
	/* The '+8' is for the %ebx and %ebp on the kernel stack */
	movl	USERSTACKPOINTER__LRETFRAME+8(%ebp), %edx
	leal	1<<2(%edx), %eax
	movl	(%edx), %ebx	/* move up the return from stub */
	movl	%ebx, (%eax)
	movl	%eax, USERSTACKPOINTER__LRETFRAME+8(%ebp)
#endif
	jmp	.proxyDone

#endif

.userProxy:
	/*
	 * stack is all set up by i586GenericContext::setNewRing to jump
	 *    to user.  User will return to i586CPU::ProxyReturnAssist
	 *    below.
	 */

	/*
	 * Even though the following are compiler non-volatile, we can't
	 *   trust that the user won't write his own assembly code to
	 *   mess with these registers so we need to save them.
	 */
	movl	%edi, EDI__USERPROXYSTATE+SZLRETFRAME(%esp)
	movl	%esi, ESI__USERPROXYSTATE+SZLRETFRAME(%esp)
	movl	%ebx, EBX__USERPROXYSTATE+SZLRETFRAME(%esp)
	movl	%ebp, EBP__USERPROXYSTATE+SZLRETFRAME(%esp)
	
	pop	%ds
	lret



CPU_VOID_PROXY_RETURN_ASSIST
	movw	$SYSTEM_DS, %dx	/* switch to system data segment */
	movw	%dx, %ds

	/*
	 * Restore the compiler non-volatile registers
	 * We haven't saved %ds on the stack, thus the -4.
	 */
	movl	EDI__USERPROXYSTATE+SZLRETFRAME-4(%esp), %edi
	movl	ESI__USERPROXYSTATE+SZLRETFRAME-4(%esp), %esi
	movl	EBX__USERPROXYSTATE+SZLRETFRAME-4(%esp), %ebx
	movl	EBP__USERPROXYSTATE+SZLRETFRAME-4(%esp), %ebp

	jmp	.proxyReturn
