/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Ruth A. Aydt (aydt@cs.uiuc.edu)
 * Contributing Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Contributing Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Contributing Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * i586MMU.c: An 80586 as an AddressTranslator.
 */

#include "ADDRESS_TRANSLATION.h"
#include "Assert.h"
#include "OutputStream.h"
#include "PageTableConstants.h"
#include "PROCESSOR_CONTEXT.h"
#include "SegmentDescriptor.h"
#include "TaskStateSegment.h"
#include "Util.h"
#include "i586MMU.h"

i586MMU::i586MMU()
{
  Debug << this << "::ctor()\n" << eor;
}

void
i586MMU::enable()
{
  Debug << "i586MMU::enable()\n" << eor;

	/* 
	 * Turn on the paging.  First get the current value of the
	 * Machine Status Word (CR0).  Then, turn on the PG bit and
	 * reload MSW.  Note, I did try using the asm statements
	 * "smsw" and "lmsw" but didn't have any luck with them.
	 */

	unsigned int cmsw = Smsw();
	_Lmsw ( cmsw | 0x80000000 );

	Debug << this << ":Paging just turned on!\n" << eor;

	//BreakIt();		// We expect an interrupt 14

}

int
i586MMU::enabled()
{
	return((Smsw() & 0x80000000) == 0x80000000);
}


extern TaskStateSegment chtss;

void
i586MMU::basicActivate( AddressTranslation * at )
{
  Debug << "i586MMU::basicActivate( translation:" << asAddress(at) << ")\n" << eor;

	POINTER_TABLE_ENTRY * pointerTable =
			((ADDRESS_TRANSLATION *) at)->pointerTable();

	/*
	 * Next we put the base address of the Level 0 Page Table into
	 * into the TSS PDBR of the currently running task.
	 */
	chtss.pdbr = (unsigned int) pointerTable;

	/* Finally Load CR3 (Page Directory Base Register) */
	_Lcr3( (unsigned int) pointerTable );
}

/*
 * On the 80586 we can't flush just one entry  of the TLB so use FlushTLB().
 * Should be smarter so this doesn't have to be called first, but for now
 * that's life.
 */
void
i586MMU::flushCache( const char *, unsigned int )
{
  Debug << "i586MMU::flushCache() -- did it work???? \n" << eor;
  FlushTLB();
}

#include "ClassInstaller.h"
DEFINECLASS("i586MMU",i586MMU,AddressTranslator,0,0);
