/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Contributing Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Contributing Author: Bjorn A. Helgaas (helgaas@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * i586PageTableEntry.cc:  Page table entries for a i586 MMU.
 */

#include "i586PageTableEntry.h"


void
i586PointerTableEntry::setProtection( ProtectionLevel protection )
{
	Assert( protection < Pte_PROTECT_LEVELS );
	Assert( valid() );

	i586Protection realProt;

	/*
	 * Convert protection to the corresponding i586 level.
	 */
	if( protection == ReadWrite ) {
		realProt = USERRW;
	}
	else if( protection == ReadOnly ) {
		realProt = USERRO;
	}
	else if( protection == NoAccess ) {
		realProt = SYSTEMRW;
	}
	else {
		realProt = SYSTEMRO;	// shut up compiler warning.
		Assert( NOTREACHED );
	}

	/*
	 * Set the protection.
	 */
	entry &= ~0x6;
	entry |= realProt << 1;
}

void
i586PageTableEntry::setProtection( ProtectionLevel protection )
{
	Assert( protection < Pte_PROTECT_LEVELS );
	Assert( valid() );

	i586Protection realProt;

	/*
	 * Convert protection to the corresponding i586 level.
	 */
	if( protection == ReadWrite ) {
		realProt = USERRW;
	}
	else if( protection == ReadOnly ) {
		realProt = USERRO;
	}
	else if( protection == NoAccess ) {
		realProt = SYSTEMRW;
	}
	else {
		realProt = SYSTEMRO;	// shut up compiler warning.
		Assert( NOTREACHED );
	}

	/*
	 * Set the protection.
	 */
	entry &= ~0x6;
	entry |= realProt << 1;
}

int
i586PageTableEntry::okToPerform( AccessType attemptedAccess )
{
	i586Protection currentProtection = (i586Protection) ((entry & 0x6) >> 1);

	i586Protection neededProtection;
	switch( attemptedAccess ) {
		case ApplicationWrite:
			neededProtection = USERRW;
			break;
		case ApplicationRead:
			neededProtection = USERRO;
			break;
		case SystemWrite:
			neededProtection = SYSTEMRW;
			break;
		case SystemRead:
			neededProtection = SYSTEMRO;
			break;
		default:
			neededProtection = SYSTEMRO;	// shut up warning.
			Assert( NOTREACHED );
	}

	return( currentProtection >= neededProtection );
}

void
i586PageTableEntry::mapToAddress( const char * addr,
	ProtectionLevel protection )
{
	Assert( !valid() );
	Assert( PageAligned( (const char *) addr ) );

	entry = (unsigned int) addr | 0x1;

	i586Protection realProt;
	switch( protection ) {
		case ReadWrite:
			realProt = USERRW;
			break;
		case ReadOnly:
			realProt = USERRO;
			break;
		case NoAccess:
			realProt = SYSTEMRW;
			break;
		default:
			realProt = SYSTEMRO;	// shut up warning.
			Assert( NOTREACHED );
	}

	entry = (unsigned int) addr | ( realProt << 1 ) | 0x1;
}
