/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Contributing Author: Gary M. Johnston (johnston@cs.uiuc.edu)
 * Contributing Author: Peter W. Madany (madany@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SourceOfZerosMemoryObject.c - fill a physical buffer with zeros.
 */

#include "i586SourceOfZerosMemoryObject.h"
#include "Assert.h"
#include "Domain.h"
#include "OutputStream.h"
#include "PhysicallyAddressableUnit.h"
#include "PhysicalMemoryChain.h"
#include "ThisProcess.h"
#include "Util.h"


i586SourceOfZerosMemoryObject theZeroObject;
i586SourceOfZerosMemoryObject * TheZeroObject = &theZeroObject;


extern "C" {
   void PhysZeroBlock(unsigned char *, int);
};

int
i586SourceOfZerosMemoryObject::write( unsigned int, int count,
				char * addr, int nbytes )
{
	Assert (NOTREACHED);
}

int
i586SourceOfZerosMemoryObject::read( unsigned int, int count,
				char * addr, int nbytes )
{
	PhysicalMemoryChain chain = thisProcess()->domain()->constructChain(
						addr, nbytes );

	Debug.printf( "SourceOfZerosMO::read( count:%d chain:%A )\n",
		count, &chain );
	Assert( &chain != 0 );

	PhysicalMemoryChainContiguousBlockIterator iterator( &chain, count );
	unsigned int length;
	const char * nextBlock;

	unsigned int total = 0;

	while( ( nextBlock = iterator.next( length ) ) != 0 ) {
		Debug.printf( "SourceOfZerosMO::read: zeroBlock( addr:%A len:%d )\n", 
			nextBlock, length );
		PhysZeroBlock( (unsigned char *) nextBlock, length );
		total += length;
	}
	Assert( total == count );
	return( total );
}


#include "ClassInstaller.h"
DEFINECLASS("i586SourceOfZerosMemoryObject",i586SourceOfZerosMemoryObject,MemoryObject,0,0);
