#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * FIFOSchedulerUSS.c: a first-in first-out scheduler
 */

#include "Assert.h"
#include "CPU.h"
#include "OutputStream.h"
#include "Process.h"
#include "SchedulerUSS.h"


#ifdef __GNUC__
#pragma implementation
#endif


/*
 *  Initialize the FIFO Scheduler as a queue.
 */
FIFOSchedulerUSS::FIFOSchedulerUSS( char * name )  
{
	Debug << this << "::FIFOSchedulerUSS( " << name << " )\n" << eor;
	if( readyList == 0 ) readyList = (SchedList *) new SchedQueue( name );
}

/*
 * The priority function sets the process priority in its schedulerInfo
 * class.  In this case it just indicates that the process is to be added
 * to the front of the queue.
 */
void
FIFOSchedulerUSS::priority( Process * process )
{
	Debug << this << "::priority( " << process << " )\n" << eor;
	USSSchedulerInfo * myInfo = 
		(USSSchedulerInfo *)process->schedulerInfo();

	/*
	 * This is a new process in the system so create a schedulerInfo
	 * class for it.
	 */
	if( myInfo == 0 ) {
		myInfo = new USSSchedulerInfo( process );
		process->setSchedulerInfo( myInfo );
	}
}
