#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */

/*
 * HRNSchedulerUSS.c: Highest-Response-Ratio-Next Scheduler
 */

#include "Assert.h"
#include "CPU.h"
#include "FreeRunningTimer.h"
#include "Object.h"
#include "OutputStream.h"
#include "Process.h"
#include "SchedulerUSS.h"


#ifdef __GNUC__
#pragma implementation
#endif


/*
 * Contains the per-process scheduling info.  The time the process entered
 * the scheduler is saved and used to calculate priority.
 */
HRNSchedulerInfo::HRNSchedulerInfo( Process * process ) : ( process )
{
	startTime = SystemTimer->time();
	Debug << this << "::HRNSchedulerInfo( " << process << " ) wait at "
		<< startTime << ", service = " << process->getServiceTime()
		<< "\n" << eor;
}

/*
 *                (time waiting + service time)
 *     priority = -----------------------------
 *		 	service time
 */
double
HRNSchedulerInfo::getDoublePriority( unsigned int now )
{
	unsigned int serviceTime = getProcess()->getServiceTime();
	return( (double)(now - startTime + serviceTime)/serviceTime );
}

/*
 *  Initialize the HRN Scheduler using a search list.
 */
HRNSchedulerUSS::HRNSchedulerUSS( char * name )
{
	Debug << this << "::HRNSchedulerUSS( " << name << " )\n" << eor;
	readyList = (SchedList *) new SchedSearchList( name );
}

/*
 * Setup the process schedulerInfo.  Since HRN is a nonpreemptive algorithm
 * and a process should run to completion once started, if the process comes
 * through here more than once then some outside interrupt occurred.  In this 
 * case that process should not lose its priority so its service time is not 
 * reduced and its start time is not altered.
 */
void
HRNSchedulerUSS::priority( Process * process )
{
	Debug << this << "::priority( " << process << " )\n" << eor;
	Assert( process != 0 );

	/* set the process start waiting time on a new process */
	HRNSchedulerInfo * myInfo = 
		(HRNSchedulerInfo *) process->schedulerInfo();
	if( myInfo == 0 ) {
		myInfo = new HRNSchedulerInfo( process );
		process->setSchedulerInfo( myInfo );
	}
}
