#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Douglas E. Leyens (leyens@cs.uiuc.edu)
 * Contributing Author: Vincent F. Russo (russo@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * SystemScheduler.c: system scheduler
 */

//#define DEBUG 1

#include "Assert.h"
#include "FIFOScheduler.h"
#include "OutputStream.h"
#include "Process.h"
#include "SystemScheduler.h"


#ifdef __GNUC__
#pragma implementation
#endif


SystemScheduler::SystemScheduler( ProcessContainer * UserScheduler,
	char * name ) 
{
	Debug << this << "::SystemScheduler( " << UserScheduler << ", "
		<< name << " )\n" << eor;

	/* create a FIFO scheduler for nonpreemptible processed */
	NonpreemptScheduler = new FIFOScheduler();

	/* create another scheduler for user processes */
	PreemptScheduler = UserScheduler;
}

SystemScheduler::~SystemScheduler()
{
	Assert( this != 0 );
	delete NonpreemptScheduler;
	Debug << this << "::~SystemScheduler()\n" << eor;
}

/*
 * Add an element to the scheduler queue.  If it is preemptible then add it
 * to the user scheduler.  Otherwise add it to the system FIFO scheduler.
 */
void
SystemScheduler::add( Process * process )
{
	// Debug << this << "::add( " << process << " )\n" << eor;
	Assert( process != 0 );
	Assert( this != 0 );

	if( process->isPreemptable() )
		PreemptScheduler->add( process );
	else
		NonpreemptScheduler->add( process );
	
	//JLC Debug << this << "::add: returning\n" << eor;
}

/*
 * Remove the first entry from the nonpreemptable scheduler.  If empty then
 * remove one from the preemptable scheduler.
 */
Process *
SystemScheduler::remove()
{
	Process * retval = 0;

	Assert( this != 0 );

	if( ! NonpreemptScheduler->isEmpty() )
		retval = NonpreemptScheduler->remove();
	if( (retval == 0) && (! PreemptScheduler->isEmpty()) )
		retval = PreemptScheduler->remove();
	
	// if( retval )
	//	Debug << this << "::remove: " << retval << "\n" << eor;
	return( retval );
}

int
SystemScheduler::isEmpty()
{
	return( NonpreemptScheduler->isEmpty() && PreemptScheduler->isEmpty() );
}
