"""IDL back-end:client"""

from omniidl import idlast, idltype, idlutil, idlvisitor, output
import sys, string

includes = {}

friends = {}

defines = ""

nonproxy = []

class ClientVisitor (idlvisitor.AstVisitor):

    	def visitAST(self, node):
    		for n in node.declarations():
        		n.accept(self)
		

    	def visitModule(self, node):
        	if node.identifier() != "Choices":
			print "Error in %(file)s at line %(line)d: expected module Choices" % { 'file': node.file(), 'line': node.line() } 
		else:
			self.class_number = 0
			for n in node.definitions():
            			n.accept(self)

    	def visitInterface(self, node):
		global defines
		if node.pragmas() != [] and str(node.pragmas()[0]) == "nonproxy":
			return
    		self.class_name = node.identifier()
		if self.class_number>=255:
			print "Error in %(file)s at line %(line)d: can not use more than 255 classes" % { 'file': node.file(), 'line': node.line() }
			return
		self.class_number+=1
		if node.pragmas() != []:
			defines+="#define "+self.class_name+"ID "+str(self.class_number)+"\n"
			return
		self.op_number = 1
		self.parents = []
		self.pmo = ""
		if node.inherits() != []:
			parent = node.inherits()[0];
			self.base = parent.identifier();
			self.parents.append(parent.identifier());
			while parent.inherits() != []:
				parent = parent.inherits()[0];
				self.parents.append(parent.identifier());
			if "PersistentObject" in self.parents:
				self.pmo = "Persistent";
			else:
				self.pmo = "";
		else:
			self.base = ""
		self.h_file = output.Stream(open("../../Includes/Libraries/SystemInterface/ARM/" + self.class_name + '.h','w'),2)
		self.cc_file = output.Stream(open("../../Libraries/SystemInterface/ARM/" + self.class_name + '.cc','w'),2)
	
		self.public_constructor = False
		self.public_destructor = False
		
		forlist=""
		inclist=""
		
		for c in includes[node.identifier()]:
			inclist+=('#include "' + c +'.h"\n')
			forlist+=('class ' + c +';\n')
			
		friendlist=""
		
		for c in friends[node.identifier()]:
			if c=="TheInterface":
				friendlist+=('friend class Interface;\n')
			else:
				friendlist+=('friend class ' + c +';\n')
			
		h_base = ""
		private_base="void *kernelObj;"
		self.cons_base=""
		if self.base != "":
			forlist+=('#include "' + self.base +'.h"\n')
			h_base = (':public ' + self.base)
			private_base=""
			self.cons_base=(':' + self.base + '((void*)0)')
		if self.base=="":
			self.h_file.out("""\
#ifndef _@name@_H_
#define _@name@_H_
#include "common.h"
@forl@
extern @pmo@ClassStar @name@Class;
class @name@ @h_base@
{
  public:
    virtual Class *classOf();
""", name=self.class_name, forl=forlist, h_base=h_base, pmo=self.pmo)
		else:
			self.h_file.out("""\
#ifndef _@name@_H_
#define _@name@_H_
#include "common.h"
@forl@
extern @pmo@ClassStar @name@Class;
class @name@ @h_base@
{
  public:
    @name@(ProxiableObject* p) @cons_base@ {kernelObj=p->getKernelObj();}
    virtual Class *classOf();
""", name=self.class_name, forl=forlist, h_base=h_base, cons_base=self.cons_base,pmo=self.pmo)
		if self.class_name == "ProxiableObject":
			self.h_file.out("""\
    void* getKernelObj(){return kernelObj;}
""")
		if self.class_name == "OutputStream":
			self.h_file.out("""\
    enum OutputStreamPadChar { aZero, aSpace };
    enum OutputStreamPadSide { padLeft, padRight };

    void emitChar( char c, int width, OutputStreamPadSide padSide );
    void emitString( const char * string, int width, OutputStreamPadSide );
    void emitSpaces( int spaceCount );
    void emitZeros( int zeroCount );
    void emitNumber( unsigned int arg, int base, int width,
                     OutputStreamPadSide padSide, OutputStreamPadChar padChar);

    OutputStream & operator <<( int );
    OutputStream & operator <<( unsigned int );
    OutputStream & operator <<( long );
    OutputStream & operator <<( unsigned long );
    OutputStream & operator <<( char );
    OutputStream & operator <<( const char * );
    OutputStream & operator <<( const void * );
    OutputStream & operator <<( Flush  fl ) { flush(); return(*this); }
""")

		if self.class_name == "InputStream":
			self.h_file.out("""\
    InputStream & operator >>( int & num) { formattedRead("%d",(void *)&num); return( *this); };
""")
 	
		self.cc_file.out("""\
@inc@
#include "@name@.h"
#include "exception.h"

@name@::@name@(void* k) @cons_base@ {kernelObj=k;}
""",inc=inclist, name=self.class_name, cons_base=self.cons_base)

		for c in node.callables():
			c.accept(self)
	
		self.h_file.out("""\
    virtual ~@name@();
    
  protected:
    @private_base@
    @name@(void* k);
    @fr@
};

#endif""",name = self.class_name, fr = friendlist, private_base=private_base)

		if self.class_name == "OutputStream":
			self.cc_file.out("""\
#include "OutputStreamExtra.cc"
""")

		if self.public_destructor:
			self.cc_file.out("""\
@name@::~@name@() 
{
  kernel_exception::EXCODE code;
  asm volatile (
		"mov r0, #1\\n\\t"
		"mov r1, %1\\n\\t"
		"swi $@cnum@\\n\\t"
		"str r0, %0"
		: "=m" (code) :"r" (kernelObj) : "r0", "r1", "r2", "r3"
		);
  kernel_exception::throw_exception(code);
}""",name = self.class_name, cnum=3840+self.class_number)
		else:
			self.cc_file.out("""\
@name@::~@name@() {}""",name = self.class_name)	
		if self.base == "":
			self.base = "Null"
		self.cc_file.out("""\
#include "ClassInstaller.h"
DEFINE@pmo@CLASS("@name@",@name@,@base@,0,0);
""",name = self.class_name, base = self.base, pmo = string.upper(self.pmo))


	def visitOperation(self, node):
		self.precode=""
		self.postcode=""
		self.sizeofs=""
		self.op_name=node.identifier().rstrip('0123456789')
		if self.op_name.find(self.class_name)==0:
			self.op_name = self.op_name[len(self.class_name):]
		if self.op_name == "constructor": #the constructor
			if node.returnType().kind() != idltype.tk_void:
				 print "Error in %(file)s at line %(line)d: no return type is admissible in constructor" % { 'file': node.file(), 'line': node.line() }
				 return
			if self.public_constructor == True:
				print "Error in %(file)s at line %(line)d: only one public constructor is allowed" % { 'file': node.file(), 'line': node.line() }
				return
			self.public_constructor = True
			if node.parameters() == []: #no param
				self.h_file.out("""\
    @name@(); // __attribute((always_inline));
""",name=self.class_name)
				self.cc_file.out("""\
@name@::@name@() @cons_base@
{
  kernel_exception::EXCODE code;
  asm volatile (
		"mov r0, #0\\n\\t"
		"mov r3, %1\\n\\t"
		"swi #@cnum@\\n\\t"
		"str r0, %0"
		: "=m"(code) :"r"(&kernelObj) : "r0", "r1", "r2", "r3" 
		);
  kernel_exception::throw_exception(code);
}
""",name=self.class_name, cnum=3840+self.class_number, cons_base=self.cons_base)

			else: #constructor parameter

				self.param = ""
				for t in node.parameters():
					t.accept(self)
				
				self.h_file.out("""\
    @name@(@param@); // __attribute((always_inline));
""",name=self.class_name, param=self.param)
				self.cc_file.out("""\
@name@::@name@(@param@) @cons_base@
{
  kernel_exception::EXCODE code;
  char paramBuffer[0@size@];
  @precode@
  asm volatile (
		"mov r0, #0\\n\\t"
		"mov r2, %1\\n\\t"
		"mov r3, %2\\n\\t"
		"swi #@cnum@\\n\\t"
		"str r0, %0"
		: "=m"(code) : "r" (paramBuffer), "r"(&kernelObj) : "r0", "r1", "r2", "r3" 
		);
  kernel_exception::throw_exception(code);
  @postcode@
}
""",name=self.class_name, param = self.param, size=self.sizeofs, cnum=3840+self.class_number, precode=self.precode, postcode=self.postcode, cons_base=self.cons_base)
		
		elif self.op_name == "destructor": #the destructor
			if node.returnType().kind() != idltype.tk_void:
				 print "Error in %(file)s at line %(line)d: no return type is admissible in destructor" % { 'file': node.file(), 'line': node.line() }
				 return
			if self.public_destructor == True:
				print "Error in %(file)s at line %(line)d: only one public destructor is allowed" % { 'file': node.file(), 'line': node.line() }
				return
			self.public_destructor = True
			
		else: #standard method
			self.op_number+=1
			self.param= ""
			self.direction = 3 #return
			node.returnType().accept(self)
			returnparam=self.param
			self.param = ""
			for t in node.parameters():
				t.accept(self)
			if returnparam == "void" and self.param == "":
				self.h_file.out("""\
    virtual void @op@(); // __attribute((always_inline));
""",op=self.op_name)
				self.cc_file.out("""\
void @name@::@op@()
{
  kernel_exception::EXCODE code;
  @precode@
  asm volatile ("mov r0, #@opnum@\\n\\t"
		"mov r1, %1\\n\\t"
		"swi #@cnum@\\n\\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  @postcode@
}
""",name=self.class_name, op=self.op_name, opnum=self.op_number, cnum=3840+self.class_number, precode=self.precode, postcode=self.postcode)
			
			elif returnparam == "void" and self.param != "":
				self.h_file.out("""\
    virtual void @op@(@param@); // __attribute((always_inline));
""",name=self.class_name, op=self.op_name, param=self.param)
				self.cc_file.out("""\
void @name@::@op@(@param@)
{
  kernel_exception::EXCODE code;
  char paramBuffer[0@size@];
  @precode@
  asm volatile ("mov r0, #@opnum@\\n\\t"
		"mov r1, %1\\n\\t"
		"mov r2, %2\\n\\t"
		"swi #@cnum@\\n\\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  @postcode@
}
""",name=self.class_name, op=self.op_name, param = self.param, size=self.sizeofs, opnum=self.op_number, cnum=3840+self.class_number, precode=self.precode, postcode=self.postcode)
			
			
			elif returnparam != "void" and self.param == "":
				self.h_file.out("""\
    virtual @retparam@ @op@(); // __attribute((always_inline));
""",name=self.class_name, op=self.op_name, retparam=returnparam)
				if self.op_name != "classOf":
					self.cc_file.out("""\
@retparam@ @name@::@op@()
{
  @retparam@ ret;
  kernel_exception::EXCODE code;
  @precode@
  asm volatile ("mov r0, #@opnum@\\n\\t"
		"mov r1, %1\\n\\t"
		"mov r3, %2\\n\\t"
		"swi #@cnum@\\n\\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  @postcode@
  return ret;
}
""",name=self.class_name, op=self.op_name, retparam = returnparam, opnum=self.op_number, cnum=3840+self.class_number, precode=self.precode, postcode=self.postcode)
			
			else:
				self.h_file.out("""\
    virtual @retparam@ @op@(@param@); // __attribute((always_inline));
""",name=self.class_name, op=self.op_name, retparam=returnparam, param = self.param)
				self.cc_file.out("""\
@retparam@ @name@::@op@(@param@)
{
  @retparam@ ret;
  kernel_exception::EXCODE code;
  char paramBuffer[0@size@];
  @precode@
  asm volatile ("mov r0, #@opnum@\\n\\t"
		"mov r1, %1\\n\\t"
		"mov r2, %2\\n\\t"
		"mov r3, %3\\n\\t"
		"swi #@cnum@\\n\\t"
		"str r0, %0"
		: "=m"(code) : "r" (kernelObj), "r" (paramBuffer), "r" (&ret) : "r0", "r1", "r2", "r3" );
  kernel_exception::throw_exception(code);
  @postcode@
  return ret;
}
""",name=self.class_name, op=self.op_name, retparam = returnparam, param = self.param, size=self.sizeofs, opnum=self.op_number, cnum=3840+self.class_number, precode=self.precode, postcode=self.postcode)
				

	def visitParameter(self, node):
		self.direction = node.direction()
		self.param_name=node.identifier()
		self.ref=False
		if self.param!="":
			self.param+=", "
		node.paramType().accept(self);
		self.param+= (" " + node.identifier() )
		if self.ref==True:
			self.sizeofs+=("+sizeof(void*)")
		else:
			self.sizeofs+=("+sizeof("+node.identifier()+")")
		
	ttsMap = {
        	idltype.tk_void:       "void",
       		idltype.tk_short:      "short",
       	 	idltype.tk_long:       "long",
        	idltype.tk_ushort:     "unsigned short",
        	idltype.tk_ulong:      "unsigned long",
        	idltype.tk_float:      "float",
        	idltype.tk_double:     "double",
        	idltype.tk_boolean:    "bool",
        	idltype.tk_char:       "char",
		idltype.tk_any:        "void*",
        }
		
	def visitBaseType(self, type):
		self.param += self.ttsMap[type.kind()]
		if self.direction==0: #in
			self.precode+=("*("+self.ttsMap[type.kind()]+"*)(paramBuffer" + self.sizeofs + ") = " + self.param_name + ";\n" )
		elif self.direction==1: #out
			self.param += "*"
			self.precode+=("*("+self.ttsMap[type.kind()]+"**)(paramBuffer" + self.sizeofs + ") = " + self.param_name + ";\n" )
		elif self.direction==2: #inout
			self.param += "&"
			self.ref=True
			self.precode+=("*("+self.ttsMap[type.kind()]+"**)(paramBuffer" + self.sizeofs + ") = &" + self.param_name + ";\n" )
			
	def visitDeclaredType(self, type):
		global nonproxy
		if type.name() == "int":
			self.param += type.name()
			if self.direction==0: #in
				self.precode+=("*(int*)(paramBuffer" + self.sizeofs + ") = " + self.param_name + ";\n" )
			elif self.direction==1: #out
				self.param += "*"
				self.precode+=("*(int**)(paramBuffer" + self.sizeofs + ") = " + self.param_name + ";\n" )
			elif self.direction==2: #inout
				self.param += "&"
				self.ref=True
				self.precode+=("*(int**)(paramBuffer" + self.sizeofs + ") = &" + self.param_name + ";\n" )
		elif type.name() == "uint":
			self.param += "unsigned int"
			if self.direction==0: #in
				self.precode+=("*(unsigned int*)(paramBuffer" + self.sizeofs + ") = " + self.param_name + ";\n" )
			elif self.direction==1: #out
				self.param += "*"
				self.precode+=("*(unsigned int**)(paramBuffer" + self.sizeofs + ") = " + self.param_name + ";\n" )
			elif self.direction==2: #inout
				self.param += "&"
				self.ref=True
				self.precode+=("*(unsigned int**)(paramBuffer" + self.sizeofs + ") = &" + self.param_name + ";\n" )
		elif type.name() in includes[self.class_name] or type.name() == self.class_name:
			if self.direction==1: #out
				self.param += type.name() +"**"
				self.precode+= ("*("+type.name()+"***)(paramBuffer" + self.sizeofs + ") = " +self.param_name + ";\n" )
				if type.name() not in nonproxy:
					self.postcode+=("*" + self.param_name + " = new " + type.name() + "( (void*)(*" + self.param_name + ") );\n" )
			elif self.direction == 3: #ret
				self.param += type.name() +"*"
				if type.name() not in nonproxy:
					self.postcode+=( "ret = new " + type.name() + "( (void*)ret );\n" )
			elif self.direction == 2: #inout
				self.param += type.name() +"&"
				self.ref=True
				if type.name() not in nonproxy:
					self.precode+= ("*("+type.name()+"**)(paramBuffer" + self.sizeofs + ") = (" +type.name() + "*)(" + self.param_name + ".kernelObj);\n" )
				else:
					self.precode+=("*("+type.name()+"**)(paramBuffer" + self.sizeofs + ") = &" + self.param_name + ";\n" )
			else: #in
				self.param += type.name() +"*"
				if type.name() not in nonproxy:
					self.precode+= ("*("+type.name()+"**)(paramBuffer" + self.sizeofs + ") = (" +type.name() + "*)(" + self.param_name + "->kernelObj);\n" )
				else:
					self.precode+=("*("+type.name()+"**)(paramBuffer" + self.sizeofs + ") = " + self.param_name + ";\n" )

		else:
			self.param += type.name()
			if self.direction==0: #in
				self.precode+=("*("+type.name()+"*)(paramBuffer" + self.sizeofs + ") = " + self.param_name + ";\n" )
			elif self.direction==1: #out
				self.param += "*"
				self.precode+=("*("+type.name()+"**)(paramBuffer" + self.sizeofs + ") = " + self.param_name + ";\n" )
			elif self.direction==2: #inout
				self.ref=True
				self.param += "&"
				self.precode+=("*("+type.name()+"**)(paramBuffer" + self.sizeofs + ") = &" + self.param_name + ";\n" )
			
				
	def visitStringType(self, type):
		self.param+="char*"
		if self.direction==0: #in
			self.precode+=("*(char**)(paramBuffer" + self.sizeofs + ") = " + self.param_name + ";\n" )
		elif self.direction==1: #out
			self.param += "*"
			self.precode+=("*(char***)(paramBuffer" + self.sizeofs + ") = " + self.param_name + ";\n" )
		elif self.direction==2: #inout
			self.param += "&"
			self.ref=True
			self.precode+=("*(char***)(paramBuffer" + self.sizeofs + ") = &" + self.param_name + ";\n" )
			
class RelationVisitor (idlvisitor.AstVisitor):

    	def visitAST(self, node):
    		for n in node.declarations():
        		n.accept(self)
		
    	def visitModule(self, node):
		for n in node.definitions():
            		n.accept(self)
			
	def visitForward(self, node):
		#print "Forward interface "+node.identifier()
		if node.identifier() not in includes:
			includes[node.identifier()]=[]
		if node.identifier() not in friends:
			friends[node.identifier()]=[]

    	def visitInterface(self, node):
		global nonproxy
		self.interface = node.identifier()
		if node.pragmas()!=[] and str(node.pragmas()[0]) == "nonproxy":
			nonproxy.append(node.identifier())
		if node.identifier() not in includes:
			includes[node.identifier()]=[]
		if node.identifier() not in friends:
			friends[node.identifier()]=[]
		for c in node.callables():
			c.accept(self)
	
	def visitOperation(self, node):
		node.returnType().accept(self)
		for t in node.parameters():
			t.paramType().accept(self)
			
	def visitBaseType(self, type):
		return
			
	def visitDeclaredType(self, type):
		if type.name() in includes:
			if self.interface != type.name():
				if type.name() not in includes[self.interface]:
					includes[self.interface].append(type.name())
				if self.interface not in friends[type.name()]:
					friends[type.name()].append(self.interface)
				
	def visitStringType(self, type):
		return
				
def run(tree, args):
    visitor = RelationVisitor()
    tree.accept(visitor)
    visitor = ClientVisitor()
    tree.accept(visitor)
    
    common_file = output.Stream(open('../../Includes/Libraries/SystemInterface/ARM/common.h','w'),2)
    common_file.out("""
#ifndef __COMMON__
#define __COMMON__
@defines@
#include "ProtectionLevel.h"
#include "FormatAssists.h"
#include "ClassStar.h"
#include "PersistentClassStar.h"
#include "Assert.h"
class ProxiableObject;
class OutputStream;
class InputStream;
class SystemInterface;
class FileSystemInterface;
class NameServer;

/*
 * A pointer to a function taking an object of a Class.
 * Used by the applyTo... methods to do something to each object.
 */
typedef void (*ClassApplicationFunction)(ProxiableObject *);

extern ClassStar NullClass;
extern OutputStream * StandardOutput;
extern InputStream * StandardInput;
extern SystemInterface * StandardSystemInterface;
extern FileSystemInterface * StandardFileSystemInterface;
extern NameServer * StandardNameServer;

typedef void (*PFV)(void *);
typedef unsigned int Offset;
typedef ProxiableObject *(*PConstructor)(char *, unsigned int, unsigned int,
						unsigned int, unsigned int);
typedef ProxiableObject * (*POFPO)( ProxiableObject * );
typedef void (*ClassApplicationFunction)(ProxiableObject *);

#define proxiable
#define Console() (*StandardOutput)

enum ProcessState {
  ProcessNew= 0,
  ProcessReady = 1,
  ProcessRunning = 2,
  ProcessBlocked = 3,
  ProcessDead = 4
};

typedef int StackSize;
const StackSize heapBootStack = -1;
const StackSize none=0;
const StackSize tiny=2;
const StackSize small=4;
const StackSize normal=8;
const StackSize application=16;
const StackSize large=32;
const StackSize huge=64;
const StackSize gigantic=128;
	
enum Return_Code {
	OK = 0,
	ERROR = -1,
			// Just generally wrong. Unspecified error.
	ERROR_IN_SPECIFICATION = -2,
			// E.g. illegal protocol specification
	NO_SUCH_ADDRESS = -3,
			// The given handle doesn't specify anything
	NO_SUCH_CONDUIT = -4,
			//       "            "            "
	NO_SUCH_DESTINATION = -5,
			//       "            "            "
	NO_SUCH_MESSAGE = -6,
			//       "            "            "
	NO_SUCH_CONNECTION = -7,
			// TCP error
	PORT_IN_USE = -8,
			// E.g. passive open on a port that someone else
			//   is already connected to.
	PROTOCOL_IN_USE = -9,
			// For Ethernet, specifies that someone else wants
			//   packets with that ethernet type/protocol field
	ILLEGAL_PORT = -10,
			// E.g. sending to a port to which you're not
			//   connected.
	ILLEGAL_OPERATION = -11,
			// For example, trying to connectFromBelow to a
			//   conduit that has no bottom.
	TOO_MANY_CONNECTIONS = -12,
			// I.e. no more space in the tables.
	BUFFER_FULL = -13,
	CHECKSUM_ERROR = -14,
	CONNECTION_CLOSING = -15,
			// TCP error
	CONNECTION_RESET = -16,
			// I.e. refused/aborted by foreign host
	CANNOT_SEND = -17,
			// For instance, if the window is too small to send, or
			//   some other resource is unavailable. It means the
			//   message has _not_ been accepted, so responsibility
			//   for re-trying to send or deleting the thing is
			//   left with the sender.
	CANNOT_DELIVER = -18,
			// For instance, if there is no conduit connected to
			//   this one, insertFromBelow could return this code.
	CANNOT_CONNECT = -19,
			// Conduit unable to connect for unspecified reason
	INVALID_PROTOCOL = -20,
			// Conduit cannot connect to Conduit of that type
	INVALID_MESSAGE_TYPE = -21,
			// Conduit cannot accept messages of that type
			//   (protocol number).
	INVALID_ENCAPSULATION = -22,
			// UDP/TCP message does not have IP message as holder
	NOT_TRANSMITTED = -23,
			// Conduit of Buffer (or whatever) accepted the
			//   message, but was unable to send it out yet
			//   (for example, if the network isn't quite
			//   initialized yet, or a connection is still in
			//   the process of being initialized) and will keep
			//   trying until it is successful. Informational.
	INCOMPLETE_ADDRESS = -24,
			// Message missing (some of) its address, and is
			//   thus not deliverable.
	NOT_FOUND = -25,
			// MachineConfiguration::lookup() uses this
	END_OF_THE_WORLD = -26
			// Some extraordinarily serious error has
			//   occured. Should never be seen.
};
	
#endif""", defines=defines)

