#!/usr/bin/python

# generate IDL file from C++ header files.
# Enzhou Wang, CS 523, UIUC, Spring, 2006

import os, glob, re

#state: a list of strings since python don't have a native enum

def is_type_keyword(symbol):
    symbols = ["unsigned", "int", "string", "void", "EXCODE", "char", "double", "*", "**"]
    if symbols.count(symbol) > 0:
        return True
    else:
        return False

class CppHeader:
    """
    parse a C++ header file to a list holding all the needed info.
    """
    state = ["Global"]
    classList = [] # ((name, method, method, ...), ...)
    classLists = [] 
    inheritedList = []
    inheritedLists = [] #((name1, name2, ...), ...)
    attribList = [] #[isAbstract, ...]
    methodStr = ""
    filepath = ""
    debug = False
    isPublic = False
    isAbstract = False
    def cur_state(self):
        return self.state[-1]

    def init(self):
        self.state = ["Global"]
        del self.classLists[:]
        del self.classList[:]
        del self.inheritedList[:]
        del self.inheritedLists[:]
        isPublic = False
        isAbstract = False
        
    def parse(self, filepath):
        self.init()
        self.filepath = filepath
        fin = open(filepath)
        for line in fin.readlines():
            self.proc_line(line)
        fin.close()

    def check_comment(self, line):
        tokens = []
        lst = line.split()
        if (line.find("class Process : ") >= 0):
            self.debug = True
#            print line
#            print self.state
        #print lst
        inComment = False
        if (self.cur_state() == "Comment"):
            inComment = True
        for item in lst:
            if (item.startswith("//")):
                return (False, tokens)
            elif (item.startswith("/*")):
                self.state.append("Comment")
            elif (item.endswith("/*")):
                curItem = item[0:-2]
                if (not curItem == ""):
                    tokens.append(curItem)
                self.state.append("Comment")
            elif (item.endswith("*/")):
                self.state.pop()
            elif (not self.cur_state() == "Comment"):
                tokens.append(item)
        #print tokens
        return (inComment, tokens)
            
    def proc_line(self, line):
        (inComment, tokens) = self.check_comment(line)
        if (not inComment):
            access = False
            accessChange = True
            if (re.search(r"\bpublic\s*:", line)):
                access = True
            elif (re.search(r"\bprivate\s*:", line)):
                access = False
            elif (re.search(r"\bprotected\s*:", line)):
                access = False
            else:
                accessChange = False
            if ((not (self.cur_state() == "InnerClass")) and accessChange):
                self.isPublic = access
            self.proc_tokens(tokens)

    def proc_tokens(self, tokens):
        counter = 0
        #if (self.debug):
        #    print self.state, tokens
        #print tokens
        for item in tokens:
            ++counter
            if (item == "class"): # ignore inner class
                if (self.state.count("Class") == 0):
                    self.state.append("Class")
                else:
                    lineStr = " ".join(tokens)
                    lineStr.strip()
                    if (not lineStr.endswith(";")):
                        self.state.append("InnerClass");
            elif (self.cur_state() == "Class"):
                if (item.endswith(";")):
                    #self.init()
                    return #ignore
                else:
                    #print self.state
                    self.classList.append(item)
                    self.state.append("ClassName")
            elif (self.cur_state() == "ClassName"):
                if (item == ":"):
                    self.state.append("ClassInheritance")
                elif (item.endswith(";")):
                    self.init()
                elif (item == "{"):
                    self.state.append("InClass")
                    self.isPublic = False
                    self.isAbstract = False
            elif (self.cur_state() == "ClassInheritance"):
                if (item == "{"):
                    #print tokens
                    self.state.append("InClass")
                elif (not item  == "public" and (not item == "private")):
                    self.inheritedList.append(item)
            elif (self.cur_state() == "ClassName" or self.cur_state() == "ClassInheritance"):
                self.state.append("InClass")
            elif (self.cur_state() == "InClass" and item == "proxiable"):
                #print tokens
                self.state.append("Method")
            elif (self.cur_state() == "Method"):
                lineStr = " ".join(tokens)
                #print lineStr
                codeStart = lineStr.find("{")
                if (codeStart >= 0):
                    lineStr = lineStr[:codeStart]
                lineStr.strip()
                self.methodStr = lineStr
                if (lineStr.endswith(";") or lineStr.endswith(")")):
                    method = self.convert_method(self.methodStr)
                    self.classList.append(method)
                    self.state.pop()
                    self.methodStr = ""
                else:
                    self.state.append("MethodCont")
                return
            elif (self.cur_state() == "MethodCont"):
                paramStr = " ".join(tokens)
                self.methodStr += " " + paramStr
                #print self.methodStr
                if (paramStr.endswith(";")):
                    method = self.convert_method(self.methodStr)
                    self.classList.append(method)
                    self.methodStr = ""
                    self.state.pop()
                    self.state.pop()
                return
            elif (self.cur_state() == "InnerClass" and item == "};"):
                #print tokens
                self.state.pop()
            elif (self.cur_state() == "InClass" and item == "};"):
                self.classLists.append(self.classList)
                self.inheritedLists.append(self.inheritedList)
                self.attribList.append(self.isAbstract)
                self.classList = []
                self.inheritedList = []
                del self.state[1:] # ["Global"]


    def is_identifier(self, s):
        if (not s.find(";") >= 0 and not s.find(")") >= 0):
            return True
        else:
            return False

    def get_paramList(self, paramsStr):
        argCount = 1
        paramsStr = paramsStr.replace("(", "")
        params = paramsStr.split(",")
        paramList = []
        #print params
        for item in params:
            #print item
            item = item.replace(")", "")
            item = item.replace(";", "")
            #ignore (default parameter)
            param_default = item.split("=")
            item = param_default[0]
            type_name = item.split()
            param_type = ""
            paramName = "arg" + str(argCount)
            argCount += 1
            if (len(type_name) == 0):
                continue
            elif (len(type_name) == 1):
                paramPair = [type_name[0], paramName]
                paramList.append(paramPair)
                continue
            elif (not is_type_keyword(type_name[-1])):
                paramName = type_name.pop()
                starIdx = paramName.rfind("*")
                paramType = " ".join(type_name)
                if (starIdx >= 0):
                    paramType += paramName[0:starIdx + 1]
                    paramName = paramName[starIdx + 1:]
                paramPair = [paramType, paramName]
                paramList.append(paramPair)
                continue
            else:
                paramPair = [item, paramName]
                paramList.append(paramPair)
                continue
        return paramList

    def convert_method(self, methodStr):
        #print "methodStr: " , methodStr
        if (re.search("=\s*0\s*;\s*$", methodStr)):
            self.isAbstract = True
        lst = []
        method_params = methodStr.split("(")
        #print "#".join(method_params)
        method = method_params[0]
        name_list = method.split()
        return_type = ""
        methodName = "NullMethodName"
        for item in name_list[0:-1]:
            if (item == "proxiable"):
                continue
            elif (item == "virtual"):
                continue
            else:
                return_type += item + " "
        methodName = name_list[-1]
        lst.append(methodName)
        lst.append(return_type.strip())
        paramList = []
        if (len(method_params) == 1):
            lst.append(paramList)
            return lst
        paramsStr = method_params[1]
        paramList = self.get_paramList(paramsStr)
        #print paramList
        lst.append(paramList)
        lst.append(self.isPublic)
        return lst

def translate_type(type_in):
    if (type_in.endswith("Ref")):
        return type_in[:-3]
    elif (type_in == "unsigned"):
        return "uint"
    type_out = re.sub("unsigned\s+int", "uint", type_in)
    return type_out

def HeaderToInterface(header):
    """
    Output: [{interfaceName -> [[inherited interface names], pre, {methods}, post,}, ...]
    """
    #print header.classList
    content = ""
    outputDict = {}
    for i in range(0, len(header.classLists)):
        classList = header.classLists[i]
        #print classList
        inheritedList = header.inheritedLists[i]
        isAbstract = header.attribList[i]
        className = classList[0]
        if (className == "MemoryObject" or className == "PersistentMemoryObject" \
            or className == "RecordStream"):
            isAbstract = True
        if (className == "MultiRecMessContainer"):
            continue
        elif (className == "Object"):
            continue #omit object interface
            className = "CObject"
        elif (className.endswith(":")):
            className = className[:-1]
        elif className.find("Proxy") == 0:
            continue #do not include proxy interface
        interfaceContent = []
        output_inheritedList = []
        pre = "// " + header.filepath + "\n\n"
        pre += "interface " + className
        indent = "  "
        #print output
        # support inheritance except ProxiableObject
        if (len(inheritedList) > 0):
            pre += " : "
        for item in inheritedList:
            #if (not item == "ProxiableObject"):
                if (item == "Object"):
                    continue
            #        item = "CObject"
                output_inheritedList.append(item)
                pre += item + ", "
        if pre.endswith(", "):
            pre = pre[:-2]
        elif pre.endswith(" : "): # ProxiableObject is the only base class
            pre = pre[:-3]
        pre += "\n{\n"
        methods = ""
        methodDict = {}
        methodPostfixDict = {} # if there are several methods with the same name, postfix "1", "2", ...
        methodList = []
        hasConstructor = False
        for i in range(1, len(classList)):
            methodStr = ""
            method = classList[i]
            isPublic = method[3]
            if (not isPublic):
                continue
            #print method
            #skip constructor and destructor
            tokens = {}
            returnType = ""
            methodName = method[0]
            methodName = methodName.replace("*", "")
            excludeMethod = False
            if (methodName == "constructor"):
                excludeMethod = True
            if (method[0] == classList[0]):
                if (hasConstructor or isAbstract):
                    continue
                methodName = className + "constructor"
                if className == "Class":
                    methodName += "1"
                returnType = "void"
                hasConstructor = True
            elif (method[0] == "~" + classList[0]):
                methodName = className + "destructor"
                returnType = "void"
            if (not method[1] == ""):
                returnType = method[1]
                returnType = translate_type(returnType)
                returnType = re.sub("char\s*\*", "string", returnType)
                returnType = returnType.replace("*", "")
                returnType = returnType.replace("&", "")
                if (returnType.find("Proxy") == 0):
                    excludeMethod = True
            methodStr = returnType + " "
            if methodName.lower() == returnType.lower():
                #print methodName
                #print returnType
                methodName += "1"
            methodStr += methodName + " ("
            tokens[methodName.lower()] = 1
            #print methodStr
            paramList = method[2]
            #if (methodName == "execFromObjectFile"):
            #    print paramList
            param_i = 1
            
            for item in paramList:
                dir_attr = "in"
                if (len(item) < 2):
                    print "classList: ", classList
                    assert(False)
                #print item
                paramType = item[0]
                paramName = item[1]
                starInName = paramName.rfind("*")
                if (starInName >= 0):
                    paramName = paramName[starInName + 1:]
                    paramType += paramName[:starInName + 1]
                if (re.search("Ref\s*&", paramType)):
                    excludeMethod = True
                paramType = re.sub("char\s*\*", "string", paramType)
                paramType = re.sub("void\s*\*", "any", paramType)
                two_star = re.search("\*\s*\*", paramType)
                realType = re.sub("\s*\*.*", "", paramType)
		realType = realType.strip()
                if (realType.find("Proxy") == 0):
                    excludeMethod = True
                if two_star:
                    dir_attr = "out"
                elif paramType.find("*") > 0 or paramName.find("[]") > 0:
                    if (is_type_keyword(realType)):
                        dir_attr = "out"
                elif paramType.find("&") > 0:
                    dir_attr = "inout"
                    realType = realType.replace("&", "")
                realType = realType.strip() 
                realType = translate_type(realType)
                paramName = item[1]
                tokens[realType.lower()] = 1
                if (tokens.has_key(paramName.lower())):
                    paramName += str(param_i)
                    param_i += 1
                    tokens[paramName.lower()] = 1
                #if (paramName.lower() == "nameserver"):
                #    print tokens
                paramName = paramName.replace("[]", "") 
                paramStr = dir_attr + " " + realType + " " + paramName + ", "
                paramStr = paramStr.replace(" const ", " ")
                methodStr += paramStr
            if (methodStr.endswith(", ")):
                methodStr = methodStr[:-2]
            #print methodStr
            if (methodDict.has_key(methodName)):
                method_i = 1
                if (methodPostfixDict.has_key(methodName)):
                    method_i = methodPostfixDict[methodName]
                modifiedMethodName = methodName + str(method_i)
                methodStr = methodStr.replace(" " + methodName + " ", " " + modifiedMethodName + " ") # assume only one duplicate
                methodPostfixDict[methodName] = method_i + 1
                methodName = modifiedMethodName
            if (not excludeMethod):
                methodDict[methodName] = methodStr + ");"
                methodList.append(methodName)
        if (len(methodDict) == 0):
            continue #do not add empty interface
        else:
            post = "};\n"
            interfaceContent.append(output_inheritedList)
            interfaceContent.append(methodList)
            interfaceContent.append(pre)
            interfaceContent.append(methodDict)
            interfaceContent.append(post)
            outputDict[className] = interfaceContent
    return outputDict

class HeaderVisitor:
    """
    """
    def __init__(self):
        self.header = CppHeader()
        self.output = ""
        self.outputList = []
        self.total = 0
        self.interfaceDict = {}
        self.processed = {"Boo" : 1}
        self.interfaceList = []
        self.customCode = """
  interface TheInterface{
  	#pragma nonproxy
	FileSystemInterface theFileSystemInterface();
	SystemInterface theSystemInterface();
	NameServer theNameServer();
	OutputStream theOutputStream();
	InputStream theInputStream();
  };
  
  //../../Includes/Common/UNIXFile.h
  
  interface UNIXFileInfo{
  	#pragma nonproxy
  };
  
  //../../Includes/Common/UnixTime.h
  
  interface UnixTime{
  	#pragma nonproxy
  };
  
  //../../Includes/SystemTime.h
  
  interface SystemTime{
  	#pragma nonproxy
  };
  
  interface ProvidedVersionInstaller{
  	#pragma nonproxy
  };
  
  interface RequiredVersionInstaller{
  	#pragma nonproxy
  };
  
  interface ProcessResourceUsage{
  	#pragma nonproxy
  };
  
  interface UserSessionInfo{
  	#pragma nonproxy
  };
  //../../Includes/Memory/Domain.h
  
  interface Domain{
  
  };
  
  //../../Includes/IODevices/Disk.h
  
  interface Disk{
  
  };  
"""
    def gen_text(self):
        print "start generating IDL..."
        processed = {}
        #print self.interfaceDict
        for name, content in self.interfaceDict.iteritems():
            self.proc_output(name, content)
        #print self.outputList
        self.output = "\n".join(self.outputList)
        print "end generating IDL."
        
    def visit(self, path):
        for fpath in glob.glob(path + "/*.h"):
            self.visitfile(fpath)
        self.gen_text()
        
    def recursive_visit(self, path):
        #print path
        os.path.walk(path, self.visitor, None)
        self.gen_text()
        
    def visitor(self, data, dirName, filesInDir):
        #print dirName
        ignoreList = [".svn", "MachineDependent", "ProcessorDependent", "PCi586", "Instrument", "Networks", "Distributed"]
        for ignoreItem in ignoreList:
            if (filesInDir.count(ignoreItem) > 0):
                filesInDir.remove(ignoreItem)
        
        for fname in filesInDir:
            #print fname
            fpath = os.path.join(dirName, fname)
            if (not os.path.isdir(fpath) and not fpath.find(".svn") >= 0 and (fpath[len(fpath)-1] == 'h')):
                self.visitfile(fpath)
                
    def proc_output(self, interfaceName, inputList):
        idlText = ""
        returnList = []
        inheritedMethodDict = {}
        returnMethodDict = {}
        #print inputList
        #debug = False
        #if (interfaceName == "CharacterInputStream"):
        #    debug = True
        if (not self.processed.has_key(interfaceName)):
            inheritedList = inputList[0]
            #if (debug):
            #    print inheritedList
            for interface in inheritedList:
                if (not self.interfaceDict.has_key(interface)):
                    if (not self.processed.has_key(interface)):
                        dummyText = "// dummy interface TODO: need real interface\n"
                        dummyText += "interface " + interface + " {};\n";
                        self.outputList.append(dummyText)
                        self.processed[interface] = {}
                else:
                    #if (debug):
                    #    print interface
                    dict = self.proc_output(interface, self.interfaceDict[interface])
                    for key in dict.keys():
                        inheritedMethodDict[key] = 1

            methodList = inputList[1]
            idlText = inputList[2]
            methodDict = inputList[3]
            indent = "  "
            returnMethodDict = inheritedMethodDict
            for name in methodList:
                text = methodDict[name]
                if (name == "ApplicationProcessconstructor"):
                    text = "void ApplicationProcessconstructor (in PFV arg3, in Domain arg4, in int arg1, in int arg2, in StackSize arg5);"
                elif (name == "execFromObjectFile" and interfaceName == "SystemInterface"):
                    continue
                if (inheritedMethodDict.has_key(name)):
                    orig = text
                    text = re.sub(r"\b" + name + r"\b",  interfaceName + name, text)
                    if (text == orig):
                        print "before: ", text
                        print name
                        print interfaceName + name
                        print "after: ", text
                        assert(False)
                idlText += indent + text + "\n"
                returnMethodDict[name] = 1
                
            idlText += inputList[4]
            self.outputList.append(idlText)
            self.total += 1
            self.interfaceList.append(interfaceName)
            self.processed[interfaceName] = returnMethodDict
            if (interfaceName == "ProxiableObject"):
                self.outputList.append(self.customCode)
            return returnMethodDict
        else:
            return self.processed[interfaceName]
    
    def visitfile(self, filepath):
        #print "start processing...", filepath
        self.header.parse(filepath)
        #outputList = HeaderToInterface(self.header)
        interfaceDict = HeaderToInterface(self.header)
        for name, content in interfaceDict.iteritems():
            self.interfaceDict[name] = content
        #print "end processing", filepath
        
def proc_dir():
    print "start processing..."
    indent = "  "
    output = "module Choices {\n\n"
    typedefLst = ["long int", "unsigned long uint", \
                   "long PFV", "long Offset", "long PConstructor", \
                   "long POFPO", "long ProtectionLevel", \
                  "long ProcessState", "long ClassApplicationFunction", \
                  "long StackSize", "long Return_Code", "long formatArgs"]
    for typedef in typedefLst:
        output += indent + "typedef " + typedef + ";\n"
    output += "\n"
    
    visitor = HeaderVisitor()
    fo = open('choices.idl', 'w')
    visitor.recursive_visit(r"../../Includes")

    forwardDecl = ""
    for name in visitor.interfaceList:
        forwardDecl += "  interface " + name + ";\n"
    forwardDecl += "  interface OutputStream;\n"
    output += forwardDecl + "\n"
    
    output += visitor.output
    output += "};"
    #print output
    fo.write(output)
    fo.write("\n//Total: " + str(visitor.total) + " interfaces\n")
    fo.close();
    
if __name__ == '__main__':
    proc_dir()
    """
    header = CppHeader()
    header.parse("../../Includes/FileSystems/PersistentMemoryObject.h")
    output = HeaderToInterface(header)
    print output
    """
    print "end processing."
