#!/usr/bin/env perl

# OMAP backtrace v 0.1
# Script to output symbolic information from a stack backtrace
# Author: Ellick Chan (emchan@uiuc.edu)

use strict;

my $gdb = `which arm-none-eabi-gdb` || "/projects/choices/omap/codesourcery/bin/arm-none-eabi-gdb";
chomp $gdb;
my @tracedata = ();
my $i;
my $line;
my @tracecommands;

print "Enter backtrace data:\nPress ctrl-d when done\n";

while( $line = <> )
{
	# Find lines that look like a stack trace
	if($line =~ /pc: (0x[a-f0-9]+) lr: (0x[a-f0-9]+)/) {
		push(@tracedata, hex("$2")-4);
	}
	if($line =~ /pc : (0x[a-f0-9]+) lr :.*$/) {
		push(@tracedata, "$1");
	}
}

foreach $i(@tracedata)
{
	# Build up a gdb query list
	
	push(@tracecommands, "info line *".$i);
}
my $random = "gdb." . int (rand (999));

open(FILEHANDLE,">$random");
foreach $i(@tracecommands)
{
	print FILEHANDLE "$i\n";
}
print FILEHANDLE "quit\n";
close(FILEHANDLE);
# Do gdb query
system "$gdb ./Choices --quiet -x $random > ${random}b 2> /dev/null";
unlink ("$random");

# Parse gdb data
open(FILEHANDLE,"${random}b");
my @lines=<FILEHANDLE>;
close(FILEHANDLE);
unlink ("${random}b");

print "\n\n\nTrace data:\n";

my @functions = ();

my @reallines = ();
my $joined;
#my $linepattern = 'Line ([0-9]+) of (\".*\") starts at address (0x[a-f0-9]+) \<(_*.*)_*(\+[0-9]+)*\>.*and ends at (0x[0-9a-f]+) .*';
my $linepattern = '^Line\s+([0-9]+)\s+of\s+(\".*\")\s+starts\s+at\s+address\s+(0x[a-f0-9]+)\s+\<(_*.*)_*(\+[0-9]+)*\>\s+and\s+ends\s+at\s+(0x[0-9a-f]+).*$';

# Join multi-lines created by gdb
foreach $i(@lines)
{
	if($i =~ m/($linepattern)/ )
	{
		# Trivial case, no joining necessary
		push(@reallines, $i);
		$joined = "";
	} elsif ( $i =~ /.*ends\sat.*$/ ) {
		# End of line
		chomp $i;
		$joined = $joined . $i;
		push(@reallines, $joined);
		$joined = "";
	} else {
		# Middle of line
		chomp $i;
		$joined = $joined . $i;
	}
}

my $addr;
@tracedata =reverse @tracedata;
# Reformat gdb data
foreach $i(@reallines)
{
	# Format of info line *addr
	# Line 724 of "../../../Kernel/Kernel.cc" starts at address 0x1000aed4 <_ZN6Kernel17setupProcessEntryEPS_+12> and ends at 0x1000aedc 
	# Line 52 of "../../../Kernel/ObjectProxyVtable.cc" starts at address 0x100d8cd8 <ObjectProxy+12> and ends at 0x100d8cec <ObjectProxy+32>.
	if ( $i =~ /^Line\s+([0-9]+)\s+of\s+\"(.*)\"\s+starts\s+at\s+address\s+(0x[a-f0-9]+)\s+\<(_*.*)_*(\+[0-9]+)*\>\s+and\s+ends\s+at\s+(0x[0-9a-f]+).*$/ ) 
	{	
		# Demangle the names
		system ( "echo -e $4 | arm-none-eabi-c++filt" );
		
		$addr = pop @tracedata;
		# Print file:line information
		print "$2:$1 pc=$addr\n\n"
		#system ( "echo -e $2:$1 \t $4 +$5 \t addresses $3 - $6 $7 | c++filt" );
	} else {
		print "REJECTED LINE: ****** $i\n\n"
	}
}
