#!/usr/bin/perl

use Getopt::Long;

my $op_help;
my $op_all;
my $op_v;
my $op_hfile;

my $result = GetOptions( "help",\$op_help,
			 "hfile",\$op_hfile,
			 "v",\$op_v,
			 "all" ,\$op_all   );


exit &main();


sub processParams {
    my $badparams = 0;

    if ( @ARGV < 1 ){
	print "Provide at least one symbol file!\n";
	$badparams = 1;
    }

    if ( $badparams ) {
	print "usage:\n readdwarf.pl <options> <files>\n";
	exit;
    }
}


sub main {
    processParams();

    my @flist;

    @flist = @ARGV;

    my $file = shift (@ARGV);

    if ($op_v) { print "Opening $file.\n"; }

    my @debug = getDebugSectionsList ( $file );  

    if ($op_v and 0) {

	while(@debug) {
	    my $o = shift(@debug);
	    
	    if( $o =~ /^\.L.+/ ) {
		print "$o\n";
	    } elsif ( $o =~ /\.file/) {
		my $oo = shift(@debug);
		my $ooo = shift(@debug);
		print "$o $oo $ooo\n";
	    } else {
		my $oo = shift(@debug);
		print "$o\t$oo\n";
	    }
	}
    }

    
    my %dwarf_raw =  getRawDwarfInfo( @debug ) ;

    my $count=1;

    my %dwarf_annotated = parseRawDwarfInfo ( \%dwarf_raw, \$count ) ;

    if ( $op_v ) {
	printDwarf ( \%dwarf_annotated, \%dwarf_raw , 0);
    }

    if ( $op_hfile ) {
	dump_header_declarations ( \%dwarf_annotated, \%dwarf_raw  );
    }
}

sub printDwarf {
    my $dwarf = shift @_;

    my $dwarf_raw = shift @_;

    my $tab = shift @_;

    my $dak;

    my %inv = invDwarfTags();
    my %invAt = invDwarfAts();
    
    print "\n\n";
    tabn($tab);
    print "printDwarf $dwarf\n";

    foreach $dak (sort numerically keys %$dwarf  ) {
	
	my $child;
	
	tabn($tab);
	print ".debug_info($dak):\t";

	my $abbrev = $dwarf->{$dak}{'abbrev'};
	my $debug_abbr_tag = $dwarf_raw->{'.debug_abbrev'}{'rec'}{$abbrev}{'tag'};
	
	tabn($tab);
	print "Abbr($abbrev)=TAG($debug_abbr_tag):$inv{hex($debug_abbr_tag)} ";


	my $e;

	foreach $e (sort keys %{ $dwarf_raw->{'.debug_info'}{'rec'}{$dak}{'attr'}  } ) {
	    print "$e\{ ";
	    my $ee;

	    foreach $ee ( @{$dwarf_raw->{'.debug_info'}{'rec'}{$dak}{'attr'}{$e}} ) {
		print "$ee ";
	    }
	    print "\} ";
	}



	print "\n";

	if( exists( $dwarf->{$dak}{'children'} ) ) {
	    printDwarf( $dwarf->{$dak}{'children'},$dwarf_raw, $tab+1 );
	    print "\n";
	}
    }
}

sub dump_header_declarations {
    my $dwarf = shift @_;
    my $dwarf_raw = shift @_;


    my %declarations = (
			TaskStateSegment => {
			    data => {
				esp0 => 'ESP0__TASKSTATESEGMENT',
			    },

			},

			LretFrame => {
			    data => {
				userStackPointer => 'USERSTACKPOINTER__LRETFRAME',
			    },
			    
			    
			    size => {
				this => 'SZLRETFRAME',
			    },
			},

			UserProxyState => {
			    data => {
				edi  => 'EDI__USERPROXYSTATE',
				esi  => 'ESI__USERPROXYSTATE',
				ebx  => 'EBX__USERPROXYSTATE',
				ebp  => 'EBP__USERPROXYSTATE',
			    },

			},

			ProxyReturnFlags => {
			    size => {
				this => 'SZPROXYRETURNFLAGS',

			    },

			    data => {
				returnType => 'RETURNTYPE__PROXYRETURNFLAGS',
				calledMethod => 'CALLEDMETHOD__PROXYRETURNFLAGS',
			    },

			},
			
			i586GenericContext => {
			    data => {
				startKernelStack => 'STARTKERNELSTACK__I586GENERICCONTEXT',
				stackPointer => 'STACKPOINTER__I586GENERICCONTEXT',
				edi  => 'EDI__I586GENERICCONTEXT',
				esi  => 'ESI__I586GENERICCONTEXT',
				ebx  => 'EBX__I586GENERICCONTEXT',
				ebp  => 'EBP__I586GENERICCONTEXT',
				pc  => 'PC__I586GENERICCONTEXT',
			    },
			    
			    function => {
				checkpoint => 'GENERICCONTEXT_VOID_CHECKPOINT_NAME',
				basicRestore => 'GENERICCONTEXT_PROCESS_BASICRESTORE_NAME',
			    },				    
			},
			
			i586ApplicationContext => {
			    
			    function => {
				checkpoint => 'APPLICATIONCONTEXT_VOID_CHECKPOINT_NAME',
				basicRestore => 'APPLICATIONCONTEXT_PROCESS_BASICRESTORE_NAME',
				restoreFromInitialContext => 'APPLICATIONCONTEXT_PROCESS_RESTOREFROMINITIALCONTEXT_NAME',				
				initialRestoreAssist => 'APPLICATIONCONTEXT_PROCESS_INITIALRESTOREASSIST_NAME',
			    },				    

			},

			i586SystemContext => {
			    
			    function => {
				restoreFromInitialContext => 'SYSTEMCONTEXT_PROCESS_RESTOREFROMINITIALCONTEXT_NAME',				
				initialRestoreAssist => 'SYSTEMCONTEXT_PROCESS_INITIALRESTOREASSIST_NAME',
			    },				    

			},
			
			i586CPU => {
			    function => {
				interruptAssist => 'CPU_VOID_INTERRUPT_ASSIST_NAME',
				hardwareInterruptAssist => 'CPU_STACKFRAME_HARDWAREINTERRUPTASSIST_NAME',
				ProxyAssist => 'CPU_VOID_PROXY_ASSIST_NAME',
				ProxyReturnAssist => 'CPU_VOID_PROXY_RETURN_ASSIST_NAME',
			    },
			}

			);

    print "// readdwarf.pl Automatically generated header file.            \n";
    print "//                                                              \n";
    print "// readdwarf.pl was written by James Tuck (jtuck\@cs.uiuc.edu)   \n";
    print "// James will gladly accept patches and updates for this utility\n";
    print "//                                                              \n";
    print "//                                                              \n";
    print "// i586asmdefs.h is part of Choices port, PCi586                \n";
    print "//     Authors: Mark Brandyberry, Naveen Neelkantam, Nick Cohen,\n";
    print "//              and James Tuck                                  \n";
    print "//                                                              \n";
    print "//                                                              \n";
    
    print "\n\n";

    my $class;
    foreach $class (keys %declarations) {

	my $data;
	my $function;
	my $size;

	print "\n\n//Entries for $class:\n";

	my $class_entry = getTagAndName ($dwarf,$dwarf_raw,'DW_TAG_structure_type',$class);

	if ($class_entry == 0) {
	    print "Failed to get $class!\n";
	    exit;
	}

	print "\n";

	if ( exists ( $declarations{$class}{size} ) ) {
	    	    
	    my $s;
	    $s = $dwarf_raw->{'.debug_info'}{'rec'}{$class_entry->{'index'}}{'attr'}{'DW_AT_byte_size'}[0];

	    print "// Size of $class \n";
	    print "#define $declarations{$class}{size}{this} \t $s\n";

	}

	foreach $data (keys %{ $declarations{$class}{data} }) {

	    my $data_entry = getTagAndName ($class_entry->{'children'},
					    $dwarf_raw, 'DW_TAG_member',
					    $data);
	    
	    print "// $class\:\:$data location \n";
	    
	    my $s;
	    $s = $dwarf_raw->{'.debug_info'}{'rec'}{$data_entry->{'index'}}{'attr'}{'DW_AT_data_member_location'}[1];

	    for ($s) { s/\"//; }

	    print "#define $declarations{$class}{data}{$data} \t $s\n";

	    
	}

	foreach $function (keys %{ $declarations{$class}{function} }) {
	    my $func_entry = getTagAndName ($class_entry->{'children'},
					    $dwarf_raw, 'DW_TAG_subprogram',
					    $function);
	    
	    print "// $class\:\:$function name \n";

	    my $s;
	    $s = $dwarf_raw->{'.debug_info'}{'rec'}{$func_entry->{'index'}}{'attr'}{'DW_AT_mangled_name'}[0];
	    for ($s) { s/\"//; s/\"//;}
	    print "#define $declarations{$class}{function}{$function} \t $s\n";
	}

    }


}

sub getTagAndName {
    my $dwarf = shift @_;
    my $dwarf_raw = shift @_;
    my $tag = shift @_;
    my $name = shift @_;

    my $dak;

    foreach $dak (sort numerically keys %$dwarf  ) {	
	my $child;

	#print "$dak:$dwarf->{$dak}{'tag'}\n";
	
	if( $dwarf->{$dak}{'tag'} eq $tag ) {

	    #print "EQUAL!!!!!!!! $dwarf_raw->{'.debug_info'}{'rec'}{$dak}{'attr'}{'DW_AT_name'} ";


	    if( exists( $dwarf_raw->{'.debug_info'}{'rec'}{$dak}{'attr'}{'DW_AT_name'} ) ) {
		my $s = $dwarf_raw->{'.debug_info'}{'rec'}{$dak}{'attr'}{'DW_AT_name'}[0];
		my $ss = "$name";

		if( $dwarf_raw->{'.debug_info'}{'rec'}{$dak}{'attr'}{'DW_AT_name'}[0] =~ /\"$name\"/ ) {

		    return $dwarf->{$dak};
		}
		
	    }
	    
	    #print "\n";
	}
	
	if( exists( $dwarf->{$dak}{'children'} ) ) {
	    my $dref = getTagAndName($dwarf->{$dak}{'children'},$dwarf_raw,$tag,$name);
	    
	    if( $dref != 0 ) {
		return $dref;
	    }
	}
    }

    return 0;
}


sub tabn {
    my $n = shift @_;

    my $i;

    for($i=0;$i<$n;$i++) {
	print "   ";
    }
}

sub getRawDwarfInfo {
    my @debug = @_;

    my %section;

    if(@debug) {
	getDebugAbbrevSection ( \%section, @debug );
	getDebugStrSection( \%section, @debug );
	
	getDebugInfoSection( \%section, @debug );
    }
    
    return %section;
}

sub numerically  { $a <=> $b }

sub parseRawDwarfInfo {
    my $rd = shift @_;
    my $k = shift @_;
    
    my %dwarf;
    
    my $limit = $rd->{'.debug_info'}{'entries'};

    my $key = $rd->{'.debug_info'}{'rec'}{$$k}{'key'};
    
    my $debug_abbrev = $rd->{'.debug_abbrev'}{'rec'}{$key};
    
    my %invTag = invDwarfTags();

    my $z=0;

    while ( $$k < $limit and hex($key) != 0 ) {

	$dwarf{$$k}{'index'} = $$k;
	$dwarf{$$k}{'abbrev'} = $key;
	$dwarf{$$k}{'tag'} = $invTag{hex($rd->{'.debug_abbrev'}{'rec'}{$key}{'tag'})};

	if( hex( $rd->{'.debug_abbrev'}{'rec'}{$key}{'children'}) == 1 ) {

	    my $tmp = $$k;

	    $$k++;

	    $dwarf{$tmp}{'children'} = { parseRawDwarfInfo ($rd, $k) };

	    #my $dak;
	    #foreach $dak ( keys %{ $dwarf{$$k}{'children'} } ) {
	    #	print "$dwarf{$$k}{'children'}{$dak} ";
	    #}
	} else {
	    $$k++;
	}

	$key = $rd->{'.debug_info'}{'rec'}{$$k}{'key'};
	$debug_abbrev = $rd->{'.debug_abbrev'}{'rec'}{$key};
    }


    if($$k < $limit) {
	$$k++;
    }

    return %dwarf;
}

sub getSection {

    my $name = shift @_;
    my %record = {};

    if ( $name =~ /\.debug_abbrev/ ) {
	getDebugAbbrevSection($name,@_);
    } elsif ( $name =~ /\.debug_info/ ) {
	getDebugInfoSection($name,@_);
    } elsif ( $name =~ /\.debug_str/ ) {
	print "Error: Should not see this twice!\n";
	exit;
    } 
}

sub getDebugInfoSection {
    my $section = shift @_;

    my $name = '.debug_info';
    
    my @debug=();
    push @debug,@_;

    my %inv = invDwarfTags();
    my %invA = invDwarfAts();
    my %invF = invDwarfForms();


    my $done=0;
    while(@debug and $done==0) {
	my $d = shift @debug;	
	if ( $d =~ /\.section/ ) {	    
	    my $dd = shift @debug;	    
	    if ( $dd =~ /\.debug_info/ ) {
		if (exists( $section{$dd} ) ) {
		    $done = 1;
		} else {
		    $section{$dd}{'name'} = $dd;		    
		}
	    } 
	}
    }


    my $cnt = 1;
    my $z=0;

    shift @debug;
    $section->{$name}{'entries'} = shift @debug;
    shift @debug;
    $section->{$name}{'version'} = shift @debug;
    shift @debug;
    $section->{$name}{'offset'} = shift @debug;
    shift @debug;
    $section->{$name}{'addr_size'} = shift @debug;
    
    $section->{$name}{'zeros'} = 0;

    while(@debug and debug->[0] !~ /\.section/) {

	shift @debug;
	my $key = shift @debug;

	$section->{$name}{'rec'}{$cnt}{'key'} = $key;
	
	
	if ( hex($key)!= 0 ) {

	    my @attr = @{ $section->{'.debug_abbrev'}{'rec'}{$key}{'attr'} };

	    while( @attr ) {
		my $a = shift @attr;
		my $f = shift @attr;
		$section->{$name}{'rec'}{$cnt}{'attr'}{$a} = [()];

		my @list = getDwarfFormData($section, $f,\@debug);


		push @{ $section->{$name}{'rec'}{$cnt}{'attr'}{$a} }, @list;
	
	    }
	    	    	    
	} else {	
	    $z++;
	    $section->{$name}{'zeros'} = $z;
	}

	$cnt++;
    }

    $section->{$name}{'entries'} = $cnt;
}

sub getDwarfFormData {
    my $section = shift @_;
    my $form = shift @_;
    my $ref_debug = shift @_;

    my %func_hash = (
		       'DW_FORM_addr'   => \&getDwarfAddr, 
		       'DW_FORM_block2' => \&getDwarfBlock,
		       'DW_FORM_block4' => \&getDwarfBlock,
		       'DW_FORM_data2'  => \&getDwarfConst,
		       'DW_FORM_data4'  => \&getDwarfConst,
		       'DW_FORM_data8'  => \&getDwarfConst,
		       'DW_FORM_string' => \&getDwarfString,
		       'DW_FORM_block'  => \&getDwarfBlock,
		       'DW_FORM_block1' => \&getDwarfBlock,
		       'DW_FORM_data1'  => \&getDwarfConst,
		       'DW_FORM_flag'   => \&getDwarfConst,
		       'DW_FORM_sdata'  => \&getDwarfUleb128,
		       'DW_FORM_strp'   => \&getDwarfString,
		       'DW_FORM_udata'  => \&getDwarfUleb128,
		       'DW_FORM_ref_addr' => \&getDwarfAddr, #TODO
		       'DW_FORM_ref1'   => \&getDwarfConst,
		       'DW_FORM_ref2'   => \&getDwarfConst,
		       'DW_FORM_ref4'   => \&getDwarfConst,
		       'DW_FORM_ref8'   => \&getDwarfConst,
		       'DW_FORM_ref_udata' => \&getDwarfUleb128, 
		       'DW_FORM_indirect'  => \&getDwarfIndirect, 
		     );


    #print "calling function $form\n";

    return $func_hash{$form} ($section, $form, $ref_debug);
}

sub getDwarfAddr {
    my $section = shift @_;
    my $form = shift @_;
    my $ref_debug = shift @_;

    my @list=();

    my $addr_size = $section->{$name}{'addr_size'};

    
    my $type = shift @$ref_debug;

    my $size_type = getSizeOf($type);

    push @list, (shift @$ref_debug);

    while( $size_type < $addr_size ) {
	$type = shift @$ref_debug;
	$size_type += getSizeOf($type);
	push @list, (shift @$ref_debug);	
    }

    return @list;
}

sub getDwarfBlock {
    my $section = shift @_;
    my $form = shift @_;
    my $ref_debug = shift @_;

    my @list=();

    my $addr_size = $section->{$name}{'addr_size'};

    shift @$ref_debug;
    my $size = hex(shift @$ref_debug);

    my $i=0;

    

    while($i<$size) {
	my $unit = getSizeOf(shift @$ref_debug);	
	$i += $unit;
	push @list, (shift @$ref_debug);
    }

    return @list;
}

sub getDwarfConst {
    my $section = shift @_;
    my $form = shift @_;
    my $ref_debug = shift @_;

    my @list=();

    my $addr_size = $section->{$name}{'addr_size'};

    my $const_size  = ($form =~ /DW_FORM_.+(\d)/ ) ? $1 : $1 ;  
    
    my $type = shift @$ref_debug;
    my $type_size = getSizeOf($type);
    
    #print "getDwarfConst! $const_size\n";

    push @list, (shift @$ref_debug);

    while( $type_size < $const_size ) {
	$type = shift @$ref_debug;
	$type_size += getSizeOf($type);	
	push @list, (shift @$ref_debug);
    }
    
    return @list;
}


sub getDwarfString {
    my $section = shift @_;
    my $form = shift @_;
    my $ref_debug = shift @_;

    my @list=();

    my $addr_size = $section->{$name}{'addr_size'};

    if( $form =~ /string/ ) {
	shift @$ref_debug;
	my $byte = shift @$ref_debug;
	push @list, ($byte);
	while( hex($byte) != 0 ) {
	    shift @$ref_debug;
	    $byte = shift @$ref_debug;
	    push @list, ($byte);
	}

    } else {
	shift @$ref_debug;
	my $ptr = shift @$ref_debug;
	push @list, ($section->{'.debug_str'}{'rec'}{$ptr});
    }

    return @list;
}

sub getDwarfUleb128 {
   my $section = shift @_;
   my $form = shift @_;
   my $ref_debug = shift @_;
   
   my @list=();
   
   my $addr_size = $section->{$name}{'addr_size'};

   shift @$ref_debug;

   push @list, (shift @$ref_debug);

   return @list;
}

sub getDwarfIndirect {
    my @list=();

    return @list;
}


sub getSizeOf {
    my $s = shift @_;

    if( $s =~ /long/ ) {
	return 4;
    } elsif ( $s =~ /byte/ ) {
	return 1;
    } else {
	return 4;
    }
}

sub getDebugAbbrevSection {
    my $section = shift @_;

    my $name = '.debug_abbrev';
    
    my @debug=();
    push @debug,@_;

    my %inv = invDwarfTags();
    my %invA = invDwarfAts();
    my %invF = invDwarfForms();

    my $done=0;
    while(@debug and $done==0) {
	my $d = shift @debug;	
	if ( $d =~ /\.section/ ) {	    
	    my $dd = shift @debug;	    
	    if ( $dd =~ /\.debug_abbrev/ ) {
		if (exists( $section{$dd} ) ) {
		    $done = 1;
		} else {
		    $section{$dd}{'name'} = $dd;		    
		}
	    } 
	}
    }

    while(@debug and !isFinished(@debug) ) {
	
	shift @debug;

	my $abbrev = shift @debug;

	$section->{$name}{'rec'}{$abbrev}{'attr'} = [ () ];

	shift @debug;
	$section->{$name}{'rec'}{$abbrev}{'tag'} = shift @debug;

	shift @debug;
	$section->{$name}{'rec'}{$abbrev}{'children'} = shift @debug;

	if ($op_v) {
	    print "$abbrev: TAG:$inv{hex($section->{$name}{'rec'}{$abbrev}{'tag'})} C:$section->{$name}{'rec'}{$abbrev}{'children'} ";
	}

	while(!isEndofRecord(@debug)) {	
	    shift @debug;	    
	    my $a = shift @debug;
	    shift @debug;
	    my $f = shift @debug;

	    push @{ $section->{$name}{'rec'}{$abbrev}{'attr'} }, ( $invA{hex($a)} );
	    push @{ $section->{$name}{'rec'}{$abbrev}{'attr'} }, ( $invF{hex($f)} );

	    if ($op_v) {print " hex($a):$invA{hex($a)} $invF{hex($f)}";}
	}
	
	if ($op_v) {print "\n";}
	
	shift @debug;
	shift @debug;	
	shift @debug;
	shift @debug;	
    }

    shift @debug;
    shift @debug;	
}

sub getDebugStrSection {
    my $section = shift @_;

    my $name = '.debug_str';
    
    my @debug=();
    push @debug,@_;

    my %inv = invDwarfTags();
    my %invA = invDwarfAts();
    my %invF = invDwarfForms();

    my $done=0;
    while(@debug and $done==0) {
	my $d = shift @debug;	
	if ( $d =~ /\.section/ ) {	    
	    my $dd = shift @debug;	    
	    if ( $dd =~ /\.debug_str/ ) {
		$section{$dd}{'name'} = $dd;

		$section{$dd}{'2'} = shift @debug;
		$section{$dd}{'3'} = shift @debug;
		$section{$dd}{'4'} = shift @debug;
		
		$done=1;
	    } 
	}
    }

    while(@debug) {

	my $key = shift @debug;

	if( $key =~ /\.ident/ ) {
	    return;
	}

	shift @debug;
	my $string = shift @debug;
	
	$section->{$name}{'rec'}{$key} = $string;

	if ($op_v) { print "$key => $string\n" }
    }
}

sub isFinished {
    return ( $_[0] =~ /\.byte/ and hex($_[1]) == 0 );
}

sub isEndofRecord {
    return ( $_[0] =~ /\.byte/ and hex($_[1]) == 0 and
	$_[2] =~ /\.byte/ and hex($_[3]) == 0 );
}

sub getDebugSectionsList {

    my $f = shift (@_);
    my $debug_section = 0;
    
    my @debug=();

    opendir(DIR,".");

    open(DATA,"<./${f}") or die "Could not open file!\n";

    while (<DATA>) {
	
	chop();

	if ( $_ =~ /\s+\.section/ ) {
	    if ( $_ =~ /\s+\.debug/ ) {
		$debug_section = 1;		
	    } else {
		$debug_section = 0;
	    }
	}

	if ( $debug_section ) {
	 
	    if ( $_ =~ /\.string/ ) {
		push @debug, ( ".string" );

		if ( $_ =~ /(\".*\")/ ) {
		    push @debug, ($1);		
		}

	    } elsif ( $_ =~ /\.ident/ ) {
		push @debug, ( ".ident" );
		
		if ( $_ =~ /(\".*\")/ ) {
		    push @debug, ($1);		
		}

	    } else {

		my @line = split(/:|,|\s/,$_);
		
		push @debug, @line;	    
	    }
	}

    }

    my @ret = ();
    
    my $d;
    foreach $d (@debug) {
	if( $d =~ /\S+/ ) {
	    push @ret,($d);
	}
    } 

    close(DATA);
    closedir(DIR);

    return @ret;
}

sub getDwarfTags {
    my %DWARF_TAGS = (
	'DW_TAG_array_type' => 0x01, 
	'DW_TAG_class_type' => 0x02, 
	'DW_TAG_entry_point'=> 0x03, 
	'DW_TAG_enumeration_type'=> 0x04, 
	'DW_TAG_formal_parameter'=> 0x05 ,
	'DW_TAG_imported_declaration'=> 0x08, 
	'DW_TAG_label' => 0x0a, 
	'DW_TAG_lexical_block'=> 0x0b, 
	'DW_TAG_member' => 0x0d, 
	'DW_TAG_pointer_type'=>  0x0f, 
	'DW_TAG_reference_type'=> 0x10, 
	'DW_TAG_compile_unit' => 0x11, 
	'DW_TAG_string_type' => 0x12, 
	'DW_TAG_structure_type'=> 0x13, 
	'DW_TAG_subroutine_type'=> 0x15, 
	'DW_TAG_typedef'=> 0x16, 
	'DW_TAG_union_type'=> 0x17, 
	'DW_TAG_unspecified_parameters'=> 0x18, 
	'DW_TAG_variant'=> 0x19, 
	'DW_TAG_common_block'=> 0x1a, 
	'DW_TAG_common_inclusion'=> 0x1b, 
	'DW_TAG_inheritance'=> 0x1c, 
	'DW_TAG_inlined_subroutine'=> 0x1d, 
	'DW_TAG_module'=> 0x1e, 
	'DW_TAG_ptr_to_member_type'=> 0x1f, 
	'DW_TAG_set_type'=> 0x20 ,
	'DW_TAG_subrange_type'=> 0x21, 
	'DW_TAG_with_stmt'=> 0x22, 
	'DW_TAG_access_declaration'=> 0x23, 
	'DW_TAG_base_type'=> 0x24, 
	'DW_TAG_catch_block'=> 0x25, 
	'DW_TAG_const_type'=> 0x26, 
	'DW_TAG_constant'=> 0x27, 
	'DW_TAG_enumerator'=> 0x28, 
	'DW_TAG_file_type'=> 0x29,
	'DW_TAG_friend'=> 0x2a, 
	'DW_TAG_namelist'=> 0x2b, 
	'DW_TAG_namelist_item'=> 0x2c, 
	'DW_TAG_packed_type'=> 0x2d ,
	'DW_TAG_subprogram'=> 0x2e ,
	'DW_TAG_template_type_param'=> 0x2f, 
	'DW_TAG_template_value_param'=> 0x30, 
	'DW_TAG_thrown_type'=> 0x31, 
	'DW_TAG_try_block'=> 0x32, 
	'DW_TAG_variant_part'=> 0x33, 
	'DW_TAG_variable'=> 0x34, 
	'DW_TAG_volatile_type'=> 0x35, 
	'DW_TAG_lo_user'=> 0x4080, 
	'DW_TAG_hi_user'=> 0xffff, 
	);

    return %DWARF_TAGS;
}




sub invDwarfTags {
    my %dwarf_tags = getDwarfTags ();

    my %invert;

    my $key;

    foreach $key ( keys %dwarf_tags ) {       
	$invert{ $dwarf_tags{$key}}  = $key;
    }
    
    return %invert;
}

sub getDwarfAts {
    my %DWARF_ATS = (
		     'DW_AT_sibling' => 0x01, 
		     'DW_AT_location' => 0x02, 
		     'DW_AT_name' => 0x03, 
		     'DW_AT_ordering' => 0x09, 
		     'DW_AT_byte_size' => 0x0b, 
		     'DW_AT_bit_offset' => 0x0c,
		     'DW_AT_bit_size' => 0x0d, 
		     'DW_AT_stmt_list' => 0x10,
		     'DW_AT_low_pc' => 0x11, 
		     'DW_AT_high_pc' => 0x12, 
		     'DW_AT_language' => 0x13, 
		     'DW_AT_discr' => 0x15, 
		     'DW_AT_discr_value' => 0x16, 
		     'DW_AT_visibility' => 0x17, 
		     'DW_AT_import' => 0x18, 
		     'DW_AT_string_length' => 0x19, 
		     'DW_AT_common_reference' => 0x1a, 
		     'DW_AT_comp_dir' => 0x1b, 
		     'DW_AT_const_value' => 0x1c, 
		     'DW_AT_containing_type' => 0x1d,
		     'DW_AT_default_value' => 0x1e, 
		     'DW_AT_inline' => 0x20, 
		     'DW_AT_is_optional' => 0x21, 
		     'DW_AT_lower_bound' => 0x22, 
		     'DW_AT_producer' => 0x25, 
		     'DW_AT_prototyped' => 0x27, 
		     'DW_AT_return_addr' => 0x2a, 
		     'DW_AT_start_scope' => 0x2c, 
		     'DW_AT_stride_size' => 0x2e, 
		     'DW_AT_upper_bound' => 0x2f,
		     'DW_AT_abstract_origin' => 0x31, 
		     'DW_AT_accessibility' => 0x32, 
		     'DW_AT_address_class' => 0x33, 
		     'DW_AT_artificial' => 0x34, 
		     'DW_AT_base_types' => 0x35, 
		     'DW_AT_calling_convention' => 0x36, 
		     'DW_AT_count' => 0x37, 
		     'DW_AT_data_member_location' => 0x38, 
		     'DW_AT_decl_column' => 0x39, 
		     'DW_AT_decl_file' => 0x3a, 
		     'DW_AT_decl_line' => 0x3b, 
		     'DW_AT_declaration' => 0x3c, 
		     'DW_AT_discr_list' => 0x3d, 
		     'DW_AT_encoding' => 0x3e, 
		     'DW_AT_external' => 0x3f, 
		     'DW_AT_frame_base' => 0x40, 
		     'DW_AT_friend' => 0x41, 
		     'DW_AT_identifier_case' => 0x42, 
		     'DW_AT_macro_info' => 0x43, 
		     'DW_AT_namelist_item' => 0x44, 
		     'DW_AT_priority' => 0x45, 
		     'DW_AT_segment' => 0x46, 
		     'DW_AT_specification' => 0x47, 
		     'DW_AT_static_link' => 0x48, 
		     'DW_AT_type' => 0x49, 
		     'DW_AT_use_location' => 0x4a, 
		     'DW_AT_variable_parameter' => 0x4b, 
		     'DW_AT_virtuality' => 0x4c, 
		     'DW_AT_vtable_elem_location' => 0x4d, 
		     'DW_AT_lo_user' => 0x2000,
		     'DW_AT_mangled_name' => 0x2007,
		     'DW_AT_hi_user' => 0x3fff 
		     );

    return %DWARF_ATS;
}

sub invDwarfAts {
    my %dwarf_ats = getDwarfAts ();

    my %invert;

    my $key;

    foreach $key ( keys %dwarf_ats ) {       
	$invert{ $dwarf_ats{$key}}  = $key;
    }
    
    return %invert;
}

sub invHash {
    my $hash = shift @_;

    my %inv;

    my $key;
    foreach $key ( keys %$hash ) {
	$inv{ $hash->{$key} } = $key;
    }
    return %inv;
}

sub invDwarfForms {
    my %df = getDwarfForms();

    return invHash(\%df);
}

sub getDwarfForms {
    my %dwarf_forms = (		       
		       'DW_FORM_addr' => 0x01, 
		       'DW_FORM_block2' => 0x03, 
		       'DW_FORM_block4' => 0x04, 
		       'DW_FORM_data2' => 0x05, 
		       'DW_FORM_data4' => 0x06,
		       'DW_FORM_data8' => 0x07, 
		       'DW_FORM_string' => 0x08, 
		       'DW_FORM_block' => 0x09,
		       'DW_FORM_block1' => 0x0a,
		       'DW_FORM_data1' => 0x0b, 
		       'DW_FORM_flag' => 0x0c, 
		       'DW_FORM_sdata' => 0x0d, 
		       'DW_FORM_strp' => 0x0e,
		       'DW_FORM_udata' => 0x0f, 
		       'DW_FORM_ref_addr' => 0x10, 
		       'DW_FORM_ref1' => 0x11, 
		       'DW_FORM_ref2' => 0x12, 
		       'DW_FORM_ref4' => 0x13, 
		       'DW_FORM_ref8' => 0x14, 
		       'DW_FORM_ref_udata' => 0x15, 
		       'DW_FORM_indirect' => 0x16
		       );

    return %dwarf_forms;
}
