#!/bin/ksh
#
# Shell script wrapper around proxify program
# Dave Dykstra, dykstra@cs.uiuc.edu
#

usage()
{
    echo "Usage: Proxify++ [cpp_opts] [proxify++_opts] [-hmachine] header_file [...]" >&2

    echo "  proxify++_opts are [-all|-tableonly] [-version]" >&2
    exit 1
}

case $# in
    0) usage;;
esac

CPP=${CPP:-/usr/lib/cpp}
#CPP=$CPP0" -+ -undef"
PROXIFY=`dirname $0`/proxify

TMP=""
ALL=""
PROXOPTS=""
REALBASE=""
trap 'rm -f $TMP $TMP.2; \
    if [ "$ALL" = "" ]; \
    then rm -f T/${REALBASE}T.cc; \
    else rm -f ${REALBASE}T.cc; \
    fi; exit' 1 2 3 15 

OPTS=""
HOST=""
for F
do
    case $F in
	-all|-tableonly) ALL=$F
	      continue
	      ;;
	-version) PROXOPTS="$PROXOPTS $F"
	      continue
	      ;;
	-h*)  HOST=`echo $F|sed 's/-h//'`
	      continue
	      ;;
	-*) OPTS="$OPTS $F"
	    continue
	    ;;
    esac
    BASE=`basename $F`
    REALBASE=`echo $BASE|sed 's/\..*//'`
    TMP=$REALBASE.P
    rm -f $TMP $TMP.2

    echo "Preprocessing $F"
    if $CPP -DPROXIFY -DCFRONT $OPTS $F >$TMP
    then
	:
    else
	exit $?
    fi
    # operator deletion temporary to avoid grammar bug
    sed -e '/^[ 	]*operator/d' -e '/^[ 	]*asm/d' < $TMP >$TMP.2
    mv $TMP.2 $TMP

    E=0
    case $HOST in
	"") echo "Proxifying $F"
	    if $PROXIFY $ALL $PROXOPTS $BASE <$TMP
	    then
		:
	    else
		E=$?
	    fi
	    ;;
	*)  echo "Remoting proxify $F to $HOST"
	    # the 'sh -c' is to get it to print out core dump messages
	    # the '</dev/null' is to keep it from wanting tty input
	    # redirecting output to a file is because rsh won't return
	    #   an error code.
	    if rsh $HOST "cd `pwd`;sh -c \"$PROXIFY $ALL $PROXOPTS $BASE\" <$TMP"\
				</dev/null  2>&1 | tee $TMP.2 >&2
	    then
		if [ -s $TMP.2 ]
		then
		    if grep -v '[Ww]arning' $TMP.2|grep -v took >/dev/null
		    then
			# there's more than warning messages and
			#   the 'took N seconds' information messages
			E=15
		    fi
		fi
	    else
		E=$?
	    fi
	    rm -f $TMP.2
	    ;;
    esac
    case $E in
	0);;
	*)  echo "Error proxifying, preprocessed source left in $TMP" >&2
	    case $ALL in
		"") TFILE=T/${REALBASE}T.cc;;
		*)  TFILE=${REALBASE}T.cc;;
	    esac
	    if [ -f $TFILE ]
	    then
		echo "  and $TFILE removed" >&2
		rm -f $TFILE
	    fi
	    exit $E
	    ;;
    esac

    rm -f $TMP $TMP.2
done

