#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/******************************************************************************
 *
 * error.c
 *
 * 	This file contains the functions for error handling.
 *
 * Ed Loyot 7/12/90
 *
 *              Copyright (c) 1990, 1991
 *              University of Virginia
 *              All Rights Reserved
 *
 *		This software is the property of the University of Virginia.
 *		Permission is granted to copy and distribute this software as
 *		long as this notice is intact on all copies.  Derived works 
 *  		must retain this notice.  This software is provided `as is'
 *		and without any express or implied warrenties.
 *
 *              Send problems & suggestions to:
 *		Ed Loyot (ecl2v@virginia.edu) or
 *              Andrew Grimshaw (grimshaw@cs.virginia.edu)
 *
 *****************************************************************************/

#include <stream.h>
#include <strstream.h>
#include "common.h"
#include "syntax_tree.h"
#include "parser.tab.h"

extern int linenum;
extern char program_name[];
extern symbol_table global_table;
extern symbol_table_stack scope_stack;

/*******************************************************************************
 *
 * yyerror -
 *
 *	This function is called when a parse error is encountered.
 *
 ******************************************************************************/

void yyerror(char *string)
{
    cerr << program_name << ": line " << linenum << ": error! : ";
    cerr << string << "\n";
}

/*******************************************************************************
 *
 * yywarning -
 *
 *	This function can be called to issue a warning.
 *
 ******************************************************************************/

void yywarning(char *string)
{
    cerr << program_name << ": line " << linenum << ": warning! : ";
    cerr << string << "\n";
}

/*******************************************************************************
 *
 * id_redeclared_error -
 *
 * 	This function issues the identifier being redeclared warning message.
 * This error is encountered in so many spots that it warrents a seperate
 * function.
 *
 *******************************************************************************/

identifier *id_redeclared_error(identifier *redeclared_id)
{
    /*
     * An identifier is being redeclared!  Log the error, remove the first
     * declaration and enter the new declaration in the symbol table.
     */

    ostrstream error(message_buffer, max_message_size);
    error << "Identifier " << redeclared_id->id_string;
    error << " is being redeclared!\n";
    error.put('\0');
    yyerror(message_buffer);
    identifier *temp_id_ptr = scope_stack.remove(redeclared_id);
    return (scope_stack.insert(redeclared_id));
}

identifier *id_redeclared_error(char *redeclared_id)
{
    /*
     * An identifier is being redeclared!  Log the error, remove the first
     * declaration and enter the new declaration in the symbol table.
     */

    ostrstream error(message_buffer, max_message_size);
    error << "Identifier " << redeclared_id;
    error << " is being redeclared!\n";
    error.put('\0');
    yyerror(message_buffer);
    identifier *temp_id_ptr = scope_stack.remove(redeclared_id);
    return (scope_stack.insert(redeclared_id, IDENTIFIER));
}
