%{
/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/******************************************************************************
 *
 * lexer.l
 *
 *	This file contains the lex/flex rules and actions for the mpl compiler.
 * This lexer is derived from one posted to the comp.lang.c++ newgroup by Tony
 * Sanders (cs.utexas.edu!ibmaus!auschs!sanders.austin.ibm.com!sanders).
 *
 * Ed Loyot 7/12/90
 *
 *              Copyright (c) 1990, 1991
 *              University of Virginia
 *              All Rights Reserved
 *
 *		This software is the property of the University of Virginia.
 *		Permission is granted to copy and distribute this software as
 *		long as this notice is intact on all copies.  Derived works 
 *  		must retain this notice.  This software is provided `as is'
 *		and without any express or implied warrenties.
 *
 *              Send problems & suggestions to:
 *		Ed Loyot (ecl2v@virginia.edu) or
 *              Andrew Grimshaw (grimshaw@cs.virginia.edu) 
 *
 *
 ****************************************************************************/

#include <stdio.h>
#include "String.h"

#ifdef SABER_CC
#define perror saber_perror
#include <libc.h>
#undef perror
#endif

#include "symbol_table.h"

//Gross hack, stream.h which is included in syntax_tree.h uses the name input
// this collides with the lex function input.  Use cpp to rename input in 
// stream.h.
#define input stream_input
#include "syntax_tree.h"
#undef input

#include "parser.tab.h"
#include "common.h"
#include "tokens.h"

#define STRDUP(X) ((char *)strcpy(new char[strlen(X)+1],X))
#ifdef DEBUG_LEXER
extern ostream *Debug_stream;
#define LEXER_DEBUG(X) *Debug_stream << X
#else
#define LEXER_DEBUG(X)
#endif DEBUG_LEXER

#ifdef CFRONT
extern "C" int read(...);
#endif

#ifdef __cplusplus
#define input yyinput
#endif

extern char program_name[];
extern int column;
extern int linenum;

void cpp_line();
void skip_comments();
void skip_CC_comments();
void count(p_boolean);
extern symbol_table global_table;
extern symbol_table_stack scope_stack;

%}

alpha    [a-zA-Z]
nz_digit [1-9] 
digit    [0-9]
hex_digit [a-fA-F0-9]
octs	 [0-7]
special  [\_]
ident    (({alpha}|{special})({alpha}|{digit}|{special})*)
int	 ({nz_digit}{digit}*L?)
octal    (0{octs}*L?)
hex      (0[xX]{hex_digit}+L?)
exp      ([Ee][-+]?{digit}+)
float1   ({digit}+\.{digit}*)
float2   ({digit}+{exp})
float3	 (\.{digit}+{exp}?)
float4   ({digit}+\.{digit}+{exp})
float    ({float1}|{float2}|{float3}|{float4})
char     ((\'.\')|(\'\\[ntbrf]\')|(\'\\\')|(\'\'\')|(\'\\{octs}{octs}?{octs}?\'))

%%

^\#.*		{ count(FALSE); cpp_line(); }
[ \n\t\f]+	{ count(FALSE); /* skip white space */ }
"/*"		{ count(TRUE); skip_comments(); }
"//"		{ count(TRUE); skip_CC_comments(); }
"->"		{ count(TRUE); yylval.token = ARROW;	   return ARROW;      }
"++"		{ count(TRUE); yylval.token = ICR;	   return ICR;        }
"--"		{ count(TRUE); yylval.token = DECR;	   return DECR;       }
"<<"		{ count(TRUE); yylval.token = LS;	   return LS;         }
">>"		{ count(TRUE); yylval.token = RS;	   return RS;         }
"<="		{ count(TRUE); yylval.token = LE;	   return LE;         }
">="		{ count(TRUE); yylval.token = GE;	   return GE;         }
"=="		{ count(TRUE); yylval.token = EQ;	   return EQ;         }
"!="		{ count(TRUE); yylval.token = NE;	   return NE;         }
"&&"		{ count(TRUE); yylval.token = ANDAND;	   return ANDAND;     }
"||"		{ count(TRUE); yylval.token = OROR;	   return OROR;       }
"..."		{ count(TRUE); yylval.token = ELLIPSIS;	   return ELLIPSIS;   }
"::"		{ count(TRUE); yylval.token = CLCL;	   return CLCL;       }
".*"		{ count(TRUE); yylval.token = DOTstar;	   return DOTstar;    }
"->*"		{ count(TRUE); yylval.token = ARROWstar;   return ARROWstar;  }
"*="		{ count(TRUE); yylval.token = MULTassign;  return MULTassign; }
"/="		{ count(TRUE); yylval.token = DIVassign;   return DIVassign;  }
"%="		{ count(TRUE); yylval.token = MODassign;   return MODassign;  }
"+="		{ count(TRUE); yylval.token = PLUSassign;  return PLUSassign; }
"-="		{ count(TRUE); yylval.token = MINUSassign; return MINUSassign;}
"<<="		{ count(TRUE); yylval.token = LSassign;	   return LSassign;   }
">>="		{ count(TRUE); yylval.token = RSassign;	   return RSassign;   }
"&="		{ count(TRUE); yylval.token = ANDassign;   return ANDassign;  }
"^="		{ count(TRUE); yylval.token = ERassign;	   return ERassign;   }
"|="		{ count(TRUE); yylval.token = ORassign;	   return ORassign;   }

"("		{ count(TRUE); yylval.token = '('; return '('; }
","		{ count(TRUE); yylval.token = ','; return ','; }
")"		{ count(TRUE); yylval.token = ')'; return ')'; }
";"		{ count(TRUE); yylval.token = ';'; return ';'; }
"{"		{ count(TRUE); yylval.token = '{'; return '{'; }
"}"		{ count(TRUE); yylval.token = '}'; return '}'; }
"["		{ count(TRUE); yylval.token = '['; return '['; }
"]"		{ count(TRUE); yylval.token = ']'; return ']'; }
"*"		{ count(TRUE); yylval.token = '*'; return '*'; }
"/"		{ count(TRUE); yylval.token = '/'; return '/'; }
"+"		{ count(TRUE); yylval.token = '+'; return '+'; }
"-"		{ count(TRUE); yylval.token = '-'; return '-'; }
"%"		{ count(TRUE); yylval.token = '%'; return '%'; }
"^"		{ count(TRUE); yylval.token = '^'; return '^'; }
"&"		{ count(TRUE); yylval.token = '&'; return '&'; }
"|"             { count(TRUE); yylval.token = '|'; return '|'; }
"?"		{ count(TRUE); yylval.token = '?'; return '?'; }
":"		{ count(TRUE); yylval.token = ':'; return ':'; }
"!"		{ count(TRUE); yylval.token = '!'; return '!'; }
"."		{ count(TRUE); yylval.token = '.'; return '.'; }
"~"		{ count(TRUE); yylval.token = '~'; return '~'; }
"<"		{ count(TRUE); yylval.token = '<'; return '<'; }
">"		{ count(TRUE); yylval.token = '>'; return '>'; }
"."		{ count(TRUE); yylval.token = '.'; return '.'; }
"="		{ count(TRUE); yylval.token = '='; return '='; }

auto		{ count(TRUE); yylval.token = AUTO; 	return AUTO; 	}
break		{ count(TRUE); yylval.token = BREAK; 	return BREAK; 	}
case		{ count(TRUE); yylval.token = CASE; 	return CASE; 	}
char		{ count(TRUE); yylval.token = CHAR; 	return CHAR; 	}
const		{ count(TRUE); yylval.token = CONST; 	return CONST; 	}
continue	{ count(TRUE); yylval.token = CONTINUE; return CONTINUE;}
default		{ count(TRUE); yylval.token = DEFAULT; 	return DEFAULT; }
do		{ count(TRUE); yylval.token = DO; 	return DO; 	}
double		{ count(TRUE); yylval.token = DOUBLE; 	return DOUBLE; 	}
else		{ count(TRUE); yylval.token = ELSE; 	return ELSE; 	}
enum		{ count(TRUE); yylval.token = ENUM; 	return ENUM; 	}
extern		{ count(TRUE); yylval.token = EXTERN; 	return EXTERN; 	}
float		{ count(TRUE); yylval.token = FLOAT; 	return FLOAT; 	}
for		{ count(TRUE); yylval.token = FOR; 	return FOR; 	}
goto		{ count(TRUE); yylval.token = GOTO; 	return GOTO; 	}
if		{ count(TRUE); yylval.token = IF; 	return IF; 	}
int		{ count(TRUE); yylval.token = INT; 	return INT; 	}
long		{ count(TRUE); yylval.token = LONG; 	return LONG; 	}
register	{ count(TRUE); yylval.token = REGISTER; return REGISTER;}
return		{ count(TRUE); yylval.token = RETURN; 	return RETURN; 	}
short		{ count(TRUE); yylval.token = SHORT; 	return SHORT; 	}
signed		{ count(TRUE); yylval.token = SIGNED; 	return SIGNED; 	}
sizeof		{ count(TRUE); yylval.token = SIZEOF; 	return SIZEOF; 	}
static		{ count(TRUE); yylval.token = STATIC; 	return STATIC; 	}
struct		{ count(TRUE); yylval.token = STRUCT; 	return STRUCT; 	}
switch		{ count(TRUE); yylval.token = SWITCH; 	return SWITCH; 	}
typedef		{ count(TRUE); yylval.token = TYPEDEF; 	return TYPEDEF; }
union		{ count(TRUE); yylval.token = UNION; 	return UNION; 	}
unsigned	{ count(TRUE); yylval.token = UNSIGNED; return UNSIGNED;}
void		{ count(TRUE); yylval.token = VOID; 	return VOID; 	}
volatile	{ count(TRUE); yylval.token = VOLATILE; return VOLATILE;}
while		{ count(TRUE); yylval.token = WHILE; 	return WHILE; 	}

new		{ count(TRUE); yylval.token = NEW; 	return NEW; 	}
delete		{ count(TRUE); yylval.token = DELETE; 	return DELETE; 	}
this		{ count(TRUE); yylval.token = THIS; 	return THIS; 	}
operator	{ count(TRUE); yylval.token = OPERATOR; return OPERATOR;}
class		{ count(TRUE); yylval.token = CLASS; 	return CLASS; 	}
public		{ count(TRUE); yylval.token = PUBLIC; 	return PUBLIC; 	}
protected	{ count(TRUE); yylval.token = PROTECTED;return PROTECTED;}
private		{ count(TRUE); yylval.token = PRIVATE; 	return PRIVATE; }
virtual		{ count(TRUE); yylval.token = VIRTUAL; 	return VIRTUAL; }
friend		{ count(TRUE); yylval.token = FRIEND; 	return FRIEND; 	}
inline		{ count(TRUE); yylval.token = INLINE; 	return INLINE; 	}
overload	{ count(TRUE); yylval.token = OVERLOAD; return OVERLOAD;}

proxiable 	{ count(TRUE); yylval.token = PROXIABLE; return PROXIABLE;}


mentat  	{ count(TRUE); yylval.token = MENTAT;     return MENTAT;}
persistent	{ count(TRUE); yylval.token = PERSISTENT; return PERSISTENT;}
regular 	{ count(TRUE); yylval.token = REGULAR;    return REGULAR;}


{char}	{
	count(TRUE);
	yylval.string = STRDUP(yytext+1);
	yylval.string[strlen(yylval.string)-1] = 0;
	/* return STRING without quotes */
	return CHARACTERconstant;
}

\".*\" {
	count(TRUE);
	yylval.string = STRDUP(yytext+1);
	yylval.string[strlen(yylval.string)-1] = 0;
	/* return STRING without quotes */
	return STRINGliteral;
}

{int}   {
	count(TRUE);
	yylval.string = STRDUP(yytext);
	return INTEGERconstant;
}

{octal} {
	count(TRUE);
	yylval.string = STRDUP(yytext);
	return OCTALconstant;
}

{hex}   {
	count(TRUE);
	yylval.string = STRDUP(yytext);
	return HEXconstant;
}

{float} {
	count(TRUE);
	yylval.string = STRDUP(yytext);
	return FLOATINGconstant;
}

{ident} {
	count(TRUE);
	/*
	 *
	 */
	yylval.string = STRDUP(yytext);
	identifier *temp_id_ptr = scope_stack.lookup(yylval.string);
	if (temp_id_ptr != NULL)
	    switch(temp_id_ptr->id_token)
	    {
	      case CLASS:
	      case STRUCT:
	      case UNION:
	      case ENUM:
	      case TYPEDEFname:
		return TYPEDEFname;
	      default:
		return IDENTIFIER;
	    }
	else
	    return IDENTIFIER;
}

.	{
	count(TRUE); 
	fprintf(stderr,"%s: line %d: Unrecognized character in input!",
	        program_name, linenum); 
	fprintf(stderr,": %s will abort!\n",Me); 
	exit(-1); 
}

%%

void skip_comments()
{
    char c;

    c = input();
    while (1) {
	while (c != '*') {
	    if (c == '\n') {
		column = 0;
		linenum++;
	    }
	    else if (c == '\t')
		column += 8 - (column % 8);
	    else if (c == EOF)
	    {
		fprintf(stderr,"%s: line %d: Input terminited with open",
			program_name, linenum); 
		fprintf(stderr," comment!: %s will abort!\n",Me); 
		exit(-1); 
	    }
	    else
		column++;
	    c = input();
	}
	if ((c = input()) == '/') {
	    column++;
	    LEXER_DEBUG("symbol found: */\n");
	    return;
	}
    }
}

void skip_CC_comments()
{
    char c;

	while (((c = input()) != '\n') && (c != EOF))
	{
	    if (c == '\t')
		column += 8 - (column % 8);
	    else
		column++;
	}
	linenum++;
}

void cpp_line()
{
    int num_matched, new_linenum, end_num;
    char temp_string[MAX_PROGRAM_NAME];
    
    end_num = 1;
    num_matched = sscanf(yytext,"#%d %s%d", &new_linenum, temp_string, &end_num);
    if (num_matched >= 2)
    {
	linenum = new_linenum;
	char *p;
	if (temp_string[0] == '"')
	{
	    /* skip quotes */
	    (void) strcpy(program_name, &temp_string[1]);
	    if ((p = rindex(program_name, '"')) != NULL)
		*p = '\0';
	}
	else
	    (void) strcpy(program_name, &temp_string[0]);

#ifdef PROXIFY
	{
	    extern void proxify_line(p_boolean);

	    if ((new_linenum == 1) && (end_num == 1))
		proxify_line(TRUE);
	    else
		proxify_line(FALSE);
	}
#endif
    }
    linenum--;
}

void count (p_boolean notwhite)
{
    register char *s;

    if (notwhite)
	LEXER_DEBUG("symbol found: " <<  yytext << "\n");
    for (s = yytext; *s; s++)
	if (*s == '\n') {
	    column = 0;
	    linenum++;
	}
	else if (*s == '\t')
	    column += 8 - (column % 8);
	else
	    column++;
}
