#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/******************************************************************************
 *
 * main.c
 *
 *	This program contains the function main() which drives the parser.
 *
 * Ed Loyot 7/12/90
 *
 *              Copyright (c) 1990, 1991
 *              University of Virginia
 *              All Rights Reserved
 *
 *		This software is the property of the University of Virginia.
 *		Permission is granted to copy and distribute this software as
 *		long as this notice is intact on all copies.  Derived works 
 *  		must retain this notice.  This software is provided `as is'
 *		and without any express or implied warrenties.
 *
 *              Send problems & suggestions to:
 *		Ed Loyot (ecl2v@virginia.edu) or
 *              Andrew Grimshaw (grimshaw@cs.virginia.edu)
 *
 *****************************************************************************/

#include <stddef.h>
#include <stream.h>
#include <fstream.h>
#include "String.h"
#include <stdio.h>
#include <stdlib.h>

#include "fdstream.h"
#include "syntax_tree.h"
#include "symbol_table.h"
#include "common.h"

extern int  yyparse();
extern void back_end_gen(definition_list*);

char 		    *Me = "Proxify++";
char 		    program_name[MAX_PROGRAM_NAME] = "no_name.c";
int 		    column = 0;
int 		    linenum = 1;
symbol_table        global_table;
symbol_table        *parameter_table = NULL;
symbol_table_stack  scope_stack(&global_table);
definition_list     *parser_return_value = NULL;
class_definition    *mentat_class;
function_definition *main_function = NULL;
ostream		    *Debug_stream = &cerr;

#ifdef YYDEBUG
#include <fcntl.h>
extern "C" char *getenv(const char *);
// extern "C" int open(const char *, int, ...);
#endif

/******************************************************************************
 *
 * main() -
 *
 *	This function invokes the parser on the input.  The parser builds a 
 * parse tree for the entire input then returns a pointer to that tree.
 * The value returned from the parsing function yyparse is 0 if there were no
 * parsing errors.  The pointer to the syntax tree is returned in the global
 * variable parser_return_value.  Next various passes can be made to manipulate
 * the syntax tree.  These passes include generating code for a mentat backend,
 * detecting mentat potential result variables and generating the code for the
 * mentat extensions.  The final step is to print out the syntax tree.
 *
 *****************************************************************************/

#ifdef PROXIFY

extern void proxify_args(int, char **);
extern void proxify_exit(int);
extern "C" long time(long *);
extern int Doall;
extern "C" void signal( int, void *());


char *program;

#ifdef SYSV
void 
crashed()
{
	fprintf(stderr, "\n%s: Crashed on line number %d\n", program_name, linenum);
        exit(1);
}

#else /* SYSV */
void 
crashed(int no, void* info)
{
    if(no != 0) {
	fprintf(stderr, "exit code : %d\n", no);
	fprintf(stderr, "\n%s: Crashed on line number %d\n", program_name, linenum);
        exit(1);
    }
	exit(0);
}
#endif /* SYSV */

#endif

int main(int argc, char *argv[])
{
	program = argv[0];
#ifdef YYDEBUG
    char *deb;
    if (((deb = getenv("YYDEBUG")) == NULL) || (*deb < '2'))
    {
	int fd;
	fd = open("/dev/null", O_WRONLY, 0666);
	if (fd == -1)
	{
	    cerr << Me << ": cannot open /dev/null!\n";
	    exit(1);
	}
	//Debug_stream = new ostream(fd);
	Debug_stream = new fdostream(fd);
    }
#endif

#ifdef PROXIFY

#ifdef SYSV
	atexit(crashed);
#else
#ifndef MULTIMAX
	on_exit(crashed, 0);
#endif
#endif
	signal(11, (void *(*)())crashed);
	signal(10, (void *(*)())crashed);

    proxify_args(argc, argv);
    long tm = time(0);
#endif


#ifdef MPLFRONT
    p_boolean do_mentat = TRUE;
    
    // Process the arguments if any.
    if (argc > 1)
	if (strcmp(argv[1],"-N") == 0)
	    do_mentat = FALSE;
#endif
    
    // Invoke the parser on the input.
    int yyparse_return_value = yyparse();

    if (yyparse_return_value == 0)
    {
	// No errors during the parse.
	if (parser_return_value != NULL)
	{
#ifdef MPLFRONT
	    if (do_mentat)
	    {
		// Mark mentat potential result variables (prvs).
		parser_return_value->mark_prvs();

		// Generate code for the mentat extensions.
		parser_return_value->mpl_code_gen();

		// Generate the mentat backend code.
		parser_return_value->mpl_server_gen();
	    }
#endif

#ifdef PROXIFY
	    long tm2;
	    if (Doall == 1)
	    {
		tm2 = time(0);
		printf("Proxify++: parse phase took %d seconds\n", tm2 - tm);
	    }
	    parser_return_value->proxify();
	    if (Doall == 1)
	    {
		tm = time(0);
		printf("Proxify++: proxify phase took %d seconds\n", tm - tm2);
	    }
#else
	    // Print the parse tree.
	    parser_return_value->print();
#endif
	}
#ifdef PROXIFY
//        printf("Calling the last proxify_exit(0)\n");
	proxify_exit(0);
#endif

	// No errors during parse.
	return (0);
    }

    else
    {
	// There was an error during parsing.
	cout << "/* Error.\n*/";

#ifdef PROXIFY
	proxify_exit(5);
#endif
	// There was an error during the parse.
	return (5);
    }
}	

