#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Contributing Author: Amitabh Dave (dave@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/******************************************************************************
 *
 * print.c
 *
 *	This file contains the member functions that print the nodes of the
 * syntax tree.  The classes that use these member functions are defined in 
 * syntax_tree.h.
 *
 * Ed Loyot 7/13/90
 *
 *              Copyright (c) 1990, 1991
 *              University of Virginia
 *              All Rights Reserved
 *
 *		This software is the property of the University of Virginia.
 *		Permission is granted to copy and distribute this software as
 *		long as this notice is intact on all copies.  Derived works 
 *  		must retain this notice.  This software is provided `as is'
 *		and without any express or implied warrenties.
 *
 *              Send problems & suggestions to:
 *		Ed Loyot (ecl2v@virginia.edu) or
 *              Andrew Grimshaw (grimshaw@cs.virginia.edu)
 *
 *****************************************************************************/

#ifdef SABER_CC
#include <libc.h>
#endif

#include <stddef.h>
#include <stdlib.h>
#include <stream.h>
#include <strstream.h>
#include "String.h"

#include "common.h"
#include "tokens.h"
#include "syntax_tree.h"
#include "parser.tab.h"

#ifdef  DEBUG_PRINT
extern ostream *Debug_stream;
#define PRINT_DEBUG(X) *Debug_stream << X
#else
#define PRINT_DEBUG(X)
#endif
   
#ifdef PROXIFY
extern HeaderDescript    *currentHead;
extern int Parameter_flag;
extern int Parameter_count;
extern char *currentClassName;
p_boolean Declarator_id_found = FALSE;
#endif

int indent_level = 0;

char *spaces[] = 
{
    "", " ", "  ", "   ", "    ", "     ", "      ", "       ", "        ", 
    "         ", "          ", "           ", "            ", "             ", 
    "              ", "               ", "                ", 
    "                 ", "                  ", "                   ", 
    "                    "
};

int tab_column = 0;

p_boolean untab  = FALSE;
p_boolean indent = TRUE;

int tab_size = 4;

inline int range(int number)
{
    int range_value;
    if (number < 0)
	range_value = 0;
    else
	if (number > 20)
	    range_value = 20;
	else
	    range_value = number;
    return range_value;
}

void incr_tab()
{
    tab_column = range(tab_column + tab_size);
}

void decr_tab()
{
    tab_column = range(tab_column - tab_size);
}

/******************************************************************************
 *
 *				Statements
 *
 *****************************************************************************/

void statement::print(ostream*, p_boolean)
{    
    cerr << Me << " : statement::print() : internal error! print";
    cerr << " member function called on an abstract base class!\n";
}

void statement_list::print(ostream *output_stream)
{
    PRINT_DEBUG("Printing a statement list!\n");
    if (statement_ptr != NULL)
	statement_ptr->print(output_stream);
    *output_stream << "\n";    
    *Debug_stream << "\n";    
    for (statement_list *temp_stmt_list = next; temp_stmt_list != NULL; 
	 temp_stmt_list = temp_stmt_list->next)
    {
	if (temp_stmt_list->get_statement() != NULL)
	    temp_stmt_list->get_statement()->print(output_stream);
	*output_stream << "\n";
	*Debug_stream << "\n";
    }
}

void compound_statement::print(ostream *output_stream, p_boolean)
{
    PRINT_DEBUG("Printing a compound statement!\n");
    p_boolean remember_untab = FALSE;    
    if (untab == TRUE)
    {
	remember_untab = TRUE;
	decr_tab();
    }
    untab = FALSE;
    *output_stream << spaces[tab_column] << "{\n";
    *Debug_stream << spaces[tab_column] << "{\n";
    incr_tab();
    if (declarations != NULL)
	declarations->print(output_stream);
    if (statements != NULL)
	statements->print(output_stream);
    decr_tab();
    *output_stream << spaces[tab_column] << "}";
    *Debug_stream << spaces[tab_column] << "}";
    if (remember_untab == TRUE)
	incr_tab();
}

void selection_statement::print(ostream *output_stream, p_boolean)
{
    PRINT_DEBUG("Printing an if or switch statement!\n");
    if (statement_token == IF) {
	*output_stream << spaces[tab_column] << "if (";
	*Debug_stream << spaces[tab_column] << "if (";
    }
    else {
	*output_stream << spaces[tab_column] << "switch (";
	*Debug_stream << spaces[tab_column] << "switch (";
    }
    if (expression_ptr != NULL)
	expression_ptr->print(output_stream);
    *output_stream << ")\n";
    *Debug_stream << ")\n";
    incr_tab();
    untab = TRUE;
    statement_ptr->print(output_stream);
    decr_tab();
    if (else_statement_ptr != NULL)
    {
	*output_stream << "\n" << spaces[tab_column] << "else\n";
	*Debug_stream << "\n" << spaces[tab_column] << "else\n";
	incr_tab();
	untab = TRUE;
	else_statement_ptr->print(output_stream);
	decr_tab();
    }
    untab = FALSE;
}

void while_statement::print(ostream *output_stream, p_boolean)
{
    PRINT_DEBUG("Printing a do or while statement!\n");
    if (statement_token == WHILE)
    {
	*output_stream << spaces[tab_column] << "while (";
	*Debug_stream << spaces[tab_column] << "while (";
	if (expression_ptr != NULL)
	    expression_ptr->print(output_stream);
	*output_stream << ")\n";
	*Debug_stream << ")\n";
	incr_tab();
	untab = TRUE;
	statement_ptr->print(output_stream);
	decr_tab();
	untab = FALSE;
    }
    else
    {
	*output_stream << spaces[tab_column] << "do\n";
	*Debug_stream << spaces[tab_column] << "do\n";
	incr_tab();
	untab = TRUE;
	statement_ptr->print(output_stream);
	decr_tab();
	untab = FALSE;
	*output_stream << "\n" << spaces[tab_column] << "while (";
	*Debug_stream << "\n" << spaces[tab_column] << "while (";
	if (expression_ptr != NULL)
	    expression_ptr->print(output_stream);
	*output_stream << ");";
	*Debug_stream << ");";
    }
}

void for_statement::print(ostream *output_stream, p_boolean)
{
    int remember_tab_column;
    PRINT_DEBUG("Printing a for statement!\n");
    *output_stream << spaces[tab_column] << "for (";
    *Debug_stream << spaces[tab_column] << "for (";
    remember_tab_column = tab_column;
    tab_column = 0;
    if (initializer_decl_or_expr != NULL)
	initializer_decl_or_expr->print(output_stream);
    *output_stream << " ";
    *Debug_stream << " ";
    if (test_ptr != NULL)
	test_ptr->print(output_stream);
    *output_stream << "; ";
    *Debug_stream << "; ";
    if (increment_ptr != NULL)
	increment_ptr->print(output_stream);
    *output_stream << ")\n";
    *Debug_stream << ")\n";
    tab_column = remember_tab_column;
    incr_tab();
    untab = TRUE;
    statement_ptr->print(output_stream);
    decr_tab();
    untab = FALSE;
}

void labeled_statement::print(ostream *output_stream, p_boolean)
{
    int remember_tab_column;
    PRINT_DEBUG("Printing a labeled, case or default statement!\n");
    switch (statement_token)
    {
      case LABEL:
	*output_stream << spaces[tab_column];
	*Debug_stream << spaces[tab_column];
	case_expr_or_label_ptr->print(output_stream);
	*output_stream << " : ";
	*Debug_stream << " : ";
	remember_tab_column = tab_column;
	tab_column = 0;
	if (statement_ptr != NULL)
	    statement_ptr->print(output_stream);
	tab_column = remember_tab_column;
	break;
      case CASE:
	*output_stream << spaces[tab_column] << "case ";
	*Debug_stream << spaces[tab_column] << "case ";
	case_expr_or_label_ptr->print(output_stream);
	*output_stream << " : \n";
	*Debug_stream << " : \n";
	if (statement_ptr != NULL)
	    statement_ptr->print(output_stream);
	break;
      case DEFAULT:
	*output_stream << spaces[tab_column] << "default : \n";
	*Debug_stream << spaces[tab_column] << "default : \n";
	if (statement_ptr != NULL)
	    statement_ptr->print(output_stream);
	break;
    }
}

void jump_statement::print(ostream *output_stream, p_boolean)
{
    PRINT_DEBUG("Printing a goto, continue, break or return statement!\n");
    switch (statement_token)
    {
      case GOTO:
	*output_stream << spaces[tab_column] << "goto ";
	*Debug_stream << spaces[tab_column] << "goto ";
	return_expr_or_label_ptr->print(output_stream);
	*output_stream << ";";
	*Debug_stream << ";";
	break;
      case CONTINUE:
	*output_stream << spaces[tab_column] << "continue;";
	*Debug_stream << spaces[tab_column] << "continue;";
	break;
      case BREAK:
	*output_stream << spaces[tab_column] << "break;";
	*Debug_stream << spaces[tab_column] << "break;";
	break;
      case RETURN:
	if (return_expr_or_label_ptr != NULL)
	{
	    *output_stream << spaces[tab_column] << "return ";
	    *Debug_stream << spaces[tab_column] << "return ";
	    return_expr_or_label_ptr->print(output_stream);
	}
	else {
	    *output_stream << spaces[tab_column] << "return";
	    *Debug_stream << spaces[tab_column] << "return";
	}
	*output_stream << ";";
	*Debug_stream << ";";
	break;
      default:
	cerr << Me << ": Internal error in jump_statement::print(): ";
	cerr << "Invalid statement_token! " << Me << " will abort!\n";
	exit(1);
    }
}

void expression_statement::print(ostream *output_stream, p_boolean)
{
    PRINT_DEBUG("Printing an expression statement!\n");
    if (expression_ptr != NULL)
    {
	*output_stream << spaces[tab_column];
	*Debug_stream << spaces[tab_column];
	expression_ptr->print(output_stream);
	*output_stream << ";";
	*Debug_stream << ";";
    }
    else {
	*output_stream << spaces[tab_column] << ";";
	*Debug_stream << spaces[tab_column] << ";";
    }
}

#ifdef MPLFRONT
void code_gen_statement::print(ostream *output_stream, p_boolean)
{
    PRINT_DEBUG("Printing a code_gen statement!\n");
    *output_stream << spaces[tab_column] << code_string;
    *Debug_stream << spaces[tab_column] << code_string;
}
#endif

/******************************************************************************
 *
 *				Declarations
 *
 *****************************************************************************/

void  declarator::print(ostream *output_stream, p_boolean print_scope)
{
    PRINT_DEBUG("Printing a declarator!\n");
#ifdef PROXIFY
    int arg_count = Parameter_count;
#endif

    char post_modifiers[256], pre_modifiers[256];

    strcpy(post_modifiers, "");
    strcpy(pre_modifiers, "");

    if (declarator_type != NULL)
    {
	char scratch[256];
	int previous_precedence = 1;
  
	ostrstream post_declarator_stream(post_modifiers, 256);

	for (type *type_ptr = declarator_type;
	     type_ptr != NULL; 
	     type_ptr = type_ptr->get_next())
	{
            strcpy(scratch, "");
	    ostrstream scratch_stream(scratch, 256);
	    type_ptr->get_type_element()->print(&scratch_stream,
						TRUE,
						&post_declarator_stream,
						&previous_precedence);
	    scratch_stream.put('\0');
	    strcat(scratch, pre_modifiers);
	    strcpy(pre_modifiers, scratch);
	}
	post_declarator_stream.put('\0');
	*output_stream << pre_modifiers;
	*Debug_stream << pre_modifiers;
    }
    if (declarator_id != NULL)
    {
	if (pre_modifiers[0] != '\0') {
	    *output_stream << " ";
	    *Debug_stream << " ";
	}
	if (declarator_id->scope_list != NULL && print_scope)
	    declarator_id->scope_list->print(CLCL, output_stream);
#ifdef PROXIFY
	if (arg_count == 0)
	{
	    if (Parameter_flag > 1) {
		*output_stream << currentClassName << "::";
		*Debug_stream << currentClassName << "::";
	    }
	    declarator_id->print(output_stream);
	}
	else if (Parameter_flag == 3)
	{
	    Declarator_id_found = TRUE;
	    *output_stream << "a" << arg_count;
	    *Debug_stream << "a" << arg_count;
	}
#else
	declarator_id->print(output_stream);
#endif
    }
    *output_stream << post_modifiers;
    *Debug_stream << post_modifiers;

    if (parenthesized_initializers != NULL)
	parenthesized_initializers->print(output_stream);

    if (initializers != NULL)
	initializers->print(output_stream);

    if (bit_field_size_expression != NULL)
	bit_field_size_expression->print(output_stream);

#ifdef PROXIFY
    if (Parameter_flag == 0)
#endif
    if (pure_virtual_flag) {
	*output_stream << " = 0";
	*Debug_stream << " = 0";
    }
}

void declarator_list::print(ostream *output_stream)
{
    PRINT_DEBUG("Printing a declarator list!\n");
    if (declarator_ptr != NULL)
    {
	declarator_ptr->print(output_stream);
	if (next != NULL) {
	    *output_stream << ", ";
	    *Debug_stream << ", ";
	}
    }
    for (declarator_list *temp_decl_list = next; temp_decl_list != NULL; 
	 temp_decl_list = temp_decl_list->next)
    {
	temp_decl_list->get_declarator()->print(output_stream);
	if (temp_decl_list->next != NULL) {
	    *output_stream << ", ";
	    *Debug_stream << ", ";
        }
    }
}

void variable_declaration::print(ostream *output_stream,p_boolean print_semicolon)
{
    PRINT_DEBUG("Printing a declaration!\n");
    if (indent) {
	*output_stream << spaces[tab_column];
	*Debug_stream << spaces[tab_column];
    }
    if (declaration_type != NULL)
    {
	declaration_type->print(output_stream, FALSE);
	if (declarators != NULL) {
	    *output_stream << " ";
	    *Debug_stream << " ";
	}
    }
    if (declarators != NULL)
	declarators->print(output_stream);
    if (print_semicolon) {
	*output_stream << ";";
	*output_stream << ";";
    }
}    

void csue_declaration::print(ostream *output_stream, p_boolean)
{
    PRINT_DEBUG("Printing a csue declaration!\n");
    csue_type_ptr->print(output_stream, TRUE);
}

void member_declaration::print(ostream *output_stream, p_boolean)
{
    PRINT_DEBUG("Printing a member declaration!\n");
    if (var_declaration != NULL)
	var_declaration->print(output_stream);
    else
	if (func_definition != NULL)
	    func_definition->print(output_stream);
        else
	    if (cl_definition != NULL)
		cl_definition->print(output_stream);
            else
	    {
		*output_stream << spaces[tab_column];
		*Debug_stream << spaces[tab_column];
		class_rescoped_identifier->print(output_stream);
		*output_stream << ";";
		*Debug_stream << ";";
	    }
}

void initializer::print(ostream *output_stream, p_boolean print_equals)
{
    PRINT_DEBUG("Printing an initializer!\n");
#ifdef PROXIFY
    if (Parameter_flag > 1)
    {
	PRINT_DEBUG("Skipping initializer!\n");
	return;
    }
#endif

    if (print_equals) {
	*output_stream << " = ";
	*Debug_stream << " = ";
    }

    if (assignment_expression != NULL)
	assignment_expression->print(output_stream);

    if (initializers != NULL)
	initializers->print(output_stream);
}

void initializer_list::print(ostream *output_stream)
{
    PRINT_DEBUG("Printing an initializer list!\n");
    *output_stream << "{";
    *Debug_stream << "{";
    initializer_ptr->print(output_stream, FALSE);
    for (initializer_list *temp_init_list = next; temp_init_list != NULL; 
	 temp_init_list = temp_init_list->next)
    {
	*output_stream << ", ";
	*Debug_stream << ", ";
	temp_init_list->get_initializer()->print(output_stream, FALSE);
    }
    *output_stream << "}";
    *Debug_stream << "}";
}    

void ctor_initializer::print(ostream *output_stream)
{
    PRINT_DEBUG("Printing a ctor_initializer!\n");
    if (name_string != NULL) {
	*output_stream << name_string;
	*Debug_stream << name_string;
    }
    if (arguments != NULL)
	arguments->print(output_stream);
    else
    {
	*output_stream << "(";
	*Debug_stream << "(";
	*output_stream << ")";
	*Debug_stream << ")";
    }
}

void ctor_initializer_list::print(ostream *output_stream)
{
    PRINT_DEBUG("Printing a ctor_initializer list!\n");
    *output_stream << " : ";
    *Debug_stream << " : ";
    ctor_initializer_ptr->print(output_stream);
    for (ctor_initializer_list *temp_init_list = next; temp_init_list != NULL; 
	 temp_init_list = temp_init_list->next)
    {
	*output_stream << ", ";
	*Debug_stream << ", ";
	temp_init_list->get_ctor_initializer()->print(output_stream);
    }
}    

/******************************************************************************
 *
 *				Expressions
 *
 *****************************************************************************/

void expression::print(ostream*)
{
    cerr << Me << " : expression::print() : internal error! print";
    cerr << " member function called on an abstract base class!\n";
}

void expression_list::print(ostream *output_stream)
{
    PRINT_DEBUG("Printing an expression list!\n");
    *output_stream << "(";
    *Debug_stream << "(";
    expression *temp_expr_ptr;
    if (expression_ptr != NULL)
	expression_ptr->print(output_stream);
    for (expression_list *temp_expr_list = next; temp_expr_list != NULL; 
	 temp_expr_list = temp_expr_list->next)
    {
	temp_expr_ptr = temp_expr_list->get_expression();
	if (temp_expr_ptr != NULL)
	{
	    *output_stream << ", ";
	    *Debug_stream << ", ";
	    temp_expr_ptr->print(output_stream);
	}
    }
    *output_stream << ")";
    *Debug_stream << ")";
}

void identifier::print(ostream *output_stream)
{
    PRINT_DEBUG("Printing an identifier!\n");
//    if (scope_list != NULL)
//	scope_list->print(CLCL, output_stream);
    if (destructor_flag == TRUE) {
	*output_stream << "~";
	*Debug_stream << "~";
    }
    if (operator_flag == TRUE)
    {
	*output_stream << "operator";
	*Debug_stream << "operator";
/*	if (operator_token == NONE)
	{
	    // This must be a conversion function.
	    *output_stream << " ";
	    *Debug_stream << " ";
	    conversion_type_ptr->print(output_stream);
	}
	else
	    // Print the operator.
	    switch (operator_token)
	    {
	      case '+':
		*output_stream << "+";
		*Debug_stream << "+";
		break;
	      case '-':
		*output_stream << "-";
		*Debug_stream << "-";
		break;
	      case '*':
		*output_stream << "*";
		*Debug_stream << "*";
		break;
	      case '/':
		*output_stream << "/";
		*Debug_stream << "/";
		break;
	      case '%':
		*output_stream << "%";
		*Debug_stream << "%";
		break;
	      case '^':
		*output_stream << "^";
		*Debug_stream << "^";
		break;
	      case '&':
		*output_stream << "&";
		*Debug_stream << "&";
		break;
	      case '|':
		*output_stream << "|";
		*Debug_stream << "|";
		break;
	      case '~':
		*output_stream << "~";
		*Debug_stream << "~";
		break;
	      case '!':
		*output_stream << "!";
		*Debug_stream << "!";
		break;
	      case '<':
		*output_stream << "<";
		*Debug_stream << "<";
		break;
	      case '>':
		*output_stream << ">";
		*Debug_stream << ">";
		break;
	      case LS:
		*output_stream << "<<";
		*Debug_stream << "<<";
		break;
	      case RS:
		*output_stream << ">>";
		*Debug_stream << ">>";
		break;
	      case ANDAND:
		*output_stream << "&&";
		*Debug_stream << "&&";
		break;
	      case OROR:
		*output_stream << "||";
		*Debug_stream << "||";
		break;
	      case ARROW:
		*output_stream << "->";
		*Debug_stream << "->";
		break;
	      case ARROWstar:
		*output_stream << "->*";
		*Debug_stream << "->*";
		break;
	      case ICR:
		*output_stream << "++";
		*Debug_stream << "++";
		break;
	      case DECR:
		*output_stream << "--";
		*Debug_stream << "--";
		break;
	      case LE:
		*output_stream << "<=";
		*Debug_stream << "<=";
		break;
	      case GE:
		*output_stream << ">=";
		*Debug_stream << ">=";
		break;
	      case EQ:
		*output_stream << "==";
		*Debug_stream << "==";
		break;
	      case NE:
		*output_stream << "!=";
		*Debug_stream << "!=";
		break;
	      case '=':
		*output_stream << "=";
		*Debug_stream << "=";
		break;
	      case MULTassign:
		*output_stream << "*=";
		*Debug_stream << "*=";
		break;
	      case DIVassign:
		*output_stream << "/=";
		*Debug_stream << "/=";
		break;
	      case MODassign:
		*output_stream << "%=";
		*Debug_stream << "%=";
		break;
	      case PLUSassign:
		*output_stream << "+=";
		*Debug_stream << "+=";
		break;
	      case MINUSassign:
		*output_stream << "-=";
		*Debug_stream << "-=";
		break;
	      case LSassign:
		*output_stream << "<<=";
		*Debug_stream << "<<=";
		break;
	      case RSassign:
		*output_stream << ">>=";
		*Debug_stream << ">>=";
		break;
	      case ANDassign:
		*output_stream << "&=";
		*Debug_stream << "&=";
		break;
	      case ERassign:
		*output_stream << "^=";
		*Debug_stream << "^=";
		break;
	      case ORassign:
		*output_stream << "|=";
		*Debug_stream << "|=";
		break;
	      case '(':
		*output_stream << "()";
		*Debug_stream << "()";
		break;
	      case '[':
		*output_stream << "[]";
		*Debug_stream << "[]";
		break;
	      case NEW:
		*output_stream << " new";
		*Debug_stream << " new";
		break;
	      case DELETE:
		*output_stream << " delete";
		*Debug_stream << " delete";
		break;
	      case ',':
		*output_stream << ",";
		*Debug_stream << ",";
		break;
	      default:
		 cerr << Me << ": Illegal overloaded operator #";
		 cerr << operator_token;
		 cerr << ": " << Me << " will abort!\n";
		 exit(1);
	     }*/
    }
    if (id_string != NULL) {
        if (!strcmp(id_string,"PersistentClassStar")) {
		currentHead->persistentFlag = 1;
		}
//	cout << "Current Class Name is " << currentClassName << "\n";
	*output_stream << id_string;
	*Debug_stream << id_string;
    }
}

void constant::print(ostream *output_stream)
{
    PRINT_DEBUG("Printing a constant!\n");
    switch(constant_token)
    {
      case INTEGERconstant :
      case FLOATINGconstant:
      case HEXconstant:
      case OCTALconstant:
	  *output_stream << constant_string;
	  *Debug_stream << constant_string;
	  break;
      case CHARACTERconstant:
	  *output_stream << "'" << constant_string << "'";
	  *Debug_stream << "'" << constant_string << "'";
	  break;
      case STRINGliteral:
	  *output_stream << "\"" << constant_string << "\"";
	  *Debug_stream << "\"" << constant_string << "\"";
	  break;
      default:
	  cerr << Me << ": Internal error in constant::print()";
	  cerr << ": Invalid token in constant_token! " << Me << " will abort!\n";
	  exit(1);
    }
}

void binary_expression::print(ostream *output_stream)
{
    PRINT_DEBUG("Printing a binary expression!\n");
    if (left_operand != NULL)
	left_operand->print(output_stream);
    switch(operator_token)
    {
      case '=' :
	  *output_stream << " = ";
	  *Debug_stream << " = ";
          break;
      case '+' :
	  *output_stream << " + ";
	  break;
      case '-' :
	  *output_stream << " - ";
	  break;
      case '*' :
	  *output_stream << " * ";
	  break;
      case '/' :
	  *output_stream << " / ";
	  break;
      case ICR :
	  *output_stream << "++";
	  break;
      case DECR :
	  *output_stream << "--";
	  break;
      case '.' :
	  *output_stream << ".";
	  break;
      case ARROW :
	  *output_stream << "->";
	  break;
      case '~' :
	  *output_stream << "~";
	  break;
      case '!' :
	  *output_stream << "!";
	  break;
      case LS :
	  *output_stream << "<<";
	  break;
      case RS :
	  *output_stream << ">>";
	  break;
      case '<' :
	  *output_stream << " < ";
	  break;
      case '>' :
	  *output_stream << " > ";
	  break;
      case LE :
	  *output_stream << " <= ";
	  break;
      case GE :
	  *output_stream << " >= ";
	  break;
      case EQ :
	  *output_stream << " == ";
	  break;
      case NE :
	  *output_stream << " != ";
	  break;
      case '&' :
	  *output_stream << " & ";
	  break;
      case '^' :
	  *output_stream << " ^ ";
	  break;
      case '|' :
	  *output_stream << " | ";
	  break;
      case ANDAND :
	  *output_stream << " && ";
	  break;
      case OROR :
	  *output_stream << " || ";
	  break;
      case MULTassign :
	  *output_stream << " *= ";
	  break;
      case DIVassign :
	  *output_stream << " /= ";
	  break;
      case MODassign :
	  *output_stream << " %= ";
	  break;
      case PLUSassign :
	  *output_stream << " += ";
	  break;
      case MINUSassign :
	  *output_stream << " -= ";
	  break;
      case LSassign :
	  *output_stream << " <<= ";
	  break;
      case RSassign :
	  *output_stream << " >>= ";
	  break;
      case ANDassign :
	  *output_stream << " &= ";
	  break;
      case ERassign :
	  *output_stream << " ^= ";
	  break;
      case ORassign :
	  *output_stream << " |= ";
	  break;
      case ',' :
	  *output_stream << " , ";
	  break;
      case '(' :
	  break;
      case '[' :
	  *output_stream << "[";
	  break;
      case THIS :
	  *output_stream << "this ";
	  break;
      case SIZEOF :
	  *output_stream << "sizeof ";
	  break;
      case CLCL :
	  *output_stream << "::";
	  break;
      case '%':
	  *output_stream << " % ";
	  break;
      case NONE :
	  break;
      default:
	  *output_stream << " Printing for operand not implemented yet! ";
    }
    if (right_operand != NULL)
	right_operand->print(output_stream);
    if (operator_token == '[')
	*output_stream << "]";
}

void allocation_expression::print(ostream *output_stream)
{
    PRINT_DEBUG("Printing an allocation expression!\n");
    if (allocation_token == NEW)
	*output_stream << "new ";
    else if (allocation_token == CLCL)
	*output_stream << ":: new ";
    if (argument_list != NULL)
	argument_list->print(output_stream);
    if (allocation_type != NULL)
	allocation_type->print(output_stream);
    if (type_expression != NULL)
	type_expression->print(output_stream);
    if (initializers != NULL)
	initializers->print(output_stream);
}

void deallocation_expression::print(ostream *output_stream)
{
    PRINT_DEBUG("Printing a deallocation expression!\n");
    if (deallocation_token == DELETE)
	*output_stream << "delete ";
    else if (deallocation_token == CLCL)
	*output_stream << ":: delete ";
    if (bracket_expression != NULL)
	bracket_expression->print(output_stream);
    if (deallocation_expr != NULL)
	deallocation_expr->print(output_stream);
}

void type_expression::print(ostream *output_stream)
{
    type_declaration->print(output_stream, FALSE);
}

#ifdef MPLFRONT
void code_gen_expression::print(ostream *output_stream)
{
    PRINT_DEBUG("Printing a type expression!\n");
    *output_stream << expression_string;
}
#endif

void cast_expression::print(ostream *output_stream)
{
    p_boolean saveindent = indent;
    indent = FALSE;
    type_expr->print(output_stream);
    indent = saveindent;
    cast_expr->print(output_stream);
}

void string_list::print(ostream *output_stream)
{
    PRINT_DEBUG("Printing an string list!\n");
    string_ptr->print(output_stream);
    for (string_list *temp_string_list = next; temp_string_list != NULL; 
	 temp_string_list = temp_string_list->next)
	temp_string_list->get_string()->print(output_stream);
}

void c_string_list::print(TOKEN seperator, ostream *output_stream)
{
    PRINT_DEBUG("Printing a c_string list!\n");
    if (virtual_flag == TRUE) {
	*output_stream << "virtual ";
	*Debug_stream << "virtual ";
    }
    switch (access_token)
    {
      case NONE:
	break;
      case PUBLIC:
	*output_stream << "public ";
	*Debug_stream << "public ";
	break;
      case PRIVATE:
	*output_stream << "private ";
	*Debug_stream << "private ";
	break;
      case PROTECTED:
	*output_stream << "protected ";
	*Debug_stream << "protected ";
	break;
      default:
	cerr << Me << ": Internal error in c_string_list::print(): ";
	cerr << "Invalid access_token! " << Me << " will abort!\n";
	exit(1);
    }
    *output_stream << string_ptr;
    *Debug_stream << string_ptr;
    if (seperator == CLCL) {
	*output_stream << "::";
	*Debug_stream << "::";
    }
    if (next != NULL && seperator == ',') {
	*output_stream << ", ";
	*Debug_stream << ", ";
    }
    for (c_string_list *temp_string_list = next; temp_string_list != NULL; 
	 temp_string_list = temp_string_list->next)
    {
	
	if (temp_string_list->virtual_flag == TRUE) {
	    *output_stream << "virtual ";
	    *Debug_stream << "virtual ";
	}
	switch (temp_string_list->access_token)
	{
	  case NONE:
	    break;
	  case PUBLIC:
	    *output_stream << "public ";
	    *Debug_stream << "public ";
	    break;
	  case PRIVATE:
	    *output_stream << "private ";
	    *Debug_stream << "private ";
	    break;
	  case PROTECTED:
	    *output_stream << "protected ";
	    *Debug_stream << "protected ";
	    break;
	  default:
	    cerr << Me << ": Internal error in c_string_list::print(): ";
	    cerr << "Invalid access_token! " << Me << " will abort!\n";
	    exit(1);
	}
	*output_stream << temp_string_list->get_string();
	*Debug_stream << temp_string_list->get_string();
	if (seperator == CLCL)  {
	    *output_stream << "::";
	    *Debug_stream << "::";
	}
	if (temp_string_list->next != NULL && seperator == ',')
	    *output_stream << ", ";
    }
}

/******************************************************************************
 *		
 *					Types
 *
 *****************************************************************************/

void type_element::print(ostream*, p_boolean, ostream*, int*)
{
    cerr << Me << " : type_element::print() : internal error! print";
    cerr << " member function called on an abstract base class!\n";
}

void type_element::print_type(ostream*)
{
    ;
}

void base_type::print(ostream *output_stream, p_boolean print_semicolon, 
		      ostream*, int*)
{
    PRINT_DEBUG("Printing a base type!\n");
    switch(type_element_token)
    {
      case INT:
	*output_stream << "int";
	break;
      case FLOAT:
	*output_stream << "float";
	break;
      case CHAR:
	*output_stream << "char";
	break;
      case SHORT:
	*output_stream << "short";
	break;
      case LONG:
	*output_stream << "long";
	break;
      case DOUBLE:
	*output_stream << "double";
	break;
      case SIGNED:
	*output_stream << "signed";
	break;
      case UNSIGNED:
	*output_stream << "unsigned";
	break;
      case STRUCT:
	if (type_name_id_ptr->csue_definition_ptr != NULL && print_body)
	    type_name_id_ptr->csue_definition_ptr->print(output_stream,
							 print_semicolon);
	else
	{
	    *output_stream << "struct ";
	    type_name_id_ptr->print(output_stream);
	    if (print_semicolon)
		*output_stream << ";";
   	}
	break;
      case CLASS:
	if (type_name_id_ptr->csue_definition_ptr != NULL && print_body)
	    type_name_id_ptr->csue_definition_ptr->print(output_stream,
							 print_semicolon);
	else
	{
	    *output_stream << "class ";
	    *Debug_stream << "class ";
	    type_name_id_ptr->print(output_stream);
	    if (print_semicolon)
		*output_stream << ";";
   	}
	break;
      case UNION:
	if (type_name_id_ptr->csue_definition_ptr != NULL && print_body)
	    type_name_id_ptr->csue_definition_ptr->print(output_stream,
							 print_semicolon);
	else
	{
	    *output_stream << "union ";
	    type_name_id_ptr->print(output_stream);
	    if (print_semicolon)
		*output_stream << ";";
   	}
	break;
      case ENUM:
	if (type_name_id_ptr->enum_definition_ptr != NULL)
	    type_name_id_ptr->enum_definition_ptr->print(output_stream);
	break;
      case TYPEDEFname:
	type_name_id_ptr->print(output_stream);
	break;
      case VOID:
	*output_stream << "void";
	break;
      default:
	cerr << Me << ": Internal error in base_type::print(): unknown type";
	cerr << " name token : " << type_element_token;
	cerr << ", " << Me << " will abort!\n";
	exit(-1);
    }
}

void base_type::print_type(ostream *output_stream)
{
    PRINT_DEBUG("Printing a base type!\n");
    switch(type_element_token)
    {
      case INT:
	*output_stream << "int";
	break;
      case FLOAT:
	*output_stream << "float";
	break;
      case CHAR:
	*output_stream << "char";
	break;
      case SHORT:
	*output_stream << "short";
	break;
      case LONG:
	*output_stream << "long";
	break;
      case DOUBLE:
	*output_stream << "double";
	break;
      case SIGNED:
	*output_stream << "signed";
	break;
      case UNSIGNED:
	*output_stream << "unsigned";
	break;
      case STRUCT:
	type_name_id_ptr->print(output_stream);
	break;
      case CLASS:
	type_name_id_ptr->print(output_stream);
	break;
      case UNION:
	type_name_id_ptr->print(output_stream);
	break;
      case ENUM:
	type_name_id_ptr->enum_definition_ptr->print(output_stream);
	break;
      case TYPEDEFname:
	type_name_id_ptr->print(output_stream);
	break;
      case VOID:
	*output_stream << "void";
	break;
      default:
	cerr << Me << ": Internal error in base_type::print_type(): unknown type";
	cerr << " name token : " << type_element_token;
	cerr << ", " << Me << " will abort!\n";
	exit(-1);
    }
}

void csue_type_element::print(ostream *output_stream, p_boolean print_semicolon,
			      ostream*, int*)
{
    PRINT_DEBUG("Printing a csue type element!\n");
    if (csu_definition_ptr != NULL)
	csu_definition_ptr->print(output_stream, print_semicolon);
    else
	enum_definition_ptr->print(output_stream);
}
 
void type_modifier::print(ostream *pre_declarator_stream,
			  p_boolean,
			  ostream *post_declarator_stream, 
			  int *previous_precedence)
{
    PRINT_DEBUG("Printing a type modifier!\n");
    switch(type_element_token)
    {
      case ARRAY:
	if (*previous_precedence < 1)
	{
	    *pre_declarator_stream << "(";
	    *post_declarator_stream << ")";
	}
	*post_declarator_stream << "[";
	if (size != NULL)
	    size->print(post_declarator_stream);
	*post_declarator_stream << "]";
	*previous_precedence = 1;
	break;
      case FUNCTION:
	PRINT_DEBUG("(a function)\n");
	if (*previous_precedence < 1)
	{
	    *pre_declarator_stream << "(";
	    *post_declarator_stream << ")";
	}
	*post_declarator_stream << "(";
        {
        p_boolean saveindent = indent;
	indent = FALSE;
	if (parameters != NULL)
	    parameters->print(post_declarator_stream);
	*post_declarator_stream << ")";
	indent = saveindent;
        }
	if (parameters != NULL && parameters->type_qualifier_list_ptr != NULL)
	    parameters->type_qualifier_list_ptr->print(post_declarator_stream);
	*previous_precedence = 1;
	break;
      case '*':
	*pre_declarator_stream << "*";
	if (previous_precedence != NULL)
	    *previous_precedence = 0;
        break;
      case '&':
	*pre_declarator_stream << "&";
	if (previous_precedence != NULL)
	    *previous_precedence = 0;
	break;
      case CONST:
	*pre_declarator_stream << "const";
	if (previous_precedence != NULL)
	    *previous_precedence = 0;
	break;
      case VOLATILE:
	*pre_declarator_stream << "volatile";
	if (previous_precedence != NULL)
	    *previous_precedence = 0;
	break;
      default:
	cerr << Me << ": Internal error in type_modifier::print(): unknown";
	cerr << " type_element_token!: " << Me << " will abort!\n";
	exit(-1);
    }    
}

void storage_class::print(ostream *output_stream, p_boolean, ostream*, int*)
{
    PRINT_DEBUG("Printing a storage class!\n");
    switch(type_element_token)
    {
      case TYPEDEF:
	*output_stream << "typedef";
	break;
      case EXTERN:
	*output_stream << "extern";
	*Debug_stream << "extern";
	break;
      case STATIC:
	*output_stream << "static";
	break;
      case AUTO:
	*output_stream << "auto";
	break;
      case REGISTER:
	*output_stream << "register";
	break;
      case FRIEND:
	*output_stream << "friend";
	break;
      case OVERLOAD:
	*output_stream << "overload";
	break;
      case INLINE:
	*output_stream << "inline";
	break;
      case VIRTUAL:
	*output_stream << "virtual";
	break;
#ifdef PROXIFY
      case PROXIABLE:
	*output_stream << "proxiable";
	*Debug_stream << "proxiable";
	break;
#endif
      default:
	cerr << Me << ": Internal error in storage_class::print(): unknown";
	cerr << " type_element_token!: " << Me << " will abort!\n";
	exit(-1);
    }    
}

void type::print(ostream * output_stream, p_boolean print_semicolon)
{
    PRINT_DEBUG("Printing a type!\n");
    type_element_ptr->print(output_stream, print_semicolon);
    if (next != NULL) {
	*output_stream << " ";
	*Debug_stream << " ";
    }
    for (type *temp_type = next; temp_type != NULL;temp_type = temp_type->next)
    {
	temp_type->get_type_element()->print(output_stream, print_semicolon);
	if (temp_type->next != NULL) {
	    *output_stream << " ";
	    *Debug_stream << " ";
	}
    }
}

void type::print_type(ostream *output_stream)
{
    PRINT_DEBUG("Print type printing a type!\n");

    // Print the base type.
    type *temp_type_ptr;
    for (temp_type_ptr = this; temp_type_ptr != NULL; 
	 temp_type_ptr = temp_type_ptr->next)
    {
	if (temp_type_ptr->type_element_ptr != NULL &&
	    temp_type_ptr->type_element_ptr->type_element_token != '*' &&
	    temp_type_ptr->type_element_ptr->type_element_token != '&')
	    temp_type_ptr->type_element_ptr->print_type(output_stream);
    }
    for (temp_type_ptr = this; temp_type_ptr != NULL; 
	 temp_type_ptr = temp_type_ptr->next)
    {
	if (temp_type_ptr->type_element_ptr != NULL && 
	    temp_type_ptr->type_element_ptr->type_element_token == '*')
	    { *output_stream << "*";
	      *Debug_stream << "*";}
	else
	    if (temp_type_ptr->type_element_ptr != NULL &&
		temp_type_ptr->type_element_ptr->type_element_token == '&')
		{*output_stream << "&";
		*Debug_stream << "&";}
    }    
}

void parameter::print(ostream *output_stream)
{
    PRINT_DEBUG("Printing a parameter!\n");
    if (parameter_token == ELLIPSIS)
	*output_stream << "...";
    if (parameter_declaration != NULL)
	parameter_declaration->print(output_stream, FALSE);
    if (parameter_initializer != NULL)
	parameter_initializer->print(output_stream);
}

void parameter_list::print(ostream *output_stream)
{
    PRINT_DEBUG("Printing a parameter list!\n");
    if (parameter_ptr->parameter_declaration == NULL)
	return;

    for (parameter_list *temp_parameter = this; temp_parameter != NULL; 
	 temp_parameter = temp_parameter->next)
    {
#ifdef PROXIFY
	if (Parameter_flag > 1)
	    Parameter_count++;
	Declarator_id_found = FALSE;
#endif
	temp_parameter->get_parameter()->print(output_stream);
#ifdef PROXIFY
	if ((Parameter_flag == 3) && !Declarator_id_found) {
	    *output_stream << " a" << Parameter_count;
	    *Debug_stream << " a" << Parameter_count;
	}
#endif
	if (temp_parameter->next != NULL)
	    *output_stream << ", ";
    }
}    

/******************************************************************************
 *
 *			     	Definitions
 *
 *****************************************************************************/

void definition::print(ostream*, p_boolean)
{
    cerr << Me << " : definition::print() : internal error! print";
    cerr << " member function called on an abstract base class!\n";
}

void function_definition::print(ostream *output_stream, p_boolean)
{
    PRINT_DEBUG("Printing a function definition!\n");
    *output_stream << spaces[tab_column];
    if (declaration_type != NULL)
    {
	declaration_type->print(output_stream);
	*output_stream << " ";
    }
    function_name->print(output_stream, TRUE);
    if (ctor_initializers != NULL)
	ctor_initializers->print(output_stream);
    if (body != NULL)
    {
	*output_stream << "\n";
	body->print(output_stream);
    }
    else
	*output_stream << ";";
    *output_stream << "\n";
}

void class_definition::print(ostream *output_stream, p_boolean print_semicolon)
{
    PRINT_DEBUG("Printing a class definition!\n");

#ifdef MPLFRONT
    if (backend_target_flag && identifier_ptr != NULL)
    {
	*output_stream << "class ";
	*Debug_stream << "class ";
	identifier_ptr->print(output_stream);
	*output_stream << " : public mentat_object\n";
	*Debug_stream << " : public mentat_object\n";
	*output_stream << "{\n";
	*output_stream << "  public:\n";
	*output_stream << "    ";
	identifier_ptr->print(output_stream);
	*output_stream << "() { set_object_name(&i_name, \"";
	identifier_ptr->print(output_stream);
	*output_stream << "\"); }\n";
	*output_stream << "};\n\n";
    }
#endif

    *output_stream << spaces[tab_column];

    switch(csu_token)
    {
      case CLASS:
	*output_stream << "class ";
	*Debug_stream << "class ";
	break;
      case STRUCT:
	*output_stream << "struct ";
	break;
      case UNION:
	*output_stream << "union ";
	break;
      default:
	cerr << Me << ": Internal error: class_definition";
	cerr << "::print(): unknown csu token: " << Me << " will abort!\n";
	exit(-1);
    }

#ifdef MPLFRONT
    if (backend_target_flag)
	*output_stream << "_mentat_";
#endif

    if (identifier_ptr != NULL)
	identifier_ptr->print(output_stream);

#ifdef MPLFRONT
    if (mentat_class_flag && !backend_target_flag)
	*output_stream << " : public mentat_object";
    else
#endif
	if (derivation_list_ptr != NULL)
	{
	    *output_stream << " : ";
	    derivation_list_ptr->print(',', output_stream);
	}
    
#ifdef MPLFRONT
    if (!mentat_class_flag || (mentat_class_flag && backend_target_flag))
    {
#endif
	if (declarations != NULL)
	{
	    *output_stream << "\n";
	    *output_stream << spaces[tab_column] << "{\n";
	    incr_tab();
	    declarations->print(output_stream);
	    decr_tab();
	    *output_stream << spaces[tab_column] << "}";
	}
#ifdef MPLFRONT
    }
    else
	{
	    *output_stream << "\n";
	    *output_stream << spaces[tab_column] << "{\n";
	    *output_stream << spaces[tab_column] << "  public:\n";
	    *output_stream << spaces[tab_column] << "    ";
	    *output_stream << identifier_ptr->id_string;
	    *output_stream << "() { set_object_name(" << "&i_name, \"";
	    *output_stream << identifier_ptr->id_string << "\"); }\n";
	    *output_stream << spaces[tab_column] << "}";
	}
#endif

    if (print_semicolon)
	*output_stream << ";\n";

#ifdef MPLFRONT
    if (backend_target_flag && identifier_ptr != NULL)
    {
	*output_stream << "\n";
	identifier_ptr->print(output_stream);
	*output_stream << " SELF;\n";
    }
#endif
}

void enum_definition::print(ostream *output_stream, p_boolean)
{
    PRINT_DEBUG("Printing an enum definition!\n");
    *output_stream << spaces[tab_column];
    *output_stream << "enum ";
    if (enum_id_ptr != NULL)
	enum_id_ptr->print(output_stream);
    if (enum_list_ptr != NULL)
    {
	*output_stream << " {\n";
	enum_list_ptr->print(output_stream);
	*output_stream << "\n}";
    }
    *output_stream << ";\n";
}

void definition_list::print(ostream *output_stream)
{
    PRINT_DEBUG("Printing a definition list!\n");
    if (definition_ptr != NULL)
	definition_ptr->print(output_stream);
    for (definition_list *temp_stmt_list = next; temp_stmt_list != NULL; 
	 temp_stmt_list = temp_stmt_list->next)
	temp_stmt_list->get_definition()->print(output_stream);
}

void declaration_access_indicator::print(ostream *output_stream, p_boolean)
{
    PRINT_DEBUG("Printing a declaration access indicator!\n");
    switch(access_token)
    {
      case PUBLIC:
	*output_stream << "public:";
	break;
      case PRIVATE:
	*output_stream << "private:";
	break;
      case PROTECTED:
	*output_stream << "protected:";
	break;
      default:
	cerr << Me << ": Internal error: declaration_access_indicator";
	cerr << "::print(): unknown acess token: " << Me << " will abort!\n";
	exit(-1);
    }
}

void external_definition::print(ostream *output_stream, p_boolean)
{
    PRINT_DEBUG("Printing an external declaration!\n");
    if (linkage_specifier != NULL)
    {
	*output_stream << "extern ";
	*Debug_stream << "extern ";
	linkage_specifier->print(output_stream);
	*output_stream << " ";
    }
    if (external_declaration != NULL)
	external_declaration->print(output_stream);
    else 
	if (translation_unit != NULL)
	{
	    *output_stream << "{\n";
	    translation_unit->print(output_stream);
	    *output_stream << "}";
	}
	else 
	    if (external_function_definition != NULL)
		external_function_definition->print(output_stream);
    if (definition_group != NULL)
	definition_group->print(output_stream);
    
    *output_stream << "\n";
}

#ifdef MPLFRONT
void code_gen_definition::print(ostream *output_stream, p_boolean)
{
    PRINT_DEBUG("Printing a code_gen definition!\n");
    *output_stream << code_gen_string << "\n";
}
#endif

void derivation_list::print(TOKEN seperator, ostream *output_stream)
{
    PRINT_DEBUG("Printing a derivation_list!\n");
    if (virtual_flag == TRUE)
	*output_stream << "virtual ";
    switch (access_specifier)
    {
      case NONE:
	break;
      case PUBLIC:
	*output_stream << "public ";
	break;
      case PRIVATE:
	*output_stream << "private ";
	break;
      case PROTECTED:
	*output_stream << "protected ";
	break;
      default:
	cerr << Me << ": Internal error in derivation_list::print(): ";
	cerr << "Invalid access_token! " << Me << " will abort!\n";
	exit(1);
    }
#ifdef MPLFRONT
    if (definition_ptr->backend_target_flag && seperator == CLCL)
	*output_stream << "_mentat_";
#endif
    *output_stream << definition_ptr->identifier_ptr->id_string;
    if (seperator == CLCL)
	*output_stream << "::";
    if (next != NULL && seperator == ',')
	*output_stream << ", ";
    for (derivation_list *derivation_list_ptr = next; 
	 derivation_list_ptr != NULL; 
	 derivation_list_ptr = derivation_list_ptr->next)
    {
	
	if (derivation_list_ptr->virtual_flag == TRUE)
	    *output_stream << "virtual ";
	switch (derivation_list_ptr->access_specifier)
	{
	  case NONE:
	    break;
	  case PUBLIC:
	    *output_stream << "public ";
	    break;
	  case PRIVATE:
	    *output_stream << "private ";
	    break;
	  case PROTECTED:
	    *output_stream << "protected ";
	    break;
	  default:
	    cerr << Me << ": Internal error in derivation_list::print(): ";
	    cerr << "Invalid access_token! " << Me << " will abort!\n";
	    exit(1);
	}
	*output_stream << 
	    derivation_list_ptr->definition_ptr->identifier_ptr->id_string;
	if (seperator == CLCL)
	    *output_stream << "::";
	if (derivation_list_ptr->next != NULL && seperator == ',')
	    *output_stream << ", ";
    }
}
