#ifdef __GNUC__
#pragma implementation
#endif

#ifdef PROXIFY

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: David W. Dykstra (dykstra@cs.uiuc.edu)
 * Hacker: Amitabh Dave (dave@cs.uiuc.edu)
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/******************************************************************************
 *
 * proxify.c
 *
 * modified from print.c
 *
 * print.c is   Copyright (c) 1990, 1991
 *              University of Virginia
 *              All Rights Reserved
 *
 *		This software is the property of the University of Virginia.
 *		Permission is granted to copy and distribute this software as
 *		long as this notice is intact on all copies.  Derived works 
 *  		must retain this notice.  This software is provided `as is'
 *		and without any express or implied warrenties.
 *
 *              Send problems & suggestions to:
 *		Ed Loyot (ecl2v@virginia.edu) or
 *              Andrew Grimshaw (grimshaw@cs.virginia.edu)
 *
 *
 *****************************************************************************/

char *Version = "Proxify++ version 0.1";

#ifdef SABER_CC
#include <libc.h>
#endif

#include <stddef.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include <unistd.h>

#include <stream.h>
#include "String.h"
#define ASSERT
#include "Assert.h"
#include <strstream.h>

#include "ProxyTable.h"
#include "AssocTable.h"

#include "common.h"
#include "tokens.h"
#include "syntax_tree.h"
#include "parser.tab.h"
#include "fdstream.h"

#define MAXCLASSES	1024
#define DUMMY

extern ostream *Debug_stream;

#ifdef  DEBUG_PROXIFY
#define PROXIFY_DEBUG(X) *Debug_stream << X
#else
#define PROXIFY_DEBUG(X)
#endif

char message_buffer[100];

ostream *Header_stream = &cout;
ostream *Table_stream = &cout;
ostream *Constructor_stream = &cout;
ostream *Stub_stream = &cout;
ostream *Pragma_stream = &cout;

AssocTable ClassHash( MAXCLASSES );

/*
 * The baseClass is the first proxiable class and the subBaseClass is the
 *   parent of that class.  The subBaseClass must not have another parent with
 *   any virtual or proxiable functions.  If the first proxiable class has no
 *   parent, set the subBaseClassName pointer to NULL.
 */
char *subBaseClassName = "Object";
char *baseClassName = "ProxiableObject";

char *currentClassName = NULL;
char *currentParentName = NULL;
char *currentMethodName = NULL;

class MethodDescriptor {
public:
    MethodDescriptor(identifier *, int, int);

    MethodDescriptor *prev;	/* pointer to previous method descriptor */
    char *className;		/* name of class method is first defined in */
    p_boolean isProxiable;	/* class is proxiable */
    identifier *method;		/* syntax sub-tree of description of method */
				/* (null if this is a constructor descriptor) */
    int methodNumber;		/* proxiable method number, -1 if unproxiable */
    int realMethodNumber;	/* virtual method number on real class */
    int constructorArgs;	/* number of arguments in constructor */
};

AssertionFailure::AssertionFailure(char const*, char*, char*, int)
{
	cerr << "Assertion failed at: " << this;
	exit(1);
}

AssertionFailure::~AssertionFailure()
{
}


MethodDescriptor *currentMethodList = NULL;
MethodDescriptor *subBaseMethodList = NULL;

MethodDescriptor::MethodDescriptor(identifier *meth, int proxiable,
				int realMethNumber)
{
    prev = currentMethodList;
    className = currentClassName;
    method = meth;
    isProxiable = FALSE; /* not known yet */
    constructorArgs = 0; /* not known yet */
    if (!proxiable)
	methodNumber = -1;
    else
    {
	methodNumber = 0;
	for (MethodDescriptor *p = prev; p != NULL; p = p->prev)
	{
	    /*
	     * can stop as soon as we hit a proxiable method (in order)
	     */
	    if (p->methodNumber != -1)
	    {
		methodNumber = p->methodNumber + 1;
		break;
	    }
	}
    }

    if (realMethNumber != -1)
	realMethodNumber = realMethNumber;
    else
    {
	realMethodNumber = 0;
	for (MethodDescriptor *p = prev; p != NULL; p = p->prev)
	{
	    /*
	     * have to go through whole list because realMethods can
	     *   possibly be in crazy orders
	     */
	    if (realMethodNumber <= p->realMethodNumber)
		realMethodNumber = p->realMethodNumber + 1;
	}
    }
}

p_boolean proxiableClass = FALSE;
p_boolean proxifyingMethod = FALSE;
char *currentType = NULL;
char *currentTypeModifier = NULL;
int currentParameterNumber = 0;
int currentConstructorNumber = -1;

char *retTypeModifiers[] = {
    "Star",
    "Ref",
};

void
badTypeModifier()
{
    if (currentType == NULL)
	return;

//     int status;
//     (void) ClassHash.lookup(currentType, status);
//     if (status == -1)
// 	/* Only classes need to be restricted */ 
// 	return;

    unsigned int value;
    
    if (!ClassHash.lookup(currentType,value))
	// Only classes need to be restricted
	return;

    cerr << Me << ": unsupported modifier combination on ";
    if (currentParameterNumber == 0)
	cerr << "return value of \n";
    else
	cerr << "parameter " << currentParameterNumber << " of\n";
    cerr << "  " << currentClassName << "::" << currentMethodName;
    cerr << "  (must have one and only one of ";
    if (currentParameterNumber == 0)
    {
	for (unsigned int i = 0; i < sizeof(retTypeModifiers)/sizeof(char *); i++)
	{
	    cerr << "'" << retTypeModifiers[i] << "', ";
	}
    }
    cerr << "'*' or '&')\n";
    currentType = NULL;
}

declaration_access_indicator *	Last_declaration_access_indicator = NULL;

p_boolean proxiable_method_printed;
p_boolean proxiable_constructor_printed;

/*
  Parameter_flag:
    0: normal print of parameter identifiers
    1: print no pure_virtual
    2: print no parameter identifiers
    3: choose "a1, a2, etc." for parameter identifiers
    2 & 3: print no initializers, and print a 'currentClassName::' for
	    function names instead of just the function name
 */
int Parameter_flag = 0;    
int Parameter_count = 0;


/* externs for variables in print.c */
extern char *spaces[];
extern int tab_column;
extern p_boolean indent;
extern void incr_tab();
extern void decr_tab();

/******************************************************************************
 *
 *				Initialization
 *
 *****************************************************************************/

char *proxify_name = NULL;

int Doall = 0;
p_boolean Output_version = FALSE;

fdostream *Null_stream;
fdostream *Real_Header_stream, *Real_Constructor_stream, *Real_Table_stream,
		*Real_Stub_stream,  *Real_Pragma_stream;

char *Hprefix = "H/";
char *Hsuffix = ".h";
char *Tprefix = "T/";
char *Tsuffix = "T.cc";
char *Cprefix = "C/";
char *Csuffix = "C.cc";
char *Sprefix = "S/";
char *Ssuffix = "S.cc";
char *Pprefix = "S/";
char *Psuffix = "P.cc";

char *Tinclude = "ProxyTable.h";

#define MAXFILES	1024

AssocTable Header_hash( MAXFILES );

#include <fcntl.h>

#ifdef CFRONT
extern "C" int unlink(char *);
extern "C" int open(char *, int, int);
extern "C" int close(int);
extern "C" int getpid();
extern "C" char * getenv(const char *);
#endif

char undotted_name[256];
char stub_filename[256];

void print_version(ostream *o)
{
    *o << "/* This file was generated by " << Version << " */\n\n";
}

void proxify_args(int argc, char **argv)
{
    if ((argc > 1) && (strcmp(argv[1], "-all") == 0))
    {
	Doall = 1;
	argc--;
	argv++;
    }

    if ((argc > 1) && (strcmp(argv[1], "-tableonly") == 0))
    {
	Doall = 2;
	argc--;
	argv++;
    }

    if ((argc > 1) && (strcmp(argv[1], "-version") == 0))
    {
	Output_version = TRUE;
	argc--;
	argv++;
    }

    if (argc != 2)
    {
	cerr << "usage: Proxify++ [-all|-tableonly] [-version] input_header_name\n";
	cerr << "  Reads from standard input\n";
	exit(1);
    }

    proxify_name = argv[1];

    strcpy(undotted_name, proxify_name);

    char *dot;
    if ((dot = rindex(undotted_name, '.')) != NULL)
	*dot = '\0';
    
    int fd;
    fd = open("/dev/null", O_WRONLY, 0666);
    if (fd == -1)
    {
	cerr << "Proxify++: cannot open /dev/null!\n";
	exit(1);
    }
    Null_stream = new fdostream(fd);
	
    char filename[256];

    if (Doall)
	Header_stream = Null_stream;
    else
    {
	void open_header(char *, p_boolean);
	open_header(proxify_name, TRUE);
    }
    Real_Header_stream = (fdostream*) Header_stream;
    
    filename[0] = '\0';
    if (!Doall)
	strcat(filename, Tprefix);
    strcat(filename, undotted_name);
    strcat(filename, Tsuffix);
    (void) unlink(filename);
    fd = open(filename, O_CREAT|O_WRONLY, 0666);
    if (fd == -1)
    {
	cerr << "Proxify++: cannot create " << filename << "!\n";
	exit(1);
    }
    Real_Table_stream = new fdostream(fd);
    Table_stream = Real_Table_stream;
    
    if (Doall != 2)
    {
	filename[0] = '\0';
	if (!Doall)
	    strcat(filename, Cprefix);
	strcat(filename, undotted_name);
	strcat(filename, Csuffix);
	(void) unlink(filename);
	fd = open(filename, O_CREAT|O_WRONLY, 0666);
	if (fd == -1)
	{
	    cerr << "Proxify++: cannot create " << filename << "!\n";
	    exit(1);
	}
	Real_Constructor_stream = new fdostream(fd);
	Constructor_stream = Real_Constructor_stream;

	filename[0] = '\0';
	if (!Doall)
	    strcat(filename, Pprefix);
	strcat(filename, undotted_name);
	strcat(filename, Psuffix);
	(void) unlink(filename);
	fd = open(filename, O_CREAT|O_WRONLY, 0666);
	if (fd == -1)
	{
	    cerr << "Proxify++: cannot create " << filename << "!\n";
	    exit(1);
	}
	Real_Stub_stream = new fdostream(fd);
	Stub_stream = Real_Stub_stream;
        strcpy(stub_filename, filename);

	filename[0] = '\0';
	if (!Doall)
	    strcat(filename, Sprefix);
	strcat(filename, undotted_name);
	strcat(filename, Ssuffix);
	(void) unlink(filename);
	fd = open(filename, O_CREAT|O_WRONLY, 0666);
	if (fd == -1)
	{
	    cerr << "Proxify++: cannot create " << filename << "!\n";
	    exit(1);
	}
	Real_Pragma_stream = new fdostream(fd);
	Pragma_stream = Real_Pragma_stream;

//	*Stub_stream << "#include \"" << filename <<"\"\n"; 
	if (Doall)
	{
	    print_version(Stub_stream);
	    print_version(Constructor_stream);
	}

    }
    else
    {
	Real_Stub_stream = (fdostream*) Stub_stream = Null_stream;
	Real_Pragma_stream = (fdostream*) Pragma_stream = Null_stream;
	Real_Constructor_stream = (fdostream*) Constructor_stream = Null_stream;
    }

    char *deb;

    if (((deb = getenv("YYDEBUG")) == NULL) || (*deb < '2'))
	Debug_stream = Null_stream;
}

int Header_level = 0;

class IncludeList {
public:
    IncludeList(IncludeList *, char *);
    IncludeList *next;
    char *name;
};

IncludeList::IncludeList(IncludeList *nxt, char *nm)
{
    next = nxt;
    name = nm;
}


HeaderDescript::HeaderDescript()
{
    lowest_level = Header_level;
    include_constructor = TRUE;
    include_stub = TRUE;
    include_table = TRUE;
    persistentFlag = 0;
};

HeaderDescript *currentHead = NULL;

IncludeList *allFiles = NULL;

int Header_fd = -1;

void close_header(char *new_name)
{
    if (Header_fd != -1)
    {
	delete Header_stream;
	Header_stream = Null_stream;
	close(Header_fd);
	Header_fd = -1;
    }
    if (new_name != 0)
    {
// 	int status;
// 	currentHead = (HeaderDescript *) Header_hash.lookup(new_name, status);
// 	if (status == -1)
// 	{
// 	    cerr << "Proxify++ error: cannot lookup " << new_name
// 					    << " in hash table\n";
// 	    exit(1);
// 	}
	unsigned int value;
	if (Header_hash.lookup(new_name, value))
	    currentHead = (HeaderDescript *) value;
	else
	{
	    cerr << "Proxify++ error: cannot lookup " << new_name
					    << " in hash table\n";
	    exit(1);
	}
    }
}

void open_header(char *header_name, p_boolean new_file)
{
    char filename[256];
    char *dot;

    if (header_name == NULL)
	return;

    strcpy(undotted_name, header_name);
    if ((dot = rindex(undotted_name, '.')) != NULL)
	*dot = '\0';

    if (Doall != 2)
    {
	strcpy(filename, Hprefix);
	strcat(filename, undotted_name);
	strcat(filename, Hsuffix);
	if (new_file)
	    (void) unlink(filename);
	close_header(0);
	Header_fd = open(filename, O_CREAT|O_WRONLY|O_APPEND, 0666);
	if (Header_fd == -1)
	{
	    perror("");
	    cerr << "Proxify++: cannot open " << filename << "!\n";
	    exit(1);
	}
	Header_stream = new fdostream(Header_fd);
    }

    if (new_file)
    {
	print_version(Header_stream);
	*Header_stream << "#ifndef " << undotted_name << "_h\n";
	*Header_stream << "#define " << undotted_name << "_h\n";
	*Header_stream << "#ifdef __GNUC__\n";
	*Header_stream << "#pragma interface\n";
	*Header_stream << "#endif\n";
	*Header_stream << "#include \"NewAndDelete.h\"\n";
	
// 	if (Header_hash.bind(header_name,
// 			(unsigned int) new HeaderDescript()) == -1)
// 	{
// 	    cerr << "Proxify++ error: cannot bind " << header_name 
// 					    << " into hash table\n";
// 	    exit(1);
// 	}
	unsigned int value;
	if (Header_hash.lookup(header_name,value))
	{
	    cerr << "Proxify++ error: " << header_name 
		<< " exists with value " << value << "\n";
	    exit(1);
	}
	else
	    Header_hash.bind(header_name, (unsigned int) new HeaderDescript());
	
	allFiles = new IncludeList(allFiles, header_name);
    }

//     int status;
//     currentHead = (HeaderDescript *) Header_hash.lookup(header_name, status);
//     if (status == -1)
//     {
// 	cerr << "Proxify++ error: cannot lookup " << header_name
// 					<< " in hash table\n";
// 	exit(1);
//     }
    unsigned int value;
    if (!Header_hash.lookup(header_name, value))
    {
	cerr << "Proxify++ error: cannot lookup " << header_name
					<< " in hash table\n";
	exit(1);
    }
    currentHead = (HeaderDescript *) value;
}

void close_headers()
{
    for (IncludeList *list = allFiles; list != NULL; list = list->next)
    {
	open_header(list->name, FALSE);

	if ( Output_version && currentHead->lowest_level <= 1 ) {
	    *Header_stream << "#include \"VersionInstaller.h\"\n";
	    *Header_stream << "REQUIREDVERSION( " << undotted_name << " );\n";
	}
	*Header_stream << "#endif " << undotted_name << "_h\n";
    }
    close_header(0);
    *Pragma_stream << "#include \"" << stub_filename <<"\"\n"; 

}

void proxify_exit(int errcode)
{
    /* flush the output streams */
    if (!Doall)
	Header_stream = Real_Header_stream;
    close_headers();
    delete Real_Table_stream;
    if (Real_Constructor_stream != Null_stream)
	delete Real_Constructor_stream;
    if (Real_Stub_stream != Null_stream)
	delete Real_Stub_stream;
    if (Real_Pragma_stream != Null_stream)
	delete Real_Pragma_stream;
    _exit(errcode);
}

extern char program_name[];
p_boolean in_proxify_source = TRUE;
char *in_proxify_include = NULL;

void proxify_line(p_boolean firstline)
{
    char *p;

    if ((p = rindex(program_name, '/')) == NULL)
	p = program_name;
    else
	p++;
    if (Doall)
    {
	static char *last_p;

	char *current_include = in_proxify_include;
	in_proxify_include = new char[strlen(p) + 1];
	strcpy(in_proxify_include, p);
	p = in_proxify_include;

	if (firstline)
	{
	    if (Header_stream == Null_stream)
		open_header(current_include, FALSE);

	    //*Header_stream << "#ifdef __GNUC__\n";
	    //*Header_stream << "#endif\n";
	    *Header_stream << "#include \"" << p << "\"\n";

// 	    int status;
// 	    (void) Header_hash.lookup(p, status);
// 	    if (status == -1)
// 		open_header(p, TRUE);
// 	    else
// 	    {
// 		close_header(p);
// 		if (Header_level < currentHead->lowest_level)
// 		    currentHead->lowest_level = Header_level;
// 	    }
	    unsigned int value;
	    if (!Header_hash.lookup(p, value))
		open_header(p, TRUE);
	    else
	    {
		close_header(p);
		if (Header_level < currentHead->lowest_level) {
		    currentHead->lowest_level = Header_level;
/*                    if ((currentHead->lowest_level == 1) && (Stub_stream == Real_Stub_stream)){
		      if(Stub_stream == Real_Stub_stream) {
                        *Stub_stream << "#ifdef __GNUC__\n";
                        *Stub_stream << "#pragma implementation \"" << undotted_name <<".h\"\n";
                        *Stub_stream << "#endif\n";
                    }
*/
                }

	    }
	    
	    Header_level++;
	}
	else
	{
	    close_header(p);

	    if (strcmp(p, last_p) != 0)
		--Header_level;
	}

	last_p = p;
    }
    else
    {
	if (strcmp(p, proxify_name) == 0)
	    in_proxify_source = TRUE;
	else
	{
	    if (in_proxify_source)
	    {
		in_proxify_include = new char[strlen(p) + 1];
		strcpy(in_proxify_include, p);
	    }
	    in_proxify_source = FALSE;
	}
    }
}

/******************************************************************************
 *
 *				Statements
 *
 *****************************************************************************/

void statement::proxify()
{    
    cerr << Me << ": statement::proxify() : internal error! proxify";
    cerr << " member function called on an abstract base class!\n";
}

void statement_list::proxify()
{
    PROXIFY_DEBUG("Proxifying a statement list!\n");
    for (statement_list *temp_stmt_list = this; temp_stmt_list != NULL; 
	 temp_stmt_list = temp_stmt_list->next)
    {
	if (temp_stmt_list->get_statement() != NULL)
	    temp_stmt_list->get_statement()->proxify();
    }
}

/******************************************************************************
 *
 *				Declarations
 *
 *****************************************************************************/

void  declarator::proxify()
{
    PROXIFY_DEBUG("Proxifying a declarator! ");
    if ((declarator_id != NULL) && (currentMethodName == NULL))
    {
	static char id_buf[100];
	ostrstream id_stream(id_buf, 100);
	for (int i = 0; i < 100; i++)
	    id_buf[i] = '\0';
	declarator_id->print(&id_stream);
	PROXIFY_DEBUG("(method " << id_buf << ")\n");
	currentMethodName = id_buf;
    }
    else
	PROXIFY_DEBUG("\n");
    if (declarator_type != NULL)
    {
	for (type *type_ptr = declarator_type;
	     type_ptr != NULL; 
	     type_ptr = type_ptr->get_next())
	{
	    type_ptr->get_type_element()->proxify();
	}
    }

}

void declarator_list::proxify()
{
    PROXIFY_DEBUG("Proxifying a declarator list!\n");

    for (declarator_list *temp_decl_list = this; temp_decl_list != NULL; 
	 temp_decl_list = temp_decl_list->next)
    {
	temp_decl_list->get_declarator()->proxify();
    }
}

p_boolean print_method(type *declaration_type, declarator *method_declarator)
{
    if (Last_declaration_access_indicator != NULL) 
    {
	Last_declaration_access_indicator->print(Header_stream);
	Last_declaration_access_indicator = NULL;
	*Header_stream << "\n";
    }

    *Header_stream << spaces[tab_column];

    p_boolean saveindent = indent;
    indent = FALSE;

    /*
     * First the declaration in the header file.
     */
    identifier *method_id = method_declarator->declarator_id;

    p_boolean constructorFlag = FALSE;
    if ((strcmp(method_id->id_string, currentClassName) == 0) &&
						!method_id->destructor_flag)
	constructorFlag = TRUE;

    /* make the generated methods also be proxiable */
    *Header_stream << "proxiable ";

    if (!constructorFlag)
    {
	proxiable_method_printed = TRUE;


	if ((declaration_type == NULL) || 
	  (declaration_type->get_type_element()->type_element_token != VIRTUAL))
	{
	    /* doesn't start with virtual */
	    *Header_stream << "virtual ";;
	}
    }
    else
	proxiable_constructor_printed = TRUE;

    if (declaration_type != NULL)
    {
	declaration_type->print(Header_stream, FALSE);
	if (method_declarator != NULL)
	    *Header_stream << " ";
    }
    Parameter_flag = 1;
    method_declarator->print(Header_stream);
    Parameter_flag = 0;
    *Header_stream << ";\n";
    indent = saveindent;

    if (currentHead->lowest_level > 1)
    {
	/*
	 * no stubs for files included within includes only
	 */
	return constructorFlag;
    }

    if (constructorFlag)
    {
	/*
	 * constructor
	 */
	PROXIFY_DEBUG("Constructor detected!\n");

	if (currentHead->include_constructor &&
		(Constructor_stream == Real_Constructor_stream))
	{
	    currentHead->include_constructor = FALSE;
	    if (!Doall)
		print_version(Constructor_stream);
#ifdef NEVER
	    *Constructor_stream << "#ifdef __GNUC__\n";
	    *Constructor_stream << "#pragma implementation \"" 
		<< undotted_name << ".h\"\n";
	    *Constructor_stream << "#endif\n";
#endif
	    *Constructor_stream << "#include \"" << undotted_name << ".h\"\n\n";
	}

	Parameter_flag = 2;
	indent = FALSE;
	method_declarator->print(Constructor_stream);
	indent = saveindent;
	Parameter_flag = 0;
	Parameter_count = 0;

	MethodDescriptor *m = currentMethodList;
	while ((m != NULL) && ((m->method != NULL) ||
		    (strcmp(m->className, currentClassName) == 0)))
	    m = m->prev;
	if ((m != NULL) && (m->constructorArgs > 0))
	{
	    *Constructor_stream << " : " << currentParentName << " (0";;
	    for (int i = 1; i < m->constructorArgs; i++)
		*Constructor_stream << ", 0";
	    *Constructor_stream << ")";
	}
	*Constructor_stream << " { }\n\n";

	return TRUE;
    }

    /*
     * Now the stub in the stub file
     */
    if ((currentHead->include_stub) && (Stub_stream == Real_Stub_stream))
    {
	currentHead->include_stub = FALSE;
	if (!Doall)
	    print_version(Stub_stream);
//	if (!Doall) {	
		*Pragma_stream << "#ifdef __GNUC__\n";
		*Pragma_stream << "#pragma implementation \"" << undotted_name <<".h\"\n";
		*Pragma_stream << "#endif\n";
//	}

	*Stub_stream << "#include \"" << undotted_name << ".h\"\n\n";

	*Stub_stream << "#include \"ClassInstaller.h\"\n";

/* Modification to handle PersistentClass class */
//	cout << "Printing COPY statement for " << currentClassName <<
//		", persistentFlag == " << currentHead->persistentFlag << "\n" ;
        if (!currentHead->persistentFlag)
	*Stub_stream << "COPYCLASS(\"" << currentClassName << "\"," <<
						currentClassName << ",";
	else {
        *Stub_stream << "COPYPERSISTENTCLASS(\"" << currentClassName << "\"," <<
                                               currentClassName << ",";
        }
	if (currentParentName == NULL)
	    *Stub_stream << "Null";
	else
	    *Stub_stream << currentParentName;
	*Stub_stream << ");\n\n";
    }
    currentHead->persistentFlag = 0;

    indent = FALSE;
    if (declaration_type != NULL)
    {
	if (declaration_type->get_type_element()->type_element_token == VIRTUAL)
	{
	    if (declaration_type->get_next() != NULL)
		declaration_type->get_next()->print(Stub_stream, FALSE);
	}
	else
	    declaration_type->print(Stub_stream, FALSE);
	*Stub_stream << " ";
    }


    Parameter_flag = 3;
    method_declarator->print(Stub_stream);
    Parameter_flag = 0;
    indent = saveindent;
    int arg_count = Parameter_count;
    Parameter_count = 0;

    *Stub_stream << " {\n";
    *Stub_stream << "    ";

    if (method_id->destructor_flag)
    {
	/*
	 * destructor
	 */
	*Stub_stream << "}\n\n";
	return FALSE;
    }
	
    *Stub_stream << "return ";

    *Stub_stream << "((" << currentClassName << " *)" <<
				" (ProxiableObject *) _real)->";
    method_id->print(Stub_stream);
    *Stub_stream << "(";

    for (int i = 0; i < arg_count; i++) {
	if (i > 0)
	    *Stub_stream << ", ";
	*Stub_stream << "a" << i + 1;
    }
    *Stub_stream << ");";

    *Stub_stream << "\n}\n\n";

    return FALSE;
}

void proxify_method(type *declaration_type, declarator * method_declarator)
{
    p_boolean proxiable = FALSE;
    p_boolean constructor = FALSE;

    if ((declaration_type != NULL) && 
	(declaration_type->get_type_element()->type_element_token == PROXIABLE))
    {
	PROXIFY_DEBUG("(proxiable)\n");

	proxiable = TRUE;

	declaration_type = declaration_type->get_next();

	constructor = print_method(declaration_type, method_declarator);
    }
    else
	PROXIFY_DEBUG("(non-proxiable)\n");

    identifier *method_id = method_declarator->declarator_id;

    if (!constructor)
    {

	/*
	 * Find out if this is overloading a parent method
	 */

	int realMethodNumber = -1;

	for (MethodDescriptor *descript = currentMethodList; descript != NULL;
				descript = descript->prev)
	{
	    /*
	     * Just need to compare function name and parameters, not return
	     *   type; the C++ compiler would have already flagged it as an
	     *   error if the return types do not match.
	     */

	    if (descript->method == NULL)
		/* constructor descriptor */
		continue;

	    if ((method_id->destructor_flag &&
				descript->method->destructor_flag) ||
	       (!method_id->destructor_flag &&
				!descript->method->destructor_flag &&
		(strcmp(method_id->id_string,
				descript->method->id_string) == 0) &&
		    (method_id->type_ptr->compare(descript->method->type_ptr))))
	    {
		PROXIFY_DEBUG(" overloading real method " <<
				    descript->realMethodNumber << "!\n");
		
		if (proxiable && (descript->methodNumber == -1))
		{
		    /*
		     * Overloading a method that was virtual but not proxiable
		     *   before this class.  Go ahead anyway, keeping track of
		     *   the real method number.
		     */
		    realMethodNumber = descript->realMethodNumber;
		    break;
		}

		return;
	    }
	}

#ifdef DUMMY
	if (strcmp(method_id->id_string, "_dummy") == 0)
	    /* use the first method; nobody should be calling this anyway */
	    realMethodNumber = 0;
#endif

	if (proxiableClass && !proxiable && (realMethodNumber == -1)
		    && (currentHead->lowest_level <= 1) && (Doall != 2))
	{
	    cerr << Me << ": warning: non-proxiable member function " <<
		    method_id->id_string << " in proxiable class " <<
			currentClassName << "\n";
	}

	if ((realMethodNumber != -1) || ((declaration_type != NULL) &&
	    (declaration_type->get_type_element()->type_element_token == VIRTUAL)))
	{
	    currentMethodList = new MethodDescriptor(method_id, proxiable,
							    realMethodNumber);

	    if (!proxiable)
		return;
	}
	else
	{
	    if (proxiable)
		cerr << Me << ": error on " <<
			currentClassName << "::" << currentMethodName << 
			    "(): proxiable methods must be virtual\n"; 
	    return;
	}
    }
    else
    {
	/*
	 * A proxiable constructor
	 */
	currentMethodList = new MethodDescriptor(0, 0, 0);

	currentConstructorNumber++;

    }

    proxifyingMethod = TRUE;

    currentMethodName = NULL;
    /* function name and parameters */
    method_declarator->proxify();

    int nParams = currentParameterNumber;
    currentParameterNumber = 0;

    if (declaration_type != NULL)
	declaration_type->proxify(); /* return type */

    proxifyingMethod = FALSE;

    if ((currentHead->include_table) && (Table_stream == Real_Table_stream))
    {
	currentHead->include_table = FALSE;
	print_version(Table_stream);
	*Table_stream << "#include \"" << Tinclude << "\"\n\n";
    }

    *Table_stream << "MethodTableEntry " << currentClassName;

    if (!constructor)
    {
	*Table_stream << "Method" << currentMethodList->methodNumber
	    		<< "TableEntry = { /* " <<
			    currentMethodList->method->id_string << " */\n";
	if (currentType == NULL)
	{
	    *Table_stream << "  0,\n";
	    *Table_stream << "  0,\n";
	}
	else if (currentTypeModifier == NULL)
	{
	    badTypeModifier();
	    *Table_stream << "  0,\n";
	    *Table_stream << "  0,\n";
	}
	else
	{
	    if (strcmp(currentTypeModifier, "Star") == 0)
	    {
		cerr << Me << ": use of Star as return type on method " <<
		    currentMethodList->methodNumber << " of class " <<
			currentClassName << " is not supported; use Ref\n";
	    }
	    *Table_stream << "  \"" << currentType << "\",\n";
	    *Table_stream << "  '" << *currentTypeModifier << "',\n";
	}

	*Table_stream << "  " << currentMethodList->realMethodNumber << ",\n";
    }
    else
    {
	*Table_stream << "Constructor" << currentConstructorNumber
			<< "TableEntry = {\n";
	*Table_stream << "  \"" << currentClassName << "\",\n";
	*Table_stream << "  '*',\n";
	*Table_stream << "  0,\n";

	currentMethodList->constructorArgs = nParams;
    }

    *Table_stream << "  " << nParams << ",\n";

    if (nParams > MAXPARAMS)
    {
	cerr << "Warning: more than " << MAXPARAMS << " parameters on ";
	if (constructor)
		cerr << " constructor " << currentConstructorNumber;
	else
		cerr << " method " << currentMethodList->methodNumber;
	cerr << " in class " << currentClassName << "\n";
    }

    if (nParams == 0)
	*Table_stream << "  0,\n";
    else if (constructor)
	*Table_stream << "  " << currentClassName << "Constructor" <<
	    currentConstructorNumber << "ParamClasses,\n";
    else
	*Table_stream << "  " << currentClassName << "Method" <<
	    currentMethodList->methodNumber << "ParamClasses,\n";

    *Table_stream << "};\n\n";
}

void declaration::check_source()
{
    /*
     * This function is called when something is about to be output to
     *   a header file. 
     */

    if (Doall)
	open_header(proxify_include, FALSE);
    else
    {
	if (proxify_source)
	{
	    Header_stream = Real_Header_stream;
	    Table_stream = Real_Table_stream;
	    Constructor_stream = Real_Constructor_stream;
	    Stub_stream = Real_Stub_stream;
	    Pragma_stream = Real_Pragma_stream;
	    currentHead->lowest_level = 0;
	}
	else
	{
	    Header_stream = Null_stream;
	    Table_stream = Null_stream;
	    Constructor_stream = Null_stream;
	    Stub_stream = Null_stream;
	    Pragma_stream = Null_stream;
	    currentHead->lowest_level = 2;

	    static char *last_proxify_include = NULL;

	    if (proxify_include != NULL)
	    {
		if ((last_proxify_include == NULL) ||
		    (strcmp(proxify_include, last_proxify_include) != 0))
		
		    *Real_Header_stream << "#include \"" << proxify_include << "\"\n";
		
		last_proxify_include = proxify_include;
	    }
	}
    }
}

void variable_declaration::proxify()
{
    PROXIFY_DEBUG("Proxifying a variable declaration! ");

    if (proxifyingMethod == TRUE)
    {
	PROXIFY_DEBUG("\n");
	if (declaration_type != NULL)
	    declaration_type->proxify();
	if (declarators != NULL)
	    declarators->proxify();
	return;
    }
    
    type * typ;
    if (declaration_type != NULL)
    {
	TOKEN tok = declaration_type->get_type_element()->type_element_token;
	if ((tok == TYPEDEF) || (tok == EXTERN) || ((tok == STATIC) && 
	    ((typ = declaration_type->get_next()) != NULL) &&
		typ->get_type_element()->type_element_token == CONST) || (tok == CONST))
	{
	    PROXIFY_DEBUG("\n");
	    check_source();
	    print(Header_stream);
	    *Header_stream << "\n";
	    return;
	}
    }

    if ((declaration_type == NULL) || (declarators == NULL) ||
						(currentClassName == NULL))
    {
	PROXIFY_DEBUG("(ignoring)\n");
	return;
    }

    /*
     * know there will be only one declarator in the declarator list
     */
    
    typ = declarators->get_declarator()->declarator_id->type_ptr;
    if ((typ == NULL) || !typ->function_declaration())
    {
	PROXIFY_DEBUG("(ignoring, not a function)\n");
	return;
    }

    proxify_method(declaration_type, declarators->get_declarator());

}

void csue_declaration::proxify()
{
    PROXIFY_DEBUG("Proxifying a csue declaration!");
    if (currentClassName != NULL)
    {
	PROXIFY_DEBUG(" (ignoring)\n");
	return;
    }
    PROXIFY_DEBUG("\n");

    check_source();

    proxiableClass = FALSE;

    csue_type_ptr->proxify();

    *Header_stream << "\n";
}

void member_declaration::proxify()
{
    /*
     * constructor and destructor definitions go through here
     */
    PROXIFY_DEBUG("Proxifying a member declaration! ");
    if (var_declaration != NULL)
	PROXIFY_DEBUG("(ignoring)\n");
    else
	if (func_definition != NULL)
	    proxify_method(func_definition->declaration_type,
					func_definition->function_name);
        else
	    PROXIFY_DEBUG("(ignoring)\n");
}

/******************************************************************************
 *		
 *					Types
 *
 *****************************************************************************/

void type_element::proxify()
{
    cerr << Me << ": type_element::proxify() : internal error! proxify";
    cerr << " member function called on an abstract base class!\n";
}

p_boolean print_space = FALSE;

void base_type::proxify()
{
    PROXIFY_DEBUG("Proxifying a base type!\n");
    if (proxifyingMethod == TRUE)
    {
	char *s;
	static char id_buf[100];
	if (type_name_id_ptr != NULL)
	{
	    s = type_name_id_ptr->id_string;
	    if (currentParameterNumber == 0)
	    {
		/*
		 * Check for retTypeModifiers
		 */
		int l = strlen(s);
		if (l > 100)
		{
		    cerr << Me << ": error! identifier > 100 bytes\n";
		    s[100-1] = '\0';
		    l = 100-1;
		}
		for (unsigned int i = 0; i < sizeof(retTypeModifiers)/sizeof(char *); i++)
		{
		    int m = strlen(retTypeModifiers[i]);
		    if (strcmp(retTypeModifiers[i], &s[l - m]) == 0)
		    {
			strcpy(id_buf, s);
			s = id_buf;
			s[l - m] = '\0';
			if (currentTypeModifier == NULL)
			    currentTypeModifier = retTypeModifiers[i];
			else
			    badTypeModifier();
			break;
		    }
		}
	    }
	}
	else
	{
	    ostrstream id_stream(id_buf, 100);
            for (int i = 0; i < 100; i++)
                id_buf[i] = '\0';
	    print(&id_stream);
	    s = id_buf;
	}

	currentType = s;
    }
    else
    {
	if (type_element_token == CLASS)
	{
	    /* add the class name to the table if it's not there */
// 	    int status;
// 
// 	    (void) ClassHash.lookup(type_name_id_ptr->id_string, status);
// 	    if (status == -1)
// 		(void) ClassHash.bind(type_name_id_ptr->id_string, 0);
	    unsigned int value;
	    if (!ClassHash.lookup(type_name_id_ptr->id_string, value))
		(void) ClassHash.bind(type_name_id_ptr->id_string, 0);
	}

	if (((type_element_token == CLASS) || (type_element_token == UNION) ||
				(type_element_token == STRUCT)) &&
		(type_name_id_ptr->csue_definition_ptr != NULL) && print_body )
	    type_name_id_ptr->csue_definition_ptr->proxify();
	else
	{
	    print(Header_stream);
	    if (print_space)
		*Header_stream << ' ';;
	}
    }
}

void type_modifier::proxify()
{
    PROXIFY_DEBUG("Proxifying a type modifier! ");
    switch(type_element_token)
    {
      case FUNCTION:
	PROXIFY_DEBUG("(a function)\n");
	parameters->proxify();
	if (parameters != NULL && parameters->type_qualifier_list_ptr != NULL)
	    parameters->type_qualifier_list_ptr->proxify();
	break;
      case '*':
	PROXIFY_DEBUG("(*)\n");
	if (currentTypeModifier == NULL)
	    currentTypeModifier = "*";
	else
	    badTypeModifier();
	break;
      case '&':
	PROXIFY_DEBUG("(&)\n");
	if (currentTypeModifier == NULL)
	    currentTypeModifier = "&";
	else
	    badTypeModifier();
	break;
      default:
	PROXIFY_DEBUG("(ignoring)\n");
    }
}

void storage_class::proxify()
{
    PROXIFY_DEBUG("Proxifying a storage class!\n");
    if (type_element_token == PROXIABLE)
	proxiableClass = TRUE;
}

void type::proxify()
{
    PROXIFY_DEBUG("Proxifying a type!\n");
    for (type *temp_type = this; temp_type != NULL;temp_type = temp_type->next)
    {
	if (temp_type->next != NULL)
	    print_space = FALSE;
	else
	    print_space = TRUE;
	temp_type->get_type_element()->proxify();
    }
}

void parameter::proxify()
{
    PROXIFY_DEBUG("Proxifying a parameter!\n");
    if (parameter_token == ELLIPSIS)
	cerr << Me << ": error! elipsis not supported!\n";
    
    if (parameter_declaration != NULL)
	parameter_declaration->proxify();
}

void parameter_list::proxify()
{
    PROXIFY_DEBUG("Proxifying a parameter list! ");

    static p_boolean proxifying_parameter_list = FALSE;

    if (proxifying_parameter_list)
    {
	PROXIFY_DEBUG("(but already doing one, ignoring)\n");
	return;
    }

    currentParameterNumber = 0;
    if (parameter_ptr->parameter_declaration == NULL)
    {
	PROXIFY_DEBUG("(empty)\n");
	currentTypeModifier = NULL;
	currentType = NULL;
	return;
    }
    PROXIFY_DEBUG("\n");

    if ((currentHead->include_table) && (Table_stream == Real_Table_stream))
    {
	currentHead->include_table = FALSE;
	print_version(Table_stream);
	*Table_stream << "#include \"" << Tinclude << "\"\n\n";
    }

    proxifying_parameter_list = TRUE;

    if (currentMethodList->method == NULL)
	*Table_stream << "char *" << currentClassName << "Constructor" <<
	    currentConstructorNumber << "ParamClasses[] = {\n";
    else
	*Table_stream << "char *" << currentClassName << "Method" <<
	    currentMethodList->methodNumber << "ParamClasses[] = {\n";

    for (parameter_list *temp_parameter = this; temp_parameter != NULL; 
	 temp_parameter = temp_parameter->next)
    {
	currentParameterNumber++;
	currentTypeModifier = NULL;
	currentType = NULL;

	temp_parameter->get_parameter()->proxify();

	if (currentType != NULL)
	{
	    if (currentTypeModifier == NULL)
	    {
		badTypeModifier();
		*Table_stream << "  0,\n";
	    }
	    else
	    {
		if (strcmp(currentTypeModifier, "Star") == 0)
		{
		    cerr << Me << ": use of Star as parameter type on method "
			<< currentMethodList->methodNumber << " of class " <<
			    currentClassName << " is discouraged; use '*'\n";
		}
		*Table_stream << "  \"" << currentType << "\",\n";
	    }
	}
	else
	    *Table_stream << "  0,\n";

    }

    *Table_stream << "};\n\n";

    currentTypeModifier = NULL;	/* have to do this here because return */
				/*   type is processed next            */
    currentType = NULL;

    proxifying_parameter_list = FALSE;
}

/******************************************************************************
 *
 *			     	Definitions
 *
 *****************************************************************************/

void definition::proxify()
{
    cerr << Me << ": definition::proxify() : internal error! proxify";
    cerr << " member function called on an abstract base class!\n";
}

void function_definition::proxify()
{
    PROXIFY_DEBUG("Proxifying a function definition! (ignoring)\n");
}

void printMethodExterns(MethodDescriptor *descript)
{
    if (descript == NULL)
	return;
    
    printMethodExterns(descript->prev);

    if ((descript->methodNumber != -1) &&
	    (strcmp(descript->className, currentClassName) != 0))
	*Table_stream << "extern MethodTableEntry " << descript->className <<
		"Method" << descript->methodNumber << "TableEntry;\n";
}

void printMethodList(MethodDescriptor *descript)
{
    if (descript == NULL)
	return;
    
    printMethodList(descript->prev);

    if (descript->methodNumber != -1)
	*Table_stream << "  &" << descript->className << "Method" <<
		    descript->methodNumber << "TableEntry,\n";
}

void printConstructorList(int constructorNumber)
{
    if (constructorNumber < 0)
	return;
    
    printConstructorList(constructorNumber - 1);

    *Table_stream << "  &" << currentClassName << "Constructor" <<
		constructorNumber << "TableEntry,\n";
}

void class_definition::proxify()
{
    PROXIFY_DEBUG("Proxifying a class definition!\n");
    *Header_stream << spaces[tab_column];
    if (proxiableClass)
	*Header_stream << "proxiable ";
    switch(identifier_ptr->id_token)
    {
      case CLASS:
	*Header_stream << "class ";
	break;
      case STRUCT:
	*Header_stream << "struct ";
	break;
      case UNION:
	*Header_stream << "union ";
	break;
      default:
	cerr << Me << ": Internal error: class_definition";
	cerr << "::proxify(): unknown csu token: " << Me << " will abort!\n";
	exit(-1);
    }
    if (identifier_ptr != NULL)
	identifier_ptr->print(Header_stream);

    if (identifier_ptr->id_token == CLASS)
    {
	currentClassName = identifier_ptr->id_string;

	currentMethodList = NULL;

	currentConstructorNumber = -1;

	if (derivation_list_ptr != NULL)
	    derivation_list_ptr->proxify();

	if ( ((currentMethodList != NULL) && (subBaseClassName == NULL)) ||
	     ((subBaseClassName != NULL) &&
		(strcmp(currentClassName, subBaseClassName) == 0)) ||
	     ((baseClassName != NULL) && 
		(strcmp(currentClassName, baseClassName) == 0)) )
	{

	    if (derivation_list_ptr != NULL)
	    {
		currentParentName = currentMethodList->className;
		if (proxiableClass && !currentMethodList->isProxiable)
		    cerr << Me << ": warning: class " << currentClassName <<
			" is proxiable but its parent is not\n";

		*Header_stream << " : ";
		derivation_list_ptr->print(',', Header_stream);
	    }
	    else
		currentParentName = NULL;

	    *Header_stream << " {\n";

	    if ((baseClassName != NULL) && 
		(strcmp(currentClassName, baseClassName) == 0))
	    {
		*Header_stream << "protected:\n";
		incr_tab();
		*Header_stream << spaces[tab_column];
		*Header_stream << baseClassName << "Star _real;\n";
		decr_tab();
	    }

	    proxiable_method_printed = FALSE;
	    proxiable_constructor_printed = FALSE;

	    if (declarations != NULL)
	    {
		incr_tab();
		declarations->proxify();
		decr_tab();
	    }

	    
#ifdef DUMMY
	    if ( proxiable_constructor_printed && !proxiable_method_printed )
	    {
		/*
		 * No proxiable methods, need to add a dummy so CFRONT
		 *   will generate a vtable.
		 */
		type * type_ptr, * type2_ptr;
	        type_ptr =  new type(new type_modifier(FUNCTION,
		    			new parameter_list(new parameter())));
		identifier * id_ptr = new identifier( "_dummy", IDENTIFIER );
		id_ptr->type_ptr = type_ptr;

		declarator * declarator_ptr = new declarator(
					id_ptr, type_ptr );

		type_ptr = new type(new base_type(VOID));
		type2_ptr = new type(new storage_class(VIRTUAL));
		type2_ptr->add_type( type_ptr );
		type_ptr = new type(new storage_class(PROXIABLE));
		type_ptr->add_type( type2_ptr );

		incr_tab();
		proxify_method( type_ptr, declarator_ptr );
		decr_tab();
	    }
#endif

	    // SBL
	    incr_tab();
	    *Header_stream << spaces[tab_column] 
		<< "void * operator new(size_t) { return 0; }\n";
	    decr_tab();
	    // SBL
	    *Header_stream << spaces[tab_column] << "};";

	    if ((subBaseClassName != NULL) &&
			    (strcmp(currentClassName, subBaseClassName) == 0))
		subBaseMethodList = currentMethodList;
	    else
	    {
		subBaseClassName = NULL; /* we won't be needing these anymore */
		baseClassName = NULL;

// 		int status;
// 
// 		/* just in case it's already been bound, unbind it */
// 		(void) ClassHash.unbind(currentClassName, status);
// 
// 		if (!ClassHash.bind(currentClassName,
// 				    (unsigned int) currentMethodList))
// 		{
// 		    cerr << Me << ": cannot install " << currentClassName <<
// 			    " class in class table!\n";
// 		}
		unsigned int value;
 		(void) ClassHash.unbind(currentClassName, value);
 		ClassHash.bind(currentClassName,
			       (unsigned int) currentMethodList);
		// bind cannot really fail, so eventually, I'll make this
		// an Assert.
		if (!ClassHash.lookup(currentClassName,value) ||
		    value != (unsigned int)currentMethodList)
		{
		    cerr << Me << ": cannot install " << currentClassName <<
			    " class in class table!\n";
		}
		
	    }
	    if (!currentMethodList->isProxiable)
		currentMethodList->isProxiable = proxiableClass;

	    MethodDescriptor * meth = currentMethodList;
	    for (; meth != NULL && meth->methodNumber == -1; meth = meth->prev)
		;

	    char *methClassName = currentClassName;
	    int numMethods = 0;
	    if (meth != NULL)
	    {
		numMethods = meth->methodNumber + 1;
		if (strcmp(meth->className, currentClassName) != 0)
		{
		    methClassName = meth->className;
		    meth = NULL;
		}
		else if (numMethods > MAXMETHODS)
		{
		    cerr << "Warning: more than " << MAXMETHODS <<
			" methods in class " << currentClassName << "\n";
		}
	    }

	    if ((subBaseClassName == NULL) &&
		    ((meth != NULL) || (currentConstructorNumber >= 0)))
	    {
		if ((currentHead->include_table) &&
				(Table_stream == Real_Table_stream))
		{
		    currentHead->include_table = FALSE;
		    print_version(Table_stream);
		    *Table_stream << "#include \"" << Tinclude << "\"\n\n";
		}

		if (meth != NULL)
		{
		    printMethodExterns(meth);
		    *Table_stream << "\n";

		    *Table_stream << "MethodTableEntry *" << currentClassName <<
		    			"MethodTable[] = {\n";
		    printMethodList(meth);
		    *Table_stream << "};\n\n";
		}
		else
		    *Table_stream << "extern MethodTableEntry *" <<
				methClassName << "MethodTable[];\n\n";

		if (currentConstructorNumber >= 0)
		{
		    *Table_stream << "MethodTableEntry *" << currentClassName <<
		    			"ConstructorTable[] = {\n";
		    printConstructorList(currentConstructorNumber);
		    *Table_stream << "};\n\n";
		}
		    
		*Table_stream << "ProxyTable " << currentClassName <<
					"ProxyTable = {\n";
		*Table_stream << "  \"" << currentClassName << "\",\n";

		if (proxiableClass)
		    *Table_stream << "  1,\n";
		else
		    *Table_stream << "  0,\n";

		*Table_stream << "  " << numMethods << ",\n";
		*Table_stream << "  " << methClassName << "MethodTable,\n";

		if (currentConstructorNumber >= 0)
		{
		    *Table_stream << "  " << currentConstructorNumber + 1
		    				<< ",\n  ";
		    identifier_ptr->print(Table_stream);
		    *Table_stream << "ConstructorTable,\n";
		}
		else
		{
		    *Table_stream << "  0,\n";
		    *Table_stream << "  0,\n";
		}

		*Table_stream << "};\n\n";

		*Table_stream << "static ProxyTableInstaller ";
		identifier_ptr->print(Table_stream);
		*Table_stream << "Installer( &";
		identifier_ptr->print(Table_stream);
		*Table_stream << "ProxyTable ); \n\n";
	    }
	}
	else 
	    *Header_stream << ";";

	currentClassName = NULL;
    }
    else 
	*Header_stream << ";";

}

void definition_list::proxify()
{
    PROXIFY_DEBUG("Proxifying a definition list!\n");
    if (definition_ptr != NULL)
	definition_ptr->proxify();
    for (definition_list *temp_stmt_list = next; temp_stmt_list != NULL; 
	 temp_stmt_list = temp_stmt_list->next) {
	temp_stmt_list->get_definition()->proxify();
	}
}

void declaration_access_indicator::proxify()
{
    PROXIFY_DEBUG("Proxifying a declaration access indicator!\n");
    Last_declaration_access_indicator = this;
}

void external_definition::proxify()
{
    PROXIFY_DEBUG("Proxifying an external declaration!\n");
    if (external_declaration != NULL) {
	external_declaration->proxify();
    }
}

void derivation_list::proxify()
{
    PROXIFY_DEBUG("Proxifying a derivation_list!\n");
    
    int status;
    unsigned int value;
    status = ClassHash.lookup(definition_ptr->identifier_ptr->id_string,
								    value);

    PROXIFY_DEBUG("Looked up " << definition_ptr->identifier_ptr->id_string <<
	", status " << status << ", value " << value << "\n");

    if ((status != 0) && (value != 0))
	currentMethodList = (MethodDescriptor *) value;
    else if ((subBaseClassName != NULL) && (strcmp(subBaseClassName,
			    definition_ptr->identifier_ptr->id_string) == 0))
	currentMethodList = subBaseMethodList;
    else
	currentMethodList = NULL;

    if (next != NULL) {
	cerr << Me << ": multiple inheritance not supported!\n";
    }
}

#endif //PROXIFY
