#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/******************************************************************************
 *
 * syntax_tree.c
 *
 *	This file contains the member functions for the classes used in the
 * syntax tree.  The classes for these functions are defined in syntax_tree.h.
 * The print member functions are contained in the file print.c
 *
 * Ed Loyot 7/12/90
 *
 *              Copyright (c) 1990, 1991
 *              University of Virginia
 *              All Rights Reserved
 *
 *		This software is the property of the University of Virginia.
 *		Permission is granted to copy and distribute this software as
 *		long as this notice is intact on all copies.  Derived works 
 *  		must retain this notice.  This software is provided `as is'
 *		and without any express or implied warrenties.
 *
 *              Send problems & suggestions to:
 *		Ed Loyot (ecl2v@virginia.edu) or
 *              Andrew Grimshaw (grimshaw@cs.virginia.edu)
 *
 ****************************************************************************/

#ifdef SABER_CC
#include <libc.h>
#endif

#include <stream.h>
#include <strstream.h>
#include "String.h"

#include "common.h"
#include "tokens.h"
#include "syntax_tree.h"
#include "parser.tab.h"

const int function_offset = 100;
extern void yyerror(char *);
extern void parameter_declaration(type*, declarator*);
extern type *get_parameter_type(type*, declarator*);
extern symbol_table *current_table;
extern symbol_table *parameter_table;
extern int enumerate_number;
extern symbol_table_stack scope_stack;

/******************************************************************************
 *
 *				Expressions
 *
 *****************************************************************************/

expression *expression::duplicate()
{
    cerr << Me << " : expression::duplicate() : internal error! duplicate";
    cerr << " member function called on an abstract base class expression!\n";
    return NULL;
}

expression *expression::get_function()
{
    return NULL;
}

p_boolean expression::constant_expression()
{
    return FALSE;
}

expression *binary_expression::get_function()
{
    if (left_operand->type_ptr->function_type())
	return left_operand->duplicate();
    else
	return NULL;
}

identifier *expression::get_function_name()
{
    return NULL;
}

identifier *binary_expression::get_function_name()
{
    if (right_operand->type_ptr->function_type())
	return (identifier *)right_operand->duplicate();
    else
	return NULL;
}

expression *expression::get_class()
{
    return NULL;
}

expression *binary_expression::get_class()
{
    if (left_operand->type_ptr->class_object_type())
	return left_operand->duplicate();
    else
    {
	type *class_type_ptr = left_operand->type_ptr->get_dereference_type();
	if (class_type_ptr->class_object_type())
	    return left_operand->duplicate();
	else
	    return NULL;
    }
}

expression_list *expression::get_arguments()
{
    return NULL;
}

expression_list *binary_expression::get_arguments()
{
    if (left_operand->type_ptr->function_type())
	return (expression_list *)right_operand->duplicate();
    else
	return NULL;
}

TOKEN expression::get_operator()
{
    return -1;
}

TOKEN binary_expression::get_operator()
{
    return operator_token;
}

expression_list::expression_list(expression *new_expression)
{
    expression_ptr = new_expression;
    next = NULL;
    if (new_expression == NULL)
	size = 0;
    else
	size = 1;
}	

expression_list::expression_list(expression *new_expression, 
				 type *expression_type) : expression(expression_type)
{
    expression_ptr = new_expression;
    next = NULL;
    if (new_expression == NULL)
	size = 0;
    else
	size = 1;
}

expression *expression_list::duplicate()
{
    expression_list *new_expr_list = new expression_list(NULL);

    if (expression_ptr != NULL)
	new_expr_list->expression_ptr = expression_ptr->duplicate();

    for (expression_list *temp_list_ptr = next; temp_list_ptr != NULL;
	 temp_list_ptr = temp_list_ptr->next)
	if (temp_list_ptr->expression_ptr != NULL)
	    new_expr_list->add_expression(temp_list_ptr->expression_ptr->duplicate());

    new_expr_list->size = size;

    if (type_ptr != NULL)
	new_expr_list->type_ptr = type_ptr->duplicate();

    return new_expr_list;
}

void expression_list::add_expression(expression *new_expression)
{
    // See if there are any expressions in this list.
    if (expression_ptr == NULL)
    {
	// List is empty simply add the new expression.
	expression_ptr = new_expression;
	size++;
	return;
    }
    else
	if (next == NULL)
	{
	    // Only one expressions in this list, simply add the new one.
	    next = new expression_list(new_expression);
	    size++;
	}
	else
	{
	    // Find the end of the expression list.
	    expression_list* temp_expr_list;
	    for (temp_expr_list = next;
		 temp_expr_list->next != NULL;
		 temp_expr_list = temp_expr_list->next)
		;
	    
	    // Add the new expression.
	    temp_expr_list->next = new expression_list(new_expression);
	    size++;
	}
}

void expression_list::add_list(expression_list *new_list_ptr)
{
    if (new_list_ptr == NULL)
	return;

    // See if there are any expressions in this list.
    if (next == NULL)
    {
	// No expressions in this list, simply add the new list.
	next = new_list_ptr;
	size += new_list_ptr->size;
    }
    else
    {
	// Find the end of the expression list.
	expression_list* temp_expr_list;
	for (*temp_expr_list = next;
	     temp_expr_list->next != NULL;
	     temp_expr_list = temp_expr_list->next)
	    ;
	
	// Add the new expression.
	temp_expr_list->next = new_list_ptr;
	size += new_list_ptr->size;
    }
}    

expression *binary_expression::duplicate()
{
    binary_expression *new_binary_expression = 
	new binary_expression(NULL, NULL, operator_token);

    if (left_operand != NULL)
	new_binary_expression->left_operand = left_operand->duplicate();

    if (right_operand != NULL)
	new_binary_expression->right_operand = right_operand->duplicate();

    if (type_ptr != NULL)
	new_binary_expression->type_ptr = type_ptr->duplicate();
    
    return new_binary_expression;
}

expression *allocation_expression::duplicate()
{
    allocation_expression *new_allocation_expression = 
	new allocation_expression(allocation_token, allocation_type->duplicate());

    if (argument_list != NULL)
	new_allocation_expression->argument_list = 
	    (expression_list *)argument_list->duplicate();

    if (initializers != NULL)
	new_allocation_expression->initializers = 
	    (expression_list *)initializers->duplicate();

    if (type_expression != NULL)
	new_allocation_expression->type_expression = 
	    (expression_list *)type_expression->duplicate();

    if (type_ptr != NULL)
	new_allocation_expression->type_ptr = 
	    type_ptr->duplicate();

    return new_allocation_expression;
}

expression *deallocation_expression::duplicate()
{
    deallocation_expression *new_deallocation_expression = 
	new deallocation_expression(deallocation_token, 
				    deallocation_expr->duplicate());

    if (bracket_expression != NULL)
	new_deallocation_expression->bracket_expression = 
	    bracket_expression->duplicate();

    if (type_ptr != NULL)
	new_deallocation_expression->type_ptr = type_ptr->duplicate();
    
    return new_deallocation_expression;
}

expression *type_expression::duplicate()
{
    type_expression *new_type_expression = 
	new type_expression(type_declaration->duplicate());
    
    if (type_ptr != NULL)
	new_type_expression->type_ptr = type_ptr->duplicate();
    
    return new_type_expression;
}

#ifdef MPLFRONT
expression *code_gen_expression::duplicate()
{
    return new code_gen_expression(expression_string);
}
#endif

expression *cast_expression::duplicate()
{
    cast_expression *new_cast_expression = 
	new cast_expression(type_expr->duplicate(), cast_expr->duplicate());

    if (type_ptr != NULL)
	new_cast_expression->type_ptr = type_ptr->duplicate();
    
    return new_cast_expression;
}

int identifier::get_arg_count()
{
    if (type_ptr->function_type())
	return type_ptr->get_arg_count();
    else
	// Log an error.  This is not a function name.
	return -1;
}

type_element* identifier::get_base_type()
{
	type_element *rval=0;
	type *current = 0;
	if (type_ptr == NULL) return NULL;
	// Else the identifer has a type
	current = type_ptr;
	while (current->get_next() != NULL) current = current->get_next();
	// Now current points to the "base" type
	rval = current->get_type_element();
	if (rval!=NULL) {
//		BACKEND_PRINT("the token of the base class is");
//		BACKEND_PRINT((int)rval->type_element_token);
		rval->print();	
	}
	return rval;
}

p_boolean identifier::ctor_or_dtor()
{
    if (scope_list == NULL)
	return FALSE;
    else
    {
	char *class_name = scope_list->get_last()->identifier_ptr->id_string;
	if (strcmp(id_string, class_name) == 0)
	    return TRUE;
	else
	    return FALSE;
    }
}

identifier *identifier::get_function_name()
{
    return (identifier *)duplicate();
}

expression *identifier::duplicate()
{
    char *temp_string = NULL;

    if (id_string != NULL)
    {
	temp_string = new char[strlen(id_string)+1];
	strcpy(temp_string, id_string);
    }

    identifier *new_identifier = new identifier(temp_string, id_token);

    new_identifier->type_ptr = type_ptr;

    if (storage_class_element != NULL)
	new_identifier->storage_class_element = 
	    storage_class_element->duplicate();

    if (conversion_type_ptr != NULL)
	new_identifier->conversion_type_ptr = conversion_type_ptr->duplicate();

    new_identifier->access_token           = access_token;
    new_identifier->inline_flag            = inline_flag;
    new_identifier->overload_flag          = overload_flag;
    new_identifier->virtual_flag           = virtual_flag;
#ifdef PROXIFY
    new_identifier->proxiable_flag         = proxiable_flag;
#endif
    new_identifier->pure_virtual_flag      = pure_virtual_flag;
    new_identifier->friend_flag            = friend_flag;
    new_identifier->function_definition_ptr= function_definition_ptr;
    new_identifier->csue_definition_ptr    = csue_definition_ptr;
    new_identifier->enum_definition_ptr    = enum_definition_ptr;
    new_identifier->scope_list             = scope_list;
    new_identifier->destructor_flag        = destructor_flag;
    new_identifier->operator_flag	   = operator_flag;
    new_identifier->operator_token	   = operator_token;
#ifdef MPLFRONT
    new_identifier->prv_flag               = prv_flag;
#endif

    return new_identifier;
}

p_boolean identifier::compare(identifier *compare_ptr)
{
    if (compare_ptr == NULL)
	return FALSE;

    if (destructor_flag != compare_ptr->destructor_flag)
	return FALSE;

    if (operator_flag)
    {
	if (!compare_ptr->operator_flag)
	    return FALSE;
	
	if (operator_token != compare_ptr->operator_token)
	    return FALSE;
    }
    else
    {
	/* two null strings are a no-match */
	if (id_string == NULL)
	    return FALSE;
	if (compare_ptr->id_string == NULL)
	    return FALSE;

	if (strcmp(id_string, compare_ptr->id_string) != 0)
	    return FALSE;
    }

    return TRUE;
}

type *identifier::make_typedef_type()
{
    type *typedef_type_ptr = new type(new base_type(TYPEDEFname, this));
    typedef_type_ptr->add_type(type_ptr->duplicate());
    return typedef_type_ptr;
}

expression *constant::duplicate()
{
    char *temp_string = NULL;
    
    if (constant_string != NULL)
    {
	temp_string = new char[strlen(constant_string)+1];
	strcpy(temp_string, constant_string);
    }
    
    constant *new_constant = 
	new constant(temp_string, constant_type->type_element_token,
		     constant_token);

    if (type_ptr != NULL)
	new_constant->type_ptr = type_ptr->duplicate();
    
    return new_constant;
}

constant::constant(char *const_string, TOKEN type_token, TOKEN const_token)
{
    constant_string = const_string;
    constant_token = const_token;

    // Build a type record for the constant.
    constant_type = new base_type(type_token);

    // Set the type_ptr field in the base class expression.
    type_ptr = new type(new base_type(type_token));
}

void string_list::add_string(constant *new_string_literal)
{
    // See if there are any strings in this list.
    if (next == NULL)
	// No strings in this list, simply add the new one.
	next = new string_list(new_string_literal);
    else
    {
	// Find the end of the string list.
	string_list* temp_string_list;
	for (temp_string_list = next;
	     temp_string_list->next != NULL;
	     temp_string_list = temp_string_list->next)
	    ;
	
	// Add the new string.
	temp_string_list->next = new string_list(new_string_literal);
    }
}

expression *string_list::duplicate()
{
    string_list *new_string_list = new string_list(NULL);

    if (string_ptr != NULL)
	new_string_list->string_ptr = (constant *)string_ptr->duplicate();

    for (string_list *temp_list_ptr = next; temp_list_ptr != NULL;
	 temp_list_ptr = temp_list_ptr->next)
	if (temp_list_ptr->string_ptr != NULL)
	    new_string_list->add_string((constant *)temp_list_ptr->string_ptr->duplicate());

    if (type_ptr != NULL)
	new_string_list->type_ptr = type_ptr->duplicate();

    return new_string_list;
}

void c_string_list::add_string(char *new_string)
{
    // See if there are any strings in this list.
    if (next == NULL)
	// No strings in this list, simply add the new one.
	next = new c_string_list(new_string);
    else
    {
	// Find the end of the string list.
	c_string_list* temp_string_list;
	for (temp_string_list = next;
	     temp_string_list->next != NULL;
	     temp_string_list = temp_string_list->next)
	    ;
	
	// Add the new string.
	temp_string_list->next = new c_string_list(new_string);
    }
}

void c_string_list::add_string_front(char *new_string)
{
    c_string_list *new_list_ptr = new c_string_list(string_ptr);
    new_list_ptr->next = next;
    next = new_list_ptr;
    string_ptr = new_string;
}
    
void c_string_list::add_element(c_string_list *new_element)
{
    // See if there are any elements in this list.
    if (next == NULL)
	// No strings in this list, simply add the new one.
	next = new_element;
    else
    {
	// Find the end of the string list.
	c_string_list* temp_string_list;
	for (temp_string_list = next;
	     temp_string_list->next != NULL;
	     temp_string_list = temp_string_list->next)
	    ;
	
	// Add the new string.
	temp_string_list->next = new_element;
    }
}

/******************************************************************************
 *
 *				Statements
 *
 *****************************************************************************/

void statement_list::add_statement(statement *new_statement)
{
    // See if there are any statements in this list.
    if (next == NULL)
	// No statements in this list, simply add the new one.
	next = new statement_list(new_statement);
    else
    {
	// Find the end of the statement list.
	statement_list *temp_stmt_list;
	for (temp_stmt_list = next;
	     temp_stmt_list->next != NULL;
	     temp_stmt_list = temp_stmt_list->next)
	    ;
	
	// Add the new statement.
	temp_stmt_list->next = new statement_list(new_statement);
    }
}

void statement_list::add_statement_front(statement *new_statement)
{
    statement_list *new_stmt_list_ptr = new statement_list(statement_ptr);
    new_stmt_list_ptr->next = next;
    next = new_stmt_list_ptr;
    statement_ptr = new_statement;
}

p_boolean statement::member_function()
{
    return FALSE;
}

identifier *statement::get_identifier()
{
    cerr << "Error get_identifier called on abstract base class.\n";
    return NULL;
}

/******************************************************************************
 *
 *				Declarations
 *
 *****************************************************************************/

void declarator::set_parameter_symbol_table()
{
    /*
     * This function sets up a symbol table containing the parameter
     * declarations for a function definition.  The symbol table is pointed
     * to by the global parameter_table and will be used when the function
     * body is parsed.
     */

    // Make sure the declarator is a function!
       
    type_element *type_element_ptr = NULL;

    if (declarator_type != NULL)
	type_element_ptr = declarator_type->get_type_element();

    if (type_element_ptr != NULL && type_element_ptr->type_element_token == 
	                            FUNCTION)
	((type_modifier *)type_element_ptr)->parameters->set_parameter_symbol_table();
    else
    {
	/*
	 * An identifier that is not a function name is being used where a 
	 * function name is expected in a function definition.  Log the error.
	 */
	
	ostrstream error(message_buffer, max_message_size);
	error << "No parameter list for function : ";
	error << declarator_id->id_string;
	error << "\n!";
	error.put('\0');
	yyerror(message_buffer);

	parameter_table = NULL;
    }
}

declarator *declarator::duplicate()
{
    declarator *new_declarator = NULL;
    if (declarator_id != NULL)
    {
        new_declarator = 
	    new declarator((identifier *)declarator_id->duplicate());
	if (declarator_type != NULL)
	    new_declarator->declarator_type = declarator_type->duplicate();
    }
    else
	if (declarator_type != NULL)
	    new_declarator = 
		new declarator(declarator_type->duplicate());

    if (parenthesized_initializers != NULL)
	new_declarator->parenthesized_initializers = 
	    (expression_list *)parenthesized_initializers->duplicate();

    if (initializers != NULL)
	new_declarator->initializers = initializers->duplicate();

    if (bit_field_size_expression != NULL)
	new_declarator->bit_field_size_expression = 
	    bit_field_size_expression->duplicate();

    new_declarator->pure_virtual_flag = pure_virtual_flag;

    return new_declarator;
}

void declarator_list::add_declarator(declarator *new_declarator)
{
    // See if there are any declarators in this list.
    if (next == NULL)
	// No declarators in this list, simply add the new one.
	next = new declarator_list(new_declarator);
    else
    {
	// Find the end of the declarator list.
	declarator_list *temp_decl_list;
	for (temp_decl_list = next;
	     temp_decl_list->next != NULL;
	     temp_decl_list = temp_decl_list->next)
	    ;
	
	// Add the new declarator.
	temp_decl_list->next = new declarator_list(new_declarator);
    }
}

declarator_list *declarator_list::duplicate()
{
    declarator_list *new_declarator_list = 
	new declarator_list(declarator_ptr->duplicate());

    for (declarator_list *temp_declarator_list = next, 
	 *temp_new_declarator_list = new_declarator_list;
	 temp_declarator_list != NULL; 
	 temp_declarator_list = temp_declarator_list->next,
	 temp_new_declarator_list = temp_new_declarator_list->next)
    {
	temp_new_declarator_list->next = 
	    new declarator_list
		(temp_declarator_list->get_declarator()->duplicate());
    }

    return new_declarator_list;    
}

identifier *variable_declaration::get_identifier()
{
    return declarators->get_declarator()->declarator_id;
}

variable_declaration *variable_declaration::duplicate()
{
    variable_declaration *duplicate_declaration = 
	new variable_declaration(declaration_type->duplicate());

    if (declarators != NULL)
	duplicate_declaration->declarators = declarators->duplicate();

    return duplicate_declaration;
}

void variable_declaration::declare_parameter()
{
    /*
     * The variable being declared is a parameter.  When this is a declaration
     * for a parameter there will only be at most one entry in the declarator
     * list.  Declare this declarator in the parameter symbol table.
     */

    if (declarators != NULL)
	parameter_declaration(declaration_type->duplicate(), 
			      declarators->get_declarator());
}

type *variable_declaration::get_type()
{
    /*
     * Get the type field for the parameter type.  Since this is a declaration
     * for a parameter there will only be at most one entry in the declarator
     * list.  Build the canonical type for this parameter and return it.
     */

    if (declarators != NULL)
	return get_parameter_type(declaration_type, declarators->get_declarator());
    else
	if (declaration_type != NULL)
	    return get_parameter_type(declaration_type, NULL);
	else
	    return NULL;
}

identifier *member_declaration::get_identifier()
{
    return func_definition->function_name->declarator_id;
}

p_boolean member_declaration::member_function()
{
    if (var_declaration != NULL)
    {
	declarator_list *declarator_list_ptr = 
	    ((variable_declaration *)var_declaration)->declarators;
	declarator *declarator_ptr = declarator_list_ptr->get_declarator();
	identifier *id_ptr = declarator_ptr->declarator_id;
	type *type_ptr = id_ptr->type_ptr;
	return type_ptr->function_declaration();
    }
    else
	if (func_definition != NULL)
	{
	    declarator *declarator_ptr = func_definition->function_name;
	    identifier *id_ptr = declarator_ptr->declarator_id;
	    type *type_ptr = id_ptr->type_ptr;
	    return type_ptr->function_declaration();
	}
    
    return FALSE;
}

p_boolean variable_declaration::member_function()
{
    if (declarators != NULL)
    {
	declarator *declarator_ptr = declarators->get_declarator();
	identifier *id_ptr = declarator_ptr->declarator_id;
	type *type_ptr = id_ptr->type_ptr;
	return type_ptr->function_declaration();
    }
    else
	return FALSE;
}

void definition_list::add_definition(definition *new_definition)
{
    // See if there are any definitions in this list.
    if (next == NULL)
	// No declarators in this list, simply add the new one.
	next = new definition_list(new_definition);
    else
    {
	// Find the end of the definition list.
	definition_list *temp_def_list;
	for (temp_def_list = next;
	     temp_def_list->next != NULL;
	     temp_def_list = temp_def_list->next)
	    ;
	
	// Add the new definition.
	temp_def_list->next = new definition_list(new_definition);
    }
}

void definition_list::add_definition_front(definition *new_definition)
{
    definition_list *new_def_list_ptr = new definition_list(definition_ptr);
    new_def_list_ptr->next = next;
    next = new_def_list_ptr;
    definition_ptr = new_definition;
}

initializer *initializer::duplicate()
{
    initializer *new_initializer = new initializer();

    if (initializers != NULL)
	new_initializer->initializers->duplicate();

    if (assignment_expression != NULL)
	new_initializer->assignment_expression =
	    assignment_expression->duplicate();

    return new_initializer;
}
	    
void initializer_list::add_initializer(initializer *new_initializer)
{
    // See if there are any initializers in this list.
    if (next == NULL)
	// No initializers in this list, simply add the new one.
	next = new initializer_list(new_initializer);
    else
    {
	// Find the end of the initializer list.
	initializer_list *temp_init_list;
	for (temp_init_list = next;
	     temp_init_list->next != NULL;
	     temp_init_list = temp_init_list->next)
	    ;
	
	// Add the new initializer.
	temp_init_list->next = new initializer_list(new_initializer);
    }
}

initializer_list *initializer_list::duplicate()
{
    initializer_list *new_initializer_list = new initializer_list();

    if (initializer_ptr != NULL)
	new_initializer_list->initializer_ptr = initializer_ptr->duplicate();

    for (initializer_list *temp_list_ptr = next; temp_list_ptr != NULL;
	 temp_list_ptr = temp_list_ptr->next)
	if (temp_list_ptr->initializer_ptr != NULL)
	    new_initializer_list->add_initializer(temp_list_ptr->initializer_ptr->duplicate());

    return new_initializer_list;
}

void ctor_initializer_list::add_ctor_initializer(ctor_initializer *initializer)
{
    // See if there are any initializers in this list.
    if (next == NULL)
	// No initializers in this list, simply add the new one.
	next = new ctor_initializer_list(initializer);
    else
    {
	// Find the end of the initializer list.
	ctor_initializer_list *temp_init_list;
	for (temp_init_list = next;
	     temp_init_list->next != NULL;
	     temp_init_list = temp_init_list->next)
	    ;
	
	// Add the new initializer.
	temp_init_list->next = new ctor_initializer_list(initializer);
    }
}

void parameter::set_type()
{
    if (parameter_declaration != NULL)
	parameter_type = parameter_declaration->get_type();
}

p_boolean parameter::compare_type(parameter *compare_ptr)
{
    if (parameter_type == NULL)
	set_type();
    if (compare_ptr->parameter_type == NULL)
	compare_ptr->set_type();
    
    if (parameter_type != NULL && compare_ptr->parameter_type != NULL)
	return (parameter_type->compare(compare_ptr->parameter_type));
    else
	if (parameter_type == NULL && compare_ptr->parameter_type == NULL)
	    return TRUE;
	else
	    return FALSE;
}

parameter *parameter::duplicate()
{
    parameter *new_parameter = new parameter(parameter_token);
    if (parameter_declaration != NULL)
       new_parameter->parameter_declaration=parameter_declaration->duplicate();
    if (parameter_initializer != NULL)
       new_parameter->parameter_initializer=parameter_initializer->duplicate();
    if (parameter_type != NULL)
       new_parameter->parameter_type = parameter_type->duplicate();
    return new_parameter;
}

int parameter_list::size()
{
    int count = 0;
    if (parameter_ptr != NULL && parameter_ptr->parameter_declaration != NULL)
	count++;
    for (parameter_list *temp_list_ptr = next; temp_list_ptr != NULL;
	 temp_list_ptr = temp_list_ptr->next)
	if (temp_list_ptr->parameter_ptr != NULL &&
	    temp_list_ptr->parameter_ptr->parameter_declaration != NULL)
	    count++;
    return count;
}

parameter_list *parameter_list::duplicate()
{
    parameter_list *new_parameter_list = new parameter_list(NULL);
    if (parameter_ptr != NULL)
	new_parameter_list->parameter_ptr = parameter_ptr->duplicate();
    for (parameter_list *temp_list_ptr = next; temp_list_ptr != NULL;
	 temp_list_ptr = temp_list_ptr->next)
	if (temp_list_ptr->parameter_ptr != NULL)
	    new_parameter_list->add_parameter(temp_list_ptr->parameter_ptr->duplicate());
    return new_parameter_list;
}

void parameter_list::set_type()
{
    if (parameter_ptr != NULL)
	parameter_ptr->set_type();
    for (parameter_list *temp_list_ptr = next; temp_list_ptr != NULL;
	 temp_list_ptr = temp_list_ptr->next)
	if (temp_list_ptr->parameter_ptr != NULL)
	    temp_list_ptr->parameter_ptr->set_type();
    return;
}

p_boolean parameter_list::compare_type(parameter_list *compare_ptr)
{
    if (parameter_ptr != NULL && compare_ptr->parameter_ptr != NULL)
    {
	if (parameter_ptr->compare_type(compare_ptr->parameter_ptr) == FALSE)
	    return FALSE;
    }
    else
	if ((parameter_ptr != NULL && compare_ptr->parameter_ptr == NULL) ||
	    (parameter_ptr == NULL && compare_ptr->parameter_ptr != NULL))
	    return FALSE;
    
    parameter_list *list1_ptr, *list2_ptr;
    for (list1_ptr = next, list2_ptr = compare_ptr->next;
	 (list1_ptr != NULL) && (list2_ptr != NULL);
	 list1_ptr = list1_ptr->next, list2_ptr = list2_ptr->next)
	if (list1_ptr->parameter_ptr != NULL && 
	    list2_ptr->parameter_ptr != NULL)
	{
	    if (list1_ptr->parameter_ptr->compare_type(list2_ptr->parameter_ptr)
		== FALSE)
		return FALSE;
	}
	else
	    if ((list1_ptr->parameter_ptr != NULL && 
		 list2_ptr->parameter_ptr == NULL) ||
		(list1_ptr->parameter_ptr == NULL && 
		 list2_ptr->parameter_ptr != NULL))
		return FALSE;
    
    if ((list1_ptr == NULL) && (list2_ptr == NULL))
	return TRUE;

    return FALSE;
}

void parameter_list::add_parameter(parameter *new_parameter)
{
    // See if there are any parameters in this list.
    if (next == NULL)
	// No parameters in this list, simply add the new one.
	next = new parameter_list(new_parameter);
    else
    {
	// Find the end of the parameter list.
	parameter_list *temp_param_list;
	for (temp_param_list = next;
	     temp_param_list->next != NULL;
	     temp_param_list = temp_param_list->next)
	    ;
	
	// Add the new parameter.
	temp_param_list->next = new parameter_list(new_parameter);
    }
}

void parameter_list::set_parameter_symbol_table()
{
    /* 
     * Build a symbol table and enter the parameters in it.  Make the global
     * parameter_table point to this new symbol table so it can be used to
     * while parsing the function body.
     */

    parameter_table = new symbol_table;

    /*
     * Loop through the parameters in the parameter list entering them in
     * the parameters symbol table.
     */

    if (parameter_ptr != NULL && parameter_ptr->parameter_declaration != NULL)
	parameter_ptr->parameter_declaration->declare_parameter();

    parameter *temp_parameter_ptr;

    for (parameter_list *temp_list_ptr = next; temp_list_ptr != NULL; 
	 temp_list_ptr = temp_list_ptr->next)

	if (((temp_parameter_ptr = temp_list_ptr->get_parameter()) != NULL) && 
	    (temp_parameter_ptr->parameter_declaration != NULL))

	    temp_parameter_ptr->parameter_declaration->declare_parameter();
}

int class_definition::get_function_number(char *function_name_string)
{
    identifier *function_id_ptr;
    int result;
    int max_function_number = count_member_functions();
    for (int function_number = 1; function_number <= max_function_number; 
	 function_number++)
    {
	function_id_ptr = get_function_identifier(function_number);
	if (function_id_ptr == NULL)
	    // log an error.
	    return -1;
	result = strcmp(function_id_ptr->id_string, function_name_string);
	if (result == 0)
	    return function_number + function_offset;
    }
    
    // Log an error.
    return -1;
}

identifier *class_definition::get_function_identifier(int fn_number)
{
    int count = 0;
    for (statement_list *member_list_ptr = declarations; 
	 member_list_ptr != NULL; 
	 member_list_ptr = member_list_ptr->get_next())
	if (member_list_ptr->get_statement()->member_function())
	{
	    count++;
	    if (count == fn_number)
	    return member_list_ptr->get_statement()->get_identifier();
	}
    
    // Log an error. The function was not found.
    return NULL;
}

int class_definition::max_args()
{
    int current_args;
    int max_args = 0;
    identifier *function_id_ptr;
    for (statement_list *member_list_ptr = declarations; 
	 member_list_ptr != NULL; 
	 member_list_ptr = member_list_ptr->get_next())
	if (member_list_ptr->get_statement()->member_function())
	{
	    function_id_ptr=member_list_ptr->get_statement()->get_identifier();
	    current_args = function_id_ptr->get_arg_count();
	    if (current_args > max_args)
		max_args = current_args;
	}
    
    return max_args;
}

int class_definition::count_member_functions()
{
    int count = 0;
    if (declarations == NULL)
	return count;
    if (declarations->get_statement()->member_function())
	count++;
    for (statement_list *statement_list_ptr = declarations->get_next(); 
	 statement_list_ptr != NULL;
	 statement_list_ptr = statement_list_ptr->get_next())
	if (statement_list_ptr->get_statement()->member_function())
	    count++;
    return count;
} 

void class_definition::set_scope()
{
    /*
     * Build a scope stack for the class and all its base classes.  This table
     * can then be used to look up members for the class.
     */
    
    symbol_table_stack *class_scope_ptr = new symbol_table_stack();
    if (derivation_list_ptr != NULL)
	derivation_list_ptr->build_scope(class_scope_ptr);
    class_scope_ptr->push(class_table);
    class_scope_stack_ptr = class_scope_ptr;
    scope_stack.push(class_scope_stack_ptr);
}

void class_definition::build_scope(symbol_table_stack *scope_ptr)
{
    if (derivation_list_ptr != NULL)
	derivation_list_ptr->build_scope(scope_ptr);
    scope_ptr->push(class_table);
}

void class_definition::reset_scope()
{
    for (int stack_size = class_scope_stack_ptr->stack_size(); stack_size > 0;
	 stack_size--)
	scope_stack.pop();
}

identifier *class_definition::lookup(identifier *id_ptr)
{
    if (class_scope_stack_ptr != NULL)
	return class_scope_stack_ptr->lookup(id_ptr);
    else
	if (class_table != NULL)
	    return class_table->lookup(id_ptr);
	else
	    return NULL;
}

identifier *class_definition::lookup(char *string_ptr)
{
    if (class_scope_stack_ptr != NULL)
	return class_scope_stack_ptr->lookup(string_ptr);
    else
	if (class_table != NULL)
	    return class_table->lookup(string_ptr);
	else
	    return NULL;
}

void derivation_list::add_derivation(derivation_list *new_class_definition)
{
    // See if there are any definitions in this list.
    if (next == NULL)
	// No class definitions in this list, simply add the new one.
	next = new_class_definition;
    else
    {
	// Find the end of the derivation list.
	derivation_list *derivation_list_ptr;
	for (derivation_list_ptr = next;
	     derivation_list_ptr->next != NULL;
	     derivation_list_ptr = derivation_list_ptr->next)
	    ;
	
	// Add the new class definition.
	derivation_list_ptr->next = new_class_definition;
    }
}

void derivation_list::add_derivation_front(class_definition *new_definition)
{
    derivation_list *new_list_ptr = new derivation_list(new_definition);
    new_list_ptr->next = next;
    next = new_list_ptr;
    definition_ptr = new_definition;
}

void derivation_list::set_scope()
{
    if (definition_ptr != NULL)
	definition_ptr->set_scope();
    for (derivation_list *derivation_list_ptr = next;
	 derivation_list_ptr != NULL;
	 derivation_list_ptr = derivation_list_ptr->next)
	derivation_list_ptr->definition_ptr->set_scope();
}

void derivation_list::build_scope(symbol_table_stack *scope_ptr)
{
    if (definition_ptr != NULL)
	definition_ptr->build_scope(scope_ptr);
    for (derivation_list *derivation_list_ptr = next;
	 derivation_list_ptr != NULL;
	 derivation_list_ptr = derivation_list_ptr->next)
	derivation_list_ptr->definition_ptr->build_scope(scope_ptr);
}

void derivation_list::reset_scope()
{
    if (definition_ptr != NULL)
	definition_ptr->reset_scope();
    for (derivation_list *derivation_list_ptr = next;
	 derivation_list_ptr != NULL;
	 derivation_list_ptr = derivation_list_ptr->next)
	derivation_list_ptr->definition_ptr->reset_scope();
}

class_definition *derivation_list::get_last()
{
    if (next == NULL)
	return definition_ptr;

    // Find the end of the derivation list.
    derivation_list *derivation_list_ptr;
    for (derivation_list_ptr = next;
	 derivation_list_ptr->next != NULL;
	 derivation_list_ptr = derivation_list_ptr->next)
	;
    
    return derivation_list_ptr->definition_ptr;
}

expression *derivation_list::build_scope_expression()
{
    expression *temp_expr_ptr = definition_ptr->identifier_ptr;
    
    identifier *id_ptr = NULL;
    type *type_ptr = NULL;

    for (derivation_list *derivation_list_ptr = next;
	 derivation_list_ptr != NULL;
	 derivation_list_ptr = derivation_list_ptr->next)
    {
	id_ptr = derivation_list_ptr->definition_ptr->identifier_ptr;
	if (id_ptr != NULL && id_ptr->type_ptr != NULL)
	    type_ptr = id_ptr->type_ptr->duplicate();
	temp_expr_ptr = 
	    new binary_expression(temp_expr_ptr, id_ptr, CLCL, type_ptr);
    }
    
    return temp_expr_ptr;
}
