/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/******************************************************************************
 *
 * syntax_tree.h
 *
 *	This file contains the definitions for the nodes of the syntax tree.
 *
 * Ed Loyot 6/12/90
 *
 *              Copyright (c) 1990, 1991
 *              University of Virginia
 *              All Rights Reserved
 *
 *		This software is the property of the University of Virginia.
 *		Permission is granted to copy and distribute this software as
 *		long as this notice is intact on all copies.  Derived works 
 *  		must retain this notice.  This software is provided `as is'
 *		and without any express or implied warrenties.
 *
 *              Send problems & suggestions to:
 *		Ed Loyot (ecl2v@virginia.edu) or
 *              Andrew Grimshaw (grimshaw@cs.virginia.edu)
 *
 *****************************************************************************/

#ifndef SYNTAX_TREE
#define SYNTAX_TREE

#include <stream.h>

#include "common.h"
#include "tokens.h"
#include "symbol_table.h"

/******************************************************************************
 *
 *				Types
 *
 *****************************************************************************/

class identifier;

class declarator;

class expression;

class parameter_list;

class type_element
{
  public:
    TOKEN type_element_token;
    p_boolean const_flag;
    p_boolean volatile_flag;
    type_element(TOKEN element_token)
    {
	type_element_token = element_token;
	const_flag = FALSE;
	volatile_flag = FALSE;
    }
    virtual type_element *duplicate();
    virtual void dump_type(ostream* = &cerr);
    virtual p_boolean compare(type_element *);
    virtual void print(ostream* = &cout, p_boolean = TRUE,
		       ostream* = &cout, int* = NULL);
    virtual void print_type(ostream* = &cout);
#ifdef PROXIFY
    virtual void proxify();
#endif
};

class type_modifier : public type_element
{
  public:
    expression        *size;		// Used for ARRAY only.
    parameter_list    *parameters;      // Used for FUNCTION only.
    type_modifier(TOKEN type_element_token ) : type_element(type_element_token) 
    {
	size       = NULL;
	parameters = NULL;
    }
    type_modifier(TOKEN type_element_token, expression *size_expression) :
	type_element(type_element_token)
    {
	size       = size_expression;
	parameters = NULL;
    }
    type_modifier(TOKEN type_element_token, parameter_list *parameters_list) :
	type_element(type_element_token)
    {
	size       = NULL;
	parameters = parameters_list;
    }
    type_element *duplicate();
    p_boolean compare(type_element *);
    void print(ostream* = &cout, p_boolean = TRUE, ostream* = &cout,int* = NULL);
#ifdef PROXIFY
    virtual void proxify();
#endif
};

class storage_class : public type_element
{
  public:
    storage_class(TOKEN type_element_token) : type_element(type_element_token) {}
    type_element *duplicate();
    void print(ostream* = &cout, p_boolean = TRUE, ostream* = &cout,int* = NULL);
#ifdef PROXIFY
    virtual void proxify();
#endif
};

class class_definition;

class type
{
    type_element *type_element_ptr;
    type         *next;
  public:
    type (type_element *new_type_element)
    {
	type_element_ptr = new_type_element;
	next = NULL;
    }
    type         *get_next()		           { return next;  }
    void 	 add_type_element(type_element *);
    void 	 add_type(type *);
    type	 *remove_type(type **, TOKEN);
    type_element *get_type_element()	           { return type_element_ptr; }
    type         *duplicate();
    p_boolean	 typedef_type();
    p_boolean	 function_declaration();
    void	 set_storage_class_flags(identifier *);
    void	 set_qualifier_flags();
    p_boolean	 compare(type *);
    p_boolean	 array_type();
    p_boolean	 function_type();
    p_boolean	 pointer_type();
    p_boolean	 ptr_or_ref_type();
    p_boolean	 arithmetic_type();
    p_boolean	 integral_type();
    p_boolean      class_object_type();
    p_boolean      cip_type();
    p_boolean	 void_type();
    p_boolean	 char_type();
    type	 *convert_arithmetic(type *);
    type	 *get_array_base_type();
    type	 *get_function_return_type();
    type	 *get_dereference_type();
    type	 *promote_integer();
    class_definition *get_class_definition();
    void         dump_type(ostream* = &cerr);
    void         print(ostream* = &cout, p_boolean = TRUE);
    void         print_type(ostream*);
    void	 expand_typedefs();
    int          get_arg_count();
#ifdef PROXIFY
    virtual void proxify();
#endif
};

class class_definition;

class enum_definition;

class base_type : public type_element
{
  public:
    type             *typedef_type_ptr;	    	// The real type for typedefs.
    identifier       *type_name_id_ptr;     	// The typedef name.
	p_boolean	     print_body;

    base_type(TOKEN);
    base_type(TOKEN, identifier *);

    type_element *duplicate();
    p_boolean compare(type_element *);
    void print(ostream* = &cout, p_boolean = TRUE, ostream* = &cout,int* = NULL);
    void print_type(ostream* = &cout);
    char *get_string();
#ifdef PROXIFY
    virtual void proxify();
#endif
};

class enum_definition;

class csue_type_element : public type_element
{
  public:
    class_definition *csu_definition_ptr;
    enum_definition  *enum_definition_ptr;
    csue_type_element(TOKEN csue_token, class_definition *csu_def):type_element(csue_token)
    {
	csu_definition_ptr  = csu_def;
	enum_definition_ptr = NULL;
    }
    csue_type_element(TOKEN csue_token, enum_definition *enum_def):type_element(csue_token)
    {
	csu_definition_ptr  = NULL;
	enum_definition_ptr = enum_def;
    }
    type_element *duplicate();
    p_boolean compare(type_element *);
    void print(ostream* = &cout, p_boolean = TRUE, ostream* = &cout,int* = NULL);
};

class initializer;

class variable_declaration;

class parameter
{
  public:
    TOKEN                parameter_token;	  // PARAMETER or ELLIPSIS.
    variable_declaration *parameter_declaration;
    initializer          *parameter_initializer;
    type  		 *parameter_type;
    parameter(TOKEN new_token = PARAMETER)
    {
	parameter_token       = new_token;
	parameter_declaration = NULL;
	parameter_initializer = NULL;
	parameter_type        = NULL;
    }
    parameter(variable_declaration *new_declaration,
	      initializer *new_initializer = NULL)
    {
	parameter_token       = PARAMETER;
	parameter_declaration = new_declaration;
	parameter_initializer = new_initializer;
	parameter_type        = NULL;
    }
    parameter *duplicate();
    p_boolean compare_type(parameter *);
    void set_type();
    void print(ostream* = &cout);
#ifdef PROXIFY
    virtual void proxify();
#endif
};

class parameter_list
{
    parameter      *parameter_ptr;
    parameter_list *next;
  public:
    type *type_qualifier_list_ptr;
    parameter_list ()
    {
	parameter_ptr = NULL;
	next = NULL;
	type_qualifier_list_ptr = NULL;
    }
    parameter_list (parameter *new_parameter)
    {
	parameter_ptr = new_parameter;
	next = NULL;
	type_qualifier_list_ptr = NULL;
    }
    parameter_list (parameter *new_parameter, type *type_qualifier_list)
    {
	parameter_ptr = new_parameter;
	next = NULL;
	type_qualifier_list_ptr = type_qualifier_list;
    }
    parameter_list *get_next()		    { return next;  }
    void 	   add_parameter(parameter *);
    parameter      *get_parameter()	    { return parameter_ptr; }
    parameter_list *duplicate();
    p_boolean compare_type(parameter_list *);
    void set_type();
    void set_parameter_symbol_table();
    void print(ostream* = &cout);
    int size();
#ifdef PROXIFY
    virtual void proxify();
#endif
};
    
/******************************************************************************
 *
 *			 	Expressions
 *
 *****************************************************************************/

class expression_list;

class expression
{
  public:
    type *type_ptr;
    expression()
    {
	type_ptr = NULL;
    }
    expression(type *expression_type_ptr)
    {
	type_ptr = expression_type_ptr;
    }
    virtual expression *duplicate();
    virtual void print(ostream* = &cout);
#ifdef MPLFRONT
    virtual void mark_prvs();
    virtual void set_prv_flag();
    virtual p_boolean mentat_expression();
    virtual void mpl_code_gen(p_boolean = TRUE);
    virtual expression *args_code_gen();
    virtual p_boolean prv_expression();
    virtual p_boolean cdb_expression();
#endif
    virtual expression *get_function();
    virtual identifier *get_function_name();
    virtual expression *get_class();
    virtual expression_list *get_arguments();
    virtual TOKEN get_operator();
    virtual p_boolean constant_expression();
#ifdef MPLFRONT
    virtual p_boolean rtf_expression();
#endif
};

class expression_list : public expression
{
    expression	    *expression_ptr;
    expression_list *next;
    int size;
  public:
    expression_list(expression *);
    expression_list(expression *, type *);
    expression_list *get_next()			    { return next;  }
    void            add_expression(expression *);
    void 	    add_list(expression_list *);
    expression      *get_expression()		    { return expression_ptr;  }
    expression *duplicate();
    void print(ostream* = &cout);
#ifdef MPLFRONT
    void mark_prvs();
    int get_size()                                  { return size;  }
    expression *args_code_gen();
    void mpl_code_gen(p_boolean = TRUE);
#endif
};

class binary_expression : public expression
{
  public:
    expression *left_operand;
    expression *right_operand;
    TOKEN operator_token;

    binary_expression(expression *left_expression,
		      expression *right_expression,
		      TOKEN expression_operator,
		      type *expression_type_ptr = 0) : expression(expression_type_ptr)
       {  
	   left_operand = left_expression;
	   right_operand = right_expression;
	   operator_token = expression_operator;  
       }
#ifdef NEVER
    binary_expression(expression *left_expression,
		      expression *right_expression,
		      TOKEN expression_operator)
       {  
	   left_operand = left_expression;
	   right_operand = right_expression;
	   operator_token = expression_operator;  
       }
#endif
    expression *duplicate();
    void print(ostream* = &cout);
#ifdef MPLFRONT
    void mark_prvs();
    void set_prv_flag();
    p_boolean mentat_expression();
    void mpl_code_gen(p_boolean = TRUE);
#endif
    expression *get_function();
    identifier *get_function_name();
    expression *get_class();
    expression_list *get_arguments();
    TOKEN get_operator();
#ifdef MPLFRONT
    p_boolean prv_expression();
    expression *args_code_gen();
#endif
};

class allocation_expression : public expression
{
  public:
    TOKEN allocation_token;
    declarator *allocation_type;
    expression_list *type_expression;
    expression_list *argument_list;
    expression_list *initializers;
    allocation_expression(TOKEN operator_token, expression_list *type_expr)
    {
	allocation_token = operator_token;
	type_expression = type_expr;
	allocation_type = NULL;
	argument_list = NULL;
	initializers = NULL;
    }	
    allocation_expression(TOKEN operator_token, declarator *type_declarator)
    {
	allocation_token = operator_token;
	allocation_type = type_declarator;
	type_expression = NULL;
	argument_list = NULL;
	initializers = NULL;
    }	
    expression *duplicate();
    void print(ostream* = &cout);
};

class deallocation_expression : public expression
{
  public:
    TOKEN deallocation_token;
    expression *bracket_expression;
    expression *deallocation_expr;
    deallocation_expression(TOKEN operator_token,expression *deallocation_ex)
    {
	deallocation_token = operator_token;
	deallocation_expr = deallocation_ex;
	bracket_expression = NULL;
        expression::type_ptr = NULL;
    }	
    deallocation_expression(TOKEN operator_token, expression *deallocation_ex,
			    type * type_ptr)
    {
	deallocation_token = operator_token;
	deallocation_expr = deallocation_ex;
	bracket_expression = NULL;
        expression::type_ptr = type_ptr;
    }	
    expression *duplicate();
    void print(ostream* = &cout);
};

class cast_expression : public expression
{
  public:
    expression *type_expr;
    expression *cast_expr;
    cast_expression(expression *type_name, expression *cast_ex)
    {
	type_expr = type_name;
	cast_expr = cast_ex;
	expression::type_ptr = NULL;
    }
    cast_expression(expression *type_name, expression *cast_ex, type* type_ptr)
    {
	type_expr = type_name;
	cast_expr = cast_ex;
	expression::type_ptr  = type_ptr;
    }
    expression *duplicate();
    void print(ostream* = &cout);
#ifdef MPLFRONT
    void mark_prvs();
#endif
};

class type_expression : public expression
{
  public:
    variable_declaration *type_declaration;
    type_expression(variable_declaration *type_decl)
    {
	type_declaration = type_decl;
    }
    expression *duplicate();
    void print(ostream* = &cout);
};

#ifdef MPLFRONT
class code_gen_expression : public expression
{
  public:
    char *expression_string;
    code_gen_expression(char *expr_string)
    {
	expression_string = expr_string;
    }
    expression *duplicate();
    void print(ostream* = &cout);
};
#endif

class function_definition;

class enum_definition;

class c_string_list;

class derivation_list;

class identifier : public expression
{
  public:
    TOKEN 	  id_token;              // IDENTIFIER, TYPEDEFname, FUNCTION
	                                 // ENUM, CLASS, STRUCT, UNION, LABEL.
    char  	  *id_string;            // String form for identifier name.
    TOKEN 	  access_token;          // PUBLIC, PRIVATE or PROTECTED, only
                                         // used for definitions in a class.
    type_element* storage_class_element; // EXTERN, AUTO, REGISTER or STATIC
                                         // token in storage_class_element.
    p_boolean             inline_flag;
    p_boolean 	        overload_flag;
    p_boolean 	        virtual_flag;
#ifdef PROXIFY
    p_boolean 	        proxiable_flag;
#endif
    p_boolean 	        pure_virtual_flag;
    p_boolean 	        friend_flag;
    function_definition *function_definition_ptr;  // Used for function names.
    class_definition    *csue_definition_ptr;      // Used for class names.
    enum_definition     *enum_definition_ptr;	   // Used for enum names.
    derivation_list    	*scope_list;
    p_boolean		destructor_flag;
    p_boolean	        operator_flag;
    TOKEN		operator_token;
    type		*conversion_type_ptr;
#ifdef MPLFRONT
    p_boolean		prv_flag;
#endif

    identifier(char *input_string, TOKEN token)
    { 
        id_token                = token;
        id_string               = input_string;
	type_ptr                = NULL;
	access_token            = NONE;
	storage_class_element   = new storage_class(NONE);
	inline_flag             = FALSE;
	overload_flag           = FALSE;
	virtual_flag            = FALSE;
#ifdef PROXIFY
	proxiable_flag          = FALSE;
#endif
	pure_virtual_flag       = FALSE;
	friend_flag             = FALSE;
	function_definition_ptr = NULL;
	csue_definition_ptr     = NULL;
	enum_definition_ptr     = NULL;
	scope_list 		= NULL;
	destructor_flag		= FALSE;
	operator_flag		= FALSE;
	operator_token		= NONE;
	conversion_type_ptr	= NULL;
#ifdef MPLFRONT
	prv_flag                = FALSE;
#endif
    }
    void initialize(char *input_string, TOKEN token)
    { 
        id_token                = token;
        id_string               = input_string;
	type_ptr                = NULL;
	access_token            = NONE;
	storage_class_element   = new storage_class(NONE);
	inline_flag             = FALSE;
	overload_flag           = FALSE;
	virtual_flag            = FALSE;
#ifdef PROXIFY
	proxiable_flag          = FALSE;
#endif
	pure_virtual_flag       = FALSE;
	friend_flag             = FALSE;
	function_definition_ptr = NULL;
	csue_definition_ptr     = NULL;
	enum_definition_ptr     = NULL;
	scope_list 		= NULL;
	destructor_flag		= FALSE;
	operator_flag		= FALSE;
	operator_token		= NONE;
	conversion_type_ptr	= NULL;
#ifdef MPLFRONT
	prv_flag                = FALSE;
#endif
    }
    p_boolean ctor_or_dtor();
    expression *duplicate();
    p_boolean compare(identifier *);
    type *make_typedef_type();
    void print(ostream* = &cout);
    type_element *get_base_type();
#ifdef MPLFRONT
    void set_prv_flag();
    p_boolean prv_expression();
    p_boolean cdb_expression();
#endif
    identifier *get_function_name();
    int get_arg_count();
#ifdef MPLFRONT
    p_boolean rtf_expression();
    expression *args_code_gen();
#endif
};

class constant : public expression
{
  public:
    char *constant_string;
    base_type *constant_type;
    TOKEN constant_token;

    constant(char *, TOKEN, TOKEN);
    expression *duplicate();
    void print(ostream* = &cout);
    p_boolean constant_expression()     {	return TRUE; }
#ifdef MPLFRONT
    expression *args_code_gen();
#endif
};

class string_list : public expression
{
    constant	    *string_ptr;
    string_list     *next;
  public:
    string_list(constant *new_string_literal)
    {
	string_ptr = new_string_literal;
	next = NULL;
    }
    string_list     *get_next()			    { return next;  }
    void            add_string(constant *);
    constant        *get_string()		    { return string_ptr;  }
    expression *duplicate();
    void print(ostream* = &cout);
};

class c_string_list
{
    char	      *string_ptr;
    c_string_list     *next;
  public:
    p_boolean virtual_flag;
    TOKEN   access_token;
    c_string_list(char *new_string)
    {
	string_ptr = new_string;
	next = NULL;
	virtual_flag = FALSE;
	access_token = NONE;
    }
    c_string_list   *get_next()			    { return next;  }
    void	    add_string_front(char *);
    void            add_string(char *);
    void	    add_element(c_string_list *);
    char            *get_string()		    { return string_ptr;  }
    void print(TOKEN, ostream* = &cout);
};

/******************************************************************************
 *
 *				Statements
 *
 *****************************************************************************/

class statement
{
  public:
    virtual p_boolean member_function();
    virtual identifier *get_identifier();
    virtual void print(ostream* = &cout, p_boolean = TRUE);
#ifdef MPLFRONT
    virtual void mark_prvs();
    virtual void mpl_code_gen();
#endif
#ifdef PROXIFY
    virtual void proxify();
#endif
};

class statement_list
{
    statement	   *statement_ptr;
    statement_list *next;
  public:
    statement_list(statement *new_statement)
    {
	statement_ptr = new_statement;
	next = NULL;
    }
    statement_list *get_next()			      { return next;  }
    void           add_statement(statement *);
    void           add_statement_front(statement *);
    statement      *get_statement()		      { return statement_ptr; }
    void print(ostream* = &cout);
#ifdef MPLFRONT
    void mark_prvs();
    void mpl_code_gen();
#endif
#ifdef PROXIFY
    virtual void proxify();
#endif
};

class compound_statement : public statement
{
  public:
    symbol_table   *table;
    statement_list *declarations;  // These are only some initial declarations.
	                           // There may be more in the statement list.
    statement_list *statements;

    compound_statement(symbol_table *new_table)	
    {  
	table        = new_table;  
	declarations = NULL;
	statements   = NULL;
    }
    void           print_table()                {  table->print();  }
    void           print(ostream* = &cout, p_boolean = TRUE);
#ifdef MPLFRONT
    void mark_prvs();
    void mpl_code_gen();
#endif
};

class selection_statement : public statement
{
  public:
    TOKEN      statement_token;		// IF or SWITCH.
    expression *expression_ptr;
    statement  *statement_ptr;
    statement  *else_statement_ptr;

    selection_statement(TOKEN stmt_token, expression *expr, statement *stmt,
	      statement *else_stmt)
    {
	statement_token    = stmt_token;
	expression_ptr     = expr;
	statement_ptr      = stmt;
	else_statement_ptr = else_stmt;
    }
    void print(ostream* = &cout, p_boolean = TRUE);
#ifdef MPLFRONT
    void mark_prvs();
    void mpl_code_gen();
#endif
};

class expression_statement : public statement
{
  public:
    expression *expression_ptr;

    expression_statement(expression *new_expr) {  expression_ptr = new_expr;  }
    void print(ostream* = &cout, p_boolean = TRUE);
#ifdef MPLFRONT
    void mark_prvs();
    void mpl_code_gen();
#endif
};

#ifdef MPLFRONT
class code_gen_statement : public statement
{
  public:
    char *code_string;
    code_gen_statement(char *code_ptr) 
    {
	code_string = code_ptr;
    }
    void print(ostream* = &cout, p_boolean = TRUE);
};
#endif

class labeled_statement : public statement
{
  public:
    TOKEN      statement_token;          // CASE, DEFAULT or LABEL.
    statement  *statement_ptr;  
    expression *case_expr_or_label_ptr;  // For a label this points to the
	                                 // identifier, for a case this points
					 // to the constant expression.

    labeled_statement(TOKEN stmt_token, expression *expr, statement *stmt)
    {  
	statement_token        = stmt_token;
	statement_ptr          = stmt;
	case_expr_or_label_ptr = expr;
    }  
    void print(ostream* = &cout, p_boolean = TRUE);
#ifdef MPLFRONT
    void mark_prvs();
    void mpl_code_gen();
#endif
};

class while_statement : public statement
{
  public:			// Use for do or while.
    TOKEN      statement_token; // DO or WHILE.
    statement  *statement_ptr;
    expression *expression_ptr;

    while_statement(TOKEN stmt_token, expression *expr, statement *stmt)
    { 
	statement_token = stmt_token;
	statement_ptr   = stmt;
	expression_ptr  = expr;
    }
    void print(ostream* = &cout, p_boolean = TRUE);
#ifdef MPLFRONT
    void mark_prvs();
    void mpl_code_gen();
#endif
};

class for_statement : public statement
{
  public:
    statement  *initializer_decl_or_expr; // Can be an expression statement or
	                                  // a declaration.
    expression *test_ptr;
    expression *increment_ptr;
    statement  *statement_ptr;

    for_statement(statement *init_decl, expression *test_expr,
		  expression *incr_expr, statement *stmt)
    {
	initializer_decl_or_expr = init_decl;
	test_ptr = test_expr;
	increment_ptr = incr_expr;
	statement_ptr = stmt;
    }
    void print(ostream* = &cout, p_boolean = TRUE);
#ifdef MPLFRONT
    void mark_prvs();
    void mpl_code_gen();
#endif
};

class jump_statement : public statement
{
  public:
    TOKEN      statement_token;            // GOTO, RETURN, BREAK or CONTINUE.
    expression *return_expr_or_label_ptr;  // For a label this points to the
	                                   // labels identifier, for a return
                                           // this points to the expression.

    jump_statement(TOKEN stmt_token, expression *expr)
    {  
        statement_token = stmt_token;
        return_expr_or_label_ptr = expr;
    }
    void print(ostream* = &cout, p_boolean = TRUE);
#ifdef MPLFRONT
    void mark_prvs();
    void mpl_code_gen();
#endif
};    

/******************************************************************************
 *
 *				Declarations
 *
 *****************************************************************************/

class initializer;

class declarator
{
  public:
    identifier       *declarator_id;
    expression_list  *parenthesized_initializers;
    initializer      *initializers;
    expression       *bit_field_size_expression;
    p_boolean          pure_virtual_flag;
    type             *declarator_type;

    declarator(identifier *identifier_ptr)
    {
	declarator_id              = identifier_ptr;
	declarator_type            = NULL;
	parenthesized_initializers = NULL;
	initializers               = NULL;
	bit_field_size_expression  = NULL;
	pure_virtual_flag          = FALSE;
    }

    declarator(identifier *identifier_ptr, type *type_ptr)
    {
	declarator_id              = identifier_ptr;
	declarator_type            = type_ptr;
	parenthesized_initializers = NULL;
	initializers               = NULL;
	bit_field_size_expression  = NULL;
	pure_virtual_flag          = FALSE;
    }

    declarator(type *type_ptr)
    {
	declarator_id              = NULL;
	declarator_type            = type_ptr;
	parenthesized_initializers = NULL;
	initializers               = NULL;
	bit_field_size_expression  = NULL;
	pure_virtual_flag          = FALSE;
    }

    void set_pure_virtual()
    {
	pure_virtual_flag = TRUE;
	if (declarator_id != NULL)
	    declarator_id->pure_virtual_flag = TRUE;
    }

    void add_type(type *type_ptr)
    {
	if (declarator_type == NULL)
	    declarator_type = type_ptr;
	else
	    declarator_type->add_type(type_ptr);
    }

    void set_parameter_symbol_table();
    declarator *duplicate();
    void print(ostream* = &cout, p_boolean = FALSE);
#ifdef MPLFRONT
    void mark_prvs();
    void mpl_code_gen();
#endif
#ifdef PROXIFY
    virtual void proxify();
#endif
};

class declarator_list
{
    declarator	    *declarator_ptr;
    declarator_list *next;
  public:
    declarator_list(declarator *new_declarator)
    {
	declarator_ptr = new_declarator;
	next = NULL;
    }
    declarator_list *get_next()	        	     { return next;  }
    void            add_declarator(declarator *);
    declarator      *get_declarator()		     { return declarator_ptr; }
    declarator_list *duplicate();
    void print(ostream* = &cout);
#ifdef MPLFRONT
    void mark_prvs();
    void mpl_code_gen();
#endif
#ifdef PROXIFY
    virtual void proxify();
#endif
};

#ifdef PROXIFY
extern p_boolean in_proxify_source;
extern char *in_proxify_include;
#endif

class declaration : public statement
{
  public:
#ifdef MPLFRONT
    virtual void mark_prvs();
    virtual void mpl_code_gen();
#endif
#ifdef PROXIFY
    p_boolean		proxify_source;
    char		*proxify_include;
    declaration()
    {
	proxify_source        = in_proxify_source;
	proxify_include	      = in_proxify_include;
    }
    void check_source();
#endif
};

class variable_declaration : public declaration	// Declarations are statements.
{
  public:
    type 	    *declaration_type;
    declarator_list *declarators;

    variable_declaration(type *new_type, declarator *new_declarator)
    {
	declaration_type = new_type; 
	declarators = new declarator_list(new_declarator);
    }
    variable_declaration(type *new_type)
    {
	declaration_type = new_type; 
	declarators = NULL;
    }
    void add_declarator(declarator *new_declarator)
    {
	if (declarators == NULL)
	    declarators = new declarator_list(new_declarator);
	else
	    declarators->add_declarator(new_declarator);
    }
    variable_declaration *duplicate();
    void declare_parameter();
    type *get_type();
    p_boolean member_function();
    void print(ostream* = &cout, p_boolean = TRUE);
#ifdef MPLFRONT
    void mark_prvs();
    void mpl_code_gen();
#endif
    identifier *get_identifier();
#ifdef PROXIFY
    virtual void proxify();
#endif
};

class csue_declaration : public declaration
{
  public:
    type *csue_type_ptr;
    csue_declaration(type *csue_type)
    {
	csue_type_ptr = csue_type;
    }
    void print(ostream* = &cout, p_boolean = TRUE);
#ifdef MPLFRONT
    void mark_prvs();
    void mpl_code_gen();
#endif
#ifdef PROXIFY
    virtual void proxify();
#endif
};    

class member_declaration : public declaration
{
  public:
    declaration         *var_declaration;
    function_definition *func_definition;
    class_definition    *cl_definition;
    identifier 		*class_rescoped_identifier;
    member_declaration(declaration *var_decl)
    {
	var_declaration = var_decl;
	func_definition = NULL;
	cl_definition = NULL;
	class_rescoped_identifier = NULL;
    }
    member_declaration(function_definition *func_def)
    {
	var_declaration = NULL;
	func_definition = func_def;
	cl_definition = NULL;
	class_rescoped_identifier = NULL;
    }
    member_declaration(class_definition *cl_def)
    {
	var_declaration = NULL;
	func_definition = NULL;
	cl_definition = cl_def;
	class_rescoped_identifier = NULL;
    }
    member_declaration(identifier *id_ptr)
    {
	var_declaration = NULL;
	func_definition = NULL;
	cl_definition = NULL;
	class_rescoped_identifier = id_ptr;
    }
    p_boolean member_function();
    identifier *get_identifier();
    void print(ostream* = &cout, p_boolean = TRUE);
#ifdef MPLFRONT
    void mark_prvs();
    void mpl_code_gen();
#endif
#ifdef PROXIFY
    virtual void proxify();
#endif
};

class declaration_access_indicator : public statement
{
  public:
    TOKEN access_token;
    declaration_access_indicator(TOKEN indicator_token)
    {
	access_token = indicator_token;
    }
    void print(ostream* = &cout, p_boolean = TRUE);
#ifdef PROXIFY
    virtual void proxify();
#endif
};

class initializer_list;

class initializer
{
  public:
    initializer_list *initializers;
    expression *assignment_expression;
    initializer()
    {
	initializers          = NULL;
	assignment_expression = NULL;
    }
    initializer(initializer_list *new_initializers)
    {
	initializers          = new_initializers;
	assignment_expression = NULL;
    }
    initializer(expression *new_assignment_expression)
    {
	initializers          = NULL;
	assignment_expression = new_assignment_expression;
    }
    initializer *duplicate();
    void print(ostream* = &cout, p_boolean = TRUE);
#ifdef MPLFRONT
    p_boolean mentat_expression();
#endif
};

class initializer_list
{
    initializer	     *initializer_ptr;
    initializer_list *next;
  public:
    initializer_list()
    {
	initializer_ptr = NULL;
	next            = NULL;
    }
    initializer_list(initializer *new_initializer)
    {
	initializer_ptr = new_initializer;
	next            = NULL;
    }
    initializer_list *get_next()		     { return next;  }
    void             add_initializer(initializer *);
    initializer      *get_initializer()		     { return initializer_ptr;}
    initializer_list *duplicate();
    void print(ostream* = &cout);
};    

class ctor_initializer
{
  public:
    char            *name_string;
    expression_list *arguments;
    ctor_initializer(char *name, expression_list *args)
    {
	name_string = name;
	arguments   = args;
    }
    void print(ostream* = &cout);
};

class ctor_initializer_list
{
    ctor_initializer	  *ctor_initializer_ptr;
    ctor_initializer_list *next;
  public:
    ctor_initializer_list(ctor_initializer *new_initializer)
    {
	ctor_initializer_ptr = new_initializer;
	next                 = NULL;
    }
    ctor_initializer_list *get_next()		 {return next;}
    void                  add_ctor_initializer(ctor_initializer *);
    ctor_initializer      *get_ctor_initializer(){return ctor_initializer_ptr;}
    void print(ostream* = &cout);
};    

/******************************************************************************
 *
 *				Definitions
 *
 *****************************************************************************/

class definition
{
  public:
    virtual void print(ostream* = &cout, p_boolean = TRUE);
#ifdef MPLFRONT
    virtual void mark_prvs();
    virtual void mpl_code_gen();
#endif
#ifdef PROXIFY
    virtual void proxify();
#endif
};

class definition_list
{
    definition	    *definition_ptr;
    definition_list *next;
  public:
    definition_list(definition *new_definition)
    {
	definition_ptr = new_definition;
	next = NULL;
    }
    definition_list *get_next()			    {  return next;  }
    void            add_definition(definition *);
    void            add_definition_front(definition *);
    definition      *get_definition()		    {  return definition_ptr; }
    void print(ostream* = &cout);
#ifdef MPLFRONT
    void mark_prvs();
    void mpl_code_gen();
    void mpl_server_gen();
#endif
#ifdef PROXIFY
    virtual void proxify();
#endif
};

class function_definition : public definition
{
  public:
    type                  *declaration_type;
    declarator            *function_name;
    compound_statement    *body;
    ctor_initializer_list *ctor_initializers;

    function_definition(type *function_declaration_type, 
			declarator *declarator_name, 
			compound_statement *function_body)
    {
	body              = function_body;
	declaration_type  = function_declaration_type;
	function_name     = declarator_name;
	ctor_initializers = NULL;
    }
    function_definition(declarator *declarator_name, 
			compound_statement *function_body)
    {
	body              = function_body;
	declaration_type  = NULL;
	function_name     = declarator_name;
	ctor_initializers = NULL;
    }
    function_definition(declarator *declarator_name)
    {
	body              = NULL;
	declaration_type  = NULL;
	function_name     = declarator_name;
	ctor_initializers = NULL;
    }
    function_definition()
    {
	body              = NULL;
	declaration_type  = NULL;
	function_name     = NULL;
	ctor_initializers = NULL;
    }
    function_definition(compound_statement *function_body)
    {
	body              = function_body;
	declaration_type  = NULL;
	function_name     = NULL;
	ctor_initializers = NULL;
    }
    void print(ostream* = &cout, p_boolean = TRUE);
#ifdef MPLFRONT
    void mark_prvs();
    void mpl_code_gen();
#endif
#ifdef PROXIFY
    virtual void proxify();
#endif
};

class derivation_list;

class class_definition : public definition
{
  public:
    TOKEN	       csu_token;
    identifier         *identifier_ptr;
    statement_list     *declarations;
    symbol_table       *class_table;
#ifdef MPLFRONT
    p_boolean	       mentat_class_flag;
    p_boolean	       backend_target_flag;
    p_boolean	       persistent_flag;
#endif
    int		       member_function_count;
    derivation_list    *derivation_list_ptr;
    symbol_table_stack *class_scope_stack_ptr;
    p_boolean	       print_body;

    class_definition(TOKEN token, identifier *csu_identifier)
    {
	csu_token	      = token;
	identifier_ptr        = csu_identifier;
	declarations          = NULL;
	class_table           = NULL;
#ifdef MPLFRONT
	mentat_class_flag     = FALSE;
	backend_target_flag   = FALSE;
	persistent_flag       = FALSE;
#endif
	member_function_count = -1;
	derivation_list_ptr   = NULL;
	class_scope_stack_ptr = NULL;
	print_body	      = TRUE;
    }
    class_definition(TOKEN	     token,
		     identifier      *csu_identifier, 
		     symbol_table    *table,
		     derivation_list *inheritance_list)
    {
	csu_token	      = token;
	identifier_ptr        = csu_identifier;
	class_table           = table;
	declarations          = NULL;
#ifdef MPLFRONT
	mentat_class_flag     = FALSE;
	backend_target_flag   = FALSE;
	persistent_flag       = FALSE;
#endif
	member_function_count = -1;
	derivation_list_ptr   = inheritance_list;
	class_scope_stack_ptr = NULL;
	print_body	      = TRUE;
    }

    int count_member_functions();
    identifier *get_function_identifier(int);
    int get_function_number(char *);
    void set_scope();
    void build_scope(symbol_table_stack *);
    void reset_scope();
    identifier *lookup(identifier *);
    identifier *lookup(char *);
    void print(ostream* = &cout, p_boolean = TRUE);
#ifdef MPLFRONT
    p_boolean mentat_class()
    {
	return mentat_class_flag;
    }
    void mark_prvs();
    void mpl_code_gen();
#endif
    int max_args();
#ifdef PROXIFY
    virtual void proxify();
#endif
};

class derivation_list
{
  public:
    class_definition *definition_ptr;
    derivation_list  *next;
    TOKEN	     access_specifier;
    p_boolean	     virtual_flag;
    derivation_list(class_definition *new_definition)
    {
	definition_ptr   = new_definition;
	next             = NULL;
	access_specifier = NONE; //Set this to public or private,(the default?)
	virtual_flag     = FALSE;
    }
    derivation_list *get_next()			    {  return next;  }
    void            add_derivation(derivation_list *);
    void            add_derivation_front(class_definition *);
    void	    set_scope();
    void	    build_scope(symbol_table_stack *);
    void	    reset_scope();
    class_definition *get_last();
    void print(TOKEN, ostream* = &cout);
    expression *build_scope_expression();
#ifdef PROXIFY
    virtual void proxify();
#endif
};

class enum_definition : public definition
{
  public:
    identifier      *enum_id_ptr;
    declarator_list *enum_list_ptr;

    enum_definition(identifier *id_ptr, declarator_list *list_ptr)
    {
	enum_id_ptr   = id_ptr;
	enum_list_ptr = list_ptr;
    }
    enum_definition(identifier *id_ptr)
    {
	enum_id_ptr   = id_ptr;
	enum_list_ptr = NULL;
    }
    void print(ostream* = &cout, p_boolean = TRUE);
};

class external_definition : public definition
{
  public:
    definition       *external_function_definition;
    declaration      *external_declaration;
    definition_list  *translation_unit;
    constant 	     *linkage_specifier;
    definition_list  *definition_group;
    
    external_definition(declaration *ext_decl) 
    { 
	external_function_definition = NULL;
	external_declaration         = ext_decl;  
	translation_unit             = NULL;
	linkage_specifier            = NULL;
	definition_group	     = NULL;
    }
    external_definition(definition_list *trans_unit, constant *linkage) 
    { 
	external_function_definition = NULL;
	external_declaration 	     = NULL;  
	translation_unit     	     = trans_unit;
	linkage_specifier    	     = linkage;
	definition_group	     = NULL;
    }
    external_definition(definition *func_def, constant *linkage) 
    { 
	external_function_definition = func_def;
	external_declaration 	     = NULL;  
	translation_unit     	     = NULL;
	linkage_specifier    	     = linkage;
	definition_group	     = NULL;
    }
    external_definition(definition_list *def_group) 
    { 
	external_function_definition = NULL;
	external_declaration 	     = NULL;  
	definition_group     	     = def_group;
	translation_unit	     = NULL;
	linkage_specifier    	     = NULL;
    }
    void print(ostream* = &cout, p_boolean = TRUE);
#ifdef MPLFRONT
    void mark_prvs();
    void mpl_code_gen();
#endif
#ifdef PROXIFY
    virtual void proxify();
#endif
};

#ifdef MPLFRONT
class code_gen_definition : public definition
{
  public:
    char *code_gen_string;
    code_gen_definition(char *string)
    {
	code_gen_string = string;
    }
    void print(ostream* = &cout, p_boolean = TRUE);
};
#endif

class stack_bucket
{
  public:
    class_definition *class_def_ptr;
    stack_bucket *next;
    stack_bucket(class_definition *new_class_def)
    {
	class_def_ptr = new_class_def;
	next = NULL;
    }
};

class class_def_stack
{
  public:
    stack_bucket *top;
    int size;
    class_def_stack()
    {
	top = NULL;
	size = 0;
    }
    void push(class_definition *new_class_def)
    {
	stack_bucket *bucket_ptr = new stack_bucket(new_class_def);
	if (top == NULL)
	{
	    top = bucket_ptr;
	    size++;
	}
	else
	{
	    bucket_ptr->next = top;
	    top = bucket_ptr;
	    size++;
	}
    }
    void pop()
    {
	class_definition *return_def_ptr;

	if (top != NULL)
	{
	    return_def_ptr = top->class_def_ptr;
	    top = top->next;
	    size--;
	}
    }
};

#endif //SYNTAX_TREE

