#ifdef __GNUC__
#pragma implementation
#endif

/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/******************************************************************************
 *
 * types.c
 *
 *	This file contains the functions and member functions for type 
 * processing.  The classes for these functions are defined in syntax_tree.h.
 *
 * Ed Loyot 7/12/90
 *
 *              Copyright (c) 1990, 1991
 *              University of Virginia
 *              All Rights Reserved
 *
 *		This software is the property of the University of Virginia.
 *		Permission is granted to copy and distribute this software as
 *		long as this notice is intact on all copies.  Derived works 
 *  		must retain this notice.  This software is provided `as is'
 *		and without any express or implied warrenties.
 *
 *              Send problems & suggestions to:
 *		Ed Loyot (ecl2v@virginia.edu) or
 *              Andrew Grimshaw (grimshaw@cs.virginia.edu)
 *
 ****************************************************************************/

#ifdef SABER_CC
#include <libc.h>
#endif

#include <stddef.h>
#include <stdlib.h>
#include <stream.h>
#include <strstream.h>
#include "String.h"

#include "common.h"
#include "tokens.h"
#include "syntax_tree.h"
#include "parser.tab.h"

extern void yyerror(char *);
extern void parameter_declaration(type*, declarator*);
extern type *get_parameter_type(type*, declarator*);
extern symbol_table *current_table;
extern symbol_table *parameter_table;
extern int enumerate_number;
extern symbol_table_stack scope_stack;

/******************************************************************************
 *
 *				       	Types
 *
 *****************************************************************************/

void type::add_type(type *new_type)
{
    // See if there are any types in this list.
    if (next == NULL)
	// Only one type in this list, simply add the new one.
	next = new_type;
    else
    {
	// Find the end of the type list.
	type* temp_type;
	for (temp_type = next;
	     temp_type->next != NULL;
	     temp_type = temp_type->next)
	    ;
	
	// Add the new type.
	temp_type->next = new_type;
    }
}

type *type::remove_type(type **type_list, TOKEN token)
{
    // Remove a type with a type_element_token of 'token' from list

    type *type_ptr;
    type *prev_type_ptr = NULL;
    for (type_ptr = *type_list; type_ptr != NULL; type_ptr = type_ptr->next)
    {
	if (token == type_ptr->type_element_ptr->type_element_token)
	    break;
	prev_type_ptr = type_ptr;
    }

    if (type_ptr == NULL)
	// it wasn't found
	return NULL;
    
    if (prev_type_ptr == NULL)
	// at front of list
	*type_list = type_ptr->next;
    else
	prev_type_ptr->next = type_ptr->next;
    
    type_ptr->next = NULL;

    return type_ptr;
}

void type::add_type_element(type_element *new_type_element)
{
    // See if there are any types in this list.
    if (next == NULL)
	// Only one type in this list, simply add the new one.
	next = new type(new_type_element);
    else
    {
	// Find the end of the type list.
	type *temp_type;
	for (temp_type = next;
	     temp_type->next != NULL;
	     temp_type = temp_type->next)
	    ;
	
	// Add the new type.
	temp_type->next = new type(new_type_element);
    }
}
    
type *type::duplicate()
{
    type *new_type = new type(type_element_ptr->duplicate());
    for (type *temp_type = next, *temp_new_type = new_type;
	 temp_type != NULL; 
	 temp_type = temp_type->next, temp_new_type = temp_new_type->next)
    {
	temp_new_type->next = 
	    new type(temp_type->get_type_element()->duplicate());
    }
    return new_type;
}

/*
 * type::typedef_type -
 *
 * 	This function removes any typedef elements from the type list.  If any
 * were removed it returns TRUE.  It reports errors to the user if more than
 * one typedef was present in the list.
 */

p_boolean type::typedef_type()

{
    p_boolean typedef_present_flag = FALSE;
    type *delete_type_ptr;
    type_element *delete_element_ptr;

    /*
     * Remove any typedef type elements from the front of the list.
     *
     * The linked list structure used for types requires a special case to 
     * remove elements from the front of the list.  Since other data structures
     * may have pointers to the first element of the list, the first element
     * cannot be deleted.  To remove the first element, replace its data with
     * the data from the second element, set the first element's next pointer 
     * to what is pointed to by the second element's next pointer, then delete
     * the second element.  Keep doing this until all typedef records have been
     * removed from the front of the list or until the list is empty.
     */

    while (type_element_ptr != NULL &&
	   type_element_ptr->type_element_token == TYPEDEF)
    {
	/*
	 * If a typedef has been seen already, then there is more than one
	 * typedef keyword in the declaration.  This is an error.
	 */

	if (typedef_present_flag == TRUE)
	{
	    ostrstream error(message_buffer, max_message_size);
	    error << "Duplicate key word typedef in declaration!";
	    error.put('\0');
	    yyerror(message_buffer);
	}
	else
	    typedef_present_flag = TRUE;

	/*
	 * Replace the first elements data and next pointers with those from
	 * the second element.  Delete the second element and the first
	 * element's data.
	 */
	
	if (next != NULL)
	{
	    // There is another element in the list.
	    delete_element_ptr = type_element_ptr;
	    delete_type_ptr = next;
	    type_element_ptr = next->type_element_ptr;
	    next = next->next;
	    delete delete_element_ptr;
	    delete delete_type_ptr;
	}
	else
	{
	    // Deleting the only element in the list.
	    delete type_element_ptr;
	    type_element_ptr = NULL;
	}

    }  // End while.

    /*
     * The first element in the list is now either a non-typedef element or
     * an empty element(data pointer = NULL).  Step down the list keeping a
     * pointer to the previous element.  The previous element must always
     * be a non-typedef element so always make sure all typedef elements are
     * removed from the next element spot before stepping down the list.
     */

    for (type *temp_type = this;
	 temp_type->next != NULL; 
	 temp_type = temp_type->next)
    {
	while (temp_type->next != NULL &&
	       temp_type->next->type_element_ptr->type_element_token==TYPEDEF)
	{
	    /*
	     * If a typedef has been seen already, then there is more than one
	     * typedef keyword in the declaration.  This is an error.
	     */

	    if (typedef_present_flag == TRUE)
	    {
	    	ostrstream error(message_buffer, max_message_size);
		error << "Duplicate key word typedef in declaration!";
		error.put('\0');
		yyerror(message_buffer);
	    }
	    else
		typedef_present_flag = TRUE;

	    /*
	     * If the next element is not the end of the list, simply set the
	     * current element's next pointer to the element that follows the
	     * element that will be deleted, then delete the element's data
	     * and finally delete the element itself.  If the next element is
	     * the last element, simply delete it and set the current element's
	     * data pointer to NULL.
	     */
	    
	    if (temp_type->next->next != NULL)
	    {
		// There is another element in the list.
		delete_element_ptr = temp_type->next->type_element_ptr;	   
		delete_type_ptr = temp_type->next;
		temp_type->next = temp_type->next->next;
		delete delete_element_ptr;
		delete delete_type_ptr;
	    }
	    else
	    {
		// Deleting the last element in the list.
		delete temp_type->next->type_element_ptr;
		delete temp_type->next;
		temp_type->next = NULL;
	    }

	} // End while.

	/*
	 * If the list no longer has any elements because of deletions break
	 * out of the loop.
	 */

	if (temp_type->next == NULL)
	    break;

    } // End for.

    return typedef_present_flag;
}

/*
 * type::function_declaration -
 *
 *	This function checks to see if the declaration that generated this 
 * is a function declaration, if so it returns TRUE otherwise it returns FALSE.
 */

p_boolean type::function_declaration()
{
    if (type_element_ptr != NULL &&
	type_element_ptr->type_element_token == FUNCTION)
	return TRUE;

    for (type *temp_type_ptr = next; temp_type_ptr != NULL;
	 temp_type_ptr = temp_type_ptr->next)
	if (temp_type_ptr->type_element_ptr->type_element_token == FUNCTION)
	    return TRUE;

    return FALSE;
}

/*
 * type::set_storage_class_flags -
 *
 *	This function removes any storage class elements from the type list.
 * Storage class elements are extern, auto, register, static, inline, friend, 
 * overload and virtual.  The function sets the appropriate flags in an
 * identifier object that is passes in to it.  If more than one of extern, auto
 * register or static is specified then there is an error.  If any of virtual,
 * overload, friend or inline are specified more than one there is also an 
 * error.
 */

void type::set_storage_class_flags(identifier *id)

{
    type_element *delete_element_ptr;
    type *delete_type_ptr;

    /*
     * Remove any storage class type elements from the front of the list.
     *
     * The linked list structure used for types requires a special case to 
     * remove elements from the front of the list.  Since other data structures
     * may have pointers to the first element of the list, the first element
     * cannot be deleted.  To remove the first element, replace its data with
     * the data from the second element, set the first element's next pointer 
     * to what is pointed to by the second element's next pointer, then delete
     * the second element.  Keep doing this until all storage class elements
     * have been removed from the front of the list or until the list is empty.
     */

    while (type_element_ptr != NULL &&
	   (type_element_ptr->type_element_token == EXTERN   ||
	    type_element_ptr->type_element_token == STATIC   ||
	    type_element_ptr->type_element_token == AUTO     ||
	    type_element_ptr->type_element_token == REGISTER ||
	    type_element_ptr->type_element_token == INLINE   ||
	    type_element_ptr->type_element_token == FRIEND   ||
	    type_element_ptr->type_element_token == VIRTUAL  ||
#ifdef PROXIFY
	    type_element_ptr->type_element_token == PROXIABLE ||
#endif
	    type_element_ptr->type_element_token == OVERLOAD))
    {
	/*
	 * If more than one of extern, static, register or auto has been seen
	 * then there is an error.  Also if any of inline, friend, overload or
	 * virtual has been seen more than once then there is also an error.
	 * There are several other possible errors to be checked for like
	 * an extern inline etc., also some storage classes are not available
	 * at certain scoping levels.  Dont worry about these for now.
	 */
	
	switch (type_element_ptr->type_element_token)
	{
	  case EXTERN:
	  case STATIC:
	  case AUTO:
	  case REGISTER:
	    if (id->storage_class_element->type_element_token != NONE)
	    {
	    	ostrstream error(message_buffer, max_message_size);
		error << "Multiple storage classes";
		id->storage_class_element->dump_type(&error);
		error << " and";
		type_element_ptr->dump_type(&error);
		error << " in declaration!\n";
		error.put('\0');
		yyerror(message_buffer);
	    }
	    delete id->storage_class_element;
	    id->storage_class_element = type_element_ptr->duplicate();
	    break;
	  case INLINE:	
	    if (id->inline_flag == TRUE)
	    {
	    	ostrstream error(message_buffer, max_message_size);
		error << "Duplicate key word inline in declaration!\n";
		error.put('\0');
		yyerror(message_buffer);
	    }
	    else
		id->inline_flag = TRUE;
	    break;
	  case FRIEND:
	    if (id->friend_flag == TRUE)
	    {
	    	ostrstream error(message_buffer, max_message_size);
		error << "Duplicate key word friend in declaration!\n";
		error.put('\0');
		yyerror(message_buffer);
	    }
	    else
		id->friend_flag = TRUE;
	    break;
	  case VIRTUAL:
	    if (id->virtual_flag == TRUE)
	    {
	    	ostrstream error(message_buffer, max_message_size);
		error << "Duplicate key word virtual in declaration!\n";
		error.put('\0');
		yyerror(message_buffer);
	    }
	    else
		id->virtual_flag = TRUE;
	    break;
#ifdef PROXIFY
	  case PROXIABLE:
	    if (id->proxiable_flag == TRUE)
	    {
	    	ostrstream error(message_buffer, max_message_size);
		error << "Duplicate key word proxiable in declaration!\n";
		error.put('\0');
		yyerror(message_buffer);
	    }
	    else
		id->proxiable_flag = TRUE;
	    break;
#endif
	  case OVERLOAD:
	    if (id->overload_flag == TRUE)
	    {
	    	ostrstream error(message_buffer, max_message_size);
		error << "Duplicate key word overload in declaration!\n";
		error.put('\0');
		yyerror(message_buffer);
	    }
	    else
		id->overload_flag = TRUE;
	    break;
	}

	/*
	 * Replace the first elements data and next pointers with those from
	 * the second element.  Delete the second element and the first
	 * element's data.
	 */
	
	if (next != NULL)
	{
	    // There is another element in the list.
	    delete_element_ptr = type_element_ptr;
	    delete_type_ptr = next;
	    type_element_ptr = next->type_element_ptr;
	    next = next->next;
	    delete delete_element_ptr;
	    delete delete_type_ptr;
	}
	else
	{
	    // Deleting the only element in the list.
	    delete type_element_ptr;
	    type_element_ptr = NULL;
	}

    }  // End while.

    /*
     * The first element in the list is now either a non-typedef element or
     * an empty element(data pointer = NULL).  Step down the list keeping a
     * pointer to the previous element.  The previous element must always
     * be a non-typedef element so always make sure all typedef elements are
     * removed from the next element spot before stepping down the list.
     */

    for (type *temp_type = this;
	 temp_type->next != NULL; 
	 temp_type = temp_type->next)
    {
	while (temp_type->next != NULL &&
	     (temp_type->next->type_element_ptr->type_element_token==EXTERN  ||
	      temp_type->next->type_element_ptr->type_element_token==STATIC  ||
	      temp_type->next->type_element_ptr->type_element_token==AUTO    ||
	      temp_type->next->type_element_ptr->type_element_token==REGISTER||
	      temp_type->next->type_element_ptr->type_element_token==INLINE  ||
 	      temp_type->next->type_element_ptr->type_element_token==FRIEND  ||
	      temp_type->next->type_element_ptr->type_element_token==VIRTUAL ||
#ifdef PROXIFY
	      temp_type->next->type_element_ptr->type_element_token==PROXIABLE||
#endif
	      temp_type->next->type_element_ptr->type_element_token==OVERLOAD))
	{
	    /*
	     * If more than one of extern, static, register or auto has been 
	     * seen then there is an error.  Also if any of inline, friend, 
	     * overload or virtual has been seen more than once then there is 
	     * also an error. There are several other possible errors to be 
	     * checked for like an extern inline etc., also some storage 
	     * classes are not available at certain scoping levels.  Don't 
	     * worry about these for now.
	     */
	    
	    switch (temp_type->next->type_element_ptr->type_element_token)
	    {
	      case EXTERN:
	      case STATIC:
	      case AUTO:
	      case REGISTER:
		if (id->storage_class_element->type_element_token != NONE)
		{
	    	    ostrstream error(message_buffer, max_message_size);
		    error << "Multiple storage classes";
		    id->storage_class_element->dump_type(&error);
		    error << " and";
		    temp_type->next->type_element_ptr->dump_type(&error);
		    error << " in declaration!\n";
		    error.put('\0');
		    yyerror(message_buffer);
		}
		delete id->storage_class_element;
		id->storage_class_element = 
		    temp_type->next->type_element_ptr->duplicate();
		break;
	      case INLINE:	
		if (id->inline_flag == TRUE)
		{
	    	    ostrstream error(message_buffer, max_message_size);
		    error << "Duplicate key word inline in declaration!\n";
		    error.put('\0');
		    yyerror(message_buffer);
		}
		else
		    id->inline_flag = TRUE;
		break;
	      case FRIEND:
		if (id->friend_flag == TRUE)
		{
	    	    ostrstream error(message_buffer, max_message_size);
		    error << "Duplicate key word friend in declaration!\n";
		    error.put('\0');
		    yyerror(message_buffer);
		}
		else
		    id->friend_flag = TRUE;
		break;
	      case VIRTUAL:
		if (id->virtual_flag == TRUE)
		{
	    	    ostrstream error(message_buffer, max_message_size);
		    error << "Duplicate key word virtual in declaration!\n";
		    error.put('\0');
		    yyerror(message_buffer);
		}
		else
		    id->virtual_flag = TRUE;
		break;
#ifdef PROXIFY
	      case PROXIABLE:
		if (id->proxiable_flag == TRUE)
		{
	    	    ostrstream error(message_buffer, max_message_size);
		    error << "Duplicate key word proxiable in declaration!\n";
		    error.put('\0');
		    yyerror(message_buffer);
		}
		else
		    id->proxiable_flag = TRUE;
		break;
#endif
	      case OVERLOAD:
		if (id->overload_flag == TRUE)
		{
	    	    ostrstream error(message_buffer, max_message_size);
		    error << "Duplicate key word overload in declaration!\n";
		    error.put('\0');
		    yyerror(message_buffer);
		}
		else
		    id->overload_flag = TRUE;
		break;
	    }
	    
	    /*
	     * Replace the first elements data and next pointers with those 
	     * from the second element.  Delete the second element and the 
	     * first element's data.
	     */
	    
	    if (temp_type->next->next != NULL)
	    {
		// There is another element in the list.
		delete_element_ptr = temp_type->next->type_element_ptr;
		delete_type_ptr = temp_type->next;
		temp_type->next = temp_type->next->next;
		delete delete_element_ptr;
		delete delete_type_ptr;
	    }
	    else
	    {
		// Deleting the last element in the list.
		delete temp_type->next->type_element_ptr;
		delete temp_type->next;
		temp_type->next = NULL;
	    }
	    
	}  // End while.
	    
	/*
	 * If the list no longer has any elements because of deletions break
	 * out of the loop.
	 */

	if (temp_type->next == NULL)
	    break;

    }  // End for.

}

/*
 * type::set_qualifier_flags -
 *
 *	This function removes any const and typedef elements from the list and
 * sets the appropriate flags in other elements to indicate if these other 
 * elements are const or volatile.
 */

void type::set_qualifier_flags()

{
    p_boolean const_present_flag = FALSE;
    p_boolean volatile_present_flag = FALSE;
    type *delete_type_ptr;
    type_element *delete_element_ptr;

    /*
     * Remove any const or volatile type_elements from the front of the list.
     *
     * The linked list structure used for types requires a special case to 
     * remove elements from the front of the list.  Since other data structures
     * may have pointers to the first element of the list, the first element
     * cannot be deleted.  To remove the first element, replace its data with
     * the data from the second element, set the first element's next pointer 
     * to what is pointed to by the second element's next pointer, then delete
     * the second element.  Keep doing this until all typedef records have been
     * removed from the front of the list or until the list is empty.
     */

    while (type_element_ptr != NULL &&
	   (type_element_ptr->type_element_token == CONST ||
	    type_element_ptr->type_element_token == VOLATILE))
    {
	/*
	 * If a const or volatile has been seen already, then there is more 
	 * than one const or volatile in the declaration.  This is an error.
	 */

	if (const_present_flag == TRUE && 
	    type_element_ptr->type_element_token == CONST)
	{
	    ostrstream error(message_buffer, max_message_size);
	    error << "Duplicate const in declaration!";
	    error.put('\0');
	    yyerror(message_buffer);
	}
	else
	    if (volatile_present_flag == TRUE && 
		type_element_ptr->type_element_token == VOLATILE)
	    {
	        ostrstream error(message_buffer, max_message_size);
		error << "Duplicate volatile in declaration!";
		error.put('\0');
		yyerror(message_buffer);
	    }
	    else
		if (type_element_ptr->type_element_token == CONST)
		    const_present_flag = TRUE;
		else
		    volatile_present_flag = TRUE;	    

	/*
	 * Replace the first elements data and next pointers with those from
	 * the second element.  Delete the second element and the first
	 * element's data.
	 */
	
	if (next != NULL)
	{
	    // There is another element in the list.
	    delete_element_ptr = type_element_ptr;
	    delete_type_ptr = next;
	    type_element_ptr = next->type_element_ptr;
	    next = next->next;
	    delete delete_element_ptr;
	    delete delete_type_ptr;
	}
	else
	{
	    // Deleting the only element in the list.
	    delete type_element_ptr;
	    type_element_ptr = NULL;
	}

    } // End while.

    /*
     * The first element in the list is now either a non-typedef element or
     * an empty element(data pointer = NULL).  Set its const and volatile
     * flags then step down the list keeping a pointer to the previous element.
     * The previous element must always be a non-typedef element so always make
     * sure all typedef elements are removed from the next element spot and the
     * next elements flags are set before stepping down the list.
     */

    if (type_element_ptr != NULL)
    {
	// Set the const and volatile flags.
        type_element_ptr->const_flag    = const_present_flag;
	type_element_ptr->volatile_flag = volatile_present_flag;
	const_present_flag = volatile_present_flag = FALSE;
    }
	
    for (type *temp_type = this;
	 temp_type->next != NULL; 
	 temp_type = temp_type->next)
    {
	while (temp_type->next != NULL &&
	       (temp_type->next->type_element_ptr->type_element_token==CONST||
	       temp_type->next->type_element_ptr->type_element_token==VOLATILE)
	      )
	{
	    /*
	     * If a const or volatile has been seen already, then there is more
	     * than one const or volatile in the declaration. This is an error.
	     */

	    if (const_present_flag == TRUE && 
		temp_type->next->type_element_ptr->type_element_token == 
		CONST)
	    {
	        ostrstream error(message_buffer, max_message_size);
		error << "Duplicate const in declaration!";
		error.put('\0');
		yyerror(message_buffer);
	    }
	    else
		if (volatile_present_flag == TRUE && 
		    temp_type->next->type_element_ptr->type_element_token==
		    VOLATILE)
		{
	            ostrstream error(message_buffer, max_message_size);
		    error << "Duplicate volatile in declaration!";
		    error.put('\0');
		    yyerror(message_buffer);
		}
		else
		    if(temp_type->next->type_element_ptr->type_element_token
		       == CONST)
			const_present_flag = TRUE;
		    else
			volatile_present_flag = TRUE;	    

	    /*
	     * If the next element is not the end of the list, simply set the
	     * current element's next pointer to the element that follows the
	     * element that will be deleted, then delete the element's data
	     * and finally delete the element itself.  If the next element is
	     * the last element, simply delete it and set the current element's
	     * data pointer to NULL.
	     */
	    
	    if (temp_type->next->next != NULL)
	    {
		delete_type_ptr = temp_type->next;
		delete_element_ptr = temp_type->next->type_element_ptr;
		temp_type->next = temp_type->next->next;
		delete delete_element_ptr;
		delete delete_type_ptr;
	    }
	    else
	    {
		delete temp_type->next->type_element_ptr;
		delete temp_type->next;
		temp_type->next = NULL;
	    }
	    
	} // End the while.
	    
	/*
	 * At this point the next element is either a non-const, non-volatile
	 * or a NULL.
	 * If it is not a NULL update its const and volatile flags,
	 */

	if (temp_type->next != NULL)
	{
	    // Set the const and volatile flags.
	    temp_type->type_element_ptr->const_flag = const_present_flag;
	    temp_type->type_element_ptr->volatile_flag = volatile_present_flag;
	}

	/*
	 * If the list no longer has any elements because of deletions break
	 * out of the loop.
	 */

	if (temp_type->next == NULL)
	    break;
	
    } // End the for.
	    
}

void type::dump_type(ostream *output_stream)
{
    type_element_ptr->dump_type(output_stream);
    for (type *temp_type = next; temp_type != NULL; 
	 temp_type = temp_type->next)
	temp_type->get_type_element()->dump_type(output_stream);
}

p_boolean type::compare(type *compare_ptr)
{
    p_boolean same_type = TRUE;
    if (!type_element_ptr->compare(compare_ptr->type_element_ptr))
	same_type = FALSE;
    else
	for (type *type1_ptr = next, *type2_ptr = compare_ptr->next; 
	     type1_ptr != NULL || type2_ptr != NULL;
	     type1_ptr = type1_ptr->next, type2_ptr = type2_ptr->next)
	{
	    if (type1_ptr == NULL || type2_ptr == NULL)
	    {
		same_type = FALSE;
		break;
	    }
	    if (!type1_ptr->type_element_ptr->compare(type2_ptr->type_element_ptr))
	    {
		same_type = FALSE;
		break;
	    }
	}
    return same_type;
}

type_element *type_element::duplicate()
{
    cerr << Me << ": type_element::duplicate() : internal error! duplicate";
    cerr << " member function called on abstract base class!\n";
    return NULL;
}

p_boolean type_element::compare(type_element *compare_ptr)
{
#ifndef PROXIFY
    cerr << Me << " : type_element::compare() : internal error! compare";
    cerr << " member function called on abstract base class!\n";
#endif

    if (type_element_token != compare_ptr->type_element_token)
	return FALSE;
    if (const_flag != compare_ptr->const_flag)
	return FALSE;
    if (volatile_flag != compare_ptr->volatile_flag)
	return FALSE;
    
    return TRUE;
}

void type::expand_typedefs()
{
    // Check the first element to see if it is the TYPEDEFname.
    if (type_element_ptr != NULL && 
	type_element_ptr->type_element_token == TYPEDEFname)
    {
	/*
	 * The first element is the TYPEDEFname. Remove the TYPEDEFname element
	 * and insert the real type for this TYPEDEFname at the begining of 
	 * this  type list.
	 */

	// Get the real type from the TYPEDEFname type field.
	type *real_type_ptr = 
	    ((base_type *)type_element_ptr)->typedef_type_ptr->duplicate();

	// Save the pointer to the rest of the current type list.
	type *rest_of_type_ptr = next;

	// Copy the data from the head of the real type into the current head.
	type_element_ptr = real_type_ptr->type_element_ptr;
	next = real_type_ptr->next;

	// Add the rest of the original type list to the expanded type list. 
	add_type(rest_of_type_ptr);
    }
    
    // Search down the type list for a TYPEDEFname.
    for (type *type_ptr = this; type_ptr != NULL;
	 type_ptr = type_ptr->next)

	if (type_ptr->next != NULL &&
	   (type_ptr->next->type_element_ptr->type_element_token==TYPEDEFname))
	{
	    /*
	     * The next field points to a TYPEDEFname element.  Remove the 
	     * TYPEDEFname element and insert the TYPEDEFname's real type.
	     */

	    // Get the real type from the TYPEDEFname identifier object.
	    type *typedef_ptr = type_ptr->next;
	    type *real_type_ptr =
		((base_type *)typedef_ptr->type_element_ptr)->typedef_type_ptr->duplicate();

	    // Add the real type onto the type before the TYPEDEFname object.
	    type_ptr->next = real_type_ptr;
	    
	    // Add the remander of the type list onto the end of the real type.
	    add_type(typedef_ptr->next);
	}
}

void type_element::dump_type(ostream *output_stream)
{
    switch(type_element_token)
    {
      case INT:
	*output_stream << " int";
	break;
      case FLOAT:
	*output_stream << " float";
	break;
      case CHAR:
	*output_stream << " char";
	break;
      case SHORT:
	*output_stream << " short";
	break;
      case LONG:
	*output_stream << " long";
	break;
      case DOUBLE:
	*output_stream << " double";
	break;
      case SIGNED:
	*output_stream << " signed";
	break;
      case UNSIGNED:
	*output_stream << " unsigned";
	break;
      case STRUCT:
	*output_stream << " struct ";
	*output_stream << ((base_type *)this)->type_name_id_ptr->id_string;
	break;
      case CLASS:
	*output_stream << " class ";
	*output_stream << ((base_type *)this)->type_name_id_ptr->id_string;
	break;
      case UNION:
	*output_stream << " union ";
	*output_stream << ((base_type *)this)->type_name_id_ptr->id_string;
	break;
      case ENUM:
	*output_stream << " enum ";
	*output_stream << ((base_type *)this)->type_name_id_ptr->id_string;
	break;
      case VOID:
	*output_stream << " void";
	break;
      case ARRAY:
	*output_stream << " array of";
	break;
      case FUNCTION:
	*output_stream << " function(";
	*output_stream << ") returning";
	break;
      case '*':
	*output_stream << " pointer to";
	break;
      case '&':
	*output_stream << " reference to";
	break;
      case CONST:
	*output_stream << " const";
	break;
      case VOLATILE:
	*output_stream << " volatile";
	break;
      case TYPEDEF:
	*output_stream << " typedef";
	break;
      case EXTERN:
	*output_stream << " extern";
	break;
      case STATIC:
	*output_stream << " static";
	break;
      case AUTO:
	*output_stream << " auto";
	break;
      case REGISTER:
	*output_stream << " register";
	break;
      case FRIEND:
	*output_stream << " friend";
	break;
      case OVERLOAD:
	*output_stream << " overload";
	break;
      case INLINE:
	*output_stream << " inline";
	break;
      case VIRTUAL:
	*output_stream << " virtual";
	break;
#ifdef PROXIFY
      case PROXIABLE:
	*output_stream << " proxiable";
	break;
#endif
      case NONE:
	*output_stream << " none";
	break;
      case TYPEDEFname:
	*output_stream << " TYPE";
	break;
      default:
	cerr << Me << ": Internal error in type::dump_type(): unknown type";
	cerr << " element token!: " << Me << " will abort!\n";
	exit(-1);
    }
}    

type_element *type_modifier::duplicate()
{
    type_modifier *new_type_modifier = new type_modifier(type_element_token);
    new_type_modifier->const_flag = const_flag;
    new_type_modifier->volatile_flag = volatile_flag;
    if (size != NULL)
	new_type_modifier->size = size->duplicate();
    else
	if (parameters != NULL)
	    new_type_modifier->parameters = parameters->duplicate();
    return new_type_modifier;
}

p_boolean type_modifier::compare(type_element *compare_ptr)
{
    if (type_element_token != compare_ptr->type_element_token)
	return FALSE;
    if (const_flag != compare_ptr->const_flag)
	return FALSE;
    if (volatile_flag != compare_ptr->volatile_flag)
	return FALSE;
    if (parameters != NULL && 
	((type_modifier *)compare_ptr)->parameters != NULL)
    {
       if (parameters->compare_type(((type_modifier *)compare_ptr)->parameters)
	   == FALSE)
	   return FALSE;
    }
    else
	if ((parameters != NULL && 
	     ((type_modifier *)compare_ptr)->parameters == NULL) ||
	    (parameters == NULL && 
	     ((type_modifier *)compare_ptr)->parameters != NULL))
	    return FALSE;
    
    // Deal with size. -ecl

    return TRUE;

}    
type_element *storage_class::duplicate()
{
    storage_class *new_storage_class = new storage_class(type_element_token);
    new_storage_class->const_flag = const_flag;
    new_storage_class->volatile_flag = volatile_flag;
    return new_storage_class;
}

type_element *base_type::duplicate()
{
    base_type *new_base_type = new base_type(type_element_token);
    new_base_type->const_flag = const_flag;
    new_base_type->volatile_flag = volatile_flag;
    if (typedef_type_ptr != NULL)
	new_base_type->typedef_type_ptr = typedef_type_ptr->duplicate();
    if (type_name_id_ptr != NULL)
	new_base_type->type_name_id_ptr = type_name_id_ptr;
    if (!print_body)
	new_base_type->print_body = FALSE;
    
    return new_base_type;
}

base_type::base_type(TOKEN type_token) : type_element(type_token)
{
    type_name_id_ptr = NULL;
    typedef_type_ptr = NULL;
    print_body       = TRUE;
}

base_type::base_type(TOKEN type_token, identifier *id_ptr) : type_element(type_token)
{
    type_name_id_ptr = id_ptr;
    typedef_type_ptr = NULL;
    print_body       = TRUE;
}

p_boolean base_type::compare(type_element *compare_ptr)
{
    if (type_element_token != compare_ptr->type_element_token)
	return FALSE;
    if (const_flag != compare_ptr->const_flag)
	return FALSE;
    if (volatile_flag != compare_ptr->volatile_flag)
	return FALSE;
    if (typedef_type_ptr != NULL && 
	((base_type *)compare_ptr)->typedef_type_ptr != NULL &&
	    !(typedef_type_ptr->compare(((base_type *)compare_ptr)->typedef_type_ptr)))
	return FALSE;
    if (type_name_id_ptr != NULL &&
	((base_type *)compare_ptr)->type_name_id_ptr != NULL &&
	    !(type_name_id_ptr->compare(((base_type *)compare_ptr)->type_name_id_ptr)))
	return FALSE;
    
    return TRUE;
}

type_element *csue_type_element::duplicate()
{
    csue_type_element *new_csue_type_element;
    if (csu_definition_ptr != NULL)
	new_csue_type_element = new csue_type_element(CUE, csu_definition_ptr);
    else
	new_csue_type_element = new csue_type_element(CUE,enum_definition_ptr);
    new_csue_type_element->const_flag = const_flag;
    new_csue_type_element->volatile_flag = volatile_flag;

    return new_csue_type_element;
}

p_boolean csue_type_element::compare(type_element *compare_ptr)
{
    if (type_element_token != compare_ptr->type_element_token)
	return FALSE;
    if (const_flag != compare_ptr->const_flag)
	return FALSE;
    if (volatile_flag != compare_ptr->volatile_flag)
	return FALSE;
    if (csu_definition_ptr != 
	((csue_type_element *)compare_ptr)->csu_definition_ptr)
	return FALSE;
    if (enum_definition_ptr != 
	((csue_type_element *)compare_ptr)->enum_definition_ptr)
	return FALSE;
    
    return TRUE;
}

p_boolean type::array_type()
{
    if (type_element_ptr->type_element_token == TYPEDEFname)
	return(((base_type *) type_element_ptr)->typedef_type_ptr->
					array_type());

    if (type_element_ptr->type_element_token == ARRAY ||
	type_element_ptr->type_element_token == '*')
	return TRUE;
    else
	return FALSE;
}



p_boolean type::void_type()
{
    if (type_element_ptr->type_element_token == TYPEDEFname)
	return(((base_type *) type_element_ptr)->typedef_type_ptr->
					void_type());

    if (type_element_ptr->type_element_token == VOID)
	return TRUE;
    else
	return FALSE;
}

p_boolean type::char_type()
{
    if (type_element_ptr->type_element_token == TYPEDEFname)
	return(((base_type *) type_element_ptr)->typedef_type_ptr->
					char_type());

    if (type_element_ptr->type_element_token == CHAR)
	return TRUE;
    else
	return FALSE;
}

p_boolean type::cip_type()
{
    if (type_element_ptr->type_element_token == TYPEDEFname)
	return(((base_type *) type_element_ptr)->typedef_type_ptr->
					cip_type());

    if (type_element_ptr->type_element_token == CIP)
	return TRUE;
    else
	return FALSE;
}

p_boolean type::class_object_type()
{
    if (type_element_ptr->type_element_token == TYPEDEFname)
	return(((base_type *) type_element_ptr)->typedef_type_ptr->
					class_object_type());

    type_element *element_ptr = type_element_ptr;

    if ((element_ptr->type_element_token == '&') && (next != NULL))
	element_ptr = next->type_element_ptr;

    if (element_ptr->type_element_token == CLASS ||
	element_ptr->type_element_token == STRUCT ||
	element_ptr->type_element_token == UNION)
	return TRUE;
    else
	return FALSE;
}

p_boolean type::pointer_type()
{
    if (type_element_ptr->type_element_token == TYPEDEFname)
	return(((base_type *) type_element_ptr)->typedef_type_ptr->
					pointer_type());

    if (type_element_ptr->type_element_token == '*')
	return TRUE;
    else
	return FALSE;
}

p_boolean type::ptr_or_ref_type()
{
    if (type_element_ptr->type_element_token == TYPEDEFname)
	return(((base_type *) type_element_ptr)->typedef_type_ptr->
					ptr_or_ref_type());

    if (type_element_ptr == NULL)
	return FALSE;
    if (type_element_ptr->type_element_token == '*' || 
	type_element_ptr->type_element_token == '&')
	return TRUE;
    return FALSE;
}

p_boolean type::function_type()
{
    if (type_element_ptr->type_element_token == TYPEDEFname)
	return(((base_type *) type_element_ptr)->typedef_type_ptr->
					function_type());

    if (type_element_ptr->type_element_token == FUNCTION)
	return TRUE;
    else
	return FALSE;
}

p_boolean type::integral_type()
{
    if (type_element_ptr->type_element_token == TYPEDEFname)
	return(((base_type *) type_element_ptr)->typedef_type_ptr->
					integral_type());

    if (type_element_ptr->type_element_token == LONG)
    {
	if (next == NULL)
	    return TRUE;
	else
	    if (next->type_element_ptr->type_element_token == INT)
		return TRUE;
	    else
		return FALSE;
    }
    else
	switch(type_element_ptr->type_element_token)
	{
	case CHAR:
	case SIGNED:
	case UNSIGNED:
	case INT:
	case SHORT:
	case ENUM:
	    return TRUE;
	default:
	    return FALSE;
	}
}

p_boolean type::arithmetic_type()
{
    if (type_element_ptr->type_element_token == TYPEDEFname)
	return(((base_type *) type_element_ptr)->typedef_type_ptr->
					arithmetic_type());

    if (integral_type())
	return TRUE;
    else
    {
	if (type_element_ptr->type_element_token == LONG)
	{
	    if (next != NULL && 
		next->type_element_ptr->type_element_token == DOUBLE)
		return TRUE;
            else
		return FALSE;
	}
	else
	    if (type_element_ptr->type_element_token == FLOAT ||
		type_element_ptr->type_element_token == DOUBLE)
		return TRUE;
	    else
		return FALSE;
    }
}

char *base_type::get_string()
{
    switch(type_element_token)
    {
      case INT:
	return "int";
      case FLOAT:
	return "float";
      case CHAR:
	return "char";
      case SHORT:
	return "short";
      case LONG:
	return "long";
      case DOUBLE:
	return "double";
      case SIGNED:
	return "signed";
      case UNSIGNED:
	return "unsigned";
      case VOID:
	return "void";
      default:
	cerr << Me << ": Internal error in base_type::get_string(): ";
	cerr << "unknown type element token!: " << Me << " will abort!\n";
	exit(-1);
    }
}    

type *type::get_function_return_type()
{
    if (type_element_ptr->type_element_token == TYPEDEFname)
	return(((base_type *) type_element_ptr)->typedef_type_ptr->
					get_function_return_type());

    if (type_element_ptr->type_element_token != FUNCTION ||
	next == NULL)
	return NULL;
    else
	return next->duplicate();
}

int type::get_arg_count()
{
    if (type_element_ptr->type_element_token == TYPEDEFname)
	return(((base_type *) type_element_ptr)->typedef_type_ptr->
					get_arg_count());

    if (!function_type())
	// Log an error.
	return -1;
    else
	if (((type_modifier *)type_element_ptr)->parameters == NULL)
	    return 0;
	else
	    return ((type_modifier *)type_element_ptr)->parameters->size();
}

type *type::get_array_base_type()
{
    if (type_element_ptr->type_element_token == TYPEDEFname)
	return(((base_type *) type_element_ptr)->typedef_type_ptr->
					get_array_base_type());

    if ((type_element_ptr->type_element_token == ARRAY ||
	    type_element_ptr->type_element_token == '*') && next != NULL)
	return next->duplicate();
    else
	return NULL;
}


type *type::get_dereference_type()
{
    if (type_element_ptr->type_element_token == TYPEDEFname)
	return(((base_type *) type_element_ptr)->typedef_type_ptr->
					get_dereference_type());

    if (next == NULL)
	return NULL;
    if (type_element_ptr->type_element_token == '*' && next != NULL)
	return next->duplicate();
    if (type_element_ptr->type_element_token == '&' &&
	next != NULL)
	if (next->type_element_ptr->type_element_token == '*' &&
	    next->next != NULL)
	    return next->next->duplicate();

    return NULL;
}

class_definition *type::get_class_definition()
{
    if (type_element_ptr->type_element_token == TYPEDEFname)
	return(((base_type *) type_element_ptr)->typedef_type_ptr->
					get_class_definition());

    type_element *element_ptr = type_element_ptr;

    if ((element_ptr->type_element_token == '&') && (next != NULL))
	element_ptr = next->type_element_ptr;

    if ((element_ptr->type_element_token != CLASS &&
	 element_ptr->type_element_token != STRUCT &&
	 element_ptr->type_element_token != UNION) ||
	((base_type *)element_ptr)->type_name_id_ptr == NULL)
	return NULL;
    else
	return ((base_type *)element_ptr)->type_name_id_ptr->csue_definition_ptr;
}

type *type::promote_integer()
{
    if (type_element_ptr->type_element_token == TYPEDEFname)
	return(((base_type *) type_element_ptr)->typedef_type_ptr->
					promote_integer());

    /*
     * This function is machine dependent.  The ANSI C rule is that a char,
     * a short, or an int bit field, or their unsigned varieties, will be
     * widened to int if int can represent all the values of the original type,
     * or to unsigned int otherwise.
     */

    if (!integral_type())
	return this;

    if (type_element_ptr->type_element_token == UNSIGNED)
    {
	if (next == NULL || next->type_element_ptr->type_element_token == LONG)
	    return this;
	else
	{
	    type *new_type_ptr = new type(new base_type(UNSIGNED));
	    new_type_ptr->add_type_element(new base_type(INT));
	    return new_type_ptr;
	}
    }
    else
	return new type(new base_type(INT));
}

type *type::convert_arithmetic(type *other_type_ptr)
{
    if (type_element_ptr->type_element_token == TYPEDEFname)
	return(((base_type *) type_element_ptr)->typedef_type_ptr->
		convert_arithmetic(other_type_ptr));
    if (other_type_ptr->type_element_ptr->type_element_token == TYPEDEFname)
	return(convert_arithmetic(
	    ((base_type *) other_type_ptr->type_element_ptr)->typedef_type_ptr));
    /*
     * See E&S p.34 for conversion rules.
     */

    type *new_type_ptr;
    if ((type_element_ptr->type_element_token == LONG && 
	next != NULL &&
	next->type_element_ptr->type_element_token == DOUBLE) ||
	(other_type_ptr->type_element_ptr->type_element_token == LONG && 
	 other_type_ptr->next != NULL &&
	 other_type_ptr->next->type_element_ptr->type_element_token == DOUBLE))
    {
	new_type_ptr = new type(new base_type(LONG));
	new_type_ptr->add_type_element(new base_type(DOUBLE));
	return new_type_ptr;
    }
    
    if ((type_element_ptr->type_element_token == DOUBLE) || 
	(other_type_ptr->type_element_ptr->type_element_token == DOUBLE))
    {
	return new type(new base_type(DOUBLE));
    }
    
    if ((type_element_ptr->type_element_token == FLOAT) || 
	(other_type_ptr->type_element_ptr->type_element_token == FLOAT))
    {
	return new type(new base_type(FLOAT));
    }
    
    // Perform integral promotions on both types.

    type *type1_ptr, *type2_ptr;
    type1_ptr = promote_integer();
    type2_ptr = other_type_ptr->promote_integer();
    
    if ((type1_ptr->type_element_ptr->type_element_token == UNSIGNED && 
	type1_ptr->next != NULL &&
	type1_ptr->next->type_element_ptr->type_element_token == LONG) ||
	(type2_ptr->type_element_ptr->type_element_token == UNSIGNED && 
	 type2_ptr->next != NULL &&
	 type2_ptr->next->type_element_ptr->type_element_token == LONG))
    {
	new_type_ptr = new type(new base_type(UNSIGNED));
	new_type_ptr->add_type_element(new base_type(LONG));
	return new_type_ptr;
    }
    
    if ((type1_ptr->type_element_ptr->type_element_token == LONG) || 
	(type2_ptr->type_element_ptr->type_element_token == UNSIGNED))
    {
	// Machine dependent! May need to return unsigned long int.
	return new type(new base_type(LONG));
    }
    
    if ((type1_ptr->type_element_ptr->type_element_token == UNSIGNED) || 
	(type2_ptr->type_element_ptr->type_element_token == LONG))
    {
	// Machine dependent! May need to return unsigned long int.
	return new type(new base_type(LONG));
    }
    
    if ((type1_ptr->type_element_ptr->type_element_token == LONG) || 
	(type2_ptr->type_element_ptr->type_element_token == LONG))
    {
	return new type(new base_type(LONG));
    }
    
    if ((type1_ptr->type_element_ptr->type_element_token == UNSIGNED) || 
	(type2_ptr->type_element_ptr->type_element_token == UNSIGNED))
    {
	return new type(new base_type(UNSIGNED));
    }

    return new type(new base_type(INT));
}
