/*
 * Copyright (c) 1989 The Regents of the University of California.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms are permitted provided
 * that: (1) source distributions retain this entire copyright notice and
 * comment, and (2) distributions including binaries display the following
 * acknowledgement:  ``This product includes software developed by the
 * University of California, Berkeley and its contributors'' in the
 * documentation or other materials provided with the distribution and in
 * all advertising materials mentioning features or use of this software.
 * Neither the name of the University nor the names of its contributors may
 * be used to endorse or promote products derived from this software without
 * specific prior written permission.
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

#ifndef lint
char copyright[] =
"@(#) Copyright (c) 1989 The Regents of the University of California.\n\
 All rights reserved.\n";
#endif /* not lint */

#ifndef lint
static char sccsid[] = "@(#)chhint.c	5.15 (Berkeley) 6/16/90";
#endif /* not lint */

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/fcntl.h>
#include <fts.h>
#include <stdio.h>
#include <string.h>

extern int errno;
int retval;

int vflag;
int fflag;
unsigned int cachehint;

int gethint(fd, hint)
	int fd;
	unsigned int * hint;
{
	return ioctl(fd, FIOGETHINT, (char *) hint);
}

int sethint(fd, hint)
	int fd;
	unsigned int hint;
{
	return ioctl(fd, FIOSETHINT, (char *) hint);
}

void process(name, pname)
	char * name;
	char * pname;
{
	unsigned oldhint;
	char oldstr[30];
	unsigned newhint;
	char newstr[30];
	int fd;
	fd = open(name, O_RDWR);
	if (fd < 0) {
		error(pname);
	}
	if (vflag && gethint(fd, &oldhint)) {
		error(pname);
	}
	if (fflag && sethint(fd, cachehint)) {
		error(pname);
	}
	if (vflag) {
		if (fflag) {
			if (gethint(fd, &newhint)) {
				error(pname);
			}
			hintstring(oldhint, oldstr);
			hintstring(newhint, newstr);
			printf("%11s %8x %11s %8x %s\n", 
			       oldstr, oldhint, newstr, newhint, pname);
		} else {
			hintstring(oldhint, oldstr);
			printf("%11s %8x %s\n", oldstr, oldhint, pname);
		}	
	}
	close(fd);
}

int
main(argc, argv)
	int argc;
	char **argv;
{
	extern int optind;
	register FTS *fts;
	register FTSENT *p;
	register char *hint;
	int ch, rflag;

	vflag = rflag = fflag = 0;
	while ((ch = getopt(argc, argv, "Rvf")) != EOF) {
		switch((char)ch) {
		case 'R':
			rflag++;
			break;
		case 'v':
			vflag++;
			break;
		case 'f':
			fflag++;
			break;
		case '?':
		default:
			usage();
		}
	}
	argv += optind;
	argc -= optind;

	if (argc < 1 || (argc < 2 && fflag) )
		usage();

	if (fflag) {
		hint = *argv;
		cachehint = 0;
		while (*hint) {
			char c = *hint;
			unsigned val;
			if (c >= '0' && c <= '9') {
				val = c - '0';
			} else if (c >= 'a' && c <= 'f') {
				val = (c - 'a') + 10;
			} else if (c >= 'A' && c <= 'F') {
				val = (c - 'F') + 10;
			} else {
				val = 0;
				usage();
			}
			cachehint = (cachehint << 4) + val;
			hint++;
		}
	}

	retval = 0;
	if (rflag) {
		if (!(fts = ftsopen(argv, FTS_NOSTAT|FTS_PHYSICAL, 0))) {
			(void)fprintf(stderr, "chhint: %s.\n", strerror(errno));
			exit(1);
		}
		while (p = ftsread(fts)) {
			if (p->fts_info == FTS_D)
				continue;
			if (p->fts_info == FTS_ERR) {
				error(p->fts_path);
				continue;
			}
			process(p->fts_accpath, p->fts_path);
		}
		exit(retval);
	}
	while (*argv) {
		process(*argv, *argv);
		argv++;
	}
	exit(retval);
}

int
error(name)
	char *name;
{
	(void)fprintf(stderr, "chhint: %s: %s.\n", name, strerror(errno));
	retval = 1;
	return 0;
}

int
usage()
{
	(void)fprintf(stderr, "chhint: chhint [-vR] file ...\n");
	(void)fprintf(stderr, "        chhint -f[vR] mode file ...\n");
	exit(1);
	return 0;
}
