#include "tools.h"
#include "iod_32.h"
#include "request.h"
#include <stdio.h>
#include <math.h>
#include <limits.h>

#define READ_FLAG 0x01
#define WRITE_FLAG 0x02
#define SEND_FLAG 0x04
#define RECV_FLAG 0x08
#define REOPEN_FLAG 0x10
#define RAND_FLAG 0x20
#define FULL_FLAG 0x40
#define OTHER_FLAG 0x80

#define IO_FLAG ( READ_FLAG | WRITE_FLAG | SEND_FLAG | RECV_FLAG )

#define MAX_FDS 1000
#define MAX_SYS_FDS 256

#define SIMPLE_MODE 1
#define DETAIL_MODE 2

int free_slot = MAX_SYS_FDS;
int mode = DETAIL_MODE;

struct {
  int flags;
  int offset;
  int max_offset;
  int total_transfer;
  int sequence;
  int fd;
  int link;
} fd_info[MAX_FDS];

copy_to_new_fd_info(fd)
int fd;
{
  int link;

  link = fd;
  while (fd_info[link].link != fd)
    link = fd_info[link].link;
  fd_info[link].link = free_slot;
  fd_info[free_slot].link = fd;
  fd_info[free_slot].flags = fd_info[fd].flags;
  fd_info[free_slot].offset = fd_info[fd].offset;
  fd_info[free_slot].max_offset = fd_info[fd].max_offset;
  fd_info[free_slot].total_transfer = fd_info[fd].total_transfer;
  fd_info[free_slot].sequence = fd_info[fd].sequence;
  fd_info[free_slot].fd = fd;
  fd_info[fd].flags = fd_info[fd].offset = 
    fd_info[fd].max_offset = fd_info[fd].total_transfer = 0;
  fd_info[fd].sequence++;
  free_slot++;
}

init_params(argc, argv)
int argc;
char *argv[];
{
  int i;
  if (*argv[1] == '?') {
    fprintf(stderr,"%s -[ds]\n",argv[0]);
    fprintf(stderr,"  d detail mode\n");
    fprintf(stderr,"  s simple mode\n");
    exit(0);
  }
  if (argc == 2) {
    switch (argv[1][1]) {
      case 'd': mode = DETAIL_MODE; break;
      case 's': mode = SIMPLE_MODE; break;
    }
  }
}

main(argc, argv)
int argc;
char *argv[];
{
  int cmd,fd,transfer;

  struct iod_msg msg;

  init_params(argc, argv);
  init_files();
  symtable_init();
  for (fd = 0; fd < MAX_SYS_FDS; fd++) 
    fd_info[fd].link = fd;
  while (read_msg(&msg)) {
#ifdef DEBUG
    dump_iod_msg(&msg);
#endif
    field_value(&msg, FIELD_CMD, &cmd);
    if (field_value(&msg, FIELD_FD, &fd) == TYPE_NULL) continue;
    if (fd < 0 || fd >= MAX_SYS_FDS) continue;
    if (cmd == IOD_READ || cmd == IOD_READA ||
	cmd == IOD_READL || cmd == IOD_READLA ||
	cmd == IOD_READV) {
      check_seq(fd, &msg);
      fd_info[fd].flags |= READ_FLAG;
    } else if (cmd == IOD_WRITE || cmd == IOD_WRITEA ||
	       cmd == IOD_WRITEL || cmd == IOD_WRITELA ||
	       cmd == IOD_WRITEV) {
      check_seq(fd, &msg);
      fd_info[fd].flags |= WRITE_FLAG;
    } if (cmd == IOD_RECV || cmd == IOD_RECVFROM || cmd == IOD_RECVMSG) {
      fd_info[fd].flags |= RECV_FLAG;
    } else if (cmd == IOD_SEND || cmd == IOD_SENDTO || cmd == IOD_SENDMSG) {
      fd_info[fd].flags |= SEND_FLAG;
    } else if (cmd == IOD_OPEN && (fd_info[fd].flags & IO_FLAG)) {
      if (mode == DETAIL_MODE)
        copy_to_new_fd_info(fd);
      else {
        fd_info[fd].flags |= REOPEN_FLAG;
        fd_info[fd].offset = 0;
      }
    } else {
      fd_info[fd].flags |= OTHER_FLAG;
    }
    if (field_value(&msg, FIELD_IO_ACTUAL, &transfer) != TYPE_NULL) 
      fd_info[fd].total_transfer += transfer;
  }
  for (fd = 0; fd < MAX_SYS_FDS; fd++) {
    int link;
    link = fd_info[fd].link;
    if (link != fd) {
      do {
        print_fd_info(fd,link);
        link = fd_info[link].link;
      }  while (link != fd);
    }
    print_fd_info(fd,link);
  }
}

print_fd_info(fd,slot)
int fd,slot;
{
      int flags;
      flags = fd_info[slot].flags;
      if (flags == 0) return;
      printf("FD %3d %2d ( %9d , %9d ):",fd, 
             fd_info[slot].sequence, fd_info[slot].max_offset,
             fd_info[slot].total_transfer);
      if (flags & READ_FLAG) printf(" read");
      if (flags & WRITE_FLAG) printf(" write");
      if (flags & SEND_FLAG) printf(" send");
      if (flags & RECV_FLAG) printf(" recv");
      if (flags & REOPEN_FLAG) printf(" reopen");
      if (flags & RAND_FLAG) printf(" random");
      if (flags & FULL_FLAG) printf(" full");
      if (flags & OTHER_FLAG) printf(" other");
      printf("\n");
}

check_seq(fd, msgp)
int fd;
struct iod_msg *msgp;
{
  int offset,actual;
  field_value(msgp, FIELD_IO_ACTUAL, &actual);
  field_value(msgp, FIELD_IO_OFFSET, &offset);
  if ((fd_info[fd].flags & IO_FLAG) == 0 && (fd >= 0 || fd <= 2)) 
    fd_info[fd].offset = offset;
  if (fd_info[fd].offset != offset)
    fd_info[fd].flags |= RAND_FLAG;
  if (offset == 0)
    fd_info[fd].flags |= FULL_FLAG;
  fd_info[fd].offset = offset + actual;
  if (fd_info[fd].offset > fd_info[fd].max_offset) 
    fd_info[fd].max_offset = fd_info[fd].offset;
}

static FILE *input_file;
static FILE *output_file;

int init_files()
{
  open_input_file();
  open_output_file();
}
int open_input_file()
{
  input_file = stdin;
}

int open_output_file()
{
  output_file = stdout;
}

int read_msg(msgp)
struct iod_msg *msgp;
{
  return fread((char *) msgp, sizeof(struct iod_msg), 1, input_file);
}

int write_msg(msgp)
struct iod_msg *msgp;
{
  return fwrite((char *) msgp, sizeof(struct iod_msg), 1, output_file);
}

