#include "tools.h"
#include "iod_32.h"
#include "request.h"
#include <stdio.h>
#include <math.h>
#include <limits.h>

#define SUM 0
#define READ 1
#define WRITE 2

char *names[3] = { "Sum", "Read", "Write" };
int fd_history[MAX_TIME][255][3];
int fd_active[255];
int sum_history[MAX_TIME];
char sum, sums, in_s, out_s;

init_params(argc, argv)
int argc;
char *argv[];
{
  int i;
  time_scale = 1;
  max_time = MAX_TIME;
  if (*argv[1] == '?') {
    printf("%s time_scale -sum -sums -ins -outs\n",argv[0]);
    exit(0);
  }
  if (argc > 1) time_scale = atoi(argv[1]);
  sum = sums = in_s = out_s = 1;
  for (i = 2; i < argc; i++) {
    if (strcmp(argv[i],"-sum") == 0) 
      sum = 0;
    else if (strcmp(argv[i],"-sums") == 0) 
      sums = 0;
    else if (strcmp(argv[i],"-ins") == 0) 
      in_s = 0;
    else if (strcmp(argv[i],"-outs") == 0) 
      out_s = 0;
    else {
      fprintf(stderr,"%s illegal option\n",argv[i]);
      exit(0);
    }
  }
}

main(argc, argv)
int argc;
char *argv[];
{
  int i, field, value, fd, msg_cmd;
  int t_region;
  time_type t;

  struct iod_msg *msgp;

  init_params(argc, argv);
  init_files();
  symtable_init();
  while (read_msg(msgp)) {
#ifdef DEBUG
    dump_iod_msg(msgp);
#endif
    field_value(msgp, FIELD_CMD, &msg_cmd);
    field_value(msgp, FIELD_TIME, &t);
    t_region = time_region(t);
    if (field_value(msgp, FIELD_FD, &fd) == TYPE_NULL) continue;
    if (field_value(msgp, FIELD_IO_ACTUAL, &value) == TYPE_NULL) continue;
    switch (msg_cmd) {
      case IOD_READ:
      case IOD_READA:
      case IOD_READL:
      case IOD_READLA:
      case IOD_READV:
      case IOD_RECV:
      case IOD_RECVFROM:
      case IOD_RECVMSG:
        fd_history[t_region][fd][READ] += value;
        break;
      case IOD_WRITE:
      case IOD_WRITEA:
      case IOD_WRITEL:
      case IOD_WRITELA:
      case IOD_WRITEV:
      case IOD_SEND:
      case IOD_SENDTO:
      case IOD_SENDMSG:
        fd_history[t_region][fd][WRITE] += value;
        break;
    }
    fd_history[t_region][fd][SUM] += value;
    fd_active[fd] += value;
    sum_history[t_region] += value; 
  }
  init_bar_graph("IO Size vs Time");
  if (sum) {
    printf("\n\"Sum\"\n");
    for (i = 0; i <= time_max; i++) {
      if (sum_history[i] == 0) continue;
      printf("%d %d\n",i*time_scale,
             sum_history[i]);
    }
  }
  for (fd = 0; fd < 255; fd++) {
    int k;
    if (fd_active[fd] == 0) continue;
    for (k = 0; k < 3; k++) {
      if (!sums && k == 0) continue;
      if (!in_s && k == 1) continue;
      if (!out_s && k == 2) continue;
      printf("\n\"%s FD %d\"\n", names[k], fd);
      for (i = 0; i <= time_max; i++) {
        if (fd_history[i][fd][k] == 0) continue;
        printf("%d %d\n",i*time_scale, fd_history[i][fd][k]);
      }
    }
  }
}

init_bar_graph(title)
char *title;
{
  printf("\
Device: Postscript\n\
TitleText: Input/Output Profile\n\
BarBase: 0.0\n\
NoLines: true\n\
Markers: true\n\
YLowLimit: 1.0\n\
XUnitText: Time (microsec)\n\
YUnitText: Size (bytes)\n\
LogY: true\n");
}

static FILE *input_file;
static FILE *output_file;

int init_files()
{
  open_input_file();
  open_output_file();
}
int open_input_file()
{
  input_file = stdin;
}

int open_output_file()
{
  output_file = stdout;
}

int read_msg(msgp)
struct iod_msg *msgp;
{
  return fread((char *) msgp, sizeof(struct iod_msg), 1, input_file);
}

