#include "request.h"
#include <stdio.h>
#include <limits.h>


#define MAXSIZE 500000

#define SUM 1
#define COUNT 2
#define DISTRIBUTE 3
#define TOTAL 4

int sum[MAXSIZE];
int maxsize = MAXSIZE;
int division = 1;
int max = 0;
int by = 10;
int mode;
int accum;

int two_columns;
 
main(argc,argv)
int argc;
char *argv[];
{
  int i,j,inc;
  char string[80];
  struct statistics stat;

  if (argc < 2 || argv[1][0] == '?' || argv[1][0] != '-' || argc > 4) {
    fprintf(stderr,"%s -[2][cdst][a] [no of slots] [compress by no of times]\n",
            argv[0]);
    fprintf(stderr,"  2 2-column input\n");
    fprintf(stderr,"  c percent of count of values\n");
    fprintf(stderr,"  d same as c, but do not convert to percent\n");
    fprintf(stderr,"  s percent of sum of values\n");
    fprintf(stderr,"  t same as s, but do not convert to percent\n");
    fprintf(stderr,"  a option performs accumulation\n");
    exit(1);
  }
  if (argv[1][1] == '2') {
    two_columns = 1;
    argv[1]++;
  }
  switch (argv[1][1]) {
    case 'c' : 
      mode = COUNT; break;
    case 'd' :
      mode = DISTRIBUTE; break;
    case 's' : 
      mode = SUM; break;
    case 't' : 
      mode = TOTAL; break;
    default : 
     fprintf(stderr,"%s: mode is invalid\n", argv[0]);
     break;
  }
  switch (argv[1][2]) {
    case 'a' :
      accum = 1;
      break;
  }
  if (argc > 2) {
    maxsize = atoi(argv[2]);
    if (maxsize > MAXSIZE) {
      fprintf(stderr,"Maximum [no of slots] allowed is %d\n",MAXSIZE);
      exit(1);
    }
  }
  if (argc == 4) {
    by = atoi(argv[3]);
  } 
  while (gets(string)) {
    if (two_columns) {
      sscanf(string,"%d %d", &i, &inc);
      update_stat(&stat,inc);
    } else {
      i = atoi(string);
      update_stat(&stat,i);
    }
    while ((j = i/division) >= maxsize) 
      compress(by);
    if ((mode == SUM || mode == TOTAL) && !two_columns)
      inc = i;
    else if (mode == COUNT || mode == DISTRIBUTE)
      inc = 1;
    sum[j] += inc;
    if (accum) {
      if (j > max) {
        int k;
        for (k = max+1;k <= j;k++) 
          sum[k] += sum[max];  
      } else {
        int k;
        for (k = j+1;k <= max;k++)
          sum[k] += inc;
      }
    }
#ifdef DEBUG
      printf("\n");
      dump_sum();
      printf("\n");
#endif
    if (j > max) max = j;
  }
  dump_stat(&stat);
  for (i = 0; i <= max; i++) 
    if (sum[i] > 0)
      if (mode == SUM)
        printf("%d %3.3lf\n", i * division, 
               (double) sum[i]/ (double) stat.sum * 100.0);
      else if (mode == COUNT)
        printf("%d %3.3lf\n", i * division, 
               (double) sum[i]/ (double) stat.count * 100.0);
      else if (mode == DISTRIBUTE || mode == TOTAL)
        printf("%d %d\n", i * division, sum[i]);
}

compress(by)
int by;
{
  int i,j,sum_sum;

#ifdef DEBUG
  printf("COMPRESS\n");
  dump_sum();
#endif
  for (i = 0; i < maxsize; i += by) {
    if (accum) {
      int slot;
      slot = i+by-1;
      sum_sum = sum[(slot > max ? max : slot)];
    } else {
      for (sum_sum = j = 0; j < by; j++)
        sum_sum += sum[i+j];
    }
    sum[i/by] = sum_sum;
  }
  division = division * by;
  max = max / by;
  for (i = max+1;i < maxsize;i++)
    sum[i] = 0;
#ifdef DEBUG
  dump_sum();
  printf("END COMPRESS\n");
#endif
}

dump_sum()
{
  int i;
  printf("Max %d\n",max);
  for (i = 0; i <= max+3;i++) {
    printf("\t%d\t%d\n",i,sum[i]);
  }
}


