#include "request.h"
#include <stdio.h>
#include <limits.h>
#include <math.h>

#define MAXSIZE 256

#define SUM 1
#define COUNT 2
#define DISTRIBUTE 3
#define TOTAL 4

int sum[MAXSIZE];
int max = 0;
int mode;
int accum;
int subdivide = 0;
int power2[32];
double fraction[32];

int two_columns;
 
int mylog2(i)
int i;
{
  int k;

  for (k = 0; k < 31; k++) {
    if (i >= power2[k] && i < power2[k+1]) 
      return k;
  }
  return k;
}

int log2sub(i)
int i;
{
  int k,j;

  k = j = mylog2(i);
  k -= subdivide;
  if (k >= 0)
    k = (i - power2[j]) / power2[k];
  else 
    k = (i - power2[j]) * power2[-k];
/* printf("i = %d, j = %d, k = %d\n",i,j,k); */
  return (j * power2[subdivide]) + k;
}

main(argc,argv)
int argc;
char *argv[];
{
  int i,j,inc;
  char string[80];
  struct statistics stat;

  j = 1;
  for (i = 0; i < 32; i++) { 
    power2[i] = j;
    j *= 2;
  }  
  if (argc < 2 || argv[1][0] == '?' || argv[1][0] != '-' || argc > 3) {
    fprintf(stderr,"%s -[2][cdst][a] [no of subdivisions]\n",
            argv[0]);
    fprintf(stderr,"  2 2-column input\n");
    fprintf(stderr,"  c percent of count of values\n");
    fprintf(stderr,"  d same as c, but do not convert to percent\n");
    fprintf(stderr,"  s percent of sum of values\n");
    fprintf(stderr,"  t same as s, but do not convert to percent\n");
    fprintf(stderr,"  a option performs accumulation\n");
    exit(1);
  }
  if (argv[1][1] == '2') {
    two_columns = 1;
    argv[1]++;
  }
  switch (argv[1][1]) {
    case 'c' : 
      mode = COUNT; break;
    case 'd' :
      mode = DISTRIBUTE; break;
    case 's' : 
      mode = SUM; break;
    case 't' : 
      mode = TOTAL; break;
    default : 
     fprintf(stderr,"%s: mode is invalid\n", argv[0]);
     break;
  }
  switch (argv[1][2]) {
    case 'a' :
      accum = 1;
      break;
  }
  if (argc > 2) {
    subdivide = mylog2(atoi(argv[2]));
  }
  for (i = 0; i < power2[subdivide]; i++) {
    fraction[i] = log(1.0 + (double) i / (double) power2[subdivide]) /
		  log(2.0); 
  }
  while (gets(string)) {
    if (two_columns) {
      sscanf(string,"%d %d", &i, &inc);
      update_stat(&stat,inc);
    } else {
      i = atoi(string);
      update_stat(&stat,i);
    }
    j = log2sub(i);
    if ((mode == SUM || mode == TOTAL) && !two_columns)
      inc = i;
    else if (mode == COUNT || mode == DISTRIBUTE)
      inc = 1;
    sum[j] += inc;
    if (accum) {
      if (j > max) {
        int k;
        for (k = max+1;k <= j;k++) 
          sum[k] += sum[max];  
      } else {
        int k;
        for (k = j+1;k <= max;k++)
          sum[k] += inc;
      }
    }
#ifdef DEBUG
      printf("\n");
      dump_sum();
      printf("\n");
#endif
    if (j > max) max = j;
  }
  dump_stat(&stat);
  for (i = 0; i <= max; i++) {
    double d;
    d = (double) ( i / power2[subdivide] ) + 
         fraction[ i % power2[subdivide] ];
    if (mode == SUM)
      printf("%.3lf %3.3lf\n", d,
           (double) sum[i]/ (double) stat.sum * 100.0);
    else if (mode == COUNT)
      printf("%.3lf %3.3lf\n", d,
           (double) sum[i]/ (double) stat.count * 100.0);
    else if (mode == DISTRIBUTE || mode == TOTAL)
      printf("%.3lf %d\n", d, sum[i]);
  }
}

dump_sum()
{
  int i;
  printf("Max %d\n",max);
  for (i = 0; i <= max+3;i++) {
    printf("\t%d\t%d\n",i,sum[i]);
  }
}


