#include "request.h"
#include <sys/types.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/stat.h>

#define DEBUG

#define FLAGS_CLOSE_ON_EXEC 1
#define FLAGS_CONSOLE 2

#define MAXFDS 256
#define MAXPIDS 64
#define MAXFILENAMES 256

char dummy[65536];

int replay_ninit(), replay_fork(), replay_stat(), replay_fstat();
int replay_open(), replay_creat(), replay_dup(), replay_close();
int replay_read(), replay_write(), replay_exit(), replay_unlink();
int replay_wait(), replay_fcntl(), replay_pipe(), replay_chdir();

typedef int (* pfn_t)();

#define DEBUG_DETAILS 1
#define DEBUG_WARNING 2
#define DEBUG_ALL DEBUG_DETAILS | DEBUG_WARNING 

int replay_debug = DEBUG_DETAILS;

int replay_max = MAXFDS;
int replay_map[MAXFDS] = {0,1,2};
int replay_map_flags[MAXFDS] = {FLAGS_CONSOLE,FLAGS_CONSOLE,FLAGS_CONSOLE};
int replay_pid = -1;
int replay_table_size;

char * replay_progname;
FILE * replay_file = 0;
char replay_filename[80];
char replay_logpath[80];
char replay_process_name[80];

int replay_pid_max = MAXPIDS;
int replay_pid_used = 0;
struct replay_pid_map_t {
  int pid;
  int replay_pid;
} replay_pid_map[MAXPIDS];

int
replay_pid_map_find(pid)
int pid;
{
  int i;
  for (i = 0; i < replay_pid_used; i++) {
    if (replay_pid_map[i].pid == pid)
      return replay_pid_map[i].replay_pid;
  }
  return -1;
}

int 
replay_pid_map_add(pid, replay_pid)
int pid, replay_pid;
{
  if (replay_pid_used == replay_pid_max) {
    fprintf(stderr,"pid map overflow\n");
    exit(1);
  }
  replay_pid_map[replay_pid_used].pid = pid;  
  replay_pid_map[replay_pid_used].replay_pid = replay_pid;
  replay_pid_used++;
  return replay_pid;
}

int replay_filename_max = MAXFILENAMES;
int replay_filename_used = 0;
struct replay_filename_map_t {
  char src[32];
  char dest[80];
} replay_filename_map[MAXFILENAMES];

char *
replay_filename_map_find(filename)
char * filename;
{
  int i;
  for (i = 0; i < replay_filename_used; i++) {
    if (strcmp(replay_filename_map[i].src,filename) == 0)
      return replay_filename_map[i].dest;
  }
  return filename;
}

int 
replay_filename_map_add(buf)
char * buf;
{
  int buflen;

  if (replay_filename_used == replay_filename_max) {
    fprintf(stderr,"pid map overflow\n");
    exit(1);
  }
  buflen = strlen(buf);
  if (buflen > 0 && buf[buflen-1] == '\n') {
    buf[buflen-1] = 0;
    buflen--;
  }
  if (buflen == 0)
    return 0;
  sscanf(buf,"%s %s",
	 replay_filename_map[replay_filename_used].src,
	 replay_filename_map[replay_filename_used].dest);
  replay_filename_used++;
  return 0;
}

struct replay_table_t {
  int cmd;
  pfn_t fn;
} replay_table[] = 
{ { IOD_NINITIATE, replay_ninit },
  { IOD_FORK, replay_fork },
  { IOD_STAT, replay_stat },
  { IOD_FSTAT, replay_fstat },
  { IOD_OPEN, replay_open },
  { IOD_CREAT, replay_creat },
  { IOD_DUP, replay_dup },
  { IOD_CLOSE, replay_close },
  { IOD_IOCTL, 0 },
  { IOD_READ, replay_read },
  { IOD_WRITE, replay_write },
  { IOD_READA, replay_read },
  { IOD_WRITEA, replay_write },
  { IOD_EXIT, replay_exit },
  { IOD_SOCKET, 0 },
  { IOD_BIND, 0 },
  { IOD_LISTEN, 0 },
  { IOD_ACCEPT, 0 },
  { IOD_CONNECT, 0 },
  { IOD_SOCKETPAIR, 0 },
  { IOD_SENDTO, 0 },
  { IOD_SEND, 0 },
  { IOD_SENDMSG, 0 },
  { IOD_RECVFROM, 0 },
  { IOD_RECV, 0 },
  { IOD_RECVMSG, 0 },
  { IOD_SHUTDOWN, 0 },
  { DAEMON_START, 0 },
  { IOD_READLA, replay_read },
  { IOD_READL, replay_read },
  { IOD_WRITELA, replay_write },
  { IOD_WRITEL, replay_write },
  { IOD_READV, replay_read },
  { IOD_WRITEV, replay_write },
  { IOD_FCNTL, replay_fcntl },
  { IOD_PIPE, replay_pipe },
  { IOD_UNLINK, replay_unlink },
  { IOD_VFORK, replay_fork },
  { IOD_WAIT, replay_wait },
  { XIO_OPEN, 0 },
  { XIO_CLOSE, 0 },
  { XIO_READ, 0 },
  { XIO_WRITE, 0 },
  { XIO_REQUEST, 0 },
  { XIO_REPLY, 0 },
  { XIO_EVENT, 0 },
  { XIO_FLUSH, 0 },
  { XIO_FLUSH_INFO, 0 },
  { IOD_CHDIR, replay_chdir },
  { XIO_RESERVED2, 0 },
  { XIO_RESERVED3, 0 },
  { XIO_RESERVED4, 0 },
  { XIO_RESERVED5, 0 },
  { XIO_RESERVED6, 0 },
  { XIO_RESERVED7, 0 },
  { XIO_RESERVED8, 0 },
  { XIO_RESERVED9, 0 },
  { XIO_RESERVED10, 0 },
  { XIO_RESERVED11, 0 },
  { XIO_RESERVED12, 0 },
  { CNTL_FLUSH, 0 },
  { NET_REQ, 0 }
};

int
replay_init()
{
  int i;
  for (i = 3; i < replay_max; i++) {
    replay_map_flags[i] = 0;
  }
  for (i = 3; i < replay_max; i++) {
    replay_map[i] = -1;
  }
  replay_table_size = sizeof( replay_table ) / 
	              sizeof( struct replay_table_t );
}

int
replay_open_file()
{
  if (replay_file != 0)
    fclose(replay_file);
  sprintf(replay_filename,"%s/%d",replay_logpath,replay_pid);
  fprintf(stderr,"Opening replay filename %s\n",replay_filename);
  replay_file = fopen(replay_filename,"r");
  if ( replay_file == 0 ) {
    fprintf(stderr,"ERROR: cannot open %s\n",replay_filename);
    exit(1);
  }
}

int
replay(msgp)
struct iod_msg *msgp;
{
  int cmd;
  pfn_t fn;

  cmd = msgp->cmd;
  if (replay_table[cmd].cmd != cmd) {
    fprintf(stderr,"ERROR: replay_table fatal error\n");
    exit(1);
  }
  fn = replay_table[cmd].fn;
  if (fn == 0) {
    if (replay_debug & DEBUG_WARNING) {
      fdump_iod_msg(stderr,msgp);
      fprintf(stderr,"WARNING: %d cannot replay %s\n",
	      msgp->pid,symtable_lookup_int(cmd));
    }
    return -1;
  }
  return fn(msgp);
}

int 
replay_ninit(msgp)
struct iod_msg *msgp;
{
  int i;
  strncpy(replay_process_name,iod_msg_name(msgp),
	  sizeof(replay_process_name)-1);
  if (replay_debug & DEBUG_DETAILS) {
    fprintf(stderr,"%d replaying %s\n",replay_pid,replay_process_name);
  }
  for(i = 0; i < replay_max; i++) {
    if (replay_map[i] != -1) {
      if (replay_map_flags[i] & FLAGS_CLOSE_ON_EXEC) {
        close(replay_map[i]);
        replay_map[i] = -1;
        replay_map_flags[i] = 0;
      }
    }
  }
  return 0;
}

int 
replay_fork(msgp)
struct iod_msg *msgp;
{
  int status;
  int new_pid = msgp->arg.fork_arg.new_pid;
  if (new_pid < 0)
    return 0;
  if (replay_debug & DEBUG_DETAILS) {
    fdump_iod_msg(stderr,msgp);
    fprintf(stderr,"forking\n");
  }
  status = fork();
  if (status == 0) {
    replay_pid = new_pid;
    replay_open_file();
    return 0;
  }
  if (status > 0) {
    replay_pid_map_add(new_pid,status);
    return 0;
  }
  fprintf(stderr,"ERROR: %d fork %d error\n",msgp->pid,new_pid);
  return -1;
}

int 
replay_stat(msgp)
struct iod_msg *msgp;
{
  int status;
  char filename[25];
  char *realname;
  struct stat buf;

  strncpy(filename,msgp->arg.string,24);
  filename[24] = 0;
  realname = replay_filename_map_find(filename);
  status = stat(realname,&buf);
  return 0;
}

int 
replay_chdir(msgp)
struct iod_msg *msgp;
{
  int status;
  int result;
  char filename[25];
  char *realname;

  status = msgp->fd;
  if (status < 0)
    return 0;
  strncpy(filename,msgp->arg.string,24);
  filename[24] = 0;
  realname = replay_filename_map_find(filename);
  result = chdir(realname);
  if (result != 0 && status == 0) {
    fdump_iod_msg(msgp);
    fprintf(stderr,"chdir failed\n");
    return -1;
  }
  return 0;
}

int
replay_fstat(msgp)
struct iod_msg *msgp;
{
  int fd;
  struct stat buf;
  fd = msgp->fd;
  if (fd < 0)
    return 0;
  if (replay_map[fd] == -1) {
    fdump_iod_msg(stderr,msgp);
    fprintf(stderr,"ERROR: fd not inuse\n");
    return -1;
  }
  fstat(replay_map[fd],&buf);
  return 0;
}

int
replay_pipe(msgp)
struct iod_msg *msgp;
{
  static int first = 0;
  static int fd[2] = {-1,-1};
  int assigned_fd[2];
  int status;
  int i;
  
  if (msgp->fd == -1)
    return 0;
  if (first == 0) {
    fd[0] = msgp->fd;
    first = 1;
    return 0;
  }  
  fd[1] = msgp->fd;
  first = 0;
  status = pipe(assigned_fd);
  if (status != 0) {
    fdump_iod_msg(stderr,msgp);
    fprintf(stderr,"ERROR: pipe() failed\n");
    return -1;
  }
  status = 0;
  for(i = 0; i < 2; i++) {
    if (replay_map[fd[i]] != -1) {
      fdump_iod_msg(stderr,msgp);
      fprintf(stderr,"ERROR: fd %d inuse, forced substitution\n",fd[i]);
      status = -1;
    }
    replay_map[fd[i]] = assigned_fd[i];
    replay_map_flags[fd[i]] = 0;
    if (replay_debug & DEBUG_DETAILS) {
      fprintf(stderr,"fd %d assigned to %d\n",fd[i],assigned_fd[i]);
    }
  }   
  return status;
}

int
replay_fcntl(msgp)
struct iod_msg *msgp;
{
  int fd;
  int status;
  int cmd;
  int arg;
  int assigned_fd;

  fd = msgp->fd;
  status = msgp->arg.fcntl_arg.status;
  cmd = msgp->arg.fcntl_arg.cmd;
  arg = msgp->arg.fcntl_arg.arg;
  if (status == -1)
    return 0;
  assigned_fd = replay_map[fd];
  if (assigned_fd == -1) {
    fdump_iod_msg(stderr,msgp);
    fprintf(stderr,"ERROR: fcntl on invalid fd\n");
    return -1;
  }
  switch(cmd) {
    case 2:
      if (arg & 1) {
        replay_map_flags[fd] |= FLAGS_CLOSE_ON_EXEC;
      } else {
	replay_map_flags[fd] &= ~FLAGS_CLOSE_ON_EXEC;
      }
      break;
    default:
      fdump_iod_msg(stderr,msgp);
      fprintf(stderr,"ERROR: unknown fcntl command\n");
      return -1;
  }
  return 0;
}

int
replay_open(msgp)
struct iod_msg *msgp;
{
  int assigned_fd;
  int fd;
  char filename[21];
  char *realname;
  fd = msgp->fd;
  if (fd < 0)
    return 0;
  strncpy(filename,msgp->arg.open_arg.path,20);
  filename[20] = 0;
  realname = replay_filename_map_find(filename);
  assigned_fd = open(realname,msgp->arg.open_arg.flags);
  if (fd >= 0 && assigned_fd < 0) {
    fdump_iod_msg(stderr,msgp);
    fprintf(stderr,"ERROR: cannot open file %s\n",realname);
    return -1;
  }
  if (replay_map[fd] != -1) {
    fdump_iod_msg(stderr,msgp);
    fprintf(stderr,"ERROR: fd %d slot in use, overriding it\n",fd);
  }
  replay_map[fd] = assigned_fd;
  replay_map_flags[fd] = 0;
  if (replay_debug & DEBUG_DETAILS) {
    fprintf(stderr,"fd %d assigned to %d\n",fd,assigned_fd);
  }
  return 0;
}

int
replay_creat(msgp)
struct iod_msg *msgp;
{
  int assigned_fd;
  int fd;
  char filename[21];
  char * realname;
  fd = msgp->fd;
  if (fd < 0)
    return 0;
  strncpy(filename,msgp->arg.open_arg.path,20);
  filename[20] = 0;
  realname = replay_filename_map_find(filename);
  assigned_fd = creat(realname,msgp->arg.open_arg.flags);
  if (fd >= 0 && assigned_fd < 0) {
    fdump_iod_msg(stderr,msgp);
    fprintf(stderr,"ERROR: cannot creat file %s\n",realname);
    return -1;
  }
  if (replay_map[fd] != -1) {
    fdump_iod_msg(stderr,msgp);
    fprintf(stderr,"ERROR: fd %d slot in use, overriding it\n",fd);
  }
  replay_map[fd] = assigned_fd;
  replay_map_flags[fd] = 0;
  if (replay_debug & DEBUG_DETAILS) {
    fprintf(stderr,"fd %d assigned to %d\n",fd,assigned_fd);
  }
  return 0;
}

int
replay_dup(msgp)
struct iod_msg *msgp;
{
  int fd;
  int dupfd;
  int assigned_fd;
  fd = msgp->fd;
  dupfd = msgp->arg.io_arg.actual;
  if (fd < 0)
    return 0;
  if (replay_map[dupfd] == -1) {
    fdump_iod_msg(stderr,msgp);
    fprintf(stderr,"ERROR: trying dup unused fd\n");
    return -1;
  }
  if (replay_map[fd] != -1) {
    close(replay_map[fd]);
    replay_map[fd] = -1;
    replay_map_flags[fd] = 0;
  }
  assigned_fd = dup(replay_map[dupfd]);
  if (assigned_fd < 0) {
    fdump_iod_msg(stderr,msgp);
    fprintf(stderr,"ERROR: dup failure\n");
    return -1;
  }
  replay_map[fd] = assigned_fd;
  replay_map_flags[fd] = 0;
  if (replay_debug & DEBUG_DETAILS) {
    fprintf(stderr,"fd %d assigned to %d\n",fd,assigned_fd);
  }
  return 0;
}

int
replay_close(msgp)
struct iod_msg *msgp;
{
  int fd;
  fd = msgp->fd;
  if (fd < 0)
    return 0;
  if (replay_map[fd] == -1) {
    if (replay_debug & DEBUG_WARNING) {
      fdump_iod_msg(stderr,msgp);
      fprintf(stderr,"WARNING: fd not inuse\n");
      return -1;
    }
  }
  close(replay_map[fd]);
  replay_map[fd] = -1;
  replay_map_flags[fd] = 0;
  return 0;
}

int 
replay_read(msgp)
struct iod_msg *msgp;
{
  int fd;
  int assigned_fd;
  int length;
  int actual;
  int offset;
  int cmd;
  int status;
  long current_offset;

  fd = msgp->fd;
  cmd = msgp->cmd;
  length = msgp->arg.io_arg.length;
  offset = msgp->arg.io_arg.offset;
  if (cmd == IOD_READA || cmd == IOD_READLA) {
    actual = length;
  } else {
    actual = msgp->arg.io_arg.actual; 
  }
  if (actual < 0)
    return 0;
  assigned_fd = replay_map[fd];
  if (assigned_fd == -1) {
    fdump_iod_msg(stderr,msgp);
    fprintf(stderr,"ERROR: fd not inuse\n");
    return -1;
  }
  current_offset = tell(assigned_fd);
  if (current_offset != offset && !(replay_map_flags[fd] & FLAGS_CONSOLE)) {
    status = lseek(assigned_fd,offset,SEEK_SET);
    if (status != offset) {
      fdump_iod_msg(stderr,msgp);
      fprintf(stderr,"ERROR: lseek(%d,%d) returns %d\n",
	      assigned_fd,offset,status);
      return -1;
    }
  }
  if (actual > sizeof(dummy)) {
    fdump_iod_msg(stderr,msgp);
    fprintf(stderr,"FATAL ERROR: dummy too small\n");
    exit(1);
  }
  status = read(assigned_fd,dummy,actual);
  if (status != actual) {
    fdump_iod_msg(stderr,msgp);
    fprintf(stderr,"ERROR: read returns %d should be %d\n",status,actual);
    return -1;
  }
  return 0;
}

int 
replay_write(msgp)
struct iod_msg *msgp;
{
  int fd;
  int assigned_fd;
  int length;
  int actual;
  int offset;
  int cmd;
  int status;
  long current_offset;

  fd = msgp->fd;
  cmd = msgp->cmd;
  length = msgp->arg.io_arg.length;
  offset = msgp->arg.io_arg.offset;
  if (cmd == IOD_WRITEA || cmd == IOD_WRITELA) {
    actual = length;
  } else {
    actual = msgp->arg.io_arg.actual; 
  }
  if (actual < 0)
    return 0;
  assigned_fd = replay_map[fd];
  if (assigned_fd == -1) {
    fdump_iod_msg(stderr,msgp);
    fprintf(stderr,"ERROR: fd not inuse\n");
    return -1;
  }
  current_offset = tell(assigned_fd);
  if (current_offset != offset && !(replay_map_flags[fd] & FLAGS_CONSOLE)) {
    status = lseek(assigned_fd,offset,SEEK_SET);
    if (status != offset) {
      fdump_iod_msg(stderr,msgp);
      fprintf(stderr,"ERROR: lseek(%d,%d) returns %d\n",
	      assigned_fd,offset,status);
      return -1;
    }
  }
  if (actual > sizeof(dummy)) {
    fdump_iod_msg(stderr,msgp);
    fprintf(stderr,"ERROR: dummy too small\n");
    return -1;
  }
  if (assigned_fd == 1 || assigned_fd == 2) {
    if (actual > 0) {
      int len;
      char * p;
      len = strlen(replay_process_name);
      strcpy(dummy,replay_process_name);
      if (actual > len) {
        p = dummy + len;
	len = actual-len-1;
        while(len-- > 0)
          *p++ = ' ';
      }
      dummy[actual-1] = '\n';
    }
  }
  status = write(assigned_fd,dummy,actual);
  if (status != actual) {
    fdump_iod_msg(stderr,msgp);
    fprintf(stderr,"ERROR: write(%d,%d) returns %d should be %d\n",
	    assigned_fd,actual,status,actual);
    return -1;
  }
  return 0;
}

int
replay_exit(msgp)
struct iod_msg *msgp;
{
/*
  exit(0);
*/
  return 0;
}

int 
replay_unlink(msgp)
struct iod_msg *msgp;
{
  int status;
  char filename[25];
  char *realname;

  strncpy(filename,msgp->arg.string,24);
  filename[24] = 0;
  realname = replay_filename_map_find(filename);
  status = unlink(realname);
  return 0;
}

int 
replay_wait(msgp)
struct iod_msg *msgp;
{
  int dummy;
  int options = 0;
  int replay_pid;
  int result;
  if (msgp->arg.wait_arg.result == -1)
    return 0;
  replay_pid = replay_pid_map_find(msgp->arg.wait_arg.result);
  if (replay_pid == -1) {
    fdump_iod_msg(stderr,msgp);
    fprintf(stderr,"ERROR: wait unknown pid\n");
    return -1;
  }
  result = waitpid(replay_pid,&dummy,0);
  if (result != replay_pid) {
    fdump_iod_msg(stderr,msgp);
    fprintf(stderr,"ERROR: wait return %d\n",result);
    return -1;
  }
  return 0;
}

int
init_params(argc, argv)
int argc;
char *argv[];
{
  int i;
  extern int hz;
  char buf[80];
  FILE * subfile;
  struct stat stat_buf;

  replay_progname = argv[0];
  if (*argv[1] == '?' || argc != 5) {
    fprintf(stderr,"%s hz filename_substitutions logpath pid\n",
	    replay_progname);
    exit(0);
  }

  hz = atoi(argv[1]);
  if (hz == 0) {
    fprintf(stderr,"ERROR: hz != 0\n",hz);
    exit(1);
  }

  fprintf(stderr,"Processing filename substitutions (%s)\n",argv[2]);
  subfile = fopen(argv[2],"r");
  if (subfile == 0) {
    fprintf(stderr,"ERROR: cannot open %s\n",argv[2]);
    exit(1);
  }
  while(fgets(buf,sizeof(buf),subfile) != 0) {
    replay_filename_map_add(buf);
  }
  fclose(subfile);

  strcpy(replay_logpath,argv[3]);
  if (stat(replay_logpath,&stat_buf) != 0) {
    fprintf(stderr,"ERROR: %s does not exist\n",replay_logpath);
    exit(1);
  }

  replay_pid = atoi(argv[4]);
  replay_open_file();
}

int
main(argc, argv)
int argc;
char *argv[];
{
  struct iod_msg msg;

  init_params(argc, argv);
  symtable_init();
  replay_init();
  while (fread((char *) &msg, sizeof(struct iod_msg), 1, replay_file)) {
    if (replay_debug & DEBUG_DETAILS)
      fdump_iod_msg(stderr,&msg);
    if (msg.pid != replay_pid) {
      fdump_iod_msg(stderr,&msg);
      fprintf(stderr,"ERROR: incorrect pid\n");
      exit(1);
    }
    replay(&msg);
  }
  fclose(replay_file);
  exit(0);
}




