#include "request.h"
#include <stdio.h>
#include <math.h>
#include <limits.h>
#ifdef unicos
#ifdef CRAY1
#include <sys/machd.h>
#endif
#ifdef CRAY2
#include <sys/param.h>
#endif
#endif

#define HZ 60 /* sun default */

#ifndef HZ
#define HZ 243902439L   /* 4.1 ns default */
#endif

int hz = HZ;
int time_scale;
int time_max;
int max_time;

int time_region(t)
time_type t;
{
  int i;
  static int count = 0;

  if (time_scale == 0) 
    i = (int) t;
  else
    i = (int) ((t * ((time_type) 1000000 / (time_type) time_scale)) / 
               (time_type) hz) ;
  if (max_time != 0 && i >= max_time) {
    fprintf(stderr,"Time out of range\n");
    count++;
    if (count > 20) {
      fprintf(stderr,"Too many Time out of ranges, exiting...\n");
      exit(1);
    }
    i = max_time - 1;
  }
  if (i > time_max) time_max = i;
  return i;
}

fdump_iod_msg(stream,msgp)
FILE * stream;
struct iod_msg *msgp;
{
  int class;
  char usname[64], prname[64];

  fprintf(stream,"%-5d %s\t", msgp->pid, symtable_lookup_int(msgp->cmd));
  if (msgp->cmd == IOD_NINITIATE) {
  	sscanf(iod_msg_name(msgp), "%s%s", usname, prname);
  	fprintf(stream,"%-10s %-24s\n", usname, prname);
	return;
  }
  fprintf(stream,"%12d",time_region(iod_msg_time(msgp)));
  if (msgp->cmd == IOD_EXIT) {
		fprintf(stream,"\n");
		return;
  }
  fprintf(stream,"\t%-2d", msgp->fd);
  field_value(msgp, FIELD_CLASS, &class);
  switch (class) {
  	case CLASS_FORK:
  		fprintf(stream,"\t%d\n", msgp->arg.fork_arg.new_pid);
  		break;
  	case CLASS_FD:
  		fprintf(stream,"\n");
  		break;
  	case CLASS_FD_AND_NAME:
		if (msgp->cmd == IOD_OPEN || msgp->cmd == IOD_CREAT) {
			fprintf(stream,"\t%-20s\t%x\n", msgp->arg.open_arg.path,
			       msgp->arg.open_arg.flags);
		} else {
			fprintf(stream,"\t%-24s\n", msgp->arg.string);
		}
  		break;
  	case CLASS_DUP:
  		fprintf(stream,"\t%d\n", msgp->arg.io_arg.actual);
  		break;
	case CLASS_IO_SYNC:
  		fprintf(stream,"\t%d\t%d\t%d\n", 
  		       (int)msgp->arg.io_arg.length, 
  		       (int)msgp->arg.io_arg.actual,
  		       (int)msgp->arg.io_arg.offset);
  		break;
	case CLASS_IO_ASYNC:
  		fprintf(stream,"\t%d\t%d\t%d\n", 
  		       (int)msgp->arg.io_arg.length,
  		       (int)msgp->arg.io_arg.length,
  		       (int)msgp->arg.io_arg.offset);
  		break;
  	case CLASS_IOCTL:
  		fprintf(stream,"\t%x\n", msgp->arg.ioctl_arg.request);
  		break;
  	case CLASS_FCNTL:
  		fprintf(stream,"\t%x\t%x\t%x\n", 
		       msgp->arg.fcntl_arg.cmd,
		       msgp->arg.fcntl_arg.arg, 
		       msgp->arg.fcntl_arg.status);
  		break;
        case CLASS_WAIT:
  		fprintf(stream,"\t%d\t%d\t%x\t%x\n", 
		       msgp->arg.wait_arg.result,
		       msgp->arg.wait_arg.pid,
		       msgp->arg.wait_arg.status,
		       msgp->arg.wait_arg.options);
  		break;
	case CLASS_SOCKET:
  		fprintf(stream,"\t%d\t%d\t%d\n", 
  		       (int)msgp->arg.soc_arg.af, 
  		       (int)msgp->arg.soc_arg.protocol,
  		       (int)msgp->arg.soc_arg.type);
  		break;
  	case CLASS_ACCEPT:
  		fprintf(stream,"\t%d\n", msgp->arg.accept_arg.old_fd);
  		break;
  	case CLASS_SHUTDOWN:
  		fprintf(stream,"\t%d\n", msgp->arg.shutdown_arg.how);
  		break;
  	case CLASS_LISTEN:
  		fprintf(stream,"\t%d\n", msgp->arg.listen_arg.backlog);
  		break;
  	default:
  		fprintf(stream,"\n");
  		break;
  }
}

dump_iod_msg(msgp)
struct iod_msg *msgp;
{
  fdump_iod_msg(stdout,msgp);
}

dump_stat(stat)
struct statistics *stat;
{
  printf(
  "sum = %d, count = %d, max = %d, min = %d, mean = %.2lf, std_dev = %.2lf\n",
  stat->sum, stat->count, stat->max, stat->min, mean_stat(stat), 
  std_dev_stat(stat) );
}

double std_dev_stat(stat)
struct statistics *stat;
{
  double mean;
  double f;
  double sqrt();

  if (stat->count == 0) return (double) 0.0;
  mean = mean_stat(stat);
  f = (double) stat->sum_sqr / (double) stat->count - mean * mean;   
  return sqrt (f < (double) 0.0 ? -f : f);
}

double mean_stat(stat)
struct statistics *stat;
{
  if (stat->count == 0) return (double) 0.0;
  return (double) stat->sum / (double) stat->count;
}

update_stat(stat, value)
struct statistics *stat;
int value;
{
  stat->sum_sqr += (double) value * (double) value;
  stat->sum += value;
  if (stat->count == 0) stat->min = INT_MAX;
  stat->count++;
  if (value < stat->min) stat->min = value;
  if (value > stat->max) stat->max = value;
}

