#include "tools.h"
#include "iod_32.h"
#include "request.h"
#include <stdio.h>
#include <math.h>
#include <limits.h>

char reads;
char writes;
char sends;
char recvs;
char others;

init_params(argc, argv)
int argc;
char *argv[];
{
  int i;
  if (*argv[1] == '?') {
    printf("%s reads writes sends recvs others\n",argv[0]);
    exit(0);
  }
  for (i = 1; i < argc; i++) {
    if (strcmp(argv[i], "reads") == 0) 
      reads = 1;
    else if (strcmp(argv[i], "writes") == 0)
      writes = 1;
    else if (strcmp(argv[i], "sends") == 0)
      sends = 1;
    else if (strcmp(argv[i], "recvs") == 0)
      recvs = 1;
    else if (strcmp(argv[i], "others") == 0)
      others = 1;
    else {
      fprintf(stderr, "Bad argument %s\n", argv[i]);
      exit(0);
    }
  }
}

main(argc, argv)
int argc;
char *argv[];
{
  int cmd;

  struct iod_msg msg;

  init_params(argc, argv);
  init_files();
  symtable_init();
  while (read_msg(&msg)) {
#ifdef DEBUG
    dump_iod_msg(&msg);
#endif
    field_value(&msg, FIELD_CMD, &cmd);
    if ( (reads && (cmd == IOD_READ || cmd == IOD_READA ||
		    cmd == IOD_READL || cmd == IOD_READLA ||
		    cmd == IOD_READV )) ||
         (writes && (cmd == IOD_WRITE || cmd == IOD_WRITEA ||
		     cmd == IOD_WRITEL || cmd == IOD_WRITELA ||
		     cmd == IOD_WRITEV )) ||
         (recvs && (cmd == IOD_RECV || cmd == IOD_RECVFROM || 
                    cmd == IOD_RECVMSG)) ||
         (sends && (cmd == IOD_SEND || cmd == IOD_SENDTO || 
                    cmd == IOD_SENDMSG)) ||
         (others && (cmd != IOD_READ && cmd != IOD_READA &&
		     cmd != IOD_READL && cmd != IOD_READLA &&
		     cmd != IOD_READV &&
		     cmd != IOD_WRITE && cmd != IOD_WRITEA &&
		     cmd != IOD_WRITEL && cmd != IOD_WRITELA &&
		     cmd != IOD_WRITEV &&
		     cmd != IOD_RECV && cmd != IOD_RECVFROM && 
		     cmd != IOD_RECVMSG && cmd != IOD_SEND &&
		     cmd != IOD_SENDTO && cmd != IOD_SENDMSG)) )
      write_msg(&msg);
  }
}

static FILE *input_file;
static FILE *output_file;

int init_files()
{
  open_input_file();
  open_output_file();
}
int open_input_file()
{
  input_file = stdin;
}

int open_output_file()
{
  output_file = stdout;
}

int read_msg(msgp)
struct iod_msg *msgp;
{
  return fread((char *) msgp, sizeof(struct iod_msg), 1, input_file);
}

int write_msg(msgp)
struct iod_msg *msgp;
{
  return fwrite((char *) msgp, sizeof(struct iod_msg), 1, output_file);
}

