/* Field extraction routine */

#include "iod_32.h"
#include "tools.h"

static int _ninit(), _fork(), _fd_only(), _fd_and_name_only(), _exit();
static int _dup(), _io_sync(), _io_async(), _ioctl(), _socket(), _accept();
static int _shutdown(), _listen(), _default(), _common(), _fcntl(), _wait();


int all_fields[] = 
{ 	FIELD_PID,
        FIELD_CMD,
	FIELD_USERNAME,
	FIELD_PROGNAME, 
	FIELD_NAME,
	FIELD_FD, 
	FIELD_DUP_FD, 
	FIELD_IO_ACTUAL,
	FIELD_IO_LENGTH,
	FIELD_IO_OFFSET,
	FIELD_IOCTL, 
	FIELD_SOC_AF,
	FIELD_SOC_PROTO,
	FIELD_SOC_TYPE,
	FIELD_SHUTDOWN,
	FIELD_ACCEPT_FD,
	FIELD_LISTEN, 
	FIELD_FORK_PID,
	FIELD_CLASS,
   	FIELD_TIME,
	FIELD_STIME,
	FIELD_FCNTL_CMD,
	FIELD_FCNTL_ARG,
	FIELD_FCNTL_STATUS,
	FIELD_WAIT_RESULT,
	FIELD_WAIT_PID,
	FIELD_WAIT_STATUS,
	FIELD_WAIT_OPTIONS,
	0
};

int field_type(field)
int field;
{
  static int field_types[] = {
	TYPE_INT,
	TYPE_SYMBOL,
	TYPE_STR,
	TYPE_STR,
	TYPE_STR,
	TYPE_INT,
	TYPE_INT,
	TYPE_INT,
	TYPE_INT,
	TYPE_INT,
	TYPE_INT,
	TYPE_INT,
	TYPE_INT,
	TYPE_INT,
	TYPE_INT,
	TYPE_INT,
	TYPE_INT,
	TYPE_INT,
	TYPE_SYMBOL,
        TYPE_TIME, 
	TYPE_TIME,
	TYPE_INT,
	TYPE_INT,
	TYPE_INT,
	TYPE_INT,
	TYPE_INT,
	TYPE_INT,
	TYPE_INT,
	0
  };
  return field_types[field - FIELD_PID];
}

#ifdef DEBUG
field_value(msgp, field, value)
#else
_field_value(msgp, field, value)
#endif
struct iod_msg *msgp;
int field;
int *value;
{
  int i;
  i = _field_value(msgp, field, value);
  printf("%s ->", symtable_lookup_int(field));
  switch (i) {
    case TYPE_INT: 
      printf("(int) %d\n", *value);
      break;
    case TYPE_SYMBOL:
      printf("(symbol) %s\n", symtable_lookup_int(*value));
      break;
    case TYPE_STR:
      printf("(string) %s\n", value);
      break;
    case TYPE_NULL:
      printf("(NULL)\n");
      break;
  }
  return i;
}
 	 
#ifdef DEBUG
_field_value(msgp, field, value)
#else
field_value(msgp, field, value)
#endif
struct iod_msg *msgp;
int field;
void *value;
{
  if (_common( msgp, field, value )) 
    return 1;
  switch (msgp->cmd) {
    case IOD_NINITIATE: return _ninit( msgp, field, value );
    case IOD_VFORK:
    case IOD_FORK:	
			return _fork( msgp, field, value );
    case IOD_FSTAT:
    case IOD_CLOSE:
    case IOD_PIPE:
    			return _fd_only( msgp, field, value );
    case IOD_CHDIR:
    case IOD_RMDIR:
    case IOD_MKDIR:
    case IOD_LINK:
    case IOD_SYMLINK:
    case IOD_RENAME:
    case IOD_STAT:
    case IOD_UNLINK:
    case IOD_OPEN:
    case IOD_CREAT:
    case IOD_BIND:
    case IOD_CONNECT:
			return _fd_and_name_only( msgp, field, value );
    case IOD_EXIT:      
			return _exit( msgp, field, value ); 
    case IOD_DUP:	
			return _dup( msgp, field, value );
    case IOD_READ:
    case IOD_WRITE:
    case IOD_READL:
    case IOD_WRITEL:
    case IOD_READV:
    case IOD_WRITEV:
    case IOD_SENDTO:
    case IOD_SEND:
    case IOD_SENDMSG:
    case IOD_RECVFROM:
    case IOD_RECV:
    case IOD_RECVMSG:
			return _io_sync( msgp, field, value );
    case IOD_READA:
    case IOD_WRITEA:
    case IOD_READLA:
    case IOD_WRITELA:
			return _io_async( msgp, field, value );
    case IOD_IOCTL:
			return _ioctl( msgp, field, value );
    case IOD_FCNTL:
			return _fcntl( msgp, field, value );
    case IOD_WAIT:
		        return _wait( msgp, field, value );
    case IOD_SOCKET:
    case IOD_SOCKETPAIR:
			return _socket( msgp, field, value );
    case IOD_ACCEPT:	return _accept( msgp, field, value );
    case IOD_SHUTDOWN:  return _shutdown( msgp, field, value );
    case IOD_LISTEN:	return _listen( msgp, field, value );
    case DAEMON_START:
    default:		return _default( msgp, field, value );
  }
}

static int _common( msgp, field, value )
struct iod_msg *msgp;
int field;
int *value;
{
  switch (field) {
    case FIELD_PID:	
	*value = msgp->pid; 
	return TYPE_INT;
    case FIELD_CMD:	
	*value = msgp->cmd; 
	return TYPE_SYMBOL;
    case FIELD_TIME:
	*((time_type *) value) = (msgp->cmd == IOD_NINITIATE ? 0 :
			      iod_msg_time(msgp));
	return TYPE_TIME;
    case FIELD_STIME:
	*((time_type *) value) = (msgp->cmd == IOD_NINITIATE ? 0 :
			      iod_msg_stime(msgp));
	return TYPE_TIME;
    default:		
	return TYPE_NULL;
  }
}

static int _ninit( msgp, field, value )
struct iod_msg *msgp;
int field;
char *value;
{
  char usname[16];
  char prname[16];
  char *s;
  char *d;

  switch (field) {
    case FIELD_USERNAME:
	s = iod_msg_name(msgp);
	d = usname;
	while( isspace(*s) && *s != 0 ) s++;
	while( !isspace(*s) && *s != 0 ) *d++ = *s++;
	*d = 0;
       	strcpy( value, usname );
	return TYPE_STR;
    case FIELD_PROGNAME:
	s = iod_msg_name(msgp);
	d = prname;
	while( isspace(*s) && *s != 0 ) s++;
	while( !isspace(*s) && *s != 0 ) s++;
        while( isspace(*s) && *s != 0) s++;
	while( !isspace(*s) && *s != 0 ) *d++ = *s++;
	*d = 0;
	strncpy( value, prname );
	return TYPE_STR;
    case FIELD_CLASS:
	*value = CLASS_NINIT;
	return TYPE_SYMBOL;
    default:		
	return TYPE_NULL;
  }
}

static int _fork( msgp, field, value )
struct iod_msg *msgp;
int field;
int *value;
{
  switch (field) {
    case FIELD_FORK_PID:
	*value = msgp->arg.fork_arg.new_pid;
	return TYPE_INT;
    case FIELD_CLASS:
	*value = CLASS_FORK;
	return TYPE_SYMBOL;
    default:
	return TYPE_NULL;
  }
}

static int _fd_only( msgp, field, value )
struct iod_msg *msgp;
int field;
int *value;
{
  switch (field) {
    case FIELD_FD:
	*value = msgp->fd;
	return TYPE_INT;
    case FIELD_CLASS:
	*value = CLASS_FD;
	return TYPE_SYMBOL;
    default:
	return TYPE_NULL;
  }
}

static int _fd_and_name_only( msgp, field, value )
struct iod_msg *msgp;
int field;
int *value;
{
  switch (field) {
    case FIELD_FD:
	*value = msgp->fd;
	return TYPE_INT;
    case FIELD_NAME:
	strncpy( (char *) value, msgp->arg.string, 24 );
	return TYPE_STR;
    case FIELD_CLASS:
	*value = CLASS_FD_AND_NAME;
	return TYPE_SYMBOL;
    default:
	return TYPE_NULL;
  }
}

static int _exit( msgp, field, value )
struct iod_msg *msgp;
int field;
int *value;
{
  switch (field) {
    case FIELD_CLASS:
	*value = CLASS_EXIT;
	return TYPE_SYMBOL;
    default:
	return TYPE_NULL;
  }
}

static int _dup( msgp, field, value )
struct iod_msg *msgp;
int field;
int *value;
{
  switch (field) {
    case FIELD_FD:
	*value = msgp->fd;
	return TYPE_INT;
    case FIELD_DUP_FD:
	*value = msgp->arg.io_arg.actual;
	return TYPE_INT;
    case FIELD_CLASS:
	*value = CLASS_DUP;
	return TYPE_SYMBOL;
    default:
	return TYPE_NULL;
  }
}

static int _io_sync( msgp, field, value )
struct iod_msg *msgp;
int field;
int *value;
{
  switch (field) {
    case FIELD_FD:
	*value = msgp->fd;
	return TYPE_INT;
    case FIELD_IO_ACTUAL:
	*value = msgp->arg.io_arg.actual;
	return TYPE_INT;
    case FIELD_IO_LENGTH:
	*value = msgp->arg.io_arg.length;
	return TYPE_INT;
    case FIELD_IO_OFFSET:
	*value = msgp->arg.io_arg.offset;
	return TYPE_INT;
    case FIELD_CLASS:
	*value = CLASS_IO_SYNC;
	return TYPE_SYMBOL;
    default:
	return TYPE_NULL;
  }
}

static int _io_async( msgp, field, value )
struct iod_msg *msgp;
int field;
int *value;
{
  switch (field) {
    case FIELD_FD:
	*value = msgp->fd;
	return TYPE_INT;
    case FIELD_IO_ACTUAL:
    case FIELD_IO_LENGTH:
	*value = msgp->arg.io_arg.length;
	return TYPE_INT;
    case FIELD_IO_OFFSET:
	*value = msgp->arg.io_arg.offset;
	return TYPE_INT;
    case FIELD_CLASS:
	*value = CLASS_IO_ASYNC;
	return TYPE_SYMBOL;
    default:
	return TYPE_NULL;
  }
}

static int _ioctl( msgp, field, value )
struct iod_msg *msgp;
int field;
int *value;
{
  switch (field) {
    case FIELD_FD:
	*value = msgp->fd;
	return TYPE_INT;
    case FIELD_IOCTL:
	*value = msgp->arg.ioctl_arg.request;
	return TYPE_INT;
    case FIELD_CLASS:
	*value = CLASS_IOCTL;
	return TYPE_SYMBOL;
    default:
	return TYPE_NULL;
  }
}

static int _fcntl( msgp, field, value )
struct iod_msg *msgp;
int field;
int *value;
{
  switch (field) {
    case FIELD_FD:
	*value = msgp->fd;
	return TYPE_INT;
    case FIELD_FCNTL_CMD:
	*value = msgp->arg.fcntl_arg.cmd;
	return TYPE_INT;
    case FIELD_FCNTL_ARG:
	*value = msgp->arg.fcntl_arg.arg;
	return TYPE_INT;
    case FIELD_FCNTL_STATUS:
	*value = msgp->arg.fcntl_arg.status;
	return TYPE_INT;
    case FIELD_CLASS:
	*value = CLASS_FCNTL;
	return TYPE_SYMBOL;
    default:
	return TYPE_NULL;
  }
}

static int _wait( msgp, field, value )
struct iod_msg *msgp;
int field;
int *value;
{
  switch (field) {
    case FIELD_FD:
	*value = msgp->fd;
	return TYPE_INT;
    case FIELD_WAIT_RESULT:
	*value = msgp->arg.wait_arg.result;
	return TYPE_INT;
    case FIELD_WAIT_PID:
	*value = msgp->arg.wait_arg.pid;
	return TYPE_INT;
    case FIELD_WAIT_STATUS:
	*value = msgp->arg.wait_arg.status;
	return TYPE_INT;
    case FIELD_WAIT_OPTIONS:
	*value = msgp->arg.wait_arg.options;
	return TYPE_INT;	
    case FIELD_CLASS:
	*value = CLASS_WAIT;
	return TYPE_SYMBOL;
    default:
	return TYPE_NULL;
  }
}

static int _socket( msgp, field, value )
struct iod_msg *msgp;
int field;
int *value;
{
  switch (field) {
    case FIELD_FD:
	*value = msgp->fd;
	return TYPE_INT;
    case FIELD_SOC_AF:
	*value = msgp->arg.soc_arg.af;
	return TYPE_INT;
    case FIELD_SOC_PROTO:
	*value = msgp->arg.soc_arg.protocol;
	return TYPE_INT;
    case FIELD_SOC_TYPE:
	*value = msgp->arg.soc_arg.type;
	return TYPE_INT;
    case FIELD_CLASS:
	*value = CLASS_SOCKET;
	return TYPE_SYMBOL;
    default:
	return TYPE_NULL;
  }
}

static int _accept( msgp, field, value )
struct iod_msg *msgp;
int field;
int *value;
{
  switch (field) {
    case FIELD_FD:
	*value = msgp->fd;
	return TYPE_INT;
    case FIELD_ACCEPT_FD:
	*value = msgp->arg.accept_arg.old_fd;
	return TYPE_INT;
    case FIELD_CLASS:
	*value = CLASS_ACCEPT;
	return TYPE_SYMBOL;
    default:
	return TYPE_NULL;
  }
}

static int _shutdown( msgp, field, value )
struct iod_msg *msgp;
int field;
int *value;
{
  switch (field) {
    case FIELD_FD:
	*value = msgp->fd;
	return TYPE_INT;
    case FIELD_SHUTDOWN:
	*value = msgp->arg.shutdown_arg.how;
	return TYPE_INT;
    case FIELD_CLASS:
	*value = CLASS_SHUTDOWN;
	return TYPE_SYMBOL;
    default:
	return TYPE_NULL;
  }
}

static int _listen( msgp, field, value )
struct iod_msg *msgp;
int field;
int *value;
{
  switch (field) {
    case FIELD_FD:
	*value = msgp->fd;
	return TYPE_INT;
    case FIELD_LISTEN:
	*value = msgp->arg.listen_arg.backlog;
	return TYPE_INT;
    case FIELD_CLASS:
	*value = CLASS_LISTEN;
	return TYPE_SYMBOL;
    default:
	return TYPE_NULL;
  }
}

static int _default( msgp, field, value )
struct iod_msg *msgp;
int field;
int *value;
{
  switch (field) {
    case FIELD_CLASS:
	*value = CLASS_RESERVED;
	return TYPE_SYMBOL;
    default:
	return TYPE_NULL;
  }
}

