/*
 *  iod_32.h -- 64 to 32 data conversion routine Include file
 *
 */

#ifndef iod_32_h
#define iod_32_h

#ifdef CRAY 
#define LITTLE_ENDIAN 
#define BITS_64
#endif
#ifdef hpux
#define LITTLE_ENDIAN
#define BITS_32
#endif
#ifdef sun
#define LITTLE_ENDIAN
#define BITS_32
#endif

#ifdef LITTLE_ENDIAN

#ifdef BITS_64
#define B32_32(x) unsigned int x:32
#define B32_32S(x) int x:32
#define B64_32(x) unsigned int x
#define TIME(x) unsigned int x
#define iod_msg_time(x) x->tm 
#define iod_msg_stime(x) x->stm 
#define iod_msg_name(x) ((char *) &x->tm)
#define iod_msg_print_time "%d"
#define time_type unsigned int
#endif /* BITS_64 */

#ifdef BITS_32
#define B32_32(x) unsigned int x
#define B32_32S(x) int x
#ifdef __STDC__
#define B64_32(x) unsigned int high_##x; unsigned int x
#define TIME(x) unsigned int high_##x; unsigned int x
#else
#define B64_32(x) unsigned int high_/**/x; unsigned int x
#define TIME(x) unsigned int high_/**/x; unsigned int x
#endif
#ifdef FLOAT
#define iod_msg_shift_factor (double)4294967296.0 
#define iod_msg_time(x) ((double) x->high_tm * iod_msg_shift_factor + x->tm)
#define iod_msg_stime(x) ((double) x->high_stm * iod_msg_shift_factor + x->stm)
#define iod_msg_print_time "%lf"
#define time_type double
#else /* FLOAT */
#define iod_msg_time(x) (x->tm)
#define iod_msg_stime(x) (x->stm)
#define iod_msg_print_time "%d"
#define time_type unsigned 
#endif /* FLOAT */
#define iod_msg_name(x) ((char *) &x->high_tm)
#endif /* BITS_32 */

#endif /* LITTLE_ENDIAN */

enum iod_cmd {

    /*  define the system calls that we are logging right now */
	
	IOD_NINITIATE,
	IOD_FORK,
	IOD_STAT,
	IOD_FSTAT,
	IOD_OPEN,
	IOD_CREAT,
	IOD_DUP,
	IOD_CLOSE,
	IOD_IOCTL,
	IOD_READ,
	IOD_WRITE,
	IOD_READA,
	IOD_WRITEA,
	IOD_EXIT,

    /*  network system calls */

	IOD_SOCKET,
	IOD_BIND,
	IOD_LISTEN,
	IOD_ACCEPT,
	IOD_CONNECT,
	IOD_SOCKETPAIR,
	IOD_SENDTO,
	IOD_SEND,
	IOD_SENDMSG,
	IOD_RECVFROM,
	IOD_RECV,
	IOD_RECVMSG,
	IOD_SHUTDOWN,

    /*  reserve some for future system calls that we might log */

        DAEMON_START,
	IOD_READLA,
	IOD_READL,
	IOD_WRITELA,
	IOD_WRITEL,
	IOD_READV,
	IOD_WRITEV,
	IOD_FCNTL,
	IOD_PIPE,
	IOD_UNLINK,
	IOD_VFORK,
	IOD_WAIT,

    /*  X11 stuff, which we don't log right now, but might */

    	XIO_OPEN,
    	XIO_CLOSE,
    	XIO_READ,
    	XIO_WRITE,
    	XIO_REQUEST,
    	XIO_REPLY,
    	XIO_EVENT,
    	XIO_FLUSH,
    	XIO_FLUSH_INFO,

	IOD_CHDIR,
	IOD_LINK,
	IOD_SYMLINK,
 	IOD_RENAME,
	IOD_MKDIR,
	IOD_RMDIR,
	IOD_LSTAT,
	XIO_RESERVED8,
	XIO_RESERVED9,
	XIO_RESERVED10,
	XIO_RESERVED11,
	XIO_RESERVED12,

   /*  control messages used by the daemon, control messages
	are never saved by the daemon into the log file        */

    	CNTL_FLUSH,

   /* added on Nov. 15 by Scott Cattanach */
        NET_REQ
	
};


/*  Structure of data being logged by this tool 		*/

struct s_open {
	char path[20];		/* path of file			*/
	B32_32(flags);		/* flags			*/
};

struct s_io {
	B32_32(length);        	/* length read or write 	*/
	B32_32S(actual);	/* actual no of bytes		*/
    	B64_32(offset);      	/* current file offset 		*/
};

struct s_ioctl {
	B64_32(request);	/* request from ioctl		*/
	B64_32(status);		/* return value			*/
};

struct s_fcntl {
	B64_32(cmd);		/* cmd				*/
	B64_32(arg);		/* arg				*/
	B64_32(status);		/* status			*/
};

struct s_wait {
	B32_32(result);
	B32_32(pid);
	B32_32(status);
	B32_32(options);
};
	
struct s_xio {
    	B64_32(length);		/* size of packet		*/
    	B64_32(packet_type);	/* X packet type		*/
};

struct s_fork {
	B64_32(new_pid);	/* new pid			*/
};

struct s_socket {
	B32_32(af);		/* address format		*/
	B32_32(protocol);	/* protocol			*/
	B64_32(type);		/* type				*/
};

struct s_listen {
	B64_32(backlog);	/* backlog 			*/
};

struct s_accept {
	B64_32(old_fd);		/* old fd			*/
};

struct s_shutdown {
	B64_32(how);		/* how to shutdown		*/
};

struct iod_msg {
	B64_32(pid);		/* the process id 		*/
	B64_32(cmd);		/* event being logged		*/
	TIME(tm);		/* time of operation		*/
	B64_32(fd);		/* the fd of interest		*/
	union {			/* arguments			*/
            struct s_io io_arg;  
	    struct s_xio xio_arg;
	    struct s_ioctl ioctl_arg;
	    struct s_fcntl fcntl_arg;
	    struct s_open open_arg;
	    struct s_fork fork_arg;
	    struct s_socket soc_arg;	
	    struct s_listen listen_arg;
	    struct s_accept accept_arg;
	    struct s_shutdown shutdown_arg;
	    struct s_wait wait_arg;
	    char string[24];
	} arg;
	TIME(stm);
};

#define CHECKSIGN_32(x) ((x) >= 0x80000000 ? (x) - 0x100000000 : (x))


#endif /* iod_32_h */
