#include <stdio.h>
#include "iod_32.h"
#include "tools.h"

#define HASH_TABLE_SIZE 997

static struct entry {
  int int_value;
  char *str_value;
} table[] = {

	{ FIELD_PID, "pid" },
	{ FIELD_CMD, "cmd" },
	{ FIELD_USERNAME, "username" },
	{ FIELD_PROGNAME, "progname" },
	{ FIELD_NAME, "name" },
	{ FIELD_FD, "fd" },
	{ FIELD_DUP_FD, "dup_fd" },
	{ FIELD_IO_ACTUAL, "io_actual" },
	{ FIELD_IO_LENGTH, "io_length" },
	{ FIELD_IO_OFFSET, "io_offset" },
	{ FIELD_IOCTL, "ioctl" },
	{ FIELD_SOC_AF, "soc_af" },
	{ FIELD_SOC_PROTO, "soc_proto" },
	{ FIELD_SOC_TYPE, "soc_type" },
	{ FIELD_SHUTDOWN, "shutdown" },
	{ FIELD_ACCEPT_FD, "accept_fd" },
	{ FIELD_LISTEN, "listen" },
	{ FIELD_FORK_PID, "fork_pid" },
	{ FIELD_CLASS, "class" },
	{ FIELD_FCNTL_CMD, "fcntl_cmd" },
	{ FIELD_FCNTL_ARG, "fcntl_arg" },
	{ FIELD_FCNTL_STATUS, "fcntl_status" },
	{ FIELD_WAIT_RESULT, "wait_result" },
        { FIELD_WAIT_PID, "wait_pud" },
	{ FIELD_WAIT_STATUS, "wait_status" },
        { FIELD_WAIT_OPTIONS, "wait_options" },

	{ CLASS_NINIT, "_ninit" },
	{ CLASS_FORK, "_fork" },
	{ CLASS_FD, "_fd" },
	{ CLASS_FD_AND_NAME, "_fd_and_name" },
	{ CLASS_EXIT, "_exit" },
	{ CLASS_DUP, "_dup" },
	{ CLASS_IOCTL, "_ioctl" },
	{ CLASS_SOCKET, "_socket" },
	{ CLASS_ACCEPT, "_accept" },
	{ CLASS_LISTEN, "_listen" },
	{ CLASS_SHUTDOWN, "_shutdown" },
	{ CLASS_RESERVED, "_reserved" },
	{ CLASS_IO_ASYNC, "_io_async" },
	{ CLASS_IO_SYNC, "_io_sync" },
	{ CLASS_FCNTL, "_fcntl" },
	{ CLASS_WAIT, "_wait" },

	{ IOD_NINITIATE,"NINITIATE     " },
	{ IOD_FORK,     "FORK          " },
	{ IOD_STAT,	"STAT          " },
	{ IOD_FSTAT,	"FSTAT         " },
	{ IOD_OPEN,	"OPEN          " },
	{ IOD_CREAT,	"CREAT         " },
	{ IOD_DUP,	"DUP           " },
	{ IOD_CLOSE, 	"CLOSE         " },
	{ IOD_IOCTL,	"IOCTL         " },
	{ IOD_READ,	"READ          " },
	{ IOD_WRITE,	"WRITE         " },
	{ IOD_READA,	"READA         " },
	{ IOD_WRITEA,	"WRITEA        " },
	{ IOD_EXIT,	"EXIT          " },
	{ IOD_SOCKET,	"SOCKET        " },
	{ IOD_BIND,	"BIND          " },
	{ IOD_LISTEN,	"LISTEN        " },
	{ IOD_ACCEPT,	"ACCEPT        " },
	{ IOD_CONNECT,	"CONNECT       " },
	{ IOD_SOCKETPAIR,"SOCKETPAIR    " },
	{ IOD_SENDTO,	"SENDTO        " },
	{ IOD_SEND,	"SEND          " },
	{ IOD_SENDMSG,	"SENDMSG       " },
	{ IOD_RECVFROM,	"RECVFROM      " },
	{ IOD_RECV,	"RECV          " },
	{ IOD_RECVMSG,	"RECVMSG       " },
	{ IOD_SHUTDOWN,	"SHUTDOWN      " },
	{ DAEMON_START, "DAEMON_START  " },
	{ IOD_READLA,   "READLA        " },
	{ IOD_READL,    "READL         " },
	{ IOD_WRITELA,  "WRITELA       " },
	{ IOD_WRITEL,   "WRITEL        " },
	{ IOD_READV,	"READV         " },
	{ IOD_WRITEV,   "WRITEV        " },
        { IOD_FCNTL,    "FCNTL         " },
        { IOD_PIPE,     "PIPE          " },
        { IOD_UNLINK,   "UNLINK        " },
        { IOD_VFORK,    "VFORK         " },
	{ IOD_WAIT,     "WAIT          " },
    	{ XIO_OPEN, 	"XIO_OPEN      " },
    	{ XIO_CLOSE,	"XIO_CLOSE     " },
    	{ XIO_READ,	"XIO_READ      " },
    	{ XIO_WRITE,	"XIO_WRITE     " },
    	{ XIO_REQUEST,	"XIO_REQUEST   " },
     	{ XIO_REPLY,	"XIO_REPLY     " },
    	{ XIO_EVENT,	"XIO_EVENT     " },
    	{ XIO_FLUSH,	"XIO_FLUSH     " },
    	{ XIO_FLUSH_INFO,"XIO_FLUSH_INFO" },
	{ IOD_CHDIR,    "CHDIR         " },
	{ IOD_LINK,     "LINK          " },
	{ IOD_SYMLINK,  "SYMLINK       " },
	{ IOD_RENAME,   "RENAME        " },
	{ IOD_MKDIR,	"MKDIR         " },
	{ IOD_RMDIR,	"RMDIR         " },
	{ IOD_LSTAT,	"LSTAT         " },
	{ XIO_RESERVED8,"XIO_RESERVED8 " },
	{ XIO_RESERVED9,"XIO_RESERVED9 " },
	{ XIO_RESERVED10,"XIO_RESERVED10" },
	{ XIO_RESERVED11,"XIO_RESERVED11" },
	{ XIO_RESERVED12,"XIO_RESERVED12" },
    	{ CNTL_FLUSH,	"CNTL_FLUSH    " }

};


static struct entry *int_table[HASH_TABLE_SIZE];
static struct entry *str_table[HASH_TABLE_SIZE];

static int table_size = 0;

static int hash_str(), hash_int(), rehash_int(), hash_add();
int symtable_lookup_str();
char *symtable_lookup_int();

symtable_init()
{
  int i;

  table_size = sizeof(table) / sizeof(struct entry);
  for (i = 0; i < HASH_TABLE_SIZE; i++)
    int_table[i] = str_table[i] = 0;
  for (i = 0; i < table_size; i++) {
    hash_add( int_table, hash_int( table[i].int_value ), &table[i] );
    hash_add( str_table, hash_str( table[i].str_value ), &table[i] );    
  }
}

char *symtable_lookup_int( i )
int i;
{
  int hash_key;
  hash_key = hash_int( i );
  while (int_table[hash_key]) {
    if (int_table[hash_key]->int_value == i) 
      return int_table[hash_key]->str_value;
    else
      hash_key = rehash_int( hash_key );
  }
  return (char *)-1;
}

int symtable_lookup_str( s )
char *s;
{
  int hash_key;
  hash_key = hash_str( s );
  while (str_table[hash_key]) {
    if (strcpy( str_table[hash_key]->str_value, s ) == 0) 
      return str_table[hash_key]->int_value;
    else
      hash_key = rehash_int( hash_key );
  }
  return 0;
}

static int hash_add( table, hash_key, value )
struct entry *table[];
int hash_key;
struct entry * value;
{
  int count;
  count = 0;
  while (table[hash_key] && count < HASH_TABLE_SIZE) {
    hash_key = rehash_int( hash_key );
    count++;
  }
  if (count == HASH_TABLE_SIZE) {
    fprintf(stderr,"hash_add(): hash table full\n");    
    exit(1);
  }
  table[hash_key] = value;
}

static int hash_str( s )
char *s;
{
  int len, i, sum;
  sum = 0;
  len = strlen( s );
  len = (len > 4 ? 4 : len);
  for (i = 0; i < len; i++) {
    sum += *s++;
    sum *= 128;
    if ( sum < 0 ) sum = -sum;
  }
  return hash_int( sum );
}

static int hash_int( i )
int i;
{
  return i % HASH_TABLE_SIZE;
}

static int rehash_int( i )
int i;
{ 
  return hash_int( i + 283 );
}
