#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>

extern int errno;

char filename[256][16];
int count;

int main(int argc, char *argv[])
{
	int i;
	int count;

	time_start(0);
	if ( argc != 2 ) {
		fprintf(stderr,"%s [count]\n",argv[0]);
		exit(1);
	}
	count = atoi(argv[1]);
	if ( count == 0 ) {
		fprintf(stderr,"%s count = 0\n",argv[0]);
		exit(1);
	}
	time_done(0);

	for (i = 0;i < count; i++) {
		sprintf(filename[i],"tmp%d",i);
		unlink(filename[i]);
	}
	time_start(1);
	dotest(count);
	time_done(1);
	return 0;
}

int dotest(int count)
{
	int fd;
	int i;
	int offset;

	for(i = 0; i < count; i++) {
 		fd = open(filename[i], O_RDWR | O_CREAT | O_TRUNC, 0666);
		if (fd < 0) {
			fprintf(stderr, "open error errno %d\n",errno);
			exit(1);
		}
		close(fd);
	}
	for (i = 0;i < count; i++) {
		unlink(filename[i]);
	}
	return 0;
}

#include <sys/time.h>
#include <sys/resource.h>

struct timezone time_zone;
struct timeval start_timeval;
struct timeval done_timeval;
struct rusage start_usage;
struct rusage done_usage;
struct rusage total_usage;

int subtract_timeval(struct timeval *out, struct timeval *in1, 
		     struct timeval *in2)
{
  out->tv_sec = in1->tv_sec - in2->tv_sec;
  if (in2->tv_usec > in1->tv_usec) {
    out->tv_sec--;
    out->tv_usec = in1->tv_usec + 1000000 - in2->tv_usec;
  } else {
    out->tv_usec = in1->tv_usec - in2->tv_usec;
  }
}

int time_start(int real)
{
  int status;

  status = gettimeofday(&start_timeval, &time_zone);
  status = getrusage(RUSAGE_SELF, &start_usage);
  return 0;
}

int time_done(int real)
{
  int status;
  struct timeval wall_clock_time;

  status = getrusage(RUSAGE_SELF, &done_usage);
  status = gettimeofday(&done_timeval, &time_zone);

  subtract_timeval(&wall_clock_time, &done_timeval, &start_timeval);
  subtract_timeval(&(total_usage.ru_utime), 
		   &(done_usage.ru_utime), &(start_usage.ru_utime));
  subtract_timeval(&(total_usage.ru_stime), 
		   &(done_usage.ru_stime), &(start_usage.ru_stime));
  subtract_timeval(&wall_clock_time, &done_timeval, &start_timeval);
  total_usage.ru_minflt = done_usage.ru_minflt - start_usage.ru_minflt;
  total_usage.ru_majflt = done_usage.ru_majflt - start_usage.ru_majflt;
  total_usage.ru_nswap = done_usage.ru_nswap - start_usage.ru_nswap;
  total_usage.ru_inblock = done_usage.ru_inblock - start_usage.ru_inblock;
  total_usage.ru_oublock = done_usage.ru_oublock - start_usage.ru_oublock;
  total_usage.ru_msgsnd = done_usage.ru_msgsnd - start_usage.ru_msgsnd;
  total_usage.ru_msgrcv = done_usage.ru_msgrcv - start_usage.ru_msgrcv;
  total_usage.ru_nvcsw = done_usage.ru_nvcsw - start_usage.ru_nvcsw;
  total_usage.ru_nivcsw = done_usage.ru_nivcsw - start_usage.ru_nivcsw;

  printf("Wall Clock Time: %ld sec, %ld usec\n",
	 wall_clock_time.tv_sec, wall_clock_time.tv_usec);
  if ( real ) {
    printf("Resource Usage :\n");
    printf("  user time     : %ld sec %ld usec\n", 
  	   total_usage.ru_utime.tv_sec, total_usage.ru_utime.tv_usec);
    printf("  system time   : %ld sec %ld usec\n", 
  	   total_usage.ru_stime.tv_sec, total_usage.ru_stime.tv_usec);
    printf("  minor faults  : %d\n", total_usage.ru_minflt);
    printf("  major faults  : %d\n", total_usage.ru_majflt);
    printf("  swaps         : %d\n", total_usage.ru_nswap);
    printf("  in blocks     : %d\n", total_usage.ru_inblock);
    printf("  out blocks    : %d\n", total_usage.ru_oublock);
    printf("  msg sends     : %d\n", total_usage.ru_msgsnd);
    printf("  msg receives  : %d\n", total_usage.ru_msgrcv);
    printf("  volunt. csw   : %d\n", total_usage.ru_nvcsw);
    printf("  involunt. csw : %d\n", total_usage.ru_nivcsw);
  }
  return 0;
}


