#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <stdio.h>

extern int errno;


int main(int argc, char *argv[])
{
	int i;
	int kb;
	int bytes;
	int blocking;
	int flags;
	char * filename = "tmptest";

	time_start(0);
	if ( argc != 3 ) {
		fprintf(stderr,"%s [kbytes] [blocking in kb]\n",argv[0]);
		exit(1);
	}
	bytes = atoi(argv[1]) * 1024;
	blocking = atoi(argv[2]);
#ifdef CHOICES
	flags = 0;
	kb = 4;
	while (blocking > kb) {
		kb += kb;
		flags ++;
	}
	flags = flags << 28;
#else
	flags = 0;
#endif
	if ( bytes == 0 ) {
		fprintf(stderr,"%s bytes = 0\n",argv[0]);
		exit(1);
	}
	time_done(0);

	time_start(1);
	dotest(filename, bytes, blocking, flags);
	time_done(1);
	unlink(filename);
	return 0;
}

int dotest(char * filename, int bytes, int blocking, int flags)
{
	int fd;
	int i;
	int count;
	int result;
	char buf[8192];

 	fd = open(filename , O_TRUNC | O_CREAT | O_RDWR | flags, 0666);
	if (fd < 0) {
		fprintf(stderr, "open error errno %d\n",errno);
		exit(1);
	}
	for( i = 0; i < bytes; i += 8192 ) {
		count = bytes - i;
		if ( count > 8192 ) count = 8192;
		result = write(fd, buf, count);
		if ( result != count ) {
			fprintf(stderr,"result %d != expected %d\n", 
			        result, count );
			exit(1);
		}
	}
	lseek(fd, 0, SEEK_SET);
	for( i = 0; i < bytes; i += 8192 ) {
		count = bytes - i;
		if ( count > 8192 ) count = 8192;
		result = read(fd, buf, count);
		if ( result != count ) {
			fprintf(stderr,"result %d != expected %d\n", 
			        result, count );
			exit(1);
		}
	}
	fsync(fd);
	close(fd);
	return 0;
}

#include <sys/time.h>
#include <sys/resource.h>

struct timezone time_zone;
struct timeval start_timeval;
struct timeval done_timeval;
struct rusage start_usage;
struct rusage done_usage;
struct rusage total_usage;

int subtract_timeval(struct timeval *out, struct timeval *in1, 
		     struct timeval *in2)
{
  out->tv_sec = in1->tv_sec - in2->tv_sec;
  if (in2->tv_usec > in1->tv_usec) {
    out->tv_sec--;
    out->tv_usec = in1->tv_usec + 1000000 - in2->tv_usec;
  } else {
    out->tv_usec = in1->tv_usec - in2->tv_usec;
  }
}

int time_start(int real)
{
  int status;

  status = gettimeofday(&start_timeval, &time_zone);
  status = getrusage(RUSAGE_SELF, &start_usage);
  return 0;
}

int time_done(int real)
{
  int status;
  struct timeval wall_clock_time;

  status = getrusage(RUSAGE_SELF, &done_usage);
  status = gettimeofday(&done_timeval, &time_zone);

  subtract_timeval(&wall_clock_time, &done_timeval, &start_timeval);
  subtract_timeval(&(total_usage.ru_utime), 
		   &(done_usage.ru_utime), &(start_usage.ru_utime));
  subtract_timeval(&(total_usage.ru_stime), 
		   &(done_usage.ru_stime), &(start_usage.ru_stime));
  subtract_timeval(&wall_clock_time, &done_timeval, &start_timeval);
  total_usage.ru_minflt = done_usage.ru_minflt - start_usage.ru_minflt;
  total_usage.ru_majflt = done_usage.ru_majflt - start_usage.ru_majflt;
  total_usage.ru_nswap = done_usage.ru_nswap - start_usage.ru_nswap;
  total_usage.ru_inblock = done_usage.ru_inblock - start_usage.ru_inblock;
  total_usage.ru_oublock = done_usage.ru_oublock - start_usage.ru_oublock;
  total_usage.ru_msgsnd = done_usage.ru_msgsnd - start_usage.ru_msgsnd;
  total_usage.ru_msgrcv = done_usage.ru_msgrcv - start_usage.ru_msgrcv;
  total_usage.ru_nvcsw = done_usage.ru_nvcsw - start_usage.ru_nvcsw;
  total_usage.ru_nivcsw = done_usage.ru_nivcsw - start_usage.ru_nivcsw;

  printf("Wall Clock Time: %ld sec, %ld usec\n",
	 wall_clock_time.tv_sec, wall_clock_time.tv_usec);
  if ( real ) {
    printf("Resource Usage :\n");
    printf("  user time     : %ld sec %ld usec\n", 
  	   total_usage.ru_utime.tv_sec, total_usage.ru_utime.tv_usec);
    printf("  system time   : %ld sec %ld usec\n", 
  	   total_usage.ru_stime.tv_sec, total_usage.ru_stime.tv_usec);
    printf("  minor faults  : %d\n", total_usage.ru_minflt);
    printf("  major faults  : %d\n", total_usage.ru_majflt);
    printf("  swaps         : %d\n", total_usage.ru_nswap);
    printf("  in blocks     : %d\n", total_usage.ru_inblock);
    printf("  out blocks    : %d\n", total_usage.ru_oublock);
    printf("  msg sends     : %d\n", total_usage.ru_msgsnd);
    printf("  msg receives  : %d\n", total_usage.ru_msgrcv);
    printf("  volunt. csw   : %d\n", total_usage.ru_nvcsw);
    printf("  involunt. csw : %d\n", total_usage.ru_nivcsw);
  }
  return 0;
}


