/*
 * This file is part of the Choices Operating System
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Ave.
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 * CONFIDENTIAL INFORMATION. Distribution restricted under license agreement.
 *
 * Author: Panagiotis Kougiouris ( kougiour@cs.uiuc.edu )
 * Project Manager and Principal Investigator: Roy Campbell (roy@cs.uiuc.edu)
 *
 * Funded by: NSF TAPESTRY Grant No. 1-5-30035, NASA ICLASS Grant 
 *   No. 1-5-25469 and No. NSG1471 and AT&T Metronet Grant No. 1-5-37411.
 */
/*
 * RAMDisk.h - Memory Object representing a disk located in the
 * physical Memory of the machine.
 * WARNING:
 *     The class expects there is a function 
 * void PhysCopyBlock( unsigned char * FROM, unsigned char * TO, int N );
 * which knows how to transfer a block of N characters from the
 * the physical Address FROM to the physical Address TO. 
 * Usually this function is implemented in Assembly because it should turn on 
 * and off the MMU. 
 */

#ifndef RAMDisk_h
#define	RAMDisk_h


#ifdef __GNUC__
#pragma interface
#endif

#include "Disk.h"

extern ClassStar RAMDiskClass;

class RAMDisk : public Disk {
public:
	/* DESCR: 
	 */
	Class * classOf();


	/* DESCR: A RAMDisk has only one partition
	 */
	int partitions();

	/* DESCR: The constructor of the disk.
	 * INPUT: 1] The physical address of the machine where the
	 *	     disk starts.
	 *        2] The number of units in the disk. 
	 *	  3] The base two logarithm of the unit size.
	 */
	RAMDisk( char * Base, unsigned int numberOfUnits, int log2UnitSize );

protected:

	/* We keep the disk base address here. Remember that this
	 * address is physical not virtual.
	 */
	char* RAMDiskBasePhysicalAddress;
	
        /* DESCR: A  function that does I/O of a single
         *        block without retries as required from the
	 *	  supercass. 
         * INPUT:       1] The number of block to read or write.
         *              2] A physical address representing where the
         *                 block should be transferred to.
         *              3] DiskReadOperation meaning read
         *                 DiskWriteOperation meaning write
         * OUTPUT:      DiskOperSucceded or DiskOperFailed
         */
        int
        basicDoio( unsigned int start, char * buf, int oper);

	/* DESCR: This method is here only because our superclass
	 *	  needs it. It always returns 1.
	 */
	int reset();

	/* DESCRIPTION:	A virtual method that reads or writes a number
	 *		of blocks of data. The method iterates through
	 *		the PhysicalMemoryChain and uses the basicDoio.
	 *		The method also retries _retries number of
	 *		times before aborting when it tries to read a single
	 *		block. 
	 *		   Subclasses can redefine this method if
	 *		they want to gain in efficiency, if for example
	 *		they want to fully exploit the hardware.
	 *		   This method can be sure that only one thread of
	 *		control will ever be allowd inside this method.
	 * INPUT:	1] The number of block to start.
 	 *		2] The number of blocks to  read or write.
	 *		3] The virtual address of the buffer.
	 *		4] The number of bytes in the buffer.
	 *		5] DiskReadOperation mening read
			   DiskWriteOperation meaning write
	 * OUTPUT:	number of blocks read.
	 *
	 * Overridden from Disk
	 */
	friend class DiskScheduler;
	friend class OneAtATimeDiskScheduler;

	virtual int 
	doio( unsigned int start, int count, char * addr, int nbytes, int op );

};

#endif /* RAMDisk_h */
